/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: Access.c,v 1.38 2002/11/16 19:30:19 pbj Exp $
____________________________________________________________________________*/

#include <windows.h>
#include "PGPadmin.h"
#include <commctrl.h>
#include "resource.h"
#include "Utils.h"

#include "PGPadminHelp.h"
#include "pgpClientLib.h"
#include "pgpHotKeys.h"
#include "pgpKeyServerPrefs.h"
#include "pflPrefTypes.h"
#include "pgpClientPrefs.h"
#include "pgpNetPrefs.h"

#define IMAGE_UNCHECK	0
#define IMAGE_CHECK		1
#define IMAGE_PARTIAL	2
#define IMAGE_ITEM		3

#define LOCK_PREF_FLAGS		kPGPPrefFlags_Hash			|	\
							kPGPPrefFlags_Overwrite		|	\
							kPGPPrefFlags_DisableGUI

#define PREFFLAGS_DISABLEGUI (nFlags & kPGPPrefFlags_DisableGUI) / \
								kPGPPrefFlags_DisableGUI	

#define LoadItemString(x)	LoadString(g_hInstance, x, szFormat, \
								sizeof(szFormat))

#define InsertItemInTree(x,y)	tvis.item.iImage = x;				\
								tvis.item.iSelectedImage = x;		\
								tvis.item.lParam = y;				\
								if (tvis.hParent != NULL)			\
								{									\
									if (x == IMAGE_CHECK)			\
										nCheck++;					\
									else if (x == IMAGE_UNCHECK)	\
										nUncheck++;					\
								}									\
								hItem = TreeView_InsertItem(hCtl, &tvis)

#define UpdateCatagory()		tvi.hItem = tvis.hParent;			\
								if (nCheck)							\
								{									\
									if (nUncheck)					\
										tvi.iImage = IMAGE_PARTIAL;	\
									else							\
										tvi.iImage = IMAGE_CHECK;	\
								}									\
								else								\
									tvi.iImage = IMAGE_UNCHECK;		\
								tvi.iSelectedImage = tvi.iImage;	\
								TreeView_SetItem(hCtl, &tvi);		\
								nCheck = 0;							\
								nUncheck = 0


static void sFillTree(HWND hwndDlg, pgpConfigInfo *pConfig);

static void sAddHotkeySequence(PGPUInt32 nHotKey, PGPBoolean *pbValue, 
							   char *szValue);

static void	sGetKeyUserID(PGPKeyDBRef keyDB, PGPByte *pData,
						  char *szValue, PGPSize nSize);

static void sInitGUI(HWND hwndDlg, pgpConfigInfo *pConfig);

static void sSetPrefFlag(pgpConfigInfo *pConfig, PGPUInt32 parentIndex, 
						 PGPUInt32 prefIndex, PGPUInt32 flag);

static LPSTR sGetFileDescription(PGPMemoryMgrRef memoryMgr, LPSTR pszFile);

static DWORD aHelpIds[] = {
	IDC_INCLUDEOPTIONS,		IDH_PGPADMIN_INCLUDEOPTIONS,
	IDC_OPTIONS,			IDH_PGPADMIN_OPTIONS,
	IDC_OPTIONSLABEL,		-1,
    0,0
};


BOOL CALLBACK AccessDlgProc(HWND hwndDlg,
							UINT uMsg,
							WPARAM wParam,
							LPARAM lParam)
{
	LPNMHDR lpnmh;
	BOOL			bReturnCode = FALSE;
	pgpConfigInfo *	pConfig		= NULL;
	PGPUInt32		panelFlags;
	PGPError		err;
	char			szWarning[1024];
	char			szTitle[256];

	g_hCurrentDlgWnd = hwndDlg;
	pConfig = (pgpConfigInfo *) GetWindowLong(hwndDlg, GWL_USERDATA);

	switch(uMsg)
	{
	case WM_INITDIALOG:
		CommonDlgProc(hwndDlg, uMsg, wParam, lParam);
		pConfig = (pgpConfigInfo *) GetWindowLong(hwndDlg, GWL_USERDATA);
		sFillTree(hwndDlg, pConfig);
		sInitGUI(hwndDlg, pConfig);
		bReturnCode = TRUE;
		break;

	case WM_HELP:
	case WM_CONTEXTMENU:
		return PGPclHtmlHelp (hwndDlg, uMsg, wParam, lParam, 
				(char *) kPGPclAdminHelpFile, aHelpIds);

	case WM_NOTIFY:
		lpnmh = (LPNMHDR) lParam;
		switch (lpnmh->code)
		{
		case PSN_HELP:
			PGPclHtmlHelp (hwndDlg, kPGPclHelpContext, 0, 0, 
					(char *) kPGPclAdminHelpFile, 
					(DWORD *) IDH_PGPADMIN_ACCESSDIALOG);
			break;

		case NM_CLICK:
		case TVN_KEYDOWN:
			if (lpnmh->idFrom == IDC_OPTIONS)
			{
				BOOL bCheck = FALSE;
				HTREEITEM hItem;
				
				if (lpnmh->code == NM_CLICK)
				{
					TVHITTESTINFO hitTest;
					DWORD dwPos;
					
					dwPos = GetMessagePos();
					hitTest.pt.x = LOWORD(dwPos);
					hitTest.pt.y = HIWORD(dwPos);
					ScreenToClient(lpnmh->hwndFrom, &(hitTest.pt));
					
					hItem = TreeView_HitTest(lpnmh->hwndFrom, &hitTest);
					if (hitTest.flags & TVHT_ONITEMICON)
						bCheck = TRUE;
				}
				
				if (lpnmh->code == TVN_KEYDOWN)
				{
					LPNMTVKEYDOWN ptvkd = (LPNMTVKEYDOWN) lParam;
					
					if (ptvkd->wVKey == ' ')
					{
						hItem = TreeView_GetSelection(lpnmh->hwndFrom);
						bCheck = TRUE;
					}
				}
				
				if (bCheck)
				{
					TVITEM tvi;
					HTREEITEM hParent;
					
					tvi.mask = TVIF_HANDLE | TVIF_IMAGE | TVIF_SELECTEDIMAGE |
								TVIF_CHILDREN | TVIF_PARAM;
					
					tvi.hItem = hItem;
					TreeView_GetItem(lpnmh->hwndFrom, &tvi);
					
					if (tvi.iImage == IMAGE_ITEM)
						break;
					
					tvi.iImage = !tvi.iImage;
					tvi.iSelectedImage = !tvi.iSelectedImage;
					TreeView_SetItem(lpnmh->hwndFrom, &tvi);
					
					tvi.mask -= TVIF_CHILDREN;
					hParent = TreeView_GetParent(lpnmh->hwndFrom, hItem);
					if (tvi.cChildren && !hParent)
					{
						HTREEITEM hChild;
						PGPUInt32 nNetPref;
						
						nNetPref = tvi.lParam;
						hChild = TreeView_GetChild(lpnmh->hwndFrom, hItem);
						
						while (hChild)
						{
							tvi.hItem = hChild;
							tvi.mask = TVIF_HANDLE | TVIF_IMAGE 
										| TVIF_SELECTEDIMAGE;
							TreeView_SetItem(lpnmh->hwndFrom, &tvi);

							tvi.mask = TVIF_HANDLE | TVIF_IMAGE | 
										TVIF_SELECTEDIMAGE | TVIF_PARAM;
							TreeView_GetItem(lpnmh->hwndFrom, &tvi);
							sSetPrefFlag(pConfig, nNetPref, tvi.lParam, 
								tvi.iImage);

							hChild = TreeView_GetNextSibling(lpnmh->hwndFrom,
										hChild);
						}
					}
					else
					{
						HTREEITEM hChild;
						BOOL bSame = TRUE;
						TVITEM tviChild;
						TVITEM tviParent;
						
						hParent = TreeView_GetParent(lpnmh->hwndFrom, hItem);
						hChild = TreeView_GetChild(lpnmh->hwndFrom, hParent);

						tviParent.mask = TVIF_HANDLE | TVIF_PARAM;
						tviParent.hItem = hParent;
						TreeView_GetItem(lpnmh->hwndFrom, &tviParent);
						sSetPrefFlag(pConfig, tviParent.lParam, tvi.lParam, 
							tvi.iImage);

						tviChild.mask = TVIF_HANDLE | TVIF_IMAGE;
						
						while (hChild && bSame)
						{
							tviChild.hItem = hChild;
							TreeView_GetItem(lpnmh->hwndFrom, &tviChild);
							if (tviChild.iImage != tvi.iImage)
								bSame = FALSE;
							else
								hChild = 
									TreeView_GetNextSibling(lpnmh->hwndFrom,
										hChild);
						}
						
						if (!bSame)
						{
							tvi.iImage = IMAGE_PARTIAL;
							tvi.iSelectedImage = IMAGE_PARTIAL;
						}
						
						tvi.hItem = hParent;
						tvi.mask = TVIF_HANDLE | TVIF_IMAGE | 
									TVIF_SELECTEDIMAGE;
						TreeView_SetItem(lpnmh->hwndFrom, &tvi);
					}
				}
				
				bReturnCode = TRUE;
				PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			}
			break;

		default:
			bReturnCode = CommonDlgProc(hwndDlg, uMsg, wParam, lParam);
			break;
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_INCLUDEOPTIONS:
			{
				if (IsDlgButtonChecked(hwndDlg, IDC_INCLUDEOPTIONS) == 
					BST_CHECKED)
				{
					pConfig->bCopyClientPrefs = TRUE;
				}
				else
				{
					pConfig->bCopyClientPrefs = FALSE;
				}

				sInitGUI(hwndDlg, pConfig);
				bReturnCode = TRUE;
				PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
				break;
			}

		case IDC_SETOPTIONS:
			panelFlags = 0;
			if (PGPclIsComponentInstalled (kPGPclPGPkeys))
				panelFlags |= ADMINKEYSPREFSPAGES;
			if (PGPclIsComponentInstalled (kPGPclPGPtools))
				panelFlags |= ADMINMAILPREFSPAGES;
			if (PGPclIsComponentInstalled(kPGPclPGPdisk))
				panelFlags |= ADMINDISKPREFSPAGES;
			if (PGPclIsComponentInstalled (kPGPclPFIDS))
				panelFlags |= ADMINFIREPREFSPAGES;
			if (PGPclIsComponentInstalled(kPGPclVPN))
				panelFlags |= ADMINVPNPREFSPAGES;

			err = PGPclAdminPreferences(pConfig->pgpContext, hwndDlg, 
					pConfig->prefRef, pConfig->prefRefNet, 
					kPGPclRespectDisableFlags, panelFlags, 0, pConfig->keyDB);

			if (IsntPGPError(err))
			{
				TreeView_DeleteAllItems(GetDlgItem(hwndDlg, IDC_OPTIONS));
				sFillTree(hwndDlg, pConfig);
			}
			else if (err != kPGPError_UserAbort)
				PGPclErrorBox(hwndDlg, err);

			bReturnCode = TRUE;
			break;

		case IDC_IMPORTOPTIONS:
			LoadString(g_hInstance, IDS_W_IMPORTOPTIONS, szWarning, sizeof(szWarning));
			LoadString(g_hInstance, IDS_TITLE, szTitle, sizeof(szTitle));
			if (MessageBox(hwndDlg, szWarning, szTitle, MB_YESNO | MB_ICONQUESTION)
				== IDYES)
			{
				PGPPrefRef		clientPrefs;
				PGPPrefRef		netPrefs;
				PGPBoolean		bNet;

				SaveAdminOptions(pConfig);
				FreeAdminOptions(pConfig);

				bNet = PGPclIsComponentInstalled(kPGPclPGPnet);
				if (bNet)
					err = PGPclPeekClientLibPrefRefs(&clientPrefs, &netPrefs);
				else
					err = PGPclPeekClientLibPrefRefs(&clientPrefs, NULL);
				
				if (IsntPGPError(err))
				{
					PGPPrefRef tempPrefs;

					PGPNewMemoryPrefs(pConfig->memoryMgr, clientDefaults,
						clientDefaultsSize, &tempPrefs);
					PGPAddPrefs(pConfig->prefRef, tempPrefs);

					err = PGPclMergePrefs(tempPrefs, clientPrefs);
					if (IsntPGPError(err))
						err = PGPAddPrefs(clientPrefs, pConfig->prefRef);
					PGPFreePrefs(tempPrefs);
				}
				
				if (IsntPGPError(err) && bNet)
				{
					PGPPrefRef tempPrefs;

					PGPNewMemoryPrefs(pConfig->memoryMgr, netDefaults,
						netDefaultsSize, &tempPrefs);
					PGPAddPrefs(pConfig->prefRefNet, tempPrefs);

					err = PGPclMergeNetPrefs(tempPrefs, netPrefs);
					if (IsntPGPError(err))
						err = PGPAddPrefs(netPrefs, pConfig->prefRefNet);
					PGPFreePrefs(tempPrefs);
				}

				if (IsntPGPError(err))
				{
					InitAdminOptions(pConfig->pgpContext, pConfig, TRUE);
					LoadAdminOptions(pConfig);
					TreeView_DeleteAllItems(GetDlgItem(hwndDlg, IDC_OPTIONS));
					sFillTree(hwndDlg, pConfig);
				}
				else if (err != kPGPError_UserAbort)
					PGPclErrorBox(hwndDlg, err);
			}

			bReturnCode = TRUE;
			break;
		}
		break;
	}

	return bReturnCode;
}


static void sFillTree(HWND hwndDlg, pgpConfigInfo *pConfig)
{
	HWND hCtl;
	HTREEITEM hItem;
	HTREEITEM hParent;
	TVINSERTSTRUCT tvis;
	TVITEM tvi;
	HIMAGELIST hImages;
	HBITMAP hBmp;
	BOOL bAES;
	char szFormat[255];
	char szText[1024];
	char szValue[255];
	char szTemp[255];
	char szTrue[10];
	char szFalse[10];
	PGPBoolean bValue;
	PGPUInt32 nValue;
	PGPUInt32 nFlags;
	SYSTEMTIME sysTime;
	PGPHotKeys *pHotKeys;
	PGPKeyServerEntry *aKeyServer;
	PGPUInt32 nNumServers;
	PGPUInt32 nIndex;
	PGPByte *pData;
	PGPSize dataSize;
	PGPPrefArray *prefArray;
	PGPPrefStruct *prefStruct;
	PGPUInt32 nCheck;
	PGPUInt32 nUncheck;
	PGPNetPrefFirewallRule *pRule;
	PGPUInt32 nNumRules;
	PGPUInt8 nPort;
	HTREEITEM *ahGateway;

	hCtl = GetDlgItem(hwndDlg, IDC_OPTIONS);
	
	hImages = ImageList_Create(16, 16, ILC_COLOR | ILC_MASK, 4, 0);
	
	hBmp = LoadBitmap (g_hInstance, MAKEINTRESOURCE (IDB_UNCHECK));
	ImageList_AddMasked(hImages, hBmp, RGB(255, 0, 255));
	DeleteObject(hBmp);
	
	hBmp = LoadBitmap (g_hInstance, MAKEINTRESOURCE (IDB_CHECKED));
	ImageList_AddMasked(hImages, hBmp, RGB(255, 0, 255));
	DeleteObject(hBmp);
	
	hBmp = LoadBitmap (g_hInstance, MAKEINTRESOURCE (IDB_GREYCHECK));
	ImageList_AddMasked(hImages, hBmp, RGB(255, 0, 255));
	DeleteObject(hBmp);
	
	hBmp = LoadBitmap (g_hInstance, MAKEINTRESOURCE (IDB_ITEM));
	ImageList_AddMasked(hImages, hBmp, RGB(255, 0, 255));
	DeleteObject(hBmp);
	
	TreeView_SetImageList(hCtl, hImages, TVSIL_NORMAL);
	
	LoadString(g_hInstance, IDS_TRUE, szTrue, 10);
	LoadString(g_hInstance, IDS_FALSE, szFalse, 10);

	tvi.mask = TVIF_IMAGE | TVIF_SELECTEDIMAGE;
	nCheck = 0;
	nUncheck = 0;

	tvis.hParent = NULL;
	tvis.hInsertAfter = TVI_LAST;
	tvis.item.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM;
	tvis.item.iImage = IMAGE_UNCHECK;
	tvis.item.iSelectedImage = IMAGE_UNCHECK;
	tvis.item.lParam = 0;
	tvis.item.pszText = szText;

	if (PGPclIsComponentInstalled (kPGPclPGPkeys)) {

	LoadString(g_hInstance, IDS_GENERAL, szText, sizeof(szText));
	hItem = TreeView_InsertItem(hCtl, &tvis);
	tvis.hParent = hItem;

	LoadItemString(IDS_ENCRYPTTODEFAULT);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefEncryptToSelf, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefEncryptToSelf, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefEncryptToSelf);

	LoadItemString(IDS_FASTKEYGEN);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefFastKeyGen, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefFastKeyGen, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefFastKeyGen);

	LoadItemString(IDS_SHOWTRAY);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefShowTrayIcon, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefShowTrayIcon, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefShowTrayIcon);

	LoadItemString(IDS_CACHE);
	PGPGetPrefNumber(pConfig->prefRef, kPGPPrefCacheType, &nValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefCacheType, &nFlags);

	switch (nValue)
	{
	case kPGPPrefCacheTypeNone:
		wsprintf(szText, szFormat, szFalse);
		break;

	case kPGPPrefCacheTypeSeconds:
		PGPGetPrefNumber(pConfig->prefRef, kPGPPrefCacheSeconds, &nValue);
		SecondsToSystemTime(nValue, &sysTime);
		strcat(szFormat, " [%02d:%02d:%02d]");
		wsprintf(szText, szFormat, szTrue, sysTime.wHour, sysTime.wMinute, 
			sysTime.wSecond);
		break;

	case kPGPPrefCacheTypeLogon:
		wsprintf(szText, szFormat, szTrue);
		break;
	}
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefCacheType);

	LoadItemString(IDS_CACHESHARE);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefShareCache, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefShareCache, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefShareCache);

	LoadItemString(IDS_COMMENTBLOCK);
	PGPGetPrefStringBuffer(pConfig->prefRef, kPGPPrefComment, sizeof(szValue),
		szValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefComment, &nFlags);
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefComment);

	LoadItemString(IDS_WIPEPASSES);
	PGPGetPrefNumber(pConfig->prefRef, kPGPPrefFileWipePasses, &nValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefFileWipePasses, &nFlags);
	wsprintf(szText, szFormat, nValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefFileWipePasses);

	LoadItemString(IDS_WARNWIPE);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefWarnOnWipe, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefWarnOnWipe, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefWarnOnWipe);

	LoadItemString(IDS_AUTOWIPE);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefAutoWipeOnDelete, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefAutoWipeOnDelete, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefAutoWipeOnDelete);

	LoadItemString(IDS_AUTOWIPEPROGRESS);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefShowWipeProgress, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefShowWipeProgress, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefShowWipeProgress);

	UpdateCatagory();
	}

	if (PGPclIsComponentInstalled (kPGPclPGPtools))	{
	
	tvis.hParent = NULL;
	LoadString(g_hInstance, IDS_EMAIL, szText, sizeof(szText));
	InsertItemInTree(IMAGE_UNCHECK, 0);
	tvis.hParent = hItem;

	LoadItemString(IDS_PGPMIME);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefMailEncryptPGPMIME, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefMailEncryptPGPMIME, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefMailEncryptPGPMIME);

	LoadItemString(IDS_ENCRYPTBYDEFAULT);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefMailEncryptDefault, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefMailEncryptDefault, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefMailEncryptDefault);

	LoadItemString(IDS_SIGNBYDEFAULT);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefMailSignDefault, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefMailSignDefault, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefMailSignDefault);

	LoadItemString(IDS_AUTODECRYPT);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefAutoDecrypt, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefAutoDecrypt, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefAutoDecrypt);

	LoadItemString(IDS_USESECUREVIEWER);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefAlwaysUseSecureViewer, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefAlwaysUseSecureViewer, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefAlwaysUseSecureViewer);

	LoadItemString(IDS_WORDWRAP);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefWordWrapEnable, &bValue);
	PGPGetPrefNumber(pConfig->prefRef, kPGPPrefWordWrapWidth, &nValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefWordWrapEnable, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse, nValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefWordWrapEnable);

	UpdateCatagory();
	}

	if (PGPclIsComponentInstalled (kPGPclPGPkeys)) {

	tvis.hParent = NULL;
	LoadString(g_hInstance, IDS_HOTKEYTITLE, szText, sizeof(szText));
	InsertItemInTree(IMAGE_UNCHECK, 0);
	tvis.hParent = hItem;

	LoadString(g_hInstance, IDS_HOTKEYS, szText, sizeof(szText));
	PGPGetPrefData(pConfig->prefRef, kPGPPrefHotKeyData, &nValue, &pHotKeys);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefHotKeyData, &nFlags);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefHotKeyData);
	UpdateCatagory();
	tvis.hParent = hItem;

	if (pHotKeys != NULL)
	{
		szValue[0] = 0;
		sAddHotkeySequence(pHotKeys->hotkeyPurgePassphraseCache, &bValue, 
			szValue);
		LoadItemString(IDS_HOTKEYPURGE);
		wsprintf(szText, szFormat, bValue ? szTrue : szFalse, szValue);
		InsertItemInTree(IMAGE_ITEM, 0);
		
		szValue[0] = 0;
		sAddHotkeySequence(pHotKeys->hotkeyEncrypt, &bValue, szValue);
		LoadItemString(IDS_HOTKEYENCRYPT);
		wsprintf(szText, szFormat, bValue ? szTrue : szFalse, szValue);
		InsertItemInTree(IMAGE_ITEM, 0);
		
		szValue[0] = 0;
		sAddHotkeySequence(pHotKeys->hotkeySign, &bValue, szValue);
		LoadItemString(IDS_HOTKEYSIGN);
		wsprintf(szText, szFormat, bValue ? szTrue : szFalse, szValue);
		InsertItemInTree(IMAGE_ITEM, 0);
		
		szValue[0] = 0;
		sAddHotkeySequence(pHotKeys->hotkeyEncryptSign, &bValue, szValue);
		LoadItemString(IDS_HOTKEYENCRYPTSIGN);
		wsprintf(szText, szFormat, bValue ? szTrue : szFalse, szValue);
		InsertItemInTree(IMAGE_ITEM, 0);
		
		szValue[0] = 0;
		sAddHotkeySequence(pHotKeys->hotkeyDecrypt, &bValue, szValue);
		LoadItemString(IDS_HOTKEYDECRYPT);
		wsprintf(szText, szFormat, bValue ? szTrue : szFalse, szValue);
		InsertItemInTree(IMAGE_ITEM, 0);
		
		szValue[0] = 0;
		sAddHotkeySequence(pHotKeys->hotkeyDiskUnmount, &bValue, szValue);
		LoadItemString(IDS_HOTKEYUNMOUNT);
		wsprintf(szText, szFormat, bValue ? szTrue : szFalse, szValue);
		InsertItemInTree(IMAGE_ITEM, 0);

		PGPFreeData(pHotKeys);
	}

	tvis.hParent = NULL;
	LoadString(g_hInstance, IDS_SERVERS, szText, sizeof(szText));
	InsertItemInTree(IMAGE_UNCHECK, 0);
	tvis.hParent = hItem;

	LoadItemString(IDS_KEYSERVERS);
	PGPGetKeyServerPrefs(pConfig->prefRef, &aKeyServer, &nNumServers);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefKeyServerList, &nFlags);
	strcpy(szText, szFormat);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefKeyServerList);
	hParent = tvis.hParent;
	tvis.hParent = hItem;

	for (nIndex=0; nIndex<nNumServers; nIndex++)
	{
		wsprintf(szText, "%s:%d", aKeyServer[nIndex].serverDNS,
			aKeyServer[nIndex].serverPort);
		
		if (aKeyServer[nIndex].flags & kKeyServerIsRoot)
			strcat(szText, " (root)");
		if (aKeyServer[nIndex].flags & kKeyServerListed)
			strcat(szText, " (listed)");

		InsertItemInTree(IMAGE_ITEM, 0);
	}

	PGPFreeKeyServerList(aKeyServer);
	tvis.hParent = hParent;

	LoadItemString(IDS_SYNCENCRYPT);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefKeyServerSyncUnknownKeys, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefKeyServerSyncUnknownKeys, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefKeyServerSyncUnknownKeys);

	LoadItemString(IDS_SYNCSIGN);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefKeyServerSyncOnKeySign, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefKeyServerSyncOnKeySign, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefKeyServerSyncOnKeySign);

	LoadItemString(IDS_SYNCADD);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefKeyServerSyncOnAdd, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefKeyServerSyncOnAdd, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefKeyServerSyncOnAdd);

	LoadItemString(IDS_SYNCREVOKE);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefKeyServerSyncOnRevocation, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefKeyServerSyncOnRevocation, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefKeyServerSyncOnRevocation);

	LoadItemString(IDS_SYNCVERIFY);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefKeyServerSyncOnVerify, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefKeyServerSyncOnVerify, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefKeyServerSyncOnVerify);

	UpdateCatagory();
	tvis.hParent = NULL;
	LoadString(g_hInstance, IDS_CA, szText, sizeof(szText));
	InsertItemInTree(IMAGE_UNCHECK, 0);
	tvis.hParent = hItem;

	LoadItemString(IDS_CAURL);
	PGPGetPrefStringBuffer(pConfig->prefRef, kPGPPrefCAServerURL, 
		sizeof(szValue), szValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefCAServerURL, &nFlags);
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefCAServerURL);

	LoadItemString(IDS_CAREVOKEURL);
	PGPGetPrefStringBuffer(pConfig->prefRef, kPGPPrefCARevocationServerURL, 
		sizeof(szValue), szValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefCARevocationServerURL, &nFlags);
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefCARevocationServerURL);

	LoadItemString(IDS_CATYPE);
	switch (pConfig->nCAType)
	{
	case kPGPKeyServerClass_Invalid:
		LoadString(g_hInstance, IDS_CAINVALID, szValue, sizeof(szValue));
		break;

	case kPGPKeyServerClass_NetToolsCA:
		LoadString(g_hInstance, IDS_CANETTOOLS, szValue, sizeof(szValue));
		break;

	case kPGPKeyServerClass_Verisign:
		LoadString(g_hInstance, IDS_CAVERISIGN, szValue, sizeof(szValue));
		break;

	case kPGPKeyServerClass_Entrust:
		LoadString(g_hInstance, IDS_CAENTRUST, szValue, sizeof(szValue));
		break;

	case kPGPKeyServerClass_Netscape:
		LoadString(g_hInstance, IDS_CANETSCAPE, szValue, sizeof(szValue));
		break;

	case kPGPKeyServerClass_Microsoft:
		LoadString(g_hInstance, IDS_CAMICROSOFT, szValue, sizeof(szValue));
		break;
	}
	wsprintf(szText, szFormat, szValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefCAType, &nFlags);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefCAType);

	LoadItemString(IDS_CACERT);
	PGPGetPrefData(pConfig->prefRef, kPGPPrefCARootKeyID, &nValue, &pData);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefCARootKeyID, &nFlags);
	if (pData != NULL)
	{
		sGetKeyUserID(pConfig->keyDB, pData, szValue, sizeof(szValue));
		PGPFreeData(pData);
	}
	else
		strcpy(szValue, "None");
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefCARootKeyID);

	UpdateCatagory();
	tvis.hParent = NULL;
	LoadString(g_hInstance, IDS_ADVANCED, szText, sizeof(szText));
	InsertItemInTree(IMAGE_UNCHECK, 0);
	tvis.hParent = hItem;

	LoadItemString(IDS_PREFERREDALG);
	PGPGetPrefNumber(pConfig->prefRef, kPGPPrefPreferredAlgorithm, &nValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefPreferredAlgorithm, &nFlags);
	switch (nValue)
	{
	case kPGPCipherAlgorithm_CAST5:
		strcpy(szValue, "CAST");
		break;

	case kPGPCipherAlgorithm_IDEA:
		strcpy(szValue, "IDEA");
		break;

	case kPGPCipherAlgorithm_3DES:
		strcpy(szValue, "TripleDES");
		break;

	case kPGPCipherAlgorithm_Twofish256:
		strcpy(szValue, "Twofish");
		break;

	case kPGPCipherAlgorithm_AES256:
	case kPGPCipherAlgorithm_AES192:
	case kPGPCipherAlgorithm_AES128:
		strcpy(szValue, "AES");
		break;
	}
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefPreferredAlgorithm);

	LoadItemString(IDS_ALLOWEDALGS);
	PGPGetPrefArray(pConfig->prefRef, kPGPPrefAllowedAlgorithmsList, 
		&prefArray);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefAllowedAlgorithmsList, &nFlags);
	szValue[0] = 0;
	bAES = FALSE;
	for (nIndex=0; nIndex<prefArray->numElements; nIndex++)
	{
		PGPGetPrefArrayNumber(pConfig->prefRef, prefArray, nIndex, &nValue);
		switch (nValue)
		{
		case kPGPCipherAlgorithm_CAST5:
			strcat(szValue, "CAST");
			break;
			
		case kPGPCipherAlgorithm_IDEA:
			strcat(szValue, "IDEA");
			break;
			
		case kPGPCipherAlgorithm_3DES:
			strcat(szValue, "TripleDES");
			break;
			
		case kPGPCipherAlgorithm_Twofish256:
			strcat(szValue, "Twofish");
			break;

		case kPGPCipherAlgorithm_AES256:
		case kPGPCipherAlgorithm_AES192:
		case kPGPCipherAlgorithm_AES128:
			if (!bAES)
			{
				strcat(szValue, "AES");
				bAES = TRUE;
			}
			break;
		}
		if (nIndex < (prefArray->numElements - 1))
			strcat(szValue, ", ");
	}
	PGPFreePrefArray(prefArray);
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefAllowedAlgorithmsList);

	LoadItemString(IDS_DISPLAYMARGINAL);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefDisplayMarginalValidity, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefDisplayMarginalValidity, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefDisplayMarginalValidity);

	LoadItemString(IDS_MARGINALINVALID);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefMarginalIsInvalid, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefMarginalIsInvalid, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefMarginalIsInvalid);

	LoadItemString(IDS_WARNADK);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefWarnOnADK, &bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefWarnOnADK, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefWarnOnADK);

	LoadItemString(IDS_EXPORTFORMAT);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefExportKeysCompatible, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefExportKeysCompatible, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefExportKeysCompatible);

	LoadItemString(IDS_BACKUPKEYRING);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefAutoBackupToKeyringFolder, 
		&bValue);
	if (bValue)
		LoadString(g_hInstance, IDS_KEYRINGFOLDER, szValue, sizeof(szValue));
	else
		PGPGetPrefStringBuffer(pConfig->prefRef, kPGPPrefAutoBackupFolder, 
			sizeof(szValue), szValue);
		
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefAutoBackupKeyring, 
		&bValue);
	if (!bValue)
		strcpy(szValue, "");
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefAutoBackupKeyring, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefAutoBackupKeyring);

	LoadItemString(IDS_SMARTCARD);
	PGPGetPrefStringBuffer(pConfig->prefRef, kPGPPrefPKCS11DrvFile, 
	    sizeof(szValue), szValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefPKCS11DrvFile, &nFlags);
	nValue = strlen(szValue);
	LoadString(g_hInstance, IDS_CUSTOM, szTemp, sizeof(szTemp));
	if (nValue == 0)
		LoadString(g_hInstance, IDS_NONE, szTemp, sizeof(szTemp));
	if (nValue >= 9)
	{
		if (!stricmp(&(szValue[nValue-9]), "SLBck.dll"))
			LoadString(g_hInstance, IDS_SCHLUMBERGER, szTemp, sizeof(szTemp));
	}
	if (nValue >= 11)
	{
		if (!stricmp(&(szValue[nValue-11]), "PK2priv.dll"))
			LoadString(g_hInstance, IDS_GEMPLUS, szTemp, sizeof(szTemp));
		if (!stricmp(&(szValue[nValue-11]), "DKck232.dll"))
			LoadString(g_hInstance, IDS_RAINBOW, szTemp, sizeof(szTemp));
	}
	if (nValue >= 12)
	{
		if (!stricmp(&(szValue[nValue-12]), "eTpkcs11.dll"))
			LoadString(g_hInstance, IDS_ALADDIN, szTemp, sizeof(szTemp));
	}
	wsprintf(szText, szFormat, szTemp);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefPKCS11DrvFile);

	UpdateCatagory();
	}

	if (PGPclIsComponentInstalled(kPGPclPGPdisk)) {

	tvis.hParent = NULL;
	LoadString(g_hInstance, IDS_PGPDISK, szText, sizeof(szText));
	InsertItemInTree(IMAGE_UNCHECK, 0);
	tvis.hParent = hItem;

	LoadItemString(IDS_ALLOWFORCEUNMOUNT);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefDiskAllowForcibleUnmounts, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefDiskAllowForcibleUnmounts, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefDiskAllowForcibleUnmounts);

	LoadItemString(IDS_ASKUNMOUNT);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefDiskAlwaysForceUnmounts, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefDiskAlwaysForceUnmounts, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefDiskAlwaysForceUnmounts);

	LoadItemString(IDS_AUTOUNMOUNT);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefDiskUnmountOnInactivity, 
		&bValue);
	PGPGetPrefNumber(pConfig->prefRef, kPGPPrefDiskAutoUnmountTimeout, 
		&nValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefDiskUnmountOnInactivity, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse, nValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefDiskUnmountOnInactivity);

	LoadItemString(IDS_SLEEPUNMOUNT);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefDiskAutoUnmountOnSleep, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefDiskAutoUnmountOnSleep, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefDiskAutoUnmountOnSleep);

	LoadItemString(IDS_PREVENTSLEEP);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefDiskNoSleepIfFailAU, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefDiskNoSleepIfFailAU, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefDiskNoSleepIfFailAU);

/*	LoadItemString(IDS_VIEWCONTENTS);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefDiskBrowseAfterMount, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefDiskBrowseAfterMount, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefDiskBrowseAfterMount);*/

	LoadItemString(IDS_WARNDELETE);
	PGPGetPrefBoolean(pConfig->prefRef, kPGPPrefDiskWarnOnDeletion, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRef, kPGPPrefDiskWarnOnDeletion, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPPrefDiskWarnOnDeletion);

	UpdateCatagory();
	}

	if (PGPclIsComponentInstalled (kPGPclPFIDS)) {

	tvis.hParent = NULL;
	LoadString(g_hInstance, IDS_NETFIREWALL, szText, sizeof(szText));
	InsertItemInTree(IMAGE_UNCHECK, 1);
	tvis.hParent = hItem;

	LoadItemString(IDS_PGPNET);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefEnablePGPnet, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefEnablePGPnet, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefEnablePGPnet);

	LoadItemString(IDS_PGPFIRE);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefEnableFirewall, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefEnableFirewall, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefEnableFirewall);

	LoadItemString(IDS_LEARNMODE);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefUseLearnMode, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefUseLearnMode, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefUseLearnMode);

	LoadItemString(IDS_PROTECTIONLEVEL);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPnetPrefFirewallRuleSet, &nFlags);
	PGPGetPrefNumber(pConfig->prefRefNet, kPGPnetPrefFirewallRuleSet, &nValue);
	
	switch (nValue)
	{
	case kPGPNetFirewallRuleSet_Custom:
		LoadString(g_hInstance, IDS_FIREWALLCUSTOM, szValue, sizeof(szValue));
		break;
		
	case kPGPNetFirewallRuleSet_Minimal:
		LoadString(g_hInstance, IDS_FIREWALLMINIMAL, szValue, sizeof(szValue));
		break;
		
	case kPGPNetFirewallRuleSet_ClientMedium:
		LoadString(g_hInstance, IDS_CLIENTMEDIUM, szValue, sizeof(szValue));
		break;
		
	case kPGPNetFirewallRuleSet_ClientHigh:
		LoadString(g_hInstance, IDS_CLIENTHIGH, szValue, sizeof(szValue));
		break;
		
	case kPGPNetFirewallRuleSet_ServerMedium:
		LoadString(g_hInstance, IDS_SERVERMEDIUM, szValue, sizeof(szValue));
		break;
		
	case kPGPNetFirewallRuleSet_ServerHigh:
		LoadString(g_hInstance, IDS_SERVERHIGH, szValue, sizeof(szValue));
		break;
		
	case kPGPNetFirewallRuleSet_LearnStarter:
		LoadString(g_hInstance, IDS_LEARNSTARTER, szValue, sizeof(szValue));
		break;
	}

	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPnetPrefFirewallRuleSet);
	hParent = tvis.hParent;
	tvis.hParent = hItem;

	PGPGetNetFirewallRuleSet(pConfig->prefRefNet, &pRule, &nNumRules);
	for (nIndex=0; nIndex<nNumRules; nIndex++)
	{
		if (strlen(pRule[nIndex].description) > 0)
		{
			strcpy(szText, pRule[nIndex].description);
			InsertItemInTree(IMAGE_ITEM, 0);
			continue;
		}

		if (pRule[nIndex].active)
			LoadItemString(IDS_ACTIVE);
		else
			LoadItemString(IDS_INACTIVE);

		strcpy(szText, szFormat);

		if (!strlen(pRule[nIndex].appPath))
			LoadItemString(IDS_ANYAPPLICATION);
		else
		{
			char *szPtr;

			szPtr = sGetFileDescription(pConfig->memoryMgr, 
						pRule[nIndex].appPath);

			strcpy(szFormat, szPtr);
			PGPFreeData(szPtr);
		}

		strcat(szText, ", ");
		strcat(szText, szFormat);

		if (pRule[nIndex].permit)
			LoadItemString(IDS_PERMIT);
		else
			LoadItemString(IDS_DENY);

		strcat(szText, ", ");
		strcat(szText, szFormat);

		if (pRule[nIndex].matchIntrusion)
		{
			LoadItemString(IDS_ISINTRUSION);
			strcat(szText, ", ");
			strcat(szText, szFormat);
		}

		switch (pRule[nIndex].direction)
		{
		case 0:
			LoadItemString(IDS_BOTHDIRECTIONS);
			break;

		case 1:
			LoadItemString(IDS_INBOUND);
			break;

		case 2:
			LoadItemString(IDS_OUTBOUND);
			break;
		}

		strcat(szText, ", ");
		strcat(szText, szFormat);
		strcat(szText, ", ");

		switch (pRule[nIndex].ipProtocol)
		{
		case kPGPnetIPProtocol_ICMP:
			strcat(szText, "ICMP");
			break;

		case kPGPnetIPProtocol_IGMP:
			strcat(szText, "IGMP");
			break;

		case kPGPnetIPProtocol_TCP:
			strcat(szText, "TCP");
			break;

		case kPGPnetIPProtocol_UDP:
			strcat(szText, "UDP");
			break;

		case kPGPnetIPProtocol_ESP:
			strcat(szText, "ESP");
			break;

		case kPGPnetIPProtocol_AH:
			strcat(szText, "AH");
			break;
		}

		strcat(szText, ", ");

		switch (pRule[nIndex].remoteAddrType)
		{
		case kPGPnetAddrType_Host:
			nValue = pRule[nIndex].remoteIPAddrStart;
			
			wsprintf(szFormat, "%d.%d.%d.%d", nValue & 0xff,
				(nValue >> 8) & 0xff, (nValue >> 16) & 0xff,
				(nValue >> 24) & 0xff);
			break;

		case kPGPnetAddrType_Subnet:
			nValue =	pRule[nIndex].remoteIPAddrStart &
						pRule[nIndex].remoteIPMaskEnd;

			wsprintf(szFormat, "%d.%d.%d.%d", nValue & 0xff,
				(nValue >> 8) & 0xff, (nValue >> 16) & 0xff,
				(nValue >> 24) & 0xff);
			strcat(szText, szFormat);

			nValue = 0;
			while (pRule[nIndex].remoteIPMaskEnd & 0x1)
			{
				nValue++;
				pRule[nIndex].remoteIPMaskEnd >>= 1;
			}
			wsprintf(szFormat, "/%d", nValue);
			break;

		case kPGPnetAddrType_LocalSubnet:
			LoadItemString(IDS_LOCALSUBNET);
			break;

		case kPGPnetAddrType_Range:
			nValue = pRule[nIndex].remoteIPAddrStart;
			
			wsprintf(szFormat, "%d.%d.%d.%d", nValue & 0xff,
				(nValue >> 8) & 0xff, (nValue >> 16) & 0xff,
				(nValue >> 24) & 0xff);

			nValue = pRule[nIndex].remoteIPMaskEnd;
			
			wsprintf(szFormat, " - %d.%d.%d.%d", nValue & 0xff,
				(nValue >> 8) & 0xff, (nValue >> 16) & 0xff,
				(nValue >> 24) & 0xff);
			break;

		case kPGPnetAddrType_Any:
			LoadItemString(IDS_ANYADDRESS);
			break;
		}

		strcat(szText, szFormat);
		strcat(szText, ", ");
		LoadItemString(IDS_REMOTETYPES);
		strcat(szText, szFormat);

		switch (pRule[nIndex].remoteServiceType)
		{
		case kPGPnetIPServiceType_Single:
		case kPGPnetIPServiceType_Range:
		case kPGPnetIPServiceType_List:
			for (nPort=0; nPort<pRule[nIndex].numRemoteServices; nPort++)
			{
				switch (pRule[nIndex].ipProtocol)
				{
				case kPGPnetIPProtocol_ICMP:
					PGPGetICMPTypeString(pRule[nIndex].remoteService[nPort],
						sizeof(szFormat), szFormat);
					break;

				case kPGPnetIPProtocol_IGMP:
				case kPGPnetIPProtocol_ESP:
				case kPGPnetIPProtocol_AH:
					szFormat[0] = 0;
					break;

				case kPGPnetIPProtocol_TCP:
				case kPGPnetIPProtocol_UDP:
					PGPGetTCPServiceString(pRule[nIndex].remoteService[nPort],
						sizeof(szFormat), szFormat);
					break;
				}

				if (pRule[nIndex].remoteServiceType == 
					kPGPnetIPServiceType_Range)
				{
					if (nPort == 0)
						strcat(szText, " ");

					wsprintf(szFormat, "%d", 
						pRule[nIndex].remoteService[nPort]);

					strcat(szText, szFormat);

					if (nPort == 0)
						strcat(szText, "-");
				}
				else
				{
					strcat(szText, " ");
					strcat(szText, szFormat);
				}
			}
			break;

		case kPGPnetIPServiceType_Any:
			strcat(szText, " ");
			LoadItemString(IDS_ANYTYPE);
			strcat(szText, szFormat);
			break;
		}

		strcat(szText, ", ");
		LoadItemString(IDS_LOCALTYPES);
		strcat(szText, szFormat);

		switch (pRule[nIndex].localServiceType)
		{
		case kPGPnetIPServiceType_Single:
		case kPGPnetIPServiceType_Range:
		case kPGPnetIPServiceType_List:
			for (nPort=0; nPort<pRule[nIndex].numLocalServices; nPort++)
			{
				switch (pRule[nIndex].ipProtocol)
				{
				case kPGPnetIPProtocol_ICMP:
					PGPGetICMPTypeString(pRule[nIndex].localService[nPort],
						sizeof(szFormat), szFormat);
					break;

				case kPGPnetIPProtocol_IGMP:
				case kPGPnetIPProtocol_ESP:
				case kPGPnetIPProtocol_AH:
					szFormat[0] = 0;
					break;

				case kPGPnetIPProtocol_TCP:
				case kPGPnetIPProtocol_UDP:
					PGPGetTCPServiceString(pRule[nIndex].localService[nPort],
						sizeof(szFormat), szFormat);
					break;
				}

				if (pRule[nIndex].localServiceType == 
					kPGPnetIPServiceType_Range)
				{
					if (nPort == 0)
						strcat(szText, " ");

					wsprintf(szFormat, "%d", 
						pRule[nIndex].localService[nPort]);

					strcat(szText, szFormat);

					if (nPort == 0)
						strcat(szText, "-");
				}
				else
				{
					strcat(szText, " ");
					strcat(szText, szFormat);
				}
			}
			break;

		case kPGPnetIPServiceType_Any:
			strcat(szText, " ");
			LoadItemString(IDS_ANYTYPE);
			strcat(szText, szFormat);
			break;
		}

		InsertItemInTree(IMAGE_ITEM, 0);
	}
	PGPFreeData(pRule);
	tvis.hParent = hParent;

	LoadItemString(IDS_INTRUSION);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefUseIntrusionDetection, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefUseIntrusionDetection, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefUseIntrusionDetection);

	LoadItemString(IDS_AUTOBLOCK);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAutoBlockAttacks, 
		&bValue);
	if (bValue)
	{
		PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefBlockUntilRemoved, 
			&bValue);

		if (bValue)
		{
			LoadString(g_hInstance, IDS_UNTILREMOVED, szValue, 
				sizeof(szValue));
		}
		else
		{
			PGPGetPrefNumber(pConfig->prefRefNet, kPGPnetPrefBlockMinutes, 
				&nValue);

			LoadItemString(IDS_BLOCKMINUTES);
			wsprintf(szValue, szFormat, nValue);
			LoadItemString(IDS_AUTOBLOCK);
			bValue = TRUE;
		}
	}
	else
		strcpy(szValue, "");

	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefAutoBlockAttacks, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefAutoBlockAttacks);

	LoadItemString(IDS_EMAILALERT);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAttackEmailNotify, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefAttackEmailNotify, 
		&nFlags);
	if (bValue)
		PGPGetPrefStringBuffer(pConfig->prefRefNet, 
			kPGPNetPrefAttackNotifyEmail, sizeof(szValue), szValue);
	else
		strcpy(szValue, "");
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefAttackEmailNotify);

	LoadItemString(IDS_SOUNDALERT);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAttackSound, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefAttackSound, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefAttackSound);

	LoadItemString(IDS_FLASHTRAY);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAttackNotify, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefAttackNotify, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefAttackNotify);

	UpdateCatagory();
	}

	if (PGPclIsComponentInstalled(kPGPclVPN)) {

	tvis.hParent = NULL;
	LoadString(g_hInstance, IDS_NETVPN, szText, sizeof(szText));
	InsertItemInTree(IMAGE_UNCHECK, 1);
	tvis.hParent = hItem;

	LoadItemString(IDS_HOSTLIST);
	PGPGetPrefArray(pConfig->prefRefNet, kPGPNetPrefHostArray, 
		&prefArray);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefHostArray, 
		&nFlags);
	strcpy(szText, szFormat);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefHostArray);
	hParent = tvis.hParent;
	tvis.hParent = hItem;

	ahGateway = (HTREEITEM *) calloc(sizeof(HTREEITEM), prefArray->numElements);
	for (nIndex=0; nIndex<prefArray->numElements; nIndex++)
	{
		szText[0] = 0;
		PGPGetPrefArrayStruct(pConfig->prefRefNet, prefArray, nIndex,
			&prefStruct);

		if ((PGPInt32) prefStruct->members[3].data != -1)
			continue;

		strcat(szText, (char *) prefStruct->members[4].data);
		strcat(szText, ", ");

		switch ((PGPNetHostType) prefStruct->members[0].data)
		{
		case kPGPnetInsecureHost:
			if ((PGPInt32) prefStruct->members[2].data == -1)
				LoadString(g_hInstance, IDS_INSECUREHOST, szFormat, 
					sizeof(szFormat));
			else
				LoadString(g_hInstance, IDS_INSECURESUBNET, szFormat, 
					sizeof(szFormat));
			break;

		case kPGPnetSecureHost:
			if ((PGPInt32) prefStruct->members[2].data == -1)
				LoadString(g_hInstance, IDS_SECUREHOST, szFormat, 
					sizeof(szFormat));
			else
				LoadString(g_hInstance, IDS_SECURESUBNET, szFormat, 
					sizeof(szFormat));
			break;

		case kPGPnetSecureGateway:
			LoadString(g_hInstance, IDS_SECUREGATEWAY, szFormat, 
				sizeof(szFormat));
			break;
		}

		strcat(szText, szFormat);
		strcat(szText, ", ");
		strcat(szText, inet_ntoa(*((struct in_addr *) 
						&(prefStruct->members[1].data)))); 

		if ((PGPInt32) prefStruct->members[2].data != -1)
		{
			strcat(szText, ", ");
			strcat(szText, inet_ntoa(*((struct in_addr *) 
							&(prefStruct->members[2].data)))); 
		}


		InsertItemInTree(IMAGE_ITEM, 0);
		ahGateway[nIndex] = hItem;
	}

	for (nIndex=0; nIndex<prefArray->numElements; nIndex++)
	{
		szText[0] = 0;
		PGPGetPrefArrayStruct(pConfig->prefRefNet, prefArray, nIndex,
			&prefStruct);

		if ((PGPInt32) prefStruct->members[3].data == -1)
			continue;

		strcat(szText, (char *) prefStruct->members[4].data);
		strcat(szText, ", ");

		switch ((PGPNetHostType) prefStruct->members[0].data)
		{
		case kPGPnetInsecureHost:
			if ((PGPInt32) prefStruct->members[2].data == -1)
				LoadString(g_hInstance, IDS_INSECUREHOST, szFormat, 
					sizeof(szFormat));
			else
				LoadString(g_hInstance, IDS_INSECURESUBNET, szFormat, 
					sizeof(szFormat));
			break;

		case kPGPnetSecureHost:
			if ((PGPInt32) prefStruct->members[2].data == -1)
				LoadString(g_hInstance, IDS_SECUREHOST, szFormat, 
					sizeof(szFormat));
			else
				LoadString(g_hInstance, IDS_SECURESUBNET, szFormat, 
					sizeof(szFormat));
			break;

		case kPGPnetSecureGateway:
			LoadString(g_hInstance, IDS_SECUREGATEWAY, szFormat, 
				sizeof(szFormat));
			break;
		}

		strcat(szText, szFormat);
		strcat(szText, ", ");
		strcat(szText, inet_ntoa(*((struct in_addr *) 
						&(prefStruct->members[1].data)))); 

		if ((PGPInt32) prefStruct->members[2].data != -1)
		{
			strcat(szText, ", ");
			strcat(szText, inet_ntoa(*((struct in_addr *) 
							&(prefStruct->members[2].data)))); 
		}

		tvis.hParent = ahGateway[(PGPInt32) prefStruct->members[3].data];
		InsertItemInTree(IMAGE_ITEM, 0);
	}

	PGPFreePrefArray(prefArray);
	free(ahGateway);
	tvis.hParent = hParent;

	LoadItemString(IDS_ENABLEVPN);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefEnableVPN, 
		&bValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefEnableVPN, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefEnableVPN);

	LoadItemString(IDS_UNKNOWNHOST);
	PGPGetPrefNumber(pConfig->prefRefNet, kPGPnetPrefUnconfiguredMode, 
		&nValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPnetPrefUnconfiguredMode, 
		&nFlags);
	switch (nValue)
	{
	case kPGPnet_UM_Attempt:
		LoadString(g_hInstance, IDS_HOSTATTEMPT, szValue, sizeof(szValue));
		break;

	case kPGPnet_UM_Allow:
		LoadString(g_hInstance, IDS_HOSTALLOW, szValue, sizeof(szValue));
		break;

	case kPGPnet_UM_Require:
		LoadString(g_hInstance, IDS_HOSTREQUIRE, szValue, sizeof(szValue));
		break;
	}
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPnetPrefUnconfiguredMode);

	LoadItemString(IDS_SETUPKEYTIME);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefEnableIKETimeExpiration,
		&bValue);
	PGPGetPrefNumber(pConfig->prefRefNet, kPGPNetPrefIKETimeExpiration,
		&nValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefEnableIKETimeExpiration, 
		&nFlags);
	SecondsToSystemTime(nValue, &sysTime);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse, sysTime.wHour,
		sysTime.wMinute, sysTime.wSecond);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, 
		kPGPNetPrefEnableIKETimeExpiration);

	LoadItemString(IDS_PRIMARYKEYTIME);
	PGPGetPrefBoolean(pConfig->prefRefNet, 
		kPGPNetPrefEnableIPSECTimeExpiration, &bValue);
	PGPGetPrefNumber(pConfig->prefRefNet, kPGPNetPrefIPSECTimeExpiration,
		&nValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefEnableIPSECTimeExpiration, 
		&nFlags);
	SecondsToSystemTime(nValue, &sysTime);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse, sysTime.wHour,
		sysTime.wMinute, sysTime.wSecond);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, 
		kPGPNetPrefEnableIPSECTimeExpiration);

	LoadItemString(IDS_PRIMARYKEYDATA);
	PGPGetPrefBoolean(pConfig->prefRefNet, 
		kPGPNetPrefEnableIPSECKByteExpiration, &bValue);
	PGPGetPrefNumber(pConfig->prefRefNet, kPGPNetPrefIPSECKByteExpiration,
		&nValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefEnableIPSECKByteExpiration, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse, nValue / 1024);
	InsertItemInTree(PREFFLAGS_DISABLEGUI,
		kPGPNetPrefEnableIPSECKByteExpiration);

	UpdateCatagory();
	tvis.hParent = NULL;
	LoadString(g_hInstance, IDS_NETAUTH, szText, sizeof(szText));
	InsertItemInTree(IMAGE_UNCHECK, 1);
	tvis.hParent = hItem;

	LoadItemString(IDS_PGPKEY);
	PGPGetPrefData(pConfig->prefRefNet, kPGPNetPrefPGPAuthKeyID, &dataSize,
		&pData);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefPGPAuthKeyID, &nFlags);
	if (pData != NULL)
	{
		sGetKeyUserID(pConfig->keyDB, pData, szValue, sizeof(szValue));
		PGPFreeData(pData);
	}
	else
		strcpy(szValue, "None");
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefPGPAuthKeyID);

	LoadItemString(IDS_X509KEY);
	PGPGetPrefData(pConfig->prefRefNet, kPGPNetPrefX509AuthKeyID, &dataSize,
		&pData);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefX509AuthKeyID, &nFlags);
	if (pData != NULL)
	{
		sGetKeyUserID(pConfig->keyDB, pData, szValue, sizeof(szValue));
		PGPFreeData(pData);
	}
	else
		strcpy(szValue, "None");
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefX509AuthKeyID);

	LoadItemString(IDS_REQUIRECONFIGURED);
	PGPGetPrefBoolean(pConfig->prefRefNet, 
		kPGPNetPrefRequireValidKeyConfigured, &bValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefRequireValidKeyConfigured, 
		&nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, 
		kPGPNetPrefRequireValidKeyConfigured);

	LoadItemString(IDS_REQUIREUNCONFIGURED);
	PGPGetPrefBoolean(pConfig->prefRefNet, 
		kPGPNetPrefRequireValidKeyUnconfigured, &bValue);
	PGPGetPrefFlags(pConfig->prefRefNet, 
		kPGPNetPrefRequireValidKeyUnconfigured, &nFlags);
	wsprintf(szText, szFormat, bValue ? szTrue : szFalse);
	InsertItemInTree(PREFFLAGS_DISABLEGUI,
		kPGPNetPrefRequireValidKeyUnconfigured);

	UpdateCatagory();
	tvis.hParent = NULL;
	LoadString(g_hInstance, IDS_NETADVANCED, szText, sizeof(szText));
	InsertItemInTree(IMAGE_UNCHECK, 1);
	tvis.hParent = hItem;

	LoadItemString(IDS_ALLOWEDCIPHERS);
	szValue[0] = 0;
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefAllowCipherNone, &nFlags);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAllowCipherNone, 
		&bValue);
	strcat(szValue, bValue ? "None " : "");
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAllowCipherCAST, 
		&bValue);
	strcat(szValue, bValue ? "CAST " : "");
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAllowCipher3DES, 
		&bValue);
	strcat(szValue, bValue ? "TripleDES " : "");
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAllowCipherDES56, 
		&bValue);
	strcat(szValue, bValue ? "DES56 " : "");
	szValue[strlen(szValue)-1] = 0;
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefAllowCipherNone);

	LoadItemString(IDS_ALLOWEDHASHES);
	szValue[0] = 0;
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefAllowHashNone, &nFlags);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAllowHashNone, 
		&bValue);
	strcat(szValue, bValue ? "None " : "");
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAllowHashSHA1, 
		&bValue);
	strcat(szValue, bValue ? "SHA-1 " : "");
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAllowHashMD5, 
		&bValue);
	strcat(szValue, bValue ? "MD5 " : "");
	szValue[strlen(szValue)-1] = 0;
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefAllowHashNone);

	LoadItemString(IDS_ALLOWEDDH);
	szValue[0] = 0;
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefAllowDH768, &nFlags);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAllowDH768, 
		&bValue);
	strcat(szValue, bValue ? "768 " : "");
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAllowDH1024, 
		&bValue);
	strcat(szValue, bValue ? "1024 " : "");
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAllowDH1536, 
		&bValue);
	strcat(szValue, bValue ? "1536 " : "");
	szValue[strlen(szValue)-1] = 0;
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefAllowDH768);

	LoadItemString(IDS_ALLOWEDCOMP);
	szValue[0] = 0;
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefAllowCompLZS, &nFlags);
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAllowCompLZS, 
		&bValue);
	strcat(szValue, bValue ? "LZS " : "");
	PGPGetPrefBoolean(pConfig->prefRefNet, kPGPNetPrefAllowCompDeflate, 
		&bValue);
	strcat(szValue, bValue ? "Deflate " : "");
	szValue[strlen(szValue)-1] = 0;
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefAllowCompLZS);

	LoadItemString(IDS_IKEPROPOSALS);
	PGPGetPrefArray(pConfig->prefRefNet, kPGPNetPrefIKEProposalArray, 
		&prefArray);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefIKEProposalArray, 
		&nFlags);
	strcpy(szText, szFormat);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefIKEProposalArray);
	hParent = tvis.hParent;
	tvis.hParent = hItem;
	
	for (nIndex=0; nIndex<prefArray->numElements; nIndex++)
	{
		szText[0] = 0;
		PGPGetPrefArrayStruct(pConfig->prefRefNet, prefArray, nIndex,
			&prefStruct);

		switch ((PGPikeAuthMethod) prefStruct->members[0].data)
		{
		case kPGPike_AM_PreSharedKey:
			strcat(szText, "Shared key, ");
			break;

		case kPGPike_AM_DSS_Sig:
			strcat(szText, "DSS signature, ");
			break;

		case kPGPike_AM_RSA_Sig:
			strcat(szText, "RSA signature, ");
			break;
		}

		switch((PGPikeHash) prefStruct->members[1].data)
		{
		case kPGPike_HA_MD5:
			strcat(szText, "MD5, ");
			break;

		case kPGPike_HA_SHA1:
			strcat(szText, "SHA, ");
			break;
		}

		switch((PGPikeCipher) prefStruct->members[2].data)
		{
		case kPGPike_SC_DES_CBC:
			strcat(szText, "DES56, ");
			break;

		case kPGPike_SC_3DES_CBC:
			strcat(szText, "TripleDES, ");
			break;

		case kPGPike_SC_CAST_CBC:
			strcat(szText, "CAST, ");
			break;
		}

		switch((PGPikeGroupID) prefStruct->members[3].data)
		{
		case kPGPike_GR_MODPTwo:
			strcat(szText, "1024");
			break;

		case kPGPike_GR_MODPFive:
			strcat(szText, "1536");
			break;
		}

		InsertItemInTree(IMAGE_ITEM, 0);
	}
	PGPFreePrefArray(prefArray);
	tvis.hParent = hParent;

	LoadItemString(IDS_IPSECPROPOSALS);
	PGPGetPrefArray(pConfig->prefRefNet, kPGPNetPrefIPSecProposalArray, 
		&prefArray);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPNetPrefIPSecProposalArray, 
		&nFlags);
	strcpy(szText, szFormat);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPNetPrefIPSecProposalArray);
	hParent = tvis.hParent;
	tvis.hParent = hItem;
	
	for (nIndex=0; nIndex<prefArray->numElements; nIndex++)
	{
		szText[0] = 0;
		PGPGetPrefArrayStruct(pConfig->prefRefNet, prefArray, nIndex,
			&prefStruct);

		switch ((PGPipsecAHTransformID) prefStruct->members[4].data)
		{
		case kPGPike_AH_None:
			strcat(szText, "None, ");
			break;

		case kPGPike_AH_MD5:
			strcat(szText, "MD5, ");
			break;

		case kPGPike_AH_SHA:
			strcat(szText, "SHA, ");
			break;
		}

		switch((PGPipsecESPTransformID) prefStruct->members[1].data)
		{
		case kPGPike_ET_NULL:
			strcat(szText, "None, ");
			break;

		case kPGPike_ET_DES:
			strcat(szText, "DES56, ");
			break;

		case kPGPike_ET_3DES:
			strcat(szText, "TripleDES, ");
			break;

		case kPGPike_ET_CAST:
			strcat(szText, "CAST, ");
			break;
		}

		switch((PGPipsecAuthAttribute) prefStruct->members[2].data)
		{
		case kPGPike_AA_HMAC_MD5:
			strcat(szText, "MD5, ");
			break;

		case kPGPike_AA_HMAC_SHA:
			strcat(szText, "SHA, ");
			break;

		}

		switch((PGPipsecIPCOMPTransformID) prefStruct->members[7].data)
		{
		case kPGPike_IC_None:
			strcat(szText, "None");
			break;

		case kPGPike_IC_Deflate:
			strcat(szText, "Deflate");
			break;

		case kPGPike_IC_LZS:
			strcat(szText, "LZS");
			break;
		}

		InsertItemInTree(IMAGE_ITEM, 0);
	}
	PGPFreePrefArray(prefArray);
	tvis.hParent = hParent;

	LoadItemString(IDS_PFSPROPOSAL);
	PGPGetPrefNumber(pConfig->prefRefNet, kPGPnetPrefIPSECGroupID, &nValue);
	PGPGetPrefFlags(pConfig->prefRefNet, kPGPnetPrefIPSECGroupID, &nFlags);
	switch (nValue)
	{
	case kPGPike_GR_None:
		strcpy(szValue, "0");
		break;

	case kPGPike_GR_MODPTwo:
		strcpy(szValue, "1024");
		break;

	case kPGPike_GR_MODPFive:
		strcpy(szValue, "1536");
		break;
	}
	wsprintf(szText, szFormat, szValue);
	InsertItemInTree(PREFFLAGS_DISABLEGUI, kPGPnetPrefIPSECGroupID);
	
	UpdateCatagory();
	}
	return;
}


static void sAddHotkeySequence(PGPUInt32 nHotKey, PGPBoolean *pbValue, 
							   char *szValue)
{
	unsigned char uChar = nHotKey & 0xff;
	char szChar[5];

	if (nHotKey & kPGPHotKeyEnabled)
		*pbValue = TRUE;
	else
		*pbValue = FALSE;

	if (nHotKey & kPGPHotKeyCtrlModifier)
		strcat(szValue, "Ctrl + ");
	if (nHotKey & kPGPHotKeyAltModifier)
		strcat(szValue, "Alt + ");
	if (nHotKey & kPGPHotKeyShiftModifier)
		strcat(szValue, "Shift + ");

	if ((uChar >= VK_F1) && (uChar <= VK_F12))
		wsprintf(szChar, "F%d", uChar - VK_F1 + 1);
	else
		wsprintf(szChar, " %c", uChar);

	strcat(szValue, szChar);

	return;
}


static void	sGetKeyUserID(PGPKeyDBRef keyDB, PGPByte *pData,
						  char *szValue, PGPSize nSize)
{
	PGPKeyID keyID;
	PGPKeyDBObjRef key;

	memcpy(keyID.opaqueBytes, pData, sizeof(keyID.opaqueBytes));
	PGPFindKeyByKeyID(keyDB, &keyID, &key);
	PGPclGetPrimaryUserIDName(key, szValue, nSize, &nSize);
	return;
}


static void sInitGUI(HWND hwndDlg, pgpConfigInfo *pConfig)
{
	if (pConfig->bCopyClientPrefs)
		CheckDlgButton(hwndDlg, IDC_INCLUDEOPTIONS, BST_CHECKED);
	else
		CheckDlgButton(hwndDlg, IDC_INCLUDEOPTIONS, BST_UNCHECKED);

	EnableWindow(GetDlgItem(hwndDlg, IDC_SETOPTIONS), pConfig->bCopyClientPrefs);
	EnableWindow(GetDlgItem(hwndDlg, IDC_IMPORTOPTIONS), pConfig->bCopyClientPrefs);
	EnableWindow(GetDlgItem(hwndDlg, IDC_OPTIONSLABEL), 
		pConfig->bCopyClientPrefs);
	EnableWindow(GetDlgItem(hwndDlg, IDC_OPTIONS), pConfig->bCopyClientPrefs);

	if (!(pConfig->bActiveX))
	{
		RECT rcOrig;
		RECT rcList;
		POINT *pOrig = (POINT *) &rcOrig;
		POINT *pList = (POINT *) &rcList;
		LONG lOffset;

		GetWindowRect(GetDlgItem(hwndDlg, IDC_OPTIONSLABEL), &rcOrig);
		GetWindowRect(GetDlgItem(hwndDlg, IDC_OPTIONS), &rcList);
		ScreenToClient(hwndDlg, pOrig);
		ScreenToClient(hwndDlg, pList);
		lOffset = pList->y - pOrig->y;
		GetWindowRect(GetDlgItem(hwndDlg, IDC_OPTIONSPLACE), &rcList);
		ScreenToClient(hwndDlg, pList);
		ShowWindow(GetDlgItem(hwndDlg, IDC_SETOPTIONS), SW_HIDE);
		EnableWindow(GetDlgItem(hwndDlg, IDC_SETOPTIONS), FALSE);
		ShowWindow(GetDlgItem(hwndDlg, IDC_IMPORTOPTIONS), SW_HIDE);
		EnableWindow(GetDlgItem(hwndDlg, IDC_IMPORTOPTIONS), FALSE);
		SetWindowPos(GetDlgItem(hwndDlg, IDC_OPTIONSLABEL), 0, pList->x,
			pList->y, 0, 0, 
			SWP_NOOWNERZORDER | SWP_NOSIZE | SWP_SHOWWINDOW);
		SetWindowPos(GetDlgItem(hwndDlg, IDC_OPTIONS), 0, pList->x,
			pList->y + lOffset, 0, 0, 
			SWP_NOOWNERZORDER | SWP_NOSIZE | SWP_SHOWWINDOW);
	}

	return;
}


static void sSetPrefFlag(pgpConfigInfo *pConfig, PGPUInt32 parentIndex, 
						 PGPUInt32 prefIndex, PGPUInt32 flag)
{
	if (parentIndex == 0)
	{
		if (flag)
			PGPSetPrefFlags(pConfig->prefRef, prefIndex, LOCK_PREF_FLAGS);
		else
			PGPClearPrefFlags(pConfig->prefRef, prefIndex, LOCK_PREF_FLAGS);

		switch (prefIndex)
		{
		case kPGPPrefCacheType:
			prefIndex = kPGPPrefCacheSeconds;
			break;

		case kPGPPrefWordWrapEnable:
			prefIndex = kPGPPrefWordWrapWidth;
			break;

		case kPGPPrefDiskUnmountOnInactivity:
			prefIndex = kPGPPrefDiskAutoUnmountTimeout;
			break;

		case kPGPPrefAutoBackupKeyring:
			if (flag)
			{
				PGPSetPrefFlags(pConfig->prefRef, 
					kPGPPrefAutoBackupToKeyringFolder, LOCK_PREF_FLAGS);
				PGPSetPrefFlags(pConfig->prefRef, 
					kPGPPrefAutoBackupFolder, LOCK_PREF_FLAGS);
			}
			else
			{
				PGPClearPrefFlags(pConfig->prefRef, 
					kPGPPrefAutoBackupToKeyringFolder, LOCK_PREF_FLAGS);
				PGPClearPrefFlags(pConfig->prefRef, 
					kPGPPrefAutoBackupFolder, LOCK_PREF_FLAGS);
			}
			break;

		default:
			return;
		}

		if (flag)
			PGPSetPrefFlags(pConfig->prefRef, prefIndex, LOCK_PREF_FLAGS);
		else
			PGPClearPrefFlags(pConfig->prefRef, prefIndex, LOCK_PREF_FLAGS);
	}
	else
	{
		if (flag)
			PGPSetPrefFlags(pConfig->prefRefNet, prefIndex, LOCK_PREF_FLAGS);
		else
			PGPClearPrefFlags(pConfig->prefRefNet, prefIndex, LOCK_PREF_FLAGS);

		switch (prefIndex)
		{
		case kPGPNetPrefHostArray:
			if (flag)
				pConfig->bCopyNetHosts = TRUE;
			break;

		case kPGPNetPrefEnableIKETimeExpiration:
			prefIndex = kPGPNetPrefIKETimeExpiration;
			break;

		case kPGPNetPrefEnableIKEKByteExpiration:
			prefIndex = kPGPNetPrefIKEKByteExpiration;
			break;

		case kPGPNetPrefEnableIPSECTimeExpiration:
			prefIndex = kPGPNetPrefIPSECTimeExpiration;
			break;

		case kPGPNetPrefEnableIPSECKByteExpiration:
			prefIndex = kPGPNetPrefIPSECKByteExpiration;
			break;

		case kPGPNetPrefAllowCipherNone:
			if (flag)
			{
				PGPSetPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowCipherCAST, LOCK_PREF_FLAGS);
				PGPSetPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowCipher3DES, LOCK_PREF_FLAGS);
				PGPSetPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowCipherDES56, LOCK_PREF_FLAGS);
			}
			else
			{
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowCipherCAST, LOCK_PREF_FLAGS);
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowCipher3DES, LOCK_PREF_FLAGS);
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowCipherDES56, LOCK_PREF_FLAGS);
			}
			return;

		case kPGPNetPrefAllowHashNone:
			if (flag)
			{
				PGPSetPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowHashSHA1, LOCK_PREF_FLAGS);
				PGPSetPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowHashMD5, LOCK_PREF_FLAGS);
			}
			else
			{
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowHashSHA1, LOCK_PREF_FLAGS);
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowHashMD5, LOCK_PREF_FLAGS);
			}
			return;

		case kPGPNetPrefAllowDH768:
			if (flag)
			{
				PGPSetPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowDH1024, LOCK_PREF_FLAGS);
				PGPSetPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowDH1536, LOCK_PREF_FLAGS);
			}
			else
			{
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowDH1024, LOCK_PREF_FLAGS);
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAllowDH1536, LOCK_PREF_FLAGS);
			}
			return;

		case kPGPNetPrefAllowCompLZS:
			prefIndex = kPGPNetPrefAllowCompDeflate;
			break;

		case kPGPnetPrefFirewallRuleSet:
			if (flag)
			{
				PGPSetPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefFirewallRuleArray, LOCK_PREF_FLAGS);
				PGPSetPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefUseLearnMode, LOCK_PREF_FLAGS);
				PGPSetPrefBoolean(pConfig->prefRefNet, 
					kPGPNetPrefUseLearnMode, FALSE);
			}
			else
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefFirewallRuleArray, LOCK_PREF_FLAGS);
			return;

		case kPGPNetPrefUseLearnMode:
			if (!flag)
			{
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPnetPrefFirewallRuleSet, LOCK_PREF_FLAGS);
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefFirewallRuleArray, LOCK_PREF_FLAGS);
			}
			return;

		case kPGPNetPrefAutoBlockAttacks:
			if (flag)
			{
				PGPSetPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefBlockUntilRemoved, LOCK_PREF_FLAGS);
				PGPSetPrefFlags(pConfig->prefRefNet, 
					kPGPnetPrefBlockMinutes, LOCK_PREF_FLAGS);
			}
			else
			{
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefBlockUntilRemoved, LOCK_PREF_FLAGS);
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPnetPrefBlockMinutes, LOCK_PREF_FLAGS);
			}
			break;

		case kPGPNetPrefAttackEmailNotify:
			if (flag)
				PGPSetPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAttackNotifyEmail, LOCK_PREF_FLAGS);
			else
				PGPClearPrefFlags(pConfig->prefRefNet, 
					kPGPNetPrefAttackNotifyEmail, LOCK_PREF_FLAGS);
			break;

		default:
			return;
		}

		if (flag)
			PGPSetPrefFlags(pConfig->prefRefNet, prefIndex, LOCK_PREF_FLAGS);
		else
			PGPClearPrefFlags(pConfig->prefRefNet, prefIndex, LOCK_PREF_FLAGS);
	}

	return;
}


static LPSTR sGetFileDescription(PGPMemoryMgrRef memoryMgr, LPSTR pszFile)
{
	LPSTR	psz				= NULL;

	CHAR	szSubBlock[64];
	INT		i, iNumTrans;
	DWORD	dwDummy;
	DWORD	dwLen;
	LPVOID	pBlock;
	LPVOID	pBuf;
	SHFILEINFO		sfi;

	struct LANGANDCODEPAGE {
		WORD wLanguage;
		WORD wCodePage;
	} *lpTranslate;

	SHGetFileInfo (pszFile, 0, &sfi, sizeof(sfi), SHGFI_DISPLAYNAME);
	dwLen = GetFileVersionInfoSize (pszFile, &dwDummy);
	if (dwLen > 0)
	{
		pBlock = malloc (dwLen);
		if (pBlock != NULL)
		{
			if (GetFileVersionInfo (pszFile, dwDummy, dwLen, pBlock))
			{
				VerQueryValue(pBlock, 
					TEXT("\\VarFileInfo\\Translation"),
					(LPVOID*)&lpTranslate, &dwLen);

				iNumTrans = dwLen / sizeof(struct LANGANDCODEPAGE);
				for (i=0; i<iNumTrans; i++)
				{
					if (lpTranslate[i].wLanguage == 0x0409)
					{
						wsprintf (szSubBlock, 
							TEXT("\\StringFileInfo\\%04x%04x\\FileDescription"),
							lpTranslate[i].wLanguage, lpTranslate[i].wCodePage);

						if (VerQueryValue (pBlock, szSubBlock, &pBuf, &dwLen))
						{
							dwLen += lstrlen (sfi.szDisplayName) +4;
							psz = PGPNewData (memoryMgr, dwLen, 0);
							if (psz)
							{
								lstrcpy (psz, pBuf);
								lstrcat (psz, " (");
								lstrcat (psz, sfi.szDisplayName);
								lstrcat (psz, ")");
							}
							break;
						}
					}
				}
			}

			free (pBlock);
		}
	}

	if (psz == NULL)
	{
		dwLen = lstrlen (sfi.szDisplayName);
		psz = PGPNewData (memoryMgr, dwLen+1, 0);
		if (psz != NULL)
			lstrcpy (psz, sfi.szDisplayName);
	}

	return psz;
}


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
