/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: AdminOptions.c,v 1.37 2002/11/16 19:33:21 pbj Exp $
____________________________________________________________________________*/

#include <windows.h>
#include "PGPadmin.h"
#include "Utils.h"
#include "resource.h"

#include "pgpUtilities.h"
#include "pgpClientLib.h"
#include "pgpClientPrefs.h"
#include "pgpNetPrefs.h"
#include "pgpHashPrefs.h"
#include "pgpFileSpec.h"
#include "pgpVersionHeader.h"

#define DEFAULT_USERINSTALLDIR	"C:\\Program Files\\PGP Corporation\\PGP"


void InitAdminOptions(PGPContextRef pgpContext, pgpConfigInfo *pConfig,
					  PGPBoolean bIsActiveX)
{
	PGPclOpenDefaultKeyrings(pgpContext, kPGPOpenKeyDBFileOptions_None, 
		&(pConfig->keyDB));

	pConfig->pgpContext = pgpContext;
	PGPNewTLSContext(pgpContext, &(pConfig->tlsContext));
	pConfig->memoryMgr = PGPPeekContextMemoryMgr(pConfig->pgpContext);
	pConfig->szAdminInstaller = NULL;
	pConfig->szClientInstaller = NULL;

	pConfig->szUserInstallDir = (char *) PGPNewData(pConfig->memoryMgr,
											strlen(DEFAULT_USERINSTALLDIR)+1, 
											kPGPMemoryMgrFlags_Clear);
	
	strcpy(pConfig->szUserInstallDir, DEFAULT_USERINSTALLDIR);
	strcpy(pConfig->szDefaultKeys, "");
	strtok(pConfig->szDefaultKeys, "."); /* Dummy call to link strtok */
	pConfig->szDefaultKeysBuffer = NULL;
	pConfig->nDefaultKeysBufferSize = 0;
	pConfig->szProductKey = NULL;
	pConfig->bActiveX = bIsActiveX;
	return;
}


void LoadAdminOptions(pgpConfigInfo *pConfig)
{
	PGPPrefRef		prefRef = pConfig->prefRef;
	PGPSize			dataSize;
	PGPByte *		pData;
	PGPPrefArray *	prefArray;
	char *			szInstallDir = NULL;

	PGPGetPrefBoolean(prefRef, kPGPPrefUseOutgoingADK, 
		&(pConfig->bUseOutgoingADK));
	PGPGetPrefBoolean(prefRef, kPGPPrefUseDHADK, 
		&(pConfig->bUseIncomingADK));
	PGPGetPrefBoolean(prefRef, kPGPPrefUsePGPdiskADK, 
		&(pConfig->bUseDiskADK));
	PGPGetPrefBoolean(prefRef, kPGPPrefEnforceIncomingADK, 
		&(pConfig->bEnforceIncomingADK));
	PGPGetPrefBoolean(prefRef, kPGPPrefEnforceOutgoingADK, 
		&(pConfig->bEnforceOutgoingADK));
	PGPGetPrefBoolean(prefRef, kPGPPrefEnforceRemoteADKClass, 
		&(pConfig->bEnforceRemoteADK));
	PGPGetPrefBoolean(prefRef, kPGPPrefEnforceMinChars, 
		&(pConfig->bEnforceMinChars));
	PGPGetPrefBoolean(prefRef, kPGPPrefEnforceMinQuality, 
		&(pConfig->bEnforceMinQuality));
	PGPGetPrefBoolean(prefRef, kPGPPrefAutoSignTrustCorp,
		&(pConfig->bAutoSignTrustCorp));
	PGPGetPrefBoolean(prefRef, kPGPPrefMetaIntroducerCorp,
		&(pConfig->bMetaIntroducerCorp));
	PGPGetPrefBoolean(prefRef, kPGPPrefAllowKeyGeneration,
		&(pConfig->bAllowKeyGen));
	PGPGetPrefBoolean(prefRef, kPGPPrefPresetKeyGeneration,
		&(pConfig->bSetKeyGen));
	PGPGetPrefBoolean(prefRef, kPGPPrefGenerateKeysOnSmartCard, 
		&(pConfig->bUseSmartCard));
	PGPGetPrefBoolean(prefRef, kPGPPrefAllowConventionalEncryption,
		&(pConfig->bAllowConventionalEncryption));
	PGPGetPrefBoolean(prefRef, kPGPPrefAllowEncryptInvalidKeys,
		&(pConfig->bAllowEncryptInvalidKeys));
	PGPGetPrefBoolean(prefRef, kPGPPrefAllowKeySigning,
		&(pConfig->bAllowKeySigning));
	PGPGetPrefBoolean(prefRef, kPGPPrefAutoAddExchangeServerAddress,
		&(pConfig->bAutoAddExchangeServerAddress));
	PGPGetPrefBoolean(prefRef, kPGPPrefAddLotusNotesUserID,
		&(pConfig->bAutoAddLotusNotesAddress));
	PGPGetPrefBoolean(prefRef, kPGPPrefUpdateAllKeys,
		&(pConfig->bUpdateAllKeys));
	PGPGetPrefBoolean(prefRef, kPGPPrefUpdateTrustedIntroducers,
		&(pConfig->bUpdateTrustedIntroducers));
	PGPGetPrefBoolean(prefRef, kPGPPrefAutoAddRevoker,
		&(pConfig->bAutoAddRevoker));
	PGPGetPrefBoolean(prefRef, kPGPPrefKeyGenX509CertRequest,
		&(pConfig->bKeyGenCertRequest));
	PGPGetPrefBoolean(prefRef, kPGPPrefAllowManualX509CertRequest,
		&(pConfig->bAllowManualCertRequest));
	PGPGetPrefBoolean(prefRef, kPGPPrefAutoUpdateX509CRL,
		&(pConfig->bAutoUpdateCRL));
	PGPGetPrefBoolean(prefRef, kPGPPrefUpdatePrefs,
		&(pConfig->bUpdateAdminPrefs));
	PGPGetPrefBoolean(prefRef, kPGPPrefAlwaysSendKeys,
		&(pConfig->bAlwaysSendKeys));
	PGPGetPrefBoolean(prefRef, kPGPPrefAutoKeyReconstruction,
		&(pConfig->bAutoKeyReconstruction));
	PGPGetPrefBoolean(prefRef, kPGPPrefAutoSignRootCAKey,
		&(pConfig->bAutoSignRootCAKey));
	PGPGetPrefBoolean(prefRef, kPGPPrefCopyClientPrefs, 
		&(pConfig->bCopyClientPrefs));
	PGPGetPrefBoolean(prefRef, kPGPPrefRemainSilentAfterInstall,
		&(pConfig->bRemainSilentAfterInstall));
	PGPGetPrefBoolean(prefRef, kPGPPrefPreselectInstall, 
		&(pConfig->bPreselectInstall));
	PGPGetPrefBoolean(prefRef, kPGPPrefInstallPrograms, 
		&(pConfig->bInstallPrograms));
	PGPGetPrefBoolean(prefRef, kPGPPrefInstallLogin, 
		&(pConfig->bInstallLogin));
	PGPGetPrefBoolean(prefRef, kPGPPrefInstallDisk, 
		&(pConfig->bInstallDisk));
	PGPGetPrefBoolean(prefRef, kPGPPrefInstallEudora, 
		&(pConfig->bInstallEudora));
	PGPGetPrefBoolean(prefRef, kPGPPrefInstallOutlook, 
		&(pConfig->bInstallOutlook));
	PGPGetPrefBoolean(prefRef, kPGPPrefInstallOutlookExpress, 
		&(pConfig->bInstallOutExpress));
	PGPGetPrefBoolean(prefRef, kPGPPrefInstallICQ, 
		&(pConfig->bInstallICQ));
	PGPGetPrefBoolean(prefRef, kPGPPrefInstallLotusNotes, 
		&(pConfig->bInstallLotus));
	PGPGetPrefBoolean(prefRef, kPGPPrefInstallGroupwise, 
		&(pConfig->bInstallGroupwise));
	PGPGetPrefBoolean(prefRef, kPGPPrefInstallDocumentation, 
		&(pConfig->bInstallManual));
	PGPGetPrefBoolean(prefRef, kPGPPrefUninstallOldPGP, 
		&(pConfig->bUninstallOld));
	PGPGetPrefBoolean(prefRef, kPGPPrefRebootAfterSilentInstall, 
		&(pConfig->bAutoReboot));

	PGPGetPrefBoolean(prefRef, kPGPPrefDiskCreate, 
		&(pConfig->bDiskCreate));
	
	PGPGetPrefNumber(prefRef, kPGPPrefMinChars, 
		&(pConfig->nMinChars));
	PGPGetPrefNumber(prefRef, kPGPPrefMinQuality, 
		&(pConfig->nMinQuality));
	PGPGetPrefNumber(prefRef, kPGPPrefKeySize, 
		&(pConfig->nKeySize));
	PGPGetPrefNumber(prefRef, kPGPPrefLDAPReconServerType, 
		&(pConfig->nReconServerType));
	PGPGetPrefNumber(prefRef, kPGPPrefDaysUpdateAllKeys,
		&(pConfig->nDaysUpdateAllKeys));
	PGPGetPrefNumber(prefRef, kPGPPrefDaysUpdateTrustedIntroducers,
		&(pConfig->nDaysUpdateTrustedIntroducers));
	PGPGetPrefNumber(prefRef, kPGPPrefDaysUpdatePrefs,
		&(pConfig->nDaysUpdatePrefs));
	PGPGetPrefNumber(prefRef, kPGPPrefKeyPublicKeyAlgorithm,
		&(pConfig->keyType));
	PGPGetPrefNumber(prefRef, kPGPPrefKeyExpiration,
		&(pConfig->keyExpiration));
	PGPGetPrefNumber(prefRef, kPGPPrefCorpKeyTrustDepth,
		&(pConfig->nCorpKeyTrustDepth));
	PGPGetPrefNumber(prefRef, kPGPPrefCertRequestType,
		&(pConfig->nCertRequestType));

	PGPGetPrefNumber(prefRef, kPGPPrefDiskSize,
		&(pConfig->nDiskSize));
	PGPGetPrefNumber(prefRef, kPGPPrefDiskAlgorithm,
		&(pConfig->nDiskAlgorithm));
	PGPGetPrefNumber(prefRef, kPGPPrefDiskFileSystem,
		&(pConfig->nDiskFileSystem));

	PGPGetPrefStringAlloc(prefRef, kPGPPrefLDAPPrefsServer, 
		&(pConfig->szLDAPServer));
	
	PGPGetPrefStringAlloc(prefRef, kPGPPrefLDAPReconServer, 
		&(pConfig->szReconServer));
	
	PGPGetPrefStringAlloc(prefRef, kPGPPrefInstallDir, 
		&szInstallDir);

	if (szInstallDir != NULL)
	{
		if (strlen(szInstallDir) > 0)
		{
			PGPFreeData(pConfig->szUserInstallDir);
			pConfig->szUserInstallDir = szInstallDir;
		}
		else
			PGPFreeData(szInstallDir);
	}
	
	PGPGetPrefData(prefRef, kPGPPrefOutgoingADKID, &dataSize, &pData);
	if (pData != NULL)
	{
		memcpy(pConfig->outgoingADKID.opaqueBytes, pData, dataSize);
		PGPFreeData(pData);
		pConfig->bUseOutgoingADK = TRUE;
	}
	else
		pConfig->bUseOutgoingADK = FALSE;
	
	PGPGetPrefData(prefRef, kPGPPrefDHADKID, &dataSize, &pData);
	if (pData != NULL)
	{
		memcpy(pConfig->incomingADKID.opaqueBytes, pData, dataSize);
		PGPFreeData(pData);
		pConfig->bUseIncomingADK = TRUE;
	}
	else
		pConfig->bUseIncomingADK = FALSE;
	
	PGPGetPrefData(prefRef, kPGPPrefPGPdiskADKKeyID, &dataSize, &pData);
	if (pData != NULL)
	{
		memcpy(pConfig->diskADKID.opaqueBytes, pData, dataSize);
		PGPFreeData(pData);
		pConfig->bUseDiskADK = TRUE;
	}
	else
		pConfig->bUseDiskADK = FALSE;
	
	PGPGetPrefData(prefRef, kPGPPrefCorpKeyID, &dataSize, &pData);
	if (pData != NULL)
	{
		memcpy(pConfig->corpKeyID.opaqueBytes, pData, dataSize);
		PGPFreeData(pData);
		pConfig->bAutoSignTrustCorp = TRUE;
	}
	else
		pConfig->bAutoSignTrustCorp = FALSE;
	
	PGPGetPrefData(prefRef, kPGPPrefRevokerKeyID, &dataSize, &pData);
	if (pData != NULL)
	{
		memcpy(pConfig->revokerKeyID.opaqueBytes, pData, dataSize);
		PGPFreeData(pData);
		pConfig->bAutoAddRevoker = TRUE;
	}
	else
		pConfig->bAutoAddRevoker = FALSE;
	
	pConfig->avData = NULL;
	pConfig->avDataSize = 0;
	PGPGetPrefNumber(prefRef, kPGPPrefCAType, &(pConfig->nCAType));
	if (pConfig->nCAType != kPGPKeyServerClass_Invalid)
		PGPGetPrefData(prefRef, kPGPPrefExtraAVPairs, &(pConfig->avDataSize), 
			&(pConfig->avData));

	pConfig->defaultKeys = NULL;
	pConfig->nNumDefaultKeys = 0;

	PGPGetPrefArray(prefRef, kPGPPrefDefaultKeys, &prefArray);
	if (prefArray != NULL)
	{
		PGPSize dataSize;
		void *pData;

		PGPGetPrefArrayByte(prefRef, prefArray, 0, &dataSize, &pData);
		if (pData != NULL)
		{
			UINT nIndex;

			PGPFreeData(pData);

			pConfig->defaultKeys = (PGPKeyID *) PGPNewData(pConfig->memoryMgr,
												sizeof(PGPKeyID) *
												prefArray->numElements,
												kPGPMemoryMgrFlags_Clear);

			pConfig->nNumDefaultKeys = prefArray->numElements;

			for (nIndex=0; nIndex<prefArray->numElements; nIndex++)
			{
				PGPGetPrefArrayByte(prefRef, prefArray, nIndex, &dataSize,
					&pData);
				memcpy(pConfig->defaultKeys[nIndex].opaqueBytes, pData,
					dataSize);
				PGPFreeData(pData);
			}
		}

		PGPFreePrefArray(prefArray);
	}
	
	return;
}


void AdminOptions(HWND hwndMain, pgpConfigInfo *pConfig)
{
	PROPSHEETPAGE		psp[NUM_ADMIN_PAGES];
	PROPSHEETHEADER		psh;
	int					nIndex;
	int					nNumPages;
	InstallerProduct	nProduct;

	pConfig->bCancel = FALSE;
	pConfig->bCenter = TRUE;
	pConfig->nPageCount = 0;
	pConfig->nApplyCount = 0;
	pConfig->bApplied = FALSE;
	nNumPages = 0;
	nProduct = GetInstalledProduct(NULL);

	// Set the values common to all pages

	for (nIndex=0; nIndex<NUM_ADMIN_PAGES; nIndex++)
	{
		psp[nIndex].dwSize		= sizeof(PROPSHEETPAGE);
		psp[nIndex].dwFlags		= PSP_HASHELP;
		psp[nIndex].hInstance	= g_hInstance;
		psp[nIndex].pszTemplate	= NULL;
		psp[nIndex].hIcon		= NULL;
		psp[nIndex].pszTitle	= NULL;
		psp[nIndex].pfnDlgProc	= NULL;
		psp[nIndex].lParam		= (LPARAM) pConfig;
		psp[nIndex].pfnCallback	= NULL;
		psp[nIndex].pcRefParent	= NULL;
	}

	if (nProduct & (Installer_PGPds | Installer_PGPmail | Installer_PGPdisk | 
		Installer_PGPvpn))
	{
		// Set up the additional decryption key page
		
		psp[nNumPages].pszTemplate	= MAKEINTRESOURCE(IDD_ADK);
		psp[nNumPages].pfnDlgProc	= (DLGPROC) ADKDlgProc;
		nNumPages++;
		
		// Set up the pass phrase page
		
		psp[nNumPages].pszTemplate	= MAKEINTRESOURCE(IDD_PASSPHRASE);
		psp[nNumPages].pfnDlgProc	= (DLGPROC) PassPhraseDlgProc;
		nNumPages++;
		
		// Set up the key generation page
		
		psp[nNumPages].pszTemplate	= MAKEINTRESOURCE(IDD_KEYGEN);
		psp[nNumPages].pfnDlgProc	= (DLGPROC) KeyGenDlgProc;
		nNumPages++;
		
		// Set up the default key selection page
		
		psp[nNumPages].pszTemplate	= MAKEINTRESOURCE(IDD_DEFKEYSEL);
		psp[nNumPages].pfnDlgProc	= (DLGPROC) DefaultKeysDlgProc;
		nNumPages++;
		
		// Set up the corporate key page
		
		psp[nNumPages].pszTemplate	= MAKEINTRESOURCE(IDD_CORPKEY);
		psp[nNumPages].pfnDlgProc	= (DLGPROC) CorpKeyDlgProc;
		nNumPages++;

		// Set up the revoker key page
		
		psp[nNumPages].pszTemplate	= MAKEINTRESOURCE(IDD_REVOKER);
		psp[nNumPages].pfnDlgProc	= (DLGPROC) RevokerDlgProc;
		nNumPages++;
	}
	
	// Set up the server updates page

	psp[nNumPages].pszTemplate	= MAKEINTRESOURCE(IDD_SERVERUPDATES);
	psp[nNumPages].pfnDlgProc	= (DLGPROC) ServerUpdatesDlgProc;
	nNumPages++;
	
	// Set up the access page

	psp[nNumPages].pszTemplate	= MAKEINTRESOURCE(IDD_ACCESS);
	psp[nNumPages].pfnDlgProc	= (DLGPROC) AccessDlgProc;
	nNumPages++;
	
	// Set up the install options page

	psp[nNumPages].pszTemplate	= MAKEINTRESOURCE(IDD_INSTALLOPTIONS);
	psp[nNumPages].pfnDlgProc	= (DLGPROC) InstallOptionsDlgProc;
	nNumPages++;
	
	if (nProduct & (Installer_PGPds | Installer_PGPmail | Installer_PGPdisk | 
		Installer_PGPvpn))
	{
		// Set up the miscellaneous page
		
		psp[nNumPages].pszTemplate	= MAKEINTRESOURCE(IDD_MISC);
		psp[nNumPages].pfnDlgProc	= (DLGPROC) MiscDlgProc;
		nNumPages++;
	}
	
	// Create the header

	psh.dwSize		= sizeof(PROPSHEETHEADER);
	psh.dwFlags		= PSH_PROPSHEETPAGE;
	psh.hwndParent	= hwndMain;
	psh.hInstance	= g_hInstance;
	psh.hIcon		= NULL;
	psh.pszCaption	= NULL;
	psh.nPages		= nNumPages;
	psh.nStartPage	= 0;
	psh.ppsp		= psp;
	psh.pfnCallback	= NULL;

	// Display the Property Sheet - doesn't return until OK or Cancel

	PropertySheet(&psh);

	if (!(pConfig->bCancel) && !(pConfig->bApplied))
		pConfig->bSave = TRUE;

	return;
}


void SaveAdminOptions(pgpConfigInfo *pConfig)
{
	PGPPrefRef		prefRef;
	PGPPrefArray *	prefArray;
	PGPUInt32		index;
	PGPKeyDBObjRef	sigCA;

	prefRef = pConfig->prefRef;

	PGPSetPrefBoolean(prefRef, kPGPPrefUseOutgoingADK, 
		pConfig->bUseOutgoingADK);
	PGPSetPrefBoolean(prefRef, kPGPPrefUseDHADK, 
		pConfig->bUseIncomingADK);
	PGPSetPrefBoolean(prefRef, kPGPPrefUsePGPdiskADK, 
		pConfig->bUseDiskADK);
	PGPSetPrefBoolean(prefRef, kPGPPrefEnforceIncomingADK, 
		pConfig->bEnforceIncomingADK);
	PGPSetPrefBoolean(prefRef, kPGPPrefEnforceOutgoingADK, 
		pConfig->bEnforceOutgoingADK);
	PGPSetPrefBoolean(prefRef, kPGPPrefEnforceRemoteADKClass, 
		pConfig->bEnforceRemoteADK);
	PGPSetPrefBoolean(prefRef, kPGPPrefEnforceMinChars, 
		pConfig->bEnforceMinChars);
	PGPSetPrefBoolean(prefRef, kPGPPrefEnforceMinQuality, 
		pConfig->bEnforceMinQuality);
	PGPSetPrefBoolean(prefRef, kPGPPrefAutoSignTrustCorp,
		pConfig->bAutoSignTrustCorp);
	PGPSetPrefBoolean(prefRef, kPGPPrefMetaIntroducerCorp,
		pConfig->bMetaIntroducerCorp);
	PGPSetPrefBoolean(prefRef, kPGPPrefAllowKeyGeneration,
		pConfig->bAllowKeyGen);
	PGPSetPrefBoolean(prefRef, kPGPPrefPresetKeyGeneration,
		pConfig->bSetKeyGen);
	PGPSetPrefBoolean(prefRef, kPGPPrefGenerateKeysOnSmartCard,
		pConfig->bUseSmartCard);
	PGPSetPrefBoolean(prefRef, kPGPPrefAllowConventionalEncryption,
		pConfig->bAllowConventionalEncryption);
	PGPSetPrefBoolean(prefRef, kPGPPrefAllowEncryptInvalidKeys,
		pConfig->bAllowEncryptInvalidKeys);
	PGPSetPrefBoolean(prefRef, kPGPPrefAllowKeySigning,
		pConfig->bAllowKeySigning);
	PGPSetPrefBoolean(prefRef, kPGPPrefAutoAddExchangeServerAddress,
		pConfig->bAutoAddExchangeServerAddress);
	PGPSetPrefBoolean(prefRef, kPGPPrefAddLotusNotesUserID,
		pConfig->bAutoAddLotusNotesAddress);
	PGPSetPrefBoolean(prefRef, kPGPPrefUpdateAllKeys,
		pConfig->bUpdateAllKeys);
	PGPSetPrefBoolean(prefRef, kPGPPrefUpdateTrustedIntroducers,
		pConfig->bUpdateTrustedIntroducers);
	PGPSetPrefBoolean(prefRef, kPGPPrefAutoAddRevoker,
		pConfig->bAutoAddRevoker);
	PGPSetPrefBoolean(prefRef, kPGPPrefKeyGenX509CertRequest,
		pConfig->bKeyGenCertRequest);
	PGPSetPrefBoolean(prefRef, kPGPPrefAllowManualX509CertRequest,
		pConfig->bAllowManualCertRequest);
	PGPSetPrefBoolean(prefRef, kPGPPrefAutoUpdateX509CRL,
		pConfig->bAutoUpdateCRL);
	PGPSetPrefBoolean(prefRef, kPGPPrefUpdatePrefs,
		pConfig->bUpdateAdminPrefs);
	PGPSetPrefBoolean(prefRef, kPGPPrefAlwaysSendKeys,
		pConfig->bAlwaysSendKeys);
	PGPSetPrefBoolean(prefRef, kPGPPrefAutoKeyReconstruction,
		pConfig->bAutoKeyReconstruction);
	PGPSetPrefBoolean(prefRef, kPGPPrefAutoSignRootCAKey,
		pConfig->bAutoSignRootCAKey);
	PGPSetPrefBoolean(prefRef, kPGPPrefCopyClientPrefs, 
		pConfig->bCopyClientPrefs);
	PGPSetPrefBoolean(prefRef, kPGPPrefRemainSilentAfterInstall,
		pConfig->bRemainSilentAfterInstall);
	PGPSetPrefBoolean(prefRef, kPGPPrefPreselectInstall, 
		pConfig->bPreselectInstall);
	PGPSetPrefBoolean(prefRef, kPGPPrefInstallPrograms, 
		pConfig->bInstallPrograms);
	PGPSetPrefBoolean(prefRef, kPGPPrefInstallLogin, 
		pConfig->bInstallLogin);
	PGPSetPrefBoolean(prefRef, kPGPPrefInstallDisk, 
		pConfig->bInstallDisk);
	PGPSetPrefBoolean(prefRef, kPGPPrefInstallEudora, 
		pConfig->bInstallEudora);
	PGPSetPrefBoolean(prefRef, kPGPPrefInstallOutlook, 
		pConfig->bInstallOutlook);
	PGPSetPrefBoolean(prefRef, kPGPPrefInstallOutlookExpress, 
		pConfig->bInstallOutExpress);
	PGPSetPrefBoolean(prefRef, kPGPPrefInstallICQ, 
		pConfig->bInstallICQ);
	PGPSetPrefBoolean(prefRef, kPGPPrefInstallLotusNotes, 
		pConfig->bInstallLotus);
	PGPSetPrefBoolean(prefRef, kPGPPrefInstallGroupwise, 
		pConfig->bInstallGroupwise);
	PGPSetPrefBoolean(prefRef, kPGPPrefInstallDocumentation, 
		pConfig->bInstallManual);
	PGPSetPrefBoolean(prefRef, kPGPPrefUninstallOldPGP, 
		pConfig->bUninstallOld);
	PGPSetPrefBoolean(prefRef, kPGPPrefRebootAfterSilentInstall, 
		pConfig->bAutoReboot);
	
	PGPSetPrefBoolean(prefRef, kPGPPrefDiskCreate, 
		pConfig->bDiskCreate);
	
	PGPSetPrefNumber(prefRef, kPGPPrefMinChars, 
		pConfig->nMinChars);
	PGPSetPrefNumber(prefRef, kPGPPrefMinQuality, 
		pConfig->nMinQuality);
	PGPSetPrefNumber(prefRef, kPGPPrefKeySize, 
		pConfig->nKeySize);
	PGPSetPrefNumber(prefRef, kPGPPrefLDAPReconServerType, 
		pConfig->nReconServerType);
	PGPSetPrefNumber(prefRef, kPGPPrefDaysUpdateAllKeys,
		pConfig->nDaysUpdateAllKeys);
	PGPSetPrefNumber(prefRef, kPGPPrefDaysUpdateTrustedIntroducers,
		pConfig->nDaysUpdateTrustedIntroducers);
	PGPSetPrefNumber(prefRef, kPGPPrefDaysUpdatePrefs,
		pConfig->nDaysUpdatePrefs);
	PGPSetPrefNumber(prefRef, kPGPPrefKeyPublicKeyAlgorithm,
		pConfig->keyType);
	PGPSetPrefNumber(prefRef, kPGPPrefKeyExpiration,
		pConfig->keyExpiration);
	PGPSetPrefNumber(prefRef, kPGPPrefCorpKeyTrustDepth,
		pConfig->nCorpKeyTrustDepth);
	PGPSetPrefNumber(prefRef, kPGPPrefCertRequestType,
		pConfig->nCertRequestType);

	PGPSetPrefNumber(prefRef, kPGPPrefDiskSize,
		pConfig->nDiskSize);
	PGPSetPrefNumber(prefRef, kPGPPrefDiskAlgorithm,
		pConfig->nDiskAlgorithm);
	PGPSetPrefNumber(prefRef, kPGPPrefDiskFileSystem,
		pConfig->nDiskFileSystem);

	if (pConfig->szLDAPServer != NULL)
		PGPSetPrefString(prefRef, kPGPPrefLDAPPrefsServer, 
			pConfig->szLDAPServer);
	else
		PGPSetPrefString(prefRef, kPGPPrefLDAPPrefsServer, "");

	if (pConfig->szReconServer != NULL)
		PGPSetPrefString(prefRef, kPGPPrefLDAPReconServer, 
			pConfig->szReconServer);
	else
		PGPSetPrefString(prefRef, kPGPPrefLDAPReconServer, "");

	if (pConfig->szUserInstallDir != NULL)
		PGPSetPrefString(prefRef, kPGPPrefInstallDir, 
			pConfig->szUserInstallDir);
	else
		PGPSetPrefString(prefRef, kPGPPrefInstallDir, "");

	if (pConfig->bUseOutgoingADK)
		PGPSetPrefData(prefRef, kPGPPrefOutgoingADKID, sizeof(PGPKeyID), 
			pConfig->outgoingADKID.opaqueBytes);
	else
		PGPSetPrefData(prefRef, kPGPPrefOutgoingADKID, 0, NULL);
	
	if (pConfig->bUseIncomingADK)
		PGPSetPrefData(prefRef, kPGPPrefDHADKID, sizeof(PGPKeyID), 
			pConfig->incomingADKID.opaqueBytes);
	else
		PGPSetPrefData(prefRef, kPGPPrefDHADKID, 0, NULL);
	
	if (pConfig->bUseDiskADK)
		PGPSetPrefData(prefRef, kPGPPrefPGPdiskADKKeyID, sizeof(PGPKeyID), 
			pConfig->diskADKID.opaqueBytes);
	else
		PGPSetPrefData(prefRef, kPGPPrefPGPdiskADKKeyID, 0, NULL);
	
	if (pConfig->bAutoSignTrustCorp)
		PGPSetPrefData(prefRef, kPGPPrefCorpKeyID, sizeof(PGPKeyID), 
			pConfig->corpKeyID.opaqueBytes);
	else
		PGPSetPrefData(prefRef, kPGPPrefCorpKeyID, 0, NULL);
	
	if (pConfig->bAutoAddRevoker)
		PGPSetPrefData(prefRef, kPGPPrefRevokerKeyID, sizeof(PGPKeyID), 
			pConfig->revokerKeyID.opaqueBytes);
	else
		PGPSetPrefData(prefRef, kPGPPrefRevokerKeyID, 0, NULL);
	
	// Save the AV list, if specified
	
	PGPSetPrefNumber(prefRef, kPGPPrefCAType, pConfig->nCAType);
	if (pConfig->nCAType != kPGPKeyServerClass_Invalid)
		PGPSetPrefData(prefRef, kPGPPrefExtraAVPairs, pConfig->avDataSize, 
				pConfig->avData);
	
	// See if the root CA key needs to be added to the default keys

	if (IsntPGPError(PGPclGetRootCACertPrefs(pConfig->pgpContext, prefRef,
		pConfig->keyDB, &sigCA)))
	{
		PGPKeyDBObjRef	keyCA;
		PGPKeyID 		caKeyID;
		BOOL bGotCA = FALSE;

		keyCA = PGPPeekKeyDBObjKey(sigCA);
		PGPGetKeyID(keyCA, &caKeyID);
		
		if (pConfig->defaultKeys != NULL)
		{
			for (index=0; index<pConfig->nNumDefaultKeys; index++)
			{
				if (!memcmp(pConfig->defaultKeys[index].opaqueBytes, 
					caKeyID.opaqueBytes, sizeof(PGPKeyID)))
				{
					bGotCA = TRUE;
					break;
				}
			}
		}
		
		if (!bGotCA)
		{
			PGPReallocData(pConfig->memoryMgr, &(pConfig->defaultKeys),
				sizeof(PGPKeyID) * (pConfig->nNumDefaultKeys + 1), 0);
			
			memcpy(pConfig->defaultKeys[pConfig->nNumDefaultKeys].opaqueBytes,
				caKeyID.opaqueBytes, sizeof(PGPKeyID));
			pConfig->nNumDefaultKeys++;
		}
	}

	// Add any ADK's to the default keys

	if (pConfig->bUseDiskADK) 
	{
		BOOL bGotKey = FALSE;

		for (index=0; index<pConfig->nNumDefaultKeys; index++)
		{
			if (!memcmp(pConfig->defaultKeys[index].opaqueBytes, 
				pConfig->diskADKID.opaqueBytes, sizeof(PGPKeyID)))
			{
				bGotKey = TRUE;
				break;
			}
		}

		if (!bGotKey)
		{
			PGPReallocData(pConfig->memoryMgr, &(pConfig->defaultKeys),
				sizeof(PGPKeyID) * (pConfig->nNumDefaultKeys + 1), 0);
			memcpy(pConfig->defaultKeys[pConfig->nNumDefaultKeys].opaqueBytes,
				pConfig->diskADKID.opaqueBytes, sizeof(PGPKeyID));
			pConfig->nNumDefaultKeys++;
		}
	}

	if (pConfig->bUseIncomingADK) 
	{
		BOOL bGotKey = FALSE;

		for (index=0; index<pConfig->nNumDefaultKeys; index++)
		{
			if (!memcmp(pConfig->defaultKeys[index].opaqueBytes, 
				pConfig->incomingADKID.opaqueBytes, sizeof(PGPKeyID)))
			{
				bGotKey = TRUE;
				break;
			}
		}

		if (!bGotKey)
		{
			PGPReallocData(pConfig->memoryMgr, &(pConfig->defaultKeys),
				sizeof(PGPKeyID) * (pConfig->nNumDefaultKeys + 1), 0);
			memcpy(pConfig->defaultKeys[pConfig->nNumDefaultKeys].opaqueBytes,
				pConfig->incomingADKID.opaqueBytes, sizeof(PGPKeyID));
			pConfig->nNumDefaultKeys++;
		}
	}

	if (pConfig->bUseOutgoingADK) 
	{
		BOOL bGotKey = FALSE;

		for (index=0; index<pConfig->nNumDefaultKeys; index++)
		{
			if (!memcmp(pConfig->defaultKeys[index].opaqueBytes, 
				pConfig->outgoingADKID.opaqueBytes, sizeof(PGPKeyID)))
			{
				bGotKey = TRUE;
				break;
			}
		}

		if (!bGotKey)
		{
			PGPReallocData(pConfig->memoryMgr, &(pConfig->defaultKeys),
				sizeof(PGPKeyID) * (pConfig->nNumDefaultKeys + 1), 0);
			memcpy(pConfig->defaultKeys[pConfig->nNumDefaultKeys].opaqueBytes,
				pConfig->outgoingADKID.opaqueBytes, sizeof(PGPKeyID));
			pConfig->nNumDefaultKeys++;
		}
	}

	// Add the Corporate Key

	if (pConfig->bAutoSignTrustCorp) 
	{
		BOOL bGotKey = FALSE;

		for (index=0; index<pConfig->nNumDefaultKeys; index++)
		{
			if (!memcmp(pConfig->defaultKeys[index].opaqueBytes, 
				pConfig->corpKeyID.opaqueBytes, sizeof(PGPKeyID)))
			{
				bGotKey = TRUE;
				break;
			}
		}

		if (!bGotKey)
		{
			PGPReallocData(pConfig->memoryMgr, &(pConfig->defaultKeys),
				sizeof(PGPKeyID) * (pConfig->nNumDefaultKeys + 1), 0);
			memcpy(pConfig->defaultKeys[pConfig->nNumDefaultKeys].opaqueBytes,
				pConfig->corpKeyID.opaqueBytes, sizeof(PGPKeyID));
			pConfig->nNumDefaultKeys++;
		}
	}

	// Add the Designated Revoker Key

	if (pConfig->bAutoAddRevoker) 
	{
		BOOL bGotKey = FALSE;

		for (index=0; index<pConfig->nNumDefaultKeys; index++)
		{
			if (!memcmp(pConfig->defaultKeys[index].opaqueBytes, 
				pConfig->revokerKeyID.opaqueBytes, sizeof(PGPKeyID)))
			{
				bGotKey = TRUE;
				break;
			}
		}

		if (!bGotKey)
		{
			PGPReallocData(pConfig->memoryMgr, &(pConfig->defaultKeys),
				sizeof(PGPKeyID) * (pConfig->nNumDefaultKeys + 1), 0);
			memcpy(pConfig->defaultKeys[pConfig->nNumDefaultKeys].opaqueBytes,
				pConfig->revokerKeyID.opaqueBytes, sizeof(PGPKeyID));
			pConfig->nNumDefaultKeys++;
		}
	}

	// Save the default keys if any were specified
	
	if (pConfig->szDefaultKeysBuffer != NULL)
	{
		PGPFreeData(pConfig->szDefaultKeysBuffer);
		pConfig->szDefaultKeysBuffer = NULL;
	}

	if (pConfig->defaultKeys != NULL)
	{
		char szComments[256];
		PGPKeyDBRef newKeyDB;
		PGPKeySetRef defaultKeySet;
		PGPKeyDBObjRef key;
		PGPFileSpecRef defaultKeyFile = NULL;
		PGPOptionListRef options;

		PGPCreatePrefArray(prefRef, kPGPPrefType_Byte, pConfig->nNumDefaultKeys,
			NULL, &prefArray);

		PGPNewKeyDB(pConfig->pgpContext, &newKeyDB);
		PGPNewEmptyKeySet(pConfig->keyDB, &defaultKeySet);

		for (index=0; index<pConfig->nNumDefaultKeys; index++)
		{
			PGPSetPrefArrayByte(prefRef, prefArray, index, 
				sizeof(pConfig->defaultKeys[index].opaqueBytes), 
				pConfig->defaultKeys[index].opaqueBytes);

			PGPFindKeyByKeyID(pConfig->keyDB, &(pConfig->defaultKeys[index]), &key);
			PGPAddKey(key, defaultKeySet);
		}

		PGPSetPrefArray(prefRef, kPGPPrefDefaultKeys, prefArray);
		PGPFreePrefArray(prefArray);
	
		szComments[0] = 0;
		PGPGetPrefStringBuffer(prefRef, kPGPPrefComment, sizeof(szComments),
			szComments);

		if (strlen(szComments) > 0)
		{
			PGPBuildOptionList(pConfig->pgpContext, &options,
				PGPOCommentString(pConfig->pgpContext, szComments),
				PGPOLastOption(pConfig->pgpContext));
		}
		else
		{
			PGPBuildOptionList(pConfig->pgpContext, &options,
				PGPOLastOption(pConfig->pgpContext));
		}

		if (strlen(pConfig->szDefaultKeys) > 0)
		{
			PGPNewFileSpecFromFullPath(pConfig->pgpContext, pConfig->szDefaultKeys, 
				&defaultKeyFile);
			PGPAppendOptionList(options,
				PGPOOutputFile(pConfig->pgpContext, defaultKeyFile),
				PGPOLastOption(pConfig->pgpContext));
		}
		else
		{
			PGPAppendOptionList(options,
				PGPOAllocatedOutputBuffer(pConfig->pgpContext, 
					&(pConfig->szDefaultKeysBuffer),
					INT_MAX,
					&(pConfig->nDefaultKeysBufferSize)),
				PGPOLastOption(pConfig->pgpContext));
		}

		PGPExport(pConfig->pgpContext,
			PGPOExportKeySet(pConfig->pgpContext, defaultKeySet),
			PGPOExportFormat(pConfig->pgpContext, kPGPExportFormat_Complete),
			PGPOVersionString(pConfig->pgpContext, pgpVersionHeaderString()),
			options,
			PGPOExportPrivateKeys(pConfig->pgpContext, FALSE),
			PGPOArmorOutput(pConfig->pgpContext, TRUE),
			PGPOLastOption(pConfig->pgpContext));

		if (defaultKeyFile != NULL)
			PGPFreeFileSpec(defaultKeyFile);

		PGPFreeOptionList(options);
		PGPFreeKeySet(defaultKeySet);
		PGPFreeKeyDB(newKeyDB);
	}
	else
		PGPSetPrefArray(prefRef, kPGPPrefDefaultKeys, &DefaultKeys);
	
	if (!pConfig->bCopyClientPrefs)
	{
		for (index=kPGPPrefBoolFirst; index<kPGPPrefBoolLast; index++)
			PGPClearPrefFlags(prefRef, index, kPGPPrefFlags_All);
		
		for (index=kPGPPrefNumberFirst; index<kPGPPrefNumberLast; index++)
			PGPClearPrefFlags(prefRef, index, kPGPPrefFlags_All);
		
		for (index=kPGPPrefStringFirst; index<kPGPPrefStringLast; index++)
			PGPClearPrefFlags(prefRef, index, kPGPPrefFlags_All);
		
		for (index=kPGPPrefByteFirst; index<kPGPPrefByteLast; index++)
			PGPClearPrefFlags(prefRef, index, kPGPPrefFlags_All);
		
		for (index=kPGPPrefStructFirst; index<kPGPPrefStructLast; index++)
			PGPClearPrefFlags(prefRef, index, kPGPPrefFlags_All);
		
		for (index=kPGPPrefArrayFirst; index<kPGPPrefArrayFirst; index++)
			PGPClearPrefFlags(prefRef, index, kPGPPrefFlags_All);

		for (index=kPGPNetPrefBoolFirst; index<kPGPPrefBoolLast; index++)
			PGPClearPrefFlags(pConfig->prefRefNet, index, kPGPPrefFlags_All);
		
		for (index=kPGPNetPrefNumberFirst; index<kPGPPrefNumberLast; index++)
			PGPClearPrefFlags(pConfig->prefRefNet, index, kPGPPrefFlags_All);
		
		for (index=kPGPNetPrefStringFirst; index<kPGPPrefStringLast; index++)
			PGPClearPrefFlags(pConfig->prefRefNet, index, kPGPPrefFlags_All);
		
		for (index=kPGPNetPrefByteFirst; index<kPGPPrefByteLast; index++)
			PGPClearPrefFlags(pConfig->prefRefNet, index, kPGPPrefFlags_All);
		
		for (index=kPGPNetPrefStructFirst; index<kPGPPrefStructLast; index++)
			PGPClearPrefFlags(pConfig->prefRefNet, index, kPGPPrefFlags_All);
		
		for (index=kPGPNetPrefArrayFirst; index<kPGPPrefArrayFirst; index++)
			PGPClearPrefFlags(pConfig->prefRefNet, index, kPGPPrefFlags_All);
	}

	for (index=kPGPAdminPrefBoolFirst; index<kPGPAdminPrefBoolLast; index++)
		PGPSetPrefFlags(prefRef, index, 
			kPGPPrefFlags_Hash | kPGPPrefFlags_Overwrite);

	for (index=kPGPAdminPrefNumberFirst; index<kPGPAdminPrefNumberLast; 
	index++)
		PGPSetPrefFlags(prefRef, index, 
			kPGPPrefFlags_Hash | kPGPPrefFlags_Overwrite);

	for (index=kPGPAdminPrefStringFirst; index<kPGPAdminPrefStringLast;
	index++)
		PGPSetPrefFlags(prefRef, index, 
			kPGPPrefFlags_Hash | kPGPPrefFlags_Overwrite);

	for (index=kPGPAdminPrefByteFirst; index<kPGPAdminPrefByteLast; index++)
		PGPSetPrefFlags(prefRef, index, 
			kPGPPrefFlags_Hash | kPGPPrefFlags_Overwrite);

	for (index=kPGPAdminPrefStructFirst; index<kPGPAdminPrefStructLast;
	index++)
		PGPSetPrefFlags(prefRef, index, 
			kPGPPrefFlags_Hash | kPGPPrefFlags_Overwrite);

	for (index=kPGPAdminPrefArrayFirst; index<kPGPAdminPrefArrayFirst; 
	index++)
		PGPSetPrefFlags(prefRef, index, 
			kPGPPrefFlags_Hash | kPGPPrefFlags_Overwrite);

	PGPSetPrefFlags(prefRef, kPGPPrefLicenseName, 
		kPGPPrefFlags_Hash | kPGPPrefFlags_Overwrite | kPGPPrefFlags_DisableGUI);
	PGPSetPrefFlags(prefRef, kPGPPrefLicenseCompany, 
		kPGPPrefFlags_Hash | kPGPPrefFlags_Overwrite | kPGPPrefFlags_DisableGUI);
	PGPSetPrefFlags(prefRef, kPGPPrefLicenseNumber, 
		kPGPPrefFlags_Hash | kPGPPrefFlags_Overwrite | kPGPPrefFlags_DisableGUI);
	PGPSetPrefFlags(prefRef, kPGPPrefLicenseAuthorization, 
		kPGPPrefFlags_Hash | kPGPPrefFlags_Overwrite | kPGPPrefFlags_DisableGUI);

	PGPWritePrefsHash(pConfig->pgpContext, prefRef);
	PGPSavePrefFile(prefRef);

	if (pConfig->bCopyClientPrefs)
	{
		PGPWritePrefsHash(pConfig->pgpContext, pConfig->prefRefNet);
		PGPSavePrefFile(pConfig->prefRefNet);
	}

	return;
}


void FreeAdminOptions(pgpConfigInfo *pConfig)
{
	// Free allocated memory and objects

	if (pConfig->szLDAPServer)
	{
		PGPFreeData(pConfig->szLDAPServer);
		pConfig->szLDAPServer = NULL;
	}

	if (pConfig->szReconServer)
	{
		PGPFreeData(pConfig->szReconServer);
		pConfig->szReconServer = NULL;
	}

	if (pConfig->defaultKeys != NULL)
	{
		PGPFreeData(pConfig->defaultKeys);
		pConfig->defaultKeys = NULL;
	}

	if (pConfig->szDefaultKeysBuffer != NULL)
	{
		PGPFreeData(pConfig->szDefaultKeysBuffer);
		pConfig->szDefaultKeysBuffer = NULL;
	}

	if (pConfig->szAdminInstaller)
	{
		PGPFreeData(pConfig->szAdminInstaller);
		pConfig->szAdminInstaller = NULL;
	}

	if (pConfig->szClientInstaller)
	{
		PGPFreeData(pConfig->szClientInstaller);
		pConfig->szClientInstaller = NULL;
	}

	if (pConfig->szUserInstallDir)
	{
		PGPFreeData(pConfig->szUserInstallDir);
		pConfig->szUserInstallDir = NULL;
	}

	if (pConfig->avData)
	{
		PGPFreeData(pConfig->avData);
		pConfig->avData = NULL;
		pConfig->avDataSize = 0;
	}

	if (pConfig->keyDB)
	{
		PGPFreeKeyDB(pConfig->keyDB);
		pConfig->keyDB = NULL;
	}

	if (pConfig->tlsContext)
	{
		PGPFreeTLSContext(pConfig->tlsContext);
		pConfig->tlsContext = NULL;
	}

	return;
}


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
