/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: Installer.c,v 1.7 2002/10/19 00:18:58 wjb Exp $
____________________________________________________________________________*/

#include <windows.h>
#include "PGPadmin.h"
#include "Utils.h"
#include <stdio.h>
#include <io.h>
#include <direct.h>
#include <shlwapi.h>
#include "PGPadminRes.h"

#include "pgpClientLib.h"
#include "pgpSEA.h"
#include "pgpSDAcreate.h"
#include "progress.h"

HINSTANCE g_hinst;

#define DEL_KEYS	1
#define DEL_TOOLS	2
#define DEL_LOGIN	4
#define DEL_NET		8
#define DEL_VPN		16
#define DEL_FIRE	32
#define DEL_DISK	64
#define DEL_OUTLOOK 128
#define DEL_OE		256
#define DEL_EUDORA	512
#define DEL_ICQ		1024
#define DEL_LOTUS	2048
#define DEL_DOCS	4096
#define DEL_CONTEXT	8192
#define DEL_GROUPWISE 16384

static char *g_InstallArray[][20]=
{
//PGP Keys
	{"pgpkeys.exe",
	NULL},

//PGP Tools
	{"pgpmail.exe",
	"pgplog.exe",
	NULL},

//PGP Login Integration 
	{"pgpnpi.dll",
	"pgpgina.dll",
	NULL}, 

//PGPnet
	{"pgpnet.vxd",
	"pgpnet.sys",
	"pgpnetwin2k.sys",
	"pgprebind.inf",
	"pgpnet.chm",
	"pgpnet.exe",
	"setadapter.exe",
	"pgpnet.inf",
	"pgpnet_m.inf",
	NULL},

//PGPvpn
	{"pgpcfg.vxd",
	"pgpcfg.sys",
	"PGPcfgwin2k.sys",
	"pgpcfg2k.inf",
	NULL},
	
//PGPfire
	{"pgptdi.vxd",
	"pgptdi.sys",
	NULL},
	
//PGPdisk Volume Security 
	{"pgpdiskengine.dll",
	"pgpdiskih.dll",
	"pgpdiskui.dll",
	"pgpdisk.pdr",
	"pgpdisk.sys",
	"pgpdiskwin2k.sys",
	"pgpdisk.exe",
	NULL},
	
//PGP Plugin for Microsoft Outlook 
	{"pgpexch.dll",
	NULL},

//PGP Plugin for Microsoft Outlook Express 
	{"pgpoe.dll",
	NULL},
	
//PGP Plugin for Qualcomm Eudora 
	{"eudoraplugin.dll",
	"eudorapluginv4.dll",
	NULL},
	
//PGP Plugin for ICQ 
	{"pgpicq.dll",
	NULL},
	
//PGP Plugin for Lotus Notes 
	{"ndbpgp.dll",
	NULL}, 

//PGP Documentation 
	{"pgpwinusersguide.pdf", 
	"introtocrypto.pdf", 
	"readme.htm",
	NULL},

//PGP context menu
	{"pgpmn.dll",
	NULL},

//PGP Plugin for Grouwise 
	{"pgpgw.dll",
	NULL} 
};

PGPError SDADecryptFiles(SDAHEADER *SDAHeader,
						PGPUInt32 *ExpandedKey,
						FILELIST **fl,
						char *szInFileName,
						char *szPrefixPath,
						SDACREATECALLBACK UserProc,void *pUserValue);

static void sCreateInstaller(HWND hwndMain, pgpConfigInfo *pConfig);

UINT FileListFromFile(FILELIST **filelist,char *filename,BOOL *UserCancel);
void RemoveFiles(FILELIST *fl);

// find if iPstr2 is in iPstr1
// case doesn't matter
// return is same as strstr 
static char *mystrstri(char *iPstr1, char *iPstr2)
{            
	int i;
	int Len    = strlen(iPstr2);
	int MaxLen = strlen(iPstr1)-Len;
  
	for(i=0; i<=MaxLen; i++)
	{
		if(!strnicmp(iPstr1+i, iPstr2, Len)) 
			break;
	}

	if(i>MaxLen) 
		return NULL;

	return (iPstr1+i);
}

FILELIST *DeleteFLEntry(FILELIST *flin,char *szFile)
{
	FILELIST *flout,*flprev,*freeatlast;
	DWORD retval;

	flout=flin;
	flprev=NULL;

	while(flin!=NULL)
	{
		if(mystrstri(flin->name,szFile)!=NULL)
		{
			// We found it!
			freeatlast=flin;

			if(flprev==NULL)
			{
				flout=flin->next;
			}
			else
			{
				flprev->next=flin->next;
			}

			// Advance
			flin=flin->next;

			// Erase file and entry
			if(freeatlast->IsDirectory)
			{
				retval=_rmdir(freeatlast->name);
			}
			else
			{
				retval=_unlink(freeatlast->name);
			}

			memset(freeatlast->name,0x00,strlen(freeatlast->name));
			free(freeatlast->name);

			free(freeatlast);
		}
		else
		{
			// Update previous
			flprev=flin;

			//Advance
			flin=flin->next;
		}
	}

	return flout;
}

FILELIST *DeleteComponents(FILELIST *flin,DWORD dwDelete)
{
	int compindex;
	int fileindex;
	FILELIST *flout;

	flout=flin;

	for(compindex=0;compindex<32;compindex++)
	{
		if((dwDelete&1)==1)
		{
			fileindex=0;

			while(g_InstallArray[compindex][fileindex]!=NULL)
			{
				flout=DeleteFLEntry(flout,
					g_InstallArray[compindex][fileindex]);

				fileindex++;
			}
		}

		dwDelete=dwDelete>>1;
	}

	return flout;
}

#define NTSIGNATURE(a) ((LPVOID)((BYTE *)a		     +	\
			((PIMAGE_DOS_HEADER)a)->e_lfanew))

FILELIST *ReverseFileList(FILELIST *flin)
{
	FILELIST *flnext,*flout;

	flout=NULL;

	while(flin!=NULL)
	{
		flnext=flin->next;
		
		flin->next=flout;
		flout=flin;

		flin=flnext;
	}

	return (flout);
}

PGPError SEAGetTempPath(char *szPrefixPath)
{
	char HexString[100];
	int ret_val;

	// Note that GetTempPath returns a path with a backslash
	if(GetTempPath(MAX_PATH,szPrefixPath ) == 0)
	{	
		return kPGPError_UnknownError;
	}

	srand( (unsigned)time( NULL ) );
	sprintf(HexString,"%x",rand()*rand());
	HexString[5]=0; // Truncate at length 5

	strcat(szPrefixPath,"PGP");
	strcat(szPrefixPath,HexString);
	strcat(szPrefixPath,"\\");

	ret_val=_mkdir(szPrefixPath);

	if(ret_val!=0)
	{
		return kPGPError_UnknownError;
	}

	return kPGPError_NoErr;
}

BOOL WriteAdminFile(HWND hwnd,
					char *filename,
					char *prefpath,
					FILELIST **p_fl,
					char *buf,
					DWORD size)
{
	FILE *fout;
	char name[MAX_PATH+1];

	if(buf==NULL)
		return TRUE;

	strcpy(name,prefpath);
	strcat(name,filename);

	fout=fopen(name,"wb");

	if(fout==0)
	{
		MessageBox(hwnd,"Can't create file",
			name,
			MB_OK|MB_ICONSTOP|MB_SETFOREGROUND);

		return FALSE;
	}
	else
	{
		fwrite(buf,1,size,fout);
		fclose(fout);

		// Remember file so we can delete it later
		FileListFromFile(p_fl,name,NULL);
	}

	return TRUE;
}

typedef struct
{
	HWND hwndWorking;
	DWORD dwDelete;
	char *szInFile;
	char *szOutFile;
	char *ClientBuffer;
	DWORD ClientSize;
	char *NetBuffer;
	DWORD NetSize;
	char *SetupBuffer;
	DWORD SetupSize;
	char *KeysBuffer;
	DWORD KeysSize;
} ADMINSEAWORK;

/* Generic event handler */
PGPError
SDAEvents(
	PGPContextRef context,
	SDAEvent *event,
	PGPUserValue userValue
	)
{
	ADMINSEAWORK *AdminSEAWork;
	(void) context;

	AdminSEAWork=(ADMINSEAWORK *)userValue;

	switch( event->type ) 
	{
		case kSDAEvent_ErrorEvent:
		{	
			SDAEventErrorData *d = &event->data.errorData;

			MessageBox(AdminSEAWork->hwndWorking,
				"Error encoding file",d->fileName,
				MB_OK|MB_SETFOREGROUND|MB_ICONSTOP);
			break;
		}

		case kSDAEvent_ProgressEvent:
		{	
			SDAEventProgressData *d = &event->data.progressData;

			if(d->bytesTotal!=0)
			{
				return PGPscProgressSetBar(AdminSEAWork->hwndWorking,
					(DWORD)(d->bytesWritten*100/d->bytesTotal),FALSE);
			}
			break;
		}

		case kSDAEvent_NewFileEvent:
		{	
			SDAEventNewFileData *d = &event->data.newfileData;

			PGPscProgressSetNewFilename(AdminSEAWork->hwndWorking,"In '%s'",
				d->fileName,FALSE);
			break;
		}

		case kSDAEvent_AskFileEvent:
		{
			SDAEventAskFileData *d = &event->data.askfileData;

			strcpy(d->outName,d->inName);
			break;
		}

		case kSDAEvent_GetSDARandomBitsEvent:
		{
			SDAEventGetSDARandomBitsData *d = &event->data.randomData;
			break;
		}

		case kSDAEvent_WriteSDAStubEvent:
		{
			DWORD dwStubSize;
			HRSRC hRCStub;
			HGLOBAL hGBStub;
			char *pStubData;
			HMODULE hModule;

			SDAEventWriteSDAStubData *d = &event->data.stubData;

			hModule=GetModuleHandle(NULL);

			hRCStub=FindResource(hModule,
				MAKEINTRESOURCE(IDR_SEASTUB),
				RT_RCDATA);

			dwStubSize=SizeofResource(hModule,hRCStub);

			hGBStub=LoadResource(hModule,hRCStub);
			pStubData=(char *)LockResource(hGBStub);

			// Sanity checking here for resource munging
			if(!((*(USHORT *)pStubData == IMAGE_DOS_SIGNATURE) &&
		      (*(DWORD *)NTSIGNATURE (pStubData) == IMAGE_NT_SIGNATURE)))
			{
				MessageBox(AdminSEAWork->hwndWorking,
					"Error writing stub file","PGPadmin",
					MB_OK|MB_SETFOREGROUND|MB_ICONSTOP);

				return kPGPError_UserAbort;
			}

			// Copy SDA.exe prefix executable into SDA file and
			// set header offset so we can find data again.
			// Write out stub to disk from resources
			fwrite(pStubData,1,dwStubSize,d->fOutput);

			UnlockResource(hGBStub);
			FreeResource(hGBStub);
			break;
		}

		default:
			break;
	}
				
	return kPGPError_NoErr;
}

PGPError RebuildInstallerStub(HWND hwndWorking,void *pUserValue)
{
	SDAHEADER SDAHeader;
	FILE *fin;
	FILELIST *fl;
	PGPError err;
	ADMINSEAWORK *AdminSEAWork;
	char szPrefixPath[MAX_PATH+1];

	AdminSEAWork=(ADMINSEAWORK *)pUserValue;
	AdminSEAWork->hwndWorking=hwndWorking;

	fin=fopen(AdminSEAWork->szInFile,"rb");

	fseek(fin, -(int)(sizeof(SDAHEADER)), SEEK_END);
	fread(&SDAHeader,1,sizeof(SDAHEADER),fin);
	fclose(fin);

	if(memcmp(SDAHeader.szPGPSDA,"PGPSEA",6)!=0)
	{
		MessageBox(AdminSEAWork->hwndWorking,
			"Not a valid SEA",
			"PGPadmin",
			MB_OK|MB_ICONSTOP|MB_SETFOREGROUND);
		return kPGPError_UserAbort;
	}

	err=SEAGetTempPath(szPrefixPath);

	if(err!=kPGPError_NoErr)
		return err;

	fl=NULL;
		// Keep track of it so we can delete it later
	FileListFromFile(&fl,szPrefixPath,NULL);
	fl->IsDirectory=TRUE;

	err=SDADecryptFiles(&SDAHeader,
						NULL,
						&fl,
						AdminSEAWork->szInFile,
						szPrefixPath,
						SDAEvents,AdminSEAWork);

	if(IsntPGPError(err))
	{
		// ********************************************
		// Now is the time to delete our old files
		// ********************************************
		fl=DeleteComponents(fl,AdminSEAWork->dwDelete);

		if(AdminSEAWork->ClientBuffer!=NULL)
			fl=DeleteFLEntry(fl,"Disk1\\PGPPrefs.txt");
		if(AdminSEAWork->NetBuffer!=NULL)
			fl=DeleteFLEntry(fl,"Disk1\\PGPnetPrefs.txt");
		if(AdminSEAWork->SetupBuffer!=NULL)
			fl=DeleteFLEntry(fl,"Disk1\\setup.ini");
		if(AdminSEAWork->KeysBuffer!=NULL)
			fl=DeleteFLEntry(fl,"Disk1\\DefaultKeys.asc");

		WriteAdminFile(AdminSEAWork->hwndWorking,
			"Disk1\\PGPPrefs.txt",
			szPrefixPath,
			&fl,
			AdminSEAWork->ClientBuffer,
			AdminSEAWork->ClientSize);
		WriteAdminFile(AdminSEAWork->hwndWorking,
			"Disk1\\PGPnetPrefs.txt",
			szPrefixPath,
			&fl,
			AdminSEAWork->NetBuffer,
			AdminSEAWork->NetSize);
		WriteAdminFile(AdminSEAWork->hwndWorking,
			"Disk1\\setup.ini",
			szPrefixPath,
			&fl,
			AdminSEAWork->SetupBuffer,
			AdminSEAWork->SetupSize);
		WriteAdminFile(AdminSEAWork->hwndWorking,
			"Disk1\\DefaultKeys.asc",
			szPrefixPath,
			&fl,
			AdminSEAWork->KeysBuffer,
			AdminSEAWork->KeysSize);

		// Reverse the list to re-encode
		fl=ReverseFileList(fl);

		// Re-encode to same file
		err=SDACreate(NULL,fl,
			AdminSEAWork->szOutFile,NULL,
			SDAEvents,AdminSEAWork,0);

		// Reverse the list to delete
		fl=ReverseFileList(fl);
	}

	RemoveFiles(fl);

	return err;
}

BOOL RebuildInstaller(HWND hwnd,
					DWORD dwDelete,
				   char *szInFile,
				   char *szOutFile,
				   char *ClientBuffer,
				   DWORD ClientSize,
				   char *NetBuffer,
				   DWORD NetSize,
				   char *SetupBuffer,
				   DWORD SetupSize,
				   char *KeysBuffer,
				   DWORD KeysSize)
{
	ADMINSEAWORK AdminSEAWork;

	g_hinst=g_hInstance;

	memset(&AdminSEAWork,0x00,sizeof(ADMINSEAWORK));
	AdminSEAWork.szInFile=szInFile;
	AdminSEAWork.szOutFile=szOutFile;
	AdminSEAWork.ClientBuffer=ClientBuffer;
	AdminSEAWork.ClientSize=ClientSize;
	AdminSEAWork.NetBuffer=NetBuffer;
	AdminSEAWork.NetSize=NetSize;
	AdminSEAWork.SetupBuffer=SetupBuffer;
	AdminSEAWork.SetupSize=SetupSize;
	AdminSEAWork.KeysBuffer=KeysBuffer;
	AdminSEAWork.KeysSize=KeysSize;
	AdminSEAWork.dwDelete=dwDelete;

	if(IsPGPError(PGPscProgressDialog(hwnd,RebuildInstallerStub,&AdminSEAWork,
		  0,NULL,
		  NULL,NULL,0)))
		  return FALSE;

	return TRUE;
}


BOOL CALLBACK InstallerDlgProc(HWND hwndDlg, 
							   UINT uMsg, 
							   WPARAM wParam, 
							   LPARAM lParam)
{
	BOOL			bReturnCode	= FALSE;
	pgpConfigInfo *	pConfig		= NULL;

	g_hCurrentDlgWnd = hwndDlg;

	if (uMsg != WM_INITDIALOG)
		pConfig = (pgpConfigInfo *) GetWindowLong(hwndDlg, GWL_USERDATA);

	switch(uMsg)
	{
	case WM_INITDIALOG:
		pConfig = (pgpConfigInfo *) lParam;
		SetWindowLong(hwndDlg, GWL_USERDATA, lParam);
		
		if (pConfig->szAdminInstaller == NULL)
			SetWindowText(GetDlgItem(hwndDlg, IDC_ADMININSTALLER), "");
		else
			SetWindowText(GetDlgItem(hwndDlg, IDC_ADMININSTALLER), 
				pConfig->szAdminInstaller);
		
		if (pConfig->szClientInstaller == NULL)
			SetWindowText(GetDlgItem(hwndDlg, IDC_CLIENTINSTALLER), "");
		else
			SetWindowText(GetDlgItem(hwndDlg, IDC_CLIENTINSTALLER), 
				pConfig->szClientInstaller);

		break;

	case WM_CLOSE:
		EndDialog(hwndDlg, 0);
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
			{
				char szMsg[255];
				char szTitle[255];
				char szAdmin[MAX_PATH];
				char szClient[MAX_PATH];
				HANDLE hFile;
				
				bReturnCode = TRUE;

				if (!GetWindowTextLength(GetDlgItem(hwndDlg, 
						IDC_ADMININSTALLER)))
				{
					LoadString(g_hInstance, IDS_E_BLANKADMIN, szMsg, 254);
					LoadString(g_hInstance, IDS_TITLE, szTitle, 254);
					MessageBox(hwndDlg, szMsg, szTitle, MB_ICONEXCLAMATION);
					
					break;
				}
				
				if (!GetWindowTextLength(GetDlgItem(hwndDlg, 
						IDC_CLIENTINSTALLER)))
				{
					LoadString(g_hInstance, IDS_E_BLANKCLIENT, szMsg, 254);
					LoadString(g_hInstance, IDS_TITLE, szTitle, 254);
					MessageBox(hwndDlg, szMsg, szTitle, MB_ICONEXCLAMATION);
					
					break;
				}
				
				GetWindowText(GetDlgItem(hwndDlg, IDC_ADMININSTALLER), 
					szAdmin, MAX_PATH-1);
				GetWindowText(GetDlgItem(hwndDlg, IDC_CLIENTINSTALLER), 
					szClient, MAX_PATH-1);
				
				if (strlen(szAdmin) >= 4)
				{
					if (strnicmp(&(szAdmin[strlen(szAdmin)-4]), ".exe", 4))
					{
						strcat(szAdmin, ".exe");
					
						SetWindowText(GetDlgItem(hwndDlg, IDC_ADMININSTALLER), 
							szAdmin);
					}
				}

				if (strlen(szClient) >= 4)
				{
					if (strnicmp(&(szClient[strlen(szClient)-4]), ".exe", 4))
					{
						strcat(szClient, ".exe");
					
						SetWindowText(GetDlgItem(hwndDlg, IDC_CLIENTINSTALLER), 
							szClient);
					}
				}

				if (!stricmp(szAdmin, szClient))
				{
					LoadString(g_hInstance, IDS_E_INSTALLDIR, szMsg, 254);
					LoadString(g_hInstance, IDS_TITLE, szTitle, 254);
					MessageBox(hwndDlg, szMsg, szTitle, MB_ICONEXCLAMATION);
					
					break;
				}
				
				hFile = CreateFile(szClient, GENERIC_READ | GENERIC_WRITE, 0, 
							NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL);

				if (hFile == INVALID_HANDLE_VALUE)
				{
					if ((GetLastError() == ERROR_ALREADY_EXISTS) ||
						(GetLastError() == ERROR_FILE_EXISTS))
					{
						LoadString(g_hInstance, IDS_Q_ALREADYEXISTS, szMsg, 254);
						LoadString(g_hInstance, IDS_TITLE, szTitle, 254);
						if (MessageBox(hwndDlg, szMsg, szTitle, MB_YESNO | 
							MB_ICONQUESTION) == IDNO)
							break;
					}
					else if (!SetFileAttributes(szClient, FILE_ATTRIBUTE_NORMAL))
					{
						LoadString(g_hInstance, IDS_E_CANTCOPY, szMsg, 254);
						LoadString(g_hInstance, IDS_TITLE, szTitle, 254);
						MessageBox(hwndDlg, szMsg, szTitle, MB_ICONEXCLAMATION);
						
						break;
					}
				}
				else
					CloseHandle(hFile);

				if (pConfig->szAdminInstaller)
					PGPFreeData(pConfig->szAdminInstaller);
			
				pConfig->szAdminInstaller = (char *) 
											PGPNewData(pConfig->memoryMgr,
												strlen(szAdmin) + 1,
												kPGPMemoryMgrFlags_Clear);

				strcpy(pConfig->szAdminInstaller, szAdmin);

				if (pConfig->szClientInstaller)
					PGPFreeData(pConfig->szClientInstaller);
			
				pConfig->szClientInstaller = (char *) 
											PGPNewData(pConfig->memoryMgr,
												strlen(szClient) + 1,
												kPGPMemoryMgrFlags_Clear);

				strcpy(pConfig->szClientInstaller, szClient);
			}

			sCreateInstaller(hwndDlg, pConfig);
			EndDialog(hwndDlg, 1);
			break;

		case IDCANCEL:
			EndDialog(hwndDlg, 0);
			break;

		case IDC_ADMINBROWSE:
			if (HIWORD(wParam) == BN_CLICKED)
			{
				OPENFILENAME openFile;
				char szAdminFile[MAX_PATH];
				char szTitle[256];

				strcpy(szAdminFile, "\0");
				LoadString(g_hInstance, IDS_GETADMININSTALLER, szTitle, 
					255);

				openFile.lStructSize = sizeof(OPENFILENAME);
				openFile.hwndOwner = hwndDlg;
				openFile.lpstrFilter =	"Executable Files (*.exe)\0"
										"*.exe\0";
				openFile.lpstrCustomFilter = NULL;
				openFile.nFilterIndex = 1;
				openFile.lpstrFile = szAdminFile;
				openFile.nMaxFile = MAX_PATH-1;
				openFile.lpstrFileTitle = NULL;
				openFile.lpstrInitialDir = NULL;
				openFile.lpstrTitle = szTitle;
				openFile.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY |
									OFN_PATHMUSTEXIST | OFN_NOCHANGEDIR;
				openFile.lpstrDefExt = "exe";

				while (GetOpenFileName(&openFile))
				{
					SDAHEADER SDAHeader;
					FILE *fin;
					
					fin=fopen(szAdminFile, "rb");
					
					fseek(fin, -(int)(sizeof(SDAHEADER)), SEEK_END);
					fread(&SDAHeader,1,sizeof(SDAHEADER),fin);
					fclose(fin);
					
					if (memcmp(SDAHeader.szPGPSDA,"PGPSEA",6)!=0)
					{
						char szMsg[255];

						LoadString(g_hInstance, IDS_E_NOTINSTALLER, szMsg, 
							254);
						LoadString(g_hInstance, IDS_TITLE, szTitle, 254);
						MessageBox(hwndDlg, szMsg, szTitle, 
							MB_ICONEXCLAMATION);
						continue;
					}

					if (SDAHeader.NumFiles < 30)
					{
						char szMsg[255];

						LoadString(g_hInstance, IDS_E_WRONGINSTALLER, szMsg, 
							254);
						LoadString(g_hInstance, IDS_TITLE, szTitle, 254);
						MessageBox(hwndDlg, szMsg, szTitle, 
							MB_ICONEXCLAMATION);
						continue;
					}
					
					SetWindowText(GetDlgItem(hwndDlg, IDC_ADMININSTALLER),
						szAdminFile);
					break;
				}

				bReturnCode = TRUE;
			}
			break;

		case IDC_CLIENTBROWSE:
			if (HIWORD(wParam) == BN_CLICKED)
			{
				OPENFILENAME openFile;
				char szClientFile[MAX_PATH];
				char szTitle[256];

				strcpy(szClientFile, "\0");
				LoadString(g_hInstance, IDS_GETCLIENTINSTALLER, szTitle, 
					255);

				openFile.lStructSize = sizeof(OPENFILENAME);
				openFile.hwndOwner = hwndDlg;
				openFile.lpstrFilter =	"Executable Files (*.exe)\0"
										"*.exe\0";
				openFile.lpstrCustomFilter = NULL;
				openFile.nFilterIndex = 1;
				openFile.lpstrFile = szClientFile;
				openFile.nMaxFile = MAX_PATH-1;
				openFile.lpstrFileTitle = NULL;
				openFile.lpstrInitialDir = NULL;
				openFile.lpstrTitle = szTitle;
				openFile.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_NOCHANGEDIR;
				openFile.lpstrDefExt = "exe";

				if (GetSaveFileName(&openFile))
				{
					SetWindowText(GetDlgItem(hwndDlg, IDC_CLIENTINSTALLER),
						szClientFile);
					DeleteFile(szClientFile);
				}

				bReturnCode = TRUE;
			}
			break;
		}
		
		break;
	}

	return(bReturnCode);
}


BOOL CALLBACK WaitProc(HWND hwndDlg, 
					   UINT uMsg, 
					   WPARAM wParam, 
					   LPARAM lParam)
{
	switch(uMsg)
	{
	case WM_INITDIALOG:
		{
			PGPclCenterWindowOnMonitor(hwndDlg, kPGPclPrimaryMonitor);
			break;
		}
	}

	return 0;
}


static void sCreateInstaller(HWND hwndMain, pgpConfigInfo *pConfig)
{
	char			szSetupIni[MAX_PATH];
	char *			szClientBuffer = NULL;
	char *			szNetBuffer = NULL;
	char *			szSetupBuffer = NULL;
	char *			szKeysBuffer = NULL;
	PGPSize			nClientBufferSize = 0;
	PGPSize			nNetBufferSize = 0;
	PGPSize			nSetupBufferSize = 0;
	PGPSize			nKeysBufferSize = 0;
	HANDLE			hFile;
	char			szMsg[255];
	char			szTitle[255];
	DWORD			dwBytesRead;
	DWORD			dwDelete;
	BOOL			bInstallMail;
	BOOL			bDesktopSecurity;
	
	// Load the prefs to a buffer
	
	hFile = CreateFile(pConfig->szClientPrefs, GENERIC_READ, 0, NULL, 
				OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	nClientBufferSize = GetFileSize(hFile, NULL);
		
	szClientBuffer = (char *) PGPNewData(pConfig->memoryMgr,
								nClientBufferSize,
								kPGPMemoryMgrFlags_Clear);
		
	ReadFile(hFile, szClientBuffer, nClientBufferSize, &dwBytesRead, NULL);
	CloseHandle(hFile);

	hFile = CreateFile(pConfig->szNetPrefs, GENERIC_READ, 0, NULL, 
				OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	nNetBufferSize = GetFileSize(hFile, NULL);
		
	szNetBuffer = (char *) PGPNewData(pConfig->memoryMgr,
								nNetBufferSize,
								kPGPMemoryMgrFlags_Clear);
		
	ReadFile(hFile, szNetBuffer, nNetBufferSize, &dwBytesRead, NULL);
	CloseHandle(hFile);

	if (pConfig->defaultKeys)
	{
		// Load the default keys to a buffer
		
		hFile = CreateFile(pConfig->szDefaultKeys, GENERIC_READ, 0, NULL, 
					OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		nKeysBufferSize = GetFileSize(hFile, NULL);
		
		if (nKeysBufferSize > 0)
		{
			szKeysBuffer = (char *) PGPNewData(pConfig->memoryMgr,
										nKeysBufferSize,
										kPGPMemoryMgrFlags_Clear);
			
			ReadFile(hFile, szKeysBuffer, nKeysBufferSize, &dwBytesRead, NULL);
		}
		CloseHandle(hFile);
	}

	// Now create the client installer
	
	if (GetTempPath(MAX_PATH-1, szSetupIni))
	{
		DWORD dwBytesRead;
		
		strcat(szSetupIni, "setup.ini");
		
		WritePrivateProfileString("Startup", "AppName", "PGP", szSetupIni);
		WritePrivateProfileString("Startup", "FreeDiskSpace", "0", szSetupIni);
		WritePrivateProfileString("ISUPDATE", "UpdateURL", "", szSetupIni);
		WritePrivateProfileString("Languages", "Default", "0x0009", szSetupIni);
		WritePrivateProfileString("Languages", "Count", "1", szSetupIni);
		WritePrivateProfileString("Languages", "key0", "0x0009", szSetupIni);

		// Write the installation options if applicable
		
		if (pConfig->bPreselectInstall)
		{
			char szPrograms[3];
			char szLogin[3];
			char szNet[3];
			char szFire[3];
			char szDisk[3];
			char szEudora[3];
			char szOutlook[3];
			char szOutlookExpress[3];
			char szICQ[3];
			char szLotus[3];
			char szGroupwise[3];
			char szManual[3];
			char szUninstallOld[3];
			char szAutoReboot[3];
			
			WritePrivateProfileString("Startup", "EasyInstall", "1",
				szSetupIni);
			
			WritePrivateProfileString("Startup", "InstallDir",
				pConfig->szUserInstallDir, szSetupIni);
			
			wsprintf(szPrograms, "%d", pConfig->bInstallPrograms);
			wsprintf(szLogin, "%d", 0);
			wsprintf(szNet, "%d", pConfig->bInstallNet);
			wsprintf(szFire, "%d", pConfig->bInstallFire);
			wsprintf(szDisk, "%d", pConfig->bInstallDisk);
			wsprintf(szEudora, "%d", pConfig->bInstallEudora);
			wsprintf(szOutlook, "%d", pConfig->bInstallOutlook);
			wsprintf(szOutlookExpress, "%d", pConfig->bInstallOutExpress);
			wsprintf(szICQ, "%d", pConfig->bInstallICQ);
			wsprintf(szLotus, "%d", pConfig->bInstallLotus);
			wsprintf(szGroupwise, "%d", pConfig->bInstallGroupwise);
			wsprintf(szManual, "%d", pConfig->bInstallManual);
			wsprintf(szUninstallOld, "%d", pConfig->bUninstallOld);
			wsprintf(szAutoReboot, "%d", pConfig->bAutoReboot);
			
			WritePrivateProfileString("Startup", "ProgramFiles",
				szPrograms, szSetupIni);
			WritePrivateProfileString("Startup", "PGPLogin",
				szLogin, szSetupIni);
			WritePrivateProfileString("Startup", "PGPnet",
				szNet, szSetupIni);
			WritePrivateProfileString("Startup", "PGPfire",
				szFire, szSetupIni);
			WritePrivateProfileString("Startup", "PGPdisk",
				szDisk, szSetupIni);
			WritePrivateProfileString("Startup", "EudoraPlugin",
				szEudora, szSetupIni);
			WritePrivateProfileString("Startup", "OutlookPlugin",
				szOutlook, szSetupIni);
			WritePrivateProfileString("Startup", "OutlookExpressPlugin",
				szOutlookExpress, szSetupIni);
			WritePrivateProfileString("Startup", "ICQPlugin",
				szICQ, szSetupIni);
			WritePrivateProfileString("Startup", "LotusPlugin",
				szLotus, szSetupIni);
			WritePrivateProfileString("Startup", "GroupwisePlugin",
				szGroupwise, szSetupIni);
			WritePrivateProfileString("Startup", "Documentation",
				szManual, szSetupIni);
			WritePrivateProfileString("Startup", "UninstallOld",
				szUninstallOld, szSetupIni);
			WritePrivateProfileString("Startup", "AutoReboot",
				szAutoReboot, szSetupIni);
		}
		else
			WritePrivateProfileString("Startup", "EasyInstall", "0",
			szSetupIni);
		
		// On windows 95, we must flush the ini file by using three
		// nulls -wjb
		WritePrivateProfileString(NULL,NULL,NULL,
			szSetupIni);
		
		hFile = CreateFile(szSetupIni, GENERIC_READ, 0, NULL, OPEN_ALWAYS,
					FILE_ATTRIBUTE_NORMAL, NULL);
		nSetupBufferSize = GetFileSize(hFile, NULL);
		
		szSetupBuffer = (char *) PGPNewData(pConfig->memoryMgr,
									nSetupBufferSize,
									kPGPMemoryMgrFlags_Clear);
		
		ReadFile(hFile, szSetupBuffer, nSetupBufferSize, 
			&dwBytesRead, NULL);
		CloseHandle(hFile);
		DeleteFile(szSetupIni);
	}

	// Tell install rebuild which components to delete
	dwDelete=0;
	bDesktopSecurity = GetInstalledProduct(NULL) & Installer_PGPds;
	bInstallMail = pConfig->bInstallEudora || pConfig->bInstallOutlook ||
					pConfig->bInstallICQ || pConfig->bInstallOutExpress ||
					pConfig->bInstallLotus || pConfig->bInstallGroupwise;

	if(!pConfig->bInstallDisk && !pConfig->bInstallNet && !bInstallMail &&
		!bDesktopSecurity)
		dwDelete|=DEL_KEYS;
	if(!bInstallMail && !bDesktopSecurity)
		dwDelete|=DEL_TOOLS;
	if(!pConfig->bInstallDisk && !bInstallMail && !bDesktopSecurity)
		dwDelete|=DEL_CONTEXT;
	if(!pConfig->bInstallLogin)
		dwDelete|=DEL_LOGIN;
	if(!pConfig->bInstallNet && !pConfig->bInstallFire)
		dwDelete|=DEL_NET;
	if(!pConfig->bInstallNet)
		dwDelete|=DEL_VPN;
	if(!pConfig->bInstallFire)
		dwDelete|=DEL_FIRE;
	if(!pConfig->bInstallDisk)
		dwDelete|=DEL_DISK;
	if(!pConfig->bInstallEudora)
		dwDelete|=DEL_EUDORA;
	if(!pConfig->bInstallOutlook)
		dwDelete|=DEL_OUTLOOK;
	if(!pConfig->bInstallOutExpress)
		dwDelete|=DEL_OE;
	if(!pConfig->bInstallICQ)
		dwDelete|=DEL_ICQ;
	if(!pConfig->bInstallLotus)
		dwDelete|=DEL_LOTUS;
	if(!pConfig->bInstallGroupwise)
		dwDelete|=DEL_GROUPWISE;
	if(!pConfig->bInstallManual)
		dwDelete|=DEL_DOCS;

	if (RebuildInstaller(hwndMain,
					dwDelete,
					pConfig->szAdminInstaller,
					pConfig->szClientInstaller,
					szClientBuffer, 
					nClientBufferSize, 
					szNetBuffer, 
					nNetBufferSize, 
					szSetupBuffer, 
					nSetupBufferSize,
					szKeysBuffer,
					nKeysBufferSize))
	{
		if (SetFileAttributes(pConfig->szClientInstaller, 
			FILE_ATTRIBUTE_NORMAL))
		{
			// We're done!
			
			LoadString(g_hInstance, IDS_DONE, szMsg, 254);
			LoadString(g_hInstance, IDS_TITLE, szTitle, 254);
			MessageBox(hwndMain, szMsg, szTitle, MB_ICONINFORMATION);
		}
		else
		{
			// Setting attributes failed
			
			LoadString(g_hInstance, IDS_FAILED, szMsg, 254);
			LoadString(g_hInstance, IDS_TITLE, szTitle, 254);
			MessageBox(hwndMain, szMsg, szTitle, MB_ICONEXCLAMATION);
			DeleteFile(pConfig->szClientInstaller);
		}
	}
	else
	{
		// Copying file failed
		
		LoadString(g_hInstance, IDS_FAILED, szMsg, 254);
		LoadString(g_hInstance, IDS_TITLE, szTitle, 254);
		MessageBox(hwndMain, szMsg, szTitle, MB_ICONEXCLAMATION);
		DeleteFile(pConfig->szClientInstaller);
	}
	
	if (szClientBuffer != NULL)
		PGPFreeData(szClientBuffer);
	if (szNetBuffer != NULL)
		PGPFreeData(szNetBuffer);
	if (szSetupBuffer != NULL)
		PGPFreeData(szSetupBuffer);
	if (szKeysBuffer != NULL)
		PGPFreeData(szKeysBuffer);

	return;
}


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
