/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	CLgenwiz.c - handle wizard for creating new keys

	$Id: CLdiskwiz.c,v 1.20 2002/11/22 03:12:09 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "SpaceEdit.h"
#include "UTF8Edit.h"

// pgp header files
#include "pgpRandomPool.h"
#include "pgpUserInterface.h"
#include "pgpReconstruct.h"

#include <commdlg.h>
#include <tchar.h>

// pgp disk files
#define PGPDISK_DYNLINK

#include "pgpDiskEngine.h"
#include "pgpDiskUI.h"

#define	kDiskFat12FileSys		0 
#define	kDiskFat16FileSys		1 
#define	kDiskFat12Or16FileSys	2 
#define	kDiskFat32FileSys		3 
#define	kDiskNTFSFileSys		4 
#define	kDiskUnknownFileSys		5

#define	kDiskInvalidAlgorithm	0 
#define	kDiskCopyDataAlgorithm	1 
#define	kDiskOldCAST5Algorithm	2 
#define	kDiskCAST5Algorithm		3 
#define	kDiskTwoFishAlgorithm	4 
#define	kDiskAESAlgorithm		5

#include <stdio.h>

// system header files
#include <process.h>
#include <lmaccess.h>
#include <lmerr.h>

#define PROGRESS_TIMER			2345L
#define PROGRESSTIMERPERIOD		150L

#define NUM_WIZ_PAGES 5
#define MAX_TASKS 2

#define TASK_DISKGEN			0
#define TASK_DISKFORMAT			1

#define IDS_DISKGEN 0
#define IDS_DISKFORMAT 1
#define IDS_DISKTITLE 2
#define IDS_DISKCONFIRMCANCEL 3

#define ICON_NONE				0
#define ICON_ARROW				1
#define ICON_CHECK				2
#define ICON_ERROR				3

#define KGWIZ_CORPDISKINTRO		0
#define KGWIZ_CORPDISKOPTS		1
#define KGWIZ_ENTROPY			2
#define KGWIZ_CORPDISKCREATION	3
#define KGWIZ_CORPDISK_DONE		4

// external globals
extern HINSTANCE		g_hInst;
extern PGPContextRef	g_context;
extern HIMAGELIST		g_hilClientLib;

// typedefs
typedef struct _DISKINFO
{
	PGPContextRef		context;
	HWND				hwndWiz;
	HWND				hwndParent;
	PGPKeyDBRef			keydbMain;

	HBITMAP				hbitmapIntro;
	HBITMAP				hbitmapDone;
	HPALETTE			hpaletteIntro;
	HPALETTE			hpaletteDone;
	HFONT				hfontIntro;
	HFONT				hfontInterior;
	HICON				hiconAlert;
	HICON				hiconDoing;
	HICON				hiconDone;
	HICON				hiconNotDone;

	PGPKeyDBObjRef		keyADK;
	PGPKeyDBObjRef		keyCorp;
	BOOL				bEnforceADK;
	BOOL				bMetaCorpKey;

	LONG				lRandomBitsNeeded;
	LONG				lOriginalEntropy;
	PGPKeyDBObjRef		keyNew;

	PGPBoolean			bTaskComplete;
	BOOL				bCancelPending;

	PGPError			err;
	PGPBoolean			areFormatting;
	PGPUInt32			percent;

	PGPBoolean			oldAreFormatting;
	PGPBoolean			oldTaskComplete;
	PGPUInt32			oldPercent;

	HANDLE				hThread;

	PGPUInt32			diskAlgorithm;
	PGPUInt32			diskSize;
	PGPUInt32			diskFS;
	char				diskName[MAX_PATH];
	PGPKeyDBObjRef		diskADK;

	PGPdiskContextRef	diskContext;

} DISKINFO, *PDISKINFO;

static HWND		s_hwndCollectEntropy	= NULL;

PGPBoolean NewDiskStatus(PGPBoolean areFormatting,PGPUInt32 percent, void *userVal)
{
	PDISKINFO pkgi;

	pkgi=(PDISKINFO)userVal;

	pkgi->areFormatting=areFormatting;
	pkgi->percent=percent;

	if(pkgi->bCancelPending)
		return FALSE;

	return TRUE;
}

static char	kRegistryMappedDrive98Section[]	= "Network\\Persistent";
static char	kRegistryMappedDriveNTSection[]	= "Network";

PGPBoolean IsDriveNetworkMapped(char drive)
{
	PGPclWindowsGenus genus;
	char *regPathToMapped;
	char szReg[200];
	HKEY hkey;

	PGPclGetWindowsVersion (NULL, &genus, NULL);

	if (genus == kPGPclWindowsGenus95)
		regPathToMapped = kRegistryMappedDrive98Section;
	else
		regPathToMapped = kRegistryMappedDriveNTSection;

	sprintf(szReg,"%s\\%c",regPathToMapped,drive+'a');

	if(RegOpenKeyEx (HKEY_CURRENT_USER, szReg, 0, KEY_READ, &hkey)==ERROR_SUCCESS)
	{
		RegCloseKey(hkey);
		return TRUE;
	}

	return FALSE;
}

DWORD WINAPI sDiskGenerationThread(LPVOID lpvParam)
{
	PDISKINFO pkgi;
	PGPdiskDiskSetRef diskSet;
	PGPdiskDiskRef diskDisk;
	PGPError err;
	char volName[MAX_PATH];
	PGPUInt32 drives;
	char i;

	pkgi = (PDISKINFO) lpvParam;

	drives = GetLogicalDrives();

	for (i = 2; i < 26; i++)
	{
		if (!(drives & (1 << i)))
		{
			sprintf(volName,"%c:\\",i+'a');
			
			if (!IsDriveNetworkMapped(i))
				break;
		}
	}

	err=PGPdiskNewDiskSet(pkgi->diskContext, &diskSet);

	if(IsntPGPError(err))
	{
		err=PGPdiskDiskSetCreateDiskWithPub(
			diskSet,
			pkgi->diskName,
			pkgi->diskSize,
			pkgi->diskAlgorithm,
			pkgi->keyNew,
			volName,
			NewDiskStatus,
			pkgi,
			&diskDisk);

		if(IsntPGPError(err))
		{
			err=PGPdiskFormatNewDisk(pkgi->diskContext,
				volName,"PGPdisk", 
				pkgi->diskFS, NewDiskStatus, pkgi);
		}

		PGPdiskFreeDiskSet(diskSet);
	}

	pkgi->err=err;
	pkgi->bTaskComplete=TRUE;

	return 0;
}

//	______________________________________________
//
//  setup the current task text

static VOID
sSetCurrentTaskInfo (
		PDISKINFO		pkgi,
		UINT			uIcon,
		UINT			taskicon)
{
	HICON	hicon		= NULL;

	switch (uIcon) 
	{
		case ICON_ARROW :
			hicon = pkgi->hiconDoing;
			break;

		case ICON_CHECK :
			hicon = pkgi->hiconDone;
			break;

		case ICON_ERROR :
			hicon = pkgi->hiconNotDone;
			break;

		default:
			break;
	}

	SendDlgItemMessage (pkgi->hwndWiz, taskicon,
		STM_SETIMAGE, (WPARAM)IMAGE_ICON, (LPARAM)hicon);
}

//	______________________________________________
//
//  Dialog procedure for "Finish" dialog

static LRESULT WINAPI
sKeyWizardDoneDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PDISKINFO		pkgi;
	BOOL			bReturnCode = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PDISKINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontIntro, (LPARAM)FALSE);
		return TRUE;
	}

	case WM_PAINT :
		pkgi = (PDISKINFO)GetWindowLong (hwnd, GWL_USERDATA);

		if (pkgi->hpaletteDone)
		{
			PAINTSTRUCT ps;
			HDC			hDC = BeginPaint (hwnd, &ps);

			SelectPalette (hDC, pkgi->hpaletteDone, FALSE);
			RealizePalette (hDC);
			EndPaint (hwnd, &ps);
		}
		break;

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch (pnmh->code) {
		case PSN_SETACTIVE:
			pkgi = (PDISKINFO)GetWindowLong (hwnd, GWL_USERDATA);

			PostMessage (GetParent (hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_BACK|PSWIZB_FINISH);

			SendDlgItemMessage (hwnd, IDC_WIZBITMAP, STM_SETIMAGE,
					IMAGE_BITMAP, (LPARAM) pkgi->hbitmapDone);

			SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;

		case PSN_WIZFINISH:
			pkgi = (PDISKINFO)GetWindowLong (hwnd, GWL_USERDATA);

			SetWindowLong(hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for actually generating the key(s)

static LRESULT WINAPI
sKeyWizardProgressDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PDISKINFO		pkgi;
	BOOL			bReturnCode = FALSE;
	DWORD			dwThreadId;

	switch (uMsg) 
	{
		case WM_INITDIALOG:
		{
			PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

			pkgi = (PDISKINFO) ppspMsgRec->lParam;
			SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

			SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
					(WPARAM)pkgi->hfontInterior, (LPARAM)FALSE);

			pkgi->hwndWiz = hwnd;

			// initialize Intel RNG icon and text
			if (!PGPGlobalRandomPoolHasIntelRNG ())
			{
				ShowWindow (GetDlgItem (hwnd, IDC_INTELICON), SW_HIDE);
				ShowWindow (GetDlgItem (hwnd, IDC_INTELTEXT), SW_HIDE);
			}

			// Kick off generation thread here
			pkgi->hThread=CreateThread(NULL, 0, sDiskGenerationThread, (void *) pkgi, 0, &dwThreadId);

			return TRUE;
		}

		case WM_TIMER :
		{
			if (wParam == PROGRESS_TIMER)
			{
				char szText[100];

				pkgi = (PDISKINFO)GetWindowLong (hwnd, GWL_USERDATA);

				if((pkgi->percent!=pkgi->oldPercent)||
					(pkgi->areFormatting!=pkgi->oldAreFormatting))
				{
					pkgi->oldPercent=pkgi->percent;
					pkgi->oldAreFormatting=pkgi->areFormatting;
	
					SendMessage(GetDlgItem(hwnd,IDC_CURTASK_BAR),
						PBM_SETPOS,(WPARAM)(pkgi->percent),0);

					if (pkgi->areFormatting)
					{
						sSetCurrentTaskInfo (pkgi,ICON_CHECK,IDC_TASKICON1);
						sSetCurrentTaskInfo (pkgi,ICON_ARROW,IDC_TASKICON2);

						strcpy(szText,"Formatting PGPdisk");

						SendMessage(GetDlgItem(hwnd,IDC_ALLTASK_BAR),
							PBM_SETPOS,(WPARAM)(50+(pkgi->percent/2)),0);
					}
					else
					{
						sSetCurrentTaskInfo (pkgi,ICON_ARROW,IDC_TASKICON1);

						strcpy(szText,"Encrypting PGPdisk");

						SendMessage(GetDlgItem(hwnd,IDC_ALLTASK_BAR),
							PBM_SETPOS,(WPARAM)(pkgi->percent/2),0);
					}
					SetWindowText(GetDlgItem(hwnd,IDC_CURTASK_TEXT),
						szText);
				}

				if(pkgi->bTaskComplete!=pkgi->oldTaskComplete)
				{
					pkgi->oldTaskComplete=pkgi->bTaskComplete;
				
					if(pkgi->err==kPGPError_UserAbort)
					{
						strcpy(szText,"PGPdisk creation aborted");
						sSetCurrentTaskInfo (pkgi,ICON_ERROR,
							pkgi->areFormatting?IDC_TASKICON2:IDC_TASKICON1);
					}
					else if(IsPGPError(pkgi->err))
					{
						char szError[1024];
						DWORD dwDebug;

						PGPclErrorToString(pkgi->err, szError, 1023, &dwDebug);

						SetWindowText(GetDlgItem(hwnd,IDC_CORPDISKERROR),
							szError);

						strcpy(szText,"PGPdisk creation failed");
						sSetCurrentTaskInfo (pkgi,ICON_ERROR,
							pkgi->areFormatting?IDC_TASKICON2:IDC_TASKICON1);
					}
					else
					{
						strcpy(szText,"PGPdisk creation finished");
						sSetCurrentTaskInfo (pkgi,ICON_CHECK,IDC_TASKICON2);
						SendMessage (GetParent(hwnd),
								PSM_SETWIZBUTTONS, 0, PSWIZB_NEXT);
					}
				SetWindowText(GetDlgItem(hwnd,IDC_CURTASK_TEXT),
					szText);
				}
			}
			break;
		}

		case WM_NOTIFY:
		{
			LPNMHDR pnmh = (LPNMHDR) lParam;

			pkgi = (PDISKINFO)GetWindowLong (hwnd, GWL_USERDATA);

			switch (pnmh->code) 
			{
				case PSN_SETACTIVE:
				{
					SendMessage (GetParent(hwnd), PSM_SETWIZBUTTONS, 0, 0);
					if(pkgi->bTaskComplete)
					{
						SendMessage (GetParent(hwnd),
							PSM_SETWIZBUTTONS, 0, PSWIZB_NEXT);
					}
					SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
					bReturnCode = TRUE;

					SetTimer (hwnd, PROGRESS_TIMER, PROGRESSTIMERPERIOD, NULL);
					break;
				}

				case PSN_KILLACTIVE:
					KillTimer (hwnd, PROGRESS_TIMER);
					break;

				case PSN_QUERYCANCEL:
				{
					pkgi->bCancelPending = TRUE;

					SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
					bReturnCode = TRUE;
					break;
				}
			}
			break;
		}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for getting entropy
//  from user

static LRESULT WINAPI
sKeyWizardRandobitsDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PDISKINFO		pkgi;
	BOOL			bReturnCode = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PDISKINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontInterior, (LPARAM)FALSE);
		return TRUE;
	}

	case WM_MOUSEMOVE:
	{
		INT		iPercentComplete = 0;
		FLOAT	fTotal, fSoFar;

		pkgi = (PDISKINFO)GetWindowLong (hwnd, GWL_USERDATA);

		if (pkgi->lRandomBitsNeeded)
		{
			fSoFar = (float)(PGPContextReserveRandomBytes (pkgi->context,
					(pkgi->lRandomBitsNeeded + 7) / 8) * 8);
			fTotal = (float) pkgi->lRandomBitsNeeded;
			fSoFar -= (float) pkgi->lOriginalEntropy;
			fTotal -= (float) pkgi->lOriginalEntropy;
			iPercentComplete = (INT) ((fSoFar / fTotal) * 100.0);

			if (fSoFar >= fTotal)
			{
				pkgi->lRandomBitsNeeded = 0;
				iPercentComplete = 100;
				s_hwndCollectEntropy = NULL;
				SendMessage (GetParent(hwnd),
						PSM_SETWIZBUTTONS, 0, PSWIZB_NEXT|PSWIZB_BACK);
			}

			SendDlgItemMessage (hwnd,
					IDC_PROGRESS, PBM_SETPOS, iPercentComplete, 0);
		}
		break;
	}

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch(pnmh->code) {
		case PSN_SETACTIVE:
			pkgi = (PDISKINFO)GetWindowLong (hwnd, GWL_USERDATA);
			PostMessage(GetParent(hwnd), PSM_SETWIZBUTTONS, 0, 0);

			{
				PGPOptionListRef	optionlist		= kInvalidPGPOptionListRef;
				PGPPrefRef			prefref			= kInvalidPGPPrefRef;
				PGPContextRef		ctx				= pkgi->context;

				LONG				lTotalBitsNeeded, lRandPoolSize;
				PGPBoolean			bFastGen		= FALSE;

				s_hwndCollectEntropy = hwnd;

				// Check to see if there are random bits needed
	
				lTotalBitsNeeded=
					PGPdiskGetNeededRandomDataSize(pkgi->diskAlgorithm)*8*2;

				lRandPoolSize = PGPGlobalRandomPoolGetSize ();
				pgpAssert (lRandPoolSize > lTotalBitsNeeded);
				pkgi->lRandomBitsNeeded = lTotalBitsNeeded;
			}

			PGPContextReserveRandomBytes (pkgi->context, 
					(pkgi->lRandomBitsNeeded + 7) / 8);

			if ((pkgi->lRandomBitsNeeded -
					8*(LONG)PGPContextReserveRandomBytes(pkgi->context, 0)) > 0)
			{
				pkgi->lOriginalEntropy = 
						8*PGPContextReserveRandomBytes(pkgi->context, 0);

				SendDlgItemMessage (hwnd, IDC_PROGRESS,
						PBM_SETRANGE, 0, MAKELPARAM(0, 100));
				SendDlgItemMessage (hwnd, IDC_PROGRESS,
						PBM_SETPOS, 0, 0);
				SetWindowLong(hwnd, DWL_MSGRESULT, 0L);
				bReturnCode = TRUE;

			}
			else
			{
				SetWindowLong (hwnd, DWL_MSGRESULT, -1L); // skip page
				bReturnCode = TRUE;
				s_hwndCollectEntropy = NULL;
			}
			break;

		case PSN_KILLACTIVE:

			SetWindowLong (hwnd, DWL_MSGRESULT, 0);
			bReturnCode = TRUE;
			s_hwndCollectEntropy = NULL;
			break;
		}
		break;
	}
	}

	return bReturnCode;
}

typedef struct
{
	TCHAR *wszString;
	PGPUInt32 iType;
} STRINGINFO;

static STRINGINFO AlgInfo[]={
	{TEXT("Invalid"),kDiskInvalidAlgorithm},
	{TEXT("COPYDATA"),kDiskCopyDataAlgorithm},
	{TEXT("CAST5"),kDiskOldCAST5Algorithm},
	{TEXT("CAST5"),kDiskCAST5Algorithm},
	{TEXT("TwoFish"),kDiskTwoFishAlgorithm},
	{TEXT("AES"),kDiskAESAlgorithm}};

static STRINGINFO FSInfo[]={
	{TEXT("FAT12"),kDiskFat12FileSys},
	{TEXT("FAT16"),kDiskFat16FileSys},
	{TEXT("FAT12 or FAT16"),kDiskFat12Or16FileSys},
	{TEXT("FAT32"),kDiskFat32FileSys},
	{TEXT("NTFS"),kDiskNTFSFileSys},
	{TEXT("Unknown"),kDiskUnknownFileSys}};

BOOL DoFileSave (HWND hWnd,TCHAR *wszTitle,TCHAR *wszFilename,BOOL bForce)
{
	TCHAR *p;
	BOOL bRst;
	OPENFILENAME SaveFileName;
	HANDLE *hFile;
	TCHAR StrRes[500];
	BOOL bAskUser,bRetry,bAddDefault;
	TCHAR DefaultExtension[MAX_PATH];
	int FileStart,FileExtensionStart;

	bAskUser=bForce;

	do
	{
		FileStart=FileExtensionStart=0;
		bRetry=FALSE;
		bRst=TRUE;
		_tcscpy(DefaultExtension,TEXT("\0"));

		if((p = _tcsrchr(wszFilename, (TCHAR)'\\')))
		{
			p++;
			FileStart = p - wszFilename;
		}

		if((p = _tcsrchr(wszFilename, (TCHAR)'.')))
		{
			p++;

			FileExtensionStart = p - wszFilename;
			_tcscpy(DefaultExtension,p); // Save old extension
									// it might get stripped
		}

		// Ask the user
		if(bAskUser)
		{
			memset (&SaveFileName, 0x00, sizeof (OPENFILENAME));

			LoadString (g_hInst, IDS_SAVEFILTER, StrRes, sizeof(StrRes));
			while (p = _tcsrchr (StrRes, '@')) *p = '\0';

			SaveFileName.lStructSize=sizeof(SaveFileName); 
			SaveFileName.hwndOwner=hWnd; 
			SaveFileName.hInstance=NULL; 
			SaveFileName.lpstrFilter=StrRes;
			SaveFileName.lpstrCustomFilter=NULL; 
			SaveFileName.nMaxCustFilter=0; 
			SaveFileName.nFilterIndex=1; 
  			SaveFileName.lpstrFile=wszFilename; 
			SaveFileName.nMaxFile=MAX_PATH; 
			SaveFileName.lpstrFileTitle=NULL; 
			SaveFileName.nMaxFileTitle=0; 
			SaveFileName.lpstrInitialDir=NULL; 
			SaveFileName.lpstrTitle=wszTitle; 
			SaveFileName.Flags= OFN_HIDEREADONLY|OFN_NOCHANGEDIR;
			SaveFileName.nFileOffset=FileStart; 
			SaveFileName.nFileExtension=FileExtensionStart; 
			SaveFileName.lpstrDefExt=DefaultExtension; 
			SaveFileName.lCustData=(long)NULL; 
			SaveFileName.lpfnHook=NULL;
			SaveFileName.lpTemplateName=NULL; 

			bRst = GetSaveFileName(&SaveFileName);
		}

		if(bRst)
		{
			// We'll likely always have some kind of extension
			if(DefaultExtension[0]!=0)
			{
				bAddDefault=FALSE;

				p = _tcsrchr(wszFilename, '.');

				if(p)
				{
					p++;

					// Extension found
					if(!_tcsicmp(DefaultExtension,p))
					{
						// They are the same. Woohoo! Do nothing
						bAddDefault=FALSE;
					}
					else
					{
						bAddDefault=TRUE;
					}
				}
				else
				{
					bAddDefault=TRUE;
				}

				if(bAddDefault)
				{
					int lastone;
					BOOL bDot;

					// They are different. Must be that 
					// pesky hide extensions option.
					lastone=_tcslen(wszFilename)-1;
					bDot=FALSE;

					if(lastone>=0)
					{
						if(wszFilename[lastone]==L'.')
							bDot=TRUE;
					}

					if(!bDot)
						_tcscat(wszFilename,TEXT("."));
					_tcscat(wszFilename,DefaultExtension);
				}
			}

			hFile=CreateFile (wszFilename, GENERIC_READ,
				0, NULL,OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

			// If we could open the file, we need to ask the user
			if(hFile!=INVALID_HANDLE_VALUE)
			{
				CloseHandle(hFile);

				if(!bAskUser)
				{
					// If we haven't asked, don't error out just yet. Show dlg
					bAskUser=bRetry=TRUE;
					continue;
				}
				else
				{
					// We've been here before. Error before doing anything
					if(IDCANCEL==MessageBox(hWnd,
						wszFilename,TEXT("File exists. Overwrite it?"),
						MB_OKCANCEL|MB_ICONSTOP|MB_SETFOREGROUND))
					{
						// Do over
						bAskUser=bRetry=TRUE;
						continue;
					}
				}

				// File doesn't exist, but can we create it?
				hFile=CreateFile (wszFilename, GENERIC_READ|GENERIC_WRITE,
					0, NULL,CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

				if(hFile==INVALID_HANDLE_VALUE)
				{
					if(!bAskUser)
					{
						// Couldn't create it, but don't error out yet. Dlg.
						bAskUser=bRetry=TRUE;
						continue;
					}
					else
					{
						// Been here before. Show error and do over
						MessageBox(hWnd,
							wszFilename,TEXT("Could not write to file!"),
							MB_OK|MB_ICONSTOP|MB_SETFOREGROUND);
						bAskUser=bRetry=TRUE;
						continue;
					}
				}
				else
				{
					// We can create the file. Close it and erase it
					CloseHandle(hFile);
					DeleteFile(wszFilename);
					// Should be done after this!
				}
			}
		}
	} while(bRetry);

	return(bRst);
}

//	______________________________________________
//
//  Dialog procedure for corporate options dialog

static LRESULT WINAPI
sKeyWizardCorpOptsDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PDISKINFO		pkgi;
	BOOL			bReturnCode = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PDISKINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontInterior, (LPARAM)FALSE);

		UTF8EditInit (GetDlgItem (hwnd, IDC_CORPDISKADK));

		return TRUE;
	}

	case WM_DESTROY:
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_CORPDISKADK));
		return FALSE;

	case WM_COMMAND:
	{
		switch(LOWORD(wParam)) 
		{
			case IDC_BROWSE:
			{
				pkgi = (PDISKINFO)GetWindowLong (hwnd, GWL_USERDATA);

				if(DoFileSave (hwnd,"Select filename",pkgi->diskName,TRUE))
				{
					SetWindowText(GetDlgItem(hwnd,IDC_CORPDISKFILENAME),
						pkgi->diskName);
				}
				break;
			}
		}
		break;
	}

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch (pnmh->code) 
		{
			case PSN_SETACTIVE:
			{
				int i;
				char szSize[100];
		
				pkgi = (PDISKINFO)GetWindowLong (hwnd, GWL_USERDATA);

				SetWindowText(GetDlgItem(hwnd,IDC_CORPDISKFILENAME),
					pkgi->diskName);

				for(i=0;i<(sizeof(AlgInfo)/sizeof(STRINGINFO));i++)
				{
					if(AlgInfo[i].iType==pkgi->diskAlgorithm)
						SetWindowText(GetDlgItem(hwnd,IDC_CORPDISKALGORITHM),
							AlgInfo[i].wszString);
				}

				for(i=0;i<(sizeof(FSInfo)/sizeof(STRINGINFO));i++)
				{
					if(FSInfo[i].iType==pkgi->diskFS)
						SetWindowText(GetDlgItem(hwnd,IDC_CORPDISKFILESYSTEM),
							FSInfo[i].wszString);
				}

				if((pkgi->diskSize%(1024*1024*2))==0)
					sprintf(szSize,"%d GB",pkgi->diskSize/(1024*1024*2));
				else if((pkgi->diskSize%(1024*2))==0)
					sprintf(szSize,"%d MB",pkgi->diskSize/(1024*2));
				else 
					sprintf(szSize,"%d KB",pkgi->diskSize/2);

				SetWindowText(GetDlgItem(hwnd,IDC_CORPDISKSIZE),
					szSize);

				SetWindowText(GetDlgItem(hwnd,IDC_CORPDISKFILENAME),
					pkgi->diskName);

				{
					CHAR sz[kPGPMaxUserIDSize];
					UINT u;

					PostMessage (GetParent (hwnd),
							PSM_SETWIZBUTTONS, 0, PSWIZB_BACK|PSWIZB_NEXT);

					if (PGPKeyDBObjRefIsValid (pkgi->diskADK))
					{
						ShowWindow (GetDlgItem (hwnd, IDC_CORPDISKADKTEXT), SW_SHOW);
						ShowWindow (UTF8EditGetHandle (
								GetDlgItem (hwnd, IDC_CORPDISKADK)), SW_SHOW);
						ShowWindow (GetDlgItem (hwnd, IDC_ADKTEXT2), SW_SHOW);
						u = sizeof(sz);
						PGPclGetPrimaryUserIDNameUTF8 (
								pkgi->diskADK, sz, sizeof(sz), &u);
						UTF8EditSetText (GetDlgItem (hwnd, IDC_CORPDISKADK), sz);
					}

					SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
				}

				bReturnCode = TRUE;
				break;
			}

			case PSN_KILLACTIVE :
			{
				pkgi = (PDISKINFO)GetWindowLong (hwnd, GWL_USERDATA);

				GetWindowText(GetDlgItem(hwnd,IDC_CORPDISKFILENAME),
					pkgi->diskName,MAX_PATH);
				break;
			}
		}
		break;
	}
	}

	return bReturnCode;
}



//	______________________________________________
//
//  Dialog procedure for handling beginning introductory
//  dialog.

static LRESULT WINAPI
sKeyWizardIntroDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	BOOL			bReturnCode = FALSE;
	PDISKINFO		pkgi		= NULL;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PDISKINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		// center dialog on screen
		PGPclCenterWindowOnMonitor (GetParent(hwnd), kPGPclWindowMonitor);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontIntro, (LPARAM)FALSE);

		return TRUE;
	}

	case WM_ACTIVATE :
		InvalidateRect (hwnd, NULL, TRUE);
		break;

	case WM_PAINT :
		pkgi = (PDISKINFO)GetWindowLong (hwnd, GWL_USERDATA);

		if (pkgi->hpaletteIntro)
		{
			PAINTSTRUCT ps;
			HDC			hDC = BeginPaint (hwnd, &ps);

			SelectPalette (hDC, pkgi->hpaletteIntro, FALSE);
			RealizePalette (hDC);
			EndPaint (hwnd, &ps);
		}
		break;

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch(pnmh->code) {
		case PSN_SETACTIVE:
			pkgi = (PDISKINFO)GetWindowLong (hwnd, GWL_USERDATA);

			PostMessage(GetParent(hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_NEXT);
			SendDlgItemMessage(hwnd, IDC_WIZBITMAP, STM_SETIMAGE,
					IMAGE_BITMAP, (LPARAM) pkgi->hbitmapIntro);

			SetWindowLong(hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;

		case PSN_KILLACTIVE :
			break;
		}

		break;
	}
	}

	return bReturnCode;
}

void ConstructValidDefaultPath(char *filename)
{
	char defaultPath[MAX_PATH];
	int len;
	PGPUInt32 i;
	FILE *ftest;

	PGPclGetPath(kPGPclDefaultSaveAsFolder, 
		defaultPath, 
		MAX_PATH);

	// Make sure it ends in a slash
	len=strlen(defaultPath);
	if(len!=0)
	{
		if(defaultPath[len-1]!='\\')
			strcat(defaultPath,"\\");
	}

	i=1;

	while (TRUE)
	{
		sprintf(filename,"%sPGPdisk%d.pgd",defaultPath,i);

		ftest=fopen(filename,"rb");

		if(ftest!=0)
		{
			fclose(ftest);
		}
		else
		{
			break;
		}
		i++;
	}
}

//	______________________________________________
//
//  Create wizard data structures and call PropertySheet
//  to actually create wizard

static PGPError
sDiskGenerationWizard (
	PGPContextRef		context,
	HWND				hwndParent,
	PGPKeyID			*pKeyID)
{
	PGPError			err					= kPGPError_NoErr;

	DISKINFO			KGInfo;
	PROPSHEETPAGE		pspWiz[NUM_WIZ_PAGES];
	PROPSHEETHEADER		pshWiz;
	INT					iIndex;
	INT					iNumBits, iBitmap;
	HDC					hDC;
	PGPPrefRef			userPrefRef;
	PGPBoolean			bUseADK,bAbort,bCreateDisk;

	bAbort=bCreateDisk=FALSE;

	// intialize windows
	s_hwndCollectEntropy = NULL;

	memset(&KGInfo,0x00,sizeof(DISKINFO));

	// Set defaults
	if (PGPContextRefIsValid (context))
		KGInfo.context = context;
	else
		KGInfo.context = g_context;

	KGInfo.hwndParent			= hwndParent;
	KGInfo.hwndWiz				= NULL;
	KGInfo.hpaletteIntro		= NULL;
	KGInfo.hpaletteDone			= NULL;

	// Determine which bitmap will be displayed in the wizard
	hDC = GetDC (NULL);	 // DC for desktop
	iNumBits = GetDeviceCaps (hDC, BITSPIXEL) * GetDeviceCaps (hDC, PLANES);
	ReleaseDC (NULL, hDC);

	if (iNumBits <= 1)
		iBitmap = IDB_KEYWIZ1;
	else if (iNumBits <= 4)
		iBitmap = IDB_KEYWIZ4;
	else
		iBitmap = IDB_KEYWIZ8;

	KGInfo.hbitmapIntro = CLLoadResourceBitmap (g_hInst,
			MAKEINTRESOURCE (iBitmap), &KGInfo.hpaletteIntro);
	KGInfo.hbitmapDone = CLLoadResourceBitmap (g_hInst,
			MAKEINTRESOURCE (iBitmap), &KGInfo.hpaletteDone);

	// load special fonts
	KGInfo.hfontIntro = CLLoadHeadingFont (FALSE);
	KGInfo.hfontInterior = CLLoadHeadingFont (TRUE);

	// load icon bitmaps
	KGInfo.hiconAlert = ImageList_GetIcon (
			g_hilClientLib, IDX_ALERT, ILD_TRANSPARENT);
	KGInfo.hiconDoing = ImageList_GetIcon (
			g_hilClientLib, IDX_ARROW, ILD_TRANSPARENT);
	KGInfo.hiconDone = ImageList_GetIcon (
			g_hilClientLib, IDX_CHECK, ILD_TRANSPARENT);
	KGInfo.hiconNotDone = ImageList_GetIcon (
			g_hilClientLib, IDX_ERROR, ILD_TRANSPARENT);

	// Set the values common to all pages
	for (iIndex=0; iIndex<NUM_WIZ_PAGES; iIndex++)
	{
		pspWiz[iIndex].dwSize		= sizeof(PROPSHEETPAGE);
		pspWiz[iIndex].dwFlags		= PSP_DEFAULT;
		pspWiz[iIndex].hInstance	= g_hInst;
		pspWiz[iIndex].pszTemplate  = NULL;
		pspWiz[iIndex].hIcon		= NULL;
		pspWiz[iIndex].pszTitle		= NULL;
		pspWiz[iIndex].pfnDlgProc   = NULL;
		pspWiz[iIndex].lParam		= (LPARAM) &KGInfo;
		pspWiz[iIndex].pfnCallback  = NULL;
		pspWiz[iIndex].pcRefParent  = NULL;
	}

	// Set up the intro page
	pspWiz[KGWIZ_CORPDISKINTRO].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_CORPDISKINTRO);
	pspWiz[KGWIZ_CORPDISKINTRO].pfnDlgProc = sKeyWizardIntroDlgProc;

	// Set up the corporate options page
	pspWiz[KGWIZ_CORPDISKOPTS].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_CORPDISKOPTS);
	pspWiz[KGWIZ_CORPDISKOPTS].pfnDlgProc = sKeyWizardCorpOptsDlgProc;

	// Set up the entropy page
	pspWiz[KGWIZ_ENTROPY].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_RANDOBITS);
	pspWiz[KGWIZ_ENTROPY].pfnDlgProc = sKeyWizardRandobitsDlgProc;

	// Set up the progress dialog
	pspWiz[KGWIZ_CORPDISKCREATION].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_CORPDISKCREATION);
	pspWiz[KGWIZ_CORPDISKCREATION].pfnDlgProc = sKeyWizardProgressDlgProc;

	// Set up the done page
	pspWiz[KGWIZ_CORPDISK_DONE].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_CORPDISK_DONE);
	pspWiz[KGWIZ_CORPDISK_DONE].pfnDlgProc = sKeyWizardDoneDlgProc;

	// Create the header
	pshWiz.dwSize		= sizeof(PROPSHEETHEADER);
	pshWiz.dwFlags		= PSH_WIZARD | PSH_PROPSHEETPAGE;
	pshWiz.hwndParent   = hwndParent;
	pshWiz.hInstance	= g_hInst;
	pshWiz.hIcon		= NULL;
	pshWiz.pszCaption   = NULL;
	pshWiz.nPages		= NUM_WIZ_PAGES;
	pshWiz.nStartPage   = KGWIZ_CORPDISKINTRO;
	pshWiz.ppsp			= pspWiz;
	pshWiz.pfnCallback  = NULL;

	PGPclPeekClientLibPrefRefs(&userPrefRef, NULL);

	if(userPrefRef==NULL)
	{
		MessageBox(hwndParent,
			"Error reading preferences. Please consult your administrator.",
			"PGP Error",
			MB_OK|MB_ICONSTOP|MB_SETFOREGROUND);

		bAbort=TRUE;
	}
	else
	{
		err = PGPGetPrefBoolean( userPrefRef,
					kPGPPrefDiskCreate, &bCreateDisk);

		if(IsPGPError(err)||(!bCreateDisk))
		{
			bAbort=TRUE;
		}
		else
		{
			err = PGPGetPrefNumber( userPrefRef,
						kPGPPrefDiskSize, &(KGInfo.diskSize) );

			err = PGPGetPrefNumber( userPrefRef,
						kPGPPrefDiskAlgorithm, &(KGInfo.diskAlgorithm) );		
			
			err = PGPGetPrefNumber( userPrefRef,
						kPGPPrefDiskFileSystem, &(KGInfo.diskFS) );

			err = PGPGetPrefBoolean( userPrefRef,
						kPGPPrefUsePGPdiskADK, &bUseADK );
		}
	}	

	ConstructValidDefaultPath(KGInfo.diskName);

	if(!bAbort)
	{
		// Execute the Wizard - doesn't return until Cancel or Save
		err = PGPdiskEngineInitLibrary(0);

		if(IsPGPError(err))
		{
			MessageBox(hwndParent,
				"Could not init DiskEngine library. Please consult your administrator.",
				"PGP Error",
				MB_OK|MB_ICONSTOP|MB_SETFOREGROUND);
		}
		else
		{
			err = PGPdiskUIInitLibrary();

			if(IsPGPError(err))
			{
				MessageBox(hwndParent,
					"Could not init DiskUI library. Please consult your administrator.",
					"PGP Error",
					MB_OK|MB_ICONSTOP|MB_SETFOREGROUND);
			}
			else
			{
				err=PGPdiskNewContext(kPGPdiskAPIVersion, &(KGInfo.diskContext));

				if(IsPGPError(err))
				{
					MessageBox(hwndParent,
						"Could not get Disk context. Please consult your administrator.",
						"PGP Error",
						MB_OK|MB_ICONSTOP|MB_SETFOREGROUND);
				}
				else
				{
					PGPdiskDirtyPGPKeyDB(KGInfo.diskContext);

					err=PGPdiskOpenPeekContextPGPKeyDB(KGInfo.diskContext,&(KGInfo.keydbMain));
					
					if(IsntPGPError(err))
					{
						if( bUseADK )
						{
							PGPSize	dataLength;
							PGPByte	*data;
								
							err = PGPGetPrefData( userPrefRef,
												kPGPPrefPGPdiskADKKeyID,
												&dataLength, &data );
							if( IsntPGPError( err ) )
							{
								err=PGPFindKeyByKeyID(
									KGInfo.keydbMain,
									(PGPKeyID *)data,
									&(KGInfo.diskADK));

								if(IsPGPError(err))
								{
									MessageBox(hwndParent,
										"The ADK for PGPdisk could not be found. Please consult your administrator.",
										"PGP Error",
										MB_OK|MB_ICONSTOP|MB_SETFOREGROUND);
									bAbort=TRUE;
								}

								PGPFreeData(data);
							}
						}

						if(!bAbort)
						{
							err=PGPFindKeyByKeyID(KGInfo.keydbMain,pKeyID,&(KGInfo.keyNew));

							if(IsntPGPError(err))
							{
								PropertySheet(&pshWiz);

								// Just in case the thread hasn't finished cancelling
								if(KGInfo.hThread!=0)
								{
									WaitForSingleObject(KGInfo.hThread,INFINITE);
									CloseHandle(KGInfo.hThread);
								}
							}
						}
					}

					PGPdiskFreeContext(KGInfo.diskContext);
				}
				PGPdiskUICleanupLibrary();
			}
			PGPdiskEngineCleanupLibrary();
		}
	}

	s_hwndCollectEntropy = NULL;

	DeleteObject (KGInfo.hbitmapIntro);
	DeleteObject (KGInfo.hbitmapDone);
	DeleteObject (KGInfo.hfontIntro);
	DeleteObject (KGInfo.hfontInterior);

	DestroyIcon (KGInfo.hiconAlert);
	DestroyIcon (KGInfo.hiconDoing);
	DestroyIcon (KGInfo.hiconDone);
	DestroyIcon (KGInfo.hiconNotDone);

	return kPGPError_NoErr;
}


//	______________________________________________
//
//  Create wizard data structures and call PropertySheet
//  to actually create wizard

PGPError PGPclExport
PGPclDiskGenerationWizard (
	PGPContextRef		context,
	HWND				hwndParent,
	PGPKeyID			*pKeyID)
{
	return sDiskGenerationWizard (context,
			hwndParent, pKeyID);
}
