/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	CLgenwiz.c - handle wizard for creating new keys

	$Id: CLgenwiz.c,v 1.115 2002/11/22 03:12:09 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "SpaceEdit.h"
#include "UTF8Edit.h"

// pgp header files
#include "pgpRandomPool.h"
#include "pgpUserInterface.h"
#include "pgpReconstruct.h"
#include "pgpUnicode.h"

// pgp disk files
#define PGPDISK_DYNLINK

#include "pgpDiskEngine.h"
#include "pgpDiskUI.h"

#include <stdio.h>

// system header files
#include <process.h>
#include <lmaccess.h>
#include <lmerr.h>

// constant definitions
#define TASK_KEYGEN				0
#define TASK_SUBKEYGEN			1
#define TASK_EXCHANGELOOKUP		2
#define TASK_SENDTOSERVER		3
#define TASK_REQUESTCERT		4
#define TASK_SENDRECONSTRUCT	5
#define MAX_TASKS				6

#define ICON_NONE				0
#define ICON_ARROW				1
#define ICON_CHECK				2
#define ICON_ERROR				3

#define MAX_FULL_NAME_LEN		126
#define MAX_EMAIL_LEN			126

#define BADPHRASE_LENGTH		0x01
#define BADPHRASE_QUALITY		0x02
#define BADPHRASE_CONFIRMATION	0x04
#define BADPHRASE_SMARTCARD		0x08

#define DEFAULT_MIN_KEY_SIZE	1024
#define MIN_RSA_KEY_SIZE		1024
#define MAX_RSA_KEY_SIZE		4096
#define MIN_RSA_LEGACY_KEY_SIZE	1024
#define MAX_RSA_LEGACY_KEY_SIZE	2048
#define MIN_DHDSS_KEY_SIZE		1024
#define MAX_DHDSS_KEY_SIZE		4096
#define MIN_EC_KEY_SIZE			283
#define MAX_EC_KEY_SIZE			283

#define DEFAULT_MINPHRASELEN	8
#define DEFAULT_MINPHRASEQUAL	60
#define DEFAULT_KEYSIZE			2048
#define DEFAULT_SUBKEYSIZE		2048
#define DEFAULT_KEYALGORITHM	kPGPPublicKeyAlgorithm_DSA
#define DEFAULT_EXPIREDAYS		0

#define PROGRESS_TIMER			2345L
#define LEDTIMERPERIOD			150L
#define NUMLEDS					10
#define LEDSPACING				2

#define CL_M_GENERATION_COMPLETE		WM_APP+97

#define KGWIZ_INTRO		0
#define KGWIZ_NAME		1
#define KGWIZ_EXPERT	2
#define KGWIZ_EXCHANGE	3
#define KGWIZ_CORPOPTS	4
#define KGWIZ_PHRASE	5
#define KGWIZ_BADPHRASE	6
#define KGWIZ_RECON		7
#define KGWIZ_ENTROPY   8
#define KGWIZ_PROGRESS	9
#define KGWIZ_DONE		10
#define NUM_WIZ_PAGES	11


// external globals
extern HINSTANCE		g_hInst;
extern PGPContextRef	g_context;
extern HIMAGELIST		g_hilClientLib;
extern BOOL				g_bUnicodeOS;

// local globals
static HHOOK	s_hhookKeyboard;
static HHOOK	s_hhookMouse;

static HHOOK	s_hhookCBT;
static HHOOK	s_hhookGetMessage;
static HHOOK	s_hhookMsgFilter;

static BOOL		s_bHooksInstalled		= FALSE;
static HWND		s_hwndCollectEntropy	= NULL;

// typedefs
typedef struct _KeyGenInfo
{
	PGPContextRef		context;
	PGPtlsContextRef	tlscontext;
	HWND				hwndWiz;
	HWND				hwndParent;
	PGPKeyDBRef			keydbMain;

	BOOL				bRecenter;
	HBITMAP				hbitmapIntro;
	HBITMAP				hbitmapDone;
	HPALETTE			hpaletteIntro;
	HPALETTE			hpaletteDone;
	HFONT				hfontIntro;
	HFONT				hfontInterior;
	HICON				hiconAlert;
	HICON				hiconDoing;
	HICON				hiconDone;
	HICON				hiconNotDone;

	BOOL				bExpertMode;
	BOOL				bIsExchangeUser;
	BOOL				bLookupExchangeUserID;
	BOOL				bAutoAddExchangeUserID;
	LPSTR				pszFullName;
	LPSTR				pszEmail;
	LPSTR				pszNotesFullName;
	LPSTR				pszNotesEmail;
	LPSTR				pszUserID;
	LPSTR				pszExchangeUserID;
	LPSTR				pszPassPhrase;

	PGPKeyDBObjRef		keyADK;
	PGPKeyDBObjRef		keyCorp;
	PGPKeyDBObjRef		keyRevoker;
	PGPKeyDBObjRef		keyRootCA;
	BOOL				bEnforceADK;
	BOOL				bMetaCorpKey;
	UINT				uMetaTrustLevel;

	BOOL				bPreset;
	BOOL				bDoSend;
	BOOL				bDoCertRequest;
	BOOL				bDoReconstruction;
	BOOL				bAllowRSAGen;
	BOOL				bAllowECGen;
	BOOL				bMinPhraseQuality;
	INT					iMinPhraseQuality;
	BOOL				bMinPhraseLength;
	INT					iMinPhraseLength;
	BOOL				bAddNotesUserID;

	UINT				uMinKeySize;
	UINT				uPhraseFlags;
	UINT				uKeyType;
	UINT				uKeyAlg;
	UINT				uKeyFlags;
	UINT				uKeySize;
	UINT				uSubKeyAlg;
	UINT				uSubKeyFlags;
	UINT				uSubKeySize;
	UINT				uExpireDays;
	BOOL				bOnToken;
	SYSTEMTIME			stPresetExpiration;

	HWND				hwndExpirationDate;
	LONG				lRandomBitsNeeded;
	LONG				lOriginalEntropy;
	PGPKeyDBObjRef		keyNew;

	PGPReconPrompts		reconPrompts;
	PGPReconPasses		reconPasses;

	UINT				uNumTasks;
	UINT				uCurrentTask;
	BOOL				bTaskError;
	UINT				uTask[MAX_TASKS];
	BOOL				bTaskComplete;

	BOOL				bFinishSelected;
	BOOL				bCancelPending;
	BOOL				bCancelled;
	BOOL				bInGeneration;

	INT					iStatusValue;
	INT					iStatusDirection;

	INT					iFinalResult;
} KEYGENINFO, *PKEYGENINFO;

//	____________________________________
//
// Secure memory allocation routines
//

static VOID*
secAlloc (UINT uBytes)
{
	return (PGPNewSecureData (
				PGPPeekContextMemoryMgr (g_context),
				uBytes, 0));
}


static VOID
secFree (VOID* p)
{
	if (p)
	{
		FillMemory (p, lstrlen (p), '\0');
		PGPFreeData (p);
	}
}


//	____________________________________
//
//  Hook procedure for WH_KEYBOARD hook

static LRESULT CALLBACK
sWizKeyboardHookProc (
		INT		iCode,
		WPARAM	wParam,
		LPARAM	lParam)
{
	if (iCode >= 0)
	{
		// only use key presses (not releases)
		if (!(lParam & 0x80000000))
		{
			PGPGlobalRandomPoolAddKeystroke (wParam);
			if (s_hwndCollectEntropy)
			{
				if (wParam != VK_ESCAPE)
				{
					PostMessage (s_hwndCollectEntropy, WM_MOUSEMOVE, 0, 0);
					return 1;
				}
			}
		}
	}

	return 0;
}


//	____________________________________
//
//  Hook procedure for WH_MOUSE hook

static LRESULT CALLBACK
sWizMouseHookProc (
		INT		iCode,
		WPARAM	wParam,
		LPARAM	lParam)
{
	if (iCode >= 0)
		PGPGlobalRandomPoolMouseMoved ();

	return 0;
}


//	____________________________________
//
//  Hook procedure for various hooks -- used to prevent
//  passing the hook info on to next hook procedure

static LRESULT CALLBACK
sWizGenericHookProc (
		INT		iCode,
		WPARAM	wParam,
		LPARAM	lParam)
{
	return 0;
}


//	____________________________________
//
//  set the message hooks for gathering entropy

static VOID
sInstallEntropyHooks (void)
{
	DWORD	dwThreadID;

	dwThreadID = GetCurrentThreadId ();

	// keyboard hook is to trap entropy
	s_hhookKeyboard = SetWindowsHookEx (
			WH_KEYBOARD, sWizKeyboardHookProc, NULL, dwThreadID);

	// mouse hook is to trap entropy
	s_hhookMouse = SetWindowsHookEx (
			WH_MOUSE, sWizMouseHookProc, NULL, dwThreadID);
}


//	____________________________________
//
//  set the message hooks for preventing snooping

static VOID
sInstallSecurityHooks (void)
{
	DWORD	dwThreadID;

	dwThreadID = GetCurrentThreadId ();

	s_hhookCBT = SetWindowsHookEx (
			WH_CBT, sWizGenericHookProc, NULL, dwThreadID);
	s_hhookGetMessage = SetWindowsHookEx (
			WH_GETMESSAGE, sWizGenericHookProc, NULL, dwThreadID);
	s_hhookMsgFilter = SetWindowsHookEx (
			WH_MSGFILTER, sWizGenericHookProc, NULL, dwThreadID);
}


//	____________________________________
//
//  remove the entropy message hooks

static VOID
sUninstallEntropyHooks (void)
{
	UnhookWindowsHookEx (s_hhookMouse);
	UnhookWindowsHookEx (s_hhookKeyboard);
}


//	____________________________________
//
//  remove the security message hooks

static VOID
sUninstallSecurityHooks (void)
{
	UnhookWindowsHookEx (s_hhookMsgFilter);
	UnhookWindowsHookEx (s_hhookGetMessage);
	UnhookWindowsHookEx (s_hhookCBT);
}


//	______________________________________________________
//
//	Convert time to string format based on system settings

static INT
sPGPTimeToDays (
		PGPTime			pgptime,
		SYSTEMTIME*		pst)
{
	struct tm*	ptm				= NULL;
	INT			iDays			= 0;
	time_t		ttTime;

	if (pgptime > 0)
	{
		ttTime = PGPGetStdTimeFromPGPTime (pgptime);
		ptm = localtime (&ttTime);
	}

	if (ptm)
	{
		CLTimeToSystemTime (ptm, pst);
		PGPclSystemTimeToDays (pst, &iDays);
	}

	return iDays;
}

//	____________________________________
//
//  return DSA key size on basis of requested ElGamal key size

static ULONG
sGetDSAKeySize (ULONG ulRequested)
{
	ULONG ulActualBits = ulRequested;

	if (ulRequested > 1024)
		ulActualBits = 1024;

	return ulActualBits;
}


//	____________________________________
//
//  load up admin prefs

static BOOL
sValidateConfiguration (PKEYGENINFO pkgi)
{
	BOOL		bPrefsOK	= TRUE;
	PGPPrefRef	prefref		= kInvalidPGPPrefRef;

	PGPError	err;
	PGPUInt32	u;

	PGPByte*	pbyte;
	PGPSize		size;
	PGPBoolean	b;

	pkgi->uMinKeySize		= DEFAULT_MIN_KEY_SIZE;
	pkgi->uSubKeySize		= DEFAULT_SUBKEYSIZE;
	pkgi->pszPassPhrase		= NULL;
	pkgi->uPhraseFlags		= 0;
	pkgi->lRandomBitsNeeded	= 0;
	pkgi->lOriginalEntropy	= 0;
	pkgi->keyNew			= kInvalidPGPKeyDBObjRef;
	pkgi->bFinishSelected	= FALSE;
	pkgi->bCancelPending	= FALSE;
	pkgi->bCancelled		= FALSE;
	pkgi->bInGeneration		= FALSE;
	pkgi->bTaskComplete		= FALSE;
	pkgi->iFinalResult		= 0;

	err = PGPclPeekClientLibPrefRefs (&prefref, NULL);
	if (IsPGPError (err))
		return FALSE;

	// Use preset keygen parameters?
	if(PGPlnEnterprise())
	{
	b = FALSE;
	PGPGetPrefBoolean (prefref, kPGPPrefPresetKeyGeneration, &b);
	pkgi->bPreset = b;
	}
	else
	{
	pkgi->bPreset = FALSE;
	}

	// Preset algorithm
	if(PGPlnEnterprise())
	{
		u = DEFAULT_KEYALGORITHM;
		PGPGetPrefNumber (prefref, kPGPPrefKeyPublicKeyAlgorithm, &u);
		pkgi->uKeyType = u;
	}
	else
	{
		pkgi->uKeyType = DEFAULT_KEYALGORITHM;
	}
	if (pkgi->uKeyType == kPGPPublicKeyAlgorithm_DSA)
	{
		pkgi->uKeyAlg = kPGPPublicKeyAlgorithm_DSA;
		pkgi->uKeyFlags = kPGPKeyPropertyFlags_UsageSign;
		pkgi->uSubKeyAlg = kPGPPublicKeyAlgorithm_ElGamal;
		pkgi->uSubKeyFlags = kPGPKeyPropertyFlags_UsageEncrypt;
	}
	else if (pkgi->uKeyType == kPGPPublicKeyAlgorithm_RSASignOnly)
	{
		pkgi->uKeyAlg = kPGPPublicKeyAlgorithm_RSA;
		pkgi->uKeyFlags = kPGPKeyPropertyFlags_UsageSign;
		pkgi->uSubKeyAlg = kPGPPublicKeyAlgorithm_RSA;
		pkgi->uSubKeyFlags = kPGPKeyPropertyFlags_UsageEncrypt;
	}
	else if (pkgi->uKeyType == kPGPPublicKeyAlgorithm_RSA)
	{
		pkgi->uKeyAlg = kPGPPublicKeyAlgorithm_RSA;
		pkgi->uKeyFlags = 0;
		pkgi->uSubKeyAlg = 0;
		pkgi->uSubKeyFlags = 0;
	}
	else
	{
		pkgi->uKeyAlg = kPGPPublicKeyAlgorithm_ECSign;
		pkgi->uKeyFlags = kPGPKeyPropertyFlags_UsageSign;
		pkgi->uSubKeyAlg = kPGPPublicKeyAlgorithm_ECEncrypt;
		pkgi->uSubKeyFlags = kPGPKeyPropertyFlags_UsageEncrypt;
	}

	// Preset size
	u = DEFAULT_KEYSIZE;
	if(PGPlnEnterprise())
	{
		PGPGetPrefNumber (prefref, kPGPPrefKeySize, &u);
	}
	if (pkgi->uKeyType == kPGPPublicKeyAlgorithm_DSA)
	{
	 	pkgi->uKeySize = sGetDSAKeySize (u);
	 	pkgi->uSubKeySize = u;
	}
	else
	{
	 	pkgi->uKeySize = u;
	 	pkgi->uSubKeySize = u;
	}

	// Preset expiration
	if(PGPlnEnterprise())
	{
		u = 0;
		PGPGetPrefNumber (prefref, kPGPPrefKeyExpiration, &u);
		if (u != 0)
		{
			u += PGPGetTime ();
			u = (UINT) sPGPTimeToDays ((PGPTime)u, &pkgi->stPresetExpiration);
		}
		pkgi->uExpireDays = u;
	}
	else
	{
//#else
		pkgi->uExpireDays = DEFAULT_EXPIREDAYS;
	}

	// Smart Card
	if(PGPlnEnterprise())
	{
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefGenerateKeysOnSmartCard, &b);
		pkgi->bOnToken = b;
		if (pkgi->bOnToken)
		{
			CHAR	szDriver[MAX_PATH];
			PGPGetPrefStringBuffer (prefref, kPGPPrefPKCS11DrvFile,
					sizeof (szDriver), szDriver);

			if (szDriver[0] == '\0')
			{
				bPrefsOK = FALSE;
				PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONERROR,
						IDS_SMARTCARDNOTCONFIGURED, MB_OK|MB_ICONSTOP);
			}
			else
			{
				UINT	uNumTokens;

				PGPCountTokens (g_context, &uNumTokens);
				if (uNumTokens == 0)
				{
					bPrefsOK = FALSE;
					PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONERROR,
							IDS_SMARTCARDNOTINSERTED, MB_OK|MB_ICONSTOP);
				}
			}
		}
	}
	else
	{
		pkgi->bOnToken = FALSE;
	}

	// minimum passphrase length
	if(PGPlnEnterprise())
	{
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefEnforceMinChars, &b);
		pkgi->bMinPhraseLength = b;
		if (b)
		{
			u = DEFAULT_MINPHRASELEN;
			PGPGetPrefNumber (prefref, kPGPPrefMinChars, &u);
			pkgi->iMinPhraseLength = (INT)u;
		}
		else
			pkgi->iMinPhraseLength = DEFAULT_MINPHRASELEN;
	}
	else
	{
		pkgi->bMinPhraseLength = FALSE;
		pkgi->iMinPhraseLength = DEFAULT_MINPHRASELEN;
	}

	// minimum passphrase quality
	if(PGPlnEnterprise())
	{
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefEnforceMinQuality, &b);
		pkgi->bMinPhraseQuality = b;
		if (b)
		{
			u = DEFAULT_MINPHRASEQUAL;
			PGPGetPrefNumber (prefref, kPGPPrefMinQuality, &u);
			pkgi->iMinPhraseQuality = (INT)u;
		}
		else
			pkgi->iMinPhraseQuality = 0;
	}
	else
	{
		pkgi->bMinPhraseQuality = FALSE;
		pkgi->iMinPhraseQuality = 0;
	}

	// RSA key generation
	pkgi->bAllowRSAGen = TRUE;
	err = PGPclCheckSDKSupportForPKAlg (
			kPGPPublicKeyAlgorithm_RSA, FALSE, FALSE);
	if (IsPGPError (err))
		pkgi->bAllowRSAGen = FALSE;

	// EC key generation
	pkgi->bAllowECGen = TRUE;
	err = PGPclCheckSDKSupportForPKAlg (
			kPGPPublicKeyAlgorithm_ECSign, FALSE, FALSE);
	if (IsPGPError (err))
		pkgi->bAllowECGen = FALSE;

	// ADK stuff
	if(PGPlnEnterprise())
	{
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefEnforceIncomingADK, &b);
		pkgi->bEnforceADK = b;

		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefUseDHADK, &b);
		if (b)
		{
			err = PGPGetPrefData (prefref,
					kPGPPrefDHADKID, &size, &pbyte);

			if (IsntPGPError (err))
			{
				err = PGPFindKeyByKeyID (pkgi->keydbMain,
					(PGPKeyID*)pbyte, &pkgi->keyADK);
				PGPFreeData (pbyte);
			}

			if (IsPGPError (err))
			{
				bPrefsOK = FALSE;
				PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONERROR,
						IDS_MISSINGADK, MB_OK|MB_ICONSTOP);
			}
			else
			{
				PGPGetKeyDBObjBooleanProperty (pkgi->keyADK,
						kPGPKeyProperty_IsExpired, &b);
				if (b)
				{
					bPrefsOK = FALSE;
					PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONERROR,
							IDS_EXPIREDADK, MB_OK|MB_ICONSTOP);
				}
				else
				{
					PGPGetKeyDBObjBooleanProperty (pkgi->keyADK,
							kPGPKeyProperty_IsRevoked, &b);
					if (b)
					{
						bPrefsOK = FALSE;
						PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONERROR,
								IDS_REVOKEDADK, MB_OK|MB_ICONSTOP);
					}
				}
			}
		}
		else
			pkgi->keyADK = kInvalidPGPKeyDBObjRef;
	}
	else
	{
		pkgi->keyADK = kInvalidPGPKeyDBObjRef;
		pkgi->bEnforceADK = FALSE;
	}

	// Corporate Key stuff
	if(PGPlnEnterprise())
	{
		// corp key signing type
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefMetaIntroducerCorp, &b);
		pkgi->bMetaCorpKey = b;

		u = 2;
		PGPGetPrefNumber (prefref, kPGPPrefMetaIntroducerTrustDepth, &u);
		pkgi->uMetaTrustLevel = u;

		// corp key
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefAutoSignTrustCorp, &b);
		if (b)
		{
			err = PGPGetPrefData (prefref,
					kPGPPrefCorpKeyID, &size, &pbyte);

			if (IsntPGPError (err))
			{
				err = PGPFindKeyByKeyID (pkgi->keydbMain,
						(PGPKeyID*)pbyte, &pkgi->keyCorp);
				PGPFreeData (pbyte);
			}

			if (IsPGPError (err))
			{
				bPrefsOK = FALSE;
				PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONERROR,
						IDS_MISSINGCORPKEY, MB_OK|MB_ICONSTOP);
			}
			else
			{
				PGPGetKeyDBObjBooleanProperty (pkgi->keyCorp,
						kPGPKeyProperty_IsExpired, &b);
				if (b)
				{
					bPrefsOK = FALSE;
					PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONERROR,
							IDS_EXPIREDCORPKEY, MB_OK|MB_ICONSTOP);
				}
				else
				{
					PGPGetKeyDBObjBooleanProperty (pkgi->keyCorp,
							kPGPKeyProperty_IsRevoked, &b);
					if (b)
					{
						bPrefsOK = FALSE;
						PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONERROR,
								IDS_REVOKEDCORPKEY, MB_OK|MB_ICONSTOP);
					}
				}
			}
		}
		else
			pkgi->keyCorp = kInvalidPGPKeyDBObjRef;
	}
	else
	{
		pkgi->keyCorp = kInvalidPGPKeyDBObjRef;
		pkgi->bMetaCorpKey = FALSE;
	}

	// Designated Revoker Key stuff
	if(PGPlnEnterprise())
	{
		// revoker key
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefAutoAddRevoker, &b);
		if (b)
		{
			err = PGPGetPrefData (prefref,
					kPGPPrefRevokerKeyID, &size, &pbyte);

			if (IsntPGPError (err))
			{
				err = PGPFindKeyByKeyID (pkgi->keydbMain,
						(PGPKeyID*)pbyte, &pkgi->keyRevoker);
				PGPFreeData (pbyte);
			}

			if (IsPGPError (err))
			{
				bPrefsOK = FALSE;
				PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONERROR,
						IDS_MISSINGREVOKERKEY, MB_OK|MB_ICONSTOP);
			}
			else
			{
				PGPGetKeyDBObjBooleanProperty (pkgi->keyRevoker,
						kPGPKeyProperty_IsExpired, &b);
				if (b)
				{
					bPrefsOK = FALSE;
					PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONERROR,
							IDS_EXPIREDREVOKERKEY, MB_OK|MB_ICONSTOP);
				}
				else
				{
					PGPGetKeyDBObjBooleanProperty (pkgi->keyRevoker,
							kPGPKeyProperty_IsRevoked, &b);
					if (b)
					{
						bPrefsOK = FALSE;
						PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONERROR,
								IDS_REVOKEDREVOKERKEY, MB_OK|MB_ICONSTOP);
					}
				}
			}
		}
		else
			pkgi->keyRevoker = kInvalidPGPKeyDBObjRef;
	}
	else
	{
		pkgi->keyRevoker = kInvalidPGPKeyDBObjRef;
	}

	// Root CA Key stuff
	if(PGPlnEnterprise())
	{
		// auto sign root CA
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefAutoSignRootCAKey, &b);
		if (b)
		{
			err = PGPGetPrefData (prefref,
					kPGPPrefCARootKeyID, &size, &pbyte);

			if (IsntPGPError (err))
			{
				err = PGPFindKeyByKeyID (pkgi->keydbMain,
						(PGPKeyID*)pbyte, &pkgi->keyRootCA);
				PGPFreeData (pbyte);
			}

			if (IsPGPError (err))
			{
				bPrefsOK = FALSE;
				PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONINFO,
						IDS_MISSINGROOTCAKEY, MB_OK|MB_ICONINFORMATION);
			}
			else
			{
				PGPGetKeyDBObjBooleanProperty (pkgi->keyRootCA,
						kPGPKeyProperty_IsExpired, &b);
				if (b)
				{
					bPrefsOK = FALSE;
					PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONINFO,
							IDS_EXPIREDROOTCAKEY, MB_OK|MB_ICONINFORMATION);
					b = FALSE;
				}
				else
				{
					PGPGetKeyDBObjBooleanProperty (pkgi->keyRootCA,
							kPGPKeyProperty_IsRevoked, &b);
					if (b)
					{
						bPrefsOK = FALSE;
						PGPclMessageBox (pkgi->hwndParent, IDS_CAPTIONINFO,
								IDS_REVOKEDROOTCAKEY, MB_OK|MB_ICONINFORMATION);
					}
				}
			}
		}
		else
			pkgi->keyRootCA = kInvalidPGPKeyDBObjRef;
	}
	else
	{
		pkgi->keyRootCA = kInvalidPGPKeyDBObjRef;
	}

	// Lotus Notes stuff
	if(PGPlnEnterprise())
	{
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefAddLotusNotesUserID, &b);
		pkgi->bAddNotesUserID = b;
	}
	else
	{
		pkgi->bAddNotesUserID = FALSE;
	}

	if(PGPlnEnterprise())
	{
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefAutoAddExchangeServerAddress, &b);
		pkgi->bAutoAddExchangeUserID = b;
	}
	else
	{
		pkgi->bAutoAddExchangeUserID = FALSE;
	}

	// Auto Send stuff
	if(PGPlnEnterprise())
	{
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefAlwaysSendKeys, &b);
		pkgi->bDoSend = b;
	}
	else
	{
		pkgi->bDoSend = FALSE;
	}

	// Auto Cert Request stuff
	if(PGPlnEnterprise())
	{
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefKeyGenX509CertRequest, &b);
		pkgi->bDoCertRequest = b;
	}
	else
	{
		pkgi->bDoCertRequest = FALSE;
	}

	// Auto Key Reconstruction stuff
	if(PGPlnEnterprise())
	{
		b = FALSE;
		PGPGetPrefBoolean (prefref, kPGPPrefAutoKeyReconstruction, &b);
		pkgi->bDoReconstruction = b;
	}
	else
	{
		pkgi->bDoReconstruction = FALSE;
	}

	return bPrefsOK;
}

//	____________________________________
//
//  try to get the user's name and email

typedef NET_API_STATUS (__stdcall *NetUserGetInfoProc)
		(LPCWSTR, LPCWSTR, DWORD, LPBYTE*);

typedef NET_API_STATUS (__stdcall *NetApiBufferFreeProc) (LPVOID);

static VOID
sGetUserInfo (
		LPSTR*	ppszName,
		LPSTR*	ppszEmail)
{
	PGPclWindowsFamily	family;

	PGPclGetWindowsVersion (&family, NULL, NULL);

	switch (family) {

	// under NT or 2000, we can use the NetUserGetInfo API for getting
	// the full username.  Don't know how to get email address.
	case kPGPclWindowsFamilyNT :
		if (IsNull (*ppszName))
		{
			HINSTANCE	hmod;

			hmod = LoadLibrary ("NetApi32.DLL");
			if (hmod)
			{
				NetUserGetInfoProc		fpNetUserGetInfo;
				NetApiBufferFreeProc	fpNetApiBufferFree;

				fpNetUserGetInfo = (NetUserGetInfoProc)
						GetProcAddress (hmod, "NetUserGetInfo");

				fpNetApiBufferFree = (NetApiBufferFreeProc)
						GetProcAddress (hmod, "NetApiBufferFree");

				if (fpNetUserGetInfo && fpNetApiBufferFree)
				{
					LPUSER_INFO_10	pui10	= NULL;
					NET_API_STATUS	ns;
					CHAR			szUser[UNLEN+1];
					WCHAR			wszUser[UNLEN+1];
					DWORD			dw;

					dw = sizeof(szUser);
					GetUserName (szUser, &dw);
					MultiByteToWideChar (CP_ACP, 0,
							szUser, -1, wszUser, UNLEN+1);

					ns = fpNetUserGetInfo (NULL, wszUser, 10, (LPBYTE*)&pui10);

					if (ns == NERR_Success)
					{
						CHAR	sz[256];
						INT		ilen;

						ilen = WideCharToMultiByte (CP_ACP, 0,
								pui10->usri10_full_name,
								-1, sz, sizeof(sz), NULL, NULL);

						if (ilen > 0)
						{
							*ppszName = clAlloc (ilen);
							if (*ppszName)
								lstrcpy (*ppszName, sz);
						}
					}

					if (IsntNull (pui10))
						fpNetApiBufferFree (pui10);
				}

				FreeLibrary (hmod);
			}
		}
		break;

	// don't know how do get any info in Win9x
	default :
		break;
	}
}


//	______________________________________________
//
//  create standard PGP userid from name and email address

static ULONG
sCreatePGPUserID (
		LPSTR*	ppszUserID,
		LPSTR	szFullName,
		LPSTR	szEmail)
{
	INT		iReturnCode		= kPGPError_NoErr;
	UINT	uUserIDLen		= 0;

	CHAR	szTemp[kPGPMaxUserIDSize];

	if (szEmail[0])
		wsprintf (szTemp, "%s <%s>", szFullName, szEmail);
	else
		lstrcpy (szTemp, szFullName);

	*ppszUserID = clAlloc (sizeof(char) * kPGPMaxUserIDSize);
	if (*ppszUserID)
		lstrcpy (*ppszUserID, szTemp);
	else
		return kPGPError_OutOfMemory;

	return iReturnCode;
}


//	____________________________________
//
//  try to get the user's MS Exchange name and email address

static VOID
sLookupExchangeUserID (
		HWND		hwnd,
		PKEYGENINFO	pkgi)
{
	CHAR		szName[128];
	CHAR		szAddr[128];
	PGPError	err;

	err = PGPclGetExchangeServerAddress (
			hwnd, TRUE, szName, sizeof(szName), szAddr, sizeof(szAddr));

	if (err == kPGPError_FeatureNotAvailable)
	{
		PGPclMessageBox (hwnd, IDS_CAPTIONINFO, IDS_EXCHANGEINFO, 
				MB_OK|MB_ICONINFORMATION);

		err = PGPclGetExchangeServerAddress (
				hwnd, FALSE, szName, sizeof(szName), szAddr, sizeof(szAddr));
	}

	if (IsntPGPError (err))
		sCreatePGPUserID (&pkgi->pszExchangeUserID, szName, szAddr);
}


//	____________________________________
//
//  parse the Notes "Location" string into Full Name and Email address

static VOID
sParseNotesLocation (
		LPSTR	pszNotesLocation,
		LPSTR*	ppszName,
		LPSTR*	ppszEmail)
{
	LPSTR	pSrc;
	LPSTR	p;
	INT		ilen;

	pSrc = strchr (pszNotesLocation, ',');
	if (pSrc)
	{
		pSrc++;
		pSrc = strchr (pSrc, ',');
		if (pSrc)
		{
			pSrc++;
			if (strncmp (pSrc, "CN=", 3) == 0)
			{
				pSrc += 3;

				// first get the Full Name
				p = strchr (pSrc, '/');
				if (p)
				{
					*p = '\0';
					ilen = p-pSrc+1;
				}
				else
					ilen = lstrlen (pSrc) +1;

				*ppszName = clAlloc (ilen);
				if (*ppszName)
					lstrcpy (*ppszName, pSrc);

				if (p)
					*p = '/';

				// now construct the email address
				*ppszEmail = clAlloc (lstrlen (pSrc)); // worst case
				if (*ppszEmail)
				{
					*ppszEmail[0] = '\0';

					while (*pSrc)
					{
						p = strchr (pSrc, '/');
						if (p)
							*p = '\0';

						lstrcat (*ppszEmail, pSrc);

						if (p)
						{
							lstrcat (*ppszEmail, "/");
							*p = '/';
							p = strchr (p, '=');
						}

						if (p)
							pSrc = p+1;
						else
							pSrc += lstrlen(pSrc);
					}
				}
			}
		}
	}
}


//	____________________________________
//
//  try to get the user's Lotus Notes name and email address

static VOID
sGetNotesUserInfo (
		LPSTR*	ppszName,
		LPSTR*	ppszEmail)
{
	HKEY		hkey;
	LONG		lResult;
	DWORD		dwValueType, dwSize;
	CHAR		sz[256];
	CHAR		szPath[MAX_PATH];

	// first try the Notes version 5 key
	LoadString (g_hInst, IDS_LOTUSNOTES5KEY, sz, sizeof(sz));
	lResult = RegOpenKeyEx (HKEY_LOCAL_MACHINE, sz, 0, KEY_READ, &hkey);

	if (lResult == ERROR_SUCCESS)
	{
		dwSize = sizeof(szPath);
		lResult = RegQueryValueEx (hkey,
				"Path", 0, &dwValueType, szPath, &dwSize);
		RegCloseKey (hkey);
	}
	else
	{
		// now try the Notes version 4 key
		LoadString (g_hInst, IDS_LOTUSNOTES4KEY, sz, sizeof(sz));
		lResult = RegOpenKeyEx (HKEY_LOCAL_MACHINE, sz, 0, KEY_READ, &hkey);

		if (lResult == ERROR_SUCCESS)
		{
			dwSize = sizeof(szPath);
			lResult = RegQueryValueEx (hkey,
					"Path", 0, &dwValueType, szPath, &dwSize);
			RegCloseKey (hkey);
		}
	}

	// we got the path, now try to open INI file
	if (lResult == ERROR_SUCCESS)
	{
		if (szPath[lstrlen(szPath) -1] != '\\')
			lstrcat (szPath, "\\");

		LoadString (g_hInst, IDS_LOTUSNOTESINIFILE, sz, sizeof(sz));
		lstrcat (szPath, sz);

		GetPrivateProfileString (
				"Notes", "Location", "", sz, sizeof(sz), szPath);

		if (sz[0] != '\0')
		{
			sParseNotesLocation (sz, ppszName, ppszEmail);
		}
	}
}


//	____________________________________
//
//  check that the specified properties mesh with those
//	supported by the token

static BOOL
sVerifyTokenProperties (
		HWND		hwnd,
		PKEYGENINFO	pkgi)
{
	BOOL			bValid		= TRUE;
	PGPTokenInfo	pti;

	pti.size =  sizeof(pti);
	PGPGetTokenInfo (pkgi->context, 0, &pti);

	switch (pkgi->uKeyAlg) {
	case kPGPPublicKeyAlgorithm_RSA :
		if (!pti.bRsa)
		{
			PGPclMessageBox (hwnd, IDS_CAPTIONINFO, 
					IDS_TOKENALGNOTSUPPORTED, MB_OK|MB_ICONINFORMATION);
			bValid = FALSE;
		}
		else
		{
			if ((pkgi->uKeySize < pti.minRsaKeySize) ||
				(pkgi->uKeySize > pti.maxRsaKeySize))
			{
				CHAR	szCaption[32], szFmt[128], szMsg[128];
				LoadString (g_hInst, IDS_CAPTIONINFO, 
						szCaption, sizeof(szCaption));
				LoadString (g_hInst, IDS_TOKENKEYSIZERANGE, 
						szFmt, sizeof(szFmt));
				wsprintf (szMsg, szFmt, pti.minRsaKeySize, pti.maxRsaKeySize);
				MessageBox (hwnd, szMsg, szCaption, 
						MB_OK|MB_ICONINFORMATION);
				bValid = FALSE;
			}
		}
		break;

	default :
		PGPclMessageBox (hwnd, IDS_CAPTIONINFO, 
				IDS_TOKENALGNOTSUPPORTED, MB_OK|MB_ICONINFORMATION);
		bValid = FALSE;
		break;
	}

	return bValid;
}


//	____________________________________
//
//  set font of heading text

HFONT
CLLoadHeadingFont (
		BOOL	bInterior)
{
	LOGFONT		lf;

	memset (&lf, 0, sizeof(lf));

	if (bInterior)
	{
		LoadString (g_hInst, IDS_INTERIORWIZFONT,
				lf.lfFaceName, sizeof(lf.lfFaceName));
		lf.lfHeight = 8;
		lf.lfWeight = FW_BOLD;
	}
	else
	{
		LoadString (g_hInst, IDS_INTROWIZFONT,
				lf.lfFaceName, sizeof(lf.lfFaceName));
		lf.lfWeight = FW_BOLD;
	}

	return (CreateFontIndirect (&lf));
}


//	____________________________________
//
//  force redrawing of the "LED" progress indicator

static VOID
sInvalidateLEDs (
		HWND		hwnd,
		UINT		idc)
{
	RECT	rc;

	GetWindowRect (GetDlgItem (hwnd, idc), &rc);
	MapWindowPoints (NULL, hwnd, (LPPOINT)&rc, 2);
	InvalidateRect (hwnd, &rc, FALSE);
}


//	____________________________________
//
//  Draw the "LED" progress indicator

static VOID
sDrawSendStatus (
		HWND		hwnd,
		PKEYGENINFO	pkgi)
{
	HBRUSH			hBrushLit, hBrushUnlit, hBrushOld;
	HPEN			hPen, hPenOld;
	INT				i;
	INT				itop, ibot, ileft, iright, iwidth;
	RECT			rc;
	HDC				hdc;
	PAINTSTRUCT		ps;

	if (pkgi->iStatusValue < -1)
		return;

	hdc = BeginPaint (hwnd, &ps);

	// draw 3D shadow
	GetClientRect (hwnd, &rc);
	itop = rc.top+1;
	ibot = rc.bottom-2;

	iwidth = (rc.right-rc.left) / NUMLEDS;
	iwidth -= LEDSPACING;

	ileft = rc.left + 4;
	for (i=0; i<NUMLEDS; i++)
	{
		iright = ileft + iwidth;

		MoveToEx (hdc, ileft, ibot, NULL);
		LineTo (hdc, iright, ibot);
		LineTo (hdc, iright, itop);

		ileft += iwidth + LEDSPACING;
	}

	hPen = CreatePen (PS_SOLID, 0, RGB (128, 128, 128));
	hPenOld = SelectObject (hdc, hPen);
	hBrushLit = CreateSolidBrush (RGB (0, 255, 0));
	hBrushUnlit = CreateSolidBrush (RGB (0, 128, 0));

	ileft = rc.left + 4;

	// draw "Knight Rider" LEDs
	if (pkgi->iStatusDirection)
	{
		hBrushOld = SelectObject (hdc, hBrushUnlit);
		for (i=0; i<NUMLEDS; i++)
		{
			iright = ileft + iwidth;

			if (i == pkgi->iStatusValue)
			{
				SelectObject (hdc, hBrushLit);
				Rectangle (hdc, ileft, itop, iright, ibot);
				SelectObject (hdc, hBrushUnlit);
			}
			else
				Rectangle (hdc, ileft, itop, iright, ibot);

			ileft += iwidth + LEDSPACING;
		}
	}

	SelectObject (hdc, hBrushOld);
	SelectObject (hdc, hPenOld);
	DeleteObject (hPen);
	DeleteObject (hBrushLit);
	DeleteObject (hBrushUnlit);

	EndPaint (hwnd, &ps);
}


//	______________________________________________
//
//  call SetThreadExecutionState if available

static VOID
sSetThreadExecutionState (UINT uState)
{
	PGPclWindowsGenus		genus;

	PGPclGetWindowsVersion (NULL, &genus, NULL);

	switch (genus) {
	case kPGPclWindowsGenus98 :
	case kPGPclWindowsGenusMillenium :
	case kPGPclWindowsGenusPostMillenium :
	case kPGPclWindowsGenus2000 :
	case kPGPclWindowsGenusXp :
	case kPGPclWindowsGenusPostKnown :
	{
		HINSTANCE	hMod;
		FARPROC		pSetThreadExecutionState;

		hMod = LoadLibrary ("KERNEL32.DLL");
		pSetThreadExecutionState =
				GetProcAddress (hMod, "SetThreadExecutionState");

		if (pSetThreadExecutionState)
		{
			(*pSetThreadExecutionState)(uState);
		}

		FreeLibrary (hMod);
		break;
	}
	}
}


//	______________________________________________
//
//  setup the current task text

static VOID
sSetCurrentTaskInfo (
		PKEYGENINFO		pkgi,
		UINT			uIcon)
{
	HICON	hicon		= NULL;
	INT		iprog		= 0;
	INT		ids;
	CHAR	sz[64];

	switch (uIcon) {
	case ICON_ARROW :
		hicon = pkgi->hiconDoing;
		iprog = 3;
		break;

	case ICON_CHECK :
		hicon = pkgi->hiconDone;
		iprog = 10;
		break;

	case ICON_ERROR :
		hicon = pkgi->hiconNotDone;
		iprog = 10;
		break;

	default:
		break;
	}

	if (pkgi->uCurrentTask < pkgi->uNumTasks)
	{
		SendDlgItemMessage (pkgi->hwndWiz, IDC_TASKICON1 + pkgi->uCurrentTask,
				STM_SETIMAGE, (WPARAM)IMAGE_ICON, (LPARAM)hicon);

		switch (pkgi->uTask[pkgi->uCurrentTask]) {
			case TASK_KEYGEN:			ids = IDS_TASKKEYGEN;			break;
			case TASK_SUBKEYGEN:		ids = IDS_TASKSUBKEYGEN;		break;
			case TASK_EXCHANGELOOKUP:	ids = IDS_TASKEXCHANGELOOKUP;	break;
			case TASK_SENDTOSERVER:		ids = IDS_TASKSENDKEY;			break;
			case TASK_REQUESTCERT:		ids = IDS_TASKCERTREQ;			break;
			case TASK_SENDRECONSTRUCT:	ids = IDS_TASKRECONST;			break;
		}

		LoadString (g_hInst, ids, sz, sizeof(sz));
		SetDlgItemText (pkgi->hwndWiz, IDC_TASKTEXT1 + pkgi->uCurrentTask, sz);

		iprog += pkgi->uCurrentTask * 10;

		if (pkgi->bCancelPending)
			LoadString (g_hInst, IDS_CANCELLING, sz, sizeof(sz));

		SendDlgItemMessage (pkgi->hwndWiz,
				IDC_PROGRESSBAR, PBM_SETPOS, (WPARAM)iprog, 0);
	}
	else
	{
		if (pkgi->bCancelPending)
			LoadString (g_hInst, IDS_CANCELLED, sz, sizeof(sz));
		else
			LoadString (g_hInst, IDS_DONE, sz, sizeof(sz));
	}

	SetDlgItemText (pkgi->hwndWiz, IDC_CURRENTTASK, sz);
}


//	______________________________________________
//
//  setup the task list

static VOID
sInitProgressTaskList (
		HWND			hwnd,
		PKEYGENINFO		pkgi)
{
	CHAR	sz[64];
	INT		idc;

	pkgi->uNumTasks = 1;
	LoadString (g_hInst, IDS_TASKKEYGEN, sz, sizeof(sz));
	SetDlgItemText (hwnd, IDC_TASKTEXT1, sz);
	ShowWindow (GetDlgItem (hwnd, IDC_TASKTEXT1), SW_SHOW);
	pkgi->uTask[0] = TASK_KEYGEN;

	ShowWindow (GetDlgItem (hwnd, IDC_TASKICON1), SW_SHOW);

	if (pkgi->uSubKeyAlg != 0)
	{
		idc = IDC_TASKTEXT1 + pkgi->uNumTasks;
		LoadString (g_hInst, IDS_TASKSUBKEYGEN, sz, sizeof(sz));
		SetDlgItemText (hwnd, idc, sz);
		ShowWindow (GetDlgItem (hwnd, idc), SW_SHOW);
		pkgi->uTask[pkgi->uNumTasks] = TASK_SUBKEYGEN;

		idc = IDC_TASKICON1 + pkgi->uNumTasks;
		ShowWindow (GetDlgItem (hwnd, idc), SW_SHOW);
		pkgi->uNumTasks++;
	}

	if (pkgi->bIsExchangeUser &&
		(pkgi->bAutoAddExchangeUserID ||
		 pkgi->bLookupExchangeUserID))
	{
		idc = IDC_TASKTEXT1 + pkgi->uNumTasks;
		LoadString (g_hInst, IDS_TASKEXCHANGELOOKUP, sz, sizeof(sz));
		SetDlgItemText (hwnd, idc, sz);
		ShowWindow (GetDlgItem (hwnd, idc), SW_SHOW);
		pkgi->uTask[pkgi->uNumTasks] = TASK_EXCHANGELOOKUP;

		idc = IDC_TASKICON1 + pkgi->uNumTasks;
		ShowWindow (GetDlgItem (hwnd, idc), SW_SHOW);
		pkgi->uNumTasks++;
	}

	if (pkgi->bDoSend)
	{
		idc = IDC_TASKTEXT1 + pkgi->uNumTasks;
		LoadString (g_hInst, IDS_TASKSENDKEY, sz, sizeof(sz));
		SetDlgItemText (hwnd, idc, sz);
		ShowWindow (GetDlgItem (hwnd, idc), SW_SHOW);
		pkgi->uTask[pkgi->uNumTasks] = TASK_SENDTOSERVER;

		idc = IDC_TASKICON1 + pkgi->uNumTasks;
		ShowWindow (GetDlgItem (hwnd, idc), SW_SHOW);
		pkgi->uNumTasks++;
	}

	if (pkgi->bDoCertRequest)
	{
		idc = IDC_TASKTEXT1 + pkgi->uNumTasks;
		LoadString (g_hInst, IDS_TASKCERTREQ, sz, sizeof(sz));
		SetDlgItemText (hwnd, idc, sz);
		ShowWindow (GetDlgItem (hwnd, idc), SW_SHOW);
		pkgi->uTask[pkgi->uNumTasks] = TASK_REQUESTCERT;

		idc = IDC_TASKICON1 + pkgi->uNumTasks;
		ShowWindow (GetDlgItem (hwnd, idc), SW_SHOW);
		pkgi->uNumTasks++;
	}

	if (pkgi->bDoReconstruction)
	{
		idc = IDC_TASKTEXT1 + pkgi->uNumTasks;
		LoadString (g_hInst, IDS_TASKRECONST, sz, sizeof(sz));
		SetDlgItemText (hwnd, idc, sz);
		ShowWindow (GetDlgItem (hwnd, idc), SW_SHOW);
		pkgi->uTask[pkgi->uNumTasks] = TASK_SENDRECONSTRUCT;

		idc = IDC_TASKICON1 + pkgi->uNumTasks;
		ShowWindow (GetDlgItem (hwnd, idc), SW_SHOW);
		pkgi->uNumTasks++;
	}

	SendDlgItemMessage (hwnd, IDC_PROGRESSBAR, PBM_SETRANGE, 0,
			MAKELPARAM (0, pkgi->uNumTasks*10));
	SendDlgItemMessage (hwnd, IDC_PROGRESSBAR, PBM_SETPOS, 0, 0);
}


//	______________________________________________
//
//  callback from spaceedit control for reconstruction panel

static VOID
sSpaceEditCallbackRecon (
		HWND	hwnd,
		PVOID	pUserValue)
{
	HWND	hwndParent;
	HWND	hwndGrandParent;
	DWORD	dwKey;

	hwndParent	= GetParent (hwnd);

	dwKey = SEGetSpecialKey (hwnd);
	switch (dwKey) {
	case VK_RETURN :
		hwndGrandParent = GetParent (hwndParent);
		PropSheet_PressButton (hwndGrandParent, PSBTN_NEXT);
		return;

	case VK_ESCAPE :
		hwndGrandParent = GetParent (hwndParent);
		PropSheet_PressButton (hwndGrandParent, PSBTN_CANCEL);
		return;
	}
}


//	______________________________________________
//
//  callback from spaceedit control

static VOID
sSpaceEditCallback (
		HWND	hwnd,
		PVOID	pUserValue)
{
	PKEYGENINFO		pkgi		= (PKEYGENINFO)pUserValue;

	BOOL	bShowCapsWarning;
	UINT	uPassphraseQuality;
	HWND	hwndParent;
	HWND	hwndGrandParent;
	DWORD	dwKey;

	hwndParent	= GetParent (hwnd);

	dwKey = SEGetSpecialKey (hwnd);
	switch (dwKey) {
	case VK_RETURN :
		hwndGrandParent = GetParent (hwndParent);
		PropSheet_PressButton (hwndGrandParent, PSBTN_NEXT);
		return;

	case VK_ESCAPE :
		hwndGrandParent = GetParent (hwndParent);
		PropSheet_PressButton (hwndGrandParent, PSBTN_CANCEL);
		return;
	}

	uPassphraseQuality	= SEGetTextQuality (hwnd);
	bShowCapsWarning	= SEGetShowWarning (hwnd);

	if (bShowCapsWarning)
	{
		ShowWindow (GetDlgItem (hwndParent, IDC_CAPSWARNING), SW_SHOW);
	}
	else
	{
		ShowWindow (GetDlgItem (hwndParent, IDC_CAPSWARNING), SW_HIDE);
	}

	if (hwnd == GetDlgItem (hwndParent, IDC_PASSPHRASE))
	{
		SendDlgItemMessage (hwndParent, IDC_PHRASEQUALITY,
				PBM_SETPOS, uPassphraseQuality, 0);
	}
}


//	______________________________________________
//
//  callback routine called by library key generation routine
//  every so often with status of keygen.  Returning a nonzero
//  value cancels the key generation.

static PGPError
sKeyGenCallback (
		PGPContextRef	context,
		PGPEvent*		event,
		PGPUserValue	userValue)
{
	INT					iReturnCode		= kPGPError_NoErr;
	PKEYGENINFO			pkgi;

	switch (event->type) {
	case kPGPEvent_KeyGenEvent:
		pkgi = (PKEYGENINFO) userValue;
		if (pkgi->bCancelPending)
			iReturnCode = kPGPError_UserAbort;
		break;
	}

	return iReturnCode;
}

/* Find just created container and put it on the token.
   For old iKey driver create key container. */
PGPError PGPclExport
PGPclProcessKeyConts( 
		PGPContextRef ctx, PGPKeyDBObjRef key, 
		const char *pszPassPhrase,
		const char *CSP, const PGPTokenInfo *pti, 
		const char *conts1, int conts1_n, 
		const char *conts2, int conts2_n )  
{
	PGPError err = kPGPError_NoErr;
    char *cont = NULL;

	if( conts2_n == conts1_n + 1 )   
		CLIEDiffKeyContainers( conts1, conts2, &cont );

#if 1
	/* if we didn't find CSP-generated container, 
	then create our own container name. 
	It is worth only for the old driver of iKey, 
	so one day this code should go away. 
	*/
	if( *CSP && cont == NULL && 
		conts2_n == conts1_n && conts1_n == 0 &&
		strstr(pti->manufacturerID, "Rainbow") )  
		CLIEGenerateNewUUID( &cont );
#endif
        
    /* We found new key container, which contains just 
	   generated keys. Tell it to SDK. */
	if( cont )  {
		if( IsntPGPError(err) )  {
			PGPKeySetRef keyset;
			
			err = PGPNewOneKeySet(key, &keyset); 
			
			if( IsntPGPError(err) )  {
				err = PGPExport (ctx,
				   /* The key is not exported, 
				      it is used for the additional info */
				   PGPOExportKeySet (ctx, keyset),
				   PGPOExportFormat (ctx, 
				   kPGPExportFormat_TokenKeyContainer),
				   PGPOOutputToken (ctx,
						-1 /* SDK will determine, based on keyID */),
				   PGPOKeyContainer(ctx, cont, strlen(cont)),
				   PGPOPassphrase (ctx, pszPassPhrase),
				   PGPOLastOption (ctx)
				);
			}
			
			PGPFreeKeySet(keyset);
		}
	}

    free( cont );

	return err;
}


//	______________________________________________
//
//  thread for actually creating key

static VOID
sKeyGenerationThread (
	void	*pArgs)
{
	PKEYGENINFO			pkgi			= (PKEYGENINFO) pArgs;
	BOOL				bNewDefaultKey	= TRUE;
	PGPUInt32			AlgFlags		= 0;
	PGPSize				prefDataSize	= 0;
	PGPKeySetRef		keysetADK		= kInvalidPGPKeySetRef;
	PGPKeySetRef		keysetRevoker	= kInvalidPGPKeySetRef;
	PGPOptionListRef	optionlist		= kInvalidPGPOptionListRef;
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	PGPCipherAlgorithm*	pAlgs			= NULL;
	PGPPrefArray *		prefArray		= NULL;
	PGPUInt32			uKeyFlags		= 0;

	PGPKeyDBObjRef		keyDefault;
	PGPKeyDBObjRef		subkey;
	PGPKeyDBObjRef		useridCorp;
	PGPBoolean			bFastGen;
	PGPContextRef		ctx;
	PGPByte				enforce;
	PGPError			err;
	INT					uTrustLevel;
	PGPUInt32			numAlgs;

    const tokenNum = 0; /*Always token 0 for now*/

    char *conts1 = NULL;
    char *conts2 = NULL;
    PGPUInt32 conts1_n=0;
    PGPUInt32 conts2_n=0;

    PGPTokenInfo pti = { sizeof(PGPTokenInfo) };
    char CSP[80];
	const BOOL bHasSubKey = (pkgi->uSubKeyAlg != 0);

	ctx = pkgi->context;

	sSetThreadExecutionState (ES_SYSTEM_REQUIRED | ES_CONTINUOUS);
	Sleep (100);

	//
	//	setup and perform key generation
	//
	pkgi->uCurrentTask = 0;
	sSetCurrentTaskInfo (pkgi, ICON_ARROW);

	PGPclGetDefaultPrivateKey (pkgi->keydbMain, &keyDefault);
	if (PGPKeyDBObjRefIsValid (keyDefault))
		bNewDefaultKey = FALSE;

	// construct userid
	err = sCreatePGPUserID (&pkgi->pszUserID,
			pkgi->pszFullName, pkgi->pszEmail); CKERR;

	// get client preferences
	err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
	PGPGetPrefBoolean (prefref, kPGPPrefFastKeyGen, &bFastGen);
	PGPGetPrefArray (prefref,
			kPGPPrefAllowedAlgorithmsList, &prefArray);

	numAlgs	= prefArray->numElements;
	pAlgs = (PGPCipherAlgorithm *) PGPNewData (PGPPeekContextMemoryMgr (ctx),
			(sizeof(PGPCipherAlgorithm) * (numAlgs+2)), 
			kPGPMemoryMgrFlags_Clear);

	if (IsntNull(pAlgs))
	{
		PGPUInt32	uOut		= 0;
		PGPUInt32	uIn;

		for (uIn=0; uIn<numAlgs; uIn++)
		{
			pAlgs[uOut] = (PGPCipherAlgorithm) prefArray->elements[uIn].data;

			// if AES(256) is in the list, make sure the other AES 
			// variants are too
			if (pAlgs[uOut++] == kPGPCipherAlgorithm_AES256)
			{
				pAlgs[uOut++] = kPGPCipherAlgorithm_AES192;
				pAlgs[uOut++] = kPGPCipherAlgorithm_AES128;
			}
		}

		numAlgs = uOut;
	}
	else
	{
		numAlgs = 0;
		err = kPGPError_OutOfMemory;
		CKERR;
	}

	// build list of common options
	err = PGPBuildOptionList (ctx, &optionlist,
			PGPOKeyDBRef (ctx, pkgi->keydbMain),
			PGPOKeyGenParams (ctx, pkgi->uKeyAlg, pkgi->uKeySize),
			PGPOKeyGenFast (ctx, bFastGen),
			PGPOKeyGenName (ctx, pkgi->pszUserID, lstrlen (pkgi->pszUserID)),
			PGPOPassphrase (ctx, pkgi->pszPassPhrase),
			PGPOExpiration (ctx, pkgi->uExpireDays),
			PGPOPreferredAlgorithms (ctx, pAlgs, numAlgs),
			PGPOEventHandler (ctx, sKeyGenCallback, pkgi),
			PGPOLastOption (ctx)); CKERR;

	// add key flags
	if (pkgi->uKeyFlags != 0)
	{
		err = PGPAppendOptionList (optionlist,
				PGPOKeyFlags (ctx, pkgi->uKeyFlags),
				PGPOLastOption (ctx)); CKERR;
	}

	// add ADK option
	if ((PGPKeyDBObjRefIsValid (pkgi->keyADK)) &&
		(pkgi->uKeyType != kPGPPublicKeyAlgorithm_RSA))
	{
		if (pkgi->bEnforceADK)
			enforce = 0x80;
		else
			enforce = 0x00;
		err = PGPNewOneKeySet (pkgi->keyADK, &keysetADK); CKERR;
		err = PGPAppendOptionList (optionlist,
				PGPOAdditionalRecipientRequestKeySet (ctx, keysetADK, enforce),
				PGPOLastOption (ctx)); CKERR;
	}

	// add revoker key option
	if ((PGPKeyDBObjRefIsValid (pkgi->keyRevoker)) &&
		(pkgi->uKeyType != kPGPPublicKeyAlgorithm_RSA))
	{
		err = PGPNewOneKeySet (pkgi->keyRevoker, &keysetRevoker); CKERR;
		err = PGPAppendOptionList (optionlist,
				PGPORevocationKeySet (ctx, keysetRevoker),
				PGPOLastOption (ctx)); CKERR;
	}

	// verify token capabilities and add token option
	if (pkgi->bOnToken)
	{
		UINT	uNumTokens;

		PGPCountTokens (g_context, &uNumTokens);

		if (uNumTokens == 0)
		{
			err = kPGPError_Win32_TokenNotInserted;
			goto done;
		}
		else
		{
			err = PGPGetTokenInfo (g_context, tokenNum, &pti);
			if (IsPGPError (err))
			{
				err = kPGPError_Win32_TokenNotInserted;
				goto done;
			}

			if (pkgi->uKeyAlg != kPGPPublicKeyAlgorithm_RSA)
			{
				err = kPGPError_Win32_TokenParameterError;
				goto done;
			}

			if ((pkgi->uKeySize < pti.minRsaKeySize) ||
				(pkgi->uKeySize > pti.maxRsaKeySize))
			{
				err = kPGPError_Win32_TokenParameterError;
				goto done;
			}
		}

		err = PGPAppendOptionList (optionlist,
				PGPOKeyGenOnToken (ctx, tokenNum),
				PGPOLastOption (ctx)); CKERR;

		PGPclIETokenProvToCSP (pti.manufacturerID, CSP, sizeof(CSP));

		if (pti.numPrivKeys > 0)  
		{
			/* This is an optimization for all vendors and a 
			requirement for some vendors, who may crash. 
			If token has no private keys, then it has no key 
			containers. */
			PGPclIEGetExistingKeyContainersRemote (CSP, &conts1, &conts1_n);
		}
	}

	// generate key using specified options
	err = PGPGenerateKey (ctx,
			&(pkgi->keyNew), optionlist, PGPOLastOption (ctx));

	// note: PGPGenerateKey returns kPGPError_OutOfMemory when user aborts!
	if (err == kPGPError_OutOfMemory)
		err = kPGPError_UserAbort;

	if (IsPGPError (err) || pkgi->bCancelPending)
		sSetCurrentTaskInfo (pkgi, ICON_ERROR);
	else
		sSetCurrentTaskInfo (pkgi, ICON_CHECK);

    if (IsntPGPError(err) && pkgi->bOnToken)  
		PGPclIEGetExistingKeyContainersRemote (CSP, &conts2, &conts2_n);
    
	//
	//	setup and perform subkey generation
	//
	pkgi->uCurrentTask++;
	if (IsntPGPError (err) && !pkgi->bCancelPending)
	{
		sSetCurrentTaskInfo (pkgi, ICON_ARROW);

		if (bHasSubKey)
		{
			if (PGPOptionListRefIsValid (optionlist))
				PGPFreeOptionList (optionlist);

			// build list of common options
			err = PGPBuildOptionList (ctx, &optionlist,
				PGPOKeyGenMasterKey (ctx, pkgi->keyNew),
				PGPOKeyGenParams (ctx, pkgi->uSubKeyAlg, pkgi->uSubKeySize),
				PGPOKeyGenFast (ctx, bFastGen),
				PGPOPassphrase (ctx, pkgi->pszPassPhrase),
				PGPOExpiration (ctx, pkgi->uExpireDays),
				PGPOEventHandler (ctx, sKeyGenCallback, pkgi),
				PGPOLastOption (ctx)); CKERR;

			// add subkey flags
			if (pkgi->uSubKeyFlags != 0)
			{
				err = PGPAppendOptionList (optionlist,
						PGPOKeyFlags (ctx, pkgi->uSubKeyFlags),
						PGPOLastOption (ctx)); CKERR;
			}

			// add token option
			if (pkgi->bOnToken)
			{
				UINT	uNumTokens;

				PGPCountTokens (g_context, &uNumTokens);

				if (uNumTokens == 0)
					err = kPGPError_Win32_TokenNotInserted;
				else
				{
					err = PGPAppendOptionList (optionlist,
							PGPOKeyGenOnToken (ctx, 0),
							PGPOLastOption (ctx));
				}
				CKERR;
    		}

			err = PGPGenerateSubKey (
				ctx, &subkey, optionlist,
				PGPOLastOption (ctx));

			// note: PGPGenerateSubKey returns kPGPError_OutOfMemory
			// when user aborts!
			if (err == kPGPError_OutOfMemory)
				err = kPGPError_UserAbort;

		}
	}

	if (IsntPGPError(err) && pkgi->bOnToken)
	{
		err = PGPclProcessKeyConts (ctx, pkgi->keyNew, 
			pkgi->pszPassPhrase, 
			CSP, &pti, conts1, conts1_n, conts2, conts2_n);
	}

    PGPclIEFreeMem (conts1);
    PGPclIEFreeMem (conts2);

	if (IsPGPError (err) || pkgi->bCancelPending)
		sSetCurrentTaskInfo (pkgi, ICON_ERROR);
	else
	{
		sSetCurrentTaskInfo (pkgi, ICON_CHECK);
		pkgi->iFinalResult = kPGPclKeyCreate_NewKey;
		if (bNewDefaultKey)
			pkgi->iFinalResult |= kPGPclKeyCreate_NewDefaultKey;
		if (pkgi->bOnToken)
			pkgi->iFinalResult |= kPGPclKeyCreate_OnToken;
	}

	//
	// add MS Exchange userid
	//
	if (IsntPGPError (err) && !pkgi->bCancelPending)
	{
		if (pkgi->bIsExchangeUser &&
			(pkgi->bAutoAddExchangeUserID ||
			 pkgi->bLookupExchangeUserID))
		{
			pkgi->uCurrentTask++;
			sSetCurrentTaskInfo (pkgi, ICON_ARROW);
			pkgi->bTaskError = FALSE;

			sLookupExchangeUserID (pkgi->hwndWiz, pkgi);

			if (pkgi->pszExchangeUserID)
			{
				// make sure we have enough entropy
				PGPclRandom (ctx, pkgi->hwndWiz, 0);

				err = PGPAddUserID (pkgi->keyNew, pkgi->pszExchangeUserID,
						PGPOPassphrase (ctx, pkgi->pszPassPhrase),
						PGPOLastOption (ctx));
			}
			
			if (IsNull (pkgi->pszExchangeUserID) ||
				IsPGPError (err))
			{
				pkgi->bTaskError = TRUE;
				sSetCurrentTaskInfo (pkgi, ICON_ERROR);
			}
			else
				sSetCurrentTaskInfo (pkgi, ICON_CHECK);
		}
	}

	//
	// add Lotus Notes userid
	//
	if (IsntPGPError (err) && !pkgi->bCancelPending)
	{
		if (pkgi->bAddNotesUserID &&
			pkgi->pszNotesFullName &&
			pkgi->pszNotesEmail)
		{
			if (pkgi->pszUserID)
				clFree (pkgi->pszUserID);

			// construct userid
			err = sCreatePGPUserID (&pkgi->pszUserID,
					pkgi->pszNotesFullName, pkgi->pszNotesEmail);

			if (IsntPGPError (err))
			{
				// make sure we have enough entropy
				PGPclRandom (ctx, pkgi->hwndWiz, 0);

				err = PGPAddUserID (pkgi->keyNew, pkgi->pszUserID,
						PGPOPassphrase (ctx, pkgi->pszPassPhrase),
						PGPOLastOption (ctx));
			}
		}
	}

	//
	// sign and trust corporate key
	//
	if (IsntPGPError (err) && !pkgi->bCancelPending)
	{
		if (PGPKeyDBObjRefIsValid (pkgi->keyCorp))
		{
			err = PGPGetPrimaryUserID (pkgi->keyCorp, &useridCorp);
			if (IsntPGPError (err))
			{
				if (pkgi->bMetaCorpKey)
					uTrustLevel = pkgi->uMetaTrustLevel;
				else
					uTrustLevel = 0;

				// make sure we have enough entropy
				PGPclRandom (ctx, pkgi->hwndWiz, 0);

				err = PGPCertifyUserID (useridCorp, pkgi->keyNew,
						PGPOPassphrase (ctx, pkgi->pszPassPhrase),
						PGPOExpiration (ctx, 0),
						PGPOExportable (ctx, FALSE),
						PGPOSigTrust (ctx, uTrustLevel, kPGPKeyTrust_Complete),
						PGPOLastOption (ctx));

				if (IsntPGPError (err))
				{
					err = PGPSetKeyTrust (
							pkgi->keyCorp, kPGPKeyTrust_Complete);

					// ignore errors here.  If key is axiomatic,
					// setting trust will cause an error.
					err = kPGPError_NoErr;
				}
			}
		}
	}

	//
	// sign root CA key
	//
	if (IsntPGPError (err) && !pkgi->bCancelPending)
	{
		if (PGPKeyDBObjRefIsValid (pkgi->keyRootCA))
		{
			err = PGPGetPrimaryUserID (pkgi->keyRootCA, &useridCorp);
			if (IsntPGPError (err))
			{
				uTrustLevel = 0;

				// make sure we have enough entropy
				PGPclRandom (ctx, pkgi->hwndWiz, 0);

				err = PGPCertifyUserID (useridCorp, pkgi->keyNew,
						PGPOPassphrase (ctx, pkgi->pszPassPhrase),
						PGPOExpiration (ctx, 0),
						PGPOExportable (ctx, FALSE),
						PGPOSigTrust (ctx, uTrustLevel, kPGPKeyTrust_Complete),
						PGPOLastOption (ctx));
			}
		}
	}

	//
	//	send key to server
	//
	if (pkgi->bDoSend)
	{
		PGPKeySetRef	keysetSend	= kInvalidPGPKeySetRef;

		pkgi->uCurrentTask++;
		sSetCurrentTaskInfo (pkgi, ICON_ARROW);
		pkgi->bTaskError = TRUE;

		if (IsntPGPError (err) && !pkgi->bCancelPending)
		{
			err = PGPNewOneKeySet (pkgi->keyNew, &keysetSend);
			if (IsntPGPError (err))
			{
				pkgi->bTaskComplete = FALSE;
				pkgi->bTaskError = FALSE;

				err = PGPclSendKeysToRootServerNotify (
						pkgi->context, pkgi->tlscontext,
						pkgi->hwndWiz, pkgi->keydbMain, keysetSend);

				if (IsntPGPError (PGPclErrorBox (pkgi->hwndWiz, err)))
				{
					while (!pkgi->bTaskComplete)
						Sleep (200);
				}
				else
					pkgi->bTaskError = TRUE;

				PGPFreeKeySet (keysetSend);
			}

			err = kPGPError_NoErr;
		}

		if (pkgi->bTaskError)
		{
			sSetCurrentTaskInfo (pkgi, ICON_ERROR);
			pkgi->bDoSend = FALSE;
		}
		else
			sSetCurrentTaskInfo (pkgi, ICON_CHECK);
	}

	//
	//	setup and send certificate request
	//
	if (pkgi->bDoCertRequest)
	{
		PGPKeySetRef	keysetSend	= kInvalidPGPKeySetRef;

		pkgi->uCurrentTask++;
		sSetCurrentTaskInfo (pkgi, ICON_ARROW);
		pkgi->bTaskError = TRUE;

		if (IsntPGPError (err) && !pkgi->bCancelPending)
		{
			err = PGPNewOneKeySet (pkgi->keyNew, &keysetSend);
			if (IsntPGPError (err))
			{
				pkgi->bTaskComplete = FALSE;
				pkgi->bTaskError = FALSE;

				err = PGPclSendCertificateRequestToServerNotify (
						pkgi->context, pkgi->tlscontext, pkgi->hwndWiz,
						pkgi->keydbMain, kInvalidPGPKeyDBObjRef,
						keysetSend, pkgi->pszPassPhrase);

				if (IsntPGPError (PGPclErrorBox (pkgi->hwndWiz, err)))
				{
					while (!pkgi->bTaskComplete)
						Sleep (200);
				}
				else
					pkgi->bTaskError = TRUE;

				PGPFreeKeySet (keysetSend);
			}

			err = kPGPError_NoErr;
		}

		if (pkgi->bTaskError)
		{
			sSetCurrentTaskInfo (pkgi, ICON_ERROR);
			pkgi->bDoCertRequest = FALSE;
		}
		else
			sSetCurrentTaskInfo (pkgi, ICON_CHECK);
	}

	//
	//	setup and send reconstruction blob
	//
	if (pkgi->bDoReconstruction)
	{
		pkgi->uCurrentTask++;
		sSetCurrentTaskInfo (pkgi, ICON_ARROW);
		pkgi->bTaskError = TRUE;

		if (IsntPGPError (err) && !pkgi->bCancelPending)
		{
			pkgi->bTaskComplete = FALSE;
			pkgi->bTaskError = FALSE;

			err = PGPclSendReconstructionBlobToServerNotify (
					pkgi->context, pkgi->tlscontext, pkgi->hwndWiz,
					pkgi->keyNew, pkgi->keydbMain, pkgi->reconPrompts, 
					pkgi->reconPasses, pkgi->pszPassPhrase);

			if (IsntPGPError (PGPclErrorBox (pkgi->hwndWiz, err)))
			{
				while (!pkgi->bTaskComplete)
					Sleep (200);
			}
			else
				pkgi->bTaskError = TRUE;

			err = kPGPError_NoErr;
		}

		if (pkgi->bTaskError)
		{
			sSetCurrentTaskInfo (pkgi, ICON_ERROR);
			pkgi->bDoReconstruction = FALSE;
		}
		else
			sSetCurrentTaskInfo (pkgi, ICON_CHECK);
	}

done :
	pkgi->uCurrentTask = pkgi->uNumTasks;
	sSetCurrentTaskInfo (pkgi, ICON_NONE);

	if (IsntNull (pAlgs))
		PGPFreeData(pAlgs);
	if (IsntNull(prefArray))
		PGPFreePrefArray(prefArray);
	if (PGPKeySetRefIsValid (keysetADK))
		PGPFreeKeySet (keysetADK);
	if (PGPKeySetRefIsValid (keysetRevoker))
		PGPFreeKeySet (keysetRevoker);
	if (PGPOptionListRefIsValid (optionlist))
		PGPFreeOptionList (optionlist);

	if (pkgi->bCancelPending)
	{
		pkgi->bCancelled = TRUE;
		pkgi->bCancelPending = FALSE;
	}
	PGPclErrorBox (pkgi->hwndWiz, err);
	PostMessage (pkgi->hwndWiz, CL_M_GENERATION_COMPLETE, 0, 0);

	sSetThreadExecutionState (ES_CONTINUOUS);
}

//	______________________________________________
//
//  Dialog procedure for "Finish" dialog

static LRESULT WINAPI
sKeyWizardDoneDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PKEYGENINFO		pkgi;
	BOOL			bReturnCode = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PKEYGENINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontIntro, (LPARAM)FALSE);
		return TRUE;
	}

	case WM_PAINT :
		pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

		if (pkgi->hpaletteDone)
		{
			PAINTSTRUCT ps;
			HDC			hDC = BeginPaint (hwnd, &ps);

			SelectPalette (hDC, pkgi->hpaletteDone, FALSE);
			RealizePalette (hDC);
			EndPaint (hwnd, &ps);
		}
		break;

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch (pnmh->code) {
		case PSN_SETACTIVE:
			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

			PostMessage (GetParent (hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_BACK|PSWIZB_FINISH);

			SendDlgItemMessage (hwnd, IDC_WIZBITMAP, STM_SETIMAGE,
					IMAGE_BITMAP, (LPARAM) pkgi->hbitmapDone);

			if (pkgi->bDoSend ||
				pkgi->bDoCertRequest ||
				pkgi->bDoReconstruction ||
				pkgi->bOnToken)
			{
				EnableWindow (
						GetDlgItem (GetParent (hwnd), IDCANCEL), FALSE);
			}

			if (!pkgi->bDoSend)
			{
				CHAR szText[256];
				LoadString (g_hInst,
						IDS_SENDLATER, szText, sizeof(szText));
				SetDlgItemText(hwnd, IDC_KSTEXT, szText);
			}

			if (pkgi->bDoCertRequest)
			{
				ShowWindow (GetDlgItem (hwnd, IDC_X509TEXT), SW_SHOW);
			}

			SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;

		case PSN_WIZFINISH:
			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

			pkgi->bFinishSelected = TRUE;
			SetWindowLong(hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for actually generating the key(s)

static LRESULT WINAPI
sKeyWizardProgressDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PKEYGENINFO		pkgi;
	BOOL			bReturnCode = FALSE;
	DWORD			dwThreadID;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PKEYGENINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontInterior, (LPARAM)FALSE);

		// setup list of tasks
		sInitProgressTaskList (hwnd, pkgi);

		pkgi->uCurrentTask = 0;
		pkgi->hwndWiz = hwnd;
		pkgi->bInGeneration = TRUE;
		pkgi->bTaskComplete = FALSE;

		// initialize Intel RNG icon and text
		if (!PGPGlobalRandomPoolHasIntelRNG ())
		{
			ShowWindow (GetDlgItem (hwnd, IDC_INTELICON), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_INTELTEXT), SW_HIDE);
		}

		// Kick off generation thread here
		_beginthreadex (NULL, 0,
				(LPTHREAD_START_ROUTINE)sKeyGenerationThread,
				(void *) pkgi, 0, &dwThreadID);

		return TRUE;
	}

	case WM_PAINT :
		pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);
		sDrawSendStatus (GetDlgItem (hwnd, IDC_PROGRESS), pkgi);
		break;

	case WM_TIMER :
		if (wParam == PROGRESS_TIMER)
		{
			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

			pkgi->iStatusValue += pkgi->iStatusDirection;
			if (pkgi->iStatusValue <= 0)
			{
				pkgi->iStatusValue = 0;
				pkgi->iStatusDirection = 1;
			}
			else if (pkgi->iStatusValue >= NUMLEDS-1)
			{
				pkgi->iStatusValue = NUMLEDS-1;
				pkgi->iStatusDirection = -1;
			}
			sInvalidateLEDs (hwnd, IDC_PROGRESS);
		}
		break;

	case CL_M_GENERATION_COMPLETE:
	{
		BOOL	bDisableCancel		= FALSE;
		CHAR	sz[128];

		pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

		KillTimer (hwnd, PROGRESS_TIMER);
		pkgi->iStatusValue = -1;
		pkgi->iStatusDirection = 1;
		ShowWindow (GetDlgItem (hwnd, IDC_PROGRESS), SW_HIDE);
		InvalidateRect (hwnd, NULL, FALSE);

		if (pkgi->bCancelPending)
			LoadString (g_hInst, IDS_CANCELLING, sz, sizeof(sz));
		else
		{
			if (pkgi->bCancelled)
			{
				LoadString (g_hInst,
						IDS_CANCELLED, sz, sizeof(sz));
			}
			else if (pkgi->iFinalResult)
			{
				LoadString (g_hInst, IDS_DONE, sz, sizeof(sz));
				SendMessage (GetParent(hwnd),
						PSM_SETWIZBUTTONS, 0, PSWIZB_NEXT);
			}
			else
			{
				LoadString (g_hInst,
						IDS_KEYGENUNABLETOCOMPLETE, sz, sizeof(sz));
			}
		}
		SetDlgItemText (hwnd, IDC_CURRENTTASK, sz);
		pkgi->bInGeneration = FALSE;

		if (pkgi->iFinalResult)
		{
			bDisableCancel = pkgi->bDoSend || pkgi->bDoCertRequest ||
					pkgi->bDoReconstruction || pkgi->bOnToken;
		}
		EnableWindow (GetDlgItem (GetParent (hwnd), IDCANCEL), !bDisableCancel);

		bReturnCode = TRUE;
		break;
	}

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

		if (pnmh->hwndFrom == GetParent (hwnd))
		{
			switch (pnmh->code) {
			case PSN_SETACTIVE:
				if (pkgi->iFinalResult)
				{
					SendMessage (GetParent(hwnd),
							PSM_SETWIZBUTTONS, 0, PSWIZB_NEXT);
				}
				else
				{
					SendMessage (GetParent(hwnd), PSM_SETWIZBUTTONS, 0, 0);

					// start the LEDs moving
					pkgi->iStatusValue = -1;
					pkgi->iStatusDirection = 1;
					SetTimer (hwnd, PROGRESS_TIMER, LEDTIMERPERIOD, NULL);
				}

				if (pkgi->bOnToken)
				{
					EnableWindow (GetDlgItem (GetParent (hwnd),
							IDCANCEL), FALSE);
				}

				SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
				bReturnCode = TRUE;
				break;

			case PSN_KILLACTIVE:
				KillTimer (hwnd, PROGRESS_TIMER);
				break;

			case PSN_QUERYCANCEL:
			{
				//If we're generating a key, don't let the user press
				//cancel without asking.  If he says, "yes, I want to cancel,"
				//then we'll still reject the message, but set CancelPending
				//to TRUE, so that the next time the library comes around, we
				//can nuke the thread.  The thread will then clear the
				//InGeneration flag and re-send us the cancel message.

				UINT	uResponse	= IDYES;

				if (pkgi->bInGeneration)
				{
					if (!pkgi->bCancelPending)
					{
						uResponse = PGPclMessageBox (hwnd, IDS_KEYGENTITLE,
								IDS_KEYGENCONFIRMCANCEL,
								MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2);
						if (uResponse == IDYES)
						{
							if (pkgi->bInGeneration)
							{
								pkgi->bCancelPending = TRUE;
								sSetCurrentTaskInfo (pkgi, ICON_ERROR);
								EnableWindow (GetDlgItem (GetParent (hwnd),
										IDCANCEL), FALSE);
							}
						}
					}
				}
				if (pkgi->bInGeneration || (uResponse == IDNO))
				{
					SetWindowLong (hwnd, DWL_MSGRESULT, 1L);
					bReturnCode = TRUE;
				}
				else
				{
					SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
					bReturnCode = TRUE;
				}
				break;
			}
			}
		}

		// notification must be from PGPclientLib server code
		else
		{
			switch (pnmh->code) {
			case kPGPclNotifyServerProgress :
			{
				PPGPclServerEvent pEvent = (PPGPclServerEvent)lParam;

				SetDlgItemText (hwnd, IDC_CURRENTTASK, pEvent->szmessage);
				pEvent->cancel = pkgi->bCancelPending;

				return FALSE;
			}

			case kPGPclNotifyServerError :
			{
				PPGPclServerEvent	pEvent = (PPGPclServerEvent)lParam;

				pkgi->bTaskError = TRUE;

				if (pkgi->bCancelPending)
					pEvent->cancel = TRUE;
				else
				{
					CHAR	sz[64];

					KillTimer (hwnd, PROGRESS_TIMER);

					LoadString (g_hInst, IDS_CAPTIONALERT, sz, sizeof(sz));
					MessageBox (hwnd,
							pEvent->szmessage, sz, MB_OK|MB_ICONEXCLAMATION);

					SetTimer (hwnd, PROGRESS_TIMER, LEDTIMERPERIOD, NULL);
				}

				SetDlgItemText (hwnd, IDC_CURRENTTASK, pEvent->szmessage);

				pkgi->bTaskComplete = TRUE;
				return FALSE;
			}

			case kPGPclNotifyServerAbort :
				pkgi->bTaskError = TRUE;
			case kPGPclNotifyServerDone :
			{
				PPGPclServerEvent	pEvent = (PPGPclServerEvent)lParam;

				SetDlgItemText (hwnd, IDC_CURRENTTASK, pEvent->szmessage);
				pEvent->cancel = pkgi->bCancelPending;

				pkgi->bTaskComplete = TRUE;
				return FALSE;
			}
			}
		}

		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for getting entropy
//  from user

static LRESULT WINAPI
sKeyWizardRandobitsDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PKEYGENINFO		pkgi;
	BOOL			bReturnCode = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PKEYGENINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontInterior, (LPARAM)FALSE);
		return TRUE;
	}

	case WM_MOUSEMOVE:
	{
		INT		iPercentComplete = 0;
		FLOAT	fTotal, fSoFar;

		pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

		if (pkgi->lRandomBitsNeeded)
		{
			fSoFar = (float)(PGPContextReserveRandomBytes (pkgi->context,
					(pkgi->lRandomBitsNeeded + 7) / 8) * 8);
			fTotal = (float) pkgi->lRandomBitsNeeded;
			fSoFar -= (float) pkgi->lOriginalEntropy;
			fTotal -= (float) pkgi->lOriginalEntropy;
			iPercentComplete = (INT) ((fSoFar / fTotal) * 100.0);

			if (fSoFar >= fTotal)
			{
				pkgi->lRandomBitsNeeded = 0;
				iPercentComplete = 100;
				s_hwndCollectEntropy = NULL;
				SendMessage (GetParent(hwnd),
						PSM_SETWIZBUTTONS, 0, PSWIZB_NEXT|PSWIZB_BACK);
			}

			SendDlgItemMessage (hwnd,
					IDC_PROGRESS, PBM_SETPOS, iPercentComplete, 0);
		}
		break;
	}

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch(pnmh->code) {
		case PSN_SETACTIVE:
			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);
			PostMessage(GetParent(hwnd), PSM_SETWIZBUTTONS, 0, 0);

			{
				PGPOptionListRef	optionlist		= kInvalidPGPOptionListRef;
				PGPPrefRef			prefref			= kInvalidPGPPrefRef;
				PGPContextRef		ctx				= pkgi->context;

				LONG				lTotalBitsNeeded, lRandPoolSize;
				PGPBoolean			bFastGen		= FALSE;

				s_hwndCollectEntropy = hwnd;

				// Check to see if there are random bits needed
				if (IsntPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
				{
					PGPGetPrefBoolean (prefref,
							kPGPPrefFastKeyGen, &bFastGen);
				}

				// create optionlist of common options
				PGPBuildOptionList (ctx, &optionlist,
						PGPOKeyGenParams (ctx,
								pkgi->uKeyAlg, pkgi->uKeySize),
						PGPOKeyGenFast (ctx, bFastGen),
						PGPOLastOption (ctx));

				// add key flags
				if (pkgi->uKeyFlags != 0)
				{
					PGPAppendOptionList (optionlist,
							PGPOKeyFlags (ctx, pkgi->uKeyFlags),
							PGPOLastOption (ctx));
				}

				lTotalBitsNeeded = PGPGetKeyEntropyNeeded (ctx,
						optionlist, PGPOLastOption (ctx));
				PGPFreeOptionList (optionlist);

				if (pkgi->uSubKeyAlg != 0)
				{
					// create optionlist of common options
					PGPBuildOptionList (ctx, &optionlist,
							PGPOKeyGenParams (ctx,
									pkgi->uSubKeyAlg, pkgi->uSubKeySize),
							PGPOKeyGenFast (ctx, bFastGen),
							PGPOLastOption (ctx));

					// add subkey flags
					if (pkgi->uSubKeyFlags != 0)
					{
						PGPAppendOptionList (optionlist,
								PGPOKeyFlags (ctx, pkgi->uSubKeyFlags),
								PGPOLastOption (ctx));
					}

					lTotalBitsNeeded += PGPGetKeyEntropyNeeded (ctx,
							optionlist, PGPOLastOption (ctx));
					PGPFreeOptionList (optionlist);
				}

				lRandPoolSize = PGPGlobalRandomPoolGetSize ();
				pgpAssert (lRandPoolSize > lTotalBitsNeeded);
				pkgi->lRandomBitsNeeded = lTotalBitsNeeded;
			}

			PGPContextReserveRandomBytes (pkgi->context, 
					(pkgi->lRandomBitsNeeded + 7) / 8);

			if ((pkgi->lRandomBitsNeeded -
					8*(LONG)PGPContextReserveRandomBytes(pkgi->context, 0)) > 0)
			{
				pkgi->lOriginalEntropy = 
						8*PGPContextReserveRandomBytes(pkgi->context, 0);

				SendDlgItemMessage (hwnd, IDC_PROGRESS,
						PBM_SETRANGE, 0, MAKELPARAM(0, 100));
				SendDlgItemMessage (hwnd, IDC_PROGRESS,
						PBM_SETPOS, 0, 0);
				SetWindowLong(hwnd, DWL_MSGRESULT, 0L);
				bReturnCode = TRUE;

				if (!s_bHooksInstalled)
					sInstallEntropyHooks ();
			}
			else
			{
				SetWindowLong (hwnd, DWL_MSGRESULT, -1L); // skip page
				bReturnCode = TRUE;
				s_hwndCollectEntropy = NULL;
			}
			break;

		case PSN_KILLACTIVE:
			if (!s_bHooksInstalled)
				sUninstallEntropyHooks ();

			SetWindowLong (hwnd, DWL_MSGRESULT, 0);
			bReturnCode = TRUE;
			s_hwndCollectEntropy = NULL;
			break;
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for performing key reconstruction

static LRESULT WINAPI
sKeyWizardReconDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	BOOL				bReturnCode = FALSE;
	PKEYGENINFO			pkgi;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		CHAR	sz[kPGPRecon_MaxPromptLen];

		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PKEYGENINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontInterior, (LPARAM)FALSE);

		// use the spaceedit control
		SEInit (pkgi->context, GetDlgItem (hwnd, IDC_ANSWER1),
				TRUE, sSpaceEditCallbackRecon, NULL);
		SEInit (pkgi->context, GetDlgItem (hwnd, IDC_ANSWER2),
				TRUE, sSpaceEditCallbackRecon, NULL);
		SEInit (pkgi->context, GetDlgItem (hwnd, IDC_ANSWER3),
				TRUE, sSpaceEditCallbackRecon, NULL);
		SEInit (pkgi->context, GetDlgItem (hwnd, IDC_ANSWER4),
				TRUE, sSpaceEditCallbackRecon, NULL);
		SEInit (pkgi->context, GetDlgItem (hwnd, IDC_ANSWER5),
				TRUE, sSpaceEditCallbackRecon, NULL);

		// and the UTF8Edit control
		UTF8EditInit (GetDlgItem (hwnd, IDC_PROMPT1));
		UTF8EditInit (GetDlgItem (hwnd, IDC_PROMPT2));
		UTF8EditInit (GetDlgItem (hwnd, IDC_PROMPT3));
		UTF8EditInit (GetDlgItem (hwnd, IDC_PROMPT4));
		UTF8EditInit (GetDlgItem (hwnd, IDC_PROMPT5));

		// initialize questions to defaults
		LoadString (g_hInst, IDS_DEFAULTRECONPROMPT1, sz, sizeof(sz));
		UTF8EditSetText (GetDlgItem (hwnd, IDC_PROMPT1), sz);
		LoadString (g_hInst, IDS_DEFAULTRECONPROMPT2, sz, sizeof(sz));
		UTF8EditSetText (GetDlgItem (hwnd, IDC_PROMPT2), sz);
		LoadString (g_hInst, IDS_DEFAULTRECONPROMPT3, sz, sizeof(sz));
		UTF8EditSetText (GetDlgItem (hwnd, IDC_PROMPT3), sz);
		LoadString (g_hInst, IDS_DEFAULTRECONPROMPT4, sz, sizeof(sz));
		UTF8EditSetText (GetDlgItem (hwnd, IDC_PROMPT4), sz);
		LoadString (g_hInst, IDS_DEFAULTRECONPROMPT5, sz, sizeof(sz));
		UTF8EditSetText (GetDlgItem (hwnd, IDC_PROMPT5), sz);

		return TRUE;
	}

	case WM_DESTROY:
		SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER1));
		SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER2));
		SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER3));
		SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER4));
		SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER5));

		SEDestroy (GetDlgItem (hwnd, IDC_ANSWER1));
		SEDestroy (GetDlgItem (hwnd, IDC_ANSWER2));
		SEDestroy (GetDlgItem (hwnd, IDC_ANSWER3));
		SEDestroy (GetDlgItem (hwnd, IDC_ANSWER4));
		SEDestroy (GetDlgItem (hwnd, IDC_ANSWER5));

		UTF8EditDestroy (GetDlgItem (hwnd, IDC_PROMPT1));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_PROMPT2));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_PROMPT3));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_PROMPT4));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_PROMPT5));
		break;

	case WM_COMMAND:
		switch(LOWORD(wParam)) {
		case IDC_HIDETYPING:
		{
			BOOL	bHideTyping;

			if (IsDlgButtonChecked (hwnd, IDC_HIDETYPING) == BST_CHECKED)
				bHideTyping = TRUE;
			else
				bHideTyping = FALSE;

			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_ANSWER1), bHideTyping);
			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_ANSWER2), bHideTyping);
			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_ANSWER3), bHideTyping);
			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_ANSWER4), bHideTyping);
			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_ANSWER5), bHideTyping);
			break;
		}

		case IDC_MOREINFO :
			PGPclHtmlHelp (hwnd, kPGPclHelpMoreInformation, 0, 0,
					(char*)kPGPclHelpFile, (DWORD*)IDH_PGPCLWIZ_RECONMOREINFO);
			break;
		}
		break;

	case WM_NOTIFY:
	{
		LPNMHDR		pnmh = (LPNMHDR) lParam;

		pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

		if (pnmh->hwndFrom == GetParent (hwnd))
		{
			switch (pnmh->code) {
			case PSN_SETACTIVE:
				if (!pkgi->bDoReconstruction)
				{
					SetWindowLong(hwnd, DWL_MSGRESULT, -1L); // skip
					bReturnCode = TRUE;
				}
				else
				{
					CheckDlgButton (hwnd, IDC_HIDETYPING, BST_CHECKED);
					SEChangeHideTyping (
							GetDlgItem (hwnd, IDC_ANSWER1), TRUE);
					SEChangeHideTyping (
							GetDlgItem (hwnd, IDC_ANSWER2), TRUE);
					SEChangeHideTyping (
							GetDlgItem (hwnd, IDC_ANSWER3), TRUE);
					SEChangeHideTyping (
							GetDlgItem (hwnd, IDC_ANSWER4), TRUE);
					SEChangeHideTyping (
							GetDlgItem (hwnd, IDC_ANSWER5), TRUE);

					SendMessage (GetParent(hwnd),
							PSM_SETWIZBUTTONS, 0, PSWIZB_NEXT|PSWIZB_BACK);
					bReturnCode = TRUE;
				}
				break;

			case PSN_WIZNEXT:
			{
				BOOL	bContinue = TRUE;
				INT		i;

				// first collect prompts
				UTF8EditGetText (GetDlgItem (hwnd, IDC_PROMPT1),
						&pkgi->reconPrompts[0][0], kPGPRecon_MaxPromptLen);
				UTF8EditGetText (GetDlgItem (hwnd, IDC_PROMPT2),
						&pkgi->reconPrompts[1][0], kPGPRecon_MaxPromptLen);
				UTF8EditGetText (GetDlgItem (hwnd, IDC_PROMPT3),
						&pkgi->reconPrompts[2][0], kPGPRecon_MaxPromptLen);
				UTF8EditGetText (GetDlgItem (hwnd, IDC_PROMPT4),
						&pkgi->reconPrompts[3][0], kPGPRecon_MaxPromptLen);
				UTF8EditGetText (GetDlgItem (hwnd, IDC_PROMPT5),
						&pkgi->reconPrompts[4][0], kPGPRecon_MaxPromptLen);

				// now get passphrases
				i = SEGetTextLength (GetDlgItem (hwnd, IDC_ANSWER1));
				if (i == 0)
				{
					PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
							IDS_NEEDRECONANSWER, MB_OK|MB_ICONEXCLAMATION);
					SetFocus (GetDlgItem (hwnd, IDC_ANSWER1));
					bContinue = FALSE;
				}
				else if (i >= kPGPRecon_MaxPassLen)
				{
					PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
							IDS_RECONANSWERTOOLONG, MB_OK|MB_ICONEXCLAMATION);
					SetFocus (GetDlgItem (hwnd, IDC_ANSWER1));
					bContinue = FALSE;
				}
				else
				{
					SEGetText (GetDlgItem (hwnd, IDC_ANSWER1),
							&pkgi->reconPasses[0][0], 
							kPGPRecon_MaxPassLen +1);
				}

				if (bContinue)
				{
					i = SEGetTextLength (GetDlgItem (hwnd, IDC_ANSWER2));
					if (i == 0)
					{
						PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
								IDS_NEEDRECONANSWER, 
								MB_OK|MB_ICONEXCLAMATION);
						SetFocus (GetDlgItem (hwnd, IDC_ANSWER2));
						bContinue = FALSE;
					}
					else if (i >= kPGPRecon_MaxPassLen)
					{
						PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
								IDS_RECONANSWERTOOLONG, 
								MB_OK|MB_ICONEXCLAMATION);
						SetFocus (GetDlgItem (hwnd, IDC_ANSWER2));
						bContinue = FALSE;
					}
					else
					{
						SEGetText (GetDlgItem (hwnd, IDC_ANSWER2),
								&pkgi->reconPasses[1][0], 
								kPGPRecon_MaxPassLen +1);
					}
				}

				if (bContinue)
				{
					i = SEGetTextLength (GetDlgItem (hwnd, IDC_ANSWER3));
					if (i == 0)
					{
						PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
								IDS_NEEDRECONANSWER, 
								MB_OK|MB_ICONEXCLAMATION);
						SetFocus (GetDlgItem (hwnd, IDC_ANSWER3));
						bContinue = FALSE;
					}
					else if (i >= kPGPRecon_MaxPassLen)
					{
						PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
								IDS_RECONANSWERTOOLONG, 
								MB_OK|MB_ICONEXCLAMATION);
						SetFocus (GetDlgItem (hwnd, IDC_ANSWER3));
						bContinue = FALSE;
					}
					else
					{
						SEGetText (GetDlgItem (hwnd, IDC_ANSWER3),
								&pkgi->reconPasses[2][0],
								kPGPRecon_MaxPassLen +1);
					}
				}

				if (bContinue)
				{
					i = SEGetTextLength (GetDlgItem (hwnd, IDC_ANSWER4));
					if (i == 0)
					{
						PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
								IDS_NEEDRECONANSWER, 
								MB_OK|MB_ICONEXCLAMATION);
						SetFocus (GetDlgItem (hwnd, IDC_ANSWER4));
						bContinue = FALSE;
					}
					else if (i >= kPGPRecon_MaxPassLen)
					{
						PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
								IDS_RECONANSWERTOOLONG, 
								MB_OK|MB_ICONEXCLAMATION);
						SetFocus (GetDlgItem (hwnd, IDC_ANSWER4));
						bContinue = FALSE;
					}
					else
					{
						SEGetText (GetDlgItem (hwnd, IDC_ANSWER4),
								&pkgi->reconPasses[3][0],
								kPGPRecon_MaxPassLen +1);
					}
				}

				if (bContinue)
				{
					i = SEGetTextLength (GetDlgItem (hwnd, IDC_ANSWER5));
					if (i == 0)
					{
						PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
								IDS_NEEDRECONANSWER, 
								MB_OK|MB_ICONEXCLAMATION);
						SetFocus (GetDlgItem (hwnd, IDC_ANSWER5));
						bContinue = FALSE;
					}
					else if (i >= kPGPRecon_MaxPassLen)
					{
						PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
								IDS_RECONANSWERTOOLONG, 
								MB_OK|MB_ICONEXCLAMATION);
						SetFocus (GetDlgItem (hwnd, IDC_ANSWER5));
						bContinue = FALSE;
					}
					else
					{
						SEGetText (GetDlgItem (hwnd, IDC_ANSWER5),
								&pkgi->reconPasses[4][0],
								kPGPRecon_MaxPassLen +1);
					}
				}

				if (bContinue)
					SetWindowLong(hwnd, DWL_MSGRESULT, 0);
				else
					SetWindowLong(hwnd, DWL_MSGRESULT, -1L);

				bReturnCode = TRUE;
				break;
			}

			case PSN_WIZBACK :
				SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER1));
				SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER2));
				SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER3));
				SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER4));
				SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER5));
				break;
			}
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for "Bad Passphrase" dialog

static LRESULT WINAPI
sKeyWizardBadPassphraseDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PKEYGENINFO		pkgi;
	BOOL			bReturnCode = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PKEYGENINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontInterior, (LPARAM)FALSE);

		SendDlgItemMessage (hwnd, IDC_ALERTICON1,
				STM_SETIMAGE, (WPARAM)IMAGE_ICON, (LPARAM)pkgi->hiconAlert);
		SendDlgItemMessage (hwnd, IDC_ALERTICON2,
				STM_SETIMAGE, (WPARAM)IMAGE_ICON, (LPARAM)pkgi->hiconAlert);
		return TRUE;
	}

	case WM_NOTIFY:
	{
		LPNMHDR pnmh		= (LPNMHDR) lParam;
		BOOL	bPhraseOK	= TRUE;
		INT		iShow;
		CHAR	sz1[128], sz2[128];

		switch (pnmh->code) {
		case PSN_SETACTIVE:
		{
			BOOL	bRejected;
			BOOL	bLengthRejected, bQualityRejected;
			BOOL	bConfirmRejected, bSmartCardRejected;
			INT		ids;

			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

			if (pkgi->uPhraseFlags == 0)
			{
				SetWindowLong (hwnd, DWL_MSGRESULT, -1L); // skip page
				bReturnCode = TRUE;
				break;
			}

			if (pkgi->bMinPhraseLength)
				ids = IDS_REQPHRASELENGTH;
			else
				ids = IDS_SUGPHRASELENGTH;
			LoadString (g_hInst, ids, sz1, sizeof(sz1));
			wsprintf (sz2, sz1, pkgi->iMinPhraseLength);
			SetDlgItemText (hwnd, IDC_BADLENGTH, sz2);

			if (pkgi->bMinPhraseQuality)
				ids = IDS_REQPHRASEQUALITY;
			else
				ids = IDS_SUGPHRASEQUALITY;
			LoadString (g_hInst, ids, sz1, sizeof(sz1));
			wsprintf (sz2, sz1, pkgi->iMinPhraseQuality);
			SetDlgItemText (hwnd, IDC_BADQUALITY1, sz2);

			bRejected = FALSE;
			bLengthRejected = FALSE;
			bQualityRejected = FALSE;
			bConfirmRejected = FALSE;
			bSmartCardRejected = FALSE;

			if ((pkgi->bMinPhraseLength) &&
				(pkgi->uPhraseFlags & BADPHRASE_LENGTH))
			{
				bLengthRejected = TRUE;
				bRejected = TRUE;
			}

			if ((pkgi->bMinPhraseQuality) &&
				(pkgi->uPhraseFlags & BADPHRASE_QUALITY))
			{
				bQualityRejected = TRUE;
				bRejected = TRUE;
			}

			if (pkgi->uPhraseFlags & BADPHRASE_CONFIRMATION)
			{
				bConfirmRejected = TRUE;
				bRejected = TRUE;
			}

			if (pkgi->uPhraseFlags & BADPHRASE_SMARTCARD)
			{
				bSmartCardRejected = TRUE;
				bRejected = TRUE;
			}

			if (bRejected)
			{
				LoadString (g_hInst, IDS_PHRASEREJECTED, sz1, sizeof(sz1));
				SetDlgItemText (hwnd, IDC_REJECTTEXT, sz1);
				PostMessage (GetParent (hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_BACK);
			}
			else
			{
				LoadString (g_hInst, IDS_PHRASEWARNED, sz1, sizeof(sz1));
				SetDlgItemText (hwnd, IDC_REJECTTEXT, sz1);
				PostMessage (GetParent (hwnd),
						PSM_SETWIZBUTTONS, 0, PSWIZB_BACK|PSWIZB_NEXT);
			}

			iShow = SW_HIDE;
			if (pkgi->uPhraseFlags & BADPHRASE_CONFIRMATION)
			{
				if (!bRejected || bConfirmRejected)
					iShow = SW_SHOW;
			}

			ShowWindow (GetDlgItem (hwnd, IDC_BADCONFIRM), iShow);
			ShowWindow (GetDlgItem (hwnd, IDC_ALERTICON1), iShow);

			if (iShow == SW_HIDE)
			{
				if (pkgi->uPhraseFlags & BADPHRASE_SMARTCARD)
				{
					if (!bRejected || bSmartCardRejected)
						iShow = SW_SHOW;
				}

				ShowWindow (GetDlgItem (hwnd, IDC_BADSMARTCARD), iShow);
				ShowWindow (GetDlgItem (hwnd, IDC_ALERTICON1), iShow);
			}
			else
				ShowWindow (GetDlgItem (hwnd, IDC_BADSMARTCARD), SW_HIDE);

			if (iShow == SW_HIDE)
			{
				if (pkgi->uPhraseFlags & BADPHRASE_LENGTH)
				{
					if (!bRejected || bLengthRejected)
						iShow = SW_SHOW;
				}

				ShowWindow (GetDlgItem (hwnd, IDC_BADLENGTH), iShow);
				ShowWindow (GetDlgItem (hwnd, IDC_ALERTICON1), iShow);
			}
			else
				ShowWindow (GetDlgItem (hwnd, IDC_BADLENGTH), SW_HIDE);

			iShow = SW_HIDE;
			if (pkgi->uPhraseFlags & BADPHRASE_QUALITY)
			{
				if (!bRejected || bQualityRejected)
					iShow = SW_SHOW;
			}

			ShowWindow (GetDlgItem (hwnd, IDC_BADQUALITY1), iShow);
			ShowWindow (GetDlgItem (hwnd, IDC_BADQUALITY2), iShow);
			ShowWindow (GetDlgItem (hwnd, IDC_ALERTICON2), iShow);

			SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;
		}
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for getting passphrase
//  from user

static LRESULT WINAPI
sKeyWizardPassphraseDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PKEYGENINFO		pkgi;
	BOOL			bReturnCode = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		CHAR sz1[256], sz2[256];
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PKEYGENINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontInterior, (LPARAM)FALSE);

		SEInit (pkgi->context, GetDlgItem (hwnd, IDC_PASSPHRASE),
				TRUE, sSpaceEditCallback, (void *)pkgi);

		SEInit (pkgi->context, GetDlgItem (hwnd, IDC_PASSPHRASE2),
				TRUE, sSpaceEditCallback, (void *)pkgi);

		if (pkgi->bOnToken)
			LoadString (g_hInst, IDS_TOKENPHRASETEXT, sz2, sizeof(sz2));
		else
		{
			LoadString (g_hInst, IDS_PHRASELENGTHTEXT, sz1, sizeof(sz1));
			wsprintf (sz2, sz1, pkgi->iMinPhraseLength);
		}
		SetDlgItemText (hwnd, IDC_PHRASELENGTHTEXT, sz2);

		SetFocus (GetDlgItem (hwnd, IDC_PASSPHRASE));
		return FALSE;
	}

	case WM_DESTROY:
		SEWipeEditBox (GetDlgItem (hwnd, IDC_PASSPHRASE));
		SEWipeEditBox (GetDlgItem (hwnd, IDC_PASSPHRASE2));

		SEDestroy (GetDlgItem (hwnd, IDC_PASSPHRASE));
		SEDestroy (GetDlgItem (hwnd, IDC_PASSPHRASE2));
		break;

	case WM_COMMAND:
		switch(LOWORD(wParam)) {
		case IDC_HIDETYPING:
		{
			BOOL	bHideTyping;

			if (IsDlgButtonChecked (hwnd, IDC_HIDETYPING) == BST_CHECKED)
				bHideTyping = TRUE;
			else
				bHideTyping = FALSE;

			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_PASSPHRASE), bHideTyping);
			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_PASSPHRASE2), bHideTyping);
			break;
		}
		}
		break;

	case WM_NOTIFY:
	{
		LPNMHDR	pnmh	= (LPNMHDR) lParam;

		switch (pnmh->code) {
		case PSN_SETACTIVE:
			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);
			PostMessage (GetParent(hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_BACK|PSWIZB_NEXT);
			SetWindowLong (hwnd, DWL_MSGRESULT, 0L);

			// install the security hooks, if not already set
			if (!s_bHooksInstalled)
				sInstallSecurityHooks ();

			CheckDlgButton (hwnd, IDC_HIDETYPING, BST_CHECKED);
			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_PASSPHRASE), TRUE);
			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_PASSPHRASE2), TRUE);

			// if generating on a token, the confirmation phrase is
			// unnecessary and annoying
			if (pkgi->bOnToken)
			{
				ShowWindow (GetDlgItem (hwnd, IDC_QUALITYTEXT), SW_HIDE);
				ShowWindow (GetDlgItem (hwnd, IDC_PHRASEQUALITY), SW_HIDE);
				ShowWindow (GetDlgItem (hwnd, IDC_CONFIRMATIONTEXT), SW_HIDE);
				EnableWindow (GetDlgItem (hwnd, IDC_PASSPHRASE2), FALSE);
				ShowWindow (GetDlgItem (hwnd, IDC_PASSPHRASE2), SW_HIDE);
			}
			else
			{
				ShowWindow (GetDlgItem (hwnd, IDC_QUALITYTEXT), SW_SHOW);
				ShowWindow (GetDlgItem (hwnd, IDC_PHRASEQUALITY), SW_SHOW);
				ShowWindow (GetDlgItem (hwnd, IDC_CONFIRMATIONTEXT), SW_SHOW);
				EnableWindow (GetDlgItem (hwnd, IDC_PASSPHRASE2), TRUE);
				ShowWindow (GetDlgItem (hwnd, IDC_PASSPHRASE2), SW_SHOW);
			}

			if ((pkgi->bMinPhraseQuality) && (!pkgi->bOnToken))
			{
				SendDlgItemMessage (hwnd, IDC_MINPHRASEQUALITY,
							PBM_SETRANGE, 0, MAKELPARAM (0, 100));
				SendDlgItemMessage (hwnd, IDC_MINPHRASEQUALITY,
							PBM_SETPOS, pkgi->iMinPhraseQuality, 0);
			}
			else
			{
				ShowWindow (GetDlgItem (hwnd, IDC_MINPHRASEQUALITY),
									SW_HIDE);
				ShowWindow (GetDlgItem (hwnd, IDC_MINQUALITYTEXT),
									SW_HIDE);
			}

			SendDlgItemMessage (hwnd, IDC_PHRASEQUALITY, PBM_SETRANGE,
									0, MAKELPARAM (0, 100));
			SendDlgItemMessage (hwnd, IDC_PHRASEQUALITY, PBM_SETPOS,
									0, 0);

			SetFocus (GetDlgItem (hwnd, IDC_PASSPHRASE));

			bReturnCode = TRUE;
			break;

		case PSN_WIZNEXT:
		{
			INT		i1, i2;
			INT		iQuality;
			LPSTR	pszPhrase;

			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

			if (pkgi->pszPassPhrase)
			{
				secFree (pkgi->pszPassPhrase);
				pkgi->pszPassPhrase = NULL;
			}

			i1 = SEGetTextLength (GetDlgItem (hwnd, IDC_PASSPHRASE));
			pkgi->pszPassPhrase = secAlloc ((i1+1) * sizeof(char));

			i2 = SEGetTextLength (GetDlgItem (hwnd, IDC_PASSPHRASE2));
			pszPhrase = secAlloc ((i2+1) * sizeof(char));

			if (IsNull (pkgi->pszPassPhrase) || IsNull (pszPhrase))
				break;

			SEGetText (GetDlgItem (hwnd, IDC_PASSPHRASE), 
					pkgi->pszPassPhrase, i1+1);
			SEGetText (GetDlgItem (hwnd, IDC_PASSPHRASE2), 
					pszPhrase, i2+1);

			pkgi->uPhraseFlags = 0;

			if (pkgi->bOnToken)
			{
				if (IsPGPError (PGPTokenPassphraseIsValid (pkgi->context,
						0, pkgi->pszPassPhrase, lstrlen (pkgi->pszPassPhrase))))
				{
					pkgi->uPhraseFlags |= BADPHRASE_SMARTCARD;
				}
			}
			else
			{
				PGPUInt32	ulen	= 0;

				if (strcmp (pkgi->pszPassPhrase, pszPhrase) != 0)
					pkgi->uPhraseFlags |= BADPHRASE_CONFIRMATION;

				pgpUTF8StringToUCS2 (pkgi->pszPassPhrase, 
						kPGPUnicodeNullTerminated, NULL, 0, &ulen);

				if (ulen < (PGPUInt32)pkgi->iMinPhraseLength)
					pkgi->uPhraseFlags |= BADPHRASE_LENGTH;

				iQuality = PGPEstimatePassphraseQuality (pkgi->pszPassPhrase);
				if (iQuality < pkgi->iMinPhraseQuality)
					pkgi->uPhraseFlags |= BADPHRASE_QUALITY;
			}

			secFree (pszPhrase);

			SEWipeEditBox (GetDlgItem (hwnd, IDC_PASSPHRASE));
			SEWipeEditBox (GetDlgItem (hwnd, IDC_PASSPHRASE2));
			SetFocus (GetDlgItem (hwnd, IDC_PASSPHRASE));

			SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;
		}

		case PSN_KILLACTIVE :
			if (!s_bHooksInstalled)
				sUninstallSecurityHooks ();
			break;

		case PSN_WIZBACK :
			SEWipeEditBox (GetDlgItem (hwnd, IDC_PASSPHRASE));
			SEWipeEditBox (GetDlgItem (hwnd, IDC_PASSPHRASE2));
			break;
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for corporate options dialog

static LRESULT WINAPI
sKeyWizardCorpOptsDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PKEYGENINFO		pkgi;
	BOOL			bReturnCode = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PKEYGENINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontInterior, (LPARAM)FALSE);

		UTF8EditInit (GetDlgItem (hwnd, IDC_CORPKEY));
		UTF8EditInit (GetDlgItem (hwnd, IDC_REVOKERKEY));
		UTF8EditInit (GetDlgItem (hwnd, IDC_ADK));

		return TRUE;
	}

	case WM_DESTROY:
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_CORPKEY));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_REVOKERKEY));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_ADK));
		break;

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch (pnmh->code) {
		case PSN_SETACTIVE:
			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);
			if (PGPKeyDBObjRefIsValid (pkgi->keyCorp) ||
				(PGPKeyDBObjRefIsValid (pkgi->keyRevoker) &&
					(pkgi->uKeyType != kPGPPublicKeyAlgorithm_RSA)) ||
				(PGPKeyDBObjRefIsValid (pkgi->keyADK) &&
					(pkgi->uKeyType != kPGPPublicKeyAlgorithm_RSA)) ||
				(pkgi->bAutoAddExchangeUserID && 
					pkgi->bIsExchangeUser) ||
				pkgi->bDoSend ||
				pkgi->bDoCertRequest)
			{
				CHAR sz[kPGPMaxUserIDSize];
				UINT u;

				PostMessage (GetParent (hwnd),
						PSM_SETWIZBUTTONS, 0, PSWIZB_BACK|PSWIZB_NEXT);

				if (PGPKeyDBObjRefIsValid (pkgi->keyCorp))
				{
					ShowWindow (GetDlgItem (hwnd, IDC_CORPKEYTEXT), SW_SHOW);
					ShowWindow (UTF8EditGetHandle (
							GetDlgItem (hwnd, IDC_CORPKEY)), SW_SHOW);
					u = sizeof(sz);
					PGPclGetPrimaryUserIDNameUTF8 (
							pkgi->keyCorp, sz, sizeof(sz), &u);
					UTF8EditSetText (GetDlgItem (hwnd, IDC_CORPKEY), sz);
				}

				if (PGPKeyDBObjRefIsValid (pkgi->keyRevoker) &&
					(pkgi->uKeyType != kPGPPublicKeyAlgorithm_RSA))
				{
					ShowWindow (GetDlgItem (hwnd, IDC_REVOKERKEYTEXT), SW_SHOW);
					ShowWindow (UTF8EditGetHandle (
							GetDlgItem (hwnd, IDC_REVOKERKEY)), SW_SHOW);
					u = sizeof(sz);
					PGPclGetPrimaryUserIDNameUTF8 (
							pkgi->keyRevoker, sz, sizeof(sz), &u);
					UTF8EditSetText (GetDlgItem (hwnd, IDC_REVOKERKEY), sz);
				}

				if (PGPKeyDBObjRefIsValid (pkgi->keyADK) &&
					(pkgi->uKeyType != kPGPPublicKeyAlgorithm_RSA))
				{
					ShowWindow (GetDlgItem (hwnd, IDC_ADKTEXT1), SW_SHOW);
					ShowWindow (UTF8EditGetHandle (
							GetDlgItem (hwnd, IDC_ADK)), SW_SHOW);
					ShowWindow (GetDlgItem (hwnd, IDC_ADKTEXT2), SW_SHOW);
					u = sizeof(sz);
					PGPclGetPrimaryUserIDNameUTF8 (
							pkgi->keyADK, sz, sizeof(sz), &u);
					UTF8EditSetText (GetDlgItem (hwnd, IDC_ADK), sz);
				}

				if (pkgi->bDoSend)
				{
					ShowWindow (GetDlgItem (hwnd, IDC_UPLOADTEXT1), SW_SHOW);
					ShowWindow (GetDlgItem (hwnd, IDC_UPLOADTEXT2), SW_SHOW);
				}

				if (pkgi->bDoCertRequest)
				{
					ShowWindow (GetDlgItem (hwnd, IDC_X509TEXT1), SW_SHOW);
					ShowWindow (GetDlgItem (hwnd, IDC_X509TEXT2), SW_SHOW);
				}

				if (pkgi->bAutoAddExchangeUserID &&
					pkgi->bIsExchangeUser)
				{
					ShowWindow (GetDlgItem (hwnd, IDC_EXCHANGETEXT1), SW_SHOW);
					ShowWindow (GetDlgItem (hwnd, IDC_EXCHANGETEXT2), SW_SHOW);
				}

				SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
			}
			else
				SetWindowLong (hwnd, DWL_MSGRESULT, -1L); // skip page

			bReturnCode = TRUE;
			break;
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for MS Exchange lookup option

static LRESULT WINAPI
sKeyWizardExchangeDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PKEYGENINFO		pkgi;
	BOOL			bReturnCode = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PKEYGENINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontInterior, (LPARAM)FALSE);

		CheckDlgButton (hwnd, IDC_ENABLEEXCHANGELOOKUP, BST_CHECKED);
		return TRUE;
	}

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch (pnmh->code) {
		case PSN_SETACTIVE:
			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);
			if (pkgi->bIsExchangeUser &&
				!pkgi->bAutoAddExchangeUserID)
			{
				PostMessage (GetParent (hwnd),
						PSM_SETWIZBUTTONS, 0, PSWIZB_BACK|PSWIZB_NEXT);


				SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
			}
			else
				SetWindowLong (hwnd, DWL_MSGRESULT, -1L); // skip page

			bReturnCode = TRUE;
			break;

		case PSN_WIZNEXT :
			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);
			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLEEXCHANGELOOKUP) == BST_CHECKED)
			{
				pkgi->bLookupExchangeUserID = TRUE;
			}
			else
				pkgi->bLookupExchangeUserID = FALSE;
			break;
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for getting type of key (RSA/DSA)
//  from user

static LRESULT WINAPI
sKeyWizardExpertDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PKEYGENINFO		pkgi;
	BOOL			bReturnCode = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE*	ppspMsgRec	= (PROPSHEETPAGE *) lParam;
		INT				iIndex, iIndexSet;
		CHAR			sz[64];
		RECT			rc;
		SYSTEMTIME		stExpire;
		PGPUInt32		uTokens;

		pkgi = (PKEYGENINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontInterior, (LPARAM)FALSE);

		// initialize name and email
		UTF8EditInit (GetDlgItem (hwnd, IDC_FULLNAME));
		SendMessage (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_FULLNAME)),
				EM_SETLIMITTEXT, MAX_FULL_NAME_LEN, 0);
		UTF8EditInit (GetDlgItem (hwnd, IDC_EMAIL));
		SendMessage (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_EMAIL)),
				EM_SETLIMITTEXT, MAX_EMAIL_LEN, 0);

		if (pkgi->pszFullName)
		{
			UTF8EditSetText (GetDlgItem (hwnd, IDC_FULLNAME), 
					pkgi->pszFullName);
		}
		if (pkgi->pszEmail)
		{
			UTF8EditSetText (GetDlgItem (hwnd, IDC_EMAIL), 
					pkgi->pszEmail);
		}

		// initialize key algorithm and size controls
		iIndexSet = 0;
		LoadString (g_hInst, IDS_DHDSS, sz, sizeof(sz));
		iIndex = SendDlgItemMessage (hwnd,
				IDC_KEYTYPE, CB_ADDSTRING, 0, (LPARAM)sz);
		SendDlgItemMessage (hwnd,
				IDC_KEYTYPE, CB_SETITEMDATA, 
				iIndex, kPGPPublicKeyAlgorithm_DSA);
		if (pkgi->uKeyType == kPGPPublicKeyAlgorithm_DSA)
		{
			iIndexSet = iIndex;
			SetDlgItemInt (hwnd, IDC_KEYSIZE, pkgi->uSubKeySize, FALSE);
		}

		if (pkgi->bAllowRSAGen)
		{
			LoadString (g_hInst, IDS_RSA, sz, sizeof(sz));
			iIndex = SendDlgItemMessage (hwnd,
					IDC_KEYTYPE, CB_ADDSTRING, 0, (LPARAM)sz);
			SendDlgItemMessage (hwnd,
					IDC_KEYTYPE, CB_SETITEMDATA, 
					iIndex, kPGPPublicKeyAlgorithm_RSASignOnly);
			if (pkgi->uKeyType == kPGPPublicKeyAlgorithm_RSASignOnly)
			{
				iIndexSet = iIndex;
				SetDlgItemInt (hwnd, IDC_KEYSIZE, pkgi->uKeySize, FALSE);
			}

			LoadString (g_hInst, IDS_RSA_LEGACY, sz, sizeof(sz));
			iIndex = SendDlgItemMessage (hwnd,
					IDC_KEYTYPE, CB_ADDSTRING, 0, (LPARAM)sz);
			SendDlgItemMessage (hwnd,
					IDC_KEYTYPE, CB_SETITEMDATA, 
					iIndex, kPGPPublicKeyAlgorithm_RSA);
			if (pkgi->uKeyType == kPGPPublicKeyAlgorithm_RSA)
			{
				iIndexSet = iIndex;
				SetDlgItemInt (hwnd, IDC_KEYSIZE, pkgi->uKeySize, FALSE);
			}
		}

		if (pkgi->bAllowECGen)
		{
			LoadString (g_hInst, IDS_EC, sz, sizeof(sz));
			iIndex = SendDlgItemMessage (hwnd,
					IDC_KEYTYPE, CB_ADDSTRING, 0, (LPARAM)sz);
			SendDlgItemMessage (hwnd,
					IDC_KEYTYPE, CB_SETITEMDATA, 
					iIndex, kPGPPublicKeyAlgorithm_ECSign);
			if (pkgi->uKeyType == kPGPPublicKeyAlgorithm_ECSign)
			{
				iIndexSet = iIndex;
				SetDlgItemInt (hwnd, IDC_KEYSIZE, pkgi->uKeySize, FALSE);
			}
		}

		SendDlgItemMessage (hwnd,
				IDC_KEYTYPE, CB_SETCURSEL, iIndexSet, 0);
		PostMessage (hwnd, WM_COMMAND, 
				MAKEWPARAM (IDC_KEYTYPE, CBN_SELCHANGE), 0);

		// initialize key expiration controls
		// first create and initialize date/time picker control
		GetWindowRect (GetDlgItem (hwnd, IDC_EXPIRATIONDATE), &rc);
		MapWindowPoints (NULL, hwnd, (LPPOINT)&rc, 2);
		pkgi->hwndExpirationDate = CreateWindowEx (0, DATETIMEPICK_CLASS,
		                     "DateTime",
		                     WS_BORDER|WS_CHILD|WS_VISIBLE|WS_TABSTOP,
		                     rc.left, rc.top,
							 rc.right-rc.left, rc.bottom-rc.top,
							 hwnd, NULL, g_hInst, NULL);

		SendMessage (pkgi->hwndExpirationDate, DTM_SETMCCOLOR,
				MCSC_MONTHBK, (LPARAM)GetSysColor (COLOR_3DFACE));

		// either use preset date or set default date to one year from today
		if (pkgi->bPreset && (pkgi->uExpireDays != 0))
		{
			stExpire = pkgi->stPresetExpiration;
		}
		else
		{
			GetLocalTime (&stExpire);
			stExpire.wYear++;
		}
		SendMessage (pkgi->hwndExpirationDate,
				DTM_SETSYSTEMTIME, GDT_VALID, (LPARAM)&stExpire);

		if (pkgi->uExpireDays == 0)
		{
			CheckRadioButton (hwnd, IDC_NEVER, IDC_DATE, IDC_NEVER);
			EnableWindow (pkgi->hwndExpirationDate, FALSE);
		}
		else
		{
			CheckRadioButton (hwnd, IDC_NEVER, IDC_DATE, IDC_DATE);
			EnableWindow (pkgi->hwndExpirationDate, TRUE);
		}

		// initialize "generate on token" checkbox
		PGPCountTokens (g_context, &uTokens);
		if (uTokens > 0)
		{
			if (pkgi->bAllowRSAGen && !pkgi->bPreset)
			{
				PGPTokenInfo	pti;

				pti.size =  sizeof(pti);
				PGPGetTokenInfo (pkgi->context, 0, &pti);

				EnableWindow (GetDlgItem (hwnd, IDC_ONTOKEN), pti.bRsa);
			}
			ShowWindow (GetDlgItem (hwnd, IDC_ONTOKEN), SW_SHOW);
		}

		// if values are preset, disable controls
		if (pkgi->bPreset)
		{
			EnableWindow (GetDlgItem (hwnd, IDC_KEYTYPE), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_KEYSIZE), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_NEVER), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_DATE), FALSE);
			EnableWindow (pkgi->hwndExpirationDate, FALSE);
		}

		// initialize Intel RNG icon and text
		if (!PGPGlobalRandomPoolHasIntelRNG ())
		{
			ShowWindow (GetDlgItem (hwnd, IDC_INTELICON), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_INTELTEXT), SW_HIDE);
		}
		return TRUE;
	}

	case WM_DESTROY:
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_FULLNAME));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_EMAIL));
		break;

	case WM_COMMAND:
		pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

		switch (LOWORD(wParam)) {
		case IDC_EXPIRATIONDATE :
			SetFocus (pkgi->hwndExpirationDate);
			break;

		case IDC_NEVER :
		case IDC_DATE :
			if (IsDlgButtonChecked (hwnd, IDC_DATE) == BST_CHECKED)
				EnableWindow (pkgi->hwndExpirationDate, TRUE);
			else
				EnableWindow (pkgi->hwndExpirationDate, FALSE);
			break;

		case IDC_KEYTYPE :
			if (HIWORD (wParam) == CBN_SELCHANGE)
			{
				BOOL	bEnable		= TRUE;

				UINT	uIndex, uKeyType;
				CHAR	sz[32];

				uIndex = SendDlgItemMessage (hwnd,
						IDC_KEYTYPE, CB_GETCURSEL, 0, 0);
				uKeyType = SendDlgItemMessage (hwnd,
						IDC_KEYTYPE, CB_GETITEMDATA, uIndex, 0);

				if (IsDlgButtonChecked (hwnd, IDC_ONTOKEN) == BST_CHECKED)
				{
					PGPTokenInfo	pti;
					PGPUInt32		uMin, uSize;

					pti.size =  sizeof(pti);
					PGPGetTokenInfo (pkgi->context, 0, &pti);

					switch (uKeyType) {
					case kPGPPublicKeyAlgorithm_RSA :
						if (lParam)
						{
							PGPclMessageBox (hwnd, IDS_CAPTIONINFO,
									IDS_RSALEGACYWARN, 
									MB_OK|MB_ICONINFORMATION);
						}
						uMin = MIN_RSA_LEGACY_KEY_SIZE;
						break;

					case kPGPPublicKeyAlgorithm_RSASignOnly :
						uMin = MIN_RSA_KEY_SIZE;
						break;

					case kPGPPublicKeyAlgorithm_DSA :
					case kPGPPublicKeyAlgorithm_ECSign :
						if (lParam)
						{
							PGPclMessageBox (hwnd, IDS_CAPTIONINFO, 
									IDS_TOKENALGNOTSUPPORTED, 
									MB_OK|MB_ICONINFORMATION);
						}
						// since IDC_ONTOKEN is only enabled if RSA keygen
						// is enabled, we know that keytype item #1 is "RSA"
						SendDlgItemMessage (hwnd, IDC_KEYTYPE,
								CB_SETCURSEL, 1, 0);
						uMin = MIN_RSA_KEY_SIZE;
						break;
					}

					wsprintf (sz, "%i - %i", uMin, pti.maxRsaKeySize);

					uSize = GetDlgItemInt (hwnd, IDC_KEYSIZE, NULL, FALSE);

					if (uSize < uMin)
						uSize = uMin;
					if (uSize > pti.maxRsaKeySize)
						uSize = pti.maxRsaKeySize;

					SetDlgItemInt (hwnd, IDC_KEYSIZE, uSize, FALSE);
				}
				else
				{
					UINT	uMin, uMax, uSize;

					switch (uKeyType) {
					case kPGPPublicKeyAlgorithm_RSA :
						if (lParam)
						{
							PGPclMessageBox (hwnd, IDS_CAPTIONINFO,
									IDS_RSALEGACYWARN, 
									MB_OK|MB_ICONINFORMATION);
						}
						uMin = MIN_RSA_LEGACY_KEY_SIZE;
						uMax = MAX_RSA_LEGACY_KEY_SIZE;
						uSize = MIN_RSA_LEGACY_KEY_SIZE;
						break;

					case kPGPPublicKeyAlgorithm_RSASignOnly :
						uMin = MIN_RSA_KEY_SIZE;
						uMax = MAX_RSA_KEY_SIZE;
						uSize = MIN_RSA_KEY_SIZE;
						break;

					case kPGPPublicKeyAlgorithm_DSA :
						uMin = MIN_DHDSS_KEY_SIZE;
						uMax = MAX_DHDSS_KEY_SIZE;
						uSize = MIN_DHDSS_KEY_SIZE;
						break;

					case kPGPPublicKeyAlgorithm_ECSign :
						bEnable = FALSE;
						uMin = MIN_EC_KEY_SIZE;
						uMax = MAX_EC_KEY_SIZE;
						uSize = MIN_EC_KEY_SIZE;
						break;
					}

					wsprintf (sz, "%i - %i", uMin, uMax);
					uSize = GetDlgItemInt (hwnd, IDC_KEYSIZE, NULL, FALSE);
					if (uSize < uMin)
						uSize = uMin;
					if (uSize > uMax)
						uSize = uMax;

					SetDlgItemInt (hwnd, IDC_KEYSIZE, uSize, FALSE);
				}

				SetDlgItemText (hwnd, IDC_KEYSIZETEXT, sz);
				EnableWindow (GetDlgItem (hwnd, IDC_KEYSIZE), bEnable);
			}
			break;

		case IDC_ONTOKEN :
			PostMessage (hwnd, WM_COMMAND, 
					MAKEWPARAM (IDC_KEYTYPE, CBN_SELCHANGE), 0);
			break;

		case IDC_MOREINFO :
			PGPclHtmlHelp (hwnd, kPGPclHelpMoreInformation, 0, 0,
					(char*)kPGPclHelpFile, (DWORD*)IDH_PGPCLWIZ_EXPERTMOREINFO);
			break;
		}
		break;

	case WM_NOTIFY:
	{
		BOOL		bContinue	= TRUE;
		LPNMHDR		pnmh		= (LPNMHDR) lParam;

		switch(pnmh->code) {
		case PSN_SETACTIVE:
			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

			if (!pkgi->bExpertMode)
			{
				SetWindowLong (hwnd, DWL_MSGRESULT, -1L); // skip page
			}
			else
			{
				PostMessage(GetParent(hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_BACK|PSWIZB_NEXT);
				SetWindowLong(hwnd, DWL_MSGRESULT, 0L);
			}
			bReturnCode = TRUE;
			break;

		case PSN_WIZNEXT :
		{
			INT			iTextLen;
			INT			iExpireDays;
			UINT		uKeySize;
			UINT		uIndex, uKeyType;
			UINT		uMinKeySize;
			UINT		uMaxKeySize;
			SYSTEMTIME	stExpire;
			CHAR		sz[32];

			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

			if (pkgi->pszFullName)
			{
				clFree (pkgi->pszFullName);
				pkgi->pszFullName = NULL;
			}

			if (pkgi->pszEmail)
			{
				clFree (pkgi->pszEmail);
				pkgi->pszEmail = NULL;
			}

			// allocate string and get name from edit box
			iTextLen = UTF8EditGetTextLength (
					GetDlgItem (hwnd, IDC_FULLNAME)) +1;
			pkgi->pszFullName = clAlloc (iTextLen * sizeof(char));
			if (pkgi->pszFullName)
			{
				UTF8EditGetText (GetDlgItem (hwnd, IDC_FULLNAME),
						pkgi->pszFullName, iTextLen);
			}

			// no name entered, warn user
			if (iTextLen <= 1)
			{
				PGPclMessageBox (hwnd, IDS_KEYGENTITLE,
								 IDS_KEYGENNOFULLNAME,
								 MB_OK|MB_ICONINFORMATION);
				bContinue = FALSE;
				SetFocus (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_FULLNAME)));
			}

			if (bContinue)
			{
				iTextLen = UTF8EditGetTextLength (
						GetDlgItem (hwnd, IDC_EMAIL)) +1;
				pkgi->pszEmail = clAlloc (iTextLen * sizeof(char));
				if (pkgi->pszEmail)
				{
					UTF8EditGetText (GetDlgItem (hwnd, IDC_EMAIL),
							pkgi->pszEmail, iTextLen);
				}

				if (iTextLen <= 1)
				{
					if (pkgi->bDoCertRequest)
					{
						PGPclMessageBox (hwnd, IDS_KEYGENTITLE,
								IDS_KEYGENNEEDEMAIL,
								MB_OK|MB_ICONINFORMATION);
						bContinue = FALSE;
					}
					else
					{
						if (PGPclMessageBox (hwnd, IDS_KEYGENTITLE,
								IDS_KEYGENNOEMAIL,
								MB_YESNO|MB_ICONINFORMATION|
								MB_DEFBUTTON2) != IDYES)
							bContinue = FALSE;
					}
					SetFocus (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_EMAIL)));
				}
			}
			if (!bContinue)
			{
				SetWindowLong(hwnd, DWL_MSGRESULT, -1L);
				bReturnCode = TRUE;
				break;
			}

			// handle key type
			uIndex = SendDlgItemMessage (hwnd,
					IDC_KEYTYPE, CB_GETCURSEL, 0, 0);
			uKeyType = SendDlgItemMessage (hwnd,
					IDC_KEYTYPE, CB_GETITEMDATA, uIndex, 0);
			if (uKeyType == kPGPPublicKeyAlgorithm_RSA)
			{
				pkgi->uKeyType = kPGPPublicKeyAlgorithm_RSA;
				pkgi->uKeyAlg = kPGPPublicKeyAlgorithm_RSA;
				pkgi->uKeyFlags = 0;
				pkgi->uSubKeyAlg = 0;
				pkgi->uSubKeyFlags = 0;

				uMinKeySize = MIN_RSA_LEGACY_KEY_SIZE;
				uMaxKeySize = MAX_RSA_LEGACY_KEY_SIZE;
				LoadString (g_hInst, IDS_RSA_LEGACY, sz, sizeof(sz));
			}
			else if (uKeyType == kPGPPublicKeyAlgorithm_RSASignOnly)
			{
				pkgi->uKeyType = kPGPPublicKeyAlgorithm_RSASignOnly;
				pkgi->uKeyAlg = kPGPPublicKeyAlgorithm_RSA;
				pkgi->uKeyFlags = kPGPKeyPropertyFlags_UsageSign;
				pkgi->uSubKeyAlg = kPGPPublicKeyAlgorithm_RSA;
				pkgi->uSubKeyFlags = kPGPKeyPropertyFlags_UsageEncrypt;

				uMinKeySize = MIN_RSA_KEY_SIZE;
				uMaxKeySize = MAX_RSA_KEY_SIZE;
				LoadString (g_hInst, IDS_RSA, sz, sizeof(sz));
			}
			else if (uKeyType == kPGPPublicKeyAlgorithm_DSA)
			{
				pkgi->uKeyType = kPGPPublicKeyAlgorithm_DSA;
				pkgi->uKeyAlg = kPGPPublicKeyAlgorithm_DSA;
				pkgi->uKeyFlags = kPGPKeyPropertyFlags_UsageSign;
				pkgi->uSubKeyAlg = kPGPPublicKeyAlgorithm_ElGamal;
				pkgi->uSubKeyFlags = kPGPKeyPropertyFlags_UsageEncrypt;

				uMinKeySize = MIN_DHDSS_KEY_SIZE;
				uMaxKeySize = MAX_DHDSS_KEY_SIZE;
				LoadString (g_hInst, IDS_DHDSS, sz, sizeof(sz));
			}
			else if (uKeyType == kPGPPublicKeyAlgorithm_ECSign)
			{
				pkgi->uKeyType = kPGPPublicKeyAlgorithm_ECSign;
				pkgi->uKeyAlg = kPGPPublicKeyAlgorithm_ECSign;
				pkgi->uKeyFlags = kPGPKeyPropertyFlags_UsageSign;
				pkgi->uSubKeyAlg = kPGPPublicKeyAlgorithm_ECEncrypt;
				pkgi->uSubKeyFlags = kPGPKeyPropertyFlags_UsageEncrypt;

				uMinKeySize = MIN_EC_KEY_SIZE;
				uMaxKeySize = MAX_EC_KEY_SIZE;
				LoadString (g_hInst, IDS_EC, sz, sizeof(sz));
			}

			// handle key size
			uKeySize = GetDlgItemInt (hwnd, IDC_KEYSIZE, NULL, FALSE);

			if (uKeyType != kPGPPublicKeyAlgorithm_ECSign)
				uMinKeySize = max (uMinKeySize, pkgi->uMinKeySize);

			if ((uKeySize < uMinKeySize) ||
				(uKeySize > uMaxKeySize))
			{
				CHAR szError[256], szTitle[256], szFormat[256];

				LoadString (g_hInst,
						IDS_KEYGENTITLE, szTitle, sizeof(szTitle));
				LoadString (g_hInst,
						IDS_INVALIDKEYSIZE, szFormat, sizeof(szFormat));
				wsprintf (szError, szFormat,
						sz, uMinKeySize, uMaxKeySize);

				MessageBox (hwnd, szError, szTitle, MB_OK|MB_ICONINFORMATION);
				SetWindowLong (hwnd, DWL_MSGRESULT, -1L);
				bReturnCode = TRUE;
				break;
			}
			else
			{
 				if (pkgi->uKeyType == kPGPPublicKeyAlgorithm_DSA)
				{
 					pkgi->uKeySize = sGetDSAKeySize (uKeySize);
 					pkgi->uSubKeySize = uKeySize;
 				}
 				else
				{
 					pkgi->uKeySize = uKeySize;
 					pkgi->uSubKeySize = uKeySize;
				}
			}

			// handle expiration
			if (IsDlgButtonChecked (hwnd, IDC_NEVER) == BST_CHECKED)
				iExpireDays = 0;
			else
			{
				SendMessage (pkgi->hwndExpirationDate,
						DTM_GETSYSTEMTIME, 0, (LPARAM)&stExpire);
				PGPclSystemTimeToDays (&stExpire, &iExpireDays);
				if (iExpireDays <= 0)
				{
					PGPclMessageBox (hwnd, IDS_KEYGENTITLE,
							IDS_KEYGENINVALIDEXPIRATION,
							MB_OK|MB_ICONINFORMATION);
					SetWindowLong (hwnd, DWL_MSGRESULT, -1L);
					bReturnCode = TRUE;
				}
			}

			// handle on token or not
			if (IsDlgButtonChecked (hwnd, IDC_ONTOKEN) == BST_CHECKED)
			{
				pkgi->bOnToken = TRUE;
				if (!sVerifyTokenProperties (hwnd, pkgi))
					SetWindowLong (hwnd, DWL_MSGRESULT, -1L);
			}
			else
				pkgi->bOnToken = FALSE;


			pkgi->uExpireDays = (UINT)iExpireDays;
			bReturnCode = TRUE;
			break;
		}
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for getting name and email address
//  from user

static LRESULT WINAPI
sKeyWizardNameDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PKEYGENINFO		pkgi;
	BOOL			bReturnCode = FALSE;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PKEYGENINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontInterior, (LPARAM)FALSE);

		// initialize name and email
		UTF8EditInit (GetDlgItem (hwnd, IDC_FULLNAME));
		SendMessage (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_FULLNAME)),
				EM_SETLIMITTEXT, MAX_FULL_NAME_LEN, 0);
		UTF8EditInit (GetDlgItem (hwnd, IDC_EMAIL));
		SendMessage (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_EMAIL)),
				EM_SETLIMITTEXT, MAX_EMAIL_LEN, 0);

		if (pkgi->pszFullName)
		{
			UTF8EditSetText (GetDlgItem (hwnd, IDC_FULLNAME), 
					pkgi->pszFullName);
		}
		if (pkgi->pszEmail)
		{
			UTF8EditSetText (GetDlgItem (hwnd, IDC_EMAIL), 
					pkgi->pszEmail);
		}

		return TRUE;
	}

	case WM_DESTROY:
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_FULLNAME));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_EMAIL));
		break;

	case WM_NOTIFY:
	{
		INT		iTextLen;
		BOOL	bContinue = TRUE;
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch(pnmh->code) {
		case PSN_SETACTIVE:
			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

			if (pkgi->bExpertMode)
			{
				SetWindowLong (hwnd, DWL_MSGRESULT, -1L); // skip page
			}
			else
			{
				PostMessage(GetParent(hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_BACK|PSWIZB_NEXT);
				SetWindowLong(hwnd, DWL_MSGRESULT, 0L);
			}
			bReturnCode = TRUE;
			break;

		case PSN_WIZNEXT:
			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

			if (pkgi->pszFullName)
			{
				clFree (pkgi->pszFullName);
				pkgi->pszFullName = NULL;
			}

			if (pkgi->pszEmail)
			{
				clFree (pkgi->pszEmail);
				pkgi->pszEmail = NULL;
			}

			// allocate string and get name from edit box
			iTextLen = UTF8EditGetTextLength (
					GetDlgItem (hwnd, IDC_FULLNAME)) +1;
			pkgi->pszFullName = clAlloc (iTextLen * sizeof(char));
			if (pkgi->pszFullName)
			{
				UTF8EditGetText (GetDlgItem (hwnd, IDC_FULLNAME), 
						pkgi->pszFullName, iTextLen);
			}

			// no name entered, warn user
			if (iTextLen <= 1)
			{
				PGPclMessageBox (hwnd, IDS_KEYGENTITLE,
								 IDS_KEYGENNOFULLNAME,
								 MB_OK|MB_ICONINFORMATION);
				bContinue = FALSE;
				SetFocus (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_FULLNAME)));
			}

			if (bContinue)
			{
				iTextLen = UTF8EditGetTextLength (
						GetDlgItem (hwnd, IDC_EMAIL)) +1;
				pkgi->pszEmail = clAlloc (iTextLen * sizeof(char));
				if (pkgi->pszEmail)
				{
					UTF8EditGetText (GetDlgItem (hwnd, IDC_EMAIL), 
							pkgi->pszEmail, iTextLen);
				}

				if (iTextLen <= 1)
				{
					if (pkgi->bDoCertRequest)
					{
						PGPclMessageBox (hwnd, IDS_KEYGENTITLE,
								IDS_KEYGENNEEDEMAIL,
								MB_OK|MB_ICONINFORMATION);
						bContinue = FALSE;
					}
					else
					{
						if (PGPclMessageBox (hwnd, IDS_KEYGENTITLE,
								IDS_KEYGENNOEMAIL,
								MB_YESNO|MB_ICONINFORMATION|
								MB_DEFBUTTON2) != IDYES)
							bContinue = FALSE;
					}
					SetFocus (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_EMAIL)));
				}
			}

			if (!bContinue)
			{
				SetWindowLong(hwnd, DWL_MSGRESULT, -1L);
				bReturnCode = TRUE;
			}
		}
		}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for handling beginning introductory
//  dialog.

static LRESULT WINAPI
sKeyWizardIntroDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	BOOL			bReturnCode = FALSE;
	PKEYGENINFO		pkgi		= NULL;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pkgi = (PKEYGENINFO) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pkgi);

		// center dialog on screen
		PGPclCenterWindowOnMonitor (GetParent(hwnd), kPGPclWindowMonitor);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pkgi->hfontIntro, (LPARAM)FALSE);

		// if parameters are preset, disable "Expert" button
		if (pkgi->bPreset)
		{
			EnableWindow (GetDlgItem (hwnd, IDC_EXPERT), FALSE);
			ShowWindow (GetDlgItem (hwnd, IDC_EXPERT), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_EXPERTTEXT), SW_HIDE);
		}

		return TRUE;
	}

	case WM_ACTIVATE :
		InvalidateRect (hwnd, NULL, TRUE);
		break;

	case WM_PAINT :
		pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

		if (pkgi->hpaletteIntro)
		{
			PAINTSTRUCT ps;
			HDC			hDC = BeginPaint (hwnd, &ps);

			SelectPalette (hDC, pkgi->hpaletteIntro, FALSE);
			RealizePalette (hDC);
			EndPaint (hwnd, &ps);
		}
		break;

	case WM_COMMAND:
		pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);

		switch (HIWORD(wParam)) {
		case BN_CLICKED:
			if (LOWORD (wParam) == IDC_EXPERT)
			{
				pkgi->bExpertMode = TRUE;
				SendMessage (GetParent (hwnd), PSM_PRESSBUTTON,
						(WPARAM)PSBTN_NEXT, 0);
			}
			break;
		}
		break;

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch(pnmh->code) {
		case PSN_SETACTIVE:
			pkgi = (PKEYGENINFO)GetWindowLong (hwnd, GWL_USERDATA);
			pkgi->bExpertMode = FALSE;

			PostMessage(GetParent(hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_NEXT);
			SendDlgItemMessage(hwnd, IDC_WIZBITMAP, STM_SETIMAGE,
					IMAGE_BITMAP, (LPARAM) pkgi->hbitmapIntro);

			SetWindowLong(hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;

		case PSN_KILLACTIVE :
			break;
		}

		break;
	}
	}

	return bReturnCode;
}

//	______________________________________________
//
//  Create wizard data structures and call PropertySheet
//  to actually create wizard in unicode

static PGPError
sKeyGenerationWizardW (
	BOOL				bRecenter,
	PGPContextRef		context,
	PGPtlsContextRef	tlscontext,
	HWND				hwndParent,
	PGPKeyDBRef			keydb,
	PGPKeyDBObjRef*		pkeyNew,
	PGPUInt32*			puFlags)
{
	PGPError			err					= kPGPError_NoErr;

	KEYGENINFO			KGInfo;
	PROPSHEETPAGEW		pspWiz[NUM_WIZ_PAGES];
	PROPSHEETHEADERW	pshWiz;
	INT					iIndex;
	INT					iNumBits, iBitmap;
	HDC					hDC;

	// intialize windows
	s_hwndCollectEntropy = NULL;

	// Set defaults
	if (PGPContextRefIsValid (context))
		KGInfo.context = context;
	else
		KGInfo.context = g_context;

	if (PGPtlsContextRefIsValid (tlscontext))
		KGInfo.tlscontext = tlscontext;
	else
	{
		err = PGPNewTLSContext (KGInfo.context, &KGInfo.tlscontext);
		if (IsPGPError (err))
			return err;
	}

	KGInfo.bRecenter			= bRecenter;
	KGInfo.uNumTasks			= 0;
	KGInfo.bExpertMode			= FALSE;
	KGInfo.bDoSend				= FALSE;
	KGInfo.bDoCertRequest		= FALSE;
	KGInfo.bDoReconstruction	= FALSE;
	KGInfo.bLookupExchangeUserID= FALSE;
	KGInfo.bFinishSelected		= FALSE;
	KGInfo.keydbMain			= keydb;
	KGInfo.keyNew				= kInvalidPGPKeyDBObjRef;

	KGInfo.hwndParent			= hwndParent;
	KGInfo.hwndWiz				= NULL;
	KGInfo.hpaletteIntro		= NULL;
	KGInfo.hpaletteDone			= NULL;

	KGInfo.pszPassPhrase		= NULL;
	KGInfo.pszFullName			= NULL;
	KGInfo.pszEmail				= NULL;
	KGInfo.pszNotesFullName		= NULL;
	KGInfo.pszNotesEmail		= NULL;
	KGInfo.pszUserID			= NULL;
	KGInfo.pszExchangeUserID	= NULL;

	KGInfo.bIsExchangeUser = PGPclIsExchangeServerUser ();

	// Determine which bitmap will be displayed in the wizard
	hDC = GetDC (NULL);	 // DC for desktop
	iNumBits = GetDeviceCaps (hDC, BITSPIXEL) * GetDeviceCaps (hDC, PLANES);
	ReleaseDC (NULL, hDC);

	if (iNumBits <= 1)
		iBitmap = IDB_KEYWIZ1;
	else if (iNumBits <= 4)
		iBitmap = IDB_KEYWIZ4;
	else
		iBitmap = IDB_KEYWIZ8;

	KGInfo.hbitmapIntro = CLLoadResourceBitmap (g_hInst,
			MAKEINTRESOURCE (iBitmap), &KGInfo.hpaletteIntro);
	KGInfo.hbitmapDone = CLLoadResourceBitmap (g_hInst,
			MAKEINTRESOURCE (iBitmap), &KGInfo.hpaletteDone);

	// load special fonts
	KGInfo.hfontIntro = CLLoadHeadingFont (FALSE);
	KGInfo.hfontInterior = CLLoadHeadingFont (TRUE);

	// load icon bitmaps
	KGInfo.hiconAlert = ImageList_GetIcon (
			g_hilClientLib, IDX_ALERT, ILD_TRANSPARENT);
	KGInfo.hiconDoing = ImageList_GetIcon (
			g_hilClientLib, IDX_ARROW, ILD_TRANSPARENT);
	KGInfo.hiconDone = ImageList_GetIcon (
			g_hilClientLib, IDX_CHECK, ILD_TRANSPARENT);
	KGInfo.hiconNotDone = ImageList_GetIcon (
			g_hilClientLib, IDX_ERROR, ILD_TRANSPARENT);

	// Set the values common to all pages
	for (iIndex=0; iIndex<NUM_WIZ_PAGES; iIndex++)
	{
		pspWiz[iIndex].dwSize		= sizeof(PROPSHEETPAGEW);
		pspWiz[iIndex].dwFlags		= PSP_DEFAULT;
		pspWiz[iIndex].hInstance	= g_hInst;
		pspWiz[iIndex].pszTemplate  = NULL;
		pspWiz[iIndex].hIcon		= NULL;
		pspWiz[iIndex].pszTitle		= NULL;
		pspWiz[iIndex].pfnDlgProc   = NULL;
		pspWiz[iIndex].lParam		= (LPARAM) &KGInfo;
		pspWiz[iIndex].pfnCallback  = NULL;
		pspWiz[iIndex].pcRefParent  = NULL;
	}

	// Set up the intro page
	pspWiz[KGWIZ_INTRO].pszTemplate = MAKEINTRESOURCEW(IDD_KEYWIZ_INTRO);
	pspWiz[KGWIZ_INTRO].pfnDlgProc = sKeyWizardIntroDlgProc;

	// Set up the name page
	pspWiz[KGWIZ_NAME].pszTemplate = MAKEINTRESOURCEW(IDD_KEYWIZ_NAME);
	pspWiz[KGWIZ_NAME].pfnDlgProc = sKeyWizardNameDlgProc;

	// Set up the expert page
	pspWiz[KGWIZ_EXPERT].pszTemplate = MAKEINTRESOURCEW(IDD_KEYWIZ_EXPERT);
	pspWiz[KGWIZ_EXPERT].pfnDlgProc = sKeyWizardExpertDlgProc;

	// Set up the expert page
	pspWiz[KGWIZ_EXCHANGE].pszTemplate = MAKEINTRESOURCEW(IDD_KEYWIZ_EXCHANGEID);
	pspWiz[KGWIZ_EXCHANGE].pfnDlgProc = sKeyWizardExchangeDlgProc;

	// Set up the corporate options page
	pspWiz[KGWIZ_CORPOPTS].pszTemplate = MAKEINTRESOURCEW(IDD_KEYWIZ_CORPOPTS);
	pspWiz[KGWIZ_CORPOPTS].pfnDlgProc = sKeyWizardCorpOptsDlgProc;

	// Set up the passphrase page
	pspWiz[KGWIZ_PHRASE].pszTemplate = MAKEINTRESOURCEW(IDD_KEYWIZ_PASSPHRASE);
	pspWiz[KGWIZ_PHRASE].pfnDlgProc = sKeyWizardPassphraseDlgProc;

	// Set up the bad passphrase page
	pspWiz[KGWIZ_BADPHRASE].pszTemplate =
									MAKEINTRESOURCEW(IDD_KEYWIZ_BADPHRASE);
	pspWiz[KGWIZ_BADPHRASE].pfnDlgProc = sKeyWizardBadPassphraseDlgProc;

	// Set up the key reconstruction page
	pspWiz[KGWIZ_RECON].pszTemplate = MAKEINTRESOURCEW(IDD_KEYWIZ_RECON);
	pspWiz[KGWIZ_RECON].pfnDlgProc = sKeyWizardReconDlgProc;

	// Set up the entropy page
	pspWiz[KGWIZ_ENTROPY].pszTemplate = MAKEINTRESOURCEW(IDD_KEYWIZ_RANDOBITS);
	pspWiz[KGWIZ_ENTROPY].pfnDlgProc = sKeyWizardRandobitsDlgProc;

	// Set up the progress dialog
	pspWiz[KGWIZ_PROGRESS].pszTemplate = MAKEINTRESOURCEW(IDD_KEYWIZ_PROGRESS);
	pspWiz[KGWIZ_PROGRESS].pfnDlgProc = sKeyWizardProgressDlgProc;

	// Set up the done page
	pspWiz[KGWIZ_DONE].pszTemplate = MAKEINTRESOURCEW(IDD_KEYWIZ_DONE);
	pspWiz[KGWIZ_DONE].pfnDlgProc = sKeyWizardDoneDlgProc;

	// Create the header
	pshWiz.dwSize		= sizeof(PROPSHEETHEADERW);
	pshWiz.dwFlags		= PSH_WIZARD | PSH_PROPSHEETPAGE;
	pshWiz.hwndParent   = hwndParent;
	pshWiz.hInstance	= g_hInst;
	pshWiz.hIcon		= NULL;
	pshWiz.pszCaption   = NULL;
	pshWiz.nPages		= NUM_WIZ_PAGES;
	pshWiz.nStartPage   = KGWIZ_INTRO;
	pshWiz.ppsp			= pspWiz;
	pshWiz.pfnCallback  = NULL;

	// find the user full name and email address
	sGetUserInfo (&KGInfo.pszFullName, &KGInfo.pszEmail);

	sInstallEntropyHooks ();
	sInstallSecurityHooks ();
	s_bHooksInstalled = TRUE;

	if (IsntPGPError (err) && sValidateConfiguration (&KGInfo))
	{
		// get the Lotus Notes address, if so configured
		if (KGInfo.bAddNotesUserID)
		{
			sGetNotesUserInfo (
					&KGInfo.pszNotesFullName, &KGInfo.pszNotesEmail);
		}

		// Execute the Wizard - doesn't return until Cancel or Save
		PropertySheetW(&pshWiz);
	}

	sUninstallSecurityHooks ();
	sUninstallEntropyHooks ();
	s_bHooksInstalled = FALSE;

	s_hwndCollectEntropy = NULL;

	// Free allocated memory and objects

	if (KGInfo.pszPassPhrase)
	{
		secFree (KGInfo.pszPassPhrase);
		KGInfo.pszPassPhrase = NULL;
	}

	if (KGInfo.pszFullName)
	{
		clFree (KGInfo.pszFullName);
		KGInfo.pszFullName = NULL;
	}

	if (KGInfo.pszEmail)
	{
		clFree (KGInfo.pszEmail);
		KGInfo.pszEmail = NULL;
	}

	if (KGInfo.pszNotesFullName)
	{
		clFree (KGInfo.pszNotesFullName);
		KGInfo.pszNotesFullName = NULL;
	}

	if (KGInfo.pszNotesEmail)
	{
		clFree (KGInfo.pszNotesEmail);
		KGInfo.pszNotesEmail = NULL;
	}

	if (KGInfo.pszExchangeUserID)
	{
		clFree (KGInfo.pszExchangeUserID);
		KGInfo.pszExchangeUserID = NULL;
	}

	if (KGInfo.pszUserID)
	{
		clFree (KGInfo.pszUserID);
		KGInfo.pszUserID = NULL;
	}

	DeleteObject (KGInfo.hbitmapIntro);
	DeleteObject (KGInfo.hbitmapDone);
	DeleteObject (KGInfo.hfontIntro);
	DeleteObject (KGInfo.hfontInterior);

	DestroyIcon (KGInfo.hiconAlert);
	DestroyIcon (KGInfo.hiconDoing);
	DestroyIcon (KGInfo.hiconDone);
	DestroyIcon (KGInfo.hiconNotDone);

	if (!KGInfo.bFinishSelected)
	{
		if (PGPKeyDBObjRefIsValid (KGInfo.keyNew))
			PGPclErrorBox (hwndParent, PGPDeleteKeyDBObj (KGInfo.keyNew));
		KGInfo.iFinalResult = 0;
	}

	if (KGInfo.iFinalResult & kPGPclKeyCreate_NewKey)
		PGPFlushKeyDB (keydb);

	if (pkeyNew)
	{
		*pkeyNew = KGInfo.keyNew;
/*
		PGPclDiskGenerationWizard (
			KGInfo.context,
			KGInfo.hwndParent,
			KGInfo.keydbMain,
			KGInfo.keyNew);*/
	}

	if (puFlags)
		*puFlags = KGInfo.iFinalResult;

	if (!PGPtlsContextRefIsValid (tlscontext))
		PGPFreeTLSContext (KGInfo.tlscontext);

	return kPGPError_NoErr;
}

//	______________________________________________
//
//  Create wizard data structures and call PropertySheet
//  to actually create wizard

static PGPError
sKeyGenerationWizard (
	BOOL				bRecenter,
	PGPContextRef		context,
	PGPtlsContextRef	tlscontext,
	HWND				hwndParent,
	PGPKeyDBRef			keydb,
	PGPKeyDBObjRef*		pkeyNew,
	PGPUInt32*			puFlags)
{
	PGPError			err					= kPGPError_NoErr;

	KEYGENINFO			KGInfo;
	PROPSHEETPAGE		pspWiz[NUM_WIZ_PAGES];
	PROPSHEETHEADER		pshWiz;
	INT					iIndex;
	INT					iNumBits, iBitmap;
	HDC					hDC;

	//if unicode is supported in OS thru and thru (post NT)
	if (g_bUnicodeOS)
	{
		return sKeyGenerationWizardW( bRecenter, context, tlscontext, 
				hwndParent, keydb, pkeyNew, puFlags );//do the unicode wizard
	}

	// intialize windows
	s_hwndCollectEntropy = NULL;

	// Set defaults
	if (PGPContextRefIsValid (context))
		KGInfo.context = context;
	else
		KGInfo.context = g_context;

	if (PGPtlsContextRefIsValid (tlscontext))
		KGInfo.tlscontext = tlscontext;
	else
	{
		err = PGPNewTLSContext (KGInfo.context, &KGInfo.tlscontext);
		if (IsPGPError (err))
			return err;
	}

	KGInfo.bRecenter			= bRecenter;
	KGInfo.uNumTasks			= 0;
	KGInfo.bExpertMode			= FALSE;
	KGInfo.bDoSend				= FALSE;
	KGInfo.bDoCertRequest		= FALSE;
	KGInfo.bDoReconstruction	= FALSE;
	KGInfo.bLookupExchangeUserID= FALSE;
	KGInfo.bFinishSelected		= FALSE;
	KGInfo.keydbMain			= keydb;
	KGInfo.keyNew				= kInvalidPGPKeyDBObjRef;

	KGInfo.hwndParent			= hwndParent;
	KGInfo.hwndWiz				= NULL;
	KGInfo.hpaletteIntro		= NULL;
	KGInfo.hpaletteDone			= NULL;

	KGInfo.pszPassPhrase		= NULL;
	KGInfo.pszFullName			= NULL;
	KGInfo.pszEmail				= NULL;
	KGInfo.pszNotesFullName		= NULL;
	KGInfo.pszNotesEmail		= NULL;
	KGInfo.pszUserID			= NULL;
	KGInfo.pszExchangeUserID	= NULL;

	KGInfo.bIsExchangeUser = PGPclIsExchangeServerUser ();

	// Determine which bitmap will be displayed in the wizard
	hDC = GetDC (NULL);	 // DC for desktop
	iNumBits = GetDeviceCaps (hDC, BITSPIXEL) * GetDeviceCaps (hDC, PLANES);
	ReleaseDC (NULL, hDC);

	if (iNumBits <= 1)
		iBitmap = IDB_KEYWIZ1;
	else if (iNumBits <= 4)
		iBitmap = IDB_KEYWIZ4;
	else
		iBitmap = IDB_KEYWIZ8;

	KGInfo.hbitmapIntro = CLLoadResourceBitmap (g_hInst,
			MAKEINTRESOURCE (iBitmap), &KGInfo.hpaletteIntro);
	KGInfo.hbitmapDone = CLLoadResourceBitmap (g_hInst,
			MAKEINTRESOURCE (iBitmap), &KGInfo.hpaletteDone);

	// load special fonts
	KGInfo.hfontIntro = CLLoadHeadingFont (FALSE);
	KGInfo.hfontInterior = CLLoadHeadingFont (TRUE);

	// load icon bitmaps
	KGInfo.hiconAlert = ImageList_GetIcon (
			g_hilClientLib, IDX_ALERT, ILD_TRANSPARENT);
	KGInfo.hiconDoing = ImageList_GetIcon (
			g_hilClientLib, IDX_ARROW, ILD_TRANSPARENT);
	KGInfo.hiconDone = ImageList_GetIcon (
			g_hilClientLib, IDX_CHECK, ILD_TRANSPARENT);
	KGInfo.hiconNotDone = ImageList_GetIcon (
			g_hilClientLib, IDX_ERROR, ILD_TRANSPARENT);

	// Set the values common to all pages
	for (iIndex=0; iIndex<NUM_WIZ_PAGES; iIndex++)
	{
		pspWiz[iIndex].dwSize		= sizeof(PROPSHEETPAGE);
		pspWiz[iIndex].dwFlags		= PSP_DEFAULT;
		pspWiz[iIndex].hInstance	= g_hInst;
		pspWiz[iIndex].pszTemplate  = NULL;
		pspWiz[iIndex].hIcon		= NULL;
		pspWiz[iIndex].pszTitle		= NULL;
		pspWiz[iIndex].pfnDlgProc   = NULL;
		pspWiz[iIndex].lParam		= (LPARAM) &KGInfo;
		pspWiz[iIndex].pfnCallback  = NULL;
		pspWiz[iIndex].pcRefParent  = NULL;
	}

	// Set up the intro page
	pspWiz[KGWIZ_INTRO].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_INTRO);
	pspWiz[KGWIZ_INTRO].pfnDlgProc = sKeyWizardIntroDlgProc;

	// Set up the name page
	pspWiz[KGWIZ_NAME].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_NAME);
	pspWiz[KGWIZ_NAME].pfnDlgProc = sKeyWizardNameDlgProc;

	// Set up the expert page
	pspWiz[KGWIZ_EXPERT].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_EXPERT);
	pspWiz[KGWIZ_EXPERT].pfnDlgProc = sKeyWizardExpertDlgProc;

	// Set up the expert page
	pspWiz[KGWIZ_EXCHANGE].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_EXCHANGEID);
	pspWiz[KGWIZ_EXCHANGE].pfnDlgProc = sKeyWizardExchangeDlgProc;

	// Set up the corporate options page
	pspWiz[KGWIZ_CORPOPTS].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_CORPOPTS);
	pspWiz[KGWIZ_CORPOPTS].pfnDlgProc = sKeyWizardCorpOptsDlgProc;

	// Set up the passphrase page
	pspWiz[KGWIZ_PHRASE].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_PASSPHRASE);
	pspWiz[KGWIZ_PHRASE].pfnDlgProc = sKeyWizardPassphraseDlgProc;

	// Set up the bad passphrase page
	pspWiz[KGWIZ_BADPHRASE].pszTemplate =
									MAKEINTRESOURCE(IDD_KEYWIZ_BADPHRASE);
	pspWiz[KGWIZ_BADPHRASE].pfnDlgProc = sKeyWizardBadPassphraseDlgProc;

	// Set up the key reconstruction page
	pspWiz[KGWIZ_RECON].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_RECON);
	pspWiz[KGWIZ_RECON].pfnDlgProc = sKeyWizardReconDlgProc;

	// Set up the entropy page
	pspWiz[KGWIZ_ENTROPY].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_RANDOBITS);
	pspWiz[KGWIZ_ENTROPY].pfnDlgProc = sKeyWizardRandobitsDlgProc;

	// Set up the progress dialog
	pspWiz[KGWIZ_PROGRESS].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_PROGRESS);
	pspWiz[KGWIZ_PROGRESS].pfnDlgProc = sKeyWizardProgressDlgProc;

	// Set up the done page
	pspWiz[KGWIZ_DONE].pszTemplate = MAKEINTRESOURCE(IDD_KEYWIZ_DONE);
	pspWiz[KGWIZ_DONE].pfnDlgProc = sKeyWizardDoneDlgProc;

	// Create the header
	pshWiz.dwSize		= sizeof(PROPSHEETHEADER);
	pshWiz.dwFlags		= PSH_WIZARD | PSH_PROPSHEETPAGE;
	pshWiz.hwndParent   = hwndParent;
	pshWiz.hInstance	= g_hInst;
	pshWiz.hIcon		= NULL;
	pshWiz.pszCaption   = NULL;
	pshWiz.nPages		= NUM_WIZ_PAGES;
	pshWiz.nStartPage   = KGWIZ_INTRO;
	pshWiz.ppsp			= pspWiz;
	pshWiz.pfnCallback  = NULL;

	// find the user full name and email address
	sGetUserInfo (&KGInfo.pszFullName, &KGInfo.pszEmail);

	sInstallEntropyHooks ();
	sInstallSecurityHooks ();
	s_bHooksInstalled = TRUE;

	if (IsntPGPError (err) && sValidateConfiguration (&KGInfo))
	{
		// get the Lotus Notes address, if so configured
		if (KGInfo.bAddNotesUserID)
		{
			sGetNotesUserInfo (
					&KGInfo.pszNotesFullName, &KGInfo.pszNotesEmail);
		}

		// Execute the Wizard - doesn't return until Cancel or Save
		PropertySheet(&pshWiz);
	}

	sUninstallSecurityHooks ();
	sUninstallEntropyHooks ();
	s_bHooksInstalled = FALSE;

	s_hwndCollectEntropy = NULL;

	// Free allocated memory and objects

	if (KGInfo.pszPassPhrase)
	{
		secFree (KGInfo.pszPassPhrase);
		KGInfo.pszPassPhrase = NULL;
	}

	if (KGInfo.pszFullName)
	{
		clFree (KGInfo.pszFullName);
		KGInfo.pszFullName = NULL;
	}

	if (KGInfo.pszEmail)
	{
		clFree (KGInfo.pszEmail);
		KGInfo.pszEmail = NULL;
	}

	if (KGInfo.pszNotesFullName)
	{
		clFree (KGInfo.pszNotesFullName);
		KGInfo.pszNotesFullName = NULL;
	}

	if (KGInfo.pszNotesEmail)
	{
		clFree (KGInfo.pszNotesEmail);
		KGInfo.pszNotesEmail = NULL;
	}

	if (KGInfo.pszExchangeUserID)
	{
		clFree (KGInfo.pszExchangeUserID);
		KGInfo.pszExchangeUserID = NULL;
	}

	if (KGInfo.pszUserID)
	{
		clFree (KGInfo.pszUserID);
		KGInfo.pszUserID = NULL;
	}

	DeleteObject (KGInfo.hbitmapIntro);
	DeleteObject (KGInfo.hbitmapDone);
	DeleteObject (KGInfo.hfontIntro);
	DeleteObject (KGInfo.hfontInterior);

	DestroyIcon (KGInfo.hiconAlert);
	DestroyIcon (KGInfo.hiconDoing);
	DestroyIcon (KGInfo.hiconDone);
	DestroyIcon (KGInfo.hiconNotDone);

	if (!KGInfo.bFinishSelected)
	{
		if (PGPKeyDBObjRefIsValid (KGInfo.keyNew))
			PGPclErrorBox (hwndParent, PGPDeleteKeyDBObj (KGInfo.keyNew));
		KGInfo.iFinalResult = 0;
	}

	if (KGInfo.iFinalResult & kPGPclKeyCreate_NewKey)
		PGPFlushKeyDB (keydb);

	if (pkeyNew)
	{
		*pkeyNew = KGInfo.keyNew;
/*
		PGPclDiskGenerationWizard (
			KGInfo.context,
			KGInfo.hwndParent,
			KGInfo.keydbMain,
			KGInfo.keyNew);*/
	}

	if (puFlags)
		*puFlags = KGInfo.iFinalResult;

	if (!PGPtlsContextRefIsValid (tlscontext))
		PGPFreeTLSContext (KGInfo.tlscontext);

	return kPGPError_NoErr;
}


//	______________________________________________
//
//  Create wizard data structures and call PropertySheet
//  to actually create wizard

PGPError PGPclExport
PGPclKeyGenerationWizard (
	PGPContextRef		context,
	PGPtlsContextRef	tlscontext,
	HWND				hwndParent,
	PGPKeyDBRef			keydb,
	PGPKeyDBObjRef*		pkeyNew,
	PGPUInt32*			puFlags)
{
	return sKeyGenerationWizard (FALSE, context,
			tlscontext, hwndParent, keydb, pkeyNew, puFlags);
}


PGPError PGPclExport WINAPI
PGPclRebootKeyGeneration (
		HWND	hwndParent,
		BOOL	bCheckForKeyPairs)
{
	PGPContextRef		context			= kInvalidPGPContextRef;
	PGPtlsContextRef	tlscontext		= kInvalidPGPtlsContextRef;
	PGPKeyDBRef			keydb			= kInvalidPGPKeyDBRef;
	PGPError			err				= kPGPError_NoErr;
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	PGPBoolean			bKeyGenEnabled	= FALSE;
	BOOL				bRunWizard		= FALSE;
	BOOL				bLocalMode		= FALSE;
	BOOL				bRunDiskWizard	= FALSE;
	PGPKeyID			keyid;

	// initialize SDK and clientlib
	err = PGPclInitLibrary (&context, 0);
	if (err == kPGPError_RPCFailed)
	{
		PGPclCloseLibrary ();
		err = PGPclInitLibrary (&context, kPGPclUseLocalSDKMode); CKERR;
		bLocalMode = TRUE;
	}

	err = PGPNewTLSContext (context, &tlscontext); CKERR;
	err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
	PGPGetPrefBoolean (prefref, 
			kPGPPrefAllowKeyGeneration, &bKeyGenEnabled);

	if (bKeyGenEnabled)
	{
		PGPBoolean		bPreset, bOnToken;

		PGPGetPrefBoolean (prefref, 
				kPGPPrefPresetKeyGeneration, &bPreset);
		PGPGetPrefBoolean (prefref, 
				kPGPPrefGenerateKeysOnSmartCard, &bOnToken);

		if (!bLocalMode || !bPreset || !bOnToken)
		{
			// flag that we've run the wizard once
			err = PGPSetPrefBoolean (
					prefref, kPGPPrefFirstKeyGenerated, TRUE);
			if (IsntPGPError (err))
				PGPclFlushClientLibPrefs (prefref, NULL);

			if(!PGPlnIsAuthorized())
			{
				PGPclEnterLicense(g_context,NULL);
			}

			// load keyrings
			err = PGPclOpenDefaultKeyrings (context,
					kPGPOpenKeyDBFileOptions_Mutable, &keydb); CKERR;

			// check for secret keys
			if (bCheckForKeyPairs)
			{
				if (!CLCheckForSecretKeys (PGPPeekKeyDBRootKeySet (keydb)))
					bRunWizard = TRUE;
			}
			else
				bRunWizard = TRUE;
		}
	}

	if (bRunWizard)
	{
		PGPKeyDBObjRef	keyNew		= kInvalidPGPKeyDBObjRef;

		sKeyGenerationWizard (TRUE, context, tlscontext,
				hwndParent, keydb, &keyNew, NULL);

		// set the new key as the default key
		if (PGPKeyDBObjRefIsValid (keyNew))
		{
			if (IsntPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
			{
				err = PGPGetKeyID (keyNew, &keyid);

				if (IsntPGPError (err))
				{
					bRunDiskWizard=TRUE;

					err = PGPSetPrefData (prefref,
							kPGPPrefDefaultKeyID, sizeof(PGPKeyID), &keyid);
				}

				if (IsntPGPError (err))
					err = PGPclFlushClientLibPrefs (prefref, NULL);
			}
		}
	}

	// cleanup SDK
done:
	if (PGPKeyDBRefIsValid (keydb))
		PGPFreeKeyDB (keydb);
	if (PGPtlsContextRefIsValid (tlscontext))
		PGPFreeTLSContext (tlscontext);

	if(bRunDiskWizard)
	{
		PGPclDiskGenerationWizard (
				context,
				hwndParent,
				&keyid);
	}

	PGPclCloseLibrary ();

	return err;
}

//	______________________________________________
//
//  Convenience function for installer -- do not call
//	from "normal" code

PGPError PGPclExport WINAPI
PGPclInstallerKeyGeneration (
		HWND	hwndParent,
		BOOL	bCheckForKeyPairs)
{
	PGPContextRef		context			= kInvalidPGPContextRef;
	PGPError			err				= kPGPError_NoErr;
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	PGPBoolean			bDiskCreate		= FALSE;
	BOOL				bRunWizard		= TRUE;
	BOOL				bLocalMode		= FALSE;

	// initialize SDK and clientlib
	err = PGPclInitLibrary (&context, 0);
	if (err == kPGPError_RPCFailed)
	{
		PGPclCloseLibrary ();
		err = PGPclInitLibrary (&context, kPGPclUseLocalSDKMode); CKERR;
		bLocalMode = TRUE;
	}

	err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
	PGPGetPrefBoolean (prefref, 
			kPGPPrefDiskCreate, &bDiskCreate);

	// Don't run the wizard from the installer if we need
	// to also create a PGPdisk... wait for it to happen
	// on reboot!
	if(bDiskCreate)
		bRunWizard=FALSE;

	if(bRunWizard)
		err=PGPclRebootKeyGeneration(hwndParent,bCheckForKeyPairs);

done:
	PGPclCloseLibrary ();

	return err;
}
