/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	CLimport.c - selective import dialog

	$Id: CLimport.c,v 1.17 2002/08/06 20:09:19 dallen Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "PGPklx.h"

// system header files
#include <zmouse.h>

// External globals
extern HINSTANCE	g_hInst;
extern UINT			g_uMouseWheelMessage;

typedef struct {
	PGPContextRef		context;
	PGPtlsContextRef	tlscontext;
	LPSTR				pszPrompt;
	PGPKeySetRef		keysetToDisplay;
	PGPKeyDBRef			keydbToAddTo;
	PGPKeySetRef		keysetSelected;
	PGPKeySetRef		keysetAdded;
	HKEYLIST			hKL;
	PGPclKeyListConfig	klConfig;
	UINT				uFlags;
	HWND				hwndTree;
	INT					iyOffset;
	INT					iyTop;
} QUERYSTRUCT;

#define IDC_QUERYTREELIST		0x0101
#define BUTTONOFFSETY			10
#define DIALOGMINWIDTH			400

#define BOTTOM_BUTTON_OFFSET	10
#define BUTTON_SPACING			8
#define HOR_OFFSET				4

// local globals
static DWORD aIds[] = {			// Help IDs
	IDOK,				IDH_PGPCL_IMPORT,
	IDC_SELECTALL,		IDH_PGPCL_IMPORTSELALL,
	IDC_UNSELECTALL,	IDH_PGPCL_IMPORTUNSELALL,
	IDC_QUERYTREELIST,	IDH_PGPCL_IMPORTLIST,
    0,0 
}; 

//	___________________________________________________
//
//  Put up appropriate context menu on basis of selection flags.
//  Called in response to right mouse click.
//
//	hWnd		= handle of parent window
//	uSelFlags	= bits indicate what types of objects are selected
//	x, y		= mouse position when right button clicked (screen coords)

static VOID 
sQueryContextMenu (
		HWND		hWnd, 
		HKEYLIST	hKL, 
		UINT		uSelFlags, 
		INT			x, 
		INT			y) 
{
	PKEYLIST	pKL				= (PKEYLIST)hKL;
	HMENU		hMC;
	HMENU		hMenuTrackPopup;

	switch (uSelFlags) {
	case kPGPclKeyList_Key :
		hMC = LoadMenu (g_hInst, MAKEINTRESOURCE (IDR_MENUIMPORT));
		break;

	default :
		hMC = NULL;
		break;
	}

	if (!hMC) 
		return;

  	hMenuTrackPopup = GetSubMenu (hMC, 0);
	if (KLMultipleSelected (pKL)) 
	{
		EnableMenuItem (hMenuTrackPopup, IDM_KEYPROPERTIES, 
				MF_BYCOMMAND|MF_GRAYED);
	}

	TrackPopupMenu (hMenuTrackPopup, TPM_LEFTALIGN|TPM_RIGHTBUTTON,
			x, y, 0, hWnd, NULL);

	DestroyMenu (hMC);
}

//	____________________________________
//
//  get number of wheel scroll lines and pass to treelist controls

static VOID
sUpdateWheelScrollLines (QUERYSTRUCT* pqs)
{
	UINT uLines;

	if (!SystemParametersInfo (SPI_GETWHEELSCROLLLINES, 0, &uLines, 0)) 
	{
		HWND hwnd = NULL;
		UINT umsg = 0;

		umsg = RegisterWindowMessage (MSH_SCROLL_LINES);
		hwnd = FindWindow (MSH_WHEELMODULE_CLASS, MSH_WHEELMODULE_TITLE);

		if (hwnd && umsg) 
			uLines = (UINT)SendMessage (hwnd, umsg, 0, 0);
	}

	TreeList_SetWheelScrollLines (pqs->hwndTree, uLines);
}

//	____________________________________
//
//	we're resizing -- move all the buttons 

static VOID
sMoveButtons (
		HWND	hwnd,
		WORD	wXsize,
		WORD	wYsize)
{
	HDWP	hdwp; 
	INT		iButtonWidth, iButtonHeight;
	RECT	rc;

	GetWindowRect (GetDlgItem (hwnd, IDOK), &rc);
	iButtonWidth = rc.right - rc.left;
	iButtonHeight = rc.bottom - rc.top;

	hdwp = BeginDeferWindowPos (4);

	// position the "cancel" button
	DeferWindowPos (hdwp, 
			GetDlgItem (hwnd, IDCANCEL), NULL, 
			wXsize - HOR_OFFSET - iButtonWidth, 
			wYsize - BOTTOM_BUTTON_OFFSET - iButtonHeight, 
			0, 0, SWP_NOSIZE|SWP_NOZORDER);

	// position the "OK" button
	DeferWindowPos (hdwp, 
			GetDlgItem (hwnd, IDOK), NULL, 
			wXsize - HOR_OFFSET - BUTTON_SPACING - 2*iButtonWidth, 
			wYsize - BOTTOM_BUTTON_OFFSET - iButtonHeight, 
			0, 0, SWP_NOSIZE|SWP_NOZORDER);

	// position the "unselectall" button
	DeferWindowPos (hdwp, 
			GetDlgItem (hwnd, IDC_UNSELECTALL), NULL, 
			wXsize - HOR_OFFSET - 2*BUTTON_SPACING - 3*iButtonWidth, 
			wYsize - BOTTOM_BUTTON_OFFSET - iButtonHeight, 
			0, 0, SWP_NOSIZE|SWP_NOZORDER);

	// position the "selectall" button
	DeferWindowPos (hdwp, 
			GetDlgItem (hwnd, IDC_SELECTALL), NULL, 
			wXsize - HOR_OFFSET - 3*BUTTON_SPACING - 4*iButtonWidth, 
			wYsize - BOTTOM_BUTTON_OFFSET - iButtonHeight, 
			0, 0, SWP_NOSIZE|SWP_NOZORDER);

	EndDeferWindowPos(hdwp);
}

//	___________________________________________________
//
//  Selective import dialog message procedure

static BOOL CALLBACK 
sImportDlgProc (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam, 
		LPARAM	lParam) 
{
	QUERYSTRUCT*	pqs;

	switch (uMsg) {

	case WM_INITDIALOG :
	{
		PGPPrefRef		prefref		= kInvalidPGPPrefRef;
		RECT			rc;
		INT				iy;
		POINT			pt;
		HWND			hwndControl;
		PGPBoolean		b;
		CHAR			sz[16];

		SetWindowLong (hwnd, GWL_USERDATA, lParam);
		pqs = (QUERYSTRUCT*)lParam;

		// change "OK" button to "Import"
		LoadString (g_hInst, IDS_IMPORT, sz, sizeof(sz));
		SetDlgItemText (hwnd, IDOK, sz);

		// set prompt text
		hwndControl = GetDlgItem (hwnd, IDC_PROMPT);
		SetWindowText (hwndControl, pqs->pszPrompt);

		// create keymanager window
		GetWindowRect (hwndControl, &rc);
		pt.x = rc.left;
		pt.y = rc.bottom;
		ScreenToClient (hwnd, &pt);
		pqs->iyTop = pt.y;

		hwndControl = GetDlgItem (hwnd, IDC_SELECTALL);
		GetWindowRect (hwndControl, &rc);
		pt.x = rc.left;
		pt.y = rc.top;
		ScreenToClient (hwnd, &pt);
		iy = pt.y;
		GetClientRect (hwnd, &rc);
		pqs->iyOffset = rc.bottom - (iy - BUTTONOFFSETY) + pqs->iyTop;
		pqs->hKL = PGPclKeyListCreate (pqs->context, pqs->tlscontext,
				hwnd, IDC_QUERYTREELIST, 0, pqs->iyTop, 
				rc.right-rc.left, rc.bottom-pqs->iyOffset, 
				kPGPclKeyList_ShowSelection, &pqs->hwndTree);

		// configure key manager window
		pqs->klConfig.keyserver.protocol = kPGPKeyServerProtocol_Invalid;
		pqs->klConfig.hwndStatusBar = NULL;
		pqs->klConfig.uShowColumns = 0;
		pqs->klConfig.uOptions = 
				kPGPclKeyList_SelectKeysOnly|kPGPclKeyList_ReadOnly;

		// get those options which are based on prefs
		if (IsPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
			EndDialog (hwnd, 0);

		PGPGetPrefBoolean (prefref, kPGPPrefDisplayMarginalValidity, &b);
		if (!b) 
			pqs->klConfig.uOptions |= kPGPclKeyList_GreenDotMode;

		PGPGetPrefBoolean (prefref, kPGPPrefMarginalIsInvalid, &b);
		if (b) 
			pqs->klConfig.uOptions |= kPGPclKeyList_MarginalIsInvalid;

		pqs->klConfig.uMask = kPGPclKeyList_ConfigureAll;
		PGPclKeyListConfigure (pqs->hKL, &pqs->klConfig);

		// initialize keymanager
		PGPclKeyListLoadKeys (pqs->hKL, 
				pqs->keydbToAddTo,
				pqs->keysetToDisplay);
		PGPclKeyListSetTree (pqs->hKL, 
				kPGPclSelectAll, kInvalidPGPKeyDBObjRef); 

		// initialize wheel scrolling
		sUpdateWheelScrollLines (pqs);

		// position buttons
		GetClientRect (hwnd, &rc);
		sMoveButtons (hwnd, (WORD)rc.right, (WORD)rc.bottom);

		// post message to sync keysets
		PostMessage (hwnd, WM_APP, 0, 0);

		return FALSE;
	}

	case WM_APP :
	{
		HCURSOR			hCursorOld;

		// this syncs the import keyset with the main one
		pqs = (QUERYSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		hCursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
		PGPCheckKeyRingSigs (pqs->keysetToDisplay, 
				pqs->keydbToAddTo, TRUE, NULL, NULL);
		PGPclKeyListReloadKeys (pqs->hKL, FALSE);

		PGPclKeyListSetTree (pqs->hKL, 
				kPGPclSelectAll, kInvalidPGPKeyDBObjRef); 

		SetCursor (hCursorOld);

		SetForegroundWindow (hwnd);
		EnableWindow (GetDlgItem (hwnd, IDOK), TRUE);
		SetFocus (pqs->hwndTree);
		break;
	}

	case WM_ACTIVATE :
		pqs = (QUERYSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		SetFocus (pqs->hwndTree);
		break;

	case WM_SIZING :
		{
			LPRECT prc = (LPRECT)lParam;
			switch (wParam) {
			case WMSZ_TOPLEFT :
			case WMSZ_LEFT :
			case WMSZ_BOTTOMLEFT :
				if ((prc->right - prc->left) < DIALOGMINWIDTH)
					prc->left = prc->right - DIALOGMINWIDTH;
				break;

			case WMSZ_TOPRIGHT :
			case WMSZ_RIGHT :
			case WMSZ_BOTTOMRIGHT :
				if ((prc->right - prc->left) < DIALOGMINWIDTH)
					prc->right = prc->left + DIALOGMINWIDTH;
				break;
			}
		}
		break;

	case WM_SIZE :
		pqs = (QUERYSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		// resize keymanager window
		SetWindowPos (pqs->hwndTree, NULL, 0, pqs->iyTop, 
				LOWORD(lParam), HIWORD(lParam) - pqs->iyOffset, 
				SWP_NOMOVE|SWP_NOZORDER);

		sMoveButtons (hwnd, LOWORD(lParam), HIWORD(lParam));
		return 0;

	case WM_SYSCOLORCHANGE :
		pqs = (QUERYSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		PostMessage (pqs->hwndTree, WM_SYSCOLORCHANGE, 0, 0);
		break;

	case WM_SETTINGCHANGE :
		pqs = (QUERYSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		if (wParam == SPI_SETWHEELSCROLLLINES)
			sUpdateWheelScrollLines (pqs);
		else 
			PostMessage (pqs->hwndTree, WM_SYSCOLORCHANGE, 0, 0);
		break;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aIds);

	case WM_NOTIFY :
		if (wParam == IDC_QUERYTREELIST) 
		{
			pqs = (QUERYSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
			// let keymanager have first whack at the notification
			PGPclKeyListDefaultNotificationProc (pqs->hKL, lParam);

			switch (((LPNM_TREELIST)lParam)->hdr.code) {
			// post the context menu
			case TLN_CONTEXTMENU :
				sQueryContextMenu (hwnd, pqs->hKL,
						((LPNM_TREELIST)lParam)->flags,
						((LPNM_TREELIST)lParam)->ptDrag.x,
						((LPNM_TREELIST)lParam)->ptDrag.y);
				break;

			// enable or disable the "import" button depending on whether
			// item(s) are selected or not
			case TLN_SELCHANGED :
				if ((((LPNM_TREELIST)lParam)->itemNew.hItem) ||
						(((LPNM_TREELIST)lParam)->flags & TLC_MULTIPLE)) 
				{
					EnableWindow (GetDlgItem (hwnd, IDOK), TRUE);
				}
				else
					EnableWindow (GetDlgItem (hwnd, IDOK), FALSE);
				break;

			default :
				break;
			}
		}
		break;

	case WM_COMMAND:
		pqs = (QUERYSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		switch (LOWORD(wParam)) {

		case IDOK :
		{
			PGPError		err;

			pqs->keysetSelected = kInvalidPGPKeySetRef;
			err = PGPclKeyListGetSelectedKeys (pqs->hKL, &pqs->keysetSelected);
			if (IsntPGPError (PGPclErrorBox (hwnd, err)))
			{
				err = PGPCopyKeys (pqs->keysetSelected, 
						pqs->keydbToAddTo, &pqs->keysetAdded);
				PGPclKeyListDestroy (pqs->hKL, FALSE);
				pqs->hKL = NULL;
				pqs->hwndTree = NULL;
				EndDialog (hwnd, err);
			}
			break;
		}

		case IDCANCEL :
			PGPclKeyListDestroy (pqs->hKL, FALSE);
			pqs->hKL = NULL;
			pqs->hwndTree = NULL;
			EndDialog (hwnd, kPGPError_UserAbort);
			break;

		case IDC_SELECTALL :
			PGPclKeyListSetTree (pqs->hKL, 
					kPGPclSelectAll, kInvalidPGPKeyDBObjRef); 
			EnableWindow (GetDlgItem (hwnd, IDOK), TRUE);
			SetFocus (pqs->hwndTree);
			break;

		case IDC_UNSELECTALL :
			PGPclKeyListSetTree (pqs->hKL, 
					kPGPclDeselectAll, kInvalidPGPKeyDBObjRef); 
			EnableWindow (GetDlgItem (hwnd, IDOK), FALSE);
			break;

		case IDM_KEYPROPERTIES :
		{
			PGPKeyDBObjRef	key;

			PGPclKeyListGetSelectionInfo (pqs->hKL, NULL, &key, NULL, NULL);
			PGPclKeyDBObjProperties (pqs->context, pqs->tlscontext, 
					hwnd, key, pqs->keydbToAddTo, 
					kPGPclProperties_ReadOnly|kPGPclProperties_Key, NULL);
			break;
		}

		default :
			break;
		}
		return TRUE;

	default :
		if (uMsg == g_uMouseWheelMessage) 
		{
			WORD iDelta;

			pqs = (QUERYSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
			iDelta = (WORD)wParam;
			
			SendMessage (pqs->hwndTree, 
								WM_MOUSEWHEEL, MAKEWPARAM (0, iDelta), 0);
			return FALSE;
		}
		break;
	}
	return FALSE;
}

//	____________________________________
//
//  Look for secret keys

BOOL 
CLCheckForSecretKeys (
		PGPKeySetRef	keyset) 
{
	PGPKeyIterRef	keyiter;
	PGPKeyDBObjRef	key;
	PGPBoolean		bSecret;
	BOOL			bSecretKeys;

	PGPNewKeyIterFromKeySet (keyset, &keyiter);

	bSecretKeys = FALSE;
	PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);

	while (key && !bSecretKeys) 
	{
		PGPGetKeyDBObjBooleanProperty (key, 
				kPGPKeyProperty_IsSecret, &bSecret);
		if (bSecret) 
			bSecretKeys = TRUE;

		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);
	}

	PGPFreeKeyIter (keyiter);

	return bSecretKeys;
}


//	___________________________________________________
//
//  Display and handle selective import dialog

PGPError PGPclExport
PGPclImportKeys (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeySetRef		keysetToDisplay,
		PGPKeyDBRef			keydbToAddTo,
		PGPUInt32			uFlags)
{
	BOOL			bCopied			= FALSE;
	QUERYSTRUCT		qs;
	CHAR			sz[256];
	PGPUInt32		uNumKeys;
	PGPError		err;
	PGPPrefRef		prefref;
	PGPBoolean		bMarginalIsInvalid;

	qs.keysetSelected = kInvalidPGPKeySetRef;

	if (!PGPKeyDBRefIsValid (keydbToAddTo))
		return kPGPError_BadParams;

	err = PGPclPeekClientLibPrefRefs (&prefref, NULL);
	if (IsPGPError (err))
		return err;

	PGPGetPrefBoolean (prefref, 
			kPGPPrefMarginalIsInvalid, &bMarginalIsInvalid);

	if (uFlags & kPGPclNoImportDialog)
	{
		err = PGPCopyKeys (keysetToDisplay, keydbToAddTo, &qs.keysetAdded);
		bCopied = TRUE;
	}

	else if (uFlags & kPGPclNoDialogForValidSingletons)
	{
		PGPCountKeys (keysetToDisplay, &uNumKeys);
		if (uNumKeys == 1)
		{
			PGPKeyIterRef	keyiter;
			PGPKeyDBObjRef	key;
			PGPUInt32		uVal;

			PGPCheckKeyRingSigs (
					keysetToDisplay, keydbToAddTo, TRUE, NULL, NULL);
			PGPNewKeyIterFromKeySet (keysetToDisplay, &keyiter);
			PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);
			PGPFreeKeyIter (keyiter);

			PGPGetPrimaryUserIDValidity (key, &uVal);
			uVal = CLConvertFromPGPValidity (uVal);

			if (bMarginalIsInvalid)
			{
				if (uVal >= PGPCL_VALIDITY_COMPLETE) 
				{
					err = PGPCopyKeys (
							keysetToDisplay, keydbToAddTo, &qs.keysetAdded);
					bCopied = TRUE;
				}
			}
			else
			{
				if (uVal > PGPCL_VALIDITY_MARGINAL) 
				{
					err = PGPCopyKeys (
							keysetToDisplay, keydbToAddTo, &qs.keysetAdded);
					bCopied = TRUE;
				}
			}
		}
	}

	if (bCopied)
	{
		PGPCalculateTrust (
				PGPPeekKeyDBRootKeySet (keydbToAddTo), kInvalidPGPKeyDBRef);

		if (!(uFlags & kPGPclNoTrustWarningMessage))
		{
			if (CLCheckForSecretKeys (keysetToDisplay))
			{
				PGPclMessageBox (hwndParent, IDS_CAPTIONINFO, 
						IDS_IMPORTEDSECRETKEYS, MB_OK|MB_ICONINFORMATION);
			}
		}
	}
	else
	{
		qs.hKL				= NULL;
		qs.context			= context;
		qs.tlscontext		= tlscontext;
		qs.keysetToDisplay	= keysetToDisplay;
		qs.keysetAdded		= kInvalidPGPKeySetRef;
		qs.keydbToAddTo		= keydbToAddTo;
		qs.uFlags			= uFlags;

		LoadString (g_hInst, IDS_QUERYADDPROMPT, sz, sizeof(sz));
		qs.pszPrompt = sz;

		err = DialogBoxParam (g_hInst, MAKEINTRESOURCE (IDD_QUERYADD), 
				hwndParent, sImportDlgProc, (LPARAM)&qs);

		if (IsntPGPError (err))
		{
			PGPCalculateTrust (
					PGPPeekKeyDBRootKeySet (keydbToAddTo), kInvalidPGPKeyDBRef);
		}

		if (PGPKeySetRefIsValid (qs.keysetSelected))
		{
			if (!(uFlags & kPGPclNoTrustWarningMessage))
			{
				if (CLCheckForSecretKeys (qs.keysetSelected))
				{
					PGPclMessageBox (hwndParent, IDS_CAPTIONINFO, 
							IDS_IMPORTEDSECRETKEYS, MB_OK|MB_ICONINFORMATION);
				}
			}

			PGPFreeKeySet (qs.keysetSelected);
		}
	}

	return err;
}

