/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	CLmisc.c - PGP ClientLib DLL miscellaneous routines

	$Id: CLmisc.c,v 1.137 2002/11/21 22:02:56 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "pgpImageList.h"
#include "pgpWin32IPC.h"

#include "pgpFileSpec.h"
#include "pgpMemoryIO.h"
#include "pgpWordWrap.h"
#include "pgpFeatures.h"
#include "pgpNetPrefs.h"
#include "pgpDebug.h"
#include "pgpNetHostUtils.h"
#include "pgpUnicode.h"
#include "pgpUnicodeWin32.h"
#include "UTF8Edit.h"

#include <ole2.h>
#include <shellapi.h>
#include <shlobj.h>
#include <htmlhelp.h>
#include <mmsystem.h>
#define COMPILE_MULTIMON_STUBS
#include <multimon.h>

#define CHICKENANDEGG() TRUE //PGPclEnterprise()

// constants
// turn on/off the storage of the randseed file name in the prefs
#define PGP_PREFS_RANDSEEDFILE	0

#ifndef CSIDL_COMMON_APPDATA
#define CSIDL_COMMON_APPDATA	0x0023
#endif
#ifndef CSIDL_LOCAL_APPDATA
#define CSIDL_LOCAL_APPDATA		0x001C
#endif
#ifndef CSIDL_FLAG_CREATE
#define CSIDL_FLAG_CREATE		0x8000
#endif

extern FARPROC g_fpSHGetFolderPath;


typedef struct {
	PGPContextRef			context;
	LPSTR					pszRemoteHost;
	PGPKeyDBObjRef			keyAuthenticating;
	PGPtlsCipherSuiteNum	tlsCipher;
	PGPKeyDBRef				keydbMain;
	CHAR					szName[kPGPMaxUserIDSize];
	PGPclAuthType			authtype;
	PGPValidity				validityThreshold;
} CONFIRMAUTHSTRUCT, *PCONFIRMAUTHSTRUCT;

// external globals
extern HINSTANCE g_hInst;

// local globals
static DWORD aIds[] = {			// Help IDs
    IDC_SERVERNAME,		IDH_PGPCLMISC_AUTHSERVERNAME,
    IDC_KEYNAME,		IDH_PGPCLMISC_AUTHKEYNAME,
    IDC_FINGERPRINT,	IDH_PGPCLMISC_AUTHKEYFINGERPRINT,
    IDC_VALIDITY,		IDH_PGPCLMISC_AUTHKEYVALIDITY,
    IDC_IMPORTKEY,		IDH_PGPCLMISC_AUTHIMPORTKEY,
	IDC_CERTIFICATE,	IDH_PGPCLMISC_AUTHCERTIFICATE,
	IDC_SIGNATURE,		IDH_PGPCLMISC_AUTHSIGNATURE,
	IDC_EXCHANGE,		IDH_PGPCLMISC_AUTHEXCHANGE,
	IDC_CIPHER,			IDH_PGPCLMISC_AUTHCIPHER,
	IDC_HASH,			IDH_PGPCLMISC_AUTHHASH,
	IDOK,				IDH_PGPCLMISC_AUTHCONFIRM,
    0,0
};

//	___________________________________________________
//
//	Internal memory allocation routines
//

VOID*
clAlloc (UINT uBytes)
{
	VOID* p;
	p = malloc (uBytes);
	if (p)
		memset (p, 0, uBytes);
	return p;
}


VOID
clFree (VOID* p)
{
	if (p)
		free (p);
}

//	___________________________________________________
//
//	Message box routine using string table resource IDs

LRESULT
PGPclMessageBox (
		 HWND	hWnd,
		 INT	iCaption,
		 INT	iMessage,
		 ULONG	ulFlags)
{
	CHAR szCaption [128];
	CHAR szMessage [512];

	LoadString (g_hInst, iCaption, szCaption, sizeof(szCaption));
	LoadString (g_hInst, iMessage, szMessage, sizeof(szMessage));

	ulFlags |= MB_SETFOREGROUND;
	return (MessageBox (hWnd, szMessage, szCaption, ulFlags));
}

//	___________________________________________________
//
//	convert SYSTEMTIME structure to number of days from today

PGPError PGPclExport
PGPclSystemTimeToDays (
		 SYSTEMTIME*	pst,
		 INT*			piDays)
{
	SYSTEMTIME	stToday;
	struct tm	tmstruct;
	time_t		timeToday;
	time_t		timeInQuestion;
	UINT		uDayToday;
	UINT		uDayInQuestion;

	pgpAssert (pst != NULL);
	pgpAssert (piDays != NULL);

	*piDays = -1;

	if (pst->wYear > 2037)
		return kPGPError_BadParams;

	GetLocalTime (&stToday);

	tmstruct.tm_mday = stToday.wDay;
	tmstruct.tm_mon = stToday.wMonth -1;
	tmstruct.tm_year = stToday.wYear -1900;
	tmstruct.tm_hour = 12;
	tmstruct.tm_min = 0;
	tmstruct.tm_sec = 0;
	tmstruct.tm_isdst = -1;

	timeToday = mktime (&tmstruct);
	if (timeToday == (time_t)-1)
		return kPGPError_BadParams;

	tmstruct.tm_mday = pst->wDay;
	tmstruct.tm_mon = pst->wMonth -1;
	tmstruct.tm_year = pst->wYear -1900;
	tmstruct.tm_hour = 12;
	tmstruct.tm_min = 0;
	tmstruct.tm_sec = 0;
	tmstruct.tm_isdst = -1;

	timeInQuestion = mktime (&tmstruct);
	if (timeInQuestion == (time_t)-1)
		return kPGPError_BadParams;

	uDayToday = timeToday / 86400;
	uDayInQuestion = timeInQuestion / 86400;

	*piDays = uDayInQuestion - uDayToday;

	return kPGPError_NoErr;
}


//	___________________________________________________
//
//	create logical palette from bitmap color table

static HPALETTE
sCreateDIBPalette (
		  LPBITMAPINFO	lpbmi,
		  LPINT			lpiNumColors)
{
	LPBITMAPINFOHEADER	lpbi;
	LPLOGPALETTE		lpPal;
	HANDLE				hLogPal;
	HPALETTE			hPal = NULL;
	INT					i;

	lpbi = (LPBITMAPINFOHEADER)lpbmi;
	if (lpbi->biBitCount <= 8)
		*lpiNumColors = (1 << lpbi->biBitCount);
	else
		*lpiNumColors = 0;  // No palette needed for 24 BPP DIB

	if (*lpiNumColors)
	{
		hLogPal = GlobalAlloc (GHND, sizeof (LOGPALETTE) +
				sizeof (PALETTEENTRY) * (*lpiNumColors));
		lpPal = (LPLOGPALETTE) GlobalLock (hLogPal);
		lpPal->palVersion = 0x300;
		lpPal->palNumEntries = *lpiNumColors;

		for (i = 0;  i < *lpiNumColors;  i++)
		{
			lpPal->palPalEntry[i].peRed   = lpbmi->bmiColors[i].rgbRed;
			lpPal->palPalEntry[i].peGreen = lpbmi->bmiColors[i].rgbGreen;
			lpPal->palPalEntry[i].peBlue  = lpbmi->bmiColors[i].rgbBlue;
			lpPal->palPalEntry[i].peFlags = 0;
		}
		hPal = CreatePalette (lpPal);
		GlobalUnlock (hLogPal);
		GlobalFree (hLogPal);
   }
   return hPal;
}

//	___________________________________________________
//
//	Load DIB bitmap and associated palette

HBITMAP
CLLoadResourceBitmap (
		HINSTANCE		hInstance,
		LPSTR			lpString,
		HPALETTE FAR*	lphPalette)
{
	HRSRC				hRsrc;
	HGLOBAL				hGlobal;
	HBITMAP				hBitmapFinal = NULL;
	LPBITMAPINFOHEADER	lpbi;
	HDC					hdc;
    INT					iNumColors;

	if (hRsrc = FindResource (hInstance, lpString, RT_BITMAP))
	{
		hGlobal = LoadResource (hInstance, hRsrc);
		lpbi = (LPBITMAPINFOHEADER)LockResource (hGlobal);

		hdc = GetDC(NULL);
		*lphPalette =  sCreateDIBPalette ((LPBITMAPINFO)lpbi, &iNumColors);
		if (*lphPalette)
		{
			SelectPalette (hdc,*lphPalette,FALSE);
			RealizePalette (hdc);
		}

		hBitmapFinal = CreateDIBitmap (hdc,
                   (LPBITMAPINFOHEADER)lpbi,
                   (LONG)CBM_INIT,
                   (LPSTR)lpbi + lpbi->biSize + iNumColors * sizeof(RGBQUAD),
                   (LPBITMAPINFO)lpbi,
                   DIB_RGB_COLORS );

		ReleaseDC (NULL,hdc);
		UnlockResource (hGlobal);
		FreeResource (hGlobal);
	}
	return (hBitmapFinal);
}


//	___________________________________________________
//
//	Broadcast reload message

VOID PGPclExport
PGPclNotifyKeyringChanges (
		PGPclChangeType	type,
		PGPUInt32		uParam)
{
	UINT uMessageID;

	uMessageID = RegisterWindowMessage (kPGPclReloadKeyRingMsg);
	PostMessage (HWND_BROADCAST, uMessageID, (WPARAM)type, uParam);
}

//	___________________________________________________
//
//	Broadcast prefs reload message

VOID PGPclExport
PGPclNotifyPrefsChanges (
		PGPUInt32		uParam)
{
	UINT uMessageID;

	uMessageID = RegisterWindowMessage (kPGPclReloadPrefsMsg);
	PostMessage (HWND_BROADCAST, uMessageID, 0, uParam);
}

//	___________________________________________________
//
//	Broadcast PGPnet prefs reload message

VOID PGPclExport
PGPclNotifyNetPrefsChanges (
		PGPUInt32 			uParam)
{
	UINT uMessageID;

	uMessageID = RegisterWindowMessage (kPGPclReloadNetPrefsMsg);
	PostMessage (HWND_BROADCAST, uMessageID, 0, uParam);
}

//	___________________________________________________
//
//	Broadcast messages indicating keyserver prefs may have changed

VOID PGPclExport
PGPclNotifyKeyserverPrefsChanges (
		PGPUInt32		uParam)
{
	UINT uMessageID;

	uMessageID = RegisterWindowMessage (kPGPclReloadKeyserverPrefsMsg);
	PostMessage (HWND_BROADCAST, uMessageID, 0, uParam);
}

//	___________________________________________________
//
//	add trailing slash

static PGPError
sFinalizePath (
		LPSTR			pszPath, 
		PGPUInt32		uLen)
{
	if (pszPath[lstrlen (pszPath) -1] == '\\')
		return kPGPError_NoErr;
	else
	{
		if (lstrlen (pszPath) < (INT)uLen)
		{
			lstrcat (pszPath, "\\");
			return kPGPError_NoErr;
		}
		else
			return kPGPError_OutputBufferTooSmall;
	}
}

//	___________________________________________________
//
//	get installed path from registry

static PGPError
sGetInstalledPath (
		LPSTR			pszPath, 
		PGPUInt32		uLen)
{
	PGPError	err						= kPGPError_FileNotFound;
	HKEY		hkey;
	LONG		lResult;
	DWORD		dwValueType, dwSize;
	CHAR		sz[256];

	PGPclGetPath (kPGPclRegistryKey, sz, sizeof(sz));
	lResult = RegOpenKeyEx (HKEY_LOCAL_MACHINE, sz, 0, KEY_READ, &hkey);

	if (lResult == ERROR_SUCCESS)
	{
		dwSize = uLen;
		lResult = RegQueryValueEx (hkey, 
				PGP_INSTALLPATHVALUE, 0, &dwValueType, pszPath, &dwSize);
		RegCloseKey (hkey);

		if (lResult == ERROR_SUCCESS)
			err = sFinalizePath (pszPath, uLen);
	}

	return err;
}


//	___________________________________________________
//
//	get desktop path from registry

static PGPError
sGetDesktopFolder (
		LPSTR			pszPath, 
		PGPUInt32		uLen)
{
	LONG	lResult;
	HKEY	hkey;
	DWORD	dwSize, dwValueType;

	lResult = RegOpenKeyEx (HKEY_CURRENT_USER, 
			"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\"
			"Shell Folders", 
			0, KEY_READ, &hkey);

	if (lResult == ERROR_SUCCESS) 
	{
		dwSize = uLen;
		lResult = RegQueryValueEx (hkey, "Desktop", 0, &dwValueType, 
			pszPath, &dwSize);

		return kPGPError_NoErr;
	}

	return kPGPError_FileNotFound;
}


#if PGP_DEBUG
//	____________________________________________________________
//
//	determine how if to run SDK backend or not

static BOOL
sUseLocalKeyrings (VOID)
{
	PGPBoolean	bLocal			= FALSE;
	HKEY		hkey;
	LONG		lResult;
	DWORD		dw, dwValueType, dwSize;

	lResult = RegOpenKeyEx (HKEY_LOCAL_MACHINE, 
			PGP_REGISTRYKEY, 0, KEY_READ, &hkey);

	if (lResult == ERROR_SUCCESS) 
	{
		dwSize = sizeof(DWORD);

		lResult = RegQueryValueEx (hkey, "PGPclForceLocalKeyrings", 
				0, &dwValueType, (LPBYTE)&dw, &dwSize);

		if (lResult == ERROR_SUCCESS)
		{
			if (dw == 1)
				bLocal = TRUE;
		}

		RegCloseKey (hkey);
	}

	return bLocal;
}

static VOID
sLocalKeyring (
		BOOL		bSet,
		INT			iFile,
		LPSTR		pszPath,
		INT			ilen)
{
	HKEY		hkey;
	LONG		lResult;
	DWORD		dwValueType, dwSize;
	CHAR		sz[256];

	lResult = RegOpenKeyEx (HKEY_LOCAL_MACHINE, 
			PGP_REGISTRYKEY, 0, KEY_READ|KEY_SET_VALUE, &hkey);

	switch (iFile) {
	case kPGPclPublicKeyringFile :
		lstrcpy (sz, "PGPclLocalPubring");
		break;

	case kPGPclPrivateKeyringFile :
		lstrcpy (sz, "PGPclLocalSecring");
		break;

	case kPGPclNetPublicKeyringFile :
		lstrcpy (sz, "PGPclLocalNetPubring");
		break;

	case kPGPclNetPrivateKeyringFile :
		lstrcpy (sz, "PGPclLocalNetSecring");
		break;
	}

	if (lResult == ERROR_SUCCESS) 
	{
		if (bSet)
		{
			lResult = RegSetValueEx (hkey, sz, 
					0, REG_SZ, (LPBYTE)pszPath, ilen);
		}
		else
		{
			dwSize = ilen;
			lResult = RegQueryValueEx (hkey, sz, 
					0, &dwValueType, (LPBYTE)pszPath, &dwSize);
		}

		RegCloseKey (hkey);
	}
}

#endif //PGP_DEBUG

//	___________________________________________________
//
//	get specified path 
//	note: folder paths include trailing '\'

PGPError PGPclExport
PGPclGetPath (
		PGPclPathIndex	index,
		LPSTR			pszPath, 
		PGPInt32		iLen)
{
	PGPError	err					= kPGPError_NoErr;
	BOOL		bLocalKeyrings		= FALSE;
	PGPPrefRef	prefref;
	CHAR		szPath[MAX_PATH];
	CHAR		sz[256];

	// initialize strings
	sz[0] = '\0';
	szPath[0] = '\0';

	// check parameters
	if (IsNull (pszPath))
		return kPGPError_BadParams;
	pszPath[0] = '\0';

	// make sure SHFOLDER is loaded
	if (IsNull (g_fpSHGetFolderPath))
	{
		err = CLLoadSHFolderGetPath (NULL);
		if (IsPGPError (err))
			return err;
	}

#if PGP_DEBUG
	bLocalKeyrings = sUseLocalKeyrings ();
#endif

	// return path string
	switch (index) {
	case kPGPclRegistryKey :
		lstrcpyn (pszPath, PGP_REGISTRYKEY, iLen);
		break;

	case kPGPclInstallFolder :
		err = sGetInstalledPath (pszPath, iLen);
		break;

	case kPGPclHelpFile :
		err = sGetInstalledPath (pszPath, iLen);
		if (IsntPGPError (err))
		{
			if ((lstrlen (pszPath) + lstrlen (PGP_HELPFILENAME) +1) < iLen)
				lstrcat (pszPath, PGP_HELPFILENAME);
			else
				err = kPGPError_OutputBufferTooSmall;
		}
		break;

	case kPGPclNetHelpFile :
		err = sGetInstalledPath (pszPath, iLen);
		if (IsntPGPError (err))
		{
			if ((lstrlen (pszPath) + lstrlen (PGP_NETHELPFILENAME) +1) < iLen)
				lstrcat (pszPath, PGP_NETHELPFILENAME);
			else
				err = kPGPError_OutputBufferTooSmall;
		}
		break;

	case kPGPclFireHelpFile :
		err = sGetInstalledPath (pszPath, iLen);
		if (IsntPGPError (err))
		{
			if ((lstrlen (pszPath) + lstrlen (PGP_FIREHELPFILENAME) +1) < iLen)
				lstrcat (pszPath, PGP_FIREHELPFILENAME);
			else
				err = kPGPError_OutputBufferTooSmall;
		}
		break;

	case kPGPclMailHelpFile :
		err = sGetInstalledPath (pszPath, iLen);
		if (IsntPGPError (err))
		{
			if ((lstrlen (pszPath) + lstrlen (PGP_MAILHELPFILENAME) +1) < iLen)
				lstrcat (pszPath, PGP_MAILHELPFILENAME);
			else
				err = kPGPError_OutputBufferTooSmall;
		}
		break;

	case kPGPclAdminHelpFile :
		err = sGetInstalledPath (pszPath, iLen);
		if (IsntPGPError (err))
		{
			if ((lstrlen (pszPath) + lstrlen (PGP_ADMINHELPFILENAME) +1) < iLen)
				lstrcat (pszPath, PGP_ADMINHELPFILENAME);
			else
				err = kPGPError_OutputBufferTooSmall;
		}
		break;

	case kPGPclDiskHelpFile :
		err = sGetInstalledPath (pszPath, iLen);
		if (IsntPGPError (err))
		{
			if ((lstrlen (pszPath) + lstrlen (PGP_DISKHELPFILENAME) +1) < iLen)
				lstrcat (pszPath, PGP_DISKHELPFILENAME);
			else
				err = kPGPError_OutputBufferTooSmall;
		}
		break;

	case kPGPclPGPkeysExeFile :
		err = sGetInstalledPath (pszPath, iLen);
		if (IsntPGPError (err))
		{
			if ((lstrlen (pszPath) + lstrlen (PGP_PGPKEYSAPPNAME) +1) < iLen)
				lstrcat (pszPath, PGP_PGPKEYSAPPNAME);
			else
				err = kPGPError_OutputBufferTooSmall;
		}
		break;

	case kPGPclPrefsFile :
	case kPGPclGroupsFile :
		err = kPGPError_NoErr;
		if (!SUCCEEDED (g_fpSHGetFolderPath (
				NULL, CSIDL_APPDATA|CSIDL_FLAG_CREATE, 
				NULL, 0, szPath)) ||
			(szPath[0] == '\0'))
		{
			err = sGetInstalledPath (szPath, iLen);
			if (IsntPGPError (err))
				err = sFinalizePath (szPath, sizeof(szPath));
		}
		else
		{
			err = sFinalizePath (szPath, sizeof(szPath));
			if (IsntPGPError (err))
			{
				if ((lstrlen (szPath) + 
						lstrlen (PGP_RELATIVEPATHNAME) +1) < iLen)
				{
					lstrcat (szPath, PGP_RELATIVEPATHNAME);
					err = kPGPError_NoErr;
				}
				else
					err = kPGPError_OutputBufferTooSmall;
			}
		}

		if (IsntPGPError (err))
		{
			if (index == kPGPclPrefsFile)
				lstrcpy (sz, PGP_PREFSFILENAME);
			else
				lstrcpy (sz, PGP_GROUPSFILENAME);

			if ((lstrlen (szPath) + lstrlen (sz) +1) < iLen)
			{
				lstrcpy (pszPath, szPath);
				lstrcat (pszPath, sz);
				err = kPGPError_NoErr;
			}
			else
				err = kPGPError_OutputBufferTooSmall;
		}
		break;

	case kPGPclNetPrefsFile :
	case kPGPclNetLogFile :
	case kPGPclNetLogEventDataFolder :
		err = kPGPError_NoErr;
		if (!SUCCEEDED (g_fpSHGetFolderPath (
				NULL, CSIDL_COMMON_APPDATA|CSIDL_FLAG_CREATE,
				NULL, 0, szPath)) ||
			(szPath[0] == '\0'))
		{
			err = sGetInstalledPath (szPath, iLen);
			if (IsntPGPError (err))
				err = sFinalizePath (szPath, sizeof(szPath));
		}
		else
		{
			err = sFinalizePath (szPath, sizeof(szPath));
			if (IsntPGPError (err))
			{
				if ((lstrlen (szPath) + 
						lstrlen (PGP_RELATIVEPATHNAME) +1) < iLen)
				{
					lstrcat (szPath, PGP_RELATIVEPATHNAME);
					err = kPGPError_NoErr;
				}
				else
					err = kPGPError_OutputBufferTooSmall;
			}
		}

		if (IsntPGPError (err))
		{
			if (index == kPGPclNetPrefsFile)
				lstrcpy (sz, PGP_NETPREFSFILENAME);
			else if (index == kPGPclNetLogFile)
				lstrcpy (sz, PGP_NETLOGFILENAME);
			else
				lstrcpy (sz, PGP_NETEVENTDATAFOLDER);

			if ((lstrlen (szPath) + lstrlen (sz) +1) < iLen)
			{
				lstrcpy (pszPath, szPath);
				lstrcat (pszPath, sz);
				err = kPGPError_NoErr;
			}
			else
				err = kPGPError_OutputBufferTooSmall;
		}
		break;

	case kPGPclDefaultRandseedFolder :
		if (SUCCEEDED (g_fpSHGetFolderPath (
				NULL, CSIDL_LOCAL_APPDATA|CSIDL_FLAG_CREATE, 
				NULL, 0, szPath)) &&
			(szPath[0] != '\0'))
		{
			err = sFinalizePath (szPath, sizeof(szPath));
			if (IsntPGPError (err))
			{
				err = kPGPError_OutputBufferTooSmall;

				if ((lstrlen (szPath) + 
						lstrlen (PGP_RELATIVEPATHNAME) +1) < iLen)
				{
					lstrcpy (pszPath, szPath);
					lstrcat (pszPath, PGP_RELATIVEPATHNAME);
					err = kPGPError_NoErr;
				}
			}
		}
		else if (SUCCEEDED (g_fpSHGetFolderPath (
					NULL, CSIDL_PERSONAL|CSIDL_FLAG_CREATE, 
					NULL, 0, szPath)) &&
				(szPath[0] != '\0'))
		{
			err = sFinalizePath (szPath, sizeof(szPath));
			if (IsntPGPError (err))
			{
				err = kPGPError_OutputBufferTooSmall;

				if ((lstrlen (szPath) + 
						lstrlen (PGP_PERSONALPATHNAME) +1) < iLen)
				{
					lstrcpy (pszPath, szPath);
					lstrcat (pszPath, PGP_PERSONALPATHNAME);
					err = kPGPError_NoErr;
				}
			}
		}
		else
			err = sGetInstalledPath (pszPath, iLen);
		break;

	case kPGPclDefaultKeyringFolder :
		err = kPGPError_FileNotFound;
		if (SUCCEEDED (g_fpSHGetFolderPath (
				NULL, CSIDL_PERSONAL|CSIDL_FLAG_CREATE, 
				NULL, 0, szPath)) &&
			(szPath[0] != '\0'))
		{
			err = sFinalizePath (szPath, sizeof(szPath));
			if (IsntPGPError (err))
			{
				if ((lstrlen (szPath) + 
						lstrlen (PGP_PERSONALPATHNAME) +1) < iLen)
				{
					lstrcpy (pszPath, szPath);
					lstrcat (pszPath, PGP_PERSONALPATHNAME);
					err = kPGPError_NoErr;
				}
				else
					err = kPGPError_OutputBufferTooSmall;
			}
		}
		else
		{
			err = sGetInstalledPath (pszPath, iLen);
			if (IsntPGPError (err))
			{
				if ((lstrlen (szPath) + 
						lstrlen (PGP_DEFAULTKEYRINGFOLDER) +1) < iLen)
				{
					lstrcpy (pszPath, szPath);
					lstrcat (pszPath, PGP_DEFAULTKEYRINGFOLDER);
					err = kPGPError_NoErr;
				}
				else
					err = kPGPError_OutputBufferTooSmall;
			}
		}
		break;

	case kPGPclDefaultOpenFolder :
	case kPGPclDefaultSaveAsFolder :
		err = kPGPError_FileNotFound;
		if (SUCCEEDED (g_fpSHGetFolderPath (
				NULL, CSIDL_PERSONAL|CSIDL_FLAG_CREATE, 
				NULL, 0, pszPath)) &&
			(pszPath[0] != '\0'))
		{
			err = sFinalizePath (pszPath, iLen);
		}
		else if (IsntPGPError (sGetDesktopFolder (pszPath, iLen)))
		{
			err = sFinalizePath (pszPath, iLen);
		}
		else
		{
			err = sGetInstalledPath (pszPath, iLen);
		}
		break;

	case kPGPclDesktopFolder :
		err = kPGPError_FileNotFound;
		if (IsntPGPError (sGetDesktopFolder (pszPath, iLen)))
		{
			err = sFinalizePath (pszPath, iLen);
		}
		break;

	case kPGPclPublicKeyringFile :
	case kPGPclPrivateKeyringFile :
		err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;

		if (index == kPGPclPublicKeyringFile)
		{
#if PGP_DEBUG
			if (bLocalKeyrings)
				sLocalKeyring (FALSE, kPGPclPublicKeyringFile, pszPath, iLen);
			else
#endif // PGP_DEBUG
			{
				err = PGPGetPrefStringBuffer (prefref, 
						kPGPPrefPublicKeyringFile, iLen, pszPath);
			}
		}
		else
		{
#if PGP_DEBUG
			if (bLocalKeyrings)
				sLocalKeyring (FALSE, kPGPclPrivateKeyringFile, pszPath, iLen);
			else
#endif // PGP_DEBUG
			{
				err = PGPGetPrefStringBuffer (prefref, 
						kPGPPrefPrivateKeyringFile, iLen, pszPath);
			}
		}

		// if file name is not a full path, then it is 
		// in the roaming folder (CSIDL_APPDATA)
		if (IsntPGPError (err) && 
			(strchr (pszPath, '\\') == NULL))
		{
			CHAR	sz2[256];

			lstrcpy (sz2, pszPath);

			err = kPGPError_FileNotFound;
			if (SUCCEEDED (g_fpSHGetFolderPath (
					NULL, CSIDL_APPDATA|CSIDL_FLAG_CREATE, 
					NULL, 0, szPath)) &&
				(szPath[0] != '\0'))
			{
				err = sFinalizePath (szPath, sizeof(szPath));
				if (IsntPGPError (err))
				{
					err = kPGPError_OutputBufferTooSmall;

					if ((lstrlen (szPath) + 
							lstrlen (PGP_RELATIVEPATHNAME) +1) < iLen)
					{
						lstrcpy (pszPath, szPath);
						lstrcat (pszPath, PGP_RELATIVEPATHNAME);
						err = kPGPError_NoErr;
					}
				}
			}
			else
				err = PGPclGetPath (kPGPclDefaultKeyringFolder, pszPath, iLen);

			if (IsntPGPError (err))
			{
				if ((lstrlen (pszPath) + lstrlen (sz2) +1) < iLen)
				{
					lstrcat (pszPath, sz2);
					err = kPGPError_NoErr;
				}
			}
		}
		break;

	case kPGPclNetPublicKeyringFile :
		err = PGPclPeekClientLibPrefRefs (NULL, &prefref); CKERR;
		err = PGPGetPrefStringBuffer (prefref, 
				kPGPNetPrefPublicKeyringFile, iLen, pszPath);
		break;

	case kPGPclNetPrivateKeyringFile :
		err = PGPclPeekClientLibPrefRefs (NULL, &prefref); CKERR;
		err = PGPGetPrefStringBuffer (prefref, 
				kPGPNetPrefPrivateKeyringFile, iLen, pszPath);
		break;

	case kPGPclRandseedFile :
#if PGP_PREFS_RANDSEEDFILE
		err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
		err = PGPGetPrefStringBuffer (prefref, 
				kPGPPrefRNGSeedFile, iLen, pszPath);
#else
		err = PGPclGetPath (kPGPclDefaultRandseedFolder, pszPath, iLen);
		if (IsntPGPError (err))
		{
			LoadString (g_hInst, IDS_DEFAULTRNGFILENAME, sz, sizeof(sz));
			if ((lstrlen (pszPath) + lstrlen (sz) +1) < iLen)
			{
				lstrcat (pszPath, sz);
				err = kPGPError_NoErr;
			}
		}
#endif // PGP_PREFS_RANDSEEDFILE
		break;

	case kPGPclDefaultKeysFile :
		err = sGetInstalledPath (pszPath, iLen);
		if (IsntPGPError (err))
		{
			if ((lstrlen (pszPath) + lstrlen (PGP_DEFAULTKEYSFILENAME) +1) < iLen)
				lstrcat (pszPath, PGP_DEFAULTKEYSFILENAME);
			else
				err = kPGPError_OutputBufferTooSmall;
		}
		break;

	default :
		err = kPGPError_BadParams;
	}

done:
	return err;
}


//	___________________________________________________
//
//	set specified path 
//	note: only keyring and randseed file names can be set

PGPError PGPclExport
PGPclSetPath (
		PGPPrefRef		prefref,
		PGPclPathIndex	index,
		LPSTR			pszPath)
{
	PGPError	err					= kPGPError_NoErr;
	BOOL		bLocalKeyrings		= FALSE;
	CHAR		szPath[MAX_PATH];

	// check parameters
	if (!PGPPrefRefIsValid (prefref))
		return kPGPError_BadParams;
	if (IsNull (pszPath))
		return kPGPError_BadParams;

	// make sure SHFOLDER is loaded
	if (IsNull (g_fpSHGetFolderPath))
	{
		err = CLLoadSHFolderGetPath (NULL);
		if (IsPGPError (err))
			return err;
	}

#if PGP_DEBUG
	bLocalKeyrings = sUseLocalKeyrings ();
#endif

	// set path string
	switch (index) {
	case kPGPclNetPublicKeyringFile :
#if PGP_DEBUG
		if (bLocalKeyrings)
		{
			sLocalKeyring (TRUE, kPGPclNetPublicKeyringFile, 
					pszPath, lstrlen (pszPath) +1);
		}
		else
#endif // PGP_DEBUG
		{
			err = PGPSetPrefString (prefref, 
					kPGPNetPrefPublicKeyringFile, pszPath);
		}
		break;

	case kPGPclNetPrivateKeyringFile :
#if PGP_DEBUG
		if (bLocalKeyrings)
		{
			sLocalKeyring (TRUE, kPGPclNetPrivateKeyringFile, 
					pszPath, lstrlen (pszPath) +1);
		}
		else
#endif // PGP_DEBUG
		{
			err = PGPSetPrefString (prefref, 
					kPGPNetPrefPrivateKeyringFile, pszPath);
		}
		break;

	case kPGPclRandseedFile :
#if PGP_PREFS_RANDSEEDFILE
		err = PGPSetPrefString (prefref, 
				kPGPPrefRNGSeedFile, pszPath);
#endif // PGP_PREFS_RANDSEEDFILE
		break;

	case kPGPclPublicKeyringFile :
	case kPGPclPrivateKeyringFile :
	{
		// if file is in the roaming folder, then strip the path
		// and save only the file name
		LPSTR	p = strrchr (pszPath, '\\');

		if (IsntNull (p))
		{
			INT		ncharsPath;

			ncharsPath = p - pszPath;

			if (ncharsPath <= 2)
				p = pszPath;
			else
			{
				err = kPGPError_FileNotFound;
				if (SUCCEEDED (g_fpSHGetFolderPath (
						NULL, CSIDL_APPDATA|CSIDL_FLAG_CREATE, 
						NULL, 0, szPath)))
				{
					err = sFinalizePath (szPath, sizeof(szPath));
					if (IsntPGPError (err))
					{
						err = kPGPError_OutputBufferTooSmall;

						if ((lstrlen (szPath) + 
								lstrlen (PGP_RELATIVEPATHNAME)+1) < 
								sizeof(szPath))
						{
							lstrcat (szPath, PGP_RELATIVEPATHNAME);

							if (strnicmp (szPath, pszPath, ncharsPath) == 0)
								p++;
							else
								p = pszPath;
						}
					}
				}
			}
		}

		if (index == kPGPclPublicKeyringFile)
		{
#if PGP_DEBUG
			if (bLocalKeyrings)
				sLocalKeyring (TRUE, kPGPclPublicKeyringFile, p, lstrlen (p) +1);
			else
#endif // PGP_DEBUG
			{
				err = PGPSetPrefString (prefref, 
						kPGPPrefPublicKeyringFile, p);
			}
		}
		else
		{
#if PGP_DEBUG
			if (bLocalKeyrings)
				sLocalKeyring (TRUE, kPGPclPrivateKeyringFile, p, lstrlen (p) +1);
			else
#endif // PGP_DEBUG
			{
				err = PGPSetPrefString (prefref, 
						kPGPPrefPrivateKeyringFile, p);
			}
		}
		break;
	}

	default :
		err = kPGPError_BadParams;
	}

	return err;
}


//	_________________________________________________
//
//	get RNG file name, initializing if necessary

PGPError
CLInitRNGSeedFilePref (
		PGPPrefRef		prefrefIn,
		LPSTR			pszPath, 
		PGPInt32		iLen)
{
	PGPError	err;

#if PGP_PREFS_RANDSEEDFILE
	PGPPrefRef	prefref;

	if (IsNull (pszPath))
		return kPGPError_BadParams;

	pszPath[0] = '\0';

	if (!PGPPrefRefIsValid (prefrefIn))
	{
		err = PGPclGetPath (kPGPclRandseedFile, pszPath, iLen); CKERR;
	}

	if (pszPath[0] == '\0')
	{
		CHAR	szFile[16];

		err = PGPclGetPath (kPGPclDefaultRandseedFolder, pszPath, iLen); CKERR;
		LoadString (g_hInst, IDS_DEFAULTRNGFILENAME, szFile, sizeof(szFile));
		if ((lstrlen (pszPath) + lstrlen (szFile) +1) < iLen)
		{
			lstrcat (pszPath, szFile);
			CLCreatePath (pszPath);

			if (PGPPrefRefIsValid (prefrefIn))
				prefref = prefrefIn;
			else
			{
				err = PGPclPeekClientLibPrefRefs (&prefref, NULL);
				CKERR;
			}

			PGPclSetPath (prefref, kPGPclRandseedFile, pszPath);

			if (!PGPPrefRefIsValid (prefrefIn))
				PGPclFlushClientLibPrefs (prefref, NULL);
		}
		else
			err = kPGPError_OutputBufferTooSmall;
	}
#else
	if (IsNull (pszPath))
		return kPGPError_BadParams;

	pszPath[0] = '\0';

	err = PGPclGetPath (kPGPclRandseedFile, pszPath, iLen); CKERR;
	CLCreatePath (pszPath);

#endif // PGP_PREFS_RANDSEEDFILE

done:
	return err;
}

//	_________________________________________________
//
//	Installer convenience function for creating special folders

PGPError PGPclExport WINAPI
PGPclInstallerCreateFolders (
		char*				pszSHFolderPath)
{
	CHAR		sz[MAX_PATH];
	PGPError	err;

	err = CLLoadSHFolderGetPath (pszSHFolderPath);
	if (IsntPGPError (err))
	{
		if (IsntNull (g_fpSHGetFolderPath))
		{
			g_fpSHGetFolderPath (
					NULL, CSIDL_APPDATA|CSIDL_FLAG_CREATE, 
					NULL, 0, sz);
			g_fpSHGetFolderPath (
					NULL, CSIDL_COMMON_APPDATA|CSIDL_FLAG_CREATE, 
					NULL, 0, sz);
			g_fpSHGetFolderPath (
					NULL, CSIDL_LOCAL_APPDATA|CSIDL_FLAG_CREATE, 
					NULL, 0, sz);
			g_fpSHGetFolderPath (
					NULL, CSIDL_PERSONAL|CSIDL_FLAG_CREATE, 
					NULL, 0, sz);

			err = kPGPError_NoErr;
		}
		else
			err = kPGPError_FileOpFailed;
	}

	return err;
}


//	_________________________________________________
//
//	merge in the admin prefs with the specified prefref

PGPError 
CLMergeInAdminPrefs (
		PGPContextRef	context,
		PGPPrefRef		prefref, 
		BOOL			bNetPrefs)
{
	PGPError		err				= kPGPError_NoErr;
	PFLFileSpecRef	fileref			= NULL;

	CHAR			szAdmin[MAX_PATH];
	PGPPrefRef		prefrefAdmin;

	// get the admin prefs file
	PGPclGetPath (kPGPclInstallFolder, szAdmin, sizeof(szAdmin));

	if (bNetPrefs)
	{
		if ((lstrlen (szAdmin) + 
				lstrlen (PGP_NETPREFSFILENAME) +1) < sizeof(szAdmin))
		{
			lstrcat (szAdmin, PGP_NETPREFSFILENAME);
		}
	}
	else
	{
		if ((lstrlen (szAdmin) + 
				lstrlen (PGP_PREFSFILENAME) +1) < sizeof(szAdmin))
		{
			lstrcat (szAdmin, PGP_PREFSFILENAME);
		}
	}

	err = PFLNewFileSpecFromFullPath (
			PGPPeekContextMemoryMgr (context), szAdmin, &fileref); 

	if (IsntPGPError (err))
	{
		if (bNetPrefs)
		{
			err = PGPOpenPrefFile (fileref,
					netDefaults, netDefaultsSize, &prefrefAdmin);

			if (IsntPGPError (err))
			{
				PGPNetPrefHostEntry*	pHosts			= NULL;
				PGPNetPrefHostEntry*	pHostsNew		= NULL;
				PGPNetPrefHostEntry*	pHostsMerged	= NULL;

				PGPUInt32	uHostCount			= 0;
				PGPUInt32	uHostCountNew		= 0;
				PGPUInt32	uHostCountMerged	= 0;
			
				PGPBoolean		bCopyHosts		= FALSE;
				PGPBoolean		bMergeHosts		= FALSE;

				PGPGetPrefBoolean (prefrefAdmin, 
						kPGPPrefCopyNetHosts, &bCopyHosts);
				PGPGetPrefBoolean (prefrefAdmin,
						kPGPPrefMergeNetHosts, &bMergeHosts);

				PGPclMergeNetPrefs (prefrefAdmin, prefref);

				PGPGetNetHostPrefs (prefref,
						&pHosts, &uHostCount);
				PGPGetNetHostPrefs (prefrefAdmin,
						&pHostsNew, &uHostCountNew);

				if (IsntPGPError (err))
				{
					if (bMergeHosts)
					{
						err = pgpMergeNetHostLists (
								PGPPeekContextMemoryMgr (context),
								pHosts, uHostCount,
								pHostsNew, uHostCountNew,
								&pHostsMerged, &uHostCountMerged);

						if (IsntPGPError (err))
						{
							PGPSetNetHostPrefs (prefref,
									pHostsMerged, uHostCountMerged);
							PGPFreeData (pHostsMerged);
						}
					}
					else if (bCopyHosts)
					{
						PGPSetNetHostPrefs (prefref,
								pHostsNew, uHostCountNew);
					}
				}

				if (IsntNull (pHosts))
					PGPFreeData (pHosts);
				if (IsntNull (pHostsNew))
					PGPFreeData (pHostsNew);

				PGPFreePrefs (prefrefAdmin);
			}
		}
		else
		{
			err = PGPOpenPrefFile (fileref,
					clientDefaults, clientDefaultsSize, &prefrefAdmin);

			if (IsntPGPError (err))
			{
				PGPclMergePrefs (prefrefAdmin, prefref);
				PGPFreePrefs (prefrefAdmin);
			}
		}

		PFLFreeFileSpec (fileref);
		fileref = NULL;
	}

	return err;
}


//	_________________________________________________
//
//	Installer convenience function for setting keyring files

PGPError PGPclExport WINAPI
PGPclInstallerSetPaths (
		char*				pszPubRing,
		char*				pszSecRing,
		char*				pszNetPubRing,
		char*				pszNetSecRing,
		char*				pszRNGSeedFile,
		char*				pszDefaultKeysFile)
{
	PGPError			err				= kPGPError_NoErr;
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	PFLFileSpecRef		fileref			= NULL;
	PGPFileSpecRef		filerefDefault	= kInvalidPGPFileSpecRef;
	PGPFileSpecRef		filerefPub		= kInvalidPGPFileSpecRef;
	PGPFileSpecRef		filerefSec		= kInvalidPGPFileSpecRef;
	PGPContextRef		context			= kInvalidPGPContextRef;
	PGPKeyDBRef			keydbMain		= kInvalidPGPKeyDBRef;
	PGPUInt32			uAccountStatus	= kPGPPrefPGPNotInitialized;

	CHAR				szPrefPath[MAX_PATH];

	// Initialize the sdk and create a context
	err = PGPsdkInit (kPGPFlags_ForceLocalExecution); CKERR;
	err = PGPNewContext (kPGPsdkAPIVersion, &context); CKERR;

	// create/open PGP prefs file for saving PGP keyring paths
	err = PGPclGetPath (kPGPclPrefsFile, 
			szPrefPath, sizeof(szPrefPath)); CKERR;
	CLCreatePath (szPrefPath);

	err = PFLNewFileSpecFromFullPath (PGPPeekContextMemoryMgr (context),
			szPrefPath, &fileref); CKERR;

	if(CHICKENANDEGG())
	{
		// copy the admin specified prefs
		if (PGPclIsClientInstall ())
		{
			CHAR szAdmin[MAX_PATH];
			
			PGPclGetPath (kPGPclInstallFolder, szAdmin, sizeof(szAdmin));
			if ((lstrlen (szAdmin) + 
				lstrlen (PGP_PREFSFILENAME) +1) < sizeof(szAdmin))
			{
				lstrcat (szAdmin, PGP_PREFSFILENAME);
				CopyFile (szAdmin, szPrefPath, FALSE);
			}
		}
	}

	err = PGPOpenPrefFile (fileref,
			clientDefaults, clientDefaultsSize, &prefref);

	if ((err == kPGPError_FileOpFailed) ||
		(err == kPGPError_FileNotFound))
	{
		err = PFLFileSpecCreate (fileref); CKERR;
		err = PGPOpenPrefFile (fileref,
				clientDefaults, clientDefaultsSize, &prefref);
	}
	CKERR;

	PFLFreeFileSpec (fileref);
	fileref = NULL;

	err = PGPclSetPath (prefref, kPGPclPublicKeyringFile, pszPubRing); CKERR;
	err = PGPclSetPath (prefref, kPGPclPrivateKeyringFile, pszSecRing); CKERR;

	if (IsntNull (pszRNGSeedFile) && (pszRNGSeedFile[0] != '\0'))
	{
		err = PGPclSetPath (prefref, 
				kPGPclRandseedFile, pszRNGSeedFile); CKERR;
	}
	else
	{
		CHAR	szRNGFile[MAX_PATH];

		err = CLInitRNGSeedFilePref (
				prefref, szRNGFile, sizeof(szRNGFile)); CKERR;
		err = PGPclSetPath (
				prefref, kPGPclRandseedFile, szRNGFile); CKERR;
	}

	if(CHICKENANDEGG())
	{
		// merge in any admin specified prefs
		if (PGPclIsClientInstall ())
		{
			CLMergeInAdminPrefs (context, prefref, FALSE);
		}
	}

	// if account has not been disabled, then set flag 
	// indicating that this user has initialized the keyrings
	PGPGetPrefNumber (prefref,
			kPGPPrefUserAccountStatus, &uAccountStatus);
	if (uAccountStatus != kPGPPrefPGPDisabled)
	{
		PGPSetPrefNumber (prefref, 
				kPGPPrefUserAccountStatus, kPGPPrefPGPInitialized);
	}

	err = PGPSavePrefFile (prefref); CKERR;

	PGPFreePrefs (prefref);
	prefref = kInvalidPGPPrefRef;

	// create/open PGPnet prefs file for saving PGPnet keyring paths
	if (IsntNull (pszNetPubRing) || (IsntNull (pszNetSecRing)))
	{
		err = PGPclGetPath (kPGPclNetPrefsFile, 
				szPrefPath, sizeof(szPrefPath)); CKERR;
		CLCreatePath (szPrefPath);

		err = PFLNewFileSpecFromFullPath (PGPPeekContextMemoryMgr (context),
				szPrefPath, &fileref); CKERR;

		if(CHICKENANDEGG())
		{
			// copy the admin specified prefs
			if (PGPclIsClientInstall ())
			{
				CHAR szAdmin[MAX_PATH];
				
				PGPclGetPath (kPGPclInstallFolder, szAdmin, sizeof(szAdmin));
				if ((lstrlen (szAdmin) + 
					lstrlen (PGP_NETPREFSFILENAME) +1) < sizeof(szAdmin))
				{
					lstrcat (szAdmin, PGP_NETPREFSFILENAME);
					CopyFile (szAdmin, szPrefPath, FALSE);
				}
			}
		}

		err = PGPOpenPrefFile (fileref,
				netDefaults, netDefaultsSize, &prefref);

		if ((err == kPGPError_FileOpFailed) ||
			(err == kPGPError_FileNotFound))
		{
			err = PFLFileSpecCreate (fileref); CKERR;
			err = PGPOpenPrefFile (fileref,
					netDefaults, netDefaultsSize, &prefref);
		}
		CKERR;

		PFLFreeFileSpec (fileref);
		fileref = NULL;

		err = PGPclSetPath (prefref, 
				kPGPclNetPublicKeyringFile, pszNetPubRing); CKERR;
		err = PGPclSetPath (prefref, 
				kPGPclNetPrivateKeyringFile, pszNetSecRing); CKERR;

		if(CHICKENANDEGG())
		{
			// merge in any admin specified prefs
			if (PGPclIsClientInstall ())
			{
				CLMergeInAdminPrefs (context, prefref, TRUE);
			}
		}

		err = PGPSavePrefFile (prefref); CKERR;

		PGPFreePrefs (prefref);
		prefref = kInvalidPGPPrefRef;
	}

	// now handle importing the default keys to the default keyrings
	if (IsntNull (pszDefaultKeysFile))
	{
		PGPKeyDBRef		keydbImported;

		err = PGPNewFileSpecFromFullPath (context, 
				pszDefaultKeysFile, &filerefDefault); CKERR;
		err = PGPNewFileSpecFromFullPath (context,
				pszPubRing, &filerefPub); CKERR;
		err = PGPNewFileSpecFromFullPath (context,
				pszSecRing, &filerefSec); CKERR;
		err = PGPOpenKeyDBFile (context,
				kPGPOpenKeyDBFileOptions_Mutable|kPGPOpenKeyDBFileOptions_Create,
				filerefPub, filerefSec, &keydbMain); CKERR;
		err = PGPImport (context, &keydbImported, 							
				PGPOInputFile (context, filerefDefault),
				PGPOLastOption (context)); CKERR;

		if (PGPKeyDBRefIsValid (keydbImported))
		{
			err = PGPCopyKeys (PGPPeekKeyDBRootKeySet (keydbImported), 
					keydbMain, NULL);
			PGPFreeKeyDB (keydbImported);
		}
	}

done:
	if (PGPKeyDBRefIsValid (keydbMain))
		PGPFreeKeyDB (keydbMain);
	if (PGPFileSpecRefIsValid (filerefDefault))
		PGPFreeFileSpec (filerefDefault);
	if (PGPFileSpecRefIsValid (filerefPub))
		PGPFreeFileSpec (filerefPub);
	if (PGPFileSpecRefIsValid (filerefSec))
		PGPFreeFileSpec (filerefSec);
	if (PGPPrefRefIsValid (prefref))
		PGPFreePrefs (prefref);
	if (IsntNull (fileref))
		PFLFreeFileSpec (fileref);
	if (PGPContextRefIsValid (context))
		PGPFreeContext (context);

	PGPsdkCleanup ();
	return err;
}


//	_________________________________________________
//
//	Wrapper routine for platform independent
//	word wrap code.

PGPError PGPclExport
PGPclWrapBuffer (
		LPSTR		szInText,
		PGPUInt16	wrapColumn,
		LPSTR*		pszOutText)
{
	PGPError		err				= kPGPError_NoErr;
	PGPIORef		inRef, outRef;
	PGPMemoryMgrRef	memMgr;
	PGPFileOffset	outSize;
	PGPSize			outRead;
	INT				InSize;

	outRead = 0;

	err = PGPNewMemoryMgr ( 0, &memMgr);

	if (IsntPGPError (err))
	{
		PGPNewMemoryIO (
			memMgr,
			(PGPMemoryIORef *)(&inRef));

		InSize = strlen (szInText);
		PGPIOWrite (inRef,
			InSize,
			szInText);
		PGPIOFlush (inRef);
		PGPIOSetPos (inRef,0);

		PGPNewMemoryIO (
			memMgr,
			(PGPMemoryIORef *)(&outRef));

		err = pgpWordWrapIO(
			  inRef,outRef,
			  wrapColumn,
			  "\r\n");

		if (IsntPGPError (err))
		{
			INT memamt;

			PGPIOGetPos (outRef, &outSize);

			memamt = (INT)outSize+1;
			*pszOutText = (CHAR *)malloc (memamt);

			if (*pszOutText)
			{
				memset (*pszOutText, 0x00, memamt);

				PGPIOSetPos (outRef,0);
				PGPIORead (outRef,
					(INT)outSize,
					*pszOutText,
					&outRead);
				PGPIOFlush (outRef);
			}
			else
				err = kPGPError_OutOfMemory;
		}
		PGPFreeIO (inRef);
		PGPFreeIO (outRef);
	}
	PGPFreeMemoryMgr (memMgr);

	return err;
}

//	__________________________________________________
//
//	free previously-wrapped buffer

PGPError PGPclExport
PGPclFreeWrapBuffer (LPSTR textBuffer)
{
	memset (textBuffer, 0x00, lstrlen (textBuffer));
	free (textBuffer);

	return kPGPError_NoErr;
}


//	________________________
//
//	find out if SDK supports the specified public key algorithm

PGPError PGPclExport
PGPclCheckSDKSupportForPKAlg (
		PGPPublicKeyAlgorithm alg,
		PGPBoolean mustEncrypt,
		PGPBoolean mustSign)
{
	PGPError	err			= kPGPError_FeatureNotAvailable;

	PGPUInt32					i, iNumAlgs;
	PGPPublicKeyAlgorithmInfo	alginfo;

	err = PGPCountPublicKeyAlgorithms (&iNumAlgs);
	if (IsPGPError (err))
		return kPGPError_FeatureNotAvailable;

	for (i=0; i<iNumAlgs; i++)
	{
		err = PGPGetIndexedPublicKeyAlgorithmInfo (i, &alginfo);
		if (IsntPGPError (err))
		{
			if (alginfo.algID == alg)
			{
				err = kPGPError_NoErr;
				if (mustEncrypt && !(alginfo.canEncrypt))
					err = kPGPError_FeatureNotAvailable;
				if (mustSign && !(alginfo.canSign))
					err = kPGPError_FeatureNotAvailable;
				return err;
			}
		}
	}

	return kPGPError_FeatureNotAvailable;
}

//	________________________
//
//	find out if SDK supports the specified cipher algorithm

PGPError PGPclExport
PGPclCheckSDKSupportForCipherAlg (PGPCipherAlgorithm alg)
{
	PGPError	err		= kPGPError_FeatureNotAvailable;

	PGPUInt32					i, iNumAlgs;
	PGPSymmetricCipherInfo		cipherinfo;

	err = PGPCountSymmetricCiphers (&iNumAlgs);
	if (IsPGPError (err)) return kPGPError_FeatureNotAvailable;

	for (i=0; i<iNumAlgs; i++)
	{
		err = PGPGetIndexedSymmetricCipherInfo (i, &cipherinfo);
		if (IsntPGPError (err))
		{
			if (cipherinfo.algID == alg)
				return kPGPError_NoErr;
		}
	}

	return kPGPError_FeatureNotAvailable;
}


//	___________________________________________________
//
//	Sign key dialog message procedure

static BOOL CALLBACK
sConfirmAuthDlgProc (
				HWND hDlg,
				UINT uMsg,
				WPARAM wParam,
				LPARAM lParam)
{
	PCONFIRMAUTHSTRUCT	pcas;

	switch (uMsg) {

	case WM_INITDIALOG :
	{
		CHAR				sz[64];
		INT					i;
		PGPValidity			validity;
		PGPUInt32			uKeyVersion;
		UINT				uIDS;
		CHAR				szText[256];

		SetWindowLong (hDlg, GWL_USERDATA, lParam);
		pcas = (PCONFIRMAUTHSTRUCT)lParam;

		// initialize validity bar
		i = CLConvertFromPGPValidity (kPGPValidity_Complete);
		SendMessage (GetDlgItem (hDlg, IDC_VALIDITY),
				PBM_SETRANGE, 0, MAKELPARAM (0,i));
		PGPGetKeyDBObjNumericProperty (pcas->keyAuthenticating,
				kPGPKeyProperty_Validity, &validity);
		i = CLConvertFromPGPValidity (validity);
		SendMessage (GetDlgItem (hDlg, IDC_VALIDITY),
				PBM_SETPOS, (WPARAM)i, 0);

		// initialize text
		uIDS = 0;
		switch (pcas->authtype) {
		case kPGPclAuthShowAuthentication :
			ShowWindow (GetDlgItem (hDlg, IDOK), SW_HIDE);
			LoadString (g_hInst, IDS_DONEBUTTON, sz, sizeof(sz));
			SetDlgItemText (hDlg, IDCANCEL, sz);
			break;

		case kPGPclAuthExpectedKey :
		case kPGPclAuthConfirmIfInvalid :
			if (validity >= pcas->validityThreshold) 
			{
				EndDialog (hDlg, 1);
				return FALSE;
			}
			else
				uIDS = IDS_INVALIDAUTHKEY;
			break;

		case kPGPclAuthReconstituting :
		case kPGPclAuthConfirmAlways :
			if (validity < pcas->validityThreshold)
				uIDS = IDS_INVALIDAUTHKEY;
			else
				uIDS = IDS_VALIDAUTHKEY;
			break;

		case kPGPclAuthNewKey :
			uIDS = IDS_NEWAUTHKEY;
			break;

		case kPGPclAuthUnexpectedKey :
			uIDS = IDS_UNEXPECTEDAUTHKEY;
			break;

		case kPGPclAuthDoNotConfirm :
			EndDialog (hDlg, 1);
			return FALSE;
		}

		if (uIDS)
		{
			LoadString (g_hInst, uIDS, szText, sizeof(szText));
			SetDlgItemText (hDlg, IDC_AUTHTEXT, szText);
		}

		// initialize server name
		SetDlgItemText (hDlg, IDC_SERVERNAME, pcas->pszRemoteHost);

		// initialize key name
		UTF8EditInit (GetDlgItem (hDlg, IDC_KEYNAME));
		UTF8EditSetText (GetDlgItem (hDlg, IDC_KEYNAME), pcas->szName);

		// initialize fingerprint edit control
		PGPGetKeyDBObjDataProperty (pcas->keyAuthenticating,
				kPGPKeyProperty_Fingerprint, sz, sizeof (sz), &i);
		PGPGetKeyDBObjNumericProperty (pcas->keyAuthenticating,
				kPGPKeyProperty_Version, &uKeyVersion);
		PGPclConvertFingerprintToString (uKeyVersion, sz);
		SetDlgItemText (hDlg, IDC_FINGERPRINT, sz);

		// initialize security text strings
		uIDS = 0;
		switch (pcas->tlsCipher) {
		case kPGPtls_TLS_NULL_WITH_NULL_NULL :
			uIDS = IDS_TLS_NULL_WITH_NULL_NULL;
			break;
		case kPGPtls_TLS_PGP_DHE_DSS_WITH_CAST_CBC_SHA :
			uIDS = IDS_TLS_PGP_DHE_DSS_WITH_CAST_CBC_SHA;
			break;
		case kPGPtls_TLS_PGP_DHE_RSA_WITH_CAST_CBC_SHA :
			uIDS = IDS_TLS_PGP_DHE_RSA_WITH_CAST_CBC_SHA;
			break;
		case kPGPtls_TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA :
			uIDS = IDS_TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA;
			break;
		case kPGPtls_TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA :
			uIDS = IDS_TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA;
			break;
		case kPGPtls_TLS_RSA_WITH_3DES_EDE_CBC_SHA :
			uIDS = IDS_TLS_RSA_WITH_3DES_EDE_CBC_SHA;
			break;
		case kPGPtls_TLS_RSA_WITH_IDEA_CBC_SHA :
			uIDS = IDS_TLS_RSA_WITH_IDEA_CBC_SHA;
			break;
		case kPGPtls_TLS_PGP_RSA_WITH_CAST_CBC_SHA :
			uIDS = IDS_TLS_PGP_RSA_WITH_CAST_CBC_SHA;
			break;
		case kPGPtls_TLS_PGP_DHE_DSS_WITH_NULL_SHA :
			uIDS = IDS_TLS_PGP_DHE_DSS_WITH_NULL_SHA;
			break;
		case kPGPtls_TLS_DHE_DSS_WITH_NULL_SHA :
			uIDS = IDS_TLS_DHE_DSS_WITH_NULL_SHA;
			break;
		}
		if (uIDS)
		{
			LPSTR	pszStart;
			LPSTR	pszStop;

			LoadString (g_hInst, uIDS, szText, sizeof(szText));
			pszStart = szText;
			pszStop = strchr (pszStart, '/');
			*pszStop = '\0';
			SetDlgItemText (hDlg, IDC_CERTIFICATE, pszStart);

			pszStart = pszStop+1;
			pszStop = strchr (pszStart, '/');
			*pszStop = '\0';
			SetDlgItemText (hDlg, IDC_SIGNATURE, pszStart);

			pszStart = pszStop+1;
			pszStop = strchr (pszStart, '/');
			*pszStop = '\0';
			SetDlgItemText (hDlg, IDC_EXCHANGE, pszStart);

			pszStart = pszStop+1;
			pszStop = strchr (pszStart, '/');
			*pszStop = '\0';
			SetDlgItemText (hDlg, IDC_CIPHER, pszStart);

			pszStart = pszStop+1;
			SetDlgItemText (hDlg, IDC_HASH, pszStart);
		}

		return TRUE;
	}

	case WM_DESTROY :
		UTF8EditDestroy (GetDlgItem (hDlg, IDC_KEYNAME));
		return FALSE;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hDlg, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aIds);

	case WM_COMMAND :
		switch (LOWORD(wParam)) {
		case IDCANCEL :
			EndDialog (hDlg, 0);
			break;

		case IDOK :
			EndDialog (hDlg, 1);
			break;

		case IDC_IMPORTKEY :
			pcas = (PCONFIRMAUTHSTRUCT)GetWindowLong (hDlg, GWL_USERDATA);
			{
				PGPKeySetRef		keysetAuth;
				PGPError			err;

				err = PGPNewOneKeySet (
							pcas->keyAuthenticating, &keysetAuth);
				if (IsntPGPError (err) && PGPKeySetRefIsValid (keysetAuth))
				{
					if (CLAddKeysToMain (pcas->context, hDlg,
								keysetAuth, pcas->keydbMain))
					{
						PGPclMessageBox (hDlg, IDS_CAPTIONINFO, 
								IDS_AUTHKEYIMPORTED, MB_OK|MB_ICONINFORMATION);
					}
					PGPFreeKeySet (keysetAuth);
				}
			}
			break;
		}
		return TRUE;
	}
	return FALSE;
}

//	___________________________________________________
//
//  post confirmation dialog for authentication key

PGPError PGPclExport
PGPclConfirmRemoteAuthentication (
		PGPContextRef			context,
		HWND					hwndParent,
		LPSTR					pszServer,
		PGPKeyDBObjRef			keyAuth,
		PGPtlsCipherSuiteNum	tlsCipher,
		PGPKeyDBRef				keydbMain,
		PGPclAuthType			authtype)
{
	PGPPrefRef			prefref				= kInvalidPGPPrefRef;
	PGPKeySetRef		keyset				= kInvalidPGPKeySetRef;
	CONFIRMAUTHSTRUCT	cas;
	UINT				u;
	PGPError			err;
	PGPBoolean			bMargIsInvalid;

	// if parent hwnd is NULL, then this is a "silent" server communication
	if (hwndParent == NULL)
		return kPGPError_UserAbort;

	cas.context				= context;
	cas.pszRemoteHost		= pszServer;
	cas.keyAuthenticating	= keyAuth;
	cas.tlsCipher			= tlsCipher;
	cas.keydbMain			= keydbMain;
	cas.authtype			= authtype;
	cas.szName[0]			= '\0';

	if (PGPKeyDBObjRefIsValid (keyAuth))
	{
		PGPclGetPrimaryUserIDNameUTF8 (
				keyAuth, cas.szName, sizeof(cas.szName), &u);

		// propagate validity from main keydb
		if (IsntPGPError (PGPNewOneKeySet (keyAuth, &keyset)))
		{
			if (IsntPGPError (PGPCheckKeyRingSigs (
					keyset, keydbMain, TRUE, NULL, NULL)))
			{
				PGPGetKeyDBObjNumericProperty (keyAuth,
						kPGPKeyProperty_Validity, &u);
			}
			PGPFreeKeySet (keyset);
		}
	}

	// use prefs to determine validity threshold
	err = PGPclPeekClientLibPrefRefs (&prefref, NULL);
	if (IsPGPError (err))
		return err;

	PGPGetPrefBoolean (prefref, kPGPPrefMarginalIsInvalid, &bMargIsInvalid);
	if (bMargIsInvalid)
		cas.validityThreshold = kPGPValidity_Complete;
	else
		cas.validityThreshold = kPGPValidity_Marginal;

	if (DialogBoxParam (g_hInst,
		MAKEINTRESOURCE(IDD_CONFIRMAUTHENTICATION),
		hwndParent, sConfirmAuthDlgProc, (LPARAM)&cas))
	{
		return kPGPError_NoErr;
	}
	else
		return kPGPError_UserAbort;
}

//	___________________________________________________
//
//  find unknown signer key

PGPError PGPclExport
PGPclLookupUnknownSigner(
		PGPContextRef		context,
		PGPKeyDBRef			keydbMain,
		PGPtlsContextRef	tlsContext,
		HWND				hwnd,
		PGPEvent			*event,
		PGPKeyID			signingKeyID,
		PGPBoolean*			pbGotKeys,
		PGPKeyDBRef			*pkeydbNew)
{
	PGPKeySetRef	keysetNew	= kInvalidPGPKeySetRef;
	PGPUInt32		uNumKeys	= 0;
	PGPError		err;
	int				nOleErr;

	if (pbGotKeys == NULL)
		return kPGPError_BadParams;

	*pbGotKeys = FALSE;

	err = PGPclSearchServerForKeyIDs (context,
					NULL, hwnd, &signingKeyID, 1,
					kPGPclDefaultServer, keydbMain,
					pkeydbNew);

	if (IsntPGPError (PGPclErrorBox (hwnd, err)))
	{
		PGPNewKeySet (*pkeydbNew, &keysetNew);
		PGPCountKeys (keysetNew, &uNumKeys);

		if (uNumKeys > 0)
		{
			nOleErr = OleInitialize (NULL);
			if ((nOleErr == S_OK) || (nOleErr == S_FALSE))
			{
				PGPclImportKeys (context, tlsContext, hwnd, keysetNew,
						keydbMain, kPGPclDefaultImportFlags);
				OleUninitialize ();
			}

			PGPAddJobOptions (event->job,
				PGPOKeyDBRef (context, *pkeydbNew),
				PGPOLastOption (context));

			PGPFreeKeySet (keysetNew);
			*pbGotKeys = TRUE;
		}
	}

	return kPGPError_NoErr;
}


//	___________________________________________________
//
//	Add keys in specified keyset to main keydb

BOOL
CLAddKeysToMain (
		PGPContextRef	context,
		HWND			hwnd,
		PGPKeySetRef	keysetToAdd,
		PGPKeyDBRef		keydbMain)
{
	PGPError			err					= 0;
	PGPKeySetRef		keyset				= kInvalidPGPKeySetRef;

	if (PGPKeyDBRefIsValid (keydbMain))
	{
		err = PGPCopyKeys (keysetToAdd, keydbMain, NULL);
		if (IsntPGPError (err))
		{
			if (IsntPGPError (err))
			{
				PGPclNotifyKeyringChanges (
						kPGPclKeyringContentsModified, GetCurrentProcessId ());
			}
		}
	}

	if (IsPGPError (err))
	{
		PGPclMessageBox (hwnd, IDS_CAPTIONALERT, 
				IDS_IMPORTKEYERROR, MB_OK|MB_ICONEXCLAMATION);
	}
	return (IsntPGPError (err));
}


//	___________________________________________________
//
//	determine if we are running under an Admin account

PGPBoolean PGPclExport
PGPclLoggedInAsAdministrator (VOID)
{
	TOKEN_GROUPS*		ptg				= NULL;
	BOOL				bAdmin;
	HANDLE				hThread;
	DWORD				cbTokenGroups;
	DWORD				dwGroup;
	PSID				psidAdmin;
	PGPclWindowsFamily	family;

	SID_IDENTIFIER_AUTHORITY SystemSidAuthority	= SECURITY_NT_AUTHORITY;

	// if not running under NT, just return TRUE
	PGPclGetWindowsVersion (&family, NULL, NULL);
	if (family != kPGPclWindowsFamilyNT)
		return TRUE;

	// open a handle to the access token for this thread
	if (!OpenThreadToken (GetCurrentThread(), TOKEN_QUERY, FALSE, &hThread))
	{
		if (GetLastError() == ERROR_NO_TOKEN)
		{
			// the thread does not have an access token --
			// use that of the process
			if (!OpenProcessToken (GetCurrentProcess(), TOKEN_QUERY, &hThread))
				return FALSE;
		}
		else
			return FALSE;
	}

	// query the size of the group information associated with the token.
	// Note that we expect a FALSE result from GetTokenInformation
	// because we've given it a NULL buffer. On exit cbTokenGroups will tell
	// the size of the group information.
	if (GetTokenInformation (hThread, TokenGroups, NULL, 0, &cbTokenGroups))
		return FALSE;

	// verify that GetTokenInformation failed for lack of a large
	// enough buffer.
	if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
		return FALSE;

	// allocate a buffer for the group information.
	// Since _alloca allocates on the stack, we don't have
	// to explicitly deallocate it. That happens automatically
	// when we exit this function.
	if (!(ptg = _alloca (cbTokenGroups)))
		return FALSE;

	// ask for the group information again.
	// This may fail if an administrator has added this account
	// to an additional group between our first call to
	// GetTokenInformation and this one.
	if (!GetTokenInformation (hThread, TokenGroups,
						ptg, cbTokenGroups, &cbTokenGroups))
		return FALSE;

	// create a System Identifier for the Admin group.
	if (!AllocateAndInitializeSid (&SystemSidAuthority, 2,
            SECURITY_BUILTIN_DOMAIN_RID,
            DOMAIN_ALIAS_RID_ADMINS,
            0, 0, 0, 0, 0, 0, &psidAdmin))
		return FALSE;

	// iterate through the list of groups for this access
	// token looking for a match against the SID we created above.
	bAdmin = FALSE;
	for (dwGroup = 0; dwGroup < ptg->GroupCount; dwGroup++)
	{
		if (EqualSid (ptg->Groups[dwGroup].Sid, psidAdmin))
		{
			bAdmin = TRUE;
			break;
		}
	}

	FreeSid (psidAdmin);

	return bAdmin;
}


//	___________________________________________________
//
//	report dimensions of specified monitor

PGPError PGPclExport
PGPclGetMonitorRect (
		HWND			hwnd,
		PGPclMonitorID	monitor,
		RECT*			prcMonitor)
{
	HMONITOR		hmonitor;
	MONITORINFO		mi;
	POINT			pt;

	switch (monitor) {
	case kPGPclWindowMonitor :
		hmonitor = MonitorFromWindow (hwnd, MONITOR_DEFAULTTONEAREST);
		break;
	default :
		pt.x = 0;
		pt.y = 0;
		hmonitor = MonitorFromPoint (pt, MONITOR_DEFAULTTOPRIMARY);
		break;
	}

	mi.cbSize = sizeof(mi);
	GetMonitorInfo (hmonitor, &mi);

	*prcMonitor = mi.rcWork;

	return kPGPError_NoErr;
}


//	___________________________________________________
//
//	center window on specified monitor

PGPError PGPclExport
PGPclCenterWindowOnMonitor (
		HWND	hwnd,
		UINT	uMonitor)
{
	HMONITOR		hmonitor;
	MONITORINFO		mi;
	RECT			rcWnd;
	RECT			rcMon;
	POINT			pt;

	GetWindowRect (hwnd, &rcWnd);

	switch (uMonitor) {
	case kPGPclWindowMonitor :
		hmonitor = MonitorFromWindow (hwnd, MONITOR_DEFAULTTONEAREST);
		break;
	default :
		pt.x = 0;
		pt.y = 0;
		hmonitor = MonitorFromPoint (pt, MONITOR_DEFAULTTOPRIMARY);
		break;
	}

	mi.cbSize = sizeof(mi);
	GetMonitorInfo (hmonitor, &mi);

	rcMon = mi.rcWork;

	rcWnd.left = rcMon.left +
			((rcMon.right - rcMon.left) - (rcWnd.right - rcWnd.left)) / 2;
	rcWnd.top = rcMon.top +
			((rcMon.bottom - rcMon.top) - (rcWnd.bottom - rcWnd.top)) / 2;

	SetWindowPos (hwnd, NULL, rcWnd.left, rcWnd.top, 0, 0,
			SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);

	return kPGPError_NoErr;
}


//	____________________________________
//
//	Make sure that window is visible on multimonitor system

PGPError PGPclExport
PGPclEnsureWindowIsOnMonitor (
		HWND	hwndToMove,
		HWND	hwndParent)
{
	BOOL	bMoveWindow		= FALSE;

	RECT	rcWin;
	RECT	rcMon;
	HDC		hdc;
	INT		iX, iY;

	GetWindowRect (hwndToMove, &rcWin);
	iX = rcWin.right - rcWin.left;
	iY = rcWin.bottom - rcWin.top;

	if (IsntNull (hwndParent))
	{
		RECT	rc;

		PGPclGetMonitorRect (hwndParent, kPGPclWindowMonitor, &rcMon);

		if (!IntersectRect (&rc, &rcWin, &rcMon))
			bMoveWindow = TRUE;
	}
	else
	{
		hdc = GetDC (NULL);

		if (!RectVisible (hdc, &rcWin))
		{
			PGPclGetMonitorRect (hwndToMove, kPGPclWindowMonitor, &rcMon);
			bMoveWindow = TRUE;
		}

		ReleaseDC (NULL, hdc);
	}

	if (bMoveWindow)
	{
		rcWin.left = max (rcMon.left, min (rcMon.right-iX, rcWin.left));
		rcWin.top = max (rcMon.top, min (rcMon.bottom-iY, rcWin.top));

		SetWindowPos (hwndToMove, NULL, rcWin.left, rcWin.top, 0, 0,
			SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
	}

	return kPGPError_NoErr;
}


//	____________________________________
//
//	get service pack info from registry

static DWORD
sGetServicePackNumber (void)
{
	HKEY		hkey;
	LRESULT		lResult;
	DWORD		dwValueType, dwSize, dwServicePack;

	dwServicePack = 0;	

	lResult = RegOpenKeyEx (HKEY_LOCAL_MACHINE, 
			"System\\CurrentControlSet\\Control\\Windows", 
			0, KEY_READ, &hkey);

	if (lResult == ERROR_SUCCESS) 
	{
		// see if component is installed
		dwSize = sizeof(dwServicePack);
		RegQueryValueEx (hkey, "CSDVersion", 
				0, &dwValueType, (LPBYTE)&dwServicePack, &dwSize);
		RegCloseKey (hkey);
	}

	return dwServicePack;
}


//	____________________________________
//
//	Determine the Windows version

PGPError PGPclExport
PGPclGetWindowsVersion (
		PGPclWindowsFamily*		pFamily,
		PGPclWindowsGenus*		pGenus,
		PGPclWindowsSpecies*	pSpecies)
{
	OSVERSIONINFO	osi;

	PGPclWindowsFamily		family		= kPGPclWindowsFamilyUnknown;
	PGPclWindowsGenus		genus		= kPGPclWindowsGenusUnknown;
	PGPclWindowsSpecies		species		= kPGPclWindowsSpeciesUnknown;

	osi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

	if (GetVersionEx(&osi))
	{
		switch (osi.dwPlatformId) {

		case VER_PLATFORM_WIN32_WINDOWS:
			family = kPGPclWindowsFamily9x;

			if (osi.dwMinorVersion == 0)
			{
				genus = kPGPclWindowsGenus95;

				if (LOWORD(osi.dwBuildNumber) > 1080)
					species = kPGPclWindowsSpecies95OSR2;
				else
					species = kPGPclWindowsSpecies95OSR1;
			}
			else if (osi.dwMinorVersion == 10)
			{
				genus = kPGPclWindowsGenus98;

				if (LOWORD(osi.dwBuildNumber) < 2222)
					species = kPGPclWindowsSpecies98;
				else
					species = kPGPclWindowsSpecies98SE;
			}
			else if (osi.dwMinorVersion == 90)
			{
				genus = kPGPclWindowsGenusMillenium;
				species = kPGPclWindowsSpeciesMillenium;
			}
			else
			{
				genus = kPGPclWindowsGenusPostMillenium;
				species = kPGPclWindowsSpeciesPostMillenium;
			}
			break;

		case VER_PLATFORM_WIN32_NT:
			family = kPGPclWindowsFamilyNT;

			if (osi.dwMajorVersion == 3)
			{
				genus = kPGPclWindowsGenusNT3;
				species = kPGPclWindowsSpeciesNT3;
			}
			else if (osi.dwMajorVersion == 4)
			{
				DWORD	dwSP	= sGetServicePackNumber ();

				genus = kPGPclWindowsGenusNT4;

				switch (LOWORD (dwSP)) {
				case 0x0000:
					species = kPGPclWindowsSpeciesNT4NoSp;
					break;
				case 0x0100:
					species = kPGPclWindowsSpeciesNT4Sp1;
					break;
				case 0x0200:
					species = kPGPclWindowsSpeciesNT4Sp2;
					break;
				case 0x0300:
					species = kPGPclWindowsSpeciesNT4Sp3;
					break;
				case 0x0400:
					species = kPGPclWindowsSpeciesNT4Sp4;
					break;
				case 0x0500:
					species = kPGPclWindowsSpeciesNT4Sp5;
					break;
				case 0x0600:
					species = kPGPclWindowsSpeciesNT4Sp6;
					break;
				default :
					species = kPGPclWindowsSpeciesNT4PostSp6;
					break;
				}
			}
			else if (osi.dwMajorVersion == 5)
			{
				if (osi.dwMinorVersion == 0)
				{
					genus = kPGPclWindowsGenus2000;

					if (LOWORD(osi.dwBuildNumber) < 2195)
						species = kPGPclWindowsSpecies2000Beta;
					else
					{
						DWORD	dwSP	= sGetServicePackNumber ();

						switch (LOWORD (dwSP)) {
						case 0x0000:
							species = kPGPclWindowsSpecies2000;
							break;

						case 0x0100:
							species = kPGPclWindowsSpecies2000Sp1;
							break;

						case 0x0200:
							species = kPGPclWindowsSpecies2000Sp2;
							break;

						case 0x0300:
							species = kPGPclWindowsSpecies2000Sp3;
							break;

						default:
							species = kPGPclWindowsSpecies2000;
							break;
						}
					}
				}
				else if (osi.dwMinorVersion == 1)
				{
					genus = kPGPclWindowsGenusXp;

					if (LOWORD(osi.dwBuildNumber) < 2600)
						species = kPGPclWindowsSpeciesXpBeta;
					else
					{
						DWORD	dwSP	= sGetServicePackNumber ();

						switch (LOWORD (dwSP)) {
						case 0x0000:
							species = kPGPclWindowsSpeciesXp;
							break;

						case 0x0100:
							species = kPGPclWindowsSpeciesXpSp1;
							break;

						default:
							species = kPGPclWindowsSpeciesXp;
							break;
						}
					}
/*
					// we're currently not distinguishing "home" vs. "pro"
					// but here is how we could do it :
					if (osi.wSuiteMask & VER_SUITE_PERSONAL)
						species = kPGPclWindowsSpeciesXpHome;
					else
						species = kPGPclWindowsSpeciesXpPro;
*/
				}
			}
			else if (osi.dwMajorVersion > 5)
			{
				genus = kPGPclWindowsGenusPostKnown;
				species = kPGPclWindowsSpeciesPostKnown;
			}
			break;
		}
	}

	if (pFamily)
		*pFamily	= family;
	if (pGenus)
		*pGenus		= genus;
	if (pSpecies)
		*pSpecies	= species;

	return kPGPError_NoErr;
}


//	___________________________________________________
//
//  Determine if we're running as a terminal services client

// the prototype for the ProcessToSessionId call
typedef DWORD (WINAPI *ProcToSessIdProc) (
			DWORD	dwProcessId,
			LPDWORD pdwSessionId);

PGPError PGPclExport
PGPclGetTerminalServicesSessionId (
		PGPUInt32*	puSessionId)
{
	static ProcToSessIdProc		s_fpProcToSessId	= NULL;

	PGPclWindowsSpecies		species			= kPGPclWindowsSpeciesUnknown;
	DWORD					dwSessionId		= 0;
	HINSTANCE				hLib			= NULL;

	*puSessionId = 0;

	PGPclGetWindowsVersion (NULL, NULL, &species);
	if (species < kPGPclWindowsSpeciesNT4Sp4)
		return kPGPError_NoErr;

	if (IsNull (s_fpProcToSessId))
	{
		if (IsntNull (hLib = LoadLibrary ("KERNEL32.DLL"))) 
		{
			s_fpProcToSessId = (ProcToSessIdProc)GetProcAddress (
					hLib, "ProcessIdToSessionId");
		}
	}

	if (IsntNull (s_fpProcToSessId))
	{
		if ((*s_fpProcToSessId)(GetCurrentProcessId (), puSessionId))
			return kPGPError_NoErr;
	}

	return kPGPError_Win32DllOpFailed;
}


PGPBoolean PGPclExport
PGPclRunningUnderTerminalServer (void)
{
	PGPUInt32	uSessionId;

	if (IsntPGPError (PGPclGetTerminalServicesSessionId (&uSessionId)))
	{
		if (uSessionId != 0)
			return TRUE;
	}
	return FALSE;
}


//	___________________________________________________
//
//  Determine the appropriate icon for a userid, based on
//	its properties

PGPInt32 PGPclExport
PGPclDetermineUserIDIcon (
		PGPKeyDBObjRef	userid,
		PGPBoolean*		pbItalics)
{
	PGPBoolean		bAttrib		= FALSE;
	PGPInt32		iType		= 0;
	PGPInt32		iAlg;
	PGPKeyDBObjRef	key;

	if (pbItalics)
		*pbItalics = FALSE;

	if (userid)
	{
		PGPGetKeyDBObjBooleanProperty (userid,
				kPGPUserIDProperty_IsAttribute, &bAttrib);
		if (bAttrib)
		{
			PGPGetKeyDBObjNumericProperty (userid,
					kPGPUserIDProperty_AttributeType, &iType);
			switch (iType) {
			case kPGPAttribute_Image :
				return IDX_PHOTOUSERID;

			default :
				if (pbItalics)
					*pbItalics = TRUE;
				return IDX_INVALIDUSERID;
			}
		}
	}

	key = PGPPeekKeyDBObjKey (userid);
	PGPGetKeyDBObjNumericProperty (key, kPGPKeyProperty_AlgorithmID, &iAlg);
	if (iAlg == kPGPPublicKeyAlgorithm_RSA)
		return (IDX_RSAUSERID);
	else
		return (IDX_DSAUSERID);
}


//	___________________________________________________
//
//  Determine the appropriate icon for a cert, based on
//	its properties

PGPInt32 PGPclExport
PGPclDetermineCertIcon (
		PGPKeyDBObjRef	cert,
		PGPBoolean*		pbItalics,
		PGPBoolean*		pbX509)
{
	PGPBoolean	bRevoked, bVerified, bTried, bExpired;
	PGPBoolean	bNotCorrupt, bExportable, bX509;
	PGPUInt32	uTrustLevel;
	INT			idx;

	PGPGetKeyDBObjBooleanProperty (cert,
			kPGPSigProperty_IsRevoked, &bRevoked);
	PGPGetKeyDBObjBooleanProperty (cert,
			kPGPSigProperty_IsExpired, &bExpired);
	PGPGetKeyDBObjBooleanProperty (cert,
			kPGPSigProperty_IsVerified, &bVerified);
	PGPGetKeyDBObjBooleanProperty (cert,
			kPGPSigProperty_IsTried, &bTried);
	PGPGetKeyDBObjBooleanProperty (cert,
			kPGPSigProperty_IsNotCorrupt, &bNotCorrupt);
	PGPGetKeyDBObjBooleanProperty (cert,
			kPGPSigProperty_IsExportable, &bExportable);
	PGPGetKeyDBObjBooleanProperty (cert,
			kPGPSigProperty_IsX509, &bX509);

	PGPGetKeyDBObjNumericProperty (cert,
			kPGPSigProperty_TrustLevel, &uTrustLevel);

	if (pbX509)
		*pbX509 = bX509;

	if (bTried && !bVerified) 
	{
		if (bX509)
			idx = IDX_X509BADCERT;
		else
			idx = IDX_BADCERT;
	}
	else if (bX509)
	{
		if (bRevoked) idx = IDX_X509REVCERT;
		else if (bExpired) idx = IDX_X509EXPCERT;
		else idx = IDX_X509CERT;
	}
	else if (bRevoked) idx = IDX_REVCERT;
	else if (bExpired) idx = IDX_EXPCERT;
	else if (bVerified)
	{
		if (bExportable)
		{
			if (uTrustLevel >= 1) idx = IDX_TRUSTEDCERT;
			else idx = IDX_EXPORTCERT;
		}
		else
		{
			if (uTrustLevel >= 2) idx = IDX_METACERT;
			else idx = IDX_CERT;
		}
	}
	else if (bNotCorrupt)
	{
		if (bExportable) idx = IDX_EXPORTCERT;
		else idx = IDX_CERT;
	}
	else idx = IDX_BADCERT;


	if ((idx == IDX_BADCERT) ||
		(idx == IDX_X509BADCERT) ||
		(idx == IDX_REVCERT) || 
		(idx == IDX_EXPCERT))
	{
		if (pbItalics)
			*pbItalics = TRUE;
	}

	return idx;
}


//	___________________________________________________
//
//  Determine the appropriate icon for a key, based on
//	its properties

PGPInt32 PGPclExport
PGPclDetermineKeyIcon (
		PGPKeyDBObjRef	key,
		PGPBoolean*		pbItalics)
{
	PGPBoolean bRevoked, bSecret, bDisabled, bExpired, bSplit;
	PGPBoolean bCanSign, bCanEncrypt, bCantDoAnything, bOnToken;
	PGPUInt32 uIdx, uAlg, uVer;

	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_IsRevoked, &bRevoked);
	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_IsSecret, &bSecret);
	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_IsDisabled, &bDisabled);
	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_IsExpired, &bExpired);
	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_IsSecretShared, &bSplit);
	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_CanSign, &bCanSign);
	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_CanEncrypt, &bCanEncrypt);
	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_IsOnToken, &bOnToken);

	PGPGetKeyDBObjNumericProperty (key,
			kPGPKeyProperty_AlgorithmID, &uAlg);
	PGPGetKeyDBObjNumericProperty (key,
			kPGPKeyProperty_Version, &uVer);

	bCantDoAnything = FALSE;

	// RSA
	if (uAlg == kPGPPublicKeyAlgorithm_RSA)
	{
		if (!bCanSign && !bCanEncrypt)
			bCantDoAnything = TRUE;

		if (uVer <= kPGPKeyVersion_V3)
		{
			if (bSecret)
			{
				if (bOnToken)
				{
					if (bRevoked) uIdx = IDX_RSATOKREVKEY;
					else if (bExpired) uIdx = IDX_RSATOKEXPKEY;
					else if (bDisabled) uIdx = IDX_RSATOKDISKEY;
					else uIdx = IDX_RSATOKKEY;
				}
				else
				{
					if (bRevoked) uIdx = IDX_RSASECREVKEY;
					else if (bExpired) uIdx = IDX_RSASECEXPKEY;
					else if (bDisabled) uIdx = IDX_RSASECDISKEY;
					else if (bSplit) uIdx = IDX_RSASECSHRKEY;
					else uIdx = IDX_RSASECKEY;
				}
			}
			else
			{
				if (bRevoked) uIdx = IDX_RSAPUBREVKEY;
				else if (bExpired) uIdx = IDX_RSAPUBEXPKEY;
				else if (bDisabled) uIdx = IDX_RSAPUBDISKEY;
				else uIdx = IDX_RSAPUBKEY;
			}
		}
		else
		{
			if (bSecret)
			{
				if (bOnToken)
				{
					if (bRevoked) uIdx = IDX_RSAV4TOKREVKEY;
					else if (bExpired) uIdx = IDX_RSAV4TOKEXPKEY;
					else if (bDisabled) uIdx = IDX_RSAV4TOKDISKEY;
					else uIdx = IDX_RSAV4TOKKEY;
				}
				else
				{
					if (bRevoked) uIdx = IDX_RSAV4SECREVKEY;
					else if (bExpired) uIdx = IDX_RSAV4SECEXPKEY;
					else if (bDisabled) uIdx = IDX_RSAV4SECDISKEY;
					else if (bSplit) uIdx = IDX_RSAV4SECSHRKEY;
					else uIdx = IDX_RSAV4SECKEY;
				}
			}
			else
			{
				if (bRevoked) uIdx = IDX_RSAV4PUBREVKEY;
				else if (bExpired) uIdx = IDX_RSAV4PUBEXPKEY;
				else if (bDisabled) uIdx = IDX_RSAV4PUBDISKEY;
				else uIdx = IDX_RSAV4PUBKEY;
			}
		}

	}
	// DSA/ElGamal
	else if (uAlg == kPGPPublicKeyAlgorithm_DSA)
	{
		if (bSecret)
		{
			if (bRevoked) uIdx = IDX_DSASECREVKEY;
			else if (bExpired) uIdx = IDX_DSASECEXPKEY;
			else if (bDisabled) uIdx = IDX_DSASECDISKEY;
			else if (bSplit) uIdx = IDX_DSASECSHRKEY;
			else uIdx = IDX_DSASECKEY;
		}
		else
		{
			if (bRevoked) uIdx = IDX_DSAPUBREVKEY;
			else if (bExpired) uIdx = IDX_DSAPUBEXPKEY;
			else if (bDisabled) uIdx = IDX_DSAPUBDISKEY;
			else uIdx = IDX_DSAPUBKEY;
		}
	}
	// Elliptic Curve
	else if (uAlg == kPGPPublicKeyAlgorithm_ECSign)
	{
		if (bSecret)
		{
			if (bRevoked) uIdx = IDX_ECSECREVKEY;
			else if (bExpired) uIdx = IDX_ECSECEXPKEY;
			else if (bDisabled) uIdx = IDX_ECSECDISKEY;
			else if (bSplit) uIdx = IDX_ECSECSHRKEY;
			else uIdx = IDX_ECSECKEY;
		}
		else
		{
			if (bRevoked) uIdx = IDX_ECPUBREVKEY;
			else if (bExpired) uIdx = IDX_ECPUBEXPKEY;
			else if (bDisabled) uIdx = IDX_ECPUBDISKEY;
			else uIdx = IDX_ECPUBKEY;
		}
	}
	// unknown
	else
	{
		uIdx = IDX_UNKNOWNKEY;
		bCantDoAnything = TRUE;
	}

	if (pbItalics)
		*pbItalics = bRevoked || bExpired || bDisabled || bCantDoAnything;
	return uIdx;
}


//	___________________________________________________
//
//  Determine the appropriate icon for a key, based on
//	its properties

PGPInt32
CLDetermineSubkeyIcon (
		PGPKeyDBObjRef	key)
{
	PGPBoolean bRevoked, bSecret, bDisabled, bExpired;
	PGPBoolean bOnToken;
	PGPUInt32 uIdx, uAlg, uVer;

	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_IsRevoked, &bRevoked);
	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_IsSecret, &bSecret);
	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_IsDisabled, &bDisabled);
	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_IsExpired, &bExpired);
	PGPGetKeyDBObjBooleanProperty (key,
			kPGPKeyProperty_IsOnToken, &bOnToken);

	PGPGetKeyDBObjNumericProperty (key,
			kPGPKeyProperty_AlgorithmID, &uAlg);
	PGPGetKeyDBObjNumericProperty (key,
			kPGPKeyProperty_Version, &uVer);

	// RSA
	if (uAlg == kPGPPublicKeyAlgorithm_RSA)
	{
		if (uVer <= kPGPKeyVersion_V3)
		{
			if (bSecret)
			{
				if (bOnToken)
				{
					if (bRevoked) uIdx = IDX_RSATOKREVKEY;
					else if (bExpired) uIdx = IDX_RSATOKEXPKEY;
					else if (bDisabled) uIdx = IDX_RSATOKDISKEY;
					else uIdx = IDX_RSATOKKEY;
				}
				else
				{
					if (bRevoked) uIdx = IDX_RSASECREVKEY;
					else if (bExpired) uIdx = IDX_RSASECEXPKEY;
					else if (bDisabled) uIdx = IDX_RSASECDISKEY;
					else uIdx = IDX_RSASECKEY;
				}
			}
			else
			{
				if (bRevoked) uIdx = IDX_RSAPUBREVKEY;
				else if (bExpired) uIdx = IDX_RSAPUBEXPKEY;
				else if (bDisabled) uIdx = IDX_RSAPUBDISKEY;
				else uIdx = IDX_RSAPUBKEY;
			}
		}
		else
		{
			if (bSecret)
			{
				if (bOnToken)
				{
					if (bRevoked) uIdx = IDX_RSAV4TOKREVKEY;
					else if (bExpired) uIdx = IDX_RSAV4TOKEXPKEY;
					else if (bDisabled) uIdx = IDX_RSAV4TOKDISKEY;
					else uIdx = IDX_RSAV4TOKKEY;
				}
				else
				{
					if (bRevoked) uIdx = IDX_RSAV4SECREVKEY;
					else if (bExpired) uIdx = IDX_RSAV4SECEXPKEY;
					else if (bDisabled) uIdx = IDX_RSAV4SECDISKEY;
					else uIdx = IDX_RSAV4SECKEY;
				}
			}
			else
			{
				if (bRevoked) uIdx = IDX_RSAV4PUBREVKEY;
				else if (bExpired) uIdx = IDX_RSAV4PUBEXPKEY;
				else if (bDisabled) uIdx = IDX_RSAV4PUBDISKEY;
				else uIdx = IDX_RSAV4PUBKEY;
			}
		}

	}
	// DSA/ElGamal
	else if ((uAlg == kPGPPublicKeyAlgorithm_ElGamal) ||
			 (uAlg == kPGPPublicKeyAlgorithm_DSA))
	{
		if (bSecret)
		{
			if (bRevoked) uIdx = IDX_DSASECREVKEY;
			else if (bExpired) uIdx = IDX_DSASECEXPKEY;
			else if (bDisabled) uIdx = IDX_DSASECDISKEY;
			else uIdx = IDX_DSASECKEY;
		}
		else
		{
			if (bRevoked) uIdx = IDX_DSAPUBREVKEY;
			else if (bExpired) uIdx = IDX_DSAPUBEXPKEY;
			else if (bDisabled) uIdx = IDX_DSAPUBDISKEY;
			else uIdx = IDX_DSAPUBKEY;
		}
	}
	// Elliptic Curve
	else if (uAlg == kPGPPublicKeyAlgorithm_ECEncrypt)
	{
		if (bSecret)
		{
			if (bRevoked) uIdx = IDX_ECSECREVKEY;
			else if (bExpired) uIdx = IDX_ECSECEXPKEY;
			else if (bDisabled) uIdx = IDX_ECSECDISKEY;
			else uIdx = IDX_ECSECKEY;
		}
		else
		{
			if (bRevoked) uIdx = IDX_ECPUBREVKEY;
			else if (bExpired) uIdx = IDX_ECPUBEXPKEY;
			else if (bDisabled) uIdx = IDX_ECPUBDISKEY;
			else uIdx = IDX_ECPUBKEY;
		}
	}
	// unknown
	else
	{
		uIdx = IDX_UNKNOWNKEY;
	}

	return uIdx;
}


//	___________________________________________________
//
//  Test the status of the specified path

PGPUInt32 PGPclExport
PGPclTestPathNameStatus (
		char*				pszPath)
{
	PGPUInt32	uReturn			= kPGPclInvalidPath;
	DWORD		dwAttr, dwErr;
	UINT		uPrevErrMode;

	uPrevErrMode = SetErrorMode (SEM_FAILCRITICALERRORS);

	dwAttr = GetFileAttributes (pszPath);
	if (dwAttr == 0xFFFFFFFF)
	{
		dwErr = GetLastError ();
		if (dwErr == ERROR_NOT_READY)
			uReturn = kPGPclMediaNotInserted;
	}
	else
	{
		uReturn = 0;

		if (dwAttr & FILE_ATTRIBUTE_DIRECTORY)
			uReturn |= kPGPclIsFolder;
		else
			uReturn |= kPGPclIsFile;

		if (dwAttr & FILE_ATTRIBUTE_READONLY)
			uReturn |= kPGPclIsReadOnly;

		if (SetFileAttributes (pszPath, FILE_ATTRIBUTE_NORMAL))
			SetFileAttributes (pszPath, dwAttr);
		else
		{
			uReturn |= kPGPclIsInReadOnlyFolder;
		}
	}

	SetErrorMode (uPrevErrMode);

	return uReturn;
}


//	___________________________________________________
//
//  Open the specified keyring files, doing some error checking

PGPError PGPclExport
PGPclOpenKeyrings (
		PGPContextRef				context,
		char*						pszPubPath,
		char*						pszPrivPath,
		PGPOpenKeyDBFileOptions		options,
		PGPKeyDBRef*				pkeydbOut)
{
	PGPError		err					= kPGPError_NoErr;
	PGPFileSpecRef	filerefPub			= kInvalidPGPFileSpecRef;
	PGPFileSpecRef	filerefPriv			= kInvalidPGPFileSpecRef;
	BOOL			bReadOnly			= FALSE;
	PGPUInt32		uStatus;

	*pkeydbOut = kInvalidPGPKeyDBRef;

	// test the pubring for various conditions
	uStatus = PGPclTestPathNameStatus (pszPubPath);

	if (uStatus & kPGPclMediaNotInserted)
	{
		err = kPGPError_CantOpenFile;
		goto done;
	}

	if (options & kPGPOpenKeyDBFileOptions_Mutable)
	{
		if (uStatus & (kPGPclIsReadOnly | kPGPclIsInReadOnlyFolder))
		{
			err = kPGPError_FilePermissions;
			goto done;
		}
	}

	// now do the same for the secring
	uStatus = PGPclTestPathNameStatus (pszPrivPath);

	if (uStatus & kPGPclMediaNotInserted)
	{
		err = kPGPError_CantOpenFile;
		goto done;
	}

	if (options & kPGPOpenKeyDBFileOptions_Mutable)
	{
		if (uStatus & (kPGPclIsReadOnly | kPGPclIsInReadOnlyFolder))
		{
			err = kPGPError_FilePermissions;
			goto done;
		}
	}

	err = PGPNewFileSpecFromFullPath (context,
			pszPubPath, &filerefPub);CKERR;
	err = PGPNewFileSpecFromFullPath (context,
			pszPrivPath, &filerefPriv);CKERR;
	err = PGPOpenKeyDBFile (context,
			options, filerefPub, filerefPriv, pkeydbOut); CKERR;

done:
	if (PGPFileSpecRefIsValid (filerefPub))
		PGPFreeFileSpec (filerefPub );
	if (PGPFileSpecRefIsValid (filerefPriv))
		PGPFreeFileSpec (filerefPriv);

	return err;
}


//	___________________________________________________
//
//  Open the keyring files specified by the prefs file

PGPError PGPclExport
PGPclOpenDefaultKeyrings (
		PGPContextRef				context,
		PGPOpenKeyDBFileOptions		options,
		PGPKeyDBRef*				pkeydbOut)
{
	PGPError		err;
	CHAR			szPubPath[MAX_PATH];
	CHAR			szPrivPath[MAX_PATH];

	err = PGPclGetPath (kPGPclPublicKeyringFile, 
			szPubPath, sizeof(szPubPath)); CKERR;

	err = PGPclGetPath (kPGPclPrivateKeyringFile, 
			szPrivPath, sizeof(szPrivPath)); CKERR;

	err = PGPclOpenKeyrings (context, 
			szPubPath, szPrivPath, options, pkeydbOut);

done:
	return err;
}


//	___________________________________________________
//
//  Get the default key specified in the prefs file

PGPError PGPclExport
PGPclGetDefaultKey (
		PGPKeyDBRef			keydb,
		PGPKeyDBObjRef*		pkeyOut)
{
	PGPError		err				= kPGPError_NoErr;
	PGPSize			size;
	PGPByte*		pbyte;
	PGPPrefRef		prefref;

	if (IsNull (pkeyOut))
		return kPGPError_BadParams;

	*pkeyOut = kInvalidPGPKeyDBObjRef;

	err = PGPclPeekClientLibPrefRefs (&prefref, NULL);

	if (IsntPGPError (err))
	{
		err = PGPGetPrefData (prefref, 
				kPGPPrefDefaultKeyID, &size, &pbyte);

		if (IsntPGPError (err))
		{
			if ((size == sizeof(PGPKeyID)) && IsntNull (pbyte))
				err = PGPFindKeyByKeyID (keydb, (PGPKeyID*)pbyte, pkeyOut);
			else
				err = kPGPError_ItemNotFound;

			PGPFreeData (pbyte);
		}
	}

	return err;
}


//	___________________________________________________
//
//  Get the default key specified in the prefs file, if it's private

PGPError PGPclExport
PGPclGetDefaultPrivateKey (
		PGPKeyDBRef			keydb,
		PGPKeyDBObjRef*		pkeyOut)
{
	PGPError	err;
	PGPBoolean	bSecret;

	err = PGPclGetDefaultKey (keydb, pkeyOut);
	if (IsntPGPError (err))
	{
		PGPGetKeyDBObjBooleanProperty (*pkeyOut,
				kPGPKeyProperty_IsSecret, &bSecret);
		if (!bSecret)
		{
			*pkeyOut = NULL;
			err = kPGPError_ItemNotFound;
		}
	}

	return err;
}


//	_______________________________________________
//
//  Construct string representation of number of key bits

VOID 
CLGetKeyBitsString (
		PGPKeyDBObjRef	key, 
		LPSTR			sz, 
		UINT			u) 
{
	UINT			uKeyBits;
	UINT			uSubKeyBits;
	PGPKeyDBObjRef	subkey;
	PGPKeyIterRef	keyiter;
	CHAR			szbuf[32];

	PGPGetKeyDBObjNumericProperty (key, 
			kPGPKeyProperty_Bits, &uKeyBits);
	wsprintf (szbuf, "%i", uKeyBits);

	// now try to get subkey bits
	PGPNewKeyIterFromKeyDB (PGPPeekKeyDBObjKeyDB (key), &keyiter);
	PGPKeyIterSeek (keyiter, key);
	PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_SubKey, &subkey);
	if (subkey) 
	{
		PGPGetKeyDBObjNumericProperty (subkey, 
				kPGPSubKeyProperty_Bits, &uSubKeyBits);
		wsprintf (szbuf, "%i/%i", uSubKeyBits, uKeyBits);
	}
	PGPFreeKeyIter (keyiter);

	lstrcpyn (sz, szbuf, u);
}


//	_______________________________________________
//
//  Execute file (as in WinExec)

PGPBoolean PGPclExport
PGPclExecute (
		char* 		pszPath,
		PGPUInt32	uCmdShow)
{
	SHELLEXECUTEINFO	sei;
	CHAR				szPath[MAX_PATH];
	CHAR				szParams[64];
	LPSTR				p;

	szParams[0] = '\0';
	lstrcpy (szPath, pszPath);

	p = strchr (szPath, '/');
	if (p)
	{
		lstrcpy (szParams, p);
		*p = '\0';
	}

	sei.cbSize			= sizeof(SHELLEXECUTEINFO);
	sei.fMask			= 0;
	sei.hwnd			= NULL;
	sei.lpDirectory		= NULL;
	sei.hInstApp		= NULL;

	sei.nShow			= uCmdShow;
	sei.lpVerb			= "open";
	sei.lpFile			= szPath;
	sei.lpParameters	= szParams;

	return (ShellExecuteEx (&sei));
}

//	____________________________
//
//  determine if module is installed

PGPBoolean PGPclExport
PGPclIsComponentInstalled (
		PGPclComponentIndex	index)
{
	PGPBoolean	bInstalled			= FALSE;
	HKEY		hkey;
	LONG		lResult;
	DWORD		dw, dwValueType, dwSize;
	CHAR		sz[128];
	CHAR		szKey[256];

	PGPclGetPath (kPGPclRegistryKey, szKey, sizeof(szKey));
	lstrcat (szKey, PGP_INSTALLEDCOMPONENTSKEY);

	switch (index) {
	case kPGPclUsersManual:		lstrcpy (sz, PGP_MANUALINSTALLED);		break;
	case kPGPclPGPkeys:			lstrcpy (sz, PGP_PGPKEYSINSTALLED);		break;
	case kPGPclPFIDS:			lstrcpy (sz, PGP_PFIDSINSTALLED);		break;
	case kPGPclPGPdisk:	
		lstrcpy (sz, PGP_PGPDISKINSTALLED);		
		break;
	case kPGPclCommandLine:		lstrcpy (sz, PGP_COMMANDLINEINSTALLED);	break;
	case kPGPclICQplugin:		lstrcpy (sz, PGP_ICQINSTALLED);			break;
	case kPGPclEudoraPlugin:	lstrcpy (sz, PGP_EUDORAINSTALLED);		break;
	case kPGPclExchangePlugin:	lstrcpy (sz, PGP_EXCHANGEINSTALLED);	break;
	case kPGPclOutlookExpressPlugin:
								lstrcpy (sz, PGP_OEINSTALLED);			break;
	case kPGPclLotusNotesPlugin:lstrcpy (sz, PGP_NOTESINSTALLED);		break;
	case kPGPclPGPlogin:		lstrcpy (sz, PGP_PGPLOGININSTALLED);	break;
	case kPGPclPGPtools:
		lstrcpy (sz, PGP_PGPTOOLSINSTALLED);	
		break;
	case kPGPclVPN:				lstrcpy (sz, PGP_VPNINSTALLED);			break;
	case kPGPclPGPnet:			lstrcpy (sz, PGP_VPNINSTALLED);			break;
	default :					return FALSE;
	}

	lResult = RegOpenKeyEx (HKEY_LOCAL_MACHINE, szKey, 0, KEY_READ, &hkey);

	if (lResult == ERROR_SUCCESS) 
	{
		// see if component is installed
		dwSize = sizeof(DWORD);
		lResult = RegQueryValueEx (
				hkey, sz, 0, &dwValueType, (LPBYTE)&dw, &dwSize);
		if (lResult == ERROR_SUCCESS)
		{
			if (dw == 1)
				bInstalled = TRUE;
		}

		if (!bInstalled && (index == kPGPclPGPnet))
		{
			lstrcpy (sz, PGP_PFIDSINSTALLED);
			dwSize = sizeof(DWORD);
			lResult = RegQueryValueEx (
					hkey, sz, 0, &dwValueType, (LPBYTE)&dw, &dwSize);
			if (lResult == ERROR_SUCCESS)
			{
				if (dw == 1)
					bInstalled = TRUE;
			}
		}

		RegCloseKey (hkey);
	}

	return bInstalled;
}


//	___________________________________________________
//
//	play the sound specified by the index.  must query the registry
//	to find the name of the sound file.

PGPError PGPclExport
PGPclPlaySound (
		PGPclSoundIndex		index,
		PGPBoolean			bSynchronous)
{
	HKEY		hkey;
	LONG		lResult;
	DWORD		dwFlags, dwValueType, dwSize;
	CHAR		sz[128];
	CHAR		szKey[256];
	CHAR		szPath[MAX_PATH];
	INT			ids;

	// intruder alert sound
	switch (index) {
	case kPGPclFirewallIntruderAlert:	ids = IDS_INTRUDERALERTSOUND;	break;
	case kPGPclNoSound:					return kPGPError_NoErr;
	default :							return kPGPError_BadParams;
	}

	LoadString (g_hInst, ids, sz, sizeof(sz));

	LoadString (g_hInst, IDS_SOUNDSKEY, szKey, sizeof(szKey));
	lstrcat (szKey, sz);
	LoadString (g_hInst, IDS_CURRENTSOUNDFILE, sz, sizeof(sz));
	lstrcat (szKey, sz);

	szPath[0] = '\0';
	lResult = RegOpenKeyEx (HKEY_CURRENT_USER, szKey, 0, KEY_READ, &hkey);
	if (lResult == ERROR_SUCCESS)
	{
		dwSize = sizeof(szPath);
		lResult = RegQueryValueEx (hkey, NULL, 0, &dwValueType, szPath, &dwSize);
		RegCloseKey (hkey);

		if (bSynchronous)
			dwFlags = SND_FILENAME | SND_SYNC;
		else
			dwFlags = SND_FILENAME | SND_ASYNC;

		PlaySound (szPath, NULL, dwFlags);
	}
	else
		MessageBeep (MB_ICONEXCLAMATION);

	return kPGPError_NoErr;
}


//	____________________________________
//
//	derive meta introducers from keyset

static PGPError 
sGetMetaIntroducers (
		PGPKeyDBRef		keydbMain,
		PGPKeySetRef*	pkeysetMeta)
{
	PGPPrefRef		prefref				= kInvalidPGPPrefRef;
	PGPError		err					= kPGPError_NoErr;
	PGPKeyIterRef	keyiter				= kInvalidPGPKeyIterRef;
	PGPKeyDBObjRef	key					= kInvalidPGPKeyDBObjRef;
	PGPKeySetRef	keysetMeta			= kInvalidPGPKeySetRef;
	PGPBoolean		bMargIsInvalid;
	PGPValidity		validityThreshold;

	// use prefs to determine validity threshold
	err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
	PGPGetPrefBoolean (prefref, kPGPPrefMarginalIsInvalid, &bMargIsInvalid);

	if (bMargIsInvalid) 
		validityThreshold = kPGPValidity_Complete;
	else
		validityThreshold = kPGPValidity_Marginal;

	// create the new keyset to collect the meta-introducer keys
	err = PGPNewEmptyKeySet (keydbMain, &keysetMeta); CKERR;

	// now we'll iterate the entire keyset
	err = PGPNewKeyIterFromKeyDB (keydbMain, &keyiter); CKERR;

	PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);
	while (PGPKeyDBObjRefIsValid (key)) 
	{
		PGPBoolean		bDisabled;
		PGPBoolean		bExpired;
		PGPBoolean		bRevoked;
		PGPBoolean		bMetaIntroducer;
		PGPValidity		validity;
		PGPKeyDBObjRef	userid;
		PGPKeyDBObjRef	sig;

		bMetaIntroducer = FALSE;

		// we'll exclude expired, revoked, disabled and invalid keys
		PGPGetKeyDBObjBooleanProperty (key, 
				kPGPKeyProperty_IsExpired, &bExpired);
		PGPGetKeyDBObjBooleanProperty (key, 
				kPGPKeyProperty_IsRevoked, &bRevoked);
		PGPGetKeyDBObjBooleanProperty (key, 
				kPGPKeyProperty_IsDisabled, &bDisabled);
		PGPGetPrimaryUserIDValidity (key, &validity);

		if (!bDisabled && !bExpired && !bRevoked &&
			(validity >= validityThreshold)) 
		{
			PGPKeyIterNextKeyDBObj (keyiter, 
					kPGPKeyDBObjType_UserID, &userid);

			while (PGPKeyDBObjRefIsValid (userid) &&
					!bMetaIntroducer) 
			{
				PGPKeyIterNextKeyDBObj (keyiter, 
						kPGPKeyDBObjType_Signature, &sig);

				while (PGPKeyDBObjRefIsValid (sig) &&
						!bMetaIntroducer) 
				{
					PGPBoolean		bExportable;
					PGPUInt32		uTrustLevel;

					PGPGetKeyDBObjBooleanProperty (sig, 
							kPGPSigProperty_IsRevoked, &bRevoked);
					PGPGetKeyDBObjBooleanProperty (sig, 
							kPGPSigProperty_IsExportable, &bExportable);
					PGPGetKeyDBObjNumericProperty  (sig, 
							kPGPSigProperty_TrustLevel, &uTrustLevel);

					if (!bRevoked && !bExportable && (uTrustLevel >= 2))
						 bMetaIntroducer = TRUE;

					PGPKeyIterNextKeyDBObj (keyiter, 
							kPGPKeyDBObjType_Signature, &sig);
				}
				PGPKeyIterNextKeyDBObj (keyiter, 
						kPGPKeyDBObjType_UserID, &userid);
			}
			
			// this is a meta introducer, add it to our growing collection
			if (bMetaIntroducer) 
			{
				err = PGPAddKey (key, keysetMeta); CKERR;
			}
		}
		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);
	}

	// return keyset
	*pkeysetMeta = keysetMeta;
	keysetMeta = kInvalidPGPKeySetRef;

done :
	if (PGPKeySetRefIsValid (keysetMeta))
		PGPFreeKeySet (keysetMeta);
	if (PGPKeyIterRefIsValid (keyiter))
		PGPFreeKeyIter (keyiter);
	
	return err;
}

//	____________________________________
//
//	find keys signed by specified key and update them from keyserver

static PGPError 
sFindAndUpdateKeysIntroducedBy (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwnd,
		PGPKeyDBObjRef		key,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBRef			keydbSigned)
{
	PGPError		err				= kPGPError_NoErr;
	PGPFilterRef	filter			= kInvalidPGPFilterRef;
	PGPKeyIterRef	keyiter			= kInvalidPGPKeyIterRef;
	PGPKeyDBRef		keydbFound		= kInvalidPGPKeyDBRef;

	PGPKeyID		keyidMeta;

	// get the keyid of this meta introducer key
	err = PGPGetKeyID (key, &keyidMeta); CKERR;

	// create a filter to find keys signed by this key
	err = PGPNewKeyDBObjDataFilter (context, kPGPSigProperty_KeyID,
			&keyidMeta, sizeof(PGPKeyID), kPGPMatchCriterion_Equal,
			&filter); CKERR;

	// go out and search the root keyserver with this filter
	err = PGPclSearchServerWithFilter (context, tlscontext,
			hwnd, filter, kPGPclRootServer, keydbMain, &keydbFound); CKERR;
	if (!PGPKeyDBRefIsValid (keydbFound)) 
		goto done;

	// we found some keys signed by the meta introducer, now iterate them
	err = PGPNewKeyIterFromKeyDB (keydbFound, &keyiter);
	PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);
 
	while (PGPKeyDBObjRefIsValid (key))
	{
		PGPBoolean		bSignedByMeta	= FALSE;
		PGPKeyDBObjRef	userid;
		PGPKeyDBObjRef	sig;
		PGPBoolean		bExportable;
		PGPUInt32		uTrustLevel;
		PGPKeyID		keyidSigner;
		PGPSize			size;

		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_UserID, &userid);

		while (PGPKeyDBObjRefIsValid (userid) &&
				!bSignedByMeta)
		{
			PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Signature, &sig);

			while (PGPKeyDBObjRefIsValid (sig) &&
					!bSignedByMeta)
			{
				err = PGPGetKeyDBObjDataProperty (sig, 
						kPGPSigProperty_KeyID, &keyidSigner,
						sizeof(PGPKeyID), &size); CKERR;
				err = PGPGetKeyDBObjBooleanProperty (sig, 
						kPGPSigProperty_IsExportable, &bExportable); CKERR;
				err = PGPGetKeyDBObjNumericProperty  (sig, 
						kPGPSigProperty_TrustLevel, &uTrustLevel); CKERR;

				// check if this cert was created by meta introducer
				// and if it has the appropriate properties
				if (bExportable && (uTrustLevel >= 1)) 
				{
					if (PGPCompareKeyIDs (&keyidMeta, &keyidSigner) == 0)
						bSignedByMeta = TRUE;
				}

				PGPKeyIterNextKeyDBObj (
						keyiter, kPGPKeyDBObjType_Signature, &sig);
			}

			PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_UserID, &userid);
		}
		
		// this key is one that we're looking for, add it to keyset
		if (bSignedByMeta) 
		{
			err = PGPCopyKeyDBObj (key, keydbSigned, NULL); CKERR;
		}

		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);
	}

done:
	if (PGPKeyIterRefIsValid (keyiter))
		PGPFreeKeyIter (keyiter);
	if (PGPKeyDBRefIsValid (keydbFound))
		PGPFreeKeyDB (keydbFound);
	if (PGPFilterRefIsValid (filter))
		PGPFreeFilter (filter);

	return err;
}

//	____________________________________
//
//	Auto update trusted introducers in keyring

PGPError PGPclExport 
PGPclAutoUpdateIntroducers (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwnd, 
		PGPKeyDBRef			keydbMain,
		BOOL				bForce) 
{
	PGPError		err					= kPGPError_NoErr;
	PGPKeySetRef	keysetMeta			= kInvalidPGPKeySetRef;
	PGPKeyDBRef		keydbUpdatedMeta	= kInvalidPGPKeyDBRef;
	PGPKeyDBRef		keydbSigned			= kInvalidPGPKeyDBRef;
	PGPKeyIterRef	keyiter				= kInvalidPGPKeyIterRef;
	HCURSOR			hcursorOld			= NULL;

	PGPKeyDBObjRef		key;
	PGPInt32			iNumKeys;

	// get meta introducers from main keydb
	err = sGetMetaIntroducers (keydbMain, &keysetMeta); CKERR;

	// if we have some, update them from root keyserver
	PGPCountKeys (keysetMeta, &iNumKeys);
	if (iNumKeys > 0) 
	{
		err = PGPclUpdateKeySetFromServer (context, tlscontext, hwnd, 
				keysetMeta, kPGPclRootServer, keydbMain, 
				&keydbUpdatedMeta); CKERR;
	}

	// if we found keys on keyserver, add them to our main keyset
	if (PGPKeyDBRefIsValid (keydbUpdatedMeta)) 
	{
		if (hwnd)
			hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));

		err = PGPCopyKeys (PGPPeekKeyDBRootKeySet (keydbUpdatedMeta), 
				keydbMain, NULL); 

		if (hcursorOld)
			SetCursor (hcursorOld);
		CKERR;
	}

	// create new keydb to collect keys signed by the meta introducers
	err = PGPNewKeyDB (context, &keydbSigned); CKERR;

	// now prepare to iterate through the meta introducers
	err = PGPNewKeyIterFromKeySet (keysetMeta, &keyiter); CKERR;
	PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key); CKERR; 

	while (PGPKeyDBObjRefIsValid (key))
	{
		// go look for keys signed by this meta introducer
		err = sFindAndUpdateKeysIntroducedBy (context, tlscontext,
					hwnd, key, keydbMain, keydbSigned); CKERR;

		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key); 
	}
	CKERR;

	// now add the updated signed keys to our main keyset
	if (bForce) 
	{
		if (hwnd)
			hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));

		err = PGPCopyKeys (PGPPeekKeyDBRootKeySet (keydbSigned), 
				keydbMain, NULL);

		if (hcursorOld)
			SetCursor (hcursorOld);
	}
	else 
	{
		PGPCountKeys (PGPPeekKeyDBRootKeySet (keydbSigned), &iNumKeys);
		if (iNumKeys > 0)
		{
			err = PGPclImportKeys (context, tlscontext, hwnd,
					PGPPeekKeyDBRootKeySet (keydbSigned), keydbMain,
					kPGPclDefaultImportFlags);
		}
		else
		{
			PGPclMessageBox (hwnd, IDS_AUTOUPDATECAPTION, 
					IDS_NONEWTRUSTEDINTROS, MB_OK|MB_ICONEXCLAMATION);
		}
	}

done:
	if (PGPKeyIterRefIsValid (keyiter))
		PGPFreeKeyIter (keyiter);
	if (PGPKeySetRefIsValid (keysetMeta))
		PGPFreeKeySet (keysetMeta);
	if (PGPKeyDBRefIsValid (keydbUpdatedMeta))
		PGPFreeKeyDB (keydbUpdatedMeta);
	if (PGPKeyDBRefIsValid (keydbSigned))
		PGPFreeKeyDB (keydbSigned);

	return err;
}


//	__________________________________________________________
//
//  Schedule the next CRL update

PGPError PGPclExport
PGPclScheduleNextCRLUpdate (
		PGPContextRef	context, 
		PGPKeyDBRef		keydbMain)
{
	PGPError	err					= kPGPError_NoErr;

	PGPPrefRef			prefref		= kInvalidPGPPrefRef;

	PGPKeyDBObjRef		keyRootCA;
	PGPKeyDBObjRef		sigRootCA;
	PGPBoolean			bAutoUpdate;
	PGPBoolean			bHasCRL;
	PGPUInt32			uNextUpdate;

	if(PGPlnEnterprise())
	{
		err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
		err = PGPGetPrefBoolean (prefref, 
				kPGPPrefAutoUpdateX509CRL, &bAutoUpdate); CKERR;
		if (bAutoUpdate)
		{
			err = PGPclGetRootCACertPrefs (context, 
					prefref, keydbMain, &sigRootCA); CKERR;
			keyRootCA = PGPPeekKeyDBObjKey (sigRootCA);

			err = PGPGetKeyDBObjBooleanProperty (keyRootCA, 
					kPGPKeyProperty_HasCRL, &bHasCRL); CKERR;

			if (bHasCRL)
			{
				err = PGPGetKeyDBObjTimeProperty (keyRootCA, 
						kPGPKeyProperty_CRLNextUpdate, &uNextUpdate); CKERR;

				err = PGPSetPrefNumber (prefref, 
						kPGPPrefNextAutoCRLUpdate, uNextUpdate); CKERR;

				err = PGPclFlushClientLibPrefs (prefref, NULL);
			}
			CKERR;
		}
	}
done:
	return err;
}


//	____________________________________________________________
//
//	determine if we are to post help system debugging info

static BOOL
sHelpDebugger (VOID)
{
	BOOL		bDebug			= FALSE;

	HKEY		hkey;
	LONG		lResult;
	DWORD		dw, dwValueType, dwSize;
	CHAR		szKey[256];

	PGPclGetPath (kPGPclRegistryKey, szKey, sizeof(szKey));
	lResult = RegOpenKeyEx (
			HKEY_LOCAL_MACHINE, szKey, 0, KEY_READ, &hkey);

	if (lResult == ERROR_SUCCESS) 
	{
		dwSize = sizeof(DWORD);
		lResult = RegQueryValueEx (hkey,
				"DebugHelp", 0, &dwValueType, (LPBYTE)&dw, &dwSize);
		if (lResult == ERROR_SUCCESS)
		{
			if (dw == 1)
				bDebug = TRUE;
		}

		RegCloseKey (hkey);
	}

	return bDebug;
}


//	__________________________________________________________
//
//  HtmlHelp API convenience wrapper

BOOL PGPclExport
PGPclHtmlHelp (
		HWND	hwnd, 
		UINT	uMsgOrHelpType, 
		WPARAM	wParam, 
		LPARAM	lParam,
		char*	pszHelpFile,
		DWORD*	pdwIDarray)
{
	CHAR	sz[MAX_PATH];

	if ((UINT)pszHelpFile < kPGPclNumPathIndices)
		PGPclGetPath ((PGPclPathIndex)pszHelpFile, sz, sizeof(sz));
	else
		lstrcpy (sz, pszHelpFile);

	switch (uMsgOrHelpType) {
	case kPGPclHelpContentsAndIndex :

		if (sHelpDebugger ())
		{
			CHAR	szDebugMessage[512];

			wsprintf (szDebugMessage, 
					"About to call HtmlHelp with\n"
					"    hwndCaller = 0x%X\n"
					"    pszFile = %s\n"
					"    uCommand = HH_DISPLAY_TOPIC\n"
					"    dwData = 0",
					GetDesktopWindow (), sz);

			MessageBox (NULL, szDebugMessage, 
					"PGP Help Debug Info", MB_OK|MB_ICONINFORMATION);
		}

		HtmlHelp (GetDesktopWindow (), sz, HH_DISPLAY_TOPIC, 0);
		return TRUE;

	case kPGPclHelpContext :
	case kPGPclHelpMoreInformation :
	{
		lstrcat (sz, ">context");

		if (sHelpDebugger ())
		{
			CHAR	szDebugMessage[512];

			wsprintf (szDebugMessage, 
					"About to call HtmlHelp with\n"
					"    hwndCaller = 0x%X\n"
					"    pszFile = %s\n"
					"    uCommand = HH_HELP_CONTEXT\n"
					"    dwData (ID) = %i",
					hwnd, sz, (DWORD)pdwIDarray);

			MessageBox (NULL, szDebugMessage, 
					"PGP Help Debug Info", MB_OK|MB_ICONINFORMATION);
		}

		HtmlHelp (hwnd, sz, HH_HELP_CONTEXT, (DWORD)pdwIDarray);
		return TRUE;
	}

	case WM_HELP :
	{
		LPHELPINFO	phi = (LPHELPINFO)lParam;

		lstrcat (sz, "::/context.txt");

		if (phi->iCtrlId != -1)
		{
			if (sHelpDebugger ())
			{
				CHAR	szDebugMessage[512];

				INT		i = 0;
				HWND	hwndControl, hwndParent;

				hwndParent = GetParent (((LPHELPINFO)lParam)->hItemHandle);

				while (hwndControl = GetDlgItem (hwndParent, pdwIDarray[i]))
				{
					if (hwndControl == ((LPHELPINFO)lParam)->hItemHandle)
						break;
					i += 2;
				}
				i += 1;

				wsprintf (szDebugMessage, 
					"About to call HtmlHelp with\n"
					"    hwndCaller = 0x%X\n"
					"    pszFile = %s\n"
					"    uCommand = HH_TP_HELP_WM_HELP\n"
					"    ID = %i",
					((LPHELPINFO)lParam)->hItemHandle, sz, pdwIDarray[i]);

				MessageBox (NULL, szDebugMessage, 
						"PGP Help Debug Info", MB_OK|MB_ICONINFORMATION);
			}

			HtmlHelp (((LPHELPINFO)lParam)->hItemHandle, sz, 
					HH_TP_HELP_WM_HELP, (DWORD)pdwIDarray);
		}
		return TRUE;
	}

	case WM_CONTEXTMENU :
	{
		lstrcat (sz, "::/context.txt");

		if ((HWND)wParam != hwnd)
		{
			if (sHelpDebugger ())
			{
				CHAR	szDebugMessage[512];

				INT		i = 0;
				HWND	hwndThis, hwndControl, hwndParent;

				// if control is a UTF8Edit control, this will get the 
				// handle to the edit control (as opposed to the richedit control).
				hwndThis = UTF8EditGetEditHandle ((HWND)wParam);
				hwndParent = GetParent (hwndThis);

				while (hwndControl = GetDlgItem (hwndParent, pdwIDarray[i]))
				{
					if (hwndControl == hwndThis)
						break;
					i += 2;
				}
				i += 1;

				wsprintf (szDebugMessage, 
					"About to call HtmlHelp with\n"
					"    hwndCaller = 0x%X\n"
					"    pszFile = %s\n"
					"    uCommand = HH_TP_HELP_CONTEXTMENU\n"
					"    ID = %i",
					wParam, sz, pdwIDarray[i]);

				MessageBox (NULL, szDebugMessage, 
						"PGP Help Debug Info", MB_OK|MB_ICONINFORMATION);
			}

			HtmlHelp ((HWND)wParam, sz, 
					HH_TP_HELP_CONTEXTMENU, (DWORD)pdwIDarray); 
		}
		return TRUE;
	}

	default :
		return FALSE;
	}
}


//	__________________________________________________________
//
//  search SharedDlls registry key for DLL name

static BOOL
sFindDllPathInRegistry (
		LPSTR	pszDllName,
		LPSTR	pszDllPath)
{
	HKEY		hkey;
	LONG		lResult;
	DWORD		dwLen, dwSize, dwIndex;
	CHAR		sz[MAX_PATH];

	dwLen = lstrlen (pszDllName);
	if (dwLen == 0)
		return FALSE;

	lstrcpy (sz, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\SharedDlls");
	lResult = RegOpenKeyEx (HKEY_LOCAL_MACHINE, sz, 0, KEY_READ, &hkey);

	dwIndex = 0;
	pszDllPath[0] = '\0';

	if (lResult == ERROR_SUCCESS)
	{
		while ((lResult == ERROR_SUCCESS) &&
				(pszDllPath[0] == '\0'))
		{
			dwSize = sizeof(sz);
			lResult = RegEnumValue (hkey, dwIndex,
					sz, &dwSize, NULL, NULL, NULL, NULL);

			if (lstrcmpi (pszDllName, &sz[dwSize-dwLen]) == 0)
				lstrcpy (pszDllPath, sz);

			dwIndex++;
		}

		RegCloseKey (hkey);
	}

	return (pszDllPath[0] != '\0');
}


//	__________________________________________________________
//
//  pass path of token support DLL to SDK

PGPError
CLSetTokenDll (
		PGPPrefRef	prefref,
		LPSTR		pszDllName,
		BOOL		bTestOnly)
{
	PGPError	err				= kPGPError_NoErr;
	BOOL		bClientPrefs	= FALSE;
	BOOL		bReturn			= FALSE;

    char		szDllName[MAX_PATH];
    char		szDllPath[MAX_PATH];

	szDllName[0] = '\0';

	// use passed in prefref or get clientlib prefref
	if (!PGPPrefRefIsValid (prefref))
	{
		err = PGPclPeekClientLibPrefRefs (&prefref, NULL);
		if (IsPGPError (err))
			return err;
		bClientPrefs = TRUE;
	}

	// use passed in DLL name or get it from prefs
	if (IsntNull (pszDllName))
		lstrcpyn (szDllName, pszDllName, sizeof(szDllName));
	else
	{
		PGPGetPrefStringBuffer (prefref, kPGPPrefPKCS11DrvFile, 
				sizeof(szDllName), szDllName);
	}

	if (szDllName[0] == '\0')
	{
		// disable token support
		PGPSetPKCS11DrvFile (szDllName);
	}
	else
	{
		// if DLL name is path, then just use it as is
		if (strchr (szDllName, '\\'))
		{
			if (!bTestOnly)
				PGPSetPKCS11DrvFile (szDllName);
			bReturn = TRUE;
		}
		// otherwise look around for the file
		else
		{
			// first look in system path
			GetSystemDirectory (szDllPath, sizeof(szDllPath));
			if (szDllPath[lstrlen(szDllPath) -1] != '\\')
				lstrcat (szDllPath, "\\");

			lstrcat (szDllPath, szDllName);

			if (PGPclTestPathNameStatus (szDllPath) & kPGPclIsFile)
			{
				if (!bTestOnly)
					PGPSetPKCS11DrvFile (szDllPath);
				bReturn = TRUE;
			}
			else
			{
				// then look in SharedDlls registry key
				if (sFindDllPathInRegistry (szDllName, szDllPath))
				{
					if (PGPclTestPathNameStatus (szDllPath) & kPGPclIsFile)
					{
						if (!bTestOnly)
							PGPSetPKCS11DrvFile (szDllPath);
						bReturn = TRUE;
					}
				}
			}

			// if not just testing, then set prefs to full path
			if (bReturn && !bTestOnly)
			{
				PGPUInt32	uFlags	= 0;

				PGPGetPrefFlags (prefref, kPGPPrefPKCS11DrvFile, &uFlags);
				if (!(uFlags & kPGPPrefFlags_DisableGUI))
				{
					PGPSetPrefString (prefref, 
							kPGPPrefPKCS11DrvFile, szDllPath);
					if (bClientPrefs)
						PGPclFlushClientLibPrefs (prefref, NULL);
				}
			}
		}
	}

	if (bReturn)
		return kPGPError_NoErr;
	else
		return kPGPError_FileNotFound;
}


//	______________________________________________
//
//  get the primary userid name string for a key, ensuring 
//	that string is in UTF-8 format

PGPError PGPclExport
PGPclGetPrimaryUserIDNameUTF8 (
		PGPKeyDBObjRef	key,
		LPSTR			pszName,
		PGPUInt32		uMaxLen,
		PGPUInt32*		puLen)
{
	CHAR		sz[kPGPMaxUserIDSize];
	PGPUInt32	u;
	PGPError	err;

	err = PGPGetPrimaryUserIDName (key, pszName, uMaxLen, puLen);

	// if not UTF-8, convert from local code page
	if (IsntPGPError (err))
	{
		if (pgpIsntUTF8String (pszName, *puLen))
		{
			u = *puLen;
			lstrcpyn (sz, pszName, u+1);
			err = pgpLocalStringToUTF8 (0, 
					GetACP(), sz, u, pszName, uMaxLen, puLen);
		}
	}

	return err;
}


//	______________________________________________
//
//  get the name string for a userid, ensuring that string
//	is in UTF-8 format

PGPError PGPclExport
PGPclGetUserIDNameUTF8 (
		PGPKeyDBObjRef	userid,
		LPSTR			pszName,
		PGPUInt32		uMaxLen,
		PGPUInt32*		puLen)
{
	PGPError	err;

	err = PGPGetKeyDBObjDataProperty (userid,
			kPGPUserIDProperty_Name, pszName, uMaxLen, puLen);

	// if not UTF-8, convert from local code page
	if (IsntPGPError (err))
	{
		if (pgpIsntUTF8String (pszName, *puLen))
		{
			CHAR		sz[kPGPMaxUserIDSize];
			PGPUInt32	u;

			u = *puLen;
			lstrcpyn (sz, pszName, u+1);
			err = pgpLocalStringToUTF8 (0, 
					GetACP(), sz, u, pszName, uMaxLen, puLen);
		}
	}

	return err;
}


//	______________________________________________
//
//  get the primary userid name string for a key, converting
//	from UTF-8 if necessary

PGPError PGPclExport
PGPclGetPrimaryUserIDName (
		PGPKeyDBObjRef	key,
		LPSTR			pszName,
		PGPUInt32		uMaxLen,
		PGPUInt32*		puLen)
{
	PGPError	err;

	err = PGPGetPrimaryUserIDName (key, pszName, uMaxLen, puLen);

	// if UTF-8, convert to local code page
	if (IsntPGPError (err))
	{
		if (!pgpIsntUTF8String (pszName, *puLen))
		{
			CHAR		sz[kPGPMaxUserIDSize];
			PGPUInt32	u;

			u = *puLen;
			lstrcpyn (sz, pszName, u+1);
			err = pgpUTF8StringToLocal (0, 
					GetACP(), sz, u, pszName, uMaxLen, puLen);
		}
	}

	return err;
}


//	______________________________________________
//
//  get the primary userid name string for a key in UCS2

PGPError PGPclExport
PGPclGetPrimaryUserIDNameUCS2(
		PGPKeyDBObjRef	key,
		LPWSTR			pwszName,
		PGPUInt32		uMaxLen,
		PGPUInt32*		puLen)
{
	CHAR		sz[kPGPMaxUserIDSize];
	PGPUInt32	u;
	PGPError	err;

	err = PGPGetPrimaryUserIDName (key, sz, sizeof(sz), &u);

	// if not UTF-8, convert from local code page
	if (IsntPGPError (err))
	{
		if (pgpIsntUTF8String (sz, u))
		{
			CHAR sz2[kPGPMaxUserIDSize];
			lstrcpyn (sz2, sz, u+1);
			err = pgpLocalStringToUTF8 (0, 
					GetACP(), sz2, u, sz, sizeof(sz), &u);
		}
	}

	if (IsntPGPError (err))
		err = pgpUTF8StringToUCS2 (sz, u, pwszName, uMaxLen, puLen);

	return err;
}


//	______________________________________________
//
//  get the name string for a userid, converting
//	from UTF-8 if necessary

PGPError PGPclExport
PGPclGetUserIDName (
		PGPKeyDBObjRef	userid,
		LPSTR			pszName,
		PGPUInt32		uMaxLen,
		PGPUInt32*		puLen)
{
	PGPError	err;

	err = PGPGetKeyDBObjDataProperty (userid,
			kPGPUserIDProperty_Name, pszName, uMaxLen, puLen);

	// if UTF-8, convert to local code page
	if (IsntPGPError (err))
	{
		if (pgpIsntUTF8String (pszName, *puLen))
		{
			CHAR		sz[kPGPMaxUserIDSize];
			PGPUInt32	u;

			u = *puLen;
			lstrcpyn (sz, pszName, u+1);
			err = pgpUTF8StringToLocal (0, 
					GetACP(), sz, u, pszName, uMaxLen, puLen);
		}
	}

	return err;
}


//	______________________________________________
//
//  post a messagebox, but using UTF8 strings
//  NB: MessageBoxW exists on all Win32 platforms

PGPUInt32 PGPclExport
PGPclMessageBoxUTF8 (
		HWND		hwnd,
		CHAR*		pszTextUTF8,
		CHAR*		pszCaptionUTF8,
		UINT		uType)
{
	PGPUInt32	uReturn		= 0;
	PGPUInt32	uLen;
	WCHAR*		pwszText	= NULL;
	WCHAR*		pwszCaption	= NULL;

	pgpUTF8StringToUCS2 (pszTextUTF8, 
			kPGPUnicodeNullTerminated, NULL, 0, &uLen);
	if (uLen)
	{
		pwszText = clAlloc ((uLen+1) * sizeof(WCHAR));
		if (pwszText)
		{
			pgpUTF8StringToUCS2 (pszTextUTF8, 
					kPGPUnicodeNullTerminated, pwszText, uLen, NULL);

			pgpUTF8StringToUCS2 (pszCaptionUTF8, 
					kPGPUnicodeNullTerminated, NULL, 0, &uLen);

			if (uLen)
			{
				pwszCaption = clAlloc ((uLen+1) * sizeof(WCHAR));
				if (pwszText)
				{
					pgpUTF8StringToUCS2 (pszCaptionUTF8, 
							kPGPUnicodeNullTerminated, pwszCaption, uLen, NULL);

					uReturn = MessageBoxW (hwnd, pwszText, pwszCaption, uType);
				}
			}
		}
	}

	if (pwszText)
		clFree (pwszText);
	if (pwszCaption)
		clFree (pwszCaption);

	return uReturn;
}
