/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	CLnewuser.c - handle initializing things for new user
	
	$Id: CLnewuser.c,v 1.11 2002/10/21 19:54:17 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"	

// project header files
#include "PGPclx.h"
#include "pgpWin32IPC.h"
#include <commdlg.h>

#define NEWUSER_CANCEL				0
#define NEWUSER_DISABLE				1
#define NEWUSER_EXISTINGKEYRINGS	2
#define NEWUSER_NEWUSER				3

#define NEWUSERWIZ_INTRO			0
#define NEWUSERWIZ_ENABLE			1
#define NEWUSERWIZ_TYPE				2
#define NEWUSERWIZ_FILES			3
#define NEWUSERWIZ_DONE				4
#define NUM_WIZ_PAGES				5

// external globals
extern HINSTANCE g_hInst;

// typedefs
typedef struct
{
	PGPContextRef		context;
	PGPtlsContextRef	tlscontext;
	PGPPrefRef			prefref;
	HWND				hwndWiz;
	HWND				hwndParent;

	HBITMAP				hbitmapIntro;
	HBITMAP				hbitmapDone;
	HPALETTE			hpaletteIntro;
	HPALETTE			hpaletteDone;
	HFONT				hfontIntro;
	HFONT				hfontInterior;

	BOOL				bEnabled;
	BOOL				bNewUser;
	BOOL				bFinishSelected;
	CHAR				szPubring[MAX_PATH];
	CHAR				szSecring[MAX_PATH];
} NEWUSERSTRUCT, *PNEWUSERSTRUCT;


//	____________________________________
//
//  let user browse to keyring

static BOOL
sBrowseKeyring (
		HWND	hwnd,
		BOOL	bSecring,
		LPSTR	pszFile,
		INT		iLen)
{
	CHAR			szInitDir[MAX_PATH];
	CHAR			szFile[MAX_PATH];
	CHAR			szFilter[128];
	CHAR			szTitle[128];
	CHAR			szDefExt[16];
	OPENFILENAME	ofn;
	CHAR*			p;

	if (bSecring)
	{
		LoadString (g_hInst, IDS_PRIVKEYCAPTION, szTitle, sizeof(szTitle));
		LoadString (g_hInst, IDS_DEFPRIVRINGEXT, szDefExt, sizeof(szDefExt));
		LoadString (g_hInst, IDS_PRIVKEYRINGFILTER, szFilter, sizeof(szFilter));
	}
	else
	{
		LoadString (g_hInst, IDS_PUBKEYCAPTION, szTitle, sizeof(szTitle));
		LoadString (g_hInst, IDS_DEFPUBRINGEXT, szDefExt, sizeof(szDefExt));
		LoadString (g_hInst, IDS_PUBKEYRINGFILTER, szFilter, sizeof(szFilter));
	}
	while (p = strrchr (szFilter, '@')) *p = '\0';

	lstrcpy (szInitDir, pszFile);
	p = strrchr (szInitDir, '\\');
	if (p)
		*p = '\0';

	szFile[0] = '\0';

	ofn.lStructSize			= sizeof (ofn);
	ofn.hwndOwner			= hwnd;
	ofn.hInstance			= (HANDLE)g_hInst;
	ofn.lpstrFilter			= szFilter;
	ofn.lpstrCustomFilter	= (LPTSTR)NULL;
	ofn.nMaxCustFilter		= 0L;
	ofn.nFilterIndex		= 1L;
	ofn.lpstrFile			= szFile;
	ofn.nMaxFile			= iLen;
	ofn.lpstrFileTitle		= NULL;
	ofn.nMaxFileTitle		= 0;
	ofn.lpstrInitialDir		= szInitDir;
	ofn.lpstrTitle			= szTitle;
	ofn.Flags				= OFN_HIDEREADONLY|OFN_SHAREAWARE|OFN_NOCHANGEDIR;
	ofn.nFileOffset			= 0;
	ofn.nFileExtension		= 0;
	ofn.lpstrDefExt			= szDefExt;
	ofn.lCustData			= 0;

	if (GetOpenFileName (&ofn))
	{
		lstrcpy (pszFile, szFile);
		return TRUE;
	}

	return FALSE;
}


//	______________________________________________
//
//  Dialog procedure for handling beginning introductory panel

static LRESULT WINAPI
sNewUserWizIntroDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	BOOL			bReturnCode = FALSE;
	PNEWUSERSTRUCT	pnus		= NULL;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pnus = (PNEWUSERSTRUCT) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pnus);

		// center dialog on screen
		PGPclCenterWindowOnMonitor (GetParent(hwnd), kPGPclWindowMonitor);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pnus->hfontIntro, (LPARAM)FALSE);
		break;
	}

	case WM_ACTIVATE :
		InvalidateRect (hwnd, NULL, TRUE);
		break;

	case WM_PAINT :
		pnus = (PNEWUSERSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		if (pnus->hpaletteIntro)
		{
			PAINTSTRUCT ps;
			HDC			hDC = BeginPaint (hwnd, &ps);

			SelectPalette (hDC, pnus->hpaletteIntro, FALSE);
			RealizePalette (hDC);
			EndPaint (hwnd, &ps);
		}
		break;

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch(pnmh->code) {
		case PSN_SETACTIVE:
			pnus = (PNEWUSERSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			PostMessage(GetParent(hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_NEXT);
			SendDlgItemMessage(hwnd, IDC_WIZBITMAP, STM_SETIMAGE,
					IMAGE_BITMAP, (LPARAM) pnus->hbitmapIntro);

			SetWindowLong(hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;
		}

		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for enabling PGP panel

static LRESULT WINAPI
sNewUserWizEnableDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	BOOL			bReturnCode = FALSE;
	PNEWUSERSTRUCT	pnus		= NULL;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;
		pnus = (PNEWUSERSTRUCT) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pnus);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pnus->hfontInterior, (LPARAM)FALSE);

		CheckRadioButton (hwnd, 
				IDC_ENABLEPGP, IDC_DISABLEPGP, IDC_ENABLEPGP);
		break;
	}

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch (pnmh->code) {
		case PSN_SETACTIVE:
			pnus = (PNEWUSERSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			PostMessage(GetParent(hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_NEXT|PSWIZB_BACK);

			SetWindowLong(hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;

		case PSN_KILLACTIVE :
			pnus = (PNEWUSERSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			if (IsDlgButtonChecked (hwnd, IDC_ENABLEPGP) == BST_CHECKED)
				pnus->bEnabled = TRUE;
			else
				pnus->bEnabled = FALSE;

			SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for user type panel

static LRESULT WINAPI
sNewUserWizTypeDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	BOOL			bReturnCode = FALSE;
	PNEWUSERSTRUCT	pnus		= NULL;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;
		pnus = (PNEWUSERSTRUCT) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pnus);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pnus->hfontInterior, (LPARAM)FALSE);

		CheckRadioButton (hwnd, 
				IDC_NEWUSER, IDC_EXISTINGKEYRINGS, IDC_NEWUSER);
		break;
	}

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch (pnmh->code) {
		case PSN_SETACTIVE:
			pnus = (PNEWUSERSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			PostMessage (GetParent (hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_BACK|PSWIZB_NEXT);

			if (!pnus->bEnabled)
				SetWindowLong(hwnd, DWL_MSGRESULT, -1L);	// skip page
			else
				SetWindowLong(hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;

		case PSN_KILLACTIVE :
			pnus = (PNEWUSERSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			if (IsDlgButtonChecked (hwnd, IDC_NEWUSER) == BST_CHECKED)
				pnus->bNewUser = TRUE;
			else
				pnus->bNewUser = FALSE;

			SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for keyring file selection panel

static LRESULT WINAPI
sNewUserWizFileDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	BOOL			bReturnCode = FALSE;
	PNEWUSERSTRUCT	pnus		= NULL;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pnus = (PNEWUSERSTRUCT) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pnus);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pnus->hfontInterior, (LPARAM)FALSE);
		break;
	}

	case WM_COMMAND :
	{	
		INT		iLenPub, iLenSec;

		pnus = (PNEWUSERSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		switch (LOWORD (wParam)) {
		case IDC_PUBRINGBROWSE :
			if (sBrowseKeyring (hwnd, FALSE, pnus->szPubring, MAX_PATH))
				SetDlgItemText (hwnd, IDC_PUBRING, pnus->szPubring);
			break;

		case IDC_SECRINGBROWSE :
			if (sBrowseKeyring (hwnd, TRUE, pnus->szSecring, MAX_PATH))
				SetDlgItemText (hwnd, IDC_SECRING, pnus->szSecring);
			break;
		}

		iLenPub = SendDlgItemMessage (
				hwnd, IDC_PUBRING, WM_GETTEXTLENGTH, 0, 0);
		iLenSec = SendDlgItemMessage (
				hwnd, IDC_SECRING, WM_GETTEXTLENGTH, 0, 0);

		if ((iLenPub > 0) && (iLenSec > 0))
		{
			PostMessage (GetParent (hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_BACK|PSWIZB_NEXT);
		}
		break;
	}

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch (pnmh->code) {
		case PSN_SETACTIVE:
			pnus = (PNEWUSERSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			PostMessage (GetParent (hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_BACK);

			if (!pnus->bEnabled || pnus->bNewUser)
				SetWindowLong(hwnd, DWL_MSGRESULT, -1L);	// skip page
			else
				SetWindowLong(hwnd, DWL_MSGRESULT, 0L);

			bReturnCode = TRUE;
			break;
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Dialog procedure for "Finish" dialog

static LRESULT WINAPI
sNewUserWizDoneDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	BOOL			bReturnCode = FALSE;
	PNEWUSERSTRUCT	pnus		= NULL;

	switch (uMsg) {
	case WM_INITDIALOG:
	{
		PROPSHEETPAGE *ppspMsgRec = (PROPSHEETPAGE *) lParam;

		pnus = (PNEWUSERSTRUCT) ppspMsgRec->lParam;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)pnus);

		SendDlgItemMessage (hwnd, IDC_HEADING, WM_SETFONT,
				(WPARAM)pnus->hfontIntro, (LPARAM)FALSE);
		return TRUE;
	}

	case WM_PAINT :
		pnus = (PNEWUSERSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		if (pnus->hpaletteDone)
		{
			PAINTSTRUCT ps;
			HDC			hDC = BeginPaint (hwnd, &ps);

			SelectPalette (hDC, pnus->hpaletteDone, FALSE);
			RealizePalette (hDC);
			EndPaint (hwnd, &ps);
		}
		break;

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		switch (pnmh->code) {
		case PSN_SETACTIVE:
			pnus = (PNEWUSERSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			PostMessage (GetParent (hwnd),
					PSM_SETWIZBUTTONS, 0, PSWIZB_BACK|PSWIZB_FINISH);

			SendDlgItemMessage (hwnd, IDC_WIZBITMAP, STM_SETIMAGE,
					IMAGE_BITMAP, (LPARAM) pnus->hbitmapDone);

			if (pnus->bEnabled)
			{
				ShowWindow (GetDlgItem (hwnd, IDC_RUNPGPKEYSTEXT), SW_HIDE);
				ShowWindow (GetDlgItem (hwnd, IDC_KEYRINGTEXT), SW_SHOW);

				if (pnus->bNewUser)
				{
					ShowWindow (GetDlgItem (hwnd, IDC_KEYGENTEXT1), SW_SHOW);
					ShowWindow (GetDlgItem (hwnd, IDC_KEYGENTEXT2), SW_HIDE);
					ShowWindow (GetDlgItem (hwnd, IDC_NEWUSERDONE), SW_SHOW);
					ShowWindow (GetDlgItem (hwnd, IDC_OLDUSERDONE), SW_HIDE);
				}
				else
				{
					ShowWindow (GetDlgItem (hwnd, IDC_KEYGENTEXT1), SW_HIDE);
					ShowWindow (GetDlgItem (hwnd, IDC_KEYGENTEXT2), SW_SHOW);
					ShowWindow (GetDlgItem (hwnd, IDC_NEWUSERDONE), SW_HIDE);
					ShowWindow (GetDlgItem (hwnd, IDC_OLDUSERDONE), SW_SHOW);
				}
			}
			else
			{
				ShowWindow (GetDlgItem (hwnd, IDC_RUNPGPKEYSTEXT), SW_SHOW);
				ShowWindow (GetDlgItem (hwnd, IDC_KEYGENTEXT1), SW_HIDE);
				ShowWindow (GetDlgItem (hwnd, IDC_KEYGENTEXT2), SW_HIDE);
				ShowWindow (GetDlgItem (hwnd, IDC_KEYRINGTEXT), SW_HIDE);
				ShowWindow (GetDlgItem (hwnd, IDC_NEWUSERDONE), SW_HIDE);
				ShowWindow (GetDlgItem (hwnd, IDC_OLDUSERDONE), SW_SHOW);
			}

			SetWindowLong (hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;

		case PSN_WIZFINISH:
			pnus = (PNEWUSERSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			pnus->bFinishSelected = TRUE;
			SetWindowLong(hwnd, DWL_MSGRESULT, 0L);
			bReturnCode = TRUE;
			break;
		}
		break;
	}
	}

	return bReturnCode;
}


//	______________________________________________
//
//  Create wizard data structures and call PropertySheet
//  to actually create wizard

static PGPInt32
sNewUserWizard (
	PGPContextRef		context,
	HWND				hwndParent,
	PNEWUSERSTRUCT		pnus)
{
	PROPSHEETPAGE		pspWiz[NUM_WIZ_PAGES];
	PROPSHEETHEADER		pshWiz;
	INT					iIndex;
	INT					iNumBits, iBitmap;
	HDC					hDC;
	PGPError			err;
	CHAR				szFile[32];

	// Set defaults
	pnus->context			= context;

	pnus->prefref			= kInvalidPGPPrefRef;
	pnus->hwndParent		= hwndParent;
	pnus->hwndWiz			= NULL;
	pnus->hpaletteIntro		= NULL;
	pnus->hpaletteDone		= NULL;

	pnus->bEnabled			= TRUE;
	pnus->bNewUser			= TRUE;
	pnus->bFinishSelected	= FALSE;

	LoadString (g_hInst, IDS_DEFAULTPUBRING, szFile, sizeof(szFile));
	PGPclGetPath (kPGPclDefaultKeyringFolder, pnus->szPubring, MAX_PATH);
	lstrcat (pnus->szPubring, szFile);

	LoadString (g_hInst, IDS_DEFAULTSECRING, szFile, sizeof(szFile));
	PGPclGetPath (kPGPclDefaultKeyringFolder, pnus->szSecring, MAX_PATH);
	lstrcat (pnus->szSecring, szFile);

	// Determine which bitmap will be displayed in the wizard
	hDC = GetDC (NULL);	 // DC for desktop
	iNumBits = GetDeviceCaps (hDC, BITSPIXEL) * GetDeviceCaps (hDC, PLANES);
	ReleaseDC (NULL, hDC);

	if (iNumBits <= 1)
		iBitmap = IDB_KEYWIZ1;
	else if (iNumBits <= 4)
		iBitmap = IDB_KEYWIZ4;
	else
		iBitmap = IDB_KEYWIZ8;

	pnus->hbitmapIntro = CLLoadResourceBitmap (g_hInst,
			MAKEINTRESOURCE (iBitmap), &pnus->hpaletteIntro);
	pnus->hbitmapDone = CLLoadResourceBitmap (g_hInst,
			MAKEINTRESOURCE (iBitmap), &pnus->hpaletteDone);

	// load special fonts
	pnus->hfontIntro = CLLoadHeadingFont (FALSE);
	pnus->hfontInterior = CLLoadHeadingFont (TRUE);

	// Set the values common to all pages
	for (iIndex=0; iIndex<NUM_WIZ_PAGES; iIndex++)
	{
		pspWiz[iIndex].dwSize		= sizeof(PROPSHEETPAGE);
		pspWiz[iIndex].dwFlags		= PSP_DEFAULT;
		pspWiz[iIndex].hInstance	= g_hInst;
		pspWiz[iIndex].pszTemplate  = NULL;
		pspWiz[iIndex].hIcon		= NULL;
		pspWiz[iIndex].pszTitle		= NULL;
		pspWiz[iIndex].pfnDlgProc   = NULL;
		pspWiz[iIndex].lParam		= (LPARAM) pnus;
		pspWiz[iIndex].pfnCallback  = NULL;
		pspWiz[iIndex].pcRefParent  = NULL;
	}

	// Set up the intro page
	pspWiz[NEWUSERWIZ_INTRO].pszTemplate = MAKEINTRESOURCE(IDD_NEWUSERWIZ_INTRO);
	pspWiz[NEWUSERWIZ_INTRO].pfnDlgProc = sNewUserWizIntroDlgProc;

	// Set up the name page
	pspWiz[NEWUSERWIZ_ENABLE].pszTemplate = MAKEINTRESOURCE(IDD_NEWUSERWIZ_ENABLE);
	pspWiz[NEWUSERWIZ_ENABLE].pfnDlgProc = sNewUserWizEnableDlgProc;

	// Set up the name page
	pspWiz[NEWUSERWIZ_TYPE].pszTemplate = MAKEINTRESOURCE(IDD_NEWUSERWIZ_USERTYPE);
	pspWiz[NEWUSERWIZ_TYPE].pfnDlgProc = sNewUserWizTypeDlgProc;

	// Set up the expert page
	pspWiz[NEWUSERWIZ_FILES].pszTemplate = MAKEINTRESOURCE(IDD_NEWUSERWIZ_FILES);
	pspWiz[NEWUSERWIZ_FILES].pfnDlgProc = sNewUserWizFileDlgProc;

	// Set up the done page
	pspWiz[NEWUSERWIZ_DONE].pszTemplate = MAKEINTRESOURCE(IDD_NEWUSERWIZ_DONE);
	pspWiz[NEWUSERWIZ_DONE].pfnDlgProc = sNewUserWizDoneDlgProc;

	// Create the header
	pshWiz.dwSize		= sizeof(PROPSHEETHEADER);
	pshWiz.dwFlags		= PSH_WIZARD | PSH_PROPSHEETPAGE;
	pshWiz.hwndParent   = hwndParent;
	pshWiz.hInstance	= g_hInst;
	pshWiz.hIcon		= NULL;
	pshWiz.pszCaption   = NULL;
	pshWiz.nPages		= NUM_WIZ_PAGES;
	pshWiz.nStartPage   = NEWUSERWIZ_INTRO;
	pshWiz.ppsp			= pspWiz;
	pshWiz.pfnCallback  = NULL;

	err = PGPclPeekClientLibPrefRefs (&pnus->prefref, NULL);
	if (IsntPGPError (err))
	{
		// Execute the Wizard - doesn't return until Cancel or Finish
		PropertySheet(&pshWiz);
	}

	// Free allocated memory and objects
	DeleteObject (pnus->hbitmapIntro);
	DeleteObject (pnus->hbitmapDone);
	DeleteObject (pnus->hfontIntro);
	DeleteObject (pnus->hfontInterior);

	if (pnus->bFinishSelected)
	{
		if (pnus->bEnabled)
		{
			if (pnus->bNewUser)
				return NEWUSER_NEWUSER;
			else
				return NEWUSER_EXISTINGKEYRINGS;
		}
		else
			return NEWUSER_DISABLE;
	}

	return NEWUSER_CANCEL;
}


//	______________________________________________
//
//  save file names

static PGPError
sSaveFileNames (
		PNEWUSERSTRUCT	pnus)
{
	PGPPrefRef		prefref		= kInvalidPGPPrefRef;
	PGPError		err;

	CLCreatePath (pnus->szPubring);
	CLCreatePath (pnus->szSecring);

	err = PGPclPeekClientLibPrefRefs (&prefref, NULL);

	if (IsntPGPError (err))
	{
		PGPclSetPath (prefref, 
				kPGPclPublicKeyringFile, pnus->szPubring);
		PGPclSetPath (prefref, 
				kPGPclPrivateKeyringFile, pnus->szSecring);

		err = PGPclFlushClientLibPrefs (prefref, NULL);
	}

	return err;
}


//	______________________________________________
//
//  import keys from default keys file to default keyring


static PGPError
sImportDefaultKeys (
		PGPContextRef	context,
		HWND			hwnd)
{
	PGPKeyDBRef			keydbMain		= kInvalidPGPKeyDBRef;
	PGPKeyDBRef			keydbImported	= kInvalidPGPKeyDBRef;
	PGPFileSpecRef		filerefDefault	= kInvalidPGPFileSpecRef;
	PGPError			err				= kPGPError_NoErr;

	CHAR				szFile[MAX_PATH];

	err = PGPclOpenDefaultKeyrings (context, 
			kPGPOpenKeyDBFileOptions_Mutable|kPGPOpenKeyDBFileOptions_Create, 
			&keydbMain); CKERR;

	err = PGPclGetPath (
			kPGPclDefaultKeysFile, szFile, sizeof(szFile)); CKERR;
	err = PGPNewFileSpecFromFullPath (
			context, szFile, &filerefDefault); 
	if (IsPGPError (err))
	{
		err = kPGPError_NoErr;
		goto done;
	}

	err = PGPImport (context, &keydbImported, 							
			PGPOInputFile (context, filerefDefault),
			PGPOLastOption (context)); CKERR;

	{
		PGPUInt32 u;
		PGPCountKeys (PGPPeekKeyDBRootKeySet (keydbImported), &u);
	}

	if (PGPKeyDBRefIsValid (keydbImported))
	{
		err = PGPCopyKeys (PGPPeekKeyDBRootKeySet (keydbImported), 
				keydbMain, NULL);
	}

done:
	if (PGPKeyDBRefIsValid (keydbImported))
		PGPFreeKeyDB (keydbImported);
	if (PGPKeyDBRefIsValid (keydbMain))
		PGPFreeKeyDB (keydbMain);
	if (PGPFileSpecRefIsValid (filerefDefault))
		PGPFreeFileSpec (filerefDefault);

	return err;
}

//	______________________________________________
//
//  Exported wrapper function for sImportDefaultKeys

PGPError PGPclExport
PGPclImportDefaultKeys (
		PGPContextRef	context,
		HWND			hwnd)
{
	if (!PGPContextRefIsValid (context))
		return kPGPError_BadParams;

	return sImportDefaultKeys (context, hwnd);
}


//	______________________________________________
//
//  Check if this is a new user and handle initialization if it is

PGPError
CLHandleNewUser (
	PGPContextRef		context,
	HWND				hwndParent)
{
	PGPPrefRef		prefref			= kInvalidPGPPrefRef;
	PGPError		err				= kPGPError_NoErr;
	PGPUInt32		uOldStatus		= kPGPPrefPGPInitialized;
	PGPUInt32		uNewStatus		= kPGPPrefPGPNotInitialized;
	PGPInt32		iret;
	NEWUSERSTRUCT	nus;

	iret = sNewUserWizard (context, hwndParent, &nus);

	switch (iret) {
	case NEWUSER_EXISTINGKEYRINGS :
		uNewStatus = kPGPPrefPGPInitialized;
		err = sSaveFileNames (&nus); CKERR;
		sImportDefaultKeys (context, hwndParent);
		PGPclRebootKeyGeneration (hwndParent, TRUE);
		break;

	case NEWUSER_NEWUSER :
		uNewStatus = kPGPPrefPGPInitialized;
		err = sSaveFileNames (&nus); CKERR;
		sImportDefaultKeys (context, hwndParent);
		PGPclRebootKeyGeneration (hwndParent, FALSE);
		break;

	case NEWUSER_DISABLE :
		uNewStatus = kPGPPrefPGPDisabled;
		err = kPGPError_Win32_UserNotInitialized;
		break;

	default :
		err = kPGPError_UserAbort;
		break;
	}

	if (uNewStatus != kPGPPrefPGPNotInitialized)
	{
		PGPError	errSet;

		errSet = PGPclPeekClientLibPrefRefs (&prefref, NULL);
		if (IsntPGPError (errSet))
		{
			CHAR	szFile[MAX_PATH];

			PGPGetPrefNumber (prefref, 
					kPGPPrefUserAccountStatus, &uOldStatus);

			errSet = PGPSetPrefNumber (prefref, 
					kPGPPrefUserAccountStatus, uNewStatus);
			if (IsntPGPError (errSet))
				errSet = PGPclFlushClientLibPrefs (prefref, NULL);

			err = PGPclGetPath (
					kPGPclInstallFolder, szFile, sizeof(szFile));

			if (IsntPGPError (err)) 
			{
				if ((lstrlen (szFile) + lstrlen (PGP_TRAYAPPNAME)) 
						< sizeof(szFile))
				{
					lstrcat (szFile, PGP_TRAYAPPNAME);
					PGPclExecute (szFile, SW_SHOW);
				}
			}
		}

		if (IsPGPError (errSet))
			err = errSet;
	}

done:
	return err;
}


