/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	CLprefs.c - handle PGP preferences dialogs

	$Id: CLprefs.c,v 1.89 2002/11/21 22:29:20 sdas Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "CLprefs.h"
#include "pgpWin32IPC.h"

#include <process.h>

#include "pflPrefs.h"
#include "pgpFileSpec.h"
#include "pgpNetPrefs.h"
#include "pgpHashPrefs.h"
#include "pgpRandomPool.h"

#define CHICKENANDEGG() TRUE //PGPclEnterprise()

#define SERVICE_MESSAGE_TIMEOUT		3000
#define PREFSEMAPHORENAME			("PGPprefsInstSem")

#define PGPCL_WINDOWNAME	("PGPclientLib Prefs Communication")

#define FILECHANGE_TIMEOUT			10000

#define PGPCLVPNPREFPANELS	( \
			kPGPclPGPnetVPNPrefs | kPGPclPGPnetAuthPrefs | \
			kPGPclPGPnetAdvancedPrefs )

// local globals
static PGPPrefRef		s_prefref				= kInvalidPGPPrefRef;
static PGPPrefRef		s_prefrefNet			= kInvalidPGPPrefRef;
static BOOL				s_bUsingPrefsService	= FALSE;
static BOOL				s_bNetInstalled			= FALSE;
static BOOL				s_bPGPPrefsModified		= FALSE;
static BOOL				s_bNetPrefsModified		= FALSE;
static HANDLE			s_hContactService		= NULL;
static HWND				s_hwndService			= NULL;
static HWND				s_hwndHidden			= NULL;

static INT				s_iPGPSequenceNumber	= PGP_INVALIDPREFSEQUENCE;
static INT				s_iNetSequenceNumber	= PGP_INVALIDPREFSEQUENCE;

static HANDLE			s_heventKill			= NULL;
static HANDLE			s_hthreadPrefs			= NULL;
static CHAR				s_szPGPPrefFile[MAX_PATH];
static CHAR				s_szNetPrefFile[MAX_PATH];
static FILETIME			s_filetimePGPPrefs;
static FILETIME			s_filetimeNetPrefs;

static LPSTR			s_pszPubringFile		= NULL;
static LPSTR			s_pszSecringFile		= NULL;

// external globals
extern HINSTANCE		g_hInst;
extern PGPContextRef	g_context;


//	______________________________________________
//
//  The following functions are wrappers for
//  the pfl prefs routines. These wrappers will eventually call
//	into the PGPservice for prefs operations.

PGPError PGPclExport
PGPclGetPrefFlags(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex,
		PGPUInt32*		flags)
{
	return PGPGetPrefFlags (prefref, prefIndex, flags);
}

PGPError PGPclExport
PGPclGetPrefArray(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex,
		PGPPrefArray**	array)
{
	return PGPGetPrefArray (prefref, prefIndex, array);
}

PGPError PGPclExport
PGPclSetPrefArray(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex,
		PGPPrefArray*	array)
{
	return PGPSetPrefArray (prefref, prefIndex, array);
}

PGPError PGPclExport
PGPclGetPrefBoolean(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex,
		PGPBoolean*		data)
{
	return PGPGetPrefBoolean (prefref, prefIndex, data);
}

PGPError PGPclExport
PGPclSetPrefBoolean(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex,
		PGPBoolean		data)
{
	return PGPSetPrefBoolean (prefref, prefIndex, data);
}

PGPError PGPclExport
PGPclGetPrefNumber(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex,
		PGPUInt32*		data)
{
	return PGPGetPrefNumber (prefref, prefIndex, data);
}

PGPError PGPclExport
PGPclSetPrefNumber(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex,
		PGPUInt32		data)
{
	return PGPSetPrefNumber (prefref, prefIndex, data);
}

PGPError PGPclExport
PGPclGetPrefStringAlloc(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex,
		char**			string)
{
	return PGPGetPrefStringAlloc (prefref, prefIndex, string);
}

PGPError PGPclExport
PGPclGetPrefStringBuffer(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex,
		PGPSize			maxSize,
		char*			string)
{
	return PGPGetPrefStringBuffer (prefref, prefIndex, maxSize, string);
}

PGPError PGPclExport
PGPclSetPrefString(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex,
		const char*		string)
{
	return PGPSetPrefString (prefref, prefIndex, string);
}

PGPError PGPclExport
PGPclGetPrefData(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex,
		PGPSize*		dataLength,
		void**			inBuffer)
{
	return PGPGetPrefData (prefref, prefIndex, dataLength, inBuffer);
}

PGPError PGPclExport
PGPclSetPrefData(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex,
		PGPSize			dataLength,
		const void*		outBuffer)
{
	return PGPSetPrefData (prefref, prefIndex, dataLength, outBuffer);
}

PGPError PGPclExport
PGPclRemovePref(
		PGPPrefRef		prefref,
		PGPPrefIndex	prefIndex)
{
	return PGPRemovePref (prefref, prefIndex);
}


//	______________________________________________
//
//  create the specified path if it doesn't already exist

BOOL
CLCreatePath (
		LPSTR	pszPath)
{
	DWORD	dw;
	LPSTR	p;

	dw = GetFileAttributes (pszPath);
	if ((dw != 0xFFFFFFFF) &&
		(dw & FILE_ATTRIBUTE_DIRECTORY))
	{
		return TRUE;
	}

	if (dw != 0xFFFFFFFF)
		return FALSE;

	p = strchr (pszPath, '\\');
	while (p)
	{
		*p = '\0';
		CreateDirectory (pszPath, NULL);
		*p = '\\';
		p++;
		p = strchr (p, '\\');
	}
	return TRUE;
}


//	_______________________________________
//
//  Save and close down client preferences file

static PGPError
sClosePrefs (
		PGPPrefRef	prefref,
		PGPBoolean	bSave)
{
	PGPError		err		= kPGPError_NoErr;

	if (bSave)
	{
		err = PGPSavePrefFile (prefref);
		pgpAssert (IsntPGPError(err));
	}

	if (IsPGPError (err))
		PGPFreePrefs (prefref);
	else
	{
		err = PGPFreePrefs (prefref);
		pgpAssert (IsntPGPError(err));
	}

	return err;
}


//	_______________________________________
//
//  Save and close down PGPnet preferences file

static PGPError
sCloseNetPrefs (
		PGPPrefRef	PrefRef,
		PGPBoolean	bSave)
{
	return sClosePrefs (PrefRef, bSave);
}


//	_______________________________________
//
//  Query registry to determine if client or admin installation

PGPBoolean PGPclExport
PGPclIsAdminInstall (VOID)
{
	HKEY		hKey;
	LONG		lResult;
	DWORD		dwValueType, dwSize;
	CHAR		szKey[128];

	PGPclGetPath (kPGPclRegistryKey, szKey, sizeof(szKey));
	lResult = RegOpenKeyEx (HKEY_LOCAL_MACHINE, szKey, 0, KEY_READ, &hKey);

	if (lResult == ERROR_SUCCESS)
	{
		dwSize = sizeof(szKey);
		lResult = RegQueryValueEx (
			hKey, "InstallDate", 0, &dwValueType, (LPBYTE)szKey, &dwSize);
		RegCloseKey (hKey);
	}

	return (lResult == ERROR_SUCCESS);
}

PGPBoolean PGPclExport
PGPclIsClientInstall (VOID)
{
	return !PGPclIsAdminInstall();
}


//	______________________________________
//
//	Check for previously running instances

static HANDLE
sShowExistingPropertySheet (
		LPSTR lpszSemName,
		LPSTR lpszWindowTitle)
{
    HANDLE	hSem;
    HWND	hWndMe;

    // Create or open a named semaphore.
    hSem = CreateSemaphore (NULL, 0, 1, lpszSemName);

    // Close handle and return NULL if existing semaphore was opened.
    if ((hSem != NULL) && (GetLastError() == ERROR_ALREADY_EXISTS)) {
        CloseHandle(hSem);
        hWndMe = FindWindow ((LPCTSTR)32770L, lpszWindowTitle);
        if (hWndMe) {
			SetForegroundWindow (hWndMe);
		}
        return NULL;
    }

    // If new semaphore was created, return FALSE.
    return hSem;
}


//	____________________________
//
//  set boolean pref on basis of checkbox

PGPBoolean
CLSaveCheckBoxPref (
		HWND		hwnd,
		INT			idc,
		PGPPrefRef	prefref,
		UINT		uPref)
{
	PGPBoolean	b;

	b = (IsDlgButtonChecked (hwnd, idc) == BST_CHECKED);

	PGPclErrorBox (hwnd, PGPSetPrefBoolean (prefref, uPref, b));

	return b;
}


//	____________________________
//
//  set checkbox on basis of boolean pref

PGPBoolean
CLDisablePrefControl (
		HWND			hwnd,
		INT				idc,
		PPREFSDLGSTRUCT	ppds,
		PGPPrefRef		prefref,
		UINT			uPref)
{
	PGPBoolean	b		= FALSE;
	PGPUInt32	u;

	if (ppds->bRespectDisables)
	{
		if (IsntPGPError (PGPGetPrefFlags (prefref, uPref, &u)))
		{
			b = u & kPGPPrefFlags_DisableGUI;
			if (b)
			{
				if (idc == 0)
					EnableWindow (hwnd, FALSE);
				else
					EnableWindow (GetDlgItem (hwnd, idc), FALSE);
			}
		}
	}

	return b;
}


//	____________________________
//
//  get the appropriate prefref

PGPPrefRef
CLGetPrefRef (
		PPREFSDLGSTRUCT	ppds)
{
	PGPPrefRef	prefref		= kInvalidPGPPrefRef;

	if (ppds->bUseClientLibPrefs)
		PGPclPeekClientLibPrefRefs (&prefref, NULL);
	else
		prefref = ppds->prefref;

	return prefref;
}


//	____________________________
//
//  get the appropriate prefref

PGPPrefRef
CLGetPrefRefNet (
		PPREFSDLGSTRUCT	ppds)
{
	PGPPrefRef	prefref		= kInvalidPGPPrefRef;

	if (ppds->bUseClientLibNetPrefs)
		PGPclPeekClientLibPrefRefs (NULL, &prefref);
	else
		prefref = ppds->prefrefNet;

	return prefref;
}


//	____________________________
//
//  set checkbox on basis of boolean pref

PGPBoolean
CLLoadCheckBoxPref (
		HWND			hwnd,
		INT				idc,
		PPREFSDLGSTRUCT	ppds,
		PGPPrefRef		prefref,
		UINT			uPref)
{
	PGPBoolean	b;

	if (IsntPGPError (PGPGetPrefBoolean (prefref, uPref, &b)))
	{
		CheckDlgButton (hwnd, idc, b ? BST_CHECKED : BST_UNCHECKED);
	}

	CLDisablePrefControl (hwnd, idc, ppds, prefref, uPref);

	return b;
}


//	________________________
//
//	Send message to service, if running

VOID
sSendPGPnetServiceMessage (
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	HWND	hwnd;

	hwnd = FindWindow (PGPNET_SERVICECOMMWINDOWNAME,
			PGPNET_SERVICECOMMWINDOWNAME);
	if (hwnd)
	{
		PostMessage (hwnd, uMsg, wParam, lParam);
	}
}


//	____________________________
//
//  Create property sheet dialog -- internal version

static PGPError
sPreferencesInternal (
		BOOL			bAdminCalled,
		PGPContextRef	context,
		HWND			hwnd,
		PGPPrefRef		prefref,
		PGPPrefRef		prefrefNet,
		PGPUInt32		uFlags,
		PGPUInt32		uShowSheets,
		PGPUInt32		uStartSheet,
		PGPKeyDBRef		keydb)
{
	PREFSDLGSTRUCT		pds;
    PROPSHEETPAGE		psp[kPGPclNumPrefsPanels];
    PROPSHEETHEADER		psh;
	CHAR				szTitle[128];
	HANDLE				hSemaphore;
	PGPError			err;
	HWND				hwndTrayApp;
	INT					ipage, istart;
	PGPBoolean			bReenableHotKeys;

	bReenableHotKeys=FALSE;

	// if preferences dialog already exists, move it to the foreground
	LoadString (g_hInst, IDS_PROPCAPTION, szTitle, sizeof(szTitle));
	hSemaphore = sShowExistingPropertySheet (PREFSEMAPHORENAME, szTitle);
	if (!hSemaphore)
		return kPGPError_Win32_AlreadyOpen;

	// zero out PREFSDLGSTRUCT
	memset (&pds, 0, sizeof(pds));

	// use the user-supplied context, if supplied
	if (context != NULL)
		pds.context = context;
	else
		PGPNewContext (kPGPsdkAPIVersion, &pds.context);

	// initialize PREFSDLGSTRUCT
	if (uFlags & kPGPclRespectDisableFlags)
		pds.bRespectDisables = TRUE;
	else
		pds.bRespectDisables = FALSE;

	pds.bAdminCalled = bAdminCalled;
	pds.prefref = kInvalidPGPPrefRef;
	pds.prefrefNet = kInvalidPGPPrefRef;

	pds.hwndParent = hwnd;
	pds.keydbMain = keydb;
	pds.bNeedsCentering = TRUE;

	pds.iFilePage = -1;
	pds.bKeyringModified = FALSE;
	pds.bPrefsModified = FALSE;
	pds.file.bMediaNotInserted = FALSE;
	pds.szSmartCardDriver[0] = '\0';

	pds.ks.bPrefsModified = FALSE;
	pds.disk.bPrefsModified = FALSE;
	pds.net.bPrefsModified = FALSE;
	pds.net.bKeyringModified = FALSE;
	pds.net.bRulesRenumbered = FALSE;

	pds.net.keydb = kInvalidPGPKeyDBRef;
	pds.net.uNumIKEProposals = 0;
	pds.net.pIKEProposals = NULL;
	pds.net.uNumIPSECProposals = 0;
	pds.net.pIPSECProposals = NULL;

	pds.net.pX509AuthCertIASN = NULL;
	pds.net.bUsePGPAuthKey = FALSE;
	pds.net.bUseX509AuthKey = FALSE;

	if (PGPPrefRefIsValid (prefref))
	{
		pds.prefref = prefref;
		pds.bUseClientLibPrefs = FALSE;
	}
	else
	{
		pds.prefref = kInvalidPGPPrefRef;
		pds.bUseClientLibPrefs = TRUE;
	}

	if (PGPPrefRefIsValid (prefrefNet))
	{
		pds.prefrefNet = prefrefNet;
		pds.bUseClientLibNetPrefs = FALSE;
	}
	else
	{
		pds.prefrefNet = kInvalidPGPPrefRef;
		pds.bUseClientLibNetPrefs = TRUE;
	}

	err = PGPclGetPath (kPGPclPublicKeyringFile,
			pds.szPubringFile,
			sizeof(pds.szPubringFile)); CKERR;

	err = PGPclGetPath (kPGPclPrivateKeyringFile,
			pds.szSecringFile,
			sizeof(pds.szSecringFile)); CKERR;

	err = PGPclGetPath (kPGPclRandseedFile,
			pds.szRandseedFile,
			sizeof(pds.szRandseedFile)); CKERR;

	if (uShowSheets & PGPCLVPNPREFPANELS)
	{
		err = PGPclGetPath (kPGPclNetPublicKeyringFile,
				pds.net.szPubringFile,
				sizeof(pds.net.szPubringFile)); CKERR;

		err = PGPclGetPath (kPGPclNetPrivateKeyringFile,
				pds.net.szSecringFile,
				sizeof(pds.net.szSecringFile)); CKERR;
	}

	ipage = 0;
	istart = 0;

	if (uShowSheets & kPGPclGeneralPrefs)
	{
		psp[ipage].dwSize = sizeof(PROPSHEETPAGE);
		psp[ipage].dwFlags = PSP_USETITLE|PSP_HASHELP;
		psp[ipage].hInstance = g_hInst;
		psp[ipage].pszTemplate = MAKEINTRESOURCE(IDD_PREF_GENERAL);
		psp[ipage].pszIcon = NULL;
		psp[ipage].pfnDlgProc = CLGeneralPropDlgProc;
		psp[ipage].pszTitle = (LPSTR)IDS_PREF_GENERAL;
		psp[ipage].lParam = (LPARAM)&pds;
		psp[ipage].pfnCallback = NULL;

		if (uStartSheet == kPGPclGeneralPrefs)
			istart = ipage;

		ipage++;
	}

	if (uShowSheets & kPGPclKeyringPrefs)
	{
		psp[ipage].dwSize = sizeof(PROPSHEETPAGE);
		psp[ipage].dwFlags = PSP_USETITLE|PSP_HASHELP;
		psp[ipage].hInstance = g_hInst;
		psp[ipage].pszTemplate = MAKEINTRESOURCE(IDD_PREF_FILES);
		psp[ipage].pszIcon = NULL;
		psp[ipage].pfnDlgProc = CLFilePropDlgProc;
		psp[ipage].pszTitle = (LPSTR)IDS_PREF_FILES;
		psp[ipage].lParam = (LPARAM)&pds;
		psp[ipage].pfnCallback = NULL;

		if (uStartSheet == kPGPclKeyringPrefs)
			istart = ipage;

		pds.iFilePage = ipage;

		ipage++;
	}

	if (uShowSheets & kPGPclEmailPrefs)
	{
		psp[ipage].dwSize = sizeof(PROPSHEETPAGE);
		psp[ipage].dwFlags = PSP_USETITLE|PSP_HASHELP;
		psp[ipage].hInstance = g_hInst;
		psp[ipage].pszTemplate = MAKEINTRESOURCE(IDD_PREF_EMAIL);
		psp[ipage].pszIcon = NULL;
		psp[ipage].pfnDlgProc = CLEmailPropDlgProc;
		psp[ipage].pszTitle = (LPSTR)IDS_PREF_EMAIL;
		psp[ipage].lParam = (LPARAM)&pds;
		psp[ipage].pfnCallback = NULL;

		pds.bToolsPrefsEnabled = TRUE;

		if (uStartSheet == kPGPclEmailPrefs)
			istart = ipage;

		ipage++;
	}

	if (uShowSheets & kPGPclHotkeyPrefs)
	{
		psp[ipage].dwSize = sizeof(PROPSHEETPAGE);
		psp[ipage].dwFlags = PSP_USETITLE|PSP_HASHELP;
		psp[ipage].hInstance = g_hInst;
		psp[ipage].pszTemplate = MAKEINTRESOURCE(IDD_PREF_HOTKEYS);
		psp[ipage].pszIcon = NULL;
		psp[ipage].pfnDlgProc = CLHotkeyPropDlgProc;
		psp[ipage].pszTitle = (LPSTR)IDS_PREF_HOTKEYS;
		psp[ipage].lParam = (LPARAM)&pds;
		psp[ipage].pfnCallback = NULL;

		if (uStartSheet == kPGPclHotkeyPrefs)
			istart = ipage;

		ipage++;
	}

	if (uShowSheets & kPGPclKeyserverPrefs)
	{
		psp[ipage].dwSize = sizeof(PROPSHEETPAGE);
		psp[ipage].dwFlags = PSP_USETITLE|PSP_HASHELP;
		psp[ipage].hInstance = g_hInst;
		psp[ipage].pszTemplate = MAKEINTRESOURCE(IDD_PREF_SERVER);
		psp[ipage].pszIcon = NULL;
		psp[ipage].pfnDlgProc = CLKeyserverPropDlgProc;
		psp[ipage].pszTitle = (LPSTR)IDS_PREF_SERVER;
		psp[ipage].lParam = (LPARAM)&pds;
		psp[ipage].pfnCallback = NULL;

		if (uStartSheet == kPGPclKeyserverPrefs)
			istart = ipage;

		ipage++;
	}

	if (uShowSheets & kPGPclCAPrefs)
	{
		psp[ipage].dwSize = sizeof(PROPSHEETPAGE);
		psp[ipage].dwFlags = PSP_USETITLE|PSP_HASHELP;
		psp[ipage].hInstance = g_hInst;
		psp[ipage].pszTemplate = MAKEINTRESOURCE(IDD_PREF_CA);
		psp[ipage].pszIcon = NULL;
		psp[ipage].pfnDlgProc = CLCAPropDlgProc;
		psp[ipage].pszTitle = (LPSTR)IDS_PREF_CA;
		psp[ipage].lParam = (LPARAM)&pds;
		psp[ipage].pfnCallback = NULL;

		if (uStartSheet == kPGPclCAPrefs)
			istart = ipage;

		ipage++;
	}

	if (uShowSheets & kPGPclAdvancedPrefs)
	{
		psp[ipage].dwSize = sizeof(PROPSHEETPAGE);
		psp[ipage].dwFlags = PSP_USETITLE|PSP_HASHELP;
		psp[ipage].hInstance = g_hInst;
		psp[ipage].pszTemplate = MAKEINTRESOURCE(IDD_PREF_ADVANCED);
		psp[ipage].pszIcon = NULL;
		psp[ipage].pfnDlgProc = CLAdvancedPropDlgProc;
		psp[ipage].pszTitle = (LPSTR)IDS_PREF_ADVANCED;
		psp[ipage].lParam = (LPARAM)&pds;
		psp[ipage].pfnCallback = NULL;

		if (uStartSheet == kPGPclAdvancedPrefs)
			istart = ipage;

		ipage++;
	}

	if(PGPlnIsAuthorized() && PGPlnPGPdiskEnabled ())
	{
		if (uShowSheets & kPGPclPGPdiskPrefs)
		{
			psp[ipage].dwSize = sizeof(PROPSHEETPAGE);
			psp[ipage].dwFlags = PSP_USETITLE|PSP_HASHELP;
			psp[ipage].hInstance = g_hInst;
			psp[ipage].pszTemplate = MAKEINTRESOURCE(IDD_PREF_PGPDISK);
			psp[ipage].pszIcon = NULL;
			psp[ipage].pfnDlgProc = CLDiskPropDlgProc;
			psp[ipage].pszTitle = (LPSTR)IDS_PREF_PGPDISK;
			psp[ipage].lParam = (LPARAM)&pds;
			psp[ipage].pfnCallback = NULL;

			pds.disk.bPrefsEnabled = TRUE;

			if (uStartSheet == kPGPclPGPdiskPrefs)
				istart = ipage;

			ipage++;
		}
	}

/*
#if !PGP_FREEWARE
	if (uShowSheets & kPGPclPGPnetIDRPrefs)
	{
		psp[ipage].dwSize = sizeof(PROPSHEETPAGE);
		psp[ipage].dwFlags = PSP_USETITLE|PSP_HASHELP;
		psp[ipage].hInstance = g_hInst;
		psp[ipage].pszTemplate = MAKEINTRESOURCE(IDD_PREF_NETIDR);
		psp[ipage].pszIcon = NULL;
		psp[ipage].pfnDlgProc = CLNetIDRPropDlgProc;
		psp[ipage].pszTitle = (LPSTR)IDS_PREF_PGPNETIDR;
		psp[ipage].lParam = (LPARAM)&pds;
		psp[ipage].pfnCallback = NULL;

		if (uStartSheet == kPGPclPGPnetIDRPrefs)
			istart = ipage;

		ipage++;
	}
#endif // !PGP_FREEWARE

	if (uShowSheets & kPGPclPGPnetVPNPrefs)
	{
		psp[ipage].dwSize = sizeof(PROPSHEETPAGE);
		psp[ipage].dwFlags = PSP_USETITLE|PSP_HASHELP;
		psp[ipage].hInstance = g_hInst;
		psp[ipage].pszTemplate = MAKEINTRESOURCE(IDD_PREF_NETVPN);
		psp[ipage].pszIcon = NULL;
		psp[ipage].pfnDlgProc = CLNetVPNPropDlgProc;
		psp[ipage].pszTitle = (LPSTR)IDS_PREF_PGPNETVPN;
		psp[ipage].lParam = (LPARAM)&pds;
		psp[ipage].pfnCallback = NULL;

		pds.net.bVPNPrefsEnabled = TRUE;

		if (uStartSheet == kPGPclPGPnetVPNPrefs)
			istart = ipage;

		ipage++;
	}

	if (uShowSheets & kPGPclPGPnetAuthPrefs)
	{
		psp[ipage].dwSize = sizeof(PROPSHEETPAGE);
		psp[ipage].dwFlags = PSP_USETITLE|PSP_HASHELP;
		psp[ipage].hInstance = g_hInst;
		psp[ipage].pszTemplate = MAKEINTRESOURCE(IDD_PREF_NETAUTH);
		psp[ipage].pszIcon = NULL;
		psp[ipage].pfnDlgProc = CLNetAuthPropDlgProc;
		psp[ipage].pszTitle = (LPSTR)IDS_PREF_PGPNETAUTH;
		psp[ipage].lParam = (LPARAM)&pds;
		psp[ipage].pfnCallback = NULL;

		if (uStartSheet == kPGPclPGPnetAuthPrefs)
			istart = ipage;

		ipage++;
	}

	if (uShowSheets & kPGPclPGPnetAdvancedPrefs)
	{
		psp[ipage].dwSize = sizeof(PROPSHEETPAGE);
		psp[ipage].dwFlags = PSP_USETITLE|PSP_HASHELP;
		psp[ipage].hInstance = g_hInst;
		psp[ipage].pszTemplate = MAKEINTRESOURCE(IDD_PREF_NETADV);
		psp[ipage].pszIcon = NULL;
		psp[ipage].pfnDlgProc = CLNetAdvPropDlgProc;
		psp[ipage].pszTitle = (LPSTR)IDS_PREF_PGPNETADV;
		psp[ipage].lParam = (LPARAM)&pds;
		psp[ipage].pfnCallback = NULL;

		if (uStartSheet == kPGPclPGPnetAdvancedPrefs)
			istart = ipage;

		ipage++;
	}
*/

	psh.dwSize = sizeof(PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW;
	psh.hwndParent = pds.hwndParent;
	psh.hInstance = g_hInst;
	psh.pszIcon = NULL;
	psh.pszCaption = (LPSTR) szTitle;
	psh.nPages = ipage;
	psh.nStartPage = istart;
	psh.ppsp = (LPCPROPSHEETPAGE) &psp;
	psh.pfnCallback = NULL;

	// disable hotkeys because they can interfere with the hotkey panel
	hwndTrayApp = FindWindow (PGPTRAY_WINDOWNAME, PGPTRAY_WINDOWNAME);
	if (hwndTrayApp)
	{
		SendMessage (hwndTrayApp, PGPTRAY_M_DISABLEHOTKEYS, 0, 0);
		bReenableHotKeys=TRUE;
	}

	// actually display and handle the property sheet
	PropertySheet (&psh);

	// if prefs have changed, flush them
	if (!PGPPrefRefIsValid (prefref))
	{
		if (pds.bPrefsModified || pds.disk.bPrefsModified)
		{
			if (PGPPrefRefIsValid (pds.prefref))
			{
				err = PGPclFlushClientLibPrefs (pds.prefref, NULL);
				bReenableHotKeys=FALSE;
				PGPclErrorBox (hwnd, err);
			}
		}
	}

	if (IsntPGPError (err))
	{
		if (uFlags & kPGPclEnableBroadcasts)
		{
			if (pds.bKeyringModified)
				PGPclNotifyKeyringChanges (kPGPclNewDefaultKeyrings, 0);

			if (pds.ks.bPrefsModified)
				PGPclNotifyKeyserverPrefsChanges (0);
		}

		if (!PGPPrefRefIsValid (prefrefNet))
		{
			if (pds.net.bPrefsModified)
			{
				if (PGPPrefRefIsValid (pds.prefrefNet))
				{
					err = PGPclFlushClientLibPrefs (NULL, pds.prefrefNet);
					bReenableHotKeys=FALSE;
					PGPclErrorBox (hwnd, err);
				}
			}
		}
	}
/*
	// why was this here?

	if (IsntPGPError (err))
	{
		if (uFlags & kPGPclEnableBroadcasts)
		{
			if (pds.net.bPrefsModified)
			{
				PGPclNotifyNetPrefsChanges (0);
			}
		}
	}
*/
	if (pds.file.bCopiedKeyrings)
	{
		if (pds.file.bKeyringCopyError)
		{
			PGPclMessageBox (hwnd, IDS_CAPTIONWARNING, 
					IDS_KEYRINGCOPYERROR, MB_OK|MB_ICONEXCLAMATION);
		}
		else
		{
			PGPclMessageBox (hwnd, IDS_CAPTIONINFO, 
					IDS_KEYRINGSCOPIED, MB_OK|MB_ICONINFORMATION);
		}
	}

done:
	if(bReenableHotKeys)
	{
		if (hwndTrayApp)
		{
			SendMessage (hwndTrayApp, PGPTRAY_M_ENABLEHOTKEYS, 0, 0);
		}
	}

	if (IsntNull (pds.net.pX509AuthCertIASN))
		PGPFreeData (pds.net.pX509AuthCertIASN);

	if (!PGPKeyDBRefIsValid (keydb))
	{
		if (PGPKeyDBRefIsValid (pds.keydbMain))
			PGPFreeKeyDB (pds.keydbMain);
	}

	if (PGPKeyDBRefIsValid (pds.net.keydb))
		PGPFreeKeyDB (pds.net.keydb);

	if (context == NULL)
		PGPFreeContext (pds.context);

	CloseHandle (hSemaphore);

	if (IsntPGPError (pds.returnErr))
	{
		if ((!pds.bPrefsModified) &&
			(!pds.bKeyringModified) &&
			(!pds.disk.bPrefsModified) &&
			(!pds.ks.bPrefsModified) &&
			(!pds.net.bPrefsModified) &&
			(!pds.net.bKeyringModified))
		{
			pds.returnErr = kPGPError_UserAbort;
		}
	}

    return pds.returnErr;
}


//	____________________________
//
//  Create property sheet dialog -- Admin version

PGPError PGPclExport
PGPclAdminPreferences (
		PGPContextRef	context,
		HWND			hwnd,
		PGPPrefRef		prefref,
		PGPPrefRef		prefrefNet,
		PGPUInt32		uFlags,
		PGPUInt32		uShowSheets,
		PGPUInt32		uStartSheet,
		PGPKeyDBRef		keydb)
{
	return sPreferencesInternal (TRUE, context, hwnd, 
			prefref, prefrefNet, uFlags, uShowSheets, uStartSheet, keydb);
}


//	____________________________
//
//  Create property sheet dialog

PGPError PGPclExport
PGPclPreferences (
		PGPContextRef	context,
		HWND			hwnd,
		PGPUInt32		ustartsheet,
		PGPKeyDBRef		keydb)
{
	PGPUInt32	uShow	= 0;

	if (PGPclIsComponentInstalled (kPGPclPGPkeys))
	{
		uShow |= kPGPclGeneralPrefs		|
				kPGPclKeyringPrefs		|
				kPGPclHotkeyPrefs		|
				kPGPclKeyserverPrefs	|
				kPGPclCAPrefs			|
				kPGPclAdvancedPrefs;
	}

	if (PGPclIsComponentInstalled (kPGPclPGPtools))
	{
		uShow |= kPGPclEmailPrefs;
	}

	if (PGPclIsComponentInstalled (kPGPclPGPdisk))
	{
		uShow |= kPGPclPGPdiskPrefs;
	}

	if (PGPclIsComponentInstalled (kPGPclPFIDS))
	{
		uShow |= kPGPclPGPnetIDRPrefs;
	}

	if (PGPclIsComponentInstalled (kPGPclVPN))
	{
		uShow |= kPGPclPGPnetVPNPrefs		| 
				kPGPclPGPnetAuthPrefs		|
				kPGPclPGPnetAdvancedPrefs;
	}

	return sPreferencesInternal (
				FALSE, context, hwnd,
				kInvalidPGPPrefRef, kInvalidPGPPrefRef,
				kPGPclEnableBroadcasts |
				kPGPclRespectDisableFlags,
				uShow, ustartsheet, keydb);
}


//	____________________________________
//
//  hidden communication window message proc

static LONG APIENTRY
sPrefsServiceWindowProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	switch (uMsg) {
	case WM_CREATE :
	{
		LPCREATESTRUCT	lpcs = (LPCREATESTRUCT)lParam;
		s_hwndHidden = hwnd;
		SetEvent ((HANDLE)(lpcs->lpCreateParams));
		return 0;
	}

	case WM_COPYDATA :
	{
		PGPPrefsCopyDataStructure*		ppcds;
		PCOPYDATASTRUCT					pcds;
		PGPError						err;

		pcds = (PCOPYDATASTRUCT)lParam;
		ppcds = pcds->lpData;

		if (pcds->dwData & PGP_PGPPREFSDATA)
		{
			if (PGPPrefRefIsValid (s_prefref))
				PGPFreePrefs (s_prefref);

			err = PGPImportBufferToMemoryPrefs (
					PGPPeekContextMemoryMgr (g_context),
					ppcds->buffer, ppcds->bufferLength,
					clientDefaults, clientDefaultsSize, &s_prefref);
			if (IsntPGPError (err))
			{
				s_iPGPSequenceNumber = ppcds->sequenceNumber;
			}
			else
			{
				s_prefref = kInvalidPGPPrefRef;
				s_iPGPSequenceNumber = PGP_INVALIDPREFSEQUENCE;
			}
		}

		else if (pcds->dwData & PGP_PGPNETPREFSDATA)
		{
			if (PGPPrefRefIsValid (s_prefrefNet))
				PGPFreePrefs (s_prefrefNet);

			err = PGPImportBufferToMemoryPrefs (
					PGPPeekContextMemoryMgr (g_context),
					ppcds->buffer, ppcds->bufferLength,
					netDefaults, netDefaultsSize, &s_prefrefNet);
			if (IsntPGPError (err))
			{
				s_iNetSequenceNumber = ppcds->sequenceNumber;
			}
			else
			{
				s_prefrefNet = kInvalidPGPPrefRef;
				s_iNetSequenceNumber = PGP_INVALIDPREFSEQUENCE;
			}
		}

		return TRUE;
	}

	case WM_CLOSE :
		DestroyWindow (hwnd);
		break;

	case WM_DESTROY :
		PostQuitMessage (0);
		break;

	default :
		return (DefWindowProc (hwnd, uMsg, wParam, lParam));
	}

	return FALSE;
}


//	______________________________________________
//
//	thread function that handles communication with prefs service

static DWORD WINAPI
sPrefServiceCommThread (
		LPVOID		hEvent)
{
	WNDCLASS	wc;
	MSG			msg;

	// Register the Window Class
	wc.style			= 0;
	wc.lpfnWndProc		= sPrefsServiceWindowProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= g_hInst;
	wc.hIcon			= 0;
	wc.hCursor			= 0;
	wc.hbrBackground	= 0;
	wc.lpszMenuName		= 0;
	wc.lpszClassName	= PGPCL_WINDOWNAME;

	RegisterClass (&wc);

	CreateWindow (
		PGPCL_WINDOWNAME, PGPCL_WINDOWNAME, 0,
		0, 0, 0, 0, NULL, NULL, g_hInst, hEvent);

	if (!s_hwndHidden)
		return 0;

	while (GetMessage (&msg, s_hwndHidden, 0, 0))
	{
		DispatchMessage (&msg);
	}

	return 0;
}


//	______________________________________________
//
//	determine if prefs service is running by trying to communicate

static BOOL
sLogonToPrefsService (VOID)
{
	DWORD			dw;
	COPYDATASTRUCT	cds;

	s_hwndService = FindWindow (PGP_PREFSSERVICEWINDOWNAME,
			PGP_PREFSSERVICEWINDOWNAME);

	if (s_hwndService)
	{
		HANDLE	hEvent;

		hEvent = CreateEvent (
				NULL, FALSE, FALSE, NULL);
		s_hthreadPrefs = (HANDLE)_beginthreadex (
				NULL, 0, sPrefServiceCommThread, hEvent, 0, &dw);
		if (IsntNull (s_hthreadPrefs))
		{
			WaitForSingleObject (hEvent, 10000);
			CloseHandle (hEvent);

			if (s_hwndHidden)
			{
				cds.dwData = PGP_PGPPREFSLOGON;
				cds.lpData = s_szPGPPrefFile;
				cds.cbData = sizeof(s_szPGPPrefFile);

				if (SendMessageTimeout (s_hwndService, WM_COPYDATA,
						(WPARAM)s_hwndHidden, (LPARAM)&cds,
						SMTO_NORMAL, SERVICE_MESSAGE_TIMEOUT, &dw) != 0)
				{
					if (dw == 1)
					{
						//success
						return TRUE;
					}
				}
				// if here, then there was an error.  Kill thread.
				SendMessage (s_hwndHidden, WM_CLOSE, 0, 0);
				s_hthreadPrefs = NULL;
			}
		}
	}
	return FALSE;
}


//	______________________________________________
//
//	determine if the prefs file has changed by querying the timestamp

static BOOL
sPGPPrefsFileTimeChanged (VOID)
{
	BOOL		bChanged		= FALSE;
	FILETIME	filetime;
	HANDLE		hfile;

	hfile = CreateFile (s_szPGPPrefFile, 0,
			FILE_SHARE_READ|FILE_SHARE_WRITE, NULL,
			OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hfile == INVALID_HANDLE_VALUE)
		memset (&s_filetimePGPPrefs, 0, sizeof(FILETIME));
	else
	{
		GetFileTime (hfile, NULL, NULL, &filetime);
		CloseHandle (hfile);
	}

	if (CompareFileTime (&filetime, &s_filetimePGPPrefs))
	{
		s_filetimePGPPrefs = filetime;
		bChanged = TRUE;
	}

	return bChanged;
}


//	______________________________________________
//
//	determine if the prefs file has changed by querying the timestamp

static BOOL
sNetPrefsFileTimeChanged (VOID)
{
	BOOL		bChanged		= FALSE;
	FILETIME	filetime;
	HANDLE		hfile;

	hfile = CreateFile (s_szNetPrefFile, 0,
			FILE_SHARE_READ|FILE_SHARE_WRITE, NULL,
			OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hfile == INVALID_HANDLE_VALUE)
		memset (&s_filetimeNetPrefs, 0, sizeof(FILETIME));
	else
	{
		GetFileTime (hfile, NULL, NULL, &filetime);
		CloseHandle (hfile);
	}

	if (CompareFileTime (&filetime, &s_filetimeNetPrefs))
	{
		s_filetimeNetPrefs = filetime;
		bChanged = TRUE;
	}

	return bChanged;
}


//	______________________________________________
//
//	thread function that waits for file change notification or kill
//  notification and acts accordingly

static DWORD WINAPI
sFileChangeThread (
		LPVOID		lpVoid)
{
	BOOL		bKilled		= FALSE;
	DWORD		dw;

	while (!bKilled)
	{
		dw = WaitForSingleObject (s_heventKill, FILECHANGE_TIMEOUT);

		if (dw != WAIT_TIMEOUT)
		{
			bKilled = TRUE;
		}
		else
		{
			if (s_hContactService != NULL)
			{
				if (WaitForSingleObject(s_hContactService, FILECHANGE_TIMEOUT)
					== WAIT_TIMEOUT)
				{
					bKilled = TRUE;
					break;
				}
			}

			if (sLogonToPrefsService ())
			{
				s_bUsingPrefsService = TRUE;

				s_iPGPSequenceNumber = PGP_INVALIDPREFSEQUENCE;
				s_iNetSequenceNumber = PGP_INVALIDPREFSEQUENCE;

				if (PGPPrefRefIsValid (s_prefref))
				{
					PGPFreePrefs (s_prefref);
					s_prefref = kInvalidPGPPrefRef;
				}

				if (PGPPrefRefIsValid (s_prefrefNet))
				{
					PGPFreePrefs (s_prefrefNet);
					s_prefrefNet = kInvalidPGPPrefRef;
				}

				if (s_hContactService)
					ReleaseMutex(s_hContactService);

				break;
			}

			if (s_hContactService)
				ReleaseMutex(s_hContactService);

			if (sPGPPrefsFileTimeChanged ())
			{
				s_bPGPPrefsModified = TRUE;
				PGPclNotifyPrefsChanges (0);
			}

			if (sNetPrefsFileTimeChanged ())
			{
				s_bNetPrefsModified = TRUE;
				PGPclNotifyNetPrefsChanges (0);
			}
		}
	}

	return 0;
}


//	______________________________________________
//
//	initialize and open up the PGPclientLib prefs system

PGPError
CLInitPrefs (
		BOOL	bLocalMode,
		BOOL	bCreatePGPPrefsFile)
{
	PGPError		err			= kPGPError_NoErr;

	DWORD			dwID;
	HANDLE			hfile;
	CHAR			szPath[MAX_PATH];

	s_prefref			= kInvalidPGPPrefRef;
	s_prefrefNet		= kInvalidPGPPrefRef;
	s_bNetInstalled		= FALSE;
	s_hthreadPrefs		= NULL;
	s_szPGPPrefFile[0]	= '\0';

	// create the PGP prefs folder
	if (bCreatePGPPrefsFile)
	{
		PGPclGetPath (kPGPclPrefsFile,
				s_szPGPPrefFile, sizeof(s_szPGPPrefFile));
		CLCreatePath (s_szPGPPrefFile);

		if(CHICKENANDEGG())// Chicken and egg here.... This is all client install stuff so could be OK
		{
			if (!(PGPclTestPathNameStatus (s_szPGPPrefFile) & kPGPclIsFile))
			{
				if (PGPclIsClientInstall ())
				{
					// copy default prefs file
					PGPclGetPath (kPGPclInstallFolder, szPath, sizeof(szPath));
					if ((lstrlen (szPath) + lstrlen (PGP_PREFSFILENAME) +1) <
						sizeof(szPath))
					{
						lstrcat (szPath, PGP_PREFSFILENAME);
						CopyFile (szPath, s_szPGPPrefFile, TRUE);
					}
				}
			}
		}
	}

	// create the PGPnet prefs folder
	PGPclGetPath (kPGPclNetPrefsFile, s_szNetPrefFile, sizeof(s_szNetPrefFile));
	CLCreatePath (s_szNetPrefFile);

	if (!bLocalMode && sLogonToPrefsService ())
	{
		s_bUsingPrefsService = TRUE;
		s_bNetInstalled = PGPclIsComponentInstalled (kPGPclPGPnet);

		s_iPGPSequenceNumber = PGP_INVALIDPREFSEQUENCE;
		s_iNetSequenceNumber = PGP_INVALIDPREFSEQUENCE;

		s_prefref		= kInvalidPGPPrefRef;
		s_prefrefNet	= kInvalidPGPPrefRef;
	}
	else
	{
		s_bUsingPrefsService = FALSE;
		s_hContactService = CreateMutex(NULL, FALSE,
								"PGPclientLib Contact Service Mutex");

		// setup code to monitor PGP prefs file
		hfile = CreateFile (s_szPGPPrefFile, 0,
				FILE_SHARE_READ|FILE_SHARE_WRITE, NULL,
				OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

		if (hfile == INVALID_HANDLE_VALUE)
			memset (&s_filetimePGPPrefs, 0, sizeof(FILETIME));
		else
		{
			GetFileTime (hfile, NULL, NULL, &s_filetimePGPPrefs);
			CloseHandle (hfile);
		}

		if (PGPclIsComponentInstalled (kPGPclPGPnet))
		{
			s_bNetInstalled = TRUE;

			// setup code to monitor PGPnet prefs file
			hfile = CreateFile (s_szNetPrefFile, 0,
					FILE_SHARE_READ|FILE_SHARE_WRITE, NULL,
					OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
			if (hfile == INVALID_HANDLE_VALUE)
				memset (&s_filetimeNetPrefs, 0, sizeof(FILETIME));
			else
			{
				GetFileTime (hfile, NULL, NULL, &s_filetimeNetPrefs);
				CloseHandle (hfile);
			}
		}

		s_heventKill = CreateEvent (
				NULL, FALSE, FALSE, NULL);

		if (IsntNull (s_heventKill))
		{
			s_hthreadPrefs = (HANDLE)_beginthreadex (
					NULL, 0, sFileChangeThread, NULL, 0, &dwID);
		}
	}

	if (IsntPGPError (err))
	{
		err = CLSetTokenDll (kInvalidPGPPrefRef, NULL, FALSE);

		if (err == kPGPError_FileNotFound)
			err = kPGPError_NoErr;
	}

	return err;
}


//	______________________________________________
//
//	close down the PGPclientLib prefs system

PGPError
CLClosePrefs (VOID)
{
	PGPError	err1	= kPGPError_NoErr;
	PGPError	err2	= kPGPError_NoErr;

	// kill the file change or hidden window thread
	if (IsntNull (s_hthreadPrefs))
	{
		if (s_bUsingPrefsService)
		{
			SendMessage (s_hwndHidden, WM_CLOSE, 0, 0);
			WaitForSingleObject (s_hthreadPrefs, INFINITE);
			CloseHandle (s_hthreadPrefs);
		}
		else
		{
			SetEvent (s_heventKill);
			WaitForSingleObject (s_hthreadPrefs, INFINITE);
			CloseHandle (s_hthreadPrefs);
		}
	}

	if (s_heventKill)
	{
		CloseHandle (s_heventKill);
		s_heventKill = NULL;
	}

	if (s_hContactService)
	{
		CloseHandle (s_hContactService);
		s_hContactService = NULL;
	}

	if (IsntNull (s_pszPubringFile))
	{
		PGPFreeData (s_pszPubringFile);
		s_pszPubringFile = NULL;
	}

	if (IsntNull (s_pszSecringFile))
	{
		PGPFreeData (s_pszSecringFile);
		s_pszSecringFile = NULL;
	}

	if (PGPPrefRefIsValid (s_prefref))
	{
		err1 = sClosePrefs (s_prefref, FALSE);
		s_prefref = kInvalidPGPPrefRef;
	}

	if (PGPPrefRefIsValid (s_prefrefNet))
	{
		err2 = sClosePrefs (s_prefrefNet, FALSE);
		s_prefrefNet = kInvalidPGPPrefRef;
	}

	if (IsPGPError (err1))
		return err1;
	else
		return err2;
}


//	______________________________________________
//
//	test the prefs file hash for corruption

static PGPError
sCheckPrefsHash (
		PGPPrefRef		prefref,
		BOOL			bNetPrefs)
{
	PGPError		err			= kPGPError_NoErr;

	if(CHICKENANDEGG()) // Chicken and egg here.... This is all client install stuff so could be OK
	{
		if (PGPclIsClientInstall ())
		{
			err = PGPCheckPrefsHash (g_context, prefref);
			if ((err == kPGPError_CorruptData) ||
				(err == kPGPError_PrefNotFound))
			{
				CLMergeInAdminPrefs (g_context, prefref, bNetPrefs);

				// recheck hash ...
				err = PGPCheckPrefsHash (g_context, prefref);
				if ((err == kPGPError_CorruptData) ||
					(err == kPGPError_PrefNotFound))
				{
					err = kPGPError_Win32_PrefsHashBad;
				}
				else
					err = kPGPError_Win32_PrefRefNoLongerValid;
			}
		}
	}

	return err;
}


//	______________________________________________
//
//	see if the default keyring file name has been changed.
//	this is useful when running under Terminal Services since
//	the reload keyring broadcast message doesn't travel across
//	terminal sessions.

static BOOL
sIsKeyringFileNameChanged (
		PGPPrefRef		prefref)
{
	PGPError	err					= kPGPError_NoErr;
	BOOL		bAlloc				= FALSE;
	BOOL		bReturn				= FALSE;
	CHAR		szPath[MAX_PATH];

	err = PGPGetPrefStringBuffer (prefref, 
			kPGPPrefPublicKeyringFile, sizeof(szPath), szPath);

	if (IsntPGPError (err))
	{
		if (IsNull (s_pszPubringFile))
			bAlloc = TRUE;
		else
		{
			if (lstrcmp (s_pszPubringFile, szPath))
			{
				PGPFreeData (s_pszPubringFile);
				s_pszPubringFile = NULL;

				bAlloc = TRUE;
				bReturn = TRUE;
			}
		}

		if (bAlloc)
		{
			s_pszPubringFile = PGPNewData (
					PGPPeekContextMemoryMgr (g_context),
					lstrlen (szPath) +1, 0);
			lstrcpy (s_pszPubringFile, szPath);
		}
	}

	bAlloc = FALSE;
	err = PGPGetPrefStringBuffer (prefref, 
			kPGPPrefPrivateKeyringFile, sizeof(szPath), szPath);

	if (IsntPGPError (err))
	{
		if (IsNull (s_pszSecringFile))
			bAlloc = TRUE;
		else
		{
			if (lstrcmp (s_pszSecringFile, szPath))
			{
				PGPFreeData (s_pszSecringFile);
				s_pszSecringFile = NULL;
				bAlloc = TRUE;
				bReturn = TRUE;
			}
		}

		if (bAlloc)
		{
			s_pszSecringFile = PGPNewData (
					PGPPeekContextMemoryMgr (g_context),
					lstrlen (szPath) +1, 0);
			lstrcpy (s_pszSecringFile, szPath);
		}
	}

	return bReturn;
}


//	______________________________________________
//
//	return the requested prefrefs to the caller

PGPError PGPclExport
PGPclPeekClientLibPrefRefs (
		PGPPrefRef*		pprefref,
		PGPPrefRef*		pprefrefNet)
{
	PGPError		err			= kPGPError_NoErr;
	DWORD			dw;
	PFLFileSpecRef	fileref;
	PGPPrefRef		prefrefPrev;

	if (IsntNull (pprefref))
		*pprefref = kInvalidPGPPrefRef;
	if (IsntNull (pprefrefNet))
		*pprefrefNet = kInvalidPGPPrefRef;

	if (s_hContactService)
		WaitForSingleObject(s_hContactService, INFINITE);

	if (s_bUsingPrefsService)
	{
		if (IsntNull (pprefref))
		{
			prefrefPrev = s_prefref;

			if (SendMessageTimeout (s_hwndService, PGP_M_QUERYPREFS,
					(WPARAM)s_iPGPSequenceNumber, (LPARAM)s_hwndHidden,
					SMTO_NORMAL, SERVICE_MESSAGE_TIMEOUT, &dw) == 0)
			{
				err = kPGPError_Win32_NoPrefsServiceResponse;
			}

			if (IsntPGPError (err) &&
				(s_prefref != prefrefPrev))
			{
				err = sCheckPrefsHash (s_prefref, FALSE);
				if (err == kPGPError_Win32_PrefRefNoLongerValid)
				{
					err = PGPclFlushClientLibPrefs (s_prefref, NULL);
				}
			}

			if (IsPGPError (err))
			{
				if (PGPPrefRefIsValid (s_prefref))
					PGPFreePrefs (s_prefref);
				s_prefref = kInvalidPGPPrefRef;
				if (s_hContactService)
					ReleaseMutex(s_hContactService);
			}

			if (IsPGPError (err))
				return err;

			*pprefref = s_prefref;
		}

		if (IsntNull (pprefrefNet))
		{
			if (!s_bNetInstalled)
			{
				if (s_hContactService)
					ReleaseMutex(s_hContactService);
				return kPGPError_BadParams;
			}

			prefrefPrev = s_prefrefNet;

			if (SendMessageTimeout (s_hwndService, PGP_M_QUERYNETPREFS,
					(WPARAM)s_iNetSequenceNumber, (LPARAM)s_hwndHidden,
					SMTO_NORMAL, SERVICE_MESSAGE_TIMEOUT, &dw) == 0)
			{
				err = kPGPError_Win32_NoPrefsServiceResponse;
			}

			if (IsntPGPError (err) &&
				(s_prefrefNet != prefrefPrev))
			{
				err = sCheckPrefsHash (s_prefrefNet, TRUE);
				if (err == kPGPError_Win32_PrefRefNoLongerValid)
				{
					err = PGPclFlushClientLibPrefs (NULL, s_prefrefNet);
				}
			}

			if (IsPGPError (err))
			{
				if (PGPPrefRefIsValid (s_prefrefNet))
					PGPFreePrefs (s_prefrefNet);
				s_prefrefNet = kInvalidPGPPrefRef;
				if (s_hContactService)
					ReleaseMutex(s_hContactService);
			}

			if (IsPGPError (err))
				return err;

			*pprefrefNet = s_prefrefNet;
		}
	}
	else
	{
		// we're using stand-alone file-based prefs
		if (PGPPrefRefIsValid (s_prefref))
		{
			if (s_bPGPPrefsModified || sPGPPrefsFileTimeChanged ())
			{
				PGPFreePrefs (s_prefref);
				s_prefref = kInvalidPGPPrefRef;
				s_bPGPPrefsModified = FALSE;
			}
		}

		if (PGPPrefRefIsValid (s_prefrefNet))
		{
			if (s_bNetPrefsModified || sNetPrefsFileTimeChanged ())
			{
				PGPFreePrefs (s_prefrefNet);
				s_prefrefNet = kInvalidPGPPrefRef;
				s_bNetPrefsModified = FALSE;
			}
		}

		if (IsntNull (pprefref))
		{
			if (!PGPPrefRefIsValid (s_prefref) &&
				(s_szPGPPrefFile[0] != '\0'))
			{
				PFLNewFileSpecFromFullPath (PGPPeekContextMemoryMgr (g_context),
						s_szPGPPrefFile, &fileref);

				err = PGPOpenPrefFile (fileref,
						clientDefaults, clientDefaultsSize, &s_prefref);

				if ((err == kPGPError_FileOpFailed) ||
					(err == kPGPError_FileNotFound))
				{
					err = PFLFileSpecCreate (fileref);
					pgpAssert (IsntPGPError (err));
					err = PGPOpenPrefFile (fileref,
							clientDefaults, clientDefaultsSize, &s_prefref);
					pgpAssert (IsntPGPError (err));
				}
				PFLFreeFileSpec (fileref);

				if (IsntPGPError (err))
				{
					err = sCheckPrefsHash (s_prefref, FALSE);
					if (IsntPGPError (err))
					{
						if (sIsKeyringFileNameChanged (s_prefref))
						{
							PGPclNotifyKeyringChanges (
									kPGPclNewDefaultKeyrings, 
									GetCurrentProcessId ());
						}
					}
					else if (err == kPGPError_Win32_PrefRefNoLongerValid)
					{
						err = PGPSavePrefFile (s_prefref);
						if (IsntPGPError (err))
						{
							Sleep (100);
							s_bPGPPrefsModified = FALSE;
							memset (&s_filetimePGPPrefs, 0, sizeof(FILETIME));
						}
					}
				}

				if (IsPGPError (err))
				{
					if (PGPPrefRefIsValid (s_prefref))
						PGPFreePrefs (s_prefref);
					s_prefref = kInvalidPGPPrefRef;
					if (s_hContactService)
						ReleaseMutex(s_hContactService);
				}

				if (IsPGPError (err))
					return err;
			}
			*pprefref = s_prefref;
		}

		if (IsntNull (pprefrefNet))
		{
			if (!s_bNetInstalled)
			{
				if (s_hContactService)
					ReleaseMutex(s_hContactService);
				return kPGPError_BadParams;
			}

			if (!PGPPrefRefIsValid (s_prefrefNet))
			{
				PFLNewFileSpecFromFullPath (PGPPeekContextMemoryMgr (g_context),
						s_szNetPrefFile, &fileref);

				err = PGPOpenPrefFile (fileref,
						netDefaults, netDefaultsSize, &s_prefrefNet);

				if ((err == kPGPError_FileOpFailed) ||
					(err == kPGPError_FileNotFound))
				{
					err = PFLFileSpecCreate (fileref);
					pgpAssert (IsntPGPError (err));
					err = PGPOpenPrefFile (fileref, netDefaults,
							netDefaultsSize, &s_prefrefNet);
					pgpAssert (IsntPGPError (err));
				}

				if (fileref)
					PFLFreeFileSpec (fileref);

				if (IsntPGPError (err))
				{
					err = sCheckPrefsHash (s_prefrefNet, TRUE);
					if (err == kPGPError_Win32_PrefRefNoLongerValid)
					{
						err = PGPSavePrefFile (s_prefrefNet);
						if (IsntPGPError (err))
						{
							Sleep (100);
							s_bNetPrefsModified = FALSE;
							memset (&s_filetimeNetPrefs, 0, sizeof(FILETIME));
						}
					}
				}

				if (IsPGPError (err))
				{
					if (PGPPrefRefIsValid (s_prefrefNet))
						PGPFreePrefs (s_prefrefNet);
					s_prefrefNet = kInvalidPGPPrefRef;
					if (s_hContactService)
						ReleaseMutex(s_hContactService);
				}

				if (IsPGPError (err))
					return err;
			}
			*pprefrefNet = s_prefrefNet;
		}
	}

	if (s_hContactService)
		ReleaseMutex(s_hContactService);
	return err;
}


//	______________________________________________
//
//	flush the specified prefrefs to disk or service

PGPError PGPclExport
PGPclFlushClientLibPrefs (
		PGPPrefRef		prefref,
		PGPPrefRef		prefrefNet)
{
	PGPError	err			= kPGPError_NoErr;
	BOOL		bPGPSaved	= FALSE;
	BOOL		bNetSaved	= FALSE;
	PGPByte*	pbyte;
	DWORD		dw;

	if (s_bUsingPrefsService)
	{
		COPYDATASTRUCT					cds;
		PGPPrefsCopyDataStructure*		ppcds;

		if (PGPPrefRefIsValid (prefref))
		{
			err = PGPExportPrefsToBuffer (s_prefref, &dw, &pbyte);
			if (IsntPGPError (err))
			{
				ppcds = PGPNewData (
						PGPPeekContextMemoryMgr (g_context),
						dw + sizeof (PGPPrefsCopyDataStructure) - 1, 0);

				if (IsntNull (ppcds))
				{
					ppcds->sequenceNumber = s_iPGPSequenceNumber;
					ppcds->bufferLength = dw;
					pgpCopyMemory (pbyte, ppcds->buffer, dw);
					PGPFreeData (pbyte);

					cds.cbData = sizeof(PGPPrefsCopyDataStructure) + dw - 1;
					cds.lpData = ppcds;
					cds.dwData = PGP_PGPPREFSDATA;

					if (SendMessageTimeout (s_hwndService, WM_COPYDATA,
							(WPARAM)s_hwndHidden, (LPARAM) &cds,
							SMTO_NORMAL, SERVICE_MESSAGE_TIMEOUT, &dw) == 0)
					{
						err = kPGPError_Win32_NoPrefsServiceResponse;
					}
					else
					{
						if (dw == PGP_INVALIDPREFSEQUENCE)
							err = kPGPError_Win32_PrefRefNoLongerValid;
						else
							s_iPGPSequenceNumber = dw;
					}
					PGPFreeData (ppcds);
				}
			}
		}

		if (s_bNetInstalled && PGPPrefRefIsValid (prefrefNet))
		{
			err = PGPExportPrefsToBuffer (s_prefrefNet, &dw, &pbyte);
			if (IsntPGPError (err))
			{
				ppcds = PGPNewData (
						PGPPeekContextMemoryMgr (g_context),
						dw + sizeof (PGPPrefsCopyDataStructure) - 1, 0);

				if (IsntNull (ppcds))
				{
					ppcds->sequenceNumber = s_iNetSequenceNumber;
					ppcds->bufferLength = dw;
					pgpCopyMemory (pbyte, ppcds->buffer, dw);
					PGPFreeData (pbyte);

					cds.cbData = sizeof(PGPPrefsCopyDataStructure) + dw - 1;
					cds.lpData = ppcds;
					cds.dwData = PGP_PGPNETPREFSDATA;

					if (SendMessageTimeout (s_hwndService, WM_COPYDATA,
							(WPARAM)s_hwndHidden, (LPARAM) &cds,
							SMTO_NORMAL, SERVICE_MESSAGE_TIMEOUT, &dw) == 0)
					{
						err = kPGPError_Win32_NoPrefsServiceResponse;
					}
					else
					{
						if (dw == PGP_INVALIDPREFSEQUENCE)
							err = kPGPError_Win32_PrefRefNoLongerValid;
						else
							s_iNetSequenceNumber = dw;
					}
					PGPFreeData (ppcds);
				}
			}
		}
	}
	else
	{
		if (s_bPGPPrefsModified && PGPPrefRefIsValid (s_prefref))
		{
			PGPFreePrefs (s_prefref);
			s_prefref = kInvalidPGPPrefRef;
			s_bPGPPrefsModified = FALSE;
		}

		if (s_bNetPrefsModified && PGPPrefRefIsValid (s_prefrefNet))
		{
			PGPFreePrefs (s_prefrefNet);
			s_prefrefNet = kInvalidPGPPrefRef;
			s_bNetPrefsModified = FALSE;
		}

		if (PGPPrefRefIsValid (prefref))
		{
			if (prefref == s_prefref)
			{
				err = PGPSavePrefFile (prefref);
				if (IsntPGPError (err))
					bPGPSaved = TRUE;
			}
			else
				err = kPGPError_Win32_PrefRefNoLongerValid;
		}

		if (IsPGPError (err))
			return err;

		if (s_bNetInstalled && PGPPrefRefIsValid (prefrefNet))
		{
			if (prefrefNet == s_prefrefNet)
			{
				err = PGPSavePrefFile (prefrefNet);
				if (IsntPGPError (err))
					bNetSaved = TRUE;
			}
			else
				err = kPGPError_Win32_PrefRefNoLongerValid;
		}

		if (bPGPSaved || bNetSaved)
			Sleep (100);

		if (bPGPSaved)
		{
			s_bPGPPrefsModified = FALSE;
			memset (&s_filetimePGPPrefs, 0, sizeof(FILETIME));
			PGPclNotifyPrefsChanges (0);
		}

		if (bNetSaved)
		{
			s_bNetPrefsModified = FALSE;
			memset (&s_filetimeNetPrefs, 0, sizeof(FILETIME));
			PGPclNotifyNetPrefsChanges (0);
		}
	}

	return err;
}


//	______________________________________________
//
//	change the default prefs to use another file

PGPError PGPclExport
PGPclChangeDefaultPrefsFile (
		char*			pszPath)
{
	PGPError		err;

	if (s_bUsingPrefsService)
	{
		err = kPGPError_Win32_CantChangePrefsFile;

		if (s_hwndHidden)
		{
			COPYDATASTRUCT	cds;
			DWORD			dw;

			if (pszPath)
				lstrcpy (s_szPGPPrefFile, pszPath);
			else
				s_szPGPPrefFile[0] = '\0';

			cds.dwData = PGP_PGPPREFSLOGON;
			cds.lpData = s_szPGPPrefFile;
			cds.cbData = sizeof (s_szPGPPrefFile);

			if (SendMessageTimeout (s_hwndService, WM_COPYDATA,
					(WPARAM)s_hwndHidden, (LPARAM)&cds,
					SMTO_NORMAL, SERVICE_MESSAGE_TIMEOUT, &dw) != 0)
			{
				if (dw == 1)
					err = kPGPError_NoErr;
			}
		}
	}

	// using local prefs file code
	else
	{
		if (PGPPrefRefIsValid (s_prefref))
		{
			PGPFreePrefs (s_prefref);
			s_prefref = kInvalidPGPPrefRef;
		}

		if (pszPath)
			lstrcpy (s_szPGPPrefFile, pszPath);
		else
		{
			PGPclGetPath (kPGPclPrefsFile,
					s_szPGPPrefFile, sizeof(s_szPGPPrefFile));
		}

		err = kPGPError_NoErr;
	}

	// attempt to change randseed file to that specified in pref file
	if (IsntPGPError (err))
	{
		PFLFileSpecRef	filespec		= kInvalidPFLFileSpecRef;
		CHAR			szPath[MAX_PATH];
		PGPPrefRef		prefref;

		err = PGPclPeekClientLibPrefRefs (&prefref, NULL);
		if (IsntPGPError (err))
		{
			err = PGPGetPrefStringBuffer (prefref,
					kPGPPrefRNGSeedFile, sizeof(szPath), szPath);

			if (IsntPGPError (err))
			{
				err = PFLNewFileSpecFromFullPath (
						PGPPeekContextMemoryMgr (g_context),
						szPath, &filespec);
				if (IsntPGPError (err))
					err = PGPSetRandSeedFile (filespec);
				if (PFLFileSpecRefIsValid (filespec))
					PFLFreeFileSpec (filespec);
			}
		}
	}

	return err;
}

