/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	CLprefsCA.c - handle PGP CA preferences dialogs

	$Id: CLprefsCA.c,v 1.23 2002/08/06 20:09:19 dallen Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "CLprefs.h"

// definitions
#define DELAYTIMER	234

// external globals
extern HINSTANCE	g_hInst;

static DWORD aCAIds[] = {			// Help IDs
	IDC_URL,				IDH_PGPCLPREF_CAURL,
	IDC_REVURL,				IDH_PGPCLPREF_CAREVOCATIONURL,
	IDC_TYPE,				IDH_PGPCLPREF_CATYPE,
	IDC_ROOTCERT,			IDH_PGPCLPREF_CAROOTCERT,
	IDC_CLEARCERT,			IDH_PGPCLPREF_CACLEARROOTCERT,
	IDC_SETCERT,			IDH_PGPCLPREF_CASELECTROOTCERT,
    0,0
};


//	_____________________________________________________
//
//  get root CA cert from prefs

PGPError PGPclExport
PGPclGetRootCACertPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PGPKeyDBRef			keydb,
		PGPKeyDBObjRef*		psigCert)
{
	PGPByte*		pIASNX509			= NULL;
	PGPKeyID*		pkeyidX509			= NULL;

	PGPUInt32		uX509IASNLength;
	PGPSize			size;
	PGPError		err;

	err = PGPGetPrefData (prefref,
					kPGPPrefCARootKeyID,
					&size, &pkeyidX509); CKERR;

	err = PGPGetPrefData (prefref,
					kPGPPrefCARootX509IASN,
					&uX509IASNLength, &pIASNX509); CKERR;

	err = PGPclX509CertFromExport (context, pkeyidX509,
					pIASNX509, uX509IASNLength, keydb, psigCert);

done:
	if (pkeyidX509)
		PGPFreeData (pkeyidX509);
	if (pIASNX509)
		PGPFreeData (pIASNX509);

	if (err == kPGPError_PrefNotFound)
		err = kPGPError_Win32_NoRootCACertPrefs;

	return err;
}


//	_____________________________________________________
//
//  set root CA cert prefs

static PGPError
sSetRootCACertPrefs (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		BOOL			bCertSet,
		PGPKeyDBObjRef	sigCert)
{
	PGPKeyID		keyidX509;
	PGPByte*		pIASNX509;
	PGPUInt32		uX509IASNLength;
	PGPError		err;

	if (bCertSet)
	{
		if (PGPKeyDBObjRefIsValid (sigCert))
		{
			err = PGPclX509CertToExport (context, sigCert,
					&keyidX509, &pIASNX509, &uX509IASNLength);

			if (IsntPGPError (err))
			{
				if (IsntNull (pIASNX509))
				{
					err = PGPSetPrefData (prefref,
							kPGPPrefCARootKeyID,
							sizeof(PGPKeyID), &keyidX509); CKERR;

					err = PGPSetPrefData (prefref,
							kPGPPrefCARootX509IASN,
							uX509IASNLength, pIASNX509); CKERR;

					PGPFreeData (pIASNX509);
				}
			}
		}
	}
	else
	{
		err = PGPSetPrefData (prefref,
				kPGPPrefCARootKeyID,
				0, NULL); CKERR;

		err = PGPSetPrefData (prefref,
				kPGPPrefCARootX509IASN,
				0, NULL); CKERR;
	}

done:
	return err;
}


//	_____________________________________________________
//
//  get CA server info from prefs

PGPError
CLGetCAServerPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PGPKeyServerEntry**	ppentry)
{
	PGPError		err			= kPGPError_BadParams;
	PGPKeyID*		pkeyid		= NULL;
	PGPSize			size;

	if (!ppentry)
		return kPGPError_BadParams;

	*ppentry = PGPNewData (PGPPeekContextMemoryMgr (context),
				2*sizeof(PGPKeyServerEntry), kPGPMemoryMgrFlags_Clear);

	if (*ppentry)
	{
		// server
		err = PGPGetPrefNumber (prefref,
					kPGPPrefCAType,
					&(*ppentry)[0].type); CKERR;

		err = PGPGetPrefStringBuffer (prefref,
					kPGPPrefCAServerURL,
					kMaxServerNameLength,
					(*ppentry)[0].serverDNS); CKERR;

		err = PGPGetPrefData (prefref,
					kPGPPrefCAServerAuthKeyID,
					&size, &pkeyid);

		if (IsntPGPError (err) && IsntNull (pkeyid))
		{
			err = PGPGetKeyIDString (pkeyid, kPGPKeyIDString_Full,
					(*ppentry)[0].authKeyIDString); CKERR;
			err = PGPGetKeyIDAlgorithm (
					pkeyid, &(*ppentry)[0].authAlg); CKERR;
			PGPFreeData (pkeyid);
		}
		else
		{
			(*ppentry)[0].authAlg = kPGPPublicKeyAlgorithm_Invalid;
			err = kPGPError_NoErr;
		}

		// revocation server
		(*ppentry)[1].type = (*ppentry)[0].type;

		err = PGPGetPrefStringBuffer (prefref,
					kPGPPrefCARevocationServerURL,
					kMaxServerNameLength,
					(*ppentry)[1].serverDNS); CKERR;

		err = PGPGetPrefData (prefref,
					kPGPPrefCARevocationServerAuthKeyID,
					&size, &pkeyid);

		if (IsntPGPError (err) && IsntNull (pkeyid))
		{
			err = PGPGetKeyIDString (pkeyid, kPGPKeyIDString_Full,
					(*ppentry)[1].authKeyIDString); CKERR;
			err = PGPGetKeyIDAlgorithm (
					pkeyid, &(*ppentry)[1].authAlg); CKERR;
			PGPFreeData (pkeyid);
		}
		else
		{
			(*ppentry)[1].authAlg = kPGPPublicKeyAlgorithm_Invalid;
			err = kPGPError_NoErr;
		}
	}
	else
		err = kPGPError_OutOfMemory;

done:
	return err;
}


//	_____________________________________________________
//
//  set CA server info

PGPError
CLSetCAServerPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PGPKeyServerEntry*	pentry)
{
	PGPError		err		= kPGPError_NoErr;
	PGPKeyID		keyid;

	if (pentry)
	{
		// server
		err = PGPSetPrefNumber (prefref,
					kPGPPrefCAType,
					pentry->type); CKERR;

		err = PGPSetPrefString (prefref,
					kPGPPrefCAServerURL,
					pentry->serverDNS); CKERR;

		if (pentry->authAlg != kPGPPublicKeyAlgorithm_Invalid)
		{
			err = PGPNewKeyIDFromString (pentry->authKeyIDString,
					pentry->authAlg, &keyid); CKERR;

			err = PGPSetPrefData (prefref,
					kPGPPrefCAServerAuthKeyID,
					sizeof(PGPKeyID), &keyid); CKERR;
		}
		else
		{
			err = PGPSetPrefData (prefref,
					kPGPPrefCAServerAuthKeyID,
					0, NULL); CKERR;
		}
	}
	else
		return kPGPError_BadParams;

done:
	return err;
}


//	_____________________________________________________
//
//  set CA revocation server info

PGPError
CLSetCARevocationServerPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PGPKeyServerEntry*	pentry)
{
	PGPError		err			= kPGPError_NoErr;
	PGPKeyID		keyid;

	if (pentry)
	{
		// revocation server
		err = PGPSetPrefString (prefref,
					kPGPPrefCARevocationServerURL,
					pentry->serverDNS); CKERR;

		if (pentry->authAlg != kPGPPublicKeyAlgorithm_Invalid)
		{
			err = PGPNewKeyIDFromString (pentry->authKeyIDString,
					pentry->authAlg, &keyid); CKERR;

			err = PGPSetPrefData (prefref,
					kPGPPrefCARevocationServerAuthKeyID,
					sizeof(PGPKeyID), &keyid); CKERR;
		}
		else
		{
			err = PGPSetPrefData (prefref,
					kPGPPrefCARevocationServerAuthKeyID,
					0, NULL); CKERR;
		}
	}
	else
		return kPGPError_BadParams;

done:
	return err;
}


//	_____________________________________________________
//
//  Message processing function for property sheet dialog

LRESULT WINAPI
CLCAPropDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PPREFSDLGSTRUCT	ppds;

	switch (uMsg) {

	case WM_INITDIALOG:
	{
		CHAR		sz[256];
		UINT		u, uSelect;
		PGPUInt32	uType;
		PGPPrefRef	prefref;

		SetWindowLong (hwnd, GWL_USERDATA, ((PROPSHEETPAGE*)lParam)->lParam);
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		if (ppds->bNeedsCentering)
		{
			PGPclCenterWindowOnMonitor (GetParent (hwnd), kPGPclWindowMonitor);
			ppds->bNeedsCentering = FALSE;
		}

		prefref = CLGetPrefRef (ppds);

		// initialize URLs
		PGPGetPrefStringBuffer (prefref,
				kPGPPrefCAServerURL, sizeof(sz), sz);
		SetDlgItemText (hwnd, IDC_URL, sz);

		CLDisablePrefControl (hwnd, IDC_URL,
				ppds, prefref, kPGPPrefCAServerURL);


		PGPGetPrefStringBuffer (prefref,
				kPGPPrefCARevocationServerURL, sizeof(sz), sz);
		SetDlgItemText (hwnd, IDC_REVURL, sz);

		CLDisablePrefControl (hwnd, IDC_REVURL,
				ppds, prefref, kPGPPrefCARevocationServerURL);

		// init CA server type combo box
		uSelect = 0;
		uType = kPGPKeyServerClass_Invalid;
		PGPGetPrefNumber (prefref, kPGPPrefCAType, &uType);

		LoadString (g_hInst, IDS_CATYPENETTOOLS, sz, sizeof(sz));
		u = SendDlgItemMessage (hwnd, IDC_TYPE, CB_ADDSTRING, 0, (LPARAM)sz);
		if (u != CB_ERR)
		{
			SendDlgItemMessage (hwnd, IDC_TYPE, CB_SETITEMDATA,
					u, (LPARAM)kPGPKeyServerClass_NetToolsCA);
			if (uType == kPGPKeyServerClass_NetToolsCA)
				uSelect = u;
		}

		LoadString (g_hInst, IDS_CATYPEVERISIGN, sz, sizeof(sz));
		u = SendDlgItemMessage (hwnd, IDC_TYPE, CB_ADDSTRING, 0, (LPARAM)sz);
		if (u != CB_ERR)
		{
			SendDlgItemMessage (hwnd, IDC_TYPE, CB_SETITEMDATA,
					u, (LPARAM)kPGPKeyServerClass_Verisign);
			if (uType == kPGPKeyServerClass_Verisign)
				uSelect = u;
		}

		LoadString (g_hInst, IDS_CATYPEENTRUST, sz, sizeof(sz));
		u = SendDlgItemMessage (hwnd, IDC_TYPE, CB_ADDSTRING, 0, (LPARAM)sz);
		if (u != CB_ERR)
		{
			SendDlgItemMessage (hwnd, IDC_TYPE, CB_SETITEMDATA,
					u, (LPARAM)kPGPKeyServerClass_Entrust);
			if (uType == kPGPKeyServerClass_Entrust)
				uSelect = u;
		}

		LoadString (g_hInst, IDS_CATYPENETSCAPE, sz, sizeof(sz));
		u = SendDlgItemMessage (hwnd, IDC_TYPE, CB_ADDSTRING, 0, (LPARAM)sz);
		if (u != CB_ERR)
		{
			SendDlgItemMessage (hwnd, IDC_TYPE, CB_SETITEMDATA,
					u, (LPARAM)kPGPKeyServerClass_Netscape);
			if (uType == kPGPKeyServerClass_Netscape)
				uSelect = u;
		}

		LoadString (g_hInst, IDS_CATYPEMICROSOFT, sz, sizeof(sz));
		u = SendDlgItemMessage (hwnd, IDC_TYPE, CB_ADDSTRING, 0, (LPARAM)sz);
		if (u != CB_ERR)
		{
			SendDlgItemMessage (hwnd, IDC_TYPE, CB_SETITEMDATA,
					u, (LPARAM)kPGPKeyServerClass_Microsoft);
			if (uType == kPGPKeyServerClass_Microsoft)
				uSelect = u;
		}

		SendDlgItemMessage (hwnd, IDC_TYPE, CB_SETCURSEL, uSelect, 0);
		CLDisablePrefControl (hwnd, IDC_TYPE,
				ppds, prefref, kPGPPrefCAType);

		// initialize root cert edit box
		ppds->ca.sigCert = kInvalidPGPKeyDBObjRef;

		CLDisablePrefControl (hwnd, IDC_ROOTCERT,
				ppds, prefref, kPGPPrefCARootKeyID);
		CLDisablePrefControl (hwnd, IDC_SETCERT,
				ppds, prefref, kPGPPrefCARootKeyID);
		CLDisablePrefControl (hwnd, IDC_CLEARCERT,
				ppds, prefref, kPGPPrefCARootKeyID);

		return TRUE;
	}

	case WM_TIMER :
	{
		PGPError	err		= kPGPError_NoErr;

		KillTimer (hwnd, DELAYTIMER);

		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		// load keyset if not already loaded
		if (!PGPKeyDBRefIsValid (ppds->keydbMain))
		{
			HCURSOR		hcursorOld;

			hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
			err = PGPclErrorBox (hwnd,
					PGPclOpenDefaultKeyrings (
							ppds->context, kPGPOpenKeyDBFileOptions_None,
							&ppds->keydbMain));
			SetCursor (hcursorOld);
		}

		if (IsntPGPError (err))
		{
			PGPError	err;
			PGPPrefRef	prefref		= CLGetPrefRef (ppds);

			err = PGPclGetRootCACertPrefs (ppds->context, prefref,
					ppds->keydbMain, &ppds->ca.sigCert);

			if (err == kPGPError_Win32_NoRootCACertPrefs)
				ppds->ca.bCertSet = FALSE;
			else
				ppds->ca.bCertSet = TRUE;

			PostMessage (hwnd, WM_APP, 0, 0);
		}
		break;
	}

	case WM_APP :
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		if ((PGPKeyDBObjRefIsValid (ppds->ca.sigCert)) ||
			(ppds->ca.bCertSet))
		{
			CHAR		sz[256];
			PGPBoolean	b;
			PGPPrefRef	prefref;
			PGPSize		size;

			prefref = CLGetPrefRef (ppds);

			if (PGPKeyDBObjRefIsValid (ppds->ca.sigCert))
			{
				LPSTR	psz;

				ppds->ca.bCertSet = TRUE;

				psz = NULL;
				PGPGetKeyDBObjAllocatedDataProperty (ppds->ca.sigCert, 
						kPGPSigProperty_X509LongName, &psz, &size);

				SetDlgItemText (hwnd, IDC_ROOTCERT, psz);
				if (IsntNull (psz))
					PGPFreeData (psz);
			}
			else
			{
				PGPKeyID*	pkeyid;
				CHAR		szID[kPGPMaxKeyIDStringSize +1];

				if (IsntPGPError (PGPGetPrefData (prefref,
						kPGPPrefCARootKeyID, &size, &pkeyid)))
				{
					PGPGetKeyIDString (pkeyid,
							kPGPKeyIDString_Abbreviated, szID);
					PGPFreeData (pkeyid);

					LoadString (g_hInst, IDS_UNKNOWNCERT, sz, sizeof(sz));
					lstrcat (sz, szID);
				}
				else
					LoadString (g_hInst, IDS_UNKNOWNKEY, sz, sizeof(sz));

				SetDlgItemText (hwnd, IDC_ROOTCERT, sz);
			}

			b = CLDisablePrefControl (hwnd, IDC_CLEARCERT,
					ppds, prefref, kPGPPrefCARootKeyID);
			if (!b)
				EnableWindow (GetDlgItem (hwnd, IDC_CLEARCERT), TRUE);
		}
		else
		{
			SetDlgItemText (hwnd, IDC_ROOTCERT, "");
			EnableWindow (GetDlgItem (hwnd, IDC_CLEARCERT), FALSE);
		}
		break;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aCAIds);

	case WM_COMMAND :
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		switch (LOWORD (wParam)) {
		case IDC_URL :
			if (HIWORD (wParam) == EN_CHANGE)
				ppds->ca.bUrlModified = TRUE;
			break;

		case IDC_REVURL :
			if (HIWORD (wParam) == EN_CHANGE)
				ppds->ca.bRevUrlModified = TRUE;
			break;

		case IDC_CLEARCERT :
			if (ppds->ca.bCertSet)
			{
				ppds->ca.bCertSet = FALSE;
				ppds->ca.sigCert = kInvalidPGPKeyDBObjRef;
				PostMessage (hwnd, WM_APP, 0, 0);
			}
			break;

		case IDC_SETCERT :
		{
			PGPKeyDBObjRef	sig;
			CHAR			sz[128];

			LoadString (g_hInst, IDS_ROOTCAPROMPT, sz, sizeof(sz));

			if (IsntPGPError (PGPclSelectX509Cert (ppds->context,
					hwnd, sz, PGPPeekKeyDBRootKeySet (ppds->keydbMain),
					kPGPclSelectX509NoSplitKeys,
					&sig)))
			{
				ppds->ca.bCertSet = TRUE;
				ppds->ca.sigCert = sig;
				PostMessage (hwnd, WM_APP, 0, 0);
			}
			break;
		}
		}
		break;

	case WM_NOTIFY :
		switch (((NMHDR FAR *) lParam)->code) {
		case PSN_SETACTIVE :
			ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			ppds->ca.bUrlModified = FALSE;
			ppds->ca.bRevUrlModified = FALSE;
			SetTimer (hwnd, DELAYTIMER, 100, NULL);
			break;

		case PSN_HELP :
			PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0, 
					(char*)kPGPclHelpFile, (DWORD*)IDH_PGPCLPREF_CADIALOG);
			break;

		case PSN_APPLY :
		{
			CHAR			szNew[256];
			CHAR			szOld[256];
			UINT			u;
			PGPInt32		iType;
			PGPError		err;

			ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			if (ppds->bUseClientLibPrefs && 
				!PGPPrefRefIsValid (ppds->prefref))
			{
				PGPclPeekClientLibPrefRefs (&ppds->prefref, NULL);
			}

			// get old and new URL strings and compare
			PGPGetPrefStringBuffer (ppds->prefref,
					kPGPPrefCAServerURL, sizeof(szOld), szOld);
			GetDlgItemText (hwnd, IDC_URL, szNew, sizeof(szNew));
			if (lstrcmpi (szNew, szOld))
			{
				GetDlgItemText (hwnd, IDC_URL, szNew, sizeof(szNew));
				PGPclErrorBox (hwnd, PGPSetPrefString (ppds->prefref,
						kPGPPrefCAServerURL, szNew));

				PGPclErrorBox (hwnd, PGPSetPrefData (ppds->prefref,
						kPGPPrefCAServerAuthKeyID,
						0, NULL));
			}

			// get old and new RevURL strings and compare
			PGPGetPrefStringBuffer (ppds->prefref,
					kPGPPrefCARevocationServerURL, sizeof(szOld), szOld);
			GetDlgItemText (hwnd, IDC_REVURL, szNew, sizeof(szNew));
			if (lstrcmpi (szNew, szOld))
			{
				GetDlgItemText (hwnd, IDC_REVURL, szNew, sizeof(szNew));
				PGPclErrorBox (hwnd, PGPSetPrefString (ppds->prefref,
						kPGPPrefCARevocationServerURL, szNew));

				PGPclErrorBox (hwnd, PGPSetPrefData (ppds->prefref,
						kPGPPrefCARevocationServerAuthKeyID,
						0, NULL));
			}

			// get and save CA type
			u = SendDlgItemMessage (hwnd, IDC_TYPE, CB_GETCURSEL, 0, 0);
			iType = SendDlgItemMessage (hwnd, IDC_TYPE, CB_GETITEMDATA, u, 0);
			PGPclErrorBox (hwnd, PGPSetPrefNumber (ppds->prefref,
					kPGPPrefCAType, iType));

			// get and save root cert
			err = sSetRootCACertPrefs (ppds->context, ppds->prefref,
					ppds->ca.bCertSet, ppds->ca.sigCert);

			ppds->bPrefsModified = TRUE;
			ppds->returnErr = kPGPError_NoErr;
            SetWindowLong (hwnd, DWL_MSGRESULT, PSNRET_NOERROR);
            return TRUE;
		}

		case PSN_KILLACTIVE :
		{
			CHAR			szURL[256];
			UINT			u;

			ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			// if the user modified the URL, look for trailing slash
			if (ppds->ca.bUrlModified)
			{
				u = GetDlgItemText (hwnd, IDC_URL, szURL, sizeof(szURL));
				if (u > 0)
				{
					if ((szURL[u-1] == '\\') || (szURL[u-1] == '/'))
					{
						if (PGPclMessageBox (hwnd,
								IDS_CAPTIONWARNING, IDS_TRAILINGCASLASH,
								MB_ICONEXCLAMATION|MB_OKCANCEL) == IDCANCEL)
						{
							SendDlgItemMessage (
									hwnd, IDC_URL, EM_SETSEL, 0, -1);
							SetFocus (GetDlgItem (hwnd, IDC_URL));
							SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
							return TRUE;
						}
					}
				}
			}

			// if the user modified the Rev URL, look for trailing slash
			if (ppds->ca.bRevUrlModified)
			{
				u = GetDlgItemText (hwnd, IDC_REVURL, szURL, sizeof(szURL));
				if (u > 0)
				{
					if ((szURL[u-1] == '\\') || (szURL[u-1] == '/'))
					{
						if (PGPclMessageBox (hwnd,
								IDS_CAPTIONWARNING, IDS_TRAILINGCASLASH,
								MB_ICONEXCLAMATION|MB_OKCANCEL) == IDCANCEL)
						{
							SendDlgItemMessage (
									hwnd, IDC_REVURL, EM_SETSEL, 0, -1);
							SetFocus (GetDlgItem (hwnd, IDC_REVURL));
							SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
							return TRUE;
						}
					}
				}
			}

			ppds->returnErr = kPGPError_NoErr;
            SetWindowLong (hwnd, DWL_MSGRESULT, FALSE);
            return TRUE;
		}
		}
	}

	return FALSE;
}


