/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	CLprefsFiles.c - handle PGP file preferences dialogs

	$Id: CLprefsFiles.c,v 1.22 2002/10/19 00:18:58 wjb Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"
#include <direct.h>
#include <shlobj.h>

// project header files
#include "PGPclx.h"
#include "CLprefs.h"

#include "pgpFileSpec.h"
#include "pgpRandomPool.h"

// external globals
extern HINSTANCE		g_hInst;
extern PGPContextRef	g_context;


static DWORD aFileIds[] = {			// Help IDs
    IDC_PUBRING,			IDH_PGPCLPREF_PRIVATEKEYRING,
	IDC_PUBRINGBROWSE,		IDH_PGPCLPREF_BROWSEPRIVATEKEYRING,
    IDC_SECRING,			IDH_PGPCLPREF_PUBLICKEYRING,
	IDC_SECRINGBROWSE,		IDH_PGPCLPREF_BROWSEPUBLICKEYRING,
	IDC_NETPUBRING,			IDH_PGPCLPREF_NETPUBKEYRING,
	IDC_NETPUBRINGBROWSE,	IDH_PGPCLPREF_BROWSENETPUBKEYRING,
	IDC_NETSECRING,			IDH_PGPCLPREF_NETSECKEYRING,
	IDC_NETSECRINGBROWSE,	IDH_PGPCLPREF_BROWSENETSECKEYRING,
	IDC_NETSETTOCURRENT,	IDH_PGPCLPREF_USEMYKEYRINGS,
	IDC_RNGSEEDFILE,		IDH_PGPCLPREF_RANDOMSEEDFILE,
	IDC_RNGSEEDFILEBROWSE,	IDH_PGPCLPREF_BROWSERANDOMSEEDFILE,
	IDC_NETKEYRINGGROUP,	-1,
	0,0
};


//	____________________________________
//
//	compare set keyrings to current user's and enable button accordingly

static VOID
sCompareNetToPGPKeyringFiles (
		HWND			hwnd)
{
	BOOL			bSameAsCurrentUser	= TRUE;
	CHAR			szNet[MAX_PATH];
	CHAR			szPGP[MAX_PATH];

	GetDlgItemText (hwnd, IDC_PUBRING, szPGP, sizeof(szPGP));
	GetDlgItemText (hwnd, IDC_NETPUBRING, szNet, sizeof(szNet));
	if (lstrcmpi (szPGP, szNet))
		bSameAsCurrentUser = FALSE;

	GetDlgItemText (hwnd, IDC_SECRING, szPGP, sizeof(szPGP));
	GetDlgItemText (hwnd, IDC_NETSECRING, szNet, sizeof(szNet));
	if (lstrcmpi (szPGP, szNet))
		bSameAsCurrentUser = FALSE;

	if (bSameAsCurrentUser)
		EnableWindow (GetDlgItem (hwnd, IDC_NETSETTOCURRENT), FALSE);
	else
		EnableWindow (GetDlgItem (hwnd, IDC_NETSETTOCURRENT), TRUE);
}


//	____________________________
//
//  Check for illegal file names

static BOOL
sIsFileNameOK (LPSTR sz)
{
	UINT	uLen;
	DWORD	dwAttrib;

	uLen = lstrlen (sz);

	if (uLen < 4) return FALSE;
	if ((sz[1] != ':') && (sz[1] != '\\')) return FALSE;
	if ((sz[1] == ':') && (sz[2] != '\\')) return FALSE;
	if (strcspn (sz, "/*?""<>|") != uLen) return FALSE;
	if (sz[uLen-1] == ':') return FALSE;
	if (sz[uLen-1] == '\\') return FALSE;

	dwAttrib = GetFileAttributes (sz);
	if (dwAttrib == 0xFFFFFFFF) return TRUE;	// file doesn't exist
	if (dwAttrib & FILE_ATTRIBUTE_DIRECTORY) return FALSE;
	if (dwAttrib & FILE_ATTRIBUTE_SYSTEM) return FALSE;

	return TRUE;
}


//	____________________________
//
//  Check for illegal file names

static BOOL
sCheckFileName (
	HWND	hwnd,
	INT		idc)
{
	CHAR	sz[MAX_PATH];

	GetDlgItemText (hwnd, idc, sz, sizeof(sz));

	if (!sIsFileNameOK (sz))
	{
		PGPclMessageBox (GetParent (hwnd),
				IDS_CAPTIONINFO, IDS_ILLEGALFILENAME,
				MB_OK|MB_ICONINFORMATION);

		SendDlgItemMessage (hwnd, idc, EM_SETSEL, 0, -1);
		SetFocus (GetDlgItem (hwnd, idc));

		return FALSE;
	}

	return TRUE;
}

//	_____________________________________________________
//
//  verify that file or backup of file exists.
//	if not, replace with default file name in install dir
//	if no install dir found, use Windows dir

static BOOL
sVerifyFileExists (
		LPSTR		pszFile)
{
	CHAR		sz[MAX_PATH];
	PGPUInt32	uStatus;
	CHAR*		p;

	uStatus = PGPclTestPathNameStatus (pszFile);

	if (uStatus & kPGPclMediaNotInserted)
		return FALSE;

	if ((uStatus & kPGPclInvalidPath) || 
		(uStatus & kPGPclIsFolder))
	{
		lstrcpy (sz, pszFile);
		p = strrchr (sz, '\\');
		if (p) 
			p++;
		else 
			p = sz;

		if (IsPGPError (PGPclGetPath (
					kPGPclDefaultKeyringFolder, pszFile, MAX_PATH)))
		{
			GetWindowsDirectory (pszFile, MAX_PATH);
		}

		lstrcat (pszFile, p);
	}

	return TRUE;
}

//	_____________________________________________________
//
//  put up common dialog to browse for keyring file

static VOID
sBrowseForKeyringFile (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds,
		INT				idc,
		INT				idsFilter,
		INT				idsCaption,
		INT				idsDefaultExt,
		BOOL			bRandseed,
		LPSTR			pszFile,
		INT				iLen)
{
	OPENFILENAME	ofn;
	CHAR			szFilter[128];
	CHAR			szTitle[128];
	CHAR			szInitDir[MAX_PATH];
	LPSTR			pszInitDir;
	CHAR			szDefExt[8];
	CHAR*			p;
	DWORD			dw;

	GetDlgItemText (hwnd, idc, pszFile, iLen);
	dw = GetFileAttributes (pszFile);
	if (dw != 0xFFFFFFFF)
	{
		if (ppds->file.szInitDir[0] == '\0')
		{
			lstrcpy (ppds->file.szInitDir, pszFile);
			p = strrchr (ppds->file.szInitDir, '\\');
			if (p)
				*p = '\0';
			if (dw & FILE_ATTRIBUTE_DIRECTORY)
				pszFile[0] = '\0';
			else
				lstrcpy (pszFile, ++p);
		}
		else
			pszFile[0] = '\0';

		pszInitDir = ppds->file.szInitDir;
	}
	else 
	{
		if (bRandseed)
		{
			PGPclGetPath (kPGPclDefaultRandseedFolder, 
					szInitDir, sizeof(szInitDir));
			pszInitDir = szInitDir;
		}
		else
		{
			if (ppds->file.szInitDir[0])
				pszFile[0] = '\0';
			else
			{
				PGPclGetPath (kPGPclDefaultKeyringFolder, 
						ppds->file.szInitDir, sizeof(ppds->file.szInitDir));
			}
			pszInitDir = ppds->file.szInitDir;
		}
	}

	LoadString (g_hInst, idsCaption, szTitle, sizeof(szTitle));
	LoadString (g_hInst, idsDefaultExt, szDefExt, sizeof(szDefExt));
	LoadString (g_hInst, idsFilter, szFilter, sizeof(szFilter));
	while (p = strrchr (szFilter, '@')) *p = '\0';

	ofn.lStructSize			 = sizeof (ofn);
	ofn.hwndOwner			= hwnd;
	ofn.hInstance			= (HANDLE)g_hInst;
	ofn.lpstrFilter			= szFilter;
	ofn.lpstrCustomFilter	= (LPTSTR)NULL;
	ofn.nMaxCustFilter		= 0L;
	ofn.nFilterIndex		= 1L;
	ofn.lpstrFile			= pszFile;
	ofn.nMaxFile			= iLen;
	ofn.lpstrFileTitle		= NULL;
	ofn.nMaxFileTitle		= 0;
	ofn.lpstrInitialDir		= pszInitDir;
	ofn.lpstrTitle			= szTitle;
	ofn.Flags				= OFN_HIDEREADONLY|OFN_SHAREAWARE|OFN_NOCHANGEDIR;
	ofn.nFileOffset			= 0;
	ofn.nFileExtension		= 0;
	ofn.lpstrDefExt			= szDefExt;
	ofn.lCustData			= 0;

	if (GetOpenFileName (&ofn)) 
	{
		SetDlgItemText (hwnd, idc, pszFile);
		if (ofn.nFileOffset > 0)
			pszFile[ofn.nFileOffset-1] = '\0';

		if (!bRandseed)
			lstrcpy (ppds->file.szInitDir, pszFile);
	}
}


//	___________________________________________________
//
//  Open the PGPnet keyring files specified by the prefs file

static PGPError
sOpenKeyrings (
		PGPContextRef				context,
		LPSTR						pszPubPath,
		LPSTR						pszPrivPath,
		PGPOpenKeyDBFileOptions		options,
		PGPKeyDBRef*				keydbOut)
{
	PGPError		err					= kPGPError_NoErr;
	PGPFileSpecRef	filerefPub			= kInvalidPGPFileSpecRef;
	PGPFileSpecRef	filerefPriv			= kInvalidPGPFileSpecRef;
	BOOL			bReadOnly			= FALSE;
	PGPUInt32		uStatus;

	*keydbOut = kInvalidPGPKeyDBRef;

	// test the pubring for various conditions
	uStatus = PGPclTestPathNameStatus (pszPubPath);

	if (uStatus & kPGPclMediaNotInserted)
	{
		err = kPGPError_CantOpenFile;
		goto done;
	}

	if (options & kPGPOpenKeyDBFileOptions_Mutable)
	{
		if (uStatus & (kPGPclIsReadOnly | kPGPclIsInReadOnlyFolder))
		{
			err = kPGPError_FilePermissions;
			goto done;
		}
	}


	// now do the same for the secring
	uStatus = PGPclTestPathNameStatus (pszPrivPath);

	if (uStatus & kPGPclMediaNotInserted)
	{
		err = kPGPError_CantOpenFile;
		goto done;
	}

	if (options & kPGPOpenKeyDBFileOptions_Mutable)
	{
		if (uStatus & (kPGPclIsReadOnly | kPGPclIsInReadOnlyFolder))
		{
			err = kPGPError_FilePermissions;
			goto done;
		}
	}

	err = PGPNewFileSpecFromFullPath (context,
			pszPubPath, &filerefPub);CKERR;
	err = PGPNewFileSpecFromFullPath (context,
			pszPrivPath, &filerefPriv);CKERR;
	err = PGPOpenKeyDBFile (context,
			options, filerefPub, filerefPriv, keydbOut); CKERR;

done:
	if (PGPFileSpecRefIsValid (filerefPub))
		PGPFreeFileSpec (filerefPub );
	if (PGPFileSpecRefIsValid (filerefPriv))
		PGPFreeFileSpec (filerefPriv);

	return err;
}


//	___________________________________________________
//
//  If this is a new, non-existant file, ask user what to do

static VOID
sQueryCopyNewKeyringFile (
		PPREFSDLGSTRUCT	ppds,
		LPSTR			pszFile,
		UINT			uFile,
		LPSTR			pszPub,
		LPSTR			pszPriv)
{
	BOOL			bError			= TRUE;

	CHAR			szTemp[MAX_PATH];
	CHAR			szPref[MAX_PATH];
	CHAR			szText[MAX_PATH+MAX_PATH+128];
	CHAR			sz[128];
	HANDLE			hfile;
	LPSTR			pszPubNew, pszPrivNew;
	PGPKeyDBRef		keydbOld, keydbNew;

	PGPclGetPath (uFile, szPref, sizeof(szPref));

	if (lstrcmpi (pszFile, szPref) == 0)
		return;

	// set the prefs
	if ((uFile == kPGPclNetPublicKeyringFile) ||
		(uFile == kPGPclNetPrivateKeyringFile))
	{
		PGPclSetPath (ppds->prefrefNet, uFile, pszFile);
		ppds->net.bPrefsModified = TRUE;
		ppds->net.bKeyringModified = TRUE;
	}
	else
	{
		PGPclSetPath (ppds->prefref, uFile, pszFile);
		ppds->bKeyringModified = TRUE;
	}

	if (PGPclTestPathNameStatus (pszFile) & kPGPclIsFile)
		return;

	// create new emtpy file
	hfile = CreateFile (pszFile, GENERIC_READ|GENERIC_WRITE,
			0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL);
	CloseHandle (hfile);

	// ask user what to do
	LoadString (g_hInst, IDS_QUERYKEYRINGCOPY, sz, sizeof(sz));
	wsprintf (szText, sz, pszFile, szPref);
	LoadString (g_hInst, IDS_CAPTIONINFO, sz, sizeof(sz));

	if (MessageBox (ppds->hwndParent, szText, sz,
			MB_YESNO|MB_ICONINFORMATION) == IDYES)
	{
		ppds->file.bCopiedKeyrings = TRUE;

		GetTempPath (sizeof(szTemp), szTemp);
		GetTempFileName (szTemp, "PGP", 0, szTemp);

		if (PGPclTestPathNameStatus (szTemp) & kPGPclIsFile)
		{
			if ((uFile == kPGPclPublicKeyringFile) ||
				(uFile == kPGPclNetPublicKeyringFile))
			{
				pszPubNew = pszFile;
				pszPrivNew = szTemp;
			}
			else
			{
				pszPubNew = szTemp;
				pszPrivNew = pszFile;
			}

			if (IsntPGPError (sOpenKeyrings (
					ppds->context, pszPubNew, pszPrivNew, 
					kPGPOpenKeyDBFileOptions_Mutable, &keydbNew)))
			{
				if (IsntPGPError (sOpenKeyrings (
						ppds->context, pszPub, pszPriv, 0, &keydbOld)))
				{
					if (IsntPGPError (PGPCopyKeys (
						PGPPeekKeyDBRootKeySet (keydbOld), keydbNew, NULL)))
					{
						bError = FALSE;
					}
					PGPFreeKeyDB (keydbOld);
				}
				PGPFreeKeyDB (keydbNew);
			}
			DeleteFile (szTemp);
		}

		if (bError)
			ppds->file.bKeyringCopyError = TRUE;
	}
}


//	_____________________________________________________
//
//  Message processing function for property sheet dialog

LRESULT WINAPI
CLFilePropDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PPREFSDLGSTRUCT			ppds;

	switch (uMsg) {

	case WM_INITDIALOG:
	{
		PGPBoolean	b;
		PGPPrefRef	prefref;
		PGPPrefRef	prefrefNet;

		SetWindowLong (hwnd, GWL_USERDATA, ((PROPSHEETPAGE*)lParam)->lParam);
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		if (ppds->bNeedsCentering) 
		{
			PGPclCenterWindowOnMonitor (GetParent (hwnd), kPGPclWindowMonitor);
			ppds->bNeedsCentering = FALSE;
		}

		prefref = CLGetPrefRef (ppds);
		prefrefNet = CLGetPrefRefNet (ppds);

		// initialize PGP pubring file name
		if (!sVerifyFileExists (ppds->szPubringFile))
			ppds->file.bMediaNotInserted = TRUE;

		SetDlgItemText (hwnd, IDC_PUBRING, ppds->szPubringFile);
		CLDisablePrefControl (hwnd, IDC_PUBRING,
				ppds, prefref, kPGPPrefPublicKeyringFile);
		CLDisablePrefControl (hwnd, IDC_PUBRINGBROWSE,
				ppds, prefref, kPGPPrefPublicKeyringFile);

		// initialize PGP secring file name
		if (!sVerifyFileExists (ppds->szSecringFile))
			ppds->file.bMediaNotInserted = TRUE;

		SetDlgItemText (hwnd, IDC_SECRING, ppds->szSecringFile);
		CLDisablePrefControl (hwnd, IDC_SECRING,
				ppds, prefref, kPGPPrefPrivateKeyringFile);
		CLDisablePrefControl (hwnd, IDC_SECRINGBROWSE,
				ppds, prefref, kPGPPrefPrivateKeyringFile);

		if (ppds->net.bVPNPrefsEnabled &&
			PGPPrefRefIsValid (prefrefNet))
		{
			// initialize PGPnet pubring file name
			SetDlgItemText (hwnd, IDC_NETPUBRING, ppds->net.szPubringFile);
			CLDisablePrefControl (hwnd, IDC_NETPUBRING,
					ppds, prefrefNet, kPGPNetPrefPublicKeyringFile);
			CLDisablePrefControl (hwnd, IDC_NETPUBRINGBROWSE,
					ppds, prefrefNet, kPGPNetPrefPublicKeyringFile);

			// initialize PGPnet secring file name
			SetDlgItemText (hwnd, IDC_NETSECRING, ppds->net.szSecringFile);
			CLDisablePrefControl (hwnd, IDC_NETSECRING,
					ppds, prefrefNet, kPGPNetPrefPrivateKeyringFile);
			CLDisablePrefControl (hwnd, IDC_NETSECRINGBROWSE,
					ppds, prefrefNet, kPGPNetPrefPrivateKeyringFile);

			// initialize "set to current" button
			b = CLDisablePrefControl (hwnd, IDC_NETSETTOCURRENT,
					ppds, prefrefNet, kPGPNetPrefPublicKeyringFile);
			if (!b)
			{
				CLDisablePrefControl (hwnd, IDC_NETSETTOCURRENT,
						ppds, prefrefNet, kPGPNetPrefPrivateKeyringFile);
			}
		}
		else
		{
			EnableWindow (GetDlgItem (hwnd, IDC_NETPUBRING), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_NETPUBRINGBROWSE), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_NETSECRING), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_NETSECRINGBROWSE), FALSE);

			ShowWindow (GetDlgItem (hwnd, IDC_NETPUBRING), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_NETPUBRINGBROWSE), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_NETSECRING), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_NETSECRINGBROWSE), SW_HIDE);

			ShowWindow (GetDlgItem (hwnd, IDC_NETKEYRINGGROUP), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_NETKEYRINGTEXT1), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_NETKEYRINGTEXT2), SW_HIDE);

			ShowWindow (GetDlgItem (hwnd, IDC_NETKEYRINGICON), SW_HIDE);

			EnableWindow (GetDlgItem (hwnd, IDC_NETSETTOCURRENT), FALSE);
			ShowWindow (GetDlgItem (hwnd, IDC_NETSETTOCURRENT), SW_HIDE);
		}

		// initialize PGP randseed file name
		SetDlgItemText (hwnd, IDC_RNGSEEDFILE, ppds->szRandseedFile);
		CLDisablePrefControl (hwnd, IDC_RNGSEEDFILE,
				ppds, prefref, kPGPPrefRNGSeedFile);
		CLDisablePrefControl (hwnd, IDC_RNGSEEDFILEBROWSE,
				ppds, prefref, kPGPPrefRNGSeedFile);

		ppds->file.szInitDir[0] = '\0';

		return TRUE;
	}

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aFileIds);

	case WM_COMMAND :
	{
		CHAR		szFile[MAX_PATH];
		PGPPrefRef	prefrefNet;

		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		switch (LOWORD (wParam)) {
		case IDC_PUBRINGBROWSE :
			sBrowseForKeyringFile (hwnd, ppds, IDC_PUBRING, 
					IDS_PUBKEYRINGFILTER, IDS_PUBKEYCAPTION, 
					IDS_DEFPUBRINGEXT, FALSE, szFile, sizeof(szFile));
			break;

		case IDC_SECRINGBROWSE :
			sBrowseForKeyringFile (hwnd, ppds, IDC_SECRING, 
					IDS_PRIVKEYRINGFILTER, IDS_PRIVKEYCAPTION, 
					IDS_DEFPRIVRINGEXT, FALSE, szFile, sizeof(szFile));
			break;

		case IDC_NETPUBRINGBROWSE :
			sBrowseForKeyringFile (hwnd, ppds, IDC_NETPUBRING, 
					IDS_PUBKEYRINGFILTER, IDS_PUBKEYCAPTION, 
					IDS_DEFPUBRINGEXT, FALSE, szFile, sizeof(szFile));
			break;

		case IDC_NETSECRINGBROWSE :
			sBrowseForKeyringFile (hwnd, ppds, IDC_NETSECRING, 
					IDS_PRIVKEYRINGFILTER, IDS_PRIVKEYCAPTION, 
					IDS_DEFPRIVRINGEXT, FALSE, szFile, sizeof(szFile));
			break;

		case IDC_RNGSEEDFILEBROWSE :
			sBrowseForKeyringFile (hwnd, ppds, IDC_RNGSEEDFILE, 
					IDS_SEEDFILEFILTER, IDS_SEEDFILECAPTION, 
					IDS_DEFSEEDFILEEXT, TRUE, szFile, sizeof(szFile));
			break;

		case IDC_NETSETTOCURRENT :
			GetDlgItemText (hwnd, IDC_PUBRING, szFile, sizeof(szFile));
			SetDlgItemText (hwnd, IDC_NETPUBRING, szFile);
			GetDlgItemText (hwnd, IDC_SECRING, szFile, sizeof(szFile));
			SetDlgItemText (hwnd, IDC_NETSECRING, szFile);
			break;
		}

		if (ppds->net.bVPNPrefsEnabled)
		{
			prefrefNet = CLGetPrefRefNet (ppds);
			if (PGPPrefRefIsValid (prefrefNet))
				sCompareNetToPGPKeyringFiles (hwnd);
		}
		break;
	}

	case WM_NOTIFY :
		switch (((NMHDR FAR *) lParam)->code) {

		case PSN_HELP :
			PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0,
					(char*)kPGPclHelpFile, (DWORD*)IDH_PGPCLPREF_FILEDIALOG);
			break;

		case PSN_APPLY :
		{
			PGPError		err					= kPGPError_NoErr;
			CHAR			szFile[MAX_PATH];
			CHAR			szPub[MAX_PATH];
			CHAR			szPriv[MAX_PATH];
			CHAR			szPubNet[MAX_PATH];
			CHAR			szPrivNet[MAX_PATH];
			CHAR			szPref[MAX_PATH];
			HANDLE			hfile;

			ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			if (ppds->bUseClientLibPrefs && 
				!PGPPrefRefIsValid (ppds->prefref))
			{
				PGPclPeekClientLibPrefRefs (&ppds->prefref, NULL);
			}

			if (ppds->bUseClientLibNetPrefs && 
				!PGPPrefRefIsValid (ppds->prefrefNet))
			{
				PGPclPeekClientLibPrefRefs (NULL, &ppds->prefrefNet);
			}

			// save the pre-existing file names
			PGPclGetPath (kPGPclPublicKeyringFile, 
					szPub, sizeof(szPub));
			PGPclGetPath (kPGPclPrivateKeyringFile, 
					szPriv, sizeof(szPriv));
			PGPclGetPath (kPGPclNetPublicKeyringFile, 
					szPubNet, sizeof(szPubNet));
			PGPclGetPath (kPGPclNetPrivateKeyringFile, 
					szPrivNet, sizeof(szPrivNet));

			// PGP pubring file
			GetDlgItemText (hwnd, IDC_PUBRING, szFile, sizeof(szFile));
			sQueryCopyNewKeyringFile (ppds, szFile, 
					kPGPclPublicKeyringFile, szPub, szPriv);

			// PGP secring file
			GetDlgItemText (hwnd, IDC_SECRING, szFile, sizeof(szFile));
			sQueryCopyNewKeyringFile (ppds, szFile, 
					kPGPclPrivateKeyringFile, szPub, szPriv);

			// RNG seed file
			// compare with selected file and set pref if different
			GetDlgItemText (hwnd, IDC_RNGSEEDFILE, szFile, sizeof(szFile));

			// create file if it doesn't exist
			hfile = CreateFile (szFile, GENERIC_READ|GENERIC_WRITE,
					0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL);
			CloseHandle (hfile);

			PGPclGetPath (kPGPclRandseedFile, szPref, sizeof(szPref));
			if (lstrcmpi (szFile, szPref) != 0) 
			{
				PFLFileSpecRef	filespec		= kInvalidPFLFileSpecRef;
				CHAR			szPath[MAX_PATH];

				PGPclSetPath (ppds->prefref, 
						kPGPclRandseedFile, szFile);

				err = PFLNewFileSpecFromFullPath (
						PGPPeekContextMemoryMgr (g_context), 
						szPath, &filespec);
				if (IsntPGPError (err))
					PGPSetRandSeedFile (filespec);
				if (PFLFileSpecRefIsValid (filespec))
					PFLFreeFileSpec (filespec);
			}

			if (ppds->net.bVPNPrefsEnabled &&
				PGPPrefRefIsValid (ppds->prefrefNet))
			{
				// PGPnet pubring file
				// compare with selected file and set pref if different
				GetDlgItemText (hwnd, IDC_NETPUBRING, szFile, sizeof(szFile));
				sQueryCopyNewKeyringFile (ppds, szFile, 
						kPGPclNetPublicKeyringFile, szPubNet, szPrivNet);

				// PGPnet secring file
				// compare with selected file and set pref if different
				GetDlgItemText (hwnd, IDC_NETSECRING, szFile, sizeof(szFile));
				sQueryCopyNewKeyringFile (ppds, szFile, 
						kPGPclNetPrivateKeyringFile, szPubNet, szPrivNet);
			}

			if (ppds->bKeyringModified) 
			{
				// reset warn flag
				PGPSetPrefBoolean (ppds->prefref,
					kPGPPrefWarnOnReadOnlyKeyRings, (PGPBoolean)TRUE);
			}

			ppds->returnErr = kPGPError_NoErr;
            SetWindowLong (hwnd, DWL_MSGRESULT, PSNRET_NOERROR);

			return TRUE;
		}

		case PSN_KILLACTIVE :
		{
			PGPPrefRef	prefrefNet;

			ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			prefrefNet = CLGetPrefRefNet (ppds);

			if (!sCheckFileName (hwnd, IDC_PUBRING))
			{
				SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
				return TRUE;
			}
			if (!sCheckFileName (hwnd, IDC_SECRING))
			{
				SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
				return TRUE;
			}

			if (ppds->net.bVPNPrefsEnabled &&
				PGPPrefRefIsValid (prefrefNet))
			{
				if (!sCheckFileName (hwnd, IDC_NETPUBRING))
				{
					SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
					return TRUE;
				}
				if (!sCheckFileName (hwnd, IDC_NETSECRING))
				{
					SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
					return TRUE;
				}
			}

			if (!sCheckFileName (hwnd, IDC_RNGSEEDFILE))
			{
				SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
				return TRUE;
			}

			if (ppds->file.bMediaNotInserted)
				ppds->bKeyringModified = TRUE;

			// store the strings
			GetDlgItemText (hwnd, IDC_PUBRING, 
					ppds->szPubringFile, sizeof(ppds->szPubringFile));
			GetDlgItemText (hwnd, IDC_SECRING, 
					ppds->szSecringFile, sizeof(ppds->szSecringFile));
			GetDlgItemText (hwnd, IDC_NETPUBRING, 
					ppds->net.szPubringFile, sizeof(ppds->net.szPubringFile));
			GetDlgItemText (hwnd, IDC_NETSECRING, 
					ppds->net.szSecringFile, sizeof(ppds->net.szSecringFile));
			GetDlgItemText (hwnd, IDC_RNGSEEDFILE, 
					ppds->szRandseedFile, sizeof(ppds->szRandseedFile));

			SetWindowLong (hwnd, DWL_MSGRESULT, FALSE);
			return TRUE;
		}
		}

	}
	return FALSE;
}

