/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	CLprefsHotkeys.c - handle PGP hotkeys preferences dialogs

	$Id: CLprefsHotkeys.c,v 1.15 2002/11/12 18:55:25 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "CLprefs.h"
#include "pgpHotKeys.h"

// external globals
extern HINSTANCE	g_hInst;


static DWORD aHotkeyIds[] = {			// Help IDs
	IDC_ENABLEPURGECACHE,	IDH_PGPCLPREF_ENABLEPURGEHOTKEY,
	IDC_PURGEHOTKEY,		IDH_PGPCLPREF_PURGEHOTKEY,
	IDC_ENABLEENCRYPT,		IDH_PGPCLPREF_ENABLEENCRYPTHOTKEY,
	IDC_ENCRYPTHOTKEY,		IDH_PGPCLPREF_ENCRYPTHOTKEY,
	IDC_ENABLESIGN,			IDH_PGPCLPREF_ENABLESIGNHOTKEY,
	IDC_SIGNHOTKEY,			IDH_PGPCLPREF_SIGNHOTKEY,
	IDC_ENABLEENCRYPTSIGN,	IDH_PGPCLPREF_ENABLEESHOTKEY,
	IDC_ENCRYPTSIGNHOTKEY,	IDH_PGPCLPREF_ENCRYPTSIGNHOTKEY,
	IDC_ENABLEDECRYPT,		IDH_PGPCLPREF_ENABLEDECRYPTHOTKEY,
	IDC_DECRYPTHOTKEY,		IDH_PGPCLPREF_DECRYPTHOTKEY,
	IDC_ENABLEUNMOUNT,		IDH_PGPCLPREF_ENABLEUNMOUNTHOTKEY,
	IDC_UNMOUNTHOTKEY,		IDH_PGPCLPREF_UNMOUNTHOTKEY,
	0,0
};


//	_____________________________________________________
//
//  Message processing function for property sheet dialog

static VOID
sSetHotKeyControl (
		HWND		hwnd,
		PGPUInt32	u)
{
	UINT	uKey	= LOWORD (u);
	UINT	uMod	= 0;

	if (u & kPGPHotKeyAltModifier)
		uMod |= HOTKEYF_ALT;

	if (u & kPGPHotKeyCtrlModifier)
		uMod |= HOTKEYF_CONTROL;

	if (u & kPGPHotKeyExtModifier)
		uMod |= HOTKEYF_EXT;

	if (u & kPGPHotKeyShiftModifier)
		uMod |= HOTKEYF_SHIFT;

	SendMessage (hwnd, HKM_SETHOTKEY, MAKEWORD (LOBYTE (uKey), uMod), 0);

	if (u & kPGPHotKeyEnabled)
		EnableWindow (hwnd, TRUE);
	else
		EnableWindow (hwnd, FALSE);
}


//	_____________________________________________________
//
//  Message processing function for property sheet dialog

static PGPUInt32
sGetHotKeyControl (
		HWND		hwnd)
{
	UINT	uKey;
	WORD	wMod;

	uKey = SendMessage (hwnd, HKM_GETHOTKEY, 0, 0);

	wMod = HIBYTE (LOWORD (uKey));
	uKey &= 0xFF;

	if (wMod & HOTKEYF_ALT)
		uKey |= kPGPHotKeyAltModifier;

	if (wMod & HOTKEYF_CONTROL)
		uKey |= kPGPHotKeyCtrlModifier;

	if (wMod & HOTKEYF_EXT)
		uKey |= kPGPHotKeyExtModifier;

	if (wMod & HOTKEYF_SHIFT)
		uKey |= kPGPHotKeyShiftModifier;

	return uKey;
}


//	_____________________________________________________
//
//  Message processing function for property sheet dialog

static BOOL
sDoHotKeysOverlap (
		PGPHotKeys* photkeys)
{
	PGPUInt32* pStart	= &(photkeys->hotkeyPurgePassphraseCache);
	PGPUInt32* pEnd		= &(photkeys->hotkeyDiskUnmount);
	PGPUInt32* p;

	do
	{
		p = pStart+1;
		while (p <= pEnd)
		{
			if ((*p & kPGPHotKeyEnabled) &&
				(*p == *pStart))
				return TRUE;
			p++;
		}
		pStart++;
	}
	while (pStart < pEnd);

	return FALSE;
}


//	_____________________________________________________
//
//  determine if hotkey is already in use -- this works because
//	we have already disabled all PGP hotkeys (except PGPdisk)

static BOOL
sIsHotKeyInUse (
		PGPUInt32 uHotKey)
{
	UINT	uMod		= 0;
	BOOL	bInUse		= TRUE;

	if (uHotKey & kPGPHotKeyAltModifier)
		uMod |= MOD_ALT;
	if (uHotKey & kPGPHotKeyCtrlModifier)
		uMod |= MOD_CONTROL;
	if (uHotKey & kPGPHotKeyShiftModifier)
		uMod |= MOD_SHIFT;

	if (RegisterHotKey (NULL, 0xC000, uMod, (uHotKey & 0xFF)))
	{
		UnregisterHotKey (NULL, 0xC000);
		bInUse = FALSE;
	}

	return bInUse;
}


//	_____________________________________________________
//
//  Message processing function for property sheet dialog

LRESULT WINAPI
CLHotkeyPropDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PPREFSDLGSTRUCT			ppds;

	switch (uMsg) {

	case WM_INITDIALOG:
	{
		PGPHotKeys*		phks		= NULL;
		PGPSize			size;
		PGPHotKeys		hotkeys;
		PGPError		err;
		PGPPrefRef		prefref;

		SetWindowLong (hwnd, GWL_USERDATA, ((PROPSHEETPAGE*)lParam)->lParam);
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		if (ppds->bNeedsCentering) 
		{
			PGPclCenterWindowOnMonitor (GetParent (hwnd), kPGPclWindowMonitor);
			ppds->bNeedsCentering = FALSE;
		}

		prefref = CLGetPrefRef (ppds);

		pgpCopyMemory (&sDefaultHotKeys, &hotkeys, sizeof(hotkeys));
		err = PGPGetPrefData (prefref, kPGPPrefHotKeyData, &size, &phks);

		if ((IsntPGPError (err)) &&
			(size == sizeof (hotkeys)))
				pgpCopyMemory (phks, &hotkeys, size);

		// initialize purge passphrase hotkey
		sSetHotKeyControl (GetDlgItem (hwnd, IDC_PURGEHOTKEY),
				hotkeys.hotkeyPurgePassphraseCache);

		if (hotkeys.hotkeyPurgePassphraseCache & kPGPHotKeyEnabled)
			CheckDlgButton (hwnd, IDC_ENABLEPURGECACHE, BST_CHECKED);
		else
			CheckDlgButton (hwnd, IDC_ENABLEPURGECACHE, BST_UNCHECKED);

		if (ppds->bToolsPrefsEnabled)
		{
			// initialize encrypt hotkey
			sSetHotKeyControl (GetDlgItem (hwnd, IDC_ENCRYPTHOTKEY),
					hotkeys.hotkeyEncrypt);

			if (hotkeys.hotkeyEncrypt & kPGPHotKeyEnabled)
				CheckDlgButton (hwnd, IDC_ENABLEENCRYPT, BST_CHECKED);
			else
				CheckDlgButton (hwnd, IDC_ENABLEENCRYPT, BST_UNCHECKED);

			// initialize sign hotkey
			sSetHotKeyControl (GetDlgItem (hwnd, IDC_SIGNHOTKEY),
					hotkeys.hotkeySign);

			if (hotkeys.hotkeySign & kPGPHotKeyEnabled)
				CheckDlgButton (hwnd, IDC_ENABLESIGN, BST_CHECKED);
			else
				CheckDlgButton (hwnd, IDC_ENABLESIGN, BST_UNCHECKED);

			// initialize encrypt & sign hotkey
			sSetHotKeyControl (GetDlgItem (hwnd, IDC_ENCRYPTSIGNHOTKEY),
					hotkeys.hotkeyEncryptSign);

			if (hotkeys.hotkeyEncryptSign & kPGPHotKeyEnabled)
				CheckDlgButton (hwnd, IDC_ENABLEENCRYPTSIGN, BST_CHECKED);
			else
				CheckDlgButton (hwnd, IDC_ENABLEENCRYPTSIGN, BST_UNCHECKED);

			// initialize decrypt & verify hotkey
			sSetHotKeyControl (GetDlgItem (hwnd, IDC_DECRYPTHOTKEY),
					hotkeys.hotkeyDecrypt);

			if (hotkeys.hotkeyDecrypt & kPGPHotKeyEnabled)
				CheckDlgButton (hwnd, IDC_ENABLEDECRYPT, BST_CHECKED);
			else
				CheckDlgButton (hwnd, IDC_ENABLEDECRYPT, BST_UNCHECKED);
		}
		else
		{
			CheckDlgButton (hwnd, IDC_ENABLEENCRYPT, BST_UNCHECKED);
			CheckDlgButton (hwnd, IDC_ENABLESIGN, BST_UNCHECKED);
			CheckDlgButton (hwnd, IDC_ENABLEENCRYPTSIGN, BST_UNCHECKED);
			CheckDlgButton (hwnd, IDC_ENABLEDECRYPT, BST_UNCHECKED);

			EnableWindow (GetDlgItem (hwnd, IDC_ENABLEENCRYPT), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_ENABLESIGN), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_ENABLEENCRYPTSIGN), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_ENABLEDECRYPT), FALSE);

			EnableWindow (GetDlgItem (hwnd, IDC_ENCRYPTHOTKEY), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_SIGNHOTKEY), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_ENCRYPTSIGNHOTKEY), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_DECRYPTHOTKEY), FALSE);

			ShowWindow (GetDlgItem (hwnd, IDC_ENABLEENCRYPT), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_ENABLESIGN), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_ENABLEENCRYPTSIGN), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_ENABLEDECRYPT), SW_HIDE);

			ShowWindow (GetDlgItem (hwnd, IDC_ENCRYPTHOTKEY), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_SIGNHOTKEY), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_ENCRYPTSIGNHOTKEY), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_DECRYPTHOTKEY), SW_HIDE);
		}

		// initialize unmount hotkey
		if(PGPlnIsAuthorized() && PGPlnPGPdiskEnabled ())
		{
			if (ppds->disk.bPrefsEnabled)
			{
				sSetHotKeyControl (GetDlgItem (hwnd, IDC_UNMOUNTHOTKEY),
						hotkeys.hotkeyDiskUnmount);

				if (hotkeys.hotkeyDiskUnmount & kPGPHotKeyEnabled)
					CheckDlgButton (hwnd, IDC_ENABLEUNMOUNT, BST_CHECKED);
				else
					CheckDlgButton (hwnd, IDC_ENABLEUNMOUNT, BST_UNCHECKED);
			}
			else
			{
				CheckDlgButton (hwnd, IDC_ENABLEUNMOUNT, BST_UNCHECKED);

				EnableWindow (GetDlgItem (hwnd, IDC_ENABLEUNMOUNT), FALSE);
				EnableWindow (GetDlgItem (hwnd, IDC_UNMOUNTHOTKEY), FALSE);

				ShowWindow (GetDlgItem (hwnd, IDC_ENABLEUNMOUNT), SW_HIDE);
				ShowWindow (GetDlgItem (hwnd, IDC_UNMOUNTHOTKEY), SW_HIDE);
			}
		}
		else
		{
			CheckDlgButton (hwnd, IDC_ENABLEUNMOUNT, BST_UNCHECKED);

			EnableWindow (GetDlgItem (hwnd, IDC_ENABLEUNMOUNT), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_UNMOUNTHOTKEY), FALSE);

			ShowWindow (GetDlgItem (hwnd, IDC_ENABLEUNMOUNT), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_UNMOUNTHOTKEY), SW_HIDE);
		}
		// end screwy #if code

		if (IsntNull (phks))
			PGPFreeData (phks);

		if (CLDisablePrefControl (hwnd, IDC_ENABLEPURGECACHE, 
				ppds, prefref, kPGPPrefHotKeyData))
		{
			EnableWindow (GetDlgItem (hwnd, IDC_ENABLEPURGECACHE), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_ENABLEENCRYPT), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_ENABLESIGN), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_ENABLEENCRYPTSIGN), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_ENABLEDECRYPT), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_ENABLEUNMOUNT), FALSE);

			EnableWindow (GetDlgItem (hwnd, IDC_PURGEHOTKEY), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_ENCRYPTHOTKEY), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_SIGNHOTKEY), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_ENCRYPTSIGNHOTKEY), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_DECRYPTHOTKEY), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_UNMOUNTHOTKEY), FALSE);
		}

		return TRUE;
	}

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aHotkeyIds);

	case WM_COMMAND :
		switch (LOWORD (wParam)) {
		case IDC_ENABLEPURGECACHE :
			EnableWindow (GetDlgItem (hwnd, IDC_PURGEHOTKEY),
				(IsDlgButtonChecked (
						hwnd, IDC_ENABLEPURGECACHE) == BST_CHECKED));
			break;

		case IDC_ENABLEENCRYPT :
			EnableWindow (GetDlgItem (hwnd, IDC_ENCRYPTHOTKEY),
				(IsDlgButtonChecked (
						hwnd, IDC_ENABLEENCRYPT) == BST_CHECKED));
			break;

		case IDC_ENABLESIGN :
			EnableWindow (GetDlgItem (hwnd, IDC_SIGNHOTKEY),
				(IsDlgButtonChecked (
						hwnd, IDC_ENABLESIGN) == BST_CHECKED));
			break;

		case IDC_ENABLEENCRYPTSIGN :
			EnableWindow (GetDlgItem (hwnd, IDC_ENCRYPTSIGNHOTKEY),
				(IsDlgButtonChecked (
						hwnd, IDC_ENABLEENCRYPTSIGN) == BST_CHECKED));
			break;

		case IDC_ENABLEDECRYPT :
			EnableWindow (GetDlgItem (hwnd, IDC_DECRYPTHOTKEY),
				(IsDlgButtonChecked (
						hwnd, IDC_ENABLEDECRYPT) == BST_CHECKED));

		case IDC_ENABLEUNMOUNT :
			EnableWindow (GetDlgItem (hwnd, IDC_UNMOUNTHOTKEY),
				(IsDlgButtonChecked (
						hwnd, IDC_ENABLEUNMOUNT) == BST_CHECKED));
			break;
		}
		break;

	case WM_NOTIFY :
		switch (((NMHDR FAR *) lParam)->code) {
		case PSN_SETACTIVE :
			ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			if (ppds->hk.hwndFocus)
			{
				SetFocus (ppds->hk.hwndFocus);
				ppds->hk.hwndFocus = NULL;
			}
			break;

		case PSN_HELP :
			PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0,
					(char*)kPGPclHelpFile, (DWORD*)IDH_PGPCLPREF_HOTKEYDIALOG);
			break;

		case PSN_APPLY :
		{
			PGPHotKeys	hotkeys;

			ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			if (ppds->bUseClientLibPrefs && 
				!PGPPrefRefIsValid (ppds->prefref))
			{
				PGPclPeekClientLibPrefRefs (&ppds->prefref, NULL);
			}

			// save purge passphrase hotkey
			hotkeys.hotkeyPurgePassphraseCache =
				sGetHotKeyControl (GetDlgItem (hwnd, IDC_PURGEHOTKEY));

			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLEPURGECACHE) == BST_CHECKED)
			{
				hotkeys.hotkeyPurgePassphraseCache |= kPGPHotKeyEnabled;
			}

			// save encrypt hotkey
			hotkeys.hotkeyEncrypt =
				sGetHotKeyControl (GetDlgItem (hwnd, IDC_ENCRYPTHOTKEY));

			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLEENCRYPT) == BST_CHECKED)
			{
				hotkeys.hotkeyEncrypt |= kPGPHotKeyEnabled;
			}

			// save sign hotkey
			hotkeys.hotkeySign =
				sGetHotKeyControl (GetDlgItem (hwnd, IDC_SIGNHOTKEY));

			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLESIGN) == BST_CHECKED)
			{
				hotkeys.hotkeySign |= kPGPHotKeyEnabled;
			}

			// save encrypt & sign hotkey
			hotkeys.hotkeyEncryptSign =
				sGetHotKeyControl (GetDlgItem (hwnd, IDC_ENCRYPTSIGNHOTKEY));

			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLEENCRYPTSIGN) == BST_CHECKED)
			{
				hotkeys.hotkeyEncryptSign |= kPGPHotKeyEnabled;
			}

			// save decrypt hotkey
			hotkeys.hotkeyDecrypt =
				sGetHotKeyControl (GetDlgItem (hwnd, IDC_DECRYPTHOTKEY));

			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLEDECRYPT) == BST_CHECKED)
			{
				hotkeys.hotkeyDecrypt |= kPGPHotKeyEnabled;
			}

			// save unmount hotkey
			hotkeys.hotkeyDiskUnmount =
				sGetHotKeyControl (GetDlgItem (hwnd, IDC_UNMOUNTHOTKEY));

			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLEUNMOUNT) == BST_CHECKED)
			{
				hotkeys.hotkeyDiskUnmount |= kPGPHotKeyEnabled;
			}

			// write prefs
			PGPclErrorBox (hwnd, PGPSetPrefData (ppds->prefref,
						kPGPPrefHotKeyData, sizeof(hotkeys), &hotkeys));

			ppds->bPrefsModified = TRUE;
			ppds->disk.bPrefsModified = TRUE;
			ppds->returnErr = kPGPError_NoErr;
            SetWindowLong (hwnd, DWL_MSGRESULT, PSNRET_NOERROR);
            return TRUE;
		}

		case PSN_KILLACTIVE :
		{
			PGPHotKeys	hotkeys;

			ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			// get purge passphrase hotkey
			hotkeys.hotkeyPurgePassphraseCache =
				sGetHotKeyControl (GetDlgItem (hwnd, IDC_PURGEHOTKEY));

			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLEPURGECACHE) == BST_CHECKED)
			{
				if (sIsHotKeyInUse (hotkeys.hotkeyPurgePassphraseCache))
				{
					PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
							IDS_PURGEHOTKEYINUSE, MB_ICONEXCLAMATION|MB_OK);
					SetFocus (GetDlgItem (hwnd, IDC_PURGEHOTKEY));
					SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
					return TRUE;
				}
				hotkeys.hotkeyPurgePassphraseCache |= kPGPHotKeyEnabled;
			}

			// get encrypt hotkey
			hotkeys.hotkeyEncrypt =
				sGetHotKeyControl (GetDlgItem (hwnd, IDC_ENCRYPTHOTKEY));

			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLEENCRYPT) == BST_CHECKED)
			{
				if (sIsHotKeyInUse (hotkeys.hotkeyEncrypt))
				{
					PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
							IDS_ENCRYPTHOTKEYINUSE, MB_ICONEXCLAMATION|MB_OK);
					ppds->hk.hwndFocus = 
							GetDlgItem (hwnd, IDC_ENCRYPTHOTKEY);
					SetFocus (GetDlgItem (hwnd, IDC_ENCRYPTHOTKEY));
					SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
					return TRUE;
				}
				hotkeys.hotkeyEncrypt |= kPGPHotKeyEnabled;
			}

			// get sign hotkey
			hotkeys.hotkeySign =
				sGetHotKeyControl (GetDlgItem (hwnd, IDC_SIGNHOTKEY));

			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLESIGN) == BST_CHECKED)
			{
				if (sIsHotKeyInUse (hotkeys.hotkeySign))
				{
					PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
							IDS_SIGNHOTKEYINUSE, MB_ICONEXCLAMATION|MB_OK);
					SetFocus (GetDlgItem (hwnd, IDC_SIGNHOTKEY));
					SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
					return TRUE;
				}
				hotkeys.hotkeySign |= kPGPHotKeyEnabled;
			}

			// get encrypt & sign hotkey
			hotkeys.hotkeyEncryptSign =
				sGetHotKeyControl (GetDlgItem (hwnd, IDC_ENCRYPTSIGNHOTKEY));

			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLEENCRYPTSIGN) == BST_CHECKED)
			{
				if (sIsHotKeyInUse (hotkeys.hotkeyEncryptSign))
				{
					PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
							IDS_ENCRYPTSIGNHOTKEYINUSE, MB_ICONEXCLAMATION|MB_OK);
					SetFocus (GetDlgItem (hwnd, IDC_ENCRYPTSIGNHOTKEY));
					SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
					return TRUE;
				}
				hotkeys.hotkeyEncryptSign |= kPGPHotKeyEnabled;
			}

			// get decrypt hotkey
			hotkeys.hotkeyDecrypt =
				sGetHotKeyControl (GetDlgItem (hwnd, IDC_DECRYPTHOTKEY));

			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLEDECRYPT) == BST_CHECKED)
			{
				if (sIsHotKeyInUse (hotkeys.hotkeyDecrypt))
				{
					PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
							IDS_DECRYPTHOTKEYINUSE, MB_ICONEXCLAMATION|MB_OK);
					SetFocus (GetDlgItem (hwnd, IDC_DECRYPTHOTKEY));
					SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
					return TRUE;
				}
				hotkeys.hotkeyDecrypt |= kPGPHotKeyEnabled;
			}

			// get unmount hotkey
			hotkeys.hotkeyDiskUnmount =
				sGetHotKeyControl (GetDlgItem (hwnd, IDC_UNMOUNTHOTKEY));

			if (IsDlgButtonChecked (
					hwnd, IDC_ENABLEUNMOUNT) == BST_CHECKED)
			{
				if (sIsHotKeyInUse (hotkeys.hotkeyDiskUnmount))
				{
					PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
							IDS_UNMOUNTHOTKEYINUSE, MB_ICONEXCLAMATION|MB_OK);
					SetFocus (GetDlgItem (hwnd, IDC_UNMOUNTHOTKEY));
					SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
					return TRUE;
				}
				hotkeys.hotkeyDiskUnmount |= kPGPHotKeyEnabled;
			}

			// check for overlapping hotkeys
			if (sDoHotKeysOverlap (&hotkeys))
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT, 
						IDS_OVERLAPPINGHOTKEYS, MB_ICONEXCLAMATION|MB_OK);
				SetWindowLong (hwnd, DWL_MSGRESULT, TRUE);
				return TRUE;
			}

			// allow closure
            SetWindowLong (hwnd, DWL_MSGRESULT, FALSE);
            return TRUE;
		}
		}
	}

	return FALSE;

}

