/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	CLprefsKeyservers.c - handle PGP keyserver preferences dialogs

	$Id: CLprefsKeyservers.c,v 1.24 2002/10/29 18:30:41 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "CLprefs.h"
#include "UTF8Edit.h"

// constant definitions
#define TREEOFFSETLEFT		8
#define BUTTONOFFSETX		10
#define BUTTONOFFSETY		10

#define DOMAINLISTWIDTH		53
#define CHECKLISTWIDTH		48

#define DELAYTIMER			543

#define PGPKeyserverHTTP	0
#define PGPKeyserverLDAP	1
#define PGPKeyserverLDAPS	2
//#define PGPLDAP			3
//#define PGPLDAPS			4
#define X509LDAP			3
#define X509LDAPS			4

// typedefs
typedef struct {
	PPREFSDLGSTRUCT			ppds;
	HWND					hwndTree;
	BOOL					bEdit;
	CHAR					szServer[256];
	PGPUInt32				iType;
	PGPUInt16				iPort;
	CHAR					szKeyStoreDN[kMaxKeyStoreDNLength];
	PGPKeyServerProtocol	protocol;
	CHAR					szKeyIDAuth[kPGPMaxKeyIDStringSize];
	PGPPublicKeyAlgorithm	authAlg;
	CHAR					szUserIDAuth[kPGPMaxUserIDSize +1];
	CHAR					szDomain[256];
	BOOL					bListed;
	BOOL					bRoot;
	VOID*					pOrigStruct;
} KEYSERVERSTRUCT, *PKEYSERVERSTRUCT;


// external globals
extern HINSTANCE	g_hInst;
extern HIMAGELIST	g_hilClientLib;


static DWORD aServerIds[] = {			// Help IDs
	IDC_NEWKEYSERVER,		IDH_PGPCLPREF_CREATESERVER,
	IDC_REMOVEKEYSERVER,	IDH_PGPCLPREF_REMOVESERVER,
	IDC_EDITKEYSERVER,		IDH_PGPCLPREF_EDITSERVER,
	IDC_SETASROOT,			IDH_PGPCLPREF_SETROOTSERVER,
	IDC_MOVEUP,				IDH_PGPCLPREF_MOVESERVERUP,
	IDC_MOVEDOWN,			IDH_PGPCLPREF_MOVESERVERDOWN,
	IDC_SYNCONUNKNOWN,		IDH_PGPCLPREF_SYNCONUNKNOWNKEYS,
	IDC_SYNCONADDNAME,		IDH_PGPCLPREF_SYNCONADDNAME,
	IDC_SYNCONKEYSIGN,		IDH_PGPCLPREF_SYNCONSIGN,
	IDC_SYNCONREVOKE,		IDH_PGPCLPREF_SYNCONREVOKE,
	IDC_SYNCONVERIFY,		IDH_PGPCLPREF_SYNCONVERIFY,
	IDC_KSTREELIST,			IDH_PGPCLPREF_SERVERLIST,
	IDC_SYNCONSTATICTEXT,	-1,
	0,0
};

static DWORD aEditServerIds[] = {		// Help IDs
	IDC_TYPE,				IDH_PGPCLPREF_SERVERTYPE,
    IDC_SERVERNAME,			IDH_PGPCLPREF_SERVERNAME, 
	IDC_PORT,				IDH_PGPCLPREF_SERVERPORT,
	IDC_SERVERBASEDN,		IDH_PGPCLPREF_SERVERBASEDN,
	IDC_SERVERKEY,			IDH_PGPCLPREF_SERVERAUTHKEY,
	IDC_ANYDOMAIN,			IDH_PGPCLPREF_SERVERNODOMAIN,
    IDC_DOMAIN,				IDH_PGPCLPREF_SERVERDOMAIN, 
    IDC_SPECIFIEDDOMAIN,	IDH_PGPCLPREF_SERVERDOMAIN, 
    IDC_LISTED,				IDH_PGPCLPREF_SERVERLISTED, 
    0,0 
}; 


//	_____________________________________________________
//
//  concatenate protocol and server names

static VOID
sURLfromInfo (
		PGPKeyServerProtocol	protocol, 
		LPSTR					szServer, 
		PGPUInt16				iPort, 
		LPSTR					szURL)
{
	switch (protocol) {
	case kPGPKeyServerProtocol_LDAP :	lstrcpy (szURL, "ldap://"); break;
	case kPGPKeyServerProtocol_LDAPS :	lstrcpy (szURL, "ldaps://"); break;
	case kPGPKeyServerProtocol_HTTPS :	lstrcpy (szURL, "https://"); break;
	default :							lstrcpy (szURL, "http://"); break;
	}

	lstrcat (szURL, szServer);

	if (iPort != 0) 
	{
		CHAR szPort[8];
		wsprintf (szPort, ":%i", iPort);
		lstrcat (szURL, szPort);
	}
}

//	_____________________________________________________
//
//  put data for one keyserver into treelist

static HTLITEM
sSetOneKSItem (
		HWND				hwndTree, 
		PKEYSERVERSTRUCT	pkss)
{
	TL_TREEITEM		tlI;
	TL_LISTITEM		tlL;
	TL_INSERTSTRUCT tlIns;
	HTLITEM			hTNew;
	CHAR			sz[256];

	sURLfromInfo (pkss->protocol, pkss->szServer, pkss->iPort, sz);

	tlI.hItem = NULL;
	tlI.mask = TLIF_TEXT | TLIF_IMAGE | TLIF_STATE | TLIF_PARAM;
	tlI.stateMask = TLIS_BOLD;
	if (pkss->bRoot) 
		tlI.state = TLIS_BOLD;
	else
		tlI.state = 0;
	tlI.pszText = sz;
	tlI.cchTextMax = lstrlen (sz);
	tlI.iImage = IDX_SERVER;
	tlI.iSelectedImage = IDX_SERVER;
	tlI.lParam = (LPARAM)pkss;

	tlIns.item = tlI;
	tlIns.hInsertAfter = (HTLITEM)TLI_LAST;
	tlIns.hParent = NULL;
	hTNew = TreeList_InsertItem (hwndTree, &tlIns);

	if (lstrlen (pkss->szDomain))
		tlL.pszText = pkss->szDomain;
	else 
		tlL.pszText = NULL;
	tlL.hItem = hTNew;
	tlL.stateMask = TLIS_VISIBLE;
	tlL.iSubItem = 1;
	tlL.mask = TLIF_DATAVALUE | TLIF_TEXT | TLIF_STATE;
	tlL.state = TLIS_VISIBLE;
	tlL.lDataValue = IDX_WORLDDOMAIN;

	TreeList_SetListItem (hwndTree, &tlL, FALSE);

	tlL.pszText = NULL;
	tlL.hItem = hTNew;
	tlL.stateMask = TLIS_VISIBLE;
	tlL.iSubItem = 2;
	tlL.mask = TLIF_DATAVALUE | TLIF_STATE;
	tlL.state = TLIS_VISIBLE;

	if (pkss->bListed) tlL.lDataValue = IDX_GREENDOT;
	else tlL.lDataValue = IDX_GREYDOT;

	TreeList_SetListItem (hwndTree, &tlL, FALSE);

	return hTNew;
}

//	_____________________________________________________
//
//  populate keyserver treelist with data from preferences

static INT
sFillTreeList (
		PPREFSDLGSTRUCT ppds)
{
	PGPKeyServerEntry*	keyserverList	= NULL;
	PGPKeyDBObjRef		key;
	PGPUInt32			u, u2;
	PGPError			err;
	PKEYSERVERSTRUCT	pkss;
	PGPKeyID			keyid;
	PGPPrefRef			prefref;

	// load keyserverprefs
	prefref = CLGetPrefRef (ppds);
	err = PGPGetKeyServerPrefs (prefref, 
				&keyserverList, &ppds->ks.uNumberOfServers);

	if (IsPGPError (err)) 
		ppds->ks.uNumberOfServers = 0;

	for (u=0; u<ppds->ks.uNumberOfServers; u++) 
	{
		pkss = clAlloc (sizeof(KEYSERVERSTRUCT));
		if (pkss) 
		{
			pkss->ppds		= ppds;
			pkss->hwndTree	= ppds->ks.hwndTree;
			pkss->bEdit		= FALSE;
			lstrcpy (pkss->szServer, keyserverList[u].serverDNS);
			pkss->iType		= keyserverList[u].type;
			pkss->iPort		= keyserverList[u].serverPort;
			lstrcpy (pkss->szKeyStoreDN, keyserverList[u].keystoredn);
			pkss->protocol	= keyserverList[u].protocol;
			lstrcpy (pkss->szDomain, keyserverList[u].domain);
			pkss->bListed	= (keyserverList[u].flags & kKeyServerListed);
			pkss->authAlg	= keyserverList[u].authAlg;
			pkss->bRoot		= (keyserverList[u].flags & kKeyServerIsRoot);

			// get auth key userid
			pkss->szUserIDAuth[0] = '\0';
			if ((pkss->authAlg != kPGPPublicKeyAlgorithm_Invalid) &&
					keyserverList[u].authKeyIDString[0]) 
			{
				lstrcpy (pkss->szKeyIDAuth, 
						keyserverList[u].authKeyIDString);
				err = PGPNewKeyIDFromString (
						keyserverList[u].authKeyIDString, 
						pkss->authAlg, &keyid);
				if (IsntPGPError (err)) 
				{
					err = PGPFindKeyByKeyID (ppds->keydbMain, &keyid, &key);
					if (IsntPGPError (err)) 
					{
						PGPclGetPrimaryUserIDNameUTF8 (key, 
								pkss->szUserIDAuth, 
								sizeof(pkss->szUserIDAuth), &u2);
					}
					else 
					{
						lstrcpy (pkss->szUserIDAuth, 
								keyserverList[u].authKeyIDString);
					}
				}
			}
			if (pkss->szUserIDAuth[0] == '\0') 
			{
				LoadString (g_hInst, IDS_UNKNOWNKEY, pkss->szUserIDAuth, 
						sizeof(pkss->szUserIDAuth));
			}

			sSetOneKSItem (ppds->ks.hwndTree, pkss); 
		}
	}

	if (keyserverList) 
		PGPFreeKeyServerList(keyserverList);

	return 0;
}


//	_____________________________________________________
//
//  add columns to treelist

static BOOL 
sAddColumns (
		HWND hwndTree, 
		INT iWidth) 
{
	TL_COLUMN tlc;
	CHAR sz[64];

	TreeList_DeleteAllColumns (hwndTree);

	tlc.mask = TLCF_FMT | TLCF_WIDTH | TLCF_TEXT | 
				TLCF_SUBITEM | TLCF_DATATYPE | TLCF_DATAMAX |
				TLCF_MOUSENOTIFY;
	tlc.pszText = sz;

	tlc.iSubItem = 0;
	tlc.fmt = TLCFMT_LEFT;
	tlc.iDataType = TLC_DATASTRING;
	tlc.cx = iWidth - DOMAINLISTWIDTH - CHECKLISTWIDTH - 20;
	tlc.bMouseNotify = FALSE;
	LoadString (g_hInst, IDS_SERVER, sz, sizeof(sz));
	TreeList_InsertColumn (hwndTree, 0, &tlc);

	tlc.fmt = TLCFMT_IMAGE;
	tlc.iDataType = TLC_DATALONG;
	tlc.cx = DOMAINLISTWIDTH;
	tlc.bMouseNotify = FALSE;
	LoadString (g_hInst, IDS_DOMAIN, sz, sizeof(sz));
	TreeList_InsertColumn (hwndTree, 1, &tlc);

	tlc.fmt = TLCFMT_IMAGE;
	tlc.iDataType = TLC_DATALONG;
	tlc.cx = CHECKLISTWIDTH;
	tlc.bMouseNotify = TRUE;
	LoadString (g_hInst, IDS_LISTED, sz, sizeof(sz));
	TreeList_InsertColumn (hwndTree, 2, &tlc);

	return TRUE;
}


//	_____________________________________________________
//
//  create new treelist control for keyserver prefs

static HWND
sCreateKeyserverTreeList (
		HWND			hwndParent,
		PPREFSDLGSTRUCT ppds)
{
	RECT rc;
	INT iwidth, iheight, itop;
	POINT pt;

	// Ensure that the common control DLL is loaded.
	InitCommonControls ();

	// Ensure that the custom control DLL is loaded.
	InitTreeListControl ();

	// initialize flags
	ppds->ks.iSelectedType	= IDX_NONE;
	ppds->ks.bSelectedRoot	= FALSE;
	ppds->ks.bLastInList	= TRUE;
	ppds->ks.bFirstInList	= TRUE;

	// create tree view window
	GetWindowRect (GetDlgItem (hwndParent, IDC_NEWKEYSERVER), &rc);
	pt.x = rc.left;
	pt.y = rc.top;
	ScreenToClient (hwndParent, &pt);
	itop = pt.y;
	iwidth = pt.x - BUTTONOFFSETX - TREEOFFSETLEFT;

	GetWindowRect (GetDlgItem (hwndParent, IDC_SYNCONSTATICTEXT), &rc);
	pt.x = rc.left;
	pt.y = rc.top;
	ScreenToClient (hwndParent, &pt);
	iheight = pt.y - BUTTONOFFSETY - itop;

	GetClientRect (hwndParent, &rc);

	ppds->ks.hwndTree = CreateWindowEx (WS_EX_CLIENTEDGE, WC_TREELIST, "",
			WS_VISIBLE | WS_CHILD | WS_BORDER | WS_TABSTOP | WS_GROUP |
			TLS_AUTOSCROLL | TLS_SINGLESELECT | 
			TLS_SHOWSELECTIONALWAYS | TLS_USEDEFAULTGUIFONT,
			TREEOFFSETLEFT, itop, iwidth, iheight,
			hwndParent, (HMENU)IDC_KSTREELIST, g_hInst, NULL);

	if (ppds->ks.hwndTree == NULL) 
		return NULL;

	// Associate the image list with the tree view control.
	TreeList_SetImageList (ppds->ks.hwndTree, g_hilClientLib);
	sAddColumns (ppds->ks.hwndTree, iwidth);
	sFillTreeList (ppds);

	return ppds->ks.hwndTree;
}

//	_____________________________________________________
//
//  destroy treelist control for keyserver prefs

static INT
sDestroyKeyserverTreeList (
		HWND	hwndTree) 
{
	HTLITEM		hTServer;
	TL_TREEITEM tlI;

	// free all KEYSERVERSTRUCT structures
	hTServer = TreeList_GetFirstItem (hwndTree);
	while (hTServer) 
	{
		tlI.hItem = hTServer;
		tlI.mask = TLIF_NEXTHANDLE | TLIF_PARAM;
		TreeList_GetItem (hwndTree, &tlI);

		clFree ((VOID*)(tlI.lParam));

		hTServer = tlI.hItem;
	}

	// close treelist
	SendMessage (hwndTree, WM_CLOSE, 0, 0);
	return 0;
}

//	_____________________________________________________
//
//  check if specified server is already in list

static BOOL
sIsThisServerAlreadyInList (
		PKEYSERVERSTRUCT	pkssThis)
{
	TL_TREEITEM			tlI;
	HTLITEM				hTServer;
	INT					iServerCount;
	PKEYSERVERSTRUCT	pkss;

	// count number of servers
	iServerCount = 0;
	hTServer = TreeList_GetFirstItem (pkssThis->hwndTree);
	while (hTServer) 
	{
		++iServerCount;
		tlI.hItem = hTServer;
		tlI.mask = TLIF_NEXTHANDLE;
		TreeList_GetItem (pkssThis->hwndTree, &tlI);
		hTServer = tlI.hItem;
	}

	hTServer = TreeList_GetFirstItem (pkssThis->hwndTree);
	while (hTServer) 
	{
		// get server structure
		tlI.hItem = hTServer;
		tlI.mask = TLIF_PARAM | TLIF_NEXTHANDLE;
		TreeList_GetItem (pkssThis->hwndTree, &tlI);

		pkss = (PKEYSERVERSTRUCT)(tlI.lParam);

		if (pkss != (PKEYSERVERSTRUCT)pkssThis->pOrigStruct) 
		{
			if (!lstrcmpi (pkss->szServer, pkssThis->szServer)) 
			{
				if (pkss->protocol == pkssThis->protocol) 
				{
					if (pkss->iPort == pkssThis->iPort) 
						return TRUE;
				}
			}
		}

		// step to next server
		hTServer = tlI.hItem;
	}

	return FALSE;
}

//	_____________________________________________________
//
//  find out if there are any domain-less servers among those
//  which are not selected.

static BOOL
sIsThisTheOnlyWorldServer (
		PPREFSDLGSTRUCT	ppds)
{
	HTLITEM				hTServer;
	TL_TREEITEM			tlI;
	PKEYSERVERSTRUCT	pkss;

	// is the selected server a "world" server?
	tlI.hItem = ppds->ks.hTSelected;
	tlI.mask = TLIF_PARAM;
	TreeList_GetItem (ppds->ks.hwndTree, &tlI);
	pkss = (PKEYSERVERSTRUCT)tlI.lParam;

	// if server has a domain, then it's not a "world" server.
	if (pkss->szDomain[0]) 
		return FALSE;

	// interate through tree
	hTServer = TreeList_GetFirstItem (ppds->ks.hwndTree);
	while (hTServer) 
	{
		if (hTServer != ppds->ks.hTSelected) 
		{
			// get keyserver structure
			tlI.hItem = hTServer;
			tlI.mask = TLIF_PARAM;
			TreeList_GetItem (ppds->ks.hwndTree, &tlI);

			pkss = (PKEYSERVERSTRUCT)tlI.lParam;

			// if this server doesn't have a domain, then the
			// selected server isn't the last "world" server
			if (pkss->szDomain[0] == 0)
				return FALSE;
		}

		// get next server
		tlI.hItem = hTServer;
		tlI.mask = TLIF_NEXTHANDLE;
		TreeList_GetItem (ppds->ks.hwndTree, &tlI);

		hTServer = tlI.hItem;
	}
	
	// didn't find any other "world" servers
	return TRUE;
}

//	___________________________________________
//
//  New/Edit Keyserver Dialog Message procedure

static BOOL CALLBACK 
sEditServerDlgProc (
		HWND	hDlg, 
		UINT	uMsg, 
		WPARAM	wParam,
		LPARAM	lParam) 
{
	PKEYSERVERSTRUCT	pkss;
	CHAR				sz[256];
	LPSTR				p;
	INT					iSelected;

	switch (uMsg) {

	case WM_INITDIALOG :
		SetWindowLong (hDlg, GWL_USERDATA, lParam);
		pkss = (PKEYSERVERSTRUCT)lParam;
		if (pkss->bEdit) 
		{
			LoadString (g_hInst, IDS_EDITSERVERTITLE, sz, sizeof(sz));
			SetWindowText (hDlg, sz);
		}

		UTF8EditInit (GetDlgItem (hDlg, IDC_SERVERKEY));

		//set the max length of the DN field to one less than our buffer length
		SendDlgItemMessage (hDlg, IDC_SERVERBASEDN, EM_LIMITTEXT, 
				kMaxKeyStoreDNLength-1, 0);

		// load type combo box (order is important)
		LoadString (g_hInst, IDS_HTTP, sz, sizeof(sz));
		SendDlgItemMessage (hDlg, IDC_TYPE, CB_ADDSTRING, 0, (LPARAM)sz);

		LoadString (g_hInst, IDS_LDAP, sz, sizeof(sz));
		SendDlgItemMessage (hDlg, IDC_TYPE, CB_ADDSTRING, 0, (LPARAM)sz);
		
		LoadString (g_hInst, IDS_LDAPS, sz, sizeof(sz));
		SendDlgItemMessage (hDlg, IDC_TYPE, CB_ADDSTRING, 0, (LPARAM)sz);
		
//		LoadString (g_hInst, IDS_PGPLDAP, sz, sizeof(sz));
//		SendDlgItemMessage (hDlg, IDC_TYPE, CB_ADDSTRING, 0, (LPARAM)sz);
		
//		LoadString (g_hInst, IDS_PGPLDAPS, sz, sizeof(sz));
//		SendDlgItemMessage (hDlg, IDC_TYPE, CB_ADDSTRING, 0, (LPARAM)sz);
		
		LoadString (g_hInst, IDS_X509LDAP, sz, sizeof(sz));
		SendDlgItemMessage (hDlg, IDC_TYPE, CB_ADDSTRING, 0, (LPARAM)sz);

		LoadString (g_hInst, IDS_X509LDAPS, sz, sizeof(sz));
		SendDlgItemMessage (hDlg, IDC_TYPE, CB_ADDSTRING, 0, (LPARAM)sz);
		
		// initialize protocol combo box
		iSelected = 0;
		if (pkss->iType == kPGPKeyServerClass_PGP)
		{
			if (pkss->protocol == kPGPKeyServerProtocol_HTTP)
				iSelected = PGPKeyserverHTTP;
			else if (pkss->protocol == kPGPKeyServerProtocol_LDAP)
				iSelected = PGPKeyserverLDAP;
			else if (pkss->protocol == kPGPKeyServerProtocol_LDAPS)
				iSelected = PGPKeyserverLDAPS;
		}
//		else if (pkss->iType == kPGPKeyServerClass_LDAPPGP)
//		{
//			if (pkss->protocol == kPGPKeyServerProtocol_LDAP)
//				iSelected = PGPLDAP;
//			else if (pkss->protocol == kPGPKeyServerProtocol_LDAPS)
//				iSelected = PGPLDAPS;
//		}
		else if (pkss->iType == kPGPKeyServerClass_LDAPX509)
		{
			if (pkss->protocol == kPGPKeyServerProtocol_LDAP)
				iSelected = X509LDAP;
			else if (pkss->protocol == kPGPKeyServerProtocol_LDAPS)
				iSelected = X509LDAPS;
		}
		SendDlgItemMessage (hDlg, IDC_TYPE, CB_SETCURSEL, iSelected, 0);

		// initialize server name
		SetDlgItemText (hDlg, IDC_SERVERNAME, pkss->szServer);

		// initialize port number
		if (pkss->iPort != 0)
			SetDlgItemInt (hDlg, IDC_PORT, pkss->iPort, FALSE);

		//if the type of server is anything other than pgp directory
		if ((iSelected==PGPKeyserverLDAP)||(iSelected==PGPKeyserverLDAPS)) //pkss->iType == kPGPKeyServerClass_LDAPPGP)
			SetDlgItemText (hDlg, IDC_SERVERBASEDN, pkss->szKeyStoreDN);
		else
		{
			//disable the DN edit box itself
			EnableWindow (GetDlgItem (hDlg, IDC_SERVERBASEDN), FALSE);
			EnableWindow (GetDlgItem (hDlg, IDC_SERVERBASEDNTEXT), FALSE);
		}

		// initialize authentication key name
		UTF8EditSetText (GetDlgItem (hDlg, IDC_SERVERKEY), pkss->szUserIDAuth);

		// initialize domain buttons
		if (pkss->szDomain[0]) 
		{
			CheckRadioButton (hDlg, IDC_ANYDOMAIN, IDC_SPECIFIEDDOMAIN,
								IDC_SPECIFIEDDOMAIN);
			SetDlgItemText (hDlg, IDC_DOMAIN, pkss->szDomain);
		}
		else 
		{
			CheckRadioButton (hDlg, IDC_ANYDOMAIN, IDC_SPECIFIEDDOMAIN,
								IDC_ANYDOMAIN);
			SetDlgItemText (hDlg, IDC_DOMAIN, "");
			EnableWindow (GetDlgItem (hDlg, IDC_DOMAIN), FALSE);
		}

		// initialize checkboxes
		if (pkss->bListed) 
			CheckDlgButton (hDlg, IDC_LISTED, BST_CHECKED);
		else
			CheckDlgButton (hDlg, IDC_LISTED, BST_UNCHECKED);
			
		SetFocus (GetDlgItem (hDlg, IDC_SERVERNAME));
		return TRUE;

	case WM_DESTROY :
		UTF8EditDestroy (GetDlgItem (hDlg, IDC_SERVERKEY));
		return 0;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hDlg, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aEditServerIds);

	case WM_COMMAND :
		switch(LOWORD (wParam)) {

		case IDC_ANYDOMAIN :
			EnableWindow (GetDlgItem (hDlg, IDC_DOMAIN), FALSE);
			break;

		case IDC_SPECIFIEDDOMAIN :
			pkss = (PKEYSERVERSTRUCT)GetWindowLong (hDlg, GWL_USERDATA);
			if ((pkss->bEdit) &&
				(sIsThisTheOnlyWorldServer (pkss->ppds))) 
			{
				PGPclMessageBox (hDlg, IDS_PROPCAPTION, 
					IDS_MUSTHAVEDEFAULTSERVER, MB_OK|MB_ICONEXCLAMATION);
				CheckRadioButton (hDlg, IDC_ANYDOMAIN, IDC_SPECIFIEDDOMAIN,
									IDC_ANYDOMAIN);
			}
			else 
				EnableWindow (GetDlgItem (hDlg, IDC_DOMAIN), TRUE);
			break;

		case IDC_TYPE:
			if (HIWORD (wParam) == CBN_SELCHANGE)
			{
				BOOL	bEnabled;

				//get the current server type selected
				iSelected = SendDlgItemMessage (
						hDlg, IDC_TYPE, CB_GETCURSEL, 0, 0);

				//enable/disable the DN edit box. the edit is enabled only 
				// if the server type is pgp directory ldap or ldaps
//				bEnabled = ((iSelected == PGPLDAP) || (iSelected == PGPLDAPS));
				bEnabled = ((iSelected == PGPKeyserverLDAP) || (iSelected == PGPKeyserverLDAPS));
				EnableWindow (
						GetDlgItem (hDlg, IDC_SERVERBASEDN), bEnabled);
				EnableWindow (
						GetDlgItem (hDlg, IDC_SERVERBASEDNTEXT), bEnabled);
			}
			break;

		case IDOK :
			pkss = (PKEYSERVERSTRUCT)GetWindowLong (hDlg, GWL_USERDATA);
				
			// get type
			iSelected = SendDlgItemMessage (
					hDlg, IDC_TYPE, CB_GETCURSEL, 0, 0);

			switch (iSelected) {
			case PGPKeyserverHTTP :
				pkss->iType		= kPGPKeyServerClass_PGP;
				pkss->protocol	= kPGPKeyServerProtocol_HTTP;
				break;

			case PGPKeyserverLDAP :
				pkss->iType		= kPGPKeyServerClass_PGP;
				pkss->protocol	= kPGPKeyServerProtocol_LDAP;
				break;

			case PGPKeyserverLDAPS :
				pkss->iType		= kPGPKeyServerClass_PGP;
				pkss->protocol	= kPGPKeyServerProtocol_LDAPS;
				break;

//			case PGPLDAP :
//				pkss->iType		= kPGPKeyServerClass_LDAPPGP;
//				pkss->protocol	= kPGPKeyServerProtocol_LDAP;
//				break;

//			case PGPLDAPS :
//				pkss->iType		= kPGPKeyServerClass_LDAPPGP;
//				pkss->protocol	= kPGPKeyServerProtocol_LDAPS;
//				break;

			case X509LDAP :
				pkss->iType		= kPGPKeyServerClass_LDAPX509;
				pkss->protocol	= kPGPKeyServerProtocol_LDAP;
				break;

			case X509LDAPS :
				pkss->iType		= kPGPKeyServerClass_LDAPX509;
				pkss->protocol	= kPGPKeyServerProtocol_LDAPS;
				break;
			}

			// get server name
			if (!GetDlgItemText (hDlg, IDC_SERVERNAME, sz, sizeof (sz))) 
			{
				PGPclMessageBox (hDlg, IDS_PROPCAPTION, IDS_NOSERVERNAME,
					MB_OK|MB_ICONEXCLAMATION);
				SetFocus (GetDlgItem (hDlg, IDC_SERVERNAME));
				return TRUE;
			}
			else 
			{
				// make sure user didn't type in prefix
				p = strstr (sz, "//");
				if (!p) p = strstr (sz, "\\\\");
				if (p) p += 2;
				else p = &sz[0];
				lstrcpy (pkss->szServer, p);
			}

			// get port number
			pkss->iPort = GetDlgItemInt (hDlg, IDC_PORT, NULL, FALSE);

			//get the keyspace DN if the control is enabled. 
			if (IsWindowEnabled (GetDlgItem (hDlg, IDC_SERVERBASEDN)))
			{
				GetDlgItemText(hDlg, IDC_SERVERBASEDN, 
					 pkss->szKeyStoreDN, sizeof(pkss->szKeyStoreDN));
			}
			else //otherwise make sure nothing is read in dn field
				ZeroMemory (&pkss->szKeyStoreDN, sizeof(pkss->szKeyStoreDN));

			// get domain name
			if (IsDlgButtonChecked (hDlg, IDC_SPECIFIEDDOMAIN)) 
			{
				GetDlgItemText (hDlg, IDC_DOMAIN, pkss->szDomain, 
									sizeof (pkss->szDomain));
			}
			else 
				pkss->szDomain[0] = '\0';

			// get checkbox states			
			if (IsDlgButtonChecked (hDlg, IDC_LISTED) == BST_CHECKED) 
				pkss->bListed = TRUE;
			else 
				pkss->bListed = FALSE;

			if (sIsThisServerAlreadyInList (pkss)) 
			{
				PGPclMessageBox (hDlg, IDS_PROPCAPTION, IDS_DUPLICATESERVER,
					MB_OK|MB_ICONEXCLAMATION);
				SetFocus (GetDlgItem (hDlg, IDC_SERVERNAME));
				return TRUE;
			}

			EndDialog (hDlg, 1);
			break;

		case IDCANCEL:
			EndDialog (hDlg, 0);
			break;
		}
		return TRUE;
	}
	return FALSE;
}

//	_____________________________________________________
//
//  set all text to non-bold

static VOID
sRemoveAllRoots (
		HWND hwndTree) 
{
	HTLITEM				hTServer;
	TL_TREEITEM			tlI;
	PKEYSERVERSTRUCT	pkss;

	// interate through tree
	hTServer = TreeList_GetFirstItem (hwndTree);
	while (hTServer) 
	{
		// get current state
		tlI.hItem = hTServer;
		tlI.mask = TLIF_STATE | TLIF_PARAM;
		TreeList_GetItem (hwndTree, &tlI);

		// make non-root
		pkss = (PKEYSERVERSTRUCT)tlI.lParam;
		pkss->bRoot = FALSE;

		// remove "bold" property
		tlI.state &= ~TLIS_BOLD;
		tlI.stateMask = TLIS_BOLD;
		TreeList_SetItem (hwndTree, &tlI);

		// get next item
		tlI.hItem = hTServer;
		tlI.mask = TLIF_NEXTHANDLE;
		TreeList_GetItem (hwndTree, &tlI);

		hTServer = tlI.hItem;
	}
}

//	_____________________________________________________
//
//  create new keyserver and add to treelist

static INT
sNewKeyserver (
		HWND			hwndParent, 
		PPREFSDLGSTRUCT ppds)
{
	PKEYSERVERSTRUCT pkss;

	pkss = (PKEYSERVERSTRUCT)clAlloc (sizeof(KEYSERVERSTRUCT));
	if (!pkss) 
		return 1;

	pkss->ppds = ppds;
	pkss->hwndTree = ppds->ks.hwndTree;
	pkss->bEdit = FALSE;

	pkss->szServer[0] = '\0';
	pkss->iType = kPGPKeyServerClass_PGP;
	pkss->iPort = 0;
	ZeroMemory (&(pkss->szKeyStoreDN), sizeof(pkss->szKeyStoreDN));
	pkss->protocol = kPGPKeyServerProtocol_LDAP;
	LoadString (g_hInst, IDS_UNKNOWNKEY, pkss->szUserIDAuth, 
					sizeof(pkss->szUserIDAuth));
	pkss->szDomain[0] = '\0';
	pkss->bListed = TRUE;
	pkss->bRoot = FALSE;
	pkss->authAlg = kPGPPublicKeyAlgorithm_Invalid;
	pkss->pOrigStruct = NULL;

	if (!DialogBoxParam (g_hInst, MAKEINTRESOURCE(IDD_NEWSERVER), hwndParent, 
				sEditServerDlgProc, (LPARAM)pkss)) 
	{
		clFree (pkss);
		SetFocus (ppds->ks.hwndTree);
		return 0;
	}

	sSetOneKSItem (ppds->ks.hwndTree, pkss); 
	InvalidateRect (ppds->ks.hwndTree, NULL, TRUE);

	++ppds->ks.uNumberOfServers;

	EnableWindow (GetDlgItem (hwndParent, IDC_REMOVEKEYSERVER), 
						FALSE);
	EnableWindow (GetDlgItem (hwndParent, IDC_EDITKEYSERVER),
						FALSE);
	EnableWindow (GetDlgItem (hwndParent, IDC_SETASROOT),
						FALSE);

	return 0;
}

//	_____________________________________________________
//
//  edit existing keyserver params

static VOID
sEditKeyserver (
		HWND			hwndParent, 
		PPREFSDLGSTRUCT	ppds)
{
	TL_TREEITEM			tlI;
	TL_LISTITEM			tlL;
	KEYSERVERSTRUCT		kss;
	PKEYSERVERSTRUCT	pkss;
	CHAR				sz[256];

	// get server struct
	tlI.hItem = ppds->ks.hTSelected;
	tlI.mask = TLIF_PARAM;
	TreeList_GetItem (ppds->ks.hwndTree, &tlI);

	pkss = (PKEYSERVERSTRUCT)tlI.lParam;
	memcpy (&kss, pkss, sizeof(KEYSERVERSTRUCT));
	kss.bEdit = TRUE;
	kss.pOrigStruct = (VOID*)pkss;

	if (DialogBoxParam (g_hInst, MAKEINTRESOURCE(IDD_NEWSERVER), hwndParent, 
			sEditServerDlgProc, (LPARAM)&kss)) 
	{
		memcpy (pkss, &kss, sizeof(KEYSERVERSTRUCT));

		sURLfromInfo (pkss->protocol, pkss->szServer, pkss->iPort, sz);
		pkss->authAlg = kPGPPublicKeyAlgorithm_Invalid;

		// insert new server text into treelist
		tlI.hItem = ppds->ks.hTSelected;
		tlI.mask = TLIF_TEXT | TLIF_STATE;
		tlI.stateMask = TLIS_BOLD;
		if (pkss->bRoot) 
			tlI.state = TLIS_BOLD;
		else
			tlI.state = 0;
		tlI.cchTextMax = lstrlen (sz);
		tlI.pszText = sz;
		TreeList_SetItem (ppds->ks.hwndTree, &tlI);

		// set new domain text
		tlL.hItem = ppds->ks.hTSelected;
		tlL.mask = TLIF_TEXT;
		tlL.iSubItem = 1;
		tlL.cchTextMax = lstrlen (pkss->szDomain);
		if (tlL.cchTextMax)
			tlL.pszText = pkss->szDomain;
		else
			tlL.pszText = NULL;
		TreeList_SetListItem (ppds->ks.hwndTree, &tlL, FALSE);

		// set listed state
		tlL.hItem = ppds->ks.hTSelected;
		tlL.mask = TLIF_DATAVALUE;
		tlL.iSubItem = 2;
		if (pkss->bListed) 
			tlL.lDataValue = IDX_GREENDOT;
		else 
			tlL.lDataValue = IDX_GREYDOT;
		TreeList_SetListItem (ppds->ks.hwndTree, &tlL, FALSE);
	}

	SetFocus (ppds->ks.hwndTree);
}

//	_____________________________________________________
//
//  remove selected keyserver from treelist

static VOID
sRemoveKeyserver (
		HWND			hwndParent, 
		PPREFSDLGSTRUCT	ppds)
{
	CHAR sz0[256];
	CHAR sz1[256];
	CHAR sz2[256];
	TL_TREEITEM tlI;
	HTLITEM hTParent;

	if (sIsThisTheOnlyWorldServer (ppds)) 
	{
		PGPclMessageBox (hwndParent, IDS_PROPCAPTION, 
				IDS_MUSTHAVEDEFAULTSERVER, MB_OK|MB_ICONEXCLAMATION);
		return;
	}

	tlI.hItem = ppds->ks.hTSelected;
	tlI.mask = TLIF_TEXT | TLIF_PARAM;
	tlI.pszText = sz1;
	tlI.cchTextMax = sizeof(sz1);
	TreeList_GetItem (ppds->ks.hwndTree, &tlI);
	hTParent = tlI.hItem;

	LoadString (g_hInst, IDS_DELKSPROMPT, sz0, sizeof(sz0));
	wsprintf (sz2, sz0, sz1);

	LoadString (g_hInst, IDS_PROPCAPTION, sz1, sizeof(sz1));
	if (MessageBox (hwndParent, sz2, sz1, 
			MB_YESNO|MB_ICONEXCLAMATION) == IDYES) 
	{
		// free KEYSERVERSTRUCT
		clFree ((VOID*)(tlI.lParam));

		// delete selected item
		tlI.hItem = ppds->ks.hTSelected;
		TreeList_DeleteItem (ppds->ks.hwndTree, &tlI);

		ppds->ks.hTSelected = NULL;
		ppds->ks.iSelectedType = IDX_NONE;
		ppds->ks.bSelectedRoot = FALSE;

		// finish up
		InvalidateRect (ppds->ks.hwndTree, NULL, TRUE);
		EnableWindow (GetDlgItem (hwndParent, IDC_REMOVEKEYSERVER), 
							FALSE);
		EnableWindow (GetDlgItem (hwndParent, IDC_EDITKEYSERVER),
							FALSE);
		EnableWindow (GetDlgItem (hwndParent, IDC_SETASROOT),
							FALSE);
		--ppds->ks.uNumberOfServers;
	}
	else 
		SetFocus (ppds->ks.hwndTree);
}


//	_____________________________________________________
//
//  set selected server as root server

static VOID
sSetKeyserverAsRoot (
		HWND			hwndParent, 
		PPREFSDLGSTRUCT	ppds)
{
	TL_TREEITEM			tlI;
	PKEYSERVERSTRUCT	pkss;

	sRemoveAllRoots (ppds->ks.hwndTree);

	tlI.hItem = ppds->ks.hTSelected;
	tlI.mask = TLIF_PARAM;
	TreeList_GetItem (ppds->ks.hwndTree, &tlI);
	pkss = (PKEYSERVERSTRUCT)tlI.lParam;

	pkss->bRoot = TRUE;

	tlI.hItem = ppds->ks.hTSelected;
	tlI.stateMask = TLIS_BOLD;
	tlI.state = TLIS_BOLD;
	tlI.mask = TLIF_STATE;
	TreeList_SetItem (ppds->ks.hwndTree, &tlI);

	EnableWindow (GetDlgItem (hwndParent, IDC_SETASROOT), FALSE);
	SetFocus (ppds->ks.hwndTree);
}


//	_____________________________________________________
//
//  process treelist notification

static VOID
sUpdateSelectionFlags (
		PPREFSDLGSTRUCT ppds)
{
	TL_TREEITEM			tlI;

	tlI.hItem = ppds->ks.hTSelected;
	tlI.mask = TLIF_IMAGE | TLIF_STATE | TLIF_NEXTHANDLE;
	TreeList_GetItem (ppds->ks.hwndTree, &tlI);
	ppds->ks.iSelectedType = tlI.iImage;
	ppds->ks.bSelectedRoot = tlI.state & TLIS_BOLD;
	ppds->ks.bLastInList = !tlI.hItem;

	tlI.hItem = ppds->ks.hTSelected;
	tlI.mask = TLIF_PREVHANDLE;
	TreeList_GetItem (ppds->ks.hwndTree, &tlI);
	ppds->ks.bFirstInList = !tlI.hItem;
}


//	_____________________________________________________
//
//  move selected keyserver up by reinserting item

static VOID
sMoveKeyserverUp (
		HWND			hwndParent, 
		PPREFSDLGSTRUCT ppds)
{
	TL_TREEITEM			tlI;
	TL_INSERTSTRUCT		tlIns;

	tlI.hItem = ppds->ks.hTSelected;
	tlI.mask = TLIF_PREVHANDLE;
	TreeList_GetItem (ppds->ks.hwndTree, &tlI);

	if (tlI.hItem)
	{
		tlI.mask = TLIF_PREVHANDLE;
		TreeList_GetItem (ppds->ks.hwndTree, &tlI);

		if (tlI.hItem)
			tlIns.hInsertAfter = tlI.hItem;
		else
			tlIns.hInsertAfter = TLI_FIRST;

		tlI.hItem = ppds->ks.hTSelected;
		tlIns.item = tlI;
		tlIns.hParent = NULL;
		TreeList_ReInsertItem (ppds->ks.hwndTree, &tlIns);

		InvalidateRect (ppds->ks.hwndTree, NULL, FALSE);
		sUpdateSelectionFlags (ppds);
		EnableWindow (GetDlgItem (hwndParent, IDC_MOVEUP), 
				!ppds->ks.bFirstInList);
		EnableWindow (GetDlgItem (hwndParent, IDC_MOVEDOWN), 
				!ppds->ks.bLastInList);
	}
}


//	_____________________________________________________
//
//  move selected keyserver down by reinserting item

static VOID
sMoveKeyserverDown (
		HWND			hwndParent, 
		PPREFSDLGSTRUCT	ppds)
{
	TL_TREEITEM			tlI;
	TL_INSERTSTRUCT		tlIns;

	tlI.hItem = ppds->ks.hTSelected;
	tlI.mask = TLIF_NEXTHANDLE;
	TreeList_GetItem (ppds->ks.hwndTree, &tlI);

	if (tlI.hItem)
	{
		tlIns.hInsertAfter = tlI.hItem;
		tlI.hItem = ppds->ks.hTSelected;
		tlIns.item = tlI;
		tlIns.hParent = NULL;
		TreeList_ReInsertItem (ppds->ks.hwndTree, &tlIns);

		InvalidateRect (ppds->ks.hwndTree, NULL, FALSE);
		sUpdateSelectionFlags (ppds);
		EnableWindow (GetDlgItem (hwndParent, IDC_MOVEUP), 
				!ppds->ks.bFirstInList);
		EnableWindow (GetDlgItem (hwndParent, IDC_MOVEDOWN), 
				!ppds->ks.bLastInList);
	}
}


//	_____________________________________________________
//
//  process context menu

static VOID
sServerContextMenu (
		HWND				hwnd, 
		PPREFSDLGSTRUCT		ppds,
		PKEYSERVERSTRUCT	pkss,
		INT					iX, 
		INT					iY)
{
	HMENU	hMC;
	HMENU	hMenuTrackPopup;

	if (ppds->ks.bDisabled)
		return;

	hMC = LoadMenu (g_hInst, MAKEINTRESOURCE (IDR_MENUEDITSERVER));
	if (ppds->ks.iSelectedType == IDX_SERVER) 
	{
		EnableMenuItem (hMC, IDM_EDITSERVER, MF_BYCOMMAND|MF_ENABLED);

		if (ppds->ks.uNumberOfServers > 1)
			EnableMenuItem (hMC, IDM_REMOVESERVER, MF_BYCOMMAND|MF_ENABLED);

		if (!ppds->ks.bSelectedRoot && 
			(pkss->iType == kPGPKeyServerClass_PGP))
		{
			EnableMenuItem (hMC, IDM_SETASROOT, MF_BYCOMMAND|MF_ENABLED);
		}

		if (!ppds->ks.bFirstInList)
			EnableMenuItem (hMC, IDM_MOVEUP, MF_BYCOMMAND|MF_ENABLED);

		if (!ppds->ks.bLastInList)
			EnableMenuItem (hMC, IDM_MOVEDOWN, MF_BYCOMMAND|MF_ENABLED);
	}

	hMenuTrackPopup = GetSubMenu (hMC, 0);

	TrackPopupMenu (hMenuTrackPopup, TPM_LEFTALIGN | TPM_RIGHTBUTTON,
					iX, iY, 0, hwnd, NULL);

	DestroyMenu (hMC);
}


//	_____________________________________________________
//
//  process treelist notification

static INT
sProcessKeyserverTreeList (
		HWND			hwndParent, 
		PPREFSDLGSTRUCT	ppds, 
		WPARAM			wParam, 
		LPARAM			lParam)
{
	TL_TREEITEM			tlI;
	TL_LISTITEM			tlL;
	BOOL				bEnable;
	PKEYSERVERSTRUCT	pkss;

	if (ppds->ks.bDisabled)
		return 0;

	switch (((LPNM_TREELIST)lParam)->hdr.code) {

	case TLN_SELCHANGED :
		ppds->ks.hTSelected = ((LPNM_TREELIST)lParam)->itemNew.hItem;

		tlI.hItem = ppds->ks.hTSelected;
		tlI.mask = TLIF_PARAM;
		TreeList_GetItem (ppds->ks.hwndTree, &tlI);

		pkss = (PKEYSERVERSTRUCT)(tlI.lParam);

		if (!ppds->ks.hTSelected)
		{
			ppds->ks.iSelectedType = IDX_NONE;
			ppds->ks.bSelectedRoot = FALSE;
			EnableWindow (GetDlgItem (hwndParent, IDC_EDITKEYSERVER),
								FALSE);
			EnableWindow (GetDlgItem (hwndParent, IDC_REMOVEKEYSERVER), 
								FALSE);
			EnableWindow (GetDlgItem (hwndParent, IDC_SETASROOT), 
								FALSE);
			EnableWindow (GetDlgItem (hwndParent, IDC_MOVEUP), 
								FALSE);
			EnableWindow (GetDlgItem (hwndParent, IDC_MOVEDOWN), 
								FALSE);
			break;
		}

		sUpdateSelectionFlags (ppds);

		switch (ppds->ks.iSelectedType) {
		case IDX_SERVER :
			EnableWindow (
					GetDlgItem (hwndParent, IDC_EDITKEYSERVER), TRUE);

			if (ppds->ks.uNumberOfServers > 1)
				bEnable = TRUE;
			else 
				bEnable = FALSE;

			EnableWindow (
					GetDlgItem (hwndParent, IDC_REMOVEKEYSERVER), bEnable);

			EnableWindow (GetDlgItem (hwndParent, IDC_SETASROOT), 
					!ppds->ks.bSelectedRoot && 
					(pkss->iType == kPGPKeyServerClass_PGP));

			EnableWindow (GetDlgItem (hwndParent, IDC_MOVEUP), 
					!ppds->ks.bFirstInList);
			EnableWindow (GetDlgItem (hwndParent, IDC_MOVEDOWN), 
					!ppds->ks.bLastInList);

			break;
		}
		break;

	case TLN_LISTITEMCLICKED : 
		tlI.hItem = ppds->ks.hTSelected;
		tlI.mask = TLIF_PARAM;
		TreeList_GetItem (ppds->ks.hwndTree, &tlI);

		pkss = (PKEYSERVERSTRUCT)(tlI.lParam);
		pkss->bListed = !pkss->bListed;

		tlL.hItem = ppds->ks.hTSelected;
		tlL.iSubItem = 2;
		tlL.mask = TLIF_DATAVALUE;

		if (pkss->bListed) 
			tlL.lDataValue = IDX_GREENDOT;
		else 
			tlL.lDataValue = IDX_GREYDOT;

		TreeList_SetListItem (ppds->ks.hwndTree, &tlL, FALSE);
		break;

	case TLN_ITEMDBLCLICKED :
		sEditKeyserver (hwndParent, ppds);
		break;

	case TLN_CONTEXTMENU :
		sUpdateSelectionFlags (ppds);

		tlI.hItem = ppds->ks.hTSelected;
		tlI.mask = TLIF_PARAM;
		TreeList_GetItem (ppds->ks.hwndTree, &tlI);

		pkss = (PKEYSERVERSTRUCT)(tlI.lParam);

		sServerContextMenu (hwndParent, ppds, pkss,
					((LPNM_TREELIST)lParam)->ptDrag.x,
					((LPNM_TREELIST)lParam)->ptDrag.y);
		break;

	}
	return 0;
}

//	_____________________________________________________
//
//  traverse treelist and save contents to preferences

static INT
sSaveKeyserverPrefs (
		HWND			hwndParent, 
		PPREFSDLGSTRUCT	ppds)
{
	TL_TREEITEM			tlI;
	HTLITEM				hTServer;
	PGPKeyServerEntry*	keyserverList;
	INT					iServer, iServerCount;
	PKEYSERVERSTRUCT	pkss;

	// count number of servers
	iServerCount = 0;
	hTServer = TreeList_GetFirstItem (ppds->ks.hwndTree);
	while (hTServer) 
	{
		++iServerCount;
		tlI.hItem = hTServer;
		tlI.mask = TLIF_NEXTHANDLE;
		TreeList_GetItem (ppds->ks.hwndTree, &tlI);
		hTServer = tlI.hItem;
	}

	keyserverList = clAlloc (iServerCount * sizeof(PGPKeyServerEntry));
	if (!keyserverList) 
		return kPGPError_OutOfMemory;

	iServer = 0;
	hTServer = TreeList_GetFirstItem (ppds->ks.hwndTree);
	while (hTServer) 
	{
		// get server structure
		tlI.hItem = hTServer;
		tlI.mask = TLIF_PARAM | TLIF_NEXTHANDLE;
		TreeList_GetItem (ppds->ks.hwndTree, &tlI);

		pkss = (PKEYSERVERSTRUCT)(tlI.lParam);

		keyserverList[iServer].type = pkss->iType;
		keyserverList[iServer].protocol = pkss->protocol;
		lstrcpy (keyserverList[iServer].domain, pkss->szDomain);
		lstrcpy (keyserverList[iServer].serverDNS, pkss->szServer);
		keyserverList[iServer].serverPort = pkss->iPort;
		lstrcpy (keyserverList[iServer].keystoredn, pkss->szKeyStoreDN);

		// authentication key
		if (pkss->authAlg != kPGPPublicKeyAlgorithm_Invalid) 
		{
			keyserverList[iServer].authAlg = pkss->authAlg;
			lstrcpy (keyserverList[iServer].authKeyIDString, 
						pkss->szKeyIDAuth);
		}
		else 
		{
			keyserverList[iServer].authAlg = kPGPPublicKeyAlgorithm_Invalid;
			keyserverList[iServer].authKeyIDString[0] = '\0';
		}

		// listed flag
		keyserverList[iServer].flags = 0;
		if (pkss->bListed) 
			keyserverList[iServer].flags |= kKeyServerListed;
		if (pkss->bRoot) 
			keyserverList[iServer].flags |= kKeyServerIsRoot;

		// step to next server
		++iServer;
		hTServer = tlI.hItem;
	}

	// done, set preferences
	PGPSetKeyServerPrefs (ppds->prefref, keyserverList, iServerCount);

	clFree (keyserverList);

	return 0;
}


//	_____________________________________________________
//
//  Message processing function for property sheet dialog

LRESULT WINAPI
CLKeyserverPropDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PPREFSDLGSTRUCT		ppds;

	switch (uMsg) {

	case WM_INITDIALOG:
	{
		PGPUInt32	u;
		PGPPrefRef	prefref;

		SetWindowLong (hwnd, GWL_USERDATA, ((PROPSHEETPAGE*)lParam)->lParam);
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		if (ppds->bNeedsCentering) 
		{
			PGPclCenterWindowOnMonitor (GetParent (hwnd), kPGPclWindowMonitor);
			ppds->bNeedsCentering = FALSE;
		}

		prefref = CLGetPrefRef (ppds);

		ppds->ks.hwndTree = sCreateKeyserverTreeList (hwnd, ppds);
		SetWindowPos (ppds->ks.hwndTree, 
				HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);

		// initialize "Sync On ..." checkboxes
		CLLoadCheckBoxPref (hwnd, IDC_SYNCONUNKNOWN, 
				ppds, prefref, kPGPPrefKeyServerSyncUnknownKeys);

		CLLoadCheckBoxPref (hwnd, IDC_SYNCONADDNAME, 
				ppds, prefref, kPGPPrefKeyServerSyncOnAdd);

		CLLoadCheckBoxPref (hwnd, IDC_SYNCONKEYSIGN, 
				ppds, prefref, kPGPPrefKeyServerSyncOnKeySign);

		CLLoadCheckBoxPref (hwnd, IDC_SYNCONREVOKE, 
				ppds, prefref, kPGPPrefKeyServerSyncOnRevocation);

		CLLoadCheckBoxPref (hwnd, IDC_SYNCONVERIFY, 
				ppds, prefref, kPGPPrefKeyServerSyncOnVerify);

		// disable host list if locked
		PGPGetPrefFlags (prefref, kPGPPrefKeyServerList, &u);
		ppds->ks.bDisabled = u & kPGPPrefFlags_DisableGUI;
		if (ppds->ks.bDisabled)
			EnableWindow (GetDlgItem (hwnd, IDC_NEWKEYSERVER), FALSE);

		return TRUE;
	}

	case WM_TIMER :
		KillTimer (hwnd, DELAYTIMER);
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		// load keyset if not already loaded
		if (!PGPKeyDBRefIsValid (ppds->keydbMain))
		{
			HCURSOR	hCursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
			PGPclErrorBox (hwnd, PGPclOpenDefaultKeyrings (
					ppds->context, kPGPOpenKeyDBFileOptions_None, 
					&ppds->keydbMain));
			SetCursor (hCursorOld);
		}
		break;

	case WM_KEYDOWN :
	case WM_SYSKEYDOWN :
	case WM_KEYUP :
	case WM_CHAR :
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		if (GetFocus () == ppds->ks.hwndTree)
			SendMessage (ppds->ks.hwndTree, uMsg, wParam, lParam);
		break;

	case WM_DESTROY :
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		sDestroyKeyserverTreeList (ppds->ks.hwndTree);
		break;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aServerIds);

	case WM_COMMAND :
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		switch (LOWORD (wParam)) {
		case IDM_NEWSERVER :
		case IDC_NEWKEYSERVER :
			sNewKeyserver (hwnd, ppds);
			break;

		case IDM_REMOVESERVER :
		case IDC_REMOVEKEYSERVER :
			sRemoveKeyserver (hwnd, ppds);
			break;

		case IDM_EDITSERVER :
		case IDC_EDITKEYSERVER :
			sEditKeyserver (hwnd, ppds);
			break;

		case IDM_SETASROOT :
		case IDC_SETASROOT :
			sSetKeyserverAsRoot (hwnd, ppds);
			break;

		case IDM_MOVEUP :
		case IDC_MOVEUP :
			sMoveKeyserverUp (hwnd, ppds);
			break;

		case IDM_MOVEDOWN :
		case IDC_MOVEDOWN :
			sMoveKeyserverDown (hwnd, ppds);
			break;

		case IDM_SERVERHELP :
			PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0,
					(char*)kPGPclHelpFile, 
					(DWORD*)IDH_PGPCLPREF_SERVERLISTWHATSTHIS);
			break;
		}
		break;

	case WM_NOTIFY :
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		if (((NMHDR FAR *) lParam)->hwndFrom == ppds->ks.hwndTree) 
		{
			sProcessKeyserverTreeList (hwnd, ppds, wParam, lParam);
		}
		else 
		{
			switch (((NMHDR FAR *) lParam)->code) {
			case PSN_HELP :
				PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0,
						(char*)kPGPclHelpFile, 
						(DWORD*)IDH_PGPCLPREF_KEYSERVERDIALOG);
				break;

			case PSN_SETACTIVE :
				SetTimer (hwnd, DELAYTIMER, 100, NULL);
				break;

			case PSN_APPLY :
				if (ppds->bUseClientLibPrefs && 
					!PGPPrefRefIsValid (ppds->prefref))
				{
					PGPclPeekClientLibPrefRefs (&ppds->prefref, NULL);
				}

				sSaveKeyserverPrefs (hwnd, ppds);

				// get and save checkbox values
				CLSaveCheckBoxPref (hwnd, IDC_SYNCONUNKNOWN, 
						ppds->prefref, kPGPPrefKeyServerSyncUnknownKeys);

				CLSaveCheckBoxPref (hwnd, IDC_SYNCONADDNAME, 
						ppds->prefref, kPGPPrefKeyServerSyncOnAdd);

				CLSaveCheckBoxPref (hwnd, IDC_SYNCONKEYSIGN, 
						ppds->prefref, kPGPPrefKeyServerSyncOnKeySign);

				CLSaveCheckBoxPref (hwnd, IDC_SYNCONREVOKE, 
						ppds->prefref, kPGPPrefKeyServerSyncOnRevocation);

				CLSaveCheckBoxPref (hwnd, IDC_SYNCONVERIFY, 
						ppds->prefref, kPGPPrefKeyServerSyncOnVerify);

				ppds->bPrefsModified = TRUE;
				ppds->ks.bPrefsModified = TRUE;
				ppds->returnErr = kPGPError_NoErr;
	            SetWindowLong (hwnd, DWL_MSGRESULT, PSNRET_NOERROR);
	            return TRUE;
			}
		}
	}

	return FALSE;
}

