/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	CLprefsNetAdv.c - handle PGPnet advanced preferences dialog

	$Id: CLprefsNetAdv.c,v 1.16 2002/08/06 20:09:19 dallen Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "PGPvpnHelp.h"
#include "CLprefs.h"
#include "pgpFileSpec.h"

// definitions
#define PGP_ENABLE_EC_GUI	PGP_DEBUG

#define OFFSET_FIRST		2
#define OFFSET_OTHER		6

#define MAXPROPOSALS		16

#define	IKEAUTHCOLWIDTH		100
#define	IKEHASHCOLWIDTH		60
#define	IKECIPHERCOLWIDTH	70
#define	IKEDHCOLWIDTH		60

#define	IKECOL_AUTH			0
#define	IKECOL_HASH			1
#define	IKECOL_CIPHER		2
#define	IKECOL_DH			3

#define	IPSECAHCOLWIDTH		60
#define	IPSECESPCOLWIDTH	130
#define	IPSECCOMPCOLWIDTH	60

#define	IPSECCOL_AH			0
#define	IPSECCOL_ESP		1
#define	IPSECCOL_COMP		2

// external globals
extern HINSTANCE	g_hInst;

static DWORD aAdvIds[] = {			// Help IDs
    IDC_ALLOWCAST,				IDH_PGPCLPREF_ALLOWCAST,
	IDC_ALLOW3DES,				IDH_PGPCLPREF_ALLOW3DES,
	IDC_ALLOWDES,				IDH_PGPCLPREF_ALLOWDES,
	IDC_ALLOWNONECIPHER,		IDH_PGPCLPREF_ALLOWNULLCIPHER,
	IDC_ALLOWSHA1,				IDH_PGPCLPREF_ALLOWSHA,
	IDC_ALLOWMD5,				IDH_PGPCLPREF_ALLOWMD5,
	IDC_ALLOWNONEHASH,			IDH_PGPCLPREF_ALLOWNULLHASH,
	IDC_ALLOW1024,				IDH_PGPCLPREF_ALLOW1024,
	IDC_ALLOW1536,				IDH_PGPCLPREF_ALLOW1536,
	IDC_ALLOWEC163,				0,
	IDC_ALLOWEC283,				0,
	IDC_ALLOWLZS,				IDH_PGPCLPREF_ALLOWLZS,
	IDC_ALLOWDEFLATE,			IDH_PGPCLPREF_ALLOWDEFLATE,
	IDC_DEFAULTS,				IDH_PGPCLPREF_DEFAULTSETTINGS,
	IDC_IKEPROPOSALLIST,		IDH_PGPCLPREF_IKEPROPOSALS,
	IDC_IPSECPROPOSALLIST,		IDH_PGPCLPREF_IPSECPROPOSALS,
	IDC_NEWPROPOSAL,			IDH_PGPCLPREF_NEWPROPOSAL,
	IDC_EDITPROPOSAL,			IDH_PGPCLPREF_EDITPROPOSAL,
	IDC_REMOVEPROPOSAL,			IDH_PGPCLPREF_REMOVEPROPOSAL,
	IDC_PROPOSALUP,				IDH_PGPCLPREF_MOVEPROPOSALUP,
	IDC_PROPOSALDOWN,			IDH_PGPCLPREF_MOVEPROPOSALDOWN,
	IDC_PERFECTSECRECY,			IDH_PGPCLPREF_PERFECTFWDSECRECY,
	0,0
};

static DWORD aIkeIds[] = {			// Help IDs
    IDC_IKEAUTHCOMBO,			IDH_PGPCLPREF_IKEAUTHENTICATION,
    IDC_IKEHASHCOMBO,			IDH_PGPCLPREF_IKEHASH,
    IDC_IKECIPHERCOMBO,			IDH_PGPCLPREF_IKECIPHER,
    IDC_IKEDHCOMBO,				IDH_PGPCLPREF_IKEDIFFIEHELLMAN,
	0,0
};

static DWORD aIpsecIds[] = {		// Help IDs
    IDC_ENABLEAH,				IDH_PGPCLPREF_IPSECAHENABLE,
    IDC_IPSECAHCOMBO,			IDH_PGPCLPREF_IPSECAHHASH,
    IDC_ENABLEESP,				IDH_PGPCLPREF_IPSECESPENABLE,
    IDC_IPSECESPHASHCOMBO,		IDH_PGPCLPREF_IPSECESPHASH,
    IDC_IPSECESPCIPHERCOMBO,	IDH_PGPCLPREF_IPSECESPCIPHER,
    IDC_ENABLECOMP,				IDH_PGPCLPREF_IPSECCOMPENABLE,
    IDC_IPSECCOMPCOMBO,			IDH_PGPCLPREF_IPSECCOMPALGORITHM,
	0,0
};


//	____________________________________
//
//	select the specified item

static VOID
sSelectItem (
		HWND	hwnd,
		INT		iControl,
		INT		iItem)
{
	HWND	hwndList;

	hwndList = GetDlgItem (hwnd, iControl);

	ListView_SetItemState (hwndList, iItem, LVIS_SELECTED, LVIS_SELECTED);
	ListView_EnsureVisible (hwndList, iItem, FALSE);
}


//	____________________________________
//
//	initialize allowed algorithms checkboxes

static VOID
sInitAllowedAlgorithms (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds,
		PGPPrefRef		prefref)
{
	// ciphers
	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOWCAST)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOWCAST,
			ppds, prefref, kPGPNetPrefAllowCipherCAST);
	}

	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOW3DES)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOW3DES,
			ppds, prefref, kPGPNetPrefAllowCipher3DES);
	}

	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOWDES)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOWDES,
			ppds, prefref, kPGPNetPrefAllowCipherDES56);
	}

	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOWNONECIPHER)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOWNONECIPHER,
			ppds, prefref, kPGPNetPrefAllowCipherNone);
	}

	// hashes
	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOWSHA1)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOWSHA1,
			ppds, prefref, kPGPNetPrefAllowHashSHA1);
	}

	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOWMD5)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOWMD5,
			ppds, prefref, kPGPNetPrefAllowHashMD5);
	}

	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOWNONEHASH)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOWNONEHASH,
			ppds, prefref, kPGPNetPrefAllowHashNone);
	}

	// compression
	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOWLZS)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOWLZS,
			ppds, prefref, kPGPNetPrefAllowCompLZS);
	}

	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOWDEFLATE)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOWDEFLATE,
			ppds, prefref, kPGPNetPrefAllowCompDeflate);
	}

	// Diffie-Hellman
	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOW1024)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOW1024,
			ppds, prefref, kPGPNetPrefAllowDH1024);
	}

	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOW1536)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOW1536,
			ppds, prefref, kPGPNetPrefAllowDH1536);
	}

#if PGP_ENABLE_EC_GUI
	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOWEC163)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOWEC163,
			ppds, prefref, kPGPNetPrefAllowEC163);
	}

	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_ALLOWEC283)))
	{
		CLLoadCheckBoxPref (hwnd, IDC_ALLOWEC283,
			ppds, prefref, kPGPNetPrefAllowEC283);
	}
#else
	EnableWindow (GetDlgItem (hwnd, IDC_ALLOWEC163), FALSE);
	ShowWindow (GetDlgItem (hwnd, IDC_ALLOWEC163), SW_HIDE);
	EnableWindow (GetDlgItem (hwnd, IDC_ALLOWEC283), FALSE);
	ShowWindow (GetDlgItem (hwnd, IDC_ALLOWEC283), SW_HIDE);
#endif // PGP_ENABLE_EC_GUI
}


//	____________________________________
//
//	retrieve allowed algorithm prefs from checkbox controls

static VOID
sSaveAllowedAlgorithms (
		HWND			hwnd,
		PGPPrefRef		prefref)
{
	// ciphers
	CLSaveCheckBoxPref (hwnd, IDC_ALLOWCAST,
			prefref, kPGPNetPrefAllowCipherCAST);

	CLSaveCheckBoxPref (hwnd, IDC_ALLOW3DES,
			prefref, kPGPNetPrefAllowCipher3DES);

	PGPSetPrefBoolean (prefref, kPGPNetPrefAllowCipherDES56, FALSE);

	CLSaveCheckBoxPref (hwnd, IDC_ALLOWNONECIPHER,
			prefref, kPGPNetPrefAllowCipherNone);

	// hashes
	CLSaveCheckBoxPref (hwnd, IDC_ALLOWSHA1,
			prefref, kPGPNetPrefAllowHashSHA1);

	CLSaveCheckBoxPref (hwnd, IDC_ALLOWMD5,
			prefref, kPGPNetPrefAllowHashMD5);

	CLSaveCheckBoxPref (hwnd, IDC_ALLOWNONEHASH,
			prefref, kPGPNetPrefAllowHashNone);

	// compression
	CLSaveCheckBoxPref (hwnd, IDC_ALLOWLZS,
			prefref, kPGPNetPrefAllowCompLZS);

	CLSaveCheckBoxPref (hwnd, IDC_ALLOWDEFLATE,
			prefref, kPGPNetPrefAllowCompDeflate);

	// Diffie-Hellman
	CLSaveCheckBoxPref (hwnd, IDC_ALLOW1024,
			prefref, kPGPNetPrefAllowDH1024);

	CLSaveCheckBoxPref (hwnd, IDC_ALLOW1536,
			prefref, kPGPNetPrefAllowDH1536);

#if PGP_ENABLE_EC_GUI
	CLSaveCheckBoxPref (hwnd, IDC_ALLOWEC163,
			prefref, kPGPNetPrefAllowEC163);

	CLSaveCheckBoxPref (hwnd, IDC_ALLOWEC283,
			prefref, kPGPNetPrefAllowEC283);
#else
	PGPSetPrefBoolean (prefref, kPGPNetPrefAllowEC163, FALSE);
	PGPSetPrefBoolean (prefref, kPGPNetPrefAllowEC283, FALSE);
#endif // PGP_ENABLE_EC_GUI
}

//	____________________________________
//
//	initialize IPSEC groupID combobox

static VOID
sLoadIpsecGroupID (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds,
		PGPPrefRef		prefref)
{
	UINT	u;
	INT		iIndex;

	if (IsWindowEnabled (GetDlgItem (hwnd, IDC_PERFECTSECRECY)))
	{
		PGPGetPrefNumber (prefref, kPGPnetPrefIPSECGroupID, &u);

		switch (u) {
			case kPGPike_GR_None :		iIndex = 0; break;
			case kPGPike_GR_MODPTwo :	iIndex = 1; break;
			case kPGPike_GR_MODPFive :	iIndex = 2; break;
			default :					iIndex = 0; break;
		}

		SendMessage (GetDlgItem (hwnd, IDC_PERFECTSECRECY),
				CB_SETCURSEL, (WPARAM)iIndex, 0);
		CLDisablePrefControl (hwnd, IDC_PERFECTSECRECY,
				ppds, prefref, kPGPnetPrefIPSECGroupID);
	}
}


//	____________________________________
//
//	retrieve group ID pref from combobox control

static VOID
sSaveIpsecGroupID (
		HWND			hwnd,
		PGPPrefRef		prefref)
{
	INT		iIndex;
	UINT	u;

	iIndex = SendMessage (
			GetDlgItem (hwnd, IDC_PERFECTSECRECY), CB_GETCURSEL, 0, 0);

	switch (iIndex) {
		case 0 :	u = kPGPike_GR_None;		break;
		case 1 :	u = kPGPike_GR_MODPTwo;		break;
		case 2 :	u = kPGPike_GR_MODPFive;	break;
		default :	u = kPGPike_GR_None;		break;
	}

	PGPSetPrefNumber (prefref, kPGPnetPrefIPSECGroupID, u);
}


//	____________________________________
//
//	initialize IKE proposal list view

static VOID
sInitIkeProposalListControl (
		HWND	hwnd)
{
	HWND			hwndList;
	LV_COLUMN		lvc;
	CHAR			sz[64];

	hwndList = GetDlgItem (hwnd, IDC_IKEPROPOSALLIST);

	lvc.mask = LVCF_FMT|LVCF_SUBITEM|LVCF_TEXT|LVCF_WIDTH;
	lvc.pszText = sz;

	LoadString (g_hInst, IDS_IKEAUTHCOLNAME, sz, sizeof(sz));
	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = IKEAUTHCOLWIDTH;
	lvc.iSubItem = IKECOL_AUTH;
	ListView_InsertColumn (hwndList, 0, &lvc);

	LoadString (g_hInst, IDS_IKEHASHCOLNAME, sz, sizeof(sz));
	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = IKEHASHCOLWIDTH;
	lvc.iSubItem = IKECOL_HASH;
	ListView_InsertColumn (hwndList, 1, &lvc);

	LoadString (g_hInst, IDS_IKECIPHERCOLNAME, sz, sizeof(sz));
	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = IKECIPHERCOLWIDTH;
	lvc.iSubItem = IKECOL_CIPHER;
	ListView_InsertColumn (hwndList, 2, &lvc);

	LoadString (g_hInst, IDS_IKEDHCOLNAME, sz, sizeof(sz));
	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = IKEDHCOLWIDTH;
	lvc.iSubItem = IKECOL_DH;
	ListView_InsertColumn (hwndList, 3, &lvc);
}

//	____________________________________
//
//	insert a single entry into list

static VOID
sDisplayIkeProposalList (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds)
{
	LVITEM				lvi;
	CHAR				sz[64];
	INT					ids;
	UINT				u;
	HWND				hwndList;

	PGPNetPrefIKEProposalEntry*		pIke;

	hwndList = GetDlgItem (hwnd, IDC_IKEPROPOSALLIST);
	ListView_DeleteAllItems (hwndList);

	lvi.pszText = sz;
	lvi.stateMask = 0;
	lvi.mask = LVIF_TEXT;

	for (u=0; u<ppds->net.uNumIKEProposals; u++)
	{
		pIke = &ppds->net.pIKEProposals[u];

		lvi.iItem = u;

		// authentication string
		lvi.iSubItem = IKECOL_AUTH;
		switch (pIke->authMethod) {
			case kPGPike_AM_None :			ids = IDS_NONE;			break;
			case kPGPike_AM_PreSharedKey :	ids = IDS_SHAREDKEY;	break;
			case kPGPike_AM_DSS_Sig :		ids = IDS_DSSSIGNATURE;	break;
			case kPGPike_AM_RSA_Sig :		ids = IDS_RSASIGNATURE;	break;
			default :						ids = IDS_UNKNOWNITEM;	break;
		}
		LoadString (g_hInst, ids, sz, sizeof(sz));
		ListView_InsertItem (hwndList, &lvi);

		lvi.mask = LVIF_TEXT;

		// hash string
		lvi.iSubItem = IKECOL_HASH;
		switch (pIke->hash) {
			case kPGPike_HA_None :			ids = IDS_NONE;			break;
			case kPGPike_HA_MD5 :			ids = IDS_MD5;			break;
			case kPGPike_HA_SHA1 :			ids = IDS_SHA;			break;
			default :						ids = IDS_UNKNOWNITEM;	break;
		}
		LoadString (g_hInst, ids, sz, sizeof(sz));
		ListView_SetItem (hwndList, &lvi);

		// IPCOMP string
		lvi.iSubItem = IKECOL_CIPHER;
		switch (pIke->cipher) {
			case kPGPike_SC_None :			ids = IDS_NONE;			break;
			case kPGPike_SC_DES_CBC :		ids = IDS_DES56;		break;
			case kPGPike_SC_3DES_CBC :		ids = IDS_3DES;			break;
			case kPGPike_SC_CAST_CBC :		ids = IDS_CAST;			break;
			default :						ids = IDS_UNKNOWNITEM;	break;
		}
		LoadString (g_hInst, ids, sz, sizeof(sz));
		ListView_SetItem (hwndList, &lvi);

		// DH string
		lvi.iSubItem = IKECOL_DH;
		switch (pIke->groupID) {
			case kPGPike_GR_None :			ids = IDS_NONE;			break;
			case kPGPike_GR_MODPOne :		ids = IDS_768;			break;
			case kPGPike_GR_MODPTwo :		ids = IDS_1024;			break;
			case kPGPike_GR_MODPFive :		ids = IDS_1536;			break;
#if PGP_ENABLE_EC_GUI
			case kPGPike_GR_ECSix :			ids = IDS_EC163;		break;
			case kPGPike_GR_ECEight :		ids = IDS_EC283;		break;
#endif // PGP_ENABLE_EC_GUI
			default :						ids = IDS_UNKNOWNITEM;	break;
		}
		LoadString (g_hInst, ids, sz, sizeof(sz));
		ListView_SetItem (hwndList, &lvi);
	}
}


//	____________________________________
//
//	load IKE proposal list view with entries

static VOID
sLoadIkeProposalList (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds,
		PGPPrefRef		prefref)
{
	PGPError			err			= kPGPError_NoErr;
	PGPNetPrefIKEProposalEntry*		pProposals;

	if (ppds->net.pIKEProposals)
	{
		PGPFreeData (ppds->net.pIKEProposals);
		ppds->net.uNumIKEProposals = 0;
	}

	err = PGPGetNetIKEProposalPrefs (prefref,
				&pProposals, &ppds->net.uNumIKEProposals);

	if (IsntPGPError (err))
	{
		PGPSize size = sizeof(PGPNetPrefIKEProposalEntry) *
				ppds->net.uNumIKEProposals;
		ppds->net.pIKEProposals = PGPNewData (
				PGPPeekContextMemoryMgr (ppds->context),
				size, kPGPMemoryMgrFlags_Clear);
		if (ppds->net.pIKEProposals)
		{
			pgpCopyMemory (pProposals, ppds->net.pIKEProposals, size);
			sDisplayIkeProposalList (hwnd, ppds);
		}
		PGPFreeData (pProposals);
	}
}


//	____________________________________
//
//	initialize IKE proposal list view

static VOID
sInitIpsecProposalListControl (
		HWND	hwnd)
{
	HWND			hwndList;
	LV_COLUMN		lvc;
	CHAR			sz[64];

	hwndList = GetDlgItem (hwnd, IDC_IPSECPROPOSALLIST);

	lvc.mask = LVCF_FMT|LVCF_SUBITEM|LVCF_TEXT|LVCF_WIDTH;
	lvc.pszText = sz;

	LoadString (g_hInst, IDS_IPSECAHCOLNAME, sz, sizeof(sz));
	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = IPSECAHCOLWIDTH;
	lvc.iSubItem = IPSECCOL_AH;
	ListView_InsertColumn (hwndList, 0, &lvc);

	LoadString (g_hInst, IDS_IPSECESPCOLNAME, sz, sizeof(sz));
	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = IPSECESPCOLWIDTH;
	lvc.iSubItem = IPSECCOL_ESP;
	ListView_InsertColumn (hwndList, 1, &lvc);

	LoadString (g_hInst, IDS_IPSECCOMPCOLNAME, sz, sizeof(sz));
	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = IPSECCOMPCOLWIDTH;
	lvc.iSubItem = IPSECCOL_COMP;
	ListView_InsertColumn (hwndList, 2, &lvc);
}


//	____________________________________
//
//	cat a string resource onto string, using comma if necessary

VOID
sCommaCat (
		LPSTR	psz,
		INT		ids)
{
	CHAR	szTemp[32];;

	// if string is not empty, cat comma, space onto it
	if (psz[0])
		lstrcat (psz, ", ");

	LoadString (g_hInst, ids, szTemp, sizeof(szTemp));
	lstrcat (psz, szTemp);
}


//	____________________________________
//
//	load IPSEC proposal list view with entries

static VOID
sDisplayIpsecProposalList (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds)
{
	CHAR				sz[64];
	LVITEM				lvi;
	INT					ids;
	UINT				u;
	HWND				hwndList;

	PGPNetPrefIPSECProposalEntry*		pIpsec;

	hwndList = GetDlgItem (hwnd, IDC_IPSECPROPOSALLIST);
	ListView_DeleteAllItems (hwndList);

	lvi.pszText = sz;
	lvi.stateMask = 0;
	lvi.mask = LVIF_TEXT;

	for (u=0; u<ppds->net.uNumIPSECProposals; u++)
	{
		pIpsec = &ppds->net.pIPSECProposals[u];

		lvi.iItem = u;

		// AH authentication string
		lvi.iSubItem = IPSECCOL_AH;
		switch (pIpsec->ahHash) {
			case kPGPike_AA_None :			ids = IDS_NONE;			break;
			case kPGPike_AA_HMAC_MD5 :		ids = IDS_MD5;			break;
			case kPGPike_AA_HMAC_SHA :		ids = IDS_SHA;			break;
			default :						ids = IDS_UNKNOWN;		break;
		}
		LoadString (g_hInst, ids, sz, sizeof(sz));
		ListView_InsertItem (hwndList, &lvi);

		// ESP hash string
		sz[0] = '\0';
		lvi.iSubItem = IPSECCOL_ESP;
		switch (pIpsec->espHash) {
			case kPGPike_AA_None :			ids = IDS_NONE;			break;
			case kPGPike_AA_HMAC_MD5 :		ids	= IDS_MD5;			break;
			case kPGPike_AA_HMAC_SHA :		ids = IDS_SHA;			break;
			default :						ids = IDS_UNKNOWN;		break;
		}
		if (ids != IDS_NONE)
			LoadString (g_hInst, ids, sz, sizeof(sz));

		// ESP cipher string
		lvi.iSubItem = IPSECCOL_ESP;
		switch (pIpsec->espCipher) {
			case kPGPike_ET_NULL :			ids = IDS_NONE;			break;
			case kPGPike_ET_DES :			ids	= IDS_DES56;		break;
			case kPGPike_ET_DES_IV64 :		ids	= IDS_DES56;		break;
			case kPGPike_ET_3DES :			ids = IDS_3DES;			break;
			case kPGPike_ET_CAST :			ids = IDS_CAST;			break;
			default :						ids = IDS_UNKNOWN;		break;
		}
		if ((ids != IDS_NONE) || (sz[0] == '\0'))
			sCommaCat (sz, ids);
		ListView_SetItem (hwndList, &lvi);

		// IPCOMP string
		lvi.iSubItem = IPSECCOL_COMP;
		switch (pIpsec->ipcomp) {
			case kPGPike_IC_None :			ids = IDS_NONE;			break;
			case kPGPike_IC_Deflate :		ids = IDS_DEFLATE;		break;
			case kPGPike_IC_LZS :			ids = IDS_LZS;			break;
			default :						ids = IDS_UNKNOWN;		break;
		}
		LoadString (g_hInst, ids, sz, sizeof(sz));
		ListView_SetItem (hwndList, &lvi);
	}
}


//	____________________________________
//
//	load IPSEC proposal list view with entries

static VOID
sLoadIpsecProposalList (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds,
		PGPPrefRef		prefref)
{
	PGPError			err			= kPGPError_NoErr;
	PGPNetPrefIPSECProposalEntry*	pProposals;

	if (ppds->net.pIPSECProposals)
	{
		PGPFreeData (ppds->net.pIPSECProposals);
		ppds->net.uNumIPSECProposals = 0;
	}

	err = PGPGetNetIPSECProposalPrefs (prefref,
				&pProposals, &ppds->net.uNumIPSECProposals);

	if (IsntPGPError (err))
	{
		PGPSize size = sizeof(PGPNetPrefIPSECProposalEntry) *
				ppds->net.uNumIPSECProposals;
		ppds->net.pIPSECProposals = PGPNewData (
				PGPPeekContextMemoryMgr (ppds->context),
				size, kPGPMemoryMgrFlags_Clear);
		if (ppds->net.pIPSECProposals)
		{
			pgpCopyMemory (pProposals, ppds->net.pIPSECProposals, size);
			sDisplayIpsecProposalList (hwnd, ppds);
		}
		PGPFreeData (pProposals);
	}
}


//	____________________________________
//
//	dialog proc of IKE proposal dialog

static BOOL CALLBACK
sIkeProposalDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PGPNetPrefIKEProposalEntry*	pIke;

	switch (uMsg) {
	case WM_INITDIALOG :
	{
		CHAR	sz[32];
		HWND	hwndCombo;
		UINT	u;

		SetWindowLong (hwnd, GWL_USERDATA, lParam);
		pIke = (PGPNetPrefIKEProposalEntry*)GetWindowLong (hwnd, GWL_USERDATA);

		// load combo boxes with text strings
		hwndCombo = GetDlgItem (hwnd, IDC_IKEAUTHCOMBO);
		LoadString (g_hInst, IDS_SHAREDKEY, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_DSSSIGNATURE, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_RSASIGNATURE, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);

		hwndCombo = GetDlgItem (hwnd, IDC_IKEHASHCOMBO);
		LoadString (g_hInst, IDS_SHA, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_MD5, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);

		hwndCombo = GetDlgItem (hwnd, IDC_IKECIPHERCOMBO);
		LoadString (g_hInst, IDS_CAST, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_3DES, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);

		hwndCombo = GetDlgItem (hwnd, IDC_IKEDHCOMBO);
		LoadString (g_hInst, IDS_1024, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_1536, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
#if PGP_ENABLE_EC_GUI
		LoadString (g_hInst, IDS_EC163, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_EC283, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
#endif

		// initialize combo boxes
		switch (pIke->authMethod) {
			case kPGPike_AM_PreSharedKey :	u = 0;	break;
			case kPGPike_AM_DSS_Sig :		u = 1;	break;
			case kPGPike_AM_RSA_Sig :		u = 2;	break;
			default :						u = 0;	break;
		}
		SendDlgItemMessage (hwnd, IDC_IKEAUTHCOMBO, CB_SETCURSEL, u, 0);

		switch (pIke->hash) {
			case kPGPike_HA_SHA1 :			u = 0;	break;
			case kPGPike_HA_MD5 :			u = 1;	break;
			default :						u = 0;	break;
		}
		SendDlgItemMessage (hwnd, IDC_IKEHASHCOMBO, CB_SETCURSEL, u, 0);

		switch (pIke->cipher) {
			case kPGPike_SC_CAST_CBC :		u = 0;	break;
			case kPGPike_SC_3DES_CBC :		u = 1;	break;
			default :						u = 0;	break;
		}
		SendDlgItemMessage (hwnd, IDC_IKECIPHERCOMBO, CB_SETCURSEL, u, 0);

		switch (pIke->groupID) {
			case kPGPike_GR_MODPTwo :		u = 0;	break;
			case kPGPike_GR_MODPFive :		u = 1;	break;
#if PGP_ENABLE_EC_GUI
			case kPGPike_GR_ECSix :			u = 2;	break;
			case kPGPike_GR_ECEight :		u = 3;	break;
#endif
			default :						u = 0;	break;
		}
		SendDlgItemMessage (hwnd, IDC_IKEDHCOMBO, CB_SETCURSEL, u, 0);

		return FALSE;
	}

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclNetHelpFile, aIkeIds);

	case WM_COMMAND :
		pIke = (PGPNetPrefIKEProposalEntry*)GetWindowLong (hwnd, GWL_USERDATA);

		switch (LOWORD (wParam)) {

		case IDOK :
		{
			UINT	u;

			u = SendDlgItemMessage (
						hwnd, IDC_IKEAUTHCOMBO, CB_GETCURSEL, 0, 0);
			switch (u) {
			case 0	:	pIke->authMethod = kPGPike_AM_PreSharedKey;	break;
			case 1	:	pIke->authMethod = kPGPike_AM_DSS_Sig;		break;
			case 2	:	pIke->authMethod = kPGPike_AM_RSA_Sig;		break;
			}

			u = SendDlgItemMessage (
						hwnd, IDC_IKEHASHCOMBO, CB_GETCURSEL, 0, 0);
			switch (u) {
			case 0	:	pIke->hash = kPGPike_HA_SHA1;			break;
			case 1	:	pIke->hash = kPGPike_HA_MD5;			break;
			}

			u = SendDlgItemMessage (
						hwnd, IDC_IKECIPHERCOMBO, CB_GETCURSEL, 0, 0);
			switch (u) {
			case 0	:	pIke->cipher = kPGPike_SC_CAST_CBC;		break;
			case 1	:	pIke->cipher = kPGPike_SC_3DES_CBC;		break;
			case 2	:	pIke->cipher = kPGPike_SC_DES_CBC;		break;
			}

			u = SendDlgItemMessage (
						hwnd, IDC_IKEDHCOMBO, CB_GETCURSEL, 0, 0);
			switch (u) {
			case 0	:	pIke->groupID = kPGPike_GR_MODPTwo;		break;
			case 1	:	pIke->groupID = kPGPike_GR_MODPFive;	break;
			case 2	:	pIke->groupID = kPGPike_GR_ECSix;		break;
			case 3	:	pIke->groupID = kPGPike_GR_ECEight;		break;
			}

			if ((pIke->authMethod == kPGPike_AM_DSS_Sig) &&
				(pIke->hash != kPGPike_HA_SHA1))
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONWARNING, 
						IDS_DSSNONSHAWARN, MB_OK|MB_ICONEXCLAMATION);
				break;
			}

			if (pIke->cipher == kPGPike_SC_DES_CBC)
			{
				if (PGPclMessageBox (hwnd, IDS_CAPTIONWARNING, IDS_DESWARN,
						MB_OKCANCEL|MB_DEFBUTTON2|MB_ICONEXCLAMATION) == IDOK)
				{
					EndDialog (hwnd, 1);
				}
			}
			else
				EndDialog (hwnd, 1);

			break;
		}

		case IDCANCEL :
			EndDialog (hwnd, 0);
			break;
		}
		break;
	}

	return FALSE;
}


//	_____________________________________________________
//
//  display IKE proposal dialog

static VOID
sNewIkeProposalDialog (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds)
{
	PGPNetPrefIKEProposalEntry		ike;

	ike.authMethod		= kPGPike_AM_PreSharedKey;
	ike.hash			= kPGPike_HA_SHA1;
	ike.cipher			= kPGPike_SC_CAST_CBC;
	ike.groupID			= kPGPike_GR_MODPTwo;

	if (DialogBoxParam (g_hInst, MAKEINTRESOURCE(IDD_IKEPROPOSAL),
				hwnd, sIkeProposalDlgProc, (LPARAM)&ike))
	{
		PGPNetPrefIKEProposalEntry*	pIke;
		PGPSize						size;
		UINT						uNumProposals;

		uNumProposals = ppds->net.uNumIKEProposals +1;
		size = uNumProposals * sizeof(PGPNetPrefIKEProposalEntry);

		pIke = PGPNewData (PGPPeekContextMemoryMgr (ppds->context),
										size, kPGPMemoryMgrFlags_Clear);
		if (pIke)
		{
			ppds->net.uNumIKEProposals = uNumProposals;

			pgpCopyMemory (&ike, pIke, sizeof(PGPNetPrefIKEProposalEntry));
			pgpCopyMemory (ppds->net.pIKEProposals, &pIke[1],
					(uNumProposals-1)*sizeof(PGPNetPrefIKEProposalEntry));

			PGPFreeData (ppds->net.pIKEProposals);
			ppds->net.pIKEProposals = pIke;

			sDisplayIkeProposalList (hwnd, ppds);

			sSelectItem (hwnd, IDC_IKEPROPOSALLIST, 0);
			ppds->net.iSelectedIkeItem = 0;
		}
	}
}


//	____________________________________
//
//	dialog proc of IPSEC proposal dialog

static BOOL CALLBACK
sIpsecProposalDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PGPNetPrefIPSECProposalEntry*	pIpsec;

	switch (uMsg) {
	case WM_INITDIALOG :
	{
		CHAR	sz[32];
		HWND	hwndCombo;
		UINT	u;

		SetWindowLong (hwnd, GWL_USERDATA, lParam);
		pIpsec = (PGPNetPrefIPSECProposalEntry*)
				GetWindowLong (hwnd, GWL_USERDATA);

		// load combo box strings
		hwndCombo = GetDlgItem (hwnd, IDC_IPSECAHCOMBO);
		LoadString (g_hInst, IDS_SHA, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_MD5, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);

		hwndCombo = GetDlgItem (hwnd, IDC_IPSECESPCIPHERCOMBO);
		LoadString (g_hInst, IDS_NONE, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_CAST, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_3DES, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);

		hwndCombo = GetDlgItem (hwnd, IDC_IPSECESPHASHCOMBO);
		LoadString (g_hInst, IDS_NONE, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_SHA, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_MD5, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);

		hwndCombo = GetDlgItem (hwnd, IDC_IPSECCOMPCOMBO);
		LoadString (g_hInst, IDS_LZS, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_DEFLATE, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);

		// initialize check boxes
		if (pIpsec->useAH)
		{
			CheckDlgButton (hwnd, IDC_ENABLEAH, BST_CHECKED);
			EnableWindow (GetDlgItem (hwnd, IDC_IPSECAHCOMBO), TRUE);
		}

		if (pIpsec->useESP)
		{
			CheckDlgButton (hwnd, IDC_ENABLEESP, BST_CHECKED);
			EnableWindow (GetDlgItem (hwnd, IDC_IPSECESPCIPHERCOMBO), TRUE);
			EnableWindow (GetDlgItem (hwnd, IDC_IPSECESPHASHCOMBO), TRUE);
		}

		if (pIpsec->useIPCOMP)
		{
			CheckDlgButton (hwnd, IDC_ENABLECOMP, BST_CHECKED);
			EnableWindow (GetDlgItem (hwnd, IDC_IPSECCOMPCOMBO), TRUE);
		}

		// initialize combo boxes
		switch (pIpsec->ahHash) {
			case kPGPike_AA_HMAC_SHA :		u = 0;	break;
			case kPGPike_AA_HMAC_MD5 :		u = 1;	break;
			default :						u = 0;	break;
		}
		SendDlgItemMessage (hwnd, IDC_IPSECAHCOMBO, CB_SETCURSEL, u, 0);

		switch (pIpsec->espCipher) {
			case kPGPike_ET_NULL :			u = 0;	break;
			case kPGPike_ET_CAST :			u = 1;	break;
			case kPGPike_ET_3DES :			u = 2;	break;
			case kPGPike_ET_DES :			u = 3;	break;
			case kPGPike_ET_DES_IV64 :		u = 3;	break;
			default :						u = 0;	break;
		}
		SendDlgItemMessage (
					hwnd, IDC_IPSECESPCIPHERCOMBO, CB_SETCURSEL, u, 0);

		switch (pIpsec->espHash) {
			case kPGPike_AA_None :			u = 0;	break;
			case kPGPike_AA_HMAC_SHA :		u = 1;	break;
			case kPGPike_AA_HMAC_MD5 :		u = 2;	break;
			default :						u = 0;	break;
		}
		SendDlgItemMessage (hwnd, IDC_IPSECESPHASHCOMBO, CB_SETCURSEL, u, 0);

		switch (pIpsec->ipcomp) {
			case kPGPike_IC_LZS :			u = 0;	break;
			case kPGPike_IC_Deflate :		u = 1;	break;
			default :						u = 0;	break;
		}
		SendDlgItemMessage (hwnd, IDC_IPSECCOMPCOMBO, CB_SETCURSEL, u, 0);

		return FALSE;
	}

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclNetHelpFile, aIpsecIds);

	case WM_COMMAND :
		pIpsec = (PGPNetPrefIPSECProposalEntry*)
				GetWindowLong (hwnd, GWL_USERDATA);

		switch (LOWORD (wParam)) {

		case IDC_ENABLEAH :
			if (IsDlgButtonChecked (hwnd, IDC_ENABLEAH) == BST_CHECKED)
				EnableWindow (GetDlgItem (hwnd, IDC_IPSECAHCOMBO), TRUE);
			else
				EnableWindow (GetDlgItem (hwnd, IDC_IPSECAHCOMBO), FALSE);
			break;

		case IDC_ENABLEESP :
			if (IsDlgButtonChecked (hwnd, IDC_ENABLEESP) == BST_CHECKED)
			{
				EnableWindow (GetDlgItem (
								hwnd, IDC_IPSECESPHASHCOMBO), TRUE);
				EnableWindow (GetDlgItem (
								hwnd, IDC_IPSECESPCIPHERCOMBO), TRUE);
			}
			else
			{
				EnableWindow (GetDlgItem (
								hwnd, IDC_IPSECESPHASHCOMBO), FALSE);
				EnableWindow (GetDlgItem (
								hwnd, IDC_IPSECESPCIPHERCOMBO), FALSE);
			}
			break;

		case IDC_ENABLECOMP :
			if (IsDlgButtonChecked (hwnd, IDC_ENABLECOMP) == BST_CHECKED)
				EnableWindow (GetDlgItem (hwnd, IDC_IPSECCOMPCOMBO), TRUE);
			else
				EnableWindow (GetDlgItem (hwnd, IDC_IPSECCOMPCOMBO), FALSE);
			break;

		case IDCANCEL :
			EndDialog (hwnd, 0);
			break;

		case IDOK :
		{
			UINT	u;

			if (IsDlgButtonChecked (hwnd, IDC_ENABLEAH) == BST_CHECKED)
			{
				pIpsec->useAH = TRUE;

				u = SendDlgItemMessage (
							hwnd, IDC_IPSECAHCOMBO, CB_GETCURSEL, 0, 0);
				switch (u) {
				case 0	:
					pIpsec->ahAuthAlg = kPGPike_AH_SHA;
					pIpsec->ahHash = kPGPike_AA_HMAC_SHA;
					break;

				case 1	:
					pIpsec->ahAuthAlg = kPGPike_AH_MD5;
					pIpsec->ahHash = kPGPike_AA_HMAC_MD5;
					break;
				}
			}
			else
			{
				pIpsec->useAH = FALSE;
				pIpsec->ahAuthAlg = kPGPike_AH_None;
				pIpsec->ahHash = kPGPike_AA_None;
			}

			if (IsDlgButtonChecked (hwnd, IDC_ENABLEESP) == BST_CHECKED)
			{
				pIpsec->useESP = TRUE;

				u = SendDlgItemMessage (
						hwnd, IDC_IPSECESPCIPHERCOMBO, CB_GETCURSEL, 0, 0);
				switch (u) {
				case 0	:	pIpsec->espCipher = kPGPike_ET_NULL;		break;
				case 1	:	pIpsec->espCipher = kPGPike_ET_CAST;		break;
				case 2	:	pIpsec->espCipher = kPGPike_ET_3DES;		break;
				case 3	:	pIpsec->espCipher = kPGPike_ET_DES;			break;
				}

				u = SendDlgItemMessage (
						hwnd, IDC_IPSECESPHASHCOMBO, CB_GETCURSEL, 0, 0);
				switch (u) {
				case 0	:	pIpsec->espHash = kPGPike_AA_None;		break;
				case 1	:	pIpsec->espHash = kPGPike_AA_HMAC_SHA;	break;
				case 2	:	pIpsec->espHash = kPGPike_AA_HMAC_MD5;	break;
				}
			}
			else
			{
				pIpsec->useESP = FALSE;
				pIpsec->espCipher = kPGPike_ET_NULL;
				pIpsec->espHash = kPGPike_AA_None;
			}

			if (IsDlgButtonChecked (hwnd, IDC_ENABLECOMP) == BST_CHECKED)
			{
				pIpsec->useIPCOMP = TRUE;

				u = SendDlgItemMessage (
						hwnd, IDC_IPSECCOMPCOMBO, CB_GETCURSEL, 0, 0);
				switch (u) {
				case 0	:	pIpsec->ipcomp = kPGPike_IC_LZS;		break;
				case 1	:	pIpsec->ipcomp = kPGPike_IC_Deflate;	break;
				}
			}
			else
			{
				pIpsec->useIPCOMP = FALSE;
				pIpsec->ipcomp = kPGPike_IC_None;
			}

			// check for valid combinations
			if ((!pIpsec->useAH) &&
				(!pIpsec->useESP))
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONINFO, 
						IDS_NEEDAHORESP, MB_OK|MB_ICONINFORMATION);
				break;
			}

			if ((pIpsec->useESP) &&
				(pIpsec->espCipher == kPGPike_ET_NULL) &&
				(pIpsec->espHash == kPGPike_AA_None))
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONINFO, 
						IDS_ESPNEEDSALGORITHM, MB_OK|MB_ICONINFORMATION);
				break;
			}

			// warn user about DES
			if ((pIpsec->espCipher == kPGPike_ET_DES_IV64) ||
				(pIpsec->espCipher == kPGPike_ET_DES))
			{
				if (PGPclMessageBox (hwnd, IDS_CAPTIONWARNING, IDS_DESWARN,
						MB_OKCANCEL|MB_DEFBUTTON2|MB_ICONEXCLAMATION) == IDOK)
				{
					EndDialog (hwnd, 1);
				}
			}
			else
				EndDialog (hwnd, 1);

			break;
		}
		}
		break;
	}

	return FALSE;
}


//	_____________________________________________________
//
//  display IPSEC proposal dialog

static VOID
sNewIpsecProposalDialog (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds)
{
	PGPNetPrefIPSECProposalEntry	ipsec;

	ipsec.useAH				= FALSE;
	ipsec.ahAuthAlg			= kPGPike_AH_MD5;
	ipsec.ahHash			= kPGPike_AA_HMAC_SHA;
	ipsec.useESP			= TRUE;
	ipsec.espCipher			= kPGPike_ET_CAST;
	ipsec.espHash			= kPGPike_AA_HMAC_SHA;
	ipsec.useIPCOMP			= FALSE;
	ipsec.ipcomp			= kPGPike_IC_None;

	if (DialogBoxParam (g_hInst, MAKEINTRESOURCE(IDD_IPSECPROPOSAL),
				hwnd, sIpsecProposalDlgProc, (LPARAM)&ipsec))
	{
		PGPNetPrefIPSECProposalEntry*	pIpsec;
		PGPSize							size;
		UINT							uNumProposals;

		uNumProposals = ppds->net.uNumIPSECProposals +1;
		size = uNumProposals * sizeof(PGPNetPrefIPSECProposalEntry);

		pIpsec = PGPNewData (PGPPeekContextMemoryMgr (ppds->context),
										size, kPGPMemoryMgrFlags_Clear);
		if (pIpsec)
		{
			ppds->net.uNumIPSECProposals = uNumProposals;

			pgpCopyMemory (&ipsec,
					pIpsec, sizeof(PGPNetPrefIPSECProposalEntry));
			pgpCopyMemory (ppds->net.pIPSECProposals, &pIpsec[1],
					(uNumProposals-1)*sizeof(PGPNetPrefIPSECProposalEntry));

			PGPFreeData (ppds->net.pIPSECProposals);
			ppds->net.pIPSECProposals = pIpsec;

			sDisplayIpsecProposalList (hwnd, ppds);

			sSelectItem (hwnd, IDC_IPSECPROPOSALLIST, 0);
			ppds->net.iSelectedIpsecItem = 0;
		}
	}
}


//	_____________________________________________________
//
//  post "New" button context menu

static VOID
sNewProposalContextMenu (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds)
{
	HMENU			hMC;
	HMENU			hMenuTrackPopup;
	RECT			rc;

	hMC = LoadMenu (g_hInst, MAKEINTRESOURCE (IDR_MENUNEWPROPOSAL));
	hMenuTrackPopup = GetSubMenu (hMC, 0);

	if ((ppds->net.uNumIKEProposals >= MAXPROPOSALS) ||
		(ppds->net.bIKEProposalsDisabled))
	{
		EnableMenuItem (hMenuTrackPopup, IDM_NEWIKEPROPOSAL,
							MF_BYCOMMAND | MF_GRAYED);
	}

	if ((ppds->net.uNumIPSECProposals >= MAXPROPOSALS) ||
		(ppds->net.bIPSECProposalsDisabled))
	{
		EnableMenuItem (hMenuTrackPopup, IDM_NEWIPSECPROPOSAL,
							MF_BYCOMMAND | MF_GRAYED);
	}

	GetWindowRect (GetDlgItem (hwnd, IDC_NEWPROPOSAL), &rc);
	TrackPopupMenu (hMenuTrackPopup, TPM_LEFTALIGN | TPM_RIGHTBUTTON,
					rc.left, rc.bottom, 0, hwnd, NULL);

	DestroyMenu (hMC);
}


//	_____________________________________________________
//
//  set proposal button states

static VOID
sEnableProposalButtons (
		HWND			hwnd,
		BOOL			bEnable,
		PPREFSDLGSTRUCT	ppds)
{
	INT		iSelectedItem	= -1;
	HWND	hwndCtrl;
	INT		iNumItems;

	if (ppds->net.iFocusedProposalList == IDC_IKEPROPOSALLIST)
		bEnable &= !ppds->net.bIKEProposalsDisabled;
	else if (ppds->net.iFocusedProposalList == IDC_IPSECPROPOSALLIST)
		bEnable &= !ppds->net.bIPSECProposalsDisabled;

	if (bEnable)
	{
		EnableWindow (GetDlgItem (hwnd, IDC_EDITPROPOSAL), TRUE);

		if (ppds->net.iFocusedProposalList == IDC_IKEPROPOSALLIST)
		{
			iNumItems = ppds->net.uNumIKEProposals;
			iSelectedItem = ppds->net.iSelectedIkeItem;
		}
		else if (ppds->net.iFocusedProposalList == IDC_IPSECPROPOSALLIST)
		{
			iNumItems = ppds->net.uNumIPSECProposals;
			iSelectedItem = ppds->net.iSelectedIpsecItem;
		}

		if (iSelectedItem >= 0)
		{
			EnableWindow (GetDlgItem (hwnd, IDC_REMOVEPROPOSAL),
								(iNumItems > 1));

			hwndCtrl = GetDlgItem (hwnd, IDC_PROPOSALUP);
			if (iSelectedItem > 0)
				EnableWindow (hwndCtrl, TRUE);
			else
			{
				if (GetFocus () == hwndCtrl)
				{
					SetFocus (GetDlgItem (hwnd, IDC_PROPOSALDOWN));
					SendMessage (hwnd, DM_SETDEFID, IDC_PROPOSALDOWN, 0);
				}
				EnableWindow (hwndCtrl, FALSE);
			}

			hwndCtrl = GetDlgItem (hwnd, IDC_PROPOSALDOWN);
			if (iSelectedItem < iNumItems-1)
				EnableWindow (hwndCtrl, TRUE);
			else
			{
				if (GetFocus () == hwndCtrl)
				{
					SetFocus (GetDlgItem (hwnd, IDC_PROPOSALUP));
					SendMessage (hwnd, DM_SETDEFID, IDC_PROPOSALUP, 0);
				}
				EnableWindow (hwndCtrl, FALSE);
			}
		}
	}
	else
	{
		EnableWindow (GetDlgItem (hwnd, IDC_EDITPROPOSAL), FALSE);
		EnableWindow (GetDlgItem (hwnd, IDC_REMOVEPROPOSAL), FALSE);
		EnableWindow (GetDlgItem (hwnd, IDC_PROPOSALUP), FALSE);
		EnableWindow (GetDlgItem (hwnd, IDC_PROPOSALDOWN), FALSE);
	}
}


//	_____________________________________________________
//
//  edit the selected proposal

static VOID
sEditSelectedProposal (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds)
{
	if ((ppds->net.iFocusedProposalList == IDC_IKEPROPOSALLIST) &&
		(!ppds->net.bIKEProposalsDisabled))
	{
		PGPNetPrefIKEProposalEntry		ike;
		PGPNetPrefIKEProposalEntry*		pCurrentIke;

		pCurrentIke = &ppds->net.pIKEProposals[ppds->net.iSelectedIkeItem];
		pgpCopyMemory (pCurrentIke,
				&ike, sizeof(PGPNetPrefIKEProposalEntry));

		if (DialogBoxParam (g_hInst, MAKEINTRESOURCE(IDD_IKEPROPOSAL),
				hwnd, sIkeProposalDlgProc, (LPARAM)&ike))
		{
			pgpCopyMemory (&ike,
					pCurrentIke, sizeof(PGPNetPrefIKEProposalEntry));
			sDisplayIkeProposalList (hwnd, ppds);
		}

		sSelectItem (hwnd,
				ppds->net.iFocusedProposalList, ppds->net.iSelectedIkeItem);
	}

	else if ((ppds->net.iFocusedProposalList == IDC_IPSECPROPOSALLIST) &&
		(!ppds->net.bIPSECProposalsDisabled))
	{
		PGPNetPrefIPSECProposalEntry	ipsec;
		PGPNetPrefIPSECProposalEntry*	pCurrentIpsec;

		pCurrentIpsec = &ppds->net.pIPSECProposals[ppds->net.iSelectedIpsecItem];
		pgpCopyMemory (pCurrentIpsec,
				&ipsec, sizeof(PGPNetPrefIPSECProposalEntry));

		if (DialogBoxParam (g_hInst, MAKEINTRESOURCE(IDD_IPSECPROPOSAL),
				hwnd, sIpsecProposalDlgProc, (LPARAM)&ipsec))
		{
			pgpCopyMemory (&ipsec,
					pCurrentIpsec, sizeof(PGPNetPrefIPSECProposalEntry));
			sDisplayIpsecProposalList (hwnd, ppds);
		}

		sSelectItem (hwnd,
				ppds->net.iFocusedProposalList, ppds->net.iSelectedIpsecItem);
	}
}


//	_____________________________________________________
//
//  remove the selected proposal

static VOID
sRemoveSelectedProposal (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds)
{
	UINT	u;

	if ((ppds->net.iFocusedProposalList == IDC_IKEPROPOSALLIST) &&
		(!ppds->net.bIKEProposalsDisabled))
	{
		PGPNetPrefIKEProposalEntry*		pIkeSrc;
		PGPNetPrefIKEProposalEntry*		pIkeDst;

		pIkeDst = &ppds->net.pIKEProposals[ppds->net.iSelectedIkeItem];

		for (u=ppds->net.iSelectedIkeItem; u<ppds->net.uNumIKEProposals-1; u++)
		{
			pIkeSrc = &ppds->net.pIKEProposals[u+1];

			pgpCopyMemory (pIkeSrc,
					pIkeDst, sizeof(PGPNetPrefIKEProposalEntry));
			pIkeDst = pIkeSrc;
		}

		ppds->net.uNumIKEProposals--;
		sDisplayIkeProposalList (hwnd, ppds);

		ppds->net.iSelectedIkeItem = -1;
		sEnableProposalButtons (hwnd, FALSE, ppds);
	}

	else if ((ppds->net.iFocusedProposalList == IDC_IPSECPROPOSALLIST) &&
		(!ppds->net.bIPSECProposalsDisabled))
	{
		PGPNetPrefIPSECProposalEntry*	pIpsecSrc;
		PGPNetPrefIPSECProposalEntry*	pIpsecDst;

		pIpsecDst = &ppds->net.pIPSECProposals[ppds->net.iSelectedIpsecItem];

		for (u=ppds->net.iSelectedIpsecItem; u<ppds->net.uNumIPSECProposals-1; u++)
		{
			pIpsecSrc = &ppds->net.pIPSECProposals[u+1];

			pgpCopyMemory (pIpsecSrc,
					pIpsecDst, sizeof(PGPNetPrefIPSECProposalEntry));
			pIpsecDst = pIpsecSrc;
		}

		ppds->net.uNumIPSECProposals--;
		sDisplayIpsecProposalList (hwnd, ppds);

		ppds->net.iSelectedIpsecItem = -1;
		sEnableProposalButtons (hwnd, FALSE, ppds);
	}
}


//	_____________________________________________________
//
//  set proposal button states

static VOID
sMoveSelectedProposal (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds,
		INT				iDir)
{
	if ((ppds->net.iFocusedProposalList == IDC_IKEPROPOSALLIST) &&
		(!ppds->net.bIKEProposalsDisabled))
	{
		PGPNetPrefIKEProposalEntry		ike;
		PGPNetPrefIKEProposalEntry*		pIkeSrc;
		PGPNetPrefIKEProposalEntry*		pIkeDst;

		pIkeSrc = &ppds->net.pIKEProposals[ppds->net.iSelectedIkeItem];
		pgpCopyMemory (
				pIkeSrc, &ike, sizeof(PGPNetPrefIKEProposalEntry));

		pIkeDst = &ppds->net.pIKEProposals[ppds->net.iSelectedIkeItem-iDir];
		pgpCopyMemory (
				pIkeDst, pIkeSrc, sizeof(PGPNetPrefIKEProposalEntry));
		pgpCopyMemory (
				&ike, pIkeDst, sizeof(PGPNetPrefIKEProposalEntry));

		ppds->net.iSelectedIkeItem -= iDir;
		sDisplayIkeProposalList (hwnd, ppds);
		sEnableProposalButtons (hwnd, TRUE, ppds);

		sSelectItem (hwnd,
				ppds->net.iFocusedProposalList, ppds->net.iSelectedIkeItem);
	}

	else if ((ppds->net.iFocusedProposalList == IDC_IPSECPROPOSALLIST) &&
		(!ppds->net.bIPSECProposalsDisabled))
	{
		PGPNetPrefIPSECProposalEntry	ipsec;
		PGPNetPrefIPSECProposalEntry*	pIpsecSrc;
		PGPNetPrefIPSECProposalEntry*	pIpsecDst;

		pIpsecSrc = &ppds->net.pIPSECProposals[ppds->net.iSelectedIpsecItem];
		pgpCopyMemory (
				pIpsecSrc, &ipsec, sizeof(PGPNetPrefIPSECProposalEntry));

		pIpsecDst = &ppds->net.pIPSECProposals[ppds->net.iSelectedIpsecItem-iDir];

		pgpCopyMemory (
				pIpsecDst, pIpsecSrc, sizeof(PGPNetPrefIPSECProposalEntry));
		pgpCopyMemory (
				&ipsec, pIpsecDst, sizeof(PGPNetPrefIPSECProposalEntry));

		ppds->net.iSelectedIpsecItem -= iDir;
		sDisplayIpsecProposalList (hwnd, ppds);
		sEnableProposalButtons (hwnd, TRUE, ppds);

		sSelectItem (hwnd,
				ppds->net.iFocusedProposalList, ppds->net.iSelectedIpsecItem);
	}
}


//	_____________________________________________________
//
//  process host list context menu

static VOID
sContextMenu (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds,
		INT				iX,
		INT				iY)
{
	INT				iSelectedItem		= -1;
	HWND			hwndList			= NULL;

	INT				iNumItems;
	HMENU			hMC;
	HMENU			hMenuTrackPopup;
	MENUITEMINFO	mii;
	LVITEM			lvi;

	mii.cbSize		= sizeof(MENUITEMINFO);
	mii.fMask		= MIIM_ID;

	lvi.mask		= LVIF_STATE;
	lvi.iSubItem	= 0;
	lvi.stateMask	= LVIS_SELECTED;

	hMC = LoadMenu (g_hInst, MAKEINTRESOURCE (IDR_MENUPROPOSALCONTEXT));
	hMenuTrackPopup = GetSubMenu (hMC, 0);

	if (ppds->net.iFocusedProposalList == IDC_IKEPROPOSALLIST)
	{
		if (ppds->net.bIKEProposalsDisabled)
			return;

		mii.wID = IDM_NEWIKEPROPOSAL;
		iNumItems = ppds->net.uNumIKEProposals;

		lvi.iItem = ppds->net.iSelectedIkeItem;
		hwndList = GetDlgItem (hwnd, IDC_IKEPROPOSALLIST);
		ListView_GetItem (hwndList, &lvi);
		if (lvi.state == LVIS_SELECTED)
			iSelectedItem = ppds->net.iSelectedIkeItem;
	}
	else if (ppds->net.iFocusedProposalList == IDC_IPSECPROPOSALLIST)
	{
		if (ppds->net.bIPSECProposalsDisabled)
			return;

		mii.wID = IDM_NEWIPSECPROPOSAL;
		iNumItems = ppds->net.uNumIPSECProposals;

		lvi.iItem = ppds->net.iSelectedIpsecItem;
		hwndList = GetDlgItem (hwnd, IDC_IPSECPROPOSALLIST);
		ListView_GetItem (hwndList, &lvi);
		if (lvi.state == LVIS_SELECTED)
			iSelectedItem = ppds->net.iSelectedIpsecItem;
	}

	if (iNumItems < MAXPROPOSALS)
	{
		EnableMenuItem (hMenuTrackPopup, IDC_NEWPROPOSAL,
								MF_BYCOMMAND|MF_ENABLED);
		SetMenuItemInfo (hMenuTrackPopup, IDC_NEWPROPOSAL, FALSE, &mii);
	}

	if (iSelectedItem >= 0)
	{
		EnableMenuItem (hMenuTrackPopup, IDC_EDITPROPOSAL,
									MF_BYCOMMAND|MF_ENABLED);

		if (iNumItems > 1)
			EnableMenuItem (hMenuTrackPopup, IDC_REMOVEPROPOSAL,
									MF_BYCOMMAND|MF_ENABLED);

		if (iSelectedItem > 0)
			EnableMenuItem (hMenuTrackPopup, IDC_PROPOSALUP,
									MF_BYCOMMAND|MF_ENABLED);

		if (iSelectedItem < iNumItems-1)
			EnableMenuItem (hMenuTrackPopup, IDC_PROPOSALDOWN,
									MF_BYCOMMAND|MF_ENABLED);
	}

	if ((iX == 0xFFFF) && (iY == 0xFFFF))
	{
		if (hwndList)
		{
			POINT	pt;

			ListView_GetItemPosition (hwndList, max (0, iSelectedItem), &pt);
			MapWindowPoints (hwndList, NULL, &pt, 1);
			iX = pt.x;
			iY = pt.y;
		}
	}

	TrackPopupMenu (hMenuTrackPopup, TPM_LEFTALIGN | TPM_RIGHTBUTTON,
					iX, iY, 0, hwnd, NULL);

	DestroyMenu (hMC);
}


//	_____________________________________________________
//
//  draw listview item

static VOID
sDrawItem (
		LPDRAWITEMSTRUCT	pdis)
{
	RECT			rc;
	RECT			rcAll;
	RECT			rcLabel;
	RECT			rcDraw;

	BOOL			bFocus = (GetFocus() == pdis->hwndItem);
	BOOL			bSelected;

	COLORREF		clrTextSave, clrBkSave;

	static CHAR		sz[128];

	LV_COLUMN		lvc;
	LVITEM			lvi;
	UINT			nJustify = DT_LEFT;
	INT				iCol;

	// get item data
	lvi.mask		= LVIF_TEXT | LVIF_STATE;
	lvi.iItem		= pdis->itemID;
	lvi.iSubItem	= 0;
	lvi.pszText		= sz;
	lvi.cchTextMax	= sizeof(sz);
	lvi.stateMask	= 0xFFFF;     // get all state flags
	ListView_GetItem (pdis->hwndItem, &lvi);

	bSelected = lvi.state & LVIS_SELECTED;

	ListView_GetItemRect (pdis->hwndItem, pdis->itemID, &rcAll, LVIR_BOUNDS);
	ListView_GetItemRect (pdis->hwndItem, pdis->itemID, &rcLabel, LVIR_LABEL);
	rcAll.left = rcLabel.left;

	// set colors if item is selected
	if (bSelected && bFocus)
	{
		clrTextSave = SetTextColor(pdis->hDC, GetSysColor(COLOR_HIGHLIGHTTEXT));
		clrBkSave = SetBkColor(pdis->hDC, GetSysColor(COLOR_HIGHLIGHT));

		FillRect (pdis->hDC, &rcAll, (HBRUSH)(COLOR_HIGHLIGHT+1));
	}
	else if (bSelected && !bFocus)
	{
		clrTextSave = SetTextColor(pdis->hDC, GetSysColor(COLOR_BTNTEXT));
		clrBkSave = SetBkColor(pdis->hDC, GetSysColor(COLOR_BTNFACE));

		FillRect (pdis->hDC, &rcAll, (HBRUSH)(COLOR_BTNFACE+1));
	}
	else
		FillRect (pdis->hDC, &rcAll, (HBRUSH)(COLOR_WINDOW+1));

	// draw item label
	rcDraw = rcLabel;
	rcDraw.left += OFFSET_FIRST;
	rcDraw.right -= OFFSET_FIRST;

	DrawText (pdis->hDC, sz, -1, &rcDraw,
		DT_LEFT | DT_SINGLELINE | DT_NOPREFIX | DT_NOCLIP | DT_VCENTER |
		DT_END_ELLIPSIS);

	// draw labels for extra columns
	rc = rcLabel;
	lvc.mask = LVCF_FMT | LVCF_WIDTH;
	for (iCol = 1; ListView_GetColumn (pdis->hwndItem, iCol, &lvc); iCol++)
	{
		rc.left = rc.right;
		rc.right += lvc.cx;

		ListView_GetItemText (pdis->hwndItem, pdis->itemID, iCol,
						sz, sizeof(sz));
		if (lstrlen (sz) == 0)
			continue;

		switch (lvc.fmt & LVCFMT_JUSTIFYMASK)
		{
		case LVCFMT_RIGHT:
			nJustify = DT_RIGHT;
			break;
		case LVCFMT_CENTER:
			nJustify = DT_CENTER;
			break;
		default:
			break;
		}

		rcDraw = rc;
		rcDraw.left += OFFSET_OTHER;
		rcDraw.right -= OFFSET_OTHER;

		DrawText (pdis->hDC, sz, -1, &rcDraw,
			nJustify | DT_SINGLELINE | DT_NOPREFIX | DT_NOCLIP | DT_VCENTER |
			DT_END_ELLIPSIS);
	}

	// draw focus rectangle if item has focus
	if (lvi.state & LVIS_FOCUSED && bFocus && bSelected)
		DrawFocusRect (pdis->hDC, &rcAll);

	// set original colors if item was selected
	if (bSelected)
	{
		SetTextColor (pdis->hDC, clrTextSave);
		SetBkColor (pdis->hDC, clrBkSave);
	}
}


//	____________________________________
//
//	load default advanced prefs by opening a dummy file
//	and reading the prefs.

static PGPError
sSetDefaultAdvancedPrefs (
		HWND			hwnd,
		PPREFSDLGSTRUCT	ppds)
{
	PFLFileSpecRef		fileref		= NULL;
	PGPPrefRef			prefref		= kInvalidPGPPrefRef;
	PGPError			err;

	err = PFLGetTempFileSpec (PGPPeekContextMemoryMgr (ppds->context),
			NULL, &fileref); CKERR;

	err = PGPOpenPrefFile (fileref, netDefaults,
			netDefaultsSize, &prefref); CKERR;

	sInitAllowedAlgorithms (hwnd, ppds, prefref);
	if (!ppds->net.bIKEProposalsDisabled)
		sLoadIkeProposalList (hwnd, ppds, prefref);
	if (!ppds->net.bIPSECProposalsDisabled)
		sLoadIpsecProposalList (hwnd, ppds, prefref);
	sLoadIpsecGroupID (hwnd, ppds, prefref);

done :
	if (PGPPrefRefIsValid (prefref))
		PGPFreePrefs (prefref);
	if (IsntNull (fileref))
		PFLFreeFileSpec (fileref);

	return err;
}



//	_____________________________________________________
//
//  Message processing function for PGPnet advanced prefs property sheet

LRESULT WINAPI
CLNetAdvPropDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PPREFSDLGSTRUCT			ppds;

	switch (uMsg) {
	case WM_INITDIALOG :
	{
		HWND		hwndCombo;
		CHAR		sz[64];
		PGPUInt32	u;
		PGPPrefRef	prefrefNet;

		SetWindowLong (hwnd, GWL_USERDATA, ((PROPSHEETPAGE*)lParam)->lParam);
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		if (ppds->bNeedsCentering)
		{
			PGPclCenterWindowOnMonitor (GetParent (hwnd), kPGPclWindowMonitor);
			ppds->bNeedsCentering = FALSE;
		}

		prefrefNet = CLGetPrefRefNet (ppds);

		ppds->net.iSelectedIkeItem = 0;
		ppds->net.iSelectedIpsecItem = 0;
		ppds->net.iFocusedProposalList = 0;

		hwndCombo = GetDlgItem (hwnd, IDC_PERFECTSECRECY);
		LoadString (g_hInst, IDS_NONE, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_1024, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);
		LoadString (g_hInst, IDS_1536, sz, sizeof(sz));
		SendMessage (hwndCombo, CB_ADDSTRING, 0, (LPARAM)sz);

		sInitAllowedAlgorithms (hwnd, ppds, prefrefNet);
		sInitIkeProposalListControl (hwnd);
		sLoadIkeProposalList (hwnd, ppds, prefrefNet);
		sInitIpsecProposalListControl (hwnd);
		sLoadIpsecProposalList (hwnd, ppds, prefrefNet);
		sLoadIpsecGroupID (hwnd, ppds, prefrefNet);

		// disable controls if proposal lists are locked
		PGPGetPrefFlags (prefrefNet, kPGPNetPrefIKEProposalArray, &u);
		ppds->net.bIKEProposalsDisabled = u & kPGPPrefFlags_DisableGUI;

		PGPGetPrefFlags (prefrefNet, kPGPNetPrefIPSecProposalArray, &u);
		ppds->net.bIPSECProposalsDisabled = u & kPGPPrefFlags_DisableGUI;

		if (ppds->net.bIKEProposalsDisabled &&
			ppds->net.bIPSECProposalsDisabled)
		{
			EnableWindow (GetDlgItem (hwnd, IDC_NEWPROPOSAL), FALSE);
		}

		return FALSE;
	}

	case WM_NOTIFY :
	{
		LPNMHDR pnmhdr = (LPNMHDR)lParam;
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		if ((pnmhdr->idFrom == IDC_IKEPROPOSALLIST) ||
			(pnmhdr->idFrom == IDC_IPSECPROPOSALLIST))
		{
			switch (pnmhdr->code) {
			case NM_SETFOCUS :
				ppds->net.iFocusedProposalList = pnmhdr->idFrom;

				switch (pnmhdr->idFrom) {
				case IDC_IKEPROPOSALLIST :
					ListView_SetItemState (pnmhdr->hwndFrom,
						ppds->net.iSelectedIkeItem, LVIS_SELECTED, LVIS_SELECTED);
					ListView_SetItemState (
						GetDlgItem (hwnd, IDC_IPSECPROPOSALLIST),
						ppds->net.iSelectedIpsecItem, 0, LVIS_SELECTED);
					break;

				case IDC_IPSECPROPOSALLIST :
					ListView_SetItemState (pnmhdr->hwndFrom,
						ppds->net.iSelectedIpsecItem, LVIS_SELECTED, LVIS_SELECTED);
					ListView_SetItemState (
						GetDlgItem (hwnd, IDC_IKEPROPOSALLIST),
						ppds->net.iSelectedIkeItem, 0, LVIS_SELECTED);
					break;
				}

				sEnableProposalButtons (hwnd, TRUE, ppds);
				break;

			case LVN_ITEMCHANGED :
			{
				NM_LISTVIEW* pnmlv = (NM_LISTVIEW*)lParam;

				if (pnmlv->iItem >= 0)
				{
					if (pnmlv->uNewState & LVIS_SELECTED)
					{
						ppds->net.iFocusedProposalList = pnmhdr->idFrom;
						if (pnmhdr->idFrom == IDC_IKEPROPOSALLIST)
						{
							ppds->net.iSelectedIkeItem = pnmlv->iItem;
							ListView_SetItemState (
									GetDlgItem (hwnd, IDC_IPSECPROPOSALLIST),
									ppds->net.iSelectedIpsecItem,
									0, LVIS_SELECTED);
						}
						else {
							ppds->net.iSelectedIpsecItem = pnmlv->iItem;
							ListView_SetItemState (
									GetDlgItem (hwnd, IDC_IKEPROPOSALLIST),
									ppds->net.iSelectedIkeItem,
									0, LVIS_SELECTED);
						}
						sEnableProposalButtons (hwnd, TRUE, ppds);
					}
					else
					{
						sEnableProposalButtons (hwnd, FALSE, ppds);
					}
				}
				break;
			}

			case LVN_KEYDOWN :
				if (((LV_KEYDOWN*)lParam)->wVKey == VK_DELETE)
				{
					if (IsWindowEnabled (
							GetDlgItem (hwnd, IDC_REMOVEPROPOSAL)))
					{
						PostMessage (hwnd, WM_COMMAND, IDC_REMOVEPROPOSAL, 0);
					}
				}
				break;

			case NM_DBLCLK :
				PostMessage (hwnd, WM_COMMAND, IDC_EDITPROPOSAL, 0);
				break;
			}

			break;
		}

		switch (((NMHDR FAR *) lParam)->code) {
		case PSN_HELP :
			PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0,
					(char*)kPGPclNetHelpFile, 
					(DWORD*)IDH_PGPCLPREF_NETADVDIALOG);
			break;

		case PSN_APPLY :
			ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			if (ppds->bUseClientLibNetPrefs && 
				!PGPPrefRefIsValid (ppds->prefrefNet))
			{
				PGPclPeekClientLibPrefRefs (NULL, &ppds->prefrefNet);
			}

			sSaveAllowedAlgorithms (hwnd, ppds->prefrefNet);
			sSaveIpsecGroupID (hwnd, ppds->prefrefNet);

			PGPSetNetIKEProposalPrefs (ppds->prefrefNet,
					ppds->net.pIKEProposals, ppds->net.uNumIKEProposals);

			PGPSetNetIPSECProposalPrefs (ppds->prefrefNet,
					ppds->net.pIPSECProposals, ppds->net.uNumIPSECProposals);

			ppds->net.bPrefsModified = TRUE;
			ppds->returnErr = kPGPError_NoErr;
			SetWindowLong (hwnd, DWL_MSGRESULT, PSNRET_NOERROR);
			return TRUE;
		}
		break;
	}

	case WM_DESTROY :
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		if (ppds->net.pIKEProposals)
			PGPFreeData (ppds->net.pIKEProposals);
		if (ppds->net.pIPSECProposals)
			PGPFreeData (ppds->net.pIPSECProposals);
		break;

	case WM_DRAWITEM :
		sDrawItem ((LPDRAWITEMSTRUCT)lParam);
		break;

	case WM_HELP :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclNetHelpFile, aAdvIds);

	case WM_CONTEXTMENU :
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		if ((HWND)wParam == GetDlgItem (hwnd, IDC_IKEPROPOSALLIST) ||
			(HWND)wParam == GetDlgItem (hwnd, IDC_IPSECPROPOSALLIST))
		{
			sContextMenu (hwnd, ppds, 
					(SHORT)LOWORD(lParam), (SHORT)HIWORD(lParam));
		}
		else
		{
			return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
					(char*)kPGPclNetHelpFile, aAdvIds);
		}
		break;

	case WM_COMMAND :
		ppds = (PPREFSDLGSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		switch (LOWORD (wParam)) {

		case IDC_DEFAULTS :
			sSetDefaultAdvancedPrefs (hwnd, ppds);
			break;

		case IDC_ALLOWDES :
			if (IsDlgButtonChecked (hwnd, IDC_ALLOWDES) == BST_CHECKED)
			{
				if (PGPclMessageBox (hwnd, IDS_CAPTIONWARNING, IDS_DESWARN,
						MB_OKCANCEL|MB_DEFBUTTON2|MB_ICONEXCLAMATION) != IDOK)
				{
					CheckDlgButton (hwnd, IDC_ALLOWDES, BST_UNCHECKED);
				}
			}
			break;

		case IDC_NEWPROPOSAL :
			sNewProposalContextMenu (hwnd, ppds);
			break;

		case IDM_NEWIKEPROPOSAL :
			if (!ppds->net.bIKEProposalsDisabled)
				sNewIkeProposalDialog (hwnd, ppds);
			break;

		case IDM_NEWIPSECPROPOSAL :
			if (!ppds->net.bIPSECProposalsDisabled)
				sNewIpsecProposalDialog (hwnd, ppds);
			break;

		case IDC_EDITPROPOSAL :
			sEditSelectedProposal (hwnd, ppds);
			break;

		case IDC_REMOVEPROPOSAL :
			sRemoveSelectedProposal (hwnd, ppds);
			break;

		case IDC_PROPOSALUP :
			sMoveSelectedProposal (hwnd, ppds, 1);
			break;

		case IDC_PROPOSALDOWN :
			sMoveSelectedProposal (hwnd, ppds, -1);
			break;

		case IDM_PROPOSALHELP :
			if (ppds->net.iFocusedProposalList == IDC_IKEPROPOSALLIST)
			{
				PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0,
						(char*)kPGPclNetHelpFile, 
						(DWORD*)IDH_PGPCLPREF_IKEPROPOSALS);
			}
			else if (ppds->net.iFocusedProposalList == IDC_IPSECPROPOSALLIST)
			{
				PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0,
						(char*)kPGPclNetHelpFile, 
						(DWORD*)IDH_PGPCLPREF_IPSECPROPOSALS);
			}
			break;

		}
		return 0;
	}

	return FALSE;
}
