/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	CLprops.c - handle Key properties dialogs

	$Id: CLprops.c,v 1.57 2002/10/31 04:13:35 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "TreeList.h"
#include "pgpImageList.h"
#include "pgpHashWords.h"
#include "UTF8Edit.h"

// system header files
#include <process.h>

// constant definitions
#define ENFORCELISTWIDTH	52

#define MAXSHEETS			16		// max number of simultaneous dialogs

#define MINSUBKEYSIZE		1024
#define MAXSUBKEYSIZE		4096
#define DEFAULTSUBKEYSIZE	2048
#define ECSUBKEYSIZE		283

#define PROGRESS_TIMER		5432L
#define PROGRESS_TIMER_MS	100
#define NUMLEDS				10
#define LEDSPACING			2

#define NOTIFYTIMER			1111L
#define NOTIFYTIMEMS		100

#define NUMHASHWORDCOLS		4

#define TREELISTOFFSET		8

// external globals
extern HINSTANCE	g_hInst;
extern HIMAGELIST	g_hilKeys;
extern BOOL			g_bUnicodeOS;
// typedefs
typedef struct {
	PGPContextRef		context;
	PGPtlsContextRef	tlscontext;
	HWND				hwndParent;
	WNDPROC				wpOrigProcPhoto;
	WNDPROC				wpOrigProcBox;
	PGPKeyDBObjRef		key;
	PGPKeyDBObjRef		userid;
	PGPKeyDBObjRef		cert;
	PGPKeyDBRef			keydbBacking;
	PGPKeyDBRef			keydbAlt;

	PGPInt32			iValidityThreshold;
	PGPBoolean			bReadOnly;
	PGPBoolean			bNeedsCentering;
	PGPBoolean			bSubkeyOpsDisabled;
	PGPBoolean			bSubkeyGenEnabled;

	PGPUInt32			uKeyAlg;
	PGPUInt32			uKeyVer;
	PGPUInt32			uTrust;
	PGPUInt32			uValidity;
	PGPInt32			iExpireDays;
	PGPUInt32			uNumberADKs;
	PGPUInt32			uNumberRevokers;

	PGPBoolean			bSecret;
	PGPBoolean			bSplit;
	PGPBoolean			bDisabled;
	PGPBoolean			bAxiomatic;
	PGPBoolean			bOnToken;
	PGPBoolean			bInvalid;
	PGPBoolean			bRevoked;
	PGPBoolean			bExpired;
	PGPBoolean			bPhotoInvalid;
	PGPBoolean			bX509;
	PGPBoolean			bShowHexFingerprint;
	PGPBoolean			bReadyToPaint;

	HWND				hwndValidity;
	HWND				hwndStartDate;
	HWND				hwndExpireDate;
	HWND				hwndSubKeys;
	HWND				hwndADKs;
	HWND				hwndRevokers;
	HWND				hwndRevokerDlg;

	HTLITEM				hTSelectedADK;
	HTLITEM				hTSelectedRevoker;

	PGPInt32			iNumPhotoIDs;
	PGPInt32			iImageIndex;
	HBITMAP				hbitmapPhotoID;
	HPALETTE			hpalettePhotoID;
	PGPInt32			iwidthPhotoID;
	PGPInt32			iheightPhotoID;
	PGPByte*			pPhotoBuffer;
	PGPSize				iPhotoBufferLength;

} CLPROPSHEETSTRUCT, *PCLPROPSHEETSTRUCT;

typedef struct {
	PGPKeyDBObjRef		subkey;
	PGPTime				timeStart;
	INT					iExpireDays;
	BOOL				bNeverExpires;
	UINT				uSize;
} SUBKEYSTRUCT, *PSUBKEYSTRUCT;

typedef struct {
	PCLPROPSHEETSTRUCT	pclpss;
	PSUBKEYSTRUCT		psks;
	PGPError			err;
	LPSTR				pszPhrase;
	PGPByte*			pPasskey;
	PGPSize				sizePasskey;
	HWND				hwndProgress;
	BOOL				bCancelPending;
	BOOL				bGenDone;
	BOOL				bGenOK;
	INT					iStatusValue;
	INT					iStatusDirection;
} SUBKEYGENSTRUCT, *PSUBKEYGENSTRUCT;

// local globals
static UINT uSubkeySizes[] = {1024, 1536, 2048, 3072};

static DWORD aKeyPropIds[] = {
	IDC_CHANGEPHRASE,		IDH_PGPCL_CHANGEPHRASE,	// this must be first item
	IDC_KEYID,				IDH_PGPCL_KEYID,
	IDC_KEYTYPE,			IDH_PGPCL_KEYTYPE,
	IDC_KEYSIZE,			IDH_PGPCL_KEYSIZE,
	IDC_CREATEDATE,			IDH_PGPCL_CREATEDATE,
	IDC_EXPIREDATE,			IDH_PGPCL_KEYEXPIRES,
	IDC_CIPHER,				IDH_PGPCL_CIPHER,
	IDC_HEXFINGERPRINT,		IDH_PGPCL_HEXFINGERPRINT,
	IDC_FINGERPRINT1,		IDH_PGPCL_FINGERPRINT,
	IDC_FINGERPRINT2,		IDH_PGPCL_FINGERPRINT,
	IDC_FINGERPRINT3,		IDH_PGPCL_FINGERPRINT,
	IDC_FINGERPRINT4,		IDH_PGPCL_FINGERPRINT,
	IDC_USEHEXFINGERPRINT,	IDH_PGPCL_DISPLAYHEXFINGERPRINT,
	IDC_ENABLED,			IDH_PGPCL_ENABLED,
	IDC_ONTOKEN,			IDH_PGPCL_ONTOKEN,
	IDC_PHOTOID,			IDH_PGPCL_PHOTOID,
	IDC_VALIDITYBAR,		IDH_PGPCL_VALIDITYBAR,
	IDC_TRUSTSLIDER,		IDH_PGPCL_TRUSTSLIDER,
	IDC_AXIOMATIC,			IDH_PGPCL_AXIOMATIC,
	0,0 
}; 

static DWORD aNewSubkeyIds[] = {
	IDC_SUBKEYSIZE,		IDH_PGPCL_NEWSUBKEYSIZE,
	IDC_STARTDATE,		IDH_PGPCL_NEWSUBKEYSTARTDATE,
	IDC_NEVEREXPIRES,	IDH_PGPCL_NEWSUBKEYNEVEREXPIRES,
	IDC_EXPIRESON,		IDH_PGPCL_NEWSUBKEYEXPIRES,
	IDC_EXPIRATIONDATE,	IDH_PGPCL_NEWSUBKEYEXPIREDATE,
	0,0 
}; 

static DWORD aSubkeyIds[] = {
	IDC_SUBKEYLIST,		IDH_PGPCL_SUBKEYLIST,
	IDC_NEWSUBKEY,		IDH_PGPCL_SUBKEYCREATE,
	IDC_REVOKESUBKEY,	IDH_PGPCL_SUBKEYREVOKE,
	IDC_REMOVESUBKEY,	IDH_PGPCL_SUBKEYREMOVE,
	IDC_STATICTEXT1,	-1,
	IDC_STATICTEXT2,	-1,
	0,0 
}; 

static DWORD aADKIds[] = {
	IDC_FRAME,			IDH_PGPCL_ADKLIST,
	IDC_ADKTREELIST,	IDH_PGPCL_ADKLIST,
	IDC_UPDATE,			IDH_PGPCL_UPDATEADK,
	0,0 
}; 

static DWORD aRevokerIds[] = {
	IDC_FRAME,			IDH_PGPCL_REVOKERLIST,
	IDC_REVOKERTREELIST,IDH_PGPCL_REVOKERLIST,
	IDC_UPDATE,			IDH_PGPCL_UPDATEREVOKER,
	0,0 
}; 

static DWORD aPGPcertIds[] = {
	IDC_NAME,			IDH_PGPCL_PGPCERTNAME,
	IDC_KEYID,			IDH_PGPCL_PGPCERTKEYID,
	IDC_CREATIONDATE,	IDH_PGPCL_PGPCERTCREATION,
	IDC_EXPIRATIONDATE,	IDH_PGPCL_PGPCERTEXPIRATION,
	IDC_EXPORTABLE,		IDH_PGPCL_PGPCERTEXPORTABLE,
	IDC_EXPIRED,		IDH_PGPCL_PGPCERTEXPIRED,
	IDC_REVOKED,		IDH_PGPCL_PGPCERTREVOKED,
	IDC_SHOWSIGNER,		IDH_PGPCL_PGPCERTSHOWSIGNER,
	IDC_DOMAIN,			IDH_PGPCL_PGPCERTDOMAIN,
	IDC_ICONFRAME,		-1,
	0,0 
}; 

static DWORD aX509certIds[] = {
	IDC_NAME,			IDH_PGPCL_X509CERTNAME,
	IDC_ISSUER,			IDH_PGPCL_X509CERTISSUER,
	IDC_LASTCRL,		IDH_PGPCL_X509CERTLASTCRL,
	IDC_NEXTCRL,		IDH_PGPCL_X509CERTNEXTCRL,
	IDC_CREATIONDATE,	IDH_PGPCL_X509CERTCREATION,
	IDC_EXPIRATIONDATE,	IDH_PGPCL_X509CERTEXPIRATION,
	IDC_EXPORTABLE,		IDH_PGPCL_X509CERTEXPORTABLE,
	IDC_EXPIRED,		IDH_PGPCL_X509CERTEXPIRED,
	IDC_REVOKED,		IDH_PGPCL_X509CERTREVOKED,
	IDC_SHOWSIGNER,		IDH_PGPCL_X509CERTSHOWSIGNER,
	IDC_ICONFRAME,		-1,
	0,0 
}; 

//	___________________________________________________
//
//	copy the appropriate hash word to the string buffer

static VOID
sGetHashWord (
		PGPByte		bIndex,
		PGPBoolean	bEven,
		LPSTR		psz)
{
	if (bEven)
		lstrcpy (psz, &hashWordListEven[bIndex][0]);
	else
		lstrcpy (psz, &hashWordListOdd[bIndex][0]);
}

//	___________________________________________________
//
//	set the fingerprint controls on basis of "usehex" flag

static VOID
sSetFingerprintControls (
		 HWND			hwnd,
		 PGPBoolean		bUseHex,
		 PGPKeyDBObjRef	key,
		 PGPUInt32		uKeyVer)
{
	UINT		u, uWordsTotal;
	PGPBoolean	bEven;
	CHAR		sz[64];
	CHAR		sz1[64];
	CHAR		sz2[256];

	if (bUseHex) 
	{
		ShowWindow (GetDlgItem (hwnd, IDC_HEXFINGERPRINT), TRUE);
		ShowWindow (GetDlgItem (hwnd, IDC_FINGERPRINT1), FALSE);
		ShowWindow (GetDlgItem (hwnd, IDC_FINGERPRINT2), FALSE);
		ShowWindow (GetDlgItem (hwnd, IDC_FINGERPRINT3), FALSE);
		ShowWindow (GetDlgItem (hwnd, IDC_FINGERPRINT4), FALSE);

		PGPGetKeyDBObjDataProperty (key, 
				kPGPKeyProperty_Fingerprint, sz, sizeof(sz), &u);
		PGPclConvertFingerprintToString (uKeyVer, sz);
		SetDlgItemText (hwnd, IDC_HEXFINGERPRINT, sz);
	}
	else 
	{
		ShowWindow (GetDlgItem (hwnd, IDC_HEXFINGERPRINT), FALSE);
		ShowWindow (GetDlgItem (hwnd, IDC_FINGERPRINT1), TRUE);
		ShowWindow (GetDlgItem (hwnd, IDC_FINGERPRINT2), TRUE);
		ShowWindow (GetDlgItem (hwnd, IDC_FINGERPRINT3), TRUE);
		ShowWindow (GetDlgItem (hwnd, IDC_FINGERPRINT4), TRUE);

		PGPGetKeyDBObjDataProperty (key, 
				kPGPKeyProperty_Fingerprint, sz1, sizeof(sz1), &u);

		if (uKeyVer > kPGPKeyVersion_V3) 
			uWordsTotal = 20;
		else
			uWordsTotal = 16;

		sz2[0] = '\0';
		bEven = TRUE;
		for (u=0; u<uWordsTotal; u+=NUMHASHWORDCOLS) 
		{
			sGetHashWord (sz1[u], bEven, sz);
			lstrcat (sz2, sz);
			lstrcat (sz2, "\n");
		}
		SetDlgItemText (hwnd, IDC_FINGERPRINT1, sz2);

		sz2[0] = '\0';
		bEven = !bEven;
		for (u=1; u<uWordsTotal; u+=NUMHASHWORDCOLS) 
		{
			sGetHashWord (sz1[u], bEven, sz);
			lstrcat (sz2, sz);
			lstrcat (sz2, "\n");
		}
		SetDlgItemText (hwnd, IDC_FINGERPRINT2, sz2);

		sz2[0] = '\0';
		bEven = !bEven;
		for (u=2; u<uWordsTotal; u+=NUMHASHWORDCOLS) 
		{
			sGetHashWord (sz1[u], bEven, sz);
			lstrcat (sz2, sz);
			lstrcat (sz2, "\n");
		}
		SetDlgItemText (hwnd, IDC_FINGERPRINT3, sz2);

		sz2[0] = '\0';
		bEven = !bEven;
		for (u=3; u<uWordsTotal; u+=NUMHASHWORDCOLS) 
		{
			sGetHashWord (sz1[u], bEven, sz);
			lstrcat (sz2, sz);
			lstrcat (sz2, "\n");
		}
		SetDlgItemText (hwnd, IDC_FINGERPRINT4, sz2);
	}
}

//	___________________________________________________
//
//	convert SYSTEMTIME structure to number of days from today

static PGPError
sSystemTimeToPGPTime (
		 SYSTEMTIME*	pst, 
		 PGPTime*		ptime) 
{
	struct tm	tmstruct;
	time_t		timeStd;

	pgpAssert (pst != NULL);
	pgpAssert (ptime != NULL);

	*ptime = 0;

	if (pst->wYear > 2037) 
		return kPGPError_BadParams;

	tmstruct.tm_mday = pst->wDay;
	tmstruct.tm_mon = pst->wMonth -1;
	tmstruct.tm_year = pst->wYear -1900;
	tmstruct.tm_hour = 0;
	tmstruct.tm_min = 0;
	tmstruct.tm_sec = 0;
	tmstruct.tm_isdst = -1;

	timeStd = mktime (&tmstruct);
	if (timeStd == (time_t)-1) return kPGPError_BadParams;

	*ptime = PGPGetPGPTimeFromStdTime (timeStd);

	return kPGPError_NoErr;
}


//	_____________________________________________________
//
//  attempt to update keyid from server

static BOOL 
sUpdateKeyIDFromServer (
		HWND				hwnd,
		PCLPROPSHEETSTRUCT	pclpss,
		PGPKeyID*			pkeyid)
{
	BOOL			bUpdated		= FALSE;
	PGPKeyDBRef		keydb			= kInvalidPGPKeyDBRef;
	PGPUInt32		uCount;
	PGPError		err;

	SendMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE, 
			kPGPclProperties_EnableWindows, (LPARAM)FALSE);

	err = PGPclSearchServerForKeyIDs (
			pclpss->context, pclpss->tlscontext, 
			hwnd, pkeyid, 1, kPGPclDefaultServer, 
			pclpss->keydbBacking, &keydb);

	if (IsntPGPError (err))
	{
		if (PGPKeyDBRefIsValid (keydb)) 
		{
			PGPCountKeys (PGPPeekKeyDBRootKeySet (keydb), &uCount);
			if (uCount > 0)
			{
				err = PGPCopyKeys (PGPPeekKeyDBRootKeySet (keydb), 
						pclpss->keydbBacking, NULL);

				if (IsntPGPError (err))
				{
					bUpdated = TRUE;

					SendMessage (pclpss->hwndParent, 
							PGPCL_M_PROPERTYMESSAGE, 
							kPGPclProperties_UpdateKeyring, 0);
				}
			}
			PGPFreeKeyDB (keydb);
		}
	}

	SendMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE, 
			kPGPclProperties_EnableWindows, (LPARAM)TRUE);

	return bUpdated;
}


//	______________________________________________
//
//  callback routine called by library key generation routine
//  every so often with status of keygen.  Returning a nonzero
//  value cancels the key generation.

static PGPError 
sSubkeyGenEventHandler (
		PGPContextRef	context, 
		PGPEvent*		event,
		PGPUserValue	userValue)
{
	INT					iReturnCode = kPGPError_NoErr;
	PSUBKEYGENSTRUCT	pskgs;

	pskgs = (PSUBKEYGENSTRUCT) userValue;

	if (pskgs->bCancelPending) 
		iReturnCode = kPGPError_UserAbort;
	
	return (iReturnCode);
}


//	___________________________________________________
//
//  subkey generation thread

static VOID 
sSubkeyGenerationThread (void *pArgs)
{
	PSUBKEYGENSTRUCT	pskgs			= (PSUBKEYGENSTRUCT)pArgs;
	BOOL				bRetVal			= FALSE;
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	UINT				uSubKeyAlg		= 0;

	PGPBoolean			bFastGen;
	PGPContextRef		ctx;
	UINT				uEntropyNeeded;

	if (pskgs->pclpss->uKeyAlg == kPGPPublicKeyAlgorithm_DSA)
		uSubKeyAlg = kPGPPublicKeyAlgorithm_ElGamal;
	else if (pskgs->pclpss->uKeyAlg == kPGPPublicKeyAlgorithm_ECSign)
		uSubKeyAlg = kPGPPublicKeyAlgorithm_ECEncrypt;
	else
		uSubKeyAlg = kPGPPublicKeyAlgorithm_RSA;

	// get client preferences
	pskgs->err = PGPclPeekClientLibPrefRefs (&prefref, NULL);
	if (IsntPGPError (pskgs->err))
	{
		ctx = pskgs->pclpss->context;
		PGPGetPrefBoolean (prefref, kPGPPrefFastKeyGen, &bFastGen);

		uEntropyNeeded = PGPGetKeyEntropyNeeded (ctx,	
				PGPOKeyGenParams (ctx, uSubKeyAlg, pskgs->psks->uSize),
				PGPOKeyFlags (ctx, kPGPKeyPropertyFlags_UsageEncrypt),
				PGPOKeyGenFast (ctx, bFastGen),
				PGPOLastOption (ctx));
		PGPclRandom (ctx, pskgs->hwndProgress, uEntropyNeeded);

		// generate subkey
		if (pskgs->pszPhrase) 
		{
			pskgs->err = PGPGenerateSubKey (
				ctx, &pskgs->psks->subkey,
				PGPOKeyGenMasterKey (ctx, pskgs->pclpss->key),
				PGPOKeyGenParams (ctx, uSubKeyAlg, pskgs->psks->uSize),
				PGPOKeyFlags (ctx, kPGPKeyPropertyFlags_UsageEncrypt),
				PGPOKeyGenFast (ctx, bFastGen),
				PGPOPassphrase (ctx, pskgs->pszPhrase),
				PGPOCreationDate (ctx, pskgs->psks->timeStart),
				PGPOExpiration (ctx, pskgs->psks->iExpireDays),
				PGPOEventHandler (ctx, sSubkeyGenEventHandler, pskgs),
				pskgs->pclpss->bOnToken ? 
					PGPOKeyGenOnToken (ctx, 0) :
					PGPONullOption (ctx),
				PGPOLastOption (ctx));
		}
		else 
		{
			pskgs->err = PGPGenerateSubKey (
				ctx, &pskgs->psks->subkey,
				PGPOKeyGenMasterKey (ctx, pskgs->pclpss->key),
				PGPOKeyGenParams (ctx, uSubKeyAlg, pskgs->psks->uSize),
				PGPOKeyFlags (ctx, kPGPKeyPropertyFlags_UsageEncrypt),
				PGPOKeyGenFast (ctx, bFastGen),
				PGPOPasskeyBuffer (ctx, pskgs->pPasskey, pskgs->sizePasskey),
				PGPOCreationDate (ctx, pskgs->psks->timeStart),
				PGPOExpiration (ctx, pskgs->psks->iExpireDays),
				PGPOEventHandler (ctx, sSubkeyGenEventHandler, pskgs),
				pskgs->pclpss->bOnToken ? 
					PGPOKeyGenOnToken (ctx, 0) :
					PGPONullOption (ctx),
				PGPOLastOption (ctx));
		}

		// note: PGPGenerateSubKey returns kPGPError_OutOfMemory 
		// when user aborts!
		if ((pskgs->err == kPGPError_OutOfMemory) ||
			(pskgs->bCancelPending))
		{
			pskgs->err = kPGPError_UserAbort;
		}
	}

	pskgs->bGenDone = TRUE;
	if (IsntPGPError (pskgs->err))
		pskgs->bGenOK = TRUE;

	SendMessage (pskgs->hwndProgress, WM_CLOSE, 0, 0);
	return;
}


//	___________________________________________________
//
//  check for valid date settings

static VOID 
sValidateSubKeyDates (
		HWND				hwnd, 
		PCLPROPSHEETSTRUCT	pclpss) 
{
	BOOL		bOK;
	SYSTEMTIME	st;
	INT			iStartDays;
	INT			iExpireDays;

	// get starting date
	SendMessage (pclpss->hwndStartDate, 
			DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
	PGPclSystemTimeToDays (&st, &iStartDays);

	// get expiration date
	iExpireDays = iStartDays+1;
	if (IsDlgButtonChecked (hwnd, IDC_NEVEREXPIRES) == BST_UNCHECKED) 
	{
		SendMessage (pclpss->hwndExpireDate, 
				DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
		PGPclSystemTimeToDays (&st, &iExpireDays);
	}

	if ((iStartDays >= 0) && 
		(iExpireDays > iStartDays) &&
		((pclpss->iExpireDays == -1) ||
		 (iExpireDays <= pclpss->iExpireDays)))
	{
		bOK = TRUE;
	}
	else 
		bOK = FALSE;

	if (bOK)
		EnableWindow (GetDlgItem (hwnd, IDOK), TRUE);
	else
		EnableWindow (GetDlgItem (hwnd, IDOK), FALSE);
}


//	____________________________________
//
//  Draw the "LED" progress indicator

static VOID
sDrawSendStatus (
		HWND				hwnd,
		PSUBKEYGENSTRUCT	pskgs)
{
	HBRUSH			hBrushLit, hBrushUnlit, hBrushOld;
	HPEN			hPen, hPenOld;
	INT				i;
	INT				itop, ibot, ileft, iright, iwidth;
	RECT			rc;
	HDC				hdc;
	PAINTSTRUCT		ps;

	if (pskgs->iStatusValue < -1)
		return;

	hdc = BeginPaint (hwnd, &ps);

	// draw 3D shadow
	GetClientRect (hwnd, &rc);
	itop = rc.top+1;
	ibot = rc.bottom-2;

	iwidth = (rc.right-rc.left) / NUMLEDS;
	iwidth -= LEDSPACING;

	ileft = rc.left + 4;
	for (i=0; i<NUMLEDS; i++)
	{
		iright = ileft + iwidth;

		MoveToEx (hdc, ileft, ibot, NULL);
		LineTo (hdc, iright, ibot);
		LineTo (hdc, iright, itop);

		ileft += iwidth + LEDSPACING;
	}

	hPen = CreatePen (PS_SOLID, 0, RGB (128, 128, 128));
	hPenOld = SelectObject (hdc, hPen);
	hBrushLit = CreateSolidBrush (RGB (0, 255, 0));
	hBrushUnlit = CreateSolidBrush (RGB (0, 128, 0));

	ileft = rc.left + 4;

	// draw "Knight Rider" LEDs
	if (pskgs->iStatusDirection)
	{
		hBrushOld = SelectObject (hdc, hBrushUnlit);
		for (i=0; i<NUMLEDS; i++)
		{
			iright = ileft + iwidth;

			if (i == pskgs->iStatusValue)
			{
				SelectObject (hdc, hBrushLit);
				Rectangle (hdc, ileft, itop, iright, ibot);
				SelectObject (hdc, hBrushUnlit);
			}
			else
				Rectangle (hdc, ileft, itop, iright, ibot);

			ileft += iwidth + LEDSPACING;
		}
	}

	SelectObject (hdc, hBrushOld);
	SelectObject (hdc, hPenOld);
	DeleteObject (hPen);
	DeleteObject (hBrushLit);
	DeleteObject (hBrushUnlit);

	EndPaint (hwnd, &ps);
}


//	____________________________________
//
//  force redrawing of the "LED" progress indicator

static VOID
sInvalidateLEDs (
		HWND		hwnd,
		UINT		idc)
{
	RECT	rc;

	GetWindowRect (GetDlgItem (hwnd, idc), &rc);
	MapWindowPoints (NULL, hwnd, (LPPOINT)&rc, 2);
	InvalidateRect (hwnd, &rc, FALSE);
}


//	___________________________________________________
//
//  subkey generation progress dialog procedure

static BOOL CALLBACK
sSubkeyGenProgressDlgProc (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam,
		LPARAM	lParam) 
{
	PSUBKEYGENSTRUCT	pskgs;
	DWORD				dw;

	switch (uMsg) {

	case WM_INITDIALOG:
		// store pointer to data structure
		SetWindowLong (hwnd, GWL_USERDATA, lParam);
		pskgs = (PSUBKEYGENSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		pskgs->hwndProgress = hwnd;
		pskgs->iStatusValue = 0;
		pskgs->iStatusDirection = 1;

		// Kick off generation proc, here
		_beginthreadex (NULL, 0, 
				(LPTHREAD_START_ROUTINE)sSubkeyGenerationThread, 
				(void *)pskgs, 0, &dw);

		SetTimer (hwnd, PROGRESS_TIMER, PROGRESS_TIMER_MS, NULL);  
		return TRUE;

	case WM_PAINT :
		pskgs = (PSUBKEYGENSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		sDrawSendStatus (GetDlgItem (hwnd, IDC_PROGRESS), pskgs);
		break;

	case WM_TIMER :
		if (wParam == PROGRESS_TIMER)
		{
			pskgs = (PSUBKEYGENSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			pskgs->iStatusValue += pskgs->iStatusDirection;
			if (pskgs->iStatusValue <= 0)
			{
				pskgs->iStatusValue = 0;
				pskgs->iStatusDirection = 1;
			}
			else if (pskgs->iStatusValue >= NUMLEDS-1)
			{
				pskgs->iStatusValue = NUMLEDS-1;
				pskgs->iStatusDirection = -1;
			}
			sInvalidateLEDs (hwnd, IDC_PROGRESS);
		}
		break;
	
	case WM_DESTROY :
		KillTimer (hwnd, PROGRESS_TIMER);
		break;

	case WM_CLOSE :
		pskgs = (PSUBKEYGENSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		EndDialog (hwnd, pskgs->bGenOK);
		break;

	case WM_COMMAND :
		switch(LOWORD (wParam)) {
		case IDCANCEL :
			pskgs = (PSUBKEYGENSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			pskgs->bCancelPending = TRUE;
			EnableWindow (GetDlgItem (hwnd, IDCANCEL), FALSE);
			break;
		}
		break;
	}

	return FALSE;
}

//	___________________________________________________
//
//  generate new subkey

static BOOL 
sAddNewSubkey (
		HWND				hwnd, 
		PCLPROPSHEETSTRUCT	pclpss) 
{
	BOOL				bRetVal				= FALSE;

	PSUBKEYSTRUCT		psks;
	SUBKEYGENSTRUCT		skgs;

	SYSTEMTIME			st;
	UINT				uSubKeySize;
	CHAR				szSize[16];
	CHAR				szValid[32];
	CHAR				szExpires[32];
	LV_ITEM				lvI;
	INT					iItem, iStartDays;
	CHAR				sz[64];
	CHAR				sz2[128];
	PGPError			err;

	// initialize structs
	skgs.pclpss			= pclpss;
	skgs.err			= kPGPError_NoErr;
	skgs.pszPhrase		= NULL;
	skgs.pPasskey		= NULL;
	skgs.bCancelPending = FALSE;
	skgs.bGenDone		= FALSE;
	skgs.bGenOK			= FALSE;

	// get subkey size
	uSubKeySize = 0;
	GetDlgItemText (hwnd, IDC_SUBKEYSIZE, szSize, sizeof(szSize) -1);
	uSubKeySize = atoi (szSize);

	if (pclpss->uKeyAlg != kPGPPublicKeyAlgorithm_ECSign)
	{
		if ((uSubKeySize < MINSUBKEYSIZE) || 
			(uSubKeySize > MAXSUBKEYSIZE)) 
		{
			LoadString (g_hInst, IDS_BADSUBKEYSIZE, sz, sizeof(sz));
			wsprintf (sz2, sz, MINSUBKEYSIZE);
			LoadString (g_hInst, IDS_CAPTIONINFO, sz, sizeof(sz));
			MessageBox (hwnd, sz2, sz, MB_OK|MB_ICONINFORMATION);
			return FALSE;
		}
	}
	wsprintf (szSize, "%i", uSubKeySize);

	// allocate new structure
	psks = clAlloc (sizeof(SUBKEYSTRUCT));
	if (!psks) 
		return FALSE;
	skgs.psks = psks;

	// initialize structure
	psks->subkey		= kInvalidPGPKeyDBObjRef;
	psks->timeStart		= 0;
	psks->iExpireDays	= 0;
	psks->uSize			= uSubKeySize;

	// get starting date
	SendMessage (pclpss->hwndStartDate, 
			DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
	sSystemTimeToPGPTime (&st, &psks->timeStart);
	GetDateFormat (LOCALE_USER_DEFAULT, 
			DATE_SHORTDATE, &st, NULL, szValid, sizeof(szValid));
	PGPclSystemTimeToDays (&st, &iStartDays);

	// get expiration date
	if (IsDlgButtonChecked (hwnd, IDC_NEVEREXPIRES) == BST_CHECKED) 
	{
		psks->bNeverExpires = TRUE;
		psks->iExpireDays = 0;
	}
	else 
	{
		psks->bNeverExpires = FALSE;
		SendMessage (pclpss->hwndExpireDate, 
				DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
		PGPclSystemTimeToDays (&st, &psks->iExpireDays);
		psks->iExpireDays -= iStartDays;
	}

	if (psks->bNeverExpires)
		LoadString (g_hInst, IDS_NEVER, szExpires, sizeof(szExpires));
	else
	{
		GetDateFormat (LOCALE_USER_DEFAULT, 
				DATE_SHORTDATE, &st, NULL, szExpires, sizeof(szExpires));
	}

	// get phrase from user
	LoadString (g_hInst, IDS_SELKEYPASSPHRASE, sz, sizeof(sz)); 
	err = PGPclGetKeyPhrase (pclpss->context, pclpss->tlscontext,
			hwnd, sz, pclpss->keydbBacking, pclpss->key,
			&skgs.pszPhrase, &skgs.pPasskey, &skgs.sizePasskey);
	PGPclErrorBox (NULL, err);

	if (IsntPGPError (err)) 
	{
		if (DialogBoxParam (g_hInst, MAKEINTRESOURCE (IDD_SUBKEYGENPROG), 
						hwnd, sSubkeyGenProgressDlgProc, (LPARAM)&skgs))
		{
			// figure item index to use
			iItem = ListView_GetItemCount (pclpss->hwndSubKeys);

			// insert listview item
			lvI.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
			lvI.state = 0;      
			lvI.stateMask = 0;
			lvI.iImage = CLDetermineSubkeyIcon (psks->subkey);
		
			lvI.iItem = iItem;
			lvI.iSubItem = 0;
			lvI.pszText	= szValid; 
			lvI.cchTextMax = 0;
			lvI.lParam = (LPARAM)psks;

			iItem = ListView_InsertItem (pclpss->hwndSubKeys, &lvI);
			if (iItem == -1) 
				clFree (psks);
			else 
			{
				// add strings for other columns
				ListView_SetItemText (
						pclpss->hwndSubKeys, iItem, 1, szExpires);
				ListView_SetItemText (
						pclpss->hwndSubKeys, iItem, 2, szSize);
				bRetVal = TRUE;
			}

			SendMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE, 
					kPGPclProperties_UpdateObject, (LPARAM)(pclpss->key));
		}
		else
		{
			PGPclErrorBox (hwnd, skgs.err);
		}
	}

	if (skgs.pszPhrase)
		PGPclFreePhrase (skgs.pszPhrase);

	if (skgs.pPasskey) 
		PGPclFreePasskey (skgs.pPasskey, skgs.sizePasskey);

	return bRetVal;
}


//	___________________________________________________
//
//  new subkey dialog procedure

static BOOL CALLBACK 
sNewSubkeyDlgProc (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam,
		LPARAM	lParam) 
{
	PCLPROPSHEETSTRUCT	pclpss;
	RECT				rc;
	INT					i;
	INT					iMinDefaultSize;			
	CHAR				sz[8];
	SYSTEMTIME			st;

	pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

	switch (uMsg) {

	case WM_INITDIALOG:
		// store pointer to data structure
		SetWindowLong (hwnd, GWL_USERDATA, lParam);
		pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		// create and initialize start date control
		GetWindowRect (GetDlgItem (hwnd, IDC_STARTDATE), &rc);
		MapWindowPoints (NULL, hwnd, (LPPOINT)&rc, 2);
		pclpss->hwndStartDate = CreateWindowEx (0, DATETIMEPICK_CLASS, 
				"DateTime", WS_BORDER|WS_CHILD|WS_VISIBLE|WS_TABSTOP,
				rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top, 
				hwnd, (HMENU)IDC_STARTDATE, g_hInst, NULL);
		SetWindowPos (pclpss->hwndStartDate, 
				GetDlgItem (hwnd, IDC_STARTDATETEXT),
				0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
		SendMessage (pclpss->hwndStartDate, DTM_SETMCCOLOR, 
				MCSC_MONTHBK, (LPARAM)GetSysColor (COLOR_3DFACE));

		// create and initialize expire date control
		GetWindowRect (GetDlgItem (hwnd, IDC_EXPIRATIONDATE), &rc);
		MapWindowPoints (NULL, hwnd, (LPPOINT)&rc, 2);
		pclpss->hwndExpireDate = CreateWindowEx (0, DATETIMEPICK_CLASS,
				"DateTime", WS_BORDER|WS_CHILD|WS_VISIBLE|WS_TABSTOP,
				rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top, 
				hwnd, (HMENU)IDC_EXPIRATIONDATE, g_hInst, NULL);
		SendMessage (pclpss->hwndExpireDate, DTM_SETMCCOLOR, 
				MCSC_MONTHBK, (LPARAM)GetSysColor (COLOR_3DFACE));
		SetWindowPos (pclpss->hwndExpireDate, 
				GetDlgItem (hwnd, IDC_EXPIRESON),
				0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
		EnableWindow (pclpss->hwndExpireDate, FALSE);
		CheckDlgButton (hwnd, IDC_NEVEREXPIRES, BST_CHECKED);

		GetLocalTime (&st);
		st.wYear++;
		SendMessage (pclpss->hwndExpireDate, 
				DTM_SETSYSTEMTIME, GDT_VALID, (LPARAM)&st);

		// initialize subkey size combo box
		if (pclpss->uKeyAlg == kPGPPublicKeyAlgorithm_ECSign)
		{
			iMinDefaultSize = ECSUBKEYSIZE;
			EnableWindow (GetDlgItem (hwnd, IDC_SUBKEYSIZE), FALSE);
		}
		else
		{
			iMinDefaultSize = 0;
			for (i=0; i<(sizeof(uSubkeySizes)/sizeof(UINT)); i++) 
			{
				if (uSubkeySizes[i] >= MINSUBKEYSIZE) 
				{
					wsprintf (sz, "%i", uSubkeySizes[i]);
					SendDlgItemMessage (hwnd, IDC_SUBKEYSIZE, 
							CB_ADDSTRING, 0, (LPARAM)sz);
					if (iMinDefaultSize == 0) 
						iMinDefaultSize = uSubkeySizes[i];
				}
			}

			if (iMinDefaultSize < DEFAULTSUBKEYSIZE)
				iMinDefaultSize = DEFAULTSUBKEYSIZE;
		}

		wsprintf (sz, "%i", iMinDefaultSize);
		SetDlgItemText (hwnd, IDC_SUBKEYSIZE, sz);

		sValidateSubKeyDates (hwnd, pclpss);

		return TRUE;

	case WM_COMMAND:
		switch(LOWORD (wParam)) {
		case IDCANCEL :
			EndDialog (hwnd, FALSE);
			break;

		case IDC_NEVEREXPIRES :
		case IDC_EXPIRESON :
			if (IsDlgButtonChecked (hwnd, IDC_EXPIRESON) == BST_CHECKED) 
			{
				EnableWindow (pclpss->hwndExpireDate, TRUE);
				sValidateSubKeyDates (hwnd, pclpss);
			}
			else 
			{
				EnableWindow (pclpss->hwndExpireDate, FALSE);
				sValidateSubKeyDates (hwnd, pclpss);
			}
			break;

		case IDOK :
			if (sAddNewSubkey (hwnd, pclpss))
				EndDialog (hwnd, TRUE);
			break;
		}
		return TRUE;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aNewSubkeyIds);

	case WM_NOTIFY :
		sValidateSubKeyDates (hwnd, pclpss);
		break;
	}
	return FALSE;
}

//	___________________________________________________
//
//  remove subkey from listview control

static BOOL 
sRemoveSubKey (
		HWND				hwnd, 
		PCLPROPSHEETSTRUCT	pclpss)
{
	INT				iIndex;
	INT				iNumItems;
	INT				ids;
	LV_ITEM			lvI;
	PGPError		err;
	PSUBKEYSTRUCT	psks;

	iNumItems = ListView_GetItemCount (pclpss->hwndSubKeys);
	if (iNumItems > 1)
		ids = IDS_REMOVESUBKEYCONF;
	else 
		ids = IDS_REMOVEONLYSUBKEYCONF;

	if (PGPclMessageBox (hwnd, IDS_CAPTIONWARNING,
			ids, MB_YESNO|MB_ICONEXCLAMATION) == IDNO) 
	{
		return FALSE;
	}

	iIndex = ListView_GetNextItem (pclpss->hwndSubKeys, -1, LVNI_SELECTED);

	if (iIndex > -1) 
	{
		lvI.mask = LVIF_PARAM;
		lvI.iItem = iIndex;
		lvI.iSubItem = 0;
		ListView_GetItem (pclpss->hwndSubKeys, &lvI);

		psks = (PSUBKEYSTRUCT)(lvI.lParam);

		if (pclpss->bOnToken)
		{
			LPSTR		pszPIN		= NULL;
			CHAR		sz[128];

			LoadString (g_hInst, IDS_TOKENPHRASEPROMPT, sz, sizeof(sz));
			err = PGPclGetConventionalPhrase (
					pclpss->context, pclpss->hwndParent, sz, &pszPIN);

			if (IsntPGPError (err))
			{
				HCURSOR		hcursorOld	= SetCursor (LoadCursor (NULL, IDC_WAIT));
				err = PGPDeleteKeyOnToken (
						psks->subkey, -1, pszPIN, lstrlen(pszPIN));
				SetCursor (hcursorOld);
			}

			if (IsntNull (pszPIN))
				PGPclFreePhrase (pszPIN);
		}
		else
			err = PGPDeleteKeyDBObj (psks->subkey);
	
		if (err == kPGPError_BadPassphrase)
		{
			PGPclMessageBox (pclpss->hwndParent, IDS_CAPTIONALERT, 
					IDS_TOKENPHRASEREJECTED, MB_OK|MB_ICONEXCLAMATION);
		}
		else if (IsntPGPError (PGPclErrorBox (hwnd, err))) 
		{
			clFree ((VOID*)lvI.lParam);
			ListView_DeleteItem (pclpss->hwndSubKeys, iIndex);

			SendMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE, 
					kPGPclProperties_UpdateObject, (LPARAM)(pclpss->key));
		}
	}

	return TRUE;
}


//	___________________________________________________
//
//  revoke subkey from listview control

static BOOL 
sRevokeSubKey (
		HWND				hwnd, 
		PCLPROPSHEETSTRUCT	pclpss)
{
	PGPByte*		pPasskey			= NULL;
	PGPSize			sizePasskey			= 0;

	INT				iIndex;
	LV_ITEM			lvI;
	PGPError		err;
	PSUBKEYSTRUCT	psks;
	CHAR			sz[64];

	if (PGPclMessageBox (hwnd, IDS_CAPTIONWARNING, 
			IDS_REVOKESUBKEYCONF, MB_YESNO|MB_ICONEXCLAMATION) == IDNO) 
	{
		return FALSE;
	}

	// get phrase from user if necessary
	LoadString (g_hInst, IDS_SELKEYPASSPHRASE, sz, sizeof(sz)); 
	err = PGPclGetKeyPhrase (pclpss->context, pclpss->tlscontext,
			hwnd, sz, pclpss->keydbBacking, pclpss->key,
			NULL, &pPasskey, &sizePasskey);
	PGPclErrorBox (NULL, err);

	if (IsntPGPError (err)) 
	{
		iIndex = ListView_GetNextItem (pclpss->hwndSubKeys, -1, LVNI_SELECTED);
		if (iIndex > -1) 
		{
			lvI.mask = LVIF_PARAM;
			lvI.iItem = iIndex;
			lvI.iSubItem = 0;
			ListView_GetItem (pclpss->hwndSubKeys, &lvI);

			psks = (PSUBKEYSTRUCT)(lvI.lParam);

			err = PGPRevoke (psks->subkey, 
					pPasskey ?
						PGPOPasskeyBuffer (pclpss->context, 
							pPasskey, sizePasskey) :
						PGPONullOption (pclpss->context),
					PGPOLastOption (pclpss->context));

			if (IsntPGPError (PGPclErrorBox (hwnd, err))) 
			{
				lvI.mask = LVIF_IMAGE;
				lvI.iItem = iIndex;
				lvI.iSubItem = 0;
				lvI.iImage = CLDetermineSubkeyIcon (psks->subkey);
				ListView_SetItem (pclpss->hwndSubKeys, &lvI);

				SendMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE, 
						kPGPclProperties_UpdateObject, (LPARAM)(pclpss->key));
			}
		}
	}

	if (pPasskey) 
		PGPclFreePasskey (pPasskey, sizePasskey);

	return TRUE;
}


//	___________________________________________________
//
//  Populate ListView with subkeys

static BOOL 
sInsertSubkeysIntoList (
		HWND			hwndList, 
		PGPKeyDBObjRef	key,
		PGPKeySetRef	keyset) 
{
	PGPKeyDBObjRef	subkey;
	PGPKeyIterRef	keyiter;
	LV_ITEM			lvI;
	CHAR			szSize[16];
	CHAR			szValid[32];
	CHAR			szExpires[32];
	INT				iItem;
	PGPTime			time;
	PSUBKEYSTRUCT	psks;

	iItem = 0;

	PGPNewKeyIterFromKeySet (keyset, &keyiter);
	PGPKeyIterSeek (keyiter, key);
	PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_SubKey, &subkey);
	while (subkey) 
	{
		// allocate structure to hold info
		psks = clAlloc (sizeof(SUBKEYSTRUCT));
		if (!psks) 
			break;

		// initialize structure
		psks->subkey = subkey;
		psks->uSize = 0;

		// get subkey info
		PGPGetKeyDBObjNumericProperty (subkey, 
				kPGPSubKeyProperty_Bits, &psks->uSize);
		wsprintf (szSize, "%i", psks->uSize);

		PGPGetKeyDBObjTimeProperty (subkey, 
				kPGPSubKeyProperty_Creation, &psks->timeStart);
		CLConvertTimeToString (psks->timeStart, szValid, sizeof (szValid));

		PGPGetKeyDBObjTimeProperty (subkey, 
				kPGPSubKeyProperty_Expiration, &time);
		if (time != kPGPExpirationTime_Never) 
		{
			psks->bNeverExpires = FALSE;
			CLConvertTimeToDays (time, &psks->iExpireDays);
			CLConvertTimeToString (time, szExpires, sizeof (szExpires));
		}
		else 
		{
			psks->bNeverExpires = TRUE;
			psks->iExpireDays = 0;
			LoadString (g_hInst, IDS_NEVER, szExpires, sizeof (szExpires));
		}

		// insert listview item
		lvI.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
		lvI.state = 0;      
		lvI.stateMask = 0;

		lvI.iImage = CLDetermineSubkeyIcon (subkey);

		lvI.iItem = iItem;
		lvI.iSubItem = 0;
		lvI.pszText	= szValid; 
		lvI.cchTextMax = 0;
		lvI.lParam = (LPARAM)psks;

		if (ListView_InsertItem (hwndList, &lvI) == -1) return FALSE;

		// add strings for other columns
		ListView_SetItemText (hwndList, iItem, 1, szExpires);
		ListView_SetItemText (hwndList, iItem, 2, szSize);

		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_SubKey, &subkey);

		iItem++;
	}

	PGPFreeKeyIter (keyiter);

	return TRUE;
}


//	___________________________________________________
//
//  destroy all data structures associated with ListView

static VOID
sDestroySubKeyListAndStructures (
		PCLPROPSHEETSTRUCT	pclpss)
{
	INT		iIndex;
	INT		iNumItems;
	LV_ITEM	lvI;

	if (pclpss->hwndSubKeys == NULL)
		return;

	iNumItems = ListView_GetItemCount (pclpss->hwndSubKeys);
	for (iIndex=0; iIndex<iNumItems; iIndex++) 
	{
		lvI.mask = LVIF_PARAM;
		lvI.iItem = iIndex;
		lvI.iSubItem = 0;
		ListView_GetItem (pclpss->hwndSubKeys, &lvI);
		clFree ((VOID*)(lvI.lParam));
	}

	ListView_DeleteAllItems (pclpss->hwndSubKeys);
}


//	___________________________________________________
//
//  setup subkey ListView 

static VOID 
sFillSubKeyList (
		HWND				hwnd, 
		PCLPROPSHEETSTRUCT	pclpss) 
{

	LV_COLUMN	lvC; 
	CHAR		sz[256];

	ListView_SetImageList (pclpss->hwndSubKeys, g_hilKeys, LVSIL_SMALL);

	lvC.mask = LVCF_FMT | LVCF_TEXT | LVCF_WIDTH | LVCF_SUBITEM;
	lvC.fmt = LVCFMT_CENTER; 
	lvC.pszText = sz;
	
	LoadString (g_hInst, IDS_VALIDFROM, sz, sizeof(sz));
	lvC.cx = 120;  
	lvC.iSubItem = 0;
	if (ListView_InsertColumn (pclpss->hwndSubKeys, 0, &lvC) == -1) return;

	LoadString (g_hInst, IDS_EXPIRES, sz, sizeof(sz));
	lvC.cx = 70;   
	lvC.iSubItem = 1;
	if (ListView_InsertColumn (pclpss->hwndSubKeys, 1, &lvC) == -1) return;

	LoadString (g_hInst, IDS_SIZE, sz, sizeof(sz));
	lvC.cx = 70;   
	lvC.iSubItem = 2;
	if (ListView_InsertColumn (pclpss->hwndSubKeys, 2, &lvC) == -1) return;

	// populate control by iterating through subkeys
	sInsertSubkeysIntoList (pclpss->hwndSubKeys, pclpss->key, 
			PGPPeekKeyDBRootKeySet(pclpss->keydbBacking));
}


//	___________________________________________________
//
//  Key Properties Dialog Message procedure - subkey panel

static BOOL CALLBACK 
sKeyPropDlgProcSubkey (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam,
		LPARAM	lParam) 
{
	PCLPROPSHEETSTRUCT	pclpss;
	INT					iIndex;
	NMHDR*				pnmh;

	pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

	switch (uMsg) {

	case WM_INITDIALOG:
		// store pointer to data structure
		SetWindowLong (hwnd, GWL_USERDATA, ((PROPSHEETPAGE*)lParam)->lParam);
		pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		pclpss->hwndSubKeys = GetDlgItem (hwnd, IDC_SUBKEYLIST);

		// hide explanatory text for non-keypairs
		if (!pclpss->bSecret) 
		{
			ShowWindow (GetDlgItem (hwnd, IDC_STATICTEXT1), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_STATICTEXT2), SW_HIDE);
		}

		// initialize all controls
		PostMessage (hwnd, WM_APP, 0, 0);
		return TRUE;

	case WM_APP :
		sFillSubKeyList (hwnd, pclpss);
		if (!pclpss->bSubkeyOpsDisabled && pclpss->bSubkeyGenEnabled) 
		{
			if (pclpss->bSecret && !pclpss->bRevoked && 
					!pclpss->bExpired && !pclpss->bReadOnly &&
					(pclpss->uKeyVer > kPGPKeyVersion_V3))
			{
				BOOL	bEnable;

				switch (pclpss->uKeyAlg) {
				case kPGPPublicKeyAlgorithm_DSA :
					bEnable = TRUE;
					break;

				case kPGPPublicKeyAlgorithm_ECSign :
					if (IsntPGPError (PGPclCheckSDKSupportForPKAlg (
							kPGPPublicKeyAlgorithm_ECSign, FALSE, FALSE)))
					{
						bEnable = TRUE;
					}
					break;

				default :
					if (IsntPGPError (PGPclCheckSDKSupportForPKAlg (
							kPGPPublicKeyAlgorithm_RSA, FALSE, FALSE)))
					{
						bEnable = TRUE;
					}
					break;
				}

				EnableWindow (GetDlgItem (hwnd, IDC_NEWSUBKEY), bEnable);
			}
		}
		break;

	case WM_COMMAND:
		switch(LOWORD (wParam)) {
		case IDC_NEWSUBKEY :
			SendMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE, 
					kPGPclProperties_EnableWindows, (LPARAM)FALSE);
			DialogBoxParam (g_hInst, MAKEINTRESOURCE (IDD_NEWSUBKEY), 
					hwnd, sNewSubkeyDlgProc, (LPARAM)pclpss);
			SendMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE, 
					kPGPclProperties_EnableWindows, (LPARAM)TRUE);
			break;

		case IDC_REVOKESUBKEY :
			if (sRevokeSubKey (hwnd, pclpss)) 
			{
				EnableWindow (GetDlgItem (hwnd, IDC_REVOKESUBKEY), FALSE);
				EnableWindow (GetDlgItem (hwnd, IDC_REMOVESUBKEY), FALSE);
			}
			break;

		case IDC_REMOVESUBKEY :
			if (sRemoveSubKey (hwnd, pclpss)) 
			{
				EnableWindow (GetDlgItem (hwnd, IDC_REVOKESUBKEY), FALSE);
				EnableWindow (GetDlgItem (hwnd, IDC_REMOVESUBKEY), FALSE);
			}
			break;
		}
		return TRUE;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aSubkeyIds);

	case WM_DESTROY :
		ListView_SetImageList (pclpss->hwndSubKeys, NULL, LVSIL_SMALL);
		break;

	case WM_NOTIFY :
		pnmh = (NMHDR*)lParam;
		switch (pnmh->code) {

		case PSN_HELP :
			PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0, 
					(char*)kPGPclHelpFile, 
					(DWORD*)IDH_PGPCL_PROPSUBKEYDIALOG);
			break;

		case PSN_SETACTIVE :
			ListView_SetImageList (pclpss->hwndSubKeys, g_hilKeys, LVSIL_SMALL);
			break;

		case PSN_KILLACTIVE :
			ListView_SetImageList (pclpss->hwndSubKeys, NULL, LVSIL_SMALL);
			break;

		case LVN_ITEMCHANGED :
			if ((pnmh->idFrom == IDC_SUBKEYLIST) && 
				!pclpss->bSubkeyOpsDisabled &&
				 pclpss->bSecret &&
				!pclpss->bReadOnly &&
				(pclpss->uKeyVer > kPGPKeyVersion_V3)) 
			{
				iIndex = ListView_GetNextItem (
						pclpss->hwndSubKeys, -1, LVNI_SELECTED);

				if (iIndex > -1)
				{
					LV_ITEM			lvI;
					PSUBKEYSTRUCT	psks;
					PGPBoolean		bRevoked, bExpired;

					lvI.mask = LVIF_IMAGE|LVIF_PARAM;
					lvI.iItem = iIndex;
					lvI.iSubItem = 0;
					ListView_GetItem (pclpss->hwndSubKeys, &lvI);

					psks = (PSUBKEYSTRUCT)(lvI.lParam);
					PGPGetKeyDBObjBooleanProperty (psks->subkey, 
							kPGPKeyProperty_IsRevoked,&bRevoked);
					PGPGetKeyDBObjBooleanProperty (psks->subkey, 
							kPGPKeyProperty_IsExpired,&bExpired);

					if (pclpss->bSecret && !bRevoked && !bExpired)
					{
						EnableWindow (
							GetDlgItem (hwnd, IDC_REVOKESUBKEY), TRUE);
					}
					else
					{
						EnableWindow (
							GetDlgItem (hwnd,IDC_REVOKESUBKEY), FALSE);
					}

					EnableWindow (GetDlgItem (hwnd,IDC_REMOVESUBKEY), TRUE);
				}
				else 
				{
					EnableWindow (GetDlgItem (hwnd,IDC_REVOKESUBKEY), FALSE);
					EnableWindow (GetDlgItem (hwnd,IDC_REMOVESUBKEY), FALSE);
				}
			}
			break;
		}
		break;
	}
	return FALSE;
}

//	___________________________________________________
//
//  Populate ListView with ADKs

static VOID 
sInsertADKsIntoTree (
		HWND			hwndTree, 
		PGPKeyDBObjRef	key,
		UINT			uNumberADKs) 
{
	TL_TREEITEM		tlI;
	TL_LISTITEM		tlL;
	TL_INSERTSTRUCT tlIns;
	HTLITEM			hTNew;
	PGPKeyDBObjRef	keyADK;
	PGPKeyID		keyidADK;
	CHAR			szName[kPGPMaxUserIDSize];
	PGPError		err;
	CHAR			szID[kPGPMaxKeyIDStringSize];
	UINT			u, u2;
	PGPByte			byteClass;
	PGPBoolean		bItalics;

	tlI.hItem = NULL;
	tlI.mask = TLIF_TEXT | TLIF_IMAGE | TLIF_STATE | TLIF_PARAM;
	tlI.stateMask = TLIS_ITALICS;
	tlI.pszText = szName;

	for (u=0; u<uNumberADKs; u++) 
	{
		byteClass = 0;
		tlI.iImage = IDX_DSAPUBDISKEY;
		tlI.state = 0;
		tlI.lParam = u;

		err = PGPGetIndexedAdditionalRecipientRequestKey (key, 
				u, &keyADK, &keyidADK, &byteClass);

		if (IsntPGPError (err)) 
		{ 
			if (PGPKeyDBObjRefIsValid (keyADK)) 
			{
				keyADK = PGPPeekKeyDBObjKey (keyADK);
				PGPclGetPrimaryUserIDNameUTF8 (keyADK, szName, sizeof(szName), &u2);
				tlI.iImage = PGPclDetermineKeyIcon (keyADK, &bItalics);
				if (bItalics) 
					tlI.state = TLIS_ITALICS;
			}
			else 
			{
				tlI.state = TLIS_ITALICS;

				LoadString (g_hInst, 
						IDS_UNKNOWNADK, szName, sizeof(szName));
				err = PGPGetKeyIDString (&keyidADK, 
						kPGPKeyIDString_Abbreviated, szID);
				if (IsntPGPError (err)) 
				{
					LoadString (g_hInst, 
							IDS_UNKNOWNADKID, szName, sizeof(szName));
					lstrcat (szName, szID);
				}
			}
		}

		else 
			LoadString (g_hInst, IDS_ERRONEOUSADK, szName, sizeof(szName));

		tlI.cchTextMax = lstrlen (szName);
		tlIns.hInsertAfter = (HTLITEM)TLI_SORT;
		tlIns.item = tlI;
		tlIns.hParent = NULL;
		hTNew = TreeList_InsertItem (hwndTree, &tlIns);

		tlL.pszText = NULL;
		tlL.hItem = hTNew;
		tlL.stateMask = TLIS_VISIBLE;
		tlL.iSubItem = 1;
		tlL.mask = TLIF_DATAVALUE | TLIF_STATE;
		tlL.state = TLIS_VISIBLE;

		if (byteClass & 0x80) 
			tlL.lDataValue = IDX_ADK;
		else 
			tlL.lDataValue = IDX_NOADK;

		TreeList_SetListItem (hwndTree, &tlL, FALSE);
	}
}


//	_____________________________________________________
//
//  add columns to treelist

static BOOL 
sAddADKColumns (
		HWND	hwndTree, 
		INT		iWidth) 
{
	TL_COLUMN tlc;
	CHAR sz[64];

	TreeList_DeleteAllColumns (hwndTree);

	tlc.mask = TLCF_FMT | TLCF_WIDTH | TLCF_TEXT | 
				TLCF_SUBITEM | TLCF_DATATYPE | TLCF_DATAMAX;
	tlc.pszText = sz;

	tlc.iSubItem = 0;
	tlc.fmt = TLCFMT_LEFT;
	tlc.iDataType = TLC_DATASTRING;
	tlc.cx = iWidth - ENFORCELISTWIDTH - 20;
	tlc.bMouseNotify = FALSE;
	LoadString (g_hInst, IDS_NAMEFIELD, sz, sizeof(sz));
	TreeList_InsertColumn (hwndTree, 0, &tlc);

	tlc.fmt = TLCFMT_IMAGE;
	tlc.iDataType = TLC_DATALONG;
	tlc.cx = ENFORCELISTWIDTH;
	tlc.bMouseNotify = FALSE;
	LoadString (g_hInst, IDS_ENFORCEFIELD, sz, sizeof(sz));
	TreeList_InsertColumn (hwndTree, 1, &tlc);

	return TRUE;
}


//	_____________________________________________________
//
//  attempt to update ADK from server

static BOOL 
sUpdateSelectedADK (
		HWND				hwnd, 
		PCLPROPSHEETSTRUCT	pclpss)
{
	BOOL			bUpdated		= FALSE;
	TL_TREEITEM		tlI;
	PGPKeyDBObjRef	keyADK;
	PGPKeyID		keyidADK;
	PGPByte			byteClass;
	PGPError		err;

	tlI.hItem = pclpss->hTSelectedADK;
	tlI.mask = TLIF_PARAM;
	if (TreeList_GetItem (pclpss->hwndADKs, &tlI))
	{
		err = PGPGetIndexedAdditionalRecipientRequestKey (pclpss->key, 
				tlI.lParam, &keyADK, &keyidADK, &byteClass);

		if (IsntPGPError (err))
			bUpdated = sUpdateKeyIDFromServer (hwnd, pclpss, &keyidADK);
	}

	return bUpdated;
}


//	___________________________________________________
//
//  Key Properties Dialog Message procedure - ADK panel

static BOOL CALLBACK 
sKeyPropDlgProcADK (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam,
		LPARAM	lParam) 
{
	PCLPROPSHEETSTRUCT	pclpss;

	pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

	switch (uMsg) {

	case WM_INITDIALOG:
	{
		RECT				rc;
		INT					i;
		DWORD				dwStyle;
		PGPclWindowsGenus	genus;

		// store pointer to data structure
		SetWindowLong (hwnd, GWL_USERDATA, ((PROPSHEETPAGE*)lParam)->lParam);
		pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		// create tree view window
		GetClientRect (GetDlgItem (hwnd, IDC_UPDATE), &rc);
		i = rc.bottom - rc.top;

		GetClientRect (GetDlgItem (hwnd, IDC_FRAME), &rc);
		MapWindowPoints (GetDlgItem (hwnd, IDC_FRAME), hwnd, (LPPOINT)&rc, 2);
		rc.left += TREELISTOFFSET;
		rc.right -= TREELISTOFFSET;
		rc.top += TREELISTOFFSET*2;
		rc.bottom -= (i + TREELISTOFFSET*2);

		dwStyle = WS_VISIBLE | WS_CHILD | WS_BORDER | WS_TABSTOP | WS_GROUP |
				TLS_AUTOSCROLL | TLS_SINGLESELECT;

		PGPclGetWindowsVersion (NULL, &genus, NULL);
		if (genus >= kPGPclWindowsGenusXp)
			dwStyle |= TLS_USEDEFAULTGUIFONT;

		pclpss->hwndADKs = CreateWindowEx (WS_EX_CLIENTEDGE, WC_TREELIST, "",
				dwStyle, rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top,
				hwnd, (HMENU)IDC_ADKTREELIST, g_hInst, NULL);

		SetWindowPos (pclpss->hwndADKs, GetDlgItem (hwnd, IDC_FRAME), 
				0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE);

		TreeList_SetImageList (pclpss->hwndADKs, g_hilKeys);
		sAddADKColumns (pclpss->hwndADKs, rc.right-rc.left);

		// initialize all controls
		PostMessage (hwnd, WM_APP, 0, 0);
		return TRUE;
	}

	case WM_APP :
		TreeList_DeleteTree (pclpss->hwndADKs, TRUE);
		sInsertADKsIntoTree (
				pclpss->hwndADKs, pclpss->key, pclpss->uNumberADKs);
		break;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aADKIds);

	case WM_NOTIFY :
		switch (((NMHDR FAR *) lParam)->code) {

		case PSN_HELP :
			PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0, 
					(char*)kPGPclHelpFile, 
					(DWORD*)IDH_PGPCL_PROPADKDIALOG);
			break;

		case TLN_SELCHANGED :
			if (wParam == IDC_ADKTREELIST)
			{
				pclpss->hTSelectedADK = 
						((LPNM_TREELIST)lParam)->itemNew.hItem;

				if (pclpss->hTSelectedADK)
					EnableWindow (GetDlgItem (hwnd, IDC_UPDATE), TRUE);
				else
					EnableWindow (GetDlgItem (hwnd, IDC_UPDATE), FALSE);
			}
			break;
		}
		break;

	case WM_COMMAND :
		switch(LOWORD (wParam)) {
		case IDC_UPDATE :
			EnableWindow (GetDlgItem (hwnd, IDC_UPDATE), FALSE);
			if (sUpdateSelectedADK (hwnd, pclpss))
			{
				SendMessage (hwnd, WM_APP, 0, 0);
				InvalidateRect (pclpss->hwndADKs, NULL, FALSE);
				PGPclMessageBox (hwnd, IDS_CAPTIONINFO, 
						IDS_SERVERSEARCHSUCCEEDED,
						MB_OK|MB_ICONINFORMATION);
			}
			else
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT, 
						IDS_SERVERSEARCHFAILED,
						MB_OK|MB_ICONEXCLAMATION);
			}
			break;
		}
		return TRUE;
	}

	return FALSE;
}


//	___________________________________________________
//
//  Populate ListView with Designated Revokers

static VOID 
sInsertRevokersIntoTree (
		PCLPROPSHEETSTRUCT	pclpss)
{
	TL_TREEITEM		tlI;
	TL_INSERTSTRUCT tlIns;
	HTLITEM			hTNew;
	PGPKeyDBObjRef	keyRevoker;
	PGPKeyID		keyidRevoker;
	CHAR			szName[kPGPMaxUserIDSize];
	PGPError		err;
	CHAR			szID[kPGPMaxKeyIDStringSize];
	UINT			u, u2;
	PGPBoolean		bItalics;

	tlI.hItem = NULL;
	tlI.mask = TLIF_TEXT | TLIF_IMAGE | TLIF_STATE | TLIF_PARAM;
	tlI.stateMask = TLIS_ITALICS;
	tlI.pszText = szName;

	for (u=0; u<pclpss->uNumberRevokers; u++) 
	{
		tlI.iImage = IDX_DSAPUBDISKEY;
		tlI.state = 0;
		tlI.lParam = u;

		err = PGPGetIndexedRevocationKey (pclpss->key, 
				u, &keyRevoker, &keyidRevoker);

		if (IsntPGPError (err)) 
		{ 
			if (!PGPKeyDBObjRefIsValid (keyRevoker)) 
			{
				err = PGPFindKeyByKeyID (pclpss->keydbBacking, 
						&keyidRevoker, &keyRevoker);
			}

			if (!PGPKeyDBObjRefIsValid (keyRevoker) &&
				PGPKeyDBRefIsValid (pclpss->keydbAlt))
			{
				err = PGPFindKeyByKeyID (pclpss->keydbAlt, 
						&keyidRevoker, &keyRevoker);
			}

			keyRevoker = PGPPeekKeyDBObjKey (keyRevoker);
			if (PGPKeyDBObjRefIsValid (keyRevoker)) 
			{
				PGPclGetPrimaryUserIDNameUTF8 (
						keyRevoker, szName, sizeof(szName), &u2);
				tlI.iImage = PGPclDetermineKeyIcon (
						keyRevoker, &bItalics);

				if (bItalics) 
					tlI.state = TLIS_ITALICS;
			}
			else 
			{
				tlI.state = TLIS_ITALICS;

				LoadString (g_hInst, 
						IDS_UNKNOWNADK, szName, sizeof(szName));

				err = PGPGetKeyIDString (&keyidRevoker, 
						kPGPKeyIDString_Abbreviated, szID);

				if (IsntPGPError (err)) 
				{
					LoadString (g_hInst, 
							IDS_UNKNOWNADKID, szName, sizeof(szName));
					lstrcat (szName, szID);
				}
			}
		}

		else 
		{
			LoadString (g_hInst, 
					IDS_ERRONEOUSREVOKER, szName, sizeof(szName));
		}

		tlI.cchTextMax = lstrlen (szName);
		tlIns.hInsertAfter = (HTLITEM)TLI_SORT;
		tlIns.item = tlI;
		tlIns.hParent = NULL;

		hTNew = TreeList_InsertItem (pclpss->hwndRevokers, &tlIns);
	}
}


//	_____________________________________________________
//
//  add columns to treelist

static BOOL 
sAddRevokerColumns (HWND hWndTree, INT iWidth) {
	TL_COLUMN tlc;
	CHAR sz[64];

	TreeList_DeleteAllColumns (hWndTree);

	tlc.mask = TLCF_FMT | TLCF_WIDTH | TLCF_TEXT | 
				TLCF_SUBITEM | TLCF_DATATYPE | TLCF_DATAMAX;
	tlc.pszText = sz;

	tlc.iSubItem = 0;
	tlc.fmt = TLCFMT_LEFT;
	tlc.iDataType = TLC_DATASTRING;
	tlc.cx = iWidth - 8;
	tlc.bMouseNotify = FALSE;
	LoadString (g_hInst, IDS_NAMEFIELD, sz, sizeof(sz));
	TreeList_InsertColumn (hWndTree, 0, &tlc);

	return TRUE;
}


//	_____________________________________________________
//
//  attempt to update revoker key from server

static BOOL 
sUpdateSelectedRevoker (
		HWND				hwnd, 
		PCLPROPSHEETSTRUCT	pclpss)
{
	BOOL			bUpdated		= FALSE;
	TL_TREEITEM		tlI;
	PGPKeyDBObjRef	keyRevoker;
	PGPKeyID		keyidRevoker;
	PGPError		err;

	tlI.hItem = pclpss->hTSelectedRevoker;
	tlI.mask = TLIF_PARAM;
	if (TreeList_GetItem (pclpss->hwndRevokers, &tlI))
	{
		err = PGPGetIndexedRevocationKey (pclpss->key, 
				tlI.lParam, &keyRevoker, &keyidRevoker);

		if (IsntPGPError (err))
			bUpdated = sUpdateKeyIDFromServer (hwnd, pclpss, &keyidRevoker);
	}

	return bUpdated;
}


//	___________________________________________________
//
//  Key Properties Dialog Message procedure - Revokers panel

static BOOL CALLBACK 
sKeyPropDlgProcRevokers (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam,
		LPARAM	lParam) 
{
	PCLPROPSHEETSTRUCT	pclpss;

	pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

	switch (uMsg) {

	case WM_INITDIALOG:
	{
		RECT				rc;
		INT					i;
		DWORD				dwStyle;
		PGPclWindowsGenus	genus;

		// store pointer to data structure
		SetWindowLong (hwnd, GWL_USERDATA, ((PROPSHEETPAGE*)lParam)->lParam);
		pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		pclpss->hwndRevokerDlg = hwnd;

		// create tree view window
		GetClientRect (GetDlgItem (hwnd, IDC_UPDATE), &rc);
		i = rc.bottom - rc.top;

		GetClientRect (GetDlgItem (hwnd, IDC_FRAME), &rc);
		MapWindowPoints (GetDlgItem (hwnd, IDC_FRAME), hwnd, (LPPOINT)&rc, 2);
		rc.left += TREELISTOFFSET;
		rc.right -= TREELISTOFFSET;
		rc.top += TREELISTOFFSET*2;
		rc.bottom -= (i + TREELISTOFFSET*2);

		dwStyle = WS_VISIBLE | WS_CHILD | WS_BORDER | WS_TABSTOP | WS_GROUP |
				TLS_AUTOSCROLL | TLS_SINGLESELECT;

		PGPclGetWindowsVersion (NULL, &genus, NULL);
		if (genus >= kPGPclWindowsGenusXp)
			dwStyle |= TLS_USEDEFAULTGUIFONT;

		pclpss->hwndRevokers = CreateWindowEx (WS_EX_CLIENTEDGE, WC_TREELIST,
			"", dwStyle, rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top,
			hwnd, (HMENU)IDC_REVOKERTREELIST, g_hInst, NULL);

		SetWindowPos (pclpss->hwndRevokers, GetDlgItem (hwnd, IDC_FRAME), 
				0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE);

		TreeList_SetImageList (pclpss->hwndRevokers, g_hilKeys);
		sAddRevokerColumns (pclpss->hwndRevokers, rc.right-rc.left);

		// initialize all controls
		PostMessage (hwnd, WM_APP, 0, 0);
		return TRUE;
	}

	case WM_APP :
		TreeList_DeleteTree (pclpss->hwndRevokers, TRUE);
		PGPCountRevocationKeys (pclpss->key, &pclpss->uNumberRevokers);
		sInsertRevokersIntoTree (pclpss);
		break;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aRevokerIds);

	case WM_NOTIFY :
		switch (((NMHDR FAR *) lParam)->code) {

		case PSN_HELP :
			PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0, 
					(char*)kPGPclHelpFile, 
					(DWORD*)IDH_PGPCL_PROPREVOKERSDIALOG);
			break;

		case TLN_SELCHANGED :
			if (wParam == IDC_REVOKERTREELIST)
			{
				pclpss->hTSelectedRevoker = 
						((LPNM_TREELIST)lParam)->itemNew.hItem;

				if (pclpss->hTSelectedRevoker)
					EnableWindow (GetDlgItem (hwnd, IDC_UPDATE), TRUE);
				else
					EnableWindow (GetDlgItem (hwnd, IDC_UPDATE), FALSE);
			}
			break;
		}
		break;

	case WM_COMMAND :
		switch(LOWORD (wParam)) {
		case IDC_UPDATE :

			EnableWindow (GetDlgItem (hwnd, IDC_UPDATE), FALSE);
			if (sUpdateSelectedRevoker (hwnd, pclpss))
			{
				SendMessage (hwnd, WM_APP, 0, 0);
				InvalidateRect (pclpss->hwndRevokers, NULL, FALSE);
				PGPclMessageBox (hwnd, IDS_CAPTIONINFO, 
						IDS_SERVERSEARCHSUCCEEDED,
						MB_OK|MB_ICONINFORMATION);
			}
			else
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT, 
						IDS_SERVERSEARCHFAILED,
						MB_OK|MB_ICONEXCLAMATION);
			}
			break;
		}
		return TRUE;
	}
	return FALSE;
}


//	___________________________________________________
//
//  update controls dealing with validity

static VOID
sSetValidityControls (
		HWND				hwnd, 
		PCLPROPSHEETSTRUCT	pclpss, 
		UINT				uVal) 
{
	UINT u;

	u = CLConvertFromPGPValidity (uVal);
	SendMessage (pclpss->hwndValidity, PBM_SETPOS, u, 0);

	if (u < (UINT)pclpss->iValidityThreshold)
		pclpss->bInvalid = TRUE;
	else
		pclpss->bInvalid = FALSE;

	InvalidateRect (hwnd, NULL, FALSE);
}


//	___________________________________________________
//
//  convert slider control values

static UINT
sConvertFromPGPTrust (UINT u) 
{
#if 0 // vertical trust slider
	return (CLConvertFromPGPTrust (kPGPKeyTrust_Complete) -
			CLConvertFromPGPTrust (u));
#else // horizontal trust slider
	return (CLConvertFromPGPTrust (u));
#endif
}


static UINT
sConvertToPGPTrust (UINT u) 
{
#if 0 // vertical trust slider
	return CLConvertToPGPTrust (
		CLConvertFromPGPTrust (kPGPKeyTrust_Complete) - u);
#else // horizontal trust slider
	return CLConvertToPGPTrust (u);
#endif
}


//	___________________________________________________
//
//  update controls dealing with trust

static VOID
sSetTrustControls (
		HWND				hwnd, 
		PCLPROPSHEETSTRUCT	pclpss, 
		UINT				uTrust) 
{
	INT i;

	i = sConvertFromPGPTrust (uTrust);
	SendDlgItemMessage (hwnd, IDC_TRUSTSLIDER, TBM_SETPOS, (WPARAM)TRUE, i);
}


//	___________________________________________________
//
//  display photo userID with appropriate overwriting

static VOID 
sPaintPhotoID (
		HWND		hwnd,
		HBITMAP		hbitmapID,
		HPALETTE	hpaletteID,
		INT			iwidthBM,
		INT			iheightBM,
		BOOL		bInvalid,
		BOOL		bRevoked,
		BOOL		bExpired)
{
	HPALETTE		hpaletteOld		= NULL;
	HDC				hdc;
	HDC				hdcMem;
	HDC				hdcMask;
	HBITMAP			hbitmap;
	PAINTSTRUCT		ps;
	RECT			rc;
	INT				icent;
	INT				ileft, itop, iwidth, iheight;
	BITMAP			bm;

	hdc = BeginPaint (hwnd, &ps);

	GetWindowRect (GetDlgItem (hwnd, IDC_PHOTOID), &rc);
	rc.left += 2;
	rc.top += 2;
	rc.right -= 2;
	rc.bottom -= 2;
	MapWindowPoints (NULL, hwnd, (LPPOINT)&rc, 2);
	FillRect (hdc, &rc, (HBRUSH)(COLOR_3DFACE+1));

	// if photoid is available ... draw it
	if (hbitmapID)
	{
		// check if bitmap needs shrinking
		if ((iheightBM > (rc.bottom-rc.top-2)) ||
			(iwidthBM  > (rc.right-rc.left-2))) 
		{
			if (iheightBM > (iwidthBM * 1.25)) 
			{
				itop = rc.top +1;
				iheight = rc.bottom-rc.top -2;
				icent = (rc.right+rc.left) / 2;
				iwidth = ((rc.bottom-rc.top) * iwidthBM) / iheightBM;
				ileft = icent -(iwidth/2);
			}
			else 
			{
				ileft = rc.left +1;
				iwidth = rc.right-rc.left -2;
				icent = (rc.bottom+rc.top) / 2;
				iheight = ((rc.right-rc.left) * iheightBM) / iwidthBM;
				itop = icent - (iheight/2);
			}
		}
		// otherwise draw it at its real size
		else 
		{
			iwidth = iwidthBM;
			iheight = iheightBM;
			icent = (rc.right+rc.left) / 2;
			ileft = icent - (iwidth/2);
			icent = (rc.bottom+rc.top) / 2;
			itop = icent - (iheight/2);
		}

		hdcMem = CreateCompatibleDC (hdc);

		if (hpaletteID) 
		{
			hpaletteOld = SelectPalette (hdc, hpaletteID, FALSE);
			RealizePalette (hdc);
		}

		SetStretchBltMode (hdc, COLORONCOLOR);
		SelectObject (hdcMem, hbitmapID);
		StretchBlt (hdc, ileft, itop, iwidth, iheight,
					hdcMem, 0, 0, iwidthBM, iheightBM, SRCCOPY);

		// overlay the question mark
		if (bInvalid && !bRevoked && !bExpired) 
		{
			hdcMask = CreateCompatibleDC (hdc);
			hbitmap = 
				LoadBitmap (g_hInst, MAKEINTRESOURCE (IDB_QUESTIONMARK));
			GetObject (hbitmap, sizeof(BITMAP), (LPSTR)&bm);
	
			SelectObject (hdcMask, hbitmap);
	
			SetTextColor(hdc, RGB (255,0,0));
			SetBkColor(hdc, RGB (0,0,0));
			StretchBlt (hdc, rc.left, rc.top, 
					rc.right-rc.left-2, rc.bottom-rc.top-2,
					hdcMask, 0, 0, bm.bmWidth, bm.bmHeight, SRCINVERT);
	
			SetTextColor(hdc, RGB (0,0,0));
			SetBkColor(hdc, RGB (255,255,255));
			StretchBlt (hdc, rc.left, rc.top, 
					rc.right-rc.left-2, rc.bottom-rc.top-2,
					hdcMask, 0, 0, bm.bmWidth, bm.bmHeight, SRCAND);
	
			SetTextColor(hdc, RGB (255,0,0));
			SetBkColor(hdc, RGB (0,0,0));
			StretchBlt (hdc, rc.left, rc.top, 
					rc.right-rc.left-2, rc.bottom-rc.top-2,
					hdcMask, 0, 0, bm.bmWidth, bm.bmHeight, SRCINVERT);
	
			DeleteDC (hdcMask);
		}

		if (hpaletteOld) 
		{
			SelectPalette (hdc, hpaletteOld, TRUE);
			RealizePalette (hdc);
		}

		DeleteDC (hdcMem);
	}

	// overlay the text
	if (bRevoked) 
	{
		hdcMask = CreateCompatibleDC (hdc);
		hbitmap = LoadBitmap (g_hInst, MAKEINTRESOURCE (IDB_REVOKED));
		GetObject (hbitmap, sizeof(BITMAP), (LPSTR)&bm);

		SelectObject (hdcMask, hbitmap);

		SetTextColor(hdc, RGB (255,0,0));
		SetBkColor(hdc, RGB (0,0,0));
		StretchBlt (hdc, rc.left, rc.top, 
				rc.right-rc.left-2, rc.bottom-rc.top-2,
				hdcMask, 0, 0, bm.bmWidth, bm.bmHeight, SRCINVERT);

		SetTextColor(hdc, RGB (0,0,0));
		SetBkColor(hdc, RGB (255,255,255));
		StretchBlt (hdc, rc.left, rc.top, 
				rc.right-rc.left-2, rc.bottom-rc.top-2,
				hdcMask, 0, 0, bm.bmWidth, bm.bmHeight, SRCAND);

		SetTextColor(hdc, RGB (255,0,0));
		SetBkColor(hdc, RGB (0,0,0));
		StretchBlt (hdc, rc.left, rc.top, 
				rc.right-rc.left-2, rc.bottom-rc.top-2,
				hdcMask, 0, 0, bm.bmWidth, bm.bmHeight, SRCINVERT);

		DeleteDC (hdcMask);
	}
	else if (bExpired) 
	{
		hdcMask = CreateCompatibleDC (hdc);
		hbitmap = LoadBitmap (g_hInst, MAKEINTRESOURCE (IDB_EXPIRED));
		GetObject (hbitmap, sizeof(BITMAP), (LPSTR)&bm);

		SelectObject (hdcMask, hbitmap);

		SetTextColor(hdc, RGB (255,0,0));
		SetBkColor(hdc, RGB (0,0,0));
		StretchBlt (hdc, rc.left, rc.top, 
				rc.right-rc.left-2, rc.bottom-rc.top-2,
				hdcMask, 0, 0, bm.bmWidth, bm.bmHeight, SRCINVERT);

		SetTextColor(hdc, RGB (0,0,0));
		SetBkColor(hdc, RGB (255,255,255));
		StretchBlt (hdc, rc.left, rc.top, 
				rc.right-rc.left-2, rc.bottom-rc.top-2,
				hdcMask, 0, 0, bm.bmWidth, bm.bmHeight, SRCAND);

		SetTextColor(hdc, RGB (255,0,0));
		SetBkColor(hdc, RGB (0,0,0));
		StretchBlt (hdc, rc.left, rc.top, 
				rc.right-rc.left-2, rc.bottom-rc.top-2,
				hdcMask, 0, 0, bm.bmWidth, bm.bmHeight, SRCINVERT);

		DeleteDC (hdcMask);
	}

	EndPaint (hwnd, &ps);
} 


//	___________________________________________________
//
//  PhotoID subclass procedure

static LRESULT APIENTRY 
sPhotoIDSubclassProc (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam, 
		LPARAM	lParam) 
{
	PCLPROPSHEETSTRUCT pclpss;
	pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (
			GetParent (hwnd), GWL_USERDATA);

    switch (uMsg) {

	case WM_CONTEXTMENU :
		{
			HMENU	hMC;
			HMENU	hMenuTrackPopup;

			hMC = LoadMenu (g_hInst, MAKEINTRESOURCE (IDR_MENUPHOTOID));
			if (pclpss->hbitmapPhotoID)
			{
				EnableMenuItem (hMC, 
						IDM_COPYBITMAP, MF_BYCOMMAND|MF_ENABLED);
			}
			else
			{
				EnableMenuItem (hMC, 
						IDM_COPYBITMAP, MF_BYCOMMAND|MF_GRAYED);
			}

			hMenuTrackPopup = GetSubMenu (hMC, 0);

			TrackPopupMenu (hMenuTrackPopup, TPM_LEFTALIGN|TPM_RIGHTBUTTON,
					(SHORT)LOWORD(lParam), (SHORT)HIWORD(lParam), 
					0, GetParent (hwnd), NULL);

			DestroyMenu (hMC);
		}
		break;

	default :
		return CallWindowProc (pclpss->wpOrigProcPhoto, 
				hwnd, uMsg, wParam, lParam); 
	}
	return TRUE;
} 


//	___________________________________________________
//
//  Checkbox subclass procedure

static LRESULT APIENTRY 
sCheckboxSubclassProc (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam, 
		LPARAM	lParam) 
{
	PCLPROPSHEETSTRUCT pclpss;
	pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (
			GetParent (hwnd), GWL_USERDATA);

    switch (uMsg) {

	case WM_SETFOCUS :
		return 0;

	case WM_LBUTTONDOWN :
	case WM_LBUTTONDBLCLK :
		return 0;

	default :
		return CallWindowProc (pclpss->wpOrigProcBox, 
				hwnd, uMsg, wParam, lParam); 
	}
} 


//	___________________________________________________
//
//  update system palette

static BOOL
sUpdatePalette (
		HWND				hwnd,
		PCLPROPSHEETSTRUCT	pclpss)
{
	BOOL		bretval		= FALSE;
	HDC			hdc;
	HPALETTE	hpaletteOld;

	if (pclpss->hpalettePhotoID == NULL) 
		return FALSE;

	hdc = GetDC (hwnd);

	hpaletteOld = SelectPalette (hdc, pclpss->hpalettePhotoID, FALSE);
	if (RealizePalette (hdc)) 
	{
		InvalidateRect (hwnd, NULL, TRUE); 
		bretval = TRUE;
	}

	SelectPalette (hdc, hpaletteOld, TRUE);
	RealizePalette (hdc);
	ReleaseDC (hwnd, hdc);

	return bretval;
}


//	___________________________________________________
//
//  Key Properties Dialog Message procedure - General panel

static BOOL CALLBACK 
sKeyPropDlgProcGeneral (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam,
		LPARAM	lParam) 
{
	PCLPROPSHEETSTRUCT	pclpss;

	pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

	switch (uMsg) {

	case WM_INITDIALOG:
	{
		HWND		hwndParent;
		RECT		rc;

		// store pointer to data structure
		SetWindowLong (hwnd, GWL_USERDATA, ((PROPSHEETPAGE*)lParam)->lParam);
		pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		// center dialog on screen
		hwndParent = GetParent (hwnd);
		if (pclpss->bNeedsCentering) 
			PGPclCenterWindowOnMonitor (hwndParent, kPGPclWindowMonitor);

		// subclass photoID control to handle dropping, dragging
		pclpss->wpOrigProcPhoto = 
				(WNDPROC) SetWindowLong(GetDlgItem (hwnd, IDC_PHOTOID), 
						GWL_WNDPROC, (LONG) sPhotoIDSubclassProc); 

		// subclass checkbox control to disable focus
		pclpss->wpOrigProcBox = 
				(WNDPROC) SetWindowLong(GetDlgItem (hwnd, IDC_ONTOKEN), 
						GWL_WNDPROC, (LONG) sCheckboxSubclassProc); 

		// create smooth progress bar
		GetWindowRect (GetDlgItem (hwnd, IDC_VALIDITYBAR), &rc);
		MapWindowPoints (NULL, hwnd, (LPPOINT)&rc, 2);
		pclpss->hwndValidity = CreateWindowEx (0, PROGRESS_CLASS, 
				(LPSTR) NULL, WS_CHILD|WS_VISIBLE|PBS_SMOOTH, 
				rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top, 
				hwnd, (HMENU) 0, g_hInst, NULL); 

		// disable and hide cancel button; and move "OK" button over
		SendMessage (hwndParent, PSM_CANCELTOCLOSE, 0, 0);
		GetWindowRect (GetDlgItem (hwndParent, IDCANCEL), &rc);
		MapWindowPoints (NULL, hwndParent, (LPPOINT)&rc, 2);
		SetWindowPos (GetDlgItem (hwndParent, IDOK), NULL, rc.left,
				rc.top, rc.right-rc.left, rc.bottom-rc.top,
				SWP_NOZORDER);
		ShowWindow (GetDlgItem (hwndParent, IDCANCEL), SW_HIDE);

		// initialize all controls
		PostMessage (hwnd, WM_APP, 0, 0);
		return TRUE;
	}

	case WM_APP :
	{
		CHAR				sz[kPGPMaxUserIDSize];
		INT					i;
		UINT				u;
		PGPUInt32			prefAlg[6];
		PGPTime				tm;
		PGPKeyDBObjRef		userid;
		LPBITMAPINFO		lpbmi;
		PGPKeyID			keyid;
		PGPBoolean			b;
		PGPPrefRef			prefref;

		PGPGetKeyDBObjBooleanProperty (pclpss->key, 
				kPGPKeyProperty_IsSecret, &pclpss->bSecret);
		PGPGetKeyDBObjBooleanProperty (pclpss->key, 
				kPGPKeyProperty_IsSecretShared, &pclpss->bSplit);
		PGPGetKeyDBObjBooleanProperty (pclpss->key, 
				kPGPKeyProperty_IsRevoked,&pclpss->bRevoked);
		PGPGetKeyDBObjBooleanProperty (pclpss->key, 
				kPGPKeyProperty_IsExpired,&pclpss->bExpired);
		PGPGetKeyDBObjBooleanProperty (pclpss->key, 
				kPGPKeyProperty_IsAxiomatic, &pclpss->bAxiomatic);

		// initialize key id edit control
		PGPGetKeyID (pclpss->key, &keyid);
		PGPGetKeyIDString (&keyid, kPGPKeyIDString_Abbreviated, sz);
		SetDlgItemText (hwnd, IDC_KEYID, sz);

		// initialize key type edit control
		switch (pclpss->uKeyAlg) {
		case kPGPPublicKeyAlgorithm_RSA :
			if (pclpss->uKeyVer <= kPGPKeyVersion_V3)
				LoadString (g_hInst, IDS_RSA_LEGACY, sz, sizeof (sz));
			else
				LoadString (g_hInst, IDS_RSA, sz, sizeof (sz));
			break;

		case kPGPPublicKeyAlgorithm_DSA :
			LoadString (g_hInst, IDS_DSA_ELGAMAL, sz, sizeof (sz));
			break;

		case kPGPPublicKeyAlgorithm_ECSign :
			LoadString (g_hInst, IDS_EC, sz, sizeof (sz));
			break;

		default :
			LoadString (g_hInst, IDS_UNKNOWN, sz, sizeof (sz));
			break;
		}
		SetDlgItemText (hwnd, IDC_KEYTYPE, sz);

		// initialize key size edit control
		CLGetKeyBitsString (pclpss->key, sz, sizeof(sz));
		SetDlgItemText (hwnd, IDC_KEYSIZE, sz);

		// initialize key creation date edit control
		PGPGetKeyDBObjTimeProperty (pclpss->key, 
				kPGPKeyProperty_Creation, &tm);
		CLConvertTimeToString (tm, sz, sizeof (sz));
		SetDlgItemText (hwnd, IDC_CREATEDATE, sz);

		// initialize key expiration date edit control
		PGPGetKeyDBObjTimeProperty (pclpss->key, 
				kPGPKeyProperty_Expiration, &tm);
		if (tm == kPGPExpirationTime_Never) 
		{
			LoadString (g_hInst, IDS_NEVER, sz, sizeof (sz));
			pclpss->iExpireDays = -1;
		}
		else 
		{
			CLConvertTimeToDays (tm, &pclpss->iExpireDays);
			CLConvertTimeToString (tm, sz, sizeof (sz));
		}
		SetDlgItemText (hwnd, IDC_EXPIREDATE, sz);

		// initialize preferred cipher edit control
		PGPGetKeyDBObjDataProperty (pclpss->key, 
				kPGPKeyProperty_PreferredAlgorithms,
				(PGPByte*)&prefAlg[0], sizeof(prefAlg), &u);

		if (u < sizeof(PGPCipherAlgorithm))
		{
			if (pclpss->uKeyVer <= kPGPKeyVersion_V3)
				// V3 keys use IDEA
				prefAlg[0] = kPGPCipherAlgorithm_IDEA;
			else
				// according to the OpenPGP spec, V4 keys w/o
				// cipher prefs should use 3DES, but the SDK
				// predates OpenPGP and it uses CAST in this case.
				prefAlg[0] = kPGPCipherAlgorithm_CAST5;
		}

		switch (prefAlg[0]) {
			case kPGPCipherAlgorithm_IDEA :
				LoadString (g_hInst, IDS_IDEA, sz, sizeof(sz));
				break;

			case kPGPCipherAlgorithm_CAST5 :
				LoadString (g_hInst, IDS_CAST, sz, sizeof(sz));
				break;

			case kPGPCipherAlgorithm_3DES :
				LoadString (g_hInst, IDS_3DES, sz, sizeof(sz));
				break;

			case kPGPCipherAlgorithm_Twofish256 :
				LoadString (g_hInst, IDS_TWOFISH, sz, sizeof(sz));
				break;

			case kPGPCipherAlgorithm_AES128 :
				LoadString (g_hInst, IDS_AES128, sz, sizeof(sz));
				break;

			case kPGPCipherAlgorithm_AES192 :
				LoadString (g_hInst, IDS_AES192, sz, sizeof(sz));
				break;

			case kPGPCipherAlgorithm_AES256 :
				LoadString (g_hInst, IDS_AES256, sz, sizeof(sz));
				break;

			default :
				LoadString (g_hInst, IDS_UNKNOWN, sz, sizeof(sz));
				break;
		}
		SetDlgItemText (hwnd, IDC_CIPHER, sz);

		// initialize fingerprint edit control
		if (pclpss->bShowHexFingerprint) 
			CheckDlgButton (hwnd, IDC_USEHEXFINGERPRINT, BST_CHECKED);
		else 
			CheckDlgButton (hwnd, IDC_USEHEXFINGERPRINT, BST_UNCHECKED);

		sSetFingerprintControls (hwnd, 
				pclpss->bShowHexFingerprint, 
				pclpss->key, pclpss->uKeyVer);

		// initialize validity edit and bar controls
		i = CLConvertFromPGPValidity (kPGPValidity_Complete);
		SendMessage (pclpss->hwndValidity, 
				PBM_SETRANGE, 0, MAKELPARAM (0,i));
		PGPGetKeyDBObjNumericProperty (pclpss->key, 
				kPGPKeyProperty_Validity, &pclpss->uValidity);
		sSetValidityControls (hwnd, pclpss, pclpss->uValidity);

		// initialize "on token" checkbox
		PGPGetKeyDBObjBooleanProperty (pclpss->key, 
				kPGPKeyProperty_IsOnToken, &pclpss->bOnToken);
		if (pclpss->bOnToken)
		{
			CheckDlgButton (hwnd, IDC_ONTOKEN, BST_CHECKED);
			ShowWindow (GetDlgItem (hwnd, IDC_ONTOKEN), SW_SHOW);
		}

		// initialize "Axiomatic" checkbox control
		if (pclpss->bAxiomatic) 
			CheckDlgButton (hwnd, IDC_AXIOMATIC, BST_CHECKED);
		else 
			CheckDlgButton (hwnd, IDC_AXIOMATIC, BST_UNCHECKED);

		if (pclpss->bRevoked || 
			pclpss->bExpired || 
			pclpss->bReadOnly ||
			pclpss->bOnToken)
		{
			EnableWindow (GetDlgItem (hwnd, IDC_AXIOMATIC), FALSE);
		}
		else 
			EnableWindow (GetDlgItem (hwnd, IDC_AXIOMATIC), TRUE);

		if (!pclpss->bSecret && !pclpss->bAxiomatic) 
			ShowWindow (GetDlgItem (hwnd, IDC_AXIOMATIC), SW_HIDE);
		else
			ShowWindow (GetDlgItem (hwnd, IDC_AXIOMATIC), SW_SHOW);

		// initialize trust edit and slider controls
		i = CLConvertFromPGPTrust (kPGPKeyTrust_Complete);
		SendDlgItemMessage (hwnd, 
				IDC_TRUSTSLIDER, TBM_SETRANGE, 0, MAKELPARAM (0, i));
		PGPGetKeyDBObjNumericProperty (pclpss->key, 
				kPGPKeyProperty_Trust, &pclpss->uTrust);
		sSetTrustControls (hwnd, pclpss, pclpss->uTrust);

		if (pclpss->bAxiomatic || pclpss->bRevoked || pclpss->bReadOnly)
		{
			EnableWindow (GetDlgItem (hwnd, IDC_TRUSTSLIDER), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_TRUSTTEXT1), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_TRUSTTEXT2), FALSE);
		}
		else
		{
			EnableWindow (GetDlgItem (hwnd, IDC_TRUSTSLIDER), TRUE);
			EnableWindow (GetDlgItem (hwnd, IDC_TRUSTTEXT1), TRUE);
			EnableWindow (GetDlgItem (hwnd, IDC_TRUSTTEXT2), TRUE);
		}

		// initialize enable/disable checkbox
		PGPGetKeyDBObjBooleanProperty (pclpss->key, 
				kPGPKeyProperty_IsDisabled, &pclpss->bDisabled);
		if (pclpss->bDisabled) 
			CheckDlgButton (hwnd, IDC_ENABLED, BST_UNCHECKED);
		else 
			CheckDlgButton (hwnd, IDC_ENABLED, BST_CHECKED);

		if (pclpss->bRevoked || pclpss->bExpired || pclpss->bReadOnly)
			EnableWindow (GetDlgItem (hwnd, IDC_ENABLED), FALSE);
		else
			EnableWindow (GetDlgItem (hwnd, IDC_ENABLED), TRUE);

		if (pclpss->bSecret && !pclpss->bReadOnly) 
		{
			if (pclpss->bAxiomatic) 
				EnableWindow (GetDlgItem (hwnd, IDC_ENABLED), FALSE);
			else
				EnableWindow (GetDlgItem (hwnd, IDC_ENABLED), TRUE);
		}

		// initialize change passphrase button
		if (pclpss->bSplit) 
		{
			aKeyPropIds[1] = IDH_PGPCL_JOINKEY;
			LoadString (g_hInst, IDS_RECONSTITUTEKEY, sz, sizeof(sz));
		}
		else 
		{
			aKeyPropIds[1] = IDH_PGPCL_CHANGEPHRASE;
			LoadString (g_hInst, IDS_CHANGEPHRASE, sz, sizeof(sz));	
		}
		SetDlgItemText (hwnd, IDC_CHANGEPHRASE, sz);

		if (!pclpss->bSecret) 
			ShowWindow (GetDlgItem (hwnd, IDC_CHANGEPHRASE), SW_HIDE);

		if (pclpss->bReadOnly)
			EnableWindow (GetDlgItem (hwnd, IDC_CHANGEPHRASE), FALSE);
		else
			EnableWindow (GetDlgItem (hwnd, IDC_CHANGEPHRASE), TRUE);

		// display photo ID
		if (pclpss->hbitmapPhotoID) 
		{
			DeleteObject (pclpss->hbitmapPhotoID);
			pclpss->hbitmapPhotoID = NULL;
		}
		if (pclpss->pPhotoBuffer) 
		{
			PGPFreeData (pclpss->pPhotoBuffer);
			pclpss->pPhotoBuffer = NULL;
		}
		if (pclpss->hbitmapPhotoID) 
		{
			DeleteObject (pclpss->hbitmapPhotoID);
			pclpss->hbitmapPhotoID = NULL;
		}
		if (pclpss->hpalettePhotoID) 
		{
			DeleteObject (pclpss->hpalettePhotoID);
			pclpss->hpalettePhotoID = NULL;
		}

		b = TRUE;
		if (IsntPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		{
			PGPGetPrefBoolean (prefref, 
					kPGPPrefMarginalIsInvalid, &b);
		}
		if (b) 
			pclpss->iValidityThreshold = PGPCL_VALIDITY_COMPLETE;
		else
			pclpss->iValidityThreshold = PGPCL_VALIDITY_MARGINAL;

		PGPGetPrimaryAttributeUserID (pclpss->key, 
				kPGPAttribute_Image, &userid);
		if (userid == NULL) 
		{
			ShowWindow (GetDlgItem (hwnd, IDC_PHOTOSCROLLBAR), SW_HIDE);
			pclpss->iImageIndex = -1;
		}
		else 
		{
			PGPKeyIterRef	keyiter;
			INT				iNumPhotoID;
			PGPUInt32		iType;
			PGPBoolean		bAttrib;
			SCROLLINFO		si;

			if (pclpss->iImageIndex < 0)
				pclpss->iImageIndex = 0;

			// get number of Photo IDs
			PGPNewKeyIterFromKeyDB (pclpss->keydbBacking, &keyiter);
			PGPKeyIterSeek (keyiter, pclpss->key);
			PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_UserID, &userid);
			iNumPhotoID = 0;
			while (userid) 
			{
				PGPGetKeyDBObjBooleanProperty (userid, 
						kPGPUserIDProperty_IsAttribute, &bAttrib);
				if (userid == pclpss->userid)
					pclpss->iImageIndex = iNumPhotoID;

				if (bAttrib) 
				{
					PGPGetKeyDBObjNumericProperty (userid, 
							kPGPUserIDProperty_AttributeType, &iType);
					if (iType == kPGPAttribute_Image)
						iNumPhotoID++;
				}
				PGPKeyIterNextKeyDBObj (
						keyiter, kPGPKeyDBObjType_UserID, &userid);
			}
			pclpss->iNumPhotoIDs = iNumPhotoID;

			// setup photoid scrollbar
			if (iNumPhotoID <= 1) 
				ShowWindow (GetDlgItem (hwnd, IDC_PHOTOSCROLLBAR), SW_HIDE);
			else 
			{
				si.cbSize = sizeof(SCROLLINFO);
				si.fMask = SIF_ALL;
				si.nMin = 0;
				si.nMax = iNumPhotoID-1;
				si.nPage = 1;
				si.nPos = pclpss->iImageIndex;
				SetScrollInfo (GetDlgItem (hwnd, IDC_PHOTOSCROLLBAR), SB_CTL,
								&si, TRUE);
				ShowWindow (GetDlgItem (hwnd, IDC_PHOTOSCROLLBAR), SW_SHOW);
			}

			// reset userid to selected one
			PGPKeyIterSeek (keyiter, pclpss->key);
			PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_UserID, &userid);
			iNumPhotoID = -1;
			while (userid) 
			{
				PGPGetKeyDBObjBooleanProperty (userid, 
						kPGPUserIDProperty_IsAttribute, &bAttrib);
				if (bAttrib) 
				{
					PGPGetKeyDBObjNumericProperty (userid, 
							kPGPUserIDProperty_AttributeType, &iType);
					if (iType == kPGPAttribute_Image)
						iNumPhotoID++;
				}
				if (pclpss->iImageIndex == iNumPhotoID) 
				{
					PGPGetKeyDBObjNumericProperty (userid, 
							kPGPUserIDProperty_Validity, &u);
					u = CLConvertFromPGPValidity (u);
					if (u < (UINT)pclpss->iValidityThreshold)
						pclpss->bPhotoInvalid = TRUE;
					else
						pclpss->bPhotoInvalid = FALSE;
					break;
				}
				PGPKeyIterNextKeyDBObj (
						keyiter, kPGPKeyDBObjType_UserID, &userid);
			}
			PGPFreeKeyIter (keyiter);

			// get length of photo id buffer
			PGPGetKeyDBObjDataProperty (userid, 
					kPGPUserIDProperty_AttributeData, 
					NULL, 1, &(pclpss->iPhotoBufferLength));

			if (pclpss->iPhotoBufferLength > 0) 
			{
				// now actually get the buffer data
				pclpss->pPhotoBuffer = PGPNewData (
						PGPPeekContextMemoryMgr (pclpss->context), 
						pclpss->iPhotoBufferLength, 0);

				if (pclpss->pPhotoBuffer)
				{
					PGPGetKeyDBObjDataProperty (userid, 
							kPGPUserIDProperty_AttributeData,
							pclpss->pPhotoBuffer, pclpss->iPhotoBufferLength,  
							&(pclpss->iPhotoBufferLength));
				}

				if (IsntPGPError (PGPclDIBfromPhoto (
						pclpss->context, pclpss->pPhotoBuffer, 
						pclpss->iPhotoBufferLength, TRUE, &lpbmi))) 
				{
					if (lpbmi) 
					{
						PGPclGetDIBSize (lpbmi, &(pclpss->iwidthPhotoID),
								&(pclpss->iheightPhotoID), NULL);
						PGPclDDBfromDIB (lpbmi, &pclpss->hbitmapPhotoID,
								&(pclpss->hpalettePhotoID)); 

						PGPFreeData (lpbmi);
					}
				}
			}
		}

		if (pclpss->hwndRevokerDlg) 
		{
			SendMessage (pclpss->hwndRevokerDlg, WM_APP, wParam, 0);
			InvalidateRect (pclpss->hwndRevokers, NULL, TRUE);
			UpdateWindow (pclpss->hwndRevokers);
		}
		
		// register this window with the parent for update notifications
		PostMessage (pclpss->hwndParent, PGPCL_M_REGISTERPROPERTYWINDOW, 
				(WPARAM)GetParent(hwnd), (LPARAM)hwnd);

		pclpss->bReadyToPaint = TRUE;
		InvalidateRect (hwnd, NULL, FALSE);
		break;
	}

	case PGPCL_M_PROPERTYMESSAGE :
		switch (wParam) {
		case kPGPclProperties_UpdateObject :
			PostMessage (hwnd, WM_APP, 0, 0);
			break;

		case kPGPclProperties_DisableWindow :
			EnableWindow (GetDlgItem (hwnd, IDC_TRUSTSLIDER), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_TRUSTTEXT1), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_TRUSTTEXT2), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_AXIOMATIC), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_ENABLED), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_CHANGEPHRASE), FALSE);
			EnableWindow (GetDlgItem (GetParent(hwnd), IDOK), FALSE);
			break;

		case kPGPclProperties_CloseWindow :
			break;

		case kPGPclProperties_RefreshPhotoID :
			pclpss->userid = (PGPKeyDBObjRef)lParam;
			PostMessage (hwnd, WM_APP, 0, 0);
			break;
		}
		break;

	case WM_PAINT :
		if (pclpss->bReadyToPaint)
		{
			sPaintPhotoID (hwnd, 
					pclpss->hbitmapPhotoID, pclpss->hpalettePhotoID,
					pclpss->iwidthPhotoID, pclpss->iheightPhotoID,
					pclpss->bPhotoInvalid, pclpss->bRevoked,
					pclpss->bExpired);
		}
		else 
		{
			PAINTSTRUCT	ps;

			BeginPaint (hwnd, &ps);
			EndPaint (hwnd, &ps);
		}
		break;

	case WM_PALETTECHANGED :
		if ((HWND)wParam != hwnd) 
			sUpdatePalette (hwnd, pclpss);
		break;

	case WM_QUERYNEWPALETTE :
		return (sUpdatePalette (hwnd, pclpss));

	case WM_HSCROLL :
	case WM_VSCROLL :
	{
		UINT	u, uTrust;

		// check if originating from trust slider
		if ((HWND)lParam == GetDlgItem (hwnd, IDC_TRUSTSLIDER)) 
		{
			if (LOWORD (wParam) == TB_ENDTRACK) 
			{
				u = SendDlgItemMessage (hwnd, IDC_TRUSTSLIDER, TBM_GETPOS, 
										0, 0);
				uTrust = sConvertToPGPTrust (u);
				if (uTrust == pclpss->uTrust) break;

				if ((pclpss->uValidity <= kPGPValidity_Invalid) && 
					(uTrust >= kPGPKeyTrust_Marginal)) 
				{
					sSetTrustControls (hwnd, pclpss, kPGPKeyTrust_Never);
					PGPclMessageBox (hwnd, IDS_CAPTIONINFO, 
							IDS_TRUSTONINVALIDKEY, 
							MB_OK|MB_ICONINFORMATION);
				}
				else 
				{
					pclpss->uTrust = uTrust;
					if (IsntPGPError (PGPclErrorBox (hwnd, 
						PGPSetKeyTrust (pclpss->key, uTrust))))
					{
						// this will cause reload message on closing
						SendMessage (pclpss->hwndParent, 
								PGPCL_M_PROPERTYMESSAGE, 
								kPGPclProperties_UpdateObject, 
								(LPARAM)(pclpss->key));
					}
				}
			}
		}
		// otherwise, must be coming from photoid scrollbar
		else 
		{
			INT	iNewPos;

			iNewPos = pclpss->iImageIndex;

			switch (LOWORD(wParam)) {
			case SB_TOP :
				iNewPos = 0;
				break;

			case SB_BOTTOM :
				iNewPos = pclpss->iNumPhotoIDs-1;
				break;

			case SB_PAGELEFT :
			case SB_LINELEFT :
				iNewPos = pclpss->iImageIndex-1;
				if (iNewPos < 0) iNewPos = 0;
				break;

			case SB_PAGERIGHT :
			case SB_LINERIGHT :
				iNewPos = pclpss->iImageIndex+1;
				if (iNewPos >= pclpss->iNumPhotoIDs) 
					iNewPos = pclpss->iNumPhotoIDs-1;
				break;

			case SB_THUMBPOSITION :
			case SB_THUMBTRACK :
				iNewPos = HIWORD(wParam);

			default :
				break;
			}

			if (iNewPos != pclpss->iImageIndex) 
			{
				pclpss->userid = kInvalidPGPKeyDBObjRef;
				pclpss->iImageIndex = iNewPos;
				PostMessage (hwnd, WM_APP, 0, 0);
			}
		}
		return 0;
	}

	case WM_DESTROY: 
		sDestroySubKeyListAndStructures (pclpss);

		SetWindowLong(GetDlgItem(hwnd, IDC_PHOTOID), 
				GWL_WNDPROC, (LONG)pclpss->wpOrigProcPhoto);
		
		SetWindowLong(GetDlgItem(hwnd, IDC_ONTOKEN), 
				GWL_WNDPROC, (LONG)pclpss->wpOrigProcBox); 

		if (pclpss->hbitmapPhotoID) 
		{
			DeleteObject (pclpss->hbitmapPhotoID);
			pclpss->hbitmapPhotoID = NULL;
		}
		if (pclpss->hpalettePhotoID) 
		{
			DeleteObject (pclpss->hpalettePhotoID);
			pclpss->hpalettePhotoID = NULL;
		}
		if (pclpss->pPhotoBuffer) 
		{
			PGPFreeData (pclpss->pPhotoBuffer);
			pclpss->pPhotoBuffer = NULL;
		}

		PGPFreeData (pclpss);
		break; 

	case WM_COMMAND:
		switch(LOWORD (wParam)) {
			case IDC_CHANGEPHRASE :
				if (pclpss->bSecret) 
				{
					PGPclChangePhrase (pclpss->context, pclpss->tlscontext,
							GetParent (hwnd), pclpss->key, TRUE);
					SendMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE, 
							kPGPclProperties_UpdateObject,
							(LPARAM)(pclpss->key));
					PostMessage (hwnd, WM_APP, 0, 0);
				}
				break;

			case IDC_AXIOMATIC :
				if (IsDlgButtonChecked (
							hwnd, IDC_AXIOMATIC) == BST_CHECKED) 
				{
					if (IsDlgButtonChecked (
							hwnd, IDC_ENABLED) == BST_UNCHECKED) 
					{
						PGPSetKeyEnabled (pclpss->key, TRUE);
					}
					PGPSetKeyAxiomatic (pclpss->key, TRUE,
							PGPOLastOption (pclpss->context));
				}
				else
					PGPSetKeyAxiomatic (pclpss->key, FALSE,
							PGPOLastOption (pclpss->context));

				SendMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE, 
						kPGPclProperties_UpdateObject, (LPARAM)(pclpss->key));
				PostMessage (hwnd, WM_APP, 0, 0);
				break;
				
			case IDC_ENABLED :
				if (IsDlgButtonChecked (
							hwnd, IDC_ENABLED) == BST_CHECKED) 
					PGPSetKeyEnabled (pclpss->key, TRUE);
				else 
					PGPSetKeyEnabled (pclpss->key, FALSE);

				SendMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE, 
						kPGPclProperties_UpdateObject, (LPARAM)(pclpss->key));
				PostMessage (hwnd, WM_APP, 0, 0);
				break;

			case IDM_COPYBITMAP :
				PGPclCopyPhotoToClipboard (pclpss->context, hwnd, 
						pclpss->pPhotoBuffer, pclpss->iPhotoBufferLength);
				break;

			case IDC_USEHEXFINGERPRINT :
			{
				PGPPrefRef		prefref;

				if (IsDlgButtonChecked (
						hwnd, IDC_USEHEXFINGERPRINT) == BST_CHECKED) 
					pclpss->bShowHexFingerprint = TRUE;
				else
					pclpss->bShowHexFingerprint = FALSE;

				sSetFingerprintControls (hwnd, 
						pclpss->bShowHexFingerprint, 
						pclpss->key, pclpss->uKeyVer);

				if (IsntPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
				{
					PGPSetPrefBoolean (prefref, 
							kPGPPrefUseHexFingerprint, 
							pclpss->bShowHexFingerprint);
					PGPclFlushClientLibPrefs (prefref, NULL);
				}
				break;
			}
		}
		return TRUE;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aKeyPropIds);

	case WM_NOTIFY :
		switch (((NMHDR FAR *) lParam)->code) {

		case PSN_HELP :
			PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0, 
					(char*)kPGPclHelpFile, 
					(DWORD*)IDH_PGPCL_PROPGENERALDIALOG);
			break;

		case PSN_APPLY :
			// send message to main window, requesting that it destroy us
			PostMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE,
					kPGPclProperties_CloseWindow, (LPARAM)(pclpss->key));
			break;
		}
		return TRUE;
	}
	return FALSE;
}


//	___________________________________________________
//
//  Key Properties Dialog Message procedure - General panel

static BOOL CALLBACK 
sCertPropDlgProc (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam,
		LPARAM	lParam) 
{
	PCLPROPSHEETSTRUCT	pclpss;

	pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

	switch (uMsg) {

	case WM_INITDIALOG:
	{
		HWND	hwndParent;
		RECT	rc;

		// store pointer to data structure
		SetWindowLong (hwnd, GWL_USERDATA, ((PROPSHEETPAGE*)lParam)->lParam);
		pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		// center dialog on screen
		hwndParent = GetParent (hwnd);
		if (pclpss->bNeedsCentering) 
			PGPclCenterWindowOnMonitor (hwndParent, kPGPclWindowMonitor);

		// disable and hide cancel button; and move "OK" button over
		SendMessage (hwndParent, PSM_CANCELTOCLOSE, 0, 0);
		GetWindowRect (GetDlgItem (hwndParent, IDCANCEL), &rc);
		MapWindowPoints (NULL, hwndParent, (LPPOINT)&rc, 2);
		SetWindowPos (GetDlgItem (hwndParent, IDOK), NULL, rc.left,
				rc.top, rc.right-rc.left, rc.bottom-rc.top,
				SWP_NOZORDER);
		ShowWindow (GetDlgItem (hwndParent, IDCANCEL), SW_HIDE);

		// initialize all controls
		PostMessage (hwnd, WM_APP, 0, 0);
		return TRUE;
	}

	case WM_APP :
	{
		PGPError			err;
		PGPBoolean			b;
		PGPSize				size;
		CHAR				sz[kPGPMaxUserIDSize];
		PGPTime				tm;
		PGPKeyDBObjRef		keySigner;
		PGPKeyDBObjRef		certSigner;
		PGPKeyID			keyid;
		LPSTR				psz;


		pclpss = (PCLPROPSHEETSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		UTF8EditInit (GetDlgItem (hwnd, IDC_NAME));

		// initialize name control
		if (pclpss->bX509)
		{
			psz = NULL;
			PGPGetKeyDBObjAllocatedDataProperty (pclpss->cert, 
					kPGPSigProperty_X509LongName, &psz, &size);
			if (IsntNull (psz))
			{
				UTF8EditSetText (GetDlgItem (hwnd, IDC_NAME), psz);
				PGPFreeData (psz);
			}

			psz = NULL;
			PGPGetKeyDBObjAllocatedDataProperty (pclpss->cert, 
					kPGPSigProperty_X509IssuerLongName, &psz, &size);
			if (IsntNull (psz))
			{
				SetDlgItemText (hwnd, IDC_ISSUER, psz);
				PGPFreeData (psz);
			}

			PGPGetKeyDBObjBooleanProperty (pclpss->key, 
					kPGPKeyProperty_HasCRL, &b);
			if (b)
			{
				PGPGetKeyDBObjTimeProperty (pclpss->key,
						kPGPKeyProperty_CRLThisUpdate, &tm);
				CLConvertTimeToString (tm, sz, sizeof (sz));
				SetDlgItemText (hwnd, IDC_LASTCRL, sz);

				PGPGetKeyDBObjTimeProperty (pclpss->key,
						kPGPKeyProperty_CRLNextUpdate, &tm);
				CLConvertTimeToString (tm, sz, sizeof (sz));
				SetDlgItemText (hwnd, IDC_NEXTCRL, sz);
			}
			else
			{
				LoadString (g_hInst, IDS_NA, sz, sizeof(sz));
				SetDlgItemText (hwnd, IDC_LASTCRL, sz);
				SetDlgItemText (hwnd, IDC_NEXTCRL, sz);
			}
		}
		else
		{
			PGPUInt32	u;

			err = PGPGetSigCertifierKey (
					pclpss->cert, kInvalidPGPKeyDBRef, &keySigner);
			if (err == kPGPError_ItemNotFound) 
			{
				err = kPGPError_NoErr;
				keySigner = NULL;
			}

			if (!PGPKeyDBObjRefIsValid (keySigner) && 
				(pclpss->keydbBacking != pclpss->keydbAlt)) 
			{
				err = PGPGetSigCertifierKey (	
						pclpss->cert, pclpss->keydbAlt, &keySigner);
			}

			if (err == kPGPError_ItemNotFound) 
			{
				err = kPGPError_NoErr;
				keySigner = NULL;
			}

			PGPclErrorBox (hwnd, err);
			if (PGPKeyDBObjRefIsValid (keySigner))
				PGPclGetPrimaryUserIDNameUTF8 (keySigner, sz, sizeof(sz), &size);
			else 
				LoadString (g_hInst, IDS_UNKNOWN, sz, sizeof(sz));
			UTF8EditSetText (GetDlgItem (hwnd, IDC_NAME), sz);

			PGPGetKeyDBObjNumericProperty (pclpss->cert, 
					kPGPSigProperty_TrustLevel, &u);
			SetDlgItemInt (hwnd, IDC_TRUSTDEPTH, u, FALSE);

			// initialize domain restriction control
			psz = NULL;
			err = PGPGetKeyDBObjAllocatedDataProperty (pclpss->cert, 
					kPGPSigProperty_RegularExpression, &psz, &size);
			if (IsNull (psz))
			{
				LoadString (g_hInst, IDS_NA, sz, sizeof(sz));
				SetDlgItemText (hwnd, IDC_DOMAIN, sz);
			}
			else
			{
				if (IsntPGPError (err))
					SetDlgItemText (hwnd, IDC_DOMAIN, psz);
				PGPFreeData (psz);
			}
		}

		// initialize keyid control
		PGPGetKeyDBObjDataProperty (pclpss->cert, 
				kPGPSigProperty_KeyID, &keyid, sizeof(keyid), &size);
		PGPGetKeyIDString (&keyid, kPGPKeyIDString_Abbreviated, sz);
		SetDlgItemText (hwnd, IDC_KEYID, sz);

		// initialize cert creation date edit control
		PGPGetKeyDBObjTimeProperty (pclpss->cert, 
				kPGPSigProperty_Creation, &tm);
		CLConvertTimeToString (tm, sz, sizeof (sz));
		SetDlgItemText (hwnd, IDC_CREATIONDATE, sz);

		// initialize cert expiration date edit control
		PGPGetKeyDBObjTimeProperty (pclpss->cert, 
				kPGPSigProperty_Expiration, &tm);
		if (tm != kPGPExpirationTime_Never) 
			CLConvertTimeToString (tm, sz, sizeof (sz));
		else 
			LoadString (g_hInst, IDS_NEVER, sz, sizeof (sz));
		SetDlgItemText (hwnd, IDC_EXPIRATIONDATE, sz);

		// initialize checkboxes
		PGPGetKeyDBObjBooleanProperty (pclpss->cert, 
				kPGPSigProperty_IsRevoked, &b);
		CheckDlgButton (
				hwnd, IDC_REVOKED, (b ? BST_CHECKED : BST_UNCHECKED));

		PGPGetKeyDBObjBooleanProperty (pclpss->cert, 
				kPGPSigProperty_IsExpired, &b);
		CheckDlgButton (
				hwnd, IDC_EXPIRED, (b ? BST_CHECKED : BST_UNCHECKED));

		PGPGetKeyDBObjBooleanProperty (pclpss->cert, 
				kPGPSigProperty_IsExportable, &b);
		CheckDlgButton (
				hwnd, IDC_EXPORTABLE, (b ? BST_CHECKED : BST_UNCHECKED));

		// subclass checkboxes
		pclpss->wpOrigProcBox = 
				(WNDPROC) SetWindowLong(GetDlgItem (hwnd, IDC_REVOKED), 
						GWL_WNDPROC, (LONG) sCheckboxSubclassProc); 
		SetWindowLong(GetDlgItem (hwnd, IDC_EXPIRED), 
						GWL_WNDPROC, (LONG) sCheckboxSubclassProc); 
		SetWindowLong(GetDlgItem (hwnd, IDC_EXPORTABLE), 
						GWL_WNDPROC, (LONG) sCheckboxSubclassProc); 

		// initialize show certifier button
		if (pclpss->bX509)
		{
			err = PGPGetSigX509CertifierSig (
					pclpss->cert, kInvalidPGPKeyDBRef, &certSigner);
			// if this is a "self-sig" then disable button
			if (certSigner == pclpss->cert)
				err = kPGPError_KeyInvalid;
		}
		else
		{
			err = PGPGetSigCertifierKey (
					pclpss->cert, kInvalidPGPKeyDBRef, &keySigner);
		}
		EnableWindow (
				GetDlgItem (hwnd, IDC_SHOWSIGNER), (IsntPGPError (err)));

		// initialize icon
		pclpss->iImageIndex = 
				PGPclDetermineCertIcon (pclpss->cert, NULL, NULL);

		InvalidateRect (hwnd, NULL, FALSE);
		break;
	}

	case WM_DESTROY :
		SetWindowLong(GetDlgItem(hwnd, IDC_REVOKED), 
				GWL_WNDPROC, (LONG)pclpss->wpOrigProcBox); 
		SetWindowLong(GetDlgItem(hwnd, IDC_EXPIRED), 
				GWL_WNDPROC, (LONG)pclpss->wpOrigProcBox); 
		SetWindowLong(GetDlgItem(hwnd, IDC_EXPORTABLE), 
				GWL_WNDPROC, (LONG)pclpss->wpOrigProcBox); 

		UTF8EditDestroy (GetDlgItem (hwnd, IDC_NAME));
		PGPFreeData (pclpss);
		break;

	case WM_PAINT :
	{
		PAINTSTRUCT ps;
		HDC			hdc;
		RECT		rc;
		INT			ix, iy;

		hdc = BeginPaint (hwnd, &ps);
		if (g_hilKeys)
		{
			GetWindowRect (GetDlgItem (hwnd, IDC_ICONFRAME), &rc);
			MapWindowPoints (NULL, hwnd, (LPPOINT)&rc, 2);
			ix = rc.left + (rc.right-rc.left)/2 - 8;
			iy = rc.top + (rc.bottom-rc.top)/2 - 8;
			ImageList_Draw (g_hilKeys, pclpss->iImageIndex, 
						hdc, ix, iy, ILD_TRANSPARENT);
		}
		EndPaint (hwnd, &ps);
		break;
	}

	case WM_HELP: 
	case WM_CONTEXTMENU :
		if (pclpss->bX509)
		{
			return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
					(char*)kPGPclHelpFile, aX509certIds);
		}
		else
		{
			return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
					(char*)kPGPclHelpFile, aPGPcertIds);
		}
		break; 

	case WM_NOTIFY :
		switch (((NMHDR FAR *) lParam)->code) {

		case PSN_HELP :
			if (pclpss->bX509)
			{
				PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0, 
						(char*)kPGPclHelpFile, 
						(DWORD*)IDH_PGPCL_X509CERTDIALOG);
			}
			else
			{
				PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0, 
						(char*)kPGPclHelpFile, 
						(DWORD*)IDH_PGPCL_PGPCERTDIALOG);
			}
			break;

		case PSN_APPLY :
			// send message to main window, requesting that it destroy us
			PostMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE,
					kPGPclProperties_CloseWindow, (LPARAM)(pclpss->cert));
			break;
		}
		break;

	case WM_COMMAND:
	{
		PGPError			err;
		PGPKeyDBObjRef		keySigner;
		PGPKeyDBObjRef		certSigner;

		switch(LOWORD (wParam)) {
		case IDC_SHOWSIGNER :
			if (pclpss->bX509)
			{
				err = PGPGetSigX509CertifierSig (
						pclpss->cert, pclpss->keydbAlt, &certSigner);
				if (IsntPGPError (PGPclErrorBox (hwnd, err)))
				{
					PostMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE,
							kPGPclProperties_ShowCertProps, (LPARAM)certSigner);
				}
			}
			else
			{
				err = PGPGetSigCertifierKey (
						pclpss->cert, pclpss->keydbAlt, &keySigner);
				if (IsntPGPError (PGPclErrorBox (hwnd, err)))
				{
					PostMessage (pclpss->hwndParent, PGPCL_M_PROPERTYMESSAGE,
							kPGPclProperties_ShowKeyProps, (LPARAM)keySigner);
				}
			}
			break;
		}
		return TRUE;
	}
 	}
	return FALSE;
}

static HWND
sKeyPropertiesW(
	PCLPROPSHEETSTRUCT	pclpss,
	PGPBoolean			bModeless) 
{
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	HWND				hwnd			= NULL;
	WCHAR				wszTitle[32]={0};
	WCHAR				wszUserName[kPGPMaxUserIDSize]={0};
    PROPSHEETPAGEW		pspShPageW={0};
    PROPSHEETHEADERW	pshShHeaderW={0};
	HPROPSHEETPAGE		hpsp[4]={0};
	PGPUInt32			puOutLen=0;
	PGPError			err;

	pclpss->iImageIndex			= -1;

	// get the "show hex fingerprint" flag
	err = PGPclPeekClientLibPrefRefs (&prefref, NULL);
	if (IsntPGPError (err))
	{
		PGPGetPrefBoolean (prefref, kPGPPrefUseHexFingerprint, 
				&pclpss->bShowHexFingerprint);

		if(PGPlnEnterprise())
		{
			pclpss->bSubkeyOpsDisabled = TRUE;
			pclpss->bSubkeyGenEnabled = FALSE;

			PGPGetPrefBoolean (prefref, kPGPPrefPresetKeyGeneration, 
					&pclpss->bSubkeyOpsDisabled);
			PGPGetPrefBoolean (prefref, kPGPPrefAllowKeyGeneration, 
					&pclpss->bSubkeyGenEnabled);
		}
		else
		{
			pclpss->bSubkeyOpsDisabled = FALSE;
			pclpss->bSubkeyGenEnabled = TRUE;
		}
	}

	PGPCountAdditionalRecipientRequests (pclpss->key, &pclpss->uNumberADKs);
	PGPCountRevocationKeys (pclpss->key, &pclpss->uNumberRevokers);
	PGPGetKeyDBObjNumericProperty (pclpss->key, 
			kPGPKeyProperty_AlgorithmID, &pclpss->uKeyAlg);
	PGPGetKeyDBObjNumericProperty (pclpss->key,
			kPGPKeyProperty_Version, &pclpss->uKeyVer);

	LoadStringW (g_hInst, IDS_PROPTITLEGENERAL, wszTitle, sizeof(wszTitle));
	pshShHeaderW.nPages			= 1;		//always show the general sheet
    pspShPageW.dwSize			= sizeof(PROPSHEETPAGEW);
    pspShPageW.dwFlags			= PSP_USETITLE | PSP_HASHELP;
    pspShPageW.hInstance		= g_hInst;
	pspShPageW.pszTemplate		= MAKEINTRESOURCEW (IDD_KEYPROPGENERAL);
    pspShPageW.pszIcon			= NULL;
    pspShPageW.pfnDlgProc		= sKeyPropDlgProcGeneral;
    pspShPageW.pszTitle			= wszTitle;
    pspShPageW.lParam			= (LPARAM)pclpss;
    pspShPageW.pfnCallback		= NULL;
	hpsp[0]				= CreatePropertySheetPageW (&pspShPageW);

	LoadStringW (g_hInst, IDS_PROPTITLESUBKEYS, wszTitle, sizeof(wszTitle));
	pspShPageW.dwSize			= sizeof(PROPSHEETPAGEW);
	pspShPageW.dwFlags			= PSP_USETITLE | PSP_HASHELP;
	pspShPageW.hInstance		= g_hInst;
	pspShPageW.pszTemplate		= MAKEINTRESOURCEW (IDD_KEYPROPSUBKEYS);
	pspShPageW.pszIcon			= NULL;
	pspShPageW.pfnDlgProc		= sKeyPropDlgProcSubkey;
	pspShPageW.pszTitle			= wszTitle;
	pspShPageW.lParam			= (LPARAM)pclpss;
	pspShPageW.pfnCallback		= NULL;
	hpsp[pshShHeaderW.nPages]	= CreatePropertySheetPageW (&pspShPageW);
	pshShHeaderW.nPages++;

	if (pclpss->uNumberADKs > 0) 
	{
		LoadStringW (g_hInst, IDS_PROPTITLEADKS, wszTitle, sizeof(wszTitle));
	    pspShPageW.dwSize			= sizeof(PROPSHEETPAGEW);
	    pspShPageW.dwFlags			= PSP_USETITLE | PSP_HASHELP;
	    pspShPageW.hInstance		= g_hInst;
		pspShPageW.pszTemplate		= MAKEINTRESOURCEW (IDD_KEYPROPADK);
	    pspShPageW.pszIcon			= NULL;
	    pspShPageW.pfnDlgProc		= sKeyPropDlgProcADK;
	    pspShPageW.pszTitle			= wszTitle;
		pspShPageW.lParam			= (LPARAM)pclpss;
	    pspShPageW.pfnCallback		= NULL;
		hpsp[pshShHeaderW.nPages]	= CreatePropertySheetPageW (&pspShPageW);
		pshShHeaderW.nPages++;
	}

	if (pclpss->uNumberRevokers > 0) 
	{
		LoadStringW (g_hInst, IDS_PROPTITLEREVOKERS, wszTitle, sizeof(wszTitle));
	    pspShPageW.dwSize			= sizeof(PROPSHEETPAGEW);
	    pspShPageW.dwFlags			= PSP_USETITLE | PSP_HASHELP;
	    pspShPageW.hInstance		= g_hInst;
		pspShPageW.pszTemplate		= MAKEINTRESOURCEW (IDD_KEYPROPREVOKERS);
	    pspShPageW.pszIcon			= NULL;
	    pspShPageW.pfnDlgProc		= sKeyPropDlgProcRevokers;
	    pspShPageW.pszTitle			= wszTitle;
		pspShPageW.lParam			= (LPARAM)pclpss;
	    pspShPageW.pfnCallback		= NULL;
		hpsp[pshShHeaderW.nPages]	= CreatePropertySheetPageW (&pspShPageW);
		pshShHeaderW.nPages++;
	}

	PGPclGetPrimaryUserIDNameUCS2(pclpss->key, wszUserName, sizeof(wszUserName), &puOutLen);

    pshShHeaderW.dwSize = sizeof(PROPSHEETHEADERW);
    pshShHeaderW.dwFlags = PSH_NOAPPLYNOW;
	if (bModeless)
	{
		pshShHeaderW.dwFlags |= PSH_MODELESS;
		pshShHeaderW.hwndParent = NULL;
	}
	else
	{
		pshShHeaderW.hwndParent = pclpss->hwndParent;
	}

    pshShHeaderW.hInstance	= g_hInst;
    pshShHeaderW.pszIcon		= NULL;
    pshShHeaderW.pszCaption	=  wszUserName;

    pshShHeaderW.nStartPage	= 0;
    pshShHeaderW.phpage		= &hpsp[0];
    pshShHeaderW.pfnCallback = NULL;

    hwnd = (HWND)PropertySheetW (&pshShHeaderW);

	return hwnd;
}

//	___________________________________________________
//
//  Post Key Properties Dialog 

static HWND
sKeyProperties (
	PCLPROPSHEETSTRUCT	pclpss,
	PGPBoolean			bModeless) 
{
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	HWND				hwnd			= NULL;

	CHAR				szTitle[32];
	CHAR				szUserName[kPGPMaxUserIDSize];
    PROPSHEETPAGE		psp;
    PROPSHEETHEADER		psh;
	HPROPSHEETPAGE		hpsp[4];
	INT					i;
	PGPError			err;

	//if we are capable of unicode, call the unicode version of this function
	if (g_bUnicodeOS)
		return sKeyPropertiesW (pclpss, bModeless);

	pclpss->iImageIndex			= -1;

	// get the "show hex fingerprint" flag
	err = PGPclPeekClientLibPrefRefs (&prefref, NULL);
	if (IsntPGPError (err))
	{
		PGPGetPrefBoolean (prefref, kPGPPrefUseHexFingerprint, 
				&pclpss->bShowHexFingerprint);

		if(PGPlnEnterprise())
		{
			pclpss->bSubkeyOpsDisabled = TRUE;
			pclpss->bSubkeyGenEnabled = FALSE;

			PGPGetPrefBoolean (prefref, kPGPPrefPresetKeyGeneration, 
					&pclpss->bSubkeyOpsDisabled);
			PGPGetPrefBoolean (prefref, kPGPPrefAllowKeyGeneration, 
					&pclpss->bSubkeyGenEnabled);
		}
		else
		{
			pclpss->bSubkeyOpsDisabled = FALSE;
			pclpss->bSubkeyGenEnabled = TRUE;
		}
	}

	PGPCountAdditionalRecipientRequests (pclpss->key, &pclpss->uNumberADKs);
	PGPCountRevocationKeys (pclpss->key, &pclpss->uNumberRevokers);
	PGPGetKeyDBObjNumericProperty (pclpss->key, 
			kPGPKeyProperty_AlgorithmID, &pclpss->uKeyAlg);
	PGPGetKeyDBObjNumericProperty (pclpss->key,
			kPGPKeyProperty_Version, &pclpss->uKeyVer);

	LoadString (g_hInst, IDS_PROPTITLEGENERAL, szTitle, sizeof(szTitle));
	psh.nPages			= 1;		//always show the general sheet
    psp.dwSize			= sizeof(PROPSHEETPAGE);
    psp.dwFlags			= PSP_USETITLE | PSP_HASHELP;
    psp.hInstance		= g_hInst;
	psp.pszTemplate		= MAKEINTRESOURCE (IDD_KEYPROPGENERAL);
    psp.pszIcon			= NULL;
    psp.pfnDlgProc		= sKeyPropDlgProcGeneral;
    psp.pszTitle		= szTitle;
    psp.lParam			= (LPARAM)pclpss;
    psp.pfnCallback		= NULL;
	hpsp[0]				= CreatePropertySheetPage (&psp);

	LoadString (g_hInst, IDS_PROPTITLESUBKEYS, szTitle, sizeof(szTitle));
	psp.dwSize			= sizeof(PROPSHEETPAGE);
	psp.dwFlags			= PSP_USETITLE | PSP_HASHELP;
	psp.hInstance		= g_hInst;
	psp.pszTemplate		= MAKEINTRESOURCE (IDD_KEYPROPSUBKEYS);
	psp.pszIcon			= NULL;
	psp.pfnDlgProc		= sKeyPropDlgProcSubkey;
	psp.pszTitle		= szTitle;
	psp.lParam			= (LPARAM)pclpss;
	psp.pfnCallback		= NULL;
	hpsp[psh.nPages]	= CreatePropertySheetPage (&psp);
	psh.nPages++;

	if (pclpss->uNumberADKs > 0) 
	{
		LoadString (g_hInst, IDS_PROPTITLEADKS, szTitle, sizeof(szTitle));
	    psp.dwSize			= sizeof(PROPSHEETPAGE);
	    psp.dwFlags			= PSP_USETITLE | PSP_HASHELP;
	    psp.hInstance		= g_hInst;
		psp.pszTemplate		= MAKEINTRESOURCE (IDD_KEYPROPADK);
	    psp.pszIcon			= NULL;
	    psp.pfnDlgProc		= sKeyPropDlgProcADK;
	    psp.pszTitle		= szTitle;
		psp.lParam			= (LPARAM)pclpss;
	    psp.pfnCallback		= NULL;
		hpsp[psh.nPages]	= CreatePropertySheetPage (&psp);
		psh.nPages++;
	}

	if (pclpss->uNumberRevokers > 0) 
	{
		LoadString (g_hInst, IDS_PROPTITLEREVOKERS, szTitle, sizeof(szTitle));
	    psp.dwSize			= sizeof(PROPSHEETPAGE);
	    psp.dwFlags			= PSP_USETITLE | PSP_HASHELP;
	    psp.hInstance		= g_hInst;
		psp.pszTemplate		= MAKEINTRESOURCE (IDD_KEYPROPREVOKERS);
	    psp.pszIcon			= NULL;
	    psp.pfnDlgProc		= sKeyPropDlgProcRevokers;
	    psp.pszTitle		= szTitle;
		psp.lParam			= (LPARAM)pclpss;
	    psp.pfnCallback		= NULL;
		hpsp[psh.nPages]	= CreatePropertySheetPage (&psp);
		psh.nPages++;
	}

	PGPclGetPrimaryUserIDName (pclpss->key, szUserName, sizeof(szUserName), &i);
    psh.dwSize = sizeof(PROPSHEETHEADER);
    psh.dwFlags = PSH_NOAPPLYNOW;
	if (bModeless)
	{
		psh.dwFlags |= PSH_MODELESS;
		psh.hwndParent = NULL;
	}
	else
	{
		psh.hwndParent = pclpss->hwndParent;
	}

    psh.hInstance	= g_hInst;
    psh.pszIcon		= NULL;
    psh.pszCaption	= (LPSTR) szUserName;

    psh.nStartPage	= 0;
    psh.phpage		= &hpsp[0];
    psh.pfnCallback = NULL;

    hwnd = (HWND)PropertySheet (&psh);

	return hwnd;
}


//	___________________________________________________
//
//  Post Cert Properties Dialog 

static HWND
sCertProperties (
	PCLPROPSHEETSTRUCT	pclpss,
	PGPBoolean			bModeless) 
{
	CHAR				szTitle[32];
    PROPSHEETPAGE		psp;
    PROPSHEETHEADER		psh;
	HPROPSHEETPAGE		hpsp[4];
	HWND				hwnd;

	pclpss->iImageIndex			= -1;

	PGPGetKeyDBObjBooleanProperty (pclpss->cert, 
			kPGPSigProperty_IsX509, &pclpss->bX509);

	if (pclpss->bX509)
		psp.pszTemplate = MAKEINTRESOURCE (IDD_X509CERTPROP);
	else
		psp.pszTemplate = MAKEINTRESOURCE (IDD_PGPCERTPROP);

	pclpss->key = PGPPeekKeyDBObjKey (pclpss->cert);

	psh.nPages		= 1;		
	psp.dwSize		= sizeof(PROPSHEETPAGE);
	psp.dwFlags		= PSP_HASHELP;
	psp.hInstance	= g_hInst;
	psp.pszIcon		= NULL;
	psp.pfnDlgProc	= sCertPropDlgProc;
	psp.pszTitle	= szTitle;
	psp.lParam		= (LPARAM)pclpss;
	psp.pfnCallback = NULL;
	hpsp[0]			= CreatePropertySheetPage (&psp);

	LoadString (g_hInst, IDS_CERTPROPSTITLE, szTitle, sizeof(szTitle));
	psh.dwSize		= sizeof(PROPSHEETHEADER);
	psh.dwFlags		= PSH_NOAPPLYNOW;

	if (bModeless)
	{
		psh.dwFlags |= PSH_MODELESS;
		psh.hwndParent = NULL;
	}
	else
	{
		psh.hwndParent = pclpss->hwndParent;
	}

	psh.hInstance	= g_hInst;
	psh.pszIcon		= NULL;
	psh.pszCaption	= (LPSTR) szTitle;

	psh.nStartPage	= 0;
	psh.phpage		= &hpsp[0];
	psh.pfnCallback = NULL;

	hwnd = (HWND)PropertySheet (&psh);

	return hwnd;
}


//	___________________________________________________
//
//  display the appropriate properties dialog for the object

PGPError PGPclExport
PGPclKeyDBObjProperties (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBObjRef		obj,
		PGPKeyDBRef			keydbMain,
		PGPUInt32			uFlags,
		HWND*				phwnd)
{
	PCLPROPSHEETSTRUCT	pclpss		= NULL;
	PGPBoolean			bModeless	= FALSE;
	HWND				hwnd		= NULL;

	pclpss = PGPNewData (PGPPeekContextMemoryMgr (context), 
			sizeof (CLPROPSHEETSTRUCT), kPGPMemoryMgrFlags_Clear);

	if (IsNull (pclpss))
		return kPGPError_OutOfMemory;

	pclpss->context			= context;
	pclpss->tlscontext		= tlscontext;
	pclpss->hwndParent		= hwndParent;

	pclpss->keydbBacking	= PGPPeekKeyDBObjKeyDB (obj);
	if (keydbMain != pclpss->keydbBacking)
		pclpss->keydbAlt = keydbMain;

	if (uFlags & kPGPclProperties_ReadOnly)
		pclpss->bReadOnly = TRUE;

	if (uFlags & kPGPclProperties_CenterWindow)
		pclpss->bNeedsCentering = TRUE;

	if (uFlags & kPGPclProperties_Modeless)
		bModeless = TRUE;

	if (uFlags & kPGPclProperties_Key)
	{
		pclpss->key = obj;
		hwnd = sKeyProperties (pclpss, bModeless);
	}

	else if (uFlags & kPGPclProperties_UserID)
	{
		pclpss->key = PGPPeekKeyDBObjKey (obj);
		pclpss->userid = obj;
		hwnd = sKeyProperties (pclpss, bModeless);
	}

	else if (uFlags & kPGPclProperties_Signature)
	{
		pclpss->cert = obj;
		hwnd = sCertProperties (pclpss, bModeless);
	}

	if (bModeless)
	{
		if (phwnd)
			*phwnd = hwnd;
	}
	else
		SetActiveWindow (hwndParent);

	return kPGPError_NoErr;
}

