/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	CLsdksrv.c - wrapper functions for sdkui keyserver calls

	$Id: CLsdksrv.c,v 1.20 2002/10/29 18:30:41 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "pgpclx.h"
#include "pgpUnicodeWin32.h"

// external globals
extern HINSTANCE g_hInst;

//	___________________________________________________
//
//	all keyserver queries are funneled through here

static PGPError
sQueryServerInternal (
	HWND					hwndParent,
	PGPContextRef			context,
	PGPtlsContextRef		tlsContext,
	PGPOptionListRef		optionSearch,
	UINT					uServer,
	LPSTR					pszUserID,
	PGPKeyID*				pkeyid,
	PGPKeyDBRef				keydbMain,
	PGPKeyDBRef*			pkeydbResult)
{
	PGPError				err				= kPGPError_NoErr;
	PGPKeyServerEntry*		pentryList		= NULL;
	PGPKeyServerSpec*		pserverList		= NULL;
	PGPInt32				iNumEntries		= 0;
	PCLIENTSERVERSTRUCT		pcss			= NULL;

	err = CLInitKeyServerPrefs (uServer, NULL,
			hwndParent, context, keydbMain, pszUserID,
			&pcss, &pentryList, &pserverList, &iNumEntries);

	if (IsntPGPError (err))
	{
		if (hwndParent || (pkeyid == NULL))
		{
			err = PGPSearchKeyServerDialog (context, iNumEntries,
					pserverList, tlsContext,
					FALSE, pkeydbResult, optionSearch,
					PGPOUIParentWindowHandle (context, hwndParent),
					PGPOLastOption (context));
		}
		else
		{
			PGPFilterRef		filter	= kInvalidPGPFilterRef;
			INT					i;

			PGPKeyServerThreadStorageRef	previousStorage;

			if (IsntNull (optionSearch))
				PGPFreeOptionList (optionSearch);

			err = PGPNewKeyDBObjDataFilter (context, 
					kPGPKeyProperty_KeyID, pkeyid, sizeof(PGPKeyID), 
					kPGPMatchCriterion_Equal, &filter);

			PGPKeyServerInit ();
			PGPKeyServerCreateThreadStorage (&previousStorage);

			for (i=0; i<iNumEntries; i++)
			{
				err = PGPKeyServerOpen (pserverList[i].server, NULL);
				if (IsntPGPError (err))
				{
					err = PGPQueryKeyServer (
							pserverList[i].server, filter, pkeydbResult);
					PGPKeyServerClose (pserverList[i].server);

					if (IsntPGPError (err))
					{
						PGPUInt32	uNumKeys	= 0;
						PGPCountKeys (
								PGPPeekKeyDBRootKeySet (*pkeydbResult), &uNumKeys);
						if (uNumKeys > 0)
							break;
						else
						{
							PGPFreeKeyDB (*pkeydbResult);
							*pkeydbResult = kInvalidPGPKeyDBRef;
						}
					}
				}
			}

			PGPKeyServerDisposeThreadStorage (previousStorage);
			PGPKeyServerCleanup ();

			if (PGPFilterRefIsValid (filter))
				PGPFreeFilter (filter);
		}
	}

	CLUninitKeyServerPrefs (uServer,
			pcss, pentryList, pserverList, iNumEntries);

	return err;
}

//	___________________________________________________
//
//	all keyserver sends are funneled through here

static PGPError
sSendToServerInternal (
		HWND					hwndParent,
		PGPContextRef			context,
		PGPtlsContextRef		tlsContext,
		PGPclServerType			servertype,
		PGPKeyServerEntry*		pkeyserver,
		LPSTR					pszUserID,
		PGPKeyDBRef				keydbMain,
		PGPKeySetRef			keysetToSend)
{
	PGPError				err				= kPGPError_NoErr;
	PGPKeyServerEntry*		pentryList		= NULL;
	PGPKeyServerSpec*		pserverList		= NULL;
	PGPInt32				iNumEntries		= 0;
	PCLIENTSERVERSTRUCT		pcss			= NULL;
	PGPKeySetRef			keysetFailed	= kInvalidPGPKeySetRef;

	if(IsntPGPError(err))
	{
		err = CLInitKeyServerPrefs (servertype, pkeyserver,
				hwndParent, context, keydbMain, pszUserID,
				&pcss, &pentryList, &pserverList, &iNumEntries);
	}

	if (IsntPGPError (err))
	{
		err = PGPSendToKeyServerDialog (context, &(pserverList[0]),
				tlsContext, keysetToSend, &keysetFailed,
				PGPOUIParentWindowHandle (context, hwndParent),
				PGPOLastOption (context));

		if (PGPKeySetRefIsValid (keysetFailed))
			PGPFreeKeySet (keysetFailed);
	}

	CLUninitKeyServerPrefs (servertype,
			pcss, pentryList, pserverList, iNumEntries);

	return err;
}

//	___________________________________________________
//
//	send keyset to keyserver

PGPError PGPclExport
PGPclSendKeysToServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlsContext,
		HWND				hwndParent,
		PGPclServerType		servertype,
		PGPKeyServerEntry*	pkeyserver,
		PGPKeyDBRef			keydbMain,
		PGPKeySetRef		keysetToSend)
{
	PGPError				err				= kPGPError_NoErr;
	PGPKeySetRef			keyset			= kInvalidPGPKeySetRef;
	PGPKeyIterRef			keyiter			= kInvalidPGPKeyIterRef;

	// derive server from userid
	if (servertype == kPGPclUserIDBasedServer)
	{
		PGPKeyDBObjRef	key		= kInvalidPGPKeyDBObjRef;

		CHAR			szUserID[kPGPMaxUserIDSize+1];
		PGPSize			size;

		err = PGPNewKeyIterFromKeySet (keysetToSend, &keyiter); CKERR;
		err = PGPKeyIterNextKeyDBObj (keyiter,
				kPGPKeyDBObjType_Key, &key); CKERR;

		while (PGPKeyDBObjRefIsValid (key))
		{
			err = PGPNewOneKeySet (key, &keyset); CKERR;

			err = PGPclGetPrimaryUserIDNameUTF8 (key,
					szUserID, sizeof(szUserID), &size); CKERR;

			err = sSendToServerInternal (hwndParent, context,
					tlsContext, servertype, pkeyserver,
					szUserID, keydbMain, keyset); CKERR;

			PGPKeyIterNextKeyDBObj (keyiter,
					kPGPKeyDBObjType_Key, &key); CKERR;
		}
	}

	// use indicated or caller-supplied server
	else
	{
		err = sSendToServerInternal (hwndParent, context,
				tlsContext, servertype, pkeyserver, "",
				keydbMain, keysetToSend);
	}

done :
	if (PGPKeyIterRefIsValid (keyiter))
		PGPFreeKeyIter (keyiter);
	if (PGPKeySetRefIsValid (keyset))
		PGPFreeKeySet (keyset);

	return err;
}

//	___________________________________________________
//
//	search keyserver(s) for userid

PGPError PGPclExport
PGPclSearchServerForUserID (
		PGPContextRef		context,
		PGPtlsContextRef	tlsContext,
		HWND				hwndParent,
		LPSTR				szUserID,
		PGPclServerType		servertype,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBRef*		pkeydbResult)
{
	PGPError				err				= kPGPError_NoErr;
	PGPKeyDBRef				keydb			= kInvalidPGPKeyDBRef;
	PGPOptionListRef		optionSearch	= kInvalidPGPOptionListRef;
	PGPFilterRef			filter			= kInvalidPGPFilterRef;
	PGPUInt32				ulen			= 0;

	char					szUTF8[kPGPMaxUserIDSize];

	// initialize return keyset
	*pkeydbResult = kInvalidPGPKeyDBRef;

	// convert userid to UTF-8
	pgpLocalStringToUTF8 (0, 0, szUserID, 
			lstrlen(szUserID), szUTF8, sizeof(szUTF8), &ulen);

	err = PGPNewKeyDBObjDataFilter (context, kPGPUserIDProperty_Name,
			szUTF8, ulen, kPGPMatchCriterion_SubString, &filter);

	if (IsntPGPError (err) &&
		PGPFilterRefIsValid (filter))
	{
		optionSearch = PGPOUIKeyServerSearchFilter (context, filter);

		if (PGPOptionListRefIsValid (optionSearch))
		{
			err = sQueryServerInternal (hwndParent, context,
					tlsContext, optionSearch, servertype,
					szUserID, NULL, keydbMain, &keydb);

			if (IsntPGPError (err))
			{
				*pkeydbResult = keydb;
				keydb = kInvalidPGPKeyDBRef;
			}
		}
	}

	if (PGPKeyDBRefIsValid (keydb))
		PGPFreeKeyDB (keydb);
	if (PGPFilterRefIsValid (filter))
		PGPFreeFilter (filter);

	return err;
}

//	___________________________________________________
//
//	Update keyset from specified keyserver

PGPError PGPclExport
PGPclUpdateKeySetFromServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlsContext,
		HWND				hwndParent,
		PGPKeySetRef		keysetToUpdate,
		PGPclServerType		servertype,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBRef*		pkeydbResult)
{
	PGPError				err				= kPGPError_NoErr;
	PGPKeyDBRef				keydb			= kInvalidPGPKeyDBRef;
	PGPKeyDBRef				keydbCollect	= kInvalidPGPKeyDBRef;
	PGPOptionListRef		optionSearch	= kInvalidPGPOptionListRef;
	PGPKeyIterRef			keyiter			= kInvalidPGPKeyIterRef;
	PGPKeySetRef			keyset			= kInvalidPGPKeySetRef;

	PGPKeyID				keyid;

	// initialize return keyset
	*pkeydbResult = kInvalidPGPKeyDBRef;

	if (servertype == kPGPclUserIDBasedServer)
	{
		PGPKeyDBObjRef	key			= kInvalidPGPKeyDBObjRef;

		CHAR			szUserID[kPGPMaxUserIDSize+1];
		PGPSize			size;

		err = PGPNewKeyDB (context, &keydbCollect); CKERR;
		err = PGPNewKeyIterFromKeySet (keysetToUpdate, &keyiter); CKERR;
		err = PGPKeyIterNextKeyDBObj (keyiter,
				kPGPKeyDBObjType_Key, &key); CKERR;

		while (	IsntPGPError (err) &&
				PGPKeyDBObjRefIsValid (key))
		{
			err = PGPclGetPrimaryUserIDNameUTF8 (key,
					szUserID, sizeof(szUserID), &size); CKERR;

			err = PGPGetKeyID (key, &keyid); CKERR;

			optionSearch = PGPOUIKeyServerSearchKeyIDList (
					context, 1, &keyid);

			if (PGPOptionListRefIsValid (optionSearch))
			{
				err = sQueryServerInternal (hwndParent, context,
						tlsContext, optionSearch, servertype,
						szUserID, &keyid, keydbMain, &keydb);

				if (hwndParent)
					PGPclErrorBox (hwndParent, err);

				if (IsntPGPError (err))
				{
					if (PGPKeyDBRefIsValid (keydb))
					{
						err = PGPCopyKeys (
								PGPPeekKeyDBRootKeySet(keydb), 
								keydbCollect, NULL);
						if (hwndParent)
							PGPclErrorBox (hwndParent, err);
						PGPFreeKeyDB (keydb);
					}
				}
				else if (err != kPGPError_UserAbort)
					err = kPGPError_NoErr;
			}

			PGPKeyIterNextKeyDBObj (keyiter,
					kPGPKeyDBObjType_Key, &key);
		}

		if (IsPGPError (err))
		{
			PGPFreeKeyDB (keydbCollect);
			keydbCollect = kInvalidPGPKeyDBRef;
		}
		else if (PGPKeyDBRefIsValid (keydbCollect))
		{
			*pkeydbResult = keydbCollect;
			keydbCollect = kInvalidPGPKeyDBRef;
		}
	}

	else
	{
		optionSearch = PGPOUIKeyServerSearchKeySet (context, keysetToUpdate);

		if (PGPOptionListRefIsValid (optionSearch))
		{
			err = sQueryServerInternal (hwndParent, context,
					tlsContext, optionSearch, servertype, "",
					NULL, keydbMain, &keydbCollect);

			if (IsntPGPError (err))
			{
				*pkeydbResult = keydbCollect;
				keydbCollect = kInvalidPGPKeyDBRef;
			}
		}
	}

done :
	if (PGPKeyIterRefIsValid (keyiter))
		PGPFreeKeyIter (keyiter);
	if (PGPKeyDBRefIsValid (keydbCollect))
		PGPFreeKeyDB (keydbCollect);

	return err;
}

//	___________________________________________________
//
//	search keyserver for keys with these keyids

PGPError PGPclExport
PGPclSearchServerForKeyIDs (
		PGPContextRef		context,
		PGPtlsContextRef	tlsContext,
		HWND				hwndParent,
		PGPKeyID*			pkeyidList,
		INT					iNumKeyIDs,
		PGPclServerType		servertype,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBRef*		pkeydbResult)
{
	PGPError				err				= kPGPError_NoErr;
	PGPKeyDBRef				keydb			= kInvalidPGPKeyDBRef;
	PGPKeyDBRef				keydbCollect	= kInvalidPGPKeyDBRef;
	PGPOptionListRef		optionSearch	= kInvalidPGPOptionListRef;
	PGPInt32				i				= 0;

	// initialize return keyset
	*pkeydbResult = kInvalidPGPKeyDBRef;

	// create keyset to collect all found keys
	err = PGPNewKeyDB (context, &keydbCollect); CKERR;

	// search for keyids one-by-one.  We have to do it this way
	// because the sdk libraries will fail if we pass a huge filter
	// to a HTTP server
	while (	IsntPGPError (err) &&
			(i < iNumKeyIDs))
	{
		optionSearch = PGPOUIKeyServerSearchKeyIDList (context,
							1, &pkeyidList[i++]);

		if (PGPOptionListRefIsValid (optionSearch)) 
		{
			err = sQueryServerInternal (hwndParent, context,
					tlsContext, optionSearch, servertype, "",
					NULL, keydbMain, &keydb);

			if (hwndParent)
				PGPclErrorBox (hwndParent, err);

			if (IsntPGPError (err))
			{
				if (PGPKeyDBRefIsValid (keydb))
				{
					err = PGPCopyKeys (
							PGPPeekKeyDBRootKeySet(keydb), keydbCollect, NULL);
					if (hwndParent)
						PGPclErrorBox (hwndParent, err);
					PGPFreeKeyDB (keydb);
				}
			}
			else if (err != kPGPError_UserAbort)
				err = kPGPError_NoErr;
		}
	}

	if (IsPGPError (err))
	{
		PGPFreeKeyDB (keydbCollect);
		keydbCollect = kInvalidPGPKeyDBRef;
	}
	else if (PGPKeyDBRefIsValid (keydbCollect))
	{
		*pkeydbResult = keydbCollect;
		keydbCollect = kInvalidPGPKeyDBRef;
	}

done:
	if (PGPKeyDBRefIsValid (keydbCollect))
		PGPFreeKeyDB (keydbCollect);

	return err;
}

//	___________________________________________________
//
//	search specified keyserver using specified filter

PGPError PGPclExport
PGPclSearchServerWithFilter (
		PGPContextRef		context,
		PGPtlsContextRef	tlsContext,
		HWND				hwndParent,
		PGPFilterRef		filter,
		PGPclServerType		servertype,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBRef*		pkeydbResult)
{
	PGPError				err				= kPGPError_NoErr;
	PGPKeyDBRef				keydb			= kInvalidPGPKeyDBRef;
	PGPOptionListRef		optionSearch	= kInvalidPGPOptionListRef;

	// initialize return keyset
	*pkeydbResult = kInvalidPGPKeyDBRef;

	optionSearch = PGPOUIKeyServerSearchFilter (context, filter);
	if (PGPOptionListRefIsValid (optionSearch))
	{
		err = sQueryServerInternal (hwndParent, context,
				tlsContext, optionSearch, servertype, "",
				NULL, keydbMain, &keydb);

		if (IsntPGPError (err))
		{
			*pkeydbResult = keydb;
			keydb = kInvalidPGPKeyDBRef;
		}
	}

	if (PGPKeyDBRefIsValid (keydb))
		PGPFreeKeyDB (keydb);

	return err;
}


//	___________________________________________________
//
//	get/update the admin-specified default keys from the default server

PGPError PGPclExport
PGPclGetDefaultKeysFromServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBRef			keydb)
{
	PGPError		err				= kPGPError_NoErr;
	PGPBoolean		bUpdate			= FALSE;
	PGPKeyDBRef		keydbResult		= kInvalidPGPKeyDBRef;
	PGPPrefRef		prefref			= kInvalidPGPPrefRef;
	PGPPrefArray*	parray			= NULL;
	PGPKeyID*		pkeyids			= NULL;

	INT				i, numKeyIDs, numToFetch;
	PGPKeyDBObjRef	key;

	err = PGPclPeekClientLibPrefRefs (&prefref, NULL);  CKERR;
	err = PGPGetPrefArray (prefref, kPGPPrefDefaultKeys, &parray);  CKERR;

	if (parray->type != kPGPPrefType_Byte)
	{
		err = kPGPError_CorruptData;
		goto done;
	}

	numKeyIDs = parray->numElements;
	if (numKeyIDs > 0)
	{
		pkeyids = PGPNewData (PGPPeekContextMemoryMgr (context), 
				numKeyIDs * sizeof(PGPKeyID), 0);
		if (IsNull (pkeyids))
		{
			err = kPGPError_OutOfMemory;
			goto done;
		}

		PGPGetPrefBoolean (prefref, kPGPPrefUpdateAllKeys, &bUpdate);
		if (!bUpdate)
		{
			PGPGetPrefBoolean (prefref, 
					kPGPPrefUpdateTrustedIntroducers, &bUpdate);
		}

		numToFetch = 0;
		for (i=0; i<numKeyIDs; i++)
		{
			pgpCopyMemory (parray->elements[i].data, 
					&pkeyids[numToFetch], sizeof(PGPKeyID));

			// if an auto-update interval is defined, only get missing keys
			if (bUpdate)
			{
				key = kInvalidPGPKeyDBObjRef;
				PGPFindKeyByKeyID (keydb, &pkeyids[numToFetch], &key);
				if (PGPKeyDBObjRefIsValid (key))
					continue;
			}

			if (IsNull (hwndParent))
			{
				err = sQueryServerInternal (NULL, context, tlscontext, 
						NULL, kPGPclDefaultServer, "", 
						&pkeyids[numToFetch], keydb, &keydbResult);

				if (IsntPGPError (err))
				{
					err = PGPCopyKeys (
							PGPPeekKeyDBRootKeySet (keydbResult), keydb, NULL);
				}

				if (PGPKeyDBRefIsValid (keydbResult))
				{
					PGPFreeKeyDB (keydbResult);
					keydbResult = kInvalidPGPKeyDBRef;
				}
			}
			else
				numToFetch++;
		}

		if (IsntNull (hwndParent))
		{
			err = PGPclSearchServerForKeyIDs (context, tlscontext, hwndParent,
					pkeyids, numToFetch, kPGPclDefaultServer, keydb,
					&keydbResult);

			if (IsntPGPError (err))
			{
				err = PGPCopyKeys (
						PGPPeekKeyDBRootKeySet (keydbResult), keydb, NULL);
			}
		}
	}

done:
	if (PGPKeyDBRefIsValid (keydbResult))
		PGPFreeKeyDB (keydbResult);

	if (IsntNull (pkeyids))
		PGPFreeData (pkeyids);

	if (IsntNull (parray))
		PGPFreePrefArray(parray);

	return err;
}


