/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	CLserver.c -	handle those types of communication with keyserver
					which are not handled by SDKuiLib

	$Id: CLserver.c,v 1.67 2002/10/31 04:13:35 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "pgpKeyServer.h"
#include "pgpRemotePrefs.h"
#include "pgpNetPrefs.h"
#include "pgpNetHostUtils.h"
#include "SpaceEdit.h"
#include "UTF8Edit.h"

#include <process.h>

// constant definitions
#define	MAX_AV			10

#define LEDTIMER		111L
#define LEDTIMERPERIOD	100L

#define NUMLEDS			10
#define LEDSPACING		2

#define MAXPROGRESSTEXT	68

#define RECONSTRUCTIONMAX			5
#define RECONSTRUCTIONTHRESHOLD		3

// keyserver operations
#define SENDGROUPS					0x01
#define GETGROUPS					0x02
#define SENDKEY						0x03
#define	DISABLEKEY					0x04
#define DELETEKEY					0x05

// CA operations
#define UPDATEREVOCATIONS			0x11
#define REQUESTCERTIFICATE			0x12
#define RETRIEVECERTIFICATE			0x13

// LDAP operations
#define SENDRECONSTRUCTIONBLOB		0x21
#define RETRIEVERECONSTRUCTIONBLOB	0x22
#define SENDPREFS					0x23
#define GETPREFS					0x24

// flags
#define USEFULLERRORTEXT			0x0001
#define TLSCONFIRMONLYIFINVALID		0x0002


// external globals
extern HINSTANCE	g_hInst;

// typedefs
typedef struct {
	PGPContextRef			context;
	PGPtlsContextRef		tlsContext;
	PGPPrefRef				prefref;
	PGPPrefRef				prefrefNet;
	HWND					hwnd;
	VOID*					lpThread;
	BOOL					bOpInProgress;
	BOOL					bCancel;
	BOOL					bThreadFree;
	INT						iStatusValue;
	INT						iStatusDirection;
	UINT					uOperation;
	UINT					uFlags;
	PGPError				err;
	PGPKeyDBObjRef			keyIn;
	PGPKeySetRef			keysetIn;
	PGPKeyDBRef				keydbOut;
	PGPKeyDBRef				keydbMain;
	PGPKeyServerKeySpace	space;
	PGPKeyServerEntry		ksentry;
	PGPKeyServerRef			server;
	PGPGroupSetRef			groupset;
	PGPAttributeValue*		pAVlist;
	PGPUInt32				numAVs;
	LPSTR					pszPassPhrase;
	PGPByte*				pPasskey;
	PGPSize					sizePasskey;
	LPSTR					pszUserID;
	LPSTR					pszUserPass;
	PGPReconPrompts			reconPrompts;
	PGPReconPasses			reconPasses;
} SERVERTHREADSTRUCT, *PSERVERTHREADSTRUCT;

// local globals
static DWORD aReconstructIds[] = {			// Help IDs
	IDC_HIDETYPING,		IDH_PGPCLRECONSTRUCT_HIDEANSWER,
	IDC_PROMPT1,		IDH_PGPCLRECONSTRUCT_PROMPT,
	IDC_PROMPT2,		IDH_PGPCLRECONSTRUCT_PROMPT,
	IDC_PROMPT3,		IDH_PGPCLRECONSTRUCT_PROMPT,
	IDC_PROMPT4,		IDH_PGPCLRECONSTRUCT_PROMPT,
	IDC_PROMPT5,		IDH_PGPCLRECONSTRUCT_PROMPT,
	IDC_ANSWER1,		IDH_PGPCLRECONSTRUCT_ANSWER,
	IDC_ANSWER2,		IDH_PGPCLRECONSTRUCT_ANSWER,
	IDC_ANSWER3,		IDH_PGPCLRECONSTRUCT_ANSWER,
	IDC_ANSWER4,		IDH_PGPCLRECONSTRUCT_ANSWER,
	IDC_ANSWER5,		IDH_PGPCLRECONSTRUCT_ANSWER,
    0,0
};

static DWORD aLDAPloginIds[] = {			// Help IDs
	IDC_USERID,			IDH_PGPCLRECONSTRUCT_USERID,
	IDC_PASSWORD,		IDH_PGPCLRECONSTRUCT_PASSWORD,
	0,0
};

//	___________________________________________________
//
//	update keyserver entry info in prefs

PGPError PGPclExport
PGPclSyncKeyserverPrefs (
		PGPContextRef		context,
		PGPKeyServerEntry*	keyserver)
{
	PGPKeyServerEntry*	keyserverList		= NULL;
	PGPPrefRef			prefRef				= kInvalidPGPPrefRef;
	PGPUInt32			keyserverCount;
	PGPError			err;

	// load keyserverprefs
	err = PGPclPeekClientLibPrefRefs (&prefRef, NULL);
	if (IsntPGPError (err))
	{
		err = PGPGetKeyServerPrefs (prefRef,
				&keyserverList, &keyserverCount);

		if (IsntPGPError (err))
		{
			PGPUInt32	index;

			for (index = 0; index < keyserverCount; index++) 
			{
				if (KeyServersAreEqual (keyserverList[index], *keyserver))
				{
					keyserverList[index].authAlg = keyserver->authAlg;
					lstrcpy (keyserverList[index].authKeyIDString,
							keyserver->authKeyIDString);

					err = PGPSetKeyServerPrefs (prefRef,
							keyserverList, keyserverCount);

					if (IsntPGPError (err))
						err = PGPclFlushClientLibPrefs (prefRef, NULL);

					break;
				}
			}

		}
	}

	if (keyserverList)
		PGPFreeKeyServerList(keyserverList);

	return err;
}


//	___________________________________________________
//
//  Dialog Message procedure

static BOOL CALLBACK
sReconstructionDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{

	PSERVERTHREADSTRUCT		psts;

	switch(uMsg) {

	case WM_INITDIALOG:
		psts = (PSERVERTHREADSTRUCT)lParam;
		SetWindowLong (hwnd, GWL_USERDATA, lParam);

		// use the spaceedit control
		SEInit (psts->context, GetDlgItem (hwnd, IDC_ANSWER1),
				TRUE, NULL, NULL);
		SEInit (psts->context, GetDlgItem (hwnd, IDC_ANSWER2),
				TRUE, NULL, NULL);
		SEInit (psts->context, GetDlgItem (hwnd, IDC_ANSWER3),
				TRUE, NULL, NULL);
		SEInit (psts->context, GetDlgItem (hwnd, IDC_ANSWER4),
				TRUE, NULL, NULL);
		SEInit (psts->context, GetDlgItem (hwnd, IDC_ANSWER5),
				TRUE, NULL, NULL);

		// and the UTF8Edit control
		UTF8EditInit (GetDlgItem (hwnd, IDC_PROMPT1));
		UTF8EditInit (GetDlgItem (hwnd, IDC_PROMPT2));
		UTF8EditInit (GetDlgItem (hwnd, IDC_PROMPT3));
		UTF8EditInit (GetDlgItem (hwnd, IDC_PROMPT4));
		UTF8EditInit (GetDlgItem (hwnd, IDC_PROMPT5));

		CheckDlgButton (hwnd, IDC_HIDETYPING, BST_CHECKED);

		if (psts->uOperation == SENDRECONSTRUCTIONBLOB)
		{
			LoadString (g_hInst, IDS_DEFAULTRECONPROMPT1,
					&psts->reconPrompts[0][0], kPGPRecon_MaxPromptLen);
			LoadString (g_hInst, IDS_DEFAULTRECONPROMPT2,
					&psts->reconPrompts[1][0], kPGPRecon_MaxPromptLen);
			LoadString (g_hInst, IDS_DEFAULTRECONPROMPT3,
					&psts->reconPrompts[2][0], kPGPRecon_MaxPromptLen);
			LoadString (g_hInst, IDS_DEFAULTRECONPROMPT4,
					&psts->reconPrompts[3][0], kPGPRecon_MaxPromptLen);
			LoadString (g_hInst, IDS_DEFAULTRECONPROMPT5,
					&psts->reconPrompts[4][0], kPGPRecon_MaxPromptLen);
		}

		UTF8EditSetText (GetDlgItem (hwnd, IDC_PROMPT1), 
				&psts->reconPrompts[0][0]);
		UTF8EditSetText (GetDlgItem (hwnd, IDC_PROMPT2), 
				&psts->reconPrompts[1][0]);
		UTF8EditSetText (GetDlgItem (hwnd, IDC_PROMPT3), 
				&psts->reconPrompts[2][0]);
		UTF8EditSetText (GetDlgItem (hwnd, IDC_PROMPT4), 
				&psts->reconPrompts[3][0]);
		UTF8EditSetText (GetDlgItem (hwnd, IDC_PROMPT5), 
				&psts->reconPrompts[4][0]);

		if (psts->uOperation == RETRIEVERECONSTRUCTIONBLOB)
		{
			SendMessage (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_PROMPT1)), 
					EM_SETREADONLY, TRUE, 0);
			SendMessage (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_PROMPT2)), 
					EM_SETREADONLY, TRUE, 0);
			SendMessage (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_PROMPT3)), 
					EM_SETREADONLY, TRUE, 0);
			SendMessage (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_PROMPT4)), 
					EM_SETREADONLY, TRUE, 0);
			SendMessage (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_PROMPT5)), 
					EM_SETREADONLY, TRUE, 0);

			ShowWindow (GetDlgItem (hwnd, IDC_SENDTEXT1), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_SENDTEXT2), SW_HIDE);
			SetFocus (GetDlgItem (hwnd, IDC_ANSWER1));

			return FALSE;
		}
		else
		{
			ShowWindow (GetDlgItem (hwnd, IDC_RETRIEVETEXT1), SW_HIDE);
			ShowWindow (GetDlgItem (hwnd, IDC_RETRIEVETEXT2), SW_HIDE);
			return TRUE;
		}

	case WM_DESTROY:
		SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER1));
		SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER2));
		SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER3));
		SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER4));
		SEWipeEditBox (GetDlgItem (hwnd, IDC_ANSWER5));

		SEDestroy (GetDlgItem (hwnd, IDC_ANSWER1));
		SEDestroy (GetDlgItem (hwnd, IDC_ANSWER2));
		SEDestroy (GetDlgItem (hwnd, IDC_ANSWER3));
		SEDestroy (GetDlgItem (hwnd, IDC_ANSWER4));
		SEDestroy (GetDlgItem (hwnd, IDC_ANSWER5));

		UTF8EditDestroy (GetDlgItem (hwnd, IDC_PROMPT1));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_PROMPT2));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_PROMPT3));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_PROMPT4));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_PROMPT5));
		break;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam,
				(char*)kPGPclHelpFile, aReconstructIds);

	case WM_COMMAND:

		switch(LOWORD (wParam)) {
		case IDOK:
		{
			INT		iNumAnswers		= 0;
			INT		i;

			psts = (PSERVERTHREADSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			// first collect prompts
			UTF8EditGetText (GetDlgItem (hwnd, IDC_PROMPT1),
					&psts->reconPrompts[0][0], kPGPRecon_MaxPromptLen);
			UTF8EditGetText (GetDlgItem (hwnd, IDC_PROMPT2),
					&psts->reconPrompts[1][0], kPGPRecon_MaxPromptLen);
			UTF8EditGetText (GetDlgItem (hwnd, IDC_PROMPT3),
					&psts->reconPrompts[2][0], kPGPRecon_MaxPromptLen);
			UTF8EditGetText (GetDlgItem (hwnd, IDC_PROMPT4),
					&psts->reconPrompts[3][0], kPGPRecon_MaxPromptLen);
			UTF8EditGetText (GetDlgItem (hwnd, IDC_PROMPT5),
					&psts->reconPrompts[4][0], kPGPRecon_MaxPromptLen);

			// now get passphrases
			i = SEGetTextLength (GetDlgItem (hwnd, IDC_ANSWER1));
			if ((i == 0) && (psts->uOperation == SENDRECONSTRUCTIONBLOB))
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
						IDS_NEEDRECONANSWER, MB_OK|MB_ICONEXCLAMATION);
				SetFocus (GetDlgItem (hwnd, IDC_ANSWER1));
				break;
			}
			else if (i >= kPGPRecon_MaxPassLen)
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
						IDS_RECONANSWERTOOLONG, MB_OK|MB_ICONEXCLAMATION);
				SetFocus (GetDlgItem (hwnd, IDC_ANSWER1));
				break;
			}
			else if (i > 0)
			{
				SEGetText (GetDlgItem (hwnd, IDC_ANSWER1),
						&psts->reconPasses[0][0],
						kPGPRecon_MaxPassLen +1);
				iNumAnswers++;
			}

			i = SEGetTextLength (GetDlgItem (hwnd, IDC_ANSWER2));
			if ((i == 0) && (psts->uOperation == SENDRECONSTRUCTIONBLOB))
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
						IDS_NEEDRECONANSWER, MB_OK|MB_ICONEXCLAMATION);
				SetFocus (GetDlgItem (hwnd, IDC_ANSWER2));
				break;
			}
			else if (i >= kPGPRecon_MaxPassLen)
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
						IDS_RECONANSWERTOOLONG, MB_OK|MB_ICONEXCLAMATION);
				SetFocus (GetDlgItem (hwnd, IDC_ANSWER2));
				break;
			}
			else if (i > 0)
			{
				SEGetText (GetDlgItem (hwnd, IDC_ANSWER2),
						&psts->reconPasses[1][0],
						kPGPRecon_MaxPassLen +1);
				iNumAnswers++;
			}

			i = SEGetTextLength (GetDlgItem (hwnd, IDC_ANSWER3));
			if ((i == 0) && (psts->uOperation == SENDRECONSTRUCTIONBLOB))
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
						IDS_NEEDRECONANSWER, MB_OK|MB_ICONEXCLAMATION);
				SetFocus (GetDlgItem (hwnd, IDC_ANSWER3));
				break;
			}
			else if (i >= kPGPRecon_MaxPassLen)
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
						IDS_RECONANSWERTOOLONG, MB_OK|MB_ICONEXCLAMATION);
				SetFocus (GetDlgItem (hwnd, IDC_ANSWER3));
				break;
			}
			else if (i > 0)
			{
				SEGetText (GetDlgItem (hwnd, IDC_ANSWER3),
						&psts->reconPasses[2][0],
						kPGPRecon_MaxPassLen +1);
				iNumAnswers++;
			}

			i = SEGetTextLength (GetDlgItem (hwnd, IDC_ANSWER4));
			if ((i == 0) && (psts->uOperation == SENDRECONSTRUCTIONBLOB))
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
						IDS_NEEDRECONANSWER, MB_OK|MB_ICONEXCLAMATION);
				SetFocus (GetDlgItem (hwnd, IDC_ANSWER4));
				break;
			}
			else if (i >= kPGPRecon_MaxPassLen)
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
						IDS_RECONANSWERTOOLONG, MB_OK|MB_ICONEXCLAMATION);
				SetFocus (GetDlgItem (hwnd, IDC_ANSWER4));
				break;
			}
			else if (i > 0)
			{
				SEGetText (GetDlgItem (hwnd, IDC_ANSWER4),
						&psts->reconPasses[3][0],
						kPGPRecon_MaxPassLen +1);
				iNumAnswers++;
			}

			i = SEGetTextLength (GetDlgItem (hwnd, IDC_ANSWER5));
			if ((i == 0) && (psts->uOperation == SENDRECONSTRUCTIONBLOB))
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
						IDS_NEEDRECONANSWER, MB_OK|MB_ICONEXCLAMATION);
				SetFocus (GetDlgItem (hwnd, IDC_ANSWER5));
				break;
			}
			else if (i >= kPGPRecon_MaxPassLen)
			{
				PGPclMessageBox (hwnd, IDS_CAPTIONALERT,
						IDS_RECONANSWERTOOLONG, MB_OK|MB_ICONEXCLAMATION);
				SetFocus (GetDlgItem (hwnd, IDC_ANSWER5));
				break;
			}
			else if (i > 0)
			{
				SEGetText (GetDlgItem (hwnd, IDC_ANSWER5),
						&psts->reconPasses[4][0],
						kPGPRecon_MaxPassLen +1);
				iNumAnswers++;
			}

			if (iNumAnswers < RECONSTRUCTIONTHRESHOLD)
			{
				CHAR	sz1[128];
				CHAR	sz2[256];

				LoadString (g_hInst, IDS_BELOWRECONTHRESH, sz1, sizeof(sz1));
				wsprintf (sz2, sz1, RECONSTRUCTIONTHRESHOLD);
				LoadString (g_hInst, IDS_CAPTIONALERT, sz1, sizeof(sz1));

				MessageBox (hwnd, sz2, sz1, MB_OK|MB_ICONEXCLAMATION);
			}
			else
				EndDialog (hwnd, 1);

			return TRUE;
		}

		case IDCANCEL:
			EndDialog (hwnd, 0);
			return TRUE;

		case IDC_HIDETYPING:
		{
			BOOL	bHideTyping;

			if (IsDlgButtonChecked (hwnd, IDC_HIDETYPING) == BST_CHECKED)
				bHideTyping = TRUE;
			else
				bHideTyping = FALSE;

			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_ANSWER1), bHideTyping);
			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_ANSWER2), bHideTyping);
			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_ANSWER3), bHideTyping);
			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_ANSWER4), bHideTyping);
			SEChangeHideTyping (
					GetDlgItem (hwnd, IDC_ANSWER5), bHideTyping);
			break;
		}

		case IDC_MOREINFO :
		{
			INT		iHelpID;

			psts = (PSERVERTHREADSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			if (psts->uOperation == SENDRECONSTRUCTIONBLOB)
				iHelpID = IDH_PGPCLRECONSTRUCT_MOREINFOSEND;
			else
				iHelpID = IDH_PGPCLRECONSTRUCT_MOREINFOGET;

			PGPclHtmlHelp (hwnd, kPGPclHelpMoreInformation, 0, 0,
					(char*)kPGPclHelpFile, (DWORD*)iHelpID);
			break;
		}
		}
		return TRUE;
	}
	return FALSE;
}


//	___________________________________________________
//
//  Dialog Message procedure

static BOOL CALLBACK
sLDAPUserNameDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{

	PSERVERTHREADSTRUCT		psts;

	switch(uMsg) {

	case WM_INITDIALOG:
		psts = (PSERVERTHREADSTRUCT)lParam;
		SetWindowLong (hwnd, GWL_USERDATA, lParam);
		return TRUE;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam,
				(char*)kPGPclHelpFile, aLDAPloginIds);

	case WM_COMMAND:
		switch(LOWORD (wParam)) {
		case IDOK:
		{
			INT	i;
			psts = (PSERVERTHREADSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			i = GetWindowTextLength (GetDlgItem (hwnd, IDC_USERID));
			if (i > 0)
			{
				psts->pszUserID = PGPNewData (
						PGPPeekContextMemoryMgr (psts->context), i+1, 0);
				if (psts->pszUserID)
					GetDlgItemText (hwnd, IDC_USERID, psts->pszUserID, i+1);
			}

			i = GetWindowTextLength (GetDlgItem (hwnd, IDC_PASSWORD));
			if (i > 0)
			{
				psts->pszUserPass = PGPNewData (
						PGPPeekContextMemoryMgr (psts->context), i+1, 0);
				if (psts->pszUserPass)
					GetDlgItemText (hwnd, IDC_PASSWORD, psts->pszUserPass, i+1);
			}

			EndDialog (hwnd, 1);
			return TRUE;
		}

		case IDCANCEL:
			EndDialog (hwnd, 0);
			return TRUE;
		}
		return TRUE;
	}
	return FALSE;
}


//	___________________________________________________
//
//	event handler for keyserver functions

static PGPError
sServerEventHandler(
		PGPContextRef	context,
		PGPEvent*		pevent,
		PGPUserValue	userValue)
{
	PSERVERTHREADSTRUCT		psts		= (PSERVERTHREADSTRUCT)userValue;
	PGPError				err			= kPGPError_NoErr;
	BOOL					bUserCancel = FALSE;
	BOOL					bAllow		= FALSE;

	if (psts->bCancel)
		return kPGPError_UserAbort;

	switch (pevent->type) {

	case kPGPEvent_KeyServerEvent:
	{
		INT						ids		= 0;
		PGPEventKeyServerData	data	= pevent->data.keyServerData;

		PGPclServerEvent		serverevent;

		if (psts->hwnd == NULL)
			break;

		memset(&serverevent, 0x00, sizeof(PGPclServerEvent));

		serverevent.nmhdr.hwndFrom = (HWND)psts->hwnd;
		serverevent.nmhdr.idFrom = 0;
		serverevent.nmhdr.code = kPGPclNotifyServerProgress;
		serverevent.cancel = FALSE;
		serverevent.step = kPGPServerInfinite;
		serverevent.total = kPGPServerInfinite;

		switch (data.state) {
		case kPGPKeyServerState_Opening :
									ids = IDS_CONNECTING;	break;
		case kPGPKeyServerState_Querying :
									ids = IDS_QUERYING;		break;
		case kPGPKeyServerState_ReceivingResults :
									ids = IDS_RECEIVING;	break;
		case kPGPKeyServerState_ProcessingResults :
									ids = IDS_PROCESSING;	break;
		case kPGPKeyServerState_Uploading :
									ids = IDS_EXCHANGING;	break;
		case kPGPKeyServerState_Deleting :
									ids = IDS_DELETING;		break;
		case kPGPKeyServerState_Disabling :
									ids = IDS_DISABLING;	break;
		case kPGPKeyServerState_Closing :
									ids = IDS_CLOSING;		break;
		}

		if (ids)
		{
			LoadString (g_hInst, ids, serverevent.szmessage,
						sizeof(serverevent.szmessage));
			lstrcat (serverevent.szmessage, psts->ksentry.serverDNS);
		}

		SendMessage (psts->hwnd, WM_NOTIFY,
							(WPARAM)(psts->hwnd), (LPARAM)&serverevent);
		bUserCancel = serverevent.cancel;

		break;
	}

	case kPGPEvent_KeyServerSignEvent:
	{
		PGPKeyDBObjRef	keySigning	= kInvalidPGPKeyDBObjRef;
		PGPByte*		pPasskey	= NULL;
		PGPSize			sizePasskey;
		CHAR			sz[64];

		if (!psts->bCancel) {
			LoadString (g_hInst, IDS_PHRASEPROMPT, sz, sizeof(sz));

			err = PGPclGetSigningKeyPhrase (
				psts->context,		// in context
				psts->tlsContext,	// in tlscontext
				psts->hwnd,			// in hwnd of parent
				sz,					// in prompt
				psts->keydbMain,	// in keyset
				FALSE,				// in reject split keys
				&keySigning,		// in/out signing key
				NULL,				// out phrase
				&pPasskey,			// out passkey buffer
				&sizePasskey);		// out passkey length

			if (IsntPGPError(err)) 
			{
				err = PGPAddJobOptions (pevent->job,
						PGPOSignWithKey (
							psts->context,
							keySigning,
							(pPasskey) ?
								PGPOPasskeyBuffer (psts->context,
									pPasskey, sizePasskey) :
								PGPONullOption (psts->context),
							PGPOLastOption (psts->context)),
						PGPOClearSign (psts->context, TRUE),
						PGPOLastOption (psts->context));
			}

			// burn and free passkey
			if (pPasskey)
				PGPFreeData (pPasskey);
		}
		break;
	}

	case kPGPEvent_KeyServerTLSEvent:
	{
		PGPclAuthType	auth;

		switch (psts->uOperation) {
		case GETPREFS :
			auth = kPGPclAuthDoNotConfirm;
			break;

		case SENDRECONSTRUCTIONBLOB :
			if (psts->uFlags & TLSCONFIRMONLYIFINVALID)
				auth = kPGPclAuthConfirmIfInvalid;
			else
				auth = kPGPclAuthConfirmAlways;
			break;

		case RETRIEVERECONSTRUCTIONBLOB :
			auth = kPGPclAuthConfirmAlways;
			break;

		default :
			auth = kPGPclAuthUseAppropriate;
			break;
		}

		err = CLHandleTLSEvent (context, pevent, psts->hwnd,
				psts->keydbMain, &psts->ksentry, auth, &bAllow);

		if (psts->uOperation == GETPREFS)
		{
			if (err == kPGPError_UserAbort)
			{
				if (psts->hwnd != NULL)
				{
					PGPclMessageBox (psts->hwnd, IDS_CAPTIONALERT, 
							IDS_BADPREFSSERVERKEY, MB_ICONEXCLAMATION|MB_OK);
				}
			}
		}

		break;
	}
	}

	if (psts->bCancel || bUserCancel)
		err = kPGPError_UserAbort;

	return err;
}

//	___________________________________________________
//
//	thread routine to handle all keyserver operations

static DWORD
sKeyserverThreadRoutine (LPVOID lpvoid)
{
	PSERVERTHREADSTRUCT			psts			= (PSERVERTHREADSTRUCT)lpvoid;
	PGPPrefRef					prefref			= kInvalidPGPPrefRef;
	HWND						hwndParent		= psts->hwnd;
	PGPError					err				= kPGPError_NoErr;
	PGPKeyDBObjRef				key				= kInvalidPGPKeyDBObjRef;
	PGPKeySetRef				keysetFailed	= kInvalidPGPKeySetRef;

	PGPKeyServerAccessType		ksaccess	= kPGPKeyServerAccessType_Normal;

	PGPclServerEvent				event;
	BOOL							bThreadFree;
	PGPKeyServerThreadStorageRef	previousStorage;

	if(PGPlnEnterprise())
	{
		if (PGPclIsAdminInstall ())
			ksaccess = kPGPKeyServerAccessType_Administrator;
	}

	memset (&event, 0x00, sizeof(event));
	event.nmhdr.hwndFrom = hwndParent;
	event.nmhdr.idFrom = 0;
	event.nmhdr.code = kPGPclNotifyServerProgress;
	event.pData = NULL;
	event.cancel = FALSE;
	event.step = kPGPServerInfinite;
	event.total = kPGPServerInfinite;

	err = PGPKeyServerInit ();
	if (IsntPGPError (err))
	{
		PGPKeyServerCreateThreadStorage(&previousStorage);

		// check for cases where we use root server exclusively
		if ((psts->uOperation == GETGROUPS) ||
			(psts->uOperation == SENDGROUPS) ||
			(psts->uOperation == SENDKEY))
		{
			err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
			err = PGPGetRootKeyServer (prefref, &psts->ksentry);
			if (err == kPGPError_ItemNotFound)
			{
				PGPclMessageBox (hwndParent, IDS_CAPTIONINFO,
						IDS_NOROOTSERVER, MB_OK|MB_ICONINFORMATION);
				err = kPGPError_UserAbort;
				goto done;
			}
		}

		// send message to update status text
		LoadString (g_hInst, IDS_LOOKINGFORSERVER, event.szmessage,
				sizeof(event.szmessage));
		lstrcat (event.szmessage, psts->ksentry.serverDNS);
		SendMessage (hwndParent,
				WM_NOTIFY, (WPARAM)hwndParent, (LPARAM)&event);

		// we need to set keyserver access type to admin to allow
		// users to delete their own keys.
		if (psts->uOperation == DELETEKEY)
			ksaccess = kPGPKeyServerAccessType_Administrator;

		err = PGPNewKeyServer (psts->context,
				psts->ksentry.type,
				&psts->server,
				PGPONetHostName (psts->context, psts->ksentry.serverDNS,
						psts->ksentry.serverPort),
				PGPOKeyServerProtocol (psts->context, psts->ksentry.protocol),
				PGPOKeyServerAccessType (psts->context, ksaccess),
				PGPOKeyServerKeySpace (psts->context, psts->space),
				PGPOKeyServerKeyStoreDN (psts->context, 
						(const char *)&psts->ksentry.keystoredn),
				PGPOLastOption (psts->context));

		if (IsntPGPError (err))
		{
			PGPtlsSessionRef tls = kInvalidPGPtlsSessionRef;

			PGPSetKeyServerEventHandler (psts->server,
							sServerEventHandler, psts);

			if ((psts->ksentry.protocol ==
								kPGPKeyServerProtocol_LDAPS) ||
				(psts->ksentry.protocol ==
								kPGPKeyServerProtocol_HTTPS))
			{
				err = PGPNewTLSSession (psts->tlsContext, &tls);

				if (IsPGPError (err))
				{
					// warn beyond the icon?
					tls = kInvalidPGPtlsSessionRef;
				}
			}

			// if disabling or deleting using TLS, then get local key
			if (((psts->uOperation == DISABLEKEY) ||
				 (psts->uOperation == DELETEKEY) ||
				 (psts->uOperation == SENDGROUPS)) &&
				 PGPtlsSessionRefIsValid (tls))
			{
				PGPKeyDBObjRef	keyAuth			= kInvalidPGPKeyDBObjRef;
				PGPByte*		pPasskey		= NULL;
				LPSTR			pszPhrase		= NULL;
				PGPSize			sizePasskey;
				CHAR			sz[64];

				LoadString (g_hInst, IDS_PHRASEPROMPT, sz, sizeof(sz));

				err = PGPclGetSigningKeyPhrase (
					psts->context,		// in context
					psts->tlsContext,	// in tlscontext
					psts->hwnd,			// in hwnd of parent
					sz,					// in prompt
					psts->keydbMain,	// in keyset
					FALSE,				// in reject split keys
					&keyAuth,			// in/out signing key
					&pszPhrase,			// out phrase
					&pPasskey,			// out passkey buffer
					&sizePasskey);		// out passkey length

				if (IsntPGPError (err))
				{
					if (pszPhrase)
						err = PGPtlsSetLocalPrivateKey (tls, keyAuth,
							kInvalidPGPKeySetRef,
							PGPOPassphrase (psts->context, pszPhrase),
							PGPOLastOption (psts->context));
					else
						err = PGPtlsSetLocalPrivateKey (tls, keyAuth,
							kInvalidPGPKeySetRef,
							(pPasskey) ?
								PGPOPasskeyBuffer (psts->context,
									pPasskey, sizePasskey) :
								PGPONullOption (psts->context),
							PGPOLastOption (psts->context));
				}

				// burn and free phrase
				if (IsntNull (pszPhrase))
					PGPclFreePhrase (pszPhrase);
				if (pPasskey)
					PGPFreeData (pPasskey);
			}

			// perform the keyserver operation
			if (IsntPGPError (err))
			{
				err = PGPKeyServerOpen (psts->server, tls);
				if (IsntPGPError (err))
				{
					switch (psts->uOperation) {
					case GETGROUPS :
						err = PGPRetrieveGroupsFromServer (psts->server,
										&psts->groupset);
						break;

					case SENDGROUPS :
						err = PGPSendGroupsToServer (psts->server,
										psts->groupset);
						break;

					case SENDKEY :
						err = PGPUploadToKeyServer (psts->server,
										psts->keysetIn,
										&keysetFailed);
						if (PGPKeySetRefIsValid (keysetFailed))
							PGPFreeKeySet (keysetFailed);
						break;

					case DISABLEKEY :
						err = PGPDisableFromKeyServer (psts->server,
										psts->keysetIn,
										&keysetFailed);
						if (PGPKeySetRefIsValid (keysetFailed))
							PGPFreeKeySet (keysetFailed);
						break;

					case DELETEKEY :
						err = PGPDeleteFromKeyServer (psts->server,
										psts->keysetIn,
										&keysetFailed);
						if (PGPKeySetRefIsValid (keysetFailed))
							PGPFreeKeySet (keysetFailed);
						break;
					}
					PGPKeyServerClose (psts->server);
				}
			}

			PGPFreeKeyServer (psts->server);
			psts->server = kInvalidPGPKeyServerRef;

			if (PGPtlsSessionRefIsValid (tls))
				PGPFreeTLSSession (tls);
		}

		PGPKeyServerDisposeThreadStorage(previousStorage);

		PGPKeyServerCleanup ();
	}

done :
	// let window know that we're done
	psts->err = err;
	event.szmessage[0] = '\0';
	if (err == kPGPError_UserAbort)
	{
		event.nmhdr.code = kPGPclNotifyServerAbort;
		LoadString (g_hInst, IDS_SERVERABORT, event.szmessage,
					sizeof(event.szmessage));
		lstrcat (event.szmessage, psts->ksentry.serverDNS);
	}
	else if (IsPGPError (err))
	{
		event.nmhdr.code = kPGPclNotifyServerError;
		if (psts->uFlags & USEFULLERRORTEXT)
		{
			PGPclErrorToString (err,
					event.szmessage, sizeof(event.szmessage), NULL);
		}
		else
		{
			LoadString (g_hInst, IDS_SERVERERROR, event.szmessage,
					sizeof(event.szmessage));
			lstrcat (event.szmessage, psts->ksentry.serverDNS);
		}
	}
	else
	{
		event.nmhdr.code = kPGPclNotifyServerDone;
		LoadString (g_hInst, IDS_SERVEROK, event.szmessage,
					sizeof(event.szmessage));
		lstrcat (event.szmessage, psts->ksentry.serverDNS);
	}

	bThreadFree = psts->bThreadFree;
	SendMessage (hwndParent,
			WM_NOTIFY, (WPARAM)hwndParent, (LPARAM)&event);

	// if thread should free data structure, do it
	if (bThreadFree)
	{
		if (psts->keysetIn)
			PGPFreeKeySet (psts->keysetIn);
		free (psts);
	}

	return 0;
}


//	___________________________________________________
//
//	get the first key from a keyset

static PGPError
sGetFirstKeyInSet(
	PGPKeySetRef	 	keyset,
	PGPKeyDBObjRef*		pkey)
{
	PGPError		err;
	PGPKeyIterRef	keyiter;

	*pkey = kInvalidPGPKeyDBObjRef;

	err = PGPNewKeyIterFromKeySet (keyset, &keyiter);
	if (IsntPGPError (err))
	{
		err = PGPKeyIterNextKeyDBObj (keyiter,
				kPGPKeyDBObjType_Key, pkey);

		if (err == kPGPError_EndOfIteration)
			err = kPGPError_ItemNotFound;

		PGPFreeKeyIter (keyiter);
	}

	return err;
}


//	___________________________________________________
//
//	create the CA cert server request

static PGPError
sCreateServerRequest (
		PSERVERTHREADSTRUCT		psts,
		PGPKeyDBObjRef			key,
		PGPByte*				pPasskey,
		PGPSize					sizePasskey,
		PVOID*					ppRequest,
		PGPSize*				psizeRequest)
{
	PVOID				pRequest		= NULL;
	PVOID				pBuffer			= NULL;
	LPSTR				pszPhrase		= NULL;

	PGPUInt32			uExportFormat;
	PGPUInt32			uOutputFormat;
	PGPSize				sizeRequest;
	PGPSize				sizeBuffer;
	PGPError			err;

	if (!PGPKeyDBObjRefIsValid (key))
		sGetFirstKeyInSet (psts->keysetIn, &key);

	pszPhrase = psts->pszPassPhrase;
	if (IsNull (ppRequest))
	{
		if (psts->pszPassPhrase == NULL)
		{
			err = PGPclGetKeyPhrase (
					psts->context,		// in context
					psts->tlsContext,	// in tlscontext
					psts->hwnd,			// in hwnd of parent
					NULL,				// in prompt
					psts->keydbMain,	// in keyset
					key,				// in key
					&pszPhrase,			// out phrase
					&pPasskey,			// out passkey buffer
					&sizePasskey);		// out passkey length
		}
	}

	switch (psts->ksentry.type) {
	case kPGPKeyServerClass_NetToolsCA :
		uExportFormat = kPGPExportFormat_NetToolsCAV1_CertReq;
		uOutputFormat = kPGPOutputFormat_NetToolsCAV1_CertReqInPKCS7;
		break;

	case kPGPKeyServerClass_Verisign :
		uExportFormat = kPGPExportFormat_VerisignV1_CertReq;
		uOutputFormat = kPGPOutputFormat_VerisignV1_CertReqInPKCS7;
		break;

	case kPGPKeyServerClass_Entrust :
		uExportFormat = kPGPExportFormat_EntrustV1_CertReq;
		uOutputFormat = kPGPOutputFormat_EntrustV1_CertReqInPKCS7;
		break;

	case kPGPKeyServerClass_Netscape :
		uExportFormat = kPGPExportFormat_NetscapeV1_CertReq;
		uOutputFormat = kPGPOutputFormat_NetscapeV1_CertReqInPKCS7;
		break;

	case kPGPKeyServerClass_Microsoft :
		uExportFormat = kPGPExportFormat_MicrosoftV1_CertReq;
		uOutputFormat = kPGPOutputFormat_MicrosoftV1_CertReqInPKCS7;
		break;
	}

	err = PGPExport (psts->context,
			PGPOExportKeySet (psts->context,
					psts->keysetIn),
			PGPOAllocatedOutputBuffer (psts->context,
					&pBuffer, MAX_PGPSize, &sizeBuffer),
			PGPOExportFormat (psts->context,
					uExportFormat),
			pszPhrase ?
				PGPOPassphrase (psts->context,
						pszPhrase) :
				PGPOPasskeyBuffer (psts->context,
						pPasskey, sizePasskey),
			PGPOAttributeValue (psts->context,
					psts->pAVlist, psts->numAVs),
			(IsNull (ppRequest)) ?
				PGPOArmorOutput (psts->context, TRUE) :
				PGPONullOption (psts->context),
			PGPOLastOption (psts->context));

	if (IsntPGPError (err) &&
		(IsntNull (ppRequest)))
	{
		PGPOptionListRef optionlist	= kInvalidPGPOptionListRef;

		err = PGPBuildOptionList (psts->context, &optionlist,
				PGPOInputBuffer (psts->context,
						pBuffer, sizeBuffer),
				PGPOOutputFormat (psts->context,
						uOutputFormat),
				PGPOAllocatedOutputBuffer (psts->context,
						&pRequest, MAX_PGPSize,
						&sizeRequest),
				PGPOSignWithKey (psts->context, key,
						pszPhrase ?
						PGPOPassphrase (psts->context,
								pszPhrase) :
						PGPOPasskeyBuffer (psts->context,
								pPasskey, sizePasskey),
						PGPOLastOption (psts->context)),
				PGPOLastOption (psts->context));

		if ((IsntPGPError (err)) &&
			((psts->ksentry.type == kPGPKeyServerClass_Netscape) ||
			 (psts->ksentry.type == kPGPKeyServerClass_Microsoft)))
		{
			err = PGPAppendOptionList (optionlist,
					PGPOEncryptToKeyDBObj (
						psts->context, psts->keyIn),
					PGPOLastOption (psts->context));
		}

		if (IsntPGPError (err))
		{
			err = PGPEncode (psts->context, optionlist,
					PGPOLastOption (psts->context));
		}

		if (PGPOptionListRefIsValid (optionlist))
			PGPFreeOptionList (optionlist);
	}

	if (IsntPGPError (err))
	{
		if (IsntNull (ppRequest))
		{
			*ppRequest = pRequest;
			*psizeRequest = sizeRequest;
		}
		else
		{
			HANDLE			hMem;
			LPSTR			pMem;

			err = kPGPError_OutOfMemory;

			if (pBuffer)
			{
				hMem = GlobalAlloc (GMEM_MOVEABLE|GMEM_DDESHARE, sizeBuffer+1);
				if (hMem)
				{
					pMem = GlobalLock (hMem);
					if (pMem)
					{
						memcpy (pMem, pBuffer, sizeBuffer);
						pMem[sizeBuffer] = '\0';

						GlobalUnlock (hMem);

						if (OpenClipboard (NULL))
						{
							EmptyClipboard ();
							SetClipboardData (CF_TEXT, hMem);
							CloseClipboard ();

							PGPclMessageBox (psts->hwnd, IDS_CAPTIONINFO,
									IDS_PKCS10COPIED,
									MB_OK|MB_ICONINFORMATION);
							err = kPGPError_NoErr;
						}
					}
					else
						GlobalFree (hMem);
				}
			}
		}
	}

	if (IsntNull (pBuffer))
		PGPFreeData (pBuffer);

	if (IsNull (ppRequest))
	{
		if (IsNull (psts->pszPassPhrase))
		{
			if (IsntNull (pszPhrase))
				PGPclFreePhrase (pszPhrase);
		}

		if (IsntNull (pPasskey))
			PGPFreeData (pPasskey);
	}

	return err;
}


//	___________________________________________________
//
//	thread routine to handle all CA server operations

static DWORD
sCAThreadRoutine (LPVOID lpvoid)
{
	PSERVERTHREADSTRUCT			psts			= (PSERVERTHREADSTRUCT)lpvoid;
	HWND						hwndParent		= psts->hwnd;
	PGPError					err				= kPGPError_NoErr;
	PGPKeyDBObjRef				key				= kInvalidPGPKeyDBObjRef;
	PGPByte*					pPasskey		= NULL;
	LPSTR						pszPhrase		= NULL;
	PGPSize						sizePasskey		= 0;
	PGPUInt32					numAVItems		= 0;

	PGPKeyServerAccessType		ksaccess	= kPGPKeyServerAccessType_Normal;

	PGPclServerEvent				event;
	BOOL							bThreadFree;
	PGPKeyServerThreadStorageRef	previousStorage;
	PGPContextRef					ctx;

	memset (&event, 0x00, sizeof(event));
	event.nmhdr.hwndFrom = hwndParent;
	event.nmhdr.idFrom = 0;
	event.nmhdr.code = kPGPclNotifyServerProgress;
	event.pData = psts->keydbOut;
	event.cancel = FALSE;
	event.step = kPGPServerInfinite;
	event.total = kPGPServerInfinite;

	err = PGPKeyServerInit ();
	if (IsntPGPError (err))
	{
		PGPKeyServerCreateThreadStorage (&previousStorage);

		// send message to update status text
		LoadString (g_hInst, IDS_LOOKINGFORSERVER, event.szmessage,
					sizeof(event.szmessage));
		lstrcat (event.szmessage, psts->ksentry.serverDNS);
		SendMessage (hwndParent, WM_NOTIFY, (WPARAM)hwndParent,
							(LPARAM)&event);

		ctx = psts->context;
		err = PGPNewKeyServer (ctx,
				psts->ksentry.type,
				&psts->server,
				PGPONetURL (ctx, psts->ksentry.serverDNS),
				PGPOKeyServerKeySpace (ctx, psts->space),
				PGPOKeyServerAccessType (ctx, ksaccess),
				PGPOKeyServerKeyStoreDN (ctx, 
						(const char *)&psts->ksentry.keystoredn),
				PGPOLastOption (ctx));

		if (IsntPGPError (err))
		{
			PGPtlsSessionRef	tls		= kInvalidPGPtlsSessionRef;

			PGPSetKeyServerEventHandler (psts->server,
					sServerEventHandler, psts);

			err = PGPNewTLSSession (psts->tlsContext, &tls);

			if (IsntPGPError (err) &&
				(PGPtlsSessionRefIsValid (tls)) &&
				(psts->uOperation == REQUESTCERTIFICATE))
			{
				sGetFirstKeyInSet (psts->keysetIn, &key);

				if (IsntPGPError (err) &&
					(psts->pszPassPhrase == NULL))
				{
					err = PGPclGetKeyPhrase (
							psts->context,		// in context
							psts->tlsContext,	// in tlscontext
							psts->hwnd,			// in hwnd of parent
							NULL,				// in prompt
							psts->keydbMain,	// in keyset
							key,				// in key
							&pszPhrase,			// out phrase
							&pPasskey,			// out passkey buffer
							&sizePasskey);		// out passkey length

					if (IsntPGPError (err))
					{
						psts->pszPassPhrase = pszPhrase;

						// cache the phrase so that it can be used
						// by the auto cert retrieval code in PGPtray
						PGPclCachePhrase (ctx, key, pszPhrase, NULL, 0);
					}
				}
			}

			// perform the CA server operation
			if (IsntPGPError (err))
			{
				err = PGPKeyServerOpen (psts->server, tls);
				if (IsntPGPError (err))
				{
					PVOID		pBuffer;

					switch (psts->uOperation) {
					case REQUESTCERTIFICATE :
					{
						PVOID		pRequest;
						PGPSize		sizeRequest;

						err = sCreateServerRequest (
								psts, key, pPasskey, sizePasskey,
								&pRequest, &sizeRequest);

						if (IsntPGPError (err))
						{
							err = PGPSendCertificateRequest (
									psts->server,
									PGPOKeyServerCAKey (psts->context,
											psts->keyIn),
									PGPOKeyServerRequestKey (
											psts->context, key),
									PGPOInputBuffer (psts->context,
										pRequest, sizeRequest),
									PGPOLastOption (psts->context));

							PGPFreeData (pRequest);

							// schedule automatic updates
							if (IsntPGPError (err) && IsntNull (psts->hwnd))
							{
								PGPKeyID	keyid;

								if (IsntPGPError (PGPGetKeyID (key, &keyid)))
								{
									PGPPrefRef	prefref	= kInvalidPGPPrefRef;

									err = PGPclPeekClientLibPrefRefs (
											&prefref, NULL);
									if (IsntPGPError (err))
									{
										PGPSetPrefData (prefref,
												kPGPPrefRequestedCertKeyID,
												sizeof(PGPKeyID), &keyid);

										PGPSetPrefNumber (prefref,
												kPGPPrefLastCertRequest,
												PGPGetTime ());

										PGPclFlushClientLibPrefs (
												prefref, NULL);
									}
								}
							}
						}

						break;
					}

					case RETRIEVECERTIFICATE :
					{
						PGPFilterRef	filter		= kInvalidPGPFilterRef;
						PGPByte			md5HashBuffer[128];
						PGPUInt32		uInputFormat;
						PGPKeyDBObjRef	key;
						PGPSize			size;

						pBuffer = NULL;

						switch (psts->ksentry.type) {
						case kPGPKeyServerClass_NetToolsCA :
							uInputFormat =
								kPGPInputFormat_NetToolsCAV1_DataInPKCS7;
							break;
						case kPGPKeyServerClass_Verisign :
							uInputFormat =
									kPGPInputFormat_VerisignV1_DataInPKCS7;
							break;
						case kPGPKeyServerClass_Entrust :
							uInputFormat =
									kPGPInputFormat_EntrustV1_DataInPKCS7;
							break;
						case kPGPKeyServerClass_Netscape :
							uInputFormat =
									kPGPInputFormat_NetscapeV1_DataInPKCS7;
							break;
						case kPGPKeyServerClass_Microsoft :
							uInputFormat =
									kPGPInputFormat_MicrosoftV1_DataInPKCS7;
							break;
						}

						sGetFirstKeyInSet (psts->keysetIn, &key);

						switch (psts->ksentry.type) {

						case kPGPKeyServerClass_NetToolsCA :
							err = PGPGetKeyDBObjDataProperty (key,
								kPGPKeyProperty_X509MD5Hash, md5HashBuffer,
								sizeof(md5HashBuffer), &size);

							if (IsntPGPError (err))
							{
								err = PGPNewKeyDBObjDataFilter (
										psts->context,
										kPGPKeyProperty_X509MD5Hash,
										md5HashBuffer, size,
										kPGPMatchCriterion_Equal,
										&filter);

								if (IsntPGPError (err))
								{
									err = PGPRetrieveCertificate (
											psts->server,
											PGPOKeyServerSearchFilter (
													psts->context,
													filter),
											PGPOAllocatedOutputBuffer (
													psts->context,
													&pBuffer,
													MAX_PGPSize,
													&size),
											PGPOLastOption (psts->context));
								}
							}
							break;

						case kPGPKeyServerClass_Verisign :
						case kPGPKeyServerClass_Entrust :
						case kPGPKeyServerClass_Netscape :
						case kPGPKeyServerClass_Microsoft :

							if (IsntNull (psts->pPasskey))
							{
								pszPhrase = NULL;
								pPasskey = psts->pPasskey;
								sizePasskey = psts->sizePasskey;
							}
							else if (PGPPassphraseIsValid (key,
										PGPOLastOption(ctx)))
							{
								pszPhrase = PGPNewData (
										PGPPeekContextMemoryMgr (psts->context),
										1, kPGPMemoryMgrFlags_Clear);
								pPasskey = NULL;
								sizePasskey = 0;
							}
							else
							{
								err = PGPclGetKeyPhrase (
									psts->context,		// in context
									psts->tlsContext,	// in tlscontext
									psts->hwnd,			// in hwnd of parent
									NULL,				// in prompt
									psts->keydbMain,	// in keyset
									key,				// in key
									&pszPhrase,			// out phrase
									&pPasskey,			// out passkey buffer
									&sizePasskey);		// out passkey length
							}

							if (IsntPGPError (err))
							{
								psts->pszPassPhrase = pszPhrase;

								err = PGPRetrieveCertificate (
										psts->server,
    									PGPOSignWithKey (
											psts->context,
											key,
											pszPhrase ?
											PGPOPassphrase (psts->context,
												pszPhrase) :
											PGPOPasskeyBuffer (psts->context,
												pPasskey, sizePasskey),
											PGPOLastOption (
												psts->context)),
										PGPOKeyServerCAKey (psts->context,
												psts->keyIn),
										PGPOKeyServerSearchKey (
											psts->context,
											key),
										PGPOAllocatedOutputBuffer (
											psts->context,
											&pBuffer,
											MAX_PGPSize,
											&size),
										PGPOLastOption (psts->context));
							}
							break;
						}

						if (IsntPGPError (err))
						{
							PGPOptionListRef optionlist	=
									kInvalidPGPOptionListRef;
							PGPKeyDBRef decodeDB;

							/* Make new keydb holding just decode key */
							/* This tells PGPDecode what to decrypt with */
							PGPNewKeyDB (psts->context, &decodeDB);
							PGPCopyKeyDBObj (key, decodeDB, NULL);

							err = PGPBuildOptionList (psts->context, &optionlist,
									PGPOInputBuffer (psts->context,
											pBuffer, size),
									PGPOKeyDBRef (psts->context, decodeDB),
									PGPODiscardOutput (psts->context,
											TRUE),
									PGPOImportKeysTo (psts->context,
											psts->keydbOut),
									PGPOInputFormat (psts->context,
											uInputFormat),
									PGPOLastOption (psts->context));

							if ((psts->ksentry.type == kPGPKeyServerClass_Netscape) ||
								(psts->ksentry.type == kPGPKeyServerClass_Microsoft))
							{
								err = PGPAppendOptionList (optionlist,
									pszPhrase ?
										PGPOPassphrase (psts->context,
											pszPhrase) :
										PGPOPasskeyBuffer (psts->context,
											pPasskey, sizePasskey),
										PGPOLastOption (psts->context));
							}

							err = PGPDecode (psts->context, optionlist,
										PGPOLastOption (psts->context));

							if (PGPOptionListRefIsValid (optionlist))
								PGPFreeOptionList (optionlist);
							if (PGPKeyDBRefIsValid (decodeDB))
								PGPFreeKeyDB (decodeDB);
						}
						if (pBuffer)
							PGPFreeData (pBuffer);
						if (PGPFilterRefIsValid (filter))
							PGPFreeFilter (filter);
						break;
					}

					case UPDATEREVOCATIONS :
					{
						PGPUInt32		uInputFormat;
						PGPKeyDBObjRef	keySigning;
						PGPSize			size;

						switch (psts->ksentry.type) {
						case kPGPKeyServerClass_NetToolsCA :
							uInputFormat =
									kPGPInputFormat_NetToolsCAV1_DataInPKCS7;
							break;

						case kPGPKeyServerClass_Verisign :
							uInputFormat =
									kPGPInputFormat_VerisignV1_DataInPKCS7;
							break;

						case kPGPKeyServerClass_Entrust :
							uInputFormat =
									kPGPInputFormat_EntrustV1_DataInPKCS7;
							break;

						case kPGPKeyServerClass_Netscape :
							uInputFormat =
									kPGPInputFormat_NetscapeV1_DataInPKCS7;
							break;

						case kPGPKeyServerClass_Microsoft :
							uInputFormat =
									kPGPInputFormat_MicrosoftV1_DataInPKCS7;
							break;
						}

						switch (psts->ksentry.type) {
						case kPGPKeyServerClass_NetToolsCA :
							err = PGPRetrieveCertificateRevocationList (
								psts->server,
								PGPOKeyServerCAKey (psts->context,
										psts->keyIn),
								PGPOAllocatedOutputBuffer (psts->context,
										&pBuffer, MAX_PGPSize, &size),
								PGPOLastOption (psts->context));
							break;

						case kPGPKeyServerClass_Netscape :
						case kPGPKeyServerClass_Microsoft :
							err = PGPRetrieveCertificateRevocationList (
								psts->server,
								PGPOAllocatedOutputBuffer (psts->context,
										&pBuffer, MAX_PGPSize, &size),
								PGPOLastOption (psts->context));
							break;

						case kPGPKeyServerClass_Verisign :
						case kPGPKeyServerClass_Entrust :
							keySigning = kInvalidPGPKeyDBObjRef;
							err = PGPclGetSigningKeyPhrase (
									psts->context,		// in context
									psts->tlsContext,	// in tlscontext
									psts->hwnd,			// in hwnd of parent
									NULL,				// in prompt
									psts->keydbMain,	// in keyset
									TRUE,				// in no split keys
									&keySigning,		// out key to use
									&pszPhrase,			// out phrase
									&pPasskey,			// out passkey buffer
									&sizePasskey);		// out passkey length

							if (IsntPGPError (err))
							{
								psts->pszPassPhrase = pszPhrase;

								err = PGPRetrieveCertificateRevocationList (
									psts->server,
									PGPOKeyServerCAKey (psts->context,
											psts->keyIn),
									PGPOKeyDBRef (psts->context,
											psts->keydbMain),
    								PGPOSignWithKey (
										psts->context,
										keySigning,
										pszPhrase ?
										PGPOPassphrase (psts->context,
											pszPhrase) :
										PGPOPasskeyBuffer (psts->context,
											pPasskey, sizePasskey),
										PGPOLastOption (
											psts->context)),
									PGPOAllocatedOutputBuffer (psts->context,
											&pBuffer, MAX_PGPSize, &size),
									PGPOLastOption (psts->context));
							}

							break;
						}

						if (IsntPGPError (err) &&
							(pBuffer != NULL))
						{
							err = PGPDecode (psts->context,
									PGPOInputBuffer (psts->context,
											pBuffer, size),
									PGPODiscardOutput (psts->context, TRUE),
									PGPOImportKeysTo (psts->context,
											psts->keydbMain),
									PGPOInputFormat (psts->context,
											uInputFormat),
									PGPOLastOption (psts->context));

							PGPFreeData (pBuffer);
						}
						break;
					}
					}
				}

				// burn and free phrase
				if (IsntNull (pszPhrase))
				{
					PGPclFreePhrase (pszPhrase);
					psts->pszPassPhrase = NULL;
				}
				if (pPasskey)
				{
					if (pPasskey != psts->pPasskey)
						PGPFreeData (pPasskey);
				}

				PGPKeyServerClose (psts->server);
			}

			PGPFreeKeyServer (psts->server);
			psts->server = kInvalidPGPKeyServerRef;

			if (PGPtlsSessionRefIsValid (tls))
				PGPFreeTLSSession (tls);
		}

		PGPKeyServerDisposeThreadStorage(previousStorage);
		PGPKeyServerCleanup ();
	}

//done :
	// let window know that we're done
	psts->err = err;
	event.szmessage[0] = '\0';
	if (err == kPGPError_UserAbort)
	{
		event.nmhdr.code = kPGPclNotifyServerAbort;
		LoadString (g_hInst,
				IDS_SERVERABORT, event.szmessage, sizeof(event.szmessage));
		lstrcat (event.szmessage, psts->ksentry.serverDNS);
	}
	else if (IsPGPError (err))
	{
		event.nmhdr.code = kPGPclNotifyServerError;
		if (psts->uFlags & USEFULLERRORTEXT)
		{
			PGPclErrorToString (err,
					event.szmessage, sizeof(event.szmessage), NULL);
		}
		else
		{
			LoadString (g_hInst, IDS_SERVERERROR, event.szmessage,
					sizeof(event.szmessage));
			lstrcat (event.szmessage, psts->ksentry.serverDNS);
		}
	}
	else
	{
		event.nmhdr.code = kPGPclNotifyServerDone;
		LoadString (g_hInst,
				IDS_SERVEROK, event.szmessage, sizeof(event.szmessage));
		lstrcat (event.szmessage, psts->ksentry.serverDNS);
	}

	bThreadFree = psts->bThreadFree;
	SendMessage (hwndParent,
			WM_NOTIFY, (WPARAM)hwndParent, (LPARAM)&event);

	// if thread should free data structure, do it
	if (bThreadFree)
	{
		if (PGPKeySetRefIsValid (psts->keysetIn))
			PGPFreeKeySet (psts->keysetIn);
		PGPclFreeCACertRequestAVList (psts->pAVlist, psts->numAVs);
		PGPFreeData (psts);
	}

	return 0;
}


//	___________________________________________________
//
//	thread routine to handle prefs and key reconstruction server operations

static DWORD
sLDAPThreadRoutine (LPVOID lpvoid)
{
	PSERVERTHREADSTRUCT			psts			= (PSERVERTHREADSTRUCT)lpvoid;
	HWND						hwndParent		= psts->hwnd;
	PGPError					err				= kPGPError_NoErr;
	PGPKeyDBObjRef				key				= kInvalidPGPKeyDBObjRef;
	PGPReconContextRef			reconcontext	= kInvalidPGPReconContextRef;
	PGPPrefRef					prefref			= kInvalidPGPPrefRef;
	LPSTR						pszPhrase		= NULL;
	BOOL						bNeedsLDAPAuth	= FALSE;
	PGPByte*					pReconData		= NULL;
	PGPSize						sizeReconData	= 0;


	PGPclServerEvent			event;
	BOOL						bThreadFree;
	PGPUInt32					uServerType;

	memset (&event, 0x00, sizeof(event));
	event.nmhdr.hwndFrom = hwndParent;
	event.nmhdr.idFrom = 0;
	event.nmhdr.code = kPGPclNotifyServerProgress;
	event.pData = NULL;
	event.cancel = FALSE;
	event.step = kPGPServerInfinite;
	event.total = kPGPServerInfinite;

	// get server name (for progress text)
	switch (psts->uOperation) {
	case SENDRECONSTRUCTIONBLOB :
	case RETRIEVERECONSTRUCTIONBLOB :
		err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;

		PGPGetPrefNumber (prefref,
				kPGPPrefLDAPReconServerType, &uServerType);

		if (uServerType != kPGPKeyServerClass_PGP)
			bNeedsLDAPAuth = TRUE;

		err = PGPGetPrefStringBuffer (prefref,
				kPGPPrefLDAPReconServer, sizeof(psts->ksentry.serverDNS),
				psts->ksentry.serverDNS); CKERR;

		psts->ksentry.authAlg = kPGPPublicKeyAlgorithm_Invalid;
		break;

	case SENDPREFS :
		bNeedsLDAPAuth = TRUE;
		// fall through
	case GETPREFS :
		prefref = psts->prefref;

		if (prefref == kInvalidPGPPrefRef)
		{
			err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
		}

		err = PGPGetPrefStringBuffer (prefref,
				kPGPPrefLDAPPrefsServer, sizeof(psts->ksentry.serverDNS),
				psts->ksentry.serverDNS); CKERR;

		err = PGPGetPrefStringBuffer (prefref,
				kPGPPrefLDAPPrefsServerKeyID,
				sizeof(psts->ksentry.authKeyIDString),
				psts->ksentry.authKeyIDString); CKERR;

		err = PGPGetPrefNumber (prefref,
				kPGPPrefLDAPPrefsServerAlg, &psts->ksentry.authAlg); CKERR;
		break;
	}

	// send message to update status text
	LoadString (g_hInst, IDS_CONNECTING, event.szmessage,
			sizeof(event.szmessage));
	lstrcat (event.szmessage, psts->ksentry.serverDNS);
	SendMessage (hwndParent,
			WM_NOTIFY, (WPARAM)hwndParent, (LPARAM)&event);

	if ((psts->uOperation == SENDRECONSTRUCTIONBLOB) &&
		(psts->pszPassPhrase == NULL))
	{
		PGPByte*	pPasskey		= NULL;
		PGPSize		sizePasskey;

		err = PGPclGetKeyPhrase (
				psts->context,		// in context
				psts->tlsContext,	// in tlscontext
				psts->hwnd,			// in hwnd of parent
				NULL,				// in prompt
				kInvalidPGPKeyDBRef,// in keydb
				psts->keyIn,		// in key
				&pszPhrase,			// out phrase
				&pPasskey,			// out passkey buffer
				&sizePasskey);		// out passkey length

		if (IsntNull (pPasskey))
			PGPclFreePasskey (pPasskey, sizePasskey);

		if (IsntPGPError (err))
			psts->pszPassPhrase = pszPhrase;
	}
	CKERR;

	// get user id and password if server is LDAP
	if (bNeedsLDAPAuth)
	{
		if (!DialogBoxParam (g_hInst, MAKEINTRESOURCE(IDD_LDAPUSERNAME),
				psts->hwnd, sLDAPUserNameDlgProc, (LPARAM)psts))
		{
			err = kPGPError_UserAbort;
			goto done;
		}
	}

	// do the operation
	switch (psts->uOperation) {
	case SENDRECONSTRUCTIONBLOB :
		err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;

		err = PGPNewReconstruct (psts->keyIn, prefref,
				psts->pszUserID, psts->pszUserPass,
				(PGPReconstructEventHandler)sServerEventHandler,
				psts, &reconcontext); CKERR;

		err = PGPMakeReconstruction (reconcontext,
				psts->reconPrompts, psts->reconPasses,
				psts->pszPassPhrase); CKERR;

		err = PGPSendReconstruction (reconcontext);
		break;

	case RETRIEVERECONSTRUCTIONBLOB :
		err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;

		err = PGPNewReconstruct (psts->keyIn, prefref,
				psts->pszUserID, psts->pszUserPass,
				(PGPReconstructEventHandler)sServerEventHandler,
				psts, &reconcontext); CKERR;

		err = PGPGetReconstructionPrompts (reconcontext, psts->reconPrompts);
		if (IsntPGPError (err))
		{
			if (!DialogBoxParam (g_hInst, MAKEINTRESOURCE(IDD_RECONSTRUCTION),
					psts->hwnd, sReconstructionDlgProc, (LPARAM)psts))
			{
				err = kPGPError_UserAbort;
				goto done;
			}

			err = PGPGetReconstructionData (reconcontext,
					psts->reconPasses, &pReconData, &sizeReconData); CKERR;

			err = PGPReconstruct (reconcontext, psts->reconPasses, 
					pReconData, sizeReconData, &psts->keydbOut); CKERR;

			err = sGetFirstKeyInSet (
					PGPPeekKeyDBRootKeySet (psts->keydbOut), &key); CKERR;

			err = PGPclChangePhrase (psts->context, psts->tlsContext,
					psts->hwnd, key, FALSE);
		}
		break;

	case SENDPREFS :
		err = PGPSendPrefsToServer (psts->context,
				psts->prefref, NULL, // Byebyenet! psts->prefrefNet,
				psts->ksentry.serverDNS, psts->pszUserID, psts->pszUserPass,
				sServerEventHandler, psts);

		if (IsntPGPError (err))
		{
			err = PGPSetPrefString (psts->prefref,
					kPGPPrefLDAPPrefsServerKeyID,
					psts->ksentry.authKeyIDString);
		}

		if (IsntPGPError (err))
		{
			err = PGPSetPrefNumber (psts->prefref,
					kPGPPrefLDAPPrefsServerAlg, psts->ksentry.authAlg);
		}

		break;

	case GETPREFS :
	{
		PGPPrefRef		prefref			= kInvalidPGPPrefRef;
		PGPPrefRef		prefrefNet		= kInvalidPGPPrefRef;
		PGPPrefRef		prefrefNew		= kInvalidPGPPrefRef;
		PGPPrefRef		prefrefNetNew	= kInvalidPGPPrefRef;
		PGPBoolean		bCopyHosts		= FALSE;
		PGPBoolean		bMergeHosts		= FALSE;
		BOOL			bClientLibPrefs	= FALSE;

		err = PGPGetPrefsFromServer (
				psts->context, psts->ksentry.serverDNS,
				&prefrefNew, NULL, // Byebyenet! &prefrefNetNew
				sServerEventHandler, psts); 

		if (IsntPGPError (err))
		{
			PGPGetPrefBoolean (prefrefNew, kPGPPrefCopyNetHosts, &bCopyHosts);

			if (PGPPrefRefIsValid (psts->prefref) ||
				PGPPrefRefIsValid (psts->prefrefNet))
			{
				prefref = psts->prefref;
				prefrefNet = psts->prefrefNet;
			}
			else
			{
				bClientLibPrefs = TRUE;

				if (PGPclIsComponentInstalled (kPGPclPGPnet))
					err = PGPclPeekClientLibPrefRefs (&prefref, &prefrefNet);
				else
					err = PGPclPeekClientLibPrefRefs (&prefref, NULL);
			}
		}

		// check merge hosts pref only if using clientlib prefs, 
		// or if called by auto-update thread.  Otherwise we are
		// being called from PGPadmin and no merging is to be done.
		if (IsntPGPError (err) && 
			(bClientLibPrefs || IsNull (psts->hwnd)))
		{
			PGPGetPrefBoolean (prefrefNew,
					kPGPPrefMergeNetHosts, &bMergeHosts);
		}

		if (IsntPGPError (err) && PGPPrefRefIsValid (prefref))
		{
			err = PGPSetPrefString (prefref,
					kPGPPrefLDAPPrefsServerKeyID,
					psts->ksentry.authKeyIDString);
		}

		if (IsntPGPError (err) && PGPPrefRefIsValid (prefref))
		{
			err = PGPSetPrefNumber (prefref,
					kPGPPrefLDAPPrefsServerAlg, psts->ksentry.authAlg);
		}

		if (IsntPGPError (err) && PGPPrefRefIsValid (prefref))
		{
			if (PGPPrefRefIsValid (prefrefNew))
				err = PGPclMergePrefs (prefrefNew, prefref);
		}

		if (IsntPGPError (err) && PGPPrefRefIsValid (prefrefNet))
		{
			PGPNetPrefHostEntry*	pHosts			= NULL;
			PGPNetPrefHostEntry*	pHostsNew		= NULL;
			PGPNetPrefHostEntry*	pHostsMerged	= NULL;

			PGPUInt32	uHostCount			= 0;
			PGPUInt32	uHostCountNew		= 0;
			PGPUInt32	uHostCountMerged	= 0;

			if (PGPPrefRefIsValid (prefrefNetNew))
			{
				PGPGetNetHostPrefs (prefrefNet,
						&pHosts, &uHostCount);
				PGPGetNetHostPrefs (prefrefNetNew,
						&pHostsNew, &uHostCountNew);

				err = PGPclMergeNetPrefs (prefrefNetNew, prefrefNet);

				if (IsntPGPError (err))
				{
					if (bMergeHosts)
					{
						err = pgpMergeNetHostLists (
								PGPPeekContextMemoryMgr (psts->context),
								pHosts, uHostCount,
								pHostsNew, uHostCountNew,
								&pHostsMerged, &uHostCountMerged);

						if (IsntPGPError (err))
						{
							PGPSetNetHostPrefs (prefrefNet,
									pHostsMerged, uHostCountMerged);
							PGPFreeData (pHostsMerged);
						}
					}
					else if (bCopyHosts)
					{
						PGPSetNetHostPrefs (prefrefNet,
								pHostsNew, uHostCountNew);
					}
				}

				if (IsntNull (pHosts))
					PGPFreeData (pHosts);
				if (IsntNull (pHostsNew))
					PGPFreeData (pHostsNew);
			}
		}

		if (bClientLibPrefs)
		{
			if (IsntPGPError (err))
				err = PGPclFlushClientLibPrefs (prefref, prefrefNet);
		}

		if (PGPPrefRefIsValid (prefrefNew))
			PGPFreePrefs (prefrefNew);
		if (PGPPrefRefIsValid (prefrefNetNew))
			PGPFreePrefs (prefrefNetNew);

		break;
	}
	}

done :
	if (IsntNull (pReconData))
		PGPFreeData (pReconData);
	if (PGPReconContextRefIsValid (reconcontext))
		PGPFreeReconstruct (reconcontext);

	// let window know that we're done
	psts->err = err;
	event.pData	= psts->keydbOut;
	event.szmessage[0] = '\0';
	if (err == kPGPError_UserAbort)
	{
		event.nmhdr.code = kPGPclNotifyServerAbort;
		LoadString (g_hInst, IDS_SERVERABORT, event.szmessage,
				sizeof(event.szmessage));
		lstrcat (event.szmessage, psts->ksentry.serverDNS);
	}
	else if (IsPGPError (err))
	{
		event.nmhdr.code = kPGPclNotifyServerError;
		if (psts->uFlags & USEFULLERRORTEXT)
		{
			PGPclErrorToString (err,
					event.szmessage, sizeof(event.szmessage), NULL);
		}
		else
		{
			LoadString (g_hInst, IDS_SERVERERROR, event.szmessage,
					sizeof(event.szmessage));
			lstrcat (event.szmessage, psts->ksentry.serverDNS);
		}
	}
	else
	{
		event.nmhdr.code	= kPGPclNotifyServerDone;

		LoadString (g_hInst, IDS_SERVEROK, event.szmessage,
				sizeof(event.szmessage));
		lstrcat (event.szmessage, psts->ksentry.serverDNS);
	}

	bThreadFree = psts->bThreadFree;
	SendMessage (hwndParent,
			WM_NOTIFY, (WPARAM)hwndParent, (LPARAM)&event);

	// if thread should free data structure, do it
	if (bThreadFree)
	{
		if (psts->pszUserID)
			PGPFreeData (psts->pszUserID);
		if (psts->pszUserPass)
			PGPFreeData (psts->pszUserPass);
		if (pszPhrase)
			PGPFreeData (pszPhrase);

		PGPFreeData (psts);
	}

	return 0;
}


//	____________________________________
//
//  Draw the "LED" progress indicator

static VOID
sDrawStatus (
		HWND				hwnd,
		PSERVERTHREADSTRUCT psts)
{
	HBRUSH			hBrushLit, hBrushUnlit, hBrushOld;
	HPEN			hPen, hPenOld;
	INT				i;
	INT				itop, ibot, ileft, iright, iwidth;
	PAINTSTRUCT		ps;
	HDC				hdc;
	RECT			rc;

	if (psts->iStatusValue < -1)
		return;

	hdc = BeginPaint (hwnd, &ps);

	// draw 3D shadow
	GetClientRect (hwnd, &rc);
	itop = rc.top+1;
	ibot = rc.bottom-2;

	iwidth = (rc.right-rc.left) / NUMLEDS;
	iwidth -= LEDSPACING;

	ileft = rc.left + 4;
	for (i=0; i<NUMLEDS; i++)
	{
		iright = ileft + iwidth;

		MoveToEx (hdc, ileft, ibot, NULL);
		LineTo (hdc, iright, ibot);
		LineTo (hdc, iright, itop);

		ileft += iwidth + LEDSPACING;
	}

	hPen = CreatePen (PS_SOLID, 0, RGB (128, 128, 128));
	hPenOld = SelectObject (hdc, hPen);
	hBrushLit = CreateSolidBrush (RGB (0, 255, 0));
	hBrushUnlit = CreateSolidBrush (RGB (0, 128, 0));

	ileft = rc.left + 4;

	// draw "Knight Rider" LEDs
	if (psts->iStatusDirection)
	{
		hBrushOld = SelectObject (hdc, hBrushUnlit);
		for (i=0; i<NUMLEDS; i++)
		{
			iright = ileft + iwidth;

			if (i == psts->iStatusValue)
			{
				SelectObject (hdc, hBrushLit);
				Rectangle (hdc, ileft, itop, iright, ibot);
				SelectObject (hdc, hBrushUnlit);
			}
			else
				Rectangle (hdc, ileft, itop, iright, ibot);

			ileft += iwidth + LEDSPACING;
		}
	}

	// draw "progress bar" LEDs
	else {
		if (psts->iStatusValue >= 0)
			hBrushOld = SelectObject (hdc, hBrushLit);
		else
			hBrushOld = SelectObject (hdc, hBrushUnlit);

		for (i=0; i<NUMLEDS; i++)
		{
			iright = ileft + iwidth;

			if (i > psts->iStatusValue)
				SelectObject (hdc, hBrushUnlit);

			Rectangle (hdc, ileft, itop, iright, ibot);

			ileft += iwidth + LEDSPACING;
		}
	}

	SelectObject (hdc, hBrushOld);
	SelectObject (hdc, hPenOld);
	DeleteObject (hPen);
	DeleteObject (hBrushLit);
	DeleteObject (hBrushUnlit);

	EndPaint (hwnd, &ps);
}

//	______________________________________________________
//
//  Server progress dialog procedure (used for disable and delete)

static UINT CALLBACK
sServerProgressDlgProc (
		 HWND	hDlg,
		 UINT	uMsg,
		 WPARAM wParam,
		 LPARAM lParam)
{

	switch (uMsg) {

	case WM_INITDIALOG:
		{
			PSERVERTHREADSTRUCT psts = (PSERVERTHREADSTRUCT)lParam;

			SetWindowLong (hDlg, GWL_USERDATA, lParam);

			psts->hwnd = hDlg;
			_beginthread (psts->lpThread, 0, (void*)psts);
			return TRUE;
		}

	case WM_PAINT :
		{
			PSERVERTHREADSTRUCT psts =
				(PSERVERTHREADSTRUCT)GetWindowLong (hDlg, GWL_USERDATA);
			sDrawStatus (GetDlgItem (hDlg, IDC_PROGRESS), psts);
			break;
		}

	case WM_TIMER :
		{
			PSERVERTHREADSTRUCT psts =
				(PSERVERTHREADSTRUCT)GetWindowLong (hDlg, GWL_USERDATA);
			psts->iStatusValue += psts->iStatusDirection;
			if (psts->iStatusValue <= 0) {
				psts->iStatusValue = 0;
				psts->iStatusDirection = 1;
			}
			else if (psts->iStatusValue >= NUMLEDS-1) {
				psts->iStatusValue = NUMLEDS-1;
				psts->iStatusDirection = -1;
			}
			InvalidateRect (hDlg, NULL, FALSE);
			break;
		}

	case WM_NOTIFY :
		{
			LPNMHDR pnmh = (LPNMHDR) lParam;
			PSERVERTHREADSTRUCT psts =
				(PSERVERTHREADSTRUCT)GetWindowLong (hDlg, GWL_USERDATA);

			if (pnmh->hwndFrom != hDlg) break;

			switch (pnmh->code) {
				case kPGPclNotifyServerProgress :
				{
					PPGPclServerEvent	pEvent = (PPGPclServerEvent)lParam;

					if (!(psts->bOpInProgress))
					{
						psts->bOpInProgress = TRUE;
						psts->iStatusValue = 0;
						if (pEvent->step == kPGPServerInfinite)
						{
							psts->iStatusDirection = 1;
							SetTimer (hDlg, LEDTIMER, LEDTIMERPERIOD, NULL);
						}
						else
						{
							psts->iStatusDirection = 0;
							psts->iStatusValue = 0;
						}
					}
					else {
						if (pEvent->step != kPGPServerInfinite)
						{
							psts->iStatusDirection = 0;
							psts->iStatusValue = (pEvent->step * 9) /
													pEvent->total;
							InvalidateRect (hDlg, NULL, FALSE);
						}
					}
					if (lstrlen (pEvent->szmessage) > MAXPROGRESSTEXT)
					{
						pEvent->szmessage[MAXPROGRESSTEXT] = '\0';
						lstrcat (pEvent->szmessage, "...");
					}
					SetDlgItemText (hDlg,
							IDC_PROGRESSTEXT, pEvent->szmessage);
					return FALSE;
				}

				case kPGPclNotifyServerDone :
				case kPGPclNotifyServerAbort :
				case kPGPclNotifyServerError :
				{
					PPGPclServerEvent	pEvent = (PPGPclServerEvent)lParam;
					PGPError			err;

					KillTimer (hDlg, LEDTIMER);
					psts->keydbOut = (PGPKeyDBRef)(pEvent->pData);
					err = psts->err;

					EndDialog (hDlg, err);
					return FALSE;
				}
			}
			break;
		}


	case WM_COMMAND :
		switch(LOWORD (wParam)) {
		case IDCANCEL :
			{
				PSERVERTHREADSTRUCT psts =
					(PSERVERTHREADSTRUCT)GetWindowLong (hDlg, GWL_USERDATA);
				KillTimer (hDlg, LEDTIMER);
				psts->bThreadFree = TRUE;
				psts->bCancel = TRUE;
				if (psts->server)
				{
					PGPError err;

					err = PGPCancelKeyServerCall (psts->server);
					PGPclErrorBox (hDlg, err);
				}
				EndDialog (hDlg, kPGPError_UserAbort);
				break;
			}

		}
		return TRUE;
	}

	return FALSE;
}


//	___________________________________________________
//
//	update all groups from keyserver

PGPError PGPclExport
PGPclGetGroupsFromRootServer (
		PGPContextRef			context,
		PGPtlsContextRef		tlsContext,
		HWND					hwndParent,
		PGPKeyDBRef				keydbMain,
		PGPGroupSetRef*			pgroupsetDownloaded)
{
	PGPError			err		= kPGPError_NoErr;

	PSERVERTHREADSTRUCT psts;

	psts = (PSERVERTHREADSTRUCT)malloc (sizeof(SERVERTHREADSTRUCT));
	pgpClearMemory (psts, sizeof(SERVERTHREADSTRUCT));

	if (psts)
	{
		psts->uOperation		= GETGROUPS;
		psts->uFlags			= 0;
		psts->context			= context;
		psts->tlsContext		= tlsContext;
		psts->keydbMain			= keydbMain;
		psts->hwnd				= hwndParent;
		psts->lpThread			= sKeyserverThreadRoutine;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= FALSE;
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->space				= kPGPKeyServerKeySpace_Default;
		psts->groupset			= kInvalidPGPGroupSetRef;

		err = DialogBoxParam (g_hInst, MAKEINTRESOURCE(IDD_SERVERPROGRESS),
				hwndParent, sServerProgressDlgProc, (LPARAM)psts);

		if (IsntPGPError (err))
		{
			if (pgroupsetDownloaded)
				*pgroupsetDownloaded = psts->groupset;
		}

		if (!psts->bThreadFree)
			free (psts);
	}

	return err;
}


//	___________________________________________________
//
//	send all groups to keyserver

PGPError PGPclExport
PGPclSendGroupsToRootServer (
		PGPContextRef			context,
		PGPtlsContextRef		tlsContext,
		HWND					hwndParent,
		PGPKeyDBRef				keydbMain,
		PGPGroupSetRef			groupsetToSend)
{
	PGPError			err		= kPGPError_NoErr;

	PSERVERTHREADSTRUCT psts;

	psts = (PSERVERTHREADSTRUCT)malloc (sizeof(SERVERTHREADSTRUCT));
	pgpClearMemory (psts, sizeof(SERVERTHREADSTRUCT));

	if (psts)
	{
		psts->uOperation		= SENDGROUPS;
		psts->uFlags			= 0;
		psts->context			= context;
		psts->tlsContext		= tlsContext;
		psts->keydbMain			= keydbMain;
		psts->hwnd				= hwndParent;
		psts->lpThread			= sKeyserverThreadRoutine;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= FALSE;
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->space				= kPGPKeyServerKeySpace_Default;
		psts->groupset			= groupsetToSend;

		err = DialogBoxParam (g_hInst, MAKEINTRESOURCE(IDD_SERVERPROGRESS),
				hwndParent, sServerProgressDlgProc, (LPARAM)psts);

		if (!psts->bThreadFree)
			free (psts);
	}

	return err;
}


//	___________________________________________________
//
//	send keyset to keyserver

PGPError PGPclExport
PGPclSendKeysToRootServerNotify (
		PGPContextRef		context,
		PGPtlsContextRef	tlsContext,
		HWND				hwndToNotify,
		PGPKeyDBRef			keydbMain,
		PGPKeySetRef		keysetToSend)
{
	PSERVERTHREADSTRUCT psts;

	psts = (PSERVERTHREADSTRUCT)malloc (sizeof(SERVERTHREADSTRUCT));
	pgpClearMemory (psts, sizeof(SERVERTHREADSTRUCT));

	if (psts)
	{
		PGPNewEmptyKeySet (
				PGPPeekKeySetKeyDB (keysetToSend), &(psts->keysetIn));
		PGPAddKeys (keysetToSend, psts->keysetIn);

		psts->uOperation		= SENDKEY;
		psts->uFlags			= USEFULLERRORTEXT;
		psts->context			= context;
		psts->tlsContext		= tlsContext;
		psts->keydbMain			= keydbMain;
		psts->hwnd				= hwndToNotify;
		psts->lpThread			= NULL;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= TRUE;
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->space				= kPGPKeyServerKeySpace_Default,
		psts->groupset			= kInvalidPGPGroupSetRef;

		_beginthread (sKeyserverThreadRoutine, 0, (void*)psts);
	}

	return kPGPError_NoErr;
}


//	___________________________________________________
//
//	disable keyset on keyserver

PGPError PGPclExport
PGPclDisableKeysOnServer (
		PGPContextRef			context,
		PGPtlsContextRef		tlsContext,
		HWND					hwndParent,
		PGPKeyServerEntry*		pkeyserver,
		PGPKeyServerKeySpace	space,
		PGPKeyDBRef				keydbMain,
		PGPKeySetRef			keysetToSend)
{
	PSERVERTHREADSTRUCT	psts;
	PGPError			err;


	psts = (PSERVERTHREADSTRUCT)malloc (sizeof(SERVERTHREADSTRUCT));
	pgpClearMemory (psts, sizeof(SERVERTHREADSTRUCT));

	if (psts)
	{
		PGPNewEmptyKeySet (
				PGPPeekKeySetKeyDB (keysetToSend), &(psts->keysetIn));
		PGPAddKeys (keysetToSend, psts->keysetIn);

		psts->uOperation		= DISABLEKEY;
		psts->uFlags			= 0;
		psts->context			= context;
		psts->tlsContext		= tlsContext;
		psts->keydbMain			= keydbMain;
		psts->lpThread			= sKeyserverThreadRoutine;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= FALSE;
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->space				= space;
		psts->groupset			= kInvalidPGPGroupSetRef;

		CopyMemory (&psts->ksentry, pkeyserver, sizeof(PGPKeyServerEntry));

		err = DialogBoxParam (g_hInst, MAKEINTRESOURCE(IDD_SERVERPROGRESS),
				hwndParent, sServerProgressDlgProc, (LPARAM)psts);

		if (!psts->bThreadFree)
		{
			if (psts->keysetIn) PGPFreeKeySet (psts->keysetIn);
			free (psts);
		}
	}

	return err;
}


//	___________________________________________________
//
//	delete keyset from keyserver

PGPError PGPclExport
PGPclDeleteKeysFromServer (
		PGPContextRef			context,
		PGPtlsContextRef		tlsContext,
		HWND					hwndParent,
		PGPKeyServerEntry*		pkeyserver,
		PGPKeyServerKeySpace	space,
		PGPKeyDBRef				keydbMain,
		PGPKeySetRef			keysetToSend)
{
	PSERVERTHREADSTRUCT	psts;
	PGPError			err;


	psts = (PSERVERTHREADSTRUCT)malloc (sizeof(SERVERTHREADSTRUCT));
	pgpClearMemory (psts, sizeof(SERVERTHREADSTRUCT));

	if (psts)
	{
		PGPNewEmptyKeySet (
				PGPPeekKeySetKeyDB (keysetToSend), &(psts->keysetIn));
		PGPAddKeys (keysetToSend, psts->keysetIn);

		psts->uOperation		= DELETEKEY;
		psts->uFlags			= 0;
		psts->context			= context;
		psts->tlsContext		= tlsContext;
		psts->keydbMain			= keydbMain;
		psts->lpThread			= sKeyserverThreadRoutine;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= FALSE;
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->space				= space;
		psts->groupset			= kInvalidPGPGroupSetRef;

		CopyMemory (&psts->ksentry, pkeyserver, sizeof(PGPKeyServerEntry));

		err = DialogBoxParam (g_hInst, MAKEINTRESOURCE(IDD_SERVERPROGRESS),
				hwndParent, sServerProgressDlgProc, (LPARAM)psts);

		if (!psts->bThreadFree)
		{
			if (psts->keysetIn)
				PGPFreeKeySet (psts->keysetIn);
			free (psts);
		}
	}

	return err;
}


//	___________________________________________________
//
//	request a certificate from a CA server

PGPError PGPclExport
PGPclSendCertificateRequestToServer (
		PGPContextRef			context,
		PGPtlsContextRef		tlsContext,
		HWND					hwndParent,
		PGPKeyDBRef				keydbMain,
		PGPKeyDBObjRef			userid,
		PGPKeySetRef			keysetKey)
{
	PGPError			err				= kPGPError_NoErr;
	PGPBoolean			bCheckForCRL	= FALSE;
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	PSERVERTHREADSTRUCT	psts			= NULL;
	PGPKeyServerEntry*	pentry			= NULL;

	PGPKeyDBObjRef		sigRootCA;
	PGPUInt32			uRequestType;

	err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
	err = PGPclGetRootCACertPrefs (context,
			prefref, keydbMain, &sigRootCA); CKERR;

	psts = PGPNewData (PGPPeekContextMemoryMgr (context),
			sizeof(SERVERTHREADSTRUCT), kPGPMemoryMgrFlags_Clear);
	if (psts)
	{
		psts->keyIn				= PGPPeekKeyDBObjKey (sigRootCA);
		psts->keysetIn			= keysetKey;
		psts->pszPassPhrase		= NULL;
		psts->keydbOut			= kInvalidPGPKeyDBRef;
		psts->uOperation		= REQUESTCERTIFICATE;
		psts->uFlags			= 0;
		psts->context			= context;
		psts->tlsContext		= tlsContext;
		psts->keydbMain			= keydbMain;
		psts->lpThread			= sCAThreadRoutine;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= FALSE;
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->space				= kPGPKeyServerKeySpace_Default;
		psts->groupset			= kInvalidPGPGroupSetRef;

		err = CLGetCAServerPrefs (context, prefref, &pentry);
		if (IsPGPError (err) ||
			(pentry[0].serverDNS[0] == '\0'))
		{
			err = kPGPError_Win32_NoCAServerPrefs;
			goto done;
		}
		pgpCopyMemory (&pentry[0],
				&psts->ksentry, sizeof(PGPKeyServerEntry));

		{
			PGPKeyDBObjRef	key;

			if (!PGPKeyDBObjRefIsValid (userid))
			{
				sGetFirstKeyInSet (keysetKey, &key);
				PGPGetPrimaryUserID (key, &userid);
			}

			err = PGPclGetCACertRequestAVList (
					hwndParent, context, FALSE, userid,
					pentry->type, &uRequestType,
					&(psts->pAVlist), &(psts->numAVs)); CKERR;
		}

		if (uRequestType == kPGPPrefCertRequestCRS)
		{
			err = DialogBoxParam (g_hInst,
					MAKEINTRESOURCE(IDD_SERVERPROGRESS),
					hwndParent, sServerProgressDlgProc, (LPARAM)psts);

			if (IsntPGPError (err) && IsntNull (psts->hwnd))
			{
				err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
				CLSetCAServerPrefs (context, prefref, &psts->ksentry);
				err = PGPclFlushClientLibPrefs (prefref, NULL); CKERR;
			}
		}
		else
		{
			err = sCreateServerRequest (psts, NULL, NULL, 0, NULL, NULL);
			if (IsntPGPError (err))
				err = kPGPError_UserAbort;
		}
	}

done :
	if (psts)
	{
		if (!psts->bThreadFree)
		{
			PGPclFreeCACertRequestAVList (psts->pAVlist, psts->numAVs);
			PGPFreeData (psts);
		}
	}
	if (IsntNull (pentry))
		PGPFreeData (pentry);

	return err;
}

//	___________________________________________________
//
//	request a certificate from a CA server

PGPError PGPclExport
PGPclSendCertificateRequestToServerNotify (
		PGPContextRef			context,
		PGPtlsContextRef		tlsContext,
		HWND					hwndToNotify,
		PGPKeyDBRef				keydbMain,
		PGPKeyDBObjRef			userid,
		PGPKeySetRef			keysetKey,
		LPSTR					pszPassPhrase)
{
	PGPError			err				= kPGPError_NoErr;
	PGPBoolean			bCheckForCRL	= FALSE;
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	PSERVERTHREADSTRUCT	psts			= NULL;
	PGPKeyServerEntry*	pentry			= NULL;

	PGPKeyDBObjRef		sigRootCA;
	PGPUInt32			uRequestType;

	err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
	err = PGPclGetRootCACertPrefs (context,
			prefref, keydbMain, &sigRootCA); CKERR;

	psts = PGPNewData (PGPPeekContextMemoryMgr (context),
			sizeof(SERVERTHREADSTRUCT), kPGPMemoryMgrFlags_Clear);
	if (psts)
	{
		psts->keyIn				= PGPPeekKeyDBObjKey (sigRootCA);
		psts->keysetIn			= keysetKey;
		psts->pszPassPhrase		= pszPassPhrase;
		psts->keydbOut			= kInvalidPGPKeyDBRef;
		psts->uOperation		= REQUESTCERTIFICATE;
		psts->uFlags			= 0;
		psts->context			= context;
		psts->tlsContext		= tlsContext;
		psts->keydbMain			= keydbMain;
		psts->hwnd				= hwndToNotify;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= FALSE;  // see below!
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->space				= kPGPKeyServerKeySpace_Default;
		psts->groupset			= kInvalidPGPGroupSetRef;

		err = CLGetCAServerPrefs (context, prefref, &pentry); CKERR;
		if (pentry[0].serverDNS[0] == '\0')
		{
			err = kPGPError_Win32_NoCAServerPrefs;
			goto done;
		}
		pgpCopyMemory (&pentry[0],
				&psts->ksentry, sizeof(PGPKeyServerEntry));

		{
			PGPKeyDBObjRef		key;

			if (!PGPKeyDBObjRefIsValid (userid))
			{
				sGetFirstKeyInSet (keysetKey, &key);
				PGPGetPrimaryUserID (key, &userid);
			}

			err = PGPclGetCACertRequestAVList (
					hwndToNotify, context, FALSE, userid,
					pentry->type, &uRequestType,
					&(psts->pAVlist), &(psts->numAVs)); CKERR;
		}

		if (uRequestType == kPGPPrefCertRequestCRS)
		{
			psts->bThreadFree = TRUE;
			_beginthread (sCAThreadRoutine, 0, (void*)psts);
		}
		else
		{
			err = sCreateServerRequest (psts, NULL, NULL, 0, NULL, NULL);
			if (IsntPGPError (err))
				err = kPGPError_UserAbort;
		}
	}

done :
	if (psts)
	{
		if (!psts->bThreadFree)
		{
			if (PGPKeySetRefIsValid (psts->keysetIn))
				PGPFreeKeySet (psts->keysetIn);
			PGPclFreeCACertRequestAVList (psts->pAVlist, psts->numAVs);
			PGPFreeData (psts);
		}
	}
	if (IsntNull (pentry))
		PGPFreeData (pentry);

	return err;
}

//	___________________________________________________
//
//	retrieve a previously-requested certificate from a CA server

PGPError PGPclExport
PGPclRetrieveCertificateFromServer (
		PGPContextRef			context,
		PGPtlsContextRef		tlsContext,
		HWND					hwndParent,
		PGPKeyDBRef				keydbMain,
		PGPKeySetRef			keysetKey,
		PGPKeyDBObjRef			userid,
		PGPByte*				pPasskey,
		PGPSize					sizePasskey,
		PGPKeyDBRef*			pkeydbCert)
{
	PGPError			err				= kPGPError_NoErr;
	PGPBoolean			bCheckForCRL	= FALSE;
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	PSERVERTHREADSTRUCT	psts			= NULL;
	PGPKeyServerEntry*	pentry			= NULL;

	PGPKeyDBObjRef		sigRootCA;

	err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
	err = PGPclGetRootCACertPrefs (context,
			prefref, keydbMain, &sigRootCA); CKERR;

	psts = PGPNewData (PGPPeekContextMemoryMgr (context),
			sizeof(SERVERTHREADSTRUCT), kPGPMemoryMgrFlags_Clear);
	if (psts)
	{
		// just verify that we can get key
		err = sGetFirstKeyInSet (keysetKey, &psts->keyIn);  CKERR;

		psts->keyIn				= PGPPeekKeyDBObjKey (sigRootCA);
		psts->keysetIn			= keysetKey;
		psts->pszPassPhrase		= NULL;
		psts->pPasskey			= pPasskey;
		psts->sizePasskey		= sizePasskey;
		psts->uOperation		= RETRIEVECERTIFICATE;
		psts->uFlags			= 0;
		psts->context			= context;
		psts->tlsContext		= tlsContext;
		psts->hwnd				= hwndParent;
		psts->keydbMain			= keydbMain;
		psts->lpThread			= sCAThreadRoutine;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= FALSE;
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->space				= kPGPKeyServerKeySpace_Default;
		psts->groupset			= kInvalidPGPGroupSetRef;

		err = PGPNewKeyDB (context, &psts->keydbOut); CKERR;

		err = CLGetCAServerPrefs (context, prefref, &pentry); CKERR;
		if (pentry[0].serverDNS[0] == '\0')
		{
			err = kPGPError_Win32_NoCAServerPrefs;
			goto done;
		}
		pgpCopyMemory (&pentry[0],
				&psts->ksentry, sizeof(PGPKeyServerEntry));

		if (hwndParent == NULL)
		{
			sCAThreadRoutine ((LPVOID)psts);
			err = psts->err;
		}
		else
		{
			err = DialogBoxParam (g_hInst,
					MAKEINTRESOURCE(IDD_SERVERPROGRESS),
					hwndParent, sServerProgressDlgProc, (LPARAM)psts);
		}

		if (IsPGPError (err))
		{
			PGPFreeKeyDB (psts->keydbOut);
			psts->keydbOut = kInvalidPGPKeyDBRef;
		}
		else
		{
			if (IsntNull (psts->hwnd))
			{
				err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
				CLSetCAServerPrefs (context, prefref, &psts->ksentry);
				if (PGPKeyDBRefIsValid (psts->keydbOut))
				{
					PGPUInt32	uCount	= 0;
					PGPCountKeys (
							PGPPeekKeyDBRootKeySet (psts->keydbOut), &uCount);

					if (uCount > 0)
						PGPSetPrefNumber (prefref, kPGPPrefLastCertRequest, 0);
				}
				err = PGPclFlushClientLibPrefs (prefref, NULL); CKERR;
			}

			if (pkeydbCert)
			{
				*pkeydbCert = psts->keydbOut;
				psts->keydbOut = kInvalidPGPKeyDBRef;
			}
		}
	}

done :
	if (psts)
	{
		if (!psts->bThreadFree)
		{
			if (PGPKeyDBRefIsValid (psts->keydbOut))
				PGPFreeKeyDB (psts->keydbOut);
			PGPFreeData (psts);
		}
	}
	if (IsntNull (pentry))
		PGPFreeData (pentry);

	return err;
}


//	___________________________________________________
//
//	retrieve certificate revocation list for root CA

PGPError PGPclExport
PGPclGetCertificateRevocationsFromServer (
		PGPContextRef			context,
		PGPtlsContextRef		tlsContext,
		HWND					hwndParent,
		PGPKeyDBRef				keydbMain)
{
	PGPError			err				= kPGPError_NoErr;
	PGPBoolean			bCheckForCRL	= FALSE;
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	PSERVERTHREADSTRUCT	psts			= NULL;
	PGPKeyServerEntry*	pentry			= NULL;

	PGPKeyDBObjRef		sigRootCA;
	PGPBoolean			bHasCRL;
	PGPTime				timeExpiration;

	err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
	err = PGPclGetRootCACertPrefs (context,
			prefref, keydbMain, &sigRootCA); CKERR;
	err = PGPGetKeyDBObjBooleanProperty (
			PGPPeekKeyDBObjKey (sigRootCA),
			kPGPKeyProperty_HasCRL, &bHasCRL); CKERR;

	if (bHasCRL)
	{
		err = PGPGetKeyDBObjTimeProperty (PGPPeekKeyDBObjKey (sigRootCA),
				kPGPKeyProperty_CRLNextUpdate, &timeExpiration); CKERR;

		if (timeExpiration <= PGPGetTime())
			bCheckForCRL = TRUE;
	}
	else
		bCheckForCRL = TRUE;

	if (bCheckForCRL)
	{
		psts = PGPNewData (PGPPeekContextMemoryMgr (context),
					sizeof(SERVERTHREADSTRUCT), kPGPMemoryMgrFlags_Clear);

		if (psts)
		{
			psts->keyIn				= PGPPeekKeyDBObjKey (sigRootCA);
			psts->keysetIn			= kInvalidPGPKeySetRef;
			psts->pszPassPhrase		= NULL;
			psts->uOperation		= UPDATEREVOCATIONS;
			psts->uFlags			= 0;
			psts->context			= context;
			psts->tlsContext		= tlsContext;
			psts->hwnd				= hwndParent;
			psts->keydbMain			= keydbMain;
			psts->lpThread			= sCAThreadRoutine;
			psts->bOpInProgress		= FALSE;
			psts->bCancel			= FALSE;
			psts->bThreadFree		= FALSE;
			psts->iStatusValue		= -1;
			psts->iStatusDirection	= 1;
			psts->space				= kPGPKeyServerKeySpace_Default;
			psts->groupset			= kInvalidPGPGroupSetRef;

			err = CLGetCAServerPrefs (context, prefref, &pentry); CKERR;
			if (pentry[1].serverDNS[0] == '\0')
			{
				err = kPGPError_Win32_NoCAServerPrefs;
				goto done;
			}
			pgpCopyMemory (&pentry[1],
					&psts->ksentry, sizeof(PGPKeyServerEntry));

			if (hwndParent == NULL)
			{
				sCAThreadRoutine ((LPVOID)psts);
				err = psts->err;
			}
			else
			{
				err = DialogBoxParam (g_hInst,
						MAKEINTRESOURCE(IDD_SERVERPROGRESS),
						hwndParent, sServerProgressDlgProc, (LPARAM)psts);
			}

			if (IsPGPError (err))
			{
				psts->keydbOut = kInvalidPGPKeyDBRef;
			}
			else
			{
				if (IsntNull (hwndParent))
				{
					err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
					CLSetCARevocationServerPrefs (
							context, prefref, &psts->ksentry);
					err = PGPclFlushClientLibPrefs (prefref, NULL); CKERR;
				}
			}
		}
	}
	else
		err = kPGPError_Win32_NoNewCRL;

done :
	if (psts)
	{
		if (!psts->bThreadFree)
		{
			PGPFreeData (psts);
		}
	}
	if (IsntNull (pentry))
		PGPFreeData (pentry);

	return err;
}


//	___________________________________________________
//
//	Create and send key reconstruction blob to server

PGPError PGPclExport
PGPclSendReconstructionBlobToServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBObjRef		key,
		PGPKeyDBRef			keydbMain)
{
	PGPError			err				= kPGPError_NoErr;
	PSERVERTHREADSTRUCT	psts			= NULL;

	psts = PGPNewSecureData (PGPPeekContextMemoryMgr (context),
			sizeof(SERVERTHREADSTRUCT), kPGPMemoryMgrFlags_Clear);
	if (psts)
	{
		psts->keyIn				= key;
		psts->pszPassPhrase		= NULL;
		psts->keydbMain			= keydbMain;
		psts->keydbOut			= kInvalidPGPKeyDBRef;
		psts->uOperation		= SENDRECONSTRUCTIONBLOB;
		psts->uFlags			= 0;
		psts->context			= context;
		psts->tlsContext		= tlscontext;
		psts->hwnd				= hwndParent;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= FALSE;
		psts->lpThread			= sLDAPThreadRoutine;
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->pszUserID			= NULL;
		psts->pszUserPass		= NULL;

		if (!DialogBoxParam (g_hInst,
				MAKEINTRESOURCE(IDD_RECONSTRUCTION),
				hwndParent, sReconstructionDlgProc, (LPARAM)psts))
		{
			err = kPGPError_UserAbort;
			goto done;
		}

		err = DialogBoxParam (g_hInst,
				MAKEINTRESOURCE(IDD_SERVERPROGRESS),
				hwndParent, sServerProgressDlgProc, (LPARAM)psts);
	}

done:
	if (psts)
	{
		if (!psts->bThreadFree)
		{
			if (IsntNull (psts->pszUserID))
				PGPFreeData (psts->pszUserID);
			if (IsntNull (psts->pszUserPass))
				PGPFreeData (psts->pszUserPass);
			if (IsntNull (psts->pszPassPhrase))
				PGPclFreePhrase (psts->pszPassPhrase);

			PGPFreeData (psts);
		}
	}

	return err;
}


//	___________________________________________________
//
//	Create and send key reconstruction blob to server with notifications

PGPError PGPclExport
PGPclSendReconstructionBlobToServerNotify (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndToNotify,
		PGPKeyDBObjRef		key,
		PGPKeyDBRef			keydbMain,
		PGPReconPrompts		prompts,
		PGPReconPasses		passes,
		char*				pszPassPhrase)
{
	PGPError			err				= kPGPError_NoErr;
	PSERVERTHREADSTRUCT	psts			= NULL;

	psts = PGPNewSecureData (PGPPeekContextMemoryMgr (context),
			sizeof(SERVERTHREADSTRUCT), kPGPMemoryMgrFlags_Clear);
	if (psts)
	{
		psts->keyIn				= key;
		psts->pszPassPhrase		= pszPassPhrase;
		psts->keydbMain			= keydbMain;
		psts->keydbOut			= kInvalidPGPKeyDBRef;
		psts->uOperation		= SENDRECONSTRUCTIONBLOB;
		psts->uFlags			= USEFULLERRORTEXT | TLSCONFIRMONLYIFINVALID;
		psts->context			= context;
		psts->tlsContext		= tlscontext;
		psts->hwnd				= hwndToNotify;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= FALSE;
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->pszUserID			= NULL;
		psts->pszUserPass		= NULL;

		memcpy (psts->reconPrompts, prompts, sizeof(PGPReconPrompts));
		memcpy (psts->reconPasses, passes, sizeof(PGPReconPasses));

		psts->bThreadFree = TRUE;
		_beginthread (sLDAPThreadRoutine, 0, (void*)psts);
	}

	if (psts)
	{
		if (!psts->bThreadFree)
		{
			if (IsntNull (psts->pszUserID))
				PGPFreeData (psts->pszUserID);
			if (IsntNull (psts->pszUserPass))
				PGPFreeData (psts->pszUserPass);
			PGPFreeData (psts);
		}
	}

	return err;
}

//	___________________________________________________
//
//	Get the previously-sent reconstruction blob and then reconstruct key

PGPError PGPclExport
PGPclRetrieveReconstructedKeyFromServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBObjRef		key,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBRef*		pkeydbReconstructed)
{
	PGPError			err				= kPGPError_NoErr;
	PSERVERTHREADSTRUCT	psts			= NULL;

	psts = PGPNewSecureData (PGPPeekContextMemoryMgr (context),
			sizeof(SERVERTHREADSTRUCT), kPGPMemoryMgrFlags_Clear);
	if (psts)
	{
		psts->keyIn				= key;
		psts->pszPassPhrase		= NULL;
		psts->keydbMain			= keydbMain;
		psts->keydbOut			= kInvalidPGPKeyDBRef;
		psts->uOperation		= RETRIEVERECONSTRUCTIONBLOB;
		psts->uFlags			= 0;
		psts->context			= context;
		psts->tlsContext		= tlscontext;
		psts->hwnd				= hwndParent;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= FALSE;
		psts->lpThread			= sLDAPThreadRoutine;
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->pszUserID			= NULL;
		psts->pszUserPass		= NULL;

		err = DialogBoxParam (g_hInst,
				MAKEINTRESOURCE(IDD_SERVERPROGRESS),
				hwndParent, sServerProgressDlgProc, (LPARAM)psts);

		if (IsntPGPError (err))
		{
			if (pkeydbReconstructed)
			{
				*pkeydbReconstructed = psts->keydbOut;
				psts->keydbOut = kInvalidPGPKeyDBRef;
			}
		}
	}

	if (psts)
	{
		if (PGPKeyDBRefIsValid (psts->keydbOut))
			PGPFreeKeyDB (psts->keydbOut);

		if (!psts->bThreadFree)
		{
			if (IsntNull (psts->pszUserID))
				PGPFreeData (psts->pszUserID);
			if (IsntNull (psts->pszUserPass))
				PGPFreeData (psts->pszUserPass);

			PGPFreeData (psts);
		}
	}

	return err;
}


//	___________________________________________________
//
//	Send preferences to server

PGPError PGPclExport
PGPclSendPrefsToServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPPrefRef			prefref,
		PGPPrefRef			prefrefNet,
		PGPKeyDBRef			keydbMain)
{
	PGPError			err				= kPGPError_NoErr;
	PSERVERTHREADSTRUCT	psts			= NULL;

	psts = PGPNewSecureData (PGPPeekContextMemoryMgr (context),
			sizeof(SERVERTHREADSTRUCT), kPGPMemoryMgrFlags_Clear);
	if (psts)
	{
		psts->pszPassPhrase		= NULL;
		psts->keydbOut			= kInvalidPGPKeyDBRef;
		psts->keydbMain			= keydbMain;
		psts->prefref			= prefref;
		psts->prefrefNet		= prefrefNet;
		psts->uOperation		= SENDPREFS;
		psts->uFlags			= 0;
		psts->context			= context;
		psts->tlsContext		= tlscontext;
		psts->hwnd				= hwndParent;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= FALSE;
		psts->lpThread			= sLDAPThreadRoutine;
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->pszUserID			= NULL;
		psts->pszUserPass		= NULL;

		if (hwndParent == NULL)
		{
			sLDAPThreadRoutine ((LPVOID)psts);
			err = psts->err;
		}
		else
		{
			err = DialogBoxParam (g_hInst,
					MAKEINTRESOURCE(IDD_SERVERPROGRESS),
					hwndParent, sServerProgressDlgProc, (LPARAM)psts);
		}
	}

	if (psts)
	{
		if (!psts->bThreadFree)
		{
			if (IsntNull (psts->pszUserID))
				PGPFreeData (psts->pszUserID);
			if (IsntNull (psts->pszUserPass))
				PGPFreeData (psts->pszUserPass);

			PGPFreeData (psts);
		}
	}

	return err;
}


//	___________________________________________________
//
//	Get preferences from server

PGPError PGPclExport
PGPclGetPrefsFromServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPPrefRef			prefref,
		PGPPrefRef			prefrefNet)
{
	PGPError			err				= kPGPError_NoErr;
	PSERVERTHREADSTRUCT	psts			= NULL;

	psts = PGPNewData (PGPPeekContextMemoryMgr (context),
			sizeof(SERVERTHREADSTRUCT), kPGPMemoryMgrFlags_Clear);
	if (psts)
	{
		psts->pszPassPhrase		= NULL;
		psts->keydbOut			= kInvalidPGPKeyDBRef;
		psts->prefref			= prefref;
		psts->prefrefNet		= prefrefNet;
		psts->uOperation		= GETPREFS;
		psts->uFlags			= 0;
		psts->context			= context;
		psts->tlsContext		= tlscontext;
		psts->hwnd				= hwndParent;
		psts->bOpInProgress		= FALSE;
		psts->bCancel			= FALSE;
		psts->bThreadFree		= FALSE;
		psts->lpThread			= sLDAPThreadRoutine;
		psts->iStatusValue		= -1;
		psts->iStatusDirection	= 1;
		psts->pszUserID			= NULL;
		psts->pszUserPass		= NULL;

		if (hwndParent == NULL)
		{
			sLDAPThreadRoutine ((LPVOID)psts);
			err = psts->err;
		}
		else
		{
			err = DialogBoxParam (g_hInst,
					MAKEINTRESOURCE(IDD_SERVERPROGRESS),
					hwndParent, sServerProgressDlgProc, (LPARAM)psts);
		}
	}

	if (psts)
	{
		if (!psts->bThreadFree)
		{
			PGPFreeData (psts);
		}
	}

	return err;
}

//	_______________________________________________
//
//  Export X.509 cert to the token

PGPError PGPclExport
PGPclCopyX509CertToToken(
						 PGPContextRef	context, 
						 PGPtlsContextRef tlscontext, 
						 HWND			hwnd,  
						 LPSTR			pszPhrase, 
						 PGPKeyDBRef	keydb,
						 PGPKeyDBObjRef	obj, 
						 PGPKeySetRef	keyset)  
{
	PGPError		    err = kPGPError_NoErr;
	HCURSOR			    hcursorOld;
	PGPByte *			pPasskey = NULL;
	PGPSize				sizePasskey = 0;
	PGPBoolean			bPhraseWasNull = (pszPhrase==NULL);
	
	if( bPhraseWasNull )
		err = PGPclGetKeyPhrase (context, tlscontext, hwnd,
			NULL, keydb, obj, 
			&pszPhrase, &pPasskey, &sizePasskey);
	
	if( IsntPGPError(err) )  {
		
		PGPByte *contOut=NULL;
		PGPSize contOutSize=0;
		PGPByte *certData = NULL;
		PGPSize certDataSize = 0;
		
		hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
		
		err = PGPExport (context,
			PGPOExportKeySet (context, keyset),
			// SDK will determine, based on keyID
			PGPOOutputToken (context, -1),
			PGPOAllocatedOutputKeyContainer( context, &contOut, 
			MAX_PGPUInt32, &contOutSize ), 
			PGPOExportFormat (context, kPGPExportFormat_X509Cert),
			PGPOPassphrase (context, pszPhrase),
			PGPOAllocatedOutputBuffer(context, 
			(void **)&certData, MAX_PGPUInt32, 
			&certDataSize),
			PGPOLastOption (context)
			);
		
		// Make certificate visible for Internet Explorer, if
		//  key container is available
		if( (IsntPGPError(err) || err==kPGPError_SmartCardX509Exists) && 
			contOutSize > 0 )  
		{
			PGPTokenInfo	pti = { sizeof(PGPTokenInfo) };
			CHAR            szID[kPGPMaxKeyIDStringSize] = "";
			
			PGPKeyID		keyID;
			PGPError        error;
			char			CSP[80];
			
			PGPByte *pUserID;
			PGPSize pUserID_len;
			PGPKeyDBObjRef userID;
			
			PGPGetTokenInfo (context, 0/*Always token 0 for now*/, &pti);
			PGPclIETokenProvToCSP( pti.manufacturerID, CSP, sizeof(CSP) );
			
			PGPGetKeyID( obj, &keyID );
			error = PGPGetKeyIDString (&keyID, 
				kPGPKeyIDString_Abbreviated, szID);
			
			PGPGetPrimaryUserID( obj, &userID );
			
			PGPGetKeyDBObjAllocatedDataProperty( userID, kPGPUserIDProperty_CommonName,
				&pUserID, &pUserID_len );
			
			PGPclIEExportX509( 
				certData, certDataSize, 
				pUserID, szID, 
				contOut, contOutSize, CSP );
			
			PGPFreeData( pUserID );
		}
		
		SetCursor (hcursorOld);
		
		
		if( !IsNull(contOut) )
			PGPFreeData (contOut);
		if( !IsNull(certData) )
			PGPFreeData (certData);
	}

	if( bPhraseWasNull )  {
		if (pPasskey)
			PGPFreeData (pPasskey);
		if( pPasskey )
			PGPFreeData (pszPhrase);
	}
	
    return err;
}
