/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	CLsrvprf.c - does the drudge work of setting up keyservers

	$Id: CLsrvprf.c,v 1.18 2002/10/29 18:30:41 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "pgpFeatures.h"

// external globals
extern HINSTANCE g_hInst;

//	___________________________________________________
//
//	handler for keyserver events that we care about (TLS events)

PGPError
CLHandleTLSEvent (
		PGPContextRef		context,
		PGPEvent*			pevent,
		HWND				hwnd,
		PGPKeyDBRef			keydb,
		PGPKeyServerEntry*	pentry,
		PGPclAuthType		authType,
		BOOL*				pbAllow)
{
	PGPError				err			= kPGPError_NoErr;

	PGPtlsSessionRef		tls;
	PGPtlsCipherSuiteNum	tlsCipher;
	PGPKeyDBObjRef			keyAuth;
	PGPPublicKeyAlgorithm	algAuth;
	PGPKeyID				keyidAuth;
	PGPKeyID				keyidExpected;

	*pbAllow = FALSE;

	if (pevent->type == kPGPEvent_KeyServerTLSEvent)
	{
		PGPEventKeyServerTLSData data = pevent->data.keyServerTLSData;

		if (data.state == kPGPKeyServerState_TLSConnectionSecured)
		{
			CHAR	szServer[256];

			if (strstr (pentry->serverDNS, "://"))
				lstrcpy (szServer, pentry->serverDNS);
			else
				PGPGetKeyServerURL (pentry, szServer);

			tls = pevent->data.keyServerTLSData.tlsSession;

			err = PGPtlsGetNegotiatedCipherSuite (
						tls, &tlsCipher); CKERR;

			err = PGPtlsGetRemoteAuthenticatedKey (
						tls, &keyAuth, NULL); CKERR;

			err = PGPGetKeyID (keyAuth, &keyidAuth); CKERR;

			err = PGPGetKeyDBObjNumericProperty (keyAuth,
						kPGPKeyProperty_AlgorithmID, &algAuth); CKERR;

			if (pentry->authAlg == kPGPPublicKeyAlgorithm_Invalid)
			{
				// no key previously set for this server
				err = PGPGetKeyIDString (&keyidAuth,
							kPGPKeyIDString_Full,
							pentry->authKeyIDString); CKERR;

				pentry->authAlg = algAuth;

				if (hwnd)
					PGPclSyncKeyserverPrefs (context, pentry);

				if (authType == kPGPclAuthUseAppropriate)
					authType = kPGPclAuthNewKey;

				if (hwnd)
				{
					err = PGPclConfirmRemoteAuthentication (
							context, hwnd, szServer, keyAuth,
							tlsCipher, keydb, authType);
				}

				// allow connection if user approves, or if this is
				// a silent operation (i.e. hwnd == NULL)
				if (IsntPGPError (err))
				{
					*pbAllow = TRUE;
				}
			}
			else
			{
				// key previously set -- see if we got the same one ...
				err = PGPNewKeyIDFromString (
							pentry->authKeyIDString,
							kPGPPublicKeyAlgorithm_Invalid,
							&keyidExpected); CKERR;

				if (PGPCompareKeyIDs (&keyidExpected, &keyidAuth))
				{
					// previously auth key and this auth key are different
					if (authType == kPGPclAuthDoNotConfirm)
					{
						// we get here only when doing a "get prefs" 
						// operation and the auth key does not match.
						// this condition should be reported through
						// ePO event reporting mechanism ...
						// for now, we just abort the current operation.
						err = kPGPError_UserAbort;
					}
					else
					{
						if (authType == kPGPclAuthUseAppropriate)
							authType = kPGPclAuthUnexpectedKey;

						if (hwnd)
						{
							err = PGPclConfirmRemoteAuthentication (
									context, hwnd, szServer, keyAuth,
									tlsCipher, keydb, authType);

							// if user explicitly approves
							if (IsntPGPError (err))
							{
								err = PGPGetKeyIDString (&keyidAuth,
											kPGPKeyIDString_Full,
											pentry->authKeyIDString); CKERR;

								pentry->authAlg = algAuth;

								if (hwnd)
									PGPclSyncKeyserverPrefs (context, pentry);

								*pbAllow = TRUE;
							}
						}
						else
							err = kPGPError_UserAbort;
					}
				}
				else
				{
					// previously auth key and this auth key are same
					if (authType == kPGPclAuthUseAppropriate)
						authType = kPGPclAuthExpectedKey;

					if (hwnd)
					{
						err = PGPclConfirmRemoteAuthentication (
								context, hwnd, szServer, keyAuth,
								tlsCipher, keydb, authType);
					}

					// if user approves or this is a silent operation,
					// then allow the connection
					if (IsntPGPError (err))
					{
						*pbAllow = TRUE;
					}
				}
			}
		}
	}

done :
	return err;
}


//	___________________________________________________
//
//	handler for keyserver events that we care about (TLS events)

static PGPError
sServerEventHandler (
		PGPContextRef	context,
		PGPEvent*		pevent,
		PGPUserValue	uservalue)
{
	PCLIENTSERVERSTRUCT	pcss	= (PCLIENTSERVERSTRUCT)uservalue;
	PGPError			err		= kPGPError_NoErr;
	BOOL				bDummy;

	switch (pevent->type) {
	case kPGPEvent_KeyServerTLSEvent:
		err = CLHandleTLSEvent (context, pevent, pcss->hwnd,
				pcss->keydbMain, pcss->pentry, 
				kPGPclAuthUseAppropriate, &bDummy);
		break;
	}

	return err;
}

//	___________________________________________________
//
//	Init keyservers

PGPError CLInitKeyServerPrefs(
		PGPUInt32			uServer,
		PGPKeyServerEntry*	pkeyserver,
		HWND				hwndParent,
		PGPContextRef		context,
		PGPKeyDBRef			keydbMain,
		CHAR*				pszUserID,
		PCLIENTSERVERSTRUCT	*ppcss,
		PGPKeyServerEntry	**ksEntries,
		PGPKeyServerSpec	**serverList,
		PGPUInt32*			numKSEntries)
{
	PGPError				err				= kPGPError_NoErr;
	CHAR					szDomain[kMaxServerNameLength+1];
	PGPKeyServerEntry		entryRoot;
	PGPPrefRef				clientPrefsRef;

	err = PGPclPeekClientLibPrefRefs(&clientPrefsRef,NULL);
	if (IsPGPError (err))
		return err;

	switch (uServer)
	{
		case kPGPclSpecifiedServer :
		{
			(*ksEntries) =
				(PGPKeyServerEntry *)pgpAlloc(sizeof(PGPKeyServerEntry));
			memcpy (*ksEntries, pkeyserver, sizeof(PGPKeyServerEntry));
			(*numKSEntries) = 1;
			break;
		}

		case kPGPclUserIDBasedServer :
		{
			szDomain[0] = '\0';
			if (pszUserID)
				PGPFindEmailDomain (pszUserID, szDomain);
			err = PGPCreateKeyServerPath (clientPrefsRef, szDomain,
				ksEntries, &(*numKSEntries));
		}
		break;

		case kPGPclDefaultServer :
		{
			err = PGPCreateKeyServerPath (clientPrefsRef, "",
				ksEntries, &(*numKSEntries));
			break;
		}

		case kPGPclRootServer :
		{
			err = PGPGetRootKeyServer (clientPrefsRef, &entryRoot);

			if( err == kPGPError_ItemNotFound )
			{
				PGPclMessageBox (hwndParent, IDS_CAPTIONINFO, 
						IDS_NOROOTSERVER, MB_OK|MB_ICONINFORMATION);
				return kPGPError_UserAbort;
			}
			(*ksEntries) =
				(PGPKeyServerEntry *)pgpAlloc(sizeof(PGPKeyServerEntry));
			memcpy (*ksEntries, &entryRoot, sizeof(PGPKeyServerEntry));
			(*numKSEntries) = 1;
			break;
		}

		case kPGPclAllServers :
		{
			PGPKeyServerEntry *keyserverList;

			err = PGPGetKeyServerPrefs (clientPrefsRef, &keyserverList, 
				&(*numKSEntries));

			if ( IsntPGPError( err ) )
			{
				(*ksEntries) = (PGPKeyServerEntry *) 
					pgpAlloc(sizeof(PGPKeyServerEntry) * (*numKSEntries));

				memcpy (*ksEntries, keyserverList,
					sizeof(PGPKeyServerEntry) * (*numKSEntries));

				PGPFreeKeyServerList(keyserverList);
			}
			break;
		}
	}

	if ((IsntPGPError (err)) &&
		(IsntNull (*ksEntries)) &&
		((*numKSEntries) > 0))
	{
		*ppcss = clAlloc ((*numKSEntries) * sizeof(CLIENTSERVERSTRUCT));

		if (*ppcss!=0)
		{
			*serverList = (PGPKeyServerSpec*)PGPNewData(
				PGPPeekContextMemoryMgr (context),
				(*numKSEntries) * sizeof(PGPKeyServerSpec),
				kPGPMemoryMgrFlags_Clear);

			if (IsntNull (*serverList))
			{
				PGPUInt32						iIndex;
				PGPKeyServerThreadStorageRef	previousStorage;

				PGPKeyServerInit();

				PGPKeyServerCreateThreadStorage(&previousStorage);

				for (iIndex = 0; iIndex < (*numKSEntries); iIndex++)
				{
					err = PGPNewKeyServer (context,
						(*ksEntries)[iIndex].type,
						&(*serverList)[iIndex].server,
						PGPONetHostName (context,
								(*ksEntries)[iIndex].serverDNS,
								(*ksEntries)[iIndex].serverPort),
						PGPOKeyServerProtocol (context,
								(*ksEntries)[iIndex].protocol),
						PGPOKeyServerAccessType (context,
								kPGPKeyServerAccessType_Normal),
						PGPOKeyServerKeySpace (context,
								kPGPKeyServerKeySpace_Normal),
						PGPOKeyServerKeyStoreDN(context, 
								((*ksEntries)[iIndex].keystoredn)),
						PGPOLastOption (context));

					if (IsPGPError (err))
						break;

					(*serverList)[iIndex].serverName =
						(*ksEntries)[iIndex].serverDNS;

					(*serverList)[iIndex].serverDomain =
						(*ksEntries)[iIndex].domain;

					(*ppcss)[iIndex].hwnd		= hwndParent;
					(*ppcss)[iIndex].keydbMain	= keydbMain;
					(*ppcss)[iIndex].pentry		= &(*ksEntries)[iIndex];

					err = PGPSetKeyServerEventHandler (
								(*serverList)[iIndex].server,
								sServerEventHandler,
								(PGPUserValue)&(*ppcss)[iIndex]);
				}

				PGPKeyServerDisposeThreadStorage(previousStorage);
			}
		}
	}

	return err;
}

//	___________________________________________________
//
//	Uninit keyservers

PGPError CLUninitKeyServerPrefs(
		PGPUInt32			uServer,
		PCLIENTSERVERSTRUCT	pcss,
		PGPKeyServerEntry	*ksEntries,
		PGPKeyServerSpec	*serverList,
		PGPUInt32			numKSEntries)
{
	PGPUInt32						iIndex;
	PGPError						err;
	PGPKeyServerThreadStorageRef	previousStorage;

	err=kPGPError_NoErr;

	PGPKeyServerCreateThreadStorage(&previousStorage);

	for (iIndex = 0; iIndex < numKSEntries; iIndex++)
	{
		if (PGPKeyServerRefIsValid (serverList[iIndex].server))
			PGPFreeKeyServer (serverList[iIndex].server);
	}

	if (IsntNull (serverList))
		PGPFreeData (serverList);

	if (IsntNull (ksEntries))
		PGPDisposeKeyServerPath (ksEntries);

	if (IsntNull (pcss))
		clFree(pcss);

	PGPKeyServerDisposeThreadStorage(previousStorage);

	PGPKeyServerCleanup ();

	return err;
}

