/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
		
	KLTree.c - handle creating and filling TreeList control	

	$Id: KLTree.c,v 1.36 2002/10/27 18:17:28 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "PGPklx.h"
#include "pgpImageList.h"
#include "shlwapi.h"

// constant defitions
#define BITMAP_WIDTH	16
#define BITMAP_HEIGHT	16

// macro defitions
#define PACKVERSION(major,minor) MAKELONG(minor,major)

typedef struct {
	FARPROC			lpfnCallback;
	HWND			hwndTree;
} EXPANDCOLLAPSESTRUCT;

typedef struct {
	FARPROC			lpfnCallback;
	HWND			hwndTree;
	PGPKeySetRef	keysetSelected;
} SELECTEDSTRUCT;

typedef struct _USERVALSTRUCT {
	struct _USERVALSTRUCT* next;
	PKEYLIST		pKL;
	LONG			lValue;
} USERVALSTRUCT;

// external globals
extern HINSTANCE	g_hInst;
extern HIMAGELIST	g_hilKeys;


//	_______________________________________________
//
//  get user value list item

PGPError 
KLGetObjUserVal (
		PKEYLIST		pKL, 
		PGPKeyDBObjRef	obj, 
		LONG*			lValue) 
{
	USERVALSTRUCT* puvs;
	PGPError err;

	err = PGPGetKeyDBObjUserValue (obj, &puvs);
	if (err) 
		return err;

	// there is an existing linked list
	if (puvs) 
	{
		// search for the element inserted by this KL
		do {
			if (puvs->pKL == pKL) 
			{
				*lValue = puvs->lValue;
				return kPGPError_NoErr;
			}
			puvs = puvs->next;
		} while (puvs);

		// no element in list inserted by this KL
		*lValue = 0;
	}

	// no user value
	else 
		*lValue = 0;

	return kPGPError_NoErr;
}


//	_______________________________________________
//
//  set key user value list item

PGPError 
KLSetObjUserVal (
		PKEYLIST		pKL, 
		PGPKeyDBObjRef	obj, 
		LONG			lValue) 
{
	USERVALSTRUCT* puvs;
	USERVALSTRUCT* puvsPrev;
	PGPError err;

	err = PGPGetKeyDBObjUserValue (obj, &puvs);
	if (err) 
		return err;

	// there is an existing linked list
	if (puvs) 
	{
		puvsPrev = NULL;
		// search for the element inserted by this KL
		do {
			if (puvs->pKL == pKL) 
			{
				// if value is zero, remove item from list
				if (!lValue) 
				{
					if (!puvsPrev) 
						PGPSetKeyDBObjUserValue (obj, 
								(PGPUserValue)puvs->next);
					else
						puvsPrev->next = puvs->next;
					KLFree (puvs);
				}
				// otherwise set the list element to the desired value
				else 
					puvs->lValue = lValue;
				return kPGPError_NoErr;
			}
			puvsPrev = puvs;
			puvs = puvs->next;
		} while (puvs);

		// no element in list inserted by this KL, create and append one
		if (!lValue) 
			return kPGPError_NoErr;
		puvs = KLAlloc (sizeof(USERVALSTRUCT));
		puvsPrev->next = puvs;
	}
	// no user value, create one
	else 
	{
		if (!lValue)
			return kPGPError_NoErr;

		puvs = KLAlloc (sizeof(USERVALSTRUCT));
		PGPSetKeyDBObjUserValue (obj, (PGPUserValue)puvs);
	}

	// set contents of linked list element
	puvs->pKL = pKL;
	puvs->next = NULL;
	puvs->lValue = lValue;

	return kPGPError_NoErr;
}


//	_______________________________________________
//
//  set key user value list item

static DWORD 
sGetComCtl32Version (VOID)
{
    HINSTANCE	hinstDll;
    DWORD		dwVersion = 0;

    hinstDll = LoadLibrary (TEXT("comctl32.dll"));
    if (hinstDll)
    {
        DLLGETVERSIONPROC pDllGetVersion;
        pDllGetVersion = 
				(DLLGETVERSIONPROC)GetProcAddress (hinstDll, "DllGetVersion");

        if (pDllGetVersion)
        {
            DLLVERSIONINFO	dvi;
            HRESULT			hr;

            ZeroMemory (&dvi, sizeof(dvi));
            dvi.cbSize = sizeof(dvi);

            hr = (*pDllGetVersion)(&dvi);

            if (SUCCEEDED(hr))
               dwVersion = PACKVERSION(dvi.dwMajorVersion, dvi.dwMinorVersion);
        }

        FreeLibrary(hinstDll);
    }
    return dwVersion;
}


//	_______________________________________________
//
//  Create TreeList Window

HKEYLIST PGPclExport 
PGPclKeyListCreate (
		PGPContextRef		context, 
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		INT					iID, 
		INT					iX, 
		INT					iY,
		INT					iWidth, 
		INT					iHeight,
		UINT				uFlags,
		HWND*				phwndTree) 
{
	PKEYLIST				pKL;
	DWORD					dwStyle;
	PGPclWindowsGenus		genus;

	pKL = KLAlloc (sizeof (KEYLIST));
	if (!pKL) 
		return NULL;

	memset (pKL, 0x00, sizeof (KEYLIST));

	pKL->hwndParent = hwndParent;
	pKL->hwndTree = NULL;

	pKL->iID = iID;
	pKL->pDropTarget = NULL;			//pointer to DropTarget object
	ZeroMemory (&pKL->keyserver, sizeof(PGPKeyServerEntry));	

	pKL->context = context;
	pKL->tlscontext = tlscontext;
	pKL->keysetDisp = kInvalidPGPKeySetRef;
	pKL->keydbBacking = kInvalidPGPKeyDBRef;
	pKL->keydbMain = kInvalidPGPKeyDBRef;
	pKL->bMainKeySet = FALSE;
	pKL->ulOptionFlags = 0;
	pKL->ulDisableActions = 0;
	pKL->ulHideColumns = 0;
	pKL->ulShowColumns = 0;

	pKL->bMultipleSelected = FALSE;
	pKL->uSelectedFlags = 0;
	pKL->iFocusedItemType = 0;
	pKL->iFocusedObjectType = 0;
	pKL->hFocusedItem = NULL;
	pKL->pFocusedObject = NULL;

	pKL->iValidityThreshold = PGPCL_VALIDITY_MARGINAL;

	pKL->iNumberSheets = 0;

	KLGetColumnPreferences (pKL);

	// Create the tree view window.

	dwStyle = WS_VISIBLE | WS_CHILD | WS_BORDER | WS_TABSTOP | WS_GROUP |
			TLS_HASBUTTONS | TLS_HASLINES | TLS_AUTOSCROLL | 
			TLS_PROMISCUOUS | TLS_DRAGABLEHEADERS | TLS_DRAGSELECTION;

	PGPclGetWindowsVersion (NULL, &genus, NULL);
	if (genus >= kPGPclWindowsGenusXp)
		dwStyle |= TLS_USEDEFAULTGUIFONT;

	// The new style of tooltips requires a recent version of the Common
	// Control DLL.
	if (sGetComCtl32Version() >= PACKVERSION(5,81))
		dwStyle |= TLS_TOOLTIPNOTIFY;

	if (uFlags & kPGPclKeyList_SingleSelection)
		dwStyle |= TLS_SINGLESELECT;

	if (uFlags & kPGPclKeyList_ShowSelection)
		dwStyle |= TLS_SHOWSELECTIONALWAYS;

	pKL->hwndTree = CreateWindowEx (WS_EX_CLIENTEDGE, WC_TREELIST, "",
			dwStyle, iX, iY, iWidth, iHeight,
			hwndParent, (HMENU)iID, g_hInst, NULL);

	if (pKL->hwndTree == NULL) 
		return NULL;
	*phwndTree = pKL->hwndTree;

	// Associate the image list with the tree view control.
	TreeList_SetImageList (pKL->hwndTree, g_hilKeys);

	KLSetFocus (pKL, NULL, FALSE);

 	return (HKEYLIST)pKL;
}


//	_______________________________________________
//
//  Insert column information into control

BOOL 
KLAddColumns (PKEYLIST pKL) 
{
	TL_COLUMN tlc;
	CHAR sz[64];
	INT iField, iCol, ids;

	TreeList_DeleteAllColumns (pKL->hwndTree);

	tlc.mask = TLCF_FMT | TLCF_WIDTH | TLCF_TEXT | 
				TLCF_SUBITEM | TLCF_DATATYPE | TLCF_DATAMAX |
				TLCF_MOUSENOTIFY;
	tlc.pszText = sz;

	tlc.iSubItem = 0;
	tlc.fmt = TLCFMT_LEFT;
	tlc.iDataType = TLC_DATASTRING;
	tlc.cx = pKL->wFieldWidth[0];
	tlc.bMouseNotify = FALSE;
	LoadString (g_hInst, IDS_NAMEFIELD, sz, sizeof(sz));
	TreeList_InsertColumn (pKL->hwndTree, 0, &tlc);

	for (iCol=1; iCol<NUMBERFIELDS; iCol++) 
	{
		iField = pKL->wColumnField[iCol];
		if (iField) 
		{
			switch (iField) {
			case KLI_VALIDITY :
				ids = IDS_VALIDITYFIELD;
				if (pKL->ulOptionFlags & kPGPclKeyList_GreenDotMode) 
					tlc.fmt = TLCFMT_IMAGE;
				else 
					tlc.fmt = TLCFMT_LINBAR;
				tlc.cchTextMax = 
						CLConvertFromPGPValidity (kPGPValidity_Complete);
				tlc.iDataType = TLC_DATALONG;
				tlc.bMouseNotify = TRUE;
				break;

			case KLI_SIZE :
				ids = IDS_SIZEFIELD;
				tlc.fmt = TLCFMT_LEFT;
				tlc.cchTextMax = 0;
				tlc.iDataType = TLC_DATASTRING;
				tlc.bMouseNotify = FALSE;
				break;

			case KLI_DESCRIPTION :
				ids = IDS_DESCRIPTIONFIELD;
				tlc.fmt = TLCFMT_LEFT;
				tlc.cchTextMax = 0;
				tlc.iDataType = TLC_DATASTRING;
				tlc.bMouseNotify = FALSE;
				break;

			case KLI_KEYID :
				ids = IDS_KEYIDFIELD;
				tlc.fmt = TLCFMT_LEFT;
				tlc.cchTextMax = 0;
				tlc.iDataType = TLC_DATASTRING;
				tlc.bMouseNotify = FALSE;
				break;
				
			case KLI_TRUST :
				ids = IDS_TRUSTFIELD;
				tlc.fmt = TLCFMT_LINBAR;
				tlc.cchTextMax = 
						CLConvertFromPGPTrust (kPGPKeyTrust_Complete);
				tlc.iDataType = TLC_DATALONG;
				tlc.bMouseNotify = FALSE;
				break;

			case KLI_CREATION :
				ids = IDS_CREATIONFIELD;
				tlc.fmt = TLCFMT_LEFT;
				tlc.cchTextMax = 0;
				tlc.iDataType = TLC_DATASTRING;
				tlc.bMouseNotify = FALSE;
				break;
				
			case KLI_EXPIRATION :
				ids = IDS_EXPIRATIONFIELD;
				tlc.fmt = TLCFMT_LEFT;
				tlc.cchTextMax = 0;
				tlc.iDataType = TLC_DATASTRING;
				tlc.bMouseNotify = FALSE;
				break;
				
			case KLI_ADK :
				ids = IDS_ADKFIELD;
				tlc.fmt = TLCFMT_IMAGE;
				tlc.cchTextMax = 0;
				tlc.iDataType = TLC_DATALONG;
				tlc.bMouseNotify = FALSE;
				break;
			}
			LoadString (g_hInst, ids, sz, sizeof(sz));
			tlc.cx = pKL->wFieldWidth[iField];
			TreeList_InsertColumn (pKL->hwndTree, iCol, &tlc);
		}
	}

	return TRUE;
}


//	_______________________________________________
//
//  Set (or add) a tree item to the tree

static HTLITEM 
sSetOneItem (
		PKEYLIST		pKL, 
		BOOL		bReInsertExisting, 
		HTLITEM		hItem, 
		HTLITEM		hParent, 
		LPSTR		szText, 
		HTLITEM		hInsAfter, 
		INT			iImage, 
		UINT		uState, 
		LPARAM		lParam) 
{
	TL_TREEITEM tlI;
	TL_INSERTSTRUCT tlIns;

	tlI.hItem = hItem;
	tlI.mask = TLIF_TEXT | TLIF_IMAGE | TLIF_STATE | TLIF_PARAM;
	tlI.stateMask = TLIS_BOLD | TLIS_ITALICS;
	tlI.stateMask |= uState;
	tlI.state = uState;
	tlI.pszText = szText;
	tlI.cchTextMax = lstrlen (szText);
	tlI.iImage = iImage;
	tlI.iSelectedImage = iImage;
	tlI.lParam = lParam;

	// Insert the data into the tree.
	if (bReInsertExisting || !hItem) 
	{
		tlIns.item = tlI;
		tlIns.hInsertAfter = hInsAfter;
		tlIns.hParent = hParent;
		return (TreeList_InsertItem (pKL->hwndTree, &tlIns));
	}
	else 
	{
		TreeList_SetItem (pKL->hwndTree, &tlI);
		return hItem;
	}
}


//	_______________________________________________
//
//  Set list data for a key

static VOID 
sFillDescription (
		INT		idx, 
		INT		iVal, 
		LPSTR	sz, 
		INT		iLen) 
{
	INT		ids				= 0;
	BOOL	bSig			= FALSE;
	CHAR	szTemp[128];

	switch (idx) {

	case IDX_RSAPUBKEY :		ids = IDS_RSAPUBKEY;		break;
	case IDX_RSAPUBDISKEY :		ids = IDS_RSAPUBDISKEY;		break;
	case IDX_RSAPUBREVKEY :		ids = IDS_RSAPUBREVKEY;		break;
	case IDX_RSAPUBEXPKEY :		ids = IDS_RSAPUBEXPKEY;		break;
	
	case IDX_RSASECKEY :		ids = IDS_RSASECKEY;		break;
	case IDX_RSASECDISKEY :		ids = IDS_RSASECDISKEY;		break;
	case IDX_RSASECREVKEY :		ids = IDS_RSASECREVKEY;		break;
	case IDX_RSASECEXPKEY :		ids = IDS_RSASECEXPKEY;		break;
	case IDX_RSASECSHRKEY :		ids = IDS_RSASECSHRKEY;		break;

	case IDX_RSAV4PUBKEY :		ids = IDS_RSAV4PUBKEY;		break;
	case IDX_RSAV4PUBDISKEY :	ids = IDS_RSAV4PUBDISKEY;	break;
	case IDX_RSAV4PUBREVKEY :	ids = IDS_RSAV4PUBREVKEY;	break;
	case IDX_RSAV4PUBEXPKEY :	ids = IDS_RSAV4PUBEXPKEY;	break; 

	case IDX_RSAV4SECKEY :		ids = IDS_RSAV4SECKEY;		break;
	case IDX_RSAV4SECDISKEY :	ids = IDS_RSAV4SECDISKEY;	break; 
	case IDX_RSAV4SECREVKEY :	ids = IDS_RSAV4SECREVKEY;	break; 
	case IDX_RSAV4SECEXPKEY :	ids = IDS_RSAV4SECEXPKEY;	break; 
	case IDX_RSAV4SECSHRKEY :	ids = IDS_RSAV4SECSHRKEY;	break; 

	case IDX_RSATOKKEY :		ids = IDS_RSATOKKEY;		break;
	case IDX_RSATOKDISKEY :		ids = IDS_RSATOKDISKEY;		break;
	case IDX_RSATOKREVKEY :		ids = IDS_RSATOKREVKEY;		break;
	case IDX_RSATOKEXPKEY :		ids = IDS_RSATOKEXPKEY;		break;

	case IDX_RSAV4TOKKEY :		ids = IDS_RSAV4TOKKEY;		break;
	case IDX_RSAV4TOKDISKEY :	ids = IDS_RSAV4TOKDISKEY;	break;
	case IDX_RSAV4TOKREVKEY :	ids = IDS_RSAV4TOKREVKEY;	break;
	case IDX_RSAV4TOKEXPKEY :	ids = IDS_RSAV4TOKEXPKEY;	break;

	case IDX_DSAPUBKEY :		ids = IDS_DSAPUBKEY;		break;
	case IDX_DSAPUBDISKEY :		ids = IDS_DSAPUBDISKEY;		break;
	case IDX_DSAPUBREVKEY :		ids = IDS_DSAPUBREVKEY;		break;
	case IDX_DSAPUBEXPKEY :		ids = IDS_DSAPUBEXPKEY;		break;

	case IDX_DSASECKEY :		ids = IDS_DSASECKEY;		break;
	case IDX_DSASECDISKEY :		ids = IDS_DSASECDISKEY;		break;
	case IDX_DSASECREVKEY :		ids = IDS_DSASECREVKEY;		break;
	case IDX_DSASECEXPKEY :		ids = IDS_DSASECEXPKEY;		break;
	case IDX_DSASECSHRKEY :		ids = IDS_DSASECSHRKEY;		break;

	case IDX_ECPUBKEY :			ids = IDS_ECPUBKEY;			break;
	case IDX_ECPUBDISKEY :		ids = IDS_ECPUBDISKEY;		break;
	case IDX_ECPUBREVKEY :		ids = IDS_ECPUBREVKEY;		break;
	case IDX_ECPUBEXPKEY :		ids = IDS_ECPUBEXPKEY;		break;

	case IDX_ECSECKEY :			ids = IDS_ECSECKEY;			break;
	case IDX_ECSECDISKEY :		ids = IDS_ECSECDISKEY;		break;
	case IDX_ECSECREVKEY :		ids = IDS_ECSECREVKEY;		break;
	case IDX_ECSECEXPKEY :		ids = IDS_ECSECEXPKEY;		break;
	case IDX_ECSECSHRKEY :		ids = IDS_ECSECSHRKEY;		break;

	case IDX_RSAUSERID :		ids = IDS_RSAUSERID;		break;
	case IDX_DSAUSERID :		ids = IDS_DSAUSERID;		break;
	case IDX_PHOTOUSERID :		ids = IDS_PHOTOUSERID;		break;
	case IDX_INVALIDUSERID :	ids = IDS_UNKNOWNFORMAT;	break;

	case IDX_CERT :		
		ids = IDS_CERT;	
		bSig = TRUE;
		break;

	case IDX_REVCERT :		
		ids = IDS_REVCERT;			
		bSig = TRUE;
		break;

	case IDX_EXPCERT :		
		ids = IDS_EXPCERT;			
		bSig = TRUE;
		break;

	case IDX_BADCERT :		
		ids = IDS_BADCERT;			
		bSig = TRUE;
		break;

	case IDX_EXPORTCERT :	
		ids = IDS_EXPORTCERT;		
		bSig = TRUE;
		break;

	case IDX_TRUSTEDCERT :		
		ids = IDS_EXPORTMETACERT;	
		bSig = TRUE;
		break;

	case IDX_METACERT:
		ids = IDS_METACERT;			
		bSig = TRUE;
		break;

	case IDX_X509CERT :
		ids = IDS_X509CERT;
		bSig = FALSE;
		break;

	case IDX_X509EXPCERT :
		ids = IDS_X509EXPCERT;
		bSig = FALSE;
		break;

	case IDX_X509REVCERT :
		ids = IDS_X509REVCERT;
		bSig = FALSE;
		break;

	case IDX_X509BADCERT :
		ids = IDS_X509BADCERT;
		bSig = FALSE;
		break;

	default :				
		ids = IDS_UNKNOWNFORMAT;			
		break;
	}

	sz[0] = '\0';

	if (bSig) 
	{
		switch (iVal) {
		case kPGPPublicKeyAlgorithm_RSA :
			LoadString (g_hInst, IDS_RSA, sz, iLen);
			break;

		case kPGPPublicKeyAlgorithm_DSA :
			LoadString (g_hInst, IDS_DSS, sz, iLen);
			break;

		case kPGPPublicKeyAlgorithm_ECSign :
			LoadString (g_hInst, IDS_EC, sz, iLen);
			break;

		default :
			LoadString (g_hInst, IDS_UNKNOWN, sz, iLen);
			break;
		}
		lstrcat (sz, " ");
	}

	LoadString (g_hInst, ids, szTemp, sizeof(szTemp));
	lstrcat (sz, szTemp);
}


//	_______________________________________________
//
//  Set list data for a key

static HTLITEM 
sSetKeyData (
		PKEYLIST		pKL, 
		HTLITEM			hItem, 
		PGPKeyDBObjRef	key, 
		INT				idx) 
{ 
	TL_LISTITEM		tlL;
	INT				iField, iCol;
	PGPBoolean		bAxiomatic;
	CHAR			szText [128];
	INT				iValue;
	PGPTime			time;
	UINT			u;

	tlL.pszText = szText;
	tlL.hItem = hItem;
	tlL.stateMask = TLIS_VISIBLE;

	for (iCol=1; iCol<NUMBERFIELDS; iCol++) 
	{
		iField = pKL->wColumnField[iCol];
		if (iField) 
		{
			switch (iField) {
			case KLI_VALIDITY :
				PGPGetPrimaryUserIDValidity (key, &iValue);
				iValue = CLConvertFromPGPValidity (iValue);

				PGPGetKeyDBObjBooleanProperty (key, 
						kPGPKeyProperty_IsAxiomatic, &bAxiomatic);
				if (bAxiomatic) 
				{
					iValue = 
						CLConvertFromPGPValidity (kPGPValidity_Complete)+1;
				}

				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_DATAVALUE | TLIF_STATE;
				if (pKL->ulOptionFlags & kPGPclKeyList_GreenDotMode) 
				{
					if (iValue > PGPCL_VALIDITY_COMPLETE)
						tlL.lDataValue = IDX_AXIOMATIC;
					else if (iValue >= pKL->iValidityThreshold) 
						tlL.lDataValue = IDX_VALID;
					else 
						tlL.lDataValue = IDX_INVALID;
				}
				else 
					tlL.lDataValue = iValue;
				break;

			case KLI_SIZE :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_TEXT | TLIF_STATE;
				CLGetKeyBitsString (key, szText, sizeof(szText));
				break;

			case KLI_DESCRIPTION :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_TEXT | TLIF_STATE;
				sFillDescription (idx, 0, szText, sizeof(szText));
				break;

			case KLI_KEYID :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_TEXT | TLIF_STATE;
				KLGetKeyIDFromKey (key, szText, sizeof(szText));
				break;
				
			case KLI_TRUST :
				PGPGetKeyDBObjNumericProperty (key, 
						kPGPKeyProperty_Trust, &iValue);
				iValue = CLConvertFromPGPTrust (iValue);

				PGPGetKeyDBObjBooleanProperty (key, 
						kPGPKeyProperty_IsAxiomatic, &bAxiomatic);
				if (bAxiomatic) 
				{
					iValue = 
						CLConvertFromPGPTrust (kPGPKeyTrust_Ultimate)+1;
				}

				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_DATAVALUE | TLIF_STATE;
				tlL.lDataValue = iValue;
				break;

			case KLI_CREATION :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_TEXT | TLIF_STATE;
				PGPGetKeyDBObjTimeProperty (key, 
						kPGPKeyProperty_Creation, &time);
				CLConvertTimeToString (time, szText, sizeof(szText));
				break;
				
			case KLI_EXPIRATION :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_TEXT | TLIF_STATE;
				PGPGetKeyDBObjTimeProperty (key, 
						kPGPKeyProperty_Expiration, &time);
				if (time != kPGPExpirationTime_Never) 
					CLConvertTimeToString (time, szText, sizeof (szText));
				else 
					LoadString (g_hInst, IDS_NEVER, szText, sizeof (szText));
				break;
				
			case KLI_ADK :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_DATAVALUE | TLIF_STATE;
				PGPCountAdditionalRecipientRequests (key, &u);
				if (u > 0) 
					tlL.lDataValue = IDX_ADK;
				else
					tlL.lDataValue = IDX_NOADK;
				break;
			}

			tlL.iSubItem = iCol;
			hItem = (HTLITEM) TreeList_SetListItem (
					pKL->hwndTree, &tlL, FALSE);
		}
	}

	return (hItem);
}


//	_______________________________________________
//
//  Set treelist list data for a userID

static HTLITEM 
sSetIDData (
		PKEYLIST		pKL, 
		HTLITEM			hItem, 
		PGPKeyDBObjRef	userid, 
		INT				idx) 
{
	TL_LISTITEM tlL;
	CHAR szText [128];
	INT iField, iCol;
	INT iValue;

	tlL.pszText = szText;
	tlL.hItem = hItem;
	tlL.stateMask = TLIS_VISIBLE;

	for (iCol=1; iCol<NUMBERFIELDS; iCol++) 
	{
		iField = pKL->wColumnField[iCol];
		if (iField) 
		{
			switch (iField) {
			case KLI_VALIDITY :
				PGPGetKeyDBObjNumericProperty (userid, 
						kPGPUserIDProperty_Validity, &iValue);
				iValue = CLConvertFromPGPValidity (iValue);
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_DATAVALUE | TLIF_STATE;
				if (pKL->ulOptionFlags & kPGPclKeyList_GreenDotMode) 
				{
					if (iValue >= pKL->iValidityThreshold) 
						tlL.lDataValue = IDX_VALID;
					else 
						tlL.lDataValue = IDX_INVALID;
				}
				else tlL.lDataValue = iValue;
				break;

			case KLI_SIZE :
				tlL.state = 0;
				tlL.mask = TLIF_STATE;
				break;

			case KLI_DESCRIPTION :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_TEXT | TLIF_STATE;
				sFillDescription (idx, 0, szText, sizeof(szText));
				break;

			case KLI_KEYID :
				tlL.state = 0;
				tlL.mask = TLIF_STATE;
				break;
				
			case KLI_TRUST :
				tlL.state = 0;
				tlL.mask = TLIF_STATE;
				break;

			case KLI_CREATION :
				tlL.state = 0;
				tlL.mask = TLIF_STATE;
				break;
				
			case KLI_EXPIRATION :
				tlL.state = 0;
				tlL.mask = TLIF_STATE;
				break;
				
			case KLI_ADK :
				tlL.state = 0;
				tlL.mask = TLIF_STATE;
				break;
			}
			tlL.iSubItem = iCol;
			hItem = (HTLITEM) TreeList_SetListItem (pKL->hwndTree, 
													&tlL, FALSE);
		}
	}
	return (hItem);
}


//	_______________________________________________
//
//  Set treelist list data for a certification

static HTLITEM 
sSetCertData (
		PKEYLIST		pKL, 
		HTLITEM			hItem, 
		PGPKeyDBObjRef	cert, 
		INT				idx,
		BOOL			bX509) 
{
	TL_LISTITEM tlL;
	CHAR		szText [128];
	INT			iField, iCol;
	PGPTime		time;
	INT			iAlg;

	tlL.pszText = szText;
	tlL.hItem = hItem;
	tlL.stateMask = TLIS_VISIBLE;

	for (iCol=1; iCol<NUMBERFIELDS; iCol++) 
	{
		iField = pKL->wColumnField[iCol];
		if (iField) 
		{
			switch (iField) {
			case KLI_VALIDITY :
				tlL.state = 0;
				tlL.mask = TLIF_STATE;
				break;

			case KLI_SIZE :
				tlL.state = 0;
				tlL.mask = TLIF_STATE;
				break;

			case KLI_DESCRIPTION :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_TEXT | TLIF_STATE;
				PGPGetKeyDBObjNumericProperty (cert, 
						kPGPSigProperty_AlgorithmID, &iAlg);
				sFillDescription (idx, iAlg, szText, sizeof(szText));
				break;

			case KLI_KEYID :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_TEXT | TLIF_STATE;
				KLGetKeyIDFromCert (cert, bX509, szText, sizeof(szText));
				break;
				
			case KLI_TRUST :
				tlL.state = 0;
				tlL.mask = TLIF_STATE;
				break;

			case KLI_CREATION :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_TEXT | TLIF_STATE;
				PGPGetKeyDBObjTimeProperty (cert, 
						kPGPSigProperty_Creation, &time);
				CLConvertTimeToString (time, szText, sizeof(szText));
				break;
				
			case KLI_EXPIRATION :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_TEXT | TLIF_STATE;
				PGPGetKeyDBObjTimeProperty (cert, 
						kPGPSigProperty_Expiration, &time);
				if (time != kPGPExpirationTime_Never) 
					CLConvertTimeToString (time, szText, sizeof (szText));
				else 
					LoadString (g_hInst, IDS_NEVER, szText, sizeof (szText));
				break;
				
			case KLI_ADK :
				tlL.state = 0;
				tlL.mask = TLIF_STATE;
				break;
			}
			tlL.iSubItem = iCol;
			hItem = (HTLITEM) TreeList_SetListItem (pKL->hwndTree, 
													&tlL, FALSE);
		}
	}
	return (hItem);
}


//	_______________________________________________
//
//  Reload a single key

static HTLITEM 
sReloadKey (
		PKEYLIST		pKL, 
		PGPKeyIterRef	keyiter, 
		PGPKeyDBObjRef	key, 
		BOOL			bReInsertExisting, 
		BOOL			bForceNewAlloc, 
		BOOL			bExpandNew, 
		BOOL			bFirstCall, 
		HTLITEM			hTPrev) 
{
	HTLITEM			hTKey, hTUID, hTCert;
	TL_TREEITEM		tli;
	INT				idx;
	CHAR			sz[kPGPMaxUserIDSize +1];
	CHAR			sz2[64];
	CHAR			szID[32];
	UINT			uState;
	PGPBoolean		bItalics, bNew, bX509;
	PGPKeyDBObjRef	userid;
	PGPKeyDBObjRef	sig;
	PGPKeyDBObjRef	keyCert;
	PGPSize			size;
	PGPError		err;

	PGPError		errOld;

	static BOOL		bNewKeyExpanded;

	uState = 0;
	bNew = FALSE;
	if (bFirstCall) 
		bNewKeyExpanded = FALSE;

	KLGetObjUserVal (pKL, key, (long*)&hTKey);
	if (!hTKey) 
		bNew = TRUE;

	// determine icon and italics
	idx = PGPclDetermineKeyIcon (key, &bItalics);
	if (bItalics) 
		uState |= TLIS_ITALICS;

	// get primary userid name string
	KLGetKeyName (key, sz, sizeof(sz));

	// insert key item into tree and save pointer to tree item
	if (!hTKey && bExpandNew) 
		uState |= TLIS_SELECTED;
	if (bForceNewAlloc) 
		hTKey = NULL;
	hTKey = sSetOneItem (pKL, bReInsertExisting, hTKey, 
			NULL, sz, hTPrev, idx, uState, (LPARAM)key);
	KLSetObjUserVal (pKL, key, (long)hTKey);

	// if a reinsertion, then we're done
	if (bReInsertExisting) 
		return hTKey;

	sSetKeyData (pKL, hTKey, key, idx);

	// iterate through userids
	PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_UserID, &userid);
	while (userid) 
	{
		uState = 0;
		KLGetObjUserVal (pKL, userid, (long*)&hTUID);
		if (!hTUID) 
			bNew = TRUE;

		// get and set treelist tree data for this userid
		KLGetUserIDName (userid, sz, sizeof(sz));
		if (bForceNewAlloc) 
			hTUID = NULL;

		idx = PGPclDetermineUserIDIcon (userid, &bItalics);
		if (bItalics) 
			uState |= TLIS_ITALICS;

		hTUID = sSetOneItem (pKL, FALSE, hTUID, hTKey, sz, 
				(HTLITEM)TLI_LAST, idx, uState, (LPARAM)userid);
		KLSetObjUserVal (pKL, userid, (long)hTUID);

		// get and set treelist list data for this userid
		sSetIDData (pKL, hTUID, userid, idx);
		
		// iterate through certifications
		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Signature, &sig);
		while (sig) 
		{
			uState = 0;
			KLGetObjUserVal (pKL, sig, (long*)&hTCert);
			if (!hTCert) 
				bNew = TRUE;

			// get and set treelist tree data for this cert
			err = PGPGetSigCertifierKey (sig, kInvalidPGPKeyDBRef, &keyCert);
			errOld = err;
			if (err == kPGPError_ItemNotFound) 
			{
				err = kPGPError_NoErr;
				keyCert = kInvalidPGPKeyDBObjRef;
			}

			if (!PGPKeyDBObjRefIsValid (keyCert) && 
				(pKL->keydbMain != pKL->keydbBacking)) 
			{
				err = PGPGetSigCertifierKey (sig, pKL->keydbMain, &keyCert);
			}

			if (err == kPGPError_ItemNotFound) 
			{
				err = kPGPError_NoErr;
				keyCert = kInvalidPGPKeyDBObjRef;
			}

			PGPclErrorBox (NULL, err);
			bItalics = FALSE;
			idx = PGPclDetermineCertIcon (sig, &bItalics, &bX509);
			if (bX509)
			{
				PGPGetKeyDBObjDataProperty (sig, 
						kPGPSigProperty_X509IssuerLongName, 
						sz, sizeof(sz), &size);
				sz[min(sizeof(sz)-1, size)] = '\0';
				if (!PGPKeyDBObjRefIsValid (keyCert))
					bItalics = TRUE;
			}
			else
			{
				if (PGPKeyDBObjRefIsValid (keyCert))
				{
					if (!KLGetKeyName (keyCert, sz, sizeof(sz)))
						err = errOld;
				}
				else 
				{
					bItalics = TRUE;
					KLGetKeyIDFromCert (sig, FALSE, szID, sizeof(szID));
					LoadString (g_hInst, 
							IDS_UNAVAILABLECERT, sz2, sizeof(sz2));
					wsprintf (sz, sz2, szID);
				}
			}
			if (bItalics) 
				uState |= TLIS_ITALICS;
			if (bForceNewAlloc) 
				hTCert = NULL;
			hTCert = sSetOneItem (pKL, FALSE, hTCert, hTUID, sz, 
					(HTLITEM)TLI_LAST, idx, uState, (LPARAM)sig);
			KLSetObjUserVal (pKL, sig, (long)hTCert);

			// get and set treelist list data for this cert
			sSetCertData (pKL, hTCert, sig, idx, bX509);

			PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Signature, &sig);
		} 
		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_UserID, &userid);
	}

	// select and expand key, if appropriate 
	if (bExpandNew && bNew) 
	{
		tli.hItem = hTKey;
		if (!bNewKeyExpanded) 
		{
			TreeList_Select (pKL->hwndTree, &tli, TRUE);
			TreeList_Expand (pKL->hwndTree, &tli, TLE_EXPANDALL);
			bNewKeyExpanded = TRUE;
		}
		else 
		{
			KLSetFocus (pKL, KLFocusedItem (pKL), TRUE);
			TreeList_Expand (pKL->hwndTree, &tli, TLE_EXPANDALL);
		}
	}

	return hTKey;
}


//	_______________________________________________
//
//  Scan entire keyring loading in any new data
//
//	bReInsertExisting	- is only used when reordering keys in window.
//						  This is set TRUE if the caller has already
//						  deleted the treelist but not deallocated the items.
//	bExpandNew			- causes new keys to be expanded
//	bForceRealloc		- forces reallocation of treelist items.  Used after
//						  the caller has deleted the treelist and deallocated
//						  all the items. 

BOOL 
KLLoadKeyRingIntoTree (
		PKEYLIST	pKL, 
		BOOL		bReInsertExisting, 
		BOOL		bExpandNew, 
		BOOL		bForceRealloc) 
{
	PGPKeyDBObjRef	keyDef			= kInvalidPGPKeyDBObjRef;

	HCURSOR			hCursorOld;
	PGPKeyListRef	keylist;
	PGPKeyIterRef	keyiter;
	PGPKeyDBObjRef	key;
	TL_TREEITEM		tli;
	HTLITEM			hTPrevKey;
	BOOL			bFirst;
	PGPBoolean		bSecret, bRevoked, bExpired, bCanSign;

	hTPrevKey = (HTLITEM)TLI_FIRST;
	bFirst = TRUE;

	if (PGPKeySetRefIsValid (pKL->keysetDisp))
	{
		hCursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
		PGPOrderKeySet (pKL->keysetDisp, 
				pKL->lKeyListSortField, pKL->bKeyListSortInvert, &keylist);
		PGPNewKeyIter (keylist, &keyiter);

		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);

		while (PGPKeyDBObjRefIsValid (key)) 
		{
			hTPrevKey = sReloadKey (pKL, keyiter, key, bReInsertExisting, 
					bForceRealloc, bExpandNew, bFirst, hTPrevKey);
			PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);
			bFirst = FALSE;
		}
		PGPFreeKeyIter (keyiter);
		PGPFreeKeyList (keylist);


		if (pKL->bMainKeySet) 
		{
			PGPclGetDefaultPrivateKey (pKL->keydbBacking, &keyDef);
			if (PGPKeyDBObjRefIsValid (keyDef))
			{
				PGPGetKeyDBObjBooleanProperty (keyDef, 
						kPGPKeyProperty_IsSecret, &bSecret);
				PGPGetKeyDBObjBooleanProperty (keyDef, 
						kPGPKeyProperty_IsRevoked, &bRevoked);
				PGPGetKeyDBObjBooleanProperty (keyDef, 
						kPGPKeyProperty_IsExpired, &bExpired);
				PGPGetKeyDBObjBooleanProperty (keyDef, 
						kPGPKeyProperty_CanSign, &bCanSign);

				if (bSecret && bCanSign && !bRevoked && !bExpired) 
				{
					KLGetObjUserVal (pKL, keyDef, (long*)&(tli.hItem));
					if (tli.hItem) 
					{
						tli.state = TLIS_BOLD;
						tli.stateMask = TLIS_BOLD;
						tli.mask = TLIF_STATE;
						TreeList_SetItem (pKL->hwndTree, &tli);
					}
				}
			}
		}

		SetCursor (hCursorOld);
		if (bExpandNew) 
		{
			tli.hItem = KLFocusedItem (pKL);
			if (tli.hItem) 
				TreeList_EnsureVisible (pKL->hwndTree, &tli);
		}
	}

	return TRUE;
}


//	_______________________________________________
//
//  iterate entire keyset, deleting user value linked-list elements

BOOL 
KLDeleteAllUserValues (PKEYLIST pKL) 
{

	PGPKeyIterRef	keyiter;
	PGPKeyDBObjRef	key;
	PGPKeyDBObjRef	userid;
	PGPKeyDBObjRef	sig;

	if (!pKL) 
		return FALSE;
	if (!PGPKeySetRefIsValid (pKL->keysetDisp))
		return FALSE;

	PGPNewKeyIterFromKeySet (pKL->keysetDisp, &keyiter);
	PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);

	while (PGPKeyDBObjRefIsValid (key)) 
	{
		PGPKeyIterNextKeyDBObj (keyiter, 
				kPGPKeyDBObjType_UserID, &userid);
		while (PGPKeyDBObjRefIsValid (userid)) 
		{
			PGPKeyIterNextKeyDBObj (keyiter, 
					kPGPKeyDBObjType_Signature, &sig);
			while (PGPKeyDBObjRefIsValid (sig))
			{
				KLSetObjUserVal (pKL, sig, 0);
				PGPKeyIterNextKeyDBObj (keyiter, 
						kPGPKeyDBObjType_Signature, &sig);
			}
			KLSetObjUserVal (pKL, userid, 0);
			PGPKeyIterNextKeyDBObj (keyiter, 
					kPGPKeyDBObjType_UserID, &userid);
		}
		KLSetObjUserVal (pKL, key, 0);
		PGPKeyIterNextKeyDBObj (keyiter, 
				kPGPKeyDBObjType_Key, &key);
	}
	PGPFreeKeyIter (keyiter);

	return TRUE;
}


//	_______________________________________________
//
//  Insert single key into treelist

BOOL 
KLUpdateKeyInTree (
		PKEYLIST		pKL, 
		PGPKeyDBObjRef	key, 
		BOOL			bForceNew,
		BOOL			bSetFocus) 
{
	PGPKeyDBObjRef	keyDefault		= kInvalidPGPKeyDBObjRef;

	PGPKeyListRef	keylist;
	PGPKeyIterRef	keyiter;
	PGPKeyDBObjRef	keyPrev;
	PGPKeySetRef	keyset;
	TL_TREEITEM		tli;
	HTLITEM			hTPrevKey;
	HTLITEM			hTFocused;
	PGPBoolean		bSecret, bRevoked, bExpired, bCanSign;

	if (PGPKeySetRefIsValid (pKL->keysetDisp))
	{	
		hTFocused = KLFocusedItem (pKL);
		if (IsntPGPError (PGPclErrorBox (
				NULL, KLGetObjUserVal (pKL, key, (long*)&(tli.hItem))))) 
		{
			if (bForceNew && tli.hItem) 
			{
				TreeList_DeleteItem (pKL->hwndTree, &tli);
				KLSetObjUserVal (pKL, key, 0L);
			}

			PGPOrderKeySet (pKL->keysetDisp, 
					pKL->lKeyListSortField, pKL->bKeyListSortInvert, &keylist);
			PGPNewKeyIter (keylist, &keyiter);
			PGPKeyIterSeek (keyiter, key);
			PGPKeyIterPrevKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &keyPrev);
			PGPKeyIterSeek (keyiter, key);

			hTPrevKey = (HTLITEM)TLI_FIRST;
			if (PGPKeyDBObjRefIsValid (keyPrev))
			{
				if (IsntPGPError (PGPclErrorBox (NULL, KLGetObjUserVal (
						pKL, keyPrev, (long*)&(tli.hItem))))) 
				{
					hTPrevKey = tli.hItem;
				}
			}	

			if (IsntPGPError (PGPNewOneKeySet (key, &keyset)))
			{
				PGPCalculateTrust (keyset, pKL->keydbMain);
				PGPFreeKeySet (keyset);
			}

			sReloadKey (pKL, keyiter, 
					key, FALSE, bForceNew, FALSE, TRUE, hTPrevKey);

			PGPFreeKeyIter (keyiter);
			PGPFreeKeyList (keylist);

			// only set default key if this is the main keyset
			if (pKL->bMainKeySet) 
			{
				PGPclGetDefaultPrivateKey (pKL->keydbBacking, &keyDefault);
				if (keyDefault == key) 
				{
					PGPGetKeyDBObjBooleanProperty (key, 
							kPGPKeyProperty_IsSecret, &bSecret);
					PGPGetKeyDBObjBooleanProperty (key, 
							kPGPKeyProperty_IsRevoked, &bRevoked);
					PGPGetKeyDBObjBooleanProperty (key, 
							kPGPKeyProperty_IsExpired, &bExpired);
					PGPGetKeyDBObjBooleanProperty (key, 
							kPGPKeyProperty_CanSign, &bCanSign);

					if (bSecret && bCanSign && !bRevoked && !bExpired) 
					{
						KLGetObjUserVal (pKL, 
								keyDefault, (long*)&(tli.hItem));
						if (tli.hItem) 
						{
							tli.state = TLIS_BOLD;
							tli.stateMask = TLIS_BOLD;
							tli.mask = TLIF_STATE;
							TreeList_SetItem (pKL->hwndTree, &tli);
						}
					}
				}
			}

			// set selection appropriately
			if (bSetFocus)
			{
				if (bForceNew) 
					KLGetObjUserVal (pKL, key, (long*)&(tli.hItem));
				else
					tli.hItem = hTFocused;

				TreeList_Select (pKL->hwndTree, &tli, TRUE);
			}

			return TRUE;
		}
	}
	return FALSE;
}


//	_______________________________________________
//
//  Set validity for a treelist item

static HTLITEM 
sSetItemValidity (
		PKEYLIST	pKL, 
		HTLITEM		hItem, 
		INT			iColumn,
		INT			iValidity) 
{
	TL_LISTITEM tlL;

	tlL.hItem = hItem;

	tlL.iSubItem = iColumn;
	tlL.mask = TLIF_DATAVALUE;
	if (pKL->ulOptionFlags & kPGPclKeyList_GreenDotMode) 
	{
		if (iValidity > PGPCL_VALIDITY_COMPLETE)
			tlL.lDataValue = IDX_AXIOMATIC;
		else if (iValidity >= pKL->iValidityThreshold) 
			tlL.lDataValue = IDX_VALID;
		else 
			tlL.lDataValue = IDX_INVALID;
	}
	else 
		tlL.lDataValue = iValidity;

	hItem = (HTLITEM) TreeList_SetListItem (pKL->hwndTree, &tlL, FALSE);

	return (hItem);
}


//	_______________________________________________
//
//  update validity values for all keys/userids

BOOL 
KLUpdateAllValidities (PKEYLIST pKL) 
{
	HCURSOR			hCursorOld;
	HTLITEM			hTKey, hTUID;
	PGPKeyListRef	keylist;
	PGPKeyIterRef	keyiter;
	PGPKeyDBObjRef	key;
	PGPKeyDBObjRef	userid;
	PGPBoolean		bAxiomatic;
	UINT			u;
	INT				i, iColumn;

	// find validity column
	iColumn = -1;
	for (i=1; i<NUMBERFIELDS; i++) 
	{
		if (pKL->wColumnField[i] == KLI_VALIDITY) 
		{
			iColumn = i;
			break;
		}
	}

	if (iColumn == -1)
		return FALSE;

	if (PGPKeySetRefIsValid (pKL->keysetDisp)) 
	{
		hCursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
		PGPOrderKeySet (pKL->keysetDisp, 
				pKL->lKeyListSortField, pKL->bKeyListSortInvert, &keylist);
		PGPNewKeyIter (keylist, &keyiter);

		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);

		// iterate through keys
		while (key) 
		{
			KLGetObjUserVal (pKL, key, (long*)&hTKey);
			if (hTKey) 
			{
				// if axiomatic set trust and validity to out-of-range values
				//  in order to flag different graphical representation
				PGPGetKeyDBObjBooleanProperty (key, 
						kPGPKeyProperty_IsAxiomatic, &bAxiomatic);

				if (bAxiomatic) 
					u = CLConvertFromPGPValidity (kPGPValidity_Complete) + 1;
				else 
				{
					if (IsPGPError (PGPclErrorBox (NULL,
							PGPGetPrimaryUserIDValidity (key, &u)))) 
						u = 0;
					else 
						u = CLConvertFromPGPValidity (u);
				}
				sSetItemValidity (pKL, hTKey, iColumn, u);
			}

			// iterate through userids
			PGPKeyIterNextKeyDBObj (keyiter, 
					kPGPKeyDBObjType_UserID, &userid);
			while (userid) 
			{
				KLGetObjUserVal (pKL, userid, (long*)&hTUID);
				if (hTUID) 
				{
					PGPGetKeyDBObjNumericProperty (userid, 
							kPGPUserIDProperty_Validity, &u);
					u = CLConvertFromPGPValidity (u);
					sSetItemValidity (pKL, hTUID, iColumn, u);
				}
				PGPKeyIterNextKeyDBObj (keyiter, 
						kPGPKeyDBObjType_UserID, &userid);
			}
			PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);
		}
		PGPFreeKeyIter (keyiter);
		PGPFreeKeyList (keylist);
		SetCursor (hCursorOld);

		return TRUE;
	}

	// error on open key rings
	else 
		return FALSE;
}


//	_______________________________________________
//
//  Update a single object

PGPError PGPclExport
PGPclKeyListUpdateTree (
		HKEYLIST				hkeylist,
		PGPclKeyListUpdateOp	op,
		PGPKeyDBObjRef			obj,
		PGPBoolean				bRedraw)
{
	PKEYLIST		pKL			= (PKEYLIST)hkeylist;
	PGPBoolean		bSuccess	= FALSE;
	TL_TREEITEM		tli;

	if (!pKL) 
		return kPGPError_BadParams;

	switch (op) {
	case kPGPclKeyListUpdateObject :
		bSuccess = KLUpdateKeyInTree (pKL, obj, FALSE, FALSE);
		break;

	case kPGPclKeyListUpdateObjectFocus :
		bSuccess = KLUpdateKeyInTree (pKL, obj, FALSE, TRUE);
		break;

	case kPGPclKeyListReplaceObject :
		bSuccess = KLUpdateKeyInTree (pKL, obj, TRUE, TRUE);
		break;

	case kPGPclKeyListRemoveObject :
		KLGetObjUserVal (pKL, obj, (long*)&(tli.hItem));
		KLSetObjUserVal (pKL, obj, 0);
		TreeList_DeleteItem (pKL->hwndTree, &tli);
		bSuccess = TRUE;
		break;

	case kPGPclKeyListInsertObject :
		break;

	case kPGPclKeyListUpdateValidities :
		PGPCalculateTrust (pKL->keysetDisp, pKL->keydbBacking);
		KLUpdateAllValidities (pKL);
		break;
	}

	if (bSuccess)
	{
		if (bRedraw)
			InvalidateRect (pKL->hwndTree, NULL, FALSE);
		return kPGPError_NoErr;
	}
	else
		return kPGPError_ItemNotFound;
}


//	_______________________________________________
//
//  Expand a single item
//	routine called as a
//	callback function from the TreeList control to 
//	expand a single item.
//
//	lptli	= pointer to TreeList item to expand

static BOOL CALLBACK 
sExpandSingleItem (
		TL_TREEITEM*	lptli, 
		LPARAM			lParam) 
{
	EXPANDCOLLAPSESTRUCT* pecs = (EXPANDCOLLAPSESTRUCT*)lParam;
	TreeList_Expand (pecs->hwndTree, lptli, TLE_EXPANDALL);
	return TRUE;
}


//	_______________________________________________
//
//  Expand the selected items

static BOOL 
sExpandSelected (PKEYLIST pKL) 
{
	EXPANDCOLLAPSESTRUCT ecs;

	ecs.lpfnCallback = sExpandSingleItem;
	ecs.hwndTree = pKL->hwndTree;
	TreeList_IterateSelected (pKL->hwndTree, &ecs);
	InvalidateRect (pKL->hwndTree, NULL, TRUE);

	return TRUE;
}


//	_______________________________________________
//
//  Collapse a single item
//	routine called as a
//	callback function from the TreeList control to 
//	collapse a single item.
// 
//	lptli	= pointer to TreeList item to collapse

static BOOL CALLBACK 
sCollapseSingleItem (
		TL_TREEITEM*	lptli, 
		LPARAM			lParam) 
{
	EXPANDCOLLAPSESTRUCT* pecs = (EXPANDCOLLAPSESTRUCT*)lParam;
	TreeList_Expand (pecs->hwndTree, lptli, TLE_COLLAPSEALL);
	return TRUE;
}


//	_______________________________________________
//
//  Collapse the selected items

static BOOL 
sCollapseSelected (PKEYLIST pKL) 
{
	EXPANDCOLLAPSESTRUCT ecs;

	ecs.lpfnCallback = sCollapseSingleItem;
	ecs.hwndTree = pKL->hwndTree;
	TreeList_IterateSelected (pKL->hwndTree, &ecs);
	InvalidateRect (pKL->hwndTree, NULL, TRUE);

	return TRUE;
}


//	_______________________________________________
//
//  Cleanup treelist

PGPError PGPclExport 
PGPclKeyListDestroy (
		HKEYLIST	hKL,
		PGPBoolean	bSaveColumnInfo) 
{
	PKEYLIST pKL = (PKEYLIST)hKL;
	if (!hKL) 
		return kPGPError_BadParams;

	KLDeleteAllUserValues (pKL);

	if (bSaveColumnInfo) 
		KLSetColumnPreferences (pKL);

	SendMessage (pKL->hwndTree, WM_CLOSE, 0, 0);

	KLFree (pKL);

	return kPGPError_NoErr;
}


//	_______________________________________________
//
//  Load keyset into treelist

PGPError PGPclExport 
PGPclKeyListLoadKeys (
		HKEYLIST		hkeyman, 
		PGPKeyDBRef		keydbMain,
		PGPKeySetRef	keysetDisp)
{
	TL_TREEITEM		tli;
	PKEYLIST		pKL = (PKEYLIST)hkeyman;

	if (!pKL) 
		return kPGPError_BadParams;

	KLDeleteAllUserValues (pKL);
	KLSetFocus (pKL, NULL, FALSE);
	pKL->keysetDisp = kInvalidPGPKeySetRef;
	pKL->keydbMain = kInvalidPGPKeyDBRef;
	TreeList_DeleteTree (pKL->hwndTree, TRUE);

	KLGetColumnPreferences (pKL);
	KLAddColumns (pKL);

	if (!PGPKeyDBRefIsValid (keydbMain) &&
		!PGPKeySetRefIsValid (keysetDisp))
	{
		return kPGPError_NoErr;
	}

	pKL->keysetDisp = keysetDisp;

	if (PGPKeyDBRefIsValid (keydbMain))
		pKL->keydbMain = keydbMain;
	else
		pKL->keydbMain = PGPPeekKeySetKeyDB (keysetDisp);

	pKL->keydbBacking = PGPPeekKeySetKeyDB (keysetDisp);

	if (pKL->keysetDisp == PGPPeekKeyDBRootKeySet (pKL->keydbMain))
		pKL->bMainKeySet = TRUE;
	else
		pKL->bMainKeySet = FALSE;

	if (PGPKeySetRefIsValid (keysetDisp))
		KLLoadKeyRingIntoTree (pKL, FALSE, FALSE, TRUE);

	tli.hItem = TreeList_GetFirstItem (pKL->hwndTree);
	if (tli.hItem) 
	{
		KLSetFocus (pKL, TreeList_GetFirstItem (pKL->hwndTree), FALSE);
		TreeList_Select (pKL->hwndTree, &tli, FALSE);
	}

	InvalidateRect (pKL->hwndTree, NULL, TRUE);
	UpdateWindow (pKL->hwndTree);

	return kPGPError_NoErr;
}


//	_______________________________________________
//
//  Load keyset into treelist

PGPError PGPclExport 
PGPclKeyListReloadKeys (
		HKEYLIST	hkeylist, 
		PGPBoolean	bExpandNew) 
{
	PKEYLIST pKL = (PKEYLIST)hkeylist;
	if (!pKL) 
		return kPGPError_BadParams;
	if (!pKL->keysetDisp) 
		return kPGPError_BadParams;

	KLLoadKeyRingIntoTree (pKL, FALSE, bExpandNew, FALSE);

	InvalidateRect (pKL->hwndTree, NULL, TRUE);
	UpdateWindow (pKL->hwndTree);

	return kPGPError_NoErr;
}


//	_______________________________________________
//
//  Select specified key

static VOID 
sSelectKey (
		PKEYLIST		pKL, 
		PGPKeyDBObjRef	key, 
		BOOL			bDeselect) 
{
	TL_TREEITEM		tli;

	KLGetObjUserVal (pKL, key, (long*)&(tli.hItem));
	if (tli.hItem) 
		TreeList_Select (pKL->hwndTree, &tli, bDeselect);
}


//	___________________________________________________
//
//	select all keys

static VOID
sSelectAll (PKEYLIST pKL) 
{
	HTLITEM			hFirst;
	TL_TREEITEM		tlI;

	// get first item in list
	TreeList_SelectChildren (pKL->hwndTree, NULL);
	hFirst = TreeList_GetFirstItem (pKL->hwndTree);

	// try to get second item
	tlI.hItem = hFirst;
	if (tlI.hItem) 
	{
		tlI.mask = TLIF_NEXTHANDLE;
		TreeList_GetItem (pKL->hwndTree, &tlI);
	}

	// if second item exists, then multiple select
	if (tlI.hItem)
		KLSetFocus (pKL, hFirst, TRUE);
	else 
		KLSetFocus (pKL, hFirst, FALSE);

	SetFocus (pKL->hwndTree);
}


//	_______________________________________________
//
//  Select specified key

PGPError PGPclExport
PGPclKeyListSetTree (
		HKEYLIST	 			hkeylist,
		PGPclKeyListTreeOp	 	op,
		PGPKeyDBObjRef 			key)
{
	PKEYLIST		pKL = (PKEYLIST)hkeylist;

	switch (op) {
	case kPGPclSelectSpecified :
		sSelectKey (pKL, key, FALSE);
		break;

	case kPGPclSelectSpecifiedOnly :
		sSelectKey (pKL, key, TRUE);
		break;

	case kPGPclSelectAll :
		sSelectAll (pKL);
		break;

	case kPGPclDeselectAll :
		TreeList_Select (pKL->hwndTree, NULL, TRUE);
		KLSetFocus (pKL, NULL, FALSE);
		break;

	case kPGPclExpand :
		sExpandSelected (pKL);
		break;

	case kPGPclCollapse :
		sCollapseSelected (pKL);
		break;

	case kPGPclResort :
		TreeList_DeleteTree (pKL->hwndTree, FALSE);
		InvalidateRect (pKL->hwndTree, NULL, TRUE);
		UpdateWindow (pKL->hwndTree);
		KLLoadKeyRingIntoTree (pKL, TRUE, FALSE, FALSE);
		InvalidateRect (pKL->hwndTree, NULL, TRUE);
		break;
	}

	return kPGPError_NoErr;
}


//	_______________________________________________
//
//  Get a keyset of all selected keys

PGPError PGPclExport 
PGPclKeyListGetSelectedKeys (
		HKEYLIST		hkeylist, 
		PGPKeySetRef*	pkeysetSelected) 
{
	PKEYLIST			pKL = (PKEYLIST)hkeylist;

	return (KLGetSelectedKeys (pKL, pkeysetSelected, NULL));
}
