/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: PGPclientLib.c,v 1.60 2002/10/31 04:13:35 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPclx.h"
#include "TreeList.h"
#include "DurationControl.h"
#include "IPAddressControl.h"
#include "pgpImageList.h"

#include "pgpFileSpec.h"
#include "pgpRandomPool.h"

#include <zmouse.h>
#include <ole2.h>
#include <time.h>
#include <winsock2.h>
#include <shlwapi.h>

#define IMPORTDLGCLASS	("PGPDLGCLASS")


// DLL globals
UINT				g_uRefCount				= 0;
PGPContextRef		g_context				= kInvalidPGPContextRef;
HINSTANCE			g_hInst					= NULL;
HIMAGELIST			g_hilKeys				= NULL;
HIMAGELIST			g_hilClientLib			= NULL;
SHGetFolderPathProc g_fpSHGetFolderPath		= NULL;
UINT				g_uMouseWheelMessage	= 0;
BOOL				s_bWinsockInitialized	= FALSE;
BOOL				s_bLocalSDK				= FALSE;
BOOL				g_bUnicodeOS			= FALSE;

// prototype from pgpPassCach.h
// this function is only needed when using the SDK in local mode
void pgpPassphraseCacheRemoveClient ( PGPUInt32 id );


//	___________________________________________________
//
//	Register window class for selective import dialog
//	This allows setting the icon to the PGPkeys icon

static VOID
sRegisterSelectiveImportClass (VOID)
{
	WNDCLASSEX	wc;

	// class 0x8002 is the "hardwired" class of standard dialogs
	wc.cbSize = sizeof (WNDCLASSEX);
	GetClassInfoEx (NULL, MAKEINTRESOURCE(0x8002), &wc);

	wc.lpszClassName = IMPORTDLGCLASS;
	wc.hIcon = LoadIcon (g_hInst, MAKEINTRESOURCE(IDI_KEYICON));
	wc.hInstance = g_hInst;

	RegisterClassEx (&wc);
}


//	____________________________________________________________
//
//	explicit link to the SHFOLDER.DLL library

PGPError
CLLoadSHFolderGetPath (
		LPSTR	pszSHFolderPath)
{
	if (IsNull (g_fpSHGetFolderPath))
	{
		HMODULE		hmod;

		if (IsntNull (pszSHFolderPath))
			hmod = LoadLibrary (pszSHFolderPath);
		else
			hmod = LoadLibrary ("SHFOLDER.DLL");

		if (IsNull (hmod))
			return kPGPError_Win32_InvalidSHFolder;

		if (hmod)
		{
			g_fpSHGetFolderPath = (SHGetFolderPathProc)
					GetProcAddress (hmod, "SHGetFolderPathA");
			if (IsNull (g_fpSHGetFolderPath))
				return kPGPError_Win32_InvalidSHFolder;
		}
	}

	return kPGPError_NoErr;
}


//	____________________________________________________________
//
//	check if proper version of COMCTL32.DLL is installed

#define MIN_MAJORVERSION	4
#define MIN_MINORVERSION	70

static PGPError
sCheckComctl32Version (VOID)
{
	PGPError		err			= kPGPError_NoErr;
	HINSTANCE		hComCtl;

	hComCtl = LoadLibrary ("comctl32.dll");

	if (hComCtl)
	{
 		FARPROC	pDllGetVersion;

		pDllGetVersion = GetProcAddress (hComCtl, "DllGetVersion");

		if (pDllGetVersion)
		{
			DLLVERSIONINFO	dvi;
			LONG			hr;		// this should be an HRESULT

			ZeroMemory (&dvi, sizeof(dvi));
			dvi.cbSize = sizeof(dvi);

			hr = (LONG)(*pDllGetVersion)(&dvi);

			if (!(hr & 0x01)) 		// HRESULT returns error in bit 0
			{
				if (dvi.dwMajorVersion < MIN_MAJORVERSION)
					err = kPGPError_Win32_InvalidComctl32;
				else if (dvi.dwMajorVersion == MIN_MAJORVERSION)
				{
					if (dvi.dwMinorVersion < MIN_MINORVERSION)
						err = kPGPError_Win32_InvalidComctl32;
				}
			}
			else
				err = kPGPError_Win32_InvalidComctl32;
		}
		else
			err = kPGPError_Win32_InvalidComctl32;

		FreeLibrary(hComCtl);
	}
	else
		err = kPGPError_Win32_InvalidComctl32;


	return err;
}


//	____________________________________________________________
//
//	load appropriate imagelists for display

static VOID
sInitImageLists (VOID)
{
	HDC			hdc;
	HBITMAP		hbmp;
	INT			iNumBits;

	// create imagelist and load the appropriate bitmaps based on
	// current display capabilities.
	hdc = GetDC (NULL);		// DC for desktop
	iNumBits = GetDeviceCaps (hdc, BITSPIXEL) * GetDeviceCaps (hdc, PLANES);
	ReleaseDC (NULL, hdc);

	if (iNumBits <= 8)
	{
		g_hilClientLib = ImageList_Create (16, 16,
				ILC_COLOR|ILC_MASK, NUM_CLIENTLIB_ICONS, 0);
		hbmp = LoadBitmap (g_hInst, MAKEINTRESOURCE (IDB_IMAGES4));
		ImageList_AddMasked (g_hilClientLib, hbmp, RGB(255, 0, 255));
		DeleteObject (hbmp);

		g_hilKeys = ImageList_Create (16, 16,
				ILC_COLOR|ILC_MASK, NUM_IMAGELIST_ICONS, 0);
		hbmp = LoadBitmap (g_hInst, MAKEINTRESOURCE (IDB_KEYICONS4BIT));
		ImageList_AddMasked (g_hilKeys, hbmp, RGB(255, 0, 255));
		DeleteObject (hbmp);
	}
	else
	{
		g_hilClientLib = ImageList_Create (16, 16,
				ILC_COLOR24|ILC_MASK, NUM_CLIENTLIB_ICONS, 0);
		hbmp = LoadBitmap (g_hInst, MAKEINTRESOURCE (IDB_IMAGES24));
		ImageList_AddMasked (g_hilClientLib, hbmp, RGB(255, 0, 255));
		DeleteObject (hbmp);

		g_hilKeys = ImageList_Create (16, 16,
				ILC_COLOR24|ILC_MASK, NUM_IMAGELIST_ICONS, 0);
		hbmp = LoadBitmap (g_hInst, MAKEINTRESOURCE (IDB_KEYICONS24BIT));
		ImageList_AddMasked (g_hilKeys, hbmp, RGB(255, 0, 255));
		DeleteObject (hbmp);
	}
}


//	____________________________________________________________
//
//	check if current date is beyond supported range

static PGPError
sCheckCurrentDate (VOID)
{
	PGPError	err			= kPGPError_NoErr;
	time_t		tTimeNow;

	time (&tTimeNow);
	if (tTimeNow <= 0)
		err = kPGPError_Win32_InvalidCurrentDate;

	return err;
}

//	____________________________________________________________
//
//	Dialog procedure for driver warning dialog

static BOOL CALLBACK
sDriverWarnDlgProc (
		HWND	hDlg,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	switch(uMsg) {

	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND :
		switch(LOWORD (wParam)) {

		case IDOK :
			if (IsDlgButtonChecked (hDlg, IDC_NOMOREWARNINGS) == BST_CHECKED)
				EndDialog (hDlg, 2);
			else
				EndDialog (hDlg, 1);
			break;

		case IDCANCEL :
			EndDialog (hDlg, 0);
			break;
		}
		return TRUE;
	}
	return FALSE;
}


//________________________________________
//
//	code to start PGPsdk driver in NT
//		driver must already have been copied to
//		c:\winnt\system32\drivers directory and
//		appropriate registry entries created

#define DRIVERNAME				"PGPsdkDriver"
#define DRIVERFILENAME			"PGPsdk.sys"
#define DRIVER_NO_ERROR			0
#define DRIVER_NOT_WINNT		1
#define DRIVER_ACCESS_ERROR		2
#define DRIVER_CREATE_FAIL		3
#define DRIVER_ALREADY_STARTED	4
#define DRIVER_MISC_ERROR		5

static PGPError
sStartUtilDriver (VOID)
{
	PGPError			err			= kPGPError_UnknownError;
	SC_HANDLE			schSCMan	= NULL;
	SC_HANDLE			schServ		= NULL;
	DWORD				dwErr;
	BOOL				bRet;
	PGPclWindowsFamily	family;
	SERVICE_STATUS		ss;
	CHAR				szPath[MAX_PATH];

	// check if we're running under NT
	PGPclGetWindowsVersion (&family, NULL, NULL);
	if (family != kPGPclWindowsFamilyNT)
	{
		err = DRIVER_NOT_WINNT;
		goto done;
	}

	// open service control manager
	schSCMan = OpenSCManager (NULL, NULL, SC_MANAGER_CREATE_SERVICE);
	if (schSCMan == NULL)
	{
		dwErr = GetLastError();
		if (dwErr == ERROR_ACCESS_DENIED)
		{
			err = DRIVER_ACCESS_ERROR;
			goto done;
		}
		else
		{
			err = DRIVER_MISC_ERROR;
			goto done;
		}
	}
	// OK, success open of service control manager
	else {
		// try to open service
		schServ = OpenService (schSCMan, DRIVERNAME,
						SERVICE_START|SERVICE_QUERY_STATUS);

		if (schServ == NULL)
		{
			// couldn't open service
			dwErr = GetLastError ();
			if (dwErr != ERROR_SERVICE_DOES_NOT_EXIST)
			{
				err = DRIVER_MISC_ERROR;
				goto done;
			}

			// try to create new service ...
			GetSystemDirectory (szPath, sizeof(szPath));
			if (szPath[lstrlen (szPath) -1] != '\\')
				lstrcat (szPath, "\\");
			lstrcat (szPath, "drivers\\");
			lstrcat (szPath, DRIVERFILENAME);
			schServ = CreateService (schSCMan, DRIVERNAME, DRIVERNAME,
						SERVICE_ALL_ACCESS, SERVICE_KERNEL_DRIVER,
						SERVICE_AUTO_START, SERVICE_ERROR_NORMAL,
						szPath, NULL, NULL, NULL, NULL, NULL);

			if (schServ == NULL)
			{
				err = DRIVER_CREATE_FAIL;
				goto done;
			}
		}

		bRet = QueryServiceStatus (schServ, &ss);
		if (!bRet)
		{
			err = DRIVER_MISC_ERROR;
			goto done;
		}
		if (ss.dwCurrentState == SERVICE_STOPPED)
		{
			bRet = StartService (schServ, 0, NULL);
			if (!bRet) {
				dwErr = GetLastError ();
				err = DRIVER_MISC_ERROR;
				goto done;
			}
		}
		else
		{
			err = DRIVER_ALREADY_STARTED;
			goto done;
		}
	}
	err = DRIVER_NO_ERROR;

done :
	// cleanup service handle
	if (schServ)
		CloseServiceHandle (schServ);

	// clean up service control manager
	if (schSCMan)
		CloseServiceHandle (schSCMan);

	return err;
}


//	____________________________________________________________
//
//	check if page-locking driver is working by attempting to lock memory

static PGPError
sCheckMemoryLockingDriver (VOID)
{
	PGPError		err			= kPGPError_NoErr;
	PGPContextRef	context		= kInvalidPGPContextRef;
	PGPPrefRef		prefs		= kInvalidPGPPrefRef;
	PGPBoolean		bWarn		= TRUE;
	PGPFlags		flags		= 0;
	LPVOID			pmem		= NULL;
	INT				iret		= 0;

	err = PGPNewContext (kPGPsdkAPIVersion, &context); CKERR;

	pmem = PGPNewSecureData (PGPPeekContextMemoryMgr (context), 32, 0);
	if (!pmem)
	{
		err = kPGPError_OutOfMemory;
		goto done;
	}

	flags = PGPGetMemoryMgrDataInfo (pmem);
	if (!(flags & kPGPMemoryMgrBlockInfo_NonPageable))
	{
		// memory is pageable, thus the driver is not working
		err = PGPclPeekClientLibPrefRefs (&prefs, NULL); CKERR;
		PGPGetPrefBoolean (prefs, kPGPPrefWarnOnMemLockFailure, &bWarn);

		if (bWarn)
		{
			iret = DialogBox (g_hInst, MAKEINTRESOURCE(IDD_DRIVERWARNING),
							NULL, sDriverWarnDlgProc);
			switch (iret) {
			case 0 :
				err = kPGPError_UserAbort;
				break;

			case 2 :
				PGPSetPrefBoolean (prefs, kPGPPrefWarnOnMemLockFailure, FALSE);
				PGPclFlushClientLibPrefs (prefs, NULL);
			case 1 :
				err = kPGPError_NoErr;
				break;
			}
		}
	}

done :
	if (pmem)
		PGPFreeData (pmem);
	if (PGPContextRefIsValid (context))
		PGPFreeContext (context);

	return err;
}

//	____________________________________________________________
//
//	initialize common controls

static VOID
sInitControls (VOID)
{
	INITCOMMONCONTROLSEX	icex;
	HINSTANCE				hComCtl;


	// Ensure that the common control DLL is loaded.
	InitCommonControls ();

	// Ensure that the custom controls are initialized.
	InitTreeListControl ();
	InitPGPIPAddressControl ();
	InitPGPDurationControl ();

	hComCtl = LoadLibrary ("comctl32.dll");
	if (hComCtl)
	{
 		FARPROC	pInitCommonControlsEx;

		pInitCommonControlsEx =
			GetProcAddress (hComCtl, "InitCommonControlsEx");

		if (pInitCommonControlsEx) {

			// Ensure that the extended common controls are loaded
			icex.dwSize = sizeof(icex);
			icex.dwICC = ICC_BAR_CLASSES|ICC_DATE_CLASSES;
			(*pInitCommonControlsEx)(&icex);
		}

		FreeLibrary(hComCtl);
	}
}


#if PGP_DEBUG
//	____________________________________________________________
//
//	determine how if to run SDK backend or not

static BOOL
sUseLocalExecution (
		BOOL	bSDK)
{
	PGPBoolean	bLocal			= FALSE;
	HKEY		hkey;
	LONG		lResult;
	DWORD		dw, dwValueType, dwSize;
	CHAR		szKey[256];

	PGPclGetPath (kPGPclRegistryKey, szKey, sizeof(szKey));
	lResult = RegOpenKeyEx (
			HKEY_LOCAL_MACHINE, szKey, 0, KEY_READ, &hkey);

	if (lResult == ERROR_SUCCESS) 
	{
		dwSize = sizeof(DWORD);

		if (bSDK)
		{
			lResult = RegQueryValueEx (hkey, "PGPclForceLocalSDK", 
					0, &dwValueType, (LPBYTE)&dw, &dwSize);
		}
		else
		{
			lResult = RegQueryValueEx (hkey, "PGPclForceLocalPrefs", 
					0, &dwValueType, (LPBYTE)&dw, &dwSize);
		}

		if (lResult == ERROR_SUCCESS)
		{
			if (dw == 1)
				bLocal = TRUE;
		}

		RegCloseKey (hkey);
	}

	return bLocal;
}
#endif //PGP_DEBUG

//	____________________________________________________________
//
//	check if user account needs initialization and do it if enabled

static PGPError
sHandleNewUser (
		PGPUInt32		uFlags)
{
	PGPPrefRef		prefref			= kInvalidPGPPrefRef;
	PGPError		err				= kPGPError_NoErr;
	PGPBoolean		bKeyGenerated	= TRUE;

	PGPUInt32		uStatus;

	if (uFlags & kPGPclNoPrefsFileInit)
		return kPGPError_NoErr;

	if (!PGPclIsComponentInstalled (kPGPclPGPkeys))
		return kPGPError_NoErr;

	err = PGPclPeekClientLibPrefRefs (&prefref, NULL);
	if (IsPGPError (err))
		return err;

	err = PGPGetPrefNumber (prefref, 
			kPGPPrefUserAccountStatus, &uStatus);
	if (IsPGPError (err))
		return err;

	if (uStatus == kPGPPrefPGPInitialized)
	{
		if (uFlags & kPGPclEnableNewUserWizard)
		{
			err = PGPGetPrefBoolean (prefref, 
					kPGPPrefFirstKeyGenerated, &bKeyGenerated);

			if (!bKeyGenerated)
			{
				PGPclRebootKeyGeneration (NULL, TRUE);
			}
		}

		return kPGPError_NoErr;
	}

	if (!(uFlags & kPGPclEnableNewUserWizard))
		return kPGPError_Win32_UserNotInitialized;

	if (uStatus == kPGPPrefPGPDisabled)
	{
		if (!(uFlags & kPGPclForceNewUserWizard))
			return kPGPError_Win32_UserNotInitialized;
	}

	err = CLHandleNewUser (g_context, NULL);

	return err;
}


//	____________________________________________________________
//
//	initialize DLL

PGPError PGPclExport
PGPclInitLibrary (
		PGPContextRef*	pcontext,
		PGPUInt32		uFlags)
{
	PGPError		err				= kPGPError_NoErr;
	PFLFileSpecRef	filespec		= kInvalidPFLFileSpecRef;
	PGPUInt32		uLocalMode		= 0;
	BOOL			bUseLocalPrefs	= FALSE;

	CHAR			szPath[MAX_PATH];
	WSADATA			wsa;
	OSVERSIONINFO	oviOsVerInfo={sizeof(OSVERSIONINFO), 0};

	if (IsntNull (pcontext))
		*pcontext = kInvalidPGPContextRef;

	if (g_uRefCount++ == 0)
	{
		err = sCheckCurrentDate ();
		if (IsPGPError (err))
			return err;

		err = sCheckComctl32Version ();
		if (IsPGPError (err))
			return err;

		err = CLLoadSHFolderGetPath (NULL);
		if (IsPGPError (err))
			return err;

		// initialize Winsock 1.1, if present
		s_bWinsockInitialized = (WSAStartup (MAKEWORD (1,1), &wsa) == 0);

		// attempt to startup the utility driver, in case it is stopped
		sStartUtilDriver ();

		// load the richedit 2.0 control
		LoadLibrary ("RichEd20.dll");

		// Initialize the sdk library
		if (uFlags & kPGPclUseLocalSDKMode)
			uLocalMode = kPGPFlags_ForceLocalExecution;
#if PGP_DEBUG
		if (sUseLocalExecution (TRUE))
			uLocalMode |= kPGPFlags_ForceLocalExecution;
#endif
		s_bLocalSDK = uLocalMode & kPGPFlags_ForceLocalExecution;

		err = PGPsdkInit (uLocalMode);
		if (IsPGPError (err))
			return err;

		err = PGPsdkNetworkLibInit (0);
		if (IsPGPError (err))
		{
			PGPsdkCleanup ();
			return err;
		}

		err = PGPsdkUILibInit (0);
		if (IsPGPError (err))
		{
			PGPsdkNetworkLibCleanup ();
			PGPsdkCleanup ();
			return err;
		}

		// create a new sdk context
		err = PGPNewContext (kPGPsdkAPIVersion, &g_context);
		if (IsPGPError (err))
			return err;

#if PGP_DEBUG
		bUseLocalPrefs = sUseLocalExecution (TRUE);
#endif
		err = CLInitPrefs (bUseLocalPrefs, 
				!(uFlags & kPGPclNoPrefsFileInit));

		if(IsntPGPError(err))
		{
			PGPlnLicenseCheck(g_context);
		}
	
		if (IsPGPError (err))
			return err;

		err = sCheckMemoryLockingDriver ();
		if (IsPGPError (err))
			return err;

		// initialize the RNG seed file
		if (!(uFlags & kPGPclNoPrefsFileInit))
		{
			err = CLInitRNGSeedFilePref (
					kInvalidPGPPrefRef, szPath, sizeof(szPath));
			if (IsPGPError (err))
				return err;

			err = PFLNewFileSpecFromFullPath (
					PGPPeekContextMemoryMgr (g_context), szPath, &filespec);
			if (IsntPGPError (err))
				err = PGPSetRandSeedFile (filespec);
			if (PFLFileSpecRefIsValid (filespec))
				PFLFreeFileSpec (filespec);
			if (IsPGPError (err))
				return err;
		}

		//  initialize OLE for drag and drop functionality
		OleInitialize (NULL);

		// register the mouse wheel message
		g_uMouseWheelMessage = RegisterWindowMessage (MSH_MOUSEWHEEL);

		sInitControls ();
		sInitImageLists ();

		// see if user has initialized keyrings, etc.
		err = sHandleNewUser (uFlags);
		if (IsPGPError (err))
			return err;

		//check if the os supports unicode thru and thru (=NT onwards)
		GetVersionEx (&oviOsVerInfo);   
		g_bUnicodeOS = (VER_PLATFORM_WIN32_NT == oviOsVerInfo.dwPlatformId);
	}

	if (IsntNull (pcontext))
		*pcontext = g_context;

	return err;
}


//	____________________________________________________________
//
//	close down DLL

PGPError PGPclExport
PGPclCloseLibrary (void)
{
	if ((g_uRefCount > 0) && (--g_uRefCount == 0))
	{
		PGPlnLicenseFree (g_context);

		if (g_hilKeys)
		{
			ImageList_Destroy (g_hilKeys);
			g_hilKeys = NULL;
		}

		if (g_hilClientLib)
		{
			ImageList_Destroy (g_hilClientLib);
			g_hilClientLib = NULL;
		}

		CLClosePrefs ();
		OleUninitialize ();

		if (PGPContextRefIsValid (g_context))
		{
			if (s_bLocalSDK)
			{
				PGPPurgePassphraseCache (g_context);
				pgpPassphraseCacheRemoveClient (0);
				PGPPurgeKeyDBCache (g_context); 
			}

			PGPFreeContext (g_context);
			g_context = kInvalidPGPContextRef;
		}

		PGPsdkUILibCleanup ();
		PGPsdkNetworkLibCleanup ();
		PGPsdkCleanup ();

		if (s_bWinsockInitialized)
			WSACleanup ();
	}

	return kPGPError_NoErr;
}


//	____________________
//
//	DLL entry/exit point

BOOL WINAPI
DllMain (
		 HINSTANCE hInstDll,
		 DWORD dwReason,
		 LPVOID lpvReserved)
{
	switch (dwReason) {
	case DLL_PROCESS_ATTACH:
		g_uRefCount = 0;
		g_hInst = hInstDll;
		sRegisterSelectiveImportClass ();
		break;
    case DLL_THREAD_ATTACH:
        break;
    case DLL_THREAD_DETACH:
		break;
    case DLL_PROCESS_DETACH:
		break;
    }
    return TRUE;
}


