/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	PGPclx.h - internal header for PGPcl project

	$Id: PGPclx.h,v 1.37 2002/10/31 04:13:35 pbj Exp $
____________________________________________________________________________*/
#ifndef Included_PGPclx_h	/* [ */
#define Included_PGPclx_h

#define WIN32_LEAN_AND_MEAN
#define _WIN32_WINNT 0x0400

// system header files
#include <windows.h>
#include <commctrl.h>
#include <stddef.h>

// PGP build flags
#include "pgpBuild.h"

// PGP SDK header files
#include "pgpUserInterface.h"
#include "pgpBase.h"
#include "pgpMem.h"
#include "pgpKeys.h"
#include "pgpErrors.h"
#include "pgpUtilities.h"
#include "pgpClientPrefs.h"

// PGP client header files
#include "PGPHelp.h"
#include "pgpWin32Errors.h"
#include "pgpClientLib.h"
#include "pgpLnLib.h"

// local header files
#include "resource.h"

// macro definitions
#define CKERR		if (IsPGPError (err)) goto done

// common definitions
#define IDC_KSTREELIST				5555
#define IDC_ADKTREELIST				5556
#define IDC_REVOKERTREELIST			5557

#define PGPCL_VALIDITY_INVALID		0
#define PGPCL_VALIDITY_MARGINAL		1
#define PGPCL_VALIDITY_COMPLETE		2

#define CLIENTLIB_ICON_WIDTH		16

#define IDX_NONE					-1
#define IDX_GREYDOT					0
#define IDX_GREYDOTPUSHED			1
#define IDX_GREENDOT				2
#define IDX_GREENDOTPUSHED			3
#define IDX_WORLDDOMAIN				4
#define IDX_SERVER					5
#define IDX_ALERT					6
#define IDX_ARROW					7
#define IDX_CHECK					8
#define IDX_ERROR					9
#define IDX_REDDOT					10
#define IDX_CHECKEDBOX				11
#define IDX_UNCHECKEDBOX			12
#define IDX_RULEIN					13
#define IDX_RULEOUT					14
#define IDX_RULEEITHER				15
#define IDX_ALARM					16
#define NUM_CLIENTLIB_ICONS			17

// typedefs
typedef struct {
	HWND				hwnd;
	PGPKeyDBRef			keydbMain;
	PGPKeyServerEntry*	pentry;
} CLIENTSERVERSTRUCT, *PCLIENTSERVERSTRUCT;

typedef HRESULT (*SHGetFolderPathProc)(HWND, int, HANDLE, DWORD, LPTSTR);


// prototypes
// PGPclientLib.c
PGPError CLLoadSHFolderGetPath (LPSTR pszSHFolderPath);

// CLconvert.c
VOID CLTimeToSystemTime (struct tm*	ptm, SYSTEMTIME* pst);
VOID CLConvertTimeToDays (PGPTime tm, INT* piDays);
VOID CLConvertTimeToString (PGPTime tm, LPSTR sz, INT ilen);
UINT CLConvertFromPGPTrust (UINT uPGPTrust);
UINT CLConvertToPGPTrust (UINT uTrust);
UINT CLConvertFromPGPValidity (UINT uPGPValidity);

// CLexpire.c
PGPError CLGetEvalRegistryValues (
		BOOL* pbEvaluation, PGPTime* ptimeInstall,
		PGPUInt32* puNoNagDays, PGPUInt32* puNagDays);

// CLgenwiz.c
HFONT CLLoadHeadingFont (BOOL bInterior);

// CLIExplorer.cpp
int CLIEDiffKeyContainers (const char *conts1, const char *conts2, char **out); 
int CLIEGenerateNewUUID (char **cont);

// CLImport.c
BOOL CLCheckForSecretKeys (PGPKeySetRef keyset);

// CLMisc.c
VOID* clAlloc (UINT uBytes);
VOID clFree (VOID* p);
VOID CLPaintUserInfo (
		PGPMemoryMgrRef	memMgr,
		HWND			hwnd,
		HDC				hDC,
		LPSTR			pszVersion);
LRESULT PGPclMessageBox (
		HWND			hWnd,
		INT				iCaption,
		INT				iMessage,
		ULONG			ulFlags);
HBITMAP CLLoadResourceBitmap (
		HINSTANCE		hInstance,
		LPSTR			lpString,
		HPALETTE*		lphPalette);
BOOL CLAddKeysToMain (
		PGPContextRef	context,
		HWND			hwnd,
		PGPKeySetRef	keysetToAdd,
		PGPKeyDBRef		keydbMain);
VOID CLGetKeyBitsString (
		PGPKeyDBObjRef	key, 
		LPSTR			sz,
		UINT			u);
PGPError CLInitRNGSeedFilePref (
		PGPPrefRef		prefrefIn,
		LPSTR			pszPath, 
		PGPInt32		iLen);
PGPError CLMergeInAdminPrefs (
		PGPContextRef	context,
		PGPPrefRef		prefref, 
		BOOL			bNetPrefs);
PGPError CLSetTokenDll (
		PGPPrefRef	prefref,
		LPSTR		pszDllName,
		BOOL		bTestOnly);
PGPInt32 CLDetermineSubkeyIcon (
		PGPKeyDBObjRef	subkey);

// CLnewuser.c
PGPError
CLHandleNewUser (
		PGPContextRef		context,
		HWND				hwndParent);


// CLPrefs.c
PGPError CLInitPrefs (
		BOOL bLocalMode,
		BOOL bCreatePGPPrefsFile);
PGPError CLClosePrefs (VOID);
BOOL CLCreatePath (LPSTR pszPath);

// CLprefsCA.c
PGPError CLGetCAServerPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PGPKeyServerEntry**	ppentry);
PGPError CLSetCAServerPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PGPKeyServerEntry*	pentry);
PGPError CLSetCARevocationServerPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PGPKeyServerEntry*	pentry);

// CLsdksrv.c
PGPError CLHandleTLSEvent (
		PGPContextRef		context,
		PGPEvent*			pevent,
		HWND				hwnd,
		PGPKeyDBRef			keydb,
		PGPKeyServerEntry*	pentry,
		PGPclAuthType		authType,
		BOOL*				pbAllow);

// CLsrvprf.c
PGPError CLInitKeyServerPrefs(
		PGPUInt32			uServer,
		PGPKeyServerEntry*	pkeyserver,
		HWND				hwndParent,
		PGPContextRef		context,
		PGPKeyDBRef			keydbMain,
		CHAR*				pszUserID,
		PCLIENTSERVERSTRUCT	*ppcss,
		PGPKeyServerEntry	**ksEntries,
		PGPKeyServerSpec	**serverList,
		PGPUInt32*			numKSEntries);

PGPError CLUninitKeyServerPrefs(
		PGPUInt32			uServer,
		PCLIENTSERVERSTRUCT	pcss,
		PGPKeyServerEntry	*ksEntries,
		PGPKeyServerSpec	*serverList,
		PGPUInt32			numKSEntries);

#endif /* ] Included_PGPclx_h */


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
