/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CDiskUsersView.cpp,v 1.12 2002/09/24 19:58:28 wjb Exp $
____________________________________________________________________________*/

#include "pgpClassesConfig.h"
#include <stdio.h>

#include "UString.h"

#include "CBitmap.h"
#include "CBrush.h"
#include "CDC.h"
#include "CFont.h"
#include "CIcon.h"
#include "CImageList.h"
#include "CMenu.h"
#include "CMessageBox.h"
#include "CPen.h"
#include "CRect.h"
#include "UModuleInstance.h"

#include "UCommonStrings.h"
#include "CPGPdiskEngine.h"
#include "CPGPKey.h"
#include "CPGPKeyDB.h"

#include "CDiskUsersView.h"
#include "CPGPKeySearcher.h"
#include "ImageIDs.h"

#include "pgpUnicode.h"

#include "Resource.h"

_USING_PGP

_UNNAMED_BEGIN

// Constants

const float	kUserNameColRatio	= 0.48F;
const float	kReadOnlyColRatio	= 0.14F;
const float	kKindColRatio		= 0.36F;

_UNNAMED_END

// Types

struct CDiskUsersView::DiskUserItem : 
	public CListableObject<CDiskUsersView::DiskUserItem>
{
	CPGPdiskUser	user;

	CString		userName;
	CString		kindText;
	PGPBoolean	isPubAndNotInRing;

	PGPUInt32	icon;
};

struct CDiskUsersView::DKViewDrawElements
{
	COLORREF	barColor;
	CBrush		stdBarBrush;
	CBrush		spcBarBrush;
	CPen		gSelTextPen;
	CPen		gUnselTextPen;
	CPen		hilightPen;
	CPen		shadowPen;
	CPen		buttonPen;

	CBrush		barBGBrush;
	CBrush		highBrush;
	CBrush		backBrush;
	CFont		hFont;
	CFont		hBold;
	CFont		hItalic;
	CFont		hStrikeOut;
	CFont		hItalicStrikeOut;
	CFont		hBoldStrikeOut;

	CImageList	image;
};


// Class CDiskUsersView member functions

CDiskUsersView::CDiskUsersView() : 

	mPContext(NULL), mPDiskUsers(NULL), mPDE(NULL), mUserNameColIndex(0), 
		mReadOnlyColIndex(0), mKindColIndex(0)
{
}

PGPBoolean 
CDiskUsersView::AreUnknownUsers() const
{
	DiskUserItem	*pItem	= mItemList.Head();

	while (IsntNull(pItem))
	{
		if (pItem->isPubAndNotInRing)
			return TRUE;

		pItem = mItemList.Next(pItem);
	}

	return FALSE;
}

void 
CDiskUsersView::SelectIndex(PGPInt32 i)
{
	SetItemState(i, LVIS_FOCUSED | LVIS_SELECTED, 
		LVIS_FOCUSED | LVIS_SELECTED);
	RedrawItems(i, i);
}

void 
CDiskUsersView::SelectAllItems()
{
	PGPInt32	count	= GetItemCount();

	for (PGPInt32 i = 0; i < count; i++)
	{
		if (i == 0)
		{
			SetItemState(i, LVIS_FOCUSED | LVIS_SELECTED, 
				LVIS_FOCUSED | LVIS_SELECTED);
		}
		else
		{
			SetItemState(i, LVIS_SELECTED, LVIS_SELECTED);
		}
	}

	RedrawItems(0, count);
}

void 
CDiskUsersView::GetSelectedUsers(CPGPdiskUserSet *pUsers) const
{
	PGPUInt32	itemCount	= GetItemCount();

	for (PGPUInt32 i = 0; i < itemCount; i++)
	{
		if (GetItemState(i, LVIS_SELECTED))
			pUsers->Add(GetUserAtIndex(i));
	}
}

void 
CDiskUsersView::SetActiveColumns(PGPUInt32 columns)
{
	mActiveColumns = columns;

	UpdateColumns();
	InvalidateRect(NULL, TRUE);
}

void 
CDiskUsersView::SortOnColumn(PGPUInt32 column)
{
	if (mLastSortColumn == column)
		mSortAscending = !mSortAscending;
	else
		mSortAscending = TRUE;

	mLastSortColumn = column;

	SortItems(DiskUserViewCompareProc, reinterpret_cast<LPARAM>(this));
}

void 
CDiskUsersView::LookupUnknownKeys()
{
	CList<CPGPKeySearcher::SearchItem>	searchItems;

	try
	{
		// Look for users for which we have only the key ID.
		DiskUserItem	*pItem	= mItemList.Head();

		while (IsntNull(pItem))
		{
			if (pItem->isPubAndNotInRing)
			{
				auto_ptr<CPGPKeySearcher::SearchItem>	pSearchItem(
					new CPGPKeySearcher::SearchItem);

				pItem->user.GetPGPKeyID(pSearchItem->keyID);
				pSearchItem->userData = pItem;

				searchItems.AddTail(pSearchItem.release());
			}

			pItem = mItemList.Next(pItem);
		}

		// Look up the keys.
		if (searchItems.Count() > 0)
		{
			CPGPKeyDB	keyDB(mPContext->PGPKeyDB());
			PGPBoolean	wasAKeyNotFound	= FALSE;

			CPGPKeySearcher::SearchKeys(*mPContext, GetParent(), 
				searchItems, keyDB);
			CPGPKeySearcher::SearchItem	*pSearchItem	= searchItems.Head();

			while (IsntNull(pSearchItem))
			{
				// Update found keys in the UI.
				if (pSearchItem->wasFound)
				{
					pItem = static_cast<DiskUserItem *>(
						pSearchItem->userData);
					CalcItemDisplayData(pItem, TRUE);
				}
				else
				{
					wasAKeyNotFound = TRUE;
				}

				pSearchItem = searchItems.Next(pSearchItem);
			}

			// warn if at least one key not found
			if (wasAKeyNotFound)
			{
				using namespace UCommonStrings;

				CMessageBox	messageBox;
				messageBox.Display(Get(kTellUnknownKeyNotFound), 
					Get(kEditorMsgBoxTitle), Handle(), CMessageBox::kWarningTone);
			}
		}

		searchItems.EmptyWithDelete();
	}
	catch (CComboError& error)
	{
		using namespace UCommonStrings;

		ReportError(kTellSearchForKeysFailed, kDiskUserViewMsgBoxTitle, 
			error, Handle());

		searchItems.EmptyWithDelete();
		throw;
	}
}

void 
CDiskUsersView::Fill(
	const CPGPdiskContext	*pContext, 
	const CPGPdiskUserSet&	diskUsers, 
	PGPBoolean				singleSelect)
{
	pgpAssert(WeSubclassed());

	Clear();
	CreateDrawElements();

	mPDiskUsers = new CPGPdiskUserSet;

	if (IsNull(mPContext))
	{
		// First time.
		mPContext = pContext;

		mSingleSelect = singleSelect;
		mSortAscending = FALSE;
		mLastSortColumn = 0;

		if (singleSelect)
			SetStyle(GetStyle() | LVS_SINGLESEL);
		else
			SetStyle(GetStyle() & ~LVS_SINGLESEL);

		SetActiveColumns(kUserNameColFlag | kReadOnlyColFlag | kKindColFlag);
		GenerateMeasureItem();
	}

	// Update internal key list.
	CPGPdiskUserIter	curUsersIter(*mPDiskUsers), newUsersIter(diskUsers);
	CPGPdiskUser		curUser, newUser;

	while (curUsersIter.Next(curUser))
	{
		if (!newUsersIter.FindEqualUser(curUser, newUser))
			RemoveUser(curUser);
	}

	newUsersIter.Rewind();

	while (newUsersIter.Next(newUser))
	{
		if (curUsersIter.FindEqualUser(newUser, curUser))
			UpdateUser(newUser, TRUE);
		else
			AddUser(newUser);
	}
}

void 
CDiskUsersView::Clear()
{
	DeleteAllItems();
	mItemList.EmptyWithDelete();

	if (IsntNull(mPDiskUsers))
	{
		delete mPDiskUsers;
		mPDiskUsers = NULL;
	}

	if (IsntNull(mPDE))
		DeleteDrawElements();
}

void 
CDiskUsersView::Create(
	HWND		parent, 
	PGPUInt32	id, 
	PGPInt32	xOffset, 
	PGPInt32	yOffset, 
	PGPInt32	width, 
	PGPInt32	height)
{
	CListView::Create(parent, id, WS_TABSTOP | LVS_REPORT | 
		LVS_OWNERDRAWFIXED, WS_EX_CLIENTEDGE | WS_EX_NOPARENTNOTIFY, 
		xOffset, yOffset, width, height);
}

int 
CDiskUsersView::DiskUserViewCompareProcAux(
	const DiskUserItem	*pKey1, 
	const DiskUserItem	*pKey2)
{
	pgpAssertAddrValid(pKey1, DiskUserItem);
	pgpAssertAddrValid(pKey2, DiskUserItem);

	if (mLastSortColumn == mUserNameColIndex)
	{
		if (mSortAscending)
			return strcmp(pKey1->userName, pKey2->userName);
		else
			return strcmp(pKey2->userName, pKey1->userName);
	}
	else if (mLastSortColumn == mReadOnlyColIndex)
	{
		if (mSortAscending)
			return (!pKey1->user.IsReadOnly() && pKey2->user.IsReadOnly());
		else
			return (!pKey2->user.IsReadOnly() && pKey1->user.IsReadOnly());
	}
	else if (mLastSortColumn == mKindColIndex)
	{
		if (mSortAscending)
			return strcmp(pKey1->kindText, pKey2->kindText);
		else
			return strcmp(pKey2->kindText, pKey1->kindText);
	}

	return 0;
}

int 
CALLBACK 
CDiskUsersView::DiskUserViewCompareProc(
	LPARAM	lParam1, 
	LPARAM	lParam2, 
	LPARAM	lParamSort)
{
	CDiskUsersView	*pView	= reinterpret_cast<CDiskUsersView *>(lParamSort);
	pgpAssertAddrValid(pView, CDiskUsersView);

	return pView->DiskUserViewCompareProcAux(
		reinterpret_cast<DiskUserItem *>(lParam1), 
		reinterpret_cast<DiskUserItem *>(lParam2));
}

void 
CDiskUsersView::DrawTrueFalseButton(
	CDC&			itemDC, 
	const CRect&	ptrBarRect, 
	PGPBoolean		trueFalse, 
	PGPBoolean		isSelected) const
{
	if (isSelected)
		itemDC.FillRect(ptrBarRect, mPDE->highBrush);
	else
		itemDC.FillRect(ptrBarRect, mPDE->backBrush);

	if (ptrBarRect.Width() >= CIcon::kSmallIconWidth)
	{
		PGPUInt32	noviceIcon;

		if (trueFalse)
		{
			noviceIcon = ImageIDs::kADKPushed;

			mPDE->image.Draw(noviceIcon, itemDC, 
				ptrBarRect.Left() + 
				(ptrBarRect.Width() - CIcon::kSmallIconWidth)/2, 
				ptrBarRect.Top() + 
				(ptrBarRect.Height() - CIcon::kSmallIconWidth)/2, 
				ILD_TRANSPARENT);
		}
	}
}


//	_______________________________________________
//
//  Truncate text string

static BOOL sTruncateText (
		HDC		hdc, 
		WCHAR*	wsz, 
		int		xmax) 
{
	BOOL bShowToolTip	= FALSE;
	SIZE s;
	int l, w;

	// truncate at <newline>, if present
	l = 0;
	while (wsz[l])
	{
		if (wsz[l] == '\n')
		{
			wsz[l] = 0;
			bShowToolTip = TRUE;
			break;
		}
		l++;
	}

	GetTextExtentPoint32W (hdc, wsz, lstrlenW (wsz), &s);
	w = s.cx + 4;

	l = lstrlenW (wsz);
	if (l < 3) 
	{
		if (w > xmax) 
		{
			bShowToolTip = TRUE;
			wsz[0] = 0;
		}
	}
	else 
	{
		l = lstrlenW (wsz) - 3;

		while ((w > xmax) && (l >= 0))
		{
			bShowToolTip = TRUE;
			lstrcpyW (&wsz[l], L"...");
			GetTextExtentPoint32W (hdc, wsz, lstrlenW (wsz), &s);
			w = s.cx + 4;
			l--;
		}

		if (l < 0) 
			wsz[0] = 0;
	}

	return bShowToolTip;
}

#define MAXUSERIDSTRING				kPGPMaxUserIDSize + 32

void 
CDiskUsersView::DrawItemColumn(
	CDC&			itemDC, 
	const char		*userID, 
	const CRect&	prcClip) const
{
	WCHAR wsz[MAXUSERIDSTRING];
	PGPUInt32 ulen;

//	CString	cuserID(userID);
//	itemDC.FitStringToWidth(cuserID, prcClip.Width());

//	itemDC.ExtTextOut(prcClip.Left() + 2, prcClip.Top() + 1, 
//		ETO_CLIPPED | ETO_OPAQUE, prcClip, cuserID, cuserID.Length(), NULL);

	pgpUTF8StringToUCS2 (userID, kPGPUnicodeNullTerminated, 
			wsz, MAXUSERIDSTRING, &ulen);

	sTruncateText (itemDC.DC(), wsz, prcClip.Right() - prcClip.Left());

    // print the text
    ExtTextOutW(itemDC.DC(), prcClip.Left() + 2, prcClip.Top() + 1, 
               ETO_CLIPPED | ETO_OPAQUE,
               prcClip, 
			   wsz, 
			   lstrlenW(wsz), 
			   NULL);
}

void 
CDiskUsersView::DrawUserItemAux(
	LPDRAWITEMSTRUCT	pDIS, 
	PGPBoolean			isFocused) const
{
	DiskUserItem	*pItem	= reinterpret_cast<DiskUserItem *>(
		pDIS->itemData);
	pgpAssertAddrValid(pItem, DiskUserItem);

	CDC		itemDC(pDIS->hDC);
	CRect	itemRect	= pDIS->rcItem;

	mPDE->image.Draw(pItem->icon, itemDC, kPixelsLeftMargin, itemRect.Top(), 
		ILD_TRANSPARENT);

	itemRect.Left() = CIcon::kSmallIconWidth + kPixelsLeftMargin;

	if (mActiveColumns & kUserNameColFlag)
	{
		itemRect.Right() = GetColumnWidth(mUserNameColIndex);

		CFont	oldFont(itemDC.SelectObject(mPDE->hFont.Get()));
		CRect	origRect(itemRect);

		if (pItem->user.IsAdminUser())
			itemDC.SelectObject(mPDE->hBold.Get());
		else if (pItem->user.IsDisabled())
			itemDC.SelectObject(mPDE->hItalic.Get());

		// See if we need the lock icon.
		if (pItem->user.IsLocked())
		{
			CRect	tempRect(itemRect);
			tempRect.Left() = tempRect.Right() - CIcon::kSmallIconWidth;

			// Enough space. Draw Lock.
			if (tempRect.Left() > itemRect.Left())
			{
				if ((pDIS->itemState & ODS_SELECTED) && isFocused)
					itemDC.FillRect(tempRect, mPDE->highBrush);
				else
					itemDC.FillRect(tempRect, mPDE->backBrush);
			}

			mPDE->image.Draw(ImageIDs::kClosedLock, itemDC, tempRect.Left(), 
				tempRect.Top(), ILD_TRANSPARENT);

			// Make space for lock from text.
			itemRect.Right() -= CIcon::kSmallIconWidth;
		}

		DrawItemColumn(itemDC, pItem->userName, itemRect);

		// Set rectangle back to original
		itemRect = origRect;
		itemDC.SelectObject(oldFont.Get());

		itemRect.Left() = itemRect.Right();
	}

	if (mActiveColumns & kReadOnlyColFlag)
	{
		itemRect.Right() += GetColumnWidth(mReadOnlyColIndex);

		DrawTrueFalseButton(itemDC, itemRect, pItem->user.IsReadOnly(), 
			((pDIS->itemState & ODS_SELECTED) && isFocused));

		itemRect.Left() = itemRect.Right();
	}

	if (mActiveColumns & kKindColFlag)
	{
		itemRect.Right() += GetColumnWidth(mKindColIndex);

		CFont	oldFont(itemDC.SelectObject(mPDE->hFont.Get()));
			DrawItemColumn(itemDC, pItem->kindText, itemRect);

		itemDC.SelectObject(oldFont.Get());
		itemRect.Left() = itemRect.Right();
	}
}

void 
CDiskUsersView::DrawUserItem(LPDRAWITEMSTRUCT pDIS) const
{
	PGPBoolean	isFocused	= TRUE;

	if (GetFocus() != Handle())
		isFocused = FALSE;

	CDC			dc(pDIS->hDC);
	PGPUInt32	uiFlags	= ILD_TRANSPARENT;

	// Check to see if this item is selected
	if ((pDIS->itemState & ODS_SELECTED) && isFocused)
	{
		// Set the text background and foreground colors
		dc.SetTextColor(GetSysColor(COLOR_HIGHLIGHTTEXT));
		dc.SetBkColor(GetSysColor(COLOR_HIGHLIGHT));

		// Also add the ILD_BLEND50 so the images come out selected
		uiFlags |= ILD_BLEND50;
	}
	else
	{
		// Set the text background and foreground colors to the 
		// standard window colors

		dc.SetTextColor(GetSysColor(COLOR_WINDOWTEXT));
		dc.SetBkColor(GetSysColor(COLOR_WINDOW));
	}

	DrawUserItemAux(pDIS, isFocused);

	pDIS->rcItem.left += CIcon::kSmallIconWidth;

	if (!isFocused)
		return;

	// If we changed the colors for the selected item, undo it
	if (pDIS->itemState & ODS_SELECTED)
	{
		// Set the text background and foreground colors
		dc.SetTextColor(GetSysColor(COLOR_WINDOWTEXT));
		dc.SetBkColor(GetSysColor(COLOR_WINDOW));
	}

	// If the item is focused, now draw a focus rect around the entire row
//	if (pDIS->itemState & ODS_FOCUS)
//		dc.DrawFocusRect(pDIS->rcItem);
}

CDiskUsersView::DiskUserItem * 
CDiskUsersView::GetItemAtIndex(PGPInt32 i) const
{
	LVITEM	lvi;

	lvi.mask		= LVIF_PARAM;
	lvi.iItem		= i;							 
	lvi.iSubItem	= 0;

	GetItem(&lvi);
	return reinterpret_cast<DiskUserItem *>(lvi.lParam);
}

PGPdiskUserRef 
CDiskUsersView::GetUserAtIndex(PGPInt32 i) const
{
	return GetItemAtIndex(i)->user;
}

PGPInt32 
CDiskUsersView::GetIndexOfItem(const DiskUserItem *pItem) const
{
	LVFINDINFO	lvfi;
	pgpClearMemory(&lvfi, sizeof(lvfi));

	lvfi.flags	= LVFI_PARAM;
	lvfi.lParam	= reinterpret_cast<LPARAM>(pItem);

	return FindItem(-1, &lvfi);
}

PGPInt32 
CDiskUsersView::GetIndexOfEqualUser(const CPGPdiskUser& diskUser) const
{
	PGPUInt32	itemCount	= GetItemCount();

	for (PGPUInt32 i = 0; i < itemCount; i++)
	{
		CPGPdiskUser	curUser(GetUserAtIndex(i));

		if (diskUser.IsEqualUser(curUser))
			return i;
	}

	return -1;
}

void 
CDiskUsersView::CalcItemDisplayData(
	DiskUserItem	*pItem, 
	PGPBoolean	redraw) const
{
	using namespace UCommonStrings;
	pgpAssertAddrValid(pItem, DiskUserItem);

	pItem->user.GetUserName(pItem->userName);
	pItem->isPubAndNotInRing = FALSE;

	if (pItem->user.HasPublicKey())
	{
		pItem->icon = ImageIDs::kDiskPubKeyUser;

		// Make note if the associated public key is not in keyring.
		CPGPKeyID	keyID;
		pItem->user.GetPGPKeyID(keyID);

		PGPPublicKeyAlgorithm	algorithm	= keyID.Algorithm();

		try
		{
			CPGPdiskContext	context(pItem->user.Context());
			CPGPKey	pgpKey(context.PGPKeyDB(), keyID);
		}
		catch (CComboError&)
		{
			pItem->isPubAndNotInRing = TRUE;
		}

		if (algorithm == kPGPPublicKeyAlgorithm_RSA)
			pItem->kindText = Get(kDiskUserViewPubKeyRSAText);
		else
			pItem->kindText = Get(kDiskUserViewPubKeyDHText);
	}
	else
	{
		pItem->icon = ImageIDs::kDiskPassphraseUser;
		pItem->kindText = Get(kDiskUserViewSymKeyText);
	}

	if (pItem->isPubAndNotInRing)
		pItem->icon = ImageIDs::kInvalidUserId;

	if (redraw)
	{
		PGPInt32	index	= GetIndexOfItem(pItem);
		RedrawItems(index, index);
	}
}

void 
CDiskUsersView::AddUser(const CPGPdiskUser& diskUser)
{
	pgpAssert(GetIndexOfEqualUser(diskUser) == -1);

	// Add key to disk user set.
	mPDiskUsers->Add(diskUser);

	// Find key ref of new user in disk key set.
	CPGPdiskUserIter	diskUserIter(*mPDiskUsers);

	CPGPdiskUser	equalUser;
	diskUserIter.FindEqualUser(diskUser, equalUser);

	// Create display item using that key ref.
	auto_ptr<DiskUserItem>	pItem(new DiskUserItem);

	pItem->user.Attach(equalUser);
	CalcItemDisplayData(pItem.get());

	// Add the list item.
	LV_ITEM  lvItem;
	pgpClearMemory(&lvItem, sizeof(lvItem));

	lvItem.mask 	= LVIF_PARAM | LVIF_IMAGE | LVIF_TEXT;
	lvItem.lParam	= reinterpret_cast<LPARAM>(pItem.get());
	lvItem.iImage	= 0;
	lvItem.pszText	= const_cast<char *>(pItem->userName.Get());
	lvItem.iSubItem	= 0;
	lvItem.iItem	= GetItemCount();

	InsertItem(&lvItem);
	mItemList.AddTail(pItem.release());
}

void 
CDiskUsersView::RemoveUser(const CPGPdiskUser& diskUser)
{
	PGPInt32	index	= GetIndexOfEqualUser(diskUser);
	pgpAssert(index != -1);

	DiskUserItem	*pItem	= GetItemAtIndex(index);
	mPDiskUsers->Remove(pItem->user);

	delete mItemList.Remove(pItem);
	DeleteItem(index);
}

void 
CDiskUsersView::UpdateUser(const CPGPdiskUser& updatedUser, PGPBoolean redraw)
{
	PGPInt32	index	= GetIndexOfEqualUser(updatedUser);
	pgpAssert(index != -1);

	DiskUserItem	*pItem	= GetItemAtIndex(index);

	mPDiskUsers->Remove(updatedUser);
	mPDiskUsers->Add(updatedUser);

	CPGPdiskUserIter	diskUsersIter(*mPDiskUsers);

	CPGPdiskUser	EqualUser;
	diskUsersIter.FindEqualUser(updatedUser, EqualUser);

	pItem->user.Attach(EqualUser);
	CalcItemDisplayData(pItem, redraw);
}

void 
CDiskUsersView::CreateDrawElements()
{
	pgpAssert(IsNull(mPDE));

	auto_ptr<DKViewDrawElements>	pDE(new DKViewDrawElements);

	pDE->stdBarBrush.CreateSolid(GetSysColor(COLOR_3DSHADOW));
	pDE->spcBarBrush.CreateHatch(HS_BDIAGONAL, 
		GetSysColor(COLOR_WINDOW));
	pDE->gSelTextPen.Create(PS_SOLID, 0, GetSysColor(COLOR_WINDOWTEXT));
	pDE->gUnselTextPen.Create(PS_SOLID, 0, GetSysColor(COLOR_WINDOW));

	pDE->barColor = GetSysColor(COLOR_3DSHADOW);

	pDE->buttonPen.Create(PS_SOLID, 0, GetSysColor(COLOR_3DSHADOW));
	pDE->hilightPen.Create(PS_SOLID, 0, GetSysColor(COLOR_3DHILIGHT));
	pDE->barBGBrush.CreateSolid(GetSysColor(COLOR_3DFACE));
	pDE->highBrush.CreateSolid(GetSysColor(COLOR_HIGHLIGHT));
	pDE->backBrush.CreateSolid(GetSysColor(COLOR_WINDOW));

	LOGFONT lf;
	SystemParametersInfo(SPI_GETICONTITLELOGFONT, sizeof(LOGFONT), &lf, 0);

	pDE->hFont.CreateIndirect(lf);

	lf.lfWeight = 700;
	pDE->hBold.CreateIndirect(lf);

	lf.lfStrikeOut = TRUE;
	pDE->hBoldStrikeOut.CreateIndirect(lf);

	lf.lfWeight = 400;
	lf.lfItalic = !lf.lfItalic;
	lf.lfStrikeOut = FALSE;
	pDE->hItalic.CreateIndirect(lf);

	lf.lfStrikeOut = TRUE;
	pDE->hItalicStrikeOut.CreateIndirect(lf);

	lf.lfItalic = !lf.lfItalic;
	pDE->hStrikeOut.CreateIndirect(lf);

	// Initialize image list.
	CDC	dc;
	dc.AttachFromWindow(NULL, TRUE);	// DC for desktop

	PGPUInt32	numBits	= dc.GetDeviceCaps(BITSPIXEL) * 
		dc.GetDeviceCaps(PLANES);

	if (numBits <= 8)
	{
		pDE->image.Create(CIcon::kSmallIconWidth, CIcon::kSmallIconHeight, 
			ILC_COLOR | ILC_MASK, ImageIDs::kNumPubKeyBitmaps, 0);

		CBitmap	images(UModuleInstance::Get(), 
			MAKEINTRESOURCE(IDB_PKIMAGES4BIT));
		pDE->image.AddMasked(images, ImageIDs::kMaskColor);
	}
	else
	{
		pDE->image.Create(CIcon::kSmallIconWidth, CIcon::kSmallIconHeight, 
			ILC_COLOR24 | ILC_MASK, ImageIDs::kNumPubKeyBitmaps, 0);

		CBitmap	images(UModuleInstance::Get(), 
			MAKEINTRESOURCE(IDB_PKIMAGES24BIT));
		pDE->image.AddMasked(images, ImageIDs::kMaskColor);
	}

	mPDE = pDE.release();
}

void 
CDiskUsersView::DeleteDrawElements()
{
	pgpAssert(IsntNull(mPDE));

    delete mPDE;
	mPDE = NULL;
}

void 
CDiskUsersView::UpdateColumns()
{
	using namespace UCommonStrings;

	DeleteColumn(mKindColIndex);
	DeleteColumn(mReadOnlyColIndex);
	DeleteColumn(mUserNameColIndex);

	CRect	listRect;
	GetClientRect(listRect);

	LV_COLUMN	lvCol;
	pgpClearMemory(&lvCol, sizeof(lvCol));

	lvCol.mask	= LVCF_WIDTH | LVCF_FMT | LVCF_SUBITEM | LVCF_TEXT;

	PGPUInt32	i			= 0;
	PGPInt32	listWidth	= listRect.Width();

	if (mActiveColumns & kUserNameColFlag)
	{
		lvCol.fmt		= LVCFMT_LEFT;
		lvCol.pszText	= const_cast<char *>(Get(kDiskUserViewUserNameCol));
		lvCol.cx		= static_cast<PGPInt32>(kUserNameColRatio*listWidth);

		mUserNameColIndex = InsertColumn(i++, &lvCol);
	}

	if (mActiveColumns & kReadOnlyColFlag)
	{
		lvCol.fmt		= LVCFMT_CENTER;
		lvCol.pszText	= const_cast<char *>(Get(kDiskUserViewReadOnlyCol));
		lvCol.cx		= static_cast<PGPInt32>(kReadOnlyColRatio*listWidth);

		mReadOnlyColIndex = InsertColumn(i++, &lvCol);
	}

	if (mActiveColumns & kKindColFlag)
	{
		lvCol.fmt		= LVCFMT_LEFT;
		lvCol.pszText	= const_cast<char *>(Get(kDiskUserViewKindCol));
		lvCol.cx		= static_cast<PGPInt32>(kKindColRatio*listWidth);

		mKindColIndex = InsertColumn(i++, &lvCol);
	}
}

void 
CDiskUsersView::GenerateMeasureItem() const
{
	// Fool the system into sending us a new WM_MEASUREITEM.
	CRect	rect;
	GetWindowRect(rect);

	WINDOWPOS	wp;

	wp.hwnd		= Handle();
	wp.cx		= rect.Width();
	wp.cy		= rect.Height();
	wp.flags	= SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | 
		SWP_NOZORDER;

	SendMessage(WM_WINDOWPOSCHANGED, 0, reinterpret_cast<LPARAM>(&wp));
}

void 
CDiskUsersView::OnContextMenu(HWND menuWnd, CPoint coords)
{
	CWindow(GetParent()).SendMessage(CurMsg().Message(), CurMsg().WParam(), 
		CurMsg().LParam());
}

BOOL 
CDiskUsersView::OnDrawItem(PGPUInt16 ctrlId, LPDRAWITEMSTRUCT pDIS)
{
	if (!IsWindowEnabled())
		return FALSE;

	switch (pDIS->itemAction)
	{
		case ODA_DRAWENTIRE:
		case ODA_FOCUS:
		case ODA_SELECT:
			DrawUserItem(pDIS);
			break;
	}

	return TRUE;
}

void 
CDiskUsersView::OnKeyDown(PGPUInt32 vKey, PGPUInt32 keyData)
{
	switch (vKey)
	{
	case 'a':
	case 'A':
		// Select all items on ctrl-A.
		if (!mSingleSelect && (GetAsyncKeyState(VK_CONTROL) & 1))
			SelectAllItems();
		else
			CListView::OnKeyDown(vKey, keyData);
		break;

	default:
		CListView::OnKeyDown(vKey, keyData);
		break;
	}
}

BOOL 
CDiskUsersView::OnMeasureItem(PGPUInt16 ctrlId, LPMEASUREITEMSTRUCT pMIS)
{
    if (pMIS->CtlType != ODT_LISTVIEW)
		return FALSE;

	CDC		listDC;
	CFont	font(GetFont());

	listDC.AttachFromWindow(Handle());
	HFONT	oldFont	= listDC.SelectObject(font.Get());

	TEXTMETRIC	tm;

	if (!listDC.GetTextMetrics(&tm))
		return TRUE;
		
    // Add a little extra space between items.
	pMIS->itemHeight = tm.tmHeight + 1;

	if (pMIS->itemHeight < (CIcon::kSmallIconHeight + 1))
		pMIS->itemHeight = CIcon::kSmallIconHeight + 1;

	listDC.SelectObject(oldFont);
	listDC.Clear();

	return TRUE;
}

void 
CDiskUsersView::OnMenuSelect(
	PGPUInt16	ctrlId, 
	PGPUInt16	flags, 
	HMENU		hwndMenu)
{
	CListView::OnMenuSelect(ctrlId, flags, hwndMenu);

	// Let parent see it too.
	CWindow(GetParent()).SendMessage(WM_MENUSELECT, CurMsg().WParam(), 
		CurMsg().LParam());
}

PGPUInt32 
CDiskUsersView::OnNotify(PGPUInt16 ctrlId, LPNMHDR pNMHDR)
{
	if (pNMHDR->hwndFrom == Handle())
	{
		NMLISTVIEW	*pNMLV	= reinterpret_cast<NMLISTVIEW *>(pNMHDR);

		if (pNMHDR->code == LVN_COLUMNCLICK)
			SortOnColumn(pNMLV->iSubItem);

		return 0;
	}
	else
	{
		return CListView::OnNotify(ctrlId, pNMHDR);
	}
}

void 
CDiskUsersView::OnNcDestroy()
{
	CListView::OnNcDestroy();
	Clear();
}

void 
CDiskUsersView::OnSysColorChange()
{
	CListView::OnSysColorChange();

	DeleteDrawElements();
	CreateDrawElements();
}
