/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CEditorWindow.cpp,v 1.29 2002/11/05 02:10:53 wjb Exp $
____________________________________________________________________________*/

#include "pgpClassesConfig.h"
#include <stdlib.h>

#include "CFile.h"

#include "CBitmap.h"
#include "CButton.h"
#include "CDC.h"
#include "CEdit.h"
#include "CUTF8Edit.h"
#include "CFont.h"
#include "CMessageBox.h"
#include "CRect.h"
#include "CStatic.h"
#include "CUpDownControl.h"
#include "UModuleInstance.h"

#include "pgpClientLib.h"

#include "UCommonStrings.h"
#include "CPGPclPrefs.h"
#include "CPGPdata.h"
#include "CPGPdiskEngine.h"
#include "CPGPKey.h"
#include "CPGPKeyDB.h"

#include "CConfirmPassphraseDialog.h"
#include "CEditorWindow.h"
#include "CHelpShower.h"
#include "CNewUserWizard.h"
#include "CPassphraseDialog.h"
#include "CPGPdiskMounter.h"
#include "CPGPdiskUnmounter.h"
#include "CReEncryptWizard.h"
#include "UEditMount.h"
#include "ImageIDs.h"
#include "PGPdiskHelp.h"
#include "PGPdiskUIPriv.h"
#include "Resource.h"

#include "utf8edit.h"

_USING_PGP

_UNNAMED_BEGIN

// Constants

enum
{
	kMountMenuPos	= 0, 
	kTimerMs		= 2000
};

const char	*kEditorWindowClassName	= "PGPdisk Editor Window Class";

DWORD	DiskPropertiesDlgHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_CURALGO_CAPTION,	IDH_PGPDISKUI_DISKPROPSDLG_CURALGORITHM, 
	IDC_CURALGO_EDIT,		IDH_PGPDISKUI_DISKPROPSDLG_CURALGORITHM, 
	IDC_REENCRYPT_BUTTON,	IDH_PGPDISKUI_DISKPROPSDLG_REENCRYPT, 
	IDC_MOUNTSTART_CHECK,	IDH_PGPDISKUI_DISKPROPSDLG_MOUNTSTART, 
	IDC_OVERRIDE_CHECK,		IDH_PGPDISKUI_DISKPROPSDLG_OVERRIDE, 
	IDC_UNMOUNT_CHECK,		IDH_PGPDISKUI_DISKPROPSDLG_AUTOUNMOUNT, 
	IDC_MINUTES_TEXT,		IDH_PGPDISKUI_DISKPROPSDLG_AUTOUNMOUNT, 
	IDC_MINUTES_EDIT,		IDH_PGPDISKUI_DISKPROPSDLG_MINUTES, 
	IDC_MINUTES_UDCTRL,		IDH_PGPDISKUI_DISKPROPSDLG_MINUDCTRL, 
	IDOK,					IDH_PGPDISKUI_DISKPROPSDLG_OK, 
	IDCANCEL,				IDH_PGPDISKUI_DISKPROPSDLG_CANCEL, 
	0,						0
};

// Prefs constants.
enum
{
	kDefaultPosX			= 100, 
	kDefaultPosY			= 100, 
	kDefaultWidth			= 600, 
	kDefaultHeight			= 400, 
	kDefaultDiskUserColumns	= 
		CDiskUsersView::kUserNameColFlag | 
		CDiskUsersView::kReadOnlyColFlag | 
		CDiskUsersView::kKindColFlag, 

	kMinWidth	= 400, 
	kMinHeight	= 175
};

const PGPBoolean	kDefaultShowToolbar		= TRUE;
const PGPBoolean	kDefaultLargeButtons	= FALSE;
const PGPBoolean	kDefaultShowText		= TRUE;
const PGPBoolean	kDefaultToolTips		= TRUE;

// Toolbar constants.

enum
{
	kXMarginRebar	= 1, 
	kYMarginRebar	= 1, 

	kXMarginDiskUsers	= 1, 
	kYMarginDiskUsers	= 1, 
	kYMarginToolbarDK	= 2, 
	kYMarginDKStatusBar	= 2, 

	kNumToolbarButtons	= 5, 

	kXSmallImage	= 16, 
	kYSmallImage	= 16, 
	kXBigImage		= 20, 
	kYBigImage		= 20, 

	kMaxButtonWidth	= 70
};

// Menu constants.

enum
{
	kIndexFileMenu	= 0, 
	kIndexUsersMenu	= 1, 
	kIndexViewMenu	= 2, 
	kIndexHelpMenu	= 3
};

_UNNAMED_END

// Types

struct CEditorWindow::EditorWindowPrefs
{
	PGPInt32	posX;
	PGPInt32	posY;
	PGPInt32	width;
	PGPInt32	height;

	PGPUInt32	diskUserColumns;

	PGPBoolean	showToolbar;
	PGPBoolean	largeButtons;
	PGPBoolean	showText;
	PGPBoolean	toolTips;
};

struct CEditorWindow::ToolbarButton : 
	public CListableObject<CEditorWindow::ToolbarButton>
{
	TBBUTTON	tbButton;
	PGPInt32	index;
};


_UNNAMED_BEGIN

// Class CDiskProperties

class CDiskProperties : public CDialog
{
public:
	CDiskProperties::CDiskProperties() : mPContext(NULL) { }
	CDiskProperties::~CDiskProperties() { }

	PGPBoolean	WasReEncryptionBegun() const {return mReEncryptBegun;}
	PGPBoolean	WasReEncryptionCompleted() const {return mReEncryptCompleted;}

	PGPUInt16	GetDialogId() const
	{
		return IDD_DISKPROPS_DLG;
	}

	void	Display(const CPGPdiskContext *pContext, const char *path, 
		HWND parentWnd);

private:
	CButton			mReEncryptButton;
	CButton			mMountStartupCheck;
	CButton			mOverrideCheck;
	CButton			mAutoUnmountCheck;
	CUTF8Edit			mCurAlgoEdit;
	CEdit			mMinutesEdit;
	CStatic			mMinutesText;
	CUpDownControl	mMinutesUDCtrl;

	const CPGPdiskContext	*mPContext;
	CPath					mPath;

	PGPBoolean	mMountStartupValue;
	PGPBoolean	mOverrideValue;
	PGPBoolean	mAutoUnmountValue;
	PGPUInt32	mMinutesValue;

	PGPBoolean	mReEncryptBegun;
	PGPBoolean	mReEncryptCompleted;
	PGPBoolean	mTreatAsRO;

	void	ImportValuesFromUI();
	void	ExportValuesToUI();

	PGPBoolean	FixWarnMinutes(PGPBoolean silently = FALSE);

	void	InitControls();
	void	OnAutoUnmountCheck();
	void	OnMountStartupCheck();
	void	OnOk();
	void	OnOverrideCheck();
	void	OnReEncryptButton();
	void	WriteChanges();

	BOOL	OnCommand(PGPUInt16 notifyCode, PGPUInt16 itemId, HWND ctrl);
	void	OnContextMenu(HWND menuWnd, CPoint coords);
	void	OnHelp(HELPINFO *pInfo);
	BOOL	OnInitDialog(HWND focusCtrl);
};

_UNNAMED_END


// Class CDiskProperties public member functions

void 
CDiskProperties::Display(
	const CPGPdiskContext	*pContext, 
	const char				*path, 
	HWND					parentWnd)
{
	pgpAssertAddrValid(pContext, CPGPdiskContext);
	pgpAssertStrValid(path);

	mPContext = pContext;
	mPath = path;

	mReEncryptBegun = FALSE;
	mReEncryptCompleted	= FALSE;

	CreateModal(parentWnd);
}

void 
CDiskProperties::ImportValuesFromUI()
{
	mMountStartupValue	= (mMountStartupCheck.GetCheck() == BST_CHECKED);
	mOverrideValue		= (mOverrideCheck.GetCheck() == BST_CHECKED);
	mAutoUnmountValue	= (mAutoUnmountCheck.GetCheck() == BST_CHECKED);

	CString	minutesText;

	mMinutesEdit.GetWindowText(minutesText);
	mMinutesValue = atoi(minutesText);
};

void 
CDiskProperties::ExportValuesToUI()
{
	mMountStartupCheck.SetCheck(mMountStartupValue ? 
		BST_CHECKED : BST_UNCHECKED);
	mOverrideCheck.SetCheck(mOverrideValue ? 
		BST_CHECKED : BST_UNCHECKED);
	mAutoUnmountCheck.SetCheck(mAutoUnmountValue ? 
		BST_CHECKED : BST_UNCHECKED);

	CString	minutesText;

	_itoa(mMinutesValue, minutesText.GetBuffer(33), 10);
	minutesText.ReleaseBuffer();

	mMinutesEdit.SetWindowText(minutesText);
}

PGPBoolean 
CDiskProperties::FixWarnMinutes(PGPBoolean silently)
{
	using namespace UCommonStrings;

	CommonString	failString;
	PGPBoolean		failThis	= FALSE;
	PGPBoolean		userWarned	= FALSE;

	ImportValuesFromUI();

	// Verify that the minutes value is not too big or small.
	if (!mAutoUnmountValue)
		silently = TRUE;

	if (mMinutesValue == 0)
	{
		failThis = TRUE;
		mMinutesValue = 1;

		failString = kPGPdiskOptionsMinutesTooSmall;
	}
	else if (mMinutesValue > kPGPdiskMaxMinutesTimeout)
	{
		failThis = TRUE;
		mMinutesValue = kPGPdiskMaxMinutesTimeout;

		failString = kPGPdiskOptionsMinutesTooLarge;
	}

	if (failThis)
	{
		if (!silently)
		{
			CMessageBox	messageBox;
			messageBox.Display(Get(failString), 
				Get(kPGPdiskOptionsMsgBoxTitle), Handle());

			userWarned = TRUE;
		}

		ExportValuesToUI();

		if (!silently)
		{
			// Point out the incorrect minutes entered by the user.
			mMinutesEdit.SetFocus();
			mMinutesEdit.SetSel(0, -1);
		}
	}

	return userWarned;
}

void 
CDiskProperties::InitControls()
{
	using namespace UCommonStrings;

	mTreatAsRO = CPGPdiskEngine::IsPathMountedPGPdisk(mPath) || 
		!UEditMount::CheckWarnCanModifyDisk(*mPContext, mPath, Handle(), 
		Get(kEditorMsgBoxTitle));

	CPGPdiskDisk	disk;
	disk.Open(*mPContext, mPath, TRUE);

	CPGPclPrefs	prefs(mPContext->PGPclPrefs());

	// Display existing algorithm in static edit box.
	CString	curAlgoText;
	CPGPdiskEngine::GetDiskAlgorithmIdString(disk.Algorithm(), curAlgoText);

	mCurAlgoEdit.SetWindowText(curAlgoText);

	// Display mount prefs.
	mMountStartupValue = UEditMount::IsDiskMarkedForStartup(*mPContext, 
		mPath);
	mMinutesUDCtrl.SetRange(1, kPGPdiskMaxMinutesTimeout);

	PGPUInt32	customTimeout	= disk.GetCustomTimeout();

	mOverrideValue = disk.GetUsesCustomTimeout();
	mAutoUnmountValue = (customTimeout > 0);

	if (mAutoUnmountValue && (customTimeout > 0))
		mMinutesValue = customTimeout;
	else
		mMinutesValue = prefs.GetNumber(kPGPPrefDiskAutoUnmountTimeout);

	mMinutesEdit.EnableWindow(mAutoUnmountValue);
	mMinutesUDCtrl.EnableWindow(mAutoUnmountValue);

	mAutoUnmountCheck.EnableWindow(mOverrideValue);
	mMinutesText.EnableWindow(mOverrideValue);

	if (!mOverrideValue)
	{
		mMinutesEdit.EnableWindow(FALSE);
		mMinutesUDCtrl.EnableWindow(FALSE);
	}

	// Disable controls if read-only.
	if (mTreatAsRO)
	{
		mReEncryptButton.EnableWindow(FALSE);
		mMountStartupCheck.EnableWindow(FALSE);
		mOverrideCheck.EnableWindow(FALSE);
		mAutoUnmountCheck.EnableWindow(FALSE);
		mMinutesEdit.EnableWindow(FALSE);
		mMinutesText.EnableWindow(FALSE);
		mMinutesUDCtrl.EnableWindow(FALSE);
	}

	ExportValuesToUI();
	FixWarnMinutes(TRUE);
}

void 
CDiskProperties::OnAutoUnmountCheck()
{
	ImportValuesFromUI();

	mMinutesEdit.EnableWindow(mAutoUnmountValue);
	mMinutesUDCtrl.EnableWindow(mAutoUnmountValue);
}

void 
CDiskProperties::OnMountStartupCheck()
{
	ImportValuesFromUI();
}

void 
CDiskProperties::OnOk()
{
	ImportValuesFromUI();

	if (FixWarnMinutes())
		return;

	if (!mTreatAsRO)
		WriteChanges();

	CDialog::OnOk();
}

void 
CDiskProperties::OnOverrideCheck()
{
	ImportValuesFromUI();

	mAutoUnmountCheck.EnableWindow(mOverrideValue);
	mMinutesText.EnableWindow(mOverrideValue);

	if (!mOverrideValue)
	{
		mMinutesEdit.EnableWindow(FALSE);
		mMinutesUDCtrl.EnableWindow(FALSE);
	}
	else
	{
		mMinutesEdit.EnableWindow(mAutoUnmountValue);
		mMinutesUDCtrl.EnableWindow(mAutoUnmountValue);
	}
}

void 
CDiskProperties::OnReEncryptButton()
{
	try
	{
		using namespace UCommonStrings;

		if (!UEditMount::CheckWarnCanModifyDisk(*mPContext, mPath, Handle(), 
			Get(kEditorMsgBoxTitle)))
		{
			THROW_PGPERROR(kPGPError_UserAbort);
		}
		
		ImportValuesFromUI();

		if (FixWarnMinutes())
			return;

		WriteChanges();

		CReEncryptWizard	reEncryptWiz;
		reEncryptWiz.Display(*mPContext, mPath, Handle(), &mReEncryptBegun);
		mReEncryptCompleted = TRUE;	// we finished successfully

		InitControls();
	}
	catch (CComboError& error)
	{
		using namespace UCommonStrings;

		ReportError(kTellDiskReEncryptFailed, kDiskPropertiesMsgBoxTitle, 
			error, Handle());

		// Failed/interrupted conversions result in window close.
		if (mReEncryptBegun && !mReEncryptCompleted)
			CDialog::OnOk();
	}
}

void 
CDiskProperties::WriteChanges()
{
	try
	{
		if (mMountStartupValue)
			UEditMount::MarkDiskForStartup(*mPContext, mPath);
		else
			UEditMount::UnmarkDiskForStartup(*mPContext, mPath);
	}
	catch (CComboError&)
	{
		using namespace UCommonStrings;

		CMessageBox	messageBox;
		messageBox.Display(Get(kTellCantMarkDiskFor), 
			Get(kPGPdiskOptionsMsgBoxTitle), Handle(), 
			CMessageBox::kWarningTone);
	}
	
	try
	{
		CPGPdiskDisk	disk;
		disk.Open(*mPContext, mPath);

		disk.SetUsesCustomTimeout(mOverrideValue);

		if (mAutoUnmountValue)
			disk.SetCustomTimeout(mMinutesValue);
		else
			disk.SetCustomTimeout(0);
	}
	catch (CComboError&)
	{
		using namespace UCommonStrings;

		CMessageBox	messageBox;
		messageBox.Display(Get(kTellCantSetCustomTimeout), 
			Get(kPGPdiskOptionsMsgBoxTitle), Handle(), 
			CMessageBox::kWarningTone);
	}
}

BOOL 
CDiskProperties::OnCommand(
	PGPUInt16	notifyCode, 
	PGPUInt16	itemId, 
	HWND		ctrl)
{
	switch (itemId)
	{
	case IDC_MOUNTSTART_CHECK:
		OnMountStartupCheck();
		return TRUE;

	case IDC_OVERRIDE_CHECK:
		OnOverrideCheck();
		return TRUE;

	case IDC_REENCRYPT_BUTTON:
		OnReEncryptButton();
		return TRUE;

	case IDC_UNMOUNT_CHECK:
		OnAutoUnmountCheck();
		return TRUE;
	}

	return CDialog::OnCommand(notifyCode, itemId, ctrl);
}

void 
CDiskProperties::OnContextMenu(HWND menuWnd, CPoint coords)
{
	CHelpShower::ShowContextMenu(menuWnd, DiskPropertiesDlgHelpIDs);
}

void 
CDiskProperties::OnHelp(HELPINFO *pInfo)
{
	CHelpShower::ShowForControl(static_cast<HWND>(pInfo->hItemHandle), 
		DiskPropertiesDlgHelpIDs);
}

BOOL 
CDiskProperties::OnInitDialog(HWND focusCtrl)
{
	CDialog::OnInitDialog(focusCtrl);

	mReEncryptButton.Subclass(GetDlgItem(IDC_REENCRYPT_BUTTON));
	mMountStartupCheck.Subclass(GetDlgItem(IDC_MOUNTSTART_CHECK));
	mOverrideCheck.Subclass(GetDlgItem(IDC_OVERRIDE_CHECK));
	mAutoUnmountCheck.Subclass(GetDlgItem(IDC_UNMOUNT_CHECK));
	mCurAlgoEdit.Subclass(GetDlgItem(IDC_CURALGO_EDIT));
	mMinutesEdit.Subclass(GetDlgItem(IDC_MINUTES_EDIT));
	mMinutesText.Subclass(GetDlgItem(IDC_MINUTES_TEXT));
	mMinutesUDCtrl.Subclass(GetDlgItem(IDC_MINUTES_UDCTRL));

	InitControls();

	Center();
	SetForegroundWindow();

	return TRUE;
}


// Class CEditorWindow variables

CIcon	CEditorWindow::WindowIcon;
CCursor	CEditorWindow::WindowCursor;


// Class CEditorWindow member functions

CEditorWindow::CEditorWindow() : mPContext(NULL)
{
	mPPrefs = new EditorWindowPrefs;
}

CEditorWindow::~CEditorWindow()
{
	delete mPPrefs;
}

void 
CEditorWindow::Display(
	const CPGPdiskContext&	context, 
	const char				*path, 
	HWND					owner)
{
	pgpAssertStrValid(path);
	pgpAssert(IsWindow(owner));

	mPContext = &context;
	mPath = path;

	ReadWindowPrefs();

	// Load the accelerator.
	mAccel.Load(UModuleInstance::Get(), MAKEINTRESOURCE(IDR_EDITOR_ACCEL));

	// Create the menu.
	mMainMenu.Load(UModuleInstance::Get(), MAKEINTRESOURCE(IDR_EDITOR_MENU));

	// Create the window.
	CPath	cpath(path), fileName;
	cpath.GetDisplayFileName(fileName);

	CString	title;

	title.Format(UCommonStrings::Get(UCommonStrings::kEditorWindowTitle), 
		fileName.Get());

	Create(kEditorWindowClassName, NULL, title, WS_OVERLAPPEDWINDOW, 0, 
		mPPrefs->posX, mPPrefs->posY, mPPrefs->width, mPPrefs->height, 
		owner, mMainMenu);

	// Enter simple modal loop.
	ShowModal(mAccel);

	// flush prefs
	try
	{
		CPGPclPrefs	prefs(context.PGPclPrefs());
		prefs.Flush();
	}
	catch (CComboError&) { }
}

void 
CEditorWindow::InitClass()
{
	WindowCursor.Load(NULL, IDC_ARROW, TRUE);
	WindowIcon.Load(UModuleInstance::Get(), MAKEINTRESOURCE(IDI_EDITICON));

	RegisterClass(kEditorWindowClassName, 0, 0, 0, WindowIcon, NULL, 
		WindowCursor, reinterpret_cast<HBRUSH>(COLOR_BTNFACE + 1));
}

void 
CEditorWindow::CleanupClass()
{
	UnregisterClass(kEditorWindowClassName);
}

void 
CEditorWindow::ReadWindowPrefs()
{
	CPGPclPrefs	prefs(mPContext->PGPclPrefs());
	CPGPData	prefData;

	try
	{
		prefs.GetData(kPGPPrefDiskEditorWinPos, prefData);

		if (prefData.Size() < sizeof(*mPPrefs))
			THROW_PGPERROR(kPGPError_BadParams);

		pgpCopyMemory(prefData.Get(), mPPrefs, sizeof(*mPPrefs));
	}
	catch (CComboError&)
	{
		mPPrefs->posX				= kDefaultPosX;
		mPPrefs->posY				= kDefaultPosY;
		mPPrefs->width				= kDefaultWidth;
		mPPrefs->height				= kDefaultHeight;
		mPPrefs->diskUserColumns	= kDefaultDiskUserColumns;
		mPPrefs->showToolbar		= kDefaultShowToolbar;
		mPPrefs->largeButtons		= kDefaultLargeButtons;
		mPPrefs->showText			= kDefaultShowText;
		mPPrefs->toolTips			= kDefaultToolTips;

		prefs.SetData(kPGPPrefDiskEditorWinPos, prefData);
	}
}

void 
CEditorWindow::WriteWindowPrefs() const
{
	CRect	wndRect;
	GetWindowRect(wndRect);

	mPPrefs->posX	= wndRect.Left();
	mPPrefs->posY	= wndRect.Top();
	mPPrefs->width	= wndRect.Width();
	mPPrefs->height	= wndRect.Height();

	CPGPclPrefs	prefs(mPContext->PGPclPrefs());
	CPGPData	prefData(const_cast<EditorWindowPrefs *>(mPPrefs), 
		sizeof(*mPPrefs));

	prefs.SetData(kPGPPrefDiskEditorWinPos, prefData);
}

void 
CEditorWindow::ShowStatusBarHelp(PGPUInt16 ctrlId)
{
	using namespace UCommonStrings;

	CommonString	stringID;

	switch (ctrlId)
	{
	// The editor main menu/toolbar.
	case IDM_MOUNT:
		stringID = kEditorMountHelp;
		break;

	case IDM_UNMOUNT:
		stringID = kEditorUnmountHelp;
		break;

	case IDM_REENCRYPT:
		stringID = kEditorReEncryptHelp;
		break;

	case IDM_DISKPROPS:
		stringID = kEditorPropertiesHelp;
		break;

	case IDM_DISKOPTIONS:
		stringID = kEditorOptionsHelp;
		break;

	case IDM_EXIT:
		stringID = kEditorExitHelp;
		break;

	case IDM_ADDUSER:
		stringID = kEditorAddUserHelp;
		break;

	case IDM_REMOVEUSER:
		stringID = kEditorRemUserHelp;
		break;

	case IDM_MOUNTUSING:
		stringID = kEditorMountUsingHelp;
		break;

	case IDM_CHANGEPASS:
		stringID = kEditorChangePassHelp;
		break;

	case IDM_SETASADMIN:
		stringID = kEditorSetAsAdminHelp;
		break;

	case IDM_TOGGLEDISABLE:
		stringID = kEditorToggleDisableHelp;
		break;

	case IDM_TOGGLERO:
		stringID = kEditorToggleROHelp;
		break;

	case IDM_VIEWKIND:
		stringID = kEditorViewKindHelp;
		break;

	case IDM_VIEWRO:
		stringID = kEditorViewROHelp;
		break;

	case IDM_VIEWTOOLBAR:
		stringID = kEditorViewToolbarHelp;
		break;

	case IDM_HELP_CONTENTS:
		stringID = kEditorHelpContentsHelp;
		break;

	case IDM_ABOUTPGP:
		stringID = kEditorAboutPGPHelp;
		break;

	// The disk users context menu.
	case IDM_LOOKUP:
		stringID = kEditorLookupHelp;
		break;

	// The toolbar context menu.
	case IDM_LARGEBUTTONS:
		stringID = kEditorLargeButtonsHelp;
		break;

	case IDM_SHOWTEXT:
		stringID = kEditorShowTextHelp;
		break;

	case IDM_SHOWTOOLTIPS:
		stringID = kEditorShowTooltipsHelp;
		break;

	default:
		mStatusBar.SetText(0, 0, "");
		return;
	}

	mStatusBar.SetText(0, 0, const_cast<char *>(Get(stringID)));
}

void 
CEditorWindow::ResizeUsersView()
{
	CRect	clientRect;
	GetClientRect(clientRect);

	CRect	rebarRect;

	if (mPPrefs->showToolbar)
	{
		mTBRebar.GetWindowRect(rebarRect);
		ScreenToClient(rebarRect);
	}

	CRect	statusBarRect;

	mStatusBar.GetWindowRect(statusBarRect);
	ScreenToClient(statusBarRect);

	CRect	usersRect;

	usersRect.Left()	= kXMarginDiskUsers;
	usersRect.Top()		= rebarRect.Bottom() + kYMarginToolbarDK;
	usersRect.Right()	= clientRect.Right() - kXMarginDiskUsers;
	usersRect.Bottom()	= statusBarRect.Top() - kYMarginDKStatusBar;

	mUsersView.MoveWindow(usersRect.Left(), usersRect.Top(), 
		usersRect.Width(), usersRect.Height());
}

void 
CEditorWindow::ResizeToolbar()
{
	CRect	clientRect;
	GetClientRect(clientRect);

	// Set rebar size.
	mToolbar.AutoSize();

	PGPUInt16	buttonHeight, buttonWidth;
	mToolbar.GetButtonSize(buttonWidth, buttonHeight);

	REBARBANDINFO	rbbInfo;
	pgpClearMemory(&rbbInfo, sizeof(rbbInfo));

	rbbInfo.cbSize		= sizeof(rbbInfo);
	rbbInfo.fMask		= RBBIM_CHILDSIZE | RBBIM_SIZE;
	rbbInfo.cxMinChild	= 0;
	rbbInfo.cyMinChild	= buttonHeight + 4;
	rbbInfo.cx			= clientRect.Right() - kXMarginRebar - 1;

	mTBRebar.SetBandInfo(0, rbbInfo);

	mTBRebar.MoveWindow(kXMarginRebar, kYMarginRebar, rbbInfo.cx, 
		buttonHeight + 4);

	mToolbar.AutoSize();
}

void 
CEditorWindow::ReFillUsersView()
{
	CPGPdiskDisk	disk;
	disk.Open(*mPContext, mPath, TRUE);

	CPGPdiskUserSet	diskUsers;
	disk.ExportUserSet(diskUsers);

	mUsersView.Fill(mPContext, diskUsers);
}

void 
CEditorWindow::CreateUsersView()
{
	mUsersView.Create(Handle(), IDC_EDITORKEYS_VIEW);

	ResizeUsersView();
	ReFillUsersView();

	mUsersView.SetActiveColumns(mPPrefs->diskUserColumns);
	mUsersView.ShowWindow(SW_SHOW);
}

void 
CEditorWindow::CreateStatusBar()
{
	mStatusBar.Create(Handle(), IDC_EDITOR_STATUSBAR);
}

void 
CEditorWindow::CreateToolbar()
{
	using namespace UCommonStrings;

	// Create the rebar.
	mTBRebar.Create(Handle(), IDC_EDITOR_REBAR, WS_BORDER | CCS_NODIVIDER | 
		CCS_NOPARENTALIGN | CCS_NORESIZE | RBS_VARHEIGHT | RBS_BANDBORDERS | 
		RBS_FIXEDORDER);

	REBARINFO	rbInfo;
	pgpClearMemory(&rbInfo, sizeof(rbInfo));

	rbInfo.cbSize	= sizeof(rbInfo);
	rbInfo.fMask	= 0;

	mTBRebar.SetBarInfo(rbInfo);

	// Create the toolbar.
	mToolbar.Create(mTBRebar.Handle(), IDC_EDITOR_TOOLBAR, CCS_NODIVIDER | 
		TBSTYLE_FLAT | (mPPrefs->toolTips ? TBSTYLE_TOOLTIPS : 0));
	mToolbar.SetButtonWidth(0, kMaxButtonWidth);

	// Create private toolbar structures.
	ToolbarButton	*pMount			= new ToolbarButton;
	mButtons.AddTail(pMount);
	ToolbarButton	*pUnmount		= new ToolbarButton;
	mButtons.AddTail(pUnmount);
	ToolbarButton	*pProperties	= new ToolbarButton;
	mButtons.AddTail(pProperties);
	ToolbarButton	*pSep1			= new ToolbarButton;
	mButtons.AddTail(pSep1);
	ToolbarButton	*pAdd		= new ToolbarButton;
	mButtons.AddTail(pAdd);
	ToolbarButton	*pRemUser		= new ToolbarButton;
	mButtons.AddTail(pRemUser);

	// Load appropriate imagelist.
	CDC	dc;
	dc.AttachFromWindow(NULL, TRUE);	// DC for desktop

	PGPUInt32	numBits	= dc.GetDeviceCaps(BITSPIXEL) * 
		dc.GetDeviceCaps(PLANES);
	PGPUInt32	colorFlag, bitmapID;

	if (numBits <= 8)
	{
		colorFlag = ILC_COLOR4;

		if (mPPrefs->largeButtons)
			bitmapID = IDB_TOOLBAR4B;
		else
			bitmapID = IDB_TOOLBAR4BSM;
	}
	else
	{
		colorFlag = ILC_COLOR8;

		if (mPPrefs->largeButtons)
			bitmapID = IDB_TOOLBAR8B;
		else
			bitmapID = IDB_TOOLBAR8BSM;
	}

	CBitmap	bitmap(UModuleInstance::Get(), MAKEINTRESOURCE(bitmapID));

	if (mPPrefs->largeButtons)
	{
		mImages.Create(kXBigImage, kYBigImage, ILC_MASK | colorFlag, 
			kNumToolbarButtons, 0);
	}
	else
	{
		mImages.Create(kXSmallImage, kYSmallImage, ILC_MASK | colorFlag, 
			kNumToolbarButtons, 0);
	}

	mImages.AddMasked(bitmap, ImageIDs::kMaskColor);
	mToolbar.SetImageList(mImages);

	// Add mount button.
	pMount->tbButton.iBitmap	= ImageIDs::kPGPdiskMount;
	pMount->tbButton.idCommand	= IDM_MOUNT;
	pMount->tbButton.fsState	= TBSTATE_ENABLED;
	pMount->tbButton.fsStyle	= TBSTYLE_BUTTON;
	pMount->tbButton.dwData		= reinterpret_cast<PGPUInt32>(pMount);

	pMount->tbButton.iString = (mPPrefs->showText ? 
		mToolbar.AddString(0, Get(kEditorMountCaption)) : NULL);

	pMount->index = mToolbar.AddButtons(1, &pMount->tbButton);

	// Add unmount button.
	pUnmount->tbButton.iBitmap		= ImageIDs::kPGPdiskUnmount;
	pUnmount->tbButton.idCommand	= IDM_UNMOUNT;
	pUnmount->tbButton.fsState		= TBSTATE_ENABLED;
	pUnmount->tbButton.fsStyle		= TBSTYLE_BUTTON;
	pUnmount->tbButton.dwData		= reinterpret_cast<PGPUInt32>(pUnmount);

	pUnmount->tbButton.iString = (mPPrefs->showText ? 
		mToolbar.AddString(0, Get(kEditorUnmountCaption)) : NULL);

	pUnmount->index = mToolbar.AddButtons(1, &pUnmount->tbButton);

	// Add properties button.
	pProperties->tbButton.iBitmap	= ImageIDs::kPGPdiskProps;
	pProperties->tbButton.idCommand	= IDM_DISKPROPS;
	pProperties->tbButton.fsState	= TBSTATE_ENABLED;
	pProperties->tbButton.fsStyle	= TBSTYLE_BUTTON;
	pProperties->tbButton.dwData	= reinterpret_cast<PGPUInt32>(
		pProperties);

	pProperties->tbButton.iString = (mPPrefs->showText ? 
		mToolbar.AddString(0, Get(kEditorPropertiesCaption)) : NULL);
	
	pProperties->index = mToolbar.AddButtons(1, &pProperties->tbButton);

	// Add separator.
	pSep1->tbButton.iBitmap		= 0;
	pSep1->tbButton.idCommand	= 0;
	pSep1->tbButton.fsState		= 0;
	pSep1->tbButton.fsStyle		= TBSTYLE_SEP;
	pSep1->tbButton.dwData		= reinterpret_cast<PGPUInt32>(pSep1);
	pSep1->tbButton.iString		= NULL;
	
	pSep1->index = mToolbar.AddButtons(1, &pSep1->tbButton);

	// Add add user button.
	pAdd->tbButton.iBitmap		= ImageIDs::kPGPdiskAddUser;
	pAdd->tbButton.idCommand	= IDM_ADDUSER;
	pAdd->tbButton.fsState		= TBSTATE_ENABLED;
	pAdd->tbButton.fsStyle		= TBSTYLE_BUTTON;
	pAdd->tbButton.dwData		= reinterpret_cast<PGPUInt32>(pAdd);

	pAdd->tbButton.iString = (mPPrefs->showText ? 
		mToolbar.AddString(0, Get(kEditorAddUserCaption)) : NULL);

	pAdd->index = mToolbar.AddButtons(1, &pAdd->tbButton);

	// Add remove user button.
	pRemUser->tbButton.iBitmap		= ImageIDs::kPGPdiskRemUser;
	pRemUser->tbButton.idCommand	= IDM_REMOVEUSER;
	pRemUser->tbButton.fsState		= TBSTATE_ENABLED;
	pRemUser->tbButton.fsStyle		= TBSTYLE_BUTTON;
	pRemUser->tbButton.dwData		= reinterpret_cast<PGPUInt32>(pRemUser);

	pRemUser->tbButton.iString = (mPPrefs->showText ? 
		mToolbar.AddString(0, Get(kEditorRemUserCaption)) : NULL);

	pRemUser->index = mToolbar.AddButtons(1, &pRemUser->tbButton);

	// Add the toolbar to the rebar.
	REBARBANDINFO	rbbInfo;
	pgpClearMemory(&rbbInfo, sizeof(rbbInfo));

	rbbInfo.cbSize		= sizeof(rbbInfo);
	rbbInfo.fMask		= RBBIM_CHILD | RBBIM_STYLE;
	rbbInfo.fStyle		= RBBS_CHILDEDGE | RBBS_FIXEDSIZE;
	rbbInfo.hwndChild	= mToolbar.Handle();

	mTBRebar.InsertBand(0, rbbInfo);

	ResizeToolbar();

	// Show the toolbar.
	if (mPPrefs->showToolbar)
	{
		mTBRebar.ShowWindow(SW_SHOW);
		mToolbar.ShowWindow(SW_SHOW);
	}
}

void 
CEditorWindow::DestroyToolbar()
{
	mTBRebar.DestroyWindow();
	mImages.Clear();

	mButtons.EmptyWithDelete();
}

void 
CEditorWindow::UpdateUIState(HMENU contextMenu)
{
	using namespace UCommonStrings;

	// Get selected users.
	CPGPdiskUserSet		selUsers;
	CPGPdiskUserIter	selUsersIter(selUsers);

	mUsersView.GetSelectedUsers(&selUsers);

	CPGPdiskUser	firstUser;
	selUsersIter.Next(firstUser);

	// Disk still mounted?
	PGPBoolean	isMounted	= CPGPdiskEngine::IsPathMountedPGPdisk(mPath);

	CMenu	subMenu;

	if (IsNull(contextMenu))
	{
		// Modify toolbar.
		mToolbar.EnableButton(IDM_MOUNT, !isMounted);
		mToolbar.EnableButton(IDM_UNMOUNT, isMounted);
		mToolbar.EnableButton(IDM_ADDUSER, !isMounted);
		mToolbar.EnableButton(IDM_REMOVEUSER, !isMounted && 
			((selUsers.Count() > 1) || ((selUsers.Count() == 1) && 
			!firstUser.IsLocked() && !firstUser.IsAdminUser())));

		if (mPPrefs->showToolbar)
			mTBRebar.ShowWindow(SW_SHOW);
		else
			mTBRebar.ShowWindow(SW_HIDE);

		// Modify file menu.
		mMainMenu.GetSubMenu(kIndexFileMenu, subMenu);

		MENUITEMINFO	mii;

		mii.cbSize = sizeof(mii);
		mii.fMask = MIIM_ID;

		subMenu.GetMenuItemInfo(kMountMenuPos, TRUE, &mii);

		if (isMounted && (mii.wID == IDM_MOUNT))
		{
			subMenu.ModifyMenu(kMountMenuPos, MF_BYPOSITION | MF_ENABLED, 
				IDM_UNMOUNT, Get(kEditorUnmountMenuOption));
		}
		else if (!isMounted && (mii.wID == IDM_UNMOUNT))
		{
			subMenu.ModifyMenu(kMountMenuPos, MF_BYPOSITION | MF_ENABLED, 
				IDM_MOUNT, Get(kEditorMountMenuOption));
		}

		subMenu.EnableMenuItem(IDM_REENCRYPT, MF_BYCOMMAND | 
			(!isMounted ? MF_ENABLED : MF_GRAYED));

		// Modify view menu.
		mMainMenu.GetSubMenu(kIndexViewMenu, subMenu);

		subMenu.CheckMenuItem(IDM_VIEWTOOLBAR, MF_BYCOMMAND | 
			(mPPrefs->showToolbar ? MF_CHECKED : MF_UNCHECKED));

		subMenu.CheckMenuItem(IDM_VIEWRO, MF_BYCOMMAND | 
			((mPPrefs->diskUserColumns & CDiskUsersView::kReadOnlyColFlag) ? 
			MF_CHECKED : MF_UNCHECKED));

		subMenu.CheckMenuItem(IDM_VIEWKIND, MF_BYCOMMAND | 
			((mPPrefs->diskUserColumns & CDiskUsersView::kKindColFlag) ? 
			MF_CHECKED : MF_UNCHECKED));

		mMainMenu.GetSubMenu(kIndexUsersMenu, subMenu);
	}
	else
	{
		subMenu.Attach(contextMenu);
	}

	// Modify either Users menu or the popup menu.
	subMenu.EnableMenuItem(IDM_ADDUSER, MF_BYCOMMAND | 
		(!isMounted ? MF_ENABLED : MF_GRAYED));

	subMenu.EnableMenuItem(IDM_REMOVEUSER, MF_BYCOMMAND | 
		(!isMounted && ((selUsers.Count() > 1) || 
		((selUsers.Count() == 1) && !firstUser.IsLocked() && 
		!firstUser.IsAdminUser())) ? MF_ENABLED : MF_GRAYED));

	subMenu.EnableMenuItem(IDM_MOUNTUSING, MF_BYCOMMAND | 
		(!isMounted && (selUsers.Count() == 1) && 
		!firstUser.IsDisabled() ? MF_ENABLED : MF_GRAYED));

	subMenu.EnableMenuItem(IDM_CHANGEPASS, MF_BYCOMMAND | 
		(!isMounted && (selUsers.Count() == 1) && 
		firstUser.HasSymmetricKey() && !firstUser.IsLocked() && 
		!firstUser.IsDisabled() ? MF_ENABLED : MF_GRAYED));

	CommonString	disableString;
	PGPUInt32		disableFlags	= MF_BYCOMMAND | MF_STRING;

	if (!isMounted && (selUsers.Count() == 1) && !firstUser.IsLocked() && 
		!firstUser.IsAdminUser())
	{
		disableFlags |= MF_ENABLED;

		if (firstUser.IsDisabled())
			disableString = kEditorEnableMenuOption;
		else
			disableString = kEditorDisableMenuOption;
	}
	else
	{
		disableFlags |= MF_GRAYED;
		disableString = kEditorDisableMenuOption;
	}

	subMenu.ModifyMenu(IDM_TOGGLEDISABLE, disableFlags, IDM_TOGGLEDISABLE, 
		Get(disableString));

	subMenu.EnableMenuItem(IDM_TOGGLERO, MF_BYCOMMAND | 
		(!isMounted && (selUsers.Count() == 1) && !firstUser.IsLocked() && 
		!firstUser.IsAdminUser() && !firstUser.IsDisabled() ? 
		MF_ENABLED : MF_GRAYED));

	subMenu.EnableMenuItem(IDM_SETASADMIN, MF_BYCOMMAND | 
		(!isMounted && (selUsers.Count() == 1) && !firstUser.IsLocked() && 
		!firstUser.IsAdminUser() && !firstUser.IsDisabled() && 
		(!firstUser.HasPublicKey() || IsPrivateKeyAvailable(firstUser)) ? 
		MF_ENABLED : MF_GRAYED));

	subMenu.EnableMenuItem(IDM_LOOKUP, MF_BYCOMMAND | 
		(mUsersView.AreUnknownUsers() ? MF_ENABLED : MF_GRAYED));

	DrawMenuBar();
	ResizeUsersView();
}

PGPBoolean 
CEditorWindow::IsPrivateKeyAvailable(const CPGPdiskUser& user) const
{
	try
	{
		CPGPKeyID	keyID;
		user.GetPGPKeyID(keyID);

		CPGPKey		pgpKey(mPContext->PGPKeyDB(), keyID);
		return pgpKey.GetBooleanProp(kPGPKeyProperty_CanDecrypt);
	}
	catch (CComboError&)
	{
		return FALSE;
	}
} 

void 
CEditorWindow::ModifyUsersAux(ModifyUsersOp op)
{
	using namespace UCommonStrings;

	if (!UEditMount::CheckWarnCanModifyDisk(*mPContext, mPath, Handle(), 
		Get(kEditorMsgBoxTitle)))
	{
		THROW_PGPERROR(kPGPError_UserAbort);
	}

	// Re-export the user set.
	CPGPdiskDisk	disk;
	disk.Open(*mPContext, mPath);

	CPGPdiskUserSet		diskUsers;
	CPGPdiskUserIter	diskUsersIter(diskUsers);

	disk.ExportUserSet(diskUsers);

	// Get selected users.
	CPGPdiskUserSet		selUsers;
	CPGPdiskUserIter	selUsersIter(selUsers);

	mUsersView.GetSelectedUsers(&selUsers);

	// Get first selected user.
	CPGPdiskUser	firstUser;

	selUsersIter.Next(firstUser);
	selUsersIter.Rewind();

	// Get admin user.
	CPGPdiskUser	adminUser;

	if (!diskUsersIter.FindAdminUser(adminUser))
		THROW_PGPERROR(kPGPError_ItemNotFound);

	// Warn if admin not present.
	if (adminUser.HasPublicKey() && !IsPrivateKeyAvailable(adminUser))
	{
		CMessageBox	messageBox;
		messageBox.Display(Get(kWarnAdminUserNotUsable), 
			Get(kEditorMsgBoxTitle), Handle(), CMessageBox::kWarningTone);
	}

	// Sanity checks.
	if (selUsers.Count() == 0)
		return;

	if (op == kRemoveUsersOp)
	{
		CPGPdiskUser	selUser;

		while (selUsersIter.Next(selUser))
		{
			if (selUser.IsAdminUser() || selUser.IsLocked())
			{
				CMessageBox	messageBox;
				messageBox.Display(Get(kTellCantRemoveALUser), 
					Get(kEditorMsgBoxTitle), Handle());

				return;
			}
		}

		selUsersIter.Rewind();
	}
	else if (op == kChangePassOp)
	{
		if (selUsers.Count() != 1)
			return;

		if (!firstUser.HasSymmetricKey() || firstUser.IsDisabled())
			return;

		selUsersIter.Rewind();
	}
	else if (op == kToggleDisableOp)
	{
		if (selUsers.Count() != 1)
			return;

		if (firstUser.IsAdminUser() || firstUser.IsLocked())
			return;

		selUsersIter.Rewind();
	}
	else if (op == kToggleROOp)
	{
		if (selUsers.Count() != 1)
			return;

		if (firstUser.IsAdminUser() || firstUser.IsLocked())
			return;

		selUsersIter.Rewind();
	}
	else if (op == kSetAsAdminOp)
	{
		if (selUsers.Count() != 1)
			return;

		if (firstUser.IsAdminUser() || firstUser.IsLocked() || 
			firstUser.IsDisabled())
		{
			return;
		}

		selUsersIter.Rewind();
	}

	// Create empty user set.
	CPGPdiskUserSet	usersToAskPass;
	usersToAskPass.CreateEmpty();

	// Determine acceptable users and passphrase dialog text.
	CPGPdiskUserSet	oneUser;

	CString	passDlgSubString;
	CString	passDlgText;

	if (op == kChangePassOp)
	{
		// We can take passphrase of selected user only.
		usersToAskPass.Add(firstUser);

		passDlgText = Get(kSinglePassUserText);
		firstUser.GetUserName(passDlgSubString);
	}
	else if ((op == kRemoveUsersOp) && (selUsers.Count() == 1) && 
		(!firstUser.HasPublicKey() || IsPrivateKeyAvailable(firstUser)))
	{
		// We can take passphrase of either the admin or a specific user.
		usersToAskPass.Add(adminUser);
		usersToAskPass.Add(firstUser);

		passDlgText = Get(kSinglePassUserOrAdminText);
		firstUser.GetUserName(passDlgSubString);
	}
	else
	{
		// We can take passphrase of the admin user only.
		usersToAskPass.Add(adminUser);

		CPath	fileName;
		mPath.GetDisplayFileName(fileName);

		passDlgText = Get(kSinglePassAdminText);
		passDlgSubString = fileName;
	}

	// Ask for the passphrase of one of the target users.
	CPassphraseDialog	passDlg;
	CPGPdiskUser		userForCiphering;

	passDlg.AskForPassphrase(disk, usersToAskPass, Handle(), 
		userForCiphering, passDlgText, passDlgSubString);

	CSecureArray<char>	askedPass;
	passDlg.Passphrase().Extract(askedPass);

	// Perform the modification.
	if (op == kRemoveUsersOp)
	{
		if ((selUsers.Count() > 1) || 
			(adminUser.IsEqualUser(userForCiphering)))
		{
			// Remove the selected users.
			diskUsers.Remove(selUsers);

			// Import the updated user set.
			disk.ImportUserSet(diskUsers, userForCiphering);
		}
		else
		{
			// Remove the selected user only.
			disk.RemoveOneUser(userForCiphering);
		}
	}
	else if (op == kChangePassOp)
	{
		// Ask for a new passphrase.
		CString	userName;
		firstUser.GetUserName(userName);

		CConfirmPassphraseDialog	newPassDlg;

		newPassDlg.AskPassphrase(*mPContext, Handle(), 
			Get(kConfirmPassUserText), userName);

		CSecureArray<char>	newPass;
		newPassDlg.Passphrase().Extract(newPass);

		// Change passphrase on selected user.
		disk.ChangeOnePassphrase(userName, askedPass.Get(), newPass.Get());
	}
	else if (op == kToggleDisableOp)
	{
		// Toggle selected user  in DISK user set.
		CPGPdiskUser	diskUser;

		if (diskUsersIter.FindEqualUser(firstUser, diskUser))
			diskUser.SetIsDisabled(!diskUser.IsDisabled());

		// Import the updated user set.
		disk.ImportUserSet(diskUsers, userForCiphering);
	}
	else if (op == kToggleROOp)
	{
		// Toggle selected user in DISK user set.
		CPGPdiskUser	diskUser;

		if (diskUsersIter.FindEqualUser(firstUser, diskUser))
			diskUser.SetIsReadOnly(!diskUser.IsReadOnly());

		// Import the updated user set.
		disk.ImportUserSet(diskUsers, userForCiphering);
	}
	else if (op == kSetAsAdminOp)
	{
		// Get passphrase of selected user.
		CPassphraseDialog	firstUserPassDlg;
		CPGPdiskUser		firstUserForCiphering;
		CString				userName;

		firstUser.GetUserName(userName);
		oneUser.CreateSingleton(firstUser);

		firstUserPassDlg.AskForPassphrase(disk, oneUser, Handle(), 
			firstUserForCiphering, Get(kSinglePassUserText), userName);

		CSecureArray<char>	firstUserPass;
		firstUserPassDlg.Passphrase().Extract(firstUserPass);

		// Peform operation on users in DISK user set.
		CPGPdiskUser	firstUserDiskKeys;

		if (diskUsersIter.FindEqualUser(firstUser, firstUserDiskKeys))
			// This is just conventional and the zero ok. -wjb
			firstUserDiskKeys.EnableCiphering(disk, firstUserPass.Get(),0); 
		else
			THROW_PGPERROR(kPGPError_ItemNotFound);

		CPGPdiskUser	adminUserDiskKeys;

		if (diskUsersIter.FindEqualUser(adminUser, adminUserDiskKeys))
			// This is just conventional and the zero ok. -wjb
			adminUserDiskKeys.EnableCiphering(disk, askedPass.Get(),0 );
		else
			THROW_PGPERROR(kPGPError_ItemNotFound);

		diskUsersIter.SetAdminUser(firstUserDiskKeys, adminUserDiskKeys);

		// Import the updated user set.
		disk.ImportUserSet(diskUsers, firstUserDiskKeys);
	}

	// Update the PGPdisk users view.
	ReFillUsersView();
}

void 
CEditorWindow::ModifyUsers(ModifyUsersOp op)
{
	try
	{
		ModifyUsersAux(op);
	}
	catch (CComboError& error)
	{
		using namespace UCommonStrings;
		CommonString	errString;

		switch (op)
		{
		case kRemoveUsersOp:
			errString = kTellDiskRemoveUsersFailed;
			break;

		case kChangePassOp:
			errString = kTellDiskChangeUserPassFailed;
			break;

		case kToggleDisableOp:
			errString = kTellDiskToggleUserDisabledFailed;
			break;

		case kToggleROOp:
			errString = kTellDiskToggleUserROFailed;
			break;

		case kSetAsAdminOp:
			errString = kTellDiskSetAdminUserFailed;
			break;
		}

		ReportError(errString, kEditorMsgBoxTitle, error, Handle());
	}
}

void 
CEditorWindow::OnAddUser()
{
	using namespace UCommonStrings;

	try
	{
		if (!UEditMount::CheckWarnCanModifyDisk(*mPContext, mPath, Handle(), 
			Get(kEditorMsgBoxTitle)))
		{
			THROW_PGPERROR(kPGPError_UserAbort);
		}

		// Get admin user.
		CPGPdiskDisk	disk;
		disk.Open(*mPContext, mPath);

		CPGPdiskUserSet		diskUsers;
		CPGPdiskUserIter	diskUsersIter(diskUsers);
		CPGPdiskUser		adminUser;

		disk.ExportUserSet(diskUsers);

		if (!diskUsersIter.FindAdminUser(adminUser))
			THROW_PGPERROR(kPGPError_ItemNotFound);

		// Bail if admin not present.
		if (adminUser.HasPublicKey() && !IsPrivateKeyAvailable(adminUser))
		{
			CMessageBox	messageBox;
			messageBox.Display(Get(kTellAdminUserNotUsable), 
				Get(kEditorMsgBoxTitle), Handle());

			THROW_PGPERROR(kPGPError_UserAbort);
		}

		disk.Close();
	
		// Show the wizard.
		CNewUserWizard	newUserWiz;
		newUserWiz.Display(*mPContext, mPath, Handle());

		ReFillUsersView();
	}
	catch (CComboError& error)
	{
		using namespace UCommonStrings;

		ReportError(kTellDiskAddUserFailed, kEditorMsgBoxTitle, error, 
			Handle());
	}
}

void 
CEditorWindow::OnDiskOptions()
{
	try
	{
		PGPError	error	= PGPclPreferences(mPContext->PGPContext(), 
			Handle(), kPGPclPGPdiskPrefs, mPContext->PGPKeyDB());
		THROW_IF_PGPERROR(error);
	}
	catch (CComboError& error)
	{
		using namespace UCommonStrings;

		ReportError(kTellDiskShowOptionsFailed, kEditorMsgBoxTitle, error, 
			Handle());
	}
}

void 
CEditorWindow::OnDiskProps()
{
	try
	{
		CDiskProperties	diskProps;
		diskProps.Display(mPContext, mPath, Handle());

		if (diskProps.WasReEncryptionBegun())
		{
			if (diskProps.WasReEncryptionCompleted())
				ReFillUsersView();
			else
				Close();
		}
	}
	catch (CComboError& error)
	{
		using namespace UCommonStrings;

		ReportError(kTellDiskShowPropsFailed, kEditorMsgBoxTitle, error, 
			Handle());
	}
}

void 
CEditorWindow::OnMountDiskUsing()
{
	using namespace UCommonStrings;

	try
	{
		// Get selected users.
		CPGPdiskUserSet	selUsers;
		mUsersView.GetSelectedUsers(&selUsers);

		// Get first user.
		CPGPdiskUserIter	selUsersIter(selUsers);
		CPGPdiskUser		selUser;

		selUsersIter.Next(selUser);

		// Sanity checks.
		if ((selUsers.Count() != 1) || selUser.IsDisabled())
		{
			return;
		}
		else if ((selUser.HasPublicKey() && !IsPrivateKeyAvailable(selUser)))
		{
			CMessageBox	messageBox;
			messageBox.Display(Get(kTellCantMountWOPrivKey), 
				Get(kEditorMsgBoxTitle), Handle());

			return;
		}

		// Mount using this user.
		CPGPdiskMounter	mounter;
		mounter.AskMount(*mPContext, mPath, Handle(), &selUser);
	}
	catch (CComboError& error)
	{
		ReportError(kTellDiskMountFailed, kEditorMsgBoxTitle, error, 
			Handle());
	}
}

void 
CEditorWindow::OnReEncryptDisk()
{
	using namespace UCommonStrings;

	PGPBoolean	reEncryptBegun	= FALSE;

	try
	{
		if (!UEditMount::CheckWarnCanModifyDisk(*mPContext, mPath, Handle(), 
			Get(kEditorMsgBoxTitle)))
		{
			THROW_PGPERROR(kPGPError_UserAbort);
		}

		CReEncryptWizard	reEncryptWiz;
		reEncryptWiz.Display(*mPContext, mPath, Handle(), &reEncryptBegun);

		ReFillUsersView();
	}
	catch (CComboError& error)
	{
		ReportError(kTellDiskReEncryptFailed, kEditorMsgBoxTitle, error, 
			Handle());

		// Failed/interrupted conversions result in window close.
		if (reEncryptBegun)
			Close();
	}
}

void 
CEditorWindow::OnUnmountDisk()
{
	try
	{
		CPGPdiskUnmounter	diskUnmounter;
		diskUnmounter.AskUnmount(*mPContext, mPath);

		UpdateUIState();
	}
	catch (CComboError& error)
	{
		using namespace UCommonStrings;

		ReportError(kTellDiskUnmountFailed, kEditorMsgBoxTitle, error, 
			Handle());
	}
}

BOOL 
CEditorWindow::OnCommand(
	PGPUInt16	notifyCode, 
	PGPUInt16	itemId, 
	HWND		ctrl)
{
	using namespace UCommonStrings;

	switch (itemId)
	{
	case IDM_MOUNT:
		try
		{
			CPGPdiskMounter	mounter;
			mounter.AskMount(*mPContext, mPath, Handle());
		}
		catch (CComboError& error)
		{
			ReportError(kTellDiskMountFailed, kEditorMsgBoxTitle, error, 
				Handle());
		}
		return TRUE;

	case IDM_UNMOUNT:
		OnUnmountDisk();
		return TRUE;

	case IDM_REENCRYPT:
		OnReEncryptDisk();
		return TRUE;

	case IDM_DISKPROPS:
		OnDiskProps();
		return TRUE;
	
	case IDM_DISKOPTIONS:
		OnDiskOptions();
		return TRUE;

	case IDM_EXIT:
		Close();
		return TRUE;

	case IDM_ADDUSER:
		OnAddUser();
		return TRUE;

	case IDM_REMOVEUSER:
		ModifyUsers(kRemoveUsersOp);
		return TRUE;

	case IDM_MOUNTUSING:
		OnMountDiskUsing();
		return TRUE;

	case IDM_CHANGEPASS:
		ModifyUsers(kChangePassOp);
		return TRUE;

	case IDM_SETASADMIN:
		ModifyUsers(kSetAsAdminOp);
		return TRUE;

	case IDM_TOGGLEDISABLE:
		ModifyUsers(kToggleDisableOp);
		return TRUE;

	case IDM_TOGGLERO:
		ModifyUsers(kToggleROOp);
		return TRUE;

	case IDM_VIEWRO:
	{
		CMenu	subMenu;
		mMainMenu.GetSubMenu(kIndexViewMenu, subMenu);

		mPPrefs->diskUserColumns ^= CDiskUsersView::kReadOnlyColFlag;

		mUsersView.SetActiveColumns(mPPrefs->diskUserColumns);
		UpdateUIState();
		return TRUE;
	}

	case IDM_VIEWKIND:
	{
		CMenu	subMenu;
		mMainMenu.GetSubMenu(kIndexViewMenu, subMenu);

		mPPrefs->diskUserColumns ^= CDiskUsersView::kKindColFlag;

		mUsersView.SetActiveColumns(mPPrefs->diskUserColumns);
		UpdateUIState();

		return TRUE;
	}

	case IDM_VIEWTOOLBAR:
	{
		CMenu	subMenu;
		mMainMenu.GetSubMenu(kIndexViewMenu, subMenu);

		mPPrefs->showToolbar = !mPPrefs->showToolbar;
		UpdateUIState();

		return TRUE;
	}

	case IDM_HELP_CONTENTS:
		CHelpShower::ShowContents(Handle());
		return TRUE;

	case IDM_ABOUTPGP:
		PGPclHelpAbout(mPContext->PGPContext(), Handle(), NULL, NULL, NULL);
		return TRUE;

	case IDM_LOOKUP:
		mUsersView.LookupUnknownKeys();
		break;
	}

	return CWindow::OnCommand(notifyCode, itemId, ctrl);
}

void 
CEditorWindow::OnContextMenu(HWND menuWnd, CPoint coords)
{
	if (menuWnd == mTBRebar.Handle())
	{
		CMenu	menu(UModuleInstance::Get(), 
			MAKEINTRESOURCE(IDR_TOOLBAR_MENU));
		CMenu	subMenu;

		menu.GetSubMenu(0, subMenu);

		subMenu.CheckMenuItem(IDM_LARGEBUTTONS, MF_BYCOMMAND | 
			(mPPrefs->largeButtons ? MF_CHECKED : MF_UNCHECKED));
		subMenu.CheckMenuItem(IDM_SHOWTEXT, MF_BYCOMMAND | 
			(mPPrefs->showText ? MF_CHECKED : MF_UNCHECKED));
		subMenu.CheckMenuItem(IDM_SHOWTOOLTIPS, MF_BYCOMMAND | 
			(mPPrefs->toolTips ? MF_CHECKED : MF_UNCHECKED));

		PGPUInt32	cmdID	= subMenu.TrackPopupMenu(TPM_NONOTIFY | 
			TPM_RETURNCMD | TPM_RIGHTBUTTON, coords.X(), coords.Y(), 
			Handle());

		switch (cmdID)
		{
		case IDM_LARGEBUTTONS:
			mPPrefs->largeButtons = !mPPrefs->largeButtons;
			break;

		case IDM_SHOWTEXT:
			mPPrefs->showText = !mPPrefs->showText;
			break;

		case IDM_SHOWTOOLTIPS:
			mPPrefs->toolTips = !mPPrefs->toolTips;
			break;
		}

		DestroyToolbar();
		CreateToolbar();
		UpdateUIState();
	}
	else if (menuWnd == mUsersView.Handle())
	{
		CMenu	menu(UModuleInstance::Get(), 
			MAKEINTRESOURCE(IDR_DISKUSERS_MENU));

		CMenu	subMenu;
		menu.GetSubMenu(0, subMenu);

		UpdateUIState(subMenu);

		subMenu.TrackPopupMenu(TPM_RIGHTBUTTON, coords.X(), coords.Y(), 
			Handle());
	}
	else
	{
		CWindow::OnContextMenu(menuWnd, coords);
	}
}

PGPInt32 
CEditorWindow::OnCreate(LPCREATESTRUCT pCS)
{
	PGPInt32	result	= CWindow::OnCreate(pCS);

	if (result == -1)
		return result;

	PGPclEnsureWindowIsOnMonitor(Handle(), NULL);

	CreateStatusBar();
	CreateToolbar();
	CreateUsersView();
	UpdateUIState();

	mTimerID = SetTimer(Handle(), 1, kTimerMs, NULL);

	return 0;
}

void 
CEditorWindow::OnDestroy()
{
	KillTimer(Handle(), mTimerID);

	DestroyToolbar();
	mMainMenu.Clear();

	try
	{
		WriteWindowPrefs();
	}
	catch (CComboError&) { }

	CWindow::OnDestroy();
}

void 
CEditorWindow::OnGetMinMaxInfo(LPMINMAXINFO pMMI)
{
	pMMI->ptMinTrackSize.x = kMinWidth;
	pMMI->ptMinTrackSize.y = kMinHeight;
}

void 
CEditorWindow::OnHelp(HELPINFO *pInfo)
{
	CHelpShower::ShowContents(Handle());
}

void 
CEditorWindow::OnMenuSelect(PGPUInt16 ctrlId, PGPUInt16 flags, HMENU hwndMenu)
{
	CWindow::OnMenuSelect(ctrlId, flags, hwndMenu);
	ShowStatusBarHelp(ctrlId);
}

PGPUInt32 
CEditorWindow::OnNotify(PGPUInt16 ctrlId, LPNMHDR pNMHDR)
{
	switch (pNMHDR->code)
	{
	case LVN_ITEMCHANGED:
	{
		UpdateUIState();
	}
	break;
	
	case LVN_KEYDOWN:
	{
		NMLVKEYDOWN	*pLVKD	= reinterpret_cast<NMLVKEYDOWN *>(pNMHDR);

		switch (pLVKD->wVKey)
		{
		case VK_DELETE:
			if (mUsersView.GetSelectedCount() > 0)
				ModifyUsers(kRemoveUsersOp);
			break;
		}
	}
	break;
	
	case NM_DBLCLK:
	{
		NMITEMACTIVATE	*pNMIA	= reinterpret_cast<NMITEMACTIVATE *>(pNMHDR);

		if (mUsersView.GetSelectedCount() == 1)
			OnMountDiskUsing();
	}
	break;
	
	case TBN_HOTITEMCHANGE:
	{
		NMTBHOTITEM	*pTBHI	= reinterpret_cast<NMTBHOTITEM *>(pNMHDR);

		if (pTBHI->dwFlags & HICF_MOUSE)
			ShowStatusBarHelp(pTBHI->idNew);
	}
	break;
		
	case TTN_GETDISPINFO:
	{
		using namespace UCommonStrings;
		NMTTDISPINFO	*pTTDI	= reinterpret_cast<NMTTDISPINFO *>(pNMHDR);

		switch (pNMHDR->idFrom)
		{
		case IDM_MOUNT:
			pTTDI->lpszText = const_cast<char *>(Get(
				kEditorMountTooltip));
			break;

		case IDM_UNMOUNT:
			pTTDI->lpszText = const_cast<char *>(Get(
				kEditorUnmountTooltip));
			break;

		case IDM_DISKPROPS:
			pTTDI->lpszText = const_cast<char *>(
				Get(kEditorPropertiesTooltip));
			break;

		case IDM_ADDUSER:
			pTTDI->lpszText = const_cast<char *>(
				Get(kEditorAddUserTooltip));
			break;

		case IDM_REMOVEUSER:
			pTTDI->lpszText = const_cast<char *>(
				Get(kEditorRemUserTooltip));
			break;

		default:
			pTTDI->lpszText = "";
			break;
		}
	}
	break;

	default:
		return CWindow::OnNotify(ctrlId, pNMHDR);
	}

	return 0;
}

void 
CEditorWindow::OnSetFocus()
{
	CWindow::OnSetFocus();
	mUsersView.SetFocus();
}

void 
CEditorWindow::OnSize(PGPUInt16 flags, PGPUInt16 width, PGPUInt16 height)
{
	CWindow::OnSize(flags, width, height);

	mStatusBar.SendMessage(WM_SIZE, CurMsg().WParam(), CurMsg().LParam());

	ResizeToolbar();
	ResizeUsersView();
}

void 
CEditorWindow::OnTimer(PGPUInt32 timerID, TIMERPROC *timerProc)
{
	CWindow::OnTimer(timerID, timerProc);
	UpdateUIState();
}
