/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CEditorWindow.h,v 1.6 2002/08/06 20:09:23 dallen Exp $
____________________________________________________________________________*/

#ifndef Included_CEditorWindow_h	// [
#define Included_CEditorWindow_h

#include "CList.h"

#include "CAccelerator.h"
#include "CCursor.h"
#include "CIcon.h"
#include "CImageList.h"
#include "CMenu.h"
#include "CPath.h"
#include "CRebar.h"
#include "CStatusBar.h"
#include "CToolbar.h"
#include "CWindow.h"

#include "CDiskUsersView.h"

_PGP_BEGIN

// Types

class CPGPdiskContext;
class CPGPdiskDiskIter;
class CPGPdiskDiskSet;
class CPGPdiskUserSet;


// Class CEditorWindow

class CEditorWindow : public CWindow
{
private:
	enum ModifyUsersOp {kRemoveUsersOp, kChangePassOp, kToggleDisableOp, 
		kToggleROOp, kSetAsAdminOp};

	struct EditorWindowPrefs;
	struct ToolbarButton;

public:
	CEditorWindow();
	~CEditorWindow();

	void	Display(const CPGPdiskContext& context, const char *path, 
		HWND owner);

	static void	InitClass();
	static void	CleanupClass();

private:
	static CCursor	WindowCursor;
	static CIcon	WindowIcon;

	CAccelerator	mAccel;
	CDiskUsersView	mUsersView;
	CMenu			mMainMenu;
	CRebar			mTBRebar;
	CStatusBar		mStatusBar;
	CToolbar		mToolbar;

	const CPGPdiskContext	*mPContext;
	CPath					mPath;

	CList<ToolbarButton>	mButtons;
	CImageList				mImages;

	EditorWindowPrefs	*mPPrefs;
	PGPUInt32			mTimerID;

	void	ReadWindowPrefs();
	void	WriteWindowPrefs() const;

	void	ShowStatusBarHelp(PGPUInt16 ctrlId);

	void	ResizeUsersView();
	void	ResizeToolbar();

	void	ReFillUsersView();

	void	CreateUsersView();
	void	CreateStatusBar();
	void	CreateToolbar();
	void	DestroyToolbar();

	void		UpdateUIState(HMENU contextMenu = NULL);
	PGPBoolean	CheckWarnCanModifyUsers(PGPBoolean silently = FALSE) const;
	PGPBoolean	IsPrivateKeyAvailable(const CPGPdiskUser& user) const;

	void	ModifyUsersAux(ModifyUsersOp op);
	void	ModifyUsers(ModifyUsersOp op);

	void	OnAddUser();
	void	OnDiskOptions();
	void	OnDiskProps();
	void	OnMountDiskUsing();
	void	OnReEncryptDisk();
	void	OnUnmountDisk();

	BOOL		OnCommand(PGPUInt16 notifyCode, PGPUInt16 itemId, HWND ctrl);
	void		OnContextMenu(HWND menuWnd, CPoint coords);
	PGPInt32	OnCreate(LPCREATESTRUCT pCS);
	void		OnDestroy();
	void		OnGetMinMaxInfo(LPMINMAXINFO pMMI);
	void		OnHelp(HELPINFO *pInfo);
	void		OnMenuSelect(PGPUInt16 ctrlId, PGPUInt16 flags, 
		HMENU hwndMenu);
	PGPUInt32	OnNotify(PGPUInt16 ctrlId, LPNMHDR pNMHDR);
	void		OnSetFocus();
	void		OnSize(PGPUInt16 flags, PGPUInt16 width, PGPUInt16 height);
	void		OnTimer(PGPUInt32 timerID, TIMERPROC *timerProc);
};

_PGP_END

#endif	// ] Included_CEditorWindow_h
