/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CNewDiskWizard.cpp,v 1.43 2002/11/05 02:10:53 wjb Exp $
____________________________________________________________________________*/

#include "pgpClassesConfig.h"
#include <stdlib.h>

#include "CFile.h"
#include "CMutex.h"
#include "CThread.h"

#include "CDC.h"
#include "CIcon.h"
#include "CImageList.h"
#include "CMessageBox.h"
#include "CPath.h"
#include "CProgressBar.h"
#include "CSaveFileDialog.h"
#include "CUpDownControl.h"
#include "UModuleInstance.h"
#include "USecurity.h"
#include "UWinVersion.h"

#include "pgpClientErrors.h"
#include "UCommonStrings.h"
#include "UFileSys.h"
#include "CPGPclPrefs.h"
#include "CPGPdiskEngine.h"
#include "CPGPGlobalRandomPool.h"
#include "CPGPKey.h"
#include "CPGPKeyDB.h"

#include "PGPdiskUIPriv.h"

#include "CAlgorithmCombo.h"
#include "CCreateScaleCombo.h"
#include "CDriveLetterCombo.h"
#include "CFileSystemCombo.h"
#include "CHelpShower.h"
#include "CNewUserPages.h"
#include "CNewDiskWizard.h"
#include "CPassphraseDialog.h"
#include "CPercentDone.h"
#include "CPGPdiskFormatter.h"
#include "CPGPdiskPage.h"
#include "CRandomDataPage.h"
#include "ImageIDs.h"
#include "PGPdiskHelp.h"
#include "Resource.h"
#include "UEditMount.h"
#include "UMountPoint.h"

#include "CUTF8Edit.h"

_USING_PGP

_UNNAMED_BEGIN

// Constants

const double	kSizeFudgeFraction	= 0.95;

enum
{
	kScaleStepFactor	= 10
};

DWORD	NewDiskWizADKPageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_DISKSTATIC3,		-1, 
	IDC_HEADING,			-1, 
	IDC_ADK_CAPTION,		IDH_PGPDISKUI_DISKWIZADKPAGE_ADKUSERID, 
	IDC_ADK_EDIT,			IDH_PGPDISKUI_DISKWIZADKPAGE_ADKUSERID, 
	0,						0
};

DWORD	NewDiskWizCreationPageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_DISKSTATIC3,		-1, 
	IDC_DISKSTATIC4,		-1, 
	IDC_DISKSTATIC5,		-1, 
	IDC_HEADING,			-1, 
	IDC_TASKICON1,			-1, 
	IDC_TASKICON2,			-1, 
	IDC_CURTASK_CAPTION,	IDH_PGPDISKUI_DISKWIZCREATEPAGE_CURTASK, 
	IDC_CURTASK_TEXT,		IDH_PGPDISKUI_DISKWIZCREATEPAGE_CURTASK, 
	IDC_CURTASK_BAR,		IDH_PGPDISKUI_DISKWIZCREATEPAGE_CURTASK, 
	IDC_ALLTASK_CAPTION,	IDH_PGPDISKUI_DISKWIZCREATEPAGE_ALLTASK, 
	IDC_ALLTASK_BAR,		IDH_PGPDISKUI_DISKWIZCREATEPAGE_ALLTASK, 
	0,						0
};

DWORD	NewDiskWizDonePageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_DISKSTATIC3,		-1, 
	IDC_DISKSTATIC4,		-1, 
	IDC_DISKSTATIC5,		-1, 
	IDC_HEADING,			-1, 
	IDC_SIDEBAR,			-1, 
	0,						0
};

DWORD	NewDiskWizIntroPageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_DISKSTATIC3,		-1, 
	IDC_DISKSTATIC4,		-1, 
	IDC_HEADING,			-1, 
	IDC_SIDEBAR,			-1, 
	0,						0
};

DWORD	NewDiskWizOptionsDlgHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_DISKSTATIC3,		-1, 
	IDC_DRIVE_RADIO,		IDH_PGPDISKUI_DISKWIZOPTSDLG_DRIVE, 
	IDC_DRIVE_COMBO,		IDH_PGPDISKUI_DISKWIZOPTSDLG_DRIVE, 
	IDC_DIR_RADIO,			IDH_PGPDISKUI_DISKWIZOPTSDLG_DIR, 
	IDC_DIR_EDIT,			IDH_PGPDISKUI_DISKWIZOPTSDLG_DIR, 
	IDC_BROWSE_BUTTON,		IDH_PGPDISKUI_DISKWIZOPTSDLG_BROWSEDIR, 
	IDC_ALGORITHM_CAPTION,	IDH_PGPDISKUI_DISKWIZOPTSDLG_ALGORITHM, 
	IDC_ALGORITHM_COMBO,	IDH_PGPDISKUI_DISKWIZOPTSDLG_ALGORITHM, 
	IDC_FILESYSTEM_CAPTION,	IDH_PGPDISKUI_DISKWIZOPTSDLG_FILESYSTEM, 
	IDC_FILESYSTEM_COMBO,	IDH_PGPDISKUI_DISKWIZOPTSDLG_FILESYSTEM, 
	IDC_STARTUP_CHECK,		IDH_PGPDISKUI_DISKWIZOPTSDLG_STARTUP, 
	IDOK,					IDH_PGPDISKUI_DISKWIZOPTSDLG_OK, 
	IDCANCEL,				IDH_PGPDISKUI_DISKWIZOPTSDLG_CANCEL, 
	0,						0
};

DWORD	NewDiskWizVolInfoPageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_DISKSTATIC3,		-1, 
	IDC_DISKSTATIC4,		-1, 
	IDC_HEADING,			-1, 
	IDC_SIDEBAR,			-1, 
	IDC_FREESIZE_TEXT,		-1, 
	IDC_PATH_CAPTION,		IDH_PGPDISKUI_DISKWIZVOLINFOPAGE_PATH, 
	IDC_PATH_EDIT,			IDH_PGPDISKUI_DISKWIZVOLINFOPAGE_PATH, 
	IDC_BROWSE_BUTTON,		IDH_PGPDISKUI_DISKWIZVOLINFOPAGE_BROWSEPATH, 
	IDC_SIZE_CAPTION,		IDH_PGPDISKUI_DISKWIZVOLINFOPAGE_SIZE, 
	IDC_SIZE_EDIT,			IDH_PGPDISKUI_DISKWIZVOLINFOPAGE_SIZE, 
	IDC_SIZE_UDCTRL,		IDH_PGPDISKUI_DISKWIZVOLINFOPAGE_SIZE, 
	IDC_SCALE_COMBO,		IDH_PGPDISKUI_DISKWIZVOLINFOPAGE_SIZE, 
	IDC_OPTIONS_BUTTON,		IDH_PGPDISKUI_DISKWIZVOLINFOPAGE_ADVOPTS, 
	0,						0
};


// Types

class CNewDiskWizRandomPage;

struct DiskWizardInfo
{
	CComboError	error;

	CNewUserPages	*pNewUserPages;
	CPath			path;
	PGPUInt64		blocks;
	PGPBoolean		mountAtStartup;

	const CPGPdiskContext		*pContext;
	CPGPGlobalRandomPool		*pRandomPool;
	PGPdiskEncryptionAlgorithm	algorithm;
	FileSys::Type				fileSys;

	PGPBoolean	fileCreated;
	CPath		root;

	// Data from CNewUserPages::Finish(...).
	PGPBoolean		userChosePub;
	CPGPKey			pubKey[100];
	PGPUInt32		uNumPubKeys;
	CString			userName;
	CSecureString	securePass;
};


// Class CNewDiskOptionsDialog

class CNewDiskOptionsDialog : public CDialog
{
public:
	CNewDiskOptionsDialog() { }
	~CNewDiskOptionsDialog() { }

	PGPUInt16	GetDialogId() const
	{
		return IDD_DISKWIZ_OPTIONS_DLG;
	}

	void	Display(HWND parentWnd, PGPUInt64 blocks, 
		PGPdiskEncryptionAlgorithm& algorithm, FileSys::Type& fileSys, 
		PGPUInt8& drive, CPath& mountPoint, PGPBoolean& choseDrive, 
		PGPBoolean& mountStartup);

private:
	CAlgorithmCombo		mAlgoCombo;
	CButton				mBrowseButton;
	CButton				mDriveRadio;
	CButton				mDirRadio;
	CButton				mStartupCheck;
	CDriveLetterCombo	mDriveCombo;
	CUTF8Edit				mDirEdit;
	CFileSystemCombo	mFileSysCombo;

	PGPUInt64	mBlocks;

	PGPdiskEncryptionAlgorithm	mAlgorithm;
	FileSys::Type				mFileSys;

	PGPUInt8	mDrive;
	CPath		mMountPoint;
	PGPBoolean	mChoseDrive;
	PGPBoolean	mMountStartup;

	void	InitControls();

	void	OnBrowseButton();
	void	OnDirCheck();
	void	OnDriveCheck();
	void	OnOk();

	BOOL	OnCommand(PGPUInt16 notifyCode, PGPUInt16 itemId, HWND ctrl);
	void	OnContextMenu(HWND menuWnd, CPoint coords);
	void	OnHelp(HELPINFO *pInfo);
	BOOL	OnInitDialog(HWND focusCtrl);
};


// Class CNewDiskWizADKPage

class CNewDiskWizADKPage : public CPGPdiskPage
{
public:
	CNewDiskWizADKPage(DiskWizardInfo *pInfo) : mPInfo(pInfo)
	{
		pgpAssertAddrValid(pInfo, DiskWizardInfo);
	}

	~CNewDiskWizADKPage()
	{
	}

	PGPUInt16	GetDialogId() const
	{
		return IDD_DISKWIZ_ADK;
	}
	
private:
	DiskWizardInfo	*mPInfo;

	CUTF8Edit	mADKEdit;

	DWORD	GetPageHelpId() const {return IDH_PGPDISKUI_DISKWIZADKPAGE;}
	PVOID	GetPageHelpIds() const {return NewDiskWizADKPageHelpIDs;}

	BOOL		OnInitDialog(HWND focusCtrl);
	PGPInt32	OnPsnSetActive();
};


// Class CNewDiskWizCreationPage

class CNewDiskWizCreationPage : public CPGPdiskPage
{
private:
	enum
	{
		WM_CREATE_HALFWAYDONE	= WM_USER + 10, 
		WM_CREATE_DONE			= WM_USER + 11
	};

public:
	CNewDiskWizCreationPage(DiskWizardInfo *pInfo) : mPInfo(pInfo)
	{
		pgpAssertAddrValid(pInfo, DiskWizardInfo);
	}

	~CNewDiskWizCreationPage()
	{
	}

	PGPUInt16	GetDialogId() const
	{
		return IDD_DISKWIZ_CREATION;
	}
	
private:
	DiskWizardInfo	*mPInfo;

	CIcon			mDoingIcon;
	CIcon			mDoneIcon;
	CIcon			mFailedIcon;
	CImageList		mImages;
	CProgressBar	mCurTaskBar;
	CProgressBar	mAllTaskBar;
	CStatic			mCurTaskText;
	CStatic			mTaskIcon1;
	CStatic			mTaskIcon2;

	CPath	mFileName;
	CThread	mCreateThread;
	CMutex	mCancelMutex;

	PGPBoolean	mShouldCancel;

	DWORD	GetPageHelpId() const {return IDH_PGPDISKUI_DISKWIZCREATEPAGE;}
	PVOID	GetPageHelpIds() const {return NewDiskWizCreationPageHelpIDs;}

	PGPBoolean			CreateStatusCallbackAux(PGPBoolean areFormatting, 
		PGPUInt32 percent);
	static PGPBoolean	CreateStatusCallback(PGPBoolean areFormatting, 
		PGPUInt32 percent, void *userVal);

	void				CreateCallbackAux();
	static void _cdecl	CreateCallback(void *userVal);

	void		InitImages();
	void		OnCreateDone();
	void		OnCreateHalfwayDone();
	BOOL		OnInitDialog(HWND focusCtrl);
	PGPBoolean	OnPsnQueryCancel();
	PGPInt32	OnPsnSetActive();
	PGPInt32	OnPsnWizNext();

	PGPBoolean	PreProcess(const CMessage& msg, LRESULT& result);
};


// Class CNewDiskWizDonePage

class CNewDiskWizDonePage : public CPGPdiskPage
{
public:
	CNewDiskWizDonePage(DiskWizardInfo *pInfo) : mPInfo(pInfo)
	{
		pgpAssertAddrValid(pInfo, DiskWizardInfo);
	}

	~CNewDiskWizDonePage()
	{
	}

	PGPUInt16	GetDialogId() const
	{
		return IDD_DISKWIZ_DONE;
	}

private:
	DiskWizardInfo	*mPInfo;

	DWORD	GetPageHelpId() const {return IDH_PGPDISKUI_DISKWIZDONEPAGE;}
	PVOID	GetPageHelpIds() const {return NewDiskWizDonePageHelpIDs;}

	PGPInt32	OnPsnSetActive();
};


// Class CNewDiskWizIntroPage

class CNewDiskWizIntroPage : public CPGPdiskPage
{
public:
	CNewDiskWizIntroPage(DiskWizardInfo *pInfo) : mPInfo(pInfo)
	{
		  pgpAssertAddrValid(pInfo, DiskWizardInfo);
	}

	~CNewDiskWizIntroPage()
	{
	}
	
	PGPUInt16	GetDialogId() const
	{
		return IDD_DISKWIZ_INTRO;
	}

private:
	DiskWizardInfo	*mPInfo;

	DWORD	GetPageHelpId() const {return IDH_PGPDISKUI_DISKWIZINTROPAGE;}
	PVOID	GetPageHelpIds() const {return NewDiskWizIntroPageHelpIDs;}

	BOOL		OnInitDialog(HWND focusCtrl);
	PGPInt32	OnPsnSetActive();
};


// Class CNewDiskWizVolInfoPage

class CNewDiskWizVolInfoPage : public CPGPdiskPage
{
private:
	enum {kTypicalBlocksHeaders = 8};

public:
	CNewDiskWizVolInfoPage::CNewDiskWizVolInfoPage(DiskWizardInfo *pInfo) : 
	  mPInfo(pInfo)
	{
		pgpAssertAddrValid(pInfo, DiskWizardInfo);
	}

	CNewDiskWizVolInfoPage::~CNewDiskWizVolInfoPage()
	{
	}

	PGPUInt16	GetDialogId() const
	{
		return IDD_DISKWIZ_VOLINFO;
	}

	void	SavePrefs();

private:
	DiskWizardInfo	*mPInfo;

	CButton				mOptionsButton;
	CCreateScaleCombo	mScaleCombo;
	CUTF8Edit				mPathEdit;
	CEdit				mSizeEdit;
	CStatic				mFreeSizeText;
	CUpDownControl		mSizeUDCtrl;

	PGPUInt32					mSizeValue;
	PGPdiskUIPriv::CreateScale	mCreateScale;
	PGPUInt64					mKbSize;

	PGPUInt8	mDrive;
	CPath		mMountPoint;
	PGPBoolean	mChoseDrive;

	PGPUInt64	mKbFree;
	PGPUInt64	mPlatformMaxKbSize;
	PGPUInt64	mPlatformMinKbSize;
	PGPUInt64	mLocalMaxKbSize;
	PGPUInt64	mLocalMinKbSize;

	PGPBoolean	mAreWeFailingDiskFree;
	PGPBoolean	mUserChangedOptions;

	DWORD	GetPageHelpId() const {return IDH_PGPDISKUI_DISKWIZVOLINFOPAGE;}
	PVOID	GetPageHelpIds() const {return NewDiskWizVolInfoPageHelpIDs;}

	void		FormatSizeText(PGPUInt64 kbSize, CString& text) const;
	PGPUInt32	FudgeSizeValue(PGPUInt32 size, 
		PGPBoolean scale = FALSE) const;

	void	SizeScaleToKb(PGPUInt32 sizeValue, 
		PGPdiskUIPriv::CreateScale createScale, PGPUInt64& kbSize);

	void	KbToSizeScale(PGPUInt64 kbSize);

	void	ExportValuesToUI();
	void	ImportValuesFromUI();

	PGPBoolean			SaveDialogValidateCallbackAux(const CPath& path);
	static PGPBoolean	SaveDialogValidateCallback(const CPath& path, 
		void *userVal);

	PGPBoolean	ValidateDiskPath();

	PGPBoolean	FixWarnOptions(PGPBoolean silently = FALSE);
	PGPBoolean	FixWarnSizeAndScale(PGPBoolean silently = FALSE);
	void		UpdateMinMaxSizeFromPath(const CPath& path);
	void		InitControls();

	void	OnOptionsButton();
	void	OnBrowseButton();

	BOOL		OnCommand(PGPUInt16 notifyCode, PGPUInt16 itemId, HWND ctrl);
	BOOL		OnInitDialog(HWND focusCtrl);
	PGPInt32	OnPsnSetActive();
	PGPInt32	OnPsnWizNext();
};

_UNNAMED_END

// Class CNewDiskOptionsDialog member functions

void 
CNewDiskOptionsDialog::Display(
	HWND						parentWnd, 
	PGPUInt64					blocks, 
	PGPdiskEncryptionAlgorithm&	algorithm, 
	FileSys::Type&				fileSys, 
	PGPUInt8&					drive, 
	CPath&						mountPoint, 
	PGPBoolean&					choseDrive, 
	PGPBoolean&					mountStartup)
{
	mBlocks			= blocks;
	mAlgorithm		= algorithm;
	mFileSys		= fileSys;
	mDrive			= drive;
	mMountPoint		= mountPoint;
	mChoseDrive		= choseDrive;
	mMountStartup	= mountStartup;

	CreateModal(parentWnd);

	algorithm		= mAlgorithm;
	fileSys			= mFileSys;
	drive			= mDrive;
	mountPoint		= mMountPoint;
	choseDrive		= mChoseDrive;
	mountStartup	= mMountStartup;
}

void 
CNewDiskOptionsDialog::InitControls()
{
	mAlgoCombo.Fill();
	mAlgoCombo.Set(mAlgorithm);

	mDriveCombo.Fill();
	mDriveCombo.Set(mDrive);

	mFileSysCombo.Fill(mBlocks);
	mFileSysCombo.Set(mFileSys);

	mStartupCheck.SetCheck(mMountStartup ? BST_CHECKED : BST_UNCHECKED);

	if (mChoseDrive)
		OnDriveCheck();
	else
		OnDirCheck();

	if (UWinVersion::IsWin2000Compatible())
	{
		if (USecurity::IsRemoteSession())
		{
			mDriveRadio.EnableWindow(FALSE);
			mDriveCombo.EnableWindow(FALSE);

			mChoseDrive = FALSE;
			OnDirCheck();
		}

		mDirEdit.SetWindowText(mMountPoint);
	}
	else
	{
		mDirRadio.EnableWindow(FALSE);
		mDirEdit.EnableWindow(FALSE);
		mBrowseButton.EnableWindow(FALSE);
	}
}

void 
CNewDiskOptionsDialog::OnBrowseButton()
{
	using namespace UCommonStrings;

	try
	{
		UMountPoint::AskChooseMountPoint(Handle(), 
			Get(kCreateOptionsMsgBoxTitle), mMountPoint);

		mMountPoint.EndInSlash();
		mDirEdit.SetWindowText(mMountPoint);
	}
	catch (CComboError& error)
	{
		if (error.pgpErr != kPGPError_UserAbort)
			throw;
	}
}

void 
CNewDiskOptionsDialog::OnDirCheck()
{
	mDriveRadio.SetCheck(BST_UNCHECKED);
	mDirRadio.SetCheck(BST_CHECKED);

	mDriveCombo.EnableWindow(FALSE);
	mDirEdit.EnableWindow(TRUE);
	mBrowseButton.EnableWindow(TRUE);

	mDirEdit.SetFocus();
	mDirEdit.SetSel(0, -1);
}

void 
CNewDiskOptionsDialog::OnDriveCheck()
{
	mDriveRadio.SetCheck(BST_CHECKED);
	mDirRadio.SetCheck(BST_UNCHECKED);

	mDriveCombo.EnableWindow(TRUE);
	mDirEdit.EnableWindow(FALSE);
	mBrowseButton.EnableWindow(FALSE);

	mDriveCombo.SetFocus();
}

void 
CNewDiskOptionsDialog::OnOk()
{
	mAlgorithm = mAlgoCombo.Get();
	mFileSys = mFileSysCombo.Get();
	mDrive = mDriveCombo.Get();

	mDirEdit.GetWindowText(mMountPoint);

	mChoseDrive = (mDriveRadio.GetCheck() == BST_CHECKED);
	mMountStartup = (mStartupCheck.GetCheck() == BST_CHECKED);

	if (!mChoseDrive && 
		UMountPoint::FixWarnRoot(mMountPoint, TRUE, FALSE, Handle(), 
		UCommonStrings::Get(UCommonStrings::kSinglePassMsgBoxTitle)))
	{
		mDirEdit.SetFocus();
		mDirEdit.SetSel(0, -1);
	}
	else
	{
		CDialog::OnOk();
	}
}

BOOL 
CNewDiskOptionsDialog::OnCommand(
	PGPUInt16	notifyCode, 
	PGPUInt16	itemId, 
	HWND		ctrl)
{
	switch (notifyCode)
	{
	case BN_CLICKED:
		switch (itemId)
		{
		case IDC_DRIVE_RADIO:
				OnDriveCheck();
				break;

			case IDC_DIR_RADIO:
				OnDirCheck();
				break;

			case IDC_BROWSE_BUTTON:
				OnBrowseButton();
				return TRUE;
		}
		break;
	}

	return CDialog::OnCommand(notifyCode, itemId, ctrl);
}

void 
CNewDiskOptionsDialog::OnContextMenu(HWND menuWnd, CPoint coords)
{
	CHelpShower::ShowContextMenu(menuWnd, NewDiskWizOptionsDlgHelpIDs);
}

void 
CNewDiskOptionsDialog::OnHelp(HELPINFO *pInfo)
{
	CHelpShower::ShowForControl(static_cast<HWND>(pInfo->hItemHandle), 
		NewDiskWizOptionsDlgHelpIDs);
}

BOOL 
CNewDiskOptionsDialog::OnInitDialog(HWND focusCtrl)
{
	CDialog::OnInitDialog(focusCtrl);

	mAlgoCombo.Subclass(GetDlgItem(IDC_ALGORITHM_COMBO));
	mBrowseButton.Subclass(GetDlgItem(IDC_BROWSE_BUTTON));
	mDriveCombo.Subclass(GetDlgItem(IDC_DRIVE_COMBO));
	mFileSysCombo.Subclass(GetDlgItem(IDC_FILESYSTEM_COMBO));
	mDriveRadio.Subclass(GetDlgItem(IDC_DRIVE_RADIO));
	mDirRadio.Subclass(GetDlgItem(IDC_DIR_RADIO));
	mDirEdit.Subclass(GetDlgItem(IDC_DIR_EDIT));
	mStartupCheck.Subclass(GetDlgItem(IDC_STARTUP_CHECK));

	InitControls();

	Center();
	SetForegroundWindow();

	return FALSE;
}


// Class CNewDiskWizADKPage member functions

BOOL 
CNewDiskWizADKPage::OnInitDialog(HWND focusCtrl)
{
	CPGPdiskPage::OnInitDialog(focusCtrl);
	mADKEdit.Subclass(GetDlgItem(IDC_ADK_EDIT));

	return FALSE;
}

PGPInt32 
CNewDiskWizADKPage::OnPsnSetActive()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	if (!mPInfo->pContext->IsThereADK())
		return -1;

	CString	userID;

	mPInfo->pContext->GetADKUserID(userID);
	mADKEdit.SetWindowText(userID);

	SetWizButtons(PSWIZB_BACK | PSWIZB_NEXT);

	return 0;
}


// Class CNewDiskWizCreationPage member functions

PGPBoolean 
CNewDiskWizCreationPage::CreateStatusCallbackAux(
	PGPBoolean	areFormatting, 
	PGPUInt32	percent)
{
	mCurTaskBar.SetPosWTimeout(percent, 1000);

	if (areFormatting)
		mAllTaskBar.SetPosWTimeout(50 + percent/2, 1000);
	else
		mAllTaskBar.SetPosWTimeout(percent/2, 1000);

	mCancelMutex.Enter();
	mCancelMutex.Leave();

	return !mShouldCancel;
}

PGPBoolean 
CNewDiskWizCreationPage::CreateStatusCallback(
	PGPBoolean	areFormatting, 
	PGPUInt32	percent, 
	void		*userVal)
{
	CNewDiskWizCreationPage	*pPage	= static_cast<CNewDiskWizCreationPage *>(
		userVal);
	pgpAssertAddrValid(pPage, CNewDiskWizCreationPage);

	return pPage->CreateStatusCallbackAux(areFormatting, percent);
}

void 
CNewDiskWizCreationPage::CreateCallbackAux()
{
	try
	{
		// Create and mount the PGPdisk file.
		CPGPdiskDiskSet	diskSet(*mPInfo->pContext);
		CPGPdiskDisk	newDisk;

		if (mPInfo->userChosePub)
		{
			diskSet.CreateDisk(mPInfo->path, mPInfo->blocks, 
				mPInfo->algorithm, mPInfo->pubKey[0], mPInfo->root, 
				CreateStatusCallback, this, newDisk);
		}
		else
		{
			CSecureArray<char>	passphrase;
			mPInfo->securePass.Extract(passphrase);

			diskSet.CreateDisk(mPInfo->path, mPInfo->blocks, 
				mPInfo->algorithm, passphrase.Get(), mPInfo->userName, 
				mPInfo->root, CreateStatusCallback, this, newDisk);
		}

		mPInfo->fileCreated = TRUE;

		// check in-between for cancel also
		if (mShouldCancel)
			THROW_PGPERROR(kPGPError_UserAbort);

		// format the PGPdisk volume
		PostMessage(WM_CREATE_HALFWAYDONE);

		CPGPdiskFormatter	formatter;

		formatter.Format(*mPInfo->pContext, mPInfo->root, mFileName, 
			mPInfo->fileSys, CreateStatusCallback, this);

		PostMessage(WM_CREATE_DONE);
	}
	catch (CComboError& error)
	{
		mPInfo->error = error;
		PostMessage(WM_CREATE_DONE);
	}
}

void 
_cdecl 
CNewDiskWizCreationPage::CreateCallback(void *userVal)
{
	CNewDiskWizCreationPage	*pPage	= static_cast<CNewDiskWizCreationPage *>(
		userVal);
	pgpAssertAddrValid(pPage, CNewDiskWizCreationPage);

	pPage->CreateCallbackAux();
}

void 
CNewDiskWizCreationPage::InitImages()
{
	// initialize image list
	CDC	dc;
	dc.AttachFromWindow(NULL, TRUE);	// DC for desktop

	PGPUInt32	numBits	= dc.GetDeviceCaps(BITSPIXEL) * 
		dc.GetDeviceCaps(PLANES);

	if (numBits <= 8)
	{
		mImages.Create(CIcon::kSmallIconWidth, CIcon::kSmallIconHeight, 
			ILC_COLOR | ILC_MASK, ImageIDs::kNumImageBitmaps, 0);

		CBitmap	images(UModuleInstance::Get(), 
			MAKEINTRESOURCE(IDB_IMAGES4BIT));
		mImages.AddMasked(images, ImageIDs::kMaskColor);
	}
	else
	{
		mImages.Create(CIcon::kSmallIconWidth, CIcon::kSmallIconHeight, 
			ILC_COLOR24 | ILC_MASK, ImageIDs::kNumImageBitmaps, 0);

		CBitmap	images(UModuleInstance::Get(), 
			MAKEINTRESOURCE(IDB_IMAGES24BIT));
		mImages.AddMasked(images, ImageIDs::kMaskColor);
	}

	// load icons
	mDoingIcon.Attach(
		mImages.GetIcon(ImageIDs::kDoingIcon, ILD_TRANSPARENT), TRUE);
	mDoneIcon.Attach(
		mImages.GetIcon(ImageIDs::kDoneIcon, ILD_TRANSPARENT), TRUE);
	mFailedIcon.Attach(
		mImages.GetIcon(ImageIDs::kFailedIcon, ILD_TRANSPARENT), TRUE);
}

void 
CNewDiskWizCreationPage::OnCreateDone()
{
	using namespace UCommonStrings;
	mCreateThread.KillThread();

	if (mShouldCancel || mPInfo->error.IsError())
	{
		// kill the infant PGPdisk if it still lives
		CPGPdiskDiskSet		diskSet(*mPInfo->pContext);
		CPGPdiskDiskIter	diskIter(diskSet);
		CPGPdiskDisk		disk;

		if (diskIter.SearchOnPath(mPInfo->path, disk))
			diskSet.Unmount(disk);

		// update the UI to reflect the error status
		SetWizButtons(0);

		if (mPInfo->error.IsError() && 
			(mPInfo->error.pgpErr != kPGPError_UserAbort))
		{
			ReportError(kTellDiskCreateFailed, kNewDiskMsgBoxTitle, 
				mPInfo->error);

			mCurTaskBar.ShowWindow(SW_HIDE);
			mAllTaskBar.SetPos(100);

			mCurTaskText.SetWindowText(Get(kNewDiskProgFailedText));
		}
		else
		{
			mCurTaskText.SetWindowText(Get(kNewDiskProgCanceledText));
		}

		mTaskIcon1.SetIcon(mFailedIcon);
		mTaskIcon2.SetIcon(mFailedIcon);
	}
	else
	{
		SetWizButtons(PSWIZB_NEXT);

		mCurTaskText.SetWindowText(Get(kNewDiskProgDoneText));
		mTaskIcon2.SetIcon(mDoneIcon);
		mAllTaskBar.SetPos(100);
		mCurTaskBar.ShowWindow(SW_HIDE);
	}
}

void 
CNewDiskWizCreationPage::OnCreateHalfwayDone()
{
	using namespace UCommonStrings;

	mCurTaskText.FitFormattedTextToWindow(Get(kNewDiskProgFormattingText), 
		mFileName);

	mTaskIcon1.SetIcon(mDoneIcon);
	mTaskIcon2.SetIcon(mDoingIcon);
}

BOOL 
CNewDiskWizCreationPage::OnInitDialog(HWND focusCtrl)
{
	CPGPdiskPage::OnInitDialog(focusCtrl);

	mTaskIcon1.Subclass(GetDlgItem(IDC_TASKICON1));
	mTaskIcon2.Subclass(GetDlgItem(IDC_TASKICON2));

	mAllTaskBar.Subclass(GetDlgItem(IDC_ALLTASK_BAR));
	mCurTaskBar.Subclass(GetDlgItem(IDC_CURTASK_BAR));
	mCurTaskText.Subclass(GetDlgItem(IDC_CURTASK_TEXT));

	InitImages();

	return FALSE;
}

PGPBoolean 
CNewDiskWizCreationPage::OnPsnQueryCancel()
{
	using namespace UCommonStrings;

	if (!mCreateThread.IsRunning())
		return FALSE;	// nothing's happening, we're set
	else if (mShouldCancel)
		return TRUE;	// cancel in progress, hold tight

	// Pause the creation thread.
	mCancelMutex.Enter();

	// Ask the user for cancel permission.
	CMessageBox	messageBox;

	CMessageBox::Button	result	= messageBox.Display(
		Get(kAskIfCancelCreation), Get(kNewDiskMsgBoxTitle), Handle(), 
		CMessageBox::kQuestionTone, CMessageBox::kYesNoStyle, 
		CMessageBox::kNoButton);

	mShouldCancel = (result == CMessageBox::kYesButton);
	mCancelMutex.Leave();

	if (mShouldCancel && mCreateThread.IsRunning())
	{
		mCurTaskText.SetWindowText(Get(kNewDiskProgFailingText));
		mCurTaskBar.ShowWindow(SW_HIDE);
		mAllTaskBar.SetPos(100);
	}

	return TRUE;
}

PGPInt32 
CNewDiskWizCreationPage::OnPsnSetActive()
{
	using namespace UCommonStrings;

	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	SetWizButtons(0);

	// Get key information.
	PGPBoolean	userChoseRO;	// not relevant here

	mPInfo->pNewUserPages->GetChoices(mPInfo->userChosePub, mPInfo->pubKey, 
		mPInfo->uNumPubKeys,mPInfo->userName, mPInfo->securePass, userChoseRO);

	// Prepare controls.
	mAllTaskBar.SetRange(0, 100);
	mAllTaskBar.SetPos(0);

	mCurTaskBar.SetRange(0, 100);
	mCurTaskBar.SetPos(0);

	mPInfo->path.GetFileNamePart(mFileName);
	mFileName.RemoveExtension();

	mCurTaskText.FitFormattedTextToWindow(Get(kNewDiskProgEncryptingText), 
		mFileName);
	
	mTaskIcon1.SetIcon(mDoingIcon);

	// Start creation.
	mCreateThread.StartThread();

	try
	{
		mShouldCancel = FALSE;
		mCreateThread.PerformAsyncCallback(CreateCallback, this);
	}
	catch (CComboError&)
	{
		mCreateThread.KillThread();
		throw;
	}

	return 0;
}

PGPInt32 
CNewDiskWizCreationPage::OnPsnWizNext()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnWizNext();

	if (result != 0)
		return result;

	try
	{
		if (mPInfo->mountAtStartup)
			UEditMount::MarkDiskForStartup(*mPInfo->pContext, mPInfo->path);
		else
			UEditMount::UnmarkDiskForStartup(*mPInfo->pContext, mPInfo->path);
	}
	catch (CComboError&)
	{
		using namespace UCommonStrings;

		CMessageBox	messageBox;
		messageBox.Display(Get(kTellCantMarkDiskFor), 
			Get(kNewDiskMsgBoxTitle), Handle(), CMessageBox::kWarningTone);
	}

	return 0;
}

PGPBoolean 
CNewDiskWizCreationPage::PreProcess(const CMessage& msg, LRESULT& result)
{
	switch (msg.Message())
	{
	case WM_CREATE_DONE:
		OnCreateDone();
		result = 0;
		return TRUE;

	case WM_CREATE_HALFWAYDONE:
		OnCreateHalfwayDone();
		result = 0;
		return TRUE;

	default:
		return FALSE;
	}
}


// Class CNewDiskWizDonePage member functions

PGPInt32 
CNewDiskWizDonePage::OnPsnSetActive()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	SetWizButtons(PSWIZB_FINISH);

	return 0;
}


// Class CNewDiskWizIntroPage member functions

BOOL 
CNewDiskWizIntroPage::OnInitDialog(HWND focusCtrl)
{
	CPGPdiskPage::OnInitDialog(focusCtrl);

	CWindow(GetParent()).Center();

	return FALSE;
}

PGPInt32 
CNewDiskWizIntroPage::OnPsnSetActive()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	SetWizButtons(PSWIZB_NEXT);

	return 0;
}


// Class CNewDiskWizVolInfoPage member functions

void 
CNewDiskWizVolInfoPage::SavePrefs()
{
	// Save prefs.
	CPGPclPrefs	prefs(mPInfo->pContext->PGPclPrefs());

	prefs.SetNumber(kPGPPrefDiskLastCreateSize, 
		static_cast<PGPUInt32>(mSizeValue));
	prefs.SetNumber(kPGPPrefDiskLastCreateScale, mCreateScale);

	CPath	newPrefDir;
	mPInfo->path.GetDirPart(newPrefDir);

	prefs.SetBoolean(kPGPPrefDiskDefaultMountAtStartup, 
		mPInfo->mountAtStartup);
	prefs.SetBoolean(kPGPPrefDiskLastAlgorithm, mPInfo->algorithm);
	prefs.SetNumber(kPGPPrefDiskLastAlgorithm, mPInfo->algorithm);
	prefs.SetNumber(kPGPPrefDiskLastFileSystem, mPInfo->fileSys);
	prefs.SetString(kPGPPrefDiskLastCreateRoot, mPInfo->root);

	prefs.Flush();
}

void 
CNewDiskWizVolInfoPage::FormatSizeText(PGPUInt64 kbSize, CString& text) const
{
	CString	numString;

	if (kbSize < FudgeSizeValue(PFLConstants::kKbPerMb, TRUE))
	{
		_ui64toa(kbSize, numString.GetBuffer(33), 10);
		numString.ReleaseBuffer();

		text.Format("%s KB", numString.Get());
	}
	else if (kbSize < FudgeSizeValue(PFLConstants::kKbPerGb, TRUE))
	{
		_ui64toa(kbSize/PFLConstants::kKbPerMb, numString.GetBuffer(33), 10);
		numString.ReleaseBuffer();

		text.Format("%s MB", numString.Get());
	}
	else
	{
		_ui64toa(kbSize/PFLConstants::kKbPerGb, numString.GetBuffer(33), 10);
		numString.ReleaseBuffer();

		text.Format("%s GB", numString.Get());
	}
}

PGPUInt32 
CNewDiskWizVolInfoPage::FudgeSizeValue(PGPUInt32 size, PGPBoolean scale) const
{
	if (scale)
	{
		return static_cast<PGPUInt32>(
			(kScaleStepFactor-1 + kSizeFudgeFraction)*size);
	}
	else
	{
		return static_cast<PGPUInt32>(kSizeFudgeFraction*size);
	}
}

void 
CNewDiskWizVolInfoPage::SizeScaleToKb(
	PGPUInt32					sizeValue, 
	PGPdiskUIPriv::CreateScale	createScale, 
	PGPUInt64&					kbSize)
{
	switch (createScale)
	{
	case PGPdiskUIPriv::kKbScale:
		kbSize = sizeValue;
		break;

	case PGPdiskUIPriv::kMbScale:

		if ((mLocalMaxKbSize%PFLConstants::kKbPerMb > 
			FudgeSizeValue(PFLConstants::kKbPerMb)) && 
			(sizeValue == mLocalMaxKbSize/PFLConstants::kKbPerMb + 1))
		{
			kbSize = mLocalMaxKbSize;
		}
		else
		{
			kbSize = sizeValue*PFLConstants::kKbPerMb;
		}
		break;

	case PGPdiskUIPriv::kGbScale:

		if ((mLocalMaxKbSize%PFLConstants::kKbPerGb > 
			FudgeSizeValue(PFLConstants::kKbPerGb)) && 
			(sizeValue == mLocalMaxKbSize/PFLConstants::kKbPerGb + 1))
		{
			kbSize = mLocalMaxKbSize;
		}
		else
		{
			kbSize = sizeValue*PFLConstants::kKbPerGb;
		}
		break;
	}
}

void 
CNewDiskWizVolInfoPage::KbToSizeScale(PGPUInt64 kbSize)
{
	if (kbSize < FudgeSizeValue(PFLConstants::kKbPerMb, TRUE))
	{
		mSizeValue = static_cast<PGPUInt32>(kbSize);
		mCreateScale = PGPdiskUIPriv::kKbScale;
	}
	else if (kbSize < FudgeSizeValue(PFLConstants::kKbPerGb, TRUE))
	{
		mSizeValue = static_cast<PGPUInt32>(kbSize/PFLConstants::kKbPerMb);
		mCreateScale = PGPdiskUIPriv::kMbScale;
	}
	else
	{
		mSizeValue = static_cast<PGPUInt32>(kbSize/PFLConstants::kKbPerGb);
		mCreateScale = PGPdiskUIPriv::kGbScale;
	}
}

PGPBoolean 
CNewDiskWizVolInfoPage::SaveDialogValidateCallbackAux(const CPath& path)
{
	using namespace UCommonStrings;

	CPath	root;
	path.GetRootPart(root);

	CommonString	warnString;
	PGPBoolean		refuseChoice	= FALSE;

	if (!root.IsValidDirectory())
	{
		refuseChoice = TRUE;
		warnString = kTellChoseInvalidVol;
	}
	else if (root.IsReadOnly())
	{
		refuseChoice = TRUE;
		warnString = kTellChoseReadOnlyVol;
	}
	else if (path.IsValidFile() && path.IsReadOnly())
	{
		refuseChoice = TRUE;
		warnString = kTellChoseReplaceFileInUseOrRO;
	}

	if (refuseChoice)
	{
		CMessageBox	messageBox;
		messageBox.Display(Get(warnString), Get(kNewDiskMsgBoxTitle), 
			Handle());

		return FALSE;
	}
	else
	{
		return TRUE;
	}
}

PGPBoolean 
CNewDiskWizVolInfoPage::SaveDialogValidateCallback(
	const CPath&	path, 
	void			*userVal)
{
	CNewDiskWizVolInfoPage	*pInfoPage	= reinterpret_cast<
		CNewDiskWizVolInfoPage *>(userVal);
	pgpAssertAddrValid(pInfoPage, CNewDiskWizVolInfoPage);

	return pInfoPage->SaveDialogValidateCallbackAux(path);
}

PGPBoolean 
CNewDiskWizVolInfoPage::ValidateDiskPath()
{
	using namespace UCommonStrings;

	ImportValuesFromUI();

	CMessageBox			messageBox;
	CMessageBox::Button	result;

	CPath	root;
	mPInfo->path.GetRootPart(root);

	CPath	dir;
	mPInfo->path.GetDirPart(dir);

	CPath	fileName;
	mPInfo->path.GetFileNamePart(fileName);

	CPath	pathWithExt(mPInfo->path);
	pathWithExt.AddExtension(kPGPdiskFileExtension);

	PGPBoolean	shouldFail	= FALSE;

	if (!mPInfo->path.IsLegalPath())
	{
		messageBox.Display(Get(kTellIllegalPath), Get(kNewDiskMsgBoxTitle), 
			Handle());
		shouldFail = TRUE;
	}

	if (fileName.IsEmpty())
	{
		messageBox.Display(Get(kTellEmptyFileName), 
			Get(kNewDiskMsgBoxTitle), Handle());
		shouldFail = TRUE;
	}
	
	if (!shouldFail && !root.IsValidDirectory())
	{
		messageBox.Display(Get(kTellChoseInvalidVol), 
			Get(kNewDiskMsgBoxTitle), Handle());
		shouldFail = TRUE;
	}
	
	if (!shouldFail && root.IsReadOnly())
	{
		messageBox.Display(Get(kTellChoseReadOnlyVol), 
			Get(kNewDiskMsgBoxTitle), Handle());
		shouldFail = TRUE;
	}

	if (!shouldFail && pathWithExt.IsValidFile())
	{
		if (pathWithExt.IsReadOnly())
		{
			messageBox.Display(Get(kTellChoseReplaceFileInUseOrRO), 
				Get(kNewDiskMsgBoxTitle), Handle());
			shouldFail = TRUE;
		}

		if (!shouldFail)
		{
			CString	replaceFileStr;

			replaceFileStr.Format(Get(kAskIfReplaceFile), mPInfo->path.Get());

			result = messageBox.Display(replaceFileStr, 
				Get(kNewDiskMsgBoxTitle), Handle(), 
				CMessageBox::kWarningTone, CMessageBox::kYesNoStyle, 
				CMessageBox::kNoButton);

			if (result == CMessageBox::kNoButton)
				shouldFail = TRUE;
		}
	}

	if (!shouldFail && !dir.IsValidDirectory())
	{
		CString	replaceDirStr;

		replaceDirStr.Format(Get(kAskIfCreateDir), dir.Get());

		result	= messageBox.Display(replaceDirStr, 
			Get(kNewDiskMsgBoxTitle), Handle(), CMessageBox::kWarningTone, 
			CMessageBox::kYesNoStyle, CMessageBox::kNoButton);

		if (result == CMessageBox::kNoButton)
			shouldFail = TRUE;
		else
			dir.RecurseCreateDir();
	}

	if (shouldFail)
	{
		mPathEdit.SetFocus();
		mPathEdit.SetSel(0, -1);

		return FALSE;
	}
	else
	{
		return TRUE;
	}
}

void 
CNewDiskWizVolInfoPage::ExportValuesToUI()
{
	mPathEdit.SetWindowText(mPInfo->path);
	mScaleCombo.Set(mCreateScale);

	CString	sizeText;

	_ui64toa(mSizeValue, sizeText.GetBuffer(33), 10);
	sizeText.ReleaseBuffer();

	mSizeEdit.SetWindowText(sizeText);
}

void 
CNewDiskWizVolInfoPage::ImportValuesFromUI()
{
	mPathEdit.GetWindowText(mPInfo->path);

	CString	sizeText;
	mSizeEdit.GetWindowText(sizeText);

	mSizeValue = atoi(sizeText);
	mCreateScale = mScaleCombo.Get();

	SizeScaleToKb(mSizeValue, mCreateScale, mKbSize);

	// Fudge a little bit on total blocks - need some for the headers.
	mPInfo->blocks = ((mKbSize * PFLConstants::kBytesPerKb) / 
		kPGPdiskBlockSize) - kTypicalBlocksHeaders;
}

PGPBoolean 
CNewDiskWizVolInfoPage::FixWarnOptions(PGPBoolean silently)
{
	using namespace UCommonStrings;

	PGPBoolean	userWarned	= FALSE;

	// Make sure root is legal.
	userWarned = UMountPoint::FixWarnRoot(mPInfo->root, FALSE, 
		silently, Handle(), Get(kNewDiskMsgBoxTitle));

	// Make sure algorithm is legal.
	CArray<PGPdiskEncryptionAlgorithm>	algArray;
	PGPUInt32							numAlgs;

	CPGPdiskEngine::GetSupportedDiskAlgorithms(algArray, &numAlgs);

	PGPBoolean	isAlgoLegal	= FALSE;

	for (PGPUInt32 i = 0; i < numAlgs; i++)
	{
		if (algArray[i] == kPGPdiskOldCAST5Algorithm)
			continue;

		if (mPInfo->algorithm == algArray[i])
		{
			isAlgoLegal = TRUE;
			break;
		}
	}

	if (!isAlgoLegal)
		mPInfo->algorithm = kPGPdiskOldCAST5Algorithm;

	// Make sure filesystem is legal.
	CArray<CPGPdiskFormatter::FileSysInfo>	fsArray;
	PGPUInt32								numFSes;

	CPGPdiskFormatter::GetValidFilesystems(mPInfo->blocks, fsArray, numFSes);

	PGPBoolean	isFileSysLegal	= FALSE;

	for (i = 0; i < numFSes; i++)
	{
		if (fsArray[i].fsType == mPInfo->fileSys)
		{
			isFileSysLegal = TRUE;
			break;
		}
	}

	if (!isFileSysLegal)
	{
		// There has to be SOME legal filesystem.
		pgpAssert(numFSes > 0);
		mPInfo->fileSys = fsArray[0].fsType;

		if (!silently)
		{
			CMessageBox	messageBox;
			messageBox.Display(Get(kTellInvalidFileSystem), 
				Get(kNewDiskMsgBoxTitle), Handle());

			OnOptionsButton();
			userWarned = TRUE;
		}
	}

	return userWarned;
}

PGPBoolean 
CNewDiskWizVolInfoPage::FixWarnSizeAndScale(PGPBoolean silently)
{
	using namespace UCommonStrings;

	CommonString	failString;
	PGPBoolean		failThis	= FALSE;
	PGPBoolean		userWarned	= FALSE;

	// Verify that the space being asked is not too big or small.
	ImportValuesFromUI();

	if (mKbSize < mLocalMinKbSize)
	{
		failThis = TRUE;
		KbToSizeScale(mLocalMinKbSize);

		failString = kNewDiskSizeTooSmall;
	}
	else if (mKbSize > mPlatformMaxKbSize)
	{
		failThis = TRUE;
		KbToSizeScale(mLocalMaxKbSize);

		failString = kNewDiskSizeTooBig;
	}
	else if (mKbSize > mLocalMaxKbSize)
	{
		failThis = TRUE;
		KbToSizeScale(mLocalMaxKbSize);

		failString = kNewDiskSizeNotEnoughSpace;
	}
	else
	{
		try
		{
			// check for files too big for Fat32
			CString	fileSys;
			mPInfo->path.GetVolumeFileSys(fileSys);

			PGPUInt64	maxFat32Kb	= 
				FileSys::kMaxFat32FileMegs*PFLConstants::kKbPerMb;

			// Added IsWin95Compatible for situations where we try to create 4 gig 
			// disks on an NTFS networked drive.
			if ((
				// FAT32 only allows 4 GB
				(fileSys == FileSys::kFat32IdStr)||
				// Win 9x can't support files over 4 GB
				(UWinVersion::IsWin95Compatible())||
				// NT4 can't id FAT32 networked volumes. No 4 GB unless NTFS.
				((fileSys != FileSys::kNTFSIdStr)&&(UWinVersion::IsWinNT4Compatible())&&(!UWinVersion::IsWin2000Compatible()))
				)
				&& (mKbSize > maxFat32Kb))
			{
				failThis = TRUE;
				KbToSizeScale(maxFat32Kb);

				failString = kNewDiskSizeTooBigFat32;
			}
		}
		catch (CComboError&) {}
	}

	// Handle case where we couldn't get free space of the drive.
	if (mAreWeFailingDiskFree && !failThis)
	{
		try
		{
			if (!mPInfo->path.TestFreeVolumeSpace(mKbSize))
			{
				failThis = TRUE;
				KbToSizeScale(mLocalMinKbSize);

				failString = kNewDiskSizeNotEnoughSpace;
			}
		}
		catch (CComboError&) { }
	}

	if (failThis)
	{
		if (!silently)
		{
			CMessageBox	messageBox;
			messageBox.Display(Get(failString), Get(kNewDiskMsgBoxTitle), 
				Handle());

			userWarned = TRUE;
		}

		ExportValuesToUI();

		if (!silently)
		{
			// Point out the incorrect size entered by the user.
			mSizeEdit.SetFocus();
			mSizeEdit.SetSel(0, -1);
		}
	}

	return userWarned;
}

void 
CNewDiskWizVolInfoPage::UpdateMinMaxSizeFromPath(const CPath& path)
{
	using namespace UCommonStrings;

	ImportValuesFromUI();

	// Get min/max possible PGPdisk sizes.
	mPlatformMaxKbSize = PGPdiskUIPriv::kDiskMaxGbSize;
	mPlatformMaxKbSize *= PFLConstants::kKbPerGb;	// stop C4307 woohoo!

	mPlatformMinKbSize = PGPdiskUIPriv::kDiskMinKbSize;

	// Get free space. Note if failed (which will happen on network volumes).
	PGPUInt64	bytesFree;

	try
	{
		bytesFree = path.GetFreeVolumeSpace();

		mAreWeFailingDiskFree = FALSE;
		mKbFree = bytesFree / PFLConstants::kBytesPerKb;
	}
	catch (CComboError&)
	{
		mAreWeFailingDiskFree = TRUE;
		mKbFree = mPlatformMaxKbSize;	// fake max amount of space
	}

	// Is user replacing an existing file? If so, add its size to free space.
	try
	{
		CPath	path2(path);
		path2.AddExtension(kPGPdiskFileExtension);

		CFile	existingFile;
		existingFile.Open(path2, CFile::kReadOnlyFlag);

		PGPUInt64	bytesFile	= existingFile.GetLength();

		mKbFree += bytesFile/PFLConstants::kBytesPerKb;
		existingFile.Close();
	}
	catch (CComboError&) { }

	// Calculate max and min PGPdisk sizes we can create.
	mLocalMaxKbSize	= pgpMin(mKbFree, mPlatformMaxKbSize);
	mLocalMinKbSize	= mPlatformMinKbSize;

	pgpAssert(mLocalMaxKbSize >= mLocalMinKbSize);

	// Update the contents of the free size message shown to the user.
	CString	freeSizeText;

	if (mAreWeFailingDiskFree)
	{
		freeSizeText = Get(kNewDiskFreeSpaceUnknownText);
	}
	else
	{
		CString freeSpaceStr;
		FormatSizeText(mKbFree, freeSpaceStr);

		freeSizeText.Format(Get(kNewDiskFreeSpaceKnownText), 
			freeSpaceStr.Get());
	}

	mFreeSizeText.SetWindowText(freeSizeText);
}

void 
CNewDiskWizVolInfoPage::InitControls()
{
	CPGPclPrefs	prefs(mPInfo->pContext->PGPclPrefs());

	mScaleCombo.Fill();
	mSizeUDCtrl.SetRange(1, UD_MAXVAL);

	// Get default path.
	mPathEdit.LimitText(MAX_PATH);
	mPathEdit.SetWindowText(mPInfo->path);

	// Get default root.
	prefs.GetString(kPGPPrefDiskLastCreateRoot, mPInfo->root);

	UMountPoint::FixWarnRoot(mPInfo->root, FALSE, TRUE, Handle());

	if (mPInfo->root.IsPlainLocalRoot() || 
		!UWinVersion::IsWin2000Compatible())
	{
		mDrive = mPInfo->root.GetDriveLet();
		mMountPoint.Empty();
		mChoseDrive = TRUE;
	}
	else
	{
		mDrive = CPath::kMaxDrives + 1;
		mMountPoint = mPInfo->root;
		mChoseDrive = FALSE;
	}

	// Get default size and scale.
	mSizeValue = prefs.GetNumber(kPGPPrefDiskLastCreateSize);

	mCreateScale = static_cast<PGPdiskUIPriv::CreateScale>(
		prefs.GetNumber(kPGPPrefDiskLastCreateScale));

	// Get default algorithm and fileSys.
	mPInfo->algorithm = static_cast<PGPdiskEncryptionAlgorithm>(
		prefs.GetNumber(kPGPPrefDiskLastAlgorithm));
	mPInfo->fileSys = static_cast<FileSys::Type>(
		prefs.GetNumber(kPGPPrefDiskLastFileSystem));

	// Get default mount at startup value.
	mPInfo->mountAtStartup = prefs.GetBoolean(
		kPGPPrefDiskDefaultMountAtStartup);

	ExportValuesToUI();

	// Calculate min/max size values.
	UpdateMinMaxSizeFromPath(mPInfo->path);

	// Legalize default values.
	FixWarnSizeAndScale(TRUE);
	FixWarnOptions(TRUE);

	// Output all values to the dialog.
	ExportValuesToUI();
}

void 
CNewDiskWizVolInfoPage::OnOptionsButton()
{
	ImportValuesFromUI();

	if (!ValidateDiskPath())
		return;

	UpdateMinMaxSizeFromPath(mPInfo->path);

	if (FixWarnSizeAndScale())
		return;

	try
	{
		CNewDiskOptionsDialog	optionsDlg;

		optionsDlg.Display(Handle(), mPInfo->blocks, mPInfo->algorithm, 
			mPInfo->fileSys, mDrive, mMountPoint, mChoseDrive, 
			mPInfo->mountAtStartup);

		if (mChoseDrive)
			mPInfo->root.MakePlainLocalRoot(mDrive);
		else
			mPInfo->root = mMountPoint;

		mUserChangedOptions = TRUE;
	}
	catch (CComboError& error)
	{
		if (error.pgpErr != kPGPError_UserAbort)
			throw;
	}
}

void 
CNewDiskWizVolInfoPage::OnBrowseButton()
{
	ImportValuesFromUI();

	try
	{
		CPath	defaultPath(mPInfo->path);

		CSaveFileDialog	saveDialog;

		saveDialog.Choose(Handle(), mPInfo->path, defaultPath, 
			UCommonStrings::Get(UCommonStrings::kNewDiskSaveDialogTitle), 
			PGPdiskUIPriv::kDiskFileFilter, 
			kPGPdiskFileExtension, 
			SaveDialogValidateCallback, this);

		if (CPath::AreShowingExtensions())
			mPInfo->path.AddExtension(kPGPdiskFileExtension);
		else
			mPInfo->path.RemoveExtension();

		ExportValuesToUI();
		UpdateMinMaxSizeFromPath(mPInfo->path);
	}
	catch (CComboError& error)
	{
		if (error.pgpErr != kPGPError_UserAbort)
			throw;
	}
}

BOOL 
CNewDiskWizVolInfoPage::OnCommand(
	PGPUInt16	notifyCode, 
	PGPUInt16	itemId, 
	HWND		ctrl)
{
	switch (notifyCode)
	{
	case BN_CLICKED:
		switch (itemId)
		{
			case IDC_OPTIONS_BUTTON:
				OnOptionsButton();
				return TRUE;

			case IDC_BROWSE_BUTTON:
				OnBrowseButton();
				return TRUE;
		}
		break;
	}

	return CPGPdiskPage::OnCommand(notifyCode, itemId, ctrl);
}

BOOL 
CNewDiskWizVolInfoPage::OnInitDialog(HWND focusCtrl)
{
	CPGPdiskPage::OnInitDialog(focusCtrl);

	mOptionsButton.Subclass(GetDlgItem(IDC_OPTIONS_BUTTON));
	mScaleCombo.Subclass(GetDlgItem(IDC_SCALE_COMBO));
	mPathEdit.Subclass(GetDlgItem(IDC_PATH_EDIT));
	mSizeEdit.Subclass(GetDlgItem(IDC_SIZE_EDIT));
	mFreeSizeText.Subclass(GetDlgItem(IDC_FREESIZE_TEXT));
	mSizeUDCtrl.Subclass(GetDlgItem(IDC_SIZE_UDCTRL));

	InitControls();

	return FALSE;
}

PGPInt32 
CNewDiskWizVolInfoPage::OnPsnSetActive()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	mUserChangedOptions = FALSE;
	SetWizButtons(PSWIZB_BACK | PSWIZB_NEXT);

	return 0;
}

PGPInt32 
CNewDiskWizVolInfoPage::OnPsnWizNext()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnWizNext();

	if (result != 0)
		return result;

	ImportValuesFromUI();

	if (!ValidateDiskPath())
		return -1;

	UpdateMinMaxSizeFromPath(mPInfo->path);

	if (FixWarnSizeAndScale())
		return -1;

	if (FixWarnOptions(!mUserChangedOptions))
	{
		OnOptionsButton();
		return -1;
	}

	if (USecurity::IsRemoteSession() && 
		UWinVersion::IsWin2000Compatible())
	{
		using namespace UCommonStrings;

		UMountPoint::FixWarnRoot(mPInfo->root, TRUE, TRUE, 
			Handle());

		if (mPInfo->root.IsPlainLocalRoot())
		{
			CMessageBox	messageBox;
			messageBox.Display(Get(kTellChoseDriveInWin2kTerm), 
				Get(kNewDiskMsgBoxTitle), Handle());

			mDrive = mPInfo->root.GetDriveLet();
			mChoseDrive = FALSE;
			mMountPoint.Empty();

			OnOptionsButton();
			return -1;
		}
	}

	mPInfo->path.AddExtension(kPGPdiskFileExtension);
	SavePrefs();

	return 0;
}

BOOL IsWindowsXPOrGreater()
{
    OSVERSIONINFOEXA osvi = {0};

    osvi.dwOSVersionInfoSize = sizeof(osvi);

    if (!GetVersionExA((OSVERSIONINFOA*)&osvi))
    {
        // If it failed, it must be a down level platform
        osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA);
        GetVersionExA((OSVERSIONINFOA*)&osvi);
    }
    
    return (VER_PLATFORM_WIN32_NT == osvi.dwPlatformId &&
        ((osvi.dwMajorVersion > 5) ||
        (osvi.dwMajorVersion == 5 && (osvi.dwMinorVersion > 0 ||
        (osvi.dwMinorVersion == 0 && LOWORD(osvi.dwBuildNumber) > 2195)))));
}

// Class CNewDiskWizard member functions

void 
CNewDiskWizard::Display(
	const CPGPdiskContext&	context, 
	const char				*defaultPath, 
	HWND					parentWnd)
{
	using namespace UCommonStrings;
	pgpAssert(CWindow(parentWnd).IsWindow());

	// Get a valid default path.
	CPath	cDefaultPath;

	if (IsntNull(defaultPath))
		cDefaultPath = defaultPath;

	ConstructValidDefaultPath(context, cDefaultPath);

	// Create the wizard pages.
	CNewUserPages			newUserPages;
	CPGPGlobalRandomPool	randomPool(context.PGPContext());
	DiskWizardInfo			diInfo;
	const char				*title	= Get(kNewDiskTitle);

	diInfo.pNewUserPages	= &newUserPages;
	diInfo.pContext			= &context;
	diInfo.pRandomPool		= &randomPool;
	diInfo.path				= cDefaultPath;
	diInfo.blocks			= 0;
	diInfo.fileCreated		= FALSE;

	CPGPdiskUserSet	diskUsers;	// empty
	newUserPages.Create(&context, &diskUsers, title, 
		Get(kNewDiskMsgBoxTitle), TRUE, TRUE, TRUE);

	CNewDiskWizADKPage		adkPage(&diInfo);
	CNewDiskWizCreationPage	creationPage(&diInfo);
	CNewDiskWizDonePage		donePage(&diInfo);
	CNewDiskWizIntroPage	introPage(&diInfo);
	CNewDiskWizVolInfoPage	volInfoPage(&diInfo);
	CRandomDataPage			randomPage(&diInfo.algorithm, 
		diInfo.pRandomPool);

	LoadSidebar();

	adkPage.Create(title);
	creationPage.Create(title);
	donePage.Create(title, FALSE, &mSidebarBitmap, &mSidebarPalette);
	introPage.Create(title, FALSE, &mSidebarBitmap, &mSidebarPalette);
	randomPage.Create(title);
	volInfoPage.Create(title);

	AddPage(&introPage);
	AddPage(&volInfoPage);
	AddPage(&adkPage);
	AddPage(newUserPages.GetAskChoicePage());
	AddPage(newUserPages.GetAskPassPage());
	AddPage(newUserPages.GetBadPassPage());
	AddPage(newUserPages.GetAskPubKeyPage());
	AddPage(&randomPage);
	AddPage(&creationPage);
	AddPage(&donePage);

	// Show the wizard.
	try
	{
		CPropertySheet::Create(parentWnd, TRUE);

		// We can't browse after creation with XP. The shell pukes.
//		if(!IsWindowsXPOrGreater())
//			diInfo.root.Browse();
	}
	catch (CComboError&)
	{
		try
		{
			// "I can make you all go away, any time I want to..."
			if (diInfo.fileCreated)
			{
				CPGPdiskDiskSet		diskSet(context);
				CPGPdiskDiskIter	diskIter(diskSet);
				CPGPdiskDisk		disk;

				if (diskIter.SearchOnPath(diInfo.path, disk))
					diskSet.Unmount(disk);

				// work around warn on delete
				CString temp(diInfo.path);
				temp += "not";

				CFile	deleter;

				deleter.Move(diInfo.path, temp);
				deleter.Delete(temp);
			}
		}
		catch (CComboError&) { }

		throw;
	}

	// flush prefs
	try
	{
		CPGPclPrefs	prefs(context.PGPclPrefs());
		prefs.Flush();
	}
	catch (CComboError&) { }
}

void 
CNewDiskWizard::ConstructValidDefaultPath(
	const CPGPdiskContext&	context, 
	CPath&					defaultPath)
{
	CPath	dir;
	defaultPath.GetDirPart(dir);

	// Get valid dir.
	if (dir.IsEmpty())
	{
		try
		{
			CPGPclPrefs	prefs(context.PGPclPrefs());
			prefs.GetString(kPGPPrefDiskLastSaveDir, dir);

			if (!dir.IsValidDirectory() || dir.IsReadOnly())
			{
				PGPclGetPath(kPGPclDefaultSaveAsFolder, 
					dir.GetBuffer(kPGPdiskMaxPathLength), 
					kPGPdiskMaxPathLength);
				dir.ReleaseBuffer();
			}
		}
		catch (CComboError&)
		{
			dir.ReleaseBuffer();
		}
	}
	else
	{
		// If default path specified, FAIL if it isn't valid for use.
		if (!dir.IsValidDirectory())
			THROW_PGPERROR(kPGPClientError_DiskDirNotFound);
		else if (dir.IsReadOnly())
			THROW_PGPERROR(kPGPClientError_DiskDirWriteProtected);
	}

	// Get a valid default filename.
	CPath	fileName;

	defaultPath.GetFileNamePart(fileName);
	fileName.RemoveExtension();

	if (fileName.IsEmpty())
	{
		fileName = UCommonStrings::Get(
			UCommonStrings::kNewDiskDefaultFilename);
	}

	// Find a filename that isn't taken.
	CString		suffix;
	PGPUInt32	i	= 1;

	while (TRUE)
	{
		CPath	curFilename(fileName);

		curFilename.Append(suffix);
		curFilename.AddExtension(kPGPdiskFileExtension);

		CPath	curPath(dir);
		curPath.EndInSlash();
		curPath.Append(curFilename);

		if (!curPath.IsValidPath())
		{
			defaultPath = curPath;
			break;
		}

		suffix.Format("%d", i++);
	}

	if (CPath::AreShowingExtensions())
		defaultPath.AddExtension(kPGPdiskFileExtension);
}

void 
CNewDiskWizard::LoadSidebar()
{
	CDC	dc;
	dc.AttachFromWindow(NULL);

	PGPUInt32	numBits	= dc.GetDeviceCaps(BITSPIXEL) * 
		dc.GetDeviceCaps(PLANES);

	PGPUInt32	uintRsrc;

	if (numBits <= 1)
		uintRsrc = IDB_SIDEBAR1;
	else if (numBits <= 4)
		uintRsrc = IDB_SIDEBAR4;
	else
		uintRsrc = IDB_SIDEBAR8;

	mSidebarBitmap.CreateWithPalette(UModuleInstance::Get(), 
		MAKEINTRESOURCE(uintRsrc), mSidebarPalette);
}
