/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CNewUserPages.cpp,v 1.27 2002/11/05 02:10:53 wjb Exp $
____________________________________________________________________________*/

#include "pgpClassesConfig.h"
#include "pgpBuild.h"
#include "pgpPassphraseUtils.h"

#include "CSecureString.h"
#include "CString.h"

#include "CButton.h"
#include "CDC.h"
#include "CEdit.h"
#include "CUTF8Edit.h"
#include "CInfoBar.h"
#include "CMessageBox.h"
#include "CProgressBar.h"
#include "CRect.h"
#include "CSecureRichEdit.h"
#include "UModuleInstance.h"

#include "UCommonStrings.h"
#include "CPGPclPrefs.h"
#include "CPGPdiskEngine.h"
#include "CPGPKey.h"
#include "CPGPKeyDB.h"

#include "PGPdiskUIPriv.h"

#include "CNewUserPages.h"
#include "CPGPdiskPage.h"
#include "CPublicKeysView.h"
#include "PGPdiskHelp.h"
#include "Resource.h"

_USING_PGP

_UNNAMED_BEGIN

// Constants

enum
{
	kEnforcePassMinLength	= 0x01, 
	kEnforcePassMinQuality	= 0x02, 

	kBadPassConfirm		= 0x01, 
	kBadPassMinLength	= 0x02, 
	kBadPassMinQuality	= 0x04, 

	kDLUMarginControl	= 4, 
	kDLUMarginDialog	= 7
};

DWORD	WizAskChoicePageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_DISKSTATIC3,		-1, 
	IDC_HEADING,			-1, 
	IDC_PUBKEY_RADIO,		IDH_PGPDISKUI_WIZASKCHOICEPAGE_PUBKEY, 
	IDC_PASS_RADIO,			IDH_PGPDISKUI_WIZASKCHOICEPAGE_PASSPHRASE, 
	IDC_USERNAME_CAPTION,	IDH_PGPDISKUI_WIZASKCHOICEPAGE_USERNAME, 
	IDC_USERNAME_EDIT,		IDH_PGPDISKUI_WIZASKCHOICEPAGE_USERNAME, 
	IDC_ROUSER_CHECK,		IDH_PGPDISKUI_WIZASKCHOICEPAGE_READONLY, 
	0,						0
};

DWORD	WizAskPassPageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_HEADING,			-1, 
	IDC_PHRASELENGTH_TEXT,	-1, 
	IDC_HIDETYPE_CHECK,		IDH_PGPDISKUI_WIZASKPASSPAGE_HIDETYPING, 
	IDC_PASSPHRASE_CAPTION,	IDH_PGPDISKUI_WIZASKPASSPAGE_PASSPHRASE, 
	IDC_PASSPHRASE_EDIT,	IDH_PGPDISKUI_WIZASKPASSPAGE_PASSPHRASE, 
	IDC_CONFIRM_CAPTION,	IDH_PGPDISKUI_WIZASKPASSPAGE_CONFIRMPHRASE, 
	IDC_QUALITY_CAPTION,	IDH_PGPDISKUI_WIZASKPASSPAGE_QUALITYBAR,
	IDC_QUALITY_BAR,		IDH_PGPDISKUI_WIZASKPASSPAGE_QUALITYBAR,
	IDC_MINQUALITY_CAPTION,	IDH_PGPDISKUI_WIZASKPASSPAGE_MINQUALITYBAR,
	IDC_MINQUALITY_BAR,		IDH_PGPDISKUI_WIZASKPASSPAGE_MINQUALITYBAR,
	IDC_CONFIRM_EDIT,		IDH_PGPDISKUI_WIZASKPASSPAGE_CONFIRMPHRASE, 
	0,						0
};

DWORD	WizAskPubKeyPageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_HEADING,			-1, 
	IDC_KEYS_VIEW,			IDH_PGPDISKUI_WIZASKPUBKEYPAGE_PUBKEYSVIEW, 
	0,						0
};

DWORD	WizBadPassPageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_HEADING,			-1, 
	IDC_REJECT_TEXT,		-1, 
	0,						0
};

_UNNAMED_END

// Types

struct CNewUserPages::PagesInfo
{
	PGPBoolean	isFinished;

	const CPGPdiskContext	*pContext;
	const CPGPdiskUserSet	*pDiskUsers;

	CPGPKeySet	keysToChoose;
	PGPBoolean	autoUseDefKey;
	PGPBoolean	defKeyPrefExists;
	PGPBoolean	defKeySeen;
	CPGPKeyID	defKeyID;
	PGPBoolean	isForAdminUser;
	CString		warnDlgTitle;

	PGPUInt32	enforcePassFlags;
	PGPUInt32	badPassFlags;
	PGPUInt32	minPassLength;
	PGPUInt32	minPassQuality;

	PGPBoolean		userChosePub;
	PGPBoolean		userChoseRO;
	CPGPKey			pgpKey[100];
	PGPUInt32		uNumPubKeys;
	PGPBoolean		singleSelect;
	CString			userName;
	CSecureString	securePass;
};


// Class CNewUserAskChoicePage

class PGP::CNewUserAskChoicePage : public CPGPdiskPage
{
public:
	CNewUserAskChoicePage(CNewUserPages::PagesInfo *pInfo) : mPInfo(pInfo)
	{
		pgpAssertAddrValid(pInfo, CNewUserPages::PagesInfo);
	}

	~CNewUserAskChoicePage()
	{
	}
	
	PGPUInt16	GetDialogId() const
	{
		return IDD_WIZ_ASKCHOICE;
	}

private:
	CButton	mPassRadio;
	CButton	mPubKeyRadio;
	CButton	mROUserCheck;
	CUTF8Edit	mUserNameEdit;
	CStatic	mUserNameText;

	CNewUserPages::PagesInfo	*mPInfo;

	DWORD	GetPageHelpId() const {return IDH_PGPDISKUI_WIZASKCHOICEPAGE;}
	PVOID	GetPageHelpIds() const {return WizAskChoicePageHelpIDs;}

	void		InitControls();
	PGPBoolean	ValidateUserName(const char * userName);

	void	OnPassRadio();
	void	OnPubKeyRadio();

	BOOL		OnCommand(PGPUInt16 notifyCode, PGPUInt16 itemId, HWND ctrl);
	BOOL		OnInitDialog(HWND focusCtrl);
	PGPInt32	OnPsnSetActive();
	PGPInt32	OnPsnWizNext();
};


// Class CNewUserBadPassPage

class PGP::CNewUserBadPassPage : public CPGPdiskPage
{
private:
	enum
	{
		kDLUStartYForInfoBar	= 37, 

		kBadConfirmStringId		= 0, 
		kBadMinLengthStringId	= 1, 
		kBadMinQualityStringId	= 2, 

		kMinNewPassLength		= 8, 
	};

public:
	CNewUserBadPassPage(CNewUserPages::PagesInfo *pInfo) : mPInfo(pInfo)
	{
		pgpAssertAddrValid(pInfo, PagesInfo);
	}

	~CNewUserBadPassPage()
	{
	}
	
	PGPUInt16	GetDialogId() const
	{
		return IDD_WIZ_BADPASS;
	}

private:
	CInfoBar	mBadPassBar;
	CStatic		mRejectText;

	CNewUserPages::PagesInfo	*mPInfo;

	DWORD	GetPageHelpId() const {return IDH_PGPDISKUI_WIZBADPASSPAGE;}
	PVOID	GetPageHelpIds() const {return WizBadPassPageHelpIDs;}

	void	InitControls();

	BOOL		OnInitDialog(HWND focusCtrl);
	PGPInt32	OnPsnSetActive();
	PGPInt32	OnPsnWizNext();
};


// Class CNewUserAskPassPage

class PGP::CNewUserAskPassPage : public CPGPdiskPage
{
private:
	enum
	{
		kDLUYOffsetCapsLockBar	= 15, 
		kDLUWidthCapsLockBar	= 110, 
		kCapsLockStringId		= 0
	};

public:
	CNewUserAskPassPage(CNewUserPages::PagesInfo *pInfo) : mPInfo(pInfo)
	{
		pgpAssertAddrValid(pInfo, CNewUserPages::PagesInfo);
	}

	~CNewUserAskPassPage()
	{
	}
	
	PGPUInt16	GetDialogId() const
	{
		return IDD_WIZ_ASKPASS;
	}

private:
	CNewUserPages::PagesInfo	*mPInfo;

	CButton			mHideTypingCheck;
	CInfoBar		mCapsLockBar;
	CProgressBar	mMinQualityBar;
	CProgressBar	mQualityBar;
	CSecureRichEdit	mPassphraseEdit;
	CSecureRichEdit	mConfirmEdit;
	CStatic			mMinQualityText;
	CStatic			mPhraseLengthText;

	CSecureRichEdit	*mIHadFocus;

	DWORD	GetPageHelpId() const {return IDH_PGPDISKUI_WIZASKPASSPAGE;}
	PVOID	GetPageHelpIds() const {return WizAskPassPageHelpIDs;}

	void	CreateCapsLockBar();
	void	InitControls();
	void	InitDialogText();
	void	InitQualityPrefs();
	void	UpdatePassphraseFlags();
	void	UpdatePassphraseQuality();

	void	SetCapsLockMessageState();
	void	WarnCapsLock(PGPBoolean warn);

	void	OnHideTypeCheck();

	BOOL		OnCommand(PGPUInt16 notifyCode, PGPUInt16 itemId, HWND ctrl);
	BOOL		OnInitDialog(HWND focusCtrl);
	void		OnKeyUp(PGPUInt32 vKey, PGPUInt32 keyData);
	PGPInt32	OnPsnSetActive();
	PGPInt32	OnPsnWizNext();
	PGPUInt32	OnSecureEditEvent(HWND view, PGPUInt32 event);
	
	PGPBoolean	PreProcess(const CMessage& msg, LRESULT& result);
};


// Class CNewUserAskPubKeyPage

class PGP::CNewUserAskPubKeyPage : public CPGPdiskPage
{
public:
	CNewUserAskPubKeyPage(CNewUserPages::PagesInfo *pInfo) : mPInfo(pInfo)
	{
		pgpAssertAddrValid(pInfo, CNewUserPages::PagesInfo);
	}

	~CNewUserAskPubKeyPage()
	{
	}
	
	PGPUInt16	GetDialogId() const
	{
		return IDD_WIZ_ASKPUBKEY;
	}

private:
	CNewUserPages::PagesInfo	*mPInfo;

	CPublicKeysView	mPubKeysView;

	DWORD	GetPageHelpId() const {return IDH_PGPDISKUI_WIZASKPUBKEYPAGE;}
	PVOID	GetPageHelpIds() const {return WizAskPubKeyPageHelpIDs;}

	void	InitPublicKeysView();

	BOOL		OnInitDialog(HWND focusCtrl);
	PGPUInt32	OnNotify(PGPUInt16 ctrlId, LPNMHDR pNMHDR);
	PGPInt32	OnPsnSetActive();
	PGPInt32	OnPsnWizNext();
};

// Class CNewUserAskChoicePage member functions

void 
CNewUserAskChoicePage::InitControls()
{
	if (mPInfo->keysToChoose.Count() == 0)
	{
		mPubKeyRadio.EnableWindow(FALSE);
		OnPassRadio();
	}
	else
	{
		OnPubKeyRadio();
	}

	if (mPInfo->isForAdminUser)
	{
		mROUserCheck.EnableWindow(FALSE);
		mROUserCheck.ShowWindow(FALSE);
	}
}

PGPBoolean 
CNewUserAskChoicePage::ValidateUserName(const char *userName)
{
	using namespace UCommonStrings;
	pgpAssertStrValid(userName);

	CMessageBox	messageBox;
	PGPUInt32	length	= strlen(userName);

	if (length == 0)
	{
		messageBox.Display(Get(kTellUserNameEmpty), mPInfo->warnDlgTitle, 
			Handle());
		return FALSE;
	}
	else if (length > kPGPdiskMaxUserNameSize)
	{
		CString	tooShort;
		tooShort.Format(Get(kTellUserNameTooLong), kPGPdiskMaxUserNameSize);

		messageBox.Display(tooShort, mPInfo->warnDlgTitle, Handle());
		return FALSE;
	}

	CPGPdiskUserIter	keyIter(*mPInfo->pDiskUsers);
	CPGPdiskUser		key;

	if (keyIter.SearchOnUserName(userName, key))
	{
		messageBox.Display(Get(kTellUserNameInUse), mPInfo->warnDlgTitle, 
			Handle());
		return FALSE;
	}

	return TRUE;
}

void 
CNewUserAskChoicePage::OnPassRadio()
{
	mPassRadio.SetCheck(BST_CHECKED);
	mPubKeyRadio.SetCheck(BST_UNCHECKED);

	mUserNameEdit.EnableWindow(TRUE);
	mUserNameText.EnableWindow(TRUE);

	mUserNameEdit.SetFocus();
}

void 
CNewUserAskChoicePage::OnPubKeyRadio()
{
	mPassRadio.SetCheck(BST_UNCHECKED);
	mPubKeyRadio.SetCheck(BST_CHECKED);

	mUserNameEdit.EnableWindow(FALSE);
	mUserNameText.EnableWindow(FALSE);
}

BOOL 
CNewUserAskChoicePage::OnCommand(
	PGPUInt16	notifyCode, 
	PGPUInt16	itemId, 
	HWND		ctrl)
{
	if (notifyCode == BN_CLICKED)
	{
		switch (itemId)
		{
		case IDC_PASS_RADIO:
			OnPassRadio();
			return TRUE;

		case IDC_PUBKEY_RADIO:
			OnPubKeyRadio();
			return TRUE;
		}
	}

	return CPGPdiskPage::OnCommand(notifyCode, itemId, ctrl);
}

BOOL 
CNewUserAskChoicePage::OnInitDialog(HWND focusCtrl)
{
	CPGPdiskPage::OnInitDialog(focusCtrl);

	// Attach controls.
	mPassRadio.Subclass(GetDlgItem(IDC_PASS_RADIO));
	mPubKeyRadio.Subclass(GetDlgItem(IDC_PUBKEY_RADIO));
	mROUserCheck.Subclass(GetDlgItem(IDC_ROUSER_CHECK));
	mUserNameEdit.Subclass(GetDlgItem(IDC_USERNAME_EDIT));
	mUserNameText.Subclass(GetDlgItem(IDC_USERNAME_CAPTION));

	// Prepare controls.
	InitControls();

	return TRUE;
}

PGPInt32 
CNewUserAskChoicePage::OnPsnSetActive()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	if (mPInfo->autoUseDefKey && 
		(mPInfo->keysToChoose.Count() > 0))
	{
		mPInfo->userChoseRO = FALSE;
		mPInfo->userChosePub = TRUE;

		return -1;
	}

	SetWizButtons(PSWIZB_BACK | PSWIZB_NEXT);

	return 0;
}

PGPInt32 
CNewUserAskChoicePage::OnPsnWizNext()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnWizNext();

	if (result != 0)
		return result;

	if (mPassRadio.GetCheck() == BST_CHECKED)
	{
		mPInfo->userChosePub = FALSE;

		CString	userName;
		mUserNameEdit.GetWindowText(userName);

		if (!ValidateUserName(userName))
		{
			mUserNameEdit.SetFocus();
			mUserNameEdit.SetSel(0, -1);
			result = -1;
		}
		else
		{
			mPInfo->userName = userName;
		}
	}
	else
	{
		mPInfo->userChosePub = TRUE;
	}

	if (!mPInfo->isForAdminUser)
		mPInfo->userChoseRO = (mROUserCheck.GetCheck() == BST_CHECKED);
	else
		mPInfo->userChoseRO = FALSE;

	return result;
}


// Class CNewUserBadPassPage member functions

void 
CNewUserBadPassPage::InitControls()
{
	// Create bad pass info bar.
	CRect	messageRect;

	mRejectText.GetWindowRect(messageRect);
	ScreenToClient(messageRect);

	CPoint	origin(messageRect.Left(), VertDLUToPixel(kDLUStartYForInfoBar));
	PGPUInt32	width = messageRect.Right() - messageRect.Left();

	mBadPassBar.Create(UModuleInstance::Get(), Handle(), origin, width);
}

BOOL 
CNewUserBadPassPage::OnInitDialog(HWND focusCtrl)
{
	CPGPdiskPage::OnInitDialog(focusCtrl);

	// Attach controls.
	mRejectText.Subclass(GetDlgItem(IDC_REJECT_TEXT));

	// Initialize controls.
	InitControls();

	return TRUE;
}

PGPInt32 
CNewUserBadPassPage::OnPsnSetActive()
{
	using namespace UCommonStrings;

	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	if (mPInfo->userChosePub || (mPInfo->badPassFlags == 0))
		return -1;

	mBadPassBar.Empty();
	PGPBoolean	rejected	= FALSE;

	if (mPInfo->badPassFlags & kBadPassConfirm)
	{
		rejected = TRUE;

		mBadPassBar.AddString(mBadPassBar.NumLines(), 
			Get(kNewUserBadPassConfirm), kBadConfirmStringId);
	}
	else
	{
		if ((mPInfo->badPassFlags & kBadPassMinLength) && 
			(mPInfo->enforcePassFlags & kEnforcePassMinLength))
		{
			rejected = TRUE;

			CString	minLengthString;

			minLengthString.Format(Get(kNewUserBadPassMinLengthReject), 
				mPInfo->minPassLength);

			mBadPassBar.AddString(mBadPassBar.NumLines(), minLengthString, 
				kBadMinLengthStringId);
		}

		if ((mPInfo->enforcePassFlags & kEnforcePassMinQuality) && 
			(mPInfo->badPassFlags & kBadPassMinQuality))
		{
			rejected = TRUE;

			CString	minQualityString;

			minQualityString.Format(Get(kNewUserBadPassMinQualityReject), 
				mPInfo->minPassQuality);

			mBadPassBar.AddString(mBadPassBar.NumLines(), minQualityString, 
				kBadMinQualityStringId);
		}
	}

	if (!rejected && (mPInfo->badPassFlags & kBadPassMinLength))
	{
		CString	minLengthString;

		minLengthString.Format(Get(kNewUserBadPassMinLengthWarn), 
			mPInfo->minPassLength);

		mBadPassBar.AddString(mBadPassBar.NumLines(), minLengthString, 
			kBadMinLengthStringId);
	}

	if (rejected)
	{
		SetWizButtons(PSWIZB_BACK);
		mBadPassBar.SetIcon(CInfoBar::kStopIcon);

		mRejectText.SetWindowText(Get(kNewUserPassRejectedText));
	}
	else
	{
		SetWizButtons(PSWIZB_BACK | PSWIZB_NEXT);
		mBadPassBar.SetIcon(CInfoBar::kInfoIcon);

		mRejectText.SetWindowText(Get(kNewUserPassWarningText));
	}

	mBadPassBar.ResizeRedraw();

	return 0;
}

PGPInt32 
CNewUserBadPassPage::OnPsnWizNext()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnWizNext();

	if (result != 0)
		return result;

	mPInfo->isFinished = TRUE;

	return 0;
}


// Class CNewUserAskPassPage member functions

void 
CNewUserAskPassPage::CreateCapsLockBar()
{
	// Create caps lock info bar.
	CRect	passRect;

	mPassphraseEdit.GetWindowRect(passRect);
	ScreenToClient(passRect);

	CPoint	origin(passRect.Left(), passRect.Top() - 
		VertDLUToPixel(kDLUYOffsetCapsLockBar));
	PGPUInt32	width	= HorizDLUToPixel(kDLUWidthCapsLockBar);

	mCapsLockBar.Create(UModuleInstance::Get(), Handle(), origin, width);
}

void 
CNewUserAskPassPage::InitDialogText()
{
	using namespace UCommonStrings;

	CString	lengthText;

	if (mPInfo->enforcePassFlags & kEnforcePassMinLength)
	{
		lengthText.Format(Get(kNewUserPassTellMinLengthText), 
			mPInfo->minPassLength);
	}
	else
	{
		lengthText.Format(Get(kNewUserPassSugMinLengthText), 
			mPInfo->minPassLength);
	}

	mPhraseLengthText.SetWindowText(lengthText);
}

void 
CNewUserAskPassPage::InitControls()
{
	mHideTypingCheck.SetCheck(BST_CHECKED);
	mPassphraseEdit.SetHideTypingPref(TRUE);
	mConfirmEdit.SetHideTypingPref(TRUE);

	mPassphraseEdit.SetMaxSizeContents(kPGPdiskMaxPassLength);
	mConfirmEdit.SetMaxSizeContents(kPGPdiskMaxPassLength);
	
	mIHadFocus = &mPassphraseEdit;
	
	mQualityBar.SetRange(0, 100);
	mQualityBar.SetPos(0);

	if (mPInfo->enforcePassFlags & kEnforcePassMinQuality)
	{
		mMinQualityBar.SetRange(0, 100);
		mMinQualityBar.SetPos(mPInfo->minPassQuality);
	}
	else
	{
		mMinQualityBar.ShowWindow(SW_HIDE);
		mMinQualityText.ShowWindow(SW_HIDE);
	}

	CreateCapsLockBar();
}

void 
CNewUserAskPassPage::InitQualityPrefs()
{
	mPInfo->enforcePassFlags = 0;
	mPInfo->badPassFlags = 0;

	// We warn about min pass length by default, but not quality.
	mPInfo->minPassLength = PGPdiskUIPriv::kDiskSugMinPassLength;
	mPInfo->minPassQuality = 0;

	if(PGPclEnterprise())
	{
		// Minimum passphrase length and quality.
		CPGPclPrefs	clPrefs(mPInfo->pContext->PGPclPrefs());

		if (clPrefs.GetBoolean(kPGPPrefEnforceMinChars))
		{
			mPInfo->enforcePassFlags |= kEnforcePassMinLength;
			mPInfo->minPassLength = clPrefs.GetNumber(kPGPPrefMinChars);
		}

		if (clPrefs.GetBoolean(kPGPPrefEnforceMinQuality))
		{
			mPInfo->enforcePassFlags |= kEnforcePassMinQuality;
			mPInfo->minPassQuality = clPrefs.GetNumber(kPGPPrefMinQuality);
		}
	}
}

void 
CNewUserAskPassPage::UpdatePassphraseFlags()
{
	mPInfo->badPassFlags = 0;

	// Passphrases must match.
	if (mPassphraseEdit.Contents() != mConfirmEdit.Contents())
		mPInfo->badPassFlags |= kBadPassConfirm;

	// Check minimum length.
	if (mPassphraseEdit.ContentsLength() < mPInfo->minPassLength)
		mPInfo->badPassFlags |= kBadPassMinLength;

	// Check minimum quality.
	if (mPInfo->enforcePassFlags & kEnforcePassMinQuality)
	{
		if (static_cast<PGPUInt32>(mQualityBar.GetPos()) < 
			mPInfo->minPassQuality)
		{
			mPInfo->badPassFlags |= kBadPassMinQuality;
		}
	}
}

void 
CNewUserAskPassPage::UpdatePassphraseQuality()
{
	CSecureArray<char>	passphrase;
	mPassphraseEdit.Contents().Extract(passphrase);

	PGPUInt32	quality	= pgpEstimatePassphraseQuality(passphrase.Get());
	mQualityBar.SetPos(quality);
}

void 
CNewUserAskPassPage::SetCapsLockMessageState()
{
	if (GetKeyState(VK_CAPITAL) & 1)
		WarnCapsLock(TRUE);
	else
		WarnCapsLock(FALSE);
}

void 
CNewUserAskPassPage::WarnCapsLock(PGPBoolean warn)
{
	PGPInt32	curIndex	= mCapsLockBar.GetIndexOfId(kCapsLockStringId);

	if (warn && (curIndex == -1))
	{
		mCapsLockBar.AddString(0, UCommonStrings::Get(
			UCommonStrings::kWarnCapsLockDown), kCapsLockStringId);
		mCapsLockBar.ResizeRedraw();
	}
	else if (!warn && (curIndex != -1))
	{
		mCapsLockBar.RemoveString(curIndex);
		mCapsLockBar.ResizeRedraw();
	}
}

void 
CNewUserAskPassPage::OnHideTypeCheck()
{
	PGPBoolean	hideTyping	= (mHideTypingCheck.GetCheck() == BST_CHECKED);

	mPassphraseEdit.SetHideTypingPref(hideTyping);
	mConfirmEdit.SetHideTypingPref(hideTyping);

	mIHadFocus->SetFocus();
}

BOOL 
CNewUserAskPassPage::OnCommand(
	PGPUInt16	notifyCode, 
	PGPUInt16	itemId, 
	HWND		ctrl)
{
	switch (notifyCode)
	{
	case BN_CLICKED:
		switch (itemId)
		{
		case IDC_HIDETYPE_CHECK:
			OnHideTypeCheck();
			return TRUE;
		}
		break;

	case EN_CHANGE:
		UpdatePassphraseQuality();
		break;

	case EN_KILLFOCUS:
		switch (itemId)
		{
		case IDC_PASSPHRASE_EDIT:
			mIHadFocus = &mPassphraseEdit;
			break;

		case IDC_CONFIRM_EDIT:
			mIHadFocus = &mConfirmEdit;
			break;
		}
		break;

	case EN_SETFOCUS:
		SetCapsLockMessageState();
		break;
	}

	return CPGPdiskPage::OnCommand(notifyCode, itemId, ctrl);
}

BOOL 
CNewUserAskPassPage::OnInitDialog(HWND focusCtrl)
{
	CPGPdiskPage::OnInitDialog(focusCtrl);

	// Attach controls.
	mHideTypingCheck.Subclass(GetDlgItem(IDC_HIDETYPE_CHECK));
	mMinQualityBar.Subclass(GetDlgItem(IDC_MINQUALITY_BAR));
	mQualityBar.Subclass(GetDlgItem(IDC_QUALITY_BAR));
	mPassphraseEdit.Subclass(GetDlgItem(IDC_PASSPHRASE_EDIT),mPInfo->pContext->PGPContext());
	mConfirmEdit.Subclass(GetDlgItem(IDC_CONFIRM_EDIT),mPInfo->pContext->PGPContext());
	mMinQualityText.Subclass(GetDlgItem(IDC_MINQUALITY_CAPTION));
	mPhraseLengthText.Subclass(GetDlgItem(IDC_PHRASELENGTH_TEXT));

	// Initialize page.
	InitQualityPrefs();
	InitControls();
	InitDialogText();

	return TRUE;
}

void 
CNewUserAskPassPage::OnKeyUp(PGPUInt32 vKey, PGPUInt32 keyData)
{
	SetCapsLockMessageState();
	CPGPdiskPage::OnKeyUp(vKey, keyData);
}

PGPInt32 
CNewUserAskPassPage::OnPsnSetActive()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	if (mPInfo->userChosePub)
		return -1;

	SetWizButtons(PSWIZB_BACK | PSWIZB_NEXT);
	SetCapsLockMessageState();

	mConfirmEdit.ClearContents();
	mPassphraseEdit.ClearContents();

	return 0;
}

PGPInt32 
CNewUserAskPassPage::OnPsnWizNext()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnWizNext();

	if (result != 0)
		return result;

	if (mPassphraseEdit.Contents().Length() == 0)
	{
		using namespace UCommonStrings;

		CMessageBox	messageBox;
		messageBox.Display(Get(kTellPassphraseEmpty), mPInfo->warnDlgTitle, 
			Handle());

		mPassphraseEdit.SetFocus();

		result = -1;
	}
	else
	{
		mPInfo->securePass = mPassphraseEdit.Contents();
		UpdatePassphraseFlags();

		if (mPInfo->badPassFlags == 0)
			mPInfo->isFinished = TRUE;
	}

	return result;
}

PGPUInt32 
CNewUserAskPassPage::OnSecureEditEvent(HWND view, PGPUInt32 event)
{
	switch (event)
	{
	case CSecureRichEdit::kContentsChangedEvent:
		UpdatePassphraseQuality();
		break;

	case CSecureRichEdit::kSawCapsLockKeyEvent:
		SetCapsLockMessageState();
		break;

	case CSecureRichEdit::kSawReturnKeyEvent:
		PressButton(PSBTN_NEXT);
		break;
	}

	return TRUE;
}

PGPBoolean 
CNewUserAskPassPage::PreProcess(const CMessage& msg, LRESULT& result)
{
	if (msg.Message() == CSecureRichEdit::WM_SECUREEDIT_EVENT)
	{
		result = OnSecureEditEvent(reinterpret_cast<HWND>(msg.WParam()), 
			msg.LParam());
		return TRUE;
	}

	return FALSE;
}


// Class CNewUserAskPubKeyPage member functions

void 
CNewUserAskPubKeyPage::InitPublicKeysView()
{
	// Initialize the keys view.
	CPGPclPrefs	clPrefs(mPInfo->pContext->PGPclPrefs());

	mPubKeysView.Initialize(mPInfo->pContext, mPInfo->singleSelect, 
		clPrefs.GetBoolean(kPGPPrefMarginalIsInvalid), 
		clPrefs.GetBoolean(kPGPPrefDisplayMarginalValidity));

	// Add keys to choose.
	CPGPKeyList	keyList(mPInfo->keysToChoose, kPGPKeyOrdering_Any);
	CPGPKeyIter	keyIter(keyList);
	CPGPKey		key;

	while (keyIter.Next(key))
	{
		CPGPKey	subKey;
		if (key.GetKeyForUsage(subKey))
			mPubKeysView.AddKey(key, subKey);
	}

	mPubKeysView.SortOnColumn(0);

	// Select default key.
	PGPInt32	index	= -1;
	
	if (mPInfo->defKeyPrefExists && mPInfo->defKeySeen)
		index = mPubKeysView.FindKeyIDIndex(mPInfo->defKeyID);

	if (index == -1)
		index = 0;

	mPubKeysView.SelectIndex(index);
	mPubKeysView.EnsureVisable(index, FALSE);
}

BOOL 
CNewUserAskPubKeyPage::OnInitDialog(HWND focusCtrl)
{
	CPGPdiskPage::OnInitDialog(focusCtrl);

	mPubKeysView.Subclass(GetDlgItem(IDC_KEYS_VIEW));
	InitPublicKeysView();

	return TRUE;
}

PGPUInt32 
CNewUserAskPubKeyPage::OnNotify(PGPUInt16 ctrlId, LPNMHDR pNMHDR)
{
	switch (pNMHDR->code)
	{
	case NM_DBLCLK:
	{
		NMITEMACTIVATE	*pNMIA	= reinterpret_cast<NMITEMACTIVATE *>(pNMHDR);

		if (mPubKeysView.GetSelectedCount() > 0)
			PressButton(PSBTN_NEXT);
	}
	break;

	default:
		return CPGPdiskPage::OnNotify(ctrlId, pNMHDR);
	}

	return 0;
}

PGPInt32 
CNewUserAskPubKeyPage::OnPsnSetActive()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	if (!mPInfo->userChosePub)
		return -1;

	if (mPInfo->autoUseDefKey)
	{
		// default key was attached earlier
		mPInfo->isFinished = TRUE;
		return -1;
	}

	SetWizButtons(PSWIZB_BACK | PSWIZB_NEXT);

	return 0;
}

PGPInt32 
CNewUserAskPubKeyPage::OnPsnWizNext()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnWizNext();

	if (result != 0)
		return result;

	// Get the key the user selected.
	CArray<CPublicKeysView::PubKeyItem>	items;
	PGPUInt32							numItems;

	mPubKeysView.GetSelectedItems(items, numItems);

	if (numItems == 0)
	{
		using namespace UCommonStrings;

		CMessageBox	messageBox;
		messageBox.Display(Get(kTellNoPubKeySelected), mPInfo->warnDlgTitle, 
			Handle());

		result = -1;
	}
	else if (numItems > 100)
	{
		CMessageBox	messageBox;
		messageBox.Display("Cannot select more than 100 keys at a time", 
			mPInfo->warnDlgTitle, 
			Handle());

//		pgpAssert(FALSE);	// should be single-selection control
		result = -1;
	}
	else
	{
		PGPUInt32 i;

		mPInfo->uNumPubKeys=numItems;

		for(i=0;i<mPInfo->uNumPubKeys;i++)
		{
			mPInfo->pgpKey[i].Attach(items[i].key);
		}
		mPInfo->isFinished = TRUE;
	}

	return result;
}


// Class CNewUserPages member functions

CNewUserPages::CNewUserPages() : 

	mIsCreated(FALSE), mPInfo(new PagesInfo), mPAskChoicePage(NULL), 
		mPAskPassPage(NULL), mPBadPassPage(NULL), mPAskPubKeyPage(NULL)
{
}

CNewUserPages::~CNewUserPages()
{
	try
	{
		Clear();
		delete mPInfo;
	}
	catch (CComboError&) { }
}

CPGPdiskPage * 
CNewUserPages::GetAskChoicePage()
{
	return mPAskChoicePage;
}

CPGPdiskPage * 
CNewUserPages::GetAskPassPage()
{
	return mPAskPassPage;
}

CPGPdiskPage * 
CNewUserPages::GetBadPassPage()
{
	return mPBadPassPage;
}

CPGPdiskPage * 
CNewUserPages::GetAskPubKeyPage()
{
	return mPAskPubKeyPage;
}

void 
CNewUserPages::Create(
	const CPGPdiskContext	*pContext, 
	const CPGPdiskUserSet	*pDiskUsers, 
	const char				*title, 
	const char				*warnDlgTitle,
	PGPBoolean				singleSelect,
	PGPBoolean				isForAdminUser, 
	PGPBoolean				ignoreDiskADK)
{
	pgpAssert(!IsCreated());
	pgpAssertAddrValid(pContext, CPGPdiskContext);
	pgpAssertAddrValid(pDiskUsers, CPGPdiskUserSet);
	pgpAssertStrValid(title);
	pgpAssertStrValid(warnDlgTitle);

	Clear();

	try
	{
		mPInfo->isFinished		= FALSE;
		mPInfo->pContext		= pContext;
		mPInfo->pDiskUsers		= pDiskUsers;
		mPInfo->isForAdminUser	= isForAdminUser;
		mPInfo->warnDlgTitle	= warnDlgTitle;
		mPInfo->uNumPubKeys		= 0;
		mPInfo->singleSelect	= singleSelect;

		mPAskChoicePage		= new CNewUserAskChoicePage(mPInfo);
		mPAskPassPage		= new CNewUserAskPassPage(mPInfo);
		mPBadPassPage		= new CNewUserBadPassPage(mPInfo);
		mPAskPubKeyPage		= new CNewUserAskPubKeyPage(mPInfo);

		PreCalcKeysToChoose();

		mPAskChoicePage->Create(title);
		mPBadPassPage->Create(title);
		mPAskPassPage->Create(title);
		mPAskPubKeyPage->Create(title);

		mIsCreated = TRUE;
	}
	catch (CComboError&)
	{
		Clear();
		throw;
	}
}

void 
CNewUserPages::GetChoices(
	PGPBoolean&		userChosePub, 
	CPGPKey*		pubKey, 
	PGPUInt32&		uNumPubKeys,
	CString&		userName, 
	CSecureString&	securePass, 
	PGPBoolean		&readOnly)
{
	pgpAssert(mPInfo->isFinished);

	if (mPInfo->userChosePub)
	{
		PGPUInt32 i;

		for(i=0;i<mPInfo->uNumPubKeys;i++)
		{
			pubKey[i].Attach(mPInfo->pgpKey[i].Get());
		}

		uNumPubKeys=mPInfo->uNumPubKeys;
	}

	userChosePub	= mPInfo->userChosePub;
	userName		= mPInfo->userName;
	securePass		= mPInfo->securePass;
	readOnly		= mPInfo->userChoseRO;
}

void 
CNewUserPages::Clear()
{
	if (IsntNull(mPAskChoicePage))
		delete mPAskChoicePage;

	if (IsntNull(mPAskPassPage))
		delete mPAskPassPage;

	if (IsntNull(mPBadPassPage))
		delete mPBadPassPage;

	if (IsntNull(mPAskPubKeyPage))
		delete mPAskPubKeyPage;

	mPAskChoicePage		= NULL;
	mPAskPassPage		= NULL;
	mPBadPassPage		= NULL;
	mPAskPubKeyPage		= NULL;

	mIsCreated = FALSE;
}

void 
CNewUserPages::PreCalcKeysToChoose()
{
	mPInfo->pContext->ReloadPGPKeyDB();
	mPInfo->keysToChoose.CreateEmpty(mPInfo->pContext->PGPKeyDB());

	// Get PGPdisk ADK key info.
	PGPBoolean	isDiskADK	= mPInfo->pContext->IsThereADK();
	CPGPKeyID	adkKeyID;

	if (isDiskADK && mPInfo->isForAdminUser)
	{
		CPGPKey	adkKey;

		mPInfo->pContext->GetADK(adkKey);
		adkKey.GetKeyID(adkKeyID);
	}

	// Get default key info
	CPGPclPrefs	clPrefs(mPInfo->pContext->PGPclPrefs());
	CPGPData	defKeyIDData;

	mPInfo->autoUseDefKey = FALSE;

	try
	{
		if(PGPclEnterprise())
		{
			if (mPInfo->isForAdminUser)
			{
				mPInfo->autoUseDefKey = clPrefs.GetBoolean(
					kPGPPrefDiskUseDefaultKey);
			}
		}
	}
	catch (CComboError&) {}

	try
	{
		clPrefs.GetData(kPGPPrefDefaultKeyID, defKeyIDData);

		mPInfo->defKeyID = *static_cast<PGPKeyID *>(defKeyIDData.Get());
		mPInfo->defKeyPrefExists = TRUE;
	}
	catch (CComboError&)
	{
		mPInfo->defKeyPrefExists = FALSE;
	}

	mPInfo->defKeySeen = FALSE;

	// Add appropriate public keys to the choice set.
	CPGPKeySet	pgpKeys(mPInfo->pContext->PGPKeyDB());
	CPGPKeyList	keyList(pgpKeys, kPGPKeyOrdering_Any);
	CPGPKeyIter	keyIter(keyList);
	CPGPKey		key;

	CPGPdiskUserIter	diskUserIter(*mPInfo->pDiskUsers);
	CPGPdiskUser		diskUser;

	while (keyIter.Next(key))
	{
		try
		{
			CPGPKeyID	keyID;
			key.GetKeyID(keyID);

			// Skip revoked, disabled, etc. keys.
			if (key.GetBooleanProp(kPGPKeyProperty_IsRevoked) || 
				key.GetBooleanProp(kPGPKeyProperty_IsDisabled) || 
				key.GetBooleanProp(kPGPKeyProperty_IsExpired) || 
				!key.GetBooleanProp(kPGPKeyProperty_IsEncryptionKey))
			{
				continue;
			}

			// Skip non-private keys if adding for admin user.
			if (mPInfo->isForAdminUser && 
				!key.GetBooleanProp(kPGPKeyProperty_CanDecrypt))
			{
				continue;
			}

			// Skip ADK if choosing for admin.
			if (isDiskADK && mPInfo->isForAdminUser)
			{
				if (keyID.IsEqual(adkKeyID))
					continue;
			}

			// Remember if saw default key.
			if (mPInfo->defKeyPrefExists && 
				!mPInfo->defKeySeen && 
				keyID.IsEqual(mPInfo->defKeyID))
			{
				mPInfo->defKeySeen = TRUE;
				mPInfo->pgpKey[0].Attach(key);
				mPInfo->uNumPubKeys=1;
			}

			// Skip keys already on disk.
			diskUserIter.Rewind();
			PGPBoolean	sawEqualKey	= FALSE;

			while (diskUserIter.Next(diskUser))
			{
				if (diskUser.HasPublicKey())
				{
					CPGPKeyID	diskKeyID;
					diskUser.GetPGPKeyID(diskKeyID);

					if (keyID.IsEqual(diskKeyID))
					{
						sawEqualKey = TRUE;
						break;
					}
				}
			}

			if (sawEqualKey)
				continue;

			// Add the key.
			mPInfo->keysToChoose.Add(key);
		}
		catch (CComboError&)
		{
			pgpAssert(FALSE);
		}
	}
}
