/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CPGPdiskEditorImpWin32.cpp,v 1.8 2002/08/06 20:09:23 dallen Exp $
____________________________________________________________________________*/

#include "pgpClassesConfig.h"

#include "CFile.h"
#include "CMessageBox.h"
#include "CPath.h"

#include "CPGPdiskEngine.h"

#include "CEditorWindow.h"
#include "CPassphraseDialog.h"
#include "CPGPdiskEditorImpWin32.h"
#include "PGPdiskUIPriv.h"
#include "SClaimedCommand.h"
#include "UEditMount.h"

_USING_PGP

// Class CPGPdiskEditorImpWin32 member functions

void 
CPGPdiskEditorImpWin32::AskEdit(
	const CPGPdiskContext&	context, 
	const char				*path, 
	void					*parentWnd)
{
	pgpAssertStrValid(path);

	// Need valid parent window.
	HWND	hwndParent	= static_cast<HWND>(parentWnd);

	if (IsNull(hwndParent))
		hwndParent = CWindow::GetDesktopWindow();

	// Need valid path.
	CPath	cpath(path);
	cpath.Canonicalize(kPGPdiskFileExtension);

	// resolve mounted disks
	CPGPdiskDiskSet		diskSet(context);
	CPGPdiskDiskIter	diskIter(diskSet);
	CPGPdiskDisk		disk;

	if (diskIter.SearchOnRoot(cpath, disk))
		disk.GetPath(cpath);

	// Check uniqueness of current command.
	SClaimedCommand	command(CCommandClaimer::kEditDisk, cpath);

	// Open the PGPdisk.
	disk.Clear();
	disk.Open(context, cpath, TRUE);

	// Check for in-progress re-encryptions.
	UEditMount::DetectAndHandleInProgressReEncrypts(context, disk, cpath, 
		hwndParent);

	// Check for wiped PGPdisks.
	UEditMount::DetectAndHandleWipedPGPdisks(context, disk, cpath, 
		hwndParent);

	// Show the editor.
	disk.Close();

	CEditorWindow	editorWindow;
	editorWindow.Display(context, cpath, hwndParent);
}
