/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CPublicKeysView.cpp,v 1.7 2002/09/24 19:58:28 wjb Exp $
____________________________________________________________________________*/

#include "pgpClassesConfig.h"
#include <stdio.h>

#include "UString.h"

#include "UModuleInstance.h"
#include "CBitmap.h"
#include "CBrush.h"
#include "CDC.h"
#include "CFont.h"
#include "CIcon.h"
#include "CImageList.h"
#include "CMenu.h"
#include "CPen.h"
#include "CRect.h"

#include "CPGPContext.h"
#include "CPGPdiskEngine.h"
#include "CPGPKey.h"
#include "UCommonStrings.h"

#include "CPublicKeysView.h"
#include "ImageIDs.h"

#include "pgpUnicode.h"

#include "Resource.h"

_USING_PGP

_UNNAMED_BEGIN

// Constants

const float	kUserIDColRatio		= 0.60F;
const float	kValidityColRatio	= 0.16F;
const float	kSizeColRatio		= 0.24F;

_UNNAMED_END


// Types

struct CPublicKeysView::PKViewDrawElements
{
	COLORREF	barColor;
	CBrush		stdBarBrush;
	CBrush		spcBarBrush;
	CPen		gSelTextPen;
	CPen		gUnselTextPen;
	CPen		hilightPen;
	CPen		shadowPen;
	CPen		buttonPen;

	CBrush		barBGBrush;
	CBrush		highBrush;
	CBrush		backBrush;
	CFont		hFont;
	CFont		hItalic;
	CFont		hStrikeOut;

	CImageList	image;
};


// Class CPublicKeysView member functions

CPublicKeysView::CPublicKeysView() : 

	mPContext(NULL), mPDE(NULL), mUserIDColIndex(0), mValidityColIndex(0), 
		mSizeColIndex(0)
{
}

CPublicKeysView::CPublicKeysView(HWND hWnd) : 
	CListView(hWnd), mPContext(NULL), mPDE(NULL)
{
}

PGPInt32 
CPublicKeysView::FindKeyIDIndex(const CPGPKeyID& keyID) const
{
	PubKeyItem	*pItem	= mItemList.Head();

	while (IsntNull(pItem))
	{
		if (keyID.IsEqual(pItem->keyID))
			return GetIndexOfItem(pItem);

		pItem = mItemList.Next(pItem);
	}

	return -1;
}

void 
CPublicKeysView::GetSelectedItems(
	CArray<PubKeyItem>&	items, 
	PGPUInt32&			numItems) const
{
	numItems = GetSelectedCount();
	items.Resize(numItems);

	PGPUInt32	itemCount	= GetItemCount();
	PGPUInt32	arrayIndex	= 0;

	for (PGPInt32 i = itemCount - 1; i >= 0; i--)
	{
		PubKeyItem	*pItem	= GetItemAtIndex(i);

		if (GetItemState(i, LVIS_SELECTED))
			items[arrayIndex++] = *pItem;
	}
}

void 
CPublicKeysView::SelectIndex(PGPInt32 i)
{
	SetItemState(i, LVIS_FOCUSED | LVIS_SELECTED, 
		LVIS_FOCUSED | LVIS_SELECTED);
	RedrawItems(i, i);
}

void 
CPublicKeysView::SelectAllItems()
{
	PGPInt32	count	= GetItemCount();

	for (PGPInt32 i = 0; i < count; i++)
	{
		if (i == 0)
		{
			SetItemState(i, LVIS_FOCUSED | LVIS_SELECTED, 
				LVIS_FOCUSED | LVIS_SELECTED);
		}
		else
		{
			SetItemState(i, LVIS_SELECTED, LVIS_SELECTED);
		}
	}

	RedrawItems(0, count - 1);
}

void 
CPublicKeysView::AddKey(const CPGPKey& key, const CPGPKey &subKey)
{
	auto_ptr<PubKeyItem>	pItem(new PubKeyItem);

	pItem->isOnlyKeyIDValid = FALSE;
	pItem->key = key;
	pItem->subKey = subKey;

	CPGPKeyID	keyID;
	key.GetKeyID(keyID);

	pItem->keyID	= keyID;
	pItem->readOnly	= FALSE;
	pItem->locked	= FALSE;

	AddKeyItem(pItem.get());
	pItem.release();
}

void 
CPublicKeysView::AddKey(
	const CPGPKeyID&		keyID, 
	PGPBoolean				readOnly, 
	PGPBoolean				locked)
{
	auto_ptr<PubKeyItem>	pItem(new PubKeyItem);

	pItem->isOnlyKeyIDValid = TRUE;
	pItem->keyID = keyID;

	pItem->readOnly = readOnly;
	pItem->locked = locked;

	AddKeyItem(pItem.get());
	pItem.release();
}

void 
CPublicKeysView::RemoveKeyAtIndex(PGPInt32 i)
{
	PubKeyItem	*pItem	= GetItemAtIndex(i);

	DeleteItem(i);
	delete mItemList.Remove(pItem);
}

void 
CPublicKeysView::SetActiveColumns(PGPUInt32 columns)
{
	mActiveColumns = columns;

	UpdateColumns();
	InvalidateRect(NULL, TRUE);
}

void 
CPublicKeysView::SortOnColumn(PGPUInt32 column)
{
	if (mLastSortColumn == column)
		mSortAscending = !mSortAscending;
	else
		mSortAscending = TRUE;

	mLastSortColumn = column;
	SortItems(PubKeyViewCompareProc, reinterpret_cast<LPARAM>(this));
}

void 
CPublicKeysView::Initialize(
	const CPGPdiskContext	*pContext, 
	PGPBoolean				singleSelect, 
	PGPBoolean				marginalInvalid, 
	PGPBoolean				displayMarginal)
{
	pgpAssertAddrValid(pContext, CPGPdiskContext);

	Clear();
	CreateDrawElements();

	mPContext = pContext;

	mSingleSelect		= singleSelect;
	mMarginalInvalid	= marginalInvalid;
	mDisplayMarginal	= displayMarginal;

	mSortAscending = FALSE;
	mLastSortColumn = 0;

	if (singleSelect)
		SetStyle(GetStyle() | LVS_SINGLESEL);

	SetActiveColumns(kUserIDColFlag | kValidityColFlag | kSizeColFlag);
	GenerateMeasureItem();
}

void 
CPublicKeysView::Clear()
{
	DeleteAllItems();
	mItemList.EmptyWithDelete();

	if (IsntNull(mPDE))
		DeleteDrawElements();
}

PGPUInt32 
CPublicKeysView::ConvertFromPGPValidity(PGPUInt32 uPGPValidity) const
{
	switch (uPGPValidity)
	{
	case kPGPValidity_Unknown:
	case kPGPValidity_Invalid:
		return 0;
	case kPGPValidity_Marginal:
		return 1;
	case kPGPValidity_Complete:
		return 2;
	}

	return 0;
}

PGPUInt32 
CPublicKeysView::ConvertFromPGPTrust(PGPUInt32 uPGPTrust) const
{
	switch (uPGPTrust & kPGPKeyTrust_Mask)
	{
	case kPGPKeyTrust_Undefined:
	case kPGPKeyTrust_Unknown:
	case kPGPKeyTrust_Never:
		return 0;
	case kPGPKeyTrust_Marginal:
		return 1;
	case kPGPKeyTrust_Complete:
		return 2;
	case kPGPKeyTrust_Ultimate:
		return 3;
	}

	return 0;
}

void 
CPublicKeysView::GetUnknownUserIDString(
	const CPGPKeyID&	keyID, 
	CString&			userID) const
{
	CString	keyIDStr;
	keyID.GetKeyIDString(kPGPKeyIDString_Abbreviated, keyIDStr);

	userID.Format(UCommonStrings::Get(
		UCommonStrings::kUnknownUserString), keyIDStr.Get());
}

int 
CPublicKeysView::PubKeyViewCompareProcAux(
	const PubKeyItem	*pKey1, 
	const PubKeyItem	*pKey2)
{
	pgpAssertAddrValid(pKey1, PubKeyItem);
	pgpAssertAddrValid(pKey2, PubKeyItem);

	if (mLastSortColumn == mUserIDColIndex)
	{
		if (mSortAscending)
			return PGPCompareUserIDStrings(pKey1->userID, pKey2->userID);
		else
			return PGPCompareUserIDStrings(pKey2->userID, pKey1->userID);
	}
	else if (mLastSortColumn == mValidityColIndex)
	{
		if (mSortAscending)
			return pKey2->validity - pKey1->validity;
		else
			return pKey1->validity - pKey2->validity;
	}
	else if (mLastSortColumn == mSizeColIndex)
	{
		if (mSortAscending)
			return (atoi(pKey2->sizeStr) - atoi(pKey1->sizeStr));
		else
			return (atoi(pKey1->sizeStr) - atoi(pKey2->sizeStr));
	}

	return 0;
}

int 
CALLBACK 
CPublicKeysView::PubKeyViewCompareProc(
	LPARAM	lParam1, 
	LPARAM	lParam2, 
	LPARAM	lParamSort)
{
	CPublicKeysView	*pView	= reinterpret_cast<CPublicKeysView *>(lParamSort);
	pgpAssertAddrValid(pView, CPublicKeysView);

	return pView->PubKeyViewCompareProcAux(
		reinterpret_cast<PubKeyItem *>(lParam1), 
		reinterpret_cast<PubKeyItem *>(lParam2));
}

void 
CPublicKeysView::DrawBar(
	CDC&			itemDC, 
	const CRect&	ptrBarRect, 
	PGPInt32		dataValue, 
	PGPInt32		maxValue, 
	PGPBoolean		isSelected) const
{
	CPen	oldPen;
	CRect	rect(ptrBarRect);

	if (isSelected)
	{
		itemDC.FillRect(rect, mPDE->highBrush);
		oldPen = itemDC.SelectObject(mPDE->gUnselTextPen.Get());
	}
	else
	{
		itemDC.FillRect(rect, mPDE->backBrush);
		oldPen = itemDC.SelectObject(mPDE->gSelTextPen.Get());
	}

	rect.Top()		+= 5;
	rect.Left()		+= 5;
	rect.Bottom()	-= 5;
	rect.Right()	-= 5;

	if (rect.Right() <= rect.Left())
		return;

	CBrush	oldBrush(itemDC.SelectObject(mPDE->barBGBrush.Get()));
	itemDC.SelectObject(mPDE->buttonPen.Get());

	itemDC.Rectangle(rect.Left() - 1, rect.Top() - 1, rect.Right() + 2, 
		rect.Bottom() + 2);

	itemDC.SelectObject(mPDE->shadowPen.Get());
	itemDC.MoveToEx(rect.Left(), rect.Bottom(), NULL);
	itemDC.LineTo(rect.Left(), rect.Top());
	itemDC.LineTo(rect.Right(), rect.Top());

	itemDC.SelectObject(mPDE->hilightPen.Get());
	itemDC.LineTo(rect.Right(), rect.Bottom());
	itemDC.LineTo(rect.Left(), rect.Bottom());

	CBrush	barBrush;

	if (maxValue != 0)
	{
		if (dataValue > maxValue)
		{
			barBrush.Attach(mPDE->spcBarBrush);
		}
		else
		{
			barBrush.Attach(mPDE->stdBarBrush);

			rect.Right() = rect.Left() + static_cast<PGPInt32>(
				(static_cast<float>(dataValue) / maxValue) * rect.Width());
		}
	}
	else
	{
		rect.Left() = rect.Right();
	}

	rect.Top()++;
	rect.Left()++;

	if (rect.Right() > rect.Left())
	{
		COLORREF	oldBkColor	= itemDC.SetBkColor(mPDE->barColor);
		itemDC.FillRect(rect, barBrush);

		rect.Top()--;
		rect.Left()--;

		// Highlight pen already selected.
		itemDC.MoveToEx(rect.Right(), rect.Top(), NULL);
		itemDC.LineTo(rect.Left(), rect.Top());
		itemDC.LineTo(rect.Left(), rect.Bottom());

		itemDC.SelectObject(mPDE->shadowPen.Get());
		itemDC.LineTo(rect.Right(), rect.Bottom());
		itemDC.LineTo(rect.Right(), rect.Top());

		itemDC.SetBkColor(oldBkColor);
	}

	itemDC.SelectObject(oldBrush.Get());
	itemDC.SelectObject(oldPen.Get());
}

void 
CPublicKeysView::DrawNoviceButton(
	CDC&			itemDC, 
	const CRect&	ptrBarRect, 
	PGPInt32		dataValue, 
	PGPInt32		maxValue, 
	PGPBoolean		isSelected) const
{
	if (isSelected)
		itemDC.FillRect(ptrBarRect, mPDE->highBrush);
	else
		itemDC.FillRect(ptrBarRect, mPDE->backBrush);

	if (ptrBarRect.Width() >= CIcon::kSmallIconWidth)
	{
		PGPUInt32	noviceIcon	= ImageIDs::kInvalid;

		if (dataValue > 2)
			noviceIcon = ImageIDs::kAxiomatic;

		if (dataValue == 2)
			noviceIcon = ImageIDs::kValid;

		if (!mMarginalInvalid && (dataValue == 1))
			noviceIcon = ImageIDs::kValid;

		mPDE->image.Draw(noviceIcon, itemDC, 
			ptrBarRect.Left() + 
			(ptrBarRect.Width() - CIcon::kSmallIconWidth)/2, 
			ptrBarRect.Top() + 
			(ptrBarRect.Height() - CIcon::kSmallIconWidth)/2, 
			ILD_TRANSPARENT);
	}
}

void 
CPublicKeysView::DrawTrueFalseButton(
	CDC&			itemDC, 
	const CRect&	ptrBarRect, 
	PGPBoolean		trueFalse, 
	PGPBoolean		isSelected) const
{
	if (isSelected)
		itemDC.FillRect(ptrBarRect, mPDE->highBrush);
	else
		itemDC.FillRect(ptrBarRect, mPDE->backBrush);

	if (ptrBarRect.Width() >= CIcon::kSmallIconWidth)
	{
		PGPUInt32	noviceIcon;

		if (trueFalse)
			noviceIcon = ImageIDs::kValid;
		else
			noviceIcon = ImageIDs::kInvalid;

		mPDE->image.Draw(noviceIcon, itemDC, 
			ptrBarRect.Left() + 
				(ptrBarRect.Width() - CIcon::kSmallIconWidth)/2, 
			ptrBarRect.Top() + 
				(ptrBarRect.Height() - CIcon::kSmallIconWidth)/2, 
			ILD_TRANSPARENT);
	}
}



//	_______________________________________________
//
//  Truncate text string

static BOOL sTruncateText (
		HDC		hdc, 
		WCHAR*	wsz, 
		int		xmax) 
{
	BOOL bShowToolTip	= FALSE;
	SIZE s;
	int l, w;

	// truncate at <newline>, if present
	l = 0;
	while (wsz[l])
	{
		if (wsz[l] == '\n')
		{
			wsz[l] = 0;
			bShowToolTip = TRUE;
			break;
		}
		l++;
	}

	GetTextExtentPoint32W (hdc, wsz, lstrlenW (wsz), &s);
	w = s.cx + 4;

	l = lstrlenW (wsz);
	if (l < 3) 
	{
		if (w > xmax) 
		{
			bShowToolTip = TRUE;
			wsz[0] = 0;
		}
	}
	else 
	{
		l = lstrlenW (wsz) - 3;

		while ((w > xmax) && (l >= 0))
		{
			bShowToolTip = TRUE;
			lstrcpyW (&wsz[l], L"...");
			GetTextExtentPoint32W (hdc, wsz, lstrlenW (wsz), &s);
			w = s.cx + 4;
			l--;
		}

		if (l < 0) 
			wsz[0] = 0;
	}

	return bShowToolTip;
}

#define MAXUSERIDSTRING				kPGPMaxUserIDSize + 32

void 
CPublicKeysView::DrawItemColumn(
	CDC&			itemDC, 
	const char		*userID, 
	const CRect&	prcClip) const
{
	PGPUInt32 ulen;
	WCHAR wsz[MAXUSERIDSTRING];

//	CString	cuserID(userID);
//	itemDC.FitStringToWidth(cuserID, prcClip.Width());

//	itemDC.ExtTextOut(prcClip.Left() + 2, prcClip.Top() + 1, 
//		ETO_CLIPPED | ETO_OPAQUE, prcClip, cuserID, cuserID.Length(), NULL);
	pgpUTF8StringToUCS2 (userID, kPGPUnicodeNullTerminated, 
			wsz, MAXUSERIDSTRING, &ulen);

	sTruncateText (itemDC.DC(), wsz, prcClip.Right() - prcClip.Left());

    // print the text
    ExtTextOutW(itemDC.DC(), prcClip.Left() + 2, prcClip.Top() + 1, 
               ETO_CLIPPED | ETO_OPAQUE,
               prcClip, 
			   wsz, 
			   lstrlenW(wsz), 
			   NULL);
}

void 
CPublicKeysView::DrawKeyItemAux(
	LPDRAWITEMSTRUCT	pDIS, 
	PGPBoolean			isFocused) const
{
	PubKeyItem	*pItem	= reinterpret_cast<PubKeyItem *>(pDIS->itemData);
	pgpAssertAddrValid(pItem, PubKeyItem);

	CDC		itemDC(pDIS->hDC);
	CRect	itemRect	= pDIS->rcItem;

	mPDE->image.Draw(pItem->icon, itemDC, PixelsLeftMargin, itemRect.Top(), 
		ILD_TRANSPARENT);

	itemRect.Left() = CIcon::kSmallIconWidth + PixelsLeftMargin;

	if (mActiveColumns & kUserIDColFlag)
	{
		itemRect.Right() = GetColumnWidth(mUserIDColIndex);

		CFont	oldFont(itemDC.SelectObject(mPDE->hFont.Get()));
		CRect	origRect(itemRect);

		if (pItem->isOnlyKeyIDValid)
			itemDC.SelectObject(mPDE->hStrikeOut.Get());

		// See if we need the lock icon.
		if (pItem->locked)
		{
			CRect	tempRect(itemRect);
			tempRect.Left() = tempRect.Right() - CIcon::kSmallIconWidth;

			// Enough space. Draw Lock.
			if (tempRect.Left() > itemRect.Left())
			{
				if ((pDIS->itemState & ODS_SELECTED) && isFocused)
					itemDC.FillRect(tempRect, mPDE->highBrush);
				else
					itemDC.FillRect(tempRect, mPDE->backBrush);
			}

			mPDE->image.Draw(ImageIDs::kClosedLock, itemDC, tempRect.Left(), 
				tempRect.Top(), ILD_TRANSPARENT);

			// Make space for lock from text.
			itemRect.Right() -= CIcon::kSmallIconWidth;
		}

		DrawItemColumn(itemDC, pItem->userID, itemRect);
		itemRect.Left() = itemRect.Right();
	}

	if (mActiveColumns & kValidityColFlag)
	{
		itemRect.Right() += GetColumnWidth(mValidityColIndex);

		if (mDisplayMarginal)
		{
			DrawBar(itemDC, itemRect, pItem->validity, 2, 
				((pDIS->itemState & ODS_SELECTED) && isFocused));
		}
		else
		{
			DrawNoviceButton(itemDC, itemRect, pItem->validity, 2, 
				((pDIS->itemState & ODS_SELECTED) && isFocused));
		}

		itemRect.Left() = itemRect.Right();
	}

	if (mActiveColumns & kSizeColFlag)
	{
		itemRect.Right() += GetColumnWidth(mSizeColIndex);

		CFont	oldFont(itemDC.SelectObject(mPDE->hFont.Get()));
		DrawItemColumn(itemDC, pItem->sizeStr, itemRect);

		itemDC.SelectObject(oldFont.Get());
		itemRect.Left() = itemRect.Right();
	}
}

void 
CPublicKeysView::DrawKeyItem(LPDRAWITEMSTRUCT pDIS) const
{
	PGPBoolean	isFocused	= TRUE;

	if (GetFocus() != Handle())
		isFocused = FALSE;

	CDC			dc(pDIS->hDC);
	PGPUInt32	uiFlags	= ILD_TRANSPARENT;

	// Check to see if this item is selected
	if ((pDIS->itemState & ODS_SELECTED) && isFocused)
	{
		// Set the text background and foreground colors
		dc.SetTextColor(GetSysColor(COLOR_HIGHLIGHTTEXT));
		dc.SetBkColor(GetSysColor(COLOR_HIGHLIGHT));

		// Also add the ILD_BLEND50 so the images come out selected
		uiFlags |= ILD_BLEND50;
	}
	else
	{
		// Set the text background and foreground colors to the 
		// standard window colors

		dc.SetTextColor(GetSysColor(COLOR_WINDOWTEXT));
		dc.SetBkColor(GetSysColor(COLOR_WINDOW));
	}

	DrawKeyItemAux(pDIS, isFocused);

	pDIS->rcItem.left += CIcon::kSmallIconWidth;

	if (!isFocused)
		return;

	// If we changed the colors for the selected item, undo it
	if (pDIS->itemState & ODS_SELECTED)
	{
		// Set the text background and foreground colors
		dc.SetTextColor(GetSysColor(COLOR_WINDOWTEXT));
		dc.SetBkColor(GetSysColor(COLOR_WINDOW));
	}

	// If the item is focused, now draw a focus rect around the entire row
//	if (pDIS->itemState & ODS_FOCUS)
//		dc.DrawFocusRect(pDIS->rcItem);
}

CPublicKeysView::PubKeyItem * 
CPublicKeysView::GetItemAtIndex(PGPInt32 i) const
{
	LVITEM	lvi;

	lvi.mask		= LVIF_PARAM;
	lvi.iItem		= i;							 
	lvi.iSubItem	= 0;

	GetItem(&lvi);

	return reinterpret_cast<PubKeyItem *>(lvi.lParam);
}

PGPInt32 
CPublicKeysView::GetIndexOfItem(const PubKeyItem *pItem) const
{
	LVFINDINFO	lvfi;
	pgpClearMemory(&lvfi, sizeof(lvfi));

	lvfi.flags	= LVFI_PARAM;
	lvfi.lParam	= reinterpret_cast<LPARAM>(pItem);

	return FindItem(-1, &lvfi);
}

void 
CPublicKeysView::UpdateKeyItemInfo(PubKeyItem *pItem)
{
	pgpAssertAddrValid(pItem, PubKeyItem);

	CPGPKeyID	keyID(pItem->keyID);

	if (pItem->isOnlyKeyIDValid)
	{
		pItem->icon = ImageIDs::kInvalidUserId;

		pItem->validity = 0;
		pItem->trust = 0;

		strcpy(pItem->sizeStr, UCommonStrings::Get(
			UCommonStrings::kUnknownUserSizeString));

		CString		userID;
		GetUnknownUserIDString(keyID, userID);

		strcpy(pItem->userID, userID);
	}
	else
	{
	 	CPGPKey	key(pItem->key);

		if (key.GetBooleanProp(kPGPKeyProperty_IsAxiomatic))
		{
			pItem->validity = ConvertFromPGPValidity(
				kPGPValidity_Complete) + 1;
			pItem->trust = ConvertFromPGPTrust(kPGPKeyTrust_Ultimate) + 1;
		}
		else
		{
			pItem->validity = ConvertFromPGPValidity(
				key.GetNumericProp(kPGPKeyProperty_Validity));
			pItem->trust = ConvertFromPGPTrust(
				key.GetNumericProp(kPGPKeyProperty_Trust));
		}

		if (keyID.Algorithm() == kPGPPublicKeyAlgorithm_RSA)
		{
			if (key.GetBooleanProp(kPGPKeyProperty_CanDecrypt))
				pItem->icon = ImageIDs::kRSASecKey;
			else
				pItem->icon = ImageIDs::kRSAPubKey;

			sprintf(pItem->sizeStr, "%i", 
				key.GetNumericProp(kPGPKeyProperty_Bits));
		}
		else
		{
			if (key.GetBooleanProp(kPGPKeyProperty_CanDecrypt))
				pItem->icon = ImageIDs::kDSASecKey;
			else
				pItem->icon = ImageIDs::kDSAPubKey;

			CPGPKey	subKey(pItem->subKey);

			sprintf(pItem->sizeStr, "%i/%i", 
				subKey.GetNumericProp(kPGPSubKeyProperty_Bits), 
				key.GetNumericProp(kPGPKeyProperty_Bits));
		}

		CString	userID;
		key.GetPrimaryUserIDName(userID);

		UString::SmartStringCopy(pItem->userID, userID, 
			sizeof(pItem->userID));
	}
}

void 
CPublicKeysView::AddKeyItem(PubKeyItem *pItem)
{
	pgpAssert(FindKeyIDIndex(pItem->keyID) == -1);

	UpdateKeyItemInfo(pItem);

	LV_ITEM  lvItem;
	pgpClearMemory(&lvItem, sizeof(lvItem));

	lvItem.mask 	= LVIF_PARAM | LVIF_IMAGE | LVIF_TEXT;
	lvItem.lParam	= reinterpret_cast<LPARAM>(pItem);
	lvItem.iImage	= 0;
	lvItem.pszText	= pItem->userID;
	lvItem.iSubItem	= 0;
	lvItem.iItem	= GetItemCount();

	InsertItem(&lvItem);
	mItemList.AddTail(pItem);
}

void 
CPublicKeysView::CreateDrawElements()
{
	pgpAssert(IsNull(mPDE));

	auto_ptr<PKViewDrawElements>	pDE(new PKViewDrawElements);

	pDE->stdBarBrush.CreateSolid(GetSysColor(COLOR_3DSHADOW));
	pDE->spcBarBrush.CreateHatch(HS_BDIAGONAL, GetSysColor(COLOR_WINDOW));
	pDE->gSelTextPen.Create(PS_SOLID, 0, GetSysColor(COLOR_WINDOWTEXT));
	pDE->gUnselTextPen.Create(PS_SOLID, 0, GetSysColor(COLOR_WINDOW));

	pDE->barColor = GetSysColor(COLOR_3DSHADOW);

	pDE->buttonPen.Create(PS_SOLID, 0, GetSysColor(COLOR_3DSHADOW));
	pDE->hilightPen.Create(PS_SOLID, 0, GetSysColor(COLOR_3DHILIGHT));
	pDE->barBGBrush.CreateSolid(GetSysColor(COLOR_3DFACE));
	pDE->highBrush.CreateSolid(GetSysColor(COLOR_HIGHLIGHT));
	pDE->backBrush.CreateSolid(GetSysColor(COLOR_WINDOW));

	LOGFONT lf;
	SystemParametersInfo(SPI_GETICONTITLELOGFONT, sizeof(LOGFONT), &lf, 0);

	pDE->hFont.CreateIndirect(lf);

	lf.lfItalic = !lf.lfItalic;
	pDE->hItalic.CreateIndirect(lf);

	lf.lfItalic = !lf.lfItalic;
	lf.lfStrikeOut = TRUE;
	pDE->hStrikeOut.CreateIndirect(lf);

	// Initialize image list.
	CDC	dc;
	dc.AttachFromWindow(NULL, TRUE);	// DC for desktop

	PGPUInt32	numBits	= dc.GetDeviceCaps(BITSPIXEL) * 
		dc.GetDeviceCaps(PLANES);

	if (numBits <= 8)
	{
		pDE->image.Create(CIcon::kSmallIconWidth, CIcon::kSmallIconHeight, 
			ILC_COLOR | ILC_MASK, ImageIDs::kNumPubKeyBitmaps, 0);

		CBitmap	images(UModuleInstance::Get(), 
			MAKEINTRESOURCE(IDB_PKIMAGES4BIT));
		pDE->image.AddMasked(images, ImageIDs::kMaskColor);
	}
	else
	{
		pDE->image.Create(CIcon::kSmallIconWidth, CIcon::kSmallIconHeight, 
			ILC_COLOR24 | ILC_MASK, ImageIDs::kNumPubKeyBitmaps, 0);

		CBitmap	images(UModuleInstance::Get(), 
			MAKEINTRESOURCE(IDB_PKIMAGES24BIT));
		pDE->image.AddMasked(images, ImageIDs::kMaskColor);
	}

	mPDE = pDE.release();
}

void 
CPublicKeysView::DeleteDrawElements()
{
	pgpAssert(IsntNull(mPDE));

	delete mPDE;
	mPDE = NULL;
}

void 
CPublicKeysView::UpdateColumns()
{
	using namespace UCommonStrings;

	DeleteColumn(mUserIDColIndex);
	DeleteColumn(mValidityColIndex);
	DeleteColumn(mSizeColIndex);

	CRect	listRect;
	GetClientRect(listRect);

	LV_COLUMN	lvCol;
	pgpClearMemory(&lvCol, sizeof(lvCol));

	lvCol.mask	= LVCF_WIDTH | LVCF_FMT | LVCF_SUBITEM | LVCF_TEXT;

	PGPUInt32	i			= 0;
	PGPInt32	listWidth	= listRect.Width();

	if (mActiveColumns & kUserIDColFlag)
	{
		lvCol.fmt		= LVCFMT_LEFT;
		lvCol.pszText	= const_cast<char *>(Get(kPubKeysViewUserIDCol));
		lvCol.cx		= static_cast<PGPInt32>(kUserIDColRatio*listWidth);

		mUserIDColIndex = InsertColumn(i++, &lvCol);
	}

	if (mActiveColumns & kValidityColFlag)
	{
		lvCol.fmt		= LVCFMT_CENTER;
		lvCol.pszText	= const_cast<char *>(Get(kPubKeysViewValidityCol));
		lvCol.cx		= static_cast<PGPInt32>(kValidityColRatio*listWidth);

		mValidityColIndex = InsertColumn(i++, &lvCol);
	}

	if (mActiveColumns & kSizeColFlag)
	{
		lvCol.fmt		= LVCFMT_LEFT;
		lvCol.pszText	= const_cast<char *>(Get(kPubKeysViewSizeCol));
		lvCol.cx		= static_cast<PGPInt32>(kSizeColRatio*listWidth);

		mSizeColIndex = InsertColumn(i++, &lvCol);
	}
}

void 
CPublicKeysView::GenerateMeasureItem() const
{
	// Fool the system into sending us a new WM_MEASUREITEM.
	CRect	rect;
	GetWindowRect(rect);

	WINDOWPOS	wp;

	wp.hwnd		= Handle();
	wp.cx		= rect.Width();
	wp.cy		= rect.Height();
	wp.flags	= SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | 
		SWP_NOZORDER;

	SendMessage(WM_WINDOWPOSCHANGED, 0, reinterpret_cast<LPARAM>(&wp));
}

BOOL 
CPublicKeysView::OnDrawItem(PGPUInt16 ctrlId, LPDRAWITEMSTRUCT pDIS)
{
	if (!IsWindowEnabled())
		return FALSE;

	switch (pDIS->itemAction)
	{
		case ODA_DRAWENTIRE:
		case ODA_FOCUS:
		case ODA_SELECT:
			DrawKeyItem(pDIS);
			break;
	}

	return TRUE;
}

void 
CPublicKeysView::OnKeyDown(PGPUInt32 vKey, PGPUInt32 keyData)
{
	switch (vKey)
	{
	case 'a':
	case 'A':
		// Select all items on ctrl-A.
		if (!mSingleSelect && (GetAsyncKeyState(VK_CONTROL) & 1))
			SelectAllItems();
		else
			CListView::OnKeyDown(vKey, keyData);
		break;

	default:
		CListView::OnKeyDown(vKey, keyData);
		break;
	}
}

BOOL 
CPublicKeysView::OnMeasureItem(PGPUInt16 ctrlId, LPMEASUREITEMSTRUCT pMIS)
{
    if (pMIS->CtlType != ODT_LISTVIEW)
		return FALSE;

	CDC		listDC;
	CFont	font(GetFont());

	listDC.AttachFromWindow(Handle());
	HFONT	oldFont	= listDC.SelectObject(font.Get());

	TEXTMETRIC	tm;

	if (!listDC.GetTextMetrics(&tm))
		return TRUE;
		
    // Add a little extra space between items.
	pMIS->itemHeight = tm.tmHeight + 1;

	if (pMIS->itemHeight < (CIcon::kSmallIconHeight + 1))
		pMIS->itemHeight = CIcon::kSmallIconHeight + 1;

	listDC.SelectObject(oldFont);
	listDC.Clear();

	return TRUE;
}

PGPUInt32 
CPublicKeysView::OnNotify(PGPUInt16 ctrlId, LPNMHDR pNMHDR)
{
	if (pNMHDR->hwndFrom == Handle())
	{
		NMLISTVIEW	*pNMLV	= reinterpret_cast<NMLISTVIEW *>(pNMHDR);

		if (pNMHDR->code == LVN_COLUMNCLICK)
			SortOnColumn(pNMLV->iSubItem);

		return 0;
	}
	else
	{
		return CListView::OnNotify(ctrlId, pNMHDR);
	}
}

void 
CPublicKeysView::OnNcDestroy()
{
	CListView::OnNcDestroy();
	Clear();
}

void 
CPublicKeysView::OnSysColorChange()
{
	CListView::OnSysColorChange();

	DeleteDrawElements();
	CreateDrawElements();
}
