/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CReEncryptWizard.cpp,v 1.16 2002/11/05 02:10:53 wjb Exp $
____________________________________________________________________________*/

#include "pgpClassesConfig.h"

#include "CFile.h"
#include "CMutex.h"
#include "CSecureString.h"
#include "CThread.h"

#include "CComboBox.h"
#include "CButton.h"
#include "CDC.h"
#include "CEdit.h"
#include "CUTF8Edit.h"
#include "CIcon.h"
#include "CImageList.h"
#include "CMessageBox.h"
#include "CPath.h"
#include "CPoint.h"
#include "CProgressBar.h"
#include "CRect.h"
#include "CStatic.h"
#include "UModuleInstance.h"

#include "Crypto.h"
#include "UCommonStrings.h"

#include "CPGPclPrefs.h"
#include "CPGPdiskEngine.h"
#include "CPGPGlobalRandomPool.h"

#include "CAlgorithmCombo.h"
#include "CReEncryptWizard.h"
#include "CPassphraseDialog.h"
#include "CPercentDone.h"
#include "CPGPdiskPage.h"
#include "CRandomDataPage.h"
#include "ImageIDs.h"
#include "PGPdiskHelp.h"
#include "Resource.h"

_USING_PGP

_UNNAMED_BEGIN

// Constants

DWORD	ReEncryptWizAlgoPageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_HEADING,			-1, 
	IDC_CURALGO_CAPTION,	IDH_PGPDISKUI_REENCRYPTWIZALGOPAGE_CURALGO, 
	IDC_CURALGO_EDIT,		IDH_PGPDISKUI_REENCRYPTWIZALGOPAGE_CURALGO, 
	IDC_ALGO_CAPTION,		IDH_PGPDISKUI_REENCRYPTWIZALGOPAGE_NEWALGO, 
	IDC_ALGO_COMBO,			IDH_PGPDISKUI_REENCRYPTWIZALGOPAGE_NEWALGO, 
	IDC_SAMEALGO_CHECK,		IDH_PGPDISKUI_REENCRYPTWIZALGOPAGE_SAMEALGO, 
	0,						0
};

DWORD	ReEncryptWizReEncryptPageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_DISKSTATIC3,		-1, 
	IDC_DISKSTATIC4,		-1, 
	IDC_DISKSTATIC5,		-1, 
	IDC_HEADING,			-1, 
	IDC_CURTASK_CAPTION,	IDH_PGPDISKUI_REENCRYPTWIZREENCRYPTPAGE_CURTASK, 
	IDC_CURTASK_TEXT,		IDH_PGPDISKUI_REENCRYPTWIZREENCRYPTPAGE_CURTASK, 
	IDC_CURTASK_BAR,		IDH_PGPDISKUI_REENCRYPTWIZREENCRYPTPAGE_CURTASK, 
	0,						0
};

DWORD	ReEncryptWizDonePageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_HEADING,			-1, 
	IDC_SIDEBAR,			-1, 
	0,						0
};

DWORD	ReEncryptWizIntroPageHelpIDs[]	= 
{
	IDC_DISKSTATIC1,		-1, 
	IDC_DISKSTATIC2,		-1, 
	IDC_HEADING,			-1, 
	IDC_SIDEBAR,			-1, 
	0,						0
};


// Types

struct ReEncryptInfo
{
	CComboError	error;

	const CPGPdiskContext	*pContext;
	CPGPdiskDisk			*pDisk;
	const CSecureString		*pPassphrase;
	PGPBoolean				resumingConversion;
	PGPBoolean				*pReEncryptionBegun;

	CPGPGlobalRandomPool		*pRandomPool;
	PGPdiskEncryptionAlgorithm	algorithm;
};


// Class CReEncryptWizAlgoPage

class CReEncryptWizAlgoPage : public CPGPdiskPage
{
public:
	CReEncryptWizAlgoPage(ReEncryptInfo *pInfo) : mPInfo(pInfo)
	{
		pgpAssertAddrValid(pInfo, ReEncryptInfo);
	}

	~CReEncryptWizAlgoPage()
	{
	}

	PGPUInt16	GetDialogId() const
	{
		return IDD_REWIZ_ALGO;
	}

	void	SavePrefs();

private:
	CAlgorithmCombo	mAlgoCombo;
	CButton			mSameAlgoCheck;
	CUTF8Edit			mCurAlgoEdit;

	ReEncryptInfo	*mPInfo;

	PGPBoolean					mSameAlgoValue;
	PGPdiskEncryptionAlgorithm	mPrevChoice;

	DWORD	GetPageHelpId() const {return IDH_PGPDISKUI_REENCRYPTWIZALGOPAGE;}
	PVOID	GetPageHelpIds() const {return ReEncryptWizAlgoPageHelpIDs;}

	void	InitControls();
	void	OnSameAlgoCheck();

	BOOL		OnCommand(PGPUInt16 notifyCode, PGPUInt16 itemId, HWND ctrl);
	BOOL		OnInitDialog(HWND focusCtrl);
	PGPInt32	OnPsnSetActive();
	PGPInt32	OnPsnWizNext();
};


// Class CReEncryptWizReEncryptPage

class CReEncryptWizReEncryptPage : public CPGPdiskPage
{
private:
	enum {WM_CONVERT_DONE = WM_USER + 10};

public:
	CReEncryptWizReEncryptPage(ReEncryptInfo *pInfo) : mPInfo(pInfo)
	{
		pgpAssertAddrValid(pInfo, ReEncryptInfo);
	}

	~CReEncryptWizReEncryptPage()
	{
	}

	PGPUInt16	GetDialogId() const
	{
		return IDD_REWIZ_REENCRYPT;
	}

private:
	CIcon			mDoingIcon;
	CIcon			mDoneIcon;
	CIcon			mFailedIcon;
	CImageList		mImages;
	CProgressBar	mCurTaskBar;
	CStatic			mCurTaskText;
	CStatic			mTaskIcon1;

	ReEncryptInfo	*mPInfo;

	CPath		mFileName;
	CThread		mReEncryptThread;
	CMutex		mCancelMutex;
	PGPBoolean	mShouldCancel;

	DWORD	GetPageHelpId() const
	{
		return IDH_PGPDISKUI_REENCRYPTWIZREENCRYPTPAGE;
	}

	PVOID	GetPageHelpIds() const {return ReEncryptWizReEncryptPageHelpIDs;}

	PGPBoolean			ReEncryptStatusCallbackAux(PGPUInt32 percent);
	static PGPBoolean	ReEncryptStatusCallback(PGPUInt32 percent, 
		void *userVal);

	void				ReEncryptCallbackAux();
	static void _cdecl	ReEncryptCallback(void *userVal);

	void	InitImages();

	void		OnReEncryptDone();
	BOOL		OnInitDialog(HWND focusCtrl);
	PGPBoolean	OnPsnQueryCancel();
	PGPInt32	OnPsnSetActive();

	PGPBoolean	PreProcess(const CMessage& msg, LRESULT& result);
};


// Class CReEncryptWizDonePage

class CReEncryptWizDonePage : public CPGPdiskPage
{
public:
	CReEncryptWizDonePage(ReEncryptInfo *pInfo) : mPInfo(pInfo)
	{
		pgpAssertAddrValid(pInfo, ReEncryptInfo);
	}

	~CReEncryptWizDonePage()
	{
	}

	PGPUInt16	GetDialogId() const
	{
		return IDD_REWIZ_DONE;
	}

private:
	ReEncryptInfo	*mPInfo;

	DWORD	GetPageHelpId() const {return IDH_PGPDISKUI_REENCRYPTWIZDONEPAGE;}
	PVOID	GetPageHelpIds() const {return ReEncryptWizDonePageHelpIDs;}

	PGPInt32	OnPsnSetActive();
};


// Class CReEncryptWizIntroPage

class CReEncryptWizIntroPage : public CPGPdiskPage
{
public:
	CReEncryptWizIntroPage(ReEncryptInfo *pInfo) : mPInfo(pInfo)
	{
		pgpAssertAddrValid(pInfo, ReEncryptInfo);
	}

	~CReEncryptWizIntroPage()
	{
	}
	
	PGPUInt16	GetDialogId() const
	{
		return IDD_REWIZ_INTRO;
	}

private:
	ReEncryptInfo	*mPInfo;

	DWORD	GetPageHelpId() const
	{
		return IDH_PGPDISKUI_REENCRYPTWIZINTROPAGE;
	}

	PVOID	GetPageHelpIds() const {return ReEncryptWizIntroPageHelpIDs;}

	BOOL		OnInitDialog(HWND focusCtrl);
	PGPInt32	OnPsnSetActive();
};


// Class CReEncryptWizAlgoPage member functions

void 
CReEncryptWizAlgoPage::SavePrefs()
{
	// Select last-used algorithm.
	CPGPclPrefs	prefs(mPInfo->pContext->PGPclPrefs());

	prefs.SetNumber(kPGPPrefDiskLastAlgorithm, mPInfo->algorithm);
}

_UNNAMED_END


// Class CReEncryptWizAlgoPage member functions

void 
CReEncryptWizAlgoPage::InitControls()
{
	mSameAlgoValue = FALSE;
	mSameAlgoCheck.SetCheck(BST_UNCHECKED);

	// Display existing algorithm in static edit box.
	CString	curAlgoText;
	CPGPdiskEngine::GetDiskAlgorithmIdString(mPInfo->algorithm, curAlgoText);

	mCurAlgoEdit.SetWindowText(curAlgoText);

	// Fill in combo box.
	mAlgoCombo.Fill();

	// Don't display existing algorithm in combo box.
	mAlgoCombo.Remove(mPInfo->algorithm);

	// Select last-used algorithm.
	CPGPclPrefs	prefs(mPInfo->pContext->PGPclPrefs());

	mAlgoCombo.Set(static_cast<PGPdiskEncryptionAlgorithm>(
		prefs.GetNumber(kPGPPrefDiskLastAlgorithm)));

	// Disable re-encryption to old CAST algorithm.
	if (mPInfo->algorithm == kPGPdiskOldCAST5Algorithm)
		mSameAlgoCheck.EnableWindow(FALSE);
}

void 
CReEncryptWizAlgoPage::OnSameAlgoCheck()
{
	if (mSameAlgoCheck.GetCheck() == BST_CHECKED)
	{
		mSameAlgoValue = TRUE;
		mPrevChoice = mAlgoCombo.Get();

		mAlgoCombo.Add(mPInfo->algorithm);
		mAlgoCombo.Set(mPInfo->algorithm);

		mAlgoCombo.EnableWindow(FALSE);
	}
	else
	{
		mSameAlgoValue = FALSE;

		mAlgoCombo.Remove(mPInfo->algorithm);
		mAlgoCombo.Set(mPrevChoice);

		mAlgoCombo.EnableWindow(TRUE);
	}
}

BOOL 
CReEncryptWizAlgoPage::OnCommand(
	PGPUInt16	notifyCode, 
	PGPUInt16	itemId, 
	HWND		ctrl)
{
	if (notifyCode == BN_CLICKED)
	{
		switch (itemId)
		{
		case IDC_SAMEALGO_CHECK:
			OnSameAlgoCheck();
			return TRUE;
		}
	}

	return CPGPdiskPage::OnCommand(notifyCode, itemId, ctrl);
}

BOOL 
CReEncryptWizAlgoPage::OnInitDialog(HWND focusCtrl)
{
	CPGPdiskPage::OnInitDialog(focusCtrl);

	// Attach controls.
	mSameAlgoCheck.Subclass(GetDlgItem(IDC_SAMEALGO_CHECK));
	mAlgoCombo.Subclass(GetDlgItem(IDC_ALGO_COMBO));
	mCurAlgoEdit.Subclass(GetDlgItem(IDC_CURALGO_EDIT));

	// Prepare controls.
	InitControls();

	return TRUE;
}

PGPInt32 
CReEncryptWizAlgoPage::OnPsnSetActive()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	SetWizButtons(PSWIZB_BACK | PSWIZB_NEXT);
	mCurAlgoEdit.SetSel(-1, -1);

	return 0;
}

PGPInt32 
CReEncryptWizAlgoPage::OnPsnWizNext()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnWizNext();

	if (result != 0)
		return result;

	if (!mSameAlgoValue)
		mPInfo->algorithm = mAlgoCombo.Get();

	SavePrefs();

	return 0;
}


// Class CReEncryptWizReEncryptPage member functions

PGPBoolean 
CReEncryptWizReEncryptPage::ReEncryptStatusCallbackAux(PGPUInt32 percent)
{
	mCurTaskBar.SetPosWTimeout(percent, 1000);

	mCancelMutex.Enter();
	mCancelMutex.Leave();

	return !mShouldCancel;
}

PGPBoolean 
CReEncryptWizReEncryptPage::ReEncryptStatusCallback(
	PGPUInt32	percent, 
	void		*userVal)
{
	CReEncryptWizReEncryptPage	*pPage	= static_cast<
		CReEncryptWizReEncryptPage *>(userVal);
	pgpAssertAddrValid(pPage, CReEncryptWizReEncryptPage);

	return pPage->ReEncryptStatusCallbackAux(percent);
}

void 
CReEncryptWizReEncryptPage::ReEncryptCallbackAux()
{
	try
	{
		CSecureArray<char>	passphrase;
		mPInfo->pPassphrase->Extract(passphrase);

		if (IsntNull(mPInfo->pReEncryptionBegun))
			*mPInfo->pReEncryptionBegun = TRUE;

		mPInfo->pDisk->ReEncryptDisk(passphrase.Get(), mPInfo->algorithm, 
			ReEncryptStatusCallback, this);

		PostMessage(WM_CONVERT_DONE);
	}
	catch (CComboError& error)
	{
		mPInfo->error = error;
		PostMessage(WM_CONVERT_DONE);
	}
}

void 
_cdecl 
CReEncryptWizReEncryptPage::ReEncryptCallback(void *userVal)
{
	CReEncryptWizReEncryptPage	*pPage	= static_cast<
		CReEncryptWizReEncryptPage *>(userVal);
	pgpAssertAddrValid(pPage, CReEncryptWizReEncryptPage);

	pPage->ReEncryptCallbackAux();
}

void 
CReEncryptWizReEncryptPage::InitImages()
{
	// initialize image list
	CDC	dc;
	dc.AttachFromWindow(NULL, TRUE);	// DC for desktop

	PGPUInt32	numBits	= dc.GetDeviceCaps(BITSPIXEL) * 
		dc.GetDeviceCaps(PLANES);

	if (numBits <= 8)
	{
		mImages.Create(CIcon::kSmallIconWidth, CIcon::kSmallIconHeight, 
			ILC_COLOR | ILC_MASK, ImageIDs::kNumImageBitmaps, 0);

		CBitmap	images(UModuleInstance::Get(), 
			MAKEINTRESOURCE(IDB_IMAGES4BIT));
		mImages.AddMasked(images, ImageIDs::kMaskColor);
	}
	else
	{
		mImages.Create(CIcon::kSmallIconWidth, CIcon::kSmallIconHeight, 
			ILC_COLOR24 | ILC_MASK, ImageIDs::kNumImageBitmaps, 0);

		CBitmap	images(UModuleInstance::Get(), 
			MAKEINTRESOURCE(IDB_IMAGES24BIT));
		mImages.AddMasked(images, ImageIDs::kMaskColor);
	}

	// load icons
	mDoingIcon.Attach(
		mImages.GetIcon(ImageIDs::kDoingIcon, ILD_TRANSPARENT), TRUE);
	mDoneIcon.Attach(
		mImages.GetIcon(ImageIDs::kDoneIcon, ILD_TRANSPARENT), TRUE);
	mFailedIcon.Attach(
		mImages.GetIcon(ImageIDs::kFailedIcon, ILD_TRANSPARENT), TRUE);
}

void 
CReEncryptWizReEncryptPage::OnReEncryptDone()
{
	using namespace UCommonStrings;
	mReEncryptThread.KillThread();

	if (mShouldCancel || mPInfo->error.IsError())
	{
		SetWizButtons(0);

		if (mPInfo->error.IsError() && 
			(mPInfo->error.pgpErr != kPGPError_UserAbort))
		{
			ReportError(kTellDiskReEncryptFailed, kReEncryptMsgBoxTitle, 
				mPInfo->error);

			mCurTaskBar.ShowWindow(SW_HIDE);
			mCurTaskText.SetWindowText(Get(kReEncryptProgFailedText));
		}
		else
		{
			mCurTaskText.SetWindowText(Get(kReEncryptProgCanceledText));
		}

		mTaskIcon1.SetIcon(mFailedIcon);
	}
	else
	{
		SetWizButtons(PSWIZB_NEXT);

		mCurTaskText.SetWindowText(Get(kReEncryptProgDoneText));
		mTaskIcon1.SetIcon(mDoneIcon);
		mCurTaskBar.SetPos(100);
	}
}

BOOL 
CReEncryptWizReEncryptPage::OnInitDialog(HWND focusCtrl)
{
	CPGPdiskPage::OnInitDialog(focusCtrl);

	mTaskIcon1.Subclass(GetDlgItem(IDC_TASKICON1));
	mCurTaskBar.Subclass(GetDlgItem(IDC_CURTASK_BAR));
	mCurTaskText.Subclass(GetDlgItem(IDC_CURTASK_TEXT));

	InitImages();

	return TRUE;
}

PGPBoolean 
CReEncryptWizReEncryptPage::OnPsnQueryCancel()
{
	using namespace UCommonStrings;

	if (!mReEncryptThread.IsRunning())
		return FALSE;	// nothing's happening, we're set
	else if (mShouldCancel)
		return TRUE;	// cancel in progress, hold tight

	// Pause the conversion thread, ask user for cancel permission.
	mCancelMutex.Enter();

	CMessageBox	messageBox;
	CMessageBox::Button	result	= messageBox.Display(
		Get(kAskIfCancelReEncrypt), Get(kReEncryptMsgBoxTitle), Handle(), 
		CMessageBox::kQuestionTone, CMessageBox::kYesNoStyle, 
		CMessageBox::kNoButton);

	mShouldCancel = (result == CMessageBox::kYesButton);
	mCancelMutex.Leave();

	if (mShouldCancel && mReEncryptThread.IsRunning())
	{
		mCurTaskText.SetWindowText(Get(kReEncryptProgFailingText));
		mCurTaskBar.SetPos(100);
	}

	return TRUE;
}

PGPInt32 
CReEncryptWizReEncryptPage::OnPsnSetActive()
{
	using namespace UCommonStrings;
	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	SetWizButtons(0);

	// init controls
	mCurTaskBar.SetRange(0, 100);
	mCurTaskBar.SetPos(0);

	CPath	path;
	mPInfo->pDisk->GetPath(path);

	CPath	fileName;
	path.GetFileNamePart(mFileName);
	mFileName.RemoveExtension();

	mCurTaskText.FitFormattedTextToWindow(
		Get(kReEncryptProgReEncryptingText), mFileName);
	
	mTaskIcon1.SetIcon(mDoingIcon);

	// start re-encryption
	mReEncryptThread.StartThread();

	try
	{
		mShouldCancel = FALSE;
		mReEncryptThread.PerformAsyncCallback(ReEncryptCallback, this);
	}
	catch (CComboError&)
	{
		mReEncryptThread.KillThread();
		throw;
	}

	return 0;
}

PGPBoolean 
CReEncryptWizReEncryptPage::PreProcess(const CMessage& msg, LRESULT& result)
{
	if (msg.Message() == WM_CONVERT_DONE)
	{
		OnReEncryptDone();
		result = 0;

		return TRUE;
	}

	return FALSE;
}


// Class CReEncryptWizDonePage member functions

PGPInt32 
CReEncryptWizDonePage::OnPsnSetActive()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	SetWizButtons(PSWIZB_FINISH);

	return 0;
}


// Class CReEncryptWizIntroPage member functions

BOOL 
CReEncryptWizIntroPage::OnInitDialog(HWND focusCtrl)
{
	CPGPdiskPage::OnInitDialog(focusCtrl);

	CWindow(GetParent()).Center();

	return FALSE;
}

PGPInt32 
CReEncryptWizIntroPage::OnPsnSetActive()
{
	PGPInt32	result	= CPGPdiskPage::OnPsnSetActive();

	if (result != 0)
		return result;

	if (mPInfo->resumingConversion)
		return IDD_REWIZ_REENCRYPT;

	SetWizButtons(PSWIZB_NEXT);

	return 0;
}


// Class CReEncryptWizard member functions

void 
CReEncryptWizard::Display(
	const CPGPdiskContext&	context, 
	const char				*path, 
	HWND					parentWnd, 
	PGPBoolean				*pReEncryptionBegun)
{
	using namespace UCommonStrings;

	pgpAssertStrValid(path);
	pgpAssert(CWindow(parentWnd).IsWindow());

	if (IsntNull(pReEncryptionBegun))
		*pReEncryptionBegun = FALSE;

	// Open the PGPdisk.
	CPGPdiskDisk	disk;
	disk.Open(context, path);

	// Get admin user.
	CPGPdiskUserSet	diskSet;
	disk.ExportUserSet(diskSet);

	CPGPdiskUserIter	userIter(diskSet);
	CPGPdiskUser		adminUser;

	if (!userIter.FindAdminUser(adminUser))
		THROW_PGPERROR(kPGPError_ItemNotFound);

	// Make sure user wants to remove passphrases and missing public keys.
	CPGPdiskUser	user;
	PGPBoolean		sawAtRiskUser	= FALSE;

	userIter.Rewind();

	while (userIter.Next(user))
	{
		if (!user.IsAdminUser() && (user.HasSymmetricKey() || 
			IsPubNotInRing(user)))
		{
			sawAtRiskUser = TRUE;
		}
	}

	if (sawAtRiskUser)
	{
		CMessageBox	messageBox;
		CMessageBox::Button	result	= messageBox.Display(
			Get(kAskIfNukeSymsAndMissingPubs), 
			Get(kReEncryptMsgBoxTitle), parentWnd, 
			CMessageBox::kWarningTone, CMessageBox::kYesNoStyle, 
			CMessageBox::kNoButton);

		if (result == CMessageBox::kNoButton)
			THROW_PGPERROR(kPGPError_UserAbort);
	}

	// Get passphrase for admin user.
	CPath	fileName;

	CPath(path).GetFileNamePart(fileName);
	fileName.RemoveExtension();

	CPGPdiskUserSet		adminUserSet(adminUser);
	CPassphraseDialog	adminPassDlg;

	adminPassDlg.AskForPassphrase(disk, adminUserSet, parentWnd, adminUser, 
		Get(kSinglePassAdminText), fileName);

	// Create the wizard pages.
	ReEncryptInfo			ciInfo;
	CPGPGlobalRandomPool	randomPool(context.PGPContext());
	const char				*title	= Get(kReEncryptTitle);

	ciInfo.pContext				= &context;
	ciInfo.pDisk				= &disk;
	ciInfo.pPassphrase			= &adminPassDlg.Passphrase();
	ciInfo.resumingConversion	= disk.IsBeingReEncrypted();
	ciInfo.pReEncryptionBegun	= pReEncryptionBegun;
	ciInfo.pRandomPool			= &randomPool;
	ciInfo.algorithm			= disk.Algorithm();

	CReEncryptWizAlgoPage		algoPage(&ciInfo);
	CReEncryptWizReEncryptPage	convertPage(&ciInfo);
	CReEncryptWizDonePage		donePage(&ciInfo);
	CReEncryptWizIntroPage		introPage(&ciInfo);
	CRandomDataPage				randomPage(&ciInfo.algorithm, 
		ciInfo.pRandomPool);

	LoadSidebar();

	algoPage.Create(title);
	randomPage.Create(title);
	convertPage.Create(title);
	donePage.Create(title, FALSE, &mSidebarBitmap, &mSidebarPalette);
	introPage.Create(title, FALSE, &mSidebarBitmap, &mSidebarPalette);

	AddPage(&introPage);
	AddPage(&algoPage);
	AddPage(&randomPage);
	AddPage(&convertPage);
	AddPage(&donePage);

	// Show the wizard.
	CPropertySheet::Create(parentWnd, TRUE);

	// flush prefs
	try
	{
		CPGPclPrefs	prefs(context.PGPclPrefs());
		prefs.Flush();
	}
	catch (CComboError&) { }
}

PGPBoolean 
CReEncryptWizard::IsPubNotInRing(const CPGPdiskUser& user) const
{
	CPGPKeyID	keyID;
	user.GetPGPKeyID(keyID);

	PGPPublicKeyAlgorithm	algorithm	= keyID.Algorithm();

	try
	{
		CPGPdiskContext	context(user.Context());
		CPGPKey	pgpKey(context.PGPKeyDB(), keyID);

		return FALSE;
	}
	catch (CComboError&)
	{
		return TRUE;
	}
}

void 
CReEncryptWizard::LoadSidebar()
{
	CDC	dc;
	dc.AttachFromWindow(NULL);

	PGPUInt32	numBits	= dc.GetDeviceCaps(BITSPIXEL) * 
		dc.GetDeviceCaps(PLANES);

	PGPUInt32	uintRsrc;

	if (numBits <= 1)
		uintRsrc = IDB_SIDEBAR1;
	else if (numBits <= 4)
		uintRsrc = IDB_SIDEBAR4;
	else
		uintRsrc = IDB_SIDEBAR8;

	mSidebarBitmap.CreateWithPalette(UModuleInstance::Get(), 
		MAKEINTRESOURCE(uintRsrc), mSidebarPalette);
}
