/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CPGPdiskImpDrv98.cpp,v 1.6 2002/08/06 20:09:30 dallen Exp $
____________________________________________________________________________*/

#include "pgpClassesConfig.h"
#include <winerror.h>

#include "CString.h"
#include "UDebug.h"
#include "UMath.h"

#include "UFileSys.h"

#include "CFilesystemWatcher.h"
#include "CIop.h"
#include "CPGPdiskImpDrv98.h"
#include "CDriverSubsystemsDrv98.h"
#include "UConstantNamesDrv98.h"

_USING_PGP

// Class CPGPdiskImpDrv98 member functions

CComboError 
CPGPdiskImpDrv98::Read(
	void		*buf, 
	PGPUInt64	posInBlocks, 
	PGPUInt32	nBlocks) const
{
	CComboError	error;

	// Satisfy from cache if bootblock read.
	if (posInBlocks == 0)
	{
		pgpAssertAddrValid(buf, VoidAlign);

		UDebug::DebugOut("PGPdisk: Reading PGPdisk %s from bootblock "
			"cache.\n", Root());

		pgpCopyMemory(mBootBlock, buf, kPGPdiskBlockSize);

		if (nBlocks > 1)
		{
			error = CPGPdiskImp::Read(static_cast<PGPUInt8 *>(buf) + 
				kPGPdiskBlockSize, 1, nBlocks - 1);
		}
	}
	else
	{
		error = CPGPdiskImp::Read(buf, posInBlocks, nBlocks);
	}

	return error;
}

CComboError 
CPGPdiskImpDrv98::Write(
	const void	*buf, 
	PGPUInt64	posInBlocks, 
	PGPUInt32	nBlocks)
{
	CComboError	error;

	// Update cache if bootblock written.
	if (posInBlocks == 0)
	{
		pgpAssertAddrValid(buf, VoidAlign);
		pgpCopyMemory(buf, mBootBlock, kPGPdiskBlockSize);
	}

	error = CPGPdiskImp::Write(buf, posInBlocks, nBlocks);

	return error;
}

CComboError 
CPGPdiskImpDrv98::Unmount(PGPBoolean isForced)
{
	CComboError	error;

	error = CPGPdiskImp::Unmount(isForced);

	if (error.IsntError())
	{
		CDriverSubsystemsDrv98::FilesystemWatcher().StopWatchingForBuffer(
			mBufferToWatchFor);
		mBufferToWatchFor = NULL;
	}

	return error;
}

CComboError 
CPGPdiskImpDrv98::CalcBlockSizeOfHost(
	const char	*path, 
	PGPUInt32&	hostBlockSize) const
{
	pgpAssertStrValid(path);

	CComboError	error;

	// Determine block size.
	CString	hostRoot;
	CVolume	host;

	error = hostRoot.Status();

	if (error.IsntError())
		hostRoot.Format("%c:\\", path[0]);

	if (error.IsntError())
	{
		error = host.AttachVolume(hostRoot);

		if (error.IsntError())
		{
			hostBlockSize = host.BlockSize();
			host.DetachVolume();
		}

		// Network path if error.
		if (error.IsError())
		{
			error = CComboError();
			hostBlockSize = FileSys::kTypicalBlockSize;
		}
	}

	return error;
}

CComboError 
CPGPdiskImpDrv98::GetDevParamsFat16(FileSys::DevParamsFat16& dp16) const
{
	pgpAssert(IsMounted());

	CComboError	error;

	// concoct a likely story
	FileSys::Type		fsType;
	PGPUInt32			secsPerTrack, tracksPerCyl;
	PGPUInt64			cylinders, megsDisk;
	UFileSys::FatData	fatData;

	pgpClearMemory(&dp16, sizeof(dp16));
	megsDisk = UFileSys::CalcMegsDisk(TotalBlocks(), kPGPdiskBlockSize);

	error = GetFsId(fsType);

	if (error.IsntError() && UFileSys::IsFatFileSys(fsType))
	{
		// use the boot block
		FileSys::BootSectorFat16	*pBS16	= 
			reinterpret_cast<FileSys::BootSectorFat16 *>(mBootBlock);

		dp16.dpDevType		= 0x05;		// we are a hard disk
		dp16.dpDevAttr		= 1;		// we are not removable
		dp16.dpCylinders	= FileSys::kMaxCyls - 1;
		dp16.dpMediaType	= pBS16->bsMedia;

		dp16.dpBytesPerSec	= pBS16->bsBytesPerSec;
		dp16.dpSecPerClust	= pBS16->bsSecPerClust;
		dp16.dpResSectors	= pBS16->bsResSectors;
		dp16.dpFats			= pBS16->bsFats;
		dp16.dpRootDirEnts	= pBS16->bsRootDirEnts;
		dp16.dpSectors		= pBS16->bsSectors;
		dp16.dpMedia		= pBS16->bsMedia;
		dp16.dpFatSecs		= pBS16->bsFatSecs;
		dp16.dpSecPerTrack	= pBS16->bsSecPerTrack;
		dp16.dpHeads		= pBS16->bsHeads;
		dp16.dpHiddenSecs	= pBS16->bsHiddenSecs;
		dp16.dpHugeSectors	= pBS16->bsHugeSectors;
	}
	else
	{
		// the boot block is wigged, fake it
		error = CComboError();

		if (megsDisk < FileSys::kMinFat16Megs)
			fsType = FileSys::kFat12FileSys;
		else
			fsType = FileSys::kFat16FileSys;

		UFileSys::InitFatData(static_cast<PGPUInt32>(TotalBlocks()), fsType, 
			fatData);
		UFileSys::CalcFakeFatGeom(kPGPdiskBlockSize, tracksPerCyl, 
			cylinders, secsPerTrack);

		dp16.dpDevType		= 0x05;				// we are a hard disk
		dp16.dpDevAttr		= 1;				// we are not removable
		dp16.dpCylinders	= static_cast<PGPUInt16>(cylinders);
		dp16.dpMediaType	= FileSys::kFatMediaByte;

		dp16.dpBytesPerSec	= kPGPdiskBlockSize;
		dp16.dpSecPerClust	= static_cast<PGPUInt8>(fatData.fdSpc);
		dp16.dpResSectors	= fatData.fdReservedSecs;
		dp16.dpFats			= fatData.fdFatCount;
		dp16.dpRootDirEnts	= fatData.fdRootDirEnts;
		dp16.dpSectors		= static_cast<PGPUInt16>(megsDisk < 32 ? 
			TotalBlocks() : 0);
		dp16.dpMedia		= FileSys::kFatMediaByte;
		dp16.dpFatSecs		= static_cast<PGPUInt16>(fatData.fdFatSize);
		dp16.dpSecPerTrack	= static_cast<PGPUInt16>(secsPerTrack);
		dp16.dpHeads		= static_cast<PGPUInt16>(tracksPerCyl);
		dp16.dpHiddenSecs	= 0;
		dp16.dpHugeSectors	= static_cast<PGPUInt32>(megsDisk >= 32 ? 
			TotalBlocks() : 0);
	}

	return error;
}

CComboError 
CPGPdiskImpDrv98::GetDevParamsFat32(FileSys::DevParamsFat32 &dp32) const
{
	pgpAssert(IsMounted());

	CComboError	error;

	// concoct a likely story
	FileSys::Type		fsType;
	PGPUInt32			secsPerTrack, tracksPerCyl;
	PGPUInt64			cylinders, megsDisk;
	UFileSys::FatData	fatData;
	
	pgpClearMemory(&dp32, sizeof(dp32));

	megsDisk = UFileSys::CalcMegsDisk(TotalBlocks(), 
		kPGPdiskBlockSize);

	error = GetFsId(fsType);

	if (error.IsntError() && UFileSys::IsFatFileSys(fsType))
	{
		FileSys::BootSectorFat32	*pBS32	= 
			reinterpret_cast<FileSys::BootSectorFat32 *>(mBootBlock);

		dp32.dpDevType		= 0x05;			// we are a hard disk
		dp32.dpDevAttr		= 1;			// we are not removable
		dp32.dpCylinders	= FileSys::kMaxCyls - 1;
		dp32.dpMediaType	= pBS32->bsMedia;

		dp32.dpBytesPerSector	= pBS32->bsBytesPerSec;
		dp32.dpSectorsPerClust	= pBS32->bsSecPerClust;
		dp32.dpReservedSectors	= pBS32->bsResSectors;
		dp32.dpNumberOfFats		= pBS32->bsFats;
		dp32.dpRootEntries		= pBS32->bsRootDirEnts;
		dp32.dpTotalSectors		= pBS32->bsSectors;
		dp32.dpMediaDescriptor	= pBS32->bsMedia;
		dp32.dpSectorsPerFat	= pBS32->bsFatSecs;
		dp32.dpSectorsPerTrack	= pBS32->bsSecPerTrack;
		dp32.dpHeads			= pBS32->bsHeads;
		dp32.dpHiddenSectors	= pBS32->bsHiddenSecs;
		dp32.dpBigTotalSectors	= pBS32->bsHugeSectors;
		dp32.dpBigSectorsPerFat	= pBS32->bsBigSectorsPerFat;
		dp32.dpExtFlags			= pBS32->bsExtFlags;
		dp32.dpFS_Version		= pBS32->bsFS_Version;
		dp32.dpRootDirStrtClus	= pBS32->bsRootDirStrtClus;
		dp32.dpFsInfoSec		= pBS32->bsFsInfoSec;
		dp32.dpBkUpBootSec		= pBS32->bsBkUpBootSec;
	}
	else
	{
		// the boot block is wigged, fake it
		error = CComboError();

		if (megsDisk < FileSys::kMinFat16Megs)
			fsType = FileSys::kFat12FileSys;
		else if (megsDisk < FileSys::kMinFat32Megs)
			fsType = FileSys::kFat16FileSys;
		else
			fsType = FileSys::kFat32FileSys;

		UFileSys::InitFatData(static_cast<PGPUInt32>(TotalBlocks()), fsType, 
			fatData);
		UFileSys::CalcFakeFatGeom(kPGPdiskBlockSize, tracksPerCyl, cylinders, 
			secsPerTrack);

		dp32.dpDevType		= 0x05;			// we are a hard disk
		dp32.dpDevAttr		= 1;			// we are not removable
		dp32.dpCylinders	= static_cast<PGPUInt16>(cylinders);
		dp32.dpMediaType	= FileSys::kFatMediaByte;

		dp32.dpBytesPerSector	= kPGPdiskBlockSize;
		dp32.dpSectorsPerClust	= static_cast<PGPUInt8>(fatData.fdSpc);
		dp32.dpReservedSectors	= fatData.fdReservedSecs;
		dp32.dpNumberOfFats		= fatData.fdFatCount;
		dp32.dpRootEntries		= fatData.fdRootDirEnts;
		dp32.dpTotalSectors		= static_cast<PGPUInt16>(
			megsDisk < 32 ? TotalBlocks() : 0);
		dp32.dpMediaDescriptor	= FileSys::kFatMediaByte;
		dp32.dpSectorsPerFat	= (fsType == FileSys::kFat32FileSys ? 0 : 
			static_cast<PGPUInt16>(fatData.fdFatSize));
		dp32.dpSectorsPerTrack	= static_cast<PGPUInt16>(secsPerTrack);
		dp32.dpHeads			= static_cast<PGPUInt16>(tracksPerCyl);
		dp32.dpHiddenSectors	= 0;
		dp32.dpBigTotalSectors	= static_cast<PGPUInt32>(
			megsDisk >= 32 ? TotalBlocks() : 0);
		dp32.dpBigSectorsPerFat	= (fsType != FileSys::kFat32FileSys ? 0 : 
			fatData.fdFatSize);
	}

	return error;
}

CComboError 
CPGPdiskImpDrv98::GetFsId(FileSys::Type &fsType) const
{
	pgpAssert(IsMounted());

	CComboError	error;
	fsType = FileSys::kUnknownFileSys;

	FileSys::MID	mid;
	error = GetMID(mid);

	if (error.IsntError())
	{
		char	*fsStr	= reinterpret_cast<char *>(mid.midFileSysType);

		if (!strncmp(fsStr, FileSys::kFat12IdStr, 5))
			fsType = FileSys::kFat12FileSys;
		else if (!strncmp(fsStr, FileSys::kFat16IdStr, 5))
			fsType = FileSys::kFat16FileSys;
		else if (!strncmp(fsStr, FileSys::kFat32IdStr, 5))
			fsType = FileSys::kFat32FileSys;
	}

	return error;
}

CComboError 
CPGPdiskImpDrv98::GetMID(FileSys::MID& mid) const
{
	pgpAssert(IsMounted());

	CComboError	error;
	PGPBoolean	foundRecognizedFileSys	= FALSE;

	mid.midInfoLevel = 0;

	char	*fsStr	= reinterpret_cast<char *>(mBootBlock) + 
		FileSys::kFat12Offset;

	if (!strncmp(fsStr, FileSys::kFat12IdStr, 5))
	{
		FileSys::BootSectorFat12	*pBS12	= reinterpret_cast<
			FileSys::BootSectorFat12 *>(mBootBlock);

		foundRecognizedFileSys = TRUE;
		mid.midSerialNum = pBS12->bsVolumeId;

		strncpy(reinterpret_cast<char *>(mid.midVolLabel), 
			pBS12->bsVolumeLabel, 11);
		strncpy(reinterpret_cast<char *>(mid.midFileSysType), 
			pBS12->bsFileSysType, 8);
	}

	fsStr = reinterpret_cast<char *>(mBootBlock) + FileSys::kFat16Offset;

	if (!strncmp(fsStr, FileSys::kFat16IdStr, 5))
	{
		FileSys::BootSectorFat16	*pBS16	= reinterpret_cast<
			FileSys::BootSectorFat16 *>(mBootBlock);

		foundRecognizedFileSys = TRUE;
		mid.midSerialNum = pBS16->bsVolumeId;

		strncpy(reinterpret_cast<char *>(mid.midVolLabel), 
			pBS16->bsVolumeLabel, 11);
		strncpy(reinterpret_cast<char *>(mid.midFileSysType), 
			pBS16->bsFileSysType, 8);
	}

	fsStr = reinterpret_cast<char *>(mBootBlock) + FileSys::kFat32Offset;

	if (!strncmp(fsStr, FileSys::kFat32IdStr, 5))
	{
		FileSys::BootSectorFat32	*pBS32	= reinterpret_cast<
			FileSys::BootSectorFat32 *>(mBootBlock);

		foundRecognizedFileSys = TRUE;
		mid.midSerialNum = pBS32->bsVolumeId;

		strncpy(reinterpret_cast<char *>(mid.midVolLabel), 
			pBS32->bsVolumeLabel, 11);
		strncpy(reinterpret_cast<char *>(mid.midFileSysType), 
			pBS32->bsFileSysType, 8);
	}

	// Special case for incorrectly formatted floppy drives. If the
	// FileSystem is unknown but the media is a floppy, assume FAT12.

	if (!foundRecognizedFileSys)
	{
		FileSys::BootSectorFat12	*pBS12	= reinterpret_cast<
			FileSys::BootSectorFat12 *>(mBootBlock);

		if (pBS12->bsMedia == FileSys::kFloppyMediaId)
		{
			mid.midSerialNum = pBS12->bsVolumeId;

			strncpy(reinterpret_cast<char *>(mid.midVolLabel), 
				pBS12->bsVolumeLabel, 11);
			strncpy(reinterpret_cast<char *>(mid.midFileSysType), 
				FileSys::kFat12IdStr, 8);
		}
	}

	return error;
}

CComboError
CPGPdiskImpDrv98::PerformPremountTasks()
{
	CComboError	error;

	// Save contents of boot block.
	error = mFile.Read(mBootBlock, mFirstDataBlock*kPGPdiskBlockSize, 
		kPGPdiskBlockSize);

	if (error.IsntError())
		error = mCipherContext.DecryptCFB(0, 1, mBootBlock, mBootBlock);

	// Massage FileSystem IO requests on 98 to prevent hangs.
	if (error.IsntError())
	{
		mBufferToWatchFor = mBuffer;

		error = CDriverSubsystemsDrv98::FilesystemWatcher().WatchForBuffer(
			mBuffer);
	}

	return error;
}

PGPUInt16 
CPGPdiskImpDrv98::ProcessIorGenIoctl(CIop& iop)
{
	pgpAssert(IsMounted());

	CComboError	error;
	PGPUInt32	ioctlStatus	= 0;
	PGPUInt32	iopStatus	= 0;

	UDebug::DebugOut("PGPdisk: Ioctl - function=%x minor %x\n", 
		iop.IoctlFunction(), iop.IoctlControlParam() & 0xff);

	// We only process some 440D functions and the 4411 function.
	switch (iop.IoctlFunction())
	{

	case 0x440D:
		switch (UMath::GetLowByte(UMath::GetLowWord(
			iop.IoctlControlParam())))
		{

		case 0x40:			// Set Device Parameters
		{
			// We don't actually have to 'do' anything here. The system
			// will set these parameters when it writes the boot block,
			// which is where we get them from anyways.

			ioctlStatus = ERROR_SUCCESS;
			iopStatus = IORS_SUCCESS;
			break;
		}

		case 0x41:			// Write Track on Logical Drive
		{
			FileSys::RwBlock	*pRWB	= static_cast<FileSys::RwBlock *>(
				iop.IoctlBuffer());
			pgpAssertAddrValid(pRWB, RwBlock);

			// Calculate the starting sector.
			PGPUInt64	pos	= pRWB->rwCylinder * FileSys::kSizeCyl;

			pos	+= pRWB->rwHead * FileSys::kSizeHead;
			pos	+= pRWB->rwFirstSector;

			// Calculate the number of blocks to write.
			PGPUInt32	nBlocks	= pRWB->rwSectors;

			// Calculate the buffer address.
			void	*buf	= Map16BitAddrToLinear(
				UMath::GetHighWord(pRWB->rwBuffer), 
				UMath::GetLowWord(pRWB->rwBuffer));

			// Perform the write.
			error = Write(buf, pos, nBlocks);

			ioctlStatus = (error.IsntError() ? ERROR_SUCCESS : 
				ERROR_INVALID_FUNCTION);
			iopStatus = IORS_SUCCESS;
			break;
		}

		case 0x42:			// Format Track on Logical Drive
		{		
			// Don't actually do anything, since PGPdisks aren't physical
			// drives that need any special treatment of sectors before
			// being written to.

			FileSys::FvBlock	*pFVB	= static_cast<FileSys::FvBlock *>(
				iop.IoctlBuffer());
			pgpAssertAddrValid(pFVB, FvBlock);

			if (pFVB->fvSpecFunc == 1)
				pFVB->fvSpecFunc = 0;

			ioctlStatus = ERROR_SUCCESS;
			iopStatus = IORS_SUCCESS;
			break;
		}

		case 0x60:			// Get Device Parameters
		{
			switch (UMath::GetHighByte(UMath::GetLowWord(
				iop.IoctlControlParam())))
			{
			case 0x08:
			{
				FileSys::DevParamsFat16	*pDp16	= static_cast<
					FileSys::DevParamsFat16 *>(iop.IoctlBuffer());

				pgpAssertAddrValid(pDp16, DevParamsFat16);
				error = GetDevParamsFat16(*pDp16);
			}
			break;

			case 0x48:
			{
				FileSys::DevParamsFat32	*pDp32	= static_cast<
					FileSys::DevParamsFat32 *>(iop.IoctlBuffer());

				pgpAssertAddrValid(pDp32, DevParamsFat32);
				error = GetDevParamsFat32(*pDp32);
			}
			}

			ioctlStatus = (error.IsntError() ? ERROR_SUCCESS : 
				ERROR_INVALID_FUNCTION);
			iopStatus = IORS_SUCCESS;
			break;
		}

		case 0x61:			// Read Track on Logical Drive
		{
			FileSys::RwBlock	*pRWB	= static_cast<FileSys::RwBlock *>(
				iop.IoctlBuffer());
			pgpAssertAddrValid(pRWB, RwBlock);

			// Calculate the starting sector.
			PGPUInt64	pos	= pRWB->rwCylinder * FileSys::kSizeCyl;
			pos	+= pRWB->rwHead * FileSys::kSizeHead;
			pos	+= pRWB->rwFirstSector;

			// Calculate the number of blocks to read.
			PGPUInt32	nBlocks	= pRWB->rwSectors;

			// Calculate the buffer address.
			void	*buf	= Map16BitAddrToLinear(
				UMath::GetHighWord(pRWB->rwBuffer), 
				UMath::GetLowWord(pRWB->rwBuffer));

			// Perform the read.
			error = Read(buf, pos, nBlocks);

			ioctlStatus = (error.IsntError() ? ERROR_SUCCESS : 
				ERROR_INVALID_FUNCTION);
			iopStatus = IORS_SUCCESS;
			break;
		}

		case 0x66:			// Get Media ID
		{	
			FileSys::MID	*pMid	= static_cast<FileSys::MID *>(
				iop.IoctlBuffer());

			error = GetMID(*pMid);

			ioctlStatus = (error.IsntError() ? ERROR_SUCCESS : 
				ERROR_INVALID_FUNCTION);
			iopStatus = IORS_SUCCESS;
			break;
		}

		default:
			ioctlStatus = ERROR_INVALID_FUNCTION;
			iopStatus = IORS_INVALID_COMMAND;
			break;
		}
		break;

	case 0x4411:		// Query if we handle specified 440D code
		switch (UMath::GetLowByte(UMath::GetLowWord(
			iop.IoctlControlParam())))
		{
		case 0x40:
		case 0x41:
		case 0x42:
		case 0x60:
		case 0x61:
		case 0x66:
			ioctlStatus = ERROR_SUCCESS;
			iopStatus = IORS_SUCCESS;
			break;

		default:
			ioctlStatus = ERROR_INVALID_FUNCTION;
			iopStatus = IORS_SUCCESS;
			break;
		}
		break;

	default:
		ioctlStatus = ERROR_INVALID_FUNCTION;
		iopStatus = IORS_INVALID_COMMAND;
		break;
	}

	iop.IoctlReturn() = ioctlStatus;

	return static_cast<PGPUInt16>(iopStatus);
}

PGPUInt16 
CPGPdiskImpDrv98::ProcessIorReadWrite(CIop& iop)
{
	pgpAssert(IsMounted());

	CComboError	error;
	PGPBoolean	isAWrite	= (iop.Func() == IOR_WRITE);

	if (isAWrite && IsReadOnly())
		return IORS_WRITE_PROTECT;

	if (!error.HaveNonPGPError())
	{
		PGPUInt32	pos	= static_cast<PGPUInt32>(iop.StartAddress());

		// Requests can either be in pieces (scatter/gather) or not.
		if (iop.IsScatterGather())
		{
			PBDSGD	pBDSGD;

			pBDSGD = static_cast<PBDSGD>(iop.Buffer());
			pgpAssertAddrValid(pBDSGD, BDSGD);

			while (TRUE)
			{
				PGPUInt32	nBlocks	= pBDSGD->SG_buff_size;

				if (nBlocks == 0)
					break;

				if (iop.IsInBytes())
				{
					pgpAssert(nBlocks % kPGPdiskBlockSize == 0);
					nBlocks /= kPGPdiskBlockSize;
				}

				void	*buf	= reinterpret_cast<void *>(
					pBDSGD->SG_buff_ptr);
				pgpAssertAddrValid(buf, VoidAlign);

				if (isAWrite)
					error = Write(buf, pos, nBlocks);
				else
					error = Read(buf, pos, nBlocks);

				pos += nBlocks;
				pBDSGD++;

				if (error.IsError())
					break;
			}
		}
		else
		{
			PGPUInt32	nBlocks	= iop.TransferCount();

			if (iop.IsInBytes())
			{
				pgpAssert(nBlocks % kPGPdiskBlockSize == 0);
				nBlocks /= kPGPdiskBlockSize;
			}

			void	*buf	= iop.Buffer();
			pgpAssertAddrValid(buf, VoidAlign);

			if (isAWrite)
				error = Write(buf, pos, nBlocks);
			else
				error = Read(buf, pos, nBlocks);
		}
	}

	if (error.IsntError())
		return IORS_SUCCESS;
	else
		return IORS_MEDIA_ERROR;
}

void 
CPGPdiskImpDrv98::ProcessIopCallbackAux(CIop& iop)
{
	pgpAssert(IsMounted());

	CComboError	error;
	PGPUInt16	iorFunc		= iop.Func();
	PGPUInt32	iopAddress	= reinterpret_cast<PGPUInt32>(
		static_cast<PIOP>(iop));

	UDebug::DebugOut("PGPdisk: Processing an %s address %X.", 
		UConstantNamesDrv98::NameIorFunction(iorFunc), iopAddress);

	// Process the IOP according to function.
	switch (iorFunc)
	{
	case IOR_READ:
 	case IOR_WRITE:
		error.err = ProcessIorReadWrite(iop);
		break;

	default:
		pgpAssert(FALSE);
		error.err = IORS_INVALID_COMMAND;
		break;
	}

	iop.Callback(static_cast<PGPUInt16>(error.err));

	UDebug::DebugOut("PGPdisk: Done processing an %s address %X status %X.", 
		UConstantNamesDrv98::NameIorFunction(iorFunc), iopAddress, error.err);
}

void 
CPGPdiskImpDrv98::ProcessIopCallback(void *refPtr)
{
	CIop	iop(static_cast<PIOP>(refPtr));

	CPGPdiskImpDrv98	*pImp	= reinterpret_cast<CPGPdiskImpDrv98 *>(
		iop.PrivatePort());
	pgpAssertAddrValid(pImp, CPGPdiskImpDrv98);

	pImp->ProcessIopCallbackAux(iop);
}

CComboError 
CPGPdiskImpDrv98::QueueIopToIoThread(CIop& iop)
{
	CComboError	error;
	
	iop.PrivatePort() = reinterpret_cast<unsigned long>(this);
	mIoThread->PerformAsyncCallback(ProcessIopCallback, iop.Get());

	return error;
}

PGPUInt32 
CPGPdiskImpDrv98::ProcessExternalIo(void *ioPacket, PGPBoolean& wasCompleted)
{
	pgpAssert(IsMounted());

	// This is going to be an IOP.
	CComboError	error;
	CIop		iop(static_cast<PIOP>(ioPacket));
	PGPBoolean	completeNow	= TRUE;
	PGPUInt16	iorFunc		= iop.Func();
	PGPUInt32	iopAddress	= reinterpret_cast<PGPUInt32>(ioPacket);

	UDebug::DebugOut("PGPdisk: Seeing an %s address %X.", 
		UConstantNamesDrv98::NameIorFunction(iorFunc), iopAddress);

	switch (iorFunc)
	{
	case IOR_GEN_IOCTL:
		// NEVER SCHEDULE IOCTL REQUESTS. ALWAYS PROCESS IN SAME CONTEXT IN
		// WHICH THEY ARE RECEIVED.

		error.err = ProcessIorGenIoctl(iop);
		break;

	case IOR_READ:
	case IOR_WRITE:
		// Queue the IOP to the thread.
		error = QueueIopToIoThread(iop);

		if (error.IsError())
			error.err = IORS_MEMORY_ERROR;
		else
			completeNow = FALSE;

		break;

	default:
		error.err = IORS_INVALID_COMMAND;
		break;
	}

	// Complete if not scheduled.
	if (completeNow)
	{
		iop.Callback(static_cast<PGPUInt16>(error.err));
		wasCompleted = TRUE;

		UDebug::DebugOut("PGPdisk: Completed an %s address %X status %X.", 
			UConstantNamesDrv98::NameIorFunction(iorFunc), 
			iopAddress, static_cast<PGPUInt32>(error.err));
	}
	else
	{
		UDebug::DebugOut("PGPdisk: Scheduled an %s IOP address %X.", 
			UConstantNamesDrv98::NameIorFunction(iorFunc), iopAddress);
	}

	return error.err;
}
