/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CVolumeImpDrv98.cpp,v 1.9 2002/08/06 20:09:31 dallen Exp $
____________________________________________________________________________*/

#include "pgpClassesConfig.h"
#include "pgpDiskBase.h"

#include "UDebug.h"
#include "UMath.h"
#include "UString.h"

#include "UFileSys.h"

#include "CDriverSubsystemsDrv98.h"
#include "CIop.h"
#include "CIosInterface.h"
#include "CVolumeImpDrv98.h"

_USING_PGP


_UNNAMED_BEGIN

// Constants

const char	*kDcbVendorId	= "PGP     ";
const char	*kDcbProductId	= "PGPdisk         ";
const char	*kDcbRevLevel	= "0001";

_UNNAMED_END


// Class CVolumeImpDrv98 member functions

CVolumeImpDrv98::CVolumeImpDrv98()
{
	Status() = mDeviceName.Status();
}

CVolumeImpDrv98::~CVolumeImpDrv98()
{
	if (IsMounted())
	{
		if (IsAttachedToVolume())
			DetachVolume();
		else
			Unmount(TRUE);
	}
}

PGPBoolean
CVolumeImpDrv98::HasOpenFiles()
{
	pgpAssert(IsMounted());

	CComboError	error;

	char		*pathBuf;
	CString		path;
	PGPBoolean	hasOpenFiles;

	LockLevel	oldLock;
	error = GetLockLevel(oldLock);

	if (error.IsntError())
		error = path.Status();

	if (error.IsntError())
		error = path.GetBuffer(kPGPdiskMaxPathLength, pathBuf);

	if (error.IsntError())
	{
		error = SetLockLevel(LockLevel3);

		if (error.IsntError())
		{
			PGPUInt8	drive	= UString::LetterToNumber(mRoot[0]);
			VxdIntRegs	regs;

			regs.eax	= 0x440D;			// function 440D
			regs.ebx	= drive + 1;		// drive to enumerate on
			regs.ecx	= 0x086D;			// minor code 6D (Enum Open Files)
			regs.edx	= reinterpret_cast<PGPUInt32>(pathBuf);	// file path
			regs.esi	= 0;				// index to 0 to get first file
			regs.edi	= 0;				// enumerate all files (0)
			regs.flags	= 0x0001;			// carry flag set on error

			Exec_VxD_Int(0x21, &regs);
			hasOpenFiles = ((regs.flags & 0x0001) ? FALSE : TRUE);

			SetLockLevel(oldLock);
		}

		path.ReleaseBuffer();
	}

	if (error.IsError())
		return TRUE;
	else
		return hasOpenFiles;
}

CComboError
CVolumeImpDrv98::AttachVolume(const char *root, const char *deviceName)
{
	pgpAssert(!IsMounted());

	CComboError	error;

	if (error.IsntError())
		error = mRoot.Assign(root);

	if (error.IsntError())
	{
		if (IsntNull(deviceName))
			error = mDeviceName.Assign(deviceName);
	}

	if (error.IsntError())
		error = mDcb.GetDcbOfDrive(UString::LetterToNumber(root[0]));

	if (error.IsntError())
		error = FillInBlockInfo();

	if (error.IsntError())
	{
		mIsAttachedToVolume = TRUE;

		mVolState = kVolStateMounted;
		mLockState = kLockStateNone;
	}

	return error;
}

void
CVolumeImpDrv98::DetachVolume()
{
	pgpAssert(IsMounted());
	pgpAssert(IsAttachedToVolume());

	if (IsLockedForIO() || IsLockedForFormat())
		UnlockVolume();

	mIsAttachedToVolume = FALSE;

	mVolState = kVolStateUnmounted;
	mLockState = kLockStateNone;
}

CComboError
CVolumeImpDrv98::Mount(
	const char	*deviceName,
	const char	*root,
	PGPUInt64	totalBlocks,
	PGPUInt32	blockSize,
	const void	*iopDispatchFunc,
	void		*refPtr,
	PGPBoolean	readOnly)
{
	pgpAssert(IsUnmounted());
	pgpAssertStrValid(deviceName);
	pgpAssertStrValid(root);
	pgpAssertAddrValid(iopDispatchFunc, IopDispatchFunc);

	CComboError	error;
	error = mRoot.Assign(root);

	if (error.IsntError())
		error = mDeviceName.Assign(deviceName);

	// Need to create a new Dcb.
	if (error.IsntError())
		error = mDcb.Create();

	if (error.IsntError())
	{
		PGPUInt8	drive	= UString::LetterToNumber(root[0]);

		InitDcb(drive, totalBlocks, blockSize, readOnly);

		// Tell system to send us requests for this Dcb.
		if (!CDriverSubsystemsDrv98::IosInterface().IspInsertCalldown(mDcb,
			DispatchIop, 0, mDcb.DemandFlags()))
		{
			error.pgpErr = kPGPError_VolumeOpFailed;
		}

		if (error.IsntError())
		{
			// Associate the Dcb with a drive letter.
			if (!CDriverSubsystemsDrv98::IosInterface().IspAssociateDcb(
				mDcb, drive, NULL))
			{
				error.pgpErr = kPGPError_VolumeOpFailed;
			}
		}

		// Initialize variables on success.
		if (error.IsntError())
		{
			mVolState			= kVolStateMounted;
			mLockState			= kLockStateNone;
			mIsAttachedToVolume = FALSE;

			mDispatchFunc = static_cast<IopDispatchFunc>(iopDispatchFunc);
			mRefPtr = refPtr;

			mTotalBlocks = totalBlocks;
			mBlockSize = blockSize;
		}
	}

	return error;
}

CComboError
CVolumeImpDrv98::Unmount(PGPBoolean isForced)
{
	pgpAssert(IsMounted());
	pgpAssert(!IsAttachedToVolume());

	CComboError	error;

	if (IsLockedForFormat() || IsLockedForIO())
		error = UnlockVolume();

	PGPUInt8	drive	= UString::LetterToNumber(mRoot[0]);

	// Flush the volume.
	if (error.IsntError())
	{
		_VolFlush(drive, NULL);

		// Get a lock on the drive.
		if (!isForced)	// hang on BS open file warning
			error = LockForIO();
	}

	if (error.IsntError())
	{
		// this fixes DriveSpace problems
		NotifyVolumeRemoval(drive);

		// Disassociate the Dcb with its drive letter.
		if (!CDriverSubsystemsDrv98::IosInterface().IspDisassociateDcb(
			drive))
		{
			error.pgpErr = kPGPError_VolumeOpFailed;
		}

		if (error.IsntError())
		{
			mRoot.Empty();
			mDeviceName.Empty();

			mVolState = kVolStateUnmounted;
			mLockState = kLockStateNone;
		}

		if (error.IsError() && !isForced)
			UnlockVolume();
	}

	return error;
}

CComboError
CVolumeImpDrv98::LockForIO()
{
	pgpAssert(IsMounted());
	pgpAssert(!IsLockedForIO() && !IsLockedForFormat());

	CComboError	error;
	error = SetLockLevel(LockLevel3);

	if (error.IsntError())
		mLockState = kLockStateIO;

	return error;
}

CComboError
CVolumeImpDrv98::LockForFormat()
{
	pgpAssert(IsMounted());
	pgpAssert(!IsLockedForIO() && !IsLockedForFormat());

	CComboError	error;
	error = AcquireFormatLock();

	if (error.IsntError())
		mLockState = kLockStateFormat;

	return error;
}

CComboError
CVolumeImpDrv98::UnlockVolume()
{
	pgpAssert(IsMounted());
	pgpAssert(IsLockedForIO() || IsLockedForFormat());

	CComboError	error;

	if (IsLockedForIO())
		error = SetLockLevel(LockLevelNone);
	else
		error = ReleaseFormatLock();

	if (error.IsntError())
		mLockState = kLockStateNone;

	return error;
}

CComboError
CVolumeImpDrv98::Read(void *buf, PGPUInt64 pos, PGPUInt32 nBlocks) const
{
	CComboError	error;

	error = DiskAccess(IOR_READ, buf, pos, nBlocks);

	return error;
}

CComboError
CVolumeImpDrv98::Write(const void *buf, PGPUInt64 pos, PGPUInt32 nBlocks)
{
	CComboError	error;

	error = DiskAccess(IOR_WRITE, const_cast<void *>(buf), pos, nBlocks);

	return error;
}

CComboError
CVolumeImpDrv98::GetLockLevel(LockLevel& lockLevel) const
{
	pgpAssert(IsMounted());

	CComboError	error;
	PGPUInt32	lockFlags, lockOwner, lockType;

	if (IFSMgr_GetLockState(UString::LetterToNumber(mRoot[0]), &lockType,
		&lockFlags, &lockOwner) != 0)
	{
		error.pgpErr = kPGPError_VolumeOpFailed;
	}

	if (error.IsntError())
	{
		switch (lockType)
		{
		case -1:
			lockLevel = LockLevelNone;
			break;
		case 0:
			lockLevel = (lockFlags & LOCKP_LOCK_FOR_FORMAT ? LockLevel0MR :
				LockLevel0);
			break;
		case 1:
			lockLevel = LockLevel1;
			break;
		case 2:
			lockLevel = LockLevel2;
			break;
		case 3:
			lockLevel = LockLevel3;
			break;
		default:
			pgpAssert(FALSE);
			break;
		}
	}

	return error;
}

CComboError
CVolumeImpDrv98::SetLockLevel(LockLevel lockLevel, PGPUInt8 permissions)
{
	pgpAssert(IsMounted());

	CComboError	error;
	LockLevel	curLock;
	PGPUInt8	drive	= UString::LetterToNumber(mRoot[0]);

	// Don't take format locks.
	if ((lockLevel == LockLevel0) || (lockLevel == LockLevel0MR))
		error.pgpErr = kPGPError_BadParams;

	// Make sure we aren't trying to alter a level-0 lock.
	if (error.IsntError())
		error = GetLockLevel(curLock);

	if (error.IsntError())
	{
		UDebug::DebugOut("PGPdisk: Want lock %d on drive %u curlock %d",
			lockLevel, drive, curLock);

		if ((curLock == LockLevel0) || (lockLevel == LockLevel0MR))
			error.pgpErr = kPGPError_VolumeOpFailed;
	}

	// Since there is a special call to acquire a L1 lock, we do this first
	// before anything else.

	if (error.IsntError())
	{
		VxdIntRegs 	regs;

		if ((curLock == LockLevelNone) && (lockLevel != LockLevelNone))
		{
			curLock = LockLevel1;

			regs.eax	= 0x440D;				// function 440D
			regs.ebx	= 0x0100 + drive + 1;	// first get L1 lock
			regs.ecx	= 0x084A;				// (Lock Volume)
			regs.edx	= permissions;			// locking permissions
			regs.flags	= 0x0001;				// carry set if error

			Exec_VxD_Int(0x21, &regs);

			if (regs.flags & 0x0001)
			{
				error.pgpErr = kPGPError_VolumeOpFailed;
				error.err = regs.eax;
			}
		}

		// Now we calculate how many lock levels we need to go up or down.
		if (error.IsntError())
		{
			PGPUInt8	locksUp 	= 0;
			PGPUInt8	locksDown	= 0;

			if (curLock != lockLevel)
			{
				switch (curLock)
				{
				case LockLevel1:
					switch (lockLevel)
					{
					case LockLevelNone:
						locksDown = 1;
						break;
					case LockLevel2:
						locksUp = 1;
						break;
					case LockLevel3:
						locksUp = 2;
						break;
					default:
						pgpAssert(FALSE);
						break;
					}
					break;

				case LockLevel2:
					switch (lockLevel)
					{
					case LockLevelNone:
						locksDown = 2;
						break;
					case LockLevel1:
						locksDown = 1;
						break;
					case LockLevel3:
						locksUp = 1;
						break;
					default:
						pgpAssert(FALSE);
						break;
					}
					break;

				case LockLevel3:
					switch (lockLevel)
					{
					case LockLevelNone:
						locksDown = 3;
						break;
					case LockLevel1:
						locksDown = 2;
						break;
					case LockLevel2:
						locksDown = 1;
						break;
					default:
						pgpAssert(FALSE);
						break;
					}
					break;

				default:
					pgpAssert(FALSE);
					break;
				}
			}

			// If we need to get locks, we do that here.
			while (error.IsntError() && (locksUp != 0))
			{
				PGPUInt16	lockValue;

				locksUp--;

				if (curLock == LockLevel1)
				{
					curLock = LockLevel2;
					lockValue = 2;
				}
				else
				{
					curLock = LockLevel3;
					lockValue = 3;
				}

				regs.eax	= 0x440D;		// function 440D
				regs.ebx	= (lockValue << 8) + drive + 1; // get lock
				regs.ecx	= 0x084A;		// code 4A (Lock Volume)
				regs.edx	= 0;			// > L1s expect zero here
				regs.flags	= 0x0001;		// carry set if error

				Exec_VxD_Int(0x21, &regs);

				if (regs.flags & 0x0001)
				{
					error.pgpErr = kPGPError_VolumeOpFailed;
					error.err = regs.eax;
				}
			}

			// If we need to release locks, we do that here.
			while (error.IsntError() && (locksDown !=0))
			{
				locksDown--;

				if (curLock == LockLevel3)
					curLock = LockLevel2;
				else if (curLock == LockLevel2)
					curLock = LockLevel1;
				else if (curLock == LockLevel1)
					curLock = LockLevelNone;

				regs.eax	= 0x440D;		// function 440D
				regs.ebx	= drive + 1;	// decrement lock level
				regs.ecx	= 0x086A;		// code 6A (Unlock Volume)
				regs.flags	= 0x0001;		// carry set if error

				Exec_VxD_Int(0x21, &regs);

				if (regs.flags & 0x0001)
				{
					error.pgpErr = kPGPError_VolumeOpFailed;
					error.err = regs.eax;
				}
			}
		}
	}

	UDebug::DebugOut("PGPdisk: Lock on drive %u is now %d", drive, lockLevel);

	return error;
}

CComboError
CVolumeImpDrv98::AcquireFormatLock()
{
	pgpAssert(IsMounted());

	CComboError	error;

	// Make sure we aren't already locked;
	LockLevel	lockLevel;
	error = GetLockLevel(lockLevel);

	if (error.IsntError())
	{
		if (lockLevel != LockLevelNone)
			error.pgpErr = kPGPError_VolumeOpFailed;
	}

	// First get a 'normal' level-0 lock.
	if (error.IsntError())
	{
		PGPUInt8	drive		= UString::LetterToNumber(mRoot[0]);
		PGPUInt32	locksTaken	= 0;
		VxdIntRegs	regs;

		regs.eax	= 0x440D;				// function 440D
		regs.ebx	= 0x0000 + drive + 1;	// L0 lock
		regs.ecx	= 0x084A;				// (Lock Volume)
		regs.edx	= 0;					// must be 0
		regs.flags	= 0x0001;				// carry set if error

		Exec_VxD_Int(0x21, &regs);

		if (regs.flags & 0x0001)
		{
			error.pgpErr = kPGPError_VolumeOpFailed;
			error.err = regs.eax;
		}
		else
		{
			locksTaken++;
		}

		// Now get a 'more restrictive' level-0 lock.
		if (error.IsntError())
		{
			regs.eax	= 0x440D;				// function 440D
			regs.ebx	= 0x0000 + drive + 1;	// L0 lock
			regs.ecx	= 0x084A;				// (Lock Volume)
			regs.edx	= 4;					// must be 4
			regs.flags	= 0x0001;				// carry set if error

			Exec_VxD_Int(0x21, &regs);

			if (regs.flags & 0x0001)
			{
				error.pgpErr = kPGPError_VolumeOpFailed;
				error.err = regs.eax;
			}
			else
			{
				locksTaken++;
			}
		}

		// Undo what we did on error.
		if (error.IsError())
		{
			while (locksTaken-- > 0)
			{
				regs.eax	= 0x440D;		// function 440D
				regs.ebx	= drive + 1;	// decrement lock level
				regs.ecx	= 0x086A;		// code 6A (Unlock Volume)
				regs.flags	= 0x0001;		// carry flag set on error

				Exec_VxD_Int(0x21, &regs);
			}
		}
	}

	return error;
}

CComboError
CVolumeImpDrv98::ReleaseFormatLock()
{
	pgpAssert(IsLockedForFormat());

	CComboError	error;

	// Make sure we aren't already locked;
	LockLevel	lockLevel;
	error = GetLockLevel(lockLevel);

	if (error.IsntError())
	{
		if (lockLevel != LockLevel0MR)
			error.pgpErr = kPGPError_VolumeOpFailed;
	}

	// Release locks.
	PGPUInt8	drive		= UString::LetterToNumber(mRoot[0]);
	PGPUInt32	locksTaken	= 2;

	while (error.IsntError() && (locksTaken-- > 0))
	{
		VxdIntRegs	regs;

		regs.eax	= 0x440D;		// function 440D
		regs.ebx	= drive + 1;	// decrement lock level
		regs.ecx	= 0x086A;		// code 6A (Unlock Volume)
		regs.flags	= 0x0001;		// carry flag set on error

		Exec_VxD_Int(0x21, &regs);

		if (regs.flags & 0x0001)
		{
			error.pgpErr = kPGPError_VolumeOpFailed;
			error.err = regs.eax;
		}
	}

	return error;
}

CComboError
CVolumeImpDrv98::FillInBlockInfo()
{
	CComboError	error;

	if (error.IsntError())
	{
		mBlockSize = static_cast<PGPUInt32>(mDcb.CalcBlockSizeFromShift());
		mTotalBlocks = mDcb.SectorCount();
	}

	return error;
}

void
CVolumeImpDrv98::InitDcb(
	PGPUInt8	drive,
	PGPUInt64	totalBlocks,
	PGPUInt32	blockSize,
	PGPBoolean	readOnly)
{
	PGPUInt32	secsPerTrack, tracksPerCyl;
	PGPUInt64	cylinders;

	UFileSys::CalcFakeFatGeom(blockSize, tracksPerCyl, cylinders,
		secsPerTrack);

	mDcb.PhysicalDcb()	= static_cast<PDCB>(mDcb);
	mDcb.Drive()		= drive;
	mDcb.TsdFlags()		= DCB_TSD_NO_USER_INT13 | DCB_TSD_BAD_MBR;
	mDcb.DeviceFlags()	= DCB_DEV_LOGICAL;

	mDcb.BlockShift()		= UMath::SimpleLog2(blockSize);
	mDcb.PartitionType()	= FileSys::kBigFat16PartId;
	mDcb.DeviceType()		= DCB_type_disk;
	mDcb.UserDrive()		= 0xffff;

	mDcb.MaxTransferLength()	= FileSys::kMaxHeads*FileSys::kMaxSpt*
		FileSys::kMaxCyls;
	mDcb.SectorCount()			= totalBlocks;
	mDcb.BlockSize()			= blockSize;
	mDcb.Heads()				= tracksPerCyl;
	mDcb.Cylinders()			= static_cast<PGPUInt32>(cylinders);
	mDcb.Spt()					= secsPerTrack;

	mDcb.PortSpecific()		= reinterpret_cast<PGPUInt32>(this);
	mDcb.BusType()			= DCB_BUS_ESDI;
	mDcb.BddDeviceSubtype()	= BDT_Fixed_Disk;
	mDcb.BddFlags()			= BDF_VERSION_002;

	mDcb.BddSectorCount()	= totalBlocks;
	mDcb.BddBlockSize()		= blockSize;
	mDcb.BddHeads()			= tracksPerCyl;
	mDcb.BddCylinders()		= static_cast<PGPUInt32>(cylinders);
	mDcb.BddSpt()			= secsPerTrack;

	strncpy(mDcb.VendorId(), kDcbVendorId, strlen(kDcbVendorId));
	strncpy(mDcb.ProductId(), kDcbProductId, strlen(kDcbProductId));
	strncpy(mDcb.RevLevel(), kDcbRevLevel, strlen(kDcbRevLevel));

	if (CVolume::kUseRemovableFlag)
	{
		mDcb.DeviceFlags() |= DCB_DEV_REMOVABLE;
		mDcb.BddFlags() |= BDF_Removable;
	}

	if (!readOnly)
	{
		mDcb.DeviceFlags() |= DCB_DEV_WRITEABLE;
		mDcb.BddFlags() |= BDF_Writeable;
	}
}

CComboError
CVolumeImpDrv98::DiskAccess(
	PGPUInt16	func,
	void		*buf,
	PGPUInt64	pos,
	PGPUInt32	nBlocks) const
{
	pgpAssert(IsMounted());
	pgpAssert((func == IOR_READ) || (func == IOR_WRITE));
	pgpAssertAddrValid(buf, VoidAlign);

	CComboError	error;

	// Make sure we check that the VRP field in the DCB exists; the system
	// has a habit of nullifying it when an I/O error occurs.

	PVRP	pVrp	= mDcb.Vrp();

	if (IsNull(pVrp))
		error.pgpErr = kPGPError_VolumeOpFailed;

	if (error.IsntError())
	{
		CIop	iop;

		// Create our IOP/IOR.
		error = iop.Create(pVrp, ISP_M_FL_INTERRUPT_TIME);

		if (error.IsntError())
		{
			// Fill in the IOP/IOR.
			iop.Func()		= func;			// IOR_READ or IOR_WRITE
			iop.Flags()		|= IORF_HIGH_PRIORITY			// high priority
							| IORF_LOGICAL_START_SECTOR		// in partition
							| IORF_SYNC_COMMAND;			// synchronous

			iop.StartAddress()	= pos;
			iop.TransferCount()	= nBlocks;
			iop.Buffer()		= buf;
			iop.ReqVolHandle()	= pVrp;
			iop.Drive()			= UString::LetterToNumber(mRoot[0]);

			// Ask the system to massage the IOP.
			if (!iop.DoIlbCriteria())
			{
				UDebug::DebugOut("PGPdisk: IlbIoCriteria in "
					"CVolumeImpDrv98::DiskAccess failed!");
			}

			UDebug::DebugOut("PGPdisk: Before volume access IOR %X func %u "
				"pos %u nBlocks %u drive %u", &iop.Ior(), func,
				UMath::GetLowDWord(pos), nBlocks,
				UString::LetterToNumber(mRoot[0]));

			// Call down the IOP.
			iop.SendCommand(mDcb);

			UDebug::DebugOut("PGPdisk: After volume access IOR %X",
				&iop.Ior());

			if (iop.IsStatusError())
			{
				if (func == IOR_READ)
				{
					error.pgpErr = kPGPError_ReadFailed;
					error.err = iop.GetErrorAsWin32();
				}
				else
				{
					error.pgpErr = kPGPError_WriteFailed;
					error.err = iop.GetErrorAsWin32();
				}
			}

			iop.Destroy();
		}
	}

	return error;
}

void
CVolumeImpDrv98::DispatchIopAux(CIop& iop)
{
	mDispatchFunc(iop, mRefPtr);
}

void
CVolumeImpDrv98::DispatchIopStub(PIOP pIop)
{
	CIop	iop(pIop);
	CDcb	dcb(reinterpret_cast<PDCB>(iop.OriginalDcb()));

	CVolumeImpDrv98	*pImp	= reinterpret_cast<CVolumeImpDrv98 *>(
		dcb.PortSpecific());

	pgpAssertAddrValid(pImp, CVolumeImpDrv98);
	pImp->DispatchIopAux(iop);
}

void
CVolumeImpDrv98::DispatchIop(PIOP pIop)
{
	// Get around a MSVC5 compiler bug (involving the stack and IOS).
	DispatchIopStub(pIop);
}
