/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CPGPdiskImpDrvNT.cpp,v 1.11 2002/09/26 01:32:11 wjb Exp $
____________________________________________________________________________*/

#include "pgpClassesConfig.h"
#include <devioctl.h>

#if	(_WIN32_WINNT >= 0x0500)
#include <mountdev.h>
#endif	// _WIN32_WINNT >= 0x5000

#include "CString.h"
#include "UDebug.h"

#include "CDevice.h"
#include "CUnicodeString.h"

#include "DriverAPI.h"

#include "CDeviceCache.h"
#include "CDriverSubsystemsDrvNT.h"
#include "CPGPdiskImpDrvNT.h"
#include "UConstantNamesDrvNT.h"
#include "UMessageLog.h"

_USING_PGP

_UNNAMED_BEGIN

// Constants
enum {kStandardBlockSize = 512};

_UNNAMED_END


// Class CPGPdiskImpDrvNT member functions

CComboError 
CPGPdiskImpDrvNT::Mount(
	const char	*path, 
	const char	*root, 
	const char	*deviceName, 
	PGPdiskEncryptionAlgorithm	algorithm, 
	const void	*exportedContext, 
	PGPUInt32	sizeContext, 
	PGPUInt64	firstDataBlock, 
	PGPUInt64	numDataBlocks, 
	const void	*ioHandlerFunc, 
	void		*refPtr, 
	PGPUInt32	procId, 
	PGPBoolean	isReadOnly)
{
	CComboError	error;

	// Initialize security tokens.
	error = mTokenForMountUnmount.AssignProcessId(procId);

	if (error.IsntError())
	{
		error = mTokenForIo.AssignProcessId(procId);

		if (error.IsntError())
		{
			// Assume security token of the user's original process ID.
			mTokenForMountUnmount.ImpersonateToken();

			error = CPGPdiskImp::Mount(path, root, deviceName, algorithm, 
				exportedContext, sizeContext, firstDataBlock, numDataBlocks, 
				ioHandlerFunc, refPtr, procId, isReadOnly);

			mTokenForMountUnmount.RevertToSelf();

			if (error.IsError())
				mTokenForIo.UnassignProcessId();
		}

		if (error.IsError())
			mTokenForMountUnmount.UnassignProcessId();
	}

	return error;
}

CComboError 
CPGPdiskImpDrvNT::Unmount(PGPBoolean isForced)
{
	CComboError	error;

	// Assume security token of the user's original process ID.
	mTokenForMountUnmount.ImpersonateToken();  // NETABUG security 
	error = CPGPdiskImp::Unmount(isForced);
	mTokenForMountUnmount.RevertToSelf();   // NETABUG security 

	// Uninitialize the previously initialize security tokens.
	if (error.IsntError())
	{
		mTokenForMountUnmount.UnassignProcessId();
		mTokenForIo.UnassignProcessId();
	}

	return error;
}

CComboError 
CPGPdiskImpDrvNT::CalcBlockSizeOfHost(
	const char	*path, 
	PGPUInt32&	hostBlockSize) const
{
	pgpAssertStrValid(path);

	CComboError	error;

	// Determine block size.
	CString	hostRoot;
	CVolume	host;

	error = hostRoot.Status();

	if (error.IsntError())
		hostRoot.Format("%c:\\", path[0]);

	if (error.IsntError())
	{
		error = host.AttachVolume(hostRoot);

		if (error.IsntError())
		{
			hostBlockSize = host.BlockSize();
			host.DetachVolume();
		}

		// Network path if error.
		if (error.IsError())
		{
			error = CComboError();
			hostBlockSize = kStandardBlockSize;
		}
	}

	return error;
}

void 
CPGPdiskImpDrvNT::CalcNTFakeDriveLayout(
	DRIVE_LAYOUT_INFORMATION	*pDLI) const
{
	pgpAssertAddrValid(pDLI, DRIVE_LAYOUT_INFORMATION);

	pDLI->PartitionCount = 1;
	pDLI->Signature = 0;

	CalcNTFakePartitionInfo(&pDLI->PartitionEntry[0]);
}

void 
CPGPdiskImpDrvNT::CalcNTFakeGeometry(DISK_GEOMETRY *pGeom) const
{
	pgpAssertAddrValid(pGeom, DISK_GEOMETRY);

	pGeom->Cylinders.QuadPart	= TotalBlocks();
	pGeom->MediaType			= (CVolume::kUseRemovableFlag ? 
		RemovableMedia : FixedMedia);
	pGeom->TracksPerCylinder	= 1;
	pGeom->SectorsPerTrack		= 1;
	pGeom->BytesPerSector		= kPGPdiskBlockSize;
}

void 
CPGPdiskImpDrvNT::CalcNTFakePartitionInfo(
	PARTITION_INFORMATION	*pPI) const
{
	pgpAssertAddrValid(pPI, PARTITION_INFORMATION);

	pPI->StartingOffset.QuadPart	= kPGPdiskBlockSize;
	pPI->PartitionLength.QuadPart	= TotalBlocks() * kPGPdiskBlockSize;
	pPI->HiddenSectors				= 0;
	pPI->PartitionNumber			= 1;
	pPI->PartitionType				= PARTITION_ENTRY_UNUSED; // like I know
	pPI->BootIndicator				= FALSE;
	pPI->RecognizedPartition		= TRUE;
	pPI->RewritePartition			= FALSE;
}

NTSTATUS 
CPGPdiskImpDrvNT::ProcessIrpMjRead(CIrp& irp)
{
	pgpAssert(irp.MajorFunction() == IRP_MJ_READ);

	CComboError	error;
	void		*buf;

	// Extract parameters. Note that 0 IS a valid length!
	PGPUInt64	posInBlocks	= irp.ReadPos() / kPGPdiskBlockSize;

	PGPUInt32	nBlocks	= static_cast<PGPUInt32>(
		irp.ReadLength() / kPGPdiskBlockSize);

	if (nBlocks > 0)
	{
		// Extract the buffer.
	#if	(_WIN32_WINNT >= 0x0500)

		buf = MmGetSystemAddressForMdlSafe(irp.MdlAddress(), 
			NormalPagePriority);

		if (IsNull(buf))
			error.pgpErr = kPGPError_MemoryOpFailed;

	#else	// _WIN32_WINNT < 0x5000

		buf = MmGetSystemAddressForMdl(irp.MdlAddress());
		pgpAssertAddrValid(buf, VoidAlign);

	#endif	// _WIN32_WINNT >= 0x5000					
	}

	if (error.IsntError())
		error = Read(buf, posInBlocks, nBlocks);

	if (error.IsntError())
	{
		irp.Information() = irp.ReadLength();
		return STATUS_SUCCESS;
	}
	else
	{
		irp.Information() = 0;
		return STATUS_DEVICE_DATA_ERROR;
	}
}

NTSTATUS 
CPGPdiskImpDrvNT::ProcessIrpMjWrite(CIrp& irp)
{
	pgpAssert(irp.MajorFunction() == IRP_MJ_WRITE);

	CComboError	error;
	void		*buf;

	if (IsReadOnly())
		return STATUS_MEDIA_WRITE_PROTECTED;
	
	// Extract parameters. Note that 0 IS a valid length!
	PGPUInt64	posInBlocks	= irp.WritePos() / kPGPdiskBlockSize;

	PGPUInt32	nBlocks	= static_cast<PGPUInt32>(
		irp.WriteLength() / kPGPdiskBlockSize);

	if (nBlocks > 0)
	{
		// Extract the buffer.
	#if	(_WIN32_WINNT >= 0x0500)

		buf = MmGetSystemAddressForMdlSafe(irp.MdlAddress(), 
			NormalPagePriority);

		if (IsNull(buf))
			error.pgpErr = kPGPError_MemoryOpFailed;

	#else	// _WIN32_WINNT < 0x5000

		buf = MmGetSystemAddressForMdl(irp.MdlAddress());
		pgpAssertAddrValid(buf, VoidAlign);

	#endif	// _WIN32_WINNT >= 0x5000					
	}

	if (error.IsntError())
		error = Write(buf, posInBlocks, nBlocks);

	if (error.IsntError())
	{
		irp.Information() = irp.WriteLength();
		return STATUS_SUCCESS;
	}
	else
	{
		irp.Information() = 0;
		return STATUS_DEVICE_DATA_ERROR;
	}
}

void 
CPGPdiskImpDrvNT::ProcessIrpCallbackAux(
	CIrp&		irp, 
	PGPBoolean&	wasIrpCompleted)
{
	CComboError	error;
	PGPUInt8	majorFunc	= irp.MajorFunction();
	PGPUInt32	irpAddress	= reinterpret_cast<PGPUInt32>(
		static_cast<PIRP>(irp));

	UDebug::DebugOut("PGPdisk: Processing an %s address %X.", 
		UConstantNamesDrvNT::NameIrpMajorFunction(majorFunc), irpAddress);

	// Assume appropriate security context for file access.
	if (!mTokenForIo.IsTokenSet())
		mTokenForIo.ImpersonateToken();

	// Process the IRP according to function.
	switch (majorFunc)
	{
	case IRP_MJ_READ:
 		error.err = ProcessIrpMjRead(irp);
		break;

	case IRP_MJ_WRITE:
		error.err = ProcessIrpMjWrite(irp);
		break;

	default:
		pgpAssert(FALSE);

		irp.Information() = 0;
		error.err = STATUS_NOT_IMPLEMENTED;
		break;
	}

	if (error.HaveNonPGPError())
		irp.Complete(error.err, IO_DISK_INCREMENT);
	else
		irp.Complete(error.err);

	wasIrpCompleted = TRUE;

	UDebug::DebugOut("PGPdisk: Completed an %s address %X status %X.", 
		UConstantNamesDrvNT::NameIrpMajorFunction(majorFunc), 
		irpAddress, static_cast<PGPUInt32>(error.err));
}

void 
_cdecl 
CPGPdiskImpDrvNT::ProcessIrpCallback(void *refPtr)
{
	CIrp		irp(static_cast<PIRP>(refPtr));
	PGPBoolean	wasIrpCompleted	= FALSE;

	__try 
	{
		CPGPdiskImpDrvNT	*pImp	= static_cast<CPGPdiskImpDrvNT *>(
			irp.DriverContext1());
		pgpAssertAddrValid(pImp, CPGPdiskImpDrvNT);

		pImp->ProcessIrpCallbackAux(irp, wasIrpCompleted);
	}
	__except(EXCEPTION_EXECUTE_HANDLER) 
	{
		UDebug::DebugOut("PGPdisk: Exception in CPGPdiskImpDrvNT::"
			"ProcessIrpCallback.");
		UMessageLog::Output("Exception caught");

		// We don't exit with uncompleted IRPs.
		if (!wasIrpCompleted)
		{
			irp.Information() = 0;
			irp.Complete(STATUS_DEVICE_DATA_ERROR);

			wasIrpCompleted = TRUE;
		}
	}
}

CComboError 
CPGPdiskImpDrvNT::QueueIrpToIoThread(CIrp& irp)
{
	CComboError	error;

	irp.DriverContext1() = static_cast<PVOID>(this);
	error = mIoThread->PerformAsyncCallback(ProcessIrpCallback, irp.Get());
	
	return error;
}

// Added by wjb for Windows XP. 
// These are selected snippets from the XP DDK
#ifndef IOCTL_DISK_GET_LENGTH_INFO

#define FTTYPE  ((ULONG)'f')
#define IOCTL_VOLUME_BASE   ((ULONG) 'V')

#define IOCTL_DISK_GET_LENGTH_INFO          CTL_CODE(IOCTL_DISK_BASE, 0x0017, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_DISK_GET_PARTITION_INFO_EX    CTL_CODE(IOCTL_DISK_BASE, 0x0012, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_STORAGE_GET_HOTPLUG_INFO		CTL_CODE(IOCTL_STORAGE_BASE, 0x0305, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define FT_BALANCED_READ_MODE				CTL_CODE(FTTYPE, 6, METHOD_NEITHER,  FILE_ANY_ACCESS)
#define IOCTL_VOLUME_GET_GPT_ATTRIBUTES		CTL_CODE(IOCTL_VOLUME_BASE, 14, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_VOLUME_ONLINE					CTL_CODE(IOCTL_VOLUME_BASE, 2, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define IOCTL_MOUNTDEV_QUERY_STABLE_GUID	CTL_CODE(MOUNTDEVCONTROLTYPE, 6, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_STORAGE_GET_DEVICE_NUMBER		CTL_CODE(IOCTL_STORAGE_BASE, 0x0420, METHOD_BUFFERED, FILE_ANY_ACCESS)

#endif // IOCTL_DISK_GET_LENGTH_INFO

#ifndef GET_LENGTH_INFORMATION
//
// The structure GET_LENGTH_INFORMATION is used with the ioctl
// IOCTL_DISK_GET_LENGTH_INFO to obtain the length, in bytes, of the
// disk, partition, or volume.
//

typedef struct _GET_LENGTH_INFORMATION {
    LARGE_INTEGER   Length;
} GET_LENGTH_INFORMATION, *PGET_LENGTH_INFORMATION;

//
// There are currently two ways a disk can be partitioned. With a traditional
// AT-style master boot record (PARTITION_STYLE_MBR) and with a new, GPT
// partition table (PARTITION_STYLE_GPT). RAW is for an unrecognizable
// partition style. There are a very limited number of things you can
// do with a RAW partititon.
//

typedef enum _PARTITION_STYLE {
    PARTITION_STYLE_MBR,
    PARTITION_STYLE_GPT,
    PARTITION_STYLE_RAW
} PARTITION_STYLE;

//
// The following structure defines information in an MBR partition that is not
// common to both GPT and MBR partitions.
//

typedef struct _PARTITION_INFORMATION_MBR {
    UCHAR PartitionType;
    BOOLEAN BootIndicator;
    BOOLEAN RecognizedPartition;
    ULONG HiddenSectors;
} PARTITION_INFORMATION_MBR, *PPARTITION_INFORMATION_MBR;

//
// The following structure defines information in a GPT partition that is
// not common to both GPT and MBR partitions.
//

typedef struct _PARTITION_INFORMATION_GPT {
    GUID PartitionType;                 // Partition type. See table 16-3.
    GUID PartitionId;                   // Unique GUID for this partition.
    _int64 Attributes;                 // See table 16-4.
    WCHAR Name [36];                    // Partition Name in Unicode.
} PARTITION_INFORMATION_GPT, *PPARTITION_INFORMATION_GPT;

//
// The PARTITION_INFORMATION_EX structure is used with the
// IOCTL_DISK_GET_DRIVE_LAYOUT_EX, IOCTL_DISK_SET_DRIVE_LAYOUT_EX,
// IOCTL_DISK_GET_PARTITION_INFO_EX and IOCTL_DISK_GET_PARTITION_INFO_EX calls.
//

typedef struct _PARTITION_INFORMATION_EX {
    PARTITION_STYLE PartitionStyle;
    LARGE_INTEGER StartingOffset;
    LARGE_INTEGER PartitionLength;
    ULONG PartitionNumber;
    BOOLEAN RewritePartition;
    union {
        PARTITION_INFORMATION_MBR Mbr;
        PARTITION_INFORMATION_GPT Gpt;
    };
} PARTITION_INFORMATION_EX, *PPARTITION_INFORMATION_EX;

//
// IOCTL_STORAGE_GET_HOTPLUG_INFO
//

typedef struct _STORAGE_HOTPLUG_INFO {
    ULONG Size; // version
    BOOLEAN MediaRemovable; // ie. zip, jaz, cdrom, mo, etc. vs hdd
    BOOLEAN MediaHotplug;   // ie. does the device succeed a lock even though its not lockable media?
    BOOLEAN DeviceHotplug;  // ie. 1394, USB, etc.
    BOOLEAN WriteCacheEnableOverride; // This field should not be relied upon because it is no longer used
} STORAGE_HOTPLUG_INFO, *PSTORAGE_HOTPLUG_INFO;

//
// Output structure for IOCTL_VOLUME_GET_GPT_ATTRIBUTES.
//

typedef struct _VOLUME_GET_GPT_ATTRIBUTES_INFORMATION {
    ULONGLONG   GptAttributes;
} VOLUME_GET_GPT_ATTRIBUTES_INFORMATION, *PVOLUME_GET_GPT_ATTRIBUTES_INFORMATION;

//
// Output structure for IOCTL_MOUNTDEV_QUERY_STABLE_GUID.
//

typedef struct _MOUNTDEV_STABLE_GUID {
    GUID    StableGuid;
} MOUNTDEV_STABLE_GUID, *PMOUNTDEV_STABLE_GUID;

#endif // GET_LENGTH_INFORMATION
// End wjb

NTSTATUS 
CPGPdiskImpDrvNT::ProcessExtIrpMjDeviceControl(CIrp& irp)
{
	pgpAssert(irp.MajorFunction() == IRP_MJ_DEVICE_CONTROL);

	CComboError	error;

	UDebug::DebugOut("PGPdisk: Seen an %s IRP.", 
		UConstantNamesDrvNT::NameIoctlCode(irp.IoctlCode()));

	irp.Information() = 0;

	// Perform the IOCTL.
	switch (irp.IoctlCode())
	{
	case IOCTL_DISK_IS_WRITABLE:
		// check if disk is read-only
		if (IsReadOnly())
			error.err = STATUS_MEDIA_WRITE_PROTECTED;
		else
			error.err = STATUS_SUCCESS;
		break;
	
	case IOCTL_DISK_GET_DRIVE_GEOMETRY:
		// Return drive geometry.
		if (irp.IoctlOutputBufferLength() < sizeof(DISK_GEOMETRY))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			PDISK_GEOMETRY	pGeom	= static_cast<PDISK_GEOMETRY>(
				irp.IoctlInputBuffer());
			pgpAssertAddrValid(pGeom, DISK_GEOMETRY);

			CalcNTFakeGeometry(pGeom);

			error.err = STATUS_SUCCESS;
			irp.Information() = sizeof(DISK_GEOMETRY);
		}
		break;

	case IOCTL_DISK_GET_DRIVE_LAYOUT:
		// Return drive geometry.
		if (irp.IoctlOutputBufferLength() < sizeof(DRIVE_LAYOUT_INFORMATION))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			PDRIVE_LAYOUT_INFORMATION	pDLI	= 
				static_cast<PDRIVE_LAYOUT_INFORMATION>(
				irp.IoctlInputBuffer());
			pgpAssertAddrValid(pDLI, DRIVE_LAYOUT_INFORMATION);

			CalcNTFakeDriveLayout(pDLI);

			error.err = STATUS_SUCCESS;
			irp.Information() = sizeof(DRIVE_LAYOUT_INFORMATION);
		}
		break;

	case IOCTL_DISK_GET_PARTITION_INFO: 
		// Return information about the partition.
		if (irp.IoctlOutputBufferLength() < sizeof(PARTITION_INFORMATION))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			PPARTITION_INFORMATION	pPartInfo	= 
				(PPARTITION_INFORMATION) irp.IoctlInputBuffer();
			pgpAssertAddrValid(pPartInfo, PARTITION_INFORMATION);

			CalcNTFakePartitionInfo(pPartInfo);

			error.err = STATUS_SUCCESS;
			irp.Information() = sizeof(PARTITION_INFORMATION);
		}
		break;

	case IOCTL_DISK_CHECK_VERIFY:
	case IOCTL_STORAGE_CHECK_VERIFY:
		// Do we have optional media change buffer?
		if (irp.IoctlOutputBufferLength() > 0)
		{
			if (irp.IoctlOutputBufferLength() < sizeof(ULONG))
				error.err = STATUS_INFO_LENGTH_MISMATCH;

			if (error.IsntError())
			{
				ULONG	*pChange	= static_cast<ULONG *>(
					irp.IoctlInputBuffer());
				*pChange = 0;

				irp.Information() = sizeof(ULONG);
			}
		}

		if (error.IsntError())
			error.err = STATUS_SUCCESS;
		break;

	case IOCTL_DISK_VERIFY:
		// Verify disk extent.
		if (irp.IoctlInputBufferLength() < sizeof(VERIFY_INFORMATION))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			PVERIFY_INFORMATION	pVI	= static_cast<PVERIFY_INFORMATION>(
				irp.IoctlInputBuffer());
			pgpAssertAddrValid(pVI, PVERIFY_INFORMATION);

			// Do freak-all...

			error.err = STATUS_SUCCESS;
			irp.Information() = pVI->Length;
		}
		break;

#if	(_WIN32_WINNT >= 0x0500)

	// Added by WJB for Windows XP
	// IOCTL_DISK_GET_LENGTH_INFO must be defined for mounting on XP
    case IOCTL_DISK_GET_LENGTH_INFO: 
	{
		PGET_LENGTH_INFORMATION lengthInfo;

		if (irp.IoctlOutputBufferLength() < sizeof(GET_LENGTH_INFORMATION))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			lengthInfo = (PGET_LENGTH_INFORMATION)irp.IoctlInputBuffer();
			lengthInfo->Length.QuadPart	= TotalBlocks() * kPGPdiskBlockSize;
			irp.Information() = sizeof(GET_LENGTH_INFORMATION);
			error.err = STATUS_SUCCESS;
		}
        break;
    }
/*
	// The rest of these IOCTLs are not implemented but code is left for reference
	case IOCTL_DISK_GET_PARTITION_INFO_EX:
	{
		if (irp.IoctlOutputBufferLength() < sizeof(PARTITION_INFORMATION_EX))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			PPARTITION_INFORMATION_EX	pPartInfo	= 
				(PPARTITION_INFORMATION_EX) irp.IoctlInputBuffer();
	//		pgpAssertAddrValid(pPartInfo, PARTITION_INFORMATION_EX);

			pPartInfo->PartitionStyle			= PARTITION_STYLE_MBR;
			pPartInfo->StartingOffset.QuadPart	= kPGPdiskBlockSize;
			pPartInfo->PartitionLength.QuadPart	= TotalBlocks() * kPGPdiskBlockSize;
			pPartInfo->PartitionNumber			= 1;
			pPartInfo->RewritePartition			= FALSE;

			pPartInfo->Mbr.PartitionType		= PARTITION_ENTRY_UNUSED; // like I know
			pPartInfo->Mbr.BootIndicator		= FALSE;
			pPartInfo->Mbr.RecognizedPartition	= TRUE;
			pPartInfo->Mbr.HiddenSectors		= 0;

			error.err = STATUS_SUCCESS;
			irp.Information() = sizeof(PARTITION_INFORMATION_EX);
		}

		break;
	}

	case IOCTL_STORAGE_GET_HOTPLUG_INFO:
	{
		if (irp.IoctlOutputBufferLength() < sizeof(STORAGE_HOTPLUG_INFO))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			PSTORAGE_HOTPLUG_INFO	pPartInfo	= 
				(PSTORAGE_HOTPLUG_INFO) irp.IoctlInputBuffer();
	//		pgpAssertAddrValid(pPartInfo, PARTITION_INFORMATION_EX);

			pPartInfo->Size=sizeof(STORAGE_HOTPLUG_INFO);
			pPartInfo->MediaRemovable=FALSE; // ie. zip, jaz, cdrom, mo, etc. vs hdd
			pPartInfo->MediaHotplug=FALSE;   // ie. does the device succeed a lock even though its not lockable media?
			pPartInfo->DeviceHotplug=FALSE;  // ie. 1394, USB, etc.
			pPartInfo->WriteCacheEnableOverride=FALSE; // This field should not be relied upon because it is no longer used

			error.err = STATUS_SUCCESS;
			irp.Information() = sizeof(STORAGE_HOTPLUG_INFO);
		}

		break;
	}

	case FT_BALANCED_READ_MODE:
	{
		error.err = STATUS_INVALID_PARAMETER;
		irp.Information() = 0;
		break;
	}

	case IOCTL_VOLUME_GET_GPT_ATTRIBUTES:
	{
		if (irp.IoctlOutputBufferLength() < sizeof(VOLUME_GET_GPT_ATTRIBUTES_INFORMATION))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			PVOLUME_GET_GPT_ATTRIBUTES_INFORMATION	pPartInfo	= 
				(PVOLUME_GET_GPT_ATTRIBUTES_INFORMATION) irp.IoctlInputBuffer();
	//		pgpAssertAddrValid(pPartInfo, PARTITION_INFORMATION_EX);

			pPartInfo->GptAttributes=0;

			error.err = STATUS_SUCCESS;
			irp.Information() = sizeof(VOLUME_GET_GPT_ATTRIBUTES_INFORMATION);
		}
		break;
	}

	case IOCTL_VOLUME_ONLINE:
	{
		error.err = STATUS_SUCCESS;
		irp.Information() = 0;
		break;
	}

	case IOCTL_MOUNTDEV_QUERY_STABLE_GUID:
	{
		if (irp.IoctlOutputBufferLength() < sizeof(MOUNTDEV_STABLE_GUID))
			error.err = STATUS_INVALID_PARAMETER;
			irp.Information() = 0;

		if (error.IsntError())
		{
			PMOUNTDEV_STABLE_GUID	pPartInfo	= 
				(PMOUNTDEV_STABLE_GUID) irp.IoctlInputBuffer();
	//		pgpAssertAddrValid(pPartInfo, PARTITION_INFORMATION_EX);

//			pPartInfo->StableGuid=0;

			error.err = STATUS_SUCCESS;
			irp.Information() = sizeof(MOUNTDEV_STABLE_GUID);
		}
		break;
	}

	case IOCTL_STORAGE_GET_DEVICE_NUMBER:
	{
		if (irp.IoctlOutputBufferLength() < sizeof(STORAGE_DEVICE_NUMBER))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			PSTORAGE_DEVICE_NUMBER	pPartInfo	= 
				(PSTORAGE_DEVICE_NUMBER) irp.IoctlInputBuffer();
	//		pgpAssertAddrValid(pPartInfo, PARTITION_INFORMATION_EX);

			pPartInfo->DeviceType=FILE_DEVICE_UNKNOWN;
			pPartInfo->DeviceNumber=0;
			pPartInfo->PartitionNumber=1;

			error.err = STATUS_SUCCESS;
			irp.Information() = sizeof(STORAGE_DEVICE_NUMBER);
		}
		break;
	}
*/
	// end WJB for XP

	case IOCTL_MOUNTDEV_QUERY_DEVICE_NAME:
		// Return PGPdisk device name.
		if (irp.IoctlOutputBufferLength() < sizeof(MOUNTDEV_NAME))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			CUnicodeString	deviceName(DeviceName());
			error = deviceName.Status();

			if (error.IsntError())
				error = deviceName.Prepend('\\');

			if (error.IsntError())
				error = deviceName.Prepend(DriverAPI::kPGPdiskDeviceDir);

			if (error.IsntError())
				error = deviceName.Prepend(kNTDevicePathPrefix);

			if (error.IsntError())
			{
				PMOUNTDEV_NAME	pMN	= static_cast<PMOUNTDEV_NAME>(
					irp.IoctlInputBuffer());
				pMN->NameLength = deviceName.Length() * sizeof(WCHAR);

				if (irp.IoctlOutputBufferLength() < pMN->NameLength + 
					sizeof(WCHAR))
				{
					error.err = STATUS_BUFFER_OVERFLOW;
					irp.Information() = sizeof(MOUNTDEV_NAME);
				}

				if (error.IsntError())
				{
					pgpCopyMemory(deviceName.Get()->Buffer, pMN->Name, 
						pMN->NameLength);

					error.err = STATUS_SUCCESS;
					irp.Information() = sizeof(WCHAR) + pMN->NameLength;
				}
			}
		}
		break;

	case IOCTL_MOUNTDEV_QUERY_UNIQUE_ID:
		// Return unique ID for the PGPdisk.
		if (irp.IoctlOutputBufferLength() < sizeof(MOUNTDEV_UNIQUE_ID))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			// Just use device name.
			CUnicodeString	deviceName(DeviceName());
			error = deviceName.Status();

			if (error.IsntError())
			{
				PMOUNTDEV_UNIQUE_ID	pUI	= static_cast<PMOUNTDEV_UNIQUE_ID>(
					irp.IoctlInputBuffer());

				pUI->UniqueIdLength = deviceName.Length() * sizeof(WCHAR);

				if (irp.IoctlOutputBufferLength() < 
					(pUI->UniqueIdLength + sizeof(WCHAR)))
				{
					error.err = STATUS_BUFFER_OVERFLOW;
					irp.Information() = sizeof(MOUNTDEV_UNIQUE_ID);
				}

				if (error.IsntError())
				{
					pgpCopyMemory(deviceName.Get()->Buffer, pUI->UniqueId, 
						pUI->UniqueIdLength);

					error.err = STATUS_SUCCESS;
					irp.Information() = sizeof(WCHAR) + pUI->UniqueIdLength;
				}
			}
		}
		break;

	case IOCTL_MOUNTDEV_QUERY_SUGGESTED_LINK_NAME:
		// Return link name for the PGPdisk.
		if (irp.IoctlOutputBufferLength() < 
			sizeof(MOUNTDEV_SUGGESTED_LINK_NAME))
		{
			error.err = STATUS_INVALID_PARAMETER;
		}

		if (error.IsntError())
		{
			// Return the root specified.
			CUnicodeString	linkName(Root());
			error = linkName.Status();

			if (error.IsntError())
				error = linkName.Prepend(kNTDosDevicesPrefix);

			if (error.IsntError())
				linkName.Resize(linkName.Length() - 1);

			if (error.IsntError())
			{
				PMOUNTDEV_SUGGESTED_LINK_NAME	pSugName	= static_cast<
					PMOUNTDEV_SUGGESTED_LINK_NAME>(irp.IoctlInputBuffer());

				pSugName->UseOnlyIfThereAreNoOtherLinks = TRUE;
				pSugName->NameLength = linkName.Length() * sizeof(WCHAR);

				irp.Information() = 
					FIELD_OFFSET(MOUNTDEV_SUGGESTED_LINK_NAME, Name) + 
					pSugName->NameLength;

				if (irp.IoctlOutputBufferLength() < irp.Information())
				{
					error.err = STATUS_BUFFER_OVERFLOW;
					irp.Information() = sizeof(MOUNTDEV_SUGGESTED_LINK_NAME);
				}

				if (error.IsntError())
				{
					pgpCopyMemory(linkName.Get()->Buffer, pSugName->Name, 
						pSugName->NameLength);

					error.err = STATUS_SUCCESS;
					// Information set from above.
				}
			}
		}
		break;

	case IOCTL_MOUNTDEV_LINK_CREATED:
		// Store information about volume ID/link name.
		if (irp.IoctlInputBufferLength() < sizeof(MOUNTDEV_NAME))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			PMOUNTDEV_NAME	pMN	= static_cast<PMOUNTDEV_NAME>(
				irp.IoctlInputBuffer());

			UNICODE_STRING	temp;

			temp.Length = temp.MaximumLength = pMN->NameLength;
			temp.Buffer = pMN->Name;

			CUnicodeString	link(temp);
			error = link.Status();

			if (error.IsntError())
			{
				if ((link[4] == 'V') && (link[5] == 'o') && 
					(link[6] == 'l'))
				{
					// It's a volume ID.
					CDriverSubsystemsDrvNT::DeviceCache().SetEntryVolId(
						DeviceName(), link);
				}
				else
				{
					// It's a drive letter.
					CDriverSubsystemsDrvNT::DeviceCache().SetEntryCurLink(
						DeviceName(), link);
				}
			}

			error.err = STATUS_SUCCESS;
		}
		break;

	case IOCTL_MOUNTDEV_LINK_DELETED:
		// Delete information about link name.
		if (irp.IoctlInputBufferLength() < sizeof(MOUNTDEV_NAME))
			error.err = STATUS_INVALID_PARAMETER;

		if (error.IsntError())
		{
			CUnicodeString	empty;
			error = empty.Status();

			CDriverSubsystemsDrvNT::DeviceCache().SetEntryCurLink(
				DeviceName(), empty);

			error.err = STATUS_SUCCESS;
		}
		break;

#endif	// _WIN32_WINNT >= 0x5000

	default:
		error.err = STATUS_INVALID_DEVICE_REQUEST;
		break;
	}

	return error.err;
}

PGPUInt32 
CPGPdiskImpDrvNT::ProcessExternalIo(
	void		*ioPacket, 
	PGPBoolean&	wasCompleted)
{
	// This is going to be an IRP.
	CComboError	error;
	CIrp		irp(static_cast<PIRP>(ioPacket));
	PGPUInt8	majorFunc	= irp.MajorFunction();
	PGPUInt32	irpAddress	= reinterpret_cast<PGPUInt32>(ioPacket);

	UDebug::DebugOut("PGPdisk: Seeing an %s address %X.", 
		UConstantNamesDrvNT::NameIrpMajorFunction(majorFunc), irpAddress);

	irp.Information() = 0;

	switch (majorFunc)
	{
		case IRP_MJ_CLOSE:
		case IRP_MJ_CREATE:
		case IRP_MJ_FLUSH_BUFFERS:
		case IRP_MJ_SHUTDOWN:

			error.err = STATUS_SUCCESS;
			break;

		case IRP_MJ_DEVICE_CONTROL:
			// Don't queue device controls.
			error.err = ProcessExtIrpMjDeviceControl(irp);
			break;

		case IRP_MJ_READ:
		case IRP_MJ_WRITE:
			// Queue the IRP to the thread.
			irp.MarkPending();
			error = QueueIrpToIoThread(irp);

			if (error.IsntError())
			{
				error.err = STATUS_PENDING;
			}
			else
			{
				error.err = STATUS_INSUFFICIENT_RESOURCES;
				irp.UnmarkPending();
			}
			break;

		default:
			error.err = STATUS_NOT_IMPLEMENTED;
			break;
	}

	// Complete if not pending.
	if (error.err == STATUS_PENDING)
	{
		UDebug::DebugOut("PGPdisk: Scheduled an %s address %X.", 
			UConstantNamesDrvNT::NameIrpMajorFunction(majorFunc), 
			irpAddress);
	}
	else
	{
		irp.Complete(error.err);
		wasCompleted = TRUE;

		UDebug::DebugOut("PGPdisk: Completed an %s address %X status "
			"%X.", UConstantNamesDrvNT::NameIrpMajorFunction(majorFunc), 
			irpAddress, static_cast<PGPUInt32>(error.err));
	}

	return error.err;
}
