/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CVolumeImpDrvNT.cpp,v 1.10 2002/09/26 01:32:11 wjb Exp $
____________________________________________________________________________*/

#include "pgpClassesConfig.h"

#if	(_WIN32_WINNT >= 0x0500)
#include <mountmgr.h>
#endif	// _WIN32_WINNT >= 0x5000

#include "CDevice.h"
#include "CEvent.h"
#include "CUnicodeString.h"
#include "FsctlCodes.h"

#include "pgpDiskBase.h"
#include "DriverAPI.h"

#include "CDeviceCache.h"
#include "CDriverSubsystemsDrvNT.h"
#include "CVolumeImpDrvNT.h"
#include "DriverMain.h"

_USING_PGP

_UNNAMED_BEGIN

// Constants

const WCHAR		*kMountPointDeviceName		= L"MountPointManager";
const PGPUInt32	IO_REPARSE_TAG_MOUNT_POINT	= 0xA0000003;

// Types

struct REPARSE_GUID_DATA_BUFFER
{
	PGPUInt32	ReparseTag;
	PGPUInt16	ReparseDataLength;
	PGPUInt16	Reserved;
	GUID		ReparseGuid;

	struct
	{
		PGPByte	DataBuffer[1];

	} GenericReparseBuffer;
};

_UNNAMED_END

// Class CVolumeImpDrvNT member functions

#if	(_WIN32_WINNT >= 0x0500)

CVolumeImpDrvNT::CVolumeImpDrvNT() : mPDevice(NULL), mVHFileObject(NULL),
	mVolumeHandle(NULL), mDirFileObject(NULL), mDirHandle(NULL)
{
	Status() = mAsciiDeviceName.Status();

	if (Status().IsntError())
		Status() = mVHDevice.Status();

	if (Status().IsntError())
		Status() = mMountMgr.Status();

	if (Status().IsntError())
		Status() = mDirDevice.Status();
}

#else	// !(_WIN32_WINNT < 0x0500)

CVolumeImpDrvNT::CVolumeImpDrvNT() : mPDevice(NULL), mVHFileObject(NULL),
	mVolumeHandle(NULL)
{
	Status() = mAsciiDeviceName.Status();

	if (Status().IsntError())
		Status() = mVHDevice.Status();
}

#endif	// (_WIN32_WINNT < 0x0500)

CVolumeImpDrvNT::~CVolumeImpDrvNT()
{
	if (IsMounted())
	{
		if (IsAttachedToVolume())
			DetachVolume();
		else
			Unmount(TRUE);
	}
}

PGPBoolean
CVolumeImpDrvNT::HasOpenFiles()
{
	pgpAssert(IsMounted());

	CComboError	error;

	if (IsLockedForIO() || IsLockedForFormat())
		return TRUE;

	// If lock fails, then the volume has open files.
	error = LockForIO();  // NETABUG has open files?

	if (error.IsntError())
		UnlockVolume();

	return error.IsError();
}

CComboError
CVolumeImpDrvNT::AttachVolume(const char *root, const char *deviceName)
{
	pgpAssert(!IsMounted());

	CComboError	error;
	error = mRoot.Assign(root);

	if (error.IsntError())
	{
		if (IsntNull(deviceName))
			error = mAsciiDeviceName.Assign(deviceName);
	}

	if (error.IsntError())
		error = FillInBlockInfo();

	if (error.IsntError())
	{
		mIsAttachedToVolume = TRUE;

		mVolState = kVolStateMounted;
		mLockState = kLockStateNone;
	}

	return error;
}

void
CVolumeImpDrvNT::DetachVolume()
{
	pgpAssert(IsMounted());
	pgpAssert(IsAttachedToVolume());

	if (IsLockedForIO() || IsLockedForFormat())
		UnlockVolume();

	mIsAttachedToVolume = FALSE;

	mVolState = kVolStateUnmounted;
	mLockState = kLockStateNone;
}

CComboError
CVolumeImpDrvNT::Mount(
	const char	*deviceName,
	const char	*root,
	PGPUInt64	totalBlocks,
	PGPUInt32	blockSize,
	const void	*ioHandlerFunc,
	void		*refPtr,
	PGPBoolean	readOnly)
{
	pgpAssert(IsUnmounted());
	pgpAssertStrValid(deviceName);
	pgpAssertStrValid(root);
	pgpAssertAddrValid(ioHandlerFunc, VoidAlign);

	CComboError	error;

	// Prepare strings.
	CUnicodeString	fullDevName;
	CUnicodeString	fullLinkName;

	error = MakeDeviceStrings(deviceName, root, fullDevName, fullLinkName);

	if (error.IsntError())
		error = mAsciiDeviceName.Assign(deviceName);

	if (error.IsntError())
		error = mRoot.Assign(root);

	if (error.IsntError())
	{
		mVolState			= kVolStateMounted;
		mLockState			= kLockStateNone;
		mIsAttachedToVolume = FALSE;

		mTotalBlocks = totalBlocks;
		mBlockSize = blockSize;

		// Is device for this device name in cache?
		if (CDriverSubsystemsDrvNT::DeviceCache().DoesEntryExist(deviceName))
		{
			// It's in cache, get it.
			error = CDriverSubsystemsDrvNT::DeviceCache().GetEntryDevice(
				deviceName, mPDevice);

			if (readOnly)
				mPDevice->Get()->Characteristics |= FILE_READ_ONLY_DEVICE;
			else
				mPDevice->Get()->Characteristics &= ~FILE_READ_ONLY_DEVICE;

			mPDevice->UpdateRefPtr(refPtr);
		}
		else
		{
			// Not in cache, create new entry.
			error = CDriverSubsystemsDrvNT::DeviceCache().CreateEntry(
				deviceName);
			mPDevice = new CDevice;

			if (IsNull(mPDevice))
				error.err = kPGPError_OutOfMemory;

			if (error.IsntError())
			{
				error = mPDevice->Status();

				if (error.IsntError())
				{
					PGPUInt32	attribs	= 0;

					if (CVolume::kUseRemovableFlag)
						attribs |= FILE_REMOVABLE_MEDIA;

					if (readOnly)
						attribs |= FILE_READ_ONLY_DEVICE;

					error = mPDevice->Create(
						DriverMain::GetDriverObjectInitParam(), fullDevName,
						FILE_DEVICE_DISK, attribs, FALSE,
						static_cast<IrpDispatchFunc>(ioHandlerFunc), refPtr);
				}

				if (error.IsntError())
				{
					error = CDriverSubsystemsDrvNT::DeviceCache().
						SetEntryDevice(deviceName, mPDevice);

					if (error.IsError())
						mPDevice->Delete();
				}

				if (error.IsError())
				{
					delete mPDevice;
					mPDevice = NULL;
				}
			}
		}
	}

#if	(_WIN32_WINNT >= 0x0500)

	// Win2k, announce arrival and create mount point.
	if (error.IsntError())
		error = OpenMountMgr();

	if (error.IsntError())
	{
		error = AnnounceWin2kArrival(fullDevName);

		if (error.IsntError())
		{
			CUnicodeString	curLink;
			CUnicodeString	volId;

			error = GetVolumeStrings(deviceName, volId, curLink);

			if (error.IsntError())
			{
				if (!curLink.IsEmpty() && (fullLinkName != curLink))
					error = DeleteMountPoint(volId, fullDevName, curLink);
			}

			if (error.IsntError())
			{
				if (fullLinkName != curLink)
				{
					error = CreateMountPoint(volId, fullDevName, 
						fullLinkName);
				}
			}
		}

		CloseMountMgr();
	}

#else	// !(_WIN32_WINNT >= 0x0500)

	// NT4, create symbolic link.
	if (error.IsntError())
		error = mPDevice->Link(fullLinkName);

#endif	// (_WIN32_WINNT >= 0x0500)

	if (error.IsError())
	{
		if (IsntNull(mPDevice))
		{
			mPDevice->UpdateRefPtr(NULL);
			mPDevice = NULL;	// no need to delete, it's cached
		}

		mAsciiDeviceName.Empty();
		mRoot.Empty();

		mVolState = kVolStateUnmounted;
		mLockState = kLockStateNone;
	}

	return error;
}

CComboError
CVolumeImpDrvNT::Unmount(PGPBoolean isForced)
{
	pgpAssert(IsMounted());
	pgpAssert(!IsAttachedToVolume());

	CComboError	error;

	// Prepare strings.
	CUnicodeString	fullDevName;
	CUnicodeString	fullLinkName;

	error = MakeDeviceStrings(DeviceName(), Root(), fullDevName,
		fullLinkName);

	// Dismount the volume.
	if (error.IsntError() && !IsLockedForFormat() && !IsLockedForIO())
	{
		error = LockForIO(); 

		if (error.IsError() && isForced)
			error = OpenVolumeHandle(); // NETABUG forced unmount failure/crash?
	}

	// Dismount the volume.
	if (error.IsntError())
	{
		error = mVHDevice.SendFsctlRequest(IRP_MN_USER_FS_REQUEST,
			FSCTL_DISMOUNT_VOLUME);

		if (isForced)
			error = CComboError();
	}

#if	(_WIN32_WINNT >= 0x0500)

	// Win2k, delete the mount point.
	if (error.IsntError())
		error = OpenMountMgr();

	if (error.IsntError())
	{
		CUnicodeString	curLink;
		CUnicodeString	volId;

		error = GetVolumeStrings(DeviceName(), volId, curLink);

		if (error.IsntError())
			error = DeleteMountPoint(volId, fullDevName, fullLinkName);

		if (error.IsntError())
		{
			// Make sure cached link name is empty.
			CUnicodeString	empty;
			error = empty.Status();

			CDriverSubsystemsDrvNT::DeviceCache().SetEntryCurLink(
				DeviceName(), empty);
		}

		if (isForced)
			error = CComboError();

		CloseMountMgr();
	}

#else	// !(_WIN32_WINNT >= 0x0500)

	// NT4, delete the symbolic link.
	if (error.IsntError())
		error = mPDevice->Unlink();

#endif	// (_WIN32_WINNT >= 0x0500)

	if (IsLockedForIO() || IsLockedForFormat())
		UnlockVolume();

	if (IsVolumeHandleOpened())
		CloseVolumeHandle();

	if (error.IsntError())
	{
		mPDevice->UpdateRefPtr(NULL);
		mPDevice = NULL;	// no need to delete, it's cached

		mAsciiDeviceName.Empty();
		mRoot.Empty();

		mVolState = kVolStateUnmounted;
		mLockState = kLockStateNone;
	}

	return error;
}

CComboError
CVolumeImpDrvNT::LockForIO()
{
	pgpAssert(IsMounted());
	pgpAssert(!IsLockedForIO() && !IsLockedForFormat());

	CComboError	error;
	error = OpenVolumeHandle(); // NETABUG has open files?

	if (error.IsntError())
	{
		error = mVHDevice.SendFsctlRequest(IRP_MN_USER_FS_REQUEST,
			FSCTL_LOCK_VOLUME);
	}

	if (error.IsntError())
		mLockState = kLockStateIO;

	if (error.IsError())
	{
		if (IsVolumeHandleOpened())
			CloseVolumeHandle();
	}

	return error;
}

CComboError
CVolumeImpDrvNT::LockForFormat()
{
	pgpAssert(IsMounted());
	pgpAssert(!IsLockedForIO() && !IsLockedForFormat());

	CComboError	error;
	error = OpenVolumeHandle();

	if (error.IsntError())
	{
		error = mVHDevice.SendFsctlRequest(IRP_MN_USER_FS_REQUEST,
			FSCTL_LOCK_VOLUME);
	}

	if (error.IsntError())
		mLockState = kLockStateFormat;

	if (error.IsError())
	{
		if (IsVolumeHandleOpened())
			CloseVolumeHandle();
	}

	return error;
}

CComboError
CVolumeImpDrvNT::UnlockVolume()
{
	pgpAssert(IsMounted());
	pgpAssert(IsVolumeHandleOpened());
	pgpAssert(IsLockedForIO() || IsLockedForFormat());

	CComboError	error;

	if (IsLockedForFormat())
	{
		error = mVHDevice.SendFsctlRequest(IRP_MN_USER_FS_REQUEST,
			FSCTL_DISMOUNT_VOLUME);
	}

	if (error.IsntError())
	{
		error = mVHDevice.SendFsctlRequest(IRP_MN_USER_FS_REQUEST,
			FSCTL_UNLOCK_VOLUME);
	}

	if (error.IsntError())
	{
		mLockState = kLockStateNone;
		CloseVolumeHandle();
	}

	return error;
}

CComboError
CVolumeImpDrvNT::Read(void *buf, PGPUInt64 pos, PGPUInt32 nBlocks) const
{
	pgpAssert(IsMounted());
	pgpAssert(IsVolumeHandleOpened());
	pgpAssert(IsLockedForIO() || IsLockedForFormat());
	pgpAssertAddrValid(buf, VoidAlign);

	CComboError		error;
	IO_STATUS_BLOCK	ioStatus;
	LARGE_INTEGER	bigPos;

	bigPos.QuadPart = pos * BlockSize();

	error.err = ZwReadFile(mVolumeHandle, NULL, NULL, NULL, &ioStatus, buf,
		nBlocks * BlockSize(), &bigPos, NULL);

	if (error.HaveNonPGPError())
		error.pgpErr = kPGPError_VolumeOpFailed;

	return error;
}

CComboError
CVolumeImpDrvNT::Write(const void *buf, PGPUInt64 pos, PGPUInt32 nBlocks)
{
	pgpAssert(IsMounted());
	pgpAssert(IsVolumeHandleOpened());
	pgpAssert(IsLockedForIO() || IsLockedForFormat());
	pgpAssertAddrValid(buf, VoidAlign);

	CComboError		error;
	IO_STATUS_BLOCK	ioStatus;
	LARGE_INTEGER	bigPos;

	bigPos.QuadPart = pos * BlockSize();

	error.err = ZwWriteFile(mVolumeHandle, NULL, NULL, NULL, &ioStatus,
		const_cast<void *>(buf), nBlocks * BlockSize(), &bigPos, NULL);

	if (error.HaveNonPGPError())
		error.pgpErr = kPGPError_VolumeOpFailed;

	return error;
}

CComboError
CVolumeImpDrvNT::MakeDeviceStrings(
	const char		*deviceName,
	const char		*linkName,
	CUnicodeString&	fullDevName,
	CUnicodeString&	fullLinkName) const
{
	pgpAssertStrValid(deviceName);
	pgpAssertStrValid(linkName);

	CComboError	error;

	error = fullDevName.Status();

	if (error.IsntError())
		error = fullLinkName.Status();

	if (error.IsntError())
		error = fullDevName.Assign(deviceName);

	if (error.IsntError())
		error = fullDevName.Prepend('\\');

	if (error.IsntError())
		error = fullDevName.Prepend(DriverAPI::kPGPdiskDeviceDir);

	if (error.IsntError())
		error = fullDevName.Prepend(kNTDevicePathPrefix);

	if (error.IsntError())
		error = fullLinkName.Assign(linkName);

	if (error.IsntError())
		error = fullLinkName.Prepend(kNTDosDevicesPrefix);

	if (error.IsntError())
	{
		if (strlen(linkName) == 3)
			error = fullLinkName.Resize(fullLinkName.Length() - 1);
	}

	return error;
}

#if	(_WIN32_WINNT >= 0x0500)

CComboError
CVolumeImpDrvNT::GetVolumeStrings(
	const char		*deviceName,
	CUnicodeString&	volId,
	CUnicodeString&	curLink) const
{
	pgpAssertStrValid(deviceName);

	CComboError	error;

	error = volId.Status();

	if (error.IsntError())
		error = curLink.Status();

	if (error.IsntError())
	{
		error = CDriverSubsystemsDrvNT::DeviceCache().GetEntryCurLink(
			deviceName, curLink);
	}

	if (error.IsntError())
	{
		error = CDriverSubsystemsDrvNT::DeviceCache().GetEntryVolId(
			deviceName, volId);
	}

	if (error.IsntError())
		error = volId.Append('\\');

	return error;
}

CComboError
CVolumeImpDrvNT::OpenMountMgr()
{
	pgpAssert(!mMountMgr.IsAttached());

	CComboError		error;
	CUnicodeString	mpDevName(kMountPointDeviceName);

	error = mpDevName.Status();

	if (error.IsntError())
		error = mpDevName.Prepend(kNTDevicePathPrefix);

	if (error.IsntError())
		error = mMountMgr.Attach(mpDevName);

	return error;
}

void
CVolumeImpDrvNT::CloseMountMgr()
{
	pgpAssert(mMountMgr.IsAttached());
	mMountMgr.Detach();
}

CComboError
CVolumeImpDrvNT::OpenMountPointDir(const CUnicodeString& fullDirName)
{
	pgpAssert(!mDirDevice.IsAttached());

	CComboError	error;

	IO_STATUS_BLOCK		ioStatus;
	OBJECT_ATTRIBUTES	objAttribs;

	InitializeObjectAttributes(&objAttribs, const_cast<UNICODE_STRING *>(
		fullDirName.Get()), OBJ_CASE_INSENSITIVE, NULL, NULL);

	error.err = ZwCreateFile(&mDirHandle, FILE_GENERIC_READ |
		FILE_GENERIC_WRITE | SYNCHRONIZE, &objAttribs, &ioStatus, NULL,
		FILE_ATTRIBUTE_NORMAL, FILE_SHARE_READ | FILE_SHARE_WRITE, 
		FILE_OPEN, FILE_OPEN_REPARSE_POINT | FILE_OPEN_FOR_BACKUP_INTENT | 
		FILE_SYNCHRONOUS_IO_NONALERT, NULL, 0);

	if (error.HaveNonPGPError())
		error.pgpErr = kPGPError_CantOpenFile;

	// Get file object from this handle.
	if (error.IsntError())
	{
		error.err = ObReferenceObjectByHandle(mDirHandle, FILE_READ_DATA,
			NULL, KernelMode, reinterpret_cast<void **>(&mDirFileObject),
			NULL);

		if (error.HaveNonPGPError())
			error.pgpErr = kPGPError_NTDrvObjectOpFailed;

		// Get device object from this file object.
		if (error.IsntError())
		{
			error = mDirDevice.Attach(IoGetRelatedDeviceObject(
				mDirFileObject));

			if (error.IsntError())
				mDirDevice.UseThisFileObject(mDirFileObject);

			if (error.IsError())
			{
				ObDereferenceObject(mDirFileObject);
				mDirFileObject = NULL;
			}
		}

		if (error.IsError())
		{
			ZwClose(mDirHandle);
			mDirHandle = NULL;
		}
	}

	return error;
}

void
CVolumeImpDrvNT::CloseMountPointDir()
{
	pgpAssert(mDirDevice.IsAttached());

	ObDereferenceObject(mDirFileObject);
	mDirFileObject = NULL;

	mDirDevice.Detach();

	ZwClose(mDirHandle);
	mDirHandle = NULL;
}

CComboError
CVolumeImpDrvNT::AnnounceWin2kArrival(const CUnicodeString& fullDevName)
{
	pgpAssert(mMountMgr.IsAttached());

	CArray<PGPByte>	sendBytes(kPGPdiskMaxPathLength * 3);
	CComboError		error;

	error = sendBytes.Status();

	if (error.IsntError())
	{
		MOUNTMGR_TARGET_NAME	*pMTN	= reinterpret_cast<
			MOUNTMGR_TARGET_NAME *>(sendBytes.Get());
		pgpClearMemory(pMTN, sizeof(*pMTN));

		pMTN->DeviceNameLength = fullDevName.Length() * sizeof(WCHAR);

		pgpCopyMemory(fullDevName.Get()->Buffer, &pMTN->DeviceName,
			pMTN->DeviceNameLength);

		error = mMountMgr.SendIoctlRequest(
			IOCTL_MOUNTMGR_VOLUME_ARRIVAL_NOTIFICATION, pMTN,
			sendBytes.Size(), NULL, NULL);
	}

	return error;
}

CComboError
CVolumeImpDrvNT::CreateMountPoint(
	const CUnicodeString&	volId,
	const CUnicodeString&	fullDevName,
	const CUnicodeString&	fullLinkName)
{
	CArray<PGPByte>	sendBytes(kPGPdiskMaxPathLength * 3);
	CComboError		error;

	error = sendBytes.Status();

	if (error.IsntError())
	{
		if (fullLinkName.LastWChar() == ':')
		{
			pgpAssert(mMountMgr.IsAttached());

			// Make a drive letter link.
			MOUNTMGR_CREATE_POINT_INPUT	*pMCPI	= reinterpret_cast<
				MOUNTMGR_CREATE_POINT_INPUT *>(sendBytes.Get());
			pgpClearMemory(pMCPI, sizeof(*pMCPI));

			pMCPI->SymbolicLinkNameOffset = sizeof(
				MOUNTMGR_CREATE_POINT_INPUT);
			pMCPI->SymbolicLinkNameLength = fullLinkName.Length() *
				sizeof(WCHAR);

			pgpCopyMemory(fullLinkName.Get()->Buffer,
				sendBytes.Get() + pMCPI->SymbolicLinkNameOffset,
				pMCPI->SymbolicLinkNameLength);

			pMCPI->DeviceNameOffset = pMCPI->SymbolicLinkNameOffset +
				pMCPI->SymbolicLinkNameLength;
			pMCPI->DeviceNameLength = fullDevName.Length() * sizeof(WCHAR);

			pgpCopyMemory(fullDevName.Get()->Buffer,
				sendBytes.Get() + pMCPI->DeviceNameOffset,
				pMCPI->DeviceNameLength);

			error = mMountMgr.SendIoctlRequest(IOCTL_MOUNTMGR_CREATE_POINT,
				pMCPI, sendBytes.Size(), NULL, NULL);
		}
		else
		{
			pgpAssert(!mDirDevice.IsAttached());
			error = OpenMountPointDir(fullLinkName);

			if (error.IsntError())
			{
				// Make a folder link.
				REPARSE_GUID_DATA_BUFFER	*pRGDB	= reinterpret_cast<
					REPARSE_GUID_DATA_BUFFER *>(sendBytes.Get());

				sendBytes.Wipe();

				pRGDB->ReparseTag = IO_REPARSE_TAG_MOUNT_POINT;
				pRGDB->ReparseDataLength = 0x6e;

				pRGDB->ReparseGuid.Data1 = 0x00620000;
				pRGDB->ReparseGuid.Data2 = 0x0064;
				pRGDB->ReparseGuid.Data3 = 0;

				pgpCopyMemory(volId.Get()->Buffer, pRGDB->ReparseGuid.Data4,
					volId.Length() * sizeof(WCHAR));

				error = mDirDevice.SendFsctlRequest(IRP_MN_USER_FS_REQUEST,
					FSCTL_SET_REPARSE_POINT, pRGDB, 0x76);
			}
		}
	}

	return error;
}

CComboError
CVolumeImpDrvNT::DeleteMountPoint(
	const CUnicodeString&	volId,
	const CUnicodeString&	fullDevName,
	const CUnicodeString&	fullLinkName)
{
	CComboError	error;

	CArray<PGPByte>	sendBytes(kPGPdiskMaxPathLength * 3);
	CArray<PGPByte>	recvBytes(kPGPdiskMaxPathLength * 3);

	error = sendBytes.Status();

	if (error.IsntError())
		error = recvBytes.Status();

	if (error.IsntError())
	{
		if (fullLinkName.LastWChar() == ':')
		{
			pgpAssert(mMountMgr.IsAttached());

			// Delete a drive letter link.
			MOUNTMGR_MOUNT_POINT	*pMMP	= reinterpret_cast<
				MOUNTMGR_MOUNT_POINT *>(sendBytes.Get());
			pgpClearMemory(pMMP, sizeof(*pMMP));

			pMMP->SymbolicLinkNameOffset = sizeof(MOUNTMGR_MOUNT_POINT);
			pMMP->SymbolicLinkNameLength = fullLinkName.Length() *
				sizeof(WCHAR);

			pgpCopyMemory(fullLinkName.Get()->Buffer, sendBytes.Get() +
				pMMP->SymbolicLinkNameOffset, pMMP->SymbolicLinkNameLength);

			error = mMountMgr.SendIoctlRequest(IOCTL_MOUNTMGR_DELETE_POINTS,
				pMMP, sendBytes.Size(), recvBytes.Get(), recvBytes.Size());
		}
		else
		{
			pgpAssert(mDirDevice.IsAttached());

			// Delete a folder link.
			REPARSE_GUID_DATA_BUFFER	*pRGDB	= reinterpret_cast<
				REPARSE_GUID_DATA_BUFFER *>(sendBytes.Get());

			sendBytes.Wipe();

			error = mDirDevice.SendFsctlRequest(IRP_MN_USER_FS_REQUEST,
				FSCTL_GET_REPARSE_POINT, NULL, 0, pRGDB, sendBytes.Size());

			if (error.IsntError())
			{
				pRGDB->ReparseDataLength = 0;

				error = mDirDevice.SendFsctlRequest(IRP_MN_USER_FS_REQUEST, 
					FSCTL_DELETE_REPARSE_POINT,pRGDB, 8);
			}

			if (error.IsntError())
				CloseMountPointDir();
		}
	}

	return error;
}

#endif	// (_WIN32_WINNT < 0x0500)

CComboError
CVolumeImpDrvNT::GetGeometry(DISK_GEOMETRY& geom)
{
	CComboError	error;
	PGPBoolean	weOpenedVolume	= FALSE;

	if (!IsVolumeHandleOpened())
	{
		error = OpenVolumeHandle();
		weOpenedVolume = error.IsntError();
	}

	if (error.IsntError())
	{
		error = mVHDevice.SendIoctlRequest(IOCTL_DISK_GET_DRIVE_GEOMETRY,
			NULL, 0, &geom, sizeof(geom));
	}

	if (weOpenedVolume)
		CloseVolumeHandle();

	return error;
}

CComboError
CVolumeImpDrvNT::GetPartitionInfo(PARTITION_INFORMATION& partInfo)
{
	CComboError	error;
	PGPBoolean	weOpenedVolume	= FALSE;

	if (!IsVolumeHandleOpened())
	{
		error = OpenVolumeHandle();
		weOpenedVolume = error.IsntError();
	}

	if (error.IsntError())
	{
		error = mVHDevice.SendIoctlRequest(IOCTL_DISK_GET_PARTITION_INFO,
			NULL, 0, &partInfo, sizeof(partInfo));
	}

	if (weOpenedVolume)
		CloseVolumeHandle();

	return error;
}

CComboError
CVolumeImpDrvNT::FillInBlockInfo()
{
	CComboError				error;
	DISK_GEOMETRY			geom;
	PARTITION_INFORMATION	partInfo;

	error = GetGeometry(geom);

	if (error.IsntError())
		error = GetPartitionInfo(partInfo);

	if (error.IsntError())
	{
		mBlockSize = static_cast<PGPUInt32>(geom.BytesPerSector);
		mTotalBlocks = partInfo.PartitionLength.QuadPart;
	}

	return error;
}

CComboError
CVolumeImpDrvNT::OpenVolumeHandle()
{
	pgpAssert(!IsVolumeHandleOpened());

	CComboError	error;

	CUnicodeString	linkName;
	error = linkName.Status();

	if (error.IsntError())
	{
		// If root is mount point on directory, use volume ID.
		if (mRoot.Length() > 3)
		{
			error = CDriverSubsystemsDrvNT::DeviceCache().GetEntryVolId(
				DeviceName(), linkName);
		}
		else
		{
			error = linkName.Assign(mRoot);

			if (error.IsntError())
				error = linkName.Resize(linkName.Length() - 1);	// lose slash

			if (error.IsntError())
				error = linkName.Prepend(kNTLinkPathPrefix);
		}
	}

	if (error.IsntError())
	{
		IO_STATUS_BLOCK		ioStatus;
		OBJECT_ATTRIBUTES	objectAttributes;

		// Get handle to root of volume.
		InitializeObjectAttributes(&objectAttributes, const_cast<
			UNICODE_STRING *>(linkName.Get()), OBJ_CASE_INSENSITIVE, NULL,
			NULL);

		error.err = ZwCreateFile(&mVolumeHandle, FILE_GENERIC_READ |
			FILE_GENERIC_WRITE | SYNCHRONIZE, &objectAttributes, &ioStatus,
			NULL, 0, FILE_SHARE_READ | FILE_SHARE_WRITE, FILE_OPEN,
			FILE_SYNCHRONOUS_IO_NONALERT, NULL, 0);

		if (error.HaveNonPGPError())
			error.pgpErr = kPGPError_VolumeOpFailed; // NETABUG has open files/forced unmount?

		if (error.IsntError())
		{
			// Get file object from this handle.
			error.err = ObReferenceObjectByHandle(mVolumeHandle,
				FILE_READ_DATA, NULL, KernelMode,
				reinterpret_cast<void **>(&mVHFileObject), NULL);

			if (error.HaveNonPGPError())
				error.pgpErr = kPGPError_NTDrvObjectOpFailed;

			if (error.IsntError())
			{
				// Get device object from this file object.
				error = mVHDevice.Attach(
					IoGetRelatedDeviceObject(mVHFileObject));

				if (error.IsntError())
					mVHDevice.UseThisFileObject(mVHFileObject);

				if (error.IsError())
				{
					ObDereferenceObject(mVHFileObject);
					mVHFileObject = NULL;
				}
			}

			if (error.IsError())
			{
				ZwClose(mVolumeHandle);
				mVolumeHandle = NULL;
			}
		}
	}

	return error;
}

void
CVolumeImpDrvNT::CloseVolumeHandle()
{
	pgpAssert(IsVolumeHandleOpened());

	ObDereferenceObject(mVHFileObject);
	mVHFileObject = NULL;

	mVHDevice.Detach();

	ZwClose(mVolumeHandle);
	mVolumeHandle = NULL;
}
