////////////////////////////////////////////////////////////////////////////////
//
//  File : GwCommand.cpp
//
//  Copyright (C) 2002 PGP Corporation
//
//  ABSTRACT
//		Create CommandFactory, EventMonitor, IConFactory, and GWCommand objects.
// 
//  Author: Satya S. Das
//
////////////////////////////////////////////////////////////////////////////////

#include "windows.h"
#include "windowsx.h"
#include "gwoapi.h"
#include "gwc3po.h"
#include "gwc3cmd.h"
#include "gwdll.h"
#include "GwCommand.h"
#include "CommandFactory.h"
#include "util.h"
#include "EventNotif.h"
#include "globals.h"
#include "globdefs.h"
#include <ole2.h>
#include "resource.h"
#include <crtdbg.h>
#include <stdio.h>
#include <richedit.h>
#include <commctrl.h>

#include "pgpclientlib.h"
#include "pgpUtilities.h"
#include "decryptverify.h"
#include "pgpsc.h"
#include "pgpclientprefs.h"
#include "pgpdebug.h"


// Do the following define stuff in only one file in your project
#undef DEFINE_GUID
#define DEFINE_GUID(name, l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8) \
        EXTERN_C const GUID CDECL name \
                = { l, w1, w2, { b1, b2,  b3,  b4,  b5,  b6,  b7,  b8 } }
// End of define stuff

extern IC3POManager *m_pIMgr;

// local definitions 

// default settings

// external functions 

// internal functions 
IUnknown * BuildIUnkDispatch(REFCLSID libid, REFCLSID iid, IUnknown *pIUnk, void *pIFac);

VOID WINAPI GWEventNotify (DWORD dwID, DWORD dwEvent);
extern HRESULT Publish(LPCSTR szTokenBuf, BSTR *pbstrResult);

// public data 

// private data 

// public functions 

// private functions 
// IGWCommand methods
//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::Set_BaseCommand				
//	
//  Description:remember the base command interface
//  				
//  In:	IGWCommand * pigwBaseCmd - the base command interface ptr
//
//  Out:none
//
//  Comments:		
///////////////////////////////////////////////////////////////////
void CGWCommand::Set_BaseCommand(IGWCommand *pigwBaseCmd)
{
	m_pigwBaseCmd=pigwBaseCmd;
}

//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::CGWCommand				
//
//  Description:CGWCommand constructor. Init reference count. Set 
//				command ID. Set IDispatch to null	
//  
//  In:	int nID - Command ID.
//  
//  Out:none
//  
//  Comments:		
///////////////////////////////////////////////////////////////////
CGWCommand::CGWCommand(int nID, DWORD dwCmdUiInfo)
{
	PgpGwTrace("CGWCommand::CGWCommand %#x, m_nID=%d\n", this, nID);
	m_cRef=1;
	m_nID=nID;
	m_dwCmdUiInfo=dwCmdUiInfo;
	m_pIUnkDisp=NULL;
	m_pigwBaseCmd=NULL;
	m_pIDispOfCmd=NULL;
	m_bstrLongPrompt=NULL;
	m_bstrToolTip=NULL;
	m_hCtxtWindow=NULL;
	m_hOverlappingWnd=NULL;
}

//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::~CGWCommand				
//
//  Description:CGWCommand destructor. If we have a IDispatch release it.
//				Release our long prompt if we have one.	
//  
//  In:	none
//  
//  Out: none
//  
//  Comments:		
///////////////////////////////////////////////////////////////////
CGWCommand::~CGWCommand()
{
	PgpGwTrace("CGWCommand::~CGWCommand - this=%#x, m_nID=%d\n", this, m_nID);	

	if((NULL != m_hCtxtWindow) && (NULL != g_pccihCmdHash))
	{
		BOOL bRet=g_pccihCmdHash->Remove((ULONG)m_hCtxtWindow);
		//pgpAssert(TRUE == bRet);
	}
		
	if(NULL != m_pIUnkDisp)
	{
		m_pIUnkDisp->Release();
		m_pIUnkDisp=NULL;
	}

	if(NULL != m_bstrLongPrompt)
	{
		SysFreeString(m_bstrLongPrompt);
		m_bstrLongPrompt=NULL;
	}

	if(NULL != m_bstrToolTip)
	{
		SysFreeString(m_bstrToolTip);
		m_bstrToolTip=NULL;
	}

	if(NULL != m_pigwBaseCmd)
	{
		m_pigwBaseCmd->Release();
		m_pigwBaseCmd=NULL;
	}

	if(NULL != m_pIDispOfCmd)
	{
		m_pIDispOfCmd->Release();
		m_pIDispOfCmd=NULL;
	}
}

//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::QueryInterface				
//
//  Description:QueryInterface	for CGWCommand.
//
//  In:	REFIID riid
//	
//  
//  Out:LPVOID FAR* ppv
//  
//  Comments:		
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::QueryInterface(REFIID riid, LPVOID FAR* ppv)
{
	//PgpGwTrace("CGWCommand::QueryInterface %#x\n", this);
	if(NULL == ppv)
		return E_INVALIDARG;

    *ppv=NULL;


	if((IID_ICommandFactory == riid) ||	(IID_IUnknown == riid))
	{
		*ppv  = this;
		((LPUNKNOWN)*ppv)->AddRef();
		return S_OK;
	}

	if(IID_IDispatch == riid)
	{
		if (NULL == m_pIUnkDisp)
		{
			m_pIUnkDisp=BuildIUnkDispatch(LIBID_PgpGwTypeLibrary,
					IID_IGWCommand,	(IUnknown *)(IGWCommand *)this,
					(void *)(IGWCommand *)this);
		}

		if (NULL != m_pIUnkDisp)
			return m_pIUnkDisp->QueryInterface(IID_IDispatch, ppv);
	}

	return E_NOINTERFACE;
}

//////////////////////////////////////////////////////////////////
//	Name: CGWCommand::AddRef				
//
//  Description:Add to reference count	
//
//  In:	none
//
//  Out:m_cRef - New reference count
//
//  Comments:		
///////////////////////////////////////////////////////////////////
STDMETHODIMP_(ULONG) CGWCommand::AddRef()
{
	//PgpGwTrace("CGWCommand::AddRef %#x, count after addref=%d\n", this, m_cRef+1);	
	return ++m_cRef;
}

//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::Release				
//
//  Description:Decrament reference count by one. If reference 
//		count goes to zero delete Command object.	
//  
//  In:none
//  
//  Out:cRef - New reference count.
//  
//  Comments:		
///////////////////////////////////////////////////////////////////
STDMETHODIMP_(ULONG) CGWCommand::Release()
{
	//PgpGwTrace("CGWCommand::Release %#x, count after rel=%d\n", this, m_cRef-1);	

	ULONG cRef = --m_cRef;
	if(0 == m_cRef)
	{
		delete this;
	}
	return cRef;
}

//////////////////////////////////////////////////////////////////
//	Name:			CGWCommand::get_BaseCmd				
//
//  Description:This routine should get the Base Command and 
//				return it as a CGWCommand.	
//  
//  In:	none
//  
//  Out: IDispatch * FAR* ppIDispBaseCmd - Base command
//  
//  Comments:		
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::get_BaseCmd(IDispatch * FAR* ppIDispBaseCmd)
{
	//PgpGwTrace("CGWCommand::get_BaseCmd\n");	
	return S_OK;
}

//////////////////////////////////////////////////////////////////
//	Name: CGWCommand::get_LongPrompt				
//
//  Description:This routine should return the long prompt for the 
//				GWCommand.	
//
//  In:none
//
//  Out:BSTR FAR* pbstrLongPrompt - pointer to long prompt string 
//
//  Comments:		
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::get_LongPrompt(BSTR FAR* pbstrLongPrompt)
{
	//PgpGwTrace("CGWCommand::get_LongPrompt\n");	
	*pbstrLongPrompt = SysAllocString(m_bstrLongPrompt);
	return S_OK;
}

//////////////////////////////////////////////////////////////////
//	Name: CGWCommand::get_Parameters				
//
//  Description: This is for later use. (Always null)	
//
//  In:	none
//
//  Out: IDispatch * FAR* ppIDispBaseCmd
//
//  Comments:		
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::get_Parameters(IDispatch * FAR* ppIDispBaseCmd)
{
	//PgpGwTrace("CGWCommand::get_Parameters\n");	
	return S_OK;
}

//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::get_PersistentID				
//
//  Description:This routine should return the persistent ID
//              for the command.	
//  
//  In:none
//  
//  Out:BSTR FAR* pbstrPersistentID
//  
//  Comments:		
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::get_PersistentID(BSTR FAR* pbstrPersistentID)
{
	//PgpGwTrace("CGWCommand::get_PersistentID this=%#x\n", this);	
	return E_FAIL;
}

//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::get_ToolTip				
//
//  Description:This routine should return the tool tip for custom 
//				toolbar commands only.	
//  
//  In:	none
//  
//  Out:BSTR FAR* pbstrToolTip
//  
//  Comments:Only commands genertated and placed on the toolbar with
//  CommandFactory.Customise Toolbar(or explicitly added by the user) have
//  this method invoked.		
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::get_ToolTip(BSTR FAR* pbstrToolTip)
{
	//PgpGwTrace("CGWCommand::get_ToolTip\n");	
	*pbstrToolTip = SysAllocString(m_bstrToolTip);
	return S_OK;
}

//////////////////////////////////////////////////////////////////
//	Name: CGWCommand::Execute				
//
//  Description: This routine should execute the command.	
//
//  In:	none
//
//  Out: none
//
//  Comments:At this point the processing of the command is the C3po's
//           responsiblity.
//
//      code for getting message object selected.
//	
//		IDispatch *pIDisp;
//		IGWClientState *pIClientState;
//		IGWMessage *pIMessage;
//		HRESULT hr;
//		//Get ClientState interfaces
//  	m_pIMgr->get_ClientState(&pIDisp);
//		hr = pIDisp->QueryInterface(IID_IClientState, (void**)&pIClientState);
//  		pIDisp->Release();
//
//		// get CommandMesage interfaces
//		pIClientState->get_CommandMessage(&pIDisp);
//		hr = pIDisp->QueryInterface(IID_IGWMessage, (void**)&pIMessage);
//		pIDisp->Release();
//
//		pIClientState->Release();			// release ClientState
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::Execute()
{
	PgpGwTrace("CGWCommand::Execute\n");	
	HRESULT hrRet=S_OK;
	PMSGINFO lpMsgInfo=NULL;
	PGPError pgpErrRet=kPGPError_NoErr;

	//fill up the members and the hash with relevant details that might be missing
	hrRet=LearnCommandContext();
	if(FAILED(hrRet))//if we have any problem determining the context
	{
		pgpAssert(FALSE);
		goto end;
	}

	//the hash should have been setup
	pgpAssert(NULL != g_pccihCmdHash);
	if(NULL == g_pccihCmdHash)
		goto end;

	//query the hash for info
	pgpAssert(0 != m_dwCmdUiInfo);
	if(NULL != m_hCtxtWindow)//if we have found the context window 
	{
		//get the info from the hash
		lpMsgInfo=g_pccihCmdHash->Get((ULONG)m_hCtxtWindow);
		pgpAssert(NULL != lpMsgInfo);
		if(NULL == lpMsgInfo)
			goto end;
	}

	//NOTE:in case of message open request we dont know the context
	//window until the very point where we are told to open the message

	switch (m_nID)//check the command ID
	{
		case IDM_LAUNCH_PGPKEYS://launch pgpkeys
			{
				//get the location where pgp keys is installed
				char szPgpKeysPath[MAX_PATH]={0};
				PGPclGetPath(kPGPclPGPkeysExeFile, szPgpKeysPath, MAX_PATH-1);

				//run the executable
				PGPclExecute(szPgpKeysPath, SW_SHOW);
				break;
			}
			break;

		case IDM_SHOW_PREFS://show the pgp options dialog
			{
				pgpAssert(NULL != g_pgpContext);
				pgpAssert(NULL != m_hCtxtWindow);

				pgpErrRet=PGPclPreferences(g_pgpContext, m_hCtxtWindow,/*main window handle*/
							kPGPclEmailPrefs, NULL);
				pgpAssert((pgpErrRet == kPGPError_UserAbort) || !IsPGPError(pgpErrRet));
				if ((pgpErrRet != kPGPError_UserAbort) && IsPGPError(pgpErrRet))
				{
					//show "failed to invoke the preferences dialog !!"
					DisplayMessage(NULL, IDS_E_PREF_LAUNCH_FAILURE);
					break;
				}
			}
			break;

		case IDM_AUTO_ENCRYPT_ONSEND:
			pgpAssert(NULL != m_hCtxtWindow);
			g_pccihCmdHash->ToggleState((ULONG)m_hCtxtWindow, MIS_AUTO_ENCRYPT_ON_SEND);
			break;

		case IDM_AUTO_SIGN_ONSEND:
			pgpAssert(NULL != m_hCtxtWindow);
			g_pccihCmdHash->ToggleState((ULONG)m_hCtxtWindow, MIS_AUTO_SIGN_ON_SEND);
			break;

		case IDM_DECRYPT_VERIFY://TODO:add the functionality for 
								//the command in main menu context
			{
				BSTR bstrResult=NULL;
				//int iHandleTokenRet=DLL_HAN_NO_ERROR;
				char szTokenBuf[200]={0};
				int iCipherTxtMsgLen=0;
				

				//PGP structs
				PGPError pgpErrRet=kPGPError_NoErr;
				PGPtlsContextRef tlsContext=NULL;
				PGPSize pgpsBufOutLen=0;
				char *pCipherBuf=NULL;
				char *pOutBuf=NULL;
				char *pFinalBuf=NULL;
				BOOL bForYourEyesOnly=FALSE;

				char szTitle[255]={0};
				int iRet=0;
				BOOL bRet=TRUE;

				pgpAssert(NULL != m_hCtxtWindow);
				pgpAssert(NULL != lpMsgInfo);
				pgpAssert((0 < (m_dwCmdUiInfo&CUIO_OLDMAIL_CONTEXT)) ||
						(0 < (m_dwCmdUiInfo&CUIO_MAINWND_CONTEXT)) ||
						(0 < (m_dwCmdUiInfo&CUIO_ATTVIEW_CONTEXT)));

				g_pccihCmdHash->Lock();
				
				if((m_dwCmdUiInfo&CUIO_MAINWND_CONTEXT) &&//if in the main window context
				   (NULL == lpMsgInfo->bstrId))//if no/multiple mails are selected
				{
					//so, we dont know which message to operate on. now we can
					//pop a message saying that or else we just do nothing
					pgpAssert(FALSE);//interesting case to watch(not a failure)
					goto idm_decrypt_verify_cleanup;
				}

				if((m_dwCmdUiInfo&CUIO_ATTVIEW_CONTEXT) &&//if in the attachment viewer context
				   (NULL != lpMsgInfo->bstrId))
				{
					//get the attachment file name(s)

				
					//SendMessage(hWndViewer, WM_SETTEXT, 0, (LPARAM)"hello");

					goto idm_decrypt_verify_cleanup;
				}


				//get the type of this message being verified/decrypted
				ZeroMemory(szTokenBuf, sizeof(szTokenBuf));
				pgpAssert(NULL != lpMsgInfo->bstrId);
				sprintf(szTokenBuf, "ItemGetType(\"%S\")", lpMsgInfo->bstrId);
				hrRet=Publish(szTokenBuf, &bstrResult);
				if(FAILED(hrRet))
				{
					pgpAssert(FALSE);
					DisplayMessage(NULL, IDS_E_ITEM_TYPE, hrRet);
					goto idm_decrypt_verify_cleanup;
				}

				//check if the message being operated is a mail
				if(0 != wcscmp(bstrResult, MAILMESSAGETYPEW))
				{
					PgpGwTrace("Skipping one non-mail item(type=%S) sent.\n", bstrResult);
					goto idm_decrypt_verify_cleanup;
				}
				
				//check if the user has license to run us. we must have a valid 
				//license with desktop flag turned on in the license number for 
				//decrypt/verify to work
				if(FALSE == PgpGwExLicenseChk())
					goto idm_decrypt_verify_cleanup;

				//get a tls context for ourselves
				pgpAssert(NULL != g_pgpContext);
				pgpErrRet=PGPNewTLSContext(g_pgpContext, &tlsContext);
				if(IsPGPError(pgpErrRet) || (NULL == tlsContext))
				{
					pgpAssert(FALSE);
					hrRet=E_FAIL;
					DisplayMessage(NULL, IDS_E_PGP_INTERNAL, pgpErrRet);
					goto idm_decrypt_verify_cleanup;
				}

				//get the message text
				ZeroMemory(szTokenBuf, sizeof(szTokenBuf));
				sprintf(szTokenBuf, "ItemGetText(\"%S\";10)", lpMsgInfo->bstrId); 
				hrRet=Publish(szTokenBuf, &bstrResult);
				if(FAILED(hrRet))
				{
					pgpAssert(FALSE);
					DisplayMessage(NULL, IDS_E_GET_MESSAGE_TXT, hrRet);
					goto idm_decrypt_verify_cleanup;
				}
				
				//if there is no message text in the mail 
				if(0 == wcslen(bstrResult))
					goto idm_decrypt_verify_cleanup;//we are done

				//convert the message to ansi
				pCipherBuf=ConvertToAnsi(bstrResult);
				pgpAssert(NULL != pCipherBuf);
				if(NULL == pCipherBuf)
				{
					hrRet=E_OUTOFMEMORY;
					goto idm_decrypt_verify_cleanup;
				}

				//decrypt and verify the message text
 				pgpErrRet=DecryptVerifyBuffer(g_hInstance, m_hCtxtWindow, g_pgpContext, tlsContext, 
							CLIENTNAME, MODULEFILENAME, pCipherBuf, strlen(pCipherBuf), FALSE, FALSE, FALSE, 
							(void **)&pOutBuf, &pgpsBufOutLen, &bForYourEyesOnly);
				pgpAssert(!IsPGPError(pgpErrRet));
				pgpAssert(NULL != pOutBuf);
				pgpAssert(0 < pgpsBufOutLen);
				if(IsPGPError(pgpErrRet) || (NULL == pOutBuf) || (0 >= pgpsBufOutLen))
				{
					hrRet=E_FAIL;
					DisplayMessage(NULL, IDS_E_DECRYPT_VERIFY, pgpErrRet);
					goto idm_decrypt_verify_cleanup;
				}
				
				//if we are decrypting text in a new mail
				if(m_dwCmdUiInfo&CUIO_NEWMAIL_CONTEXT)//this should never get executed
				{
					//allocate buffer for the token and parameters
					pFinalBuf=(char *)calloc(pgpsBufOutLen+sizeof(szTokenBuf), 1);
					pgpAssert(NULL != pFinalBuf);
					if(NULL == pFinalBuf)
					{
						hrRet=E_OUTOFMEMORY;
						goto idm_decrypt_verify_cleanup;
					}

					//change the mail body to the decrypted stuff.
					sprintf(pFinalBuf, "ItemSetText(\"%S\";10;\"%s\")", lpMsgInfo->bstrId, pOutBuf); 
					hrRet=Publish(pFinalBuf, &bstrResult);
					if(FAILED(hrRet))
					{
						pgpAssert(FALSE);
						DisplayMessage(NULL, IDS_E_SET_DECMSG_TXT, hrRet);
						goto idm_decrypt_verify_cleanup;
					}
				}
				else if(m_dwCmdUiInfo&CUIO_OLDMAIL_CONTEXT)
				{
					//if we are supposed to show decrypted stuff in secure viewer
					if((TRUE == bForYourEyesOnly) || (PGPscGetSecureViewerPref(g_pgpContext)))
					{
						//pop up secure viewer with(out) anti-tempest font
						PGPscTempestViewer(g_pgpContext, m_hCtxtWindow, pOutBuf, 
							pgpsBufOutLen, bForYourEyesOnly);
					}
					else//secure viewer option is not selected in PGP options
					{
						HWND hPlainTxtMsgWnd=NULL, hHtmlTxtMsgWnd=NULL, 
							hOverlappingWnd=NULL, hTrackedMsgWnd=NULL;

						//find out the current state of affairs
						hPlainTxtMsgWnd=lpMsgInfo->hPlainTxtMsgWnd;
						hHtmlTxtMsgWnd=lpMsgInfo->hHtmlTxtMsgWnd;
						hOverlappingWnd=lpMsgInfo->hNewMsgTxtWnd;

						//show the decrypted text
						SetTextOfReadOnlyMail(pOutBuf, hPlainTxtMsgWnd, hHtmlTxtMsgWnd, 
							hOverlappingWnd, hTrackedMsgWnd);
						
						//remember the window handles for operations later
						if((NULL != hPlainTxtMsgWnd) || (NULL != hOverlappingWnd) || (NULL != hTrackedMsgWnd))
						{
							lpMsgInfo->hPlainTxtMsgWnd=hPlainTxtMsgWnd;
							lpMsgInfo->hNewMsgTxtWnd=hOverlappingWnd;
							lpMsgInfo->hTrackedMsgWnd=hTrackedMsgWnd;
						}
						
						//if we are showing the overlapping window
						if((NULL != hOverlappingWnd) && 
						   (NULL != hPlainTxtMsgWnd) &&
						   (hPlainTxtMsgWnd != hOverlappingWnd) &&
						   (hHtmlTxtMsgWnd != hOverlappingWnd))
						{
							EnableCommands(lpMsgInfo->hMenuBar, FALSE, ECO_UNSUPPORTED);
						}
					}
				}
				else if(m_dwCmdUiInfo&CUIO_MAINWND_CONTEXT)
					0==0;//TODO: add code here for main window command

			idm_decrypt_verify_cleanup:

				g_pccihCmdHash->UnLock();

				if(NULL != bstrResult)
				{
					SysFreeString(bstrResult);
					bstrResult=NULL;
				}

				if(NULL != tlsContext)
				{
					PGPFreeTLSContext(tlsContext);
					tlsContext=NULL;
				}

				if(NULL != pOutBuf)
				{
					pgpAssert(pgpsBufOutLen > 0);
					PGPFreeData(pOutBuf);
					pOutBuf=NULL;
				}
				if(NULL != pFinalBuf)
				{
					free(pFinalBuf);
					pFinalBuf=NULL;
				}
				if(NULL != pCipherBuf)//not necessary but NTL -ssd
				{
					delete pCipherBuf;
					pCipherBuf=NULL;
				}

				//TODO:free pszRecipients array
				//return iHandleTokenRet;
			}

			break;

		case IDM_CONVERT_TO_PLAIN_TEXT:
			pgpAssert(NULL != m_hCtxtWindow);
			g_pccihCmdHash->ToggleState((ULONG)m_hCtxtWindow, MIS_CONV_TO_PLAIN_TEXT);
			break;
	
		case IDM_SHOW_ORIGINAL:
			{
				HWND hTrackedMsgWnd=NULL, hOverlappingWnd=NULL;
				HMENU hMainMenu=NULL;

				pgpAssert(NULL != m_hCtxtWindow);
				pgpAssert(NULL != lpMsgInfo);

				//get the window handles involved
				g_pccihCmdHash->Lock();
				hTrackedMsgWnd=lpMsgInfo->hTrackedMsgWnd;
				hOverlappingWnd=lpMsgInfo->hNewMsgTxtWnd;
				hMainMenu=lpMsgInfo->hMenuBar;
				g_pccihCmdHash->UnLock();

				//hide the overlapping window if it is around
				if((NULL != hOverlappingWnd) && (IsWindowVisible(hOverlappingWnd)))
				{
					ShowWindow(hTrackedMsgWnd, SW_SHOW);//show the original window
					ShowWindow(hOverlappingWnd, SW_HIDE);//hide the decrypt/verify window
					EnableCommands(hMainMenu, TRUE, 0);
				}

			}
			break;
		case IDM_MAIL_OPEN:
			{
#ifdef _DEBUG
				char szClassName[200]={0};
#endif
				pgpAssert(NULL == m_hCtxtWindow);//context window should be unknown till now
				pgpAssert(NULL == lpMsgInfo);//no info from hash should be existing for this command
				pgpAssert(NULL != m_pigwBaseCmd);
				if(NULL == m_pigwBaseCmd)
					break;

				HRESULT hrRet=m_pigwBaseCmd->Execute();
				pgpAssert(SUCCEEDED(hrRet));
				if(FAILED(hrRet))
					break;//error while opening - we dont do nothing

				//attempt to learn the context. we set the cmd context now so 
				//that learning is possible.
				m_dwCmdUiInfo |= CUIO_MAIL_CONTEXT;
				LearnCommandContext();
				pgpAssert(NULL != m_hCtxtWindow);

#ifdef _DEBUG
				//the current window should now be the opened mail window
				pgpAssert(GetClassName(m_hCtxtWindow, szClassName, sizeof(szClassName)-sizeof(char)) > 0);
				pgpAssert(0 == strcmp(szClassName, MAILVIEWWINDOWCLASSNAMEA));
#endif

				//now we need to decrypt/verify the message
				m_nID=IDM_DECRYPT_VERIFY;//we mutate "open" command to a decrypt command from now on
				hrRet=this->Execute();
				pgpAssert(SUCCEEDED(hrRet));
			}
			break;
		default:
			pgpAssert(FALSE);//control should never come here. unsupported operation attempted
			break;			 //on this command object !!
	}

end:
	return S_OK;
}

//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::Help				
//
//  Description:Invokes help for the GWCommand.	
//
//  In:	none
//
//  Out:none
//
//  Comments:		
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::Help()
{
	PgpGwTrace("CGWCommand::Help\n");	
	return S_OK;
}

////////////////////////////////////////////////////////////////////
//	Name: CGWCommand::Undo				
//  
//  Description:Required method. Never called in XTD.	
//  
//  In:	none
//  
//  Out: none
//  
//  Comments:		
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::Undo()
{
	PgpGwTrace("CGWCommand::Undo\n");	
	return S_OK;
}

//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::Validate				
//
//  Description:This interface is called to determine the state of a C3po
//       command. Determining the state is termed the validation.
//       he command is assumed by default to be in an enabled, unchecked,
//       visible state. The C3po provider can then modify that assumption
//       by returning flags. The available flags are:
//
//       eGW_CMDVAL_ALWAYS - The command is always in the same state
//	                         and there is no need to validate it again.
//       eGW_CMDVAL_CHECKED - The command has a check mark.
//       eGW_CMDVAL_DISABLED - The command is disabled.
//
//  In:	none
//
//  Out:long FAR* plValidate
//
//  Comments:		
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::Validate(long FAR* plValidate)
{
	//PgpGwTrace("CGWCommand::Validate this=%#x, m_nID=%d\n", this, m_nID);	
	HRESULT hrRet=S_OK;
	PMSGINFO lpMsgInfo=NULL;
	BOOL bRet=TRUE;

	//fill up the members and the hash with relevant details that might be missing
	hrRet=LearnCommandContext();
	if(FAILED(hrRet))//if we have any problem determining the context
	{
		*plValidate=eGW_CMDVAL_DISABLED;//dont wait till we are told to execute the command
		goto cleanup;					//so disable the command
	}

	//if the license has expired
	if(TRUE == g_bPlugInExpired)
	{
		*plValidate=eGW_CMDVAL_DISABLED;//we disable the command
		goto cleanup;
	}

	//get the context info
	pgpAssert(0 != m_dwCmdUiInfo);
	pgpAssert(NULL != m_hCtxtWindow);
	lpMsgInfo=g_pccihCmdHash->Get((ULONG)m_hCtxtWindow);
	if(NULL == lpMsgInfo)
	{
		*plValidate=eGW_CMDVAL_DISABLED;//dont wait till we are told to execute the command
		goto cleanup;
	}

	switch(m_nID)
	{
		case IDM_LAUNCH_PGPKEYS://launch pgpkeys
		case IDM_SHOW_PREFS://show the pgp options dialog
			*plValidate=0/*eGW_CMDVAL_ALWAYS*/;//always enabled
			break;

		case IDM_AUTO_ENCRYPT_ONSEND:
			pgpAssert(0 != (m_dwCmdUiInfo&CUIO_MAIL_CONTEXT));
			//lock the hash table			
			g_pccihCmdHash->Lock();

			//if the command is in the context of a discussion/note then disable it
			if(m_dwCmdUiInfo&CUIO_DISCUSSION_CONTEXT)
			{
				pgpAssert(m_dwCmdUiInfo & CUIO_IS_AN_OPTION);//should be an option
				pgpAssert(m_dwCmdUiInfo & CUIO_MAIL_CONTEXT);//discussion context is also a mail context
				
				*plValidate=eGW_CMDVAL_DISABLED;//disable it
			}
			//if the command is in the context of a new mail
			else if(m_dwCmdUiInfo&CUIO_NEWMAIL_CONTEXT)
			{
				pgpAssert(m_dwCmdUiInfo & CUIO_IS_AN_OPTION);
				if((lpMsgInfo->dwState)&MIS_AUTO_ENCRYPT_ON_SEND)
					*plValidate=eGW_CMDVAL_CHECKED;//TODO:dont read msginfo directly-claim the mutex
				else
					*plValidate=0;//just enabled otherwise
			}
			else//ie. if the command is in old mail context
			{
				pgpAssert(0 != (m_dwCmdUiInfo&CUIO_OLDMAIL_CONTEXT));
				*plValidate=eGW_CMDVAL_DISABLED;//disable it
			}
			//unlock the hash table
			g_pccihCmdHash->UnLock();
			break;

		case IDM_AUTO_SIGN_ONSEND:
			pgpAssert(0 != (m_dwCmdUiInfo&CUIO_MAIL_CONTEXT));
			//lock the hash table			
			g_pccihCmdHash->Lock();

			//if the command is in the context of a discussion/note then disable it
			if(m_dwCmdUiInfo&CUIO_DISCUSSION_CONTEXT)
			{
				pgpAssert(m_dwCmdUiInfo & CUIO_IS_AN_OPTION);//should be an option
				pgpAssert(m_dwCmdUiInfo & CUIO_MAIL_CONTEXT);//discussion context is also a mail context
				
				*plValidate=eGW_CMDVAL_DISABLED;//disable it
			}
			//if the command is in the context of a new mail
			else if(m_dwCmdUiInfo&CUIO_NEWMAIL_CONTEXT)
			{
				pgpAssert(m_dwCmdUiInfo & CUIO_IS_AN_OPTION);
				if((lpMsgInfo->dwState)&MIS_AUTO_SIGN_ON_SEND)
					*plValidate=eGW_CMDVAL_CHECKED;
				else
					*plValidate=0;//just enabled otherwise
			}
			else//ie. if the command is in old mail context
			{
				pgpAssert(0 != (m_dwCmdUiInfo&CUIO_OLDMAIL_CONTEXT));
				*plValidate=eGW_CMDVAL_DISABLED;//disable it
			}

			//unlock the hash table
			g_pccihCmdHash->UnLock();

			break;

		case IDM_CONVERT_TO_PLAIN_TEXT:
			pgpAssert(0 != (m_dwCmdUiInfo&CUIO_MAIL_CONTEXT));
			//lock the hash table			
			g_pccihCmdHash->Lock();

			//if the command is in the context of a discussion/note then disable it
			if(m_dwCmdUiInfo&CUIO_DISCUSSION_CONTEXT)
			{
				pgpAssert(m_dwCmdUiInfo & CUIO_IS_AN_OPTION);//should be an option
				pgpAssert(m_dwCmdUiInfo & CUIO_MAIL_CONTEXT);//discussion context is also a mail context
				
				*plValidate=eGW_CMDVAL_DISABLED;//disable it
			}
			//if the command is in the context of a new mail
			else if(m_dwCmdUiInfo&CUIO_NEWMAIL_CONTEXT)
			{
				pgpAssert(m_dwCmdUiInfo & CUIO_IS_AN_OPTION);
				if((lpMsgInfo->dwState)&MIS_CONV_TO_PLAIN_TEXT)
					*plValidate=eGW_CMDVAL_CHECKED;//TODO:dont read msginfo directly-claim the mutex
				else
					*plValidate=0;//just enabled otherwise
			}
			else//ie. if the command is in old mail context
			{
				pgpAssert(0 != (m_dwCmdUiInfo&CUIO_OLDMAIL_CONTEXT));
				*plValidate=eGW_CMDVAL_DISABLED;//disable it
			}

			//unlock the hash table
			g_pccihCmdHash->UnLock();
			break;

		case IDM_DECRYPT_VERIFY://decrypt/verify 
			pgpAssert(m_dwCmdUiInfo & CUIO_IS_AN_OPERATION);//should be an operation
			//if the command is in the context of a discussion/note then disable it
			if(m_dwCmdUiInfo&CUIO_DISCUSSION_CONTEXT)
			{
				pgpAssert(m_dwCmdUiInfo & CUIO_MAIL_CONTEXT);//discussion context is also a mail context
				
				*plValidate=eGW_CMDVAL_DISABLED;//disable it
			}
			else if((m_dwCmdUiInfo&CUIO_OLDMAIL_CONTEXT) || (m_dwCmdUiInfo&CUIO_MAINWND_CONTEXT))
			{
				if(m_dwCmdUiInfo&CUIO_OLDMAIL_CONTEXT)
				{
					//if in the old mail context
					if((NULL != lpMsgInfo->hNewMsgTxtWnd) && //if there is an overlapping window we have created
					   (IsWindowVisible(lpMsgInfo->hNewMsgTxtWnd)))//and it is the one being shown to the user now
						*plValidate=eGW_CMDVAL_DISABLED;
					else
						*plValidate=0;//enabled
				}
				else//must be from the main window 
				{
					pgpAssert(m_dwCmdUiInfo&CUIO_MAINWND_CONTEXT);

					//we disable the decrypt/verify button on main window toolbar
					//remove the following lines to reenable the logic of disabling
					//if multiple messages are selected
					*plValidate=eGW_CMDVAL_DISABLED;

#if 0 
					IDispatch *pDispTemp=NULL;
					IGWClientState *pClientState=NULL;
					IGWMessageList *pMsgs=NULL;
					LONG lMsgCount=0;


					//check if any message is selected or multiple messages are

					//get the client state
					hrRet=m_pIMgr->get_ClientState(&pDispTemp);
					pgpAssert(SUCCEEDED(hrRet) && (NULL != pDispTemp));
					if(FAILED(hrRet) || (NULL == pDispTemp))
						goto decrypt_verify_end;

					hrRet=pDispTemp->QueryInterface(IID_IClientState, (void **)&pClientState);
					pgpAssert(SUCCEEDED(hrRet) && (NULL != pClientState));
					if(FAILED(hrRet) || (NULL == pClientState))
						goto decrypt_verify_end;

					pgpAssert(NULL != pDispTemp);
					pDispTemp->Release();
					pDispTemp=NULL;

					//get the selected message list
					hrRet=pClientState->get_SelectedMessages(&pDispTemp);
					pgpAssert(SUCCEEDED(hrRet) && (NULL != pDispTemp));
					if(FAILED(hrRet) || (NULL == pDispTemp))
						goto decrypt_verify_end;

					hrRet=pDispTemp->QueryInterface(IID_IGWMessageList, (void **)&pMsgs);
					pgpAssert(SUCCEEDED(hrRet) && (NULL != pMsgs));
					if(FAILED(hrRet) || (NULL == pMsgs))
						goto decrypt_verify_end;

					
					//get the count of selected messages
					hrRet=pMsgs->get_Count(&lMsgCount);

					decrypt_verify_end:
					if(NULL != pClientState)
					{
						pClientState->Release();
						pClientState=NULL;
					}

					if(NULL != pDispTemp)
					{
						pDispTemp->Release();
						pDispTemp=NULL;
					}

					if(NULL != pMsgs)
					{
						pMsgs->Release();
						pMsgs=NULL;
					}

					
					if (1 == lMsgCount)//if exactly one msg is selected
						*plValidate=0;
					else//if any error happened or no msgs are selected or multiple
						//messages are selected
						*plValidate=eGW_CMDVAL_DISABLED;
#endif //#if 0
					
				}
			}
			else if(m_dwCmdUiInfo&CUIO_ATTVIEW_CONTEXT)
			{
				*plValidate=0;//enable it always in the attachment window
			}
			else
				*plValidate=eGW_CMDVAL_DISABLED;
			break;

		case IDM_SHOW_ORIGINAL://show original message
			//if the command is in the context of a discussion/note then disable it
			if(m_dwCmdUiInfo&CUIO_DISCUSSION_CONTEXT)
			{
				pgpAssert(m_dwCmdUiInfo & CUIO_MAIL_CONTEXT);//discussion context is also a mail context
				
				*plValidate=eGW_CMDVAL_DISABLED;//disable it
			}
			else if(m_dwCmdUiInfo&CUIO_OLDMAIL_CONTEXT)
			{
				//lock the hash table			
				g_pccihCmdHash->Lock();
				
				pgpAssert(m_dwCmdUiInfo & CUIO_IS_AN_OPERATION);
				if((NULL != lpMsgInfo->hNewMsgTxtWnd) && //if there is an overlapping window we have created
				   (IsWindowVisible(lpMsgInfo->hNewMsgTxtWnd)))//and it is the one being shown to the user now
					*plValidate=0;
				else
					*plValidate=eGW_CMDVAL_DISABLED;
				
				//unlock the hash table
				g_pccihCmdHash->UnLock();
			}
			else
				*plValidate=eGW_CMDVAL_DISABLED;
			break;

		default:
			break;
	}

cleanup:
	return S_OK;//we return success always hoping gw does not reject the 
				//command status we filled in on the basis of error return
}

//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::SetTextOfReadOnlyMail				
//
//  Description:This is called to show the decrypted text of the 
//      message. Since GW wound not allow us to change the message
//		and the control is set read only(violation of which leads
//		to other problems plus save prompt at close) we create a 
//		control identical to GW's and show it on top of it and 
//		resize it and all the other jugglery that is necessary
//
//  In:	pszMailTextToWrite -pointing to ANSI decrypted text
//  In/Out:hPlainTxtMsgWnd  -passed null if we have not already got 
//				an overlapped window. on return it has the orig 
//				window handle. MUST have the original window handle
//				if the caller has called this function once before
//				for the mail message and was passed back a not null
//				pointer. so caller MUST save if fn returns not null
//				handle in this parameter and caller plans to call 
//				this function again in future.
//
//	In/Out:hHtmlTxtMsgWnd - passed null if we dont know any html text
//				window in existence yet, for this message. we only
//				try to dig for this window if plain text window is 
//				found to be hidden.
//
//	In/Out:hOverlappingWnd -similar use as hPlainTxtMsgWnd. it is
//				technically possible that this window is same as
//				the original window.
//
//  Out:hTrackedMsgWnd - the message window we are tracking currently
//				depending on the html/plain text view this could be
//				either the browser control or the richedit control
//
//  Comments:		
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::SetTextOfReadOnlyMail(char *pszMailTextToWrite,
											   HWND &hPlainTxtMsgWnd,
											   HWND &hHtmlTxtMsgWnd,
											   HWND &hOverlappingWnd,
											   HWND &hTrackedMsgWnd)
{
	HRESULT hrRet=S_OK;
	HWND hVisibleMsgWindow=NULL;
	LONG lWndData=0;
	RECT rcCtrlRect={0};
	POINT ptTmpPoint={0};
	BOOL bRet=FALSE;
	DWORD dwStyle=0, dwExStyle=0;
	LPSUBCLASSINFO lpSubClassInfo=NULL;
	CHARFORMAT cfCharFmt;
	ZeroMemory(&cfCharFmt, sizeof(CHARFORMAT));
	cfCharFmt.cbSize=sizeof(CHARFORMAT);

	
	//check parameters
	//pszMailTextToWrite can be NULL when it is a blank message

	//check object state
	pgpAssert(0 < (m_dwCmdUiInfo&CUIO_OLDMAIL_CONTEXT));
	pgpAssert(NULL != m_hCtxtWindow);
	if(NULL == m_hCtxtWindow)//if we dont have the context window
	{
		hrRet=E_FAIL;//we cannot do much yet !!
		goto cleanup;
	}

	//if the message body window was not passed to us
	if(NULL == hPlainTxtMsgWnd)
	{
		//try to get the plain text message window
		PgpGwTrace("trying to get plain text message window\n");
		_VERIFY(TRUE == GetPlainTxtMsgWnd(hPlainTxtMsgWnd));
		pgpAssert(NULL != hPlainTxtMsgWnd);
		pgpAssert((0x2717 == GetDlgCtrlID(hPlainTxtMsgWnd)) || (0x2718 == GetDlgCtrlID(hPlainTxtMsgWnd)));
		PgpGwTrace("got plain text message window\n");
	}

	//if the plain text message window is visible then it is the visible message window
	if((NULL != hPlainTxtMsgWnd) && IsWindowVisible(hPlainTxtMsgWnd))
	{
		PgpGwTrace("setting visible message window to the plain text message window\n");
		hVisibleMsgWindow=hPlainTxtMsgWnd;
	}

	//if the plain text message window is not visible and htmltxt window
	//was not passed to us, let us find out what the html window handle is
	if((!IsWindowVisible(hPlainTxtMsgWnd)) && (NULL == hHtmlTxtMsgWnd))
	{
		PgpGwTrace("trying to get html text message window\n");
		_VERIFY(TRUE == GetHtmlMsgWnd(hHtmlTxtMsgWnd));
		pgpAssert(NULL != hHtmlTxtMsgWnd);
		pgpAssert(IsWindowVisible(hHtmlTxtMsgWnd));//should be visible
		PgpGwTrace("got html text message window\n");

	}

	//if the html window is visible, it is the visible message window
	if((NULL != hHtmlTxtMsgWnd) && IsWindowVisible(hHtmlTxtMsgWnd))
	{
		pgpAssert(!IsWindowVisible(hPlainTxtMsgWnd));//plain txt window should be hidden
		hVisibleMsgWindow=hHtmlTxtMsgWnd;
	}
	
	
	pgpAssert(NULL != hVisibleMsgWindow);//we should have known the visible message window by now

	//get the position of the visible message window

	//get the window rect in screen coords
	bRet=GetWindowRect(hVisibleMsgWindow, &rcCtrlRect);
	pgpAssert(TRUE == bRet);

	//convert to client coords
	ptTmpPoint.x=rcCtrlRect.left;
	ptTmpPoint.y=rcCtrlRect.top;
	bRet=ScreenToClient(m_hCtxtWindow, &ptTmpPoint); 
	pgpAssert(TRUE == bRet);
	rcCtrlRect.left=ptTmpPoint.x;
	rcCtrlRect.top=ptTmpPoint.y;

	ptTmpPoint.x=rcCtrlRect.right;
	ptTmpPoint.y=rcCtrlRect.bottom;
	bRet=ScreenToClient(m_hCtxtWindow, &ptTmpPoint); 
	pgpAssert(TRUE == bRet);
	rcCtrlRect.right=ptTmpPoint.x;
	rcCtrlRect.bottom=ptTmpPoint.y;

	if(NULL == hOverlappingWnd)//if there is no overlapping window already
	{
		//create a control identical to the original plain text control
		//positioned exactly like the currently visible message control

		//get the control styles
		dwStyle=GetWindowLong(hPlainTxtMsgWnd, GWL_STYLE); 
		dwExStyle=GetWindowLong(hPlainTxtMsgWnd, GWL_EXSTYLE);

		//create a control in the same place
		hOverlappingWnd=CreateWindowEx(dwExStyle, "RICHEDIT", NULL, 
			dwStyle, rcCtrlRect.left, rcCtrlRect.top, abs(rcCtrlRect.right-rcCtrlRect.left), 
			abs(rcCtrlRect.bottom - rcCtrlRect.top), m_hCtxtWindow, NULL, 
			g_hInstance, NULL);
		pgpAssert(NULL != hOverlappingWnd);
		if(NULL == hOverlappingWnd)//if we could not create
		{
			//we cheat groupwise by setting it to write allowed, changing text
			//and setting it back to read only
			PgpGwTrace("WARNING! setting text directly to the readonly control\n");
			SendMessage(hPlainTxtMsgWnd, EM_SETOPTIONS, ECOOP_XOR, ECO_READONLY);
			SendMessage(hPlainTxtMsgWnd, WM_SETTEXT, 0, (LPARAM)pszMailTextToWrite);
			SendMessage(hPlainTxtMsgWnd, EM_SETOPTIONS, ECOOP_XOR, ECO_READONLY);
			SendMessage(hPlainTxtMsgWnd, EM_SETMODIFY, FALSE, 0);//does not work !
			hrRet=S_OK;
			goto cleanup;
		}

		//control comes here only if the new window was created

#if(_WIN32_WINNT >= 0x0500)
		dwExStyle|=WS_EX_LAYERED;
		SetWindowLong(hOverlappingWnd, GWL_EXSTYLE, dwExStyle);
		SetLayeredWindowAttributes(hOverlappingWnd, 0, (255*70)/100, LWA_ALPHA);
#endif

		//set the control in the tab order after the visible msg text control
		SetWindowPos(hOverlappingWnd, hVisibleMsgWindow, 0, 0, 0, 0, 
				SWP_NOSIZE|SWP_NOMOVE|SWP_NOACTIVATE);

		//set the font of the control to the font of the original
		SendMessage(hPlainTxtMsgWnd, EM_GETCHARFORMAT, 0, (LPARAM)&cfCharFmt);
		pgpAssert(NULL != cfCharFmt.szFaceName[0]);
		cfCharFmt.dwMask = CFM_FACE|CFM_SIZE|CFM_BOLD|CFM_ITALIC|CFM_UNDERLINE|CFM_COLOR;
		_VERIFY(0 != SendMessage(hOverlappingWnd, EM_SETCHARFORMAT, SCF_ALL, (LPARAM)&cfCharFmt));


		pgpAssert(NULL == lpSubClassInfo);
		lpSubClassInfo=(LPSUBCLASSINFO)calloc(sizeof(SUBCLASSINFO), 1);
		if(NULL != lpSubClassInfo)
		{
			//fill up the struct with the new control window and groupwise's
			//original window proc of the message body richedit control 
			lpSubClassInfo->hWndOther=hPlainTxtMsgWnd;
			lpSubClassInfo->wpOrigProc=(WNDPROC)GetWindowLong(hOverlappingWnd, GWL_WNDPROC);
			pgpAssert(NULL != lpSubClassInfo->wpOrigProc);

			//store the info in our property for the msg text control.
			_VERIFY(0 != SetProp(hOverlappingWnd, MODULENAME, lpSubClassInfo));
			lpSubClassInfo=NULL;

			//subclass now - since the subclass proc can get the details from subclassinfo
			SetWindowLong(hOverlappingWnd, GWL_WNDPROC, (LONG)DecryptVerifyWndProc);
		}
	}
	else
	{
		//the overlapping window is already created. we just want to move it
		//to the rectangle of current visible window.
		bRet=MoveWindow(hOverlappingWnd, rcCtrlRect.left, rcCtrlRect.top, 
			abs(rcCtrlRect.right-rcCtrlRect.left), 
			abs(rcCtrlRect.bottom - rcCtrlRect.top), FALSE);
		pgpAssert(TRUE == bRet);

	}

	//if the visible window has not got our property already
	if(NULL == GetProp(hVisibleMsgWindow, MODULENAME))
	{
		//subclass the visible control so that we can resize and do our mimicry as far
		//as practicable.but before that let us store the info in our window prop
		pgpAssert(NULL == lpSubClassInfo);
		lpSubClassInfo=(LPSUBCLASSINFO)calloc(sizeof(SUBCLASSINFO), 1);
		if(NULL != lpSubClassInfo)
		{
			//fill up the struct with the new control window and groupwise's
			//original window proc of the visible message body control 
			lpSubClassInfo->hWndOther=hOverlappingWnd;
			lpSubClassInfo->wpOrigProc=(WNDPROC)GetWindowLong(hVisibleMsgWindow, GWL_WNDPROC);
			pgpAssert(NULL != lpSubClassInfo->wpOrigProc);

			//store the info in our property for the msg text control. 
			_VERIFY(0 != SetProp(hVisibleMsgWindow, MODULENAME, lpSubClassInfo));
			lpSubClassInfo=NULL;

			//subclass now - since the subclass proc can get the details from subclassinfo
			SetWindowLong(hVisibleMsgWindow, GWL_WNDPROC, (LONG)SubClassedMsgBodyWndProc);
		}
	}

	//NOTE:control comes here only if we created (just now or sometime in the past) a 
	//new window. mark the exit path when we set the text in the original control and 
	//bail out.

	//show and set the text on the new control
	SendMessage(hOverlappingWnd, WM_SETTEXT, 0, (LPARAM)pszMailTextToWrite);
	ShowWindow(hOverlappingWnd, SW_SHOW);//necessary for all those cases when we might have hidden it


	//hide the original control
	pgpAssert(NULL != hVisibleMsgWindow);
	ShowWindow(hVisibleMsgWindow, SW_HIDE);
	hTrackedMsgWnd=hVisibleMsgWindow;

cleanup:
	return hrRet;
}

//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::GetSelectedMessageId				
//
//  Description:This is called to determine the message id of the
//		selected message if any. This is an attempt to determine
//		which message is actually on view in the quick view window.
//
//
//	Out:bstrMessageId - message id as a BSTR
//
//  Comments: This function fails if more than one message is 
//		selected.
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::GetSelectedMessageId(BSTR &bstrMessageId)
{
	HRESULT hrRet=S_OK;
	IDispatch *pDispTemp=NULL;
	DIGWMessage *pGwDispMsg=NULL;
	IGWClientState *pClientState=NULL;
	IGWMessageList *pMsgs=NULL;
	IGWMessage *pSelMsg=NULL;
	LONG lMsgCount=0;
	VARIANT vtTmp={0};

	//if some message id is lying around 
	if(NULL != bstrMessageId)
	{
		SysFreeString(bstrMessageId);//free it
		bstrMessageId=NULL;
	}

	//get the client state
	pgpAssert(NULL != m_pIMgr);
	hrRet=m_pIMgr->get_ClientState(&pDispTemp);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pDispTemp));
	if(FAILED(hrRet) || (NULL == pDispTemp))
	{
		hrRet=E_FAIL;
		goto cleanup;
	}

	hrRet=pDispTemp->QueryInterface(IID_IClientState, (void **)&pClientState);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pClientState));
	if(FAILED(hrRet) || (NULL == pClientState))
	{
		hrRet=E_FAIL;
		goto cleanup;
	}

	pgpAssert(NULL != pDispTemp);
	pDispTemp->Release();
	pDispTemp=NULL;

	//get the selected message list
	hrRet=pClientState->get_SelectedMessages(&pDispTemp);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pDispTemp));
	if(FAILED(hrRet) || (NULL == pDispTemp))
	{
		hrRet=E_FAIL;
		goto cleanup;
	}

	hrRet=pDispTemp->QueryInterface(IID_IGWMessageList, (void **)&pMsgs);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pMsgs));
	if(FAILED(hrRet) || (NULL == pMsgs))
	{
		hrRet=E_FAIL;
		goto cleanup;
	}

	pgpAssert(NULL != pDispTemp);
	pDispTemp->Release();
	pDispTemp=NULL;

	//get the count of selected messages
	hrRet=pMsgs->get_Count(&lMsgCount);
	pgpAssert(SUCCEEDED(hrRet));
	if(FAILED(hrRet) || (1 != lMsgCount))//if failed or multiple selection
	{
		hrRet=E_FAIL;//we dont know which one is selected for the preview pane :(
		goto cleanup;
	}

	//zero in on the only message selected
	vtTmp.vt=VT_I4;
	vtTmp.lVal=1;
	hrRet=pMsgs->Item(vtTmp, &pGwDispMsg);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pGwDispMsg));
	if(FAILED(hrRet) || (NULL == pGwDispMsg))
	{
		hrRet=E_FAIL;
		goto cleanup;
	}

	hrRet=pGwDispMsg->QueryInterface(IID_IGWMessage, (void **)&pSelMsg);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pSelMsg));
	if(FAILED(hrRet) || (NULL == pSelMsg))
	{
		hrRet=E_FAIL;
		goto cleanup;
	}

	//get the message id of that message
	hrRet=pSelMsg->get_MessageID(&bstrMessageId);
	if(FAILED(hrRet) || (NULL == bstrMessageId))
	{
		hrRet=E_FAIL;
		goto cleanup;
	}

	cleanup:
	if(NULL != pClientState)
	{
		pClientState->Release();
		pClientState=NULL;
	}
	if(NULL != pDispTemp)
	{
		pDispTemp->Release();
		pDispTemp=NULL;
	}
	if(NULL != pGwDispMsg)
	{
		pGwDispMsg->Release();
		pGwDispMsg=NULL;
	}
	if(NULL != pMsgs)
	{
		pMsgs->Release();
		pMsgs=NULL;
	}
	if(NULL != pSelMsg)
	{
		pSelMsg->Release();
		pSelMsg=NULL;
	}
	return hrRet;	
}


#if(WINVER < 0x0500)

//copied from winuser.h. quick fix for compilation failures

//Menubar information
typedef struct tagMENUBARINFO
{
    DWORD cbSize;
    RECT  rcBar;          // rect of bar, popup, item
    HMENU hMenu;          // real menu handle of bar, popup
    HWND  hwndMenu;       // hwnd of item submenu if one
    BOOL  fBarFocused:1;  // bar, popup has the focus
    BOOL  fFocused:1;     // item has the focus
} MENUBARINFO, *PMENUBARINFO, *LPMENUBARINFO;

extern "C"
BOOL
WINAPI
GetMenuBarInfo(
    IN HWND hwnd,
    IN LONG idObject,
    IN LONG idItem,
    OUT PMENUBARINFO pmbi
);

#define     OBJID_MENU          0xFFFFFFFD

#endif // WINVER < 0x0500

//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::LearnCommandContext				
//
//  Description:This is called to determine and learn the context
//		of this command object. This is perhaps the most important
//		part of the command object and is the source and consumer
//		of the command hash maintained by us for getting details
//		such as command state when needed.
//
//  Comments: 
//
///////////////////////////////////////////////////////////////////
STDMETHODIMP CGWCommand::LearnCommandContext()
{
	HRESULT hrRet=S_OK;
	PMSGINFO lpMsgInfo=NULL;
	SUBCLASSINFO *lpSubClassInfo=NULL;
	MENUBARINFO mbiInfo={sizeof(MENUBARINFO), 0};
	BOOL bFreeMsgInfo=FALSE, bLocked=FALSE, bRet=TRUE;

	//if we dont know the context window(which is the key to the hash)
	if(NULL == m_hCtxtWindow)
	{
		
		if(m_dwCmdUiInfo&CUIO_MAIL_CONTEXT)
			m_hCtxtWindow=GetWindow(GWO_CURRENTMAIL_WINDOW);
		else if(m_dwCmdUiInfo&CUIO_MAINWND_CONTEXT)
			m_hCtxtWindow=GetWindow(GWO_CURRENTTOPLEVEL_WINDOW);
		else if(m_dwCmdUiInfo&CUIO_ATTVIEW_CONTEXT)
			m_hCtxtWindow=GetWindow(GWO_CURRENTATTVIEWER_WINDOW);
		else//if we dont know the window context yet we cannot do much
			goto cleanup;

		pgpAssert(NULL != m_hCtxtWindow);

#ifdef _DEBUG
		//verify the class names just to make sure we got the right window
		{
			char szClassName[255]={0};
			//get the window class name
			int iRet=GetClassName(m_hCtxtWindow, szClassName, 
							sizeof(szClassName)-sizeof(char));
			pgpAssert(iRet > 0);
			pgpAssert((m_dwCmdUiInfo&CUIO_MAINWND_CONTEXT)?
					(0 == strcmp(szClassName, MAINWINDOWCLASSNAMEA)):TRUE);//main window class name
			pgpAssert((m_dwCmdUiInfo&CUIO_MAIL_CONTEXT)?
					(0 == strcmp(szClassName, MAILVIEWWINDOWCLASSNAMEA)):TRUE);//mail window class name
			pgpAssert((m_dwCmdUiInfo&CUIO_ATTVIEW_CONTEXT)?
					(0 == strcmp(szClassName, ATTACHMENTVIEWERWINDOWCLASSNAMEA)):TRUE);//attachment window class name
		}
#endif
	}

	//if for some reason we could not get the context window by now
	if(NULL == m_hCtxtWindow)
		goto cleanup;//bail out

	//query the hash for other info
	lpMsgInfo=g_pccihCmdHash->Get((ULONG)m_hCtxtWindow);
	if(NULL == lpMsgInfo)//if we did not get details from the hash
	{
		//allocate one chunk of msg info
		lpMsgInfo = (PMSGINFO)calloc(sizeof(MSGINFO), 1);
		pgpAssert(NULL != lpMsgInfo);
		if(NULL == lpMsgInfo)
		{
			hrRet=E_OUTOFMEMORY;
			goto cleanup;
		}
		
		//fill the key(=context window handle)
		if(0 == lpMsgInfo->ulKey)
			lpMsgInfo->ulKey=(ULONG)m_hCtxtWindow;

		//remember the menu bar handle
		//bRet=GetMenuBarInfo(m_hCtxtWindow, OBJID_MENU, 0, &mbiInfo);
		//pgpAssert(TRUE == bRet);
		//if(TRUE == bRet)
		//	lpMsgInfo->hMenuBar=mbiInfo.hMenu;
		lpMsgInfo->hMenuBar=GetMenu(m_hCtxtWindow);
		pgpAssert(NULL != lpMsgInfo->hMenuBar);

		//initialize the state to be unknown since we are yet to learn state
		lpMsgInfo->dwState = MIS_OPTION_UNKNOWN;

		//insert the newly allocated info structure
		bRet=g_pccihCmdHash->Insert(lpMsgInfo);
		pgpAssert(TRUE == bRet);
		if(FALSE == bRet)
		{
			hrRet=E_FAIL;
			bFreeMsgInfo=TRUE;
			goto cleanup;
		}
	}

	g_pccihCmdHash->Lock();
	bLocked=TRUE;
	

	//if we are in the main window context
	if((m_dwCmdUiInfo&CUIO_MAINWND_CONTEXT) && (NULL == lpMsgInfo->bstrId))
	{
		//check the current selection of messages
		GetSelectedMessageId(lpMsgInfo->bstrId);
	}

	//if we are in some message window context and we dont know the
	//message id of it yet...
	if((NULL == lpMsgInfo->bstrId) && 
	  ((m_dwCmdUiInfo&CUIO_MAIL_CONTEXT) || (m_dwCmdUiInfo&CUIO_ATTVIEW_CONTEXT)))
	{
		//get the message id involved
		hrRet=Publish("ItemMessageIDFromView()", &(lpMsgInfo->bstrId));
		pgpAssert(SUCCEEDED(hrRet) && (NULL != lpMsgInfo->bstrId));
		if(FAILED(hrRet) || (NULL == lpMsgInfo->bstrId))
		{
			hrRet=E_FAIL;
			goto cleanup;
		}
	}

	//if we dunno if we are in old mail context or new
	if((0 < (m_dwCmdUiInfo&CUIO_MAIL_CONTEXT)) &&
	   (0 == (m_dwCmdUiInfo&CUIO_NEWMAIL_CONTEXT)) &&
	   (0 == (m_dwCmdUiInfo&CUIO_OLDMAIL_CONTEXT)))
	{
		//this code should never execute
		pgpAssert(FALSE);//we failed to learn the new mail/old mail context till now !
		
		if(NULL != lpMsgInfo->bstrId)
		{
			//check if the 
			if(0 == wcscmp(lpMsgInfo->bstrId, NEWMESSAGEMSGIDW))
				m_dwCmdUiInfo |= CUIO_NEWMAIL_CONTEXT;
			else
			{
				//NOTE:the id is not same as "X00" - this does not necessarily mean
				//that this is an old mail - as is the case with draft mails opened
				//we do this here nevertheless just as a backup for other smarter
				//methods of determining this elsewhere in the plugin.
				m_dwCmdUiInfo |= CUIO_OLDMAIL_CONTEXT;
			}
		}
	}

	//if the state flags of this command are yet to be learnt
	if(((m_dwCmdUiInfo&CUIO_NEWMAIL_CONTEXT) || 
	    (m_dwCmdUiInfo&CUIO_OLDMAIL_CONTEXT)) &&
		(lpMsgInfo->dwState&MIS_OPTION_UNKNOWN))
	{
		BOOL bRet=TRUE;
		PGPBoolean pgpbFlag=FALSE;

		//if we are in the new mail context set some of the preferences
		//as is chosen currently in pgp options dialog
		if(m_dwCmdUiInfo&CUIO_NEWMAIL_CONTEXT)
		{
			//unset the unknown state bit
			lpMsgInfo->dwState &= ~MIS_OPTION_UNKNOWN;
			pgpAssert(0 == lpMsgInfo->dwState);//no other mask should be set

			//check if the prefs are set to encrypt on send. new mails in that case
			//will have that option set by default
			bRet=PgpGwGetBoolPref(kPGPPrefMailEncryptDefault, pgpbFlag);
			pgpAssert(TRUE == bRet);
			if((TRUE == bRet) && (TRUE == pgpbFlag))
				lpMsgInfo->dwState |= MIS_AUTO_ENCRYPT_ON_SEND;

			//check if the prefs are set to sign automatically on send. new mails 
			//in that case will have that option set by default
			bRet=PgpGwGetBoolPref(kPGPPrefMailSignDefault, pgpbFlag);
			pgpAssert(TRUE == bRet);
			if((TRUE == bRet) && (TRUE == pgpbFlag))
				lpMsgInfo->dwState |= MIS_AUTO_SIGN_ON_SEND;

			//check if spell checker is on.

		}
		else if(m_dwCmdUiInfo&CUIO_OLDMAIL_CONTEXT)
		{
			//unset the unknown state bit
			lpMsgInfo->dwState &= ~MIS_OPTION_UNKNOWN;
			pgpAssert(0 == lpMsgInfo->dwState); //no other mask should be set

			//check if the prefs are set to decrypt/verify automatically on open 
			bRet=PgpGwGetBoolPref(kPGPPrefAutoDecrypt, pgpbFlag);
			pgpAssert(TRUE == bRet);
			if((TRUE == pgpbFlag) && (TRUE == pgpbFlag))
				lpMsgInfo->dwState |= MIS_AUTO_DECRYPT_ON_OPEN;
		}

		//if we are in a discussion context
		if(m_dwCmdUiInfo&CUIO_DISCUSSION_CONTEXT)
		{
			//unset the unknown state bit
			lpMsgInfo->dwState &= ~MIS_OPTION_UNKNOWN;
			lpMsgInfo->dwState |= MIS_SKIP_OPERATION;//we skip all operations on this item
		}
	}

	cleanup:
	if((TRUE == bFreeMsgInfo) && (NULL != lpMsgInfo))
	{
		//there should be no reference to this in the hash
		pgpAssert(NULL == g_pccihCmdHash->Get(lpMsgInfo->ulKey));

		if(NULL != lpMsgInfo->bstrId)
		{
			SysFreeString(lpMsgInfo->bstrId);
			lpMsgInfo->bstrId=NULL;
		}
		free(lpMsgInfo);
		lpMsgInfo=NULL;
	}
	
	if(TRUE == bLocked)
		g_pccihCmdHash->UnLock();
	return hrRet;
}


BOOL CGWCommand::EnableCommands(HMENU hMenuBar, BOOL bEnable, DWORD dwFlags)
{
	BOOL bMoreItemsExist=TRUE, bRet=TRUE;
	MENUITEMINFO miiInfo={sizeof(MENUITEMINFO), 0};
	miiInfo.fMask=MIIM_TYPE|MIIM_DATA;
	char *pszMenuString=NULL;
	char *pszMainMenuString=NULL;
	DWORD dwMenuStringLen=0;
	int iMenuIndex=0;
	HWND hWndReBar=NULL, hWndTBar=NULL;
	LPSUBCLASSINFO lpSubClassInfo=NULL;

	//get our menu string
	pgpAssert(NULL == pszMainMenuString);
	bRet=SafeLoadString(IDS_GWPLUGIN_MAIN_MENU_NAME, &pszMainMenuString);
	if((FALSE == bRet) || (NULL == pszMainMenuString))
	{
		bRet=FALSE;
		goto cleanup;
	}

	//for all the top level menu items in the menu bar
	for(iMenuIndex=0; TRUE == bMoreItemsExist; iMenuIndex++)
	{
		//get the length of string for the menu involved
		miiInfo.cch=0;
		miiInfo.dwTypeData=NULL;
		bMoreItemsExist=GetMenuItemInfo(hMenuBar, iMenuIndex, TRUE, &miiInfo);
		if(FALSE == bMoreItemsExist)//we assume all failures are because of the item being not there
			break;//we are done with all menu items

		//if the length required is less than is allocated
		pgpAssert(TRUE == bMoreItemsExist);
		if(dwMenuStringLen < (miiInfo.cch+1))
		{
			//allocate the required amount of buffer
			char *pszTempPtr=(char *)realloc(pszMenuString, miiInfo.cch+1);
			if(NULL == pszTempPtr)
			{
				pgpAssert(FALSE);
				bRet=FALSE;
				goto cleanup;
			}

			//switch to the new buffer and reset contents
			pszMenuString=pszTempPtr;
			dwMenuStringLen=miiInfo.cch+1;
			ZeroMemory(pszMenuString, dwMenuStringLen);
		}


		//get the menu item text
		pgpAssert(NULL != pszMenuString);
		pgpAssert(!IsBadWritePtr(pszMenuString, dwMenuStringLen));
		miiInfo.dwTypeData=pszMenuString;
		miiInfo.cch=dwMenuStringLen;
		bRet=GetMenuItemInfo(hMenuBar, iMenuIndex, TRUE, &miiInfo);
		pgpAssert(TRUE == bRet);
		if(FALSE == bRet)
			goto cleanup;//certainly some error condition

		//if it is not our menu item
		pgpAssert(NULL != pszMainMenuString);
		if(0 != strcmpi(pszMenuString, pszMainMenuString))
		{
			bRet=EnableMenuItem(hMenuBar, iMenuIndex, 
					MF_BYPOSITION|(bEnable?MF_ENABLED:MF_GRAYED));
			pgpAssert(-1 != bRet);//if this fails it is difficult to handle error condition
								//because we need to undo some of the things we have done already !!

			//the previous menu state should be the opposite of what we just did
			pgpAssert(bEnable?(MF_GRAYED == (MF_GRAYED&bRet)):(MF_ENABLED == (MF_ENABLED&bRet)));
		}

		//get rid of the string (lets stay clean)
		ZeroMemory(pszMenuString, dwMenuStringLen);
	}
	
	//control comes here only if we have succeeded in all changes till now
	bRet=DrawMenuBar(m_hCtxtWindow);//redraw menubar
	pgpAssert(TRUE == bRet);

	//get the rebar window
	hWndReBar=GetWindow(GWO_CURRENTREBAR_WINDOW);
	pgpAssert(NULL != hWndReBar);
	if(NULL == hWndReBar)
	{
		bRet=FALSE;
		goto cleanup;
	}

	//for all the toolbars under the rebar
	while(NULL != (hWndTBar=FindWindowEx(hWndReBar, hWndTBar, TOOLBARCLASSNAME, NULL)))
	{
		//if the toolbar is not visible go over to the next toolbar
		if(!IsWindowVisible(hWndTBar))
		{
			PgpGwTrace("toolbar (%#x) is invisible. skipping...\n", hWndTBar);
			continue;
		}
		
		//subclass the toolbar so that we can do all our stuff of disabling/enabling
		//and other odd and sundry things that GroupFool forces us to

		//first check if we have already got our property stored with it
		lpSubClassInfo=(LPSUBCLASSINFO)GetProp(hWndTBar, MODULENAME);
		if(NULL == lpSubClassInfo)//if it has not been subclassed
		{
			pgpAssert(NULL == lpSubClassInfo);
			lpSubClassInfo=(LPSUBCLASSINFO)calloc(sizeof(SUBCLASSINFO), 1);
			if(NULL != lpSubClassInfo)
			{
				//fill up the struct with the original window proc of the toolbar control 
				lpSubClassInfo->wpOrigProc=(WNDPROC)GetWindowLong(hWndTBar, GWL_WNDPROC);
				pgpAssert(NULL != lpSubClassInfo->wpOrigProc);

				//store the info in our property for the toolbar.
				_VERIFY(0 != SetProp(hWndTBar, MODULENAME, (HANDLE)lpSubClassInfo));
				lpSubClassInfo=NULL;

				//subclass now - since the subclass proc can get the details from subclassinfo
				SetWindowLong(hWndTBar, GWL_WNDPROC, (LONG)SubClassedToolBarWndProc);
			}
		}

		SendMessage(hWndTBar, WM_ENABLECOMMANDS, bEnable, 0L);
	}
	

	cleanup:
	if(NULL != pszMenuString)
	{
		free(pszMenuString);
		pszMenuString=NULL;
		dwMenuStringLen=0;
	}

	if(NULL != pszMainMenuString)
	{
		free(pszMainMenuString);
		pszMainMenuString=NULL;
	}

	return bRet;
}


VOID WINAPI GWEventNotify (DWORD dwID, DWORD dwEvent)
{	
	switch (dwEvent)
	{
		case EVT_GROUPWISE_CANCEL_ITEM:
		case EVT_GROUPWISE_CLOSE_ITEM:
		case EVT_GROUPWISE_SEND_ITEM_COMPLETED:
			//return ODMA document to the DMS
			//ODMCloseDoc(g_hODMA, (LPSTR)dwID, 0, 0, NULL, 0);
			//delete memory used to store the DocId
			//delete (LPSTR) dwID;
			break;		
	}
}



LRESULT CALLBACK DecryptVerifyWndProc(HWND hWnd, UINT uiMessageId, WPARAM wParam, LPARAM lParam)
{
	static HWND hCurWnd=NULL;
	static LPSUBCLASSINFO lscInfo=NULL;
	static HACCEL hAccTable=NULL;
	LONG lRet=0;

	if(hWnd != hCurWnd)
	{
		hCurWnd=hWnd;
		pgpAssert(NULL != hCurWnd);

		lscInfo=(LPSUBCLASSINFO)GetProp(hWnd, MODULENAME);
		pgpAssert(NULL != lscInfo);
		pgpAssert(!IsBadWritePtr(lscInfo, sizeof(SUBCLASSINFO)));
	}

	pgpAssert((NULL != lscInfo) && (NULL != lscInfo->wpOrigProc));
	if(!((NULL != lscInfo) && (NULL != lscInfo->wpOrigProc)))
	{
		PgpGwTrace("Warning: window message(%#x) not processed by decrypt/verify window !!\n", uiMessageId);	
		return 0L;
	}

	switch(uiMessageId)
	{
		case WM_CONTEXTMENU:
			{
				pgpAssert(hWnd == (HWND)wParam);
				
				POINT posClient={0};
				POINT pos={GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam)};
				
				LONG lRet=0;
				BOOL bRet=TRUE;
				HMENU hCtxtMenu=NULL;
				char szTmpBuffer[255]={0};

				//check if there is any selected text in the window
				lRet=SendMessage(hWnd, EM_SELECTIONTYPE, 0, 0);
				if(SEL_EMPTY == lRet)//no selection so no context menu
					goto contextmenu_cleanup;

				//if the context menu is generated from SHIFT+F10 or VK_APPS then get 
				//the position of cursor in the area and show there 
				if((-1 == pos.x) && (-1 == pos.y))
					GetCursorPos(&pos);

				//convert the screen coords to client coordinate system
				posClient=pos;
				ScreenToClient(hWnd, &posClient);

				//now positions should be something positive
				pgpAssert(pos.x >= 0 && pos.y >= 0);
				pgpAssert(posClient.x >= 0 && posClient.y >= 0);
				
				hCtxtMenu=CreatePopupMenu();
				pgpAssert(NULL != hCtxtMenu);
				if(NULL == hCtxtMenu)
					goto contextmenu_cleanup;

				//get the command string from resource
				lRet=LoadString(g_hInstance, IDS_COMMAND_COPY, szTmpBuffer, sizeof(szTmpBuffer));
				pgpAssert(lRet > 0);//string is not found or some error condition
				pgpAssert(lRet != sizeof(szTmpBuffer)-sizeof(char));//potential truncation otherwise

				//insert a copy command into the context menu
				bRet=AppendMenu(hCtxtMenu, MF_ENABLED|MF_STRING, IDM_COPY, szTmpBuffer);
				pgpAssert(TRUE == bRet);
				if(FALSE == bRet)
					goto contextmenu_cleanup;

				//show the context menu
				bRet=TrackPopupMenu(hCtxtMenu, TPM_LEFTALIGN|TPM_RIGHTBUTTON|TPM_RETURNCMD, 
							pos.x, pos.y, 0, hWnd, NULL);
				if(0 == bRet)//some error or user cancelled
					goto contextmenu_cleanup;

				//if copy command was given to us
				if(IDM_COPY == bRet)
					SendMessage(hWnd, WM_COPY, 0, 0);//copy selection to clipboard

				contextmenu_cleanup:
				if(NULL != hCtxtMenu)
				{
					DestroyMenu(hCtxtMenu);
					hCtxtMenu=NULL;
				}
			}
			break;

		case WM_KEYDOWN:
			{
				pgpAssert(2 == sizeof(SHORT));
				
				PgpGwTrace("wparam=%#x lparam=%#x\n", wParam, lParam);

				//if it is the character c/C 
				if((wParam == 'c') || (wParam == 'C'))
				{
					SHORT shState=GetKeyState(VK_CONTROL);
					PgpGwTrace("state=%#x\n", shState);
					if(shState&0x8000)//while control key is down
						PostMessage(hWnd, WM_COPY, 0, 0);
				}

				goto call_default;
			}
			break;

		case WM_COMMAND:
			{
				goto call_default;
			}
			break;

		case WM_DESTROY:
			{
				//let it be processed by the orig wnd proc
				lRet=CallWindowProc(lscInfo->wpOrigProc, hWnd, 
						uiMessageId, wParam, lParam);//let the orig wndproc handle it

				//we want this to be the last message processed by us.
				//so we subclass again and bail out
				pgpAssert(NULL != lscInfo->wpOrigProc);
				_VERIFY((LONG)DecryptVerifyWndProc == 
					SetWindowLong(hWnd, GWL_WNDPROC, (LONG)lscInfo->wpOrigProc));
				
				_VERIFY(lscInfo == RemoveProp(hWnd, MODULENAME));

				//free the subclass info we had put in our property for the window
				free(lscInfo);
				lscInfo=NULL;

				//destroy the accelerator table
				DestroyAcceleratorTable(hAccTable);
			}

			break;

		default:
		call_default:
			return CallWindowProc(lscInfo->wpOrigProc, hWnd, 
				uiMessageId, wParam, lParam);//let the orig wndproc handle it
			break;
		
	}

	return lRet;
}

LRESULT CALLBACK SubClassedMsgBodyWndProc(HWND hWnd, UINT uiMessageId, WPARAM wParam, LPARAM lParam)
{
	static HWND hCurWnd=NULL;
	static LPSUBCLASSINFO lscInfo=NULL;
	LONG lRet=0;

	if(hWnd != hCurWnd)
	{
		hCurWnd=hWnd;
		pgpAssert(NULL != hCurWnd);

		lscInfo=(LPSUBCLASSINFO)GetProp(hWnd, MODULENAME);
		pgpAssert(NULL != lscInfo);
		pgpAssert(!IsBadWritePtr(lscInfo, sizeof(SUBCLASSINFO)));
	}

	pgpAssert((NULL != lscInfo) && (NULL != lscInfo->wpOrigProc));
	if(!((NULL != lscInfo) && (NULL != lscInfo->wpOrigProc)))
	{
		PgpGwTrace("Warning: window message(%#x) not processed by message text window !!\n", uiMessageId);	
		return 0L;
	}

	switch(uiMessageId)
	{
		case WM_SIZE://size of this control has changed
			{
				//get the screen coordinates of the control
				RECT rcCtrlRect={0};
				POINT ptTmpPoint={0};
				_VERIFY(GetWindowRect(hWnd, &rcCtrlRect));

				//convert the top left corner to client coords
				ptTmpPoint.x=rcCtrlRect.left;
				ptTmpPoint.y=rcCtrlRect.top;
				_VERIFY(ScreenToClient(GetParent(hWnd), &ptTmpPoint)); 

				//move the other control to the same position as this
				pgpAssert(NULL != lscInfo->hWndOther);
				_VERIFY(MoveWindow(lscInfo->hWndOther, ptTmpPoint.x, ptTmpPoint.y, 
						abs(rcCtrlRect.right-rcCtrlRect.left), 
						abs(rcCtrlRect.bottom-rcCtrlRect.top), TRUE));

				pgpAssert(NULL != lscInfo->wpOrigProc);
				return CallWindowProc(lscInfo->wpOrigProc, hWnd, 
					uiMessageId, wParam, lParam);//let the orig wndproc handle it
			}
			break;

		case WM_DESTROY:
			{
				//let it be processed by the orig wnd proc
				lRet=CallWindowProc(lscInfo->wpOrigProc, hWnd, 
						uiMessageId, wParam, lParam);//let the orig wndproc handle it

				//we want this to be the last message processed by us.
				//so we subclass again and bail out
				pgpAssert(NULL != lscInfo->wpOrigProc);
				_VERIFY((LONG)SubClassedMsgBodyWndProc == 
					SetWindowLong(hWnd, GWL_WNDPROC, (LONG)lscInfo->wpOrigProc));
				
				_VERIFY(lscInfo == RemoveProp(hWnd, MODULENAME));

				//free the subclass info we had put in our property for the window
				free(lscInfo);
				lscInfo=NULL;
			}

			break;

		default:
			return CallWindowProc(lscInfo->wpOrigProc, hWnd, 
				uiMessageId, wParam, lParam);//let the orig wndproc handle it
			break;
		
	}

	return lRet;
}

LRESULT CALLBACK SubClassedToolBarWndProc(HWND hTBarWnd, UINT uiMessageId, WPARAM wParam, LPARAM lParam)
{
	static HWND hCurWnd=NULL;//cached window handle for context change
	static LPSUBCLASSINFO lscInfo=NULL;//cached subclassinfo
	LONG lRet=0;

	//if this is the very first time we are called or we have just been
	//called for a different window handle than before
	if((NULL == hCurWnd) || (hCurWnd != hTBarWnd))
	{
		hCurWnd=hTBarWnd;//update the cur window handle cache

		//we need to update the subclass info
		lscInfo=(LPSUBCLASSINFO)GetProp(hCurWnd, MODULENAME);
		pgpAssert(NULL != lscInfo);
		pgpAssert(!IsBadWritePtr(lscInfo, sizeof(SUBCLASSINFO)));
	}

	pgpAssert((NULL != lscInfo) && (NULL != lscInfo->wpOrigProc));
	if(!((NULL != lscInfo) && (NULL != lscInfo->wpOrigProc)))
	{
		PgpGwTrace("Warning: window message(%#x) not processed by message text window !!\n", uiMessageId);	
		return 0L;
	}

	switch(uiMessageId)
	{
		case WM_DESTROY:
			{
				//let it be processed by the orig wnd proc
				lRet=CallWindowProc(lscInfo->wpOrigProc, hTBarWnd, 
						uiMessageId, wParam, lParam);//let the orig wndproc handle it

				//we want this to be the last message processed by us.
				//so we subclass again and bail out
				pgpAssert(NULL != lscInfo->wpOrigProc);
				_VERIFY((LONG)SubClassedToolBarWndProc == 
					SetWindowLong(hTBarWnd, GWL_WNDPROC, (LONG)lscInfo->wpOrigProc));
				
				_VERIFY(lscInfo == RemoveProp(hTBarWnd, MODULENAME));

				//free the subclass info we had put in our property for the window
				free(lscInfo);
				lscInfo=NULL;
			}
			break;

		case WM_ENABLECOMMANDS:
			{
				PgpGwTrace("WM_ENABLECOMMANDS wParam=%#x\n", wParam);
				ULONG ulBtnCount=0, ulBtnIndex=0;
				TBBUTTON tbbInfo={0};
				BOOL bRet=TRUE;
				ULONG ulStateMask=1;

				//get the count of buttons
				ulBtnCount=SendMessage(hTBarWnd, TB_BUTTONCOUNT, 0, 0);
				if(ulBtnCount <= 0)//if no buttons or error
					return 1L;//we are done
				
				pgpAssert(ulBtnCount <= sizeof(lscInfo->ulWndItemsState)*8);//number of buttons can only be as big as our state mask
																		  //if this fails a structure or a longer data type might be required							
				if(TRUE == wParam)//if we are enabling buttons
				{
					//get the property data on the toolbar. we save the button states of the toolbar
					//in the property long value as follows. if a bit is set in the long value, the 
					//state of that button should be toggled
					ULONG ulBtnStateFlags=lscInfo->ulWndItemsState;
					
					//if we are disabling the userdata should be blank unless GW uses it
					//or pgpgw has not cleaned up properly
					if(0 == ulBtnStateFlags)//nothing to toggle
						return 1L;//we are done

					//stop ignoring enable requests
					lscInfo->ulFlags=0;

					for(ulBtnIndex=0,ulStateMask=1; ulBtnIndex < ulBtnCount; ulStateMask <<= 1, ulBtnIndex++)
					{
						//if the bit is set at the index
						if(ulBtnStateFlags & ulStateMask)
						{
							ZeroMemory(&tbbInfo, sizeof(TBBUTTON));

							bRet=SendMessage(hTBarWnd, TB_GETBUTTON, ulBtnIndex, (LPARAM)&tbbInfo);
							pgpAssert(TRUE == bRet);//success
							pgpAssert(0 != tbbInfo.idCommand);
							pgpAssert(0 == (tbbInfo.fsState&TBSTATE_ENABLED));//should be disabled since we are enabling it.
							pgpAssert(TRUE == IsStdGwCmd(tbbInfo.idCommand));//should not be a pgpgw command

							PgpGwTrace("Enabling toolbar button(index = %d, cmdid= %d\n", ulBtnIndex, tbbInfo.idCommand);
							bRet=SendMessage(hTBarWnd, TB_ENABLEBUTTON, tbbInfo.idCommand, MAKELONG(TRUE, 0));
							pgpAssert(TRUE == bRet);
						}
					}

					//reset the item state array
					lscInfo->ulWndItemsState=0;
				}
				else //if we are disabling stuff
				{
					DWORD ulBtnStateFlags=0;

					for(ulBtnIndex=0,ulStateMask=1; ulBtnIndex < ulBtnCount; ulStateMask <<= 1, ulBtnIndex++)
					{
						ZeroMemory(&tbbInfo, sizeof(TBBUTTON));

						bRet=SendMessage(hTBarWnd, TB_GETBUTTON, ulBtnIndex, (LPARAM)&tbbInfo);
						pgpAssert(TRUE == bRet);//success

						//if it is a separator we dont do anything
						if(0 == tbbInfo.idCommand)
							continue;

						//if the button is not enabled
						if(TBSTATE_ENABLED != (tbbInfo.fsState&TBSTATE_ENABLED))
							continue; //we dont do anything

						//if the command is a pgp command
						if(FALSE == IsStdGwCmd(tbbInfo.idCommand))
							continue; //we dont do anything

						//disable the button
						PgpGwTrace("Disabling toolbar button(index = %d, cmdid= %d\n", ulBtnIndex, tbbInfo.idCommand);
						bRet=SendMessage(hTBarWnd, TB_ENABLEBUTTON, tbbInfo.idCommand, MAKELONG(FALSE, 0));
						pgpAssert(TRUE == bRet);

						//remember to toggle it back later
						ulBtnStateFlags |= ulStateMask;

					}

					//remember the new state
					lscInfo->ulWndItemsState=ulBtnStateFlags;
					lscInfo->ulFlags=SCF_IGNOREENABLE;//ignore all enabling/disabling requests after this
													  //unless it is one of the pgpgw commands
				}

			}
			break;

		case TB_ENABLEBUTTON:
			//if we are not supposed to be ignoring enable/disable requests
			//do the default processing
			if(0 == (lscInfo->ulFlags&SCF_IGNOREENABLE))
				goto defaultproc;

			//check if it is a request for one of our commands. in which
			//case we basically say amen(=no faking whatsoever)
			if(FALSE == IsStdGwCmd(wParam))
				goto defaultproc;

			PgpGwTrace("Ignoring enable/disable request for cmdid= %d\n", wParam);
			return 1L;//for all other cases fake success -forget you GroupFool
			break;

		defaultproc:
		default:
			return CallWindowProc(lscInfo->wpOrigProc, hTBarWnd, 
				uiMessageId, wParam, lParam);//let the orig wndproc handle it
			break;
		
	}

	return lRet;
}

//////////////////////////////////////////////////////////////////
//	Name:CGWCommand::IsStdGwCmd			
//
//  Description:we do not store the toolbar id's assigned to us. 
//		apparently the standard items are assigned id's that are same 
//		as the token id's that correspond to their functionality. so 
//		in order to determine if it is a standard commandwe check if 
//		it lies in the ranges of tokens defined for us.
//  
//	In:	unsigned int - the toolbar item id (not same as command id) 
//  Out:none
//  
//  Comments:		
///////////////////////////////////////////////////////////////////
BOOL IsStdGwCmd(UINT uiTBarItemId)
{
	//UINT uiTBarCmdArray[]={IDM_DECRYPT_VERIFY, 
	//				   IDM_LAUNCH_PGPKEYS, 
	//				   IDM_AUTO_ENCRYPT_ONSEND, 
	//				   IDM_AUTO_SIGN_ONSEND};


	//if it is a lower than expected id
	if((uiTBarItemId < BASE_TOKEN) || (BFTKN_INVALID_TOKEN == uiTBarItemId))
		return TRUE;//perhaps it is a standard tbar item or a separator

	//check if it is a dialog token
	if((uiTBarItemId >= BASE_DIALOG_TOKEN) && (uiTBarItemId <= TOP_DIALOG_TOKEN))
		return TRUE;

	//check if it is a base function token
	if((uiTBarItemId >= BASE_BFUNC_TOKEN) && (uiTBarItemId <= TOP_BFUNC_TOKEN))
		return TRUE;
	if((uiTBarItemId >= BASE_FUNCTION_TOKEN2) && (uiTBarItemId <= TOP_FUNCTION_TOKEN2))
		return TRUE;

	//check if it is a ole token
	if((uiTBarItemId >= BASE_OLE_TOKEN) && (uiTBarItemId <= TOP_OLE_TOKEN))
		return TRUE;

	//check if it is a advanced function token
	if((uiTBarItemId >= BASE_AFUNC_TOKEN) && (uiTBarItemId <= TOP_AFUNC_TOKEN))
		return TRUE;

	return FALSE;
}


BOOL CGWCommand::GetPlainTxtMsgWnd(HWND& hPlainTxtMsgWnd)
{
	//the text window is the one richedit control that is the lowest
	//among all richedit controls. the control id's change so this
	//was necessary. change this if this logic breaks !
	HWND hWndChild=NULL, hLowestRichEdit=NULL;
	HWND hTallestControl=NULL;
	RECT rectWndOne={0}, rectWndTwo={0};
	while(NULL != (hWndChild=FindWindowEx(m_hCtxtWindow, 
		hWndChild, "RICHEDIT", NULL)))
	{
		//get the client rectangle of the rich edit control
		_VERIFY(TRUE == GetWindowRect(hWndChild, &rectWndTwo));
		
		if((IsRectEmpty(&rectWndOne)) || //if this is the first time we got a window rect
		   (rectWndTwo.top > rectWndOne.top))//or if this window is below the other window rect
		{
			CopyRect(&rectWndOne, &rectWndTwo);//copy it to the other rect since this has a higher
											 //potential of being the message text window
			hLowestRichEdit=hWndChild;
		}
	}

	if(NULL != hLowestRichEdit)
	{
		hPlainTxtMsgWnd=hLowestRichEdit;
		return TRUE;
	}

	//let's find out the tallest control in the window
	ZeroMemory(&rectWndOne, sizeof(RECT));
	ZeroMemory(&rectWndTwo, sizeof(RECT));
	while(NULL != (hWndChild=FindWindowEx(m_hCtxtWindow, 
		hWndChild, NULL, NULL)))
	{
		//get the client rectangle of the window
		_VERIFY(TRUE == GetClientRect(hWndChild, &rectWndTwo));
		
		if((IsRectEmpty(&rectWndOne)) || //if this is the first time we got a window rect
		   (abs(rectWndTwo.bottom - rectWndTwo.top) > 
		    abs(rectWndOne.bottom - rectWndOne.top)))//or if this window is taller than the last window
		{
			CopyRect(&rectWndOne, &rectWndTwo);//copy it to the other rect since this has a higher
											 //potential of being the message text window
			hTallestControl=hWndChild;
		}
	}

	//if we could not find the tallest control
	if(NULL == hTallestControl)
	{
		pgpAssert(FALSE);//failed to get the message window handle.
		return FALSE;  //time to fix this routine. new version of GroupWise ?
	}

	hPlainTxtMsgWnd=hTallestControl;
	return TRUE;
}


BOOL CGWCommand::GetHtmlMsgWnd(HWND& hHtmlMsgWnd)
{
	HWND hWndChild=NULL;
	hWndChild=FindWindowEx(m_hCtxtWindow, hWndChild, "Shell Embedding", NULL);
	if(NULL != hWndChild)
	{
		hHtmlMsgWnd=hWndChild;
		return TRUE;
	}

	return FALSE;
}