////////////////////////////////////////////////////////////////////////////////
// File		GWDLL.H
// Title	GroupWise 5 Tokens
// Owner	SDK
// Tabs		4
// Version	5.5 EP SP1
//
// Copyright (C) 1995-2000 Novell, Inc., All Rights Reserved
//
// Description:
//
//	GroupWise 5 Token Structures, enumerations, token IDs, and Menu IDs
//		Mxpsx.h
//		Oftkn.rh
//		Ofmenu.rh
//
//	Token value determination has changed to be explicitly defined.	
//	For Example:
//		Old SDK header:		#define	AFTKN_USERID				    (BASE_AFUNC_TOKEN + 193)	// AFTKN_USERID
//		This SDK header:	#define	AFTKN_USERID	764		// (BASE_AFUNC_TOKEN + 210)		// AFTKN_USERID
//
//	This file includes:
//		1. Token structures, and enumerations
//		2. Token IDs
//		3. Menu IDs
//
//	Obsolete tokens have been removed.
//
//Revision history:  
//
//	Rev 5.5.2   5 JUN 2000
//	 - Revised/Updated
//
//////////////////////////////////////////////////////////////////////////////////

#include "windows.h"


////////////////////////////////////////////////////////////////////////////////
//	Token structures, and enumerations
////////////////////////////////////////////////////////////////////////////////

#define MAC_NAME_WZCOUNT				50		// Maximum size for a name.

#define MAC_NO_ERROR			0
#define MAC_BASE_GENERAL_ERRORS	(-99)
#define MAC_TOP_APP_ERROR		(MAC_BASE_GENERAL_ERRORS-1)


typedef double				VALUE_FLOAT;

typedef VALUE_FLOAT			*PVALUE_FLOAT;
typedef VALUE_FLOAT NEAR	*NPVALUE_FLOAT;
typedef VALUE_FLOAT FAR		*LPVALUE_FLOAT;


typedef long				VALUE_INT;

typedef VALUE_INT			*PVALUE_INT;
typedef VALUE_INT NEAR		*NPVALUE_INT;
typedef VALUE_INT FAR		*LPVALUE_INT;


// General typedefs
typedef WORD					MAC_FUNCTIONID;
typedef DWORD					MAC_COUNT;
typedef DWORD					MAC_MACROID;
typedef DWORD					MAC_IPCVERSION;
typedef MAC_FUNCTIONID		MAC_TOKENID;


////////////////////////////////////////////////////////////////////////////////
//	Token structures, and enumerations
//	Begin GroupWise 5 SDK Specific
////////////////////////////////////////////////////////////////////////////////

typedef unsigned short WZ;
typedef char AZ;
typedef WZ					MAC_SYMBOLNAME[MAC_NAME_WZCOUNT + 1];
typedef double				VALUE_FLOAT;
typedef long				VALUE_INT;

typedef enum _dllHandlerType {
	DLL_HAN_NOT_HANDLED,
	DLL_HAN_NO_ERROR,
	DLL_HAN_NOT_FOUND,
	DLL_HAN_CANCEL,
	DLL_HAN_TOKEN_ERROR,
	DLL_HAN_PARM_ERROR
} DLLHandlerType;

typedef enum _dllValidationType {
	DLL_VAL_UNKNOWN_TOKEN,
	DLL_VAL_VALIDATED,
	DLL_VAL_UNVALIDATED,
	DLL_VAL_UNKNOWN_TASK
} DLLValidationType;

#define WPLM_EXECUTE_TOKEN	(WM_USER + 11)	// Message from 3rd party DLL to
											// cause the app to execute a token.

#define	WPLM_ALLOC_MEM		(WM_USER + 25)	// Sent when a third party app
											// needs to allocate memory
											// wParam - 0, not used
											// lParam - Size of buffer to alloc
											// Return lpMem if successful
											// NULL if unsuccessful.

#define	WPLM_FREE_MEM		(WM_USER + 26)	// Sent when a third party app
											// frees memory obtained using
											// WPLM_ALLOC_MEM
											// wParam - 0, not used
											// lParam - lpMem to free
											// Return NULL if successful,
											// lpMem if unsuccessful

////////////////////////////////////////////////////////////////////////////////
//	End of GroupWise 5 SDK Specific
////////////////////////////////////////////////////////////////////////////////


// ranges for the MAC_VALUE_TYPE enumeration.
#define VALUE_GENERAL_TYPES		0x0000	// base range for general types
#define VALUE_PARAMETER_TYPES	0x0040	// base range for types used only on parameters.
#define VALUE_VARIABLE_TYPES	0x0080	// base range for types used only on parameters.
#define VALUE_END_OF_TYPES		0x00C0	// end of type ranges.

typedef enum
{
eValUndefined 		= VALUE_GENERAL_TYPES,	// undefined type


	//parameter types only
	eParmAny			= VALUE_PARAMETER_TYPES,// type decided at run-time
	eParmBoolean,								// 'C' BOOL type
	eParmByteSigned,							// signed 8-bit value
	eParmByteUnsigned,							// unsigned 8-bit value
	eParmCentimeters,		  					// double
	eParmDwordSigned,							// signed 32-bit value
	eParmDwordUnsigned,							// unsigned 32-bit value
	eParmDWPUSigned,							// signed 32-bit unit (WPU)
	eParmDWPUUnsigned,							// unsigned 32-bit unit (WPU)
	eParmEnumeration,							// enumeration 16-bit value
	eParmFloat,									// double
	eParmInches,								// double
	eParmMillimeters,							// double
	eParmPoints,								// double
	eParmStringAnsi,							// ANSI string (zero terminated)
	eParmStringLabel,							// WPCORP Word STring (zero terminated)
												////  containing a label reference.
	eParmStringOem,								// OEM string (zero terminated)
	eParmStringWord,							// WPCORP Word String (zero terminated)
	eParmStringVariable,						// word string containing a variable name
	eParmTokenID,								// unsigned 16-bit value
	eParmWordSigned,							// signed 16-bit value
	eParmWordUnsigned,							// unsigned 16-bit value
	eParmWPFname,								// ANSI string (zero terminated)
												//  containing a file name.
	eParmWPUSigned,								// signed 16-bit unit (WPU)
	eParmWPUUnsigned,							// unsigned 16-bit unit (WPU)
	eParmUserDialog,							// user dialog buffer (like a string)
	eParmStringWide,							// Wide/Unicode (zero terminated)

	// variable types only
	eValArrayDefinition = VALUE_VARIABLE_TYPES,	// definition of an Array
	eValBoolean,								// 'C' BOOL type
	eValCentimeters,		  					// double
	eValFloat,									// double
	eValInches,									// double
	eValInteger,								// signed 32-bit value
	eValMillimeters,							// double
	eValOLEObject,								// ???
	eValPoints,									// double
	eValString,									// WPCORP Word String (zero terminated)
	eValStringAnsi,								// ANSI string (zero terminated)
	eValStringOem,								// OEM string (zero terminated)
	eValWPUnits,								// signed 32-bit unit (WPU)
	eValAlias,									// used for reference parameters
	eValRecord,									// definition of a record
	eValDateTime,								// unsigned 32-bit value
	eValRawBinary,								// raw binary data type
	eValStringWide,							// Wide/Unicode (zero terminated)

	eValZzzzzNoMore		= VALUE_END_OF_TYPES	// there is nothing after this.
	} MAC_VALUE_TYPE;

// Macro IPC Parameter packet.
typedef struct _tagMAC_PARAM
{
MAC_VALUE_TYPE	eType;
WORD			wFlags;
WORD			wReserved;		// Reserved, must be NULL.
union
		{
		DWORD 		dwValue;
		void 	far *lpvPtr;
		double 		drValue;
		} uData;
} MAC_PARAM, NEAR *NPMAC_PARAM, FAR *LPMAC_PARAM, *PMAC_PARAM;

// Macro IPC token packet.
typedef struct _tagMAC_TOKEN
	{
	HSZ				hszCommand;		// "RouteToken"
	MAC_IPCVERSION	Version;		// Version of IPC, must == 0L;
	HSZ				hszRequestor;	// application that invoked this macro
	MAC_MACROID		dwMacroID;		// application id for this macro
	ATOM			atomApp;		// application the token is bound for
	WORD			wReserved;		// Reserved, must be NULL.
	MAC_TOKENID		wTokenId;		// TokenId
	MAC_COUNT		cParam;			// parameter count
	DWORD			dwFlags;   		// WRITEABLE, NON_WRITABLE
	DWORD			dwReserved;		// Reserved, must be NULL
	MAC_PARAM		rgParam[1];		// first parameter, others may be chained off
								// the end of the structure
	} MAC_TOKEN, NEAR *NPMAC_TOKEN, FAR *LPMAC_TOKEN, *PMAC_TOKEN;

typedef enum
	{
	MAC_FUNCTION_OK = MAC_NO_ERROR,		// Function returned ok

	MAC_FUNCTION_UNKNOWN =
			MAC_BASE_GENERAL_ERRORS,	// Unknown token

	MAC_FUNCTION_NOTFOUND,				// Function resulted in not found
	MAC_FUNCTION_CANCEL,				// Function resulted in cancel
	MAC_FUNCTION_ERROR,					// Function resulted in error
	MAC_FUNCTION_INVALID_PARM,			// Invalid parameter on function
	MAC_FUNCTION_INVALID,				// Function invalid at this time
	MAC_FUNCTION_NOT_HANDLED,			// Function not handled
	MAC_FUNCTION_RETURN_LATER			// Don't return from the function now.
	} MAC_TOKENERROR;

// Macro variable structure.
//  Used on the VariableSet requests and replys.
typedef struct _tagMAC_VARIABLE
	{
	HSZ					hszCommand;		// "VariableSet"
	MAC_IPCVERSION		Version;		// Version of IPC, must == 0L;
	HSZ					hszRequestor;	// Name of the requesting app.(specific)
	MAC_MACROID			dwMacroID;		// Application defined macro id
	ATOM				atomApp;		// application the request is bound for
	WORD				wReserved;		// Reserved, must be NULL.
	MAC_VALUE_TYPE		eType;			// must be in the eValxxx range,
										// between VALUE_VARIABLE_TYPES+1 and
										// VALUE_END_OF_TYPES
	MAC_TOKENID			wSysVarID;		// System Variable ID
	MAC_SYMBOLNAME		wzSymbolName;	// WZ containing the symbol name.
	union
		{
		BOOL			fData;			// eValBoolean
		VALUE_FLOAT		drData;			// eValCentimeters, eValFloat, eValInches,
										// eValMillimeters, eValPoints
		VALUE_INT		lData;			// eValInteger, eValWPUnits
		struct
			{
			WORD		wStrLen;		// Length of string (# of characters).
			union
				{
				WZ		wzData[1];		// First character in the string(WZ).
										// Remaining characters follow.
										// - eValString
				AZ		azData[1];		// First character in the string(AZ).
										// Remaining characters follow.
										// - eValStringAnsi, eValStringOem
				} uStr;
			} stData;
		} uValue;

	} MAC_VARIABLE, * PMAC_VARIABLE, FAR * LPMAC_VARIABLE, *PMAC_VARIABLE;

// Macro IPC return value packet.
typedef struct _tagMAC_RETURNVAL
	{
	HSZ					hszRequestor;	// application that invoked them macro that
										// sent the token.
	MAC_MACROID			dwMacroID;		// application id for the macro that sent
										// the token.
	MAC_TOKENERROR		eReturnCode;	// MAC_TOKENERROR

	MAC_VARIABLE		rv;				// token return value
										// - fill out the eType and uValue only
										//   for a token return.
										// - all other data will be ignored.
	} MAC_RETURNVAL, NEAR *NPMAC_RETURNVAL, FAR *LPMAC_RETURNVAL, *PMAC_RETURNVAL;

typedef LPMAC_RETURNVAL	FAR *LPLPMAC_RETURNVAL;		//used in TPH subsystem

typedef struct tph_returnval_tag {
	LPMAC_TOKEN			lpToken;
	LPLPMAC_RETURNVAL	lplpmacRetVal;
} TPH_RETURNVAL, FAR * LPTPH_RETURNVAL;


////////////////////////////////////////////////////////////////////////////////
//	Token IDs
//
////////////////////////////////////////////////////////////////////////////////

#define BASE_TOKEN				32		// Tokens start here
#define BFTKN_INVALID_TOKEN		0


////////////////////////////////////////////////////////////////////////////////
//	Token IDs
//	- Dialogs Token Values
////////////////////////////////////////////////////////////////////////////////

	#define BASE_DIALOG_TOKEN		BASE_TOKEN

#define	DTKN_FILE_OPEN_DLG			32		// (BASE_DIALOG_TOKEN + 0)				// DTKN_FILE_OPEN_DLG
#define	DTKN_REM_ITEMFILTERDLG		34		// (BASE_DIALOG_TOKEN + 2)				// DTKN_REM_ITEMFILTERDLG
#define	DTKN_RENAME_FOLDER			35		// (BASE_DIALOG_TOKEN + 3)				// DTKN_RENAME_FOLDER
#define	DTKN_CREATE_FOLDER			36		// (BASE_DIALOG_TOKEN + 4)				// DTKN_CREATE_FOLDER
#define	DTKN_IMPORT_OLDCAL			37		// (BASE_DIALOG_TOKEN + 5)				// DTKN_IMPORT_OLDCAL
#define	DTKN_BTNBAR_EDIT			38		// (BASE_DIALOG_TOKEN + 6)				// DTKN_BTNBAR_EDIT
#define	DTKN_BTNBAR_OPTIONS			39		// (BASE_DIALOG_TOKEN + 7)				// DTKN_BTNBAR_OPTIONS
#define	DTKN_BTNBAR_NEW				40		// (BASE_DIALOG_TOKEN + 8)				// DTKN_BTNBAR_NEW
#define	DTKN_BTNBAR_SELECT			41		// (BASE_DIALOG_TOKEN + 9)				// DTKN_BTNBAR_SELECT
#define	DTKN_BTNBAR_SAVE			42		// (BASE_DIALOG_TOKEN + 10)			// DTKN_BTNBAR_SAVE
#define	DTKN_PRINT_CALENDAR			43		// (BASE_DIALOG_TOKEN + 11)			// DTKN_PRINT_CALENDAR
#define	DTKN_ADDRBOOK				44		// (BASE_DIALOG_TOKEN + 12)			// DTKN_ADDRBOOK
#define	DTKN_VIEWER_SAVE_AS			45		// (BASE_DIALOG_TOKEN + 13)			// DTKN_VIEWER_SAVE_AS
#define	DTKN_SELECT_FOLDER			46		// (BASE_DIALOG_TOKEN + 14)			// DTKN_SELECT_FOLDER
#define	DTKN_SEND_OPTIONS			47		// (BASE_DIALOG_TOKEN + 15)			// DTKN_SEND_OPTIONS
#define	DTKN_HELP_COACHES			49		// (BASE_DIALOG_TOKEN + 17)			// DTKN_HELP_COACHES
#define	DTKN_DELETE_FOLDERS			50		// (BASE_DIALOG_TOKEN + 18)			// DTKN_DELETE_FOLDERS
#define	DTKN_REM_ADDRESSREQUESTDLG	52		// (BASE_DIALOG_TOKEN + 20)			// DTKN_REM_ADDRESSREQUESTDLG
#define	DTKN_REM_PGROUPREQUESTDLG	53		// (BASE_DIALOG_TOKEN + 21)			// DTKN_REM_PGROUPREQUESTDLG
#define	DTKN_CUSTOM_REPLY			54		// (BASE_DIALOG_TOKEN + 22)			// DTKN_CUSTOM_REPLY
#define	DTKN_PRINT_SETUP			55		// (BASE_DIALOG_TOKEN + 23)			// DTKN_PRINT_SETUP
#define	DTKN_ACCEPT_REPLY			56		// (BASE_DIALOG_TOKEN + 24)			// DTKN_ACCEPT_REPLY
#define	DTKN_REJECT_REPLY			57		// (BASE_DIALOG_TOKEN + 25)			// DTKN_REJECT_REPLY
#define	DTKN_FILTER_ITEMLIST		58		// (BASE_DIALOG_TOKEN + 26)			// DTKN_FILTER_ITEMLIST
#define	DTKN_DELEGATE				59		// (BASE_DIALOG_TOKEN + 27)			// DTKN_DELEGATE
#define	DTKN_PRESETUP				60		// (BASE_DIALOG_TOKEN + 28)			// DTKN_PRESETUP
#define	DTKN_REM_CONNECTIONSDLG		61		// (BASE_DIALOG_TOKEN + 29)			// DTKN_REM_CONNECTIONSDLG
#define	DTKN_RULE_LIST				62		// (BASE_DIALOG_TOKEN + 30)			// DTKN_RULE_LIST
#define	DTKN_LINKS					64		// (BASE_DIALOG_TOKEN + 32)			// DTKN_LINKS
#define	DTKN_FOLDER_SHARE			65		// (BASE_DIALOG_TOKEN + 33)			// DTKN_FOLDER_SHARE
#define	DTKN_ABOUT					66		// (BASE_DIALOG_TOKEN + 34)			// DTKN_ABOUT
#define	DTKN_SEARCH					67		// (BASE_DIALOG_TOKEN + 35)			// DTKN_SEARCH
#define	DTKN_PROPERTIES	 			68		// (BASE_DIALOG_TOKEN + 36)			// DTKN_PROPERTIES
#define	DTKN_CHOOSE_FONT			69		// (BASE_DIALOG_TOKEN + 37)			// DTKN_CHOOSE_FONT
#define	DTKN_SAVE_AS_DLG			70		// (BASE_DIALOG_TOKEN + 38)			// DTKN_SAVE_AS_DLG
#define	DTKN_SAVE_DLG				71		// (BASE_DIALOG_TOKEN + 39)			// DTKN_SAVE_DLG
#define	DTKN_RETRIEVE				72		// (BASE_DIALOG_TOKEN + 40)			// DTKN_RETRIEVE
#define	DTKN_INSERT_OBJECT			73		// (BASE_DIALOG_TOKEN + 41)			// DTKN_INSERT_OBJECT
#define	DTKN_GSET_AUTODATE			74		// (BASE_DIALOG_TOKEN + 42)			// DTKN_GSET_AUTODATE
#define	DTKN_GSET_STARTDATE			75		// (BASE_DIALOG_TOKEN + 43)			// DTKN_GSET_STARTDATE
#define	DTKN_GSET_STARTTIME			76		// (BASE_DIALOG_TOKEN + 44)			// DTKN_GSET_STARTTIME
#define	DTKN_GSET_ENDTIME			77		// (BASE_DIALOG_TOKEN + 45)			// DTKN_GSET_ENDTIME
#define	DTKN_LINK_FOLDER			78		// (BASE_DIALOG_TOKEN + 46)			// DTKN_LINK_FOLDER
#define	DTKN_SORTCOLMAN				79		// (BASE_DIALOG_TOKEN + 47)			// DTKN_SORTCOLMAN
#define	DTKN_PROXY					80		// (BASE_DIALOG_TOKEN + 48)			// DTKN_PROXY
#define	DTKN_ATTACHMENTS			81		// (BASE_DIALOG_TOKEN + 49)			// DTKN_ATTACHMENTS
#define	DTKN_BUSYSRCH				82		// (BASE_DIALOG_TOKEN + 50)			// DTKN_BUSYSRCH
#define	DTKN_PRINT					83		// (BASE_DIALOG_TOKEN + 51)			// DTKN_PRINT
#define	DTKN_SET_ALARM				84		// (BASE_DIALOG_TOKEN + 52)			// DTKN_SET_ALARM
#define	DTKN_ATTACH_OBJECT			85		// (BASE_DIALOG_TOKEN + 53)			// DTKN_ATTACH_OBJECT
#define	DTKN_CHANGE_VIEW			86		// (BASE_DIALOG_TOKEN + 54)			// DTKN_CHANGE_VIEW
#define	DTKN_FOLDER_PROFILEREF		87		// (BASE_DIALOG_TOKEN + 55)			// DTKN_FOLDER_PROFILEREF
#define	DTKN_ATTACH_FILE			88		// (BASE_DIALOG_TOKEN + 56)			// DTKN_ATTACH_FILE
#define	DTKN_GOTO_DATE				89		// (BASE_DIALOG_TOKEN + 57)			// DTKN_GOTO_DATE
#define	DTKN_DATE_DIFF				90		// (BASE_DIALOG_TOKEN + 58)			// DTKN_DATE_DIFF
#define	DTKN_SOUNDS					91		// (BASE_DIALOG_TOKEN + 59)			// DTKN_SOUNDS
#define	DTKN_GSET_ENDDATE			92		// (BASE_DIALOG_TOKEN + 60)			// DTKN_GSET_ENDDATE
#define	DTKN_BTNBAR_PREF			93		// (BASE_DIALOG_TOKEN + 61)			// DTKN_BTNBAR_PREF
#define	DTKN_ADDRESS_INFO			94		// (BASE_DIALOG_TOKEN + 62)			// DTNK_ADDRESS_INFO
#define	DTKN_ATTACH_DOCREF			95		// (BASE_DIALOG_TOKEN + 63)
#define	DTKN_REM_DISK_SPACE_MGMT	96		// (BASE_DIALOG_TOKEN + 64)			// DTKN_REM_DISK_SPACE_MGMT
#define	DTKN_QUERY					97		// (BASE_DIALOG_TOKEN + 65)			// DTKN_QUERY
#define	DTKN_DM_COPY				98		// (BASE_DIALOG_TOKEN + 66)			// DTKN_DM_COPY
#define	DTKN_DM_NEW					99		// (BASE_DIALOG_TOKEN + 67)			// DTKN_DM_NEW
#define	DTKN_QUERY_EDIT				100		// (BASE_DIALOG_TOKEN + 68)			// DTKN_QUERY_EDIT
#define	DTKN_QUERY_SAVE_AS_FOLDER	101		// (BASE_DIALOG_TOKEN + 69)			// DTKN_QUERY_SAVE_AS_FOLDER
#define	DTKN_REM_UPDATE				102		// (BASE_DIALOG_TOKEN + 70)			// DTKN_REM_UPDATE
#define	DTKN_REM_SELECTEDUPDATE		103		// (BASE_DIALOG_TOKEN + 71)			// DTKN_REM_SELECTEDUPDATE
#define	DTKN_REM_SETUP				104		// (BASE_DIALOG_TOKEN + 72)			// DTKN_REM_SETUP
#define	DTKN_REM_UPDATE_INFO		105		// (BASE_DIALOG_TOKEN + 73)			// DTKN_REM_UPDATE_INFO
#define	DTKN_DM_CHECKIN				106		// (BASE_DIALOG_TOKEN + 74)			// DTKN_DM_CHECKIN
#define	DTKN_DM_CHECKOUT			107		// (BASE_DIALOG_TOKEN + 75)			// DTKN_DM_CHECKOUT
#define	DTKN_ADDRBOOK_MODAL			108		// (BASE_DIALOG_TOKEN + 76)			// Handled in ofitmvew.
#define	DTKN_SELECT_COLUMNS			109		// (BASE_DIALOG_TOKEN + 77)			// DTKN_SELECT_COLUMNS
#define	DTKN_SECURITY_PROPERTIES	110		// (BASE_DIALOG_TOKEN + 78)			// DTKN_SECURITY_PROPERTIES
#define	DTKN_DM_IMPORT				112		// (BASE_DIALOG_TOKEN + 80)			// DTNK_DM_IMPORT
#define	DTKN_DM_SETINUSE			113		// (BASE_DIALOG_TOKEN + 81)			// DTKN_DM_SETINUSE
#define	DTKN_DM_ARCHIVE				114		// (BASE_DIALOG_TOKEN + 82)			// DTKN_DM_ARCHIVE
#define	DTKN_DM_EXPORT				115		// (BASE_DIALOG_TOKEN + 83)			// DTKN_DM_EXPORT
#define	DTKN_UNASSIGNED_84			116		// (BASE_DIALOG_TOKEN + 84)
#define	DTKN_HIT_THE_ROAD			117		// (BASE_DIALOG_TOKEN + 85)			// DTKN_HIT_THE_ROAD
#define	DTKN_DM_NEW_VERSION			118		// (BASE_DIALOG_TOKEN + 86)			// DTKN_DM_NEW_VERSION
#define	DTKN_ITEM_CHANGETO			119		// (BASE_DIALOG_TOKEN + 87)			// DTKN_ITEM_CHANGETO
#define	DTKN_SELECT_TIMEZONE		120		// (BASE_DIALOG_TOKEN + 88)			// DTKN_SELECT_TIMEZONE
#define	DTKN_CREATE_ARCHIVE			121		// (BASE_DIALOG_TOKEN + 89)			// DTKN_CREATE_ARCHIVE
#define	DTKN_ATTBTNBAR_EDIT			122		// (BASE_DIALOG_TOKEN + 90)
#define	DTKN_QUERY_ODMA				123		// (BASE_DIALOG_TOKEN + 91)			//DTKN_QUERY_ODMA
#define	DTKN_MULTIUSER_LIST			124		// (BASE_DIALOG_TOKEN + 92)
#define	DTKN_DM_MASS_OPERATION		125		// (BASE_DIALOG_TOKEN + 93)
#define	DTKN_DISPLAY_SETTINGS_SELECT 126		// (BASE_DIALOG_TOKEN + 94)		// DTKN_DISPLAY_SETTINGS_SELECT
#define	DTKN_DISPLAY_SETTINGS_EDIT	127		// (BASE_DIALOG_TOKEN + 95)			// DTKN_DISPLAY_SETTINGS_EDIT
#define	DTKN_DM_REPLACE_DOC			128		// (BASE_DIALOG_TOKEN + 96)			// DTKN_DM_REPLACE_DOC
#define	DTKN_CALTAB_PROPS			129		// (BASE_DIALOG_TOKEN + 97)
#define	DTKN_ACCOUNTS 				130		// (BASE_DIALOG_TOKEN + 98)
#define	DTKN_NEWS_GROUPS			131		// (BASE_DIALOG_TOKEN + 99)
#define	DTKN_UNASSIGNED_127			159		// (BASE_DIALOG_TOKEN + 127)

#define	TOP_DIALOG_TOKEN				DTKN_UNASSIGNED_127


////////////////////////////////////////////////////////////////////////////////
//	Token IDs
//	- Application Base Functions Token IDs
////////////////////////////////////////////////////////////////////////////////

#define BASE_FUNCTION_TOKEN					160

#if TOP_DIALOG_TOKEN >= BASE_FUNCTION_TOKEN
  #error Token Ranges Overlap!
#endif

#define BASE_BFUNC_TOKEN					BASE_FUNCTION_TOKEN

#define	BFTKN_APP_EXIT						160		// (BASE_BFUNC_TOKEN + 0)			// BFTKN_APP_EXIT
#define	BFTKN_MAXIMIZE_WINDOW				161		// (BASE_BFUNC_TOKEN + 1)			// BFTKN_MAXIMIZE_WINDOW
#define	BFTKN_REM_CONNECT					162		// (BASE_BFUNC_TOKEN + 2)			// BFTKN_REM_CONNECT
#define	BFTKN_PRINT							165		// (BASE_BFUNC_TOKEN + 5)			// BFTKN_PRINT
#define BFTKN_SHOW_ATTWIN					166 	// (BASE_BFUNC_TOKEN + 6)			// BFTKN_SHOW_ATTWIN
#define	BFTKN_SEND_STDMAIL					167		// (BASE_BFUNC_TOKEN + 7)			// BFTKN_SEND_STDMAIL
#define	BFTKN_HELP_CONTENTS					168		// (BASE_BFUNC_TOKEN + 8)			// BFTKN_HELP_CONTENTS
#define	BFTKN_HELP_KEYBOARD					169		// (BASE_BFUNC_TOKEN + 9)			// BFTKN_HELP_KEYBOARD
#define	BFTKN_HELP_HOWDOI					170		// (BASE_BFUNC_TOKEN + 10)			// BFTKN_HELP_HOWDOI
#define	BFTKN_HELP_SEARCH_FOR				171		// (BASE_BFUNC_TOKEN + 11)			// BFTKN_HELP_SEARCH_FOR
#define	BFTKN_HELP_USINGHELP				172		// (BASE_BFUNC_TOKEN + 12)			// BFTKN_HELP_USINGHELP
#define	BFTKN_HELP_WHATIS					173		// (BASE_BFUNC_TOKEN + 13)			// BFTKN_HELP_WHATIS
#define	BFTKN_FILE_NEW						174		// (BASE_BFUNC_TOKEN + 14)			// BFTKN_FILE_NEW
#define	BFTKN_REFRESH						175		// (BASE_BFUNC_TOKEN + 15)			// BFTKN_REFRESH
#define	BFTKN_BTNBAR_SHOW					176		// (BASE_BFUNC_TOKEN + 16)			// BFTKN_BTNBAR_SHOW
#define	BFTKN_RETURN						179		// (BASE_BFUNC_TOKEN + 19)			// BFTKN_RETURN
#define	BFTKN_SEND_STDTODO					180		// (BASE_BFUNC_TOKEN + 20)			// BFTKN_SEND_STDTODO
#define	BFTKN_SEND_STDNOTE					181		// (BASE_BFUNC_TOKEN + 21)			// BFTKN_SEND_STDNOTE
#define	BFTKN_SEND_STDAPPT					182		// (BASE_BFUNC_TOKEN + 22)			// BFTKN_SEND_STDAPPT
#define	BFTKN_BTNBAR_HIDE					183		// (BASE_BFUNC_TOKEN + 23)			// BFTKN_BTNBAR_HIDE
#define	BFTKN_OPEN_TRASH					184		// (BASE_BFUNC_TOKEN + 24)			// BFTKN_OPEN_TRASH
#define	BFTKN_HELP_LOCAL					185		// (BASE_BFUNC_TOKEN + 25)			// BFTKN_HELP_LOCAL
#define	BFTKN_PAUSEMOVIE					186		// (BASE_BFUNC_TOKEN + 26)			// BFTKN_PAUSEMOVIE
#define	BFTKN_RESTARTMOVIE					187		// (BASE_BFUNC_TOKEN + 27)			// BFTKN_RESTARTMOVIE
#define	BFTKN_RESUMEMOVIE					188		// (BASE_BFUNC_TOKEN + 28)			// BFTKN_RESUMEMOVIE
#define	BFTKN_OPEN_INBOX					189		// (BASE_BFUNC_TOKEN + 29)			// BFTKN_OPEN_INBOX
#define	BFTKN_OPEN_OUTBOX					190		// (BASE_BFUNC_TOKEN + 30)			// BFTKN_OPEN_OUTBOX
#define	BFTKN_UNDELETE						191		// (BASE_BFUNC_TOKEN + 31)			// BFTKN_UNDELETE
#define	BFTKN_TOGGLE_ROUTING				192		// (BASE_BFUNC_TOKEN + 32)			// BFTKN_TOGGLE_ROUTING
#define	BFTKN_COMPLETE_ITEM					193		// (BASE_BFUNC_TOKEN + 33)			// BFTKN_COMPLETE_ITEM
#define	BFTKN_ACCEPT						194		// (BASE_BFUNC_TOKEN + 34)			// BFTKN_ACCEPT
#define	BFTKN_REJECT						195		// (BASE_BFUNC_TOKEN + 35)			// BFTKN_REJECT
#define	BFTKN_SEND_STDPHON 					196		// (BASE_BFUNC_TOKEN + 36)			// BFTKN_SEND_STDPHON
#define	BFTKN_HIDE_CAPTION					197		// (BASE_BFUNC_TOKEN + 37)			// BFTKN_HIDE_CAPTION
#define	BFTKN_ON_TOP						198		// (BASE_BFUNC_TOKEN + 38)			// BFTKN_ON_TOP
#define	BFTKN_ALLWIN_CLOSE					200		// (BASE_BFUNC_TOKEN + 40)			// BFTKN_ALLWIN_CLOSE
#define	BFTKN_ALLWIN_MINIMIZE				201		// (BASE_BFUNC_TOKEN + 41)			// BFTKN_ALLWIN_MINIMIZE
#define	BFTKN_CANCEL						202		// (BASE_BFUNC_TOKEN + 42)			// BFTKN_CANCEL
#define BFTKN_UP_LEVEL						203 	// (BASE_BFUNC_TOKEN + 43)			// BFTKN_UP_LEVEL
#define	BFTKN_HELP_TUTORIAL					204		// (BASE_BFUNC_TOKEN + 44)			// BFTKN_HELP_TUTORIAL
#define	BFTKN_ENV_CLEAR_CHANGED_FLAG		205		// (BASE_BFUNC_TOKEN + 45)			// BFTKN_ENV_CLEAR_CHANGED_FLAG
#define	BFTKN_OPEN_CALENDAR					206		// (BASE_BFUNC_TOKEN + 46)			// BFTKN_OPEN_CALENDAR
#define	BFTKN_EMPTY_TRASH					207		// (BASE_BFUNC_TOKEN + 47)			// BFTKN_EMPTY_TRASH
#define	BFTKN_OLE_EMBEDD					208		// (BASE_BFUNC_TOKEN + 48)			// BFTKN_OLE_EMBEDD
#define	BFTKN_DELETE_OPEN_ITEM				209		// (BASE_BFUNC_TOKEN + 49)			// BFTKN_DELETE_OPEN_ITEM
#define	BFTKN_PASTELINK						210		// (BASE_BFUNC_TOKEN + 50)			// BFTKN_PASTELINK
#define	BFTKN_BACKSPACE						211		// (BASE_BFUNC_TOKEN + 51)			// BFTKN_BACKSPACE
#define	BFTKN_BEGLINE						212		// (BASE_BFUNC_TOKEN + 52)			// BFTKN_BEGLINE
#define	BFTKN_BEGTEXT						213		// (BASE_BFUNC_TOKEN + 53)			// BFTKN_BEGTEXT
#define	BFTKN_CUT							214		// (BASE_BFUNC_TOKEN + 54)			// BFTKN_CUT
#define	BFTKN_COPY							215		// (BASE_BFUNC_TOKEN + 55)			// BFTKN_COPY
#define	BFTKN_DELETE						216		// (BASE_BFUNC_TOKEN + 56)			// BFTKN_DELETE
#define	BFTKN_DELETETOEND					217		// (BASE_BFUNC_TOKEN + 57)			// BFTKN_DELETETOEND
#define	BFTKN_DELETEWORDLEFT				218		// (BASE_BFUNC_TOKEN + 58)			// BFTKN_DELETEWORDLEFT
#define	BFTKN_DELETEWORDRIGHT				219		// (BASE_BFUNC_TOKEN + 59)			// BFTKN_DELETEWORDRIGHT
#define	BFTKN_DOWNARROW						220		// (BASE_BFUNC_TOKEN + 60)			// BFTKN_DOWNARROW
#define	BFTKN_ENDLINE						221		// (BASE_BFUNC_TOKEN + 61)			// BFTKN_ENDLINE
#define	BFTKN_ENDTEXT						222		// (BASE_BFUNC_TOKEN + 62)			// BFTKN_ENDTEXT
#define	BFTKN_LEFTARROW						223		// (BASE_BFUNC_TOKEN + 63)			// BFTKN_LEFTARROW
#define	BFTKN_PAGEDOWN						224		// (BASE_BFUNC_TOKEN + 64)			// BFTKN_PAGEDOWN
#define	BFTKN_PAGELEFT						225		// (BASE_BFUNC_TOKEN + 65)			// BFTKN_PAGELEFT
#define	BFTKN_PAGERIGHT						226		// (BASE_BFUNC_TOKEN + 66)			// BFTKN_PAGERIGHT
#define	BFTKN_PAGEUP						227		// (BASE_BFUNC_TOKEN + 67)			// BFTKN_PAGEUP
#define	BFTKN_PASTE							228		// (BASE_BFUNC_TOKEN + 68)			// BFTKN_PASTE
#define	BFTKN_RENAME						229		// (BASE_BFUNC_TOKEN + 69)			// BFTKN_RENAME
#define	BFTKN_RIGHTARROW					230		// (BASE_BFUNC_TOKEN + 70)			// BFTKN_RIGHTARROW
#define	BFTKN_SELECTBEGLINE					231		// (BASE_BFUNC_TOKEN + 71)			// BFTKN_SELECTBEGLINE
#define	BFTKN_SELECTBEGTEXT					232		// (BASE_BFUNC_TOKEN + 72)			// BFTKN_SELECTBEGTEXT
#define	BFTKN_SELECTENDLINE					233		// (BASE_BFUNC_TOKEN + 73)			// BFTKN_SELECTENDLINE
#define	BFTKN_SELECTENDTEXT					234		// (BASE_BFUNC_TOKEN + 74)			// BFTKN_SELECTENDTEXT
#define	BFTKN_SELECTDOWN					235		// (BASE_BFUNC_TOKEN + 75)			// BFTKN_SELECTDOWN
#define	BFTKN_SELECTLEFT					236		// (BASE_BFUNC_TOKEN + 76)			// BFTKN_SELECTLEFT
#define	BFTKN_SELECTLEFTWORD				237		// (BASE_BFUNC_TOKEN + 77)			// BFTKN_SELECTLEFTWORD
#define	BFTKN_SELECTPAGEDOWN				238		// (BASE_BFUNC_TOKEN + 78)			// BFTKN_SELECTPAGEDOWN
#define	BFTKN_SELECTPAGEUP					239		// (BASE_BFUNC_TOKEN + 79)			// BFTKN_SELECTPAGEUP
#define	BFTKN_SELECTRIGHT					240		// (BASE_BFUNC_TOKEN + 80)			// BFTKN_SELECTRIGHT
#define	BFTKN_SELECTRIGHTWORD				241		// (BASE_BFUNC_TOKEN + 81)			// BFTKN_SELECTRIGHTWORD
#define	BFTKN_SELECTUP						242		// (BASE_BFUNC_TOKEN + 82)			// BFTKN_SELECTUP
#define	BFTKN_INSERT						243		// (BASE_BFUNC_TOKEN + 83)			// BFTKN_INSERT
#define	BFTKN_UNDO							244		// (BASE_BFUNC_TOKEN + 84)			// BFTKN_UNDO
#define	BFTKN_UPARROW						245		// (BASE_BFUNC_TOKEN + 85)			// BFTKN_UPARROW
#define	BFTKN_WORDLEFT						246		// (BASE_BFUNC_TOKEN + 86)			// BFTKN_WORDLEFT
#define	BFTKN_WORDRIGHT						247		// (BASE_BFUNC_TOKEN + 87)			// BFTKN_WORDRIGHT
#define	BFTKN_BOLD							248		// (BASE_BFUNC_TOKEN + 88)			// BFTKN_BOLD
#define	BFTKN_UNDERLINE						249		// (BASE_BFUNC_TOKEN + 89)			// BFTKN_UNDERLINE
#define	BFTKN_ITALIC						250		// (BASE_BFUNC_TOKEN + 90)			// BFTKN_ITALIC
#define	BFTKN_NORMAL						251		// (BASE_BFUNC_TOKEN + 91)			// BFTKN_NORMAL
#define	BFTKN_SPELLER						252		// (BASE_BFUNC_TOKEN + 92)			// BFTKN_SPELLER
#define	BFTKN_THES							253		// (BASE_BFUNC_TOKEN + 93)			// BFTKN_THES
#define	BFTKN_VIEW_TAB						254		// (BASE_BFUNC_TOKEN + 94)			// BFTKN_VIEW_TAB
#define	BFTKN_VIEW_SHIFTTAB					255		// (BASE_BFUNC_TOKEN + 95)			// BFTKN_VIEW_SHIFTTAB
#define	BFTKN_SYS_MOVE						256		// (BASE_BFUNC_TOKEN + 96)			// BFTKN_SYS_MOVE
#define	BFTKN_SYS_SIZE						257		// (BASE_BFUNC_TOKEN + 97)			// BFTKN_SYS_SIZE
#define	BFTKN_SYS_SWITCHTO					258		// (BASE_BFUNC_TOKEN + 98)			// BFTKN_SYS_SWITCHTO
#define	BFTKN_SYS_APPCLOSE					259		// (BASE_BFUNC_TOKEN + 99)			// BFTKN_SYS_APPCLOSE
#define	BFTKN_CTRL_TAB						260		// (BASE_BFUNC_TOKEN + 100)		// BFTKN_CTRL_TAB
#define	BFTKN_GRAMMATIK						261		// (BASE_BFUNC_TOKEN + 101)		// BFTKN_GRAMMATIK
#define	BFTKN_SEND_SHRFOLD					262		// (BASE_BFUNC_TOKEN + 102)		// BFTKN_SEND_SHRFOLD
#define	BFTKN_VIEW_PASTESPECIAL				263		// (BASE_BFUNC_TOKEN + 103)		// BFTKN_VIEW_PASTESPECIAL
#define	BFTKN_VIEWBY_CAL					264		// (BASE_BFUNC_TOKEN + 104)		// BFTKN_VIEWBY_CAL
#define	BFTKN_ATTACH_INFO					265		// (BASE_BFUNC_TOKEN + 105)		// BFTKN_ATTACH_INFO
#define	BFTKN_SEND							266		// (BASE_BFUNC_TOKEN + 106)		// BFTKN_SEND
#define	BFTKN_OPEN_ATTACH					267		// (BASE_BFUNC_TOKEN + 107)		// BFTKN_OPEN_ATTACH
#define	BFTKN_MINIMIZE_WINDOW				268		// (BASE_BFUNC_TOKEN + 108)		// BFTKN_MINIMIZE_WINDOW
#define	BFTKN_RESTORE_WINDOW				269		// (BASE_BFUNC_TOKEN + 109)		// BFTKN_RESTORE_WINDOW
#define	BFTKN_SELECT						270		// (BASE_BFUNC_TOKEN + 110)		// BFTKN_SELECT
#define	BFTKN_REPLY							271		// (BASE_BFUNC_TOKEN + 111)		// BFTKN_REPLY
#define	BFTKN_READ							272		// (BASE_BFUNC_TOKEN + 112)		// BFTKN_READ
#define	BFTKN_INFO							273		// (BASE_BFUNC_TOKEN + 113)		// BFTKN_INFO
#define	BFTKN_FORWARD						274		// (BASE_BFUNC_TOKEN + 114)		// BFTKN_FORWARD
#define	BFTKN_RESEND						275		// (BASE_BFUNC_TOKEN + 115)		// BFTKN_RESEND
#define	BFTKN_COLUMN_ADD					276		// (BASE_BFUNC_TOKEN + 116)		// BFTKN_COLUMN_ADD
#define	BFTKN_COLUMN_DELETE					277		// (BASE_BFUNC_TOKEN + 117)		// BFTKN_COLUMN_DELETE
#define	BFTKN_MNEMONIC_DISPLAY				278		// (BASE_BFUNC_TOKEN + 118)		// BFTKN_MNEMONIC_DISPLAY
#define	BFTKN_STARTDATE						279		// (BASE_BFUNC_TOKEN + 119)		// BFTKN_STARTDATE
#define	BFTKN_AUTODATE						280		// (BASE_BFUNC_TOKEN + 120)		// BFTKN_AUTODATE
#define	BFTKN_ENDDATE						281		// (BASE_BFUNC_TOKEN + 121)		// BFTKN_ENDDATE
#define	BFTKN_DURATION						282		// (BASE_BFUNC_TOKEN + 122)		// BFTKN_DURATION
#define	BFTKN_NEXT_PANE						283		// (BASE_BFUNC_TOKEN + 123)		// BFTKN_NEXT_PANE
#define	BFTKN_PREV_PANE						284		// (BASE_BFUNC_TOKEN + 124)		// BFTKN_PREV_PANE
#define	BFTKN_SYNC_ACCOUNT					285		// (BASE_BFUNC_TOKEN + 125)		// BFTKN_SYNC_ACCOUNT
#define	BFTKN_SYNC_ACCOUNT_MENU_1			286		// (BASE_BFUNC_TOKEN + 126)		// BFTKN_SYNC_ACCOUNT_MENU_1
#define	BFTKN_SCROLL_LEFT					287		// (BASE_BFUNC_TOKEN + 127)		// BFTKN_SCROLL_LEFT
#define	BFTKN_SCROLL_RIGHT					288		// (BASE_BFUNC_TOKEN + 128)		// BFTKN_SCROLL_RIGHT
#define	BFTKN_SCROLL_NEXT					289		// (BASE_BFUNC_TOKEN + 129)		// BFTKN_SCROLL_NEXT
#define	BFTKN_SCROLL_PRIOR					290		// (BASE_BFUNC_TOKEN + 130)		// BFTKN_SCROLL_PRIOR
#define	BFTKN_PURGE							291		// (BASE_BFUNC_TOKEN + 131)		// BFTKN_PURGE
#define	BFTKN_CLOSE_WINDOW					292		// (BASE_BFUNC_TOKEN + 132)		// BFTKN_CLOSE_WINDOW
#define	BFTKN_ARCHIVE						293		// (BASE_BFUNC_TOKEN + 133)		// BFTKN_ARCHIVE
#define	BFTKN_CONVERT_TOSTATIC				294		// (BASE_BFUNC_TOKEN + 134)		// BFTKN_CONVERT_TOSTATIC
#define	BFTKN_FOLDER_EXPAND					295		// (BASE_BFUNC_TOKEN + 135)		// BFTKN_FOLDER_EXPAND
#define	BFTKN_FOLDER_EXPANDEX				296		// (BASE_BFUNC_TOKEN + 136)		// BFTKN_FOLDER_EXPANDEX
#define	BFTKN_FOLDER_CONTRACT				297		// (BASE_BFUNC_TOKEN + 137)		// BFTKN_FOLDER_CONTRACT
#define	BFTKN_FOLDER_CONTRACTEX				298		// (BASE_BFUNC_TOKEN + 138)		// BFTKN_FOLDER_CONTRACTEX
#define	BFTKN_FOLDER_EXPAND_ALL				299		// (BASE_BFUNC_TOKEN + 139)		// BFTKN_FOLDER_EXPAND_ALL
#define	BFTKN_CLOSE_MAINWND					300		// (BASE_BFUNC_TOKEN + 140)		// BFTKN_CLOSE_MAINWND
#define	BFTKN_OPEN_MAINWND					301		// (BASE_BFUNC_TOKEN + 141)		// BFTKN_OPEN_MAINWND
#define	BFTKN_VERB1							302		// (BASE_BFUNC_TOKEN + 142)		// BFTKN_VERB1
#define	BFTKN_VERB2							303		// (BASE_BFUNC_TOKEN + 143)		// BFTKN_VERB2
#define	BFTKN_VERB3							304		// (BASE_BFUNC_TOKEN + 144)		// BFTKN_VERB3
#define	BFTKN_VERB4							305		// (BASE_BFUNC_TOKEN + 145)		// BFTKN_VERB4
#define	BFTKN_VERB5							306		// (BASE_BFUNC_TOKEN + 146)		// BFTKN_VERB5
#define	BFTKN_VERB6							307		// (BASE_BFUNC_TOKEN + 147)		// BFTKN_VERB6
#define	BFTKN_VERB7							308		// (BASE_BFUNC_TOKEN + 148)		// BFTKN_VERB7
#define	BFTKN_VERB8							309		// (BASE_BFUNC_TOKEN + 149)		// BFTKN_VERB8
#define	BFTKN_VERB9							310		// (BASE_BFUNC_TOKEN + 150)		// BFTKN_VERB9
#define	BFTKN_VERB10						311		// (BASE_BFUNC_TOKEN + 151)		// BFTKN_VERB10
#define	BFTKN_VERB_PRIMARY					312		// (BASE_BFUNC_TOKEN + 152)		// BFTKN_VERB_PRIMARY
#define	BFTKN_DB_TOGGLE						313		// (BASE_BFUNC_TOKEN + 153)		// BFTKN_DB_TOGGLE
#define	BFTKN_CHANGE_TO_ARCHIVEDB			314		// (BASE_BFUNC_TOKEN + 154)		// BFTKN_CHANGE_TO_ARCHIVEDB
#define	BFTKN_CHANGE_TO_USERDB				315		// (BASE_BFUNC_TOKEN + 155)		// BFTKN_CHANGE_TO_USERDB
#define	BFTKN_SEARCH_NEXT					316		// (BASE_BFUNC_TOKEN + 156)		// BFTKN_SEARCH_NEXT
#define	BFTKN_SEARCH_PREV					317		// (BASE_BFUNC_TOKEN + 157)		// BFTKN_SEARCH_PREV
#define	BFTKN_INFO_UPDATE					318		// (BASE_BFUNC_TOKEN + 158)		// BFTKN_INFO_UPDATE
#define	BFTKN_SCULPTURE						319		// (BASE_BFUNC_TOKEN + 159)		// BFTKN_SCULPTURE
#define	BFTKN_SYS_COLORS					320		// (BASE_BFUNC_TOKEN + 160)		// BFTKN_SYS_COLORS
#define	BFTKN_ATTACH_PASTE					321		// (BASE_BFUNC_TOKEN + 161)		// BFTKN_ATTACH_PASTE
#define	BFTKN_ATTACH_PASTELINK				322		// (BASE_BFUNC_TOKEN + 162)		// BFTKN_ATTACH_PASTELINK
#define	BFTKN_DELETE_ATTACH					323		// (BASE_BFUNC_TOKEN + 163)		// BFTKN_DELETE_ATTACH
#define	BFTKN_READ_NEXT						324		// (BASE_BFUNC_TOKEN + 164)		// BFTKN_READ_NEXT
#define	BFTKN_READ_PREV						325		// (BASE_BFUNC_TOKEN + 165)		// BFTKN_READ_PREV

#define	BFTKN_WIND_MENU_1					326		// (BASE_BFUNC_TOKEN + 166)		// BFTKN_WIND_MENU_1
#define	BFTKN_WIND_MENU_2					327		// (BASE_BFUNC_TOKEN + 167)		// BFTKN_WIND_MENU_2
#define	BFTKN_WIND_MENU_3					328		// (BASE_BFUNC_TOKEN + 168)		// BFTKN_WIND_MENU_3
#define	BFTKN_WIND_MENU_4					329		// (BASE_BFUNC_TOKEN + 169)		// BFTKN_WIND_MENU_4
#define	BFTKN_WIND_MENU_5					330		// (BASE_BFUNC_TOKEN + 170)		// BFTKN_WIND_MENU_5
#define	BFTKN_WIND_MENU_6					331		// (BASE_BFUNC_TOKEN + 171)		// BFTKN_WIND_MENU_6
#define	BFTKN_WIND_MENU_7					332		// (BASE_BFUNC_TOKEN + 172)		// BFTKN_WIND_MENU_7
#define	BFTKN_WIND_MENU_8					333		// (BASE_BFUNC_TOKEN + 173)		// BFTKN_WIND_MENU_8
#define	BFTKN_WIND_MENU_9					334		// (BASE_BFUNC_TOKEN + 174)		// BFTKN_WIND_MENU_9
#define	BFTKN_WIND_MENU_MORE				335		// (BASE_BFUNC_TOKEN + 175)		// BFTKN_WIND_MENU_MORE
#define	BFTKN_MARK_PRIVATE					336		// (BASE_BFUNC_TOKEN + 176)		// BFTKN_MARK_PRIVATE
#define	BFTKN_GOTO_DATE						337		// (BASE_BFUNC_TOKEN + 177)		// BFTKN_GOTO_DATE
#define	BFTKN_LVS_LARGE_ICON				338		// (BASE_BFUNC_TOKEN + 178)		// BFTKN_LVS_LARGE_ICON
#define	BFTKN_LVS_SMALL_ICON				339		// (BASE_BFUNC_TOKEN + 179)		// BFTKN_LVS_SMALL_ICON
#define	BFTKN_LVS_LIST						340		// (BASE_BFUNC_TOKEN + 180)		// BFTKN_LVS_LIST
#define	BFTKN_LVS_DETAILS					341		// (BASE_BFUNC_TOKEN + 181)		// BFTKN_LVS_DETAILS
#define	BFTKN_PWR_MY_CALENDAR				342		// (BASE_BFUNC_TOKEN + 182)		// BFTKN_PWR_MY_CALENDAR
#define	BFTKN_PWR_SEND_MAIL					343		// (BASE_BFUNC_TOKEN + 183)		// BFTKN_PWR_SEND_MAIL
#define	BFTKN_PWR_SCHEDULE					344		// (BASE_BFUNC_TOKEN + 184)		// BFTKN_PWR_SCHEDULE
#define	BFTKN_PWR_ASSIGN_TASK				345		// (BASE_BFUNC_TOKEN + 185)		// BFTKN_PWR_ASSIGN_TASK
#define	BFTKN_PWR_WRITE_NOTE				346		// (BASE_BFUNC_TOKEN + 186)		// BFTKN_PWR_WRITE_NOTE
#define	BFTKN_PWR_PHONE_MESSAGE				347		// (BASE_BFUNC_TOKEN + 187)		// BFTKN_PWR_PHONE_MESSAGE
#define	BFTKN_DM_ENDRETRIEVE				348		// (BASE_BFUNC_TOKEN + 188)		// BFTKN_DM_ENDRETRIEVE
#define	BFTKN_VIEWBY_PROJECT_CAL			349		// (BASE_BFUNC_TOKEN + 189)		// BFTKN_VIEWBY_PROJECT_CAL
#define	BFTKN_DM_VERSION_LIST				350		// (BASE_BFUNC_TOKEN + 190)		// BFTKN_DM_VERSION_LIST
#define	BFTKN_DM_DELETE						351		// (BASE_BFUNC_TOKEN + 191)		// BFTKN_DM_DELETE
#define	BFTKN_READ_LATER					352		// (BASE_BFUNC_TOKEN + 192)		// BFTKN_READ_LATER
#define BFTKN_SYNC_ACCOUNT_MENU_2           353 	// (BASE_BFUNC_TOKEN + 193) 	// BFTKN_SYNC_ACCOUNT_MENU_2
#define	BFTKN_LVS_MSG_THREAD				354		// (BASE_BFUNC_TOKEN + 194)		// BFTKN_LVS_MSG_THREAD
#define	BFTKN_FILTER_CLEAR					355		// (BASE_BFUNC_TOKEN + 195)		// BFTKN_FILTER_CLEAR
#define	BFTKN_PARENT						356		// (BASE_BFUNC_TOKEN + 196)		// BFTKN_PARENT
#define	BFTKN_DROP_DEL_FOLDERSANDITEMS		357		// (BASE_BFUNC_TOKEN + 197)		// BFTKN_DROP_DEL_FOLDERSANDITEMS
#define	BFTKN_SPLIT_BAR						358		// (BASE_BFUNC_TOKEN + 198)		// BFTKN_SPLIT_BAR
#define	BFTKN_SEND_COPY						359		// (BASE_BFUNC_TOKEN + 199)		// BFTKN_SEND_COPY
#define	BFTKN_VIEW_ATTACH					360		// (BASE_BFUNC_TOKEN + 200)		// BFTKN_VIEW_ATTACH
#define	BFTKN_MAYBE_VIEW_ATTACH				361		// (BASE_BFUNC_TOKEN + 201)		// BFTKN_MAYBE_VIEW_ATTACH
#define	BFTKN_TEXTENTRY_DELETE_CURWORD		362		// (BASE_BFUNC_TOKEN + 202)		// BFTKN_TEXTENTRY_DELETE_CURWORD
#define	BFTKN_SELECT_FOLDERS				363		// (BASE_BFUNC_TOKEN + 203)		// BFTKN_SELECT_FOLDERS
#define	BFTKN_SET_STATE						364		// (BASE_BFUNC_TOKEN + 204)		// BFTKN_SET_STATE
#define	BFTKN_DROP_INTO_FOLDER				365		// (BASE_BFUNC_TOKEN + 205)		// BFTKN_DROP_INTO_FOLDER
#define	BFTKN_DROP_ABOVE_FOLDER				366		// (BASE_BFUNC_TOKEN + 206)		// BFTKN_DROP_ABOVE_FOLDER
#define	BFTKN_DROP_BELOW_FOLDER				367		// (BASE_BFUNC_TOKEN + 207)		// BFTKN_DROP_BELOW_FOLDER
#define	BFTKN_WPCHAR_BOX					368		// (BASE_BFUNC_TOKEN + 208)		// BFTKN_WPCHAR_BOX
#define	BFTKN_READ_2						369		// (BASE_BFUNC_TOKEN + 209)		// BFTKN_READ_2
#define	BFTKN_PLUS							370		// (BASE_BFUNC_TOKEN + 210)		// BFTKN_PLUS
#define	BFTKN_CONTROL_PLUS					371		// (BASE_BFUNC_TOKEN + 211)		// BFTKN_CONTROL_PLUS
#define	BFTKN_MINUS							372		// (BASE_BFUNC_TOKEN + 212)		// BFTKN_MINUS
#define	BFTKN_CONTROL_MINUS					373		// (BASE_BFUNC_TOKEN + 213)		// BFTKN_CONTROL_MINUS
#define	BFTKN_SET_PROMPT					374		// (BASE_BFUNC_TOKEN + 214)		// BFTKN_SET_PROMPT
#define	BFTKN_DROP_CANCEL					375		// (BASE_BFUNC_TOKEN + 215)		// BFTKN_DROP_CANCEL
#define	BFTKN_SELECT_ALL					376		// (BASE_BFUNC_TOKEN + 216)		// BFTKN_SELECT_ALL
#define	BFTKN_IMEOFF						377		// (BASE_BFUNC_TOKEN + 217)		// BFTKN_IMEOFF
#define	BFTKN_IMEON							378		// (BASE_BFUNC_TOKEN + 218)		// BFTKN_IMEON
#define	BFTKN_TANGO_TOROKU					379		// (BASE_BFUNC_TOKEN + 219)		// BFTKN_TANGO_TOROKU
#define	BFTKN_INBOX_FLT						380		// (BASE_BFUNC_TOKEN + 220)		// BFTKN_INBOX_FLT
#define	BFTKN_REM_VIEWCONNECTIONLOG			381		// (BASE_BFUNC_TOKEN + 221)		// BFTKN_REM_VIEWCONNECTIONLOG
#define	BFTKN_REM_DISCONNECT				382		// (BASE_BFUNC_TOKEN + 222)		// BFTKN_REM_DISCONNECT
#define	BFTKN_OUTBOX_FLT					383		// (BASE_BFUNC_TOKEN + 223)		// BFTKN_OUTBOX_FLT
#define	BFTKN_PERS_FLT						384		// (BASE_BFUNC_TOKEN + 224)		// BFTKN_PERS_FLT
#define	BFTKN_SYNC_ACCOUNT_MENU_3			385		// (BASE_BFUNC_TOKEN + 225)		// BFTKN_SYNC_ACCOUNT_MENU_3
#define	BFTKN_NULL_TOKEN					386		// (BASE_BFUNC_TOKEN + 226)		// BFTKN_NULL_TOKEN
#define	BFTKN_DB_ENGCALLS					387		// (BASE_BFUNC_TOKEN + 227)		// BFTKN_DB_ENGCALLS
#define	BFTKN_NEXT_THREAD					388		// (BASE_BFUNC_TOKEN + 228)
#define	BFTKN_PREV_THREAD					389		// (BASE_BFUNC_TOKEN + 229)
#define	BFTKN_NEXT_UNREAD					390		// (BASE_BFUNC_TOKEN + 230)
#define	BFTKN_PREV_UNREAD					391		// (BASE_BFUNC_TOKEN + 231)
#define BFTKN_EXPAND_THREAD					392		// (BASE_BFUNC_TOKEN + 232)
#define BFTKN_COLLAPSE_THREAD				393		// (BASE_BFUNC_TOKEN + 233)
#define	BFTKN_SYNC_ACCOUNT_MENU_4			394		// (BASE_BFUNC_TOKEN + 234)		// BFTKN_SYNC_ACCOUNT_MENU_4
#define	BFTKN_SYNC_ACCOUNT_MENU_5			395		// (BASE_BFUNC_TOKEN + 235)		// BFTKN_SYNC_ACCOUNT_MENU_5
#define	BFTKN_SYNC_ACCOUNT_MENU_6			396		// (BASE_BFUNC_TOKEN + 236)		// BFTKN_SYNC_ACCOUNT_MENU_6
#define	BFTKN_SYNC_ACCOUNT_MENU_7			397		// (BASE_BFUNC_TOKEN + 237)		// BFTKN_SYNC_ACCOUNT_MENU_7
#define	BFTKN_SYNC_ACCOUNT_MENU_8			398		// (BASE_BFUNC_TOKEN + 238)		// BFTKN_SYNC_ACCOUNT_MENU_8
#define	BFTKN_SYNC_ACCOUNT_MENU_9			399		// (BASE_BFUNC_TOKEN + 239)		// BFTKN_SYNC_ACCOUNT_MENU_9
#define	BFTKN_SYNC_ACCOUNT_MENU_10			400		// (BASE_BFUNC_TOKEN + 240)		// BFTKN_SYNC_ACCOUNT_MENU_10
#define	BFTKN_SYNC_ACCOUNT_MENU_MORE		401		// (BASE_BFUNC_TOKEN + 241)		// BFTKN_SYNC_ACCOUNT_MENU_MORE
#define	BFTKN_SEND_USING_MENU_1				402		// (BASE_BFUNC_TOKEN + 242)		// BFTKN_SEND_USING_MENU_1
#define	BFTKN_NEW_BROWSER					403		// (BASE_BFUNC_TOKEN + 243)		// BFTKN_NEW_BROWSER
#define	BFTKN_DROP_DEL_ITEMS				404		// (BASE_BFUNC_TOKEN + 244)		// BFTKN_DROP_DEL_ITEMS
#define	BFTKN_DROP_UNDEL_ITEMS				405		// (BASE_BFUNC_TOKEN + 245)		// BFTKN_DROP_UNDEL_ITEMS
#define	BFTKN_DROP_DEL_ITEMSONLY			406		// (BASE_BFUNC_TOKEN + 246)		// BFTKN_DROP_DEL_ITEMSONLY
#define	BFTKN_SHOW_LOOKER					407		// (BASE_BFUNC_TOKEN + 247)		// BFTKN_SHOW_LOOKER
#define	BFTKN_SHOW_FOLDERS					408		// (BASE_BFUNC_TOKEN + 248)		// BFTKN_SHOW_FOLDERS
#define	BFTKN_VIEW_ITEM						409		// (BASE_BFUNC_TOKEN + 249)		// BFTKN_DM_VIEW
#define	BFTKN_DRAFT_FLT						410		// (BASE_BFUNC_TOKEN + 250)		// BFTKN_DRAFT_FLT
#define	BFTKN_DROP_LINK_TO					411		// (BASE_BFUNC_TOKEN + 251) 	// BFTKN_DROP_LINK_TO
#define	BFTKN_DROP_DEL_AND_EMPTY_ITEMS		412		// (BASE_BFUNC_TOKEN + 252) 	// BFTKN_DROP_DEL_AND_EMPTY_ITEMS
#define	BFTKN_DROP_DEL_DOC_REF				413		// (BASE_BFUNC_TOKEN + 253) 	// BFTKN_DROP_DEL_DOC_REF
#define	BFTKN_DROP_DEL_DOCUMENT				414		// (BASE_BFUNC_TOKEN + 254) 	// BFTKN_DROP_DEL_DOCUMENT
#define	BFTKN_DROP_DEL_FROM_ALL_CALS		415		// (BASE_BFUNC_TOKEN + 255) 	// BFTKN_DROP_DEL_FROM_ALL_CALS
#define	BFTKN_DROP_DEL_FROM_ALL_MAILBOXES	416		// (BASE_BFUNC_TOKEN + 256) 	// BFTKN_DROP_DEL_FROM_ALL_MAILBOXES
#define	BFTKN_DROP_DEL_FROM_CAL				417		// (BASE_BFUNC_TOKEN + 257) 	// BFTKN_DROP_DEL_FROM_CAL
#define	BFTKN_DROP_DEL_FROM_MAILBOX			418		// (BASE_BFUNC_TOKEN + 258) 	// BFTKN_DROP_DEL_FROM_MAILBOX
#define	BFTKN_DROP_DEL_FROM_RECIPIENTS_CAL	419		// (BASE_BFUNC_TOKEN + 259) 	// BFTKN_DROP_DEL_FROM_RECIPIENTS_CAL
#define	BFTKN_DROP_DEL_FROM_RECIPIENTS_MAILBOX	420		// (BASE_BFUNC_TOKEN + 260) 	// BFTKN_DROP_DEL_FROM_RECIPIENTS_MAILBOX
#define	BFTKN_DROP_DEL_DOC_VER				421		// (BASE_BFUNC_TOKEN + 261)		// BFTKN_DROP_DEL_DOC_VER
#define	BFTKN_SEND_USING_MENU_2				422		// (BASE_BFUNC_TOKEN + 262)		// BFTKN_SEND_USING_MENU_2
#define BFTKN_SHOW_HELPIDS					423 	// (BASE_BFUNC_TOKEN + 263)
#define	BFTKN_INBOX_FLT_ONLY				424		// (BASE_BFUNC_TOKEN + 264)
#define	BFTKN_OUTBOX_FLT_ONLY				425		// (BASE_BFUNC_TOKEN + 265)
#define	BFTKN_PERS_FLT_ONLY					426		// (BASE_BFUNC_TOKEN + 266)
#define	BFTKN_DRAFT_FLT_ONLY				427		// (BASE_BFUNC_TOKEN + 267)
#define	BFTKN_DM_MARK_OFFICIAL_VER			428		// (BASE_BFUNC_TOKEN + 268)
#define	BFTKN_WT_TEXT_LANGUAGE				429		// (BASE_BFUNC_TOKEN + 269) 	// BFTKN_WT_TEXT_LANGUAGE
#define	BFTKN_QUICK_CORRECT					430		// (BASE_BFUNC_TOKEN + 270) 	// BFTKN_QUICK_CORRECT
#define	BFTKN_WRITING_TOOLS					431		// (BASE_BFUNC_TOKEN + 271) 	// BFTKN_WRITING_TOOLS
#define	BFTKN_CONV_PLACE					432		// (BASE_BFUNC_TOKEN + 272) 	// BFTKN_CONV_PLACE
#define	BFTKN_DIAL_SENDER					433		// (BASE_BFUNC_TOKEN + 273) 	// BFTKN_DIAL_SENDER
#define	BFTKN_ITEM_CHANGETO_MAIL			434		// (BASE_BFUNC_TOKEN + 274) 	// BFTKN_ITEM_CHANGETO_MAIL
#define	BFTKN_ITEM_CHANGETO_APPT			435		// (BASE_BFUNC_TOKEN + 275) 	// BFTKN_ITEM_CHANGETO_APPT
#define	BFTKN_ITEM_CHANGETO_TASK			436		// (BASE_BFUNC_TOKEN + 276) 	// BFTKN_ITEM_CHANGETO_TASK
#define	BFTKN_ITEM_CHANGETO_NOTE			437		// (BASE_BFUNC_TOKEN + 277)		// BFTKN_ITEM_CHANGETO_NOTE
#define	BFTKN_ITEM_CHANGETO_PHONE			438		// (BASE_BFUNC_TOKEN + 278) 	// BFTKN_ITEM_CHANGETO_PHONE
#define	BFTKN_ITEM_CHANGETO_COPY			439		// (BASE_BFUNC_TOKEN + 279) 	// BFTKN_ITEM_CHANGETO_COPY	
#define	BFTKN_ITEM_CHANGETO_MOVE			440		// (BASE_BFUNC_TOKEN + 280) 	// BFTKN_ITEM_CHANGETO_MOVE	
#define	BFTKN_HIDE_LOOKER					441 	// (BASE_BFUNC_TOKEN + 281)		// BFTKN_HIDE_LOOKER
#define	BFTKN_SEND_USING_MENU_3				442		// (BASE_BFUNC_TOKEN + 282) 	// BFTKN_SEND_USING_MENU_3
#define	BFTKN_ATTACH_PASTESPECIAL			443		// (BASE_BFUNC_TOKEN + 283)		// BFTKN_ATTACH_PASTESPECIAL
#define	BFTKN_ITEM_LIST_EMPTY_TRASH			444		// (BASE_BFUNC_TOKEN + 284) 	// BFTKN_ITEM_LIST_EMPTY_TRASH
#define	BFTKN_DROP_CHANGETO					445		// (BASE_BFUNC_TOKEN + 285) 	// BFTKN_DROP_CHANGETO
#define	BFTKN_UNSFMT_VIEWASANSI				446		// (BASE_BFUNC_TOKEN + 286) 	// BFTKN_UNSFMT_VIEWASANSI
#define	BFTKN_UNSFMT_VIEWASHEX				447		// (BASE_BFUNC_TOKEN + 287) 	// BFTKN_UNSFMT_VIEWASHEX
#define	BFTKN_UNSFMT_QUICKVIEW				448		// (BASE_BFUNC_TOKEN + 288) 	// BFTKN_UNSFMT_QUICKVIEW
#define	BFTKN_DROP_IMPORT_MOVE				449		// (BASE_BFUNC_TOKEN + 289)		// BFTKN_DROP_IMPORT_MOVE
#define	BFTKN_DROP_IMPORT_COPY				450		// (BASE_BFUNC_TOKEN + 290)		// BFTKN_DROP_IMPORT_COPY
#define	BFTKN_DROP_IMPORT_CANCEL			451		// (BASE_BFUNC_TOKEN + 291)		// BFTKN_DROP_IMPORT_CANCEL
#define	BFTKN_FILTER_MENU_1					452		// (BASE_BFUNC_TOKEN + 292)
#define	BFTKN_FILTER_MENU_2					453		// (BASE_BFUNC_TOKEN + 293)
#define	BFTKN_FILTER_MENU_3					454		// (BASE_BFUNC_TOKEN + 294)
#define	BFTKN_FILTER_MENU_4					455		// (BASE_BFUNC_TOKEN + 295)
#define	BFTKN_FILTER_MENU_5					456		// (BASE_BFUNC_TOKEN + 296)
#define	BFTKN_FILTER_MENU_6					457		// (BASE_BFUNC_TOKEN + 297)
#define	BFTKN_FILTER_MENU_7					458		// (BASE_BFUNC_TOKEN + 298)
#define	BFTKN_FILTER_MENU_8					459		// (BASE_BFUNC_TOKEN + 299)
#define	BFTKN_FILTER_MENU_9					460		// (BASE_BFUNC_TOKEN + 300)
#define	BFTKN_FILTER_MENU_10				461		// (BASE_BFUNC_TOKEN + 301)
#define	BFTKN_FILTER_MENU_11				462		// (BASE_BFUNC_TOKEN + 302)
#define	BFTKN_FILTER_MENU_12				463		// (BASE_BFUNC_TOKEN + 303)
#define	BFTKN_FILTER_MENU_13				464		// (BASE_BFUNC_TOKEN + 304)
#define	BFTKN_FILTER_MENU_14				465		// (BASE_BFUNC_TOKEN + 305)
#define	BFTKN_FILTER_MENU_15				466		// (BASE_BFUNC_TOKEN + 306)
#define	BFTKN_FILTER_MENU_16				467		// (BASE_BFUNC_TOKEN + 307)
#define	BFTKN_FILTER_MENU_17				468		// (BASE_BFUNC_TOKEN + 308)
#define	BFTKN_FILTER_MENU_18				469		// (BASE_BFUNC_TOKEN + 309)
#define	BFTKN_FILTER_MENU_19				470		// (BASE_BFUNC_TOKEN + 310)
#define	BFTKN_FILTER_MENU_20				471		// (BASE_BFUNC_TOKEN + 311)
#define	BFTKN_FILTER_MENU_MORE				472		// (BASE_BFUNC_TOKEN + 312)
#define	BFTKN_QUERY_STOP					473		// (BASE_BFUNC_TOKEN + 313)
#define	BFTKN_FOLLOW_INTERNET_LINK			474		// (BASE_BFUNC_TOKEN + 302)		// BKTKN_FOLLOW_LINK
#define	BFTKN_HELP_INTERNET					475		// (BASE_BFUNC_TOKEN + 315)
#define	BFTKN_SEND_URL						476		// (BASE_BFUNC_TOKEN + 316)
#define	BFTKN_MARK_READ						477		// (BASE_BFUNC_TOKEN + 317)
#define	BFTKN_SET_APPT_DISP_MODE_TEXT		478		// (BASE_BFUNC_TOKEN + 318)
#define	BFTKN_SET_APPT_DISP_MODE_GRID		479		// (BASE_BFUNC_TOKEN + 319)
#define	BFTKN_SET_APPT_DISP_INTVL_00		480		// (BASE_BFUNC_TOKEN + 320)		// Internal Use Only
#define	BFTKN_SET_APPT_DISP_INTVL_05		481		// (BASE_BFUNC_TOKEN + 321)		// Internal Use Only
#define	BFTKN_SET_APPT_DISP_INTVL_10		482		// (BASE_BFUNC_TOKEN + 322)		// Internal Use Only
#define	BFTKN_SET_APPT_DISP_INTVL_15		483		// (BASE_BFUNC_TOKEN + 323)		// Internal Use Only
#define	BFTKN_SET_APPT_DISP_INTVL_20		484		// (BASE_BFUNC_TOKEN + 324)		// Internal Use Only
#define	BFTKN_SET_APPT_DISP_INTVL_30		485		// (BASE_BFUNC_TOKEN + 325)		// Internal Use Only
#define	BFTKN_SET_APPT_DISP_INTVL_60		486		// (BASE_BFUNC_TOKEN + 326)		// Internal Use Only
#define	BFTKN_SET_APPT_DISP_INTVL_120		487		// (BASE_BFUNC_TOKEN + 327)		// Internal Use Only
#define	BFTKN_SET_APPT_DISP_MODE			488		// (BASE_BFUNC_TOKEN + 328)
#define	BFTKN_SET_APPT_DISP_INTVL			489		// (BASE_BFUNC_TOKEN + 329)
#define	BFTKN_HELP_COOL_SOLUTIONS			490		// (BASE_BFUNC_TOKEN + 318)
#define	BFTKN_VIEWER_PAGEUP					491		// (BASE_BFUNC_TOKEN + 331)
#define	BFTKN_VIEWER_PAGEDOWN				492		// (BASE_BFUNC_TOKEN + 332)
#define	BFTKN_VIEWER_TOP					493		// (BASE_BFUNC_TOKEN + 333)
#define	BFTKN_VIEWER_BOTTOM					494		// (BASE_BFUNC_TOKEN + 334)
#define	BFTKN_VIEW_NEXT_ATTACH				495		// (BASE_BFUNC_TOKEN + 335)
#define	BFTKN_VIEW_PREV_ATTACH				496		// (BASE_BFUNC_TOKEN + 336)
#define	BFTKN_DIGITAL_SIGN					497		// (BASE_BFUNC_TOKEN + 337)
#define	BFTKN_ENCRYPT						498		// (BASE_BFUNC_TOKEN + 338)
#define	BFTKN_SHOWAS_FREE					499		// (BASE_BFUNC_TOKEN + 339)
#define	BFTKN_SHOWAS_TENTATIVE				500		// (BASE_BFUNC_TOKEN + 340)
#define	BFTKN_SHOWAS_BUSY					501		// (BASE_BFUNC_TOKEN + 341)
#define	BFTKN_SHOWAS_OUT					502		// (BASE_BFUNC_TOKEN + 342)
#define	BFTKN_SHOWAS						503		// (BASE_BFUNC_TOKEN + 343)

#define	TOP_BFUNC_TOKEN						BFTKN_SHOWAS
#define	BFUNC_TOKEN_COUNT					(TOP_BFUNC_TOKEN - BASE_BFUNC_TOKEN + 1)

////////////////////////////////////////////////////////////////////////////////
//	Token IDs
//	- OLE Verb Token Values
////////////////////////////////////////////////////////////////////////////////

#define BASE_OLE_TOKEN					504

#define	OLETKN_OBJECT					504 	//(BASE_OLE_TOKEN + 0)		// OLETKN_OBJECT
#define	OLETKN_UNASSIGNED_1				(BASE_OLE_TOKEN + 1)
#define	OLETKN_UNASSIGNED_2				(BASE_OLE_TOKEN + 2)
#define	OLETKN_UNASSIGNED_3				(BASE_OLE_TOKEN + 3)
#define	OLETKN_UNASSIGNED_4				(BASE_OLE_TOKEN + 4)
#define	OLETKN_UNASSIGNED_5				(BASE_OLE_TOKEN + 5)
#define	OLETKN_UNASSIGNED_6				(BASE_OLE_TOKEN + 6)
#define	OLETKN_UNASSIGNED_7				(BASE_OLE_TOKEN + 7)
#define	OLETKN_UNASSIGNED_8				(BASE_OLE_TOKEN + 8)
#define	OLETKN_UNASSIGNED_9				(BASE_OLE_TOKEN + 9)
#define	OLETKN_UNASSIGNED_10			(BASE_OLE_TOKEN + 10)
#define	OLETKN_UNASSIGNED_11			(BASE_OLE_TOKEN + 11)
#define	OLETKN_UNASSIGNED_12			(BASE_OLE_TOKEN + 12)
#define	OLETKN_UNASSIGNED_13			(BASE_OLE_TOKEN + 13)
#define	OLETKN_UNASSIGNED_14			(BASE_OLE_TOKEN + 14)
#define	OLETKN_UNASSIGNED_15			(BASE_OLE_TOKEN + 15)
#define	OLETKN_UNASSIGNED_16			(BASE_OLE_TOKEN + 16)
#define	OLETKN_UNASSIGNED_17			(BASE_OLE_TOKEN + 17)
#define	OLETKN_UNASSIGNED_18			(BASE_OLE_TOKEN + 18)
#define	OLETKN_UNASSIGNED_19			(BASE_OLE_TOKEN + 19)
#define	OLETKN_UNASSIGNED_20			(BASE_OLE_TOKEN + 20)
#define	OLETKN_UNASSIGNED_21			(BASE_OLE_TOKEN + 21)
#define	OLETKN_UNASSIGNED_22			(BASE_OLE_TOKEN + 22)
#define	OLETKN_UNASSIGNED_23			(BASE_OLE_TOKEN + 23)
#define	OLETKN_UNASSIGNED_24			(BASE_OLE_TOKEN + 24)
#define	OLETKN_UNASSIGNED_25			(BASE_OLE_TOKEN + 25)
#define	OLETKN_UNASSIGNED_26			(BASE_OLE_TOKEN + 26)
#define	OLETKN_UNASSIGNED_27			(BASE_OLE_TOKEN + 27)
#define	OLETKN_UNASSIGNED_28			(BASE_OLE_TOKEN + 28)
#define	OLETKN_UNASSIGNED_29			(BASE_OLE_TOKEN + 29)
#define	OLETKN_UNASSIGNED_30			(BASE_OLE_TOKEN + 30)
#define	OLETKN_UNASSIGNED_31			(BASE_OLE_TOKEN + 31)
#define	OLETKN_UNASSIGNED_32			(BASE_OLE_TOKEN + 32)
#define	OLETKN_UNASSIGNED_33			(BASE_OLE_TOKEN + 33)
#define	OLETKN_UNASSIGNED_34			(BASE_OLE_TOKEN + 34)
#define	OLETKN_UNASSIGNED_35			(BASE_OLE_TOKEN + 35)
#define	OLETKN_UNASSIGNED_36 			(BASE_OLE_TOKEN + 36)
#define	OLETKN_UNASSIGNED_37			(BASE_OLE_TOKEN + 37)
#define	OLETKN_UNASSIGNED_38			(BASE_OLE_TOKEN + 38)
#define	OLETKN_UNASSIGNED_39			(BASE_OLE_TOKEN + 39)
#define	OLETKN_UNASSIGNED_40			(BASE_OLE_TOKEN + 40)
#define	OLETKN_UNASSIGNED_41			(BASE_OLE_TOKEN + 41)
#define	OLETKN_UNASSIGNED_42			(BASE_OLE_TOKEN + 42)
#define	OLETKN_UNASSIGNED_43			(BASE_OLE_TOKEN + 43)
#define	OLETKN_UNASSIGNED_44			(BASE_OLE_TOKEN + 44)
#define	OLETKN_UNASSIGNED_45			(BASE_OLE_TOKEN + 45)
#define	OLETKN_UNASSIGNED_46			(BASE_OLE_TOKEN + 46)
#define	OLETKN_UNASSIGNED_47			(BASE_OLE_TOKEN + 47)
#define	OLETKN_UNASSIGNED_48			(BASE_OLE_TOKEN + 48)
#define	OLETKN_UNASSIGNED_49			(BASE_OLE_TOKEN + 49)

#define	TOP_OLE_TOKEN					OLETKN_UNASSIGNED_49
#define	OLE_TOKEN_COUNT				(TOP_OLE_TOKEN - BASE_OLE_TOKEN + 1)


////////////////////////////////////////////////////////////////////////////////
//	Token IDs
//	- Advanced Functions Token Values
////////////////////////////////////////////////////////////////////////////////

#define BASE_AFUNC_TOKEN		554

#define	AFTKN_OPEN_VIEW_TYPE					554		// (BASE_AFUNC_TOKEN + 0)			// AFTKN_OPEN_VIEW_TYPE
#define	AFTKN_GOTO_DATE_ABS						556		// (BASE_AFUNC_TOKEN + 2)			// AFTKN_GOTO_DATE_ABS
#define	AFTKN_BTNBAR_OPTIONS					559		// (BASE_AFUNC_TOKEN + 5)			// AFTKN_BTNBAR_OPTIONS
#define	AFTKN_BTNBAR_SAVE						560		// (BASE_AFUNC_TOKEN + 6)			// AFTKN_BTNBAR_SAVE
#define	AFTKN_BTNBAR_SELECT						561		// (BASE_AFUNC_TOKEN + 7)			// AFTKN_BTNBAR_SELECT
#define	AFTKN_CREATE_FOLDER						562		// (BASE_AFUNC_TOKEN + 8)			// AFTKN_CREATE_FOLDER
#define	AFTKN_SETUP								563		// (BASE_AFUNC_TOKEN + 9)			// AFTKN_SETUP
#define	AFTKN_DELETE_FOLDER						564		// (BASE_AFUNC_TOKEN + 10)			// AFTKN_DELETE_FOLDER
#define	AFTKN_RENAME_FOLDER						565		// (BASE_AFUNC_TOKEN + 11)			// AFTKN_RENAME_FOLDER
#define	AFTKN_SET_TOTEXT						566		// (BASE_AFUNC_TOKEN + 12)			// AFTKN_SET_TOTEXT
#define	AFTKN_SET_FROMTEXT						567		// (BASE_AFUNC_TOKEN + 13)			// AFTKN_SET_FROMTEXT
#define	AFTKN_SET_CCTEXT						568		// (BASE_AFUNC_TOKEN + 14)			// AFTKN_SET_CCTEXT
#define	AFTKN_SET_BCTEXT						569		// (BASE_AFUNC_TOKEN + 15)			// AFTKN_SET_BCTEXT
#define	AFTKN_SET_SUBJTEXT						570		// (BASE_AFUNC_TOKEN + 16)			// AFTKN_SET_SUBJTEXT
#define	AFTKN_SET_MSGTEXT						571		// (BASE_AFUNC_TOKEN + 17)			// AFTKN_SET_MSGTEXT
#define	AFTKN_ADD_ATTACHMENT					572		// (BASE_AFUNC_TOKEN + 18)			// AFTKN_ADD_ATTACHMENT
#define	AFTKN_SET_AUTHORITYTEXT					573		// (BASE_AFUNC_TOKEN + 19)			// AFTKN_SET_AUTHORITYTEXT
#define	AFTKN_SET_PLACETEXT						574		// (BASE_AFUNC_TOKEN + 20)			// AFTKN_SET_PLACETEXT
#define	AFTKN_SET_NAMETEXT						575		// (BASE_AFUNC_TOKEN + 21)			// AFTKN_SET_NAMETEXT
#define	AFTKN_SET_COMPANYTEXT					576		// (BASE_AFUNC_TOKEN + 22)			// AFTKN_SET_COMPANYTEXT
#define	AFTKN_SET_PHONETEXT						577		// (BASE_AFUNC_TOKEN + 23)			// AFTKN_SET_PHONETEXT
#define	AFTKN_SEND_MESSAGE						578		// (BASE_AFUNC_TOKEN + 24)			// AFTKN_SEND_MESSAGE
#define	AFTKN_SEND_NOTE							579		// (BASE_AFUNC_TOKEN + 25)			// AFTKN_SEND_NOTE
#define	AFTKN_SEND_TODO							580		// (BASE_AFUNC_TOKEN + 26)			// AFTKN_SEND_TODO
#define	AFTKN_SEND_APPT							581		// (BASE_AFUNC_TOKEN + 27)			// AFTKN_SEND_APPT
#define	AFTKN_WRITE_FILE						582		// (BASE_AFUNC_TOKEN + 28)			// AFTKN_WRITE_FILE
#define	AFTKN_SET_FONT							583		// (BASE_AFUNC_TOKEN + 29)			// AFTKN_SET_FONT
#define	AFTKN_SET_DATETIMEDUR					584		// (BASE_AFUNC_TOKEN + 30)			// AFTKN_SET_DATETIMEDUR
#define	AFTKN_PARSE_DATE						585		// (BASE_AFUNC_TOKEN + 31)			// AFTKN_PARSE_DATE
#define	AFTKN_FILTER_FROM_FILE					586		// (BASE_AFUNC_TOKEN + 32)			// AFTKN_FILTER_FROM_FILE
#define	AFTKN_OPEN_VIEW_NAMED					587		// (BASE_AFUNC_TOKEN + 33)			// AFTKN_OPEN_VIEW_NAMED
#define	AFTKN_OPEN_VIEW_FILE					588		// (BASE_AFUNC_TOKEN + 34)			// AFTKN_OPEN_VIEW_FILE
#define	AFTKN_ADD_ENCAP_ATTACH					589		// (BASE_AFUNC_TOKEN + 35)			// AFTKN_ADD_ENCAP_ATTACH
#define	AFTKN_SET_CAT_PRIOR 					590		// (BASE_AFUNC_TOKEN + 36)			// AFTKN_SET_CAT_PRIOR
#define	AFTKN_SAVE_AS							591		// (BASE_AFUNC_TOKEN + 37)			// AFTKN_SAVE_AS
#define	AFTKN_GET_MAILDATA						592		// (BASE_AFUNC_TOKEN + 38)			// AFTKN_GET_MAILDATA
#define	AFTKN_READ_FILE							593		// (BASE_AFUNC_TOKEN + 39)			// AFTKN_READ_FILE
#define	AFTKN_SET_ADV							594		// (BASE_AFUNC_TOKEN + 40)			// AFTKN_SET_ADV
#define	AFTKN_SET_ALARM							595		// (BASE_AFUNC_TOKEN + 41)			// AFTKN_SET_ALARM
#define	AFTKN_SET_MESSAGE						596		// (BASE_AFUNC_TOKEN + 42)			// AFTKN_SET_MESSAGE
#define	AFTKN_SET_APPT							597		// (BASE_AFUNC_TOKEN + 43)			// AFTKN_SET_APPT
#define	AFTKN_SET_NOTE							598		// (BASE_AFUNC_TOKEN + 44)			// AFTKN_SET_NOTE
#define	AFTKN_SET_TODO							599		// (BASE_AFUNC_TOKEN + 45)			// AFTKN_SET_TODO
#define	AFTKN_CREATE_FOLDER_IMAP				600		// (BASE_AFUNC_TOKEN + 46)			// AFTKN_CREATE_FOLDER_IMAP
#define AFTKN_UNASSIGNED_46						600		// (BASE_AFUNC_TOKEN + 46)			// AFTKN_UNASSIGNED
#define	AFTKN_SET_ABENTRY						601		// (BASE_AFUNC_TOKEN + 47)			// AFTKN_SET_ABENTRY
#define	AFTKN_TYPE								602		// (BASE_AFUNC_TOKEN + 48)			// AFTKN_TYPE
#define	AFTKN_TYPE_CHAR							603		// (BASE_AFUNC_TOKEN + 49)			// AFTKN_TYPE_CHAR
#define	AFTKN_SEND_PHONE						604		// (BASE_AFUNC_TOKEN + 50)			// AFTKN_SEND_PHONE
#define	AFTKN_ADD_PERS_ATTACH					605		// (BASE_AFUNC_TOKEN + 51)			// AFTKN_ADD_PERS_ATTACH
#define	AFTKN_GET_ABDATA						606		// (BASE_AFUNC_TOKEN + 52)			// AFTKN_GET_ABDATA
#define	AFTKN_DELEGATE							607		// (BASE_AFUNC_TOKEN + 53)			// AFTKN_DELEGATE
#define	AFTKN_SET_VEW							608		// (BASE_AFUNC_TOKEN + 54)			// AFTKN_SET_VEW
#define	AFTKN_PRINT_CAL							609		// (BASE_AFUNC_TOKEN + 55)			// AFTKN_PRINT_CAL
#define	AFTKN_USER_FUNCTION						610		// (BASE_AFUNC_TOKEN + 56)			// AFTKN_USER_FUNCTION
#define	AFTKN_SET_ENV							611		// (BASE_AFUNC_TOKEN + 57)			// AFTKN_SET_ENV
#define	AFTKN_SET_DTM							613		// (BASE_AFUNC_TOKEN + 59)			// AFTKN_SET_DTM
#define	AFTKN_DATE_PARSE_MONTH					614		// (BASE_AFUNC_TOKEN + 60)			// AFTKN_DATE_PARSE_MONTH
#define	AFTKN_SET_SCH							615		// (BASE_AFUNC_TOKEN + 61)			// AFTKN_SET_SCH
#define	AFTKN_SET_LOC							616		// (BASE_AFUNC_TOKEN + 62)			// AFTKN_SET_LOC
#define	AFTKN_SETFOCUS							617		// (BASE_AFUNC_TOKEN + 63)			// AFTKN_SETFOCUS
#define	AFTKN_OPEN_MSG							618		// (BASE_AFUNC_TOKEN + 64)			// AFTKN_OPEN_MSG
#define	AFTKN_INSERT_OBJECT						619		// (BASE_AFUNC_TOKEN + 65)			// AFTKN_INSERT_OBJECT
#define	AFTKN_DOVERB							620		// (BASE_AFUNC_TOKEN + 66)			// AFTKN_DOVERB
#define	AFTKN_MOVETO_FOLDER						621		// (BASE_AFUNC_TOKEN + 67)			// AFTKN_MOVETO_FOLDER
#define	AFTKN_LINKTO_FOLDER						622		// (BASE_AFUNC_TOKEN + 68)			// AFTKN_LINKTO_FOLDER
#define	AFTKN_ALTMOVETO_FOLDER					623		// (BASE_AFUNC_TOKEN + 69)			// AFTKN_ALTMOVETO_FOLDER
#define	AFTKN_CHANGE_USER						624		// (BASE_AFUNC_TOKEN + 70)			// AFTKN_CHANGE_USER
#define	AFTKN_DATE_PARSE_YEAR					625		// (BASE_AFUNC_TOKEN + 71)			// AFTKN_DATE_PARSE_YEAR
#define	AFTKN_SEARCH							627		// (BASE_AFUNC_TOKEN + 73)			// AFTKN_SEARCH
#define	AFTKN_ATTACH_INSERT_OBJECT				628		// (BASE_AFUNC_TOKEN + 74)			// AFTKN_ATTACH_INSERT_OBJECT
#define	AFTKN_CHANGE_VIEW						629		// (BASE_AFUNC_TOKEN + 75)			// AFTKN_CHANGE_VIEW
#define	AFTKN_EXECUTE_RULE						630		// (BASE_AFUNC_TOKEN + 76)			// AFTKN_EXECUTE_RULE
#define	AFTKN_SET_DSC							631		// (BASE_AFUNC_TOKEN + 77)			// AFTKN_SET_DSC
#define	AFTKN_WIND_OPEN							632		// (BASE_AFUNC_TOKEN + 78)			// AFTKN_WIND_OPEN
#define	AFTKN_SET_DEF_VIEW						633		// (BASE_AFUNC_TOKEN + 79)			// AFTKN_SET_DEF_VIEW
#define	AFTKN_DATE_PARSE_DAY_OF_WEEK			634		// (BASE_AFUNC_TOKEN + 80)			// AFTKN_DATE_PARSE_DAY_OF_WEEK	
#define	AFTKN_BUSYSRCH							635		// (BASE_AFUNC_TOKEN + 81)			// AFTKN_BUSYSRCH
#define	AFTKN_SEND_OPTIONS						636		// (BASE_AFUNC_TOKEN + 82)			// AFTKN_SEND_OPTIONS
#define	AFTKN_DDE_QUERY_LAST_ERROR				637		// (BASE_AFUNC_TOKEN + 83)			// AFTKN_DDE_QUERY_LAST_ERROR
#define	AFTKN_RULE_CREATE						638		// (BASE_AFUNC_TOKEN + 84)			// AFTKN_RULE_CREATE
#define	AFTKN_RULE_DELETE						639		// (BASE_AFUNC_TOKEN + 85)			// AFTKN_RULE_DELETE
#define	AFTKN_CREATE_FOLDER_NNTP				640		// (BASE_AFUNC_TOKEN + 86)			// AFTKN_CREATE_FOLDER_NNTP
#define	AFTKN_SET_CHECKSTATE					641		// (BASE_AFUNC_TOKEN + 87)			// AFTKN_SET_CHECKSTATE
#define	AFTKN_SET_ENUMVAL						642		// (BASE_AFUNC_TOKEN + 88)			// AFTKN_SET_ENUMVAL
#define	AFTKN_RETRIEVE							643		// (BASE_AFUNC_TOKEN + 89)			// AFTKN_RETRIEVE
#define	AFTKN_RULE_ACT_ACCEPT					644		// (BASE_AFUNC_TOKEN + 90)			// AFTKN_RULE_ACT_ACCEPT
#define	AFTKN_RULE_ACT_DECLINE					645		// (BASE_AFUNC_TOKEN + 91)			// AFTKN_RULE_ACT_DECLINE
#define	AFTKN_RULE_ACT_DELEGATE					646		// (BASE_AFUNC_TOKEN + 92)			// AFTKN_RULE_ACT_DELEGATE
#define	AFTKN_RULE_ACT_EMPTYITEM				647		// (BASE_AFUNC_TOKEN + 93)			// AFTKN_RULE_ACT_EMPTYITEM
#define	AFTKN_RULE_ACT_FORWARD					648		// (BASE_AFUNC_TOKEN + 94)			// AFTKN_RULE_ACT_FORWARD
#define	AFTKN_RULE_ACT_LINKTOFOLDER				649		// (BASE_AFUNC_TOKEN + 95)			// AFTKN_RULE_ACT_LINKTOFOLDER
#define	AFTKN_RULE_ACT_MARKPRIVATE				650		// (BASE_AFUNC_TOKEN + 96)			// AFTKN_RULE_ACT_MARKPRIVATE
#define	AFTKN_RULE_ACT_MOVETOFOLDER				651		// (BASE_AFUNC_TOKEN + 97)			// AFTKN_RULE_ACT_MOVETOFOLDER
#define	AFTKN_RULE_ACT_REPLY					652		// (BASE_AFUNC_TOKEN + 98)			// AFTKN_RULE_ACT_REPLY
#define	AFTKN_RULE_ACT_SENDMAIL					653		// (BASE_AFUNC_TOKEN + 99)			// AFTKN_RULE_ACT_SENDMAIL
#define	AFTKN_REM_SETPREFS						654		// (BASE_AFUNC_TOKEN + 100)		// AFTKN_REM_SETPREFS
#define	AFTKN_ITEM_SET_ALARM					655		// (BASE_AFUNC_TOKEN + 101)		// AFTKN_ITEM_SET_ALARM
#define	AFTKN_TEXTENTRY_LINE_CHARCOUNT			656		// (BASE_AFUNC_TOKEN + 102)		// AFTKN_TEXTENTRY_LINE_CHARCOUNT
#define	AFTKN_TEXTENTRY_GET_LINE_COUNT			657		// (BASE_AFUNC_TOKEN + 103)		// AFTKN_TEXTENTRY_GET_LINE_COUNT
#define	AFTKN_TEXTENTRY_GET_LINE_INDEX			658		// (BASE_AFUNC_TOKEN + 104)		// AFTKN_TEXTENTRY_GET_LINE_INDEX
#define	AFTKN_TEXTENTRY_GET_CHAR_INDEX			659		// (BASE_AFUNC_TOKEN + 105)		// AFTKN_TEXTENTRY_GET_CHAR_INDEX
#define	AFTKN_TEXTENTRY_GET_CURWORD				660		// (BASE_AFUNC_TOKEN + 106)		// AFTKN_TEXTENTRY_GET_CURWORD
#define	AFTKN_TEXTENTRY_GET_LEFTCHAR			661		// (BASE_AFUNC_TOKEN + 107)		// AFTKN_TEXTENTRY_GET_LEFTCHAR
#define	AFTKN_CURRENT_WINDOW					662		// (BASE_AFUNC_TOKEN + 108)		// AFTKN_CURRENT_WINDOW
#define	AFTKN_ITEM_ANNOTATION_SAVE_AS			663		// (BASE_AFUNC_TOKEN + 109)		// AFTKN_ITEM_ANNOTATION_SAVE_AS
#define	AFTKN_ITEM_ANNOTATION_COUNT				664		// (BASE_AFUNC_TOKEN + 110)		// AFTKN_ITEM_ANNOTATION_COUNT
#define	AFTKN_ITEM_ATTACHMENT_ADD				665		// (BASE_AFUNC_TOKEN + 111)		// AFTKN_ITEM_ATTACHMENT_ADD
#define	AFTKN_ITEM_ATTACHMENT_DELETE			666		// (BASE_AFUNC_TOKEN + 112)		// AFTKN_ITEM_ATTACHMENT_DELETE
#define	AFTKN_ITEM_ATTACHMENT_COUNT				667		// (BASE_AFUNC_TOKEN + 113)		// AFTKN_ITEM_ATTACHMENT_COUNT
#define	AFTKN_ITEM_ATTACHMENT_DISP_NAME			668		// (BASE_AFUNC_TOKEN + 114)		// AFTKN_ITEM_ATTACHMENT_DISP_NAME
#define	AFTKN_ITEM_ATTACHMENT_NAME				669		// (BASE_AFUNC_TOKEN + 115)		// AFTKN_ITEM_ATTACHMENT_NAME
#define	AFTKN_ITEM_ATTACHMENT_CLASS				670		// (BASE_AFUNC_TOKEN + 116)		// AFTKN_ITEM_ATTACHMENT_CLASS
#define	AFTKN_ITEM_ATTACHMENT_SAVE_AS			671		// (BASE_AFUNC_TOKEN + 117)		// AFTKN_ITEM_ATTACHMENT_SAVE_AS
#define	AFTKN_ITEM_MSGID_FROM_VIEW				672		// (BASE_AFUNC_TOKEN + 118)		// AFTKN_ITEM_MSGID_FROM_VIEW
#define	AFTKN_ITEM_GET_ATTRIBUTE				673		// (BASE_AFUNC_TOKEN + 119)		// AFTKN_ITEM_GET_ATTRIBUTE
#define	AFTKN_ITEM_GET_DATE						674		// (BASE_AFUNC_TOKEN + 120)		// AFTKN_ITEM_GET_DATE
#define	AFTKN_ITEM_GET_PRIORITY					675		// (BASE_AFUNC_TOKEN + 121)		// AFTKN_ITEM_GET_PRIORITY
#define	AFTKN_ITEM_GET_SENDERID					676		// (BASE_AFUNC_TOKEN + 122)		// AFTKN_ITEM_GET_SENDERID
#define	AFTKN_ITEM_GET_SOURCE					677		// (BASE_AFUNC_TOKEN + 123)		// AFTKN_ITEM_GET_SOURCE
#define	AFTKN_ITEM_GET_TEXT						678		// (BASE_AFUNC_TOKEN + 124)		// AFTKN_ITEM_GET_TEXT
#define	AFTKN_ITEM_GET_TYPE						679		// (BASE_AFUNC_TOKEN + 125)		// AFTKN_ITEM_GET_TYPE
#define	AFTKN_ITEM_INFO_SAVE					680		// (BASE_AFUNC_TOKEN + 126)		// AFTKN_ITEM_INFO_SAVE
#define	AFTKN_ITEM_MSGID_ISVALID				681		// (BASE_AFUNC_TOKEN + 127)		// AFTKN_ITEM_MSGID_ISVALID
#define	AFTKN_ITEM_OPEN							682		// (BASE_AFUNC_TOKEN + 128)		// AFTKN_ITEM_OPEN
#define	AFTKN_ITEM_SAVE_MESSAGE					683		// (BASE_AFUNC_TOKEN + 129)		// AFTKN_ITEM_SAVE_MESSAGE
#define	AFTKN_ITEM_SAVE_VIEW					684		// (BASE_AFUNC_TOKEN + 130)		// AFTKN_ITEM_SAVE_VIEW
#define	AFTKN_ITEM_SET_ATTRIBUTE				685		// (BASE_AFUNC_TOKEN + 131)		// AFTKN_ITEM_SET_ATTRIBUTE
#define	AFTKN_ITEM_SET_PRIORITY					686		// (BASE_AFUNC_TOKEN + 132)		// AFTKN_ITEM_SET_PRIORITY
#define	AFTKN_ITEM_SET_TEXT						687		// (BASE_AFUNC_TOKEN + 133)		// AFTKN_ITEM_SET_TEXT
#define	AFTKN_ITEM_SET_TYPE						688		// (BASE_AFUNC_TOKEN + 134)		// AFTKN_ITEM_SET_TYPE
#define	AFTKN_AL_CREATE							689		// (BASE_AFUNC_TOKEN + 135)		// AFTKN_AL_CREATE
#define	AFTKN_AL_DELETE							690		// (BASE_AFUNC_TOKEN + 136)		// AFTKN_AL_DELETE
#define	AFTKN_AL_GET_ADDRESS					691		// (BASE_AFUNC_TOKEN + 137)		// AFTKN_AL_GET_ADDRESS
#define	AFTKN_AL_GET_COUNT						692		// (BASE_AFUNC_TOKEN + 138)		// AFTKN_AL_GET_COUNT
#define	AFTKN_AL_GET_ENTRY						693		// (BASE_AFUNC_TOKEN + 139)		// AFTKN_AL_GET_ENTRY
#define	AFTKN_AL_GET_FIELD						694		// (BASE_AFUNC_TOKEN + 140)		// AFTKN_AL_GET_FIELD
#define	AFTKN_ITEM_DELETE						695		// (BASE_AFUNC_TOKEN + 141)		// AFTKN_ITEM_DELETE
#define	AFTKN_ITEMLIST_CREATE					696		// (BASE_AFUNC_TOKEN + 142)		// AFTKN_ITEMLIST_CREATE
#define	AFTKN_ITEMLIST_CREATE_FM_CTRL			697		// (BASE_AFUNC_TOKEN + 143)		// AFTKN_ITEMLIST_CREATE_FM_CTRL
#define	AFTKN_ITEMLIST_DELETE					698		// (BASE_AFUNC_TOKEN + 144)		// AFTKN_ITEMLIST_DELETE
#define	AFTKN_ITEMLIST_GET_COUNT				699		// (BASE_AFUNC_TOKEN + 145)		// AFTKN_ITEMLIST_GET_COUNT
#define	AFTKN_ITEMLIST_GET_MSGID				700		// (BASE_AFUNC_TOKEN + 146)		// AFTKN_ITEMLIST_GET_MSGID
#define	AFTKN_ITEMLIST_SORT						701		// (BASE_AFUNC_TOKEN + 147)		// AFTKN_ITEMLIST_SORT
#define	AFTKN_FLDRLIST_ADD						702		// (BASE_AFUNC_TOKEN + 148)		// AFTKN_FLDRLIST_ADD
#define	AFTKN_FLDRLIST_APPLY					703		// (BASE_AFUNC_TOKEN + 149)		// AFTKN_FLDRLIST_APPLY
#define	AFTKN_FLDRLIST_CREATE					704		// (BASE_AFUNC_TOKEN + 150)		// AFTKN_FLDRLIST_CREATE
#define	AFTKN_FLDRLIST_CREATE_FROM_VIEW			705		// (BASE_AFUNC_TOKEN + 151)		// AFTKN_FLDRLIST_CREATE_FROM_VIEW
#define	AFTKN_FLDRLIST_DELETE					706		// (BASE_AFUNC_TOKEN + 152)		// AFTKN_FLDRLIST_DELETE
#define	AFTKN_FLDRLIST_GET_COUNT				707		// (BASE_AFUNC_TOKEN + 153)		// AFTKN_FLDRLIST_GET_COUNT
#define	AFTKN_FLDRLIST_GET_NAME					708		// (BASE_AFUNC_TOKEN + 154)		// AFTKN_FLDRLIST_GET_NAME
#define	AFTKN_FLDRLIST_SET						709		// (BASE_AFUNC_TOKEN + 155)		// AFTKN_FLDRLIST_SET
#define	AFTKN_AL_IS_IN_SECTION					710		// (BASE_AFUNC_TOKEN + 156)		// AFTKN_AL_IS_IN_SECTION
#define	AFTKN_PROMPT_FOR_PASSWORD				711		// (BASE_AFUNC_TOKEN + 157)		// AFTKN_PROMPT_FOR_PASSWORD
#define	AFTKN_DATE_PARSE_DAY					712		// (BASE_AFUNC_TOKEN + 158)		// AFTKN_DATE_PARSE_DAY
#define	AFTKN_AB_SEARCH							713		// (BASE_AFUNC_TOKEN + 159)		// AFTKN_AB_SEARCH
#define	AFTKN_AB_SEARCH_BEGIN					714		// (BASE_AFUNC_TOKEN + 160)		// AFTKN_AB_SEARCH_BEGIN
#define	AFTKN_AB_SEARCH_END						715		// (BASE_AFUNC_TOKEN + 161)		// AFTKN_AB_SEARCH_END
#define	AFTKN_AB_SEARCH_NEXT					716		// (BASE_AFUNC_TOKEN + 162)		// AFTKN_AB_SEARCH_NEXT
#define	AFTKN_AB_SEARCH_PREV					717		// (BASE_AFUNC_TOKEN + 163)		// AFTKN_AB_SEARCH_PREV
#define	AFTKN_AB_GET_ENTRY						718		// (BASE_AFUNC_TOKEN + 164)		// AFTKN_AB_GET_ENTRY
#define	AFTKN_AB_GET_FIELD						719		// (BASE_AFUNC_TOKEN + 165)		// AFTKN_AB_GET_FIELD
#define	AFTKN_AL_SAVE_AS_PER_GROUP				720		// (BASE_AFUNC_TOKEN + 166)		// AFTKN_AL_SAVE_AS_PER_GROUP
#define	AFTKN_FILTER_APPLY						721		// (BASE_AFUNC_TOKEN + 167)		// AFTKN_FILTER_APPLY
#define	AFTKN_FILTER_CREATE						722		// (BASE_AFUNC_TOKEN + 168)		// AFTKN_FILTER_CREATE
#define	AFTKN_FILTER_DELETE						723		// (BASE_AFUNC_TOKEN + 169)		// AFTKN_FILTER_DELETE
#define	AFTKN_FILTER_SET_ATTACH_CLASS			724		// (BASE_AFUNC_TOKEN + 170)		// AFTKN_FILTER_SET_ATTACH_CLASS
#define	AFTKN_FILTER_SET_ATTRIBUTE				725		// (BASE_AFUNC_TOKEN + 171)		// AFTKN_FILTER_SET_ATTRIBUTE
#define	AFTKN_FILTER_SET_DATE_ABSOLUTE			726		// (BASE_AFUNC_TOKEN + 172)		// AFTKN_FILTER_SET_DATE_ABSOLUTE
#define	AFTKN_FILTER_SET_DATE_RELATIVE			727		// (BASE_AFUNC_TOKEN + 173)		// AFTKN_FILTER_SET_DATE_RELATIVE
#define	AFTKN_FILTER_SET_ITEMTYPE				728		// (BASE_AFUNC_TOKEN + 174)		// AFTKN_FILTER_SET_ITEMTYPE
#define	AFTKN_FILTER_SET_PRIORITY				729		// (BASE_AFUNC_TOKEN + 175)		// AFTKN_FILTER_SET_PRIORITY
#define	AFTKN_FILTER_SET_SOURCE					730		// (BASE_AFUNC_TOKEN + 176)		// AFTKN_FILTER_SET_SOURCE
#define	AFTKN_FILTER_SET_TEXT					731		// (BASE_AFUNC_TOKEN + 177)		// AFTKN_FILTER_SET_TEXT
#define	AFTKN_CSTMSG_INSTALL					732		// (BASE_AFUNC_TOKEN + 178)		// AFTKN_CSTMSG_INSTALL
#define	AFTKN_BTNBAR_SETFONT					733		// (BASE_AFUNC_TOKEN + 179)		// AFTKN_BTNBAR_SETFONT
#define	AFTKN_BTNBAR_SETLOCATION				734		// (BASE_AFUNC_TOKEN + 180)		// AFTKN_BTNBAR_SETLOCATION
#define	AFTKN_BTNBAR_SETROWS					735		// (BASE_AFUNC_TOKEN + 181)		// AFTKN_BTNBAR_SETROWS
#define	AFTKN_BTNBAR_SETSTYLE					736		// (BASE_AFUNC_TOKEN + 182)		// AFTKN_BTNBAR_SETSTYLE
#define	AFTKN_CLIPBOARD_TEXT					737		// (BASE_AFUNC_TOKEN + 183)		// AFTKN_CLIPBOARD_TEXT
#define	AFTKN_IS_REMOTE							738		// (BASE_AFUNC_TOKEN + 184)		// AFTKN_IS_REMOTE
#define	AFTKN_NETWORK_LOGIN_ID					739		// (BASE_AFUNC_TOKEN + 185)		// AFTKN_NETWORK_LOGIN_ID
#define	AFTKN_AL_CREATE_FROM_GROUP				740		// (BASE_AFUNC_TOKEN + 186)		// AFTKN_AL_CREATE_FROM_GROUP
#define	AFTKN_PREF_ARCHIVE_PATH					741		// (BASE_AFUNC_TOKEN + 187)		// AFTKN_PREF_ARCHIVE_PATH
#define	AFTKN_PREF_CUSTOM_VIEW_PATH				742		// (BASE_AFUNC_TOKEN + 188)		// AFTKN_PREF_CUSTOM_VIEW_PATH
#define	AFTKN_PREF_FULL_NAME					743		// (BASE_AFUNC_TOKEN + 189)		// AFTKN_PREF_FULL_NAME
#define	AFTKN_PREF_POST_OFFICE_PATH				745		// (BASE_AFUNC_TOKEN + 191)		// AFTKN_PREF_POST_OFFICE_PATH
#define	AFTKN_PREF_SAVE_PATH					746		// (BASE_AFUNC_TOKEN + 192)		// AFTKN_PREF_SAVE_PATH
#define	AFTKN_UNASSIGNED_193					747		// (BASE_AFUNC_TOKEN + 193)		// AVAILABLE
#define	AFTKN_VERSION_NAME						748		// (BASE_AFUNC_TOKEN + 194)		// AFTKN_VERSION_NAME
#define	AFTKN_VERSION_DATE						749		// (BASE_AFUNC_TOKEN + 195)		// AFTKN_VERSION_DATE
#define	AFTKN_AB_DEL_PER_GROUP					750		// (BASE_AFUNC_TOKEN + 196)		// AFTKN_AB_DEL_PER_GROUP
#define	AFTKN_AL_ADD							751		// (BASE_AFUNC_TOKEN + 197)		// AFTKN_AL_ADD
#define	AFTKN_AL_EDIT							752		// (BASE_AFUNC_TOKEN + 198)		// AFTKN_AL_EDIT
#define	AFTKN_ITEM_ATTACHMENT_UPDATE			753		// (BASE_AFUNC_TOKEN + 199)		// AFTKN_ITEM_ATTACHMENT_UPDATE
#define	AFTKN_CSTCMD_DELETE						754		// (BASE_AFUNC_TOKEN + 200)		// AFTKN_CSTCMD_DELETE
#define	AFTKN_CSTCMD_INSTALL					755		// (BASE_AFUNC_TOKEN + 201)		// AFTKN_CSTCMD_INSTALL
#define	AFTKN_CSTCMD_EXECUTE					756		// (BASE_AFUNC_TOKEN + 202)		// AFTKN_CSTCMD_EXECUTE
#define	AFTKN_CSTMSG_DELETE						757		// (BASE_AFUNC_TOKEN + 203)		// AFTKN_CSTMSG_DELETE
#define	AFTKN_ITEM_ATTACHMENT_CURRENT			758		// (BASE_AFUNC_TOKEN + 204)		// AFTKN_ITEM_ATTACHMENT_CURRENT
#define	AFTKN_RETURNVAL_GET						760		// (BASE_AFUNC_TOKEN + 206)		// AFTKN_RETURNVAL_GET
#define	AFTKN_PRIVATEBIF						761		// (BASE_AFUNC_TOKEN + 207)		// AFTKN_PRIVATEBIF
#define	AFTKN_PUBLICBIF							762		// (BASE_AFUNC_TOKEN + 208)		// AFTKN_PUBLICBIF
#define	AFTKN_USERID							764		// (BASE_AFUNC_TOKEN + 210)		// AFTKN_USERID
#define	AFTKN_LASTERROR							765		// (BASE_AFUNC_TOKEN + 211)		// AFTKN_LASTERROR
#define	AFTKN_INHIBIT_INPUT						766		// (BASE_AFUNC_TOKEN + 212)		// AFTKN_INHIBIT_INPUT
#define	AFTKN_ITEM_FOLDER_ALTMOVE				767		// (BASE_AFUNC_TOKEN + 213)		// AFTKN_ITEM_FOLDER_ALTMOVE
#define	AFTKN_ITEM_FOLDER_LINK					768		// (BASE_AFUNC_TOKEN + 214)		// AFTKN_ITEM_FOLDER_LINK
#define	AFTKN_ITEM_FOLDER_MOVE					769		// (BASE_AFUNC_TOKEN + 215)		// AFTKN_ITEM_FOLDER_MOVE
#define	AFTKN_COMMAND_LINE						770		// (BASE_AFUNC_TOKEN + 216)		// AFTKN_COMMAND_LINE
#define	AFTKN_LAST_WINDOW_CANCELED				771		// (BASE_AFUNC_TOKEN + 217)		// AFTKN_LAST_WINDOW_CANCELED
#define	AFTKN_IS_NETWORK_USER					772		// (BASE_AFUNC_TOKEN + 218)		// AFTKN_IS_NETWORK_USER
#define	AFTKN_ITEM_GET_RECIPIENTID				773		// (BASE_AFUNC_TOKEN + 219)		// AFTKN_ITEM_GET_RECIPIENTID
#define	AFTKN_AB_COMPARE						774		// (BASE_AFUNC_TOKEN + 220)		// AFTKN_AB_COMPARE
#define	AFTKN_ALLOW_EXIT						775		// (BASE_AFUNC_TOKEN + 221)		// AFTKN_ALLOW_EXIT
#define	AFTKN_ITEM_GET_OUTBOX_MSGID				776		// (BASE_AFUNC_TOKEN + 222)		// AFTKN_ITEM_GET_OUTBOX_MSGID
#define	AFTKN_ISREMOTECONNECTED					777		// (BASE_AFUNC_TOKEN + 223)		// AFTKN_ISREMOTECONNECTED
#define	AFTKN_ISREMCONNECTNEEDED				778		// (BASE_AFUNC_TOKEN + 224)		// AFTKN_ISREMCONNECTNEEDED
#define	AFTKN_ISREMCONNECTDONE					779		// (BASE_AFUNC_TOKEN + 225)		// AFTKN_ISREMCONNECTDONE
#define	AFTKN_REM_SETPORTINFO					780		// (BASE_AFUNC_TOKEN + 226)		// AFTKN_REM_SETPORTINFO
#define	AFTKN_REM_SETRECONCILE					781		// (BASE_AFUNC_TOKEN + 227)		// AFTKN_REM_SETRECONCILE
#define	AFTKN_ENV_ISCOMMANDVALID				783		// (BASE_AFUNC_TOKEN + 229)		// AFTKN_ENV_ISCOMMANDVALID
#define	AFTKN_ENV_CURRENTVIEWNAME				784		// (BASE_AFUNC_TOKEN + 230)		// AFTKN_ENV_CURRENTVIEWNAME
#define	AFTKN_ITEM_ARCHIVE						785		// (BASE_AFUNC_TOKEN + 231)		// AFTKN_ITEM_ARCHIVE
#define	AFTKN_REM_CREATE_ASYNC_CONNECTION		786		// (BASE_AFUNC_TOKEN + 232)		// AFTKN_REM_CREATE_ASYNC_CONNECTION
#define	AFTKN_REM_CREATE_DIRECT_CONNECTION		787		// (BASE_AFUNC_TOKEN + 233)		// AFTKN_REM_CREATE_DIRECT_CONNECTION
#define	AFTKN_ITEM_DECLINE						788		// (BASE_AFUNC_TOKEN + 234)		// AFTKN_ITEM_DECLINE
#define	AFTKN_ITEM_DELEGATE						789		// (BASE_AFUNC_TOKEN + 235)		// AFTKN_ITEM_DELEGATE
#define	AFTKN_ITEM_INFO							790		// (BASE_AFUNC_TOKEN + 236)		// AFTKN_ITEM_INFO
#define	AFTKN_ITEM_UNDELETE						791		// (BASE_AFUNC_TOKEN + 237)		// AFTKN_ITEM_UNDELETE
#define	AFTKN_ITEM_SET_DATE						792		// (BASE_AFUNC_TOKEN + 238)		// AFTKN_ITEM_SET_DATE
#define	AFTKN_AB_IS_GROUP_MEMBER				793		// (BASE_AFUNC_TOKEN + 239)		// AFTKN_AB_IS_GROUP_MEMBER
#define	AFTKN_AB_RESOLVE						794		// (BASE_AFUNC_TOKEN + 240)		// AFTKN_AB_RESOLVE
#define	AFTKN_TEXTENTRY_GET_RIGHTCHAR			795		// (BASE_AFUNC_TOKEN + 241)		// AFTKN_TEXTENTRY_GET_RIGHTCHAR
#define	AFTKN_FILTER_EXECUTE					796		// (BASE_AFUNC_TOKEN + 242)		// AFTKN_FILTER_EXECUTE
#define	AFTKN_REM_SELECTCONNECTION				797		// (BASE_AFUNC_TOKEN + 243)		// AFTKN_REM_SELECTCONNECTION
#define	AFTKN_REM_GETDEFAULTCONNECTNAME			798		// (BASE_AFUNC_TOKEN + 244)		// AFTKN_REM_GETDEFAULTCONNECTNAME
#define	AFTKN_REM_SETITEMSFILTER				799		// (BASE_AFUNC_TOKEN + 245)		// AFTKN_REM_SETITEMSFILTER
#define	AFTKN_REM_SETITEMSFOLDERS				800		// (BASE_AFUNC_TOKEN + 246)		// AFTKN_REM_SETITEMSFOLDERS
#define	AFTKN_REM_SET_ADDRBOOK_DNLOAD_FILTER	801		// (BASE_AFUNC_TOKEN + 247)		// AFTKN_REM_SET_ADDRBOOK_DNLOAD_FILTER
#define	AFTKN_REM_REQUESTPGROUP					802		// (BASE_AFUNC_TOKEN + 248)		// AFTKN_REM_REQUESTPGROUP
#define	AFTKN_RULE_ACT_ARCHIVE					803		// (BASE_AFUNC_TOKEN + 249)		// AFTKN_RULE_ACT_ARCHIVE
#define	AFTKN_REM_DELETECONNECTION				804		// (BASE_AFUNC_TOKEN + 250)		// AFTKN_REM_DELETECONNECTION
#define	AFTKN_EVENT_NOTIFY						805		// (BASE_AFUNC_TOKEN + 251)		// AFTKN_EVENT_NOTIFY
#define	AFTKN_PLAY_SOUND						806		// (BASE_AFUNC_TOKEN + 252)		// AFTKN_PLAY_SOUND
#define	AFTKN_DATE_PARSE_HOURS					808		// (BASE_AFUNC_TOKEN + 254)		// AFTKN_DATE_PARSE_HOURS
#define	AFTKN_DATE_PARSE_MINUTES				809		// (BASE_AFUNC_TOKEN + 255)		// AFTKN_DATE_PARSE_MINUTES
#define	AFTKN_DATE_PARSE_DUR_HR					810		// (BASE_AFUNC_TOKEN + 256)		// AFTKN_DATE_PARSE_DUR_HR
#define	AFTKN_DATE_PARSE_DUR_MIN				811		// (BASE_AFUNC_TOKEN + 257)		// AFTKN_DATE_PARSE_DUR_MIN
#define	AFTKN_CSTMSG_COMPOSE					812		// (BASE_AFUNC_TOKEN + 258)		// AFTKN_CSTMSG_COMPOSE
#define	AFTKN_TEXTENTRY_IS_INSERT_MODE			813		// (BASE_AFUNC_TOKEN + 259)		// AFTKN_TEXTENTRY_IS_INSERT_MODE
#define	AFTKN_ITEM_GET_REPLY_MSGID				814		// (BASE_AFUNC_TOKEN + 260)		// AFTKN_ITEM_GET_REPLY_MSGID
#define	AFTKN_EN_REGISTER						815		// (BASE_AFUNC_TOKEN + 261)		// AFTKN_EN_REGISTER
#define	AFTKN_EN_UNREGISTER						816		// (BASE_AFUNC_TOKEN + 262)		// AFTKN_EN_UNREGISTER
#define	AFTKN_AB_RESOLVE_FULLNAME				817		// (BASE_AFUNC_TOKEN + 263)		// AFTKN_AB_RESOLVE_FULLNAME
#define	AFTKN_AB_GET_FULLNAME					818		// (BASE_AFUNC_TOKEN + 264)		// AFTKN_AB_GET_FULLNAME
#define	AFTKN_AL_GET_FULLNAME					819		// (BASE_AFUNC_TOKEN + 265)		// AFTKN_AL_GET_FULLNAME
#define	AFTKN_BTNBAR_COPY						820		// (BASE_AFUNC_TOKEN + 266)		// AFTKN_BTNBAR_COPY
#define	AFTKN_APP_ACTIVATE						821		// (BASE_AFUNC_TOKEN + 267)		// AFTKN_APP_ACTIVATE
#define	AFTKN_REM_MODEMCOMMAND					822		// (BASE_AFUNC_TOKEN + 268)		// AFTKN_REM_MODEMCOMMAND
#define	AFTKN_REM_GETDEFAULTCONNECTNUMBER		823		// (BASE_AFUNC_TOKEN + 269)		// AFTKN_REM_GETDEFAULTCONNECTNUMBER
#define	AFTKN_RULE_ACT_STOPRULES				824		// (BASE_AFUNC_TOKEN + 270)		// AFTKN_RULE_ACT_STOPRULES
#define	AFTKN_BTNBAR_SETSCROLL					825		// (BASE_AFUNC_TOKEN + 271)		// AFTKN_BTNBAR_SETSCROLL
#define	AFTKN_BTNBAR_SETTIPS					826		// (BASE_AFUNC_TOKEN + 272)		// AFTKN_BTNBAR_SETTIPS
#define	AFTKN_FILTER_SET_SDWORD					827		// (BASE_AFUNC_TOKEN + 273)		// AFTKN_FILTER_SET_SDWORD
#define	AFTKN_FILTER_SET_SWORD					828		// (BASE_AFUNC_TOKEN + 274)		// AFTKN_FILTER_SET_SWORD
#define	AFTKN_QUERY_SAVE_AS_FOLDER_DEFAULT		829		// (BASE_AFUNC_TOKEN + 275)		// AFTKN_QUERY_SAVE_AS_FOLDER_DEFAULT
#define	AFTKN_ITEM_GET_SENT_MSGID				830		// (BASE_AFUNC_TOKEN + 276)		// AFTKN_ITEM_GET_SENT_MSGID
#define	AFTKN_QUERY_EXECUTE_ODMA				831		// (BASE_AFUNC_TOKEN + 277)		// AFTKN_QUERY_EXECUTE_ODMA
#define	AFTKN_SHELF_ICON_GET_COUNT				832		// (BASE_AFUNC_TOKEN + 278)		// AFTKN_SHELF_ICON_GET_COUNT
#define	AFTKN_SHELF_ICON_GET_SELECTED			833		// (BASE_AFUNC_TOKEN + 279)		// AFTKN_SHELF_ICON_GET_SELECTED
#define	AFTKN_SHELF_ICON_GET_INFO				834		// (BASE_AFUNC_TOKEN + 280)		// AFTKN_SHELF_ICON_GET_INFO
#define	AFTKN_SHELF_ICON_SET_TEMP_ICON			835		// (BASE_AFUNC_TOKEN + 281)		// AFTKN_SHELF_ICON_SET_TEMP_ICON
#define	AFTKN_RULE_ACT_LAUNCH					836		// (BASE_AFUNC_TOKEN + 282)		// AFTKN_RULE_ACT_LAUNCH
#define	AFTKN_FILTER_SET_BYTE					837		// (BASE_AFUNC_TOKEN + 283)		// AFTKN_FILTER_SET_BYTE
#define	AFTKN_FILTER_SET_DWORD					838		// (BASE_AFUNC_TOKEN + 284)		// AFTKN_FILTER_SET_DWORD
#define	AFTKN_FILTER_SET_WORD					839		// (BASE_AFUNC_TOKEN + 285)		// AFTKN_FILTER_SET_WORD
#define	AFTKN_FILTER_GROUP_BEGIN				840		// (BASE_AFUNC_TOKEN + 286)		// AFTKN_FILTER_GROUP_BEGIN
#define	AFTKN_FILTER_GROUP_END					841		// (BASE_AFUNC_TOKEN + 287)		// AFTKN_FILTER_GROUP_END
#define	AFTKN_FILTER_GROUP_MARKER				842		// (BASE_AFUNC_TOKEN + 288)		// AFTKN_FILTER_GROUP_MARKER
#define	AFTKN_DISPLAY_SETTINGS_SET				843		// (BASE_AFUNC_TOKEN + 289)		// AFTKN_DISPLAY_SETTINGS_SET
#define	AFTKN_DM_VERSION_LIST					844		// (BASE_AFUNC_TOKEN + 290)		// AFTKN_DM_VERSION_LIST
#define	AFTKN_FILTER_SET_DATE					845		// (BASE_AFUNC_TOKEN + 291)		// AFTKN_FILTER_SET_DATE
#define	AFTKN_SET_COLUMNS						846		// (BASE_AFUNC_TOKEN + 292)		// AFTKN_SET_COLUMNS
#define	AFTKN_SET_SORT							847		// (BASE_AFUNC_TOKEN + 293)		// AFTKN_SET_SORT
#define	AFTKN_QUERY_SAVE_AS_FOLDER				848		// (BASE_AFUNC_TOKEN + 294)		// AFTKN_QUERY_SAVE_AS_FOLDER
#define	AFTKN_QUERY_EXECUTE						849		// (BASE_AFUNC_TOKEN + 295)		// AFTKN_QUERY_EXECUTE
#define	AFTKN_RULE_ACT_MARKREAD					850		// (BASE_AFUNC_TOKEN + 296)		// AFTKN_RULE_ACT_MARKREAD
#define	AFTKN_RULE_ACT_MARKUNREAD				851		// (BASE_AFUNC_TOKEN + 297)		// AFTKN_RULE_ACT_MARKUNREAD
#define	AFTKN_RULE_ACT_REPLY_WITH_TEXT			852		// (BASE_AFUNC_TOKEN + 298)		// AFTKN_RULE_ACT_REPLY_WITH_TEXT
#define	AFTKN_SHARED_FOLDER_ACCEPT				853		// (BASE_AFUNC_TOKEN + 299)		// AFTKN_SHARED_FOLDER_ACCEPT
#define	AFTKN_DM_OPEN							854		// (BASE_AFUNC_TOKEN + 300)		// AFTKN_DM_OPEN
#define	AFTKN_DM_CLOSE							855		// (BASE_AFUNC_TOKEN + 301)		// AFTKN_DM_CLOSE
#define	AFTKN_DM_ENDPREVIEW						856		// (BASE_AFUNC_TOKEN + 302)		// AFTKN_DM_ENDPREVIEW
#define AFTKN_DM_READONLYOPEN					857		// (BASE_AFUNC_TOKEN + 303)		// AFTKN_DM_READONLYOPEN
#define AFTKN_DM_NEW_DOC						858		// (BASE_AFUNC_TOKEN + 304)		// AFTKN_DM_NEW_DOC
#define AFTKN_DM_VIEW_DOC						859		// (BASE_AFUNC_TOKEN + 305)		// AFTKN_DM_VIEW_DOC
#define AFTKN_DM_DISPLAY_PROFILE				860		// (BASE_AFUNC_TOKEN + 306)		// AFTKN_DM_DISPLAY_PROFILE
#define AFTKN_DM_IMPORT_DOC						861		// (BASE_AFUNC_TOKEN + 307)		// AFTKN_DM_IMPORT_DOC
#define AFTKN_DM_GETFIELD						862		// (BASE_AFUNC_TOKEN + 308)		// AFTKN_DM_GETFIELD
#define AFTKN_DM_DELETE							863		// (BASE_AFUNC_TOKEN + 309)		// AFTKN_DM_DELETE
#define AFTKN_DM_ADDVER							864		// (BASE_AFUNC_TOKEN + 310)		// AFTKN_DM_ADDVER
#define AFTKN_DM_COPYVER						865		// (BASE_AFUNC_TOKEN + 311)		// AFTKN_DM_COPYVER
#define AFTKN_DM_COPYPROFILE					866		// (BASE_AFUNC_TOKEN + 312)		// AFTKN_DM_COPYPROFILE
#define AFTKN_DM_SETFIELD						867		// (BASE_AFUNC_TOKEN + 313)		// AFTKN_DM_SETFIELD
#define AFTKN_DM_GET_OPEN_RIGHTS				868		// (BASE_AFUNC_TOKEN + 314)		// AFTKN_DM_GET_OPEN_RIGHTS
#define AFTKN_SET_DAY							869		// (BASE_AFUNC_TOKEN + 315)		// AFTKN_SET_DAY
#define AFTKN_UNASSIGNED_316					870		// (BASE_AFUNC_TOKEN + 316)		// AFTKN_UNASSIGNED
#define AFTKN_ODM_SELECT_DLG					871		// (BASE_AFUNC_TOKEN + 317)		// AFTKN_ODM_SELECT_DLG
#define AFTKN_ITEM_PRINT						872		// (BASE_AFUNC_TOKEN + 318)		// AFTKN_ITEM_PRINT
#define AFTKN_FILTER_SET_VERSTATUS				873		// (BASE_AFUNC_TOKEN + 319)		// AFTKN_UNASSIGNED
#define AFTKN_DM_NEW_VERSION					874		// (BASE_AFUNC_TOKEN + 320)		// AFTKN_DM_NEW_VERSION
#define AFTKN_DM_GET_APP_KEY					875 	// (BASE_AFUNC_TOKEN + 321) 	// AFTKN_DM_GET_APP_KEY
#define AFTKN_DM_GET_PENDING_DOC				876 	// (BASE_AFUNC_TOKEN + 322) 	// AFTKN_DM_GET_PENDING_DOC
#define AFTKN_ITEM_CHANGETO						877		// (BASE_AFUNC_TOKEN + 323)		// AFTKN_ITEM_CHANGETO
#define AFTKN_REM_REQUEST_DOC					878 	// (BASE_AFUNC_TOKEN + 324) 	// AFTKN_REM_REQUEST_DOC
#define AFTKN_DM_GET_INTEGRATIONS_SWITCH		879		// (BASE_AFUNC_TOKEN + 325)		// AFTKN_DM_GET_INTEGRATIONS_SWITCH
#define AFTKN_SELECT_TIMEZONE					880		// (BASE_AFUNC_TOKEN + 326)
#define AFTKN_DM_GET_NUM_LIBRARIES				881 	// (BASE_AFUNC_TOKEN + 327) 	// AFTKN_DM_GET_NUM_LIBRARIES
#define AFTKN_DM_DISPLAY_ERRORS					882 	// (BASE_AFUNC_TOKEN + 328) 	// AFTKN_DM_DISPLAY_ERROR
#define AFTKN_RULE_EXISTS						883 	// (BASE_AFUNC_TOKEN + 329) 	// AFTKN_RULE_EXISTS
#define	AFTKN_DM_ECHO_DOC						884		// (BASE_AFUNC_TOKEN + 330)		// AFTKN_DM_ECHO_DOC
#define AFTKN_SET_SIG							885 	// (BASE_AFUNC_TOKEN + 331) 	// AFTKN_SET_SIG
#define AFTKN_DISPLAY_SETTINGS_SET_TEMP			886 	// (BASE_AFUNC_TOKEN + 332) 	// AFTKN_DISPLAY_SETTINGS_SET_TEMP
#define AFTKN_DISPLAY_SETTINGS_SET_EX			887 	// (BASE_AFUNC_TOKEN + 333) 	// AFTKN_DISPLAY_SETTINGS_SET_EX
#define AFTKN_SET_MONTH_DISPLAY_OPTIONS			888 	// (BASE_AFUNC_TOKEN + 334) 	// AFTKN_UNASSIGNED
#define AFTKN_SET_CAL_DISPLAY_OPTIONS			889 	// (BASE_AFUNC_TOKEN + 335) 	// AFTKN_UNASSIGNED
#define AFTKN_SET_WORK_SCHEDULE					890 	// (BASE_AFUNC_TOKEN + 336) 	// AFTKN_UNASSIGNED
#define AFTKN_GET_CONTEXT_INFO					891 	// (BASE_AFUNC_TOKEN + 337) 	// AFTKN_UNASSIGNED
#define AFTKN_DM_REPLACE_DOC					892 	// (BASE_AFUNC_TOKEN + 337) 	// AFTKN_DM_REPLACE_DOC
#define AFTKN_EXE_BTN							893 	// (BASE_AFUNC_TOKEN + 339) 	// AFTKN_UNASSIGNED
#define AFTKN_MSG_EDITOR_STYLE					894 	// (BASE_AFUNC_TOKEN + 340) 	// AFTKN_MSG_EDITOR_STYLE
#define AFTKN_UNASSIGNED_341					895 	// (BASE_AFUNC_TOKEN + 341) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_342					896 	// (BASE_AFUNC_TOKEN + 342) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_343					897 	// (BASE_AFUNC_TOKEN + 343) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_344					898 	// (BASE_AFUNC_TOKEN + 344) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_345					899 	// (BASE_AFUNC_TOKEN + 345) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_346					900 	// (BASE_AFUNC_TOKEN + 346) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_347					901 	// (BASE_AFUNC_TOKEN + 347) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_348					902 	// (BASE_AFUNC_TOKEN + 348) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_349					903 	// (BASE_AFUNC_TOKEN + 349) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_350					904 	// (BASE_AFUNC_TOKEN + 350) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_351					905 	// (BASE_AFUNC_TOKEN + 351) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_352					906 	// (BASE_AFUNC_TOKEN + 352) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_353					907 	// (BASE_AFUNC_TOKEN + 353) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_354					908 	// (BASE_AFUNC_TOKEN + 354) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_355					909 	// (BASE_AFUNC_TOKEN + 355) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_356					910 	// (BASE_AFUNC_TOKEN + 356) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_357					911 	// (BASE_AFUNC_TOKEN + 357) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_358					912 	// (BASE_AFUNC_TOKEN + 358) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_359					913 	// (BASE_AFUNC_TOKEN + 359) 	// AFTKN_UNASSIGNED
#define AFTKN_UNASSIGNED_360					914 	// (BASE_AFUNC_TOKEN + 360) 	// AFTKN_UNASSIGNED

#define	TOP_AFUNC_TOKEN							AFTKN_UNASSIGNED_360

#define	AFUNC_TOKEN_COUNT						(TOP_AFUNC_TOKEN - BASE_AFUNC_TOKEN + 1)

////////////////////////////////////////////////////////////////////////////////
//	Token IDs
//	- 2nd Range for Base Function Token Values
////////////////////////////////////////////////////////////////////////////////

#define BASE_FUNCTION_TOKEN2				960

#define BFTKN_CONTEXT_NEW_APPT					960 	// (BASE_FUNCTION_TOKEN2 + 1)
#define BFTKN_CONTEXT_NEW_PERS_APPT				961 	// (BASE_FUNCTION_TOKEN2 + 2)
#define BFTKN_CONTEXT_NEW_NOTE					962 	// (BASE_FUNCTION_TOKEN2 + 3)
#define BFTKN_CONTEXT_NEW_PERS_NOTE				963 	// (BASE_FUNCTION_TOKEN2 + 4)
#define BFTKN_CONTEXT_NEW_TODO					964 	// (BASE_FUNCTION_TOKEN2 + 5)
#define BFTKN_CONTEXT_NEW_PERS_TODO				965 	// (BASE_FUNCTION_TOKEN2 + 6)
#define BFTKN_HHMM_DEFAULT_FORMAT				966 	// (BASE_FUNCTION_TOKEN2 + 7)
#define BFTKN_HHMM_12HOUR						967		// (BASE_FUNCTION_TOKEN2 + 8)
#define BFTKN_HHMM_24HOUR						968		// (BASE_FUNCTION_TOKEN2 + 9)
#define BFTKN_HHMM_LEADINGZERO					969 	// (BASE_FUNCTION_TOKEN2 + 10)
#define BFTKN_AMPM_SHOWAMPM						970 	// (BASE_FUNCTION_TOKEN2 + 11)
#define BFTKN_AMPM_ABBREVIATE					971 	// (BASE_FUNCTION_TOKEN2 + 12)
#define BFTKN_AMPM_LOWERCASE					972 	// (BASE_FUNCTION_TOKEN2 + 13)
#define	BFTKN_DISPLAY_OPTIONS					973 	// (BASE_FUNCTION_TOKEN2 + 14)
#define	BFTKN_PRINT_OPTIONS						974 	// (BASE_FUNCTION_TOKEN2 + 15)
#define	BFTKN_CLIPBOARD_OPTIONS					975 	// (BASE_FUNCTION_TOKEN2 + 16)
#define BFTKN_CAL_SHOW_ICONS					976 	// (BASE_FUNCTION_TOKEN2 + 17)
#define BFTKN_CAL_HIDE_ICONS					977 	// (BASE_FUNCTION_TOKEN2 + 18)
#define BFTKN_CLEAR_ALARM						978 	// (BASE_FUNCTION_TOKEN2 + 19)
#define BFTKN_HTML_WESTERNDEF					979 	// (BASE_FUNCTION_TOKEN2 + 20)
#define BFTKN_HTML_CENTRALEUROPEAN				980 	// (BASE_FUNCTION_TOKEN2 + 21)
#define BFTKN_HTML_JAPANESESJIS					981 	// (BASE_FUNCTION_TOKEN2 + 22)
#define BFTKN_HTML_JAPANESEEUC					982 	// (BASE_FUNCTION_TOKEN2 + 23)
#define BFTKN_HTML_CHINESEBIG5					983 	// (BASE_FUNCTION_TOKEN2 + 24)
#define BFTKN_HTML_CHINESEEUC					984 	// (BASE_FUNCTION_TOKEN2 + 25)
#define BFTKN_HTML_CHINESEGB					985 	// (BASE_FUNCTION_TOKEN2 + 26)
#define BFTKN_HTML_KOREANHANGUL					986 	// (BASE_FUNCTION_TOKEN2 + 27)
#define BFTKN_HTML_CYRILLIC1251					987 	// (BASE_FUNCTION_TOKEN2 + 28)
#define BFTKN_HTML_CYRILLICKOI8					988 	// (BASE_FUNCTION_TOKEN2 + 29)

#define BFTKN_VIEW_RTF							989 	// (BASE_FUNCTION_TOKEN2 + 30)
#define BFTKN_VIEW_HTML							990 	// (BASE_FUNCTION_TOKEN2 + 31)
#define BFTKN_SEND_USING_MENU_4					991 	// (BASE_FUNCTION_TOKEN2 + 32)
#define BFTKN_SEND_USING_MENU_5					992 	// (BASE_FUNCTION_TOKEN2 + 33)
#define BFTKN_SEND_USING_MENU_6					993 	// (BASE_FUNCTION_TOKEN2 + 34)
#define BFTKN_SEND_USING_MENU_7					994 	// (BASE_FUNCTION_TOKEN2 + 35)
#define BFTKN_SEND_USING_MENU_8					995 	// (BASE_FUNCTION_TOKEN2 + 36)
#define BFTKN_SEND_USING_MENU_9					996 	// (BASE_FUNCTION_TOKEN2 + 37)
#define BFTKN_SEND_USING_MENU_10				997 	// (BASE_FUNCTION_TOKEN2 + 38)
#define BFTKN_SEND_USING_MENU_MORE				998 	// (BASE_FUNCTION_TOKEN2 + 39)
#define BFTKN_TOGGLE_AUTO_PILOT					999 	// (BASE_FUNCTION_TOKEN2 + 40)
#define BFTKN_RUN_SCRIPT					   1000 	// (BASE_FUNCTION_TOKEN2 + 41)
#define BFTKN_ACCOUNT_LIST_REFRESH			   1001 	// (BASE_FUNCTION_TOKEN2 + 42)
#define BFTKN_SHOW_STATUS_WINDOW			   1002 	// (BASE_FUNCTION_TOKEN2 + 43)
#define BFTKN_SYNC_FOLDER					   1003 	// (BASE_FUNCTION_TOKEN2 + 44)
#define BFTKN_EXPUNGE_FOLDER				   1004 	// (BASE_FUNCTION_TOKEN2 + 45)
#define BFTKN_VIEW_ENCODE					   1005 	// (BASE_FUNCTION_TOKEN2 + 46)

#define	TOP_FUNCTION_TOKEN2						1023 	// Cannot go beyond 0x3ff
#define BFUNC_TOKEN2_COUNT						(BASE_FUNCTION_TOKEN2 - TOP_FUNCTION_TOKEN2 + 1)


////////////////////////////////////////////////////////////////////////////////
//	Menu IDs
////////////////////////////////////////////////////////////////////////////////

#define	IDM_ACTION_MENU		32514	// 0x7F02
#define	IDM_BTNBAR_MENU		32515	// 0x7F03
#define	IDM_EDIT_MENU		32516	// 0x7F04
#define	IDM_FILE_MENU		32517	// 0x7F05
#define IDM_FONT_MENU		32518	// 0x7F06
#define	IDM_HELP_MENU		32519	// 0x7F07
#define IDM_OPTIONS_MENU	32521	// 0x7F09
#define IDM_REMOTE_MENU		32522	// 0x7F0A
#define IDM_SEND_MENU		32523	// 0x7F0B
#define IDM_TOOLS_MENU		32524	// 0x7F0C
#define	IDM_VIEW_MENU		32525	// 0x7F0D
#define	IDM_WINDOW_MENU		32526	// 0x7F0E
#define IDM_OPEN_VIEWS_MENU	32527	// 0x7F0F
#define	IDM_BBAR_TEMP_MENU	32528	// 0x7F10
#define IDM_OTHER_MENU		32529	// 0x7F11
#define	IDM_BBAR_SYS_MENUA	32530	// 0x7F12
#define	IDM_BBAR_SYS_MENUB	32531	// 0x7F13
#define	IDM_BBAR_SYS_MENUM	32532	// 0x7F14
#define	IDM_POWER_BAR_MENU	32533	// 0x7F15
#define IDM_FILTER_MENU		32534	// 0x7F16
#define	IDM_VIEW_BY			32535	// 0x7F17
#define	IDM_FILTER_DLG_MENU	32536	// 0x7F18
#define	IDM_NEW_MENU		32537	// 0x7F19
#define	IDM_SYS_MENU		32538	// 0x7F1A		// ID for sys menu longprompts
#define	IDM_CHANGETO_MENU	32539	// 0x7F1B
#define IDM_QRY_POPUP_MENU	32540	// 0x7F1C

