/*__________________________________________________________________________
 Copyright (C) 2002 PGP Corporation
 All rights reserved.
 
 $Id: ICQutil.c,v 1.10 2002/08/06 20:09:42 dallen Exp $
__________________________________________________________________________*/
#include "precomp.h"

void CreateBMFileFromResource(const char* file, HRSRC hres)
{
	int sizeclrtbl,bpl,rest,BytesPerLine,sizepxldata,bmfile;
	HGLOBAL hresbm;
	LPVOID lpresbm;
	BITMAPINFO *bmInfo;
	BITMAPFILEHEADER bmFileHeader;

	hresbm=LoadResource(g_hinst, hres);
	lpresbm=LockResource(hresbm);
	bmInfo=(BITMAPINFO *)lpresbm;

	// calculate the size (in bytes) of the palette and the pixel data
	sizeclrtbl=0;
	switch (bmInfo->bmiHeader.biBitCount) 
	{
		case 1: sizeclrtbl=2*4; break;
		case 4: sizeclrtbl=16*4; break;
		case 8: sizeclrtbl=256*4; break;
	}

	bpl = (bmInfo->bmiHeader.biWidth * bmInfo->bmiHeader.biBitCount) / 8;
	rest=4-(bpl % 4);
	if(rest==4) rest=0;
	BytesPerLine = bpl + rest;
	sizepxldata = BytesPerLine * bmInfo->bmiHeader.biHeight;

	// fill the BITMAPFILEHEADER for the file
	bmFileHeader.bfType = 19778;
	bmFileHeader.bfSize = 14 + 40 + sizeclrtbl + sizepxldata;
	bmFileHeader.bfReserved1 = 0;
	bmFileHeader.bfReserved2 = 0;
	bmFileHeader.bfOffBits = 54;

	// write BITMAPFILEHEADER and bitmap data to file
	bmfile = open(file, O_RDWR | O_CREAT | O_TRUNC | O_BINARY, S_IREAD | S_IWRITE);
	write(bmfile, &bmFileHeader, 14);
	write(bmfile, lpresbm, 40 + sizeclrtbl + sizepxldata);

	// clean up
	close(bmfile);

	UnlockResource(hresbm);
	FreeResource(hresbm);
}

void GetVersionOfFile (char * pszAppName, // file
                       char * pszVerBuff, // receives version
                       int iVerBuffLen)
{
	DWORD dwScratch;
	DWORD dwInfSize ;
	UINT uSize;
	BYTE *pbyInfBuff;
	char szVersion [32];
	char szResource [80];
	char * pszVersion = szVersion;
	DWORD * pdwLangChar;
	VS_FIXEDFILEINFO  *ffi;

	dwInfSize = GetFileVersionInfoSize (pszAppName, &dwScratch);

	pbyInfBuff=(BYTE *)malloc(dwInfSize);
	
	if (pbyInfBuff)
	{
		memset(pbyInfBuff,0x00,dwInfSize);

		if (GetFileVersionInfo (pszAppName, 0, 
				dwInfSize, pbyInfBuff))
		{
			if (VerQueryValue (pbyInfBuff, 
					"\\VarFileInfo\\Translation",
					(void**)(&pdwLangChar),
					&uSize))
			{
				wsprintf (szResource, "\\StringFileInfo\\%04X%04X\\FileVersion",
					LOWORD (*pdwLangChar), HIWORD (*pdwLangChar));

				if (VerQueryValue (pbyInfBuff, 
						szResource,
						(void**)(&pszVersion),
						&uSize))
				{
					strncpy (pszVerBuff, pszVersion, iVerBuffLen-1);
				}
				else
				{
					DWORD i1,i2,i3,i4;

					// No version string
					if (VerQueryValue (pbyInfBuff, 
							"\\",
							(void**)(&pszVersion),
							&uSize))
					{
						ffi=(VS_FIXEDFILEINFO *)pszVersion;

						i1=(ffi->dwProductVersionMS>>16)&0xffff;
						i2=(ffi->dwProductVersionMS)&0xffff;
						i3=(ffi->dwProductVersionLS>>16)&0xffff;
						i4=(ffi->dwProductVersionLS)&0xffff;

						if(iVerBuffLen>50)
							sprintf(pszVerBuff,"ICQ Build %d.%d.%d.%d",i1,i2,i3,i4);
					}
				}
			}
		}
		free(pbyInfBuff);
	}
}

BOOL EncryptRichEdit(PPLUGININFO plugin)
{
	DWORD inSize;
	char *pInput;
	char *pOutput;
	DWORD outSize;
	PGPOptionListRef userOptions = NULL;
	int BuffSize;
	char szNick[256];
	char szICQ[256];
	char szEmail[512];
	char *pAddresses[2];
	BOOL bResult;

	bResult=FALSE;

	GetWindowText(plugin->hwndNick,szNick,256);
	GetWindowText(plugin->hwndICQ,szICQ,256);
	sprintf(szEmail,"<ICQ:%s>",szICQ);

	BuffSize=SendMessage(plugin->hwndRichEdit,WM_GETTEXTLENGTH,0,0);
				
	if(BuffSize==0)
	{
		MessageBox (plugin->hwndRichEdit,
			"There is no text to encrypt",
			"PGP Error",
			MB_OK|MB_ICONSTOP|MB_SETFOREGROUND);
	}
	else
	{
		BuffSize=BuffSize+10;

		pInput=malloc(BuffSize);

		if(pInput!=NULL)
		{
			memset(pInput,0x00,BuffSize);

			inSize=GetWindowText(plugin->hwndRichEdit,pInput,BuffSize-1);

			if(inSize!=0)
			{
				BOOL bRst;
				BOOL bForce;

				pAddresses[0]=szEmail;
				pAddresses[1]=0;

				bForce=FALSE;

				if (GetAsyncKeyState( VK_CONTROL) & 0x8000)
					bForce=TRUE;

				bRst=ICQEncryptBuffer(plugin->hwndRichEdit,
					"PGPicq",PGPsc,PGPtls,
					plugin->bEncrypt, plugin->bEncrypt,
					pInput,inSize,
					&pOutput,&outSize,
					pAddresses,1,bForce);

				if((bRst)&&(pOutput!=NULL))
				{
					SetWindowText(plugin->hwndRichEdit,pOutput);

					memset(pOutput,0x00,outSize);
					PGPFreeData(pOutput);
					pOutput=NULL;

					bResult=TRUE;
				}
			}
			memset(pInput,0x00,inSize);
			free(pInput);
		}
	}

	return bResult;
}

PGPError GetLocalUserKey(PGPKeyDBRef KeyDB,PGPKeyDBObjRef *pLocalKey)
{
	DWORD dwICQ;
	PGPError err;

	err=kPGPError_NoErr;

	*pLocalKey=NULL;

		// Go try to fetch key that matches users ICQ #
	dwICQ=ReadICQRegKey();

	if(dwICQ!=0)
	{
		char szUserID[200];

		sprintf(szUserID,"<ICQ:%d>",dwICQ);

		// If we found one, find. If not NULL, no big deal...
		*pLocalKey=MatchICQWithKey(szUserID,KeyDB,TRUE);
	}

	if(*pLocalKey==NULL)
	{
		PGPclGetDefaultPrivateKey(KeyDB,pLocalKey);
	}

	return err;
}

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
