/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	GMTree.h - create and fill group manager control	

	$Id: GMTree.c,v 1.17 2002/11/03 16:37:36 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPgmx.h"
#include "PGPkeysx.h"

// constant definitions
#define BITMAP_WIDTH 16
#define BITMAP_HEIGHT 16

// external globals
extern HINSTANCE	g_hinst;
extern HIMAGELIST	g_hilKeys;

//	___________________________________________________
//
//  Return handle of group manager window

HWND  
PGPgmGetManagerWindow (HGROUPMAN hGM) 
{
	if (!hGM) 
		return NULL;
	return (((PGROUPMAN)hGM)->hwndTree);
}


//	___________________________________________________
//
//  Insert column information into control

BOOL 
GMAddColumns (PGROUPMAN pGM) 
{
	TL_COLUMN tlc;
	CHAR sz[64];
	INT iField; 
	INT iCol, ids;

	TreeList_DeleteAllColumns (pGM->hwndTree);

	tlc.mask = TLCF_FMT | TLCF_WIDTH | TLCF_TEXT | 
			TLCF_SUBITEM | TLCF_DATATYPE | TLCF_DATAMAX |
			TLCF_MOUSENOTIFY;
	tlc.pszText = sz;

	tlc.iSubItem = 0;
	tlc.fmt = TLCFMT_LEFT;
	tlc.iDataType = TLC_DATASTRING;
	tlc.cx = pGM->wFieldWidth[0];
	tlc.bMouseNotify = FALSE;
	LoadString (g_hinst, IDS_GROUPNAMEFIELD, sz, sizeof(sz));
	TreeList_InsertColumn (pGM->hwndTree, 0, &tlc);

	for (iCol=1; iCol<NUMBERFIELDS; iCol++) 
	{
		iField = pGM->wColumnField[iCol];
		if (iField) 
		{
			switch (iField) {
				case GMI_VALIDITY :
					ids = IDS_VALIDITYFIELD;
					if (pGM->uOptions & GMF_NOVICEMODE) 
						tlc.fmt = TLCFMT_IMAGE;
					else 
						tlc.fmt = TLCFMT_LINBAR;
					tlc.cchTextMax = 
							GMConvertFromPGPValidity (kPGPValidity_Complete);
					tlc.iDataType = TLC_DATALONG;
					tlc.bMouseNotify = FALSE;
					break;

				case GMI_DESCRIPTION :
					ids = IDS_DESCRIPTIONFIELD;
					tlc.fmt = TLCFMT_LEFT;
					tlc.iDataType = TLC_DATASTRING;
					tlc.bMouseNotify = FALSE;
					break;
			}
			LoadString (g_hinst, ids, sz, sizeof(sz));
			tlc.cx = pGM->wFieldWidth[iField];
			TreeList_InsertColumn (pGM->hwndTree, iCol, &tlc);
		}
	}

	return TRUE;
}

//	___________________________________________________
//
//  Create TreeList Window

HGROUPMAN 
PGPgmCreateGroupManager (
		PGPContextRef		Context, 
		PGPtlsContextRef	tlsContext,
		HWND				hwndParent, 
		INT					iID, 
		INT					x, 
		INT					y,
		INT					nWidth, 
		INT					nHeight) 
{
	PGROUPMAN			pGM;
	PGPclWindowsGenus	genus;
	DWORD				dwStyle;

	// Ensure that the common control DLL is loaded.
	InitCommonControls ();

	// Ensure that the custom control DLL is loaded.
	InitTreeListControl ();

	pGM = gmAlloc (sizeof (_GROUPMAN));
	if (!pGM) 
		return NULL;
	memset (pGM, 0x00, sizeof (_GROUPMAN));

	pGM->hwndParent = hwndParent;
	pGM->hwndTree = NULL;
	pGM->groupsetMain = NULL;
	pGM->iID = iID;
	pGM->pDropTarget = NULL;			//pointer to DropTarget object

	pGM->context = Context;			//PGP context
	pGM->tlsContext = tlsContext;
	pGM->uOptions = 0;

	pGM->bMultipleSelected = FALSE;
	pGM->bLocatingKeys = FALSE;
	pGM->uSelectedFlags = 0;
	pGM->iFocusedItemType = 0;
	pGM->iFocusedObjectType = 0;
	pGM->hFocusedItem = NULL;
	pGM->pFocusedObject = NULL;

	pGM->iValidityThreshold = GM_VALIDITY_MARGINAL;

	GMGetColumnPreferences (pGM);

	dwStyle = WS_VISIBLE | WS_CHILD | WS_BORDER | WS_TABSTOP | WS_GROUP |
			TLS_HASBUTTONS | TLS_HASLINES | TLS_AUTOSCROLL | 
			TLS_PROMISCUOUS | TLS_INTERNALDRAG | 
			TLS_DRAGABLEHEADERS | TLS_DRAGSELECTION;

	PGPclGetWindowsVersion (NULL, &genus, NULL);
	if (genus >= kPGPclWindowsGenusXp)
		dwStyle |= TLS_USEDEFAULTGUIFONT;

	// Create the tree view window.
	pGM->hwndTree = CreateWindowEx (WS_EX_CLIENTEDGE, WC_TREELIST, "",
			dwStyle, x, y, nWidth, nHeight,
			hwndParent, (HMENU)iID, g_hinst, NULL);

	if (pGM->hwndTree == NULL) return NULL;

	// Associate the image list with the tree view control.
	TreeList_SetImageList (pGM->hwndTree, g_hilKeys);

	GMSetFocus (pGM, NULL, FALSE, FALSE);
	pGM->pDropTarget = PKCreateDropTarget (pGM->hwndParent, 
			PK_IDROP_GROUPLIST, (VOID*)pGM); 
	CoLockObjectExternal ((IUnknown*)pGM->pDropTarget, TRUE, TRUE);
	RegisterDragDrop (pGM->hwndTree, pGM->pDropTarget);
	PKEnableDropTarget (pGM->pDropTarget, FALSE);

 	return (HGROUPMAN)pGM;
}


//	___________________________________________________
//
//  Set (or add) a tree item to the tree

static HTLITEM 
sSetOneGroupItem (
		PGROUPMAN	pGM, 
		BOOL		bReInsert, 
		HTLITEM		hItem, 
		HTLITEM		hParent, 
		LPSTR		szText, 
		HTLITEM		hInsAfter, 
		INT			iImage, 
		UINT		uState, 
		LPARAM		lParam) 
{
	TL_TREEITEM tlI;
	TL_INSERTSTRUCT tlIns;

	tlI.hItem = hItem;
	tlI.mask = TLIF_TEXT | TLIF_IMAGE | TLIF_STATE | TLIF_PARAM;
	tlI.stateMask = TLIS_BOLD | TLIS_ITALICS;
	tlI.stateMask |= uState;
	tlI.state = uState;
	tlI.pszText = szText;
	tlI.cchTextMax = lstrlen (szText);
	tlI.iImage = iImage;
	tlI.iSelectedImage = iImage;
	tlI.lParam = lParam;

	// Insert the data into the tree.
	if (bReInsert || !hItem) 
	{
		tlIns.item = tlI;
		tlIns.hInsertAfter = hInsAfter;
		tlIns.hParent = hParent;
		return (TreeList_InsertItem (pGM->hwndTree, &tlIns));
	}
	else 
	{
		TreeList_SetItem (pGM->hwndTree, &tlI);
		return hItem;
	}
}


//	___________________________________________________
//
//  Set list data for a key

static HTLITEM 
sSetGroupData (
		PGROUPMAN	pGM, 
		HTLITEM		hItem, 
		PGPGroupID	groupid, 
		LPSTR		pszDesc) 
{ 
	TL_LISTITEM tlL;
	INT iField, iCol;

	INT iValue, iNumNotFound;

	tlL.hItem = hItem;
	tlL.stateMask = TLIS_VISIBLE;

	for (iCol=1; iCol<NUMBERFIELDS; iCol++) 
	{
		iField = pGM->wColumnField[iCol];
		if (iField) 
		{
			switch (iField) {
			case GMI_VALIDITY :
				tlL.pszText = NULL;

				PGPGetGroupLowestValidity (pGM->groupsetMain, groupid, 
						pGM->keydbMain, &iValue, &iNumNotFound);

				if (iNumNotFound > 0) iValue = 0;
				else iValue = GMConvertFromPGPValidity (iValue);

				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_DATAVALUE | TLIF_STATE;
				if (pGM->uOptions & GMF_NOVICEMODE) 
				{
					if (iValue > GM_VALIDITY_COMPLETE)
						tlL.lDataValue = IDX_AXIOMATIC;
					else if (iValue >= pGM->iValidityThreshold) 
						tlL.lDataValue = IDX_VALID;
					else tlL.lDataValue = IDX_INVALID;
				}
				else tlL.lDataValue = iValue;
				break;

			case GMI_DESCRIPTION :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_TEXT | TLIF_STATE;
				tlL.pszText = pszDesc;
				break;

			}
			tlL.iSubItem = iCol;
			hItem = (HTLITEM) TreeList_SetListItem (
					pGM->hwndTree, &tlL, FALSE);
		}
	}

	return (hItem);
}


//	___________________________________________________
//
//  Set list data for a key

static HTLITEM 
sSetKeyData (
		PGROUPMAN		pGM, 
		HTLITEM			hItem, 
		PGPKeyDBObjRef	key, 
		LPSTR			pszDesc) 
{ 
	TL_LISTITEM	tlL;
	INT			iField, iCol;
	PGPBoolean	bAxiomatic;

	INT iValue;

	tlL.hItem = hItem;
	tlL.stateMask = TLIS_VISIBLE;

	for (iCol=1; iCol<NUMBERFIELDS; iCol++) 
	{
		iField = pGM->wColumnField[iCol];
		if (iField) 
		{
			switch (iField) {
			case GMI_VALIDITY :
				PGPGetPrimaryUserIDValidity (key, &iValue);
				iValue = GMConvertFromPGPValidity (iValue);

				PGPGetKeyDBObjBooleanProperty (key, 
						kPGPKeyProperty_IsAxiomatic, &bAxiomatic);
				if (bAxiomatic) 
				{
					iValue = 
						GMConvertFromPGPValidity (kPGPValidity_Complete)+1;
				}

				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_DATAVALUE | TLIF_STATE;

				if (pGM->uOptions & GMF_NOVICEMODE) 
				{
					if (iValue > GM_VALIDITY_COMPLETE)
						tlL.lDataValue = IDX_AXIOMATIC;
					else if (iValue >= pGM->iValidityThreshold) 
						tlL.lDataValue = IDX_VALID;
					else 
						tlL.lDataValue = IDX_INVALID;
				}
				else 
					tlL.lDataValue = iValue;
				break;

			case GMI_DESCRIPTION :
				tlL.state = TLIS_VISIBLE;
				tlL.mask = TLIF_TEXT | TLIF_STATE;
				tlL.pszText = pszDesc;
				break;

			}
			tlL.iSubItem = iCol;
			hItem = (HTLITEM) TreeList_SetListItem (
					pGM->hwndTree, &tlL, FALSE);
		}
	}

	return (hItem);
}


//	___________________________________________________
//
//  Reload a single group

static HTLITEM 
sReloadGroup (
		PGROUPMAN	pGM, 
		PGPGroupID	groupid, 
		INT			iIndex,
		BOOL		bReInsert, 
		BOOL		bForceNewAlloc, 
		BOOL		bExpandNew,
		BOOL		bFirstCall, 
		HTLITEM		hTParent, 
		HTLITEM		hTPrev) 
{

	HTLITEM				hTGroup, hTItem, hTItemPrev;
	TL_TREEITEM			tli;
	INT					i, iNumKeys, iNumTotal, idx;
	UINT				u, uState, uVal;
	PGPBoolean			bNew, bItalics;
	PGPGroupInfo		groupinfo;
	PGPGroupItem		groupitem;
	PGPGroupID			groupidThis, groupidParent;
	PGPKeyDBObjRef		key;
	PGPError			err;
	CHAR				sz[kPGPMaxUserIDSize];
	CHAR				szID[kPGPMaxKeyIDStringSize];
	static BOOL			bNewKeyExpanded;

	uState = 0;
	bNew = FALSE;
	hTItem = NULL;
	hTItemPrev = TLI_FIRST;

	if (bFirstCall) 
		bNewKeyExpanded = FALSE;

	if (hTParent) 
	{
		groupidParent = groupid;
		PGPGetIndGroupItem (pGM->groupsetMain, groupidParent, 
												iIndex, &groupitem);
		groupidThis = groupitem.u.groupID;
		PGPGetGroupInfo (pGM->groupsetMain, groupidThis, &groupinfo);
		hTGroup = groupitem.userValue;
	}
	else 
	{
		groupidThis = groupid;
		PGPGetGroupInfo (pGM->groupsetMain, groupidThis, &groupinfo);
		hTGroup = groupinfo.userValue;
	}
	if (!hTGroup) 
		bNew = TRUE;

	// determine icon and italics
	idx = IDX_GROUP;

	// insert group item into tree and save pointer to tree item
	if (bNew && bExpandNew) 
		uState |= TLIS_SELECTED;
	if (bForceNewAlloc) 
		hTGroup = NULL;
	if (hTParent) 
	{
		uVal = MAKELONG (iIndex, groupidParent);
		hTGroup = sSetOneGroupItem (pGM, bReInsert, hTGroup, hTParent, 
				groupinfo.name, hTPrev, idx, uState, uVal);
		PGPSetIndGroupItemUserValue (pGM->groupsetMain, groupidParent, 
				iIndex, (PGPUserValue)hTGroup);
	}
	else 
	{
		uVal = MAKELONG (groupidThis, 0);
		hTGroup = sSetOneGroupItem (pGM, bReInsert, hTGroup, hTParent, 
				groupinfo.name, hTPrev, idx, uState, uVal);
		PGPSetGroupUserValue (pGM->groupsetMain, 
				groupidThis, (PGPUserValue)hTGroup);
	}

	sSetGroupData (pGM, hTGroup, groupidThis, groupinfo.description);

	// if a reinsertion, then we're done
	if (bReInsert) 
		return hTGroup;

	// if not at root, then we don't draw items in group
	if (hTParent) 
		return hTGroup;

	// iterate through items in group
	PGPCountGroupItems (pGM->groupsetMain, 
			groupidThis, FALSE, &iNumKeys, &iNumTotal);

	for (i=0; i<iNumTotal; i++) 
	{
		PGPGetIndGroupItem (pGM->groupsetMain, groupidThis, i, &groupitem);

		if (groupitem.type == kPGPGroupItem_KeyID) 
		{
			err = PGPFindKeyByKeyID (pGM->keydbMain,
					&groupitem.u.keyID, &key);

			if (IsntPGPError (err) && PGPKeyDBObjRefIsValid (key)) 
			{
				uState = 0;
				hTItem = (HTLITEM)groupitem.userValue;
				if (!hTItem) bNew = TRUE;
	
				// get and set treelist tree data for this key
				PGPclGetPrimaryUserIDNameUTF8 (key, sz, sizeof(sz), &u);
				if (bForceNewAlloc) 
					hTItem = NULL;

				idx = PGPclDetermineKeyIcon (key, &bItalics);
				if (bItalics) 
					uState |= TLIS_ITALICS;

				uVal = MAKELONG (i, groupidThis);
				hTItem = sSetOneGroupItem (pGM, bReInsert, hTItem, hTGroup, 
						sz, hTItemPrev, idx, uState, uVal);

				PGPSetIndGroupItemUserValue (pGM->groupsetMain,
						groupidThis, i, (PGPUserValue)hTItem);

				sSetKeyData (pGM, hTItem, key, "");
			}
			else 
			{
				hTItem = (HTLITEM)groupitem.userValue;
				if (!hTItem) bNew = TRUE;
	
				// get and set treelist tree data for this key
				if (bForceNewAlloc) 
					hTItem = NULL;

				idx = IDX_DSAPUBDISKEY;
				uState = TLIS_ITALICS;

				LoadString (g_hinst, IDS_UNAVAILABLE, sz, sizeof(sz));
				PGPGetKeyIDString (&groupitem.u.keyID, 
						kPGPKeyIDString_Abbreviated, szID);
				lstrcat (sz, szID);

				uVal = MAKELONG (i, groupidThis);
				hTItem = sSetOneGroupItem (pGM, bReInsert, hTItem, hTGroup, 
						sz, hTItemPrev, idx, uState, uVal);

				PGPSetIndGroupItemUserValue (pGM->groupsetMain,
						groupidThis, i, (PGPUserValue)hTItem);
			}
		}

		// this is a group within a group
		else
		{
			hTItem = sReloadGroup (pGM, groupidThis, i, bReInsert, 
					bForceNewAlloc, bExpandNew, FALSE, hTGroup, hTItemPrev);
		}

		hTItemPrev = hTItem;
	}

	// select and expand key, if appropriate 
	if (bExpandNew && bNew) 
	{
		tli.hItem = hTGroup;
		if (!bNewKeyExpanded) 
		{
			TreeList_Select (pGM->hwndTree, &tli, TRUE);
			TreeList_Expand (pGM->hwndTree, &tli, TLE_EXPANDALL);
			bNewKeyExpanded = TRUE;
		}
		else 
		{
			GMSetFocus (pGM, GMFocusedItem (pGM), TRUE, FALSE);
			TreeList_Expand (pGM->hwndTree, &tli, TLE_EXPANDALL);
		}
	}

	return hTGroup;
}


//	___________________________________________________
//
//  Scan entire groupset loading in any new data

BOOL 
GMLoadGroupsIntoTree (
		PGROUPMAN	pGM, 
		BOOL		bReInsert, 
		BOOL		bExpandNew, 
		BOOL		bForceRealloc) 
{

	HCURSOR			hCursorOld;
	TL_TREEITEM		tli;
	HTLITEM			hTPrevKey;
	BOOL			bFirst;
	PGPGroupID		groupid;
	INT				i, iNumGroups;

	hTPrevKey = (HTLITEM)TLI_FIRST;
	bFirst = TRUE;

	if (pGM->groupsetMain) 
	{
		hCursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
		PGPCountGroupsInSet (pGM->groupsetMain, &iNumGroups);

		for (i=0; i<iNumGroups; i++) 
		{
			PGPGetIndGroupID (pGM->groupsetMain, i, &groupid);
			hTPrevKey = sReloadGroup (pGM, groupid, 0, bReInsert, 
					bForceRealloc, bExpandNew, bFirst, NULL, hTPrevKey);
			bFirst = FALSE;
		}

		SetCursor (hCursorOld);
		if (bExpandNew) 
		{
			tli.hItem = GMFocusedItem (pGM);
			if (tli.hItem) 
				TreeList_EnsureVisible (pGM->hwndTree, &tli);
		}
		return TRUE;
	}

	return FALSE;
}


//	___________________________________________________
//
//  Cleanup group manager

PGPError 
PGPgmDestroyGroupManager (HGROUPMAN hGM) 
{

	PGROUPMAN		pGM					= (PGROUPMAN)hGM;

	if (!hGM) 
		return kPGPError_BadParams;

	RevokeDragDrop (pGM->hwndTree);
	PKReleaseDropTarget (pGM->pDropTarget);  
	CoLockObjectExternal ((IUnknown*)pGM->pDropTarget, FALSE, TRUE);

	PGPclCloseGroupFile (pGM->pGroupFile);

	GMSetColumnPreferences (pGM);

	SendMessage (pGM->hwndTree, WM_CLOSE, 0, 0);

	gmFree (pGM);

	return kPGPError_NoErr;
}

//	___________________________________________________
//
//  Map dragover screen coordinates to window coords

BOOL
GMSelectGroup (PGROUPMAN pGM, POINTL ptl)
{
	POINT	pt;

	pt.x = ptl.x;
	pt.y = ptl.y;
	MapWindowPoints (NULL, pGM->hwndTree, &pt, 1);

	return (TreeList_DragOver (pGM->hwndTree, MAKELONG (pt.x, pt.y)));
}

//	___________________________________________________
//
//  Load keyset into treelist

PGPError 
PGPgmLoadGroups (HGROUPMAN hGroupMan)
{
	PGROUPMAN		pGM			= (PGROUPMAN)hGroupMan;

	if (!pGM) 
		return kPGPError_BadParams;

	if (pGM->groupsetMain) 
	{
		GMSetFocus (pGM, NULL, FALSE, FALSE);
		TreeList_DeleteTree (pGM->hwndTree, TRUE);
		PGPclCloseGroupFile (pGM->pGroupFile);
		pGM->groupsetMain = NULL;
	}

	if (PGPKeyDBRefIsValid (pGM->keydbMain)) 
	{
		PGPclOpenGroupFile (pGM->context, &(pGM->pGroupFile));
		pGM->groupsetMain = pGM->pGroupFile->groupset;

		GMAddColumns (pGM);

		GMSortGroupSet (pGM);
		GMLoadGroupsIntoTree (pGM, FALSE, FALSE, TRUE);
		PKEnableDropTarget (pGM->pDropTarget, TRUE);
	}

	InvalidateRect (pGM->hwndTree, NULL, TRUE);
	UpdateWindow (pGM->hwndTree);

	return kPGPError_NoErr;
}


//	___________________________________________________
//
//  Load keyset into treelist

PGPError PGPgmExport 
PGPgmReLoadGroups (HGROUPMAN hGroupMan) 
{
	PGROUPMAN pGM = (PGROUPMAN)hGroupMan;
	if (!pGM) 
		return kPGPError_BadParams;
	if (!pGM->groupsetMain) 
		return kPGPError_BadParams;
	if (!PGPKeyDBRefIsValid (pGM->keydbMain))
		return kPGPError_BadParams;

	GMSortGroupSet (pGM);
	GMLoadGroupsIntoTree (pGM, FALSE, TRUE, FALSE);

	InvalidateRect (pGM->hwndTree, NULL, TRUE);
	UpdateWindow (pGM->hwndTree);

	return kPGPError_NoErr;
}


//	___________________________________________________
//
//  Set configuration

PGPError PGPgmExport 
PGPgmConfigure (
		HGROUPMAN	hGroupMan, 
		PGMCONFIG	pGMConfig) 
{
	PGROUPMAN pGM = (PGROUPMAN)hGroupMan;
	if (!pGM) 
		return kPGPError_BadParams;

	if (pGM) 
	{
		pGM->ppks			= pGMConfig->ppks;
		pGM->hKL			= pGMConfig->hKL;
		pGM->keydbMain		= pGMConfig->keydbMain;
		pGM->uOptions		= pGMConfig->uOptions;
		pGM->hwndStatusBar	= pGMConfig->hwndStatusBar;

		if (pGM->uOptions & GMF_MARGASINVALID) 
			pGM->iValidityThreshold = GM_VALIDITY_COMPLETE;
		else
			pGM->iValidityThreshold = GM_VALIDITY_MARGINAL;

		return kPGPError_NoErr;
	}

	else 
		return kPGPError_BadParams;

}

