/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	PGPkeys.c - PGPkeys WinMain, associated routines, and global variables	

	$Id: PGPkeys.c,v 1.47 2002/11/12 18:57:30 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPkeysx.h"
#include "pgpWin32IPC.h"
#include "pgpImageList.h"

// pgp header files
#include "pgpRandomPool.h"
#include "pgpUnicodeWin32.h"
#include "pgpPDA.h"

// system header files
#include <ole2.h>
#include <zmouse.h>

#define INSTALLER_SWITCH	("/i")

// typedefs
typedef struct _DLGLISTSTRUCT {
	struct	_DLGLISTSTRUCT* next;
	HWND	hwnd;
	BOOL	bDlg;
	BOOL	bProperties;
} WINDOWLISTSTRUCT, *PWINDOWLISTSTRUCT;

// Application global variables
HINSTANCE			g_hinst						= NULL;
HWND				g_hwndMain					= NULL;
HWND				g_hwndActive				= NULL;
HIMAGELIST			g_hilKeys					= NULL;
HICON				g_hiconToken				= NULL;
HICON				g_hiconOpenLock				= NULL;
HICON				g_hiconClosedLock			= NULL;
PGPContextRef		g_context					= kInvalidPGPContextRef;
PGPtlsContextRef	g_tlscontext				= kInvalidPGPtlsContextRef;
PGPBoolean			g_bKeyGenEnabled			= TRUE;
PGPBoolean			g_bX509CertRequestEnabled	= TRUE;
PGPBoolean			g_bKeyReconstructionEnabled	= FALSE;
PGPBoolean			g_bExpertMode				= FALSE;
PGPBoolean			g_bMarginalAsInvalid		= FALSE;
BOOL				g_bShowGroups				= FALSE;
INT					g_iGroupsPercent			= 0;
INT					g_iToolHeight				= 0;
PGPUInt32			g_uNumTokens				= 0;
PGPPDASTRUCT*		g_pPDA						= NULL;

UINT				g_uReloadKeyringMessage			= 0;
UINT				g_uReloadPrefsMessage			= 0;	
UINT				g_uReloadKeyserverPrefsMessage	= 0;	
UINT				g_uMouseWheelMessage			= 0;

// Local globals
static HHOOK				s_hhookKeyboard;		//handle of keyboard hook 
static HHOOK				s_hhookMouse;			//handle of mouse hook 
static PWINDOWLISTSTRUCT	s_pWindowList = NULL;

//	____________________________________
//
//	Check for correct version of Windows

static BOOL 
sIsCorrectVersion (VOID) 
{
	OSVERSIONINFO osid;

	osid.dwOSVersionInfoSize = sizeof (osid);
	GetVersionEx (&osid);

	switch (osid.dwPlatformId) {

	case VER_PLATFORM_WIN32s :
		return FALSE;

	// Windows 95
	case VER_PLATFORM_WIN32_WINDOWS :
 		return TRUE;

	// Windows NT
	case VER_PLATFORM_WIN32_NT :
		if (osid.dwMajorVersion >= 4) return TRUE;
//>>>
		MessageBox (NULL, "PGPkeys is not supported on Windows NT 3.51\n\n"
			"Use at your own risk.",
		"PGPkeys Warning", MB_OK|MB_ICONEXCLAMATION);
		return TRUE;
//<<<
	default :
		return FALSE;
	}
}

//	____________________________________
//
//	Check for previously running instances

static BOOL 
sDoIExist (
		LPSTR lpszSemName, 
		LPSTR lpszClassName, 
		LPSTR lpszWindowTitle) 
{
    HANDLE			hSem;
    HWND			hWndMe;
	COPYDATASTRUCT	cds;

    // Create or open a named semaphore. 
    hSem = CreateSemaphore (NULL, 0, 1, lpszSemName);

    // Close handle and return NULL if existing semaphore was opened.
    if ((hSem != NULL) && (GetLastError() == ERROR_ALREADY_EXISTS)) 
	{
        CloseHandle(hSem);
        hWndMe = FindWindow (lpszClassName, lpszWindowTitle);
        if (hWndMe) 
		{
			if (!IsZoomed (hWndMe)) 
				ShowWindow (hWndMe, SW_RESTORE);

			SetForegroundWindow (hWndMe);
			cds.dwData = PGPPK_IMPORTKEYCOMMANDLINE | PGPPK_SELECTIVEIMPORT;
			cds.lpData = GetCommandLine ();
			cds.cbData = lstrlen (cds.lpData) +1;
			SendMessage (hWndMe, WM_COPYDATA, 0, (LPARAM)&cds);
		}
        return TRUE;
    }

    // If new semaphore was created, return FALSE.
    return FALSE;
}


//	____________________________________
//
//	Check to see if we were run by the installer

static BOOL 
sInstallerInvoked (VOID)
{
	if (strstr (GetCommandLine (), INSTALLER_SWITCH))
		return TRUE;
	else
	    return FALSE;
}


//	____________________________________________________________
//
//	load appropriate imagelist for display

static VOID
sInitImageLists (VOID)
{
	HDC			hdc;
	HBITMAP		hbmp;
	INT			iNumBits;

	// create imagelist and load the appropriate bitmaps based on 
	// current display capabilities.
	hdc = GetDC (NULL);		// DC for desktop
	iNumBits = GetDeviceCaps (hdc, BITSPIXEL) * GetDeviceCaps (hdc, PLANES);
	ReleaseDC (NULL, hdc);

	if (iNumBits <= 8) 
	{
		g_hilKeys = ImageList_Create (16, 16, 
				ILC_COLOR|ILC_MASK, NUM_IMAGELIST_ICONS, 0); 
		hbmp = LoadBitmap (g_hinst, MAKEINTRESOURCE (IDB_KEYICONS4BIT));
		ImageList_AddMasked (g_hilKeys, hbmp, RGB(255, 0, 255));
		DeleteObject (hbmp);
	}
	else 
	{
		g_hilKeys = ImageList_Create (16, 16, 
				ILC_COLOR24|ILC_MASK, NUM_IMAGELIST_ICONS, 0); 
		hbmp = LoadBitmap (g_hinst, MAKEINTRESOURCE (IDB_KEYICONS24BIT));
		ImageList_AddMasked (g_hilKeys, hbmp, RGB(255, 0, 255));
		DeleteObject (hbmp);
	}

	g_hiconToken = ImageList_GetIcon (
			g_hilKeys, IDX_TOKEN, ILD_TRANSPARENT);
	g_hiconOpenLock = ImageList_GetIcon (
			g_hilKeys, IDX_OPENLOCK, ILD_TRANSPARENT);
	g_hiconClosedLock = ImageList_GetIcon (
			g_hilKeys, IDX_CLOSEDLOCK, ILD_TRANSPARENT);
}


//	____________________________________________
//
//	Initializations for all instances of program

static BOOL 
sInitApplication (
		HINSTANCE	hInstance, 
		BOOL		bIgnoreComctlError) 
{
	WNDCLASS				wndclass;
	PGPError				err;

	// check correct version of windows
	if (!sIsCorrectVersion ()) 
		return FALSE;

	// check for previous instances
	if (sDoIExist (PGPKEYS_SEMAPHORENAME, 
				PGPKEYS_WINCLASSNAME, PGPKEYS_WINDOWTITLE)) 
		return FALSE;

	// save instance handle
	g_hinst = hInstance;

	// check if we are being run by the installer
	if (sInstallerInvoked ())
		return FALSE;

	// register the reload messages
	g_uReloadKeyringMessage = 
			RegisterWindowMessage (kPGPclReloadKeyRingMsg);
	g_uReloadPrefsMessage = 
			RegisterWindowMessage (kPGPclReloadPrefsMsg);
	g_uReloadKeyserverPrefsMessage = 
			RegisterWindowMessage (kPGPclReloadKeyserverPrefsMsg);

	// register the mouse wheel message
	g_uMouseWheelMessage = 
			RegisterWindowMessage (MSH_MOUSEWHEEL);

	// make sure PGPkeys is installed
	if (!PGPclIsComponentInstalled (kPGPclPGPkeys))
	{
		PKMessageBox (NULL, IDS_CAPTIONERROR,
				IDS_NOTINSTALLED, MB_OK|MB_ICONHAND);
		return FALSE;
	}

	// initialize client library
	// note: this fails if common control DLL is not correct version
	err = PGPclInitLibrary (&g_context, 
			kPGPclEnableNewUserWizard | kPGPclForceNewUserWizard);
	if (IsPGPError (err))
	{
		if (err == kPGPError_Win32_InvalidComctl32 && bIgnoreComctlError)
			err = kPGPError_NoErr;
		else if (err == kPGPError_FeatureNotAvailable) 
		{
			PKMessageBox (NULL, IDS_CAPTIONERROR, 
					IDS_SDKEXPIRED, MB_OK|MB_ICONSTOP);
			return FALSE;
		}
		else
		{
			PGPclErrorBox (NULL, err);
			return FALSE;
		}

		PGPclCloseLibrary ();
	}

	PGPclErrorBox (NULL, PGPNewTLSContext (g_context, &g_tlscontext));

	sInitImageLists ();

	wndclass.style = 0;                     
	wndclass.lpfnWndProc = (WNDPROC)PGPkeysWndProc; 
	wndclass.cbClsExtra = 0;              
	wndclass.cbWndExtra = 0;              
	wndclass.hInstance = hInstance;       
	wndclass.hIcon = LoadIcon (hInstance, MAKEINTRESOURCE(IDI_EXEICON));
	wndclass.hCursor = LoadCursor (NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)(COLOR_BTNFACE+1); 
    wndclass.lpszMenuName = NULL;
	wndclass.lpszClassName = PGPKEYS_WINCLASSNAME;

	return (RegisterClass (&wndclass));
}

//	____________________________________
//
//  Hook procedure for WH_MOUSE hook.
  
static LRESULT CALLBACK 
sMouseHookProc (
		INT		iCode, 
		WPARAM	wParam, 
		LPARAM	lParam) 
{ 
    if (iCode >= 0) 
		PGPGlobalRandomPoolMouseMoved ();

	return CallNextHookEx (s_hhookMouse, iCode, wParam, lParam);
} 

//	____________________________________
//
//  Hook procedure for WH_KEYBOARD hook.
  
static LRESULT CALLBACK 
sKeyboardHookProc (
		INT		iCode, 
		WPARAM	wParam, 
		LPARAM	lParam) 
{ 
    if (iCode >= 0) 
	{
		// only add key presses (not releases)
		if (!(lParam & 0x80000000))
		{
			PGPGlobalRandomPoolAddKeystroke (wParam);
		}
	}

	return CallNextHookEx (s_hhookKeyboard, iCode, wParam, lParam);
} 

//	____________________________________
//
//	Send message to all windows in list

VOID
PKSendMessageWindowList (
		UINT	uMsg, 
		WPARAM	wParam, 
		LPARAM	lParam)
{
	PWINDOWLISTSTRUCT pwls;

	pwls = s_pWindowList;

	while (pwls) 
	{
		SendMessage (pwls->hwnd, uMsg, wParam, lParam);
		pwls = pwls->next;
	}
}

//	____________________________________
//
//	Post message to all windows in list

VOID
PKPostMessageWindowList (
		UINT	uMsg, 
		WPARAM	wParam, 
		LPARAM	lParam)
{
	PWINDOWLISTSTRUCT pwls;

	pwls = s_pWindowList;

	while (pwls) 
	{
		PostMessage (pwls->hwnd, uMsg, wParam, lParam);
		pwls = pwls->next;
	}
}

//	____________________________________
//
//	Enable/Disable all windows in list

VOID
PKEnableWindowList (BOOL bEnabled)
{
	PWINDOWLISTSTRUCT pwls;

	pwls = s_pWindowList;

	while (pwls) 
	{
		EnableWindow (pwls->hwnd, bEnabled);
		pwls = pwls->next;
	}
}

//	____________________________________
//
//	Add dlg hwnd to linked list

static VOID
sAddWindowToList (
		HWND	hwnd, 
		BOOL	bDlg,
		BOOL	bProperties)
{
	PWINDOWLISTSTRUCT pwls;

	pwls = pkAlloc (sizeof (WINDOWLISTSTRUCT));
	if (pwls) 
	{
		pwls->hwnd = hwnd;
		pwls->bDlg = bDlg;
		pwls->bProperties = bProperties;
		pwls->next = s_pWindowList;
		s_pWindowList = pwls;
	}
}

//	____________________________________
//
//	Remove dlg hwnd from linked list

static VOID
sRemoveWindowFromList (HWND hwnd)
{
	PWINDOWLISTSTRUCT pwls;
	PWINDOWLISTSTRUCT pwlsPrev;

	if (!s_pWindowList) 
		return;

	pwls = s_pWindowList;

	if (pwls->hwnd == hwnd) 
	{
		s_pWindowList = s_pWindowList->next;
	}
	else 
	{
		pwls = s_pWindowList->next;
		pwlsPrev = s_pWindowList;
		while (pwls) 
		{
			if (pwls->hwnd == hwnd) 
			{
				pwlsPrev->next = pwls->next;
				break;
			}
			pwlsPrev = pwls;
			pwls = pwls->next;
		}
	}

	if (pwls) 
		pkFree (pwls);
}

//	____________________________________
//
//	Add or remove dlg hwnd from linked list

VOID
PKAddRemoveWindowList (
		HWND	hwnd, 
		BOOL	bAdd, 
		BOOL	bDlg,
		BOOL	bProperties)
{
	// if a hwnd is specified, then add/remove if from list
	if (hwnd) 
	{
		if (bAdd)
			sAddWindowToList (hwnd, bDlg, bProperties);
		else
			sRemoveWindowFromList (hwnd);
	}
}


//	___________________________________________________
//
//	Find the hwnd of one of our windows given a point on the screen

BOOL 
PKFindWindowFromPoint (
		POINT*	ppt, 
		HWND*	phwnd) 
{
	PWINDOWLISTSTRUCT	pwls;
	RECT				rc;

	if (!phwnd) 
		return FALSE;
	*phwnd = NULL;

	if (!ppt) 
		return FALSE;
	if (!s_pWindowList) 
		return FALSE;

	pwls = s_pWindowList;

	while (pwls) 
	{
		GetWindowRect (pwls->hwnd, &rc);
		if (PtInRect (&rc, *ppt)) 
		{
			//the point is within one of our main windows
			HWND hActiveWindow=GetForegroundWindow();

			//check if we are active currently
			if(NULL != hActiveWindow) 
			{
				if(hActiveWindow == pwls->hwnd)
				{
					//we are in the foreground
					*phwnd = pwls->hwnd;
					return TRUE;
				}
				return FALSE;//we are not in the foreground
			}
			
			//NOTE: there may not be any window in foreground
			//extra logic here will slow down DoDragDrop (ssd)
		}
		pwls = pwls->next;
	}

	return TRUE;
}


//	___________________________________________________
//
//	create an array of structs which include info of
//	PDA menu items

static PGPPDASTRUCT*
sCreatePDAStructArray (void)
{
	PGPPDASTRUCT*	pPDA				= NULL;
	UINT			uCount				= 0;
	HMODULE			hmod;
	CHAR			szPath[MAX_PATH];
	WIN32_FIND_DATA	wfd;
	HANDLE			hFind;

	PGPclGetPath (kPGPclInstallFolder, szPath, sizeof(szPath));
	lstrcat (szPath, "PGPpda*.DLL");

	hFind = FindFirstFile (szPath, &wfd);
	if (hFind == INVALID_HANDLE_VALUE)
		return NULL;

	do
	{
		PGPclGetPath (kPGPclInstallFolder, szPath, sizeof(szPath));
		lstrcat (szPath, wfd.cFileName);

		hmod = LoadLibrary (szPath);
		if (IsntNull (hmod))
		{
			BOOL		bPda				= FALSE;
			PGPUInt32	(*pGetPdaAPI)(void);

			pGetPdaAPI = (PGPUInt32 (*)(void))
					GetProcAddress (hmod, "PGPpdaGetAPIVersion");

			if (IsntNull (pGetPdaAPI))
			{
				PGPUInt32	uAPIVersion = (*pGetPdaAPI)();

				// we only support API version 1.0
				if ((uAPIVersion >= 0x00010000) &&
					(uAPIVersion <  0x00020000))
				{
					PGPUInt32	(*pPdaAPIEx)(PGPpdaAPIStruct*);

					bPda = TRUE;

					pPdaAPIEx = (PGPUInt32 (*)(PGPpdaAPIStruct*))
							GetProcAddress (hmod, "PGPpdaAPIVersionInfo");

					if (IsntNull (pPdaAPIEx))
					{
						PGPpdaAPIStruct	pas;

						pgpClearMemory (&pas, sizeof(pas));

						// tell the plugin which API version we support
						pas.uPGPkeysVersion = MAKELONG (
								MAKEWORD (PGPVERSIONSUBMINOR, PGPVERSIONMINOR),
								MAKEWORD (PGPVERSIONMAJOR, 0));
						pas.uPGPkeysAPIVersion = kPGPpda_APIVersion;
						pas.uPGPkeysAPIFlags = 0;

						(*pPdaAPIEx)(&pas);
						bPda = TRUE;
					}
				}
			}

			if (bPda)
			{
				if (IsntNull (GetProcAddress (
						hmod, "PGPpdaOpenKeyset")))
				{
					UINT			uNew, uNewEx, uCountNew;
					UINT			u, uPlatform, uVersion;
					PGPpdaIdStruct	pis;
					
					PGPUInt32 (*pGetPdaOpCount)(void);
					PGPUInt32 (*pGetPdaOpCountEx)(PGPUInt32);
					PGPError (*pGetPdaId)(PGPpdaIdStruct*);

					pGetPdaId = (PGPError (*)(PGPpdaIdStruct*))
							GetProcAddress (hmod, "PGPpdaGetIdentification");

					uPlatform = kPGPpda_PlatformPalmOS;
					uVersion = 0x0001000;
					if (IsntNull (pGetPdaId))
					{
						if (IsntPGPError ((*pGetPdaId)(&pis)))
						{
							uPlatform = pis.uPlatform;
							uVersion = pis.uVersion;
						}
					}

					// we only allow one plugin per platform
					for (u=0; u<uCount; u++)
					{
						if (pPDA[u].uPlatform == uPlatform)
						{
							if (pPDA[u].uVersion >= uVersion)
							{
								bPda = FALSE;
								break;
							}
							else
							{
								// flag that this item is not to be used
								pPDA[u].uPlatform = 0;
							}
						}
					}

					if (bPda)
					{
						pGetPdaOpCount = (PGPUInt32 (*)(void))
							GetProcAddress (hmod, "PGPpdaGetOperationCount");

						if (IsntNull (pGetPdaOpCount))
							uNew = (*pGetPdaOpCount)();

						pGetPdaOpCountEx = (PGPUInt32 (*)(PGPUInt32))
							GetProcAddress (hmod, "PGPpdaGetOperationCountEx");

						uNewEx = 0;
						if (IsntNull (pGetPdaOpCountEx))
						{
							uNew = (*pGetPdaOpCountEx)(kPGPpdaOpType_MainWindow);
							uNewEx = (*pGetPdaOpCountEx)(kPGPpdaOpType_PDAWindow);
						}

						uCountNew = uCount+uNew+uNewEx +1;

						pPDA = realloc (pPDA, 
								(uCountNew +1) * sizeof(PGPPDASTRUCT));

						pPDA[uCount].bKeyring	= TRUE;
						pPDA[uCount].hmod		= hmod;
						pPDA[uCount].hwnd		= NULL;
						pPDA[uCount].uOp		= 0;
						pPDA[uCount].uPlatform	= uPlatform;
						pPDA[uCount].uVersion	= uVersion;
						pPDA[uCount].uFlags		= 0;

						for (u=uCount+1; u<uCount+uNew+1; u++)
						{
							pPDA[u].bKeyring	= FALSE;
							pPDA[u].hmod		= hmod;
							pPDA[u].hwnd		= NULL;
							pPDA[u].uOp			= u-(uCount+1);
							pPDA[u].uPlatform	= uPlatform;
							pPDA[u].uVersion	= uVersion;
							pPDA[u].uFlags		= 0;
						}

						for (; u<uCountNew; u++)
						{
							pPDA[u].bKeyring	= FALSE;
							pPDA[u].hmod		= hmod;
							pPDA[u].hwnd		= NULL;
							pPDA[u].uOp			= u-(uCount+1);
							pPDA[u].uPlatform	= uPlatform;
							pPDA[u].uVersion	= uVersion;
							pPDA[u].uFlags		= PK_PDAWINDOWOP;
						}

						pPDA[uCountNew].bKeyring	= FALSE;
						pPDA[uCountNew].hmod		= NULL;
						pPDA[uCountNew].hwnd		= NULL;
						pPDA[uCountNew].uOp			= 0;
						pPDA[uCountNew].uPlatform	= 0;
						pPDA[uCountNew].uVersion	= 0;
						pPDA[uCountNew].uFlags		= 0;

						uCount = uCountNew;
					}
				}
				else
					bPda = FALSE;
			}
			
			if (!bPda)
				FreeLibrary (hmod);
		}
	} while (FindNextFile (hFind, &wfd));

	if (hFind)
		FindClose (hFind);

	return pPDA;
}


//	___________________________________________________
//
//	destroy the array of PDA structs

static void
sDestroyPDAStructArray (
		PGPPDASTRUCT* pPDA)
{
	if (IsntNull (pPDA))
	{
		PGPPDASTRUCT*	p = pPDA;

		while (p->bKeyring && IsntNull (p->hmod))
		{
			FreeLibrary (p->hmod);
			p++;
		}

		free (pPDA);
	}
}


//	____________________________________
//
//	Main Windows routine

INT APIENTRY 
WinMain (
		HINSTANCE	hInstance, 
		HINSTANCE	hPrevInstance,
		LPSTR		lpCmdLine, 
		INT			nCmdShow) 
{
	PGPPrefRef			prefref				= kInvalidPGPPrefRef;

	MSG					msg;
	HACCEL				haccelMain, haccelDialog;
	INT					lWindowPosX, lWindowPosY, lWindowWidth, lWindowHeight;
	DWORD				dwShow;
	CHAR				sz[128];
	HMENU				hMenuMain;
	BOOL				bDlgMessage;
	BOOL				bIgnoreComctlError;
	PWINDOWLISTSTRUCT	pwls, pwlsNext;
	HWND				hwndThis;
	PGPError			err;

	// initialize OLE
	OleInitialize (NULL);

	//	initialize application stuff
	LoadString (hInstance, IDS_COMCTLERRORSWITCH, sz, sizeof(sz));
	if (strstr (lpCmdLine, sz))
		bIgnoreComctlError = TRUE;
	else
		bIgnoreComctlError = FALSE;

	if (!sInitApplication (hInstance, bIgnoreComctlError)) 
		return 0;

	//	attempt to set event hooks
	s_hhookMouse = SetWindowsHookEx (WH_MOUSE, sMouseHookProc, 
									NULL, GetCurrentThreadId());
	s_hhookKeyboard = SetWindowsHookEx (WH_KEYBOARD, sKeyboardHookProc, 
									NULL, GetCurrentThreadId());
	if (!s_hhookMouse || !s_hhookKeyboard) 
		return 0;

	//	check that admin file can be found
	err = PGPclPeekClientLibPrefRefs (&prefref, NULL);
	if (IsPGPError (err)) 
		return 0;

	PGPlnLicenseCheck (g_context);
	if(PGPlnEnterprise())
	{
		PGPGetPrefBoolean (prefref, 
				kPGPPrefAllowKeyGeneration, &g_bKeyGenEnabled);
		PGPGetPrefBoolean (prefref, 
				kPGPPrefAllowManualX509CertRequest, &g_bX509CertRequestEnabled);
		PGPGetPrefStringBuffer (prefref, 
				kPGPPrefLDAPReconServer, sizeof(sz), sz);
		g_bKeyReconstructionEnabled = (sz[0] != '\0');
	}

	// get window position and size information from registry
	PGPGetPrefBoolean (prefref, 
			kPGPPrefDisplayMarginalValidity, &g_bExpertMode);
	PGPGetPrefBoolean (prefref, 
			kPGPPrefMarginalIsInvalid, &g_bMarginalAsInvalid);
	PKGetPrivatePrefData (&dwShow, &lWindowPosX, &lWindowPosY, 
			&lWindowWidth, &lWindowHeight, &g_bShowGroups,
			&g_iGroupsPercent, &g_iToolHeight);
				
	// load up the PGPpda DLL, if installed
	g_pPDA = sCreatePDAStructArray ();

	hMenuMain = LoadMenu (hInstance, MAKEINTRESOURCE (IDR_MENUKEYMAN));

	g_hwndMain = CreateWindow (PGPKEYS_WINCLASSNAME, PGPKEYS_WINDOWTITLE, 
		WS_OVERLAPPEDWINDOW, lWindowPosX, 
		lWindowPosY, lWindowWidth,
		lWindowHeight, NULL, hMenuMain, hInstance, NULL);	

	if (!g_hwndMain) 
		return 0;

	ShowWindow (g_hwndMain, nCmdShow);	// Show the window
	UpdateWindow (g_hwndMain);			// Sends WM_PAINT message

	haccelMain = 
		LoadAccelerators (g_hinst, MAKEINTRESOURCE (IDR_KEYMANACCEL));  
	haccelDialog = 
		LoadAccelerators (g_hinst, MAKEINTRESOURCE (IDR_DIALOGACCEL));  

	while (GetMessage (&msg, NULL, 0,0)) 
	{
		if (s_pWindowList) 
		{
			bDlgMessage = FALSE;
			pwls = s_pWindowList;
			while (pwls) 
			{
				pwlsNext = pwls->next;

				if (pwls->bDlg) 
				{
					hwndThis = pwls->hwnd;

					if (pwls->bProperties)
					{
						if (PropSheet_IsDialogMessage (hwndThis, &msg)) 
						{
							if (msg.message == WM_KEYDOWN)
							{
								if (msg.wParam == VK_ESCAPE)
								{
									PKDeletePropertiesWindow (hwndThis);
									break;
								}
							}
							TranslateAccelerator (
									hwndThis, haccelDialog, &msg);
							bDlgMessage = TRUE;
							break;
						}
					}
					else
					{
						if (IsDialogMessage (hwndThis, &msg)) 
						{
							TranslateAccelerator (
									hwndThis, haccelDialog, &msg);
							bDlgMessage = TRUE;
							break;
						}
					}
				}
				pwls = pwlsNext;
			}
			if (bDlgMessage) 
				continue;
		}

		if (!TranslateAccelerator (g_hwndActive, haccelMain, &msg)) 
		{ 
			TranslateMessage (&msg); 
			DispatchMessage (&msg); 
		} 
	}

	sDestroyPDAStructArray (g_pPDA);

	if (PGPtlsContextRefIsValid (g_tlscontext))
		PGPFreeTLSContext (g_tlscontext);

	// remove hook procedures
	UnhookWindowsHookEx (s_hhookKeyboard);
	UnhookWindowsHookEx (s_hhookMouse);

	// clean out caches and shutdown sdk
	PGPclCloseLibrary ();

	// uninitialize OLE
	OleUninitialize ();

	if (g_hilKeys)
		ImageList_Destroy (g_hilKeys);
	if (g_hiconToken)
		DestroyIcon (g_hiconToken);
	if (g_hiconOpenLock)
		DestroyIcon (g_hiconOpenLock);
	if (g_hiconClosedLock)
		DestroyIcon (g_hiconClosedLock);

	return (msg.wParam);  
}


