/*____________________________________________________________________________
	Copyright (C) 2000 Networks Associates Technology, Inc.
	All rights reserved.
	
	PKAddID.c - add user and/or photo ID to key
	
	$Id: PKAddID.c,v 1.21 2002/11/22 03:15:51 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPkeysx.h"
#include "UTF8Edit.h"

// system header files
#include <commdlg.h>

// constant definitions
#define MAX_FULL_NAME_LEN		126
#define MAX_EMAIL_LEN			126

// typedefs
typedef struct {
	PPGPKEYSSTRUCT	ppks;
	LPSTR			pszUserID;
	PGPBoolean		bExchangeUser;
	PGPBoolean		bSyncWithServer;
} ADDNAMESTRUCT, *PADDNAMESTRUCT;

typedef struct {
	PPGPKEYSSTRUCT	ppks;
	WNDPROC			wpOrigPhotoIDProc;
	HBITMAP			hbitmapPhotoID;
	HPALETTE		hpalettePhotoID;
	INT				iwidthPhotoID;
	INT				iheightPhotoID;
	LPBYTE			pPhotoBuffer;
	PGPSize			iPhotoBufferLength;
	PGPBoolean		bSyncWithServer;
} ADDPHOTOSTRUCT, *PADDPHOTOSTRUCT;

// external globals
extern HINSTANCE		g_hinst;
extern PGPContextRef	g_context;
extern PGPtlsContextRef	g_tlscontext;

// local globals
static DWORD aNewUserIds[] = {			// Help IDs
    IDC_NEWUSERNAME,	IDH_PGPPK_NEWUSERID, 
	IDC_NEWEMAILADDR,	IDH_PGPPK_NEWEMAILADDR,
	IDC_EXCHANGEID,		IDH_PGPPK_USEEXCHANGEID,
    0,0 
}; 

static DWORD aNewPhotoIds[] = {			// Help IDs
	IDC_PHOTOID,		IDH_PGPPK_NEWPHOTOID,
    IDC_SELECTFILE,		IDH_PGPPK_BROWSENEWPHOTOID, 
    0,0 
}; 


//	____________________________________
//
//  try to get the user's MS Exchange name and email address

static VOID
sLookupExchangeUserID (
		HWND		hwnd)
{
	CHAR		szName[128];
	CHAR		szAddr[128];
	PGPError	err;

	err = PGPclGetExchangeServerAddress (
			hwnd, TRUE, szName, sizeof(szName), szAddr, sizeof(szAddr));

	if (err == kPGPError_FeatureNotAvailable)
	{
		PKMessageBox (hwnd, IDS_CAPTIONINFO, IDS_EXCHANGEPROMPT, 
				MB_OK|MB_ICONINFORMATION);

		err = PGPclGetExchangeServerAddress (
				hwnd, FALSE, szName, sizeof(szName), szAddr, sizeof(szAddr));
	}

	if (IsPGPError (err))
	{
		PKMessageBox (hwnd, IDS_CAPTIONALERT, IDS_EXCHANGEERROR,
				MB_OK|MB_ICONEXCLAMATION);
	}
	else
	{
		UTF8EditSetText (GetDlgItem (hwnd, IDC_NEWUSERNAME), szName);
		UTF8EditSetText (GetDlgItem (hwnd, IDC_NEWEMAILADDR), szAddr);
	}
}


//	___________________________________________________
//
//  Dialog Message procedure
//	When user asks to add a userID to a key, a dialog 
//	appears asking for the new userID to be typed in.
//	This is the message processing procedure for that
//	dialog.

static BOOL CALLBACK 
sAddUserDlgProc (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam,
		LPARAM	lParam) 
{

	PADDNAMESTRUCT pans;

	switch(uMsg) {

	case WM_INITDIALOG:
		pans = (PADDNAMESTRUCT)lParam;
		SetWindowLong (hwnd, GWL_USERDATA, lParam);
		UTF8EditInit (GetDlgItem (hwnd, IDC_NEWUSERNAME));
		SendMessage (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_NEWUSERNAME)), 
				EM_SETLIMITTEXT, MAX_FULL_NAME_LEN, 0);
		UTF8EditInit (GetDlgItem (hwnd, IDC_NEWEMAILADDR));
		SendMessage (UTF8EditGetHandle (GetDlgItem (hwnd, IDC_NEWEMAILADDR)),
				EM_SETLIMITTEXT, MAX_EMAIL_LEN, 0);

		if (pans->bExchangeUser)
		{
			EnableWindow (GetDlgItem (hwnd, IDC_EXCHANGEID), TRUE);
			ShowWindow (GetDlgItem (hwnd, IDC_EXCHANGEID), SW_SHOW);
		}
		return TRUE;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aNewUserIds);

	case WM_DESTROY :
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_NEWUSERNAME));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_NEWEMAILADDR));
		return 0;

	case WM_COMMAND :
		switch(LOWORD (wParam)) {
		case IDOK: 
			pans = (PADDNAMESTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			if (PKConstructUserID (hwnd, IDC_NEWUSERNAME, IDC_NEWEMAILADDR, 
					&(pans->pszUserID))) 
			{
				EndDialog (hwnd, 1);
			}
			return TRUE;

		case IDCANCEL:
			EndDialog (hwnd, 0);
			return TRUE;

		case IDC_EXCHANGEID :
			sLookupExchangeUserID (hwnd);
			break;
		}
		return TRUE;
	}
	return FALSE;
}

//	___________________________________________________
//
//  Add User to key
//	This routine is called when the user chooses to add
//	a userID to an existing secret key.

BOOL 
PKAddUserToKey (
		PGPKEYSSTRUCT*	ppks) 
{
	PGPPrefRef		prefref			= kInvalidPGPPrefRef;
	PGPByte*		pPasskey		= NULL;
	BOOL			bRetVal			= TRUE;
	PGPError		err;
	PGPKeyDBObjRef	key;
	PGPSize			sizePasskey;
	ADDNAMESTRUCT	ans;
	CHAR			sz[256];

	// initialize struct
	ans.ppks = ppks;

	if (IsPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		return FALSE;

	PGPGetPrefBoolean (prefref, 
			kPGPPrefKeyServerSyncOnAdd, &(ans.bSyncWithServer));

	ans.bExchangeUser = PGPclIsExchangeServerUser ();

	// get selected key
	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &key, NULL, NULL);

	// get new userid from user
	if (DialogBoxParam (g_hinst, MAKEINTRESOURCE (IDD_NEWUSERID), 
			ppks->hwndMain, sAddUserDlgProc, (LPARAM)&ans)) 
	{
		// get valid passphrase, if required
		LoadString (g_hinst, IDS_SELKEYPASSPHRASE, sz, sizeof(sz)); 
		err = PGPclGetKeyPhrase (g_context, g_tlscontext,
				ppks->hwndMain, sz, ppks->keydbMain, key,
				NULL, &pPasskey, &sizePasskey);
		PGPclErrorBox (ppks->hwndMain, err);

		// now we have a valid passphrase, if required
		if (IsntPGPError (err)) 
		{
			// update from server
			if (ans.bSyncWithServer) 
			{
				if (!PKGetFromServerInternal (ppks, FALSE, FALSE, FALSE)) 
				{
					if (PKMessageBox (ppks->hwndMain, IDS_CAPTIONINFO, 
							IDS_QUERYCONTINUEADDING, 	
							MB_YESNO|MB_ICONQUESTION) == IDNO) 
					{
						bRetVal = FALSE;
					}
				}
			}
		
			if (bRetVal) 
			{
				// make sure we have enough entropy
				PGPclRandom (g_context, ppks->hwndMain, 0);

				if (pPasskey) 
				{
					err = PGPAddUserID (key, ans.pszUserID, 
							PGPOPasskeyBuffer (g_context, 
									pPasskey, sizePasskey),
							PGPOLastOption (g_context));
				}
				else 
				{
					err = PGPAddUserID (key, ans.pszUserID, 
							PGPOLastOption (g_context));
				}
					 						
				if (IsntPGPError (PGPclErrorBox (ppks->hwndMain, err))) 
				{
					PKKeyDBModified (ppks, PK_MOD_INDEX_0);
					PGPclKeyListUpdateTree (ppks->hKL, 
							kPGPclKeyListUpdateObject, key, FALSE);
					PGPclKeyListSetTree (ppks->hKL,
							kPGPclSelectSpecifiedOnly, key);

					// send to server
					if (ans.bSyncWithServer)
						PKSendToServer (ppks, kPGPclDefaultServer);
				}
				else 
					bRetVal = FALSE;
			}
		}
		else 
			bRetVal = FALSE;

		pkFree (ans.pszUserID);
	}
	else 
		bRetVal = FALSE;

	if (pPasskey) 
	{
		PGPclFreePasskey (pPasskey, sizePasskey);
		pPasskey = NULL;
	}

	return bRetVal;
}


//	___________________________________________________
//
//  display photo userID with appropriate overwriting

static VOID 
sPaintPhotoID (
		HWND		hwnd,
		HBITMAP		hbitmapID,
		HPALETTE	hpaletteID,
		INT			iwidthBM,
		INT			iheightBM)
{
	HPALETTE		hpaletteOld		= NULL;
	HDC				hdc;
	HDC				hdcMem;
	PAINTSTRUCT		ps;
	RECT			rc;
	INT				icent;
	INT				ileft, itop, iwidth, iheight;

	hdc = BeginPaint (hwnd, &ps);

	if (hbitmapID) {
		GetWindowRect (GetDlgItem (hwnd, IDC_PHOTOID), &rc);
		MapWindowPoints (NULL, hwnd, (LPPOINT)&rc, 2);

		// check if bitmap needs shrinking
		if ((iheightBM > (rc.bottom-rc.top-2)) ||
			(iwidthBM  > (rc.right-rc.left-2))) 
		{
			if (iheightBM > (iwidthBM * 1.25)) 
			{
				itop = rc.top +1;
				iheight = rc.bottom-rc.top -2;
				icent = (rc.right+rc.left) / 2;
				iwidth = ((rc.bottom-rc.top) * iwidthBM) / iheightBM;
				ileft = icent -(iwidth/2);
			}
			else 
			{
				ileft = rc.left +1;
				iwidth = rc.right-rc.left -2;
				icent = (rc.bottom+rc.top) / 2;
				iheight = ((rc.right-rc.left) * iheightBM) / iwidthBM;
				itop = icent - (iheight/2);
			}
		}
		// otherwise draw it at its real size
		else
		{
			iwidth = iwidthBM;
			iheight = iheightBM;
			icent = (rc.right+rc.left) / 2;
			ileft = icent - (iwidth/2);
			icent = (rc.bottom+rc.top) / 2;
			itop = icent - (iheight/2);
		}

		hdcMem = CreateCompatibleDC (hdc);

		if (hpaletteID) 
		{
			hpaletteOld = SelectPalette (hdc, hpaletteID, FALSE);
			RealizePalette (hdc);
		}

		SetStretchBltMode (hdc, COLORONCOLOR);
		SelectObject (hdcMem, hbitmapID);
		StretchBlt (hdc, ileft, itop, iwidth, iheight,
					hdcMem, 0, 0, iwidthBM, iheightBM, SRCCOPY);

		if (hpaletteOld) 
		{
			SelectPalette (hdc, hpaletteOld, TRUE);
			RealizePalette (hdc);
		}

		DeleteDC (hdcMem);
	}

	EndPaint (hwnd, &ps);
} 


//	___________________________________________________
//
//  update system palette

static BOOL
sUpdatePalette (
		HWND			hwnd,
		PADDPHOTOSTRUCT	paps)
{
	BOOL		bretval		= FALSE;
	HDC			hdc;
	HPALETTE	hpaletteOld;

	if (paps->hpalettePhotoID == NULL) 
		return FALSE;

	hdc = GetDC (hwnd);

	hpaletteOld = SelectPalette (hdc, paps->hpalettePhotoID, FALSE);
	if (RealizePalette (hdc)) 
	{
		InvalidateRect (hwnd, NULL, TRUE); 
		bretval = TRUE;
	}

	SelectPalette (hdc, hpaletteOld, TRUE);
	RealizePalette (hdc);
	ReleaseDC (hwnd, hdc);

	return bretval;
}


//	___________________________________________________
//
//  PhotoID subclass procedure

static LRESULT APIENTRY 
sPhotoIDSubclassProc (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam, 
		LPARAM	lParam) 
{
	PADDPHOTOSTRUCT paps;
	paps = (PADDPHOTOSTRUCT)GetWindowLong (GetParent (hwnd), GWL_USERDATA);

    switch (uMsg) {

	case WM_CONTEXTMENU :
	{
		HMENU	hMC;
		HMENU	hMenuTrackPopup;

		hMC = LoadMenu (g_hinst, MAKEINTRESOURCE (IDR_MENUNEWPHOTOID));
		if (paps->hbitmapPhotoID)
			EnableMenuItem (hMC, IDM_COPYBITMAP, MF_BYCOMMAND|MF_ENABLED);
		else
			EnableMenuItem (hMC, IDM_COPYBITMAP, MF_BYCOMMAND|MF_GRAYED);

		if (IsClipboardFormatAvailable (CF_BITMAP) ||
			 IsClipboardFormatAvailable (CF_HDROP))
		{
			EnableMenuItem (hMC, IDM_PASTEBITMAP, MF_BYCOMMAND|MF_ENABLED);
		}
		else
			EnableMenuItem (hMC, IDM_PASTEBITMAP, MF_BYCOMMAND|MF_GRAYED);

		hMenuTrackPopup = GetSubMenu (hMC, 0);

		TrackPopupMenu (hMenuTrackPopup, TPM_LEFTALIGN|TPM_RIGHTBUTTON,
				(SHORT)LOWORD(lParam), (SHORT)HIWORD(lParam), 
				0, GetParent (hwnd), NULL);

		DestroyMenu (hMC);
		break;
	}

	case WM_DROPFILES :
	{
		CHAR				szFile[MAX_PATH];
		INT					iLen;
		LPBITMAPINFO		lpbmi;
		LPBYTE				pPhoto;
		INT					isize;
		PGPError			err;
		HWND				hwndParent;

		iLen = DragQueryFile ((HDROP)wParam, 0, szFile, sizeof(szFile));
		hwndParent = GetParent (hwnd);

		err = PGPclReadPhotoFromFile (g_context, szFile, &pPhoto, &isize);
		if (IsntPGPError (PGPclErrorBox (hwnd, err))) 
		{
			SetActiveWindow (GetParent (hwndParent));
			if (paps->pPhotoBuffer) 
				PGPFreeData (paps->pPhotoBuffer);
			if (paps->hbitmapPhotoID) 
				DeleteObject (paps->hbitmapPhotoID);
			if (paps->hpalettePhotoID) 
				DeleteObject (paps->hpalettePhotoID);

			paps->pPhotoBuffer = pPhoto;
			paps->iPhotoBufferLength = isize;
			EnableWindow (GetDlgItem (hwndParent, IDOK), TRUE); 

			err = PGPclDIBfromPhoto (g_context, pPhoto, isize, TRUE, &lpbmi);

			if (IsntPGPError (err)) 
			{
				PGPclGetDIBSize (lpbmi, &(paps->iwidthPhotoID),
						&(paps->iheightPhotoID), NULL);

				PGPclDDBfromDIB (lpbmi, 
						&(paps->hbitmapPhotoID),
						&(paps->hpalettePhotoID)); 

				PGPFreeData (lpbmi);

				InvalidateRect (hwndParent, NULL, TRUE);
			}
		} 

		DragFinish ((HDROP)wParam);
		break;
	}

	default :
		return CallWindowProc (paps->wpOrigPhotoIDProc, 
				hwnd, uMsg, wParam, lParam); 
	}
	return TRUE;
} 


//	___________________________________________________
//
//  select BMP file 

static VOID 
sSelectBitmapFile (
		HWND			hwnd,
		PADDPHOTOSTRUCT	paps)
{
	LPBYTE			pPhoto;
	INT				isize;
	CHAR			szFile[MAX_PATH];
	CHAR			szInitDir[MAX_PATH];
	CHAR			szTitle[128];
	CHAR			szFilter[128];
	CHAR*			p;
	PGPError		err;
	OPENFILENAME	ofn;
	LPBITMAPINFO	lpbmi;

	szFile[0] = '\0';

	LoadString (g_hinst, IDS_BITMAPFILTER, szFilter, sizeof(szFilter));
	while (p = strrchr (szFilter, '@')) *p = '\0';

	LoadString (g_hinst, IDS_OPENBITMAPTITLE, szTitle, sizeof(szTitle));
	PGPclGetPath (kPGPclDefaultOpenFolder, szInitDir, sizeof(szInitDir));

	ofn.lStructSize       = sizeof (OPENFILENAME);
	ofn.hwndOwner         = hwnd;
	ofn.hInstance         = (HANDLE)g_hinst;
	ofn.lpstrFilter       = szFilter;
	ofn.lpstrCustomFilter = (LPTSTR)NULL;
	ofn.nMaxCustFilter    = 0L;
	ofn.nFilterIndex      = 1L;
	ofn.lpstrFile         = szFile;
	ofn.nMaxFile          = sizeof (szFile);
	ofn.lpstrFileTitle    = szTitle;
	ofn.nMaxFileTitle     = 0;
	ofn.lpstrInitialDir   = szInitDir;
	ofn.lpstrTitle        = szTitle;
	ofn.Flags			  = OFN_HIDEREADONLY|OFN_NOCHANGEDIR;
	ofn.nFileOffset       = 0;
	ofn.nFileExtension    = 0;
	ofn.lpstrDefExt       = NULL;
	ofn.lCustData         = 0;

	if (!GetOpenFileName (&ofn)) 
		return;

	err = PGPclReadPhotoFromFile (g_context, szFile, &pPhoto, &isize);

	if (IsntPGPError (PGPclErrorBox (hwnd, err))) 
	{
		if (paps->pPhotoBuffer) 
			PGPFreeData (paps->pPhotoBuffer);
		if (paps->hbitmapPhotoID) 
			DeleteObject (paps->hbitmapPhotoID);
		if (paps->hpalettePhotoID) 
			DeleteObject (paps->hpalettePhotoID);

		paps->pPhotoBuffer = pPhoto;
		paps->iPhotoBufferLength = isize;
		EnableWindow (GetDlgItem (hwnd, IDOK), TRUE);

		err = PGPclDIBfromPhoto (g_context, pPhoto, isize, TRUE, &lpbmi);

		if (IsntPGPError (err)) 
		{
			PGPclGetDIBSize (lpbmi, &(paps->iwidthPhotoID),
					&(paps->iheightPhotoID), NULL);

			PGPclDDBfromDIB (lpbmi, 
					&(paps->hbitmapPhotoID),
					&(paps->hpalettePhotoID)); 

			PGPFreeData (lpbmi);

			InvalidateRect (hwnd, NULL, TRUE);
		}
	} 
}


//	___________________________________________________
//
//  Add PhotoID Dialog Message procedure

static BOOL CALLBACK 
sAddPhotoDlgProc (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam,
		LPARAM	lParam) 
{
	PADDPHOTOSTRUCT paps;
	LPBITMAPINFO	lpbmi;

	switch(uMsg) {

	case WM_INITDIALOG:
		paps = (PADDPHOTOSTRUCT)lParam;
		SetWindowLong (hwnd, GWL_USERDATA, lParam);

		// subclass photoID control to handle dropping, dragging
		paps->wpOrigPhotoIDProc = 
			(WNDPROC) SetWindowLong(GetDlgItem (hwnd, IDC_PHOTOID), 
						GWL_WNDPROC, (LONG) sPhotoIDSubclassProc); 

		return TRUE;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aNewPhotoIds);

	case WM_DESTROY :
 		paps = (PADDPHOTOSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		SetWindowLong (GetDlgItem(hwnd, IDC_PHOTOID), 
							GWL_WNDPROC, (LONG)paps->wpOrigPhotoIDProc); 
		break;

	case WM_PAINT :
		paps = (PADDPHOTOSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		sPaintPhotoID (hwnd, paps->hbitmapPhotoID, paps->hpalettePhotoID,
				paps->iwidthPhotoID, paps->iheightPhotoID);
		break;

	case WM_PALETTECHANGED :
		if ((HWND)wParam != hwnd) 
		{
			paps = (PADDPHOTOSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			sUpdatePalette (hwnd, paps);
		}
		break;

	case WM_QUERYNEWPALETTE :
		paps = (PADDPHOTOSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		return (sUpdatePalette (hwnd, paps));

	case WM_COMMAND:
		paps = (PADDPHOTOSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
		switch(LOWORD (wParam)) {
		case IDOK: 
			EndDialog (hwnd, 1);
			return TRUE;

		case IDCANCEL:
			EndDialog (hwnd, 0);
			return TRUE;

		case IDC_SELECTFILE :
			sSelectBitmapFile (hwnd, paps);
			break;

		case IDM_COPYBITMAP :
			PGPclCopyPhotoToClipboard (g_context, hwnd, 
					paps->pPhotoBuffer, paps->iPhotoBufferLength);
			break;

		case IDM_PASTEBITMAP :
		{
			LPBYTE		pPhoto				= NULL;
			INT			isize				= 0;
			PGPError	err					= kPGPError_UserAbort;
			INT			iLen;
			CHAR		szFile[MAX_PATH];
			HANDLE		hMem;
			HDROP		hDrop;

			if (IsClipboardFormatAvailable (CF_DIB))
			{
				err = PGPclPastePhotoFromClipboard (g_context, 
						hwnd, &pPhoto, &isize);
			}

			if (IsPGPError (err)) 
			{
				if (IsClipboardFormatAvailable (CF_HDROP)) 
				{
					if (OpenClipboard (hwnd)) 
					{ 
						hMem = GetClipboardData (CF_HDROP); 
						if (hMem) 
						{
							hDrop = GlobalLock (hMem);
							iLen = DragQueryFile (hDrop, 
									0, szFile, sizeof(szFile));

							err = PGPclReadPhotoFromFile (
									g_context, szFile, &pPhoto, &isize);

							GlobalUnlock (hMem);
						}
						CloseClipboard ();
					}
				}
			}

			if (IsntPGPError (PGPclErrorBox (hwnd, err)))
			{
				if (paps->pPhotoBuffer) 
					PGPFreeData (paps->pPhotoBuffer);
				if (paps->hbitmapPhotoID) 
					DeleteObject (paps->hbitmapPhotoID);
				if (paps->hpalettePhotoID) 
					DeleteObject (paps->hpalettePhotoID);

				paps->pPhotoBuffer = pPhoto;
				paps->iPhotoBufferLength = isize;
				EnableWindow (GetDlgItem (hwnd, IDOK), TRUE);

				err = PGPclDIBfromPhoto (g_context, 
						pPhoto, isize, TRUE, &lpbmi);

				if (IsntPGPError (err)) 
				{
					PGPclGetDIBSize (lpbmi, &(paps->iwidthPhotoID),
							&(paps->iheightPhotoID), NULL);

					PGPclDDBfromDIB (lpbmi, 
							&(paps->hbitmapPhotoID),
							&(paps->hpalettePhotoID)); 

					PGPFreeData (lpbmi);

					InvalidateRect (hwnd, NULL, TRUE);
				}
			} 
			break;
		}
		}
		return TRUE;
	}
	return FALSE;
}


//	___________________________________________________
//
//  Add Photo UserID to key
//	This routine is called when the user chooses to add
//	a PhotoID to an existing secret key.

BOOL 
PKAddPhotoToKey (
		PGPKEYSSTRUCT*	ppks) 
{
	PGPPrefRef		prefref			= kInvalidPGPPrefRef;
	PGPByte*		pPasskey		= NULL;
	BOOL			bRetVal			= TRUE;
	PGPError		err;
	PGPKeyDBObjRef	key;
	PGPSize			sizePasskey;
	ADDPHOTOSTRUCT	aps;
	CHAR			sz[256];
	HCURSOR			hcursorOld;

	// initialize struct
	aps.ppks = ppks;
	aps.pPhotoBuffer = NULL;
	aps.hbitmapPhotoID = NULL;
	aps.hpalettePhotoID = NULL;

	if (IsPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		return FALSE;

	PGPGetPrefBoolean (prefref, 
			kPGPPrefKeyServerSyncOnAdd, &(aps.bSyncWithServer));

	// get selected key
	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &key, NULL, NULL);

	// check if there is a pre-existing photoid
	if (PKExistingPhotoID (key)) 
	{
		PKMessageBox (ppks->hwndMain, IDS_CAPTIONINFO,
				IDS_PHOTOIDALREADYEXISTS, MB_OK|MB_ICONINFORMATION);
		return FALSE;
	}

	// get new photoid from user
	if (DialogBoxParam (g_hinst, MAKEINTRESOURCE (IDD_NEWPHOTOID), 
					ppks->hwndMain, sAddPhotoDlgProc, (LPARAM)&aps)) 
	{
		// get valid passphrase
		LoadString (g_hinst, IDS_SELKEYPASSPHRASE, sz, sizeof(sz)); 
		err = PGPclGetKeyPhrase (g_context, g_tlscontext,
				ppks->hwndMain, sz, ppks->keydbMain, key,
				NULL, &pPasskey, &sizePasskey);
		PGPclErrorBox (ppks->hwndMain, err);

		// now we have a valid passphrase, if required
		if (IsntPGPError (err)) 
		{
			// update from server
			if (aps.bSyncWithServer) 
			{
				if (!PKGetFromServerInternal (ppks, FALSE, FALSE, FALSE)) 
				{
					if (PKMessageBox (ppks->hwndMain, IDS_CAPTIONINFO, 
							IDS_QUERYCONTINUEADDING, 	
							MB_YESNO|MB_ICONINFORMATION) == IDNO) 
					{
						bRetVal = FALSE;
					}
				}
			}
		
			if (bRetVal) 
			{
				// make sure we have enough entropy
				PGPclRandom (g_context, ppks->hwndMain, 0);

				hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
				if (pPasskey) 
				{
					err = PGPAddAttributeUserID (key,
							kPGPAttribute_Image,
							aps.pPhotoBuffer,
							aps.iPhotoBufferLength,
							PGPOPasskeyBuffer (g_context, 
								pPasskey, sizePasskey),
							PGPOLastOption (g_context));
				}
				else 
				{
					err = PGPAddAttributeUserID (key,
							kPGPAttribute_Image,
							aps.pPhotoBuffer,
							aps.iPhotoBufferLength,
							PGPOLastOption (g_context));
				}
				SetCursor (hcursorOld);
						
				if (IsntPGPError (PGPclErrorBox (ppks->hwndMain, err))) 
				{
					PKKeyDBModified (ppks, PK_MOD_INDEX_0);
					PGPclKeyListUpdateTree (ppks->hKL, 
							kPGPclKeyListUpdateObject, key, FALSE);
					PGPclKeyListSetTree (ppks->hKL,
							kPGPclSelectSpecifiedOnly, key);

					// send to server
					if (aps.bSyncWithServer) 
						PKSendToServer (ppks, kPGPclDefaultServer);
				}
				else 
					bRetVal = FALSE;
			}
		}
		else 
			bRetVal = FALSE;
	}
	else 
		bRetVal = FALSE;

	// free passkey
	if (pPasskey) 
	{
		PGPclFreePasskey (pPasskey, sizePasskey);
		pPasskey = NULL;
	}

	// free photoID objects
	if (aps.hbitmapPhotoID)
		DeleteObject (aps.hbitmapPhotoID);
	if (aps.hpalettePhotoID) 
		DeleteObject (aps.hpalettePhotoID);
	if (aps.pPhotoBuffer) 
		PGPFreeData (aps.pPhotoBuffer);

	return bRetVal;
}
