/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	PKKeyIO.c - implements various operations related to importing 
				and exporting keys  
	
	$Id: PKKeyIO.c,v 1.33 2002/08/06 20:09:46 dallen Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPkeysx.h"
#include "pgpVersionHeader.h"
#include "pgpImageList.h"

// system header files
#include <commdlg.h>
#include <mapi.h>

// typedefs
typedef struct {
	PGPBoolean	bX509;
	PGPBoolean	bEnableExportPrivKeys;
	PGPBoolean	bExportPrivKeys;
	PGPBoolean	bEnableCompatible;
	PGPBoolean	bCompatible;
} EXPORTPRIVKEYSTRUCT, *PEXPORTPRIVKEYSTRUCT;

// external globals
extern HINSTANCE		g_hinst;
extern PGPContextRef	g_context;
extern PGPtlsContextRef	g_tlscontext;

// local globals
static BOOL s_bDraggingOut = FALSE;	//NB: valid across keymanagers!
									//prevents bonus selective import dialog

static DWORD aExportKeyIds[] = {			// Help IDs
    IDC_EXPORTPRIVATEKEYS,		IDH_PGPPK_EXPORTPRIVATEKEYS, 
    IDC_INCLUDEEXTENSIONS,		IDH_PGPPK_INCLUDEEXTENSIONS, 
    0,0 
}; 

//	_______________________________________________
//
//  Evaluate filename for PGP data vs. X.509 data

static PGPError 
sCreateFileImportOptions (
		PGPKEYSSTRUCT*		ppks,
		LPSTR				pszFilename,
		PGPFileSpecRef		fileref,
		BOOL				bFirst,
		PGPOptionListRef*	pOptions) 
{
	PGPInputFormat	format			= kPGPInputFormat_Unknown;
	CHAR			szPrompt[64];
	LPSTR			p;
	UINT			u;
	LPSTR			psz;
	PGPError		err;

	u = lstrlen (pszFilename);

	if (u > 4)
	{
		p = &pszFilename[u-4];

		if (stricmp (p, ".pem") == 0)
			format = kPGPInputFormat_PEMEncodedX509Cert;
		else if ((stricmp (p, ".pfx") == 0) ||
			(stricmp (p, ".p12") == 0))
		{
			format = kPGPInputFormat_PKCS12;
		}
	}

	err = PGPBuildOptionList (g_context, pOptions,
					PGPOInputFile (g_context, fileref),
					PGPOInputFormat (g_context, format),
					PGPOLastOption (g_context));

	// if format is PKCS-12 and first time, try w/o passphrase
	// if PKCS-12 and not first time, then we need a passphrase
	if ((format == kPGPInputFormat_PKCS12) && !bFirst)
	{
		LoadString (g_hinst, IDS_PKCS12PROMPT, szPrompt, sizeof(szPrompt));
		p = strrchr (pszFilename, '\\');
		if (p)
			p++;
		else
			p = pszFilename;

		lstrcat (szPrompt, p);

		err = PGPclGetConventionalPhrase (
					g_context, ppks->hwndMain, szPrompt, &psz);

		if (IsntPGPError (err))
		{
			err = PGPAppendOptionList (*pOptions, 
					PGPOPassphrase (g_context, psz),
					PGPOLastOption (g_context));
		}

		PGPclFreePhrase (psz);
	}

	return err;
}


//	_______________________________________________
//
//  Evaluate pasted buffer for PGP data vs. X.509 data

static PGPInputFormat 
sEvaluateBuffer (
		LPSTR	pszOrig,
		LPSTR*	ppszBegin) 
{
	CHAR	szBegin[]		= "-----BEGIN";
	CHAR	szBeginPGP[]	= "-----BEGIN PGP";

	PGPInputFormat	format	= kPGPInputFormat_Unknown;

	*ppszBegin = strstr (pszOrig, szBegin);
	if (*ppszBegin)
	{
		if (!strncmp (*ppszBegin, szBeginPGP, sizeof(szBeginPGP)-1))
			format = kPGPInputFormat_PGP;
		else
			format = kPGPInputFormat_PEMEncodedX509Cert;
	}
	else
		*ppszBegin = pszOrig;

	return format;
}


//	_______________________________________________
//
//  Import key from text file
//
//	hDrop	== NULL	=> prompt user for file name
//			!= NULL => hDrop is a handle passed in from 
//						the WM_DROPFILE message

BOOL 
PKInputFile (
		PGPKEYSSTRUCT*	ppks,
		HDROP			hDrop) 
{
	PGPOptionListRef	optionsImport		= kInvalidPGPOptionListRef;
	PGPError			err					= kPGPError_NoErr;
	BOOL				bImport				= FALSE;
	BOOL				bImported			= FALSE;

	OPENFILENAME		OpenFileName;
	CHAR				szFile[MAX_PATH];
	CHAR				szInitDir[MAX_PATH];
	CHAR				szFilter[256];
	CHAR				szTitle[64];
	CHAR*				p;
	INT					iIndex;
	BOOL				bFirst;
	PGPFileSpecRef		fileref;
	PGPKeyDBRef			keydb;
	UINT				uType;

	// files have been dropped
	if (hDrop) 
	{
		iIndex = 0;

		while (DragQueryFile (hDrop, iIndex, szFile, sizeof(szFile))) 
		{
			if (PKFileIsForImporting (szFile, &uType, NULL))
			{
				if (uType == PK_EXPORTEDGROUPFILE)
				{
					if (IsntPGPError (PGPgmImportGroups (ppks->hGM, szFile)))
					{
						bImported = TRUE;
						ppks->bGroupsVisible = FALSE;
						PostMessage (ppks->hwndMain, WM_COMMAND,
								MAKEWPARAM (IDM_VIEWGROUPS, 0), 0);
					}
				}
				else
				{
					PGPNewFileSpecFromFullPath (g_context, szFile, &fileref);
					if (PGPFileSpecRefIsValid (fileref))
					{
						bFirst = TRUE;
						do 
						{
							if (PGPOptionListRefIsValid (optionsImport))
								PGPFreeOptionList (optionsImport);

							err = sCreateFileImportOptions (ppks,
									szFile, fileref, bFirst, &optionsImport);

							if (IsntPGPError (err))
							{
								err = PGPImport (g_context, 
										&keydb, optionsImport,
										PGPOLastOption (g_context));
							}

							if (!bFirst || (err != kPGPError_BadPassphrase))
								PGPclErrorBox (ppks->hwndMain, err);

							bFirst = FALSE;

						} while (err == kPGPError_BadPassphrase);

						if (IsntPGPError (err) && PGPKeyDBRefIsValid (keydb))
						{
							PGPUInt32	uNumKeys;

							PGPCountKeys (
									PGPPeekKeyDBRootKeySet (keydb), &uNumKeys);
							if (uNumKeys > 0) 
							{
								bImported = TRUE;

								if (s_bDraggingOut) 
								{
									if (IsntPGPError (PGPclErrorBox (NULL, 
											PGPCopyKeys (
												PGPPeekKeyDBRootKeySet (keydb), 
												ppks->keydbMain, NULL))))
									{
										bImport = TRUE;
									}
								}
								else 
								{
									if (PGPclImportKeys (g_context, 
											g_tlscontext, ppks->hwndMain, 
											PGPPeekKeyDBRootKeySet (keydb), 
											ppks->keydbMain, 
											kPGPclDefaultImportFlags) 
										== kPGPError_NoErr) 
									{
										bImport = TRUE;
									}
								}
							}
							if (PGPKeyDBRefIsValid (keydb))
								PGPFreeKeyDB (keydb);
						}

						if (PGPOptionListRefIsValid (optionsImport))
						{
							PGPFreeOptionList (optionsImport);
							optionsImport = kInvalidPGPOptionListRef;
						}

						PGPFreeFileSpec (fileref);
					}
				}
			}
			else
			{
				bImported = TRUE;
				PKNewWindowForFile (ppks->hwndMain, ppks, FALSE, szFile);
			}
			iIndex++;
		}

		if (IsntPGPError (err) && !bImported) 
		{
			PKMessageBox (ppks->hwndMain, IDS_CAPTIONINFO, 
					IDS_NOIMPORTKEYS, MB_OK|MB_ICONINFORMATION);
		}
	}

	// files weren't dropped, query user for file to import
	else 
	{
		lstrcpy (szFile, "");
		LoadString (g_hinst, IDS_IMPORTFILTER, szFilter, sizeof(szFilter));
		while (p = strrchr (szFilter, '@')) *p = '\0';
		LoadString (g_hinst, IDS_IMPORTCAPTION, szTitle, sizeof(szTitle));
		PGPclGetPath (kPGPclDefaultOpenFolder, szInitDir, sizeof(szInitDir));

		OpenFileName.lStructSize       = sizeof (OPENFILENAME);
		OpenFileName.hwndOwner         = ppks->hwndMain;
		OpenFileName.hInstance         = (HANDLE)g_hinst;
		OpenFileName.lpstrFilter       = szFilter;
		OpenFileName.lpstrCustomFilter = (LPTSTR)NULL;
		OpenFileName.nMaxCustFilter    = 0L;
		OpenFileName.nFilterIndex      = 1L;
		OpenFileName.lpstrFile         = szFile;
		OpenFileName.nMaxFile          = sizeof (szFile);
		OpenFileName.lpstrFileTitle    = NULL;
		OpenFileName.nMaxFileTitle     = 0;
		OpenFileName.lpstrInitialDir   = szInitDir;
		OpenFileName.lpstrTitle        = szTitle;
		OpenFileName.Flags			   = OFN_HIDEREADONLY|OFN_NOCHANGEDIR;
		OpenFileName.nFileOffset       = 0;
		OpenFileName.nFileExtension    = 0;
		OpenFileName.lpstrDefExt       = "";
		OpenFileName.lCustData         = 0;

		if (GetOpenFileName (&OpenFileName)) 
		{
			PGPNewFileSpecFromFullPath (g_context, szFile, &fileref);
			if (PGPFileSpecRefIsValid (fileref)) 
			{
				bFirst = TRUE;
				do 
				{
					if (PGPOptionListRefIsValid (optionsImport))
						PGPFreeOptionList (optionsImport);

					err = sCreateFileImportOptions (ppks,
							szFile, fileref, bFirst, &optionsImport);

					if (IsntPGPError (err))
					{
						err = PGPImport (g_context, &keydb, 
								optionsImport, PGPOLastOption (g_context));
					}

					if (!bFirst || (err != kPGPError_BadPassphrase))
						PGPclErrorBox (ppks->hwndMain, err);

					bFirst = FALSE;
				} while (err == kPGPError_BadPassphrase);

				if (IsntPGPError (err) && PGPKeyDBRefIsValid (keydb))
				{
					PGPUInt32	uNumKeys;
					
					PGPCountKeys (PGPPeekKeyDBRootKeySet (keydb), &uNumKeys);
					if (uNumKeys > 0) 
					{
						bImported = TRUE;

						if (PGPclImportKeys (g_context, 
								g_tlscontext, ppks->hwndMain, 
								PGPPeekKeyDBRootKeySet (keydb), 
								ppks->keydbMain, 
								kPGPclDefaultImportFlags) 
							== kPGPError_NoErr) 
						{
							bImport = TRUE;
						}
					}
					if (PGPKeyDBRefIsValid (keydb))
						PGPFreeKeyDB (keydb);
				}

				if (PGPOptionListRefIsValid (optionsImport))
				{
					PGPFreeOptionList (optionsImport);
					optionsImport = kInvalidPGPOptionListRef;
				}

				PGPFreeFileSpec (fileref);
			}
			if (IsntPGPError (err) && !bImported) 
			{
				PKMessageBox (ppks->hwndMain, IDS_CAPTIONINFO, 
						IDS_NOIMPORTKEYS, MB_OK|MB_ICONINFORMATION);
			}
		}
	}

	if (bImport) 
	{
		PKKeyDBModified (ppks, PK_MOD_INDEX_0);
		PGPclKeyListReloadKeys (ppks->hKL, TRUE);
	}

	return bImport;
}


//	_______________________________________________
//
//  Get appropriate comment string

static VOID 
sGetCommentString (
		PGPMemoryMgrRef	memMgr,
		LPSTR			psz, 
		UINT			uLen) 
{
	PGPPrefRef	prefref;

	psz[0] = '\0';

	if (IsntPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		PGPGetPrefStringBuffer (prefref, kPGPPrefComment, uLen, psz);
}

//	_______________________________________________
//
//  Get export format based on prefs

static PGPExportFormat 
sGetExportFormat (PGPMemoryMgrRef	memMgr)
{
	PGPBoolean	bCompatible		= TRUE;
	PGPPrefRef	prefref			= kInvalidPGPPrefRef;

	if (IsntPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		PGPGetPrefBoolean (prefref, kPGPPrefExportKeysCompatible, &bCompatible);

	if (bCompatible) 
		return kPGPExportFormat_Basic;
	else
		return kPGPExportFormat_Complete;
}


//	_______________________________________________
//
//  Export key(s) to text file

static BOOL 
sExportKeys (
		PGPKEYSSTRUCT*	ppks,
		PGPKeySetRef	keyset,
		PGPKeyDBObjRef	key,
		LPSTR			szFile,
		PGPBoolean		bExportPrivateKeys,
		PGPBoolean		bX509cert,
		PGPBoolean		bCompatible) 
{
	BOOL			bRetVal				= FALSE;
	PGPByte*		pPasskey			= NULL;
	LPSTR			pszPhrase			= NULL;
	PGPFileSpecRef	fileref;
	CHAR			szComment[256];
	PGPExportFormat	exportformat;
	PGPSize			sizePasskey;
	PGPError		err;

	if (bX509cert && bExportPrivateKeys)
	{
		err = PGPclGetKeyPhrase (g_context, g_tlscontext,
				ppks->hwndMain, NULL, ppks->keydbMain, key,
				&pszPhrase, &pPasskey, &sizePasskey);

		if (IsPGPError (PGPclErrorBox (ppks->hwndMain, err)))
			return FALSE;
	}

	PGPNewFileSpecFromFullPath (g_context, szFile, &fileref);
	if (PGPFileSpecRefIsValid (fileref))
	{	
		sGetCommentString (PGPPeekContextMemoryMgr (g_context), 
				szComment, sizeof(szComment));

		if (bX509cert)
			exportformat = kPGPExportFormat_X509Cert;
		else if (bCompatible)
			exportformat = kPGPExportFormat_Basic;
		else
			exportformat = kPGPExportFormat_Complete;

		if (IsntPGPError (PGPclErrorBox (NULL,
				PGPExport (g_context,
						PGPOExportKeySet (g_context, keyset), 
						PGPOOutputFile (g_context, fileref),
						PGPOExportFormat (g_context, 
								exportformat),
						PGPOVersionString (g_context, 
								pgpVersionHeaderString()),
						PGPOCommentString (g_context, szComment),
						PGPOExportPrivateKeys (g_context,
								bExportPrivateKeys),
						pszPhrase ?
							PGPOPassphrase (g_context, pszPhrase) :
							PGPONullOption (g_context),
						(bX509cert && !bExportPrivateKeys) ?
							PGPOArmorOutput (g_context, TRUE) :
							PGPONullOption (g_context),
						PGPOLastOption (g_context))))) 
		{
			bRetVal = TRUE;
		}
		PGPFreeFileSpec (fileref);
	}

	if (IsntNull (pszPhrase))
		PGPclFreePhrase (pszPhrase);
	if (IsntNull (pPasskey))
		PGPclFreePasskey (pPasskey, sizePasskey);

	return bRetVal;
}


//	___________________________
//
//  Compute name of export file

static VOID 
sGetX509ExportFileName (
		PGPKeyDBObjRef	cert,
		LPSTR			sz, 
		UINT			uLen,
		BOOL			bIncludeExtension) 
{
	CHAR			szDefExt[8];
	INT				i;

	PGPclGetPrimaryUserIDName (PGPPeekKeyDBObjKey (cert), sz, uLen, &i);

	i = strcspn (sz, "\\/:*?\"<>|");

	// truncate really long userids
	if (i > (MAX_PATH/2))
		i = MAX_PATH/2;

	sz[i--] = '\0';

	while ((i >= 0) && (sz[i] == ' '))
	{
		sz[i--] = '\0';
	}

	if (sz[0] == '\0') 
		LoadString (g_hinst, IDS_DROPFILENAME, sz, uLen);

	if (bIncludeExtension)
	{
		lstrcat (sz, ".");
		LoadString (g_hinst, 
				IDS_DEFX509EXPORTEXTENSION, szDefExt, sizeof(szDefExt));
		lstrcat (sz, szDefExt);
	}
}


//	___________________________
//
//  Compute name of export file

static VOID 
sGetPGPExportName (
		PGPKEYSSTRUCT*	ppks,
		LPSTR			sz, 
		UINT			uLen) 
{
	CHAR			szDefExt[8];
	PGPKeyDBObjRef	key;
	INT				i;
	PGPBoolean		bMultiple;

	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &key, &bMultiple, NULL);
	if (!bMultiple) 
	{
		PGPclGetPrimaryUserIDName (key, sz, uLen, &i);

		i = strcspn (sz, "\\/:*?\"<>|");

		// truncate really long userids
		if (i > (MAX_PATH/2))
			i = MAX_PATH/2;

		sz[i--] = '\0';

		while ((i >= 0) && (sz[i] == ' '))
		{
			sz[i--] = '\0';
		}

		if (sz[0] == '\0') 
			LoadString (g_hinst, IDS_DROPFILENAME, sz, uLen);
	}
	else 
		LoadString (g_hinst, IDS_DROPFILENAME, sz, uLen);

	lstrcat (sz, ".");
	LoadString (g_hinst, IDS_DEFEXPORTEXTENSION, szDefExt, sizeof(szDefExt));
	lstrcat (sz, szDefExt);
}


//----------------------------------------------------|
// Export Private Key dialog message procedure

static BOOL CALLBACK 
sExportPrivKeyDlgProc (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam, 
		LPARAM	lParam) 
{
	OPENFILENAME*			pofn;
	PEXPORTPRIVKEYSTRUCT	pepks;

	switch (uMsg) {

	case WM_INITDIALOG :
		pofn = (OPENFILENAME*)lParam;
		SetWindowLong (hwnd, GWL_USERDATA, pofn->lCustData);
		pepks = (PEXPORTPRIVKEYSTRUCT)pofn->lCustData;

		if (!pepks->bEnableExportPrivKeys) 
			EnableWindow (GetDlgItem (hwnd, IDC_EXPORTPRIVATEKEYS), FALSE);
		if (!pepks->bEnableCompatible) 
			EnableWindow (GetDlgItem (hwnd, IDC_INCLUDEEXTENSIONS), FALSE);

		pepks->bExportPrivKeys = FALSE;
		if (!pepks->bCompatible)
			CheckDlgButton (hwnd, IDC_INCLUDEEXTENSIONS, BST_CHECKED);
		break;

	case WM_HELP :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aExportKeyIds);

	case WM_COMMAND:

		switch (LOWORD(wParam)) {
		case IDC_EXPORTPRIVATEKEYS :
			pepks = (PEXPORTPRIVKEYSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

			if (IsDlgButtonChecked (hwnd, IDC_EXPORTPRIVATEKEYS) == BST_CHECKED)
			{
				pepks->bExportPrivKeys = TRUE;
				if (pepks->bX509)
				{
					CHAR	sz[8];
					LoadString (g_hinst, 
							IDS_PKCS12EXPORTEXTENSION, sz, sizeof(sz));
					SendMessage (GetParent (hwnd), CDM_SETDEFEXT, 0, (LPARAM)sz);
				}
			}
			else 
			{
				pepks->bExportPrivKeys = FALSE;
				if (pepks->bX509)
				{
					CHAR	sz[8];
					LoadString (g_hinst, 
							IDS_DEFX509EXPORTEXTENSION, sz, sizeof(sz));
					SendMessage (GetParent (hwnd), CDM_SETDEFEXT, 0, (LPARAM)sz);
				}
			}
			return TRUE;

		case IDC_INCLUDEEXTENSIONS :
			pepks = (PEXPORTPRIVKEYSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			if (IsDlgButtonChecked (hwnd, IDC_INCLUDEEXTENSIONS) == BST_CHECKED)
				pepks->bCompatible = FALSE;
			else 
				pepks->bCompatible = TRUE;
			return TRUE;

		}
		return FALSE;
	}
	return FALSE;
}


//	_______________________________________________
//
//  construct keyset to export

static PGPError 
sGetKeysetToExport (
		PGPKEYSSTRUCT*	ppks,
		PGPKeySetRef*	pkeyset,
		PGPBoolean*		pbX509cert,
		PGPBoolean*		pbX509onDSS,
		PGPKeyDBObjRef*	pcert) 
{
	PGPKeyDBObjRef		obj			= kInvalidPGPKeyDBObjRef;
	PGPUInt32			uSel		= 0;
	PGPError			err;

	if (pbX509onDSS)
		*pbX509onDSS = FALSE;

	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &obj, NULL, &uSel);

	if (uSel == kPGPclKeyList_Key)
	{
		*pbX509cert = FALSE;
		err = PGPclKeyListGetSelectedKeys (ppks->hKL, pkeyset);
	}
	else if (uSel == kPGPclKeyList_Signature)
	{
		PGPKeyDBObjRef key;

		*pbX509cert = TRUE;
		key = PGPPeekKeyDBObjKey (obj);
		err = PGPNewOneKeySet (key, pkeyset); 

		if (pbX509onDSS)
		{
			PGPUInt32	uAlg;

			PGPGetKeyDBObjNumericProperty (key, 
					kPGPKeyProperty_AlgorithmID, &uAlg);

			if (uAlg == kPGPPublicKeyAlgorithm_DSA)
				*pbX509onDSS = TRUE;
		}
	}
	else
		err = kPGPError_BadParams;

	if (IsntNull (pcert))
		*pcert = obj;

	return err;
}


//	_______________________________________________
//
//  Export key to text file

BOOL 
PKExportKeys (
		PGPKEYSSTRUCT*	ppks,
		LPSTR			pszFile) 
{
	PGPKeySetRef		keysetExport		= kInvalidPGPKeySetRef;
	PGPPrefRef			prefref				= kInvalidPGPPrefRef;
	PGPError			err					= kPGPError_NoErr;
	PGPKeyDBObjRef		cert				= kInvalidPGPKeyDBObjRef;
	PGPUInt32			uSel				= 0;
	PGPBoolean			bX509onDSSKey		= FALSE;

	OPENFILENAME		OpenFileName;
	EXPORTPRIVKEYSTRUCT	epks;
	CHAR				szFile[MAX_PATH];
	CHAR				szInitDir[MAX_PATH];
	CHAR				szDefExt[8];
	CHAR				szFilter[256];
	CHAR				szTitle[64];
	CHAR*				p;
	BOOL				bRetVal;

	sGetKeysetToExport (ppks, 
			&keysetExport, &epks.bX509, &bX509onDSSKey, &cert);

	if (!PGPKeySetRefIsValid (keysetExport))
		return FALSE;

	epks.bEnableCompatible = !epks.bX509;

	if (pszFile) 
	{
		if (IsntPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		{
			PGPGetPrefBoolean (prefref, 
					kPGPPrefExportKeysCompatible, &epks.bCompatible);

			bRetVal = sExportKeys (ppks, keysetExport, 
					PGPPeekKeyDBObjKey (cert), pszFile, 
					FALSE, epks.bX509, epks.bCompatible);
		}
	}
	else 
	{
		OpenFileName.lStructSize       = sizeof (OPENFILENAME);
		OpenFileName.hwndOwner         = ppks->hwndMain;
		OpenFileName.hInstance         = (HANDLE)g_hinst;
		PGPclGetPath (kPGPclDefaultSaveAsFolder, szInitDir, sizeof(szInitDir));

		if (epks.bX509)
		{
			LoadString (g_hinst, 
					IDS_X509EXPORTCAPTION, szTitle, sizeof(szTitle));
			LoadString (g_hinst, 
					IDS_EXPORTFILTERX509, szFilter, sizeof(szFilter));

			sGetX509ExportFileName (cert, szFile, sizeof (szFile), TRUE);

			LoadString (g_hinst, IDS_DEFX509EXPORTEXTENSION, szDefExt, 
					sizeof(szDefExt));
		}
		else
		{
			LoadString (g_hinst, 
					IDS_EXPORTCAPTION, szTitle, sizeof(szTitle));
			LoadString (g_hinst, 
					IDS_EXPORTFILTER, szFilter, sizeof(szFilter));

			LoadString (g_hinst, IDS_DEFEXPORTEXTENSION, szDefExt, 
					sizeof(szDefExt));

			sGetPGPExportName (ppks, szFile, sizeof (szFile));
		}

		while (p = strrchr (szFilter, '@')) *p = '\0';
		OpenFileName.lpstrFilter	   = szFilter;
		OpenFileName.lpstrCustomFilter = (LPTSTR)NULL;
		OpenFileName.nMaxCustFilter    = 0L;
		OpenFileName.nFilterIndex      = 1L;
		OpenFileName.lpstrFile         = szFile;
		OpenFileName.nMaxFile          = sizeof (szFile);
		OpenFileName.lpstrFileTitle    = NULL;
		OpenFileName.nMaxFileTitle     = 0;
		OpenFileName.lpstrInitialDir   = szInitDir;
		OpenFileName.lpstrTitle        = szTitle;
		OpenFileName.Flags			   = OFN_EXPLORER|
										 OFN_ENABLETEMPLATE|
										 OFN_ENABLEHOOK|
										 OFN_HIDEREADONLY|
										 OFN_OVERWRITEPROMPT|
										 OFN_NOREADONLYRETURN|
										 OFN_NOCHANGEDIR;
		OpenFileName.nFileOffset       = 0;
		OpenFileName.nFileExtension    = 0;
		OpenFileName.lpstrDefExt       = szDefExt;

		OpenFileName.lCustData         = (DWORD)&epks;
		OpenFileName.lpfnHook          = sExportPrivKeyDlgProc; 
		OpenFileName.lpTemplateName    = 
					MAKEINTRESOURCE (IDD_EXPORTDLGEXTENSIONS);

		if (bX509onDSSKey)
			epks.bEnableExportPrivKeys = FALSE;
		else
		{
			epks.bEnableExportPrivKeys = 
					PKCheckForNonTokenSecretKeys (keysetExport);
		}

		if (IsntPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		{
			PGPGetPrefBoolean (prefref, kPGPPrefExportKeysCompatible, 
					&epks.bCompatible);

			if (GetSaveFileName (&OpenFileName)) 
			{
				bRetVal = sExportKeys (ppks, keysetExport, 
						PGPPeekKeyDBObjKey (cert), szFile, 
						epks.bExportPrivKeys, epks.bX509, epks.bCompatible);
			}
		}
	}

	if (PGPKeySetRefIsValid (keysetExport)) 
		PGPFreeKeySet (keysetExport);

	return TRUE;
}


//	_______________________________________________
//
//	called by OLE when DropTarget asks for data

PVOID 
PKDropKeysGetData (
		PGPKEYSSTRUCT*	ppks,
		BOOL			bKeyset,
		UINT*			puLen)
{
	PGPKeySetRef	keysetDrop		= kInvalidPGPKeySetRef;
	PGPError		err				= kPGPError_NoErr;
	DWORD			dwEffect		= 0;
	BOOL			bOKtoDelete		= FALSE;
	PGPSize			slen			= 0;
	PVOID			pExport			= NULL;
	LPSTR			pszExport		= NULL;
	PGPBoolean		bX509cert		= FALSE;

	CHAR			szComment[256];
	HCURSOR			hcursorOld;
	PGPExportFormat exportformat;

	hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));

	err = sGetKeysetToExport (ppks, &keysetDrop, &bX509cert, NULL, NULL);
	if (!PGPKeySetRefIsValid (keysetDrop))
		return NULL;

	if (bKeyset)
	{
		pExport = PGPNewData (PGPPeekContextMemoryMgr (g_context), 
				sizeof(PGPKeySetRef), 0);
		memcpy (pExport, &keysetDrop, sizeof(PGPKeySetRef));
		*puLen = sizeof(PGPKeySetRef);
		return pExport;
	}

	// get comment string
	slen = 0;
	sGetCommentString (PGPPeekContextMemoryMgr (g_context), 
			szComment, sizeof(szComment));

	// determine export format
	if (bX509cert)
		exportformat = kPGPExportFormat_X509Cert;
	else 
		exportformat = sGetExportFormat (PGPPeekContextMemoryMgr (g_context));

	// export to buffer
	if (IsntPGPError (PGPclErrorBox (NULL,
		PGPExport (g_context,
				PGPOExportKeySet (g_context, keysetDrop),
				PGPOAllocatedOutputBuffer (g_context,
						&pszExport, 0x40000000, &slen),
				PGPOExportFormat (g_context, exportformat),
				PGPOVersionString (g_context, 
						pgpVersionHeaderString()),
				PGPOCommentString (g_context, szComment),
				bX509cert ?
						PGPOArmorOutput (g_context, TRUE) :
						PGPONullOption (g_context),
				PGPOLastOption (g_context))))) 
	{
		if (pszExport) 
			pszExport[slen-2] = '\0';
	}

	if (PGPKeySetRefIsValid (keysetDrop))
		PGPFreeKeySet (keysetDrop);

	SetCursor (hcursorOld);

	*puLen = slen-2;
	return pszExport;		
}


//	_______________________________________________
//
//	called by OLE when DropTarget releases data

VOID 
PKDropKeysFreeData (
		BOOL	bKeySet,
		PVOID	p) 
{
	PGPKeySetRef	keyset;

	if (IsntNull (p))
	{
		if (bKeySet)
		{
			memcpy (&keyset, p, sizeof(PGPKeySetRef));
			if (PGPKeySetRefIsValid (keyset))
				PGPFreeKeySet (keyset);
		}

		PGPFreeData (p);
	}
}


//	_______________________________________________
//
//  Drag selected key or keys out of application onto 
//  desktop or into Explorer.  Uses OLE interface.

static BOOL 
sDragAndDropKeys (
		PGPKEYSSTRUCT*	ppks)
{
	BOOL			bOKtoDelete			= FALSE;
	DWORD			dwEffect			= 0;
	LPDATAOBJECT	lpdo;
	LPDROPSOURCE	lpds;
	CHAR			szFile[MAX_PATH];

	sGetPGPExportName (ppks, szFile, sizeof(szFile));

	lpdo = PKCreateDataObject (ppks, szFile);
	lpds = PKCreateDropSource (
			ppks, ppks->hwndMain, ppks->hwndTreeList, FALSE);

	if (lpds) 
	{
		if (lpdo) 
		{
			s_bDraggingOut = TRUE;
			DoDragDrop (lpdo, lpds, 
				DROPEFFECT_COPY|DROPEFFECT_MOVE, &dwEffect); 
			bOKtoDelete = PKOKToDeleteDataObject (lpdo);
			lpdo->lpVtbl->Release(lpdo);
			s_bDraggingOut = FALSE;
		}
		lpds->lpVtbl->Release(lpds);
	}

	if (dwEffect == DROPEFFECT_MOVE) 
	{
		if (bOKtoDelete) 
		{
			if (PKIsDeleteEnabled (ppks)) 
			{
				PKDeleteObject (ppks);
			}
		}
	}

	return TRUE;
}


//	_______________________________________________
//
//  Drag selected X.509 out of application onto 
//  desktop or into Explorer.  Uses OLE interface.

static BOOL 
sDragAndDropX509Cert (
		PGPKEYSSTRUCT*	ppks,
		PGPKeyDBObjRef	cert)
{
	BOOL			bOKtoDelete			= FALSE;
	DWORD			dwEffect			= 0;
	LPDATAOBJECT	lpdo;
	LPDROPSOURCE	lpds;
	CHAR			szFile[MAX_PATH];

	sGetX509ExportFileName (cert, szFile, sizeof(szFile), TRUE);

	lpdo = PKCreateDataObject (ppks, szFile);
	lpds = PKCreateDropSource (
			ppks, ppks->hwndMain, ppks->hwndTreeList, TRUE);

	if (lpds) 
	{
		if (lpdo) 
		{
			s_bDraggingOut = TRUE;
			DoDragDrop (lpdo, lpds, 
				DROPEFFECT_COPY|DROPEFFECT_MOVE, &dwEffect); 
			bOKtoDelete = PKOKToDeleteDataObject (lpdo);
			lpdo->lpVtbl->Release(lpdo);
			s_bDraggingOut = FALSE;
		}
		lpds->lpVtbl->Release(lpds);
	}

	if (dwEffect == DROPEFFECT_MOVE) 
	{
		if (bOKtoDelete) 
		{
			if (PKIsDeleteEnabled (ppks)) 
			{
				PKDeleteObject (ppks);
			}
		}
	}

	return TRUE;
}


//	_______________________________________________
//
//  Drag selected key(s) or X.509 cert out of application onto 
//  desktop or into Explorer.  Uses OLE interface.

BOOL 
PKDragAndDrop (
		PGPKEYSSTRUCT*	ppks)
{
	PGPUInt32		uSel, uFocus;
	PGPKeyDBObjRef	obj;

	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, &obj, NULL, &uSel);

	if (uSel == kPGPclKeyList_Key) 
		return sDragAndDropKeys (ppks);

	if (uSel == kPGPclKeyList_Signature)
	{
		if ((uFocus == IDX_X509CERT) ||
			(uFocus == IDX_X509REVCERT) ||
			(uFocus == IDX_X509EXPCERT))
		{
			return sDragAndDropX509Cert (ppks, obj);
		}
	}

	return FALSE;
}


//	_______________________________________________
//
//  Copy key(s) to clipboard

BOOL 
PKCopyKeys (
		PGPKEYSSTRUCT*	ppks,
		HANDLE*			phMem) 
{
	PGPKeySetRef	keysetCopy			= kInvalidPGPKeySetRef;
	PGPError		err					= kPGPError_NoErr;
	BOOL			bRetVal				= FALSE;
	PGPBoolean		bX509cert			= FALSE;
	size_t			slen;
	HANDLE			hMem;
	LPSTR			pMem;
	LPSTR			pMemG;
	CHAR			szComment[256];
	PGPExportFormat	exportformat;

	err = sGetKeysetToExport (ppks, &keysetCopy, &bX509cert, NULL, NULL);
	if (!PGPKeySetRefIsValid (keysetCopy))
		return FALSE;

	slen = 0;
	sGetCommentString (PGPPeekContextMemoryMgr (g_context), 
			szComment, sizeof(szComment));

	// determine export format
	if (bX509cert)
		exportformat = kPGPExportFormat_X509Cert;
	else 
		exportformat = sGetExportFormat (PGPPeekContextMemoryMgr (g_context));

	if (IsntPGPError (PGPclErrorBox (NULL,
		PGPExport (g_context, 
				PGPOExportKeySet (g_context, keysetCopy),
				PGPOAllocatedOutputBuffer (g_context,
						&pMem, 0x40000000, &slen),
				PGPOExportFormat (g_context, exportformat),
				PGPOVersionString (g_context, 
						pgpVersionHeaderString()),
				PGPOCommentString (g_context, szComment),
				bX509cert ?
						PGPOArmorOutput (g_context, TRUE) :
						PGPONullOption (g_context),
				PGPOLastOption (g_context))))) 
	{
		hMem = GlobalAlloc (GMEM_MOVEABLE|GMEM_DDESHARE, slen+1);
		if (hMem) 
		{
			pMemG = GlobalLock (hMem);
			if (pMem) 
			{
				pMem[slen-2] = '\0';
				memcpy (pMemG, pMem, slen+1);
				GlobalUnlock (hMem);
				if (phMem) 
				{
					*phMem = hMem;
					bRetVal = TRUE;
				}
				else 
				{
					if (OpenClipboard (NULL)) 
					{
						EmptyClipboard ();
						SetClipboardData (CF_TEXT, hMem);
						CloseClipboard ();
						bRetVal = TRUE;
					}
				}
				PGPFreeData (pMem);
			}
			else 
				GlobalFree (hMem);
		}
	}

	if (PGPKeySetRefIsValid (keysetCopy))
		PGPFreeKeySet (keysetCopy);

	return bRetVal;
}


//	_______________________________________________
//
//  Paste key(s) from clipboard

BOOL 
PKPasteKeys (
		PGPKEYSSTRUCT*	ppks)
{
	PGPKeyDBRef		keydb		= kInvalidPGPKeyDBRef;
	BOOL			bImport;
	BOOL			bKeys;
	HANDLE			hMem;
	LPSTR			pMem;
	size_t			sLen;
	HDROP			hDrop;

	bImport = FALSE;
	bKeys = FALSE;
	if (OpenClipboard (NULL)) 
	{
		hMem = GetClipboardData (CF_HDROP);
		if (hMem) 
		{
			hDrop = GlobalLock (hMem);
			bImport = PKInputFile (ppks, hDrop);
			GlobalUnlock (hMem);
			CloseClipboard ();
			return bImport;
		}
		hMem = GetClipboardData (CF_TEXT);
		if (hMem) 
		{
			pMem = GlobalLock (hMem);
			if (pMem) 
			{
				PGPInputFormat	format;
				LPSTR			pBegin;

				format = sEvaluateBuffer (pMem, &pBegin);
				sLen = lstrlen (pBegin);

				PGPImport (g_context, &keydb, 
						PGPOInputBuffer (g_context, pBegin, sLen),
						PGPOInputFormat (g_context, format),
						PGPOLastOption (g_context));

				if (PGPKeyDBRefIsValid (keydb)) 
				{
					PGPUInt32	uNumKeys;

					PGPCountKeys (PGPPeekKeyDBRootKeySet (keydb), &uNumKeys);
					if (uNumKeys > 0) 
					{
						bKeys = TRUE;

						if (PGPclImportKeys (g_context, g_tlscontext,
								ppks->hwndMain, 
								PGPPeekKeyDBRootKeySet (keydb), 
								ppks->keydbMain,
								kPGPclDefaultImportFlags) 
							== kPGPError_NoErr) 
						{
							bImport = TRUE;
						}
					}
					PGPFreeKeyDB (keydb);
				}
				GlobalUnlock (hMem);
			}
		}
		if (!bKeys) 
		{
			PKMessageBox (ppks->hwndMain, IDS_CAPTIONINFO, 
					IDS_NOIMPORTKEYSPASTE, MB_OK|MB_ICONINFORMATION);
		}
		CloseClipboard ();
	}

	if (bImport) 
	{
		PKKeyDBModified (ppks, PK_MOD_INDEX_0);
		PGPclKeyListReloadKeys (ppks->hKL, TRUE);
	}
	
	return bImport;
}


//	_______________________________________________
//
//  Drop text key(s)

BOOL 
PKDropKeys (
		PGPKEYSSTRUCT*	ppks,
		BOOL			bKeySet,
		HANDLE			hMem) 
{
	PGPKeyDBRef		keydb;
	PGPKeySetRef	keyset;
	PGPUInt32		numKeys;
	BOOL			bImport;
	BOOL			bKeys;
	LPSTR			pMem;
	size_t			sLen;

	bImport = FALSE;
	bKeys = FALSE;
	if (hMem) 
	{
		pMem = GlobalLock (hMem);
		if (pMem) 
		{
			if (bKeySet)
			{
				pgpCopyMemory (pMem, &keyset, sizeof(keyset));

				PGPCountKeys (keyset, &numKeys);
			
				if (numKeys > 0) 
				{
					bKeys = TRUE;

					if (IsntPGPError (PGPclErrorBox (NULL,
						PGPCopyKeys (keyset, ppks->keydbMain, NULL))))
					{
						PGPCalculateTrust (
								PGPPeekKeyDBRootKeySet (ppks->keydbMain),
								kInvalidPGPKeyDBRef);
						bImport = TRUE;
					}
				}
			}
			else
			{
				PGPInputFormat	format;
				LPSTR			pBegin;

				format = sEvaluateBuffer (pMem, &pBegin);
				sLen = lstrlen (pBegin);

				PGPImport (g_context, &keydb, 
						PGPOInputBuffer (g_context, pBegin, sLen),
						PGPOInputFormat (g_context, format),
						PGPOLastOption (g_context));

				if (PGPKeyDBRefIsValid (keydb))
				{
					keyset = PGPPeekKeyDBRootKeySet (keydb);
					PGPCountKeys (keyset, &numKeys);
				
					if (numKeys > 0) 
					{
						bKeys = TRUE;

						if (s_bDraggingOut) 
						{
							PGPCheckKeyRingSigs (keyset,
									ppks->keydbMain, TRUE, NULL, NULL);

							if (IsntPGPError (PGPclErrorBox (NULL,
								PGPCopyKeys (keyset, ppks->keydbMain, NULL))))
							{
								PGPCalculateTrust (
										PGPPeekKeyDBRootKeySet (ppks->keydbMain),
										kInvalidPGPKeyDBRef);
								bImport = TRUE;
							}
						}
						else 
						{
							if (PGPclImportKeys (g_context, g_tlscontext,
									ppks->hwndMain, keyset, 
									ppks->keydbMain, 0) == kPGPError_NoErr) 
							{
								bImport = TRUE;
							}
						}
					}
					PGPFreeKeyDB (keydb);
				}
			}

			GlobalUnlock (hMem);
		}
		if (!bKeys) 
		{
			PKMessageBox (ppks->hwndMain, IDS_CAPTIONINFO, 
					IDS_NODRAGTEXTKEYS, MB_OK|MB_ICONINFORMATION);
		}
	}

	if (bImport) 
	{
		PKKeyDBModified (ppks, PK_MOD_INDEX_0);
		PGPclKeyListReloadKeys (ppks->hKL, TRUE);
	}
	
	return bImport;
}


//	___________________________________________
//
//  Determine if there is text in the clipboard

BOOL 
PKDataToPaste (VOID) 
{
	if (IsClipboardFormatAvailable (CF_TEXT)) 
		return TRUE;
	if (IsClipboardFormatAvailable (CF_HDROP)) 
		return TRUE;
	return FALSE;
}


//	_______________________________________________
//
//  Send key(s) in email message

static PGPError
sSendKeysViaEmail (
		PGPKEYSSTRUCT*	ppks,
		PGPKeySetRef	keysetToSend)
{
	PGPBoolean		bCompatible		= TRUE;
	PGPPrefRef		prefref			= kInvalidPGPPrefRef;
	HMODULE			hMAPI32			= NULL;
	LPMAPISENDMAIL	pMAPISendMail	= NULL;
	PGPSize			slen			= 0;
	PGPError		err				= kPGPError_NoErr;

	char			szFilePath[MAX_PATH];
	char			szFileName[256];
	char			szSubject[128];
	char			szBody[256];
	MapiMessage		msg;
	MapiFileDesc	file;

	hMAPI32 = LoadLibrary ("mapi32.dll");
	if (IsntNull (hMAPI32))
	{
		pMAPISendMail = 
				(LPMAPISENDMAIL) GetProcAddress (hMAPI32, "MAPISendMail");
	}

	if (IsNull (pMAPISendMail))
		return kPGPError_FeatureNotAvailable;

	GetTempPath (sizeof(szFileName), szFileName);
	GetTempFileName (szFileName, "PGP", 0, szFilePath);
	if (PKExportKeys (ppks, szFilePath))
	{
		ULONG	ret;

		sGetPGPExportName (ppks, szFileName, sizeof(szFileName));

		pgpClearMemory (&file, sizeof(file));
		file.nPosition = -1;
		file.lpszPathName = szFilePath;
		file.lpszFileName = szFileName;

		LoadString (g_hinst, IDS_KEYEMAILSUBJECT, szSubject, sizeof(szSubject));
		LoadString (g_hinst, IDS_KEYEMAILBODY, szBody, sizeof(szBody));

		pgpClearMemory (&msg, sizeof(msg));
		msg.lpszSubject = szSubject;
		msg.lpszNoteText = szBody;
		msg.nFileCount = 1;
		msg.lpFiles = &file;

		ret = pMAPISendMail (0, (ULONG)(ppks->hwndMain), 
				&msg, MAPI_DIALOG|MAPI_LOGON_UI, 0);

		if ((ret == MAPI_E_USER_ABORT) ||
			(ret == SUCCESS_SUCCESS))
		{
			err = kPGPError_NoErr;
		}
		else if (ret == MAPI_E_LOGIN_FAILURE)
		{
			PKMessageBox (ppks->hwndMain, IDS_CAPTIONALERT, 
					IDS_CANNOTLOGINTOEMAIL, MB_OK|MB_ICONEXCLAMATION);
			err = kPGPError_LazyProgrammer;
		}
		else
		{
			PKMessageBox (ppks->hwndMain, IDS_CAPTIONALERT, 
					IDS_CANNOTSENDVIAEMAIL, MB_OK|MB_ICONEXCLAMATION);
			err = kPGPError_LazyProgrammer;
		}

		DeleteFile (szFilePath);
	}

	FreeLibrary(hMAPI32);

	return err;
}


BOOL 
PKSendViaEmail (
		PGPKEYSSTRUCT*	ppks) 
{
	PGPKeySetRef	keysetToSend		= kInvalidPGPKeySetRef;
	PGPError		err					= kPGPError_UnknownError;
	BOOL			bRetVal				= FALSE;
	PGPBoolean		bX509cert			= FALSE;

	err = sGetKeysetToExport (ppks, &keysetToSend, &bX509cert, NULL, NULL);
	if (!PGPKeySetRefIsValid (keysetToSend))
		return FALSE;

	if (!bX509cert)
	{
		err = sSendKeysViaEmail (ppks, keysetToSend);
	}

	if (PGPKeySetRefIsValid (keysetToSend))
		PGPFreeKeySet (keysetToSend);

	return (err == kPGPError_NoErr);
}


