/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	PKKeyOps.c - implements various operations performed on keys.

	$Id: PKKeyOps.c,v 1.56 2002/11/19 14:11:29 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPkeysx.h"
#include "pgpImageList.h"
#include "keydrawitem.h"
#include "pgpUnicodeWin32.h"
#include "UTF8Edit.h"

#include <process.h>

// constant definitions
#define BITMAP_WIDTH	16
#define BITMAP_HEIGHT	16

#define INITIAL_SIGN_COLUMNWIDTH	210
#define MINTRUSTDEPTH				2
#define MAXTRUSTDEPTH				8
#define MAXTRUSTDEPTHDIGITS			1
#define DEFAULTTRUSTDEPTH			2

#define SIG_NONEXPORTABLE	0
#define SIG_EXPORTABLE		1
#define SIG_META			2
#define SIG_TRUST			3

// external globals
extern HINSTANCE		g_hinst;
extern HWND				g_hwndMain;
extern HIMAGELIST		g_hilKeys;
extern PGPContextRef	g_context;
extern PGPtlsContextRef	g_tlscontext;

// typedefs
typedef struct {
	FARPROC			pfnCallback;
	PPGPKEYSSTRUCT	ppks;
	BOOL			bItemModified;
	PGPKeyDBObjRef	keySigning;
	PGPByte*		pPasskey;
	PGPSize			sizePasskey;
	PGPBoolean		bExportable;
	PGPUInt32		uTrustLevel;
	PGPUInt32		uExpireDays;
	CHAR			szRegExpression[256];
} CERTIFYSTRUCT, *PCERTIFYSTRUCT;

typedef struct {
	FARPROC			pfnCallback;
	PPGPKEYSSTRUCT	ppks;
	LPSTR			pszPrompt;
	LPSTR			pszUserID;
	PGPBoolean		bMultiple;
	BOOL			bItemModified;
	BOOL			bItemNotDeleted;
	BOOL			bDeleteAll;
	BOOL			bDeletedPrimaryUserID;
	PGPKeyDBObjRef	keyDefault;
	PGPKeyDBObjRef	keyPrev;
	HTLITEM			hPostDeleteFocusItem;
} DELETESTRUCT, *PDELETESTRUCT;

typedef struct {
	FARPROC			pfnCallback;
	PPGPKEYSSTRUCT	ppks;
	PGPBoolean		bSyncWithServer;
	INT				iSigType;
	HWND			hwndList;
	HWND			hwndDTPicker;
	INT				iItem;
	BOOL			bExpires;
	PGPInt32		iExpireDays;
	BOOL			bExpandedChoices;
	PGPUInt32		uTrustLevel;
	PGPUInt32		uTrustLoLimit;
	CHAR			szDomain[120];
	HFONT			hUTF8Font;
} CERTIFYCONFIRMSTRUCT, *PCERTIFYCONFIRMSTRUCT;

typedef struct {
	HWND			hwndParent;
	HWND			hwndProgress;
	INT				iNumSigsTotal;
	BOOL			bCancel;
} REVERIFYSTRUCT, *PREVERIFYSTRUCT;

static DWORD aDeleteAllIds[] = {			// Help IDs
    IDOK,				IDH_PGPPK_DELETEKEY,
    IDC_YESTOALL,		IDH_PGPPK_DELETEALLKEYS,
    IDNO,				IDH_PGPPK_DONTDELETEKEY,
    IDCANCEL,			IDH_PGPPK_CANCELDELETE,
    0,0
};

static DWORD aSignKeyIds[] = {			// Help IDs
	IDC_KEYLIST,		IDH_PGPPK_SIGNUSERIDLIST,
	IDC_MORECHOICES,	IDH_PGPPK_MORESIGCHOICES,
	IDC_EXPORTABLECHECK,IDH_PGPPK_ALLOWSIGEXPORT,
	IDC_FEWERCHOICES,	IDH_PGPPK_FEWERSIGCHOICES,
	IDC_NONEXPORTABLE,	IDH_PGPPK_SIGNONEXPORTABLE,
	IDC_EXPORTABLE,		IDH_PGPPK_SIGEXPORTABLE,
	IDC_TRUSTED,		IDH_PGPPK_SIGTRUSTED,
	IDC_DEPTH,			IDH_PGPPK_TRUSTDEPTH,
	IDC_DEPTHSPIN,		IDH_PGPPK_TRUSTDEPTH,
	IDC_META,			IDH_PGPPK_SIGMETA,
	IDC_DOMAIN,			IDH_PGPPK_DOMAINRESTRICTION,
	IDC_NEVEREXPIRES,	IDH_PGPPK_SIGNEVEREXPIRES,
	IDC_EXPIRES,		IDH_PGPPK_SIGEXPIRES,
	IDC_EXPIRATIONDATE,	IDH_PGPPK_SIGEXPIRATIONDATE,
    0,0
};


//	_______________________________________________
//
//  Certify a single object
//	routine called either from KMCertifyKeyOrUserID or as a
//	callback function from the TreeList control to
//	certify a single item.
//
//	lptli	= pointer to TreeList item to certify

static BOOL CALLBACK
sCertifySingleObject (
		TL_TREEITEM*	lptli,
		LPARAM			lParam)
{
	PCERTIFYSTRUCT	pcs			= (PCERTIFYSTRUCT)lParam;
	PGPError		err			= kPGPError_NoErr;

	PGPKeyDBObjRef	key;
	PGPKeyDBObjRef	userid;
	CHAR			sz512[512];
	CHAR			sz256[256];
	CHAR			sz64[64];
	PGPBoolean		bMultiple;

	switch (lptli->iImage) {
	case IDX_RSAPUBKEY :
	case IDX_RSAPUBDISKEY :
	case IDX_RSASECKEY :
	case IDX_RSASECDISKEY :
	case IDX_RSASECSHRKEY :
	case IDX_DSAPUBKEY :
	case IDX_DSAPUBDISKEY :
	case IDX_DSASECKEY :
	case IDX_DSASECDISKEY :
	case IDX_DSASECSHRKEY :
	case IDX_RSAV4PUBKEY :
	case IDX_RSAV4PUBDISKEY :
	case IDX_RSAV4SECKEY :
	case IDX_RSAV4SECDISKEY :
	case IDX_RSAV4SECSHRKEY :
	case IDX_RSATOKKEY :
	case IDX_RSATOKDISKEY :
	case IDX_RSAV4TOKKEY :
	case IDX_RSAV4TOKDISKEY :
	case IDX_ECPUBKEY :
	case IDX_ECPUBDISKEY :
	case IDX_ECSECKEY :
	case IDX_ECSECDISKEY :
	case IDX_ECSECSHRKEY :
		key = (PGPKeyDBObjRef)(lptli->lParam);
		PGPGetPrimaryUserID (key, &userid);
		break;

	case IDX_RSAUSERID :
	case IDX_DSAUSERID :
	case IDX_PHOTOUSERID :
		userid = (PGPKeyDBObjRef)(lptli->lParam);
		key = PGPPeekKeyDBObjKey (userid);
		break;

	case IDX_RSAPUBREVKEY :
	case IDX_RSAPUBEXPKEY :
	case IDX_RSASECREVKEY :
	case IDX_RSASECEXPKEY :
	case IDX_DSAPUBREVKEY :
	case IDX_DSAPUBEXPKEY :
	case IDX_DSASECREVKEY :
	case IDX_DSASECEXPKEY :
	case IDX_RSAV4PUBREVKEY :
	case IDX_RSAV4PUBEXPKEY :
	case IDX_RSAV4SECREVKEY :
	case IDX_RSAV4SECEXPKEY :
	case IDX_RSATOKREVKEY :
	case IDX_RSATOKEXPKEY :
	case IDX_RSAV4TOKREVKEY :
	case IDX_RSAV4TOKEXPKEY :
	case IDX_ECPUBREVKEY :
	case IDX_ECPUBEXPKEY :
	case IDX_ECSECREVKEY :
	case IDX_ECSECEXPKEY :
		return TRUE;

	default :
		return FALSE;
	}

	// make sure we have enough entropy
	PGPclRandom (g_context, pcs->ppks->hwndMain, 0);

	err = PGPCertifyUserID (userid, pcs->keySigning,
			PGPOExpiration (g_context, pcs->uExpireDays),
			PGPOExportable (g_context, pcs->bExportable),
			PGPOSigTrust (g_context, pcs->uTrustLevel,
					kPGPKeyTrust_Complete),
			(pcs->pPasskey) ?
				PGPOPasskeyBuffer (g_context, pcs->pPasskey, pcs->sizePasskey) :
				PGPONullOption (g_context),
			(pcs->szRegExpression[0]) ?
				PGPOSigRegularExpression (g_context, pcs->szRegExpression) :
				PGPONullOption (g_context),
			PGPOLastOption (g_context));

	if (IsntPGPError (err))
	{
		pcs->bItemModified = TRUE;
		return TRUE;
	}

	LoadString (g_hinst, IDS_CERTIFYERROR, sz64, 64);
	PGPclErrorToString (err, sz256, 256, NULL);
	wsprintf (sz512, sz64, lptli->pszText, sz256);

	LoadString (g_hinst, IDS_CAPTIONALERT, sz64, 64);

	PGPclKeyListGetSelectionInfo (
			pcs->ppks->hKL, NULL, NULL, &bMultiple, NULL);
	if (bMultiple)
	{
		if (PGPclMessageBoxUTF8 (pcs->ppks->hwndMain, sz512, sz64,
				MB_OKCANCEL|MB_ICONEXCLAMATION) == IDOK)
		{
			return TRUE;
		}
	}
	else
	{
		if (PGPclMessageBoxUTF8 (pcs->ppks->hwndMain, sz512, sz64,
				MB_OK|MB_ICONEXCLAMATION) == IDOK)
		{
			return TRUE;
		}
	}

	return FALSE;
}


//	_______________________________________________
//
//  Populate ListView with userids to sign

static BOOL CALLBACK
sInsertOneID (TL_TREEITEM* lptli, LPARAM lParam)
{
	PCERTIFYCONFIRMSTRUCT pccs = (PCERTIFYCONFIRMSTRUCT)lParam;

	PGPKeyDBObjRef	key;
	PGPKeyDBObjRef	userid;
	UINT			u, uAlgorithm, uKeyVersion;
	LV_ITEM			lvI;
	PGPByte			fingerprintBytes[256];
	CHAR			sz[kPGPMaxUserIDSize];

	lvI.iImage = -1;

	switch (lptli->iImage) {
	case IDX_RSAPUBKEY :
	case IDX_RSAPUBDISKEY :
	case IDX_RSASECKEY :
	case IDX_RSASECDISKEY :
	case IDX_RSASECSHRKEY :
	case IDX_DSAPUBKEY :
	case IDX_DSAPUBDISKEY :
	case IDX_DSASECKEY :
	case IDX_DSASECDISKEY :
	case IDX_DSASECSHRKEY :
	case IDX_RSAV4PUBKEY :
	case IDX_RSAV4PUBDISKEY :
	case IDX_RSAV4SECKEY :
	case IDX_RSAV4SECDISKEY :
	case IDX_RSAV4SECSHRKEY :
	case IDX_RSATOKKEY :
	case IDX_RSATOKDISKEY :
	case IDX_RSAV4TOKKEY :
	case IDX_RSAV4TOKDISKEY :
	case IDX_ECPUBKEY :
	case IDX_ECPUBDISKEY :
	case IDX_ECSECKEY :
	case IDX_ECSECDISKEY :
	case IDX_ECSECSHRKEY :
		key = (PGPKeyDBObjRef)(lptli->lParam);
		PGPclGetPrimaryUserIDNameUTF8 (key, sz, sizeof(sz), &u);
		break;

	case IDX_RSAUSERID :
	case IDX_DSAUSERID :
		userid = (PGPKeyDBObjRef)(lptli->lParam);
		key = PGPPeekKeyDBObjKey (userid);
		PGPclGetUserIDNameUTF8 (userid, sz, sizeof(sz), &u);
		break;

	case IDX_PHOTOUSERID :
		userid = (PGPKeyDBObjRef)(lptli->lParam);
		key = PGPPeekKeyDBObjKey (userid);
		LoadString (g_hinst, IDS_PHOTOUSERID, sz, sizeof(sz));
		lvI.iImage = IDX_PHOTOUSERID;
		break;

	case IDX_RSAPUBREVKEY :
	case IDX_RSAPUBEXPKEY :
	case IDX_RSASECREVKEY :
	case IDX_RSASECEXPKEY :
	case IDX_DSAPUBREVKEY :
	case IDX_DSAPUBEXPKEY :
	case IDX_DSASECREVKEY :
	case IDX_DSASECEXPKEY :
	case IDX_RSAV4PUBREVKEY :
	case IDX_RSAV4PUBEXPKEY :
	case IDX_RSAV4SECREVKEY :
	case IDX_RSAV4SECEXPKEY :
	case IDX_RSATOKREVKEY :
	case IDX_RSATOKEXPKEY :
	case IDX_RSAV4TOKREVKEY :
	case IDX_RSAV4TOKEXPKEY :
	case IDX_ECPUBREVKEY :
	case IDX_ECPUBEXPKEY :
	case IDX_ECSECREVKEY :
	case IDX_ECSECEXPKEY :
		return TRUE;

	default :
		return FALSE;
	}

	if (lvI.iImage == -1)
	{
		PGPGetKeyDBObjNumericProperty (key,
				kPGPKeyProperty_AlgorithmID, &uAlgorithm);

		switch (uAlgorithm) {
		case kPGPPublicKeyAlgorithm_RSA :	lvI.iImage = IDX_RSAUSERID;	break;
		case kPGPPublicKeyAlgorithm_DSA :	lvI.iImage = IDX_DSAUSERID;	break;
		default :							lvI.iImage = IDX_DSAUSERID; break;
		}
	}

	lvI.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE;
	lvI.state = 0;
	lvI.stateMask = 0;

	lvI.iItem = pccs->iItem;
	lvI.iSubItem = 0;
	lvI.pszText	= sz;
	lvI.cchTextMax = 0;

	if (ListView_InsertItem (pccs->hwndList, &lvI) == -1)
		return FALSE;

	PGPGetKeyDBObjDataProperty (key, kPGPKeyProperty_Fingerprint,
			fingerprintBytes, sizeof(fingerprintBytes), &u);
	PGPGetKeyDBObjNumericProperty (key,
			kPGPKeyProperty_Version, &uKeyVersion);
	PGPclConvertFingerprintToString (uKeyVersion, fingerprintBytes);
	ListView_SetItemText (pccs->hwndList, pccs->iItem, 1, fingerprintBytes);

	(pccs->iItem)++;

	return TRUE;
}


//	_______________________________________________
//
//  Populate ListView with userids to sign

static VOID
sFillKeyList (PCERTIFYCONFIRMSTRUCT pccs)
{
	LV_COLUMN	lvC;
	CHAR		sz[256];

	ListView_SetImageList (pccs->hwndList, g_hilKeys, LVSIL_SMALL);

	lvC.mask = LVCF_FMT | LVCF_TEXT | LVCF_WIDTH | LVCF_SUBITEM;
	lvC.fmt = LVCFMT_LEFT;
	lvC.pszText = sz;

	LoadString (g_hinst, IDS_USERID, sz, sizeof(sz));
	lvC.cx = INITIAL_SIGN_COLUMNWIDTH;
	lvC.iSubItem = 0;
	if (ListView_InsertColumn (pccs->hwndList, 0, &lvC) == -1) return;

	LoadString (g_hinst, IDS_FINGERPRINT, sz, sizeof(sz));
	lvC.cx = 360;
	lvC.iSubItem = 1;
	if (ListView_InsertColumn (pccs->hwndList, 1, &lvC) == -1) return;

	// populate control by iterating through selected items
	pccs->pfnCallback = sInsertOneID;
	pccs->iItem = 0;
	TreeList_IterateSelected (pccs->ppks->hwndTreeList, pccs);
}


//	_______________________________________________
//
//	Convert domain string to regular expression

static VOID
sDomainToRegExpression (
	LPSTR		szDomain,
	LPSTR		szRegExp)
{
	LPSTR 		pszSrc		= szDomain;
	LPSTR		pszDst		= szRegExp;

	lstrcpy (pszDst, "<[^>]+[@.]");
	pszDst += lstrlen (pszDst);

	for ( ; *pszSrc; pszSrc++)
	{
		switch (*pszSrc)
		{
			case '*':
			case '+':
			case '?':
			case '.':
			case '^':
			case '$':
			case '\\':
			case '[':
			case ']':
			case '-':
				*pszDst++ = '\\';
				*pszDst++ = *pszSrc;
				break;
			default:
				*pszDst++ = *pszSrc;
				break;
		}
	}
	*pszDst++ = '>';
	*pszDst++ = '$';
	*pszDst++ = '\0';
}


//	_______________________________________________
//
//	Sign key dialog message procedure

static BOOL CALLBACK
sSignKeyDlgProc (
		HWND	hDlg,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PCERTIFYCONFIRMSTRUCT	pccs;
	INT						iNewY, iNewWindowHeight, iOffset;
	HWND					hwndControl;
	RECT					rectControl;
	RECT					rc;

	switch (uMsg) {

	case WM_INITDIALOG :
	{
		SYSTEMTIME	st;

		SetWindowLong (hDlg, GWL_USERDATA, lParam);
		pccs = (PCERTIFYCONFIRMSTRUCT)lParam;
		pccs->hwndList = GetDlgItem (hDlg, IDC_KEYLIST);

		EnableWindow (GetDlgItem (hDlg, IDC_MORECHOICES), TRUE);
		ShowWindow (GetDlgItem (hDlg, IDC_MORECHOICES), SW_SHOW);
		pccs->bExpandedChoices = FALSE;

		sFillKeyList (pccs);

		// initialize trust depth
		pccs->uTrustLoLimit = 0;
		SendDlgItemMessage (hDlg, IDC_DEPTHSPIN, UDM_SETRANGE,
				0, (LPARAM)MAKELONG (MAXTRUSTDEPTH, pccs->uTrustLoLimit));
		SendDlgItemMessage (hDlg, IDC_DEPTH, EM_SETLIMITTEXT,
				MAXTRUSTDEPTHDIGITS, 0);
		SetDlgItemInt (hDlg, IDC_DEPTH, 0, FALSE);

		// create and initialize date/time picker control
		GetWindowRect (GetDlgItem (hDlg, IDC_EXPIRATIONDATE), &rc);
		MapWindowPoints (NULL, hDlg, (LPPOINT)&rc, 2);
		pccs->hwndDTPicker = CreateWindowEx (0, DATETIMEPICK_CLASS,
                             "DateTime",
                             WS_BORDER|WS_CHILD|WS_TABSTOP,
                             rc.left, rc.top,
							 rc.right-rc.left, rc.bottom-rc.top,
							 hDlg, (HMENU)IDC_EXPIRATIONDATE,
							 g_hinst, NULL);
		SetWindowPos (pccs->hwndDTPicker,
						GetDlgItem (hDlg, IDC_EXPIRATIONDATE),
						0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
		SendMessage (pccs->hwndDTPicker, DTM_SETMCCOLOR,
						MCSC_MONTHBK, (LPARAM)GetSysColor (COLOR_3DFACE));

		// initialize to one year from today
		GetLocalTime (&st);
		st.wYear++;
		SendMessage (pccs->hwndDTPicker, DTM_SETSYSTEMTIME,
							GDT_VALID, (LPARAM)&st);

		EnableWindow (pccs->hwndDTPicker, FALSE);
		CheckDlgButton (hDlg, IDC_NEVEREXPIRES, BST_CHECKED);

		// user "more" button as desired Y location
		hwndControl = GetDlgItem (hDlg, IDC_MORECHOICES);
		GetWindowRect (hwndControl, &rectControl);
		MapWindowPoints (NULL, hDlg, (LPPOINT)&rectControl, 2);

		GetWindowRect (hDlg, &rc);
		iOffset = rc.bottom-rc.top;
		GetClientRect (hDlg, &rc);
		iOffset -= rc.bottom;
		iOffset += (rectControl.bottom - rectControl.top) / 2;

		iNewY = rectControl.top;
		iNewWindowHeight = rectControl.bottom + iOffset;

		// move OK Button
		hwndControl = GetDlgItem (hDlg, IDOK);
		GetWindowRect (hwndControl, &rectControl);
		MapWindowPoints (NULL, hDlg, (LPPOINT)&rectControl, 2);
		MoveWindow(	hwndControl,
					rectControl.left,
					iNewY,
					rectControl.right - rectControl.left,
					rectControl.bottom - rectControl.top,
					TRUE);

		// move Cancel Button
		hwndControl = GetDlgItem (hDlg, IDCANCEL);
		GetWindowRect (hwndControl, &rectControl);
		MapWindowPoints (NULL, hDlg, (LPPOINT)&rectControl, 2);
		MoveWindow(	hwndControl,
					rectControl.left,
					iNewY,
					rectControl.right - rectControl.left,
					rectControl.bottom - rectControl.top,
					TRUE);

		// move Help Button
		hwndControl = GetDlgItem (hDlg, IDHELP);
		GetWindowRect (hwndControl, &rectControl);
		MapWindowPoints (NULL, hDlg, (LPPOINT)&rectControl, 2);
		MoveWindow(	hwndControl,
					rectControl.left,
					iNewY,
					rectControl.right - rectControl.left,
					rectControl.bottom - rectControl.top,
					TRUE);

		// size Window
		GetWindowRect (hDlg, &rectControl);
		MoveWindow(	hDlg,
					rectControl.left,
					rectControl.top,
					rectControl.right - rectControl.left,
					iNewWindowHeight,
					TRUE);

		// ListView uses bad unicode font here, so we must set
		// it ourselves.
		pccs->hUTF8Font=SetUTF8Font(GetDlgItem(hDlg,IDC_KEYLIST));
		break;
	}

	case WM_DRAWITEM:
    {
        KeyDrawItem((LPDRAWITEMSTRUCT) lParam);
        return TRUE;
    }

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hDlg, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aSignKeyIds);

	case WM_DESTROY :
		pccs = (PCERTIFYCONFIRMSTRUCT)GetWindowLong (hDlg, GWL_USERDATA);
		ListView_SetImageList (pccs->hwndList, NULL, LVSIL_SMALL);
		DeleteObject(pccs->hUTF8Font);
		break;

	case WM_NOTIFY:
	{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		if (pnmh->code == DTN_DATETIMECHANGE) 
		{
			SYSTEMTIME st;

			pccs = (PCERTIFYCONFIRMSTRUCT) GetWindowLong(hDlg, GWL_USERDATA);
			SendMessage (pccs->hwndDTPicker, 
					DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
			PGPclSystemTimeToDays (&st, &(pccs->iExpireDays));

			if (pccs->iExpireDays > 0)
				EnableWindow (GetDlgItem (hDlg, IDOK), TRUE);
			else
				EnableWindow (GetDlgItem (hDlg, IDOK), FALSE);
		}
		break;
	}

	case WM_PAINT :
		pccs = (PCERTIFYCONFIRMSTRUCT)GetWindowLong (hDlg, GWL_USERDATA);
		if (pccs->bExpandedChoices)
		{
			// paint icons
			HDC	hdc = GetDC (hDlg);

			GetWindowRect (GetDlgItem (hDlg, IDC_NONEXPORTABLE), &rc);
			MapWindowPoints (NULL, hDlg, (LPPOINT)&rc, 2);
			ImageList_Draw (g_hilKeys, IDX_CERT, hdc,
							rc.left-22, rc.top, ILD_TRANSPARENT);

			GetWindowRect (GetDlgItem (hDlg, IDC_EXPORTABLE), &rc);
			MapWindowPoints (NULL, hDlg, (LPPOINT)&rc, 2);
			ImageList_Draw (g_hilKeys, IDX_EXPORTCERT, hdc,
							rc.left-22, rc.top, ILD_TRANSPARENT);

			GetWindowRect (GetDlgItem (hDlg, IDC_TRUSTED), &rc);
			MapWindowPoints (NULL, hDlg, (LPPOINT)&rc, 2);
			ImageList_Draw (g_hilKeys, IDX_TRUSTEDCERT, hdc,
							rc.left-22, rc.top, ILD_TRANSPARENT);

			GetWindowRect (GetDlgItem (hDlg, IDC_META), &rc);
			MapWindowPoints (NULL, hDlg, (LPPOINT)&rc, 2);
			ImageList_Draw (g_hilKeys, IDX_METACERT, hdc,
							rc.left-22, rc.top, ILD_TRANSPARENT);

			ReleaseDC (hDlg, hdc);
		}
		break;

	case WM_COMMAND:

		pccs = (PCERTIFYCONFIRMSTRUCT)GetWindowLong (hDlg, GWL_USERDATA);
		if (IsNull (pccs))
			return TRUE;

		switch (LOWORD(wParam)) {
		case IDCANCEL:
			pccs->bSyncWithServer = FALSE;
			EndDialog (hDlg, 1);
			break;

		case IDOK:
			if (!(pccs->bExpandedChoices)) 
			{
				pccs->bExpires = FALSE;
				if (IsDlgButtonChecked (hDlg, IDC_EXPORTABLECHECK)
														== BST_CHECKED)
				{
					pccs->iSigType = SIG_EXPORTABLE;
				}
				else 
					pccs->iSigType = SIG_NONEXPORTABLE;
			}
			else {
				if (IsDlgButtonChecked (hDlg, IDC_NEVEREXPIRES) == BST_CHECKED)
					pccs->bExpires = FALSE;
				else 
				{
					SYSTEMTIME st;
					pccs->bExpires = TRUE;
					SendMessage (pccs->hwndDTPicker, DTM_GETSYSTEMTIME, 0,
									(LPARAM)&st);
					PGPclSystemTimeToDays (&st, &(pccs->iExpireDays));
				}
			}

			pccs->uTrustLevel = GetDlgItemInt (hDlg, IDC_DEPTH, NULL, FALSE);
			GetDlgItemText (hDlg, 
					IDC_DOMAIN, pccs->szDomain, sizeof(pccs->szDomain));
			EndDialog (hDlg, 0);
			break;

		case IDC_EXPORTABLECHECK :
			if (IsDlgButtonChecked (hDlg, IDC_EXPORTABLECHECK) == BST_CHECKED)
				pccs->iSigType = SIG_EXPORTABLE;
			else 
				pccs->iSigType = SIG_NONEXPORTABLE;
			break;

		case IDC_NONEXPORTABLE :
			pccs->iSigType = SIG_NONEXPORTABLE;
			pccs->uTrustLoLimit = 0;
			EnableWindow (GetDlgItem (hDlg, IDC_DOMAIN), FALSE);
			SendDlgItemMessage (hDlg, IDC_DEPTHSPIN, UDM_SETRANGE,
					0, (LPARAM)MAKELONG (MAXTRUSTDEPTH, pccs->uTrustLoLimit));
			SetDlgItemInt (hDlg, IDC_DEPTH, 0, FALSE);
			EnableWindow (GetDlgItem (hDlg, IDC_DEPTH), FALSE);
			EnableWindow (GetDlgItem (hDlg, IDC_DEPTHSPIN), FALSE);
			break;

		case IDC_EXPORTABLE :
			pccs->iSigType = SIG_EXPORTABLE;
			pccs->uTrustLoLimit = 0;
			EnableWindow (GetDlgItem (hDlg, IDC_DOMAIN), FALSE);
			SendDlgItemMessage (hDlg, IDC_DEPTHSPIN, UDM_SETRANGE,
					0, (LPARAM)MAKELONG (MAXTRUSTDEPTH, pccs->uTrustLoLimit));
			SetDlgItemInt (hDlg, IDC_DEPTH, 0, FALSE);
			EnableWindow (GetDlgItem (hDlg, IDC_DEPTH), FALSE);
			EnableWindow (GetDlgItem (hDlg, IDC_DEPTHSPIN), FALSE);
			break;

		case IDC_META :
			pccs->iSigType = SIG_META;
			pccs->uTrustLoLimit = 2;
			EnableWindow (GetDlgItem (hDlg, IDC_DOMAIN), FALSE);
			SendDlgItemMessage (hDlg, IDC_DEPTHSPIN, UDM_SETRANGE,
					0, (LPARAM)MAKELONG (MAXTRUSTDEPTH, pccs->uTrustLoLimit));
			SetDlgItemInt (hDlg, IDC_DEPTH, 2, FALSE);
			EnableWindow (GetDlgItem (hDlg, IDC_DEPTH), TRUE);
			EnableWindow (GetDlgItem (hDlg, IDC_DEPTHSPIN), TRUE);
			break;

		case IDC_TRUSTED :
			pccs->iSigType = SIG_TRUST;
			pccs->uTrustLoLimit = 1;
			EnableWindow (GetDlgItem (hDlg, IDC_DOMAIN), TRUE);
			SendDlgItemMessage (hDlg, IDC_DEPTHSPIN, UDM_SETRANGE,
					0, (LPARAM)MAKELONG (MAXTRUSTDEPTH, pccs->uTrustLoLimit));
			SetDlgItemInt (hDlg, IDC_DEPTH, 1, FALSE);
			EnableWindow (GetDlgItem (hDlg, IDC_DEPTH), TRUE);
			EnableWindow (GetDlgItem (hDlg, IDC_DEPTHSPIN), TRUE);
			break;

		case IDC_DEPTH :
			if (HIWORD (wParam) == EN_CHANGE)
			{
				UINT	uIn, uOut;

				uIn = GetDlgItemInt (hDlg, IDC_DEPTH, NULL, FALSE);
				uOut = uIn;

				if (uIn < pccs->uTrustLoLimit)
					uOut = pccs->uTrustLoLimit;
				else if (uIn > MAXTRUSTDEPTH)
					uOut = MAXTRUSTDEPTH;

				if (uIn != uOut)
					SetDlgItemInt (hDlg, IDC_DEPTH, uOut, FALSE);
			}
			break;

		case IDC_NEVEREXPIRES :
		case IDC_EXPIRES :
			if (IsDlgButtonChecked (hDlg, IDC_EXPIRES) == BST_CHECKED) 
			{
				SYSTEMTIME st;
				EnableWindow (pccs->hwndDTPicker, TRUE);

				SendMessage (pccs->hwndDTPicker, DTM_GETSYSTEMTIME,
										0, (LPARAM)&st);
				PGPclSystemTimeToDays (&st, &(pccs->iExpireDays));
				if (pccs->iExpireDays > 0)
					EnableWindow (GetDlgItem (hDlg, IDOK), TRUE);
				else
					EnableWindow (GetDlgItem (hDlg, IDOK), FALSE);
			}
			else 
			{
				EnableWindow (pccs->hwndDTPicker, FALSE);
				EnableWindow (GetDlgItem (hDlg, IDOK), TRUE);
			}
			break;

		case IDC_MORECHOICES :
			pccs->bExpandedChoices = TRUE;
			ShowWindow (GetDlgItem (hDlg, IDC_MORECHOICES), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_EXPORTABLECHECK), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_FEWERCHOICES), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_SIGTYPE), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_NONEXPORTABLE), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_EXPORTABLE), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_TRUSTED), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_META), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_EXPIRATION), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_NEVEREXPIRES), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_EXPIRES), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_DOMAINTEXT), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_DOMAIN), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_DEPTH), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_DEPTHSPIN), SW_SHOW);
			ShowWindow (pccs->hwndDTPicker, SW_SHOW);

			// user "fewer" button as desired Y location
			hwndControl = GetDlgItem (hDlg, IDC_FEWERCHOICES);
			GetWindowRect (hwndControl, &rectControl);
			MapWindowPoints (NULL, hDlg, (LPPOINT)&rectControl, 2);

			GetWindowRect (hDlg, &rc);
			iOffset = rc.bottom-rc.top;
			GetClientRect (hDlg, &rc);
			iOffset -= rc.bottom;
			iOffset += (rectControl.bottom - rectControl.top) / 2;

			iNewY = rectControl.top;
			iNewWindowHeight = rectControl.bottom + iOffset;

			// move OK Button
			hwndControl = GetDlgItem (hDlg, IDOK);
			GetWindowRect (hwndControl, &rectControl);
			MapWindowPoints (NULL, hDlg, (LPPOINT)&rectControl, 2);
			MoveWindow(	hwndControl,
						rectControl.left,
						iNewY,
						rectControl.right - rectControl.left,
						rectControl.bottom - rectControl.top,
						TRUE);

			// move Cancel Button
			hwndControl = GetDlgItem (hDlg, IDCANCEL);
			GetWindowRect (hwndControl, &rectControl);
			MapWindowPoints (NULL, hDlg, (LPPOINT)&rectControl, 2);
			MoveWindow(	hwndControl,
						rectControl.left,
						iNewY,
						rectControl.right - rectControl.left,
						rectControl.bottom - rectControl.top,
						TRUE);

			// move Help Button
			hwndControl = GetDlgItem (hDlg, IDHELP);
			GetWindowRect (hwndControl, &rectControl);
			MapWindowPoints (NULL, hDlg, (LPPOINT)&rectControl, 2);
			MoveWindow(	hwndControl,
						rectControl.left,
						iNewY,
						rectControl.right - rectControl.left,
						rectControl.bottom - rectControl.top,
						TRUE);

			// set radio buttons
			CheckRadioButton (hDlg, IDC_NONEXPORTABLE, IDC_TRUSTED,
								IDC_NONEXPORTABLE + pccs->iSigType);

			// size Window
			GetWindowRect (hDlg, &rectControl);
			MoveWindow(	hDlg,
						rectControl.left,
						rectControl.top,
						rectControl.right - rectControl.left,
						iNewWindowHeight,
						TRUE);

			break;

		case IDC_FEWERCHOICES :
			pccs->bExpandedChoices = FALSE;
			ShowWindow (GetDlgItem (hDlg, IDC_MORECHOICES), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_EXPORTABLECHECK), SW_SHOW);
			ShowWindow (GetDlgItem (hDlg, IDC_FEWERCHOICES), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_SIGTYPE), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_NONEXPORTABLE), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_EXPORTABLE), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_TRUSTED), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_META), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_EXPIRATION), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_NEVEREXPIRES), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_EXPIRES), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_DOMAINTEXT), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_DOMAIN), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_DEPTH), SW_HIDE);
			ShowWindow (GetDlgItem (hDlg, IDC_DEPTHSPIN), SW_HIDE);
			ShowWindow (pccs->hwndDTPicker, SW_HIDE);

			// user "more" button as desired Y location
			hwndControl = GetDlgItem (hDlg, IDC_MORECHOICES);
			GetWindowRect (hwndControl, &rectControl);
			MapWindowPoints (NULL, hDlg, (LPPOINT)&rectControl, 2);

			GetWindowRect (hDlg, &rc);
			iOffset = rc.bottom-rc.top;
			GetClientRect (hDlg, &rc);
			iOffset -= rc.bottom;
			iOffset += (rectControl.bottom - rectControl.top) / 2;

			iNewY = rectControl.top;
			iNewWindowHeight = rectControl.bottom + iOffset;

			// move OK Button
			hwndControl = GetDlgItem (hDlg, IDOK);
			GetWindowRect (hwndControl, &rectControl);
			MapWindowPoints (NULL, hDlg, (LPPOINT)&rectControl, 2);
			MoveWindow(	hwndControl,
						rectControl.left,
						iNewY,
						rectControl.right - rectControl.left,
						rectControl.bottom - rectControl.top,
						TRUE);

			// move Cancel Button
			hwndControl = GetDlgItem (hDlg, IDCANCEL);
			GetWindowRect (hwndControl, &rectControl);
			MapWindowPoints (NULL, hDlg, (LPPOINT)&rectControl, 2);
			MoveWindow(	hwndControl,
						rectControl.left,
						iNewY,
						rectControl.right - rectControl.left,
						rectControl.bottom - rectControl.top,
						TRUE);

			// move Help Button
			hwndControl = GetDlgItem (hDlg, IDHELP);
			GetWindowRect (hwndControl, &rectControl);
			MapWindowPoints (NULL, hDlg, (LPPOINT)&rectControl, 2);
			MoveWindow(	hwndControl,
						rectControl.left,
						iNewY,
						rectControl.right - rectControl.left,
						rectControl.bottom - rectControl.top,
						TRUE);

			// set radio buttons
			if (pccs->iSigType == SIG_EXPORTABLE)
				CheckDlgButton (hDlg, IDC_EXPORTABLECHECK, BST_CHECKED);
			else
			{
				pccs->iSigType = SIG_NONEXPORTABLE;
				CheckDlgButton (hDlg, IDC_EXPORTABLECHECK, BST_UNCHECKED);
			}

			// disable controls
			SendDlgItemMessage (hDlg, IDC_DEPTHSPIN, UDM_SETRANGE,
					0, (LPARAM)MAKELONG (MAXTRUSTDEPTH, 0));
			SetDlgItemInt (hDlg, IDC_DEPTH, 0, FALSE);
			EnableWindow (GetDlgItem (hDlg, IDC_DEPTH), FALSE);
			EnableWindow (GetDlgItem (hDlg, IDC_DEPTHSPIN), FALSE);
			EnableWindow (GetDlgItem (hDlg, IDC_DOMAIN), FALSE);

			// size Window
			GetWindowRect (hDlg, &rectControl);
			MoveWindow(	hDlg,
						rectControl.left,
						rectControl.top,
						rectControl.right - rectControl.left,
						iNewWindowHeight,
						TRUE);
			break;

		case IDHELP :
			PGPclHtmlHelp (hDlg, kPGPclHelpContext, 0, 0, 
					(char*)kPGPclHelpFile, (DWORD*)IDH_PGPPK_SIGNDIALOG);
			break;

		}
		return FALSE;
	}
	return FALSE;
}


//	_______________________________________________
//
//  Certify selected key or userid

BOOL
PKCertifyKeyOrUserID (
		PGPKEYSSTRUCT*	ppks)
{
	PGPPrefRef				prefref			= kInvalidPGPPrefRef;
	CHAR					sz256[256];
	PGPError				err;
	CERTIFYSTRUCT			cs;
	CERTIFYCONFIRMSTRUCT	ccs;
	PGPBoolean				bAllow			= TRUE;

	if(PGPlnEnterprise())
	{
		if (IsPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
			return FALSE;

		PGPGetPrefBoolean (prefref, kPGPPrefAllowKeySigning, &bAllow);
		if (!bAllow)
		{
			PKMessageBox (ppks->hwndMain, IDS_CAPTIONINFO,
					IDS_KEYSIGNINGNOTALLOWED, MB_OK|MB_ICONINFORMATION);
			return FALSE;
		}
	}

	// initialize structures
	cs.pfnCallback = sCertifySingleObject;
	cs.ppks = ppks;
	cs.bItemModified = FALSE;
	cs.keySigning = NULL;
	cs.pPasskey = NULL;

	ccs.ppks = ppks;
	ccs.iSigType = SIG_NONEXPORTABLE;
	ccs.bExpires = FALSE;
	ccs.iExpireDays = 0;

	if (IsPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		return FALSE;

	PGPGetPrefBoolean (prefref, 
			kPGPPrefKeyServerSyncOnKeySign, &(ccs.bSyncWithServer));

	if (DialogBoxParam (g_hinst, MAKEINTRESOURCE(IDD_SIGNCONFIRM),
		ppks->hwndMain, sSignKeyDlgProc, (LPARAM)&ccs))
	{
		return FALSE;
	}

	// convert user-entered signature type to flags to pass
	// and setup domain regular expression
	switch (ccs.iSigType) {
	case SIG_NONEXPORTABLE :
		cs.bExportable = FALSE;
		cs.uTrustLevel = 0;
		cs.szRegExpression[0] = '\0';
		break;

	case SIG_EXPORTABLE :
		cs.bExportable = TRUE;
		cs.uTrustLevel = 0;
		cs.szRegExpression[0] = '\0';
		break;

	case SIG_TRUST :
		cs.bExportable = TRUE;
		cs.uTrustLevel = ccs.uTrustLevel;
		if (ccs.szDomain[0])
			sDomainToRegExpression (ccs.szDomain, cs.szRegExpression);
		else
			cs.szRegExpression[0] = '\0';
		break;

	case SIG_META :
		cs.bExportable = FALSE;
		cs.uTrustLevel = ccs.uTrustLevel;
		cs.szRegExpression[0] = '\0';
		break;
	}

	// convert expiration info to expire days
	if ((ccs.bExpires) && (ccs.iExpireDays > 0))
		cs.uExpireDays = (UINT)ccs.iExpireDays;
	else
		cs.uExpireDays = 0;

	// get valid passphrase
	LoadString (g_hinst, IDS_SIGNKEYPASSPHRASE, sz256, 256);
	err = PGPclGetSigningKeyPhrase (g_context, g_tlscontext,
			ppks->hwndMain, sz256,
			ppks->keydbMain, FALSE, &cs.keySigning,
			NULL, &cs.pPasskey, &cs.sizePasskey);

	if (IsntPGPError (err))
	{
		// update from server
		if (ccs.bSyncWithServer)
		{
			if (!PKGetFromServerInternal (ppks, FALSE, FALSE, FALSE))
			{
				if (PKMessageBox (ppks->hwndMain, IDS_CAPTIONALERT,
						IDS_QUERYCONTINUESIGNING,
						MB_YESNO|MB_ICONQUESTION) == IDNO)
				{
					if (cs.pPasskey)
					{
						PGPclFreePasskey (cs.pPasskey, cs.sizePasskey);
						cs.pPasskey = NULL;
					}
					return FALSE;
				}
			}
		}

		// call callback for all selected items
		TreeList_IterateSelected (ppks->hwndTreeList, &cs);

		// changes have been made; save them and update all validities
		if (cs.bItemModified)
		{
			PKKeyDBModified (ppks, PK_MOD_INDEX_0);
			PGPclKeyListReloadKeys (ppks->hKL, FALSE);
			PGPclKeyListUpdateTree (ppks->hKL, 
					kPGPclKeyListUpdateValidities, NULL, TRUE); 

			// send key to server, if selected
			if (ccs.bSyncWithServer && cs.bExportable)
				PKSendToServer (ppks, kPGPclDefaultServer);
		}
	}

	if (cs.pPasskey)
	{
		PGPclFreePasskey (cs.pPasskey, cs.sizePasskey);
		cs.pPasskey = NULL;
	}

	PKUpdateAllObjectPropertiesWindows (ppks);

	return (cs.bItemModified);
}


//	_______________________________________________
//
//  Enable selected key

BOOL
PKEnableKey (
		PGPKEYSSTRUCT*	ppks)
{
	PGPKeyDBObjRef	key;

	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &key, NULL, NULL);
	if (IsntPGPError (PGPclErrorBox (
				ppks->hwndMain, PGPSetKeyEnabled (key, TRUE))))
	{
		PKKeyDBModified (ppks, PK_MOD_INDEX_0);
		PGPclKeyListUpdateTree (ppks->hKL,
				kPGPclKeyListUpdateObject, key, TRUE);
		PGPclKeyListSetTree (ppks->hKL,
				kPGPclSelectSpecifiedOnly, key);
		PKUpdateAllObjectPropertiesWindows (ppks);
		return TRUE;
	}
	return FALSE;
}


//	_______________________________________________
//
//  Disable selected key

BOOL
PKDisableKey (
		PGPKEYSSTRUCT*	ppks)
{
	PGPKeyDBObjRef	key;

	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &key, NULL, NULL);
	if (IsntPGPError (PGPclErrorBox (
				ppks->hwndMain, PGPSetKeyEnabled (key, FALSE))))
	{
		PKKeyDBModified (ppks, PK_MOD_INDEX_0);
		PGPclKeyListUpdateTree (ppks->hKL,
				kPGPclKeyListUpdateObject, key, TRUE);
		PGPclKeyListSetTree (ppks->hKL,
				kPGPclSelectSpecifiedOnly, key);
		PKUpdateAllObjectPropertiesWindows (ppks);
		return TRUE;
	}
	return FALSE;
}


//	_______________________________________________
//
//	Delete All dialog message procedure

static BOOL CALLBACK
sDeleteAllDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PDELETESTRUCT pds;

	switch (uMsg) {

	case WM_INITDIALOG :
		SetWindowLong (hwnd, GWL_USERDATA, lParam);
		pds = (PDELETESTRUCT)lParam;

		UTF8EditInit (GetDlgItem (hwnd, IDC_NAME));
		UTF8EditSetText (GetDlgItem (hwnd, IDC_NAME), pds->pszUserID);
		SetDlgItemText (hwnd, IDC_PROMPT, pds->pszPrompt);

		if (!pds->bMultiple)
		{
			RECT rc;

			GetWindowRect (GetDlgItem (hwnd, IDC_YESTOALL), &rc);
			MapWindowPoints (NULL, hwnd, (LPPOINT)&rc, 2);
			SetWindowPos (GetDlgItem (hwnd, IDOK), NULL, 
					rc.left, rc.top, 0, 0, SWP_NOSIZE|SWP_NOZORDER); 

			ShowWindow (GetDlgItem (hwnd, IDC_YESTOALL), SW_HIDE);
			EnableWindow (GetDlgItem (hwnd, IDC_YESTOALL), FALSE);

			ShowWindow (GetDlgItem (hwnd, IDCANCEL), SW_HIDE);
			EnableWindow (GetDlgItem (hwnd, IDCANCEL), FALSE);
		}

		SetFocus (GetDlgItem (hwnd, IDNO));
		return FALSE;

	case WM_DESTROY :
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_NAME));
		return FALSE;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aDeleteAllIds);

	case WM_COMMAND:

		switch (LOWORD(wParam)) {
		case IDCANCEL:
			pds = (PDELETESTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			pds->bItemNotDeleted = TRUE;
			EndDialog (hwnd, IDCANCEL);
			break;

		case IDOK:
		case IDYES:
			EndDialog (hwnd, IDYES);
			break;

		case IDNO:
			EndDialog (hwnd, IDNO);
			break;

		case IDC_YESTOALL :
			pds = (PDELETESTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			pds->bDeleteAll = TRUE;
			EndDialog (hwnd, IDYES);
			break;
		}
		return TRUE;
	}
	return FALSE;
}


//	_______________________________________________
//
//  Ask user for delete confirmation

static INT
sDeleteConfirm (
		TL_TREEITEM*	lptli,
		INT				iPromptID,
		PDELETESTRUCT	pds)
{
	CHAR		szPrompt[256];
	INT			iRetVal;

	if (pds->bDeleteAll)
		return IDYES;

	LoadString (g_hinst, iPromptID, szPrompt, sizeof(szPrompt));
	pds->pszPrompt = szPrompt;
	pds->pszUserID = lptli->pszText;

	PGPclKeyListGetSelectionInfo (
			pds->ppks->hKL, NULL, NULL, &(pds->bMultiple), NULL);

	iRetVal = DialogBoxParam (g_hinst, MAKEINTRESOURCE (IDD_DELETEALL),
			pds->ppks->hwndMain, sDeleteAllDlgProc, (LPARAM)pds);

	if (pds->bMultiple)
	{
		if (!pds->bItemNotDeleted && (iRetVal == IDNO))
		{
			pds->bItemNotDeleted = TRUE;
			pds->hPostDeleteFocusItem = lptli->hItem;
		}
	}

	return iRetVal;
}


//	_______________________________________________
//
//  inform user about deleting keys on tokens

static INT
sDeleteTokenKeyMessage (
		TL_TREEITEM*	lptli,
		PDELETESTRUCT	pds)
{
	CHAR		sz1[512];
	CHAR		sz2[768];
	INT			iRetVal;
	PGPBoolean	bMultiple;
	PGPInt32	iFlags;

	LoadString (g_hinst, IDS_DELETETOKENKEY, sz1, sizeof(sz1));
	wsprintf (sz2, sz1, lptli->pszText);

	LoadString (g_hinst, IDS_CAPTIONINFO, sz1, sizeof(sz1));

	PGPclKeyListGetSelectionInfo (
			pds->ppks->hKL, NULL, NULL, &bMultiple, NULL);

	if (bMultiple)
		iFlags = MB_YESNOCANCEL|MB_TASKMODAL|MB_ICONEXCLAMATION;
	else
		iFlags = MB_YESNO|MB_TASKMODAL|MB_ICONEXCLAMATION;

	iRetVal = PGPclMessageBoxUTF8 (pds->ppks->hwndMain, sz2, sz1, iFlags);

	return iRetVal;
}


//	_______________________________________________
//
//  Get handle of nearby item
//
//	lptli	= pointer to TreeList item

static HTLITEM
sGetAdjacentItem (
		HWND			hwndTree,
		TL_TREEITEM*	lptli)
{
	TL_TREEITEM tli;

	tli.hItem = lptli->hItem;
	tli.mask = TLIF_NEXTHANDLE;
	TreeList_GetItem (hwndTree, &tli);
	if (!tli.hItem)
	{
		tli.hItem = lptli->hItem;
		tli.mask = TLIF_PREVHANDLE;
		TreeList_GetItem (hwndTree, &tli);
		if (!tli.hItem)
		{
			tli.hItem = lptli->hItem;
			tli.mask = TLIF_PARENTHANDLE;
			TreeList_GetItem (hwndTree, &tli);
		}
	}

	return tli.hItem;
}


//	_______________________________________________
//
//  Delete a single object
//	routine called either from KMDeleteObject or as a
//	callback function from the TreeList control to
//	delete a single item.
//
//	lptli	= pointer to TreeList item to delete

static BOOL CALLBACK
sDeleteSingleObject (
		TL_TREEITEM*	lptli,
		LPARAM			lParam)
{
	PDELETESTRUCT	pds			= (PDELETESTRUCT)lParam;
	PGPKeyDBObjRef	obj;
	INT				iConfirm;
	HWND			hwndParent;
	HWND			hwndTree;


	hwndParent = pds->ppks->hwndMain;
	hwndTree = pds->ppks->hwndTreeList;
	obj = (PGPKeyDBObjRef)(lptli->lParam);

	switch (lptli->iImage) {
	case IDX_RSASECKEY :
	case IDX_RSASECDISKEY :
	case IDX_RSASECSHRKEY :
	case IDX_DSASECKEY :
	case IDX_DSASECDISKEY :
	case IDX_DSASECSHRKEY :
	case IDX_RSAV4SECKEY :
	case IDX_RSAV4SECDISKEY :
	case IDX_RSAV4SECSHRKEY :
	case IDX_ECSECKEY :
	case IDX_ECSECDISKEY :
	case IDX_ECSECSHRKEY :
		iConfirm = sDeleteConfirm (lptli, IDS_DELCONFPRIVKEY, pds);
		if (iConfirm == IDYES)
		{
			if (obj == pds->keyDefault)
			{
				if (PKMessageBox (hwndParent, IDS_CAPTIONWARNING, 
						IDS_DELCONFDEFKEY,
						MB_YESNO|MB_TASKMODAL|MB_DEFBUTTON2|MB_ICONWARNING)
							==IDNO)
				{
					return TRUE;
				}
			}

			if (!pds->bItemNotDeleted)
			{
				pds->hPostDeleteFocusItem =
						sGetAdjacentItem (hwndTree, lptli);
			}

			PGPclKeyListUpdateTree (pds->ppks->hKL,
					kPGPclKeyListRemoveObject, obj, FALSE);
			PGPDeleteKeyDBObj (obj);
			PKDeletePropertiesWindowForObject (obj);
			pds->bItemModified = TRUE;
		}

		if (iConfirm == IDCANCEL)
			return FALSE;
		else
			return TRUE;

	case IDX_RSASECREVKEY :
	case IDX_RSASECEXPKEY :
	case IDX_DSASECREVKEY :
	case IDX_DSASECEXPKEY :
	case IDX_RSAV4SECREVKEY :
	case IDX_RSAV4SECEXPKEY :
	case IDX_ECSECREVKEY :
	case IDX_ECSECEXPKEY :
		iConfirm = sDeleteConfirm (lptli, IDS_DELCONFPRIVKEY, pds);
		if (iConfirm == IDYES)
		{
			if (!pds->bItemNotDeleted)
			{
				pds->hPostDeleteFocusItem =
						sGetAdjacentItem (hwndTree, lptli);
			}

			PGPclKeyListUpdateTree (pds->ppks->hKL,
					kPGPclKeyListRemoveObject, obj, FALSE);
			PGPDeleteKeyDBObj (obj);
			PKDeletePropertiesWindowForObject (obj);
			pds->bItemModified = TRUE;
		}

		if (iConfirm == IDCANCEL)
			return FALSE;
		else
			return TRUE;

	case IDX_RSATOKKEY :
	case IDX_RSATOKDISKEY :
	case IDX_RSAV4TOKKEY :
	case IDX_RSAV4TOKDISKEY :
	case IDX_RSATOKREVKEY :
	case IDX_RSATOKEXPKEY :
	case IDX_RSAV4TOKREVKEY :
	case IDX_RSAV4TOKEXPKEY :
		iConfirm = sDeleteTokenKeyMessage (lptli, pds);
		if (iConfirm == IDYES)
		{
			LPSTR		pszPIN			= NULL;

			CHAR		sz[128];
			PGPError	err;

			LoadString (g_hinst, IDS_TOKENPHRASEPROMPT, sz, sizeof(sz));
			err = PGPclGetConventionalPhrase (
					g_context, hwndParent, sz, &pszPIN);

			if (IsntPGPError (err))
			{
				HCURSOR		hcursorOld	= SetCursor (LoadCursor (NULL, IDC_WAIT));
				err = PGPDeleteKeyOnToken (obj, -1, pszPIN, lstrlen(pszPIN));
				SetCursor (hcursorOld);

				if (IsntPGPError (err))
				{
					PGPclKeyListUpdateTree (pds->ppks->hKL,
							kPGPclKeyListUpdateObject, obj, FALSE);
					pds->bItemModified = TRUE;
				}
				else if (err == kPGPError_BadPassphrase)
				{
					PKMessageBox (hwndParent, IDS_CAPTIONALERT, 
							IDS_BADTOKENPHRASE, MB_OK|MB_ICONEXCLAMATION);
				}
				else
					PGPclErrorBox (hwndParent, err);
			}

			if (IsntNull (pszPIN))
				PGPclFreePhrase (pszPIN);
		}

		if (iConfirm == IDCANCEL)
			return FALSE;
		else
			return TRUE;

	case IDX_RSAPUBKEY :
	case IDX_RSAPUBDISKEY :
	case IDX_RSAPUBREVKEY :
	case IDX_RSAPUBEXPKEY :
	case IDX_DSAPUBKEY :
	case IDX_DSAPUBDISKEY :
	case IDX_DSAPUBREVKEY :
	case IDX_DSAPUBEXPKEY :
	case IDX_RSAV4PUBKEY :
	case IDX_RSAV4PUBDISKEY :
	case IDX_RSAV4PUBREVKEY :
	case IDX_RSAV4PUBEXPKEY :
	case IDX_ECPUBKEY :
	case IDX_ECPUBDISKEY :
	case IDX_ECPUBREVKEY :
	case IDX_ECPUBEXPKEY :
	case IDX_UNKNOWNKEY :
		iConfirm = sDeleteConfirm (lptli, IDS_DELCONFKEY, pds);
		if (iConfirm == IDYES)
		{
			if (!pds->bItemNotDeleted)
			{
				pds->hPostDeleteFocusItem =
						sGetAdjacentItem (hwndTree, lptli);
			}

			PGPclKeyListUpdateTree (pds->ppks->hKL,
					kPGPclKeyListRemoveObject, obj, FALSE);
			PGPDeleteKeyDBObj (obj);
			PKDeletePropertiesWindowForObject (obj);
			pds->bItemModified = TRUE;
		}

		if (iConfirm == IDCANCEL)
			return FALSE;
		else
			return TRUE;

	case IDX_RSAUSERID :
	case IDX_DSAUSERID :
		if (PKIsThisTheOnlyUserID (obj))
		{
			PKMessageBox (hwndParent, IDS_CAPTIONINFO,
					IDS_DELONLYUSERID, MB_OK|MB_ICONINFORMATION);
			break;
		}
		// fall through

	case IDX_PHOTOUSERID :
	case IDX_INVALIDUSERID :
		iConfirm = sDeleteConfirm (lptli, IDS_DELCONFUSERID, pds);
		if (iConfirm == IDYES)
		{
			BOOL			bPrimary;
			PGPKeyDBObjRef	key;

			bPrimary = PKIsThisThePrimaryUserID (obj);
			if (bPrimary)
				pds->bDeletedPrimaryUserID = TRUE;

			if (!pds->bItemNotDeleted)
			{
				pds->hPostDeleteFocusItem =
						sGetAdjacentItem (hwndTree, lptli);
			}

			key = PGPPeekKeyDBObjKey (obj);
			PGPclKeyListUpdateTree (pds->ppks->hKL,
					kPGPclKeyListRemoveObject, obj, FALSE);
			PGPDeleteKeyDBObj (obj);
			pds->bItemModified = TRUE;

			if (key != pds->keyPrev)
			{
				if (PGPKeyDBObjRefIsValid (pds->keyPrev))
				{
					PGPclKeyListUpdateTree (pds->ppks->hKL,
							kPGPclKeyListUpdateObject, 
							pds->keyPrev, FALSE);
				}
				pds->keyPrev = key;
			}

			if (bPrimary)
			{
				PGPclKeyListUpdateTree (pds->ppks->hKL,
						kPGPclKeyListUpdateObject, key, FALSE);
			}
		}

		if (iConfirm == IDCANCEL)
			return FALSE;
		else
			return TRUE;

	case IDX_CERT :
	case IDX_REVCERT :
	case IDX_BADCERT :
	case IDX_EXPCERT :
	case IDX_EXPORTCERT :
	case IDX_TRUSTEDCERT :
	case IDX_METACERT :
	case IDX_X509CERT :
	case IDX_X509EXPCERT :
	case IDX_X509REVCERT :
		iConfirm = sDeleteConfirm (lptli, IDS_DELCONFCERT, pds);
		if (iConfirm == IDYES)
		{
			PGPKeyDBObjRef	key;

			if (!pds->bItemNotDeleted)
			{
				pds->hPostDeleteFocusItem =
						sGetAdjacentItem (hwndTree, lptli);
			}

			PGPclKeyListUpdateTree (pds->ppks->hKL,
					kPGPclKeyListRemoveObject, obj, FALSE);
			PGPDeleteKeyDBObj (obj);
			PKDeletePropertiesWindowForObject (obj);
			pds->bItemModified = TRUE;

			key = PGPPeekKeyDBObjKey (obj);
			if (key != pds->keyPrev)
			{
				if (PGPKeyDBObjRefIsValid (pds->keyPrev))
				{
					PGPclKeyListUpdateTree (pds->ppks->hKL,
							kPGPclKeyListUpdateObject, 
							pds->keyPrev, FALSE);
				}
				pds->keyPrev = key;
			}
		}

		if (iConfirm == IDCANCEL)
			return FALSE;
		else
			return TRUE;
	}

	return FALSE;
}


//	_______________________________________________
//
//  Delete selected key or keys

BOOL
PKDeleteObject (
		PGPKEYSSTRUCT*	ppks)
{
	TL_TREEITEM		tli;
	DELETESTRUCT	ds;

	ds.pfnCallback = sDeleteSingleObject;
	ds.ppks = ppks;
	ds.bItemModified = FALSE;
	ds.bDeleteAll = FALSE;
	ds.bItemNotDeleted = FALSE;
	ds.bDeletedPrimaryUserID = FALSE;
	ds.hPostDeleteFocusItem = NULL;
	ds.keyPrev = kInvalidPGPKeyDBObjRef;
	PGPclGetDefaultPrivateKey (ppks->keydbMain, &ds.keyDefault);

	// call callback function for all selected objects
	TreeList_IterateSelected (ppks->hwndTreeList, &ds);

	if (ds.bItemModified)
	{
		if (PGPKeyDBObjRefIsValid (ds.keyPrev))
		{
			PGPclKeyListUpdateTree (ppks->hKL,
					kPGPclKeyListUpdateObject, 
					ds.keyPrev, FALSE);
		}

		if (ds.bDeletedPrimaryUserID)
		{
			PGPclKeyListSetTree (ppks->hKL, kPGPclResort, NULL);
		}
		else
		{
			if (ds.bItemNotDeleted)
			{
				if (ds.hPostDeleteFocusItem)
				{
					tli.hItem = ds.hPostDeleteFocusItem;
					TreeList_Select (ppks->hwndTreeList, &tli, FALSE);
				}
				else
					PGPclKeyListSetTree (ppks->hKL, kPGPclDeselectAll, NULL);
			}
			else
			{
				PKKeyDBModified (ppks, PK_MOD_INDEX_0);
				PGPclKeyListSetTree (ppks->hKL, kPGPclDeselectAll, NULL);
				if (ds.hPostDeleteFocusItem)
				{
					tli.hItem = ds.hPostDeleteFocusItem;
					TreeList_Select (ppks->hwndTreeList, &tli, TRUE);
					tli.stateMask = TLIS_SELECTED;
					tli.state = 0;
					tli.mask = TLIF_STATE;
					TreeList_SetItem (ppks->hwndTreeList, &tli);
				}
			}
		}
		InvalidateRect (ppks->hwndTreeList, NULL, TRUE);
		PKUpdateAllObjectPropertiesWindows (ppks);

		PGPclKeyListUpdateTree (ppks->hKL, 
				kPGPclKeyListUpdateValidities, NULL, TRUE); 
	}
	return (ds.bItemModified);
}


//	_______________________________________________
//
//	Set focused key to be default signing key

BOOL
PKSetDefaultKey (
		PGPKEYSSTRUCT*	ppks)
{
	PGPKeyDBObjRef	keyNewDef;
	PGPKeyDBObjRef	keyOldDef;
	PGPError		err;
	PGPKeyID		keyid;

	PGPclGetDefaultPrivateKey (ppks->keydbMain, &keyOldDef);
	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &keyNewDef, NULL, NULL);

	err = PGPGetKeyID (keyNewDef, &keyid);
	if (IsntPGPError (err))
	{
		PGPPrefRef		prefref;

		if (IsntPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		{
			err = PGPSetPrefData (prefref,
					kPGPPrefDefaultKeyID, sizeof(PGPKeyID), &keyid);

			if (IsntPGPError (err))
			{
				err = PGPSetPrefBoolean (prefref,
						kPGPPrefWin32OutlookExchangeID, TRUE);
			}

			if (IsntPGPError (err))
				err = PGPclFlushClientLibPrefs (prefref, NULL);
		}
	}

	if (IsntPGPError (PGPclErrorBox (ppks->hwndMain, err)))
	{
		if (PGPKeyDBObjRefIsValid (keyOldDef))
		{
			PGPclKeyListUpdateTree (ppks->hKL,
					kPGPclKeyListUpdateObject, keyOldDef, FALSE);
		}

		PGPclKeyListUpdateTree (ppks->hKL,
				kPGPclKeyListUpdateObject, keyNewDef, TRUE);
		PGPclKeyListSetTree (ppks->hKL, 
				kPGPclSelectSpecifiedOnly, keyNewDef);

		return TRUE;
	}
	return FALSE;
}


//	_______________________________________________
//
//	Set focused UID to be primary UID

BOOL
PKSetPrimaryUserID (
		PGPKEYSSTRUCT*	ppks)
{
	PGPByte*		pPasskey			= NULL;
	LPSTR			pszPhrase			= NULL;

	PGPKeyDBObjRef	key;
	PGPKeyDBObjRef	userid;
	CHAR			sz[128];
	PGPError		err;
	PGPSize			sizePasskey;
	PGPBoolean		bSecret;

	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &userid, NULL, NULL);
	key = PGPPeekKeyDBObjKey (userid);

	err = PGPGetKeyDBObjBooleanProperty (key, 
			kPGPKeyProperty_IsSecret, &bSecret);
	if (IsPGPError (PGPclErrorBox (ppks->hwndMain, err)))
		return FALSE;

	if (bSecret)
	{
		// get valid passphrase
		LoadString (g_hinst, IDS_SELKEYPASSPHRASE, sz, sizeof(sz));
		err = PGPclGetKeyPhrase (g_context, g_tlscontext,
				ppks->hwndMain, sz, ppks->keydbMain, key,
				&pszPhrase, &pPasskey, &sizePasskey);
		PGPclErrorBox (ppks->hwndMain, err);

		// now we have a valid passphrase
		if (IsntPGPError (err))
		{
			err = PGPSetPrimaryUserID (userid,
				pPasskey ?
					PGPOPasskeyBuffer (g_context, pPasskey, sizePasskey) :
					PGPOPassphrase (g_context, pszPhrase),
				PGPOLastOption (g_context));
		}
	}
	else
	{
		err = PGPSetPrimaryUserID (userid,
			PGPOLastOption (g_context));
	}

	if (pszPhrase)
		PGPclFreePhrase (pszPhrase);
	if (pPasskey)
		PGPclFreePasskey (pPasskey, sizePasskey);

	if (IsntPGPError (PGPclErrorBox (ppks->hwndMain, err)))
	{
		PGPclKeyListUpdateTree (ppks->hKL,
				kPGPclKeyListReplaceObject, key, TRUE);
		PKUpdateAllObjectPropertiesWindows (ppks);
		return TRUE;
	}
	else
		return FALSE;
}


//	_______________________________________________
//
//	add selected keys to main keyset

BOOL
PKAddSelectedToMain (
		PGPKEYSSTRUCT*	ppks)
{
	PGPKeySetRef		keysetToAdd	= kInvalidPGPKeySetRef;
	PGPError			err			= kPGPError_NoErr;
	BOOL				bRet		= FALSE;

	err = PGPclKeyListGetSelectedKeys (ppks->hKL, &keysetToAdd);
	if (IsntPGPError (err) &&
		PGPKeySetRefIsValid (keysetToAdd))
	{
		err = PGPCopyKeys (keysetToAdd, ppks->keydbMain, NULL);
		if (IsntPGPError (err))
			bRet = TRUE;
		PGPFreeKeySet (keysetToAdd);
	}

	if (bRet)
	{
		PKKeyDBModified (ppks, PK_MOD_INDEX_0);
		PostMessage (g_hwndMain, PK_M_REFRESHKEYRINGS, 0, FALSE);
	}

	return bRet;
}


//	_______________________________________________
//
//	routine called as a callback function from the
//  PGPCheckKeyRingSigs function

static PGPError
sKeyCheckEventHandler(
		PGPContextRef	context,
		PGPEvent		*event,
		PGPUserValue	userValue)
{
	PREVERIFYSTRUCT prvs;
	CHAR			sz[64];
	INT				iTotal;

	prvs = (PREVERIFYSTRUCT)userValue;

	iTotal = LOWORD (event->data.nullData.bytesTotal);
	if (iTotal != prvs->iNumSigsTotal)
	{
		prvs->iNumSigsTotal = iTotal;

		EnableWindow (GetDlgItem (prvs->hwndProgress, IDCANCEL), TRUE);

		LoadString (g_hinst, IDS_VERIFYING, sz, sizeof(sz));
		SetDlgItemText (prvs->hwndProgress, IDC_TEXT, sz);

		SendDlgItemMessage (prvs->hwndProgress, IDC_PROGRESSBAR,
					PBM_SETRANGE, 0, MAKELPARAM (0, prvs->iNumSigsTotal));
	}

	SendDlgItemMessage (prvs->hwndProgress, IDC_PROGRESSBAR,
			PBM_SETPOS, LOWORD (event->data.nullData.bytesWritten), 0);

	if (prvs->bCancel)
		return kPGPError_UserAbort;
	else
		return kPGPError_NoErr;
}

//	_______________________________________________
//
//	reverify signature progress dialog message procedure

static BOOL CALLBACK
sReverifyProgressDlgProc (
		HWND	hDlg,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PREVERIFYSTRUCT prvs;
	CHAR			sz[64];

	switch (uMsg) {
	case WM_INITDIALOG :
		SetWindowLong (hDlg, GWL_USERDATA, lParam);
		prvs = (PREVERIFYSTRUCT)lParam;
		LoadString (g_hinst, IDS_PREPARINGTOVERIFY, sz, sizeof(sz));
		SetDlgItemText (hDlg, IDC_TEXT, sz);
		prvs->hwndProgress = hDlg;
		return TRUE;

	case WM_APP :
		prvs = (PREVERIFYSTRUCT)GetWindowLong (hDlg, GWL_USERDATA);
		SetForegroundWindow (prvs->hwndParent);
		EndDialog (hDlg, 0);
		break;

	case WM_COMMAND :
		switch (LOWORD (wParam)) {
		case IDCANCEL :
			prvs = (PREVERIFYSTRUCT)GetWindowLong (hDlg, GWL_USERDATA);
			prvs->bCancel = TRUE;
			break;
		}
		return TRUE;
	}
	return FALSE;
}

//	_______________________________________________
//
//	reverify signatures thread routine

static DWORD
sReverifyProgressThreadRoutine (LPVOID lpvoid)
{
	PREVERIFYSTRUCT		prvs	= (PREVERIFYSTRUCT)lpvoid;

	DialogBoxParam (g_hinst,
					MAKEINTRESOURCE(IDD_REVERIFYING), NULL,
					sReverifyProgressDlgProc, (LPARAM)prvs);

	return 0;
}

//	_______________________________________________
//
//	reverify signatures of selected keys

BOOL
PKReverifySigs (
		PGPKEYSSTRUCT*	ppks)
{
	PGPKeySetRef		keysetReverify	= kInvalidPGPKeySetRef;
	PGPError			err				= kPGPError_NoErr;
	REVERIFYSTRUCT		rvs;

	rvs.hwndParent = ppks->hwndMain;
	rvs.hwndProgress = NULL;
	rvs.iNumSigsTotal = -1;
	rvs.bCancel = FALSE;
	_beginthread (sReverifyProgressThreadRoutine, 0, &rvs);

	// wait for dialog box
	EnableWindow (ppks->hwndMain, FALSE);
	while (rvs.hwndProgress == NULL) Sleep (100);

	err = PGPclKeyListGetSelectedKeys (ppks->hKL, &keysetReverify);

	if (PGPKeySetRefIsValid (keysetReverify))
	{
		err = PGPCheckKeyRingSigs (keysetReverify,
				ppks->keydbMain, TRUE, sKeyCheckEventHandler, &rvs);
	}

	// send message to close down dialog
	SendMessage (rvs.hwndProgress, WM_APP, 0, 0);
	EnableWindow (ppks->hwndMain, TRUE);

	if (IsntPGPError (PGPclErrorBox (NULL, err)))
	{
		PGPclKeyListReloadKeys (ppks->hKL, FALSE);
	}

	if (PGPKeySetRefIsValid (keysetReverify))
		PGPFreeKeySet (keysetReverify);

	PKUpdateAllObjectPropertiesWindows (ppks);

	return (IsntPGPError (err));
}
