/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	PKMenu.c - handle menu enabling/disabling chores

	$Id: PKMenu.c,v 1.56 2002/10/31 04:13:36 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPkeysx.h"
#include "pgpImageList.h"
#include "PGPpda.h"

// pgp header files
#include "pgpKeyServerPrefs.h"

// constant definitions
#define	SENDTOSERVERPOS	0

#define MENU_FILE		0
#define MENU_EDIT		1
#define MENU_VIEW		2
#define MENU_KEYS		3
#define MENU_SERVER		4
#define MENU_GROUPS		5
#define MENU_HELP		6

#define SUBMENU_ADD		2


// External globals
extern HINSTANCE		g_hinst;
extern PGPBoolean		g_bKeyGenEnabled;
extern PGPBoolean		g_bX509CertRequestEnabled;
extern PGPBoolean		g_bKeyReconstructionEnabled;
extern PGPUInt32		g_uNumTokens;
extern PGPContextRef	g_context;
extern PGPPDASTRUCT*	g_pPDA;


//	____________________________________
//
//  Enable/Disable menu items

static VOID
sSetItem (
		HMENU	hMenu,
		INT		iId,
		BOOL	bEnable)
{
	if (bEnable)
		EnableMenuItem (hMenu, iId, MF_BYCOMMAND|MF_ENABLED);
	else
		EnableMenuItem (hMenu, iId, MF_BYCOMMAND|MF_GRAYED);
}


//	___________________________________________________
//
//	create popup menu which is list of keyservers

static HMENU
sCreateKeyserverMenu (
		PGPKEYSSTRUCT*	ppks)
{
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	PGPUInt32			uNumServers		= 0;
	HMENU				hmenu			= NULL;

	PGPKeyServerEntry*	keyserverList;
	PGPUInt32			u, uID;
	PGPError			err;
	CHAR				sz[256];

	if (IsPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		return NULL;

	hmenu = CreatePopupMenu ();
	err = PGPGetKeyServerPrefs (prefref, &keyserverList, &uNumServers);

	if (IsntPGPError (err))
	{
		LoadString (g_hinst, IDS_DOMAINSERVER, sz, sizeof(sz));
		AppendMenu (hmenu, MF_STRING, IDM_DOMAINKEYSERVERX, sz);

		if(PGPlnEnterprise())
		{
			if (g_bKeyReconstructionEnabled)
			{
				LoadString (g_hinst, IDS_RECONSTRUCTIONSERVER, sz, sizeof(sz));
				AppendMenu (hmenu, MF_STRING, IDM_DOMAINKEYSERVERX +1, sz);
				sSetItem (hmenu, IDM_DOMAINKEYSERVERX +1, 
						PKIsActionEnabled (ppks, KM_SENDTORECONSERVER));
			}
		}

		LoadString (g_hinst, IDS_TOKEN, sz, sizeof(sz));
		AppendMenu (hmenu, MF_STRING, IDM_SENDTOTOKEN, sz);
		sSetItem (hmenu, IDM_SENDTOTOKEN, 
				PKIsActionEnabled (ppks, KM_SENDTOTOKEN));

		LoadString (g_hinst, IDS_EMAILRECIPIENT, sz, sizeof(sz));
		AppendMenu (hmenu, MF_STRING, IDM_SENDVIAEMAIL, sz);

		AppendMenu (hmenu, MF_SEPARATOR, 0, NULL);

		uID = 2;
		for (u=0; u<uNumServers; u++)
		{
			if (IsKeyServerListed (keyserverList[u].flags))
			{
				PGPGetKeyServerURL (&(keyserverList[u]), sz);
				AppendMenu (hmenu, MF_STRING, IDM_DOMAINKEYSERVERX +uID, sz);
				++uID;
			}
		}

		if (keyserverList)
			PGPFreeKeyServerList (keyserverList);
	}

	return hmenu;
}


//	___________________________________________________
//
//	initialize the main window menu based on build flags

VOID
PKInitMenuKeyMan (
		HMENU	hmenuMain,
		UINT	uFlags)
{
	HMENU	hmenu;

	// get "File" menu
	hmenu = GetSubMenu (hmenuMain, MENU_FILE);

	if ((uFlags & (PK_MAINWINDOW|PK_PDAWINDOW)) &&
		IsntNull (g_pPDA))
	{
		PGPError (*pGetPdaString)(PGPpdaStringStruct*);

		PGPPDASTRUCT*		pPDA			= g_pPDA;
		UINT				uIndex			= 0;
		BOOL				bItemAdded		= FALSE;

		PGPpdaStringStruct	pss;
		MENUITEMINFO		mii;
		HMODULE				hmod;

		while (pPDA->hmod)
		{
			if (pPDA->uPlatform == 0)
			{
				pPDA++;
				uIndex++;
				continue;
			}

			hmod = pPDA->hmod;

			pGetPdaString = (PGPError (*)(PGPpdaStringStruct*))
					GetProcAddress (hmod, "PGPpdaGetString");

			if (pGetPdaString)
			{
				if (pPDA->bKeyring)
				{
					if (uFlags & PK_MAINWINDOW)
					{
						mii.cbSize = sizeof(mii);

						pss.uStringIndex = kPGPpdaString_OpenKeyringMenuText;
						if ((*pGetPdaString)(&pss) == kPGPError_NoErr)
						{
							mii.fMask = MIIM_TYPE | MIIM_ID;
							mii.fType = MFT_STRING;
							mii.wID = IDM_PDAOPERATION + uIndex;
							mii.dwTypeData = pss.szString;
							mii.cch = lstrlen (pss.szString);

							if (InsertMenuItem (hmenu, IDM_CLOSE, FALSE, &mii))
								bItemAdded = TRUE;
						}
					}
				}
				else
				{
					if (((pPDA->uFlags & PK_PDAWINDOWOP) && 
							(uFlags & PK_PDAWINDOW)) ||
						(!(pPDA->uFlags & PK_PDAWINDOWOP) && 
							(uFlags & PK_MAINWINDOW)))
					{
						mii.cbSize = sizeof(mii);

						pss.uStringIndex = 
								kPGPpdaString_OperationMenuText + pPDA->uOp;
						if ((*pGetPdaString)(&pss) == kPGPError_NoErr)
						{
							mii.fMask = MIIM_TYPE | MIIM_ID;
							mii.fType = MFT_STRING;
							mii.wID = IDM_PDAOPERATION + uIndex;
							mii.dwTypeData = pss.szString;
							mii.cch = lstrlen (pss.szString);

							if (InsertMenuItem (hmenu, IDM_CLOSE, FALSE, &mii))
								bItemAdded = TRUE;
						}
					}
				}
			}

			pPDA++;
			uIndex++;

			if (pPDA->hmod != hmod)
			{
				if (bItemAdded)
				{
					mii.fMask = MIIM_TYPE;
					mii.fType = MFT_SEPARATOR;
					InsertMenuItem (hmenu, IDM_CLOSE, FALSE, &mii);
				}
			}
		}
	}

	if(PGPlnEnterprise())
	{
		if (PGPclIsClientInstall ())
		{
			// get "Servers" menu
			hmenu = GetSubMenu (hmenuMain, MENU_SERVER);

			// delete "Send Group Lists" item
			DeleteMenu (hmenu, IDM_SENDGROUPLISTS, MF_BYCOMMAND);
		}
	}
	else
	{
		// get "Servers" menu
		hmenu = GetSubMenu (hmenuMain, MENU_SERVER);

		// delete "Update" items
		DeleteMenu (hmenu, IDM_SENDGROUPLISTS, MF_BYCOMMAND);
		DeleteMenu (hmenu, IDM_UPDATEGROUPLISTS, MF_BYCOMMAND);
		DeleteMenu (hmenu, IDM_UPDATECONFIGURATION, MF_BYCOMMAND);
		DeleteMenu (hmenu, IDM_UPDATEINTRODUCERS, MF_BYCOMMAND);

		// get "Keys" menu
		hmenu = GetSubMenu (hmenuMain, MENU_KEYS);

		// delete "Reconstruct" item
		DeleteMenu (hmenu, IDM_RECONSTRUCTKEY, MF_BYCOMMAND);
	}

	hmenu = GetSubMenu (hmenuMain, MENU_FILE);
	if (uFlags & PK_MAINWINDOW)
		DeleteMenu (hmenu, IDM_CLOSE, MF_BYCOMMAND);
	else
	{
		DeleteMenu (hmenu, IDM_FILEEXIT, MF_BYCOMMAND);

		// now delete entire "Groups" menu
		DeleteMenu (hmenuMain, MENU_GROUPS, MF_BYPOSITION);
	}
}


//	___________________________________________________
//
//	derive the keyserver string name from the menu ID

VOID
PKGetServerActionFromID (
		UINT				uID,
		PGPKeyServerEntry*	pkeyserver,
		UINT*				puAction)
{
	PGPUInt32			uNumServers		= 0;
	HMENU				hmenu			= NULL;
	PGPKeyServerEntry*	keyserverList	= NULL;
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;

	PGPUInt32			u1, u2;
	PGPError			err;

	*puAction = KM_NOACTION;

	if (IsPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		return;

	// domain server
	if (uID == IDM_DOMAINKEYSERVERX)
	{
		err = PGPCreateKeyServerPath (prefref, "",
				&keyserverList, &uNumServers);
		if (IsntPGPError (err))
		{
			*puAction = KM_SENDTOSERVER;
			CopyMemory (pkeyserver, &(keyserverList[0]),
					sizeof(PGPKeyServerEntry));
			PGPDisposeKeyServerPath (keyserverList);
		}
	}

	// reconstruction server
	else if (uID == IDM_DOMAINKEYSERVERX +1)
	{
		*puAction = KM_SENDTORECONSERVER;
		ZeroMemory (pkeyserver, sizeof(PGPKeyServerEntry));
	}

	// other servers
	else
	{
		PGPGetKeyServerPrefs (prefref, &keyserverList, &uNumServers);

		u1 = IDM_DOMAINKEYSERVERX+2;
		if ((uID - u1) < uNumServers)
		{
			for (u2=0; u2<uNumServers; u2++)
			{
				if (IsKeyServerListed (keyserverList[u2].flags))
				{
					if (uID == u1)
					{
						*puAction = KM_SENDTOSERVER;
						CopyMemory (pkeyserver, &(keyserverList[u2]),
								sizeof(PGPKeyServerEntry));
						break;
					}
					u1++;
				}
			}
		}
		if (keyserverList)
			PGPFreeKeyServerList (keyserverList);
	}

	return;
}


//	_____________________________________________________
//
//  Set the Expand/Collapse items to "All" or "Selection"

static VOID
sSetMenuAllOrSelected (
		HMENU	hMenu,
		BOOL	bAll)
{
	MENUITEMINFO	mii;
	CHAR			sz[64];

	mii.cbSize = sizeof (MENUITEMINFO);
	mii.fMask = MIIM_TYPE;
	mii.fType = MFT_STRING;
	if (bAll)
	{
		LoadString (g_hinst, IDS_COLLAPSEALL, sz, sizeof(sz));
		mii.dwTypeData = sz;
		mii.cch = lstrlen (sz);
		SetMenuItemInfo (hMenu, IDM_COLLAPSESEL, FALSE, &mii);
		LoadString (g_hinst, IDS_EXPANDALL, sz, sizeof(sz));
		mii.dwTypeData = sz;
		mii.cch = lstrlen (sz);
		SetMenuItemInfo (hMenu, IDM_EXPANDSEL, FALSE, &mii);
	}
	else
	{
		LoadString (g_hinst, IDS_COLLAPSESEL, sz, sizeof(sz));
		mii.dwTypeData = sz;
		mii.cch = lstrlen (sz);
		SetMenuItemInfo (hMenu, IDM_COLLAPSESEL, FALSE, &mii);
		LoadString (g_hinst, IDS_EXPANDSEL, sz, sizeof(sz));
		mii.dwTypeData = sz;
		mii.cch = lstrlen (sz);
		SetMenuItemInfo (hMenu, IDM_EXPANDSEL, FALSE, &mii);
	}
}

//	______________________________________________
//
//  Set the "Set As ..." to "Default" or "Primary"

static VOID
sSetMenuDefaultOrPrimary (
		HMENU	hMenu,
		BOOL	bDefault)
{
	MENUITEMINFO	mii;
	CHAR			sz[64];

	mii.cbSize = sizeof (MENUITEMINFO);
	mii.fMask = MIIM_TYPE;
	mii.fType = MFT_STRING;
	if (bDefault)
	{
		LoadString (g_hinst, IDS_SETASDEFAULT, sz, sizeof(sz));
		mii.dwTypeData = sz;
		mii.cch = lstrlen (sz);
		SetMenuItemInfo (hMenu, IDM_SETASDEFAULT, FALSE, &mii);
	}
	else
	{
		LoadString (g_hinst, IDS_SETASPRIMARY, sz, sizeof(sz));
		mii.dwTypeData = sz;
		mii.cch = lstrlen (sz);
		SetMenuItemInfo (hMenu, IDM_SETASDEFAULT, FALSE, &mii);
	}
}


//	____________________________________
//
//  returns TRUE if multiple types of objects are selected

static BOOL
sPromiscuousSelected (
		PGPUInt32	uSel)
{
	switch (uSel) {
	case kPGPclKeyList_None :
	case kPGPclKeyList_Key :
	case kPGPclKeyList_UserID :
	case kPGPclKeyList_Signature :
		return FALSE;

	default :
		return TRUE;
	}
}


//	________________________________________
//
//	Return TRUE if signing operations allowed

static BOOL
sSigningAllowed (PGPUInt32 uSel)
{
	if ((uSel == kPGPclKeyList_None) ||
		(uSel & kPGPclKeyList_Signature))
	{
		return FALSE;
	}
	else
		return TRUE;
}


//	___________________________________________________
//
//	determine if each action is enabled or not

//	copy
static BOOL
sIsCopyEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel, uFocus;
	PGPBoolean		bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, &bMul, &uSel);

	if (sPromiscuousSelected (uSel)) return FALSE;
	if (uSel == kPGPclKeyList_Key) return TRUE;
	if (uSel == kPGPclKeyList_Signature)
	{
		if (bMul) return FALSE;
		if (uFocus == IDX_X509CERT) return TRUE;
		if (uFocus == IDX_X509EXPCERT) return TRUE;
		if (uFocus == IDX_X509REVCERT) return TRUE;
	}
	return FALSE;
}

//	paste
static BOOL
sIsPasteEnabled (PPGPKEYSSTRUCT ppks)
{
	if (ppks->bReadOnly) return FALSE;
	if (!PKDataToPaste ()) return FALSE;
	return TRUE;
}

//	delete
BOOL
PKIsDeleteEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus;
	PGPKeyDBObjRef	obj;
	PGPBoolean		bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, &obj, &bMul, NULL);

	if (ppks->bReadOnly) return FALSE;
	if (bMul) return TRUE;
	switch (uFocus) {
		case IDX_NONE :
			return FALSE;
		case IDX_RSAUSERID :
		case IDX_DSAUSERID :
			if (PKIsThisTheOnlyUserID (obj))
				return FALSE;
			else
				return TRUE;
		default :
			return TRUE;
	}
}

//	delete from server
static BOOL
sIsDeleteFromServerEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel;
	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, NULL, NULL, &uSel);

	if (sPromiscuousSelected (uSel)) return FALSE;
	if (uSel != kPGPclKeyList_Key) return FALSE;
	if ((ppks->klConfig.keyserver.protocol != kPGPKeyServerProtocol_LDAP) &&
		(ppks->klConfig.keyserver.protocol != kPGPKeyServerProtocol_LDAPS))
	{
		return FALSE;
	}
	return TRUE;
}

//	select all
static BOOL
sIsSelectAllEnabled (PPGPKEYSSTRUCT ppks)
{
	return TRUE;
}

//	collapse all
static BOOL
sIsCollapseAllEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, NULL, NULL);

	if (uFocus == IDX_NONE) return TRUE;
	return FALSE;
}

//	expand all
static BOOL
sIsExpandAllEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, NULL, NULL);

	if (uFocus == IDX_NONE) return TRUE;
	return FALSE;
}

//	collapse selected
static BOOL
sIsCollapseSelEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, NULL, NULL);

	if (uFocus == IDX_NONE) return FALSE;
	return TRUE;
}

//	expand selected
static BOOL
sIsExpandSelEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, NULL, NULL);

	if (uFocus == IDX_NONE) return FALSE;
	return TRUE;
}

//	certify
static BOOL
sIsCertifyEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus, uSel;
	PGPBoolean		bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, &bMul, &uSel);

	if (ppks->bReadOnly) return FALSE;
	if (sPromiscuousSelected (uSel)) return sSigningAllowed (uSel);
	switch (uFocus) {
		case IDX_NONE :
			return FALSE;
		case IDX_CERT :
		case IDX_REVCERT :
		case IDX_EXPCERT :
		case IDX_BADCERT :
		case IDX_EXPORTCERT :
		case IDX_TRUSTEDCERT :
		case IDX_METACERT :
			return FALSE;
		case IDX_RSASECEXPKEY :
		case IDX_RSASECREVKEY :
		case IDX_RSAPUBEXPKEY :
		case IDX_RSAPUBREVKEY :
		case IDX_DSASECEXPKEY :
		case IDX_DSASECREVKEY :
		case IDX_DSAPUBEXPKEY :
		case IDX_DSAPUBREVKEY :
		case IDX_RSAV4SECEXPKEY :
		case IDX_RSAV4SECREVKEY :
		case IDX_RSAV4PUBEXPKEY :
		case IDX_RSAV4PUBREVKEY :
		case IDX_RSATOKREVKEY :
		case IDX_RSATOKEXPKEY :
		case IDX_RSAV4TOKREVKEY :
		case IDX_RSAV4TOKEXPKEY :
		case IDX_ECPUBREVKEY :
		case IDX_ECPUBEXPKEY :
		case IDX_ECSECREVKEY :
		case IDX_ECSECEXPKEY :
			if (!bMul) return FALSE;
			else return TRUE;
		default :
			return TRUE;
	}
}

// enable
static BOOL
sIsEnableEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus, uSel;
	PGPBoolean		bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, &bMul, &uSel);

	if (ppks->bReadOnly) return FALSE;
	if (sPromiscuousSelected (uSel)) return FALSE;
	if (bMul) return FALSE;
	switch (uFocus) {
		case IDX_RSASECDISKEY :
		case IDX_RSAPUBDISKEY :
		case IDX_DSASECDISKEY :
		case IDX_DSAPUBDISKEY :
		case IDX_RSAV4SECDISKEY :
		case IDX_RSAV4PUBDISKEY :
		case IDX_RSATOKDISKEY :
		case IDX_RSAV4TOKDISKEY :
		case IDX_ECSECDISKEY :
		case IDX_ECPUBDISKEY :
			return TRUE;
		default :
			return FALSE;
	}
}

//	disable
static BOOL
sIsDisableEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus, uSel;
	PGPKeyDBObjRef	obj;
	PGPBoolean		b, bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, &obj, &bMul, &uSel);


	if (ppks->bReadOnly) return FALSE;
	if (sPromiscuousSelected (uSel)) return FALSE;
	if (bMul) return FALSE;
	switch (uFocus) {
		case IDX_RSAPUBKEY :
		case IDX_DSAPUBKEY :
		case IDX_RSAV4PUBKEY :
		case IDX_ECPUBKEY :
			return TRUE;
		case IDX_RSASECKEY :
		case IDX_RSASECSHRKEY :
		case IDX_DSASECKEY :
		case IDX_DSASECSHRKEY :
		case IDX_RSAV4SECKEY :
		case IDX_RSAV4SECSHRKEY :
		case IDX_RSATOKKEY :
		case IDX_RSAV4TOKKEY :
		case IDX_ECSECKEY :
		case IDX_ECSECSHRKEY :
			PGPGetKeyDBObjBooleanProperty (obj,
					kPGPKeyProperty_IsAxiomatic, &b);
			if (b) return FALSE;
			else return TRUE;
		default :
			return FALSE;
	}
}

//	disable on server
static BOOL
sIsDisableOnServerEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel;
	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, NULL, NULL, &uSel);

	if (sPromiscuousSelected (uSel)) return FALSE;
	if (uSel != kPGPclKeyList_Key) return FALSE;
	if ((ppks->klConfig.keyserver.protocol != kPGPKeyServerProtocol_LDAP) &&
		(ppks->klConfig.keyserver.protocol != kPGPKeyServerProtocol_LDAPS))
	{
		return FALSE;
	}
	return TRUE;
}

//	add user id
static BOOL
sIsAddUserEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus, uSel;
	PGPBoolean		bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, &bMul, &uSel);

	if (ppks->bReadOnly) return FALSE;
	if (sPromiscuousSelected (uSel)) return FALSE;
	if (bMul) return FALSE;
	switch (uFocus) {
		case IDX_RSASECKEY :
		case IDX_RSASECDISKEY :
		case IDX_RSASECSHRKEY :
		case IDX_DSASECKEY :
		case IDX_DSASECDISKEY :
		case IDX_DSASECSHRKEY :
		case IDX_RSAV4SECKEY :
		case IDX_RSAV4SECDISKEY :
		case IDX_RSAV4SECSHRKEY :
		case IDX_RSATOKKEY :
		case IDX_RSATOKDISKEY :
		case IDX_RSAV4TOKKEY :
		case IDX_RSAV4TOKDISKEY :
		case IDX_ECSECKEY :
		case IDX_ECSECDISKEY :
		case IDX_ECSECSHRKEY :
			return TRUE;
		default :
			return FALSE;
	}
}

//	add photo id
static BOOL
sIsAddPhotoEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus, uSel;
	PGPBoolean		bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, &bMul, &uSel);

	if (ppks->bReadOnly) return FALSE;
	if (sPromiscuousSelected (uSel)) return FALSE;
	if (bMul) return FALSE;
	switch (uFocus) {
		case IDX_DSASECKEY :
		case IDX_DSASECDISKEY :
		case IDX_DSASECSHRKEY :
		case IDX_RSAV4SECKEY :
		case IDX_RSAV4SECDISKEY :
		case IDX_RSAV4SECSHRKEY :
		case IDX_RSAV4TOKKEY :
		case IDX_RSAV4TOKDISKEY :
		case IDX_ECSECKEY :
		case IDX_ECSECDISKEY :
		case IDX_ECSECSHRKEY :
			return TRUE;
		default :
			return FALSE;
	}
}

//	add revoker
static BOOL
sIsAddRevokerEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus, uSel;
	PGPBoolean		bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, &bMul, &uSel);

	if (ppks->bReadOnly) return FALSE;
	if (sPromiscuousSelected (uSel)) return FALSE;
	if (bMul) return FALSE;
	switch (uFocus) {
		case IDX_DSASECKEY :
		case IDX_DSASECDISKEY :
		case IDX_DSASECSHRKEY :
		case IDX_RSAV4SECKEY :
		case IDX_RSAV4SECDISKEY :
		case IDX_RSAV4SECSHRKEY :
		case IDX_RSAV4TOKKEY :
		case IDX_RSAV4TOKDISKEY :
		case IDX_ECSECKEY :
		case IDX_ECSECDISKEY :
		case IDX_ECSECSHRKEY :
			return TRUE;
		default :
			return FALSE;
	}
}

//	add X509 certificate
static BOOL
sIsAddCertificateEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus, uSel;
	PGPKeyDBObjRef	obj;
	PGPBoolean		b, bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, &obj, &bMul, &uSel);

	if (ppks->bReadOnly) return FALSE;
	if (sPromiscuousSelected (uSel)) return FALSE;
	if (bMul) return FALSE;
	switch (uFocus) {
		case IDX_RSASECKEY :
		case IDX_RSASECDISKEY :
		case IDX_RSASECSHRKEY :
		case IDX_DSASECKEY :
		case IDX_DSASECDISKEY :
		case IDX_DSASECSHRKEY :
		case IDX_RSAV4SECKEY :
		case IDX_RSAV4SECDISKEY :
		case IDX_RSAV4SECSHRKEY :
		case IDX_RSATOKKEY :
		case IDX_RSATOKDISKEY :
		case IDX_RSAV4TOKKEY :
		case IDX_RSAV4TOKDISKEY :
		case IDX_ECSECKEY :
		case IDX_ECSECDISKEY :
		case IDX_ECSECSHRKEY :
			return TRUE;
		case IDX_RSAUSERID :
		case IDX_DSAUSERID :
			PGPGetKeyDBObjBooleanProperty (PGPPeekKeyDBObjKey (obj),
					kPGPKeyProperty_IsSecret, &b);
			if (b) return TRUE;
			else return FALSE;
		default :
			return FALSE;
	}
}

//	revoke
static BOOL
sIsRevokeEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel, uFocus;
	PGPKeyDBObjRef	obj;
	PGPBoolean		b1, b2, bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, &obj, &bMul, &uSel);

	if (ppks->bReadOnly) return FALSE;
	if (sPromiscuousSelected (uSel)) return FALSE;
	if (bMul) return FALSE;

	switch (uFocus) {
		case IDX_RSASECKEY :
		case IDX_RSASECDISKEY :
		case IDX_RSASECSHRKEY :
		case IDX_DSASECKEY :
		case IDX_DSASECDISKEY :
		case IDX_DSASECSHRKEY :
		case IDX_RSAV4SECKEY :
		case IDX_RSAV4SECDISKEY :
		case IDX_RSAV4SECSHRKEY :
		case IDX_RSATOKKEY :
		case IDX_RSATOKDISKEY :
		case IDX_RSAV4TOKKEY :
		case IDX_RSAV4TOKDISKEY :
		case IDX_ECSECKEY :
		case IDX_ECSECDISKEY :
		case IDX_ECSECSHRKEY :
			PGPGetKeyDBObjBooleanProperty (obj,
					kPGPKeyProperty_CanSign, &b2);
			if (b2) return TRUE;
			else return FALSE;

		case IDX_RSAPUBKEY :
		case IDX_RSAPUBDISKEY :
		case IDX_DSAPUBKEY :
		case IDX_DSAPUBDISKEY :
		case IDX_RSAV4PUBKEY :
		case IDX_RSAV4PUBDISKEY :
		case IDX_ECPUBKEY :
		case IDX_ECPUBDISKEY :
			PGPGetKeyDBObjBooleanProperty (obj,
					kPGPKeyProperty_IsRevocable, &b1);
 			if (b1) return TRUE;
			else return FALSE;

		case IDX_CERT :
		case IDX_BADCERT :
		case IDX_EXPORTCERT :
		case IDX_TRUSTEDCERT :
		case IDX_METACERT :
			PGPGetKeyDBObjBooleanProperty (obj,
					kPGPSigProperty_IsMySig, &b1);
			if (b1) return TRUE;
			else return FALSE;

		default :
			return FALSE;
	}
}

//	set as default
static BOOL
sIsSetAsDefaultEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel, uFocus;
	PGPKeyDBObjRef	obj;
	PGPBoolean		b, bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, &obj, &bMul, &uSel);

	if (sPromiscuousSelected (uSel)) return FALSE;
	if (bMul) return FALSE;
	switch (uFocus) {
		case IDX_RSASECKEY :
		case IDX_RSASECDISKEY :
		case IDX_RSASECSHRKEY :
		case IDX_DSASECKEY :
		case IDX_DSASECDISKEY :
		case IDX_DSASECSHRKEY :
		case IDX_RSAV4SECKEY :
		case IDX_RSAV4SECDISKEY :
		case IDX_RSAV4SECSHRKEY :
		case IDX_RSATOKKEY :
		case IDX_RSATOKDISKEY :
		case IDX_RSAV4TOKKEY :
		case IDX_RSAV4TOKDISKEY :
		case IDX_ECSECKEY :
		case IDX_ECSECDISKEY :
		case IDX_ECSECSHRKEY :
			PGPGetKeyDBObjBooleanProperty (obj,
					kPGPKeyProperty_CanSign, &b);
			if (b) return TRUE;
			else return FALSE;
		default :
			return FALSE;
	}
}

//	set as primary
static BOOL
sIsSetAsPrimaryEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel, uFocus;
	PGPKeyDBObjRef	obj;
	PGPBoolean		bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, &obj, &bMul, &uSel);

	if (ppks->bReadOnly) return FALSE;
	if (sPromiscuousSelected (uSel)) return FALSE;
	if (bMul) return FALSE;
	switch (uFocus) {
		case IDX_RSAUSERID :
		case IDX_DSAUSERID :
			return (!PKIsThisThePrimaryUserID (obj));
		default :
			return FALSE;
	}
}

//	import
static BOOL
sIsImportEnabled (PPGPKEYSSTRUCT ppks)
{
	if (ppks->bReadOnly) return FALSE;
	return TRUE;
}

//	export
static BOOL
sIsExportEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel, uFocus;
	PGPBoolean		bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, &bMul, &uSel);

	if (sPromiscuousSelected (uSel)) return FALSE;
	if (uSel == kPGPclKeyList_Key) return TRUE;
	if (uSel == kPGPclKeyList_Signature)
	{
		if (bMul) return FALSE;
		if (uFocus == IDX_X509CERT) return TRUE;
		if (uFocus == IDX_X509EXPCERT) return TRUE;
		if (uFocus == IDX_X509REVCERT) return TRUE;
	}
	return FALSE;
}

//	properties
static BOOL
sIsPropertiesEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel;
	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, NULL, NULL, &uSel);

	if (uSel == kPGPclKeyList_None) return FALSE;
	return TRUE;
}

//	send to keyserver
static BOOL
sIsSendToServerEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel;
	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, NULL, NULL, &uSel);

	if (sPromiscuousSelected (uSel)) return FALSE;
	if (uSel != kPGPclKeyList_Key) return FALSE;
	return TRUE;
}

//	get from keyserver
static BOOL
sIsGetFromServerEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel, uFocus;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, NULL, &uSel);

	if (sPromiscuousSelected (uSel)) return FALSE;
	if (uSel == kPGPclKeyList_None) return FALSE;
	if (uSel == kPGPclKeyList_UserID) return FALSE;
	switch (uFocus) {
		case IDX_X509CERT :
		case IDX_X509EXPCERT :
		case IDX_X509REVCERT :
			return FALSE;
		default :
			return TRUE;
	}
}

//	retrieve X509 certificate
static BOOL
sIsRetrieveCertificateEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel, uFocus;
	PGPBoolean		bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, &bMul, &uSel);

	if (ppks->bReadOnly) return FALSE;
	if (sPromiscuousSelected (uSel)) return FALSE;
	if (bMul) return FALSE;
	switch (uFocus) {
		case IDX_RSASECKEY :
		case IDX_RSASECDISKEY :
		case IDX_RSASECSHRKEY :
		case IDX_DSASECKEY :
		case IDX_DSASECDISKEY :
		case IDX_DSASECSHRKEY :
		case IDX_RSAV4SECKEY :
		case IDX_RSAV4SECDISKEY :
		case IDX_RSAV4SECSHRKEY :
		case IDX_RSATOKKEY :
		case IDX_RSATOKDISKEY :
		case IDX_RSAV4TOKKEY :
		case IDX_RSAV4TOKDISKEY :
		case IDX_ECSECKEY :
		case IDX_ECSECDISKEY :
		case IDX_ECSECSHRKEY :
			return TRUE;
		default :
			return FALSE;
	}
}

//	unselect all
static BOOL
sIsUnselectAllEnabled (PPGPKEYSSTRUCT ppks)
{
	return TRUE;
}

//	add to main
static BOOL
sIsAddToMainEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel;
	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, NULL, NULL, &uSel);

	if (sPromiscuousSelected (uSel)) return FALSE;
	if (uSel != kPGPclKeyList_Key) return FALSE;
	return TRUE;
}

//	reverify signatures
static BOOL
sIsReverifySigsEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel;
	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, NULL, NULL, &uSel);

	if (sPromiscuousSelected (uSel)) return FALSE;
	if (uSel != kPGPclKeyList_Key) return FALSE;
	return TRUE;
}

//	split key for sharing
static BOOL
sIsSplitKeyEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel, uFocus;
	PGPKeyDBObjRef	obj;
	PGPBoolean		b, bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, &obj, &bMul, &uSel);

	if (ppks->bReadOnly) return FALSE;
	if (bMul) return FALSE;
	switch (uFocus) {
		case IDX_RSASECKEY :
		case IDX_RSASECDISKEY :
		case IDX_DSASECKEY :
		case IDX_DSASECDISKEY :
		case IDX_RSAV4SECKEY :
		case IDX_RSAV4SECDISKEY :
		case IDX_ECSECKEY :
		case IDX_ECSECDISKEY :
			PGPGetKeyDBObjBooleanProperty (obj,
					kPGPKeyProperty_CanSign, &b);
			if (!b) return FALSE;
			PGPGetKeyDBObjBooleanProperty (obj,
					kPGPKeyProperty_CanEncrypt, &b);
			if (!b) return FALSE;
			return TRUE;
		default :
			return FALSE;
	}
}

//	reconstruct key
static BOOL
sIsReconstructKeyEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel, uFocus;
	PGPBoolean		bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, &bMul, &uSel);

	if (!g_bKeyReconstructionEnabled) return FALSE;
	if (ppks->bReadOnly) return FALSE;
	if (bMul) return FALSE;
	if (uSel != kPGPclKeyList_Key) return FALSE;
	return TRUE;
}

//	send to reconstruction server
static BOOL
sIsSendToReconServerEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus;
	PGPBoolean		bMul;
	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, &bMul, NULL);

	if (bMul) return FALSE;
	switch (uFocus) {
		case IDX_RSASECKEY :
		case IDX_RSASECDISKEY :
		case IDX_DSASECKEY :
		case IDX_DSASECDISKEY :
		case IDX_RSAV4SECKEY :
		case IDX_RSAV4SECDISKEY :
		case IDX_RSATOKKEY :
		case IDX_RSATOKDISKEY :
		case IDX_RSAV4TOKKEY :
		case IDX_RSAV4TOKDISKEY :
		case IDX_ECSECKEY :
		case IDX_ECSECDISKEY :
			return TRUE;
		default :
			return FALSE;
	}
}

//	send to token
static BOOL
sIsSendToTokenEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uFocus;
	PGPBoolean		bMul;

	if (g_uNumTokens == 0) return FALSE;

	PGPclKeyListGetSelectionInfo (ppks->hKL, &uFocus, NULL, &bMul, NULL);

	if (bMul) return FALSE;
	switch (uFocus) {
		case IDX_RSASECKEY :
		case IDX_RSASECDISKEY :
		case IDX_RSAV4SECKEY :
		case IDX_RSAV4SECDISKEY :
			return TRUE;
		default :
			return FALSE;
	}
}

//	send via email
static BOOL
sIsSendViaEmailEnabled (PPGPKEYSSTRUCT ppks)
{
	PGPUInt32		uSel;
	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, NULL, NULL, &uSel);

	if (sPromiscuousSelected (uSel)) return FALSE;
	if (uSel != kPGPclKeyList_Key) return FALSE;
	return TRUE;
}

//	___________________________________________________
//
//	determine if action is enabled or not, based on
//	flags passed in and current selection

BOOL
PKIsActionEnabled (
		PGPKEYSSTRUCT*	ppks,
		ULONG			ulAction)
{
	if (!ppks) return FALSE;

	switch (ulAction) {

	case KM_COPY :				return sIsCopyEnabled (ppks);
	case KM_PASTE :				return sIsPasteEnabled (ppks);
	case KM_DELETEFROMSERVER :	return sIsDeleteFromServerEnabled (ppks);
	case KM_DELETE :			return PKIsDeleteEnabled (ppks);
	case KM_SELECTALL :			return sIsSelectAllEnabled (ppks);
	case KM_COLLAPSEALL :		return sIsCollapseAllEnabled (ppks);
	case KM_COLLAPSESEL :		return sIsCollapseSelEnabled (ppks);
	case KM_EXPANDALL :			return sIsExpandAllEnabled (ppks);
	case KM_EXPANDSEL :			return sIsExpandSelEnabled (ppks);
	case KM_CERTIFY :			return sIsCertifyEnabled (ppks);
	case KM_ENABLE :			return sIsEnableEnabled (ppks);
	case KM_DISABLEONSERVER :	return sIsDisableOnServerEnabled (ppks);
	case KM_DISABLE :			return sIsDisableEnabled (ppks);
	case KM_ADDUSERID :			return sIsAddUserEnabled (ppks);
	case KM_ADDPHOTOID :		return sIsAddPhotoEnabled (ppks);
	case KM_ADDREVOKER :		return sIsAddRevokerEnabled (ppks);
	case KM_ADDCERTIFICATE :	return sIsAddCertificateEnabled (ppks);
	case KM_REVOKE :			return sIsRevokeEnabled (ppks);
	case KM_SETASDEFAULT :		return sIsSetAsDefaultEnabled (ppks);
	case KM_SETASPRIMARY :		return sIsSetAsPrimaryEnabled (ppks);
	case KM_IMPORT :			return sIsImportEnabled (ppks);
	case KM_EXPORT :			return sIsExportEnabled (ppks);
	case KM_PROPERTIES :		return sIsPropertiesEnabled (ppks);
	case KM_SENDTOSERVER :		return sIsSendToServerEnabled (ppks);
	case KM_GETFROMSERVER :		return sIsGetFromServerEnabled (ppks);
	case KM_RETRIEVECERTIFICATE:return sIsRetrieveCertificateEnabled (ppks);
	case KM_UNSELECTALL:		return sIsUnselectAllEnabled (ppks);
	case KM_ADDTOMAIN :			return sIsAddToMainEnabled (ppks);
	case KM_REVERIFYSIGS :		return sIsReverifySigsEnabled (ppks);
	case KM_SPLITKEY	 :		return sIsSplitKeyEnabled (ppks);
	case KM_RECONSTRUCTKEY	 :	return sIsReconstructKeyEnabled (ppks);
	case KM_SENDTORECONSERVER:	return sIsSendToReconServerEnabled (ppks);
	case KM_SENDTOTOKEN:		return sIsSendToTokenEnabled (ppks);
	case KM_SENDVIAEMAIL:		return sIsSendViaEmailEnabled (ppks);
	default :					return FALSE;
	}
}


//	_______________________________________________
//
//  Enable/Disable menu items on basis of currently
//  focused key type

VOID
PKSetMainMenu (PGPKEYSSTRUCT* ppks)
{
	HMENU			hMP;
	HKEYLIST		hKL;
	HGROUPMAN		hGM;
	ULONG			ulColumns;
	CHAR			sz[64];


	hKL = ppks->hKL;
	hGM = ppks->hGM;

	// "Edit" menu
	hMP = GetSubMenu (ppks->hmenuKeyMan, MENU_EDIT);

	sSetItem (hMP, IDM_COPYKEY, PKIsActionEnabled (ppks, KM_COPY));
	if (ppks->bGroupsFocused)
	{
		sSetItem (hMP, IDM_PASTEKEY, PGPgmIsActionEnabled (hGM, GM_PASTE));
		sSetItem (hMP, IDM_DELETEKEY, PGPgmIsActionEnabled (hGM, GM_DELETE));
		sSetMenuAllOrSelected (hMP,
				!PGPgmIsActionEnabled (hGM, GM_EXPANDSEL));
	}
	else
	{
		sSetItem (hMP, IDM_PASTEKEY, PKIsActionEnabled (ppks, KM_PASTE));
		sSetItem (hMP, IDM_DELETEKEY, PKIsActionEnabled (ppks, KM_DELETE));
		sSetMenuAllOrSelected (hMP,
				!PKIsActionEnabled (ppks, KM_EXPANDSEL));
	}

	// "View" menu
	hMP = GetSubMenu (ppks->hmenuKeyMan, MENU_VIEW);
	PGPclKeyListGetSelectedColumns (hKL, &ulColumns);

	if (ulColumns & kPGPclKeyList_ValidityColumn)
		CheckMenuItem (hMP, IDM_VIEWVALIDITY, MF_BYCOMMAND|MF_CHECKED);
	else
		CheckMenuItem (hMP, IDM_VIEWVALIDITY, MF_BYCOMMAND|MF_UNCHECKED);

	if (ulColumns & kPGPclKeyList_SizeColumn)
		CheckMenuItem (hMP, IDM_VIEWSIZE, MF_BYCOMMAND|MF_CHECKED);
	else
		CheckMenuItem (hMP, IDM_VIEWSIZE, MF_BYCOMMAND|MF_UNCHECKED);

	if (ulColumns & kPGPclKeyList_DescriptionColumn)
		CheckMenuItem (hMP, IDM_VIEWDESC, MF_BYCOMMAND|MF_CHECKED);
	else
		CheckMenuItem (hMP, IDM_VIEWDESC, MF_BYCOMMAND|MF_UNCHECKED);

	if (ulColumns & kPGPclKeyList_KeyIDColumn)
		CheckMenuItem (hMP, IDM_VIEWKEYID, MF_BYCOMMAND|MF_CHECKED);
	else
		CheckMenuItem (hMP, IDM_VIEWKEYID, MF_BYCOMMAND|MF_UNCHECKED);

	if (ulColumns & kPGPclKeyList_TrustColumn)
		CheckMenuItem (hMP, IDM_VIEWTRUST, MF_BYCOMMAND|MF_CHECKED);
	else
		CheckMenuItem (hMP, IDM_VIEWTRUST, MF_BYCOMMAND|MF_UNCHECKED);

	if (ulColumns & kPGPclKeyList_CreationColumn)
		CheckMenuItem (hMP, IDM_VIEWCREATION, MF_BYCOMMAND|MF_CHECKED);
	else
		CheckMenuItem (hMP, IDM_VIEWCREATION, MF_BYCOMMAND|MF_UNCHECKED);

	if (ulColumns & kPGPclKeyList_ExpirationColumn)
		CheckMenuItem (hMP, IDM_VIEWEXPIRATION, MF_BYCOMMAND|MF_CHECKED);
	else
		CheckMenuItem (hMP, IDM_VIEWEXPIRATION, MF_BYCOMMAND|MF_UNCHECKED);

	if (ulColumns & kPGPclKeyList_ADKColumn)
		CheckMenuItem (hMP, IDM_VIEWADK, MF_BYCOMMAND|MF_CHECKED);
	else
		CheckMenuItem (hMP, IDM_VIEWADK, MF_BYCOMMAND|MF_UNCHECKED);

	if (ppks->iToolHeight > 0)
		CheckMenuItem (hMP, IDM_VIEWTOOLBAR, MF_BYCOMMAND|MF_CHECKED);
	else
		CheckMenuItem (hMP, IDM_VIEWTOOLBAR, MF_BYCOMMAND|MF_UNCHECKED);

	sSetItem (hMP, IDM_VIEWTOKENPROPERTIES, (g_uNumTokens > 0));

	// "Keys" menu
	hMP = GetSubMenu (ppks->hmenuKeyMan, MENU_KEYS);

	sSetItem (hMP, IDM_CERTIFYKEY, PKIsActionEnabled (ppks, KM_CERTIFY));
	sSetItem (hMP, IDM_ENABLEKEY, PKIsActionEnabled (ppks, KM_ENABLE));
	sSetItem (hMP, IDM_DISABLEKEY, PKIsActionEnabled (ppks, KM_DISABLE));
	sSetItem (hMP, IDM_REVOKEKEY, PKIsActionEnabled (ppks, KM_REVOKE));
	sSetItem (hMP, IDM_REVERIFY,
			PKIsActionEnabled (ppks, KM_REVERIFYSIGS));

	if (PKIsActionEnabled (ppks, KM_SETASPRIMARY))
	{
		sSetMenuDefaultOrPrimary (hMP, FALSE);
		EnableMenuItem (hMP, IDM_SETASDEFAULT, MF_BYCOMMAND|MF_ENABLED);
	}
	else
	{
		sSetMenuDefaultOrPrimary (hMP, TRUE);
		sSetItem (hMP, IDM_SETASDEFAULT,
				PKIsActionEnabled (ppks, KM_SETASDEFAULT));
	}

	sSetItem (hMP, IDM_CREATEKEY, (!ppks->bReadOnly) && g_bKeyGenEnabled);

	sSetItem (hMP, IDM_SPLITKEY, PKIsActionEnabled (ppks, KM_SPLITKEY));
	sSetItem (hMP, IDM_RECONSTRUCTKEY,
			PKIsActionEnabled (ppks, KM_RECONSTRUCTKEY));

	sSetItem (hMP, IDM_IMPORTKEYS, PKIsActionEnabled (ppks, KM_IMPORT));
	sSetItem (hMP, IDM_EXPORTKEYS, PKIsActionEnabled (ppks, KM_EXPORT));

	sSetItem (hMP, IDM_PROPERTIES,
			PKIsActionEnabled (ppks, KM_PROPERTIES));

	// "Add" popup submenu
	hMP = GetSubMenu (hMP, SUBMENU_ADD);
	sSetItem (hMP, IDM_ADDUSERID,
			PKIsActionEnabled (ppks, KM_ADDUSERID));
	sSetItem (hMP, IDM_ADDPHOTOID,
			PKIsActionEnabled (ppks, KM_ADDPHOTOID));
	sSetItem (hMP, IDM_ADDREVOKER,
			PKIsActionEnabled (ppks, KM_ADDREVOKER));
	sSetItem (hMP, IDM_ADDCERTIFICATE,
			g_bX509CertRequestEnabled &&
			PKIsActionEnabled (ppks, KM_ADDCERTIFICATE));

	// "Servers" menu
	hMP = GetSubMenu (ppks->hmenuKeyMan, MENU_SERVER);

	DeleteMenu (hMP, SENDTOSERVERPOS, MF_BYPOSITION);
	LoadString (g_hinst, IDS_SENDTOSERVERMENU, sz, sizeof(sz));
	if (PKIsActionEnabled (ppks, KM_SENDTOSERVER))
	{
		ppks->hmenuKeyserver = sCreateKeyserverMenu (ppks);
		InsertMenu (hMP, SENDTOSERVERPOS, MF_BYPOSITION|MF_POPUP|MF_STRING,
				(UINT)ppks->hmenuKeyserver, sz);
	}
	else
	{
		InsertMenu (hMP, SENDTOSERVERPOS, MF_BYPOSITION|MF_STRING,
				IDM_SENDTOSERVERPOPUP, sz);
		sSetItem (hMP, IDM_SENDTOSERVERPOPUP, FALSE);
	}

	sSetItem (hMP, IDM_GETFROMSERVER,
			PKIsActionEnabled (ppks, KM_GETFROMSERVER));
	sSetItem (hMP, IDM_RETRIEVECERTIFICATE,
			PKIsActionEnabled (ppks, KM_RETRIEVECERTIFICATE));

	// "Groups" menu
	hMP = GetSubMenu (ppks->hmenuKeyMan, MENU_GROUPS);

	if (ppks->bGroupsVisible)
	{
		sSetItem (hMP, IDM_GROUPPROPERTIES,
				PGPgmIsActionEnabled (hGM, GM_PROPERTIES));
	}
	else
		sSetItem (hMP, IDM_GROUPPROPERTIES, FALSE);

}

//	______________________________________________________
//
//  Put up appropriate context menu on basis of key idx
//  of currently focused item.  Called in response to right
//  mouse click.
//
//	hWnd		= handle of parent window
//	uSelFlags	= bits indicate what types of objects are selected
//	x, y		= mouse position when right button clicked (screen coords)

VOID
PKContextMenu (
		PGPKEYSSTRUCT*	ppks,
		HWND			hWnd,
		INT				x,
		INT				y)
{
	HMENU			hMC;
	HMENU			hMCS;
	HMENU			hMenuTrackPopup;
	HKEYLIST		hKL;
	HGROUPMAN		hGM;
	UINT			uAddIndex;
	CHAR			sz[64];

	if (ppks->bGroupsFocused)
	{
		hGM = ppks->hGM;
		switch (ppks->uGroupSelectionFlags) {
		case 0 :
			hMC = NULL;
			break;

		case PGPGM_GROUPFLAG :
			hMC = LoadMenu (g_hinst, MAKEINTRESOURCE (IDR_MENUGROUP));
			sSetItem (hMC, IDM_PASTEKEY,
							PGPgmIsActionEnabled (hGM, GM_PASTE));
			sSetItem (hMC, IDM_DELETEKEY,
							PGPgmIsActionEnabled (hGM, GM_DELETE));
			sSetItem (hMC, IDM_GETFROMSERVER,
							PGPgmIsActionEnabled (hGM, GM_GETFROMSERVER));
			sSetItem (hMC, IDM_GROUPPROPERTIES,
							PGPgmIsActionEnabled (hGM, GM_PROPERTIES));
  			hMenuTrackPopup = GetSubMenu (hMC, 0);
			break;

		case PGPGM_KEYFLAG :
			hMC = LoadMenu (g_hinst, MAKEINTRESOURCE (IDR_MENUGROUPKEY));
			sSetItem (hMC, IDM_PASTEKEY,
							PGPgmIsActionEnabled (hGM, GM_PASTE));
			sSetItem (hMC, IDM_DELETEKEY,
							PGPgmIsActionEnabled (hGM, GM_DELETE));
			sSetItem (hMC, IDM_FINDKEY,
							PGPgmIsActionEnabled (hGM, GM_LOCATEKEYS));
			sSetItem (hMC, IDM_GETFROMSERVER,
							PGPgmIsActionEnabled (hGM, GM_GETFROMSERVER));
			sSetItem (hMC, IDM_PROPERTIES,
							PGPgmIsActionEnabled (hGM, GM_LOCATEKEYS));
			hMenuTrackPopup = GetSubMenu (hMC, 0);
			break;

		default :
			hMC = LoadMenu (g_hinst, MAKEINTRESOURCE (IDR_MENUGROUP));
			sSetItem (hMC, IDM_PASTEKEY, FALSE);
			sSetItem (hMC, IDM_DELETEKEY,
							PGPgmIsActionEnabled (hGM, GM_DELETE));
			sSetItem (hMC, IDM_GETFROMSERVER,
							PGPgmIsActionEnabled (hGM, GM_GETFROMSERVER));
			sSetItem (hMC, IDM_GROUPPROPERTIES, FALSE);
  			hMenuTrackPopup = GetSubMenu (hMC, 0);
			break;
		}
	}
	else
	{
		hKL = ppks->hKL;

		switch (ppks->uKeySelectionFlags) {
		case 0 :
			hMC = LoadMenu (g_hinst, MAKEINTRESOURCE (IDR_MENUNONE));
			sSetItem (hMC, IDM_PASTEKEY,
					PKIsActionEnabled (ppks, KM_PASTE));
  			hMenuTrackPopup = GetSubMenu (hMC, 0);
			break;

		case kPGPclKeyList_Key :
			if (ppks->uFlags & PK_KEYWINDOW)
			{
				hMC = LoadMenu (g_hinst, MAKEINTRESOURCE (IDR_MENUKEY));
				uAddIndex = 6;
			}
			else
			{
				if (ppks->bLocalKeySet)
				{
					hMC = LoadMenu (g_hinst,
							MAKEINTRESOURCE (IDR_MENUKEYSEARCHLOCAL));
					uAddIndex = 6;
				}
				else
				{
					hMC = LoadMenu (g_hinst,
							MAKEINTRESOURCE (IDR_MENUKEYSEARCHSERVER));
					uAddIndex = 7;
				}
				sSetItem (hMC, IDM_ADDTOMAIN,
						PKIsActionEnabled (ppks, KM_ADDTOMAIN));
			}
  			hMenuTrackPopup = GetSubMenu (hMC, 0);

			sSetItem (hMC, IDM_COPYKEY,
					PKIsActionEnabled (ppks, KM_COPY));
			sSetItem (hMC, IDM_PASTEKEY,
					PKIsActionEnabled (ppks, KM_PASTE));
			sSetItem (hMC, IDM_DELETEKEY,
					PKIsActionEnabled (ppks, KM_DELETE));
			sSetItem (hMC, IDM_DELETESERVER,
					PKIsActionEnabled (ppks, KM_DELETEFROMSERVER));
			sSetItem (hMC, IDM_CERTIFYKEY,
					PKIsActionEnabled (ppks, KM_CERTIFY));
			sSetItem (hMC, IDM_ENABLEKEY,
					PKIsActionEnabled (ppks, KM_ENABLE));
			if (ppks->bLocalKeySet)
				sSetItem (hMC, IDM_DISABLEKEY,
						PKIsActionEnabled (ppks, KM_DISABLE));
			else
				sSetItem (hMC, IDM_DISABLEKEY,
						PKIsActionEnabled (ppks, KM_DISABLEONSERVER));

			sSetItem (hMC, IDM_SPLITKEY,
					PKIsActionEnabled (ppks, KM_SPLITKEY));

			sSetItem (hMC, IDM_REVOKEKEY,
					PKIsActionEnabled (ppks, KM_REVOKE));
			sSetItem (hMC, IDM_REVERIFY,
					PKIsActionEnabled (ppks, KM_REVERIFYSIGS));
			sSetItem (hMC, IDM_SETASDEFAULT,
					PKIsActionEnabled (ppks, KM_SETASDEFAULT));
			sSetItem (hMC, IDM_EXPORTKEYS,
					PKIsActionEnabled (ppks, KM_EXPORT));

			LoadString (g_hinst, IDS_SENDTOSERVERMENU, sz, sizeof(sz));
			if (PKIsActionEnabled (ppks, KM_SENDTOSERVER))
			{
				ppks->hmenuKeyserver = sCreateKeyserverMenu (ppks);
				InsertMenu (hMenuTrackPopup, 13,
						MF_BYPOSITION|MF_POPUP|MF_STRING,
						(UINT)ppks->hmenuKeyserver, sz);
			}
			else
			{
				InsertMenu (hMC, 13, MF_BYPOSITION|MF_STRING,
						IDM_SENDTOSERVERPOPUP, sz);
				sSetItem (hMenuTrackPopup, IDM_SENDTOSERVERPOPUP, FALSE);
			}

			sSetItem (hMC, IDM_GETFROMSERVER,
					PKIsActionEnabled (ppks, KM_GETFROMSERVER));
			sSetItem (hMC, IDM_RETRIEVECERTIFICATE,
					PKIsActionEnabled (ppks, KM_RETRIEVECERTIFICATE));
			sSetItem (hMC, IDM_PROPERTIES,
					PKIsActionEnabled (ppks, KM_PROPERTIES));

			// take care of "Add" popup submenu
			hMCS = GetSubMenu (hMenuTrackPopup, uAddIndex);
			sSetItem (hMCS, IDM_ADDUSERID,
					PKIsActionEnabled (ppks, KM_ADDUSERID));
			sSetItem (hMCS, IDM_ADDPHOTOID,
					PKIsActionEnabled (ppks, KM_ADDPHOTOID));
			sSetItem (hMCS, IDM_ADDREVOKER,
					PKIsActionEnabled (ppks, KM_ADDREVOKER));
			sSetItem (hMCS, IDM_ADDCERTIFICATE,
					g_bX509CertRequestEnabled &&
					PKIsActionEnabled (ppks, KM_ADDCERTIFICATE));
			break;

		case kPGPclKeyList_UserID :
			hMC = LoadMenu (g_hinst, MAKEINTRESOURCE (IDR_MENUUID));
			sSetItem (hMC, IDM_DELETEKEY,
					PKIsActionEnabled (ppks, KM_DELETE));
			sSetItem (hMC, IDM_CERTIFYKEY,
					PKIsActionEnabled (ppks, KM_CERTIFY));
			sSetItem (hMC, IDM_SETASDEFAULT,
					PKIsActionEnabled (ppks, KM_SETASPRIMARY));
			sSetItem (hMC, IDM_PROPERTIES,
					PKIsActionEnabled (ppks, KM_PROPERTIES));
	  		hMenuTrackPopup = GetSubMenu (hMC, 0);

			// take care of "Add" popup submenu
			uAddIndex = 4;
			hMCS = GetSubMenu (hMenuTrackPopup, uAddIndex);
			sSetItem (hMCS, IDM_ADDCERTIFICATE,
					g_bX509CertRequestEnabled &&
					PKIsActionEnabled (ppks, KM_ADDCERTIFICATE));
			break;

		case kPGPclKeyList_Signature :
			hMC = LoadMenu (g_hinst, MAKEINTRESOURCE (IDR_MENUCERT));
			sSetItem (hMC, IDM_COPYKEY,
					PKIsActionEnabled (ppks, KM_COPY));
			sSetItem (hMC, IDM_DELETEKEY,
					PKIsActionEnabled (ppks, KM_DELETE));
			sSetItem (hMC, IDM_REVOKEKEY,
					PKIsActionEnabled (ppks, KM_REVOKE));
			sSetItem (hMC, IDM_GETFROMSERVER,
					PKIsActionEnabled (ppks, KM_GETFROMSERVER));
			sSetItem (hMC, IDM_EXPORTKEYS,
					PKIsActionEnabled (ppks, KM_EXPORT));
			sSetItem (hMC, IDM_PROPERTIES,
					PKIsActionEnabled (ppks, KM_PROPERTIES));
  			hMenuTrackPopup = GetSubMenu (hMC, 0);
			break;

		default :
			hMC = LoadMenu (g_hinst, MAKEINTRESOURCE (IDR_MENUPROMISCUOUS));
			sSetItem (hMC, IDM_DELETEKEY,
					PKIsActionEnabled (ppks, KM_DELETE));
			sSetItem (hMC, IDM_CERTIFYKEY,
					PKIsActionEnabled (ppks, KM_CERTIFY));
			sSetItem (hMC, IDM_PROPERTIES,
					PKIsActionEnabled (ppks, KM_PROPERTIES));
  			hMenuTrackPopup = GetSubMenu (hMC, 0);
			break;
		}
	}

	if (!hMC) return;

	TrackPopupMenu (hMenuTrackPopup, TPM_LEFTALIGN | TPM_RIGHTBUTTON,
			x, y, 0, hWnd, NULL);

	DestroyMenu (hMC);
}


//	______________________________________________________
//
//  Put up keyserver menu for send to server button in toolbar
//
//	hWnd		= handle of parent window

VOID
PKToolbarKeyserverMenu (
		PGPKEYSSTRUCT*	ppks,
		HWND			hWnd,
		LPRECT			lprect)
{
	HMENU		hMenuKeyserver;

	hMenuKeyserver = sCreateKeyserverMenu (ppks);
	TrackPopupMenu (hMenuKeyserver,
			TPM_LEFTALIGN|TPM_TOPALIGN|TPM_LEFTBUTTON,
			lprect->left +2, lprect->bottom +4, 0, hWnd, NULL);

	DestroyMenu (hMenuKeyserver);
}


//	___________________________________________________
//
//	perform the specified action

PGPError
PKPerformAction (
		PGPKEYSSTRUCT*	ppks,
		ULONG			ulAction)
{
	if (!ppks)
		return kPGPError_BadParams;
	if (!PKIsActionEnabled (ppks, ulAction))
		return kPGPError_UnknownRequest;

	switch (ulAction) {

	case KM_COPY :
		PKCopyKeys (ppks, NULL);
		break;

	case KM_PASTE :
		PKPasteKeys (ppks);
		break;

	case KM_DELETE :
		PKDeleteObject (ppks);
		break;

	case KM_SELECTALL :
		PGPclKeyListSetTree (ppks->hKL, kPGPclSelectAll, NULL);
		break;

	case KM_UNSELECTALL :
		PGPclKeyListSetTree (ppks->hKL, kPGPclDeselectAll, NULL);
		break;

	case KM_COLLAPSEALL :
		PGPclKeyListSetTree (ppks->hKL, kPGPclSelectAll, NULL);
		PGPclKeyListSetTree (ppks->hKL, kPGPclCollapse, NULL);
		PGPclKeyListSetTree (ppks->hKL, kPGPclDeselectAll, NULL);
		break;

	case KM_COLLAPSESEL :
		PGPclKeyListSetTree (ppks->hKL, kPGPclCollapse, NULL);
		break;

	case KM_EXPANDALL :
		PGPclKeyListSetTree (ppks->hKL, kPGPclSelectAll, NULL);
		PGPclKeyListSetTree (ppks->hKL, kPGPclExpand, NULL);
		PGPclKeyListSetTree (ppks->hKL, kPGPclDeselectAll, NULL);
		break;

	case KM_EXPANDSEL :
		PGPclKeyListSetTree (ppks->hKL, kPGPclExpand, NULL);
		break;

	case KM_CERTIFY :
		PKCertifyKeyOrUserID (ppks);
		break;

	case KM_ENABLE :
		PKEnableKey (ppks);
		break;

	case KM_DISABLE :
		PKDisableKey (ppks);
		break;

	case KM_ADDUSERID :
		PKAddUserToKey (ppks);
		break;

	case KM_ADDPHOTOID :
		PKAddPhotoToKey (ppks);
		break;

	case KM_ADDREVOKER :
		PKAddRevoker (ppks);
		break;

	case KM_ADDCERTIFICATE :
		PKAddCertificate (ppks);
		break;

	case KM_REVOKE :
	{
		PGPUInt32	uSel;

		PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, NULL, NULL, &uSel);
		if (uSel == kPGPclKeyList_Signature)
			PKRevokeCert (ppks);
		else
			PKRevokeKey (ppks);
		break;
	}

	case KM_SETASDEFAULT :
		PKSetDefaultKey (ppks);
		break;

	case KM_SETASPRIMARY :
		PKSetPrimaryUserID (ppks);
		break;

	case KM_IMPORT :
		PKInputFile (ppks, NULL);
		break;

	case KM_EXPORT :
		PKExportKeys (ppks, NULL);
		break;

	case KM_PROPERTIES :
		PKObjectProperties (ppks);
		break;

	case KM_SENDTOSERVER :
		PKSendToServer (ppks, kPGPclSpecifiedServer);
		break;

	case KM_GETFROMSERVER :
		PKGetFromServer (ppks);
		break;

	case KM_RETRIEVECERTIFICATE :
		PKRetrieveCertificate (ppks);
		break;

	case KM_DELETEFROMSERVER :
		PKDeleteFromServer (ppks);
		break;

	case KM_DISABLEONSERVER :
		PKDisableOnServer (ppks);
		break;

	case KM_ADDTOMAIN :
		PKAddSelectedToMain (ppks);
		break;

	case KM_REVERIFYSIGS :
		PKReverifySigs (ppks);
		break;

	case KM_SPLITKEY :
		PKSplitKey (ppks);
		break;

	case KM_RECONSTRUCTKEY :
		PKReconstructKey (ppks);
		break;

	case KM_SENDTORECONSERVER :
		PKSendToReconServer (ppks);
		break;

	case KM_SENDTOTOKEN :
		PKSendToToken (ppks);
		break;

	case KM_SENDVIAEMAIL :
		PKSendViaEmail (ppks);
		break;
	}

	return kPGPError_NoErr;
}
