/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	PKMsgPrc - main message processing and associated routines
	
	$Id: PKMsgPrc.c,v 1.104 2002/11/03 16:37:36 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"	/* or pgpConfig.h in the CDK */

// project header files
#include "PGPkeysx.h"
#include "search.h"
#include "pgpWin32IPC.h"
#include "pgpImageList.h"

// pgp header files
#include "pgpKeyserverPrefs.h" 
#include "PGPpda.h"

// system header files
#include <commdlg.h>
#include <zmouse.h>
#include <htmlhelp.h>

// constant defitions
#define RELOADDELAY			200			//delay to allow clearing of window
#define RELOADTIMER			1112		//

#define WRITELOCKTRIES		3			//num times to try keyring when locked
#define WRITELOCKDELAY		1500		//ms delay when keyring is writelocked

#define LEDTIMER			111
#define LEDTIMERPERIOD		100

#define NUMLEDS				10
#define LEDWIDTH			6
#define LEDSPACING			2
#define TOTALLEDWIDTH		(NUMLEDS*(LEDWIDTH+LEDSPACING+1))

#define ICON_PANE				0
#define STATUS_MSG_PANE			1
#define PROGRESS_PANE			2

#define TLS_NOSEARCHYET			0
#define TLS_NOTAUTHENTICATED	1
#define TLS_AUTHENTICATED		2

#define ICONWIDTH				25

#define SEARCHCONTROLHEIGHT		100

#define GRABBARHEIGHT			2

#define MINSEARCHWINDOWHEIGHT	260
#define MINSEARCHWINDOWWIDTH	460

#define PK_M_COPYFILENAMES		WM_APP+340

// external globals
extern HINSTANCE		g_hinst;
extern HWND				g_hwndMain;
extern HWND				g_hwndActive;
extern HIMAGELIST		g_hilKeys;
extern HICON			g_hiconToken;
extern HICON			g_hiconOpenLock;
extern HICON			g_hiconClosedLock;
extern UINT				g_uReloadPrefsMessage;
extern UINT				g_uReloadKeyringMessage;
extern UINT				g_uReloadKeyserverPrefsMessage;
extern UINT				g_uMouseWheelMessage;
extern PGPContextRef	g_context;
extern PGPtlsContextRef	g_tlscontext;
extern PGPBoolean		g_bExpertMode;
extern PGPBoolean		g_bKeyGenEnabled;
extern PGPBoolean		g_bMarginalAsInvalid;
extern BOOL				g_bShowGroups;
extern INT				g_iGroupsPercent;
extern INT				g_iToolHeight;
extern PGPUInt32		g_uNumTokens;
extern PGPPDASTRUCT*	g_pPDA;

// local globals
static BOOL			s_bKeyHasBeenGenerated		= FALSE;
static BOOL			s_bIgnorePrefsReloads		= FALSE;
static BOOL			s_bPrefsReloadIgnored		= FALSE;
static HWND			s_hwndOpenSearch			= NULL;
static BOOL			s_bInPDAOp					= FALSE;
static BOOL			s_bFirstKeyringLoad			= TRUE;
static ULONG		s_ulColumns					= 0;
static UINT			s_uStatusBarHeight			= 0;

static HBITMAP		s_hbmpWatermark				= NULL;


// typedefs
typedef struct {
	BOOL			bSearch;
	BOOL			bPda;
	BOOL			bReadOnly;
	PGPKeyDBRef		keydbMain;
} PKWINDOWSTRUCT, *PPKWINDOWSTRUCT;

typedef struct {
	BOOL			bNew;
	BOOL			bHavePub;
	BOOL			bHaveSec;
	CHAR			szPub[MAX_PATH];
	CHAR			szSec[MAX_PATH];
} PKOPENFILESTRUCT, *PPKOPENFILESTRUCT;

// local prototypes
static HWND
sNewPGPkeysWindow (
		HWND			hwndParent,
		PGPKEYSSTRUCT*	ppks,
		BOOL			bSearch,
		BOOL			bPda,
		BOOL			bReadOnly,
		LPSTR			pszPath,
		PGPKeyDBRef		keydb);

static DWORD aOpenFilesIds[] = {			// Help IDs
	IDC_PROMPT,				-1,
	IDC_PUBRING,			IDH_PGPPK_OPENPUBRING,
	IDC_SECRING,			IDH_PGPPK_OPENSECRING,
	IDC_BROWSEPUBRING,		IDH_PGPPK_BROWSEPUBRING,
	IDC_BROWSESECRING,		IDH_PGPPK_BROWSESECRING,
	0,0
};


//	____________________________________
//
//  Save keyset to PDA format data file

static BOOL 
sSaveKeySetToPdaFile (
		HWND			hwnd,
		HMODULE			hmod,
		PGPKeySetRef	keyset,
		BOOL			bClose) 
{
	PGPError			err;
	PGPpdaSaveStruct	pss;

	if (IsntNull (hmod))
	{
		PGPError (*pSavePdaKeyset)(PGPpdaSaveStruct*);

		pSavePdaKeyset = (PGPError (*)(PGPpdaSaveStruct*))
				GetProcAddress (hmod, "PGPpdaSaveKeyset");

		if (pSavePdaKeyset)
		{
			pss.context = g_context;
			pss.hwndParent = hwnd;
			pss.keyset = keyset;
			pss.bClose = bClose;

			err = (*pSavePdaKeyset)(&pss);
			if (IsntPGPError (PGPclErrorBox (hwnd, err)))
				return TRUE;
		}
	}

	return FALSE;
}


//	____________________________________
//
//  Open the PGP pda keyset

static BOOL 
sOpenPdaKeyring (
		HWND			hwnd,
		PGPPDASTRUCT*	pPDA,
		PGPKEYSSTRUCT*	ppks,
		BOOL			bNew)
{
	PGPError			err;
	PGPpdaOpenStruct	pos;

	if (IsntNull (pPDA->hmod))
	{
		PGPError (*pOpenPdaKeyset)(PGPpdaOpenStruct*);

		pOpenPdaKeyset = (PGPError (*)(PGPpdaOpenStruct*))
				GetProcAddress (pPDA->hmod, "PGPpdaOpenKeyset");

		if (pOpenPdaKeyset)
		{
			pos.context = g_context;
			pos.hwndParent = hwnd;

			err = (*pOpenPdaKeyset)(&pos);
			if (IsntPGPError (PGPclErrorBox (hwnd, err)))
			{
				if (bNew)
				{
					pPDA->hwnd = sNewPGPkeysWindow (hwnd, 
							ppks, FALSE, TRUE, FALSE, pos.szTitle, pos.keydb);
				}
				else
				{
					PGPclKeyListLoadKeys (ppks->hKL, NULL, NULL);
					PGPFreeKeyDB (ppks->keydbMain);
					ppks->keydbMain = pos.keydb;
					ppks->keysetDisp = PGPPeekKeyDBRootKeySet (pos.keydb);
					PGPclKeyListLoadKeys (
							ppks->hKL, ppks->keydbMain, ppks->keysetDisp);
				}

				PostMessage (pPDA->hwnd, PK_M_REFRESHKEYRINGS, 0, 0);

				return TRUE;
			}
		}
	}

	return FALSE;
}


//	____________________________________
//
//  perform a PDA operation (other than opening keyring)

static VOID 
sPdaOperation (
		HWND			hwnd,
		HMODULE			hmod,
		UINT			uOp)
{
	PGPError			err;
	PGPpdaOpStruct		pos;

	s_bInPDAOp = TRUE;

	if (IsntNull (hmod))
	{
		PGPError (*pPdaOp)(PGPpdaOpStruct*);
		
		pPdaOp = (PGPError (*)(PGPpdaOpStruct*))
				GetProcAddress (hmod, "PGPpdaOperation");

		if (pPdaOp)
		{
			pos.context = g_context;
			pos.hwndParent = hwnd;
			pos.uOperation = uOp;

			err = (*pPdaOp)(&pos);
			PGPclErrorBox (hwnd, err);
		}
	}

	s_bInPDAOp = FALSE;
}


//	____________________________________
//
//  find PDA struct from HWND

static PGPPDASTRUCT* 
sPDAFromWindow (
		HWND	hwnd)
{
	PGPPDASTRUCT*	p	= NULL;

	if (IsntNull (g_pPDA))
	{
		p = g_pPDA;
		while (p->hmod)
		{
			if (p->hwnd == hwnd)
				return p;
			p++;
		}
	}

	return NULL;
}


//	____________________________________
//
//  Process the PDA notification message

static LRESULT 
sProcessPdaNotification (
		HWND			hwnd, 
		PGPKEYSSTRUCT*	ppks,
		WPARAM			wParam,
		LPARAM			lParam)
{
	switch (wParam) {
	case PDA_IDENTIFY :
		return PGPPDA_M_NOTIFY_PALM;

	case PDA_SYNCOKQUERY :
		if (hwnd && IsWindowEnabled (hwnd))
			return 0;
		else
		{
			PKMessageBox (hwnd, IDS_CAPTIONINFO, 
					IDS_ATTEMPTINGPDASYNC, MB_OK|MB_ICONINFORMATION);
		}
		break;

	case PDA_SYNCSTART :
		if (hwnd && IsWindowEnabled (hwnd))
		{
			PGPKeyDBRef		keydb;
			PGPPDASTRUCT*	pPDA;

			pPDA = sPDAFromWindow (hwnd);

			if (IsntNull (pPDA))
			{
				keydb = PGPPeekKeySetKeyDB (ppks->keysetDisp);
				if (sSaveKeySetToPdaFile (hwnd, pPDA->hmod, 
						PGPPeekKeyDBRootKeySet (keydb), FALSE))
				{
					EnableWindow (hwnd, FALSE);
					return 0;
				}
			}
		}
		break;

	case PDA_SYNCSUCCESS :
	{
		PGPPDASTRUCT*	pPDA;

		pPDA = sPDAFromWindow (hwnd);

		if (IsntNull (pPDA))
			sOpenPdaKeyring (hwnd, pPDA, ppks, FALSE);

		EnableWindow (hwnd, TRUE);
		break;
	}

	case PDA_SYNCFAILURE :
		EnableWindow (hwnd, TRUE);
		break;

	case PDA_UPDATEUSERS :
		break;
	}

	return 1;
}


//	____________________________________
//
//  restore main PGPkeys window to saved position

static VOID 
sRestoreWindowPos (
		HWND	hwnd)
{
	DWORD	dwJunk;
	INT		iX, iY, iWidth, iHeight;
	BOOL	bJunk;
	INT		iJunk1, iJunk2;

	WINDOWPLACEMENT		wp;

	PKGetPrivatePrefData (&dwJunk, &iX, &iY, &iWidth, &iHeight, 
			&bJunk, &iJunk1, &iJunk2);

	wp.length			= sizeof (wp);
	wp.flags			= 0;
	
	wp.showCmd			= SW_SHOWNORMAL;
	wp.ptMaxPosition.x	= 0;
	wp.ptMaxPosition.y	= 0;
	wp.ptMinPosition.x	= 0;
	wp.ptMinPosition.y	= 0;

	wp.rcNormalPosition.left	= iX;
	wp.rcNormalPosition.top		= iY;
	wp.rcNormalPosition.right	= iX + iWidth;
	wp.rcNormalPosition.bottom	= iY + iHeight;

	SetWindowPlacement (hwnd, &wp);
}


//	____________________________________
//
//  Process files in command line

static VOID 
sProcessFileList (
		LPSTR			pszList,
		PGPKEYSSTRUCT*	ppks,
		BOOL			bCommandLine, 
		BOOL			bAllowSelect) 
{
	BOOL			bImported		= FALSE;
	PGPFileSpecRef	fileref;
	PGPKeyDBRef		keydbToAdd;
	LPSTR			p, p2;
	INT				iNumKeys;
	CHAR			cTerm;
	PGPInputFormat	format;
	PGPUInt32		uType;
	
	p = pszList;

	// skip over path of program 
	if (bCommandLine)
	{
		while (*p && (*p == ' ')) 
			p++;

		if (*p) 
		{
			if (*p == '"') 
			{
				p++;
				cTerm = '"';
			}
			else 
				cTerm = ' ';
		}

		while (*p && (*p != cTerm)) 
			p++;

		if (*p && (cTerm == '"')) 
			p++;
	}

	// parse file names
	// Unfortunately, the OS hands me names in the command line in all 
	// sorts of forms: space delimited; quoted and space delimited; space
	// delimiter after the program path but then NULL terminated.
	// And this is just NT ...
	while (p && *p) 
	{
		while (*p && (*p == ' ')) 
			p++;

		if (*p) 
		{
			if (*p == '"') 
			{
				while (*p == '"')
					p++;
				cTerm = '"';
				p2 = strchr (p, cTerm);
			}
			else 
			{
				cTerm = ' ';
				p2 = NULL;
			}

			if (p2) 
				*p2 = '\0';

			// import the exported key or group file ...
			if (PKFileIsForImporting (p, &uType, &format))
			{
				if (uType == PK_EXPORTEDGROUPFILE)
				{
					PGPgmImportGroups (ppks->hGM, p);
					ppks->bGroupsVisible = FALSE;
					PostMessage (ppks->hwndMain, WM_COMMAND,
							MAKEWPARAM (IDM_VIEWGROUPS, 0), 0);
				}
				else
				{
					PGPNewFileSpecFromFullPath (g_context, p, &fileref);
					if (fileref) 
					{
						PGPImport (g_context, &keydbToAdd, 
								PGPOInputFile (g_context, fileref),
								PGPOInputFormat (g_context, format),
								PGPOLastOption (g_context));
						if (PGPKeyDBRefIsValid (keydbToAdd))
						{
							PGPCountKeys (PGPPeekKeyDBRootKeySet (keydbToAdd), 
									&iNumKeys);
							if (iNumKeys > 0) 
							{
								if (bAllowSelect) 
								{
									if (IsntPGPError (PGPclImportKeys (
										g_context, g_tlscontext, g_hwndMain, 
										PGPPeekKeyDBRootKeySet (keydbToAdd), 
										ppks->keydbMain, 
										kPGPclDefaultImportFlags)))
									{
										bImported = TRUE;
									}
								}
								else 
								{
									if (IsntPGPError (PGPclImportKeys (
										g_context, g_tlscontext, g_hwndMain, 
										PGPPeekKeyDBRootKeySet (keydbToAdd), 
										ppks->keydbMain, kPGPclNoImportDialog)))
									{
										bImported = TRUE;
									}
								}

								if (bImported)
								{
									PKKeyDBModified (ppks, PK_MOD_INDEX_0);
									PGPclKeyListReloadKeys (ppks->hKL, TRUE);
								}
							}
							PGPFreeKeyDB (keydbToAdd);
						}
						PGPFreeFileSpec (fileref);
					}
				}
			}

			// ... or create new window for it
			else
			{
				PKNewWindowForFile (g_hwndMain, ppks, FALSE, p);
			}

			if (p2)
				*p2 = cTerm;
		}

		if (cTerm == '"')
		{
			while (*p && (*p != cTerm)) 
				p++;

			if (*p) 
				p++;
		}
		else
		{
			while (*p) 
				p++;
		}
	}
}


//	____________________________________
//
//  Import keys from WM_COPYDATA struct

static BOOL 
sImportData (
		PGPKEYSSTRUCT*	ppks, 
		PCOPYDATASTRUCT pcds) 
{
	PGPKeySetRef	keyset		= kInvalidPGPKeySetRef;
	BOOL			bPrompt;

	bPrompt = pcds->dwData & PGPPK_SELECTIVEIMPORT;

	switch (pcds->dwData & PGPPK_IMPORTKEYMASK) {
	case PGPPK_IMPORTKEYCOMMANDLINE :
		sProcessFileList (pcds->lpData, ppks, TRUE, bPrompt);
		return TRUE;
	}

	return FALSE;
}


//	____________________________________
//
//  Reload keyrings

static BOOL 
sReloadKeyrings (
		PGPKEYSSTRUCT*	ppks) 
{
	BOOL					bMutable	= TRUE;
	PGPOpenKeyDBFileOptions	flags		= kPGPOpenKeyDBFileOptions_Mutable;
	INT						iLockTries	= 0;

	PGPError				err;
	HCURSOR					hcursorOld;

	hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
	err = PGPclOpenDefaultKeyrings (g_context, 
			flags, &(ppks->keydbMain));
	SetCursor (hcursorOld);

	while (!PGPKeyDBRefIsValid (ppks->keydbMain) || 
			IsPGPError (PGPclKeyListLoadKeys (
					ppks->hKL, ppks->keydbMain, 
					PGPPeekKeyDBRootKeySet (ppks->keydbMain))))
	{
		switch (err) {

		case kPGPError_FilePermissions :
			bMutable = FALSE;
			break;

		case kPGPError_FileLocked :
			iLockTries++;
			if (iLockTries < WRITELOCKTRIES) 
				Sleep (WRITELOCKDELAY);
			else 
			{
				PKMessageBox (g_hwndMain, IDS_CAPTIONERROR,
						IDS_LOCKEDKEYRING, MB_OK|MB_ICONSTOP);

				if (!PKPGPPreferences (ppks, g_hwndMain, kPGPclKeyringPrefs)) 
				{
					SendMessage (g_hwndMain, WM_CLOSE, 0, 0);
					return FALSE;
				}
			}
			break;

		case kPGPError_CantOpenFile :
		case kPGPError_FileNotFound :
			if (!PKPGPPreferences (ppks, g_hwndMain, kPGPclKeyringPrefs)) 
			{
				SendMessage (g_hwndMain, WM_CLOSE, 0, 0);
				return FALSE;
			}
			break;

		default :
			PKMessageBox (g_hwndMain, IDS_CAPTIONERROR,
					IDS_CORRUPTKEYRING, MB_OK|MB_ICONSTOP);
			if (!PKPGPPreferences (ppks, g_hwndMain, kPGPclKeyringPrefs)) 
			{
				SendMessage (g_hwndMain, WM_CLOSE, 0, 0);
				return FALSE;
			}
			break;
		}
		hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
		flags = 0;
		if (bMutable) 
			flags |= kPGPOpenKeyDBFileOptions_Mutable;
		err = PGPclOpenDefaultKeyrings (g_context, flags, &ppks->keydbMain);

		SetCursor (hcursorOld);
	}

	PGPCalculateTrust (
			PGPPeekKeyDBRootKeySet (ppks->keydbMain), kInvalidPGPKeyDBRef);
	ppks->bReadOnly = !bMutable;

	if (ppks->bReadOnly) 
	{
		ppks->klConfig.uOptions |= kPGPclKeyList_ReadOnly;
		PKReadOnlyWarning (ppks->hwndMain);
	}
	else 
		ppks->klConfig.uOptions &= ~kPGPclKeyList_ReadOnly;

	// pass readonly flag to keymanager
	ppks->klConfig.uMask = kPGPclKeyList_Options;
	PGPclKeyListConfigure (ppks->hKL, &(ppks->klConfig));

	// reload the groups file
	ppks->gmConfig.keydbMain = ppks->keydbMain;
	PGPgmConfigure (ppks->hGM, &(ppks->gmConfig));
	PGPgmLoadGroups (ppks->hGM);

	// initialize drag/drop
	PKEnableDropTarget (ppks->pDropTarget, 
			!ppks->bReadOnly && (ppks->uFlags & PK_KEYWINDOW));

	return TRUE;
}

//	____________________________________
//
//  Draw the owner-drawn part of the status bar

static VOID
sDrawStatus (
		LPDRAWITEMSTRUCT lpdis, 
		PGPKEYSSTRUCT* ppks) 
{
	HBRUSH	hBrushLit, hBrushOld;
	HPEN	hPen, hPenOld;
	INT		i;
	INT		itop, ibot, ileft, iright;

	if (lpdis->itemID == PROGRESS_PANE)
	{
		if (ppks->iStatusValue < -1) 
			return;
		
		itop = lpdis->rcItem.top+3;
		ibot = lpdis->rcItem.bottom-5;

		hPen = CreatePen (PS_SOLID, 0, RGB (128, 128, 128));
		hPenOld = SelectObject (lpdis->hDC, hPen);
		hBrushLit = CreateSolidBrush (RGB (0, 255, 0));

		ileft = lpdis->rcItem.left + 4;

		// draw "Knight Rider" LEDs
		if (ppks->iStatusDirection) 
		{
			hBrushOld = SelectObject (lpdis->hDC, hBrushLit);
			for (i=0; i<NUMLEDS; i++) 
			{
				iright = ileft + LEDWIDTH;
		
				if (i == ppks->iStatusValue) 
					Rectangle (lpdis->hDC, ileft, itop, iright, ibot);
				else
				{
					//repaint the existing lit LED if any with standard system color
					RECT rectOfLitLed={ileft, itop, iright, ibot};
					FillRect(lpdis->hDC, &rectOfLitLed, (HBRUSH)(COLOR_3DFACE+1));

					//now draw the unlit LED
					Rectangle (lpdis->hDC, ileft+1, itop+2, iright-1, ibot-2);
				}
		
				ileft += LEDWIDTH + LEDSPACING;
			}
		}

		// draw "progress bar" LEDs
		else 
		{ 
			if (ppks->iStatusValue >= 0) 
			{
				hBrushOld = SelectObject (lpdis->hDC, hBrushLit);

				for (i=0; i<NUMLEDS; i++) 
				{
					iright = ileft + LEDWIDTH;
			
					if (i <= ppks->iStatusValue) 
						Rectangle (lpdis->hDC, ileft, itop, iright, ibot);
				
					ileft += LEDWIDTH + LEDSPACING;
				}
			}
		}

		SelectObject (lpdis->hDC, hBrushOld);
		SelectObject (lpdis->hDC, hPenOld);
		DeleteObject (hPen);
		DeleteObject (hBrushLit);
	}
}


//	____________________________________
//
//  display downloaded key count in status bar

static INT
sDisplayKSKeyResult (
		PGPKeySetRef	keyset, 
		HWND			hwnd, 
		PGPError		err) 
{
	CHAR sz1[256];
	CHAR sz2[256];
	INT	 iCount;

	if (keyset)
		(void)PGPCountKeys (keyset, &iCount);
	else 
		iCount = 0;

	if (err == kPGPError_ServerPartialSearchResults)
		LoadString (g_hinst, IDS_KSTOOMANYKEYCOUNT, sz1, sizeof(sz1));
	else 
		LoadString (g_hinst, IDS_KSKEYCOUNT, sz1, sizeof(sz1));
	wsprintf (sz2, sz1, iCount);
	SendMessage (hwnd, SB_SETTEXT, STATUS_MSG_PANE, (LPARAM)sz2);
	
	return iCount;
}


//	____________________________________
//
//  get number of wheel scroll lines and pass to treelist controls

static VOID
sUpdateWheelScrollLines (PGPKEYSSTRUCT* ppks)
{
	UINT uLines;

	if (!SystemParametersInfo (SPI_GETWHEELSCROLLLINES, 0, &uLines, 0)) 
	{
		HWND hwnd = NULL;
		UINT umsg = 0;

		umsg = RegisterWindowMessage (MSH_SCROLL_LINES);
		hwnd = FindWindow (MSH_WHEELMODULE_CLASS, MSH_WHEELMODULE_TITLE);

		if (hwnd && umsg) 
			uLines = (UINT)SendMessage (hwnd, umsg, 0, 0);
	}

	if (ppks->hwndTreeList)
		TreeList_SetWheelScrollLines (ppks->hwndTreeList, uLines);
	if (ppks->hwndTreeListGroups)
		TreeList_SetWheelScrollLines (ppks->hwndTreeListGroups, uLines);
}

//	____________________________________
//
//  create new search or file window

static HWND
sNewPGPkeysWindow (
		HWND			hwndParent,
		PGPKEYSSTRUCT*	ppks,
		BOOL			bSearch,
		BOOL			bPda,
		BOOL			bReadOnly,
		LPSTR			pszPath,
		PGPKeyDBRef		keydb)
{
	HWND			hwnd		= NULL;

	RECT			rc;
	RECT			rcMonitor;
	INT				iHeight, iWidth;
	CHAR			sz[64];
	LPSTR			pszTitle;
	PKWINDOWSTRUCT	pkws;
	HMENU			hmenu;

	// save column info of main window
	PGPclKeyListGetSelectedColumns (ppks->hKL, &s_ulColumns);
	PGPclKeyListSelectColumns (ppks->hKL, s_ulColumns, FALSE);

	// create new search window
	if (bSearch)
	{
		LoadString (g_hinst, IDS_SEARCHTITLE, sz, sizeof(sz));
		pszTitle = sz;
		hmenu = NULL;
	}
	else
	{
		pszTitle = pszPath;
		hmenu = LoadMenu (g_hinst, MAKEINTRESOURCE (IDR_MENUKEYMAN));
	}

	GetWindowRect (hwndParent, &rc);

	iWidth = rc.right - rc.left;
	if (iWidth < MINSEARCHWINDOWWIDTH)
		iWidth = MINSEARCHWINDOWWIDTH;

	iHeight = ((rc.bottom-rc.top)*3)>>2;
	if (iHeight < MINSEARCHWINDOWHEIGHT) 
		iHeight = MINSEARCHWINDOWHEIGHT;

	PGPclGetMonitorRect (hwndParent, kPGPclWindowMonitor, &rcMonitor);
	if (rc.left+60+iWidth > rcMonitor.right)
		rc.left = rcMonitor.right - iWidth - 60;

	pkws.bSearch = bSearch;
	pkws.bPda = bPda;
	pkws.bReadOnly = bReadOnly;
	pkws.keydbMain = keydb;

	hwnd = CreateWindow (PGPKEYS_WINCLASSNAME, pszTitle, 
			WS_OVERLAPPEDWINDOW|WS_VISIBLE, rc.left+60, 
			rc.top+60, iWidth,
			iHeight, NULL, hmenu, g_hinst, 
			(LPVOID)(&pkws));

	if (hwnd)
	{
		SendMessage (hwnd, PK_M_COPYFILENAMES, 0, (LPARAM)ppks);
		ppks->szFilePub[0] = '\0';
		ppks->szFileSec[0] = '\0';
	}

	return hwnd;
}


//	____________________________________
//
//  based on provided file, guess the other file name

static BOOL
sGuessMatchingKeyringFile (
	PKOPENFILESTRUCT*	ppofs,
	BOOL				bGuessSecring,
	BOOL				bNew)
{
	BOOL	bExists		= FALSE;
	CHAR*	p;
	CHAR	szGuess[MAX_PATH];
	CHAR	szExt[6];


	if (bGuessSecring && !ppofs->bHavePub)
		return FALSE;

	if (!bGuessSecring && !ppofs->bHaveSec)
		return FALSE;

	if (bGuessSecring)
	{
		lstrcpy (szGuess, ppofs->szPub);
		p = strrchr (szGuess, '.');

		if (lstrcmpi (p, ".pgp") == 0)
			lstrcpy (szExt, p);
		else
			lstrcpy (szExt, ".skr");

		if (p)
		{
			if (lstrlen (p) <= 4)
				*p = '\0';
		}

		p = strrchr (szGuess, '\\');

		if (lstrcmpi (p, "\\pubring") == 0)
		{
			*p = '\0';
			lstrcat (szGuess, "\\secring");
		}
	}
	else
	{
		lstrcpy (szGuess, ppofs->szSec);
		p = strrchr (szGuess, '.');

		if (lstrcmpi (p, ".pgp") == 0)
			lstrcpy (szExt, p);
		else
			lstrcpy (szExt, ".pkr");

		if (p)
		{
			if (lstrlen (p) <= 4)
				*p = '\0';
		}

		p = strrchr (szGuess, '\\');

		if (lstrcmpi (p, "\\secring") == 0)
		{
			*p = '\0';
			lstrcat (szGuess, "\\pubring");
		}
	}

	lstrcat (szGuess, szExt);

	if (PGPclTestPathNameStatus (szGuess) & kPGPclIsFile)
		bExists = TRUE;
	else if (!bNew)
	{
		WIN32_FIND_DATA		wfd;
		HANDLE				hfind;
		CHAR				szMatch[MAX_PATH];

		lstrcpy (szMatch, szGuess);

		p = strrchr (szMatch, '\\');
		if (!p)
		{
			p = strrchr (szMatch, ':');
			if (!p)
				p = &szMatch[0];
		}

		lstrcpy (p, "\\*");
		lstrcat (p, szExt);

		hfind = FindFirstFile (szMatch, &wfd);
		if (hfind != INVALID_HANDLE_VALUE)
		{
			bExists = TRUE;

			while (strstr (wfd.cFileName, "-bak"))
			{
				if (!FindNextFile (hfind, &wfd))
					break;
			}

			*p = '\0';
			lstrcpy (szGuess, szMatch);
			lstrcat (szGuess, "\\");
			lstrcat (szGuess, wfd.cFileName);
			FindClose (hfind);
		}
	}

	if (!bNew || !bExists)
	{
		if (bGuessSecring)
			lstrcpy (ppofs->szSec, szGuess);
		else
			lstrcpy (ppofs->szPub, szGuess);
	}

	return bExists;
}


//	____________________________________
//
//  prompt user for keyring file to open

static BOOL
sGetKeyringFileName (
		HWND			hwnd, 
		LPSTR			pszFile,
		BOOL			bSecring,
		BOOL			bNew)
{
	static CHAR		szInitDir[MAX_PATH] = {""};

	BOOL			bReturn				= FALSE;
	OPENFILENAME	ofn;
	LPSTR			p;
	CHAR			szPath[MAX_PATH];
	CHAR			szFilter[128];
	CHAR			szTitle[64];

	szPath[0] = '\0';

	if (bSecring)
		LoadString (g_hinst, IDS_SECRINGFILEFILTER, szFilter, sizeof(szFilter));
	else
		LoadString (g_hinst, IDS_PUBRINGFILEFILTER, szFilter, sizeof(szFilter));
	while (p = strrchr (szFilter, '@')) *p = '\0';

	if (szInitDir[0] == '\0')
	{
		if (pszFile[0] == '\0')
		{
			PGPclGetPath (kPGPclDefaultOpenFolder, 
					szInitDir, sizeof(szInitDir));
		}
		else
		{
			lstrcpy (szInitDir, pszFile);
			p = strrchr (szInitDir, '\\');
			if (p)
				*p = '\0';
		}
	}

	ofn.lStructSize			= sizeof (OPENFILENAME);
	ofn.hwndOwner			= hwnd;
	ofn.hInstance			= (HANDLE)g_hinst;
	ofn.lpstrFilter			= szFilter;
	ofn.lpstrCustomFilter	= (LPTSTR)NULL;
	ofn.nMaxCustFilter		= 0L;
	ofn.nFilterIndex		= 1L;
	ofn.lpstrFile			= szPath;
	ofn.nMaxFile			= sizeof(szPath);
	ofn.lpstrFileTitle		= NULL;
	ofn.nMaxFileTitle		= 0;
	ofn.lpstrInitialDir		= szInitDir;
	ofn.lpstrTitle			= szTitle;
	ofn.nFileOffset			= 0;
	ofn.nFileExtension		= 0;
	ofn.lpstrDefExt			= "";
	ofn.lCustData			= 0;
	ofn.Flags				= OFN_HIDEREADONLY|OFN_NOCHANGEDIR;

	// create new file
	if (bNew)
	{
		LoadString (g_hinst, 
				IDS_NEWKEYRINGFILECAPTION, szTitle, sizeof(szTitle));
		ofn.Flags |= OFN_OVERWRITEPROMPT;
		if (GetSaveFileName (&ofn)) 
			bReturn = TRUE;
	}

	// open existing file
	else
	{
		LoadString (g_hinst, 
				IDS_KEYRINGFILECAPTION, szTitle, sizeof(szTitle));
		ofn.Flags |= OFN_FILEMUSTEXIST;
		if (GetOpenFileName (&ofn)) 
			bReturn = TRUE;
	}

	lstrcpy (szInitDir, szPath);
	p = strrchr (szInitDir, '\\');
	if (p)
		*p = '\0';

	if (bReturn)
		lstrcpy (pszFile, szPath);

	return bReturn;
}


//	____________________________________
//
//  check if file is locked by someone else

static BOOL
sFileIsNotLocked (
		HWND	hwnd,
		LPSTR	pszFile)
{
	HANDLE	hFile;
	DWORD	dwErr;
	
	hFile = CreateFile (pszFile, GENERIC_READ, 0, NULL, 
			OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	if (hFile != INVALID_HANDLE_VALUE)
	{
		CloseHandle (hFile);
		return TRUE;
	}

	dwErr = GetLastError ();
	if (dwErr == ERROR_SHARING_VIOLATION)
	{
		PKMessageBox (hwnd, IDS_CAPTIONALERT, 
				IDS_KEYRINGFILELOCKED, MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	return TRUE;
}


//	____________________________________
//
//  open file pair dialog procedure

static BOOL CALLBACK
sOpenFilesDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PPKOPENFILESTRUCT	ppofs;

	switch (uMsg) {

	case WM_INITDIALOG :
		SetWindowLong (hwnd, GWL_USERDATA, lParam);
		ppofs = (PPKOPENFILESTRUCT)lParam;

		if (ppofs->bNew)
		{
			CHAR	sz[128];
			LoadString (g_hinst, IDS_NEWKEYRINGFILES, sz, sizeof(sz));
			SetDlgItemText (hwnd, IDC_PROMPT, sz);
		}

		SetDlgItemText (hwnd, IDC_PUBRING, ppofs->szPub);
		SetDlgItemText (hwnd, IDC_SECRING, ppofs->szSec);

		if (ppofs->bHavePub)
		{
			SendDlgItemMessage (hwnd, IDC_SECRING, EM_SETSEL, 0x1000, 0x1000);
			SetFocus (GetDlgItem (hwnd, IDC_SECRING));
		}
		else
		{
			SendDlgItemMessage (hwnd, IDC_PUBRING, EM_SETSEL, 0x1000, 0x1000);
			SetFocus (GetDlgItem (hwnd, IDC_PUBRING));
		}
		return FALSE;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aOpenFilesIds);

	case WM_COMMAND:
		ppofs = (PPKOPENFILESTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		switch (LOWORD(wParam)) {
		case IDCANCEL:
			EndDialog (hwnd, 0);
			break;

		case IDOK:
			GetDlgItemText (hwnd, IDC_PUBRING, 
					ppofs->szPub, sizeof(ppofs->szPub));
			GetDlgItemText (hwnd, IDC_SECRING, 
					ppofs->szSec, sizeof(ppofs->szSec));

			if (sFileIsNotLocked (hwnd, ppofs->szPub))
			{
				if (sFileIsNotLocked (hwnd, ppofs->szSec))
					EndDialog (hwnd, 1);
			}
			break;

		case IDC_BROWSEPUBRING :
			if (sGetKeyringFileName (hwnd, ppofs->szPub, FALSE, ppofs->bNew))
			{
				ppofs->bHavePub = TRUE;
				SetDlgItemText (hwnd, IDC_PUBRING, ppofs->szPub);

				if (!ppofs->bHaveSec)
				{
					if (sGuessMatchingKeyringFile (ppofs, TRUE, ppofs->bNew) &&
						ppofs->bNew)
					{
						ppofs->szSec[0] = '\0';
					}
					SetDlgItemText (hwnd, IDC_SECRING, ppofs->szSec);
				}
			}
			break;

		case IDC_BROWSESECRING :
			if (sGetKeyringFileName (hwnd, ppofs->szSec, TRUE, ppofs->bNew))
			{
				ppofs->bHaveSec = TRUE;
				SetDlgItemText (hwnd, IDC_SECRING, ppofs->szSec);

				if (!ppofs->bHavePub)
				{
					if (sGuessMatchingKeyringFile (ppofs, FALSE, ppofs->bNew) &&
						ppofs->bNew)
					{
						ppofs->szPub[0] = '\0';
					}
					SetDlgItemText (hwnd, IDC_PUBRING, ppofs->szPub);
				}
			}
			break;

		case IDC_PUBRING :
		case IDC_SECRING :
			if ((GetWindowTextLength (GetDlgItem (hwnd, IDC_PUBRING)) > 0) &&
				(GetWindowTextLength (GetDlgItem (hwnd, IDC_SECRING)) > 0))
			{
				EnableWindow (GetDlgItem (hwnd, IDOK), TRUE);
			}
			else
				EnableWindow (GetDlgItem (hwnd, IDOK), FALSE);
			break;

		}
		return TRUE;
	}
	return FALSE;
}


//	____________________________________
//
//  create a new window for the specified file

BOOL
PKNewWindowForFile (
		HWND			hwnd, 
		PGPKEYSSTRUCT*	ppks,
		BOOL			bNew,
		LPSTR			pszPath)
{
	PGPError			err				= kPGPError_UserAbort;
	BOOL				bReadOnly		= FALSE;
	PGPKeyDBRef			keydb;
	UINT				ulen;

	PKOPENFILESTRUCT	pofs;

	pofs.bNew		= bNew;
	pofs.bHavePub	= FALSE;
	pofs.bHaveSec	= FALSE;
	pofs.szPub[0]	= '\0';
	pofs.szSec[0]	= '\0';

	if (IsntNull (pszPath))
	{
		ulen = lstrlen (pszPath);
		if (ulen >= 5)
		{
			if (lstrcmpi (&pszPath[ulen-4], ".skr") == 0)
			{
				lstrcpy (pofs.szSec, pszPath);
				pofs.bHaveSec = TRUE;
				sGuessMatchingKeyringFile (&pofs, FALSE, FALSE);
			}
		}

		if (!pofs.bHaveSec)
		{
			lstrcpy (pofs.szPub, pszPath);
			pofs.bHavePub = TRUE;
			sGuessMatchingKeyringFile (&pofs, TRUE, FALSE);
		}
	}

	if (DialogBoxParam (g_hinst, MAKEINTRESOURCE(IDD_OPENFILES), hwnd, 
			sOpenFilesDlgProc, (LPARAM)&pofs))
	{
		if (bNew)
		{
			if (pofs.bHavePub)
				DeleteFile (pofs.szPub);
			if (pofs.bHaveSec)
				DeleteFile (pofs.szSec);
		}

		err = PGPclOpenKeyrings (g_context, 
				pofs.szPub, pofs.szSec,
				kPGPOpenKeyDBFileOptions_Mutable | 
				kPGPOpenKeyDBFileOptions_Create, 
				&keydb);

		if (err == kPGPError_FilePermissions)
		{
			err = PGPclOpenKeyrings (g_context, 
					pofs.szPub, pofs.szSec, 0,
					&keydb);

			if (IsntPGPError (err))
			{
				bReadOnly = TRUE;
				PKReadOnlyWarning (ppks->hwndMain);
			}
		}

		if (IsntPGPError (err))
		{
			sNewPGPkeysWindow (hwnd, ppks, 
					FALSE, FALSE, bReadOnly, pofs.szPub, keydb);
			return TRUE;
		}
	}

	PGPclErrorBox (hwnd, err);
	return FALSE;
}


//	____________________________________
//
//  see if any tokens are available

static void
sCheckForTokens (
		HWND	hwnd)
{
	PGPUInt32	uTokens;

	PGPCountTokens (g_context, &uTokens);
	if (uTokens != g_uNumTokens)
	{
		g_uNumTokens = uTokens;
		PKReloadTokenProperties (FALSE);
		PostMessage (hwnd, WM_SIZE, 0, 0);
	}
}


//	____________________________________
//
//  callback function called by SDK when keydb changes

static void
sSDKNotificationCallback (
		PGPUserValue			userval,
		PGPNotificationReason	reason,
		PGPUInt32				uParam1,
		PGPUInt32				uParam2)
{
	if (reason == kPGPNotification_KeyDBChanged)
	{
		sCheckForTokens ((HWND)userval);
		PostMessage ((HWND)userval, PK_M_RELOADKEYRINGS, 0, 0);
	}
}


//	____________________________________
//
//  Main PGPkeys Window Message procedure

LONG APIENTRY 
PGPkeysWndProc (
		HWND	hwnd, 
		UINT	uMsg, 
		WPARAM	wParam, 
		LPARAM	lParam) 
{
	PGPKEYSSTRUCT*	ppks;

	switch (uMsg) {

	case WM_CREATE :
	{
		RECT	rc;
		INT		iWidths[3];

		ppks = (PGPKEYSSTRUCT*)pkAlloc (sizeof (PGPKEYSSTRUCT));
		if (!ppks) 
			return -1;
		SetWindowLong (hwnd, GWL_USERDATA, (LPARAM)ppks);

		PGPclEnsureWindowIsOnMonitor (hwnd, NULL);
		GetClientRect (hwnd, &rc);

		// if lpCreateParams is NULL, this is the main window
		if (((LPCREATESTRUCT)lParam)->lpCreateParams == NULL) 
		{
			sRestoreWindowPos (hwnd);

			ppks->hwndMain = hwnd;
			ppks->uFlags = PK_MAINWINDOW;
			ppks->iToolHeight = g_iToolHeight; 
			ppks->iKeysHeight = 
				rc.bottom-rc.top-ppks->iToolHeight-s_uStatusBarHeight;
			ppks->bGroupsVisible = FALSE;
			ppks->iGroupsPercent = g_iGroupsPercent;
			ppks->iGroupsHeight = 0;
			ppks->bLocalKeySet = TRUE;
			ppks->keydbMain = NULL;
			ppks->keysetDisp = NULL;
			ppks->klConfig.uOptions = 
				kPGPclKeyList_EnableDragOut | kPGPclKeyList_EnableDropIn |
				kPGPclKeyList_EnableReloads;
			if (!g_bExpertMode) 
				ppks->klConfig.uOptions |= kPGPclKeyList_GreenDotMode;
			if (g_bMarginalAsInvalid) 
				ppks->klConfig.uOptions |= kPGPclKeyList_MarginalIsInvalid;

			ppks->hmenuKeyserver = NULL;
			ppks->hmenuKeyMan = GetMenu (hwnd);
			PKInitMenuKeyMan (ppks->hmenuKeyMan, ppks->uFlags);

			// setup group manager
			ppks->hGM = PGPgmCreateGroupManager (g_context, g_tlscontext,
					hwnd, IDC_TREELISTGROUP,
					0, 
					(rc.bottom-rc.top)-ppks->iGroupsHeight-
												s_uStatusBarHeight,
					(rc.right-rc.left),
					ppks->iGroupsHeight);
			ppks->hwndTreeListGroups = PGPgmGetManagerWindow (ppks->hGM);
			ppks->gmConfig.uOptions = 0;
			if (!g_bExpertMode) 
				ppks->gmConfig.uOptions |= GMF_NOVICEMODE;
			ppks->gmConfig.keydbMain = NULL;
			ppks->hwndToolBar = PKCreateToolbar (hwnd);
		}

		// otherwise this is a search or file window and lpCreateParams is 
		// a pointer to a PKWINDOWSTRUCT
		else 
		{
			PPKWINDOWSTRUCT		ppkws;
			ppkws = (PPKWINDOWSTRUCT)(((LPCREATESTRUCT)lParam)->lpCreateParams);

			ppks->bGroupsVisible = FALSE;
			ppks->iGroupsHeight = 0;
			ppks->iGroupsPercent = 0;
			ppks->bLocalKeySet = TRUE;
			ppks->keydbMain = ppkws->keydbMain;
			ppks->pGroupFile = NULL;

			if (ppkws->bSearch)
			{
				ppks->hwndMain = hwnd;
				ppks->uFlags = PK_SEARCHWINDOW;
				s_hwndOpenSearch = hwnd;
				ppks->iToolHeight = SEARCHCONTROLHEIGHT;
				ppks->keysetDisp = NULL;
				ppks->klConfig.uOptions = 
					kPGPclKeyList_EnableDragOut;
				ppks->hmenuKeyserver = NULL;
				ppks->hmenuKeyMan = NULL;
				ppks->hwndToolBar = NULL;
			}
			else
			{
				ppks->hwndMain = hwnd;
				if (ppkws->bPda)
					ppks->uFlags = PK_PDAWINDOW;
				else
					ppks->uFlags = PK_FILEWINDOW;
				ppks->iToolHeight = g_iToolHeight; 
				ppks->keysetDisp = PGPPeekKeyDBRootKeySet (ppkws->keydbMain);
				ppks->klConfig.uOptions = 
					kPGPclKeyList_EnableDragOut |
					kPGPclKeyList_EnableDropIn;
				if (ppkws->bReadOnly)
				{
					ppks->bReadOnly = TRUE;
					ppks->klConfig.uOptions |= kPGPclKeyList_ReadOnly;
				}
				ppks->hmenuKeyserver = NULL;
				ppks->hmenuKeyMan = GetMenu (hwnd);
				PKInitMenuKeyMan (ppks->hmenuKeyMan, ppks->uFlags);
				ppks->hwndToolBar = PKCreateToolbar (hwnd);
			}

			ppks->iKeysHeight = 
				rc.bottom-rc.top-ppks->iToolHeight-s_uStatusBarHeight;
			if (!g_bExpertMode) 
				ppks->klConfig.uOptions |= kPGPclKeyList_GreenDotMode;
			if (g_bMarginalAsInvalid) 
				ppks->klConfig.uOptions |= kPGPclKeyList_MarginalIsInvalid;

			if (ppkws->bSearch && InitSearch ()) 
			{
				ppks->hwndSearchControl = CreateSearch (g_hinst, hwnd);

				SendMessage (ppks->hwndSearchControl, SEARCH_SET_MAIN_KEYDB,
						0, (LPARAM)(ppks->keydbMain));
				SendMessage (ppks->hwndSearchControl, SEARCH_SET_LOCAL_KEYSET,
						0, (LPARAM)(PGPPeekKeyDBRootKeySet (ppks->keydbMain)));
				GetWindowRect (ppks->hwndSearchControl, &rc);
				ppks->iToolHeight = rc.bottom-rc.top;
			}
		}

		PKAddRemoveWindowList (hwnd, 
				TRUE, (ppks->uFlags & PK_SEARCHWINDOW), FALSE);

		ppks->bGroupsFocused = FALSE;
		ppks->uKeySelectionFlags = 0;
		ppks->uGroupSelectionFlags = 0;
		ppks->hCursorOld = NULL;
		ppks->bGrabEnabled = FALSE;
		ppks->bGrabbed = FALSE;
		ppks->pSplitKeyDlgList = NULL;

		ppks->hKL = PGPclKeyListCreate (g_context, g_tlscontext,
				hwnd, IDC_TREELISTKEYS, 
				0, ppks->iToolHeight, 
				rc.right-rc.left, 
				ppks->iKeysHeight, 0,
				&ppks->hwndTreeList);

		ppks->klConfig.keyserver.protocol = kPGPKeyServerProtocol_Invalid;
		ppks->klConfig.uShowColumns = 0;

		// initialize drag/drop
		ppks->pDropTarget = 
				PKCreateDropTarget (ppks->hwndTreeList, PK_IDROP_KEYLIST, ppks);
		CoLockObjectExternal ((IUnknown*)ppks->pDropTarget, TRUE, TRUE);
		RegisterDragDrop (ppks->hwndTreeList, ppks->pDropTarget);
		PKEnableDropTarget (ppks->pDropTarget, FALSE);

		// create status bar
		ppks->bSearchInProgress = FALSE;
		ppks->iStatusValue = -2;
		ppks->iStatusDirection = 1;
		ppks->klConfig.hwndStatusBar = CreateWindow (
				STATUSCLASSNAME, NULL,
				WS_CHILD|WS_VISIBLE|SBS_SIZEGRIP|SBT_TOOLTIPS,
				0, 0, 0, 0,
				hwnd, (HMENU)IDC_STATUSBAR, g_hinst, NULL);

		GetWindowRect (ppks->klConfig.hwndStatusBar, &rc);
		s_uStatusBarHeight = rc.bottom - rc.top;
		
		// setup status bar
		if (ppks->uFlags & PK_KEYWINDOW)
			iWidths[0] = 0;
		else
			iWidths[0] = ICONWIDTH;
		iWidths[1] = rc.right-TOTALLEDWIDTH-16;
		iWidths[2] = rc.right-16;
	
		SendMessage (ppks->klConfig.hwndStatusBar, 
				SB_SETPARTS, 3, (LPARAM)iWidths);

		SendMessage (ppks->klConfig.hwndStatusBar, 
				SB_SETTEXT, PROGRESS_PANE|SBT_OWNERDRAW, 0);

		// initialize TLS info
		ppks->iTLSstatus = TLS_NOSEARCHYET;
		ppks->szTLSserver[0] = '\0';
		ppks->keyAuth = kInvalidPGPKeyDBObjRef;

		// set initial configuration of keymanager
		ppks->klConfig.uMask = kPGPclKeyList_ConfigureAll;
		PGPclKeyListConfigure (ppks->hKL, &(ppks->klConfig));
		
		if (ppks->uFlags & PK_MAINWINDOW) 
		{
			CHAR			sz[64];

			LoadString (g_hinst, IDS_TOKENTOOLTIP, sz, sizeof(sz));
			SendMessage (ppks->klConfig.hwndStatusBar, 
					SB_SETTIPTEXT, ICON_PANE, (LPARAM)sz);

			SendMessage (ppks->klConfig.hwndStatusBar, 
					SB_SETICON, ICON_PANE, (LPARAM)g_hiconToken);

			PGPclKeyListGetSelectedColumns (ppks->hKL, &s_ulColumns);
			ppks->gmConfig.ppks = ppks;
			ppks->gmConfig.hKL = ppks->hKL;
			ppks->gmConfig.hwndStatusBar = ppks->klConfig.hwndStatusBar;
			PGPgmConfigure (ppks->hGM, &(ppks->gmConfig));
			PostMessage (hwnd, PK_M_RELOADKEYRINGS, 0, 0);
			PGPSetNotificationCallback (sSDKNotificationCallback, hwnd);
		}
		else if (ppks->uFlags & PK_KEYWINDOW)
		{
			PGPclKeyListLoadKeys (ppks->hKL, ppks->keydbMain, ppks->keysetDisp);

			PKEnableDropTarget (ppks->pDropTarget, !ppks->bReadOnly);
		}
		// must be a search window
		else
		{
			CHAR	sz[64];

			LoadString (g_hinst, IDS_SEARCHAUTHTOOLTIP, sz, sizeof(sz));
			SendMessage (ppks->klConfig.hwndStatusBar, 
					SB_SETTIPTEXT, ICON_PANE, (LPARAM)sz);

			SendMessage (ppks->klConfig.hwndStatusBar, 
					SB_SETICON, ICON_PANE, (LPARAM)g_hiconOpenLock);

			PGPclKeyListLoadKeys (ppks->hKL, ppks->keydbMain, ppks->keysetDisp);

			PKEnableDropTarget (ppks->pDropTarget, FALSE);

			ppks->iStatusValue = -1;
			InvalidateRect (ppks->klConfig.hwndStatusBar, NULL, FALSE);
		}

		{
			TL_WATERMARK	tlwm;
			HDC				hdc;
			INT				iNumBits;

			// Determine if display will support watermark
			hdc = GetDC (NULL);
			iNumBits = GetDeviceCaps (hdc, BITSPIXEL) * 
					GetDeviceCaps (hdc, PLANES);
			ReleaseDC (NULL, hdc);

			if (iNumBits > 8)
			{
				BITMAP	bmp;

				// Associate the watermark bitmap with the tree view control.
				if (IsNull (s_hbmpWatermark))
				{
					s_hbmpWatermark = LoadBitmap (
							g_hinst, MAKEINTRESOURCE(IDB_KEYSWATERMARK));
				}
				GetObject (s_hbmpWatermark, sizeof(bmp), &bmp);

				tlwm.hbm		= s_hbmpWatermark;
				tlwm.iAlpha		= 0;
				tlwm.crBkgrnd	= RGB (255,255,255);	// white
				tlwm.size.cx	= bmp.bmWidth;
				tlwm.size.cy	= bmp.bmHeight;
				tlwm.ulFlags	= TLWM_KEEPASPECTRATIO | 
								  TLWM_CONSTANTBKGRND |
								  TLWM_SHRINKONLY;

				TreeList_SetWatermarkBmp (ppks->hwndTreeList, &tlwm);

				if (ppks->uFlags & PK_MAINWINDOW)
				{
					tlwm.hbm = NULL;
					TreeList_SetWatermarkBmp (ppks->hwndTreeListGroups, &tlwm);
				}
			}
		}

		// initialize enabled/disabled states of toolbar buttons
		PKSetToolbarButtonStates (ppks);

		// initialize the number of mouse wheel scroll lines
		sUpdateWheelScrollLines (ppks);

		// initialize the token icon
		sCheckForTokens (ppks->hwndMain);

		return 0;
	}

	case WM_QUERYENDSESSION :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		if (ppks->uFlags & PK_MAINWINDOW) 
		{
			if (s_bKeyHasBeenGenerated) 
				return FALSE;
		}
		return TRUE;

	case WM_CLOSE :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		if (ppks->uFlags & PK_MAINWINDOW) 
		{
			if (PKExistSplitKeyDialog (ppks))
			{
				PKSplitKeyWarn (hwnd);
				break;
			}

			PKAutoBackup (ppks);

			if (s_bKeyHasBeenGenerated) 
			{
				BOOL	bMakeBackups;

				if (!PKBackupWarn (hwnd, TRUE, &bMakeBackups)) 
					break;

				if (bMakeBackups)
				{
					if (!PKUserBackup (hwnd, ppks->keydbMain))
						break;
				}
			}

			PGPclScheduleNextCRLUpdate (g_context, ppks->keydbMain);

			PKSetPrivatePrefData (hwnd, 
					ppks->bGroupsVisible, ppks->iGroupsPercent,
					ppks->iToolHeight);

			HtmlHelp (hwnd, NULL, HH_CLOSE_ALL, 0);
			if (s_hwndOpenSearch) 
				SendMessage (s_hwndOpenSearch, WM_CLOSE, 0, 0);

			PKSendMessageWindowList (PK_M_CLOSEWINDOW, 0, 0);
		}
		else
		{
			SendMessage (hwnd, PK_M_CLOSEWINDOW, 0, 0);
		}

		DestroyWindow (hwnd);
		return 0;

	case PK_M_CLOSEWINDOW :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		if (ppks->uFlags & PK_PDAWINDOW)
		{
			PGPKeyDBRef		keydb;
			PGPPDASTRUCT*	pPDA;

			keydb = PGPPeekKeySetKeyDB (ppks->keysetDisp);
			pPDA = sPDAFromWindow (hwnd);

			if (PGPKeyDBRefIsValid (keydb))
			{
				if (IsntNull (pPDA))
				{
					sSaveKeySetToPdaFile (hwnd, pPDA->hmod,
							PGPPeekKeyDBRootKeySet (keydb), TRUE); 
				}
			}
		}
		else if (ppks->uFlags & PK_SEARCHWINDOW) 
		{
			KillTimer (hwnd, LEDTIMER);
			s_hwndOpenSearch = NULL;
		}
		break;

	case WM_ACTIVATE :
		if (LOWORD (wParam) == WA_INACTIVE) 
			g_hwndActive = NULL;
		else
		{
			g_hwndActive = hwnd;
			ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
			if (ppks->uFlags & PK_MAINWINDOW) 
			{
				if (ppks->bGroupsFocused) 
					SetFocus (ppks->hwndTreeListGroups);
				else 
					SetFocus (ppks->hwndTreeList);
			}
			else 
				SetFocus (ppks->hwndTreeList);
		}
		break;

	case WM_ENABLE :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		if (ppks->uFlags & PK_KEYWINDOW) 
			PKEnableWindowList ((BOOL)wParam);

		PKEnableAllObjectPropertiesWindows (ppks, (BOOL)wParam);
		break;

	case WM_ENDSESSION :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		// we're not shutting down
		if (!wParam)
		{
			// check if we are the reason
			if (s_bKeyHasBeenGenerated)
			{
				// explain to user
				PKMessageBox (hwnd, IDS_CAPTIONINFO, 
						IDS_CLOSEKEYSFIRST, MB_OK|MB_ICONINFORMATION);
			}
			break;
		}

		// otherwise session is ending
		PKAutoBackup (ppks);
		PGPclScheduleNextCRLUpdate (g_context, ppks->keydbMain);

		PKSetPrivatePrefData (hwnd, 
				ppks->bGroupsVisible, ppks->iGroupsPercent,
				ppks->iToolHeight);

		// fall through

	case WM_DESTROY :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		if (ppks->hGM) 
		{
			PGPgmDestroyGroupManager (ppks->hGM);
			ppks->hGM = NULL;
		}
	
		RevokeDragDrop (ppks->hwndTreeList);
		PKReleaseDropTarget (ppks->pDropTarget);  
		CoLockObjectExternal ((IUnknown*)ppks->pDropTarget, FALSE, TRUE);
		DragAcceptFiles (ppks->hwndTreeList, FALSE);

		PKDeleteAllObjectPropertiesWindows (ppks);
		if (ppks->uFlags & PK_MAINWINDOW)
			PGPclKeyListDestroy (ppks->hKL, TRUE);
		else
			PGPclKeyListDestroy (ppks->hKL, FALSE);

		ppks->hKL = NULL;

		if (ppks->uFlags & PK_MAINWINDOW) 
		{
			PGPPrefRef		prefref;

			// this is because we've just saved a bunch of 
			// prefs concerning columns, windows, etc.
			if (IsntPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
				PGPclFlushClientLibPrefs (prefref, NULL);

			if (PGPKeyDBRefIsValid (ppks->keydbMain))
				PGPFreeKeyDB (ppks->keydbMain);

			PKAddRemoveWindowList (hwnd, FALSE, FALSE, FALSE);
			PKPostMessageWindowList (WM_DESTROY, 0, 0);

			PKDestroyToolbar (ppks->hwndToolBar);

			if (IsntNull (s_hbmpWatermark))
				DeleteObject (s_hbmpWatermark);

			PostQuitMessage(0);
		}
		else 
		{
			PGPKeyDBRef		keydb;

			keydb = PGPPeekKeySetKeyDB (ppks->keysetDisp);
			if (PGPKeySetRefIsValid (ppks->keysetDisp))
				PGPFreeKeySet (ppks->keysetDisp);

			if (ppks->uFlags & PK_FILEWINDOW)
			{
				if (PGPKeyDBRefIsValid (ppks->keydbMain))
					PGPFreeKeyDB (ppks->keydbMain);
			}
			else if (ppks->uFlags & PK_PDAWINDOW)
			{
				PGPPDASTRUCT*	pPDA;

				pPDA = sPDAFromWindow (hwnd);
				if (IsntNull (pPDA))
					pPDA->hwnd = NULL;

				if (PGPKeyDBRefIsValid (keydb))
					PGPFreeKeyDB (keydb);
			}
			else
			{
				if (PGPKeyDBRefIsValid (keydb) &&
					(keydb != ppks->keydbMain))
				{
					PGPFreeKeyDB (keydb);
				}
			}
			
			PKAddRemoveWindowList (hwnd, FALSE, FALSE, FALSE);
		}

		if (PGPKeyDBRefIsValid (ppks->keydbAuth))
			PGPFreeKeyDB (ppks->keydbAuth);

		pkFree (ppks);
		ppks = NULL;
		break;

	case WM_INITMENU :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		if ((HMENU)wParam == ppks->hmenuKeyMan) 
			PKSetMainMenu (ppks);
		break;

	case WM_SIZE :
	{
		INT		iWidths[3];

		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		if ((HIWORD(lParam) == 0) || (LOWORD(lParam) == 0))
		{
			RECT	rc;

			GetClientRect (hwnd, &rc);
			lParam = MAKELPARAM (rc.right, rc.bottom);
		}

		if (ppks->bGroupsVisible) 
		{
			ppks->iGroupsHeight = HIWORD(lParam) * ppks->iGroupsPercent;
			ppks->iGroupsHeight /= 100;
			ppks->iKeysHeight = 
				HIWORD(lParam)-ppks->iToolHeight-s_uStatusBarHeight-
				ppks->iGroupsHeight-GRABBARHEIGHT;
			SetWindowPos (ppks->hwndTreeList, NULL, 
				0, 0, 
				LOWORD(lParam), 
				ppks->iKeysHeight,
				SWP_NOMOVE|SWP_NOZORDER);
			SetWindowPos (ppks->hwndTreeListGroups, NULL, 0,
				HIWORD(lParam)-ppks->iGroupsHeight-s_uStatusBarHeight,
				LOWORD(lParam),
				ppks->iGroupsHeight,
				SWP_NOZORDER);
		}
		else 
		{
			ppks->iKeysHeight = 
				HIWORD(lParam)-ppks->iToolHeight-s_uStatusBarHeight;

			SetWindowPos (ppks->hwndTreeList, NULL, 
				0, 0, 
				LOWORD(lParam), 
				ppks->iKeysHeight,
				SWP_NOMOVE|SWP_NOZORDER);
		}

		SendMessage (ppks->klConfig.hwndStatusBar, WM_SIZE, wParam, lParam);

		if ((ppks->uFlags & PK_MAINWINDOW) && (g_uNumTokens > 0))
			iWidths[0] = ICONWIDTH;
		else if (ppks->uFlags & PK_SEARCHWINDOW) 
			iWidths[0] = ICONWIDTH;
		else 
			iWidths[0] = 0;

		iWidths[1] = LOWORD(lParam)-TOTALLEDWIDTH-16;
		iWidths[2] = LOWORD(lParam)-16;
		SendMessage (ppks->klConfig.hwndStatusBar, SB_SETPARTS, 3, 
					(LPARAM)iWidths);

		if (ppks->hwndSearchControl) 
		{
			SendMessage (ppks->hwndSearchControl, WM_SIZE, wParam, 
				MAKELPARAM (LOWORD(lParam), ppks->iToolHeight));
			InvalidateRect (ppks->hwndSearchControl, NULL, FALSE);
		}

		if (ppks->hwndToolBar) 
		{
			INT		iToolHeight;

			iToolHeight = HIWORD(lParam) - s_uStatusBarHeight;
			if (iToolHeight > ppks->iToolHeight)
				iToolHeight = ppks->iToolHeight;

			SetWindowPos (ppks->hwndToolBar, NULL, 
				0, 0, 
				LOWORD(lParam), 
				iToolHeight - TOOLBARYOFFSET,
				SWP_NOMOVE|SWP_NOZORDER); 
		}

		return 0;
	}

	case WM_MOUSEMOVE :
	{
		RECT	rc;

		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		if (ppks->bGroupsVisible) 
		{
			INT		iY, iVSize, iYGrab;
			iY = HIWORD(lParam);
			if (ppks->bGrabbed) 
			{
				GetClientRect (hwnd, &rc);
				iVSize = rc.bottom-rc.top;
				if (iVSize <= 0) break;

				iY = (iVSize-s_uStatusBarHeight-GRABBARHEIGHT) -iY;
				if ((iY > 50) && (iY < (iVSize -50))) 
				{
					ppks->iGroupsPercent = (100 * iY) / iVSize;
					ppks->iGroupsHeight = iY;
				}

				ppks->iKeysHeight = 
					iVSize-ppks->iToolHeight-s_uStatusBarHeight-
					ppks->iGroupsHeight-GRABBARHEIGHT;

				SetWindowPos (ppks->hwndTreeList, NULL, 
					0, 0, 
					rc.right-rc.left, 
					ppks->iKeysHeight,
					SWP_NOMOVE|SWP_NOZORDER);
				SetWindowPos (ppks->hwndTreeListGroups, NULL, 0,
					iVSize-ppks->iGroupsHeight-s_uStatusBarHeight,
					rc.right-rc.left,
					ppks->iGroupsHeight,
					SWP_NOZORDER);
			}
			else 
			{
				iYGrab = ppks->iToolHeight+ppks->iKeysHeight;
				if ((iY >= iYGrab-2) && 
					(iY <= iYGrab+GRABBARHEIGHT+2)) 
				{
					ppks->bGrabEnabled = TRUE;
					ppks->hCursorOld = 
							SetCursor (LoadCursor (NULL, IDC_SIZENS));
					SetCapture (hwnd);
				}
				else 
				{
					if (ppks->bGrabEnabled) 
					{
						ppks->bGrabEnabled = FALSE;
						SetCursor (ppks->hCursorOld);
						ReleaseCapture ();
					}
				}
			}
		}
		break;
	}

	case WM_LBUTTONDOWN :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		if (ppks->bGrabEnabled) 
			ppks->bGrabbed = TRUE;
		break;

	case WM_LBUTTONUP :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		if (ppks->bGrabbed) 
			ppks->bGrabbed = FALSE;
		break;

	case WM_SYSCOLORCHANGE :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		PostMessage (ppks->hwndTreeList, WM_SYSCOLORCHANGE, 0, 0);
		break;

	case WM_SETTINGCHANGE :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		if (wParam == SPI_SETWHEELSCROLLLINES)
			sUpdateWheelScrollLines (ppks);
		else 
			PostMessage (ppks->hwndTreeList, WM_SYSCOLORCHANGE, 0, 0);
		break;

	case WM_DRAWITEM :
		if (wParam == IDC_STATUSBAR) 
		{
			ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
			sDrawStatus ((LPDRAWITEMSTRUCT)lParam, ppks);
			return TRUE;
		}
		break;

	case WM_COPYDATA :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		sImportData (ppks, (PCOPYDATASTRUCT)lParam);
		return TRUE;

	case WM_NOTIFY :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		{
		LPNMHDR pnmh = (LPNMHDR) lParam;

		// did it come from the keys treelist ?
		if (wParam == IDC_TREELISTKEYS) 
		{
			if (pnmh->code == TLN_SELCHANGED) 
				PGPgmPerformAction (ppks->hGM, GM_UNSELECTALL);

			PGPclKeyListDefaultNotificationProc (ppks->hKL, lParam);

			switch (pnmh->code) {
			case TLN_CONTEXTMENU :
				ppks->bGroupsFocused = FALSE;
				ppks->uKeySelectionFlags = ((LPNM_TREELIST)lParam)->flags;
				PKContextMenu (ppks, hwnd, 
						((LPNM_TREELIST)lParam)->ptDrag.x,
						((LPNM_TREELIST)lParam)->ptDrag.y);
				break;

			case TLN_SELCHANGED :
				ppks->bGroupsFocused = FALSE;
				ppks->uKeySelectionFlags = ((LPNM_TREELIST)lParam)->flags;
				PKSetToolbarButtonStates (ppks);
				break;

			case TLN_LISTITEMCLICKED : 
				if (PKIsActionEnabled (ppks, KM_CERTIFY))
					PKPerformAction (ppks, KM_CERTIFY);
				break;

			case TLN_ITEMDBLCLICKED :
				if (PKIsActionEnabled (ppks, KM_PROPERTIES))
					PKPerformAction (ppks, KM_PROPERTIES);
				break;

			case TLN_KEYDOWN :
				switch (((TL_KEYDOWN*)lParam)->wVKey) {
				case VK_TAB :
					if (ppks->bGroupsVisible) 
					{
						SendMessage (ppks->klConfig.hwndStatusBar, 
								SB_SETTEXT, STATUS_MSG_PANE, (LPARAM)"");
						ppks->bGroupsFocused = !ppks->bGroupsFocused;
						if (ppks->bGroupsFocused) 
							SetFocus (ppks->hwndTreeListGroups);
						else 
							SetFocus (ppks->hwndTreeList);
						PKSetToolbarButtonStates (ppks);
					}
					break;

				case VK_DELETE :
					SendMessage (hwnd, WM_COMMAND, IDM_DELETEKEY, 0);
					break;
				}
				break;

			case TLN_BEGINDRAG :
				PKEnableDropTarget (ppks->pDropTarget, FALSE);
				PKDragAndDrop (ppks);
				PKEnableDropTarget (ppks->pDropTarget, 
						!ppks->bReadOnly && (ppks->uFlags & PK_KEYWINDOW));
				break;

			case TLN_HELP :
				PostMessage (hwnd, WM_COMMAND, IDM_HELPTOPICS, 0);
				break;

			default :
				break;
			}
		}

		// did it come from the group treelist ?
		else if (wParam == IDC_TREELISTGROUP) 
		{
			if (pnmh->code == TLN_SELCHANGED) 
				PKPerformAction (ppks, KM_UNSELECTALL);

			PGPgmDefaultNotificationProc (ppks->hGM, lParam);

			switch (pnmh->code) {
			case TLN_CONTEXTMENU :
				ppks->bGroupsFocused = TRUE;
				ppks->uGroupSelectionFlags = ((LPNM_TREELIST)lParam)->flags;
				PKContextMenu (ppks, hwnd, 
						((LPNM_TREELIST)lParam)->ptDrag.x,
						((LPNM_TREELIST)lParam)->ptDrag.y);
				break;

			case TLN_SELCHANGED :
				ppks->bGroupsFocused = TRUE;
				ppks->uGroupSelectionFlags = ((LPNM_TREELIST)lParam)->flags;
				PKSetToolbarButtonStates (ppks);
				break;

			case TLN_KEYDOWN :
				switch (((TL_KEYDOWN*)lParam)->wVKey) {
				case VK_TAB :
					SendMessage (ppks->klConfig.hwndStatusBar, SB_SETTEXT, 
									STATUS_MSG_PANE, (LPARAM)"");
					ppks->bGroupsFocused = !ppks->bGroupsFocused;
					if (ppks->bGroupsFocused) 
						SetFocus (ppks->hwndTreeListGroups);
					else 
						SetFocus (ppks->hwndTreeList);
					PKSetToolbarButtonStates (ppks);
					break;
				}
				break;

			case TLN_HELP :
				PostMessage (hwnd, WM_COMMAND, IDM_HELPTOPICS, 0);
				break;

			default :
				break;
			}
		}

		// did it come from the search window keyserver code ?
		else if ((ppks->hwndSearchControl) &&
				 (pnmh->hwndFrom == ppks->hwndSearchControl)) 
		{
			switch (pnmh->code)
			{
				case SEARCH_REQUEST_SIZING: 
				{
					PSIZEREQUEST pRequest = (PSIZEREQUEST)lParam;
					RECT rc;
					GetClientRect (ppks->hwndTreeList, &rc);
					if ((rc.bottom - rc.top) > (pRequest->delta + 50))
						return TRUE;
					else
						return FALSE;
				}

				case SEARCH_SIZING: 
				{
					PSIZEREQUEST pRequest = (PSIZEREQUEST)lParam;
					RECT rcTool, rcMain;
					GetWindowRect (pnmh->hwndFrom, &rcTool);
					ppks->iToolHeight = rcTool.bottom-rcTool.top;

					GetClientRect (hwnd, &rcMain);

					SetWindowPos (ppks->hwndTreeList, NULL, 
						0, ppks->iToolHeight, 
						rcMain.right-rcMain.left, 
						rcMain.bottom-rcMain.top
							-ppks->iToolHeight
							-s_uStatusBarHeight, 
						SWP_NOZORDER);
					InvalidateRect (hwnd, NULL, FALSE);
					break;
				}

				case SEARCH_DISPLAY_KEYSET: 
				{
					PSEARCHRESULT	pResult = (PSEARCHRESULT)lParam;
					PGPKeySetRef	KeySetPrevious;
					HCURSOR			hCursorOld;
					CHAR			sz[64];

					// stop LEDs, if going
					if (ppks->bSearchInProgress) 
					{
						KillTimer (hwnd, LEDTIMER);
						ppks->bSearchInProgress = FALSE;
						ppks->iStatusValue = -1;
						InvalidateRect (ppks->klConfig.hwndStatusBar, 
											NULL, FALSE);
						UpdateWindow (ppks->klConfig.hwndStatusBar);
					}

					// if error, post message and drop out
					if (IsPGPError (pResult->error) && 
						(pResult->error != 
								kPGPError_ServerPartialSearchResults)) 
					{
						KillTimer (hwnd, LEDTIMER);
						ppks->bSearchInProgress = FALSE;
						ppks->iStatusValue = -1;
						SendMessage (ppks->klConfig.hwndStatusBar, 
								SB_SETTEXT, STATUS_MSG_PANE, (LPARAM)"");
						InvalidateRect (ppks->klConfig.hwndStatusBar, 
										NULL, FALSE);
						PGPclErrorBox (NULL, pResult->error);
						break;
					}

					// load new keyset into window
					LoadString (g_hinst, IDS_SYNCINGKEYSETS, sz, sizeof(sz));
					SendMessage (ppks->klConfig.hwndStatusBar, SB_SETTEXT, 
									STATUS_MSG_PANE, (LPARAM)sz);
					UpdateWindow (ppks->klConfig.hwndStatusBar);

					hCursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));

					KeySetPrevious = ppks->keysetDisp;
					ppks->keysetDisp = (PGPKeySetRef)(pResult->pData);

					if (!(pResult->flags & FLAG_SEARCH_CURRENT))
					{
						if (pResult->flags & FLAG_SEARCH_LOCAL_KEYSET) 
						{
							if (!ppks->bLocalKeySet)
								PKDeleteAllObjectPropertiesWindows (ppks);
							ppks->bLocalKeySet = TRUE;
						}
						else 
						{
							PKDeleteAllObjectPropertiesWindows (ppks);
							ppks->bLocalKeySet = FALSE;
						}
					}

					if (ppks->bLocalKeySet)
					{
						ppks->klConfig.keyserver.protocol = 
											kPGPKeyServerProtocol_Invalid;
						ppks->klConfig.uOptions =
							kPGPclKeyList_EnableReloads |
							kPGPclKeyList_EnableDragOut;
						if (!g_bExpertMode)
							ppks->klConfig.uOptions |= 
									kPGPclKeyList_GreenDotMode;
						if (g_bMarginalAsInvalid) 
							ppks->klConfig.uOptions |= 
									kPGPclKeyList_MarginalIsInvalid;
						if (ppks->bReadOnly)
							ppks->klConfig.uOptions |= 
									kPGPclKeyList_ReadOnly;
					}
					else 
					{
						memcpy (&ppks->klConfig.keyserver, 
								&pResult->keyserver, 
								sizeof(PGPKeyServerEntry));
						ppks->klConfig.uOptions = 
							kPGPclKeyList_EnableDragOut;
						if (!g_bExpertMode)
							ppks->klConfig.uOptions |= 
									kPGPclKeyList_GreenDotMode;
						if (g_bMarginalAsInvalid) 
							ppks->klConfig.uOptions |= 
									kPGPclKeyList_MarginalIsInvalid;
						if (pResult->flags & FLAG_AREA_PENDING) 
							ppks->klConfig.uOptions |= 
									kPGPclKeyList_PendingBucket;
					}
					ppks->klConfig.uMask = kPGPclKeyList_Options |
											kPGPclKeyList_KeyServer;
					PGPclKeyListConfigure (ppks->hKL, &(ppks->klConfig));

					// display keyset
					PGPclKeyListLoadKeys (ppks->hKL, 
							ppks->keydbMain, ppks->keysetDisp);
					SetFocus (ppks->hwndTreeList);

					// sync downloaded keyset with main and redisplay
					if (PGPKeySetRefIsValid (ppks->keysetDisp))
					{
						PGPCheckKeyRingSigs (ppks->keysetDisp, 
								ppks->keydbMain, TRUE, NULL, NULL);
						PGPclKeyListLoadKeys (ppks->hKL, 
								ppks->keydbMain, ppks->keysetDisp);
					}

					sDisplayKSKeyResult (ppks->keysetDisp, 
										ppks->klConfig.hwndStatusBar,
										pResult->error);

					// stop LEDs, if going
					if (ppks->bSearchInProgress) 
					{
						KillTimer (hwnd, LEDTIMER);
						ppks->bSearchInProgress = FALSE;
						ppks->iStatusValue = -1;
						InvalidateRect (ppks->klConfig.hwndStatusBar, 
											NULL, FALSE);
					}

					if (KeySetPrevious) 
					{
						PGPKeyDBRef		keydbPrevious;

						keydbPrevious = PGPPeekKeySetKeyDB (KeySetPrevious);
						PGPFreeKeySet (KeySetPrevious);
						if (!(pResult->flags & FLAG_SEARCH_CURRENT) &&
							(PGPKeyDBRefIsValid (keydbPrevious)) &&
							(keydbPrevious != ppks->keydbMain))
						{
							PGPFreeKeyDB (keydbPrevious);
						}
					}
					
					SetCursor (hCursorOld);

					SendMessage (ppks->hwndSearchControl, 
											SEARCH_SET_FOCUS, 0, 0);
					break;
				}

				case SEARCH_PROGRESS :
				{
					PSEARCHPROGRESS pProgress = (PSEARCHPROGRESS)lParam;

					if (pProgress->total) 
					{
						// we're starting a query
						if (!ppks->bSearchInProgress) 
						{
							ppks->bSearchInProgress = TRUE;
							ppks->klConfig.uOptions |= 
													kPGPclKeyList_DisableStatusBar;
							ppks->klConfig.uMask = kPGPclKeyList_Options;
							PGPclKeyListConfigure (ppks->hKL, &(ppks->klConfig));

							ppks->iStatusValue = 0;
							if (pProgress->step == SEARCH_PROGRESS_INFINITE) 
							{
								ppks->iStatusDirection = 1;
								SetTimer (hwnd, LEDTIMER, 
													LEDTIMERPERIOD, NULL);
							}
							else 
							{
								ppks->iStatusDirection = 0;
								ppks->iStatusValue = 0;
							}
							// reset icon to "unlocked"
							SendMessage (ppks->klConfig.hwndStatusBar, 
									SB_SETICON, ICON_PANE, 
									(LPARAM)g_hiconOpenLock);
							// signal that we're not authenticated yet
							ppks->iTLSstatus = TLS_NOTAUTHENTICATED;
						}
						else 
						{
							if (pProgress->step != SEARCH_PROGRESS_INFINITE) 
							{
								ppks->iStatusDirection = 0;
								ppks->iStatusValue = 
									(pProgress->step * 9) / pProgress->total;
								InvalidateRect (ppks->klConfig.hwndStatusBar, 
													NULL, FALSE);
							}
						}
					}
					else 
					{
						if (ppks->bSearchInProgress) 
						{
							KillTimer (hwnd, LEDTIMER);
							ppks->bSearchInProgress = FALSE;
							ppks->iStatusValue = -1;
							InvalidateRect (ppks->klConfig.hwndStatusBar, 
											NULL, FALSE);
						}
					}

					SendMessage (ppks->klConfig.hwndStatusBar, SB_SETTEXT, 
							STATUS_MSG_PANE, (LPARAM)(pProgress->message));
					break;
				}

				case SEARCH_SECURE_STATUS :
				{
					PSEARCHSECURE	pSecure = (PSEARCHSECURE)lParam;
					
					if (pSecure->secure) 
					{
						SendMessage (ppks->klConfig.hwndStatusBar, 
								SB_SETICON, ICON_PANE, 
								(LPARAM)g_hiconClosedLock);

						ppks->iTLSstatus = TLS_AUTHENTICATED;

						lstrcpy (ppks->szTLSserver, pSecure->szServerName);
						ppks->tlsCipher = pSecure->tlsCipher;

						if (PGPKeyDBRefIsValid (ppks->keydbAuth))
							PGPFreeKeyDB (ppks->keydbAuth);
						PGPNewKeyDB (g_context, &ppks->keydbAuth);
						PGPCopyKeyDBObj (pSecure->keyAuth, 
								ppks->keydbAuth, &ppks->keyAuth);
					}
					else 
					{
						SendMessage (ppks->klConfig.hwndStatusBar, 
								SB_SETICON, ICON_PANE, 
								(LPARAM)g_hiconOpenLock);
						ppks->iTLSstatus = TLS_NOTAUTHENTICATED;
					}
	
					break;
				}

				case SEARCH_ABORT :
				{
					PSEARCHABORT pAbort = (PSEARCHABORT)lParam;
					CHAR sz[256];
					if (ppks->bSearchInProgress) 
					{
						KillTimer (hwnd, LEDTIMER);
						ppks->bSearchInProgress = FALSE;
						ppks->iStatusValue = -1;
						InvalidateRect (ppks->klConfig.hwndStatusBar, 
											NULL, FALSE);
					}
					LoadString (g_hinst, IDS_SEARCHABORTED, 
									sz, sizeof(sz));
					SendMessage (ppks->klConfig.hwndStatusBar, SB_SETTEXT,
									STATUS_MSG_PANE, (LPARAM)sz);
					SendMessage (ppks->hwndSearchControl, 
											SEARCH_SET_FOCUS, 0, 0);
					break;
				}

				default :
					break;
			}

			if (!ppks->bSearchInProgress) 
			{
				ppks->klConfig.uOptions &= ~kPGPclKeyList_DisableStatusBar;
				ppks->klConfig.uMask = kPGPclKeyList_Options;
				PGPclKeyListConfigure (ppks->hKL, &(ppks->klConfig));
			}
		}

		// did it come from the status bar?
		else if ((ppks->klConfig.hwndStatusBar) &&
				 (pnmh->hwndFrom == ppks->klConfig.hwndStatusBar)) 
		{
			if (pnmh->code == NM_CLICK) 
			{
				POINT pt;
				GetCursorPos (&pt);
				ScreenToClient (ppks->klConfig.hwndStatusBar, &pt);
				if (pt.x <= ICONWIDTH) 
				{
					if ((ppks->uFlags & PK_MAINWINDOW) &&
						(g_uNumTokens > 0))
					{
						if (PKTokenProperties (ppks))
							PostMessage (hwnd, PK_M_RELOADKEYRINGS, 0, 0);
					}
					else if (ppks->uFlags & PK_SEARCHWINDOW) 
					{
						switch (ppks->iTLSstatus) {
						case TLS_NOSEARCHYET :
							PKMessageBox (hwnd, IDS_CAPTIONINFO, 
									IDS_NOCONNECTIONYET, 
									MB_OK|MB_ICONINFORMATION);
							break;

						case TLS_NOTAUTHENTICATED :
							PKMessageBox (hwnd, IDS_CAPTIONINFO, 
									IDS_CONNECTIONNOTAUTHENTICATED, 
									MB_OK|MB_ICONINFORMATION);
							break;

						case TLS_AUTHENTICATED :
							PGPclConfirmRemoteAuthentication (
									g_context, hwnd, ppks->szTLSserver,
									ppks->keyAuth, ppks->tlsCipher,
									ppks->keydbMain, 
									kPGPclAuthShowAuthentication);
							break;
						}
					}
				}
			}
		}

		// else assume it's from the tooltip control of the toolbar
		else if (pnmh->code == TTN_NEEDTEXT) 
			PKGetToolbarTooltipText ((LPTOOLTIPTEXT)lParam);
		}
		break;

	case WM_PAINT :
		{
			HDC			hdc;
			PAINTSTRUCT ps;
			RECT		rc;
			HPEN		hpen;
			HPEN		hpenOld;
			INT			iWidth;

			ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

			hdc = BeginPaint (hwnd, &ps);

			if (ppks->hwndToolBar && (ppks->iToolHeight > 0)) 
			{
				GetWindowRect (hwnd, &rc);
				iWidth = rc.right-rc.left;

				hpen = CreatePen (PS_SOLID, 0, GetSysColor (COLOR_3DSHADOW));
				hpenOld = SelectObject (hdc, hpen);
				MoveToEx (hdc, 0, 0, NULL);
				LineTo (hdc, iWidth, 0);

				hpen = CreatePen (PS_SOLID, 0, GetSysColor (COLOR_3DHILIGHT));
				DeleteObject (SelectObject (hdc, hpen));
				MoveToEx (hdc, 0, 1, NULL);
				LineTo (hdc, iWidth, 1);

				DeleteObject (SelectObject (hdc, hpenOld));
			}

			EndPaint (hwnd, &ps);
		}
		return 0;

	case WM_TIMER :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		// it's time to load the keyring from disk
		if (wParam == RELOADTIMER) 
		{
			KillTimer (hwnd, wParam);

			// do the actual loading
			if (sReloadKeyrings (ppks))
			{
				// repaint window
				InvalidateRect (hwnd, NULL, FALSE);
				PKSetToolbarButtonStates (ppks);

				// store the file names for backup purposes
				PKKeyDBOpened (ppks);

				// do some first-time post-processing
				if (s_bFirstKeyringLoad) 
				{
					s_bFirstKeyringLoad = FALSE;

					// import key files if passed from the command line
					sProcessFileList (GetCommandLine (), ppks, TRUE, TRUE);

					// display groups if groups were displayed last session
					if (g_bShowGroups) 
					{
						PostMessage (hwnd, WM_COMMAND,
								MAKEWPARAM (IDM_VIEWGROUPS, 0), 0);
					}

					// compute when we should next update CRLs
					PGPclScheduleNextCRLUpdate (g_context, ppks->keydbMain);

					SetFocus (ppks->hwndTreeList);
				}

				// if there is an open search window, notify it of change
				if ((ppks->uFlags & PK_KEYWINDOW) && s_hwndOpenSearch)
				{
					PostMessage (s_hwndOpenSearch, 
							PK_M_RELOADKEYRINGS, 0, (LPARAM)(ppks->keydbMain));
				}

				// notify the token dialog of change
				PKReloadTokenProperties (TRUE);
			}
		}
		else if (wParam == LEDTIMER)
		{
			ppks->iStatusValue += ppks->iStatusDirection;
			if (ppks->iStatusValue <= 0) 
			{
				ppks->iStatusValue = 0;
				ppks->iStatusDirection = 1;
			}
			else if (ppks->iStatusValue >= NUMLEDS-1) 
			{
				ppks->iStatusValue = NUMLEDS-1;
				ppks->iStatusDirection = -1;
			}
			InvalidateRect (ppks->klConfig.hwndStatusBar, NULL, FALSE);
		}
		break;

	case PK_M_REFRESHKEYRINGS :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		if ((HWND)wParam != hwnd)
		{
			if (lParam)
			{
				if (ppks->uFlags & PK_MAINWINDOW)
				{
					PGPclKeyListLoadKeys (ppks->hKL, NULL, NULL);
					PGPclKeyListLoadKeys (
						ppks->hKL, ppks->keydbMain, 
						PGPPeekKeyDBRootKeySet (ppks->keydbMain));
				}
				else if (ppks->uFlags & PK_SEARCHWINDOW)
				{
					if (ppks->bLocalKeySet)
					{
						PGPclKeyListLoadKeys (ppks->hKL, NULL, NULL);
						PGPclKeyListLoadKeys (
							ppks->hKL, ppks->keydbMain, ppks->keysetDisp);
					}
				}
			}
			else
				PGPclKeyListReloadKeys (ppks->hKL, FALSE);
		}

		if ((ppks->uFlags & PK_MAINWINDOW) &&
			(ppks->bGroupsVisible))
		{
			PGPgmReLoadGroups (ppks->hGM);
		}

		if (ppks->uFlags & PK_PDAWINDOW)
		{
			PGPKeyDBRef		keydbDefault;

			if (IsntPGPError (PGPclOpenDefaultKeyrings (
					g_context, 0, &keydbDefault)))
			{
				HCURSOR		hCursorOld;

				hCursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));

				PGPCheckKeyRingSigs (
						PGPPeekKeyDBRootKeySet (ppks->keydbMain), 
						keydbDefault, FALSE, NULL, NULL);
				PGPclKeyListReloadKeys (ppks->hKL, FALSE);
				PGPFreeKeyDB (keydbDefault);

				SetCursor (hCursorOld);
			}
		}

		break;

	case PK_M_RELOADKEYRINGS : 
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		PKEnableDropTarget (ppks->pDropTarget, FALSE);
		PGPclKeyListLoadKeys (ppks->hKL, NULL, NULL);
		if (ppks->uFlags & PK_MAINWINDOW) 
		{
			if (s_hwndOpenSearch != NULL)
			{
				SendMessage (s_hwndOpenSearch, PK_M_RELOADKEYRINGS, 0, 0);
			}

			if (ppks->keydbMain) 
			{
				ppks->gmConfig.keydbMain = kInvalidPGPKeyDBRef;
				PGPgmConfigure (ppks->hGM, &(ppks->gmConfig));
				PGPFreeKeyDB (ppks->keydbMain);
				ppks->keydbMain = kInvalidPGPKeyDBRef;
			}
			SetTimer (hwnd, RELOADTIMER, RELOADDELAY, NULL);
		}
		else if (ppks->uFlags & PK_SEARCHWINDOW)
		{
			if (ppks->bLocalKeySet &&
				PGPKeySetRefIsValid (ppks->keysetDisp))
			{
				PGPFreeKeySet (ppks->keysetDisp);
				ppks->keysetDisp = NULL;
			}

			if (lParam)
				ppks->keydbMain = (PGPKeyDBRef)lParam;

			SendMessage (ppks->hwndSearchControl, SEARCH_SET_MAIN_KEYDB,
						0, (LPARAM)(ppks->keydbMain));
			SendMessage (ppks->hwndSearchControl, SEARCH_SET_LOCAL_KEYSET,
					0, (LPARAM)(PGPPeekKeyDBRootKeySet (ppks->keydbMain)));

			PGPclKeyListLoadKeys (
					ppks->hKL, ppks->keydbMain, ppks->keysetDisp);
		}
		break;

	case PK_M_KEYCREATEDONE :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		ShowWindow (hwnd, SW_SHOW);
		EnableWindow (hwnd, TRUE);
		SetForegroundWindow (hwnd);

		if (ppks->bGroupsFocused) 
			SetFocus (ppks->hwndTreeListGroups);
		else 
			SetFocus (ppks->hwndTreeList);

		if (wParam & kPGPclKeyCreate_NewKey) 
		{
			PGPPrefRef prefref	= kInvalidPGPPrefRef;

			if (IsntPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
			{
				PGPSetPrefBoolean (
						prefref, kPGPPrefFirstKeyGenerated, TRUE);
				PGPclFlushClientLibPrefs (prefref, NULL);

				PKKeyDBModified (ppks, PK_MOD_INDEX_0);
				if ((ppks->uFlags & PK_MAINWINDOW) &&
					!(wParam & kPGPclKeyCreate_OnToken))
				{
					s_bKeyHasBeenGenerated = TRUE;
				}

				PGPclKeyListReloadKeys (ppks->hKL, TRUE);
				if (wParam & kPGPclKeyCreate_NewDefaultKey) 
					PKPerformAction (ppks, KM_SETASDEFAULT);
				PGPclKeyListSetTree (ppks->hKL, 
						kPGPclSelectSpecifiedOnly, (PGPKeyDBObjRef)lParam);
			}
		}
		InvalidateRect (hwnd, NULL, FALSE);
		return TRUE;

	case PGPCL_M_PROPERTYMESSAGE :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		switch (wParam) {
		case kPGPclProperties_UpdateObject :
			PKKeyDBModified (ppks, PK_MOD_INDEX_0);
			PGPclKeyListUpdateTree (ppks->hKL, 
					kPGPclKeyListUpdateObjectFocus, 
					(PGPKeyDBObjRef)lParam, FALSE); 
			PGPclKeyListUpdateTree (ppks->hKL, 
					kPGPclKeyListUpdateValidities, NULL, TRUE);
			InvalidateRect (hwnd, NULL, FALSE);
			break;

		case kPGPclProperties_CloseWindow :
			PKDeletePropertiesWindowForObject ((PGPKeyDBObjRef)lParam);
			break;

		case kPGPclProperties_ShowKeyProps :
			PKSingleObjectProperties (ppks, 
					(PGPKeyDBObjRef)lParam, kPGPclKeyList_Key);
			break;

		case kPGPclProperties_ShowCertProps :
			PKSingleObjectProperties (ppks, 
					(PGPKeyDBObjRef)lParam, kPGPclKeyList_Signature);
			break;

		case kPGPclProperties_EnableWindows :
			PKEnableWindowList ((BOOL)lParam);
			PKEnableAllObjectPropertiesWindows (ppks, (BOOL)lParam);
			break;

		case kPGPclProperties_UpdateKeyring :
			PKKeyDBModified (ppks, PK_MOD_INDEX_0);
			PGPclKeyListReloadKeys (ppks->hKL, TRUE);
			break;
		}
		break;

	case PGPCL_M_REGISTERPROPERTYWINDOW :
		PKRegisterPropertiesWindow ((HWND)wParam, (HWND)lParam);
		break;

	case PK_M_RESIZE :
	{
		RECT	rc;

		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		GetClientRect (hwnd, &rc);
		if (ppks->hwndToolBar) 
		{
			SetWindowPos (ppks->hwndToolBar, NULL, 
					0, 0, 
					(rc.right-rc.left), 
					ppks->iToolHeight, 
					SWP_NOMOVE|SWP_NOZORDER);
		}
		if (ppks->hwndTreeList) 
		{
			SetWindowPos (ppks->hwndTreeList, NULL, 
					0, ppks->iToolHeight, 
					(rc.right-rc.left), 
					ppks->iKeysHeight, 
					SWP_NOZORDER);
		}
		if (ppks->hwndTreeListGroups) 
		{
			SetWindowPos (ppks->hwndTreeListGroups, NULL, 
					0, 
					(rc.bottom-rc.top)-ppks->iGroupsHeight-s_uStatusBarHeight,
					(rc.right-rc.left),
					ppks->iGroupsHeight,
					SWP_NOZORDER);
		}
		UpdateWindow (hwnd);
		break;
	}

	case PK_M_COPYFILENAMES :
	{
		PGPKEYSSTRUCT*	ppksFile	= (PGPKEYSSTRUCT*)lParam;

		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		lstrcpy (ppks->szFilePub, ppksFile->szFilePub);
		lstrcpy (ppks->szFileSec, ppksFile->szFileSec);
		break;
	}

	case PK_M_NEWKEYRINGS :
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		if (ppks->uFlags & PK_SEARCHWINDOW)
		{
			if (((DWORD)lParam != GetCurrentProcessId ()) &&
				(wParam == kPGPclNewDefaultKeyrings) &&
				(ppks->bLocalKeySet))
			{
				PKDeleteAllObjectPropertiesWindows (ppks);

				PGPclKeyListLoadKeys (ppks->hKL, 
						ppks->keydbMain, NULL);

				if (ppks->keysetDisp)
				{
					PGPFreeKeySet (ppks->keysetDisp);
					ppks->keysetDisp = NULL;
				}

				SendMessage (ppks->hwndSearchControl, 
						SEARCH_SET_CURRENT_SEARCH, 0, 0);
			}
		}
		break;

	case PGPPDA_M_NOTIFY_PALM :
	{
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		if (g_pPDA)
		{
			PGPPDASTRUCT*	pPDA	= g_pPDA;

			if (ppks->uFlags & PK_PDAWINDOW)
			{
				return (sProcessPdaNotification (
						hwnd, ppks, wParam, lParam));
			}

			while (pPDA->hmod)
			{
				if (pPDA->bKeyring && pPDA->hwnd)
				{
					if (pPDA->uPlatform == kPGPpda_PlatformPalmOS)
						return SendMessage (pPDA->hwnd, uMsg, wParam, lParam);
				}
				pPDA++;
			}

			if (s_bInPDAOp)
			{
				return (sProcessPdaNotification (
						NULL, ppks, wParam, lParam));
			}
		}
		break;
	}

	case WM_COMMAND:
		ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		switch (LOWORD(wParam)) {
		case IDM_FILEEXIT :
		case IDM_CLOSE :
			PostMessage (hwnd, WM_CLOSE, 0, 0);
			break;

		case IDM_SENDSHARES :
			PGPclSendShares (g_context, g_tlscontext, hwnd, ppks->keydbMain);
			break;

		case IDM_HELPABOUT :
			PKHelpAbout (hwnd);
			break;

		case IDM_HELPTOPICS :
			PGPclHtmlHelp (hwnd, kPGPclHelpContentsAndIndex, 0, 0, 
					(char*)kPGPclHelpFile, NULL);
			break;

		case IDM_PREFERENCES :
			s_bIgnorePrefsReloads = TRUE;
			s_bPrefsReloadIgnored = FALSE;
			PKPGPPreferences (ppks, hwnd, kPGPclGeneralPrefs);
			if (s_bPrefsReloadIgnored)
				PKReloadPGPPreferences (ppks);
			s_bIgnorePrefsReloads = FALSE;
			break;

		case IDM_CERTIFYKEY :
			PKPerformAction (ppks, KM_CERTIFY); 
			break;

		case IDM_DISABLEKEY :
			if ((ppks->uFlags & PK_SEARCHWINDOW) && 
				(!ppks->bLocalKeySet) &&
				(ppks->uKeySelectionFlags == kPGPclKeyList_Key))
			{
				PKPerformAction (ppks, KM_DISABLEONSERVER); 
			}
			else 
				PKPerformAction (ppks, KM_DISABLE); 
			break;

		case IDM_ENABLEKEY :
			PKPerformAction (ppks, KM_ENABLE); 
			break;

		case IDM_DELETEKEY :
			if (ppks->bGroupsFocused) 
				PGPgmPerformAction (ppks->hGM, GM_DELETE); 
			else
				PKPerformAction (ppks, KM_DELETE); 
			break;

		case IDM_DELETESERVER :
			PKPerformAction (ppks, KM_DELETEFROMSERVER); 
			break;

		case IDM_CREATEKEY :
			if (g_bKeyGenEnabled)
				PKCreateKey (hwnd, ppks->keydbMain);
			break;

		case IDM_SETASDEFAULT :
			if (PKIsActionEnabled (ppks, KM_SETASPRIMARY))
				PKPerformAction (ppks, KM_SETASPRIMARY); 
			else
				PKPerformAction (ppks, KM_SETASDEFAULT); 
			break;

		case IDM_PROPERTIES :
			if (ppks->bGroupsFocused) 
			{
				PGPgmPerformAction (ppks->hGM, GM_LOCATEKEYS);
				PKPerformAction (ppks, KM_PROPERTIES); 
			}
			else 
				PKPerformAction (ppks, KM_PROPERTIES); 
			break;
				
		case IDM_GROUPPROPERTIES :
			PGPgmPerformAction (ppks->hGM, GM_PROPERTIES);
			break;
		
		case IDM_IMPORTKEYS :
			PKPerformAction (ppks, KM_IMPORT); 
			break;

		case IDM_IMPORTGROUPS :
			PGPgmPerformAction (ppks->hGM, GM_IMPORTGROUPS); 
			break;
		
		case IDM_EXPORTKEYS :
			PKPerformAction (ppks, KM_EXPORT); 
			break;

		case IDM_GETFROMSERVER :
			if (ppks->bGroupsFocused)
				PGPgmPerformAction (ppks->hGM, GM_GETFROMSERVER);
			else
				PKPerformAction (ppks, KM_GETFROMSERVER);
			break;

		case IDM_RETRIEVECERTIFICATE :
			PKPerformAction (ppks, KM_RETRIEVECERTIFICATE);
			break;

		case IDM_ADDTOMAIN :
			PKPerformAction (ppks, KM_ADDTOMAIN);
			break;

		case IDM_EXPANDSEL :
			if (ppks->bGroupsFocused) 
			{
				if (PGPgmIsActionEnabled (ppks->hGM, GM_EXPANDSEL))
					PGPgmPerformAction (ppks->hGM, GM_EXPANDSEL); 
				else
					PGPgmPerformAction (ppks->hGM, GM_EXPANDALL); 
			}
			else 
			{
				if (PKIsActionEnabled (ppks, KM_EXPANDSEL))
					PKPerformAction (ppks, KM_EXPANDSEL); 
				else
					PKPerformAction (ppks, KM_EXPANDALL); 
			}
			break;

		case IDM_COLLAPSESEL :
			if (ppks->bGroupsFocused) 
			{
				if (PGPgmIsActionEnabled (ppks->hGM, GM_COLLAPSESEL))
					PGPgmPerformAction (ppks->hGM, GM_COLLAPSESEL); 
				else
					PGPgmPerformAction (ppks->hGM, GM_COLLAPSEALL); 
			}
			else 
			{
				if (PKIsActionEnabled (ppks, KM_COLLAPSESEL))
					PKPerformAction (ppks, KM_COLLAPSESEL); 
				else
					PKPerformAction (ppks, KM_COLLAPSEALL); 
			}
			break;

		case IDM_SELECTALL :
			if (ppks->bGroupsFocused) 
				PGPgmPerformAction (ppks->hGM, GM_SELECTALL); 
			else
				PKPerformAction (ppks, KM_SELECTALL); 
			PKSetToolbarButtonStates (ppks);
			break;

		case IDM_REVOKEKEY :
			PKPerformAction (ppks, KM_REVOKE); 
			break;

		case IDM_COPYKEY :
			PKPerformAction (ppks, KM_COPY); 
			break;

		case IDM_PASTEKEY :
			if (ppks->bGroupsFocused) 
				PGPgmPerformAction (ppks->hGM, GM_PASTE); 
			else 
				PKPerformAction (ppks, KM_PASTE); 
			break;

		case IDM_ADDUSERID :
			PKPerformAction (ppks, KM_ADDUSERID); 
			break;

		case IDM_ADDPHOTOID :
			PKPerformAction (ppks, KM_ADDPHOTOID); 
			break;

		case IDM_ADDREVOKER :
			PKPerformAction (ppks, KM_ADDREVOKER); 
			break;

		case IDM_ADDCERTIFICATE :
			PKPerformAction (ppks, KM_ADDCERTIFICATE); 
			break;

		case IDM_SPLITKEY :
			PKPerformAction (ppks, KM_SPLITKEY); 
			break;

		case IDM_RECONSTRUCTKEY :
			PKPerformAction (ppks, KM_RECONSTRUCTKEY); 
			break;

		case IDM_REVERIFY :
			PKPerformAction (ppks, KM_REVERIFYSIGS);
			break;

		case IDM_VIEWVALIDITY :
			s_ulColumns ^= kPGPclKeyList_ValidityColumn;
			PGPclKeyListSelectColumns (ppks->hKL, s_ulColumns, TRUE);
			break;

		case IDM_VIEWSIZE :
			s_ulColumns ^= kPGPclKeyList_SizeColumn;
			PGPclKeyListSelectColumns (ppks->hKL, s_ulColumns, TRUE);
			break;

		case IDM_VIEWDESC :
			s_ulColumns ^= kPGPclKeyList_DescriptionColumn;
			PGPclKeyListSelectColumns (ppks->hKL, s_ulColumns, TRUE);
			break;

		case IDM_VIEWKEYID :
			s_ulColumns ^= kPGPclKeyList_KeyIDColumn;
			PGPclKeyListSelectColumns (ppks->hKL, s_ulColumns, TRUE);
			break;

		case IDM_VIEWTRUST :
			s_ulColumns ^= kPGPclKeyList_TrustColumn;
			PGPclKeyListSelectColumns (ppks->hKL, s_ulColumns, TRUE);
			break;

		case IDM_VIEWCREATION :
			s_ulColumns ^= kPGPclKeyList_CreationColumn;
			PGPclKeyListSelectColumns (ppks->hKL, s_ulColumns, TRUE);
			break;

		case IDM_VIEWEXPIRATION :
			s_ulColumns ^= kPGPclKeyList_ExpirationColumn;
			PGPclKeyListSelectColumns (ppks->hKL, s_ulColumns, TRUE);
			break;

		case IDM_VIEWADK :
			s_ulColumns ^= kPGPclKeyList_ADKColumn;
			PGPclKeyListSelectColumns (ppks->hKL, s_ulColumns, TRUE);
			break;

		case IDM_VIEWTOOLBAR :
		{
			RECT	rc;

			if (ppks->iToolHeight == 0) 
				ppks->iToolHeight = DEFAULTTOOLHEIGHT;
			else 
				ppks->iToolHeight = 0;

			GetClientRect (hwnd, &rc);
			ppks->iKeysHeight = 
					(rc.bottom-rc.top)-ppks->iToolHeight-s_uStatusBarHeight;
			if (ppks->iGroupsHeight)
				ppks->iKeysHeight -= (ppks->iGroupsHeight + GRABBARHEIGHT);
			PostMessage (hwnd, PK_M_RESIZE, 0, 0);
			break;
		}

		case IDM_VIEWTOKENPROPERTIES :
			if (PKTokenProperties (ppks))
				PostMessage (hwnd, PK_M_RELOADKEYRINGS, 0, 0);
			break;

		case IDM_FINDKEY :
			PGPgmPerformAction (ppks->hGM, GM_LOCATEKEYS);
			ppks->bGroupsFocused = FALSE;
			SetFocus (ppks->hwndTreeList);
			break;

		case IDM_UPDATEINTRODUCERS :
			if (PKAutoUpdateIntroducers (hwnd, ppks, FALSE))
				PGPclKeyListReloadKeys (ppks->hKL, TRUE);
			break;

		case IDM_UPDATEREVOCATIONS :
			PKUpdateCARevocations (hwnd, ppks->hKL, ppks->keydbMain);
			break;

		case IDM_UPDATECONFIGURATION :
			if (PKUpdateConfigurationFromServer (hwnd, ppks->keydbMain))
				PGPclKeyListReloadKeys (ppks->hKL, TRUE);				
			break;

		case IDM_UPDATEGROUPLISTS :
			PGPgmPerformAction (ppks->hGM, GM_UPDATEALLGROUPS); 
			break;

		case IDM_SENDGROUPLISTS :
			PGPgmPerformAction (ppks->hGM, GM_SENDALLGROUPS); 
			break;

		case IDM_NEWGROUP :
			PGPgmNewGroup (ppks->hGM);
			break;

		case IDM_VIEWGROUPS :
		{
			RECT	rc;

			if (ppks->bGroupsVisible) 
			{
				ppks->bGroupsVisible = FALSE;
				CheckMenuItem (ppks->hmenuKeyMan, 
						IDM_VIEWGROUPS, MF_UNCHECKED);
				ppks->iGroupsHeight = 0;
				GetClientRect (hwnd, &rc);
				ppks->iKeysHeight = 
						(rc.bottom-rc.top)-ppks->iToolHeight-s_uStatusBarHeight;
				PostMessage (hwnd, PK_M_RESIZE, 0, 0);
			}
			else 
			{
				ppks->bGroupsVisible = TRUE;
				PGPgmReLoadGroups (ppks->hGM);
				CheckMenuItem (ppks->hmenuKeyMan, IDM_VIEWGROUPS, MF_CHECKED);
				GetClientRect (hwnd, &rc);
				ppks->iGroupsHeight = 
						(rc.bottom-rc.top) * ppks->iGroupsPercent;
				ppks->iGroupsHeight /= 100;
				ppks->iKeysHeight = 
						(rc.bottom-rc.top)-ppks->iToolHeight-s_uStatusBarHeight-
						ppks->iGroupsHeight-GRABBARHEIGHT;
				PostMessage (hwnd, PK_M_RESIZE, 0, 0);
			}
			break;
		}

		case IDM_SEARCH :
			// if main window has not finished initializing, then ignore
			if (!ppks->keydbMain)
				break;

			// if search window already exists, move to front
			if (s_hwndOpenSearch) 
			{
				ShowWindow (s_hwndOpenSearch, SW_RESTORE);
				SetForegroundWindow (s_hwndOpenSearch);
				break;
			}

			sNewPGPkeysWindow (hwnd, ppks, 
					TRUE, FALSE, FALSE, NULL, ppks->keydbMain);
			break;

		case IDM_NEWFILE :
			// if main window has not finished initializing, then ignore
			if (!ppks->keydbMain)
				break;
			PKNewWindowForFile (hwnd, ppks, TRUE, NULL);
			break;

		case IDM_OPENFILE :
			// if main window has not finished initializing, then ignore
			if (!ppks->keydbMain)
				break;
			PKNewWindowForFile (hwnd, ppks, FALSE, NULL);
			break;

		case IDM_SENDVIAEMAIL :
			PKPerformAction (ppks, KM_SENDVIAEMAIL);
			break;

		case IDM_SENDTOTOKEN :
			PKPerformAction (ppks, KM_SENDTOTOKEN);
			break;

		case IDM_DOMAINKEYSERVER :
			SendMessage (hwnd, WM_COMMAND, IDM_DOMAINKEYSERVERX, 0);
			break;

		case IDM_DOMAINKEYSERVERX :
			ZeroMemory (&ppks->klConfig.keyserver, sizeof(PGPKeyServerEntry));
			ppks->klConfig.uMask = kPGPclKeyList_KeyServer;
			PGPclKeyListConfigure (ppks->hKL, &(ppks->klConfig));
			PKPerformAction (ppks, KM_SENDTOSERVER);
			break;

		case IDM_TOOLBARSENDTOSERVER :
		{
			RECT	rc;

			PKGetSendToServerButtonRect (ppks->hwndToolBar, &rc);
			MapWindowPoints (ppks->hwndMain, NULL, (LPPOINT)&rc, 2);
			PKToolbarKeyserverMenu (ppks, hwnd, &rc);
			break;
		}

		default :
			if ((LOWORD (wParam) > IDM_DOMAINKEYSERVERX) &&
				(LOWORD (wParam) <= IDM_DOMAINKEYSERVERX + 999))
			{
				UINT	uAction;

				PKGetServerActionFromID (LOWORD (wParam), 
						&ppks->klConfig.keyserver, &uAction);
				ppks->klConfig.uMask = kPGPclKeyList_KeyServer;
				PGPclKeyListConfigure (ppks->hKL, &(ppks->klConfig));
				PKPerformAction (ppks, uAction);
			}
			else if ((LOWORD (wParam) >= IDM_PDAOPERATION) &&
					(LOWORD (wParam) <= IDM_PDAOPERATION + 998))
			{
				INT		iOp;

				// if main window has not finished initializing, then ignore
				if (!ppks->keydbMain)
					break;

				iOp = LOWORD (wParam) - IDM_PDAOPERATION;

				if (g_pPDA[iOp].bKeyring)
				{
					if (g_pPDA[iOp].hwnd)
						SetForegroundWindow (g_pPDA[iOp].hwnd);
					else
						sOpenPdaKeyring (hwnd, &g_pPDA[iOp], ppks, TRUE);
					break;
				}
				else
				{
					sPdaOperation (hwnd, 
							g_pPDA[iOp].hmod, g_pPDA[iOp].uOp);
				}
			}
			break;
		}
		break;

	default :
		// check for mousewheel message
		if (uMsg == g_uMouseWheelMessage) 
		{
			WORD iDelta;

			ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
			iDelta = (WORD)wParam;
			
			if (ppks->bGroupsFocused)
				SendMessage (ppks->hwndTreeListGroups, 
								WM_MOUSEWHEEL, MAKEWPARAM (0, iDelta), 0);
			else
				SendMessage (ppks->hwndTreeList, 
								WM_MOUSEWHEEL, MAKEWPARAM (0, iDelta), 0);
			return FALSE;
		}

		// check for prefs broadcast message
		else if (uMsg == g_uReloadPrefsMessage) 
		{
			MSG		msg;

			if (s_bIgnorePrefsReloads)
				s_bPrefsReloadIgnored = TRUE;
			else
			{
				ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
				PKReloadPGPPreferences (ppks);
			}
			while (PeekMessage  (&msg, hwnd, uMsg, uMsg, PM_REMOVE));
		}

		// check for keyring broadcast message
		else if (uMsg == g_uReloadKeyringMessage) 
		{
			ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

			if (ppks->klConfig.uOptions & kPGPclKeyList_EnableReloads) 
			{
				if (ppks->uFlags & PK_MAINWINDOW) 
				{
					if (wParam == kPGPclNewDefaultKeyrings)
					{
						PKDeleteAllObjectPropertiesWindows (ppks);

						if (s_hwndOpenSearch)
						{
							SendMessage (s_hwndOpenSearch, 
									PK_M_NEWKEYRINGS, wParam, lParam);
						}

						if (s_bKeyHasBeenGenerated) 
						{
							BOOL	bMakeBackups;

							PKBackupWarn (hwnd, FALSE, &bMakeBackups);
							if (bMakeBackups)
								PKUserBackup (hwnd, ppks->keydbMain);
							s_bKeyHasBeenGenerated = FALSE;
						}
					}

					if ((DWORD)lParam == GetCurrentProcessId ()) 
					{
						PKKeyDBModified (ppks, PK_MOD_INDEX_0);
						PGPclKeyListReloadKeys (ppks->hKL, TRUE);
					}
					else
					{
						PKAutoBackup (ppks);
						PostMessage (hwnd, PK_M_RELOADKEYRINGS, 0, 0);
					}
				}
			}

			return TRUE;
		}

		// check for keyserver prefs broadcast message
		else if (uMsg == g_uReloadKeyserverPrefsMessage) 
		{
			ppks = (PGPKEYSSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
			if (ppks->uFlags & PK_SEARCHWINDOW) 
			{
				NMHDR nmhdr;

				nmhdr.hwndFrom = hwnd;
				nmhdr.idFrom = 0;
				nmhdr.code = REFRESH_KEYSERVER_LIST;

				SendMessage (ppks->hwndSearchControl, 
					WM_NOTIFY, 
					0, 
					(LPARAM)&nmhdr);
			}
			return TRUE;
		}

		else 
			return (DefWindowProc (hwnd, uMsg, wParam, lParam));
	}
	return FALSE;
}

