/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	PKRevoke.c - implements various operations performed on keys. 
	
	$Id: PKRevoke.c,v 1.21 2002/10/25 01:42:16 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPkeysx.h"
#include "UTF8Edit.h"

// constant definitions
#define BITMAP_WIDTH	16
#define BITMAP_HEIGHT	16

#define INITIAL_SIGN_COLUMNWIDTH	210

#define SIG_NONEXPORTABLE	0
#define SIG_EXPORTABLE		1
#define SIG_TRUST			2
#define SIG_META			3

// external globals  
extern HINSTANCE		g_hinst;
extern PGPContextRef	g_context;
extern PGPtlsContextRef	g_tlscontext;

// typedefs
typedef struct {
	PPGPKEYSSTRUCT	ppks;
	PGPKeyDBObjRef	key;
	PGPBoolean		bSyncWithServer;
} REVOKECERTSTRUCT, *PREVOKECERTSTRUCT;


//	___________________________________________________
//
//  revoke split key and all subkeys

static PGPError 
sRevokeKeySplit (
		PGPKeyDBRef		keydb,
		PGPKeyDBObjRef	key,
		PGPByte*		passkey,
		PGPSize			sizePasskey)
{
	PGPKeyIterRef	keyiter;
	PGPKeyDBObjRef	subkey;
	PGPError		err;

	err = PGPRevoke (key, 
			PGPOPasskeyBuffer (g_context, passkey, sizePasskey),
			PGPOLastOption (g_context));
	if (IsPGPError (err)) 
		return err;

	PGPNewKeyIterFromKeyDB (keydb, &keyiter);
	PGPKeyIterSeek (keyiter, key);
	PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_SubKey, &subkey);
	while (subkey) 
	{
		err = PGPRevoke (subkey, 
				PGPOPasskeyBuffer (g_context, passkey, sizePasskey),
				PGPOLastOption (g_context));
		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_SubKey, &subkey);
	}
	PGPFreeKeyIter (keyiter);

	return err;
}

//	___________________________________________________
//
//  revoke normal key and all subkeys

static PGPError 
sRevokeKeyNormal (
		PGPKeyDBRef		keydb,
		PGPKeyDBObjRef	key, 
		LPSTR			pszPhrase)
{
	PGPKeyIterRef	keyiter;
	PGPKeyDBObjRef	subkey;
	PGPError		err;

	err = PGPRevoke (key, 
			PGPOPassphrase (g_context, pszPhrase),
			PGPOLastOption (g_context));
	if (IsPGPError (err)) 
		return err;

	PGPNewKeyIterFromKeyDB (keydb, &keyiter);
	PGPKeyIterSeek (keyiter, key);
	PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_SubKey, &subkey);
	while (subkey) 
	{
		err = PGPRevoke (subkey, 
				PGPOPassphrase (g_context, pszPhrase),
				PGPOLastOption (g_context));
		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_SubKey, &subkey);
	}
	PGPFreeKeyIter (keyiter);

	return err;
}

//	___________________________________________________
//
//  Revoke selected key

BOOL 
PKRevokeKey (
		PPGPKEYSSTRUCT	ppks)
{
	PGPPrefRef		prefref				= kInvalidPGPPrefRef;
	BOOL			bRetVal				= TRUE;
	PGPError		err					= kPGPError_NoErr;
	LPSTR			pszPhrase			= NULL;
	PGPByte*		pPasskey			= NULL;
	PGPKeySetRef	keysetRevokers		= kInvalidPGPKeySetRef;
	PGPKeyDBObjRef	keyRevoker			= kInvalidPGPKeyDBObjRef;
	PGPBoolean		bSecret				= FALSE;
	PGPBoolean		bSplit				= FALSE;
	PGPBoolean		bSyncWithServer		= FALSE;

	PGPSize			sizePasskey;
	PGPKeyDBObjRef	key;
	PGPKeyDBObjRef	keyToRevoke;
	PGPKeyDBObjRef	keyDef;
	CHAR			sz128[128];
	PGPUInt32		u, uNumRevokers;

	if (IsPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		return FALSE;

	PGPGetPrefBoolean (prefref, kPGPPrefKeyServerSyncOnRevocation, 
						&bSyncWithServer);

	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &keyToRevoke, NULL, NULL);
	PGPclGetDefaultPrivateKey (ppks->keydbMain, &keyDef);

	if (keyToRevoke == keyDef) 
	{
		if (PKMessageBox (ppks->hwndMain, IDS_CAPTIONWARNING, IDS_REVCONFDEFKEY,
				MB_YESNO|MB_TASKMODAL|MB_DEFBUTTON2|MB_ICONWARNING) == IDNO)
		{
			return FALSE;
		}
	}
	else 
	{
		if (PKMessageBox (ppks->hwndMain, IDS_CAPTIONWARNING, IDS_REVOKECONFIRM, 
				MB_YESNO|MB_ICONWARNING) == IDNO) 
		{
			return FALSE;
		}
	}

	err = PGPGetKeyDBObjBooleanProperty (keyToRevoke, 
				kPGPKeyProperty_IsSecret, &bSecret); CKERR;

	if (bSecret) 
	{
		keyRevoker = keyToRevoke;
		err = PGPGetKeyDBObjBooleanProperty (keyToRevoke, 
				kPGPKeyProperty_IsSecretShared, &bSplit); CKERR;
	}
	else 
	{
		err = PGPCountRevocationKeys (keyToRevoke, &uNumRevokers);  CKERR;
		for (u = 0; u < uNumRevokers; u++) 
		{
			err = PGPGetIndexedRevocationKey (keyToRevoke, 
					u, &key, NULL); CKERR;
			key = PGPPeekKeyDBObjKey (key);

			if (PGPKeyDBObjRefIsValid (key)) 
			{
				err = PGPGetKeyDBObjBooleanProperty (key, 
						kPGPKeyProperty_IsSecret, &bSecret); CKERR;
				err = PGPGetKeyDBObjBooleanProperty (key, 
						kPGPKeyProperty_IsSecretShared, &bSplit); CKERR;

				if (bSecret) 
				{
					keyRevoker = key;
					if (!bSplit) 
						break;
				}
			}
		}
	}

	if (!PGPKeyDBObjRefIsValid (keyRevoker))
		goto done;

	// get valid passphrase
	LoadString (g_hinst, IDS_SELKEYPASSPHRASE, sz128, 128); 
	err = PGPclGetKeyPhrase (g_context, g_tlscontext,
			ppks->hwndMain, sz128,
			ppks->keydbMain, keyRevoker,
			&pszPhrase, &pPasskey, &sizePasskey);
	PGPclErrorBox (ppks->hwndMain, err);

	// now we have a valid passphrase, if required
	if (IsntPGPError (err)) 
	{
		// update from server
		if (bSyncWithServer) 
		{
			if (!PKGetFromServerInternal (ppks, FALSE, FALSE, FALSE)) 
			{
				if (PKMessageBox (ppks->hwndMain, IDS_CAPTIONALERT, 
						IDS_QUERYCONTINUEREVOKINGKEY, 
						MB_YESNO|MB_ICONQUESTION) == IDNO) 
				{
					bRetVal = FALSE;
				}
			}
		}
		
		if (bRetVal) 
		{
			// make sure we have enough entropy
			PGPclRandom (g_context, ppks->hwndMain, 0);

			if (bSplit) 
			{
				err = sRevokeKeySplit (ppks->keydbMain,
						keyToRevoke, pPasskey, sizePasskey);
			}
			else 
			{
				err = sRevokeKeyNormal (ppks->keydbMain,
						keyToRevoke, pszPhrase);
			}
						
			if (IsntPGPError (PGPclErrorBox (NULL, err))) 
			{
				PKKeyDBModified (ppks, PK_MOD_INDEX_0);
				PGPclKeyListUpdateTree (ppks->hKL, 
						kPGPclKeyListUpdateObject, keyToRevoke, FALSE);
				PGPclKeyListSetTree (ppks->hKL,
						kPGPclSelectSpecifiedOnly, keyToRevoke);
				PGPclKeyListUpdateTree (ppks->hKL, 
						kPGPclKeyListUpdateValidities, NULL, TRUE);

				// send to server
				if (bSyncWithServer)
					PKSendToServer (ppks, kPGPclDefaultServer);
			}
			else 
				bRetVal = FALSE;
		}
	}
	else 
		bRetVal = FALSE;

done :
	if (IsntNull (pszPhrase))
		PGPclFreePhrase (pszPhrase);
	if (IsntNull (pPasskey)) 
		PGPclFreePasskey (pPasskey, sizePasskey);
	if (PGPKeySetRefIsValid (keysetRevokers))
		PGPFreeKeySet (keysetRevokers);

	PGPclErrorBox (ppks->hwndMain, err);

	return bRetVal;
}


//	___________________________________________________
//
//	Revoke signature dialog message procedure

static BOOL CALLBACK 
sRevokeCertDlgProc (
		HWND	hwnd, 
		UINT	uMsg, 								
		WPARAM	wParam, 
		LPARAM	lParam) 
{
	PREVOKECERTSTRUCT prcs;

	switch (uMsg) {

	case WM_INITDIALOG :
		{
			CHAR		sz[kPGPMaxUserIDSize +1];
			PGPKeyID	keyid;
			PGPSize		size;

			SetWindowLong (hwnd, GWL_USERDATA, lParam);
			prcs = (PREVOKECERTSTRUCT)lParam;

			PGPGetKeyID (prcs->key, &keyid);
			PGPGetKeyIDString (&keyid, kPGPKeyIDString_Abbreviated, sz);
			SetDlgItemText (hwnd, IDC_KEYID, sz);

			UTF8EditInit (GetDlgItem (hwnd, IDC_NAME));
			PGPclGetPrimaryUserIDNameUTF8 (prcs->key, sz, sizeof(sz), &size);
			UTF8EditSetText (GetDlgItem (hwnd, IDC_NAME), sz);
		}
		break;

	case WM_DESTROY :
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_NAME));
		return FALSE;

	case WM_COMMAND:

		switch (LOWORD(wParam)) {
		case IDCANCEL:
			prcs = (PREVOKECERTSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			prcs->bSyncWithServer = FALSE;
			EndDialog (hwnd, 1);
			break;

		case IDOK:
			EndDialog (hwnd, 0);
			break;

		case IDHELP :
			PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0, 
					(char*)kPGPclHelpFile, (DWORD*)IDH_PGPPK_REVOKECERTDIALOG);
			break;
		}
		return TRUE;
	}
	return FALSE;
}


//	___________________________________________________
//
//  Revoke selected signature

BOOL 
PKRevokeCert (
		PPGPKEYSSTRUCT	ppks)
{
	PGPPrefRef			prefref			= kInvalidPGPPrefRef;
	BOOL				bRetVal			= TRUE;
	PGPByte*			pPasskey		= NULL;

	PGPSize				sizePasskey;
	PGPKeyDBObjRef		cert;
	PGPKeyDBObjRef		keySigning, keyParent;
	PGPError			err;
	CHAR				sz128[128];
	REVOKECERTSTRUCT	rcs;

	if (IsPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		return FALSE;

	PGPGetPrefBoolean (prefref, kPGPPrefKeyServerSyncOnRevocation, 
			&(rcs.bSyncWithServer));

	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &cert, NULL, NULL);

	err = PGPGetSigCertifierKey (cert, ppks->keydbMain, &keySigning);
	if (err == kPGPError_ItemNotFound) 
	{
		keySigning = NULL;
		err = kPGPError_NoErr;
	}

	if (IsntPGPError (PGPclErrorBox (NULL, err))) 
	{
		if (!keySigning) 
		{
			PKMessageBox (ppks->hwndMain, IDS_CAPTIONALERT, 
					IDS_CERTKEYNOTONRING, MB_OK|MB_ICONEXCLAMATION);
			return FALSE;
		}

		rcs.ppks = ppks;
		rcs.key = PGPPeekKeyDBObjKey (cert);

		if (DialogBoxParam (g_hinst, MAKEINTRESOURCE(IDD_REVOKECERT), 
				ppks->hwndMain, sRevokeCertDlgProc, (LPARAM)&rcs)) 
		{
			return FALSE;
		}
		
		// get valid passphrase
		LoadString (g_hinst, IDS_SIGNKEYPASSPHRASE, sz128, 128); 
		err = PGPclGetKeyPhrase (g_context, g_tlscontext,
				ppks->hwndMain, sz128, ppks->keydbMain, keySigning,
				NULL, &pPasskey, &sizePasskey);
		PGPclErrorBox (NULL, err);

		// now we have a valid passphrase, if required
		if (IsntPGPError (err)) 
		{
			// update from server
			if (rcs.bSyncWithServer) 
			{
				if (!PKGetFromServerInternal (ppks, FALSE, FALSE, FALSE)) 
				{
					if (PKMessageBox (ppks->hwndMain, IDS_CAPTIONALERT, 
							IDS_QUERYCONTINUEREVOKINGCERT, 
							MB_YESNO|MB_ICONQUESTION) == IDNO) 
					{
						bRetVal = FALSE;
					}
				}
			}
		
			if (bRetVal) 
			{
				// make sure we have enough entropy
				PGPclRandom (g_context, ppks->hwndMain, 0);

				err = PGPRevokeSig (cert,
						pPasskey ? 
							PGPOPasskeyBuffer (g_context, 
								pPasskey, sizePasskey) :
							PGPONullOption (g_context),
						PGPOLastOption (g_context));
						
				if (IsntPGPError (PGPclErrorBox (ppks->hwndMain, err))) 
				{
					PKKeyDBModified (ppks, PK_MOD_INDEX_0);
					keyParent = PGPPeekKeyDBObjKey (cert);
					PGPclKeyListUpdateTree (ppks->hKL, 
							kPGPclKeyListUpdateObject, 
							keyParent, FALSE);
					PGPclKeyListSetTree (ppks->hKL,
							kPGPclSelectSpecifiedOnly, cert);
					PKUpdateAllObjectPropertiesWindows (ppks);
					PGPclKeyListUpdateTree (ppks->hKL, 
							kPGPclKeyListUpdateValidities, 
							kInvalidPGPKeyDBObjRef, FALSE);

					// send key to server, if selected
					if (rcs.bSyncWithServer)
						PKSendToServer (ppks, kPGPclDefaultServer);
				}
				else 
					bRetVal = FALSE;
			}
		}
		else 
			bRetVal = FALSE;
	}
	else 
		bRetVal = FALSE;

	if (pPasskey) 
	{
		PGPclFreePasskey (pPasskey, sizePasskey);
		pPasskey = NULL;
	}

	return bRetVal;
}


//	___________________________________________________
//
//  Add designated revoker to key

BOOL 
PKAddRevoker (
		PPGPKEYSSTRUCT	ppks)
{
	PGPPrefRef		prefref				= kInvalidPGPPrefRef;
	PGPKeySetRef	keysetToChoose		= kInvalidPGPKeySetRef;
	PGPKeySetRef	keysetSelected		= kInvalidPGPKeySetRef;
	PGPFilterRef	filterKey			= kInvalidPGPFilterRef;
	PGPFilterRef	filterNotKey		= kInvalidPGPFilterRef;
	PGPFilterRef	filterV4			= kInvalidPGPFilterRef;
	PGPFilterRef	filterFinal			= kInvalidPGPFilterRef;
	PGPError		err					= kPGPError_NoErr;
	PGPByte*		pbyte				= NULL;
	BOOL			bRet				= FALSE;
	PGPUInt32		uCount				= 0;
	PGPBoolean		bSyncWithServer		= FALSE;

	PGPSize			size;
	PGPKeyID		keyid;
	PGPKeyDBObjRef	key;
	CHAR			szPrompt[256];


	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &key, NULL, NULL);

	err = PGPclPeekClientLibPrefRefs (&prefref, NULL); CKERR;
	err = PGPGetPrefBoolean (prefref, 
			kPGPPrefKeyServerSyncOnAdd, &bSyncWithServer); CKERR;

	// remove our own key from list
	err = PGPGetKeyID (key, &keyid); CKERR;
	err = PGPNewKeyDBObjDataFilter (g_context, 
			kPGPKeyProperty_KeyID, &keyid, sizeof(PGPKeyID),
			kPGPMatchCriterion_Equal, &filterKey); CKERR;
	err = PGPNegateFilter (filterKey, &filterNotKey); CKERR;

	// only allow V4 keys in list
	err = PGPNewKeyDBObjNumericFilter (g_context, 
			kPGPKeyProperty_Version, kPGPKeyVersion_V4,
			kPGPMatchCriterion_GreaterOrEqual, &filterV4); CKERR;

	// now actually create the keyset
	err = PGPIntersectFilters (filterNotKey, filterV4, &filterFinal); CKERR;
	err = PGPFilterKeyDB (ppks->keydbMain, filterFinal, &keysetToChoose);

	err = PGPCountKeys (keysetToChoose, &uCount); CKERR;
	if (uCount < 1) 
	{
		PKMessageBox (ppks->hwndMain, IDS_CAPTIONINFO, 
				IDS_NOTENOUGHKEYSTOADDREVOKER, MB_OK|MB_ICONINFORMATION);
		goto done;
	}

	LoadString (g_hinst, IDS_ADDREVOKERPROMPT, szPrompt, sizeof(szPrompt));
	err = PGPclSelectKeys (g_context, g_tlscontext, 
			ppks->hwndMain, szPrompt, keysetToChoose,
			kPGPclDefaultSelectionFlags, &keysetSelected);

	if (IsntPGPError (err) && PGPKeySetRefIsValid (keysetSelected))
	{
		if (PKMessageBox (ppks->hwndMain, IDS_CAPTIONWARNING, 
				IDS_ADDREVOKERCONFIRM, MB_YESNO|MB_ICONWARNING) == IDYES) 
		{
			err = PGPclGetKeyPhrase (g_context, g_tlscontext,
						ppks->hwndMain, NULL, ppks->keydbMain, key, 
						NULL, &pbyte, &size); CKERR;

			// update from server
			if (IsntPGPError (err) && bSyncWithServer) 
			{
				if (!PKGetFromServerInternal (ppks, FALSE, FALSE, FALSE)) 
				{
					if (PKMessageBox (ppks->hwndMain, IDS_CAPTIONALERT, 
								IDS_QUERYCONTINUEADDING, 	
								MB_YESNO|MB_ICONQUESTION) == IDNO) 
					{
						err = kPGPError_UserAbort;
					}
				}
			}
		
			if (IsntPGPError (err)) 
			{
				err = PGPAddKeyOptions (key, 
					PGPORevocationKeySet (g_context, keysetSelected),
					pbyte ?
						PGPOPasskeyBuffer (g_context, pbyte, size) :
						PGPONullOption (g_context),
					PGPOLastOption (g_context)); CKERR;
				PKKeyDBModified (ppks, PK_MOD_INDEX_0);
			}
		}
		else
			err = kPGPError_UserAbort;
	}

	// send to server
	if (IsntPGPError (err) && bSyncWithServer) 
		PKSendToServer (ppks, kPGPclDefaultServer);

	if (IsntPGPError (err)) 
	{
		bRet = TRUE;

		if (bSyncWithServer) 
		{
			PKMessageBox (ppks->hwndMain, IDS_CAPTIONINFO, 
					IDS_ADDEDSENTREVOKERS, MB_OK|MB_ICONINFORMATION);
		}
		else 
		{
			PKMessageBox (ppks->hwndMain, IDS_CAPTIONINFO, 
					IDS_ADDEDREVOKERS, MB_OK|MB_ICONINFORMATION);
		}
	}

done :
	if (IsntNull (pbyte))
		PGPclFreePasskey (pbyte, size);
	if (PGPKeySetRefIsValid (keysetSelected))
		PGPFreeKeySet (keysetSelected);
	if (PGPKeySetRefIsValid (keysetToChoose))
		PGPFreeKeySet (keysetToChoose);
	if (PGPFilterRefIsValid (filterFinal))
		PGPFreeFilter (filterFinal);
	else
	{
		if (PGPFilterRefIsValid (filterV4))
			PGPFreeFilter (filterV4);

		if (PGPFilterRefIsValid (filterNotKey))
			PGPFreeFilter (filterNotKey);
		else
		{
			if (PGPFilterRefIsValid (filterKey))
				PGPFreeFilter (filterKey);
		}
	}

	PGPclErrorBox (ppks->hwndMain, err);

	return bRet;
}
