/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	PKSplit.c - code for splitting secret keys.

	$Id: PKSplit.c,v 1.26 2002/11/01 15:52:08 pbj Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"

// project header files
#include "PGPkeysx.h"
#include "pgpImageList.h"

// system header files
#include <shlobj.h>

// pgp header files
#include "pgpFileUtilities.h"
#include "pgpShareFile.h"

#include "pgpUnicodeWin32.h"
#include "UTF8Edit.h"
#include "keydrawitem.h"

// constant definitions
#define INITIAL_SHAREKEY_COLUMNWIDTH	250
#define MAX_TOTAL_SHARES				255
#define MAX_SHARES						99
#define MAX_SHARES_LEN					2
#define MAX_MODIFIERS					99

// typedefs
typedef struct {
	INT		iNumSteps;
} SPLITKEYPROGRESSSTRUCT;


// external globals
extern HINSTANCE		g_hinst;
extern HIMAGELIST		g_hilKeys;
extern PGPContextRef	g_context;
extern PGPtlsContextRef	g_tlscontext;

// local global
static PSPLITKEYSTRUCT psksList = NULL;

static DWORD aSplitIds[] = {			// Help IDs
	IDC_KEYTOSPLIT,			IDH_PGPPK_KEYTOSPLIT,
	IDC_SHAREHOLDERS,		IDH_PGPPK_SHAREHOLDERS,
	IDC_CURRENTSHAREHOLDER,	IDH_PGPPK_SELECTEDSHAREHOLDER,
	IDC_SHARES,				IDH_PGPPK_SELECTEDSHARES,
	IDC_REMOVESHAREHOLDER,	IDH_PGPPK_REMOVESHARES,
	IDC_ADDSHAREHOLDER,		IDH_PGPPK_ADDSHARES,
	IDC_TOTALSHARES,		IDH_PGPPK_TOTALSHARES,
	IDC_THRESHOLD,			IDH_PGPPK_SPLITTHRESHOLD,
	IDOK,					IDH_PGPPK_SPLIT,
    0,0
};

static DWORD aNewShareIds[] = {			// Help IDs
    IDC_NEWSHAREHOLDER,		IDH_PGPPK_NEWSHAREHOLDER,
    0,0
};

//	_______________________________________________
//
//  add shareholder to listview control

static BOOL
sAddShareHolder (
		PSPLITKEYSTRUCT		psks,
		PSHAREHOLDERSTRUCT	pshs,
		INT					iImage,
		HWND				hwndList)
{
	LV_ITEM		lvI;
	INT			iItem;
	CHAR		sz[16];

	// see if we're over the limit
	if ((psks->uTotalShares + pshs->uShares) > MAX_TOTAL_SHARES)
	{
		MessageBeep (MB_ICONASTERISK);
		return FALSE;
	}

	// figure item index to use
	iItem = ListView_GetItemCount (hwndList);

	// insert listview item
	lvI.mask = LVIF_TEXT|LVIF_IMAGE|LVIF_STATE|LVIF_PARAM;
	lvI.state = 0;
	lvI.stateMask = 0;
	lvI.iImage = iImage;

	lvI.iItem = iItem;
	lvI.iSubItem = 0;
	lvI.pszText	= pshs->szUserID;
	lvI.cchTextMax = 0;
	lvI.lParam = (LPARAM)pshs;

	iItem = ListView_InsertItem (hwndList, &lvI);
	if (iItem == -1)
		return FALSE;
	else
	{
		// add strings for Shares column
		wsprintf (sz, "%i", pshs->uShares);
		ListView_SetItemText (hwndList, iItem, 1, sz);
		return TRUE;
	}
}


//	_______________________________________________
//
//  destroy shareholder data structures from listview control

static BOOL
sIsKeyIDAlreadyInList (
		PGPKeyID*				pkeyid,
		PSPLITKEYSTRUCT			psks)
{
	INT					iNumItems;
	INT					iIndex;
	LV_ITEM				lvI;
	PSHAREHOLDERSTRUCT	pshs;

	iNumItems = ListView_GetItemCount (psks->hwndList);
	for (iIndex=0; iIndex<iNumItems; iIndex++)
	{
		lvI.mask = LVIF_PARAM;
		lvI.iItem = iIndex;
		lvI.iSubItem = 0;
		ListView_GetItem (psks->hwndList, &lvI);

		pshs = (PSHAREHOLDERSTRUCT)lvI.lParam;
		if (pshs->bPublicKey)
		{
			if (PGPCompareKeyIDs (&pshs->keyid, pkeyid) == 0)
				return TRUE;
		}
	}

	return FALSE;
}


//	_______________________________________________
//
//  Drop text key(s)

BOOL
PKSplitDropKeys (
		PSPLITKEYSTRUCT psks,
		HANDLE	hMem)
{
	PGPKeyDBRef				keydb		= kInvalidPGPKeyDBRef;
	PGPKeyIterRef			keyiter		= kInvalidPGPKeyIterRef;
	LPSTR					pMem		= NULL;

	PGPError				err;
	PGPKeyDBObjRef			key;
	PGPKeyID				keyid;
	BOOL					bKeys;
	PGPBoolean				bKeyIsUsable;
	size_t					sLen;
	PSHAREHOLDERSTRUCT		pshs;

	bKeys = FALSE;

	if (hMem)
	{
		pMem = GlobalLock (hMem);
		if (pMem)
		{
			sLen = lstrlen (pMem);
			err = PGPImport (g_context, &keydb,
					PGPOInputBuffer (g_context, pMem, sLen),
					PGPOLastOption (g_context));
			if (IsPGPError (err))
				goto SplitDropCleanup;

			err = PGPNewKeyIterFromKeyDB (keydb, &keyiter);
			if (IsPGPError (err))
				goto SplitDropCleanup;

			PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);

			while (key)
			{
				bKeyIsUsable = FALSE;

				// key must either not be RSA or RSA ops must be enabled
				PGPGetKeyDBObjBooleanProperty (key, 
						kPGPKeyProperty_CanEncrypt, &bKeyIsUsable);

				// key must not be the same one that is being split
				PGPGetKeyID (key, &keyid);
				if (PGPCompareKeyIDs (&keyid, &(psks->keyidToSplit)) == 0)
					bKeyIsUsable = FALSE;

				// key must not already be in list
				if (sIsKeyIDAlreadyInList (&keyid, psks))
					bKeyIsUsable = FALSE;

				if (bKeyIsUsable)
				{
					bKeys = TRUE;
					pshs = pkAlloc (sizeof(SHAREHOLDERSTRUCT));
					if (pshs)
					{
						PGPSize			size;
						PGPKeyDBObjRef	userid;

						pshs->bPublicKey = TRUE;
						pshs->pszPassphrase = NULL;
						pshs->uShares = 1;

						PGPclGetPrimaryUserIDNameUTF8 (key,
							pshs->szUserID, sizeof(pshs->szUserID), &size);
						PGPGetKeyID (key, &(pshs->keyid));

						PGPGetPrimaryUserID (key, &userid);

						if (sAddShareHolder (psks, pshs,
								PGPclDetermineUserIDIcon (userid, NULL),
								psks->hwndList))
						{
							psks->uTotalShares += pshs->uShares;
							SetDlgItemInt (psks->hwndDlg, IDC_TOTALSHARES,
									psks->uTotalShares, FALSE);
							SendMessage (psks->hwndDlg, WM_COMMAND,
									MAKEWPARAM(IDC_THRESHOLD, EN_CHANGE), 0);
						}
						else
							pkFree (pshs);
					}
				}

				PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);
			}

SplitDropCleanup :
			if (keyiter)
				PGPFreeKeyIter (keyiter);
			if (keydb)
				PGPFreeKeyDB (keydb);
			if (pMem)
				GlobalUnlock (hMem);
		}
	}

	return bKeys;
}


//	_______________________________________________
//
//  Initialize ListView

static VOID
sInitKeyList (PSPLITKEYSTRUCT psks)
{

	LV_COLUMN	lvC;
	CHAR		sz[256];

	ListView_SetImageList (psks->hwndList, g_hilKeys, LVSIL_SMALL);

	lvC.mask = LVCF_FMT | LVCF_TEXT | LVCF_WIDTH | LVCF_SUBITEM;
	lvC.fmt = LVCFMT_LEFT;
	lvC.pszText = sz;

	LoadString (g_hinst, IDS_USERID, sz, sizeof(sz));
	lvC.cx = INITIAL_SHAREKEY_COLUMNWIDTH;
	lvC.iSubItem = 0;
	if (ListView_InsertColumn (psks->hwndList, 0, &lvC) == -1) return;

	LoadString (g_hinst, IDS_SHARES, sz, sizeof(sz));
	lvC.cx = 50;
	lvC.iSubItem = 1;
	if (ListView_InsertColumn (psks->hwndList, 1, &lvC) == -1) return;
}

//	_______________________________________________
//
//  remove shareholder from listview control

static VOID
sRemoveShareHolderFromList (HWND hwnd, PSPLITKEYSTRUCT psks)
{
	INT					iIndex;
	LV_ITEM				lvI;
	PSHAREHOLDERSTRUCT	pshs;

	iIndex = ListView_GetNextItem (psks->hwndList, -1, LVNI_SELECTED);

	if (iIndex > -1)
	{
		lvI.mask = LVIF_PARAM;
		lvI.iItem = iIndex;
		lvI.iSubItem = 0;
		ListView_GetItem (psks->hwndList, &lvI);

		// update total shares
		pshs = (PSHAREHOLDERSTRUCT)lvI.lParam;
		psks->uTotalShares -= pshs->uShares;
		SetDlgItemInt (hwnd, IDC_TOTALSHARES, psks->uTotalShares, FALSE);
		SendMessage (hwnd, WM_COMMAND,
				MAKEWPARAM (IDC_THRESHOLD, EN_CHANGE), 0);

		PGPclFreePhrase (pshs->pszPassphrase);
		pkFree (pshs);

		ListView_DeleteItem (psks->hwndList, iIndex);
	}
}


//	_______________________________________________
//
// add share holder dialog message procedure

static BOOL CALLBACK
sAddShareHolderDlgProc (HWND hwnd,
					   UINT uMsg,
					   WPARAM wParam,
					   LPARAM lParam)
{
	PSPLITKEYSTRUCT		psks;
	PSHAREHOLDERSTRUCT	pshs;

	switch (uMsg) {

	case WM_INITDIALOG :
		// save address of struct
		SetWindowLong (hwnd, GWL_USERDATA, lParam);
		UTF8EditInit (GetDlgItem (hwnd, IDC_NEWSHAREHOLDER));
		return TRUE;

	case WM_DESTROY :
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_NEWSHAREHOLDER));
		return FALSE;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aNewShareIds);

	case WM_COMMAND :
		switch (LOWORD(wParam)) {

		case IDCANCEL :
			EndDialog (hwnd, 0);
			break;

		case IDOK :
			psks = (PSPLITKEYSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			pshs = pkAlloc (sizeof(SHAREHOLDERSTRUCT));
			if (pshs)
			{
				CHAR		szPrompt[64];
				PGPError	err;

				pshs->bPublicKey = FALSE;
				pshs->pszPassphrase = NULL;
				pshs->uShares = 1;

				UTF8EditGetText (GetDlgItem (hwnd, IDC_NEWSHAREHOLDER), 
						pshs->szUserID, sizeof(pshs->szUserID));

				LoadString (g_hinst, IDS_SHAREPHRASEPROMPT,
						szPrompt, sizeof (szPrompt));
				err = PGPclGetConfirmationPhrase (g_context, hwnd,
						szPrompt, psks->ppks->keydbMain,
						1, 0, &(pshs->pszPassphrase));

				if (IsntPGPError (err))
				{
					if (sAddShareHolder (
							psks, pshs, IDX_HEAD, psks->hwndList))
					{
						psks->uTotalShares += pshs->uShares;
						EndDialog (hwnd, 1);
					}
					else
					{
						PGPclFreePhrase (pshs->pszPassphrase);
						pkFree (pshs);
						EndDialog (hwnd, 0);
					}
				}
			}
			else
				EndDialog (hwnd, 0);
			break;

		default :
			if (HIWORD(wParam) == EN_UPDATE)
			{
				EnableWindow (GetDlgItem (hwnd, IDOK), 
						(UTF8EditGetTextLength (GetDlgItem (
							hwnd, IDC_NEWSHAREHOLDER)) > 0));
			}
			break;
		}
		return TRUE;
	}
	return FALSE;
}


//	_______________________________________________
//
//  pop dialog for adding conventional encryption shareholder

static VOID
sAddShareHolderToList (HWND hwnd, PSPLITKEYSTRUCT psks)
{
	if (DialogBoxParam (g_hinst, MAKEINTRESOURCE (IDD_ADDSHAREHOLDER),
		hwnd, sAddShareHolderDlgProc, (LPARAM)psks))
	{
		SetDlgItemInt (hwnd, IDC_TOTALSHARES, psks->uTotalShares, FALSE);
		SendMessage (hwnd, WM_COMMAND,
				MAKEWPARAM (IDC_THRESHOLD, EN_CHANGE), 0);
	}
}


//	_______________________________________________
//
//  destroy shareholder data structures from listview control

static VOID
sDestroyShareHolders (PSPLITKEYSTRUCT psks)
{
	INT					iNumItems;
	INT					iIndex;
	LV_ITEM				lvI;
	PSHAREHOLDERSTRUCT	pshs;

	iNumItems = ListView_GetItemCount (psks->hwndList);
	for (iIndex=0; iIndex<iNumItems; iIndex++)
	{
		lvI.mask = LVIF_PARAM;
		lvI.iItem = iIndex;
		lvI.iSubItem = 0;
		ListView_GetItem (psks->hwndList, &lvI);

		pshs = (PSHAREHOLDERSTRUCT)lvI.lParam;
		PGPclFreePhrase (pshs->pszPassphrase);

		pkFree (pshs);
	}
}


//	_______________________________________________
//
//  destroy dialog struct and remove from list

static VOID
sDestroySplitKeyStruct (PSPLITKEYSTRUCT psksToDestroy)
{
	PSPLITKEYSTRUCT* ppsks;

	ppsks = psksToDestroy->pHeadOfList;

	while (*ppsks)
	{
		if (*ppsks == psksToDestroy)
		{
			*ppsks = psksToDestroy->next;
			pkFree (psksToDestroy);
			return;
		}
		ppsks = &((*ppsks)->next);
	}
}


//	_______________________________________________
//
//  Compute name of share file

static PGPError
sCreateFilePathFromUserName (
		LPSTR	pszFolder,
		LPSTR	pszOldUserID,
		UINT	uUserNumber,
		UINT	uNumShares,
		LPSTR	pszModifier,
		LPSTR	pszPath,
		INT		iLen)
{
	CHAR		sz[kPGPMaxUserIDSize];
	CHAR		szDefName[16];
	CHAR		szDefExt[8];
	CHAR		szShares[16];
	INT			iMinRequiredLen;
	INT			i;
	CHAR		pszUserID[kPGPMaxUserIDSize];
	INT			userLen;

	pgpUTF8StringToLocal (
 		0, 0, pszOldUserID, strlen(pszOldUserID), 
 		pszUserID, 
 		kPGPMaxUserIDSize, &userLen);

	// prepare number of shares substring
	if (uNumShares == 1)
		LoadString (g_hinst, IDS_ONESHARE, szShares, sizeof(szShares));
	else
	{
		LoadString (g_hinst, IDS_NUMSHARES, sz, sizeof(sz));
		wsprintf (szShares, sz, uNumShares);
	}

	// get default file name and extension
	LoadString (g_hinst, IDS_DEFSHARENAME, szDefName, sizeof(szDefName));
	wsprintf (sz, " #%i", uUserNumber);
	lstrcat (szDefName, sz);
	LoadString (g_hinst, IDS_DEFSHAREEXTENSION, szDefExt, sizeof(szDefExt));

	// check length of destination buffer
	iMinRequiredLen =
		lstrlen (pszFolder) + lstrlen (szDefExt) + lstrlen (szDefName) +1;
	if (iMinRequiredLen >= iLen)
	{
		return kPGPError_CantOpenFile;
	}

	// put folder into path
	lstrcpy (pszPath, pszFolder);
	iLen -= lstrlen (pszPath);
	if (pszPath[lstrlen(pszPath)-1] != '\\')
	{
		lstrcat (pszPath, "\\");
		iLen -= 1;
	}

	// look for invalid characters and truncate
	lstrcpy (sz, pszUserID);
	i = strcspn (sz, "\\/:*?""<>|");
	sz[i] = '\0';

	// remove trailing spaces
	while ((i > 0) && (sz[i-1] == ' '))
	{
		i--;
		sz[i] = '\0';
	}

	// check if we've truncated too much
	if (lstrlen (sz) < 2)
		lstrcpy (sz, szDefName);

	// check if name is too long
	iLen -= (lstrlen (szDefExt) +1);
	if ((lstrlen(sz) + lstrlen(szShares) + lstrlen(pszModifier)) >= iLen)
	{
		if ((lstrlen (sz) + lstrlen (pszModifier)) >= iLen)
		{
			if (lstrlen (sz) >= iLen)
				sz[iLen-1] = '\0';
			lstrcat (pszPath, sz);
			lstrcat (pszPath, ".");
			lstrcat (pszPath, szDefExt);

		}
		else
		{
			lstrcat (pszPath, sz);
			lstrcat (pszPath, pszModifier);
			lstrcat (pszPath, ".");
			lstrcat (pszPath, szDefExt);
		}
	}
	else
	{
		// construct full path
		lstrcat (pszPath, sz);
		lstrcat (pszPath, szShares);
		lstrcat (pszPath, pszModifier);
		lstrcat (pszPath, ".");
		lstrcat (pszPath, szDefExt);
	}

	return kPGPError_NoErr;
}


//	_______________________________________________
//
//  split the key

static PGPError
sSaveSharesToFile (
		PSHAREHOLDERSTRUCT	pshs,
		UINT				uUserNumber,
		PGPShareRef			sharesTotal,
		PGPKeyDBRef			keydb,
		LPSTR				pszFolder)
{
	PFLFileSpecRef		filespec		= NULL;
	PGPShareFileRef		sharefile		= NULL;
	PGPShareRef			sharesHolder	= NULL;
	PGPOptionListRef	encodeOptions	= NULL;
	PGPError			err				= kPGPError_NoErr;
	INT					iModifier		= 1;

	CHAR				szPath[MAX_PATH];
	CHAR				szModifier[16];
	CHAR				sz1[32];
	CHAR				sz2[kPGPMaxUserIDSize + 32];
	PGPKeyDBObjRef		key;

	// create file name and filespec
	err = sCreateFilePathFromUserName (pszFolder, pshs->szUserID,
			uUserNumber, pshs->uShares, "", szPath, sizeof(szPath));
	if (IsPGPError (err)) goto SaveFileCleanup;

	// check for pre-existence of file
	while (GetFileAttributes (szPath) != 0xFFFFFFFF)
	{
		iModifier++;
		if (iModifier > MAX_MODIFIERS)
		{
			err = kPGPError_CantOpenFile;
			goto SaveFileCleanup;
		}
		wsprintf (szModifier, " (%i)", iModifier);
		err = sCreateFilePathFromUserName (pszFolder, pshs->szUserID,
				uUserNumber, pshs->uShares, 
				szModifier, szPath, sizeof(szPath));
		if (IsPGPError (err))
			goto SaveFileCleanup;
	}

	err = PFLNewFileSpecFromFullPath (
			PGPPeekContextMemoryMgr (g_context), szPath, &filespec);
	if (IsPGPError (err))
		goto SaveFileCleanup;

	err = PFLFileSpecCreate (filespec);
	if (IsPGPError (err))
		goto SaveFileCleanup;

	err = PGPNewShareFile (filespec, &sharefile);
	if (IsPGPError (err))
		goto SaveFileCleanup;

	err = PGPSetShareFileUserID (sharefile, pshs->szUserID);
	if (IsPGPError (err))
		goto SaveFileCleanup;

	err = PGPSplitShares (sharesTotal, pshs->uShares, &sharesHolder);
	if (IsPGPError (err))
		goto SaveFileCleanup;

	// if this shareholder has public key, use it
	if (pshs->bPublicKey)
	{
		err = PGPSetShareFileOwnerKeyID (sharefile, pshs->keyid);
		if (IsPGPError (err))
			goto SaveFileCleanup;

		err = PGPFindKeyByKeyID (keydb, &(pshs->keyid), &key);
		if (IsPGPError (err))
		{
			LoadString (g_hinst, IDS_CAPTIONALERT, sz1, sizeof(sz1));
			LoadString (g_hinst, IDS_SHAREKEYGONE, sz2, sizeof(sz2));
			lstrcat (sz2, pshs->szUserID);
			PGPclMessageBoxUTF8 (NULL, sz2, sz1, 
					MB_OK|MB_ICONEXCLAMATION|MB_TOPMOST);
			err = kPGPError_UserAbort;
			goto SaveFileCleanup;
		}

		err = PGPBuildOptionList (g_context, &encodeOptions,
				PGPOEncryptToKeyDBObj (g_context, key),
				PGPOLastOption (g_context));
		if (IsPGPError (err))
			goto SaveFileCleanup;
	}

	// there is no public key for this shareholder
	else
	{
		err = PGPBuildOptionList (g_context, &encodeOptions,
				PGPOConventionalEncrypt (g_context,
					PGPOPassphrase (g_context, pshs->pszPassphrase),
					PGPOLastOption (g_context)),
				PGPOLastOption (g_context));
		if (IsPGPError (err))
			goto SaveFileCleanup;
	}

	err = PGPCopySharesToFile (g_context,
			sharefile, encodeOptions, sharesHolder);
	if (IsPGPError (err))
		goto SaveFileCleanup;

	err = PGPSaveShareFile (sharefile);

SaveFileCleanup:
	if (encodeOptions != NULL)
		PGPFreeOptionList (encodeOptions);
	if (sharesHolder != NULL)
		PGPFreeShares (sharesHolder);
	if (sharefile != NULL)
		PGPFreeShareFile (sharefile);
	if (filespec != NULL)
		PFLFreeFileSpec (filespec);

	return err;
}


//	_______________________________________________
//
//	Split key progress dialog message procedure

static BOOL CALLBACK
sSplitKeyProgressDlgProc (HWND hwnd,
					   UINT uMsg,
					   WPARAM wParam,
					   LPARAM lParam)
{
	SPLITKEYPROGRESSSTRUCT*	pskps;

	switch (uMsg) {

	case WM_INITDIALOG :
		// save address of struct
		SetWindowLong (hwnd, GWL_USERDATA, lParam);
		pskps = (SPLITKEYPROGRESSSTRUCT*)lParam;
		UTF8EditInit (GetDlgItem (hwnd, IDC_PROGRESSTEXT));
		SendDlgItemMessage (hwnd, IDC_PROGRESSBAR,
				PBM_SETRANGE, 0, MAKELPARAM(0, pskps->iNumSteps));
		SendDlgItemMessage (hwnd, IDC_PROGRESSBAR,
				PBM_SETPOS, 0, 0);
		return TRUE;

	case WM_DESTROY :
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_PROGRESSTEXT));
		return FALSE;

	case WM_APP :
		SendDlgItemMessage (hwnd, IDC_PROGRESSBAR,
				PBM_SETPOS, wParam, 0);
		UTF8EditSetText (GetDlgItem (hwnd, IDC_PROGRESSTEXT), (LPSTR)lParam);
		break;

	}
	return FALSE;
}


//	___________________________________________________
//
//  Change passphrase of key and all subkeys

static PGPError
sChangeKeyPhrase (
		PGPKeyDBObjRef	key,
		LPSTR			szOld,
		PGPByte*		pPasskeyOld,
		PGPSize			sizePasskeyOld,
		PGPByte*		pPasskey,
		PGPSize			sizePasskey)
{
	PGPKeyIterRef	keyiter;
	PGPKeyDBObjRef	subkey;
	PGPError		err;

	if (szOld)
	{
		err = PGPChangePassphrase (key,
				PGPOPassphrase (g_context, szOld),
				PGPOPasskeyBuffer (g_context, pPasskey, sizePasskey),
				PGPOLastOption (g_context));
	}
	else if (sizePasskeyOld > 0)
	{
		err = PGPChangePassphrase (key,
				PGPOPasskeyBuffer (g_context, pPasskeyOld, sizePasskeyOld),
				PGPOPasskeyBuffer (g_context, pPasskey, sizePasskey),
				PGPOLastOption (g_context));
	}
	else
	{
		err = PGPChangePassphrase (key,
				PGPOPassphrase (g_context, ""),
				PGPOPasskeyBuffer (g_context, pPasskey, sizePasskey),
				PGPOLastOption (g_context));
	}
	if (IsPGPError (err))
		return err;

	PGPNewKeyIterFromKeyDB (PGPPeekKeyDBObjKeyDB (key), &keyiter);
	PGPKeyIterSeek (keyiter, key);
	PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_SubKey, &subkey);
	while (subkey)
	{
		if (szOld)
		{
			err = PGPChangePassphrase (subkey,
					PGPOPassphrase (g_context, szOld),
					PGPOPasskeyBuffer (g_context, pPasskey, sizePasskey),
					PGPOLastOption (g_context));
		}
		else if (sizePasskeyOld > 0)
		{
			err = PGPChangePassphrase (subkey,
					PGPOPasskeyBuffer (g_context,
							pPasskeyOld, sizePasskeyOld),
					PGPOPasskeyBuffer (g_context, pPasskey, sizePasskey),
					PGPOLastOption (g_context));
		}
		else
		{
			err = PGPChangePassphrase (subkey,
					PGPOPassphrase (g_context, ""),
					PGPOPasskeyBuffer (g_context, pPasskey, sizePasskey),
					PGPOLastOption (g_context));
		}
		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_SubKey, &subkey);
	}
	PGPFreeKeyIter (keyiter);

	return err;
}


//	_______________________________________________
//
//  browse folder callback function used to set initial folder location

static INT CALLBACK 
sBrowseCallbackProc (
		HWND	hwnd, 
		UINT	uMsg, 
		LPARAM	lParam, 
		LPARAM	lpData)
{
	if (uMsg == BFFM_INITIALIZED)
	{
		CHAR	sz[MAX_PATH];
		UINT	ulen;

		PGPclGetPath (kPGPclDefaultSaveAsFolder, sz, sizeof(sz));
		ulen = lstrlen (sz);

		if (sz[ulen-1] == '\\')
			sz[ulen-1] = '\0';

		SendMessage (hwnd, BFFM_SETSELECTION, TRUE, (LPARAM)sz);
	}

	return 0;
}


//	_______________________________________________
//
//  split the key

static BOOL
sSplitKey (
		PSPLITKEYSTRUCT psks)
{
	PGPShareRef			shares				= NULL;
	PGPSize				sizePasskey			= 0;
	PGPByte*			pPasskey			= NULL;
	LPSTR				pszPhraseKeyToSplit	= NULL;
	PGPByte*			pPasskeyToSplit		= NULL;
	PGPSize				sizePasskeyToSplit	= 0;
	CHAR				szEmptyString[]		= {""};
	HWND				hwndProgress		= NULL;
	BOOL				bRetVal				= TRUE;

	PGPError				err;
	PGPKeyDBObjRef			keyToSplit;
	BROWSEINFO				bi;
	LPITEMIDLIST			pidl;
	LPMALLOC				pMalloc;
	CHAR					szFolder[MAX_PATH];
	CHAR					sz[kPGPMaxUserIDSize + 32];
	INT						iItem;
	INT						iNumItems;
	LV_ITEM					lvI;
	PSHAREHOLDERSTRUCT		pshs;
	HCURSOR					hcursorOld;
	SPLITKEYPROGRESSSTRUCT	skps;

	// get keyref from keyring
	err = PGPFindKeyByKeyID (psks->ppks->keydbMain, 
			&(psks->keyidToSplit), &keyToSplit);
	if (IsPGPError (err))
	{
		PKMessageBox (psks->hwndDlg, IDS_CAPTIONALERT,
				IDS_SPLITKEYGONE, MB_OK|MB_ICONEXCLAMATION);
		bRetVal = FALSE;
		goto SplitKeyCleanup;
	}

	// get task allocator
	if (SHGetMalloc(&pMalloc) != NOERROR)
		return FALSE;

	// get prompt string
	LoadString (g_hinst, IDS_SAVESHARES, sz, sizeof(sz));

	// initialize structure
	bi.hwndOwner = psks->hwndDlg;
	bi.pidlRoot = NULL;
	bi.pszDisplayName = szFolder;
	bi.lpszTitle = sz;
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.lpfn = sBrowseCallbackProc;
	bi.lParam = 0;

	// allow user to browse
	pidl = SHBrowseForFolder (&bi);
	if (pidl == NULL)
		return FALSE;
	SHGetPathFromIDList (pidl, szFolder);
	pMalloc->lpVtbl->Free(pMalloc, pidl);

	// get passphrase of key to split
	LoadString (g_hinst, IDS_SPLITKEYPHRASEPROMPT, sz, sizeof(sz));
	err = PGPclGetKeyPhrase (g_context, g_tlscontext,
			psks->hwndDlg, sz, psks->ppks->keydbMain, keyToSplit,
			&pszPhraseKeyToSplit, &pPasskeyToSplit, &sizePasskeyToSplit);
	PGPclErrorBox (NULL, err);

	if (IsPGPError (err))
	{
		bRetVal = FALSE;
		goto SplitKeyCleanup;
	}

	// Pre-7.0 versions of PGP set the locking algorithm incorrectly
	// at keygen time.  The following hack of changing the passphrase to 
	// itself will relock the key with the correct algorithm, thus 
	// preventing problems when we actually split the key.
	// If PGPclGetKeyPhrase returned a passkey then the key has already
	// been split and the user may already be screwed -- nothing we
	// can do about that now.
	if (pszPhraseKeyToSplit)
	{
		err = PGPChangePassphrase (keyToSplit, 
				PGPOPassphrase (g_context, pszPhraseKeyToSplit),
				PGPOPassphrase (g_context, pszPhraseKeyToSplit),
				PGPOLastOption (g_context));

		if (IsPGPError (err))
		{
			bRetVal = FALSE;
			goto SplitKeyCleanup;
		}
	}

	// make sure that this is what user wants to do
	if (PKMessageBox (psks->hwndDlg, IDS_CAPTIONWARNING, 
			IDS_SPLITKEYCONFIRMATION, MB_YESNO|MB_ICONEXCLAMATION) == IDNO)
	{
		bRetVal = FALSE;
		goto SplitKeyCleanup;
	}

	// post progress dialog
	iNumItems = ListView_GetItemCount (psks->hwndList);
	skps.iNumSteps = iNumItems +2;
	hwndProgress = CreateDialogParam (g_hinst,
			MAKEINTRESOURCE(IDD_SPLITKEYPROGRESS),
			psks->hwndDlg, sSplitKeyProgressDlgProc,
			(LPARAM)&skps);
	LoadString (g_hinst, IDS_CREATINGSHARES, sz, sizeof(sz));
	SendMessage (hwndProgress, WM_APP, 1, (LPARAM)sz);

	// create the shares
	err = PGPCreateShares (g_context, keyToSplit,
			psks->uThreshold, psks->uTotalShares, &shares);

	if (IsPGPError (PGPclErrorBox (NULL, err)))
	{
		bRetVal = FALSE;
		goto SplitKeyCleanup;
	}

	// get the passkey from the shares
	err = PGPGetPasskeyFromShares (shares, &pPasskey, &sizePasskey);
	if (IsntPGPError (PGPclErrorBox (NULL, err)))
	{
		hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));

		// save share file for each item in listview
		for (iItem=0; iItem<iNumItems; iItem++)
		{
			lvI.mask = LVIF_PARAM;
			lvI.iItem = iItem;
			lvI.iSubItem = 0;
			ListView_GetItem(psks->hwndList, &lvI);

			// update progress dialog
			pshs = (PSHAREHOLDERSTRUCT)lvI.lParam;
			LoadString (g_hinst, IDS_SAVINGSHARES, sz, sizeof(sz));
			lstrcat (sz, pshs->szUserID);
			SendMessage (hwndProgress, WM_APP, iItem+2, (LPARAM)sz);

			err = sSaveSharesToFile (pshs, iItem+1,
					shares, psks->ppks->keydbMain, szFolder);
			if (IsPGPError (err))
				break;
		}

		// update progress dialog
		LoadString (g_hinst, IDS_SPLITTINGKEY, sz, sizeof(sz));
		SendMessage (hwndProgress, WM_APP, iNumItems+2, (LPARAM)sz);

		SetCursor (hcursorOld);
		PGPclErrorBox (NULL, err);

		if (IsPGPError (err))
		{
///			delete files;
		}
		else
		{
			err = sChangeKeyPhrase (keyToSplit, pszPhraseKeyToSplit, 
					pPasskeyToSplit, sizePasskeyToSplit, 
					pPasskey, sizePasskey);
			PGPclErrorBox (NULL, err);
			PGPclKeyListUpdateTree (psks->ppks->hKL, 
					kPGPclKeyListUpdateObject, keyToSplit, FALSE);
			PGPclKeyListSetTree (psks->ppks->hKL,
					kPGPclSelectSpecifiedOnly, keyToSplit);
		}
	}

	// cleanup
SplitKeyCleanup :
	if (hwndProgress)
		DestroyWindow (hwndProgress);
	if (pszPhraseKeyToSplit)
		PGPclFreePhrase (pszPhraseKeyToSplit);
	if (pPasskeyToSplit)
		PGPclFreePasskey (pPasskeyToSplit, sizePasskeyToSplit);
	if (pPasskey)
		PGPFreeData (pPasskey);
	if (shares != NULL)
		PGPFreeShares (shares);

	return (bRetVal);
}


//	_______________________________________________
//
//	Split key dialog message procedure

static BOOL CALLBACK
sSplitKeyDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	PSPLITKEYSTRUCT		psks;
	CHAR				sz[8];
	NMHDR*				pnmh;
	UINT				u;

	switch (uMsg) {

	case WM_INITDIALOG :
	{
		CHAR	szTitle[kPGPMaxUserIDSize + 32];

		// save address of struct
		SetWindowLong (hwnd, GWL_USERDATA, lParam);
		psks = (PSPLITKEYSTRUCT)lParam;
		psks->hwndDlg = hwnd;

		// if we have a function to call to add hwnd to list, then call it
		PKAddRemoveWindowList (hwnd, TRUE, TRUE, FALSE);

		// initialize shareholder list
		psks->hwndList = GetDlgItem (hwnd, IDC_SHAREHOLDERS);
		sInitKeyList (psks);

		// initialize name of key to split
		UTF8EditInit (GetDlgItem (hwnd, IDC_KEYTOSPLIT));
		UTF8EditSetText (GetDlgItem (hwnd, IDC_KEYTOSPLIT), psks->szUserIDToSplit);

		UTF8EditInit (GetDlgItem (hwnd, IDC_CURRENTSHAREHOLDER));


		LoadString (g_hinst, IDS_SPLITKEYTITLE, szTitle, sizeof(szTitle));
		SetWindowText (hwnd, szTitle);

		// limit number of shares
		SendDlgItemMessage (hwnd, IDC_SHARES, EM_SETLIMITTEXT,
				(WPARAM)MAX_SHARES_LEN, 0);
		SendDlgItemMessage (hwnd, IDC_THRESHOLD, EM_SETLIMITTEXT,
				(WPARAM)MAX_SHARES_LEN, 0);

		// initialize spin controls
		SendDlgItemMessage (hwnd, IDC_SHARESSPIN, UDM_SETRANGE,
				0, (LPARAM)MAKELONG (MAX_SHARES, 1));
		SetDlgItemText (hwnd, IDC_SHARES, "");
		EnableWindow (GetDlgItem (hwnd, IDC_SHARES), FALSE);
		EnableWindow (GetDlgItem (hwnd, IDC_SHARESSPIN), FALSE);

		SendDlgItemMessage (hwnd, IDC_THRESHOLDSPIN, UDM_SETRANGE,
				0, (LPARAM)MAKELONG (MAX_SHARES, 1));
		SendDlgItemMessage (hwnd, IDC_THRESHOLDSPIN, UDM_SETPOS,
				0, (LPARAM)MAKELONG (psks->uThreshold, 0));

		// initialize total number of shares
		SetDlgItemInt (hwnd, IDC_TOTALSHARES, psks->uTotalShares, FALSE);

		// "split key" and "remove" buttons initially disabled
		EnableWindow (GetDlgItem (hwnd, IDC_REMOVESHAREHOLDER), FALSE);
		EnableWindow (GetDlgItem (hwnd, IDOK), FALSE);

		// initialize drag/drop
		psks->pDropTarget = PKCreateDropTarget (
				psks->hwndList, PK_IDROP_SPLITLIST, psks);
		CoLockObjectExternal ((IUnknown*)psks->pDropTarget, TRUE, TRUE);
		RegisterDragDrop (psks->hwndList, psks->pDropTarget);
		PKEnableDropTarget (psks->pDropTarget, TRUE);

		psks->hUTF8Font = SetUTF8Font(GetDlgItem(hwnd, IDC_SHAREHOLDERS));
		return TRUE;
	}

	case WM_DRAWITEM:
    {
        KeyDrawItem((LPDRAWITEMSTRUCT) lParam);
        return TRUE;
    }

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aSplitIds);

	case WM_NOTIFY :
		pnmh = (NMHDR*)lParam;
		switch (pnmh->code) {

		case LVN_ITEMCHANGED :
			if (pnmh->idFrom == IDC_SHAREHOLDERS)
			{
				INT		iIndex;

				psks = (PSPLITKEYSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
				iIndex = ListView_GetNextItem (
						psks->hwndList, -1, LVNI_SELECTED);

				if (iIndex > -1)
				{
					LV_ITEM				lvI;
					PSHAREHOLDERSTRUCT	pshs;

					lvI.mask = LVIF_PARAM;
					lvI.iItem = iIndex;
					lvI.iSubItem = 0;
					ListView_GetItem(psks->hwndList, &lvI);

					pshs = (PSHAREHOLDERSTRUCT)lvI.lParam;
					psks->pshsCurrent = pshs;
					psks->iIndexCurrent = iIndex;

					UTF8EditSetText (GetDlgItem (hwnd, IDC_CURRENTSHAREHOLDER), pshs->szUserID);
					SendDlgItemMessage (hwnd, IDC_SHARESSPIN, UDM_SETPOS,
							0, (LPARAM)MAKELONG (pshs->uShares, 0));

					EnableWindow (GetDlgItem (hwnd, IDC_SHARES), TRUE);
					EnableWindow (GetDlgItem (hwnd, IDC_SHARESSPIN), TRUE);
					EnableWindow (
							GetDlgItem (hwnd, IDC_REMOVESHAREHOLDER), TRUE);
				}
				else
				{
					psks->iIndexCurrent = -1;
					psks->pshsCurrent = NULL;
					SetDlgItemText (hwnd, IDC_SHARES, "");
					UTF8EditSetText (GetDlgItem (hwnd, IDC_CURRENTSHAREHOLDER), "");
					EnableWindow (GetDlgItem (hwnd, IDC_SHARES), FALSE);
					EnableWindow (GetDlgItem (hwnd, IDC_SHARESSPIN), FALSE);
					EnableWindow (
							GetDlgItem (hwnd, IDC_REMOVESHAREHOLDER), FALSE);
				}
			}
			break;
		}
		break;

	case WM_DESTROY :
		psks = (PSPLITKEYSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);

		// terminate drag/drop
		RevokeDragDrop (psks->hwndList);
		PKReleaseDropTarget (psks->pDropTarget);
		CoLockObjectExternal ((IUnknown*)psks->pDropTarget, FALSE, TRUE);

		// call function to remove hwnd from list
		PKAddRemoveWindowList (hwnd, FALSE, FALSE, FALSE);

		// destroy data objects
		ListView_SetImageList (psks->hwndList, NULL, LVSIL_SMALL);
		sDestroyShareHolders (psks);
		sDestroySplitKeyStruct (psks);
		DeleteObject(psks->hUTF8Font);

		// destroy edit controls
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_KEYTOSPLIT));
		UTF8EditDestroy (GetDlgItem (hwnd, IDC_CURRENTSHAREHOLDER));
		break;

	case WM_COMMAND:

		switch (LOWORD(wParam)) {
		case IDCANCEL :
			DestroyWindow (hwnd);
			break;

		case IDOK :
			psks = (PSPLITKEYSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			EnableWindow (psks->ppks->hwndMain, FALSE);
			if (sSplitKey (psks))
			{
				PKKeyDBModified (psks->ppks, PK_MOD_INDEX_0);
				EnableWindow (psks->ppks->hwndMain, TRUE);
				SetParent (hwnd, psks->ppks->hwndMain);
				DestroyWindow (hwnd);
			}
			else
				EnableWindow (psks->ppks->hwndMain, TRUE);
			break;

		case IDHELP :
			PGPclHtmlHelp (NULL, kPGPclHelpContext, 0, 0, 
					(char*)kPGPclHelpFile, (DWORD*)IDH_PGPPK_SPLITDIALOG);
			break;

		case IDC_ADDSHAREHOLDER :
			psks = (PSPLITKEYSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			EnableWindow (psks->ppks->hwndMain, FALSE);
			sAddShareHolderToList (hwnd, psks);
			EnableWindow (psks->ppks->hwndMain, TRUE);
			SetFocus (GetDlgItem (hwnd, IDC_ADDSHAREHOLDER));
			break;

		case IDC_REMOVESHAREHOLDER :
			psks = (PSPLITKEYSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			sRemoveShareHolderFromList (hwnd, psks);
			psks->iIndexCurrent = -1;
			psks->pshsCurrent = NULL;
			SetDlgItemText (hwnd, IDC_SHARES, "");
			UTF8EditSetText (GetDlgItem (hwnd, IDC_CURRENTSHAREHOLDER), "");
			EnableWindow (GetDlgItem (hwnd, IDC_SHARES), FALSE);
			EnableWindow (GetDlgItem (hwnd, IDC_REMOVESHAREHOLDER), FALSE);
			break;

		case IDC_THRESHOLD :
			switch (HIWORD(wParam)) {
			case EN_CHANGE :
				psks = (PSPLITKEYSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
				if (psks)
				{
					GetDlgItemText (hwnd, IDC_THRESHOLD, sz, sizeof(sz));
					if (lstrcmp (sz, "0"))
					{
						UINT u = GetDlgItemInt (hwnd,
								IDC_THRESHOLD, NULL, FALSE);
						if (u != 0)
						{
							psks->uThreshold = u;
							if (psks->uTotalShares >= psks->uThreshold)
								EnableWindow (GetDlgItem (hwnd, IDOK), TRUE);
							else
								EnableWindow (GetDlgItem (hwnd, IDOK), FALSE);
						}
					}
				}
				break;

			case EN_KILLFOCUS :
				u = GetDlgItemInt (hwnd, IDC_THRESHOLD, NULL, FALSE);
				if (u == 0) u = 1;
				SetDlgItemInt (hwnd, IDC_THRESHOLD, u, FALSE);
				break;
			}
			break;

		case IDC_SHARES :
			psks = (PSPLITKEYSTRUCT)GetWindowLong (hwnd, GWL_USERDATA);
			switch (HIWORD(wParam)) {
			case EN_CHANGE :
				if (psks)
				{
					PSHAREHOLDERSTRUCT	pshs;
					GetDlgItemText (hwnd, IDC_SHARES, sz, sizeof(sz));
					pshs = psks->pshsCurrent;
					if (pshs && (lstrcmp (sz, "0")))
					{
						UINT u = GetDlgItemInt (hwnd, IDC_SHARES, NULL, FALSE);
						if (u != 0)
						{
							psks->uTotalShares -= pshs->uShares;
							pshs->uShares = u;
							psks->uTotalShares += pshs->uShares;
							SetDlgItemInt (hwnd,
									IDC_TOTALSHARES, psks->uTotalShares, FALSE);

							wsprintf (sz, "%i", pshs->uShares);
							ListView_SetItemText (psks->hwndList,
									psks->iIndexCurrent, 1, sz);

							if ((psks->uTotalShares >= psks->uThreshold) &&
								(psks->uTotalShares <= MAX_TOTAL_SHARES))
							{
								EnableWindow (GetDlgItem (hwnd, IDOK), TRUE);
							}
							else
								EnableWindow (GetDlgItem (hwnd, IDOK), FALSE);
						}
					}
				}
				break;

			case EN_KILLFOCUS :
				u = GetDlgItemInt (hwnd, IDC_SHARES, NULL, FALSE);
				if (u == 0)
				{
					u = 1;
					MessageBeep (MB_ICONASTERISK);
				}
				else if (psks->uTotalShares-psks->pshsCurrent->uShares+u >
						MAX_TOTAL_SHARES)
				{
					u = MAX_TOTAL_SHARES - (psks->uTotalShares -
							psks->pshsCurrent->uShares);
					MessageBeep (MB_ICONASTERISK);
				}

				SetDlgItemInt (hwnd, IDC_SHARES, u, FALSE);
				break;
			}
			break;
		}
		return TRUE;
	}
	return FALSE;
}


//	_______________________________________________
//
//  Split selected key

BOOL
PKSplitKey (
		PPGPKEYSSTRUCT ppks)
{
	PSPLITKEYSTRUCT			psks			= NULL;
	BOOL					bOK;
	PGPKeyDBObjRef			key;
	PGPKeyID				keyidToSplit;
	HWND					hwnd;
	PGPUInt32				u;

	// find existing dialog struct for this key
	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &key, NULL, NULL);
	PGPGetKeyID (key, &keyidToSplit);

	psks = ppks->pSplitKeyDlgList;
	while (psks)
	{
		if (PGPCompareKeyIDs (&keyidToSplit, &(psks->keyidToSplit)) == 0)
			break;
		psks = psks->next;
	}

	// if dialog already exists, move to foreground
	if (psks)
	{
		SetForegroundWindow (psks->hwndDlg);
		bOK = TRUE;
	}

	// otherwise create new dialog
	else
	{
		psks = pkAlloc (sizeof (SPLITKEYSTRUCT));
		if (psks)
		{
			// initialize struct
			psks->ppks				= ppks;
			psks->uTotalShares		= 0;
			psks->uThreshold		= 2;
			psks->iIndexCurrent		= -1;
			psks->pshsCurrent		= NULL;
			psks->pDropTarget		= NULL;

			psks->pHeadOfList = (PSPLITKEYSTRUCT*)&(ppks->pSplitKeyDlgList);

			PGPGetKeyID (key, &(psks->keyidToSplit));
			PGPclGetPrimaryUserIDNameUTF8 (key, psks->szUserIDToSplit, 
					sizeof(psks->szUserIDToSplit), &u);

			// create modeless dialog
			hwnd = CreateDialogParam (g_hinst,
					MAKEINTRESOURCE (IDD_SPLITKEY),
					NULL, sSplitKeyDlgProc, (LPARAM)psks);

			// make it "floating"
			SetWindowPos (hwnd, HWND_TOPMOST,
					0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE);

			// add dialog struct to list
			psks->next = ppks->pSplitKeyDlgList;
			ppks->pSplitKeyDlgList = psks;
			bOK = TRUE;
		}
		else
			bOK = FALSE;
	}

	return bOK;
}


//	_______________________________________________
//
//  Do any split key dialogs exist?

BOOL
PKExistSplitKeyDialog (
		PPGPKEYSSTRUCT ppks)
{
	if (IsntNull (ppks->pSplitKeyDlgList))
		return TRUE;
	else
		return FALSE;
}
