/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	PKToken.c - GUI routines for smart card support
	
	$Id: PKToken.c,v 1.18 2002/08/06 20:09:46 dallen Exp $
____________________________________________________________________________*/
#include "pgpPFLConfig.h"	/* or pgpConfig.h in the CDK */

// project header files
#include "PGPkeysx.h"

// typedefs
typedef struct {
	PGPKEYSSTRUCT*		ppks;
	HWND				hwndParent;
	BOOL				bNeedsCentering;
	BOOL				bModified;
	PGPContextRef		context;
	PGPtlsContextRef	tlscontext;
	PGPKeySetRef		keysetToDisplay;
	HKEYLIST			hKL;
	PGPclKeyListConfig	klConfig;
	HWND				hwndTree;
	UINT				uFlags;
} TOKENDLGSTRUCT;


// external global variables
extern HINSTANCE		g_hinst;
extern PGPContextRef	g_context;
extern PGPtlsContextRef	g_tlscontext;

#define IDC_TOKENKEYLIST		0x0101
#define YOFFSET					6

static	HWND	s_hwndProperties	= NULL;
static	HWND	s_hwndContents		= NULL;

static DWORD aTokenIds[] = {			// Help IDs
	IDC_MANUFACTURER,		IDH_PGPPKTOKEN_MANUFACTURER,
	IDC_MODEL,				IDH_PGPPKTOKEN_MODEL,
	IDC_SERIAL,				IDH_PGPPKTOKEN_SERIALNUMBER,
	IDC_KEYSUPPORT,			IDH_PGPPKTOKEN_KEYSUPPORT,
	IDC_PINLENGTH,			IDH_PGPPKTOKEN_PINLENGTH,
	IDC_NUMPRIVKEYS,		IDH_PGPPKTOKEN_NUMPRIVKEYS,
	IDC_TOKENKEYLIST,		IDH_PGPPKTOKEN_CONTENTSKEYS,
	IDC_WIPETOKEN,			IDH_PGPPKTOKEN_WIPECONTENTS,
	0,0
};


// prototypes
PGPError PGPFormatToken ( 
		PGPContextRef	context, 
		PGPUInt32		tokNumber,
		PGPByte const * adminPin, 
		PGPSize			adminPinLen, 
		PGPByte const * newUserPin, 
		PGPSize			newUserPinLen);


//	____________________________________
//
//  Look for keys on token

PGPBoolean 
sGetKeysOnToken (
		PGPKeyDBRef		keydb,
		PGPKeySetRef*	pkeyset) 
{
	PGPKeyIterRef	keyiter;
	PGPKeyDBObjRef	key;
	PGPBoolean		b, bOnToken;

	if (IsntNull (pkeyset))
		PGPNewEmptyKeySet (keydb, pkeyset);

	PGPNewKeyIterFromKeySet (PGPPeekKeyDBRootKeySet (keydb), &keyiter);

	bOnToken = FALSE;
	PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);

	while (key) 
	{
		PGPGetKeyDBObjBooleanProperty (key, 
				kPGPKeyProperty_IsOnToken, &b);

		if (b)
		{
			bOnToken = TRUE;
			if (IsntNull (pkeyset))
				PGPAddKey (key, *pkeyset);
			else
				break;
		}

		PGPKeyIterNextKeyDBObj (keyiter, kPGPKeyDBObjType_Key, &key);
	}

	PGPFreeKeyIter (keyiter);

	return bOnToken;
}


//	___________________________________________________
//
//  query user then wipe token

static PGPError
sWipeToken (
		HWND			hwnd,
		PGPKEYSSTRUCT*	ppks)
{
	PGPError	err				= kPGPError_UserAbort;
	CHAR*		pszPhrase		= NULL;
	CHAR*		pszAdminPhrase	= NULL;
	HCURSOR		hcursorOld;
	CHAR		sz[128];

	if (PKMessageBox (hwnd, IDS_CAPTIONWARNING, 
			IDS_CONFIRMWIPETOKEN, MB_OKCANCEL|MB_ICONWARNING) == IDOK)
	{
		LoadString (g_hinst, IDS_TOKENPHRASEPROMPT, sz, sizeof(sz));
		err = PGPclGetConventionalPhrase (g_context, 
				hwnd, sz, &pszPhrase);

		if (IsntPGPError (err))
		{
			hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
			if (IsntPGPError (err))
			{
				err = PGPWipeToken (g_context, 
						0, pszPhrase, lstrlen (pszPhrase));
			}
			SetCursor (hcursorOld);


			if (IsPGPError (err) && (err != kPGPError_BadPassphrase))
			{
				LoadString (g_hinst, IDS_TOKENWIPEFAILED, sz, sizeof(sz));
				if (PKMessageBox (hwnd, IDS_CAPTIONALERT, IDS_TOKENWIPEFAILED, 
						MB_ICONEXCLAMATION|MB_YESNO) == IDYES)
				{
					LoadString (g_hinst, 
							IDS_TOKENADMINPHRASEPROMPT, sz, sizeof(sz));

					err = PGPclGetConventionalPhrase (g_context, 
							hwnd, sz, &pszAdminPhrase);

					if (IsntPGPError (err))
					{
						if (IsntNull (pszPhrase))
							PGPclFreePhrase (pszPhrase);

						LoadString (g_hinst, 
								IDS_TOKENNEWPHRASEPROMPT, sz, sizeof(sz));

						err = PGPclGetConfirmationPhrase (g_context, hwnd, sz,
								kInvalidPGPKeyDBRef, 0, -1, &pszPhrase);
					}

					if (IsntPGPError (err))
					{
						hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
						if (IsntPGPError (err))
						{
							err = PGPFormatToken (g_context, 0, 
									pszAdminPhrase, lstrlen (pszAdminPhrase),
									pszPhrase, lstrlen (pszPhrase));
						}
						SetCursor (hcursorOld);
					}
				}
				else
					err = kPGPError_UserAbort;
			}
		}
	}

	if (IsntNull (pszPhrase))
		PGPclFreePhrase (pszPhrase);

	if (IsntNull (pszAdminPhrase))
		PGPclFreePhrase (pszAdminPhrase);

	return err;
}


//	___________________________________________________
//
//  copy the selected key to the token

BOOL
PKSendToToken (
		PGPKEYSSTRUCT*	ppks)
{
	PGPError			err				= kPGPError_NoErr;
	char*				pszPhrase		= NULL;
	PGPByte*			pPasskey		= NULL;
	char*				pszPIN			= NULL;
	PGPSize				sizePasskey		= 0;
	PGPByte*			pExport			= NULL;
	PGPKeyDBRef			keydb			= kInvalidPGPKeyDBRef;
	const				tokenNum		= 0; // always token 0

	PGPKeyDBObjRef		key;
	PGPSize				sizeExport;
	HCURSOR				hcursorOld;
	char				szPrompt[64];

	PKMessageBox (ppks->hwndMain, 
			IDS_CAPTIONINFO, IDS_SENDTOTOKENINFO, MB_OK|MB_ICONINFORMATION);

	PGPclKeyListGetSelectionInfo (ppks->hKL, NULL, &key, NULL, NULL);

	err = PGPclGetKeyPhrase (g_context, g_tlscontext, ppks->hwndMain,
			NULL, ppks->keydbMain, key, 
			&pszPhrase, &pPasskey, &sizePasskey); CKERR;

	LoadString (g_hinst, IDS_TOKENPHRASEPROMPT, szPrompt, sizeof(szPrompt));
	err = PGPclGetConventionalPhrase (g_context, 
			ppks->hwndMain, szPrompt, &pszPIN); CKERR;

	hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));

	{
		PGPTokenInfo pti = { sizeof(pti) };
		char *conts1=NULL, *conts2=NULL;
		int conts1_n=0, conts2_n=0;
		char CSP[80];

		PGPGetTokenInfo (g_context, tokenNum, &pti);
		PGPclIETokenProvToCSP( pti.manufacturerID, CSP, sizeof(CSP) );

		if( pti.numPrivKeys > 0 )  
			PGPclIEGetExistingKeyContainersRemote( CSP, &conts1, &conts1_n );

		// copy Master key
		err = PGPExport( g_context, 
            PGPOExportKeyDBObj( g_context, key ),
            PGPOOutputToken(g_context,tokenNum), 
			PGPOPassphrase (g_context, pszPhrase),
			PGPOPassphrase (g_context, pszPIN),
			PGPOLastOption (g_context));

		if( IsntPGPError(err) )  
			PGPclIEGetExistingKeyContainersRemote( CSP, &conts2, &conts2_n );

		// copy subkey
		if( IsntPGPError(err) ) {
			PGPUInt32 uVer=0;

			PGPGetKeyDBObjNumericProperty (key,
				kPGPKeyProperty_Version, &uVer);
			if( uVer == kPGPKeyVersion_V4 )   {
				err = PGPExport( g_context, 
					PGPOExportKeyDBObj( g_context, key ),
					PGPOOutputToken(g_context,tokenNum), 
					PGPOExportPrivateSubkeys(g_context, TRUE),
					PGPOPassphrase (g_context, pszPhrase),
					PGPOPassphrase (g_context, pszPIN),
					PGPOLastOption (g_context)); 
			}
		}

		if( IsntPGPError(err) )  {
			if( conts2 ) 
				PGPclProcessKeyConts( g_context, key, pszPhrase,
					CSP, &pti, conts1, conts1_n, conts2, conts2_n );
		}

		PGPclIEFreeMem( conts1 );
		PGPclIEFreeMem( conts2 );
	}

	SetCursor (hcursorOld);
	CKERR;

	if (PKMessageBox (ppks->hwndMain, IDS_CAPTIONINFO, 
			IDS_QUERYREMOVEKEY, MB_YESNO|MB_ICONINFORMATION) == IDYES)
	{
		err = PGPExport (g_context,
				PGPOExportKeyDBObj (g_context, key), 
				PGPOAllocatedOutputBuffer (g_context,
						&pExport, 0x40000000, &sizeExport),
				PGPOExportFormat (g_context, 
						kPGPExportFormat_Complete),
				PGPOLastOption (g_context));  CKERR;

		PGPclKeyListUpdateTree (ppks->hKL,
				kPGPclKeyListRemoveObject, key, FALSE);
		err = PGPDeleteKeyDBObj (key); CKERR;
		PKDeletePropertiesWindowForObject (key);
		PKKeyDBModified (ppks, PK_MOD_INDEX_0);
		PGPclKeyListSetTree (ppks->hKL, kPGPclDeselectAll, NULL);

		err = PGPImport (g_context, &keydb, 
				PGPOInputBuffer (g_context, pExport, sizeExport),
				PGPOInputFormat (g_context, kPGPInputFormat_PGP),
				PGPOLastOption (g_context)); CKERR;

		err = PGPclImportKeys (g_context, g_tlscontext, ppks->hwndMain,
				PGPPeekKeyDBRootKeySet (keydb), ppks->keydbMain,
				kPGPclNoImportDialog|kPGPclNoTrustWarningMessage);
	}

	PGPclKeyListReloadKeys (ppks->hKL, TRUE);

done:
	if (IsntNull (pszPIN))
		PGPclFreePhrase (pszPIN);
	if (IsntNull (pszPhrase))
		PGPclFreePhrase (pszPhrase);
	if (IsntNull (pPasskey))
		PGPclFreePasskey (pPasskey, sizePasskey);
	if (IsntNull (pExport))
		PGPFreeData (pExport);
	if (PGPKeyDBRefIsValid (keydb))
		PGPFreeKeyDB (keydb);

	if (IsPGPError (err))
	{
		if (err == kPGPError_BadPassphrase)
		{
			PKMessageBox (ppks->hwndMain, IDS_CAPTIONALERT, 
					IDS_BADTOKENPHRASE, MB_OK|MB_ICONEXCLAMATION);
		}
		else
			PGPclErrorBox (ppks->hwndMain, err);

		return FALSE;
	}
	return TRUE;
}


//	___________________________________________________
//
//  Put up appropriate context menu on basis of selection flags.
//  Called in response to right mouse click.
//
//	hWnd		= handle of parent window
//	uSelFlags	= bits indicate what types of objects are selected
//	x, y		= mouse position when right button clicked (screen coords)

static VOID 
sTokenContextMenu (
		HWND		hWnd, 
		HKEYLIST	hKL, 
		UINT		uSelFlags, 
		INT			x, 
		INT			y) 
{
	HMENU		hMC;
	HMENU		hMenuTrackPopup;

	switch (uSelFlags) {
	case kPGPclKeyList_Key :
		hMC = LoadMenu (g_hinst, MAKEINTRESOURCE (IDR_MENUTOKEN));
		break;

	default :
		hMC = NULL;
		break;
	}

	if (!hMC) 
		return;

  	hMenuTrackPopup = GetSubMenu (hMC, 0);
	TrackPopupMenu (hMenuTrackPopup, TPM_LEFTALIGN|TPM_RIGHTBUTTON,
			x, y, 0, hWnd, NULL);

	DestroyMenu (hMC);
}


//	_____________________________________________________
//
//  Message processing function for Token property sheet

static LRESULT WINAPI
sTokenPropsDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	TOKENDLGSTRUCT*			ptds;

	switch (uMsg) {

	case WM_INITDIALOG:
	{
		RECT		rc;
		HWND		hwndParent;

		SetWindowLong (hwnd, GWL_USERDATA, ((PROPSHEETPAGE*)lParam)->lParam);
		ptds = (TOKENDLGSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		s_hwndProperties = hwnd;

		if (ptds->bNeedsCentering) 
		{
			PGPclCenterWindowOnMonitor (GetParent (hwnd), kPGPclWindowMonitor);
			ptds->bNeedsCentering = FALSE;
		}

		// disable and hide cancel button; and move "OK" button over
		hwndParent = GetParent (hwnd);
		SendMessage (hwndParent, PSM_CANCELTOCLOSE, 0, 0);
		GetWindowRect (GetDlgItem (hwndParent, IDCANCEL), &rc);
		MapWindowPoints (NULL, hwndParent, (LPPOINT)&rc, 2);
		SetWindowPos (GetDlgItem (hwndParent, IDOK), NULL, rc.left,
				rc.top, rc.right-rc.left, rc.bottom-rc.top,
				SWP_NOZORDER);
		ShowWindow (GetDlgItem (hwndParent, IDCANCEL), SW_HIDE);

		// post message to load properties
		PostMessage (hwnd, WM_APP, TRUE, 0);

		return TRUE;
	}

	case WM_APP :
	{
		PGPTokenInfo	pti;
		CHAR			sz1[32], sz2[32], szFormat[32];

		ptds = (TOKENDLGSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		pti.size =  sizeof(pti);
		PGPGetTokenInfo (ptds->context, 0, &pti);

		SetDlgItemText (hwnd, IDC_MANUFACTURER, pti.manufacturerID);
		SetDlgItemText (hwnd, IDC_MODEL, pti.model);
		SetDlgItemText (hwnd, IDC_SERIAL, pti.serialNumber);

		// indicate supported keys
		sz1[0] = '\0';

///		if (pti.bDsa)
///		{
///		}

		if (pti.bRsa)
		{
			if (sz1[0])
				lstrcat (sz1, ", ");

			LoadString (g_hinst, IDS_RSA, sz2, sizeof(sz2));
			lstrcat (sz1, sz2);
			lstrcat (sz1, " (");

			if (pti.minRsaKeySize != pti.maxRsaKeySize)
			{
				LoadString (g_hinst, 
						IDS_SMARTCARDPREFIX, szFormat, sizeof(szFormat));
				wsprintf (sz2, szFormat, pti.minRsaKeySize);
				lstrcat (sz1, sz2);
			}
			LoadString (g_hinst, 
					IDS_SMARTCARDKEYSIZE, szFormat, sizeof(szFormat));
			wsprintf (sz2, szFormat, pti.maxRsaKeySize);
			lstrcat (sz1, sz2);

			lstrcat (sz1, ")");
		}

		if (!sz1[0])
			LoadString (g_hinst, IDS_NONE, sz1, sizeof(sz1));

		SetDlgItemText (hwnd, IDC_KEYSUPPORT, sz1);

		// indicate supported PIN length range
		sz1[0] = '\0';
		if (pti.minPinLen != pti.maxPinLen)
		{
			LoadString (g_hinst, 
					IDS_SMARTCARDPREFIX, szFormat, sizeof(szFormat));
			wsprintf (sz1, szFormat, pti.minPinLen);
		}
		LoadString (g_hinst, 
				IDS_SMARTCARDPINLENGTH, szFormat, sizeof(szFormat));
		wsprintf (sz2, szFormat, pti.maxPinLen);
		lstrcat (sz1, sz2);

		SetDlgItemText (hwnd, IDC_PINLENGTH, sz1);

		// report number of private keys on card
		SetDlgItemInt (hwnd, IDC_NUMPRIVKEYS, pti.numPrivKeys, FALSE);

		break;
	}

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aTokenIds);

	case WM_NOTIFY :
		switch (((NMHDR FAR *) lParam)->code) {

		case PSN_HELP :
			PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0,
					(char*)kPGPclHelpFile, 
					(DWORD*)IDH_PGPPKTOKEN_CONTENTSDIALOG);
			break;
		}
		break;
	}

	return FALSE;
}


//	_____________________________________________________
//
//  Message processing function for Token property sheet

static LRESULT WINAPI
sTokenContentsDlgProc (
		HWND	hwnd,
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	TOKENDLGSTRUCT*			ptds;

	switch (uMsg) {

	case WM_INITDIALOG:
	{
		RECT		rc;
		INT			iY;
		PGPPrefRef	prefref;

		SetWindowLong (hwnd, GWL_USERDATA, ((PROPSHEETPAGE*)lParam)->lParam);
		ptds = (TOKENDLGSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		s_hwndContents = hwnd;

		// create keymanager window
		GetWindowRect (GetDlgItem (hwnd, IDC_WIPETOKEN), &rc);
		ScreenToClient (hwnd, (LPPOINT)&rc);
		iY = rc.top;

		GetClientRect (hwnd, &rc);

		ptds->hKL = PGPclKeyListCreate (ptds->context, ptds->tlscontext,
				hwnd, IDC_TOKENKEYLIST, 
				rc.left, rc.top, rc.right-rc.left, iY-rc.top-YOFFSET,
				kPGPclKeyList_SingleSelection|kPGPclKeyList_ShowSelection,
				&ptds->hwndTree);

		// configure key manager window
		ptds->klConfig.keyserver.protocol = kPGPKeyServerProtocol_Invalid;
		ptds->klConfig.hwndStatusBar = NULL;
		ptds->klConfig.uOptions = 
				kPGPclKeyList_SelectKeysOnly | kPGPclKeyList_ReadOnly;

		// get those options which are based on prefs
		if (IsntPGPError (PGPclPeekClientLibPrefRefs (&prefref, NULL)))
		{
			PGPBoolean	b;

			PGPGetPrefBoolean (prefref, kPGPPrefDisplayMarginalValidity, &b);
			if (!b) 
				ptds->klConfig.uOptions |= kPGPclKeyList_GreenDotMode;
			PGPGetPrefBoolean (prefref, kPGPPrefMarginalIsInvalid, &b);
			if (b) 
				ptds->klConfig.uOptions |= kPGPclKeyList_MarginalIsInvalid;
		}

		ptds->klConfig.uMask = kPGPclKeyList_ConfigureAll;
		PGPclKeyListConfigure (ptds->hKL, &ptds->klConfig);

		// post message to load keyset
		PostMessage (hwnd, WM_APP, TRUE, 0);

		return TRUE;
	}

	case WM_APP :
		ptds = (TOKENDLGSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		if (PGPKeySetRefIsValid (ptds->keysetToDisplay))
		{
			PGPclKeyListLoadKeys (ptds->hKL, NULL, NULL);
			PGPFreeKeySet (ptds->keysetToDisplay);
			ptds->keysetToDisplay = kInvalidPGPKeySetRef;
		}

		if ((BOOL)wParam)
		{
			sGetKeysOnToken (ptds->ppks->keydbMain, &(ptds->keysetToDisplay));

			PGPclKeyListLoadKeys (ptds->hKL, 
					ptds->ppks->keydbMain, ptds->keysetToDisplay);
		}

		InvalidateRect (hwnd, NULL, TRUE);
		UpdateWindow (hwnd);
		break;

	case WM_HELP :
	case WM_CONTEXTMENU :
		return PGPclHtmlHelp (hwnd, uMsg, wParam, lParam, 
				(char*)kPGPclHelpFile, aTokenIds);

	case WM_COMMAND :
		ptds = (TOKENDLGSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
		switch (LOWORD (wParam)) {
		case IDM_KEYPROPERTIES :
		{
			PGPKeyDBObjRef	key;

			PGPclKeyListGetSelectionInfo (ptds->hKL, NULL, &key, NULL, NULL);
			PGPclKeyDBObjProperties (ptds->context, ptds->tlscontext, 
					hwnd, key, ptds->ppks->keydbMain, 
					kPGPclProperties_Key, NULL);
			PostMessage (hwnd, WM_APP, TRUE, 0);
			break;
		}

		case IDC_WIPETOKEN :
		{
			PGPError	err;

			err = sWipeToken (hwnd, ptds->ppks);
			if (IsPGPError (err))
			{
				if (err == kPGPError_BadPassphrase)
				{
					PKMessageBox (hwnd, IDS_CAPTIONALERT, 
							IDS_BADTOKENPHRASE, MB_OK|MB_ICONEXCLAMATION);
				}
				else
					PGPclErrorBox (hwnd, err);
			}
			
			if (IsntPGPError (err))
			{
				ptds->bModified = TRUE;
				PostMessage (s_hwndProperties, WM_APP, 0, 0);
				PostMessage (hwnd, WM_APP, 0, 0);
			}
			break;
		}
		}
		break;

	case WM_DESTROY :
		ptds = (TOKENDLGSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);

		// destropy keylist
		PGPclKeyListDestroy (ptds->hKL, FALSE);
		break;

	case WM_NOTIFY :
		if (wParam == IDC_TOKENKEYLIST) 
		{
			ptds = (TOKENDLGSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
			// let keymanager have first whack at the notification
			PGPclKeyListDefaultNotificationProc (ptds->hKL, lParam);

			switch (((LPNM_TREELIST)lParam)->hdr.code) {
			// post the context menu
			case TLN_CONTEXTMENU :
				sTokenContextMenu (hwnd, ptds->hKL,
						((LPNM_TREELIST)lParam)->flags,
						((LPNM_TREELIST)lParam)->ptDrag.x,
						((LPNM_TREELIST)lParam)->ptDrag.y);
				break;
			}
			break;
		}

		switch (((NMHDR FAR *) lParam)->code) {

		case PSN_HELP :
			PGPclHtmlHelp (hwnd, kPGPclHelpContext, 0, 0,
					(char*)kPGPclHelpFile, 
					(DWORD*)IDH_PGPPKTOKEN_CONTENTSDIALOG);
			break;
		}
		break;

	case PGPCL_M_PROPERTYMESSAGE :
		if (wParam == kPGPclProperties_UpdateObject)
		{
			ptds = (TOKENDLGSTRUCT*)GetWindowLong (hwnd, GWL_USERDATA);
			ptds->bModified = TRUE;
		}
		break;
	}

	return FALSE;
}


//	___________________________________________________
//
//  update the tokens properties dialog

VOID 
PKReloadTokenProperties (
		BOOL	bLoadKeys)
{
	if (s_hwndContents)
	{
		SendMessage (s_hwndContents, WM_APP, (WPARAM)bLoadKeys, 0);
	}
}


//	___________________________________________________
//
//  display the token properties dialog

BOOL 
PKTokenProperties (
		PGPKEYSSTRUCT*	ppks)
{
	TOKENDLGSTRUCT		tds;
    PROPSHEETPAGE		psp[2];
    PROPSHEETHEADER		psh;
	CHAR				szTitle[128];

	LoadString (g_hinst, IDS_TOKENPROPCAPTION, szTitle, sizeof(szTitle));

	// zero out PREFSDLGSTRUCT
	memset (&tds, 0, sizeof(tds));

	tds.ppks = ppks;
	tds.context = g_context;
	tds.tlscontext = g_tlscontext;
	tds.hwndParent = ppks->hwndMain;
	tds.bNeedsCentering = TRUE;

	psp[0].dwSize = sizeof(PROPSHEETPAGE);
	psp[0].dwFlags = PSP_USETITLE|PSP_HASHELP;
	psp[0].hInstance = g_hinst;
	psp[0].pszTemplate = MAKEINTRESOURCE(IDD_TOKENPROPS);
	psp[0].pszIcon = NULL;
	psp[0].pfnDlgProc = sTokenPropsDlgProc;
	psp[0].pszTitle = (LPSTR)IDS_TOKEN_PROPERTIES;
	psp[0].lParam = (LPARAM)&tds;
	psp[0].pfnCallback = NULL;

	psp[1].dwSize = sizeof(PROPSHEETPAGE);
	psp[1].dwFlags = PSP_USETITLE|PSP_HASHELP;
	psp[1].hInstance = g_hinst;
	psp[1].pszTemplate = MAKEINTRESOURCE(IDD_TOKENCONTENTS);
	psp[1].pszIcon = NULL;
	psp[1].pfnDlgProc = sTokenContentsDlgProc;
	psp[1].pszTitle = (LPSTR)IDS_TOKEN_CONTENTS;
	psp[1].lParam = (LPARAM)&tds;
	psp[1].pfnCallback = NULL;

	psh.dwSize = sizeof(PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW;
	psh.hwndParent = tds.hwndParent;
	psh.hInstance = g_hinst;
	psh.pszIcon = NULL;
	psh.pszCaption = (LPSTR) szTitle;
	psh.nPages = 2;
	psh.nStartPage = 0;
	psh.ppsp = (LPCPROPSHEETPAGE) &psp;
	psh.pfnCallback = NULL;

	// actually display and handle the property sheet
	PropertySheet (&psh);

	s_hwndProperties = NULL;
	s_hwndContents = NULL;

	if (PGPKeySetRefIsValid (tds.keysetToDisplay))
		PGPFreeKeySet (tds.keysetToDisplay);

	return tds.bModified;
}

