/*______________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.


	$Id: ndbPGP.c,v 1.12 2002/09/09 00:31:43 sdas Exp $
______________________________________________________________________________*/

/*::: MODULE OVERVIEW :::::::::::::
Interface to the Lotus Notes "database-driver" API interface -- the only means 
by which an @Formula may call a custom DLL procedure, and the only way by which 
custom DLLs can be hooked to stay resident throughout the Notes session without 
use of a NOTES.INI reference (i.e. NSF Hooks or Extension Manager).

--- revision history ---------
7/2/01 Version 1.2 Paul Ryan
+ logic enhancement in support of robust plug-in version handling
+ listing format adjustment, documentation adjustment

10/7/00 Version 1.1.3 Paul Ryan
+ replaced version-check call handler in s_PlugInCommand() with handler that 
  simply loads this "database driver"
+ documentation adjustment

9/16/00 Version 1.1.2 Paul Ryan
+ support for special-handling enhancements made with respect to PGP-encoded 
  attachments

8/9/00 Version 1.1.1 Paul Ryan
+ support for checkmark-toggle UI for plug-in's drop-down action-bar menu

3/20/00 Version 1.1 Paul Ryan
+ Module moved out of own shell DLL into the main plug-in DLL. Associated 
  adjustment.
+ moved PGP dis/engagement into this "database-driver" module
+ minor logic, documentation improvements

8/27/99 Version 1.0.1 Paul Ryan
+ pre-mail send logic adjustment

1/10/99 Version 1.0 Paul Ryan
::::::::::::::::::::::::::::::::::::*/

#include "ndbPGP.h"


BOOL  ef_LoadedSessionLong;


/** xus_MailSendHookInitialize( ***


--- parameter & return ----

RETURN: eus_SUCCESS if no error occured; the Notes API error code otherwise

--- revision history ------
10/7/00 PR: removed logic to initialize this DLL's PGP connection due to 
	update in the plug-in subform's handling of this issue 
3/20/00	PR: moved PGP initialization into this event
12/10/98 PR: created		*/
//DOC!!
STATUS LNPUBLIC xus_MailSendHookInitialize( DBVEC * pt_drv)	{
    //fill in the subroutine vectors
    pt_drv->Init = xus_MailSendHookInitialize;
    pt_drv->Term = us_Terminating;
    pt_drv->Open = us_dmyOpen;
    pt_drv->Close = us_dmyClose;
	pt_drv->Function = s_PlugInCommand;

	return eus_SUCCESS;
} //xus_MailSendHookInitialize(


/** us_Terminating( ***
The "database driver" (and therefore the DLL) is closing. Take this chance to 
do any clean-up needed.

--- parameter & return ----
pt_drv: address of information structure describing aspects of this 
	"database driver"
RETURN: eus_SUCCESS

--- revision history ------
10/7/00 PR: minor documentation adjustment

9/16/00 PR
+ added clean-up of allocated Windows GDI resources
+ completed standard documentation

3/20/00	PR: moved PGP disengagement into this event
12/10/98 PR: created			*/
static STATUS LNPUBLIC us_Terminating( DBVEC * pt_drv) {
	//Disengage PGP as necessary. Cannot do this in DllMain() because PGP 
	//	involves worker threads.
	e_ReleasePgpContext();

	//free dynamically, statically allocated Windows GDI resources
	e_FreeGdiResources();

    return eus_SUCCESS;
} //us_dmyTerminate(


static STATUS LNPUBLIC us_dmyOpen( DBVEC * pt_drv, 
									HDBDSESSION * ppv_dmySession)	{
    *ppv_dmySession = NULL;
    return eus_SUCCESS;
} //us_dmyOpen(


static STATUS LNPUBLIC us_dmyClose( DBVEC * pt_drv, 
									HDBDSESSION  pv_dmy)	{
    return eus_SUCCESS;
} //us_dmyClose(


/** s_PlugInCommand( ***
Clearinghouse procedure for all @DbCommand calls into this "database driver" 
DLL. Currently four tasks are supported:

+ PGP secure specified rich-text field
  - 7 arguments required

+ Load Plug-In DLL on a session-long basis

+ Replace MIME Part items with previously preserved rich-text stream
  - Arg1: numberlist whose first element represents the handle to the note 
	      currently being processed
  - Arg2: textlist whose first element represents the name of the MIME-Part 
          items and the replacement rich-text field

+ Undertake special treatment of the given PGP-encrypted attachment
  - Arg1: textlist whose first element represents the object name of the 
          PGP-encrypted attachment to be handled specially and whose second 
		  element represents the original filename of the PGP-encrypted 
		  attachment
  - Arg2: numberlist whose first element represents the handle to the note 
	      currently being processed and containing the target attachment

--- parameters & return ----
pt_drv: Context information structure used by Notes to call this driver.
us_FUNC: Flags which database-driver @Formula command called this procedure. 
	Only @DbCommand is supported here.
us_ARGC: tells the number of arguments passed in the @DbCommand call
pul_ARGL: pointer to a one-to-one-correspondence array of the lengths of the 
	arguments passed in the @Formula call
ppv_ARGV: address of array of pointers to the arguments passed in the @Formula 
	call
ph_result: Output, unused. Pointer to variable in which to store the handle to 
	the result set determined by the procedure.
pul_lenResult: Output, unused. Pointer to variable in which to store the length 
	of the result set determined by the procedure.
RETURN: 
	eus_SUCCESS if no error occured
	eus_ERR_INVLD_ARG if any input parameter is obviously invalid
	the Notes API error code if a Notes API error occurred
	!eus_SUCCESS if any PGP error occurred during its initialization routine as 
		part of "Load Plug-In DLL on a session-long basis"

--- revision history -------
9/6/02 PR: minor exception-handling adjustment, minor documentation adjustment

7/2/01 PR
+ logic adjustment in support of relocation of PGP initialization to be part of 
  "Load Plug-In DLL on a session-long basis"
+ minor documentation correction, listing format adjustment

10/7/00 PR
+ replaced version-check call handler with handler that simply loads this 
  "database driver" so that this DLL will stay resident for the entire Notes 
  session, not just the message session
+ documentation adjustment

9/16/00 PR: added a call handler for when a PGP-encoded attachment's action 
	hotspot is invoked to provide special handling for the attachment
8/9/00 PR: added a call handler for support of a checkmark-toggle UI on the 
	drop-down action-bar menu

3/20/00 PR
+ improved error-return information
+ apart from specification of "PGP secure specified rich-text field" task 
  arguments, completed standard documentation

8/27/99 PR: logic adjustments to support enhancement of better emulation of 
	Lotus Notes' name-lookup behavior, including removal of recipient-count 
	parameter
12/10/98 PR: created		*/
static short LNPUBLIC s_PlugInCommand( DBVEC * pt_drv, 
										HDBDSESSION  pv_dmy, 
										WORD  us_FUNC, 
										WORD  us_ARGC, 
										DWORD * pul_ARGL, 
										void * *const  ppv_ARGV, 
										HANDLE * ph_result, 
										DWORD * pul_lenResult)	{
	const WORD  us_ARGS_PGP_SECURE_RTF = 7, 
				us_ARGS_LOAD_DLL = 0, 
				us_ARGS_TWO = 2;

	int  i_err;

	//if any of the important parameters are obviously invalid, return general 
	//	failure
	if (us_FUNC != DB_DBEXT)
		return ERR_DBD_FUNCTION;
	if (!( (us_ARGC == us_ARGS_PGP_SECURE_RTF || us_ARGC == us_ARGS_LOAD_DLL || 
									us_ARGC == us_ARGS_TWO) && pul_ARGL && 
									ppv_ARGV && ph_result && pul_lenResult))
		return eus_ERR_INVLD_ARG;

	if (us_ARGC == us_ARGS_PGP_SECURE_RTF)
		return es_PreMailSend( ppv_ARGV);
	else if (us_ARGC == us_ARGS_LOAD_DLL)	{
		//if we've already done this work, short-circuit with success
		if (ef_LoadedSessionLong)
			return eus_SUCCESS;

		//Engage PGP. Cannot do this in DllMain() because PGP involves worker 
		//	threads.
		if (i_err = ei_InitializePgpContext())
			return !eus_SUCCESS;

		//note that this DLL has been successfully enabled for the duration of 
		//	the current Notes-client session
		ef_LoadedSessionLong = TRUE;

		return eus_SUCCESS;
	}else //(us_ARGC == us_ARGS_TWO)
		if (*(WORD *) ppv_ARGV[0] == TYPE_NUMBER_RANGE)
			return es_ReplaceMimeWithRtf( ppv_ARGV);
		else
			return es_SpecialDecryptAttachment( ppv_ARGV);

	return NULL;	//should never get here; line included to avoid compiler 
					//	warning
} //s_PlugInCommand(


