/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: MainWndProc.c,v 1.12 2002/10/03 04:18:05 sdas Exp $
____________________________________________________________________________*/

#include <windows.h>
#include <commctrl.h>
#include "PluginInfo.h"
#include "HookProcs.h"
#include "UIutils.h"
#include "pgpClientLib.h"
#include "pgpDebug.h"
#include "resource.h"

void CreateMainWindowGUI(HWND hDlg, PluginInfo *plugin);
void FindAndStoreMainUIElements(HWND hMainWnd, PluginInfo *lppilPlugInf);
void PreparePGPMenu(PluginInfo *lppilPlugInf);
void InitMainToolbar(PluginInfo *lppilPlugInf);
void DeInitMainToolbar(PluginInfo *lppilPlugInf);
extern int LoadAppropriateBitmaps(PluginInfo *plugin, UINT uiCommandId);
static LRESULT CALLBACK ToolbarParentWndProc(HWND hDlg, 
											 UINT msg,
											 WPARAM wParam, 
											 LPARAM lParam);


LRESULT CALLBACK MainWndProc(HWND hDlg, 
							 UINT msg,
							 WPARAM wParam, 
							 LPARAM lParam)
{
	WNDPROC lpOldProc;
	PluginInfo *plugin;

	lpOldProc = (WNDPROC)GetProp(hDlg, "oldproc");
	plugin = GetPluginInfo(hDlg);
	
	switch(msg)
	{
	case WM_SETTINGCHANGE:
	case WM_PALETTECHANGED:
	case WM_FONTCHANGE:
	case WM_SYSCOLORCHANGE:
		{
			LONG lResult = CommonWndProc(hDlg, msg, wParam, lParam);

			//if we have inserted our stuff into the toolbar already
			if(plugin->dwInitState & PIIF_TOOLSINSERTION_DONE)
			{
				DeInitMainToolbar(plugin);//de-initialize/remove stuff
				InitMainToolbar(plugin);
			}
			return lResult;
		}

	case WM_DESTROY:
		{
			KillTimer(hDlg, WATCH_TOOLBAR_TIMER);
			break;
		}

	case WM_PARENTNOTIFY:
		if (LOWORD(wParam) == WM_CREATE)
			CreateMainWindowGUI(hDlg, plugin);
		break;
	
	case WM_TIMER:
		if (wParam == WATCH_TOOLBAR_TIMER)
		{
			int nState=0;
			LONG lRet=0;

			//if we have found the toolbar already
			if(NULL != plugin->hToolbar)
			{
				//get the button state of pgpkeys button
				nState = SendMessage(plugin->hToolbar, TB_GETSTATE, 
							IDC_PGPKEYS, 0);

				//if we got valid button state
				if(-1 != nState)
				{
					//check if it is enabled
					if (nState & TBSTATE_ENABLED)
						return 0;//we are done

					//enable the button
					nState |= TBSTATE_ENABLED;
					lRet=SendMessage(plugin->hToolbar, TB_SETSTATE, IDC_PGPKEYS, 
						MAKELONG(nState, 0));
					//pgpAssert(TRUE == lRet);
				}
			}

			return 0;
		}
		break;

	case WM_INITMENUPOPUP:
		{
			int nOptions;
			MENUITEMINFO menuInfo;
			char szOptions[256];
			char szMenu[256];
			LRESULT lResult;

			// For some reason, the old window proc disables the
			// menu items in the PGP popup menu. We'll enable
			// the items after the old proc disables them

			if ((HMENU) wParam == plugin->hPGPMenu)
			{
				lResult = CallWindowProc(lpOldProc, hDlg, msg, wParam, 
							lParam);
				
				EnableMenuItem(plugin->hPGPMenu, IDC_PREFS, 
					MF_BYCOMMAND | MF_ENABLED);
				EnableMenuItem(plugin->hPGPMenu, IDC_PGPKEYS, 
					MF_BYCOMMAND | MF_ENABLED);

				return lResult;
			}
			else if (plugin->bOE5)
			{
				UIGetString(szOptions, 254, IDS_MENU_OEOPTIONS);
				nOptions = GetMenuItemCount((HMENU) wParam);
				
				menuInfo.cbSize = sizeof(MENUITEMINFO);
				menuInfo.fMask = MIIM_TYPE | MIIM_SUBMENU | MIIM_ID;
				menuInfo.dwTypeData = szMenu;
				menuInfo.cch = 255;
				
				GetMenuItemInfo((HMENU) wParam, nOptions - 1, TRUE, 
					&menuInfo);

				if (!strcmp(szMenu, szOptions))
				{
					lResult = CallWindowProc(lpOldProc, hDlg, msg, wParam, 
								lParam);
					
					strcpy(szMenu, "&PGP");
					
					menuInfo.cbSize = sizeof(MENUITEMINFO);
					menuInfo.fMask = MIIM_TYPE | MIIM_ID | MIIM_SUBMENU | 
						MIIM_STATE;
					menuInfo.fType = MFT_STRING;
					menuInfo.fState = MFS_ENABLED;
					menuInfo.wID = IDC_PGPMENU;
					menuInfo.hSubMenu = plugin->hPGPMenu;
					menuInfo.dwTypeData = szMenu;
					
					InsertMenuItem((HMENU) wParam, nOptions, TRUE, &menuInfo);
					plugin->nPGPPosition = nOptions;
					return lResult;
				}
				else if (!strcmp(szMenu, "&PGP"))
				{
					lResult = CallWindowProc(lpOldProc, hDlg, msg, wParam, 
								lParam);
					
					DestroyMenu(plugin->hPGPMenu);
					plugin->hPGPMenu = CreatePopupMenu();
					
					UIGetString(szMenu, 254, IDS_MENU_PREFS);
					AppendMenu(plugin->hPGPMenu, MF_STRING, IDC_PREFS, 
						szMenu);
					
					UIGetString(szMenu, 254, IDS_MENU_PGPKEYS);
					AppendMenu(plugin->hPGPMenu, MF_STRING, IDC_PGPKEYS, 
						szMenu);
					
					menuInfo.cbSize = sizeof(MENUITEMINFO);
					menuInfo.fMask = MIIM_SUBMENU | MIIM_STATE;
					menuInfo.fState = MFS_ENABLED;
					menuInfo.hSubMenu = plugin->hPGPMenu;

					SetMenuItemInfo((HMENU) wParam, IDC_PGPMENU, FALSE,
						&menuInfo);

					menuInfo.fMask = MIIM_STATE;

					SetMenuItemInfo(plugin->hPGPMenu, IDC_PREFS, FALSE, 
						&menuInfo);
				
					SetMenuItemInfo(plugin->hPGPMenu, IDC_PGPKEYS, FALSE, 
						&menuInfo);

					return lResult;
				}
			}

			break;
		}
	}

	return CommonWndProc(hDlg, msg, wParam, lParam);
}


void CreateMainWindowGUI(HWND hDlg, PluginInfo *plugin)
{
	char szText[255]={0};
	
	//find and store the main UI elements we will be playing with
	FindAndStoreMainUIElements(hDlg, plugin);

	//if we have not found the main menu/toolbars yet
	if(!(plugin->dwInitState&PIIF_MAINUIELEMENTS_FOUND))
		return;
	
	//if we have not prepared the menu already
	if(NULL == plugin->hPGPMenu)
	{
		//make the pgp popup menu to be put in the OE menu
		PreparePGPMenu(plugin);
	}

	//if the menu is made and this is not OE5.
	//NOTE: we dont return if things dont work out here
	if((NULL != plugin->hPGPMenu) && (!plugin->bOE5))
	{
		if(!(plugin->dwInitState&PIIF_MENUINSERTION_DONE))
		{
			BOOL bRet=FALSE;

			//we insert pgp menu at index 4 of the menu bar
			plugin->nPGPPosition = 4;
			
			//insert our popup menu
			bRet = InsertMenu(plugin->hMainMenu, plugin->nPGPPosition, 
				MF_BYPOSITION | MF_STRING | MF_POPUP, 
				(UINT)plugin->hPGPMenu, "&PGP");
			pgpAssert(TRUE == bRet);

			if(TRUE == bRet)//if menu successfully inserted
			{
				plugin->dwInitState |= PIIF_MENUINSERTION_DONE;//remember we have already inserted
				DrawMenuBar(hDlg);//draw the menu bar
			}
			
		}
	}
	
	//attempt adding our buttons to OE toolbar
	InitMainToolbar(plugin);

	//if we have not already started the toolbar timer
	if(!(plugin->dwInitState&PIIF_TOOLBARTIMER_SET))
	{
		int nTimerId=-1;

		//set the timer on the toolbar
		nTimerId = SetTimer(hDlg, WATCH_TOOLBAR_TIMER, WATCH_TOOLBAR_TIMER_LENGTH, NULL);
		pgpAssert(0 != nTimerId);
		if(0 != nTimerId)//if timer was created successfully
			plugin->dwInitState |= PIIF_TOOLBARTIMER_SET;//remember success
	}

	//if we have not already subclassed the rebar control
	//and we have already inserted our tools into the toolbar
	if((!(plugin->dwInitState&PIIF_REBARCTRL_SUBCLASSED)) &&
	   (plugin->dwInitState & PIIF_TOOLSINSERTION_DONE))
	{
		//subclass the rebar control
		HWND hSizableRebar=NULL, hReBar=NULL;
		BOOL bRet=FALSE;

		hSizableRebar = FindWindowEx(hDlg, NULL, "SizableRebar", NULL);
		hReBar = FindWindowEx(hSizableRebar, NULL, "ReBarWindow32", NULL);
		if(NULL != hReBar)
		{
			//out custom property should not already be in there
			pgpAssert(NULL == GetProp(hReBar, "oldproc"));

			//save away toolbar parent window proc
			bRet = SetProp(hReBar, "oldproc", (HANDLE)GetWindowLong(hReBar, GWL_WNDPROC));
			if(TRUE == bRet)
			{
				//save plugin pointer
				bRet = SetProp(hReBar, PLUGIN_INFO_PROP, (HANDLE) plugin);
				if(TRUE == bRet);
				{
					//subclass the rebar window
					bRet = SetWindowLong(hReBar, GWL_WNDPROC, (DWORD)ToolbarParentWndProc);
					if(bRet != 0)
						plugin->dwInitState |= PIIF_REBARCTRL_SUBCLASSED;
				}
			}
		}
	}

	return;
}



static LRESULT CALLBACK ToolbarParentWndProc(HWND hDlg, 
											 UINT msg,
											 WPARAM wParam, 
											 LPARAM lParam)
{
	WNDPROC lpOldProc;
	PluginInfo *plugin;

	lpOldProc = (WNDPROC)GetProp( hDlg, "oldproc" );
	plugin = GetPluginInfo(hDlg);

	switch(msg)
	{
		case WM_NOTIFY:
			{
				LPNMHDR lpnm;
				LONG lResult = 0;
				lpnm = (LPNMHDR)lParam;
				

				switch(lpnm->code)
				{
					case TBN_BEGINADJUST:
						//if we have inserted our stuff into the toolbar already
						if(plugin->dwInitState & PIIF_TOOLSINSERTION_DONE)
							DeInitMainToolbar(plugin);//de-initialize/remove stuff
						break;

					case TBN_ENDADJUST:
						{
							lResult = CallWindowProc(lpOldProc, hDlg, msg, wParam, 
										lParam);

							//if we have not inserted our stuff into the toolbar already
							if(!(plugin->dwInitState & PIIF_TOOLSINSERTION_DONE))
							{
								//insert our stuff as necessary
								pgpAssert(-1 == plugin->nPGPKeysButton);
								InitMainToolbar(plugin);								
							}

							return lResult;
						}
						break;

					default:
						break;
				}
			}
			break;

		default:
			break;
	}

	return CallWindowProc(lpOldProc, hDlg, msg, wParam, lParam);
}


void FindAndStoreMainUIElements(HWND hMainWnd, PluginInfo *lppilPlugInf)
{
	HWND hSizableRebar=NULL;
	HWND hReBar=NULL;
	LONG lResult=0;

	//check parameters - anything wrong bail out
	pgpAssert(NULL != hMainWnd);
	pgpAssert((NULL != lppilPlugInf) && (NULL != lppilPlugInf->hwndCurrent));//hwndCurrent should be set
	if((NULL == hMainWnd) || (NULL == lppilPlugInf) || (NULL == lppilPlugInf->hwndCurrent))
		return;

	pgpAssert(hMainWnd == lppilPlugInf->hwndCurrent);

	//if we have already found the main menu/toolbars 
	if(lppilPlugInf->dwInitState&PIIF_MAINUIELEMENTS_FOUND)
		return;//bail out

	//let's take care of the menu if we need that in this version of OE
	if((!lppilPlugInf->bOE5) && (NULL == lppilPlugInf->hMainMenu))
	{
		lppilPlugInf->hMainMenu = GetMenu(hMainWnd);
		if(NULL == lppilPlugInf->hMainMenu)
			return;//could not find the main menu
	}

	//let's take care of the hToolbar member of plugininfo
	hSizableRebar = FindWindowEx(hMainWnd, NULL, "SizableRebar", NULL);
	if(NULL == hSizableRebar)
		return;//perhaps called too early in OE's initialization

	hReBar = FindWindowEx(hSizableRebar, NULL, "ReBarWindow32", NULL);
	if(NULL == hReBar)
		return;//perhaps called too early in OE's initialization

	if(NULL == lppilPlugInf->hToolbar)//if we have not filled that member yet
	{
		lppilPlugInf->hToolbar = FindWindowEx(hReBar, NULL, "ToolbarWindow32", NULL);
		if(NULL == lppilPlugInf->hToolbar)
			return;//perhaps called too early in OE's initialization

		if(lppilPlugInf->bOE5)
		{
			//now let's take care of hMenuToolbar member of plugininfo if we need
			//that in this version of OE. NOTE: even if lppilPlugInf->hToolbar may
			//not be the toolbar we are looking for the next child window in z-order
			//happens to be always the menu bar (thank god !!)
			if(lppilPlugInf->bOE5 && (NULL == lppilPlugInf->hMenuToolbar))
			{
				pgpAssert(NULL != lppilPlugInf->hToolbar);
				lppilPlugInf->hMenuToolbar = FindWindowEx(hReBar, lppilPlugInf->hToolbar, "ToolbarWindow32", NULL);
				if(NULL == lppilPlugInf->hMenuToolbar)
					return;//we will have to try to find it again some other time
				
				//TODO:we need some way of distinguishing between the toolbars and the menu bar
				//pgpAssert(-1 == SendMessage(lppilPlugInf->hMenuToolbar, TB_BUTTONCOUNT, 0, 0));

			}

			//if there were other child windows found under our toolbar
			if((0 == SendMessage(lppilPlugInf->hToolbar, TB_GETIMAGELIST, 0, 0)) ||
				(NULL != FindWindowEx(lppilPlugInf->hToolbar, NULL, NULL, NULL)))
			{
				//probably the handle we got for toolbar is actually the view bar

				//check for other windows of the toolbar class if any under the rebar. 
				//the first childless toolbar window different from the menutoolbar
				//is our best guess for the toolbar we are looking for
				//NOTE: this is only done on OE5 or higher

				//find the next toolbar child
				HWND hPossToolbar = FindWindowEx(hReBar, lppilPlugInf->hToolbar, "ToolbarWindow32", NULL);

				//until the child is not null 
				while(NULL != hPossToolbar)
				{
					//if the toolbar is not being thought of as a menutoolbar
					if(hPossToolbar != lppilPlugInf->hMenuToolbar)
					{
						//if the toolbar has no children
						if((NULL == FindWindowEx(hPossToolbar, NULL, NULL, NULL)) &&
						   ((0 == SendMessage(lppilPlugInf->hToolbar, TB_GETIMAGELIST, 0, 0))))//and has an imagelist
							break;//we have found it !
					}
					
					//go to next child
					hPossToolbar = FindWindowEx(hReBar, hPossToolbar, "ToolbarWindow32", NULL);
				}

				//if we got out of loop because we found a toolbar with no child
				lppilPlugInf->hToolbar = hPossToolbar;//this is our best effort guess for now
				if(NULL == lppilPlugInf->hToolbar)
					return; //could not find a toolbar to attach UI elements to
			}
		}

		if(NULL != lppilPlugInf->hToolbar)
			PGPOeTrace("Found toolbar handle %#x \n", lppilPlugInf->hToolbar);
	}
	

	//now let's take care of hStatusBar member of plugininfo
	if(NULL == lppilPlugInf->hStatusBar)
	{
		lppilPlugInf->hStatusBar = FindWindowEx(hMainWnd, NULL, "msctls_statusbar32", NULL);
		//NOTE we do not consider this abcolutely necessary
	}

	//control comes here only if all the things we need are filled up in plugininfo
	//so we remember that we have already reached this stage of plugin initialization
	lppilPlugInf->dwInitState |= PIIF_MAINUIELEMENTS_FOUND;
}

void InitMainToolbar(PluginInfo *lppilPlugInf)
{
	int nIndex = -1;
	int nPGPkeysSTR = -1;
	int nPGPkeysBMP = -1;
	LONG lResult = 0;
	TBBUTTON tbb[1]={0};
	char szText[255] = {0};
	HINSTANCE hInst = NULL;
	int nX = -1;

	//check parameters
	if(NULL == lppilPlugInf)
		return;

	//if main UI elements have not yet been found/partially found
	if(!(lppilPlugInf->dwInitState & PIIF_MAINUIELEMENTS_FOUND))
	{
		//find and store the main UI elements we will be playing with
		FindAndStoreMainUIElements(lppilPlugInf->hwndCurrent, lppilPlugInf);

		//if we have not found all the main menu/toolbars stuff
		if(!(lppilPlugInf->dwInitState&PIIF_MAINUIELEMENTS_FOUND))
			return;
	}

	//we should be having all UI elements needed from this point onwards
	pgpAssert(lppilPlugInf->dwInitState & PIIF_MAINUIELEMENTS_FOUND);

	//we should not have inserted ourselves already
	//pgpAssert(!(lppilPlugInf->dwInitState & PIIF_TOOLSINSERTION_DONE));
	if(lppilPlugInf->dwInitState & PIIF_TOOLSINSERTION_DONE)
		return;//we have already inserted ourselves to the main toolbar
	
	//we should not be there already. alternately OE could not be using
	//our command id for itself
	pgpAssert(-1 == SendMessage(lppilPlugInf->hToolbar, TB_COMMANDTOINDEX,
				IDC_PGPKEYS, 0));

	//get the number of buttons in the toolbar currently
	//lResult = SendMessage(lppilPlugInf->hToolbar, TB_BUTTONCOUNT, 0, 0);
	//if (lResult < 2)//if less than 2
	//	return;//we will try again later ?

	//load and add the appropriate bitmaps to the toolbar for pgp keys button
	nPGPkeysBMP = LoadAppropriateBitmaps(lppilPlugInf, IDC_PGPKEYS);
	pgpAssert(-1 != nPGPkeysBMP);
	if(-1 == nPGPkeysBMP)
		return;//some failure in loading bitmaps

	pgpAssert(-1 == lppilPlugInf->nPGPKeysImage);
	lppilPlugInf->nPGPKeysImage = nPGPkeysBMP;

	//if we have not already added the string for pgpkeys button
	if(-1 == lppilPlugInf->nPGPKeysString)
	{
		UIGetString(szText, 254, IDS_TOOLTIP_PGPKEYS);
		nPGPkeysSTR = SendMessage(lppilPlugInf->hToolbar, TB_ADDSTRING, 0,
						(LPARAM) szText);
		pgpAssert(-1 != nPGPkeysSTR);
		pgpAssert(-1 == lppilPlugInf->nPGPKeysString);
		lppilPlugInf->nPGPKeysString = nPGPkeysSTR;
	}
	
	//we must send TB_BUTTONSTRUCTSIZE before sending TB_ADDBITMAP 
	//or TB_ADDBUTTONS  message NOTE: this message has side effects
	SendMessage(lppilPlugInf->hToolbar, TB_BUTTONSTRUCTSIZE,
		(WPARAM)sizeof(TBBUTTON), 0);

	tbb[0].iBitmap = nPGPkeysBMP;
	tbb[0].idCommand = IDC_PGPKEYS;
	tbb[0].fsState = TBSTATE_ENABLED;
	tbb[0].fsStyle = TBSTYLE_BUTTON;
	tbb[0].dwData = 0;
	tbb[0].iString = lppilPlugInf->nPGPKeysString;

	lResult = SendMessage(lppilPlugInf->hToolbar, TB_ADDBUTTONS, 1, (LPARAM) &tbb);
	pgpAssert(TRUE == lResult);
	if(TRUE == lResult)//if we successfully added the button
	{
		//get the command id for our button
		lppilPlugInf->nPGPKeysButton = SendMessage(lppilPlugInf->hToolbar, 
			TB_COMMANDTOINDEX, IDC_PGPKEYS, 0);
		pgpAssert(-1 != lppilPlugInf->nPGPKeysButton);

		if(-1 != lppilPlugInf->nPGPKeysButton)
			lppilPlugInf->dwInitState |= PIIF_TOOLSINSERTION_DONE;//all success
	}
	return;
}

void DeInitMainToolbar(PluginInfo *lppilPlugInf)
{
	HIMAGELIST hTbImgList = NULL;
	BOOL bRet = FALSE;
	BOOL bForceRedetectTbar = FALSE;
	INT iIndex = -1;
	LONG lResult = 0;

	//if we have not inserted our stuff into the toolbar already
	if(!(lppilPlugInf->dwInitState & PIIF_TOOLSINSERTION_DONE))
		return;

	pgpAssert(-1 != lppilPlugInf->nPGPKeysButton);
	pgpAssert(NULL != lppilPlugInf->hToolbar);

	//REMOVE THE BUTTON

	//find the button index for pgp keys button
	iIndex = SendMessage(lppilPlugInf->hToolbar, TB_COMMANDTOINDEX,
				IDC_PGPKEYS, 0);
	pgpAssert(-1 != iIndex);
	
	if(-1 != iIndex)
	{
		//the button index should be the same as we know already
		pgpAssert(iIndex == lppilPlugInf->nPGPKeysButton);

		//delete the button 
		lResult = SendMessage(lppilPlugInf->hToolbar, TB_DELETEBUTTON, iIndex, 0);
		pgpAssert(TRUE == lResult);

		if(FALSE == lResult)//if we could not delete the button
			bForceRedetectTbar = TRUE;//force a redetection of toolbar stuff

		//does not matter if we could delete or not. forget the button index nevertheless
		lppilPlugInf->nPGPKeysButton = -1;
	}

	//REMOVE BITMAP INSERTIONS FOR THE BUTTONS

	pgpAssert(-1 != lppilPlugInf->nPGPKeysImage);
	if( -1 != lppilPlugInf->nPGPKeysImage)
	{
		//remove it from the hot imagelist
		hTbImgList = (HIMAGELIST) SendMessage(lppilPlugInf->hToolbar, 
											TB_GETHOTIMAGELIST, 0, 0);
		pgpAssert(NULL != hTbImgList);
		if(NULL == hTbImgList)//if we could not find an imagelist
		{
			bForceRedetectTbar = TRUE;//force a redetection of toolbar stuff
		}
		else//we found the imagelist
		{
			bRet = ImageList_Remove(hTbImgList, lppilPlugInf->nPGPKeysImage);
			pgpAssert(0 != bRet);
		}

		//remove it from the normal imagelist
		hTbImgList = (HIMAGELIST) SendMessage(lppilPlugInf->hToolbar, 
											TB_GETIMAGELIST, 0, 0);
		pgpAssert(NULL != hTbImgList);
		if(NULL == hTbImgList)//if we could not find an imagelist
		{
			bForceRedetectTbar = TRUE;//force a redetection of toolbar stuff
		}
		else//we found the imagelist
		{
			bRet = ImageList_Remove(hTbImgList, lppilPlugInf->nPGPKeysImage);
			pgpAssert(0 != bRet);
		}

		lppilPlugInf->nPGPKeysImage = -1;//cant do much if anything fails to delete
	}

	//REMOVE THE STRING ADDED TO TOOLBAR

	//there is currently no way to do this. TODO: change stuff so that we
	//do not need to have this side effect

	//if we are supposed to redetect toolbar stuff
	if(TRUE == bForceRedetectTbar)
	{
		PGPOeTrace("Forgetting toolbar handle %#x \n", lppilPlugInf->hToolbar);
		lppilPlugInf->hToolbar = NULL; //forget current toolbar handle
		lppilPlugInf->dwInitState &= (~PIIF_MAINUIELEMENTS_FOUND);//mark state appropriately
	}

	//state maintenance - toolbar insertion is not done anymore
	lppilPlugInf->dwInitState &= (~PIIF_TOOLSINSERTION_DONE);
}


void PreparePGPMenu(PluginInfo *lppilPlugInf)
{
	char szText[255]={0};
	pgpAssert(NULL != lppilPlugInf);
	pgpAssert(NULL == lppilPlugInf->hPGPMenu);

	//if invalid params or menu already made
	if((NULL == lppilPlugInf) || (NULL != lppilPlugInf->hPGPMenu))
		return;//bail

	lppilPlugInf->hPGPMenu = CreatePopupMenu();
	if(NULL == lppilPlugInf->hPGPMenu)
		return;
	
	UIGetString(szText, 254, IDS_MENU_PREFS);
	AppendMenu(lppilPlugInf->hPGPMenu, MF_STRING, IDC_PREFS, szText);
	
	UIGetString(szText, 254, IDS_MENU_PGPKEYS);
	AppendMenu(lppilPlugInf->hPGPMenu, MF_STRING, IDC_PGPKEYS, szText);

	return;//all success
}

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
