/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: SendMsgWndProc.c,v 1.22 2002/10/03 04:18:05 sdas Exp $
____________________________________________________________________________*/

#include <windows.h>
#include <commctrl.h>
#include <process.h>
#include "PluginInfo.h"
#include "HookProcs.h"
#include "UIutils.h"
#include "TextIO.h"
#include "Recipients.h"
#include "resource.h"
#include "EncryptSign.h"
#include "commctrl.h"
#include "pgpdebug.h"

static BOOL EncryptSignMessage(HWND hwnd);
static void DoubleLineFeeds(char *szOriginal, char *szNew);
static void UpdateMenus(HWND hwnd, PluginInfo *plugin);
int LoadAppropriateBitmaps(PluginInfo *plugin, UINT uiCommandId);
void RemoveSendToolbarButtons(PluginInfo *lppilPlugInf);
void DumpToolbarInfo(HWND hTBWnd);
void DumpImageListInfo(HIMAGELIST hilImgList);

typedef struct _AttStruct
{
	char *String0;
	char *String1;
	HANDLE hFile0;
	HANDLE hFile1;
} AttStruct;


LRESULT CALLBACK SendMsgWndProc(HWND hDlg, 
								UINT msg,
								WPARAM wParam, 
								LPARAM lParam)
{
	WNDPROC lpOldProc;
	PluginInfo *plugin;

	lpOldProc = (WNDPROC)GetProp( hDlg, "oldproc" );
	plugin = GetPluginInfo(hDlg);

	if (plugin != NULL)
	{
		if (msg == plugin->nCopyDoneMsg)
		{
			if (wParam)
			{
				if (plugin->nAction == PluginAction_Encrypt)
				{
					if (EncryptSignMessage(hDlg))
					{
						plugin->nRetryCount = 0;
						_beginthread(DoPasteThread, 0, hDlg);
					}
					else
					{
						RestoreClipboardText(hDlg);
						SetWindowPos(hDlg, HWND_NOTOPMOST, 0, 0, 0, 0, 
							SWP_NOMOVE | SWP_NOSIZE);
					}
				}
				else if (plugin->nAction == PluginAction_Check)
				{
					char *szCheck;
					BOOL bOK = FALSE;
					UINT nOutLength = strlen(plugin->szOutput);
					UINT nCheckLength;

					GetMessageText(hDlg, FALSE, &szCheck);
					nCheckLength = strlen(szCheck);
					RestoreClipboardText(hDlg);

					if (!strncmp(plugin->szOutput, szCheck, nOutLength))
						bOK = TRUE;
					else if (nOutLength < nCheckLength)
					{
						int nIndex;
						int nDiff = nCheckLength - nOutLength;

						for (nIndex=0; nIndex<nDiff; nIndex++)
						{
							if (plugin->szOutput[0] == szCheck[nIndex])
							{
								if (!strncmp(plugin->szOutput, 
									&szCheck[nIndex], nOutLength))
								{
									bOK = TRUE;
									break;
								}
							}
						}
					}

					if (bOK)
					{
						free(plugin->szOutput);
						plugin->szOutput = NULL;
						plugin->bEncrypt = FALSE;
						plugin->bSign = FALSE;
						
						if (plugin->hDlg != NULL)
							return CallWindowProc(lpOldProc, plugin->hDlg, 
										plugin->nMsg, plugin->wParam, 
										plugin->lParam);
						else
							return 0;
					}
					else
					{
						plugin->nRetryCount++;
						if (plugin->nRetryCount < MAX_PASTE_RETRIES)
						{
							if (!SetMessageText(hDlg, plugin->szOutput))
								return 0;
							else
							{
								plugin->nAction = PluginAction_Encrypt;
								_beginthread(DoPasteThread, 0, hDlg);
							}
						}
						else
						{
							char szTitle[256];
							char szMsg[256];

							UIGetString(szTitle, sizeof(szTitle), IDS_TITLE);
							UIGetString(szMsg, sizeof(szMsg), IDS_E_PASTEFAILED);
							MessageBox(hDlg, szMsg, szTitle, MB_ICONEXCLAMATION);

							free(plugin->szOutput);
							plugin->szOutput = NULL;
							SetWindowPos(hDlg, HWND_NOTOPMOST, 0, 0, 0, 0, 
								SWP_NOMOVE | SWP_NOSIZE);
							return 0;
						}
					}
				}
				else if (plugin->nAction == PluginAction_Reply)
				{
					char *szText;

					szText = GetReplyText();
					if (szText != NULL)
					{
						char *szReplyText=NULL;
						char *szReplyHeader=NULL;
						
						GetMessageText(hDlg, FALSE, &szReplyHeader);
						if (FindReplyHeader(szReplyHeader))
						{
							szReplyText = (char *) 
								calloc(strlen(szReplyHeader) +
									strlen(szText) * 2 + 1, 1);
							
							if(NULL != szReplyText)
							{
								strcpy(szReplyText, szReplyHeader);
								AddReply(szReplyText, szText);
								SetMessageText(hDlg, szReplyText);
								free(szReplyText);
								szReplyText = NULL;
							}
							_beginthread(DoPasteThread, 0, hDlg);
						}
						else
							RestoreClipboardText(hDlg);
						
						free(szReplyHeader);
						FreeReplyText();
					}
					else
						RestoreClipboardText(hDlg);
				}
				else
					RestoreClipboardText(hDlg);
			}
			else
				RestoreClipboardText(hDlg);

			return 0;
		}
	
		if (msg == plugin->nPasteDoneMsg)
		{
			if (plugin->nAction == PluginAction_Encrypt)
			{
				plugin->nAction = PluginAction_Check;
				_beginthread(DoCopyThread, 0, hDlg);
				return 0;
			}
			else
				SetWindowPos(hDlg, HWND_NOTOPMOST, 0, 0, 0, 0, 
					SWP_NOMOVE | SWP_NOSIZE);
		}
	}

	switch(msg)
	{
	case WM_DESTROY:
		{
			KillTimer(hDlg, WATCH_TOOLBAR_TIMER);
			break;
		}

	case WM_TIMER:
		if (wParam == CORRECT_TOOLBAR_TIMER)
		{
			KillTimer(hDlg, CORRECT_TOOLBAR_TIMER);

			SendMessage(plugin->hToolbar, TB_SETBUTTONWIDTH, 0,
				MAKELONG(24, 24));
			SendMessage(plugin->hToolbar, TB_AUTOSIZE, 0, 0);
			UpdateMenus(hDlg, plugin);

			return 0;
		}
		else if (wParam == PASTE_REPLY_TIMER)
		{
			char *szText;

			// REALLY ugly hack here to get buttons to appear on OE6 on XP. Basically we just
			// hit the rebar control and that (for some reason) updates the toolbar, just like
			// clicking the rebar manually works.
			SendMessage(GetParent(plugin->hToolbar),WM_LBUTTONDOWN,(WPARAM)MK_LBUTTON,(LPARAM)MAKELPARAM(1,54));
			SendMessage(GetParent(plugin->hToolbar),WM_LBUTTONUP,(WPARAM)0,(LPARAM)MAKELPARAM(1,54));
			
			// Paste any reply text
			KillTimer(hDlg, PASTE_REPLY_TIMER);

			szText = GetReplyText();
			if (szText != NULL)
			{
				if (!SaveClipboardText(hDlg))
					return 0;
				
				plugin->nAction = PluginAction_Reply;
				plugin->hDlg = NULL;

				_beginthread(DoCopyThread, 0, hDlg);
			}

			return 0;
		}
		break;

	case WM_MENUSELECT:
		{
			UINT uFlags = (UINT) HIWORD(wParam);
			char szText[255];
			BOOL bPluginMenu = FALSE;

			if (uFlags & MF_POPUP)
			{
				if (LOWORD(wParam) == plugin->nPGPPosition+1)
					UpdateMenus(hDlg, plugin);
			}

			switch(LOWORD(wParam))
			{
				case IDC_ENCRYPT_SMIME:
				case IDC_OE5_ENCRYPT_SMIME:
				{
					UIGetString(szText, 254, IDS_STATUS_SMIME_ENCRYPT);
					bPluginMenu = TRUE;
					break;
				}
								
				case IDC_SIGN_SMIME:
				case IDC_OE5_SIGN_SMIME:
				{
					UIGetString(szText, 254, IDS_STATUS_SMIME_SIGN);
					bPluginMenu = TRUE;
					break;
				}

				case IDC_ENCRYPT_PGP:
				{
					UIGetString(szText, 254, IDS_STATUS_PGP_ENCRYPT);
					bPluginMenu = TRUE;
					break;
				}
								
				case IDC_SIGN_PGP:
				{
					UIGetString(szText, 254, IDS_STATUS_PGP_SIGN);
					bPluginMenu = TRUE;
					break;
				}

				case IDC_PGPKEYS:
				{
					UIGetString(szText, 254, IDS_TOOLTIP_PGPKEYS);
					bPluginMenu = TRUE;
					break;
				}

			}	

			if (bPluginMenu)
			{
				CallWindowProc(lpOldProc, hDlg, msg, wParam, lParam);
				SendMessage(plugin->hStatusBar, SB_SETTEXT, 
					255 | SBT_NOBORDERS, (LPARAM) szText);

				return 0;
			}

			break;
		}

		case WM_COMMAND:
		{
			switch(LOWORD(wParam))
			{
			case IDC_SENDMESSAGE_MENU:
			case IDC_SENDMESSAGE_BUTTON:
			case IDC_SENDLATER:
			case IDC_POSTARTICLE_MENU:
			case IDC_POSTARTICLE_BUTTON:
			case IDC_POSTLATER:
			case IDC_OE5_SENDMESSAGE:
			case IDC_OE5_SENDLATER:
				{
					if (plugin->bEncrypt || plugin->bSign)
					{
						if (!SaveClipboardText(hDlg))
							return 0;

						plugin->nAction = PluginAction_Encrypt;
						plugin->hDlg = hDlg;
						plugin->nMsg = msg;
						plugin->wParam = wParam;
						plugin->lParam = lParam;

						_beginthread(DoCopyThread, 0, hDlg);
						return 0;
					}

					break;
				}

			case IDC_ENCRYPT_PGP:
				if (plugin->bOE5)
				{
					plugin->bEncrypt = !(plugin->bEncrypt);
					SendMessage(plugin->hToolbar, TB_CHECKBUTTON, 
						IDC_ENCRYPT_PGP, MAKELONG(plugin->bEncrypt, 0));
					return 0;
				}
				else
				{
					MENUITEMINFO menuInfo;

					menuInfo.cbSize = sizeof(MENUITEMINFO);
					menuInfo.fMask = MIIM_STATE;
					GetMenuItemInfo(plugin->hToolsMenu, IDC_ENCRYPT_PGP, 
						FALSE, &menuInfo);
					
					if (menuInfo.fState & MFS_CHECKED)
						plugin->bEncrypt = FALSE;
					else
						plugin->bEncrypt = TRUE;

					UpdateMenus(hDlg, plugin);
					return 0;
				}

			case IDC_SIGN_PGP:
				if (plugin->bOE5)
				{
					plugin->bSign = !(plugin->bSign);
					SendMessage(plugin->hToolbar, TB_CHECKBUTTON, 
						IDC_SIGN_PGP, MAKELONG(plugin->bSign, 0));
					return 0;
				}
				else
				{
					MENUITEMINFO menuInfo;

					menuInfo.cbSize = sizeof(MENUITEMINFO);
					menuInfo.fMask = MIIM_STATE;
					GetMenuItemInfo(plugin->hToolsMenu, IDC_SIGN_PGP, 
						FALSE, &menuInfo);
					
					if (menuInfo.fState & MFS_CHECKED)
						plugin->bSign = FALSE;
					else
						plugin->bSign = TRUE;

					UpdateMenus(hDlg, plugin);
					return 0;
				}
			}
			break;
		}
		
	case WM_INITMENUPOPUP:
		if (plugin->bOE5)
		{
			int nNumItems;
			MENUITEMINFO menuInfo;
			char szEncrypt[256];
			char szMenu[256];
			LRESULT lResult;
			int nIndex;
			BOOL bGotEncrypt = FALSE;
			BOOL bGotPGP = FALSE;
			int nEncryptPos;

			UIGetString(szEncrypt, 254, IDS_MENU_OEENCRYPT);
			nNumItems = GetMenuItemCount((HMENU) wParam);
			
			for (nIndex=0; nIndex<nNumItems; nIndex++)
			{
				menuInfo.cbSize = sizeof(MENUITEMINFO);
				menuInfo.fMask = MIIM_TYPE;
				menuInfo.dwTypeData = szMenu;
				menuInfo.cch = 255;
			
				GetMenuItemInfo((HMENU) wParam, nIndex, TRUE, &menuInfo);
				if (!strcmp(szMenu, szEncrypt))
				{
					bGotEncrypt = TRUE;
					nEncryptPos = nIndex;
					break;
				}

				if (!strcmp(szMenu, "&PGP"))
				{
					bGotPGP = TRUE;
					break;
				}
			}
			
			if (bGotEncrypt)
			{
				lResult = CallWindowProc(lpOldProc, hDlg, msg, wParam, 
							lParam);
				
				strcpy(szMenu, "&PGP");
				
				menuInfo.cbSize = sizeof(MENUITEMINFO);
				menuInfo.fMask = MIIM_TYPE | MIIM_ID | MIIM_SUBMENU | 
									MIIM_STATE;
				menuInfo.fType = MFT_STRING;
				menuInfo.wID = IDC_PGPMENU;
				menuInfo.fState = MFS_ENABLED;
				menuInfo.hSubMenu = plugin->hPGPMenu;
				menuInfo.dwTypeData = szMenu;

				InsertMenuItem((HMENU) wParam, nNumItems, TRUE, &menuInfo);
				plugin->nPGPPosition = nNumItems;

				UIGetString(szMenu, 254, IDS_MENU_SMIME_ENCRYPT);
				menuInfo.fMask = MIIM_TYPE;
				menuInfo.fType = MFT_STRING;
				menuInfo.dwTypeData = szMenu;

				SetMenuItemInfo((HMENU) wParam, nEncryptPos-1, TRUE, 
					&menuInfo);

				UIGetString(szMenu, 254, IDS_MENU_SMIME_SIGN);
				menuInfo.fMask = MIIM_TYPE;
				menuInfo.fType = MFT_STRING;
				menuInfo.dwTypeData = szMenu;

				SetMenuItemInfo((HMENU) wParam, nEncryptPos, TRUE, 
					&menuInfo);

				UIGetString(szMenu, 254, IDS_MENU_PGP_ENCRYPT);
				menuInfo.fMask = MIIM_TYPE | MIIM_ID | MIIM_STATE;
				menuInfo.fType = MFT_STRING;
				menuInfo.wID = IDC_ENCRYPT_PGP;
				menuInfo.dwTypeData = szMenu;
				
				if (plugin->bEncrypt)
					menuInfo.fState = MFS_ENABLED | MFS_CHECKED;
				else
					menuInfo.fState = MFS_ENABLED;

				InsertMenuItem((HMENU) wParam, nEncryptPos+1, TRUE, 
					&menuInfo);

				UIGetString(szMenu, 254, IDS_MENU_PGP_SIGN);
				menuInfo.fMask = MIIM_TYPE | MIIM_ID | MIIM_STATE;
				menuInfo.fType = MFT_STRING;
				menuInfo.wID = IDC_SIGN_PGP;
				menuInfo.dwTypeData = szMenu;
				
				if (plugin->bSign)
					menuInfo.fState = MFS_ENABLED | MFS_CHECKED;
				else
					menuInfo.fState = MFS_ENABLED;

				InsertMenuItem((HMENU) wParam, nEncryptPos+2, TRUE, 
					&menuInfo);

				return lResult;
			}
			else if (bGotPGP)
			{
				lResult = CallWindowProc(lpOldProc, hDlg, msg, wParam, 
							lParam);
				
				menuInfo.cbSize = sizeof(MENUITEMINFO);
				menuInfo.fMask = MIIM_STATE;
				
				if (plugin->bEncrypt)
					menuInfo.fState = MFS_ENABLED | MFS_CHECKED;
				else
					menuInfo.fState = MFS_ENABLED;

				SetMenuItemInfo((HMENU) wParam, IDC_ENCRYPT_PGP, FALSE, 
					&menuInfo);

				if (plugin->bSign)
					menuInfo.fState = MFS_ENABLED | MFS_CHECKED;
				else
					menuInfo.fState = MFS_ENABLED;

				SetMenuItemInfo((HMENU) wParam, IDC_SIGN_PGP, FALSE, 
					&menuInfo);
				
				menuInfo.fState = MFS_ENABLED;

				SetMenuItemInfo((HMENU) wParam, IDC_PGPMENU, FALSE, 
					&menuInfo);
				
				SetMenuItemInfo(plugin->hPGPMenu, IDC_PREFS, FALSE, 
					&menuInfo);
				
				SetMenuItemInfo(plugin->hPGPMenu, IDC_PGPKEYS, FALSE, 
					&menuInfo);
				
				return lResult;
			}
		}
		break;

	case WM_NOTIFY:
		if ((((LPNMHDR) lParam)->code) == TTN_GETDISPINFO) 
		{
			int idCtrl;
			NMTTDISPINFO *pInfo;
			BOOL bTooltip = FALSE;

			idCtrl = ((LPNMHDR) lParam)->idFrom;
			pInfo = (NMTTDISPINFO *) lParam;

			switch (idCtrl)
			{
			case IDC_ENCRYPT_SMIME:
				pInfo->lpszText = (LPTSTR) IDS_TOOLTIP_SMIME_ENCRYPT;
				bTooltip = TRUE;
				break;

			case IDC_SIGN_SMIME:
				pInfo->lpszText = (LPTSTR) IDS_TOOLTIP_SMIME_SIGN;
				bTooltip = TRUE;
				break;

			case IDC_ENCRYPT_PGP:
				pInfo->lpszText = (LPTSTR) IDS_TOOLTIP_PGP_ENCRYPT;
				bTooltip = TRUE;
				break;

			case IDC_SIGN_PGP:
				pInfo->lpszText = (LPTSTR) IDS_TOOLTIP_PGP_SIGN;
				bTooltip = TRUE;
				break;

			case IDC_PGPKEYS:
				pInfo->lpszText = (LPTSTR) IDS_TOOLTIP_PGPKEYS;
				bTooltip = TRUE;
				break;
			}

			if (bTooltip)
			{
				pInfo->hinst = UIGetInstance();
				return 0;
			}
		}
		break;
	}

	return CommonWndProc(hDlg, msg, wParam, lParam);
}


BOOL EncryptSignMessage(HWND hwnd)
{
	HWND hAttach;
	char *szInput = NULL;
	char *szOutput;
	DWORD dwLength;
	PGPSize nOutLength;
	PGPclRecipientDialogStruct *prds = NULL;
	PGPOptionListRef signOptions = NULL;
	PluginInfo *plugin;
	BOOL bSuccess = FALSE;
	char szExe[256];
	char szDll[256];
	PGPError nError = kPGPError_NoErr;
	
	UIGetString(szExe, sizeof(szExe), IDS_EXE);
	UIGetString(szDll, sizeof(szDll), IDS_DLL);
	
	plugin = GetPluginInfo(hwnd);

	if (plugin->bOE5)
	{
		HWND hContainer;

		hContainer = FindWindowEx(hwnd, NULL, "OE_Envelope", NULL);
		hAttach = FindWindowEx(hContainer, NULL, "SysListView32", NULL);
	}
	else
		hAttach = FindWindowEx(hwnd, NULL, "SysListView32", NULL);

	if (hAttach)
		if (ListView_GetItemCount(hAttach) > 0)
			if (!UIWarnUser(hwnd, IDS_W_ATTACHMENT, "Attachment"))
				return FALSE;

	prds = (PGPclRecipientDialogStruct *) 
			calloc(sizeof(PGPclRecipientDialogStruct), 1);
	if(NULL == prds)
		goto EncryptSignMessageError;

	nError = PGPclOpenDefaultKeyrings(plugin->pgpContext, 
				kPGPOpenKeyDBFileOptions_Mutable, &(prds->keydbOriginal));

	if (IsPGPError(nError))
		nError = PGPclOpenDefaultKeyrings(plugin->pgpContext, 
					kPGPOpenKeyDBFileOptions_None, &(prds->keydbOriginal));

	if (IsPGPError(nError))
	{
		UIDisplayErrorCode(__FILE__, __LINE__, szDll, nError, TRUE);
		goto EncryptSignMessageError;
	}

	if (!GetMessageText(hwnd, FALSE, &szInput))
		goto EncryptSignMessageError;

	dwLength = strlen(szInput);
	FixBadSpaces(szInput);

	if (plugin->bEncrypt)
	{
		char *szRecipients;

		if (!GetRecipientText(hwnd, &szRecipients))
			goto EncryptSignMessageError;

		nError = GetRecipients(hwnd, szRecipients, plugin->pgpContext, 
					plugin->tlsContext, prds);
		free(szRecipients);
	}

	if (IsPGPError(nError))
	{
		if (nError != kPGPError_UserAbort)
			UIDisplayErrorCode(__FILE__, __LINE__, szDll, 
				nError, FALSE);

		goto EncryptSignMessageError; 
	}
			
	nError = EncryptSignBuffer(UIGetInstance(), hwnd, plugin->pgpContext, 
				plugin->tlsContext, szExe, szDll, szInput, dwLength, prds, 
				NULL, &signOptions, (void **) &szOutput, &nOutLength, 
				plugin->bEncrypt, plugin->bSign, FALSE, FALSE);

	free(szInput);
	szInput = NULL;

	if (IsntPGPError(nError))
	{
		if (nOutLength)
		{
			char *szTemp;

			szTemp = (char *) PGPNewData(plugin->memoryMgr, nOutLength+3, 0);
			strcpy(&szTemp[2], szOutput);
			szTemp[0] = '\r';
			szTemp[1] = '\n';
			PGPFreeData(szOutput);
			szOutput = szTemp;
			nOutLength += 2;
		}

		if (!SetMessageText(hwnd, szOutput))
			PGPFreeData(szOutput);
		else
		{
			if (plugin->szOutput != NULL)
			{
				free(plugin->szOutput);
				plugin->szOutput = NULL;
			}

			plugin->szOutput = (char *) calloc(nOutLength + 1, 1);
			if(NULL != plugin->szOutput)
			{
				strcpy(plugin->szOutput, szOutput);
				PGPFreeData(szOutput);
				bSuccess = TRUE;//success
			}
			else
				PGPFreeData(szOutput);//failure since we could not allocate
		}
	}

EncryptSignMessageError:

	if (signOptions != NULL)
	{
		PGPFreeOptionList(signOptions);
		signOptions = NULL;
	}

	if(NULL != prds)
	{
		if (prds->keydbOriginal != NULL)
		{
			PGPFreeKeyDB(prds->keydbOriginal);
			prds->keydbOriginal = NULL;
		}

		FreeRecipients(prds);
		free(prds);
		prds = NULL;
	}

	if (szInput != NULL)
		free(szInput);

	return bSuccess;
}


void DoubleLineFeeds(char *szOriginal, char *szNew)
{
	int nOrigIndex = 0;
	int nNewIndex = 0;
	BOOL bCRLF = FALSE;

	while (szOriginal[nOrigIndex] != '\0')
	{
		szNew[nNewIndex++] = szOriginal[nOrigIndex++];
		if (szOriginal[nOrigIndex-1] == '\n')
		{
			if (bCRLF)
			{
				szNew[nNewIndex++] = '\r';
				szNew[nNewIndex++] = '\n';
				bCRLF = FALSE;
			}
			else
				bCRLF = TRUE;
		}
	}

	szNew[nNewIndex] = '\0';
	return;
}


static void UpdateMenus(HWND hwnd, PluginInfo *plugin)
{
	if (plugin->bEncrypt)
	{
		CheckMenuItem(plugin->hToolsMenu, IDC_ENCRYPT_PGP, 
			MF_BYCOMMAND | MF_CHECKED);
		SendMessage(plugin->hToolbar, TB_CHECKBUTTON, IDC_ENCRYPT_PGP,
			MAKELONG(TRUE, 0));
	}
	else
	{
		CheckMenuItem(plugin->hToolsMenu, IDC_ENCRYPT_PGP, 
			MF_BYCOMMAND | MF_UNCHECKED);
		SendMessage(plugin->hToolbar, TB_CHECKBUTTON, IDC_ENCRYPT_PGP,
			MAKELONG(FALSE, 0));
	}
	
	if (plugin->bSign)
	{
		CheckMenuItem(plugin->hToolsMenu, IDC_SIGN_PGP, 
			MF_BYCOMMAND | MF_CHECKED);
		SendMessage(plugin->hToolbar, TB_CHECKBUTTON, IDC_SIGN_PGP,
			MAKELONG(TRUE, 0));
	}
	else
	{
		CheckMenuItem(plugin->hToolsMenu, IDC_SIGN_PGP, 
			MF_BYCOMMAND | MF_UNCHECKED);
		SendMessage(plugin->hToolbar, TB_CHECKBUTTON, IDC_SIGN_PGP,
			MAKELONG(FALSE, 0));
	}
	
}


LRESULT CALLBACK SendToolbarParentWndProc(HWND hDlg, 
										  UINT msg,
										  WPARAM wParam, 
										  LPARAM lParam)
{
	WNDPROC lpOldProc;
	PluginInfo *plugin;
	LPNMHDR lpnm;
	LPNMTOOLBAR lptb;

	lpOldProc = (WNDPROC)GetProp( hDlg, "oldproc" );
	plugin = GetPluginInfo(hDlg);

	lpnm = (LPNMHDR) lParam;
	lptb = (LPNMTOOLBAR) lParam;

	switch(msg)
	{
		case WM_NOTIFY:
			{
				switch(lpnm->code)
				{
					case TBN_BEGINADJUST:
						//if we have inserted our stuff into the toolbar already
						if(plugin->dwInitState & PIIF_TOOLSINSERTION_DONE)
							RemoveSendToolbarButtons(plugin);//de-initialize/remove stuff
						break;

					case TBN_ENDADJUST:
						{
							//DWORD dwSize;
							//int nX;
							//int nY;
							LRESULT lResult;
							
							lResult = CallWindowProc(lpOldProc, hDlg, msg, wParam, 
										lParam);

							//if we have not inserted our stuff into the toolbar already
							if(!(plugin->dwInitState & PIIF_TOOLSINSERTION_DONE))
							{
								//insert our stuff as necessary
								pgpAssert(-1 == plugin->nPGPKeysButton);
								AddSendToolbarButtons(plugin);								
							}

								
							/*dwSize = SendMessage(plugin->hToolbar, TB_GETBUTTONSIZE, 
										0, 0);
							nX = LOWORD(dwSize);
							nY = HIWORD(dwSize);
							
							SendMessage(plugin->hToolbar, TB_ADDBUTTONS, 4, 
								(LPARAM) &tbb);
							
							SendMessage(plugin->hToolbar, TB_SETBUTTONSIZE, 0, 
								MAKELONG(nX, nY));*/
							
							return lResult;
						}//case TBN_ENDADJUST

						break;

					default:
						break;
				}
			}
			break;
		default:
			break;
	}

	return CallWindowProc(lpOldProc, hDlg, msg, wParam, lParam);
}


void AddSendToolbarButtons(PluginInfo *lppilPlugInf)
{
	TBBUTTON tbb[4]={0};
	char szText[255] = {0};
	int nX = -1;
	int nEncryptBMP = -1;
	int nSignBMP = -1;
	int nPGPkeysBMP = -1;
	LONG lResult = 0;
	int nEncryptSTR = -1;
	int nSignSTR = -1;
	int nPGPkeysSTR = -1;
	int nRet = 0;

	//we should not be called out of sequence
	pgpAssert(lppilPlugInf->dwInitState & PIIF_MAINUIELEMENTS_FOUND);
	pgpAssert(!(lppilPlugInf->dwInitState & PIIF_TOOLSINSERTION_DONE));
	if(lppilPlugInf->dwInitState & PIIF_TOOLSINSERTION_DONE)
		return;//we have already inserted ourselves to the toolbar
	
	//get the no of buttons in the toolbar currently
	//nX = SendMessage(lppilPlugInf->hToolbar, TB_BUTTONCOUNT, 0, 0);
	//if (nX < 2)//if less than 2 buttons
	//	return;//bail. try again later ?!!

	//we should not be there already. alternately OE could not be using
	//our command id for itself
	pgpAssert(-1 == SendMessage(lppilPlugInf->hToolbar, TB_COMMANDTOINDEX, IDC_ENCRYPT_PGP, 0));
	pgpAssert(-1 == SendMessage(lppilPlugInf->hToolbar, TB_COMMANDTOINDEX, IDC_SIGN_PGP, 0));
	pgpAssert(-1 == SendMessage(lppilPlugInf->hToolbar, TB_COMMANDTOINDEX, IDC_PGPKEYS, 0));
	
	//add the encrypt bitmap to the imagelists
	nEncryptBMP = LoadAppropriateBitmaps(lppilPlugInf, IDC_ENCRYPT_PGP);
	pgpAssert(-1 != nEncryptBMP);

	//remember the encrypt image index
	pgpAssert(-1 == lppilPlugInf->nEncryptImage);
	lppilPlugInf->nEncryptImage = nEncryptBMP;

	//load the sign bitmap
	nSignBMP = LoadAppropriateBitmaps(lppilPlugInf, IDC_SIGN_PGP);
	pgpAssert(-1 != nSignBMP);

	//remember the sign bitmap image index
	pgpAssert(-1 == lppilPlugInf->nSignImage);
	lppilPlugInf->nSignImage = nSignBMP;

	//load the pgpkeys bitmap
	nPGPkeysBMP = LoadAppropriateBitmaps(lppilPlugInf, IDC_PGPKEYS);
	pgpAssert(-1 != nPGPkeysBMP);

	//remember the keys bitmap image index
	pgpAssert(-1 == lppilPlugInf->nPGPKeysImage);
	lppilPlugInf->nPGPKeysImage = nPGPkeysBMP;

	//set the normal image list back
	//SendMessage(lppilPlugInf->hToolbar, TB_SETIMAGELIST, 0, (LPARAM) hilToolbar);
	
	if (lppilPlugInf->bOE5)
	{
		TBBUTTON tbOE;
		int nIndex;
		int nEncryptSMIME;
		int nSignSMIME;
		
		//set the hot imagelist back. this is not necessary since we are not creating
		//a separate imagelist. just appending to the existing
		//SendMessage(lppilPlugInf->hToolbar, TB_SETHOTIMAGELIST, 0, (LPARAM) hilHotToolbar);
		
		//add the strings for encrypt and sign and pgpkeys

		//if we have not already added the string for encrypt button
		if(-1 == lppilPlugInf->nEncryptString)
		{
			UIGetString(szText, 254, IDS_BUTTON_PGP_ENCRYPT);
			nEncryptSTR = SendMessage(lppilPlugInf->hToolbar, TB_ADDSTRING, 0, (LPARAM) szText);
			pgpAssert(-1 != nEncryptSTR);

			lppilPlugInf->nEncryptString = nEncryptSTR;
		}

		//if we have not already added the string for sign button
		if(-1 == lppilPlugInf->nSignString)
		{
			UIGetString(szText, 254, IDS_BUTTON_PGP_SIGN);
			nSignSTR = SendMessage(lppilPlugInf->hToolbar, TB_ADDSTRING, 0, (LPARAM) szText);
			pgpAssert(-1 != nSignSTR);

			lppilPlugInf->nSignString = nSignSTR;
		}

		//if we have not already added the string for pgpkeys button
		if(-1 == lppilPlugInf->nPGPKeysString)
		{
			UIGetString(szText, 254, IDS_TOOLTIP_PGPKEYS);
			nPGPkeysSTR = SendMessage(lppilPlugInf->hToolbar, TB_ADDSTRING, 0, (LPARAM) szText);
			pgpAssert(-1 != nPGPkeysSTR);

			lppilPlugInf->nPGPKeysString = nPGPkeysSTR;
		}	
		
		//find the smime encrypt button of OE
		nIndex = SendMessage(lppilPlugInf->hToolbar, TB_COMMANDTOINDEX,
					IDC_OE5_ENCRYPT_SMIME, 0);
		if (nIndex > -1)//if found
		{
			//add our smime string to the toolbar
			UIGetString(szText, 254, IDS_BUTTON_SMIME_ENCRYPT);
			nEncryptSMIME = SendMessage(lppilPlugInf->hToolbar, TB_ADDSTRING, 
								0, (LPARAM) szText);
			
			//get smime button details
			SendMessage(lppilPlugInf->hToolbar, TB_GETBUTTON, nIndex,
				(LPARAM) &tbOE);
			
			//set the text to our text
			tbOE.iString = nEncryptSMIME;

			//delete button and reinsert
			SendMessage(lppilPlugInf->hToolbar, TB_DELETEBUTTON, nIndex, 0);
			SendMessage(lppilPlugInf->hToolbar, TB_INSERTBUTTON, nIndex, 
				(LPARAM) &tbOE);
		}
		
		//find the smime sign button of OE
		nIndex = SendMessage(lppilPlugInf->hToolbar, TB_COMMANDTOINDEX,
					IDC_OE5_SIGN_SMIME, 0);
		if (nIndex > -1) //if found
		{
			//add our smime sign string to toolbar
			UIGetString(szText, 254, IDS_BUTTON_SMIME_SIGN);
			nSignSMIME = SendMessage(lppilPlugInf->hToolbar, TB_ADDSTRING, 0,
				(LPARAM) szText);
			
			//get smime sign button details
			SendMessage(lppilPlugInf->hToolbar, TB_GETBUTTON, nIndex,
				(LPARAM) &tbOE);
			
			tbOE.iString = nSignSMIME;//set the text to our text

			//delete button and reinsert
			SendMessage(lppilPlugInf->hToolbar, TB_DELETEBUTTON, nIndex, 0);
			SendMessage(lppilPlugInf->hToolbar, TB_INSERTBUTTON, nIndex, 
				(LPARAM) &tbOE); 
		}
	}//end if (lppilPlugInf->bOE5)
	
	//add our buttons with a separator from the rest
	tbb[0].iBitmap = -1;
	tbb[0].idCommand = -1;
	tbb[0].fsState = TBSTATE_ENABLED;
	tbb[0].fsStyle = TBSTYLE_SEP;
	tbb[0].dwData = 0;
	tbb[0].iString = -1;
	
	tbb[1].iBitmap = nEncryptBMP;
	tbb[1].idCommand = IDC_ENCRYPT_PGP;
	tbb[1].fsState = TBSTATE_ENABLED;
	tbb[1].fsStyle = TBSTYLE_CHECK;
	tbb[1].dwData = 0;
	tbb[1].iString = lppilPlugInf->nEncryptString;
	
	tbb[2].iBitmap = nSignBMP;
	tbb[2].idCommand = IDC_SIGN_PGP;
	tbb[2].fsState = TBSTATE_ENABLED;
	tbb[2].fsStyle = TBSTYLE_CHECK;
	tbb[2].dwData = 0;
	tbb[2].iString = lppilPlugInf->nSignString;

	tbb[3].iBitmap = nPGPkeysBMP;
	tbb[3].idCommand = IDC_PGPKEYS;
	tbb[3].fsState = TBSTATE_ENABLED;
	tbb[3].fsStyle = TBSTYLE_BUTTON;
	tbb[3].dwData = 0;
	tbb[3].iString = lppilPlugInf->nPGPKeysString;
	
	//we must send TB_BUTTONSTRUCTSIZE before sending TB_ADDBITMAP 
	//or TB_ADDBUTTONS  message NOTE: this message has side effects
	SendMessage(lppilPlugInf->hToolbar, TB_BUTTONSTRUCTSIZE, 
			(WPARAM) sizeof(TBBUTTON), 0);

	//add the buttons
	lResult = SendMessage(lppilPlugInf->hToolbar, TB_ADDBUTTONS, 4, (LPARAM) &tbb);
	pgpAssert(TRUE == lResult);
	if(TRUE == lResult)//if successful
	{
		//retrieve and store the index of pgpkeys button
		pgpAssert(-1 == lppilPlugInf->nPGPKeysButton);
		lppilPlugInf->nPGPKeysButton = SendMessage(lppilPlugInf->hToolbar,
								TB_COMMANDTOINDEX, IDC_PGPKEYS,	0);
		pgpAssert(-1 != lppilPlugInf->nPGPKeysButton);
		if(-1 != lppilPlugInf->nPGPKeysButton)//if we could get the index of pgpkeys button
			lppilPlugInf->dwInitState |= PIIF_TOOLSINSERTION_DONE;//all success
	}


	if (lppilPlugInf->bOE5)
	{
		int nIndex = -1;
		TBBUTTON tb = {0};

		nIndex = SendMessage(lppilPlugInf->hToolbar, TB_COMMANDTOINDEX,
					IDC_ENCRYPT_PGP, 0);
		if(-1 != nIndex)
		{
			lResult = SendMessage(lppilPlugInf->hToolbar, TB_GETBUTTON, nIndex - 2, 
				(LPARAM) &tb);

			pgpAssert(TRUE == lResult);
			pgpAssert(tb.fsStyle & TBSTYLE_SEP);
			if ((TRUE == lResult) && (tb.fsStyle & TBSTYLE_SEP))
				SendMessage(lppilPlugInf->hToolbar, TB_DELETEBUTTON, nIndex - 2, 0);
		}

		/*if (nY > 26)
		{
			SendMessage(lppilPlugInf->hToolbar, TB_SETBUTTONSIZE, 0, 
				MAKELONG(nX, nY));
			lppilPlugInf->nButtonSizeX = nX;
			lppilPlugInf->nButtonSizeY = nY;
		}*/
	}
	
	return;
}

void RemoveSendToolbarButtons(PluginInfo *lppilPlugInf)
{
	HIMAGELIST hTbImgList = NULL, hTbHotImgList = NULL;
	BOOL bRet = FALSE;
	INT iIndex = -1;
	LONG lResult = 0;
	TBBUTTON tbBtn={0};

	//check parameters
	pgpAssert(NULL != lppilPlugInf);
	if(NULL == lppilPlugInf)
		return;

	//if we have not inserted our stuff into the toolbar already
	if(!(lppilPlugInf->dwInitState & PIIF_TOOLSINSERTION_DONE))
		return;

	pgpAssert(-1 != lppilPlugInf->nPGPKeysButton);

	//REMOVE THE BUTTONS

	//find the button index for pgp keys button
	iIndex = SendMessage(lppilPlugInf->hToolbar, TB_COMMANDTOINDEX,
				IDC_PGPKEYS, 0);
	pgpAssert(-1 != iIndex);
	
	if(-1 != iIndex)
	{
		//the button index should be the same as we know already
		pgpAssert(iIndex == lppilPlugInf->nPGPKeysButton);

		//delete the button 
		lResult = SendMessage(lppilPlugInf->hToolbar, TB_DELETEBUTTON, iIndex, 0);
		pgpAssert(TRUE == lResult);

		if(TRUE == lResult)
			lppilPlugInf->nPGPKeysButton = -1;
	}

	//find the button index for sign button
	iIndex = SendMessage(lppilPlugInf->hToolbar, TB_COMMANDTOINDEX,
				IDC_SIGN_PGP, 0);
	pgpAssert(-1 != iIndex);
	
	if(-1 != iIndex)
	{
		//delete the sign button 
		lResult = SendMessage(lppilPlugInf->hToolbar, TB_DELETEBUTTON, iIndex, 0);
		pgpAssert(TRUE == lResult);
	}

	//find the button index for encrypt button
	iIndex = SendMessage(lppilPlugInf->hToolbar, TB_COMMANDTOINDEX,
				IDC_ENCRYPT_PGP, 0);
	pgpAssert(-1 != iIndex);
	
	if(-1 != iIndex)
	{
		//delete the encrypt button 
		lResult = SendMessage(lppilPlugInf->hToolbar, TB_DELETEBUTTON, iIndex, 0);
		pgpAssert(TRUE == lResult);
	}

	//remove the separator before the encrypt button (that we had inserted)
	lResult = SendMessage(lppilPlugInf->hToolbar, TB_GETBUTTON, iIndex-1, (LPARAM) &tbBtn);
	pgpAssert(TRUE == lResult);
	pgpAssert(tbBtn.fsStyle & TBSTYLE_SEP);
	if ((TRUE == lResult) && (tbBtn.fsStyle & TBSTYLE_SEP))
		SendMessage(lppilPlugInf->hToolbar, TB_DELETEBUTTON, iIndex - 1, 0);


	//REMOVE BITMAP INSERTIONS FOR THE BUTTONS

	//get the normal imagelist
	hTbImgList = (HIMAGELIST) SendMessage(lppilPlugInf->hToolbar, 
										TB_GETIMAGELIST, 0, 0);
	pgpAssert(NULL != hTbImgList);

	//get the hot imagelist
	hTbHotImgList = (HIMAGELIST) SendMessage(lppilPlugInf->hToolbar, 
										TB_GETHOTIMAGELIST, 0, 0);
	pgpAssert(NULL != hTbHotImgList);


	//remove the pgpkeys image from both image lists
	pgpAssert(-1 != lppilPlugInf->nPGPKeysImage);
	if( -1 != lppilPlugInf->nPGPKeysImage)
	{
		bRet = ImageList_Remove(hTbImgList, lppilPlugInf->nPGPKeysImage);
		pgpAssert(0 != bRet);

		bRet = ImageList_Remove(hTbHotImgList, lppilPlugInf->nPGPKeysImage);
		pgpAssert(0 != bRet);

		lppilPlugInf->nPGPKeysImage = -1;
	}

	//remove the sign image from both image lists
	pgpAssert(-1 != lppilPlugInf->nSignImage);
	if( -1 != lppilPlugInf->nSignImage)
	{
		bRet = ImageList_Remove(hTbImgList, lppilPlugInf->nSignImage);
		pgpAssert(0 != bRet);

		bRet = ImageList_Remove(hTbHotImgList, lppilPlugInf->nSignImage);
		pgpAssert(0 != bRet);

		lppilPlugInf->nSignImage = -1;
	}

	//remove the encrypt image from both image lists
	pgpAssert(-1 != lppilPlugInf->nEncryptImage);
	if( -1 != lppilPlugInf->nEncryptImage)
	{
		bRet = ImageList_Remove(hTbImgList, lppilPlugInf->nEncryptImage);
		pgpAssert(0 != bRet);

		bRet = ImageList_Remove(hTbHotImgList, lppilPlugInf->nEncryptImage);
		pgpAssert(0 != bRet);

		lppilPlugInf->nEncryptImage = -1;
	}

	
	//REMOVE THE STRINGS ADDED TO TOOLBAR

	//there is currently no way to do this. TODO: change stuff so that we
	//do not need to have this side effect

	//state maintenance - toolbar insertion is not done anymore
	lppilPlugInf->dwInitState &= (~PIIF_TOOLSINSERTION_DONE);
}

int LoadAppropriateBitmaps(PluginInfo *plugin, UINT uiCommandId)
{
	HDC hDC=NULL;
	int iBpp=0, iImgIndex=-1, iHotImgIndex=-1;
	int iRsrcId=-1, iHotRsrcId=-1;
	int iIconWidth=0, iIconHeight=0;
	int iHotIconWidth=0, iHotIconHeight=0;
	HIMAGELIST hTbImgList=NULL, hTbHotImgList=NULL;
	HBITMAP hbmBitMap=NULL, hbmHotBitMap=NULL;

	//check parameters
	pgpAssert((NULL != plugin) && (NULL != plugin->hToolbar));
	if((NULL == plugin) || (NULL == plugin->hToolbar))
		return -1;//bail 

	//get the normail imagelist of the toolbar
	hTbImgList = (HIMAGELIST) SendMessage(plugin->hToolbar, TB_GETIMAGELIST, 0, 0);
	pgpAssert(NULL != hTbImgList);
	if(NULL == hTbImgList)
		return -1;

	//get the hot image list on OE5 onwards
	if(TRUE == plugin->bOE5)
	{
		//get the hot image list of the toolbar
		hTbHotImgList = (HIMAGELIST) SendMessage(plugin->hToolbar, TB_GETHOTIMAGELIST, 0, 0);
		pgpAssert(NULL != hTbHotImgList);
	}


	//if the version is prior to OE5
	if (FALSE == plugin->bOE5)
	{
		switch(uiCommandId)
		{
			case IDC_ENCRYPT_PGP:
				iRsrcId = IDB_ENCRYPT;
				break;
			case IDC_SIGN_PGP:
				iRsrcId = IDB_SIGN;
				break;
			case IDC_PGPKEYS:
				iRsrcId = IDB_PGPKEYS;
				break;
			case IDC_DECRYPT:
				iRsrcId = IDB_DECRYPT;
				break;
			default:
				break;
		}

		goto loadbitmap;
	}

	//NOTE: plugin->bOE5 is true henceforth until loadbitmap

	//get the bpp (bits per pixel)
	hDC = GetDC(NULL);// DC for desktop
	pgpAssert(NULL != hDC);

	if(NULL != hDC)
	{
		iBpp = GetDeviceCaps(hDC, BITSPIXEL) * GetDeviceCaps (hDC, PLANES);
		pgpAssert(iBpp > 0);

		ReleaseDC(NULL, hDC);
		hDC = NULL;
	}

	//get the size of icons in the imagelist. In OE6 - big icons seem to be at 24w x 24h
	//and small icons at 16w x 24h. (geticonsize on the normal imagelist). In OE5 - big
	//icons are at 22w x 20h and small icons at 16w x 16h. So we treat width less than 20 
	//to be small icon here (ssd)
	ImageList_GetIconSize(hTbImgList, &iIconWidth, &iIconHeight);
	ImageList_GetIconSize(hTbHotImgList, &iHotIconWidth, &iHotIconHeight);
	pgpAssert(iHotIconWidth <= iIconWidth);
	pgpAssert(iHotIconHeight <= iIconHeight);

	//if we dont have these icons big icons do not work in OE6
	if (iIconHeight == 24)
	{
		switch(uiCommandId)
		{
			case IDC_ENCRYPT_PGP:
				iRsrcId = (iIconWidth < 20)?IDB_ENCRYPTXPO:IDB_ENCRYPTXPOFF;//IDB_ENCRYPTXPO:IDB_ENCRYPTXPOFF
				iHotRsrcId = (iIconWidth < 20)?IDB_ENCRYPTXP:IDB_ENCRYPTXPLARGE;//IDB_ENCRYPTXP:IDB_ENCRYPTXPLARGE
				break;
			case IDC_SIGN_PGP:
				iRsrcId = (iIconWidth < 20)?IDB_SIGNXPO:IDB_SIGNXPOFF;
				iHotRsrcId = (iIconWidth < 20)?IDB_SIGNXP:IDB_SIGNXPLARGE;
				break;
			case IDC_PGPKEYS:
				iRsrcId = (iIconWidth < 20)?IDB_PGPKEYSXPO:IDB_PGPKEYSXPOFF;
				iHotRsrcId = (iIconWidth < 20)?IDB_PGPKEYSXP:IDB_PGPKEYSXPLARGE;
				break;
			case IDC_DECRYPT:
				iRsrcId = (iIconWidth < 20)?IDB_DECRYPTXPO:IDB_DECRYPTXPOFF;
				iHotRsrcId = (iIconWidth < 20)?IDB_DECRYPTXP:IDB_DECRYPTXPLARGE;
				break;
				
			default:
				break;
		}
		goto loadbitmap;
	}

	if (iBpp > 8)
	{
		switch(uiCommandId)
		{
			case IDC_ENCRYPT_PGP:
				iRsrcId = (iIconWidth < 20)?IDB_ENCRYPTO:IDB_ENCRYPTOFF;
				iHotRsrcId = (iIconWidth < 20)?IDB_ENCRYPT:IDB_ENCRYPTLARGE;
				break;
			case IDC_SIGN_PGP:
				iRsrcId = (iIconWidth < 20)?IDB_SIGNO:IDB_SIGNOFF;
				iHotRsrcId = (iIconWidth < 20)?IDB_SIGN:IDB_SIGNLARGE;
				break;
			case IDC_PGPKEYS:
				iRsrcId = (iIconWidth < 20)?IDB_PGPKEYSO:IDB_PGPKEYSOFF;
				iHotRsrcId = (iIconWidth < 20)?IDB_PGPKEYS:IDB_PGPKEYSLARGE;
				break;
			case IDC_DECRYPT:
				iRsrcId = (iIconWidth < 20)?IDB_DECRYPTO:IDB_DECRYPTOFF;
				iHotRsrcId = (iIconWidth < 20)?IDB_DECRYPT:IDB_DECRYPTLARGE;
				break;
			default:
				break;
		}
		goto loadbitmap;
	}
	else
	{
		switch(uiCommandId)
		{
			case IDC_ENCRYPT_PGP:
				iRsrcId = (iIconWidth < 20)?IDB_ENCRYPTO4:IDB_ENCRYPTOFF4;
				iHotRsrcId = (iIconWidth < 20)?IDB_ENCRYPT4:IDB_ENCRYPTLARGE4;
				break;
			case IDC_SIGN_PGP:
				iRsrcId = (iIconWidth < 20)?IDB_SIGNO4:IDB_SIGNOFF4;
				iHotRsrcId = (iIconWidth < 20)?IDB_SIGN4:IDB_SIGNLARGE4;
				break;
			case IDC_PGPKEYS:
				iRsrcId = (iIconWidth < 20)?IDB_PGPKEYSO4:IDB_PGPKEYSOFF4;
				iHotRsrcId = (iIconWidth < 20)?IDB_PGPKEYS4:IDB_PGPKEYSLARGE4;
				break;
			case IDC_DECRYPT:
				iRsrcId = (iIconWidth < 20)?IDB_DECRYPTO4:IDB_DECRYPTOFF4;
				iHotRsrcId = (iIconWidth < 20)?IDB_DECRYPT4:IDB_DECRYPTLARGE4;
				break;
			default:
				break;
		}
		goto loadbitmap;
	}

	
	loadbitmap:


	//if we have found the resouce to load
	if(-1 != iRsrcId)
	{
		//load the bitmap
		pgpAssert(NULL == hbmBitMap);
		hbmBitMap = LoadBitmap(UIGetInstance(), MAKEINTRESOURCE(iRsrcId));
		pgpAssert(NULL != hbmBitMap);
	}

	if(-1 != iHotRsrcId)
	{
		int iImgCnt = 0, iHotImgCnt = 0;//image counts

		//load the bitmap
		pgpAssert(NULL == hbmHotBitMap);
		hbmHotBitMap = LoadBitmap(UIGetInstance(), MAKEINTRESOURCE(iHotRsrcId));
		pgpAssert(NULL != hbmHotBitMap);

		//check if the imagelists are of same size

		//get the image counts of both lists first
		iImgCnt = ImageList_GetImageCount(hTbImgList),
		iHotImgCnt = ImageList_GetImageCount(hTbHotImgList);

		if(iImgCnt != iHotImgCnt)//if not same size
		{
			//indexes returned will be different when we call ImageList_AddMasked
			//we need to take care of this before calling AddMasked on the real
			//bitmaps

			//load some other bitmap. this will be used to fill up the difference 
			//this is necessary since AddMasked changes the hBitmap passed
			HBITMAP hbmTempBmp = LoadImage(UIGetInstance(), MAKEINTRESOURCE(IDB_SIGNXP), 
				IMAGE_BITMAP, 0, 0, LR_LOADTRANSPARENT);
			pgpAssert(NULL != hbmTempBmp);
			
			//until both lists are of equal size
			while(iHotImgCnt != iImgCnt)
			{
				if(iHotImgCnt < iImgCnt)
				{
					//add the temp bitmap to hot imagelist
					iHotImgCnt = ImageList_AddMasked(hTbHotImgList, hbmTempBmp, TRANSPARENT_COLOR);
					iHotImgCnt++;
				}
				else
				{
					//add the temp bitmap to the normal imagelist
					iImgCnt = ImageList_AddMasked(hTbImgList, hbmTempBmp, TRANSPARENT_COLOR);
					iImgCnt++;
				}
			}

			//cleanup
			if(NULL != hbmTempBmp)
			{
				DeleteObject(hbmTempBmp);
				hbmTempBmp=NULL;
			}
		}
	}


	//add the bitmap to the imagelist
	iImgIndex = ImageList_AddMasked(hTbImgList, hbmBitMap, TRANSPARENT_COLOR);

	//add the hot bitmap if that is what we need to do here
	if((TRUE == plugin->bOE5) && (NULL != hbmHotBitMap))
	{
		pgpAssert(NULL != hTbHotImgList);
		iHotImgIndex = ImageList_AddMasked(hTbHotImgList, hbmHotBitMap, TRANSPARENT_COLOR);

		//should be of the same index as the normal list
		pgpAssert(iHotImgIndex == iImgIndex);
	}

	//cleanup
	if(NULL != hbmBitMap)
	{
		
		DeleteObject(hbmBitMap);
		hbmBitMap = NULL;
	}

	if(NULL != hbmHotBitMap)
	{
		
		DeleteObject(hbmHotBitMap);
		hbmHotBitMap = NULL;
	}

	return iImgIndex;
}

void DumpToolbarInfo(HWND hTBWnd)
{
	int iNoOfButtons=0;
	int iIndex=0;
	LONG lResult=0;
	COLORSCHEME csClrScheme={sizeof(COLORSCHEME), 0};
	TBBUTTON tbbBtn={0};
	TBBUTTONINFO tbbBtnInfo={0};
	char szText[255]={0};

	PGPOeTrace("Dumping details of toolbar hWnd = %#x\n", hTBWnd);
	PGPOeTrace("============================================\n");
	PGPOeTrace("\tNoOfButtons=%d\n", 
			iNoOfButtons = SendMessage(hTBWnd, TB_BUTTONCOUNT, 0, 0));

	PGPOeTrace("\tUsingUnicode=%s\n", 
			(0 == SendMessage(hTBWnd, TB_GETUNICODEFORMAT, 0, 0))?"NO":"YES");

	lResult = SendMessage(hTBWnd, TB_GETBITMAPFLAGS, 0, 0);
	PGPOeTrace("\tBitmapFlags= ");
	if(lResult & TBBF_LARGE)
		PGPOeTrace("TBBF_LARGE ");
	PGPOeTrace("\n");
	
	lResult = SendMessage(hTBWnd, TB_GETCOLORSCHEME, 0, (LPARAM)&csClrScheme);
	PGPOeTrace("\tColorScheme= clrBtnHighlight:%#x clrBtnShadow:%#x \n",
		csClrScheme.clrBtnHighlight, csClrScheme.clrBtnShadow);

	lResult = SendMessage(hTBWnd, TB_GETBUTTONSIZE, 0, 0);
	PGPOeTrace("\tButtonSize= width:%#x height:%#x \n",
		LOWORD(lResult), HIWORD(lResult));

	lResult = SendMessage(hTBWnd, TB_GETPADDING, 0, 0);
	PGPOeTrace("\tPadding= horizontal:%#x vertical:%#x \n",
		LOWORD(lResult), HIWORD(lResult));

	
	//for each button
	for(iIndex=0; iIndex < iNoOfButtons; iIndex++)
	{
		PGPOeTrace("\t\tBUTTON # %d\n", iIndex);

		ZeroMemory(&tbbBtn, sizeof(tbbBtn));

		lResult = SendMessage(hTBWnd, TB_GETBUTTON, iIndex, (LPARAM)&tbbBtn);
		PGPOeTrace("\t\t\t idCommand=%d\n", tbbBtn.idCommand);
		PGPOeTrace("\t\t\t iBitmap=%d\n", tbbBtn.iBitmap);
		PGPOeTrace("\t\t\t fsStyle=%#x\n", tbbBtn.fsStyle);
		PGPOeTrace("\t\t\t fsState=%#x\n", tbbBtn.fsState);
		PGPOeTrace("\t\t\t dwData=%#x\n", tbbBtn.dwData);

		tbbBtnInfo.cbSize=sizeof(TBBUTTONINFO);
		tbbBtnInfo.dwMask=TBIF_IMAGE|TBIF_SIZE|TBIF_TEXT;
		tbbBtnInfo.pszText=szText;
		tbbBtnInfo.cchText=sizeof(szText);

		lResult = SendMessage(hTBWnd, TB_GETBUTTON, tbbBtn.idCommand, (LPARAM)&tbbBtnInfo);
		PGPOeTrace("\t\t\t pszText=%s\n", szText);
		PGPOeTrace("\t\t\t width in pixels=%d\n", tbbBtnInfo.cx);
	}

}

void DumpImageListInfo(HIMAGELIST hilImgList)
{
	int iWidth=0, iHeight=0;
	int iIndex=0;
	int iTemp=0;
	COLORREF clrColor;
	IMAGEINFO imgInfo={0};
	BOOL bRet=ImageList_GetIconSize(hilImgList, &iWidth, &iHeight);

	PGPOeTrace("IMAGELIST INFO\n");
	PGPOeTrace("\tIconSize= Width: %d Height: %d\n", iWidth, iHeight);

	clrColor=ImageList_GetBkColor(hilImgList);
	PGPOeTrace("\tBkColor = %#x Our color = %#x\n", clrColor, TRANSPARENT_COLOR);

	iTemp=ImageList_GetImageCount(hilImgList);
	PGPOeTrace("\tImageCount= %d\n", iTemp);

	//for each button
	for(iIndex=0; iIndex < iTemp; iIndex++)
	{
		PGPOeTrace("\t\tImage Info # %d\n", iIndex);

		ZeroMemory(&imgInfo, sizeof(imgInfo));

		bRet=ImageList_GetImageInfo(hilImgList, iIndex, &imgInfo);
		if(TRUE == bRet)
		{
			PGPOeTrace("\t\t\t hbmImage=%#x\n", imgInfo.hbmImage);
			PGPOeTrace("\t\t\t hbmMask=%#x\n", imgInfo.hbmMask);
			PGPOeTrace("\t\t\t Unused1=%#x\n", imgInfo.Unused1);
			PGPOeTrace("\t\t\t Unused2=%#x\n", imgInfo.Unused2);
			PGPOeTrace("\t\t\t rcImage left=%d, right=%d, top=%d, bottom=%d\n", 
				imgInfo.rcImage.left, imgInfo.rcImage.right, imgInfo.rcImage.top, imgInfo.rcImage.bottom);
		}
	}

}


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
