/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: CPGPMsgBackup.cpp,v 1.14 2002/11/14 22:57:23 sdas Exp $
____________________________________________________________________________*/
#include "CPGPexch.h"
#include <mapiutil.h>

#include "pgpDebug.h"
#include "cpgpmsgbackuphash.h"

extern "C" void ExchPluginTrace(LPCTSTR lpszFormat, ...);
extern CPGPMsgBackupHash g_MsgBackupHash;

#ifdef _DEBUG
static void TraceNotification(LPNOTIFICATION lpNotifications);
#endif

CPGPMsgBackup::CPGPMsgBackup(void *pCPGPexch, PGPMemoryMgrRef memMgr,
							 IMsgStore *pMsgStore, IMessage *pmsg) : _cRef(1)
{
#ifdef _DEBUG
	ExchPluginTrace("CPGPMsgBackup(%#x) - constructed for CPGPexch(%#x)\n", this, pCPGPexch); 
#endif

	m_ulBkRef = 1;

	m_pEntryID = NULL;
	m_ulEntryIDSize = 0;

	pgpAssert(NULL != pMsgStore);
	if (pMsgStore != NULL)
		pMsgStore->AddRef();
	m_pMsgStore = pMsgStore;

	pgpAssert(NULL != pmsg);
	if (NULL != pmsg)
		pmsg->AddRef();
	m_pMsg = pmsg;

	pgpAssert(NULL != pCPGPexch);
	m_pCPGPexch = pCPGPexch;

	m_ulAdvise = 0;

	pgpAssert(NULL != m_memMgr);
	m_memMgr = memMgr;

	m_szMsg = NULL;
	m_dwMsgSize = 0;
	m_ulFormatIndex = 0;

	m_bHTML = FALSE;
	m_bRTF = FALSE;
	
	m_pFormat = NULL;
	m_pHead = NULL;
	m_pCurrent = NULL;
}


CPGPMsgBackup::~CPGPMsgBackup()
{
#ifdef _DEBUG
	ExchPluginTrace("CPGPMsgBackup(%#x) - being destroyed\n", this);
#endif

	//restore the backup message
	this->RestoreBackup();

	if (m_szMsg != NULL)
	{
		PGPFreeData(m_szMsg);
		m_szMsg = NULL;
	}

	FreeAttachList(m_pFormat);
	FreeAttachList(m_pHead);

	if ((m_pMsgStore != NULL) && (0 < m_ulAdvise))
	{
		m_pMsgStore->Unadvise(m_ulAdvise);
		m_ulAdvise = 0;
	}

	if (m_pMsg != NULL)
	{
		m_pMsg->Release();
		m_pMsg = NULL;
	}
	
	if (m_pMsgStore != NULL)
	{
		m_pMsgStore->Release();
		m_pMsgStore = NULL;
	}

	if(NULL != m_pEntryID)
	{
		pgpAssert(0 < m_ulEntryIDSize);
		free(m_pEntryID);
		m_pEntryID = NULL;
		m_ulEntryIDSize = 0;
	}

#ifdef _DEBUG
	ExchPluginTrace("CPGPMsgBackup(%#x) - destruction complete\n", this);
#endif

}


STDMETHODIMP CPGPMsgBackup::QueryInterface(REFIID riid, void** ppvObj)          
{
    *ppvObj = NULL;

    HRESULT hr = S_OK;
	IUnknown* punk = NULL;

    if (( IID_IUnknown == riid) || ( IID_IMAPIAdviseSink == riid) )
    {
        punk = this;
    }
    else
        hr = E_NOINTERFACE;

	if (NULL != punk)
	{
		*ppvObj = punk;
		AddRef();
	}

    return hr;
}

ULONG CPGPMsgBackup::BackupAddRef()
{
	pgpAssert(0 != m_ulBkRef);//should not already be gone

#ifdef _DEBUG
	ExchPluginTrace("CPGPMsgBackup(%#x) - BackupAddRef(incrementing to %d)\n)", 
						this, m_ulBkRef+1);
#endif

	++m_ulBkRef; 
	return m_ulBkRef;
}

ULONG CPGPMsgBackup::BackupRelease()
{
	pgpAssert(0 != m_ulBkRef);//should not already be gone

#ifdef _DEBUG
	ExchPluginTrace("CPGPMsgBackup(%#x) - BackupRelease(decrementing to %d)\n", 
						this, m_ulBkRef-1);
#endif

	ULONG ulCount = --m_ulBkRef;
    if (!ulCount) 
		delete this; //self-destruct
    return ulCount;
}


LPVOID CPGPMsgBackup::GetCreator()
{
	return m_pCPGPexch;
}

//method to set the entryid on the message around which this object operates
STDMETHODIMP CPGPMsgBackup::SetMessageEntryID(LPVOID lpEntryID, ULONG ulSize)
{

#ifdef _DEBUG
	ExchPluginTrace("CPGPMsgBackup(%#x) - SetMessageEntryID(entryid->%#x of len %d)\n", 
						this, lpEntryID, ulSize);
#endif

	BOOL bRet = FALSE;

	//check parameters
	pgpAssert(NULL != lpEntryID);
	pgpAssert(ulSize > 0);
	if((NULL == lpEntryID) || (0 == ulSize))//if null pointer or zero length
		return E_INVALIDARG;//we are done/invalid parameter(s)

	pgpAssert(!IsBadReadPtr(lpEntryID, ulSize));
	if(TRUE == IsBadReadPtr(lpEntryID, ulSize))
		return E_POINTER;//bad parameter/pointer

	//check object state
	pgpAssert(NULL == m_pEntryID);//we should be called to set the message entry ID only once
	if(NULL != m_pEntryID)//if we are being called for a second time
		return E_FAIL;//ignore such attempts
	
	//allocate for the size required
	m_pEntryID = (LPENTRYID)calloc(1, ulSize);
	if(NULL == m_pEntryID)//failed to allocate
		return E_OUTOFMEMORY;

	//copy the entry id to object member 
	memcpy(m_pEntryID, lpEntryID, ulSize);
	m_ulEntryIDSize = ulSize;

	//insert this into backup hash
	bRet = InsertThisToBackupHash();
	pgpAssert(TRUE == bRet);//failed to insert into hash !!

	return S_OK;
}

STDMETHODIMP CPGPMsgBackup::WatchMsgForDelOrMove(BOOL bStart)
{
	HRESULT hrRet = S_OK;

	pgpAssert(NULL != m_pMsgStore);//should know the store
	pgpAssert(NULL != m_pMsg);//should know the message
	pgpAssert(NULL != m_pEntryID);//should also know the entry id
	pgpAssert(bStart == (0 == m_ulAdvise));//should not be watching already if we are 
										   //being told now to (or vice versa)
	
	if((NULL == m_pMsgStore) || (NULL == m_pMsg) || 
	   (NULL == m_pEntryID) || (bStart != (0 == m_ulAdvise)))
		return E_FAIL;

	//if we are being told to start watching for this message
	if(TRUE == bStart)
	{
		//watch the message for deletion or move
		hrRet = m_pMsgStore->Advise(m_ulEntryIDSize, (LPENTRYID)m_pEntryID, 
					fnevObjectDeleted|fnevObjectMoved, 
					(IMAPIAdviseSink *) this, &m_ulAdvise);
		pgpAssert(SUCCEEDED(hrRet) && (m_ulAdvise > 0));

#ifdef _DEBUG			
		if(SUCCEEDED(hrRet) && (m_ulAdvise > 0))
			ExchPluginTrace("CPGPMsgBackup(%#x) - watching message {..%#x..%#x..%#x..} id len = %d sink = %d\n", 
				this, ((char *)m_pEntryID)[m_ulEntryIDSize/4], ((char *)m_pEntryID)[m_ulEntryIDSize/2], 
				((char *)m_pEntryID)[3*m_ulEntryIDSize/4], m_ulEntryIDSize, m_ulAdvise);
#endif
	}
	else
	{
		//we are being told to stop watching for this message
		pgpAssert(0 < m_ulAdvise);

		hrRet = m_pMsgStore->Unadvise(m_ulAdvise);
		pgpAssert(SUCCEEDED(hrRet));//cant do much if this fails
		m_ulAdvise = 0;//disremember the previous sink
	}

	return hrRet;
}

STDMETHODIMP_(ULONG)
CPGPMsgBackup::OnNotify(ULONG cNotif, LPNOTIFICATION lpNotifications)
{
	if (cNotif > 0)
	{
		ULONG ulType;
		HRESULT hrRet = S_OK;

#ifdef _DEBUG
		ExchPluginTrace("CPGPMsgBackup(%#x) - OnNotify\n", this);
		TraceNotification(lpNotifications);
#endif
		
		if(m_pMsg != NULL)
		{
			//delete the decryption tag we put in the message since we dont know
			//if we will get back at any message after releasing this
			/*SizedSPropTagArray(1, tagArr) = {1, {PR_PGPEXCH_DECRYPTED}};
			hrRet = m_pMsg->DeleteProps((SPropTagArray *)&tagArr, NULL);
			pgpAssert(SUCCEEDED(hrRet));*/

			m_pMsg->Release();
			m_pMsg = NULL;
		}

		pgpAssert(NULL == m_pMsg);
		pgpAssert(NULL != m_pMsgStore);
		hrRet = m_pMsgStore->OpenEntry(lpNotifications->info.obj.cbEntryID,
			lpNotifications->info.obj.lpEntryID, NULL, MAPI_MODIFY,
			&ulType, (IUnknown **) &m_pMsg);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != m_pMsg));
#ifdef _DEBUG			
		if(SUCCEEDED(hrRet) && (NULL != m_pMsg))
			ExchPluginTrace("CPGPMsgBackup(%#x) - now watching message %#x%#x...%#x now\n", 
				this, lpNotifications->info.obj.lpEntryID[0], 
				lpNotifications->info.obj.lpEntryID[1],
				lpNotifications->info.obj.lpEntryID[lpNotifications->info.obj.cbEntryID-1]);
#endif

	}

	return S_OK;
}


HRESULT CPGPMsgBackup::AddMessage(char *szMsg, DWORD dwSize)
{
	//check parameters
	pgpAssert(NULL != szMsg);
	pgpAssert(dwSize > 0);
	if((NULL == szMsg) || (0 == dwSize))
		return E_INVALIDARG;

	//check object state
	pgpAssert(NULL == m_szMsg);
	if(NULL != m_szMsg)
		return E_FAIL;

	//allocate as necessary
	m_szMsg = (char *) PGPNewData(m_memMgr, dwSize+1, kPGPMemoryMgrFlags_Clear);
	pgpAssert(NULL != m_szMsg);
	if(NULL == m_szMsg)
		return E_OUTOFMEMORY;

	//copy to member
	memcpy(m_szMsg, szMsg, dwSize);
	m_szMsg[dwSize] = '\0';
	m_dwMsgSize = dwSize;

	return S_OK;
}


void CPGPMsgBackup::SetFormat(BOOL bHTML, BOOL bRTF)
{
#ifdef _DEBUG
	ExchPluginTrace("CPGPMsgBackup(%#x) - SetFormat (\"%s\")\n", 
		this, bHTML?"HTML":(bRTF?"RTF":"PLAIN"));
#endif

	pgpAssert((TRUE == bHTML)?(FALSE == bRTF):TRUE);//both cannot be set
	m_bHTML = bHTML;
	m_bRTF = bRTF;
	return;
}


void CPGPMsgBackup::AddAttachment(ULONG ulIndex, BOOL bIsFormat, char *szName, 
								  char *szLongName, char *szDisplayName, 
								  char *szData, DWORD dwSize)
{
	PGPMsgBackupList *pNew = NULL;

#ifdef _DEBUG
	ExchPluginTrace("CPGPMsgBackup(%#x) - AddAttachment (\"%s\" of size %d) at index=%d format=%d\n", 
				this, szLongName, dwSize, ulIndex, bIsFormat);
#endif
	
	if (bIsFormat)
	{
		AddToAttachList(ulIndex, &m_pFormat, NULL);
		pNew = m_pFormat;
		m_ulFormatIndex = ulIndex;
	}
	else
	{
		AddToAttachList(ulIndex, &m_pHead, &m_pCurrent);
		pNew = m_pCurrent;
	}

	if (szName != NULL)
	{
		pNew->szName = (char *) PGPNewData(m_memMgr, strlen(szName)+1, kPGPMemoryMgrFlags_Clear);
		strcpy(pNew->szName, szName);
	}
	
	if (szLongName != NULL)
	{
		pNew->szLongName = (char *) PGPNewData(m_memMgr, strlen(szLongName)+1, kPGPMemoryMgrFlags_Clear);
		strcpy(pNew->szLongName, szLongName);
	}
	
	pNew->szData = (char *) PGPNewData(m_memMgr, dwSize+1, kPGPMemoryMgrFlags_Clear);
	memcpy(pNew->szData, szData, dwSize);
	pNew->szData[dwSize] = 0;
	pNew->dwSize = dwSize;

	return;
}


void CPGPMsgBackup::AddOLE(ULONG ulIndex, WCHAR *pwcsName, char *szData, 
						   DWORD dwSize)
{
	PGPMsgBackupList *pNew = NULL;
	PGPOLEBackupList *pOLE = NULL;

	pNew = FindAttachment(ulIndex);
	if (pNew == NULL)
	{
		AddToAttachList(ulIndex, &m_pHead, &m_pCurrent);
		pNew = m_pCurrent;
	}

	pOLE = (PGPOLEBackupList *) PGPNewData(m_memMgr, sizeof(PGPOLEBackupList),
									kPGPMemoryMgrFlags_Clear);

	if (pwcsName != NULL)
	{
		pOLE->pwcsName = (WCHAR *) PGPNewData(m_memMgr, (wcslen(pwcsName)+1) * sizeof(WCHAR), 
									kPGPMemoryMgrFlags_Clear);
		wcscpy(pOLE->pwcsName, pwcsName);
	}

	pOLE->szData = (char *) PGPNewData(m_memMgr, dwSize+1, kPGPMemoryMgrFlags_Clear);
	memcpy(pOLE->szData, szData, dwSize);
	pOLE->szData[dwSize] = 0;
	pOLE->dwSize = dwSize;

	if (pNew->poleHead == NULL)
		pNew->poleHead = pOLE;

	if (pNew->poleCurrent != NULL)
		pNew->poleCurrent->pNext = pOLE;

	pNew->poleCurrent = pOLE;
	return;
}

HRESULT CPGPMsgBackup::IsBackupOfMsg(IMessage *pMsg, BOOL &bResult)
{
	SPropValue *pval = NULL;
	HRESULT hrRet = S_OK;
	ULONG ulResult = 0;

	pgpAssert(NULL != pMsg);
	if(NULL == pMsg)
		return E_INVALIDARG;

	//get the entry id of the message
	hrRet = HrGetOneProp(pMsg, PR_ENTRYID, &pval);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pval) && 
			  (NULL != pval->Value.bin.lpb) && (0 < pval->Value.bin.cb));
	if(FAILED(hrRet) || (NULL == pval) || (NULL == pval->Value.bin.lpb) || 
			(0 == pval->Value.bin.cb))
		goto cleanup;

	//compare the entry id to the one we represent currently
	pgpAssert(NULL != m_pMsgStore);
	hrRet = m_pMsgStore->CompareEntryIDs(m_ulEntryIDSize, m_pEntryID, pval->Value.bin.cb,
				(LPENTRYID)pval->Value.bin.lpb, 0, &ulResult);
	pgpAssert(SUCCEEDED(hrRet));
	if(FAILED(hrRet))
		goto cleanup;

	//copy the result to the out parameter
	bResult = ulResult;

	cleanup:
	if(NULL != pval)
	{
		HRESULT hrResult = MAPIFreeBuffer(pval);
		pgpAssert(S_OK == hrResult);
		pval = NULL;
	}

	return hrRet;
}

HRESULT CPGPMsgBackup::IsAlreadyBackedUp(LPMDB lpMsgStore, LPMESSAGE lpMsg, 
			CPGPMsgBackup *&ppmbMsgBackup)
{
	SPropValue *pval = NULL;
	PBACKUPMSGINFO pbmiMsgInfo = NULL;
	HRESULT hrRet = S_OK;

	//check parameters
	pgpAssert(NULL != lpMsgStore);
	pgpAssert(NULL != lpMsg);
	if((NULL == lpMsgStore) || (NULL == lpMsg))
	{
		hrRet = E_INVALIDARG;
		goto cleanup;
	}

	//get the entry id of the message
	hrRet = HrGetOneProp(lpMsg, PR_ENTRYID, &pval);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pval) && 
			  (NULL != pval->Value.bin.lpb) && (0 < pval->Value.bin.cb));
	if(FAILED(hrRet) || (NULL == pval) || (NULL == pval->Value.bin.lpb) || 
			(0 == pval->Value.bin.cb))
	{
		hrRet = E_FAIL;
		goto cleanup;
	}
	
	//find this message in the backup hash (if it is in there somewhere)
	pbmiMsgInfo = g_MsgBackupHash.Get(lpMsgStore, (LPENTRYID)pval->Value.bin.lpb, 
			pval->Value.bin.cb);
	if(NULL != pbmiMsgInfo)//if we got the object pointer
	{
		//copy it to out param
		g_MsgBackupHash.Lock();
		ppmbMsgBackup = pbmiMsgInfo->pBackupObj;
		g_MsgBackupHash.UnLock();
	}

	hrRet = S_OK; //all success
	
	cleanup:

	if(NULL != pval)
	{
		HRESULT hrResult = MAPIFreeBuffer(pval);
		pgpAssert(S_OK == hrResult);
		pval = NULL;
	}

	return hrRet;
}

HRESULT CPGPMsgBackup::GetFormatInfoIfAny(void *&pFormatInfo, 
			DWORD &dwFormatInfoSize, DWORD &dwFormatInfoType)
{
	HRESULT hrRet = S_OK;
	//check state
	pgpAssert(NULL != m_pMsg);

	//reset contents of output params
	pFormatInfo = NULL;
	dwFormatInfoSize = 0;
	dwFormatInfoType = 0;

	//if there is no formatting information
	if((TRUE != m_bHTML) && (TRUE != m_bRTF))
		return S_OK;//we are done - this is a plain text message

	pgpAssert((NULL != m_pFormat) && (0 < m_pFormat->dwSize) && (NULL != m_pFormat->szData));
	if((NULL == m_pFormat) || (0 == m_pFormat->dwSize) || (NULL == m_pFormat->szData))
		return E_FAIL;

	//allocate space necessary
	pgpAssert(NULL == pFormatInfo);
	pFormatInfo = calloc(1, m_pFormat->dwSize);
	pgpAssert(NULL != pFormatInfo);
	if(NULL == pFormatInfo)
	{
		hrRet = E_OUTOFMEMORY;
		goto cleanup;
	}

	//copy the buffer
	memcpy(pFormatInfo, m_pFormat->szData, m_pFormat->dwSize);
	dwFormatInfoSize = m_pFormat->dwSize;
	dwFormatInfoType = m_bHTML?SFPA_FOUND_HTML_DATA:SFPA_FOUND_RTF_DATA;
	hrRet = S_OK;

	cleanup:

	if(FAILED(hrRet))//in case we failed
	{
		//free the allocated buffer if required
		if(NULL != pFormatInfo)
		{
			free(pFormatInfo);
			pFormatInfo = NULL;
		}
	}

	return hrRet;
}

void CPGPMsgBackup::RestoreBackup()
{
	HRESULT hrRet = S_OK;
	IStream *pstrmBody = NULL;
	IStream *pstrmRTF = NULL;
	LARGE_INTEGER li = {0};
	ULARGE_INTEGER uli = {1,0};
	BOOL fPartied;
	LPMAPITABLE ptableAttach = NULL;
	SizedSPropTagArray(1, tagaTable) = {1, {PR_ATTACH_NUM}};
	SRowSet *prAttach = NULL;

#ifdef _DEBUG
	ExchPluginTrace("CPGPMsgBackup(%#x) - RestoreBackup, pMsg = %#x, m_szMsg = %#x, " 
		"mail type = %s \n", this, m_pMsg, m_szMsg, m_bRTF?"RTF":(m_bHTML?"HTML":"PLAIN"));
#endif

	//remove ourselves from the hash. doing this first alleviates the problem
	//of this object being picked up from hash while it is being destroyed.
	g_MsgBackupHash.Remove(m_pMsgStore, m_pEntryID, m_ulEntryIDSize);

	hrRet = HrDispatchNotifications(0);
	pgpAssert(S_OK == hrRet);

	Sleep(250); 

	pgpAssert(NULL != m_pMsg);
	if(NULL == m_pMsg)//if no message to operate on now
		return;//bail out

	/*if(NULL != m_pMsg)
	{
		//delete the decryption tag we put in the message since we are
		//switching back to the original message now
		SPropTagArray tagArr = {0};
		tagArr.cValues = 1;
		tagArr.aulPropTag[0] = PR_PGPEXCH_DECRYPTED;
		hrRet = m_pMsg->DeleteProps(&tagArr, NULL);
		pgpAssert(SUCCEEDED(hrRet));
	}*/

	if (m_szMsg != NULL)
	{
		SPropTagArray taga = {0};

#ifdef _DEBUG
		ExchPluginTrace("\t CPGPMsgBackup(%#x) - Saved Mail => {%#x...%#x} \n", this, m_szMsg[0], 
			(0 < strlen(m_szMsg))?m_szMsg[strlen(m_szMsg)-1]:'\0');
#endif
		
		taga.cValues = 1;
		taga.aulPropTag[0] = PR_BODY_HTML;
		hrRet = m_pMsg->DeleteProps(&taga, NULL);
		pgpAssert(SUCCEEDED(hrRet));//failed to find the html body of the message
		
		if (m_bRTF)
		{
			pgpAssert(NULL == pstrmBody);
			hrRet = m_pMsg->OpenProperty(PR_RTF_COMPRESSED, &IID_IStream, 
				STGM_READWRITE, MAPI_MODIFY, (IUnknown**)&pstrmBody);
			pgpAssert((S_OK == hrRet) && (NULL != pstrmBody));
			if((S_OK == hrRet) && (NULL != pstrmBody))
			{
				hrRet = WrapCompressedRTFStream(pstrmBody, MAPI_MODIFY, &pstrmRTF);
				pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmRTF));
				
				if(SUCCEEDED(hrRet) && (NULL != pstrmRTF))
				{
					hrRet = pstrmRTF->Write("\0", 1, NULL);
					pgpAssert(S_OK == hrRet);

					hrRet = pstrmRTF->SetSize(uli);
					//pgpAssert(S_OK == hrRet);//TODO: investigate E_NOTIMPL

					hrRet = pstrmRTF->Commit(STGC_DEFAULT);
					pgpAssert(S_OK == hrRet);

					pstrmRTF->Release();
					pstrmRTF = NULL;
				}

				pstrmBody->Release();
				pstrmBody = NULL;

				hrRet = RTFSync(m_pMsg, RTF_SYNC_RTF_CHANGED, &fPartied);
				pgpAssert(S_OK == hrRet);
			}
		}
		
		pgpAssert(NULL == pstrmBody);
		pgpAssert(NULL != m_pMsg);
		if(NULL != m_pMsg)
		{
			hrRet = m_pMsg->OpenProperty(PR_BODY, &IID_IStream, STGM_READWRITE, 
					MAPI_MODIFY, (IUnknown**)&pstrmBody);
			pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));
			if (SUCCEEDED(hrRet) && (NULL != pstrmBody))
			{
				hrRet = pstrmBody->Seek(li, STREAM_SEEK_SET, NULL);
				pgpAssert(S_OK == hrRet);

				hrRet = pstrmBody->Write("\0", 1, NULL);
				pgpAssert(S_OK == hrRet);

				hrRet = pstrmBody->SetSize(uli);
				pgpAssert(S_OK == hrRet);

				hrRet = pstrmBody->Commit(STGC_DEFAULT);
				pgpAssert(S_OK == hrRet);

				pstrmBody->Release(); 
				pstrmBody = NULL;

				hrRet = RTFSync(m_pMsg, RTF_SYNC_BODY_CHANGED, &fPartied);
				pgpAssert(S_OK == hrRet);
			}
		}
		
		if(NULL != m_pMsg)
		{
			pgpAssert(NULL == pstrmBody);
			hrRet = m_pMsg->OpenProperty(PR_BODY, &IID_IStream, STGM_READWRITE, 
				MAPI_CREATE | MAPI_MODIFY, (IUnknown**)&pstrmBody);//create if does not exist
			pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));
			
			if(SUCCEEDED(hrRet) && (NULL != pstrmBody))
			{
				uli.LowPart = m_dwMsgSize;

				hrRet = pstrmBody->Seek(li, STREAM_SEEK_SET, NULL);
				pgpAssert(S_OK == hrRet);

				hrRet = pstrmBody->Write(m_szMsg, m_dwMsgSize, NULL);
				pgpAssert(S_OK == hrRet);

#ifdef _DEBUG
				ExchPluginTrace("\t CPGPMsgBackup(%#x) - Saved body (%d bytes) written to PR_BODY \n", 
					this, m_dwMsgSize);
#endif

				hrRet = pstrmBody->SetSize(uli);
				pgpAssert(S_OK == hrRet);

				hrRet = pstrmBody->Commit(STGC_DEFAULT);
				pgpAssert(S_OK == hrRet);

				pstrmBody->Release(); 
				pstrmBody = NULL;

				hrRet = RTFSync(m_pMsg, RTF_SYNC_BODY_CHANGED, &fPartied);
				pgpAssert(S_OK == hrRet);
			}
		}
	}

	if (m_pFormat)
	{
		ULONG ulAttachNum = 0;
		ULONG ulNumOfValues = 2;//to keep track of how many values we can set
		LPATTACH pAttach = NULL;
		IStream *pstrmAttach = NULL;
		SPropValue pVal[5] = {0};
		LPVOID lpBuffer = NULL;

#ifdef _DEBUG
		ExchPluginTrace("\t CPGPMsgBackup(%#x) - Trying to restore formatted buffer of size %d {%#x %#x ... %#x}\n", 
											this, m_pFormat->dwSize, m_pFormat->szData?m_pFormat->szData[0]:'\0',
											(m_pFormat->dwSize > 2)?m_pFormat->szData[1]:'\0', 
											(m_pFormat->dwSize > 3)?m_pFormat->szData[m_pFormat->dwSize-1]:'\0');
#endif

		uli.LowPart = m_pFormat->dwSize;

		pVal[0].ulPropTag = PR_ATTACH_METHOD;
		pVal[0].Value.l = ATTACH_BY_VALUE;

		pVal[1].ulPropTag = PR_RENDERING_POSITION;
		pVal[1].Value.l = -1;
		
		pVal[2].ulPropTag = PR_ATTACH_FILENAME;
		hrRet = MAPIAllocateBuffer(MAX_PATH, (void **) &(pVal[2].Value.lpszA));
		pgpAssert(SUCCEEDED(hrRet) && (NULL != pVal[2].Value.lpszA));
		if(SUCCEEDED(hrRet) && (NULL != pVal[2].Value.lpszA))
		{
			pgpAssert(NULL == lpBuffer);
			lpBuffer = pVal[2].Value.lpszA;//remember the first allocation
			strcpy(pVal[2].Value.lpszA, m_pFormat->szName);
			ulNumOfValues++;
		}
		
		if(3 == ulNumOfValues)
		{
			pVal[3].ulPropTag = PR_ATTACH_LONG_FILENAME;

			pgpAssert(NULL != lpBuffer);
			hrRet = MAPIAllocateMore(MAX_PATH, lpBuffer, (void **) &(pVal[3].Value.lpszA));
			pgpAssert(SUCCEEDED(hrRet) && (NULL != pVal[3].Value.lpszA));
			if(SUCCEEDED(hrRet) && (NULL != pVal[3].Value.lpszA))
			{
				strcpy(pVal[3].Value.lpszA, m_pFormat->szLongName);
				ulNumOfValues++;
			}
		}

		if(4 == ulNumOfValues)
		{
			pVal[4].ulPropTag = PR_DISPLAY_NAME;

			pgpAssert(NULL != lpBuffer);
			hrRet = MAPIAllocateMore(MAX_PATH, lpBuffer, (void **) &(pVal[4].Value.lpszA));
			pgpAssert(SUCCEEDED(hrRet) && (NULL != pVal[4].Value.lpszA));
			if(SUCCEEDED(hrRet) && (NULL != pVal[4].Value.lpszA))
			{
				strcpy(pVal[4].Value.lpszA, m_pFormat->szLongName);
				ulNumOfValues++;
			}
		}

		hrRet = m_pMsg->CreateAttach(NULL, 0, &ulAttachNum, &pAttach);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != pAttach));
		if(SUCCEEDED(hrRet) && (NULL != pAttach))
		{
			hrRet = pAttach->SetProps(ulNumOfValues, pVal, NULL);
			pgpAssert(S_OK == hrRet);

			pgpAssert(NULL == pstrmAttach);
			hrRet = pAttach->OpenProperty(PR_ATTACH_DATA_BIN, &IID_IStream, 0, 
						MAPI_CREATE | MAPI_MODIFY, (LPUNKNOWN *) &pstrmAttach);
			pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmAttach));
			if(SUCCEEDED(hrRet) && (NULL != pstrmAttach))
			{
				hrRet = pstrmAttach->Seek(li, STREAM_SEEK_SET, NULL);
				pgpAssert(SUCCEEDED(hrRet));

				hrRet = pstrmAttach->Write(m_pFormat->szData, m_pFormat->dwSize, NULL);
				pgpAssert(SUCCEEDED(hrRet));

				hrRet = pstrmAttach->SetSize(uli);
				pgpAssert(SUCCEEDED(hrRet));

				hrRet = pstrmAttach->Commit(STGC_DEFAULT);
				pgpAssert(SUCCEEDED(hrRet));

				pstrmAttach->Release();
				pstrmAttach = NULL;
			}

			hrRet = pAttach->SaveChanges(0);
			pgpAssert(SUCCEEDED(hrRet));
		}

		hrRet = m_pMsg->SaveChanges(KEEP_OPEN_READWRITE);	
		pgpAssert(SUCCEEDED(hrRet));

		if(NULL != lpBuffer)
		{
			hrRet = MAPIFreeBuffer(lpBuffer);
			pgpAssert(S_OK == hrRet);
			lpBuffer = NULL;
		}
	}
	
	hrRet = m_pMsg->GetAttachmentTable(0, &ptableAttach);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != ptableAttach));
	if(SUCCEEDED(hrRet) && (NULL != ptableAttach))
	{
		hrRet = HrQueryAllRows(ptableAttach, (SPropTagArray *)&tagaTable, 
					NULL, NULL, 0, 	&prAttach);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != prAttach));
		if(SUCCEEDED(hrRet) && (NULL != prAttach))
		{
			if (m_pHead && prAttach->cRows)
			{
#ifdef _DEBUG
				ExchPluginTrace("\t CPGPMsgBackup(%#x) - Restoring %d attachments of message \n", 
									this, prAttach->cRows);
#endif

				((CPGPexch *) m_pCPGPexch)->DecryptVerifyRestoreAttachment(NULL, 
												m_pMsg, prAttach, TRUE);
			}
						
			FreeProws(prAttach);
			prAttach = NULL;

			ptableAttach->Release();
			ptableAttach = NULL;

			hrRet = m_pMsg->SaveChanges(0);
			pgpAssert(SUCCEEDED(hrRet));
		}
	}
	
	
	return;			
}


void CPGPMsgBackup::AddToAttachList(ULONG ulIndex, PGPMsgBackupList **ppHead,
									PGPMsgBackupList **ppCurrent)
{
	PGPMsgBackupList *pNew;

	pNew = (PGPMsgBackupList *) PGPNewData(m_memMgr, sizeof(PGPMsgBackupList),
									kPGPMemoryMgrFlags_Clear);

	pNew->ulIndex = ulIndex;

	if (*ppHead == NULL)
		*ppHead = pNew;

	if (ppCurrent != NULL)
	{
		if (*ppCurrent != NULL)
			(*ppCurrent)->pNext = pNew;

		*ppCurrent = pNew;
	}

	return;
}


PGPMsgBackupList *CPGPMsgBackup::FindAttachment(ULONG ulIndex)
{
	PGPMsgBackupList *pFind = NULL;

#ifdef _DEBUG
	ExchPluginTrace("CPGPMsgBackup(%#x) - FindAttachment looking for attachment index=%d ", 
		this, ulIndex);
#endif

	pFind = m_pHead;
	while (pFind != NULL)
	{
		if (pFind->ulIndex == ulIndex)
			break;

		pFind = pFind->pNext;
	}

#ifdef _DEBUG	
	ExchPluginTrace("returning %#x\n", pFind); 
#endif

	return pFind;
}


PGPOLEBackupList *CPGPMsgBackup::FindOLEAttachment(ULONG ulIndex, 
												   WCHAR *pwcsName)
{
	PGPMsgBackupList *pFind;
	PGPOLEBackupList *pFindOLE = NULL;

#ifdef _DEBUG
	ExchPluginTrace("CPGPMsgBackup(%#x) - FindOLEAttachment looking for OLE attachment index=%d name = %S ", 
		this, ulIndex, pwcsName);
#endif

	pFind = FindAttachment(ulIndex);
	if (pFind != NULL)
	{
		pFindOLE = pFind->poleHead;
		while (pFindOLE != NULL)
		{
			if (!wcscmp(pwcsName, pFindOLE->pwcsName))
				break;

			pFindOLE = pFindOLE->pNext;
		}
	}

	return pFindOLE;
}


void CPGPMsgBackup::FreeAttachList(PGPMsgBackupList *pHead)
{
	PGPMsgBackupList *pTemp;
	PGPOLEBackupList *poleTemp;

#ifdef _DEBUG
	ExchPluginTrace("CPGPMsgBackup(%#x) - FreeAttachList(%#x)\n", this, pHead); 
#endif

	while (pHead != NULL)
	{
		if (pHead->szName != NULL)
			PGPFreeData(pHead->szName);
		if (pHead->szLongName != NULL)
			PGPFreeData(pHead->szLongName);
		if (pHead->szDisplayName != NULL)
			PGPFreeData(pHead->szDisplayName);
		if (pHead->szData != NULL)
			PGPFreeData(pHead->szData);

		while (pHead->poleHead != NULL)
		{
			if (pHead->poleHead->pwcsName != NULL)
				PGPFreeData(pHead->poleHead->pwcsName);
			if (pHead->poleHead->szData != NULL)
				PGPFreeData(pHead->poleHead->szData);

			poleTemp = pHead->poleHead;
			pHead->poleHead = pHead->poleHead->pNext;
			PGPFreeData(poleTemp);
		}

		pTemp = pHead;
		pHead = pHead->pNext;
		PGPFreeData(pTemp);
	}

	return;
}

HRESULT CPGPMsgBackup::GetMsgBody(void *&pBuf, DWORD &dwSize)
{
	HRESULT hrRet = S_OK;
	pgpAssert(NULL != m_pEntryID);

	//check state
	pgpAssert(NULL != m_szMsg);
	pgpAssert(0 < m_dwMsgSize);
	if((NULL == m_szMsg) || (0 == m_dwMsgSize))
	{
		hrRet = E_FAIL;
		goto cleanup;
	}

	//reset out params
	pBuf = NULL;

	//allocate space necessary
	pgpAssert(NULL == pBuf);
	pBuf = calloc(1, m_dwMsgSize);
	pgpAssert(NULL != pBuf);
	if(NULL == pBuf)
	{
		hrRet = E_OUTOFMEMORY;
		goto cleanup;
	}
	
	dwSize = m_dwMsgSize;
	hrRet = S_OK;

	cleanup:

	if(FAILED(hrRet))
	{
		if(NULL != pBuf)
		{
			free(pBuf);
			pBuf = NULL;
		}
	}

	return hrRet;
}

BOOL CPGPMsgBackup::InsertThisToBackupHash()
{
	BOOL bRet = FALSE;
	PBACKUPMSGINFO pbmiMsgInfo = NULL;

	//should have the message entry id by now
	pgpAssert(NULL != m_pEntryID);
	pgpAssert(0 < m_ulEntryIDSize);
	if((NULL == m_pEntryID) || (0 >= m_ulEntryIDSize))
		goto cleanup;
	
	//allocate BACKUPMSGINFO structure
	pbmiMsgInfo = (PBACKUPMSGINFO)calloc(1, sizeof(BACKUPMSGINFO));
	pgpAssert(NULL != pbmiMsgInfo);
	if(NULL == pbmiMsgInfo)
		goto cleanup;

	//allocate for entry id
	pbmiMsgInfo->lpEntryID = (LPENTRYID)calloc(1, m_ulEntryIDSize);
	pgpAssert(NULL != pbmiMsgInfo->lpEntryID);
	if(NULL == pbmiMsgInfo->lpEntryID)
		goto cleanup;

	//copy the size allocated
	pbmiMsgInfo->m_ulEntryIDSize = m_ulEntryIDSize;

	//copy the entry id
	memcpy(pbmiMsgInfo->lpEntryID, m_pEntryID, m_ulEntryIDSize);

	//copy the object pointer
	pbmiMsgInfo->pBackupObj = this;

	bRet = g_MsgBackupHash.Insert(pbmiMsgInfo);
	pgpAssert(TRUE == bRet);
	if(FALSE == bRet)
		goto cleanup;

	bRet = TRUE; //all success

cleanup:

	//free stuff upon failure
	if((FALSE == bRet) && (NULL != pbmiMsgInfo))
	{
		if(NULL != pbmiMsgInfo->lpEntryID)
		{
			pgpAssert(0 < pbmiMsgInfo->m_ulEntryIDSize);
			free(pbmiMsgInfo->lpEntryID);
			pbmiMsgInfo->lpEntryID = NULL;
			pbmiMsgInfo->m_ulEntryIDSize = 0;
		}

		free(pbmiMsgInfo);
		pbmiMsgInfo = NULL;
	}

	return bRet;
}

#ifdef _DEBUG

static void TraceNotification(LPNOTIFICATION lpNotifications)
{
	switch(lpNotifications->ulEventType)
	{
		case fnevCriticalError:
			ExchPluginTrace("ulEventType = fnevCriticalError\n");
			break;
		case fnevNewMail:
			ExchPluginTrace("ulEventType = fnevNewMail\n");
			break;
		case fnevObjectCreated:
			ExchPluginTrace("ulEventType = fnevObjectCreated\n");
			break;
		case fnevObjectDeleted:
			ExchPluginTrace("ulEventType = fnevObjectDeleted\n");
			break;
		case fnevObjectModified:
			ExchPluginTrace("ulEventType = fnevObjectModified\n");
			break;
		case fnevObjectMoved:
			ExchPluginTrace("ulEventType = fnevObjectMoved\n");
			break;
		case fnevObjectCopied:
			ExchPluginTrace("ulEventType = fnevObjectCopied\n");
			break;
		case fnevSearchComplete:
			ExchPluginTrace("ulEventType = fnevSearchComplete\n");
			break;
		case fnevTableModified:
			ExchPluginTrace("ulEventType = fnevTableModified\n");
			break;
		case fnevStatusObjectModified:
			ExchPluginTrace("ulEventType = fnevStatusObjectModified\n");
			break;
		case fnevReservedForMapi:
			ExchPluginTrace("ulEventType = fnevReservedForMapi\n");
			break;
		case fnevExtended:
			ExchPluginTrace("ulEventType = fnevExtended\n");
			break;
		default:
			ExchPluginTrace("ulEventType = unknown(%#x)\n", 
					lpNotifications->ulEventType);
			break;
	}
}

#endif

/*__Editor_settings____

  Local Variables:
  tab-width: 4
  End:
  vi: ts=4 sw=4
	vim: si
_____________________*/
