/*____________________________________________________________________________
	Copyright 2002 PGP Corporation
	All rights reserved.

	$Id: CPGPexchAttachedFileEvents.cpp,v 1.37 2002/11/19 02:19:38 sdas Exp $
____________________________________________________________________________*/
#include "CPGPexch.h"
#include <shlwapi.h>
#include "Globals.h"
#include "Utils.h"
#include "resource.h"

#include "EncryptSign.h"
#include "DecryptVerify.h"
#include "PGPsc.h"

#include "pgpClientPrefs.h"
#include "pgpUtilities.h"
#include "pgpDebug.h"

typedef struct _ExchWipeList ExchWipeList;
struct _ExchWipeList
{
	char *szFile;
	ExchWipeList *pNext;
};


STDMETHODIMP CPGPexch::OnReadPattFromSzFile(IAttach* lpAtt, 
											LPTSTR lpszFile, 
											ULONG ulFlags)
{
	if (!lpszFile)
		return S_FALSE;

	_bHaveAttachments = TRUE;
	return S_FALSE;
}


STDMETHODIMP CPGPexch::OnWritePattToSzFile(IAttach* lpAtt, 
										   LPTSTR lpszFile, 
										   ULONG ulFlags)
{
	char *szExtension;
	char *szOutFile = NULL;
	HKEY hKey;
	char szDrive[_MAX_DRIVE];
	char szDir[_MAX_DIR];
	char szName[_MAX_FNAME];
	char szExt[_MAX_EXT];

	_bDecryptAttach = FALSE;

	if (!lpszFile)
		return S_FALSE;

	if (_context != EECONTEXT_READNOTEMESSAGE)
		return S_FALSE;

	szExtension = strrchr(lpszFile, '.');
	if (!szExtension)
		return S_FALSE;
	if (stricmp(szExtension, ".asc") && stricmp(szExtension, ".pgp"))
		return S_FALSE;

	if (RegOpenKeyEx(HKEY_CLASSES_ROOT, szExtension, 0, KEY_QUERY_VALUE, &hKey)
		== ERROR_SUCCESS)
	{
		char szDescription[1024];
		DWORD dwSize = sizeof(szDescription);

		RegQueryValueEx(hKey, NULL, NULL, NULL, (PBYTE) szDescription, &dwSize);
		szDescription[dwSize] = 0;
		RegCloseKey(hKey);

		strcat(szDescription, "\\shell\\open\\command");
		if (RegOpenKeyEx(HKEY_CLASSES_ROOT, szDescription, 0, KEY_QUERY_VALUE,
			&hKey) == ERROR_SUCCESS)
		{
			char szPath[_MAX_PATH];
			DWORD dwPathSize = sizeof(szDescription);
			
			RegQueryValueEx(hKey, NULL, NULL, NULL, (PBYTE) szPath, &dwPathSize);
			szPath[dwPathSize] = 0;
			RegCloseKey(hKey);

			_splitpath(szPath, szDrive, szDir, szName, szExt);
			if (stricmp(szName, "pgpmail") && stricmp(szName, "pgptray"))
				return S_FALSE;
		}
	}

	_bDecryptAttach = TRUE;
	return S_FALSE;
}


STDMETHODIMP CPGPexch::QueryDisallowOpenPatt(IAttach* lpAtt)
{
	return S_FALSE;
}


STDMETHODIMP CPGPexch::OnOpenPatt(IAttach* lpAtt)
{
	return S_FALSE;
}


STDMETHODIMP CPGPexch::OnOpenSzFile(LPTSTR lpszFile, ULONG ulFlags)
{
	char *szOutFile = NULL;
	HRESULT hr;

	if (lpszFile == NULL)
		return S_FALSE;

	if (!_bDecryptAttach)
		return S_FALSE;

	hr = OpenSavePGPAttachment(lpszFile, &szOutFile);
	
	if (szOutFile == NULL)
		return hr;

	AddToWipeList(szOutFile);
	ShellExecute(NULL, "open", szOutFile, NULL, "c:\\", SW_SHOWDEFAULT);
	PGPFreeData(szOutFile);
	return S_OK;
}


HRESULT CPGPexch::OpenSavePGPAttachment(LPTSTR lpszFile, char **pszOutFile)
{
	char *szExtension;
	char *szOutFile = NULL;
	int nError;
	char szName[256];
	char szFile[1024];
	BOOL bDummy;

	*pszOutFile = NULL;

	UIGetString(szName, sizeof(szName), IDS_LOGNAME);
	UIGetString(szFile, sizeof(szFile), IDS_DLL);

	nError = DecryptVerifyFile(UIGetInstance(), NULL, _pgpContext, 
				_tlsContext, lpszFile, szFile, lpszFile, FALSE, TRUE, 
				&szOutFile, NULL, NULL, &bDummy);

	if ((nError == kPGPError_NoErr) && (szOutFile != NULL))
	{
		szExtension = strrchr(szOutFile, '.');
		
		if (szExtension)
		{
			if (!stricmp(szExtension, ".exe") || !stricmp(szExtension, ".com") || 
				!stricmp(szExtension, ".bat") || !stricmp(szExtension, ".cmd"))
			{
				OPENFILENAME saveAs;
				char *szNewFile;
				char szTempFile[MAX_PATH];
				char szTempPath[MAX_PATH];
				char szTitle[256];

				szNewFile = (char *) PGPNewData(_memoryMgr, MAX_PATH, 0);
				strcpy(szNewFile, szOutFile);
				GetTempPath(MAX_PATH, szTempPath);
				GetTempFileName(szTempPath, "pgp", 0, szTempFile);
				CopyFile(szOutFile, szTempFile, FALSE);
				WipeMessageAttachment(szOutFile);

				UIGetString(szTitle, sizeof(szTitle), IDS_SAVEDECRYPTEDFILE);

				saveAs.lStructSize = sizeof(OPENFILENAME);
				saveAs.hwndOwner = NULL;
				saveAs.lpstrFilter = NULL;
				saveAs.lpstrCustomFilter = NULL;
				saveAs.nFilterIndex = 0;
				saveAs.lpstrFile = szNewFile;
				saveAs.nMaxFile = MAX_PATH;
				saveAs.lpstrFileTitle = NULL;
				saveAs.lpstrInitialDir = NULL;
				saveAs.lpstrTitle = szTitle;
				saveAs.Flags = OFN_NOREADONLYRETURN | OFN_OVERWRITEPROMPT |
								OFN_PATHMUSTEXIST | OFN_NOCHANGEDIR;
				saveAs.lpstrDefExt = NULL;

				if (GetSaveFileName(&saveAs))
					CopyFile(szTempFile, szNewFile, FALSE);

				WipeMessageAttachment(szTempFile);
				PGPFreeData(szNewFile);
				PGPFreeData(szOutFile);

				return E_ABORT;
			}
		}

		SetFileAttributes(szOutFile, FILE_ATTRIBUTE_READONLY);
		*pszOutFile = szOutFile;
		return S_OK;
	}

	if (szOutFile != NULL)
		PGPFreeData(szOutFile);

	return S_FALSE;
}


int CPGPexch::EncryptSignAttachment(HWND hwnd,
									IMessage *psmg,
									UINT nOutSize,
									SRowSet *prAttach,
									PGPclRecipientDialogStruct *prds,
									PGPOptionListRef *pSignOptions)
{
	ULONG ulRow = 0;
	ULONG ulNum = 0;
	SizedSPropTagArray(6, tagaAttach) = {
		6, {PR_ATTACH_METHOD, PR_ATTACH_TAG, PR_RENDERING_POSITION, 
			PR_ATTACH_FILENAME, PR_ATTACH_LONG_FILENAME, 
			PR_DISPLAY_NAME}};
	enum {ivalAttachMethod=0, ivalAttachTag, ivalRenderingPos, ivalFilename, 
			ivalLongFilename, ivalDisplayName};
	SPropValue *pval = NULL;
	SPropValue PropValArr[6] = {0};
	ULONG ulNumVals = 0;
	LPATTACH pAttach = NULL;
	IStream *pstrmAttach = NULL;
	STATSTG AttStreamStats = {0};
	DWORD dwAttSize = 0;
	UINT nAttOutSize = 0;
	char *pAttachText = NULL;
	char *pOutput = NULL;
	char *szFileName = NULL;
	char *szLongFileName = NULL;
	char *szExtension = NULL;
	int nError = 0;
	char szName[256] = {0};
	char szFile[256] = {0};
	HRESULT hrRet = S_OK;
	
	UIGetString(szName, sizeof(szName), IDS_LOGNAME);
	UIGetString(szFile, sizeof(szFile), IDS_DLL);

	//for each attachment found in the message
	for (ulRow=0; ulRow<prAttach->cRows; ulRow++)
	{
		ulNum = prAttach->aRow[ulRow].lpProps[0].Value.l;

		pgpAssert(NULL == pAttach);
		hrRet = psmg->OpenAttach(ulNum, NULL, MAPI_MODIFY, &pAttach);
		pgpAssert((S_OK == hrRet) && (NULL != pAttach));
		if((S_OK != hrRet) || (NULL == pAttach))
			goto attach_cleanup;

		pgpAssert(NULL == pval);
		hrRet = pAttach->GetProps((SPropTagArray *)&tagaAttach, 0, 
					&ulNumVals, &pval);
		pgpAssert((S_OK == hrRet) || (MAPI_W_ERRORS_RETURNED == hrRet));
		if((S_OK != hrRet) && (MAPI_W_ERRORS_RETURNED != hrRet))
			goto attach_cleanup;

		switch (pval[ivalAttachMethod].Value.l)
		{
			case ATTACH_BY_VALUE:
				// Don't sign attachments unless we're also encrypting them
				if (_bEncrypt)
				{
					pgpAssert(NULL == pstrmAttach);
					hrRet = pAttach->OpenProperty(PR_ATTACH_DATA_BIN, &IID_IStream, 
						STGM_READWRITE, MAPI_MODIFY, (LPUNKNOWN *) &pstrmAttach);
					pgpAssert((S_OK == hrRet) && (NULL != pstrmAttach));
					if((S_OK != hrRet) || (NULL == pstrmAttach))
						goto attach_cleanup;

					hrRet = pstrmAttach->Stat(&AttStreamStats, STATFLAG_NONAME);
					pgpAssert(S_OK == hrRet);
					if(S_OK != hrRet)
						goto attach_cleanup;

					dwAttSize = AttStreamStats.cbSize.LowPart;
					
					pgpAssert(NULL == pAttachText);
					pgpAssert(dwAttSize > 0);
					pAttachText = (char *) calloc(dwAttSize+1, sizeof(char));
					pgpAssert(NULL != pAttachText);
					if(NULL == pAttachText)
						goto attach_cleanup;

					hrRet = pstrmAttach->Read(pAttachText, dwAttSize, &dwAttSize);
					pgpAssert(S_OK == hrRet);
					if(FAILED(hrRet))
						goto attach_cleanup;

					pAttachText[dwAttSize] = 0;

					nError = EncryptSignBuffer(UIGetInstance(), hwnd, 
						_pgpContext, _tlsContext, szName, szFile,
						pAttachText, dwAttSize, prds, NULL, pSignOptions, 
						(VOID ** )&pOutput, &nAttOutSize, _bEncrypt, _bSign, 
						TRUE, FALSE, m_pExtMdlessCallBack, m_pemOurMdlessCallBack);
					
					if (!nError)
					{
						ULONG ulAttOutSize = nAttOutSize;
						LARGE_INTEGER li = {0};
						ULARGE_INTEGER uli = {ulAttOutSize, 0};
						ULONG ulWritten = 0;
						ULARGE_INTEGER uliSeek = {0};
						
						hrRet = pstrmAttach->Seek(li, STREAM_SEEK_SET, &uliSeek);
						pgpAssert(S_OK == hrRet);
						if(S_OK != hrRet)
							goto attach_cleanup;

						pgpAssert(NULL != pOutput);
						hrRet = pstrmAttach->Write(pOutput, ulAttOutSize, &ulWritten);
						pgpAssert(S_OK == hrRet);
						pgpAssert(ulAttOutSize == ulWritten);
						if((S_OK != hrRet) || (ulAttOutSize != ulWritten))
							goto attach_cleanup;

						hrRet = pstrmAttach->SetSize(uli);
						pgpAssert(S_OK == hrRet);
						if(S_OK != hrRet)
							goto attach_cleanup;

						hrRet = pstrmAttach->Commit(STGC_DEFAULT);
						pgpAssert(S_OK == hrRet);
						if(S_OK != hrRet)
							goto attach_cleanup;
					}

					if (pval[ivalRenderingPos].Value.l > -1)
						pval[ivalRenderingPos].Value.l = nOutSize + ulRow;
					
					szFileName = (char *) calloc(MAX_PATH, sizeof(char));
					pgpAssert(NULL != szFileName);
					if(NULL == szFileName)
						goto attach_cleanup;

					szLongFileName = (char *) calloc(MAX_PATH, sizeof(char));
					pgpAssert(NULL != szLongFileName);
					if(NULL == szLongFileName)
						goto attach_cleanup;

					strcpy(szFileName, pval[ivalFilename].Value.lpszA);
					strcpy(szLongFileName, pval[ivalLongFilename].Value.lpszA);

					szExtension = strrchr(szFileName, '.');
					if (NULL != szExtension)
						*szExtension = '\0';
					
					if (strlen(szFileName) + strlen(".asc") >= MAX_PATH)
					{
						char *szTrunc = strrchr(szFileName, '.');
						
						if (szTrunc)
							strcpy(szTrunc - 4, szTrunc);
						else
						{
							szTrunc = szFileName + strlen(szFileName) - 4;
							*szTrunc = 0;
						}
					}

					if (strlen(szLongFileName) + strlen(".asc") >= MAX_PATH)
					{
						char *szTrunc = strrchr(szLongFileName, '.');
						
						if (szTrunc)
							strcpy(szTrunc - 4, szTrunc);
						else
						{
							szTrunc = szLongFileName + strlen(szLongFileName) - 4;
							*szTrunc = 0;
						}
					}

					strcat(szFileName, ".asc");
					strcat(szLongFileName, ".asc");

					//copy the property values obtained
					memcpy(PropValArr, pval, sizeof(PropValArr));
					PropValArr[ivalFilename].Value.lpszA = NULL;
					PropValArr[ivalLongFilename].Value.lpszA = NULL;

					//allocate and copy the short file name to be set
					pgpAssert(PT_STRING8 == PROP_TYPE(PropValArr[ivalFilename].ulPropTag));
					pgpAssert(NULL == PropValArr[ivalFilename].Value.lpszA);

					PropValArr[ivalFilename].Value.lpszA = _strdup(szFileName);
					pgpAssert(NULL != PropValArr[ivalFilename].Value.lpszA);
					if(NULL == PropValArr[ivalFilename].Value.lpszA)
						goto attach_cleanup;

					//allocate for longfilename property
					pgpAssert(PT_STRING8 == PROP_TYPE(PropValArr[ivalLongFilename].ulPropTag));
					pgpAssert(NULL == PropValArr[ivalLongFilename].Value.lpszA);

					PropValArr[ivalLongFilename].Value.lpszA = _strdup(szLongFileName);
					pgpAssert(NULL != PropValArr[ivalLongFilename].Value.lpszA);
					if(NULL == PropValArr[ivalLongFilename].Value.lpszA)
						goto attach_cleanup;

					//if we could not get the display name property
					if(PT_ERROR == PROP_TYPE(PropValArr[ivalDisplayName].ulPropTag))
					{
						//make the display name property same as the long file name property
						//NOTE: we do not have a problem during freeing since we do not free
						//this buffer irrespective of display name being found or not. when 
						//it is found it points to the original getprops buffer otherwise it 
						//points to strduped buffer that was passed as long file name value
						PropValArr[ivalDisplayName].Value.lpszA = 
								PropValArr[ivalLongFilename].Value.lpszA;
						pgpAssert(NULL != PropValArr[ivalDisplayName].Value.lpszA);

						//set the property type appropriately
						PropValArr[ivalDisplayName].ulPropTag = 
							CHANGE_PROP_TYPE(PR_DISPLAY_NAME, PT_STRING8);
					}

					if (!nError)
					{
						pgpAssert(NULL != pval);
						hrRet = pAttach->SetProps(6, PropValArr, NULL);
						pgpAssert(S_OK == hrRet);
						if(S_OK != hrRet)
							goto attach_cleanup;
					}

				}
				else
				{
					if (pval[ivalRenderingPos].Value.l > -1)
						pval[ivalRenderingPos].Value.l = nOutSize + ulRow;

					hrRet = pAttach->SetProps(6, pval, NULL);
					pgpAssert(S_OK == hrRet);
					if(S_OK != hrRet)
						goto attach_cleanup;
				}

				hrRet = pAttach->SaveChanges(0);
				pgpAssert(S_OK == hrRet);
				if(S_OK != hrRet)
					goto attach_cleanup;

				break;
				
			case ATTACH_BY_REFERENCE:
				UIDisplayStringID(hwnd, IDS_E_CANTENCRYPTATTACH);
				nError = kPGPError_UserAbort;
				break;
				
			case ATTACH_BY_REF_RESOLVE:
				UIDisplayStringID(hwnd, IDS_E_CANTENCRYPTATTACH);
				nError = kPGPError_UserAbort;
				break;
				
			case ATTACH_BY_REF_ONLY:
				UIDisplayStringID(hwnd, IDS_E_CANTENCRYPTATTACH);
				nError = kPGPError_UserAbort;
				break;
				
			case ATTACH_EMBEDDED_MSG:
				{
					IMessage *pmsg = NULL;
					BOOL bOrigConvert;

					hrRet = pAttach->OpenProperty(PR_ATTACH_DATA_OBJ, &IID_IMessage, 0, 
						MAPI_MODIFY, (LPUNKNOWN *) &pmsg);
					pgpAssert((S_OK == hrRet) && (NULL != pmsg));

					if(SUCCEEDED(hrRet) && (NULL != pmsg))
					{
						bOrigConvert = _bConvert;
						_bConvert = TRUE;
						nError = EncryptSignMessage(hwnd, pmsg, prds, pSignOptions);
						_bConvert = bOrigConvert;

						hrRet = pmsg->SaveChanges(0);
						pgpAssert(S_OK == hrRet);

						pmsg->Release();
						pmsg = NULL;
					}
				}

				pgpAssert(NULL != pAttach);
				hrRet = pAttach->SaveChanges(0);
				pgpAssert(SUCCEEDED(hrRet));

				break;

			case ATTACH_OLE:
				// Don't sign attachments unless we're also encrypting them
				if (_bEncrypt)
				{
					IStorage *pStg = NULL;

					hrRet = pAttach->OpenProperty(PR_ATTACH_DATA_OBJ, &IID_IStorage,
							0, MAPI_MODIFY, (LPUNKNOWN *) &pStg);
					pgpAssert(SUCCEEDED(hrRet) && (NULL != pStg));
					if (SUCCEEDED(hrRet) && (NULL != pStg))
					{
						IEnumSTATSTG *pEnum = NULL;
						STATSTG stat = {0};

						hrRet = pStg->EnumElements(0, NULL, 0, &pEnum);
						pgpAssert(SUCCEEDED(hrRet) && (NULL != pEnum));
						if(SUCCEEDED(hrRet) && (NULL != pEnum))
						{
							while (pEnum->Next(1, &stat, NULL) == S_OK)
							{
								char szOleName[256] = {0};

								wcstombs(szOleName, stat.pwcsName, sizeof(szOleName));
								if (!strncmp(&szOleName[1], "Ole", 4) ||
									!strncmp(&szOleName[1], "MailStream", 11) ||
									!strncmp(szOleName, "RichEditFlags", 14))
								{
									CoTaskMemFree(stat.pwcsName);
									continue;
								}

								if (stat.cbSize.LowPart)
								{
									pgpAssert(NULL == pstrmAttach);
									hrRet = pStg->OpenStream(stat.pwcsName, NULL, 
											STGM_READWRITE | STGM_SHARE_EXCLUSIVE, 0,
											&pstrmAttach);
									pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmAttach));
									if (FAILED(hrRet) || (NULL == pstrmAttach))
										continue;
									
									hrRet = pstrmAttach->Stat(&AttStreamStats, STATFLAG_NONAME);
									pgpAssert(S_OK == hrRet);

									dwAttSize = AttStreamStats.cbSize.LowPart;

									pAttachText = (char *) calloc(dwAttSize+1, sizeof(char));
									pgpAssert(NULL != pAttachText);
									if(NULL != pAttachText)
									{
										hrRet = pstrmAttach->Read(pAttachText, dwAttSize, &dwAttSize);
										pgpAssert(S_OK == hrRet);

										pAttachText[dwAttSize] = 0;
									}
									
									nError = EncryptSignBuffer(UIGetInstance(), hwnd, 
												_pgpContext, _tlsContext, szName, 
												szFile, pAttachText, dwAttSize, prds, 
												NULL, pSignOptions, 
												(VOID ** )&pOutput, &nAttOutSize, 
												_bEncrypt, _bSign, TRUE, FALSE, 
												m_pExtMdlessCallBack,
												m_pemOurMdlessCallBack);
									
									if (!nError)
									{
										ULONG ulAttOutSize = nAttOutSize;
										LARGE_INTEGER li = {0};
										ULARGE_INTEGER uli = {ulAttOutSize, 0};
										ULONG ulWritten = 0;
										ULARGE_INTEGER uliSeek = {0};
										
										hrRet = pstrmAttach->Seek(li, STREAM_SEEK_SET, &uliSeek);
										pgpAssert(S_OK == hrRet);

										hrRet = pstrmAttach->Write(pOutput, ulAttOutSize, &ulWritten);
										pgpAssert(S_OK == hrRet);

										hrRet = pstrmAttach->SetSize(uli);
										pgpAssert(S_OK == hrRet);

										hrRet = pstrmAttach->Commit(STGC_DEFAULT);
										pgpAssert(S_OK == hrRet);

										PGPFreeData(pOutput);
										pOutput = NULL;
									}

									free(pAttachText);
									pAttachText = NULL;
								}

								CoTaskMemFree(stat.pwcsName);
							}
						}
			
						pEnum->Release();
						pEnum = NULL;

						hrRet = pStg->Commit(STGC_DEFAULT);
						pgpAssert(S_OK == hrRet);

						pStg->Release();
						pStg = NULL;
					}
				}
				if (pval[ivalRenderingPos].Value.l > -1)
					pval[ivalRenderingPos].Value.l = nOutSize + ulRow;

				hrRet = pAttach->SetProps(6, pval, NULL);
				pgpAssert(SUCCEEDED(hrRet));

				hrRet = pAttach->SaveChanges(0);
				pgpAssert(SUCCEEDED(hrRet));

				break;
				
			default:
				break;
		}
		
		attach_cleanup:
		//free up both the properties for which we might have allocated
		if(NULL != PropValArr[ivalFilename].Value.lpszA)
		{
			free(PropValArr[ivalFilename].Value.lpszA);
			PropValArr[ivalFilename].Value.lpszA = NULL;
		}
		if(NULL != PropValArr[ivalLongFilename].Value.lpszA)
		{
			free(PropValArr[ivalLongFilename].Value.lpszA);
			PropValArr[ivalLongFilename].Value.lpszA = NULL;
		}

		//reset the setprops property value list
		ZeroMemory(PropValArr, sizeof(PropValArr));

		if (NULL != pval)
		{
			hrRet = MAPIFreeBuffer(pval);
			pgpAssert(S_OK == hrRet);
			pval = NULL;
		}

		if (NULL != pAttachText)
		{
			free(pAttachText);
			pAttachText = NULL;
		}

		if (NULL != pAttach)
		{
			pAttach->Release();
			pAttach = NULL;
		}

		if (NULL != pstrmAttach)
		{
			pstrmAttach->Release();
			pstrmAttach = NULL;
		}

		if (NULL != szFileName)
		{
			free(szFileName);
			szFileName = NULL;
		}

		if (NULL != szLongFileName)
		{
			free(szLongFileName);
			szLongFileName = NULL;
		}

		if (NULL != pOutput)
		{
			PGPFreeData(pOutput);
			pOutput = NULL;
		}
		
		if (nError)
			break;
	}

	return nError;
}


int CPGPexch::DecryptVerifyRestoreAttachment(HWND hwnd,
											 IMessage *psmg,
											 SRowSet *prAttach,
											 BOOL bRestore)
{
	ULONG ulRow = 0;
	ULONG ulNum = 0;
	SizedSPropTagArray(6, tagaAttach) = {
		6, {PR_ATTACH_METHOD, PR_ATTACH_TAG, PR_RENDERING_POSITION, 
			PR_ATTACH_FILENAME, PR_ATTACH_LONG_FILENAME, 
			PR_DISPLAY_NAME}};
	enum {ivalAttachMethod=0, ivalAttachTag, ivalRenderingPos, ivalFilename, 
			ivalLongFilename, ivalDisplayName};
	SPropValue *pval = NULL;
	SPropValue PropValArr[6] = {0};
	ULONG ulNumVals;
	LPATTACH pAttach = 0;
	IStream *pstrmAttach = NULL;
	STATSTG AttStreamStats;
	DWORD dwAttSize = 0;
	UINT nAttOutSize = 0;
	char *pAttachText = NULL;
	char *pOutput = NULL;
	char *szFileName = NULL;
	char *szLongFileName = NULL;
	char *szExtension = NULL;
	int nError = 0;
	char szName[256] = {0};
	char szFile[256] = {0};
	BOOL FYEO;
	HRESULT hrRet = S_OK;
	
	UIGetString(szName, sizeof(szName), IDS_LOGNAME);
	UIGetString(szFile, sizeof(szFile), IDS_DLL);

	//for each attachment found in the message
	for (ulRow=0; ulRow < prAttach->cRows; ulRow++)
	{
		ulNum = prAttach->aRow[ulRow].lpProps[0].Value.l;

		//open the attachment
		pgpAssert(NULL == pAttach);
		hrRet = psmg->OpenAttach(ulNum, NULL, MAPI_MODIFY, &pAttach);
		pgpAssert((S_OK == hrRet) && (NULL != pAttach));
		if((S_OK != hrRet) || (NULL == pAttach))
			goto attach_cleanup;

		//get the interesting properties of the attachment
		pgpAssert(NULL == pval);
		hrRet = pAttach->GetProps((SPropTagArray *)&tagaAttach, 0, 
					&ulNumVals, &pval);
		pgpAssert((S_OK == hrRet) || (MAPI_W_ERRORS_RETURNED == hrRet));
		if((S_OK != hrRet) && (MAPI_W_ERRORS_RETURNED != hrRet))
			goto attach_cleanup;

		//if we failed to get short or long file name props 
		if((PT_ERROR == PROP_TYPE(pval[ivalFilename].ulPropTag)) ||
		   (PT_ERROR == PROP_TYPE(pval[ivalLongFilename].ulPropTag)))
			goto attach_cleanup;//bail

		switch (pval[ivalAttachMethod].Value.l)
		{
			case ATTACH_BY_VALUE:
				if (!strstr(pval[ivalFilename].Value.lpszA, ".asc") &&
					!strstr(pval[ivalFilename].Value.lpszA, ".pgp") && !bRestore)
					goto attach_cleanup;//skip the attachment

				pgpAssert(NULL == pstrmAttach);
				hrRet = pAttach->OpenProperty(PR_ATTACH_DATA_BIN, &IID_IStream, 0, 
					MAPI_MODIFY, (LPUNKNOWN *) &pstrmAttach);
				pgpAssert((S_OK == hrRet) && (NULL != pstrmAttach));
				if((S_OK != hrRet) || (NULL == pstrmAttach))
					goto attach_cleanup;

				if (bRestore)
				{
					PGPMsgBackupList *pAtt = NULL;

					pAtt = m_pBackupMsg->FindAttachment(ulNum);
					//pgpAssert(NULL != pAtt); //TODO: attachment number returned for main body is 1 plus the number of attachments
											   //so assertion fails here everytime a message is being restored
					if (NULL == pAtt)
						goto attach_cleanup;

					nError = kPGPError_NoErr;
					szFileName = pAtt->szName;
					szLongFileName = pAtt->szLongName;
					pOutput = pAtt->szData;
					nAttOutSize = pAtt->dwSize;
				}
				else
				{
					hrRet = pstrmAttach->Stat(&AttStreamStats, STATFLAG_NONAME);
					pgpAssert(S_OK == hrRet);
					if(S_OK != hrRet)
						goto attach_cleanup;

					dwAttSize = AttStreamStats.cbSize.LowPart;
					
					pgpAssert(NULL == pAttachText);
					pgpAssert(dwAttSize > 0);
					pAttachText = (char *) calloc(dwAttSize+1, sizeof(char));
					pgpAssert(NULL != pAttachText);
					if(NULL == pAttachText)
						goto attach_cleanup;

					hrRet = pstrmAttach->Read(pAttachText, dwAttSize, &dwAttSize);
					pgpAssert(S_OK == hrRet);
					if(FAILED(hrRet))
						goto attach_cleanup;

					pAttachText[dwAttSize] = 0;
					
					pgpAssert(PT_STRING8 == PROP_TYPE(pval[ivalFilename].ulPropTag));
					pgpAssert(NULL == szFileName);

					//allocate a buffer that has a bit more space for a 3 letter extension
					szFileName = (char *) calloc(strlen(pval[ivalFilename].Value.lpszA)+5, 
									sizeof(char));
					pgpAssert(NULL != szFileName);
					if(NULL == szFileName)
						goto attach_cleanup;

					//copy the short file name to the buffer just allocated
					strcpy(szFileName, pval[ivalFilename].Value.lpszA);
					
					pgpAssert(PT_STRING8 == PROP_TYPE(pval[ivalLongFilename].ulPropTag));
					pgpAssert(NULL == szLongFileName);

					//allocate and copy the long file name returned
					szLongFileName = _strdup(pval[ivalLongFilename].Value.lpszA);
					pgpAssert(NULL != szLongFileName);
					if(NULL == szLongFileName)
						goto attach_cleanup;

					LoseExtension(szFileName);
					LoseExtension(szLongFileName);

					szExtension = strrchr(szLongFileName, '.');
					if (NULL != szExtension)
					{
						strncat(szFileName, szExtension, 4);//add the extension to the short file name
						szExtension = NULL;
					}
					
					nError = DecryptVerifyBuffer(UIGetInstance(), hwnd, 
								_pgpContext, _tlsContext, szLongFileName, szFile,
								pAttachText, dwAttSize, FALSE, TRUE, FALSE, 
								(VOID ** )&pOutput, &nAttOutSize, &FYEO);
				}

				if (!nError && (dwAttSize != nAttOutSize))
				{
					ULONG ulAttOutSize = nAttOutSize;
					LARGE_INTEGER li = {0,0};
					ULARGE_INTEGER uli = {ulAttOutSize, 0};
					ULONG ulWritten = 0;
					ULARGE_INTEGER uliSeek = {0};
						
					if (m_pBackupMsg && !bRestore)
					{
						m_pBackupMsg->AddAttachment(ulNum, FALSE,
									pval[ivalFilename].Value.lpszA,
									pval[ivalLongFilename].Value.lpszA, 
									pval[ivalDisplayName].Value.lpszA, pAttachText, 
									dwAttSize);
					}

					hrRet = pstrmAttach->Seek(li, STREAM_SEEK_SET, &uliSeek);
					pgpAssert(S_OK == hrRet);
					if(S_OK != hrRet)
						goto attach_cleanup;

					hrRet = pstrmAttach->Write(pOutput, ulAttOutSize, &ulWritten);
					pgpAssert(S_OK == hrRet);
					pgpAssert(ulAttOutSize == ulWritten);
					if((S_OK != hrRet) || (ulAttOutSize != ulWritten))
						goto attach_cleanup;

					hrRet = pstrmAttach->SetSize(uli);
					pgpAssert(S_OK == hrRet);
					if(S_OK != hrRet)
						goto attach_cleanup;

					hrRet = pstrmAttach->Commit(STGC_DEFAULT);
					pgpAssert(S_OK == hrRet);
					if(S_OK != hrRet)
						goto attach_cleanup;

					//copy the property values obtained
					memcpy(PropValArr, pval, sizeof(PropValArr));
					PropValArr[ivalFilename].Value.lpszA = NULL;
					PropValArr[ivalLongFilename].Value.lpszA = NULL;

					//allocate and copy the short file name to be set
					pgpAssert(PT_STRING8 == PROP_TYPE(PropValArr[ivalFilename].ulPropTag));
					pgpAssert(NULL == PropValArr[ivalFilename].Value.lpszA);

					PropValArr[ivalFilename].Value.lpszA = _strdup(szFileName);
					pgpAssert(NULL != PropValArr[ivalFilename].Value.lpszA);
					if(NULL == PropValArr[ivalFilename].Value.lpszA)
						goto attach_cleanup;

					//allocate for longfilename property
					pgpAssert(PT_STRING8 == PROP_TYPE(PropValArr[ivalLongFilename].ulPropTag));
					pgpAssert(NULL == PropValArr[ivalLongFilename].Value.lpszA);

					PropValArr[ivalLongFilename].Value.lpszA = _strdup(szLongFileName);
					pgpAssert(NULL != PropValArr[ivalLongFilename].Value.lpszA);
					if(NULL == PropValArr[ivalLongFilename].Value.lpszA)
						goto attach_cleanup;

					//if we could not get the display name property
					if(PT_ERROR == PROP_TYPE(PropValArr[ivalDisplayName].ulPropTag))
					{
						//make the display name property same as the long file name property
						//NOTE: we do not have a problem during freeing since we do not free
						//this buffer irrespective of display name being found or not. when 
						//it is found it points to the original getprops buffer otherwise it 
						//points to strduped buffer that was passed as long file name value
						PropValArr[ivalDisplayName].Value.lpszA = 
								PropValArr[ivalLongFilename].Value.lpszA;
						pgpAssert(NULL != PropValArr[ivalDisplayName].Value.lpszA);

						//set the property type appropriately
						PropValArr[ivalDisplayName].ulPropTag = 
							CHANGE_PROP_TYPE(PR_DISPLAY_NAME, PT_STRING8);
					}

					hrRet = pAttach->SetProps(6, PropValArr, NULL);
					pgpAssert(S_OK == hrRet);
					if(FAILED(hrRet))
						goto attach_cleanup;

					hrRet = pAttach->SaveChanges(0);
					pgpAssert(S_OK == hrRet);
					if(FAILED(hrRet))
						goto attach_cleanup;
					
				}

				break;
				
			case ATTACH_BY_REFERENCE:
				UIDisplayStringID(hwnd, IDS_E_CANTDECRYPTATTACH);
				nError = kPGPError_UserAbort;
				break;
				
			case ATTACH_BY_REF_RESOLVE:
				UIDisplayStringID(hwnd, IDS_E_CANTDECRYPTATTACH);
				nError = kPGPError_UserAbort;
				break;
				
			case ATTACH_BY_REF_ONLY:
				UIDisplayStringID(hwnd, IDS_E_CANTDECRYPTATTACH);
				nError = kPGPError_UserAbort;
				break;
				
			case ATTACH_EMBEDDED_MSG:
				/*
				if (!bRestore)
				{
					IMessage *pEmbedMsg = 0;

					pAttach->OpenProperty(PR_ATTACH_DATA_OBJ, &IID_IMessage, 0, 
						MAPI_MODIFY, (LPUNKNOWN *) &pEmbedMsg);
					
					_bNested = TRUE;
					DecryptVerifyMessage(hwnd, pEmbedMsg, NULL);
					_bNested = FALSE;

					pEmbedMsg->SaveChanges(0);
					pEmbedMsg->Release();
				}
				*/
				break;
				
			case ATTACH_OLE:
				{
					HRESULT hr;
					IStorage *pStg = NULL;

					hr = pAttach->OpenProperty(PR_ATTACH_DATA_OBJ, &IID_IStorage, 
							0, MAPI_MODIFY, (LPUNKNOWN *) &pStg);

					if (pStg != NULL)
					{
						IEnumSTATSTG *pEnum;
						STATSTG stat;

						pStg->EnumElements(0, NULL, 0, &pEnum);
						while (pEnum->Next(1, &stat, NULL) == S_OK)
						{
							char szOleName[256];

							wcstombs(szOleName, stat.pwcsName, sizeof(szOleName));
							if (!strncmp(&szOleName[1], "Ole", 4) ||
								!strncmp(&szOleName[1], "MailStream", 11) ||
								!strncmp(szOleName, "RichEditFlags", 14))
							{
								CoTaskMemFree(stat.pwcsName);
								continue;
							}

							if (stat.cbSize.LowPart)
							{
								hr = pStg->OpenStream(stat.pwcsName, NULL, 
										STGM_READWRITE | STGM_SHARE_EXCLUSIVE, 0,
										&pstrmAttach);
								
								if (hr != S_OK)
									continue;
							
								if (bRestore)
								{
									PGPOLEBackupList *pOLE;

									pOLE = m_pBackupMsg->FindOLEAttachment(ulNum,
												stat.pwcsName);
									if (!pOLE)
										continue;

									nError = kPGPError_NoErr;
									pOutput = pOLE->szData;
									nAttOutSize = pOLE->dwSize;
								}
								else
								{
									pstrmAttach->Stat(&AttStreamStats, 
										STATFLAG_NONAME);
									dwAttSize = AttStreamStats.cbSize.LowPart;
									pAttachText = (char *) calloc(dwAttSize+1, 
										sizeof(char));
									pstrmAttach->Read(pAttachText, dwAttSize, 
										&dwAttSize);
									pAttachText[dwAttSize] = 0;
									
									nError = DecryptVerifyBuffer(UIGetInstance(), 
												hwnd, _pgpContext, _tlsContext, 
												szName, szFile, pAttachText, 
												dwAttSize, FALSE, TRUE, FALSE, 
												(VOID ** )&pOutput, &nAttOutSize, 
												&FYEO);
								}

								if (!nError && (dwAttSize != nAttOutSize))
								{
									ULONG ulAttOutSize = nAttOutSize;
									LARGE_INTEGER li = {0,0};
									ULARGE_INTEGER uli = {ulAttOutSize, 0};
									ULONG ulWritten;
									ULARGE_INTEGER uliSeek;
									
									if (m_pBackupMsg && !bRestore)
									{
										m_pBackupMsg->AddOLE(ulNum, stat.pwcsName,
											pAttachText, dwAttSize);
									}

									pstrmAttach->Seek(li, STREAM_SEEK_SET, &uliSeek);
									pstrmAttach->Write(pOutput, ulAttOutSize, &ulWritten);
									pstrmAttach->SetSize(uli);
									pstrmAttach->Commit(STGC_DEFAULT);
									if (!bRestore)
										PGPFreeData(pOutput);
								}

								if ((dwAttSize == nAttOutSize) && pOutput && 
									!bRestore)
									PGPFreeData(pOutput);

								pstrmAttach->Release();
								pstrmAttach = NULL;
								free(pAttachText);
								pAttachText = NULL;
							}
							
							CoTaskMemFree(stat.pwcsName);
							if ((dwAttSize > 0) && (dwAttSize == nAttOutSize))
								break;
						}
						
						pEnum->Release();
						pStg->Commit(STGC_DEFAULT);
						pStg->Release();
					}
				}
				pAttach->SaveChanges(0);
				break;
				
			default:
				break;
		}//end switch (pval[ivalAttachMethod].Value.l)

		attach_cleanup:

		//free up both the properties for which we might have allocated
		if(NULL != PropValArr[ivalFilename].Value.lpszA)
		{
			free(PropValArr[ivalFilename].Value.lpszA);
			PropValArr[ivalFilename].Value.lpszA = NULL;
		}
		if(NULL != PropValArr[ivalLongFilename].Value.lpszA)
		{
			free(PropValArr[ivalLongFilename].Value.lpszA);
			PropValArr[ivalLongFilename].Value.lpszA = NULL;
		}

		//reset the setprops property value list
		ZeroMemory(PropValArr, sizeof(PropValArr));

		//free up the getprops value array
		if(NULL != pval)
		{
			hrRet = MAPIFreeBuffer(pval);
			pgpAssert(S_OK == hrRet);
			pval = NULL;
		}

		if (NULL != pAttachText)
		{
			free(pAttachText);
			pAttachText = NULL;
		}
		if(NULL != pAttach)
		{
			pAttach->Release();
			pAttach = NULL;
		}
		if(NULL != pstrmAttach)
		{
			pstrmAttach->Release();
			pstrmAttach = NULL;
		}

		if(!bRestore)
		{
			if (NULL != pOutput)
			{
				PGPFreeData(pOutput);
				pOutput = NULL;
			}

			if(NULL != szFileName)
			{
				free(szFileName);
				szFileName = NULL;
			}
			if(NULL != szLongFileName)
			{
				free(szLongFileName);
				szLongFileName = NULL;
			}
		}
		
		if (nError)
			break;
	}//for each attachment

	return nError;
}


void CPGPexch::WipeMessageAttachment(char *szFile)
{
	if (_pgpContext && IsntPGPError(_errContext))
	{
		FILELIST *pList = NULL;
		
		if (PGPscAddToFileList(&pList, szFile, NULL, NULL))
			PGPscWipeFileList(NULL, _pgpContext, pList,
				WIPE_DONOTWARN | WIPE_DONOTPROGRESS,0);
	}
	return;
}

void CPGPexch::AddToWipeList(char *szFile)
{
	ExchWipeList *pHead = (ExchWipeList *) _pWipeList;
	ExchWipeList *pTail = NULL;
	ExchWipeList *pCurrent;

	while (pHead != NULL)
	{
		if (!stricmp(pHead->szFile, szFile))
			return;

		pTail = pHead;
		pHead = pHead->pNext;
	}

	pCurrent = (ExchWipeList *) PGPNewData(_memoryMgr, sizeof(ExchWipeList), 0);
	pCurrent->szFile = (char *) PGPNewData(_memoryMgr, strlen(szFile)+1, 0);
	strcpy(pCurrent->szFile, szFile);
	pCurrent->pNext = NULL;
	
	if (pTail != NULL)
		pTail->pNext = pCurrent;
	else
		_pWipeList = (void *) pCurrent;

	return;
}


void CPGPexch::WipeAttachmentList(void)
{
	ExchWipeList *pHead = (ExchWipeList *) _pWipeList;
	ExchWipeList *pTail = NULL;
	FILELIST *pWipe = NULL;

	if (_pgpContext && IsntPGPError(_errContext))
	{
		while (pHead != NULL)
		{
			PGPscAddToFileList(&pWipe, pHead->szFile, NULL, NULL);
			PGPFreeData(pHead->szFile);
			pTail = pHead;
			pHead = pHead->pNext;
			PGPFreeData(pTail);
		}

		PGPscWipeFileList(NULL, _pgpContext, pWipe,
			WIPE_DONOTWARN | WIPE_DONOTPROGRESS,0);

		_pWipeList = NULL;
	}
	return;
}

void CPGPexch::LoseExtension(char *pszFileName)
{
	char *szASCExtension = NULL;
	char *szPGPExtension = NULL;
	char *szExtension = NULL;
	char *szEnd = NULL;

	szEnd = pszFileName + strlen(pszFileName) - 1;
	szASCExtension = StrRStrI(pszFileName, szEnd, ".asc");
	szPGPExtension = StrRStrI(pszFileName, szEnd, ".pgp");
	if (szASCExtension && szPGPExtension)
	{
		if (szASCExtension > szPGPExtension)
			szExtension = szASCExtension;
		else
			szExtension = szPGPExtension;
	}
	else if (szASCExtension)
		szExtension = szASCExtension;
	else
		szExtension = szPGPExtension;
	*szExtension = '\0';
}


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
