/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: CPGPexchCommands.cpp,v 1.33 2002/11/22 23:35:13 sdas Exp $
____________________________________________________________________________*/
#include "CPGPexch.h"
#include <process.h>
#include <richedit.h>
#include "Globals.h"
#include "Recipients.h"
#include "Utils.h"
#include "resource.h"

#include "EncryptSign.h"
#include "DecryptVerify.h"
#include "Prefs.h"
#include "PGPsc.h"
#include "RichEdit_IO.h"
#include "pgpClientPrefs.h"
#include "pgpDebug.h"

enum {
	kPGPexch_StdBitmap,
	kPGPexch_AltBitmap,
	kPGPexch_XPBitmap
};

typedef struct {
		HWND hwndFound;
		int yMax;
		BOOL bInternetExplorer;
		BOOL bMicrosoftWord;
		BOOL bListView;
} FindStruct;

static HWND FindMessageWindow(IExchExtCallback* pmecb, FindStruct *fs);
BOOL CALLBACK ReportChildren(HWND hwnd, LPARAM lParam);
static BOOL ByDefaultConvert(void);
static void SetConvertPref(PGPBoolean bConvert);
static HRESULT IsSelectionRelevant(IExchExtCallback* peecbExchCallBack, 
								   BOOL &bRelevant);


STDMETHODIMP CPGPexch::InstallCommands(IExchExtCallback* pmecb, 
                            HWND hWnd, HMENU hMenu,
                            UINT * pcmdidBase, LPTBENTRY lptbeArray,
                            UINT ctbe, ULONG ulFlags)
{
	if ((EECONTEXT_READNOTEMESSAGE != _context) && 
		(EECONTEXT_SENDNOTEMESSAGE != _context) &&
		(EECONTEXT_VIEWER != _context))
		return S_FALSE;

	// First, the menu

	HMENU hmenuTools;
	HMENU hmenuHelp;
	HMENU hmenuHelpTopics;
	ULONG ulBeforeTools;
	ULONG ulAfterExchange;
	HRESULT hr = pmecb->GetMenuPos(EECMDID_Tools, &hmenuTools, 
									&ulBeforeTools, NULL, 0);
	if (S_OK != hr)
		return S_FALSE; // No such menu item?  Very bad.
	hr = pmecb->GetMenuPos(EECMDID_HelpAboutMicrosoftExchange, &hmenuHelp,
							NULL, NULL, 0);
	if (S_OK != hr)
		return S_FALSE; // No such menu item?  Very bad.
	hr = pmecb->GetMenuPos(EECMDID_HelpMicrosoftExchangeHelpTopics, 
							&hmenuHelpTopics, NULL, &ulAfterExchange, 0);
	if (S_OK != hr)
		return S_FALSE; // No such menu item?  Very bad.

	char szCommand[80];
	
	_hmenuPGP = CreatePopupMenu();
	if (_context == EECONTEXT_SENDNOTEMESSAGE)
		InsertMenu(hmenuTools, ulBeforeTools, 
					MF_BYPOSITION | MF_STRING | MF_POPUP, 
					(UINT) _hmenuPGP, "P&GP");
	else
		InsertMenu(hmenuTools, ulBeforeTools, 
					MF_BYPOSITION | MF_STRING | MF_POPUP, 
					(UINT) _hmenuPGP, "&PGP");

	if (_context == EECONTEXT_SENDNOTEMESSAGE)
	{
		UIGetString(szCommand, sizeof(szCommand), IDS_ENCRYPT_MENU);
		AppendMenu(_hmenuPGP, MF_STRING, *pcmdidBase, szCommand);
		_cmdidEncrypt = *pcmdidBase;
 		++(*pcmdidBase);

		UIGetString(szCommand, sizeof(szCommand), IDS_SIGN_MENU);
		AppendMenu(_hmenuPGP, MF_STRING, *pcmdidBase, szCommand);
		_cmdidSign = *pcmdidBase;
 		++(*pcmdidBase);

		UIGetString(szCommand, sizeof(szCommand), IDS_CONVERT_MENU);
		AppendMenu(_hmenuPGP, MF_STRING, *pcmdidBase, szCommand);
		_cmdidConvert = *pcmdidBase;
		++(*pcmdidBase);
		
		AppendMenu(_hmenuPGP, MF_SEPARATOR, 0, NULL);
		
		UIGetString(szCommand, sizeof(szCommand), IDS_ENCRYPTNOW_MENU);
		AppendMenu(_hmenuPGP, MF_STRING, *pcmdidBase, szCommand);
		_cmdidEncryptNow = *pcmdidBase;
		++(*pcmdidBase);
		
		UIGetString(szCommand, sizeof(szCommand), IDS_SIGNNOW_MENU);
		AppendMenu(_hmenuPGP, MF_STRING, *pcmdidBase, szCommand);
		_cmdidSignNow = *pcmdidBase;
		++(*pcmdidBase);
		
		UIGetString(szCommand, sizeof(szCommand), IDS_ENCRYPTSIGN_MENU);
		AppendMenu(_hmenuPGP, MF_STRING, *pcmdidBase, szCommand);
		_cmdidEncryptSign = *pcmdidBase;
		++(*pcmdidBase);
		
		AppendMenu(_hmenuPGP, MF_SEPARATOR, 0, NULL);
	}

	if ((_context == EECONTEXT_READNOTEMESSAGE) || 
		(_context == EECONTEXT_VIEWER))
	{
		UIGetString(szCommand, sizeof(szCommand), IDS_DECRYPT_MENU);
		AppendMenu(_hmenuPGP, MF_STRING, *pcmdidBase, szCommand);
		_cmdidDecrypt = *pcmdidBase;
 		++(*pcmdidBase);

		AppendMenu(_hmenuPGP, MF_SEPARATOR, 0, NULL);
	}

	UIGetString(szCommand, sizeof(szCommand), IDS_PGPKEYS_MENU);
	AppendMenu(_hmenuPGP, MF_STRING, *pcmdidBase, szCommand);
	_cmdidPgpKeys = *pcmdidBase;
	++(*pcmdidBase);

	UIGetString(szCommand, sizeof(szCommand), IDS_PGPPREFS_MENU);
	AppendMenu(_hmenuPGP, MF_STRING, *pcmdidBase, szCommand);
	_cmdidPrefs = *pcmdidBase;
	++(*pcmdidBase);
	
	UIGetString(szCommand, sizeof(szCommand), IDS_PGPHELP_MENU);
	InsertMenu(hmenuHelpTopics, ulAfterExchange, 
				MF_BYPOSITION | MF_STRING, *pcmdidBase, szCommand);
	_cmdidHelp = *pcmdidBase;
	++(*pcmdidBase);
	
	UIGetString(szCommand, sizeof(szCommand), IDS_ABOUT_MENU);
	AppendMenu(hmenuHelp, MF_STRING, *pcmdidBase, szCommand);
	_cmdidAbout = *pcmdidBase;
	++(*pcmdidBase);
	
	// Next, the toolbar

	int tbindx;
	HWND hwndToolbar = NULL;
	for (tbindx = ctbe-1; (int) tbindx > -1; --tbindx)
	{
		if (EETBID_STANDARD == lptbeArray[tbindx].tbid)
		{
			hwndToolbar = lptbeArray[tbindx].hwnd;
			if (_context == EECONTEXT_SENDNOTEMESSAGE)
			{
				_hwndSendToolbar = hwndToolbar;
				_itbbEncrypt = lptbeArray[tbindx].itbbBase++;
				_itbbSign = lptbeArray[tbindx].itbbBase++;
			}

			if ((_context == EECONTEXT_READNOTEMESSAGE) ||
				(_context == EECONTEXT_VIEWER))
			{
				_hwndReadToolbar = hwndToolbar;
				_itbbDecrypt = lptbeArray[tbindx].itbbBase++;
			}
			
			_itbbPgpKeys = lptbeArray[tbindx].itbbBase++;
			break;
		}
	}

	if (hwndToolbar)
	{
		TBADDBITMAP tbab;
		PGPclWindowsGenus osType;
		UINT nBitmapType;

		PGPclGetWindowsVersion(NULL, &osType, NULL);
		if (_fOutlookXP)
			nBitmapType = kPGPexch_XPBitmap;
		else if ((osType == kPGPclWindowsGenusUnknown) ||
			(osType == kPGPclWindowsGenus95) ||
			(osType == kPGPclWindowsGenusNT3) ||
			(osType == kPGPclWindowsGenusNT4) ||
			(osType == kPGPclWindowsGenus98))
		{
			nBitmapType = kPGPexch_StdBitmap;
		}
		else
			nBitmapType = kPGPexch_AltBitmap;

		tbab.hInst = UIGetInstance();
		if (_context == EECONTEXT_SENDNOTEMESSAGE)
		{
			switch (nBitmapType)
			{
			case kPGPexch_StdBitmap:
				tbab.nID = IDB_ENCRYPT;
				break;

			case kPGPexch_AltBitmap:
				tbab.nID = IDB_ALT_ENCRYPT;
				break;

			case kPGPexch_XPBitmap:
				tbab.nID = IDB_XP_ENCRYPT;
				break;
			}

			_itbmEncrypt = SendMessage(hwndToolbar, TB_ADDBITMAP, 1, 
							(LPARAM)&tbab);

			switch (nBitmapType)
			{
			case kPGPexch_StdBitmap:
				tbab.nID = IDB_SIGN;
				break;

			case kPGPexch_AltBitmap:
				tbab.nID = IDB_ALT_SIGN;
				break;

			case kPGPexch_XPBitmap:
				tbab.nID = IDB_XP_SIGN;
				break;
			}

			_itbmSign = SendMessage(hwndToolbar, TB_ADDBITMAP, 1, 
							(LPARAM)&tbab);
		}

		if ((_context == EECONTEXT_READNOTEMESSAGE) || 
			(_context == EECONTEXT_VIEWER))
		{
			switch (nBitmapType)
			{
			case kPGPexch_StdBitmap:
				tbab.nID = IDB_DECRYPT;
				break;

			case kPGPexch_AltBitmap:
				tbab.nID = IDB_ALT_DECRYPT;
				break;

			case kPGPexch_XPBitmap:
				tbab.nID = IDB_XP_DECRYPT;
				break;
			}

			_itbmDecrypt = SendMessage(hwndToolbar, TB_ADDBITMAP, 1, 
							(LPARAM)&tbab);
		}
		
		switch (nBitmapType)
		{
		case kPGPexch_StdBitmap:
			tbab.nID = IDB_PGPKEYS;
			break;
			
		case kPGPexch_AltBitmap:
			tbab.nID = IDB_ALT_PGPKEYS;
			break;
			
		case kPGPexch_XPBitmap:
			tbab.nID = IDB_XP_PGPKEYS;
			break;
		}
		
		_itbmPgpKeys = SendMessage(hwndToolbar, TB_ADDBITMAP, 1, 
						(LPARAM)&tbab);
	}

	return S_OK;
}


STDMETHODIMP CPGPexch::QueryButtonInfo (ULONG tbid, UINT itbb, 
                            LPTBBUTTON ptbb, LPTSTR lpsz, UINT cch, 
                            ULONG ulFlags)
{
	if ((EECONTEXT_READNOTEMESSAGE != _context) &&
		(EECONTEXT_SENDNOTEMESSAGE != _context) &&
		(EECONTEXT_VIEWER != _context))
		return S_FALSE;

	HRESULT hr = S_FALSE;

	if ((itbb == _itbbEncrypt) && (_context == EECONTEXT_SENDNOTEMESSAGE))
	{
		ptbb->iBitmap = _itbmEncrypt;
		ptbb->idCommand = _cmdidEncrypt;
		ptbb->fsState = TBSTATE_ENABLED;
		ptbb->fsStyle = TBSTYLE_CHECK;
		ptbb->dwData = 0;
		ptbb->iString = -1;

		UIGetString(lpsz, cch, IDS_ENCRYPT_TOOLTIP);
		hr = S_OK;
	}

	if ((itbb == _itbbSign) && (_context == EECONTEXT_SENDNOTEMESSAGE))
	{
		ptbb->iBitmap = _itbmSign;
		ptbb->idCommand = _cmdidSign;
		ptbb->fsState = TBSTATE_ENABLED;
		ptbb->fsStyle = TBSTYLE_CHECK;
		ptbb->dwData = 0;
		ptbb->iString = -1;

		UIGetString(lpsz, cch, IDS_SIGN_TOOLTIP);
		hr = S_OK;
	}

	if (itbb == _itbbPgpKeys)
	{
		ptbb->iBitmap = _itbmPgpKeys;
		ptbb->idCommand = _cmdidPgpKeys;
		ptbb->fsState = TBSTATE_ENABLED;
		ptbb->fsStyle = TBSTYLE_BUTTON;
		ptbb->dwData = 0;
		ptbb->iString = -1;

		UIGetString(lpsz, cch, IDS_PGPKEYS_TOOLTIP);
		hr = S_OK;
	}

	if ((itbb == _itbbDecrypt) && 
		((_context == EECONTEXT_READNOTEMESSAGE) || 
		(_context == EECONTEXT_VIEWER)))
	{
		ptbb->iBitmap = _itbmDecrypt;
		ptbb->idCommand = _cmdidDecrypt;
		ptbb->fsState = TBSTATE_ENABLED;
		ptbb->fsStyle = TBSTYLE_BUTTON;
		ptbb->dwData = 0;
		ptbb->iString = -1;

		UIGetString(lpsz, cch, IDS_DECRYPT_TOOLTIP);
		hr = S_OK;
	}


	return hr;
}


STDMETHODIMP CPGPexch::ResetToolbar(ULONG tbid, ULONG ulFlags)
{
	// To implement this method,
	// the extension must cache the results of a prior call
	// to IExchExtCallback::GetToolbar.

	return S_FALSE;
}


STDMETHODIMP CPGPexch::QueryHelpText(UINT cmdid, ULONG ulFlags, 
                                      LPTSTR psz, UINT cch)
{
	if ((EECONTEXT_READNOTEMESSAGE != _context) &&
		(EECONTEXT_SENDNOTEMESSAGE != _context) &&
		(EECONTEXT_VIEWER != _context))
		return S_FALSE;

	if (ulFlags == EECQHT_STATUS)
	{
		if ((cmdid == _cmdidEncrypt) && 
			(_context == EECONTEXT_SENDNOTEMESSAGE))
		{
			UIGetString(psz, cch, IDS_ENCRYPT_STATUS);
			return S_OK;
		}

		if ((cmdid == _cmdidSign) && 
			(_context == EECONTEXT_SENDNOTEMESSAGE))
		{
			UIGetString(psz, cch, IDS_SIGN_STATUS);
			return S_OK;
		}

		if ((cmdid == _cmdidEncryptNow) && 
			(_context == EECONTEXT_SENDNOTEMESSAGE))
		{
			UIGetString(psz, cch, IDS_ENCRYPTNOW_STATUS);
			return S_OK;
		}

		if ((cmdid == _cmdidSignNow) && 
			(_context == EECONTEXT_SENDNOTEMESSAGE))
		{
			UIGetString(psz, cch, IDS_SIGNNOW_STATUS);
			return S_OK;
		}

		if ((cmdid == _cmdidEncryptSign) && 
			(_context == EECONTEXT_SENDNOTEMESSAGE))
		{
			UIGetString(psz, cch, IDS_ENCRYPTSIGN_STATUS);
			return S_OK;
		}

		if ((cmdid == _cmdidDecrypt) && 
			((_context == EECONTEXT_READNOTEMESSAGE) ||
			(_context == EECONTEXT_VIEWER)))
		{
			UIGetString(psz, cch, IDS_DECRYPT_STATUS);
			return S_OK;
		}

		if (cmdid == _cmdidPgpKeys)
		{
			UIGetString(psz, cch, IDS_PGPKEYS_STATUS);
			return S_OK;
		}

		if (cmdid == _cmdidPrefs)
		{
			UIGetString(psz, cch, IDS_PGPPREFS_STATUS);
			return S_OK;
		}

		if (cmdid == _cmdidHelp)
		{
			UIGetString(psz, cch, IDS_PGPHELP_STATUS);
			return S_OK;
		}

		if (cmdid == _cmdidAbout)
		{
			UIGetString(psz, cch, IDS_ABOUT_STATUS);
			return S_OK;
		}

	}
	else if (ulFlags == EECQHT_TOOLTIP)
	{
		if ((cmdid == _cmdidEncrypt) && 
			(_context == EECONTEXT_SENDNOTEMESSAGE))
		{
			UIGetString(psz, cch, IDS_ENCRYPT_TOOLTIP);
			return S_OK;
		}

		if ((cmdid == _cmdidSign) && 
			(_context == EECONTEXT_SENDNOTEMESSAGE))
		{
			UIGetString(psz, cch, IDS_SIGN_TOOLTIP);
			return S_OK;
		}

		if ((cmdid == _cmdidEncryptNow) && 
			(_context == EECONTEXT_SENDNOTEMESSAGE))
		{
			UIGetString(psz, cch, IDS_ENCRYPTNOW_TOOLTIP);
			return S_OK;
		}

		if ((cmdid == _cmdidSignNow) && 
			(_context == EECONTEXT_SENDNOTEMESSAGE))
		{
			UIGetString(psz, cch, IDS_SIGNNOW_TOOLTIP);
			return S_OK;
		}

		if ((cmdid == _cmdidEncryptSign) && 
			(_context == EECONTEXT_SENDNOTEMESSAGE))
		{
			UIGetString(psz, cch, IDS_ENCRYPTSIGN_TOOLTIP);
			return S_OK;
		}

		if ((cmdid == _cmdidDecrypt) && 
			((_context == EECONTEXT_READNOTEMESSAGE) ||
			(_context == EECONTEXT_VIEWER)))
		{
			UIGetString(psz, cch, IDS_DECRYPT_TOOLTIP);
			return S_OK;
		}

		if (cmdid == _cmdidPgpKeys)
		{
			UIGetString(psz, cch, IDS_PGPKEYS_TOOLTIP);
			return S_OK;
		}
	}

	return S_FALSE;
}


STDMETHODIMP CPGPexch::Help(IExchExtCallback* pmecb, UINT cmdid)
{
	return S_FALSE;
}


STDMETHODIMP_(VOID) CPGPexch::InitMenu(IExchExtCallback* pmecb)
{
	FindStruct fs;
	HWND hwndMain;
	HWND hwndToolbar;
	HRESULT hr;
	char szTitle[256];
	BOOL bRichText = FALSE;
	BOOL bShowConvert = TRUE;

	if ((EECONTEXT_READNOTEMESSAGE != _context) &&
		(EECONTEXT_SENDNOTEMESSAGE != _context))
		return;

	pmecb->GetWindow(&hwndMain);
	GetWindowText(hwndMain, szTitle, sizeof(szTitle));
	if (strlen(szTitle) > 12)
	{
		if (!strncmp(szTitle+strlen(szTitle)-12, "(Rich Text)", 11))
			bRichText = TRUE;
	}

	_hwndMessage = FindMessageWindow(pmecb, &fs);

	hr = pmecb->GetToolbar(EETBID_STANDARD, &hwndToolbar);
	if (hr != S_OK)
		return;

	if (_context == EECONTEXT_SENDNOTEMESSAGE)
	{
		if (fs.bInternetExplorer || fs.bMicrosoftWord || bRichText)
		{
			DeleteMenu(_hmenuPGP, _cmdidEncryptNow, MF_BYCOMMAND);
			DeleteMenu(_hmenuPGP, _cmdidSignNow, MF_BYCOMMAND);
			DeleteMenu(_hmenuPGP, _cmdidEncryptSign, MF_BYCOMMAND);
			DrawMenuBar(hwndMain);
		}
	}

	if (!_fInitMenuOnce)
	{
		// Only set encrypt and sign flags once.
		
		_fInitMenuOnce = TRUE;
		
		_bEncrypt = ByDefaultEncrypt(_memoryMgr);
		_bSign = ByDefaultSign(_memoryMgr);
		_bConvert = ByDefaultConvert();
	}

	if (_context == EECONTEXT_READNOTEMESSAGE)
	{
		SPropValue *pval = NULL;
		HRESULT hr;
		ULONG numVals = 1;
		SizedSPropTagArray(1, taga) = {1, {PR_PGPEXCH_HTML}};
		IMessage *pmsg = NULL;

		// Warn user if message is HTML or RTF

		pmecb->GetObject(NULL, (IMAPIProp **) &pmsg);
		hr = pmsg->GetProps((SPropTagArray *) &taga, 0, &numVals, &pval);
		if (SUCCEEDED(hr))
		{
			if (pval[0].Value.b == TRUE)
				UIWarnOk(hwndMain, IDS_W_HTMLDECRYPT, 
					kPGPPrefWin32OutlookHTMLDecrypt);

			pval[0].Value.b = FALSE;
			pmsg->SetProps(1, pval, NULL);
			MAPIFreeBuffer(pval);
		}
		pmsg->Release();
	}

	if (_context == EECONTEXT_SENDNOTEMESSAGE)
	{
		HMENU hmenu;
		MENUITEMINFO miiEncrypt;
		MENUITEMINFO miiSign;
		MENUITEMINFO miiConvert;
		
		hr = pmecb->GetMenu(&hmenu);
		if (FAILED(hr))
			return;

		miiEncrypt.cbSize = sizeof(MENUITEMINFO);
		miiEncrypt.fMask = MIIM_STATE | MIIM_CHECKMARKS;
		GetMenuItemInfo(hmenu, _cmdidEncrypt, FALSE, &miiEncrypt);
		if (_bEncrypt)
		{
			miiEncrypt.fState = MFS_CHECKED;
			miiEncrypt.hbmpChecked = NULL;
			SendMessage(hwndToolbar, TB_CHECKBUTTON, 
				_cmdidEncrypt, MAKELONG(TRUE, 0));
		}
		else
		{
			miiEncrypt.fState = MFS_UNCHECKED;
			miiEncrypt.hbmpUnchecked = NULL;
			SendMessage(hwndToolbar, TB_CHECKBUTTON, 
				_cmdidEncrypt, MAKELONG(FALSE, 0));
		}
		SetMenuItemInfo(hmenu, _cmdidEncrypt, FALSE, &miiEncrypt);

		miiSign.cbSize = sizeof(MENUITEMINFO);
		miiSign.fMask = MIIM_STATE | MIIM_CHECKMARKS;
		GetMenuItemInfo(hmenu, _cmdidSign, FALSE, &miiSign);
		if (_bSign)
		{
			miiSign.fState = MFS_CHECKED;
			miiSign.hbmpChecked = NULL;
			SendMessage(hwndToolbar, TB_CHECKBUTTON, 
				_cmdidSign, MAKELONG(TRUE, 0));
		}
		else
		{
			miiSign.fState = MFS_UNCHECKED;
			miiSign.hbmpUnchecked = NULL;
			SendMessage(hwndToolbar, TB_CHECKBUTTON, 
				_cmdidSign, MAKELONG(FALSE, 0));
		}
		SetMenuItemInfo(hmenu, _cmdidSign, FALSE, &miiSign);

		if (bShowConvert)
		{
			miiConvert.cbSize = sizeof(MENUITEMINFO);
			miiConvert.fMask = MIIM_STATE | MIIM_CHECKMARKS;
			GetMenuItemInfo(hmenu, _cmdidConvert, FALSE, &miiConvert);
			if (_bConvert)
			{
				miiConvert.fState = MFS_CHECKED;
				miiConvert.hbmpChecked = NULL;
			}
			else
			{
				miiConvert.fState = MFS_UNCHECKED;
				miiConvert.hbmpUnchecked = NULL;
			}
			SetMenuItemInfo(hmenu, _cmdidConvert, FALSE, &miiConvert);
		}
	}
}


STDMETHODIMP CPGPexch::DoCommand(IExchExtCallback* pmecb, UINT cmdid)
{
	HWND hwndMain;
	FindStruct fs;
	BOOL FYEO;

	if ((EECONTEXT_READNOTEMESSAGE != _context) &&
		(EECONTEXT_SENDNOTEMESSAGE != _context) &&
		(EECONTEXT_VIEWER != _context))
		return S_FALSE;

	if ((cmdid == _cmdidEncrypt) && (_context == EECONTEXT_SENDNOTEMESSAGE))
	{
		_bEncrypt = !_bEncrypt;
		InitMenu(pmecb);
		return S_OK;
	}

	if ((cmdid == _cmdidSign) && (_context == EECONTEXT_SENDNOTEMESSAGE))
	{
		_bSign = !_bSign;
		InitMenu(pmecb);
		return S_OK;
	}

	if ((cmdid == _cmdidConvert) && (_context == EECONTEXT_SENDNOTEMESSAGE))
	{
		_bConvert = !_bConvert;
		SetConvertPref(_bConvert);
		InitMenu(pmecb);
		return S_OK;
	}

	pmecb->GetWindow(&hwndMain);
	_hwndMessage = FindMessageWindow(pmecb, &fs);

	if (((cmdid == _cmdidEncryptNow) || (cmdid == _cmdidSignNow) || 
		(cmdid == _cmdidEncryptSign)) && 
		(_context == EECONTEXT_SENDNOTEMESSAGE))
	{
		BOOL bEncrypt;
		BOOL bSign;
		
		bEncrypt =	(cmdid == _cmdidEncryptNow) || 
					(cmdid == _cmdidEncryptSign);

		bSign = (cmdid == _cmdidSignNow) || 
				(cmdid == _cmdidEncryptSign); 

		if (_bHaveAttachments)
		{
			if (_fOutlook98)
			{
				if (!UIWarnYesNo(hwndMain, IDS_Q_NOENCRYPTATTACH, 
						kPGPPrefWin32OutlookAttachment, TRUE))
					return S_OK;
			}
			else
			{
				if (!UIWarnYesNo(hwndMain, IDS_Q_ATTACHMENT, 
						kPGPPrefWin32OutlookAttachment, TRUE))
					return S_OK;
			}
				
		}

		if (_hwndMessage)
		{
			PGPclRecipientDialogStruct *prds;
			PGPError err;

			prds = (PGPclRecipientDialogStruct *) 
					calloc(sizeof(PGPclRecipientDialogStruct), 1);

			err = 	PGPclOpenDefaultKeyrings(_pgpContext, 
						kPGPOpenKeyDBFileOptions_Mutable, 
						&(prds->keydbOriginal));

			if (IsPGPError(err))
				err = 	PGPclOpenDefaultKeyrings(_pgpContext, 
							kPGPOpenKeyDBFileOptions_None, 
							&(prds->keydbOriginal));

			if (IsPGPError(err))
			{
				UIDisplayErrorCode(__FILE__, __LINE__, NULL, err, TRUE);
				return S_FALSE;
			}

			if (EncryptSignMessageWindow(hwndMain, pmecb, bEncrypt, bSign, prds))
			{
				_bEncrypt = _bEncrypt && !bEncrypt;
				_bSign = _bSign && !bSign;
				InitMenu(pmecb);
			}

			FreeRecipients(prds);
	
			if (prds->keydbOriginal != NULL)
			{
				PGPFreeKeyDB(prds->keydbOriginal);
				prds->keydbOriginal = NULL;
			}

			free(prds);
		}

		return S_OK;
	}

	if ((cmdid == _cmdidDecrypt) && 
		((_context == EECONTEXT_READNOTEMESSAGE) ||
		(_context == EECONTEXT_VIEWER)))
	{
		if (_hwndMessage)
		{
			char *szInput;
			char *szOutput = NULL;
			long lLength;
			UINT nOutLength;
			BOOL bSelectedText = FALSE;
			CHARRANGE chRange = {0,0};
			PGPError nError = kPGPError_NoErr;
			char szName[256];
			char szFile[256];

			UIGetString(szName, sizeof(szName), IDS_LOGNAME);
			UIGetString(szFile, sizeof(szFile), IDS_DLL);

			if (fs.bInternetExplorer && !AutoDecrypt(_memoryMgr))
			{
				if (_context == EECONTEXT_VIEWER)
				{
					char szMsg[256];
					char szTitle[256];

					LoadString(UIGetInstance(), IDS_TITLE, szTitle, 255);
					LoadString(UIGetInstance(), IDS_E_HTMLPREVIEW, szMsg, 255);
					MessageBox(hwndMain, szMsg, szTitle, MB_ICONINFORMATION);
					return S_OK;
				}
				else
				{
					char szMsg[256];
					char szTitle[256];

					LoadString(UIGetInstance(), IDS_TITLE, szTitle, 255);
					LoadString(UIGetInstance(), IDS_E_HTMLDECRYPT, szMsg, 255);
					MessageBox(hwndMain, szMsg, szTitle, MB_ICONINFORMATION);
					return S_OK;
				}
			}
			else
			{
				//if we are in the main window of outlook
				if(EECONTEXT_VIEWER == _context)
				{
					//control comes here only if decrypt was chosen from the main menu
					//when the user is not in the standard inbox view such as in calendar

					//check if the current selection is relevant for us
					BOOL bSelRelevant = FALSE;
					HRESULT hrRet = IsSelectionRelevant(pmecb, bSelRelevant);
					if(SUCCEEDED(hrRet) && (FALSE == bSelRelevant))
						return S_OK; //we are done since the selection is irrelevant to PGP
				}

				// Determine if the user selected a particular piece of text
				szInput = GetRichEditContents(_hwndMessage, &lLength, 
					FALSE, FALSE);
				if (!szInput)
					return S_OK;
				
				nError = DecryptVerifyBuffer(UIGetInstance(), hwndMain, 
							_pgpContext, _tlsContext,
							szName, szFile, szInput, lLength,
							FALSE, FALSE, FALSE, (void **) &szOutput,
							&nOutLength, &FYEO);
				
				if (IsntPGPError(nError))
				{
					if ((nOutLength > 0) && (szOutput != NULL))
					{
						/*UINT nIndex;
						BOOL bHiASCII = FALSE;
						
						for (nIndex=0; nIndex<nOutLength; nIndex++)
						{
							if (szOutput[nIndex] & 0x80)
							{
								bHiASCII = TRUE;
								break;
							}
						}
						
						if (bHiASCII)
							UIDisplayStringID(hwndMain, IDS_W_HIASCII);*/

						if((FYEO)||(PGPscGetSecureViewerPref(_pgpContext)))
						{
							PGPscTempestViewer((void *)_pgpContext,hwndMain,
								szOutput,nOutLength,FYEO);
						}
						else
						{
							if ((nOutLength != (UINT) lLength) &&
								(nOutLength != (UINT) (lLength+1)))
							{
								SetRichEditContents(_hwndMessage, szOutput, 
									FALSE, FALSE);
							}
						}
						
						PGPFreeData(szOutput);
					}
					else
						UIDisplayStringID(_hwndMessage, IDS_E_NOPGP);
				}
				
				HeapFree(GetProcessHeap (), 0, szInput);
			}
		}

		return S_OK;
	}

	if (cmdid == _cmdidPgpKeys)
	{
		char szPath[MAX_PATH];

		PGPclGetPath(kPGPclPGPkeysExeFile, szPath, MAX_PATH-1);

		// run it...
		PGPclExecute(szPath, SW_SHOW);
		return S_OK;
	}

	if (cmdid == _cmdidPrefs)
	{
		PGPclPreferences(_pgpContext, hwndMain, kPGPclEmailPrefs, NULL);
		return S_OK;
	}

	if (cmdid == _cmdidHelp)
	{
		PGPclHtmlHelp (hwndMain, kPGPclHelpContentsAndIndex, 0, 0,
				(char*)kPGPclMailHelpFile, NULL);
		return S_OK;
	}

	if (cmdid == _cmdidAbout)
	{
		PGPclHelpAbout(_pgpContext, hwndMain, NULL, NULL, NULL);
		return S_OK;
	}

	return S_FALSE;
}


BOOL CPGPexch::EncryptSignMessageWindow(HWND hwndMain,
										IExchExtCallback *pmecb,
										BOOL bEncrypt,
										BOOL bSign,
										PGPclRecipientDialogStruct *prds)
{
	char *szInput;
	char *szOutput = NULL;
	long lLength;
	UINT nOutLength;
	BOOL bSelectedText = FALSE;
	BOOL bReturn = FALSE;
	CHARRANGE chRange = {0,0};
	PGPOptionListRef signOptions = NULL;
	char szName[256];
	char szFile[256];
	PGPError nError = kPGPError_NoErr;

	if ((prds == NULL) && bEncrypt)
		return FALSE;

	UIGetString(szName, sizeof(szName), IDS_LOGNAME);
	UIGetString(szFile, sizeof(szFile), IDS_DLL);

	if (bEncrypt)
		nError = GetRecipients(pmecb, _pgpContext, _tlsContext, prds, FALSE);
			
	if (IsPGPError(nError))
	{
		if (nError != kPGPError_UserAbort)
			UIDisplayErrorCode(__FILE__, __LINE__, NULL, nError, FALSE);
		return FALSE;
	}

	// Determine if the user selected a particular piece of text
	SendMessage(_hwndMessage, EM_EXGETSEL, 
		(WPARAM)0, (LPARAM) &chRange);

	bSelectedText = chRange.cpMax - chRange.cpMin;
			
	szInput = GetRichEditContents(_hwndMessage, &lLength, 
									FALSE, bSelectedText);
	if (!szInput)
		return FALSE;

	lLength = strlen(szInput);
	nError = EncryptSignBuffer(UIGetInstance(), hwndMain, _pgpContext, 
				_tlsContext, szName, szFile, szInput, 
				lLength, prds, NULL, &signOptions, (void **) &szOutput, 
				&nOutLength, bEncrypt, bSign, FALSE, FALSE, 
				m_pExtMdlessCallBack, m_pemOurMdlessCallBack);

	if (IsntPGPError(nError))
	{
		if (bSelectedText && nOutLength)
		{
			char *szTemp;

			szTemp = (char *) PGPNewData(_memoryMgr, nOutLength+3, 0);
			strcpy(&szTemp[2], szOutput);
			szTemp[0] = '\r';
			szTemp[1] = '\n';
			PGPFreeData(szOutput);
			szOutput = szTemp;
			nOutLength += 2;
		}

		SetRichEditContents(_hwndMessage, szOutput, FALSE, 
			bSelectedText);
		PGPFreeData(szOutput);
		bReturn = TRUE;
	}

	if (signOptions != NULL)
	{
		PGPFreeOptionList(signOptions);
		signOptions = NULL;
	}

	memset(szInput, 0, lLength);
	HeapFree(GetProcessHeap (), 0, szInput);

	return bReturn;
}


HWND FindMessageWindow(IExchExtCallback* pmecb, FindStruct *fs)
{
	HWND hwndMain;
	HWND hwndSearch = NULL;
	HWND hwndFound = NULL;
	HWND hwndLast = NULL;

	fs->hwndFound = NULL;
	fs->yMax = 0;
	fs->bInternetExplorer = FALSE;
	fs->bMicrosoftWord = FALSE;
	fs->bListView = FALSE;

	pmecb->GetWindow(&hwndMain);
	EnumChildWindows(hwndMain, (WNDENUMPROC) ReportChildren, 
					(LPARAM) fs);

	hwndFound = fs->hwndFound;
	return hwndFound;
}


BOOL CALLBACK ReportChildren(HWND hwnd, LPARAM lParam)
{
	char szClassName[200];
	char szOldEditClass[256];
	char szNewEditClass[256];
	char szXPEditClass[256];
	char szIEClass[256];
	char szWordClass[256];
	char szListViewClass[256];
	FindStruct *pfsRichEdit;
	RECT rc;

	UIGetString(szOldEditClass, sizeof(szOldEditClass), IDS_OLDEDITCTRLCLASS);
	UIGetString(szNewEditClass, sizeof(szNewEditClass), IDS_NEWEDITCTRLCLASS);
	UIGetString(szXPEditClass, sizeof(szXPEditClass), IDS_XPEDITCTRLCLASS);
	UIGetString(szIEClass, sizeof(szIEClass), IDS_IE40CLASS);
	UIGetString(szWordClass, sizeof(szWordClass), IDS_WORDCLASS);
	UIGetString(szListViewClass, sizeof(szListViewClass), IDS_LISTVIEWCLASS);

	pfsRichEdit = (FindStruct *) lParam;
	GetClassName(hwnd, szClassName, 199);

	if (!strcmp(szClassName, szOldEditClass) || 
		!strcmp(szClassName, szNewEditClass) ||
		!strcmp(szClassName, szXPEditClass))
	{
		GetWindowRect(hwnd, &rc);
		if (rc.bottom > pfsRichEdit->yMax)
		{
			pfsRichEdit->yMax = rc.bottom;
			pfsRichEdit->hwndFound = hwnd;
		}
	}

	if (!strcmp(szClassName, szIEClass))
	{
		pfsRichEdit->bInternetExplorer = TRUE;
		pfsRichEdit->hwndFound = hwnd;
	}

	if (!strcmp(szClassName, szWordClass))
		pfsRichEdit->bMicrosoftWord = TRUE;

	if (!strcmp(szClassName, szListViewClass))
		pfsRichEdit->bListView = TRUE;

	return TRUE;
}


BOOL ByDefaultConvert()
{
	PGPBoolean	bConvert	= FALSE;
	PGPPrefRef	prefRef		= NULL;
	PGPError	err;

	err = PGPclPeekClientLibPrefRefs(&prefRef, NULL);
	if (IsPGPError(err))
	{
		PGPclErrorBox(NULL, err);
		return FALSE;
	}

	PGPGetPrefBoolean(prefRef, kPGPPrefWin32OutlookConvertMessages, &bConvert);

	return (BOOL) bConvert;
}


void SetConvertPref(PGPBoolean bConvert)
{
	PGPPrefRef	prefRef		= NULL;
	PGPError	err;

	err = PGPclPeekClientLibPrefRefs(&prefRef, NULL);
	if (IsPGPError(err))
	{
		PGPclErrorBox(NULL, err);
		return;
	}

	PGPSetPrefBoolean(prefRef, kPGPPrefWin32OutlookConvertMessages, bConvert);
	PGPclFlushClientLibPrefs(prefRef, NULL);

	return;
}

HRESULT IsSelectionRelevant(IExchExtCallback* peecbExchCallBack, BOOL &bRelevant)
{
	HRESULT hrRet = S_OK;
	ULONG ulSelItemCount = 0;//selection item count
	ULONG ulEntryIdLen = 0, ulEntryIdType = 0;
	LPENTRYID lpEntryIdOfSel = NULL;
	TCHAR tszMsgClass[100]={0};
	ULONG ulMsgFlags = 0;//PR_MESSAGE_FLAGS

	//check parameters
	pgpAssert(NULL != peecbExchCallBack);
	if(NULL == peecbExchCallBack)
		return E_INVALIDARG;//invalid arguments passed

	//get the current selection count
	hrRet = peecbExchCallBack->GetSelectionCount(&ulSelItemCount);
	pgpAssert(SUCCEEDED(hrRet));
	if(FAILED(hrRet))
		return hrRet;

	//now we have got the selection count. since we do not support bulk operations
	//in this plugin we cannot be relevant when multiple objects are in current
	//selection. same goes if nothing is selected at this point
	if((0 == ulSelItemCount) || (1 < ulSelItemCount))
	{
		bRelevant = FALSE; //we are not relevant
		return S_OK;
	}

	//get the entry identifier of the current selection
	hrRet = peecbExchCallBack->GetSelectionItem(0, &ulEntryIdLen, &lpEntryIdOfSel,
				&ulEntryIdType, tszMsgClass, sizeof(tszMsgClass)-sizeof(TCHAR), 
				&ulMsgFlags, 0);
	pgpAssert(S_OK == hrRet);
	if(FAILED(hrRet) || (S_FALSE == hrRet))
		return E_FAIL;

	//control comes here only if there is a single selection AND we have found the
	//details of the selection
	pgpAssert(ulEntryIdLen > 0);
	pgpAssert(NULL != lpEntryIdOfSel);
	pgpAssert(0 != ulEntryIdType);
	//pgpAssert(NULL != tszMsgClass[0]); //does not return message class

	//we are relevant only if the selection is a message
	bRelevant = (MAPI_MESSAGE == ulEntryIdType);

	if(NULL != lpEntryIdOfSel)
	{
		hrRet = MAPIFreeBuffer(lpEntryIdOfSel);
		pgpAssert(S_OK == hrRet);
		lpEntryIdOfSel = NULL;
	}

	//if we are here we must have succeeded
	return S_OK;//success return
}

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
