/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: CPGPexchMessageEvents.cpp,v 1.62 2002/11/22 23:35:13 sdas Exp $
____________________________________________________________________________*/
#include "CPGPexch.h"
#include <mapiutil.h>
#include "Globals.h"
#include "Recipients.h"
#include "Utils.h"
#include "resource.h"

#include "pgpUnicodeWin32.h"
#include "EncryptSign.h"
#include "DecryptVerify.h"
#include "PGPsc.h"
#include "Prefs.h"
#include "pgpClientPrefs.h"
#include "pgpDebug.h"

STDMETHODIMP CPGPexch::OnRead(IExchExtCallback* peecb)
{
	HWND hwnd;

	peecb->GetWindow(&hwnd);
	if (!hwnd)
		hwnd = GetTopWindow(NULL);

	switch (_context)
	{
	case EECONTEXT_SENDNOTEMESSAGE:
		{
			BOOL fAddButtons = TRUE;
			int nNumButtons;
			int nIndex;
			TBBUTTON tbbNew[5];
			TBBUTTON tbbCheck;
			
			// Check to make sure the buttons aren't already on the toolbar

			nNumButtons = SendMessage(_hwndSendToolbar, TB_BUTTONCOUNT, 0, 0);
			for (nIndex=0; nIndex<nNumButtons; nIndex++)
			{
				SendMessage(_hwndSendToolbar, TB_GETBUTTON, (WPARAM) nIndex, 
							(LPARAM) &tbbCheck);

				// If a button exists with the same bitmap ID and command ID
				// as the Encrypt button, odds are good that the button is
				// already on the toolbar
				if ((tbbCheck.iBitmap == (int)_itbmEncrypt) && 
					(tbbCheck.idCommand == (int)_cmdidEncrypt))
				{
					fAddButtons = FALSE;
					nIndex = nNumButtons;
				}
			}

			if (fAddButtons)
			{
				tbbNew[0].iBitmap = 0;
				tbbNew[0].idCommand = 0;
				tbbNew[0].fsState = 0;
				tbbNew[0].fsStyle = TBSTYLE_SEP;
				tbbNew[0].dwData = 0;
				tbbNew[0].iString = -1;
			
				tbbNew[1].iBitmap = _itbmEncrypt;
				tbbNew[1].idCommand = _cmdidEncrypt;
				tbbNew[1].fsState = TBSTATE_ENABLED;
				tbbNew[1].fsStyle = TBSTYLE_CHECK;
				tbbNew[1].dwData = 0;
				tbbNew[1].iString = -1;
			
				tbbNew[2].iBitmap = _itbmSign;
				tbbNew[2].idCommand = _cmdidSign;
				tbbNew[2].fsState = TBSTATE_ENABLED;
				tbbNew[2].fsStyle = TBSTYLE_CHECK;
				tbbNew[2].dwData = 0;
				tbbNew[2].iString = -1;

				tbbNew[3].iBitmap = 0;
				tbbNew[3].idCommand = 0;
				tbbNew[3].fsState = 0;
				tbbNew[3].fsStyle = TBSTYLE_SEP;
				tbbNew[3].dwData = 0;
				tbbNew[3].iString = -1;
			
				tbbNew[4].iBitmap = _itbmPgpKeys;
				tbbNew[4].idCommand = _cmdidPgpKeys;
				tbbNew[4].fsState = TBSTATE_ENABLED;
				tbbNew[4].fsStyle = TBSTYLE_BUTTON;
				tbbNew[4].dwData = 0;
				tbbNew[4].iString = -1;
			
				SendMessage(_hwndSendToolbar, TB_ADDBUTTONS, 5, 
					(LPARAM) tbbNew);
			}
		}
		break;

    case EECONTEXT_SENDPOSTMESSAGE:
		break;
    
	case EECONTEXT_READNOTEMESSAGE:
		{
			BOOL fAddButtons = TRUE;
			UINT nNumButtons;
			UINT nIndex;
			TBBUTTON tbbNew[5];
			TBBUTTON tbbCheck;

			// Check to make sure the buttons aren't already on the toolbar

			nNumButtons = SendMessage(_hwndReadToolbar, TB_BUTTONCOUNT, 0, 0);
			for (nIndex=0; nIndex<nNumButtons; nIndex++)
			{
				SendMessage(_hwndReadToolbar, TB_GETBUTTON, 
							(WPARAM) nIndex, (LPARAM) &tbbCheck);

				// If a button exists with the same bitmap ID and command ID
				// as the Decrypt button, odds are good that the button is
				// already on the toolbar
				if ((tbbCheck.iBitmap == (int)_itbmDecrypt) && 
					(tbbCheck.idCommand == (int)_cmdidDecrypt))
				{
					fAddButtons = FALSE;
					nIndex = nNumButtons;
				}
			}

			if (fAddButtons)
			{
				tbbNew[0].iBitmap = 0;
				tbbNew[0].idCommand = 0;
				tbbNew[0].fsState = 0;
				tbbNew[0].fsStyle = TBSTYLE_SEP;
				tbbNew[0].dwData = 0;
				tbbNew[0].iString = -1;
			
				tbbNew[1].iBitmap = _itbmDecrypt;
				tbbNew[1].idCommand = _cmdidDecrypt;
				tbbNew[1].fsState = TBSTATE_ENABLED;
				tbbNew[1].fsStyle = TBSTYLE_BUTTON;
				tbbNew[1].dwData = 0;
				tbbNew[1].iString = -1;
			
				tbbNew[2].iBitmap = 0;
				tbbNew[2].idCommand = 0;
				tbbNew[2].fsState = 0;
				tbbNew[2].fsStyle = TBSTYLE_SEP;
				tbbNew[2].dwData = 0;
				tbbNew[2].iString = -1;
			
				tbbNew[3].iBitmap = _itbmPgpKeys;
				tbbNew[3].idCommand = _cmdidPgpKeys;
				tbbNew[3].fsState = TBSTATE_ENABLED;
				tbbNew[3].fsStyle = TBSTYLE_BUTTON;
				tbbNew[3].dwData = 0;
				tbbNew[3].iString = -1;
			
				SendMessage(_hwndReadToolbar, TB_ADDBUTTONS, 4, 
					(LPARAM) tbbNew);
			}

			IMAPISession* psess;

			HRESULT hr = peecb->GetSession(&psess, NULL);
			if (FAILED(hr))
			{
				UIDisplayStringID(hwnd, IDS_E_NOSESSION);
				return S_FALSE;
			}
			
			IMsgStore *pMsgStore = 0;
			IMessage *pmsg = 0;
			int nReturn;

			hr = peecb->GetObject(&pMsgStore, (IMAPIProp**)&pmsg);
			if (FAILED(hr))
			{
				psess->Release();
				return S_FALSE;
			}
			
			nReturn = DecryptVerifyMessage(hwnd, pmsg, pMsgStore);

			pmsg->Release();
			if (pMsgStore != NULL)
				pMsgStore->Release();
			psess->Release();
			return nReturn;
		}
		break;

	case EECONTEXT_READPOSTMESSAGE:
		break;

	default:
		return S_FALSE;
	}

	return S_FALSE;
}


STDMETHODIMP CPGPexch::OnReadComplete(IExchExtCallback* peecb, ULONG ulFlags)
{
	return S_FALSE;
}


STDMETHODIMP CPGPexch::OnWrite(IExchExtCallback* pecb)
{
	return S_FALSE;
}


STDMETHODIMP CPGPexch::OnWriteComplete(IExchExtCallback* pecb, ULONG ulFlags)
{
	if (_context != EECONTEXT_SENDNOTEMESSAGE)
		return S_FALSE;

	if (ulFlags == EEME_COMPLETE_FAILED) // Nothing to unwind
		return S_FALSE;

	if (!_fInSubmitState)	// This is not a submission.
		return S_FALSE;

	if (_bEncrypt || _bSign)
	{
		IMessage *pmsg = NULL;
		LPMAPITABLE ptableAttach = NULL;
		SizedSPropTagArray(1, tagaTable) = {1, {PR_ATTACH_NUM}};
		SRowSet *prAttach = NULL;
		PGPclRecipientDialogStruct *prds = NULL;
		PGPOptionListRef signOptions = NULL;
		HWND hwnd = NULL;
		BOOL bHasAttachments = FALSE;
		PGPError nError = kPGPError_NoErr;

		HRESULT hrRet = pecb->GetWindow(&hwnd);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != hwnd));
		if (NULL == hwnd)
		{
			hwnd = GetTopWindow(NULL);
			pgpAssert(NULL != hwnd);
		}
		
		hrRet = pecb->GetObject(NULL, (IMAPIProp**)&pmsg);
		if (FAILED(hrRet) || (NULL == pmsg))
		{
			UIDisplayStringID(hwnd, IDS_E_NOMESSAGE);
			hrRet = E_ABORT;
			goto cleanup;
		}

		prds = (PGPclRecipientDialogStruct *) 
				calloc(sizeof(PGPclRecipientDialogStruct), 1);
		pgpAssert(NULL != prds);
		if(NULL == prds)
		{
			hrRet = E_ABORT;//out of memory
			goto cleanup;
		}

		nError = PGPclOpenDefaultKeyrings(_pgpContext, 
					kPGPOpenKeyDBFileOptions_Mutable, &(prds->keydbOriginal));

		if (IsPGPError(nError))
			nError = PGPclOpenDefaultKeyrings(_pgpContext, 
						kPGPOpenKeyDBFileOptions_None, &(prds->keydbOriginal));

		if (IsPGPError(nError))
		{
			UIDisplayErrorCode(__FILE__, __LINE__, NULL, nError, TRUE);
			hrRet = E_ABORT;
			goto cleanup;
		}

		//try to see if there are any attachments in this message
		hrRet = pmsg->GetAttachmentTable(0, &ptableAttach);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != ptableAttach));
		if(FAILED(hrRet) || (NULL == ptableAttach))
		{
			hrRet = E_ABORT;
			goto cleanup;
		}

		hrRet = HrQueryAllRows(ptableAttach, (SPropTagArray *)&tagaTable, 
			NULL, NULL, 0, &prAttach);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != prAttach));
		if(FAILED(hrRet) || (NULL == prAttach))
		{
			hrRet = E_ABORT;
			goto cleanup;
		}
		if (prAttach->cRows)
			bHasAttachments = TRUE;


		if (_bEncrypt)
		{
			nError = GetRecipients(pecb, _pgpContext, _tlsContext, prds,
						bHasAttachments);
			pgpAssert(IsntPGPError(nError));
			if (IsPGPError(nError))
			{
				if (nError != kPGPError_UserAbort)
					UIDisplayErrorCode(__FILE__, __LINE__, NULL, 
						nError, FALSE);
				hrRet = E_ABORT;
				goto cleanup;
			}
		}


		nError = EncryptSignMessage(hwnd, pmsg, prds, &signOptions);
		if (IsPGPError(nError))
		{
			hrRet = E_ABORT;
			goto cleanup;
		}
		
		hrRet = pmsg->SaveChanges(KEEP_OPEN_READWRITE);
		pgpAssert(SUCCEEDED(hrRet));
		if(FAILED(hrRet))
		{
			hrRet = E_ABORT;
			goto cleanup;
		}

		cleanup:
		if(NULL != prAttach)
		{
			FreeProws(prAttach);
			prAttach = NULL;
		}

		if(NULL != ptableAttach)
		{
			ptableAttach->Release();
			ptableAttach = NULL;
		}

		if (signOptions != NULL)
		{
			PGPFreeOptionList(signOptions);
			signOptions = NULL;
		}

		if(NULL != prds)
		{
			FreeRecipients(prds);

			if (NULL != prds->keydbOriginal)
			{
				PGPFreeKeyDB(prds->keydbOriginal);
				prds->keydbOriginal = NULL;
			}

			free(prds);
			prds = NULL;
		}

		if(NULL != pmsg)
		{
			pmsg->Release();
			pmsg = NULL;
		}
		
		return SUCCEEDED(hrRet)?S_FALSE:hrRet;//return S_FALSE in case of success
											  //the error hresult otherwise
	}

	return S_FALSE;
}


STDMETHODIMP CPGPexch::OnCheckNames(IExchExtCallback*)
{
	return S_FALSE;
}


STDMETHODIMP CPGPexch::OnCheckNamesComplete(IExchExtCallback*, ULONG)
{
	return S_FALSE;
}


STDMETHODIMP CPGPexch::OnSubmit(IExchExtCallback*)
{
	_fInSubmitState = TRUE;
	return S_FALSE;
}


STDMETHODIMP_(VOID) CPGPexch::OnSubmitComplete(IExchExtCallback*, ULONG)
{
	_fInSubmitState = FALSE;
	return;
}


int CPGPexch::EncryptSignMessage(HWND hwnd,
								 IMessage *pmsg,
								 PGPclRecipientDialogStruct *prds,
								 PGPOptionListRef *pSignOptions)
{
	IStream *pstrmBody = NULL;
	IStream *pstrmRTF = NULL;
	IStream *pstrmRTFInput = NULL;
	STATSTG StreamStats = {0};
	DWORD dwInSize = 0;
	DWORD dwRTFSize = 0;
	DWORD dwTextSize = 0;
	UINT nOutSize = 0;
	UINT nFormatOutSize = 0;
	char *pInput = NULL;
	char *pTextInput = NULL;
	char *pRTFInput = NULL;
	char *pOutput = NULL;
	char *pFormatOutput = NULL;
	LPMAPITABLE ptableAttach = NULL;
	SizedSPropTagArray(1, tagaTable) = {1, {PR_ATTACH_NUM}};
	SRowSet *prAttach = NULL;
	SizedSPropTagArray(1, tagaCodePage) = {1, {PR_INTERNET_CPID}};
	SPropValue *pCPID = NULL;
	ULONG ulNumVals = 0;
	PGPError nError = kPGPError_NoErr;
	char szName[256] = {0};
	char szFile[256] = {0};
	char *pHeader = NULL;
	char *pOutHeader = NULL;
	char *pConverted = NULL;
	char *pOutFooter = "\n</PRE></BODY></HTML>";
	HGLOBAL hRTFInput = NULL;
	ULARGE_INTEGER ulSize = {0};
	ULARGE_INTEGER ulMaxSize = {INT_MAX, 0};
	BOOL bGotHTML = FALSE;
	BOOL bGotRTF = FALSE;
	BOOL bConvert = FALSE;
	BOOL bConvertHTML = FALSE;
	BOOL bConvertRTF = FALSE;
	//BOOL bNeedUTF8 = FALSE;
	HRESULT hrRet = S_OK;
	DWORD dwOrigOptions = 0;
	UINT nIndex = 0;

	UIGetString(szName, sizeof(szName), IDS_LOGNAME);
	UIGetString(szFile, sizeof(szFile), IDS_DLL);

	pgpAssert(NULL == pstrmBody);
	hrRet = pmsg->OpenProperty(PR_BODY, &IID_IStream, STGM_READ, 0, 
		(IUnknown**)&pstrmBody);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));
	if (FAILED(hrRet))
	{
		pgpAssert(NULL == pstrmBody);
		hrRet = pmsg->OpenProperty(PR_BODY, &IID_IStream, STGM_READ, 
				MAPI_CREATE | MAPI_MODIFY, (IUnknown**)&pstrmBody);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));
	}

	if (FAILED(hrRet) || (NULL == pstrmBody))
	{
		UIDisplayStringID(hwnd, IDS_E_NOBODY);
		nError = kPGPError_ItemNotFound;
		goto cleanup;
	}
	
	hrRet = pstrmBody->Stat(&StreamStats, STATFLAG_NONAME);
	pgpAssert(SUCCEEDED(hrRet));
	if (StreamStats.cbSize.QuadPart == 0)
	{
		pstrmBody->Release();
		pstrmBody = NULL;

		pTextInput = (char *) calloc(3, sizeof(char));
		if(NULL == pTextInput)
		{
			UIDisplayStringID(hwnd, IDS_E_NOMEMORY);
			nError = kPGPError_OutOfMemory;
			goto cleanup;
		}

		strcpy(pTextInput, "\r\n");
		dwTextSize = strlen(pTextInput);
	}
	else
	{
		dwTextSize = StreamStats.cbSize.LowPart;
		pTextInput = (char *) calloc(dwTextSize+1, sizeof(char));
		if (NULL == pTextInput)
		{
			UIDisplayStringID(hwnd, IDS_E_NOMEMORY);
			nError = kPGPError_OutOfMemory;
			goto cleanup;
			//pstrmBody->Release();
			//return kPGPError_OutOfMemory;
		}
		
		//read the email body in plain text
		hrRet = pstrmBody->Read(pTextInput, dwTextSize, &dwTextSize);
		pgpAssert(SUCCEEDED(hrRet));
		if(FAILED(hrRet))
		{
			UIDisplayStringID(hwnd, IDS_E_NOBODY);
			nError = kPGPError_ItemNotFound;
			goto cleanup;
		}

		pstrmBody->Release();
		pstrmBody = NULL;
	}

	pTextInput[dwTextSize] = NULL;

	/*if (_bEncrypt)
	{
		for (nIndex=0; nIndex < dwTextSize; nIndex++)
		{
			if (pTextInput[nIndex] & 0x80)
			{
				bNeedUTF8 = TRUE;
				break;
			}
		}
	}

	if (bNeedUTF8)
	{
		hrRet = pmsg->GetProps((SPropTagArray *)&tagaCodePage, 0, &ulNumVals, 
				&pCPID);

		pgpAssert(SUCCEEDED(hrRet) && (NULL != pCPID));
		if (SUCCEEDED(hrRet) && (NULL != pCPID))
		{
			char *pNewText = NULL;
			PGPUInt32 nNewSize = 0;
			
			pNewText = (char *) calloc(1, (dwTextSize+1)*4);
			if(NULL == pNewText)
			{
				UIDisplayStringID(hwnd, IDS_E_NOMEMORY);
				nError = kPGPError_OutOfMemory;
				goto cleanup;
			}

			pgpLocalStringToUTF8(0, pCPID[0].Value.l, pTextInput, dwTextSize, 
				pNewText, (dwTextSize+1)*4, &nNewSize);

			//free up the existing buffer
			free(pTextInput);
			pTextInput = NULL;

			//remember the new size and the buffer that has the UTF8 buffer 
			//instead of the original buffer
			pgpAssert(NULL == pTextInput);
			pTextInput = pNewText;
			dwTextSize = nNewSize;

			pCPID[0].Value.l = CP_UTF8;
			hrRet = pmsg->SetProps(1, pCPID, NULL);
			pgpAssert(SUCCEEDED(hrRet));
		}
	}*/

	//get the message attachment table
	pgpAssert(NULL == ptableAttach);
	hrRet = pmsg->GetAttachmentTable(0, &ptableAttach);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != ptableAttach));
	if(SUCCEEDED(hrRet) && (NULL != ptableAttach))
	{
		//get the attachment rows
		pgpAssert(NULL == prAttach);
		hrRet = HrQueryAllRows(ptableAttach, (SPropTagArray *)&tagaTable, 
						NULL, NULL, 0, &prAttach);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != prAttach));
	}

	if ((prds->dwOptions & kPGPclSecureViewer) == kPGPclSecureViewer)
		bConvert = TRUE;

	if (_bConvert == TRUE)
		bConvert = TRUE;

	//get the HTML text in the mail 
	pgpAssert(NULL == pstrmBody);
	hrRet = pmsg->OpenProperty(PR_BODY_HTML, &IID_IStream, STGM_READ, 
			0, (IUnknown**)&pstrmBody);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));
	if (FAILED(hrRet) || (NULL == pstrmBody))
		goto NoBodyHTML;
	
	hrRet = pstrmBody->Stat(&StreamStats, STATFLAG_NONAME);
	pgpAssert(SUCCEEDED(hrRet));

	dwInSize = StreamStats.cbSize.LowPart;
	if (dwInSize < 3)
		goto NoBodyHTML;
	
	if (bConvert)
	{
		bConvertHTML = TRUE;
		goto NoBodyHTML;
	}

	pgpAssert(NULL == pInput);
	pInput = (char *) calloc(dwInSize+1, sizeof(char));
	if (NULL == pInput)
	{
		UIDisplayStringID(hwnd, IDS_E_NOMEMORY);
		nError = kPGPError_OutOfMemory;
		goto cleanup;
	}

	hrRet = pstrmBody->Read(pInput, dwInSize, &dwInSize);
	pgpAssert(SUCCEEDED(hrRet));

	pInput[dwInSize] = NULL;

	//office versions prior to office XP
	pConverted = strstr(pInput, "<!-- Converted from text/rtf format -->");

	if(NULL == pConverted)//if not found search for office XP's little hint in there
		pConverted = strstr(pInput, "<!-- Converted from text/plain format -->");

	if (NULL != pConverted)
	{
		SPropTagArray taga={0};
		
		taga.cValues = 1;
		taga.aulPropTag[0] = PR_BODY_HTML;
		hrRet = pmsg->DeleteProps(&taga, NULL);
		pgpAssert(SUCCEEDED(hrRet));

		hrRet = pmsg->SaveChanges(KEEP_OPEN_READWRITE);
		pgpAssert(SUCCEEDED(hrRet));

		pgpAssert(NULL != pInput);
		free(pInput);
		pInput = NULL;

		dwInSize = 0;
		goto NoBodyHTML;
	}

	if (!UIWarnYesNo(hwnd, IDS_Q_HTMLENCRYPT, kPGPPrefWin32OutlookHTMLEncrypt, TRUE))
	{
		nError = kPGPError_UserAbort;//user chose not to go further with html manipulations
		goto cleanup;
	}

	pHeader = strstr(pInput, "<BODY");
	pgpAssert(NULL != pHeader);
	if(NULL == pHeader)
	{
		UIDisplayStringID(hwnd, IDS_E_MALFORMEDHTMLFOUND);
		nError = kPGPError_UnknownError;
		goto cleanup;
	}

	pgpAssert(NULL == pOutHeader);
	pOutHeader = (char *) calloc(sizeof(char), pHeader - pInput + 20);
	if(NULL == pOutHeader)
	{
		UIDisplayStringID(hwnd, IDS_E_NOMEMORY);
		nError = kPGPError_OutOfMemory;
		goto cleanup;
	}

	strncpy(pOutHeader, pInput, pHeader - pInput);
	strcat(pOutHeader, "<BODY><PRE>\n");

	bGotHTML = TRUE;
	dwOrigOptions = prds->dwOptions;
	prds->dwOptions &= ~kPGPclASCIIArmor;
	nError = EncryptSignBuffer(UIGetInstance(), hwnd, _pgpContext, 
				_tlsContext, szName, szFile, pInput, 
				dwInSize, prds, NULL, pSignOptions, (VOID ** )&pFormatOutput, 
				&nFormatOutSize, _bEncrypt, _bSign, FALSE, TRUE, 
				m_pExtMdlessCallBack, m_pemOurMdlessCallBack);
	prds->dwOptions = dwOrigOptions;

	free(pInput);
	pInput = NULL;

	if (IsPGPError(nError))
		goto cleanup;
	else
		goto NoBodyRTF;

NoBodyHTML:
	//we reuse the pointer here
	if(NULL != pstrmBody)
	{
		pstrmBody->Release();
		pstrmBody = NULL;
	}

	pgpAssert(NULL == pstrmBody);
	hrRet = pmsg->OpenProperty(PR_RTF_COMPRESSED, &IID_IStream, STGM_READ, 
			0, (IUnknown**)&pstrmBody);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));
	if (FAILED(hrRet) || (NULL == pstrmBody))
		goto NoBodyRTF;
		
	pgpAssert(NULL == pstrmRTF);
	hrRet = WrapCompressedRTFStream(pstrmBody, 0, &pstrmRTF);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmRTF));
	if(FAILED(hrRet) || (NULL == pstrmRTF))
	{
		UIDisplayStringID(hwnd, IDS_E_GENERICRTFTXTFAILURE);
		nError = kPGPError_UnknownError;
		goto cleanup;
	}

	pgpAssert(NULL == hRTFInput);
	hRTFInput = GlobalAlloc(GHND, 0);
	pgpAssert(NULL != hRTFInput);
	if(NULL == hRTFInput)
	{
		UIDisplayStringID(hwnd, IDS_E_NOMEMORY);
		nError = kPGPError_OutOfMemory;
		goto cleanup;
	}

	pgpAssert(NULL == pstrmRTFInput);
	hrRet = CreateStreamOnHGlobal(hRTFInput, TRUE, &pstrmRTFInput);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmRTFInput));
	if(FAILED(hrRet) || (NULL == pstrmRTFInput))
	{
		UIDisplayStringID(hwnd, IDS_E_GENERICFAILURE);
		nError = kPGPError_UnknownError;
		goto cleanup;
	}

	hrRet = pstrmRTF->CopyTo(pstrmRTFInput, ulMaxSize, NULL, &ulSize);
	pgpAssert(SUCCEEDED(hrRet));
	if(FAILED(hrRet))
	{
		UIDisplayStringID(hwnd, IDS_E_GENERICFAILURE);
		nError = kPGPError_UnknownError;
		goto cleanup;
	}

	hrRet = pstrmRTFInput->Commit(STGC_DEFAULT);
	pgpAssert(SUCCEEDED(hrRet));
	if(FAILED(hrRet))
	{
		UIDisplayStringID(hwnd, IDS_E_GENERICFAILURE);
		nError = kPGPError_UnknownError;
		goto cleanup;
	}

	dwRTFSize = ulSize.LowPart;

	if(NULL != pstrmRTF)
	{
		pstrmRTF->Release();
		pstrmRTF = NULL;
	}

	if(NULL != pstrmBody)
	{
		pstrmBody->Release();
		pstrmBody = NULL;
	}

	pRTFInput = (char *) GlobalLock(hRTFInput);
	pgpAssert(NULL != pRTFInput);
	if(NULL == pRTFInput)
	{
		UIDisplayStringID(hwnd, IDS_E_GENERICFAILURE);
		nError = kPGPError_UnknownError;
		goto cleanup;
	}
	
	pHeader = strstr(pRTFInput, "\\fromtext");
	if (pHeader || (dwRTFSize < 3))
		goto NoBodyRTF;

	if (bConvert)
	{
		BOOL bGotOLE = FALSE;
		bConvertRTF = TRUE;

		//check if there are any OLE attachments in this message
		hrRet = AreAnyOleAttachmentsPresent(pmsg, prAttach, bGotOLE);
		pgpAssert(SUCCEEDED(hrRet));
		if(SUCCEEDED(hrRet) && (TRUE == bGotOLE))
		{
			UIDisplayStringID(hwnd, IDS_E_CANTCONVERTOLE);
			nError = kPGPError_UserAbort;
			goto cleanup;		
		}

		goto NoBodyRTF;
	}

	if (!UIWarnYesNo(hwnd, IDS_Q_HTMLENCRYPT, kPGPPrefWin32OutlookHTMLEncrypt, TRUE))
	{
		nError = kPGPError_UserAbort;
		goto cleanup;
	}

	bGotRTF = TRUE;
	dwOrigOptions = prds->dwOptions;
	prds->dwOptions &= ~kPGPclASCIIArmor;
	nError = EncryptSignBuffer(UIGetInstance(), hwnd, _pgpContext, 
				_tlsContext, szName, szFile, pRTFInput, 
				dwRTFSize-1, prds, NULL, pSignOptions, (VOID ** )&pFormatOutput, 
				&nFormatOutSize, _bEncrypt, _bSign, FALSE, TRUE, 
				m_pExtMdlessCallBack, m_pemOurMdlessCallBack);
	prds->dwOptions = dwOrigOptions;
	if (IsPGPError(nError))
		goto cleanup;

	pHeader = strstr(pRTFInput, "\\fromhtml");
	if (NULL != pHeader)
	{
		pOutHeader = _strdup("{\\fromhtml1\n");
		pOutFooter = "\n}\0";
	}
	else
	{
		pOutHeader = _strdup("{\\fromtext\n");
		pOutFooter = "\n}\0";
	}


NoBodyRTF:	

	nError = EncryptSignBuffer(UIGetInstance(), hwnd, _pgpContext, 
				_tlsContext, szName, szFile, pTextInput, 
				dwTextSize, prds, NULL, pSignOptions, (VOID ** )&pOutput, 
				&nOutSize, _bEncrypt, _bSign, FALSE, FALSE, 
				m_pExtMdlessCallBack, m_pemOurMdlessCallBack);

	if (IsPGPError(nError))
		goto cleanup;

	if (bGotHTML || bConvertHTML)
	{

/*
		LARGE_INTEGER li = {0,0};
		ULARGE_INTEGER uli = {nOutSize + strlen(pOutHeader) + 
								strlen(pOutFooter),0};

		pstrmBody = NULL;
		pmsg->OpenProperty(PR_BODY_HTML, &IID_IStream, STGM_READWRITE, 
			MAPI_MODIFY, (IUnknown**)&pstrmBody);

		pstrmBody->Write(pOutHeader, strlen(pOutHeader), NULL);
		pstrmBody->Write(pOutput, nOutSize, NULL);
		pstrmBody->Write(pOutFooter, strlen(pOutFooter), NULL);
		pstrmBody->SetSize(uli);
		pstrmBody->Commit(STGC_DEFAULT);
		pstrmBody->Release();
*/

		SPropTagArray taga;
		
		taga.cValues = 1;
		taga.aulPropTag[0] = PR_BODY_HTML;
		pmsg->DeleteProps(&taga, NULL);
		pmsg->SaveChanges(KEEP_OPEN_READWRITE);
	}

	if (bGotRTF || bConvertRTF)
	{
		BOOL fPartied;

		//we reuse the pointer here again !!
		if(NULL != pstrmBody)
		{
			pstrmBody->Release();
			pstrmBody = NULL;
		}

		pgpAssert(NULL == pstrmBody);
		hrRet = pmsg->OpenProperty(PR_RTF_COMPRESSED, &IID_IStream, STGM_READWRITE, 
					MAPI_MODIFY, (IUnknown**)&pstrmBody);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));

		hrRet = WrapCompressedRTFStream(pstrmBody, MAPI_MODIFY, &pstrmRTF);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmRTF));

		if(NULL != pstrmRTF)
		{
			hrRet = NullifyStream(pstrmRTF);
			pgpAssert(SUCCEEDED(hrRet));

			pstrmRTF->Release();
			pstrmRTF = NULL;
		}

		if(NULL != pstrmBody)
		{
			pstrmBody->Release();
			pstrmBody = NULL;
		}

		hrRet = RTFSync(pmsg, RTF_SYNC_RTF_CHANGED, &fPartied);
		pgpAssert(SUCCEEDED(hrRet));
	}

	if (dwTextSize > 0)
	{
		BOOL fPartied;

		if(NULL != pstrmBody)
		{
			pstrmBody->Release();
			pstrmBody = NULL;
		}
		
		//get PR_BODY, create if it does not exist. On Office XP we seem
		//to be losing the mail body !!
		pgpAssert(NULL == pstrmBody);
		hrRet = pmsg->OpenProperty(PR_BODY, &IID_IStream, STGM_READWRITE, 
					MAPI_CREATE|MAPI_MODIFY, (IUnknown**)&pstrmBody);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));
		if(SUCCEEDED(hrRet) && (NULL != pstrmBody))
		{
			hrRet = NullifyStream(pstrmBody);
			pgpAssert(SUCCEEDED(hrRet));

			pstrmBody->Release();
			pstrmBody = NULL;

			hrRet = RTFSync(pmsg, RTF_SYNC_BODY_CHANGED, &fPartied);
			pgpAssert(S_OK == hrRet);

			hrRet = pmsg->OpenProperty(PR_BODY, &IID_IStream, STGM_READWRITE, 
				MAPI_MODIFY, (IUnknown**)&pstrmBody);
			pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));
		}

	}

	if (dwTextSize > 0)
	{
		LARGE_INTEGER li = {0,0};
		ULARGE_INTEGER uli = {nOutSize + prAttach->cRows + 3, 0};
		BOOL fPartied;
		UINT nLoop;

		pgpAssert(NULL != pstrmBody);
		if(NULL != pstrmBody)
		{
			hrRet = pstrmBody->Seek(li, STREAM_SEEK_SET, NULL);
			pgpAssert(SUCCEEDED(hrRet));

			hrRet = pstrmBody->Write(" \r\n", 3, NULL);
			pgpAssert(SUCCEEDED(hrRet));

			hrRet = pstrmBody->Write(pOutput, nOutSize, NULL);
			pgpAssert(SUCCEEDED(hrRet));

			for (nLoop=0; nLoop<(prAttach->cRows+3); nLoop++)
			{
				hrRet = pstrmBody->Write(" ", 1, NULL);
				pgpAssert(SUCCEEDED(hrRet));
			}

			hrRet = pstrmBody->SetSize(uli);
			pgpAssert(SUCCEEDED(hrRet));

			hrRet = pstrmBody->Commit(STGC_DEFAULT);
			pgpAssert(SUCCEEDED(hrRet));

			pstrmBody->Release();
			pstrmBody = NULL;

			hrRet = RTFSync(pmsg, RTF_SYNC_BODY_CHANGED, &fPartied);
			pgpAssert(SUCCEEDED(hrRet));

			nOutSize += 3;
		}
	}

	if (prAttach->cRows && IsntPGPError(nError))
		nError = EncryptSignAttachment(hwnd, pmsg, nOutSize,
					prAttach, prds, pSignOptions);

	if (IsPGPError(nError))
		goto cleanup;

	if (bGotHTML)
	{
		pgpAssert(FALSE == bGotRTF);//we should not have got RTF then

		ULONG ulAttachNum = 0;
		LPATTACH pAttach = NULL;
		IStream *pstrmAttach = NULL;
		SPropValue pVal[5] = {0};
		LARGE_INTEGER li = {0,0};
		ULARGE_INTEGER uli = {nFormatOutSize, 0};

		pVal[0].ulPropTag = PR_ATTACH_METHOD;
		pVal[0].Value.l = ATTACH_BY_VALUE;

		pVal[1].ulPropTag = PR_RENDERING_POSITION;
		pVal[1].Value.l = -1;
		
		pVal[2].ulPropTag = PR_ATTACH_FILENAME;
		pVal[2].Value.lpszA = "PGPexch.pgp";
		
		pVal[3].ulPropTag = PR_ATTACH_LONG_FILENAME;
		pVal[3].Value.lpszA = "PGPexch.htm.pgp";

		pVal[4].ulPropTag = PR_DISPLAY_NAME;
		pVal[4].Value.lpszA = "PGPexch.htm.pgp";

		pgpAssert(NULL == pAttach);
		hrRet = pmsg->CreateAttach(NULL, 0, &ulAttachNum, &pAttach);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != pAttach));

		if(SUCCEEDED(hrRet) && (NULL != pAttach))
		{
			hrRet = pAttach->SetProps(5, pVal, NULL);
			pgpAssert(SUCCEEDED(hrRet));

			pstrmAttach = NULL;
			hrRet = pAttach->OpenProperty(PR_ATTACH_DATA_BIN, &IID_IStream, 0, 
				MAPI_CREATE | MAPI_MODIFY, (LPUNKNOWN *) &pstrmAttach);
			pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmAttach));
			if(SUCCEEDED(hrRet) && (NULL != pstrmAttach))
			{
				hrRet = pstrmAttach->Seek(li, STREAM_SEEK_SET, NULL);
				pgpAssert(SUCCEEDED(hrRet));

				hrRet = pstrmAttach->Write(pFormatOutput, nFormatOutSize, NULL);
				pgpAssert(SUCCEEDED(hrRet));

				hrRet = pstrmAttach->SetSize(uli);
				pgpAssert(SUCCEEDED(hrRet));

				hrRet = pstrmAttach->Commit(STGC_DEFAULT);
				pgpAssert(SUCCEEDED(hrRet));

				pstrmAttach->Release();
				pstrmAttach = NULL;
			}

			hrRet = pAttach->SaveChanges(0);
			pgpAssert(SUCCEEDED(hrRet));
		}
	}

	if (bGotRTF)
	{
		pgpAssert(FALSE == bGotHTML);//we should not have got HTML then

		ULONG ulAttachNum = 0;
		LPATTACH pAttach = NULL;
		IStream *pstrmAttach = NULL;
		SPropValue pVal[5] = {0};
		LARGE_INTEGER li = {0,0};
		ULARGE_INTEGER uli = {nFormatOutSize, 0};

		pVal[0].ulPropTag = PR_ATTACH_METHOD;
		pVal[0].Value.l = ATTACH_BY_VALUE;

		pVal[1].ulPropTag = PR_RENDERING_POSITION;
		pVal[1].Value.l = -1;
		
		pVal[2].ulPropTag = PR_ATTACH_FILENAME;
		pVal[2].Value.lpszA = "PGPexch.pgp";

		pVal[3].ulPropTag = PR_ATTACH_LONG_FILENAME;
		pVal[3].Value.lpszA = "PGPexch.rtf.pgp";
		
		pVal[4].ulPropTag = PR_DISPLAY_NAME;
		pVal[4].Value.lpszA = "PGPexch.rtf.pgp";

		pgpAssert(NULL == pAttach);
		hrRet = pmsg->CreateAttach(NULL, 0, &ulAttachNum, &pAttach);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != pAttach));
		if(SUCCEEDED(hrRet) && (NULL != pAttach))
		{
			hrRet = pAttach->SetProps(5, pVal, NULL);
			pgpAssert(SUCCEEDED(hrRet));

			pgpAssert(NULL == pstrmAttach);
			hrRet = pAttach->OpenProperty(PR_ATTACH_DATA_BIN, &IID_IStream, 0, 
				MAPI_CREATE | MAPI_MODIFY, (LPUNKNOWN *) &pstrmAttach);
			pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmAttach));
			if(SUCCEEDED(hrRet) && (NULL != pstrmAttach))
			{
				hrRet = pstrmAttach->Seek(li, STREAM_SEEK_SET, NULL);
				pgpAssert(SUCCEEDED(hrRet));

				hrRet = pstrmAttach->Write(pFormatOutput, nFormatOutSize, NULL);
				pgpAssert(SUCCEEDED(hrRet));

				hrRet = pstrmAttach->SetSize(uli);
				pgpAssert(SUCCEEDED(hrRet));

				hrRet = pstrmAttach->Commit(STGC_DEFAULT);
				pgpAssert(SUCCEEDED(hrRet));

				pstrmAttach->Release();
				pstrmAttach = NULL;
			}

			hrRet = pAttach->SaveChanges(0);
			pgpAssert(SUCCEEDED(hrRet));
		}
	}

	cleanup:
	
	if(NULL != pCPID)
	{
		hrRet = MAPIFreeBuffer(pCPID);
		pgpAssert(SUCCEEDED(hrRet));
		pCPID = NULL;
	}
	if (NULL != pTextInput)
	{
		free(pTextInput);
		pTextInput = NULL;
	}
	if (NULL != pOutput)
	{
		PGPFreeData(pOutput);
		pOutput = NULL;
	}
	if (NULL != pFormatOutput)
	{
		PGPFreeData(pFormatOutput);
		pFormatOutput = NULL;
	}
	if (NULL != pOutHeader)
	{
		free(pOutHeader);
		pOutHeader = NULL;
	}
	if (NULL != pInput)
	{
		free(pInput);
		pInput = NULL;
	}

	if (NULL != pstrmBody)
	{
		pstrmBody->Release();
		pstrmBody = NULL;
	}

	if(NULL != prAttach)
	{
		FreeProws(prAttach);
		prAttach = NULL;
	}

	if (NULL != ptableAttach)
	{
		ptableAttach->Release();
		ptableAttach = NULL;
	}

	if (NULL != pstrmRTF)
	{
		pstrmRTF->Release();
		pstrmRTF = NULL;
	}
	
	if (NULL != pstrmRTFInput)
	{
		pstrmRTFInput->Release();
		pstrmRTFInput = NULL;
	}
	
	if(NULL != hRTFInput)
	{
		//NOTE: we make this handle GlobalFree'ed automatically upon
		//pstrmRTFInput release. we lock it once to get at the buffer
		//in the handle. so we need to unlock if we did that
		if(NULL != pRTFInput)
		{
			BOOL bRet = GlobalUnlock(hRTFInput);
			pgpAssert(0 != bRet);
		}

#ifdef _DEBUG
		//verify that the handle is invalid. no locks on this handle. 
		UINT uiRet = GlobalFlags(hRTFInput);
		pgpAssert(GMEM_INVALID_HANDLE == uiRet);
		if(GMEM_INVALID_HANDLE != uiRet)
			pgpAssert(0 == (uiRet&GMEM_LOCKCOUNT));
#endif
	}

	return nError;
}


int CPGPexch::DecryptVerifyMessage(HWND hwnd, IMessage *pmsg, 
								   IMsgStore *pMsgStore)
{
	IStream *pstrmBody = NULL;
	IStream *pstrmRTFComp = NULL;
	STATSTG StreamStats = {0};
	DWORD dwInSize = 0;
	UINT nOutSize = 0;
	char *pInput = NULL;
	char *pOutput = NULL;
	PGPError nError = kPGPError_NoErr;
	char szFile[256] = {0};
	char szName[256] = {0};
	char szRTFheader[] = "{\\fromhtml1\n";
	char szRTFfooter[] = "}\0";
	LPMAPITABLE ptableAttach = 0;
	SizedSPropTagArray(1, tagaTable) = {1, {PR_ATTACH_NUM}};
	SRowSet *prAttach = NULL;
	BOOL FYEO = FALSE;
	BOOL bUseSecureViewer = PGPscGetSecureViewerPref(_pgpContext);
	BOOL bNeedsBackup = TRUE;
	SPropValue val = {0};
	HRESULT hrRet = S_OK;
	SizedSPropTagArray(4, tagaAttach) = {
		4, {PR_ATTACH_METHOD, PR_ATTACH_FILENAME, PR_ATTACH_LONG_FILENAME, 
			PR_DISPLAY_NAME}};
	enum {ivalAttachMethod=0, ivalFilename, ivalLongFilename, ivalDisplayName};
	ULONG ulRow = 0;
	ULONG ulAttIndex = 0;
	SPropValue *pval = NULL;
	ULONG ulNumVals = 0;
	LPATTACH pAttach = 0;
	DWORD dwFormatInfoSize = 0;
	char *pFormatInfo = NULL;
	DWORD dwFormatInfoType = 0;
			
	UIGetString(szName, sizeof(szName), IDS_LOGNAME);
	UIGetString(szFile, sizeof(szFile), IDS_DLL);
	
	//make sure that this message has not been already decrypted
	/*hrRet = HrGetOneProp(pmsg, PR_PGPEXCH_DECRYPTED, &pval);
	pgpAssert(FAILED(hrRet) || ((NULL != pval) && (FALSE == pval->Value.b)));
	if(SUCCEEDED(hrRet) && (NULL != pval) && (TRUE == pval->Value.b))
	{
		UIDisplayStringID(hwnd, IDS_E_ALREADYDECRYPTED);
		goto cleanup;//we have already decrypted this message
	}

	//free up the returned buffer since we reuse this pointer later
	hrRet = MAPIFreeBuffer(pval);
	pgpAssert(S_OK == hrRet);
	pval = NULL;*/

	val.ulPropTag = PR_PGPEXCH_HTML;
	val.Value.b = FALSE;
	hrRet = pmsg->SetProps(1, &val, NULL);
	pgpAssert(SUCCEEDED(hrRet));
	
	//if there is already a message backup associated with this object
	if(NULL != m_pBackupMsg)
	{
		//release the reference
		m_pBackupMsg->BackupRelease();
		m_pBackupMsg = NULL;
	}

	//find if there is a backup of this message already
	pgpAssert(NULL == m_pBackupMsg);
	hrRet = CPGPMsgBackup::IsAlreadyBackedUp(pMsgStore,	pmsg, m_pBackupMsg);
	pgpAssert(SUCCEEDED(hrRet));
	if(SUCCEEDED(hrRet))
	{
		
		//we dont need to back this up if there is already a backup and 
		//we need to back up if there is no back up we could find
		bNeedsBackup = !(NULL != m_pBackupMsg);
		if(NULL != m_pBackupMsg)
		{
#ifdef _DEBUG
			ExchPluginTrace("CPGPexch(%#x) - DecryptVerifyMessage(found another backup at "
						"%#x made for CPGPexch(%#x))\n", this, m_pBackupMsg, 
						m_pBackupMsg->GetCreator());
#endif
			pgpAssert(!IsBadWritePtr(m_pBackupMsg, sizeof(CPGPMsgBackup)));

			m_pBackupMsg->BackupAddRef();//add ref count

			//get the format information from backup
			hrRet = m_pBackupMsg->GetFormatInfoIfAny((void *&)pFormatInfo, 
						dwFormatInfoSize, dwFormatInfoType);
			pgpAssert(SUCCEEDED(hrRet));
			
			//if we did not find any formatted info 
			if(SUCCEEDED(hrRet) && (0 == dwFormatInfoSize))
			{
				//this must be a plain text message. get the PR_BODY off of
				//the backup of message
				hrRet = m_pBackupMsg->GetMsgBody((void *&)pInput, dwInSize);
				pgpAssert(SUCCEEDED(hrRet));
			}

		}
	}

	//if we need to create backup object for this message
	if(TRUE == bNeedsBackup)
	{
		//let us read the body of the message then

		//read the message body into a buffer of appropriate length
		pgpAssert(NULL == pInput);
		pgpAssert(0 == dwInSize);
		hrRet = ReadMsgBodyToBuf(pmsg, (void *&)pInput, dwInSize);
		if(FAILED(hrRet))//could not get the body in buffer
			goto cleanup;
		
		//see if the body buffer has pgp headers. otherwise we are done !
		if (FALSE == DoesBufHavePGPHeaders(pInput))
			goto cleanup;

		pgpAssert(NULL == m_pBackupMsg);
		m_pBackupMsg = new CPGPMsgBackup((void *) this, _memoryMgr, pMsgStore, pmsg);
		pgpAssert(NULL != m_pBackupMsg);
		if(NULL == m_pBackupMsg)
			goto cleanup;

		//get the entry id of the message
		hrRet = HrGetOneProp(pmsg, PR_ENTRYID, &pval);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != pval) && 
				  (NULL != pval->Value.bin.lpb) && (0 < pval->Value.bin.cb));
		//if we got the entryid successfully
		if(SUCCEEDED(hrRet) && (NULL != pval) && 
		   (NULL != pval->Value.bin.lpb) && (0 < pval->Value.bin.cb))
		{
			//remember in the backup object that it stands for this entry id
			hrRet = m_pBackupMsg->SetMessageEntryID(pval->Value.bin.lpb, pval->Value.bin.cb);
			pgpAssert(SUCCEEDED(hrRet));
		}

		//free up the returned buffer since we reuse this pointer later
		hrRet = MAPIFreeBuffer(pval);
		pgpAssert(S_OK == hrRet);
		pval = NULL;

		//set the backup object to watch on this message
		hrRet = m_pBackupMsg->WatchMsgForDelOrMove(TRUE);
		pgpAssert(SUCCEEDED(hrRet));

		if (AutoDecrypt(_memoryMgr))
		{
			hrRet = m_pBackupMsg->AddMessage(pInput, dwInSize);
			pgpAssert(SUCCEEDED(hrRet));
		}

		//backup the formatting information if any for html/rtf emails
		hrRet = BackupFormatInfoIfAny(pmsg, (void *&)pFormatInfo, 
					dwFormatInfoSize, dwFormatInfoType, ulAttIndex);
		pgpAssert(SUCCEEDED(hrRet));
		pgpAssert((NULL != pFormatInfo)?dwFormatInfoSize > 0:TRUE);
	}

	//if we did find the formatting information
	if(NULL != pFormatInfo)
	{
		if (AutoDecrypt(_memoryMgr) && !bUseSecureViewer)
		{
			//we reuse buffer pInput here
			if(NULL != pInput)
			{
				free(pInput);
				pInput = NULL;
			}

			//copy the attachment text to pInput
			pInput = pFormatInfo;
			pgpAssert(NULL != pInput);
			pFormatInfo = NULL;//forget the pointer

			dwInSize = dwFormatInfoSize;
			pgpAssert(dwFormatInfoSize > 0);
		}
			
		if(NULL != pFormatInfo)
		{
			free(pFormatInfo);
			pFormatInfo = NULL;
		}
	}
	
	//if we got formatting info and we are backing up now 
	if ((0 < dwFormatInfoSize) && (TRUE == bNeedsBackup))
	{
		pgpAssert(TRUE == bNeedsBackup);//we must be needing backup since this seems like the first time

		//we delete the plug-in attachment from the mail so that user does not see that
		hrRet = pmsg->DeleteAttach(ulAttIndex, 0, NULL, 0);
		pgpAssert(SUCCEEDED(hrRet));

		hrRet = pmsg->SaveChanges(KEEP_OPEN_READWRITE);
		pgpAssert(SUCCEEDED(hrRet));
	}
	
	if (AutoDecrypt(_memoryMgr))
	{
		PGPPrefRef prefs;
		PGPUInt32 nCacheType;
		
		PGPclPeekClientLibPrefRefs(&prefs, NULL);
		PGPGetPrefNumber(prefs, kPGPPrefCacheType, &nCacheType);
		if (nCacheType == kPGPPrefCacheTypeNone)
		{
			nError = PGPSetPrefNumber(prefs, kPGPPrefCacheType, kPGPPrefCacheTypeLogon);
			pgpAssert(IsntPGPError(nError));

			nError = PGPclFlushClientLibPrefs(prefs, NULL);
			pgpAssert(IsntPGPError(nError));

			nError = PGPPurgePassphraseCache(_pgpContext);
			pgpAssert(IsntPGPError(nError));

			nError = PGPPurgeKeyDBCache(_pgpContext);
			pgpAssert(IsntPGPError(nError));
		}
		
		nError = DecryptVerifyBuffer(UIGetInstance(), hwnd, _pgpContext, 
					_tlsContext, szName, szFile, pInput, dwInSize, FALSE, 
					FALSE, (0 != dwFormatInfoSize), (VOID ** )&pOutput, &nOutSize,
					&FYEO);
		
		if (IsntPGPError(nError) && (nOutSize > 0) && (pOutput != NULL))
		{
			LARGE_INTEGER li = {0,0};
			ULARGE_INTEGER uli = {nOutSize, 0};
			BOOL fPartied;

			if(FYEO || bUseSecureViewer)
			{
				PGPscTempestViewer((void *)_pgpContext, hwnd, pOutput, nOutSize, FYEO);
			}
			else
			{
				if (SFPA_FOUND_HTML_DATA != dwFormatInfoType)//if we did not get html body as an attachment
				{
					//make sure PR_BODY_HTML is nullified
					pgpAssert(NULL == pstrmBody);
					hrRet = pmsg->OpenProperty(PR_BODY_HTML, &IID_IStream, STGM_READWRITE, 
								MAPI_MODIFY, (IUnknown**)&pstrmBody); 
					pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));
					if (SUCCEEDED(hrRet) && (NULL != pstrmBody))
					{
						hrRet = NullifyStream(pstrmBody);
						pgpAssert(SUCCEEDED(hrRet));

						pstrmBody->Release();
						pstrmBody = NULL;
					}
				}

				if (SFPA_FOUND_RTF_DATA != dwFormatInfoType)//if we did not get RTF body as an attachment
				{
					pgpAssert(NULL == pstrmBody);

					if (SFPA_FOUND_HTML_DATA == dwFormatInfoType)
						hrRet = pmsg->OpenProperty(PR_BODY_HTML, &IID_IStream, 
							STGM_READWRITE, MAPI_CREATE | MAPI_MODIFY, (IUnknown**)&pstrmBody);
					else
						hrRet = pmsg->OpenProperty(PR_BODY, &IID_IStream, 
							STGM_READWRITE, MAPI_MODIFY, (IUnknown**)&pstrmBody);
					pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));
					if(SUCCEEDED(hrRet) && (NULL != pstrmBody))
					{
						hrRet = pstrmBody->Seek(li, STREAM_SEEK_SET, NULL);
						pgpAssert(SUCCEEDED(hrRet));

						hrRet = pstrmBody->Write(pOutput, strlen(pOutput), NULL);
						pgpAssert(SUCCEEDED(hrRet));

						hrRet = pstrmBody->SetSize(uli);
						pgpAssert(SUCCEEDED(hrRet));

						hrRet = pstrmBody->Commit(STGC_DEFAULT);
						pgpAssert(SUCCEEDED(hrRet));

						pstrmBody->Release();
						pstrmBody = NULL;
					}

					hrRet = RTFSync(pmsg, RTF_SYNC_BODY_CHANGED, &fPartied);
					pgpAssert(SUCCEEDED(hrRet));
				}
				else
				{
					pgpAssert(NULL == pstrmRTFComp);
					hrRet = pmsg->OpenProperty(PR_RTF_COMPRESSED, &IID_IStream, 
						STGM_READWRITE, MAPI_CREATE | MAPI_MODIFY, (IUnknown**)&pstrmRTFComp);
					pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmRTFComp));
					
					if(SUCCEEDED(hrRet) && (NULL != pstrmRTFComp))
					{
						pgpAssert(NULL == pstrmBody);
						hrRet = WrapCompressedRTFStream(pstrmRTFComp, MAPI_MODIFY, &pstrmBody);
						pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));
						
						//	if (strnicmp(szBuffer, "[\\rtf", 5) &&
						//	strnicmp(szBuffer, "[\\fromhtml", 10))
						//	{
						//	pstrmBody->Write(szRTFheader, 
						//	strlen(szRTFheader), NULL);
						//	
						//	  uli.LowPart += strlen(szRTFheader);
						//	  }
						
						if(SUCCEEDED(hrRet) && (NULL != pstrmBody))
						{
							hrRet = pstrmBody->Write(pOutput, strlen(pOutput), NULL);
							pgpAssert(SUCCEEDED(hrRet));
							
							
							//  if (strnicmp(szBuffer, "[\\rtf", 5) &&
							//  strnicmp(szBuffer, "[\\fromhtml", 10))
							//  {
							//  pstrmBody->Write(szRTFfooter, 
							//  strlen(szRTFfooter) + 1, NULL);
							//
							//  uli.LowPart += strlen(szRTFfooter) + 1;
							//  }
							
							hrRet = pstrmBody->SetSize(uli);
							pgpAssert(SUCCEEDED(hrRet));

							hrRet = pstrmBody->Commit(STGC_DEFAULT);
							pgpAssert(SUCCEEDED(hrRet));

						}

						pstrmRTFComp->Release();
						pstrmRTFComp = NULL;

						hrRet = RTFSync(pmsg, RTF_SYNC_RTF_CHANGED, &fPartied);
						pgpAssert(SUCCEEDED(hrRet));
					}
				}
			}
		}
		
		pgpAssert(NULL == ptableAttach);
		hrRet = pmsg->GetAttachmentTable(0, &ptableAttach);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != ptableAttach));
		if(SUCCEEDED(hrRet) && (NULL != ptableAttach))
		{
			if(NULL != prAttach)
			{
				FreeProws(prAttach);
				prAttach = NULL;
			}
			
			//NOTE: we reuse prAttach again
			pgpAssert(NULL == prAttach);
			hrRet = HrQueryAllRows(ptableAttach, (SPropTagArray *)&tagaTable, 
						NULL, NULL, 0, &prAttach);
			pgpAssert(SUCCEEDED(hrRet) && (NULL != prAttach));
			
			if (prAttach->cRows && IsntPGPError(nError) && !bUseSecureViewer)
			{
				nError = DecryptVerifyRestoreAttachment(hwnd, pmsg, prAttach, 
							FALSE);
				pgpAssert(TRUE == IsntPGPError(nError));
			}
			
			ptableAttach->Release();
			ptableAttach = NULL;
		}
		
		if (nCacheType == kPGPPrefCacheTypeNone)
		{
			nError = PGPclPeekClientLibPrefRefs(&prefs, NULL);
			pgpAssert(IsntPGPError(nError));

			nError = PGPSetPrefNumber(prefs, kPGPPrefCacheType, nCacheType);
			pgpAssert(IsntPGPError(nError));

			nError = PGPclFlushClientLibPrefs(prefs, NULL);
			pgpAssert(IsntPGPError(nError));

			nError = PGPPurgePassphraseCache(_pgpContext);
			pgpAssert(IsntPGPError(nError));

			nError = PGPPurgeKeyDBCache(_pgpContext);
			pgpAssert(IsntPGPError(nError));
		}
	}//end if we are autodecrypting
	else if (0 < dwFormatInfoSize)
	{
		val.ulPropTag = PR_PGPEXCH_HTML;
		val.Value.b = TRUE;
		hrRet = pmsg->SetProps(1, &val, NULL);
		pgpAssert(SUCCEEDED(hrRet));
	}
	
	cleanup:

	if(NULL != pstrmBody)
	{
		pstrmBody->Release();
		pstrmBody = NULL;
	}

	if(NULL != pstrmRTFComp)
	{
		pstrmRTFComp->Release();
		pstrmRTFComp = NULL;
	}

	if(NULL != pInput)
	{
		free(pInput);
		pInput = NULL;
	}
	
	if(NULL != pOutput)
	{
		PGPFreeData(pOutput);
		pOutput = NULL;
	}

	if(NULL != prAttach)
	{
		FreeProws(prAttach);
		prAttach = NULL;
	}
	
	if(NULL != ptableAttach)
	{
		ptableAttach->Release();
		ptableAttach = NULL;
	}

	return S_FALSE;
}

HRESULT CPGPexch::AreAnyOleAttachmentsPresent(IMessage *pmsg, 
			SRowSet *prsAttachTabRowSet, BOOL &bPresent)
{
	SizedSPropTagArray(1, tagaAttach) = {1, {PR_ATTACH_METHOD}};
	SPropValue *ppvValArray = NULL;
	LPATTACH pAttach = NULL;
	ULONG ulNumVals = 0;
	ULONG ulRow = 0;
	ULONG ulNum = 0;
	HRESULT hrRet = S_OK;
	
	//check parameters
	pgpAssert(NULL != pmsg);
	pgpAssert(NULL != prsAttachTabRowSet);
	if((NULL == pmsg) || (NULL == prsAttachTabRowSet))
		return E_INVALIDARG;

	for (ulRow=0; ulRow < prsAttachTabRowSet->cRows; ulRow++)
	{
		ulNum = prsAttachTabRowSet->aRow[ulRow].lpProps[0].Value.l;

		if(NULL != pAttach)
		{
			pAttach->Release();
			pAttach = NULL;
		}

		pgpAssert(NULL == pAttach);
		hrRet = pmsg->OpenAttach(ulNum, NULL, 0, &pAttach);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != pAttach));
		if(FAILED(hrRet) || (NULL == pAttach))
			goto cleanup;

		if(NULL != ppvValArray)
		{
			hrRet = MAPIFreeBuffer(ppvValArray);
			pgpAssert(S_OK == hrRet);
			ppvValArray = NULL;
		}
		
		pgpAssert(NULL == ppvValArray);
		hrRet = pAttach->GetProps((SPropTagArray *)&tagaAttach, 
					0, &ulNumVals, &ppvValArray);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != ppvValArray) && (1 == ulNumVals));
		if(FAILED(hrRet) || (NULL == ppvValArray))
			goto cleanup;

		if (ATTACH_OLE == ppvValArray[0].Value.l)
		{
			
			bPresent = TRUE;
			goto cleanup;
		}
	}

	//control comes here only if there was no error in any of the calls
	pgpAssert(SUCCEEDED(hrRet));
	hrRet = S_OK;

	cleanup:

	if(NULL != pAttach)
	{
		pAttach->Release();
		pAttach = NULL;
	}

	if(NULL != ppvValArray)
	{
		HRESULT hrDiagRet = MAPIFreeBuffer(ppvValArray);
		pgpAssert(S_OK == hrDiagRet);
		ppvValArray = NULL;
	}
	
	return hrRet;
}


HRESULT CPGPexch::NullifyStream(IStream *pStream)
{
	LARGE_INTEGER li = {0,0};
	ULARGE_INTEGER uli = {1,0};

	//check parameters
	pgpAssert(NULL != pStream);
	if(NULL == pStream)
		return E_INVALIDARG;

	HRESULT hrRet = pStream->Seek(li, STREAM_SEEK_SET, NULL);
	pgpAssert(SUCCEEDED(hrRet));

	hrRet = pStream->Write("\0", 1, NULL);
	pgpAssert(SUCCEEDED(hrRet));

	hrRet = pStream->SetSize(uli);
	pgpAssert(SUCCEEDED(hrRet));

	hrRet = pStream->Commit(STGC_DEFAULT);
	pgpAssert(SUCCEEDED(hrRet));

	return hrRet;
}

//find the plugin format information if any present as an attachment in this message
HRESULT CPGPexch::SearchForPluginAttachment(IMessage *pCurMsg, SRowSet *prAttach, 
											ULONG &ulNum, DWORD &dwResult, char *szFileName,
											DWORD dwBufSize1, char *szLongFileName,
											DWORD dwBufSize2, char *szDisplayName,
											DWORD dwBufSize3)
{
	ULONG ulNumVals = 0;
	ULONG ulRow = 0, ulAttachNum = 0;
	HRESULT hrRet = S_OK;
	BOOL bOurAttFound = FALSE;
	BOOL bRet = TRUE;
	SPropValue *pval = NULL;
	SizedSPropTagArray(4, tagaAttach) = {4, {PR_ATTACH_METHOD, PR_ATTACH_FILENAME, 
											 PR_ATTACH_LONG_FILENAME, PR_DISPLAY_NAME}};
	enum {ivalAttachMethod=0, ivalFilename, ivalLongFilename, ivalDisplayName};
	LPATTACH pAttach = NULL;


	//check parameters
	pgpAssert(NULL != pCurMsg);
	pgpAssert(NULL != prAttach);
	if((NULL == pCurMsg) || (NULL == prAttach))
		return E_INVALIDARG;

	pgpAssert(!IsBadWritePtr(szFileName, dwBufSize1));
	pgpAssert(!IsBadWritePtr(szLongFileName, dwBufSize2));
	pgpAssert(!IsBadWritePtr(szDisplayName, dwBufSize3));

	//for each attachment in the message being operated on as long as we have not
	//found html or rtf body of the message...
	for (ulRow=0; (ulRow < prAttach->cRows) && !bOurAttFound; ulRow++)
	{
		ulAttachNum = prAttach->aRow[ulRow].lpProps[0].Value.l;

		//open the attachment
		pgpAssert(NULL == pAttach);
		hrRet = pCurMsg->OpenAttach(ulAttachNum, NULL, 0, &pAttach);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != pAttach));
		if(FAILED(hrRet) || (NULL == pAttach))
		{
			bRet = FALSE;//some attachments could not be opened !!
			goto attach_cleanup;
		}

		//get the interesting properties of the attachment
		pgpAssert(NULL == pval);
		hrRet = pAttach->GetProps((SPropTagArray *)&tagaAttach, 0, &ulNumVals, &pval);
		pgpAssert(SUCCEEDED(hrRet) && (4 == ulNumVals));
		pgpAssert(MAPI_W_ERRORS_RETURNED != hrRet);
		if(FAILED(hrRet))
		{
			bRet = FALSE;//some attachment properties could not be gotten
			goto attach_cleanup;
		}
		
		//if this attachment is not attached by value
		if (pval[ivalAttachMethod].Value.l != ATTACH_BY_VALUE)
		{
			//skip it - since we know the body is not attached that way
			goto attach_cleanup;
		}
		
		//if we successfully got the long file name
		if(PT_ERROR != PROP_TYPE(pval[ivalLongFilename].ulPropTag))
		{
			if (!strcmp(pval[ivalLongFilename].Value.lpszA, "PGPexch.htm.asc") ||
				!strcmp(pval[ivalLongFilename].Value.lpszA, "PGPexch.htm.pgp"))
			{
				dwResult |= SFPA_FOUND_HTML_DATA;
			}
			else if (!strcmp(pval[ivalLongFilename].Value.lpszA, "PGPexch.rtf.asc") ||
					!strcmp(pval[ivalLongFilename].Value.lpszA, "PGPexch.rtf.pgp"))
			{
				dwResult |= SFPA_FOUND_RTF_DATA;
			}
			else
				goto attach_cleanup;//skip this attachment since we know that
									//the formatted body is named as one of the above
		}
		
		//the control comes here only if we found the formatted body as an
		//attachment in this message

		//copy the attributes of the attachment
		pgpAssert(PROP_TYPE(pval[ivalFilename].ulPropTag) == PROP_TYPE(PR_ATTACH_FILENAME));
		if(PT_ERROR != PROP_TYPE(pval[ivalFilename].ulPropTag))
			strncpy(szFileName, pval[ivalFilename].Value.lpszA, dwBufSize1);

		pgpAssert(PROP_TYPE(pval[ivalLongFilename].ulPropTag) == PROP_TYPE(PR_ATTACH_LONG_FILENAME));
		if(PT_ERROR != PROP_TYPE(pval[ivalLongFilename].ulPropTag))
			strncpy(szLongFileName, pval[ivalLongFilename].Value.lpszA, dwBufSize2);

		pgpAssert(PROP_TYPE(pval[ivalDisplayName].ulPropTag) == PROP_TYPE(PR_DISPLAY_NAME));
		if(PT_ERROR != PROP_TYPE(pval[ivalDisplayName].ulPropTag))
			strncpy(szDisplayName, pval[ivalDisplayName].Value.lpszA, dwBufSize3);

		hrRet = S_OK;
		ulNum = ulAttachNum;
		bOurAttFound = TRUE;
		pgpAssert(0 != dwResult);
				
		attach_cleanup:
		if(NULL != pval)
		{
			hrRet = MAPIFreeBuffer(pval);
			pgpAssert(S_OK == hrRet);
			pval = NULL;
		}

		if(NULL != pAttach)
		{
			pAttach->Release();
			pAttach = NULL;
		}
	}

	return (TRUE == bRet)?S_OK:E_FAIL;
}

//get the attachment contents in a buffer allocated by callee
HRESULT CPGPexch::GetAttachBuf(IMessage *pCurMsg, ULONG ulNum, 
						char *&pAttachBuf, DWORD &dwBufSize)
{
	STATSTG AttStreamStats = {0};
	HRESULT hrRet = S_OK;
	BOOL bRet = FALSE;
	LPATTACH pAttach = NULL;
	LPSTREAM pstrmAttach = NULL; 
	DWORD dwAttSize = 0, dwSize = 0;

	//check parameters
	pgpAssert(NULL != pCurMsg);
	pgpAssert(NULL == pAttachBuf);//should not have anything in there
	if(NULL == pCurMsg)
		return E_INVALIDARG;
	
	//init the out parameters
	pAttachBuf = NULL;
	dwBufSize = 0;

	//open the attachment
	pgpAssert(NULL == pAttach);
	hrRet = pCurMsg->OpenAttach(ulNum, NULL, 0, &pAttach);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pAttach));
	if(FAILED(hrRet) || (NULL == pAttach))
		goto cleanup;
	
	//open attachment
	pgpAssert(NULL == pstrmAttach);
	hrRet = pAttach->OpenProperty(PR_ATTACH_DATA_BIN, &IID_IStream, 0, 0,
				(LPUNKNOWN *) &pstrmAttach);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmAttach));
	if(FAILED(hrRet) || (NULL == pstrmAttach))
		goto cleanup;

	//get the size
	hrRet = pstrmAttach->Stat(&AttStreamStats, STATFLAG_NONAME);
	pgpAssert(SUCCEEDED(hrRet));
	if(FAILED(hrRet))
		goto cleanup;

	dwAttSize = AttStreamStats.cbSize.LowPart;
	
	//allocate as necessary
	pAttachBuf = (char *) calloc(1, dwAttSize+1);
	pgpAssert(NULL != pAttachBuf);
	if(NULL == pAttachBuf)
		goto cleanup;

	//read the attachment bytes
	hrRet = pstrmAttach->Read(pAttachBuf, dwAttSize, &dwSize);
	pgpAssert(SUCCEEDED(hrRet) && (dwSize == dwAttSize));
	if(FAILED(hrRet))
		goto cleanup;
	
	//null terminate
	pAttachBuf[dwAttSize] = NULL;

	dwBufSize = dwAttSize;
	bRet = TRUE;

	cleanup:

	if(NULL != pstrmAttach)
	{
		pstrmAttach->Release();
		pstrmAttach = NULL;
	}

	if(NULL != pAttach)
	{
		pAttach->Release();
		pAttach = NULL;
	}

	if(FALSE == bRet)//if we have a failure
	{
		//free the buffer if we allocated
		if(NULL != pAttachBuf)
		{
			free(pAttachBuf);
			pAttachBuf = NULL;
			dwBufSize = 0;
		}
	}

	return bRet?S_OK:E_FAIL;
}

BOOL CPGPexch::DoesBufHavePGPHeaders(char *pInput)
{
	pgpAssert(NULL != pInput);
	char *szStart = pInput;
	char *szEnd = strstr(szStart, "\r\n");
	BOOL bGotPGP = FALSE;
	char szPGPbegin[] = "-----BEGIN PGP";
	char szPGPendMsg[] = "-----END PGP MESSAGE-----";
	char szPGPendSig[] = "-----END PGP SIGNATURE-----";
	UINT nPGPbegin = strlen(szPGPbegin);
	UINT nPGPendMsg = strlen(szPGPendMsg);
	UINT nPGPendSig = strlen(szPGPendSig);
	
	while (NULL != szEnd)
	{
		if (!strncmp(szStart, szPGPbegin, nPGPbegin))
		{
			bGotPGP = TRUE;
			break;
		}
		
		szStart = szEnd+2;
		szEnd = strstr(szStart, "\r\n");
	}
	
	if (bGotPGP)
	{
		bGotPGP = FALSE;
		do
		{
			szStart = szEnd+2;
			szEnd = strstr(szStart, "\r\n");
			
			if (!strncmp(szStart, szPGPendMsg, nPGPendMsg) ||
				!strncmp(szStart, szPGPendSig, nPGPendSig))
			{
				bGotPGP = TRUE;
				break;
			}
		}
		while (szEnd != NULL);
	}

	return bGotPGP;
}

HRESULT CPGPexch::ReadMsgBodyToBuf(IMessage *pCurMsg, void * &pvBuf, DWORD &dwSize)
{
	STATSTG StreamStats = {0};
	IStream *pstrmBody = NULL;
	DWORD dwStreamSize = 0, dwReadSize = 0;
	BOOL bRet = FALSE;
	HRESULT hrRet = S_OK;

	//check parameters
	pgpAssert(NULL != pCurMsg);
	if(NULL == pCurMsg)
		goto cleanup;

	//reset contents of output parameters
	pgpAssert(NULL == pvBuf);//leak ?
	pvBuf = NULL;
	dwSize = 0;

	//open up PR_BODY property
	pgpAssert(NULL == pstrmBody);
	hrRet = pCurMsg->OpenProperty(PR_BODY, &IID_IStream, STGM_READ, 0, 
				(IUnknown**)&pstrmBody);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != pstrmBody));
	if (FAILED(hrRet) || (NULL == pstrmBody))
		goto cleanup;
	
	//get the size
	hrRet = pstrmBody->Stat(&StreamStats, STATFLAG_NONAME);
	pgpAssert(SUCCEEDED(hrRet));
	if(FAILED(hrRet))
		goto cleanup;

	dwStreamSize = StreamStats.cbSize.LowPart;
	pgpAssert(0 < dwStreamSize);

	//allocate as necessary
	pvBuf = calloc(1, dwStreamSize+1);
	pgpAssert(NULL != pvBuf);
	if (NULL == pvBuf)
		goto cleanup;
	
	//read the stream to buffer allocated
	hrRet = pstrmBody->Read(pvBuf, dwStreamSize, &dwReadSize);
	pgpAssert(SUCCEEDED(hrRet) && (dwStreamSize == dwReadSize));
	if(FAILED(hrRet) || (dwReadSize != dwStreamSize))
		goto cleanup;

	((BYTE *)pvBuf)[dwStreamSize] = NULL;//null terminate
	dwSize = dwStreamSize;
	bRet = TRUE;//all success

	cleanup:
	
	if(NULL != pstrmBody)
	{
		pstrmBody->Release();
		pstrmBody = NULL;
	}

	if(FALSE == bRet)//if we failed anywhere
	{
		//free up the buffer if allocated
		if(NULL != pvBuf)
		{
			free(pvBuf);
			pvBuf = NULL;
		}
	}

	return bRet?S_OK:E_FAIL;
}

HRESULT CPGPexch::BackupFormatInfoIfAny(IMessage *pCurMsg, void * &pvBuf, 
					DWORD &dwSize, DWORD &dwBufType, ULONG &ulAttIndex)
{
	HRESULT hrRet = S_OK;
	LPMAPITABLE ptableAttach = 0;
	SRowSet *prAttach = NULL;
	ULONG ulNum = 0;
	SizedSPropTagArray(1, tagaTable) = {1, {PR_ATTACH_NUM}};
	DWORD dwFormatFlags = 0;
	char szFileName[50] = {0};
	char szLongFileName[256] = {0};
	char szDisplayName[256] = {0};

	pgpAssert(NULL == ptableAttach);
	hrRet = pCurMsg->GetAttachmentTable(0, &ptableAttach);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != ptableAttach));
	if(FAILED(hrRet) || (NULL == ptableAttach))
		goto cleanup;

	pgpAssert(NULL == prAttach);
	hrRet = HrQueryAllRows(ptableAttach, (SPropTagArray *)&tagaTable, NULL, 
				NULL, 0, &prAttach);
	pgpAssert(SUCCEEDED(hrRet) && (NULL != prAttach));
	if(FAILED(hrRet) || (NULL == prAttach))
		goto cleanup;
	
	//find if there is any formatting attachment in this mail
	hrRet = SearchForPluginAttachment(pCurMsg, prAttach, ulNum, dwFormatFlags,
				szFileName, sizeof(szFileName), szLongFileName, 
				sizeof(szLongFileName),	szDisplayName, sizeof(szDisplayName));
	pgpAssert(SUCCEEDED(hrRet));

	//if we did find the formatting attachment
	if(SUCCEEDED(hrRet) && (0 != dwFormatFlags))
	{
		pgpAssert(NULL != szFileName[0]);
		pgpAssert(NULL != szLongFileName[0]);
		pgpAssert(NULL != szDisplayName[0]);

		//the control comes here only if we found the formatted body as an
		//attachment in this message

		//remember the attachment number
		ulAttIndex = ulNum;

		//set the format on the backup object
		pgpAssert(NULL != m_pBackupMsg);
		m_pBackupMsg->SetFormat(dwFormatFlags&SFPA_FOUND_HTML_DATA, 
				dwFormatFlags&SFPA_FOUND_RTF_DATA);
		dwBufType = (dwFormatFlags&SFPA_FOUND_HTML_DATA)?
			SFPA_FOUND_HTML_DATA:SFPA_FOUND_RTF_DATA;
		
		//get the attachment text
		hrRet = GetAttachBuf(pCurMsg, ulNum, (char *&)pvBuf, dwSize);
		pgpAssert(SUCCEEDED(hrRet) && (NULL != pvBuf) && (0 < dwSize));
		if(SUCCEEDED(hrRet) && (NULL != pvBuf) && (0 < dwSize))
		{
			//update backup
			m_pBackupMsg->AddAttachment(ulNum, TRUE, szFileName,
					szLongFileName, szDisplayName, (char *)pvBuf, dwSize);
			
		}
	}
	
	cleanup:

	if(NULL != prAttach)
	{
		FreeProws(prAttach);
		prAttach = NULL;
	}

	if(NULL != ptableAttach)
	{
		ptableAttach->Release();
		ptableAttach = NULL;
	}

	return hrRet;
}

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
