/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: EncryptSign.cpp,v 1.3 2002/11/20 19:03:18 sdas Exp $
____________________________________________________________________________*/
// System Headers
#include <windows.h> 



// PGPsdk Headers
#include "pgpDebug.h"
#include "pgpKeys.h"
#include "pgpUtilities.h"

// Shared Headers
#include "pgpVersionHeader.h"
#include "pgpClientLib.h"
#include "pgpLnLib.h"
#include "Working.h"
#include "Prefs.h"
#include "BlockUtils.h"
#include "EncryptSign.h"
#include "WorkingResource.h"
#include "SharedStrings.h"
#include "pgpWin32Errors.h"


#ifndef PGP_NOTES
#define PGP_NOTES 0
#endif

static int nProgressCount = 0;

static PGPError EncryptSign(HINSTANCE hInst, HWND hwnd, PGPContextRef context, 
							PGPtlsContextRef tlsContext, 
							char *szName, char *szModule,
							PGPclRecipientDialogStruct *prds,
							PGPOptionListRef ioOptions,
							PGPOptionListRef mimeOptions,
							PGPOptionListRef *pSignOptions,
							BOOL bEncrypt, BOOL bSign, BOOL bBinary, 
							BOOL bHTML, LPEXCHEXTMODELESSCALLBACK peemc,
						    CPGPExtModeless *ppemOurMdClbk);

PGPError EncodeEventHandler(PGPContextRef context, 
							PGPEvent *event, 
							PGPUserValue userValue);

static void DisplayErrorCode(char *szFile, 
							 int nLine, 
							 char *szModule, 
							 int nCode,
							 BOOL bKeyrings,
							 HWND hParentWnd);

BOOL WrapBuffer(char **pszOutText,
				char *szInText,
				PGPUInt16 wrapLength)
{
	BOOL RetVal = FALSE;
	PGPError err;
	char *cmdlgBuffer;

	err=PGPclWrapBuffer(
					szInText,
					wrapLength,
					&cmdlgBuffer);

	if(IsntPGPError (err))
	{
		int memamt,length;

		length=strlen(cmdlgBuffer);
		memamt=length+1;

		*pszOutText=(char *)malloc(memamt);
		memcpy(*pszOutText,cmdlgBuffer,length);
		(*pszOutText)[length]=0;
		PGPclFreeWrapBuffer(cmdlgBuffer);
		RetVal = TRUE;
	}

	return RetVal;
}


PGPError EncryptSignBuffer(HINSTANCE hInst, HWND hwnd, PGPContextRef context, 
						   PGPtlsContextRef tlsContext, 
						   char *szName, char *szModule,
						   void *pInput, DWORD dwInSize, 
						   PGPclRecipientDialogStruct *prds,
						   PGPOptionListRef mimeOptions,
						   PGPOptionListRef *pSignOptions,
						   void **ppOutput, PGPSize *pOutSize, BOOL bEncrypt, 
						   BOOL bSign, BOOL bBinary, BOOL bHTML, 
						   LPEXCHEXTMODELESSCALLBACK peemc,
						   CPGPExtModeless *ppemOurMdClbk)
{
	PGPError			err				= kPGPError_NoErr;
	PGPMemoryMgrRef		memoryMgr		= NULL;
	PGPOptionListRef	options			= NULL;
	void *				pFinalInput		= NULL;
	long				lWrapWidth		= 0;
	BOOL				bInputWrapped	= FALSE;

	pgpAssert(pInput != NULL);
	pgpAssert(prds != NULL);
	pgpAssert(prds->keydbOriginal != NULL);
	pgpAssert(pSignOptions != NULL);
	pgpAssert(ppOutput != NULL);
	pgpAssert(pOutSize != NULL);
	pgpAssert(context != NULL);

	memoryMgr = PGPPeekContextMemoryMgr(context);
	pFinalInput = pInput;

	if (!bBinary && !bHTML)
	{
		if (ByDefaultWordWrap(memoryMgr, &lWrapWidth))
		{
			char *szWrapped = NULL;
			char *szPtr = (char *) pInput;
			char cTemp;
			ULONG ulStart = 0;
			ULONG ulSize = (ULONG) dwInSize;
			ULONG ulLength = 0;

			pFinalInput = calloc(1, 1);

			bInputWrapped=TRUE;

			dwInSize = 0;
			do
			{
				if (FindSignedBlock(szPtr, ulSize, &ulStart, &ulLength))
				{
					cTemp = szPtr[ulStart];
					szPtr[ulStart] = 0;
					WrapBuffer(&szWrapped, szPtr, 
										(short) lWrapWidth);
					dwInSize += strlen(szWrapped) + ulLength;
					pFinalInput = realloc(pFinalInput, dwInSize+1);
					strcat((char *)pFinalInput, szWrapped);
					free(szWrapped);
					szPtr[ulStart] = cTemp;
					cTemp = szPtr[ulStart+ulLength];
					szPtr[ulStart+ulLength] = 0;
					strcat((char *)pFinalInput, &szPtr[ulStart]);
					szPtr[ulStart+ulLength] = cTemp;
					szPtr += ulStart+ulLength;
					ulSize -= (ulStart+ulLength);
				}
				else if (FindEncryptedBlock(szPtr, ulSize, &ulStart, &ulLength))
				{
					cTemp = szPtr[ulStart];
					szPtr[ulStart] = 0;
					WrapBuffer(&szWrapped, szPtr, 
										(short) lWrapWidth);
					dwInSize += strlen(szWrapped) + ulLength;
					pFinalInput = realloc(pFinalInput, dwInSize+1);
					strcat((char *)pFinalInput, szWrapped);
					free(szWrapped);
					szPtr[ulStart] = cTemp;
					cTemp = szPtr[ulStart+ulLength];
					szPtr[ulStart+ulLength] = 0;
					strcat((char *)pFinalInput, &szPtr[ulStart]);
					szPtr[ulStart+ulLength] = cTemp;
					szPtr += ulStart+ulLength;
					ulSize -= (ulStart+ulLength);
				}
				else if (FindPGPKeyBlock(szPtr, ulSize, &ulStart, &ulLength))
				{
					cTemp = szPtr[ulStart];
					szPtr[ulStart] = 0;
					WrapBuffer(&szWrapped, szPtr, 
										(short) lWrapWidth);
					dwInSize += strlen(szWrapped) + ulLength;
					pFinalInput = realloc(pFinalInput, dwInSize+1);
					strcat((char *)pFinalInput, szWrapped);
					free(szWrapped);
					szPtr[ulStart] = cTemp;
					cTemp = szPtr[ulStart+ulLength];
					szPtr[ulStart+ulLength] = 0;
					strcat((char *)pFinalInput, &szPtr[ulStart]);
					szPtr[ulStart+ulLength] = cTemp;
					szPtr += ulStart+ulLength;
					ulSize -= (ulStart+ulLength);
				}
				else
				{
					dwInSize += strlen(szPtr);
					pFinalInput = realloc(pFinalInput, dwInSize+1);
					strcat((char *)pFinalInput, szPtr);
					ulLength = 0;
					dwInSize = strlen((char *) pFinalInput);
				}
			}
			while (ulLength > 0);
		}

		if (!dwInSize)
		{
			if (bInputWrapped)
				free(pFinalInput);

			bInputWrapped = FALSE;
			pFinalInput = "\r\n";
			dwInSize = strlen((char *)pFinalInput);
		}
	}

	err = PGPBuildOptionList(context, &options, 
			PGPOInputBuffer(context, pFinalInput, dwInSize),
			PGPOAllocatedOutputBuffer(context, 
				ppOutput, 
				INT_MAX, 
				pOutSize),
			PGPOLastOption(context));

	if (IsPGPError(err))
	{
		DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
		goto EncryptSignBufferError;
	}

	err = EncryptSign(hInst, hwnd, context, tlsContext, szName, szModule, 
			prds, options, mimeOptions, pSignOptions, bEncrypt, bSign, 
			bBinary, bHTML, peemc, ppemOurMdClbk);

EncryptSignBufferError:

	if (bInputWrapped)
	{
		memset(pFinalInput,0x00,dwInSize);
		free(pFinalInput);
	}

	if (options != NULL)
		PGPFreeOptionList(options);

	return err;
}


PGPError EncryptSignFile(HINSTANCE hInst, HWND hwnd, PGPContextRef context, 
						 PGPtlsContextRef tlsContext, 
						 char *szName, char *szModule, char *szInFile, 
						 PGPclRecipientDialogStruct *prds, 
						 PGPOptionListRef mimeOptions,
						 PGPOptionListRef *pSignOptions, char *szOutFile, 
						 BOOL bEncrypt, BOOL bSign, BOOL bBinary,
						 LPEXCHEXTMODELESSCALLBACK peemc,
						 CPGPExtModeless *ppemOurMdClbk)
{
	PGPError			err				= kPGPError_NoErr;
	PGPOptionListRef	options			= NULL;
	PGPFileSpecRef		inputFile		= NULL;
	PGPFileSpecRef		outputFile		= NULL;

	pgpAssert(szInFile != NULL);
	pgpAssert(prds != NULL);
	pgpAssert(prds->keydbOriginal != NULL);
	pgpAssert(pSignOptions != NULL);
	pgpAssert(szOutFile != NULL);
	pgpAssert(context != NULL);

	err = PGPNewFileSpecFromFullPath(context, szInFile, &inputFile);
	if (IsPGPError(err))
	{
		DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
		goto EncryptSignFileError;
	}

	err = PGPNewFileSpecFromFullPath(context, szOutFile, &outputFile);
	if (IsPGPError(err))
	{
		DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
		goto EncryptSignFileError;
	}

	err = PGPBuildOptionList(context, &options, 
			PGPOInputFile(context, inputFile),
			PGPOOutputFile(context, outputFile),
			PGPOLastOption(context));

	if (IsPGPError(err))
	{
		DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
		goto EncryptSignFileError;
	}

	err = EncryptSign(hInst, hwnd, context, tlsContext, szName, szModule, 
			prds, options, mimeOptions, pSignOptions, bEncrypt, bSign, 
			bBinary, FALSE, peemc, ppemOurMdClbk);

EncryptSignFileError:

	if (options != NULL)
		PGPFreeOptionList(options);

	if (inputFile != NULL)
		PGPFreeFileSpec(inputFile);

	if (outputFile != NULL)
		PGPFreeFileSpec(outputFile);

	return err;
}


PGPError EncryptSign(HINSTANCE hInst, HWND hwnd, PGPContextRef context, 
					 PGPtlsContextRef tlsContext, 
					 char *szName, char *szModule,
					 PGPclRecipientDialogStruct *prds,
					 PGPOptionListRef ioOptions,
					 PGPOptionListRef mimeOptions,
					 PGPOptionListRef *pSignOptions,
					 BOOL bEncrypt, BOOL bSign, 
					 BOOL bBinary, BOOL bHTML,
					 LPEXCHEXTMODELESSCALLBACK peemc,
					 CPGPExtModeless *ppemOurMdClbk)
{
	PGPError			err				= kPGPError_NoErr;
	PGPMemoryMgrRef		memoryMgr		= NULL;
	PGPKeyDBObjRef		signKey			= NULL;
	PGPKeyDBRef			addedKeys		= NULL;
	PGPKeySetRef		recipKeySet		= NULL;
	PGPOptionListRef	options			= NULL;
	PGPOptionListRef	encryptOptions	= NULL;
	PGPOptionListRef	tempOptions		= NULL;
	PGPCipherAlgorithm  prefAlg			= kPGPCipherAlgorithm_CAST5;
	PGPCipherAlgorithm	allowedAlgs[20];
	char *				szPassphrase	= NULL;
	char *				szConvPass		= NULL;
	PGPByte *			pPasskey		= NULL;
	PGPUInt32			nPasskeyLength	= 0;
	int					nPassphraseLen	= 0;
	int					nConvPassLen	= 0;
	int					nNumAlgs		= 0;
	BOOL				bGotPassphrase	= FALSE;
	BOOL				bGotConvPass	= FALSE;
	BOOL				bTextDocument	= FALSE;
	HWND				hwndWorking		= NULL;
	char 				szComment[256];
	char 				szWorkingTitle[256];

	UpdateWindow(hwnd);
	memoryMgr = PGPPeekContextMemoryMgr(context);

#if PGP_NOTES
	// Check license number and expiration with clientlib functions.
	// This is to avoid having to compile all the PGPlnLib code into
	// the Notes plugin
	if(!PGPclPGPmailEnabled())
	{
		PGPclNoLicense(hwnd,kPGPclLotusNotesPlugin);
		return kPGPError_UserAbort;
	}

	// Check for demo expiration
	if (PGPclIsExpired(hwnd,kPGPclEncryptSignExpired) != kPGPError_NoErr)
	{
		PGPclErrorBox(hwnd, kPGPError_Win32_Expired);
		return kPGPError_UserAbort;
	}
#else
	// Check license number and expiration with static library functions.
	// This makes it more difficult to crack.
	PGPlnLicenseCheckLite (context);
	if(!PGPlnPGPmailEnabled())
	{
		PGPclNoLicense(hwnd,kPGPclExchangePlugin);
		return kPGPError_UserAbort;
	}

	// Check for expiration
	if (PGPlnIsExpired(hwnd, kPGPlnOperationStandardExpiration) != kPGPError_NoErr)
		return kPGPError_UserAbort;
#endif

	if (bSign)
	{
		err = PGPclGetDefaultPrivateKey(prds->keydbOriginal, &signKey);
		if (IsPGPError(err))
		{
			PGPKeySetRef	pubKeySet = NULL;
			PGPKeyListRef	pubKeyList = NULL;
			PGPKeyIterRef	pubKeyIter = NULL;
			
			PGPNewKeySet(prds->keydbOriginal, &pubKeySet);
			PGPOrderKeySet(pubKeySet, kPGPKeyOrdering_Trust, FALSE, 
				&pubKeyList);
			PGPNewKeyIter(pubKeyList, &pubKeyIter);
			PGPKeyIterNextKeyDBObj(pubKeyIter, kPGPKeyDBObjType_Key, &signKey);
			PGPFreeKeyIter(pubKeyIter);
			PGPFreeKeyList(pubKeyList);
			PGPFreeKeySet(pubKeySet);
			
			err = kPGPError_NoErr;
		}
	}
	
	err = kPGPError_BadPassphrase;
	while (err == kPGPError_BadPassphrase)
	{
		if (szPassphrase != NULL)
		{
			PGPclFreeCachedPhrase(szPassphrase);
			szPassphrase = NULL;
		}

		if (szConvPass != NULL)
		{
			PGPclFreePhrase(szConvPass);
			szConvPass = NULL;
		}

		if (mimeOptions == NULL)
			err = PGPBuildOptionList(context, &options, 
					PGPOLastOption(context));
		else
			err = PGPBuildOptionList(context, &options, 
					mimeOptions,
					PGPOLastOption(context));

		if (IsPGPError(err))
		{
			DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
			goto EncryptSignError;
		}

		err = PGPBuildOptionList(context, &encryptOptions,
				PGPOLastOption(context));

		if (IsPGPError(err))
		{
			DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
			goto EncryptSignError;
		}

		if (GetCommentString(memoryMgr, szComment, 254))
		{
			err = PGPBuildOptionList(context, &tempOptions,
					options,
					PGPOCommentString(context, szComment),
					PGPOLastOption(context));
			
			if (IsPGPError(err))
			{
				DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
				goto EncryptSignError;
			}

			PGPFreeOptionList(options);
			options = tempOptions;
		}

		if (bEncrypt)
		{
			if (prds->dwOptions & kPGPclASCIIArmor)
				err = PGPBuildOptionList(context, &tempOptions,
						options,
						PGPOArmorOutput(context, TRUE),
						PGPOLastOption(context));
			else
				err = PGPBuildOptionList(context, &tempOptions,
						options,
						PGPOArmorOutput(context, FALSE),
						PGPOLastOption(context));
				
			if (IsPGPError(err))
			{
				DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
				goto EncryptSignError;
			}
			
			PGPFreeOptionList(options);
			options = tempOptions;
				
			if (*pSignOptions == NULL)
			{
				if (prds->dwOptions & kPGPclPassOnly)
				{
					if (!bGotConvPass)
					{
						char szPrompt[256];
						
						LoadString(hInst, IDS_CONVPASSPHRASE, szPrompt, 
							sizeof(szPrompt));
						
						err = PGPclGetConventionalPhrase(context, hwnd, 
								szPrompt, &szConvPass);
						
						if (err == kPGPError_UserAbort)
							goto EncryptSignError;
						
						nConvPassLen = strlen(szConvPass);
						bGotConvPass = TRUE;
					}
					
					if (!GetPreferredAlgorithm(memoryMgr, &prefAlg))
						goto EncryptSignError;
					
					err = PGPBuildOptionList(context, &tempOptions,
							encryptOptions,
							PGPOConventionalEncrypt(context,
								PGPOPassphraseBuffer(context,
									szConvPass, 
									nConvPassLen),
								PGPOLastOption(context)),
							PGPOCipherAlgorithm(context, prefAlg),
							PGPOLastOption(context));
					
					if (IsPGPError(err))
					{
						DisplayErrorCode(__FILE__, __LINE__, szModule, 
							err, FALSE, hwnd);
						goto EncryptSignError;
					}
				}
				else
				{
					if (!GetAllowedAlgorithms(memoryMgr, allowedAlgs, 
						&nNumAlgs))
						goto EncryptSignError;

					PGPNewKeySet(prds->keydbSelected, &recipKeySet);
					
					err = PGPBuildOptionList(context, &tempOptions,
							encryptOptions,
							PGPOPreferredAlgorithms(context,
								allowedAlgs, nNumAlgs ),
							PGPOEncryptToKeySet(context, recipKeySet),
							PGPOFailBelowValidity(context, 
								kPGPValidity_Unknown),
							PGPOWarnBelowValidity(context, 
								kPGPValidity_Unknown),
							PGPOLastOption(context));
					
					if (IsPGPError(err))
					{
						DisplayErrorCode(__FILE__, __LINE__, szModule, 
							err, FALSE, hwnd);
						goto EncryptSignError;
					}
				}
				
				PGPFreeOptionList(encryptOptions);
				encryptOptions = tempOptions;
			}
		}
		
		if (bSign)
		{
			if (*pSignOptions == NULL)
			{
				char szPrompt[256];

				if (bGotPassphrase)
					LoadString(hInst, IDS_PASSPHRASEREENTER, szPrompt, 
						sizeof(szPrompt));
				else
					LoadString(hInst, IDS_PASSPHRASEPROMPT, szPrompt, 
						sizeof(szPrompt));
				
				err = PGPclGetCachedSigningPhrase(context, tlsContext, hwnd, 
						szPrompt, bGotPassphrase, &szPassphrase, 
						prds->keydbOriginal, &signKey, NULL, 
						prds->dwFlags, &pPasskey, &nPasskeyLength, 
						&addedKeys, NULL);
				
				if (addedKeys != NULL)
				{
					PGPUInt32 numKeys;
					PGPKeySetRef addedKeySet;

					PGPNewKeySet(addedKeys, &addedKeySet);
					PGPCountKeys(addedKeySet, &numKeys);
					if (numKeys > 0)
						PGPclImportKeys(context, tlsContext, hwnd, 
							addedKeySet, prds->keydbOriginal, 
							kPGPclNoDialogForValidSingletons);

					PGPFreeKeySet(addedKeySet);
					PGPFreeKeyDB(addedKeys);
					addedKeys = NULL;
				}

				if (IsPGPError(err))
				{
					if (err != kPGPError_UserAbort)
						DisplayErrorCode(__FILE__, __LINE__, szModule, 
							err, FALSE, hwnd);
					goto EncryptSignError;
				}
				
				bGotPassphrase = TRUE;
				if (szPassphrase != NULL)
				{
					nPassphraseLen = strlen(szPassphrase);
				
					err = PGPBuildOptionList(context, pSignOptions,
							PGPOSignWithKey(context, 
								signKey, 
								PGPOPassphraseBuffer(context,
									szPassphrase, 
									nPassphraseLen),
								PGPOLastOption(context)),
							PGPOLastOption(context));
				}
				else if (pPasskey != NULL)
				{
					err = PGPBuildOptionList(context, pSignOptions,
							PGPOSignWithKey(context, 
								signKey, 
								PGPOPasskeyBuffer(context,
									pPasskey, 
									nPasskeyLength),
								PGPOLastOption(context)),
							PGPOLastOption(context));
				}
				else
				{
					err = PGPBuildOptionList(context, pSignOptions,
							PGPOSignWithKey(context, 
								signKey, 
								PGPOLastOption(context)),
							PGPOLastOption(context));
				}
				
				if (IsPGPError(err))
				{
					if (err != kPGPError_UserAbort)
						DisplayErrorCode(__FILE__, __LINE__, szModule, 
							err, FALSE, hwnd);
					goto EncryptSignError;
				}
			}

			err = PGPBuildOptionList(context, &tempOptions,
					options,
					PGPOClearSign(context, 
						(PGPBoolean) (!bEncrypt && !bBinary && !bHTML)),
					PGPOLastOption(context));
			
			if (IsPGPError(err))
			{
				if (err != kPGPError_UserAbort)
					DisplayErrorCode(__FILE__, __LINE__, szModule, 
						err, FALSE, hwnd);
				goto EncryptSignError;
			}

			PGPFreeOptionList(options);
			options = tempOptions;
		}
		
		if (bEncrypt && !bSign)
			LoadString(hInst, IDS_WORKINGENCRYPT, szWorkingTitle, 
				sizeof(szWorkingTitle));
		else if (!bEncrypt && bSign)
			LoadString(hInst, IDS_WORKINGSIGN, szWorkingTitle, 
				sizeof(szWorkingTitle));
		else
			LoadString(hInst, IDS_WORKINGENCRYPTSIGN, szWorkingTitle, 
				sizeof(szWorkingTitle));

		hwndWorking = WorkingDlgProcThread(GetModuleHandle(szModule), 
						hInst, hwnd, szWorkingTitle, "", peemc, 
						ppemOurMdClbk);

		if (*pSignOptions == NULL)
			PGPBuildOptionList(context, pSignOptions, 
				PGPOLastOption(context));

		bTextDocument = !bBinary || 
						((prds->dwOptions & kPGPclInputIsText)
							== kPGPclInputIsText);

		err = PGPEncode(context,
				ioOptions,
				PGPOEventHandler(context, EncodeEventHandler, hwndWorking),
				options,
				encryptOptions,
				*pSignOptions,
				PGPOSendNullEvents(context, 100),
				PGPODataIsASCII(context, (PGPBoolean) bTextDocument),
				PGPOVersionString(context, pgpVersionHeaderString()),
				PGPOAskUserForEntropy(context, TRUE),
				PGPOForYourEyesOnly(context,
					(PGPBoolean)((prds->dwOptions & kPGPclSecureViewer)
									== kPGPclSecureViewer)),
				PGPOLastOption(context));

		WorkingCallback(hwndWorking, -1, -1);

		if (options != NULL)
		{
			PGPFreeOptionList(options);
			options = NULL;
		}

		if (err == kPGPError_BadPassphrase)
		{
			if (encryptOptions != NULL)
			{
				PGPFreeOptionList(encryptOptions);
				encryptOptions = NULL;
			}

			PGPFreeOptionList(*pSignOptions);
			*pSignOptions = NULL;
		}
	}

	if (IsPGPError(err) && (err != kPGPError_UserAbort))
	{
		DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
		goto EncryptSignError;
	}

	if (IsntPGPError(err))
	{
		if (*pSignOptions == NULL)
			PGPBuildOptionList(context, pSignOptions, 
				PGPOLastOption(context));

		PGPBuildOptionList(context, &tempOptions, 
			*pSignOptions,
			encryptOptions,
			PGPOLastOption(context));

		PGPFreeOptionList(*pSignOptions);
		PGPFreeOptionList(encryptOptions);
		*pSignOptions = tempOptions;
		encryptOptions = NULL;
	}

EncryptSignError:

	if (szPassphrase != NULL)
	{
		PGPclFreeCachedPhrase(szPassphrase);
		szPassphrase = NULL;
	}

	if (pPasskey != NULL)
	{
		PGPFreeData(pPasskey);
		pPasskey = NULL;
	}

	if (szConvPass != NULL)
	{
		PGPclFreePhrase(szConvPass);
		szConvPass = NULL;
	}

	if (encryptOptions != NULL)
		PGPFreeOptionList(encryptOptions);

	if (options != NULL)
		PGPFreeOptionList(options);

	return err;
}


PGPError EncodeEventHandler(PGPContextRef context, 
							PGPEvent *event, 
							PGPUserValue userValue)
{
	HWND			hwnd			= NULL;
	PGPError		err				= kPGPError_NoErr;

	pgpAssert(context != NULL);
	pgpAssert(event != NULL);

	hwnd = (HWND) userValue;

	switch (event->type)
	{
	case kPGPEvent_EntropyEvent:
		err = PGPclRandom(context, hwnd, 
				event->data.entropyData.entropyBitsNeeded);
		break;

	case kPGPEvent_NullEvent:
		{	
			PGPEventNullData *d = &event->data.nullData;
			BOOL bCancel;

			bCancel = WorkingCallback (	(HWND)userValue, 
										(unsigned long)d->bytesWritten, 
										(unsigned long)d->bytesTotal) ;

			if(bCancel)
			{
				return kPGPError_UserAbort;
			}
		}
		break;
	}

	return err;
}


void DisplayErrorCode(char *szFile, int nLine, char *szModule, int nCode,
					  BOOL bKeyrings, HWND hParentWnd)
{
	char szErrorMsg[255];

	if (bKeyrings)
		PGPclErrorToString(nCode, szErrorMsg, 254, NULL);
	else
		PGPclEncDecErrorToString(nCode, szErrorMsg, 254);

#ifdef _DEBUG
	_CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_WNDW);
	_CrtDbgReport(_CRT_ERROR, szFile, nLine, szModule, szErrorMsg);
#endif

	MessageBox(hParentWnd, szErrorMsg, szModule, MB_ICONEXCLAMATION);
	return;
}


