/*__________________________________________________________________________
 Copyright (C) 2002 PGP Corporation
 All rights reserved.
 
 $Id: SCencrypt.c,v 1.30 2002/11/15 22:11:09 wjb Exp $
__________________________________________________________________________*/
#include "precomp.h"

BOOL GetCommentLine(PGPContextRef context,
					char *comment,PGPSize size,PGPUInt32 *PrefAlg,
					PGPCipherAlgorithm *pAllowedAlgs, int *pNumAlgs)
{
	PGPPrefRef	prefRef=NULL;
	PGPError	err;
	BOOL RetVal;

	RetVal=TRUE;

	*comment=0;
	*PrefAlg=kPGPCipherAlgorithm_CAST5;

	err=PGPclPeekClientLibPrefRefs(&prefRef,NULL);

	if(IsntPGPError(err))
	{
		PGPPrefArray *prefArray = NULL;
		int i;

		PGPGetPrefNumber(prefRef,
			kPGPPrefPreferredAlgorithm, 
			PrefAlg);

		err = PGPGetPrefArray(prefRef,
			kPGPPrefAllowedAlgorithmsList, 
			&prefArray);

		if (IsntPGPError(err))
		{
			PGPBoolean aes256=FALSE;
			PGPInt32 num = prefArray->numElements;

			// if we would have length passed, then
			//if( num > *pNumAlgs )
			//	num = *pNumAlgs;

			for (i=0; i<num; i++)  
			{
				pAllowedAlgs[i] = (PGPCipherAlgorithm) 
									prefArray->elements[i].data;
				if( pAllowedAlgs[i]==kPGPCipherAlgorithm_AES256 )
					aes256=TRUE;
			}
			
			// Also add other AES algorithms for smaller keys; we assume that 
			// preference for the AES algorithm is stored as kPGPCipherAlgorithm_AES256.

			// Better solution is to introduce client-defined kPGPCipherAlgorithm_AES
			// with invalid algorithm value (more the 0xff) and check for 
			// kPGPCipherAlgorithm_AES instead of valid id kPGPCipherAlgorithm_AES256
			if( aes256 )  
			{	
				// if( i<*pNumAlgs ) if we would have length passed
					pAllowedAlgs[i++] = kPGPCipherAlgorithm_AES128;
				// if( i<*pNumAlgs ) if we would have length passed
					pAllowedAlgs[i++] = kPGPCipherAlgorithm_AES192;

#if PGP_DEBUG
				{
					char s[80];
					sprintf(s, "Cloning AES alg pref, total size=%d\n", i);
					OutputDebugString(s);
				}
#endif
			}

			*pNumAlgs = i;

			PGPFreePrefArray(prefArray);
		}
		else
			*pNumAlgs = 0;

		if(*comment==0)
		{
			PGPGetPrefStringBuffer(prefRef,
				kPGPPrefComment,
				size,
				comment);
		}
	}
	
	return RetVal;
}

PGPError GenericEncSign(MYSTATE *ms,
					PGPOptionListRef opts,
					char *OperationTarget)
{
	PGPError err;
	int UserCancel;
	PGPBoolean DataIsASCII;
	PPGPclRecipientDialogStruct prds;
	PGPContextRef	context;
	char StrRes[500],szRDprompt[100],szPassTitle[100];
	PGPBoolean FYEO;
	PGPKeySetRef recipientSet;
	PGPUInt32 iMinPhraseLength;
	PGPUInt32 iMinPhraseQuality;

	DataIsASCII=FALSE;

	iMinPhraseLength=iMinPhraseQuality=0;

	err=kPGPError_NoErr;	
	recipientSet=NULL;

	context=ms->context;

	prds=ms->prds;

	// If this is first operation, put up recipient/sign dialog

	if(ms->numDone==0)
	{
		FreePhrases(ms);

		if(ms->Encrypt)
		{
			int RecipientReturn;

			LoadString (g_hinst, IDS_PGPKEYSELDLG, StrRes, sizeof(StrRes));
			strcpy(szRDprompt,ms->szAppName);
			strcat(szRDprompt,StrRes);

			prds->Version=kPGPCurrentRecipVersion;
			prds->hwndParent=ms->hwndWorking;
			prds->szTitle=szRDprompt;
			prds->context=ms->context;
			prds->tlsContext=ms->tlsContext;
			prds->keydbOriginal=ms->KeyDB;

			prds->dwDisableFlags|=kPGPclDisableSecureViewer;

			// FYEO only for clipboard or text files
			if((ms->Operation==MS_ENCRYPTCLIPBOARD) ||
				(FileHasThisExtension(OperationTarget,"txt")))
			{
				prds->dwDisableFlags&=(~kPGPclDisableSecureViewer);
			}

			if(!((ms->Operation==MS_ENCRYPTFILELIST)&&
				(ms->Encrypt)&&(!ms->Sign)))
				prds->dwDisableFlags|=kPGPclDisableSelfDecryptingArchive;

			RecipientReturn = PGPclRecipientDialog(prds);

			if(!RecipientReturn)
				return kPGPError_UserAbort;

		// minimum passphrase length
			if(PGPlnEnterprise())
			{
				PGPError prefErr;
				PGPBoolean b;
				PGPPrefRef prefs;

				prefErr = PGPclPeekClientLibPrefRefs (&prefs, NULL);

				if(IsntPGPError(prefErr))
				{
					prefErr = PGPGetPrefBoolean (prefs, kPGPPrefEnforceMinChars, &b);

					if (IsntPGPError(prefErr)&&(b)) 
					{
						PGPGetPrefNumber (prefs, kPGPPrefMinChars, &iMinPhraseLength);
					}

					// minimum passphrase quality
					prefErr = PGPGetPrefBoolean (prefs, kPGPPrefEnforceMinQuality, &b);

					if(IsntPGPError(prefErr)&&(b)) 
					{
						PGPGetPrefNumber (prefs, kPGPPrefMinQuality, &iMinPhraseQuality);
					}
				}
			}
			if(iMinPhraseLength<1)
				iMinPhraseLength=1;

			if(prds->dwOptions & kPGPclSelfDecryptingArchive)
			{
				LoadString (g_hinst, IDS_ENTERPASSPHRASE, StrRes, sizeof(StrRes));
				strcpy(szPassTitle,ms->szAppName);
				strcat(szPassTitle,StrRes);

				LoadString (g_hinst, IDS_ENTERSDAPASS, StrRes, sizeof(StrRes));

				UserCancel = PGPclGetPhrase (context,
					ms->KeyDB,
					ms->hwndWorking,
					StrRes,
					&(ms->ConvPassPhrase), 
					NULL,
					NULL, 
					0,
					NULL,
					NULL,
					kPGPclEncryption,
					NULL,NULL,
					iMinPhraseLength,
					iMinPhraseQuality,
					ms->tlsContext,NULL,szPassTitle);

				if(UserCancel)
					return kPGPError_UserAbort;
			}
			else if(prds->dwOptions & kPGPclPassOnly)
			{
				LoadString (g_hinst, IDS_ENTERPASSPHRASE, StrRes, sizeof(StrRes));
				strcpy(szPassTitle,ms->szAppName);
				strcat(szPassTitle,StrRes);

				LoadString (g_hinst, IDS_CONVPASSPROMPT, StrRes, sizeof(StrRes));

				UserCancel = PGPclGetPhrase (context,
					ms->KeyDB,
					ms->hwndWorking,
					StrRes,
					&(ms->ConvPassPhrase), 
					NULL,
					NULL, 
					0,
					NULL,
					NULL,
					kPGPclEncryption,
					NULL,NULL,
					iMinPhraseLength,
					iMinPhraseQuality,
					ms->tlsContext,NULL,szPassTitle);

				if(UserCancel)
					return kPGPError_UserAbort;
			}

		}

		if(ms->Sign)
		{
			DWORD *lpOptions;
			PGPError err;

			LoadString (g_hinst, IDS_ENTERPASSPHRASE, StrRes, sizeof(StrRes));
			strcpy(szPassTitle,ms->szAppName);
			strcat(szPassTitle,StrRes);

			lpOptions=&(prds->dwOptions);

			if (ms->Encrypt || (ms->Operation==MS_ENCRYPTCLIPBOARD))
				lpOptions=0;

			LoadString (g_hinst, IDS_PASSPROMPTABOVEKEY, StrRes, sizeof(StrRes));

			err=PGPclGetCachedSigningPhrase (context,
				ms->tlsContext,
				ms->hwndWorking, 
				StrRes,
				FALSE, // ForceEntry
				&ms->PassPhrase,
				ms->KeyDB,
				&(ms->SignKey),
///				NULL,
				lpOptions,
				prds->dwFlags,
				&ms->PassKey,
				&ms->PassKeyLen,
				NULL,szPassTitle);
			
			if(err != kPGPError_NoErr)
			{
				PGPclEncDecErrorBox(ms->hwndWorking,err);
				return kPGPError_UserAbort; // UserCancel 
			}
		}
	} // END of first operation recipient/sign dialog stuff

	ms->fileName=OperationTarget;

	if(prds->dwOptions & kPGPclInputIsText)
	{
		DataIsASCII=TRUE;
	}

	if(prds->dwOptions & kPGPclSelfDecryptingArchive)
	{
		// Self Decryptor
		err=SDA(ms);

		// Inform loop to stop processing files, since we've done 
		// them all
		ms->bDoingSDA=TRUE;

		// Don't do anything else like file wiping etc...
		return err;
	}

	// if encrypting, throw in recipients
	if(ms->Encrypt)
	{
		if(prds->dwOptions & kPGPclPassOnly)
		{
			int ConvPassLen;

			ConvPassLen = strlen(ms->ConvPassPhrase);

			PGPAppendOptionList(opts,
				PGPOConventionalEncrypt(context, 
					PGPOPassphraseBuffer(context,ms->ConvPassPhrase,
						ConvPassLen),
						PGPOLastOption(context) ),
				PGPOCipherAlgorithm(context,ms->PrefAlg),
				PGPOLastOption(context) );
		}
		else
		{
			PGPNewKeySet( prds->keydbSelected, &recipientSet );

			PGPAppendOptionList(opts,
				PGPOEncryptToKeySet(context,recipientSet),
				PGPOLastOption(context) );
		}
	}

	if(ms->Sign)  // If signing, throw in a passphrase and key
	{
		// Use passkey if available
		if(ms->PassKey)
		{
			PGPAppendOptionList(opts,
				PGPOSignWithKey(context,ms->SignKey,
					PGPOPasskeyBuffer(context,
						ms->PassKey,ms->PassKeyLen),
					PGPOLastOption(context) ),
				PGPOLastOption(context) );
		}
		else
		{
			PGPAppendOptionList(opts,
				PGPOSignWithKey(context,ms->SignKey,
					PGPOLastOption(context) ),
				PGPOLastOption(context) );
		}
	}

	if((prds->dwOptions & kPGPclDetachedSignature) &&
		!(ms->Operation==MS_ENCRYPTCLIPBOARD) && 
		(ms->Sign) && !(ms->Encrypt))
	{
		PGPAppendOptionList(opts,
			PGPODetachedSig(context,
				PGPOLastOption(context)), 
			PGPOLastOption(context));
	}
	else
		prds->dwOptions&=(~kPGPclDetachedSignature);

	DataIsASCII=DataIsASCII||(ms->Operation==MS_ENCRYPTCLIPBOARD);

	// If we have text out arguments....
	if(prds->dwOptions & kPGPclASCIIArmor)
	{
		if((ms->Operation==MS_ENCRYPTCLIPBOARD) && 
			(ms->Sign) && !(ms->Encrypt) && 
			((prds->dwOptions & kPGPclDetachedSignature)==0)) 
		{ // Clearsign!
			PGPAppendOptionList(opts,
				PGPOClearSign(context, TRUE),
				PGPOLastOption(context));
		}
		else if(!(ms->Operation==MS_ENCRYPTCLIPBOARD) && 
			(ms->Sign) && !(ms->Encrypt) && 
			((prds->dwOptions & kPGPclDetachedSignature)==0)&&
		  FileHasThisExtension(OperationTarget,"txt")) 
		{ // Clearsign a .txt file
			DataIsASCII=TRUE; // Set to ASCII input

			PGPAppendOptionList(opts,
				PGPOClearSign(context, TRUE),
				PGPOLastOption(context));
		} 
		else // Just Ascii Armor*/
		{
			PGPAppendOptionList(opts,
				PGPOArmorOutput(context, TRUE),
				PGPOLastOption(context));
		}
	}

	if(*(ms->comment)!=0)
	{
		PGPAppendOptionList(opts,
			PGPOCommentString(context,(ms->comment)),
			PGPOLastOption(context));
	}

	if(ms->Operation!=MS_ENCRYPTCLIPBOARD)
		PGPscProgressSetNewFilename(ms->hwndWorking,"From '%s' To '%s'",ms->fileName,TRUE);

	if(!(ms->Operation==MS_ENCRYPTCLIPBOARD))
	{
		char outname[MAX_PATH];

		ms->fileRef=0;
		strcpy(outname,ms->fileName);

		AlterEncryptedFileName(outname,prds->dwOptions);

		LoadString (g_hinst, IDS_ENCFILENAMEPROMPT, StrRes, sizeof(StrRes));

		UserCancel=SaveOutputFile(context,
			ms->hwndWorking, 
			StrRes,
			outname,
			&(ms->fileRef),
			FALSE);

		if(UserCancel)
			return kPGPError_UserAbort;

		PGPAppendOptionList(opts,
			PGPOOutputFile(context,ms->fileRef),
			PGPOLastOption(context));
	}

	// Assume False for For Your Eyes Only
	FYEO=FALSE;

	// These are the only situations we can have FYEO
	if((ms->Operation==MS_ENCRYPTCLIPBOARD) ||
		(FileHasThisExtension(OperationTarget,"txt")))
	{
		// If the recipient dialog has selected it then we're OK
		if((prds->dwOptions & kPGPclSecureViewer)==kPGPclSecureViewer)
		{
			FYEO=TRUE;
		}
	}

	err = PGPEncode(context,
		opts,
		PGPOPreferredAlgorithms(context,ms->allowedAlgs,ms->nNumAlgs),
		PGPOSendNullEvents(context,75),
		PGPOEventHandler(context,myEvents,ms),
		PGPOVersionString(context,pgpVersionHeaderString()),
		PGPOAskUserForEntropy(context, TRUE),
		PGPODataIsASCII(context,DataIsASCII),
		PGPOForYourEyesOnly(context,FYEO),
		PGPOLastOption(context));

	if(recipientSet!=NULL)
		PGPFreeKeySet(recipientSet);

	PGPscProgressSetBar(ms->hwndWorking,100,TRUE);	

	AddKeys(ms->hwndWorking,prds->keydbAdded,ms);

	return err;
}

PGPError EncryptFileListStub (MYSTATE *ms) 
{
	PGPContextRef context;
	PGPFileSpecRef inref;
	PGPOptionListRef opts;
	PGPError err;
	FILELIST *FileCurrent;
	UINT WipeReturn; 

	err=kPGPError_NoErr;

	context=ms->context;
	FileCurrent=ms->ListHead;

	while(!(PGPscProgressGetCancel(ms->hwndWorking))&&(FileCurrent!=0)&&(IsntPGPError(err)))
	{   
		if(FileCurrent->IsDirectory)
		{
			if(ms->prds->dwOptions & kPGPclWipeOriginal)
				_rmdir(FileCurrent->name);

			FileCurrent=FileCurrent->next;
			continue;
		}

		// We'll ask for output file in event handler later...
		PGPNewFileSpecFromFullPath( context,
			FileCurrent->name, &inref);

		PGPBuildOptionList(context,&opts,
			PGPOInputFile(context,inref),
			PGPOLastOption(context));
	
		// For SDA's we only have to do this once, so don't
		// do it again on the second+ times
		if(!ms->bDoingSDA)
		{
			err=GenericEncSign(ms,opts,FileCurrent->name);
		}

		PGPFreeOptionList(opts);

		PGPFreeFileSpec( inref );

		if(ms->fileRef)
			PGPFreeFileSpec(ms->fileRef);

		ms->numDone++;
		FileCurrent=FileCurrent->next;				
	}

	// Do any file wiping on a second pass
	if((IsntPGPError(err))&&(ms->prds->dwOptions & kPGPclWipeOriginal))
	{
		PGPscProgressSetAVI(ms->hwndWorking,IDR_ERASEAVI);
		PGPscProgressSetTitle(ms->hwndWorking,"Wiping File(s)...");

		FileCurrent=ms->ListHead;
		
		while(!(PGPscProgressGetCancel(ms->hwndWorking))&&(FileCurrent!=0)&&(IsntPGPError(err)))
		{   
			if(FileCurrent->IsDirectory)
			{
				// Try to remove directory if we can. Don't
				// worry if we fail (full of new files)
				if(ms->prds->dwOptions & kPGPclWipeOriginal)
					_rmdir(FileCurrent->name);

				FileCurrent=FileCurrent->next;
				continue;
			}

			WipeReturn=WipeStreams(ms->context,
				ms->hwndWorking,
				FileCurrent->name,WIPE_DEFAULT,0);

			if(WipeReturn!=WIPE_OK)
			{
				if (WipeError(ms->hwndWorking,
					FileCurrent,WipeReturn))
				{
					err=kPGPError_UnknownError;
				}
			}
			else
			{
				WipeReturn=InternalFileWipe(ms->context,
					ms->hwndWorking,
					FileCurrent->name,WIPE_DEFAULT,0,0);

				if(WipeReturn!=WIPE_OK)
				{
					if (WipeError(ms->hwndWorking,
						FileCurrent,WipeReturn))
					{
						err=kPGPError_UnknownError;
					}
				}
			}

			FileCurrent=FileCurrent->next;
		}
	}

	return err;
}

BOOL PGPscEncryptFileList(HWND hwnd,char *szApp,
					 void *PGPsc,
					 void *PGPtls,
					 FILELIST *ListHead,
					 BOOL bEncrypt, 
					 BOOL bSign)

{
	PGPContextRef context;
	PPGPclRecipientDialogStruct prds;
	MYSTATE *ms;
	char comment[80];
	PGPUInt32 PrefAlg;
	PGPCipherAlgorithm allowedAlgs[20];
	int nNumAlgs;
	PGPtlsContextRef tls;
	PGPError err;
	char StrRes[500];

	err=kPGPError_NoErr;

	context=(PGPContextRef)PGPsc;
	tls=(PGPtlsContextRef)PGPtls;

///	if(IsPGPError(PGPlnIsExpired(hwnd, kPGPlnEncryptSignExpiration)))
///		return FALSE;

	if(!GetCommentLine(context,comment,sizeof(comment),&PrefAlg,
		allowedAlgs, &nNumAlgs))
		return FALSE;

	prds=(PPGPclRecipientDialogStruct)
		malloc(sizeof(PGPclRecipientDialogStruct));

	if(prds)
	{
		memset(prds,0x00,sizeof(PGPclRecipientDialogStruct));

		LoadString (g_hinst, IDS_PGPKEYSELDLG, StrRes, sizeof(StrRes));

		prds->szTitle=StrRes;
		prds->dwDisableFlags=kPGPclDisableAutoMode;
		prds->dwOptions=kPGPclDetachedSignature;

		ms=(MYSTATE *)malloc(sizeof(MYSTATE));

		if(ms)
		{
			memset(ms, 0x00, sizeof(MYSTATE) );

			ms->context=context;
			ms->tlsContext=tls;
			ms->ListHead=ListHead;
			ms->Operation=MS_ENCRYPTFILELIST;
			ms->prds=prds;
			ms->Encrypt=bEncrypt; 
			ms->Sign=bSign;
			ms->comment=comment;
			ms->PrefAlg=PrefAlg;
			ms->allowedAlgs=allowedAlgs; 
			ms->nNumAlgs=nNumAlgs;
			ms->szAppName=szApp;

			if(OpenRings(hwnd,context,&(ms->KeyDB),&(ms->bMutable)))
			{
				err=PGPscProgressDialog(hwnd,DoWorkThread,ms,
						  0,"Encoding File(s)...",
						  "","",IDR_PROGAVI);

				if(prds->keydbSelected)
					PGPFreeKeyDB(prds->keydbSelected);

				FreePhrases(ms);

				PGPFreeKeyDB(ms->KeyDB);
			}
			free(ms);
		}
		free(prds);
	}
	
	PGPscFreeFileList(ListHead);

	if(IsPGPError(err))
		return FALSE;

	return TRUE;
}

PGPError CreateSDAFileListStub (MYSTATE *ms) 
{
	PGPError err;
	BOOL UserCancel;
	char StrRes[256];
	char szPassTitle[256];
	PGPUInt32 iMinPhraseLength;
	PGPUInt32 iMinPhraseQuality;

	iMinPhraseLength=iMinPhraseQuality=0;
	
	// minimum passphrase length
	if(PGPlnEnterprise())
	{
		PGPError prefErr;
		PGPBoolean b;
		PGPPrefRef prefs;
		
		prefErr = PGPclPeekClientLibPrefRefs (&prefs, NULL);
		
		if(IsntPGPError(prefErr))
		{
			prefErr = PGPGetPrefBoolean (prefs, kPGPPrefEnforceMinChars, &b);
			
			if (IsntPGPError(prefErr)&&(b)) 
			{
				PGPGetPrefNumber (prefs, kPGPPrefMinChars, &iMinPhraseLength);
			}
			
			// minimum passphrase quality
			prefErr = PGPGetPrefBoolean (prefs, kPGPPrefEnforceMinQuality, &b);
			
			if(IsntPGPError(prefErr)&&(b)) 
			{
				PGPGetPrefNumber (prefs, kPGPPrefMinQuality, &iMinPhraseQuality);
			}
		}
	}

	if(iMinPhraseLength<1)
		iMinPhraseLength=1;
	
	LoadString (g_hinst, IDS_ENTERPASSPHRASE, StrRes, sizeof(StrRes));
	strcpy(szPassTitle,ms->szAppName);
	strcat(szPassTitle,StrRes);

	LoadString (g_hinst, IDS_ENTERSDAPASS, StrRes, sizeof(StrRes));

	UserCancel = PGPclGetPhrase (ms->context,
		ms->KeyDB,
		ms->hwndWorking,
		StrRes,
		&(ms->ConvPassPhrase), 
		NULL,
		NULL, 
		0,
		NULL,
		NULL,
		kPGPclEncryption,
		NULL,NULL,
		iMinPhraseLength,
		iMinPhraseQuality,
		ms->tlsContext,NULL,szPassTitle);

	if(UserCancel)
		return kPGPError_UserAbort;

	err=SDA(ms);

	return err;
}

BOOL PGPscCreateSDAFileList(HWND hwnd,char *szApp,
					 void *PGPsc,
					 void *PGPtls,
					 FILELIST *ListHead)
{
	PGPContextRef context;
	MYSTATE *ms;
	PGPtlsContextRef tls;
	PGPError err;
	PGPBoolean bAllowConvEncrypt;
	PGPPrefRef prefRef;

	err=kPGPError_NoErr;

	context=(PGPContextRef)PGPsc;
	tls=(PGPtlsContextRef)PGPtls;

///	if(IsPGPError(PGPlnIsExpired(hwnd, kPGPlnEncryptSignExpiration)))
///		return FALSE;

	err=PGPclPeekClientLibPrefRefs(&prefRef,NULL);
	if (IsPGPError (err))
		return FALSE;

	// Allow Conventional
	PGPGetPrefBoolean(prefRef,
		kPGPPrefAllowConventionalEncryption,
		&(bAllowConvEncrypt));

	if(!bAllowConvEncrypt)
	{
		PGPscMessageBox (hwnd,IDS_PGPERROR,IDS_SDANOTALLOWED,
			MB_OK|MB_ICONSTOP);
		return FALSE;
	}

	ms=(MYSTATE *)malloc(sizeof(MYSTATE));

	if(ms)
	{
		memset(ms, 0x00, sizeof(MYSTATE) );

		ms->context=context;
		ms->tlsContext=tls;
		ms->ListHead=ListHead;
		ms->Operation=MS_CREATESDAFILELIST;
		ms->szAppName=szApp;

		err=PGPscProgressDialog(hwnd,DoWorkThread,ms,
						  0,"Encoding File(s)...",
						  "","",IDR_PROGAVI);

		FreePhrases(ms);

		free(ms);
	}
	
	PGPscFreeFileList(ListHead);

	if(IsPGPError(err))
		return FALSE;

	return TRUE;
}

BOOL WeNeedToWordWrap(PGPContextRef context,
					  unsigned short* WordWrapThreshold)
{
	PGPBoolean	bWrapReturn=TRUE;
	PGPUInt32	WrapWidth=72;
	PGPPrefRef	prefRef;
	PGPError err;

	err=PGPclPeekClientLibPrefRefs(&prefRef,NULL);

	if(IsntPGPError(err))
	{
		PGPGetPrefBoolean (prefRef,
			kPGPPrefWordWrapEnable, &bWrapReturn);	
		PGPGetPrefNumber (prefRef, 
			kPGPPrefWordWrapWidth, &WrapWidth);
	}

	*WordWrapThreshold=WrapWidth;

	return bWrapReturn;
}

BOOL WrapBuffer(char **pszOutText,
				char *szInText,
				PGPUInt16 wrapLength)
{
	BOOL RetVal = FALSE;
	PGPError err;
	char *cmdlgBuffer;

	err=PGPclWrapBuffer(
					szInText,
					wrapLength,
					&cmdlgBuffer);

	if(IsntPGPError (err))
	{
		int memamt,length;

		length=strlen(cmdlgBuffer);
		memamt=length+1;

		*pszOutText=(char *)malloc(memamt);
		memcpy(*pszOutText,cmdlgBuffer,length);
		(*pszOutText)[length]=0;
		PGPclFreeWrapBuffer(cmdlgBuffer);
		RetVal = TRUE;
	}

	return RetVal;
}


BOOL WrapIfNecessary(PGPContextRef context,
					 char **pszClipText,DWORD *dwClipSize)
{
	unsigned short WordWrapThreshold;
	char* szWrappedText;
	DWORD dwWrappedSize;

	if(WeNeedToWordWrap(context,&WordWrapThreshold))
	{
		if(TRUE == WrapBuffer(&szWrappedText, 
			*pszClipText,WordWrapThreshold))
		{	
			dwWrappedSize = strlen(szWrappedText);

			memset(*pszClipText,0x00,*dwClipSize);
			free(*pszClipText);
			*pszClipText=szWrappedText;

			*dwClipSize=dwWrappedSize;
		}
	}

	return TRUE;
}

PGPError EncryptClipboardStub (MYSTATE *ms) 
{
	PGPOptionListRef opts;
	PGPContextRef context;
	PGPError err;
	char StrRes[100];

	err=kPGPError_NoErr;

	context=ms->context;

	PGPBuildOptionList(context,&opts,
		PGPOInputBuffer(context,ms->pInput,ms->dwInputSize),
		PGPOAllocatedOutputBuffer(context,
			&(ms->pOutput),MAX_PGPSize ,&(ms->dwOutputSize)),
		PGPOLastOption(context));

	LoadString (g_hinst, IDS_CLIPBOARD, StrRes, sizeof(StrRes));

	err=GenericEncSign(ms,opts,StrRes);

	PGPFreeOptionList(opts);

	return err;
}

BOOL PGPscEncryptClipboard(HWND hwnd,char *szApp,void *PGPsc,void *PGPtls,BOOL Encrypt,BOOL Sign,BOOL bStartingCR)
{
	PGPContextRef context;
	PPGPclRecipientDialogStruct prds;
	char *pInput;
	DWORD dwInputSize;
	UINT ClipboardFormat;
	char comment[80];
	PGPUInt32 PrefAlg;
	PGPCipherAlgorithm allowedAlgs[20];
	int nNumAlgs;
	MYSTATE *ms;
	PGPtlsContextRef tls;
	PGPError err;
	char StrRes[500];

	// Check for files copied into clipboard from explorer
	if(OpenClipboard(hwnd)) 
	{
		if(IsClipboardFormatAvailable(CF_HDROP))
		{
			FILELIST *ListHead;
			HDROP hDrop;

			hDrop=(HDROP)GetClipboardData(CF_HDROP);
			ListHead=PGPscHDropToFileList(hDrop);

			if(ListHead!=0)
			{
				CloseClipboard();

				return PGPscEncryptFileList(hwnd,szApp,
					 PGPsc,PGPtls,ListHead,
					 Encrypt,Sign);
			}
		}
		CloseClipboard();
	}

	err=kPGPError_NoErr;

	context=(PGPContextRef)PGPsc;
	tls=(PGPtlsContextRef)PGPtls;

///	if(IsPGPError(PGPlnIsExpired(hwnd, kPGPlnEncryptSignExpiration)))
///		return FALSE;

	if(!GetCommentLine(context,comment,sizeof(comment),&PrefAlg,
		allowedAlgs, &nNumAlgs))
		return FALSE;

	pInput=RetrieveClipboardData(hwnd, &ClipboardFormat, 
			                     &dwInputSize);

	if(!pInput)
	{
		PGPscMessageBox (hwnd,IDS_PGPERROR,IDS_NOCLIPBOARDCONTENTS,
					MB_OK|MB_ICONSTOP);
	}
	else
	{
		if((ClipboardFormat != CF_TEXT)||(*pInput==0))
		{
			PGPscMessageBox (hwnd,IDS_PGPERROR,IDS_NOCLIPBOARDTEXT,
				MB_OK|MB_ICONSTOP);
		}
		else
		{
			// Don't word wrap if we are encrypting
			if(!Encrypt)
				WrapIfNecessary(context,&pInput,&dwInputSize);

			prds=(PPGPclRecipientDialogStruct)
				malloc(sizeof(PGPclRecipientDialogStruct));

			if(prds)
			{
				memset(prds,0x00,sizeof(PGPclRecipientDialogStruct));

				LoadString (g_hinst, IDS_PGPKEYSELDLG, StrRes, sizeof(StrRes));

				prds->szTitle=StrRes;
				prds->dwOptions=kPGPclASCIIArmor;
				prds->dwDisableFlags=kPGPclDisableASCIIArmor |
					kPGPclDisableWipeOriginal |
					kPGPclDisableAutoMode |
					kPGPclDisableInputIsText;

				ms=(MYSTATE *)malloc(sizeof(MYSTATE));
	
				if(ms)
				{
					memset(ms, 0x00, sizeof(MYSTATE) );

					ms->context=context;
					ms->tlsContext=tls;
					ms->pInput=pInput;
					ms->dwInputSize=dwInputSize;
					ms->Operation=MS_ENCRYPTCLIPBOARD;
					ms->prds=prds;
					ms->Encrypt=Encrypt; 
					ms->Sign=Sign;
					ms->comment=comment;
					ms->PrefAlg=PrefAlg;
					ms->allowedAlgs=allowedAlgs; 
					ms->nNumAlgs=nNumAlgs;
					ms->szAppName=szApp;
					ms->bStartingCR=bStartingCR;

					if(OpenRings(hwnd,context,&(ms->KeyDB),&(ms->bMutable)))
					{	
						err=PGPscProgressDialog(hwnd,DoWorkThread,ms,
							  0,"Encoding Clipboard...",
							  "","",IDR_PROGAVI);
	
						if(prds->keydbSelected)
							PGPFreeKeyDB(prds->keydbSelected);

						FreePhrases(ms);
				
						if((ms->pOutput!=0) && (IsntPGPError(err)))
						{
							if(ms->bStartingCR)
							{
								char *pCR;

								pCR=PGPNewData(PGPPeekContextMemoryMgr (ms->context),
									ms->dwOutputSize+2,0);

								if(pCR)
								{
									strcpy(pCR,"\r\n");
									memcpy(&(pCR[2]),ms->pOutput,ms->dwOutputSize);

									StoreClipboardData(hwnd, pCR, ms->dwOutputSize+2);

									memset(pCR,0x00,ms->dwOutputSize+2);

									PGPFreeData(pCR);
								}
							}
							else
							{
								StoreClipboardData(hwnd, ms->pOutput, ms->dwOutputSize);
							}

							memset(ms->pOutput,0x00,ms->dwOutputSize);
							PGPFreeData(ms->pOutput); // Since auto alloced by CDK
						}

						PGPFreeKeyDB(ms->KeyDB);
					}
					free(ms);
				}
				free(prds);
			}
		}
		memset(pInput,0x00,dwInputSize);
		free(pInput);
	}

	if(IsPGPError(err))
		return FALSE;

	return TRUE;
}

BOOL PGPscEncryptBuffer(HWND hwnd,char *szApp,void *PGPsc,void *PGPtls,
				   BOOL Encrypt,BOOL Sign,
				   char *pInput,DWORD dwInputSize,
				   char **pOutput,DWORD *pdwOutputSize,
				   char **szRecipientArray,DWORD dwNumRecipients,
				   BOOL bForce)
{
	PGPContextRef context;
	PPGPclRecipientDialogStruct prds;
	char comment[80];
	PGPUInt32 PrefAlg;
	PGPCipherAlgorithm allowedAlgs[20];
	int nNumAlgs;
	MYSTATE *ms;
	PGPtlsContextRef tls;
	PGPError err;
	char StrRes[500];

	err=kPGPError_NoErr;

	context=(PGPContextRef)PGPsc;
	tls=(PGPtlsContextRef)PGPtls;

///	if(IsPGPError(PGPlnIsExpired(hwnd, kPGPlnEncryptSignExpiration)))
///		return FALSE;

	if(!GetCommentLine(context,comment,sizeof(comment),&PrefAlg,
		allowedAlgs, &nNumAlgs))
		return FALSE;

//	WrapIfNecessary(context,&pInput,&dwInputSize);

	prds=(PPGPclRecipientDialogStruct)
		malloc(sizeof(PGPclRecipientDialogStruct));

	if(prds)
	{
		memset(prds,0x00,sizeof(PGPclRecipientDialogStruct));

		LoadString (g_hinst, IDS_PGPKEYSELDLG, StrRes, sizeof(StrRes));

		prds->szTitle=StrRes;
		prds->dwOptions=kPGPclASCIIArmor;
		prds->dwDisableFlags=kPGPclDisableASCIIArmor |
			kPGPclDisableWipeOriginal |
			kPGPclDisableInputIsText;

		if(bForce)
			prds->dwDisableFlags|=kPGPclDisableAutoMode;

		prds->dwNumRecipients	= dwNumRecipients;	
		prds->szRecipientArray	= szRecipientArray;

		ms=(MYSTATE *)malloc(sizeof(MYSTATE));
	
		if(ms)
		{
			memset(ms, 0x00, sizeof(MYSTATE) );

			ms->context=context;
			ms->tlsContext=tls;
			ms->pInput=pInput;
			ms->dwInputSize=dwInputSize;
			ms->Operation=MS_ENCRYPTCLIPBOARD;
			ms->prds=prds;
			ms->Encrypt=Encrypt; 
			ms->Sign=Sign;
			ms->comment=comment;
			ms->PrefAlg=PrefAlg;
			ms->allowedAlgs=allowedAlgs; 
			ms->nNumAlgs=nNumAlgs;
			ms->szAppName=szApp;

			if(OpenRings(hwnd,context,&(ms->KeyDB),&(ms->bMutable)))
			{	
				err=PGPscProgressDialog(hwnd,DoWorkThread,ms,
					  0,"Encoding...",
					  "","",IDR_PROGAVI);
	
				if(prds->keydbSelected)
					PGPFreeKeyDB(prds->keydbSelected);

				FreePhrases(ms);
				
				if((ms->pOutput!=0) && (IsntPGPError(err)))
				{
					*pOutput=ms->pOutput;
					*pdwOutputSize=ms->dwOutputSize;
				}

				PGPFreeKeyDB(ms->KeyDB);
			}
			free(ms);
		}
		free(prds);
	}

	if(IsPGPError(err))
		return FALSE;

	return TRUE;
}


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
