/*__________________________________________________________________________
 Copyright (C) 2002 PGP Corporation
 All rights reserved.
 
 $Id: SCwfNT.c,v 1.15 2002/11/16 08:55:23 wjb Exp $
__________________________________________________________________________*/
#include "precomp.h"

#define RETRIES 1

//
// Invalid longlong number
//
#define LLINVALID		((ULONGLONG) -1)

//
// Size of the buffer we read file mapping information into.
// The buffer is big enough to hold the 16 bytes that 
// come back at the head of the buffer (the number of entries 
// and the starting virtual cluster), as well as 512 pairs
// of [virtual cluster, logical cluster] pairs.
//
#define	FILEMAPSIZE		(512+2)

//
// Size of the bitmap buffer we pass in. Its large enough to
// hold information for the 16-byte header that's returned
// plus the indicated number of bytes, each of which has 8 bits 
// (imagine that!)
//
#define BITMAPBYTES		4096
#define BITMAPSIZE		(BITMAPBYTES+2*sizeof(ULONGLONG))

//
// Bit shifting array for efficient processing of the bitmap
//
BYTE		BitShift[] = { 1, 2, 4, 8, 16, 32, 64, 128 };

// StatusToError
//
// Takes the status messages and turns them into our
// native wipe free codes.
DWORD StatusToError( NTSTATUS Status )
{
	switch( Status ) 
	{
	case STATUS_SUCCESS:
		return WFE_NOERROR;
	case STATUS_INVALID_PARAMETER:
		return WFE_INVALPARAM; // Must be the metafile
	case STATUS_BUFFER_TOO_SMALL:
		return WFE_OUTOFMEMORY;
	case STATUS_ALREADY_COMMITTED:
	case STATUS_DELETE_PENDING:
		return WFE_ALREADYCOMMIT; // NTFS won't let us. Try anyway
	case STATUS_INVALID_DEVICE_REQUEST:
		return WFE_DRIVENOTSUPPORTED;
	case STATUS_DEVICELOCKED:
		return WFE_COULDNOTLOCK;
	case ERROR_ACCESS_DENIED:
		return WFE_ACCESSDENIED;
	default:
		/*
#if _DEBUG
		{
			char szWarn[100];

			sprintf(szWarn,"Status Error %d",Status);
			MessageBox(NULL,szWarn,"DEBUG INFO ONLY",MB_OK);
		}
#endif // _DEBUG
		*/
		return WFE_CLUSTERERROR;
	}		  
}

// OpenVolume
//
// Grabs the handle of our logical volume
DWORD OpenVolume( VOLINFO *vi ) 
{
	static char			volumeName[] = "\\\\.\\A:";

	//
	// open the volume
	//
	volumeName[4] = (char)vi->vol0 + 'A'; 
	vi->VolHandle = CreateFile( volumeName, GENERIC_READ, 
					FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 
					0, 0 );
	if( vi->VolHandle == INVALID_HANDLE_VALUE )	{

		return GetLastError();
	}

	return ERROR_SUCCESS;
}


// WipeCluster
//
// WipeCluster uses the patterns stored in filename to 
// eradicate the data in the target cluster of the logical
// drive specified.

DWORD ClusterNTWipeCluster( HANDLE sourceFile,VOLINFO *vi, ULONGLONG Vcn, ULONGLONG target,DWORD numClusters)
{
	DWORD						status;
	IO_STATUS_BLOCK				ioStatus;
	LARGE_INTEGER				startVcn, targetLcn;
	MOVEFILE_DESCRIPTOR			moveFile;

	DWORD RetVal;

	RetVal=WFE_NOERROR;

	if(numClusters>16)
	{
		int i;

		i=0;
	}

	// 
	// Get numeric parameters
	//
	startVcn.QuadPart=Vcn;
	targetLcn.QuadPart=target;

	//
	// Setup movefile descriptor and make the call
	//
	memset(&moveFile,0x00,sizeof(MOVEFILE_DESCRIPTOR));
	memset(&ioStatus,0x00,sizeof(IO_STATUS_BLOCK));

	moveFile.FileHandle = sourceFile;
	moveFile.StartVcn = startVcn;
	moveFile.TargetLcn = targetLcn;
	moveFile.NumVcns = numClusters;

	status = NtFsControlFile( vi->VolHandle, NULL, NULL, 0, &ioStatus,
						FSCTL_MOVE_FILE,
						&moveFile, sizeof( moveFile ),
						NULL, 0 );

	RetVal=StatusToError(status);

	// 
	// If the operation is pending, wait for it to finish
	//
	if( status == STATUS_PENDING ) 
	{
		WaitForSingleObject( sourceFile, INFINITE ); 

		RetVal=StatusToError(ioStatus.Status);
	}

	if(status!=STATUS_SUCCESS)
	{
		int i;

		i=0;
	}
	
//	// Handle our progress bars and look out for user cancel
//	vi->dwClustersWritten=vi->dwClustersWritten+1;
		
//	if(vi->dwFS==FS_NTFS)
		// NTFS works with 16 cluster segments
//		vi->dwClustersWritten=vi->dwClustersWritten+15; 

	return RetVal;
}

// WipeFreeClusters
//
// Given a logical drive number, routine iterates through free
// clusters and calls WipeCluster to zap the information stored
// there. Note that if changes take place on the drive while
// routine is executes, some clusters may be missed.

DWORD ClusterWipeFreeClusters( VOLINFO *vi, BOOL bWipe)
{
	DWORD						status,wipestatus;
	PBITMAP_DESCRIPTOR			bitMappings;
	ULONGLONG					startLcn;
	ULONGLONG					nextLcn;
	IO_STATUS_BLOCK				ioStatus;
	ULONGLONG					i;
	DWORD						numClusters;

	wipestatus=WFE_NOERROR;

	if(bWipe)
		StatusMessage(vi,"Wiping free space with cluster move method...", FALSE);
	else
		StatusMessage(vi,"Counting clusters for slack wipe...", FALSE);

	bitMappings=(PBITMAP_DESCRIPTOR)malloc(BITMAPSIZE*sizeof(BYTE)+1);

	//
	// Start scanning at the cluster offset the user specifies
	//
	nextLcn = 0;

	while( !(status = NtFsControlFile( vi->VolHandle, NULL, NULL, 0, &ioStatus,
						FSCTL_GET_VOLUME_BITMAP,
						&nextLcn, sizeof( ULONGLONG ),
						bitMappings, BITMAPSIZE )) ||
			 status == STATUS_BUFFER_OVERFLOW ||
			 status == STATUS_PENDING ) 
	{

		// 
		// If the operation is pending, wait for it to finish
		//
		if( status == STATUS_PENDING ) 
		{
			
			WaitForSingleObject( vi->VolHandle, INFINITE );
			
			//
			// Get the status from the status block
			//
			if( ioStatus.Status != STATUS_SUCCESS && 
				ioStatus.Status != STATUS_BUFFER_OVERFLOW ) 
			{
				free(bitMappings);

				// Error out
				return StatusToError(ioStatus.Status);
			}
		}

		//
		// Scan through the returned bitmap info, looking for empty clusters
		//
		startLcn = bitMappings->StartLcn;
	
		for( i = 0; i < min( bitMappings->ClustersToEndOfVol, 8*BITMAPBYTES); i++) 
		{
			numClusters=0;

			if(vi->bCancel)
			{
				free(bitMappings);

				return WFE_USERCANCEL;
			}

			if( !(bitMappings->Map[ i/8 ] & BitShift[ i % 8 ])) 
			{
				for(numClusters=1;
					((numClusters<16)&&((i+numClusters)<min( bitMappings->ClustersToEndOfVol, 8*BITMAPBYTES)));
					numClusters++)
				{
					if((bitMappings->Map[ (i+numClusters)/8 ] & BitShift[ (i+numClusters) % 8 ])) 
					{
						break;
					}
				}

				if(bWipe)
				{
					wipestatus=ClusterNTWipeCluster(vi->hWipe,vi, 0, startLcn + i,numClusters);
					vi->dwClustersWritten=vi->dwClustersWritten+numClusters;
				}

				if(wipestatus==WFE_NOERROR)
				{
					// Save this cluster for later slack movement operations
					if((vi->SlackBufferIndex<SLACKBUFFER)&&((vi->dwFS!=FS_NTFS)||(numClusters==16)))
					{
						vi->SlackBuffer[vi->SlackBufferIndex]=startLcn + i;
						vi->SlackBufferIndex=vi->SlackBufferIndex+1;
					}
				}

				i=i+numClusters-1;

				if(((wipestatus==WFE_ALREADYCOMMIT)||(wipestatus==WFE_INVALPARAM))&&(vi->dwFS==FS_NTFS))
				{
					// Couldn't do the move because of MFT, or user is mucking
					// around.... just skip it and we'll get it on the next pass.
					// Only applies to NTFS
					wipestatus=WFE_NOERROR;
				}

				if(wipestatus!=WFE_NOERROR)
				{
					free(bitMappings);
					return wipestatus;
				}
			} 

		}
	
		//
		// End of volume?
		//
		if( status != STATUS_BUFFER_OVERFLOW ) 
		{
			free(bitMappings);
			return StatusToError(STATUS_SUCCESS);
		}				

		//
		// Move to the next block
		//
		nextLcn = bitMappings->StartLcn + i;
	}

	//
	// We only get here when there's an error
	//
	free(bitMappings);

	return StatusToError(status);
}

DWORD ClusterCountFreeClusters( VOLINFO *vi)
{
	DWORD						status;
	PBITMAP_DESCRIPTOR			bitMappings;
	ULONGLONG					startLcn;
	ULONGLONG					nextLcn;
	IO_STATUS_BLOCK				ioStatus;
	ULONGLONG					i;

	StatusMessage(vi,"Counting free clusters...", FALSE);

	bitMappings=(PBITMAP_DESCRIPTOR)malloc(BITMAPSIZE*sizeof(BYTE)+1);

	if(bitMappings==NULL)
		return WFE_OUTOFMEMORY;

	//
	// Start scanning at the cluster offset the user specifies
	//
	nextLcn = 0;

	while( !(status = NtFsControlFile( vi->VolHandle, NULL, NULL, 0, &ioStatus,
						FSCTL_GET_VOLUME_BITMAP,
						&nextLcn, sizeof( ULONGLONG ),
						bitMappings, BITMAPSIZE )) ||
			 status == STATUS_BUFFER_OVERFLOW ||
			 status == STATUS_PENDING ) 
	{

		// 
		// If the operation is pending, wait for it to finish
		//
		if( status == STATUS_PENDING ) 
		{
			
			WaitForSingleObject( vi->VolHandle, INFINITE );
			
			//
			// Get the status from the status block
			//
			if( ioStatus.Status != STATUS_SUCCESS && 
				ioStatus.Status != STATUS_BUFFER_OVERFLOW ) 
			{
				free(bitMappings);

				// Error out
				return StatusToError(ioStatus.Status);
			}
		}

		//
		// Scan through the returned bitmap info, looking for empty clusters
		//
		startLcn = bitMappings->StartLcn;
	
		for( i = 0; i < min( bitMappings->ClustersToEndOfVol, 8*BITMAPBYTES); i++) 
		{
			if(vi->bCancel)
			{
				free(bitMappings);
				return WFE_USERCANCEL;
			}

			if( !(bitMappings->Map[ i/8 ] & BitShift[ i % 8 ])) 
			{
				vi->dwClusterCount++;
			} 
		}
	
		//
		// End of volume?
		//
		if( status != STATUS_BUFFER_OVERFLOW ) 
		{
			free(bitMappings);
			return StatusToError(STATUS_SUCCESS);
		}				

		//
		// Move to the next block
		//
		nextLcn = bitMappings->StartLcn + i;
	}

	//
	// We only get here when there's an error
	//
	free(bitMappings);

	return StatusToError(status);
}

// GetLastClusterOfFile
//
// Loops through till it finds the last cluster in the chain

DWORD GetLastClusterOfFile(VOLINFO *vi, char *fileName, SLACKMOVE *sm)
{
	DWORD						status;
	int							i;
	HANDLE						sourceFile;
	IO_STATUS_BLOCK				ioStatus;
	ULONGLONG					startVcn;
	PGET_RETRIEVAL_DESCRIPTOR	fileMappings;
	int							lines = 0;

	// Save name of file for later
	sm->name=fileName;
	sm->Orig=sm->Vcn=sm->Size=0;

	//
	// Open the file
	//
	sourceFile = CreateFile( fileName, GENERIC_READ, 
					FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 
					FILE_FLAG_NO_BUFFERING, 0 );

	if( sourceFile == INVALID_HANDLE_VALUE ) 
		return WFE_COULDNOTREAD;

	//
	// Start dumping the mapping information. Go until we hit the end of the
	// file.
	//
	startVcn = 0;

	fileMappings = (PGET_RETRIEVAL_DESCRIPTOR) 
		malloc(FILEMAPSIZE*sizeof(ULONGLONG));

	while( !(status = NtFsControlFile( sourceFile, NULL, NULL, 0, &ioStatus,
						FSCTL_GET_RETRIEVAL_POINTERS,
						&startVcn, sizeof( startVcn ),
						fileMappings, FILEMAPSIZE * sizeof(LARGE_INTEGER) ) ) ||
			 status == STATUS_BUFFER_OVERFLOW ||
			 status == STATUS_PENDING ) 
	{
		// 
		// If the operation is pending, wait for it to finish
		//
		if( status == STATUS_PENDING ) {
			
			WaitForSingleObject( sourceFile, INFINITE ); 

			//
			// Get the status from the status block
			//
			if( ioStatus.Status != STATUS_SUCCESS && 
				ioStatus.Status != STATUS_BUFFER_OVERFLOW ) 
			{
				free(fileMappings);
				// Error out
				return StatusToError(ioStatus.Status);
			}
		}

		//
		// Loop through the buffer of number/cluster pairs, printing them
		// out.
		//
		startVcn = fileMappings->StartVcn;

		for( i = 0; i < (ULONGLONG) fileMappings->NumberOfPairs; i++ ) 
		{
			//
			// On NT 4.0, a compressed virtual run (0-filled) is 
			// identified with a cluster offset of -1
			//
			if( fileMappings->Pair[i].Lcn == LLINVALID ) 
			{
				// Ignore virtual runs since OS is supposed to fill slack
				// with zeros
			} 
			else 
			{
				// This gives us the last cluster Vcn (First+Length-1)
				sm->Vcn=startVcn+(fileMappings->Pair[i].Vcn - startVcn )-1;
		
				if(vi->dwFS==FS_NTFS)
				{
					// We need to 16 cluster align it if NTFS
					sm->Vcn=(sm->Vcn/16)*16;
				}

				// Should always be < 16
				sm->Size=(ULONG)(fileMappings->Pair[i].Vcn - (sm->Vcn) );

				if(sm->Size>16)
				{
					sm->Size=16;
					// Really screwed up
				}

				// This gives us the last cluster Lcn (Start Location + offset)
				sm->Orig=fileMappings->Pair[i].Lcn+(sm->Vcn-startVcn);

				if(sm->Orig<fileMappings->Pair[i].Lcn)
				{
					int i;

					i=0;
				}
			}
			startVcn = fileMappings->Pair[i].Vcn;
		}

		//
		// If the buffer wasn't overflowed, then we're done
		//
		if( !status ) break;
	}

	CloseHandle( sourceFile );

	free(fileMappings);
	return StatusToError(status);
}

PGPError PGPscClusterFileGather(char *fileName,PCWF *cwflist)
{
	DWORD						status;
	int							i;
	HANDLE						sourceFile;
	IO_STATUS_BLOCK				ioStatus;
	ULONGLONG					startVcn;
	PGET_RETRIEVAL_DESCRIPTOR	fileMappings;
	int							lines = 0;
	PGPError					err;
	static char					volumeName[] = "\\\\.\\A:";
	HANDLE						hVolume;

	// Get the NtFsControlFile entry point since undocumented
	if( !(NtFsControlFile = (void *) GetProcAddress( GetModuleHandle("ntdll.dll"),
			"NtFsControlFile" )) )
	{
		return kPGPError_UnknownError;
	}

	//
	// open the volume
	//
	volumeName[4] = (char)'D';
	
	hVolume = CreateFile( volumeName, GENERIC_READ, 
					FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 
					0, 0 );
	if( hVolume == INVALID_HANDLE_VALUE )	
	{

		return kPGPError_UnknownError;
	}

	err=kPGPError_NoErr;

	//
	// Open the file
	//
	sourceFile = CreateFile( fileName, GENERIC_READ, 
					FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 
					FILE_FLAG_NO_BUFFERING, 0 );

	if( sourceFile == INVALID_HANDLE_VALUE ) 
	{
		err=kPGPError_CantOpenFile;
	}
	else
	{
		//
		// Start dumping the mapping information. Go until we hit the end of the
		// file.
		//
		startVcn = 0;

		fileMappings = (PGET_RETRIEVAL_DESCRIPTOR) 
			malloc(FILEMAPSIZE*sizeof(ULONGLONG));

		if(fileMappings==NULL)
		{
			err=kPGPError_OutOfMemory;
		}
		else
		{
			while( !(status = NtFsControlFile( sourceFile, NULL, NULL, 0, &ioStatus,
								FSCTL_GET_RETRIEVAL_POINTERS,
								&startVcn, sizeof( startVcn ),
								fileMappings, FILEMAPSIZE * sizeof(LARGE_INTEGER) ) ) ||
					 status == STATUS_BUFFER_OVERFLOW ||
					 status == STATUS_PENDING ) 
			{
				// 
				// If the operation is pending, wait for it to finish
				//
				if( status == STATUS_PENDING ) 
				{
					
					WaitForSingleObject( sourceFile, INFINITE ); 

					//
					// Get the status from the status block
					//
					if( ioStatus.Status != STATUS_SUCCESS && 
						ioStatus.Status != STATUS_BUFFER_OVERFLOW ) 
					{
						free(fileMappings);
						// Error out
						err=kPGPError_FileOpFailed;
						//dwError=StatusToError(ioStatus.Status);
						break;
					}
				}

				//
				// Loop through the buffer of number/cluster pairs, printing them
				// out.
				//
				startVcn = fileMappings->StartVcn;

				for( i = 0; i < (ULONGLONG) fileMappings->NumberOfPairs; i++ ) 
				{
					//
					// On NT 4.0, a compressed virtual run (0-filled) is 
					// identified with a cluster offset of -1
					//
					if( fileMappings->Pair[i].Lcn == LLINVALID ) 
					{
						// Ignore virtual runs since OS is supposed to fill slack
						// with zeros
					} 
					else 
					{
						PCWF cwf;

						cwf=(PCWF)malloc(sizeof(CWF));

						if(cwf==NULL)
						{
							err=kPGPError_NoErr;
							break;
						}
						else
						{
							memset(cwf,0x00,sizeof(CWF));

							// This gives us the last cluster Vcn (First+Length-1)
							cwf->Lcn=fileMappings->Pair[i].Lcn;
							cwf->Size=(ULONG)(fileMappings->Pair[i].Vcn - startVcn);
							cwf->next=(*cwflist);

							*cwflist=cwf;
						}
					}
					startVcn = fileMappings->Pair[i].Vcn;
				}

				//
				// If the buffer wasn't overflowed, then we're done
				//
				if( !status ) break;
			}
			free(fileMappings);
		}
		CloseHandle( sourceFile );
	}
	CloseHandle(hVolume);

	return err;
}

PGPError PGPscClusterFileWipe(char *fileName,PCWF cwflist)
{
	int							lines = 0;
	PGPError					err;
	char						szPGPwipepattern[MAX_PATH];
	static char					volumeName[] = "\\\\.\\A:";
	HANDLE						hVolume;

	// Get the NtFsControlFile entry point since undocumented
	if( !(NtFsControlFile = (void *) GetProcAddress( GetModuleHandle("ntdll.dll"),
			"NtFsControlFile" )) )
	{
		return kPGPError_UnknownError;
	}

	//
	// open the volume
	//
	volumeName[4] = (char)'D';
	
	hVolume = CreateFile( volumeName, GENERIC_READ, 
					FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 
					0, 0 );
	if( hVolume == INVALID_HANDLE_VALUE )	
	{

		return kPGPError_UnknownError;
	}

	strcpy(szPGPwipepattern,"d:\\PGPwipepattern.tmp");

	err=kPGPError_NoErr;

	if(IsntPGPError(err))
	{
		int memamt;
		char *pPat;
		FILE *fin;

		DeleteFile(fileName);

		memamt=4096;

		pPat=(char *)malloc(memamt);

		if(pPat==NULL)
		{
			err=kPGPError_NoErr;
		}
		else
		{
			memset(pPat,0x11,memamt);

			fin=fopen(szPGPwipepattern,"wb");

			if(fin==0)
			{
				err=kPGPError_WriteFailed;
			}
			else
			{
				PCWF freeatlast;
				HANDLE hFile;

				fwrite(pPat,1,memamt,fin);
				fclose(fin);

				free(pPat);

				hFile = CreateFile( szPGPwipepattern, GENERIC_READ, 
								FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 
								FILE_FLAG_NO_BUFFERING, 0 );

				while(cwflist!=NULL)
				{
					DWORD						status;
					IO_STATUS_BLOCK				ioStatus;
					MOVEFILE_DESCRIPTOR			moveFile;
					ULONG						i;

					//
					// Setup movefile descriptor and make the call
					//
					memset(&moveFile,0x00,sizeof(MOVEFILE_DESCRIPTOR));
					memset(&ioStatus,0x00,sizeof(IO_STATUS_BLOCK));

					moveFile.FileHandle = hFile;
					moveFile.StartVcn.QuadPart = 0;

					for(i=0;i<cwflist->Size;i=i+16)
					{
						moveFile.TargetLcn.QuadPart = cwflist->Lcn+i;
						moveFile.NumVcns = cwflist->Size-i;

						if(moveFile.NumVcns>16)
							moveFile.NumVcns=16;

						status = NtFsControlFile( hVolume, NULL, NULL, 0, &ioStatus,
										FSCTL_MOVE_FILE,
										&moveFile, sizeof( moveFile ),
										NULL, 0 );

						// 
						// If the operation is pending, wait for it to finish
						//
						if( status == STATUS_PENDING ) 
						{
							WaitForSingleObject( hFile, INFINITE ); 
						}

						StatusToError(status);
					}

					freeatlast=cwflist;

					cwflist=cwflist->next;

					free(freeatlast);
				}

				DeleteFile(szPGPwipepattern);
			}
		}
	}

	CloseHandle(hVolume);

	return (DWORD)err;
}

// GetLastClusterOfFile
//
// Loops through till it finds the last cluster in the chain

DWORD GetSlackClustersInFile(VOLINFO *vi, char *fileName)
{
	DWORD						status;
	int							i;
	HANDLE						sourceFile;
	IO_STATUS_BLOCK				ioStatus;
	ULONGLONG					startVcn;
	PGET_RETRIEVAL_DESCRIPTOR	fileMappings;
	int							lines = 0;

	//
	// Open the file
	//
	sourceFile = CreateFile( fileName, GENERIC_READ, 
					FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 
					FILE_FLAG_NO_BUFFERING, 0 );

	if( sourceFile == INVALID_HANDLE_VALUE ) 
		return WFE_COULDNOTREAD;

	//
	// Start dumping the mapping information. Go until we hit the end of the
	// file.
	//
	startVcn = 0;

	fileMappings = (PGET_RETRIEVAL_DESCRIPTOR) 
		malloc(FILEMAPSIZE*sizeof(ULONGLONG));

	while( !(status = NtFsControlFile( sourceFile, NULL, NULL, 0, &ioStatus,
						FSCTL_GET_RETRIEVAL_POINTERS,
						&startVcn, sizeof( startVcn ),
						fileMappings, FILEMAPSIZE * sizeof(LARGE_INTEGER) ) ) ||
			 status == STATUS_BUFFER_OVERFLOW ||
			 status == STATUS_PENDING ) 
	{
		// 
		// If the operation is pending, wait for it to finish
		//
		if( status == STATUS_PENDING ) {
			
			WaitForSingleObject( sourceFile, INFINITE ); 

			//
			// Get the status from the status block
			//
			if( ioStatus.Status != STATUS_SUCCESS && 
				ioStatus.Status != STATUS_BUFFER_OVERFLOW ) 
			{
				free(fileMappings);
				// Error out
				return StatusToError(ioStatus.Status);
			}
		}

		//
		// Loop through the buffer of number/cluster pairs, printing them
		// out.
		//
		startVcn = fileMappings->StartVcn;
		for( i = 0; i < (ULONGLONG) fileMappings->NumberOfPairs; i++ ) 
		{
			if(vi->bCancel)
			{
				free(fileMappings);
				CloseHandle( sourceFile );
				return WFE_USERCANCEL;
			}

			//
			// On NT 4.0, a compressed virtual run (0-filled) is 
			// identified with a cluster offset of -1
			//
			if( fileMappings->Pair[i].Lcn == LLINVALID ) 
			{
				// Ignore virtual runs since OS is supposed to fill slack
				// with zeros
			} 
			else 
			{
				ULONGLONG freecluster,chainindex,chunk;

				chunk=0;

				if(vi->dwFS==FS_NTFS)
				{
					chunk=15;
				}

				for(chainindex=startVcn; 
					chainindex<startVcn+(fileMappings->Pair[i].Vcn - startVcn );
					chainindex=chainindex+1+chunk)
				{
					if(chainindex+chunk<
						startVcn+(fileMappings->Pair[i].Vcn - startVcn ))
					{
						// Hey it's a good length chunk!
						freecluster=fileMappings->Pair[i].Lcn+
							(chainindex-startVcn);

						// Save this cluster for later slack movement operations
						if(vi->SlackBufferIndex<SLACKBUFFER)
						{
							vi->SlackBuffer[vi->SlackBufferIndex]=freecluster;
							vi->SlackBufferIndex=vi->SlackBufferIndex+1;
						}
						else
						{
							// We're done
							free(fileMappings);
							CloseHandle( sourceFile );
							return StatusToError(STATUS_SUCCESS);
						}
					}
				}
			}
			startVcn = fileMappings->Pair[i].Vcn;
		}

		//
		// If the buffer wasn't overflowed, then we're done
		//
		if( !status ) break;
	}

	CloseHandle( sourceFile );

	free(fileMappings);
	return StatusToError(status);
}

DWORD DoFollowBehinds(VOLINFO *vi,BOOL *pbDidOne,BOOL bCleanOver,BOOL bCleanBack)
{
	SLACKMOVE sm;
	DWORD RetVal;

	*pbDidOne=FALSE;

	// These steps are followed 100 clusters behind
	// in order to assure NTFS has had time to deallocate
	// the space
	if(GetSMOffQueue(vi->OverwriteQueue,&sm,bCleanOver))
	{
		int retries;

		*pbDidOne=TRUE;
		retries=0;

		do
		{
			// Move pattern from old end of file to new end of file
			RetVal=ClusterNTWipeCluster( vi->hWipe,vi, 
				0,
				sm.Orig,
				sm.Size);
			
			retries++;
				
			if(vi->bCancel)
				return FALSE;
		} while((RetVal==WFE_ALREADYCOMMIT)&&(retries<RETRIES));

//				if(RetVal!=WFE_NOERROR)
//						StatusMessage(vi,"Error", FALSE);
//				else
//						StatusMessage(vi,"OK", FALSE);

		vi->dwClustersWritten=vi->dwClustersWritten+1;

//		if(RetVal!=WFE_NOERROR)
//			MessageBox(NULL,"test2","debug",MB_OK);

		PutSMOnQueue(vi->MoveBackQueue,&sm);
	}

	// These steps are followed 200 clusters behind
	// in order to assure NTFS has had time to deallocate
	// the space
	if(GetSMOffQueue(vi->MoveBackQueue,&sm,bCleanBack))
	{
		int retries;
		HANDLE hMoveBack;

		*pbDidOne=TRUE;
		retries=0;

		hMoveBack = CreateFile( sm.name, GENERIC_READ, 
						FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 
						FILE_FLAG_NO_BUFFERING, 0 );

		free(sm.name);

		if( hMoveBack != INVALID_HANDLE_VALUE ) 
		{
			do
			{
				RetVal=ClusterNTWipeCluster( hMoveBack,vi, 
					sm.Vcn,
					sm.Orig,
					sm.Size);

				retries++;
					
				if(vi->bCancel)
				{
					CloseHandle(hMoveBack);
					return FALSE;
				}
			} while((RetVal==WFE_ALREADYCOMMIT)&&(retries<RETRIES));

			CloseHandle(hMoveBack);
		}

//				if(RetVal!=WFE_NOERROR)
//						StatusMessage(vi,"Error", FALSE);
//				else
//						StatusMessage(vi,"OK", FALSE);

		vi->dwClustersWritten=vi->dwClustersWritten+1;

//		if(RetVal!=WFE_NOERROR)
//			MessageBox(NULL,"test3","debug",MB_OK);
	}

	return TRUE;
}

// This is the callback provided to our CreateFileList call.
// It will send us every filename on the drive. We'll free
// the links in the list ourselves to cut down on memory
// consumption
BOOL WipeFreeCountCallback(FILELIST *fl,void *pUserValue)
{
	VOLINFO *vi;
	char szText[MAX_PATH+100];

	vi=(VOLINFO *)pUserValue;

	if(!fl->IsDirectory)
	{
		// We'll have to do three cluster moves per file
		// original->temp, pattern->original,temp->original
		vi->dwClusterCount=vi->dwClusterCount+3;

		strcpy(szText,"Scanning disk\n");
		strcat(szText,fl->name);
		StatusMessage(vi,szText,FALSE);
	}

	free(fl->name);
	free(fl);

	return (vi->bCancel);
}

BOOL CountFileClusterMoves(VOLINFO *vi)
{
	// For the biggie file
//	vi->dwClusterCount=vi->no_of_freeclusters;

	ClusterCountFreeClusters(vi);

	vi->FileList=NULL;

	// For the slack space
	if(!PGPscAddToFileList(&(vi->FileList),vi->drivestr,
		WipeFreeCountCallback,vi))
	{
		vi->FileList=NULL;
		return WFE_USERCANCEL;
	}

	vi->FileList=NULL;
	return WFE_NOERROR;
}

// This is the callback provided to our CreateFileList call.
// It will send us every filename on the drive. We'll free
// the links in the list ourselves to cut down on memory
// consumption
BOOL WipeFreeAddFileCallback(FILELIST *fl,void *pUserValue)
{
	VOLINFO *vi;
	SLACKMOVE sm;
	DWORD RetVal;
	BOOL bDidOne;

	vi=(VOLINFO *)pUserValue;

	if(!fl->IsDirectory)
	{
		char szText[1024];

		strcpy(szText,"Wiping slack space at end of files...\n");
		strcat(szText,fl->name);
		StatusMessage(vi,szText,FALSE);

		RetVal=GetLastClusterOfFile(vi,
				fl->name,
				&sm);

		if(vi->bCancel)
			return FALSE;

		// We got the tail end. Now wipe it

		// If this fails, it is usually because the
		// file size is zero, so no problems anyway
		if((RetVal==WFE_NOERROR)&&(sm.Orig!=0))
		{
			int retries;
			HANDLE hMoveOver;

			// Now erase slack space
			// Move end of file to temp space
			retries=0;

			hMoveOver = CreateFile( sm.name, GENERIC_READ, 
							FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 
							FILE_FLAG_NO_BUFFERING, 0 );

			if( hMoveOver != INVALID_HANDLE_VALUE ) 
			{
				do
				{
					RetVal=ClusterNTWipeCluster( hMoveOver,vi, 
						sm.Vcn,
						vi->SlackBuffer[vi->SlackMoveCount%vi->SlackBufferIndex],
						sm.Size);

					retries++;
					vi->SlackMoveCount++;

					if(vi->bCancel)
					{
						CloseHandle(hMoveOver);
						return FALSE;
					}
				} while((RetVal==WFE_ALREADYCOMMIT)&&(retries<RETRIES));

				CloseHandle(hMoveOver);
			}

//				if(RetVal!=WFE_NOERROR)
///						StatusMessage(vi,"Error", FALSE);
//				else
//						StatusMessage(vi,"OK", FALSE);

			vi->dwClustersWritten=vi->dwClustersWritten+1;

//			if(RetVal!=WFE_NOERROR)
//				MessageBox(NULL,"test1","debug",MB_OK);

			PutSMOnQueue(vi->OverwriteQueue,&sm);
		}

		if(!DoFollowBehinds(vi,&bDidOne,FALSE,FALSE))
			return FALSE;
	}
	else
	{
		free(fl->name);
	}

	free(fl);

	return (vi->bCancel);
}

DWORD CreatePGPwipepattern(VOLINFO *vi)
{
	DWORD memamt,RetVal;
	char *PatternBuf;
	FILE *fin;

	RetVal=WFE_NOERROR;

	memamt=vi->sectors_per_clus*vi->sector_size;
	if(vi->dwFS==FS_NTFS)
		memamt=memamt*16; // NTFS works with 16 cluster segments

	PatternBuf=(char *)malloc(memamt);

	if(PatternBuf==0)
	{
		RetVal=WFE_OUTOFMEMORY;
	}
	else
	{
		//	memset(PatternBuf,'o',memamt); // For testing

		WritePattern(PatternBuf,vi->pattern_buffer,memamt);

		fin=fopen(vi->PGPwipepattern,"wb");

		if(fin==0)
		{
			RetVal=WFE_COULDNOTWRITE;
		}
		else
		{
			fwrite(PatternBuf,1,memamt,fin);
			fclose(fin);
		}
		free(PatternBuf);
	}

	return RetVal;
}

DWORD NTDoWipe(VOLINFO *vi)
{
	DWORD status,RetVal;
	int delay;

	RetVal=WFE_NOERROR;

	vi->SlackMoveCount=vi->SlackBufferIndex=0;
	vi->dwClustersWritten=vi->dwOldWritten=vi->dwClusterCount=vi->extracount=0;
	
	// Get the NtFsControlFile entry point since undocumented
	if( !(NtFsControlFile = (void *) GetProcAddress( GetModuleHandle("ntdll.dll"),
			"NtFsControlFile" )) )
	{
		return WFE_FSNOTSUPPORTED;
	}

	// Open the volume
	status = OpenVolume(vi);
	
	if( status != ERROR_SUCCESS ) 
	{
		return StatusToError(status);
	}

	// Get a count beforehand
	RetVal=CountFileClusterMoves(vi);

	{
		// Now obliterate all but 150 megs of free space
		RetVal=BiggieWipeFreeClusters(vi);

		if(RetVal==WFE_NOERROR)
		{
			if(vi->bNTFSInternal)
			{
				FreeBiggie(vi);
				vi->extracount=0;
			}

			// Create smaller 16 cluster wipepattern
			RetVal=CreatePGPwipepattern(vi);

			if(RetVal==WFE_NOERROR)
			{
				vi->hWipe = CreateFile( vi->PGPwipepattern, GENERIC_READ, 
								FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 
								FILE_FLAG_NO_BUFFERING, 0 );

				if( vi->hWipe == INVALID_HANDLE_VALUE ) 
				{
					// Could not open file
					RetVal=WFE_COULDNOTREAD ;
				}
				else
				{
					if(vi->bNTFSInternal)
					{
						// Just count them for now
						RetVal=ClusterWipeFreeClusters(vi,FALSE);
					}
					else
					{
						// Wipe the free clusters in this 150 meg
						RetVal=ClusterWipeFreeClusters(vi,TRUE);
					}

					delay=2;

					if(vi->dwFS==FS_NTFS)
						delay=vi->SlackBufferIndex;

					vi->MoveBackQueue=InitSMQueue(delay);
					vi->OverwriteQueue=InitSMQueue(delay);

					if(RetVal==WFE_NOERROR)
					{
						// Free all filled space
						FreeBiggie(vi);
						vi->extracount=0;

						if(RetVal==WFE_NOERROR)
						{
							BOOL bDidOne;

							// Get the files and wipe the slack
			
							// No need to free the FileList since the callback
							// does this. Tricky, but saves space and reuses complex
							// code
							vi->FileList=NULL;

							if(!PGPscAddToFileList(&(vi->FileList),vi->drivestr,
								WipeFreeAddFileCallback,vi))
								RetVal=WFE_USERCANCEL;

							// Cleanup
							StatusMessage(vi,"Cleaning up...",FALSE);
							do
							{
								if(!DoFollowBehinds(vi,&bDidOne,TRUE,FALSE))
								{
									RetVal=WFE_USERCANCEL;
									break;
								}
							} while(bDidOne);

							do
							{
								if(!DoFollowBehinds(vi,&bDidOne,TRUE,TRUE))
								{
									RetVal=WFE_USERCANCEL;
									break;
								}
							} while(bDidOne);
						}
					}
					CloseHandle(vi->hWipe);
				}
			}
			DestroySMQueue(vi->MoveBackQueue);
			DestroySMQueue(vi->OverwriteQueue);
		}
	}	

	FlushFileBuffers(vi->VolHandle);
	CloseHandle(vi->VolHandle);

	FreeBiggie(vi);
	remove(vi->PGPwipepattern);

	return RetVal;
}

void NTGetDriveInfo(VOLINFO *vi)
{
	int rst;
	DWORD NumFreeClusters;
	DWORD SectorsPerCluster;
	DWORD BytesPerSector;
	DWORD NumClusters;

	rst=GetDiskFreeSpace(vi->drivestr,
		&SectorsPerCluster,
		&BytesPerSector,
		&NumFreeClusters,
		&NumClusters);

	vi->no_of_freeclusters=NumFreeClusters;
	vi->no_of_clusters=NumClusters;
	vi->sectors_per_clus=(USHORT)SectorsPerCluster;
	vi->sector_size=(USHORT)BytesPerSector;

	vi->fTotalCapacity=(FLOAT)
		vi->no_of_clusters*vi->sectors_per_clus*vi->sector_size;

	if(vi->dwFS==FS_FAT)
	{
		// Microsoft says 4086 here
		// FIPs says 4079. 4079 makes more sense,
		// but probably won't matter either way
		if(vi->no_of_clusters <= 4079)
			vi->dwFS=FS_FAT12;
		else
			vi->dwFS=FS_FAT16;
	}
}

DWORD NTWipeFree(HWND hwnd,VOLINFO *vi,char *szRoot,
			   DWORD* pattern, BOOL StartWipe)
{
	DWORD RetVal;


	RetVal=WFE_NOERROR;

	// Zero-based volume, 'A'=0, 'B'=1, 'C'=2, etc.
	vi->vol0=toupper(szRoot[0])-'A';

	// Create our filenames
	sprintf(vi->drivestr,"%c:\\",vi->vol0+'A');
	sprintf(vi->PGPwipepattern,"%c:\\",vi->vol0+'A');
	strcat(vi->PGPwipepattern,"PGPwipepattern.tmp");

	NTGetDriveInfo(vi);

	// For clusters greater than 4K, defrag API is not supported
	// because of msoft compression goof
	if((vi->WinNT)&&(vi->dwFS==FS_NTFS)&&(vi->sectors_per_clus*vi->sector_size>4096))
		return WFE_NTFSGREATER4K;

	DisplayDiskStats(vi);

	// Do wipe if so. Otherwise report stats.
	if(StartWipe)
	{
		StatusMessage(vi,"Starting Free Space Wipe",FALSE);

		RetVal=NTDoWipe(vi);
	}
	return RetVal;
}

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
