/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CPGPdiskEngine.h,v 1.19 2002/11/22 02:32:15 wjb Exp $
____________________________________________________________________________*/

#ifndef Included_CPGPdiskEngine_h	// [
#define Included_CPGPdiskEngine_h

#include "CArray.h"
#include "CString.h"
#include "UMath.h"

#include "pgpDiskEngine.h"
#include "CPGPKey.h"

_PGP_BEGIN

// Class CPGPdiskEngine

class CPGPdiskEngine
{
	DECLARE_MODALSTATE(CPGPdiskEngine);

public:
	static void	InitLibrary();
	static void	CleanupLibrary();

	static void			PrefsChanged(PGPdiskContextRef context, 
		PGPPrefRef prefs);
	static PGPBoolean	AreDisksTimedOut();

	static PGPBoolean	IsPathMountedPGPdisk(const char *path);
	static PGPBoolean	IsRootMountedPGPdisk(const char *root);

	static PGPUInt32	GetNumDiskAlgorithms();
	static void			GetDiskAlgorithmIdString(
		PGPdiskEncryptionAlgorithm algorithm, CString& idString);
	static void			GetSupportedDiskAlgorithms(
		CArray<PGPdiskEncryptionAlgorithm>& algArray, PGPSize *pUsedLength);
	static PGPUInt32	GetNeededRandomDataSize(
		PGPdiskEncryptionAlgorithm algorithm);

	static void			NotifyUserLogoff();
};


// Class CPGPdiskContext

class CPGPdiskContext
{
	NOT_COPYABLE(CPGPdiskContext)

public:
	CPGPdiskContext();
	CPGPdiskContext(PGPdiskContextRef context);

	~CPGPdiskContext();

	operator PGPdiskContextRef() const {return mContext;}
	CPGPdiskContext&	operator=(PGPdiskContextRef context);

	PGPdiskContextRef	Get() const {return mContext;}

	PGPBoolean	IsAttached() const;
	PGPBoolean	WeCreated() const {return mWeCreated;}
	PGPBoolean	IsThereADK() const;

	PGPContextRef	PGPContext() const;
	PGPKeyDBRef		PGPKeyDB() const;
	PGPPrefRef		PGPclPrefs() const;

	void	GetADK(CPGPKey& adkKey) const;
	void	GetADKUserID(CString& userID) const;
	void	ReloadPGPKeyDB() const;

	void	Attach(PGPdiskContextRef context);
	void	Create();
	void	Clear();

private:
	PGPBoolean			mWeCreated;
	PGPdiskContextRef	mContext;
};


// Class CPGPdiskDisk

class CPGPdiskDisk
{
	NOT_COPYABLE(CPGPdiskDisk)

public:
	CPGPdiskDisk();
	CPGPdiskDisk(PGPdiskDiskRef disk);
	~CPGPdiskDisk();

	CPGPdiskDisk&	operator=(PGPdiskDiskRef disk);

	operator PGPdiskDiskRef() const {return mDisk;}
	PGPdiskDiskRef	Get() const {return mDisk;}

	PGPBoolean	IsAttached() const;
	PGPBoolean	WeOpened() const {return mWeOpened;}
	PGPBoolean	WeLocked() const {return mWeLocked;}

	PGPBoolean	IsMounted() const;
	PGPBoolean	IsOpened() const;
	PGPBoolean	IsReadOnly() const;
	PGPBoolean	IsLockedForFormat() const;
	PGPBoolean	IsLockedForIO() const;
	PGPBoolean	IsBeingReEncrypted() const;
	PGPBoolean	HasOldHeaders() const;
	PGPBoolean	AreUsersWiped() const;
	PGPBoolean	HasInactivityTimedOut() const;
	PGPBoolean	GetUsesCustomTimeout() const;
	PGPdiskEncryptionAlgorithm	Algorithm() const;
	void		GetDefaultRoot(CString& root) const;
	PGPUInt32	GetCustomTimeout() const;
	PGPUInt64	GetBlocksDisk() const;

	void	GetPath(CString& path) const;
	void	GetRoot(CString& root) const;
	void	GetDeviceName(CString& deviceName) const;

	PGPdiskContextRef	Context() const;

	void	ExportUserSet(PGPdiskUserSetRef userSet) const;
	void	ImportUserSet(PGPdiskUserSetRef userSet, 
		PGPdiskUserRef adminUserForCiphering);

	void	ChangeOnePassphrase(const char *userName, 
		const char *oldPassphrase, const char *newPassphrase);
	void	RemoveOneUser(PGPdiskUserRef userForCiphering);

	void	WipeUsers();

	void	SetUsesCustomTimeout(PGPBoolean usesCustom);
	void	SetDefaultRoot(const char *root);
	void	SetCustomTimeout(PGPUInt32 customTimeout);

	void	ReEncryptDisk(const char *adminPassphrase, 
		PGPdiskEncryptionAlgorithm newAlgorithm, 
		ReEncryptDiskStatusFuncType statusFunc, 
		void *userValue = NULL) const;

	void	LockVolume(PGPBoolean forFormat);
	void	UnlockVolume();

	void	ReadVolume(void *buffer, PGPUInt64 blockPos, 
		PGPUInt32 blocksLength);
	void	WriteVolume(const void *buffer, PGPUInt64 blockPos, 
		PGPUInt32 blocksLength);

	void	Open(PGPdiskContextRef context, const char *path, 
		PGPBoolean readOnly = FALSE);
	void	Close();

	void	Attach(PGPdiskDiskRef disk);
	void	Clear();

private:
	PGPdiskDiskRef	mDisk;

	PGPBoolean	mWeOpened;
	PGPBoolean	mWeLocked;

	PGPBoolean	GetBooleanProp(PGPdiskDiskPropName prop) const;
	PGPInt32	GetNumericProp(PGPdiskDiskPropName prop) const;
	void		GetDataProp(PGPdiskDiskPropName prop, void *pData, 
		PGPSize availLength, PGPSize *pUsedLength = NULL) const;

	void	SetBooleanProp(PGPdiskDiskPropName prop, PGPBoolean boolVal);
	void	SetNumericProp(PGPdiskDiskPropName prop, PGPInt32 number);
	void	SetDataProp(PGPdiskDiskPropName prop, const void *pData, 
		PGPSize length);
};


// Class CPGPdiskDiskSet

class CPGPdiskDiskSet
{
	NOT_COPYABLE(CPGPdiskDiskSet);

public:
	CPGPdiskDiskSet(PGPdiskContextRef context);
	CPGPdiskDiskSet(PGPdiskDiskSetRef diskSet);
	~CPGPdiskDiskSet();

	CPGPdiskDiskSet&	operator=(PGPdiskDiskSetRef diskSet);

	operator PGPdiskDiskSetRef() const {return mDiskSet;}
	PGPdiskDiskSetRef	Get() const {return mDiskSet;}

	PGPBoolean	IsAttached() const;
	PGPBoolean	WeCreated() const {return mWeCreated;}

	PGPdiskContextRef	Context() const;
	PGPUInt32			Count() const;

	void	Mount(const char *path, PGPdiskUserRef userForCiphering, 
		const char *root, PGPBoolean readOnly, CPGPdiskDisk& disk);
	void	Unmount(PGPdiskDiskRef disk, PGPBoolean isForced = FALSE);

	void	UnmountTimedOut(PGPBoolean isForced = FALSE);
	void	UnmountAll(PGPBoolean isForced = FALSE);
	void	WipeAllUsers();

	void	CreateDisk(const char *path, PGPUInt64 blocksDisk, 
		PGPdiskEncryptionAlgorithm algorithm, const char *passphrase, 
		const char *userName, const char *defaultRoot, 
		NewDiskStatusFuncType statusFunc, void *userValue, 
		CPGPdiskDisk& disk);

	void	CreateDisk(const char *path, PGPUInt64 blocksDisk, 
		PGPdiskEncryptionAlgorithm algorithm, PGPKeyDBObjRef key, 
		const char *defaultRoot, NewDiskStatusFuncType statusFunc, 
		void *userValue, CPGPdiskDisk& disk);

	void	Update();

	void	Attach(PGPdiskDiskSetRef diskSet);
	void	Create(PGPdiskContextRef context);
	void	Clear();

private:
	PGPdiskDiskSetRef	mDiskSet;
	PGPBoolean			mWeCreated;
};


// Class CPGPdiskDiskIter

class CPGPdiskDiskIter
{
	NOT_COPYABLE(CPGPdiskDiskIter)

public:
	CPGPdiskDiskIter(PGPdiskDiskSetRef diskSet);
	CPGPdiskDiskIter(PGPdiskDiskIterRef diskIter);
	~CPGPdiskDiskIter();

	CPGPdiskDiskIter&	operator=(PGPdiskDiskIterRef diskIter);

	operator PGPdiskDiskIterRef() const {return mDiskIter;}
	PGPdiskDiskIterRef	Get() const {return mDiskIter;}

	PGPBoolean	IsAttached() const;
	PGPBoolean	WeCreated() const {return mWeCreated;}

	PGPdiskDiskSetRef	DiskSet() const;
	PGPUInt32			Count() const;
	PGPInt32			Index() const;

	PGPBoolean	Current(CPGPdiskDisk& disk) const;
	PGPBoolean	Move(PGPInt32 relOffset, PGPBoolean fromStart, 
		CPGPdiskDisk& disk);
	PGPInt32	Seek(PGPdiskDiskRef soughtDisk);
	PGPBoolean	Next(CPGPdiskDisk& disk);
	PGPBoolean	Prev(CPGPdiskDisk& disk);
	void		Rewind();
	
	PGPBoolean	SearchOnPath(const char *path, CPGPdiskDisk& disk);
	PGPBoolean	SearchOnRoot(const char *root, CPGPdiskDisk& disk);

	void	Attach(PGPdiskDiskIterRef diskIter);
	void	Create(PGPdiskDiskSetRef diskSet);
	void	Clear();

private:
	PGPdiskDiskIterRef	mDiskIter;
	PGPBoolean			mWeCreated;
};


// Class CPGPdiskUser

class CPGPdiskUser
{
	NOT_COPYABLE(CPGPdiskUser)

public:
	CPGPdiskUser();
	CPGPdiskUser(PGPdiskUserRef user);
	~CPGPdiskUser();

	CPGPdiskUser&	operator=(PGPdiskUserRef user);

	operator PGPdiskUserRef() const {return mUser;}
	PGPdiskUserRef	Get() const {return mUser;}

	PGPBoolean	IsAttached() const;
	PGPBoolean	WeCreated() const {return mWeCreated;}

	PGPBoolean	IsEqualUser(PGPdiskUserRef user) const;

	PGPBoolean	HasPublicKey() const;
	PGPBoolean	HasSymmetricKey() const;
	PGPBoolean	IsReadOnly() const;
	PGPBoolean	IsLocked() const;
	PGPBoolean	IsAdminUser() const;
	PGPBoolean	IsDisabled() const;
	PGPBoolean	IsCipheringEnabled() const;
	PGPBoolean	CanSetReadOnly() const;
	PGPBoolean	CanSetLocked() const;
	PGPBoolean	CanSetUserName() const;

	PGPdiskContextRef	Context() const;

	void	GetUserName(CString& userName) const;
	void	GetPGPKeyID(CPGPKeyID& keyID) const;

	void	SetIsReadOnly(PGPBoolean isReadOnly);
	void	SetIsLocked(PGPBoolean isLocked);
	void	SetIsDisabled(PGPBoolean isDisabled);
	void	SetUserName(const char *userName);

	void	EnableCiphering(PGPdiskDiskRef disk, const char *passphrase, int passkeylen);
	void	DisableCiphering();

	void	ChangePassphrase(const char *newPassphrase);

	void	Create(PGPdiskUserRef cipheringUser, const char *userName,  
		const char *passphrase, PGPBoolean isReadOnly = FALSE);
	void	Create(PGPdiskUserRef cipheringUser, PGPKeyDBObjRef pubKey, 
		PGPBoolean isReadOnly = FALSE, PGPBoolean isLocked = FALSE);
	void	DuplicateUser(PGPdiskUserRef user);

	void	Attach(PGPdiskUserRef user);
	void	Clear();

private:
	PGPdiskUserRef	mUser;
	PGPBoolean		mWeCreated;

	PGPBoolean	GetBooleanProp(PGPdiskUserPropName prop) const;
	PGPInt32	GetNumericProp(PGPdiskUserPropName prop) const;
	void		GetDataProp(PGPdiskUserPropName prop, void *pData, 
		PGPSize availLength, PGPSize *pUsedLength = NULL) const;

	void	SetBooleanProp(PGPdiskUserPropName prop, PGPBoolean boolVal);
	void	SetNumericProp(PGPdiskUserPropName prop, PGPInt32 number);
	void	SetDataProp(PGPdiskUserPropName prop, const void *pData, 
		PGPSize length);
};


// Class CPGPdiskUserSet

class CPGPdiskUserSet
{
	NOT_COPYABLE(CPGPdiskUserSet);

public:
	CPGPdiskUserSet();
	CPGPdiskUserSet(PGPdiskUserRef user);
	CPGPdiskUserSet(PGPdiskUserSetRef userSet);
	~CPGPdiskUserSet();

	CPGPdiskUserSet&	operator=(PGPdiskUserSetRef userSet);

	operator PGPdiskUserSetRef() const {return mUserSet;}
	PGPdiskUserSetRef	Get() const {return mUserSet;}

	PGPBoolean	IsAttached() const;
	PGPBoolean	WeCreated() const {return mWeCreated;}

	PGPUInt32	Count() const;

	void	Add(PGPdiskUserSetRef usersToAdd);
	void	Add(PGPdiskUserRef userToAdd);
	void	Remove(PGPdiskUserSetRef usersToRemove);
	void	Remove(PGPdiskUserRef userToRemove);
	void	Empty();

	void	Attach(PGPdiskUserSetRef userSet);
	void	CreateEmpty();
	void	CreateSingleton(PGPdiskUserRef user);
	void	Clear();

private:
	PGPdiskUserSetRef	mUserSet;
	PGPBoolean			mWeCreated;
};


// Class CPGPdiskUserIter

class CPGPdiskUserIter
{
	NOT_COPYABLE(CPGPdiskUserIter)

public:
	CPGPdiskUserIter(PGPdiskUserSetRef userSet);
	CPGPdiskUserIter(PGPdiskUserIterRef userIter);
	~CPGPdiskUserIter();

	CPGPdiskUserIter&	operator=(PGPdiskUserIterRef userIter);

	operator PGPdiskUserIterRef() const {return mUserIter;}
	PGPdiskUserIterRef	Get() const {return mUserIter;}

	PGPBoolean	IsAttached() const;
	PGPBoolean	WeCreated() const {return mWeCreated;}

	PGPdiskUserSetRef	UserSet() const;
	PGPUInt32			Count() const;
	PGPInt32			Index() const;

	PGPBoolean	Current(CPGPdiskUser& user) const;
	PGPBoolean	Move(PGPInt32 relOffset, PGPBoolean fromStart, 
		CPGPdiskUser& user);
	PGPInt32	Seek(PGPdiskUserRef soughtUser);
	PGPBoolean	Next(CPGPdiskUser& user);
	PGPBoolean	Prev(CPGPdiskUser& user);
	void		Rewind();
	
	PGPBoolean	SearchOnUserName(const char *userName, CPGPdiskUser& user);
	PGPBoolean	FindEqualUser(const CPGPdiskUser& user, 
		CPGPdiskUser& equalUser);
	PGPBoolean	FindAdminUser(CPGPdiskUser& user);

	void	SetAdminUser(PGPdiskUserRef userForCiphering, 
		PGPdiskUserRef oldAdminForCiphering = kInvalidPGPdiskUserRef);

	void	Attach(PGPdiskUserIterRef userIter);
	void	Create(PGPdiskUserSetRef userSet);
	void	Clear();

private:
	PGPdiskUserIterRef	mUserIter;
	PGPBoolean			mWeCreated;
};


// Class CPGPdiskEngine member functions

inline 
void 
CPGPdiskEngine::InitLibrary()
{
	PGPError	error	= PGPdiskEngineInitLibrary(
		kPGPclEnableNewUserWizard | kPGPclForceNewUserWizard);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskEngine::CleanupLibrary()
{
	PGPdiskEngineCleanupLibrary();
}

inline 
void 
CPGPdiskEngine::PrefsChanged(PGPdiskContextRef context, PGPPrefRef prefs)
{
	PGPdiskPrefsChanged(context, prefs);
}

inline 
PGPBoolean 
CPGPdiskEngine::AreDisksTimedOut()
{
	return PGPdiskAreDisksTimedOut();
}

inline 
PGPBoolean 
CPGPdiskEngine::IsPathMountedPGPdisk(const char *path)
{
	return PGPdiskIsPathMountedPGPdisk(path);
}

inline 
PGPBoolean 
CPGPdiskEngine::IsRootMountedPGPdisk(const char *root)
{
	return PGPdiskIsRootMountedPGPdisk(root);
}

inline 
PGPUInt32 
CPGPdiskEngine::GetNumDiskAlgorithms()
{
	return PGPdiskGetNumDiskAlgorithms();
}

inline 
void 
CPGPdiskEngine::GetDiskAlgorithmIdString(
	PGPdiskEncryptionAlgorithm	algorithm, 
	CString&					idString)
{
	PGPdiskGetDiskAlgorithmIdString(algorithm, 
		idString.GetBuffer(kPGPdiskMaxAlgStringIdLength), 
		kPGPdiskMaxAlgStringIdLength);
	idString.ReleaseBuffer();
}

inline 
void 
CPGPdiskEngine::GetSupportedDiskAlgorithms(
	CArray<PGPdiskEncryptionAlgorithm>&	algArray, 
	PGPSize								*pUsedLength)
{
	algArray.Resize(GetNumDiskAlgorithms());

	PGPdiskGetSupportedDiskAlgorithms(algArray.Get(), algArray.Size(), 
		pUsedLength);
}

inline 
PGPUInt32 
CPGPdiskEngine::GetNeededRandomDataSize(PGPdiskEncryptionAlgorithm algorithm)
{
	return PGPdiskGetNeededRandomDataSize(algorithm);
}

inline 
void 
CPGPdiskEngine::NotifyUserLogoff()
{
	PGPdiskNotifyUserLogoff();
}


// Class CPGPdiskContext member functions

inline 
CPGPdiskContext::CPGPdiskContext() : 
	mContext(kInvalidPGPdiskContextRef), mWeCreated(FALSE)
{
}

inline 
CPGPdiskContext::CPGPdiskContext(PGPdiskContextRef context) :
	mContext(kInvalidPGPdiskContextRef), mWeCreated(FALSE)
{
	Attach(context);
}

inline 
CPGPdiskContext::CPGPdiskContext(const CPGPdiskContext& context) :
	mContext(kInvalidPGPdiskContextRef), mWeCreated(FALSE)
{
	Attach(context);
}

inline 
CPGPdiskContext::~CPGPdiskContext()
{
	try
	{
		Clear();
	}
	catch (PGPError&) { }
}

inline 
CPGPdiskContext& 
CPGPdiskContext::operator=(PGPdiskContextRef context)
{
	Attach(context);
	return *this;
}

inline 
PGPBoolean 
CPGPdiskContext::IsAttached() const
{
	return PGPdiskContextRefIsValid(mContext);
}

inline 
PGPContextRef 
CPGPdiskContext::PGPContext() const
{
	pgpAssert(IsAttached());
	return PGPdiskPeekContextPGPContext(mContext);
}

inline 
PGPKeyDBRef 
CPGPdiskContext::PGPKeyDB() const
{
	pgpAssert(IsAttached());

	PGPKeyDBRef	pgpKeyDB;

	PGPError	error	= PGPdiskOpenPeekContextPGPKeyDB(mContext, &pgpKeyDB);
	THROW_IF_PGPERROR(error);

	return pgpKeyDB;
}

inline 
PGPPrefRef 
CPGPdiskContext::PGPclPrefs() const
{
	pgpAssert(IsAttached());
	return PGPdiskPeekContextPGPclPrefs(mContext);
}

inline 
PGPBoolean 
CPGPdiskContext::IsThereADK() const
{
	PGPBoolean	isThereADK;
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskContextIsThereADK(mContext, &isThereADK);
	THROW_IF_PGPERROR(error);

	return isThereADK;
}

inline 
void 
CPGPdiskContext::GetADK(CPGPKey& adkKey) const
{
	PGPKeyDBObjRef	adkKeyRef;

	PGPError	error	= PGPdiskContextGetADK(mContext, &adkKeyRef);
	THROW_IF_PGPERROR(error);

	adkKey.Attach(adkKeyRef);
}

inline 
void 
CPGPdiskContext::GetADKUserID(CString& userID) const
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskContextGetADKUserID(mContext, 
		userID.GetBuffer(kPGPMaxUserIDSize), kPGPMaxUserIDSize);
	userID.ReleaseBuffer();

	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskContext::ReloadPGPKeyDB() const
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskReloadContextPGPKeyDB(mContext);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskContext::Attach(PGPdiskContextRef context)
{
	if (mContext == context)
		return;

	Clear();
	pgpAssert(PGPdiskContextRefIsValid(context));

	mContext = context;
	mWeCreated = FALSE;
}

inline 
void 
CPGPdiskContext::Create()
{
	Clear();

	PGPError	error	= PGPdiskNewContext(kPGPdiskAPIVersion, 
		&mContext);
	THROW_IF_PGPERROR(error);

	mWeCreated = TRUE;
}

inline 
void 
CPGPdiskContext::Clear()
{
	if (mWeCreated)
		PGPdiskFreeContext(mContext);

	mContext = kInvalidPGPdiskContextRef;
}


// Class CPGPdiskDisk member functions

inline 
CPGPdiskDisk::CPGPdiskDisk() : 
	mDisk(kInvalidPGPdiskDiskRef), mWeOpened(FALSE), mWeLocked(FALSE)
{
}

inline 
CPGPdiskDisk::CPGPdiskDisk(PGPdiskDiskRef disk) :
	mDisk(kInvalidPGPdiskDiskRef), mWeOpened(FALSE), mWeLocked(FALSE)
{
	Attach(disk);
}

inline 
CPGPdiskDisk::~CPGPdiskDisk()
{
	try
	{
		Clear();
	}
	catch (CComboError&) { }
}

inline 
CPGPdiskDisk& 
CPGPdiskDisk::operator=(PGPdiskDiskRef disk)
{
	Attach(disk);
	return *this;
}

inline 
PGPBoolean 
CPGPdiskDisk::IsAttached() const
{
	return PGPdiskDiskRefIsValid(mDisk);
}

inline 
PGPBoolean 
CPGPdiskDisk::IsMounted() const
{
	return (IsAttached() && GetBooleanProp(kPGPdiskDiskProp_IsMounted));
}

inline 
PGPBoolean 
CPGPdiskDisk::IsOpened() const
{
	return (IsAttached() && GetBooleanProp(kPGPdiskDiskProp_IsOpened));
}

inline 
PGPBoolean 
CPGPdiskDisk::IsReadOnly() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskDiskProp_IsReadOnly);
}

inline 
PGPBoolean 
CPGPdiskDisk::IsLockedForFormat() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskDiskProp_IsLockedForFormat);
}

inline 
PGPBoolean 
CPGPdiskDisk::IsLockedForIO() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskDiskProp_IsLockedForIO);
}

inline 
PGPBoolean 
CPGPdiskDisk::IsBeingReEncrypted() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskDiskProp_IsBeingReEncrypted);
}

inline 
PGPBoolean 
CPGPdiskDisk::HasOldHeaders() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskDiskProp_HasOldHeaders);
}

inline 
PGPBoolean 
CPGPdiskDisk::AreUsersWiped() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskDiskProp_AreUsersWiped);
}

inline 
PGPBoolean 
CPGPdiskDisk::HasInactivityTimedOut() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskDiskProp_HasTimedOut);
}

inline 
PGPBoolean 
CPGPdiskDisk::GetUsesCustomTimeout() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskDiskProp_UsesCustomTimeout);
}

inline 
PGPdiskEncryptionAlgorithm 
CPGPdiskDisk::Algorithm() const
{
	pgpAssert(IsAttached());

	return static_cast<PGPdiskEncryptionAlgorithm>(
		GetNumericProp(kPGPdiskDiskProp_Algorithm));
}

inline 
void 
CPGPdiskDisk::GetDefaultRoot(CString& root) const
{
	pgpAssert(IsAttached());

	GetDataProp(kPGPdiskDiskProp_DefaultRoot, 
		root.GetBuffer(kPGPdiskMaxPathLength), kPGPdiskMaxPathLength);
	root.ReleaseBuffer();
}

inline 
PGPUInt32 
CPGPdiskDisk::GetCustomTimeout() const
{
	pgpAssert(IsAttached());
	return GetNumericProp(kPGPdiskDiskProp_CustomTimeout);
}

inline 
PGPUInt64 
CPGPdiskDisk::GetBlocksDisk() const
{
	pgpAssert(IsAttached());

	return UMath::MakeQWord(GetNumericProp(kPGPdiskDiskProp_BlocksDiskLow), 
		GetNumericProp(kPGPdiskDiskProp_BlocksDiskHigh));
}

inline 
void 
CPGPdiskDisk::GetPath(CString& path) const
{
	pgpAssert(IsAttached());

	GetDataProp(kPGPdiskDiskProp_Path, 
		path.GetBuffer(kPGPdiskMaxPathLength), kPGPdiskMaxPathLength);
	path.ReleaseBuffer();
}

inline 
void 
CPGPdiskDisk::GetRoot(CString& root) const
{
	pgpAssert(IsAttached());

	GetDataProp(kPGPdiskDiskProp_Root, root.GetBuffer(kPGPdiskMaxPathLength), 
		kPGPdiskMaxPathLength);
	root.ReleaseBuffer();
}

inline 
void 
CPGPdiskDisk::GetDeviceName(CString& deviceName) const
{
	pgpAssert(IsAttached());

	GetDataProp(kPGPdiskDiskProp_DeviceName, 
		deviceName.GetBuffer(kPGPdiskMaxDeviceNameLength), 
		kPGPdiskMaxDeviceNameLength);
	deviceName.ReleaseBuffer();
}

inline 
PGPdiskContextRef 
CPGPdiskDisk::Context() const
{
	pgpAssert(IsAttached());
	return PGPdiskPeekDiskContext(mDisk);
}

inline 
void 
CPGPdiskDisk::ExportUserSet(PGPdiskUserSetRef userSet) const
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskDiskExportUserSet(mDisk, userSet);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDisk::ImportUserSet(
	PGPdiskUserSetRef	userSet, 
	PGPdiskUserRef		adminUserForCiphering)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskDiskImportUserSet(mDisk, userSet, 
		adminUserForCiphering);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDisk::ChangeOnePassphrase(
	const char	*userName, 
	const char	*oldPassphrase, 
	const char	*newPassphrase)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskDiskChangeOnePassphrase(mDisk, userName, 
		oldPassphrase, newPassphrase);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDisk::RemoveOneUser(PGPdiskUserRef userForCiphering)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskDiskRemoveOneUser(mDisk, userForCiphering);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDisk::SetUsesCustomTimeout(PGPBoolean usesCustom)
{
	pgpAssert(IsAttached());
	SetBooleanProp(kPGPdiskDiskProp_UsesCustomTimeout, usesCustom);
}

inline 
void 
CPGPdiskDisk::SetDefaultRoot(const char *root)
{
	pgpAssert(IsAttached());
	SetDataProp(kPGPdiskDiskProp_DefaultRoot, root, strlen(root));
}

inline 
void 
CPGPdiskDisk::SetCustomTimeout(PGPUInt32 customTimeout)
{
	pgpAssert(IsAttached());
	SetNumericProp(kPGPdiskDiskProp_CustomTimeout, customTimeout);
}

inline 
void 
CPGPdiskDisk::WipeUsers()
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskWipeUsersOnDisk(mDisk);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDisk::ReEncryptDisk(
	const char					*adminPassphrase, 
	PGPdiskEncryptionAlgorithm	newAlgorithm, 
	ReEncryptDiskStatusFuncType	statusFunc, 
	void						*userValue) const
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskReEncryptDisk(mDisk, adminPassphrase, 
		newAlgorithm, statusFunc, userValue);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDisk::LockVolume(PGPBoolean forFormat)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskLockDiskVolume(mDisk, forFormat);
	THROW_IF_PGPERROR(error);

	mWeLocked = TRUE;
}

inline 
void 
CPGPdiskDisk::UnlockVolume()
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskUnlockDiskVolume(mDisk);
	THROW_IF_PGPERROR(error);

	mWeLocked = FALSE;
}

inline 
void 
CPGPdiskDisk::ReadVolume(
	void		*buffer, 
	PGPUInt64	blockPos, 
	PGPUInt32	blocksLength)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskReadDiskVolume(mDisk, buffer, blockPos, 
		blocksLength);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDisk::WriteVolume(
	const void	*buffer, 
	PGPUInt64	blockPos, 
	PGPUInt32	blocksLength)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskWriteDiskVolume(mDisk, buffer, blockPos, 
		blocksLength);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDisk::Open(
	PGPdiskContextRef	context, 
	const char			*path, 
	PGPBoolean			readOnly)
{
	pgpAssert(!WeOpened());

	PGPError	error	= PGPdiskOpenDisk(context, path, readOnly, &mDisk);
	THROW_IF_PGPERROR(error);

	mWeOpened = TRUE;
}

inline 
void 
CPGPdiskDisk::Close()
{
	pgpAssert(WeOpened());

	PGPError	error	= PGPdiskCloseDisk(mDisk);
	THROW_IF_PGPERROR(error);

	mWeOpened = FALSE;
	mDisk = kInvalidPGPdiskDiskRef;
}

inline 
void 
CPGPdiskDisk::Attach(PGPdiskDiskRef disk)
{
	if (mDisk == disk)
		return;

	Clear();

	mDisk = disk;
	mWeOpened = FALSE;
}

inline 
void 
CPGPdiskDisk::Clear()
{
	if (WeOpened())
		Close();

	if (WeLocked())
		UnlockVolume();

	mDisk = kInvalidPGPdiskDiskRef;
	mWeOpened = mWeLocked = FALSE;
}

inline 
PGPBoolean 
CPGPdiskDisk::GetBooleanProp(PGPdiskDiskPropName prop) const
{
	pgpAssert(IsAttached());

	PGPBoolean	boolVal;

	PGPError	error	= PGPdiskGetDiskBooleanProp(mDisk, prop, &boolVal);
	THROW_IF_PGPERROR(error);

	return boolVal;
}

inline 
PGPInt32 
CPGPdiskDisk::GetNumericProp(PGPdiskDiskPropName prop) const
{
	pgpAssert(IsAttached());

	PGPInt32	number;

	PGPError	error	= PGPdiskGetDiskNumericProp(mDisk, prop, &number);
	THROW_IF_PGPERROR(error);

	return number;
}

inline 
void 
CPGPdiskDisk::GetDataProp(
	PGPdiskDiskPropName	prop, 
	void				*pData, 
	PGPSize				availLength, 
	PGPSize				*pUsedLength) const
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskGetDiskDataProp(mDisk, prop, pData, 
		availLength, pUsedLength);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDisk::SetBooleanProp(PGPdiskDiskPropName prop, PGPBoolean boolVal)
{
	PGPError	error	= PGPdiskSetDiskBooleanProp(mDisk, prop, boolVal);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDisk::SetNumericProp(PGPdiskDiskPropName prop, PGPInt32 number)
{
	PGPError	error	= PGPdiskSetDiskNumericProp(mDisk, prop, number);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDisk::SetDataProp(
	PGPdiskDiskPropName	prop, 
	const void			*pData, 
	PGPSize				length)
{
	PGPError	error	= PGPdiskSetDiskDataProp(mDisk, prop, pData, length);
	THROW_IF_PGPERROR(error);
}


// Class CPGPdiskDiskSet member functions

inline 
CPGPdiskDiskSet::CPGPdiskDiskSet(PGPdiskContextRef context) : 
	mDiskSet(kInvalidPGPdiskDiskSetRef), mWeCreated(FALSE)
{
	Create(context);
}

inline 
CPGPdiskDiskSet::CPGPdiskDiskSet(PGPdiskDiskSetRef diskSet) : 
	mDiskSet(kInvalidPGPdiskDiskSetRef), mWeCreated(FALSE)
{
	Attach(diskSet);
}

inline 
CPGPdiskDiskSet::~CPGPdiskDiskSet()
{
	try
	{
		Clear();
	}
	catch (PGPError&) { }
}

inline 
PGPdiskContextRef 
CPGPdiskDiskSet::Context() const
{
	pgpAssert(IsAttached());
	return PGPdiskPeekDiskSetContext(mDiskSet);
}

inline 
PGPBoolean 
CPGPdiskDiskSet::IsAttached() const
{
	return PGPdiskDiskSetRefIsValid(mDiskSet);
}

inline 
PGPUInt32 
CPGPdiskDiskSet::Count() const
{
	pgpAssert(IsAttached());
	return PGPdiskDiskSetCount(mDiskSet);
}

inline 
void 
CPGPdiskDiskSet::Mount(
	const char		*path, 
	PGPdiskUserRef	userForCiphering, 
	const char		*root, 
	PGPBoolean		readOnly, 
	CPGPdiskDisk	&disk)
{
	PGPdiskDiskRef	diskRef;

	PGPError	error	= PGPdiskDiskSetMountDisk(mDiskSet, path, 
		userForCiphering, root, readOnly, &diskRef);
	THROW_IF_PGPERROR(error);

	disk.Attach(diskRef);
}

inline 
void 
CPGPdiskDiskSet::Unmount(PGPdiskDiskRef disk, PGPBoolean isForced)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskDiskSetUnmountDisk(mDiskSet, disk, isForced);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDiskSet::UnmountTimedOut(PGPBoolean isForced)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskDiskSetUnmountTimedOut(mDiskSet, isForced);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDiskSet::UnmountAll(PGPBoolean isForced)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskDiskSetUnmountAll(mDiskSet, isForced);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDiskSet::WipeAllUsers()
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskDiskSetWipeAllUsers(mDiskSet);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDiskSet::CreateDisk(
	const char					*path, 
	PGPUInt64					blocksDisk, 
	PGPdiskEncryptionAlgorithm	algorithm, 
	const char					*passphrase, 
	const char					*userName, 
	const char					*defaultRoot, 
	NewDiskStatusFuncType		statusFunc, 
	void						*userValue, 
	CPGPdiskDisk&				disk)
{
	pgpAssert(IsAttached());

	PGPdiskDiskRef	diskRef;
	PGPError		error	= PGPdiskDiskSetCreateDiskWithSym(mDiskSet, 
		path, blocksDisk, algorithm, passphrase, userName, defaultRoot, 
		statusFunc, userValue, &diskRef);
	THROW_IF_PGPERROR(error);

	disk.Attach(diskRef);
}

inline 
void 
CPGPdiskDiskSet::CreateDisk(
	const char					*path, 
	PGPUInt64					blocksDisk, 
	PGPdiskEncryptionAlgorithm	algorithm, 
	PGPKeyDBObjRef				key, 
	const char					*defaultRoot, 
	NewDiskStatusFuncType		statusFunc, 
	void						*userValue, 
	CPGPdiskDisk&				disk)
{
	pgpAssert(IsAttached());

	PGPdiskDiskRef	diskRef;
	PGPError		error	= PGPdiskDiskSetCreateDiskWithPub(mDiskSet, 
		path, blocksDisk, algorithm, key, defaultRoot, statusFunc, 
		userValue, &diskRef);
	THROW_IF_PGPERROR(error);

	disk.Attach(diskRef);
}

inline 
void 
CPGPdiskDiskSet::Update()
{
	PGPError	error	= PGPdiskUpdateDiskSet(mDiskSet);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskDiskSet::Attach(PGPdiskDiskSetRef diskSet)
{
	if (mDiskSet == diskSet)
		return;

	Clear();

	mDiskSet = diskSet;
	mWeCreated = FALSE;
}

inline 
void 
CPGPdiskDiskSet::Create(PGPdiskContextRef context)
{
	Clear();

	PGPError	error	= PGPdiskNewDiskSet(context, &mDiskSet);
	THROW_IF_PGPERROR(error);

	mWeCreated = TRUE;
}

inline 
void 
CPGPdiskDiskSet::Clear()
{
	if (WeCreated())
		PGPdiskFreeDiskSet(mDiskSet);

	mDiskSet = kInvalidPGPdiskDiskSetRef;
	mWeCreated = FALSE;
}


// Class CPGPdiskDiskIter member functions

inline 
CPGPdiskDiskIter::CPGPdiskDiskIter(PGPdiskDiskSetRef diskSet) : 
	mDiskIter(kInvalidPGPdiskDiskIterRef), mWeCreated(FALSE)
{
	Create(diskSet);
}

inline 
CPGPdiskDiskIter::CPGPdiskDiskIter(PGPdiskDiskIterRef diskIter) : 
	mDiskIter(kInvalidPGPdiskDiskIterRef), mWeCreated(FALSE)
{
	Attach(diskIter);
}

inline 
CPGPdiskDiskIter::~CPGPdiskDiskIter()
{
	try
	{
		Clear();
	}
	catch (PGPError&) { }
}

inline 
PGPdiskDiskSetRef 
CPGPdiskDiskIter::DiskSet() const
{
	pgpAssert(IsAttached());
	return PGPdiskPeekDiskIterDiskSet(mDiskIter);
}

inline 
PGPBoolean 
CPGPdiskDiskIter::IsAttached() const
{
	return PGPdiskDiskIterRefIsValid(mDiskIter);
}

inline 
PGPUInt32 
CPGPdiskDiskIter::Count() const
{
	pgpAssert(IsAttached());
	return PGPdiskDiskIterCount(mDiskIter);
}

inline 
PGPInt32 
CPGPdiskDiskIter::Index() const
{
	pgpAssert(IsAttached());
	return PGPdiskDiskIterIndex(mDiskIter);
}

inline 
PGPBoolean 
CPGPdiskDiskIter::Current(CPGPdiskDisk& disk) const
{
	pgpAssert(IsAttached());

	PGPdiskDiskRef	diskRef;
	PGPBoolean		succeeded	= PGPdiskDiskIterDisk(mDiskIter, &diskRef);

	if (succeeded)
		disk = diskRef;

	return succeeded;
}

inline 
PGPBoolean 
CPGPdiskDiskIter::Move(
	PGPInt32		relOffset, 
	PGPBoolean		fromStart, 
	CPGPdiskDisk&	disk)
{
	pgpAssert(IsAttached());

	PGPdiskDiskRef	diskRef;
	PGPBoolean		succeeded	= PGPdiskDiskIterMove(mDiskIter, relOffset, 
		fromStart, &diskRef);

	if (succeeded)
		disk = diskRef;

	return succeeded;
}

inline 
PGPInt32 
CPGPdiskDiskIter::Seek(PGPdiskDiskRef soughtDisk)
{
	pgpAssert(IsAttached());
	return PGPdiskDiskIterSeek(mDiskIter, soughtDisk);
}

inline 
PGPBoolean 
CPGPdiskDiskIter::Next(CPGPdiskDisk& disk)
{
	pgpAssert(IsAttached());

	PGPdiskDiskRef	diskRef;
	PGPBoolean		succeeded	= PGPdiskDiskIterNext(mDiskIter, &diskRef);

	if (succeeded)
		disk = diskRef;

	return succeeded;
}

inline 
PGPBoolean 
CPGPdiskDiskIter::Prev(CPGPdiskDisk& disk)
{
	pgpAssert(IsAttached());

	PGPdiskDiskRef	diskRef;
	PGPBoolean		succeeded	= PGPdiskDiskIterPrev(mDiskIter, &diskRef);

	if (succeeded)
		disk = diskRef;

	return succeeded;
}

inline 
void 
CPGPdiskDiskIter::Rewind()
{
	pgpAssert(IsAttached());
	PGPdiskDiskIterRewind(mDiskIter);
}

inline 
PGPBoolean 
CPGPdiskDiskIter::SearchOnPath(const char *path, CPGPdiskDisk& disk)
{
	pgpAssert(IsAttached());

	PGPdiskDiskRef	diskRef;
	PGPBoolean		succeeded	= PGPdiskDiskIterSearchOnPath(mDiskIter, 
		path, &diskRef);

	if (succeeded)
		disk = diskRef;

	return succeeded;
}

inline 
PGPBoolean 
CPGPdiskDiskIter::SearchOnRoot(const char *root, CPGPdiskDisk& disk)
{
	pgpAssert(IsAttached());

	PGPdiskDiskRef	diskRef;
	PGPBoolean		succeeded	= PGPdiskDiskIterSearchOnRoot(mDiskIter, 
		root, &diskRef);

	if (succeeded)
		disk = diskRef;

	return succeeded;
}

inline 
void 
CPGPdiskDiskIter::Attach(PGPdiskDiskIterRef diskIter)
{
	if (mDiskIter == diskIter)
		return;

	Clear();

	mDiskIter = diskIter;
	mWeCreated = FALSE;
}

inline 
void 
CPGPdiskDiskIter::Create(PGPdiskDiskSetRef diskSet)
{
	Clear();

	PGPError	error	= PGPdiskNewDiskIter(diskSet, &mDiskIter);
	THROW_IF_PGPERROR(error);

	mWeCreated = TRUE;
}

inline 
void 
CPGPdiskDiskIter::Clear()
{
	if (WeCreated())
		PGPdiskFreeDiskIter(mDiskIter);

	mDiskIter = kInvalidPGPdiskDiskIterRef;
	mWeCreated = FALSE;
}


// Class CPGPdiskUser member functions

inline 
CPGPdiskUser::CPGPdiskUser() : 
	mUser(kInvalidPGPdiskUserRef), mWeCreated(FALSE)
{
}

inline 
CPGPdiskUser::CPGPdiskUser(PGPdiskUserRef user) :
	mUser(kInvalidPGPdiskUserRef), mWeCreated(FALSE)
{
	Attach(user);
}

inline 
CPGPdiskUser::~CPGPdiskUser()
{
	try
	{
		Clear();
	}
	catch (PGPError&) { }
}

inline 
CPGPdiskUser& 
CPGPdiskUser::operator=(PGPdiskUserRef user)
{
	Attach(user);
	return *this;
}

inline 
PGPBoolean 
CPGPdiskUser::IsAttached() const
{
	return PGPdiskUserRefIsValid(mUser);
}

inline 
PGPBoolean 
CPGPdiskUser::IsEqualUser(PGPdiskUserRef user) const
{
	return PGPdiskUserAreUsersEqual(mUser, user);
}

inline 
PGPBoolean 
CPGPdiskUser::HasPublicKey() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskUserProp_HasPublicKey);
}

inline 
PGPBoolean 
CPGPdiskUser::HasSymmetricKey() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskUserProp_HasSymmetricKey);
}

inline 
PGPBoolean 
CPGPdiskUser::IsReadOnly() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskUserProp_IsReadOnly);
}

inline 
PGPBoolean 
CPGPdiskUser::IsLocked() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskUserProp_IsLocked);
}

inline 
PGPBoolean 
CPGPdiskUser::IsAdminUser() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskUserProp_IsAdminUser);
}

inline 
PGPBoolean 
CPGPdiskUser::IsDisabled() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskUserProp_IsDisabled);
}

inline 
PGPBoolean 
CPGPdiskUser::IsCipheringEnabled() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskUserProp_IsCipheringEnabled);
}

inline 
PGPBoolean 
CPGPdiskUser::CanSetReadOnly() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskUserProp_CanSetReadOnly);
}

inline 
PGPBoolean 
CPGPdiskUser::CanSetLocked() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskUserProp_CanSetLocked);
}

inline 
PGPBoolean 
CPGPdiskUser::CanSetUserName() const
{
	pgpAssert(IsAttached());
	return GetBooleanProp(kPGPdiskUserProp_CanSetUserName);
}

inline 
PGPdiskContextRef 
CPGPdiskUser::Context() const
{
	pgpAssert(IsAttached());
	return PGPdiskPeekUserContext(mUser);
}

inline 
void 
CPGPdiskUser::GetUserName(CString& userName) const
{
	pgpAssert(IsAttached());

	GetDataProp(kPGPdiskUserProp_UserName, 
		userName.GetBuffer(kPGPdiskMaxUserNameSize), 
		kPGPdiskMaxUserNameSize);
	userName.ReleaseBuffer();
}

inline 
void 
CPGPdiskUser::GetPGPKeyID(CPGPKeyID& keyID) const
{
	pgpAssert(IsAttached());

	PGPKeyID	theKeyID;

	PGPError	error	= PGPdiskUserGetPGPKeyID(mUser, &theKeyID);
	THROW_IF_PGPERROR(error);

	keyID = theKeyID;
}

inline 
void 
CPGPdiskUser::SetIsReadOnly(PGPBoolean isReadOnly)
{
	pgpAssert(IsAttached());
	SetBooleanProp(kPGPdiskUserProp_IsReadOnly, isReadOnly);
}

inline 
void 
CPGPdiskUser::SetIsLocked(PGPBoolean isLocked)
{
	pgpAssert(IsAttached());
	SetBooleanProp(kPGPdiskUserProp_IsLocked, isLocked);
}

inline 
void 
CPGPdiskUser::SetIsDisabled(PGPBoolean isDisabled)
{
	pgpAssert(IsAttached());
	SetBooleanProp(kPGPdiskUserProp_IsDisabled, isDisabled);
}

inline 
void 
CPGPdiskUser::SetUserName(const char *userName)
{
	pgpAssert(IsAttached());
	SetDataProp(kPGPdiskUserProp_UserName, userName, strlen(userName));
}

inline 
void 
CPGPdiskUser::EnableCiphering(PGPdiskDiskRef disk, const char *passphrase, int passkeylen)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskUserEnableCiphering(mUser, disk, passphrase, passkeylen);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUser::DisableCiphering()
{
	pgpAssert(IsAttached());
	PGPdiskUserDisableCiphering(mUser);
}

inline 
void 
CPGPdiskUser::ChangePassphrase(const char * newPassphrase)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskUserChangePassphrase(mUser, newPassphrase);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUser::Create(
	PGPdiskUserRef	cipheringUser, 
	const char		*userName, 
	const char		*passphrase, 
	PGPBoolean		isReadOnly)
{
	pgpAssert(!IsAttached());

	PGPError	error	= PGPdiskNewUserWithSymmetricKey(cipheringUser, 
		userName, passphrase, isReadOnly, &mUser);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUser::Create(
	PGPdiskUserRef	cipheringUser, 
	PGPKeyDBObjRef	pubKey, 
	PGPBoolean		isReadOnly, 
	PGPBoolean		isLocked)
{
	pgpAssert(!IsAttached());

	PGPError	error	= PGPdiskNewUserWithPublicKey(cipheringUser, pubKey, 
		isReadOnly, isLocked, &mUser);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUser::DuplicateUser(PGPdiskUserRef user)
{
	pgpAssert(!IsAttached());

	PGPError	error	= PGPdiskDuplicateUser(user, &mUser);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUser::Attach(PGPdiskUserRef user)
{
	if (mUser == user)
		return;

	Clear();

	mUser = user;
	mWeCreated = FALSE;
}

inline 
void 
CPGPdiskUser::Clear()
{
	if (WeCreated())
		PGPdiskFreeUser(mUser);

	mUser = kInvalidPGPdiskUserRef;
	mWeCreated = FALSE;
}

inline 
PGPBoolean 
CPGPdiskUser::GetBooleanProp(PGPdiskUserPropName prop) const
{
	pgpAssert(IsAttached());

	PGPBoolean	boolVal;

	PGPError	error	= PGPdiskGetUserBooleanProp(mUser, prop, &boolVal);
	THROW_IF_PGPERROR(error);

	return boolVal;
}

inline 
PGPInt32 
CPGPdiskUser::GetNumericProp(PGPdiskUserPropName prop) const
{
	pgpAssert(IsAttached());

	PGPInt32	number;

	PGPError	error	= PGPdiskGetUserNumericProp(mUser, prop, &number);
	THROW_IF_PGPERROR(error);

	return number;
}

inline 
void 
CPGPdiskUser::GetDataProp(
	PGPdiskUserPropName	prop, 
	void				*pData, 
	PGPSize				availLength, 
	PGPSize				*pUsedLength) const
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskGetUserDataProp(mUser, prop, pData, 
		availLength, pUsedLength);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUser::SetBooleanProp(PGPdiskUserPropName prop, PGPBoolean boolVal)
{
	PGPError	error	= PGPdiskSetUserBooleanProp(mUser, prop, boolVal);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUser::SetNumericProp(PGPdiskUserPropName prop, PGPInt32 number)
{
	PGPError	error	= PGPdiskSetUserNumericProp(mUser, prop, number);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUser::SetDataProp(
	PGPdiskUserPropName	prop, 
	const void			*pData, 
	PGPSize				length)
{
	PGPError	error	= PGPdiskSetUserDataProp(mUser, prop, pData, length);
	THROW_IF_PGPERROR(error);
}


// Class CPGPdiskUserSet member functions

inline 
CPGPdiskUserSet::CPGPdiskUserSet() : 
	mUserSet(kInvalidPGPdiskUserSetRef), mWeCreated(FALSE)
{
	CreateEmpty();
}

inline 
CPGPdiskUserSet::CPGPdiskUserSet(PGPdiskUserRef user) : 
	mUserSet(kInvalidPGPdiskUserSetRef), mWeCreated(FALSE)
{
	CreateSingleton(user);
}

inline 
CPGPdiskUserSet::CPGPdiskUserSet(PGPdiskUserSetRef userSet) : 
	mUserSet(kInvalidPGPdiskUserSetRef), mWeCreated(FALSE)
{
	Attach(userSet);
}

inline 
CPGPdiskUserSet::~CPGPdiskUserSet()
{
	try
	{
		Clear();
	}
	catch (PGPError&) { }
}

inline 
PGPBoolean 
CPGPdiskUserSet::IsAttached() const
{
	return PGPdiskUserSetRefIsValid(mUserSet);
}

inline 
PGPUInt32 
CPGPdiskUserSet::Count() const
{
	pgpAssert(IsAttached());
	return PGPdiskUserSetCount(mUserSet);
}

inline 
void 
CPGPdiskUserSet::Add(PGPdiskUserSetRef usersToAdd)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskUserSetAddUsers(mUserSet, usersToAdd);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUserSet::Add(PGPdiskUserRef userToAdd)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskUserSetAddUser(mUserSet, userToAdd);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUserSet::Remove(PGPdiskUserSetRef usersToRemove)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskUserSetRemoveUsers(mUserSet, usersToRemove);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUserSet::Remove(PGPdiskUserRef userToRemove)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskUserSetRemoveUser(mUserSet, userToRemove);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUserSet::Empty()
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskUserSetEmptyUserSet(mUserSet);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUserSet::Attach(PGPdiskUserSetRef userSet)
{
	if (mUserSet == userSet)
		return;

	Clear();

	mUserSet = userSet;
	mWeCreated = FALSE;
}

inline 
void 
CPGPdiskUserSet::CreateEmpty()
{
	Clear();

	PGPError	error	= PGPdiskNewUserSet(&mUserSet);
	THROW_IF_PGPERROR(error);

	mWeCreated = TRUE;
}

inline 
void 
CPGPdiskUserSet::CreateSingleton(PGPdiskUserRef user)
{
	Clear();

	PGPError	error	= PGPdiskNewSingletonUserSet(user, &mUserSet);
	THROW_IF_PGPERROR(error);

	mWeCreated = TRUE;
}

inline 
void 
CPGPdiskUserSet::Clear()
{
	if (WeCreated())
		PGPdiskFreeUserSet(mUserSet);

	mUserSet = kInvalidPGPdiskUserSetRef;
	mWeCreated = FALSE;
}


// Class CPGPdiskUserIter member functions

inline 
CPGPdiskUserIter::CPGPdiskUserIter(PGPdiskUserSetRef userSet) : 
	mUserIter(kInvalidPGPdiskUserIterRef), mWeCreated(FALSE)
{
	Create(userSet);
}

inline 
CPGPdiskUserIter::CPGPdiskUserIter(PGPdiskUserIterRef userIter) : 
	mUserIter(kInvalidPGPdiskUserIterRef), mWeCreated(FALSE)
{
	Attach(userIter);
}

inline 
CPGPdiskUserIter::~CPGPdiskUserIter()
{
	try
	{
		Clear();
	}
	catch (PGPError&) { }
}

inline 
PGPdiskUserSetRef 
CPGPdiskUserIter::UserSet() const
{
	pgpAssert(IsAttached());
	return PGPdiskPeekUserIterUserSet(mUserIter);
}

inline 
PGPBoolean 
CPGPdiskUserIter::IsAttached() const
{
	return PGPdiskUserIterRefIsValid(mUserIter);
}

inline 
PGPUInt32 
CPGPdiskUserIter::Count() const
{
	pgpAssert(IsAttached());
	return PGPdiskUserIterCount(mUserIter);
}

inline 
PGPInt32 
CPGPdiskUserIter::Index() const
{
	pgpAssert(IsAttached());
	return PGPdiskUserIterIndex(mUserIter);
}

inline 
PGPBoolean 
CPGPdiskUserIter::Current(CPGPdiskUser& user) const
{
	pgpAssert(IsAttached());

	PGPdiskUserRef	userRef;
	PGPBoolean		succeeded	= PGPdiskUserIterUser(mUserIter, &userRef);

	if (succeeded)
		user = userRef;

	return succeeded;
}

inline 
PGPBoolean 
CPGPdiskUserIter::Move(
	PGPInt32		relOffset, 
	PGPBoolean		fromStart, 
	CPGPdiskUser&	user)
{
	pgpAssert(IsAttached());

	PGPdiskUserRef	userRef;
	PGPBoolean		succeeded	= PGPdiskUserIterMove(mUserIter, relOffset, 
		fromStart, &userRef);

	if (succeeded)
		user = userRef;

	return succeeded;
}

inline 
PGPInt32 
CPGPdiskUserIter::Seek(PGPdiskUserRef soughtUser)
{
	pgpAssert(IsAttached());
	return PGPdiskUserIterSeek(mUserIter, soughtUser);
}

inline 
PGPBoolean 
CPGPdiskUserIter::Next(CPGPdiskUser& user)
{
	pgpAssert(IsAttached());

	PGPdiskUserRef	userRef;
	PGPBoolean		succeeded	= PGPdiskUserIterNext(mUserIter, &userRef);

	if (succeeded)
		user = userRef;

	return succeeded;
}

inline 
PGPBoolean 
CPGPdiskUserIter::Prev(CPGPdiskUser& user)
{
	pgpAssert(IsAttached());

	PGPdiskUserRef	userRef;
	PGPBoolean		succeeded	= PGPdiskUserIterPrev(mUserIter, &userRef);

	if (succeeded)
		user = userRef;

	return succeeded;
}

inline 
void 
CPGPdiskUserIter::Rewind()
{
	pgpAssert(IsAttached());
	PGPdiskUserIterRewind(mUserIter);
}

inline 
PGPBoolean 
CPGPdiskUserIter::SearchOnUserName(const char *userName, CPGPdiskUser& user)
{
	pgpAssert(IsAttached());

	PGPdiskUserRef	userRef;
	PGPBoolean		succeeded	= PGPdiskUserIterSearchOnUserName(mUserIter, 
		userName, &userRef);

	if (succeeded)
		user = userRef;

	return succeeded;
}

inline 
PGPBoolean 
CPGPdiskUserIter::FindEqualUser(
	const CPGPdiskUser&	user, 
	CPGPdiskUser&		equalUser)
{
	pgpAssert(IsAttached());

	PGPdiskUserRef	userRef;
	PGPBoolean		succeeded	= PGPdiskUserIterFindEqualUser(mUserIter, 
		user, &userRef);

	if (succeeded)
		equalUser = userRef;

	return succeeded;
}

inline 
PGPBoolean 
CPGPdiskUserIter::FindAdminUser(CPGPdiskUser& user)
{
	pgpAssert(IsAttached());

	PGPdiskUserRef	userRef;
	PGPBoolean		succeeded	= PGPdiskUserIterFindAdminUser(mUserIter, 
		&userRef);

	if (succeeded)
		user = userRef;

	return succeeded;
}

inline 
void 
CPGPdiskUserIter::SetAdminUser(
	PGPdiskUserRef userForCiphering, 
	PGPdiskUserRef oldAdminForCiphering)
{
	pgpAssert(IsAttached());

	PGPError	error	= PGPdiskUserIterSetAdminUser(mUserIter, 
		userForCiphering, oldAdminForCiphering);
	THROW_IF_PGPERROR(error);
}

inline 
void 
CPGPdiskUserIter::Attach(PGPdiskUserIterRef userIter)
{
	if (mUserIter == userIter)
		return;

	Clear();

	mUserIter = userIter;
	mWeCreated = FALSE;
}

inline 
void 
CPGPdiskUserIter::Create(PGPdiskUserSetRef userSet)
{
	Clear();

	PGPError	error	= PGPdiskNewUserIter(userSet, &mUserIter);
	THROW_IF_PGPERROR(error);

	mWeCreated = TRUE;
}

inline 
void 
CPGPdiskUserIter::Clear()
{
	if (WeCreated())
		PGPdiskFreeUserIter(mUserIter);

	mUserIter = kInvalidPGPdiskUserIterRef;
	mWeCreated = FALSE;
}

_PGP_END

#endif	// ] Included_CPGPdiskEngine_h
