/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: CPGPnetFilterAdapter.cpp,v 1.5 2002/08/06 20:10:26 dallen Exp $
____________________________________________________________________________*/

#include "pgpnetBase.h"

#include "pgpnetGlobal.h"
#include "pgpnetPacket.h"
#include "pgpnetPacketHelper.h"
#include "CPGPnetAdapter.h"
#include "CPGPnetFilterAdapter.h"
#include "pgpNetComm.h"
#include "pgpNetIPdef.h"
#include "pgpnetIpHelper.h"
#include "pgpKernelTiming.h"

/* PM includes */
#include "pgpNetPM.h"
#include "pgpNetWin32.h"

#include "CPGPnetDriver.h"
#include "CPGPnetFilterDriver.h"

#include "CPGPnetIPPacket.h"

#include "pgpnetDebug.h"

static PGPnetLastFragmentList* 
LastFragmentListQuery(PGPnetLastFragmentList *list,
					  PGPUInt32 ipAddress,
					  USHORT ipId);

static PGPnetLastFragmentList* 
LastFragmentListAdd(PGPnetLastFragmentList **list,
					PGPUInt32 ipAddress,
					USHORT ipId);

static VOID
LastFragmentListFree(PGPnetLastFragmentList **list,
					 PGPUInt32 ipAddress,
					 USHORT ipId);

static VOID
LastFragmentListFreeAll(PGPnetLastFragmentList *list);

VOID CPGPnetFilterAdapter::DoCfgEthernetTransform(PGPnetPacket *inPacket)
{
	PGPnetPacket* packet = inPacket;

	DBG_FUNC("DoCfgEthernetTransform");

	while (packet != NULL) {
		NdisMoveMemory((PCHAR)packet->xformBlock + 6, mHardwareAddress, 6);
		packet = packet->link;
	}
}

VOID CPGPnetFilterAdapter::DoMTUTransform(PGPnetPacket* packet)
{
	PETHERNET_HEADER	src_eth_header;
	PIP_HEADER			src_ip_header;

	PETHERNET_HEADER	xform_eth_header;
	USHORT				xform_eth_protocol;
	PIP_HEADER			xform_ip_header;

	PICMP_HEADER		icmp_header;
	PTYPE3_ICMP_HEADER	type3_icmp_header;

	UCHAR				hardwareAddress[6];
	USHORT				mtu;

	src_eth_header = (PETHERNET_HEADER)packet->srcBlock;
	src_ip_header = (PIP_HEADER)(packet->srcBlock + ETHER_HEADER_SIZE);

	xform_eth_header = (PETHERNET_HEADER)packet->xformBlock;
	xform_eth_protocol = *((PUSHORT)(&xform_eth_header->eth_protocolType[0]));
	xform_ip_header = (PIP_HEADER)(packet->xformBlock + ETHER_HEADER_SIZE);
	icmp_header	= (PICMP_HEADER)((UCHAR*)xform_ip_header + IP_HEADER_SIZE);
	type3_icmp_header = (PTYPE3_ICMP_HEADER)icmp_header;
	
	// switch hardware address
	NdisMoveMemory(hardwareAddress, xform_eth_header->eth_srcAddress, 6);
	NdisMoveMemory(xform_eth_header->eth_srcAddress, xform_eth_header->eth_dstAddress, 6);
	NdisMoveMemory(xform_eth_header->eth_dstAddress, hardwareAddress, 6);

	// change ethernet protocol
	// Still IP packet

	// change misc ip fields

	// change ip address
	xform_ip_header->ip_dest = xform_ip_header->ip_src;

	// change ip protocol
	xform_ip_header->ip_prot = PROTOCOL_ICMP;

	xform_ip_header->ip_chksum = 0;

	// construct icmp header
	icmp_header->icmp_type = ICMP_TYPE_UNREACHABLE;
	icmp_header->icmp_code = ICMP_UNREACHABLE_CODE_FRAGMENT;
	icmp_header->icmp_checksum = 0;

	// copy icmp data
	if (htons(xform_ip_header->ip_len) > htons(src_ip_header->ip_len))
		mtu = htons(xform_ip_header->ip_len) - htons(src_ip_header->ip_len);
	else
		mtu = 0;
	type3_icmp_header->u.icmp_unused = 0;
	type3_icmp_header->u.icmp_code5.code5_mtu = mtu;
	type3_icmp_header->u.icmp_code5.code5_unused = 0;

	NdisMoveMemory(&type3_icmp_header->icmp_ip_header, src_ip_header, IP_HEADER_SIZE);
	NdisMoveMemory(&type3_icmp_header->icmp_ip_data, src_ip_header + IP_HEADER_SIZE, 8);

	// adjust ip len
	xform_ip_header->ip_len = htons(IP_HEADER_SIZE + sizeof(TYPE3_ICMP_HEADER));

	// calculate ip checksum
	xform_ip_header->ip_chksum = HeaderChecksum((USHORT*)xform_ip_header, TYPE_IP);

	// caculate icmp checksum
	icmp_header->icmp_checksum = HeaderChecksum((USHORT*)icmp_header, TYPE_ICMP);

	// adjust transform buffer length
	packet->xformBlockLen = ETHER_HEADER_SIZE + htons(xform_ip_header->ip_len);
	
	// set last fragment
	packet->lastXformBlock = TRUE;
	
}

VOID CPGPnetFilterAdapter::DoCopyTransforms(PGPnetPacket *pgpnetPacket)
{
	DBG_FUNC("CPGPnetFilterAdapter::DoCopyTransform")

	PGPnetPacket* packet = pgpnetPacket;
	PGPnetPacket* next;

	DBG_ENTER();

	while (packet != NULL)
	{
		DoCopyTransform(packet);
		if (packet->lastSrcBlock)
			next = NULL;
		else
			next = packet->link;

		packet = next;
	}

	DBG_LEAVE(0);
}

VOID CPGPnetFilterAdapter::DoCopyTransform(PGPnetPacket* pgpnetPacket)
{
	DBG_FUNC("CPGPnetFilterAdapter::DoCopyTransform")

	DBG_ENTER();

	CopySrcBlockToXformBlock(pgpnetPacket);

	if (pgpnetPacket->firstSrcBlock)
		pgpnetPacket->firstXformBlock = TRUE;
	if (pgpnetPacket->lastSrcBlock)
		pgpnetPacket->lastXformBlock = TRUE;

	PGPnetAdjustXformPacket(pgpnetPacket);

	DBG_LEAVE(0);
}

CPGPnetFilterAdapter::CPGPnetFilterAdapter()
{
	HardwareAddressDetected = FALSE;
	RealAdapterMedia = (NDIS_MEDIUM) (PGPUInt32) -1;
	MaximumFrameSize = MAX_ETHER_FRAME_SIZE - ETHER_HEADER_SIZE;
}

CPGPnetFilterAdapter::~CPGPnetFilterAdapter()
{
}

PGPBoolean CPGPnetFilterAdapter::SetRealAdapterName(PNDIS_STRING String)
{

	DBG_FUNC("CPGPnetFitlerAdapter::SetRealAdapterName")

	NDIS_STATUS status;
	
	DBG_ENTER();
/*
	status = NdisAllocateMemory((VOID**)&RealAdapterName.Buffer, String->Length, 0, HighestAcceptableAddress);
    if (status != NDIS_STATUS_SUCCESS)
	{
        DBG_PRINT(("!!!!! NdisAllocateMemory failed status=%Xh\n", status););
		DBG_LEAVE(0);
		return (FALSE);
	}

	NdisMoveMemory(RealAdapterName.Buffer, String->Buffer, String->Length);
	RealAdapterName.MaximumLength = String->Length;
	RealAdapterName.Length = String->Length;
*/
	RtlInitUnicodeString(
	&RealAdapterName,
	String->Buffer
	);

	DBG_LEAVE(0);
	return TRUE;
}

VOID CPGPnetFilterAdapter::AllocatePacketPool()
{
	sent_plainpacket_list = (PGPnetPacket*)NULL;
	incoming_plaintransferComplete_wait_list	= (PGPnetPacket*)NULL;
	incoming_ipsectransferComplete_wait_list	= (PGPnetPacket*)NULL;
	incoming_indicateComplete_wait_list = (PGPnetPacket*)NULL;
	incoming_fragment_indicateComplete_wait_list = (PGPnetPacket*)NULL;
	incoming_mac_transferdata_wait_list = (PGPnetPacket*)NULL;

	outgoing_multiple_ipsecpacket_list = (PGPnetPacket*)NULL;

	outgoing_multiple_cfgpacket_list = (PGPnetPacket*)NULL;

	outgoing_local_packet_head_list = (PPGPnetPacketHead)NULL;
	outgoing_cfg_packet_head_list = (PPGPnetPacketHead)NULL;
	incoming_packet_head_list = (PPGPnetPacketHead)NULL;
	sent_packet_head_list = (PPGPnetPacketHead)NULL;

	last_fragment_list = (PGPnetLastFragmentList*)NULL;
}

VOID CPGPnetFilterAdapter::FreePacketPool()
{
	PPGPnetPacketHead		head;

	while ((head = PacketHeadDequeue(&outgoing_local_packet_head_list)) != NULL)
	{
		PGPNdisPacketHeadFreeList(head);
		PGPNdisPacketHeadFree(head);
	}
	while ((head = PacketHeadDequeue(&outgoing_cfg_packet_head_list)) != NULL)
	{
		PGPNdisPacketHeadFreeList(head);
		PGPNdisPacketHeadFree(head);
	}
	while ((head = PacketHeadDequeue(&incoming_packet_head_list)) != NULL)
	{
		PGPNdisPacketHeadFreeList(head);
		PGPNdisPacketHeadFree(head);
	}

	LastFragmentListFreeAll(last_fragment_list);

	sent_plainpacket_list = (PGPnetPacket*)NULL;
	incoming_plaintransferComplete_wait_list	= (PGPnetPacket*)NULL;
	incoming_ipsectransferComplete_wait_list	= (PGPnetPacket*)NULL;
	incoming_indicateComplete_wait_list = (PGPnetPacket*)NULL;
	incoming_fragment_indicateComplete_wait_list = (PGPnetPacket*)NULL;

	outgoing_multiple_ipsecpacket_list = (PGPnetPacket*)NULL;
	outgoing_multiple_cfgpacket_list = (PGPnetPacket*)NULL;

	outgoing_local_packet_head_list = (PPGPnetPacketHead)NULL;
	outgoing_cfg_packet_head_list = (PPGPnetPacketHead)NULL;
	incoming_packet_head_list = (PPGPnetPacketHead)NULL;
	sent_packet_head_list = (PPGPnetPacketHead)NULL;

	last_fragment_list = (PGPnetLastFragmentList*)NULL;
}

PGPnetPacketStatus	
CPGPnetFilterAdapter::PGPnetFilterPacketCheck(PGPnetPacket *pgpnetPacket, PGPBoolean incoming)
{

	PGPnetPacketStatus	packetStatus = PacketFurtherCheck;

	PVOID				working_block;
	UINT				working_block_len;

	PETHERNET_HEADER	eth_header;
	USHORT				eth_protocol;

	USHORT				eth_header_len;

	PIP_HEADER			ip_header;

	PGPUInt16			port = 0;

#if !defined(CHICAGO) && !defined(NDIS40)
	PGPBoolean			Broadcast = FALSE;
#endif
	DBG_FUNC("PGPnetFilterPacketCheck");

	working_block = pgpnetPacket->srcBlock;
	working_block_len = pgpnetPacket->srcBlockLen;

	do 
	{
		if (Media != NdisMedium802_3 && Media != NdisMediumWan)
		{
			packetStatus = PacketDrop;
			break;
		}


		/*
		 *	Stage 1. Get the ethernet header. Determines if this is an ip packet. If not bail out early,
		 *	otherwise go to stage 2.
		 */
		eth_header   = (PETHERNET_HEADER) working_block;
		eth_protocol = *((PUSHORT)(&eth_header->eth_protocolType[0]));
		eth_header_len = sizeof(ETHERNET_HEADER);

		if ( RealAdapterMedia == NdisMediumWan &&
			(eth_protocol == PPPOE_DISCOVERY_PORT_NET || eth_protocol == PPPOE_SESSION_PORT_NET))
		{
			packetStatus = PacketDrop;
			break;
		}

		if (eth_protocol != IPPROT_NET)
		{
			if (eth_protocol == ARPPROT_NET && Media != NdisMediumWan && !incoming)
			{
				PGPUInt32 IpAddress;
				IpAddress = GetIPAddressFromARP((PVOID)((UCHAR*)eth_header + eth_header_len));
				if (IpAddress)
					mIpAddress = IpAddress;
			}
			packetStatus = PacketPassthrough;
			break;
		}

		if (BroadcastEthernetAddress(eth_header->eth_dstAddress) && !incoming)
		{
#if !defined(CHICAGO) && !defined(NDIS40)
			Broadcast = TRUE;
#else
			packetStatus = PacketPassthrough;
			break;
#endif
		}

		if (incoming)
			NdisMoveMemory(pgpnetPacket->xformBlock, pgpnetPacket->srcBlock, ETHER_HEADER_SIZE);
		else if (!HardwareAddressDetected && 
			(eth_header->eth_srcAddress[0] != 0xFF ||
			eth_header->eth_srcAddress[1] != 0xFF ||
			eth_header->eth_srcAddress[2] != 0xFF ||
			eth_header->eth_srcAddress[3] != 0 ||
			eth_header->eth_srcAddress[4] != 0 ||
			eth_header->eth_srcAddress[5] != 0))
		{
			HardwareAddressDetected = TRUE;
			NdisMoveMemory(mHardwareAddress, eth_header->eth_srcAddress, 6);
#if defined(NDIS40)
			if (RealAdapterMedia == NdisMediumWan)
				PGPnetRASconnect();
#endif
		}

		/*
		 *	Stage 2 Get the ip header.
		 */
		
		if (working_block_len >= eth_header_len + sizeof(IP_HEADER))
		{

			ip_header = (PIP_HEADER) ( (PCHAR)working_block + eth_header_len);
			working_block = (PCHAR)working_block + eth_header_len;
			working_block_len -= eth_header_len;
		}
		else
		{
			packetStatus = PacketDrop;
			break;
		}


		if (!mWanIPAddressDetected && !incoming)
		{
			mIpAddress = ip_header->ip_src;
			mWanIPAddressDetected = TRUE;
		}

		/*
		 *	Stage 4. If it's UDP, Get the UDP header.
		 *	Determine if this is a UDP 500 packet. If it is, bail out.
		 */

		if (ip_header->ip_prot == PROTOCOL_UDP)
		{
			if( working_block_len <= sizeof(struct tag_IP_HEADER) )		// FIX!!! ONLY work for ordinary ipv4 header.
			{
				packetStatus = PacketDrop;
				break;
			}
			else
			{
				port = ((PUDP_HEADER)( (UCHAR*)working_block + sizeof(IP_HEADER)))->dest_port;

#if !defined(CHICAGO) && !defined(NDIS40)
				if (port == UDP_PORT_BOOTPS_NET || port == UDP_PORT_BOOTPC_NET)
				{
					PUDP_HEADER			udp_header;
					PDHCP_HEADER		dhcp_header;
					CPGPnetDHCPPacket	*dhcpPacket;

					udp_header = (PUDP_HEADER) ( (PCHAR)working_block + sizeof(IP_HEADER));
					working_block = (PCHAR)working_block + sizeof(IP_HEADER);
					working_block_len -= sizeof(IP_HEADER);

					dhcp_header = (PDHCP_HEADER) ( (PCHAR)working_block + sizeof(UDP_HEADER));
					working_block = (PCHAR)working_block + sizeof(UDP_HEADER);
					working_block_len -= sizeof(UDP_HEADER);

					if (working_block_len <= sizeof(DHCP_HEADER))
						break;

					// now we have the DHCP Packet
					dhcpPacket = new CPGPnetDHCPPacket(working_block, working_block_len);

					switch (dhcpPacket->GetDHCPType())
					{
					case TYPE_DISCOVER:
						break;

					case TYPE_OFFER:
						break;

					case TYPE_REQUEST:
						break;

					case TYPE_DECLINE:
						break;

					case TYPE_ACK:
						{
							PGPUInt32 dns1;
							PGPUInt32 dns2;

							dhcpPacket->GetDHCPDNSServer(&dns1, &dns2);
							PGPnetFilterDriver->dns1 = dns1;
							PGPnetFilterDriver->dns2 = dns2;
						}
						break;
					case TYPE_NAK:
						break;

					case TYPE_RELEASE:
						break;

					case TYPE_INFORM:
						break;

					case TYPE_INVALID:
						break;

					default:
						ASSERT(0);
						break;
					}

					delete dhcpPacket;
				}
#endif
			}
		} else if (ip_header->ip_prot == PROTOCOL_TCP)	{
			if( working_block_len <= sizeof(struct tag_IP_HEADER) )		// FIX!!! ONLY work for ordinary ipv4 header.
			{
				packetStatus = PacketDrop;
				break;
			}
			else
				NdisMoveMemory(&(port), (UCHAR*)working_block + sizeof(IP_HEADER) + 2, 2);
		}

	} while (0);

#if !defined(CHICAGO) && !defined(NDIS40)
	if(Broadcast && !incoming)
		packetStatus = PacketPassthrough;
#endif

	return packetStatus;
}


NDIS_STATUS	CPGPnetFilterAdapter::SendPacket(PGPnetPacket *pgpnetPacket)
{
	DBG_FUNC("CPGPnetFilterAdapter::SendPacket")

	NDIS_STATUS status;

	DBG_ENTER();

	PacketEnqueue(&sent_plainpacket_list, pgpnetPacket);

#ifdef NDIS_MINIPORT_DRIVER
	NdisSetPacketFlags(pgpnetPacket->xformPacket, pgpnetPacket->SendFlag);
#endif
	ProtocolSendRequest(&status,
				NdisBindingHandleToRealAdapter,
				pgpnetPacket->xformPacket);

	if (status != NDIS_STATUS_PENDING)
	{
		pgpnetPacket = PacketRemoveByXformPacket(&sent_plainpacket_list, pgpnetPacket->xformPacket);
		FreePGPnetPacket(pgpnetPacket);
	}

	SentPackets++;

	DBG_LEAVE(status);

	return status;
}

NDIS_STATUS	CPGPnetFilterAdapter::SendCfgPacket(PGPnetPacket *pgpnetPacket)
{
	DBG_FUNC("CPGPnetFilterAdapter::SendCfgPacket")

	NDIS_STATUS status;

	DBG_ENTER();

	PacketEnqueue(&sent_cfgpacket_list, pgpnetPacket);

	ProtocolSendRequest(&status,
				NdisBindingHandleToRealAdapter,
				pgpnetPacket->xformPacket);

	if (status != NDIS_STATUS_PENDING)
	{
		pgpnetPacket = PacketRemoveByXformPacket(&sent_cfgpacket_list, pgpnetPacket->xformPacket);
		FreePGPnetPacket(pgpnetPacket);
	}

	SentPackets++;

	DBG_LEAVE(status);

	return status;
}

NDIS_STATUS CPGPnetFilterAdapter::SendPacket(PNDIS_PACKET Packet, PGPUserValue userValue, PGPBoolean IsCfgPacket)
{
	DBG_FUNC("CPGPnetFilterAdapter::SendPacket")

	NDIS_STATUS		status = NDIS_STATUS_SUCCESS;

	PGPnetPacket*		pgpnetPacket;

	PGPnetPacketStatus	packetStatus;
	PGPnetPMStatus		pmStatus;

	PGPnetPacketHead**		packet_head_list;

	DBG_ENTER();

	if (!DeviceOn())
	{
		status = NDIS_STATUS_FAILURE;

		DBG_LEAVE(status);
		return status;

	}
	if (IsCfgPacket)
		packet_head_list = &outgoing_cfg_packet_head_list;
	else
		packet_head_list = &outgoing_local_packet_head_list;

	CPGPnetFilterDriver::GetCurrentDriver()->activeFilterAdapter = this;

	do
	{
		pgpnetPacket = AllocatePGPnetPacket();
		if (pgpnetPacket == NULL)
		{
			status = NDIS_STATUS_RESOURCES;
			break;
		}

#ifndef NDIS40_MINIPORT
		pgpnetPacket->Binding = (BindingContext*)userValue;
#else
		pgpnetPacket->SendFlag = *(PUINT)userValue;
#endif

		pgpnetPacket->srcPacket = Packet;

		PGPCopyPacketToBlock(pgpnetPacket->srcPacket, pgpnetPacket->srcBlock, &pgpnetPacket->srcBlockLen);

		packetStatus = PGPnetFilterPacketCheck(pgpnetPacket, FALSE);

		switch (packetStatus)
		{
		case PacketDrop:
			FreePGPnetPacket(pgpnetPacket);
			status = NDIS_STATUS_NOT_ACCEPTED;
			break;

		case PacketPassthrough:
			DoCopyTransform(pgpnetPacket);
			if (IsCfgPacket)
				status = SendCfgPacket(pgpnetPacket);
			else
				status = SendPacket(pgpnetPacket);
			break;

		case PacketEncrypt:
			ASSERT(0);
			break;

		case PacketFurtherCheck:

			pgpnetPacket->pSrcIPsecBuffer->data = pgpnetPacket->srcBlock + ETHER_HEADER_SIZE;
			pgpnetPacket->pSrcIPsecBuffer->dataSize = pgpnetPacket->srcBlockLen - ETHER_HEADER_SIZE;
			pgpnetPacket->ethernetAddr = pgpnetPacket->srcBlock;
			pgpnetPacket->lastSrcBlock = TRUE;
			pgpnetPacket->lastXformBlock = TRUE;
			pgpnetPacket->firstSrcBlock = TRUE;
			pgpnetPacket->firstXformBlock = TRUE;
			pgpnetPacket->defaultAdapterMTU = MaximumFrameSize;

			pmStatus = PGPnetHandleOutgoingPacket(PGPnetDriver->PolicyManagerHandle, &pgpnetPacket);

			switch (pmStatus) {	
			case kPGPNetPMPacketSent:
				PGPnetAdjustXformPackets(pgpnetPacket);
				status = SendPackets(pgpnetPacket, FALSE /* not Cfg */);

				break;
			case kPGPNetPMPacketWaiting:
			case kPGPNetPMPacketDrop:

				PGPNdisPacketFreeList(pgpnetPacket);
				status = NDIS_STATUS_SUCCESS;

				break;
			case kPGPNetPMPacketClear:

				DoCopyTransforms(pgpnetPacket);
				status = SendPackets(pgpnetPacket, IsCfgPacket);

				break;
			case kPGPNetPMPacketForget:
				status = NDIS_STATUS_SUCCESS;
				break;
			case kPGPNetPMPacketCfg:

				PGPnetAdjustXformPackets(pgpnetPacket);
				DoCfgEthernetTransform(pgpnetPacket);
				status = SendPackets(pgpnetPacket, TRUE /* Cfg */);

				break;
			default:
				ASSERT(0);
			} 
			break;
		default:
			ASSERT(0);
		}

	} while (0);

	DBG_LEAVE(status);

	return status;

}

//#define SEND_OUT_OF_ORDER_PACKETS

NDIS_STATUS CPGPnetFilterAdapter::SendPackets(
		PGPnetPacket *pgpNetPacket,
		PGPBoolean bIsCfgPacket)
{	
	
	DBG_FUNC("CPGPnetFilterAdapter::SendPackets")

	NDIS_STATUS status = NDIS_STATUS_SUCCESS;
	NDIS_STATUS returnStatus = NDIS_STATUS_SUCCESS;
	PGPnetPacket *packet, *next;

	PGPnetPacket** packet_list;

	DBG_ENTER();

	if (bIsCfgPacket)
	{
		packet_list = &outgoing_multiple_cfgpacket_list;
	}
	else
	{
		packet_list = &outgoing_multiple_ipsecpacket_list;
	}
	// Get Information from the first packet.

	packet = pgpNetPacket;

#ifdef SEND_OUT_OF_ORDER_PACKETS
	// skip to last packet
	while (packet->link != NULL && packet->link->xformBlockLen > 0) {
		packet = packet->link;
	}

	// send out the last packet in the chain first
	if (packet) {
		PacketEnqueue(packet_list, packet);

		NdisSend(&status, NdisBindingHandleToRealAdapter, packet->xformPacket);
		if (status == NDIS_STATUS_PENDING)
		{
			//status = NDIS_STATUS_SUCCESS;
		}
		else
		{
			packet = PacketRemoveByXformPacket(packet_list, packet->xformPacket);
			FreePGPnetPacket(packet);
		}
		if (packet->lastSrcBlock)
			returnStatus = status;
	}
	packet = packetHead->link;
#endif

	while (packet != NULL)
	{

#ifdef SEND_OUT_OF_ORDER_PACKETS
		if (packet->lastSrcBlock)	// we already sent the last one
			break;
#endif
		next = packet->link;

		if (packet->xformBlockLen == 0) {
			FreePGPnetPacket(packet);
			break;
		}

		PacketEnqueue(packet_list, packet);

		ProtocolSendRequest(&status, NdisBindingHandleToRealAdapter, packet->xformPacket);
		if (status == NDIS_STATUS_PENDING)
		{
			//status = NDIS_STATUS_SUCCESS;
		}
		else
		{
			packet = PacketRemoveByXformPacket(packet_list, packet->xformPacket);
			FreePGPnetPacket(packet);
		}
#ifdef SEND_OUT_OF_ORDER_PACKETS
		if (next->lastSrcBlock)
#else
		if (packet->lastSrcBlock)
#endif
			returnStatus = status;

		packet = next;
	}

	SentPackets++;

	DBG_LEAVE(returnStatus);
	return returnStatus;
}

VOID CPGPnetFilterAdapter::FragmentPacket(PGPnetPacketHead* packetHead, PGPBoolean incoming)
{
	DBG_FUNC("CPGPnetFilterAdapter::FragmentPacket")

	USHORT	IPDataTotalLength;
	UCHAR	ip_prot; // Should always be 0x32?

	USHORT	offset;

	USHORT ip_len;

	PGPnetPacket* packet;
	PGPnetPacket* next;
	PIP_HEADER first_ip_hdr;
	PIP_HEADER ip_hdr;
	PUCHAR		first_xformBlock;
	PUCHAR		first_srcBlock;
	PGPBoolean DF = FALSE;

	DBG_ENTER();

	packet = packetHead->link;
	ASSERT(packet != NULL);

	first_srcBlock = packet->srcBlock;
	first_xformBlock = packet->xformBlock;
	first_ip_hdr = (PIP_HEADER)((UCHAR*)packet->xformBlock + ETHER_HEADER_SIZE);
	IPDataTotalLength = htons(first_ip_hdr->ip_len) - IP_HEADER_SIZE;
	ip_prot = first_ip_hdr->ip_prot;

	NdisMoveMemory(first_xformBlock /*dest*/, packet->srcBlock, ETHER_HEADER_SIZE);
	offset = 0;

	DF = IP_DONT_FRAGMENT(first_ip_hdr->ip_foff);

	if ( (DF) && IP_MORE_FRAGMENT(first_ip_hdr->ip_foff) && !incoming)
	{
		DoMTUTransform(packet);
		PGPNdisPacketFreeList(packet->link);
		packet->link = NULL;
		DBG_LEAVE(0);
		return;
	}

	DF = FALSE;

	while (packet != NULL)
	{
		packet->HeaderBuffer = packet->xformBlock;
		packet->HeaderBufferSize = ETHER_HEADER_SIZE;

		NdisMoveMemory(packet->xformBlock /*dest*/, first_xformBlock, ETHER_HEADER_SIZE);

		packet->LookaheadBuffer = packet->xformBlock + ETHER_HEADER_SIZE;

		ip_hdr = (PIP_HEADER)((UCHAR*)packet->xformBlock + ETHER_HEADER_SIZE);

		NdisMoveMemory(ip_hdr /*dest*/, first_ip_hdr, IP_HEADER_SIZE);

		if ( (packet->srcBlockLen == packet->xformBlockLen) &&
				(htons(ip_hdr->ip_len) < 46) &&
				(packet->firstSrcBlock) &&
				incoming
				)
		{
			ip_hdr->ip_len = ip_hdr->ip_len;
		}
		else if ( (packet->srcBlockLen == packet->xformBlockLen) &&
				(packet->firstSrcBlock) &&
				(packet->lastSrcBlock) &&
				(packet->lastXformBlock)
				)
		{
			ip_hdr->ip_len = ip_hdr->ip_len;
		}
		else
			ip_hdr->ip_len = htons(packet->xformBlockLen - ETHER_HEADER_SIZE);

		ip_hdr->ip_foff = htons(offset >> 3);
		if (packet->lastXformBlock)
			ip_hdr->ip_foff = ~(IP_MF) & ip_hdr->ip_foff;
		else
			ip_hdr->ip_foff = IP_MF | ip_hdr->ip_foff;

		if (DF)
			ip_hdr->ip_foff = IP_DF | ip_hdr->ip_foff;

		ip_hdr->ip_prot = ip_prot;
		ip_hdr->ip_chksum = 0;
		ip_hdr->ip_chksum = HeaderChecksum((USHORT*)ip_hdr, TYPE_IP);

		if (packet->firstSrcBlock)
			packet->xformBlockLen += 0;//ETHER_HEADER_SIZE;
		else
			packet->xformBlockLen += 0; //ETHER_HEADER_SIZE + IP_HEADER_SIZE;

		PGPnetAdjustXformPacket(packet);

	    ip_len = ntohs(ip_hdr->ip_len);
		offset += ip_len - IP_HEADER_SIZE;

		if (packet->lastXformBlock)
		{
			PGPNdisPacketFreeList(packet->link);
			next = NULL;
			packet->link = NULL;
		}
		else
			next = packet->link;

		packet = next;
	}

	DBG_LEAVE(0);

}

#if XXX
VOID CPGPnetFilterAdapter::PreFragmentPacket(PGPnetPacketHead* packetHead, PGPBoolean incoming)
{
	DBG_FUNC("CPGPnetFilterAdapter::PreFragmentPacket")

	PGPnetPacket* packet;
	PGPnetPacket* next;
	PGPnetPacket* extraPacket;


	DBG_ENTER();

	packet = packetHead->link;

	while (packet != NULL)
	{
		PUCHAR	carryFrom;
		PUCHAR	carryTo;
		PUCHAR	xformIpBuffer;
		PUCHAR	xformIpDataBuffer;
		
		next = packet->link;

		xformIpBuffer = (UCHAR*)packet->xformBlock + ETHER_HEADER_SIZE;
		xformIpDataBuffer = xformIpBuffer + IP_HEADER_SIZE;

		if ( (packet->xformBlockLen + packet->Carry - ETHER_HEADER_SIZE) > MaximumIPFragmentSize)
		{
			// Carry over needed;

			// Make sure we can carry it over
			if(!next)
			{
				extraPacket = AllocatePGPnetPacket();
				if (extraPacket == NULL)
				{
					PGPNdisPacketHeadFree(packetHead);
					break;
				}
				else
					AppendPGPNdisPacket(packetHead, extraPacket);

				next = extraPacket;

			}

			// Carry over 
			next->Carry = packet->xformBlockLen + packet->Carry - ETHER_HEADER_SIZE - MaximumIPFragmentSize;
			carryFrom = xformIpBuffer + MaximumIPFragmentSize - packet->Carry;
			carryTo = next->srcBlock;
			NdisMoveMemory(carryTo, carryFrom, next->Carry);

			packet->xformBlockLen -= next->Carry;

			if ( next->xformBlockLen == 0)
				next->xformBlockLen = ETHER_HEADER_SIZE + IP_HEADER_SIZE;

			if (packet->lastXformBlock)
			{
				packet->lastXformBlock = FALSE;
				next->lastXformBlock = TRUE;
			}


		}

		if (packet->Carry)
		{
			// Take carry
			carryFrom = xformIpDataBuffer;
			carryTo = packet->srcBlock + packet->Carry;
			NdisMoveMemory(carryTo, carryFrom, packet->xformBlockLen - (ETHER_HEADER_SIZE + IP_HEADER_SIZE));

			carryFrom = packet->srcBlock;
			carryTo = xformIpDataBuffer;
			NdisMoveMemory(carryTo, carryFrom, packet->Carry + packet->xformBlockLen - (ETHER_HEADER_SIZE + IP_HEADER_SIZE));
			packet->xformBlockLen += packet->Carry;
		}

		if (packet->lastXformBlock)
		{
			PGPNdisPacketFreeList(packet->link);
			next = NULL;
			packet->link = NULL;
		}
		else
			next = packet->link;

		packet = next;
	}

	DBG_LEAVE(0);

}
#endif

NDIS_STATUS CPGPnetFilterAdapter::IndicatePacket(PGPnetPacket* pgpnetPacket)
{
	DBG_FUNC("CPGPnetFilterAdapter::IndicatePacket")

	DBG_ENTER();

	PGPUInt32 psize = 0;

	if (pgpnetPacket->xformBlockLen) {
		pgpnetPacket->HeaderBuffer = pgpnetPacket->xformBlock;
		pgpnetPacket->LookaheadBuffer = pgpnetPacket->xformBlock + ETHER_HEADER_SIZE;
		psize = pgpnetPacket->xformBlockLen - ETHER_HEADER_SIZE;
	} else {
		pgpnetPacket->HeaderBuffer = pgpnetPacket->srcBlock;
		pgpnetPacket->LookaheadBuffer = pgpnetPacket->srcBlock + ETHER_HEADER_SIZE;
		psize = pgpnetPacket->srcBlockLen - ETHER_HEADER_SIZE;
	}

	if (psize < 60)
		psize = 60;

	IndicateReceive(this,
		(PCHAR)pgpnetPacket->HeaderBuffer,
		pgpnetPacket->HeaderBufferSize,
		pgpnetPacket->LookaheadBuffer,
		psize,
		psize);

	DBG_LEAVE(0);

	return NDIS_STATUS_SUCCESS;
}

NDIS_STATUS CPGPnetFilterAdapter::IndicateCfgPacket(PGPnetPacket* pgpnetPacket)
{
	DBG_FUNC("CPGPnetFilterAdapter::IndicateCfgPacket")

	DBG_ENTER();

	PGPUInt32 psize = 0;

	pgpnetPacket->HeaderBuffer = pgpnetPacket->xformBlock;
	pgpnetPacket->LookaheadBuffer = pgpnetPacket->xformBlock + ETHER_HEADER_SIZE;
	psize = pgpnetPacket->xformBlockLen - ETHER_HEADER_SIZE;

	if (psize < 60)
		psize = 60;

	PGPnetFilterDriver->cfgReceiveHandler(PGPnetFilterDriver->cfgAdapterHandle,
					this,
					pgpnetPacket->HeaderBuffer,
					pgpnetPacket->HeaderBufferSize,
					pgpnetPacket->LookaheadBuffer,
					psize,
					psize);

	DBG_LEAVE(0);

	return NDIS_STATUS_SUCCESS;
}

NDIS_STATUS CPGPnetFilterAdapter::IndicateCfgPacketComplete()
{
	DBG_FUNC("CPGPnetFilterAdapter::IndicateCfgPacketComplete")

	DBG_ENTER();

	PGPnetFilterDriver->cfgReceiveCompleteHandler(PGPnetFilterDriver->cfgAdapterHandle);

	DBG_LEAVE(0);

	return NDIS_STATUS_SUCCESS;
}

NDIS_STATUS CPGPnetFilterAdapter::ProtocolReceive(
					NDIS_HANDLE      MacReceiveContext,
					PVOID            HeaderBuffer,
					UINT             HeaderBufferSize,
				    PVOID            LookAheadBuffer,
					UINT             LookAheadBufferSize,
					UINT             PacketSize
					)
{ 
	DBG_FUNC("CPGPnetFilterAdapter::ProtocolReceive")

	NDIS_STATUS		status;
	PGPnetPacket*	pgpnetPacket;
	UINT			BytesTransferred;

	DBG_ENTER();

	do
	{
		if ( (LookAheadBufferSize > MAX_ETHER_FRAME_SIZE - ETHER_HEADER_SIZE) ||
			(PacketSize > MAX_ETHER_FRAME_SIZE - ETHER_HEADER_SIZE) )
		{
			status = NDIS_STATUS_INVALID_PACKET;
			break;
		}

		if (MaximumFrameSize == MAX_ETHER_FRAME_SIZE - ETHER_HEADER_SIZE)
			status = NDIS_STATUS_SUCCESS;
		else
			status = NDIS_STATUS_SUCCESS;

		pgpnetPacket = AllocatePGPnetPacket();
		if (pgpnetPacket == NULL)
		{
			status = NDIS_STATUS_RESOURCES;
			break;
		}

		if (HeaderBuffer != NULL)
			NdisMoveMemory(pgpnetPacket->srcBlock, HeaderBuffer, ETHER_HEADER_SIZE);

		pgpnetPacket->HeaderBuffer = pgpnetPacket->srcBlock;
		pgpnetPacket->HeaderBufferSize = ETHER_HEADER_SIZE;
		pgpnetPacket->LookaheadBuffer = pgpnetPacket->srcBlock + ETHER_HEADER_SIZE;

		if (LookAheadBufferSize >= PacketSize)
		{
			NdisMoveMemory(pgpnetPacket->LookaheadBuffer, LookAheadBuffer, PacketSize);
			pgpnetPacket->srcBlockLen = PacketSize + ETHER_HEADER_SIZE;
			status = NDIS_STATUS_SUCCESS;
			PacketEnqueue(&incoming_indicateComplete_wait_list, pgpnetPacket);
		}
		else
		{
			// Adjust xform packet to the exact size of incoming packet minus ether header
			pgpnetPacket->xformBlockLen = PacketSize;
			PGPnetAdjustXformPacket(pgpnetPacket);

			PacketEnqueue(&incoming_ipsectransferComplete_wait_list, pgpnetPacket);

			NdisTransferData(&status, NdisBindingHandleToRealAdapter, MacReceiveContext, 0,
				PacketSize, pgpnetPacket->xformPacket, &BytesTransferred);

			if (status == NDIS_STATUS_PENDING) {
				status = NDIS_STATUS_SUCCESS;
				break;
			}

			ASSERT(BytesTransferred == PacketSize);

			pgpnetPacket =  PacketRemoveByXformPacket(&incoming_ipsectransferComplete_wait_list, pgpnetPacket->xformPacket);

			if (status == NDIS_STATUS_SUCCESS) {
				PGPnetAdjustTransferCompletePacket(pgpnetPacket);
				PacketEnqueue(&incoming_indicateComplete_wait_list, pgpnetPacket);
			}
			else {
				FreePGPnetPacket(pgpnetPacket);
			}
		}

	} while (0);

	DBG_LEAVE(status);

	return status;
}

VOID CPGPnetFilterAdapter::ProtocolReceiveComplete()
{
	DBG_FUNC("CPGPnetFilterAdapter::ProtocolReceiveComplete")

	PGPnetPacket* pgpnetPacket;
	UINT			indicated = 0;

	DBG_ENTER();

	NdisAcquireSpinLock(&AdapterLock);

	if (indicate_busy == FALSE) {
		indicate_busy = TRUE;
		NdisReleaseSpinLock(&AdapterLock);

		while ( (pgpnetPacket = PacketDequeue(&incoming_indicateComplete_wait_list)) != NULL )
		{
			TransformAndIndicate(pgpnetPacket);
		}

		NdisAcquireSpinLock(&AdapterLock);
		indicate_busy = FALSE;
	}

	NdisReleaseSpinLock(&AdapterLock);

	DBG_LEAVE(0);
}

VOID CPGPnetFilterAdapter::ProtocolTransferDataComplete(
						PNDIS_PACKET     Packet,
						NDIS_STATUS      Status,
						UINT             BytesTransferred
						)
{
	DBG_FUNC("CPGPnetAdapterMiniport:ProtocolTransferDataComplete")

	PGPnetPacket*	pgpnetPacket;

	DBG_ENTER();

	if ( (pgpnetPacket = PacketRemoveByXformPacket(&incoming_ipsectransferComplete_wait_list, Packet)) != NULL)
	{
		if (Status == NDIS_STATUS_SUCCESS) {
			PGPnetAdjustTransferCompletePacket(pgpnetPacket);
			PacketEnqueue(&incoming_indicateComplete_wait_list, pgpnetPacket);
		}
		else {
			FreePGPnetPacket(pgpnetPacket);
		}
	}
	else {
		ASSERT(FALSE);
	}

	DBG_LEAVE(Status);
}

NDIS_STATUS CPGPnetFilterAdapter::TransformAndIndicate(PGPnetPacket* pgpnetPacket)
{
	DBG_FUNC("CPGPnetFilterAdapter::TransformAndIndicate")

	NDIS_STATUS status = NDIS_STATUS_SUCCESS;

	PGPnetPacketStatus	packetStatus;
	PGPnetPMStatus		pmStatus;

	PGPBoolean			newHead = FALSE;

	DBG_ENTER();

	pgpnetPacket->HeaderBuffer = pgpnetPacket->srcBlock;
	pgpnetPacket->HeaderBufferSize = ETHER_HEADER_SIZE;
	pgpnetPacket->LookaheadBuffer = pgpnetPacket->srcBlock + ETHER_HEADER_SIZE;


	// Non IP packet, indicate
	// IP packet, assemble
	// Assemble complete, check status and do transform
	do {
		packetStatus = PGPnetFilterPacketCheck(pgpnetPacket, TRUE);

		switch (packetStatus)
		{
		case PacketDrop:
			FreePGPnetPacket(pgpnetPacket);
			status = NDIS_STATUS_NOT_ACCEPTED;
			break;

		case PacketPassthrough:
			status = IndicatePacket(pgpnetPacket);
			IndicateReceiveComplete();
			FreePGPnetPacket(pgpnetPacket);
			break;

		case PacketEncrypt:
			ASSERT(0);
			break;

		case PacketFurtherCheck:
		{

			pgpnetPacket->pSrcIPsecBuffer->data = pgpnetPacket->srcBlock + ETHER_HEADER_SIZE;
			pgpnetPacket->pSrcIPsecBuffer->dataSize = pgpnetPacket->srcBlockLen - ETHER_HEADER_SIZE;
			pgpnetPacket->ethernetAddr = pgpnetPacket->srcBlock;
			pgpnetPacket->lastSrcBlock = TRUE;
			pgpnetPacket->lastXformBlock = TRUE;
			pgpnetPacket->firstSrcBlock = TRUE;
			pgpnetPacket->firstXformBlock = TRUE;

			pmStatus = PGPnetHandleIncomingPacket(PGPnetDriver->PolicyManagerHandle, &pgpnetPacket);

			switch (pmStatus) {	
			case kPGPNetPMPacketSent:

				status = IndicatePackets(pgpnetPacket, FALSE /* not cfg */);

				break;
			case kPGPNetPMPacketWaiting:
			case kPGPNetPMPacketDrop:
				// free packets here
				PGPNdisPacketFreeList(pgpnetPacket);
				status = NDIS_STATUS_SUCCESS;
				break;
			case kPGPNetPMPacketClear:
				status = IndicatePackets(pgpnetPacket, FALSE /* not cfg */);
				break;
			case kPGPNetPMPacketForget:
				status = NDIS_STATUS_SUCCESS;
				break;
			case kPGPNetPMPacketCfg:
				status = IndicatePackets(pgpnetPacket, TRUE /* cfg */);
				break;
			default:
				ASSERT(0);
			} 
			break;
		}
		default:
			ASSERT(0);
		}

	} while (0);

	DBG_LEAVE(status);

	return status;
}

NDIS_STATUS CPGPnetFilterAdapter::IndicatePackets(PGPnetPacket *pgpNetPacket, PGPBoolean bIsCfgPacket)
{
	DBG_FUNC("CPGPnetFilterAdapter::IndicatePackets")

	NDIS_STATUS status = NDIS_STATUS_SUCCESS;

	PGPnetPacket* packet;
	PGPnetPacket* next;

	DBG_ENTER();

	packet = pgpNetPacket;

	ASSERT(packet != NULL);

	while (packet != NULL)
	{
		next = packet->link;

		if (bIsCfgPacket)
		{
			IndicateCfgPacket(packet);
			IndicateCfgPacketComplete();
		}
		else
		{
			IndicatePacket(packet);
			IndicateReceiveComplete();
		}

		FreePGPnetPacket(packet);

		packet = next;
	}

	DBG_LEAVE(0);

	return status;
}

VOID CPGPnetFilterAdapter::HeadFragmentCollection(PPGPnetPacketHead	*packet_list)
{
	DBG_FUNC("CPGPnetFilterAdapter::HeadFragmentCollection")

	PPGPnetPacketHead current;
	PPGPnetPacketHead next;
	
	PGPBoolean done = FALSE;

	if ((current = *packet_list) == NULL) {	
		return;
	}

	AdapterLevelLock();

	do {
		next = current->next;
		
		next->timeOut += 1;

		if (next->timeOut > 1)
		{
			DBG_PRINT(("Caught one overdue fragment\n"););
			if (next == current) {
				*packet_list = NULL;
				done = TRUE;
			}
			else
			{
				current->next = next->next;
				if (next == *packet_list) {
					*packet_list = next->next;
					done = TRUE;
					break;
				}
			}
			PGPNdisPacketHeadFreeList(next);
			PGPNdisPacketHeadFree(next);
		}
		else
			current = next;
	}while ( (next != *packet_list) && !done );

	AdapterLevelUnlock();
}

VOID CPGPnetFilterAdapter::FragmentCollection()
{
	DBG_FUNC("CPGPnetFilterAdapter::FragmentCollection")

	DBG_ENTER();

	HeadFragmentCollection(&outgoing_local_packet_head_list);
	HeadFragmentCollection(&outgoing_cfg_packet_head_list);
	HeadFragmentCollection(&incoming_packet_head_list);

	DBG_LEAVE(0);
}


/*******************************************************************************
 *******************************************************************************
 PGPNET_LAST_FRAGMENT Related functions
 *******************************************************************************
 *******************************************************************************/

PGPnetLastFragmentList* 
LastFragmentListQuery(PGPnetLastFragmentList *list,
					  PGPUInt32 ipAddress,
					  USHORT ipId)
{
	PGPnetLastFragmentList *head;
	PGPnetLastFragmentList *last;
	
	NdisAcquireSpinLock(&PGPnetDriver->general_lock);
	
	head = list;
	last = head;
	while (head != NULL)
	{
		if ((head->ipAddress == ipAddress) &&
			(head->ipId == ipId))
			break;
		
		last = head;
		head = head->next;
	}

	NdisReleaseSpinLock(&PGPnetDriver->general_lock);
	return head;
}

PGPnetLastFragmentList* 
LastFragmentListAdd(PGPnetLastFragmentList **list,
					PGPUInt32 ipAddress,
					USHORT ipId)
{
	PGPnetLastFragmentList *head;
	PGPnetLastFragmentList *last;
	NDIS_STATUS status;
	
	NdisAcquireSpinLock(&PGPnetDriver->general_lock);
	
	head = *list;
	last = head;
	while (head != NULL)
	{
		if ((head->ipAddress == ipAddress) &&
			(head->ipId == ipId))
			break;
		
		last = head;
		head = head->next;
	}

	if (head == NULL)
	{
		status = NdisAllocateMemory((void **) &head, 
					sizeof(PGPnetLastFragmentList), 0, 
					HighestAcceptableAddress);

		if (status != NDIS_STATUS_SUCCESS)
		{
			NdisReleaseSpinLock(&PGPnetDriver->general_lock);
			return NULL;
		}

		NdisZeroMemory(head, sizeof(PGPnetLastFragmentList));
		head->ipAddress = ipAddress;
		head->ipId = ipId;

		if (last == NULL)
			*list = head;
		else
			last->next = head;
	}

	NdisReleaseSpinLock(&PGPnetDriver->general_lock);
	return head;
}

VOID
LastFragmentListFree(PGPnetLastFragmentList **list,
					 PGPUInt32 ipAddress,
					 USHORT ipId)
{
	PGPnetLastFragmentList *head;
	PGPnetLastFragmentList *last;
	
	NdisAcquireSpinLock(&PGPnetDriver->general_lock);
	
	head = *list;
	last = head;
	while (head != NULL)
	{
		if ((head->ipAddress == ipAddress) &&
			(head->ipId == ipId))
			break;
		
		last = head;
		head = head->next;
	}

	if (head != NULL)
	{
		if (last == head)
			*list = head->next;
		else
			last->next = head->next;
		
		NdisFreeMemory(head, sizeof(PGPnetLastFragmentList), 0);
	}

	NdisReleaseSpinLock(&PGPnetDriver->general_lock);
	return;
}

VOID
LastFragmentListFreeAll(PGPnetLastFragmentList *list)
{
	PGPnetLastFragmentList*	fragHead;
	PGPnetLastFragmentList* fragFree;

	NdisAcquireSpinLock(&PGPnetDriver->general_lock);

	fragHead = list;
	while (fragHead != NULL)
	{
		fragFree = fragHead;
		fragHead = fragHead->next;
		NdisFreeMemory(fragFree, sizeof(PGPnetLastFragmentList), 0);
	}

	NdisReleaseSpinLock(&PGPnetDriver->general_lock);
	return;
}

CPGPnetFilterAdapter* FindFilterAdapterByIpAddress(PGPUInt32 IpAddress)
{
	UINT i;

	for (i = 0; i < PGPnetFilterDriver->PGPnetAdapterInstanceCount; i++)
	{
		if ((IpAddress == PGPnetFilterDriver->PGPnetAdapterInstance[i]->mIpAddress) && 
			(!EmptyEthernetAddress(PGPnetFilterDriver->PGPnetAdapterInstance[i]->mHardwareAddress)) )
			return PGPnetFilterDriver->PGPnetAdapterInstance[i];
	}

	return NULL;

}
