/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: CPGPnetIPPacket.cpp,v 1.2 2002/08/06 20:10:26 dallen Exp $
____________________________________________________________________________*/

#include "pgpnetBase.h"

#include "pgpNetIPdef.h"
#include "pgpnetIpHelper.h"

#include "CPGPnetIPPacket.h"

extern "C"
{
#include "stdio.h"
}
/******************************************************************************
Ethernet Header
******************************************************************************/
CPGPnetEthernetHeader::CPGPnetEthernetHeader()
{
	NdisZeroMemory(&ethernet_header, sizeof(ethernet_header));
}

CPGPnetEthernetHeader::~CPGPnetEthernetHeader()
{
}

PGPBoolean CPGPnetEthernetHeader::IsIP()
{
	return (GetEthernetProtocolNet() == IPPROT_NET);
}

PGPBoolean CPGPnetEthernetHeader::IsARP()
{
	return (GetEthernetProtocolNet() == ARPPROT_NET);
}

PGPBoolean CPGPnetEthernetHeader::IsBroadcast()
{
	return BroadcastEthernetAddress(GetDstAddress());
}

unsigned char* CPGPnetEthernetHeader::GetSrcAddress()
{
	return &ethernet_header.eth_srcAddress[0];
}

unsigned char* CPGPnetEthernetHeader::GetDstAddress()
{
	return &ethernet_header.eth_dstAddress[0];
}

unsigned short CPGPnetEthernetHeader::GetEthernetProtocolNet()
{
	return *((PUSHORT)(&ethernet_header.eth_protocolType[0]));
}

unsigned short CPGPnetEthernetHeader::GetEthernetProtocol()
{
	return ntohs(GetEthernetProtocolNet());
}

void CPGPnetEthernetHeader::SetIP()
{
	SetEthernetProtocolNet(IPPROT_NET);
}
void CPGPnetEthernetHeader::SetARP()
{
	SetEthernetProtocolNet(ARPPROT_NET);
}

void CPGPnetEthernetHeader::SetBroadcast()
{
	for (int i = 0; i<6; i++)
		ethernet_header.eth_dstAddress[i] = 0xFF;
}

void CPGPnetEthernetHeader::SetSrcAddress(unsigned char* srcAddress)
{
	NdisMoveMemory(&ethernet_header.eth_srcAddress[0], srcAddress, 6);
}

void CPGPnetEthernetHeader::SetDstAddress(unsigned char* dstAddress)
{
	NdisMoveMemory(&ethernet_header.eth_dstAddress[0], dstAddress, 6);
}

void CPGPnetEthernetHeader::SetEthernetProtocol(unsigned short protocol)
{
	SetEthernetProtocolNet(ntohs(protocol));
}

void CPGPnetEthernetHeader::SetEthernetProtocolNet(unsigned short protocolNet)
{
	unsigned short* ethernet_protocol = (PUSHORT)(&ethernet_header.eth_protocolType[0]);
	*ethernet_protocol = protocolNet;
}

unsigned short CPGPnetEthernetHeader::CopyToPlainBuffer(PVOID buffer)
{
	NdisMoveMemory(buffer, &ethernet_header, sizeof(ethernet_header));

	return sizeof(ethernet_header);
}

/******************************************************************************
IP Header
******************************************************************************/

CPGPnetIPHeader::CPGPnetIPHeader()
{
	ip_header.ip_versionandheaderlength = default_versionandheaderlength;
	ip_header.ip_tos = default_tos;
	ip_header.ip_foff = default_foff;
	ip_header.ip_ttl = default_ttl;

	ip_header.ip_len = sizeof(ip_header);
	ip_header.ip_id = 0;
	ip_header.ip_prot = 0;
	ip_header.ip_chksum = 0;
	ip_header.ip_src = 0;
	ip_header.ip_dest = 0;

}

CPGPnetIPHeader::~CPGPnetIPHeader()
{
}

PGPBoolean CPGPnetIPHeader::IsTCP()
{
	return (ip_header.ip_prot == PROTOCOL_TCP);
}

PGPBoolean CPGPnetIPHeader::IsUDP()
{
	return (ip_header.ip_prot == PROTOCOL_UDP);
}

PGPBoolean CPGPnetIPHeader::IsICMP()
{
	return (ip_header.ip_prot == PROTOCOL_ICMP);
}

unsigned char CPGPnetIPHeader::GetIPProtocol()
{
	return ip_header.ip_prot;
}

PGPBoolean CPGPnetIPHeader::IsBroadcast()
{
	return ip_header.ip_dest == 0xFFFFFFFF;
}

PGPUInt32 CPGPnetIPHeader::GetSrcAddress()
{
	return ntohl(GetSrcAddressNet());
}

PGPUInt32 CPGPnetIPHeader::GetDstAddress()
{
	return ntohl(GetDstAddressNet());
}

PGPUInt32 CPGPnetIPHeader::GetSrcAddressNet()
{
	return ip_header.ip_src;
}

PGPUInt32 CPGPnetIPHeader::GetDstAddressNet()
{
	return ip_header.ip_dest;
}

void CPGPnetIPHeader::SetTCP()
{
	ip_header.ip_prot = PROTOCOL_TCP;
}

void CPGPnetIPHeader::SetUDP()
{
	ip_header.ip_prot = PROTOCOL_UDP;
}

void CPGPnetIPHeader::SetICMP()
{
	ip_header.ip_prot = PROTOCOL_ICMP;
}

void CPGPnetIPHeader::SetIPProtocol(unsigned char ip_protocol)
{
	ip_header.ip_prot = ip_protocol;
}

void CPGPnetIPHeader::SetBroadcast()
{
	SetDstAddress(0xffffffff);
}

void CPGPnetIPHeader::SetSrcAddress(PGPUInt32 ip_address)
{
	SetSrcAddressNet(ntohl(ip_address));
}

void CPGPnetIPHeader::SetDstAddress(PGPUInt32 ip_address)
{
	SetDstAddressNet(ntohl(ip_address));
}

void CPGPnetIPHeader::SetSrcAddressNet(PGPUInt32 ip_address_net)
{
	ip_header.ip_src = ip_address_net;
}

void CPGPnetIPHeader::SetDstAddressNet(PGPUInt32 ip_address_net)
{
	ip_header.ip_dest = ip_address_net;
}

unsigned short CPGPnetIPHeader::GetId()
{
	return ntohs(GetIdNet());
}

unsigned short CPGPnetIPHeader::GetIdNet()
{
	return ip_header.ip_id;
}

void CPGPnetIPHeader::SetId(unsigned short ip_id)
{
	SetIdNet(ntohs(ip_id));
}

void CPGPnetIPHeader::SetIdNet(unsigned short ip_id_net)
{
	ip_header.ip_id = ip_id_net;
}

unsigned short CPGPnetIPHeader::GetChecksum()
{
	return ip_header.ip_chksum;
}

void CPGPnetIPHeader::SetChecksum(unsigned short checksum)
{
	ip_header.ip_chksum = checksum;
}

unsigned short CPGPnetIPHeader::GetIPLength()
{
	return ntohs(ip_header.ip_len);
}

void CPGPnetIPHeader::SetIPLength(unsigned short ip_len)
{
	ip_header.ip_len = ntohs(ip_len);
}

void CPGPnetIPHeader::UpdateChecksum()
{
	SetChecksum(HeaderChecksum((USHORT*)&ip_header, TYPE_IP));
}

unsigned short CPGPnetIPHeader::CopyToPlainBuffer(PVOID buffer)
{
	PUCHAR working_buffer = (PUCHAR)buffer;
	unsigned short offset = 0;

	offset = ethernet_header.CopyToPlainBuffer(buffer);
	working_buffer = working_buffer + offset;

	NdisMoveMemory(working_buffer, &ip_header, sizeof(ip_header));
	offset += sizeof(ip_header);

	return offset;

}
/******************************************************************************
UDP Header
******************************************************************************/
CPGPnetUDPHeader::CPGPnetUDPHeader()
{
	NdisZeroMemory(&udp_header, sizeof(udp_header));
	udp_header.msg_len = ntohs(8);
}

CPGPnetUDPHeader::~CPGPnetUDPHeader()
{
}

PGPBoolean CPGPnetUDPHeader::IsBOOTP()
{
	return (IsBOOTPClient() || IsBOOTPServer());
}

PGPBoolean CPGPnetUDPHeader::IsBOOTPClient()
{
	return (udp_header.source_port == UDP_PORT_BOOTPC_NET);
}

PGPBoolean CPGPnetUDPHeader::IsBOOTPServer()
{
	return (udp_header.source_port == UDP_PORT_BOOTPS_NET);
}

unsigned short CPGPnetUDPHeader::GetUDPLen()
{
	return ntohs(udp_header.msg_len);
}

unsigned short CPGPnetUDPHeader::GetSrcPort()
{
	return ntohs(GetSrcPortNet());
}

unsigned short CPGPnetUDPHeader::GetDstPort()
{
	return ntohs(GetDstPortNet());
}

unsigned short CPGPnetUDPHeader::GetSrcPortNet()
{
	return udp_header.source_port;
}

unsigned short CPGPnetUDPHeader::GetDstPortNet()
{
	return udp_header.dest_port;
}

unsigned short CPGPnetUDPHeader::GetChecksum()
{
	return udp_header.checksum;
}

void CPGPnetUDPHeader::SetUDPLen(unsigned short udp_len)
{
	udp_header.msg_len = ntohs(udp_len);
}

void CPGPnetUDPHeader::SetSrcPort(unsigned short port)
{
	SetSrcPortNet(ntohs(port));
}

void CPGPnetUDPHeader::SetDstPort(unsigned short port)
{
	SetDstPortNet(ntohs(port));
}

void CPGPnetUDPHeader::SetSrcPortNet(unsigned short port_net)
{
	udp_header.source_port = port_net;
}

void CPGPnetUDPHeader::SetDstPortNet(unsigned short port_net)
{
	udp_header.dest_port = port_net;
}

void CPGPnetUDPHeader::SetChecksum(unsigned short checksum)
{
	udp_header.checksum = checksum;
}

void CPGPnetUDPHeader::UpdateChecksum()
{
	SetChecksum(0);
}

unsigned short CPGPnetUDPHeader::CopyToPlainBuffer(PVOID buffer)
{
	PUCHAR working_buffer = (PUCHAR)buffer;
	unsigned short offset = 0;

	offset = ip_header.CopyToPlainBuffer(buffer);
	working_buffer = working_buffer + offset;

	NdisMoveMemory(working_buffer, &udp_header, sizeof(udp_header));
	offset += sizeof(udp_header);

	return offset;

}

/******************************************************************************
DHCP Packet
******************************************************************************/

CPGPnetDHCPPacket::CPGPnetDHCPPacket()
{
	// DHCP Option
	NdisZeroMemory(&OptionField, sizeof(OptionField));
	eooPos = 0;
	OptionField[eooPos] = CODE_END_OPTION;

	// DHCP Header
	NdisZeroMemory(&dhcp_header, sizeof(dhcp_header));

	// UDP ???
}

CPGPnetDHCPPacket::~CPGPnetDHCPPacket()
{
}

CPGPnetDHCPPacket::CPGPnetDHCPPacket(PVOID block, UINT block_len)
{
	if (block_len > sizeof(dhcp_header))
	{
		NdisMoveMemory(&dhcp_header, block, sizeof(dhcp_header));

		NdisMoveMemory(&OptionField, (PCHAR)block + sizeof(DHCP_HEADER), block_len - sizeof(DHCP_HEADER));
	}
}

PGPUInt32 CPGPnetDHCPPacket::GetClientIPAddress()
{
	return htonl(dhcp_header.ciaddr);
}

PGPUInt32 CPGPnetDHCPPacket::GetYourIPAddress()
{
	return htonl(dhcp_header.yiaddr);
}

PGPUInt32 CPGPnetDHCPPacket::GetServerIPAddress()
{
	return htonl(dhcp_header.siaddr);
}

unsigned char* CPGPnetDHCPPacket::GetClientEthernetAddress()
{
	return &(dhcp_header.chaddr[0]);
}

PGPUInt32 CPGPnetDHCPPacket::GetTransactionID()
{
	return dhcp_header.xid;
}

void CPGPnetDHCPPacket::SetClientIPAddress(PGPUInt32 ip_address)
{
	dhcp_header.ciaddr = htonl(ip_address);
}

void CPGPnetDHCPPacket::SetYourIPAddress(PGPUInt32 ip_address)
{
	dhcp_header.yiaddr = htonl(ip_address);
}

void CPGPnetDHCPPacket::SetServerIPAddress(PGPUInt32 ip_address)
{
	dhcp_header.siaddr = htonl(ip_address);
}

void CPGPnetDHCPPacket::SetClientEtherenetAddress(unsigned char* ethernet_address)
{
	for (int i = 0; i < 6; i++)
		dhcp_header.chaddr[i] = ethernet_address[i];
}

void CPGPnetDHCPPacket::SetTransactionID(PGPUInt32 transactionID)
{
	dhcp_header.xid = transactionID;
}

void CPGPnetDHCPPacket::BuildGenericResponse()
{
	dhcp_header.op = 0x2;			// response
	dhcp_header.htype = 0x1;		// Ethernet
	dhcp_header.hlen = 0x6;			// 6 for ethernet

	dhcp_header.secs = 0;
	dhcp_header.flags = 0;

	dhcp_header.cookie = DHCP_MAGIC_COOKIE;

	udp_header.SetSrcPortNet(UDP_PORT_BOOTPS_NET);
	udp_header.SetDstPortNet(UDP_PORT_BOOTPC_NET);
	udp_header.SetUDPLen(sizeof(DHCP_HEADER) + sizeof(UDP_HEADER));
	udp_header.SetChecksum(0);
}

void CPGPnetDHCPPacket::DecodeOptionFields()
{
	UINT i;
	UINT curPos;

	for (i = 0; i < CODE_END_OPTION; i++)
		NdisZeroMemory(&options[i], sizeof(OptionItem));

	curPos = 0;
	while (curPos < MAXIMUM_DHCP_OPTION_TOTAL_LEN)
	{
		DHCP_CODE	code;
		UINT		len;

		code = (DHCP_CODE)OptionField[curPos++];

		if (code == CODE_END_OPTION)
		{
			options[code].len = 1;
			break;
		}

		len = (UINT)OptionField[curPos++];

		options[code].len = (UINT)len;
		NdisMoveMemory(&options[code].value,
						&OptionField[curPos],
						len > MAXIMUM_DHCP_OPTION_ITEM_LEN ? MAXIMUM_DHCP_OPTION_ITEM_LEN : len);
		curPos += len;

	}
}

unsigned short CPGPnetDHCPPacket::CopyToPlainBuffer(PVOID buffer)
{
	PUCHAR working_buffer = (PUCHAR)buffer;
	unsigned short offset = 0;

	offset = udp_header.CopyToPlainBuffer(buffer);
	working_buffer = (PUCHAR)buffer + offset;

	NdisMoveMemory(working_buffer, &dhcp_header, sizeof(dhcp_header));

	offset += sizeof(dhcp_header);
	working_buffer = (PUCHAR)buffer + offset;

	NdisMoveMemory(working_buffer, OptionField, eooPos + 1);

	offset += eooPos + 1;

	return offset;
}

DHCP_TYPE CPGPnetDHCPPacket::GetDHCPType()
{
	// DHCP TYPE always the first option?

	unsigned short opp = 0;
	DHCP_TYPE dhcpType = TYPE_INVALID;

	if (CODE_DHCP_TYPE == OptionField[opp])
		dhcpType = (DHCP_TYPE)OptionField[opp + OptionField[opp + 1] + 1];

	if (dhcpType < TYPE_DISCOVER || dhcpType >> TYPE_INVALID)
		dhcpType = TYPE_INVALID;

	return dhcpType;
}

void CPGPnetDHCPPacket::SetDHCPType(DHCP_TYPE dhcpType)
{
	OptionField[eooPos++] = CODE_DHCP_TYPE;
	OptionField[eooPos++] = 1;
	OptionField[eooPos++] = dhcpType;

	OptionField[eooPos] = CODE_END_OPTION;
}

void CPGPnetDHCPPacket::SetDHCPSubnetMask(PGPUInt32 subnetMask)
{
	PGPUInt32 *longOption;

	OptionField[eooPos++] = CODE_SUBNET_MASK;
	OptionField[eooPos++] = 4;

	longOption = (PGPUInt32*)&OptionField[eooPos];

	*longOption = htonl(subnetMask);

	eooPos += 4;

	OptionField[eooPos] = CODE_END_OPTION;

}

void CPGPnetDHCPPacket::SetDHCPDomainName(char *domainName)
{
	OptionField[eooPos++] = CODE_DOMAIN_NAME;
	OptionField[eooPos++] = strlen(domainName) + 1;

	strcpy((char*)&OptionField[eooPos], domainName);

	eooPos += strlen(domainName);

	OptionField[eooPos++] = 0;

	OptionField[eooPos] = CODE_END_OPTION;

}

void CPGPnetDHCPPacket::SetDHCPRoute(PGPUInt32 route)
{
	PGPUInt32 *longOption;

	OptionField[eooPos++] = CODE_ROUTE;
	OptionField[eooPos++] = 4;

	longOption = (PGPUInt32*)&OptionField[eooPos];

	*longOption = htonl(route);

	eooPos += 4;

	OptionField[eooPos] = CODE_END_OPTION;
}

void CPGPnetDHCPPacket::SetDHCPWINSServer(PGPUInt32 wins1, PGPUInt32 wins2)
{
	PGPUInt32 *longOption1, *longOption2;

	if (wins1 == 0)
		return;

	OptionField[eooPos++] = CODE_WINS_SERVER;
	if (wins1 != wins2)
		OptionField[eooPos++] = 8;
	else
		OptionField[eooPos++] = 4;

	longOption1 = (PGPUInt32*)&OptionField[eooPos];

	*longOption1 = htonl(wins1);

	eooPos += 4;

	if (wins1 != wins2)
	{
		longOption2 = (PGPUInt32*)&OptionField[eooPos];

		*longOption2 = htonl(wins2);

		eooPos += 4;
	}

	OptionField[eooPos] = CODE_END_OPTION;
}

void CPGPnetDHCPPacket::SetDHCPWINSType(unsigned char winsType)
{
	OptionField[eooPos++] = CODE_WINS_TYPE;
	OptionField[eooPos++] = 1;

	OptionField[eooPos++] = winsType;

	OptionField[eooPos] = CODE_END_OPTION;
}

void CPGPnetDHCPPacket::GetDHCPDNSServer(PGPUInt32 *dns1, PGPUInt32 *dns2)
{
	if (!options[CODE_END_OPTION].len)
		DecodeOptionFields();

	if (options[CODE_DNS_SERVER].len == 4)
	{
		*dns1 = *dns2 = htonl(*(PGPUInt32*)(options[CODE_DNS_SERVER].value));
	}
	else if (options[CODE_DNS_SERVER].len == 8)
	{
		*dns1 = htonl(*(PGPUInt32*)(options[CODE_DNS_SERVER].value));
		*dns2 = htonl(*(PGPUInt32*)(&options[CODE_DNS_SERVER].value[4]));
	}
	else
		*dns1 = *dns2 = 0;

}

void CPGPnetDHCPPacket::SetDHCPDNSServer(PGPUInt32 dns1, PGPUInt32 dns2)
{
	PGPUInt32 *longOption1, *longOption2;

	if (dns1 == 0)
		return;

	OptionField[eooPos++] = CODE_DNS_SERVER;
	if (dns2 != dns1 && dns2 != 0)
		OptionField[eooPos++] = 8;
	else
		OptionField[eooPos++] = 4;

	longOption1 = (PGPUInt32*)&OptionField[eooPos];

	*longOption1 = htonl(dns1);

	eooPos += 4;

	if (dns2 != dns1 && dns2 != 0)
	{
		longOption2 = (PGPUInt32*)&OptionField[eooPos];

		*longOption2 = htonl(dns2);

		eooPos += 4;
	}

	OptionField[eooPos] = CODE_END_OPTION;

}

void CPGPnetDHCPPacket::SetT1(PGPUInt32 t1)
{
	PGPUInt32 *longOption;

	OptionField[eooPos++] = CODE_T1;
	OptionField[eooPos++] = 4;

	longOption = (PGPUInt32*)&OptionField[eooPos];

	*longOption = htonl(t1);

	eooPos += 4;

	OptionField[eooPos] = CODE_END_OPTION;
}

void CPGPnetDHCPPacket::SetT2(PGPUInt32 t2)
{
	PGPUInt32 *longOption;

	OptionField[eooPos++] = CODE_T2;
	OptionField[eooPos++] = 4;

	longOption = (PGPUInt32*)&OptionField[eooPos];

	*longOption = htonl(t2);

	eooPos += 4;

	OptionField[eooPos] = CODE_END_OPTION;
}

void CPGPnetDHCPPacket::SetLeaseTime(PGPUInt32 leaseTime)
{
	PGPUInt32 *longOption;

	OptionField[eooPos++] = CODE_LEASE_TIME;
	OptionField[eooPos++] = 4;

	longOption = (PGPUInt32*)&OptionField[eooPos];

	*longOption = htonl(leaseTime);

	eooPos += 4;

	OptionField[eooPos] = CODE_END_OPTION;
}

void CPGPnetDHCPPacket::SetServerId(PGPUInt32 serverId)
{
	PGPUInt32 *longOption;

	OptionField[eooPos++] = CODE_SERVER_ID;
	OptionField[eooPos++] = 4;

	longOption = (PGPUInt32*)&OptionField[eooPos];

	*longOption = htonl(serverId);

	eooPos += 4;

	OptionField[eooPos] = CODE_END_OPTION;
}

/******************************************************************************
DHCP Packet
******************************************************************************/

CPGPnetARPPacket::CPGPnetARPPacket()
{
}

CPGPnetARPPacket::~CPGPnetARPPacket()
{
}

CPGPnetARPPacket::CPGPnetARPPacket(PVOID block, UINT block_len)
{
	if (block_len >= sizeof(arp_header))
		NdisMoveMemory(&arp_header, block, sizeof(arp_header));
}

ARP_CODE CPGPnetARPPacket::GetARPCode()
{
	return (ARP_CODE)ntohs(arp_header.arp_op);
}

unsigned char* CPGPnetARPPacket::GetSrcEthernetAddress()
{
	return arp_header.arp_src_hrd_addr;
}

unsigned char* CPGPnetARPPacket::GetDstEthernetAddress()
{
	return arp_header.arp_dst_hrd_addr;
}

PGPUInt32 CPGPnetARPPacket::GetSrcIPAddress()
{
	return htonl(arp_header.arp_src_ip_addr);
}
PGPUInt32 CPGPnetARPPacket::GetDstIPAddress()
{
	return htonl(arp_header.arp_dst_ip_addr);
}

void CPGPnetARPPacket::SetARPCode(ARP_CODE arpCode)
{
	arp_header.arp_op = htons(arpCode);
}

void CPGPnetARPPacket::SetSrcEthernetAddress(unsigned char* ethernetAddress)
{
	if (ethernetAddress == NULL)
		NdisZeroMemory(arp_header.arp_src_hrd_addr, 6);
	else
		NdisMoveMemory(arp_header.arp_src_hrd_addr, ethernetAddress, 6);
}

void CPGPnetARPPacket::SetDstEthernetAddress(unsigned char* ethernetAddress)
{
	if (ethernetAddress == NULL)
		NdisZeroMemory(arp_header.arp_dst_hrd_addr, 6);
	else
		NdisMoveMemory(arp_header.arp_dst_hrd_addr, ethernetAddress, 6);

}

void CPGPnetARPPacket::SetSrcIPAddress(PGPUInt32 ipAddress)
{
	arp_header.arp_src_ip_addr = htonl(ipAddress);
}

void CPGPnetARPPacket::SetDstIPAddress(PGPUInt32 ipAddress)
{
	arp_header.arp_dst_ip_addr = htonl(ipAddress);
}

unsigned short CPGPnetARPPacket::CopyToPlainBuffer(PVOID buffer)
{
	PUCHAR working_buffer = (PUCHAR)buffer;
	unsigned short offset = 0;

	offset = ethernet_header.CopyToPlainBuffer(buffer);
	working_buffer = working_buffer + offset;

	NdisMoveMemory(working_buffer, &arp_header, sizeof(arp_header));
	offset += sizeof(arp_header);

	offset += ARP_PAD_LEN;

	return offset;
}
