/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: DecryptVerify.c,v 1.31 2002/11/20 19:17:36 sdas Exp $
____________________________________________________________________________*/

// System Headers
#include <windows.h>
#include <stdio.h>

// PGPsdk Headers
#include "pgpDebug.h"
#include "pgpKeys.h"
#include "pgpUtilities.h"

// Shared Headers 
#include "pgpClientLib.h"
#include "pgpLnLib.h"
#include "PGPsc.h"
#include "SigEvent.h"
#include "Working.h"
#include "Prefs.h"
#include "DecryptVerify.h"
#include "ParseMime.h"
#include "VerificationBlock.h"
#include "WorkingResource.h"
#include "SharedStrings.h"
#include "pgpWin32Errors.h"

#define CONTENTS_ENCRYPTED	0x01
#define CONTENTS_SIGNED		0x02

#define BEGIN_RTFHTML		"\\viewkind4\\uc1\\pard\\f0\\fs18 "
#define BEGIN_RTFHTMLALT	"\\uc1\\pard\\ulnone\\f0\\fs18 "
#define END_RTFHTML			"}\0"


typedef struct _VerificationBlock VerificationBlock;

struct _VerificationBlock
{
	VerificationBlock *		next;
	VerificationBlock *		previous;
	char *					szBlockBegin;
	char *					szBlockEnd;
	void *					pOutput;
	PGPSize					outSize;
	PGPBoolean				bEncrypted;
	PGPBoolean				FYEO;
};

//NOTE: copy of this data structure exists in notes plugin. please
//update the copy of this structure accordingly when this is changed.
typedef struct _DecodeEventData
{
	HINSTANCE			hInst;
	HWND				hwnd;
	HWND				hwndWorking;
	PGPtlsContextRef	tlsContext;
	char *				szName;
	PGPKeyDBRef			keyDB;
	PGPKeyDBRef			keydbNew;
	PGPKeySetRef		recipients;
	PGPUInt32			keyCount;
	PGPKeyID			*keyIDArray;
	VerificationBlock *	pVerBlock;
	PGPByte				contents;
	PGPFileSpecRef		fsrFileRef;
} DecodeEventData;

static PGPError DecryptVerify(HINSTANCE hInst, HWND hwnd, 
							  PGPContextRef context, 
							  PGPtlsContextRef tlsContext, 
							  char *szName, char *szModule,
							  PGPOptionListRef options, BOOL bMIME,
							  VerificationBlock *pVerBlock, 
							  BOOL bImportIfKeysOnly, BOOL *bKeysOnly);

PGPError DecodeEventHandler(PGPContextRef context, 
							PGPEvent *event, 
							PGPUserValue userValue);

static void DisplayErrorCode(char *szFile, 
							 int nLine, 
							 char *szModule, 
							 int nCode,
							 BOOL bKeyrings,
							 HWND hWndParent);

static char * sFindFormatBegin(char *szString, BOOL *pbHTML, BOOL *pbRTF, 
							   BOOL *pbRTFHTML);
static char * sFindFormatEnd(char *szString, BOOL bRTF);

static void	WriteDecryptedMimeParts(PGPMemoryMgrRef mgr,
									VerificationBlock *pVerBlock, 
									MimePart *pMimeList,
									MimePart **ppMimeFooter,
									HANDLE hFile,
									char **szOutBuffer);

static void	WriteDecryptedMimeFooter(PGPMemoryMgrRef mgr,
									 VerificationBlock *pVerBlock, 
									 MimePart *pMimeList,
									 HANDLE hFile,
									 char **szOutBuffer);


PGPError DecryptVerifyBuffer(HINSTANCE hInst, HWND hwnd, 
							 PGPContextRef context, 
							 PGPtlsContextRef tlsContext, 
							 char *szName, char *szModule,
							 void *pInput, DWORD dwInSize,
							 BOOL bMIME, BOOL bBinary, BOOL bHTML,
							 void **ppOutput, PGPSize *pOutSize, BOOL *FYEO)
{
	PGPError			err			= kPGPError_NoErr;
	PGPOptionListRef	options		= NULL;
	VerificationBlock *	pVerBlock	= NULL;
	VerificationBlock * pTempBlock	= NULL;
	PGPMemoryMgrRef		memoryMgr	= NULL;
	VerificationBlock * pVerIndex	= NULL;
	OUTBUFFLIST	      * obl			= NULL;
	OUTBUFFLIST       * nobl		= NULL;
	BOOL				bFYEO		= FALSE;
	BOOL				bGotHTML	= FALSE;
	BOOL				bGotRTF		= FALSE;
	BOOL				bGotRTFHTML	= FALSE;
	PGPByte				contents	= 0;
	BOOL				bKeysOnly	= FALSE;
	char			  * szStart		= NULL;
	char			  * szEnd		= NULL;
	char			  * szTemp		= NULL;
	char			  * szHeader	= NULL;
	UINT				nTempLength = 0;
	UINT				nFrom		= 0;
	UINT				nIndex		= 0;

	pgpAssert(pInput != NULL);
	pgpAssert(ppOutput != NULL);
	pgpAssert(pOutSize != NULL);
	pgpAssert(szName != NULL);
	pgpAssert(szModule != NULL);
	pgpAssert(context != NULL);
	pgpAssert(tlsContext != NULL);

	*ppOutput = NULL;
	*pOutSize = 0;

	memoryMgr = PGPPeekContextMemoryMgr(context);

	if (bBinary)
		err = PGPBuildOptionList(context, &options, 
				PGPOInputBuffer(context, pInput, dwInSize),
				PGPOAllocatedOutputBuffer(context, ppOutput, INT_MAX, pOutSize),
				PGPOLastOption(context));
	else
		err = PGPBuildOptionList(context, &options, 
				PGPOInputBuffer(context, pInput, dwInSize),
				PGPOLastOption(context));

	if (IsPGPError(err))
	{
		DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
		goto DecryptVerifyBufferError;
	}

	if (!bBinary)
	{
		pVerBlock = (VerificationBlock *) PGPNewData(memoryMgr,
											sizeof(VerificationBlock),
											kPGPMemoryMgrFlags_Clear);
		
		pVerBlock->next = NULL;
		pVerBlock->szBlockBegin = NULL;
		pVerBlock->szBlockEnd = NULL;
		pVerBlock->pOutput = NULL;
		pVerBlock->outSize = 0;
		pVerBlock->bEncrypted = FALSE;
		pVerBlock->FYEO = FALSE;
	}

	err = DecryptVerify(hInst, hwnd, context, tlsContext, szName, szModule, 
			options, bMIME, pVerBlock, TRUE, &bKeysOnly);

	pVerIndex=pVerBlock;
	szHeader = BEGIN_RTFHTML;

	if (!bBinary)
	{
		// Convert pVerBlock to OUTBUFFLIST
		do
		{
			if (bMIME && (pVerIndex->szBlockBegin != NULL))
			{
				BOOL bMultiMixed = FALSE;
				BOOL bMultiAlt = FALSE;
				BOOL bFirstMimePart = FALSE;
				MimePart *pMimeList = NULL;
				MimePart *pFirstMimeList = NULL;
				
				ParseMime((char *) pVerBlock->pOutput, &pMimeList);
				
				if (pMimeList != NULL)
				{
					char szTemp[] = "Mime-Version: 1.0\r\n";
					char *szOutBuffer;

					szOutBuffer = (char *) PGPNewData(memoryMgr,
											strlen(szTemp) + 1,
											kPGPMemoryMgrFlags_Clear);

					strcpy(szOutBuffer, szTemp);
					
					WriteDecryptedMimeParts(memoryMgr, pVerBlock, pMimeList, 
						NULL, NULL, &szOutBuffer);

					nobl=PGPscMakeOutBuffItem(&obl);
					nobl->pBuff=szOutBuffer;
					nobl->dwBuffSize=strlen(szOutBuffer);
					FreeMimeList(pMimeList);
				}
				
				PGPFreeData(pVerBlock->szBlockBegin);
				PGPFreeData(pVerBlock->pOutput);
				PGPFreeData(pVerBlock->szBlockEnd);
			}
			else if (pVerIndex->szBlockBegin != NULL)
			{
				if (bHTML)
				{
					if (pVerIndex->pOutput != NULL)
					{
						szStart = sFindFormatBegin(pVerIndex->pOutput,
									&bGotHTML, &bGotRTF, &bGotRTFHTML);

						if (szStart != NULL)
						{
							if (strncmp(szStart, "\\uc", 3))
								szHeader = BEGIN_RTFHTMLALT;

							nTempLength =	szStart - 
											(char *) pVerIndex->pOutput;

							szTemp = (char *) PGPNewData(memoryMgr,
												nTempLength + 1,
												kPGPMemoryMgrFlags_Clear);

							strncpy(szTemp, pVerIndex->pOutput, nTempLength);

							nobl=PGPscMakeOutBuffItem(&obl);
							nobl->pBuff=szTemp;
							nobl->dwBuffSize=nTempLength;

							szEnd = sFindFormatEnd(pVerIndex->pOutput, 
										bGotRTF);
						}
					}

					szTemp = (char *) PGPNewData(memoryMgr,
										strlen(pVerIndex->szBlockBegin)*4
										+ strlen(szHeader),
										kPGPMemoryMgrFlags_Clear);

					nTempLength = 0;
					nFrom = 0;

					if (bGotRTF)
					{
						strcpy(szTemp, szHeader);
						nTempLength = strlen(szHeader);
					}

					for (nIndex=0; nIndex<strlen(pVerIndex->szBlockBegin);
					nIndex++)
					{
						if (pVerIndex->szBlockBegin[nIndex] == '\n')
						{
							memcpy(&(szTemp[nTempLength]),
								&(pVerIndex->szBlockBegin[nFrom]),
								nIndex - nFrom + 1);

							nTempLength += nIndex - nFrom + 1;
							nFrom = nIndex;
							if (bGotRTF)
							{
								if (bGotRTFHTML)
								{
									memcpy(&(szTemp[nTempLength]), 
										"{\\*\\htmltag0 <br> }", 19);
									nTempLength += 19;
								}
								else
								{
									memcpy(&(szTemp[nTempLength]), 
										"\\par", 4);
									nTempLength += 4;
								}
							}
							else
							{
								memcpy(&(szTemp[nTempLength]), "<br>", 4);
								nTempLength += 4;
							}
						}
					}

					nobl=PGPscMakeOutBuffItem(&obl);
					nobl->pBuff=szTemp;
					nobl->dwBuffSize=strlen(szTemp);
					PGPFreeData(pVerIndex->szBlockBegin);
				}
				else
				{
					nobl=PGPscMakeOutBuffItem(&obl);
					nobl->pBuff=pVerIndex->szBlockBegin;
					nobl->dwBuffSize=strlen(pVerIndex->szBlockBegin);
				}
			}
			
			if ((!bMIME || (pVerIndex->szBlockBegin == NULL)) && 
				(pVerIndex->pOutput != NULL))
			{
				if (bHTML && (szStart != NULL))
				{
					if (szEnd != NULL)
						nTempLength = szEnd - szStart;
					else
						nTempLength = strlen(szStart);

					szTemp = (char *) PGPNewData(memoryMgr,
										nTempLength + 1,
										kPGPMemoryMgrFlags_Clear);

					strncpy(szTemp, szStart, nTempLength);

					nobl=PGPscMakeOutBuffItem(&obl);
					nobl->pBuff=szTemp;
					nobl->dwBuffSize=nTempLength;

					if (szEnd == NULL)
						PGPFreeData(pVerIndex->pOutput);
				}
				else
				{
					nobl=PGPscMakeOutBuffItem(&obl);
					nobl->pBuff=pVerIndex->pOutput;
					nobl->dwBuffSize=strlen(pVerIndex->pOutput);
				}
				
				if (PGPscGetSecureViewerPref((void *)context))
					nobl->FYEO = TRUE;
				else
				{
					nobl->FYEO=pVerIndex->FYEO;
					if (nobl->FYEO)
						bFYEO = TRUE;
				}
			}
			
			if (!bMIME && (pVerIndex->szBlockEnd != NULL))
			{
				if (bHTML)
				{
					szTemp = (char *) PGPNewData(memoryMgr,
										strlen(pVerIndex->szBlockEnd)*4
										+ strlen(szHeader)
										+ strlen(END_RTFHTML),
										kPGPMemoryMgrFlags_Clear);

					nTempLength = 0;
					nFrom = 0;

					if (bGotRTF)
					{
						strcpy(szTemp, szHeader);
						nTempLength = strlen(szHeader);
					}

					for (nIndex=0; nIndex<strlen(pVerIndex->szBlockEnd);
					nIndex++)
					{
						if (pVerIndex->szBlockEnd[nIndex] == '\n')
						{
							memcpy(&(szTemp[nTempLength]),
								&(pVerIndex->szBlockEnd[nFrom]),
								nIndex - nFrom + 1);

							nTempLength += nIndex - nFrom + 1;
							nFrom = nIndex;
							if (bGotRTF)
							{
								if (bGotRTFHTML)
								{
									memcpy(&(szTemp[nTempLength]), 
										"{\\*\\htmltag0 <br> }", 19);
									nTempLength += 19;
								}
								else
								{
									memcpy(&(szTemp[nTempLength]), 
										"\\par", 4);
									nTempLength += 4;
								}
							}
							else
							{
								memcpy(&(szTemp[nTempLength]), "<br>", 4);
								nTempLength += 4;
							}
						}
					}

					if (bGotRTF)
					{
						strcat(szTemp, END_RTFHTML);
						nTempLength += strlen(END_RTFHTML);
					}

					nobl=PGPscMakeOutBuffItem(&obl);
					nobl->pBuff=szTemp;
					nobl->dwBuffSize=strlen(szTemp);
					PGPFreeData(pVerIndex->szBlockEnd);
				}
				else
				{
					nobl=PGPscMakeOutBuffItem(&obl);
					nobl->pBuff=pVerIndex->szBlockEnd;
					nobl->dwBuffSize=strlen(pVerIndex->szBlockEnd);
				}
				
				if (bHTML)
				{
					if (szEnd != NULL)
					{
						nTempLength = strlen(szEnd);

						szTemp = (char *) PGPNewData(memoryMgr,
											nTempLength + 1,
											kPGPMemoryMgrFlags_Clear);

						strncpy(szTemp, szEnd, nTempLength);

						nobl=PGPscMakeOutBuffItem(&obl);
						nobl->pBuff=szTemp;
						nobl->dwBuffSize=nTempLength;

						PGPFreeData(pVerIndex->pOutput);
					}
				}
			}
			
			pTempBlock = pVerIndex;
			pVerIndex=pVerIndex->next;
			PGPFreeData(pTempBlock);
		}
		while (pVerIndex != NULL);
		
		// Concatinate them to ppOutput
		PGPscConcatOutBuffList((void *)context,
			obl,
			(char **)ppOutput,
			pOutSize,
			FYEO);
	}

	/* We don't want to show the FYEO warning if the user has
	   "Always use Secure Viewer" on */

	*FYEO = bFYEO;

DecryptVerifyBufferError:

	if (options != NULL)
		PGPFreeOptionList(options);

	return err;
}


PGPError DecryptVerifyFile(HINSTANCE hInst, HWND hwnd, PGPContextRef context, 
						   PGPtlsContextRef tlsContext, 
						   char *szName, char *szModule,
						   char *szInFile, BOOL bMIME, BOOL bBinary,
						   char **pszOutFile, void **ppOutput, 
						   PGPSize *pOutSize, BOOL *FYEO)
{
	PGPError			err			= kPGPError_NoErr;
	PGPMemoryMgrRef		memoryMgr	= NULL;
	PGPOptionListRef	options		= NULL;
	PGPFileSpecRef		inputFile	= NULL;
	PGPFileSpecRef		outputFile	= NULL;
	PGPFileSpecRef		finalFile	= NULL;
	PGPFileSpecRef		dataFile	= NULL;
	PGPUInt32			macCreator	= 0;
	PGPUInt32			macType		= 0;
	VerificationBlock *	pVerBlock	= NULL;
	VerificationBlock * pTempBlock	= NULL;
	char *				szExtension = NULL;
	char *				szOutFile	= NULL;
	OUTBUFFLIST	      * obl			= NULL;
	OUTBUFFLIST       * nobl		= NULL;
	BOOL				bFYEO		= FALSE;
	BOOL				bDoFYEO		= FALSE;
	PGPByte				contents	= 0;
	BOOL				bKeysOnly	= FALSE;

	pgpAssert(szInFile != NULL);
	pgpAssert(pszOutFile != NULL);
	pgpAssert(szName != NULL);
	pgpAssert(szModule != NULL);
	pgpAssert(context != NULL);
	pgpAssert(tlsContext != NULL);

	if (ppOutput != NULL)
		*ppOutput = NULL;

	if (pOutSize != NULL)
		*pOutSize = 0;

	memoryMgr = PGPPeekContextMemoryMgr(context);

	err = PGPNewFileSpecFromFullPath(context, szInFile, &inputFile);
	if (IsPGPError(err))
	{
		DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
		goto DecryptVerifyFileError;
	}

	szOutFile = (char *) calloc(sizeof(char), strlen(szInFile)+5);
	if (szOutFile == NULL)
	{
		err = kPGPError_OutOfMemory;
		DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
		goto DecryptVerifyFileError;
	}

	strcpy(szOutFile, szInFile);
 	szExtension = strrchr(szOutFile, '.');
	if (szExtension != NULL)
	{
		if (!strcmp(szExtension, ".asc") || !strcmp(szExtension, ".pgp"))
			*szExtension = '\0';
		else
			strcat(szOutFile, ".tmp");
	}
	else
		strcat(szOutFile, ".tmp");

	SetFileAttributes(szOutFile, FILE_ATTRIBUTE_NORMAL);

	if (bBinary)
	{
		err = PGPNewFileSpecFromFullPath(context, szOutFile, &outputFile);
		if (IsPGPError(err))
		{
			DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
			goto DecryptVerifyFileError;
		}
		
		err = PGPBuildOptionList(context, &options, 
				PGPOInputFile(context, inputFile),
				PGPOOutputFile(context, outputFile),
				PGPOLastOption(context));
	}
	else
	{
		pVerBlock = (VerificationBlock *) PGPNewData(memoryMgr,
											sizeof(VerificationBlock),
											kPGPMemoryMgrFlags_Clear);

		pVerBlock->next = NULL;
		pVerBlock->szBlockBegin = NULL;
		pVerBlock->szBlockEnd = NULL;
		pVerBlock->pOutput = NULL;
		pVerBlock->outSize = 0;
		pVerBlock->bEncrypted = FALSE;
		pVerBlock->FYEO = FALSE;

		err = PGPBuildOptionList(context, &options, 
				PGPOInputFile(context, inputFile),
				PGPOLastOption(context));
	}

		
	if (IsPGPError(err))
	{
		DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
		goto DecryptVerifyFileError;
	}

	err = DecryptVerify(hInst, hwnd, context, tlsContext, szName, szModule, 
			options, bMIME, pVerBlock, !bBinary, &bKeysOnly);

	if (!bBinary)
	{
		HANDLE hFile;
		DWORD dwWritten;
		BOOL bFixed = FALSE;

		hFile = CreateFile(szOutFile, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
					FILE_ATTRIBUTE_NORMAL, NULL);

		pTempBlock = pVerBlock;
		while (pVerBlock != NULL)
		{
			if (pVerBlock->FYEO)
				bDoFYEO = TRUE;

			pVerBlock = pVerBlock->next;
		}
		pVerBlock = pTempBlock;

		if (bDoFYEO || (PGPscGetSecureViewerPref((void *)context)))
		{
			// Convert pVerBlock to OUTBUFFLIST
			do
			{
				if (pVerBlock->szBlockBegin != NULL)
				{
					nobl=PGPscMakeOutBuffItem(&obl);
					nobl->pBuff=pVerBlock->szBlockBegin;
					nobl->dwBuffSize=strlen(pVerBlock->szBlockBegin);
				}
				
				if (pVerBlock->pOutput != NULL)
				{
					nobl=PGPscMakeOutBuffItem(&obl);
					nobl->pBuff=pVerBlock->pOutput;
					nobl->dwBuffSize=strlen(pVerBlock->pOutput);

					if (PGPscGetSecureViewerPref((void *)context))
						nobl->FYEO = TRUE;
					else
					{
						nobl->FYEO=pVerBlock->FYEO;
						if (nobl->FYEO)
							bFYEO = TRUE;
					}
				}
				
				if (pVerBlock->szBlockEnd != NULL)
				{
					nobl=PGPscMakeOutBuffItem(&obl);
					nobl->pBuff=pVerBlock->szBlockEnd;
					nobl->dwBuffSize=strlen(pVerBlock->szBlockEnd);
				}
				
				pTempBlock = pVerBlock;
				pVerBlock=pVerBlock->next;
				PGPFreeData(pTempBlock);
			}
			while (pVerBlock != NULL);
			
			// Concatinate them to ppOutput
			PGPscConcatOutBuffList((void *)context,
				obl,
				(char **)ppOutput,
				pOutSize,
				FYEO);

			/* We don't want to show the FYEO warning if the user has
			   "Always use Secure Viewer" on */

			*FYEO = bFYEO;
		}
		else do
		{
			// Fix for Eudora 4.0.1 bug where "boundary=..." is on
			// a separate line

			if (pVerBlock->szBlockBegin != NULL)
			{
				if (!bMIME)
				{
					WriteFile(hFile, pVerBlock->szBlockBegin, 
						strlen(pVerBlock->szBlockBegin), &dwWritten, 
						NULL);
						
					WriteFile(hFile, pVerBlock->pOutput, 
						strlen((char *) pVerBlock->pOutput), &dwWritten, 
						NULL);
					
					WriteFile(hFile, pVerBlock->szBlockEnd, 
						strlen(pVerBlock->szBlockEnd), &dwWritten, NULL);
					
					PGPFreeData(pVerBlock->szBlockBegin);
					PGPFreeData(pVerBlock->pOutput);
					PGPFreeData(pVerBlock->szBlockEnd);
				}
				else
				{
					BOOL bMultiMixed = FALSE;
					BOOL bMultiAlt = FALSE;
					BOOL bFirstMimePart = FALSE;
					MimePart *pMimeList = NULL;
					MimePart *pFirstMimeList = NULL;
					
					ParseMime((char *) pVerBlock->pOutput, &pMimeList);

					if (pMimeList != NULL)
					{
						char szTemp[] = "Mime-Version: 1.0\r\n";

						WriteFile(hFile, szTemp, strlen(szTemp), &dwWritten,
							NULL);
						WriteDecryptedMimeParts(memoryMgr, pVerBlock, 
							pMimeList, NULL, hFile, NULL);
					}

					FreeMimeList(pMimeList);
					PGPFreeData(pVerBlock->szBlockBegin);
					PGPFreeData(pVerBlock->pOutput);
					PGPFreeData(pVerBlock->szBlockEnd);
				}
			}
			else if (pVerBlock->pOutput != NULL)
			{
				WriteFile(hFile, pVerBlock->pOutput, 
					strlen((char *) pVerBlock->pOutput), &dwWritten, 
					NULL);
				
				PGPFreeData(pVerBlock->pOutput);
			}
				
			pTempBlock = pVerBlock;
			pVerBlock = pTempBlock->next;
			PGPFreeData(pTempBlock);
		}
		while (pVerBlock != NULL);
			
		CloseHandle(hFile);

		*pszOutFile = (char *) PGPNewData(memoryMgr,
									strlen(szOutFile)+1,
									kPGPMemoryMgrFlags_Clear);
		strcpy(*pszOutFile, szOutFile);
	}
	else
	{
		PGPMacBinaryToLocal(outputFile, &finalFile, &macCreator, &macType);
		if (finalFile != NULL)
			PGPGetFullPathFromFileSpec(finalFile, pszOutFile);
		else
		{
			*pszOutFile = (char *) PGPNewData(memoryMgr,
										strlen(szOutFile)+1,
										kPGPMemoryMgrFlags_Clear);
			strcpy(*pszOutFile, szOutFile);
		}

		if (bKeysOnly)
		{
			DeleteFile(*pszOutFile);
			PGPFreeData(*pszOutFile);
			*pszOutFile = NULL;
		}
	}

DecryptVerifyFileError:

	if (finalFile != NULL)
		PGPFreeFileSpec(finalFile);

	if (inputFile != NULL)
		PGPFreeFileSpec(inputFile);

	if (outputFile != NULL)
		PGPFreeFileSpec(outputFile);

	if (options != NULL)
		PGPFreeOptionList(options);

	if (szOutFile != NULL)
		free(szOutFile);

	return err;
}


PGPError DecryptVerify(HINSTANCE hInst, HWND hwnd, PGPContextRef context, 
					   PGPtlsContextRef tlsContext, 
					   char *szName, char *szModule,
					   PGPOptionListRef options, BOOL bMIME,
					   VerificationBlock *pVerBlock, BOOL bImportIfKeysOnly,
					   BOOL *bKeysOnly)
{
	PGPError			err			= kPGPError_NoErr;
	PGPKeyDBRef			keyDB		= NULL;
	PGPKeyDBRef			newKeyDB	= NULL;
	PGPOptionListRef	tempOptions	= NULL;
	PGPUInt32			nNumKeys	= 0;
	HWND				hwndWorking	= NULL;
	char				szWorkingTitle[256];
	DecodeEventData		decodeData;

	UpdateWindow(hwnd);

#if PGP_NOTES
	// Check license number and expiration with clientlib functions.
	// This is to avoid having to compile all the PGPlnLib code into
	// the Notes plugin
	if(!PGPclPGPdesktopEnabled())
	{
		PGPclNoLicense(hwnd,kPGPclLotusNotesPlugin);
		return kPGPError_UserAbort;
	}

	// Check for demo expiration
	if (PGPclIsExpired(hwnd,
			(PGPclExpType)kPGPlnOperationGraceExpiration) != kPGPError_NoErr)
	{
		PGPclErrorBox(hwnd, kPGPError_Win32_Expired);
		return kPGPError_UserAbort;
	}
#else
	// Check license number and expiration with static library functions.
	// This makes it more difficult to crack.
	PGPlnLicenseCheckLite (context);
	if(!PGPlnPGPmailEnabled())
	{
		PGPclNoLicense(hwnd,kPGPclPGPmailPlugin);
		return kPGPError_UserAbort;
	}
	// Check for expiration
	if (PGPlnIsExpired(hwnd, kPGPlnOperationGraceExpiration) != kPGPError_NoErr)
		return kPGPError_UserAbort;
#endif
	
	decodeData.hInst		= hInst;
	decodeData.hwnd			= hwnd;
	decodeData.tlsContext	= tlsContext;
	decodeData.szName		= szName;
	decodeData.recipients	= NULL;
	decodeData.keyCount		= 0;
	decodeData.keyIDArray	= NULL;
	decodeData.pVerBlock	= pVerBlock;
	decodeData.contents		= 0;
	decodeData.fsrFileRef   = NULL;

	*bKeysOnly = FALSE;

	err = PGPclOpenDefaultKeyrings(context, kPGPOpenKeyDBFileOptions_Mutable, 
			&keyDB);

	if (IsPGPError(err))
		err = PGPclOpenDefaultKeyrings(context, kPGPOpenKeyDBFileOptions_None, 
				&keyDB);

	if (IsPGPError(err))
	{
		DisplayErrorCode(__FILE__, __LINE__, szModule, err, TRUE, hwnd);
		goto DecryptVerifyError;
	}

	decodeData.keyDB = keyDB;
	decodeData.keydbNew = NULL;
	PGPNewKeyDB(context, &newKeyDB);

	LoadString(hInst, IDS_WORKINGDECRYPT, szWorkingTitle, 
		sizeof(szWorkingTitle));

	hwndWorking = WorkingDlgProcThread(GetModuleHandle(szModule), hInst, hwnd,
					szWorkingTitle, "");
	decodeData.hwndWorking = hwndWorking;

	err = PGPDecode(context,
			options,
			PGPOPassThroughIfUnrecognized(context, (PGPBoolean) !bMIME),
			PGPOPassThroughKeys(context, TRUE),
			PGPOEventHandler(context, DecodeEventHandler, &decodeData),
			PGPOSendNullEvents(context, 100),
			PGPOImportKeysTo(context, newKeyDB),
			PGPOKeyDBRef(context, decodeData.keyDB),
			PGPOLastOption(context));

	WorkingCallback(hwndWorking, -1, -1);

	if (IsPGPError(err) && (err != kPGPError_UserAbort))
	{
		DisplayErrorCode(__FILE__, __LINE__, szModule, err, FALSE, hwnd);
		goto DecryptVerifyError;
	}

	if (newKeyDB != NULL)
	{
		PGPKeySetRef newKeySet;

		PGPNewKeySet(newKeyDB, &newKeySet);
		PGPCountKeys(newKeySet, &nNumKeys);

		if (nNumKeys > 0)
		{
			if (!(decodeData.contents & CONTENTS_ENCRYPTED) && 
				!(decodeData.contents & CONTENTS_SIGNED))
				*bKeysOnly = TRUE;

			if (bImportIfKeysOnly || !(*bKeysOnly))
				PGPclImportKeys(context, tlsContext, hwnd, newKeySet, keyDB, 0);
		}

		PGPFreeKeySet(newKeySet);
	}

DecryptVerifyError:

	if (decodeData.recipients != NULL)
		PGPFreeKeySet(decodeData.recipients);

	if (decodeData.keyIDArray != NULL)
		free(decodeData.keyIDArray);

	if (newKeyDB != NULL)
		PGPFreeKeyDB(newKeyDB);

	if (keyDB != NULL)
		PGPFreeKeyDB(keyDB);

	return err;
}


BOOL GetOriginalFileRef(HWND hwnd,PGPContextRef context,
						char *InputFile,
						char *OutputFile,
						PGPFileSpecRef *OriginalFileRef,
						HWND hwndWorking, HINSTANCE hInst)
{
	char OriginalFile[MAX_PATH+1]={0};
	char *pTempPtr = NULL;
	BOOL bCancelled = FALSE;
	OPENFILENAME ofnStruct={0};
	char szFileFilter[500]={0};
	char szOpenFileTitle[500]={0};

	pgpAssert(NULL != InputFile);
	pgpAssert(NULL != OriginalFileRef);

	strcpy(OriginalFile, InputFile);

	LoadString (hInst, IDS_SAVEFILTER, szFileFilter, sizeof(szFileFilter));
	while (pTempPtr = strrchr (szFileFilter, '@')) 
		*pTempPtr = '\0';

	LoadString (hInst, IDS_SELSIGNFILE, szOpenFileTitle, sizeof(szOpenFileTitle));

	ofnStruct.lStructSize=sizeof(ofnStruct); 
    ofnStruct.hwndOwner=hwnd; 
    ofnStruct.lpstrFilter=szFileFilter; 
    ofnStruct.nFilterIndex=1; 
    ofnStruct.lpstrFile=OriginalFile; 
    ofnStruct.nMaxFile=MAX_PATH; 
    ofnStruct.lpstrTitle=szOpenFileTitle; 
    ofnStruct.Flags=OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY |
						   OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST | OFN_NOCHANGEDIR;
#ifdef WIN32
	ofnStruct.Flags=ofnStruct.Flags | OFN_EXPLORER;
#endif

	if(GetOpenFileName(&ofnStruct))
	{
		PGPNewFileSpecFromFullPath(context, OriginalFile, OriginalFileRef);
	}
	else
		bCancelled = TRUE;

	return bCancelled;
}

PGPError DecodeEventHandler(PGPContextRef context, 
							PGPEvent *event, 
							PGPUserValue userValue)
{
	HWND			hwnd			= NULL;
	char *			szPassPhrase	= NULL;
	PGPByte *		pPasskey		= NULL;
	PGPUInt32		nPasskeyLength	= 0;
	static BOOL		bAlreadyAsked	= FALSE;
	char *			szName			= NULL;
	PGPMemoryMgrRef	memoryMgr		= NULL;
	PGPKeyDBRef		keyDB			= NULL;
	PGPKeyDBRef		addedKeys		= NULL;
	PGPKeySetRef	recipients		= NULL;
	PGPUInt32		keyCount		= 0;
	PGPKeyID		*keyIDArray		= NULL;
	DecodeEventData	*userData		= NULL;
	HWND			hwndWorking		= NULL;
	PGPError		err				= kPGPError_NoErr;

	pgpAssert(context != NULL);
	pgpAssert(event != NULL);

	userData = (DecodeEventData *) userValue;

	hwnd			= userData->hwnd;
	hwndWorking		= userData->hwndWorking;
	szName			= userData->szName;
	keyDB			= userData->keyDB;
	recipients		= userData->recipients;
	keyCount		= userData->keyCount;
	keyIDArray		= userData->keyIDArray;

	memoryMgr = PGPPeekContextMemoryMgr(context);

	switch (event->type)
	{
	case kPGPEvent_NullEvent:
		{	
			PGPEventNullData *d = &event->data.nullData;
			BOOL bCancel;

			bCancel = WorkingCallback (	hwndWorking, 
										(unsigned long)d->bytesWritten, 
										(unsigned long)d->bytesTotal) ;

			if(bCancel)
			{
				return kPGPError_UserAbort;
			}
		}
		break;

	case kPGPEvent_BeginLexEvent:
		if (userData->pVerBlock != NULL)
		{
			userData->pVerBlock->bEncrypted = FALSE;
		}
		break;

	case kPGPEvent_OutputEvent:
		if (userData->pVerBlock != NULL)
		{
			PGPEventOutputData *d = &event->data.outputData;

			// Added FYEO member for tempest viewer
			userData->pVerBlock->FYEO=d->forYourEyesOnly;

			PGPAddJobOptions(event->job,
				PGPOAllocatedOutputBuffer(context, 
					&(userData->pVerBlock->pOutput), 
					INT_MAX, 
					&(userData->pVerBlock->outSize)),
				PGPOLastOption(context));

			userData->pVerBlock->next = (VerificationBlock *) 
				PGPNewData(memoryMgr,
					sizeof(VerificationBlock),
					kPGPMemoryMgrFlags_Clear);

			userData->pVerBlock->next->previous = userData->pVerBlock;
			userData->pVerBlock = userData->pVerBlock->next;
			userData->pVerBlock->szBlockBegin = NULL;
			userData->pVerBlock->szBlockEnd = NULL;
			userData->pVerBlock->pOutput = NULL;
			userData->pVerBlock->outSize = 0;
			userData->pVerBlock->bEncrypted = FALSE;
			userData->pVerBlock->FYEO = FALSE;
		}
		break;

	case kPGPEvent_RecipientsEvent:
		{
			PGPEventRecipientsData	*eventData; 

			eventData = &(event->data.recipientsData);

			if (eventData->recipientSet != NULL)
			{
				PGPIncKeySetRefCount(eventData->recipientSet);
				userData->recipients = eventData->recipientSet;
			}

			userData->keyCount = eventData->keyCount;

			if (eventData->keyCount > 0)
			{
				UINT i;

				userData->keyIDArray =	(PGPKeyID *) 
										calloc(sizeof(PGPKeyID),
												eventData->keyCount);

				for (i=0; i<eventData->keyCount; i++)
					userData->keyIDArray[i] = eventData->keyIDArray[i];
			}
			else
				userData->keyIDArray = NULL;
		}
		break;

	case kPGPEvent_PassphraseEvent:
		{
			char szPrompt[256];
			PGPEventPassphraseData *d = &event->data.passphraseData;

			if (bAlreadyAsked)
				LoadString(userData->hInst, IDS_PASSPHRASEREENTER, szPrompt, 
					sizeof(szPrompt));
			else
				LoadString(userData->hInst, IDS_PASSPHRASEPROMPT, szPrompt, 
					sizeof(szPrompt));

			// Don't cache conventional passphrases
			if(d->fConventional)
			{
				err = PGPclGetConventionalPhrase(context, hwnd, szPrompt,
						&szPassPhrase);
			}
			else
			{
				err = PGPclGetCachedDecryptionPhrase(context, 
						userData->tlsContext, keyDB, hwnd, szPrompt, 
						bAlreadyAsked, &szPassPhrase, recipients, keyIDArray,
						keyCount, &pPasskey, &nPasskeyLength, &addedKeys,
						NULL);
			}
			
			if (addedKeys != NULL)
			{
				PGPUInt32 numKeys;
				PGPKeySetRef addedKeySet;

				PGPNewKeySet(addedKeys, &addedKeySet);				
				PGPCountKeys(addedKeySet, &numKeys);
				if (numKeys > 0)
					PGPclImportKeys(context, userData->tlsContext, hwnd, 
						addedKeySet, userData->keyDB, 
						kPGPclNoDialogForValidSingletons);
				
				PGPFreeKeySet(addedKeySet);
				PGPFreeKeyDB(addedKeys);
				addedKeys = NULL;
			}
			
			switch (err)
			{
			case kPGPError_NoErr:
				bAlreadyAsked = TRUE;
				break;
				
			default:
				return err;
			}
			
			if (szPassPhrase != NULL)
			{
				err = PGPAddJobOptions(event->job,
						PGPOPassphrase(context, szPassPhrase),
						PGPOLastOption(context));
				
				PGPclFreeCachedPhrase(szPassPhrase);
				szPassPhrase = NULL;
			}
			else if (pPasskey != NULL)
			{
				err = PGPAddJobOptions(event->job, 
						PGPOPasskeyBuffer(context, pPasskey, nPasskeyLength),
						PGPOLastOption(context));
			}
			
			if (pPasskey != NULL)
			{
				PGPFreeData(pPasskey);
				pPasskey = NULL;
				nPasskeyLength = 0;
			}
		}
		break;

	case kPGPEvent_AnalyzeEvent:
		if (userData->pVerBlock != NULL)
		{
			if (event->data.analyzeData.sectionType == kPGPAnalyze_Encrypted)
				userData->pVerBlock->bEncrypted = TRUE;
		}

		switch (event->data.analyzeData.sectionType)
		{
		case kPGPAnalyze_Encrypted:
			userData->contents |= CONTENTS_ENCRYPTED;
			break;

		case kPGPAnalyze_Signed:
			userData->contents |= CONTENTS_SIGNED;
			break;
		}
		break;

	case kPGPEvent_SignatureEvent:
		{
			PGPEventSignatureData *d = &event->data.signatureData;

			if ((d->signingKey == NULL) && SyncOnVerify(memoryMgr))
			{
				PGPBoolean bGotKeys;
				PGPKeyDBRef keydbNew = NULL;

				PGPclLookupUnknownSigner(context, keyDB, 
					userData->tlsContext, hwnd, event, d->signingKeyID, 
					&bGotKeys,&keydbNew);

				userData->keydbNew = keydbNew;

				if (bGotKeys)
					return kPGPError_NoErr;
			}

			if (userData->pVerBlock == NULL)
				SigEvent(hwnd, context, d, szName);
			else
			{
				if (userData->pVerBlock->previous == NULL)
					CreateVerificationBlock(userData->hInst, context, d,
						userData->pVerBlock->bEncrypted,
						&(userData->pVerBlock->szBlockBegin),
						&(userData->pVerBlock->szBlockEnd));
				else
					CreateVerificationBlock(userData->hInst, context, d,
						userData->pVerBlock->previous->bEncrypted,
						&(userData->pVerBlock->previous->szBlockBegin),
						&(userData->pVerBlock->previous->szBlockEnd));
			}
		}
		break;

	case kPGPEvent_DetachedSignatureEvent:
		{
			int UserCancel;

			pgpAssert(NULL == userData->fsrFileRef);
			UserCancel = GetOriginalFileRef(hwndWorking, context,
				"", NULL,	&(userData->fsrFileRef), hwndWorking, 
				userData->hInst);

			if(UserCancel)
			{
				//s->bVerEncrypted=FALSE;
				return kPGPError_UserAbort;
			}

			PGPAddJobOptions(event->job,
				PGPODetachedSig(context,
					PGPOInputFile(context, userData->fsrFileRef),
					PGPOLastOption(context)),
				PGPOLastOption(context));
		}
		break;

	case kPGPEvent_FinalEvent:
		if (szPassPhrase != NULL)
		{
			PGPclFreeCachedPhrase(szPassPhrase);
			szPassPhrase = NULL;
		}
		if (pPasskey != NULL)
		{
			PGPFreeData(pPasskey);
			pPasskey = NULL;
			nPasskeyLength = 0;
		}
		if (userData->keydbNew != NULL)
		{
			PGPFreeKeyDB(userData->keydbNew);
			userData->keydbNew = NULL;
		}
		if(userData->fsrFileRef != NULL)
		{
			PGPFreeFileSpec(userData->fsrFileRef);
			userData->fsrFileRef = NULL;
		}

		bAlreadyAsked = FALSE;
		break;
	}

	return err;
}


void DisplayErrorCode(char *szFile, int nLine, char *szModule, int nCode,
					  BOOL bKeyrings, HWND hParentWnd)
{
	char szErrorMsg[255];

	if (nCode == kPGPError_BadPacket)
		nCode = kPGPError_CorruptData;

	if (bKeyrings)
		PGPclErrorToString(nCode, szErrorMsg, 254, NULL);
	else
		PGPclEncDecErrorToString(nCode, szErrorMsg, 254);

#ifdef _DEBUG
	_CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_WNDW);
	_CrtDbgReport(_CRT_ERROR, szFile, nLine, szModule, szErrorMsg);
#endif

	MessageBox(hParentWnd, szErrorMsg, szModule, MB_ICONEXCLAMATION);
	return;
}


static char * sFindFormatBegin(char *szString, BOOL *pbHTML, BOOL *pbRTF, 
							   BOOL *pbRTFHTML)
{
	char *szSearch = NULL;
	char *szStart;
	char *szEnd;
	char *szFormat;
	char *szAltFormat;
	int nLength;

	szSearch = szString;
	szEnd = szString + strlen(szString) - 6;

	*pbHTML = FALSE;
	*pbRTF = FALSE;
	*pbRTFHTML = FALSE;

	while (strnicmp(szSearch, "<html>", 6) &&
		strnicmp(szSearch, "{\\rtf", 5) &&
		(szSearch < szEnd))
	{
		szSearch++;
	}

	if (szSearch < szEnd)
	{
		szStart = szSearch;
		if (!strnicmp(szSearch, "<html>", 6))
		{
			*pbHTML = TRUE;
			szFormat = "<body";
			szAltFormat = NULL;
		}
		else
		{
			*pbRTF = TRUE;
			szFormat = "\\viewkind";
			szAltFormat = "\\uc";
		}

		nLength = strlen(szFormat);
		szEnd = szString + strlen(szString) - nLength;

		while (strnicmp(szSearch, szFormat, nLength) && (szSearch < szEnd))
			szSearch++;

		if ((szSearch == szEnd) && (szAltFormat != NULL))
		{
			szSearch = szStart;
			nLength = strlen(szAltFormat);
			szEnd = szString + strlen(szString) - nLength;

			while (strnicmp(szSearch, szAltFormat, nLength) && 
				(szSearch < szEnd))
				szSearch++;
		}
	}

	if (*pbRTF)
	{
		char *szHtmlRtf = "\\htmlrtf";
		int nHtmlRtf = strlen(szHtmlRtf);
		int nBack;

		for (nBack=0; nBack<3; nBack++)
		{
			if (!strnicmp(szSearch - nHtmlRtf - nBack, szHtmlRtf, nHtmlRtf))
			{
				szSearch -= nHtmlRtf + nBack;
				*pbRTFHTML = TRUE;
				break;
			}
		}

		return szSearch;
	}

	if (szSearch < szEnd)
	{
		szFormat = ">";
		nLength = strlen(szFormat);
		szEnd = szString + strlen(szString) - nLength;

		while (strnicmp(szSearch, szFormat, nLength) && (szSearch < szEnd))
			szSearch++;
	}

	if (szSearch < szEnd)
		szSearch += nLength;
	else
	{
		szSearch = NULL;
		*pbHTML = FALSE;
	}

	return szSearch;
}


static char * sFindFormatEnd(char *szString, BOOL bRTF)
{
	char *szSearch = NULL;
	char *szFormat;
	int nLength;

	if (bRTF)
		szFormat = "}";
	else
		szFormat = "</body>";

	nLength = strlen(szFormat);
	szSearch = szString + strlen(szString) - nLength;

	while (strnicmp(szSearch, szFormat, nLength) && (szSearch > szString))
		szSearch--;

	if (szSearch == szString)
		szSearch = NULL;

	return szSearch;
}


static void WriteFileOrBuffer(PGPMemoryMgrRef mgr, HANDLE hFile, 
							  char **szBuffer, char *szData,
							  int nDataLength, DWORD *pBytesWritten)
{
	if (hFile != NULL)
		WriteFile(hFile, szData, nDataLength, pBytesWritten, NULL);

	if (szBuffer != NULL)
	{
		int nOrigLength = strlen(*szBuffer);

		PGPReallocData(mgr, szBuffer, nOrigLength + nDataLength + 1, 0);
		memcpy(*szBuffer + nOrigLength, szData, nDataLength);
		(*szBuffer)[nOrigLength + nDataLength] = 0;
	}

	return;
}


static void	WriteDecryptedMimeParts(PGPMemoryMgrRef mgr,
									VerificationBlock *pVerBlock, 
									MimePart *pMimeList,
									MimePart **ppMimeFooter,
									HANDLE hFile,
									char **szOutBuffer)
{
	DWORD dwWritten;
	BOOL bMultipartMixed = FALSE;
	BOOL bMultipartAlt = FALSE;
	BOOL bWroteHeader = FALSE;
	BOOL bWriteFooter = FALSE;
	MimePart *grandparentPart = NULL;
	MimePart *pMimeFooter = NULL;

	if (pMimeList->parentPart != NULL)
	{
		if (pMimeList->parentPart->nContentType == ContentType_MultipartMixed)
			bMultipartMixed = TRUE;

		if (pMimeList->parentPart->nContentType == ContentType_MultipartAlternative)
			bMultipartAlt = TRUE;

		grandparentPart = pMimeList->parentPart->parentPart;
	}

	bWroteHeader = FALSE;
	while (pMimeList != NULL)
	{
		if (pMimeList->parentPart != NULL)
		{
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, "--", 2, &dwWritten);
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, 
				pMimeList->parentPart->szBoundary, 
				pMimeList->parentPart->nBoundaryLength, &dwWritten);
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, "\r\n", 2, &dwWritten);
		}

		if (pMimeList->szHeader)
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, 
				pMimeList->szHeader, pMimeList->nHeaderLength, &dwWritten);

		if (pMimeList->subPart != NULL)
			WriteDecryptedMimeParts(mgr, pVerBlock, pMimeList->subPart, 
				&pMimeFooter, hFile, szOutBuffer);

		if ((pMimeList->nContentType != ContentType_MultipartMixed) &&
			(pMimeList->nContentType != ContentType_MultipartAlternative))
		{
			if (((!bMultipartAlt && !bWroteHeader) || bMultipartAlt) &&
				!(pMimeList->bAttachment))
			{
				bWroteHeader = TRUE;

				if (pMimeList->nContentType == ContentType_TextHTML)
				{
					char szTemp[] = "<html><pre>\r\n";
					
					WriteFileOrBuffer(mgr, hFile, szOutBuffer, szTemp, 
						strlen(szTemp), &dwWritten);
				}
				
				WriteFileOrBuffer(mgr, hFile, szOutBuffer, 
					pVerBlock->szBlockBegin, strlen(pVerBlock->szBlockBegin), 
					&dwWritten);
				
				if (pMimeList->nContentType == ContentType_TextHTML)
				{
					char szTemp[] = "</html>\r\n";
					
					WriteFileOrBuffer(mgr, hFile, szOutBuffer, szTemp, 
						strlen(szTemp), &dwWritten);
				}
			}

			WriteFileOrBuffer(mgr, hFile, szOutBuffer, pMimeList->szBody, 
				pMimeList->nBodyLength, &dwWritten);

			if ((grandparentPart != NULL) && (pMimeList->parentPart != NULL))
			{
				if (pMimeList->parentPart->nextPart == NULL)
					bWriteFooter = TRUE;

				pMimeFooter = pMimeList->parentPart;
			}
			else if (!(pMimeList->bAttachment))
			{
				if (pMimeList->nextPart != NULL)
				{
					if (!(pMimeList->nextPart->bAttachment))
						bWriteFooter = TRUE;
				}
				else
					bWriteFooter = TRUE;

				pMimeFooter = pMimeList;
			}
			else if (pMimeList->nextPart == NULL)
			{
				bWriteFooter = TRUE;
				pMimeFooter = pMimeList;
			}
		}

		if (ppMimeFooter != NULL)
			*ppMimeFooter = pMimeFooter;
		
		if (bWriteFooter)
		{
			MimePart *tempPart = pMimeFooter->nextPart;

			pMimeFooter->nextPart = NULL;
			WriteDecryptedMimeFooter(mgr, pVerBlock, pMimeFooter, hFile, 
				szOutBuffer);
			pMimeFooter->nextPart = tempPart;
		}

		if (pMimeList->subPart != NULL)
		{
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, "--", 2, &dwWritten);
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, pMimeList->szBoundary, 
				pMimeList->nBoundaryLength, &dwWritten);
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, "--\r\n", 4, 
				&dwWritten);
		}

		pMimeList = pMimeList->nextPart;
	}

	return;
}


static void	WriteDecryptedMimeFooter(PGPMemoryMgrRef mgr,
									 VerificationBlock *pVerBlock, 
									 MimePart *pMimeList,
									 HANDLE hFile,
									 char **szOutBuffer)
{
	DWORD dwWritten;
	MimePart *grandparentPart = NULL;
	MimePart *thisPart;

	if (pMimeList->parentPart != NULL)
		grandparentPart = pMimeList->parentPart->parentPart;

	while (pMimeList != NULL)
	{
		if (pMimeList->bAttachment)
		{
			thisPart = pMimeList->previousPart;
			while (thisPart->bAttachment && thisPart->previousPart)
				thisPart = thisPart->previousPart;
		}
		else
			thisPart = pMimeList;

		if (((pMimeList->parentPart != NULL) && (grandparentPart != NULL))
			|| pMimeList->bAttachment)
		{
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, "--", 2, &dwWritten);
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, 
				pMimeList->parentPart->szBoundary, 
				pMimeList->parentPart->nBoundaryLength, &dwWritten);
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, "\r\n", 2, &dwWritten);

			WriteFileOrBuffer(mgr, hFile, szOutBuffer, thisPart->szHeader, 
				thisPart->nHeaderLength, &dwWritten);
		}
		
		if (thisPart->subPart != NULL)
			WriteDecryptedMimeFooter(mgr, pVerBlock, thisPart->subPart, hFile,
				szOutBuffer);
		
		if ((thisPart->nContentType != ContentType_MultipartMixed) &&
			(thisPart->nContentType != ContentType_MultipartAlternative))
		{
			if (thisPart->nContentType == ContentType_TextHTML)
			{
				char szTemp[] = "<html><pre>\r\n";
				
				WriteFileOrBuffer(mgr, hFile, szOutBuffer, szTemp, 
					strlen(szTemp), &dwWritten);
			}
			
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, pVerBlock->szBlockEnd, 
				strlen(pVerBlock->szBlockEnd), &dwWritten);
			
			if (thisPart->nContentType == ContentType_TextHTML)
			{
				char szTemp[] = "</html>\r\n";
				
				WriteFileOrBuffer(mgr, hFile, szOutBuffer, szTemp, 
					strlen(szTemp), &dwWritten);
			}
		}

		if ((pMimeList->nextPart == NULL) && (pMimeList->parentPart != NULL)
			&& (grandparentPart != NULL))
		{
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, "--", 2, &dwWritten);
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, 
				pMimeList->parentPart->szBoundary, 
				pMimeList->parentPart->nBoundaryLength, &dwWritten);
			WriteFileOrBuffer(mgr, hFile, szOutBuffer, "--\r\n", 4, &dwWritten);
		}

		pMimeList = pMimeList->nextPart;
	}
}


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
