/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: pgpClientLibWin32.h,v 1.135 2002/11/22 03:10:38 pbj Exp $
____________________________________________________________________________*/
#ifndef Included_pgpClientLibWin32_h	/* [ */
#define Included_pgpClientLibWin32_h

#include <windows.h>
#include "pgpPubTypes.h"

#include "pflPrefTypes.h"
#include "pgpGroups.h"
#include "pgpKeyServerPrefs.h"
#include "pgpReconstruct.h"

#ifdef __cplusplus
extern "C" {
#endif

//	_______________________________________________________
//
//	Library Initialization/Cleanup
//
//	Open library and initialize

//	Entry parameters :
//		pcontext		- receives new PGPContextRef
//		uFlags			- flags for special options

enum
{
	kPGPclUseLocalSDKMode			= (1UL << 0),
	kPGPclNoPrefsFileInit			= (1UL << 1),
	kPGPclEnableNewUserWizard		= (1UL << 2),
	kPGPclForceNewUserWizard		= (1UL << 3),
};

PGPError PGPclExport
PGPclInitLibrary (
		PGPContextRef*	pcontext,
		PGPUInt32		uFlags);

//	Tell clientlib to use another prefs file.  This is needed only by
//	GINA/NPI when clientlib is initialized *before* user logon.
//	Passing in pszPath == NULL will revert to default file.

PGPError PGPclExport
PGPclChangeDefaultPrefsFile (
		char*			pszPath);

//	Close down DLL and purge passphrase buffers.

PGPError PGPclExport
PGPclCloseLibrary (void);


//	_______________________________________________________
//
//	Word wrap convenience functions
//
//	Wrap text using platform independent word wrap code.
//	Output buffer must be freed with PGPclFreeWrapBuffer
//
//	Entry parameters :
//		szInText		- input buffer
//		wrapColumn		- column at which to wrap
//		pszOutText		- buffer to receive pointer to
//						  output buffer containing wrapped text

PGPError PGPclExport
PGPclWrapBuffer (
		char*			szInText,
		PGPUInt16		wrapColumn,
		char**			pszOutText);

//	Free previously-wrapped text buffer.
//
//	Entry parameters :
//		textBuffer		- buffer to free

PGPError PGPclExport
PGPclFreeWrapBuffer (
		char* 			textBuffer);


//	_______________________________________________________
//
//	Unicode convenience functions

//  wrapper for PGPGetPrimaryUserIDName.  This wrapper gets
//	the userid name string then converts from UTF8 to local
//	code page.
//
//	Entry parameters :
//		key				- key to query
//		pszName			- buffer to receive output
//		uMaxLen			- size of output buffer
//		puLen			- receives length of string

PGPError PGPclExport
PGPclGetPrimaryUserIDName (
		PGPKeyDBObjRef	key,
		LPSTR			pszName,
		PGPUInt32		uMaxLen,
		PGPUInt32*		puLen);

//  similar to PGPclGetPrimaryUserIDName, but for getting userid
//	name string directly from the specified userid.  This function
//	get the userid name string then converts from UTF8 to local
//	code page.
//
//	Entry parameters :
//		userid			- userid to query
//		pszName			- buffer to receive output
//		uMaxLen			- size of output buffer
//		puLen			- receives length of string

PGPError PGPclExport
PGPclGetUserIDName (
		PGPKeyDBObjRef	userid,
		LPSTR			pszName,
		PGPUInt32		uMaxLen,
		PGPUInt32*		puLen);

//  wrapper for PGPGetPrimaryUserIDName.  This wrapper gets
//	the userid name string and then ensures that string is
//	in UTF-8 format
//
//	Entry parameters :
//		key				- key to query
//		pszName			- buffer to receive output
//		uMaxLen			- size of output buffer
//		puLen			- receives length of string

PGPError PGPclExport
PGPclGetPrimaryUserIDNameUTF8 (
		PGPKeyDBObjRef	key,
		LPSTR			pszName,
		PGPUInt32		uMaxLen,
		PGPUInt32*		puLen);

//  wrapper for PGPGetPrimaryUserIDName.  This wrapper gets
//	the userid name string and then ensures that string is
//	in UCS2 format
//
//	Entry parameters :
//		key				- key to query
//		pwszName		- buffer to receive output
//		uMaxLen			- size of output buffer
//		puLen			- receives length of string

PGPError PGPclExport
PGPclGetPrimaryUserIDNameUCS2(
		PGPKeyDBObjRef	key,
		LPWSTR			pwszName,
		PGPUInt32		uMaxLen,
		PGPUInt32*		puLen);


//  similar to PGPclGetPrimaryUserIDName, but for getting userid
//	name string directly from the specified userid.  This function
//	get the userid name string then ensures that string is in UTF-8
//	format.
//
//	Entry parameters :
//		userid			- userid to query
//		pszName			- buffer to receive output
//		uMaxLen			- size of output buffer
//		puLen			- receives length of string


PGPError PGPclExport
PGPclGetUserIDNameUTF8 (
		PGPKeyDBObjRef	userid,
		LPSTR			pszName,
		PGPUInt32		uMaxLen,
		PGPUInt32*		puLen);


//	_______________________________________________________
//
//	Miscellaneous common dialogs and UI elements

//	Post PGP preferences property sheets.
//
//	This function posts the "PGP Preferences" property
//	sheet dialog boxes.  Appropriate calls are made set
//	the preferences.
//
//	Entry parameters :
//		context			- PGP library context
//		hwndParent		- handle of parent window
//		prefref			- optional prefref for PGP prefs
//						  NULL => load default PGP prefs
//		prefrefNet		- optional prefref for PGPnet prefs
//						  NULL => load default PGPnet prefs
//		uFlags			- flags for setting admin options
//		uSheetsToShow	- logical OR of following constants
//						  to indicate which sheets to display
//		iStartsheet		- zero-based index of property sheet
//						  page to display initially.
//						  use the below-defined constants
//		keydbMain		- main keydb, if available.  If not
//						  available, NULL is OK.
//
//	This function returns kPGPError_UserAbort (if the user
//	presses the cancel button)

//	flags for "uFlags"
enum
{
	kPGPclEnableBroadcasts			= (1UL << 0),
	kPGPclRespectDisableFlags		= (1UL << 1),

	kPGPclDefaultAdminPrefFlags 	= 0
};

//	preferences pages (used for "panelsToShow" and "panelStart")
enum
{
	kPGPclGeneralPrefs				= (1UL << 0),
	kPGPclKeyringPrefs				= (1UL << 1),
	kPGPclEmailPrefs				= (1UL << 2),
	kPGPclHotkeyPrefs				= (1UL << 3),
	kPGPclKeyserverPrefs			= (1UL << 4),
	kPGPclCAPrefs					= (1UL << 5),
	kPGPclAdvancedPrefs				= (1UL << 6),
	kPGPclPGPdiskPrefs				= (1UL << 7),
	kPGPclPGPnetIDRPrefs			= (1UL << 8),
	kPGPclPGPnetVPNPrefs			= (1UL << 9),
	kPGPclPGPnetAuthPrefs			= (1UL << 10),
	kPGPclPGPnetAdvancedPrefs		= (1UL << 11),

	kPGPclDefaultPrefsPanel			= kPGPclGeneralPrefs,
	kPGPclNumPrefsPanels			= 12
};

PGPError PGPclExport
PGPclAdminPreferences (
		PGPContextRef	context,
		HWND			hwndParent,
		PGPPrefRef		prefref,
		PGPPrefRef		prefrefNet,
		PGPUInt32		uFlags,
		PGPUInt32		uPanelsToShow,
		PGPUInt32		uStartPanel,
		PGPKeyDBRef		keydbMain);

PGPError PGPclExport
PGPclPreferences (
		PGPContextRef	context,
		HWND			hwndParent,
		PGPUInt32		uStartPanel,
		PGPKeyDBRef		keydbMain);

//
//	Get descriptive string for error code for encode/decode modules
//
//	Entry parameters :
//		iCode		- error code returned by pgp or simple library routine
//		szString	- buffer to be filled with descriptive error message
//		uLen		- length of buffer in bytes
//
//	This function evaluates iCode.  If iCode warrants an error
//	message, the buffer is filled with the message and the function
//	returns kPGPError_UnknownError.  If no message is warranted,
//	the function returns kPGPError_NoErr.

PGPError PGPclExport
PGPclEncDecErrorToString (
		PGPInt32		iCode,
		char*			szString,
		PGPUInt32		uLen);

//
//	Putup error messagebox (if necessary) for encode/decode modules
//
//	Entry parameters :
//		hwnd		- handle of parent (NULL is OK)
//		iCode		- error code returned by pgp or simple library routine
//
//	This function calls PGPclErrorToString to evaluate iCode.
//	If PGPclErrorToString returns kPGPError_UnknownError, a message
//  box is posted containing the descriptive text.  Otherwise no
//	messagebox is displayed.

PGPError PGPclExport
PGPclEncDecErrorBox (
		HWND			hwnd,
		PGPError		errIn);

//
//	Get descriptive string for error code
//
//	Entry parameters :
//		iCode		- error code returned by pgp or simple library routine
//		szString	- buffer to be filled with descriptive error message
//		uLen		- length of buffer in bytes
//		puCaptionID	- string ID of caption string
//
//	This function evaluates iCode.  If iCode warrants an error
//	message, the buffer is filled with the message and the function
//	returns kPGPError_UnknownError.  If no message is warranted,
//	the function returns kPGPError_NoErr.

enum
{
	kPGPclErrorType_Error			= 1,
	kPGPclErrorType_Warning			= 2,
	kPGPclErrorType_Alert			= 3,
	kPGPclErrorType_Information		= 4
};

PGPError PGPclExport
PGPclErrorToString (
		PGPInt32		iCode,
		char*			szString,
		PGPUInt32		uLen,
		PGPUInt32*		puCaptionType);

//
//	Putup error messagebox (if necessary)
//
//	Entry parameters :
//		hwnd		- handle of parent (NULL is OK)
//		iCode		- error code returned by pgp or simple library routine
//
//	This function calls PGPclErrorToString to evaluate iCode.
//	If PGPclErrorToString returns kPGPError_UnknownError, a message
//  box is posted containing the descriptive text.  Otherwise no
//	messagebox is displayed.  This function always returns the error
//	code which the caller passes in.

PGPError PGPclExport
PGPclErrorBox (
		HWND			hwnd,
		PGPError		errIn);
		
//
//	Convert UTF8 strings to UCS2 and put up MessageBoxW
//
//	Entry parameters :
//		hwnd			- handle of parent window
//		pszTextUTF8		- main text of messagebox
//		pszCaptionUTF8	- caption string for title bar
//		uType			- flags as used by MessageBox
//
//	This function returns kPGPError_UserAbort (if the user
//  presses the cancel button).

PGPUInt32 PGPclExport
PGPclMessageBoxUTF8 (
		HWND			hwnd,
		char*			pszTextUTF8,
		char*			pszCaptionUTF8,
		UINT			uType);

//
//	Put up dialog to collect entropy from keyboard/mouse.
//
//	Entry parameters :
//		context		- PGP library context
//		hwndParent	- handle of parent window
//		uNeeded		- bits of entropy needed
//
//	This function returns kPGPError_UserAbort (if the user
//  presses the cancel button).

PGPError PGPclExport
PGPclRandom (
		PGPContextRef	context,
		HWND			hwndParent,
		PGPUInt32		uNeeded);

//
//	Post Help|About dialog with button for browser launch to specified URL.
//
//	Entry parameters :
//		context		- current PGP context
//		hwndParent	- handle of parent window
//		szVersion	- string containing version information
//					  to be displayed
//					  NULL => use default version string
//		szLinkText	- text to be displayed on link button
//					  (should be <= ~20 chars),
//					  NULL => use default ("www.pgp.com")
//		szLink		- URL to which to jump
//					  NULL => use default ("http://www.pgp.com/")

PGPError PGPclExport
PGPclHelpAbout (
		PGPContextRef	context,
		HWND			hwndParent,
		char*			szVersion,
		char*			szLinkText,
		char*			szLink);

//
//	Post License Number Dialog
//
//	Entry parameters :
//		context		- current PGP context
//		hwndParent	- handle of parent window

PGPError PGPclExport 
PGPclEnterLicense(
		PGPContextRef	context,
		HWND			hwndParent);

//
//	Post certificate retrieval congratulations dialog
//
//	Entry parameters :
//		hwndParent		- handle of parent window
//		uCongratsType	- type of message text to display
//		pszUserID		- cert userID

enum
{
	kPGPclManualRetrieval				= 0,
	kPGPclAutoRetrievalUsingCert		= 1,
	kPGPclAutoRetrievalQueryUse			= 2
};

PGPError PGPclExport
PGPclCertCongrats (
		HWND			hwndParent,
		PGPUInt32		uCongratsType,
		char*			pszUserID);

//
//	display and handle selective key import dialog - new version
//
//	Entry parameters :
//		hwndParent		- handle of parent window
//		keysetToAdd		- keyset containing keys to add
//		keydbMain		- keydb to which keys will be added
//		uFlags			- logical OR of flags (below)
//
//	Returns kPGPError_NoErr if no error
//

enum
{
	kPGPclNoImportDialog				= (1UL << 0),
	kPGPclNoDialogForValidSingletons	= (1UL << 1),
	kPGPclNoTrustWarningMessage			= (1UL << 2),

	kPGPclDefaultImportFlags 			= 0
};

PGPError PGPclExport
PGPclImportKeys (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeySetRef		keysetToAdd,
		PGPKeyDBRef			keydbMain,
		PGPUInt32			uFlags);

//
//	display keyset and allow user to select keys
//	- original calling parameters
//
//	Entry parameters :
//		hwndParent		- handle of parent window
//		pszPrompt		- prompt string
//		keysetToDisplay	- keyset containing keys to show
//		pkeydbSelected	- buffer to receive keydb ref of selected keys
//
//
//	Returns kPGPError_NoErr if no error

// option flag bits
enum
{
	kPGPclSingleSelection			= (1UL << 0),

	kPGPclDefaultSelectionFlags 	= 0
};

PGPError PGPclExport
PGPclSelectKeys (
		PGPContextRef		context,
		PGPtlsContextRef	tlsContext,
		HWND				hwndParent,
		char*				pszPrompt,
		PGPKeySetRef		keysetToDisplay,
		PGPUInt32			uFlags,
		PGPKeySetRef*		pkeysetSelected);

//
//	display X.509 certs and allow user to select cert
//
//	Entry parameters :
//		hwndParent		- handle of parent window
//		pszPrompt		- prompt string
//		keydbMain		- backing keydb
//		uFlags			- option flags (see below)
//		pkeyCert		- buffer to receive cert keyref
//		psigCert		- buffer to receive cert sigref
//
//
//	Returns kPGPError_NoErr if no error
//

// option flag bits
enum
{
	kPGPclSelectX509CanSignOnly			= (1UL << 0),
	kPGPclSelectX509NoSplitKeys			= (1UL << 1),
	kPGPclSelectX509CACertsOnly			= (1UL << 2),

	kPGPclDefaultX509SelectionFlags 	= 0
};

PGPError PGPclExport
PGPclSelectX509Cert (
		PGPContextRef		context,
		HWND				hwndParent,
		char*				pszPrompt,
		PGPKeySetRef		keysetToDisplay,
		PGPUInt32			uFlags,
		PGPKeyDBObjRef*		psigRef);

//
//	Post RSA/DSA mix warning dialog
//
//	Entry parameters :
//		hwndParent	- handle of parent window
//
//	Returns kPGPError_NoErr, kPGPError_UserAbort, or PGPCL_NO
//

PGPError PGPclExport
PGPclRSADSAMixWarning (
		HWND 				hwnd,
		PGPBoolean 			*pbNeverShowAgain);

//	Display timeout (eval or beta) message to user
//
//	Entry parameters :
//		hwnd		Parent of window so we can show the MB or nag screen.
//					NULL is acceptable.
//		bBeta		TRUE if this is a beta timeout, FALSE otherwise
//		iGraceDays	number of days left in grace period; 0 means no grace
//		bOperation	TRUE if "operation" (vs. "module") timeout

void PGPclExport 
PGPclDisplayTimeoutMessage (
		HWND				hwnd,
		BOOL				bBeta,
		INT					iGraceDays,
		BOOL				bOperation);

//	Wrapper around PGPlnPGPmailEnabled for use by Notes plugin

PGPBoolean PGPclExport
PGPclPGPmailEnabled (void);

//	Wrapper around PGPlnDesktop for use by Notes plugin

PGPBoolean PGPclExport
PGPclPGPdesktopEnabled (void);

//	Wrapper around PGPlnEnterprise for use by Notes plugin

PGPError PGPclExport
PGPclEnterprise (void);

//	Wrapper around PGPlnIsAuthorized for use by Notes plugin

PGPError PGPclExport
PGPclIsAuthorized (void);

//	Wrappers around PGPlnIsExpired for use by Notes plugin

typedef enum PGPclExpType
{
	kPGPclDefaultExpired				= 1,
	kPGPclEncryptSignExpired			= 2
} PGPclExpType;

PGPError PGPclExport
PGPclIsExpired (
		HWND				hwnd,
		PGPclExpType		type);

PGPError PGPclExport
PGPclEvalExpired (
		HWND				hwnd,
		PGPclExpType		type);

//	_______________________________________________________
//
//	Group files convenience functions

typedef struct _PGPclGroupFile
{
	PGPFileSpecRef			filespec;
	PGPGroupSetRef			groupset;
} PGPclGroupFile;

//
//	Opens standard groups file
//
//	Entry parameters :
//		ppGroup			address of buffer to receive pointer
//						to group file structure

PGPError PGPclExport
PGPclOpenGroupFile (
		PGPContextRef		context,
		PGPclGroupFile**	ppGroup);

//
//	Saves groups file
//
//	Entry parameters :
//		pGroup			pointer to group file structure

PGPError PGPclExport
PGPclSaveGroupFile (
		PGPclGroupFile* 	pGroup);

//
//	Closes groups file
//
//	Entry parameters :
//		pGroup			pointer to group file structure

PGPError PGPclExport
PGPclCloseGroupFile (
		PGPclGroupFile* 	pGroup);


//	_______________________________________________________
//
//	Preference files convenience functions

//
//	Query if this is a "Client" installation

PGPBoolean PGPclExport
PGPclIsClientInstall (void);

//
//	Query if this is a "Admin" installation

PGPBoolean PGPclExport
PGPclIsAdminInstall (void);

//
//	Find default key in specified keydb by looking it up via the
//  keyid stored in the client prefs
//
//	Entry parameters :
//		keydb	- keydb containing default key
//		pkeyOut	- received key obj ref of default key

PGPError PGPclExport
PGPclGetDefaultKey (
		PGPKeyDBRef			keydb,
		PGPKeyDBObjRef*		pkeyOut);
		
//
//	Find default private key in specified keydb by looking it up 
//	via the keyid stored in the client prefs.  This differs from
//	PGPclGetDefaultKey in that it only returns keypairs.
//
//	Entry parameters :
//		keydb	- keydb containing default key
//		pkeyOut	- received key obj ref of default key

PGPError PGPclExport
PGPclGetDefaultPrivateKey (
		PGPKeyDBRef			keydb,
		PGPKeyDBObjRef*		pkeyOut);
		
//
//	Get the prefref(s) for the currently open PGPclientLib prefs.
//	Pass a NULL for those prefrefs you don't want
//
//	Entry parameters :
//		pprefref	- points to prefref to receive PGP prefref
//		pprefrefNet	- points to prefref to receive PGPnet prefref

PGPError PGPclExport
PGPclPeekClientLibPrefRefs (
		PGPPrefRef*			pprefref,
		PGPPrefRef*			pprefrefNet);

//
//	Flush the specified prefs to disk or to service.
//	Pass a kInvalidPGPPrefRef for those prefrefs you don't want to flush.
//
//	Entry parameters :
//		prefref		- prefref for PGP prefs
//		prefrefNet	- prefref for PGPnet prefs

PGPError PGPclExport
PGPclFlushClientLibPrefs (
		PGPPrefRef			prefref,
		PGPPrefRef			prefrefNet);

//
//  Import the default keys (DefaultKeys.asc) into the user's keyring
//

PGPError PGPclExport
PGPclImportDefaultKeys (
		PGPContextRef	context,
		HWND			hwnd);

//	______________________________________________
//
//  The following functions are wrappers for
//  the pfl prefs routines. These are needed so
//  groupwise plugin can access prefs (Delphi)
//  using code in the PGPcl DLL.

PGPError PGPclExport
PGPclGetPrefFlags(
		PGPPrefRef			prefref,
		PGPPrefIndex		prefIndex,
		PGPUInt32*			flags);

PGPError PGPclExport
PGPclGetPrefArray(
		PGPPrefRef			prefref,
		PGPPrefIndex		prefIndex,
		PGPPrefArray**		array);

PGPError PGPclExport
PGPclSetPrefArray(
		PGPPrefRef			prefref,
		PGPPrefIndex		prefIndex,
		PGPPrefArray*		array);

PGPError PGPclExport
PGPclGetPrefBoolean (
		PGPPrefRef 			prefref,
		PGPPrefIndex 		prefIndex,
		PGPBoolean 			*data);

PGPError PGPclExport
PGPclSetPrefBoolean (
		PGPPrefRef 			prefref,
		PGPPrefIndex 		prefIndex,
		PGPBoolean 			data);

PGPError PGPclExport
PGPclGetPrefNumber (
		PGPPrefRef 			prefref,
		PGPPrefIndex 		prefIndex,
		PGPUInt32 			*data);

PGPError PGPclExport
PGPclSetPrefNumber (
		PGPPrefRef 			prefref,
		PGPPrefIndex 		prefIndex,
		PGPUInt32 			data);

PGPError PGPclExport
PGPclGetPrefStringAlloc (
		PGPPrefRef 			prefref,
		PGPPrefIndex 		prefIndex,
		char**				string);

PGPError PGPclExport
PGPclGetPrefStringBuffer (
		PGPPrefRef 			prefref,
		PGPPrefIndex 		prefIndex,
		PGPSize 			maxSize,
		char*				string);

PGPError PGPclExport
PGPclSetPrefString (
		PGPPrefRef 			prefref,
		PGPPrefIndex 		prefIndex,
		const char*			string);

PGPError PGPclExport
PGPclGetPrefData (
		PGPPrefRef 			prefref,
		PGPPrefIndex 		prefIndex,
		PGPSize*			dataLength,
		void**				inBuffer);

PGPError PGPclExport
PGPclSetPrefData (
		PGPPrefRef 			prefref,
		PGPPrefIndex 		prefIndex,
		PGPSize 			dataLength,
		const void*			outBuffer);

PGPError PGPclExport
PGPclRemovePref (
		PGPPrefRef 			prefref,
		PGPPrefIndex 		prefIndex);

//
//	Get root CA key and sig that are specified by client prefs
//
//	Entry parameters :
//		prefref			- valid prefref for client prefs
//		keydb			- main keydb on which root CA key is located
//		psigCert		- pointer to buffer to receive sig

PGPError PGPclExport
PGPclGetRootCACertPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PGPKeyDBRef			keydb,
		PGPKeyDBObjRef*		psigCert);


//	_______________________________________________________
//
//	IE token support functions

// Application-independent IE wrappers, based on CryptoAPI
int PGPclExport
PGPclIEExportX509 ( 
		unsigned char*		pCertEncoded, 
		unsigned 			cbCertEncoded, 
		const char *		userID, 
		const char *		keyIDstr, 
		const char *		key_container, 
		unsigned 			key_container_size, 
		const char *		CSP);
    
int PGPclExport
PGPclIETokenProvToCSP (
		const char*			tokenProv, 
		char 				CSP[80], 
		int 				size);

int PGPclExport PGPclIEInit(const char *cmdLine);

int PGPclExport PGPclIEGetExistingKeyContainersRemote(
		const char *CSP, char **cont, int *n);

void PGPclExport PGPclIEFreeMem( void *cont );

// PGP-specific IE import routine
PGPError PGPclExport
PGPclProcessKeyConts( 
		PGPContextRef ctx, PGPKeyDBObjRef key, 
		const char *pszPassPhrase,
		const char *CSP, const PGPTokenInfo *pti, 
		const char *conts1, int conts1_n, 
		const char *conts2, int conts2_n );


//	_______________________________________________________
//
//	Exchange server support functions

PGPBoolean PGPclExport
PGPclIsExchangeServerUser (void);

PGPError PGPclExport
PGPclGetExchangeServerAddress (
		HWND		hwnd,
		PGPBoolean	bDefaultLogon,
		char*		pszName,
		PGPSize		sizeName,
		char*		pszEmailAddress, 
		PGPSize		sizeEmailAddress);

//	_______________________________________________________
//
//	Miscellaneous convenience functions

//
//	Determines which windows OS we are running under
//	Note: either argument can be NULL if information is not needed

typedef enum PGPclWindowsFamily
{
	kPGPclWindowsFamilyUnknown			= 0,
	kPGPclWindowsFamily9x				= 1,	//includes Millenium
	kPGPclWindowsFamilyNT				= 2		//includes Windows2000, XP
} PGPclWindowsFamily;

typedef enum PGPclWindowsGenus
{
	kPGPclWindowsGenusUnknown			= 0,
	kPGPclWindowsGenus95				= 1,
	kPGPclWindowsGenus98				= 2,
	kPGPclWindowsGenusMillenium			= 3,
	kPGPclWindowsGenusPostMillenium		= 4,
	kPGPclWindowsGenusNT3				= 101,
	kPGPclWindowsGenusNT4				= 102,
	kPGPclWindowsGenus2000				= 103,
	kPGPclWindowsGenusXp				= 104,
	kPGPclWindowsGenusPostKnown			= 901
} PGPclWindowsGenus;

typedef enum PGPclWindowsSpecies
{
	kPGPclWindowsSpeciesUnknown			= 0,
	kPGPclWindowsSpecies95OSR1			= 1,
	kPGPclWindowsSpecies95OSR2			= 2,
	kPGPclWindowsSpecies98				= 3,
	kPGPclWindowsSpecies98SE			= 4,
	kPGPclWindowsSpeciesMillenium		= 5,
	kPGPclWindowsSpeciesPostMillenium	= 6,
	kPGPclWindowsSpeciesNT3				= 101,
	kPGPclWindowsSpeciesNT4NoSp			= 102,
	kPGPclWindowsSpeciesNT4Sp1			= 103,
	kPGPclWindowsSpeciesNT4Sp2			= 104,
	kPGPclWindowsSpeciesNT4Sp3			= 105,
	kPGPclWindowsSpeciesNT4Sp4			= 106,
	kPGPclWindowsSpeciesNT4Sp5			= 107,
	kPGPclWindowsSpeciesNT4Sp6			= 108,
	kPGPclWindowsSpeciesNT4PostSp6		= 109,
	kPGPclWindowsSpecies2000Beta		= 201,
	kPGPclWindowsSpecies2000			= 202,
	kPGPclWindowsSpecies2000Sp1			= 203,
	kPGPclWindowsSpecies2000Sp2			= 204,
	kPGPclWindowsSpecies2000Sp3			= 205,
	kPGPclWindowsSpeciesXpBeta			= 301,
	kPGPclWindowsSpeciesXp				= 302,
	kPGPclWindowsSpeciesXpSp1			= 303,
	kPGPclWindowsSpeciesPostKnown		= 901
} PGPclWindowsSpecies;

PGPError PGPclExport
PGPclGetWindowsVersion (
		PGPclWindowsFamily*		pFamily,
		PGPclWindowsGenus*		pGenus,
		PGPclWindowsSpecies*	pSpecies);

//
//	Get our process's Terminal Services session ID

PGPError PGPclExport
PGPclGetTerminalServicesSessionId (
		PGPUInt32*			puSessionId);

//
//	Tests for whether we are running as a terminal server client

PGPBoolean PGPclExport
PGPclRunningUnderTerminalServer (void);

//
//	Tests for various conditions of interest on specified
//	file or directory path.  Return value is logical OR of following
//	flags.  Suppresses the display of system messageboxes

enum
{
	kPGPclInvalidPath				= (1UL << 0),
	kPGPclMediaNotInserted			= (1UL << 1),
	kPGPclIsFile					= (1UL << 2),
	kPGPclIsFolder					= (1UL << 3),
	kPGPclIsReadOnly				= (1UL << 4),
	kPGPclIsInReadOnlyFolder		= (1UL << 5),
};

PGPUInt32 PGPclExport
PGPclTestPathNameStatus (
		char*				pszPath);

//
// wrapper for HtmlHelp call.  does some processing based on
// message or help type.  if used in response to WM_HELP or
// WM_CONTEXTMENU messages, just pass the msg, and parameters
// directly into PGpclHtmlHelp.  if you pass one of the following
// enums for uMsgOrHelpType, the wParam and lParam arguments
// are ignored.

//	Entry parameters :
//		hwnd			- specifies the parent window
//		wMsgOrHelpType	- either one of the following enums, or
//						  WM_HELP, or WM_CONTEXTMENU
//		wParam			- the wParam of WM_HELP or WM_CONTEXTMENU
//		lParam			- the lParam of WM_HELP or WM_CONTEXTMENU
//		pszHelpFile		- the help file path, or one of the PGPclPathIndex
//						  enums (used with PGPclGetPath) specifying a help
//						  file path
//		pdwID			- for WM_HELP and WM_CONTEXTMENU, this should be a
//						  pointer to the help ID array.  Otherwise it should
//						  be the address of a DWORD containing the help ID.
//

enum
{
	kPGPclHelpContentsAndIndex		= 0,
	kPGPclHelpContext				= 1,
	kPGPclHelpMoreInformation		= 2
};

BOOL PGPclExport
PGPclHtmlHelp (
		HWND				hwnd,
		UINT				uMsgOrHelpType,
		WPARAM				wParam,
		LPARAM				lParam,
		char*				pszHelpFile,
		DWORD*				pdwID);

//
//	Determines if we are running with Admin priviledges
//	Note: returns TRUE if not running under NT

PGPBoolean PGPclExport
PGPclLoggedInAsAdministrator (void);

//
//	Get a valid IP/subnet mask for the local system.
//	Note: if there are multiple IPs, then the first is returned

PGPError PGPclExport
PGPclGetIPandSubnetMask (
		PGPUInt32*	puIP,
		PGPUInt32*	puSubnetMask);

//
//	Returns TRUE if local system is connected to the internet

PGPBoolean PGPclExport
PGPclIsLocalMachineOnline (void);

//
//	Get the IP address of a DNS server for the local machine

PGPError PGPclExport
PGPclGetDNSServerAddress (
		PGPUInt32*	puDNSAddress);

//
//	Plays the specified sound.  Note the user can change the sound
//  file associated with each event by using the control panel applet
//
//	Entry parameters :
//		index			- specifies which event to play sound for
//		bSynchrounous	- TRUE => wait for sound to end before returning
//

typedef enum PGPclSoundIndex
{
	kPGPclNoSound					= 0,
	kPGPclFirewallIntruderAlert		= 1,
} PGPclSoundIndex;

PGPError PGPclExport
PGPclPlaySound (
		PGPclSoundIndex		index,
		PGPBoolean			bSynchronous);

//
//	Find out if component is installed (uses the InstalledComponents
//  registry key as setup by the installer)
//
//	Entry parameters :
//		index		- specifies which module to report
//

typedef enum PGPclComponentIndex
{
	kPGPclUsersManual				= 0,
	kPGPclPGPkeys					= 1,
	kPGPclPFIDS						= 2,
	kPGPclPGPdisk					= 3,
	kPGPclCommandLine				= 4,
	kPGPclICQplugin					= 5,
	kPGPclEudoraPlugin				= 6,
	kPGPclExchangePlugin			= 7,
	kPGPclOutlookExpressPlugin		= 8,
	kPGPclLotusNotesPlugin			= 9,
	kPGPclPGPlogin					= 10,
	kPGPclPGPtools					= 11,
	kPGPclVPN						= 12,
	kPGPclPGPnet					= 13,	// VPN or PF/IDS
	kPGPclGroupwisePlugin			= 14,
	kPGPclPGPmailPlugin				= 15,	// OR of all plugins
	kPGPclPGPadmin					= 16,
	kPGPclPGPPersonal				= 17,
	kPGPclPGPDesktop				= 18,
	kPGPclPGPEnterprise				= 19
} PGPclComponentIndex;

PGPBoolean PGPclExport
PGPclIsComponentInstalled (
		PGPclComponentIndex	index);

PGPError PGPclExport 
PGPclNoLicense(
		HWND				hwndParent,
		PGPclComponentIndex	index);

PGPError PGPclExport 
PGPclDisplayLicense(
		PGPContextRef	context,
		HWND			hwndParent);

//
//	Get the specified file or folder path
//
//	Entry parameters :
//		index		- specifies which path to retrieve
//		pszPath		- buffer to receive path string
//		uLen		- length of buffer in bytes
//
//  Note: returned folder paths include trailing '\'

typedef enum PGPclPathIndex
{
	kPGPclRegistryKey				= 0,   // registry subkey name
	kPGPclInstallFolder				= 1,
	kPGPclHelpFile					= 2,
	kPGPclPGPkeysExeFile			= 3,
	kPGPclPrefsFile					= 4,
	kPGPclGroupsFile				= 5,
	kPGPclNetPrefsFile				= 6,
	kPGPclNetLogFile				= 7,
	kPGPclDefaultRandseedFolder		= 8,
	kPGPclDefaultKeyringFolder		= 9,
	kPGPclDefaultOpenFolder			= 10,
	kPGPclDefaultSaveAsFolder		= 11,
	kPGPclPublicKeyringFile			= 12,
	kPGPclPrivateKeyringFile		= 13,
	kPGPclNetPublicKeyringFile		= 14,
	kPGPclNetPrivateKeyringFile		= 15,
	kPGPclRandseedFile				= 16,
	kPGPclNetHelpFile				= 17,
	kPGPclDesktopFolder				= 18,
	kPGPclDiskHelpFile				= 19,
	kPGPclAdminHelpFile				= 20,
	kPGPclDefaultKeysFile			= 21,
	kPGPclNetLogEventDataFolder		= 22,
	kPGPclFireHelpFile				= 23,
	kPGPclMailHelpFile				= 24,
	kPGPclNumPathIndices			= 25
} PGPclPathIndex;

PGPError PGPclExport
PGPclGetPath (
		PGPclPathIndex		index,
		char*				pszPath,
		PGPInt32			iLen);

//
//	Set the specified file path
//
//	Entry parameters :
//		prefref		- must be the correct (PGP vs. PGPnet) prefref
//		index		- specifies which file path to set
//		pszPath		- buffer containing path string
//
//	Notes: 	- only the keyring and randseed file paths can be
//		      set.  Others will return kPGPError_BadParams.
//			- caller should pass in PGP or PGPnet prefref as appropriate
//			- caller must call PGPclFlushPrefs if using clientlib prefref

PGPError PGPclExport
PGPclSetPath (
		PGPPrefRef			prefref,
		PGPclPathIndex		index,
		char*				pszPath);

//
//	Load up the specified SHFOLDER.DLL and use it to create the
//	special folders we need.

PGPError PGPclExport
PGPclInstallerCreateFolders (
		char*				pszSHFolderPath);

//
//	Set the keyring file paths (called only by the installer)
//
//	Entry parameters :
//		pszPubRing		- buffer containing path string
//		pszSecRing		- buffer containing path string
//		pszNetPubRing	- buffer containing path string (can be NULL)
//		pszNetSecRing	- buffer containing path string (can be NULL)
//		pszRNGSeedFile	- buffer containing path string (can be NULL)
//		pszDefaultKeys	- buffer containing path string (can be NULL)
//
//	Notes:	- pass in NULL for pszNetPubRing and pszNetSecRing if
//			  PGPnet is not installed
//			- if you pass in NULL for pszRNGSeedFile, a default name
//			  will be used
//			- if you pass in NULL for pszDefaultKeys, no attempt to
//			  import the default keys to the keyring will be made
//

PGPError PGPclExport
PGPclInstallerSetPaths (
		char*				pszPubRing,
		char*				pszSecRing,
		char*				pszNetPubRing,
		char*				pszNetSecRing,
		char*				pszRNGSeedFile,
		char*				pszDefaultKeysFile);

//
//	keygen wizard wrapper convenience function (called only by the installer)
//
//	Entry parameters :
//		bCheckForKeyPairs	- FALSE => always run wizard
//							- TRUE => only run if no keypairs on keyring
//

PGPError PGPclExport
PGPclRebootKeyGeneration (
		HWND				hwndParent,
		BOOL				bCheckForKeyPairs);

// Called only from installer -wjb
PGPError PGPclExport
PGPclInstallerKeyGeneration (
		HWND				hwndParent,
		BOOL				bCheckForKeyPairs);

//
//	Drop-in replacement for WinExec
//
//	Entry parameters :
//		pszPath			- file to run
//		uCmdShow		- flags (see Docs for ShellExecute)
//
//	return TRUE if no successful

PGPBoolean PGPclExport
PGPclExecute (
		char* 				pszPath,
		PGPUInt32			uCmdShow);

//
//	Launch web browser and send to specified page
//
//	Entry parameters :
//		pszURL			- URL to open
//

PGPError PGPclExport
PGPclWebBrowse (
		char* 				pszURL);

//
//	Convert info in SYSTEMTIME structure to number of days from today
//
//	Entry parameters :
//		pst			address of SYSTEMTIME structure containing data
//					to convert
//		piDays		address of buffer to receive number of days from
//					today.  Negative numbers are prior to today.

PGPError PGPclExport
PGPclSystemTimeToDays (
		SYSTEMTIME*			pst,
		PGPInt32*			piDays);

//
//	Check whether SDK supports specified Public Key algorithm
//
//	Entry parameters :
//		alg				SDK public key algorithm constant
//		bMustEncrypt	TRUE=>SDK must support encryption with
//									this algorithm
//		bMustSign		TRUE=>SDK must support signing with
//									this algorithm
//
//	returns kPGPError_NoErr if SDK supports operations with
//	specified algorithm, kPGPError_FeatureNotAvailable otherwise.

PGPError PGPclExport
PGPclCheckSDKSupportForPKAlg (
		PGPPublicKeyAlgorithm	alg,
		PGPBoolean				bMustEncrypt,
		PGPBoolean				bMustSign);

//
//	Check whether SDK supports specified cipher algorithm
//
//	Entry parameters :
//		PGPCipherAlgorithm		SDK cipher algorithm constant
//
//	returns kPGPError_NoErr if SDK supports operations with
//	specified algorithm, kPGPError_FeatureNotAvailable otherwise.

PGPError PGPclExport
PGPclCheckSDKSupportForCipherAlg (
		PGPCipherAlgorithm 		alg);

//
//	Broadcast messages that indicates that the various things
//	have been changed and that others should reload from
//	disk.
//
//	Entry parameters :
//		lParam		- 32 value which is passed along as the LPARAM
//					  of the broadcast message.  Current usage
//					  is to set this to your process ID or your
//					  window handle so that you can ignore
//					  your own messages, if you want.  Set to
//					  zero to ensure all recipients process message.

//	broadcast message used to inform others of keyring changes
#define kPGPclReloadKeyRingMsg			("PGP Keyrings Reload")
#define kPGPclReloadPrefsMsg			("PGP Prefs Reload")
#define kPGPclReloadNetPrefsMsg			("PGP Net Prefs Reload")
#define kPGPclReloadKeyserverPrefsMsg	("PGP Keyserver Prefs Reload")

typedef enum PGPclChangeType
{
	kPGPclUnspecifiedChanges		= 0,
	kPGPclNewDefaultKeyrings		= 1,
	kPGPclKeyringContentsModified	= 2,
	kPGPclGroupsFileModified		= 3
} PGPclChangeType;

void PGPclExport
PGPclNotifyKeyringChanges (
		PGPclChangeType		type,
		PGPUInt32 			uParam);

void PGPclExport
PGPclNotifyPrefsChanges (
		PGPUInt32 			uParam);

void PGPclExport
PGPclNotifyNetPrefsChanges (
		PGPUInt32 			uParam);

void PGPclExport
PGPclNotifyKeyserverPrefsChanges (
		PGPUInt32 			uParam);

//
//	Get the ImageList index of the icon for this object
//
//	Entry parameters :
//			key			key to determine icon for
//			pbItalics	receives TRUE if associated text should be italics

PGPInt32 PGPclExport
PGPclDetermineKeyIcon (
		PGPKeyDBObjRef		key,
		PGPBoolean*			pbItalics);

PGPInt32 PGPclExport
PGPclDetermineUserIDIcon (
		PGPKeyDBObjRef		userid,
		PGPBoolean*			pbItalics);

PGPInt32 PGPclExport
PGPclDetermineCertIcon (
		PGPKeyDBObjRef		cert,
		PGPBoolean*			pbItalics,
		PGPBoolean*			pbX509);

//
//	convert the SDK representation of a fingerprint to a text representation
//	suitable for displaying in the GUI.

PGPError PGPclExport
PGPclConvertFingerprintToString (
		PGPUInt32			uKeyVersion,
		char*				sz);

//
//	open the specified keyring files, doing some extra error checking for
//	readonly conditions, floppy not inserted, etc.

PGPError PGPclExport
PGPclOpenKeyrings (
		PGPContextRef				context,
		char*						pszPubPath,
		char*						pszPrivPath,
		PGPOpenKeyDBFileOptions		options,
		PGPKeyDBRef*				pkeydbOut);

//	_______________________________________________________
//
//	Multimonitor convenience functions

typedef enum PGPclMonitorID
{
	kPGPclPrimaryMonitor				= 0,
	kPGPclWindowMonitor					= 1,
} PGPclMonitorID;

//
//	Get bounding rect of specified monitor
//
//	Entry parameters :
//		hwnd		- reference window
//					  (can be NULL, if uMonitor == PGPCL_PRIMARYMONITOR)
//		monitor	- one of above values specifying monitor
//		prcMonitor	- pointer to RECT to receive coordinates

PGPError PGPclExport
PGPclGetMonitorRect (
		HWND				hwnd,
		PGPclMonitorID		monitor,
		RECT*				prcMonitor);

//
//	Center window on multimonitor system
//
//	Entry parameters :
//		hwnd		- window to center
//		uMonitor	- one of above values specifying monitor to center on

PGPError PGPclExport
PGPclCenterWindowOnMonitor (
		HWND				hwnd,
		PGPUInt32			uMonitor);

//
//	Make sure that window position is on the same monitor as parent.
//	If parent window is NULL, then just make sure the window is visible.
//
//	Entry parameters :
//		hwndToMove		- window to check and move (if necessary)
//		hwndParent		- parent window or NULL

PGPError PGPclExport
PGPclEnsureWindowIsOnMonitor (
		HWND			hwndToMove,
		HWND			hwndParent);

//	_______________________________________________________
//
//	JPEG/PhotoID convenience functions

//	Get the size and dimensions of a bitmap from a DIB structure
//
//	Entry parameters :
//		lpbi		- pointer to DIB structure
//		piWidth		- pointer to int to receive bitmap width
//		piHeight	- pointer to int to receive bitmap height
//		piDIBSize	- pointer to int to receive buffer length

PGPError PGPclExport
PGPclGetDIBSize (
		LPBITMAPINFO	lpbi,
		PGPInt32*		piWidth,
		PGPInt32*		piHeight,
		PGPInt32*		piDIBSize);

//
//	construct a DDB from a DIB
//
//	Entry parameters :
//		lpbi		- pointer to DIB structure
//		phbitmap	- pointer to buffer to receive handle of DDB
//		phpalette	- pointer to buffer to receive handle of palette

PGPError PGPclExport
PGPclDDBfromDIB (
		LPBITMAPINFO	lpbi,
		HBITMAP*		phbitmap,
		HPALETTE*		phpalette);

//
//	construct a DIB from a PGP photoID buffer
//
//	Entry parameters :
//		pBuf		- buffer containing PGP photoID data
//		iLen		- length of photoID buffer in bytes
//		bForDisplay - TRUE => the DIB is for blitting to the screen
//		plpbi		- pointer to buffer to receive DIB structure

PGPError PGPclExport
PGPclDIBfromPhoto (
		PGPContextRef	context,
		PGPByte*		pBuf,
		PGPInt32		iLen,
		PGPBoolean		bForDisplay,
		LPBITMAPINFO*	plpbmi);

//
//	convert a PGP photoID to DIB and copy to the clipboard
//
//	Entry parameters :
//		hwnd		- HWND that will own the clipboard
//		pBuf		- buffer containing PGP photoID data
//		iLen		- length of buffer in bytes

PGPError PGPclExport
PGPclCopyPhotoToClipboard (
		PGPContextRef	context,
		HWND			hwnd,
		PGPByte*		pBuf,
		PGPInt32		iLen);

//
//	convert DIB data in clipboard to a PGP photoID
//
//	Entry parameters :
//		hwnd		- HWND that will own the clipboard
//		ppBuf		- pointer to pointer which will receive address of buffer
//		piLen		- pointer to int which will receive length of buffer

PGPError PGPclExport
PGPclPastePhotoFromClipboard (
		PGPContextRef	context,
		HWND			hwnd,
		PGPByte**		ppBuf,
		PGPInt32*		piLen);

//
//	convert DIB data in file to a PGP photoID
//
//	Entry parameters :
//		pszFileName	- full path of file to read
//		ppBuf		- pointer to pointer which will receive address of buffer
//		piLen		- pointer to int which will receive length of buffer

PGPError PGPclExport
PGPclReadPhotoFromFile (
		PGPContextRef	context,
		char*			pszFileName,
		PGPByte**		ppBuf,
		PGPInt32*		piLen);


//	_______________________________________________________
//
//	Keyserver convenience functions

typedef enum PGPclServerType
{
	kPGPclSpecifiedServer			= 0,
	kPGPclUserIDBasedServer			= 1,
	kPGPclDefaultServer				= 2,
	kPGPclRootServer				= 3,
	kPGPclAllServers				= 4
} PGPclServerType;

//
//	Searches keyserver prefs for specifed keyserver and replaces
//	authentication key.
//
//	Entry parameters :
//		keyserver					keyserver data to put in prefs file

PGPError PGPclExport
PGPclSyncKeyserverPrefs (
		PGPContextRef		context,
		PGPKeyServerEntry*	keyserver);

//
//	Searches keyservers (as defined in prefs file) for all
//	keys in the specified keyset.
//
//	Entry parameters :
//		hwndParent			parent window
//		keysetToUpdate		keyset with keys to update
//		uServer				one of above-defined server constants
//		keydbMain			keydb to which tls key is added
//		pkeydbUpdated		buffer to receive PGPKeyDBRef of updated keys

PGPError PGPclExport
PGPclUpdateKeySetFromServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeySetRef		keysetToUpdate,
		PGPclServerType		servertype,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBRef*		pkeydbUpdated);

//
//	Searches keyservers (as defined in prefs file) for the
//	specified userid string.
//
//	Entry parameters :
//		hwndParent			parent window
//		szUserID			string containing userid info
//		uServer				one of above-defined server constants
//		keydbMain			keydb to which tls key is added
//		pkeydbFound			buffer to receive PGPKeyDBRef of found keys

PGPError PGPclExport
PGPclSearchServerForUserID (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		char*				szUserID,
		PGPclServerType		servertype,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBRef*		pkeydbFound);

//
//	Searches default keyserver for the keyids in the list.
//
//	Entry parameters :
//		hwndParent			parent window
//		pkeyidList			array of PGPKeyID
//		iNumKeyIDs			number of PGPKeyIDs in list
//		uServer				one of above-defined server constants
//		keydbMain			keydb to which tls key is added
//		pkeydbFound			buffer to receive PGPKeyDBRef of found keys

PGPError PGPclExport
PGPclSearchServerForKeyIDs (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyID*			pkeyidList,
		PGPInt32			iNumKeyIDs,
		PGPclServerType		servertype,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBRef*		pkeydbFound);

//
//	Searches default keyserver using the specified filter.
//
//	Entry parameters :
//		hwndParent			parent window
//		filter				filter to use for search
//		uServer				one of above-defined server constants
//		keydbMain			keydb to which tls key is added
//		pkeydbFound			buffer to receive PGPKeyDBRef of found keys

PGPError PGPclExport
PGPclSearchServerWithFilter (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPFilterRef		filter,
		PGPclServerType		servertype,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBRef*		pkeydbFound);

//
//	Sends keys in keyset to the specified keyserver
//
//	Entry parameters :
//		hwndParent			parent window
//		szServerURL			server to send to
//		uServer				one of above-defined server constants
//							or zero to use pkeyserver
//		pkeyserver			server to send to (if uServer == 0)
//		keydbMain			keydb to which tls key is added and
//								which contains signing keys
//		keysetToSend		keyset with keys to send

PGPError PGPclExport
PGPclSendKeysToServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPclServerType		servertype,
		PGPKeyServerEntry*	pkeyserver,
		PGPKeyDBRef			keydbMain,
		PGPKeySetRef		keysetToSend);

//
//	Sends keys in keydb to the root keyserver with notifications
//
//	Entry parameters :
//		hwndToReceiveNotifications	window to get WM_NOTIFYs
//		szServerURL					server to send to
//		keydbMain					keydb to which tls key is added and
//										which contains signing keys
//		keysetToSend				keyset with keys to send
//
//	returns kPGPError_NoErr.  Progress notifications and final
//	results are sent to window in form of WM_NOTIFY messages

//  keyserver notifications to be placed in PGPclServerEvent.nmhdr.code
enum
{
	kPGPclNotifyServerDone				= 0,
	kPGPclNotifyServerProgress			= 1,
	kPGPclNotifyServerAbort				= 2,
	kPGPclNotifyServerError				= 3
};

typedef struct _PGPclServerEvent
{
	NMHDR		nmhdr;
	VOID*		pData;
	BOOL		cancel;
	LONG		step;
	LONG		total;
	CHAR		szmessage[256];
} PGPclServerEvent, *PPGPclServerEvent;

#define kPGPServerInfinite		-1L

PGPError PGPclExport
PGPclSendKeysToRootServerNotify (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndToReceiveNotifications,
		PGPKeyDBRef			keydbMain,
		PGPKeySetRef		keysetToSend);

//
//	Deletes keys in keyset from the specified keyserver
//
//	Entry parameters :
//		hwndParent			parent window
//		szServerURL			server to send to
//		space				pending or active bucket
//		keydbMain			keydb to which tls key is added and
//								which contains signing keys
//		keydbToDelete		keydb with keys to delete

PGPError PGPclExport
PGPclDeleteKeysFromServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyServerEntry*	pkeyserver,
		PGPInt32			iSpace,
		PGPKeyDBRef			keydbMain,
		PGPKeySetRef		keysetToDelete);

//
//	Disables keys in keyset on the specified keyserver
//
//	Entry parameters :
//		hwndParent			parent window
//		szServerURL			server to send to
//		space				pending or active bucket
//		keydbMain			keydb to which tls key is added and
//								which contains signing keys
//		keysetToDisable		keyset with keys to disable

PGPError PGPclExport
PGPclDisableKeysOnServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyServerEntry*	pkeyserver,
		PGPInt32			iSpace,
		PGPKeyDBRef			keydbMain,
		PGPKeySetRef		keysetToDisable);

//
//	Downloads a new groupset from the root server
//
//	Entry parameters :
//		hwndParent			parent window
//		keydbMain			keydb to which tls key is added and
//								which contains signing keys
//		groupsetDownloaded	newly downloaded groupset

PGPError PGPclExport
PGPclGetGroupsFromRootServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBRef			keydbMain,
		PGPGroupSetRef*		groupsetDownloaded);

//
//	Downloads a new groupset from the root server
//
//	Entry parameters :
//		hwndParent			parent window
//		keydbMain			keydb to which tls key is added and
//								which contains signing keys
//		groupsetToSend		groupset to send to root server

PGPError PGPclExport
PGPclSendGroupsToRootServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBRef			keydbMain,
		PGPGroupSetRef		groupsetToSend);

//
//	Create and send certificate request to CA server
//
//	Entry parameters :
//		hwndParent			parent window
//		keydbMain			main keydb
//		userid				userid for which request will be made
//		keysetKey			keyset containing single key to request
//								certificate for

PGPError PGPclExport
PGPclSendCertificateRequestToServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBObjRef		userid,
		PGPKeySetRef		keysetKey);

//
//	Create and send certificate request to CA server with notifications
//
//	Entry parameters :
//		hwndToNotify		window to receive progress notifications
//		keydbMain			main keydb
//		userid				userid for which request will be made
//		keysetKey			keyset containing single key to request
//								certificate for
//		pszPassPhrase		passphrase of key in keydbKey

PGPError PGPclExport
PGPclSendCertificateRequestToServerNotify (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndToNotify,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBObjRef		userid,
		PGPKeySetRef		keysetKey,
		char*				pszPassPhrase);

//
//	Get the previously-requested certificate from the CA server
//
//	Entry parameters :
//		hwndParent			parent window
//		keysetKey			keyset containing single key for whic request
//								was previously made
//		userid				userid to retrieve certificate for
//		pkeydbCert			buffer to receive PGPKeyDBRef of found cert

PGPError PGPclExport
PGPclRetrieveCertificateFromServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBRef			keydbMain,
		PGPKeySetRef		keysetKey,
		PGPKeyDBObjRef		userid,
		PGPByte*			pPasskey,
		PGPSize				sizePasskey,
		PGPKeyDBRef*		pkeydbCert);

//
//	Create and send certificate request to CA server
//
//	Entry parameters :
//		hwndParent			parent window
//		keydbMain			main keydb containing CA root key

PGPError PGPclExport
PGPclGetCertificateRevocationsFromServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBRef			keydbMain);

//
//	Create certificate request AV list
//
//	Entry parameters :
//		hwnd				parent window
//		bAdminCaller		TRUE=>being called from PGPadmin
//		userid				userid of key for which cert req is made
//								or kInvalidPGPUserID
//		serverclass			class of CA server or kPGPKeyServerClass_Invalid
//		puRequestType		CRS vs. PKCS10
//		pAVlist				receives pointer to AVlist
//		pNumAVs				receives number of AVs in list

PGPError PGPclExport
PGPclGetCACertRequestAVList (
		HWND				hwnd,
		PGPContextRef		context,
		PGPBoolean			bAdminCaller,
		PGPKeyDBObjRef		userid,
		PGPKeyServerClass	serverclass,
		PGPUInt32*			puRequestType,
		PGPAttributeValue**	ppAVlist,
		PGPUInt32*			pNumAVs);

//
//	Free AV list previously returned by PGPclGetCACertRequestAVList
//
//	Entry parameters :
//		pAVlist				pointer to AVlist
//		NumAVs				number of AVs in list

PGPError PGPclExport
PGPclFreeCACertRequestAVList (
		PGPAttributeValue*	pAVlist,
		PGPUInt32			uNumAVs);

//
//	return a string description of a given attribute type
//
//	Entry parameters :
//		attr				attribute
//		psz					string buffer to fill
//		uLen				size of string buffer
//
//	Note: returns kPGPError_ItemNotFound if no string available for attribute

PGPError PGPclExport
PGPclGetAVListAttributeString (
		PGPAVAttribute		attr,
		char*				psz,
		PGPUInt32			uLen);

//
//	Create and send key reconstruction blob to server
//
//	Entry parameters :
//		hwndParent			parent window
//		key					keypair for which to send blob
//		prompts, passes		data for creating blob

PGPError PGPclExport
PGPclSendReconstructionBlobToServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBObjRef		key,
		PGPKeyDBRef			keydbMain);

//
//	Create and send key reconstruction blob to server with notifications
//
//	Entry parameters :
//		hwndParent			parent window
//		key					keypair for which to send blob
//		prompts, passes		data for creating blob
//		pszPassphrase		passphrase of key

PGPError PGPclExport
PGPclSendReconstructionBlobToServerNotify (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndToNotify,
		PGPKeyDBObjRef		key,
		PGPKeyDBRef			keydbMain,
		PGPReconPrompts		prompts,
		PGPReconPasses		passes,
		char*				pszPassPhrase);

//
//	Get the previously-requested certificate from the CA server
//
//	Entry parameters :
//		hwndParent			parent window
//		key					key for which to retrieve blob
//		pkeydbReconstructed	buffer to receive PGPKeyDBRef of reconstructed key

PGPError PGPclExport
PGPclRetrieveReconstructedKeyFromServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBObjRef		key,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBRef*		pkeydbReconstructed);

//
//	Send the prefs to the server specified as kPGPPrefLDAPPrefsServer
//
//	Entry parameters :
//		hwndParent			parent window
//		prefref				client prefref to send
//		prefrefNet			net prefref to send

PGPError PGPclExport
PGPclSendPrefsToServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPPrefRef			prefref,
		PGPPrefRef			prefrefNet,
		PGPKeyDBRef			keydbMain);

//
//	Get the prefs from the server specified as kPGPPrefLDAPPrefsServer
//	and merge them with the existing prefs
//
//	Entry parameters :
//		hwndParent			parent window
//		prefref				client prefref to update
//		prefrefNet			net prefref to update
//
//	Note: if both prefrefs are kInvalidPGPPrefRef, the clientlib
//	prefrefs will be used instead.

PGPError PGPclExport
PGPclGetPrefsFromServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPPrefRef			prefref,
		PGPPrefRef			prefrefNet);

//
//	Get the admin-specified default keys from the default server
//
//	Entry parameters :
//		hwndParent			parent window
//		keydb				keydb to be updated

PGPError PGPclExport
PGPclGetDefaultKeysFromServer (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBRef			keydb);

//	_______________________________________________________
//
//	KeyDB Object properties propertysheet functions

//	messages
#define PGPCL_M_PROPERTYMESSAGE				WM_APP+330
#define PGPCL_M_REGISTERPROPERTYWINDOW		WM_APP+331

enum
{
	kPGPclProperties_UpdateObject	= 1,
	kPGPclProperties_CloseWindow	= 2,
	kPGPclProperties_DisableWindow	= 3,
	kPGPclProperties_ShowKeyProps	= 4,
	kPGPclProperties_ShowCertProps	= 5,
	kPGPclProperties_RefreshPhotoID	= 6,
	kPGPclProperties_EnableWindows	= 7,
	kPGPclProperties_UpdateKeyring	= 8
};

//	flags
enum
{
	kPGPclProperties_Modeless		= (1UL << 0),
	kPGPclProperties_CenterWindow	= (1UL << 1),
	kPGPclProperties_ReadOnly		= (1UL << 2),
	kPGPclProperties_Key			= (1UL << 3),
	kPGPclProperties_UserID			= (1UL << 4),
	kPGPclProperties_Signature		= (1UL << 5),
};

//
//	Put up property sheet about specified object
//
//	Entry parameters :
//		context			- PGP library context
//		hwndParent		- handle of parent window
//		obj				- object to display
//		keydbMain		- main keydb (if object is not in main keydb)
//		uFlags			- logical OR of above flags
//		phwnd			- pointer to buffer to receive hwnd of property
//						  sheet if kPGPclProperties_Modeless

PGPError PGPclExport
PGPclKeyDBObjProperties (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBObjRef		obj,
		PGPKeyDBRef			keydbMain,
		PGPUInt32			uFlags,
		HWND*				phwnd);

//	_______________________________________________________
//
//	Split key sharing functions

//	Send key share file to remote computer
//
//	Entry parameters :
//		context		- PGP library context
//		hwndParent	- handle of parent window
//		keydbMain	- main keydb (should contain decryption
//						and authentication keys)

PGPError PGPclExport
PGPclSendShares (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBRef			keydbMain);

//
//	Reconstitute key by collecting key shares
//
//	Entry parameters :
//		context		- PGP library context
//		hwndParent	- handle of parent window
//		keydbMain	- main keydb (should contain decryption
//						and authentication keys)
//		key			- key to reconstitute
//		ppPasskeyBuffer		- pointer to buffer to receive address
//							  of passkey buffer
//		piPasskeyLength		- pointer to buffer to receive length

PGPError PGPclExport
PGPclReconstituteKey (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBObjRef		key,
		PGPByte**			ppPasskeyBuffer,
		PGPUInt32*			piPasskeyLength);

//
//	Allow user to confirm remote authentication key
//
//	Entry parameters :
//		hwndParent		- handle of parent window
//		pszServer		- string with name of remote host
//		keyAuth			- authentication key to confirm
//		tlsCipher		- cipher suite number
//		keydbMain		- main keydb to which keyAuth may be added
//		uFlags			- flags controlling text display
//
//	returns kPGPError_NoErr or kPGPError_UserAbort

typedef enum PGPclAuthType
{
	kPGPclAuthUseAppropriate			= 0,
	kPGPclAuthShowAuthentication		= 1,
	kPGPclAuthReconstituting			= 2,
	kPGPclAuthNewKey					= 3,
	kPGPclAuthUnexpectedKey				= 4,
	kPGPclAuthExpectedKey				= 5,
	kPGPclAuthConfirmIfInvalid			= 6,
	kPGPclAuthDoNotConfirm				= 7,
	kPGPclAuthConfirmAlways				= 8
} PGPclAuthType;

PGPError PGPclExport
PGPclConfirmRemoteAuthentication (
		PGPContextRef			context,
		HWND					hwndParent,
		char*					pszServer,
		PGPKeyDBObjRef			keyAuth,
		PGPtlsCipherSuiteNum	tlsCipher,
		PGPKeyDBRef				keydbMain,
		PGPclAuthType			type);

//	_______________________________________________________
//
//	Passphrase functions

//	Get passphrase from user.
//
//	Entry parameters :
//		context		- PGP library context
//		hwndParent	- handle of parent window
//		szPrompt	- message string to be displayed to user
//		pszPhrase	- pointer to receive address of buffer which
//					  will contain passphrase.  The caller should
//					  deallocate this buffer by calling PGPclFreePhrase
//		keyset		- keyset containing keys to display in combo or list box
//					  NULL => hide key selection combo box
//		pKeyIDs		- additional keyids to tag onto end of listbox in
//					  decryption dialog
//		uKeyCount	- total number of keys to display (only necessary when
//					  keyids are to be displayed, otherwise can be zero)
//		pKey		- pointer to buffer to receive ref to selected key.
//					  if buffer contains key on entry, this will be default.
//					  buffer should be set to NULL to use keyring default key.
//					  NULL => hide key selection combo box
//		uOptions	- pointer to buffer which contains and will receive
//					  options bits
//					  NULL => hide option checkboxes
//		uFlags		- flag bits
//					  PGPcl_RSAENCRYPT => encrypting to RSA key
//					  PGPcl_DECRYPTION => display decryption dialog
//					  PGPcl_ENCRYPTION => display conventional encryption
//											  dialog
//
//	This function returns kPGPError_UserAbort if the user
//  presses the cancel button.

// options bits
enum
{
	kPGPclASCIIArmor					= (1UL << 0),
	kPGPclDetachedSignature				= (1UL << 1),
	kPGPclPassOnly						= (1UL << 2),
	kPGPclWipeOriginal					= (1UL << 3),
	kPGPclSecureViewer					= (1UL << 4),
	kPGPclSelfDecryptingArchive			= (1UL << 5),
	kPGPclInputIsText					= (1UL << 6),
};

// disable bits
enum
{
	kPGPclDisableASCIIArmor				= (1UL << 0),
	kPGPclDisableAutoMode				= (1UL << 1),
	kPGPclDisablePassOnly				= (1UL << 2),
	kPGPclDisableWipeOriginal			= (1UL << 3),
	kPGPclDisableSecureViewer			= (1UL << 4),
	kPGPclDisableSelfDecryptingArchive	= (1UL << 5),
	kPGPclDisableInputIsText			= (1UL << 6),
};

// flag bits
enum
{
	kPGPclRSAEncrypt					= (1UL << 0),
	kPGPclDecryption					= (1UL << 1),
	kPGPclEncryption					= (1UL << 2),
	kPGPclKeyPassphrase					= (1UL << 3),
	kPGPclRejectSplitKeys				= (1UL << 4),
	kPGPclHideQualityIndicator			= (1UL << 5),
};

PGPError PGPclExport
PGPclGetPhrase (
		PGPContextRef		context,
		PGPKeyDBRef			keydbMain,
		HWND				hwndParent,
		char*				szPrompt,
		char**				ppszPhrase,
		PGPKeySetRef		keyset,
		PGPKeyID*			pKeyIDs,
		PGPUInt32			uKeyCount,
		PGPKeyDBObjRef*		pkey,
		PGPUInt32*			puOptions,
		PGPUInt32			uFlags,
		PGPByte**			ppPasskeyBuffer,
		PGPUInt32*			puPasskeyLength,
		PGPUInt32			uMinLength,
		PGPUInt32			uMinQuality,
		PGPtlsContextRef	tlscontext,
		PGPKeyDBRef*		keydbAdded,
		char*				szTitle);

//
//	Recipient dialog glue call
//
//  Interfaces to PGPsdkUI calls.

#define kPGPCurrentRecipVersion 'DUKE'

typedef struct _recipientdialogstruct
{
	DWORD			Version;
	HWND			hwndParent;
	char *			szTitle;
	PGPContextRef	context;
	PGPKeyDBRef 	keydbOriginal;
	PGPKeyDBRef 	keydbSelected;
	char **			szRecipientArray;
	DWORD			dwNumRecipients;
	DWORD			dwOptions;
	DWORD			dwFlags;
	DWORD			dwDisableFlags;
	PGPtlsContextRef tlsContext;
	PGPKeyDBRef		keydbAdded;
} PGPclRecipientDialogStruct, *PPGPclRecipientDialogStruct;

PGPUInt32 PGPclExport
PGPclRecipientDialog (
		PPGPclRecipientDialogStruct prds);

//	Wipe and deallocate phrase buffer.
//
//	Entry parameters :
//		szPhrase	- passphrase buffer to wipe and deallocate
//

VOID PGPclExport
PGPclFreePhrase (
		char*				szPhrase);

//___________________________________
//
//	passphrase caching routines

//
//	This routine is called to get either cached phrase
//	(if available) or prompt user for phrase.

PGPError PGPclExport
PGPclGetCachedDecryptionPhrase (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		PGPKeyDBRef			keydbMain,
		HWND				hwnd,
		char*				szPrompt,
		BOOL				bForceUserInput,
		char**				pszBuffer,
		PGPKeySetRef		keysetEncryptedTo,
		PGPKeyID*			pkeyidEncryptedTo,
		PGPUInt32			uKeyIDCount,
		PGPByte**			ppPasskeyBuffer,
		PGPUInt32*			piPasskeyLength,
		PGPKeyDBRef*		pkeydbAdded,
		char*				szTitle);

//
//	This routine is called to get either signing cached phrase
//	(if available) or prompt user for phrase.

PGPError PGPclExport
PGPclGetCachedSigningPhrase (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwnd,
		char*				szPrompt,
		BOOL				bForceUserInput,
		char**				pszBuffer,
		PGPKeyDBRef			keydbSigning,
		PGPKeyDBObjRef*		pkeySigning,
		PGPUInt32*			puOptions,
		PGPUInt32			uFlags,
		PGPByte**			ppPasskeyBuffer,
		PGPUInt32*			piPasskeyLength,
		PGPKeyDBRef*		pkeydbAdded,
		char*				szTitle);

//
//	Called to wipe and free the phrase returned by
//	PGPGetCachedPhrase.

void PGPclExport
PGPclFreeCachedPhrase (
		char* 				szPhrase);

//	__________________________________________________________
//
//	Convenience wrapper functions for sdkUI passphrase routines

PGPError PGPclExport
PGPclGetKeyPhrase (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwnd,
		char*				szPrompt,
		PGPKeyDBRef			keydb,
		PGPKeyDBObjRef		key,
		char**				ppszPhrase,
		PGPByte**			ppPasskeyBuffer,
		PGPInt32*			piPasskeyLength);

PGPError PGPclExport
PGPclGetSigningKeyPhrase (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwnd,
		char*				szPrompt,
		PGPKeyDBRef			keydb,
		PGPBoolean			bRejectSplitKeys,
		PGPKeyDBObjRef*		pkey,
		char**				ppszPhrase,
		PGPByte**			ppPasskeyBuffer,
		PGPInt32*			piPasskeyLength);

PGPError PGPclExport
PGPclGetConfirmationPhrase (
		PGPContextRef		context,
		HWND				hwnd,
		char*				szPrompt,
		PGPKeyDBRef			keydb,
		PGPInt32			iMinPhraseLength,
		PGPInt32			iMinPhraseQuality,
		char**				ppszPhrase);

PGPError PGPclExport
PGPclGetDecryptionPhrase (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwnd,
		char*				szPrompt,
		PGPKeyDBRef			keydbMain,
		PGPKeyDBObjRef*		pkey,
		PGPKeySetRef		keysetDecryption,
		PGPInt32			iKeyIDCount,
		PGPKeyID*			keyidsDecryption,
		PGPKeyDBRef*		pkeydbToAdd,
		char**				ppszPhrase,
		PGPByte**			ppPasskeyBuffer,
		PGPInt32*			piPasskeyLength);

PGPError PGPclExport
PGPclGetConventionalPhrase (
		PGPContextRef		context,
		HWND				hwnd,
		char*				szPrompt,
		char**				ppszPhrase);

VOID PGPclExport
PGPclFreePhrase (
		char* 				pszPhrase);

VOID PGPclExport
PGPclFreePasskey (
		PGPByte*			pbyte,
		PGPSize 			size);

PGPError PGPclExport
PGPclChangePhrase (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwnd,
		PGPKeyDBObjRef		key,
		PGPBoolean			bAdvise);

PGPError PGPclExport
PGPclCachePhrase (
		PGPContextRef		context,
		PGPKeyDBObjRef		key,
		char*				pszPhrase,
		PGPByte*			pPasskey,
		PGPInt32			iPasskeyLength);


//	__________________________________________________________
//

//  Update introducer keys from server
//
//	Entry parameters :
//		hwnd						NULL => use silent mode
//		bForce						TRUE => don't ask importation confirmation

PGPError PGPclExport
PGPclAutoUpdateIntroducers (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwnd,
		PGPKeyDBRef			keydbMain,
		BOOL				bForce);

//
//  Schedule next CRL update on basis of root CA key

PGPError PGPclExport
PGPclScheduleNextCRLUpdate (
		PGPContextRef		context,
		PGPKeyDBRef			keydbMain);

//	__________________________________________________________
//
//  Look up an unknown signer's key on the keyserver
//
//	Entry parameters :
//		context			context ref
//		hwnd			handle to parent window
//		event			event data pointer
//		signingKeyID	Key ID of unknown signer
//		pbGotKeys		Pointer to boolean, TRUE if the key(s)
//						of the unknown signer were successfully fetched

PGPError PGPclExport
PGPclLookupUnknownSigner (
		PGPContextRef 		context,
		PGPKeyDBRef 		keydbMain,
		PGPtlsContextRef 	tlscontext,
		HWND 				hwnd,
		PGPEvent*			event,
		PGPKeyID 			keyidSigning,
		PGPBoolean*			pbGotKeys,
		PGPKeyDBRef			*pkeydbNew);

//________________________________________________
//
//	Creates empty keylist window
//
//	Entry parameters :
//		context		- PGP library context
//		hwndParent	- handle of parent window
//		iID			- window ID to assign to manager (used in notifications)
//		lpfnSetList	- callback to add/remove dialog windows from list
//		iX			- x coordinate of manager window relative to parent
//		iY			- y coordinate of manager window relative to parent
//		iWidth		- width of manager window
//		iHeight		- height of manager window
//		uFlags		- additional flags
//		phwndTree	- receives handle of treelist control
//
//	Returns handle to keymanager
//

// reason codes
enum
{
	kPGPclKeyList_ExistingSplitKeyDlgs	= 1
};

// columns in PGPclKeyListConfigure
enum
{
	kPGPclKeyList_ValidityColumn	= (1UL << 0),
	kPGPclKeyList_SizeColumn		= (1UL << 1),
	kPGPclKeyList_DescriptionColumn	= (1UL << 2),
	kPGPclKeyList_KeyIDColumn		= (1UL << 3),
	kPGPclKeyList_TrustColumn		= (1UL << 4),
	kPGPclKeyList_CreationColumn	= (1UL << 5),
	kPGPclKeyList_ExpirationColumn	= (1UL << 6),
	kPGPclKeyList_ADKColumn			= (1UL << 7)
};

// options bits
enum
{
	kPGPclKeyList_ReadOnly			= (1UL << 0),
	kPGPclKeyList_GreenDotMode		= (1UL << 1),
	kPGPclKeyList_MarginalIsInvalid	= (1UL << 2),
	kPGPclKeyList_SelectKeysOnly	= (1UL << 3),
	kPGPclKeyList_EnableDropIn		= (1UL << 4),
	kPGPclKeyList_EnableDragOut		= (1UL << 5),
	kPGPclKeyList_DisableStatusBar	= (1UL << 6),
	kPGPclKeyList_PendingBucket		= (1UL << 7),
	kPGPclKeyList_EnableReloads		= (1UL << 8)
};

// mask bits for PGPclKeyListConfig struct
enum
{
	kPGPclKeyList_KeyServer			= (1UL << 1),
	kPGPclKeyList_Options			= (1UL << 2),
	kPGPclKeyList_Columns			= (1UL << 3),
	kPGPclKeyList_StatusHWND		= (1UL << 4),

	kPGPclKeyList_ConfigureAll		= 0xFFFFFFFF
};

typedef struct _KL FAR* HKEYLIST;
typedef struct {				// configuration information
	PGPUInt32	uMask;			//  mask bits specifying things to configure
	PGPUInt32	uOptions;		//	logical OR of option bits (above)
	PGPUInt32	uShowColumns;	//  logical OR of columns to show
	HWND		hwndStatusBar;	//  handle of status bar window

	PGPKeyServerEntry	keyserver;	//  keyserver to use for subsequent ops
} PGPclKeyListConfig, *PPGPclKeyListConfig;


// KeyList Styles
enum
{
	kPGPclKeyList_SingleSelection	= (1UL << 0),
	kPGPclKeyList_ShowSelection		= (1UL << 1),
};

// selection bits set in TLN_CONTEXTMENU notification
enum
{
	kPGPclKeyList_None				= 0,
	kPGPclKeyList_Key				= (1UL << 0),
	kPGPclKeyList_UserID			= (1UL << 1),
	kPGPclKeyList_Signature			= (1UL << 2)
};

HKEYLIST PGPclExport
PGPclKeyListCreate (
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwndParent,
		PGPInt32			iID,
		PGPInt32			iX,
		PGPInt32			iY,
		PGPInt32			iWidth,
		PGPInt32			iHeight,
		PGPUInt32			uFlags,
		HWND*				phwndTree);

//
//	Configures keylist window
//
//	Entry parameters :
//		hkeylist	- handle of key list to configure
//		pKLConfig	- pointer to KLCONFIG struct
//
//	Returns kPGPError_NoErr if no error
//

PGPError PGPclExport
PGPclKeyListConfigure (
		HKEYLIST			hkeylist,
		PPGPclKeyListConfig	pKLConfig);

//
//	Destroys keylist window
//
//	Entry parameters :
//		hkeylist			- handle of key list to destroy
//		bSaveColumnInfo		- TRUE => save column info to pref file
//
//	Returns kPGPError_NoErr if successful
//

PGPError PGPclExport
PGPclKeyListDestroy (
		HKEYLIST			hkeylist,
		PGPBoolean			bSaveColumnInfo);

//
//	Handles default behavior for TreeList notifications
//
//	Entry parameters :
//		hkeylist	- handle of key list
//		lParam		- LPARAM from WM_NOTIFY message
//
//	Returns kPGPError_NoErr
//

PGPError PGPclExport
PGPclKeyListDefaultNotificationProc (
		HKEYLIST			hkeylist,
		LPARAM				lParam);

//
//	load keyset into list
//
//	Entry parameters :
//		hkeylist		- handle of key list
//		keydbMain		- main keydb used for commits and trust calcs
//		keysetDisplay	- keyset to display in window
//
//	Returns kPGPError_NoErr if no error
//

PGPError PGPclExport
PGPclKeyListLoadKeys (
		HKEYLIST			hkeylist,
		PGPKeyDBRef			keydbMain,
		PGPKeySetRef		keysetDisplay);

//
//	reload keyset into list
//
//	Entry parameters :
//		hkeylist		- handle of key list
//		bExpandNew		- TRUE => expand any newly-found objects
//
//	Returns kPGPError_NoErr if no error
//

PGPError PGPclExport
PGPclKeyListReloadKeys (
		HKEYLIST			hkeylist,
		PGPBoolean			bExpandNew);

//
//	update the specified key in list
//
//	Entry parameters :
//		hkeylist		- handle of key list
//		obj				- key to modify
//		op				- operation to perform
//		bRedraw			- invalidate TreeList
//
//	Returns kPGPError_NoErr if no error
//

typedef enum PGPclKeyListUpdateOp
{
	kPGPclKeyListUpdateObject		= 0,
	kPGPclKeyListReplaceObject		= 1,
	kPGPclKeyListRemoveObject		= 2,
	kPGPclKeyListInsertObject		= 3,
	kPGPclKeyListUpdateValidities	= 4,
	kPGPclKeyListUpdateObjectFocus	= 5,
} PGPclKeyListUpdateOp;

PGPError PGPclExport
PGPclKeyListUpdateTree (
		HKEYLIST				hkeylist,
		PGPclKeyListUpdateOp	op,
		PGPKeyDBObjRef			obj,
		PGPBoolean				bRedraw);

//
//	get a keyset of the currently selected keys
//
//	Entry parameters :
//		hkeylist			- handle of key list
//		pkeysetSelected		- pointer to buffer to receive keyset ref
//						  	(must be freed when done)
//
//	Returns kPGPError_NoErr if no error
//

PGPError PGPclExport
PGPclKeyListGetSelectedKeys (
		HKEYLIST 			hkeylist,
		PGPKeySetRef* 		pkeysetSelected);

//
//	set state of TreeList
//
//	Entry parameters :
//		hkeylist		- handle of key list
//		op				- operation to perform
//		key				- key to select
//
//	Returns kPGPError_NoErr if no error
//

typedef enum PGPclKeyListTreeOp
{
	kPGPclSelectSpecified		= 0,
	kPGPclSelectSpecifiedOnly	= 1,
	kPGPclSelectAll				= 2,
	kPGPclDeselectAll			= 3,
	kPGPclExpand				= 4,
	kPGPclCollapse				= 5,
	kPGPclResort				= 6
} PGPclKeyListTreeOp;

PGPError PGPclExport
PGPclKeyListSetTree (
		HKEYLIST	 			hkeylist,
		PGPclKeyListTreeOp	 	op,
		PGPKeyDBObjRef 			key);

//
//	query keylist for information on current selections
//
//	Entry parameters :
//		hkeylist			- handle of key list
//		puFocusedType		- type of object currently with focus
//		pobjFocused			- KeyDB object currently with focus
//		pbMultipleSel		- TRUE => multiple objects selected
//		puSelectionFlags 	- logical OR of selection flags
//
//	Returns kPGPError_NoErr if successful
//

PGPError PGPclExport
PGPclKeyListGetSelectionInfo (
		HKEYLIST				hkeylist,
		PGPUInt32*				puFocusedType,
		PGPKeyDBObjRef*			pobjFocused,
		PGPBoolean*				pbMultipleSel,
		PGPUInt32*				puSelectionFlags);

//
//	get currently displayed columns
//
//	Entry parameters :
//		hkeylist			- handle of key list
//		pulColumnFlags		- buffer to receive flags of selected columns
//
//	Returns kPGPError_NoErr if no error
//

PGPError PGPclExport
PGPclKeyListGetSelectedColumns (
		HKEYLIST 				hkeylist,
		PGPUInt32* 				puColumnFlags);

//
//	set columns to be displayed
//
//	Entry parameters :
//		hkeylist			- handle of key list
//		ulColumnFlags		- flags selecting columns to display
//		bRedraw				- TRUE=>forces redraw of display
//
//	Returns kPGPError_NoErr if no error
//

PGPError PGPclExport
PGPclKeyListSelectColumns (
		HKEYLIST 				hkeylist,
		PGPUInt32				uColumnFlags,
		PGPBoolean				bRedraw);

//________________________________________________
//
//	Key generation
//

//  if no error, these bits will be set in flags parameter
enum
{
	kPGPclKeyCreate_NewKey			= (1UL << 0),
	kPGPclKeyCreate_NewDefaultKey	= (1UL << 1),
	kPGPclKeyCreate_OnToken			= (1UL << 2)
};


PGPError PGPclExport
PGPclKeyGenerationWizard (
		PGPContextRef			context,
		PGPtlsContextRef		tlscontext,
		HWND					hwndParent,
		PGPKeyDBRef				keydb,
		PGPKeyDBObjRef*			pkeyNew,
		PGPUInt32*				puCreationFlags);

PGPError PGPclExport
PGPclDiskGenerationWizard (
	PGPContextRef		context,
	HWND				hwndParent,
	PGPKeyID			*pKeyID);

PGPError PGPclExport
PGPclCopyX509CertToToken(
		PGPContextRef		context,
		PGPtlsContextRef	tlscontext,
		HWND				hwnd,  
		LPSTR				pszPhrase, 
		PGPKeyDBRef			keydb,
		PGPKeyDBObjRef		obj, 
		PGPKeySetRef		keyset);


#ifdef __cplusplus
}
#endif

#endif /* ] Included_pgpClientLibWin32_h */


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
