/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: pgpDiskEngineDyn.c,v 1.11 2002/11/22 02:32:15 wjb Exp $
____________________________________________________________________________*/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "pgpMem.h"
#include "pgpPflErrors.h"

#ifndef PGPDISK_DYNLINK
#define PGPDISK_DYNLINK
#endif	// !PGPDISK_DYNLINK

#include "pgpDiskEngine.h"

/*
	Stubs for dynamic linking to the PGPdiskEngine.dll. Include this file 
	and #define PGPDISK_DYNLINK before you #include pgpDiskEngine.h.
*/

/* Types */

typedef int (_cdecl *DISKPROC)();


/* Static variables */

static HINSTANCE	PGPdiskEngineDll;

/* Function stubs */

PGPError 
PGPdiskEngineInitLibrary(PGPUInt32 Flags)
{
	PGPError	error	= kPGPError_NoErr;

	if (IsNull(PGPdiskEngineDll))
	{
		PGPdiskEngineDll = LoadLibrary("PGPdiskEngine.dll");

		if (IsNull(PGPdiskEngineDll))
			error = kPGPError_Win32DllOpFailed;
	}

	if (IsntPGPError(error))
	{
		DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(
			PGPdiskEngineDll, "PGPdiskEngineInitLibrary");

		if (IsNull(diskFunc))
			error = kPGPError_Win32DllOpFailed;

		if (IsntPGPError(error))
			error = diskFunc(Flags);
	}

	return error;
}

void 
PGPdiskEngineCleanupLibrary()
{
	if (IsntNull(PGPdiskEngineDll))
	{
		DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(
			PGPdiskEngineDll, "PGPdiskEngineCleanupLibrary");

		if (IsntNull(diskFunc))
			diskFunc();

		FreeLibrary(PGPdiskEngineDll);
		PGPdiskEngineDll = NULL;
	}
}

PGPError 
PGPdiskNewContext(PGPUInt32 clientAPIVersion, PGPdiskContextRef *pContext)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskNewContext");

	return diskFunc(clientAPIVersion, pContext);
}

void 
PGPdiskFreeContext(PGPdiskContextRef context)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskFreeContext");

	diskFunc(context);
}

PGPError 
PGPdiskEngineExport 
PGPdiskReloadContextPGPKeyDB(PGPdiskContextRef context)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskReloadContextPGPKeyDB");

	return (PGPError) diskFunc(context);
}

PGPError 
PGPdiskEngineExport 
PGPdiskDirtyPGPKeyDB(PGPdiskContextRef context)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDirtyPGPKeyDB");

	return (PGPError) diskFunc(context);
}

PGPError 
PGPdiskOpenPeekContextPGPKeyDB(
	PGPdiskContextRef	context, 
	PGPKeyDBRef			*pPGPKeyDB)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskOpenPeekContextPGPKeyDB");

	return (PGPError) diskFunc(context, pPGPKeyDB);
}

PGPContextRef 
PGPdiskPeekContextPGPContext(PGPdiskContextRef context)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskPeekContextPGPContext");

	return (PGPContextRef) diskFunc(context);
}

PGPPrefRef 
PGPdiskPeekContextPGPclPrefs(PGPdiskContextRef context)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskPeekContextPGPclPrefs");

	return (PGPPrefRef) diskFunc(context);
}

PGPError 
PGPdiskContextIsThereADK(PGPdiskContextRef context, PGPBoolean *pIsThereADK)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskContextIsThereADK");

	return diskFunc(context, pIsThereADK);
}

PGPError 
PGPdiskContextGetADK(PGPdiskContextRef context, PGPKeyDBObjRef *pADKKey)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskContextGetADK");

	return diskFunc(context, pADKKey);
}

PGPError 
PGPdiskContextGetADKUserID(
	PGPdiskContextRef	context, 
	char				*userID, 
	PGPSize				availLength)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskContextGetADKUserID");

	return diskFunc(context, userID, availLength);
}

/* PGPdiskUserRef functions */

PGPError 
PGPdiskNewUserWithSymmetricKey(
	PGPdiskUserRef	cipheringUser, 
	const char		*userName, 
	const char		*passphrase, 
	PGPBoolean		isReadOnly, 
	PGPdiskUserRef	*pNewUser)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskNewUserWithSymmetricKey");

	return diskFunc(cipheringUser, userName, passphrase, isReadOnly, 
		pNewUser);
}

PGPError 
PGPdiskNewUserWithPublicKey(
	PGPdiskUserRef	cipheringUser, 
	PGPKeyDBObjRef	pubKey, 
	PGPBoolean		isReadOnly, 
	PGPBoolean		isLocked, 
	PGPdiskUserRef	*pNewUser)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskNewUserWithPublicKey");

	return diskFunc(cipheringUser, pubKey, isReadOnly, isLocked, pNewUser);
}

PGPError 
PGPdiskDuplicateUser(PGPdiskUserRef user, PGPdiskUserRef *pNewUser)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDuplicateUser");

	return diskFunc(user, pNewUser);
}

void 
PGPdiskFreeUser(PGPdiskUserRef user)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskFreeUser");

	diskFunc(user);
}

PGPdiskContextRef 
PGPdiskPeekUserContext(PGPdiskUserRef user)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskPeekUserContext");

	return (PGPdiskContextRef) diskFunc(user);
}

PGPError 
PGPdiskUserEnableCiphering(
	PGPdiskUserRef	user, 
	PGPdiskDiskRef	disk, 
	const char		*passphrase,
	int				passkeylen)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserEnableCiphering");

	return diskFunc(user, disk, passphrase,passkeylen);
}

void 
PGPdiskUserDisableCiphering(PGPdiskUserRef user)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserDisableCiphering");

	diskFunc(user);
}

PGPError 
PGPdiskUserGetPGPKeyID(PGPdiskUserRef user, PGPKeyID *pKeyID)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserGetPGPKeyID");

	return diskFunc(user, pKeyID);
}

PGPError 
PGPdiskUserChangePassphrase(
	PGPdiskUserRef	symUserForCiphering, 
	const char		*newPassphrase)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserChangePassphrase");

	return diskFunc(symUserForCiphering, newPassphrase);
}

PGPBoolean 
PGPdiskUserAreUsersEqual(PGPdiskUserRef user1, PGPdiskUserRef user2)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserAreUsersEqual");

	return diskFunc(user1, user2);
}

PGPError 
PGPdiskGetUserBooleanProp(
	PGPdiskUserRef		user, 
	PGPdiskUserPropName	prop, 
	PGPBoolean			*pBool)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskGetUserBooleanProp");

	return diskFunc(user, prop, pBool);
}

PGPError 
PGPdiskGetUserNumericProp(
	PGPdiskUserRef		user, 
	PGPdiskUserPropName	prop, 
	PGPInt32			*pNumber)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskGetUserNumericProp");

	return diskFunc(user, prop, pNumber);
}

PGPError 
PGPdiskGetUserDataProp(
	PGPdiskUserRef		user, 
	PGPdiskUserPropName	prop, 
	void				*pData, 
	PGPSize				availLength, 
	PGPSize				*pUsedLength)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskGetUserDataProp");

	return diskFunc(user, prop, pData, availLength, pUsedLength);
}

PGPError 
PGPdiskSetUserBooleanProp(
	PGPdiskUserRef		user, 
	PGPdiskUserPropName	prop, 
	PGPBoolean			boolVal)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskSetUserBooleanProp");

	return diskFunc(user, prop, boolVal);
}

PGPError 
PGPdiskSetUserNumericProp(
	PGPdiskUserRef		user, 
	PGPdiskUserPropName	prop, 
	PGPInt32			number)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskSetUserNumericProp");

	return diskFunc(user, prop, number);
}

PGPError 
PGPdiskSetUserDataProp(
	PGPdiskUserRef		user, 
	PGPdiskUserPropName	prop, 
	const void			*pData, 
	PGPSize				dataLength)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskSetUserDataProp");

	return diskFunc(user, prop, pData, dataLength);
}

/* PGPdiskUserSetRef functions */

PGPError 
PGPdiskNewUserSet(PGPdiskUserSetRef *pUserSet)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskNewUserSet");

	return diskFunc(pUserSet);
}

PGPError 
PGPdiskNewSingletonUserSet(PGPdiskUserRef user, PGPdiskUserSetRef *pUserSet)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskNewSingletonUserSet");

	return diskFunc(user, pUserSet);
}

void 
PGPdiskFreeUserSet(PGPdiskUserSetRef userSet)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskFreeUserSet");

	diskFunc(userSet);
}

PGPUInt32 
PGPdiskUserSetCount(PGPdiskUserSetRef userSet)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserSetCount");

	return diskFunc(userSet);
}

PGPError 
PGPdiskUserSetAddUsers(
	PGPdiskUserSetRef	userSet, 
	PGPdiskUserSetRef	usersToAdd)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserSetAddUsers");

	return diskFunc(userSet, usersToAdd);
}

PGPError 
PGPdiskUserSetAddUser(
	PGPdiskUserSetRef	userSet, 
	PGPdiskUserRef		userToAdd)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserSetAddUser");

	return diskFunc(userSet, userToAdd);
}

PGPError 
PGPdiskUserSetRemoveUsers(
	PGPdiskUserSetRef	userSet, 
	PGPdiskUserSetRef	usersToRemove)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserSetRemoveUsers");

	return diskFunc(userSet, usersToRemove);
}

PGPError 
PGPdiskUserSetRemoveUser(
	PGPdiskUserSetRef	userSet, 
	PGPdiskUserRef		userToRemove)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserSetRemoveUser");

	return diskFunc(userSet, userToRemove);
}

PGPError 
PGPdiskUserSetEmptyUserSet(PGPdiskUserSetRef userSet)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserSetEmptyUserSet");

	return diskFunc(userSet);
}

/* PGPdiskUserIterRef functions */

PGPError 
PGPdiskNewUserIter(PGPdiskUserSetRef userSet, PGPdiskUserIterRef *pUserIter)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskNewUserIter");

	return diskFunc(userSet, pUserIter);
}

void 
PGPdiskFreeUserIter(PGPdiskUserIterRef userIter)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskFreeUserIter");

	diskFunc(userIter);
}

PGPdiskUserSetRef 
PGPdiskPeekUserSet(PGPdiskUserIterRef userIter)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskPeekUserSet");

	return (PGPdiskUserSetRef) diskFunc(userIter);
}

PGPUInt32 
PGPdiskUserIterCount(PGPdiskUserIterRef userIter)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserIterCount");

	return diskFunc(userIter);
}

PGPInt32 
PGPdiskUserIterIndex(PGPdiskUserIterRef userIter)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserIterIndex");

	return diskFunc(userIter);
}

PGPBoolean 
PGPdiskUserIterUser(PGPdiskUserIterRef userIter, PGPdiskUserRef *pUser)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserIterUser");

	return diskFunc(userIter, pUser);
}

PGPBoolean 
PGPdiskUserIterMove(
	PGPdiskUserIterRef	userIter, 
	PGPInt32			relOffset, 
	PGPBoolean			fromStart, 
	PGPdiskUserRef		*pUser)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserIterMove");

	return diskFunc(userIter, relOffset, fromStart, pUser);
}

PGPInt32 
PGPdiskUserIterSeek(PGPdiskUserIterRef userIter, PGPdiskUserRef user)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserIterSeek");

	return diskFunc(userIter, user);
}

PGPBoolean 
PGPdiskUserIterNext(PGPdiskUserIterRef userIter, PGPdiskUserRef *pUser)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserIterNext");

	return diskFunc(userIter, pUser);
}

PGPBoolean 
PGPdiskUserIterPrev(PGPdiskUserIterRef userIter, PGPdiskUserRef *pUser)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserIterPrev");

	return diskFunc(userIter, pUser);
}

void 
PGPdiskUserIterRewind(PGPdiskUserIterRef userIter)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserIterRewind");

	diskFunc(userIter);
}

PGPBoolean 
PGPdiskUserIterSearchOnUserName(
	PGPdiskUserIterRef	userIter, 
	const char			*userName, 
	PGPdiskUserRef		*pUser)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserIterSearchOnUserName");

	return diskFunc(userIter, userName, pUser);
}

PGPBoolean 
PGPdiskUserIterFindEqualUser(
	PGPdiskUserIterRef	userIter, 
	PGPdiskUserRef		user, 
	PGPdiskUserRef		*pEqualUser)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserIterFindEqualUser");

	return diskFunc(userIter, user, pEqualUser);
}

PGPBoolean 
PGPdiskUserIterFindAdminUser(
	PGPdiskUserIterRef	userIter, 
	PGPdiskUserRef		*pAdminUser)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserIterFindAdminUser");

	return diskFunc(userIter, pAdminUser);
}

PGPError 
PGPdiskUserIterSetAdminUser(
	PGPdiskUserIterRef	userIter, 
	PGPdiskUserRef		userForCiphering, 
	PGPdiskUserRef		oldAdminForCiphering)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUserIterSetAdminUser");

	return diskFunc(userIter, userForCiphering, oldAdminForCiphering);
}

/* PGPdiskDiskRef operations */

PGPError 
PGPdiskOpenDisk(
	PGPdiskContextRef	context, 
	const char			*path, 
	PGPBoolean			readOnly, 
	PGPdiskDiskRef		*pDisk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskOpenDisk");

	return diskFunc(context, path, readOnly, pDisk);
}

PGPError 
PGPdiskCloseDisk(PGPdiskDiskRef disk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskCloseDisk");

	return diskFunc(disk);
}

PGPdiskContextRef 
PGPdiskPeekDiskContext(PGPdiskDiskRef disk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskPeekDiskContext");

	return (PGPdiskContextRef) diskFunc(disk);
}

PGPError 
PGPdiskDiskExportUserSet(PGPdiskDiskRef disk, PGPdiskUserSetRef userSet)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskExportUserSet");

	return diskFunc(disk, userSet);
}

PGPError 
PGPdiskDiskImportUserSet(
	PGPdiskDiskRef		disk, 
	PGPdiskUserSetRef	userSet, 
	PGPdiskUserRef		adminUserForCiphering)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskImportUserSet");

	return diskFunc(disk, userSet, adminUserForCiphering);
}

PGPError 
PGPdiskDiskChangeOnePassphrase(
	PGPdiskDiskRef	disk, 
	const char		*userName, 
	const char		*oldPassphrase, 
	const char		*newPassphrase)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskChangeOnePassphrase");

	return diskFunc(disk, userName, oldPassphrase, newPassphrase);
}

PGPError 
PGPdiskDiskRemoveOneUser(PGPdiskDiskRef disk, PGPdiskUserRef userForCiphering)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskRemoveOneUser");

	return diskFunc(disk, userForCiphering);
}

PGPError 
PGPdiskWipeUsersOnDisk(PGPdiskDiskRef disk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskWipeUsersOnDisk");

	return diskFunc(disk);
}

PGPError 
PGPdiskReEncryptDisk(
	PGPdiskDiskRef				disk, 
	const char					*adminPassphrase, 
	PGPdiskEncryptionAlgorithm	newAlgorithm, 
	ReEncryptDiskStatusFuncType	statusFunc, 
	void						*userValue)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskReEncryptDisk");

	return diskFunc(disk, adminPassphrase, newAlgorithm, statusFunc, 
		userValue);
}

PGPError 
PGPdiskGetDiskBooleanProp(
	PGPdiskDiskRef		disk, 
	PGPdiskDiskPropName	prop, 
	PGPBoolean			*pBool)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskGetDiskBooleanProp");

	return diskFunc(disk, prop, pBool);
}

PGPError 
PGPdiskGetDiskNumericProp(
	PGPdiskDiskRef		disk, 
	PGPdiskDiskPropName	prop, 
	PGPInt32			*pNumber)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskGetDiskNumericProp");

	return diskFunc(disk, prop, pNumber);
}

PGPError 
PGPdiskGetDiskDataProp(
	PGPdiskDiskRef		disk, 
	PGPdiskDiskPropName	prop, 
	void				*pData, 
	PGPSize				availLength, 
	PGPSize				*pUsedLength)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskGetDiskDataProp");

	return diskFunc(disk, prop, pData, availLength, pUsedLength);
}

PGPError 
PGPdiskSetDiskBooleanProp(
	PGPdiskDiskRef		disk, 
	PGPdiskDiskPropName	prop, 
	PGPBoolean			boolVal)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskSetDiskBooleanProp");

	return diskFunc(disk, prop, boolVal);
}

PGPError 
PGPdiskSetDiskNumericProp(
	PGPdiskDiskRef		disk, 
	PGPdiskDiskPropName	prop, 
	PGPInt32			number)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskSetDiskNumericProp");

	return diskFunc(disk, prop, number);
}

PGPError 
PGPdiskSetDiskDataProp(
	PGPdiskDiskRef		disk, 
	PGPdiskDiskPropName	prop, 
	const void			*pData, 
	PGPSize				dataLength)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskSetDiskDataProp");

	return diskFunc(disk, prop, pData, dataLength);
}

/* Specialized PGPdiskDiskRef operations */

PGPError 
PGPdiskLockDiskVolume(PGPdiskDiskRef disk, PGPBoolean forFormat)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskLockDiskVolume");

	return diskFunc(disk, forFormat);
}

PGPError 
PGPdiskUnlockDiskVolume(PGPdiskDiskRef disk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUnlockDiskVolume");

	return diskFunc(disk);
}

PGPError 
PGPdiskReadDiskVolume(
	PGPdiskDiskRef	disk, 
	void			*buffer, 
	PGPUInt64		blockPos, 
	PGPUInt32		blocksLength)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskReadDiskVolume");

	return diskFunc(disk, buffer, blockPos, blocksLength);
}

PGPError 
PGPdiskWriteDiskVolume(
	PGPdiskDiskRef	disk, 
	const void		*buffer, 
	PGPUInt64		blockPos, 
	PGPUInt32		blocksLength)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskWriteDiskVolume");

	return diskFunc(disk, buffer, blockPos, blocksLength);
}

/* PGPdiskDiskSetRef functions */

PGPError 
PGPdiskNewDiskSet(PGPdiskContextRef context, PGPdiskDiskSetRef *pDiskSet)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskNewDiskSet");

	return diskFunc(context, pDiskSet);
}

void 
PGPdiskFreeDiskSet(PGPdiskDiskSetRef diskSet)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskFreeDiskSet");

	diskFunc(diskSet);
}

PGPdiskContextRef 
PGPdiskPeekDiskSetContext(PGPdiskDiskSetRef diskSet)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskPeekDiskSetContext");

	return (PGPdiskContextRef) diskFunc(diskSet);
}

PGPUInt32 
PGPdiskDiskSetCount(PGPdiskDiskSetRef userSet)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskSetCount");

	return diskFunc(userSet);
}

PGPError 
PGPdiskDiskSetMountDisk(
	PGPdiskDiskSetRef	diskSet, 
	const char			*path, 
	PGPdiskUserRef		user, 
	const char			*root, 
	PGPBoolean			readOnly, 
	PGPdiskDiskRef		*pDisk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskSetMountDisk");

	return diskFunc(diskSet, path, user, root, readOnly, pDisk);
}

PGPError 
PGPdiskDiskSetUnmountDisk(
	PGPdiskDiskSetRef	diskSet, 
	PGPdiskDiskRef		disk, 
	PGPBoolean			isForced)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskSetUnmountDisk");

	return diskFunc(diskSet, disk, isForced);
}

PGPError 
PGPdiskDiskSetCreateDiskWithSym(
	PGPdiskDiskSetRef			diskSet, 
	const char					*path, 
	PGPUInt64					blocksDisk, 
	PGPdiskEncryptionAlgorithm	algorithm, 
	const char					*passphrase, 
	const char					*userName, 
	const char					*defaultRoot, 
	NewDiskStatusFuncType		statusFunc, 
	void						*userValue, 
	PGPdiskDiskRef				*pDisk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskSetCreateDiskWithSym");

	return diskFunc(diskSet, path, blocksDisk, algorithm, passphrase, 
		userName, defaultRoot, statusFunc, userValue, pDisk);
}

PGPError 
PGPdiskDiskSetCreateDiskWithPub(
	PGPdiskDiskSetRef			diskSet, 
	const char					*path, 
	PGPUInt64					blocksDisk, 
	PGPdiskEncryptionAlgorithm	algorithm, 
	PGPKeyDBObjRef				pubKey, 
	const char					*defaultRoot, 
	NewDiskStatusFuncType		statusFunc, 
	void						*userValue, 
	PGPdiskDiskRef				*pDisk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskSetCreateDiskWithPub");

	return diskFunc(diskSet, path, blocksDisk, algorithm, pubKey, 
		defaultRoot, statusFunc, userValue, pDisk);
}

PGPError 
PGPdiskDiskSetUnmountTimedOut(PGPdiskDiskSetRef diskSet, PGPBoolean isForced)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskSetUnmountTimedOut");

	return diskFunc(diskSet, isForced);
}

PGPError 
PGPdiskDiskSetUnmountAll(PGPdiskDiskSetRef diskSet, PGPBoolean isForced)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskSetUnmountAll");

	return diskFunc(diskSet, isForced);
}

PGPError 
PGPdiskDiskSetWipeAllUsers(PGPdiskDiskSetRef diskSet)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskSetWipeAllUsers");

	return diskFunc(diskSet);
}

PGPError 
PGPdiskUpdateDiskSet(PGPdiskDiskSetRef diskSet)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskUpdateDiskSet");

	return diskFunc(diskSet);
}

/* PGPdiskDiskIterRef functions */

PGPError 
PGPdiskNewDiskIter(PGPdiskDiskSetRef diskSet, PGPdiskDiskIterRef *pDiskIter)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskNewDiskIter");

	return diskFunc(diskSet, pDiskIter);
}

void 
PGPdiskFreeDiskIter(PGPdiskDiskIterRef diskIter)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskFreeDiskIter");

	diskFunc(diskIter);
}

PGPdiskDiskSetRef 
PGPdiskPeekDiskIterDiskSet(PGPdiskDiskIterRef diskIter)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskPeekDiskIterDiskSet");

	return (PGPdiskDiskSetRef) diskFunc(diskIter);
}

PGPUInt32 
PGPdiskDiskIterCount(PGPdiskDiskIterRef diskIter)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskIterCount");

	return diskFunc(diskIter);
}

PGPInt32 
PGPdiskDiskIterIndex(PGPdiskDiskIterRef diskIter)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskIterIndex");

	return diskFunc(diskIter);
}

PGPBoolean 
PGPdiskDiskIterDisk(PGPdiskDiskIterRef diskIter, PGPdiskDiskRef *pDisk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskIterDisk");

	return diskFunc(diskIter, pDisk);
}

PGPBoolean 
PGPdiskDiskIterMove(
	PGPdiskDiskIterRef	diskIter, 
	PGPInt32			relOffset, 
	PGPBoolean			fromStart, 
	PGPdiskDiskRef		*pDisk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskIterMove");

	return diskFunc(diskIter, relOffset, fromStart, pDisk);
}

PGPInt32 
PGPdiskDiskIterSeek(PGPdiskDiskIterRef diskIter, PGPdiskDiskRef disk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskIterSeek");

	return diskFunc(diskIter, disk);
}

PGPBoolean 
PGPdiskDiskIterNext(PGPdiskDiskIterRef diskIter, PGPdiskDiskRef *pDisk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskIterNext");

	return diskFunc(diskIter, pDisk);
}

PGPBoolean 
PGPdiskDiskIterPrev(PGPdiskDiskIterRef diskIter, PGPdiskDiskRef *pDisk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskIterPrev");

	return diskFunc(diskIter, pDisk);
}

void 
PGPdiskDiskIterRewind(PGPdiskDiskIterRef diskIter)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskIterRewind");

	diskFunc(diskIter);
}

PGPBoolean 
PGPdiskDiskIterSearchOnPath(
	PGPdiskDiskIterRef	diskIter, 
	const char			*path, 
	PGPdiskDiskRef		*pDisk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskIterSearchOnPath");

	return diskFunc(diskIter, path, pDisk);
}

PGPBoolean 
PGPdiskDiskIterSearchOnRoot(
	PGPdiskDiskIterRef	diskIter, 
	const char			*root, 
	PGPdiskDiskRef		*pDisk)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskDiskIterSearchOnRoot");

	return diskFunc(diskIter, root, pDisk);
}

/* Miscellaneous functions */

void 
PGPdiskPrefsChanged(PGPdiskContextRef context, PGPPrefRef prefs)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskPrefsChanged");

	diskFunc(context, prefs);
}

PGPBoolean 
PGPdiskAreDisksTimedOut()
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskAreDisksTimedOut");

	return diskFunc();
}

PGPBoolean 
PGPdiskIsPathMountedPGPdisk(const char *path)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskIsPathMountedPGPdisk");

	return diskFunc(path);
}

PGPBoolean 
PGPdiskIsRootMountedPGPdisk(const char *root)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskIsRootMountedPGPdisk");

	return diskFunc(root);
}

PGPUInt32 
PGPdiskGetNumDiskAlgorithms()
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskGetNumDiskAlgorithms");

	return diskFunc();
}

void 
PGPdiskGetDiskAlgorithmIdString(
	PGPdiskEncryptionAlgorithm	algorithm, 
	char						*idString, 
	PGPUInt32					availLength)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskGetDiskAlgorithmIdString");

	diskFunc(algorithm, idString, availLength);
}

void 
PGPdiskGetSupportedDiskAlgorithms(
	PGPdiskEncryptionAlgorithm	*pAlgArray,	
	PGPSize						availLength, 
	PGPSize						*pUsedLength)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskGetSupportedDiskAlgorithms");

	diskFunc(pAlgArray, availLength, pUsedLength);
}

PGPUInt32 
PGPdiskGetNeededRandomDataSize(PGPdiskEncryptionAlgorithm algorithm)
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskGetNeededRandomDataSize");

	return diskFunc(algorithm);
}

PGPError 
PGPdiskNotifyUserLogoff()
{
	DISKPROC	diskFunc	= (DISKPROC) GetProcAddress(PGPdiskEngineDll, 
		"PGPdiskNotifyUserLogoff");

	return diskFunc();
}
