/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	PGPconfig.c - PGPnet read/write configuration info

	$Id: pgpNetConfig.c,v 1.43 2002/08/06 20:10:28 dallen Exp $
____________________________________________________________________________*/

#include <windows.h>

#include "pgpNetConfig.h"

#include "pflPrefTypes.h"
#include "pgpMemoryMgr.h"
#include "pgpFileSpec.h"
#include "pgpMem.h"
#include "pgpKeys.h"
#include "pgpErrors.h"
#include "pgpEndianConversion.h"
#include "pgpPFLPriv.h"
#include "pgpBuild.h"

// macro definitions
#define CKERR		if (IsPGPError (err)) goto done


//	______________________________________________
//
//  convert expiration prefs to form used by IKE

static PGPError
sComputeIkeExpirationPrefs (
		PGPContextRef		context,
		PPNCONFIG			pPNConfig)
{
	pPNConfig->IkeExpirationPrefs.pref = kPGPike_PF_Expiration;

	if (pPNConfig->bIkeKByteExpiration)
		pPNConfig->IkeExpirationPrefs.u.expiration.kbLifeTimeIKE =
			pPNConfig->uIkeKByteExpiration;
	else
		pPNConfig->IkeExpirationPrefs.u.expiration.kbLifeTimeIKE = 0;

	if (pPNConfig->bIkeTimeExpiration)
		pPNConfig->IkeExpirationPrefs.u.expiration.secLifeTimeIKE =
			pPNConfig->uIkeTimeExpiration;
	else
		pPNConfig->IkeExpirationPrefs.u.expiration.secLifeTimeIKE = 0;

	if (pPNConfig->bIpsecKByteExpiration)
		pPNConfig->IkeExpirationPrefs.u.expiration.kbLifeTimeIPSEC =
			pPNConfig->uIpsecKByteExpiration;
	else
		pPNConfig->IkeExpirationPrefs.u.expiration.kbLifeTimeIPSEC = 0;

	if (pPNConfig->bIpsecTimeExpiration)
		pPNConfig->IkeExpirationPrefs.u.expiration.secLifeTimeIPSEC =
			pPNConfig->uIpsecTimeExpiration;
	else
		pPNConfig->IkeExpirationPrefs.u.expiration.secLifeTimeIPSEC = 0;

	return kPGPError_NoErr;
}

//	______________________________________________
//
//  load algorithm prefs into structure used by IKE

PGPError
PGPnetGetIkeAlgorithmPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PPNCONFIG			pPNConfig)
{
	PGPError	err;

	pPNConfig->IkeAlgorithmPrefs.pref = kPGPike_PF_AllowedAlgorithms;

	// get cipher algorithm booleans
	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowCipherCAST,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.cast5);
	CKERR;

	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowCipher3DES,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.tripleDES);
	CKERR;

	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowCipherDES56,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.singleDES);
	CKERR;

	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowCipherNone,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.espNULL);
	CKERR;

	// get hash algorithm booleans
	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowHashSHA1,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.sha1);
	CKERR;

	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowHashMD5,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.md5);
	CKERR;

	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowHashNone,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.noAuth);
	CKERR;

	// get compression algorithm booleans
	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowCompLZS,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.lzs);
	CKERR;

	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowCompDeflate,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.deflate);
	CKERR;

	// get Diffie-Hellman algorithm booleans
	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowDH768,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.modpOne768);
	CKERR;

	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowDH1024,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.modpTwo1024);
	CKERR;

	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowDH1536,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.modpFive1536);
	CKERR;

	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowEC163,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.ec2n163 );
	CKERR;

	err = PGPGetPrefBoolean (prefref,
		kPGPNetPrefAllowEC283,
		&pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.ec2n283 );
	CKERR;

done :
	return err;
}


//	______________________________________________
//
//  save algorithm prefs from structure used by IKE

static PGPError
sSetIkeAlgorithmPrefs (
		PGPPrefRef			prefref,
		PPNCONFIG			pPNConfig)
{
	PGPError	err		= kPGPError_NoErr;

	// set cipher algorithm booleans
	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowCipherCAST,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.cast5);
	CKERR;

	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowCipher3DES,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.tripleDES);
	CKERR;

	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowCipherDES56,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.singleDES);
	CKERR;

	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowCipherNone,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.espNULL);
	CKERR;

	// set hash algorithm booleans
	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowHashSHA1,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.sha1);
	CKERR;

	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowHashMD5,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.md5);
	CKERR;

	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowHashNone,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.noAuth);
	CKERR;

	// set compression algorithm booleans
	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowCompLZS,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.lzs);
	CKERR;

	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowCompDeflate,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.deflate);
	CKERR;

	// set Diffie-Hellman algorithm booleans
	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowDH768,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.modpOne768);
	CKERR;

	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowDH1024,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.modpTwo1024);
	CKERR;

	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowDH1536,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.modpFive1536);
	CKERR;

	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowEC163,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.ec2n163 );
	CKERR;

	err = PGPSetPrefBoolean (prefref,
		kPGPNetPrefAllowEC283,
		pPNConfig->IkeAlgorithmPrefs.u.allowedAlgorithms.ec2n283 );
	CKERR;

done :
	return err;
}


//	______________________________________________
//
//  load IKE proposal prefs into structure used by IKE

PGPError
PGPnetGetIkeProposalPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PPNCONFIG			pPNConfig)
{
	PGPError			err		= kPGPError_NoErr;

	PGPSize				size;
	PGPUInt32			u, uNumProposals;
	PGPikeTransform*	pIke;

	PGPNetPrefIKEProposalEntry*		prefEntry;

	err = PGPGetNetIKEProposalPrefs (
				prefref, &prefEntry, &uNumProposals); CKERR;

	pPNConfig->IkeIkeProposalPrefs.pref = kPGPike_PF_IKEProposals;
	pPNConfig->
		IkeIkeProposalPrefs.u.ikeProposals.numTransforms = uNumProposals;

	if (uNumProposals > 0)
	{
		size = uNumProposals * sizeof(PGPikeTransform);
		pPNConfig->IkeIkeProposalPrefs.u.ikeProposals.t =
				PGPNewData (PGPPeekContextMemoryMgr (context),
							size, kPGPMemoryMgrFlags_Clear);

		if (pPNConfig->IkeIkeProposalPrefs.u.ikeProposals.t)
		{
			pIke = pPNConfig->IkeIkeProposalPrefs.u.ikeProposals.t;

			for (u=0; u<uNumProposals; u++)
			{
				pIke[u].authMethod	= prefEntry[u].authMethod;
				pIke[u].hash		= prefEntry[u].hash;
				pIke[u].cipher		= prefEntry[u].cipher;
				pIke[u].groupID		= prefEntry[u].groupID;
			}
		}
		else
			err = kPGPError_OutOfMemory;

		PGPFreeData(prefEntry);
	}

done :
	return err;
}


//	______________________________________________
//
//  save IKE proposal prefs from structure used by IKE

static PGPError
sSetIkeProposalPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PPNCONFIG			pPNConfig)
{
	PGPError			err		= kPGPError_NoErr;


	PGPSize				size;
	PGPUInt32			u, uNumProposals;
	PGPikeTransform*	pIke;

	PGPNetPrefIKEProposalEntry*		prefEntry;

	uNumProposals =
		pPNConfig->IkeIkeProposalPrefs.u.ikeProposals.numTransforms;

	if (uNumProposals > 0)
	{
		size = uNumProposals * sizeof(PGPNetPrefIKEProposalEntry);

		prefEntry = PGPNewData (PGPPeekContextMemoryMgr (context),
							size, kPGPMemoryMgrFlags_Clear);

		if (prefEntry)
		{
			pIke = pPNConfig->IkeIkeProposalPrefs.u.ikeProposals.t;

			for (u=0; u<uNumProposals; u++)
			{
				prefEntry[u].authMethod		= pIke[u].authMethod;
				prefEntry[u].hash			= pIke[u].hash;
				prefEntry[u].cipher			= pIke[u].cipher;
				prefEntry[u].groupID		= pIke[u].groupID;
			}

			err = PGPSetNetIKEProposalPrefs (
						prefref, prefEntry, uNumProposals);
			PGPFreeData (prefEntry);
		}
		else
			err = kPGPError_OutOfMemory;
	}

	return err;
}


//	______________________________________________
//
//  load IPSEC proposal prefs into structure used by IKE

PGPError
PGPnetGetIpsecProposalPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PPNCONFIG			pPNConfig)
{
	PGPError			err		= kPGPError_NoErr;

	PGPSize				size;
	PGPUInt32			u, uNumProposals;
	PGPipsecTransform*	pIpsec;

	PGPNetPrefIPSECProposalEntry*		prefEntry;

	err = PGPGetPrefNumber (
				prefref, kPGPnetPrefIPSECGroupID,
				&pPNConfig->IkeIpsecGroupID); CKERR;

	err = PGPGetNetIPSECProposalPrefs (
				prefref, &prefEntry, &uNumProposals); CKERR;

	pPNConfig->IkeIpsecProposalPrefs.pref = kPGPike_PF_IPSECProposals;
	pPNConfig->
		IkeIpsecProposalPrefs.u.ipsecProposals.numTransforms = uNumProposals;

	if (uNumProposals > 0)
	{
		size = uNumProposals * sizeof(PGPipsecTransform);
		pPNConfig->IkeIpsecProposalPrefs.u.ipsecProposals.t =
				PGPNewData (PGPPeekContextMemoryMgr (context),
							size, kPGPMemoryMgrFlags_Clear);

		if (pPNConfig->IkeIpsecProposalPrefs.u.ipsecProposals.t)
		{
			pIpsec = pPNConfig->IkeIpsecProposalPrefs.u.ipsecProposals.t;

			for (u=0; u<uNumProposals; u++)
			{
				pIpsec[u].useESP			= prefEntry[u].useESP;
				pIpsec[u].esp.cipher		= prefEntry[u].espCipher;
				pIpsec[u].esp.authAttr		= prefEntry[u].espHash;

				pIpsec[u].useAH				= prefEntry[u].useAH;
				pIpsec[u].ah.authAlg		= prefEntry[u].ahAuthAlg;
				pIpsec[u].ah.authAttr		= prefEntry[u].ahHash;

				pIpsec[u].useIPCOMP			= prefEntry[u].useIPCOMP;
				pIpsec[u].ipcomp.compAlg	= prefEntry[u].ipcomp;

				pIpsec[u].groupID			= pPNConfig->IkeIpsecGroupID;
			}
		}
		else
			err = kPGPError_OutOfMemory;

		PGPFreeData(prefEntry);
	}

done :
	return err;
}


//	______________________________________________
//
//  save IPSEC proposal prefs from structure used by IKE

static PGPError
sSetIpsecProposalPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PPNCONFIG			pPNConfig)
{
	PGPError			err		= kPGPError_NoErr;

	PGPSize				size;
	PGPUInt32			u, uNumProposals;
	PGPipsecTransform*	pIpsec;

	PGPNetPrefIPSECProposalEntry*		prefEntry;

	uNumProposals =
		pPNConfig->IkeIpsecProposalPrefs.u.ipsecProposals.numTransforms;

	if (uNumProposals > 0)
	{
		size = uNumProposals * sizeof(PGPNetPrefIPSECProposalEntry);

		prefEntry = PGPNewData (PGPPeekContextMemoryMgr (context),
							size, kPGPMemoryMgrFlags_Clear);

		if (prefEntry)
		{
			pIpsec = pPNConfig->IkeIpsecProposalPrefs.u.ipsecProposals.t;

			for (u=0; u<uNumProposals; u++)
			{
				prefEntry[u].useESP			= pIpsec[u].useESP;
				prefEntry[u].espCipher		= pIpsec[u].esp.cipher;
				prefEntry[u].espHash		= pIpsec[u].esp.authAttr;
				prefEntry[u].useAH			= pIpsec[u].useAH;
				prefEntry[u].ahAuthAlg		= pIpsec[u].ah.authAlg;
				prefEntry[u].ahHash			= pIpsec[u].ah.authAttr;
				prefEntry[u].useIPCOMP		= pIpsec[u].useIPCOMP;
				prefEntry[u].ipcomp			= pIpsec[u].ipcomp.compAlg;
			}

			err = PGPSetNetIPSECProposalPrefs (
						prefref, prefEntry, uNumProposals);
			PGPFreeData (prefEntry);
		}
		else
			err = kPGPError_OutOfMemory;
	}

	return err;
}


//	____________________________________
//
//	load configuration information from prefs file

PGPError
PGPnetLoadConfiguration (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		PPNCONFIG		pPNConfig)
{
	PGPError			err;
	PGPUInt32			u;
	PGPSize				size;
	PGPByte*			pbyte;

	if (!PGPPrefRefIsValid (prefref))
		return kPGPError_BadParams;

	pgpFillMemory(pPNConfig, sizeof(PNCONFIG), '\0');

	// get booleans
	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefEnablePGPnet,
					&pPNConfig->bPGPnetEnabled); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefEnableVPN,
					&pPNConfig->bPGPnetVPNEnabled); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefEnableFirewall,
					&pPNConfig->bPGPnetFirewallEnabled); CKERR;

	err = PGPGetPrefFlags (prefref,
					kPGPNetPrefEnablePGPnet, &u); CKERR;
	pPNConfig->bPGPnetEnabledLocked = (u & kPGPPrefFlags_DisableGUI);

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefRequireValidKeyConfigured,
					&pPNConfig->bRequireValidKeyConfigured); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefRequireValidKeyUnconfigured,
					&pPNConfig->bRequireValidKeyUnconfigured); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefReplayDetection,
					&pPNConfig->bReplayDetection); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefEnablePassphraseCache,
					&pPNConfig->bCachePassphrases); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefExpertMode,
					&pPNConfig->bExpertMode); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefFirstExecution,
					&pPNConfig->bFirstExecution); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefWarnReSharedSecret,
					&pPNConfig->bWarnReSharedSecret); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefEnableIKEKByteExpiration,
					&pPNConfig->bIkeKByteExpiration); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefEnableIKETimeExpiration,
					&pPNConfig->bIkeTimeExpiration); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefEnableIPSECKByteExpiration,
					&pPNConfig->bIpsecKByteExpiration); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefEnableIPSECTimeExpiration,
					&pPNConfig->bIpsecTimeExpiration); CKERR;

#if PGP_FREEWARE
	pPNConfig->bIntrusionDetection = FALSE;
#else
	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefUseIntrusionDetection,
					&pPNConfig->bIntrusionDetection); CKERR;
#endif

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefAutoBlockAttacks,
					&pPNConfig->bAutoBlockAttacks); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefBlockUntilRemoved,
					&pPNConfig->bBlockUntilRemoved); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefAttackSound,
					&pPNConfig->bPlayAttackSound); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefAttackNotify,
					&pPNConfig->bTrayNotify); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefAttackEmailNotify,
					&pPNConfig->bEmailNotify); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefAttackDialog,
					&pPNConfig->bAttackDialog); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefUseLearnMode,
					&(pPNConfig->bFireLearnModeOn)); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefLogAllowed,
					&(pPNConfig->bLogAllowedTraffic)); CKERR;

	err = PGPGetPrefBoolean (prefref,
					kPGPNetPrefLogBlocked,
					&(pPNConfig->bLogBlockedTraffic)); CKERR;

	// get numbers
	err = PGPGetPrefNumber (prefref,
					kPGPnetPrefUnconfiguredMode,
					&pPNConfig->uUnconfiguredMode); CKERR;

	err = PGPGetPrefNumber (prefref,
					kPGPNetPrefIKEKByteExpiration,
					&pPNConfig->uIkeKByteExpiration); CKERR;

	err = PGPGetPrefNumber (prefref,
					kPGPNetPrefIKETimeExpiration,
					&pPNConfig->uIkeTimeExpiration); CKERR;

	err = PGPGetPrefNumber (prefref,
					kPGPNetPrefIPSECKByteExpiration,
					&pPNConfig->uIpsecKByteExpiration); CKERR;

	err = PGPGetPrefNumber (prefref,
					kPGPNetPrefIPSECTimeExpiration,
					&pPNConfig->uIpsecTimeExpiration); CKERR;

	err = PGPGetPrefNumber (prefref,
					kPGPnetPrefBlockMinutes,
					&pPNConfig->uBlockMinutes); CKERR;

	err = PGPGetPrefNumber (prefref,
					kPGPnetPrefFirewallRuleSet,
					&pPNConfig->uRuleSet); CKERR;

	err = PGPGetPrefNumber (prefref,
					kPGPNetPrefTrustSubnetOption,
					&pPNConfig->uSubnetTrustOption); CKERR;

	// Get strings
	err = PGPGetPrefStringAlloc (prefref,
					kPGPNetPrefAttackNotifyEmail,
					&pPNConfig->szEmailAddress); CKERR;

	err = PGPGetPrefStringAlloc (prefref,
					kPGPNetPrefAttackNotifySMTPServer,
					&pPNConfig->szMailServer); CKERR;

	// get keyrings
	err = PGPGetPrefStringBuffer (prefref,
					kPGPNetPrefPublicKeyringFile,
					sizeof(pPNConfig->szPublicKeyringFile),
					pPNConfig->szPublicKeyringFile);

	err = PGPGetPrefStringBuffer (prefref,
					kPGPNetPrefPrivateKeyringFile,
					sizeof(pPNConfig->szPrivateKeyringFile),
					pPNConfig->szPrivateKeyringFile);

	// get PGP Authentication keyid
	err = PGPGetPrefData (prefref,
					kPGPNetPrefPGPAuthKeyID,
					&size,
					&pbyte);
	if (IsntPGPError (err))
	{
		pgpCopyMemory (pbyte, &pPNConfig->keyidPGPAuth, size);
		PGPFreeData(pbyte);
		pPNConfig->bUsePGPAuthKey = TRUE;
	}
	else
		pPNConfig->bUsePGPAuthKey = FALSE;

	// get X509 Authentication keyid
	err = PGPGetPrefData (prefref,
					kPGPNetPrefX509AuthKeyID,
					&size,
					&pbyte);
	if (IsntPGPError (err))
	{
		pgpCopyMemory (pbyte, &pPNConfig->keyidX509Auth, size);
		PGPFreeData(pbyte);

		err = PGPGetPrefData (prefref,
					kPGPNetPrefX509AuthCertIASN,
					&pPNConfig->uX509AuthCertIASNLength,
					&pPNConfig->pX509AuthCertIASN); CKERR;

		pPNConfig->bUseX509AuthKey = TRUE;
	}
	else
		pPNConfig->bUseX509AuthKey = FALSE;

	// fill host entry structure
	err = PGPGetNetHostPrefs (prefref,
					&pPNConfig->pHostList,
					&pPNConfig->uHostCount); CKERR;

	err = PGPGetPrefFlags (prefref,
					kPGPNetPrefHostArray, &u); CKERR;
	pPNConfig->bPGPnetHostListLocked = (u & kPGPPrefFlags_DisableGUI);

	// fill blocked host entry structure
	err = PGPGetNetBlockedPrefs (prefref,
					&pPNConfig->pBlockedList,
					&pPNConfig->uBlockedCount); CKERR;

	err = PGPGetPrefFlags (prefref,
					kPGPNetPrefBlockedHostArray, &u); CKERR;
	pPNConfig->bPGPnetBlockedListLocked = (u & kPGPPrefFlags_DisableGUI);

	// fill trusted host entry structure
	err = PGPGetNetTrustedPrefs (prefref,
					&pPNConfig->pTrustedList,
					&pPNConfig->uTrustedCount); CKERR;

	err = PGPGetPrefFlags (prefref,
					kPGPNetPrefTrustedHostArray, &u); CKERR;
	pPNConfig->bPGPnetTrustedListLocked = (u & kPGPPrefFlags_DisableGUI);

	// fill firewall rule structure
	if (pPNConfig->uRuleSet == kPGPNetFirewallRuleSet_Custom)
	{
		err = PGPGetNetFirewallRuleSet (prefref,
				&pPNConfig->pRuleList,
				&pPNConfig->uRuleCount); CKERR;
	}
	else
	{
		err = PGPGetNetDefaultFirewallRuleSet (prefref, 
				pPNConfig->uRuleSet, 
				&pPNConfig->pRuleList,
				&pPNConfig->uRuleCount); CKERR;
	}

	// fill in IKE prefs structures
	err = PGPnetGetIkeAlgorithmPrefs (context, prefref, pPNConfig); CKERR;
	err = PGPnetGetIkeProposalPrefs (context, prefref, pPNConfig); CKERR;
	err = PGPnetGetIpsecProposalPrefs (context, prefref, pPNConfig); CKERR;
	err = sComputeIkeExpirationPrefs (context, pPNConfig); CKERR;

done:
	return err;
}


PGPError
PGPnetLoadKnownConfig (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		PPNKNOWNCONFIG	pPNKnownConfig)
{
	PGPError					err;

	if (!PGPPrefRefIsValid (prefref))
		return kPGPError_BadParams;

	/* get auto rule list */
	err = PGPGetNetKnownRuleSet(prefref,
								kPGPNetPrefAutoRuleArray,
								&pPNKnownConfig->pAutoRuleList,
								&pPNKnownConfig->uAutoRuleCount); CKERR;
	if (pPNKnownConfig->uAutoRuleCount == 0)
	{
		PGPGetNetDefaultKnownRuleSet(prefref,
									 kPGPNetFirewallRuleSet_Auto,
									 &pPNKnownConfig->pAutoRuleList,
									 &pPNKnownConfig->uAutoRuleCount); CKERR;									
	}

	/* get known rule list */
	err = PGPGetNetKnownRuleSet	(prefref,
								 kPGPNetPrefKnownRuleArray,
								&pPNKnownConfig->pKnownRuleList,
								&pPNKnownConfig->uKnownRuleCount); CKERR;
	if (pPNKnownConfig->uKnownRuleCount == 0)
	{
		PGPGetNetDefaultKnownRuleSet(prefref,
									 kPGPNetFirewallRuleSet_Known,
									&pPNKnownConfig->pKnownRuleList,
									&pPNKnownConfig->uKnownRuleCount); CKERR;									
	}
	
	/* get smart rule list */
	err = PGPGetNetKnownRuleSet	(prefref,
								 kPGPNetPrefSmartRuleArray,
								&pPNKnownConfig->pSmartRuleList,
								&pPNKnownConfig->uSmartRuleCount); CKERR;
	if (pPNKnownConfig->uSmartRuleCount == 0)
	{
		PGPGetNetDefaultKnownRuleSet(prefref,
									 kPGPNetFirewallRuleSet_Smart,
									&pPNKnownConfig->pSmartRuleList,
									&pPNKnownConfig->uSmartRuleCount); CKERR;									
	}

done:
	return err;
}

//	____________________________________
//
//	free all the points in the config structure

PGPError
PGPnetFreeConfiguration (
		PPNCONFIG		pPNConfig)
{
	// if host list exists, dispose of it
	if (pPNConfig->pHostList)
	{
		PGPFreeData (pPNConfig->pHostList);
		pPNConfig->pHostList = NULL;
	}

	// if blocked list exists, dispose of it
	if (pPNConfig->pBlockedList)
	{
		PGPFreeData (pPNConfig->pBlockedList);
		pPNConfig->pBlockedList = NULL;
	}

	// if trusted list exists, dispose of it
	if (pPNConfig->pTrustedList)
	{
		PGPFreeData (pPNConfig->pTrustedList);
		pPNConfig->pTrustedList = NULL;
	}

	// if rule list exists, dispose of it
	if (pPNConfig->pRuleList)
	{
		PGPFreeData (pPNConfig->pRuleList);
		pPNConfig->pRuleList = NULL;
	}

	// if X509 AuthCert IASN exists, dispose of it
	if (pPNConfig->pX509AuthCertIASN)
	{
		PGPFreeData (pPNConfig->pX509AuthCertIASN);
		pPNConfig->pX509AuthCertIASN = NULL;
	}

	// dispose of IKE prefs proposal arrays
	if (pPNConfig->IkeIkeProposalPrefs.u.ikeProposals.t)
	{
		PGPFreeData (pPNConfig->IkeIkeProposalPrefs.u.ikeProposals.t);
		pPNConfig->IkeIkeProposalPrefs.u.ikeProposals.t = NULL;
	}

	if (pPNConfig->IkeIpsecProposalPrefs.u.ipsecProposals.t)
	{
		PGPFreeData (pPNConfig->IkeIpsecProposalPrefs.u.ipsecProposals.t);
		pPNConfig->IkeIpsecProposalPrefs.u.ipsecProposals.t = NULL;
	}

	if (pPNConfig->szEmailAddress)
	{
		PGPFreeData (pPNConfig->szEmailAddress);
		pPNConfig->szEmailAddress = NULL;
	}

	if (pPNConfig->szMailServer)
	{
		PGPFreeData (pPNConfig->szMailServer);
		pPNConfig->szMailServer = NULL;
	}

	return kPGPError_NoErr;
}

PGPError
PGPnetFreeKnownConfig(
		PPNKNOWNCONFIG	pPNKnownConfig)
{
	// if rule list exists, dispose of it
	if (pPNKnownConfig->pAutoRuleList)
	{
		PGPFreeData (pPNKnownConfig->pAutoRuleList);
		pPNKnownConfig->pAutoRuleList = NULL;
		pPNKnownConfig->uAutoRuleCount = 0;
	}

	if (pPNKnownConfig->pKnownRuleList)
	{
		PGPFreeData (pPNKnownConfig->pKnownRuleList);
		pPNKnownConfig->pKnownRuleList = NULL;
		pPNKnownConfig->uKnownRuleCount = 0;
	}

	if (pPNKnownConfig->pSmartRuleList)
	{
		PGPFreeData (pPNKnownConfig->pSmartRuleList);
		pPNKnownConfig->pSmartRuleList = NULL;
		pPNKnownConfig->uSmartRuleCount = 0;
	}

	return kPGPError_NoErr;
}

//	____________________________________
//
//	reload configuration information from prefs file

PGPError
PGPnetReloadConfiguration (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		PPNCONFIG		pPNConfig)
{
	PGPError	err;

	err = PGPnetFreeConfiguration (pPNConfig);
	if (IsntPGPError (err))
		err = PGPnetLoadConfiguration (context, prefref, pPNConfig);

	return err;
}

PGPError
PGPnetReloadKnownConfig (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		PPNKNOWNCONFIG	pPNKnownConfig)
{
	PGPError					err;

	if (!PGPPrefRefIsValid (prefref))
		return kPGPError_BadParams;

	err = PGPnetFreeKnownConfig (pPNKnownConfig);
	if (IsntPGPError (err))
		err = PGPnetLoadKnownConfig (context, prefref, pPNKnownConfig);

	return err;
}

//	____________________________________
//
//	save configuration information to prefs file

PGPError
PGPnetSaveConfiguration (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		PPNCONFIG		pPNConfig)
{
	PGPError					err;

	if (!PGPPrefRefIsValid (prefref))
		return kPGPError_BadParams;

	// save booleans
	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefEnablePGPnet,
					pPNConfig->bPGPnetEnabled); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefEnableVPN,
					pPNConfig->bPGPnetVPNEnabled); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefEnableFirewall,
					pPNConfig->bPGPnetFirewallEnabled); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefRequireValidKeyConfigured,
					pPNConfig->bRequireValidKeyConfigured); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefRequireValidKeyUnconfigured,
					pPNConfig->bRequireValidKeyUnconfigured); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefReplayDetection,
					pPNConfig->bReplayDetection); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefEnablePassphraseCache,
					pPNConfig->bCachePassphrases); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefExpertMode,
					pPNConfig->bExpertMode); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefFirstExecution,
					pPNConfig->bFirstExecution); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefWarnReSharedSecret,
					pPNConfig->bWarnReSharedSecret); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefEnableIKEKByteExpiration,
					pPNConfig->bIkeKByteExpiration); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefEnableIKETimeExpiration,
					pPNConfig->bIkeTimeExpiration); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefEnableIPSECKByteExpiration,
					pPNConfig->bIpsecKByteExpiration); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefEnableIPSECTimeExpiration,
					pPNConfig->bIpsecTimeExpiration); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefUseIntrusionDetection,
					pPNConfig->bIntrusionDetection); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefAutoBlockAttacks,
					pPNConfig->bAutoBlockAttacks); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefBlockUntilRemoved,
					pPNConfig->bBlockUntilRemoved); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefAttackSound,
					pPNConfig->bPlayAttackSound); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefAttackNotify,
					pPNConfig->bTrayNotify); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefAttackEmailNotify,
					pPNConfig->bEmailNotify); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefUseLearnMode,
					pPNConfig->bFireLearnModeOn); CKERR;

	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefLogAllowed,
					pPNConfig->bLogAllowedTraffic); CKERR;
	
	err = PGPSetPrefBoolean (prefref,
					kPGPNetPrefLogBlocked,
					pPNConfig->bLogBlockedTraffic); CKERR;

	// save numbers
	err = PGPSetPrefNumber (prefref,
					kPGPnetPrefUnconfiguredMode,
					pPNConfig->uUnconfiguredMode); CKERR;

	err = PGPSetPrefNumber (prefref,
					kPGPNetPrefIKEKByteExpiration,
					pPNConfig->uIkeKByteExpiration); CKERR;

	err = PGPSetPrefNumber (prefref,
					kPGPNetPrefIKETimeExpiration,
					pPNConfig->uIkeTimeExpiration); CKERR;

	err = PGPSetPrefNumber (prefref,
					kPGPNetPrefIPSECKByteExpiration,
					pPNConfig->uIpsecKByteExpiration); CKERR;

	err = PGPSetPrefNumber (prefref,
					kPGPNetPrefIPSECTimeExpiration,
					pPNConfig->uIpsecTimeExpiration); CKERR;

	err = PGPSetPrefNumber (prefref,
					kPGPnetPrefIPSECGroupID,
					pPNConfig->IkeIpsecGroupID); CKERR;

	err = PGPSetPrefNumber (prefref,
					kPGPnetPrefBlockMinutes,
					pPNConfig->uBlockMinutes); CKERR;

	err = PGPSetPrefNumber (prefref,
					kPGPnetPrefFirewallRuleSet,
					pPNConfig->uRuleSet); CKERR;

	err = PGPSetPrefNumber (prefref,
					kPGPNetPrefTrustSubnetOption,
					pPNConfig->uSubnetTrustOption); CKERR; 

	// save strings
	err = PGPSetPrefString (prefref,
					kPGPNetPrefAttackNotifyEmail,
					pPNConfig->szEmailAddress); CKERR;

	err = PGPSetPrefString (prefref,
					kPGPNetPrefAttackNotifySMTPServer,
					pPNConfig->szMailServer); CKERR;

	// save keyrings
	err = PGPSetPrefString (prefref,
					kPGPNetPrefPublicKeyringFile,
					pPNConfig->szPublicKeyringFile);

	err = PGPSetPrefString (prefref,
					kPGPNetPrefPrivateKeyringFile,
					pPNConfig->szPrivateKeyringFile);

	// save PGP authentication key info
	if (pPNConfig->bUsePGPAuthKey)
	{
		err = PGPSetPrefData (prefref,
					kPGPNetPrefPGPAuthKeyID,
					sizeof(PGPKeyID),
					&pPNConfig->keyidPGPAuth);
	}

	// save X509 authentication key info
	if (pPNConfig->bUseX509AuthKey)
	{
		err = PGPSetPrefData (prefref,
					kPGPNetPrefX509AuthKeyID,
					sizeof(PGPKeyID),
					&pPNConfig->keyidX509Auth);

		if (pPNConfig->uX509AuthCertIASNLength > 0)
		{
			err = PGPSetPrefData (prefref,
					kPGPNetPrefX509AuthCertIASN,
					pPNConfig->uX509AuthCertIASNLength,
					pPNConfig->pX509AuthCertIASN);
		}
	}

	// save host list
	err = PGPSetNetHostPrefs (prefref,
					pPNConfig->pHostList,
					pPNConfig->uHostCount); CKERR;

	// save blocked host list
	err = PGPSetNetBlockedPrefs (prefref,
					pPNConfig->pBlockedList,
					pPNConfig->uBlockedCount); CKERR;

	// save trusted host list
	err = PGPSetNetTrustedPrefs (prefref,
					pPNConfig->pTrustedList,
					pPNConfig->uTrustedCount); CKERR;

	// save firewall rule list
	if (pPNConfig->uRuleSet == kPGPNetFirewallRuleSet_Custom)
	{
		err = PGPSetNetFirewallRulePrefs (prefref,
						pPNConfig->pRuleList,
						pPNConfig->uRuleCount); CKERR;
	}

	// save IKE prefs
	err = sSetIkeAlgorithmPrefs (prefref, pPNConfig); CKERR;
	err = sSetIkeProposalPrefs (context, prefref, pPNConfig); CKERR;
	err = sSetIpsecProposalPrefs (context, prefref, pPNConfig); CKERR;

done:
	return err;
}

PGPError
PGPnetSaveKnownConfig (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		PPNKNOWNCONFIG	pPNKnownConfig)
{
	PGPError					err;

	if (!PGPPrefRefIsValid (prefref))
		return kPGPError_BadParams;

	err = PGPSetNetKnownRuleSet (prefref,
								 kPGPNetPrefAutoRuleArray,
								 pPNKnownConfig->pAutoRuleList,
								 pPNKnownConfig->uAutoRuleCount); CKERR;

	err = PGPSetNetKnownRuleSet	(prefref,
								 kPGPNetPrefKnownRuleArray,
								 pPNKnownConfig->pKnownRuleList,
								 pPNKnownConfig->uKnownRuleCount); CKERR;

	err = PGPSetNetKnownRuleSet (prefref,
								 kPGPNetPrefSmartRuleArray,
								 pPNKnownConfig->pSmartRuleList,
								 pPNKnownConfig->uSmartRuleCount); CKERR;

done:
	return err;
}


//	____________________________________
//
//	open the keyring files specified in the PNCONFIG structure

PGPError
PGPnetOpenKeyrings (
		PGPContextRef	context,
		PPNCONFIG		pPNConfig,
		UINT			uOpenFlags,
		PGPKeyDBRef*	pkeydb)
{
	PGPError		err				= kPGPError_FileNotFound;
	PGPFileSpecRef	filerefPriv		= NULL;
	PGPFileSpecRef	filerefPub		= NULL;

	if ((lstrlen (pPNConfig->szPublicKeyringFile) > 0) &&
		(lstrlen (pPNConfig->szPrivateKeyringFile) > 0))
	{
		err = PGPNewFileSpecFromFullPath (context,
					pPNConfig->szPublicKeyringFile, &filerefPub);
		if (IsPGPError (err))
			return err;

		err = PGPNewFileSpecFromFullPath (context,
					pPNConfig->szPrivateKeyringFile, &filerefPriv);
		if (IsPGPError (err))
			return err;

		err = PGPOpenKeyDBFile (context, uOpenFlags,
				filerefPub, filerefPriv, pkeydb);

		PGPFreeFileSpec (filerefPub);
		PGPFreeFileSpec (filerefPriv);
	}

	return err;
}


//	____________________________________
//
//	load configuration information from prefs file

PGPError
PGPnetAddPromiscuousHostToConfiguration (
		PGPContextRef	context,
		PPNCONFIG		pPNConfig,
		PGPUInt32		uIPAddressHost,
		PGPUInt32		uIPAddressLocal,
		PGPKeyDBObjRef	keyAuth,
		PGPKeyDBObjRef	sigAuth)
{
	PGPError				err			= kPGPError_NoErr;
	PGPSize					size;
	PGPNetPrefHostEntry*	pNewList;
	PGPUInt32				u;

	size = pPNConfig->uHostCount * sizeof (PGPNetPrefHostEntry);

	pNewList = PGPNewData (PGPPeekContextMemoryMgr (context),
							size + sizeof (PGPNetPrefHostEntry),
							kPGPMemoryMgrFlags_Clear);
	if (pNewList)
	{
		// copy old list to new
		if (pPNConfig->pHostList)
			pgpCopyMemory (pPNConfig->pHostList, pNewList, size);

		// fill in new list element
		u = pPNConfig->uHostCount;
		pNewList[u].hostType			= kPGPnetSecureHost;
		pNewList[u].ipAddrStart			= uIPAddressHost;
		pNewList[u].ipMaskEnd			= 0xFFFFFFFF;
		pNewList[u].childOf				= -1;
		strcpy (pNewList[u].hostName, "Untitled Host");
		pNewList[u].sharedSecret[0]		= '\0';
		pNewList[u].useAuthKeyR			= FALSE;
		pNewList[u].authCertIASNLengthR	= 0;

		if (PGPKeyDBObjRefIsValid (keyAuth))
		{
			err = PGPGetKeyID (keyAuth, &pNewList[u].authKeyIDR); CKERR;

			if (PGPKeyDBObjRefIsValid (sigAuth))
			{
				PGPGetKeyDBObjDataProperty (sigAuth,
						kPGPSigProperty_X509IASN, NULL, 0,
						&pNewList[u].authCertIASNLengthR);

				if (pNewList[u].authCertIASNLengthR > 0)
				{
					err = PGPGetKeyDBObjDataProperty (sigAuth,
							kPGPSigProperty_X509IASN,
							pNewList[u].authCertIASNR,
							sizeof(pNewList[u].authCertIASNR),
							&pNewList[u].authCertIASNLengthR);
					CKERR;
				}
			}
		}
	}
	else
		err = kPGPError_OutOfMemory;

done :
	if (IsntPGPError (err))
	{
		if (pPNConfig->pHostList)
			PGPFreeData (pPNConfig->pHostList);

		pPNConfig->pHostList = pNewList;
		pPNConfig->uHostCount++;
	}

	return err;
}


//	____________________________________
//
//	get authentication key and sig refs from keyset

PGPError
PGPnetGetConfiguredAuthKeys (
		PGPContextRef	context,
		PPNCONFIG		pPNConfig,
		PGPKeyDBRef		keydb,
		PGPKeyDBObjRef*	pkeyPGP,
		PGPKeyDBObjRef*	psigX509)
{
	PGPKeyListRef	keylist			= kInvalidPGPKeyListRef;
	PGPKeyIterRef	keyiter			= kInvalidPGPKeyIterRef;
	PGPKeySetRef	keysetX509		= kInvalidPGPKeySetRef;
	PGPKeyDBObjRef	key				= kInvalidPGPKeyDBObjRef;
	PGPKeyDBObjRef	keyX509			= kInvalidPGPKeyDBObjRef;
	PGPKeyDBObjRef	userid			= kInvalidPGPKeyDBObjRef;
	PGPKeyDBObjRef	sig				= kInvalidPGPKeyDBObjRef;
	PGPError		err				= kPGPError_NoErr;
	PGPByte*		pIASN			= NULL;
	PGPBoolean		bPGPKeyNotFound	= FALSE;

	PGPSize			size;

	PGPValidatePtr (pkeyPGP);
	PGPValidatePtr (psigX509);

	*pkeyPGP = kInvalidPGPKeyDBObjRef;
	*psigX509 = kInvalidPGPKeyDBObjRef;

	if (!PGPKeyDBRefIsValid (keydb))
		return kPGPError_BadParams;

	// get PGP key ref
	if (pPNConfig->bUsePGPAuthKey)
	{
		err = PGPFindKeyByKeyID (keydb, &pPNConfig->keyidPGPAuth, pkeyPGP);

		// if key is missing, we still want to look for X.509 key
		if (err == kPGPError_ItemNotFound)
		{
			err = kPGPError_NoErr;
			bPGPKeyNotFound = TRUE;
		}
		CKERR;
	}

	// get X.509 key
	if (pPNConfig->bUseX509AuthKey &&
		pPNConfig->pX509AuthCertIASN)
	{
		err = PGPFindKeyByKeyID (keydb, &pPNConfig->keyidX509Auth, &keyX509);
	}

	// get X.509 sig
	if (PGPKeyDBObjRefIsValid (keyX509))
	{
		err = PGPNewOneKeySet (keyX509, &keysetX509); CKERR;
		err = PGPOrderKeySet (keysetX509,
				kPGPKeyOrdering_Any, FALSE, &keylist); CKERR;
		err = PGPNewKeyIter (keylist, &keyiter); CKERR;

		err = PGPKeyIterNextKeyDBObj (keyiter,
				kPGPKeyDBObjType_Key, &key); CKERR;
		err = PGPKeyIterNextKeyDBObj (keyiter,
				kPGPKeyDBObjType_UserID, &userid); CKERR;
		while (userid)
		{
			PGPKeyIterNextKeyDBObj (keyiter,
					kPGPKeyDBObjType_Signature, &sig);

			while (	 PGPKeyDBObjRefIsValid (sig) &&
					!PGPKeyDBObjRefIsValid (*psigX509))
			{
				size = 0;
				PGPGetKeyDBObjDataProperty (sig,
						kPGPSigProperty_X509IASN, NULL, 0, &size);
				if (size == pPNConfig->uX509AuthCertIASNLength)
				{
					pIASN = PGPNewData (
							PGPPeekContextMemoryMgr (context),
							size, kPGPMemoryMgrFlags_Clear);
					if (pIASN)
					{
						err = PGPGetKeyDBObjDataProperty (sig,
								kPGPSigProperty_X509IASN, pIASN,
								size, &size); CKERR;

						if (pgpMemoryEqual (pIASN,
								pPNConfig->pX509AuthCertIASN, size))
						{
							*psigX509 = sig;
						}

						PGPFreeData (pIASN);
						pIASN = NULL;
					}
				}
				PGPKeyIterNextKeyDBObj (keyiter,
						kPGPKeyDBObjType_Signature, &sig);
			}
			PGPKeyIterNextKeyDBObj (keyiter,
					kPGPKeyDBObjType_UserID, &userid);
		}
	}

done :
	if (pIASN != NULL)
		PGPFreeData (pIASN);
	if (PGPKeySetRefIsValid (keysetX509))
		PGPFreeKeySet (keysetX509);
	if (PGPKeyIterRefIsValid (keyiter))
		PGPFreeKeyIter (keyiter);
	if (PGPKeyListRefIsValid (keylist))
		PGPFreeKeyList (keylist);

	if (IsntPGPError (err) && (bPGPKeyNotFound))
		err = kPGPError_ItemNotFound;

	return err;
}


//	____________________________________
//
//	get host authentication key and sig refs from keyset

PGPError
PGPnetGetConfiguredHostAuthKeys (
		PGPContextRef			context,
		PGPNetPrefHostEntry*	pHost,
		PGPKeyDBRef				keydb,
		PGPBoolean				bLocal,
		PGPKeyDBObjRef*			pkey,
		PGPKeyDBObjRef*			psig)
{
	PGPKeyListRef	keylist			= kInvalidPGPKeyListRef;
	PGPKeyIterRef	keyiter			= kInvalidPGPKeyIterRef;
	PGPKeySetRef	keysetX509		= kInvalidPGPKeySetRef;
	PGPKeyDBObjRef	key				= kInvalidPGPKeyDBObjRef;
	PGPKeyDBObjRef	userid			= kInvalidPGPKeyDBObjRef;
	PGPKeyDBObjRef	sig				= kInvalidPGPKeyDBObjRef;
	PGPError		err				= kPGPError_NoErr;

	PGPByte*		pIASN			= NULL;
	PGPSize			sizeIASN;
	PGPByte*		pHostIASN;
	PGPSize			sizeHostIASN;


	PGPValidatePtr (pkey);
	PGPValidatePtr (psig);

	*pkey = kInvalidPGPKeyDBObjRef;
	*psig = kInvalidPGPKeyDBObjRef;

	// if no authentication key, just return
	if (bLocal)
	{
		// if no authentication key, just return
		if (!pHost->useAuthKeyL)
			goto done;

		// first get key ref
		err = PGPFindKeyByKeyID (keydb, &pHost->authKeyIDL, pkey); CKERR;

		// if no IASN, must be PGP key
		if (pHost->authCertIASNLengthL == 0)
			goto done;

		sizeHostIASN = pHost->authCertIASNLengthL;
		pHostIASN = pHost->authCertIASNL;
	}
	else
	{
		// if no authentication key, just return
		if (!pHost->useAuthKeyR)
			goto done;

		// first get key ref
		err = PGPFindKeyByKeyID (keydb, &pHost->authKeyIDR, pkey); CKERR;

		// if no IASN, must be PGP key
		if (pHost->authCertIASNLengthR == 0)
			goto done;

		sizeHostIASN = pHost->authCertIASNLengthR;
		pHostIASN = pHost->authCertIASNR;
	}
	
	if (PGPKeyDBObjRefIsValid (*pkey))
	{
		err = PGPNewOneKeySet (*pkey, &keysetX509); CKERR;
		err = PGPOrderKeySet (keysetX509,
				kPGPKeyOrdering_Any, FALSE, &keylist); CKERR;
		err = PGPNewKeyIter (keylist, &keyiter); CKERR;

		err = PGPKeyIterNextKeyDBObj (keyiter,
				kPGPKeyDBObjType_Key, &key); CKERR;
		err = PGPKeyIterNextKeyDBObj (keyiter,
				kPGPKeyDBObjType_UserID, &userid); CKERR;
		while (userid)
		{
			PGPKeyIterNextKeyDBObj (keyiter,
					kPGPKeyDBObjType_Signature, &sig);

			while (	PGPKeyDBObjRefIsValid (sig) &&
					!PGPKeyDBObjRefIsValid (*psig))
			{
				sizeIASN = 0;
				PGPGetKeyDBObjDataProperty (sig,
						kPGPSigProperty_X509IASN, NULL, 0, &sizeIASN);
				if (sizeIASN == sizeHostIASN)
				{
					pIASN = PGPNewData (
							PGPPeekContextMemoryMgr (context),
							sizeIASN, kPGPMemoryMgrFlags_Clear);
					if (pIASN)
					{
						err = PGPGetKeyDBObjDataProperty (sig,
								kPGPSigProperty_X509IASN, pIASN,
								sizeIASN, &sizeIASN); CKERR;

						if (pgpMemoryEqual (pIASN, pHostIASN, sizeIASN))
							*psig = sig;

						PGPFreeData (pIASN);
						pIASN = NULL;
					}
				}
				PGPKeyIterNextKeyDBObj (keyiter,
						kPGPKeyDBObjType_Signature, &sig);
			}
			PGPKeyIterNextKeyDBObj (keyiter,
					kPGPKeyDBObjType_UserID, &userid);
		}

		if (!PGPKeyDBObjRefIsValid (*psig))
			err = kPGPError_ItemNotFound;
	}

done :
	if (pIASN != NULL)
		PGPFreeData (pIASN);
	if (PGPKeySetRefIsValid (keysetX509))
		PGPFreeKeySet (keysetX509);
	if (PGPKeyIterRefIsValid (keyiter))
		PGPFreeKeyIter (keyiter);
	if (PGPKeyListRefIsValid (keylist))
		PGPFreeKeyList (keylist);

	return err;
}

