/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	PGPnetConfig.h - PGPnet configuration utilities header


	$Id: pgpNetConfig.h,v 1.24 2002/08/06 20:10:28 dallen Exp $
____________________________________________________________________________*/

#ifndef _Included_pgpNetConfig_h
#define _Included_pgpNetConfig_h

// includes
#include "pgpPubTypes.h"
#include "pgpUtilities.h"
#include "pgpKeys.h"
#include "pgpIke.h"

#include "pgpNetPrefs.h"

PGP_BEGIN_C_DECLARATIONS

// typedefs
typedef struct {
	PGPBoolean				bPGPnetEnabledLocked;
	PGPBoolean				bPGPnetEnabled;
	PGPBoolean				bPGPnetVPNEnabled;

	PGPUInt32				uUnconfiguredMode;
	PGPBoolean				bRequireValidKeyConfigured;
	PGPBoolean				bRequireValidKeyUnconfigured;
	PGPBoolean				bReplayDetection;
	PGPBoolean				bCachePassphrases;
	PGPBoolean				bExpertMode;
	PGPBoolean				bFirstExecution;
	PGPBoolean				bWarnReSharedSecret;

	PGPBoolean				bIntrusionDetection;
	PGPBoolean				bAutoBlockAttacks;
	PGPBoolean				bAttackDialog;
	PGPBoolean				bBlockUntilRemoved;
	PGPUInt32				uBlockMinutes;
	PGPBoolean				bPlayAttackSound;
	PGPBoolean				bTrayNotify;
	PGPBoolean				bEmailNotify;
	char *					szEmailAddress;
	char *					szMailServer;
	PGPUInt32				uSubnetTrustOption;	

	PGPBoolean				bIkeKByteExpiration;
	PGPBoolean				bIkeTimeExpiration;
	PGPBoolean				bIpsecKByteExpiration;
	PGPBoolean				bIpsecTimeExpiration;

	PGPUInt32				uIkeKByteExpiration;
	PGPUInt32				uIkeTimeExpiration;
	PGPUInt32				uIpsecKByteExpiration;
	PGPUInt32				uIpsecTimeExpiration;

	char					szPublicKeyringFile[MAX_PATH];
	char					szPrivateKeyringFile[MAX_PATH];

	PGPBoolean				bUsePGPAuthKey;
	PGPKeyID				keyidPGPAuth;
	PGPBoolean				bUseX509AuthKey;
	PGPKeyID				keyidX509Auth;
	PGPByte*				pX509AuthCertIASN;
	PGPUInt32				uX509AuthCertIASNLength;

	PGPBoolean				bPGPnetHostListLocked;
	PGPUInt32				uHostCount;
	PGPNetPrefHostEntry*	pHostList;

	PGPBoolean				bPGPnetBlockedListLocked;
	PGPUInt32				uBlockedCount;
	PGPNetPrefBlockedEntry*	pBlockedList;

	PGPBoolean				bPGPnetTrustedListLocked;
	PGPUInt32				uTrustedCount;
	PGPNetPrefTrustedEntry* pTrustedList;

	PGPBoolean				bPGPnetFirewallEnabled;
	PGPBoolean				bFireLearnModeOn;
	PGPBoolean				bLogAllowedTraffic;
	PGPBoolean				bLogBlockedTraffic;
	PGPUInt32				uRuleSet;
	PGPUInt32				uRuleCount;
	PGPNetPrefFirewallRule*	pRuleList;

	PGPikeMTPref			IkeExpirationPrefs;
	PGPikeMTPref			IkeIkeProposalPrefs;
	PGPikeMTPref			IkeIpsecProposalPrefs;
	PGPikeMTPref			IkeAlgorithmPrefs;
	PGPikeGroupID			IkeIpsecGroupID;
} PNCONFIG, *PPNCONFIG;

typedef struct {
	PGPUInt32				uAutoRuleCount;
	PGPNetPrefFirewallRule*	pAutoRuleList;
	PGPUInt32				uKnownRuleCount;
	PGPNetPrefFirewallRule*	pKnownRuleList;
	PGPUInt32				uSmartRuleCount;
	PGPNetPrefFirewallRule*	pSmartRuleList;
} PNKNOWNCONFIG, *PPNKNOWNCONFIG;

// prototypes

/* Pass NULL for prefref if you want PGPnetLoadConfiguration to
   load and free the prefs file inside the function */

PGPError
PGPnetLoadConfiguration (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		PPNCONFIG		pPNConfig);

PGPError
PGPnetLoadKnownConfig (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		PPNKNOWNCONFIG	pPNKnownConfig);

PGPError
PGPnetFreeConfiguration (
		PPNCONFIG		pPNConfig);

PGPError
PGPnetFreeKnownConfig (
		PPNKNOWNCONFIG	pPNKnownConfig);


/* Pass NULL for prefref if you want PGPnetReloadConfiguration to
   load and free the prefs file inside the function */

PGPError
PGPnetReloadConfiguration (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		PPNCONFIG		pPNConfig);

PGPError
PGPnetReloadKnownConfig (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		PPNKNOWNCONFIG	pPNKnownConfig);

/* Pass NULL for prefref if you want PGPnetReloadConfiguration to
   load and free the prefs file inside the function */

PGPError
PGPnetSaveConfiguration (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		PPNCONFIG		pPNConfig);

PGPError
PGPnetSaveKnownConfig (
		PGPContextRef	context,
		PGPPrefRef		prefref,
		PPNKNOWNCONFIG	pPNKnownConfig);

PGPError
PGPnetOpenKeyrings (
		PGPContextRef	context,
		PPNCONFIG		pPNConfig,
		UINT			uOpenMode,
		PGPKeyDBRef*	pkeydb);

PGPError
PGPnetAddPromiscuousHostToConfiguration (
		PGPContextRef	context,
		PPNCONFIG		pPNConfig,
		PGPUInt32		uIPAddressHost,
		PGPUInt32		uIPAddressLocal,
		PGPKeyDBObjRef	keyAuth,
		PGPKeyDBObjRef	sigAuth);

PGPError
PGPnetGetConfiguredAuthKeys (
		PGPContextRef	context,
		PPNCONFIG		pPNConfig,
		PGPKeyDBRef		keydb,
		PGPKeyDBObjRef*	pkeyPGP,
		PGPKeyDBObjRef*	psigX509);

PGPError
PGPnetGetConfiguredHostAuthKeys (
		PGPContextRef			context,
		PGPNetPrefHostEntry*	pHost,
		PGPKeyDBRef				keydb,
		PGPBoolean				bLocal,
		PGPKeyDBObjRef*			pkey,
		PGPKeyDBObjRef*			psig);

PGPError
PGPnetGetIkeAlgorithmPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PPNCONFIG			pPNConfig);

PGPError
PGPnetGetIkeProposalPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PPNCONFIG			pPNConfig);

PGPError
PGPnetGetIpsecProposalPrefs (
		PGPContextRef		context,
		PGPPrefRef			prefref,
		PPNCONFIG			pPNConfig);

PGP_END_C_DECLARATIONS

#endif //_Included_pgpNetConfig_h