/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	pgpWin32IPC.h - PGP Win32 inter-process communication header

	$Id: pgpWin32IPC.h,v 1.48 2002/11/01 21:47:16 pbj Exp $
____________________________________________________________________________*/
#ifndef Included_pgpWin32IPC_h	/* [ */
#define Included_pgpWin32IPC_h

#include "pgpPubTypes.h"

//___________________________________________________
//
//	General PGP stuff

// This is appended to the CSIDL_xxx folder to derive the folder
// where various PGP files are kept
#define PGP_RELATIVEPATHNAME		("PGP Corporation\\PGP\\")
#define PGP_PERSONALPATHNAME		("PGP\\")
#define PGP_DEFAULTKEYRINGFOLDER	("PGP Keyrings\\")
#define PGP_NETEVENTDATAFOLDER		("PGPSavedEvents\\")

// various PGP file names
#define	PGP_PREFSFILENAME			("PGPprefs.txt")
#define PGP_HELPFILENAME			("PGP.chm")
#define PGP_GROUPSFILENAME			("PGPgroup.pgr")
#define PGP_PGPKEYSAPPNAME			("PGPkeys.exe")
#define	PGP_NETPREFSFILENAME		("PGPnetPrefs.txt")
#define PGP_NETKNOWNPREFSFILENAME	("PGPnetKnownPrefs.txt")
#define PGP_NETLOGFILENAME			("PGPnetLog.dat")
#define PGP_NETAPPNAME				("PGPnet.exe")
#define PGP_NETHELPFILENAME			("PGPnet.chm")
#define PGP_FIREHELPFILENAME		("PGPfire.chm")
#define PGP_MAILHELPFILENAME		("PGP.chm")
#define PGP_ADMINHELPFILENAME		("PGPadmin.chm")
#define PGP_DISKHELPFILENAME		("PGP.chm")
#define PGP_DEFAULTKEYSFILENAME		("DefaultKeys.asc")
#define PGP_TRAYAPPNAME				("PGPtray.exe")

// This is the registry key that should be used to find the install
// directory and contained files
#define PGP_REGISTRYKEY				("Software\\PGP Corporation\\PGP")
#define PGP_INSTALLPATHVALUE		("InstallPath")

// This is the registry key that should be used to find which components
// have been installed
#define PGP_INSTALLEDCOMPONENTSKEY	("\\InstalledComponents")
#define PGP_PGPDISKINSTALLED		("PGPdisk")
#define PGP_PFIDSINSTALLED			("PGPnet PF/IDS")
#define PGP_PGPKEYSINSTALLED		("PGPkeys")
#define PGP_PGPTOOLSINSTALLED		("PGPmail")
#define PGP_PGPLOGININSTALLED		("PGPlogin")
#define PGP_COMMANDLINEINSTALLED	("Cmdline")
#define PGP_ICQINSTALLED			("ICQ")
#define PGP_EUDORAINSTALLED			("Eudora")
#define PGP_EXCHANGEINSTALLED		("Exchange")
#define PGP_OEINSTALLED				("OutlookExpress")
#define PGP_NOTESINSTALLED			("LotusNotes")
#define PGP_MANUALINSTALLED			("UserManual")
#define PGP_VPNINSTALLED			("PGPnet VPN")

//___________________________________________________
//
//	PGPkeys stuff

#define PGPKEYS_WINCLASSNAME		("PGPkeysMainWinClass")
#define PGPKEYS_WINDOWTITLE			("PGPkeys")
#define PGPKEYS_SEMAPHORENAME		("PGPkeysInstSem")

#define	PGPPK_SELECTIVEIMPORT		0x00000001

#define	PGPPK_IMPORTKEYMASK			0xFFFF0000
#define PGPPK_IMPORTKEYBUFFER		0x00010000
#define PGPPK_IMPORTKEYFILELIST		0x00020000
#define PGPPK_IMPORTKEYCOMMANDLINE	0x00040000


//___________________________________________________
//
//	PGPtray stuff

// These are the names that the system tray app should use for its window
// class and title.
#define PGPTRAY_WINDOWNAME		("PGPtray_Hidden_Window")

// applications can send the following messages to PGPtray

// sent by Options property sheet to disable hotkeys while the property
// sheet is displayed.  Otherwise the hotkey interpretation interferes with
// the setting of the hotkey combinations
#define PGPTRAY_M_DISABLEHOTKEYS			WM_APP+120
#define PGPTRAY_M_ENABLEHOTKEYS				WM_APP+121


//___________________________________________________
//
//	PGPnet stuff

// These are values of command-line switches used to select specific
// pages of PGPnet application on startup
#define PGPNET_STATUSPAGE			0
#define PGPNET_HOSTPAGE				1
#define PGPNET_FIREWALLPAGE			2
#define PGPNET_IDSPAGE				3
#define PGPNET_LOGPAGE				4

// These are values passed from service to trayapp in WM_COPYDATA
// message (see below).
#define PGPNET_LOGGEDOFF			0
#define PGPNET_LOGGEDON				1
#define PGPNET_NOLOGONREQUIRED		2
#define PGPNET_LOGONINPROCESS		3

// These are the sound indices passed from service to trayapp in
// LPARAM of PGPNET_M_APPMESSAGE/PGPNET_PLAYSOUND message
#define PGPNET_NOSOUND				0
#define PGPNET_INTRUDERALERTSOUND	1

// These are values passed from service to app as return value to
// PGPNET_QUERYDRIVERSTATUS message (see below).
#define PGPNET_SERVICENOTRESPONDING	0
#define PGPNET_SERVICENOTAVAILABLE	1
#define PGPNET_DRIVERENABLED		2
#define PGPNET_DRIVERDISABLED		3
#define PGPNET_DRIVERNOTAVAILABLE	4

// This is the name that the Service should use for its window
// class and title.
#define PGPNET_SERVICEWINDOWNAME		("PGP IKE and Preferences Server")
#define PGPNET_SERVICECOMMWINDOWNAME	("PGP GUI-Server Communication")

// These are the names that the main GUI app should use for its window
// class and title.
#define PGPNET_APPWINDOWCLASS		("PGPnet Application Window Class")
#define PGPNET_APPWINDOWTITLE		("PGPnet")

// the PGPnet service, and main and tray applications send this
// message for any of a variety of reasons.  The WPARAM of
// this message indicates the type of the request.  The LPARAM
// indicates the HWND of the requesting window.
#define PGPNET_M_APPMESSAGE					WM_APP+100

// these are the values that can be passed in the WPARAM of the
// PGPNET_M_APPMESSAGE message.

// report the process id of a user process (PGPtray)
#define PGPNET_USERPROCESSID				0x0000

// turn ON and OFF status reporting via WM_COPYDATA messages from service
#define PGPNET_ENABLESTATUSMESSAGES			0x0001
#define PGPNET_DISABLESTATUSMESSAGES		0x0002

// turn ON and OFF log update notifications via WM_COPYDATA messages
// from service
#define PGPNET_ENABLELOGMESSAGES			0x0003
#define PGPNET_DISABLELOGMESSAGES			0x0004

// The PGPnet main application sends this to request that the
// service zeroes the log file.
#define PGPNET_CLEARLOGFILE					0x000A

// The PGPnet main or tray applications send this to request the current
// status of the driver.  The service should return ONE of
// PGPNET_DRIVERENABLED, PGPNET_DRIVERDISABLED, or
// PGPNET_DRIVERNOTAVAILABLE
#define PGPNET_QUERYDRIVERSTATUS			0x000B

// The PGPnet service sends the apps these messages when it wants to
// disable user interaction with the apps (e.g. when it puts up the
// passphrase dialog).
#define PGPNET_DISABLEGUI					0x000C
#define PGPNET_ENABLEGUI					0x000D

// Used to get the service to attempt IKE negotiations with a particular
// machine, without needing the driver. Send the host index of the machine
// as the LPARAM of the message
#define PGPNET_ATTEMPTIKE					0x000E

// The PGPnet service sends this message to the app to indicate an SA
// request has been initiated for the host whose index is contained in
// the LPARAM
#define PGPNET_SAREQUEST					0x0011

// The PGPnet service sends this message to the app to indicate an SA
// request has failed for the host whose index is contained in
// the LPARAM
#define PGPNET_SAFAILED						0x0012

// The PGPnet service sends this message to PGPtray to play a sound when
// an event occurs.  The sound index is in the LPARAM
#define PGPNET_PLAYSOUND					0x0013

// The PGPnet service sends this message to PGPtray to indicate the current
// number of existing SAs.  The SA count is in the LPARAM
#define PGPNET_SACOUNT						0x0014

// The PGPnet App sends this messages to the service when it wants to
// quickly remove a blocked host from the list. The IP address of the
// host is in the LPARAM
#define PGPNET_REMOVEBLOCKEDHOST			0x0015

// The PGPservice sends this to the app and tray to indicate that it
// is starting
#define PGPNET_SERVICESTART					0x0016

// PGPtray and PGPnet App send this back and forth to control flashing
// icon in system tray.  When PGPnet App receives this, it returns
// TRUE if PGPtray should flash and FALSE otherwise.  When PGPtray
// receives this, it starts or stops flashing on basis of the LPARAM.
// LPARAM == 0 => stop, LPARAM == -1 => flash indefinitely, else
// LPARAM == number of times to flash
#define PGPNET_FLASHICON					0x0017

// When this is sent to the service, it should respond with a
// PGPNET_STATUSFULL message
#define PGPNET_REQUESTSTATUSFULL			0x0018

// message send by PGPservice to itself in order to process
// keyring reload callbacks
#define PGPNET_RELOADKEYRING				0x0019

// message sent by PGPservice to PGPnet when a request to trace and IDS
// attack has occurred
#define PGPNET_TRACEATTACK					0x001A

// these are the values that can be passed by the Service in the dwData
// parameter of the COPYDATASTRUCT of the WM_COPYDATA message.

// lpData points to PGPnetLogEvent structure that service wants app
// to display
#define PGPNET_LOGUPDATE					0x0001

// lpData points to an array of PGPikeSA structs representing current
// active list of SAs
#define PGPNET_STATUSFULL					0x0002

// lpData points to ???
#define PGPNET_STATUSUPDATE					0x0003

// lpData points to a PGPikeSA structure containing new SA to add to list
#define PGPNET_STATUSNEWSA					0x0004

// lpData points to a PGPipsecSPI structure containing SPI of SA to remove
// (NB: also sent from main app to service to request that SA be removed)
#define PGPNET_STATUSREMOVESA				0x0005

// The service sends this to PGPtray to indicate an attack has occurred.
// lpData points to a PGPIntruderAlertStructure
typedef struct {
	PGPUInt32	intruderAddr;
	PGPUInt32	attackType;
	PGPInt32	soundToPlay;
	PGPBoolean	notifyEnabled;
} PGPIntruderAlertStructure;

#define PGPNET_INTRUDERALERT				0x0007

// lpData points to a text string to be displayed in the "Advanced"
// log window
#define PGPNET_ADVTEXTMESSAGE				0x0008

//___________________________________________________
//
//	PGP prefs service

// This is the window to communicate with for prefs
#define PGP_PREFSSERVICEWINDOWNAME	("PGP Prefs Communication")

// Call RegisterWindowMessage on this string to get notification
// for reloading prefs
#define PGP_RELOADPREFSMSG			("PGP Prefs Reload")
#define PGP_RELOADNETPREFSMSG		("PGP Net Prefs Reload")

// Prefs service and client use "sequence numbers" for synchronizing
// their access to the prefs file.  The following is used to indicate
// special cases
#define PGP_INVALIDPREFSEQUENCE		-1

// Message to "log off" the service for communicating prefs
// lParam = HWND of window in app that is logging off
#define PGP_M_LOGOFFPREFS			WM_APP+110

// Message that apps send to service to ask for the latest prefs
// wParam = last known sequence number
// lParam = HWND of window in app that will receive the prefs
// If the sequence number is different than what the service
// currently has, a WM_COPYDATA message will be sent to the
// window specified in lParam
#define PGP_M_QUERYPREFS			WM_APP+111
#define PGP_M_QUERYNETPREFS			WM_APP+112

// The following structure is sent as the lpData payload of WM_COPYDATA
// messages between client and service
typedef struct {
	PGPInt32	sequenceNumber;
	PGPInt32	bufferLength;
	PGPByte		buffer[1];
} PGPPrefsCopyDataStructure;

#define PGP_PGPPREFSLOGON			0x0001
#define PGP_PGPPREFSDATA			0x0002
#define PGP_PGPNETPREFSDATA			0x0004


//___________________________________________________
//
//	PGP PDA IPC

// The following are notifications sent to PGPkeys when syncing with Palm device
#define PGPPDA_M_NOTIFY_PALM		WM_APP + 130

#define PDA_IDENTIFY				0
#define PDA_SYNCSTART				1
#define PDA_SYNCSUCCESS				2
#define PDA_SYNCFAILURE				3
#define PDA_ADVISEUSER				4
#define PDA_SYNCOKQUERY				5
#define PDA_UPDATEUSERS				6


#endif //Included_pgpWin32IPC_h
