/*__________________________________________________________________________
 Copyright (C) 2002 PGP Corporation
 All rights reserved.
 
 $Id: PTnet.c,v 1.35 2002/08/06 20:10:33 dallen Exp $
__________________________________________________________________________*/
#include "pgpPFLConfig.h"
#include "precomp.h"

#include "pgpWin32IPC.h"
#include "PTnet.h"
#include "pgpClientPrefs.h"
#include "pgpNetPrefs.h"
#include "pgpNetAttack.h"
#include "pgpNetAttackStrings.h"
#include <winsock.h>

#define IPC_TIMEOUT				2500
#define FLASH_TIMER_MS			1000
#define KILLDELAYMS				3000
#define NUM_HOSTS_FOR_POPUP		16
#define	NUM_SA_FLASHES			30

#define NORMALINDENT			4
#define CHILDINDENT				16
#define TEXTINDENT				24
#define NOHOSTSINDENT			14
#define DOTOFFSET				20
#define BITMAPHEIGHT			16
#define BITMAPWIDTH				16
#define HALFBITMAPWIDTH			(BITMAPWIDTH>>1)

#define MENU_HEADER				1
#define MENU_FOOTER				2
#define MENU_NOHOSTS			3
#define HEADER_HEIGHT			34
#define FOOTER_HEIGHT			6
#define EDGE_WIDTH				2
#define HEADER_BITMAP_WIDTH		106
#define HEADER_BITMAP_HEIGHT	30

#define NETTRAYDEBUGLOG			PGP_DEBUG && FALSE

typedef struct _SORTEDHOSTLIST {
	struct _SORTEDHOSTLIST*	pNext;
	struct _SORTEDHOSTLIST*	pChild;
	INT						iIndex;
	BOOL					bChild;
} SORTEDHOSTLIST, *PSORTEDHOSTLIST;

static BOOL			s_bGUIEnabled		= TRUE;
static PGPBoolean	s_bNetEnabled		= FALSE;
static PGPBoolean	s_bIconEnabled		= TRUE; // wjb from FALSE
static INT			s_iFlashCount		= 0;
static UINT			s_uAttackType		= kPGPnetAttack_None;
static DWORD		s_dwAttackerIP		= 0;
static UINT			s_uServiceStatus	= PGPNET_SERVICENOTAVAILABLE;
static HIMAGELIST	s_himlIcons			= NULL;
static HFONT		s_hfontMenu			= NULL;
static UINT			s_uMenuHeight		= 0;
static UINT			s_uBitmapOffset		= 0;
static BOOL			s_bMenuStructsValid = FALSE;

static PGPNetPrefHostEntry*	s_pHostList		= NULL;
static UINT					s_uHostCount	= 0;
static PGPipsecSA*			s_pSAList		= NULL;
static UINT					s_uSACount		= 0;
static PSORTEDHOSTLIST		s_pSortedList	= NULL;
static HIMAGELIST			s_himlPGPnet	= NULL;

#if NETTRAYDEBUGLOG
#include <time.h>

sLogText (LPSTR psz)
{
	CHAR	sz[128];
	time_t	tt;
	FILE*	fp;

	time (&tt);
	lstrcpy (sz, ctime (&tt));

	lstrcat (sz, ":");
	lstrcat (sz, psz);
	lstrcat (sz, "\n");

	fp = fopen ("\\PGPtrayDebug.txt", "a+t");
	fwrite (sz, sizeof(char), lstrlen(sz), fp);
	fclose (fp);
}
#endif	// NETTRAYDEBUGLOG

//	________________________
//
//	destroy sorted list

VOID
sFreeSortedList (
	PSORTEDHOSTLIST		p)
{
	PSORTEDHOSTLIST		pNext;

	while (p)
	{
		if (p->pChild)
			sFreeSortedList (p->pChild);

		pNext = p->pNext;
		free (p);
		p = pNext;
	}
}

//	________________________
//
//	create sorted linked-list of hosts

VOID
sInsertHostIntoList (
		PSORTEDHOSTLIST			pThis,
		LPSTR					psz,
		PSORTEDHOSTLIST*		pp)
{
	while (*pp)
	{
		if (lstrcmpi (psz, s_pHostList[(*pp)->iIndex].hostName) <= 0)
			break;

		pp = &((*pp)->pNext);
	}
	pThis->pNext = *pp;
	*pp = pThis;
}

//	________________________
//
//	create sorted linked-list of hosts

VOID
sSortHosts (void)
{
	PSORTEDHOSTLIST		pThis;
	PSORTEDHOSTLIST		p;
	UINT				u;

	sFreeSortedList (s_pSortedList);
	s_pSortedList = NULL;

	for (u=0; u<s_uHostCount; u++)
	{
		pThis = malloc (sizeof (SORTEDHOSTLIST));
		if (pThis)
		{
			pThis->pNext = NULL;
			pThis->pChild = NULL;
			pThis->iIndex = u;
			pThis->bChild = FALSE;

			if (s_pSortedList == NULL)
			{
				s_pSortedList = pThis;
				continue;
			}

			if (s_pHostList[u].childOf == -1)
			{
				sInsertHostIntoList (pThis, 
						s_pHostList[u].hostName, &s_pSortedList);
			}
			else
			{
				pThis->bChild = TRUE;
				p = s_pSortedList;

				while (p)
				{
					if (p->iIndex == s_pHostList[u].childOf)
					{
						if (p->pChild == NULL)
							p->pChild = pThis;
						else
						{
							sInsertHostIntoList (pThis,
									s_pHostList[u].hostName, 
									&(p->pChild));
						}
						break;
					}
					p = p->pNext;
				}
			}
		}
	}
}

//	________________________
//
//	Launch PGPnet.exe, specifying the start page

VOID
PTNetLaunch (
		HWND	hwnd,
		UINT	uStartPage)
{
	PGPError	err					= kPGPError_OutputBufferTooSmall;
	CHAR		szFile[MAX_PATH];
	CHAR		sz[16];

	err = PGPclGetPath (kPGPclInstallFolder, szFile, sizeof(szFile));

	if (IsntPGPError (err)) 
	{
		if ((lstrlen (szFile) + lstrlen (PGP_NETAPPNAME)) <= sizeof(szFile))
		{
			lstrcat (szFile, PGP_NETAPPNAME);

			if ((lstrlen (szFile) +4) <= sizeof(szFile))
			{
				wsprintf (sz, " /%i", uStartPage);
				lstrcat (szFile, sz);

				if (!PGPclExecute (szFile, SW_SHOW))
					err = kPGPError_FileNotFound;
				else
					err = kPGPError_NoErr;
			}
		}
	}

	if (IsPGPError (err))
	{
		PTMessageBox (hwnd, IDS_CAPTION, IDS_CANNOTLAUNCHPGPNET,
							MB_OK|MB_ICONSTOP);
	}
}

//	________________________
//
//	query PGPnet App to see if we should flash or not

static BOOL 
sShouldFlash (VOID)
{
	HWND	hwnd;
	DWORD	dwResult;

	hwnd = FindWindow (PGPNET_APPWINDOWCLASS, 
									PGPNET_APPWINDOWTITLE);
	if (hwnd)
	{
		if (SendMessageTimeout (hwnd, PGPNET_M_APPMESSAGE, PGPNET_FLASHICON,
					TRUE, SMTO_NORMAL, IPC_TIMEOUT, &dwResult))
		{
			return (BOOL)dwResult;
		}
		else
			return TRUE;
	}
	else
		return TRUE;
}

//	________________________
//
//	Send message to service, if running

static BOOL 
sSendServiceMessage (
		UINT	uMsg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	HWND	hwnd;
	DWORD	dwResult;

	hwnd = FindWindow (PGPNET_SERVICECOMMWINDOWNAME, 
									PGPNET_SERVICECOMMWINDOWNAME);
	if (hwnd)
	{
		if (SendMessageTimeout (hwnd, uMsg, wParam, lParam, 
							SMTO_NORMAL, IPC_TIMEOUT, &dwResult))
		{
			return TRUE;
		}
		else
			return FALSE;
	}
	else
		return FALSE;
}

//	________________________
//
//	Send logon, logoff, or query message to service

BOOL
PTSendLogOnOffMessage (
		HWND	hwnd,
		UINT	uMessage)
{
	BOOL	bReturn		= FALSE;

	switch (uMessage) {
	case QUERYSERVICE :
		bReturn = TRUE;
		break;

	case REPORTPROCESSID :
		bReturn = sSendServiceMessage (
				PGPNET_M_APPMESSAGE, PGPNET_USERPROCESSID, 
				(LPARAM)GetCurrentProcessId ());
		break;
	}

	return bReturn;
}


//	________________________
//
//	if intrusion has been flagged, stop blinking and start timer

BOOL
PTIntrusionInProgress (VOID)
{
	if (s_uAttackType != kPGPnetAttack_None)
	{
		if (s_iFlashCount < 0)
			s_iFlashCount = 1;

		return TRUE;
	}
	return FALSE;
}


//	________________________
//
//	Query service for status

static UINT 
sGetServiceStatus (
		HWND	hwnd)
{
	HWND	hwndService;
	DWORD	dwResult;

	hwndService = FindWindow (PGPNET_SERVICECOMMWINDOWNAME, 
									PGPNET_SERVICECOMMWINDOWNAME);
	if (hwndService)
	{
		if (SendMessageTimeout (hwndService, PGPNET_M_APPMESSAGE, 
				(WPARAM)PGPNET_QUERYDRIVERSTATUS, (LPARAM)hwnd,
				SMTO_NORMAL, IPC_TIMEOUT, &dwResult))
		{
			return dwResult;
		}
		else 
			return PGPNET_SERVICENOTRESPONDING;
	}
	else
		return PGPNET_SERVICENOTAVAILABLE;
}


//	________________________
//
//	create string description of attack

static VOID 
sGetAttackDescription (
		LPSTR		psz,
		INT			iLen,
		UINT		uAttackType,
		DWORD		dwIP)
{
	CHAR			szIP[16];
	CHAR			szAttack[32];
	CHAR			szFormat[64];
	struct in_addr	ia;

	ia.S_un.S_addr = s_dwAttackerIP;
	lstrcpy (szIP, inet_ntoa (ia));

	if (uAttackType >= kPGPnetAttack_FirstFirewallRule)
	{
		wsprintf (szAttack, "%i", 
				uAttackType-kPGPnetAttack_FirstFirewallRule+1);
		LoadString (g_hinst, 
				IDS_FIREWALLVIOLATIONFORMAT, szFormat, sizeof(szFormat));
	}
	else
	{
		PGPGetNetAttackString (uAttackType, sizeof(szAttack), szAttack);
		LoadString (g_hinst, IDS_ATTACKFORMAT, szFormat, sizeof(szFormat));
	}

	if ((lstrlen (szAttack) + lstrlen (szIP) + lstrlen(szFormat)) < iLen)	
		wsprintf (psz, szFormat, szAttack, szIP);
}


//	________________________
//
//	Get icon and tooltip text based on driver/service status

static HICON
sGetIconAndText (
		HWND	hwnd,
		LPSTR	psz,
		UINT	uLen)
{
	HICON	hicon	= NULL;
	CHAR	sz[64];

	// if default icon is loaded up, that means that PGPnet is not installed,
	// so just return the default icon
	if (g_hIconTrayDefault)
	{
		LoadString (g_hinst, IDS_SZAPP, psz, uLen);
		return g_hIconTrayDefault;
	}

	s_uServiceStatus = sGetServiceStatus (hwnd);
	switch (s_uServiceStatus) {
	case PGPNET_DRIVERNOTAVAILABLE :
		hicon = LoadImage (g_hinst, MAKEINTRESOURCE(IDI_BROKENNETTRAYICON),
				IMAGE_ICON, g_uTrayIconSelector, g_uTrayIconSelector, 0);
		LoadString (g_hinst, IDS_DRIVERNOTAVAIL, psz, uLen);
		break;

	case PGPNET_DRIVERENABLED :
		if (s_uSACount == 0)
		{
			hicon = LoadImage (g_hinst, MAKEINTRESOURCE(IDI_NETTRAYICON),
					IMAGE_ICON, g_uTrayIconSelector, g_uTrayIconSelector, 0);
		}
		else
		{
			hicon = LoadImage (g_hinst, MAKEINTRESOURCE(IDI_SANETTRAYICON),
					IMAGE_ICON, g_uTrayIconSelector, g_uTrayIconSelector, 0);
		}

		// Only display SA's if VPN is installed 
		// (for instances of fire but not vpn)
		if(PGPclIsComponentInstalled (kPGPclVPN))
		{
			LoadString (g_hinst, IDS_DRIVERENABLED, sz, uLen);
			wsprintf (psz, sz, s_uSACount);
		}
		else
		{
			LoadString (g_hinst, IDS_SZAPP, psz, uLen);
		}
		break;

	case PGPNET_DRIVERDISABLED :
		hicon = LoadImage (g_hinst, MAKEINTRESOURCE(IDI_TRAYICON),
				IMAGE_ICON, g_uTrayIconSelector, g_uTrayIconSelector, 0);
		LoadString (g_hinst, IDS_DRIVERDISABLED, psz, uLen);
		break;

	case PGPNET_SERVICENOTRESPONDING :
		s_bGUIEnabled = TRUE;
		hicon = LoadImage (g_hinst, MAKEINTRESOURCE(IDI_BROKENNETTRAYICON),
				IMAGE_ICON, g_uTrayIconSelector, g_uTrayIconSelector, 0);
		LoadString (g_hinst, IDS_SERVICENOTRESPOND, psz, uLen);
		break;

	case PGPNET_SERVICENOTAVAILABLE :
	default :
		s_bGUIEnabled = TRUE;
		hicon = LoadImage (g_hinst, MAKEINTRESOURCE(IDI_BROKENNETTRAYICON),
				IMAGE_ICON, g_uTrayIconSelector, g_uTrayIconSelector, 0);
		LoadString (g_hinst, IDS_SERVICENOTAVAIL, psz, uLen);
		break;
	}

	if (s_iFlashCount != 0)
	{
		if (s_uAttackType == kPGPnetAttack_None)
		{
			if (s_iFlashCount & 0x1)
			{
				hicon = LoadImage (g_hinst, MAKEINTRESOURCE(IDI_SA2NETTRAYICON),
						IMAGE_ICON, g_uTrayIconSelector, g_uTrayIconSelector, 0);
			}
			else
			{
				hicon = LoadImage (g_hinst, MAKEINTRESOURCE(IDI_SANETTRAYICON),
						IMAGE_ICON, g_uTrayIconSelector, g_uTrayIconSelector, 0);
			}
		}
		else
		{
			if (s_iFlashCount & 0x1)
			{
				hicon = LoadIcon (
						g_hinst, MAKEINTRESOURCE (IDI_INTRUDERTRAYICON));
			}
			sGetAttackDescription (psz, 
					uLen, s_uAttackType, s_dwAttackerIP);
		}
	}

	return hicon;
}


//_________________________________________
//
//	update system tray icon

BOOL 
PTUpdateTrayIconAndText (
		HWND	hwnd,
		BOOL	bUpdateText,
		BOOL	bForceAddIcon)
{
	BOOL			result;
	NOTIFYICONDATA	nid;
	CHAR			sz[128];
	static BOOL		bIconIsVisible = FALSE;

	memset (&nid, 0x00, sizeof(NOTIFYICONDATA));
	nid.cbSize				= sizeof(NOTIFYICONDATA);
	nid.hWnd				= hwnd;
	nid.uID					= TASKBAR_TRAY_ID;
	nid.uFlags				= NIF_ICON;
	if (bUpdateText)
		nid.uFlags |= NIF_TIP;

	if (s_bIconEnabled)
	{
		nid.hIcon = sGetIconAndText (hwnd, sz, sizeof(sz));
		lstrcpy (nid.szTip, sz);

		if (bIconIsVisible && !bForceAddIcon)
			result = Shell_NotifyIcon (NIM_MODIFY, &nid);
		else
		{
			nid.uCallbackMessage = WM_TASKAREA_MESSAGE;
			nid.uFlags |= NIF_MESSAGE;
			result = Shell_NotifyIcon (NIM_ADD, &nid);
			if (result)
				bIconIsVisible = TRUE;
		}

		DestroyIcon(nid.hIcon);
	}
	else
	{
		if (bIconIsVisible)
		{
			result = Shell_NotifyIcon (NIM_DELETE, &nid);
			if (result)
				bIconIsVisible = FALSE;
		}

		if (s_iFlashCount < 0)
			s_iFlashCount = 1;
	}

	return result;
}


//	________________________
//
//	process the intruder alert message

static VOID
sIntruderAlert (
		HWND						hwnd,
		PGPIntruderAlertStructure*	ppias)
{
	#if NETTRAYDEBUGLOG
	{
		CHAR	sz[64];
		wsprintf (sz, "got PGPNET_INTRUDERALERT; attackType = %li",
						ppias->attackType);
		sLogText (sz);
	}
	#endif	// NETTRAYDEBUGLOG

	PGPclPlaySound (ppias->soundToPlay, FALSE);

	if (ppias->notifyEnabled)
	{
		if (sShouldFlash ())
		{
			s_uAttackType = ppias->attackType;
			s_dwAttackerIP = ppias->intruderAddr;
			s_iFlashCount = -1;

			KillTimer (hwnd, PGPNET_TIMER_ID);
			SetTimer (hwnd, PGPNET_TIMER_ID, FLASH_TIMER_MS, NULL);

			PTUpdateTrayIconAndText (hwnd, TRUE, FALSE);
		}
	}
}


//	________________________
//
//	prefs may have changed, do anything we need to do 

VOID
PTHandleNetPrefsChange (
		HWND		hwnd,
		PGPPrefRef	prefref,
		BOOL		bNetPref)
{
	if (bNetPref)
	{
		PGPGetPrefBoolean (prefref, 
				kPGPNetPrefEnablePGPnet, &s_bNetEnabled);

		if (s_pHostList)
		{
			PGPFreeData (s_pHostList);
			s_pHostList = NULL;
		}

		s_bMenuStructsValid = FALSE;
		PGPGetNetHostPrefs (prefref, &s_pHostList, &s_uHostCount);
		sSortHosts ();
	}
	else
	{
		if (IsntPGPError (PGPGetPrefBoolean (prefref, 
				kPGPPrefShowTrayIcon, &s_bIconEnabled)))
		{
			PTUpdateTrayIconAndText (hwnd, TRUE, FALSE);
		}
	}
}


//	________________________
//
//	check incoming message for something pertaining to PGPnet

VOID
PTCheckForNetMsg (
		HWND	hwnd,
		UINT	msg,
		WPARAM	wParam,
		LPARAM	lParam)
{
	switch (msg) 
	{
		case WM_DESTROY :
			if (s_pHostList)
			{
				PGPFreeData (s_pHostList);
				s_pHostList = NULL;
			}
			break;

		case PGPNET_M_APPMESSAGE :
			switch (wParam) {
			case PGPNET_SERVICESTART :
				#if NETTRAYDEBUGLOG
				sLogText ("got PGPNET_SERVICESTART");
				#endif	// NETTRAYDEBUGLOG

				sSendServiceMessage (PGPNET_M_APPMESSAGE, 
						PGPNET_USERPROCESSID, (LPARAM)GetCurrentProcessId ());
				break;

			case PGPNET_DISABLEGUI :
				#if NETTRAYDEBUGLOG
				sLogText ("got PGPNET_DISABLEGUI");
				#endif	// NETTRAYDEBUGLOG

				// relenquish the foreground window to the PGPnet service
				SetForegroundWindow ((HWND)lParam);
				s_bGUIEnabled = FALSE;
				break;

			case PGPNET_ENABLEGUI :
				#if NETTRAYDEBUGLOG
				sLogText ("got PGPNET_ENABLEGUI");
				#endif	// NETTRAYDEBUGLOG

				s_bGUIEnabled = TRUE;
				break;

			case PGPNET_PLAYSOUND :
				#if NETTRAYDEBUGLOG
				sLogText ("got PGPNET_PLAYSOUND");
				#endif	// NETTRAYDEBUGLOG

				PGPclPlaySound ((PGPclSoundIndex)lParam, FALSE);
				break;

			case PGPNET_FLASHICON :
				#if NETTRAYDEBUGLOG
				{
					CHAR	sz[64];
					wsprintf (sz, "got PGPNET_FLASHICON; lParam = %li; s_iFlashCount = %li",
									lParam, s_iFlashCount);
					sLogText (sz);
				}
				#endif	// NETTRAYDEBUGLOG

				if (lParam)
				{
					if (s_iFlashCount == 0)
						s_iFlashCount = lParam;
				}
				else
				{
					if (s_iFlashCount != 0)
						s_iFlashCount = 1;
				}

				if (s_iFlashCount != 0)
				{
					KillTimer (hwnd, PGPNET_TIMER_ID);
					SetTimer (hwnd, PGPNET_TIMER_ID, FLASH_TIMER_MS, NULL);
					PTUpdateTrayIconAndText (hwnd, TRUE, FALSE);
				}
				break;

			case PGPNET_SACOUNT :
				#if NETTRAYDEBUGLOG
				{
					CHAR	sz[64];
					wsprintf (sz, "got PGPNET_SACOUNT; lParam = %li; s_iFlashCount = %li",
									lParam, s_iFlashCount);
					sLogText (sz);
				}
				#endif	// NETTRAYDEBUGLOG

				if ((s_uAttackType == kPGPnetAttack_None) &&
					(s_uSACount != (UINT)lParam))
				{
					s_iFlashCount = 1;
					s_uSACount = lParam;
					PTUpdateTrayIconAndText (hwnd, TRUE, FALSE);
				}
				else
					s_uSACount = lParam;
				break;

			case PGPNET_SAREQUEST :
				if (s_uAttackType == kPGPnetAttack_None)
				{
					s_iFlashCount = NUM_SA_FLASHES;
					KillTimer (hwnd, PGPNET_TIMER_ID);
					SetTimer (hwnd, PGPNET_TIMER_ID, FLASH_TIMER_MS, NULL);
				}
				break;

			case PGPNET_SAFAILED :
				if (s_uAttackType == kPGPnetAttack_None)
				{
					if (s_iFlashCount != 0)
						s_iFlashCount = 1;
				}
				break;
			
			
			case PGPNET_TRACEATTACK :
				PTNetLaunch(hwnd, PGPNET_IDSPAGE);
				break;
			}
			break;

		case WM_TIMER :
			if (s_iFlashCount == 0)
			{
				s_uAttackType = kPGPnetAttack_None;
				PTUpdateTrayIconAndText (hwnd, TRUE, FALSE);
			}
			else
			{
				if (s_iFlashCount > 0)
				{
					if (--s_iFlashCount == 0)
					{
						KillTimer (hwnd, PGPNET_TIMER_ID);
						SetTimer (hwnd, PGPNET_TIMER_ID, PGPNET_TIMER_MS, NULL);
					}
				}
				else
					s_iFlashCount ^= 0x1;

				PTUpdateTrayIconAndText (hwnd, FALSE, FALSE);
			}
			break;

		case WM_COPYDATA :
		{
			PCOPYDATASTRUCT pcds	= (PCOPYDATASTRUCT)lParam;
			switch (pcds->dwData) {
			case PGPNET_INTRUDERALERT :
				sIntruderAlert (hwnd,
					(PGPIntruderAlertStructure*)(pcds->lpData));
				break;

			case PGPNET_STATUSFULL :
			{
				PBYTE		p;
				PBYTE		pEnd;
				DWORD		dwSizeKeyBlock;

				#if NETTRAYDEBUGLOG
				sLogText ("got PGPNET_STATUSFULL");
				#endif	// NETTRAYDEBUGLOG

				if (s_pSAList)
				{
					free (s_pSAList);
					s_pSAList = NULL;
				}

				s_uSACount = 0;
				s_pSAList = malloc (pcds->cbData);
				if (s_pSAList)
				{
					p = (pcds->lpData);
					pEnd = p + pcds->cbData;

					while (p < pEnd)
					{
						memcpy (&s_pSAList[s_uSACount++], p, sizeof(PGPipsecSA));

						// get keyblock size
						p += sizeof(PGPipsecSA);
						dwSizeKeyBlock = *((DWORD*)p);

						// get keyblock
						p += sizeof(DWORD);

						// step to next SA
						p += dwSizeKeyBlock;
					}
				}
				break;
			}
			}
			break;
		}
	}
}

//	____________________________________
//
//	find the authentication transform

static INT
sGetAuthenticationTransformIndex (
		PGPipsecSA*	pSA)
{
	INT	iTransform	= -1;
	INT nTransforms = min (pSA->numTransforms, kPGPike_MaxTransforms);
	INT	i;
	
	for (i=0; i<nTransforms; i++)
	{
		if (pSA->transform[i].protocol == kPGPike_PR_AH)
		{
			iTransform = i;
			break;
		}
	}

	return iTransform;
}

//	____________________________________
//
//	find the encryption transform

static INT
sGetEncryptionTransformIndex (
		PGPipsecSA*	pSA)
{
	INT	iTransform	= -1;
	INT nTransforms = min (pSA->numTransforms, kPGPike_MaxTransforms);
	INT	i;
	
	for (i=0; i<nTransforms; i++)
	{
		if (pSA->transform[i].protocol == kPGPike_PR_ESP)
		{
			iTransform = i;
			break;
		}
	}

	return iTransform;
}

//	____________________________________
//
//	determine if SA is tunnel mode

static BOOL
sIsTunnelMode (
		PGPipsecSA*		pSA)
{
	PGPipsecEncapsulation	encap;
	INT						iTrns;

	// determine if tunnel mode
	iTrns = sGetEncryptionTransformIndex (pSA);
	if (iTrns >= 0) 
		encap = pSA->transform[iTrns].u.esp.t.mode;
	else
	{
		iTrns = sGetAuthenticationTransformIndex (pSA);
		encap = pSA->transform[iTrns].u.ah.t.mode;
	}

	return (encap == kPGPike_PM_Tunnel);
}

//	____________________________________
//
//	find the host for this SA

PGPNetPrefHostEntry*
sFindHostForSA (
		PGPipsecSA*		pSA)
{
	PGPNetPrefHostEntry*	pHost				= NULL;
	PGPNetPrefHostEntry*	pHostHost			= NULL;
	PGPNetPrefHostEntry*	pHostSubnetRange	= NULL;
	PGPUInt32				u;

	for (u=0; u<s_uHostCount; u++)
	{
		pHost = &s_pHostList[u];

		if (pHost->destIsRange)
		{
			if ((ntohl (pSA->ipAddrStart) >= ntohl (pHost->ipAddrStart)) &&
				(ntohl (pSA->ipAddrStart) <= ntohl (pHost->ipMaskEnd)))
			{
				pHostSubnetRange = pHost;
			}
		}
		else
		{
			if ((pHost->ipAddrStart == pSA->ipAddrStart) &&
				(pHost->ipMaskEnd == pSA->ipMaskEnd))
			{
				return pHost;
			}

			if (pHost->ipAddrStart == pSA->ipAddress)
			{
				pHostHost = pHost;
			}

			if ((pHost->ipAddrStart & pHost->ipMaskEnd) == 
				(pSA->ipAddress & pHost->ipMaskEnd))
			{
				pHostSubnetRange = pHost;
			}
		}
	}

	if (pHostHost)
		return pHostHost;
	if (pHostSubnetRange)
		return pHostSubnetRange;

	return NULL;
}

//	____________________________________
//
//	kill all SAs related to a host

VOID
sRemoveHostSAs (
		HWND	hwnd,
		UINT	uHostIndex)
{
	BOOL					bRemove			= FALSE;
	BOOL					bNeedDelay		= FALSE;

	PGPipsecSA*				pSA;
	PGPNetPrefHostEntry*	pHostForSA;
	PGPNetPrefHostEntry*	pHost;
	INT						iPass;
	UINT					u;

	if (!s_pSAList || (s_uSACount ==0))
		return;

	pHost = &s_pHostList[uHostIndex];

	for (iPass = 1; iPass <= 2; iPass++)
	{
		for (u=0; u<s_uSACount; u++)
		{
			bRemove = FALSE;

			pSA = &s_pSAList[u];

			pHostForSA = sFindHostForSA (pSA);
			if (pHostForSA)
			{
				if (pHost == pHostForSA)
				{
					// in first pass remove only non-tunnel mode SAs
					// then remove all remaining matching SAs
					if (iPass == 1)
					{
						if (!sIsTunnelMode (pSA))
						{
							bRemove = TRUE;
							if (pHostForSA->childOf != -1)
								bNeedDelay = TRUE;
						}
					}
					else
						bRemove = TRUE;
				}

				if (pHostForSA->childOf != -1)
				{
					INT	iParent = pHostForSA->childOf;
					if (pHost == &s_pHostList[iParent])
						bRemove = TRUE;
				}
			}

			if (bRemove)
			{
				COPYDATASTRUCT	cds;
				PGPipsecSPI		spi;

				// we must delay after killing an child and before killing
				// the parent
				if (bNeedDelay && (iPass == 2))
				{
					HCURSOR		hcursorOld;

					bNeedDelay = FALSE;
					hcursorOld = SetCursor (LoadCursor (NULL, IDC_WAIT));
					Sleep (KILLDELAYMS);
					SetCursor (hcursorOld);
				}

				memcpy (spi, &pSA->transform[0].inSPI, sizeof(PGPipsecSPI));

				cds.dwData = PGPNET_STATUSREMOVESA;
				cds.cbData = sizeof(PGPipsecSPI);
				cds.lpData = spi;

				sSendServiceMessage (WM_COPYDATA,
						(WPARAM)hwnd, (LPARAM)&cds);
			}
		}
	}
}

//	____________________________________
//
//	determine if specified host has an SA in list
//	if address and mask are both zero, then check for any existing SAs

BOOL
sDoesHostHaveSA (
		UINT	uHostIndex)
{
	PGPNetPrefHostEntry*	pHost;
	PGPNetPrefHostEntry*	pHostForSA;
	UINT					u;

	if (!s_pSAList || (s_uSACount ==0))
		return FALSE;

	pHost = &s_pHostList[uHostIndex];

	// address/mask both zero => return TRUE if any SAs at all
	if ((pHost->ipAddrStart == 0) && (pHost->ipMaskEnd == 0))
	{
		return (s_uSACount > 0);
	}

	for (u=0; u<s_uSACount; u++)
	{
		pHostForSA = sFindHostForSA (&s_pSAList[u]);

		if (pHostForSA)
		{
			if (pHost == pHostForSA)
				return TRUE;

			if (pHostForSA->childOf != -1)
			{
				if (pHost == &s_pHostList[pHostForSA->childOf])
					return TRUE;
			}
		}
	}

	return FALSE;
}

//	____________________________________
//
//	return TRUE if a gateway with the "exclusiveGateway" flag is connected

static BOOL
sExclusiveGatewayConnected (VOID)
{
	PGPUInt32	u;

	for (u=0; u<s_uSACount; u++)
	{
		if ((s_pSAList[u].ipAddrStart == 1) && (s_pSAList[u].ipMaskEnd == 0))
			return TRUE;
	}

	return FALSE;
}


//	____________________________________
//
//	return TRUE if a gateway with the "virtualIP" flag is connected

static BOOL
sVirtualIPGatewayConnected (VOID)
{
	PSORTEDHOSTLIST			p;
	PGPNetPrefHostEntry*	pentry;

	p = s_pSortedList;
	while (p)
	{
		if ((UINT)p->iIndex < s_uHostCount)
		{
			pentry = &s_pHostList[p->iIndex];

			if (pentry->hostType == kPGPnetSecureGateway)
			{
				if (pentry->virtualIP)
				{
					if (sDoesHostHaveSA (p->iIndex))
					{
						return TRUE;
					}
				}
			}
		}
		p = p->pNext;
	}

	return FALSE;
}

//	____________________________________
//
//	determine if host entry is connectable

BOOL
sIsHostEntryConnectable (
		PSORTEDHOSTLIST		p)
{
	PGPNetPrefHostEntry*	pentry;

	if ((UINT)p->iIndex < s_uHostCount)
	{
		pentry = &s_pHostList[p->iIndex];

		if (pentry->hostType == kPGPnetSecureHost)
		{
			// secure ranges are never connectable
			if (pentry->destIsRange)
				return FALSE;

			// secure subnets are never connectable
			if (pentry->ipMaskEnd != 0xFFFFFFFF)
				return FALSE;
		}

		// is this a root item?
		if (pentry->childOf == -1)
		{
			// it's a gateway
			if (pentry->hostType == kPGPnetSecureGateway)
			{
				if (pentry->virtualIP)
				{
					if (sVirtualIPGatewayConnected ())
						return FALSE;
				}
			}
			// it's a secure host
			else if ((pentry->hostType == kPGPnetSecureHost) &&
				(pentry->ipMaskEnd == 0xFFFFFFFF))
			{
				return TRUE;
			}
			// it's an insecure host, subnet, or range
			else
				return FALSE;
		}
		// this is a child item
		else
		{
			PGPNetPrefHostEntry*	pentryParent;

			pentryParent = &s_pHostList[pentry->childOf];

			if (pentryParent->virtualIP)
			{
				if (sVirtualIPGatewayConnected ())
					return sDoesHostHaveSA (pentry->childOf);
			}
		}

		return TRUE;
	}

	return FALSE;
}

//	________________________
//
//	load host list into menu

VOID
PTNetLoadHostList (
		HWND	hwnd,
		HMENU	hMenu, 
		UINT	uMenuPos,
		BOOL	bTaskBarAtTop,
		BOOL	bDisabled)
{
	PSORTEDHOSTLIST		p;
	HMENU				hSubMenu;

	hSubMenu = GetSubMenu (hMenu, uMenuPos);
	if (GetMenuItemCount (hSubMenu) <= 5)
	{
		UINT			uState		= MF_STRING;
		UINT			uPos;
		MENUITEMINFO	mii;

		// insert the separator bar
		mii.cbSize = sizeof (mii);
		mii.fMask = MIIM_TYPE;
		mii.fType = MFT_SEPARATOR;

		uPos = 0;
		InsertMenuItem (hSubMenu, uPos, TRUE, &mii);

		// insert the header
		mii.fMask = MIIM_TYPE | MIIM_STATE | MIIM_ID | MIIM_DATA;
		mii.fType = MFT_OWNERDRAW;
		mii.fState = MFS_GRAYED;
		mii.wID = IDM_PGPNETHOSTLIST;
		mii.dwItemData = MENU_HEADER;

		InsertMenuItem (hSubMenu, uPos++, TRUE, &mii);

		// insert the actual items
		if (s_uHostCount > 0)
		{
			mii.fMask = MIIM_TYPE | MIIM_STATE | MIIM_ID | MIIM_DATA;
			mii.fType = MFT_OWNERDRAW;

			if (s_uServiceStatus == PGPNET_DRIVERENABLED)
			{
				sSendServiceMessage (PGPNET_M_APPMESSAGE, 
						(WPARAM)PGPNET_REQUESTSTATUSFULL, (LPARAM)hwnd);
			}

			p = s_pSortedList;
			while (p)
			{
				mii.fState = 0;

				if (bDisabled || !s_bNetEnabled)
					mii.fState |= MFS_GRAYED;

				if (sDoesHostHaveSA (p->iIndex))
					mii.fState |= MFS_CHECKED;
				else
				{
					if (!sIsHostEntryConnectable (p))
						mii.fState |= MFS_GRAYED;
				}

				mii.dwItemData = (DWORD)p;
				mii.wID = IDM_PGPNETHOSTLIST + p->iIndex;

				InsertMenuItem (hSubMenu, uPos++, TRUE, &mii);

				if (p->pChild)
				{
					PSORTEDHOSTLIST		pChild;

					pChild = p->pChild;

					while (pChild)
					{
						mii.fState = 0;

						if (bDisabled || !s_bNetEnabled)
							mii.fState |= MFS_GRAYED;

						if (sDoesHostHaveSA (pChild->iIndex))
							mii.fState |= MFS_CHECKED;
						else
						{
							if (!sIsHostEntryConnectable (pChild))
								mii.fState |= MFS_GRAYED;
						}

						mii.dwItemData = (DWORD)pChild;
						mii.wID = IDM_PGPNETHOSTLIST + pChild->iIndex;

						InsertMenuItem (hSubMenu, uPos++, TRUE, &mii);

						pChild = pChild->pNext;
					}
				}

				p = p->pNext;
			}
		}
		else
		{
			// insert the "no hosts" entry
			mii.fMask = MIIM_TYPE | MIIM_STATE | MIIM_ID | MIIM_DATA;
			mii.fType = MFT_OWNERDRAW;
			mii.fState = MFS_GRAYED;
			mii.wID = IDM_PGPNETHOSTLIST;
			mii.dwItemData = MENU_NOHOSTS;

			InsertMenuItem (hSubMenu, uPos++, TRUE, &mii);
		}

		// insert the footer
		mii.fMask = MIIM_TYPE | MIIM_STATE | MIIM_ID | MIIM_DATA;
		mii.fType = MFT_OWNERDRAW;
		mii.fState = MFS_GRAYED;
		mii.wID = IDM_PGPNETHOSTLIST;
		mii.dwItemData = MENU_FOOTER;

		InsertMenuItem (hSubMenu, uPos, TRUE, &mii);

		s_bMenuStructsValid = TRUE;
	}
}

//	________________________
//
//	connect/disconnect host

VOID
PTNetHostClicked (
		HWND	hwnd,
		UINT	uHostIndex)
{
	if (s_bMenuStructsValid)
	{
		if (sDoesHostHaveSA (uHostIndex))
		{
			sRemoveHostSAs (hwnd, uHostIndex);
		}
		else
		{
			if (sExclusiveGatewayConnected ())
			{
				PTMessageBox (hwnd, IDS_CAPTION, IDS_EXGATEWAYCONNECTED,
							MB_OK|MB_ICONEXCLAMATION);
			}
			else if ((s_pHostList[uHostIndex].exclusiveGateway) &&
					(s_uSACount > 0))
			{
				PTMessageBox (hwnd, IDS_CAPTION, IDS_CANNOTCONNECTEXGATEWAY,
							MB_OK|MB_ICONEXCLAMATION);
			}
			else
			{
				sSendServiceMessage (PGPNET_M_APPMESSAGE, 
						(WPARAM)PGPNET_ATTEMPTIKE, (LPARAM)uHostIndex);
			}
		}
	}
}

//	________________________
//
//	measure the owner-drawn menu item

VOID
PTNetMeasureItem (
		LPMEASUREITEMSTRUCT		pmi)
{
    HFONT					oldhfont;
	HDC						hdc;
	SIZE					size;
	PGPNetPrefHostEntry*	pHost;
	PSORTEDHOSTLIST			p;
	CHAR					sz[32];
	CHAR*					psz;
	INT						iIndent;

	switch (pmi->itemData) {
	case MENU_HEADER :
		pmi->itemWidth = HEADER_BITMAP_WIDTH + 2*EDGE_WIDTH;
		pmi->itemHeight = HEADER_HEIGHT;
		return;

	case MENU_FOOTER :
		pmi->itemWidth = 0;
		pmi->itemHeight = FOOTER_HEIGHT;
		return;

	case MENU_NOHOSTS :
		LoadString (g_hinst, IDS_NOHOSTS, sz, sizeof(sz));
		psz = sz;
		iIndent = 0;
		break;

	default :
		if (!s_bMenuStructsValid)
			return;
		p = (PSORTEDHOSTLIST)(pmi->itemData);
		pHost = &s_pHostList[p->iIndex];
		psz = pHost->hostName;
		if (pHost->childOf != -1)
			iIndent = CHILDINDENT;
		else
			iIndent = 0;
		break;
	}

	// calculate text width
	hdc = GetDC(NULL);
	oldhfont = SelectObject (hdc, s_hfontMenu);
	GetTextExtentPoint32 (hdc, psz, lstrlen (psz), &size);
	SelectObject (hdc, oldhfont);

	// add in the left margin for the icon
	size.cx += NORMALINDENT+TEXTINDENT;

	// add in the indent for child items
	size.cx += iIndent;

	// return the width and height
	pmi->itemWidth = size.cx + 4*EDGE_WIDTH;
	pmi->itemHeight = s_uMenuHeight;
}

//	________________________
//
//	draw the owner-drawn menu item

VOID
PTNetDrawItem (
		LPDRAWITEMSTRUCT		pdi)
{
    HFONT					oldhfont;
	HBRUSH					hbgb;
	INT						mode, icon;
	COLORREF				text,back;
	PGPNetPrefHostEntry*	pHost;
	PSORTEDHOSTLIST			p;

	switch (pdi->itemData) {
	case MENU_HEADER :
		if (s_bNetEnabled)
			icon = 1;
		else
			icon = 0;

		ImageList_Draw (s_himlPGPnet, icon, pdi->hDC, 
				pdi->rcItem.right-HEADER_BITMAP_WIDTH, 
				pdi->rcItem.top, ILD_TRANSPARENT);

		pdi->rcItem.top += HEADER_BITMAP_HEIGHT;
		pdi->rcItem.left += EDGE_WIDTH;
		pdi->rcItem.right -= EDGE_WIDTH;
		DrawEdge (pdi->hDC, &pdi->rcItem, 
				EDGE_SUNKEN, BF_LEFT|BF_RIGHT|BF_TOP);

		pdi->rcItem.top += EDGE_WIDTH;
		pdi->rcItem.left += EDGE_WIDTH;
		pdi->rcItem.right -= EDGE_WIDTH;
		hbgb = CreateSolidBrush (GetSysColor (COLOR_WINDOW));
		FillRect (pdi->hDC, &pdi->rcItem, hbgb);
		DeleteObject (hbgb);
		return;

	case MENU_FOOTER :
		pdi->rcItem.left += EDGE_WIDTH;
		pdi->rcItem.right -= EDGE_WIDTH;
		pdi->rcItem.bottom -= EDGE_WIDTH;
		DrawEdge (pdi->hDC, &pdi->rcItem, 
				EDGE_SUNKEN, BF_LEFT|BF_RIGHT|BF_BOTTOM);

		pdi->rcItem.left += EDGE_WIDTH;
		pdi->rcItem.right -= EDGE_WIDTH;
		pdi->rcItem.bottom -= EDGE_WIDTH;
		hbgb = CreateSolidBrush (GetSysColor (COLOR_WINDOW));
		FillRect (pdi->hDC, &pdi->rcItem, hbgb);
		DeleteObject (hbgb);
		return;

	case MENU_NOHOSTS :
	{
		CHAR	sz[32];

		pdi->rcItem.left += EDGE_WIDTH;
		pdi->rcItem.right -= EDGE_WIDTH;
		DrawEdge (pdi->hDC, &pdi->rcItem, EDGE_SUNKEN, BF_LEFT|BF_RIGHT);

		pdi->rcItem.left += EDGE_WIDTH;
		pdi->rcItem.right -= EDGE_WIDTH;
		hbgb = CreateSolidBrush (GetSysColor (COLOR_WINDOW));
		FillRect (pdi->hDC, &pdi->rcItem, hbgb);
		DeleteObject (hbgb);

        // Set the text background and foreground colors to the 
        // standard window colors
        text = SetTextColor (pdi->hDC, GetSysColor (COLOR_GRAYTEXT));
        back = SetBkColor (pdi->hDC, GetSysColor (COLOR_WINDOW));

		// draw the text left aligned and vertically centered
		LoadString (g_hinst, IDS_NOHOSTS, sz, sizeof(sz));
		pdi->rcItem.left += NOHOSTSINDENT;
		DrawText (pdi->hDC, sz, -1, 
				&pdi->rcItem, DT_SINGLELINE|DT_VCENTER|DT_LEFT);
		return;
	}
	}

	if (pdi->itemState & ODS_SELECTED)
	{
		pdi->rcItem.left += 2*EDGE_WIDTH;
		pdi->rcItem.right -= 2*EDGE_WIDTH;
		hbgb = CreateSolidBrush (GetSysColor (COLOR_HIGHLIGHT));
		FillRect (pdi->hDC, &pdi->rcItem, hbgb);
		DeleteObject (hbgb);

        // Set the text background and foreground colors
        text = SetTextColor (pdi->hDC, GetSysColor (COLOR_HIGHLIGHTTEXT));
        back = SetBkColor (pdi->hDC, GetSysColor (COLOR_HIGHLIGHT));
   	}
	else
	{
		pdi->rcItem.left += EDGE_WIDTH;
		pdi->rcItem.right -= EDGE_WIDTH;
		DrawEdge (pdi->hDC, &pdi->rcItem, EDGE_SUNKEN, BF_LEFT|BF_RIGHT);

		pdi->rcItem.left += EDGE_WIDTH;
		pdi->rcItem.right -= EDGE_WIDTH;
		hbgb = CreateSolidBrush (GetSysColor (COLOR_WINDOW));
		FillRect (pdi->hDC, &pdi->rcItem, hbgb);
		DeleteObject (hbgb);

        // Set the text background and foreground colors to the 
        // standard window colors
        text = SetTextColor (pdi->hDC, GetSysColor (COLOR_MENUTEXT));
        back = SetBkColor (pdi->hDC, GetSysColor (COLOR_MENU));
	}

	if (!s_bMenuStructsValid)
		return;

	p = (PSORTEDHOSTLIST)(pdi->itemData);
	pHost = &s_pHostList[p->iIndex];

	mode = SetBkMode (pdi->hDC, TRANSPARENT);

	pdi->rcItem.left += NORMALINDENT;

	// draw the lines
	if (p->bChild)
	{
		HPEN	hPen, hPenOld;
		int iyMid = (pdi->rcItem.bottom - pdi->rcItem.top) >> 1;
		iyMid += pdi->rcItem.top;

		hPen = CreatePen (PS_SOLID, 0, GetSysColor (COLOR_BTNFACE));
		hPenOld = SelectObject (pdi->hDC, hPen);

		MoveToEx (pdi->hDC, 
				pdi->rcItem.left+HALFBITMAPWIDTH, pdi->rcItem.top, NULL);

		if (p->pNext)
		{
			LineTo (pdi->hDC, 
					pdi->rcItem.left+HALFBITMAPWIDTH, pdi->rcItem.bottom);
			MoveToEx (pdi->hDC, 
					pdi->rcItem.left+HALFBITMAPWIDTH, iyMid, NULL);
			LineTo (pdi->hDC, pdi->rcItem.left+BITMAPWIDTH, iyMid);
		}
		else
		{
			LineTo (pdi->hDC, pdi->rcItem.left+HALFBITMAPWIDTH, iyMid);
			LineTo (pdi->hDC, pdi->rcItem.left+BITMAPWIDTH, iyMid);
		}
		SelectObject (pdi->hDC, hPenOld);

		// add in the indent for child items
		pdi->rcItem.left += CHILDINDENT;
	}

	// draw the icon
	switch (pHost->hostType) 
	{
		case kPGPnetInsecureHost :
			if (pHost->destIsRange)
				icon = IDX_INSECURESUBNET;
			else if (pHost->ipMaskEnd == 0xFFFFFFFF)
				icon = IDX_INSECUREHOST;	
			else
				icon = IDX_INSECURESUBNET;
			break;

		case kPGPnetSecureHost :	
			if (pHost->destIsRange)
				icon = IDX_SECURESUBNET;
			else if (pHost->ipMaskEnd == 0xFFFFFFFF)
				icon = IDX_SECUREHOST;	
			else
				icon = IDX_SECURESUBNET;
			break;

		case kPGPnetSecureGateway :	
			icon = IDX_SECUREGATEWAY;	
			break;
	}
	ImageList_Draw (s_himlIcons, icon,
			pdi->hDC, pdi->rcItem.left, 
			pdi->rcItem.top+s_uBitmapOffset, ILD_TRANSPARENT);

	// add the menu margin offset
	pdi->rcItem.left += TEXTINDENT;

	oldhfont = (HFONT)SelectObject (pdi->hDC, s_hfontMenu);

	// disabled items are drawn in gray
	if (pdi->itemState & ODS_DISABLED)
		SetTextColor (pdi->hDC, GetSysColor(COLOR_GRAYTEXT));

	// draw the text left aligned and vertically centered
	DrawText (pdi->hDC, pHost->hostName, -1, 
			&pdi->rcItem, DT_SINGLELINE|DT_VCENTER|DT_LEFT);

	// the "checked" state indicates an SA exist, draw green dot
	if (pdi->itemState & ODS_CHECKED)
	{
		pdi->rcItem.left = pdi->rcItem.right - DOTOFFSET;
		pdi->rcItem.left -= 2*EDGE_WIDTH;
		ImageList_Draw (s_himlIcons, IDX_SAEXISTS,
				pdi->hDC, pdi->rcItem.left, 
				pdi->rcItem.top+s_uBitmapOffset, ILD_TRANSPARENT);
	}

	SelectObject (pdi->hDC, oldhfont);
	SetBkMode (pdi->hDC, mode);
	SetTextColor (pdi->hDC, text);
	SetBkColor (pdi->hDC, back);
}

//	________________________
//
//	initialize

static void 
sCreateImageList (void)
{
	int iNumBits;
	HDC hDC;
	HBITMAP hBmp;

	// ImageList Init

	hDC = GetDC (NULL);		// DC for desktop
	iNumBits = GetDeviceCaps (hDC, BITSPIXEL) * GetDeviceCaps (hDC, PLANES);
	ReleaseDC (NULL, hDC);

	if (iNumBits <= 8) 
	{
		s_himlPGPnet =	ImageList_Create (
				HEADER_BITMAP_WIDTH, HEADER_BITMAP_HEIGHT, 
				ILC_COLOR|ILC_MASK, NUM_BITMAPS, 0); 
		hBmp = LoadBitmap (g_hinst, MAKEINTRESOURCE (IDB_NETHEADER4BIT));
		ImageList_AddMasked (s_himlPGPnet, hBmp, RGB(255, 0, 255));
		DeleteObject (hBmp);
	}
	else 
	{
		s_himlPGPnet =	ImageList_Create (
				HEADER_BITMAP_WIDTH, HEADER_BITMAP_HEIGHT, 
				ILC_COLOR24|ILC_MASK, NUM_BITMAPS, 0); 
		hBmp = LoadBitmap (g_hinst, MAKEINTRESOURCE (IDB_NETHEADER24BIT));
		ImageList_AddMasked (s_himlPGPnet, hBmp, RGB(255, 0, 255));
		DeleteObject (hBmp);
	}
}

//	________________________
//
//	initialize

VOID
PTNetStartup (
		HFONT			hfont,
		UINT			uHeight,
		UINT			uBitmapOffset,
		HIMAGELIST		himl)
{
	PGPPrefRef	prefref;

	s_hfontMenu = hfont;
	s_uMenuHeight = uHeight;
	s_uBitmapOffset = uBitmapOffset;
	s_himlIcons = himl;
	sCreateImageList ();

	if (IsntPGPError (PGPclPeekClientLibPrefRefs (NULL, &prefref)))
	{
		PGPGetPrefBoolean (prefref, 
				kPGPNetPrefEnablePGPnet, &s_bNetEnabled);
		PGPGetNetHostPrefs (prefref, &s_pHostList, &s_uHostCount);
		sSortHosts ();
	}
}

//	________________________
//
//	query booleans

BOOL
PTNetIsGUIDisabled (void)
{
	return !s_bGUIEnabled;
}



