/*__________________________________________________________________________
 Copyright (C) 2002 PGP Corporation
 All rights reserved.
 
 $Id: PTproc.c,v 1.96 2002/11/12 18:59:33 pbj Exp $
__________________________________________________________________________*/
#include "precomp.h"
#include "PThotkey.h"
#include "PTnet.h"
#include "PTdisk.h"
#include "PTupdate.h"
#include "pgpWin32IPC.h"
#include "pgpwd.h"
#include <commctrl.h>
#include "pgpUtilities.h"
#include "pgpClientPrefs.h"
#include "pgpNetPrefs.h"
#include "pgpBuild.h"
#include "pgpThreads.h"

#define BITMAP_HEIGHT			16
#define BITMAP_WIDTH			16
#define HEADER_HEIGHT			20
#define HEADER_WIDTH			133
#define HEADER_OFFSET			4
#define DISKHEADER_HEIGHT		30
#define DISKHEADER_WIDTH		109
#define DISKHEADER_OFFSET		4

#define ID_PGPHEADERBITMAP			50001
#define ID_PGPDISKHEADERBITMAP		50002

BOOL PopupTaskbarMenu(HWND hwndTarget, BOOL InPGPOperation);
BOOL PGPdiskExists=FALSE;
BOOL PGPtoolsExists=FALSE;
BOOL PGPkeysExists=FALSE;
HIMAGELIST hIml=NULL;
HIMAGELIST hImlHeader=NULL;
HIMAGELIST hImlDiskHeader=NULL;

static PGPHotKeys	s_HotKeys;
static HMENU		s_hMenu				= NULL;
static HFONT		s_hFontMenu			= NULL;
static UINT			s_uMenuHeight		= 0;
static UINT			s_uBitmapPos		= 0;
static BOOL			s_bTaskBarAtTop		= FALSE;
static BOOL			s_bDrawDividerLine	= FALSE;
static PGPBoolean	s_bIconEnabled		= TRUE;
static BOOL			s_bHighColorDisplay	= FALSE;

typedef struct
{
	char szString[20];
	UINT uIcon;
} PGPMENUINFO, *PPGPMENUINFO;

PGPMENUINFO miDisk,miTools,miKeys;

INT 
PTMessageBox (
		HWND	hwnd, 
		INT		iCaption, 
		INT		iText, 
		UINT	uStyle)
{
	CHAR	szCaption[64];
	CHAR	szText[384];

	LoadString (g_hinst, iCaption, szCaption, sizeof(szCaption));
	LoadString (g_hinst, iText, szText, sizeof(szText));

	return (MessageBox (hwnd, szText, szCaption, uStyle));
}

BOOL CheckForExistanceOfEXE(char *szEXEname)
{
	char szEXEPath[MAX_PATH];
	FILE *ftest;

	PGPscPGPpath(szEXEPath);
	strcat(szEXEPath,szEXEname);

	ftest=fopen(szEXEPath,"rb");
	
	if(ftest!=NULL)
	{
		fclose(ftest);
		return TRUE;
	}

	return FALSE;
}

BOOL CheckIfOKToClose(HWND hwnd)
{
	INT		ids;

	if (PGPclIsComponentInstalled(kPGPclOutlookExpressPlugin))
		ids = IDS_CHECKIFOKTOCLOSEOE;
	else
		ids = IDS_CHECKIFOKTOCLOSE;

	if (PTMessageBox(hwnd, IDS_CAPTION, ids, 
			MB_OKCANCEL|MB_ICONEXCLAMATION) == IDOK)
		return TRUE;
	else
		return FALSE;
}

static void sGetMenuFont (void)
{
	NONCLIENTMETRICS ncmetrics;

	ncmetrics.cbSize = sizeof (ncmetrics);
	SystemParametersInfo (SPI_GETNONCLIENTMETRICS, 0, &ncmetrics, 0);

	s_uMenuHeight = abs (ncmetrics.lfMenuFont.lfHeight) + 8;
	s_uBitmapPos = (s_uMenuHeight - BITMAP_HEIGHT) >> 1;
	s_hFontMenu = CreateFontIndirect (&ncmetrics.lfMenuFont);
}

//	________________________
//
//	append the hotkey keystroke combo to the menu item

static VOID
sSetPurgeMenuItemState (
	HMENU		hMenu)
{
	MENUITEMINFO	mii;
	UINT			iState	= MFS_DISABLED;
	PGPUInt32		uLocal, uGlobal, uLocalOther;

	if (PGPsc)
	{
		if (IsntPGPError (PGPCountCachedPassphrases (
				PGPsc, &uLocal, &uGlobal, &uLocalOther)))
		{
			if ((uLocal + uGlobal + uLocalOther) > 0)
				iState = MFS_ENABLED;
		}
	}

	mii.cbSize = sizeof(mii);
	mii.fMask = MIIM_STATE;
	mii.fType = MFT_STRING;
	mii.fState = iState;

	SetMenuItemInfo (hMenu, ID_PURGECACHES, FALSE, &mii);
}


//	________________________
//
//	append the hotkey keystroke combo to the menu item

static VOID
sSetHotKeyText (
	HMENU		hMenu, 
	INT			id,
	PGPUInt32	hotkey)
{
	CHAR			sz1[128];
	CHAR			sz2[16];
	MENUITEMINFO	mii;
	ULONG			uKey;
	PGPUInt32	uTotal		= 0;
	PGPUInt32	uLocal, uGlobal, uLocalOther;


	if (!(hotkey & kPGPHotKeyEnabled))
		return;

	// we can't handle non-ASCII hotkeys (which shouldn't exist anyway), 
	// so just in case we abort
	if (hotkey & 0x0000FF00)
		return;

	// if this is the "purge caches" menu item, then enable/disable on basis
	// of cache state

	// check if passphrases are cached and update tray icon accordingly
	if (PGPsc)
	{
		if (IsntPGPError (PGPCountCachedPassphrases (
				PGPsc, &uLocal, &uGlobal, &uLocalOther)))
		{
			uTotal = uLocal + uGlobal + uLocalOther;
		}
	}

	mii.cbSize = sizeof(mii);
	mii.fMask = MIIM_TYPE;
	mii.fType = MFT_STRING;
	mii.dwTypeData = sz1;
	mii.cch = sizeof(sz1)/2;

	// get required size of buffer
	if (GetMenuItemInfo (hMenu, id, FALSE, &mii))
	{
		lstrcat (sz1, "\t");

		if (hotkey & kPGPHotKeyCtrlModifier)
		{
			LoadString (g_hinst, IDS_HOTKEYCONTROL, sz2, sizeof(sz2));
			lstrcat (sz1, sz2);
		}
		if (hotkey & kPGPHotKeyShiftModifier)
		{
			LoadString (g_hinst, IDS_HOTKEYSHIFT, sz2, sizeof(sz2));
			lstrcat (sz1, sz2);
		}
		if (hotkey & kPGPHotKeyAltModifier)
		{
			LoadString (g_hinst, IDS_HOTKEYALT, sz2, sizeof(sz2));
			lstrcat (sz1, sz2);
		}

		uKey = hotkey & 0x0000FFFF;
		uKey = MapVirtualKey (uKey, 0);
		uKey <<= 16;
		GetKeyNameText (uKey, sz2, sizeof(sz2));
		lstrcat (sz1, sz2);

		SetMenuItemInfo (hMenu, id, FALSE, &mii);
	}
}

//	________________________
//
//	Get icon and tooltip text based on passphrase cache status

static HICON
sGetIconAndText (
		HWND	hwnd,
		LPSTR	psz,
		UINT	uLen)
{
	PGPUInt32	uTotal		= 0;
	PGPUInt32	uLocal, uGlobal, uLocalOther;

	// check if passphrases are cached and update tray icon accordingly
	if (PGPsc)
	{
		if (IsntPGPError (PGPCountCachedPassphrases (
				PGPsc, &uLocal, &uGlobal, &uLocalOther)))
		{
			uTotal = uLocal + uGlobal + uLocalOther;
		}
	}

	if (uTotal == 0)
	{
		LoadString (g_hinst, IDS_SZAPP, psz, uLen);
		return g_hIconTrayDefault;
	}
	else
	{
		LoadString (g_hinst, IDS_SZAPPCACHED, psz, uLen);
		return g_hIconTrayCached;
	}
}


//_________________________________________
//
//	update system tray icon

static BOOL 
sUpdateTrayIconAndText (
		HWND	hwnd,
		BOOL	bForceAddIcon)
{
	BOOL			result;
	NOTIFYICONDATA	nid;
	CHAR			sz[128];
	static BOOL		bIconIsVisible = FALSE;

	memset (&nid, 0x00, sizeof(NOTIFYICONDATA));
	nid.cbSize				= sizeof(NOTIFYICONDATA);
	nid.hWnd				= hwnd;
	nid.uID					= TASKBAR_TRAY_ID;
	nid.uFlags				= NIF_ICON | NIF_TIP;

	if (s_bIconEnabled)
	{
		nid.hIcon = sGetIconAndText (hwnd, sz, sizeof(sz));
		lstrcpy (nid.szTip, sz);

		if (bIconIsVisible && !bForceAddIcon)
			result = Shell_NotifyIcon (NIM_MODIFY, &nid);
		else
		{
			nid.uCallbackMessage = WM_TASKAREA_MESSAGE;
			nid.uFlags |= NIF_MESSAGE;
			result = Shell_NotifyIcon (NIM_ADD, &nid);
			if (result)
				bIconIsVisible = TRUE;
		}
	}
	else
	{
		if (bIconIsVisible)
		{
			result = Shell_NotifyIcon (NIM_DELETE, &nid);
			bIconIsVisible = FALSE;
		}
	}

	return result;
}


void PGPtrayCreateImageLists(
		HIMAGELIST *hIml,
		HIMAGELIST *hImlHeader,
		HIMAGELIST *hImlDiskHeader)
{
	int iNumBits;
	HDC hDC;
	HBITMAP hBmp;

	// ImageList Init

	hDC = GetDC (NULL);		// DC for desktop
	iNumBits = GetDeviceCaps (hDC, BITSPIXEL) * GetDeviceCaps (hDC, PLANES);
	ReleaseDC (NULL, hDC);

	if (iNumBits <= 8) {
		s_bHighColorDisplay = FALSE;

		*hIml =	ImageList_Create (
				BITMAP_WIDTH, BITMAP_HEIGHT, 
				ILC_COLOR|ILC_MASK, NUM_BITMAPS, 0); 
		hBmp = LoadBitmap (g_hinst, MAKEINTRESOURCE (IDB_EXEBITMAP4BIT));
		ImageList_AddMasked (*hIml, hBmp, RGB(255, 0, 255));
		DeleteObject (hBmp);

		*hImlHeader =	ImageList_Create (
				HEADER_WIDTH, HEADER_HEIGHT, 
				ILC_COLOR|ILC_MASK, 1, 0); 
		hBmp = LoadBitmap (g_hinst, MAKEINTRESOURCE (IDB_HEADER4));
		ImageList_AddMasked (*hImlHeader, hBmp, RGB(255, 0, 255));
		DeleteObject (hBmp);

		*hImlDiskHeader =	ImageList_Create (
				DISKHEADER_WIDTH, DISKHEADER_HEIGHT, 
				ILC_COLOR|ILC_MASK, 1, 0); 
		hBmp = LoadBitmap (g_hinst, MAKEINTRESOURCE (IDB_DISKHEADER4));
		ImageList_AddMasked (*hImlDiskHeader, hBmp, RGB(255, 0, 255));
		DeleteObject (hBmp);
	}
	else {
		s_bHighColorDisplay = TRUE;

		*hIml =	ImageList_Create (
				BITMAP_WIDTH, BITMAP_HEIGHT, 
				ILC_COLOR24|ILC_MASK, NUM_BITMAPS, 0); 
		hBmp = LoadBitmap (g_hinst, MAKEINTRESOURCE (IDB_EXEBITMAP24BIT));
		ImageList_AddMasked (*hIml, hBmp, RGB(255, 0, 255));
		DeleteObject (hBmp);

		*hImlHeader =	ImageList_Create (
				HEADER_WIDTH, HEADER_HEIGHT, 
				ILC_COLOR24|ILC_MASK, NUM_BITMAPS, 0); 
		hBmp = LoadBitmap (g_hinst, MAKEINTRESOURCE (IDB_HEADER24));
		ImageList_AddMasked (*hImlHeader, hBmp, RGB(255, 0, 255));
		DeleteObject (hBmp);

		*hImlDiskHeader =	ImageList_Create (
				DISKHEADER_WIDTH, DISKHEADER_HEIGHT, 
				ILC_COLOR24|ILC_MASK, NUM_BITMAPS, 0); 
		hBmp = LoadBitmap (g_hinst, MAKEINTRESOURCE (IDB_DISKHEADER24));
		ImageList_AddMasked (*hImlDiskHeader, hBmp, RGB(255, 0, 255));
		DeleteObject (hBmp);
	}
}

void MeasureMenuItem(LPMEASUREITEMSTRUCT mi)
{
    // create the font we will use for the title
    HFONT oldhfont;
	HDC hdc;
	SIZE size;
	PPGPMENUINFO pmi;

	if (mi->itemID == ID_PGPHEADERBITMAP)
	{
		PGPHotKeys*	phks;

		mi->itemHeight = HEADER_HEIGHT + HEADER_OFFSET;

		// if hotkey text is displayed, then menu gets too wide,
		// so we compensate here
		phks = PTGetHotKeyStruct ();
		if (phks->hotkeyPurgePassphraseCache & kPGPHotKeyEnabled)
			mi->itemWidth = 10;
		else
			mi->itemWidth = HEADER_WIDTH - GetSystemMetrics(SM_CXMENUCHECK) -12;
		return;
	}

	if (mi->itemID == ID_PGPDISKHEADERBITMAP)
	{
		mi->itemHeight = DISKHEADER_HEIGHT + DISKHEADER_OFFSET;
		mi->itemWidth = DISKHEADER_WIDTH - GetSystemMetrics(SM_CXMENUCHECK) -12;
		return;
	}

	pmi=(PPGPMENUINFO)mi->itemData;
	hdc=GetDC(NULL);
	oldhfont=SelectObject(hdc,s_hFontMenu);
	GetTextExtentPoint32(hdc,pmi->szString,strlen(pmi->szString),&size);
	SelectObject(hdc,oldhfont);

	// add in the left margin for the menu item
	size.cx += GetSystemMetrics(SM_CXMENUCHECK)+12;

	// return the width and height
	mi->itemWidth = size.cx;
	mi->itemHeight = s_uMenuHeight;
}

void DrawMenuItem(LPDRAWITEMSTRUCT di)
{
    // create the font we will use for the title
    HFONT oldhfont;
	HBRUSH hbgb;
	int mode;
	COLORREF text,back;
	PPGPMENUINFO pmi;
	RECT rc;

	pmi=(PPGPMENUINFO)di->itemData;

	if (di->itemID == ID_PGPHEADERBITMAP)
	{
		hbgb = CreateSolidBrush(GetSysColor(COLOR_MENU));
		FillRect(di->hDC, &di->rcItem, hbgb);
		DeleteObject(hbgb);

		if (s_bDrawDividerLine)
		{
			rc = di->rcItem;
			if (s_bTaskBarAtTop)
				rc.top += (HEADER_OFFSET -2);
			else
				rc.bottom -= (HEADER_OFFSET -2);

			DrawEdge (di->hDC, &rc, EDGE_ETCHED, BF_BOTTOM);
		}

		if (s_bHighColorDisplay)
			hbgb = CreateSolidBrush(RGB(176,170,142));
		else
			hbgb = GetStockObject(LTGRAY_BRUSH);

		rc = di->rcItem;
		if (s_bTaskBarAtTop)
			rc.top += HEADER_OFFSET;
		else
			rc.bottom -= HEADER_OFFSET;
		FillRect(di->hDC, &rc, hbgb);

		DeleteObject(hbgb);

		ImageList_Draw(hImlHeader,0,
				di->hDC,
				rc.left,
				rc.top,
				ILD_TRANSPARENT);
		return;
	}
	
	if (di->itemID == ID_PGPDISKHEADERBITMAP)
	{
		hbgb = CreateSolidBrush(GetSysColor(COLOR_MENU));
		FillRect(di->hDC, &di->rcItem, hbgb);
		DeleteObject(hbgb);

		if (s_bDrawDividerLine)
		{
			rc = di->rcItem;
			if (s_bTaskBarAtTop)
				rc.top += (HEADER_OFFSET -2);
			else
				rc.bottom -= (HEADER_OFFSET -2);

			DrawEdge (di->hDC, &rc, EDGE_ETCHED, BF_BOTTOM);
		}

		if (s_bHighColorDisplay)
			hbgb = CreateSolidBrush(RGB(176,170,142));
		else
			hbgb = GetStockObject(LTGRAY_BRUSH);

		rc = di->rcItem;
		rc.bottom -= HEADER_OFFSET;
		FillRect(di->hDC, &rc, hbgb);

		DeleteObject(hbgb);

		ImageList_Draw(hImlDiskHeader,0,
				di->hDC,
				di->rcItem.left,
				di->rcItem.top,
				ILD_TRANSPARENT);
		return;
	}
	
	if((BOOL)(di->itemState & ODS_SELECTED))
	{
		hbgb = CreateSolidBrush(GetSysColor(COLOR_HIGHLIGHT));
		FillRect(di->hDC, &di->rcItem, hbgb);
		DeleteObject(hbgb);

        // Set the text background and foreground colors
        text=SetTextColor(di->hDC, GetSysColor(COLOR_HIGHLIGHTTEXT));
        back=SetBkColor(di->hDC, GetSysColor(COLOR_HIGHLIGHT));
   	}
	else
	{
		hbgb = CreateSolidBrush(GetSysColor(COLOR_MENU));
		FillRect(di->hDC, &di->rcItem, hbgb);
		DeleteObject(hbgb);

        // Set the text background and foreground colors to the 
        // standard window colors
        text=SetTextColor(di->hDC, GetSysColor(COLOR_MENUTEXT));
        back=SetBkColor(di->hDC, GetSysColor(COLOR_MENU));
	}

	mode = SetBkMode(di->hDC, TRANSPARENT);

	ImageList_Draw(hIml,pmi->uIcon,
			di->hDC,
			di->rcItem.left+4,
			di->rcItem.top+s_uBitmapPos,
			ILD_TRANSPARENT);

	// add the menu margin offset
	di->rcItem.left += GetSystemMetrics(SM_CXMENUCHECK)+12;

	oldhfont = (HFONT)SelectObject(di->hDC, s_hFontMenu);

	// if disabled, draw "etched" text by drawing first
	// in white offset by 1 pixel, then in gray
	if((BOOL)(di->itemState & ODS_DISABLED))
	{
		if (!(BOOL)(di->itemState & ODS_SELECTED))
		{
			SetTextColor (di->hDC, GetSysColor(COLOR_3DHILIGHT));

			di->rcItem.left++;
			di->rcItem.top++;
			di->rcItem.bottom++;
			DrawText(di->hDC,pmi->szString, 
					-1, &di->rcItem, DT_SINGLELINE|DT_VCENTER|DT_LEFT);
			di->rcItem.left--;
			di->rcItem.top--;
			di->rcItem.bottom--;
		}

		SetTextColor (di->hDC, GetSysColor(COLOR_GRAYTEXT));
	}

	// draw the text left aligned and vertically centered
	DrawText(di->hDC,pmi->szString, 
			-1, &di->rcItem, DT_SINGLELINE|DT_VCENTER|DT_LEFT);

	SelectObject(di->hDC, oldhfont);
	SetBkMode(di->hDC, mode);
	SetTextColor(di->hDC, text);
	SetBkColor(di->hDC, back);
}

BOOL InPGPOperation = FALSE;
BOOL bInitingServices = TRUE;

static VOID
sHandlePrefsChange (HWND hwnd)
{
	PGPPrefRef	prefref;
	PGPError	err;

	// handle wipe on delete-related messages
	if ((!PGPclRunningUnderTerminalServer ())&&
		(!bInitingServices)) // WOD has to be initialized first 
		PTHandleWipeOnDeletePrefsMsg(hwnd);

	err=PGPclPeekClientLibPrefRefs(&prefref,NULL);
	if(IsntPGPError(err))
	{
		if (IsntPGPError (PGPGetPrefBoolean (prefref, 
				kPGPPrefShowTrayIcon, &s_bIconEnabled)))
		{
			sUpdateTrayIconAndText (hwnd, FALSE);
		}

		// hotkeys pref handler should be called last
		// since it will close prefs and free prefref
		PTHandleHotKeyPrefsChange(hwnd, PGPsc, prefref);

		// the PGPdisk prefs handler does not follow
		// the rules (i.e. it can invalidate the prefref),
		// so we treat it separately 
		PGPclPeekClientLibPrefRefs(&prefref,NULL);
		PTHandleDiskPrefsChange(hwnd, prefref);
	}
}

void WMCreatePostServiceInit(HWND hwnd)
{
	// perform PGPdisk initialization
	if (PGPdiskExists)
	{
		PTDiskStartup ();
		PTDiskMountStartupDisks (hwnd);
	}

	// start PGPdisk timer to unmount timed-out disks
	if (PGPdiskExists)
		SetTimer(hwnd, PGPDISK_TIMER_ID, PGPDISK_TIMER_MS, NULL);

	// start timer to query passphrase cache every so often,
	// just in case the notifications get out of sync
	SetTimer(hwnd, PGPPASSPHRASE_TIMER_ID, PGPPASSPHRASE_TIMER_MS, NULL);

	// initialize auto update thread
	PTStartAutoUpdater ();

	// load up prefs and set icons
	sHandlePrefsChange (hwnd);
}


static void
sSDKNotificationCallback (
		PGPUserValue			userval,
		PGPNotificationReason	reason,
		PGPUInt32				uParam1,
		PGPUInt32				uParam2)
{
	if (reason == kPGPNotification_PassphraseCacheChanged)
	{
		PostMessage ((HWND)userval, WM_APP, 0, 0);
	}
}


LRESULT CALLBACK EnclyptorProc(HWND hwnd, UINT msg, 
							   WPARAM wParam, LPARAM lParam)
{
	static char HelpFile[MAX_PATH + 1] = "\0";
	static HWND hwndFocus;

	if(!bInitingServices)
	{
		// check for changed prefs
		if(msg == g_uReloadPrefsMsg) 
			sHandlePrefsChange(hwnd);

		// check for hotkey-related messages
		PTCheckForHotKeyMsg(hwnd,PGPsc,msg);

		// check for PGPdisk-related messages
		PTCheckForDiskMsg(hwnd,msg,wParam,lParam);

		if(msg == g_uTaskbarRestart)
			sUpdateTrayIconAndText(hwnd, TRUE);
	}

	if((bInitingServices)&&(msg!=WM_CREATE))
		return DefWindowProc(hwnd, msg, wParam, lParam);

	switch(msg)
	{
		case WM_CREATE:
		{
			PGPclWindowsGenus	genus;

			InitCommonControls();
			sGetMenuFont ();

			// See which menu items to install
			PGPdiskExists=PGPclIsComponentInstalled (kPGPclPGPdisk);
			PGPtoolsExists=PGPclIsComponentInstalled (kPGPclPGPtools);
			PGPkeysExists=PGPclIsComponentInstalled (kPGPclPGPkeys);

			// PGPdisk only in biz and pp versions
/*			if(PGPclIsAuthorized())
			{
				PGPdiskExists=FALSE;
			}*/

			LoadString(g_hinst, IDS_PGPTOOLS, miTools.szString, sizeof(miTools.szString));
			LoadString(g_hinst, IDS_PGPDISK, miDisk.szString, sizeof(miDisk.szString));
			LoadString(g_hinst, IDS_PGPKEYS, miKeys.szString, sizeof(miKeys.szString));

			miKeys.uIcon=IDX_PGPKEYS;
			miTools.uIcon=IDX_PGPTOOLS;
			miDisk.uIcon=IDX_PGPDISK;

			PGPtrayCreateImageLists (&hIml,&hImlHeader,&hImlDiskHeader);

			// Load up default PGPtray icon
			PGPclGetWindowsVersion (NULL, &genus, NULL);
			if (genus == kPGPclWindowsGenusMillenium)
			{
				// Me can display hi-color icons in the system tray,
				g_hIconTrayDefault = 
						LoadImage (g_hinst, MAKEINTRESOURCE(IDI_TRAYICON8),
						IMAGE_ICON, 0, 0, 0);

				g_hIconTrayCached = 
						LoadImage (g_hinst, MAKEINTRESOURCE(IDI_TRAYICONCACHED8),
						IMAGE_ICON, 0, 0, 0);

				s_bDrawDividerLine = TRUE;
			}
			else if (genus >= kPGPclWindowsGenusXp)
			{
				// XP can display > 32-bit icons in the system tray,
				g_hIconTrayDefault = 
						LoadImage (g_hinst, MAKEINTRESOURCE(IDI_TRAYICON32),
						IMAGE_ICON, 0, 0, 0);

				g_hIconTrayCached = 
						LoadImage (g_hinst, MAKEINTRESOURCE(IDI_TRAYICONCACHED32),
						IMAGE_ICON, 0, 0, 0);

				s_bDrawDividerLine = FALSE;
			}
			else
			{
				// otherwise we need to use the 16 color
				g_hIconTrayDefault = 
						LoadImage (g_hinst, MAKEINTRESOURCE(IDI_TRAYICON4),
						IMAGE_ICON, 0, 0, 0);

				g_hIconTrayCached = 
						LoadImage (g_hinst, MAKEINTRESOURCE(IDI_TRAYICONCACHED4),
						IMAGE_ICON, 0, 0, 0);

				s_bDrawDividerLine = TRUE;
			}

			sUpdateTrayIconAndText (hwnd, FALSE);

			// register callback to get passphrase cache change notifications
			PGPSetNotificationCallback (sSDKNotificationCallback, hwnd);

			return 0;
		}

		case WM_APP:
		{
			sUpdateTrayIconAndText (hwnd, FALSE);
			return 0;
		}

		case WM_HOTKEY:
		{
			PTProcessHotKey(hwnd, PGPsc, wParam);
			return TRUE;
		}

		case WM_TIMER :
		{
			switch (wParam)
			{
			case PGPDISK_TIMER_ID:
				PTDiskCheckForTimedOutDisks(hwnd);
				break;

			case PGPPASSPHRASE_TIMER_ID:
				PostMessage (hwnd, WM_APP, 0, 0);
				break;
			}
			return 0;
		}

		case WM_DRAWITEM: 
        {
			DrawMenuItem((LPDRAWITEMSTRUCT)lParam);
			return TRUE;
        }

        case WM_MEASUREITEM:
		{
			MeasureMenuItem((LPMEASUREITEMSTRUCT)lParam);
            return TRUE;
		}

		case WM_QUERYOPEN:
		{
			return 0;
		}

		case WM_USERCHANGED:
		{
			PGPPurgePassphraseCache (PGPsc);
			PGPPurgeKeyDBCache (PGPsc);
			return 0;
		}

		case WM_MENUSELECT:
		{
			return 0;
		}

		case WM_TASKAREA_MESSAGE:
		{
			UINT uID = (UINT) wParam;
			UINT uMouseMsg = (UINT) lParam;

			if(uMouseMsg == WM_RBUTTONDOWN)
			{
				switch(uID)
				{
					case 1:
					{
						PopupTaskbarMenu(hwnd, InPGPOperation);
						return 0;
					}
				}
			}
			else if(uMouseMsg == WM_LBUTTONDOWN)
			{
				switch(uID)
				{
					case 1:
					{
						PopupTaskbarMenu(hwnd, InPGPOperation);
						return 0;
					}
				}
			}
			break;
		}

		case WM_COMMAND:
		{
			// Menu is greyed, but just in case hot keys are sending 
			// messages
			if(InPGPOperation)
				break;

			switch(LOWORD(wParam))
			{
				case ID_PURGECACHES:
				{
					InPGPOperation = TRUE;
					PTProcessHotKey(hwnd, PGPsc, kPurgePassphraseCache);
					InPGPOperation = FALSE;
					break;
				}

				case ID_ENCRYPTCLIPBOARD:
				{
					InPGPOperation = TRUE;
					PGPscEncryptClipboard(hwnd, szApp, PGPsc, PGPtls, TRUE, FALSE, FALSE);
					InPGPOperation = FALSE;
					break;
				}

				case ID_SIGNCLIPBOARD:
				{
					InPGPOperation = TRUE;
					PGPscEncryptClipboard(hwnd, szApp, PGPsc,PGPtls, FALSE, TRUE, FALSE);
					InPGPOperation = FALSE;
					break;
				}

				case ID_ENCRYPTSIGNCLIPBOARD:
				{
					InPGPOperation = TRUE;
					PGPscEncryptClipboard(hwnd, szApp, PGPsc,PGPtls, TRUE, TRUE, FALSE);
					InPGPOperation = FALSE;
					break;
				}

				case ID_DECRYPTVERIFYCLIPBOARD:
				{
					InPGPOperation = TRUE;
					PGPscDecryptClipboard(hwnd, szApp, PGPsc,PGPtls);
					InPGPOperation = FALSE;
					break;
				}

				case ID_ENCRYPTWINDOW:
				{
					BOOL bStartingCR;

					InPGPOperation = TRUE;
					if(PGPscDoCopy(hwnd,PGPsc,TRUE,&hwndFocus,&bStartingCR))
					{
						if(PGPscEncryptClipboard(hwnd, szApp, PGPsc, PGPtls, TRUE, FALSE, bStartingCR))
							PGPscDoPaste(TRUE,hwndFocus);
						else
							PGPscDoFocus(TRUE,hwndFocus);
					}
					InPGPOperation = FALSE;
					break;
				}

				case ID_SIGNWINDOW:
				{
					BOOL bStartingCR;

					InPGPOperation = TRUE;
					if(PGPscDoCopy(hwnd,PGPsc,TRUE,&hwndFocus,&bStartingCR))
					{
						if(PGPscEncryptClipboard(hwnd, szApp, PGPsc,PGPtls, FALSE, TRUE, bStartingCR))
							PGPscDoPaste(TRUE,hwndFocus);
						else
							PGPscDoFocus(TRUE,hwndFocus);
					}
					InPGPOperation = FALSE;
					break;
				}

				case ID_ENCRYPTSIGNWINDOW:
				{
					BOOL bStartingCR;

					InPGPOperation = TRUE;
					if(PGPscDoCopy(hwnd,PGPsc,TRUE,&hwndFocus,&bStartingCR))
					{
						if(PGPscEncryptClipboard(hwnd, szApp, PGPsc,PGPtls, TRUE, TRUE, bStartingCR))
							PGPscDoPaste(TRUE,hwndFocus);
						else
							PGPscDoFocus(TRUE,hwndFocus);
					}
					InPGPOperation = FALSE;
					break;
				}

				case ID_DECRYPTVERIFYWINDOW:
				{
					BOOL bStartingCR;

					InPGPOperation = TRUE;
					if(PGPscDoCopy(hwnd,PGPsc,TRUE,&hwndFocus,&bStartingCR))
					{
						PGPscDecryptClipboard(hwnd, szApp, PGPsc,PGPtls);
						PGPscDoFocus(TRUE,hwndFocus);
					}
					InPGPOperation = FALSE;
					break;
				}

				case ID_ABOUTPGP:
				{
					InPGPOperation = TRUE;
					PGPclHelpAbout (PGPsc, hwnd, NULL, NULL, NULL); 
					InPGPOperation = FALSE;
					break;
				}

				case ID_WIPECLIP:
				{
					InPGPOperation = TRUE;
					PGPscEraseClipboard(hwnd,PGPsc);
					InPGPOperation = FALSE;
					break;
				}
				
				case ID_VIEWCLIPBOARD:
				{
					InPGPOperation = TRUE;
					PGPscClipboardEditor(PGPsc,hwnd);
					InPGPOperation = FALSE;
					break;
				}
	
				case ID_LAUNCHPGPKEYS:
				{
					InPGPOperation = TRUE;
					PGPscDoLaunchKeys(hwnd);
					InPGPOperation = FALSE;
					break;
				}

				case ID_LAUNCHPGPTOOLS:
				{
					InPGPOperation = TRUE;
					PGPscDoLaunchTools(hwnd);
					InPGPOperation = FALSE;
					break;
				}

				case ID_PGPDISKMOUNTDISK:
				{
					InPGPOperation = TRUE;
					PTDiskOperation(hwnd, PGPDISK_MOUNTDISK);
					InPGPOperation = FALSE;
					break;
				}
				
				case ID_PGPDISKNEWDISK:
				{
					InPGPOperation = TRUE;
					PTDiskOperation(hwnd, PGPDISK_NEWDISK);
					InPGPOperation = FALSE;
					break;
				}

				case ID_PGPDISKEDITDISK:
				{
					InPGPOperation = TRUE;
					PTDiskOperation(hwnd, PGPDISK_EDITDISK);
					InPGPOperation = FALSE;
					break;
				}

				case ID_PGPDISKUNMOUNTALL:
				{
					InPGPOperation = TRUE;
					PTDiskOperation(hwnd, PGPDISK_UNMOUNTALL);
					InPGPOperation = FALSE;
					break;
				}

				case ID_PROPERTIES:
				{
					InPGPOperation = TRUE;
					PGPscPreferences(hwnd, PGPsc,PGPtls);
					InPGPOperation = FALSE;
					break;
				}

				case ID_QUIT_ENCLYPTOR:
				{
					InPGPOperation = TRUE;
#if !PGP_DEBUG
					// Hide it
					{
						PGPPrefRef prefref;

						if(IsntPGPError(PGPclPeekClientLibPrefRefs(&prefref,NULL)))
						{
							PGPSetPrefBoolean (prefref,kPGPPrefShowTrayIcon, FALSE);

							PGPclFlushClientLibPrefs (prefref, NULL);
						}
					}
#else
					// Quit it
					if (CheckIfOKToClose(hwnd))
					{
						DWORD	dwVersion;

						dwVersion = MAKELONG (
								MAKEWORD (PGPVERSIONSUBMINOR, PGPVERSIONMINOR),
								MAKEWORD (PGPVERSIONMAJOR, 0));

						SetForegroundWindow(hwnd);
						PostMessage(hwnd, WM_CLOSE, 0, dwVersion);
					}
#endif // !PGPDEBUG
					InPGPOperation = FALSE;
					break;
				}

				case ID_HELP_GENERAL:
				{
					InPGPOperation = TRUE;
					PGPclHtmlHelp (hwnd, kPGPclHelpContentsAndIndex, 0, 0,
							(char*)kPGPclHelpFile, NULL);
					InPGPOperation = FALSE;
					break;
				}
/*
///>>> 30sep02 pbj : 
	consolidated all help into single *.chm file and so only 
	one TOC exists.  Thus, merge all help into one menu item.

				case ID_HELP_MAIL:
				{
					InPGPOperation = TRUE;
					PGPclHtmlHelp (hwnd, kPGPclHelpContentsAndIndex, 0, 0,
							(char*)kPGPclMailHelpFile, NULL);
					InPGPOperation = FALSE;
					break;
				}

				case ID_HELP_DISK:
				{
					InPGPOperation = TRUE;
					PGPclHtmlHelp (hwnd, kPGPclHelpContentsAndIndex, 0, 0,
							(char*)kPGPclDiskHelpFile, NULL);
					InPGPOperation = FALSE;
					break;
				}

///<<<
*/
				case ID_LICENSE:
				{
					InPGPOperation = TRUE;

					PGPlnLicenseCheck (PGPsc);

					if(PGPlnWasAuthorized())
					{
						PGPclDisplayLicense (PGPsc, NULL); 
					}
					else
					{
						PGPclEnterLicense(PGPsc,NULL);
					}
					InPGPOperation = FALSE;
					break;
				}
			}
			break;
		}
		
		case WM_ENDSESSION :
		{
			if (wParam)
			{
				PGPPurgePassphraseCache (PGPsc);
				PGPPurgeKeyDBCache (PGPsc);
				UninitializeWipeOnDelete();
			}
			break;
		}

		case WM_CLOSE:
		{	
			BOOL	bOldInstaller	= FALSE;
			DWORD	dwVersion;

			dwVersion = MAKELONG (
					MAKEWORD (PGPVERSIONSUBMINOR, PGPVERSIONMINOR),
					MAKEWORD (PGPVERSIONMAJOR, 0));

			// if lParam is < our current version, then the message
			// did not come from our current (or subsequent) installer.  
			// Could be from a previous PGP installer (which we must
			// alert user about) or it could be from OS (e.g. Win9x
			// sends a WM_CLOSE when attempting to kill an app via
			// the task manager).  We have to try and figure out where
			// the message came from ...
			if ((DWORD)lParam < dwVersion)
			{
				HWND	hOldInstaller	= NULL;

				// if lParam != zero, then the message most likely
				// came from our own installer
				if (lParam != 0)
					bOldInstaller = TRUE;

				// otherwise we have to look for Windows suggesting
				// that one of our previous installers is running
				else
				{
					if (FindWindow ("isdelete", "InstallSHIELD Delete"))
					{
						// 6.5.x installers ran from a PGP SDA
						if (FindWindow ("PGPsda_Hidden_Window", 
								"PGPsda_Hidden_Window"))
						{
							bOldInstaller = TRUE;
							hOldInstaller = FindWindow (
									"InstallShield_Win", "Setup");
						}
						// 6.0.x installers had a "PGP ... " window
						else
						{
							hOldInstaller = FindWindow (
									"InstallShield_Win", NULL);
							if (hOldInstaller)
							{
								CHAR	sz[32];

								GetWindowText (hOldInstaller, sz, sizeof(sz));
								if (strncmp (sz, "PGP", 3) == 0)
									bOldInstaller = TRUE;
								else
									hOldInstaller = NULL;
							}
						}
					}
				}

				if (hOldInstaller)
					PostMessage (hOldInstaller, WM_CLOSE, 0, 0);

				if (bOldInstaller)
				{
					PTMessageBox (hwnd, IDS_UNINSTALLCAPTION, 
							IDS_PLEASEUNINSTALL, MB_ICONHAND|MB_OK|
							MB_SYSTEMMODAL|MB_SETFOREGROUND);
				}
			}

			if (!bOldInstaller)
			{
				PTStopAutoUpdater ();
				PTRemoveHotKeys(hwnd);
				DestroyWindow(hwnd);
			}
			return 0;
		}

		case WM_DESTROY:
		{	
			DeleteObject (s_hFontMenu);
			PostQuitMessage(0);	
			return 0;
		}

		case WM_POWERBROADCAST:
		{
			if (wParam == PBT_APMSUSPEND)
			{
				PGPPurgePassphraseCache(PGPsc);
				PGPPurgeKeyDBCache(PGPsc);
			}
			return PTDiskHandlePowerBroadcast(hwnd, wParam, lParam);
		}
	}
	return DefWindowProc(hwnd, msg, wParam, lParam);
}

void InsertOwnerDrawnMenuItem(HMENU hMenu,UINT pos,
							  UINT id,HMENU hSubMenu,
							  PPGPMENUINFO pmi,UINT iState)
{
	MENUITEMINFO mii;
	BOOL result;

	memset(&mii,0x00,sizeof(MENUITEMINFO));

	mii.cbSize=sizeof(MENUITEMINFO);
	mii.fMask=MIIM_TYPE|MIIM_DATA|MIIM_ID|MIIM_STATE|MIIM_SUBMENU;
	mii.fType=MFT_OWNERDRAW;
	mii.dwItemData=(DWORD)pmi;
	mii.wID=id;
	mii.fState=iState;
	mii.hSubMenu=hSubMenu;

	result=InsertMenuItem(hMenu, 
		pos, 
		TRUE,  
		&mii); 
}

BOOL PopupTaskbarMenu(HWND hwndTarget, BOOL InPGPOperation)
{
	UINT  uHelpPopupPos = 0;
	HMENU hMenu = NULL;
	HMENU hSubMenu = NULL;
	HMENU hMenuDisk = NULL;
	HMENU hPopupMenu = NULL;

	POINT pt;
	PGPHotKeys*	phks;

	GetCursorPos( &pt );

	// If the cursor is at the top, we need to reverse the menu
	if(pt.y<200)
	{
		s_bTaskBarAtTop = TRUE;
		uHelpPopupPos = 5;

		// load up menu
		hMenu = LoadMenu(g_hinst, MAKEINTRESOURCE(IDM_TASKBARMENUREV));
		hSubMenu = GetSubMenu(hMenu,0);

		phks = PTGetHotKeyStruct ();
		sSetHotKeyText (hSubMenu, ID_PURGECACHES, phks->hotkeyPurgePassphraseCache);
		sSetPurgeMenuItemState (hSubMenu);

		InsertOwnerDrawnMenuItem(
				hSubMenu,12,ID_PGPHEADERBITMAP,NULL,NULL,MFS_DISABLED);
#if !PGP_DEBUG
		{
			MENUITEMINFO mii;

			// Replace the "Exit" menu item with "Hide"
			char szHide[20];

			LoadString (g_hinst, IDS_HIDE, szHide, sizeof(szHide));

			mii.cbSize = sizeof(mii);
			mii.fMask = MIIM_TYPE;
			mii.fType = MFT_STRING;
			mii.dwTypeData = szHide;
			mii.cch = lstrlen (szHide);

			SetMenuItemInfo (hSubMenu, 10, TRUE, &mii); 
		}
#endif // PGP_DEBUG

		if ((PGPdiskExists))//&&(PGPclPGPdiskEnabled()))
		{
			// load up PGPdisk's popup menu
			hMenuDisk = LoadMenu(g_hinst, MAKEINTRESOURCE(IDM_PGPDISKPOPUP));
			hPopupMenu = GetSubMenu(hMenuDisk,0);

			InsertOwnerDrawnMenuItem(
					hPopupMenu,0,ID_PGPDISKHEADERBITMAP,NULL,NULL,MFS_DISABLED);

			InsertOwnerDrawnMenuItem(
					hSubMenu,3,ID_LAUNCHPGPDISK,hPopupMenu,&miDisk,MFS_ENABLED);
			uHelpPopupPos++;

			// add hotkey text to the menu items
			sSetHotKeyText (hPopupMenu, ID_PGPDISKUNMOUNTALL, phks->hotkeyDiskUnmount);
		}
		if (PGPkeysExists)
		{
			InsertOwnerDrawnMenuItem(
					hSubMenu,3,ID_LAUNCHPGPKEYS,NULL,&miKeys,MFS_ENABLED);
			uHelpPopupPos++;
		}
		if (PGPtoolsExists)
		{
			InsertOwnerDrawnMenuItem(
					hSubMenu,3,ID_LAUNCHPGPTOOLS,NULL,&miTools,MFS_ENABLED);
			uHelpPopupPos++;

			// add hotkey text to the menu items
			sSetHotKeyText (hSubMenu, ID_DECRYPTVERIFYWINDOW, phks->hotkeyDecrypt);
			sSetHotKeyText (hSubMenu, ID_ENCRYPTSIGNWINDOW, phks->hotkeyEncryptSign);
			sSetHotKeyText (hSubMenu, ID_SIGNWINDOW, phks->hotkeySign);
			sSetHotKeyText (hSubMenu, ID_ENCRYPTWINDOW, phks->hotkeyEncrypt);
		}
		else
		{
			// Remove current window and clipboard submenus
			RemoveMenu (hSubMenu, 0, MF_BYPOSITION);
			RemoveMenu (hSubMenu, 0, MF_BYPOSITION);
			RemoveMenu (hSubMenu, 0, MF_BYPOSITION);
			uHelpPopupPos-=3;
		}
	}
	else
	{
		s_bTaskBarAtTop = FALSE;
		uHelpPopupPos = 4;

		// load up menu
		hMenu = LoadMenu(g_hinst, MAKEINTRESOURCE(IDM_TASKBARMENU));
		hSubMenu = GetSubMenu(hMenu,0);

		phks = PTGetHotKeyStruct ();
		sSetHotKeyText (hSubMenu, ID_PURGECACHES, phks->hotkeyPurgePassphraseCache);
		sSetPurgeMenuItemState (hSubMenu);

		if(PGPtoolsExists)
		{
			// add hotkey text to the menu items
			sSetHotKeyText (hSubMenu, ID_DECRYPTVERIFYWINDOW, phks->hotkeyDecrypt);
			sSetHotKeyText (hSubMenu, ID_ENCRYPTSIGNWINDOW, phks->hotkeyEncryptSign);
			sSetHotKeyText (hSubMenu, ID_SIGNWINDOW, phks->hotkeySign);
			sSetHotKeyText (hSubMenu, ID_ENCRYPTWINDOW, phks->hotkeyEncrypt);

			InsertOwnerDrawnMenuItem(
					hSubMenu,9,ID_LAUNCHPGPTOOLS,NULL,&miTools,MFS_ENABLED);
		}
		else
		{
			// Remove current window and clipboard submenus
			RemoveMenu (hSubMenu, 9, MF_BYPOSITION);
			RemoveMenu (hSubMenu, 9, MF_BYPOSITION);
			RemoveMenu (hSubMenu, 9, MF_BYPOSITION);
		}

		// Make these owner drawn so we can get icons
		if (PGPkeysExists)
		{
			InsertOwnerDrawnMenuItem(
					hSubMenu,9,ID_LAUNCHPGPKEYS,NULL,&miKeys,MFS_ENABLED);
		}
		if ((PGPdiskExists))//&&(PGPclPGPdiskEnabled()))
		{
			// load up PGPdisk's popup menu
			hMenuDisk = LoadMenu(g_hinst, MAKEINTRESOURCE(IDM_PGPDISKPOPUP));
			hPopupMenu = GetSubMenu(hMenuDisk,0);
			InsertOwnerDrawnMenuItem(
					hPopupMenu,0,ID_PGPDISKHEADERBITMAP,NULL,NULL,MFS_DISABLED);

			InsertOwnerDrawnMenuItem(
					hSubMenu,9,ID_LAUNCHPGPDISK,hPopupMenu,&miDisk,MFS_ENABLED);

			// add hotkey text to the menu items
			sSetHotKeyText (hPopupMenu, ID_PGPDISKUNMOUNTALL, phks->hotkeyDiskUnmount);
		}

#if !PGP_DEBUG
		{
			MENUITEMINFO mii;

			// Replace the "Exit" menu item with "Hide"
			char szHide[20];

			LoadString (g_hinst, IDS_HIDE, szHide, sizeof(szHide));

			mii.cbSize = sizeof(mii);
			mii.fMask = MIIM_TYPE;
			mii.fType = MFT_STRING;
			mii.dwTypeData = szHide;
			mii.cch = lstrlen (szHide);

			SetMenuItemInfo (hSubMenu, 0, TRUE, &mii); 
		}
#endif // PGP_DEBUG

		// add graphical header
		InsertOwnerDrawnMenuItem(
				hSubMenu,0,ID_PGPHEADERBITMAP,NULL,NULL,MFS_DISABLED);
		uHelpPopupPos++;
	}

#if (!PGP_LICENSENUMBERS)
	EnableMenuItem(hSubMenu, 
		ID_LICENSE, 
		MF_BYCOMMAND | MF_GRAYED);
#endif

/*
///>>> 30sep02 pbj : 
	consolidated all help into single *.chm file and so only 
	one TOC exists.  Thus, merge all help into one menu item.

  // remove help menu items for those components which are not installed
	hPopupMenu = GetSubMenu(hSubMenu,uHelpPopupPos);
	if(!PGPtoolsExists)
		RemoveMenu (hPopupMenu, ID_HELP_MAIL, MF_BYCOMMAND);
	if(!((PGPdiskExists)))//&&(PGPclPGPdiskEnabled())))
		RemoveMenu (hPopupMenu, ID_HELP_DISK, MF_BYCOMMAND);

///<<<
*/
	s_hMenu = hSubMenu;

	// Chances are the hooker is disabled, but it won't work
	// without a non NULL hwnd anyway...
	if(PGPscReadCurrentFocus()==NULL)
	{
		EnableMenuItem(hMenu, 
					   ID_DECRYPTVERIFYWINDOW, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_ENCRYPTSIGNWINDOW, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_SIGNWINDOW, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_ENCRYPTWINDOW, 
					   MF_BYCOMMAND | MF_GRAYED);
	}

	if(InPGPOperation)
	{
		EnableMenuItem(hMenu, 
					   ID_ABOUTPGP, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_ENCRYPTCLIPBOARD, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_SIGNCLIPBOARD, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_ENCRYPTSIGNCLIPBOARD, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_DECRYPTVERIFYCLIPBOARD, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_ENCRYPTWINDOW, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_SIGNWINDOW, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_ENCRYPTSIGNWINDOW, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_DECRYPTVERIFYWINDOW, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_PROPERTIES, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_VIEWCLIPBOARD, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_HELP_GENERAL, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_PURGECACHES, 
					   MF_BYCOMMAND | MF_GRAYED);
/*
///>>> 30sep02 pbj : 
	consolidated all help into single *.chm file and so only 
	one TOC exists.  Thus, merge all help into one menu item.

		EnableMenuItem(hMenu, 
					   ID_HELP_MAIL, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_HELP_DISK, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_HELP_VPN, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_HELP_FIRE, 
					   MF_BYCOMMAND | MF_GRAYED);

///<<<
*/
		EnableMenuItem(hMenu, 
					   ID_LAUNCHPGPKEYS, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_LAUNCHPGPTOOLS, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_WIPECLIP, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_LAUNCHPGPTOOLS, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_QUIT_ENCLYPTOR, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, 
					   ID_LICENSE, 
					   MF_BYCOMMAND | MF_GRAYED);

		// PGPdisk submenu
		EnableMenuItem(hMenuDisk, 
					   ID_PGPDISKMOUNTDISK, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenuDisk, 
					   ID_PGPDISKNEWDISK, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenuDisk, 
					   ID_PGPDISKEDITDISK, 
					   MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenuDisk, 
					   ID_PGPDISKUNMOUNTALL, 
					   MF_BYCOMMAND | MF_GRAYED);
	}

	//  Calls to SetForegroundWindow and PostMessage to fix a bug
    //  documented in PSS ID Number: Q135788
	SetForegroundWindow(hwndTarget); 
	TrackPopupMenu( hSubMenu, TPM_LEFTALIGN | TPM_LEFTBUTTON, 
					pt.x, pt.y, 0, hwndTarget, NULL);
	PostMessage( hwndTarget, WM_NULL, 0, 0 ) ;
	// Above fixes "three clicks" problem  -wjb

	DestroyMenu(hMenu);

	return TRUE;
}

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
