/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	$Id: SpaceEdit.c,v 1.14 2002/11/19 00:47:51 wjb Exp $

	This is a Unicode version of Bill Blanke's original SpaceEdit subclass
	code.  This version stores the passphrase internally as a wide char
	(UCS-2) string and converts it to/from UTF-8 and local code page format
	as appropriate.

	Also, this version handles all drawing of the text to the screen.  This
	allows better non-ASCII support on Win9x platforms and also avoids 
	giving the passphrase string to the RichEdit control where it may be
	susceptible to being left in memory somewhere.
____________________________________________________________________________*/

#include "stdio.h"
#include "stdlib.h"
#include "windows.h"
#include "richedit.h"
#include "pgpKeys.h"
#include "pgpConfig.h"
#include "pgpErrors.h"
#include "pgpEncode.h"
#include "pgpUtilities.h"
#include "pflPrefTypes.h"
#include "pgpPubTypes.h"
#include "pgpPassphraseUtils.h"
#include "pgpUnicode.h"
#include "pgpUnicodeWin32.h"
#include "SpaceEdit.h"


#define SE_BUFSIZE		255


typedef struct _SE
{
	HWND		hwndEdit;
	BOOL		bUnicodeOS;
	BOOL		bGotLanguageNotification;
	BOOL		bHideTyping;
	BOOL		bFocused;
	BOOL		bShowWarning;
	WNDPROC		wpOrigProc;
	WCHAR		wszRealText[SE_BUFSIZE +1];
	PGPInt16	iNumSpaces[SE_BUFSIZE +1];
	int			iLen;
	int			iCaretHeight;
	int			iRowHeight;
	POINT		ptCaret;
	RECT		rcClient;
	int			iCharSet;
	DWORD		dwSpecialKey;
	DWORD		dwQuality;

	HDC			hdcMem;
	HBITMAP		hbmMem;

	int			iMaxLen;

	USERCALLBACK UserCallback;
	void *		pUserValue;
} SE,*SEP;


//_________________________________________________


#if PGP_SDK_AVAILABLE
VOID* 
SEsecAlloc (PGPContextRef context, UINT uBytes) 
{
	PGPMemoryMgrRef	memmgr;

	memmgr =  PGPPeekContextMemoryMgr (context);
	return (PGPNewSecureData (memmgr, uBytes, 0));
}

VOID 
SEsecFree (VOID* p, UINT uBytes) 
{
	if (p) 
	{
		FillMemory ((char *)p, uBytes, '\0');
		PGPFreeData ((char *)p);
	}
}
#endif

static void 
sCalculateEstimatedQuality (SEP sep)
{
#if PGP_SDK_AVAILABLE
	INT		i;
	char	szRealText[SE_BUFSIZE +1];
	PGPUInt32 ucopied;

///>>> fix this function!!!
	// "de-obfuscate" the string for computing quality
	for (i=0; i<sep->iLen ;i++)
		sep->wszRealText[i]^=sep->iNumSpaces[i];

	pgpUCS2StringToUTF8( sep->wszRealText, sep->iLen, szRealText, sizeof(szRealText)-1, &ucopied );

	sep->dwQuality = ucopied ? pgpEstimatePassphraseQuality (szRealText) : 0;

	memset(szRealText, 0, sizeof(szRealText));

	// "re-obfuscate" the string
	for (i=0; i<sep->iLen ;i++)
		sep->wszRealText[i]^=sep->iNumSpaces[i];
#endif
}

// quick and dirty function to convert language ID to character set.
static INT 
sGetCharset (VOID)
{
	DWORD	dwLang = (DWORD)GetKeyboardLayout (0);
	dwLang = LOWORD (dwLang);

	switch (dwLang) {
	case 0x0408 :
		return GREEK_CHARSET;
	case 0x040D :
		return HEBREW_CHARSET;
	case 0x0419 :
		return RUSSIAN_CHARSET;
	case 0x041E :
		return THAI_CHARSET;
	case 0x041F :
		return TURKISH_CHARSET;
	case 0x042A :
		return VIETNAMESE_CHARSET;
	case 0x0812 :
		return JOHAB_CHARSET;
	default :
		if ((dwLang & 0xFF) == 0x01)
			return ARABIC_CHARSET;
		else if ((dwLang & 0xFF) == 0x04)
			return CHINESEBIG5_CHARSET;
		else
			return ANSI_CHARSET;
	}
}

static void 
sCheckCaps (SEP sep)
{
	if (GetKeyState(VK_CAPITAL) & 1)
	{
		sep->bShowWarning=TRUE;

		if(sep->UserCallback)
			(sep->UserCallback)(sep->hwndEdit,sep->pUserValue);
	}
	else
	{
		sep->bShowWarning=FALSE;

		if(sep->UserCallback)
			(sep->UserCallback)(sep->hwndEdit,sep->pUserValue);
	}
}

static void
sDrawText (
		HDC hdc,
		SEP	sep)
{
	HFONT	hFont, hFontOld;
	int		i, ich, iYOffset;
	WCHAR	wch;
	SIZE	size;

	if (sep->bUnicodeOS)
	{
		hFont = GetStockObject (DEFAULT_GUI_FONT);
	}
	else
	{
		LOGFONT	lf;
		ZeroMemory (&lf, sizeof(lf));
		lf.lfHeight = -MulDiv(8, GetDeviceCaps (hdc, LOGPIXELSY), 72);
		lf.lfCharSet = sep->iCharSet;
		lf.lfPitchAndFamily = DEFAULT_PITCH | FF_SWISS;
		hFont = CreateFontIndirect (&lf);
	}
	hFontOld = SelectObject (sep->hdcMem, hFont);

	iYOffset = 0;

start:
	FillRect (sep->hdcMem, &sep->rcClient, (HBRUSH)(COLOR_WINDOW+1));

	sep->ptCaret.x = 0;
	sep->ptCaret.y = -iYOffset;

	for (i=0; i<sep->iLen; i++)
	{
		if (sep->bHideTyping)
		{
			ich = (sep->iNumSpaces[i] & 0x03) +1;
			wch = 0x20;
		}
		else
		{
			ich = 1;
			wch = sep->wszRealText[i]^sep->iNumSpaces[i];
		}

		while (--ich >= 0)
		{
			GetTextExtentPoint32W (sep->hdcMem, &wch, 1, &size);

			if (sep->ptCaret.x + size.cx >= sep->rcClient.right)
			{
				sep->ptCaret.x = 0;
				sep->ptCaret.y += sep->iRowHeight;

				if (sep->ptCaret.y + sep->iRowHeight > sep->rcClient.bottom)
				{
					sep->ptCaret.y -= sep->iRowHeight;
					iYOffset += sep->iRowHeight;
					goto start;
				}

				ExtTextOutW (sep->hdcMem, sep->ptCaret.x, sep->ptCaret.y, 
						ETO_CLIPPED, &sep->rcClient, &wch, 1, NULL);
			}
			else
			{
				ExtTextOutW (sep->hdcMem, sep->ptCaret.x, sep->ptCaret.y, 
						ETO_CLIPPED, &sep->rcClient, &wch, 1, NULL);
			}

			sep->ptCaret.x += size.cx;
		}
	}

	SelectObject (sep->hdcMem, hFontOld);

	BitBlt (hdc, 
			sep->rcClient.left, sep->rcClient.top, 
			sep->rcClient.right-sep->rcClient.left, 
			sep->rcClient.bottom-sep->rcClient.top,
			sep->hdcMem, 0, 0, SRCCOPY);
}


//	______________________________________
//
//  Tab focus to next or previous control on basis of shift key

static VOID
sTabToNextControl (
		HWND		hwndOrig)
{
	UINT	uDir			= GW_HWNDNEXT;
	UINT	uReset			= GW_HWNDFIRST;
	HWND	hwnd1, hwnd2;

	if (GetKeyState (VK_SHIFT) & 0x8000)
	{
		uDir = GW_HWNDPREV;
		uReset = GW_HWNDLAST;
	}

	hwnd1 = hwndOrig;

	do 
	{
		hwnd2 = GetNextWindow (hwnd1, uDir);
		if (hwnd2 == NULL)
			hwnd2 = GetNextWindow (hwnd1, uReset);
		if (hwnd2 == NULL)
			return;

		if (GetWindowLong (hwnd2, GWL_STYLE) & WS_TABSTOP)
		{
			if (IsWindowEnabled (hwnd2))
			{
				SetFocus (hwnd2);
				return;
			}
		}

		hwnd1 = hwnd2;

	} while (hwnd2 != hwndOrig);
}


//	______________________________________
//
//  common message procedure for Unicode and ANSI versions

static BOOL
sCommonMsgProc (
		HWND		hWnd, 
		UINT		uMsg, 
		WPARAM		wParam, 
		LPARAM		lParam,
		LRESULT*	plResult)
{
	*plResult = 0;

	switch (uMsg) 
	{
		case WM_INPUTLANGCHANGE :
		{
			SEP sep = (SEP)GetWindowLong (hWnd, GWL_USERDATA);
			sep->iCharSet = LOWORD(wParam);
			sep->bGotLanguageNotification = TRUE;
			InvalidateRect (hWnd, NULL, FALSE);
			return FALSE;
		}

		case WM_SETFOCUS :
		{
			SEP sep = (SEP)GetWindowLong (hWnd, GWL_USERDATA);
			sep->bFocused = TRUE;
			CreateCaret (hWnd, NULL, 0, sep->iCaretHeight);
			ShowCaret (hWnd);
			InvalidateRect (hWnd, NULL, FALSE);
			return TRUE;
		}

		case WM_GETDLGCODE :
			*plResult = DLGC_WANTTAB|DLGC_WANTCHARS;
			return TRUE;

		case WM_KILLFOCUS :
		{
			SEP sep = (SEP)GetWindowLong (hWnd, GWL_USERDATA);
			sep->bFocused = FALSE;
			DestroyCaret ();
			return TRUE;
		}

		case WM_LBUTTONDOWN:
		{
			SetFocus (hWnd);
			return TRUE;
		}

		case WM_KEYUP:
		{
			SEP sep = (SEP)GetWindowLong (hWnd, GWL_USERDATA);
			sCheckCaps (sep);

			switch (wParam) 
			{
				case VK_HOME :
				case VK_END :
				case VK_UP :
				case VK_DOWN :
				case VK_LEFT :
				case VK_RIGHT :
				case VK_NEXT :
				case VK_PRIOR :
					return TRUE;

				case VK_BACK :
				{
					SEP sep = (SEP)GetWindowLong (hWnd, GWL_USERDATA);
					if (sep->iLen > 0)
						return TRUE;
					break;
				}
			}
			break;
		}

		case WM_RBUTTONDOWN:
		case WM_LBUTTONDBLCLK :
		case WM_MOUSEMOVE :
		case WM_COPY :
		case WM_CUT :
		case WM_PASTE :
		case WM_CLEAR :
			return TRUE;

		case WM_KEYDOWN :
		{
			switch (wParam) 
			{
				case VK_HOME :
				case VK_END :
				case VK_UP :
				case VK_DOWN :
				case VK_LEFT :
				case VK_RIGHT :
				case VK_NEXT :
				case VK_PRIOR :
					return TRUE;

				case VK_TAB :
					sTabToNextControl (hWnd);
					return TRUE;

				case VK_BACK :
				{
					SEP sep = (SEP)GetWindowLong (hWnd, GWL_USERDATA);
					if (sep->iLen > 0)
						return TRUE;
					break;
				}
			}
			break;
		}

		case WM_GETTEXT :
		{
			return TRUE;
		}

		case WM_PAINT :
		{
			SEP sep = (SEP)GetWindowLong (hWnd, GWL_USERDATA);
			PAINTSTRUCT	ps;
			HDC	hdc;

			hdc = BeginPaint (hWnd, &ps);
			if (hdc)
			{
				sDrawText (hdc, sep);
				EndPaint (hWnd, &ps);
				
				if (sep->bFocused)
					SetCaretPos (sep->ptCaret.x+1, sep->ptCaret.y+1);
			}

			return TRUE;
		}
	}

	return FALSE;
}


//	______________________________________
//
//  Passphrase edit box subclass procedure

LRESULT APIENTRY 
SESubclassProc (
		HWND	hWnd, 
		UINT	uMsg, 
		WPARAM	wParam, 
		LPARAM	lParam) 
{
	LRESULT		lResult;
	SEP			sep;

	if (sCommonMsgProc (hWnd, uMsg, wParam, lParam, &lResult))
		return lResult;

	sep=(SEP)GetWindowLong (hWnd, GWL_USERDATA);

	switch (uMsg) 
	{

#if !PGP_SDA	// disable Global IME support in SDAs -- 
				//   thems the breaks for Win95 support for now
		case WM_IME_COMPOSITION :
		{
			if (lParam & GCS_RESULTSTR) 
			{
				HIMC	hIMC = ImmGetContext (hWnd);
				int		i;

				if (hIMC) 
				{
					// ImmGetCompositionStringW returns bytes
					i = ImmGetCompositionStringW (
							hIMC, GCS_RESULTSTR, NULL, 0);
					i = min (i>>1, SE_BUFSIZE-sep->iLen);

					// get the actual string
					ImmGetCompositionStringW (
							hIMC, GCS_RESULTSTR, 
							&(sep->wszRealText[sep->iLen]), i<<1);

					// obfuscate
					while (i > 0)
					{
						sep->wszRealText[sep->iLen] ^= 
								sep->iNumSpaces[sep->iLen];
						sep->iLen++;
						i--;
					}

					ImmReleaseContext (hWnd, hIMC);
				}

				InvalidateRect (hWnd, NULL, FALSE);

				sCalculateEstimatedQuality (sep);

				if (sep->UserCallback)
					(sep->UserCallback)(hWnd,sep->pUserValue);

				return 0;
            }
   			break;
		}
#endif // !PGP_SDA

		case WM_CHAR :
		{
			WCHAR wchCharCode;
			int strsize;

			strsize = sep->iLen;
			wchCharCode = (WCHAR)wParam;

			if ((wchCharCode == VK_ESCAPE) || (wchCharCode == VK_RETURN))
			{
				sep->dwSpecialKey = wchCharCode;
				return 0;
			}

			// look for backspace characters
			if (wchCharCode == 0x08)
			{
				strsize--;

				// Backspace it for us
				if (strsize >= 0)
				{
					sep->wszRealText[strsize] = 0;
					sep->iLen--;

					InvalidateRect (hWnd, NULL, FALSE);

					sCalculateEstimatedQuality (sep);

					if (sep->UserCallback)
						(sep->UserCallback)(hWnd,sep->pUserValue);
				}

				return 0;
			}

			if (strsize == SE_BUFSIZE)
				return 0;

			if (strsize >= sep->iMaxLen)
				return 0;

			if (iscntrl (wchCharCode))
				return 0;

			// if character is in local code page, convert to Unicode
			if (wchCharCode > 127)
			{
				if (!sep->bUnicodeOS)
				{
					unsigned char chCharCode = LOBYTE (wchCharCode);
					PGPUInt32	u;
					CHARSETINFO	cs;

					if (!sep->bGotLanguageNotification)
						sep->iCharSet = sGetCharset ();

					TranslateCharsetInfo (
							(DWORD*)(sep->iCharSet), &cs, TCI_SRCCHARSET);

					pgpLocalStringToUCS2 (kPGPUnicodeFlag_Secure, 
							cs.ciACP, &chCharCode, 1, 
							&wchCharCode, 1, &u);
				}
			}

			// This is XOR'ed to obsfucate, just in case something
			// "really bad" happens. The memory should be locked,
			// cleared and freed on exit however.
			sep->wszRealText[strsize]=wchCharCode^sep->iNumSpaces[strsize];
			sep->iLen++;

			InvalidateRect (hWnd, NULL, FALSE);

			sCalculateEstimatedQuality (sep);

			if (sep->UserCallback)
				(sep->UserCallback)(hWnd,sep->pUserValue);

			return 0;
		}
	}

	if (sep->bUnicodeOS)
	{
		return CallWindowProcW (sep->wpOrigProc, 
				hWnd, uMsg, wParam, lParam);
	}
	else
	{
		return CallWindowProcA (sep->wpOrigProc, 
				hWnd, uMsg, wParam, lParam);
	}
} 


//	______________________________________
//
//  initialize the subclassing

VOID
SEInit(
#if PGP_SDK_AVAILABLE
	   PGPContextRef	context,
#endif
	   HWND				hwndEdit,
	   BOOL				bHideTyping,
	   USERCALLBACK		UserCallback,
	   void*			pUserValue)
{
	OSVERSIONINFO	osvi;
	SEP				sep;
	int				index;
	TEXTMETRIC		tm;
	HDC				hdc;

#if PGP_SDK_AVAILABLE
	sep=SEsecAlloc(context,sizeof(SE));
#else
	sep=malloc(sizeof(SE));
#endif
	memset(sep,0x00,sizeof(SE));
	sep->hwndEdit=hwndEdit;
	sep->iMaxLen=0x7fffffff; // No maximum

	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx (&osvi);
	sep->bUnicodeOS = (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT);

	GetClientRect (hwndEdit, &sep->rcClient);

	hdc = GetDC (hwndEdit);
	GetTextMetrics (hdc, &tm);
	sep->iCaretHeight = tm.tmAscent;
	sep->iRowHeight = tm.tmHeight;
	if (!sep->bUnicodeOS)
		sep->iCharSet = sGetCharset ();

	sep->hdcMem = CreateCompatibleDC (hdc);
	sep->hbmMem = CreateCompatibleBitmap (hdc, 
			sep->rcClient.right-sep->rcClient.left, 
			sep->rcClient.bottom-sep->rcClient.top);
	SelectObject (sep->hdcMem, sep->hbmMem);
	ReleaseDC (hwndEdit, hdc);

	SetWindowLong (hwndEdit,GWL_USERDATA,(LONG)sep);

	if (sep->bUnicodeOS)
	{
		sep->wpOrigProc = (WNDPROC) 
			SetWindowLongW(hwndEdit,GWL_WNDPROC,(LONG)SESubclassProc);
	}
	else
	{
		SendMessage (hwndEdit, EM_SETEDITSTYLE, SES_USEAIMM, SES_USEAIMM);
		sep->wpOrigProc = (WNDPROC) 
			SetWindowLong(hwndEdit,GWL_WNDPROC,(LONG)SESubclassProc);
	}

	sep->bHideTyping=bHideTyping;
	sep->iLen=0;
	sep->UserCallback=UserCallback;
	sep->pUserValue=pUserValue;

	sCheckCaps (sep);

	srand( (unsigned)time( NULL ) );
	for (index=0; index<SE_BUFSIZE+1; index++)
		sep->iNumSpaces[index]=rand();

	// initialize the caret
	if (GetFocus () == hwndEdit)
	{
		sep->bFocused = TRUE;
		CreateCaret (hwndEdit, NULL, 0, sep->iCaretHeight);
		ShowCaret (hwndEdit);
	}
}

VOID 
SEDestroy(HWND hwndEdit)
{
	SEP sep;

	sep=(SEP)GetWindowLong (hwndEdit, GWL_USERDATA);

	if(sep!=0)
	{
		if (sep->bFocused)
			DestroyCaret ();

		DeleteObject (sep->hbmMem);
		DeleteDC (sep->hdcMem);

		if (sep->bUnicodeOS)
			SetWindowLongW(hwndEdit,GWL_WNDPROC,(LONG)sep->wpOrigProc);
		else
			SetWindowLong(hwndEdit,GWL_WNDPROC,(LONG)sep->wpOrigProc);

		SetWindowLong(hwndEdit,GWL_USERDATA,(LONG)0);

#if PGP_SDK_AVAILABLE
		SEsecFree(sep,sizeof(SE));
#else
		memset(sep,0x00,sizeof(SE));
		free(sep);
#endif
	}
}


//	__________________
//
//	Wipe edit box clean

VOID 
SEWipeEditBox (HWND hwndEdit)
{
	SEP sep;
	char szNull[1];
	int index;

	sep=(SEP)GetWindowLong (hwndEdit, GWL_USERDATA);

	if(sep!=0)
	{
		sep->dwQuality=0;
		sep->iLen=0;
		memset(sep->wszRealText,0x00,sizeof(sep->wszRealText));
		for(index=0;index<SE_BUFSIZE+1;index++)
		{
			sep->iNumSpaces[index]=rand();
		}

		szNull[0]=0;
		SendMessage(hwndEdit,WM_SETTEXT,0,(LPARAM)szNull);

		InvalidateRect(hwndEdit,NULL,TRUE);
		UpdateWindow(hwndEdit);
	}
}

VOID 
SESetMaxLength (HWND hwndEdit, int iMaxLen)
{
	SEP sep;

	sep=(SEP)GetWindowLong (hwndEdit, GWL_USERDATA);

	sep->iMaxLen=iMaxLen;
}

VOID 
SEGetText(HWND hwndEdit,char *copyto,int ilen)
{
	SEP sep;
	int index;
	PGPUInt32 ucopied;

	sep=(SEP)GetWindowLong (hwndEdit, GWL_USERDATA);

	for(index=0;index<sep->iLen;index++)
		sep->wszRealText[index]^=sep->iNumSpaces[index];

	pgpUCS2StringToUTF8 (sep->wszRealText, sep->iLen, copyto, ilen, &ucopied);

	for(index=0;index<sep->iLen;index++)
		sep->wszRealText[index]^=sep->iNumSpaces[index];

	copyto[ucopied]=0;
}

DWORD
SEGetTextLength(HWND hwndEdit)
{
	SEP sep;
	int index;
	UINT ulen;

	sep=(SEP)GetWindowLong (hwndEdit, GWL_USERDATA);

	for(index=0;index<sep->iLen;index++)
		sep->wszRealText[index]^=sep->iNumSpaces[index];

	pgpUCS2StringToUTF8 (sep->wszRealText, sep->iLen, NULL, 0, &ulen);

	for(index=0;index<sep->iLen;index++)
		sep->wszRealText[index]^=sep->iNumSpaces[index];

	return ulen;
}

BOOL
SEGetShowWarning(HWND hwndEdit)
{
	SEP sep;

	sep=(SEP)GetWindowLong (hwndEdit, GWL_USERDATA);

	return sep->bShowWarning;
}

VOID 
SEChangeHideTyping (HWND hwndEdit, BOOL bHideTyping)
{
	SEP sep;

	sep=(SEP)GetWindowLong (hwndEdit, GWL_USERDATA);

	if(bHideTyping==sep->bHideTyping)
		return;

	sep->bHideTyping=bHideTyping;

	InvalidateRect (hwndEdit, NULL, FALSE);
}

DWORD
SEGetSpecialKey(HWND hwndEdit)
{
	SEP	sep;
	DWORD dwSpecialKey;

	sep=(SEP)GetWindowLong (hwndEdit, GWL_USERDATA);

	dwSpecialKey = sep->dwSpecialKey;
	sep->dwSpecialKey = 0;

	return dwSpecialKey;
}

#if PGP_SDK_AVAILABLE
DWORD
SEGetTextQuality(HWND hwndEdit)
{
	SEP sep;

	sep=(SEP)GetWindowLong (hwndEdit, GWL_USERDATA);

	return sep->dwQuality;
}
#endif

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/

