/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	TLPaint.c - paint tree list

	$Id: TLPaint.c,v 1.21 2002/09/30 14:28:42 pbj Exp $
____________________________________________________________________________*/

#include "TLIntern.h"
#include <math.h>

typedef struct _DOTTEDSTRUCT {
	HDC hDC;
	DWORD dwDottedColor;
} DOTTEDSTRUCT;

#define TRUNCBUFSIZE	256


//	_______________________________________________
//
// Initialize painting routines -- create pens, brushes, etc.

void 
TLInitPaint (
		TLWndData*	pWD) 
{
	DWORD rgb;
	BYTE r, g, b;
	LOGFONT lf;
	HDC hdcMem, hdcScreen;
	HBITMAP hOldBitmap;
	TEXTMETRIC tm;
	RECT rc;
	HPEN pen;
	int i;

	hdcMem = CreateCompatibleDC (NULL);

	pWD->selbgbrush = CreateSolidBrush (GetSysColor (COLOR_HIGHLIGHT));
	pWD->unselbgbrush = CreateSolidBrush (GetSysColor (COLOR_WINDOW));
	pWD->selfgbrush = CreateSolidBrush (GetSysColor (COLOR_HIGHLIGHTTEXT));
	pWD->unselfgbrush = CreateSolidBrush (GetSysColor (COLOR_WINDOWTEXT));
	pWD->unfocusbgbrush = CreateSolidBrush (GetSysColor (COLOR_BTNFACE));

	pWD->unselfocuscolor = GetSysColor (COLOR_HIGHLIGHT);

	r = GetRValue (pWD->unselfocuscolor);
	if (r > 127)
		r = 0;
	else 
		r = 255;

	g = GetGValue (pWD->unselfocuscolor);
	if (g > 127) 
		g = 0;
	else 
		g = 255;

	b = GetBValue (pWD->unselfocuscolor);
	if (b > 127) 
		b = 0;
	else 
		b = 255;

	pWD->selfocuscolor = PALETTERGB (r, g, b);

	pWD->seltextpen = CreatePen (
			PS_SOLID, 0, GetSysColor (COLOR_HIGHLIGHTTEXT));
	pWD->unseltextpen = CreatePen (
			PS_SOLID, 0, GetSysColor (COLOR_WINDOWTEXT));

    pWD->buttonpen = CreatePen(PS_SOLID, 0, GetSysColor (COLOR_3DSHADOW));
	pWD->linecolor = GetSysColor (COLOR_3DSHADOW);

	pWD->barcolor = GetSysColor (COLOR_3DSHADOW);
	pWD->stdbarbrush = CreateSolidBrush (GetSysColor (COLOR_3DSHADOW));
	pWD->spcbarbrush = CreateHatchBrush (
			HS_BDIAGONAL, GetSysColor (COLOR_HIGHLIGHTTEXT));

	pWD->hilightpen = CreatePen (
			PS_SOLID, 0, GetSysColor (COLOR_3DHILIGHT));
	pWD->shadowpen = CreatePen (
			PS_SOLID, 0, GetSysColor (COLOR_3DDKSHADOW));

	// GetDeviceCaps returns positive value for displays with 2-256 colors
	if (GetDeviceCaps (hdcMem, NUMCOLORS) > 0) 
	{
		hdcScreen = GetDC (NULL);
		pWD->barbgbitmap = CreateCompatibleBitmap (hdcScreen, 8, 8);
		hOldBitmap = SelectObject (hdcMem, pWD->barbgbitmap);
		rc.left = 0;
		rc.top = 0;
		rc.bottom = 8;
		rc.right = 8;
		FillRect (hdcMem, &rc, pWD->unselbgbrush);
		pen = CreatePen (PS_SOLID, 0, GetSysColor (COLOR_3DLIGHT));
		SelectObject (hdcMem, pen);
		for (i=-8; i<8; i+=2) 
		{
			MoveToEx (hdcMem, i, 0, NULL);
			LineTo (hdcMem, i+8, 8);
		}

		DeleteObject (pen);
		pWD->barbgbrush = CreatePatternBrush (pWD->barbgbitmap);
		SelectObject (hdcMem, hOldBitmap);
		ReleaseDC (NULL, hdcScreen);
	}
	// GetDeviceCaps returns -1 for displays with more than 8 bit depth
	else 
	{
		rgb = GetSysColor (COLOR_BTNFACE);
		r = GetRValue (rgb);
		r = 255 - ((255 - r)/2);
		g = GetGValue (rgb);
		g = 255 - ((255 - g)/2);
		b = GetBValue (rgb);
		b = 255 - ((255 - b)/2);
		pWD->barbgbitmap = NULL;
		pWD->barbgbrush = CreateSolidBrush (RGB (r,g,b));
	}

	if (pWD->style & TLS_USEDEFAULTGUIFONT)
	{
		GetObject (GetStockObject (DEFAULT_GUI_FONT), 
				sizeof(LOGFONT), &lf);
	}
	else
	{
		SystemParametersInfo (
				SPI_GETICONTITLELOGFONT, sizeof(LOGFONT), &lf, 0);
	}

	pWD->hFont = CreateFontIndirect (&lf);
	lf.lfItalic = !lf.lfItalic;
	pWD->hFontItalic = CreateFontIndirect (&lf);
	lf.lfItalic = !lf.lfItalic;
	lf.lfWeight = FW_BOLD;
	pWD->hFontBold = CreateFontIndirect (&lf);

	SelectObject (hdcMem, pWD->hFont);
	GetTextMetrics (hdcMem, &tm);
	pWD->iRowHeight = tm.tmHeight + (2 * (TEXTVERTOFF+1)) -1;

	if (pWD->iRowHeight <= BITMAPHEIGHT)
		pWD->iRowHeight = BITMAPHEIGHT +1;

	DeleteDC (hdcMem);
}


//	_______________________________________________
//
// Close painting routines -- delete pens, brushes, etc.

void 
TLClosePaint (
		TLWndData*	pWD) 
{
	DeleteObject (pWD->selbgbrush);
	DeleteObject (pWD->unselbgbrush);
	DeleteObject (pWD->selfgbrush);
	DeleteObject (pWD->unselfgbrush);
	DeleteObject (pWD->unfocusbgbrush);
	DeleteObject (pWD->stdbarbrush);
	DeleteObject (pWD->spcbarbrush);
	DeleteObject (pWD->barbgbrush);
	DeleteObject (pWD->seltextpen);
	DeleteObject (pWD->unseltextpen);
	DeleteObject (pWD->buttonpen);
	DeleteObject (pWD->hilightpen);
	DeleteObject (pWD->shadowpen);
	DeleteObject (pWD->hFont);
	DeleteObject (pWD->hFontItalic);
	DeleteObject (pWD->hFontBold);

	if (pWD->barbgbitmap) 
		DeleteObject (pWD->barbgbitmap);
}


//	_______________________________________________
//
//  get wide char representation of text

static INT 
sGetWideText (
		CHAR*		psz,
		WCHAR*		wsz,
		UINT		ulenWide)
{
	const unsigned char*	psrc	= psz;
	WCHAR*					pwsz	= wsz;
	int						iIn		= 0;
	int						iOut	= 0;
	WCHAR					u;

	if (psrc)
	{
		while ((*psrc) && 
			(iOut < (int)ulenWide))
		{
			u = 0;

			if ((*psrc) < 0x80)
			{
				u = *psrc++;
				iIn++;
			}
			else if ((*psrc) < 0xC0)
			{
				// invalid char
			}
			else if ((*psrc) < 0xE0)
			{
				u = (*psrc++) & 0x1F;
				iIn++;
				if ((*psrc & 0xC0) == 0x80)
				{
					u <<= 6;
					u += (*psrc++) & 0x3F;
					iIn++;
					if (u < 0x80)
						u = 0;
				}
				else
					u = 0;

			}
			else if ((*psrc) < 0xF0)
			{
				u = (*psrc++) & 0x0F;
				iIn++;
				u <<= 6;

				if ((*psrc & 0xC0) == 0x80)
				{
					u += (*psrc++) & 0x3F;
					iIn++;
					u <<= 6;

					if ((*psrc & 0xC0) == 0x80)
					{
						u += (*psrc++) & 0x3F;
						iIn++;
						if (u < 0x800)
							u = 0;
					}
					else
						u = 0;
				}
				else 
					u = 0;
			}
			else
			{
				// too many bits to hold in a UCS-2 char
			}

			if (u)
			{
				*pwsz++ = u;
				iOut++;
			}
			else
			{
				// invalid character or NULL
				iIn = 0;
				break;
			}
		}
	}

	if (iOut < (int)ulenWide)
	{
		*pwsz = 0;
	}

	return iOut;
}


//	_______________________________________________
//
//  Truncate text string

static BOOL sTruncateText (
		HDC		hdc, 
		WCHAR*	wsz, 
		int		xmax) 
{
	BOOL bShowToolTip	= FALSE;
	SIZE s;
	int l, w;

	// truncate at <newline>, if present
	l = 0;
	while (wsz[l])
	{
		if (wsz[l] == '\n')
		{
			wsz[l] = 0;
			bShowToolTip = TRUE;
			break;
		}
		l++;
	}

	GetTextExtentPoint32W (hdc, wsz, lstrlenW (wsz), &s);
	w = s.cx + 4;

	l = lstrlenW (wsz);
	if (l < 3) 
	{
		if (w > xmax) 
		{
			bShowToolTip = TRUE;
			wsz[0] = 0;
		}
	}
	else 
	{
		l = lstrlenW (wsz) - 3;

		while ((w > xmax) && (l >= 0))
		{
			bShowToolTip = TRUE;
			lstrcpyW (&wsz[l], L"...");
			GetTextExtentPoint32W (hdc, wsz, lstrlenW (wsz), &s);
			w = s.cx + 4;
			l--;
		}

		if (l < 0) 
			wsz[0] = 0;
	}

	return bShowToolTip;
}


//	_______________________________________________
//
//  Calculate how many lines are needed

static int 
sCountRows (
		TLTreeItem*		p) 
{
	int i = 0;

	while (p) 
	{
		i++;

		if (p->childItem && (p->state & TLIS_EXPANDED))
			i += sCountRows (p->childItem);

		p = p->nextItem;
	}
	return i;
}


//	_______________________________________________
//
//  Get row number of item

BOOL 
TLGetRow (
		TLTreeItem*	root, 
		TLTreeItem*	p, 
		int*		row) 
{
	while (root)
	{
		if (p == root) 
			return TRUE;
		(*row)++;

		if (root->childItem && (root->state & TLIS_EXPANDED)) 
		{
			if (TLGetRow (root->childItem, p, row)) 
				return TRUE;
		}
		root = root->nextItem;
	}

	return FALSE;
}


//	_______________________________________________
//
//  Item has just been expanded -- scroll window appropriately

void 
TLExpandAutoScroll (
		TLWndData*		pWD, 
		TLTreeItem*		p) 
{
	int itemRow = 0;

	if (TLGetRow (pWD->rootItem, p, &itemRow)) 
	{
		if (((itemRow-pWD->iFirstRow) + p->cChildren) >= pWD->iMaxRows)
		{
			if (p->cChildren >= pWD->iMaxRows)
				pWD->iFirstRow = itemRow;
			else 
				pWD->iFirstRow = itemRow + p->cChildren - pWD->iMaxRows +1;

			if (pWD->iFirstRow < 0) 
				pWD->iFirstRow = 0;
		}
	}
	return;
}


//	_______________________________________________
//
//  Item has just been selected -- scroll window appropriately

BOOL 
TLSelectAutoScroll (
		TLWndData*		pWD, 
		TLTreeItem*		p) 
{
	int itemRow = 0;

	if (TLGetRow (pWD->rootItem, p, &itemRow)) 
	{
		if ((itemRow-pWD->iFirstRow) >= pWD->iMaxRows) 
		{
			pWD->iFirstRow = itemRow - pWD->iMaxRows +1;
			if (pWD->iFirstRow < 0) 
				pWD->iFirstRow = 0;
			return TRUE;
		}

		if (itemRow < pWD->iFirstRow) 
		{
			pWD->iFirstRow = max (0, itemRow);
			return TRUE;
		}
	}

	return FALSE;
}


//	_______________________________________________
//
//  Invalidate the specified item

void 
TLInvalidateItem (
		TLWndData*		pWD,
		TLTreeItem*		p) 
{
	RECT rc;
	rc = p->hiliteRect;
	rc.left = pWD->tlInval.left;
	InvalidateRect (pWD->hWnd, &rc, TRUE);
}


//	_______________________________________________
//
//  draw a dotted line by setting alternate pixels
//  (callback to LineDDA)

static void CALLBACK 
sDottedLineProc (
		INT		iX, 
		INT		iY, 
		LPARAM	lpData)
{
	DOTTEDSTRUCT *pDottedStruct;

	pDottedStruct = (DOTTEDSTRUCT*)lpData;
	if (!((iX+iY) & 0x0001))
		SetPixel (pDottedStruct->hDC, iX, iY, pDottedStruct->dwDottedColor);
}


//	_______________________________________________
//
//  Paint one level of list

static void 
sPaintItems (
		HDC			hdc,
		TLWndData*	pWD,
		int*		item,
		int*		y, 
		int			indent,
		TLTreeItem* p)
{
	HBRUSH			oldbrush, bgbrush, fgbrush, barbrush;
	INT				iBCent, iBMPos;
	HPEN			oldpen, textpen;
	SIZE			s;
	INT				ix, iy, cx, xoffset;
	TLColumnItem*	pci;
	TLListItem*		pli;
	RECT			rc;
	DOTTEDSTRUCT	DottedStruct;
	double			f;
	WCHAR			wsz[TRUNCBUFSIZE];
	BOOL			bEnabled;

	bEnabled = IsWindowEnabled (pWD->hWnd);

	iBCent = pWD->iBCent;
	iBMPos = pWD->iBMPos;
	DottedStruct.hDC = hdc;

	xoffset = indent - pWD->iHorizontalPos;

	while (p) 
	{
		//  increment item count
		(*item)++;
		//  check if items are visible in window
		if ((*item) > (pWD->iFirstRow + pWD->iMaxRows + 2)) 
			break;

		if ((*item) > pWD->iFirstRow) 
		{
			//  first paint the tree items  ...
			//  select colors based on state of tree item
			if (!bEnabled)
			{
				bgbrush = pWD->unfocusbgbrush;
				fgbrush = pWD->unselfgbrush;
				textpen = pWD->unseltextpen;
				DottedStruct.dwDottedColor = pWD->unselfocuscolor;
				pWD->textbgcolor = GetSysColor (COLOR_BTNFACE);
				SetTextColor (hdc, GetSysColor (COLOR_WINDOWTEXT));
			}
			else if (pWD->bTreeFocused) 
			{
				if (p->state & TLIS_SELECTED)
				{
					bgbrush = pWD->selbgbrush;
					fgbrush = pWD->selfgbrush;
					textpen = pWD->seltextpen;
					DottedStruct.dwDottedColor = pWD->selfocuscolor;
					pWD->textbgcolor = GetSysColor (COLOR_HIGHLIGHT);
					SetTextColor (hdc, GetSysColor (COLOR_HIGHLIGHTTEXT));
				}
				else 
				{
					bgbrush = pWD->unselbgbrush;
					fgbrush = pWD->unselfgbrush;
					textpen = pWD->unseltextpen;
					DottedStruct.dwDottedColor = pWD->unselfocuscolor;
					pWD->textbgcolor = GetSysColor (COLOR_WINDOW);
					SetTextColor (hdc, GetSysColor (COLOR_WINDOWTEXT));
				}
			}
			else 
			{
				if ((p->state & (TLIS_SELECTED|TLIS_DRAGGEDOVER)) &&
						(pWD->style & TLS_SHOWSELECTIONALWAYS)) 
				{
					bgbrush = pWD->unfocusbgbrush;
					fgbrush = pWD->unselfgbrush;
					textpen = pWD->unseltextpen;
					DottedStruct.dwDottedColor = pWD->unselfocuscolor;
					pWD->textbgcolor = GetSysColor (COLOR_BTNFACE);
					SetTextColor (hdc, GetSysColor (COLOR_WINDOWTEXT));
				}
				else 
				{
					bgbrush = pWD->unselbgbrush;
					fgbrush = pWD->unselfgbrush;
					textpen = pWD->unseltextpen;
					DottedStruct.dwDottedColor = pWD->unselfocuscolor;
					pWD->textbgcolor = GetSysColor (COLOR_WINDOW);
					SetTextColor (hdc, GetSysColor (COLOR_WINDOWTEXT));
				}
			}
			SetBkColor (hdc, pWD->textbgcolor);

			//  set rectangle coordinates
			//  rectangle sensitive to "selection" mouse clicks
			p->selectRect.left = xoffset + BUTTONWIDTH;
			p->selectRect.right = pWD->tlInval.right;
			p->selectRect.top = *y+1;
			p->selectRect.bottom = p->selectRect.top + pWD->iRowHeight;

			//  rectangle sensitive to "button" mouse clicks
			if (p->cChildren && (pWD->style & TLS_HASBUTTONS)) 
			{
				p->buttonRect.left = xoffset;
				p->buttonRect.right = p->buttonRect.left + BUTTONWIDTH;
				p->buttonRect.top = *y;
				p->buttonRect.bottom = p->buttonRect.top + pWD->iRowHeight;
			}
			else
			{
				p->buttonRect.left = xoffset + BUTTONWIDTH;
				p->buttonRect.right = xoffset + BUTTONWIDTH;
				p->buttonRect.top = *y;
				p->buttonRect.bottom = 0;
			}

			//  rectangle which is highlighted when item is selected
			p->hiliteRect.left =
					min (xoffset + BUTTONWIDTH+BITMAPWIDTH+HTEXTSHIFT,
					pWD->iFirstColumnWidth);
			p->hiliteRect.top = *y+1;
			p->hiliteRect.bottom = p->selectRect.top + pWD->iRowHeight;

			if (p->state & TLIS_BOLD) 
				SelectObject (hdc, pWD->hFontBold);
			else if (p->state & TLIS_ITALICS)
				SelectObject (hdc, pWD->hFontItalic);
			else 
				SelectObject (hdc, pWD->hFont);

			sGetWideText (p->pszText, wsz, sizeof(wsz));
			if (sTruncateText (hdc, wsz,
					pWD->iFirstColumnWidth-
					(indent + BUTTONWIDTH+BITMAPWIDTH+HTEXTSHIFT)))
			{
				p->state |= TLIS_TRUNCATED;
			}
			else
				p->state &= ~TLIS_TRUNCATED;

			GetTextExtentPoint32W (hdc, wsz, lstrlenW (wsz), &s);
			p->hiliteRect.right = p->hiliteRect.left + s.cx + HTEXTSHIFT;
			pli = p->listItem;
			pci = pWD->columnItem;
			ix = pWD->iFirstColumnWidth;
			while (pci && pli) 
			{
				ix += pci->cx;
				if (pli->state & TLIS_VISIBLE) p->hiliteRect.right = ix;
				pci = pci->nextItem;
				pli = pli->nextItem;
			}

			//  if focused, draw box
			if (bgbrush != pWD->unselbgbrush)
				FillRect (hdc, &p->hiliteRect, bgbrush);

			if (pWD->bTreeFocused)
			{
				if ((p->state & TLIS_FOCUSED) ||
					(p->state & TLIS_DRAGGEDOVER))
				{
					LineDDA (p->hiliteRect.left, p->hiliteRect.top,
							 p->hiliteRect.right-1, p->hiliteRect.top,
							 (LINEDDAPROC)sDottedLineProc, 
							 (LPARAM)&DottedStruct);
					LineDDA (p->hiliteRect.right-1, p->hiliteRect.top,
							 p->hiliteRect.right-1, p->hiliteRect.bottom-1,
							 (LINEDDAPROC)sDottedLineProc, 
							 (LPARAM)&DottedStruct);
					LineDDA (p->hiliteRect.right-1, p->hiliteRect.bottom-1,
							 p->hiliteRect.left, p->hiliteRect.bottom-1,
							 (LINEDDAPROC)sDottedLineProc, 
							 (LPARAM)&DottedStruct);
					LineDDA (p->hiliteRect.left, p->hiliteRect.bottom-1,
							 p->hiliteRect.left, p->hiliteRect.top,
							 (LINEDDAPROC)sDottedLineProc, 
							 (LPARAM)&DottedStruct);
				}
				else if (p->state & TLIS_DRAGGEDBEFORE)
				{
					oldpen = SelectObject (hdc, pWD->unseltextpen);
					MoveToEx (hdc, p->selectRect.left, 
							p->selectRect.top, NULL);
					LineTo (hdc, p->selectRect.right-BUTTONWIDTH, 
							p->selectRect.top);
					SelectObject (hdc, oldpen);
				}
				else if (p->state & TLIS_DRAGGEDAFTER)
				{
					oldpen = SelectObject (hdc, pWD->unseltextpen);
					MoveToEx (hdc, p->selectRect.left, 
							p->selectRect.bottom-1, NULL);
					LineTo (hdc, p->selectRect.right-BUTTONWIDTH, 
							p->selectRect.bottom-1);
					SelectObject (hdc, oldpen);
				}
			}

			//  draw tree item text
			iy = p->hiliteRect.top + TEXTVERTOFF;
			SetTextAlign (hdc, TA_LEFT);
			TextOutW (hdc, p->hiliteRect.left+2, iy, wsz, lstrlenW(wsz));

			//  draw tree item bitmap, if there is space
			if (pWD->iFirstColumnWidth >= (p->buttonRect.right+BITMAPWIDTH)) 
			{
				ImageList_Draw (pWD->hImageList, p->iImage, hdc,
						p->buttonRect.right, p->hiliteRect.top+iBMPos,
						ILD_TRANSPARENT);
			}

			//  paint list items based on their data type
			SelectObject (hdc, pWD->hFont);
			pli = p->listItem;
			pci = pWD->columnItem;
			ix = pWD->iFirstColumnWidth - pWD->iHorizontalPos;

			while (pci && pli)
			{
				if (pli->state & TLIS_VISIBLE) 
				{
					switch (pci->iDataType) {
					case TLC_DATANONE :
						break;

					case TLC_DATASTRING :
						sGetWideText (pli->pszText, wsz, sizeof(wsz));
						if (sTruncateText (hdc, wsz, pci->cx))
							pli->state |= TLIS_TRUNCATED;
						else
							pli->state &= ~TLIS_TRUNCATED;

						SetBkColor (hdc, pWD->textbgcolor);

						switch (pci->iDataFmt) {
						case TLCFMT_LEFT :
							SetTextAlign (hdc, TA_LEFT);
							TextOutW (hdc, ix+6, iy,
									wsz, lstrlenW(wsz));
							break;

						case TLCFMT_CENTER :
							SetTextAlign (hdc, TA_CENTER);
							TextOutW (hdc, ix + (pci->cx>>1), iy,
									wsz, lstrlenW(wsz));
							break;

						case TLCFMT_RIGHT :
							SetTextAlign (hdc, TA_RIGHT);
							TextOutW (hdc, ix + pci->cx - 6, iy,
									wsz, lstrlenW(wsz));
							break;
						}
						break;

					case TLC_DATALONG :
						switch (pci->iDataFmt) {
						case TLCFMT_LINBAR :
							cx = pci->cx - BARCOLUMNDIFF;
							if (cx <= 0) 
								break;

							rc.left = ix + HBARSHIFT;
							rc.top = iy + VBARSHIFT +1;
							rc.right = ix + pci->cx - HBARSHIFT -1;
							rc.bottom = iy+pWD->iRowHeight-(VBARSHIFT+2) -1;
							oldbrush = SelectObject (hdc, pWD->barbgbrush);
							oldpen = SelectObject (hdc, pWD->buttonpen);
							Rectangle (hdc, rc.left - 1, rc.top - 1,
									rc.right + 2, rc.bottom + 2);

							SelectObject (hdc, pWD->shadowpen);
							MoveToEx (hdc, rc.left, rc.bottom, NULL);
							LineTo (hdc, rc.left, rc.top);
							LineTo (hdc, rc.right, rc.top);

							SelectObject (hdc, pWD->hilightpen);
							LineTo (hdc, rc.right, rc.bottom);
							LineTo (hdc, rc.left, rc.bottom);

							if (pci->lMaxValue != 0) 
							{
								if (pli->lDataValue > pci->lMaxValue) 
								{
									barbrush = pWD->spcbarbrush;
									rc.right = rc.left + cx;
								}
								else 
								{
									barbrush = pWD->stdbarbrush;
									rc.right = rc.left +
										(int)(((float)pli->lDataValue /
											(float)pci->lMaxValue)
											* (float)cx);
								}
							}
							else
								rc.right = rc.left;

							rc.top++;
							rc.left++;

							if (rc.right > rc.left) 
							{
								SetBkColor (hdc, pWD->barcolor);
								FillRect (hdc, &rc, barbrush);

								rc.top--;
								rc.left--;

								// hilight pen already selected
								MoveToEx (hdc, rc.right, rc.top, NULL);
								LineTo (hdc, rc.left, rc.top);
								LineTo (hdc, rc.left,rc.bottom);

								SelectObject (hdc, pWD->shadowpen);
								LineTo (hdc, rc.right, rc.bottom);
								LineTo (hdc, rc.right, rc.top);
							}

							SelectObject (hdc, oldbrush);
							SelectObject (hdc, oldpen);
							break;

						case TLCFMT_LOGBAR :
							rc.left = ix + HBARSHIFT;
							rc.top = iy + VBARSHIFT +1;
							rc.right = ix + pci->cx - HBARSHIFT -1;
							rc.bottom = iy+pWD->iRowHeight-(VBARSHIFT+2) -1;
							oldbrush = SelectObject (hdc, pWD->barbgbrush);
							oldpen = SelectObject (hdc, pWD->buttonpen);
							Rectangle (hdc, rc.left - 1, rc.top - 1,
									rc.right + 2, rc.bottom + 2);

							SelectObject (hdc, pWD->shadowpen);
							MoveToEx (hdc, rc.left, rc.bottom, NULL);
							LineTo (hdc, rc.left, rc.top);
							LineTo (hdc, rc.right, rc.top);

							SelectObject (hdc, pWD->hilightpen);
							LineTo (hdc, rc.right, rc.bottom);
							LineTo (hdc, rc.left, rc.bottom);

							if (pci->lMaxValue != 0) 
							{
								if (pli->lDataValue > pci->lMaxValue)
								{
									barbrush = pWD->spcbarbrush;
									rc.right = rc.left + cx;
								}
								else 
								{
									barbrush = pWD->stdbarbrush;
									f = (float)pli->lDataValue /
										(float)pci->lMaxValue;
									f += 1.0;
									f = log (f) / log (2.0);
									rc.right = rc.left + (int)(f * (float)cx);
								}
							}
							else
								rc.right = rc.left;

							rc.top++;
							rc.left++;

							if (rc.right > rc.left) 
							{
								SetBkColor (hdc, pWD->barcolor);
								FillRect (hdc, &rc, barbrush);

								rc.top--;
								rc.left--;

								// hilight pen already selected
								MoveToEx (hdc, rc.right, rc.top, NULL);
								LineTo (hdc, rc.left, rc.top);
								LineTo (hdc, rc.left,rc.bottom);

								SelectObject (hdc, pWD->shadowpen);
								LineTo (hdc, rc.right, rc.bottom);
								LineTo (hdc, rc.right, rc.top);
							}

							SelectObject (hdc, oldbrush);
							SelectObject (hdc, oldpen);
							break;

						case TLCFMT_IMAGE :
							// use text if available
							if (pli->pszText) 
							{
								sGetWideText (pli->pszText, wsz, sizeof(wsz));
								if (sTruncateText (hdc, wsz, pci->cx))
									pli->state |= TLIS_TRUNCATED;
								else
									pli->state &= ~TLIS_TRUNCATED;

								SetBkColor (hdc, pWD->textbgcolor);
								SetTextAlign (hdc, TA_CENTER);
								TextOutW (hdc, ix + (pci->cx>>1), iy,
													wsz, lstrlenW(wsz));
							}
							// otherwise paint image
							else 
							{
								if (pci->cx >= (BITMAPWIDTH)) 
								{
									int	i = 0;
									WORD wCur = LOWORD (pli->lDataValue);

									if ((pWD->bStepAnimation) &&
										(pli->state & TLIS_ANIMATED)) 
									{
										int iLim;

										pWD->bContAnimation = TRUE;

										iLim = HIBYTE(HIWORD(pli->lDataValue));
										wCur = LOWORD(++(pli->lDataValue));

										if (wCur > iLim) {
											wCur =
												LOBYTE(HIWORD(pli->lDataValue));
											pli->lDataValue =
												MAKELONG (wCur,
													HIWORD(pli->lDataValue));
										}
									}
									else 
									{
										if (pli->state & TLIS_CLICKED) 
											i = 1;
									}

									ImageList_Draw (pWD->hImageList,
											(wCur + i), hdc,
											ix + pci->cxImageOffset,
											p->hiliteRect.top+iBMPos,
											ILD_TRANSPARENT);
								}
							}
							break;

						case TLCFMT_IMAGELIST :
						{
							int iImage = 0;
							int iXpos = 0;
							ULONG uImageMask = 1;

							// first paint images
							while (!(uImageMask & 0x80000000)) 
							{
								if (pli->lDataValue & uImageMask)
								{
									if ((iXpos+BITMAPWIDTH) <= pci->cx)
									{
										ImageList_Draw (pWD->hImageList,
												iImage, hdc,
												ix + iXpos,
												p->hiliteRect.top+iBMPos,
												ILD_TRANSPARENT);
										iXpos +=
												(BITMAPWIDTH + BITMAPSPACE);
									}
									else 
									{
										lstrcpyW (wsz, L"...");
										sTruncateText (hdc, wsz, pci->cx);
										SetBkColor (hdc,
												pWD->textbgcolor);
										SetTextAlign (hdc, TA_LEFT);
										TextOutW (hdc, ix + iXpos,
												iy, wsz, lstrlenW(wsz));
									}
								}

								uImageMask <<= 1;
								iImage++;
							}

							// then draw text if available
							if (pli->pszText) 
							{
								sGetWideText (pli->pszText, wsz, sizeof(wsz));
								if (sTruncateText (hdc, 
										wsz, (pci->cx - iXpos)))
								{
									pli->state |= TLIS_TRUNCATED;
								}
								else
									pli->state &= ~TLIS_TRUNCATED;

								SetBkColor (hdc, pWD->textbgcolor);
								SetTextAlign (hdc, TA_LEFT);
								TextOutW (hdc, ix + iXpos, iy,
										wsz, lstrlenW(wsz));
							}
							break;
						}

						case TLCFMT_HIMAGELIST :
						{
							int iXpos = 0;

							// first paint images
							if (pli->lDataValue)
							{
								if (BITMAPWIDTH <= pci->cx)
								{
									ImageList_Draw (
											(HIMAGELIST)(pli->lDataValue),
											0, hdc, ix,
											p->hiliteRect.top+iBMPos,
											ILD_TRANSPARENT);
									iXpos +=
											(BITMAPWIDTH + BITMAPSPACE);
								}
								else 
								{
									lstrcpyW (wsz, L"...");
									sTruncateText (hdc, wsz, pci->cx);
									SetBkColor (hdc,
											pWD->textbgcolor);
									SetTextAlign (hdc, TA_LEFT);
									TextOutW (hdc, ix + iXpos,
											iy, wsz, lstrlenW(wsz));
								}
							}

							// then draw text if available
							if (pli->pszText) 
							{
								sGetWideText (pli->pszText, wsz, sizeof(wsz));
								if (sTruncateText (hdc, 
										wsz, (pci->cx - iXpos)))
								{
									pli->state |= TLIS_TRUNCATED;
								}
								else
									pli->state &= ~TLIS_TRUNCATED;

								SetBkColor (hdc, pWD->textbgcolor);
								SetTextAlign (hdc, TA_LEFT);
								TextOutW (hdc, ix + iXpos, iy,
										wsz, lstrlenW(wsz));
							}
							break;
						}

						case TLCFMT_HICON :
						{
							int iXpos = 0;

							// first paint images
							if (pli->lDataValue)
							{
								if (BITMAPWIDTH <= pci->cx)
								{
									DrawIconEx (hdc, 
											ix, p->hiliteRect.top+iBMPos,
											(HICON)(pli->lDataValue),
											BITMAPWIDTH, BITMAPHEIGHT,
											0, NULL, DI_NORMAL);
									iXpos +=
											(BITMAPWIDTH + BITMAPSPACE);
								}
								else 
								{
									lstrcpyW (wsz, L"...");
									sTruncateText (hdc, wsz, pci->cx);
									SetBkColor (hdc,
											pWD->textbgcolor);
									SetTextAlign (hdc, TA_LEFT);
									TextOutW (hdc, ix + iXpos,
											iy, wsz, lstrlenW(wsz));
								}
							}

							// then draw text if available
							if (pli->pszText) 
							{
								sGetWideText (pli->pszText, wsz, sizeof(wsz));
								if (sTruncateText (hdc, 
										wsz, (pci->cx - iXpos)))
								{
									pli->state |= TLIS_TRUNCATED;
								}
								else
									pli->state &= ~TLIS_TRUNCATED;

								SetBkColor (hdc, pWD->textbgcolor);
								SetTextAlign (hdc, TA_LEFT);
								TextOutW (hdc, ix + iXpos, iy,
										wsz, lstrlenW(wsz));
							}
							break;
						}

						default :
							wsprintf ((LPSTR)wsz, "%i", pli->lDataValue);
							SetBkColor (hdc, pWD->textbgcolor);
							SetTextAlign (hdc, TA_LEFT);
							TextOut (hdc, ix+6, iy, 
									(LPSTR)wsz, lstrlen((LPSTR)wsz));
							break;
						}
						break;
					}
				}
				ix += pci->cx;

				pci = pci->nextItem;
				pli = pli->nextItem;
			}

			//  increment y coordinate to next row
			*y += pWD->iRowHeight;

			//  if there is space, paint button and lines
			if (pWD->iFirstColumnWidth >= p->buttonRect.right)
			{
				if (p->cChildren) 
				{
					oldpen = SelectObject (hdc, pWD->buttonpen);
					MoveToEx (hdc, p->buttonRect.left+HBCENT-BBOX,
							p->buttonRect.top+iBCent-BBOX, NULL);
					LineTo (hdc, p->buttonRect.left+HBCENT+BBOX,
							p->buttonRect.top+iBCent-BBOX);
					LineTo (hdc, p->buttonRect.left+HBCENT+BBOX,
							p->buttonRect.top+iBCent+BBOX);
					LineTo (hdc, p->buttonRect.left+HBCENT-BBOX,
							p->buttonRect.top+iBCent+BBOX);
					LineTo (hdc, p->buttonRect.left+HBCENT-BBOX,
							p->buttonRect.top+iBCent-BBOX);
					SelectObject (hdc, oldpen);

					if (p->state & TLIS_EXPANDED)
					{
						oldpen = SelectObject (hdc, pWD->unseltextpen);
						MoveToEx (hdc, p->buttonRect.left+HBCENT-BBAR,
								p->buttonRect.top+iBCent, NULL);
						LineTo (hdc, p->buttonRect.left+HBCENT+BBAR+1,
								p->buttonRect.top+iBCent);
						SelectObject (hdc, oldpen);
						// paint children
						sPaintItems (hdc, pWD, item, y,
								indent+INCREMENTINDENT, p->childItem);
					}
					else 
					{
						oldpen = SelectObject (hdc, pWD->unseltextpen);
						MoveToEx (hdc, p->buttonRect.left+HBCENT-BBAR,
								p->buttonRect.top+iBCent, NULL);
						LineTo (hdc, p->buttonRect.left+HBCENT+BBAR+1,
								p->buttonRect.top+iBCent);
						MoveToEx (hdc, p->buttonRect.left+HBCENT,
								p->buttonRect.top+iBCent-BBAR, NULL);
						LineTo (hdc, p->buttonRect.left+HBCENT,
								p->buttonRect.top+iBCent+BBAR+1);
						SelectObject (hdc, oldpen);
					}

					if ((pWD->style & TLS_HASLINES) && (p->parentItem ||
						(!p->parentItem && (pWD->style & TLS_LINESATROOT))))
					{
						DottedStruct.dwDottedColor = pWD->linecolor;

						if (p->prevItem || p->parentItem) 
						{
							if (p->prevItem)
							{
								LineDDA (xoffset+HBCENT,
										p->prevItem->selectRect.bottom,
										xoffset+HBCENT,
										p->buttonRect.top+iBCent-BBOX,
										(LINEDDAPROC)sDottedLineProc,
										(LPARAM)&DottedStruct);
							}
							else
							{
								LineDDA (xoffset+HBCENT, p->buttonRect.top,
										xoffset+HBCENT,
										p->buttonRect.top+iBCent-BBOX,
										(LINEDDAPROC)sDottedLineProc,
										(LPARAM)&DottedStruct);
							}
						}

						if (p->nextItem) 
						{
							LineDDA (xoffset+HBCENT,
									p->buttonRect.top+iBCent+BBOX,
									xoffset+HBCENT, *y +1,
									(LINEDDAPROC)sDottedLineProc,
									(LPARAM)&DottedStruct);
						}

						LineDDA (xoffset+HBCENT+BBOX,
								p->buttonRect.top+iBCent,
								xoffset+BUTTONWIDTH,
								p->buttonRect.top+iBCent,
								(LINEDDAPROC)sDottedLineProc,
								(LPARAM)&DottedStruct);
					}
				}

				//  if item has no children, only paint lines
				else
				{
					if ((pWD->style & TLS_HASLINES) && (p->parentItem ||
					    (!p->parentItem && (pWD->style & TLS_LINESATROOT)))) 
					{
						DottedStruct.dwDottedColor = pWD->linecolor;
						if (p->prevItem) 
						{
							ix = xoffset+HBCENT;
							iy = p->prevItem->selectRect.bottom;
						}
						else
						{
							ix = xoffset+HBCENT;
							iy = p->buttonRect.top;
						}

						if (p->nextItem) 
						{
							LineDDA (ix, iy, xoffset+HBCENT, *y +1,
									(LINEDDAPROC)sDottedLineProc,
									(LPARAM)&DottedStruct);
						}
						else 
						{
							LineDDA (ix, iy, xoffset+HBCENT,
									p->buttonRect.top+iBCent,
									(LINEDDAPROC)sDottedLineProc,
									(LPARAM)&DottedStruct);
						}

						LineDDA (xoffset+HBCENT, p->buttonRect.top+iBCent,
								xoffset+BUTTONWIDTH,
								p->buttonRect.top+iBCent,
								(LINEDDAPROC)sDottedLineProc,
								(LPARAM)&DottedStruct);
					}
				}
			}

			//  if no space, we still need to paint children
			else 
			{
				if (p->cChildren) 
				{
					if (p->state & TLIS_EXPANDED) 
					{
						sPaintItems (hdc, pWD, item, y,
							indent+INCREMENTINDENT, p->childItem);
					}
				}
			}

		}
		else 
		{		// ((*item) <= pWD->iFirstRow) {
			p->selectRect.left = 0;
			p->selectRect.right = 0;
			p->selectRect.top = 0;
			p->selectRect.bottom = 0;

			p->buttonRect.left = 0;
			p->buttonRect.right = 0;
			p->buttonRect.top = 0;
			p->buttonRect.bottom = 0;

			if (p->cChildren) 
			{
				if (p->state & TLIS_EXPANDED) 
				{
					sPaintItems (hdc, pWD, item, y,
						indent+INCREMENTINDENT, p->childItem);
				}
			}
		}
		//  step to next tree item
		p = p->nextItem;
	}

	//	set next item to unusual value to flag end of list
	if (p) 
	{
		p->selectRect.left = 0;
		p->selectRect.right = -1;
		p->selectRect.top = 0;
		p->selectRect.bottom = -1;

		p->buttonRect.left = 0;
		p->buttonRect.right = -1;
		p->buttonRect.top = 0;
		p->buttonRect.bottom = -1;
	}
}


//	_______________________________________________
//
//  Paint one level of list (selected items only)

static void 
sPaintItemsDragBitmap (
		HDC				hdc, 
		TLWndData*		pWD, 
		ULONG			ulFlags, 
		int*			item, 
		int*			y,
		int				iLevel,
		TLTreeItem*		p, 
		BOOL*			pbStarted)
{
	SIZE			s;
	INT				iy;
	WCHAR			wsz[TRUNCBUFSIZE];
	RECT			rcBound;

	SetTextColor (hdc, GetSysColor (COLOR_WINDOWTEXT));
	SetBkColor (hdc, GetSysColor (COLOR_WINDOW));

	while (p) 
	{
		//  increment item count
		(*item)++;

		//  check if items are visible in window
		if ((*item) > (pWD->iFirstRow + pWD->iMaxRows + 2)) 
			break;

		if ((*item) > pWD->iFirstRow)
		{
			//  first paint the tree items  ...
			//  select colors based on state of tree item
			if (pWD->bTreeFocused &&
				(p->state & TLIS_SELECTED) &&
				((iLevel == 0) || !(ulFlags & TLRB_TOPLEVELONLY)))
			{
				*pbStarted = TRUE;

				//  bounding rectangle for this item
				rcBound.left = iLevel * INCREMENTINDENT;
				rcBound.top = *y+1;
				rcBound.bottom = p->selectRect.top + pWD->iRowHeight;

				if (p->state & TLIS_BOLD) 
					SelectObject (hdc, pWD->hFontBold);
				else if (p->state & TLIS_ITALICS)
					SelectObject (hdc, pWD->hFontItalic);
				else 
					SelectObject (hdc, pWD->hFont);

				sGetWideText (p->pszText, wsz, sizeof(wsz));
				sTruncateText (hdc, wsz,
						pWD->iFirstColumnWidth-
						(BUTTONWIDTH+BITMAPWIDTH+HTEXTSHIFT));
				GetTextExtentPoint32W (hdc, wsz, lstrlenW (wsz), &s);
				rcBound.right = rcBound.left + s.cx + HTEXTSHIFT;

				//  draw tree item text
				iy = rcBound.top + TEXTVERTOFF;
				SetTextAlign (hdc, TA_LEFT);
				TextOutW (hdc, rcBound.left+HTEXTSHIFT+BUTTONWIDTH, iy,
						wsz, lstrlenW(wsz));

				//  draw tree item bitmap, if there is space
				if (pWD->iFirstColumnWidth >= (BUTTONWIDTH+BITMAPWIDTH))
				{
					ImageList_Draw (pWD->hImageList, p->iImage, hdc,
							rcBound.left, rcBound.top+pWD->iBMPos,
							ILD_TRANSPARENT);
				}

				//  paint list items based on their data type
				SelectObject (hdc, pWD->hFont);

			}

			//  increment y coordinate to next row
			if (*pbStarted)
				*y += pWD->iRowHeight;

			// if there are expanded children, we need to add rows
			if (p->cChildren) 
			{
				if (p->state & TLIS_EXPANDED) 
				{
					sPaintItemsDragBitmap (hdc, pWD, ulFlags, item, y,
						iLevel+1, p->childItem, pbStarted);
				}
			}
		}
		else 
		{		// ((*item) <= pWD->iFirstRow) {
			if (p->cChildren)
			{
				if (p->state & TLIS_EXPANDED) 
				{
					sPaintItemsDragBitmap (hdc, pWD, ulFlags, item, y,
						iLevel+1, p->childItem, pbStarted);
				}
			}
		}
		//  step to next tree item
		p = p->nextItem;
	}
}


//	_______________________________________________
//
//  Paint one level of list (selected items only)

static void 
sCalcItemsDragBitmap (
		TLWndData*		pWD, 
		ULONG			ulFlags, 
		int*			item, 
		int*			y,
		int				iLevel, 
		TLTreeItem*		p, 
		BOOL*			pbStarted, 
		RECT*			prc)
{
	while (p) 
	{
		//  increment item count
		(*item)++;

		//  check if items are visible in window
		if ((*item) > (pWD->iFirstRow + pWD->iMaxRows + 2)) 
			break;

		if ((*item) > pWD->iFirstRow) 
		{
			//  first paint the tree items  ...
			//  select colors based on state of tree item
			if (pWD->bTreeFocused &&
				(p->state & TLIS_SELECTED) &&
				((iLevel == 0) || !(ulFlags & TLRB_TOPLEVELONLY)))
			{
				if (*pbStarted)
					prc->bottom = *y + pWD->iRowHeight;
				else 
				{
					prc->top = *y;
					prc->bottom = *y + pWD->iRowHeight;
					*pbStarted = TRUE;
				}
			}

			//  increment y coordinate to next row
			*y += pWD->iRowHeight;

			// if there are expanded children, we need to add empty rows
			if (p->cChildren) 
			{
				if (p->state & TLIS_EXPANDED) 
				{
					sCalcItemsDragBitmap (pWD, ulFlags, item, y,
							iLevel+1, p->childItem, pbStarted, prc);
				}
			}
		}
		else 
		{		// ((*item) <= pWD->iFirstRow) {
			if (p->cChildren) 
			{
				if (p->state & TLIS_EXPANDED)
				{
					sCalcItemsDragBitmap (pWD, ulFlags, item, y,
							iLevel+1, p->childItem, pbStarted, prc);
				}
			}
		}
		//  step to next tree item
		p = p->nextItem;
	}
}


//	_______________________________________________
//
//  Compute length of string terminated either by null
//	character or newline

static int
sTextLength (WCHAR* pwsz)
{
	int	i	= 0;

	if (pwsz)
	{
		while (*pwsz && (*pwsz != '\n'))
		{
			pwsz++;
			i++;
		}
	}

	return i;
}


//	_______________________________________________
//
//  Compute minimum necessary width of column for auto-sizing

static int 
sComputeWidth (
		HDC			hdc,
		TLWndData*	pWD,
		int			icol,
		int			indent,
		TLTreeItem* p)
{
	int				iwidth		= 0;
	int				i, ilen;
	SIZE			size;
	WCHAR			wsz[TRUNCBUFSIZE];

	TLColumnItem*	pci;
	TLListItem*		pli;

	while (p) 
	{    
		if (icol == 0)
		{
			sGetWideText (p->pszText, wsz, sizeof(wsz));  
			ilen = sTextLength (wsz);
			GetTextExtentPoint32W (hdc, wsz, ilen, &size);
			i = size.cx + 12 + indent;
		}
		else
		{
			pli = p->listItem;
			pci = pWD->columnItem;

			for (i=1; i<icol; i++)
			{
				pci = pci->nextItem;
				pli = pli->nextItem;
			}
			                         
			sGetWideText (pli->pszText, wsz, sizeof(wsz));
			ilen = sTextLength (wsz);

			switch (pci->iDataType) {
			case TLC_DATASTRING :
				GetTextExtentPoint32W (hdc, wsz, ilen, &size);
				i = size.cx + 12;
				break;

			case TLC_DATALONG :
				switch (pci->iDataFmt) {
				case TLCFMT_IMAGE :
					if (pli->pszText) 
					{
						GetTextExtentPoint32W (hdc, wsz, ilen, &size);
						i = size.cx + 12;
					}
					else
					{
						i = BITMAPWIDTH + 12;
					}
					break;

				case TLCFMT_HIMAGELIST :
				case TLCFMT_HICON :
					i = 0;
					if (pli->lDataValue)
					{
						i += BITMAPWIDTH + 12;
					}
					if (pli->pszText)
					{
						GetTextExtentPoint32W (hdc, wsz, ilen, &size);
						i += size.cx + 12;
					}
					break;

				case TLCFMT_IMAGELIST :
				{
					ULONG uImageMask = 1;

					i = 12;
					while (!(uImageMask & 0x80000000)) 
					{
						if (pli->lDataValue & uImageMask)
							i += (BITMAPWIDTH + BITMAPSPACE);

						uImageMask <<= 1;
					}

					if (pli->pszText)
					{
						GetTextExtentPoint32W (hdc, wsz, ilen, &size);
						i += size.cx;
					}
					break;
				}

				default :
					i = pci->cx;
					break;
				}
				break;
			}
		}

		if (i > iwidth)
			iwidth = i;

		if (p->cChildren) 
		{
			if (p->state & TLIS_EXPANDED)
			{
				i = sComputeWidth (hdc, pWD, 
						icol, indent+INCREMENTINDENT, p->childItem);

				if (i > iwidth)
					iwidth = i;
			}
		}

		p = p->nextItem;
	}

	return iwidth;
}


//	_______________________________________________
//
//  compute the appropriate width of a column

int 
TLAutoComputeColumnWidth (
		TLWndData*	pWD, 
		int			icol) 
{
	HDC			hdc;
	TLTreeItem*	p;
	HFONT		hOldFont;

	int			iwidth			= -1;
	int			indent			= BUTTONWIDTH+BITMAPWIDTH+HTEXTSHIFT;

	hdc = GetDC (pWD->hWnd);
	if (hdc)
	{
		hOldFont = SelectObject (hdc, pWD->hFont);

		p = pWD->rootItem;

		if (pWD->style & TLS_DRAGSELECTION)
			indent += DRAGSELECTINDENT;

		iwidth = sComputeWidth (hdc, pWD, icol, indent, p);

		SelectObject (hdc, hOldFont);
		ReleaseDC (pWD->hWnd, hdc);
	}

	return iwidth;
}


//	_______________________________________________
//
//  Calculations before painting -- called in response to WM_PAINT

void 
TLPrePaint (
		TLWndData*	pWD) 
{
	SCROLLINFO si;
	INT iY;

	pWD->iBCent = (pWD->iRowHeight / 2) +2;
	pWD->iBMPos = pWD->iBCent - (BITMAPHEIGHT/2) -1;

	pWD->tlInval = pWD->tlRect;
	pWD->tlInval.top += pWD->iRowHeight+1;

	pWD->iExpandedRows = sCountRows (pWD->rootItem);
	pWD->iMaxRows = (pWD->tlInval.bottom-pWD->tlInval.top-
					(pWD->iRowHeight/2)) / pWD->iRowHeight;

	if (pWD->iMaxRows <= pWD->iExpandedRows) 
	{
		pWD->tlInval.right = 
				pWD->tlRect.right - GetSystemMetrics (SM_CXVSCROLL);
	}
	else 
	{
		pWD->iFirstRow = 0;
		pWD->tlInval.right = pWD->tlRect.right;
	}

	iY = pWD->tlInval.right - pWD->tlInval.left;

	if (pWD->iTotalWidth > iY) 
	{
		pWD->tlInval.bottom =
				pWD->tlRect.bottom - GetSystemMetrics (SM_CYHSCROLL);
	}
	else 
		pWD->tlInval.bottom = pWD->tlRect.bottom;

	pWD->iMaxRows = (pWD->tlInval.bottom-pWD->tlInval.top-
			(pWD->iRowHeight/2)) / pWD->iRowHeight;

	si.cbSize = sizeof (SCROLLINFO);
	si.fMask = SIF_PAGE | SIF_POS | SIF_RANGE;
	si.nMin = 0;
	si.nMax = pWD->iExpandedRows;
	si.nPage = pWD->iMaxRows;
	si.nPos = pWD->iFirstRow;
	SetScrollInfo (pWD->hWnd, SB_VERT, &si, TRUE);

	si.nMax = pWD->iTotalWidth;
	si.nPage = pWD->tlRect.right - pWD->tlRect.left;
	if (pWD->iMaxRows <= pWD->iExpandedRows)
		si.nPage -= GetSystemMetrics (SM_CXVSCROLL);
	si.nPos = pWD->iHorizontalPos;
	SetScrollInfo (pWD->hWnd, SB_HORZ, &si, TRUE);
}


//	_______________________________________________
//
//  Paint a tree list -- called in response to WM_PAINT

void 
TLPaint (
		TLWndData*	pWD, 
		HDC			hdc) 
{
	TLTreeItem*	p;
	HFONT		hOldFont;
	HBRUSH		hOldBrush;
	int			y, item, indent;

	hOldFont = SelectObject (hdc, pWD->hFont);

	UnrealizeObject (pWD->spcbarbrush);
	SetBrushOrgEx (hdc, -1 * pWD->iHorizontalPos, 0, NULL);
	hOldBrush = SelectObject (hdc, pWD->spcbarbrush);

	p = pWD->rootItem;
	y = pWD->tlInval.top;
	item = 0;
	indent = 0;

	if (pWD->style & TLS_DRAGSELECTION)
		indent = DRAGSELECTINDENT;

	if (pWD->bStepAnimation)
		pWD->bContAnimation = FALSE;

	sPaintItems (hdc, pWD, &item, &y, indent, p);

	SelectObject (hdc, hOldBrush);
	SelectObject (hdc, hOldFont);
}


//	_______________________________________________
//
//  delete the memory DC

void 
TLDeleteMemDC (
		TLWndData*	pWD) 
{
	if (pWD->hdcMem) 
	{
		SelectObject (pWD->hdcMem, pWD->hbmOld);
		DeleteObject (pWD->hbmMem);
		DeleteDC (pWD->hdcMem);
		pWD->hdcMem = NULL;
	}
}


//	_______________________________________________
//
//  create mem dc and pass it to routine to render treelist

void 
TLMemPaint (
		TLWndData*		pWD, 
		HDC				hdc, 
		PAINTSTRUCT*	pps) 
{
	RECT rc;

	GetClientRect (pWD->hWnd, &rc);

	if (!pWD->hdcMem) 
	{
		pWD->hdcMem = CreateCompatibleDC (hdc);
		pWD->hbmMem = CreateCompatibleBitmap (hdc, 
				rc.right-rc.left, rc.bottom-rc.top);
		pWD->hbmOld = SelectObject (pWD->hdcMem, pWD->hbmMem);
		SetBkMode (pWD->hdcMem, TRANSPARENT);
	}

	if (IsWindowEnabled (pWD->hWnd))
	{
		if (pWD->watermark.ulFlags & TLWM_CONSTANTBKGRND)
		{
			HBRUSH	hb	= CreateSolidBrush (pWD->watermark.crBkgrnd);
			FillRect (pWD->hdcMem, &rc, hb);
			DeleteObject (hb);
		}
		else
			FillRect (pWD->hdcMem, &rc, pWD->unselbgbrush);

		if (pWD->watermark.hbm)
		{
			HDC				hdcMem;
			HBITMAP			hbmpOld;
			RECT			rc2;
			INT				iSize, iCenter, iHalf;

			hdcMem = CreateCompatibleDC (hdc);
			hbmpOld = SelectObject (hdcMem, pWD->watermark.hbm);

			rc2 = pWD->tlRect;
			rc2.top += pWD->iRowHeight+1;
			rc2.bottom -= pWD->iRowHeight+1;
			if (pWD->watermark.ulFlags & TLWM_KEEPASPECTRATIO)
			{
				if (rc2.right > (rc2.bottom - rc2.top))
				{
					iCenter = rc2.right / 2;
					iHalf = (rc2.bottom - rc2.top) / 2;
					if (pWD->watermark.ulFlags & TLWM_SHRINKONLY)
					{
						iSize = pWD->watermark.size.cy / 2;
						if (iHalf > iSize)
						{
							rc2.bottom = rc2.top + (iHalf + iSize);
							rc2.top += iHalf - iSize;
							iHalf = iSize;
						}
					}
					rc2.left = iCenter - iHalf;
					rc2.right = iCenter + iHalf;
				}
				else
				{
					iCenter = (rc2.bottom - rc2.top) / 2;
					iHalf = rc2.right / 2;
					if (pWD->watermark.ulFlags & TLWM_SHRINKONLY)
					{
						iSize = pWD->watermark.size.cx / 2;
						if (iHalf > iSize)
						{
							rc2.left = iHalf - iSize;
							rc2.right = iHalf + iSize;
							iHalf = iSize;
						}
					}
					rc2.bottom = rc2.top + (iCenter + iHalf);
					rc2.top += iCenter - iHalf;
				}
			}

			StretchBlt (pWD->hdcMem, 
					rc2.left, rc2.top, 
					rc2.right-rc2.left, rc2.bottom-rc2.top,
					hdcMem, 0, 0, 
					pWD->watermark.size.cx, pWD->watermark.size.cy, 
					SRCCOPY);

			SelectObject (hdcMem, hbmpOld);
			DeleteDC (hdcMem);
		}
	}
	else
		FillRect (pWD->hdcMem, &rc, pWD->unfocusbgbrush);

	TLPaint (pWD, pWD->hdcMem);

	BitBlt (hdc, rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top,
			pWD->hdcMem, 0, 0, SRCCOPY);
}


//	_______________________________________________
//
//  Paint a tree list -- called in response to WM_PAINT

void 
TLPaintDragBitmap (
		TLWndData*	pWD, 
		ULONG		ulFlags, 
		HDC			hdc) 
{
	TLTreeItem* p;
	HFONT hOldFont;
	int y, item;
	BOOL bStarted;

	hOldFont = SelectObject (hdc, pWD->hFont);

	p = pWD->rootItem;
	y = 0;
	item = 0;
	bStarted = FALSE;
	sPaintItemsDragBitmap (hdc, pWD, ulFlags, &item, &y, 0, p, &bStarted);

	SelectObject (hdc, hOldFont);
}


//	_______________________________________________
//
//  Paint a tree list -- called in response to WM_PAINT

void 
TLCalcDragBitmap (
		TLWndData*	pWD, 
		ULONG		ulFlags, 
		RECT*		prc) 
{
	TLTreeItem* p;
	int y, item;
	BOOL bStarted;

	prc->left = BUTTONWIDTH;
	prc->right = prc->left + pWD->iFirstColumnWidth;

	prc->top = 0;
	prc->bottom = 0;
	p = pWD->rootItem;
	y = 0;
	item = 0;
	bStarted = FALSE;
	sCalcItemsDragBitmap (pWD, ulFlags, &item, &y, 0, p, &bStarted, prc);
}


//	_______________________________________________
//
//  create bitmap and paint selected items to it

BOOL 
TLMemPaintDragBitmap (
		TLWndData*			pWD,
		LPTL_DRAGBITMAP		lpdb) 
{
	HBITMAP	hbmOld;
	HDC		hdcMem;
	HDC		hdcScreen;
	RECT	rc;

	if (!lpdb) 
		return FALSE;

	// calculate bitmap size and position
	TLCalcDragBitmap (pWD, lpdb->ulFlags, &rc);
	if (rc.right <= rc.left)
		return FALSE;

	lpdb->sizeDrag.cx = rc.right - rc.left;
	lpdb->sizeDrag.cy = rc.bottom - rc.top;

	lpdb->ptHotSpot.x = lpdb->ptCursorPos.x - rc.left;
	lpdb->ptHotSpot.y = lpdb->ptCursorPos.y - rc.top - pWD->tlInval.top;

	// contstruct bitmap
	hdcMem = CreateCompatibleDC (NULL);
	hdcScreen = GetDC (pWD->hWnd);
	lpdb->hbmDrag = CreateCompatibleBitmap (hdcScreen, 
			lpdb->sizeDrag.cx, lpdb->sizeDrag.cy);
	ReleaseDC (pWD->hWnd, hdcScreen);

	hbmOld = SelectObject (hdcMem, lpdb->hbmDrag);
	SetBkMode (hdcMem, TRANSPARENT);
	rc.left = 0;
	rc.top = 0;
	rc.right = lpdb->sizeDrag.cx;
	rc.bottom = lpdb->sizeDrag.cy;
	FillRect (hdcMem, &rc, pWD->unselbgbrush);

	TLPaintDragBitmap (pWD, lpdb->ulFlags, hdcMem);

	SelectObject (hdcMem, hbmOld);

	DeleteDC (hdcMem);

	return TRUE;
}


