/*____________________________________________________________________________
		Copyright (C) 2002 PGP Corporation
        All rights reserved.

        $Id: CSecureRichEdit.h,v 1.8 2002/10/31 23:45:37 wjb Exp $
____________________________________________________________________________*/

#ifndef Included_CSecureRichEdit_h	// [
#define Included_CSecureRichEdit_h

#include "pgpClassesConfig.h"
#include "pgpUtilities.h"

#include "CArray.h"
#include "CSecureString.h"
#include "CEdit.h"

_PGP_BEGIN

// Class CSecureRichEdit

class CSecureRichEdit : public CEdit
{
public:
	enum
	{
		// For private communication with the parent dialog.
		WM_SECUREEDIT_EVENT	= WM_USER + 12, 

		kContentsChangedEvent	= 0, 
		kSawCapsLockKeyEvent	= 1, 
		kSawReturnKeyEvent		= 2
	};

public:
	CSecureRichEdit();
	CSecureRichEdit(HWND hWnd);
	~CSecureRichEdit();

	const CSecureString&	Contents() const;
	PGPUInt32 ContentsLength() const;	// returns curent number is Unicode characters

	PGPBoolean	HidingTyping() const {return mHideTyping;}
	PGPUInt32	MaxSize() const {return mMaxSize;}

	void	ClearContents();
	void	SetContents(const CSecureString& newContents);

	PGPBoolean	SetHideTypingPref(PGPBoolean hideValue);
	PGPUInt32	SetMaxSizeContents(PGPUInt32 maxSize);
	
	void	Subclass(HWND hWnd,PGPContextRef context);

	void	InstallSecurityHooks();
	void	UninstallSecurityHooks();

	HWND REhwnd(){return mREhwnd;}
	PGPUInt32	SendParentEvent(PGPUInt32 event);
	PGPBoolean  mOldCapsWarning;
	PGPBoolean	mUnicodeOS;
	WNDPROC		mOrigProc;
	PGPBoolean	mDestroyed;
	PGPBoolean  mPumping;
	PGPBoolean	mCapsOnInit;
private:
	CSecureString			mContentsUTF8;

	void		*mSEP;
	HWND		mREhwnd;

	PGPBoolean	mHideTyping;
	PGPUInt32	mMaxSize;
	PGPContextRef mContext;

	HHOOK	mHhookCBT;
	HHOOK	mHhookGetMessage;
	HHOOK	mHhookKeyboard;
	HHOOK	mHhookMsgFilter;

	static LRESULT CALLBACK	SecurityHookProc(INT iCode, WPARAM wParam, 
		LPARAM lParam);

	static LRESULT CALLBACK	SecurityGetMessageHookProc(INT iCode, 
		WPARAM wParam, LPARAM lParam);
};

_PGP_END

#endif	// ] Included_CSecureRichEdit_h
