/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.
	
	pgpUnicodeWin32.h	-	Unicode conversion calls for Win32
	
	$Id: pgpUnicodeWin32.h,v 1.7 2002/08/28 02:05:20 wjb Exp $
____________________________________________________________________________*/

#ifndef Included_pgpUnicodeWin32_h	/* [ */
#define Included_pgpUnicodeWin32_h


#if ! PGP_WIN32
#error this file should only be used for PGP_WIN32
#endif

#include "pgpTypes.h"
#include "pgpPFLErrors.h"

/*____________________________________________________________________________
 * For functions that accept a 'Flags' value, passing kPGPUnicodeFlag_Secure
 * will cause the function to break up strings into individual characters
 * when passing info to OS-supplied conversion functions.  This prevents
 * buffers of sensitive information being leaked by the OS routines.
 */

#define kPGPUnicodeFlag_Secure		0x0001

PGP_BEGIN_C_DECLARATIONS

/*____________________________________________________________________________
 * Convert a string from local character set to Unicode UTF-8
 * encoding.  
 */

PGPError
pgpLocalStringToUTF8 (
		PGPUInt32		uFlags,
		PGPUInt32		uCodePage,
		const char*		pszASCII,
		PGPUInt32		uLenASCII,
		char*			pszUTF8,
		PGPUInt32		uMaxLenUTF8,
		PGPUInt32*		puLenUTF8);

/*____________________________________________________________________________
 * Convert a string from Unicode UTF-8 to local character set
 * encoding.  
 */

PGPError
pgpUTF8StringToLocal (
		PGPUInt32		uFlags,
		PGPUInt32		uCodePage,
		const char*		pszUTF8,
		PGPUInt32		uLenUTF8,
		char*			pszASCII,
		PGPUInt32		uMaxLenASCII,
		PGPUInt32*		puLenASCII);

/*____________________________________________________________________________
 * Convert a string from local character set to Unicode UCS-2
 * encoding.  
 */

void
pgpLocalStringToUCS2 (
		PGPUInt32		uFlags,
		PGPUInt32		uCodePage,
		const char*		pszLocal,
		PGPUInt32		uLenLocal,
		PGPUInt16*		wszWide,
		PGPUInt32		uMaxLenWide,
		PGPUInt32*		puLenOut);

/*____________________________________________________________________________
 * Get current Code Page for the specified window.
 */

PGPUInt32
pgpGetCodePageFromWindow (
		void*			hwnd);


PGP_END_C_DECLARATIONS


#endif /* ] */
