/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: KeyGen.c,v 1.24 2002/10/30 18:06:01 pbj Exp $
____________________________________________________________________________*/

#include <windows.h>
#include "PGPadmin.h"
#include <commctrl.h>
#include "resource.h"
#include "Utils.h"

#include "PGPadminHelp.h"
#include "pgpClientLib.h"
#include "pgpBuild.h"
#include "pgpUtilities.h"
#include "pgpLDAP.h"

#define SECONDS_PER_DAY		86400

static void sInitGUI(HWND hwndDlg, pgpConfigInfo *pConfig);
static void sSaveSelection(HWND hwndDlg, pgpConfigInfo *pConfig);

static DWORD aHelpIds[] = {
	IDC_ALLOW_KEYGEN,		IDH_PGPADMIN_ALLOW_KEYGEN,
	IDC_SETKEYGEN,			IDH_PGPADMIN_SETKEYGEN,
	IDC_RADIO_ELGAMAL,		IDH_PGPADMIN_DHTYPE,
	IDC_RADIO_RSA,			IDH_PGPADMIN_RSATYPE,
	IDC_RADIO_RSALEGACY,	IDH_PGPADMIN_RSALEGACYTYPE,
	IDC_RADIO_NEVER,		IDH_PGPADMIN_NEVEREXPIRES,
	IDC_RADIO_EXPIRES,		IDH_PGPADMIN_EXPIRES,
	IDC_EXPIREDAYS,			IDH_PGPADMIN_EXPIRATIONDATE,
	IDC_RADIO_1024,			IDH_PGPADMIN_1024BITS,
	IDC_RADIO_1536,			IDH_PGPADMIN_1536BITS,
	IDC_RADIO_2048,			IDH_PGPADMIN_2048BITS,
	IDC_RADIO_3072,			IDH_PGPADMIN_3072BITS,
	IDC_RADIO_CUSTOM,		IDH_PGPADMIN_CUSTOMBITS,
	IDC_CUSTOM_BITS,		IDH_PGPADMIN_CUSTOMBITS,
	IDC_KEYGENCERTREQ,		IDH_PGPADMIN_KEYGENCERTREQUEST,
	IDC_ALLOWMANREQ,		IDH_PGPADMIN_ALLOWMANUALREQUEST,
	IDC_CATYPE,				IDH_PGPADMIN_CATYPE,
	IDC_ALWAYSSEND,			IDH_PGPADMIN_ALWAYSSEND,
	IDC_AUTORECON,			IDH_PGPADMIN_AUTORECONSTRUCTION,
	IDC_RECONSERVER,		IDH_PGPADMIN_RECONSTRUCTIONSERVER,
	IDC_AUTOSIGNCA,			IDH_PGPADMIN_AUTOSIGNROOTCAKEY,
	IDC_ADDATTRIBUTES,		IDH_PGPADMIN_ADDATTRIBUTES,
	IDC_SMARTCARD,			IDH_PGPADMIN_SMARTCARD,
	IDC_KEYTYPEGROUP,		-1,
	IDC_KEYEXPIREGROUP,		-1,
	IDC_EXPIRESPIN,			-1,
	IDC_EXPIRATIONDATE,		-1,
	IDC_KEYSIZEGROUP,		-1,
	IDC_X509GROUP,			-1,
	IDC_MISCGROUP,			-1,
    0,0
};


BOOL CALLBACK KeyGenDlgProc(HWND hwndDlg, 
							UINT uMsg, 
							WPARAM wParam, 
							LPARAM lParam)
{
	BOOL			bReturnCode = FALSE;
	pgpConfigInfo *	pConfig		= NULL;
	char			szKeySize[10];
	int				nIndex;
	char			szCA[255];

	g_hCurrentDlgWnd = hwndDlg;
	pConfig = (pgpConfigInfo *) GetWindowLong(hwndDlg, GWL_USERDATA);

	switch(uMsg)
	{
	case WM_INITDIALOG:
		CommonDlgProc(hwndDlg, uMsg, wParam, lParam);
		pConfig = (pgpConfigInfo *) GetWindowLong(hwndDlg, GWL_USERDATA);
		SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_RESETCONTENT, 0, 0);
		
		LoadString(g_hInstance, IDS_CAINVALID, szCA, sizeof(szCA));
		nIndex = SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_ADDSTRING, 0, 
					(LPARAM) szCA);
		
		SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_SETITEMDATA, nIndex, 
			kPGPKeyServerClass_Invalid);

		if (pConfig->nCAType == kPGPKeyServerClass_Invalid)
			SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_SETCURSEL, nIndex, 0);

		LoadString(g_hInstance, IDS_CANETTOOLS, szCA, sizeof(szCA));
		nIndex = SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_ADDSTRING, 0, 
					(LPARAM) szCA);
		
		SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_SETITEMDATA, nIndex, 
			kPGPKeyServerClass_NetToolsCA);
		
		if (pConfig->nCAType == kPGPKeyServerClass_NetToolsCA)
			SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_SETCURSEL, nIndex, 0);

		LoadString(g_hInstance, IDS_CAVERISIGN, szCA, sizeof(szCA));
		nIndex = SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_ADDSTRING, 0, 
					(LPARAM) szCA);
		
		SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_SETITEMDATA, nIndex, 
			kPGPKeyServerClass_Verisign);
		
		if (pConfig->nCAType == kPGPKeyServerClass_Verisign)
			SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_SETCURSEL, nIndex, 0);

		LoadString(g_hInstance, IDS_CAENTRUST, szCA, sizeof(szCA));
		nIndex = SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_ADDSTRING, 0, 
					(LPARAM) szCA);
		
		SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_SETITEMDATA, nIndex, 
			kPGPKeyServerClass_Entrust);
		
		if (pConfig->nCAType == kPGPKeyServerClass_Entrust)
			SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_SETCURSEL, nIndex, 0);

		LoadString(g_hInstance, IDS_CANETSCAPE, szCA, sizeof(szCA));
		nIndex = SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_ADDSTRING, 0, 
					(LPARAM) szCA);
		
		SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_SETITEMDATA, nIndex, 
			kPGPKeyServerClass_Netscape);
		
		if (pConfig->nCAType == kPGPKeyServerClass_Netscape)
			SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_SETCURSEL, nIndex, 0);

		LoadString(g_hInstance, IDS_CAMICROSOFT, szCA, sizeof(szCA));
		nIndex = SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_ADDSTRING, 0, 
					(LPARAM) szCA);
		
		SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_SETITEMDATA, nIndex, 
			kPGPKeyServerClass_Microsoft);
		
		if (pConfig->nCAType == kPGPKeyServerClass_Microsoft)
			SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_SETCURSEL, nIndex, 0);

		SendMessage(GetDlgItem(hwndDlg, IDC_EXPIRESPIN), UDM_SETRANGE, 0, 
			MAKELONG(999, 1));

		sInitGUI(hwndDlg, pConfig);
		break;

	case WM_HELP:
	case WM_CONTEXTMENU:
		return PGPclHtmlHelp (hwndDlg, uMsg, wParam, lParam, 
				(char *) kPGPclAdminHelpFile, aHelpIds);

	case WM_NOTIFY:
		{
			LPNMHDR pnmh;

			pnmh = (LPNMHDR) lParam;
			switch(pnmh->code)
			{
			case PSN_HELP:
				PGPclHtmlHelp (hwndDlg, kPGPclHelpContext, 0, 0, 
						(char *) kPGPclAdminHelpFile, 
						(DWORD *) IDH_PGPADMIN_KEYGENDIALOG);
				break;

			case PSN_KILLACTIVE:
				{
					char szErrorMsg[255];
					char szTitle[255];
					BOOL bError = FALSE;
					int nLength;
					HWND hwndCtrl;

					bReturnCode = TRUE;
					LoadString(g_hInstance, IDS_TITLE, szTitle, 254);

					// Check data validity

					if (pConfig->bSetKeyGen)
					{
						if (pConfig->keyType == kPGPPublicKeyAlgorithm_RSA)
						{
							if ((pConfig->nKeySize < 1024) || 
								(pConfig->nKeySize > 2048))
							{
								LoadString(g_hInstance, 
									IDS_E_MINKEYSIZERANGERSA, szErrorMsg, 254);
								hwndCtrl = GetDlgItem(hwndDlg, 
											IDC_CUSTOM_BITS);
								bError = TRUE;
							}
						}
						else
						{
							if ((pConfig->nKeySize < 1024) || 
								(pConfig->nKeySize > 4096))
							{
								LoadString(g_hInstance, 
									IDS_E_MINKEYSIZERANGE, szErrorMsg, 254);
								hwndCtrl = GetDlgItem(hwndDlg, 
											IDC_CUSTOM_BITS);
								bError = TRUE;
							}
						}

						if (bError)
						{
							MessageBox(hwndDlg, szErrorMsg, szTitle, MB_OK);
							SetFocus(hwndCtrl);
							SetWindowLong(hwndDlg, DWL_MSGRESULT, -1);
						}
					}

					if (pConfig->szReconServer)
					{
						PGPFreeData(pConfig->szReconServer);
						pConfig->szReconServer = NULL;
					}

					nLength = GetWindowTextLength(GetDlgItem(hwndDlg, 
								IDC_RECONSERVER));

					if (nLength)
					{
						pConfig->szReconServer = (char *)
							PGPNewData(pConfig->memoryMgr,
								nLength + 1,
								kPGPMemoryMgrFlags_Clear);

						GetWindowText(GetDlgItem(hwndDlg, IDC_RECONSERVER),
							pConfig->szReconServer, nLength + 1);

						pConfig->szReconServer[nLength] = 0;
					}

					if ((pConfig->szReconServer) &&
						(pConfig->szReconServer[0]))
					{
						PGPldapURLDesc*		pURLDesc	= NULL;
						PGPldapContextRef	ldap;
						PGPError			err;

						PGPNewLDAPContext (pConfig->pgpContext, &ldap);
						err = PGPldapURLParse (ldap, pConfig->szReconServer, &pURLDesc);
						if (err == kPGPError_LDAPNoDN)
							pConfig->nReconServerType = kPGPKeyServerClass_PGP;
						else
							pConfig->nReconServerType = kPGPKeyServerClass_LDAPX509;

						if (pURLDesc)
							PGPFreeLDAPURLDesc (pURLDesc);

						PGPFreeLDAPContext (ldap);
					}

					break;
				}

			default:
				bReturnCode = CommonDlgProc(hwndDlg, uMsg, wParam, lParam);
				break;
			}
			
			break;
		}

	case WM_COMMAND:
		if (pConfig == NULL)
			break;

		switch (LOWORD(wParam))
		{
		case IDC_ALLOW_KEYGEN:
			if (IsDlgButtonChecked(hwndDlg, IDC_ALLOW_KEYGEN) == BST_CHECKED)
				pConfig->bAllowKeyGen = TRUE;
			else
				pConfig->bAllowKeyGen = FALSE;

			sInitGUI(hwndDlg, pConfig);
			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_SETKEYGEN:
			if (IsDlgButtonChecked(hwndDlg, IDC_SETKEYGEN) == BST_CHECKED)
				pConfig->bSetKeyGen = TRUE;
			else
			{
				pConfig->bSetKeyGen = FALSE;
				pConfig->bUseSmartCard = FALSE;
			}

			sInitGUI(hwndDlg, pConfig);
			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_SMARTCARD:
			if (IsDlgButtonChecked(hwndDlg, IDC_SMARTCARD) == BST_CHECKED)
				pConfig->bUseSmartCard = TRUE;
			else
				pConfig->bUseSmartCard = FALSE;

			sInitGUI(hwndDlg, pConfig);
			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_RADIO_ELGAMAL:
		case IDC_RADIO_RSA:
		case IDC_RADIO_RSALEGACY:
			if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_ELGAMAL) == BST_CHECKED)
				pConfig->keyType = kPGPPublicKeyAlgorithm_DSA;
			else
			{
				if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_RSA) == 
					BST_CHECKED)
					pConfig->keyType = kPGPPublicKeyAlgorithm_RSASignOnly;
				else
				{
					char szWarnMsg[255];
					char szTitle[255];

					LoadString(g_hInstance, IDS_TITLE, szTitle, 254);
					LoadString(g_hInstance, IDS_W_RSALEGACY, szWarnMsg, 254);
					MessageBox(hwndDlg, szWarnMsg, szTitle, MB_ICONINFORMATION);
					pConfig->keyType = kPGPPublicKeyAlgorithm_RSA;

					if (pConfig->nKeySize > 2048)
						pConfig->nKeySize = 2048;
				}
			}

			sInitGUI(hwndDlg, pConfig);
			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_RADIO_NEVER:
		case IDC_RADIO_EXPIRES:
			if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_NEVER) == BST_CHECKED)
				pConfig->keyExpiration = 0;
			else
			{
				SYSTEMTIME stExpire;
				PGPTime oneYear;

				GetLocalTime(&stExpire);
				stExpire.wYear++;
				SystemTimeToPGPTime(&stExpire, &oneYear);
				pConfig->keyExpiration = oneYear - PGPGetTime();
			}

			sInitGUI(hwndDlg, pConfig);
			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_EXPIREDAYS:
			{
				char szBuffer[4];
				PGPUInt32 nValue;

				GetWindowText(GetDlgItem(hwndDlg, IDC_EXPIREDAYS), 
					szBuffer, 4);
				nValue = atoi(szBuffer) * SECONDS_PER_DAY;
				if (!nValue && 
					(IsDlgButtonChecked(hwndDlg, IDC_RADIO_EXPIRES) == BST_CHECKED))
				{
					char szErrMsg[255];
					char szTitle[255];

					LoadString(g_hInstance, IDS_TITLE, szTitle, 254);
					LoadString(g_hInstance, IDS_E_MINEXPIRE, szErrMsg, 254);
					SetWindowText(GetDlgItem(hwndDlg, IDC_EXPIREDAYS), "1");
					MessageBox(hwndDlg, szErrMsg, szTitle, MB_ICONEXCLAMATION);
				}
				else
					pConfig->keyExpiration = nValue;
				bReturnCode = TRUE;
				PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
				break;
			}

		case IDC_RADIO_1024:
		case IDC_RADIO_1536:
		case IDC_RADIO_2048:
		case IDC_RADIO_3072:
		case IDC_RADIO_CUSTOM:
			if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_1024) == BST_CHECKED)
				pConfig->nKeySize = 1024;
			if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_1536) == BST_CHECKED)
				pConfig->nKeySize = 1536;
			if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_2048) == BST_CHECKED)
				pConfig->nKeySize = 2048;
			if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_3072) == BST_CHECKED)
				pConfig->nKeySize = 3072;

			if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_CUSTOM) == BST_CHECKED)
			{
				GetWindowText(GetDlgItem(hwndDlg, IDC_CUSTOM_BITS), 
					szKeySize, 10);
				pConfig->nKeySize = atoi(szKeySize) - 1;
				if (pConfig->nKeySize < 1024)
					pConfig->nKeySize = 1025;
			}

			sInitGUI(hwndDlg, pConfig);
			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_CUSTOM_BITS:
			GetWindowText(GetDlgItem(hwndDlg, IDC_CUSTOM_BITS), szKeySize, 
				10);
			pConfig->nKeySize = atoi(szKeySize);
			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_KEYGENCERTREQ:
			if (IsDlgButtonChecked(hwndDlg, IDC_KEYGENCERTREQ) == BST_CHECKED)
				pConfig->bKeyGenCertRequest = TRUE;
			else
				pConfig->bKeyGenCertRequest = FALSE;

			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_ALLOWMANREQ:
			if (IsDlgButtonChecked(hwndDlg, IDC_ALLOWMANREQ) == BST_CHECKED)
				pConfig->bAllowManualCertRequest = TRUE;
			else
				pConfig->bAllowManualCertRequest = FALSE;

			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_AUTOSIGNCA:
			if (IsDlgButtonChecked(hwndDlg, IDC_AUTOSIGNCA) == BST_CHECKED)
				pConfig->bAutoSignRootCAKey = TRUE;
			else
				pConfig->bAutoSignRootCAKey = FALSE;

			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_ALWAYSSEND:
			if (IsDlgButtonChecked(hwndDlg, IDC_ALWAYSSEND) == BST_CHECKED)
				pConfig->bAlwaysSendKeys = TRUE;
			else
				pConfig->bAlwaysSendKeys = FALSE;

			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_AUTORECON:
			if (IsDlgButtonChecked(hwndDlg, IDC_AUTORECON) == BST_CHECKED)
				pConfig->bAutoKeyReconstruction = TRUE;
			else
				pConfig->bAutoKeyReconstruction = FALSE;

			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_CATYPE:
			switch (HIWORD(wParam))
			{
			case CBN_SELENDOK:
				sSaveSelection(hwndDlg, pConfig);
				sInitGUI(hwndDlg, pConfig);
				bReturnCode = TRUE;
				PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
				break;
			}
			break;

		case IDC_ADDATTRIBUTES:
			sSaveSelection(hwndDlg, pConfig);
			bReturnCode = TRUE;
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;

		case IDC_RECONSERVER:
			PropSheet_Changed(GetParent(hwndDlg), hwndDlg);
			break;
		}

		break;

	default:
		bReturnCode = CommonDlgProc(hwndDlg, uMsg, wParam, lParam);
		break;
	}

	return(bReturnCode);
}


static void sInitGUI(HWND hwndDlg, pgpConfigInfo *pConfig)
{
	char sz1536[255];
	char sz2048[255];
	char sz3072[255];
	char szCustom[255];

	if (pConfig->bAllowKeyGen)
		CheckDlgButton(hwndDlg, IDC_ALLOW_KEYGEN, BST_CHECKED);
	else
	{
		CheckDlgButton(hwndDlg, IDC_ALLOW_KEYGEN, BST_UNCHECKED);
		pConfig->bSetKeyGen = FALSE;
	}
	
	EnableWindow(GetDlgItem(hwndDlg, IDC_SETKEYGEN), pConfig->bAllowKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_MISCGROUP), pConfig->bAllowKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_ALWAYSSEND), pConfig->bAllowKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_AUTORECON), pConfig->bAllowKeyGen);

	if (pConfig->bSetKeyGen)
		CheckDlgButton(hwndDlg, IDC_SETKEYGEN, BST_CHECKED);
	else
		CheckDlgButton(hwndDlg, IDC_SETKEYGEN, BST_UNCHECKED);
	
	if (pConfig->bUseSmartCard)
		CheckDlgButton(hwndDlg, IDC_SMARTCARD, BST_CHECKED);
	else
		CheckDlgButton(hwndDlg, IDC_SMARTCARD, BST_UNCHECKED);
	
	EnableWindow(GetDlgItem(hwndDlg, IDC_SMARTCARD), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_KEYTYPEGROUP), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_KEYEXPIREGROUP), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_KEYSIZEGROUP), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_ELGAMAL), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_RSA), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_RSALEGACY), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_NEVER), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_EXPIRES), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_EXPIRATIONDATE), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_1024), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_1536), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_2048), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_3072), pConfig->bSetKeyGen);
	EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_CUSTOM), pConfig->bSetKeyGen);
	
	switch (pConfig->keyType)
	{
	case kPGPPublicKeyAlgorithm_DSA:
		CheckDlgButton(hwndDlg, IDC_RADIO_ELGAMAL, BST_CHECKED);
		ShowWindow(GetDlgItem(hwndDlg, IDC_RADIO_3072), SW_SHOW);
		LoadString(g_hInstance, IDS_KG_DH1536, sz1536, 254);
		LoadString(g_hInstance, IDS_KG_DH2048, sz2048, 254);
		LoadString(g_hInstance, IDS_KG_DH3072, sz3072, 254);
		LoadString(g_hInstance, IDS_KG_DHCUSTOM, szCustom, 254);
		break;
		
	case kPGPPublicKeyAlgorithm_RSASignOnly:
		CheckDlgButton(hwndDlg, IDC_RADIO_RSA, BST_CHECKED);
		ShowWindow(GetDlgItem(hwndDlg, IDC_RADIO_3072), SW_SHOW);
		LoadString(g_hInstance, IDS_KG_RSA1536, sz1536, 254);
		LoadString(g_hInstance, IDS_KG_RSA2048, sz2048, 254);
		LoadString(g_hInstance, IDS_KG_RSA3072, sz3072, 254);
		LoadString(g_hInstance, IDS_KG_DHCUSTOM, szCustom, 254);
		break;
		
	case kPGPPublicKeyAlgorithm_RSA:
		CheckDlgButton(hwndDlg, IDC_RADIO_RSALEGACY, BST_CHECKED);
		ShowWindow(GetDlgItem(hwndDlg, IDC_RADIO_3072), SW_HIDE);
		LoadString(g_hInstance, IDS_KG_RSA1536, sz1536, 254);
		LoadString(g_hInstance, IDS_KG_RSA2048, sz2048, 254);
		LoadString(g_hInstance, IDS_KG_RSACUSTOM, szCustom, 254);
		break;
	}
	
	SetWindowText(GetDlgItem(hwndDlg, IDC_RADIO_1536), sz1536);
	SetWindowText(GetDlgItem(hwndDlg, IDC_RADIO_2048), sz2048);
	SetWindowText(GetDlgItem(hwndDlg, IDC_RADIO_3072), sz3072);
	SetWindowText(GetDlgItem(hwndDlg, IDC_RADIO_CUSTOM), szCustom);
	
	if (pConfig->keyExpiration)
	{
		char szExpireDays[4];

		wsprintf(szExpireDays, "%d", pConfig->keyExpiration / SECONDS_PER_DAY);
		CheckDlgButton(hwndDlg, IDC_RADIO_EXPIRES, BST_CHECKED);
		SetWindowText(GetDlgItem(hwndDlg, IDC_EXPIREDAYS), szExpireDays);
		EnableWindow(GetDlgItem(hwndDlg, IDC_EXPIREDAYS), 
			pConfig->bSetKeyGen);
		EnableWindow(GetDlgItem(hwndDlg, IDC_EXPIRESPIN), 
			pConfig->bSetKeyGen);
	}
	else
	{
		CheckDlgButton(hwndDlg, IDC_RADIO_NEVER, BST_CHECKED);
		SetWindowText(GetDlgItem(hwndDlg, IDC_EXPIREDAYS), "0");
		EnableWindow(GetDlgItem(hwndDlg, IDC_EXPIREDAYS), FALSE);
		EnableWindow(GetDlgItem(hwndDlg, IDC_EXPIRESPIN), FALSE);
	}
	
	CheckDlgButton(hwndDlg, IDC_RADIO_1024, BST_UNCHECKED);
	CheckDlgButton(hwndDlg, IDC_RADIO_1536, BST_UNCHECKED);
	CheckDlgButton(hwndDlg, IDC_RADIO_2048, BST_UNCHECKED);
	CheckDlgButton(hwndDlg, IDC_RADIO_3072, BST_UNCHECKED);
	CheckDlgButton(hwndDlg, IDC_RADIO_CUSTOM, BST_UNCHECKED);
	
	switch (pConfig->nKeySize)
	{
	case 1024:
		CheckDlgButton(hwndDlg, IDC_RADIO_1024, BST_CHECKED);
		EnableWindow(GetDlgItem(hwndDlg, IDC_CUSTOM_BITS), FALSE);
		break;
		
	case 1536:
		CheckDlgButton(hwndDlg, IDC_RADIO_1536, BST_CHECKED);
		EnableWindow(GetDlgItem(hwndDlg, IDC_CUSTOM_BITS), FALSE);
		break;
		
	case 2048:
		CheckDlgButton(hwndDlg, IDC_RADIO_2048, BST_CHECKED);
		EnableWindow(GetDlgItem(hwndDlg, IDC_CUSTOM_BITS), FALSE);
		break;
		
	case 3072:
		CheckDlgButton(hwndDlg, IDC_RADIO_3072, BST_CHECKED);
		EnableWindow(GetDlgItem(hwndDlg, IDC_CUSTOM_BITS), FALSE);
		break;
		
	default:
		CheckDlgButton(hwndDlg, IDC_RADIO_CUSTOM, BST_CHECKED);
		EnableWindow(GetDlgItem(hwndDlg, IDC_CUSTOM_BITS), 
			pConfig->bSetKeyGen);
		break;
	}
	
	wsprintf(szCustom, "%d", pConfig->nKeySize);
	SetWindowText(GetDlgItem(hwndDlg, IDC_CUSTOM_BITS), szCustom);
	
	if (pConfig->bKeyGenCertRequest)
		CheckDlgButton(hwndDlg, IDC_KEYGENCERTREQ, BST_CHECKED);
	else
		CheckDlgButton(hwndDlg, IDC_KEYGENCERTREQ, BST_UNCHECKED);
	
	if (pConfig->bAllowManualCertRequest)
		CheckDlgButton(hwndDlg, IDC_ALLOWMANREQ, BST_CHECKED);
	else
		CheckDlgButton(hwndDlg, IDC_ALLOWMANREQ, BST_UNCHECKED);

	if (pConfig->bAutoSignRootCAKey)
		CheckDlgButton(hwndDlg, IDC_AUTOSIGNCA, BST_CHECKED);
	else
		CheckDlgButton(hwndDlg, IDC_AUTOSIGNCA, BST_UNCHECKED);

	if (pConfig->bAlwaysSendKeys)
		CheckDlgButton(hwndDlg, IDC_ALWAYSSEND, BST_CHECKED);
	else
		CheckDlgButton(hwndDlg, IDC_ALWAYSSEND, BST_UNCHECKED);
	
	if (pConfig->bAutoKeyReconstruction)
		CheckDlgButton(hwndDlg, IDC_AUTORECON, BST_CHECKED);
	else
		CheckDlgButton(hwndDlg, IDC_AUTORECON, BST_UNCHECKED);
	
	if (pConfig->nCAType == kPGPKeyServerClass_Invalid)
		EnableWindow(GetDlgItem(hwndDlg, IDC_ADDATTRIBUTES), FALSE);
	else
		EnableWindow(GetDlgItem(hwndDlg, IDC_ADDATTRIBUTES), TRUE);

	if (pConfig->szReconServer != NULL)
		SetWindowText(GetDlgItem(hwndDlg, IDC_RECONSERVER), 
			pConfig->szReconServer);
	else
		SetWindowText(GetDlgItem(hwndDlg, IDC_RECONSERVER), "");

	return;
}


static void sSaveSelection(HWND hwndDlg, pgpConfigInfo *pConfig)
{
	int nIndex;
	PGPAttributeValue *pAVList = NULL;
	PGPUInt32 nNumAVs = 0;
	PGPError err = kPGPError_NoErr;

	nIndex = SendDlgItemMessage(hwndDlg, IDC_CATYPE, CB_GETCURSEL, 0, 0);
				
	pConfig->nOldCAType = pConfig->nCAType;
	pConfig->nCAType = (PGPKeyServerClass) SendDlgItemMessage(hwndDlg, 
											IDC_CATYPE, CB_GETITEMDATA, 
											nIndex, 0);

	if ((pConfig->nOldCAType == pConfig->nCAType) &&
		(pConfig->avData != NULL))
		PGPclDataToAVPairs(pConfig->memoryMgr, pConfig->avData, 
			pConfig->avDataSize, &pAVList, &nNumAVs);

	
	if (pConfig->nCAType != kPGPKeyServerClass_Invalid)
		err = PGPclGetCACertRequestAVList(hwndDlg, pConfig->pgpContext, TRUE, 
				NULL, pConfig->nCAType, &(pConfig->nCertRequestType),
				&pAVList, &nNumAVs);

	if (IsntPGPError(err))
	{
		if (pConfig->avData != NULL)
		{
			PGPFreeData(pConfig->avData);
			pConfig->avData = NULL;
			pConfig->avDataSize = 0;
		}

		if (pAVList != NULL)
		{
			PGPclAVPairsToData(pConfig->memoryMgr, pAVList, nNumAVs, 
				&(pConfig->avData), &(pConfig->avDataSize));

			PGPclFreeCACertRequestAVList(pAVList, nNumAVs);
		}
	}
	else if (pAVList != NULL)
		PGPFreeData(pAVList);

	pConfig->bCopyClientPrefs = TRUE;
	return;			
}


/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
