/*____________________________________________________________________________
	Copyright (C) 2002 PGP Corporation
	All rights reserved.

	$Id: pgpNetPrefs.c,v 1.114 2002/08/06 20:10:25 dallen Exp $
____________________________________________________________________________*/
#include "pgpNetPrefs.h"
#include "pflPrefTypes.h"
#include "pgpMem.h"
#include "pgpEndianConversion.h"
#include "pgpUtilities.h"

#include "pgpPFLPriv.h"

#if PGP_WORDSBIGENDIAN
#define PGPIPAddress(a,b,c,d) a<<24|b<<16|c<<8|d
#else
#define PGPIPAddress(a,b,c,d) d<<24|c<<16|b<<8|a
#endif

/*	beginning of code formerly in pgpNetPrefs.h .... */
static PGPByte sBlankNetAdminHash[] = {
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};

static PGPPrefStructMember sPreSharedCASTPropArray[] =
{
	{kPGPPrefType_Number, (void *) kPGPike_AM_PreSharedKey},
	{kPGPPrefType_Number, (void *) kPGPike_HA_SHA1},
	{kPGPPrefType_Number, (void *) kPGPike_SC_CAST_CBC},
	{kPGPPrefType_Number, (void *) kPGPike_GR_MODPFive},
};

static PGPPrefStructMember sPreShared3DESPropArray[] =
{
	{kPGPPrefType_Number, (void *) kPGPike_AM_PreSharedKey},
	{kPGPPrefType_Number, (void *) kPGPike_HA_MD5},
	{kPGPPrefType_Number, (void *) kPGPike_SC_3DES_CBC},
	{kPGPPrefType_Number, (void *) kPGPike_GR_MODPTwo},
};

static PGPPrefStructMember sDSSSHACASTPropArray[] =
{
	{kPGPPrefType_Number, (void *) kPGPike_AM_DSS_Sig},
	{kPGPPrefType_Number, (void *) kPGPike_HA_SHA1},
	{kPGPPrefType_Number, (void *) kPGPike_SC_CAST_CBC},
	{kPGPPrefType_Number, (void *) kPGPike_GR_MODPFive},
};

static PGPPrefStructMember sDSSSHA3DESPropArray[] =
{
	{kPGPPrefType_Number, (void *) kPGPike_AM_DSS_Sig},
	{kPGPPrefType_Number, (void *) kPGPike_HA_SHA1},
	{kPGPPrefType_Number, (void *) kPGPike_SC_3DES_CBC},
	{kPGPPrefType_Number, (void *) kPGPike_GR_MODPTwo},
};

static PGPPrefStructMember sRSASHACASTPropArray[] =
{
	{kPGPPrefType_Number, (void *) kPGPike_AM_RSA_Sig},
	{kPGPPrefType_Number, (void *) kPGPike_HA_SHA1},
	{kPGPPrefType_Number, (void *) kPGPike_SC_CAST_CBC},
	{kPGPPrefType_Number, (void *) kPGPike_GR_MODPFive},
};

static PGPPrefStructMember sRSAMD53DESPropArray[] =
{
	{kPGPPrefType_Number, (void *) kPGPike_AM_RSA_Sig},
	{kPGPPrefType_Number, (void *) kPGPike_HA_MD5},
	{kPGPPrefType_Number, (void *) kPGPike_SC_3DES_CBC},
	{kPGPPrefType_Number, (void *) kPGPike_GR_MODPTwo},
};

static PGPPrefStruct sPreSharedCASTProposal =
{
	sizeof(sPreSharedCASTPropArray) / sizeof(PGPPrefStructMember),
	sPreSharedCASTPropArray,
};

static PGPPrefStruct sPreShared3DESProposal =
{
	sizeof(sPreShared3DESPropArray) / sizeof(PGPPrefStructMember),
	sPreShared3DESPropArray,
};

static PGPPrefStruct sDSSSHACASTProposal =
{
	sizeof(sDSSSHACASTPropArray) / sizeof(PGPPrefStructMember),
	sDSSSHACASTPropArray,
};

static PGPPrefStruct sDSSSHA3DESProposal =
{
	sizeof(sDSSSHA3DESPropArray) / sizeof(PGPPrefStructMember),
	sDSSSHA3DESPropArray,
};

static PGPPrefStruct sRSASHACASTProposal =
{
	sizeof(sRSASHACASTPropArray) / sizeof(PGPPrefStructMember),
	sRSASHACASTPropArray,
};

static PGPPrefStruct sRSAMD53DESProposal =
{
	sizeof(sRSAMD53DESPropArray) / sizeof(PGPPrefStructMember),
	sRSAMD53DESPropArray,
};

static PGPPrefArrayElement sDefaultIKEArray[] =
{
	{&sPreSharedCASTProposal}, {&sPreShared3DESProposal},
	{&sDSSSHACASTProposal}, {&sDSSSHA3DESProposal},
	{&sRSASHACASTProposal}, {&sRSAMD53DESProposal}
};

static PGPPrefArray sDefaultIKEProposalsA =
{
	kPGPPrefType_Struct,
	sizeof(sDefaultIKEArray) / sizeof(PGPPrefArrayElement),
	sDefaultIKEArray,
	&sPreSharedCASTProposal
};

static PGPPrefStructMember sCASTSHAIPSecArray[] =
{
	{kPGPPrefType_Boolean, (void *) TRUE},
	{kPGPPrefType_Number, (void *) kPGPike_ET_CAST},
	{kPGPPrefType_Number, (void *) kPGPike_AA_HMAC_SHA},
	{kPGPPrefType_Boolean, (void *) FALSE},
	{kPGPPrefType_Number, (void *) kPGPike_AH_None},
	{kPGPPrefType_Number, (void *) kPGPike_AA_None},
	{kPGPPrefType_Boolean, (void *) FALSE},
	{kPGPPrefType_Number, (void *) kPGPike_IC_None},
};

static PGPPrefStructMember sCASTMD5IPSecArray[] =
{
	{kPGPPrefType_Boolean, (void *) TRUE},
	{kPGPPrefType_Number, (void *) kPGPike_ET_CAST},
	{kPGPPrefType_Number, (void *) kPGPike_AA_HMAC_MD5},
	{kPGPPrefType_Boolean, (void *) FALSE},
	{kPGPPrefType_Number, (void *) kPGPike_AH_None},
	{kPGPPrefType_Number, (void *) kPGPike_AA_None},
	{kPGPPrefType_Boolean, (void *) FALSE},
	{kPGPPrefType_Number, (void *) kPGPike_IC_None},
};

static PGPPrefStructMember s3DESMD5IPSecArray[] =
{
	{kPGPPrefType_Boolean, (void *) TRUE},
	{kPGPPrefType_Number, (void *) kPGPike_ET_3DES},
	{kPGPPrefType_Number, (void *) kPGPike_AA_HMAC_MD5},
	{kPGPPrefType_Boolean, (void *) FALSE},
	{kPGPPrefType_Number, (void *) kPGPike_AH_None},
	{kPGPPrefType_Number, (void *) kPGPike_AA_None},
	{kPGPPrefType_Boolean, (void *) FALSE},
	{kPGPPrefType_Number, (void *) kPGPike_IC_None},
};

static PGPPrefStruct sCASTSHAIPSecProposal =
{
	sizeof(sCASTSHAIPSecArray) / sizeof(PGPPrefStructMember),
	sCASTSHAIPSecArray,
};

static PGPPrefStruct sCASTMD5IPSecProposal =
{
	sizeof(sCASTMD5IPSecArray) / sizeof(PGPPrefStructMember),
	sCASTMD5IPSecArray,
};

static PGPPrefStruct s3DESMD5IPSecProposal =
{
	sizeof(s3DESMD5IPSecArray) / sizeof(PGPPrefStructMember),
	s3DESMD5IPSecArray,
};

static PGPPrefArrayElement sDefaultIPSecArray[] =
{
	{&sCASTSHAIPSecProposal}, {&sCASTMD5IPSecProposal}, {&s3DESMD5IPSecProposal}
};

static PGPPrefArray sDefaultIPSecProposalsA =
{
	kPGPPrefType_Struct,
	sizeof(sDefaultIPSecArray) / sizeof(PGPPrefArrayElement),
	sDefaultIPSecArray,
	&sCASTSHAIPSecProposal
};

const PGPNetPrefIKEProposalEntry sDefaultIKEProposals[] =
{
	{
		kPGPike_AM_PreSharedKey,
		kPGPike_HA_SHA1,
		kPGPike_SC_CAST_CBC,
		kPGPike_GR_MODPFive
	},
	{
		kPGPike_AM_PreSharedKey,
		kPGPike_HA_MD5,
		kPGPike_SC_3DES_CBC,
		kPGPike_GR_MODPTwo
	},
	{
		kPGPike_AM_DSS_Sig,
		kPGPike_HA_SHA1,
		kPGPike_SC_CAST_CBC,
		kPGPike_GR_MODPFive
	},
	{
		kPGPike_AM_DSS_Sig,
		kPGPike_HA_SHA1,
		kPGPike_SC_3DES_CBC,
		kPGPike_GR_MODPTwo
	},
	{
		kPGPike_AM_RSA_Sig,
		kPGPike_HA_SHA1,
		kPGPike_SC_CAST_CBC,
		kPGPike_GR_MODPFive
	},
	{
		kPGPike_AM_RSA_Sig,
		kPGPike_HA_MD5,
		kPGPike_SC_3DES_CBC,
		kPGPike_GR_MODPTwo
	}
};

const PGPSize sDefaultIKEProposalsSize = sizeof(sDefaultIKEProposals);


const PGPNetPrefIPSECProposalEntry sDefaultIPSECProposals[] =
{
	{	/* ESP, CAST, SHA1 */
		TRUE, kPGPike_ET_CAST, kPGPike_AA_HMAC_SHA,
		FALSE, kPGPike_AH_None, kPGPike_AA_None,
		FALSE, kPGPike_IC_None
	},
	{	/* ESP, CAST, MD5 */
		TRUE, kPGPike_ET_CAST, kPGPike_AA_HMAC_MD5,
		FALSE, kPGPike_AH_None, kPGPike_AA_None,
		FALSE, kPGPike_IC_None
	},
	{	/* ESP, 3DES, MD5 */
		TRUE, kPGPike_ET_3DES, kPGPike_AA_HMAC_MD5,
		FALSE, kPGPike_AH_None, kPGPike_AA_None,
		FALSE, kPGPike_IC_None
	},
};

const PGPSize sDefaultIPSECProposalsSize = sizeof(sDefaultIPSECProposals);


static PGPPrefStructMember sTemplateHostMember[] =
{
	{kPGPPrefType_Number, (void *) kPGPnetInsecureHost},
	{kPGPPrefType_Number, (void *) 0},
	{kPGPPrefType_Number, (void *) 0xFFFFFFFF},
	{kPGPPrefType_Number, (void *) -1},
	{kPGPPrefType_String, (void *) ""},
	{kPGPPrefType_String, (void *) ""},
	{kPGPPrefType_Boolean, (void *) TRUE},
	{kPGPPrefType_Boolean, (void *) FALSE},
	{kPGPPrefType_Boolean, (void *) FALSE},
	
	{kPGPPrefType_Byte, (void *) NULL},
	{kPGPPrefType_Byte, (void *) NULL},
	{kPGPPrefType_Number, (void *) 0},

	{kPGPPrefType_Number, (void *) 0},
	{kPGPPrefType_Boolean, (void *) FALSE},
	{kPGPPrefType_Boolean, (void *) FALSE},
	{kPGPPrefType_Number, (void *) 0},
	{kPGPPrefType_String, (void *) ""},
	
	{kPGPPrefType_Byte, (void *) NULL},
	{kPGPPrefType_Byte, (void *) NULL},
	{kPGPPrefType_Number, (void *) 0},

	{kPGPPrefType_Number, (void *) kPGPike_EM_Auto}
};

static PGPPrefStruct sTemplateHostStruct =
{
	sizeof(sTemplateHostMember) / sizeof(PGPPrefStructMember),
	sTemplateHostMember,
};

static PGPPrefArray sTemplateHost =
{
	kPGPPrefType_Struct,
	0,
	NULL,
	&sTemplateHostStruct
};

static PGPPrefStructMember sTemplateBlockedMember[] =
{
	{kPGPPrefType_Number, (void *) 0},
	{kPGPPrefType_Number, (void *) 0},
	{kPGPPrefType_Number, (void *) 0},
	{kPGPPrefType_Number, (void *) 0}
};

static PGPPrefStruct sTemplateBlockedStruct =
{
	sizeof(sTemplateBlockedMember) / sizeof(PGPPrefStructMember),
	sTemplateBlockedMember,
};

static PGPPrefArray sTemplateBlocked =
{
	kPGPPrefType_Struct,
	0,
	NULL,
	&sTemplateBlockedStruct
};

static PGPPrefStructMember sTemplateTrustedMember[] =
{
	{kPGPPrefType_Boolean, (void *) 0},	/*bIsRange*/
	{kPGPPrefType_Number, (void *) 0},	/*ipAddrStart*/
	{kPGPPrefType_Number, (void *) 0}	/*ipMaskEnd*/
};

static PGPPrefStruct sTemplateTrustedStruct =
{
	sizeof(sTemplateTrustedMember) / sizeof(PGPPrefStructMember),
	sTemplateTrustedMember,
};

static PGPPrefArray sTemplateTrusted =
{
	kPGPPrefType_Struct,
	0,
	NULL,
	&sTemplateTrustedStruct
};

static PGPPrefStructMember sTemplateFirewallRuleMember[] =
{
	{kPGPPrefType_Boolean, (void *) TRUE},	/*active*/
	{kPGPPrefType_Boolean, (void *) TRUE},	/*permit*/
	{kPGPPrefType_Boolean, (void *) FALSE},	/*matchIntrusion*/
	{kPGPPrefType_Number, (void *) 0},		/*direction*/
	{kPGPPrefType_Number, (void *) 0},		/*ipProtocol*/
	{kPGPPrefType_Number, (void *) 0},		/*remoteAddrType*/
	{kPGPPrefType_Number, (void *) 0},		/*remoteIPAddrStart*/
	{kPGPPrefType_Number, (void *) 0},		/*remoteIPMaskEnd*/
	{kPGPPrefType_Number, (void *) 0},		/*remoteServiceType*/
	{kPGPPrefType_Number, (void *) 0},		/*numRemoteServices*/
	{kPGPPrefType_Number, (void *) 0},		/*remoteService[0]*/
	{kPGPPrefType_Number, (void *) 0},		/*remoteService[1]*/
	{kPGPPrefType_Number, (void *) 0},		/*remoteService[2]*/
	{kPGPPrefType_Number, (void *) 0},		/*remoteService[3]*/
	{kPGPPrefType_Number, (void *) 0},		/*localServiceType*/
	{kPGPPrefType_Number, (void *) 0},		/*numLocalServices*/
	{kPGPPrefType_Number, (void *) 0},		/*localService[0]*/
	{kPGPPrefType_Number, (void *) 0},		/*localService[1]*/
	{kPGPPrefType_Number, (void *) 0},		/*localService[2]*/
	{kPGPPrefType_Number, (void *) 0},		/*localService[3]*/
	{kPGPPrefType_String, (void *) ""},		/*description*/
	{kPGPPrefType_String, (void *) ""},		/*appPath*/
	{kPGPPrefType_Byte, (void *) NULL},		/*hash*/
	{kPGPPrefType_Number, (void *) 0},		/*transient*/
	{kPGPPrefType_Number, (void *) 0},		/*ruleType*/
	{kPGPPrefType_Boolean, (void *) FALSE}, /*logrule*/
};

static PGPPrefStruct sTemplateFirewallRuleStruct =
{
	sizeof(sTemplateFirewallRuleMember) / sizeof(PGPPrefStructMember),
	sTemplateFirewallRuleMember,
};

static PGPPrefArray sTemplateFirewallRule =
{
	kPGPPrefType_Struct,
	0,
	NULL,
	&sTemplateFirewallRuleStruct
};

const PGPPrefDefinition netDefaults[] =
{
	{kPGPNetPrefRequireValidKeyConfigured,
	"RequireValidKeyConf",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefRequireValidKeyUnconfigured,
	"RequireValidKeyUnconf",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefEnablePassphraseCache,
	"EnableCache",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefEnablePGPnet,
	"EnablePGPnet",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefEnableVPN,
	"EnableVPN",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefEnableFirewall,
	"EnableFW",
	kPGPPrefType_Boolean,
#if PGP_OSX
	(void *) FALSE},
#else
	(void *) TRUE},
#endif

	{kPGPNetPrefExpertMode,
	"ExpertMode",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefEnableIKEKByteExpiration,
	"AllowIKEKBExpiration",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefEnableIKETimeExpiration,
	"AllowIKETimeExpiration",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefEnableIPSECKByteExpiration,
	"AllowIPSecKBExpiration",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefEnableIPSECTimeExpiration,
	"AllowIPSecTimeExpiration",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefFirstExecution,
	"FirstRunNet",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefWarnReSharedSecret,
	"WarnSharedSecret",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefReplayDetection,
	"EnableReplayDetect",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefUseIntrusionDetection,
	"UseIntrusionDetect",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAutoBlockAttacks,
	"AutoBlockAttacks",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefBlockUntilRemoved,
	"BlockUntilRemoved",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAttackEmailNotify,
	"AttackEmailNotify",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefAttackDialog,
	"AttackDialog",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAttackSound,
	"AttackSound",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAttackNotify,
	"AttackNotify",
	kPGPPrefType_Boolean,
	(void *) TRUE},
	
	{kPGPNetPrefUseLearnMode,
	"FirewallLearnMode",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAllowCipherNone,
	"AllowNoCipher",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefAllowCipherAES,
	"AllowAES",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAllowCipherCAST,
	"AllowCAST",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAllowCipher3DES,
	"AllowTripleDES",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAllowCipherDES56,
	"AllowSDES",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefAllowHashNone,
	"AllowNoHash",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefAllowHashSHA1,
	"AllowSHAHash",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAllowHashMD5,
	"AllowMD5Hash",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAllowCompLZS,
	"AllowLZSCompress",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAllowCompDeflate,
	"AllowDeflateCompress",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAllowDH768,
	"Allow1MODP",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefAllowDH1024,
	"Allow2MODP",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAllowDH1536,
	"Allow5MODP",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefAllowEC163,
	"AllowEC163x",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefAllowEC283,
	"AllowEC283x",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefConfirmSADeletion,
	"ConfirmSADeletion",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefConfirmLogClearing,
	"ConfirmLogClearing",
	kPGPPrefType_Boolean,
	(void *) TRUE},

	{kPGPNetPrefLogAllowed,
	"LogAllowed",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefLogBlocked,
	"LogBlocked",
	kPGPPrefType_Boolean,
	(void *) FALSE},

	{kPGPNetPrefIKEKByteExpiration,
	"IKEKBExpiration",
	kPGPPrefType_Number,
	(void *) 5120},

	{kPGPNetPrefIKETimeExpiration,
	"IKETimeExpiration",
	kPGPPrefType_Number,
	(void *) 86400},

	{kPGPNetPrefIPSECKByteExpiration,
	"IPSecKBExpiration",
	kPGPPrefType_Number,
	(void *) 1048576},

	{kPGPNetPrefIPSECTimeExpiration,
	"IPSecTimeExpiration",
	kPGPPrefType_Number,
	(void *) 28800},

	{kPGPnetPrefIPSECGroupID,
	"PFSGroup",
	kPGPPrefType_Number,
	(void *) 0},

	{kPGPnetPrefUnconfiguredMode,
	"UnconfiguredMode",
	kPGPPrefType_Number,
	(void *) kPGPnet_UM_Allow},

	{kPGPnetPrefIKEPort,
	"IKEPortUDP",
	kPGPPrefType_Number,
	(void *) kPGPike_CommonPort},

	{kPGPnetPrefBlockMinutes,
	"AttackBlockMinutes",
	kPGPPrefType_Number,
	(void *) 20},

	{kPGPnetPrefFirewallRuleSet,
	"FirewallSetNum",
	kPGPPrefType_Number,
#if PGP_WIN32
	(void *) kPGPNetFirewallRuleSet_LearnStarter},
#else
	(void *) kPGPNetFirewallRuleSet_Minimal},
#endif

	{kPGPnetPrefLastFirewallRuleListChange,
	"LastFirewallRuleListChange",
	kPGPPrefType_Number,
	(void *) 0},

	{kPGPNetPrefTrustSubnetOption,
	"TrustedSubnetOption",
	kPGPPrefType_Number,
	(void *) 0},

	{kPGPNetPrefHostArray,
	"HostList",
	kPGPPrefType_Array,
	(void *) &sTemplateHost,
	sizeof(sTemplateHost)},

	{kPGPNetPrefIKEProposalArray,
	"IKEProposals",
	kPGPPrefType_Array,
	(void *) &sDefaultIKEProposalsA,
	sizeof(sDefaultIKEProposalsA)},

	{kPGPNetPrefIPSecProposalArray,
	"IPSecProposals",
	kPGPPrefType_Array,
	(void *) &sDefaultIPSecProposalsA,
	sizeof(sDefaultIPSecProposalsA)},

	{kPGPNetPrefBlockedHostArray,
	"BlockedHostList",
	kPGPPrefType_Array,
	(void *) &sTemplateBlocked,
	sizeof(sTemplateBlocked)},

	{kPGPNetPrefTrustedHostArray,
	"TrustedHostList",
	kPGPPrefType_Array,
	(void *) &sTemplateTrusted,
	sizeof(sTemplateTrusted)},

	{kPGPNetPrefFirewallRuleArray,
	"FirewallRuleList",
	kPGPPrefType_Array,
	(void *) &sTemplateFirewallRule,
	sizeof(sTemplateFirewallRule)},

	{kPGPNetPrefPublicKeyringFile,
	"PublicKeyringFile",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPNetPrefPrivateKeyringFile,
	"PrivateKeyringFile",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPNetPrefAttackNotifyEmail,
	"AttackNotifyEmail",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPNetPrefAttackSoundName,
	"AttackSoundName",
	kPGPPrefType_String,
#if PGP_OSX
	(void *) ""},
#else
	(void *) "Missile"},
#endif

	{kPGPNetPrefWhoisServer,
	"WhoisServer",
	kPGPPrefType_String,
	(void *) "whois.networksolutions.com"},

	{kPGPNetPrefAttackNotifySMTPServer,
	"AttackNotifySMTPServer",
	kPGPPrefType_String,
	(void *) ""},

	{kPGPNetPrefPGPAuthKeyID,
	"PGPAuthKeyID",
	kPGPPrefType_Byte,
	(void *) NULL},

	{kPGPNetPrefX509AuthKeyID,
	"X509AuthCertID",
	kPGPPrefType_Byte,
	(void *) NULL},

	{kPGPNetPrefX509AuthCertIASN,
	"X509AuthCertIASN",
	kPGPPrefType_Byte,
	(void *) NULL},

	{kPGPNetPrefWinAppWindowData,
	"Win32WindowData",
	kPGPPrefType_Byte,
	(void *) NULL},

	{kPGPNetPrefMacMainWindowPos,
	"MacWindowPosData",
	kPGPPrefType_Byte,
	(void *) NULL},

	{kPGPPrefAdminHash,
	"NetAdminHash",
	kPGPPrefType_Byte,
	(void *) sBlankNetAdminHash,
	sizeof(sBlankNetAdminHash)},
};

const PGPSize netDefaultsSize =
					sizeof(netDefaults) / sizeof(PGPPrefDefinition);

const PGPPrefDefinition hostDefaults[] =
{
	{kPGPNetPrefHostArray,
	"HostList",
	kPGPPrefType_Array,
	(void *) &sTemplateHost,
	sizeof(sTemplateHost)},
};

const PGPSize hostDefaultsSize =
					sizeof(hostDefaults) / sizeof(PGPPrefDefinition);

static const PGPPrefDefinition blockedDefaults[] =
{
	{kPGPNetPrefBlockedHostArray,
	"BlockedHostList",
	kPGPPrefType_Array,
	(void *) &sTemplateBlocked,
	sizeof(sTemplateBlocked)},
};

static const PGPSize blockedDefaultsSize =
					sizeof(blockedDefaults) / sizeof(PGPPrefDefinition);

static const PGPPrefDefinition trustedDefaults[] = 
{
	{kPGPNetPrefTrustedHostArray,
		"TrustedHostList",
		kPGPPrefType_Array,
		(void *) &sTemplateTrusted,
		sizeof(sTemplateTrusted)},
};

static const PGPSize trustedDefaultsSize =
					sizeof(trustedDefaults) / sizeof(PGPPrefDefinition);

const PGPPrefDefinition ruleDefaults[] =
{
	{kPGPNetPrefFirewallRuleArray,
	"FirewallRuleList",
	kPGPPrefType_Array,
	(void *) &sTemplateFirewallRule,
	sizeof(sTemplateFirewallRule)},
};
					
const PGPSize ruleDefaultsSize =
					sizeof(ruleDefaults) / sizeof(PGPPrefDefinition);


const PGPPrefDefinition knownRuleDefaults[] =
{
	{kPGPNetPrefAutoRuleArray,
	"AutoRuleList",
	kPGPPrefType_Array,
	(void *) &sTemplateFirewallRule,
	sizeof(sTemplateFirewallRule)},

	{kPGPNetPrefKnownRuleArray,
	"KnownRuleList",
	kPGPPrefType_Array,
	(void *) &sTemplateFirewallRule,
	sizeof(sTemplateFirewallRule)},

	{kPGPNetPrefSmartRuleArray,
	"SmartRuleList",
	kPGPPrefType_Array,
	(void *) &sTemplateFirewallRule,
	sizeof(sTemplateFirewallRule)},
};
					
const PGPSize knownRuleDefaultsSize =
					sizeof(knownRuleDefaults) / sizeof(PGPPrefDefinition);

/*	... end of code formerly in pgpNetPrefs.h */

static PGPNetPrefFirewallRule sPresetFirewallRules_Minimal[] =
{
	{
		/* Allow IPsec ESP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ESP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow IPsec ESP", "", {0}, 0, kPGPFireRuleType_Top, FALSE
	},
	{
		/* ICMP Group */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Ping and ICMP", "", {0}, 0, kPGPFireRuleType_Group, FALSE
	},
	{
		/* Block incoming pings */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_EchoRequest,0,0,0}, 
		"Block incoming pings", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{
		/* Block incoming timestamp */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_TimestampRequest,0,0,0}, 
		"Block ICMP Timestamp", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{
		/* Block incoming addr mask request */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_AddressMaskReq,0,0,0}, 
		"Block ICMP Addr Mask", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{
		/* Block incoming info request */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_InfoRequest,0,0,0}, 
		"Block ICMP Info Req", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{
		/* Block incoming router solicit */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_RouterSolicitation,0,0,0}, 
		"Block ICMP Router Solicit", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{
		/* Block incoming redirect */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_Redirect,0,0,0}, 
		"Block ICMP Redirect", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{
		/* Allow all ICMP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow all ICMP", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{	/* Allow bootp */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_bootps,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_bootpc,0,0,0}, 
		"Allow bootp", "", {0}
	},
	{	/* Allow DNS */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_dns,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow DNS", "", {0}
	},
	{	/* Allow IKE */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ike,0,0,0}, 
		"Allow IKE", "", {0}
	},
	{	/* Allow Network Time Protocol */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ntp,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ntp,0,0,0}, 
		"Allow Net Time Protocol", "", {0}
	},
	{	/* Allow auth */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 1, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_auth,0,0,0}, 
		"Allow auth", "", {0}
	},
	{	/* NetBIOS Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"NetBIOS Group", "", {0}, 0, kPGPFireRuleType_Group, FALSE
	},
	{	/* Allow NetBIOS Name Service */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ns,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ns,0,0,0}, 
		"Allow NetBIOS Name Service", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{	/* Allow NetBIOS Datagram */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_dgm,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_dgm,0,0,0}, 
		"Allow NetBIOS Datagram", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{	/* Allow NetBIOS Session Incoming */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0}, 
		"Allow NetBIOS Session Incoming", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{	/* Allow NetBIOS Session Outbound */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow NetBIOS Session Outgoing", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{	/* Allow NetBIOS TCP Session Incoming */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0}, 
		"Allow NetBIOS TCP Incoming", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{	/* Block NetBIOS Session Incoming */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0},
		"Block NetBIOS TCP Incoming", "", {0}, 0, kPGPFireRuleType_Child, FALSE
	},
	{	/* Allow high UDP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0}, 
		"Allow all high UDP", "", {0}
	},
	{	/* Allow all other TCP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow all TCP", "", {0}
	}
};

static PGPNetPrefFirewallRule sPresetFirewallRules_ClientMedium[] =
{
	{
		/* Allow IPsec ESP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ESP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow IPsec ESP", "", {0}
	},
	{
		/* Allow incoming pings */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_EchoReply,0,0,0}, 
		"Allow ping reply Incoming", "", {0}
	},
	{
		/* Allow outgoing pings */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_EchoRequest,0,0,0}, 
		"Allow ping req Outgoing", "", {0}
	},
	{
		/* Allow dest unreachable */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_DestUnreachable,0,0,0}, 
		"Allow ICMP Dest Unr Incoming", "", {0}
	},
	{
		/* Allow TTL Exceeded < */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_TimeExceeded,0,0,0}, 
		"Allow ICMP TTL Incoming", "", {0}
	},
	{
		/* Allow source quench */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_SourceQuench,0,0,0}, 
		"Allow ICMP Src Quench", "", {0}
	},
	{	/* Allow bootp */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_bootps,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_bootpc,0,0,0}, 
		"Allow bootp", "", {0}
	},
	{	/* Allow DNS */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_dns,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow DNS", "", {0}
	},
	{	/* Allow IKE */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ike,0,0,0}, 
		"Allow IKE", "", {0}
	},
	{	/* Allow Network Time Protocol */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ntp,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ntp,0,0,0}, 
		"Allow Net Time Protocol", "", {0}
	},
	{	/* Allow auth */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 1, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_auth,0,0,0}, 
		"Allow auth", "", {0}
	},
	{	/* Allow NetBIOS Name Service */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ns,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ns,0,0,0}, 
		"Allow NetBIOS Name Serv", "", {0}
	},
	{	/* Allow NetBIOS Datagram */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_dgm,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_dgm,0,0,0}, 
		"Allow NetBIOS Datagram", "", {0}
	},
	{	/* Allow NetBIOS Session Outbound */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow NetBIOS Session Outgoing", "", {0}
	},
	{	/* Allow high UDP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Allow all high UDP", "", {0}
	},
	{	/* Allow ftpdata in */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ftpdata,0,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Allow ftpdata in", "", {0}
	},
	{	/* Allow all other TCP Outbound only */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow all TCP out", "", {0}
	}
};

static PGPNetPrefFirewallRule sPresetFirewallRules_ClientHigh[] =
{
	{
		/* Allow IPsec ESP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ESP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow IPsec ESP", "", {0}
	},
	{
		/* Allow ICMP Dest Unreachable */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_DestUnreachable,0,0,0}, 
		"Allow ICMP Dest Un", "", {0}
	},
	{
		/* Allow source quench */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_SourceQuench,0,0,0}, 
		"Allow ICMP Src Quench", "", {0}
	},
	{	/* Allow bootp */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_bootps,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_bootpc,0,0,0}, 
		"Allow bootp", "", {0}
	},
	{	/* Allow DNS */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_dns,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Allow DNS", "", {0}
	},
	{	/* Allow IKE */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ike,0,0,0}, 
		"Allow IKE", "", {0}
	},
	{	/* Allow Network Time Protocol */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ntp,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ntp,0,0,0},
		"Allow Net Time Protocol", "", {0}
	},
	{	/* Allow all other TCP Outbound only */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow all TCP out", "", {0}
	}
};

static PGPNetPrefFirewallRule sPresetFirewallRules_ServerMedium[] =
{
	{
		/* Allow IPsec ESP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ESP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow IPsec ESP", "", {0}
	},
	{
		/* Allow incoming pings */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_EchoReply,0,0,0}, 
		"Allow ping reply", "", {0}
	},
	{
		/* Allow outgoing pings */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_EchoRequest,0,0,0}, 
		"Allow ping req", "", {0}
	},
	{
		/* Allow source quench */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_SourceQuench,0,0,0}, 
		"Allow ICMP Src Quench", "", {0}
	},
	{
		/* Allow TTL Exceeded < */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_TimeExceeded,0,0,0}, 
		"Allow ICMP TTL Incoming", "", {0}
	},
	{	/* Allow bootp */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_bootps,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_bootpc,0,0,0}, 
		"Allow bootp", "", 
	},
	{	/* Allow DNS */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_dns,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Allow DNS", "", {0}
	},
	{	/* Allow IKE */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ike,0,0,0}, 
		"Allow IKE", "", {0}
	},
	{	/* Allow Network Time Protocol */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ntp,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ntp,0,0,0},
		"Allow Net Time Protocol", "", {0}
	},
	{	/* Allow high UDP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Allow all high UDP", "", {0}
	},
	{	/* Allow ftpdata out */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ftpdata,0,0,0},
		"Allow ftpdata out", "", {0}
	},
	{	/* Allow SMTP out */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_smtp,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Allow SMTP out", "", {0}
	},
	{	/* Allow all other TCP Incoming only */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Allow all TCP in", "", {0}
	}
};

static PGPNetPrefFirewallRule sPresetFirewallRules_ServerHigh[] =
{
	{
		/* Allow IPsec ESP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ESP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow IPsec ESP", "", {0}
	},
	{
		/* Allow ICMP Dest Unreachable */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_DestUnreachable,0,0,0}, 
		"Allow ICMP Dest Un", "", {0}
	},
	{
		/* Allow source quench */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_SourceQuench,0,0,0}, 
		"Allow ICMP Src Quench", "", {0}
	},
	{	/* Allow bootp */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_bootps,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_bootpc,0,0,0}, 
		"Allow bootp", "", {0}
	},
	{	/* Allow DNS */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_dns,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow DNS", "", {0}
	},
	{	/* Allow IKE */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ike,0,0,0}, 
		"Allow IKE", "", {0}
	},
	{	/* Allow Network Time Protocol */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ntp,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ntp,0,0,0}, 
		"Allow Net Time Protocol", "", {0}
	},
	{	/* Allow ftpdata out */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ftpdata,0,0,0}, 
		"Allow ftpdata out", "", {0}
	},
	{	/* Allow SMTP out */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_smtp,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Allow SMTP out", "", {0}
	},
	{	/* Allow SMTP in */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_smtp,0,0,0},
		"Allow SMTP in", "", {0}
	},
	{	/* Allow IMAP in */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_imap3,0,0,0},
		"Allow IMAP3 in", "", {0}
	},
	{	/* Allow POP3 in */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_pop3,0,0,0},
		"Allow POP3 in", "", {0}
	},
	{	/* Allow HTTP in */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_http,0,0,0},
		"Allow HTTP in", "", {0}
	},
	{	/* Allow HTTPS in */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_https,0,0,0},
		"Allow HTTPS in", "", {0}
	},
	{	/* Allow LDAP in */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ldap,0,0,0},
		"Allow LDAP in", "", {0}
	},
	{	/* Allow LDAPS in */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_sldap,0,0,0},
		"Allow LDAPS in", "", {0}
	},
	{	/* Allow NNTP in */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_nntp,0,0,0},
		"Allow NNTP in", "", {0}
	},
};

static PGPNetPrefFirewallRule sPresetFirewallRules_LearnStarter[] =
{
	{
		/* Allow IPsec ESP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ESP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow IPsec ESP", "", {0}, 0, kPGPFireRuleType_Top
	},
	{
		/* ICMP Group */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Ping and ICMP", "", {0}, 0, kPGPFireRuleType_Group
	},
	{
		/* Block incoming pings */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_EchoRequest,0,0,0}, 
		"Block incoming pings", "", {0}, 0, kPGPFireRuleType_Child
	},
	{
		/* Block incoming timestamp */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_TimestampRequest,0,0,0}, 
		"Block ICMP Timestamp", "", {0}, 0, kPGPFireRuleType_Child
	},
	{
		/* Block incoming addr mask request */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_AddressMaskReq,0,0,0}, 
		"Block ICMP Addr Mask", "", {0}, 0, kPGPFireRuleType_Child
	},
	{
		/* Block incoming info request */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_InfoRequest,0,0,0}, 
		"Block ICMP Info Req", "", {0}, 0, kPGPFireRuleType_Child
	},
	{
		/* Block incoming router solicit */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_RouterSolicitation,0,0,0}, 
		"Block ICMP Router Solicit", "", {0}, 0, kPGPFireRuleType_Child
	},
	{
		/* Block incoming redirect */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_Redirect,0,0,0}, 
		"Block ICMP Redirect", "", {0}, 0, kPGPFireRuleType_Child
	},
	{
		/* Allow all ICMP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow all ICMP", "", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow bootp */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_bootps,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_bootpc,0,0,0}, 
		"Allow bootp", "", {0}, 0, kPGPFireRuleType_Top
	},
	{	/* Allow DNS */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_dns,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow DNS", "", {0}, 0, kPGPFireRuleType_Top
	},
	{	/* Allow IKE */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ike,0,0,0}, 
		"Allow IKE", "", {0}, 0, kPGPFireRuleType_Top
	},
	{	/* Allow Network Time Protocol */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ntp,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ntp,0,0,0}, 
		"Allow Net Time Protocol", "", {0}, 0, kPGPFireRuleType_Top
	},
	{	/* Allow auth */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 1, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_auth,0,0,0}, 
		"Allow auth", "", {0}, 0, kPGPFireRuleType_Top
	},
	{	/* NetBIOS Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"NetBIOS Group", "", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow NetBIOS Name Service */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ns,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ns,0,0,0}, 
		"Allow NetBIOS Name Service", "", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow NetBIOS Datagram */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_dgm,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_dgm,0,0,0}, 
		"Allow NetBIOS Datagram", "", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow NetBIOS Session Inbound */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0}, 
		"Allow NetBIOS Session Incoming", "", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow NetBIOS Session Outbound */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		"Allow NetBIOS Session Outgoing", "", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow NetBIOS TCP Session Inbound */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_netbios_ssn,kPGPnetTCPService_msds,0,0}, 
		"Allow NetBIOS TCP Incoming", "", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Block NetBIOS Session Inbound */
		TRUE, FALSE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_netbios_ssn,kPGPnetTCPService_msds,0,0},
		"Block NetBIOS TCP Incoming", "", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow NetBIOS TCP Session Outbound */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_netbios_ssn,kPGPnetTCPService_msds,0,0}, 
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Allow NetBIOS TCP Outgoing", "", {0}, 0, kPGPFireRuleType_Child
	}
};


static PGPNetPrefFirewallRule sPresetKnownRules_Auto[] =
{
	{	/* Allow DNS */
		TRUE, TRUE,FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_dns,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Automatic Rule - Allow DNS", "", {0},
		0, kPGPFireRuleType_Top
	},
	{	/* Allow IPsec ESP */
		TRUE, TRUE,FALSE, 0, kPGPnetIPProtocol_ESP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Automatic Rule - Allow IPsec ESP", "", {0},
		0, kPGPFireRuleType_Top
	},
	{	/* Allow IKE */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ike,0,0,0}, 
		"Automatic Rule - Allow IKE", "", {0},
		0, kPGPFireRuleType_Top
	},
	{	/* Allow auth */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 1, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_auth,0,0,0}, 
		"Automatic Rule - Allow auth", "", {0},
		0, kPGPFireRuleType_Top
	}
};

static PGPNetPrefFirewallRule sPresetKnownRules_Known[] =
{
	{	/* PGPService Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"PGPService",
		"[PGPDIR]\\PGPSERVICE.EXE", {0}, 0,	kPGPFireRuleType_Group
	},
	{	/* PGPService */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 4, {kPGPnetTCPService_smtp,0,0,0},
		kPGPnetIPServiceType_Any, 1, {0,0,0,0},
		"Known Rule - Allow Service SMTP Outgoing",
		"[PGPDIR]\\PGPSERVICE.EXE", {0}, 0,	kPGPFireRuleType_Child
	},
	{	/* PGPnet Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"PGPnet",
		"[PGPDIR]\\PGPNET.EXE", {0}, 0,	kPGPFireRuleType_Group
	},
	{	/* PGPnet */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_ftp,
									   kPGPnetTCPService_telnet,
									   kPGPnetTCPService_smtp,
									   kPGPnetTCPService_whois},
		kPGPnetIPServiceType_Range, 2, {1024, 65535,0,0},
		"Known Rule - Allow Service Trace Outgoing",
		"[PGPDIR]\\PGPNET.EXE", {0}, 0,	kPGPFireRuleType_Child
	},
	{	/* PGPnet */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_http,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Known Rule - Allow Web Trace Outgoing",
		"[PGPDIR]\\PGPNET.EXE", {0}, 0,	kPGPFireRuleType_Child
	},
	{	/* PGPnet */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_dns,
									   kPGPnetTCPService_netbios_ns,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Known Rule - Allow DNS, NetBIOS Trace Outgoing",
		"[PGPDIR]\\PGPNET.EXE", {0}, 0,	kPGPFireRuleType_Child
	},
	{	/* PGPnet */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Known Rule - Allow ICMP Trace",
		"[PGPDIR]\\PGPNET.EXE", {0}, 0,	kPGPFireRuleType_Child
	},
	{	/* Allow PGPNET outgoing everything */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Known Rule - Allow Outgoing",
		"[PGPDIR]\\PGPNET.EXE", {0}, 0,	kPGPFireRuleType_Child
	},
	{	/* PGPTray Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"PGPTray",
		"[PGPDIR]\\PGPTRAY.EXE", {0}, 0,	kPGPFireRuleType_Group
	},
	{	/* Allow PGPTRAY outgoing everything */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Known Rule - Allow Outgoing",
		"[PGPDIR]\\PGPTRAY.EXE", {0}, 0,	kPGPFireRuleType_Child
	},
	{	/* PGPKeys Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"PGPKeys",
		"[PGPDIR]\\PGPKEYS.EXE", {0}, 0,	kPGPFireRuleType_Group
	},
	{	/* Allow PGPKeys outgoing LDAP */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 3, {kPGPnetTCPService_ldap,11370,11371,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Known Rule - Allow LDAP Outgoing",
		"[PGPDIR]\\PGPKEYS.EXE", {0}, 0,	kPGPFireRuleType_Child
	},
	{	/* ePO Agent Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"ePolicy Orchestrator Agent",
		"[ANYDIR]\\NAIMAS32.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow ePO Agent */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 3, {kPGPnetTCPService_http,
									   81,
									   kPGPnetTCPService_webcache,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow ePO Agent HTTP",
		"[ANYDIR]\\NAIMAS32.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow ePO Agent Ping */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_tproxy,0,0,0},
		"Smart Rule - Allow ePO Agent Ping",
		"[ANYDIR]\\NAIMAS32.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* McAfee LWI Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"McAfee Lightweight Installer",
		"[ANYDIR]\\LWI.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow LWI FTP TCP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,
									   kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow LWI FTP and FTP Data TCP",
		"[ANYDIR]\\LWI.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow LWI FTP UDP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow LWI FTP and FTP Data UDP",
		"[ANYDIR]\\LWI.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow LWI FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow FTP TCP",
		"[ANYDIR]\\LWI.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Services Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP, 
		kPGPnetAddrType_Any, 0, 0, 
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"System",
		"[SYSDIR]\\KERNEL32.DLL", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow DNS */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_dns,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Known Rule - Allow System DNS",
		"[SYSDIR]\\KERNEL32.DLL", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Kernel UDP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_netbios_ns,
									   kPGPnetTCPService_netbios_dgm,0,0},
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_netbios_ns,
									   kPGPnetTCPService_netbios_dgm,0,0},
		"Known Rule - Allow System NetBIOS",
		"[SYSDIR]\\KERNEL32.DLL", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Kernel UDP */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Range, 2,{1024,65535,0,0},
		kPGPnetIPServiceType_Single, 1,{kPGPnetTCPService_netbios_ns,0,0,0},
		"Known Rule - Allow NetBIOS Name Service Incoming",
		"[SYSDIR]\\KERNEL32.DLL", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Incoming Kernel Netbios */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0},
		"Known Rule - Allow System NetBIOS Incoming",
		"[SYSDIR]\\KERNEL32.DLL", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Outgoing Kernel Netbios */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_netbios_ssn,
									   kPGPnetTCPService_msds,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Known Rule - Allow System NetBIOS Outgoing",
		"[SYSDIR]\\KERNEL32.DLL", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Outgoing Kernel Netbios */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0},
		kPGPnetIPServiceType_Single, 1, {0,0,0,0},
		"Known Rule - Allow System NetBIOS Outgoing 2",
		"[SYSDIR]\\KERNEL32.DLL", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Services Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP, 
		kPGPnetAddrType_Any, 0, 0, 
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Service Host",
		"[SYSDIR]\\SVCHOST.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Svchost RPC */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Single, 1, {135,0,0,0},
		"Known Rule - Allow RPC UDP",
		"[SYSDIR]\\SVCHOST.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Svchost RPC */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0, 
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {135,0,0,0},
		"Known Rule - Allow RPC TCP",
		"[SYSDIR]\\SVCHOST.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Svchost UDP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0, 
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Known Rule - Allow Service Host UDP",
		"[SYSDIR]\\SVCHOST.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Services Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP, 
		kPGPnetAddrType_Any, 0, 0, 
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Services",
		"[SYSDIR]\\SERVICES.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Services DNS */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP, 
		kPGPnetAddrType_Any, 0, 0, 
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_dns,0,0,0}, 
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Known Rule - Allow DNS",
		"[SYSDIR]\\SERVICES.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Local Services NetBIOS */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP, 
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_bootps,
									   kPGPnetTCPService_bootpc,
									   kPGPnetTCPService_netbios_ns,
									   kPGPnetTCPService_netbios_dgm},
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_bootps,
									   kPGPnetTCPService_bootpc,
									   kPGPnetTCPService_netbios_ns,
									   kPGPnetTCPService_netbios_dgm},
		"Known Rule - Allow DHCP and NetBios",
		"[SYSDIR]\\SERVICES.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Local Services NetBIOS */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0}, 
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0},
		"Known Rule - Allow NetBIOS Incoming",
		"[SYSDIR]\\SERVICES.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Local Services NetBIOS */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Known Rule - Allow NetBIOS Outgoing",
		"[SYSDIR]\\SERVICES.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* RPCSS Group */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {135,0,0,0},
		"Remote Procedure Call Service",
		"[SYSDIR]\\RPCSS.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Local RPCSS */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {135,0,0,0},
		"Known Rule - Allow RPC TCP Incoming",
		"[SYSDIR]\\RPCSS.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Local RPCSS */	
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Single, 1, {135,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Known Rule - Allow RPC TCP Outgoing",
		"[SYSDIR]\\RPCSS.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Local RPCSS */	
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0, 
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {135,0,0,0},
		"Known Rule - Allow RPC UDP Incoming",
		"[SYSDIR]\\RPCSS.EXE", {0}, 0, kPGPFireRuleType_Child		
	},
	{	/* Allow Local RPCSS */	
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0, 
		kPGPnetIPServiceType_Single, 1, {135,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Known Rule - Allow RPC UDP Outgoing",
		"[SYSDIR]\\RPCSS.EXE", {0}, 0, kPGPFireRuleType_Child		
	},
	{	/* MPREXE Group*/
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All, 
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"MPREXE",
		"[SYSDIR]\\MPREXE.EXE", {0}, 0, kPGPFireRuleType_Group		
	},
	{	/* Allow MPREXE */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP, 
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Known Rule - Allow NetBIOS Session Outgoing",
		"[SYSDIR]\\MPREXE.EXE", {0}, 0, kPGPFireRuleType_Child		
	},
};

/* when adding to this list always add a group rule for
   the application.  the algorithm which checks if smart
   rules exists, looks specifically for group rules.
   this save processing time instead of enumerating thru
   the entire list */

static PGPNetPrefFirewallRule sPresetKnownRules_Smart[] =
{
	{	/* ePO Server Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"ePO Server",
		"[ANYDIR]\\NAIMSERV.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow ePO Server */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_http,
									   81,
									   kPGPnetTCPService_webcache,
									   0},
		"Smart Rule - Allow ePO Server HTTP",
		"[ANYDIR]\\NAIMSERV.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* McAfee VirusScan Update Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"VirusScan Update",
		"[PRGDIR]\\MUPDATE.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow McAfee VirusScan Update */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_http,0,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow VirusScan Update Outgoing",
		"[PRGDIR]\\MUPDATE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* window update Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Windows Critical Update",
		"[WINDIR]\\WUCRTUPD.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow window update */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_http,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[WINDIR]\\WUCRTUPD.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* window update Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Windows Update Loader",
		"[WINDIR]\\WULOADER.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow window update */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_http,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[WINDIR]\\WULOADER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* window update Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Windows Update",
		"[WINDIR]\\WUAUCLT.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow window update */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_http,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[WINDIR]\\WUAUCLT.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Winipcfg */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Winipcfg",
		"[WINDIR]\\WINIPCFG.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Winipcfg */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_bootps,kPGPnetTCPService_bootpc,0,0},
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_bootps,kPGPnetTCPService_bootpc,0,0},
		"Smart Rule - Allow DHCP",
		"[WINDIR]\\WINIPCFG.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Mapisp32 Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"MAPISP32",
		"[PRGDIR]\\MAPISP32.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Mapisp32 */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0, 
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {135,0,0,0},
		"Smart Rule - Allow RPC Local",
		"[PRGDIR]\\MAPISP32.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Mapisp32 */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0, 
		kPGPnetIPServiceType_Single, 1, {135,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow RPC Remote",
		"[PRGDIR]\\MAPISP32.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Mapisp32 */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0, 
		kPGPnetIPServiceType_Range, 2, {1024, 65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024, 65535,0,0},
		"Smart Rule - Allow MAPI UDP",
		"[PRGDIR]\\MAPISP32.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Mapisp32 */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0, 
		kPGPnetIPServiceType_Range, 2, {1024, 65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024, 65535,0,0},
		"Smart Rule - Allow MAPI TCP",
		"[PRGDIR]\\MAPISP32.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Printer Spooler Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Printer Spooler",
		"[WINDIR]\\SPOOLSS.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Printer Spooler */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {631,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Printer Spooler TCP Outgoing",
		"[WINDIR]\\SPOOLSS.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Printer Spooler */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {631,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Printer Spooler UDP Outgoing",
		"[WINDIR]\\SPOOLSS.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Printer Spooler Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Printer Spooler",
		"[WINDIR]\\SPOOLSV.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Printer Spooler */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {631,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Printer Spooler TCP Outgoing",
		"[WINDIR]\\SPOOLSV.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Printer Spooler */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {631,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Printer Spooler UDP Outgoing",
		"[WINDIR]\\SPOOLSV.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* SSH Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Secure Shell",
		"[ANYDIR]\\SSH.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow SSH */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_ssh,0,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow Secure Shell Outgoing",
		"[ANYDIR]\\SSH.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* ICQ Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"ICQ",
		"[ANYDIR]\\ICQ.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow ICQ */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_http,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[ANYDIR]\\ICQ.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow ICQ */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow ICQ UDP",
		"[ANYDIR]\\ICQ.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow ICQ */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow ICQ TCP",
		"[ANYDIR]\\ICQ.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Internet Explorer Group */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Internet Explorer",
		"[ANYDIR]\\IEXPLORE.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Internet Explorer web ports*/
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_http, 81, 82, 83},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[ANYDIR]\\IEXPLORE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Internet Explorer web ports*/
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_https, 
									   kPGPnetTCPService_webcache, 
									   kPGPnetTCPService_tproxy, 
									   kPGPnetTCPService_socks},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing 2",
		"[ANYDIR]\\IEXPLORE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow  Internet Explorer */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,
									   kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data TCP",
		"[ANYDIR]\\IEXPLORE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow  Internet Explorer */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data UDP",
		"[ANYDIR]\\IEXPLORE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow  Internet Explorer */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow FTP",
		"[ANYDIR]\\IEXPLORE.EXE", {0}, 0, kPGPFireRuleType_Child
	},	
	{	/* Allow Internet Explorer Client */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Internet Explorer Outgoing",
		"[ANYDIR]\\IEXPLORE.EXE", {0}, 0, kPGPFireRuleType_Child
	},	
	{	/* Allow Internet Explorer */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {1375,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Internet Explorer Incoming",
		"[ANYDIR]\\IEXPLORE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Internet Explorer FTP Data and Gopher */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1040,1050,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Folders Incoming",
		"[ANYDIR]\\IEXPLORE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Windows Explorer Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Windows Explorer",
		"[ANYDIR]\\EXPLORER.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Incoming Explorer Netbios */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_netbios_ssn,0,0,0},
		"Smart Rule - Allow NetBIOS Session Incoming",
		"[ANYDIR]\\EXPLORER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Outgoing Explorer Netbios */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_LocalSubnet, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_netbios_ssn,
									   kPGPnetTCPService_msds,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow NetBIOS Outgoing",
		"[ANYDIR]\\EXPLORER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Windows Explorer netbios */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_netbios_ns,
									   kPGPnetTCPService_netbios_dgm,0,0},
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_netbios_ns,
									   kPGPnetTCPService_netbios_dgm,0,0},
		"Smart Rule - Allow NetBIOS",
		"[ANYDIR]\\EXPLORER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Windows Explorer web ports*/
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_http, 81, 82, 83},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[ANYDIR]\\EXPLORER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Windows Explorer web ports*/
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_https, 
									   kPGPnetTCPService_webcache, 
									   kPGPnetTCPService_tproxy, 
									   kPGPnetTCPService_socks},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing 2",
		"[ANYDIR]\\EXPLORER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Explorer */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,
									   kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data TCP",
		"[ANYDIR]\\EXPLORER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Explorer */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data UDP",
		"[ANYDIR]\\EXPLORER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Explorer */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow FTP",
		"[ANYDIR]\\EXPLORER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Windows Explorer Client */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Explorer Outgoing",
		"[ANYDIR]\\EXPLORER.EXE", {0}, 0, kPGPFireRuleType_Child
	},	
	{	/* Allow Windows Explorer */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {1375,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Explorer Incoming",
		"[ANYDIR]\\EXPLORER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Windows Explorer */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1040,1050,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Folders Incoming",
		"[ANYDIR]\\EXPLORER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Netscape Navigator Group */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Netscape Navigator",
		"[PRGDIR]\\NETSCP6.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Netscape Navigator web ports*/
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_http, 81, 82, 83},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[PRGDIR]\\NETSCP6.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape Navigator web ports*/
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_https, 
									   kPGPnetTCPService_webcache, 
									   kPGPnetTCPService_tproxy, 
									   kPGPnetTCPService_socks},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing 2",
		"[PRGDIR]\\NETSCP6.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape Navigator Mail */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_pop3,
									   kPGPnetTCPService_smtp,
									   kPGPnetTCPService_imap2,
									   kPGPnetTCPService_imap3},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Mail Outgoing",
		"[PRGDIR]\\NETSCP6.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape Navigator News and IRC */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_nntp,
									   kPGPnetTCPService_ircd,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow News and IRC Outgoing",
		"[PRGDIR]\\NETSCP6.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,
									   kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data TCP",
		"[PRGDIR]\\NETSCP6.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data UDP",
		"[PRGDIR]\\NETSCP6.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow FTP",
		"[PRGDIR]\\NETSCP6.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Netscape Navigator Group */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Netscape Navigator",
		"[PRGDIR]\\NETSCAPE.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Netscape Navigator web ports*/
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_http, 81, 82, 83},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[PRGDIR]\\NETSCAPE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape Navigator web ports*/
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_https, 
									   kPGPnetTCPService_webcache, 
									   kPGPnetTCPService_tproxy, 
									   kPGPnetTCPService_socks},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing 2",
		"[PRGDIR]\\NETSCAPE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape Navigator Mail */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_pop3,
									   kPGPnetTCPService_smtp,
									   kPGPnetTCPService_imap2,
									   kPGPnetTCPService_imap3},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Mail Outgoing",
		"[PRGDIR]\\NETSCAPE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape Navigator News and IRC */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_nntp,
									   kPGPnetTCPService_ircd,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow News and IRC Outgoing",
		"[PRGDIR]\\NETSCAPE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,
									   kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data TCP",
		"[PRGDIR]\\NETSCAPE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data UDP",
		"[PRGDIR]\\NETSCAPE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow FTP",
		"[PRGDIR]\\NETSCAPE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Mozilla Group */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Mozilla",
		"[PRGDIR]\\MOZILLA.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Mozilla web ports*/
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_http, 81, 82, 83},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[PRGDIR]\\MOZILLA.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Mozilla web ports*/
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_https, 
									   kPGPnetTCPService_webcache, 
									   kPGPnetTCPService_tproxy, 
									   kPGPnetTCPService_socks},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing 2",
		"[PRGDIR]\\MOZILLA.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape Navigator Mail */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_pop3,
									   kPGPnetTCPService_smtp,
									   kPGPnetTCPService_imap2,
									   kPGPnetTCPService_imap3},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Mail Outgoing",
		"[PRGDIR]\\MOZILLA.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Netscape Navigator News and IRC */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_nntp,
									   kPGPnetTCPService_ircd,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow News and IRC Outgoing",
		"[PRGDIR]\\MOZILLA.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Mozilla */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,
									   kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data TCP",
		"[PRGDIR]\\MOZILLA.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Mozilla */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data UDP",
		"[PRGDIR]\\MOZILLA.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Mozilla */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow FTP",
		"[PRGDIR]\\MOZILLA.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Outlook Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Outlook",
		"[PRGDIR]\\OUTLOOK.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Outlook */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_pop3,
									   kPGPnetTCPService_smtp,
									   kPGPnetTCPService_imap2,
									   kPGPnetTCPService_imap3},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow Mail Outgoing",
		"[PRGDIR]\\OUTLOOK.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Outlook */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single,1,{kPGPnetTCPService_nntp,0,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow News Outgoing",
		"[PRGDIR]\\OUTLOOK.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Outlook */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 3, {kPGPnetTCPService_http,
									   kPGPnetTCPService_https,
									   135,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[PRGDIR]\\OUTLOOK.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Outlook */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow Outlook TCP",
		"[PRGDIR]\\OUTLOOK.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Outlook */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow Outlook UDP",
		"[PRGDIR]\\OUTLOOK.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Outlook Express Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Outlook Express",
		"[PRGDIR]\\MSIMN.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Outlook Express */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_pop3,
									   kPGPnetTCPService_smtp,
									   kPGPnetTCPService_imap2,
									   kPGPnetTCPService_imap3},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow Mail Outgoing",
		"[PRGDIR]\\MSIMN.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Outlook Express */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single,1,{kPGPnetTCPService_nntp,0,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow News Outgoing",
		"[PRGDIR]\\MSIMN.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Outlook Express */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_http,
									   kPGPnetTCPService_https,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[PRGDIR]\\MSIMN.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* FTP Group */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"FTP",
		"[ANYDIR]\\FTP.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,
									   kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data TCP",
		"[ANYDIR]\\FTP.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data UDP",
		"[ANYDIR]\\FTP.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow FTP",
		"[ANYDIR]\\FTP.EXE", {0}, 0, kPGPFireRuleType_Child
	},	
	{	/* CUTFTP32 Group */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"CuteFTP32",
		"[ANYDIR]\\CUTFTP32.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data TCP",
		"[ANYDIR]\\CUTFTP32.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data UDP",
		"[ANYDIR]\\CUTFTP32.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow FTP",
		"[ANYDIR]\\CUTFTP32.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	
	{	/* CUTEFTP Group */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"CuteFTP",
		"[ANYDIR]\\CUTEFTP.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data TCP",
		"[ANYDIR]\\CUTEFTP.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data UDP",
		"[ANYDIR]\\CUTEFTP.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow FTP",
		"[ANYDIR]\\CUTEFTP.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	
	{	/* FLASHFXP Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"FlashFXP",
		"[ANYDIR]\\FLASHFXP.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,
									   kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data TCP",
		"[ANYDIR]\\FLASHFXP.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_ftpdata,
									   kPGPnetTCPService_ftp,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow FTP and FTP Data UDP",
		"[ANYDIR]\\FLASHFXP.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow FTP */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow FTP",
		"[ANYDIR]\\FLASHFXP.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	
	{	/* PING Group */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Ping",
		"[ANYDIR]\\PING.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow PING echo requests */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_EchoRequest,0,0,0},
		"Smart Rule - Allow ICMP Echo Request Outgoing",
		"[ANYDIR]\\PING.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow PING echo reply */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_ICMP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetICMPType_EchoReply,0,0,0},
		"Smart Rule - Allow ICMP Echo Reply Incoming",
		"[ANYDIR]\\PING.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* REALPLAY Group */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"RealPlayer",
		"[ANYDIR]\\REALPLAY.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Real Player RTSP/TCP connection */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {554,7070,80,8080},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow RTSP Outgoing",
		"[ANYDIR]\\REALPLAY.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Real Player RTP/UDP data stream */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Range, 2, {6970,7170,0,0},
		"Smart Rule - Allow RTP Incoming",
		"[ANYDIR]\\REALPLAY.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow REALPLAY outgoing everything */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Outgoing",
		"[ANYDIR]\\REALPLAY.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* REALJBOX Group */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Real JukeBox",
		"[ANYDIR]\\REALJBOX.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Real Player RTSP/TCP connection */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {554,7070,80,8080},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow RTSP Outgoing",
		"[ANYDIR]\\REALJBOX.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Real Player RTP/UDP data stream */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Range, 2, {6970,7170,0,0},
		"Smart Rule - Allow RTP Incoming",
		"[ANYDIR]\\REALJBOX.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow REALJBOX outgoing everything */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Outgoing",
		"[ANYDIR]\\REALJBOX.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* QUICKTIME Group */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Quicktime Player",
		"[ANYDIR]\\QUICKTIME.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Quicktime Player RTSP/TCP connection */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {554,80,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow RTSP Outgoing",
		"[ANYDIR]\\QUICKTIME.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Quicktime Player RTP/UDP data stream */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Range, 2, {6970,6999,0,0},
		"Smart Rule - Allow RTP Incoming",
		"[ANYDIR]\\QUICKTIME.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow QUICKTIME outgoing everything */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Outgoing",
		"[ANYDIR]\\QUICKTIME.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* QUICKTIMEPLAYER Group */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Quicktime Player",
		"[ANYDIR]\\QUICKTIMEPLAYER.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Quicktime Player RTSP/TCP connection */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {554,80,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow RTSP Outgoing",
		"[ANYDIR]\\QUICKTIMEPLAYER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Quicktime Player RTP/UDP data stream */
		TRUE, TRUE, FALSE, 1, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Range, 2, {6970,6999,0,0},
		"Smart Rule - Allow RTP Incoming",
		"[ANYDIR]\\QUICKTIMEPLAYER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow QUICKTIMEPLAYER outgoing everything */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Outgoing",
		"[ANYDIR]\\QUICKTIMEPLAYER.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* AOL Instant Messenger Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"AOL Instant Messenger",
		"[ANYDIR]\\AIM.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow AOL Instant Messenger */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_http,5190,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow AOL Outgoing",
		"[ANYDIR]\\AIM.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* AOL Instant Messenger Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"AOL Instant Messenger 95",
		"[ANYDIR]\\AIM95.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow AOL Instant Messenger */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_http,5190,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow AOL Outgoing",
		"[ANYDIR]\\AIM95.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* MSN Messenger Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"MSN Messenger",
		"[ANYDIR]\\MSN6.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow MSN Messenger */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {1863,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow MSN Outgoing",
		"[ANYDIR]\\MSN6.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow MSN6 outgoing everything */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Outgoing",
		"[ANYDIR]\\MSN6.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Eudora Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Eudora",
		"[ANYDIR]\\EUDORA.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Eudora client */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_pop3,
									   kPGPnetTCPService_smtp,
									   kPGPnetTCPService_imap2,
									   kPGPnetTCPService_imap3},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Mail Outgoing",
		"[ANYDIR]\\EUDORA.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Eudora client */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 3, {kPGPnetTCPService_http,
									   kPGPnetTCPService_https,
									   kPGPnetTCPService_webcache,
									   0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[ANYDIR]\\EUDORA.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Pegasus Mail Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Pegasus Mail",
		"[ANYDIR]\\WINPM-32.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Pegasus Mail client */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_pop3,
									   kPGPnetTCPService_smtp,
									   kPGPnetTCPService_imap2,
									   kPGPnetTCPService_imap3},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Mail Outgoing",
		"[ANYDIR]\\WINPM-32.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Pegasus Mail client */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 3, {kPGPnetTCPService_http,
									   kPGPnetTCPService_https,
									   kPGPnetTCPService_webcache,
									   0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Web Outgoing",
		"[ANYDIR]\\WINPM-32.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* NsLookup Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Name Server Lookup",
		"[ANYDIR]\\NSLOOKUP.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow NsLookup */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_dns,0,0,0},
		"Smart Rule - Allow DNS Lookup",
		"[ANYDIR]\\NSLOOKUP.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow NsLookup */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_dns,0,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow DNS",
		"[ANYDIR]\\NSLOOKUP.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Telnet Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_All,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Telnet",
		"[ANYDIR]\\TELNET.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Telnet */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {kPGPnetTCPService_telnet,0,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow Telnet Outgoing",
		"[ANYDIR]\\TELNET.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* SecureCrt Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"SecureCRT",
		"[ANYDIR]\\SECURECRT.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow SecureCrt */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 3, {kPGPnetTCPService_ssh,
									   kPGPnetTCPService_telnet,
									   kPGPnetTCPService_login,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow SSH, Telnet, rlogin Outgoing",
		"[ANYDIR]\\SECURECRT.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Groupwise Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Groupwise",
		"[PRGDIR]\\GRPWISE.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Groupwise */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_pop3,
									   kPGPnetTCPService_imap3,
									   kPGPnetTCPService_ldap,
									   1677},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow GroupWise Mail Outgoing",
		"[PRGDIR]\\GRPWISE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Groupwise Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Groupwise",
		"[PRGDIR]\\ADDRBOOK.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Address Book */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {1677,0,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow Address Book Outgoing",
		"[PRGDIR]\\ADDRBOOK.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Groupwise Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Groupwise",
		"[PRGDIR]\\CPLACE.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow cplace */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {1677,0,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow Mail Outgoing",
		"[PRGDIR]\\CPLACE.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Groupwise Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Groupwise",
		"[PRGDIR]\\NOTIFY.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow notify */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {1677,0,0,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow Mail Outgoing",
		"[PRGDIR]\\NOTIFY.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Lotus Notes Group */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Lotus Notes",
		"[ANYDIR]\\NLNOTES.EXE", {0}, 0, kPGPFireRuleType_Group
	},
	{	/* Allow Lotus Notes */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Single, 1, {1352,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Domino Server Connection",
		"[ANYDIR]\\NLNOTES.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Lotus Notes */
		TRUE, TRUE, FALSE, 2, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_http,
									   kPGPnetTCPService_https,
									   kPGPnetTCPService_webcache,0},
		kPGPnetIPServiceType_Range, 2, {1024,65535,0,0},
		"Smart Rule - Allow Notes Web Outgoing",
		"[ANYDIR]\\NLNOTES.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Lotus Notes */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 4, {kPGPnetTCPService_pop3,
									   kPGPnetTCPService_imap2,
									   kPGPnetTCPService_imap3,
									   kPGPnetTCPService_smtp},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Notes Mail",
		"[ANYDIR]\\NLNOTES.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Lotus Notes */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_TCP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_List, 2, {kPGPnetTCPService_nntp,
									   kPGPnetTCPService_ldap,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Notes News/AddressBook",
		"[ANYDIR]\\NLNOTES.EXE", {0}, 0, kPGPFireRuleType_Child
	},
	{	/* Allow Lotus Notes */
		TRUE, TRUE, FALSE, 0, kPGPnetIPProtocol_UDP,
		kPGPnetAddrType_Any, 0, 0,
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		kPGPnetIPServiceType_Any, 0, {0,0,0,0},
		"Smart Rule - Allow Notes UDP",
		"[ANYDIR]\\NLNOTES.EXE", {0}, 0, kPGPFireRuleType_Child
	},
};

	PGPError
PGPGetNetHostPrefs(
	PGPPrefRef				prefRef,
	PGPNetPrefHostEntry		**netHostList,
	PGPUInt32				*netHostCount)
{
	PGPError					err	= kPGPError_NoErr;
	PGPPrefArray *				prefArray = NULL;
	PGPPrefStruct *				prefStruct = NULL;
	PGPMemoryMgrRef				memoryMgr = kInvalidPGPMemoryMgrRef;
	PGPBoolean					badStruct = FALSE;
	PGPUInt32					structIndex,
								index;

	PGPValidatePtr(netHostList);
	PGPValidatePtr(netHostCount);
	*netHostList = NULL;
	*netHostCount = 0;

	err = PGPGetPrefArray(	prefRef, kPGPNetPrefHostArray, &prefArray);
	if (IsntPGPError(err))
	{
		*netHostCount = prefArray->numElements;

		if( prefArray->type != kPGPPrefType_Struct )
		{
			pgpAssertMsg(FALSE, "If this happens, there's a bug in the prefs "
				"code");
			badStruct = TRUE;
		}
		else for( index = 0; index < *netHostCount ; index++ )
		{
			PGPGetPrefArrayStruct( prefRef, prefArray, index, &prefStruct );

			if( prefStruct->numMembers != sTemplateHostStruct.numMembers )
			{
				pgpAssertMsg(FALSE, "If this happens, there's a bug in the "
					"prefs code");
				badStruct = TRUE;
			}
			else
			{
				for( structIndex = 0; structIndex < prefStruct->numMembers;
						structIndex++ )
				{
					if( prefStruct->members[structIndex].type !=
						sTemplateHostMember[structIndex].type )
					{
						pgpAssertMsg(FALSE, "If this happens, there's a bug "
							"in the prefs code");
						badStruct = TRUE;
					}
				}
			}
		}
	}
	if( badStruct )
	{
		PGPFreePrefArray(prefArray);

		err = PGPSetPrefArray(prefRef, kPGPNetPrefHostArray, &sTemplateHost );
		if (IsntPGPError(err))
			err = PGPGetPrefArray(	prefRef,
									kPGPNetPrefHostArray,
									&prefArray );
	}
	if( IsntPGPError( err ) )
	{
		PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

		*netHostList = (PGPNetPrefHostEntry *) PGPNewData( memoryMgr,
				sizeof(PGPNetPrefHostEntry) * prefArray->numElements,
				kPGPMemoryMgrFlags_Clear );

		if( IsNull( *netHostList ) )
			err = kPGPError_OutOfMemory;
	}
	if (IsntPGPError(err))
	{
		for( index=0; index < prefArray->numElements; index++ )
		{
			err = PGPGetPrefArrayStruct( prefRef, prefArray, index,
					&prefStruct );

			if (IsPGPError(err))
				break;

			(*netHostList)[index].hostType = (PGPNetHostType)
											prefStruct->members[0].data;
			(*netHostList)[index].ipAddrStart = 
					PGPStorageToUInt32 ((PGPByte*)&prefStruct->members[1].data);
			(*netHostList)[index].ipMaskEnd = (PGPUInt32)
					PGPStorageToUInt32 ((PGPByte*)&prefStruct->members[2].data);
			(*netHostList)[index].childOf = (PGPInt32)
											prefStruct->members[3].data;

			if (IsntNull(prefStruct->members[4].data))
				pgpCopyMemory( prefStruct->members[4].data,
					(*netHostList)[index].hostName,
					prefStruct->members[4].size );

			if (IsntNull(prefStruct->members[5].data))
				pgpCopyMemory( prefStruct->members[5].data,
					(*netHostList)[index].sharedSecret,
					prefStruct->members[5].size );

			(*netHostList)[index].autoConnect = (PGPBoolean)(size_t)
											prefStruct->members[6].data;
			(*netHostList)[index].virtualIP = (PGPBoolean)(size_t)
											prefStruct->members[7].data;
			(*netHostList)[index].exclusiveGateway = (PGPBoolean)(size_t)
											prefStruct->members[8].data;
			
			/* NOTE we leave useAuthKey out here for storage only */
			if (IsntNull(prefStruct->members[9].data))
			{
				pgpCopyMemory( prefStruct->members[9].data,
					&(*netHostList)[index].authKeyIDR,
					prefStruct->members[9].size );
				(*netHostList)[index].useAuthKeyR = TRUE;
			}

			if (IsntNull(prefStruct->members[10].data))
				pgpCopyMemory( prefStruct->members[10].data,
					(*netHostList)[index].authCertIASNR,
					prefStruct->members[10].size );

			(*netHostList)[index].authCertIASNLengthR = (PGPUInt32)
											prefStruct->members[11].data;

			(*netHostList)[index].authStyle = (PGPikeAuthStyle)
											prefStruct->members[12].data;
			(*netHostList)[index].aggressive = (PGPBoolean)(size_t)
											prefStruct->members[13].data;
			(*netHostList)[index].destIsRange = (PGPBoolean)(size_t)
											prefStruct->members[14].data;
			(*netHostList)[index].idType = (PGPipsecIdentity)
											prefStruct->members[15].data;
			if (IsntNull(prefStruct->members[16].data))
				pgpCopyMemory( prefStruct->members[16].data,
					(*netHostList)[index].identity,
					prefStruct->members[16].size );

			if (IsntNull(prefStruct->members[17].data))
			{
				pgpCopyMemory( prefStruct->members[17].data,
					&(*netHostList)[index].authKeyIDL,
					prefStruct->members[17].size );
				(*netHostList)[index].useAuthKeyL = TRUE;
			}

			if (IsntNull(prefStruct->members[18].data))
				pgpCopyMemory( prefStruct->members[18].data,
					(*netHostList)[index].authCertIASNL,
					prefStruct->members[18].size );

			(*netHostList)[index].authCertIASNLengthL = (PGPUInt32)
											prefStruct->members[19].data;
			(*netHostList)[index].encapsulateMode = (PGPikeEncapsulateMode)
											prefStruct->members[20].data;
		}

		if (IsntPGPError(err))
			*netHostCount = prefArray->numElements;
	}

	if (IsntNull(prefArray))
		PGPFreePrefArray(prefArray);

	return err;
}

	PGPError
PGPSetNetHostPrefs(
	PGPPrefRef				prefRef,
	PGPNetPrefHostEntry		*netHostList,
	PGPUInt32				netHostCount )
{
	PGPError				err = kPGPError_NoErr;
	PGPUInt32				ip, index;
	PGPPrefArray *			prefArray = NULL;
	PGPPrefStruct *			prefStruct = NULL;
	PGPMemoryMgrRef			memoryMgr = NULL;

	if (IsNull(netHostList))
	{
		PGPClearPrefData(prefRef, kPGPNetPrefHostArray);
		return kPGPError_NoErr;
	}

	PGPValidatePtr( netHostList );
	PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

	err = PGPCreatePrefArray( prefRef, kPGPPrefType_Struct,
			netHostCount, &sTemplateHostStruct, &prefArray );

	if( IsntPGPError( err ) )
	{
		for( index=0; index < netHostCount; index++ )
		{
			err = PGPCreatePrefStruct( prefRef, &sTemplateHostStruct, &prefStruct );
			if( IsPGPError( err ) )
				break;
			PGPSetPrefStructNumber( prefRef, prefStruct, 0, netHostList[index].hostType );
			PGPUInt32ToStorage ( netHostList[index].ipAddrStart, (PGPByte*)&ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 1, ip );
			PGPUInt32ToStorage ( netHostList[index].ipMaskEnd, (PGPByte*)&ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 2, ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 3, netHostList[index].childOf );
			PGPSetPrefStructString( prefRef, prefStruct, 4, netHostList[index].hostName );
			PGPSetPrefStructString( prefRef, prefStruct, 5, netHostList[index].sharedSecret );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 6, netHostList[index].autoConnect );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 7, netHostList[index].virtualIP );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 8, netHostList[index].exclusiveGateway );
			
			if( !netHostList[index].useAuthKeyR )
			{
				PGPSetPrefStructByte( prefRef, prefStruct, 9, 0, NULL );
				PGPSetPrefStructByte( prefRef, prefStruct, 10, 0, NULL );
				PGPSetPrefStructNumber( prefRef, prefStruct, 11, 0 );
			}
			else
			{
				PGPSetPrefStructByte( prefRef, prefStruct, 9, sizeof(PGPKeyID),
									&netHostList[index].authKeyIDR );
				PGPSetPrefStructByte( prefRef, prefStruct, 10,
									netHostList[index].authCertIASNLengthR,
									netHostList[index].authCertIASNR );
				PGPSetPrefStructNumber( prefRef, prefStruct, 11,
									netHostList[index].authCertIASNLengthR );
			}
			
			PGPSetPrefStructNumber( prefRef, prefStruct, 12, netHostList[index].authStyle );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 13, netHostList[index].aggressive );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 14, netHostList[index].destIsRange );
			PGPSetPrefStructNumber( prefRef, prefStruct, 15, netHostList[index].idType );
			PGPSetPrefStructString( prefRef, prefStruct, 16, netHostList[index].identity );

			if( !netHostList[index].useAuthKeyL )
			{
				PGPSetPrefStructByte( prefRef, prefStruct, 17, 0, NULL );
				PGPSetPrefStructByte( prefRef, prefStruct, 18, 0, NULL );
				PGPSetPrefStructNumber( prefRef, prefStruct, 19, 0 );
			}
			else
			{
				PGPSetPrefStructByte( prefRef, prefStruct, 17, sizeof(PGPKeyID),
									&netHostList[index].authKeyIDL );
				PGPSetPrefStructByte( prefRef, prefStruct, 18,
									netHostList[index].authCertIASNLengthL,
									netHostList[index].authCertIASNL );
				PGPSetPrefStructNumber( prefRef, prefStruct, 19,
									netHostList[index].authCertIASNLengthL );
			}
			PGPSetPrefStructNumber( prefRef, prefStruct, 20, netHostList[index].encapsulateMode );

			err = PGPSetPrefArrayStruct(prefRef, prefArray, index, prefStruct);
			if( IsPGPError( err ) )
				break;
		}
		if( IsPGPError( err ) )
			PGPFreePrefStruct( prefStruct );
	}
	if( IsntPGPError( err ) )
		err = PGPSetPrefArray( prefRef, kPGPNetPrefHostArray, prefArray );
	if( IsntNull( prefArray ) )
		PGPFreePrefArray(prefArray);
	return err;
}

	PGPError
PGPGetNetBlockedPrefs(
	PGPPrefRef				prefRef,
	PGPNetPrefBlockedEntry	**blockedList,
	PGPUInt32				*blockedCount)
{
	PGPError					err	= kPGPError_NoErr;
	PGPPrefArray *				prefArray = NULL;
	PGPPrefStruct *				prefStruct = NULL;
	PGPMemoryMgrRef				memoryMgr = kInvalidPGPMemoryMgrRef;
	PGPBoolean					badStruct = FALSE;
	PGPUInt32					structIndex,
								index,
								element;
	PGPTime						currentTime;

	PGPValidatePtr(blockedList);
	PGPValidatePtr(blockedCount);
	*blockedList = NULL;
	*blockedCount = 0;

	err = PGPGetPrefArray(	prefRef,
							kPGPNetPrefBlockedHostArray,
							&prefArray);
	if (IsntPGPError(err))
	{
		*blockedCount = prefArray->numElements;

		if( prefArray->type != kPGPPrefType_Struct )
		{
			pgpAssertMsg(FALSE, "If this happens, there's a bug in the prefs "
				"code");
			badStruct = TRUE;
		}
		else for( index = 0; index < *blockedCount ; index++ )
		{
			PGPGetPrefArrayStruct( prefRef, prefArray, index, &prefStruct );

			if( prefStruct->numMembers != sTemplateBlockedStruct.numMembers )
			{
				pgpAssertMsg(FALSE, "If this happens, there's a bug in the "
					"prefs code");
				badStruct = TRUE;
			}
			else
			{
				for( structIndex = 0; structIndex < prefStruct->numMembers;
						structIndex++ )
				{
					if( prefStruct->members[structIndex].type !=
						sTemplateBlockedMember[structIndex].type )
					{
						pgpAssertMsg(FALSE, "If this happens, there's a bug "
							"in the prefs code");
						badStruct = TRUE;
					}
				}
			}
		}
	}
	if( badStruct )
	{
		PGPFreePrefArray(prefArray);

		err = PGPSetPrefArray(prefRef, kPGPNetPrefBlockedHostArray, &sTemplateBlocked );
		if (IsntPGPError(err))
			err = PGPGetPrefArray(	prefRef,
									kPGPNetPrefBlockedHostArray,
									&prefArray );
	}
	if( IsntPGPError( err ) )
	{
		PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

		*blockedList = (PGPNetPrefBlockedEntry *) PGPNewData( memoryMgr,
				sizeof(PGPNetPrefBlockedEntry) * prefArray->numElements,
				kPGPMemoryMgrFlags_Clear );

		if( IsNull( *blockedList ) )
			err = kPGPError_OutOfMemory;
	}
	if (IsntPGPError(err))
	{
		index = 0;
		currentTime = PGPGetTime ();

		for( element=0; element < prefArray->numElements; element++ )
		{
			err = PGPGetPrefArrayStruct( prefRef, prefArray, element,
					&prefStruct );

			if (IsPGPError(err))
				break;

			(*blockedList)[index].ipAddress = 
					PGPStorageToUInt32 ((PGPByte*)&prefStruct->members[0].data);
			(*blockedList)[index].blockedReason = (PGPnetAttackType)
											prefStruct->members[1].data;
			(*blockedList)[index].blockUntil = (PGPInt32)
											prefStruct->members[2].data;
			(*blockedList)[index].attackTime = (PGPInt32)
											prefStruct->members[3].data;

			if (((*blockedList)[index].blockUntil == 0) ||
				((*blockedList)[index].blockUntil > currentTime))
			{
				index++;
			}
		}

		if (IsntPGPError(err))
			*blockedCount = index;
	}

	if (IsntNull(prefArray))
		PGPFreePrefArray(prefArray);

	return err;
}

	PGPError
PGPSetNetBlockedPrefs(
	PGPPrefRef				prefRef,
	PGPNetPrefBlockedEntry	*blockedList,
	PGPUInt32				blockedCount )
{
	PGPError				err = kPGPError_NoErr;
	PGPUInt32				ip, index;
	PGPPrefArray *			prefArray = NULL;
	PGPPrefStruct *			prefStruct = NULL;
	PGPMemoryMgrRef			memoryMgr = NULL;

	if (IsNull(blockedList))
	{
		PGPClearPrefData(prefRef, kPGPNetPrefBlockedHostArray);
		return kPGPError_NoErr;
	}

	PGPValidatePtr( blockedList );
	PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

	err = PGPCreatePrefArray( prefRef, kPGPPrefType_Struct,
			blockedCount, &sTemplateBlockedStruct, &prefArray );

	if( IsntPGPError( err ) )
	{
		for( index=0; index < blockedCount; index++ )
		{
			err = PGPCreatePrefStruct( prefRef, &sTemplateBlockedStruct, &prefStruct );
			if( IsPGPError( err ) )
				break;
			PGPUInt32ToStorage ( blockedList[index].ipAddress, (PGPByte*)&ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 0, ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 1, blockedList[index].blockedReason );
			PGPSetPrefStructNumber( prefRef, prefStruct, 2, blockedList[index].blockUntil );
			PGPSetPrefStructNumber( prefRef, prefStruct, 3, blockedList[index].attackTime );

			err = PGPSetPrefArrayStruct(prefRef, prefArray, index, prefStruct);
			if( IsPGPError( err ) )
				break;
		}
		if( IsPGPError( err ) )
			PGPFreePrefStruct( prefStruct );
	}
	if( IsntPGPError( err ) )
		err = PGPSetPrefArray( prefRef, kPGPNetPrefBlockedHostArray, prefArray );
	if( IsntNull( prefArray ) )
		PGPFreePrefArray(prefArray);
	return err;
}

		PGPError
PGPGetNetTrustedPrefs(
	PGPPrefRef				prefRef,
	PGPNetPrefTrustedEntry	**trustedList,
	PGPUInt32				*trustedCount)
{
	PGPError					err	= kPGPError_NoErr;
	PGPPrefArray *				prefArray = NULL;
	PGPPrefStruct *				prefStruct = NULL;
	PGPMemoryMgrRef				memoryMgr = kInvalidPGPMemoryMgrRef;
	PGPBoolean					badStruct = FALSE;
	PGPUInt32					structIndex,
								index,
								element;

	PGPValidatePtr(trustedList);
	PGPValidatePtr(trustedCount);
	*trustedList = NULL;
	*trustedCount = 0;

	err = PGPGetPrefArray(	prefRef,
							kPGPNetPrefTrustedHostArray,
							&prefArray);
	if (IsntPGPError(err))
	{
		*trustedCount = prefArray->numElements;

		if( prefArray->type != kPGPPrefType_Struct )
		{
			pgpAssertMsg(FALSE, "If this happens, there's a bug in the prefs "
				"code");
			badStruct = TRUE;
		}
		else for( index = 0; index < *trustedCount ; index++ )
		{
			PGPGetPrefArrayStruct( prefRef, prefArray, index, &prefStruct );

			if( prefStruct->numMembers != sTemplateTrustedStruct.numMembers )
			{
				pgpAssertMsg(FALSE, "If this happens, there's a bug in the "
					"prefs code");
				badStruct = TRUE;
			}
			else
			{
				for( structIndex = 0; structIndex < prefStruct->numMembers;
						structIndex++ )
				{
					if( prefStruct->members[structIndex].type !=
						sTemplateTrustedMember[structIndex].type )
					{
						pgpAssertMsg(FALSE, "If this happens, there's a bug "
							"in the prefs code");
						badStruct = TRUE;
					}
				}
			}
		}
	}
	if( badStruct )
	{
		PGPFreePrefArray(prefArray);

		err = PGPSetPrefArray(prefRef, kPGPNetPrefTrustedHostArray, &sTemplateTrusted );
		if (IsntPGPError(err))
			err = PGPGetPrefArray(	prefRef,
									kPGPNetPrefTrustedHostArray,
									&prefArray );
	}
	if( IsntPGPError( err ) )
	{
		PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

		*trustedList = (PGPNetPrefTrustedEntry *) PGPNewData( memoryMgr,
				sizeof(PGPNetPrefTrustedEntry) * prefArray->numElements,
				kPGPMemoryMgrFlags_Clear );

		if( IsNull( *trustedList ) )
			err = kPGPError_OutOfMemory;
	}
	if (IsntPGPError(err))
	{
		index = 0;

		for( element=0; element < prefArray->numElements; element++ )
		{
			err = PGPGetPrefArrayStruct( prefRef, prefArray, element,
					&prefStruct );

			if (IsPGPError(err))
				break;

			(*trustedList)[index].bIsRange = 
					(PGPBoolean)(size_t) prefStruct->members[0].data;
			(*trustedList)[index].ipAddrStart = 
					PGPStorageToUInt32 ((PGPByte*)&prefStruct->members[1].data);
			(*trustedList)[index].ipMaskEnd = 
					PGPStorageToUInt32 ((PGPByte*)&prefStruct->members[2].data);

			index++;
		}

		if (IsntPGPError(err))
			*trustedCount = index;
	}

	if (IsntNull(prefArray))
		PGPFreePrefArray(prefArray);

	return err;
}

	PGPError
PGPSetNetTrustedPrefs(
	PGPPrefRef				prefRef,
	PGPNetPrefTrustedEntry	*trustedList,
	PGPUInt32				trustedCount )
{
	PGPError				err = kPGPError_NoErr;
	PGPUInt32				ip, index;
	PGPPrefArray *			prefArray = NULL;
	PGPPrefStruct *			prefStruct = NULL;
	PGPMemoryMgrRef			memoryMgr = NULL;

	if (IsNull(trustedList))
	{
		PGPClearPrefData(prefRef, kPGPNetPrefTrustedHostArray);
		return kPGPError_NoErr;
	}

	PGPValidatePtr( trustedList );
	PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

	err = PGPCreatePrefArray( prefRef, kPGPPrefType_Struct,
			trustedCount, &sTemplateTrustedStruct, &prefArray );

	if( IsntPGPError( err ) )
	{
		for( index=0; index < trustedCount; index++ )
		{
			err = PGPCreatePrefStruct( prefRef, &sTemplateTrustedStruct, &prefStruct );
			if( IsPGPError( err ) )
				break;
			PGPSetPrefStructBoolean( prefRef, prefStruct, 0, trustedList[index].bIsRange );
			PGPUInt32ToStorage ( trustedList[index].ipAddrStart, (PGPByte*)&ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 1, ip );
			PGPUInt32ToStorage ( trustedList[index].ipMaskEnd, (PGPByte*)&ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 2, ip );

			err = PGPSetPrefArrayStruct(prefRef, prefArray, index, prefStruct);
			if( IsPGPError( err ) )
				break;
		}
		if( IsPGPError( err ) )
			PGPFreePrefStruct( prefStruct );
	}
	if( IsntPGPError( err ) )
		err = PGPSetPrefArray( prefRef, kPGPNetPrefTrustedHostArray, prefArray );
	if( IsntNull( prefArray ) )
		PGPFreePrefArray(prefArray);
	return err;
}

	PGPError
PGPGetNetDefaultFirewallRuleSet(
	PGPPrefRef				prefRef,
	PGPUInt32				setNum,
	PGPNetPrefFirewallRule	**ruleList,
	PGPUInt32				*ruleCount)
{
	PGPError				err = kPGPError_NoErr;
	PGPNetPrefFirewallRule	*ruleSet = NULL;
	PGPSize					ruleSetSize = 0;
	PGPMemoryMgrRef			memoryMgr;

	PGPValidatePtr(ruleList);
	PGPValidatePtr(ruleCount);
	*ruleList = NULL;
	*ruleCount = 0;
	switch( setNum )
	{
		case kPGPNetFirewallRuleSet_Custom:
			ruleSet = NULL;
			ruleSetSize = 0;
			break;
		case kPGPNetFirewallRuleSet_Minimal:
			ruleSet = sPresetFirewallRules_Minimal;
			ruleSetSize = sizeof(sPresetFirewallRules_Minimal);
			break;
		case kPGPNetFirewallRuleSet_ClientMedium:
			ruleSet = sPresetFirewallRules_ClientMedium;
			ruleSetSize = sizeof(sPresetFirewallRules_ClientMedium);
			break;
		case kPGPNetFirewallRuleSet_ClientHigh:
			ruleSet = sPresetFirewallRules_ClientHigh;
			ruleSetSize = sizeof(sPresetFirewallRules_ClientHigh);
			break;
		case kPGPNetFirewallRuleSet_ServerMedium:
			ruleSet = sPresetFirewallRules_ServerMedium;
			ruleSetSize = sizeof(sPresetFirewallRules_ServerMedium);
			break;
		case kPGPNetFirewallRuleSet_ServerHigh:
			ruleSet = sPresetFirewallRules_ServerHigh;
			ruleSetSize = sizeof(sPresetFirewallRules_ServerHigh);
			break;
		case kPGPNetFirewallRuleSet_LearnStarter:
			ruleSet = sPresetFirewallRules_LearnStarter;
			ruleSetSize = sizeof(sPresetFirewallRules_LearnStarter);
			break;
		case kPGPNetFirewallRuleSet_Auto:
			ruleSet = sPresetKnownRules_Auto;
			ruleSetSize = sizeof(sPresetKnownRules_Auto);
			break;
	}
	PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

	*ruleList = (PGPNetPrefFirewallRule *) PGPNewData( memoryMgr,
			ruleSetSize, 0 );
	if( IsntNull( *ruleList ) && IsntNull( ruleSet ) )
	{
		pgpCopyMemory( ruleSet, *ruleList, ruleSetSize );
		*ruleCount = ruleSetSize / sizeof(PGPNetPrefFirewallRule);
	}
	return err;
}

	PGPError
PGPGetNetFirewallRuleSet(
	PGPPrefRef				prefRef,
	PGPNetPrefFirewallRule	**ruleList,
	PGPUInt32				*ruleCount)
{
	PGPError				err = kPGPError_NoErr;
	PGPUInt32				setNum;

	PGPValidatePtr(ruleList);
	PGPValidatePtr(ruleCount);
	*ruleList = NULL;
	*ruleCount = 0;

	PGPGetPrefNumber( prefRef, kPGPnetPrefFirewallRuleSet, &setNum );
	if( setNum == kPGPNetFirewallRuleSet_Custom )
	{
		err = PGPGetNetFirewallCustomRules( prefRef, ruleList, ruleCount );
	}
	else
	{
		err = PGPGetNetDefaultFirewallRuleSet( prefRef, setNum, ruleList, ruleCount );
	}
	return err;
}

	PGPError
PGPGetNetFirewallCustomRules(
	PGPPrefRef				prefRef,
	PGPNetPrefFirewallRule	**ruleList,
	PGPUInt32				*ruleCount)
{
	PGPError					err	= kPGPError_NoErr;
	PGPPrefArray *				prefArray = NULL;
	PGPPrefStruct *				prefStruct = NULL;
	PGPMemoryMgrRef				memoryMgr = kInvalidPGPMemoryMgrRef;
	PGPBoolean					badStruct = FALSE;
	PGPUInt32					structIndex,
								index,
								element;

	PGPValidatePtr(ruleList);
	PGPValidatePtr(ruleCount);
	*ruleList = NULL;
	*ruleCount = 0;

	err = PGPGetPrefArray(	prefRef,
							kPGPNetPrefFirewallRuleArray,
							&prefArray);
	if (IsntPGPError(err))
	{
		*ruleCount = prefArray->numElements;

		if( prefArray->type != kPGPPrefType_Struct )
		{
			pgpAssertMsg(FALSE, "If this happens, there's a bug in the prefs "
				"code");
			badStruct = TRUE;
		}
		else for( index = 0; index < *ruleCount ; index++ )
		{
			PGPGetPrefArrayStruct( prefRef, prefArray, index, &prefStruct );

			if( prefStruct->numMembers != sTemplateFirewallRuleStruct.numMembers )
			{
				pgpAssertMsg(FALSE, "If this happens, there's a bug in the "
					"prefs code");
				badStruct = TRUE;
			}
			else
			{
				for( structIndex = 0; structIndex < prefStruct->numMembers;
						structIndex++ )
				{
					if( prefStruct->members[structIndex].type !=
						sTemplateFirewallRuleMember[structIndex].type )
					{
						pgpAssertMsg(FALSE, "If this happens, there's a bug "
							"in the prefs code");
						badStruct = TRUE;
					}
				}
			}
		}
	}
	if( badStruct )
	{
		PGPFreePrefArray(prefArray);

		err = PGPSetPrefArray(prefRef, kPGPNetPrefFirewallRuleArray, &sTemplateFirewallRule );
		if (IsntPGPError(err))
			err = PGPGetPrefArray(	prefRef,
									kPGPNetPrefFirewallRuleArray,
									&prefArray );
	}
	if( IsntPGPError( err ) )
	{
		PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

		*ruleList = (PGPNetPrefFirewallRule *) PGPNewData( memoryMgr,
				sizeof(PGPNetPrefFirewallRule) * prefArray->numElements,
				kPGPMemoryMgrFlags_Clear );

		if( IsNull( *ruleList ) )
			err = kPGPError_OutOfMemory;
	}
	if( IsntPGPError( err ) )
	{
		for( element=0; element < prefArray->numElements; element++ )
		{
			err = PGPGetPrefArrayStruct( prefRef, prefArray, element,
					&prefStruct );

			if (IsPGPError(err))
				break;

			(*ruleList)[element].active = (PGPBoolean)(size_t) prefStruct->members[0].data;
			(*ruleList)[element].permit = (PGPBoolean)(size_t) prefStruct->members[1].data;
			(*ruleList)[element].matchIntrusion = (PGPBoolean)(size_t) prefStruct->members[2].data;
			(*ruleList)[element].direction = (PGPUInt8)(size_t) prefStruct->members[3].data;
			(*ruleList)[element].ipProtocol = (PGPUInt16)(size_t) prefStruct->members[4].data;
			(*ruleList)[element].remoteAddrType = (PGPNetAddressType)
				prefStruct->members[5].data;
			(*ruleList)[element].remoteIPAddrStart = 
				PGPStorageToUInt32 ((PGPByte*)&prefStruct->members[6].data);
			(*ruleList)[element].remoteIPMaskEnd = 
				PGPStorageToUInt32 ((PGPByte*)&prefStruct->members[7].data);
			(*ruleList)[element].remoteServiceType = (PGPNetIPServiceType)
				prefStruct->members[8].data;
			(*ruleList)[element].numRemoteServices = (PGPUInt8)(size_t) prefStruct->members[9].data;
			(*ruleList)[element].remoteService[0] = (PGPUInt16)(size_t) prefStruct->members[10].data;
			(*ruleList)[element].remoteService[1] = (PGPUInt16)(size_t) prefStruct->members[11].data;
			(*ruleList)[element].remoteService[2] = (PGPUInt16)(size_t) prefStruct->members[12].data;
			(*ruleList)[element].remoteService[3] = (PGPUInt16)(size_t) prefStruct->members[13].data;
			(*ruleList)[element].localServiceType = (PGPNetIPServiceType)
				prefStruct->members[14].data;
			(*ruleList)[element].numLocalServices = (PGPUInt8)(size_t) prefStruct->members[15].data;
			(*ruleList)[element].localService[0] = (PGPUInt16)(size_t) prefStruct->members[16].data;
			(*ruleList)[element].localService[1] = (PGPUInt16)(size_t) prefStruct->members[17].data;
			(*ruleList)[element].localService[2] = (PGPUInt16)(size_t) prefStruct->members[18].data;
			(*ruleList)[element].localService[3] = (PGPUInt16)(size_t) prefStruct->members[19].data;
			if (IsntNull(prefStruct->members[20].data))
				pgpCopyMemory( prefStruct->members[20].data,
					(*ruleList)[element].description,
					prefStruct->members[20].size );
			if (IsntNull(prefStruct->members[21].data))
				pgpCopyMemory( prefStruct->members[21].data,
					(*ruleList)[element].appPath,
					prefStruct->members[21].size );
			if (IsntNull(prefStruct->members[22].data))
				pgpCopyMemory(prefStruct->members[22].data,
				(*ruleList)[element].hash,
				prefStruct->members[22].size );
			(*ruleList)[element].transient = (PGPUInt32)(size_t) prefStruct->members[23].data;
			(*ruleList)[element].ruleType = (PGPFireRuleType)(size_t) prefStruct->members[24].data;
			(*ruleList)[element].logrule = (PGPBoolean)(size_t) prefStruct->members[25].data;
		}

		if (IsntPGPError(err))
			*ruleCount = element;
	}

	if (IsntNull(prefArray))
		PGPFreePrefArray(prefArray);

	return err;
}

	PGPError
PGPSetNetFirewallRulePrefs(
	PGPPrefRef				prefRef,
	PGPNetPrefFirewallRule	*ruleList,
	PGPUInt32				ruleCount )
{
	PGPError				err = kPGPError_NoErr;
	PGPUInt32				ip, index;
	PGPPrefArray *			prefArray = NULL;
	PGPPrefStruct *			prefStruct = NULL;
	PGPMemoryMgrRef			memoryMgr = NULL;

	if( IsNull( ruleList ) )
	{
		PGPClearPrefData(prefRef, kPGPNetPrefFirewallRuleArray);
		return kPGPError_NoErr;
	}

	PGPValidatePtr( ruleList );
	PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

	err = PGPCreatePrefArray( prefRef, kPGPPrefType_Struct,
			ruleCount, &sTemplateFirewallRuleStruct, &prefArray );

	if( IsntPGPError( err ) )
	{
		for( index=0; index < ruleCount; index++ )
		{
			err = PGPCreatePrefStruct( prefRef, &sTemplateFirewallRuleStruct, &prefStruct );
			if( IsPGPError( err ) )
				break;
			PGPSetPrefStructBoolean( prefRef, prefStruct, 0, ruleList[index].active );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 1, ruleList[index].permit );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 2, ruleList[index].matchIntrusion );
			PGPSetPrefStructNumber( prefRef, prefStruct, 3, ruleList[index].direction );
			PGPSetPrefStructNumber( prefRef, prefStruct, 4, ruleList[index].ipProtocol );
			PGPSetPrefStructNumber( prefRef, prefStruct, 5, ruleList[index].remoteAddrType );
			PGPUInt32ToStorage ( ruleList[index].remoteIPAddrStart, (PGPByte*)&ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 6, ip );
			PGPUInt32ToStorage ( ruleList[index].remoteIPMaskEnd, (PGPByte*)&ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 7, ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 8, ruleList[index].remoteServiceType );
			PGPSetPrefStructNumber( prefRef, prefStruct, 9, ruleList[index].numRemoteServices );
			PGPSetPrefStructNumber( prefRef, prefStruct, 10, ruleList[index].remoteService[0] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 11, ruleList[index].remoteService[1] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 12, ruleList[index].remoteService[2] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 13, ruleList[index].remoteService[3] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 14, ruleList[index].localServiceType );
			PGPSetPrefStructNumber( prefRef, prefStruct, 15, ruleList[index].numLocalServices );
			PGPSetPrefStructNumber( prefRef, prefStruct, 16, ruleList[index].localService[0] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 17, ruleList[index].localService[1] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 18, ruleList[index].localService[2] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 19, ruleList[index].localService[3] );
			PGPSetPrefStructString( prefRef, prefStruct, 20, ruleList[index].description );
			PGPSetPrefStructString( prefRef, prefStruct, 21, ruleList[index].appPath );
			PGPSetPrefStructByte ( prefRef, prefStruct, 22, 16, ruleList[index].hash );
			PGPSetPrefStructNumber( prefRef, prefStruct, 23, ruleList[index].transient );
			PGPSetPrefStructNumber( prefRef, prefStruct, 24, ruleList[index].ruleType );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 25, ruleList[index].logrule );

			err = PGPSetPrefArrayStruct(prefRef, prefArray, index, prefStruct);
			if( IsPGPError( err ) )
				break;
		}
		if( IsPGPError( err ) )
			PGPFreePrefStruct( prefStruct );
	}
	if( IsntPGPError( err ) )
		err = PGPSetPrefArray( prefRef, kPGPNetPrefFirewallRuleArray, prefArray );
	if( IsntNull( prefArray ) )
		PGPFreePrefArray(prefArray);
	return err;
}

	PGPError
PGPGetNetKnownRuleSet(
		PGPPrefRef					prefRef,
		PGPNetPrefsArray			netPrefArray,
		PGPNetPrefFirewallRule		**ruleList,
		PGPUInt32					*ruleCount )
{
	PGPError					err	= kPGPError_NoErr;
	PGPPrefArray *				prefArray = NULL;
	PGPPrefStruct *				prefStruct = NULL;
	PGPMemoryMgrRef				memoryMgr = kInvalidPGPMemoryMgrRef;
	PGPBoolean					badStruct = FALSE;
	PGPUInt32					structIndex,
								index,
								element;

	PGPValidatePtr(ruleList);
	PGPValidatePtr(ruleCount);
	*ruleList = NULL;
	*ruleCount = 0;

	err = PGPGetPrefArray(	prefRef,
							netPrefArray,
							&prefArray);
	if (IsntPGPError(err))
	{
		*ruleCount = prefArray->numElements;

		if( prefArray->type != kPGPPrefType_Struct )
		{
			pgpAssertMsg(FALSE, "If this happens, there's a bug in the prefs "
				"code");
			badStruct = TRUE;
		}
		else for( index = 0; index < *ruleCount ; index++ )
		{
			PGPGetPrefArrayStruct( prefRef, prefArray, index, &prefStruct );

			if( prefStruct->numMembers != sTemplateFirewallRuleStruct.numMembers )
			{
				pgpAssertMsg(FALSE, "If this happens, there's a bug in the "
					"prefs code");
				badStruct = TRUE;
			}
			else
			{
				for( structIndex = 0; structIndex < prefStruct->numMembers;
						structIndex++ )
				{
					if( prefStruct->members[structIndex].type !=
						sTemplateFirewallRuleMember[structIndex].type )
					{
						pgpAssertMsg(FALSE, "If this happens, there's a bug "
							"in the prefs code");
						badStruct = TRUE;
					}
				}
			}
		}
	}
	if( badStruct )
	{
		PGPFreePrefArray(prefArray);

		err = PGPSetPrefArray(prefRef, netPrefArray, &sTemplateFirewallRule );
		if (IsntPGPError(err))
			err = PGPGetPrefArray(	prefRef,
									netPrefArray,
									&prefArray );
	}
	if( IsntPGPError( err ) )
	{
		PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

		*ruleList = (PGPNetPrefFirewallRule *) PGPNewData( memoryMgr,
				sizeof(PGPNetPrefFirewallRule) * prefArray->numElements,
				kPGPMemoryMgrFlags_Clear );

		if( IsNull( *ruleList ) )
			err = kPGPError_OutOfMemory;
	}
	if( IsntPGPError( err ) )
	{
		for( element=0; element < prefArray->numElements; element++ )
		{
			err = PGPGetPrefArrayStruct( prefRef, prefArray, element,
					&prefStruct );

			if (IsPGPError(err))
				break;

			(*ruleList)[element].active = (PGPBoolean)(size_t) prefStruct->members[0].data;
			(*ruleList)[element].permit = (PGPBoolean)(size_t) prefStruct->members[1].data;
			(*ruleList)[element].matchIntrusion = (PGPBoolean)(size_t) prefStruct->members[2].data;
			(*ruleList)[element].direction = (PGPUInt8)(size_t) prefStruct->members[3].data;
			(*ruleList)[element].ipProtocol = (PGPUInt16)(size_t) prefStruct->members[4].data;
			(*ruleList)[element].remoteAddrType = (PGPNetAddressType)
				prefStruct->members[5].data;
			(*ruleList)[element].remoteIPAddrStart = 
				PGPStorageToUInt32 ((PGPByte*)&prefStruct->members[6].data);
			(*ruleList)[element].remoteIPMaskEnd = 
				PGPStorageToUInt32 ((PGPByte*)&prefStruct->members[7].data);
			(*ruleList)[element].remoteServiceType = (PGPNetIPServiceType)
				prefStruct->members[8].data;
			(*ruleList)[element].numRemoteServices = (PGPUInt8)(size_t) prefStruct->members[9].data;
			(*ruleList)[element].remoteService[0] = (PGPUInt16)(size_t) prefStruct->members[10].data;
			(*ruleList)[element].remoteService[1] = (PGPUInt16)(size_t) prefStruct->members[11].data;
			(*ruleList)[element].remoteService[2] = (PGPUInt16)(size_t) prefStruct->members[12].data;
			(*ruleList)[element].remoteService[3] = (PGPUInt16)(size_t) prefStruct->members[13].data;
			(*ruleList)[element].localServiceType = (PGPNetIPServiceType)
				prefStruct->members[14].data;
			(*ruleList)[element].numLocalServices = (PGPUInt8)(size_t) prefStruct->members[15].data;
			(*ruleList)[element].localService[0] = (PGPUInt16)(size_t) prefStruct->members[16].data;
			(*ruleList)[element].localService[1] = (PGPUInt16)(size_t) prefStruct->members[17].data;
			(*ruleList)[element].localService[2] = (PGPUInt16)(size_t) prefStruct->members[18].data;
			(*ruleList)[element].localService[3] = (PGPUInt16)(size_t) prefStruct->members[19].data;
			if (IsntNull(prefStruct->members[20].data))
				pgpCopyMemory( prefStruct->members[20].data,
					(*ruleList)[element].description,
					prefStruct->members[20].size );
			if (IsntNull(prefStruct->members[21].data))
				pgpCopyMemory( prefStruct->members[21].data,
					(*ruleList)[element].appPath,
					prefStruct->members[21].size );
			if (IsntNull(prefStruct->members[22].data))
				pgpCopyMemory(prefStruct->members[22].data,
				(*ruleList)[element].hash,
				prefStruct->members[22].size );
			(*ruleList)[element].transient = (PGPUInt32)(size_t) prefStruct->members[23].data;
			(*ruleList)[element].ruleType = (PGPFireRuleType)(size_t) prefStruct->members[24].data;
			(*ruleList)[element].logrule = (PGPBoolean) (size_t) prefStruct->members[25].data;
		}

		if (IsntPGPError(err))
			*ruleCount = element;
	}

	if (IsntNull(prefArray))
		PGPFreePrefArray(prefArray);

	return err;
}

	PGPError
PGPGetNetDefaultKnownRuleSet(
		PGPPrefRef					prefRef,
		PGPNetFirewallSet			setNum,
		PGPNetPrefFirewallRule		**ruleList,
		PGPUInt32					*ruleCount )
{
	PGPError				err = kPGPError_NoErr;
	PGPNetPrefFirewallRule		*ruleSet = NULL;
	PGPSize					ruleSetSize;
	PGPMemoryMgrRef			memoryMgr;

	PGPValidatePtr(ruleList);
	PGPValidatePtr(ruleCount);
	*ruleList = NULL;
	*ruleCount = 0;
	switch( setNum )
	{
		case kPGPNetFirewallRuleSet_Auto:
			ruleSet = sPresetKnownRules_Auto;
			ruleSetSize = sizeof(sPresetKnownRules_Auto);
			break;

		case kPGPNetFirewallRuleSet_Known:
			ruleSet = sPresetKnownRules_Known;
			ruleSetSize = sizeof(sPresetKnownRules_Known);
			break;
		case kPGPNetFirewallRuleSet_Smart:
			ruleSet = sPresetKnownRules_Smart;
			ruleSetSize = sizeof(sPresetKnownRules_Smart);
			break;
		default:
			ruleSet = NULL;
			ruleSetSize = 0;
			break;
	}
	PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

	*ruleList = (PGPNetPrefFirewallRule *) PGPNewData( memoryMgr,
			ruleSetSize, 0 );
	if( IsntNull( *ruleList ) && IsntNull( ruleSet ) )
	{
		pgpCopyMemory( ruleSet, *ruleList, ruleSetSize );
		*ruleCount = ruleSetSize / sizeof(PGPNetPrefFirewallRule);
	}
	return err;
}

	PGPError
PGPSetNetKnownRuleSet(
		PGPPrefRef					prefRef,
		PGPNetPrefsArray			netPrefArray,
		PGPNetPrefFirewallRule		*ruleList,
		PGPUInt32					ruleCount )
{
	PGPError				err = kPGPError_NoErr;
	PGPUInt32				ip, index;
	PGPPrefArray *			prefArray = NULL;
	PGPPrefStruct *			prefStruct = NULL;
	PGPMemoryMgrRef			memoryMgr = NULL;

	if( IsNull( ruleList ) )
	{
		PGPClearPrefData(prefRef, netPrefArray);
		return kPGPError_NoErr;
	}

	PGPValidatePtr( ruleList );
	PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

	err = PGPCreatePrefArray( prefRef, kPGPPrefType_Struct,
			ruleCount, &sTemplateFirewallRuleStruct, &prefArray );

	if( IsntPGPError( err ) )
	{
		for( index=0; index < ruleCount; index++ )
		{
			err = PGPCreatePrefStruct( prefRef, &sTemplateFirewallRuleStruct, &prefStruct );
			if( IsPGPError( err ) )
				break;
			
			PGPSetPrefStructBoolean( prefRef, prefStruct, 0, ruleList[index].active );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 1, ruleList[index].permit );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 2, ruleList[index].matchIntrusion );
			PGPSetPrefStructNumber( prefRef, prefStruct, 3, ruleList[index].direction );
			PGPSetPrefStructNumber( prefRef, prefStruct, 4, ruleList[index].ipProtocol );
			PGPSetPrefStructNumber( prefRef, prefStruct, 5, ruleList[index].remoteAddrType );
			PGPUInt32ToStorage ( ruleList[index].remoteIPAddrStart, (PGPByte*)&ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 6, ip );
			PGPUInt32ToStorage ( ruleList[index].remoteIPMaskEnd, (PGPByte*)&ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 7, ip );
			PGPSetPrefStructNumber( prefRef, prefStruct, 8, ruleList[index].remoteServiceType );
			PGPSetPrefStructNumber( prefRef, prefStruct, 9, ruleList[index].numRemoteServices );
			PGPSetPrefStructNumber( prefRef, prefStruct, 10, ruleList[index].remoteService[0] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 11, ruleList[index].remoteService[1] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 12, ruleList[index].remoteService[2] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 13, ruleList[index].remoteService[3] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 14, ruleList[index].localServiceType );
			PGPSetPrefStructNumber( prefRef, prefStruct, 15, ruleList[index].numLocalServices );
			PGPSetPrefStructNumber( prefRef, prefStruct, 16, ruleList[index].localService[0] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 17, ruleList[index].localService[1] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 18, ruleList[index].localService[2] );
			PGPSetPrefStructNumber( prefRef, prefStruct, 19, ruleList[index].localService[3] );
			PGPSetPrefStructString( prefRef, prefStruct, 20, ruleList[index].description );
			PGPSetPrefStructString( prefRef, prefStruct, 21, ruleList[index].appPath );
			PGPSetPrefStructByte ( prefRef, prefStruct, 22, 16, ruleList[index].hash );
			PGPSetPrefStructNumber( prefRef, prefStruct, 23, ruleList[index].transient );
			PGPSetPrefStructNumber( prefRef, prefStruct, 24, ruleList[index].ruleType );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 25, ruleList[index].logrule );

			err = PGPSetPrefArrayStruct(prefRef, prefArray, index, prefStruct);
			if( IsPGPError( err ) )
				break;
		}
		if( IsPGPError( err ) )
			PGPFreePrefStruct( prefStruct );
	}
	if( IsntPGPError( err ) )
		err = PGPSetPrefArray( prefRef, netPrefArray, prefArray );
	if( IsntNull( prefArray ) )
		PGPFreePrefArray(prefArray);
	return err;
}

	PGPError
PGPGetNetIKEProposalPrefs(
	PGPPrefRef					prefRef,
	PGPNetPrefIKEProposalEntry	**proposalList,
	PGPUInt32					*proposalCount)
{
	PGPError					err	= kPGPError_NoErr;
	PGPPrefArray *				prefArray = NULL;
	PGPPrefStruct *				prefStruct = NULL;
	PGPMemoryMgrRef				memoryMgr = kInvalidPGPMemoryMgrRef;
	PGPBoolean					badStruct = FALSE;
	PGPUInt32					structIndex,
								index;

	PGPValidatePtr(proposalList);
	PGPValidatePtr(proposalCount);
	*proposalList = NULL;
	*proposalCount = 0;

	err = PGPGetPrefArray(	prefRef,
							kPGPNetPrefIKEProposalArray,
							&prefArray);
	if (IsntPGPError(err))
	{
		*proposalCount = prefArray->numElements;

		if( prefArray->type != kPGPPrefType_Struct )
		{
			pgpAssertMsg(FALSE, "If this happens, there's a bug in the prefs "
				"code");
			badStruct = TRUE;
		}
		else for( index = 0; index < *proposalCount ; index++ )
		{
			PGPGetPrefArrayStruct( prefRef, prefArray, index, &prefStruct );

			if( prefStruct->numMembers != sPreSharedCASTProposal.numMembers )
			{
				pgpAssertMsg(FALSE, "If this happens, there's a bug in the "
					"prefs code");
				badStruct = TRUE;
			}
			else
			{
				for( structIndex = 0; structIndex < prefStruct->numMembers;
						structIndex++ )
				{
					if( prefStruct->members[structIndex].type !=
						sPreSharedCASTPropArray[structIndex].type )
					{
						pgpAssertMsg(FALSE, "If this happens, there's a bug "
							"in the prefs code");
						badStruct = TRUE;
					}
				}
			}
		}
	}
	if( badStruct )
	{
		PGPFreePrefArray(prefArray);

		err = PGPSetPrefArray(prefRef, kPGPNetPrefIKEProposalArray, &sDefaultIKEProposalsA );
		if (IsntPGPError(err))
			err = PGPGetPrefArray(	prefRef,
									kPGPNetPrefIKEProposalArray,
									&prefArray );
	}
	if( IsntPGPError( err ) )
	{
		PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

		*proposalList = (PGPNetPrefIKEProposalEntry *) PGPNewData( memoryMgr,
				sizeof(PGPNetPrefIKEProposalEntry) * prefArray->numElements,
				kPGPMemoryMgrFlags_Clear );

		if( IsNull( *proposalList ) )
			err = kPGPError_OutOfMemory;
	}
	if (IsntPGPError(err))
	{
		for( index=0; index < prefArray->numElements; index++ )
		{
			err = PGPGetPrefArrayStruct( prefRef, prefArray, index,
					&prefStruct );
			if (IsPGPError(err))
				break;
			(*proposalList)[index].authMethod = (PGPikeAuthMethod) prefStruct->members[0].data;
			(*proposalList)[index].hash = (PGPikeHash) prefStruct->members[1].data;
			(*proposalList)[index].cipher = (PGPikeCipher) prefStruct->members[2].data;
			(*proposalList)[index].groupID = (PGPikeGroupID) prefStruct->members[3].data;
		}
		if (IsntPGPError(err))
			*proposalCount = prefArray->numElements;
	}
	if (IsntNull(prefArray))
		PGPFreePrefArray(prefArray);
	return err;
}


	PGPError
PGPSetNetIKEProposalPrefs(
	PGPPrefRef					prefRef,
	PGPNetPrefIKEProposalEntry	*proposalList,
	PGPUInt32					proposalCount )
{
	PGPError				err = kPGPError_NoErr;
	PGPUInt32				index;
	PGPPrefArray *			prefArray = NULL;
	PGPPrefStruct *			prefStruct = NULL;
	PGPMemoryMgrRef			memoryMgr = kInvalidPGPMemoryMgrRef;

	PGPValidatePtr( proposalList );
	PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

	err = PGPCreatePrefArray( prefRef, kPGPPrefType_Struct, proposalCount,
			&sPreSharedCASTProposal, &prefArray );

	if( IsntPGPError( err ) )
	{
		for( index=0; index < proposalCount; index++ )
		{
			err = PGPCreatePrefStruct( prefRef, &sPreSharedCASTProposal, &prefStruct );
			if( IsPGPError( err ) )
				break;
			PGPSetPrefStructNumber( prefRef, prefStruct, 0, proposalList[index].authMethod );
			PGPSetPrefStructNumber( prefRef, prefStruct, 1, proposalList[index].hash );
			PGPSetPrefStructNumber( prefRef, prefStruct, 2, proposalList[index].cipher );
			PGPSetPrefStructNumber( prefRef, prefStruct, 3, proposalList[index].groupID );
			err = PGPSetPrefArrayStruct(prefRef, prefArray, index, prefStruct);
			if( IsPGPError( err ) )
				break;
		}
		if( IsPGPError( err ) )
			PGPFreePrefStruct( prefStruct );
	}
	if( IsntPGPError( err ) )
		err = PGPSetPrefArray( prefRef, kPGPNetPrefIKEProposalArray, prefArray );
	if( IsntNull( prefArray ) )
		PGPFreePrefArray(prefArray);
	return err;
}

	PGPError
PGPGetNetIPSECProposalPrefs(
	PGPPrefRef						prefRef,
	PGPNetPrefIPSECProposalEntry	**proposalList,
	PGPUInt32						*proposalCount)
{
	PGPError						err	= kPGPError_NoErr;
	PGPPrefArray *					prefArray = NULL;
	PGPPrefStruct *					prefStruct = NULL;
	PGPMemoryMgrRef					memoryMgr = kInvalidPGPMemoryMgrRef;
	PGPBoolean						badStruct = FALSE;
	PGPUInt32						structIndex,
									index;

	PGPValidatePtr(proposalList);
	PGPValidatePtr(proposalCount);
	*proposalList = NULL;
	*proposalCount = 0;

	err = PGPGetPrefArray(	prefRef,
							kPGPNetPrefIPSecProposalArray,
							&prefArray);
	if (IsntPGPError(err))
	{
		*proposalCount = prefArray->numElements;

		if( prefArray->type != kPGPPrefType_Struct )
		{
			pgpAssertMsg(FALSE, "If this happens, there's a bug in the prefs "
				"code");
			badStruct = TRUE;
		}
		else for( index = 0; index < *proposalCount ; index++ )
		{
			PGPGetPrefArrayStruct( prefRef, prefArray, index, &prefStruct );

			if( prefStruct->numMembers != sCASTSHAIPSecProposal.numMembers )
			{
				pgpAssertMsg(FALSE, "If this happens, there's a bug in the "
					"prefs code");
				badStruct = TRUE;
			}
			else
			{
				for( structIndex = 0; structIndex < prefStruct->numMembers;
						structIndex++ )
				{
					if( prefStruct->members[structIndex].type !=
						sCASTSHAIPSecArray[structIndex].type )
					{
						pgpAssertMsg(FALSE, "If this happens, there's a bug "
							"in the prefs code");
						badStruct = TRUE;
					}
				}
			}
		}
	}
	if( badStruct )
	{
		PGPFreePrefArray(prefArray);

		err = PGPSetPrefArray(prefRef, kPGPNetPrefIPSecProposalArray, &sDefaultIPSecProposalsA );
		if (IsntPGPError(err))
			err = PGPGetPrefArray(	prefRef,
									kPGPNetPrefIPSecProposalArray,
									&prefArray );
	}
	if( IsntPGPError( err ) )
	{
		PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

		*proposalList = (PGPNetPrefIPSECProposalEntry *) PGPNewData( memoryMgr,
				sizeof(PGPNetPrefIPSECProposalEntry) * prefArray->numElements,
				kPGPMemoryMgrFlags_Clear );

		if( IsNull( *proposalList ) )
			err = kPGPError_OutOfMemory;
	}
	if (IsntPGPError(err))
	{
		for( index=0; index < prefArray->numElements; index++ )
		{
			err = PGPGetPrefArrayStruct( prefRef, prefArray, index,
					&prefStruct );
			if (IsPGPError(err))
				break;
			(*proposalList)[index].useESP = (PGPBoolean)(size_t) prefStruct->members[0].data;
			(*proposalList)[index].espCipher = (PGPipsecESPTransformID) prefStruct->members[1].data;
			(*proposalList)[index].espHash = (PGPipsecAuthAttribute) prefStruct->members[2].data;
			(*proposalList)[index].useAH = (PGPBoolean)(size_t) prefStruct->members[3].data;
			(*proposalList)[index].ahAuthAlg = (PGPipsecAHTransformID) prefStruct->members[4].data;
			(*proposalList)[index].ahHash = (PGPipsecAuthAttribute) prefStruct->members[5].data;
			(*proposalList)[index].useIPCOMP = (PGPBoolean)(size_t) prefStruct->members[6].data;
			(*proposalList)[index].ipcomp = (PGPipsecIPCOMPTransformID) prefStruct->members[7].data;
		}
		if (IsntPGPError(err))
			*proposalCount = prefArray->numElements;
	}
	if (IsntNull(prefArray))
		PGPFreePrefArray(prefArray);
	return err;
}

	PGPError
PGPSetNetIPSECProposalPrefs(
	PGPPrefRef						prefRef,
	PGPNetPrefIPSECProposalEntry	*proposalList,
	PGPUInt32						proposalCount )
{
	PGPError						err = kPGPError_NoErr;
	PGPUInt32						index;
	PGPPrefArray *					prefArray = NULL;
	PGPPrefStruct *					prefStruct = NULL;
	PGPMemoryMgrRef					memoryMgr = kInvalidPGPMemoryMgrRef;

	PGPValidatePtr( proposalList );
	PGPPeekPrefMemoryMgr( prefRef, &memoryMgr );

	err = PGPCreatePrefArray( prefRef, kPGPPrefType_Struct, proposalCount,
			&sCASTSHAIPSecProposal, &prefArray );

	if( IsntPGPError( err ) )
	{
		for( index=0; index < proposalCount; index++ )
		{
			err = PGPCreatePrefStruct( prefRef, &sCASTSHAIPSecProposal, &prefStruct );
			if( IsPGPError( err ) )
				break;
			PGPSetPrefStructBoolean( prefRef, prefStruct, 0, proposalList[index].useESP );
			PGPSetPrefStructNumber( prefRef, prefStruct, 1, proposalList[index].espCipher );
			PGPSetPrefStructNumber( prefRef, prefStruct, 2, proposalList[index].espHash );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 3, proposalList[index].useAH );
			PGPSetPrefStructNumber( prefRef, prefStruct, 4, proposalList[index].ahAuthAlg );
			PGPSetPrefStructNumber( prefRef, prefStruct, 5, proposalList[index].ahHash );
			PGPSetPrefStructBoolean( prefRef, prefStruct, 6, proposalList[index].useIPCOMP );
			PGPSetPrefStructNumber( prefRef, prefStruct, 7, proposalList[index].ipcomp );
			err = PGPSetPrefArrayStruct(prefRef, prefArray, index, prefStruct);
			if( IsPGPError( err ) )
				break;
		}
		if( IsPGPError( err ) )
			PGPFreePrefStruct( prefStruct );
	}
	if( IsntPGPError( err ) )
		err = PGPSetPrefArray( prefRef, kPGPNetPrefIPSecProposalArray, prefArray );
	if( IsntNull( prefArray ) )
		PGPFreePrefArray(prefArray);
	return err;
}

